import{r as e,d as T,j as s,B as b,D as P,a as D}from"./main-BzPVmAp-.js";import{R as u}from"./index-BF4b68Hp.js";import{A as M}from"./index-B-RBPprY.js";import{A as J}from"./index-CQOo18Hr.js";import N from"./index-DrRz1_gm.js";import{C as q}from"./index-7u0Bo8e4.js";import{S as C}from"./index-BZ6Sq__j.js";import"./index-CSq209pq.js";import"./index-sAnz6zLe.js";import"./index-BQm1iTRH.js";import"./UpOutlined-B_NQN-zy.js";import"./Table-kIlrKW-9.js";import"./addEventListener-CzbSevz3.js";import"./Dropdown-Clv_Pvnt.js";import"./index-CDeD9lyT.js";import"./index-CUuSmggi.js";import"./index-Fy_dqHL_.js";import"./index-SGIbgpRk.js";import"./index-CUrh6kIn.js";import"./index-Djhsc5K4.js";import"./study-page-D4nenWxt.js";import"./usePagination-CGdUxo17.js";import"./RedoOutlined-d_QTXv-s.js";import"./index-Cl0Hi3Gl.js";import"./callSuper-CKCGYiRV.js";import"./utils-D2B0dIuM.js";const ms=()=>{const n=e.useRef(null),[y,O]=e.useState("downstream"),[c,l]=e.useState(),[f,m]=e.useState(!1),[t,x]=e.useState(),[a,h]=e.useState(),[g,p]=e.useState(!1),[i]=T.useForm(),[j,A]=e.useState(),[S,R]=e.useState({}),[L,d]=e.useState(!1),[_,w]=e.useState(!1),F=async o=>{d(!0);const r=await D.get(`/find_downstream_analysis/${o}`);R(r.data),d(!1)};return s.jsxs("div",{style:{maxWidth:"1500px",margin:"1rem auto"},children:[s.jsx(u,{title:"分析结果",ref:n,analysisType:"analysisResult",params:{},setTableLoading:m,setRecord:o=>{const r=JSON.parse(o.request_param);console.log(r),x(o),h(r)},setTabletData:o=>{l(o)}}),s.jsx("div",{style:{marginBottom:"2rem"}}),t&&s.jsxs(s.Fragment,{children:[s.jsx(q,{title:`${t.analysis_name}(${t.analysis_method})`,extra:s.jsx(s.Fragment,{children:s.jsx(b,{onClick:()=>{p(!0),F(a.analysis_method),i.resetFields(),i.setFieldsValue(a),t!=null&&t.id&&i.setFieldValue("id",t==null?void 0:t.id)},children:"重新分析"})}),children:s.jsx(M,{plotLoading:f,filePlot:c})}),s.jsx("hr",{}),s.jsx(N,{params:{obj_key:t.analysis_method,obj_type:"analysis_img"}}),s.jsxs(P,{loading:L,title:`重新分析 - ${t.analysis_name}`,open:g,onClose:()=>p(!1),width:"50%",children:[s.jsx(u,{title:"输出文件",analysisType:y,analysisMethod:[a.inputAnalysisMenthod],shouldTrigger:!0,setResultTableList:A}),s.jsx(C,{spinning:_,children:s.jsx(J,{form:i,resultTableList:j,formJson:S.formJson,sampleGroupApI:!1,moduleName:a.moduleName,params:a.params,setPlotLoading:o=>{m(o),w(o)},activeTabKey:a.inputAnalysisMenthod.name,inputAnalysisMethod:a.inputAnalysisMenthod,saveAnalysisMethod:a.analysis_method,project:a.project,setFilePlot:l,plotReloadTable:()=>{n.current&&n.current.reload()}})})]})]})]})};export{ms as default};
