import{j as e}from"./main-BzPVmAp-.js";import{A as i}from"./index-CkGaCd3X.js";import{B as t}from"./index-Bak0iqJO.js";import{T as r}from"./index-SGIbgpRk.js";import"./index-BF4b68Hp.js";import"./index-CSq209pq.js";import"./index-sAnz6zLe.js";import"./index-BQm1iTRH.js";import"./UpOutlined-B_NQN-zy.js";import"./index-7u0Bo8e4.js";import"./Table-kIlrKW-9.js";import"./addEventListener-CzbSevz3.js";import"./Dropdown-Clv_Pvnt.js";import"./index-BZ6Sq__j.js";import"./index-IijEf41L.js";import"./index-2r9flV8X.js";import"./index-B-RBPprY.js";import"./index-CDeD9lyT.js";import"./index-CUuSmggi.js";import"./index-Fy_dqHL_.js";import"./index-CUrh6kIn.js";import"./index-Djhsc5K4.js";import"./study-page-D4nenWxt.js";import"./usePagination-CGdUxo17.js";import"./RedoOutlined-d_QTXv-s.js";import"./index-Cl0Hi3Gl.js";import"./callSuper-CKCGYiRV.js";import"./index-RAxRq3bL.js";import"./index-D0tEL95I.js";import"./index-BfzdirmR.js";import"./DeleteOutlined-DoDR38qq.js";import"./index-5ngX0NHj.js";import"./rgb-BwIoVOhg.js";import"./index-CQOo18Hr.js";const C=()=>e.jsxs(e.Fragment,{children:[e.jsx(r,{items:[{key:"function_analysis",label:"功能分析",children:e.jsx(e.Fragment,{children:e.jsx(i,{inputAnalysisMethod:[{name:"reads",label:"去宿主后的reads",inputKey:["samtools_remove_hosts"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]},{name:"abundance",label:"metaphlan丰度文件",inputKey:["metaphlan_sam_abundance"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]}],analysisPipline:"pipeline_humann",analysisMethod:[{name:"humann_profile",label:"humann_profile",inputKey:["humann_profile"],mode:"multiple"}],analysisType:"sample",children:e.jsx(t,{analysisKey:"bowtie2_align_host"})})})}]}),e.jsx("p",{})]});export{C as default};
