import{g as o,r as a,j as s,B as r,a as c}from"./main-BzPVmAp-.js";const d=()=>{const{fileId:e}=o(),[i,n]=a.useState(null),t=async()=>{const l=await c.get(`/get-pipeline-v2/${e}?component_type=file`);n(l.data)};return a.useEffect(()=>{t()},[]),s.jsxs("div",{children:[s.jsx(r,{onClick:()=>{t()},children:"刷新"}),e,s.jsx("h1",{children:"File Page"}),JSON.stringify(i)]})};export{d as default};
