import{j as e,B as m}from"./main-BzPVmAp-.js";import{A as a}from"./index-CkGaCd3X.js";import{f as l}from"./utils-D2B0dIuM.js";import{T as p}from"./index-SGIbgpRk.js";import"./index-BF4b68Hp.js";import"./index-CSq209pq.js";import"./index-sAnz6zLe.js";import"./index-BQm1iTRH.js";import"./UpOutlined-B_NQN-zy.js";import"./index-7u0Bo8e4.js";import"./Table-kIlrKW-9.js";import"./addEventListener-CzbSevz3.js";import"./Dropdown-Clv_Pvnt.js";import"./index-BZ6Sq__j.js";import"./index-IijEf41L.js";import"./index-2r9flV8X.js";import"./index-B-RBPprY.js";import"./index-CDeD9lyT.js";import"./index-CUuSmggi.js";import"./index-Fy_dqHL_.js";import"./index-CUrh6kIn.js";import"./index-Djhsc5K4.js";import"./study-page-D4nenWxt.js";import"./usePagination-CGdUxo17.js";import"./RedoOutlined-d_QTXv-s.js";import"./index-Cl0Hi3Gl.js";import"./callSuper-CKCGYiRV.js";import"./index-RAxRq3bL.js";import"./index-D0tEL95I.js";import"./index-BfzdirmR.js";import"./DeleteOutlined-DoDR38qq.js";import"./index-5ngX0NHj.js";import"./rgb-BwIoVOhg.js";import"./index-CQOo18Hr.js";const I=()=>e.jsxs(e.Fragment,{children:[e.jsx(p,{items:[{key:"ngs_individual_assembly",label:"二代测序宏基因组单独组装",children:e.jsx(e.Fragment,{children:e.jsx(a,{inputAnalysisMethod:[{name:"remove_hosts_reads",label:"remove_hosts_reads",inputKey:["samtools_remove_hosts"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]},{name:"clean_reads",label:"clean_reads",inputKey:["fastp_clean_reads"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group"}],analysisPipline:"pipeline_ngs_individual_metawrap_assembly",analysisMethod:[{name:"metawrap_assembly",label:"metawrap_assembly",inputKey:["metawrap_assembly"],mode:"multiple"}],analysisType:"sample",children:e.jsx(s,{})})})},{key:"ngs_co_assembly",label:"二代测序宏基因组共组装",children:e.jsx(e.Fragment,{children:e.jsx(a,{inputAnalysisMethod:[{name:"remove_hosts_reads",label:"remove_hosts_reads",inputKey:["samtools_remove_hosts"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]},{name:"clean_reads",label:"clean_reads",inputKey:["fastp_clean_reads"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group"}],analysisPipline:"pipeline_ngs_individual_metawrap_assembly",analysisMethod:[{name:"metawrap_assembly",label:"metawrap_assembly",inputKey:["metawrap_assembly"],mode:"multiple"}],analysisType:"sample",children:e.jsx(s,{})})})}]}),e.jsx("p",{})]}),s=({plot:r,record:t})=>e.jsx(e.Fragment,{children:t&&e.jsx(e.Fragment,{children:e.jsx(m,{onClick:()=>{r({url:l(t.content.html),tableDesc:""})},children:"查看报告"})})});export{I as default};
