import{k as o,j as t}from"./main-BzPVmAp-.js";import{R as i}from"./index-IijEf41L.js";import"./index-2r9flV8X.js";import"./index-B-RBPprY.js";import"./index-CDeD9lyT.js";import"./index-CUuSmggi.js";import"./addEventListener-CzbSevz3.js";import"./index-7u0Bo8e4.js";import"./index-SGIbgpRk.js";import"./Dropdown-Clv_Pvnt.js";import"./index-Fy_dqHL_.js";import"./index-CSq209pq.js";import"./index-sAnz6zLe.js";import"./index-BQm1iTRH.js";import"./UpOutlined-B_NQN-zy.js";import"./Table-kIlrKW-9.js";import"./index-BZ6Sq__j.js";import"./index-CUrh6kIn.js";import"./index-Djhsc5K4.js";import"./study-page-D4nenWxt.js";import"./usePagination-CGdUxo17.js";import"./RedoOutlined-d_QTXv-s.js";import"./index-Cl0Hi3Gl.js";import"./callSuper-CKCGYiRV.js";import"./index-RAxRq3bL.js";import"./index-D0tEL95I.js";import"./index-BfzdirmR.js";import"./DeleteOutlined-DoDR38qq.js";import"./index-BF4b68Hp.js";import"./index-5ngX0NHj.js";import"./rgb-BwIoVOhg.js";const D=()=>{const{project:r}=o();return t.jsx("div",{style:{maxWidth:"1500px",margin:"1rem auto"},children:t.jsx(i,{project:r,setRecord:m=>{}})})};export{D as default};
