import{j as e}from"./main-BzPVmAp-.js";import{A as i}from"./index-CkGaCd3X.js";import{T as m}from"./index-SGIbgpRk.js";import"./index-BF4b68Hp.js";import"./index-CSq209pq.js";import"./index-sAnz6zLe.js";import"./index-BQm1iTRH.js";import"./UpOutlined-B_NQN-zy.js";import"./index-7u0Bo8e4.js";import"./Table-kIlrKW-9.js";import"./addEventListener-CzbSevz3.js";import"./Dropdown-Clv_Pvnt.js";import"./index-BZ6Sq__j.js";import"./index-IijEf41L.js";import"./index-2r9flV8X.js";import"./index-B-RBPprY.js";import"./index-CDeD9lyT.js";import"./index-CUuSmggi.js";import"./index-Fy_dqHL_.js";import"./index-CUrh6kIn.js";import"./index-Djhsc5K4.js";import"./study-page-D4nenWxt.js";import"./usePagination-CGdUxo17.js";import"./RedoOutlined-d_QTXv-s.js";import"./index-Cl0Hi3Gl.js";import"./callSuper-CKCGYiRV.js";import"./index-RAxRq3bL.js";import"./index-D0tEL95I.js";import"./index-BfzdirmR.js";import"./DeleteOutlined-DoDR38qq.js";import"./index-5ngX0NHj.js";import"./rgb-BwIoVOhg.js";import"./index-CQOo18Hr.js";const P=()=>e.jsxs(e.Fragment,{children:[e.jsx(m,{items:[{key:"ngs-individual-assembly",label:"二代测序单独组装",children:e.jsx(e.Fragment,{children:e.jsx(i,{inputAnalysisMethod:[{name:"1",label:"V1_single_genome_NGS_DNA",inputKey:["V1_single_genome_NGS_DNA"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]}],analysisMethod:[{name:"1",label:"ngs-individual-assembly",inputKey:["ngs-individual-assembly"],mode:"multiple"}],analysisType:"sample"})})},{key:"tgs_individual_assembly",label:"三代测序单独组装",children:e.jsx(e.Fragment,{children:e.jsx(i,{inputAnalysisMethod:[{name:"1",label:"V1_single_genome_PACBIO_HIFI_DNA",inputKey:["V1_single_genome_PACBIO_HIFI_DNA"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]}],analysisMethod:[{name:"1",label:"tgs_individual_assembly",inputKey:["tgs_individual_assembly"],mode:"multiple"}],analysisType:"sample"})})},{key:"ngs-co-assembly",label:"二代测序混合组装",children:e.jsx(e.Fragment,{})},{key:"ngs-tgs-co-assembly",label:"二三代测序混合组装",children:e.jsx(e.Fragment,{})}]}),e.jsx("p",{})]});export{P as default};
