import{j as o}from"./main-BzPVmAp-.js";import{D as p}from"./study-page-D4nenWxt.js";import{g as c,a as l}from"./index-Djhsc5K4.js";import"./usePagination-CGdUxo17.js";import"./Table-kIlrKW-9.js";import"./addEventListener-CzbSevz3.js";import"./Dropdown-Clv_Pvnt.js";import"./index-BZ6Sq__j.js";import"./index-sAnz6zLe.js";import"./RedoOutlined-d_QTXv-s.js";import"./index-SGIbgpRk.js";const w=({close:t,graphOpt:i,ref:r})=>o.jsx(o.Fragment,{children:o.jsx(p,{columns:({openModal:a,reload:n,messageApi:s})=>{const m=c("association"),e=l("association",a,n,s);return[...m,...e]},hiddenSwitch:!0,ref:r,api:"/entity-relation/page",openModal:i.openModal,close:t})});export{w as default};
