const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["./editor-DPEsIi2T.js","./main-BzPVmAp-.js","./main-DLoYsyAm.css","./editor-D4zjdeIP.css"])))=>i.map(i=>d[i]);
import{r,x as d,_ as i,j as a,D as u,S as h,aT as x}from"./main-BzPVmAp-.js";var m={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M505.7 661a8 8 0 0012.6 0l112-141.7c4.1-5.2.4-12.9-6.3-12.9h-74.1V168c0-4.4-3.6-8-8-8h-60c-4.4 0-8 3.6-8 8v338.3H400c-6.7 0-10.4 7.7-6.3 12.9l112 141.8zM878 626h-60c-4.4 0-8 3.6-8 8v154H214V634c0-4.4-3.6-8-8-8h-60c-4.4 0-8 3.6-8 8v198c0 17.7 14.3 32 32 32h684c17.7 0 32-14.3 32-32V634c0-4.4-3.6-8-8-8z"}}]},name:"download",theme:"outlined"},p=function(o,t){return r.createElement(d,i({},o,{ref:t,icon:m}))},j=r.forwardRef(p);const v=r.lazy(()=>x(()=>import("./editor-DPEsIi2T.js").then(e=>e.e),__vite__mapDeps([0,1,2,3]),import.meta.url)),f=({onChange:e,value:o,editorRef:t,defaultLanguage:n,format:c,height:s})=>a.jsx(a.Fragment,{children:a.jsx(r.Suspense,{fallback:a.jsx(h,{active:!0}),children:a.jsx(v,{onChange:e,value:o,editorRef:t,defaultLanguage:n,format:c,height:s,children:" "})})}),E=({visible:e,onClose:o,value:t,editorRef:n,defaultLanguage:c,format:s,height:l})=>e?a.jsx(u,{open:e,onClose:o,children:a.jsx(f,{value:t,editorRef:n,defaultLanguage:c,format:s,height:l})}):null;export{f as M,j as R,E as a};
