import{k as r,c as o,j as t}from"./main-BzPVmAp-.js";import i from"./index-BDgclIvd.js";import"./index-7u0Bo8e4.js";import"./index-SGIbgpRk.js";import"./Dropdown-Clv_Pvnt.js";import"./utils-D2B0dIuM.js";import"./index-D0tEL95I.js";import"./index-Fy_dqHL_.js";import"./index-BfzdirmR.js";import"./usePagination-CGdUxo17.js";import"./Table-kIlrKW-9.js";import"./addEventListener-CzbSevz3.js";import"./index-BZ6Sq__j.js";import"./index-BQm1iTRH.js";import"./UpOutlined-B_NQN-zy.js";import"./DeleteOutlined-DoDR38qq.js";import"./index-DKqvC_C5.js";import"./RedoOutlined-d_QTXv-s.js";const E=()=>{const{project:p}=r();return o(),t.jsxs(t.Fragment,{children:[t.jsx(i,{}),!1]})};export{E as default};
