import{c3 as v}from"./main-BzPVmAp-.js";function u(e,n,d,i){var s=v.unstable_batchedUpdates?function(o){v.unstable_batchedUpdates(d,o)}:d;return e!=null&&e.addEventListener&&e.addEventListener(n,s,i),{remove:function(){e!=null&&e.removeEventListener&&e.removeEventListener(n,s,i)}}}export{u as a};
