import{s as r,k as m,j as t,B as a,a as p}from"./main-BzPVmAp-.js";import{A as n}from"./index-CkGaCd3X.js";import{f as l}from"./utils-D2B0dIuM.js";import{T as c}from"./index-SGIbgpRk.js";import"./index-BF4b68Hp.js";import"./index-CSq209pq.js";import"./index-sAnz6zLe.js";import"./index-BQm1iTRH.js";import"./UpOutlined-B_NQN-zy.js";import"./index-7u0Bo8e4.js";import"./Table-kIlrKW-9.js";import"./addEventListener-CzbSevz3.js";import"./Dropdown-Clv_Pvnt.js";import"./index-BZ6Sq__j.js";import"./index-IijEf41L.js";import"./index-2r9flV8X.js";import"./index-B-RBPprY.js";import"./index-CDeD9lyT.js";import"./index-CUuSmggi.js";import"./index-Fy_dqHL_.js";import"./index-CUrh6kIn.js";import"./index-Djhsc5K4.js";import"./study-page-D4nenWxt.js";import"./usePagination-CGdUxo17.js";import"./RedoOutlined-d_QTXv-s.js";import"./index-Cl0Hi3Gl.js";import"./callSuper-CKCGYiRV.js";import"./index-RAxRq3bL.js";import"./index-D0tEL95I.js";import"./index-BfzdirmR.js";import"./DeleteOutlined-DoDR38qq.js";import"./index-5ngX0NHj.js";import"./rgb-BwIoVOhg.js";import"./index-CQOo18Hr.js";const J=()=>{const[s,e]=r.useMessage(),{project:o}=m(),i=async()=>{await p.post(`/fast-api/add-sample-analysis?project=${o}`),r.success("导入成功!")};return t.jsxs(t.Fragment,{children:[e,t.jsx(c,{items:[{key:"ngs-sample-qc",label:"NGS样本质控",children:t.jsx(t.Fragment,{children:t.jsx(n,{cardExtra:t.jsx(a,{onClick:i,children:"从样本导入"}),analysisMethod:[{name:"fastp_clean_reads",label:"fastp_clean_reads",inputKey:["fastp_clean_reads"],mode:"multiple"}],analysisPipline:"ngs-sample-qc",inputAnalysisMethod:[{name:"reads",label:"reads",inputKey:["V1_meta_genome_NGS_DNA","V1_single_genome_NGS_DNA"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]}],analysisType:"sample",children:t.jsx(d,{})})})}]}),t.jsx("p",{})]})},d=({plot:s,record:e})=>t.jsx(t.Fragment,{children:e&&t.jsx(t.Fragment,{children:t.jsxs(a,{onClick:()=>{s({url:l(e.content.html),tableDesc:""})},children:["查看报告(",e.sample_name,")"]})})});export{J as default};
