import{g as i,r as e,j as t,B as c,a as f}from"./main-BzPVmAp-.js";const p=()=>{const{softwareId:s}=i(),[o,r]=e.useState(null),a=async()=>{const n=await f.get(`/get-pipeline-v2/${s}?component_type=software`);r(n.data)};return e.useEffect(()=>{a()},[]),t.jsxs("div",{children:[t.jsx(c,{onClick:()=>{a()},children:"刷新"}),s,t.jsx("h1",{children:"Software Page"}),JSON.stringify(o)]})};export{p as default};
