import{c as i,j as e,B as r}from"./main-BzPVmAp-.js";const o=()=>{const t=i();return e.jsxs("div",{style:{},children:[e.jsx(r,{shape:"circle",color:"cyan",variant:"solid",size:"large",style:{position:"fixed",right:"3rem",top:"5rem",zIndex:1e3,boxShadow:"0 4px 10px rgba(0,0,0,0.2)"},onClick:()=>t("/"),children:"Back"}),e.jsx("div",{style:{overflowY:"hidden",height:"calc(100vh)"},children:e.jsx("iframe",{style:{width:"100%",height:"99.5%",border:"none"},title:"BRAVE Docs",src:"https://pybrave.github.io/brave-doc"})})]})};export{o as default};
