import{j as e,r,B as s,d as p,e as l}from"./main-BzPVmAp-.js";import{A as c}from"./index-CkGaCd3X.js";import{B as d}from"./index-Bak0iqJO.js";import{T as g}from"./index-SGIbgpRk.js";import"./index-BF4b68Hp.js";import"./index-CSq209pq.js";import"./index-sAnz6zLe.js";import"./index-BQm1iTRH.js";import"./UpOutlined-B_NQN-zy.js";import"./index-7u0Bo8e4.js";import"./Table-kIlrKW-9.js";import"./addEventListener-CzbSevz3.js";import"./Dropdown-Clv_Pvnt.js";import"./index-BZ6Sq__j.js";import"./index-IijEf41L.js";import"./index-2r9flV8X.js";import"./index-B-RBPprY.js";import"./index-CDeD9lyT.js";import"./index-CUuSmggi.js";import"./index-Fy_dqHL_.js";import"./index-CUrh6kIn.js";import"./index-Djhsc5K4.js";import"./study-page-D4nenWxt.js";import"./usePagination-CGdUxo17.js";import"./RedoOutlined-d_QTXv-s.js";import"./index-Cl0Hi3Gl.js";import"./callSuper-CKCGYiRV.js";import"./index-RAxRq3bL.js";import"./index-D0tEL95I.js";import"./index-BfzdirmR.js";import"./DeleteOutlined-DoDR38qq.js";import"./index-5ngX0NHj.js";import"./rgb-BwIoVOhg.js";import"./index-CQOo18Hr.js";const Z=()=>e.jsxs(e.Fragment,{children:[e.jsx(g,{items:[{key:"gene_count",label:"gene_count",children:e.jsx(e.Fragment,{children:e.jsx(c,{analysisPipline:"gene_count",inputAnalysisMethod:[{name:"rna_seq",label:"RNA测序文件",inputKey:["V1_single_genome_NGS_RNA"],mode:"multiple"},{name:"genome_assembly",label:"基因组参考文件",inputKey:["ngs-individual-assembly","tgs_individual_assembly"],mode:"single"},{name:"prokka",label:"基因组注释文件",inputKey:["prokka"],mode:"single"}],analysisMethod:[{name:"bowtie2_RNA_mapping",label:"bowtie2_RNA_mapping",inputKey:["bowtie2_RNA_mapping"],mode:"multiple"},{name:"feature_counts_RNA_mapping",label:"feature_counts_RNA_mapping",inputKey:["feature_counts_RNA_mapping"],mode:"multiple"}],analysisType:"sample",children:e.jsx(x,{})})})}]}),e.jsx("p",{})]}),x=({resultTableList:t,plot:m,activeTabKey:o})=>{const[i,_]=r.useState([]);r.useEffect(()=>{if(console.log(t.feature_counts_RNA_mapping),t.feature_counts_RNA_mapping){const n=t.feature_counts_RNA_mapping.map(a=>({label:`${a.sample_key} `,value:`${a.id}`}));_(n)}},[t]);const u=n=>t[n].map(a=>a.content.log);return e.jsxs(e.Fragment,{children:[o=="bowtie2_RNA_mapping"&&e.jsx(e.Fragment,{children:e.jsx(d,{resultTableList:t,plot:m,analysisKey:"bowtie2_RNA_mapping"})}),o=="feature_counts_RNA_mapping"&&e.jsxs(e.Fragment,{children:[e.jsx(s,{type:"primary",onClick:()=>{m({saveAnalysisMethod:"feature_counts_RNA_mapping_stat",moduleName:"feature_counts_stat",params:{log_path_list:u("feature_counts_RNA_mapping")},tableDesc:`
                                `})},children:"基因计数统计"}),e.jsx(s,{type:"primary",onClick:()=>{m({name:"差异表达基因分析",saveAnalysisMethod:"feature_counts_RNA_deg",moduleName:"feature_counts_RNA_deg",params:{},formDom:e.jsxs(e.Fragment,{children:[e.jsx(p.Item,{label:"选择处理组",name:"treatment",children:e.jsx(l,{mode:"multiple",style:{maxWidth:"20rem"},allowClear:!0,options:i})}),e.jsx(p.Item,{label:"选择对照组",name:"control",children:e.jsx(l,{mode:"multiple",style:{maxWidth:"20rem"},allowClear:!0,options:i})})]}),tableDesc:`

                                `})},children:"差异表达基因分析"})]})]})};export{Z as default};
