const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["./freemarker2-ipp5yHNJ.js","./main-BzPVmAp-.js","./main-DLoYsyAm.css","./handlebars-CcWMc5kR.js","./html-DhzDJZot.js","./javascript-vcrMQ9kK.js","./typescript-B2FbScha.js","./liquid-wPFVW8nU.js","./mdx-D3eN1vrz.js","./python-DG5AD6tt.js","./razor-D9mSmZ6d.js","./xml-DCi87pgG.js","./yaml-BhpQBQ82.js","./cssMode-BhkQQmYI.js","./htmlMode-DlDHmIFJ.js","./jsonMode-syxg-436.js","./tsMode-DMNyIhlm.js"])))=>i.map(i=>d[i]);
var cX=Object.defineProperty;var dX=(o,e,t)=>e in o?cX(o,e,{enumerable:!0,configurable:!0,writable:!0,value:t}):o[e]=t;var Fi=(o,e,t)=>dX(o,typeof e!="symbol"?e+"":e,t);import{r as Wt,f as Bp,aT as ye,j as QE}from"./main-BzPVmAp-.js";function hX(o,e,t){return e in o?Object.defineProperty(o,e,{value:t,enumerable:!0,configurable:!0,writable:!0}):o[e]=t,o}function TB(o,e){var t=Object.keys(o);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(o);e&&(i=i.filter(function(n){return Object.getOwnPropertyDescriptor(o,n).enumerable})),t.push.apply(t,i)}return t}function RB(o){for(var e=1;e<arguments.length;e++){var t=arguments[e]!=null?arguments[e]:{};e%2?TB(Object(t),!0).forEach(function(i){hX(o,i,t[i])}):Object.getOwnPropertyDescriptors?Object.defineProperties(o,Object.getOwnPropertyDescriptors(t)):TB(Object(t)).forEach(function(i){Object.defineProperty(o,i,Object.getOwnPropertyDescriptor(t,i))})}return o}function uX(o,e){if(o==null)return{};var t={},i=Object.keys(o),n,s;for(s=0;s<i.length;s++)n=i[s],!(e.indexOf(n)>=0)&&(t[n]=o[n]);return t}function gX(o,e){if(o==null)return{};var t=uX(o,e),i,n;if(Object.getOwnPropertySymbols){var s=Object.getOwnPropertySymbols(o);for(n=0;n<s.length;n++)i=s[n],!(e.indexOf(i)>=0)&&Object.prototype.propertyIsEnumerable.call(o,i)&&(t[i]=o[i])}return t}function fX(o,e){return mX(o)||pX(o,e)||_X(o,e)||bX()}function mX(o){if(Array.isArray(o))return o}function pX(o,e){if(!(typeof Symbol>"u"||!(Symbol.iterator in Object(o)))){var t=[],i=!0,n=!1,s=void 0;try{for(var r=o[Symbol.iterator](),a;!(i=(a=r.next()).done)&&(t.push(a.value),!(e&&t.length===e));i=!0);}catch(l){n=!0,s=l}finally{try{!i&&r.return!=null&&r.return()}finally{if(n)throw s}}return t}}function _X(o,e){if(o){if(typeof o=="string")return MB(o,e);var t=Object.prototype.toString.call(o).slice(8,-1);if(t==="Object"&&o.constructor&&(t=o.constructor.name),t==="Map"||t==="Set")return Array.from(o);if(t==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t))return MB(o,e)}}function MB(o,e){(e==null||e>o.length)&&(e=o.length);for(var t=0,i=new Array(e);t<e;t++)i[t]=o[t];return i}function bX(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function CX(o,e,t){return e in o?Object.defineProperty(o,e,{value:t,enumerable:!0,configurable:!0,writable:!0}):o[e]=t,o}function AB(o,e){var t=Object.keys(o);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(o);e&&(i=i.filter(function(n){return Object.getOwnPropertyDescriptor(o,n).enumerable})),t.push.apply(t,i)}return t}function PB(o){for(var e=1;e<arguments.length;e++){var t=arguments[e]!=null?arguments[e]:{};e%2?AB(Object(t),!0).forEach(function(i){CX(o,i,t[i])}):Object.getOwnPropertyDescriptors?Object.defineProperties(o,Object.getOwnPropertyDescriptors(t)):AB(Object(t)).forEach(function(i){Object.defineProperty(o,i,Object.getOwnPropertyDescriptor(t,i))})}return o}function vX(){for(var o=arguments.length,e=new Array(o),t=0;t<o;t++)e[t]=arguments[t];return function(i){return e.reduceRight(function(n,s){return s(n)},i)}}function CC(o){return function e(){for(var t=this,i=arguments.length,n=new Array(i),s=0;s<i;s++)n[s]=arguments[s];return n.length>=o.length?o.apply(this,n):function(){for(var r=arguments.length,a=new Array(r),l=0;l<r;l++)a[l]=arguments[l];return e.apply(t,[].concat(n,a))}}}function xy(o){return{}.toString.call(o).includes("Object")}function wX(o){return!Object.keys(o).length}function Gv(o){return typeof o=="function"}function SX(o,e){return Object.prototype.hasOwnProperty.call(o,e)}function yX(o,e){return xy(e)||rh("changeType"),Object.keys(e).some(function(t){return!SX(o,t)})&&rh("changeField"),e}function LX(o){Gv(o)||rh("selectorType")}function xX(o){Gv(o)||xy(o)||rh("handlerType"),xy(o)&&Object.values(o).some(function(e){return!Gv(e)})&&rh("handlersType")}function kX(o){o||rh("initialIsRequired"),xy(o)||rh("initialType"),wX(o)&&rh("initialContent")}function DX(o,e){throw new Error(o[e]||o.default)}var IX={initialIsRequired:"initial state is required",initialType:"initial state should be an object",initialContent:"initial state shouldn't be an empty object",handlerType:"handler should be an object or a function",handlersType:"all handlers should be a functions",selectorType:"selector should be a function",changeType:"provided value of changes should be an object",changeField:'it seams you want to change a field in the state which is not specified in the "initial" state',default:"an unknown error accured in `state-local` package"},rh=CC(DX)(IX),SS={changes:yX,selector:LX,handler:xX,initial:kX};function EX(o){var e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};SS.initial(o),SS.handler(e);var t={current:o},i=CC(RX)(t,e),n=CC(TX)(t),s=CC(SS.changes)(o),r=CC(NX)(t);function a(){var c=arguments.length>0&&arguments[0]!==void 0?arguments[0]:function(d){return d};return SS.selector(c),c(t.current)}function l(c){vX(i,n,s,r)(c)}return[a,l]}function NX(o,e){return Gv(e)?e(o.current):e}function TX(o,e){return o.current=PB(PB({},o.current),e),e}function RX(o,e,t){return Gv(e)?e(o.current):Object.keys(t).forEach(function(i){var n;return(n=e[i])===null||n===void 0?void 0:n.call(e,o.current[i])}),t}var MX={create:EX},AX={paths:{vs:"https://cdn.jsdelivr.net/npm/monaco-editor@0.52.2/min/vs"}};function PX(o){return function e(){for(var t=this,i=arguments.length,n=new Array(i),s=0;s<i;s++)n[s]=arguments[s];return n.length>=o.length?o.apply(this,n):function(){for(var r=arguments.length,a=new Array(r),l=0;l<r;l++)a[l]=arguments[l];return e.apply(t,[].concat(n,a))}}}function OX(o){return{}.toString.call(o).includes("Object")}function FX(o){return o||OB("configIsRequired"),OX(o)||OB("configType"),o.urls?(WX(),{paths:{vs:o.urls.monacoBase}}):o}function WX(){console.warn(hz.deprecation)}function BX(o,e){throw new Error(o[e]||o.default)}var hz={configIsRequired:"the configuration object is required",configType:"the configuration object should be an object",default:"an unknown error accured in `@monaco-editor/loader` package",deprecation:`Deprecation warning!
    You are using deprecated way of configuration.

    Instead of using
      monaco.config({ urls: { monacoBase: '...' } })
    use
      monaco.config({ paths: { vs: '...' } })

    For more please check the link https://github.com/suren-atoyan/monaco-loader#config
  `},OB=PX(BX)(hz),HX={config:FX},VX=function(){for(var e=arguments.length,t=new Array(e),i=0;i<e;i++)t[i]=arguments[i];return function(n){return t.reduceRight(function(s,r){return r(s)},n)}};function uz(o,e){return Object.keys(e).forEach(function(t){e[t]instanceof Object&&o[t]&&Object.assign(e[t],uz(o[t],e[t]))}),RB(RB({},o),e)}var zX={type:"cancelation",msg:"operation is manually canceled"};function JE(o){var e=!1,t=new Promise(function(i,n){o.then(function(s){return e?n(zX):i(s)}),o.catch(n)});return t.cancel=function(){return e=!0},t}var UX=MX.create({config:AX,isInitialized:!1,resolve:null,reject:null,monaco:null}),gz=fX(UX,2),N1=gz[0],lI=gz[1];function $X(o){var e=HX.config(o),t=e.monaco,i=gX(e,["monaco"]);lI(function(n){return{config:uz(n.config,i),monaco:t}})}function jX(){var o=N1(function(e){var t=e.monaco,i=e.isInitialized,n=e.resolve;return{monaco:t,isInitialized:i,resolve:n}});if(!o.isInitialized){if(lI({isInitialized:!0}),o.monaco)return o.resolve(o.monaco),JE(eN);if(window.monaco&&window.monaco.editor)return fz(window.monaco),o.resolve(window.monaco),JE(eN);VX(KX,GX)(ZX)}return JE(eN)}function KX(o){return document.body.appendChild(o)}function qX(o){var e=document.createElement("script");return o&&(e.src=o),e}function GX(o){var e=N1(function(i){var n=i.config,s=i.reject;return{config:n,reject:s}}),t=qX("".concat(e.config.paths.vs,"/loader.js"));return t.onload=function(){return o()},t.onerror=e.reject,t}function ZX(){var o=N1(function(t){var i=t.config,n=t.resolve,s=t.reject;return{config:i,resolve:n,reject:s}}),e=window.require;e.config(o.config),e(["vs/editor/editor.main"],function(t){fz(t),o.resolve(t)},function(t){o.reject(t)})}function fz(o){N1().monaco||lI({monaco:o})}function YX(){return N1(function(o){var e=o.monaco;return e})}var eN=new Promise(function(o,e){return lI({resolve:o,reject:e})}),A4={config:$X,init:jX,__getMonacoInstance:YX},XX={wrapper:{display:"flex",position:"relative",textAlign:"initial"},fullWidth:{width:"100%"},hide:{display:"none"}},tN=XX,QX={container:{display:"flex",height:"100%",width:"100%",justifyContent:"center",alignItems:"center"}},JX=QX;function eQ({children:o}){return Bp.createElement("div",{style:JX.container},o)}var tQ=eQ,iQ=tQ;function nQ({width:o,height:e,isEditorReady:t,loading:i,_ref:n,className:s,wrapperProps:r}){return Bp.createElement("section",{style:{...tN.wrapper,width:o,height:e},...r},!t&&Bp.createElement(iQ,null,i),Bp.createElement("div",{ref:n,style:{...tN.fullWidth,...!t&&tN.hide},className:s}))}var sQ=nQ,mz=Wt.memo(sQ);function oQ(o){Wt.useEffect(o,[])}var pz=oQ;function rQ(o,e,t=!0){let i=Wt.useRef(!0);Wt.useEffect(i.current||!t?()=>{i.current=!1}:o,e)}var rr=rQ;function UC(){}function Xm(o,e,t,i){return aQ(o,i)||lQ(o,e,t,i)}function aQ(o,e){return o.editor.getModel(_z(o,e))}function lQ(o,e,t,i){return o.editor.createModel(e,t,i?_z(o,i):void 0)}function _z(o,e){return o.Uri.parse(e)}function cQ({original:o,modified:e,language:t,originalLanguage:i,modifiedLanguage:n,originalModelPath:s,modifiedModelPath:r,keepCurrentOriginalModel:a=!1,keepCurrentModifiedModel:l=!1,theme:c="light",loading:d="Loading...",options:h={},height:u="100%",width:g="100%",className:f,wrapperProps:p={},beforeMount:_=UC,onMount:b=UC}){let[C,v]=Wt.useState(!1),[w,S]=Wt.useState(!0),L=Wt.useRef(null),k=Wt.useRef(null),D=Wt.useRef(null),N=Wt.useRef(b),B=Wt.useRef(_),W=Wt.useRef(!1);pz(()=>{let j=A4.init();return j.then(J=>(k.current=J)&&S(!1)).catch(J=>(J==null?void 0:J.type)!=="cancelation"&&console.error("Monaco initialization: error:",J)),()=>L.current?H():j.cancel()}),rr(()=>{if(L.current&&k.current){let j=L.current.getOriginalEditor(),J=Xm(k.current,o||"",i||t||"text",s||"");J!==j.getModel()&&j.setModel(J)}},[s],C),rr(()=>{if(L.current&&k.current){let j=L.current.getModifiedEditor(),J=Xm(k.current,e||"",n||t||"text",r||"");J!==j.getModel()&&j.setModel(J)}},[r],C),rr(()=>{let j=L.current.getModifiedEditor();j.getOption(k.current.editor.EditorOption.readOnly)?j.setValue(e||""):e!==j.getValue()&&(j.executeEdits("",[{range:j.getModel().getFullModelRange(),text:e||"",forceMoveMarkers:!0}]),j.pushUndoStop())},[e],C),rr(()=>{var j,J;(J=(j=L.current)==null?void 0:j.getModel())==null||J.original.setValue(o||"")},[o],C),rr(()=>{let{original:j,modified:J}=L.current.getModel();k.current.editor.setModelLanguage(j,i||t||"text"),k.current.editor.setModelLanguage(J,n||t||"text")},[t,i,n],C),rr(()=>{var j;(j=k.current)==null||j.editor.setTheme(c)},[c],C),rr(()=>{var j;(j=L.current)==null||j.updateOptions(h)},[h],C);let V=Wt.useCallback(()=>{var he;if(!k.current)return;B.current(k.current);let j=Xm(k.current,o||"",i||t||"text",s||""),J=Xm(k.current,e||"",n||t||"text",r||"");(he=L.current)==null||he.setModel({original:j,modified:J})},[t,e,n,o,i,s,r]),$=Wt.useCallback(()=>{var j;!W.current&&D.current&&(L.current=k.current.editor.createDiffEditor(D.current,{automaticLayout:!0,...h}),V(),(j=k.current)==null||j.editor.setTheme(c),v(!0),W.current=!0)},[h,c,V]);Wt.useEffect(()=>{C&&N.current(L.current,k.current)},[C]),Wt.useEffect(()=>{!w&&!C&&$()},[w,C,$]);function H(){var J,he,fe,ue;let j=(J=L.current)==null?void 0:J.getModel();a||((he=j==null?void 0:j.original)==null||he.dispose()),l||((fe=j==null?void 0:j.modified)==null||fe.dispose()),(ue=L.current)==null||ue.dispose()}return Bp.createElement(mz,{width:g,height:u,isEditorReady:C,loading:d,_ref:D,className:f,wrapperProps:p})}var dQ=cQ;Wt.memo(dQ);function hQ(o){let e=Wt.useRef();return Wt.useEffect(()=>{e.current=o},[o]),e.current}var uQ=hQ,yS=new Map;function gQ({defaultValue:o,defaultLanguage:e,defaultPath:t,value:i,language:n,path:s,theme:r="light",line:a,loading:l="Loading...",options:c={},overrideServices:d={},saveViewState:h=!0,keepCurrentModel:u=!1,width:g="100%",height:f="100%",className:p,wrapperProps:_={},beforeMount:b=UC,onMount:C=UC,onChange:v,onValidate:w=UC}){let[S,L]=Wt.useState(!1),[k,D]=Wt.useState(!0),N=Wt.useRef(null),B=Wt.useRef(null),W=Wt.useRef(null),V=Wt.useRef(C),$=Wt.useRef(b),H=Wt.useRef(),j=Wt.useRef(i),J=uQ(s),he=Wt.useRef(!1),fe=Wt.useRef(!1);pz(()=>{let _e=A4.init();return _e.then(it=>(N.current=it)&&D(!1)).catch(it=>(it==null?void 0:it.type)!=="cancelation"&&console.error("Monaco initialization: error:",it)),()=>B.current?Pe():_e.cancel()}),rr(()=>{var it,Ne,Ct,ht;let _e=Xm(N.current,o||i||"",e||n||"",s||t||"");_e!==((it=B.current)==null?void 0:it.getModel())&&(h&&yS.set(J,(Ne=B.current)==null?void 0:Ne.saveViewState()),(Ct=B.current)==null||Ct.setModel(_e),h&&((ht=B.current)==null||ht.restoreViewState(yS.get(s))))},[s],S),rr(()=>{var _e;(_e=B.current)==null||_e.updateOptions(c)},[c],S),rr(()=>{!B.current||i===void 0||(B.current.getOption(N.current.editor.EditorOption.readOnly)?B.current.setValue(i):i!==B.current.getValue()&&(fe.current=!0,B.current.executeEdits("",[{range:B.current.getModel().getFullModelRange(),text:i,forceMoveMarkers:!0}]),B.current.pushUndoStop(),fe.current=!1))},[i],S),rr(()=>{var it,Ne;let _e=(it=B.current)==null?void 0:it.getModel();_e&&n&&((Ne=N.current)==null||Ne.editor.setModelLanguage(_e,n))},[n],S),rr(()=>{var _e;a!==void 0&&((_e=B.current)==null||_e.revealLine(a))},[a],S),rr(()=>{var _e;(_e=N.current)==null||_e.editor.setTheme(r)},[r],S);let ue=Wt.useCallback(()=>{var _e;if(!(!W.current||!N.current)&&!he.current){$.current(N.current);let it=s||t,Ne=Xm(N.current,i||o||"",e||n||"",it||"");B.current=(_e=N.current)==null?void 0:_e.editor.create(W.current,{model:Ne,automaticLayout:!0,...c},d),h&&B.current.restoreViewState(yS.get(it)),N.current.editor.setTheme(r),a!==void 0&&B.current.revealLine(a),L(!0),he.current=!0}},[o,e,t,i,n,s,c,d,h,r,a]);Wt.useEffect(()=>{S&&V.current(B.current,N.current)},[S]),Wt.useEffect(()=>{!k&&!S&&ue()},[k,S,ue]),j.current=i,Wt.useEffect(()=>{var _e,it;S&&v&&((_e=H.current)==null||_e.dispose(),H.current=(it=B.current)==null?void 0:it.onDidChangeModelContent(Ne=>{fe.current||v(B.current.getValue(),Ne)}))},[S,v]),Wt.useEffect(()=>{if(S){let _e=N.current.editor.onDidChangeMarkers(it=>{var Ct;let Ne=(Ct=B.current.getModel())==null?void 0:Ct.uri;if(Ne&&it.find(ht=>ht.path===Ne.path)){let ht=N.current.editor.getModelMarkers({resource:Ne});w==null||w(ht)}});return()=>{_e==null||_e.dispose()}}return()=>{}},[S,w]);function Pe(){var _e,it;(_e=H.current)==null||_e.dispose(),u?h&&yS.set(s,B.current.saveViewState()):(it=B.current.getModel())==null||it.dispose(),B.current.dispose()}return Bp.createElement(mz,{width:g,height:f,isEditorReady:S,loading:l,_ref:W,className:p,wrapperProps:_})}var fQ=gQ,mQ=Wt.memo(fQ),pQ=mQ;function tr(o,e=0){return o[o.length-(1+e)]}function _Q(o){if(o.length===0)throw new Error("Invalid tail call");return[o.slice(0,o.length-1),o[o.length-1]]}function ii(o,e,t=(i,n)=>i===n){if(o===e)return!0;if(!o||!e||o.length!==e.length)return!1;for(let i=0,n=o.length;i<n;i++)if(!t(o[i],e[i]))return!1;return!0}function bQ(o,e){const t=o.length-1;e<t&&(o[e]=o[t]),o.pop()}function Zv(o,e,t){return CQ(o.length,i=>t(o[i],e))}function CQ(o,e){let t=0,i=o-1;for(;t<=i;){const n=(t+i)/2|0,s=e(n);if(s<0)t=n+1;else if(s>0)i=n-1;else return n}return-(t+1)}function dR(o,e,t){if(o=o|0,o>=e.length)throw new TypeError("invalid index");const i=e[Math.floor(e.length*Math.random())],n=[],s=[],r=[];for(const a of e){const l=t(a,i);l<0?n.push(a):l>0?s.push(a):r.push(a)}return o<n.length?dR(o,n,t):o<n.length+r.length?r[0]:dR(o-(n.length+r.length),s,t)}function FB(o,e){const t=[];let i;for(const n of o.slice(0).sort(e))!i||e(i[0],n)!==0?(i=[n],t.push(i)):i.push(n);return t}function*P4(o,e){let t,i;for(const n of o)i!==void 0&&e(i,n)?t.push(n):(t&&(yield t),t=[n]),i=n;t&&(yield t)}function bz(o,e){for(let t=0;t<=o.length;t++)e(t===0?void 0:o[t-1],t===o.length?void 0:o[t])}function vQ(o,e){for(let t=0;t<o.length;t++)e(t===0?void 0:o[t-1],o[t],t+1===o.length?void 0:o[t+1])}function Kr(o){return o.filter(e=>!!e)}function WB(o){let e=0;for(let t=0;t<o.length;t++)o[t]&&(o[e]=o[t],e+=1);o.length=e}function Cz(o){return!Array.isArray(o)||o.length===0}function Qn(o){return Array.isArray(o)&&o.length>0}function Xc(o,e=t=>t){const t=new Set;return o.filter(i=>{const n=e(i);return t.has(n)?!1:(t.add(n),!0)})}function O4(o,e){return o.length>0?o[0]:e}function Is(o,e){let t=typeof e=="number"?o:0;typeof e=="number"?t=o:(t=0,e=o);const i=[];if(t<=e)for(let n=t;n<e;n++)i.push(n);else for(let n=t;n>e;n--)i.push(n);return i}function cI(o,e,t){const i=o.slice(0,e),n=o.slice(e);return i.concat(t,n)}function iN(o,e){const t=o.indexOf(e);t>-1&&(o.splice(t,1),o.unshift(e))}function LS(o,e){const t=o.indexOf(e);t>-1&&(o.splice(t,1),o.push(e))}function hR(o,e){for(const t of e)o.push(t)}function F4(o){return Array.isArray(o)?o:[o]}function wQ(o,e,t){const i=vz(o,e),n=o.length,s=t.length;o.length=n+s;for(let r=n-1;r>=i;r--)o[r+s]=o[r];for(let r=0;r<s;r++)o[r+i]=t[r]}function BB(o,e,t,i){const n=vz(o,e);let s=o.splice(n,t);return s===void 0&&(s=[]),wQ(o,n,i),s}function vz(o,e){return e<0?Math.max(e+o.length,0):Math.min(e,o.length)}var Yv;(function(o){function e(s){return s<0}o.isLessThan=e;function t(s){return s<=0}o.isLessThanOrEqual=t;function i(s){return s>0}o.isGreaterThan=i;function n(s){return s===0}o.isNeitherLessOrGreaterThan=n,o.greaterThan=1,o.lessThan=-1,o.neitherLessOrGreaterThan=0})(Yv||(Yv={}));function gs(o,e){return(t,i)=>e(o(t),o(i))}function SQ(...o){return(e,t)=>{for(const i of o){const n=i(e,t);if(!Yv.isNeitherLessOrGreaterThan(n))return n}return Yv.neitherLessOrGreaterThan}}const $r=(o,e)=>o-e,yQ=(o,e)=>$r(o?1:0,e?1:0);function wz(o){return(e,t)=>-o(e,t)}class Qc{constructor(e){this.items=e,this.firstIdx=0,this.lastIdx=this.items.length-1}get length(){return this.lastIdx-this.firstIdx+1}takeWhile(e){let t=this.firstIdx;for(;t<this.items.length&&e(this.items[t]);)t++;const i=t===this.firstIdx?null:this.items.slice(this.firstIdx,t);return this.firstIdx=t,i}takeFromEndWhile(e){let t=this.lastIdx;for(;t>=0&&e(this.items[t]);)t--;const i=t===this.lastIdx?null:this.items.slice(t+1,this.lastIdx+1);return this.lastIdx=t,i}peek(){if(this.length!==0)return this.items[this.firstIdx]}dequeue(){const e=this.items[this.firstIdx];return this.firstIdx++,e}takeCount(e){const t=this.items.slice(this.firstIdx,this.firstIdx+e);return this.firstIdx+=e,t}}const yp=class yp{constructor(e){this.iterate=e}toArray(){const e=[];return this.iterate(t=>(e.push(t),!0)),e}filter(e){return new yp(t=>this.iterate(i=>e(i)?t(i):!0))}map(e){return new yp(t=>this.iterate(i=>t(e(i))))}findLast(e){let t;return this.iterate(i=>(e(i)&&(t=i),!0)),t}findLastMaxBy(e){let t,i=!0;return this.iterate(n=>((i||Yv.isGreaterThan(e(n,t)))&&(i=!1,t=n),!0)),t}};yp.empty=new yp(e=>{});let gg=yp;class ky{constructor(e){this._indexMap=e}static createSortPermutation(e,t){const i=Array.from(e.keys()).sort((n,s)=>t(e[n],e[s]));return new ky(i)}apply(e){return e.map((t,i)=>e[this._indexMap[i]])}inverse(){const e=this._indexMap.slice();for(let t=0;t<this._indexMap.length;t++)e[this._indexMap[t]]=t;return new ky(e)}}function As(o){return typeof o=="string"}function on(o){return typeof o=="object"&&o!==null&&!Array.isArray(o)&&!(o instanceof RegExp)&&!(o instanceof Date)}function LQ(o){const e=Object.getPrototypeOf(Uint8Array);return typeof o=="object"&&o instanceof e}function Rh(o){return typeof o=="number"&&!isNaN(o)}function HB(o){return!!o&&typeof o[Symbol.iterator]=="function"}function Sz(o){return o===!0||o===!1}function Xn(o){return typeof o>"u"}function Tl(o){return!Ro(o)}function Ro(o){return Xn(o)||o===null}function mt(o,e){if(!o)throw new Error(e?`Unexpected type, expected '${e}'`:"Unexpected type")}function Kd(o){if(Ro(o))throw new Error("Assertion Failed: argument is undefined or null");return o}function Xv(o){return typeof o=="function"}function xQ(o,e){const t=Math.min(o.length,e.length);for(let i=0;i<t;i++)kQ(o[i],e[i])}function kQ(o,e){if(As(e)){if(typeof o!==e)throw new Error(`argument does not match constraint: typeof ${e}`)}else if(Xv(e)){try{if(o instanceof e)return}catch{}if(!Ro(o)&&o.constructor===e||e.length===1&&e.call(void 0,o)===!0)return;throw new Error("argument does not match one of these constraints: arg instanceof constraint, arg.constructor === constraint, nor constraint(arg) === true")}}function vc(o){if(!o||typeof o!="object"||o instanceof RegExp)return o;const e=Array.isArray(o)?[]:{};return Object.entries(o).forEach(([t,i])=>{e[t]=i&&typeof i=="object"?vc(i):i}),e}function DQ(o){if(!o||typeof o!="object")return o;const e=[o];for(;e.length>0;){const t=e.shift();Object.freeze(t);for(const i in t)if(yz.call(t,i)){const n=t[i];typeof n=="object"&&!Object.isFrozen(n)&&!LQ(n)&&e.push(n)}}return o}const yz=Object.prototype.hasOwnProperty;function Lz(o,e){return uR(o,e,new Set)}function uR(o,e,t){if(Ro(o))return o;const i=e(o);if(typeof i<"u")return i;if(Array.isArray(o)){const n=[];for(const s of o)n.push(uR(s,e,t));return n}if(on(o)){if(t.has(o))throw new Error("Cannot clone recursive data-structure");t.add(o);const n={};for(const s in o)yz.call(o,s)&&(n[s]=uR(o[s],e,t));return t.delete(o),n}return o}function dI(o,e,t=!0){return on(o)?(on(e)&&Object.keys(e).forEach(i=>{i in o?t&&(on(o[i])&&on(e[i])?dI(o[i],e[i],t):o[i]=e[i]):o[i]=e[i]}),o):e}function oo(o,e){if(o===e)return!0;if(o==null||e===null||e===void 0||typeof o!=typeof e||typeof o!="object"||Array.isArray(o)!==Array.isArray(e))return!1;let t,i;if(Array.isArray(o)){if(o.length!==e.length)return!1;for(t=0;t<o.length;t++)if(!oo(o[t],e[t]))return!1}else{const n=[];for(i in o)n.push(i);n.sort();const s=[];for(i in e)s.push(i);if(s.sort(),!oo(n,s))return!1;for(t=0;t<n.length;t++)if(!oo(o[n[t]],e[n[t]]))return!1}return!0}function IQ(o){let e=[];for(;Object.prototype!==o;)e=e.concat(Object.getOwnPropertyNames(o)),o=Object.getPrototypeOf(o);return e}function gR(o){const e=[];for(const t of IQ(o))typeof o[t]=="function"&&e.push(t);return e}function EQ(o,e){const t=n=>function(){const s=Array.prototype.slice.call(arguments,0);return e(n,s)},i={};for(const n of o)i[n]=t(n);return i}function xz(){return globalThis._VSCODE_NLS_MESSAGES}function W4(){return globalThis._VSCODE_NLS_LANGUAGE}const NQ=W4()==="pseudo"||typeof document<"u"&&document.location&&document.location.hash.indexOf("pseudo=true")>=0;function Dy(o,e){let t;return e.length===0?t=o:t=o.replace(/\{(\d+)\}/g,(i,n)=>{const s=n[0],r=e[s];let a=i;return typeof r=="string"?a=r:(typeof r=="number"||typeof r=="boolean"||r===void 0||r===null)&&(a=String(r)),a}),NQ&&(t="［"+t.replace(/[aouei]/g,"$&$&")+"］"),t}function m(o,e,...t){return Dy(typeof o=="number"?kz(o,e):e,t)}function kz(o,e){var i;const t=(i=xz())==null?void 0:i[o];if(typeof t!="string"){if(typeof e=="string")return e;throw new Error(`!!! NLS MISSING: ${o} !!!`)}return t}function Te(o,e,...t){let i;typeof o=="number"?i=kz(o,e):i=e;const n=Dy(i,t);return{value:n,original:e===i?n:Dy(e,t)}}const Qm="en";let Iy=!1,Ey=!1,F0=!1,Dz=!1,B4=!1,H4=!1,Iz=!1,xS,W0=Qm,VB=Qm,TQ,oa;const $c=globalThis;let ss;var az;typeof $c.vscode<"u"&&typeof $c.vscode.process<"u"?ss=$c.vscode.process:typeof process<"u"&&typeof((az=process==null?void 0:process.versions)==null?void 0:az.node)=="string"&&(ss=process);var lz;const RQ=typeof((lz=ss==null?void 0:ss.versions)==null?void 0:lz.electron)=="string",MQ=RQ&&(ss==null?void 0:ss.type)==="renderer";var cz;if(typeof ss=="object"){Iy=ss.platform==="win32",Ey=ss.platform==="darwin",F0=ss.platform==="linux",F0&&ss.env.SNAP&&ss.env.SNAP_REVISION,ss.env.CI||ss.env.BUILD_ARTIFACTSTAGINGDIRECTORY,xS=Qm,W0=Qm;const o=ss.env.VSCODE_NLS_CONFIG;if(o)try{const e=JSON.parse(o);xS=e.userLocale,VB=e.osLocale,W0=e.resolvedLanguage||Qm,TQ=(cz=e.languagePack)==null?void 0:cz.translationsConfigFile}catch{}Dz=!0}else typeof navigator=="object"&&!MQ?(oa=navigator.userAgent,Iy=oa.indexOf("Windows")>=0,Ey=oa.indexOf("Macintosh")>=0,H4=(oa.indexOf("Macintosh")>=0||oa.indexOf("iPad")>=0||oa.indexOf("iPhone")>=0)&&!!navigator.maxTouchPoints&&navigator.maxTouchPoints>0,F0=oa.indexOf("Linux")>=0,Iz=(oa==null?void 0:oa.indexOf("Mobi"))>=0,B4=!0,W0=W4()||Qm,xS=navigator.language.toLowerCase(),VB=xS):console.error("Unable to resolve platform.");const En=Iy,Je=Ey,ps=F0,Pl=Dz,Qf=B4,AQ=B4&&typeof $c.importScripts=="function",PQ=AQ?$c.origin:void 0,Pa=H4,Ez=Iz,Ol=oa,OQ=W0,FQ=typeof $c.postMessage=="function"&&!$c.importScripts,Nz=(()=>{if(FQ){const o=[];$c.addEventListener("message",t=>{if(t.data&&t.data.vscodeScheduleAsyncWork)for(let i=0,n=o.length;i<n;i++){const s=o[i];if(s.id===t.data.vscodeScheduleAsyncWork){o.splice(i,1),s.callback();return}}});let e=0;return t=>{const i=++e;o.push({id:i,callback:t}),$c.postMessage({vscodeScheduleAsyncWork:i},"*")}}return o=>setTimeout(o)})(),so=Ey||H4?2:Iy?1:3;let zB=!0,UB=!1;function Tz(){if(!UB){UB=!0;const o=new Uint8Array(2);o[0]=1,o[1]=2,zB=new Uint16Array(o.buffer)[0]===513}return zB}const Rz=!!(Ol&&Ol.indexOf("Chrome")>=0),WQ=!!(Ol&&Ol.indexOf("Firefox")>=0),BQ=!!(!Rz&&Ol&&Ol.indexOf("Safari")>=0),HQ=!!(Ol&&Ol.indexOf("Edg/")>=0),VQ=!!(Ol&&Ol.indexOf("Android")>=0),An={tabSize:4,indentSize:4,insertSpaces:!0,detectIndentation:!0,trimAutoWhitespace:!0,largeFileOptimizations:!0,bracketPairColorizationOptions:{enabled:!0,independentColorPoolPerBracketType:!1}};var ot;(function(o){function e(w){return w&&typeof w=="object"&&typeof w[Symbol.iterator]=="function"}o.is=e;const t=Object.freeze([]);function i(){return t}o.empty=i;function*n(w){yield w}o.single=n;function s(w){return e(w)?w:n(w)}o.wrap=s;function r(w){return w||t}o.from=r;function*a(w){for(let S=w.length-1;S>=0;S--)yield w[S]}o.reverse=a;function l(w){return!w||w[Symbol.iterator]().next().done===!0}o.isEmpty=l;function c(w){return w[Symbol.iterator]().next().value}o.first=c;function d(w,S){let L=0;for(const k of w)if(S(k,L++))return!0;return!1}o.some=d;function h(w,S){for(const L of w)if(S(L))return L}o.find=h;function*u(w,S){for(const L of w)S(L)&&(yield L)}o.filter=u;function*g(w,S){let L=0;for(const k of w)yield S(k,L++)}o.map=g;function*f(w,S){let L=0;for(const k of w)yield*S(k,L++)}o.flatMap=f;function*p(...w){for(const S of w)yield*S}o.concat=p;function _(w,S,L){let k=L;for(const D of w)k=S(k,D);return k}o.reduce=_;function*b(w,S,L=w.length){for(S<0&&(S+=w.length),L<0?L+=w.length:L>w.length&&(L=w.length);S<L;S++)yield w[S]}o.slice=b;function C(w,S=Number.POSITIVE_INFINITY){const L=[];if(S===0)return[L,w];const k=w[Symbol.iterator]();for(let D=0;D<S;D++){const N=k.next();if(N.done)return[L,o.empty()];L.push(N.value)}return[L,{[Symbol.iterator](){return k}}]}o.consume=C;async function v(w){const S=[];for await(const L of w)S.push(L);return Promise.resolve(S)}o.asyncToArray=v})(ot||(ot={}));var Wc;let Ki=(Wc=class{constructor(e){this.element=e,this.next=Wc.Undefined,this.prev=Wc.Undefined}},Wc.Undefined=new Wc(void 0),Wc);class cs{constructor(){this._first=Ki.Undefined,this._last=Ki.Undefined,this._size=0}get size(){return this._size}isEmpty(){return this._first===Ki.Undefined}clear(){let e=this._first;for(;e!==Ki.Undefined;){const t=e.next;e.prev=Ki.Undefined,e.next=Ki.Undefined,e=t}this._first=Ki.Undefined,this._last=Ki.Undefined,this._size=0}unshift(e){return this._insert(e,!1)}push(e){return this._insert(e,!0)}_insert(e,t){const i=new Ki(e);if(this._first===Ki.Undefined)this._first=i,this._last=i;else if(t){const s=this._last;this._last=i,i.prev=s,s.next=i}else{const s=this._first;this._first=i,i.next=s,s.prev=i}this._size+=1;let n=!1;return()=>{n||(n=!0,this._remove(i))}}shift(){if(this._first!==Ki.Undefined){const e=this._first.element;return this._remove(this._first),e}}pop(){if(this._last!==Ki.Undefined){const e=this._last.element;return this._remove(this._last),e}}_remove(e){if(e.prev!==Ki.Undefined&&e.next!==Ki.Undefined){const t=e.prev;t.next=e.next,e.next.prev=t}else e.prev===Ki.Undefined&&e.next===Ki.Undefined?(this._first=Ki.Undefined,this._last=Ki.Undefined):e.next===Ki.Undefined?(this._last=this._last.prev,this._last.next=Ki.Undefined):e.prev===Ki.Undefined&&(this._first=this._first.next,this._first.prev=Ki.Undefined);this._size-=1}*[Symbol.iterator](){let e=this._first;for(;e!==Ki.Undefined;)yield e.element,e=e.next}}const Ny="`~!@#$%^&*()-=+[{]}\\|;:'\",.<>/?";function zQ(o=""){let e="(-?\\d*\\.\\d\\w*)|([^";for(const t of Ny)o.indexOf(t)>=0||(e+="\\"+t);return e+="\\s]+)",new RegExp(e,"g")}const V4=zQ();function z4(o){let e=V4;if(o&&o instanceof RegExp)if(o.global)e=o;else{let t="g";o.ignoreCase&&(t+="i"),o.multiline&&(t+="m"),o.unicode&&(t+="u"),e=new RegExp(o.source,t)}return e.lastIndex=0,e}const Mz=new cs;Mz.unshift({maxLen:1e3,windowSize:15,timeBudget:150});function Qv(o,e,t,i,n){if(e=z4(e),n||(n=ot.first(Mz)),t.length>n.maxLen){let c=o-n.maxLen/2;return c<0?c=0:i+=c,t=t.substring(c,o+n.maxLen/2),Qv(o,e,t,i,n)}const s=Date.now(),r=o-1-i;let a=-1,l=null;for(let c=1;!(Date.now()-s>=n.timeBudget);c++){const d=r-n.windowSize*c;e.lastIndex=Math.max(0,d);const h=UQ(e,t,r,a);if(!h&&l||(l=h,d<=0))break;a=d}if(l){const c={word:l[0],startColumn:i+1+l.index,endColumn:i+1+l.index+l[0].length};return e.lastIndex=0,c}return null}function UQ(o,e,t,i){let n;for(;n=o.exec(e);){const s=n.index||0;if(s<=t&&o.lastIndex>=t)return n;if(i>0&&s>i)return null}return null}const sl=8;class Az{constructor(e){this._values=e}hasChanged(e){return this._values[e]}}class Pz{constructor(){this.stableMinimapLayoutInput=null,this.stableFitMaxMinimapScale=0,this.stableFitRemainingWidth=0}}class Jt{constructor(e,t,i,n){this.id=e,this.name=t,this.defaultValue=i,this.schema=n}applyUpdate(e,t){return hI(e,t)}compute(e,t,i){return i}}class $C{constructor(e,t){this.newValue=e,this.didChange=t}}function hI(o,e){if(typeof o!="object"||typeof e!="object"||!o||!e)return new $C(e,o!==e);if(Array.isArray(o)||Array.isArray(e)){const i=Array.isArray(o)&&Array.isArray(e)&&ii(o,e);return new $C(e,!i)}let t=!1;for(const i in e)if(e.hasOwnProperty(i)){const n=hI(o[i],e[i]);n.didChange&&(o[i]=n.newValue,t=!0)}return new $C(o,t)}class T1{constructor(e){this.schema=void 0,this.id=e,this.name="_never_",this.defaultValue=void 0}applyUpdate(e,t){return hI(e,t)}validate(e){return this.defaultValue}}class hb{constructor(e,t,i,n){this.id=e,this.name=t,this.defaultValue=i,this.schema=n}applyUpdate(e,t){return hI(e,t)}validate(e){return typeof e>"u"?this.defaultValue:e}compute(e,t,i){return i}}function Ce(o,e){return typeof o>"u"?e:o==="false"?!1:!!o}class gt extends hb{constructor(e,t,i,n=void 0){typeof n<"u"&&(n.type="boolean",n.default=i),super(e,t,i,n)}validate(e){return Ce(e,this.defaultValue)}}function Iu(o,e,t,i){if(typeof o>"u")return e;let n=parseInt(o,10);return isNaN(n)?e:(n=Math.max(t,n),n=Math.min(i,n),n|0)}class At extends hb{static clampedInt(e,t,i,n){return Iu(e,t,i,n)}constructor(e,t,i,n,s,r=void 0){typeof r<"u"&&(r.type="integer",r.default=i,r.minimum=n,r.maximum=s),super(e,t,i,r),this.minimum=n,this.maximum=s}validate(e){return At.clampedInt(e,this.defaultValue,this.minimum,this.maximum)}}function $Q(o,e,t,i){if(typeof o>"u")return e;const n=Oo.float(o,e);return Oo.clamp(n,t,i)}class Oo extends hb{static clamp(e,t,i){return e<t?t:e>i?i:e}static float(e,t){if(typeof e=="number")return e;if(typeof e>"u")return t;const i=parseFloat(e);return isNaN(i)?t:i}constructor(e,t,i,n,s){typeof s<"u"&&(s.type="number",s.default=i),super(e,t,i,s),this.validationFn=n}validate(e){return this.validationFn(Oo.float(e,this.defaultValue))}}class Zn extends hb{static string(e,t){return typeof e!="string"?t:e}constructor(e,t,i,n=void 0){typeof n<"u"&&(n.type="string",n.default=i),super(e,t,i,n)}validate(e){return Zn.string(e,this.defaultValue)}}function gi(o,e,t,i){return typeof o!="string"?e:i&&o in i?i[o]:t.indexOf(o)===-1?e:o}class ui extends hb{constructor(e,t,i,n,s=void 0){typeof s<"u"&&(s.type="string",s.enum=n,s.default=i),super(e,t,i,s),this._allowedValues=n}validate(e){return gi(e,this.defaultValue,this._allowedValues)}}class kS extends Jt{constructor(e,t,i,n,s,r,a=void 0){typeof a<"u"&&(a.type="string",a.enum=s,a.default=n),super(e,t,i,a),this._allowedValues=s,this._convert=r}validate(e){return typeof e!="string"?this.defaultValue:this._allowedValues.indexOf(e)===-1?this.defaultValue:this._convert(e)}}function jQ(o){switch(o){case"none":return 0;case"keep":return 1;case"brackets":return 2;case"advanced":return 3;case"full":return 4}}class KQ extends Jt{constructor(){super(2,"accessibilitySupport",0,{type:"string",enum:["auto","on","off"],enumDescriptions:[m("accessibilitySupport.auto","Use platform APIs to detect when a Screen Reader is attached."),m("accessibilitySupport.on","Optimize for usage with a Screen Reader."),m("accessibilitySupport.off","Assume a screen reader is not attached.")],default:"auto",tags:["accessibility"],description:m("accessibilitySupport","Controls if the UI should run in a mode where it is optimized for screen readers.")})}validate(e){switch(e){case"auto":return 0;case"off":return 1;case"on":return 2}return this.defaultValue}compute(e,t,i){return i===0?e.accessibilitySupport:i}}class qQ extends Jt{constructor(){const e={insertSpace:!0,ignoreEmptyLines:!0};super(23,"comments",e,{"editor.comments.insertSpace":{type:"boolean",default:e.insertSpace,description:m("comments.insertSpace","Controls whether a space character is inserted when commenting.")},"editor.comments.ignoreEmptyLines":{type:"boolean",default:e.ignoreEmptyLines,description:m("comments.ignoreEmptyLines","Controls if empty lines should be ignored with toggle, add or remove actions for line comments.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{insertSpace:Ce(t.insertSpace,this.defaultValue.insertSpace),ignoreEmptyLines:Ce(t.ignoreEmptyLines,this.defaultValue.ignoreEmptyLines)}}}function GQ(o){switch(o){case"blink":return 1;case"smooth":return 2;case"phase":return 3;case"expand":return 4;case"solid":return 5}}var fn;(function(o){o[o.Line=1]="Line",o[o.Block=2]="Block",o[o.Underline=3]="Underline",o[o.LineThin=4]="LineThin",o[o.BlockOutline=5]="BlockOutline",o[o.UnderlineThin=6]="UnderlineThin"})(fn||(fn={}));function ZQ(o){switch(o){case"line":return fn.Line;case"block":return fn.Block;case"underline":return fn.Underline;case"line-thin":return fn.LineThin;case"block-outline":return fn.BlockOutline;case"underline-thin":return fn.UnderlineThin}}class YQ extends T1{constructor(){super(143)}compute(e,t,i){const n=["monaco-editor"];return t.get(39)&&n.push(t.get(39)),e.extraEditorClassName&&n.push(e.extraEditorClassName),t.get(74)==="default"?n.push("mouse-default"):t.get(74)==="copy"&&n.push("mouse-copy"),t.get(112)&&n.push("showUnused"),t.get(141)&&n.push("showDeprecated"),n.join(" ")}}class XQ extends gt{constructor(){super(37,"emptySelectionClipboard",!0,{description:m("emptySelectionClipboard","Controls whether copying without a selection copies the current line.")})}compute(e,t,i){return i&&e.emptySelectionClipboard}}class QQ extends Jt{constructor(){const e={cursorMoveOnType:!0,seedSearchStringFromSelection:"always",autoFindInSelection:"never",globalFindClipboard:!1,addExtraSpaceOnTop:!0,loop:!0};super(41,"find",e,{"editor.find.cursorMoveOnType":{type:"boolean",default:e.cursorMoveOnType,description:m("find.cursorMoveOnType","Controls whether the cursor should jump to find matches while typing.")},"editor.find.seedSearchStringFromSelection":{type:"string",enum:["never","always","selection"],default:e.seedSearchStringFromSelection,enumDescriptions:[m("editor.find.seedSearchStringFromSelection.never","Never seed search string from the editor selection."),m("editor.find.seedSearchStringFromSelection.always","Always seed search string from the editor selection, including word at cursor position."),m("editor.find.seedSearchStringFromSelection.selection","Only seed search string from the editor selection.")],description:m("find.seedSearchStringFromSelection","Controls whether the search string in the Find Widget is seeded from the editor selection.")},"editor.find.autoFindInSelection":{type:"string",enum:["never","always","multiline"],default:e.autoFindInSelection,enumDescriptions:[m("editor.find.autoFindInSelection.never","Never turn on Find in Selection automatically (default)."),m("editor.find.autoFindInSelection.always","Always turn on Find in Selection automatically."),m("editor.find.autoFindInSelection.multiline","Turn on Find in Selection automatically when multiple lines of content are selected.")],description:m("find.autoFindInSelection","Controls the condition for turning on Find in Selection automatically.")},"editor.find.globalFindClipboard":{type:"boolean",default:e.globalFindClipboard,description:m("find.globalFindClipboard","Controls whether the Find Widget should read or modify the shared find clipboard on macOS."),included:Je},"editor.find.addExtraSpaceOnTop":{type:"boolean",default:e.addExtraSpaceOnTop,description:m("find.addExtraSpaceOnTop","Controls whether the Find Widget should add extra lines on top of the editor. When true, you can scroll beyond the first line when the Find Widget is visible.")},"editor.find.loop":{type:"boolean",default:e.loop,description:m("find.loop","Controls whether the search automatically restarts from the beginning (or the end) when no further matches can be found.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{cursorMoveOnType:Ce(t.cursorMoveOnType,this.defaultValue.cursorMoveOnType),seedSearchStringFromSelection:typeof e.seedSearchStringFromSelection=="boolean"?e.seedSearchStringFromSelection?"always":"never":gi(t.seedSearchStringFromSelection,this.defaultValue.seedSearchStringFromSelection,["never","always","selection"]),autoFindInSelection:typeof e.autoFindInSelection=="boolean"?e.autoFindInSelection?"always":"never":gi(t.autoFindInSelection,this.defaultValue.autoFindInSelection,["never","always","multiline"]),globalFindClipboard:Ce(t.globalFindClipboard,this.defaultValue.globalFindClipboard),addExtraSpaceOnTop:Ce(t.addExtraSpaceOnTop,this.defaultValue.addExtraSpaceOnTop),loop:Ce(t.loop,this.defaultValue.loop)}}}const mc=class mc extends Jt{constructor(){super(51,"fontLigatures",mc.OFF,{anyOf:[{type:"boolean",description:m("fontLigatures","Enables/Disables font ligatures ('calt' and 'liga' font features). Change this to a string for fine-grained control of the 'font-feature-settings' CSS property.")},{type:"string",description:m("fontFeatureSettings","Explicit 'font-feature-settings' CSS property. A boolean can be passed instead if one only needs to turn on/off ligatures.")}],description:m("fontLigaturesGeneral","Configures font ligatures or font features. Can be either a boolean to enable/disable ligatures or a string for the value of the CSS 'font-feature-settings' property."),default:!1})}validate(e){return typeof e>"u"?this.defaultValue:typeof e=="string"?e==="false"||e.length===0?mc.OFF:e==="true"?mc.ON:e:e?mc.ON:mc.OFF}};mc.OFF='"liga" off, "calt" off',mc.ON='"liga" on, "calt" on';let Mh=mc;const pc=class pc extends Jt{constructor(){super(54,"fontVariations",pc.OFF,{anyOf:[{type:"boolean",description:m("fontVariations","Enables/Disables the translation from font-weight to font-variation-settings. Change this to a string for fine-grained control of the 'font-variation-settings' CSS property.")},{type:"string",description:m("fontVariationSettings","Explicit 'font-variation-settings' CSS property. A boolean can be passed instead if one only needs to translate font-weight to font-variation-settings.")}],description:m("fontVariationsGeneral","Configures font variations. Can be either a boolean to enable/disable the translation from font-weight to font-variation-settings or a string for the value of the CSS 'font-variation-settings' property."),default:!1})}validate(e){return typeof e>"u"?this.defaultValue:typeof e=="string"?e==="false"?pc.OFF:e==="true"?pc.TRANSLATE:e:e?pc.TRANSLATE:pc.OFF}compute(e,t,i){return e.fontInfo.fontVariationSettings}};pc.OFF="normal",pc.TRANSLATE="translate";let Jv=pc;class JQ extends T1{constructor(){super(50)}compute(e,t,i){return e.fontInfo}}class eJ extends hb{constructor(){super(52,"fontSize",fs.fontSize,{type:"number",minimum:6,maximum:100,default:fs.fontSize,description:m("fontSize","Controls the font size in pixels.")})}validate(e){const t=Oo.float(e,this.defaultValue);return t===0?fs.fontSize:Oo.clamp(t,6,100)}compute(e,t,i){return e.fontInfo.fontSize}}const cl=class cl extends Jt{constructor(){super(53,"fontWeight",fs.fontWeight,{anyOf:[{type:"number",minimum:cl.MINIMUM_VALUE,maximum:cl.MAXIMUM_VALUE,errorMessage:m("fontWeightErrorMessage",'Only "normal" and "bold" keywords or numbers between 1 and 1000 are allowed.')},{type:"string",pattern:"^(normal|bold|1000|[1-9][0-9]{0,2})$"},{enum:cl.SUGGESTION_VALUES}],default:fs.fontWeight,description:m("fontWeight",'Controls the font weight. Accepts "normal" and "bold" keywords or numbers between 1 and 1000.')})}validate(e){return e==="normal"||e==="bold"?e:String(At.clampedInt(e,fs.fontWeight,cl.MINIMUM_VALUE,cl.MAXIMUM_VALUE))}};cl.SUGGESTION_VALUES=["normal","bold","100","200","300","400","500","600","700","800","900"],cl.MINIMUM_VALUE=1,cl.MAXIMUM_VALUE=1e3;let fR=cl;class tJ extends Jt{constructor(){const e={multiple:"peek",multipleDefinitions:"peek",multipleTypeDefinitions:"peek",multipleDeclarations:"peek",multipleImplementations:"peek",multipleReferences:"peek",multipleTests:"peek",alternativeDefinitionCommand:"editor.action.goToReferences",alternativeTypeDefinitionCommand:"editor.action.goToReferences",alternativeDeclarationCommand:"editor.action.goToReferences",alternativeImplementationCommand:"",alternativeReferenceCommand:"",alternativeTestsCommand:""},t={type:"string",enum:["peek","gotoAndPeek","goto"],default:e.multiple,enumDescriptions:[m("editor.gotoLocation.multiple.peek","Show Peek view of the results (default)"),m("editor.gotoLocation.multiple.gotoAndPeek","Go to the primary result and show a Peek view"),m("editor.gotoLocation.multiple.goto","Go to the primary result and enable Peek-less navigation to others")]},i=["","editor.action.referenceSearch.trigger","editor.action.goToReferences","editor.action.peekImplementation","editor.action.goToImplementation","editor.action.peekTypeDefinition","editor.action.goToTypeDefinition","editor.action.peekDeclaration","editor.action.revealDeclaration","editor.action.peekDefinition","editor.action.revealDefinitionAside","editor.action.revealDefinition"];super(58,"gotoLocation",e,{"editor.gotoLocation.multiple":{deprecationMessage:m("editor.gotoLocation.multiple.deprecated","This setting is deprecated, please use separate settings like 'editor.editor.gotoLocation.multipleDefinitions' or 'editor.editor.gotoLocation.multipleImplementations' instead.")},"editor.gotoLocation.multipleDefinitions":{description:m("editor.editor.gotoLocation.multipleDefinitions","Controls the behavior the 'Go to Definition'-command when multiple target locations exist."),...t},"editor.gotoLocation.multipleTypeDefinitions":{description:m("editor.editor.gotoLocation.multipleTypeDefinitions","Controls the behavior the 'Go to Type Definition'-command when multiple target locations exist."),...t},"editor.gotoLocation.multipleDeclarations":{description:m("editor.editor.gotoLocation.multipleDeclarations","Controls the behavior the 'Go to Declaration'-command when multiple target locations exist."),...t},"editor.gotoLocation.multipleImplementations":{description:m("editor.editor.gotoLocation.multipleImplemenattions","Controls the behavior the 'Go to Implementations'-command when multiple target locations exist."),...t},"editor.gotoLocation.multipleReferences":{description:m("editor.editor.gotoLocation.multipleReferences","Controls the behavior the 'Go to References'-command when multiple target locations exist."),...t},"editor.gotoLocation.alternativeDefinitionCommand":{type:"string",default:e.alternativeDefinitionCommand,enum:i,description:m("alternativeDefinitionCommand","Alternative command id that is being executed when the result of 'Go to Definition' is the current location.")},"editor.gotoLocation.alternativeTypeDefinitionCommand":{type:"string",default:e.alternativeTypeDefinitionCommand,enum:i,description:m("alternativeTypeDefinitionCommand","Alternative command id that is being executed when the result of 'Go to Type Definition' is the current location.")},"editor.gotoLocation.alternativeDeclarationCommand":{type:"string",default:e.alternativeDeclarationCommand,enum:i,description:m("alternativeDeclarationCommand","Alternative command id that is being executed when the result of 'Go to Declaration' is the current location.")},"editor.gotoLocation.alternativeImplementationCommand":{type:"string",default:e.alternativeImplementationCommand,enum:i,description:m("alternativeImplementationCommand","Alternative command id that is being executed when the result of 'Go to Implementation' is the current location.")},"editor.gotoLocation.alternativeReferenceCommand":{type:"string",default:e.alternativeReferenceCommand,enum:i,description:m("alternativeReferenceCommand","Alternative command id that is being executed when the result of 'Go to Reference' is the current location.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{multiple:gi(t.multiple,this.defaultValue.multiple,["peek","gotoAndPeek","goto"]),multipleDefinitions:t.multipleDefinitions??gi(t.multipleDefinitions,"peek",["peek","gotoAndPeek","goto"]),multipleTypeDefinitions:t.multipleTypeDefinitions??gi(t.multipleTypeDefinitions,"peek",["peek","gotoAndPeek","goto"]),multipleDeclarations:t.multipleDeclarations??gi(t.multipleDeclarations,"peek",["peek","gotoAndPeek","goto"]),multipleImplementations:t.multipleImplementations??gi(t.multipleImplementations,"peek",["peek","gotoAndPeek","goto"]),multipleReferences:t.multipleReferences??gi(t.multipleReferences,"peek",["peek","gotoAndPeek","goto"]),multipleTests:t.multipleTests??gi(t.multipleTests,"peek",["peek","gotoAndPeek","goto"]),alternativeDefinitionCommand:Zn.string(t.alternativeDefinitionCommand,this.defaultValue.alternativeDefinitionCommand),alternativeTypeDefinitionCommand:Zn.string(t.alternativeTypeDefinitionCommand,this.defaultValue.alternativeTypeDefinitionCommand),alternativeDeclarationCommand:Zn.string(t.alternativeDeclarationCommand,this.defaultValue.alternativeDeclarationCommand),alternativeImplementationCommand:Zn.string(t.alternativeImplementationCommand,this.defaultValue.alternativeImplementationCommand),alternativeReferenceCommand:Zn.string(t.alternativeReferenceCommand,this.defaultValue.alternativeReferenceCommand),alternativeTestsCommand:Zn.string(t.alternativeTestsCommand,this.defaultValue.alternativeTestsCommand)}}}class iJ extends Jt{constructor(){const e={enabled:!0,delay:300,hidingDelay:300,sticky:!0,above:!0};super(60,"hover",e,{"editor.hover.enabled":{type:"boolean",default:e.enabled,description:m("hover.enabled","Controls whether the hover is shown.")},"editor.hover.delay":{type:"number",default:e.delay,minimum:0,maximum:1e4,description:m("hover.delay","Controls the delay in milliseconds after which the hover is shown.")},"editor.hover.sticky":{type:"boolean",default:e.sticky,description:m("hover.sticky","Controls whether the hover should remain visible when mouse is moved over it.")},"editor.hover.hidingDelay":{type:"integer",minimum:0,default:e.hidingDelay,description:m("hover.hidingDelay","Controls the delay in milliseconds after which the hover is hidden. Requires `editor.hover.sticky` to be enabled.")},"editor.hover.above":{type:"boolean",default:e.above,description:m("hover.above","Prefer showing hovers above the line, if there's space.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{enabled:Ce(t.enabled,this.defaultValue.enabled),delay:At.clampedInt(t.delay,this.defaultValue.delay,0,1e4),sticky:Ce(t.sticky,this.defaultValue.sticky),hidingDelay:At.clampedInt(t.hidingDelay,this.defaultValue.hidingDelay,0,6e5),above:Ce(t.above,this.defaultValue.above)}}}class Hp extends T1{constructor(){super(146)}compute(e,t,i){return Hp.computeLayout(t,{memory:e.memory,outerWidth:e.outerWidth,outerHeight:e.outerHeight,isDominatedByLongLines:e.isDominatedByLongLines,lineHeight:e.fontInfo.lineHeight,viewLineCount:e.viewLineCount,lineNumbersDigitCount:e.lineNumbersDigitCount,typicalHalfwidthCharacterWidth:e.fontInfo.typicalHalfwidthCharacterWidth,maxDigitWidth:e.fontInfo.maxDigitWidth,pixelRatio:e.pixelRatio,glyphMarginDecorationLaneCount:e.glyphMarginDecorationLaneCount})}static computeContainedMinimapLineCount(e){const t=e.height/e.lineHeight,i=Math.floor(e.paddingTop/e.lineHeight);let n=Math.floor(e.paddingBottom/e.lineHeight);e.scrollBeyondLastLine&&(n=Math.max(n,t-1));const s=(i+e.viewLineCount+n)/(e.pixelRatio*e.height),r=Math.floor(e.viewLineCount/s);return{typicalViewportLineCount:t,extraLinesBeforeFirstLine:i,extraLinesBeyondLastLine:n,desiredRatio:s,minimapLineCount:r}}static _computeMinimapLayout(e,t){const i=e.outerWidth,n=e.outerHeight,s=e.pixelRatio;if(!e.minimap.enabled)return{renderMinimap:0,minimapLeft:0,minimapWidth:0,minimapHeightIsEditorHeight:!1,minimapIsSampling:!1,minimapScale:1,minimapLineHeight:1,minimapCanvasInnerWidth:0,minimapCanvasInnerHeight:Math.floor(s*n),minimapCanvasOuterWidth:0,minimapCanvasOuterHeight:n};const r=t.stableMinimapLayoutInput,a=r&&e.outerHeight===r.outerHeight&&e.lineHeight===r.lineHeight&&e.typicalHalfwidthCharacterWidth===r.typicalHalfwidthCharacterWidth&&e.pixelRatio===r.pixelRatio&&e.scrollBeyondLastLine===r.scrollBeyondLastLine&&e.paddingTop===r.paddingTop&&e.paddingBottom===r.paddingBottom&&e.minimap.enabled===r.minimap.enabled&&e.minimap.side===r.minimap.side&&e.minimap.size===r.minimap.size&&e.minimap.showSlider===r.minimap.showSlider&&e.minimap.renderCharacters===r.minimap.renderCharacters&&e.minimap.maxColumn===r.minimap.maxColumn&&e.minimap.scale===r.minimap.scale&&e.verticalScrollbarWidth===r.verticalScrollbarWidth&&e.isViewportWrapping===r.isViewportWrapping,l=e.lineHeight,c=e.typicalHalfwidthCharacterWidth,d=e.scrollBeyondLastLine,h=e.minimap.renderCharacters;let u=s>=2?Math.round(e.minimap.scale*2):e.minimap.scale;const g=e.minimap.maxColumn,f=e.minimap.size,p=e.minimap.side,_=e.verticalScrollbarWidth,b=e.viewLineCount,C=e.remainingWidth,v=e.isViewportWrapping,w=h?2:3;let S=Math.floor(s*n);const L=S/s;let k=!1,D=!1,N=w*u,B=u/s,W=1;if(f==="fill"||f==="fit"){const{typicalViewportLineCount:fe,extraLinesBeforeFirstLine:ue,extraLinesBeyondLastLine:Pe,desiredRatio:_e,minimapLineCount:it}=Hp.computeContainedMinimapLineCount({viewLineCount:b,scrollBeyondLastLine:d,paddingTop:e.paddingTop,paddingBottom:e.paddingBottom,height:n,lineHeight:l,pixelRatio:s});if(b/it>1)k=!0,D=!0,u=1,N=1,B=u/s;else{let Ct=!1,ht=u+1;if(f==="fit"){const Ti=Math.ceil((ue+b+Pe)*N);v&&a&&C<=t.stableFitRemainingWidth?(Ct=!0,ht=t.stableFitMaxMinimapScale):Ct=Ti>S}if(f==="fill"||Ct){k=!0;const Ti=u;N=Math.min(l*s,Math.max(1,Math.floor(1/_e))),v&&a&&C<=t.stableFitRemainingWidth&&(ht=t.stableFitMaxMinimapScale),u=Math.min(ht,Math.max(1,Math.floor(N/w))),u>Ti&&(W=Math.min(2,u/Ti)),B=u/s/W,S=Math.ceil(Math.max(fe,ue+b+Pe)*N),v?(t.stableMinimapLayoutInput=e,t.stableFitRemainingWidth=C,t.stableFitMaxMinimapScale=u):(t.stableMinimapLayoutInput=null,t.stableFitRemainingWidth=0)}}}const V=Math.floor(g*B),$=Math.min(V,Math.max(0,Math.floor((C-_-2)*B/(c+B)))+sl);let H=Math.floor(s*$);const j=H/s;H=Math.floor(H*W);const J=h?1:2,he=p==="left"?0:i-$-_;return{renderMinimap:J,minimapLeft:he,minimapWidth:$,minimapHeightIsEditorHeight:k,minimapIsSampling:D,minimapScale:u,minimapLineHeight:N,minimapCanvasInnerWidth:H,minimapCanvasInnerHeight:S,minimapCanvasOuterWidth:j,minimapCanvasOuterHeight:L}}static computeLayout(e,t){const i=t.outerWidth|0,n=t.outerHeight|0,s=t.lineHeight|0,r=t.lineNumbersDigitCount|0,a=t.typicalHalfwidthCharacterWidth,l=t.maxDigitWidth,c=t.pixelRatio,d=t.viewLineCount,h=e.get(138),u=h==="inherit"?e.get(137):h,g=u==="inherit"?e.get(133):u,f=e.get(136),p=t.isDominatedByLongLines,_=e.get(57),b=e.get(68).renderType!==0,C=e.get(69),v=e.get(106),w=e.get(84),S=e.get(73),L=e.get(104),k=L.verticalScrollbarSize,D=L.verticalHasArrows,N=L.arrowSize,B=L.horizontalScrollbarSize,W=e.get(43),V=e.get(111)!=="never";let $=e.get(66);W&&V&&($+=16);let H=0;if(b){const xs=Math.max(r,C);H=Math.round(xs*l)}let j=0;_&&(j=s*t.glyphMarginDecorationLaneCount);let J=0,he=J+j,fe=he+H,ue=fe+$;const Pe=i-j-H-$;let _e=!1,it=!1,Ne=-1;u==="inherit"&&p?(_e=!0,it=!0):g==="on"||g==="bounded"?it=!0:g==="wordWrapColumn"&&(Ne=f);const Ct=Hp._computeMinimapLayout({outerWidth:i,outerHeight:n,lineHeight:s,typicalHalfwidthCharacterWidth:a,pixelRatio:c,scrollBeyondLastLine:v,paddingTop:w.top,paddingBottom:w.bottom,minimap:S,verticalScrollbarWidth:k,viewLineCount:d,remainingWidth:Pe,isViewportWrapping:it},t.memory||new Pz);Ct.renderMinimap!==0&&Ct.minimapLeft===0&&(J+=Ct.minimapWidth,he+=Ct.minimapWidth,fe+=Ct.minimapWidth,ue+=Ct.minimapWidth);const ht=Pe-Ct.minimapWidth,Ti=Math.max(1,Math.floor((ht-k-2)/a)),ts=D?N:0;return it&&(Ne=Math.max(1,Ti),g==="bounded"&&(Ne=Math.min(Ne,f))),{width:i,height:n,glyphMarginLeft:J,glyphMarginWidth:j,glyphMarginDecorationLaneCount:t.glyphMarginDecorationLaneCount,lineNumbersLeft:he,lineNumbersWidth:H,decorationsLeft:fe,decorationsWidth:$,contentLeft:ue,contentWidth:ht,minimap:Ct,viewportColumn:Ti,isWordWrapMinified:_e,isViewportWrapping:it,wrappingColumn:Ne,verticalScrollbarWidth:k,horizontalScrollbarHeight:B,overviewRuler:{top:ts,width:k,height:n-2*ts,right:0}}}}class nJ extends Jt{constructor(){super(140,"wrappingStrategy","simple",{"editor.wrappingStrategy":{enumDescriptions:[m("wrappingStrategy.simple","Assumes that all characters are of the same width. This is a fast algorithm that works correctly for monospace fonts and certain scripts (like Latin characters) where glyphs are of equal width."),m("wrappingStrategy.advanced","Delegates wrapping points computation to the browser. This is a slow algorithm, that might cause freezes for large files, but it works correctly in all cases.")],type:"string",enum:["simple","advanced"],default:"simple",description:m("wrappingStrategy","Controls the algorithm that computes wrapping points. Note that when in accessibility mode, advanced will be used for the best experience.")}})}validate(e){return gi(e,"simple",["simple","advanced"])}compute(e,t,i){return t.get(2)===2?"advanced":i}}var Or;(function(o){o.Off="off",o.OnCode="onCode",o.On="on"})(Or||(Or={}));class sJ extends Jt{constructor(){const e={enabled:Or.OnCode};super(65,"lightbulb",e,{"editor.lightbulb.enabled":{type:"string",tags:["experimental"],enum:[Or.Off,Or.OnCode,Or.On],default:e.enabled,enumDescriptions:[m("editor.lightbulb.enabled.off","Disable the code action menu."),m("editor.lightbulb.enabled.onCode","Show the code action menu when the cursor is on lines with code."),m("editor.lightbulb.enabled.on","Show the code action menu when the cursor is on lines with code or on empty lines.")],description:m("enabled","Enables the Code Action lightbulb in the editor.")}})}validate(e){return!e||typeof e!="object"?this.defaultValue:{enabled:gi(e.enabled,this.defaultValue.enabled,[Or.Off,Or.OnCode,Or.On])}}}class oJ extends Jt{constructor(){const e={enabled:!0,maxLineCount:5,defaultModel:"outlineModel",scrollWithEditor:!0};super(116,"stickyScroll",e,{"editor.stickyScroll.enabled":{type:"boolean",default:e.enabled,description:m("editor.stickyScroll.enabled","Shows the nested current scopes during the scroll at the top of the editor."),tags:["experimental"]},"editor.stickyScroll.maxLineCount":{type:"number",default:e.maxLineCount,minimum:1,maximum:20,description:m("editor.stickyScroll.maxLineCount","Defines the maximum number of sticky lines to show.")},"editor.stickyScroll.defaultModel":{type:"string",enum:["outlineModel","foldingProviderModel","indentationModel"],default:e.defaultModel,description:m("editor.stickyScroll.defaultModel","Defines the model to use for determining which lines to stick. If the outline model does not exist, it will fall back on the folding provider model which falls back on the indentation model. This order is respected in all three cases.")},"editor.stickyScroll.scrollWithEditor":{type:"boolean",default:e.scrollWithEditor,description:m("editor.stickyScroll.scrollWithEditor","Enable scrolling of Sticky Scroll with the editor's horizontal scrollbar.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{enabled:Ce(t.enabled,this.defaultValue.enabled),maxLineCount:At.clampedInt(t.maxLineCount,this.defaultValue.maxLineCount,1,20),defaultModel:gi(t.defaultModel,this.defaultValue.defaultModel,["outlineModel","foldingProviderModel","indentationModel"]),scrollWithEditor:Ce(t.scrollWithEditor,this.defaultValue.scrollWithEditor)}}}class rJ extends Jt{constructor(){const e={enabled:"on",fontSize:0,fontFamily:"",padding:!1};super(142,"inlayHints",e,{"editor.inlayHints.enabled":{type:"string",default:e.enabled,description:m("inlayHints.enable","Enables the inlay hints in the editor."),enum:["on","onUnlessPressed","offUnlessPressed","off"],markdownEnumDescriptions:[m("editor.inlayHints.on","Inlay hints are enabled"),m("editor.inlayHints.onUnlessPressed","Inlay hints are showing by default and hide when holding {0}",Je?"Ctrl+Option":"Ctrl+Alt"),m("editor.inlayHints.offUnlessPressed","Inlay hints are hidden by default and show when holding {0}",Je?"Ctrl+Option":"Ctrl+Alt"),m("editor.inlayHints.off","Inlay hints are disabled")]},"editor.inlayHints.fontSize":{type:"number",default:e.fontSize,markdownDescription:m("inlayHints.fontSize","Controls font size of inlay hints in the editor. As default the {0} is used when the configured value is less than {1} or greater than the editor font size.","`#editor.fontSize#`","`5`")},"editor.inlayHints.fontFamily":{type:"string",default:e.fontFamily,markdownDescription:m("inlayHints.fontFamily","Controls font family of inlay hints in the editor. When set to empty, the {0} is used.","`#editor.fontFamily#`")},"editor.inlayHints.padding":{type:"boolean",default:e.padding,description:m("inlayHints.padding","Enables the padding around the inlay hints in the editor.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return typeof t.enabled=="boolean"&&(t.enabled=t.enabled?"on":"off"),{enabled:gi(t.enabled,this.defaultValue.enabled,["on","off","offUnlessPressed","onUnlessPressed"]),fontSize:At.clampedInt(t.fontSize,this.defaultValue.fontSize,0,100),fontFamily:Zn.string(t.fontFamily,this.defaultValue.fontFamily),padding:Ce(t.padding,this.defaultValue.padding)}}}class aJ extends Jt{constructor(){super(66,"lineDecorationsWidth",10)}validate(e){return typeof e=="string"&&/^\d+(\.\d+)?ch$/.test(e)?-parseFloat(e.substring(0,e.length-2)):At.clampedInt(e,this.defaultValue,0,1e3)}compute(e,t,i){return i<0?At.clampedInt(-i*e.fontInfo.typicalHalfwidthCharacterWidth,this.defaultValue,0,1e3):i}}class lJ extends Oo{constructor(){super(67,"lineHeight",fs.lineHeight,e=>Oo.clamp(e,0,150),{markdownDescription:m("lineHeight",`Controls the line height. 
 - Use 0 to automatically compute the line height from the font size.
 - Values between 0 and 8 will be used as a multiplier with the font size.
 - Values greater than or equal to 8 will be used as effective values.`)})}compute(e,t,i){return e.fontInfo.lineHeight}}class cJ extends Jt{constructor(){const e={enabled:!0,size:"proportional",side:"right",showSlider:"mouseover",autohide:!1,renderCharacters:!0,maxColumn:120,scale:1,showRegionSectionHeaders:!0,showMarkSectionHeaders:!0,sectionHeaderFontSize:9,sectionHeaderLetterSpacing:1};super(73,"minimap",e,{"editor.minimap.enabled":{type:"boolean",default:e.enabled,description:m("minimap.enabled","Controls whether the minimap is shown.")},"editor.minimap.autohide":{type:"boolean",default:e.autohide,description:m("minimap.autohide","Controls whether the minimap is hidden automatically.")},"editor.minimap.size":{type:"string",enum:["proportional","fill","fit"],enumDescriptions:[m("minimap.size.proportional","The minimap has the same size as the editor contents (and might scroll)."),m("minimap.size.fill","The minimap will stretch or shrink as necessary to fill the height of the editor (no scrolling)."),m("minimap.size.fit","The minimap will shrink as necessary to never be larger than the editor (no scrolling).")],default:e.size,description:m("minimap.size","Controls the size of the minimap.")},"editor.minimap.side":{type:"string",enum:["left","right"],default:e.side,description:m("minimap.side","Controls the side where to render the minimap.")},"editor.minimap.showSlider":{type:"string",enum:["always","mouseover"],default:e.showSlider,description:m("minimap.showSlider","Controls when the minimap slider is shown.")},"editor.minimap.scale":{type:"number",default:e.scale,minimum:1,maximum:3,enum:[1,2,3],description:m("minimap.scale","Scale of content drawn in the minimap: 1, 2 or 3.")},"editor.minimap.renderCharacters":{type:"boolean",default:e.renderCharacters,description:m("minimap.renderCharacters","Render the actual characters on a line as opposed to color blocks.")},"editor.minimap.maxColumn":{type:"number",default:e.maxColumn,description:m("minimap.maxColumn","Limit the width of the minimap to render at most a certain number of columns.")},"editor.minimap.showRegionSectionHeaders":{type:"boolean",default:e.showRegionSectionHeaders,description:m("minimap.showRegionSectionHeaders","Controls whether named regions are shown as section headers in the minimap.")},"editor.minimap.showMarkSectionHeaders":{type:"boolean",default:e.showMarkSectionHeaders,description:m("minimap.showMarkSectionHeaders","Controls whether MARK: comments are shown as section headers in the minimap.")},"editor.minimap.sectionHeaderFontSize":{type:"number",default:e.sectionHeaderFontSize,description:m("minimap.sectionHeaderFontSize","Controls the font size of section headers in the minimap.")},"editor.minimap.sectionHeaderLetterSpacing":{type:"number",default:e.sectionHeaderLetterSpacing,description:m("minimap.sectionHeaderLetterSpacing","Controls the amount of space (in pixels) between characters of section header. This helps the readability of the header in small font sizes.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{enabled:Ce(t.enabled,this.defaultValue.enabled),autohide:Ce(t.autohide,this.defaultValue.autohide),size:gi(t.size,this.defaultValue.size,["proportional","fill","fit"]),side:gi(t.side,this.defaultValue.side,["right","left"]),showSlider:gi(t.showSlider,this.defaultValue.showSlider,["always","mouseover"]),renderCharacters:Ce(t.renderCharacters,this.defaultValue.renderCharacters),scale:At.clampedInt(t.scale,1,1,3),maxColumn:At.clampedInt(t.maxColumn,this.defaultValue.maxColumn,1,1e4),showRegionSectionHeaders:Ce(t.showRegionSectionHeaders,this.defaultValue.showRegionSectionHeaders),showMarkSectionHeaders:Ce(t.showMarkSectionHeaders,this.defaultValue.showMarkSectionHeaders),sectionHeaderFontSize:Oo.clamp(t.sectionHeaderFontSize??this.defaultValue.sectionHeaderFontSize,4,32),sectionHeaderLetterSpacing:Oo.clamp(t.sectionHeaderLetterSpacing??this.defaultValue.sectionHeaderLetterSpacing,0,5)}}}function dJ(o){return o==="ctrlCmd"?Je?"metaKey":"ctrlKey":"altKey"}class hJ extends Jt{constructor(){super(84,"padding",{top:0,bottom:0},{"editor.padding.top":{type:"number",default:0,minimum:0,maximum:1e3,description:m("padding.top","Controls the amount of space between the top edge of the editor and the first line.")},"editor.padding.bottom":{type:"number",default:0,minimum:0,maximum:1e3,description:m("padding.bottom","Controls the amount of space between the bottom edge of the editor and the last line.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{top:At.clampedInt(t.top,0,0,1e3),bottom:At.clampedInt(t.bottom,0,0,1e3)}}}class uJ extends Jt{constructor(){const e={enabled:!0,cycle:!0};super(86,"parameterHints",e,{"editor.parameterHints.enabled":{type:"boolean",default:e.enabled,description:m("parameterHints.enabled","Enables a pop-up that shows parameter documentation and type information as you type.")},"editor.parameterHints.cycle":{type:"boolean",default:e.cycle,description:m("parameterHints.cycle","Controls whether the parameter hints menu cycles or closes when reaching the end of the list.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{enabled:Ce(t.enabled,this.defaultValue.enabled),cycle:Ce(t.cycle,this.defaultValue.cycle)}}}class gJ extends T1{constructor(){super(144)}compute(e,t,i){return e.pixelRatio}}class fJ extends Jt{constructor(){super(88,"placeholder",void 0)}validate(e){return typeof e>"u"?this.defaultValue:typeof e=="string"?e:this.defaultValue}}class mJ extends Jt{constructor(){const e={other:"on",comments:"off",strings:"off"},t=[{type:"boolean"},{type:"string",enum:["on","inline","off"],enumDescriptions:[m("on","Quick suggestions show inside the suggest widget"),m("inline","Quick suggestions show as ghost text"),m("off","Quick suggestions are disabled")]}];super(90,"quickSuggestions",e,{type:"object",additionalProperties:!1,properties:{strings:{anyOf:t,default:e.strings,description:m("quickSuggestions.strings","Enable quick suggestions inside strings.")},comments:{anyOf:t,default:e.comments,description:m("quickSuggestions.comments","Enable quick suggestions inside comments.")},other:{anyOf:t,default:e.other,description:m("quickSuggestions.other","Enable quick suggestions outside of strings and comments.")}},default:e,markdownDescription:m("quickSuggestions","Controls whether suggestions should automatically show up while typing. This can be controlled for typing in comments, strings, and other code. Quick suggestion can be configured to show as ghost text or with the suggest widget. Also be aware of the {0}-setting which controls if suggestions are triggered by special characters.","`#editor.suggestOnTriggerCharacters#`")}),this.defaultValue=e}validate(e){if(typeof e=="boolean"){const c=e?"on":"off";return{comments:c,strings:c,other:c}}if(!e||typeof e!="object")return this.defaultValue;const{other:t,comments:i,strings:n}=e,s=["on","inline","off"];let r,a,l;return typeof t=="boolean"?r=t?"on":"off":r=gi(t,this.defaultValue.other,s),typeof i=="boolean"?a=i?"on":"off":a=gi(i,this.defaultValue.comments,s),typeof n=="boolean"?l=n?"on":"off":l=gi(n,this.defaultValue.strings,s),{other:r,comments:a,strings:l}}}class pJ extends Jt{constructor(){super(68,"lineNumbers",{renderType:1,renderFn:null},{type:"string",enum:["off","on","relative","interval"],enumDescriptions:[m("lineNumbers.off","Line numbers are not rendered."),m("lineNumbers.on","Line numbers are rendered as absolute number."),m("lineNumbers.relative","Line numbers are rendered as distance in lines to cursor position."),m("lineNumbers.interval","Line numbers are rendered every 10 lines.")],default:"on",description:m("lineNumbers","Controls the display of line numbers.")})}validate(e){let t=this.defaultValue.renderType,i=this.defaultValue.renderFn;return typeof e<"u"&&(typeof e=="function"?(t=4,i=e):e==="interval"?t=3:e==="relative"?t=2:e==="on"?t=1:t=0),{renderType:t,renderFn:i}}}function Ty(o){const e=o.get(99);return e==="editable"?o.get(92):e!=="on"}class _J extends Jt{constructor(){const e=[],t={type:"number",description:m("rulers.size","Number of monospace characters at which this editor ruler will render.")};super(103,"rulers",e,{type:"array",items:{anyOf:[t,{type:["object"],properties:{column:t,color:{type:"string",description:m("rulers.color","Color of this editor ruler."),format:"color-hex"}}}]},default:e,description:m("rulers","Render vertical rulers after a certain number of monospace characters. Use multiple values for multiple rulers. No rulers are drawn if array is empty.")})}validate(e){if(Array.isArray(e)){const t=[];for(const i of e)if(typeof i=="number")t.push({column:At.clampedInt(i,0,0,1e4),color:null});else if(i&&typeof i=="object"){const n=i;t.push({column:At.clampedInt(n.column,0,0,1e4),color:n.color})}return t.sort((i,n)=>i.column-n.column),t}return this.defaultValue}}class bJ extends Jt{constructor(){super(93,"readOnlyMessage",void 0)}validate(e){return!e||typeof e!="object"?this.defaultValue:e}}function $B(o,e){if(typeof o!="string")return e;switch(o){case"hidden":return 2;case"visible":return 3;default:return 1}}let CJ=class extends Jt{constructor(){const e={vertical:1,horizontal:1,arrowSize:11,useShadows:!0,verticalHasArrows:!1,horizontalHasArrows:!1,horizontalScrollbarSize:12,horizontalSliderSize:12,verticalScrollbarSize:14,verticalSliderSize:14,handleMouseWheel:!0,alwaysConsumeMouseWheel:!0,scrollByPage:!1,ignoreHorizontalScrollbarInContentHeight:!1};super(104,"scrollbar",e,{"editor.scrollbar.vertical":{type:"string",enum:["auto","visible","hidden"],enumDescriptions:[m("scrollbar.vertical.auto","The vertical scrollbar will be visible only when necessary."),m("scrollbar.vertical.visible","The vertical scrollbar will always be visible."),m("scrollbar.vertical.fit","The vertical scrollbar will always be hidden.")],default:"auto",description:m("scrollbar.vertical","Controls the visibility of the vertical scrollbar.")},"editor.scrollbar.horizontal":{type:"string",enum:["auto","visible","hidden"],enumDescriptions:[m("scrollbar.horizontal.auto","The horizontal scrollbar will be visible only when necessary."),m("scrollbar.horizontal.visible","The horizontal scrollbar will always be visible."),m("scrollbar.horizontal.fit","The horizontal scrollbar will always be hidden.")],default:"auto",description:m("scrollbar.horizontal","Controls the visibility of the horizontal scrollbar.")},"editor.scrollbar.verticalScrollbarSize":{type:"number",default:e.verticalScrollbarSize,description:m("scrollbar.verticalScrollbarSize","The width of the vertical scrollbar.")},"editor.scrollbar.horizontalScrollbarSize":{type:"number",default:e.horizontalScrollbarSize,description:m("scrollbar.horizontalScrollbarSize","The height of the horizontal scrollbar.")},"editor.scrollbar.scrollByPage":{type:"boolean",default:e.scrollByPage,description:m("scrollbar.scrollByPage","Controls whether clicks scroll by page or jump to click position.")},"editor.scrollbar.ignoreHorizontalScrollbarInContentHeight":{type:"boolean",default:e.ignoreHorizontalScrollbarInContentHeight,description:m("scrollbar.ignoreHorizontalScrollbarInContentHeight","When set, the horizontal scrollbar will not increase the size of the editor's content.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e,i=At.clampedInt(t.horizontalScrollbarSize,this.defaultValue.horizontalScrollbarSize,0,1e3),n=At.clampedInt(t.verticalScrollbarSize,this.defaultValue.verticalScrollbarSize,0,1e3);return{arrowSize:At.clampedInt(t.arrowSize,this.defaultValue.arrowSize,0,1e3),vertical:$B(t.vertical,this.defaultValue.vertical),horizontal:$B(t.horizontal,this.defaultValue.horizontal),useShadows:Ce(t.useShadows,this.defaultValue.useShadows),verticalHasArrows:Ce(t.verticalHasArrows,this.defaultValue.verticalHasArrows),horizontalHasArrows:Ce(t.horizontalHasArrows,this.defaultValue.horizontalHasArrows),handleMouseWheel:Ce(t.handleMouseWheel,this.defaultValue.handleMouseWheel),alwaysConsumeMouseWheel:Ce(t.alwaysConsumeMouseWheel,this.defaultValue.alwaysConsumeMouseWheel),horizontalScrollbarSize:i,horizontalSliderSize:At.clampedInt(t.horizontalSliderSize,i,0,1e3),verticalScrollbarSize:n,verticalSliderSize:At.clampedInt(t.verticalSliderSize,n,0,1e3),scrollByPage:Ce(t.scrollByPage,this.defaultValue.scrollByPage),ignoreHorizontalScrollbarInContentHeight:Ce(t.ignoreHorizontalScrollbarInContentHeight,this.defaultValue.ignoreHorizontalScrollbarInContentHeight)}}};const xo="inUntrustedWorkspace",Es={allowedCharacters:"editor.unicodeHighlight.allowedCharacters",invisibleCharacters:"editor.unicodeHighlight.invisibleCharacters",nonBasicASCII:"editor.unicodeHighlight.nonBasicASCII",ambiguousCharacters:"editor.unicodeHighlight.ambiguousCharacters",includeComments:"editor.unicodeHighlight.includeComments",includeStrings:"editor.unicodeHighlight.includeStrings",allowedLocales:"editor.unicodeHighlight.allowedLocales"};class vJ extends Jt{constructor(){const e={nonBasicASCII:xo,invisibleCharacters:!0,ambiguousCharacters:!0,includeComments:xo,includeStrings:!0,allowedCharacters:{},allowedLocales:{_os:!0,_vscode:!0}};super(126,"unicodeHighlight",e,{[Es.nonBasicASCII]:{restricted:!0,type:["boolean","string"],enum:[!0,!1,xo],default:e.nonBasicASCII,description:m("unicodeHighlight.nonBasicASCII","Controls whether all non-basic ASCII characters are highlighted. Only characters between U+0020 and U+007E, tab, line-feed and carriage-return are considered basic ASCII.")},[Es.invisibleCharacters]:{restricted:!0,type:"boolean",default:e.invisibleCharacters,description:m("unicodeHighlight.invisibleCharacters","Controls whether characters that just reserve space or have no width at all are highlighted.")},[Es.ambiguousCharacters]:{restricted:!0,type:"boolean",default:e.ambiguousCharacters,description:m("unicodeHighlight.ambiguousCharacters","Controls whether characters are highlighted that can be confused with basic ASCII characters, except those that are common in the current user locale.")},[Es.includeComments]:{restricted:!0,type:["boolean","string"],enum:[!0,!1,xo],default:e.includeComments,description:m("unicodeHighlight.includeComments","Controls whether characters in comments should also be subject to Unicode highlighting.")},[Es.includeStrings]:{restricted:!0,type:["boolean","string"],enum:[!0,!1,xo],default:e.includeStrings,description:m("unicodeHighlight.includeStrings","Controls whether characters in strings should also be subject to Unicode highlighting.")},[Es.allowedCharacters]:{restricted:!0,type:"object",default:e.allowedCharacters,description:m("unicodeHighlight.allowedCharacters","Defines allowed characters that are not being highlighted."),additionalProperties:{type:"boolean"}},[Es.allowedLocales]:{restricted:!0,type:"object",additionalProperties:{type:"boolean"},default:e.allowedLocales,description:m("unicodeHighlight.allowedLocales","Unicode characters that are common in allowed locales are not being highlighted.")}})}applyUpdate(e,t){let i=!1;t.allowedCharacters&&e&&(oo(e.allowedCharacters,t.allowedCharacters)||(e={...e,allowedCharacters:t.allowedCharacters},i=!0)),t.allowedLocales&&e&&(oo(e.allowedLocales,t.allowedLocales)||(e={...e,allowedLocales:t.allowedLocales},i=!0));const n=super.applyUpdate(e,t);return i?new $C(n.newValue,!0):n}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{nonBasicASCII:Vp(t.nonBasicASCII,xo,[!0,!1,xo]),invisibleCharacters:Ce(t.invisibleCharacters,this.defaultValue.invisibleCharacters),ambiguousCharacters:Ce(t.ambiguousCharacters,this.defaultValue.ambiguousCharacters),includeComments:Vp(t.includeComments,xo,[!0,!1,xo]),includeStrings:Vp(t.includeStrings,xo,[!0,!1,xo]),allowedCharacters:this.validateBooleanMap(e.allowedCharacters,this.defaultValue.allowedCharacters),allowedLocales:this.validateBooleanMap(e.allowedLocales,this.defaultValue.allowedLocales)}}validateBooleanMap(e,t){if(typeof e!="object"||!e)return t;const i={};for(const[n,s]of Object.entries(e))s===!0&&(i[n]=!0);return i}}class wJ extends Jt{constructor(){const e={enabled:!0,mode:"subwordSmart",showToolbar:"onHover",suppressSuggestions:!1,keepOnBlur:!1,fontFamily:"default"};super(62,"inlineSuggest",e,{"editor.inlineSuggest.enabled":{type:"boolean",default:e.enabled,description:m("inlineSuggest.enabled","Controls whether to automatically show inline suggestions in the editor.")},"editor.inlineSuggest.showToolbar":{type:"string",default:e.showToolbar,enum:["always","onHover","never"],enumDescriptions:[m("inlineSuggest.showToolbar.always","Show the inline suggestion toolbar whenever an inline suggestion is shown."),m("inlineSuggest.showToolbar.onHover","Show the inline suggestion toolbar when hovering over an inline suggestion."),m("inlineSuggest.showToolbar.never","Never show the inline suggestion toolbar.")],description:m("inlineSuggest.showToolbar","Controls when to show the inline suggestion toolbar.")},"editor.inlineSuggest.suppressSuggestions":{type:"boolean",default:e.suppressSuggestions,description:m("inlineSuggest.suppressSuggestions","Controls how inline suggestions interact with the suggest widget. If enabled, the suggest widget is not shown automatically when inline suggestions are available.")},"editor.inlineSuggest.fontFamily":{type:"string",default:e.fontFamily,description:m("inlineSuggest.fontFamily","Controls the font family of the inline suggestions.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{enabled:Ce(t.enabled,this.defaultValue.enabled),mode:gi(t.mode,this.defaultValue.mode,["prefix","subword","subwordSmart"]),showToolbar:gi(t.showToolbar,this.defaultValue.showToolbar,["always","onHover","never"]),suppressSuggestions:Ce(t.suppressSuggestions,this.defaultValue.suppressSuggestions),keepOnBlur:Ce(t.keepOnBlur,this.defaultValue.keepOnBlur),fontFamily:Zn.string(t.fontFamily,this.defaultValue.fontFamily)}}}class SJ extends Jt{constructor(){const e={enabled:!1,showToolbar:"onHover",fontFamily:"default",keepOnBlur:!1};super(63,"experimentalInlineEdit",e,{"editor.experimentalInlineEdit.enabled":{type:"boolean",default:e.enabled,description:m("inlineEdit.enabled","Controls whether to show inline edits in the editor.")},"editor.experimentalInlineEdit.showToolbar":{type:"string",default:e.showToolbar,enum:["always","onHover","never"],enumDescriptions:[m("inlineEdit.showToolbar.always","Show the inline edit toolbar whenever an inline suggestion is shown."),m("inlineEdit.showToolbar.onHover","Show the inline edit toolbar when hovering over an inline suggestion."),m("inlineEdit.showToolbar.never","Never show the inline edit toolbar.")],description:m("inlineEdit.showToolbar","Controls when to show the inline edit toolbar.")},"editor.experimentalInlineEdit.fontFamily":{type:"string",default:e.fontFamily,description:m("inlineEdit.fontFamily","Controls the font family of the inline edit.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{enabled:Ce(t.enabled,this.defaultValue.enabled),showToolbar:gi(t.showToolbar,this.defaultValue.showToolbar,["always","onHover","never"]),fontFamily:Zn.string(t.fontFamily,this.defaultValue.fontFamily),keepOnBlur:Ce(t.keepOnBlur,this.defaultValue.keepOnBlur)}}}class yJ extends Jt{constructor(){const e={enabled:An.bracketPairColorizationOptions.enabled,independentColorPoolPerBracketType:An.bracketPairColorizationOptions.independentColorPoolPerBracketType};super(15,"bracketPairColorization",e,{"editor.bracketPairColorization.enabled":{type:"boolean",default:e.enabled,markdownDescription:m("bracketPairColorization.enabled","Controls whether bracket pair colorization is enabled or not. Use {0} to override the bracket highlight colors.","`#workbench.colorCustomizations#`")},"editor.bracketPairColorization.independentColorPoolPerBracketType":{type:"boolean",default:e.independentColorPoolPerBracketType,description:m("bracketPairColorization.independentColorPoolPerBracketType","Controls whether each bracket type has its own independent color pool.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{enabled:Ce(t.enabled,this.defaultValue.enabled),independentColorPoolPerBracketType:Ce(t.independentColorPoolPerBracketType,this.defaultValue.independentColorPoolPerBracketType)}}}class LJ extends Jt{constructor(){const e={bracketPairs:!1,bracketPairsHorizontal:"active",highlightActiveBracketPair:!0,indentation:!0,highlightActiveIndentation:!0};super(16,"guides",e,{"editor.guides.bracketPairs":{type:["boolean","string"],enum:[!0,"active",!1],enumDescriptions:[m("editor.guides.bracketPairs.true","Enables bracket pair guides."),m("editor.guides.bracketPairs.active","Enables bracket pair guides only for the active bracket pair."),m("editor.guides.bracketPairs.false","Disables bracket pair guides.")],default:e.bracketPairs,description:m("editor.guides.bracketPairs","Controls whether bracket pair guides are enabled or not.")},"editor.guides.bracketPairsHorizontal":{type:["boolean","string"],enum:[!0,"active",!1],enumDescriptions:[m("editor.guides.bracketPairsHorizontal.true","Enables horizontal guides as addition to vertical bracket pair guides."),m("editor.guides.bracketPairsHorizontal.active","Enables horizontal guides only for the active bracket pair."),m("editor.guides.bracketPairsHorizontal.false","Disables horizontal bracket pair guides.")],default:e.bracketPairsHorizontal,description:m("editor.guides.bracketPairsHorizontal","Controls whether horizontal bracket pair guides are enabled or not.")},"editor.guides.highlightActiveBracketPair":{type:"boolean",default:e.highlightActiveBracketPair,description:m("editor.guides.highlightActiveBracketPair","Controls whether the editor should highlight the active bracket pair.")},"editor.guides.indentation":{type:"boolean",default:e.indentation,description:m("editor.guides.indentation","Controls whether the editor should render indent guides.")},"editor.guides.highlightActiveIndentation":{type:["boolean","string"],enum:[!0,"always",!1],enumDescriptions:[m("editor.guides.highlightActiveIndentation.true","Highlights the active indent guide."),m("editor.guides.highlightActiveIndentation.always","Highlights the active indent guide even if bracket guides are highlighted."),m("editor.guides.highlightActiveIndentation.false","Do not highlight the active indent guide.")],default:e.highlightActiveIndentation,description:m("editor.guides.highlightActiveIndentation","Controls whether the editor should highlight the active indent guide.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{bracketPairs:Vp(t.bracketPairs,this.defaultValue.bracketPairs,[!0,!1,"active"]),bracketPairsHorizontal:Vp(t.bracketPairsHorizontal,this.defaultValue.bracketPairsHorizontal,[!0,!1,"active"]),highlightActiveBracketPair:Ce(t.highlightActiveBracketPair,this.defaultValue.highlightActiveBracketPair),indentation:Ce(t.indentation,this.defaultValue.indentation),highlightActiveIndentation:Vp(t.highlightActiveIndentation,this.defaultValue.highlightActiveIndentation,[!0,!1,"always"])}}}function Vp(o,e,t){const i=t.indexOf(o);return i===-1?e:t[i]}class xJ extends Jt{constructor(){const e={insertMode:"insert",filterGraceful:!0,snippetsPreventQuickSuggestions:!1,localityBonus:!1,shareSuggestSelections:!1,selectionMode:"always",showIcons:!0,showStatusBar:!1,preview:!1,previewMode:"subwordSmart",showInlineDetails:!0,showMethods:!0,showFunctions:!0,showConstructors:!0,showDeprecated:!0,matchOnWordStartOnly:!0,showFields:!0,showVariables:!0,showClasses:!0,showStructs:!0,showInterfaces:!0,showModules:!0,showProperties:!0,showEvents:!0,showOperators:!0,showUnits:!0,showValues:!0,showConstants:!0,showEnums:!0,showEnumMembers:!0,showKeywords:!0,showWords:!0,showColors:!0,showFiles:!0,showReferences:!0,showFolders:!0,showTypeParameters:!0,showSnippets:!0,showUsers:!0,showIssues:!0};super(119,"suggest",e,{"editor.suggest.insertMode":{type:"string",enum:["insert","replace"],enumDescriptions:[m("suggest.insertMode.insert","Insert suggestion without overwriting text right of the cursor."),m("suggest.insertMode.replace","Insert suggestion and overwrite text right of the cursor.")],default:e.insertMode,description:m("suggest.insertMode","Controls whether words are overwritten when accepting completions. Note that this depends on extensions opting into this feature.")},"editor.suggest.filterGraceful":{type:"boolean",default:e.filterGraceful,description:m("suggest.filterGraceful","Controls whether filtering and sorting suggestions accounts for small typos.")},"editor.suggest.localityBonus":{type:"boolean",default:e.localityBonus,description:m("suggest.localityBonus","Controls whether sorting favors words that appear close to the cursor.")},"editor.suggest.shareSuggestSelections":{type:"boolean",default:e.shareSuggestSelections,markdownDescription:m("suggest.shareSuggestSelections","Controls whether remembered suggestion selections are shared between multiple workspaces and windows (needs `#editor.suggestSelection#`).")},"editor.suggest.selectionMode":{type:"string",enum:["always","never","whenTriggerCharacter","whenQuickSuggestion"],enumDescriptions:[m("suggest.insertMode.always","Always select a suggestion when automatically triggering IntelliSense."),m("suggest.insertMode.never","Never select a suggestion when automatically triggering IntelliSense."),m("suggest.insertMode.whenTriggerCharacter","Select a suggestion only when triggering IntelliSense from a trigger character."),m("suggest.insertMode.whenQuickSuggestion","Select a suggestion only when triggering IntelliSense as you type.")],default:e.selectionMode,markdownDescription:m("suggest.selectionMode","Controls whether a suggestion is selected when the widget shows. Note that this only applies to automatically triggered suggestions ({0} and {1}) and that a suggestion is always selected when explicitly invoked, e.g via `Ctrl+Space`.","`#editor.quickSuggestions#`","`#editor.suggestOnTriggerCharacters#`")},"editor.suggest.snippetsPreventQuickSuggestions":{type:"boolean",default:e.snippetsPreventQuickSuggestions,description:m("suggest.snippetsPreventQuickSuggestions","Controls whether an active snippet prevents quick suggestions.")},"editor.suggest.showIcons":{type:"boolean",default:e.showIcons,description:m("suggest.showIcons","Controls whether to show or hide icons in suggestions.")},"editor.suggest.showStatusBar":{type:"boolean",default:e.showStatusBar,description:m("suggest.showStatusBar","Controls the visibility of the status bar at the bottom of the suggest widget.")},"editor.suggest.preview":{type:"boolean",default:e.preview,description:m("suggest.preview","Controls whether to preview the suggestion outcome in the editor.")},"editor.suggest.showInlineDetails":{type:"boolean",default:e.showInlineDetails,description:m("suggest.showInlineDetails","Controls whether suggest details show inline with the label or only in the details widget.")},"editor.suggest.maxVisibleSuggestions":{type:"number",deprecationMessage:m("suggest.maxVisibleSuggestions.dep","This setting is deprecated. The suggest widget can now be resized.")},"editor.suggest.filteredTypes":{type:"object",deprecationMessage:m("deprecated","This setting is deprecated, please use separate settings like 'editor.suggest.showKeywords' or 'editor.suggest.showSnippets' instead.")},"editor.suggest.showMethods":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showMethods","When enabled IntelliSense shows `method`-suggestions.")},"editor.suggest.showFunctions":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showFunctions","When enabled IntelliSense shows `function`-suggestions.")},"editor.suggest.showConstructors":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showConstructors","When enabled IntelliSense shows `constructor`-suggestions.")},"editor.suggest.showDeprecated":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showDeprecated","When enabled IntelliSense shows `deprecated`-suggestions.")},"editor.suggest.matchOnWordStartOnly":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.matchOnWordStartOnly","When enabled IntelliSense filtering requires that the first character matches on a word start. For example, `c` on `Console` or `WebContext` but _not_ on `description`. When disabled IntelliSense will show more results but still sorts them by match quality.")},"editor.suggest.showFields":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showFields","When enabled IntelliSense shows `field`-suggestions.")},"editor.suggest.showVariables":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showVariables","When enabled IntelliSense shows `variable`-suggestions.")},"editor.suggest.showClasses":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showClasss","When enabled IntelliSense shows `class`-suggestions.")},"editor.suggest.showStructs":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showStructs","When enabled IntelliSense shows `struct`-suggestions.")},"editor.suggest.showInterfaces":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showInterfaces","When enabled IntelliSense shows `interface`-suggestions.")},"editor.suggest.showModules":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showModules","When enabled IntelliSense shows `module`-suggestions.")},"editor.suggest.showProperties":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showPropertys","When enabled IntelliSense shows `property`-suggestions.")},"editor.suggest.showEvents":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showEvents","When enabled IntelliSense shows `event`-suggestions.")},"editor.suggest.showOperators":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showOperators","When enabled IntelliSense shows `operator`-suggestions.")},"editor.suggest.showUnits":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showUnits","When enabled IntelliSense shows `unit`-suggestions.")},"editor.suggest.showValues":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showValues","When enabled IntelliSense shows `value`-suggestions.")},"editor.suggest.showConstants":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showConstants","When enabled IntelliSense shows `constant`-suggestions.")},"editor.suggest.showEnums":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showEnums","When enabled IntelliSense shows `enum`-suggestions.")},"editor.suggest.showEnumMembers":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showEnumMembers","When enabled IntelliSense shows `enumMember`-suggestions.")},"editor.suggest.showKeywords":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showKeywords","When enabled IntelliSense shows `keyword`-suggestions.")},"editor.suggest.showWords":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showTexts","When enabled IntelliSense shows `text`-suggestions.")},"editor.suggest.showColors":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showColors","When enabled IntelliSense shows `color`-suggestions.")},"editor.suggest.showFiles":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showFiles","When enabled IntelliSense shows `file`-suggestions.")},"editor.suggest.showReferences":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showReferences","When enabled IntelliSense shows `reference`-suggestions.")},"editor.suggest.showCustomcolors":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showCustomcolors","When enabled IntelliSense shows `customcolor`-suggestions.")},"editor.suggest.showFolders":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showFolders","When enabled IntelliSense shows `folder`-suggestions.")},"editor.suggest.showTypeParameters":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showTypeParameters","When enabled IntelliSense shows `typeParameter`-suggestions.")},"editor.suggest.showSnippets":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showSnippets","When enabled IntelliSense shows `snippet`-suggestions.")},"editor.suggest.showUsers":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showUsers","When enabled IntelliSense shows `user`-suggestions.")},"editor.suggest.showIssues":{type:"boolean",default:!0,markdownDescription:m("editor.suggest.showIssues","When enabled IntelliSense shows `issues`-suggestions.")}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{insertMode:gi(t.insertMode,this.defaultValue.insertMode,["insert","replace"]),filterGraceful:Ce(t.filterGraceful,this.defaultValue.filterGraceful),snippetsPreventQuickSuggestions:Ce(t.snippetsPreventQuickSuggestions,this.defaultValue.filterGraceful),localityBonus:Ce(t.localityBonus,this.defaultValue.localityBonus),shareSuggestSelections:Ce(t.shareSuggestSelections,this.defaultValue.shareSuggestSelections),selectionMode:gi(t.selectionMode,this.defaultValue.selectionMode,["always","never","whenQuickSuggestion","whenTriggerCharacter"]),showIcons:Ce(t.showIcons,this.defaultValue.showIcons),showStatusBar:Ce(t.showStatusBar,this.defaultValue.showStatusBar),preview:Ce(t.preview,this.defaultValue.preview),previewMode:gi(t.previewMode,this.defaultValue.previewMode,["prefix","subword","subwordSmart"]),showInlineDetails:Ce(t.showInlineDetails,this.defaultValue.showInlineDetails),showMethods:Ce(t.showMethods,this.defaultValue.showMethods),showFunctions:Ce(t.showFunctions,this.defaultValue.showFunctions),showConstructors:Ce(t.showConstructors,this.defaultValue.showConstructors),showDeprecated:Ce(t.showDeprecated,this.defaultValue.showDeprecated),matchOnWordStartOnly:Ce(t.matchOnWordStartOnly,this.defaultValue.matchOnWordStartOnly),showFields:Ce(t.showFields,this.defaultValue.showFields),showVariables:Ce(t.showVariables,this.defaultValue.showVariables),showClasses:Ce(t.showClasses,this.defaultValue.showClasses),showStructs:Ce(t.showStructs,this.defaultValue.showStructs),showInterfaces:Ce(t.showInterfaces,this.defaultValue.showInterfaces),showModules:Ce(t.showModules,this.defaultValue.showModules),showProperties:Ce(t.showProperties,this.defaultValue.showProperties),showEvents:Ce(t.showEvents,this.defaultValue.showEvents),showOperators:Ce(t.showOperators,this.defaultValue.showOperators),showUnits:Ce(t.showUnits,this.defaultValue.showUnits),showValues:Ce(t.showValues,this.defaultValue.showValues),showConstants:Ce(t.showConstants,this.defaultValue.showConstants),showEnums:Ce(t.showEnums,this.defaultValue.showEnums),showEnumMembers:Ce(t.showEnumMembers,this.defaultValue.showEnumMembers),showKeywords:Ce(t.showKeywords,this.defaultValue.showKeywords),showWords:Ce(t.showWords,this.defaultValue.showWords),showColors:Ce(t.showColors,this.defaultValue.showColors),showFiles:Ce(t.showFiles,this.defaultValue.showFiles),showReferences:Ce(t.showReferences,this.defaultValue.showReferences),showFolders:Ce(t.showFolders,this.defaultValue.showFolders),showTypeParameters:Ce(t.showTypeParameters,this.defaultValue.showTypeParameters),showSnippets:Ce(t.showSnippets,this.defaultValue.showSnippets),showUsers:Ce(t.showUsers,this.defaultValue.showUsers),showIssues:Ce(t.showIssues,this.defaultValue.showIssues)}}}class kJ extends Jt{constructor(){super(114,"smartSelect",{selectLeadingAndTrailingWhitespace:!0,selectSubwords:!0},{"editor.smartSelect.selectLeadingAndTrailingWhitespace":{description:m("selectLeadingAndTrailingWhitespace","Whether leading and trailing whitespace should always be selected."),default:!0,type:"boolean"},"editor.smartSelect.selectSubwords":{description:m("selectSubwords","Whether subwords (like 'foo' in 'fooBar' or 'foo_bar') should be selected."),default:!0,type:"boolean"}})}validate(e){return!e||typeof e!="object"?this.defaultValue:{selectLeadingAndTrailingWhitespace:Ce(e.selectLeadingAndTrailingWhitespace,this.defaultValue.selectLeadingAndTrailingWhitespace),selectSubwords:Ce(e.selectSubwords,this.defaultValue.selectSubwords)}}}class DJ extends Jt{constructor(){const e=[];super(131,"wordSegmenterLocales",e,{anyOf:[{description:m("wordSegmenterLocales","Locales to be used for word segmentation when doing word related navigations or operations. Specify the BCP 47 language tag of the word you wish to recognize (e.g., ja, zh-CN, zh-Hant-TW, etc.)."),type:"string"},{description:m("wordSegmenterLocales","Locales to be used for word segmentation when doing word related navigations or operations. Specify the BCP 47 language tag of the word you wish to recognize (e.g., ja, zh-CN, zh-Hant-TW, etc.)."),type:"array",items:{type:"string"}}]})}validate(e){if(typeof e=="string"&&(e=[e]),Array.isArray(e)){const t=[];for(const i of e)if(typeof i=="string")try{Intl.Segmenter.supportedLocalesOf(i).length>0&&t.push(i)}catch{}return t}return this.defaultValue}}class IJ extends Jt{constructor(){super(139,"wrappingIndent",1,{"editor.wrappingIndent":{type:"string",enum:["none","same","indent","deepIndent"],enumDescriptions:[m("wrappingIndent.none","No indentation. Wrapped lines begin at column 1."),m("wrappingIndent.same","Wrapped lines get the same indentation as the parent."),m("wrappingIndent.indent","Wrapped lines get +1 indentation toward the parent."),m("wrappingIndent.deepIndent","Wrapped lines get +2 indentation toward the parent.")],description:m("wrappingIndent","Controls the indentation of wrapped lines."),default:"same"}})}validate(e){switch(e){case"none":return 0;case"same":return 1;case"indent":return 2;case"deepIndent":return 3}return 1}compute(e,t,i){return t.get(2)===2?0:i}}class EJ extends T1{constructor(){super(147)}compute(e,t,i){const n=t.get(146);return{isDominatedByLongLines:e.isDominatedByLongLines,isWordWrapMinified:n.isWordWrapMinified,isViewportWrapping:n.isViewportWrapping,wrappingColumn:n.wrappingColumn}}}class NJ extends Jt{constructor(){const e={enabled:!0,showDropSelector:"afterDrop"};super(36,"dropIntoEditor",e,{"editor.dropIntoEditor.enabled":{type:"boolean",default:e.enabled,markdownDescription:m("dropIntoEditor.enabled","Controls whether you can drag and drop a file into a text editor by holding down the `Shift` key (instead of opening the file in an editor).")},"editor.dropIntoEditor.showDropSelector":{type:"string",markdownDescription:m("dropIntoEditor.showDropSelector","Controls if a widget is shown when dropping files into the editor. This widget lets you control how the file is dropped."),enum:["afterDrop","never"],enumDescriptions:[m("dropIntoEditor.showDropSelector.afterDrop","Show the drop selector widget after a file is dropped into the editor."),m("dropIntoEditor.showDropSelector.never","Never show the drop selector widget. Instead the default drop provider is always used.")],default:"afterDrop"}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{enabled:Ce(t.enabled,this.defaultValue.enabled),showDropSelector:gi(t.showDropSelector,this.defaultValue.showDropSelector,["afterDrop","never"])}}}class TJ extends Jt{constructor(){const e={enabled:!0,showPasteSelector:"afterPaste"};super(85,"pasteAs",e,{"editor.pasteAs.enabled":{type:"boolean",default:e.enabled,markdownDescription:m("pasteAs.enabled","Controls whether you can paste content in different ways.")},"editor.pasteAs.showPasteSelector":{type:"string",markdownDescription:m("pasteAs.showPasteSelector","Controls if a widget is shown when pasting content in to the editor. This widget lets you control how the file is pasted."),enum:["afterPaste","never"],enumDescriptions:[m("pasteAs.showPasteSelector.afterPaste","Show the paste selector widget after content is pasted into the editor."),m("pasteAs.showPasteSelector.never","Never show the paste selector widget. Instead the default pasting behavior is always used.")],default:"afterPaste"}})}validate(e){if(!e||typeof e!="object")return this.defaultValue;const t=e;return{enabled:Ce(t.enabled,this.defaultValue.enabled),showPasteSelector:gi(t.showPasteSelector,this.defaultValue.showPasteSelector,["afterPaste","never"])}}}const RJ="Consolas, 'Courier New', monospace",MJ="Menlo, Monaco, 'Courier New', monospace",AJ="'Droid Sans Mono', 'monospace', monospace",fs={fontFamily:Je?MJ:ps?AJ:RJ,fontWeight:"normal",fontSize:Je?12:14,lineHeight:0,letterSpacing:0},Jm=[];function se(o){return Jm[o.id]=o,o}const Ua={acceptSuggestionOnCommitCharacter:se(new gt(0,"acceptSuggestionOnCommitCharacter",!0,{markdownDescription:m("acceptSuggestionOnCommitCharacter","Controls whether suggestions should be accepted on commit characters. For example, in JavaScript, the semi-colon (`;`) can be a commit character that accepts a suggestion and types that character.")})),acceptSuggestionOnEnter:se(new ui(1,"acceptSuggestionOnEnter","on",["on","smart","off"],{markdownEnumDescriptions:["",m("acceptSuggestionOnEnterSmart","Only accept a suggestion with `Enter` when it makes a textual change."),""],markdownDescription:m("acceptSuggestionOnEnter","Controls whether suggestions should be accepted on `Enter`, in addition to `Tab`. Helps to avoid ambiguity between inserting new lines or accepting suggestions.")})),accessibilitySupport:se(new KQ),accessibilityPageSize:se(new At(3,"accessibilityPageSize",10,1,1073741824,{description:m("accessibilityPageSize","Controls the number of lines in the editor that can be read out by a screen reader at once. When we detect a screen reader we automatically set the default to be 500. Warning: this has a performance implication for numbers larger than the default."),tags:["accessibility"]})),ariaLabel:se(new Zn(4,"ariaLabel",m("editorViewAccessibleLabel","Editor content"))),ariaRequired:se(new gt(5,"ariaRequired",!1,void 0)),screenReaderAnnounceInlineSuggestion:se(new gt(8,"screenReaderAnnounceInlineSuggestion",!0,{description:m("screenReaderAnnounceInlineSuggestion","Control whether inline suggestions are announced by a screen reader."),tags:["accessibility"]})),autoClosingBrackets:se(new ui(6,"autoClosingBrackets","languageDefined",["always","languageDefined","beforeWhitespace","never"],{enumDescriptions:["",m("editor.autoClosingBrackets.languageDefined","Use language configurations to determine when to autoclose brackets."),m("editor.autoClosingBrackets.beforeWhitespace","Autoclose brackets only when the cursor is to the left of whitespace."),""],description:m("autoClosingBrackets","Controls whether the editor should automatically close brackets after the user adds an opening bracket.")})),autoClosingComments:se(new ui(7,"autoClosingComments","languageDefined",["always","languageDefined","beforeWhitespace","never"],{enumDescriptions:["",m("editor.autoClosingComments.languageDefined","Use language configurations to determine when to autoclose comments."),m("editor.autoClosingComments.beforeWhitespace","Autoclose comments only when the cursor is to the left of whitespace."),""],description:m("autoClosingComments","Controls whether the editor should automatically close comments after the user adds an opening comment.")})),autoClosingDelete:se(new ui(9,"autoClosingDelete","auto",["always","auto","never"],{enumDescriptions:["",m("editor.autoClosingDelete.auto","Remove adjacent closing quotes or brackets only if they were automatically inserted."),""],description:m("autoClosingDelete","Controls whether the editor should remove adjacent closing quotes or brackets when deleting.")})),autoClosingOvertype:se(new ui(10,"autoClosingOvertype","auto",["always","auto","never"],{enumDescriptions:["",m("editor.autoClosingOvertype.auto","Type over closing quotes or brackets only if they were automatically inserted."),""],description:m("autoClosingOvertype","Controls whether the editor should type over closing quotes or brackets.")})),autoClosingQuotes:se(new ui(11,"autoClosingQuotes","languageDefined",["always","languageDefined","beforeWhitespace","never"],{enumDescriptions:["",m("editor.autoClosingQuotes.languageDefined","Use language configurations to determine when to autoclose quotes."),m("editor.autoClosingQuotes.beforeWhitespace","Autoclose quotes only when the cursor is to the left of whitespace."),""],description:m("autoClosingQuotes","Controls whether the editor should automatically close quotes after the user adds an opening quote.")})),autoIndent:se(new kS(12,"autoIndent",4,"full",["none","keep","brackets","advanced","full"],jQ,{enumDescriptions:[m("editor.autoIndent.none","The editor will not insert indentation automatically."),m("editor.autoIndent.keep","The editor will keep the current line's indentation."),m("editor.autoIndent.brackets","The editor will keep the current line's indentation and honor language defined brackets."),m("editor.autoIndent.advanced","The editor will keep the current line's indentation, honor language defined brackets and invoke special onEnterRules defined by languages."),m("editor.autoIndent.full","The editor will keep the current line's indentation, honor language defined brackets, invoke special onEnterRules defined by languages, and honor indentationRules defined by languages.")],description:m("autoIndent","Controls whether the editor should automatically adjust the indentation when users type, paste, move or indent lines.")})),automaticLayout:se(new gt(13,"automaticLayout",!1)),autoSurround:se(new ui(14,"autoSurround","languageDefined",["languageDefined","quotes","brackets","never"],{enumDescriptions:[m("editor.autoSurround.languageDefined","Use language configurations to determine when to automatically surround selections."),m("editor.autoSurround.quotes","Surround with quotes but not brackets."),m("editor.autoSurround.brackets","Surround with brackets but not quotes."),""],description:m("autoSurround","Controls whether the editor should automatically surround selections when typing quotes or brackets.")})),bracketPairColorization:se(new yJ),bracketPairGuides:se(new LJ),stickyTabStops:se(new gt(117,"stickyTabStops",!1,{description:m("stickyTabStops","Emulate selection behavior of tab characters when using spaces for indentation. Selection will stick to tab stops.")})),codeLens:se(new gt(17,"codeLens",!0,{description:m("codeLens","Controls whether the editor shows CodeLens.")})),codeLensFontFamily:se(new Zn(18,"codeLensFontFamily","",{description:m("codeLensFontFamily","Controls the font family for CodeLens.")})),codeLensFontSize:se(new At(19,"codeLensFontSize",0,0,100,{type:"number",default:0,minimum:0,maximum:100,markdownDescription:m("codeLensFontSize","Controls the font size in pixels for CodeLens. When set to 0, 90% of `#editor.fontSize#` is used.")})),colorDecorators:se(new gt(20,"colorDecorators",!0,{description:m("colorDecorators","Controls whether the editor should render the inline color decorators and color picker.")})),colorDecoratorActivatedOn:se(new ui(149,"colorDecoratorsActivatedOn","clickAndHover",["clickAndHover","hover","click"],{enumDescriptions:[m("editor.colorDecoratorActivatedOn.clickAndHover","Make the color picker appear both on click and hover of the color decorator"),m("editor.colorDecoratorActivatedOn.hover","Make the color picker appear on hover of the color decorator"),m("editor.colorDecoratorActivatedOn.click","Make the color picker appear on click of the color decorator")],description:m("colorDecoratorActivatedOn","Controls the condition to make a color picker appear from a color decorator")})),colorDecoratorsLimit:se(new At(21,"colorDecoratorsLimit",500,1,1e6,{markdownDescription:m("colorDecoratorsLimit","Controls the max number of color decorators that can be rendered in an editor at once.")})),columnSelection:se(new gt(22,"columnSelection",!1,{description:m("columnSelection","Enable that the selection with the mouse and keys is doing column selection.")})),comments:se(new qQ),contextmenu:se(new gt(24,"contextmenu",!0)),copyWithSyntaxHighlighting:se(new gt(25,"copyWithSyntaxHighlighting",!0,{description:m("copyWithSyntaxHighlighting","Controls whether syntax highlighting should be copied into the clipboard.")})),cursorBlinking:se(new kS(26,"cursorBlinking",1,"blink",["blink","smooth","phase","expand","solid"],GQ,{description:m("cursorBlinking","Control the cursor animation style.")})),cursorSmoothCaretAnimation:se(new ui(27,"cursorSmoothCaretAnimation","off",["off","explicit","on"],{enumDescriptions:[m("cursorSmoothCaretAnimation.off","Smooth caret animation is disabled."),m("cursorSmoothCaretAnimation.explicit","Smooth caret animation is enabled only when the user moves the cursor with an explicit gesture."),m("cursorSmoothCaretAnimation.on","Smooth caret animation is always enabled.")],description:m("cursorSmoothCaretAnimation","Controls whether the smooth caret animation should be enabled.")})),cursorStyle:se(new kS(28,"cursorStyle",fn.Line,"line",["line","block","underline","line-thin","block-outline","underline-thin"],ZQ,{description:m("cursorStyle","Controls the cursor style.")})),cursorSurroundingLines:se(new At(29,"cursorSurroundingLines",0,0,1073741824,{description:m("cursorSurroundingLines","Controls the minimal number of visible leading lines (minimum 0) and trailing lines (minimum 1) surrounding the cursor. Known as 'scrollOff' or 'scrollOffset' in some other editors.")})),cursorSurroundingLinesStyle:se(new ui(30,"cursorSurroundingLinesStyle","default",["default","all"],{enumDescriptions:[m("cursorSurroundingLinesStyle.default","`cursorSurroundingLines` is enforced only when triggered via the keyboard or API."),m("cursorSurroundingLinesStyle.all","`cursorSurroundingLines` is enforced always.")],markdownDescription:m("cursorSurroundingLinesStyle","Controls when `#editor.cursorSurroundingLines#` should be enforced.")})),cursorWidth:se(new At(31,"cursorWidth",0,0,1073741824,{markdownDescription:m("cursorWidth","Controls the width of the cursor when `#editor.cursorStyle#` is set to `line`.")})),disableLayerHinting:se(new gt(32,"disableLayerHinting",!1)),disableMonospaceOptimizations:se(new gt(33,"disableMonospaceOptimizations",!1)),domReadOnly:se(new gt(34,"domReadOnly",!1)),dragAndDrop:se(new gt(35,"dragAndDrop",!0,{description:m("dragAndDrop","Controls whether the editor should allow moving selections via drag and drop.")})),emptySelectionClipboard:se(new XQ),dropIntoEditor:se(new NJ),stickyScroll:se(new oJ),experimentalWhitespaceRendering:se(new ui(38,"experimentalWhitespaceRendering","svg",["svg","font","off"],{enumDescriptions:[m("experimentalWhitespaceRendering.svg","Use a new rendering method with svgs."),m("experimentalWhitespaceRendering.font","Use a new rendering method with font characters."),m("experimentalWhitespaceRendering.off","Use the stable rendering method.")],description:m("experimentalWhitespaceRendering","Controls whether whitespace is rendered with a new, experimental method.")})),extraEditorClassName:se(new Zn(39,"extraEditorClassName","")),fastScrollSensitivity:se(new Oo(40,"fastScrollSensitivity",5,o=>o<=0?5:o,{markdownDescription:m("fastScrollSensitivity","Scrolling speed multiplier when pressing `Alt`.")})),find:se(new QQ),fixedOverflowWidgets:se(new gt(42,"fixedOverflowWidgets",!1)),folding:se(new gt(43,"folding",!0,{description:m("folding","Controls whether the editor has code folding enabled.")})),foldingStrategy:se(new ui(44,"foldingStrategy","auto",["auto","indentation"],{enumDescriptions:[m("foldingStrategy.auto","Use a language-specific folding strategy if available, else the indentation-based one."),m("foldingStrategy.indentation","Use the indentation-based folding strategy.")],description:m("foldingStrategy","Controls the strategy for computing folding ranges.")})),foldingHighlight:se(new gt(45,"foldingHighlight",!0,{description:m("foldingHighlight","Controls whether the editor should highlight folded ranges.")})),foldingImportsByDefault:se(new gt(46,"foldingImportsByDefault",!1,{description:m("foldingImportsByDefault","Controls whether the editor automatically collapses import ranges.")})),foldingMaximumRegions:se(new At(47,"foldingMaximumRegions",5e3,10,65e3,{description:m("foldingMaximumRegions","The maximum number of foldable regions. Increasing this value may result in the editor becoming less responsive when the current source has a large number of foldable regions.")})),unfoldOnClickAfterEndOfLine:se(new gt(48,"unfoldOnClickAfterEndOfLine",!1,{description:m("unfoldOnClickAfterEndOfLine","Controls whether clicking on the empty content after a folded line will unfold the line.")})),fontFamily:se(new Zn(49,"fontFamily",fs.fontFamily,{description:m("fontFamily","Controls the font family.")})),fontInfo:se(new JQ),fontLigatures2:se(new Mh),fontSize:se(new eJ),fontWeight:se(new fR),fontVariations:se(new Jv),formatOnPaste:se(new gt(55,"formatOnPaste",!1,{description:m("formatOnPaste","Controls whether the editor should automatically format the pasted content. A formatter must be available and the formatter should be able to format a range in a document.")})),formatOnType:se(new gt(56,"formatOnType",!1,{description:m("formatOnType","Controls whether the editor should automatically format the line after typing.")})),glyphMargin:se(new gt(57,"glyphMargin",!0,{description:m("glyphMargin","Controls whether the editor should render the vertical glyph margin. Glyph margin is mostly used for debugging.")})),gotoLocation:se(new tJ),hideCursorInOverviewRuler:se(new gt(59,"hideCursorInOverviewRuler",!1,{description:m("hideCursorInOverviewRuler","Controls whether the cursor should be hidden in the overview ruler.")})),hover:se(new iJ),inDiffEditor:se(new gt(61,"inDiffEditor",!1)),letterSpacing:se(new Oo(64,"letterSpacing",fs.letterSpacing,o=>Oo.clamp(o,-5,20),{description:m("letterSpacing","Controls the letter spacing in pixels.")})),lightbulb:se(new sJ),lineDecorationsWidth:se(new aJ),lineHeight:se(new lJ),lineNumbers:se(new pJ),lineNumbersMinChars:se(new At(69,"lineNumbersMinChars",5,1,300)),linkedEditing:se(new gt(70,"linkedEditing",!1,{description:m("linkedEditing","Controls whether the editor has linked editing enabled. Depending on the language, related symbols such as HTML tags, are updated while editing.")})),links:se(new gt(71,"links",!0,{description:m("links","Controls whether the editor should detect links and make them clickable.")})),matchBrackets:se(new ui(72,"matchBrackets","always",["always","near","never"],{description:m("matchBrackets","Highlight matching brackets.")})),minimap:se(new cJ),mouseStyle:se(new ui(74,"mouseStyle","text",["text","default","copy"])),mouseWheelScrollSensitivity:se(new Oo(75,"mouseWheelScrollSensitivity",1,o=>o===0?1:o,{markdownDescription:m("mouseWheelScrollSensitivity","A multiplier to be used on the `deltaX` and `deltaY` of mouse wheel scroll events.")})),mouseWheelZoom:se(new gt(76,"mouseWheelZoom",!1,{markdownDescription:Je?m("mouseWheelZoom.mac","Zoom the font of the editor when using mouse wheel and holding `Cmd`."):m("mouseWheelZoom","Zoom the font of the editor when using mouse wheel and holding `Ctrl`.")})),multiCursorMergeOverlapping:se(new gt(77,"multiCursorMergeOverlapping",!0,{description:m("multiCursorMergeOverlapping","Merge multiple cursors when they are overlapping.")})),multiCursorModifier:se(new kS(78,"multiCursorModifier","altKey","alt",["ctrlCmd","alt"],dJ,{markdownEnumDescriptions:[m("multiCursorModifier.ctrlCmd","Maps to `Control` on Windows and Linux and to `Command` on macOS."),m("multiCursorModifier.alt","Maps to `Alt` on Windows and Linux and to `Option` on macOS.")],markdownDescription:m({key:"multiCursorModifier",comment:["- `ctrlCmd` refers to a value the setting can take and should not be localized.","- `Control` and `Command` refer to the modifier keys Ctrl or Cmd on the keyboard and can be localized."]},"The modifier to be used to add multiple cursors with the mouse. The Go to Definition and Open Link mouse gestures will adapt such that they do not conflict with the [multicursor modifier](https://code.visualstudio.com/docs/editor/codebasics#_multicursor-modifier).")})),multiCursorPaste:se(new ui(79,"multiCursorPaste","spread",["spread","full"],{markdownEnumDescriptions:[m("multiCursorPaste.spread","Each cursor pastes a single line of the text."),m("multiCursorPaste.full","Each cursor pastes the full text.")],markdownDescription:m("multiCursorPaste","Controls pasting when the line count of the pasted text matches the cursor count.")})),multiCursorLimit:se(new At(80,"multiCursorLimit",1e4,1,1e5,{markdownDescription:m("multiCursorLimit","Controls the max number of cursors that can be in an active editor at once.")})),occurrencesHighlight:se(new ui(81,"occurrencesHighlight","singleFile",["off","singleFile","multiFile"],{markdownEnumDescriptions:[m("occurrencesHighlight.off","Does not highlight occurrences."),m("occurrencesHighlight.singleFile","Highlights occurrences only in the current file."),m("occurrencesHighlight.multiFile","Experimental: Highlights occurrences across all valid open files.")],markdownDescription:m("occurrencesHighlight","Controls whether occurrences should be highlighted across open files.")})),overviewRulerBorder:se(new gt(82,"overviewRulerBorder",!0,{description:m("overviewRulerBorder","Controls whether a border should be drawn around the overview ruler.")})),overviewRulerLanes:se(new At(83,"overviewRulerLanes",3,0,3)),padding:se(new hJ),pasteAs:se(new TJ),parameterHints:se(new uJ),peekWidgetDefaultFocus:se(new ui(87,"peekWidgetDefaultFocus","tree",["tree","editor"],{enumDescriptions:[m("peekWidgetDefaultFocus.tree","Focus the tree when opening peek"),m("peekWidgetDefaultFocus.editor","Focus the editor when opening peek")],description:m("peekWidgetDefaultFocus","Controls whether to focus the inline editor or the tree in the peek widget.")})),placeholder:se(new fJ),definitionLinkOpensInPeek:se(new gt(89,"definitionLinkOpensInPeek",!1,{description:m("definitionLinkOpensInPeek","Controls whether the Go to Definition mouse gesture always opens the peek widget.")})),quickSuggestions:se(new mJ),quickSuggestionsDelay:se(new At(91,"quickSuggestionsDelay",10,0,1073741824,{description:m("quickSuggestionsDelay","Controls the delay in milliseconds after which quick suggestions will show up.")})),readOnly:se(new gt(92,"readOnly",!1)),readOnlyMessage:se(new bJ),renameOnType:se(new gt(94,"renameOnType",!1,{description:m("renameOnType","Controls whether the editor auto renames on type."),markdownDeprecationMessage:m("renameOnTypeDeprecate","Deprecated, use `editor.linkedEditing` instead.")})),renderControlCharacters:se(new gt(95,"renderControlCharacters",!0,{description:m("renderControlCharacters","Controls whether the editor should render control characters."),restricted:!0})),renderFinalNewline:se(new ui(96,"renderFinalNewline",ps?"dimmed":"on",["off","on","dimmed"],{description:m("renderFinalNewline","Render last line number when the file ends with a newline.")})),renderLineHighlight:se(new ui(97,"renderLineHighlight","line",["none","gutter","line","all"],{enumDescriptions:["","","",m("renderLineHighlight.all","Highlights both the gutter and the current line.")],description:m("renderLineHighlight","Controls how the editor should render the current line highlight.")})),renderLineHighlightOnlyWhenFocus:se(new gt(98,"renderLineHighlightOnlyWhenFocus",!1,{description:m("renderLineHighlightOnlyWhenFocus","Controls if the editor should render the current line highlight only when the editor is focused.")})),renderValidationDecorations:se(new ui(99,"renderValidationDecorations","editable",["editable","on","off"])),renderWhitespace:se(new ui(100,"renderWhitespace","selection",["none","boundary","selection","trailing","all"],{enumDescriptions:["",m("renderWhitespace.boundary","Render whitespace characters except for single spaces between words."),m("renderWhitespace.selection","Render whitespace characters only on selected text."),m("renderWhitespace.trailing","Render only trailing whitespace characters."),""],description:m("renderWhitespace","Controls how the editor should render whitespace characters.")})),revealHorizontalRightPadding:se(new At(101,"revealHorizontalRightPadding",15,0,1e3)),roundedSelection:se(new gt(102,"roundedSelection",!0,{description:m("roundedSelection","Controls whether selections should have rounded corners.")})),rulers:se(new _J),scrollbar:se(new CJ),scrollBeyondLastColumn:se(new At(105,"scrollBeyondLastColumn",4,0,1073741824,{description:m("scrollBeyondLastColumn","Controls the number of extra characters beyond which the editor will scroll horizontally.")})),scrollBeyondLastLine:se(new gt(106,"scrollBeyondLastLine",!0,{description:m("scrollBeyondLastLine","Controls whether the editor will scroll beyond the last line.")})),scrollPredominantAxis:se(new gt(107,"scrollPredominantAxis",!0,{description:m("scrollPredominantAxis","Scroll only along the predominant axis when scrolling both vertically and horizontally at the same time. Prevents horizontal drift when scrolling vertically on a trackpad.")})),selectionClipboard:se(new gt(108,"selectionClipboard",!0,{description:m("selectionClipboard","Controls whether the Linux primary clipboard should be supported."),included:ps})),selectionHighlight:se(new gt(109,"selectionHighlight",!0,{description:m("selectionHighlight","Controls whether the editor should highlight matches similar to the selection.")})),selectOnLineNumbers:se(new gt(110,"selectOnLineNumbers",!0)),showFoldingControls:se(new ui(111,"showFoldingControls","mouseover",["always","never","mouseover"],{enumDescriptions:[m("showFoldingControls.always","Always show the folding controls."),m("showFoldingControls.never","Never show the folding controls and reduce the gutter size."),m("showFoldingControls.mouseover","Only show the folding controls when the mouse is over the gutter.")],description:m("showFoldingControls","Controls when the folding controls on the gutter are shown.")})),showUnused:se(new gt(112,"showUnused",!0,{description:m("showUnused","Controls fading out of unused code.")})),showDeprecated:se(new gt(141,"showDeprecated",!0,{description:m("showDeprecated","Controls strikethrough deprecated variables.")})),inlayHints:se(new rJ),snippetSuggestions:se(new ui(113,"snippetSuggestions","inline",["top","bottom","inline","none"],{enumDescriptions:[m("snippetSuggestions.top","Show snippet suggestions on top of other suggestions."),m("snippetSuggestions.bottom","Show snippet suggestions below other suggestions."),m("snippetSuggestions.inline","Show snippets suggestions with other suggestions."),m("snippetSuggestions.none","Do not show snippet suggestions.")],description:m("snippetSuggestions","Controls whether snippets are shown with other suggestions and how they are sorted.")})),smartSelect:se(new kJ),smoothScrolling:se(new gt(115,"smoothScrolling",!1,{description:m("smoothScrolling","Controls whether the editor will scroll using an animation.")})),stopRenderingLineAfter:se(new At(118,"stopRenderingLineAfter",1e4,-1,1073741824)),suggest:se(new xJ),inlineSuggest:se(new wJ),inlineEdit:se(new SJ),inlineCompletionsAccessibilityVerbose:se(new gt(150,"inlineCompletionsAccessibilityVerbose",!1,{description:m("inlineCompletionsAccessibilityVerbose","Controls whether the accessibility hint should be provided to screen reader users when an inline completion is shown.")})),suggestFontSize:se(new At(120,"suggestFontSize",0,0,1e3,{markdownDescription:m("suggestFontSize","Font size for the suggest widget. When set to {0}, the value of {1} is used.","`0`","`#editor.fontSize#`")})),suggestLineHeight:se(new At(121,"suggestLineHeight",0,0,1e3,{markdownDescription:m("suggestLineHeight","Line height for the suggest widget. When set to {0}, the value of {1} is used. The minimum value is 8.","`0`","`#editor.lineHeight#`")})),suggestOnTriggerCharacters:se(new gt(122,"suggestOnTriggerCharacters",!0,{description:m("suggestOnTriggerCharacters","Controls whether suggestions should automatically show up when typing trigger characters.")})),suggestSelection:se(new ui(123,"suggestSelection","first",["first","recentlyUsed","recentlyUsedByPrefix"],{markdownEnumDescriptions:[m("suggestSelection.first","Always select the first suggestion."),m("suggestSelection.recentlyUsed","Select recent suggestions unless further typing selects one, e.g. `console.| -> console.log` because `log` has been completed recently."),m("suggestSelection.recentlyUsedByPrefix","Select suggestions based on previous prefixes that have completed those suggestions, e.g. `co -> console` and `con -> const`.")],description:m("suggestSelection","Controls how suggestions are pre-selected when showing the suggest list.")})),tabCompletion:se(new ui(124,"tabCompletion","off",["on","off","onlySnippets"],{enumDescriptions:[m("tabCompletion.on","Tab complete will insert the best matching suggestion when pressing tab."),m("tabCompletion.off","Disable tab completions."),m("tabCompletion.onlySnippets","Tab complete snippets when their prefix match. Works best when 'quickSuggestions' aren't enabled.")],description:m("tabCompletion","Enables tab completions.")})),tabIndex:se(new At(125,"tabIndex",0,-1,1073741824)),unicodeHighlight:se(new vJ),unusualLineTerminators:se(new ui(127,"unusualLineTerminators","prompt",["auto","off","prompt"],{enumDescriptions:[m("unusualLineTerminators.auto","Unusual line terminators are automatically removed."),m("unusualLineTerminators.off","Unusual line terminators are ignored."),m("unusualLineTerminators.prompt","Unusual line terminators prompt to be removed.")],description:m("unusualLineTerminators","Remove unusual line terminators that might cause problems.")})),useShadowDOM:se(new gt(128,"useShadowDOM",!0)),useTabStops:se(new gt(129,"useTabStops",!0,{description:m("useTabStops","Spaces and tabs are inserted and deleted in alignment with tab stops.")})),wordBreak:se(new ui(130,"wordBreak","normal",["normal","keepAll"],{markdownEnumDescriptions:[m("wordBreak.normal","Use the default line break rule."),m("wordBreak.keepAll","Word breaks should not be used for Chinese/Japanese/Korean (CJK) text. Non-CJK text behavior is the same as for normal.")],description:m("wordBreak","Controls the word break rules used for Chinese/Japanese/Korean (CJK) text.")})),wordSegmenterLocales:se(new DJ),wordSeparators:se(new Zn(132,"wordSeparators",Ny,{description:m("wordSeparators","Characters that will be used as word separators when doing word related navigations or operations.")})),wordWrap:se(new ui(133,"wordWrap","off",["off","on","wordWrapColumn","bounded"],{markdownEnumDescriptions:[m("wordWrap.off","Lines will never wrap."),m("wordWrap.on","Lines will wrap at the viewport width."),m({key:"wordWrap.wordWrapColumn",comment:["- `editor.wordWrapColumn` refers to a different setting and should not be localized."]},"Lines will wrap at `#editor.wordWrapColumn#`."),m({key:"wordWrap.bounded",comment:["- viewport means the edge of the visible window size.","- `editor.wordWrapColumn` refers to a different setting and should not be localized."]},"Lines will wrap at the minimum of viewport and `#editor.wordWrapColumn#`.")],description:m({key:"wordWrap",comment:["- 'off', 'on', 'wordWrapColumn' and 'bounded' refer to values the setting can take and should not be localized.","- `editor.wordWrapColumn` refers to a different setting and should not be localized."]},"Controls how lines should wrap.")})),wordWrapBreakAfterCharacters:se(new Zn(134,"wordWrapBreakAfterCharacters"," 	})]?|/&.,;¢°′″‰℃、。｡､￠，．：；？！％・･ゝゞヽヾーァィゥェォッャュョヮヵヶぁぃぅぇぉっゃゅょゎゕゖㇰㇱㇲㇳㇴㇵㇶㇷㇸㇹㇺㇻㇼㇽㇾㇿ々〻ｧｨｩｪｫｬｭｮｯｰ”〉》」』】〕）］｝｣")),wordWrapBreakBeforeCharacters:se(new Zn(135,"wordWrapBreakBeforeCharacters","([{‘“〈《「『【〔（［｛｢£¥＄￡￥+＋")),wordWrapColumn:se(new At(136,"wordWrapColumn",80,1,1073741824,{markdownDescription:m({key:"wordWrapColumn",comment:["- `editor.wordWrap` refers to a different setting and should not be localized.","- 'wordWrapColumn' and 'bounded' refer to values the different setting can take and should not be localized."]},"Controls the wrapping column of the editor when `#editor.wordWrap#` is `wordWrapColumn` or `bounded`.")})),wordWrapOverride1:se(new ui(137,"wordWrapOverride1","inherit",["off","on","inherit"])),wordWrapOverride2:se(new ui(138,"wordWrapOverride2","inherit",["off","on","inherit"])),editorClassName:se(new YQ),defaultColorDecorators:se(new gt(148,"defaultColorDecorators",!1,{markdownDescription:m("defaultColorDecorators","Controls whether inline color decorations should be shown using the default document color provider")})),pixelRatio:se(new gJ),tabFocusMode:se(new gt(145,"tabFocusMode",!1,{markdownDescription:m("tabFocusMode","Controls whether the editor receives tabs or defers them to the workbench for navigation.")})),layoutInfo:se(new Hp),wrappingInfo:se(new EJ),wrappingIndent:se(new IJ),wrappingStrategy:se(new nJ)};class PJ{constructor(){this.listeners=[],this.unexpectedErrorHandler=function(e){setTimeout(()=>{throw e.stack?L_.isErrorNoTelemetry(e)?new L_(e.message+`

`+e.stack):new Error(e.message+`

`+e.stack):e},0)}}emit(e){this.listeners.forEach(t=>{t(e)})}onUnexpectedError(e){this.unexpectedErrorHandler(e),this.emit(e)}onUnexpectedExternalError(e){this.unexpectedErrorHandler(e)}}const Oz=new PJ;function Oe(o){yr(o)||Oz.onUnexpectedError(o)}function Si(o){yr(o)||Oz.onUnexpectedExternalError(o)}function jB(o){if(o instanceof Error){const{name:e,message:t}=o,i=o.stacktrace||o.stack;return{$isError:!0,name:e,message:t,stack:i,noTelemetry:L_.isErrorNoTelemetry(o)}}return o}const Ry="Canceled";function yr(o){return o instanceof qr?!0:o instanceof Error&&o.name===Ry&&o.message===Ry}class qr extends Error{constructor(){super(Ry),this.name=this.message}}function OJ(){const o=new Error(Ry);return o.name=o.message,o}function gr(o){return o?new Error(`Illegal argument: ${o}`):new Error("Illegal argument")}function U4(o){return o?new Error(`Illegal state: ${o}`):new Error("Illegal state")}class FJ extends Error{constructor(e){super("NotSupported"),e&&(this.message=e)}}class L_ extends Error{constructor(e){super(e),this.name="CodeExpectedError"}static fromError(e){if(e instanceof L_)return e;const t=new L_;return t.message=e.message,t.stack=e.stack,t}static isErrorNoTelemetry(e){return e.name==="CodeExpectedError"}}class bt extends Error{constructor(e){super(e||"An unexpected bug occurred."),Object.setPrototypeOf(this,bt.prototype)}}function Ah(o,e){const t=this;let i=!1,n;return function(){return i||(i=!0,n=o.apply(t,arguments)),n}}function uI(o){return typeof o=="object"&&o!==null&&typeof o.dispose=="function"&&o.dispose.length===0}function Tt(o){if(ot.is(o)){const e=[];for(const t of o)if(t)try{t.dispose()}catch(i){e.push(i)}if(e.length===1)throw e[0];if(e.length>1)throw new AggregateError(e,"Encountered errors while disposing of store");return Array.isArray(o)?[]:o}else if(o)return o.dispose(),o}function jr(...o){return ve(()=>Tt(o))}function ve(o){return{dispose:Ah(()=>{o()})}}const Dk=class Dk{constructor(){this._toDispose=new Set,this._isDisposed=!1}dispose(){this._isDisposed||(this._isDisposed=!0,this.clear())}get isDisposed(){return this._isDisposed}clear(){if(this._toDispose.size!==0)try{Tt(this._toDispose)}finally{this._toDispose.clear()}}add(e){if(!e)return e;if(e===this)throw new Error("Cannot register a disposable on itself!");return this._isDisposed?Dk.DISABLE_DISPOSED_WARNING||console.warn(new Error("Trying to add a disposable to a DisposableStore that has already been disposed of. The added object will be leaked!").stack):this._toDispose.add(e),e}deleteAndLeak(e){e&&this._toDispose.has(e)&&this._toDispose.delete(e)}};Dk.DISABLE_DISPOSED_WARNING=!1;let K=Dk;const vB=class vB{constructor(){this._store=new K,this._store}dispose(){this._store.dispose()}_register(e){if(e===this)throw new Error("Cannot register a disposable on itself!");return this._store.add(e)}};vB.None=Object.freeze({dispose(){}});let F=vB;class Ji{constructor(){this._isDisposed=!1}get value(){return this._isDisposed?void 0:this._value}set value(e){var t;this._isDisposed||e===this._value||((t=this._value)==null||t.dispose(),this._value=e)}clear(){this.value=void 0}dispose(){var e;this._isDisposed=!0,(e=this._value)==null||e.dispose(),this._value=void 0}}class WJ{constructor(e){this._disposable=e,this._counter=1}acquire(){return this._counter++,this}release(){return--this._counter===0&&this._disposable.dispose(),this}}class BJ{constructor(e){this.object=e}dispose(){}}class $4{constructor(){this._store=new Map,this._isDisposed=!1}dispose(){this._isDisposed=!0,this.clearAndDisposeAll()}clearAndDisposeAll(){if(this._store.size)try{Tt(this._store.values())}finally{this._store.clear()}}get(e){return this._store.get(e)}set(e,t,i=!1){var n;this._isDisposed&&console.warn(new Error("Trying to add a disposable to a DisposableMap that has already been disposed of. The added object will be leaked!").stack),i||(n=this._store.get(e))==null||n.dispose(),this._store.set(e,t)}deleteAndDispose(e){var t;(t=this._store.get(e))==null||t.dispose(),this._store.delete(e)}[Symbol.iterator](){return this._store[Symbol.iterator]()}}const HJ=globalThis.performance&&typeof globalThis.performance.now=="function";class ln{static create(e){return new ln(e)}constructor(e){this._now=HJ&&e===!1?Date.now:globalThis.performance.now.bind(globalThis.performance),this._startTime=this._now(),this._stopTime=-1}stop(){this._stopTime=this._now()}reset(){this._startTime=this._now(),this._stopTime=-1}elapsed(){return this._stopTime!==-1?this._stopTime-this._startTime:this._now()-this._startTime}}var oe;(function(o){o.None=()=>F.None;function e($,H){return u($,()=>{},0,void 0,!0,void 0,H)}o.defer=e;function t($){return(H,j=null,J)=>{let he=!1,fe;return fe=$(ue=>{if(!he)return fe?fe.dispose():he=!0,H.call(j,ue)},null,J),he&&fe.dispose(),fe}}o.once=t;function i($,H){return o.once(o.filter($,H))}o.onceIf=i;function n($,H,j){return d((J,he=null,fe)=>$(ue=>J.call(he,H(ue)),null,fe),j)}o.map=n;function s($,H,j){return d((J,he=null,fe)=>$(ue=>{H(ue),J.call(he,ue)},null,fe),j)}o.forEach=s;function r($,H,j){return d((J,he=null,fe)=>$(ue=>H(ue)&&J.call(he,ue),null,fe),j)}o.filter=r;function a($){return $}o.signal=a;function l(...$){return(H,j=null,J)=>{const he=jr(...$.map(fe=>fe(ue=>H.call(j,ue))));return h(he,J)}}o.any=l;function c($,H,j,J){let he=j;return n($,fe=>(he=H(he,fe),he),J)}o.reduce=c;function d($,H){let j;const J={onWillAddFirstListener(){j=$(he.fire,he)},onDidRemoveLastListener(){j==null||j.dispose()}},he=new P(J);return H==null||H.add(he),he.event}function h($,H){return H instanceof Array?H.push($):H&&H.add($),$}function u($,H,j=100,J=!1,he=!1,fe,ue){let Pe,_e,it,Ne=0,Ct;const ht={leakWarningThreshold:fe,onWillAddFirstListener(){Pe=$(ts=>{Ne++,_e=H(_e,ts),J&&!it&&(Ti.fire(_e),_e=void 0),Ct=()=>{const xs=_e;_e=void 0,it=void 0,(!J||Ne>1)&&Ti.fire(xs),Ne=0},typeof j=="number"?(clearTimeout(it),it=setTimeout(Ct,j)):it===void 0&&(it=0,queueMicrotask(Ct))})},onWillRemoveListener(){he&&Ne>0&&(Ct==null||Ct())},onDidRemoveLastListener(){Ct=void 0,Pe.dispose()}},Ti=new P(ht);return ue==null||ue.add(Ti),Ti.event}o.debounce=u;function g($,H=0,j){return o.debounce($,(J,he)=>J?(J.push(he),J):[he],H,void 0,!0,void 0,j)}o.accumulate=g;function f($,H=(J,he)=>J===he,j){let J=!0,he;return r($,fe=>{const ue=J||!H(fe,he);return J=!1,he=fe,ue},j)}o.latch=f;function p($,H,j){return[o.filter($,H,j),o.filter($,J=>!H(J),j)]}o.split=p;function _($,H=!1,j=[],J){let he=j.slice(),fe=$(_e=>{he?he.push(_e):Pe.fire(_e)});J&&J.add(fe);const ue=()=>{he==null||he.forEach(_e=>Pe.fire(_e)),he=null},Pe=new P({onWillAddFirstListener(){fe||(fe=$(_e=>Pe.fire(_e)),J&&J.add(fe))},onDidAddFirstListener(){he&&(H?setTimeout(ue):ue())},onDidRemoveLastListener(){fe&&fe.dispose(),fe=null}});return J&&J.add(Pe),Pe.event}o.buffer=_;function b($,H){return(J,he,fe)=>{const ue=H(new v);return $(function(Pe){const _e=ue.evaluate(Pe);_e!==C&&J.call(he,_e)},void 0,fe)}}o.chain=b;const C=Symbol("HaltChainable");class v{constructor(){this.steps=[]}map(H){return this.steps.push(H),this}forEach(H){return this.steps.push(j=>(H(j),j)),this}filter(H){return this.steps.push(j=>H(j)?j:C),this}reduce(H,j){let J=j;return this.steps.push(he=>(J=H(J,he),J)),this}latch(H=(j,J)=>j===J){let j=!0,J;return this.steps.push(he=>{const fe=j||!H(he,J);return j=!1,J=he,fe?he:C}),this}evaluate(H){for(const j of this.steps)if(H=j(H),H===C)break;return H}}function w($,H,j=J=>J){const J=(...Pe)=>ue.fire(j(...Pe)),he=()=>$.on(H,J),fe=()=>$.removeListener(H,J),ue=new P({onWillAddFirstListener:he,onDidRemoveLastListener:fe});return ue.event}o.fromNodeEventEmitter=w;function S($,H,j=J=>J){const J=(...Pe)=>ue.fire(j(...Pe)),he=()=>$.addEventListener(H,J),fe=()=>$.removeEventListener(H,J),ue=new P({onWillAddFirstListener:he,onDidRemoveLastListener:fe});return ue.event}o.fromDOMEventEmitter=S;function L($){return new Promise(H=>t($)(H))}o.toPromise=L;function k($){const H=new P;return $.then(j=>{H.fire(j)},()=>{H.fire(void 0)}).finally(()=>{H.dispose()}),H.event}o.fromPromise=k;function D($,H){return $(j=>H.fire(j))}o.forward=D;function N($,H,j){return H(j),$(J=>H(J))}o.runAndSubscribe=N;class B{constructor(H,j){this._observable=H,this._counter=0,this._hasChanged=!1;const J={onWillAddFirstListener:()=>{H.addObserver(this),this._observable.reportChanges()},onDidRemoveLastListener:()=>{H.removeObserver(this)}};this.emitter=new P(J),j&&j.add(this.emitter)}beginUpdate(H){this._counter++}handlePossibleChange(H){}handleChange(H,j){this._hasChanged=!0}endUpdate(H){this._counter--,this._counter===0&&(this._observable.reportChanges(),this._hasChanged&&(this._hasChanged=!1,this.emitter.fire(this._observable.get())))}}function W($,H){return new B($,H).emitter.event}o.fromObservable=W;function V($){return(H,j,J)=>{let he=0,fe=!1;const ue={beginUpdate(){he++},endUpdate(){he--,he===0&&($.reportChanges(),fe&&(fe=!1,H.call(j)))},handlePossibleChange(){},handleChange(){fe=!0}};$.addObserver(ue),$.reportChanges();const Pe={dispose(){$.removeObserver(ue)}};return J instanceof K?J.add(Pe):Array.isArray(J)&&J.push(Pe),Pe}}o.fromObservableLight=V})(oe||(oe={}));const Lp=class Lp{constructor(e){this.listenerCount=0,this.invocationCount=0,this.elapsedOverall=0,this.durations=[],this.name=`${e}_${Lp._idPool++}`,Lp.all.add(this)}start(e){this._stopWatch=new ln,this.listenerCount=e}stop(){if(this._stopWatch){const e=this._stopWatch.elapsed();this.durations.push(e),this.elapsedOverall+=e,this.invocationCount+=1,this._stopWatch=void 0}}};Lp.all=new Set,Lp._idPool=0;let mR=Lp,VJ=-1;const Ik=class Ik{constructor(e,t,i=(Ik._idPool++).toString(16).padStart(3,"0")){this._errorHandler=e,this.threshold=t,this.name=i,this._warnCountdown=0}dispose(){var e;(e=this._stacks)==null||e.clear()}check(e,t){const i=this.threshold;if(i<=0||t<i)return;this._stacks||(this._stacks=new Map);const n=this._stacks.get(e.value)||0;if(this._stacks.set(e.value,n+1),this._warnCountdown-=1,this._warnCountdown<=0){this._warnCountdown=i*.5;const[s,r]=this.getMostFrequentStack(),a=`[${this.name}] potential listener LEAK detected, having ${t} listeners already. MOST frequent listener (${r}):`;console.warn(a),console.warn(s);const l=new zJ(a,s);this._errorHandler(l)}return()=>{const s=this._stacks.get(e.value)||0;this._stacks.set(e.value,s-1)}}getMostFrequentStack(){if(!this._stacks)return;let e,t=0;for(const[i,n]of this._stacks)(!e||t<n)&&(e=[i,n],t=n);return e}};Ik._idPool=1;let pR=Ik;class j4{static create(){const e=new Error;return new j4(e.stack??"")}constructor(e){this.value=e}print(){console.warn(this.value.split(`
`).slice(2).join(`
`))}}class zJ extends Error{constructor(e,t){super(e),this.name="ListenerLeakError",this.stack=t}}class UJ extends Error{constructor(e,t){super(e),this.name="ListenerRefusalError",this.stack=t}}class nN{constructor(e){this.value=e}}const $J=2;let P=class{constructor(e){var t,i,n,s;this._size=0,this._options=e,this._leakageMon=(t=this._options)!=null&&t.leakWarningThreshold?new pR((e==null?void 0:e.onListenerError)??Oe,((i=this._options)==null?void 0:i.leakWarningThreshold)??VJ):void 0,this._perfMon=(n=this._options)!=null&&n._profName?new mR(this._options._profName):void 0,this._deliveryQueue=(s=this._options)==null?void 0:s.deliveryQueue}dispose(){var e,t,i,n;this._disposed||(this._disposed=!0,((e=this._deliveryQueue)==null?void 0:e.current)===this&&this._deliveryQueue.reset(),this._listeners&&(this._listeners=void 0,this._size=0),(i=(t=this._options)==null?void 0:t.onDidRemoveLastListener)==null||i.call(t),(n=this._leakageMon)==null||n.dispose())}get event(){return this._event??(this._event=(e,t,i)=>{var a,l,c,d,h;if(this._leakageMon&&this._size>this._leakageMon.threshold**2){const u=`[${this._leakageMon.name}] REFUSES to accept new listeners because it exceeded its threshold by far (${this._size} vs ${this._leakageMon.threshold})`;console.warn(u);const g=this._leakageMon.getMostFrequentStack()??["UNKNOWN stack",-1],f=new UJ(`${u}. HINT: Stack shows most frequent listener (${g[1]}-times)`,g[0]);return(((a=this._options)==null?void 0:a.onListenerError)||Oe)(f),F.None}if(this._disposed)return F.None;t&&(e=e.bind(t));const n=new nN(e);let s;this._leakageMon&&this._size>=Math.ceil(this._leakageMon.threshold*.2)&&(n.stack=j4.create(),s=this._leakageMon.check(n.stack,this._size+1)),this._listeners?this._listeners instanceof nN?(this._deliveryQueue??(this._deliveryQueue=new Fz),this._listeners=[this._listeners,n]):this._listeners.push(n):((c=(l=this._options)==null?void 0:l.onWillAddFirstListener)==null||c.call(l,this),this._listeners=n,(h=(d=this._options)==null?void 0:d.onDidAddFirstListener)==null||h.call(d,this)),this._size++;const r=ve(()=>{s==null||s(),this._removeListener(n)});return i instanceof K?i.add(r):Array.isArray(i)&&i.push(r),r}),this._event}_removeListener(e){var s,r,a,l;if((r=(s=this._options)==null?void 0:s.onWillRemoveListener)==null||r.call(s,this),!this._listeners)return;if(this._size===1){this._listeners=void 0,(l=(a=this._options)==null?void 0:a.onDidRemoveLastListener)==null||l.call(a,this),this._size=0;return}const t=this._listeners,i=t.indexOf(e);if(i===-1)throw console.log("disposed?",this._disposed),console.log("size?",this._size),console.log("arr?",JSON.stringify(this._listeners)),new Error("Attempted to dispose unknown listener");this._size--,t[i]=void 0;const n=this._deliveryQueue.current===this;if(this._size*$J<=t.length){let c=0;for(let d=0;d<t.length;d++)t[d]?t[c++]=t[d]:n&&(this._deliveryQueue.end--,c<this._deliveryQueue.i&&this._deliveryQueue.i--);t.length=c}}_deliver(e,t){var n;if(!e)return;const i=((n=this._options)==null?void 0:n.onListenerError)||Oe;if(!i){e.value(t);return}try{e.value(t)}catch(s){i(s)}}_deliverQueue(e){const t=e.current._listeners;for(;e.i<e.end;)this._deliver(t[e.i++],e.value);e.reset()}fire(e){var t,i,n,s;if((t=this._deliveryQueue)!=null&&t.current&&(this._deliverQueue(this._deliveryQueue),(i=this._perfMon)==null||i.stop()),(n=this._perfMon)==null||n.start(this._size),this._listeners)if(this._listeners instanceof nN)this._deliver(this._listeners,e);else{const r=this._deliveryQueue;r.enqueue(this,e,this._listeners.length),this._deliverQueue(r)}(s=this._perfMon)==null||s.stop()}hasListeners(){return this._size>0}};const jJ=()=>new Fz;class Fz{constructor(){this.i=-1,this.end=0}enqueue(e,t,i){this.i=0,this.end=i,this.current=e,this.value=t}reset(){this.i=this.end,this.current=void 0,this.value=void 0}}class Lf extends P{constructor(e){super(e),this._isPaused=0,this._eventQueue=new cs,this._mergeFn=e==null?void 0:e.merge}pause(){this._isPaused++}resume(){if(this._isPaused!==0&&--this._isPaused===0)if(this._mergeFn){if(this._eventQueue.size>0){const e=Array.from(this._eventQueue);this._eventQueue.clear(),super.fire(this._mergeFn(e))}}else for(;!this._isPaused&&this._eventQueue.size!==0;)super.fire(this._eventQueue.shift())}fire(e){this._size&&(this._isPaused!==0?this._eventQueue.push(e):super.fire(e))}}class Wz extends Lf{constructor(e){super(e),this._delay=e.delay??100}fire(e){this._handle||(this.pause(),this._handle=setTimeout(()=>{this._handle=void 0,this.resume()},this._delay)),super.fire(e)}}class KJ extends P{constructor(e){super(e),this._queuedEvents=[],this._mergeFn=e==null?void 0:e.merge}fire(e){this.hasListeners()&&(this._queuedEvents.push(e),this._queuedEvents.length===1&&queueMicrotask(()=>{this._mergeFn?super.fire(this._mergeFn(this._queuedEvents)):this._queuedEvents.forEach(t=>super.fire(t)),this._queuedEvents=[]}))}}class qJ{constructor(){this.hasListeners=!1,this.events=[],this.emitter=new P({onWillAddFirstListener:()=>this.onFirstListenerAdd(),onDidRemoveLastListener:()=>this.onLastListenerRemove()})}get event(){return this.emitter.event}add(e){const t={event:e,listener:null};return this.events.push(t),this.hasListeners&&this.hook(t),ve(Ah(()=>{this.hasListeners&&this.unhook(t);const n=this.events.indexOf(t);this.events.splice(n,1)}))}onFirstListenerAdd(){this.hasListeners=!0,this.events.forEach(e=>this.hook(e))}onLastListenerRemove(){this.hasListeners=!1,this.events.forEach(e=>this.unhook(e))}hook(e){e.listener=e.event(t=>this.emitter.fire(t))}unhook(e){var t;(t=e.listener)==null||t.dispose(),e.listener=null}dispose(){var e;this.emitter.dispose();for(const t of this.events)(e=t.listener)==null||e.dispose();this.events=[]}}class R1{constructor(){this.data=[]}wrapEvent(e,t,i){return(n,s,r)=>e(a=>{const l=this.data[this.data.length-1];if(!t){l?l.buffers.push(()=>n.call(s,a)):n.call(s,a);return}const c=l;if(!c){n.call(s,t(i,a));return}c.items??(c.items=[]),c.items.push(a),c.buffers.length===0&&l.buffers.push(()=>{c.reducedResult??(c.reducedResult=i?c.items.reduce(t,i):c.items.reduce(t)),n.call(s,c.reducedResult)})},void 0,r)}bufferEvents(e){const t={buffers:new Array};this.data.push(t);const i=e();return this.data.pop(),t.buffers.forEach(n=>n()),i}}class KB{constructor(){this.listening=!1,this.inputEvent=oe.None,this.inputEventListener=F.None,this.emitter=new P({onDidAddFirstListener:()=>{this.listening=!0,this.inputEventListener=this.inputEvent(this.emitter.fire,this.emitter)},onDidRemoveLastListener:()=>{this.listening=!1,this.inputEventListener.dispose()}}),this.event=this.emitter.event}set input(e){this.inputEvent=e,this.listening&&(this.inputEventListener.dispose(),this.inputEventListener=e(this.emitter.fire,this.emitter))}dispose(){this.inputEventListener.dispose(),this.emitter.dispose()}}const Bz=Object.freeze(function(o,e){const t=setTimeout(o.bind(e),0);return{dispose(){clearTimeout(t)}}});var ze;(function(o){function e(t){return t===o.None||t===o.Cancelled||t instanceof B0?!0:!t||typeof t!="object"?!1:typeof t.isCancellationRequested=="boolean"&&typeof t.onCancellationRequested=="function"}o.isCancellationToken=e,o.None=Object.freeze({isCancellationRequested:!1,onCancellationRequested:oe.None}),o.Cancelled=Object.freeze({isCancellationRequested:!0,onCancellationRequested:Bz})})(ze||(ze={}));class B0{constructor(){this._isCancelled=!1,this._emitter=null}cancel(){this._isCancelled||(this._isCancelled=!0,this._emitter&&(this._emitter.fire(void 0),this.dispose()))}get isCancellationRequested(){return this._isCancelled}get onCancellationRequested(){return this._isCancelled?Bz:(this._emitter||(this._emitter=new P),this._emitter.event)}dispose(){this._emitter&&(this._emitter.dispose(),this._emitter=null)}}let li=class{constructor(e){this._token=void 0,this._parentListener=void 0,this._parentListener=e&&e.onCancellationRequested(this.cancel,this)}get token(){return this._token||(this._token=new B0),this._token}cancel(){this._token?this._token instanceof B0&&this._token.cancel():this._token=ze.Cancelled}dispose(e=!1){var t;e&&this.cancel(),(t=this._parentListener)==null||t.dispose(),this._token?this._token instanceof B0&&this._token.dispose():this._token=ze.None}};function _R(o){const e=new li;return o.add({dispose(){e.cancel()}}),e.token}class K4{constructor(){this._keyCodeToStr=[],this._strToKeyCode=Object.create(null)}define(e,t){this._keyCodeToStr[e]=t,this._strToKeyCode[t.toLowerCase()]=e}keyCodeToStr(e){return this._keyCodeToStr[e]}strToKeyCode(e){return this._strToKeyCode[e.toLowerCase()]||0}}const H0=new K4,bR=new K4,CR=new K4,Hz=new Array(230),GJ=Object.create(null),ZJ=Object.create(null),q4=[];for(let o=0;o<=193;o++)q4[o]=-1;(function(){const o="",e=[[1,0,"None",0,"unknown",0,"VK_UNKNOWN",o,o],[1,1,"Hyper",0,o,0,o,o,o],[1,2,"Super",0,o,0,o,o,o],[1,3,"Fn",0,o,0,o,o,o],[1,4,"FnLock",0,o,0,o,o,o],[1,5,"Suspend",0,o,0,o,o,o],[1,6,"Resume",0,o,0,o,o,o],[1,7,"Turbo",0,o,0,o,o,o],[1,8,"Sleep",0,o,0,"VK_SLEEP",o,o],[1,9,"WakeUp",0,o,0,o,o,o],[0,10,"KeyA",31,"A",65,"VK_A",o,o],[0,11,"KeyB",32,"B",66,"VK_B",o,o],[0,12,"KeyC",33,"C",67,"VK_C",o,o],[0,13,"KeyD",34,"D",68,"VK_D",o,o],[0,14,"KeyE",35,"E",69,"VK_E",o,o],[0,15,"KeyF",36,"F",70,"VK_F",o,o],[0,16,"KeyG",37,"G",71,"VK_G",o,o],[0,17,"KeyH",38,"H",72,"VK_H",o,o],[0,18,"KeyI",39,"I",73,"VK_I",o,o],[0,19,"KeyJ",40,"J",74,"VK_J",o,o],[0,20,"KeyK",41,"K",75,"VK_K",o,o],[0,21,"KeyL",42,"L",76,"VK_L",o,o],[0,22,"KeyM",43,"M",77,"VK_M",o,o],[0,23,"KeyN",44,"N",78,"VK_N",o,o],[0,24,"KeyO",45,"O",79,"VK_O",o,o],[0,25,"KeyP",46,"P",80,"VK_P",o,o],[0,26,"KeyQ",47,"Q",81,"VK_Q",o,o],[0,27,"KeyR",48,"R",82,"VK_R",o,o],[0,28,"KeyS",49,"S",83,"VK_S",o,o],[0,29,"KeyT",50,"T",84,"VK_T",o,o],[0,30,"KeyU",51,"U",85,"VK_U",o,o],[0,31,"KeyV",52,"V",86,"VK_V",o,o],[0,32,"KeyW",53,"W",87,"VK_W",o,o],[0,33,"KeyX",54,"X",88,"VK_X",o,o],[0,34,"KeyY",55,"Y",89,"VK_Y",o,o],[0,35,"KeyZ",56,"Z",90,"VK_Z",o,o],[0,36,"Digit1",22,"1",49,"VK_1",o,o],[0,37,"Digit2",23,"2",50,"VK_2",o,o],[0,38,"Digit3",24,"3",51,"VK_3",o,o],[0,39,"Digit4",25,"4",52,"VK_4",o,o],[0,40,"Digit5",26,"5",53,"VK_5",o,o],[0,41,"Digit6",27,"6",54,"VK_6",o,o],[0,42,"Digit7",28,"7",55,"VK_7",o,o],[0,43,"Digit8",29,"8",56,"VK_8",o,o],[0,44,"Digit9",30,"9",57,"VK_9",o,o],[0,45,"Digit0",21,"0",48,"VK_0",o,o],[1,46,"Enter",3,"Enter",13,"VK_RETURN",o,o],[1,47,"Escape",9,"Escape",27,"VK_ESCAPE",o,o],[1,48,"Backspace",1,"Backspace",8,"VK_BACK",o,o],[1,49,"Tab",2,"Tab",9,"VK_TAB",o,o],[1,50,"Space",10,"Space",32,"VK_SPACE",o,o],[0,51,"Minus",88,"-",189,"VK_OEM_MINUS","-","OEM_MINUS"],[0,52,"Equal",86,"=",187,"VK_OEM_PLUS","=","OEM_PLUS"],[0,53,"BracketLeft",92,"[",219,"VK_OEM_4","[","OEM_4"],[0,54,"BracketRight",94,"]",221,"VK_OEM_6","]","OEM_6"],[0,55,"Backslash",93,"\\",220,"VK_OEM_5","\\","OEM_5"],[0,56,"IntlHash",0,o,0,o,o,o],[0,57,"Semicolon",85,";",186,"VK_OEM_1",";","OEM_1"],[0,58,"Quote",95,"'",222,"VK_OEM_7","'","OEM_7"],[0,59,"Backquote",91,"`",192,"VK_OEM_3","`","OEM_3"],[0,60,"Comma",87,",",188,"VK_OEM_COMMA",",","OEM_COMMA"],[0,61,"Period",89,".",190,"VK_OEM_PERIOD",".","OEM_PERIOD"],[0,62,"Slash",90,"/",191,"VK_OEM_2","/","OEM_2"],[1,63,"CapsLock",8,"CapsLock",20,"VK_CAPITAL",o,o],[1,64,"F1",59,"F1",112,"VK_F1",o,o],[1,65,"F2",60,"F2",113,"VK_F2",o,o],[1,66,"F3",61,"F3",114,"VK_F3",o,o],[1,67,"F4",62,"F4",115,"VK_F4",o,o],[1,68,"F5",63,"F5",116,"VK_F5",o,o],[1,69,"F6",64,"F6",117,"VK_F6",o,o],[1,70,"F7",65,"F7",118,"VK_F7",o,o],[1,71,"F8",66,"F8",119,"VK_F8",o,o],[1,72,"F9",67,"F9",120,"VK_F9",o,o],[1,73,"F10",68,"F10",121,"VK_F10",o,o],[1,74,"F11",69,"F11",122,"VK_F11",o,o],[1,75,"F12",70,"F12",123,"VK_F12",o,o],[1,76,"PrintScreen",0,o,0,o,o,o],[1,77,"ScrollLock",84,"ScrollLock",145,"VK_SCROLL",o,o],[1,78,"Pause",7,"PauseBreak",19,"VK_PAUSE",o,o],[1,79,"Insert",19,"Insert",45,"VK_INSERT",o,o],[1,80,"Home",14,"Home",36,"VK_HOME",o,o],[1,81,"PageUp",11,"PageUp",33,"VK_PRIOR",o,o],[1,82,"Delete",20,"Delete",46,"VK_DELETE",o,o],[1,83,"End",13,"End",35,"VK_END",o,o],[1,84,"PageDown",12,"PageDown",34,"VK_NEXT",o,o],[1,85,"ArrowRight",17,"RightArrow",39,"VK_RIGHT","Right",o],[1,86,"ArrowLeft",15,"LeftArrow",37,"VK_LEFT","Left",o],[1,87,"ArrowDown",18,"DownArrow",40,"VK_DOWN","Down",o],[1,88,"ArrowUp",16,"UpArrow",38,"VK_UP","Up",o],[1,89,"NumLock",83,"NumLock",144,"VK_NUMLOCK",o,o],[1,90,"NumpadDivide",113,"NumPad_Divide",111,"VK_DIVIDE",o,o],[1,91,"NumpadMultiply",108,"NumPad_Multiply",106,"VK_MULTIPLY",o,o],[1,92,"NumpadSubtract",111,"NumPad_Subtract",109,"VK_SUBTRACT",o,o],[1,93,"NumpadAdd",109,"NumPad_Add",107,"VK_ADD",o,o],[1,94,"NumpadEnter",3,o,0,o,o,o],[1,95,"Numpad1",99,"NumPad1",97,"VK_NUMPAD1",o,o],[1,96,"Numpad2",100,"NumPad2",98,"VK_NUMPAD2",o,o],[1,97,"Numpad3",101,"NumPad3",99,"VK_NUMPAD3",o,o],[1,98,"Numpad4",102,"NumPad4",100,"VK_NUMPAD4",o,o],[1,99,"Numpad5",103,"NumPad5",101,"VK_NUMPAD5",o,o],[1,100,"Numpad6",104,"NumPad6",102,"VK_NUMPAD6",o,o],[1,101,"Numpad7",105,"NumPad7",103,"VK_NUMPAD7",o,o],[1,102,"Numpad8",106,"NumPad8",104,"VK_NUMPAD8",o,o],[1,103,"Numpad9",107,"NumPad9",105,"VK_NUMPAD9",o,o],[1,104,"Numpad0",98,"NumPad0",96,"VK_NUMPAD0",o,o],[1,105,"NumpadDecimal",112,"NumPad_Decimal",110,"VK_DECIMAL",o,o],[0,106,"IntlBackslash",97,"OEM_102",226,"VK_OEM_102",o,o],[1,107,"ContextMenu",58,"ContextMenu",93,o,o,o],[1,108,"Power",0,o,0,o,o,o],[1,109,"NumpadEqual",0,o,0,o,o,o],[1,110,"F13",71,"F13",124,"VK_F13",o,o],[1,111,"F14",72,"F14",125,"VK_F14",o,o],[1,112,"F15",73,"F15",126,"VK_F15",o,o],[1,113,"F16",74,"F16",127,"VK_F16",o,o],[1,114,"F17",75,"F17",128,"VK_F17",o,o],[1,115,"F18",76,"F18",129,"VK_F18",o,o],[1,116,"F19",77,"F19",130,"VK_F19",o,o],[1,117,"F20",78,"F20",131,"VK_F20",o,o],[1,118,"F21",79,"F21",132,"VK_F21",o,o],[1,119,"F22",80,"F22",133,"VK_F22",o,o],[1,120,"F23",81,"F23",134,"VK_F23",o,o],[1,121,"F24",82,"F24",135,"VK_F24",o,o],[1,122,"Open",0,o,0,o,o,o],[1,123,"Help",0,o,0,o,o,o],[1,124,"Select",0,o,0,o,o,o],[1,125,"Again",0,o,0,o,o,o],[1,126,"Undo",0,o,0,o,o,o],[1,127,"Cut",0,o,0,o,o,o],[1,128,"Copy",0,o,0,o,o,o],[1,129,"Paste",0,o,0,o,o,o],[1,130,"Find",0,o,0,o,o,o],[1,131,"AudioVolumeMute",117,"AudioVolumeMute",173,"VK_VOLUME_MUTE",o,o],[1,132,"AudioVolumeUp",118,"AudioVolumeUp",175,"VK_VOLUME_UP",o,o],[1,133,"AudioVolumeDown",119,"AudioVolumeDown",174,"VK_VOLUME_DOWN",o,o],[1,134,"NumpadComma",110,"NumPad_Separator",108,"VK_SEPARATOR",o,o],[0,135,"IntlRo",115,"ABNT_C1",193,"VK_ABNT_C1",o,o],[1,136,"KanaMode",0,o,0,o,o,o],[0,137,"IntlYen",0,o,0,o,o,o],[1,138,"Convert",0,o,0,o,o,o],[1,139,"NonConvert",0,o,0,o,o,o],[1,140,"Lang1",0,o,0,o,o,o],[1,141,"Lang2",0,o,0,o,o,o],[1,142,"Lang3",0,o,0,o,o,o],[1,143,"Lang4",0,o,0,o,o,o],[1,144,"Lang5",0,o,0,o,o,o],[1,145,"Abort",0,o,0,o,o,o],[1,146,"Props",0,o,0,o,o,o],[1,147,"NumpadParenLeft",0,o,0,o,o,o],[1,148,"NumpadParenRight",0,o,0,o,o,o],[1,149,"NumpadBackspace",0,o,0,o,o,o],[1,150,"NumpadMemoryStore",0,o,0,o,o,o],[1,151,"NumpadMemoryRecall",0,o,0,o,o,o],[1,152,"NumpadMemoryClear",0,o,0,o,o,o],[1,153,"NumpadMemoryAdd",0,o,0,o,o,o],[1,154,"NumpadMemorySubtract",0,o,0,o,o,o],[1,155,"NumpadClear",131,"Clear",12,"VK_CLEAR",o,o],[1,156,"NumpadClearEntry",0,o,0,o,o,o],[1,0,o,5,"Ctrl",17,"VK_CONTROL",o,o],[1,0,o,4,"Shift",16,"VK_SHIFT",o,o],[1,0,o,6,"Alt",18,"VK_MENU",o,o],[1,0,o,57,"Meta",91,"VK_COMMAND",o,o],[1,157,"ControlLeft",5,o,0,"VK_LCONTROL",o,o],[1,158,"ShiftLeft",4,o,0,"VK_LSHIFT",o,o],[1,159,"AltLeft",6,o,0,"VK_LMENU",o,o],[1,160,"MetaLeft",57,o,0,"VK_LWIN",o,o],[1,161,"ControlRight",5,o,0,"VK_RCONTROL",o,o],[1,162,"ShiftRight",4,o,0,"VK_RSHIFT",o,o],[1,163,"AltRight",6,o,0,"VK_RMENU",o,o],[1,164,"MetaRight",57,o,0,"VK_RWIN",o,o],[1,165,"BrightnessUp",0,o,0,o,o,o],[1,166,"BrightnessDown",0,o,0,o,o,o],[1,167,"MediaPlay",0,o,0,o,o,o],[1,168,"MediaRecord",0,o,0,o,o,o],[1,169,"MediaFastForward",0,o,0,o,o,o],[1,170,"MediaRewind",0,o,0,o,o,o],[1,171,"MediaTrackNext",124,"MediaTrackNext",176,"VK_MEDIA_NEXT_TRACK",o,o],[1,172,"MediaTrackPrevious",125,"MediaTrackPrevious",177,"VK_MEDIA_PREV_TRACK",o,o],[1,173,"MediaStop",126,"MediaStop",178,"VK_MEDIA_STOP",o,o],[1,174,"Eject",0,o,0,o,o,o],[1,175,"MediaPlayPause",127,"MediaPlayPause",179,"VK_MEDIA_PLAY_PAUSE",o,o],[1,176,"MediaSelect",128,"LaunchMediaPlayer",181,"VK_MEDIA_LAUNCH_MEDIA_SELECT",o,o],[1,177,"LaunchMail",129,"LaunchMail",180,"VK_MEDIA_LAUNCH_MAIL",o,o],[1,178,"LaunchApp2",130,"LaunchApp2",183,"VK_MEDIA_LAUNCH_APP2",o,o],[1,179,"LaunchApp1",0,o,0,"VK_MEDIA_LAUNCH_APP1",o,o],[1,180,"SelectTask",0,o,0,o,o,o],[1,181,"LaunchScreenSaver",0,o,0,o,o,o],[1,182,"BrowserSearch",120,"BrowserSearch",170,"VK_BROWSER_SEARCH",o,o],[1,183,"BrowserHome",121,"BrowserHome",172,"VK_BROWSER_HOME",o,o],[1,184,"BrowserBack",122,"BrowserBack",166,"VK_BROWSER_BACK",o,o],[1,185,"BrowserForward",123,"BrowserForward",167,"VK_BROWSER_FORWARD",o,o],[1,186,"BrowserStop",0,o,0,"VK_BROWSER_STOP",o,o],[1,187,"BrowserRefresh",0,o,0,"VK_BROWSER_REFRESH",o,o],[1,188,"BrowserFavorites",0,o,0,"VK_BROWSER_FAVORITES",o,o],[1,189,"ZoomToggle",0,o,0,o,o,o],[1,190,"MailReply",0,o,0,o,o,o],[1,191,"MailForward",0,o,0,o,o,o],[1,192,"MailSend",0,o,0,o,o,o],[1,0,o,114,"KeyInComposition",229,o,o,o],[1,0,o,116,"ABNT_C2",194,"VK_ABNT_C2",o,o],[1,0,o,96,"OEM_8",223,"VK_OEM_8",o,o],[1,0,o,0,o,0,"VK_KANA",o,o],[1,0,o,0,o,0,"VK_HANGUL",o,o],[1,0,o,0,o,0,"VK_JUNJA",o,o],[1,0,o,0,o,0,"VK_FINAL",o,o],[1,0,o,0,o,0,"VK_HANJA",o,o],[1,0,o,0,o,0,"VK_KANJI",o,o],[1,0,o,0,o,0,"VK_CONVERT",o,o],[1,0,o,0,o,0,"VK_NONCONVERT",o,o],[1,0,o,0,o,0,"VK_ACCEPT",o,o],[1,0,o,0,o,0,"VK_MODECHANGE",o,o],[1,0,o,0,o,0,"VK_SELECT",o,o],[1,0,o,0,o,0,"VK_PRINT",o,o],[1,0,o,0,o,0,"VK_EXECUTE",o,o],[1,0,o,0,o,0,"VK_SNAPSHOT",o,o],[1,0,o,0,o,0,"VK_HELP",o,o],[1,0,o,0,o,0,"VK_APPS",o,o],[1,0,o,0,o,0,"VK_PROCESSKEY",o,o],[1,0,o,0,o,0,"VK_PACKET",o,o],[1,0,o,0,o,0,"VK_DBE_SBCSCHAR",o,o],[1,0,o,0,o,0,"VK_DBE_DBCSCHAR",o,o],[1,0,o,0,o,0,"VK_ATTN",o,o],[1,0,o,0,o,0,"VK_CRSEL",o,o],[1,0,o,0,o,0,"VK_EXSEL",o,o],[1,0,o,0,o,0,"VK_EREOF",o,o],[1,0,o,0,o,0,"VK_PLAY",o,o],[1,0,o,0,o,0,"VK_ZOOM",o,o],[1,0,o,0,o,0,"VK_NONAME",o,o],[1,0,o,0,o,0,"VK_PA1",o,o],[1,0,o,0,o,0,"VK_OEM_CLEAR",o,o]],t=[],i=[];for(const n of e){const[s,r,a,l,c,d,h,u,g]=n;if(i[r]||(i[r]=!0,GJ[a]=r,ZJ[a.toLowerCase()]=r,s&&(q4[r]=l)),!t[l]){if(t[l]=!0,!c)throw new Error(`String representation missing for key code ${l} around scan code ${a}`);H0.define(l,c),bR.define(l,u||c),CR.define(l,g||u||c)}d&&(Hz[d]=l)}})();var xc;(function(o){function e(a){return H0.keyCodeToStr(a)}o.toString=e;function t(a){return H0.strToKeyCode(a)}o.fromString=t;function i(a){return bR.keyCodeToStr(a)}o.toUserSettingsUS=i;function n(a){return CR.keyCodeToStr(a)}o.toUserSettingsGeneral=n;function s(a){return bR.strToKeyCode(a)||CR.strToKeyCode(a)}o.fromUserSettings=s;function r(a){if(a>=98&&a<=113)return null;switch(a){case 16:return"Up";case 18:return"Down";case 15:return"Left";case 17:return"Right"}return H0.keyCodeToStr(a)}o.toElectronAccelerator=r})(xc||(xc={}));function Hi(o,e){const t=(e&65535)<<16>>>0;return(o|t)>>>0}var qB={};let zp;const sN=globalThis.vscode;var dz;if(typeof sN<"u"&&typeof sN.process<"u"){const o=sN.process;zp={get platform(){return o.platform},get arch(){return o.arch},get env(){return o.env},cwd(){return o.cwd()}}}else typeof process<"u"&&typeof((dz=process==null?void 0:process.versions)==null?void 0:dz.node)=="string"?zp={get platform(){return process.platform},get arch(){return process.arch},get env(){return qB},cwd(){return qB.VSCODE_CWD||process.cwd()}}:zp={get platform(){return En?"win32":Je?"darwin":"linux"},get arch(){},get env(){return{}},cwd(){return"/"}};const My=zp.cwd,vR=zp.env,YJ=zp.platform,XJ=65,QJ=97,JJ=90,eee=122,ah=46,$n=47,yo=92,vd=58,tee=63;class Vz extends Error{constructor(e,t,i){let n;typeof t=="string"&&t.indexOf("not ")===0?(n="must not be",t=t.replace(/^not /,"")):n="must be";const s=e.indexOf(".")!==-1?"property":"argument";let r=`The "${e}" ${s} ${n} of type ${t}`;r+=`. Received type ${typeof i}`,super(r),this.code="ERR_INVALID_ARG_TYPE"}}function iee(o,e){if(o===null||typeof o!="object")throw new Vz(e,"Object",o)}function nn(o,e){if(typeof o!="string")throw new Vz(e,"string",o)}const dd=YJ==="win32";function Dt(o){return o===$n||o===yo}function wR(o){return o===$n}function wd(o){return o>=XJ&&o<=JJ||o>=QJ&&o<=eee}function Ay(o,e,t,i){let n="",s=0,r=-1,a=0,l=0;for(let c=0;c<=o.length;++c){if(c<o.length)l=o.charCodeAt(c);else{if(i(l))break;l=$n}if(i(l)){if(!(r===c-1||a===1))if(a===2){if(n.length<2||s!==2||n.charCodeAt(n.length-1)!==ah||n.charCodeAt(n.length-2)!==ah){if(n.length>2){const d=n.lastIndexOf(t);d===-1?(n="",s=0):(n=n.slice(0,d),s=n.length-1-n.lastIndexOf(t)),r=c,a=0;continue}else if(n.length!==0){n="",s=0,r=c,a=0;continue}}e&&(n+=n.length>0?`${t}..`:"..",s=2)}else n.length>0?n+=`${t}${o.slice(r+1,c)}`:n=o.slice(r+1,c),s=c-r-1;r=c,a=0}else l===ah&&a!==-1?++a:a=-1}return n}function nee(o){return o?`${o[0]==="."?"":"."}${o}`:""}function zz(o,e){iee(e,"pathObject");const t=e.dir||e.root,i=e.base||`${e.name||""}${nee(e.ext)}`;return t?t===e.root?`${t}${i}`:`${t}${o}${i}`:i}const Ms={resolve(...o){let e="",t="",i=!1;for(let n=o.length-1;n>=-1;n--){let s;if(n>=0){if(s=o[n],nn(s,`paths[${n}]`),s.length===0)continue}else e.length===0?s=My():(s=vR[`=${e}`]||My(),(s===void 0||s.slice(0,2).toLowerCase()!==e.toLowerCase()&&s.charCodeAt(2)===yo)&&(s=`${e}\\`));const r=s.length;let a=0,l="",c=!1;const d=s.charCodeAt(0);if(r===1)Dt(d)&&(a=1,c=!0);else if(Dt(d))if(c=!0,Dt(s.charCodeAt(1))){let h=2,u=h;for(;h<r&&!Dt(s.charCodeAt(h));)h++;if(h<r&&h!==u){const g=s.slice(u,h);for(u=h;h<r&&Dt(s.charCodeAt(h));)h++;if(h<r&&h!==u){for(u=h;h<r&&!Dt(s.charCodeAt(h));)h++;(h===r||h!==u)&&(l=`\\\\${g}\\${s.slice(u,h)}`,a=h)}}}else a=1;else wd(d)&&s.charCodeAt(1)===vd&&(l=s.slice(0,2),a=2,r>2&&Dt(s.charCodeAt(2))&&(c=!0,a=3));if(l.length>0)if(e.length>0){if(l.toLowerCase()!==e.toLowerCase())continue}else e=l;if(i){if(e.length>0)break}else if(t=`${s.slice(a)}\\${t}`,i=c,c&&e.length>0)break}return t=Ay(t,!i,"\\",Dt),i?`${e}\\${t}`:`${e}${t}`||"."},normalize(o){nn(o,"path");const e=o.length;if(e===0)return".";let t=0,i,n=!1;const s=o.charCodeAt(0);if(e===1)return wR(s)?"\\":o;if(Dt(s))if(n=!0,Dt(o.charCodeAt(1))){let a=2,l=a;for(;a<e&&!Dt(o.charCodeAt(a));)a++;if(a<e&&a!==l){const c=o.slice(l,a);for(l=a;a<e&&Dt(o.charCodeAt(a));)a++;if(a<e&&a!==l){for(l=a;a<e&&!Dt(o.charCodeAt(a));)a++;if(a===e)return`\\\\${c}\\${o.slice(l)}\\`;a!==l&&(i=`\\\\${c}\\${o.slice(l,a)}`,t=a)}}}else t=1;else wd(s)&&o.charCodeAt(1)===vd&&(i=o.slice(0,2),t=2,e>2&&Dt(o.charCodeAt(2))&&(n=!0,t=3));let r=t<e?Ay(o.slice(t),!n,"\\",Dt):"";return r.length===0&&!n&&(r="."),r.length>0&&Dt(o.charCodeAt(e-1))&&(r+="\\"),i===void 0?n?`\\${r}`:r:n?`${i}\\${r}`:`${i}${r}`},isAbsolute(o){nn(o,"path");const e=o.length;if(e===0)return!1;const t=o.charCodeAt(0);return Dt(t)||e>2&&wd(t)&&o.charCodeAt(1)===vd&&Dt(o.charCodeAt(2))},join(...o){if(o.length===0)return".";let e,t;for(let s=0;s<o.length;++s){const r=o[s];nn(r,"path"),r.length>0&&(e===void 0?e=t=r:e+=`\\${r}`)}if(e===void 0)return".";let i=!0,n=0;if(typeof t=="string"&&Dt(t.charCodeAt(0))){++n;const s=t.length;s>1&&Dt(t.charCodeAt(1))&&(++n,s>2&&(Dt(t.charCodeAt(2))?++n:i=!1))}if(i){for(;n<e.length&&Dt(e.charCodeAt(n));)n++;n>=2&&(e=`\\${e.slice(n)}`)}return Ms.normalize(e)},relative(o,e){if(nn(o,"from"),nn(e,"to"),o===e)return"";const t=Ms.resolve(o),i=Ms.resolve(e);if(t===i||(o=t.toLowerCase(),e=i.toLowerCase(),o===e))return"";let n=0;for(;n<o.length&&o.charCodeAt(n)===yo;)n++;let s=o.length;for(;s-1>n&&o.charCodeAt(s-1)===yo;)s--;const r=s-n;let a=0;for(;a<e.length&&e.charCodeAt(a)===yo;)a++;let l=e.length;for(;l-1>a&&e.charCodeAt(l-1)===yo;)l--;const c=l-a,d=r<c?r:c;let h=-1,u=0;for(;u<d;u++){const f=o.charCodeAt(n+u);if(f!==e.charCodeAt(a+u))break;f===yo&&(h=u)}if(u!==d){if(h===-1)return i}else{if(c>d){if(e.charCodeAt(a+u)===yo)return i.slice(a+u+1);if(u===2)return i.slice(a+u)}r>d&&(o.charCodeAt(n+u)===yo?h=u:u===2&&(h=3)),h===-1&&(h=0)}let g="";for(u=n+h+1;u<=s;++u)(u===s||o.charCodeAt(u)===yo)&&(g+=g.length===0?"..":"\\..");return a+=h,g.length>0?`${g}${i.slice(a,l)}`:(i.charCodeAt(a)===yo&&++a,i.slice(a,l))},toNamespacedPath(o){if(typeof o!="string"||o.length===0)return o;const e=Ms.resolve(o);if(e.length<=2)return o;if(e.charCodeAt(0)===yo){if(e.charCodeAt(1)===yo){const t=e.charCodeAt(2);if(t!==tee&&t!==ah)return`\\\\?\\UNC\\${e.slice(2)}`}}else if(wd(e.charCodeAt(0))&&e.charCodeAt(1)===vd&&e.charCodeAt(2)===yo)return`\\\\?\\${e}`;return o},dirname(o){nn(o,"path");const e=o.length;if(e===0)return".";let t=-1,i=0;const n=o.charCodeAt(0);if(e===1)return Dt(n)?o:".";if(Dt(n)){if(t=i=1,Dt(o.charCodeAt(1))){let a=2,l=a;for(;a<e&&!Dt(o.charCodeAt(a));)a++;if(a<e&&a!==l){for(l=a;a<e&&Dt(o.charCodeAt(a));)a++;if(a<e&&a!==l){for(l=a;a<e&&!Dt(o.charCodeAt(a));)a++;if(a===e)return o;a!==l&&(t=i=a+1)}}}}else wd(n)&&o.charCodeAt(1)===vd&&(t=e>2&&Dt(o.charCodeAt(2))?3:2,i=t);let s=-1,r=!0;for(let a=e-1;a>=i;--a)if(Dt(o.charCodeAt(a))){if(!r){s=a;break}}else r=!1;if(s===-1){if(t===-1)return".";s=t}return o.slice(0,s)},basename(o,e){e!==void 0&&nn(e,"suffix"),nn(o,"path");let t=0,i=-1,n=!0,s;if(o.length>=2&&wd(o.charCodeAt(0))&&o.charCodeAt(1)===vd&&(t=2),e!==void 0&&e.length>0&&e.length<=o.length){if(e===o)return"";let r=e.length-1,a=-1;for(s=o.length-1;s>=t;--s){const l=o.charCodeAt(s);if(Dt(l)){if(!n){t=s+1;break}}else a===-1&&(n=!1,a=s+1),r>=0&&(l===e.charCodeAt(r)?--r===-1&&(i=s):(r=-1,i=a))}return t===i?i=a:i===-1&&(i=o.length),o.slice(t,i)}for(s=o.length-1;s>=t;--s)if(Dt(o.charCodeAt(s))){if(!n){t=s+1;break}}else i===-1&&(n=!1,i=s+1);return i===-1?"":o.slice(t,i)},extname(o){nn(o,"path");let e=0,t=-1,i=0,n=-1,s=!0,r=0;o.length>=2&&o.charCodeAt(1)===vd&&wd(o.charCodeAt(0))&&(e=i=2);for(let a=o.length-1;a>=e;--a){const l=o.charCodeAt(a);if(Dt(l)){if(!s){i=a+1;break}continue}n===-1&&(s=!1,n=a+1),l===ah?t===-1?t=a:r!==1&&(r=1):t!==-1&&(r=-1)}return t===-1||n===-1||r===0||r===1&&t===n-1&&t===i+1?"":o.slice(t,n)},format:zz.bind(null,"\\"),parse(o){nn(o,"path");const e={root:"",dir:"",base:"",ext:"",name:""};if(o.length===0)return e;const t=o.length;let i=0,n=o.charCodeAt(0);if(t===1)return Dt(n)?(e.root=e.dir=o,e):(e.base=e.name=o,e);if(Dt(n)){if(i=1,Dt(o.charCodeAt(1))){let h=2,u=h;for(;h<t&&!Dt(o.charCodeAt(h));)h++;if(h<t&&h!==u){for(u=h;h<t&&Dt(o.charCodeAt(h));)h++;if(h<t&&h!==u){for(u=h;h<t&&!Dt(o.charCodeAt(h));)h++;h===t?i=h:h!==u&&(i=h+1)}}}}else if(wd(n)&&o.charCodeAt(1)===vd){if(t<=2)return e.root=e.dir=o,e;if(i=2,Dt(o.charCodeAt(2))){if(t===3)return e.root=e.dir=o,e;i=3}}i>0&&(e.root=o.slice(0,i));let s=-1,r=i,a=-1,l=!0,c=o.length-1,d=0;for(;c>=i;--c){if(n=o.charCodeAt(c),Dt(n)){if(!l){r=c+1;break}continue}a===-1&&(l=!1,a=c+1),n===ah?s===-1?s=c:d!==1&&(d=1):s!==-1&&(d=-1)}return a!==-1&&(s===-1||d===0||d===1&&s===a-1&&s===r+1?e.base=e.name=o.slice(r,a):(e.name=o.slice(r,s),e.base=o.slice(r,a),e.ext=o.slice(s,a))),r>0&&r!==i?e.dir=o.slice(0,r-1):e.dir=e.root,e},sep:"\\",delimiter:";",win32:null,posix:null},see=(()=>{if(dd){const o=/\\/g;return()=>{const e=My().replace(o,"/");return e.slice(e.indexOf("/"))}}return()=>My()})(),Ai={resolve(...o){let e="",t=!1;for(let i=o.length-1;i>=-1&&!t;i--){const n=i>=0?o[i]:see();nn(n,`paths[${i}]`),n.length!==0&&(e=`${n}/${e}`,t=n.charCodeAt(0)===$n)}return e=Ay(e,!t,"/",wR),t?`/${e}`:e.length>0?e:"."},normalize(o){if(nn(o,"path"),o.length===0)return".";const e=o.charCodeAt(0)===$n,t=o.charCodeAt(o.length-1)===$n;return o=Ay(o,!e,"/",wR),o.length===0?e?"/":t?"./":".":(t&&(o+="/"),e?`/${o}`:o)},isAbsolute(o){return nn(o,"path"),o.length>0&&o.charCodeAt(0)===$n},join(...o){if(o.length===0)return".";let e;for(let t=0;t<o.length;++t){const i=o[t];nn(i,"path"),i.length>0&&(e===void 0?e=i:e+=`/${i}`)}return e===void 0?".":Ai.normalize(e)},relative(o,e){if(nn(o,"from"),nn(e,"to"),o===e||(o=Ai.resolve(o),e=Ai.resolve(e),o===e))return"";const t=1,i=o.length,n=i-t,s=1,r=e.length-s,a=n<r?n:r;let l=-1,c=0;for(;c<a;c++){const h=o.charCodeAt(t+c);if(h!==e.charCodeAt(s+c))break;h===$n&&(l=c)}if(c===a)if(r>a){if(e.charCodeAt(s+c)===$n)return e.slice(s+c+1);if(c===0)return e.slice(s+c)}else n>a&&(o.charCodeAt(t+c)===$n?l=c:c===0&&(l=0));let d="";for(c=t+l+1;c<=i;++c)(c===i||o.charCodeAt(c)===$n)&&(d+=d.length===0?"..":"/..");return`${d}${e.slice(s+l)}`},toNamespacedPath(o){return o},dirname(o){if(nn(o,"path"),o.length===0)return".";const e=o.charCodeAt(0)===$n;let t=-1,i=!0;for(let n=o.length-1;n>=1;--n)if(o.charCodeAt(n)===$n){if(!i){t=n;break}}else i=!1;return t===-1?e?"/":".":e&&t===1?"//":o.slice(0,t)},basename(o,e){e!==void 0&&nn(e,"ext"),nn(o,"path");let t=0,i=-1,n=!0,s;if(e!==void 0&&e.length>0&&e.length<=o.length){if(e===o)return"";let r=e.length-1,a=-1;for(s=o.length-1;s>=0;--s){const l=o.charCodeAt(s);if(l===$n){if(!n){t=s+1;break}}else a===-1&&(n=!1,a=s+1),r>=0&&(l===e.charCodeAt(r)?--r===-1&&(i=s):(r=-1,i=a))}return t===i?i=a:i===-1&&(i=o.length),o.slice(t,i)}for(s=o.length-1;s>=0;--s)if(o.charCodeAt(s)===$n){if(!n){t=s+1;break}}else i===-1&&(n=!1,i=s+1);return i===-1?"":o.slice(t,i)},extname(o){nn(o,"path");let e=-1,t=0,i=-1,n=!0,s=0;for(let r=o.length-1;r>=0;--r){const a=o.charCodeAt(r);if(a===$n){if(!n){t=r+1;break}continue}i===-1&&(n=!1,i=r+1),a===ah?e===-1?e=r:s!==1&&(s=1):e!==-1&&(s=-1)}return e===-1||i===-1||s===0||s===1&&e===i-1&&e===t+1?"":o.slice(e,i)},format:zz.bind(null,"/"),parse(o){nn(o,"path");const e={root:"",dir:"",base:"",ext:"",name:""};if(o.length===0)return e;const t=o.charCodeAt(0)===$n;let i;t?(e.root="/",i=1):i=0;let n=-1,s=0,r=-1,a=!0,l=o.length-1,c=0;for(;l>=i;--l){const d=o.charCodeAt(l);if(d===$n){if(!a){s=l+1;break}continue}r===-1&&(a=!1,r=l+1),d===ah?n===-1?n=l:c!==1&&(c=1):n!==-1&&(c=-1)}if(r!==-1){const d=s===0&&t?1:s;n===-1||c===0||c===1&&n===r-1&&n===s+1?e.base=e.name=o.slice(d,r):(e.name=o.slice(d,n),e.base=o.slice(d,r),e.ext=o.slice(n,r))}return s>0?e.dir=o.slice(0,s-1):t&&(e.dir="/"),e},sep:"/",delimiter:":",win32:null,posix:null};Ai.win32=Ms.win32=Ms;Ai.posix=Ms.posix=Ai;const Uz=dd?Ms.normalize:Ai.normalize,oee=dd?Ms.join:Ai.join,ree=dd?Ms.resolve:Ai.resolve,aee=dd?Ms.relative:Ai.relative,$z=dd?Ms.dirname:Ai.dirname,lh=dd?Ms.basename:Ai.basename,lee=dd?Ms.extname:Ai.extname,Ia=dd?Ms.sep:Ai.sep,cee=/^\w[\w\d+.-]*$/,dee=/^\//,hee=/^\/\//;function uee(o,e){if(!o.scheme&&e)throw new Error(`[UriError]: Scheme is missing: {scheme: "", authority: "${o.authority}", path: "${o.path}", query: "${o.query}", fragment: "${o.fragment}"}`);if(o.scheme&&!cee.test(o.scheme))throw new Error("[UriError]: Scheme contains illegal characters.");if(o.path){if(o.authority){if(!dee.test(o.path))throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash ("/") character')}else if(hee.test(o.path))throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters ("//")')}}function gee(o,e){return!o&&!e?"file":o}function fee(o,e){switch(o){case"https":case"http":case"file":e?e[0]!==Ca&&(e=Ca+e):e=Ca;break}return e}const Ri="",Ca="/",mee=/^(([^:/?#]+?):)?(\/\/([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?/;class Se{static isUri(e){return e instanceof Se?!0:e?typeof e.authority=="string"&&typeof e.fragment=="string"&&typeof e.path=="string"&&typeof e.query=="string"&&typeof e.scheme=="string"&&typeof e.fsPath=="string"&&typeof e.with=="function"&&typeof e.toString=="function":!1}constructor(e,t,i,n,s,r=!1){typeof e=="object"?(this.scheme=e.scheme||Ri,this.authority=e.authority||Ri,this.path=e.path||Ri,this.query=e.query||Ri,this.fragment=e.fragment||Ri):(this.scheme=gee(e,r),this.authority=t||Ri,this.path=fee(this.scheme,i||Ri),this.query=n||Ri,this.fragment=s||Ri,uee(this,r))}get fsPath(){return Py(this,!1)}with(e){if(!e)return this;let{scheme:t,authority:i,path:n,query:s,fragment:r}=e;return t===void 0?t=this.scheme:t===null&&(t=Ri),i===void 0?i=this.authority:i===null&&(i=Ri),n===void 0?n=this.path:n===null&&(n=Ri),s===void 0?s=this.query:s===null&&(s=Ri),r===void 0?r=this.fragment:r===null&&(r=Ri),t===this.scheme&&i===this.authority&&n===this.path&&s===this.query&&r===this.fragment?this:new wm(t,i,n,s,r)}static parse(e,t=!1){const i=mee.exec(e);return i?new wm(i[2]||Ri,DS(i[4]||Ri),DS(i[5]||Ri),DS(i[7]||Ri),DS(i[9]||Ri),t):new wm(Ri,Ri,Ri,Ri,Ri)}static file(e){let t=Ri;if(En&&(e=e.replace(/\\/g,Ca)),e[0]===Ca&&e[1]===Ca){const i=e.indexOf(Ca,2);i===-1?(t=e.substring(2),e=Ca):(t=e.substring(2,i),e=e.substring(i)||Ca)}return new wm("file",t,e,Ri,Ri)}static from(e,t){return new wm(e.scheme,e.authority,e.path,e.query,e.fragment,t)}static joinPath(e,...t){if(!e.path)throw new Error("[UriError]: cannot call joinPath on URI without path");let i;return En&&e.scheme==="file"?i=Se.file(Ms.join(Py(e,!0),...t)).path:i=Ai.join(e.path,...t),e.with({path:i})}toString(e=!1){return SR(this,e)}toJSON(){return this}static revive(e){if(e){if(e instanceof Se)return e;{const t=new wm(e);return t._formatted=e.external??null,t._fsPath=e._sep===jz?e.fsPath??null:null,t}}else return e}}const jz=En?1:void 0;let wm=class extends Se{constructor(){super(...arguments),this._formatted=null,this._fsPath=null}get fsPath(){return this._fsPath||(this._fsPath=Py(this,!1)),this._fsPath}toString(e=!1){return e?SR(this,!0):(this._formatted||(this._formatted=SR(this,!1)),this._formatted)}toJSON(){const e={$mid:1};return this._fsPath&&(e.fsPath=this._fsPath,e._sep=jz),this._formatted&&(e.external=this._formatted),this.path&&(e.path=this.path),this.scheme&&(e.scheme=this.scheme),this.authority&&(e.authority=this.authority),this.query&&(e.query=this.query),this.fragment&&(e.fragment=this.fragment),e}};const Kz={58:"%3A",47:"%2F",63:"%3F",35:"%23",91:"%5B",93:"%5D",64:"%40",33:"%21",36:"%24",38:"%26",39:"%27",40:"%28",41:"%29",42:"%2A",43:"%2B",44:"%2C",59:"%3B",61:"%3D",32:"%20"};function GB(o,e,t){let i,n=-1;for(let s=0;s<o.length;s++){const r=o.charCodeAt(s);if(r>=97&&r<=122||r>=65&&r<=90||r>=48&&r<=57||r===45||r===46||r===95||r===126||e&&r===47||t&&r===91||t&&r===93||t&&r===58)n!==-1&&(i+=encodeURIComponent(o.substring(n,s)),n=-1),i!==void 0&&(i+=o.charAt(s));else{i===void 0&&(i=o.substr(0,s));const a=Kz[r];a!==void 0?(n!==-1&&(i+=encodeURIComponent(o.substring(n,s)),n=-1),i+=a):n===-1&&(n=s)}}return n!==-1&&(i+=encodeURIComponent(o.substring(n))),i!==void 0?i:o}function pee(o){let e;for(let t=0;t<o.length;t++){const i=o.charCodeAt(t);i===35||i===63?(e===void 0&&(e=o.substr(0,t)),e+=Kz[i]):e!==void 0&&(e+=o[t])}return e!==void 0?e:o}function Py(o,e){let t;return o.authority&&o.path.length>1&&o.scheme==="file"?t=`//${o.authority}${o.path}`:o.path.charCodeAt(0)===47&&(o.path.charCodeAt(1)>=65&&o.path.charCodeAt(1)<=90||o.path.charCodeAt(1)>=97&&o.path.charCodeAt(1)<=122)&&o.path.charCodeAt(2)===58?e?t=o.path.substr(1):t=o.path[1].toLowerCase()+o.path.substr(2):t=o.path,En&&(t=t.replace(/\//g,"\\")),t}function SR(o,e){const t=e?pee:GB;let i="",{scheme:n,authority:s,path:r,query:a,fragment:l}=o;if(n&&(i+=n,i+=":"),(s||n==="file")&&(i+=Ca,i+=Ca),s){let c=s.indexOf("@");if(c!==-1){const d=s.substr(0,c);s=s.substr(c+1),c=d.lastIndexOf(":"),c===-1?i+=t(d,!1,!1):(i+=t(d.substr(0,c),!1,!1),i+=":",i+=t(d.substr(c+1),!1,!0)),i+="@"}s=s.toLowerCase(),c=s.lastIndexOf(":"),c===-1?i+=t(s,!1,!0):(i+=t(s.substr(0,c),!1,!0),i+=s.substr(c))}if(r){if(r.length>=3&&r.charCodeAt(0)===47&&r.charCodeAt(2)===58){const c=r.charCodeAt(1);c>=65&&c<=90&&(r=`/${String.fromCharCode(c+32)}:${r.substr(3)}`)}else if(r.length>=2&&r.charCodeAt(1)===58){const c=r.charCodeAt(0);c>=65&&c<=90&&(r=`${String.fromCharCode(c+32)}:${r.substr(2)}`)}i+=t(r,!0,!1)}return a&&(i+="?",i+=t(a,!1,!1)),l&&(i+="#",i+=e?l:GB(l,!1,!1)),i}function qz(o){try{return decodeURIComponent(o)}catch{return o.length>3?o.substr(0,3)+qz(o.substr(3)):o}}const ZB=/(%[0-9A-Za-z][0-9A-Za-z])+/g;function DS(o){return o.match(ZB)?o.replace(ZB,e=>qz(e)):o}let A=class Eu{constructor(e,t){this.lineNumber=e,this.column=t}with(e=this.lineNumber,t=this.column){return e===this.lineNumber&&t===this.column?this:new Eu(e,t)}delta(e=0,t=0){return this.with(this.lineNumber+e,this.column+t)}equals(e){return Eu.equals(this,e)}static equals(e,t){return!e&&!t?!0:!!e&&!!t&&e.lineNumber===t.lineNumber&&e.column===t.column}isBefore(e){return Eu.isBefore(this,e)}static isBefore(e,t){return e.lineNumber<t.lineNumber?!0:t.lineNumber<e.lineNumber?!1:e.column<t.column}isBeforeOrEqual(e){return Eu.isBeforeOrEqual(this,e)}static isBeforeOrEqual(e,t){return e.lineNumber<t.lineNumber?!0:t.lineNumber<e.lineNumber?!1:e.column<=t.column}static compare(e,t){const i=e.lineNumber|0,n=t.lineNumber|0;if(i===n){const s=e.column|0,r=t.column|0;return s-r}return i-n}clone(){return new Eu(this.lineNumber,this.column)}toString(){return"("+this.lineNumber+","+this.column+")"}static lift(e){return new Eu(e.lineNumber,e.column)}static isIPosition(e){return e&&typeof e.lineNumber=="number"&&typeof e.column=="number"}toJSON(){return{lineNumber:this.lineNumber,column:this.column}}},x=class hn{constructor(e,t,i,n){e>i||e===i&&t>n?(this.startLineNumber=i,this.startColumn=n,this.endLineNumber=e,this.endColumn=t):(this.startLineNumber=e,this.startColumn=t,this.endLineNumber=i,this.endColumn=n)}isEmpty(){return hn.isEmpty(this)}static isEmpty(e){return e.startLineNumber===e.endLineNumber&&e.startColumn===e.endColumn}containsPosition(e){return hn.containsPosition(this,e)}static containsPosition(e,t){return!(t.lineNumber<e.startLineNumber||t.lineNumber>e.endLineNumber||t.lineNumber===e.startLineNumber&&t.column<e.startColumn||t.lineNumber===e.endLineNumber&&t.column>e.endColumn)}static strictContainsPosition(e,t){return!(t.lineNumber<e.startLineNumber||t.lineNumber>e.endLineNumber||t.lineNumber===e.startLineNumber&&t.column<=e.startColumn||t.lineNumber===e.endLineNumber&&t.column>=e.endColumn)}containsRange(e){return hn.containsRange(this,e)}static containsRange(e,t){return!(t.startLineNumber<e.startLineNumber||t.endLineNumber<e.startLineNumber||t.startLineNumber>e.endLineNumber||t.endLineNumber>e.endLineNumber||t.startLineNumber===e.startLineNumber&&t.startColumn<e.startColumn||t.endLineNumber===e.endLineNumber&&t.endColumn>e.endColumn)}strictContainsRange(e){return hn.strictContainsRange(this,e)}static strictContainsRange(e,t){return!(t.startLineNumber<e.startLineNumber||t.endLineNumber<e.startLineNumber||t.startLineNumber>e.endLineNumber||t.endLineNumber>e.endLineNumber||t.startLineNumber===e.startLineNumber&&t.startColumn<=e.startColumn||t.endLineNumber===e.endLineNumber&&t.endColumn>=e.endColumn)}plusRange(e){return hn.plusRange(this,e)}static plusRange(e,t){let i,n,s,r;return t.startLineNumber<e.startLineNumber?(i=t.startLineNumber,n=t.startColumn):t.startLineNumber===e.startLineNumber?(i=t.startLineNumber,n=Math.min(t.startColumn,e.startColumn)):(i=e.startLineNumber,n=e.startColumn),t.endLineNumber>e.endLineNumber?(s=t.endLineNumber,r=t.endColumn):t.endLineNumber===e.endLineNumber?(s=t.endLineNumber,r=Math.max(t.endColumn,e.endColumn)):(s=e.endLineNumber,r=e.endColumn),new hn(i,n,s,r)}intersectRanges(e){return hn.intersectRanges(this,e)}static intersectRanges(e,t){let i=e.startLineNumber,n=e.startColumn,s=e.endLineNumber,r=e.endColumn;const a=t.startLineNumber,l=t.startColumn,c=t.endLineNumber,d=t.endColumn;return i<a?(i=a,n=l):i===a&&(n=Math.max(n,l)),s>c?(s=c,r=d):s===c&&(r=Math.min(r,d)),i>s||i===s&&n>r?null:new hn(i,n,s,r)}equalsRange(e){return hn.equalsRange(this,e)}static equalsRange(e,t){return!e&&!t?!0:!!e&&!!t&&e.startLineNumber===t.startLineNumber&&e.startColumn===t.startColumn&&e.endLineNumber===t.endLineNumber&&e.endColumn===t.endColumn}getEndPosition(){return hn.getEndPosition(this)}static getEndPosition(e){return new A(e.endLineNumber,e.endColumn)}getStartPosition(){return hn.getStartPosition(this)}static getStartPosition(e){return new A(e.startLineNumber,e.startColumn)}toString(){return"["+this.startLineNumber+","+this.startColumn+" -> "+this.endLineNumber+","+this.endColumn+"]"}setEndPosition(e,t){return new hn(this.startLineNumber,this.startColumn,e,t)}setStartPosition(e,t){return new hn(e,t,this.endLineNumber,this.endColumn)}collapseToStart(){return hn.collapseToStart(this)}static collapseToStart(e){return new hn(e.startLineNumber,e.startColumn,e.startLineNumber,e.startColumn)}collapseToEnd(){return hn.collapseToEnd(this)}static collapseToEnd(e){return new hn(e.endLineNumber,e.endColumn,e.endLineNumber,e.endColumn)}delta(e){return new hn(this.startLineNumber+e,this.startColumn,this.endLineNumber+e,this.endColumn)}static fromPositions(e,t=e){return new hn(e.lineNumber,e.column,t.lineNumber,t.column)}static lift(e){return e?new hn(e.startLineNumber,e.startColumn,e.endLineNumber,e.endColumn):null}static isIRange(e){return e&&typeof e.startLineNumber=="number"&&typeof e.startColumn=="number"&&typeof e.endLineNumber=="number"&&typeof e.endColumn=="number"}static areIntersectingOrTouching(e,t){return!(e.endLineNumber<t.startLineNumber||e.endLineNumber===t.startLineNumber&&e.endColumn<t.startColumn||t.endLineNumber<e.startLineNumber||t.endLineNumber===e.startLineNumber&&t.endColumn<e.startColumn)}static areIntersecting(e,t){return!(e.endLineNumber<t.startLineNumber||e.endLineNumber===t.startLineNumber&&e.endColumn<=t.startColumn||t.endLineNumber<e.startLineNumber||t.endLineNumber===e.startLineNumber&&t.endColumn<=e.startColumn)}static compareRangesUsingStarts(e,t){if(e&&t){const s=e.startLineNumber|0,r=t.startLineNumber|0;if(s===r){const a=e.startColumn|0,l=t.startColumn|0;if(a===l){const c=e.endLineNumber|0,d=t.endLineNumber|0;if(c===d){const h=e.endColumn|0,u=t.endColumn|0;return h-u}return c-d}return a-l}return s-r}return(e?1:0)-(t?1:0)}static compareRangesUsingEnds(e,t){return e.endLineNumber===t.endLineNumber?e.endColumn===t.endColumn?e.startLineNumber===t.startLineNumber?e.startColumn-t.startColumn:e.startLineNumber-t.startLineNumber:e.endColumn-t.endColumn:e.endLineNumber-t.endLineNumber}static spansMultipleLines(e){return e.endLineNumber>e.startLineNumber}toJSON(){return this}},pe=class Nr extends x{constructor(e,t,i,n){super(e,t,i,n),this.selectionStartLineNumber=e,this.selectionStartColumn=t,this.positionLineNumber=i,this.positionColumn=n}toString(){return"["+this.selectionStartLineNumber+","+this.selectionStartColumn+" -> "+this.positionLineNumber+","+this.positionColumn+"]"}equalsSelection(e){return Nr.selectionsEqual(this,e)}static selectionsEqual(e,t){return e.selectionStartLineNumber===t.selectionStartLineNumber&&e.selectionStartColumn===t.selectionStartColumn&&e.positionLineNumber===t.positionLineNumber&&e.positionColumn===t.positionColumn}getDirection(){return this.selectionStartLineNumber===this.startLineNumber&&this.selectionStartColumn===this.startColumn?0:1}setEndPosition(e,t){return this.getDirection()===0?new Nr(this.startLineNumber,this.startColumn,e,t):new Nr(e,t,this.startLineNumber,this.startColumn)}getPosition(){return new A(this.positionLineNumber,this.positionColumn)}getSelectionStart(){return new A(this.selectionStartLineNumber,this.selectionStartColumn)}setStartPosition(e,t){return this.getDirection()===0?new Nr(e,t,this.endLineNumber,this.endColumn):new Nr(this.endLineNumber,this.endColumn,e,t)}static fromPositions(e,t=e){return new Nr(e.lineNumber,e.column,t.lineNumber,t.column)}static fromRange(e,t){return t===0?new Nr(e.startLineNumber,e.startColumn,e.endLineNumber,e.endColumn):new Nr(e.endLineNumber,e.endColumn,e.startLineNumber,e.startColumn)}static liftSelection(e){return new Nr(e.selectionStartLineNumber,e.selectionStartColumn,e.positionLineNumber,e.positionColumn)}static selectionsArrEqual(e,t){if(e&&!t||!e&&t)return!1;if(!e&&!t)return!0;if(e.length!==t.length)return!1;for(let i=0,n=e.length;i<n;i++)if(!this.selectionsEqual(e[i],t[i]))return!1;return!0}static isISelection(e){return e&&typeof e.selectionStartLineNumber=="number"&&typeof e.selectionStartColumn=="number"&&typeof e.positionLineNumber=="number"&&typeof e.positionColumn=="number"}static createWithDirection(e,t,i,n,s){return s===0?new Nr(e,t,i,n):new Nr(i,n,e,t)}};const yR=Object.create(null);function y(o,e){if(As(e)){const t=yR[e];if(t===void 0)throw new Error(`${o} references an unknown codicon: ${e}`);e=t}return yR[o]=e,{id:o}}function Gz(){return yR}const _ee={add:y("add",6e4),plus:y("plus",6e4),gistNew:y("gist-new",6e4),repoCreate:y("repo-create",6e4),lightbulb:y("lightbulb",60001),lightBulb:y("light-bulb",60001),repo:y("repo",60002),repoDelete:y("repo-delete",60002),gistFork:y("gist-fork",60003),repoForked:y("repo-forked",60003),gitPullRequest:y("git-pull-request",60004),gitPullRequestAbandoned:y("git-pull-request-abandoned",60004),recordKeys:y("record-keys",60005),keyboard:y("keyboard",60005),tag:y("tag",60006),gitPullRequestLabel:y("git-pull-request-label",60006),tagAdd:y("tag-add",60006),tagRemove:y("tag-remove",60006),person:y("person",60007),personFollow:y("person-follow",60007),personOutline:y("person-outline",60007),personFilled:y("person-filled",60007),gitBranch:y("git-branch",60008),gitBranchCreate:y("git-branch-create",60008),gitBranchDelete:y("git-branch-delete",60008),sourceControl:y("source-control",60008),mirror:y("mirror",60009),mirrorPublic:y("mirror-public",60009),star:y("star",60010),starAdd:y("star-add",60010),starDelete:y("star-delete",60010),starEmpty:y("star-empty",60010),comment:y("comment",60011),commentAdd:y("comment-add",60011),alert:y("alert",60012),warning:y("warning",60012),search:y("search",60013),searchSave:y("search-save",60013),logOut:y("log-out",60014),signOut:y("sign-out",60014),logIn:y("log-in",60015),signIn:y("sign-in",60015),eye:y("eye",60016),eyeUnwatch:y("eye-unwatch",60016),eyeWatch:y("eye-watch",60016),circleFilled:y("circle-filled",60017),primitiveDot:y("primitive-dot",60017),closeDirty:y("close-dirty",60017),debugBreakpoint:y("debug-breakpoint",60017),debugBreakpointDisabled:y("debug-breakpoint-disabled",60017),debugHint:y("debug-hint",60017),terminalDecorationSuccess:y("terminal-decoration-success",60017),primitiveSquare:y("primitive-square",60018),edit:y("edit",60019),pencil:y("pencil",60019),info:y("info",60020),issueOpened:y("issue-opened",60020),gistPrivate:y("gist-private",60021),gitForkPrivate:y("git-fork-private",60021),lock:y("lock",60021),mirrorPrivate:y("mirror-private",60021),close:y("close",60022),removeClose:y("remove-close",60022),x:y("x",60022),repoSync:y("repo-sync",60023),sync:y("sync",60023),clone:y("clone",60024),desktopDownload:y("desktop-download",60024),beaker:y("beaker",60025),microscope:y("microscope",60025),vm:y("vm",60026),deviceDesktop:y("device-desktop",60026),file:y("file",60027),fileText:y("file-text",60027),more:y("more",60028),ellipsis:y("ellipsis",60028),kebabHorizontal:y("kebab-horizontal",60028),mailReply:y("mail-reply",60029),reply:y("reply",60029),organization:y("organization",60030),organizationFilled:y("organization-filled",60030),organizationOutline:y("organization-outline",60030),newFile:y("new-file",60031),fileAdd:y("file-add",60031),newFolder:y("new-folder",60032),fileDirectoryCreate:y("file-directory-create",60032),trash:y("trash",60033),trashcan:y("trashcan",60033),history:y("history",60034),clock:y("clock",60034),folder:y("folder",60035),fileDirectory:y("file-directory",60035),symbolFolder:y("symbol-folder",60035),logoGithub:y("logo-github",60036),markGithub:y("mark-github",60036),github:y("github",60036),terminal:y("terminal",60037),console:y("console",60037),repl:y("repl",60037),zap:y("zap",60038),symbolEvent:y("symbol-event",60038),error:y("error",60039),stop:y("stop",60039),variable:y("variable",60040),symbolVariable:y("symbol-variable",60040),array:y("array",60042),symbolArray:y("symbol-array",60042),symbolModule:y("symbol-module",60043),symbolPackage:y("symbol-package",60043),symbolNamespace:y("symbol-namespace",60043),symbolObject:y("symbol-object",60043),symbolMethod:y("symbol-method",60044),symbolFunction:y("symbol-function",60044),symbolConstructor:y("symbol-constructor",60044),symbolBoolean:y("symbol-boolean",60047),symbolNull:y("symbol-null",60047),symbolNumeric:y("symbol-numeric",60048),symbolNumber:y("symbol-number",60048),symbolStructure:y("symbol-structure",60049),symbolStruct:y("symbol-struct",60049),symbolParameter:y("symbol-parameter",60050),symbolTypeParameter:y("symbol-type-parameter",60050),symbolKey:y("symbol-key",60051),symbolText:y("symbol-text",60051),symbolReference:y("symbol-reference",60052),goToFile:y("go-to-file",60052),symbolEnum:y("symbol-enum",60053),symbolValue:y("symbol-value",60053),symbolRuler:y("symbol-ruler",60054),symbolUnit:y("symbol-unit",60054),activateBreakpoints:y("activate-breakpoints",60055),archive:y("archive",60056),arrowBoth:y("arrow-both",60057),arrowDown:y("arrow-down",60058),arrowLeft:y("arrow-left",60059),arrowRight:y("arrow-right",60060),arrowSmallDown:y("arrow-small-down",60061),arrowSmallLeft:y("arrow-small-left",60062),arrowSmallRight:y("arrow-small-right",60063),arrowSmallUp:y("arrow-small-up",60064),arrowUp:y("arrow-up",60065),bell:y("bell",60066),bold:y("bold",60067),book:y("book",60068),bookmark:y("bookmark",60069),debugBreakpointConditionalUnverified:y("debug-breakpoint-conditional-unverified",60070),debugBreakpointConditional:y("debug-breakpoint-conditional",60071),debugBreakpointConditionalDisabled:y("debug-breakpoint-conditional-disabled",60071),debugBreakpointDataUnverified:y("debug-breakpoint-data-unverified",60072),debugBreakpointData:y("debug-breakpoint-data",60073),debugBreakpointDataDisabled:y("debug-breakpoint-data-disabled",60073),debugBreakpointLogUnverified:y("debug-breakpoint-log-unverified",60074),debugBreakpointLog:y("debug-breakpoint-log",60075),debugBreakpointLogDisabled:y("debug-breakpoint-log-disabled",60075),briefcase:y("briefcase",60076),broadcast:y("broadcast",60077),browser:y("browser",60078),bug:y("bug",60079),calendar:y("calendar",60080),caseSensitive:y("case-sensitive",60081),check:y("check",60082),checklist:y("checklist",60083),chevronDown:y("chevron-down",60084),chevronLeft:y("chevron-left",60085),chevronRight:y("chevron-right",60086),chevronUp:y("chevron-up",60087),chromeClose:y("chrome-close",60088),chromeMaximize:y("chrome-maximize",60089),chromeMinimize:y("chrome-minimize",60090),chromeRestore:y("chrome-restore",60091),circleOutline:y("circle-outline",60092),circle:y("circle",60092),debugBreakpointUnverified:y("debug-breakpoint-unverified",60092),terminalDecorationIncomplete:y("terminal-decoration-incomplete",60092),circleSlash:y("circle-slash",60093),circuitBoard:y("circuit-board",60094),clearAll:y("clear-all",60095),clippy:y("clippy",60096),closeAll:y("close-all",60097),cloudDownload:y("cloud-download",60098),cloudUpload:y("cloud-upload",60099),code:y("code",60100),collapseAll:y("collapse-all",60101),colorMode:y("color-mode",60102),commentDiscussion:y("comment-discussion",60103),creditCard:y("credit-card",60105),dash:y("dash",60108),dashboard:y("dashboard",60109),database:y("database",60110),debugContinue:y("debug-continue",60111),debugDisconnect:y("debug-disconnect",60112),debugPause:y("debug-pause",60113),debugRestart:y("debug-restart",60114),debugStart:y("debug-start",60115),debugStepInto:y("debug-step-into",60116),debugStepOut:y("debug-step-out",60117),debugStepOver:y("debug-step-over",60118),debugStop:y("debug-stop",60119),debug:y("debug",60120),deviceCameraVideo:y("device-camera-video",60121),deviceCamera:y("device-camera",60122),deviceMobile:y("device-mobile",60123),diffAdded:y("diff-added",60124),diffIgnored:y("diff-ignored",60125),diffModified:y("diff-modified",60126),diffRemoved:y("diff-removed",60127),diffRenamed:y("diff-renamed",60128),diff:y("diff",60129),diffSidebyside:y("diff-sidebyside",60129),discard:y("discard",60130),editorLayout:y("editor-layout",60131),emptyWindow:y("empty-window",60132),exclude:y("exclude",60133),extensions:y("extensions",60134),eyeClosed:y("eye-closed",60135),fileBinary:y("file-binary",60136),fileCode:y("file-code",60137),fileMedia:y("file-media",60138),filePdf:y("file-pdf",60139),fileSubmodule:y("file-submodule",60140),fileSymlinkDirectory:y("file-symlink-directory",60141),fileSymlinkFile:y("file-symlink-file",60142),fileZip:y("file-zip",60143),files:y("files",60144),filter:y("filter",60145),flame:y("flame",60146),foldDown:y("fold-down",60147),foldUp:y("fold-up",60148),fold:y("fold",60149),folderActive:y("folder-active",60150),folderOpened:y("folder-opened",60151),gear:y("gear",60152),gift:y("gift",60153),gistSecret:y("gist-secret",60154),gist:y("gist",60155),gitCommit:y("git-commit",60156),gitCompare:y("git-compare",60157),compareChanges:y("compare-changes",60157),gitMerge:y("git-merge",60158),githubAction:y("github-action",60159),githubAlt:y("github-alt",60160),globe:y("globe",60161),grabber:y("grabber",60162),graph:y("graph",60163),gripper:y("gripper",60164),heart:y("heart",60165),home:y("home",60166),horizontalRule:y("horizontal-rule",60167),hubot:y("hubot",60168),inbox:y("inbox",60169),issueReopened:y("issue-reopened",60171),issues:y("issues",60172),italic:y("italic",60173),jersey:y("jersey",60174),json:y("json",60175),kebabVertical:y("kebab-vertical",60176),key:y("key",60177),law:y("law",60178),lightbulbAutofix:y("lightbulb-autofix",60179),linkExternal:y("link-external",60180),link:y("link",60181),listOrdered:y("list-ordered",60182),listUnordered:y("list-unordered",60183),liveShare:y("live-share",60184),loading:y("loading",60185),location:y("location",60186),mailRead:y("mail-read",60187),mail:y("mail",60188),markdown:y("markdown",60189),megaphone:y("megaphone",60190),mention:y("mention",60191),milestone:y("milestone",60192),gitPullRequestMilestone:y("git-pull-request-milestone",60192),mortarBoard:y("mortar-board",60193),move:y("move",60194),multipleWindows:y("multiple-windows",60195),mute:y("mute",60196),noNewline:y("no-newline",60197),note:y("note",60198),octoface:y("octoface",60199),openPreview:y("open-preview",60200),package:y("package",60201),paintcan:y("paintcan",60202),pin:y("pin",60203),play:y("play",60204),run:y("run",60204),plug:y("plug",60205),preserveCase:y("preserve-case",60206),preview:y("preview",60207),project:y("project",60208),pulse:y("pulse",60209),question:y("question",60210),quote:y("quote",60211),radioTower:y("radio-tower",60212),reactions:y("reactions",60213),references:y("references",60214),refresh:y("refresh",60215),regex:y("regex",60216),remoteExplorer:y("remote-explorer",60217),remote:y("remote",60218),remove:y("remove",60219),replaceAll:y("replace-all",60220),replace:y("replace",60221),repoClone:y("repo-clone",60222),repoForcePush:y("repo-force-push",60223),repoPull:y("repo-pull",60224),repoPush:y("repo-push",60225),report:y("report",60226),requestChanges:y("request-changes",60227),rocket:y("rocket",60228),rootFolderOpened:y("root-folder-opened",60229),rootFolder:y("root-folder",60230),rss:y("rss",60231),ruby:y("ruby",60232),saveAll:y("save-all",60233),saveAs:y("save-as",60234),save:y("save",60235),screenFull:y("screen-full",60236),screenNormal:y("screen-normal",60237),searchStop:y("search-stop",60238),server:y("server",60240),settingsGear:y("settings-gear",60241),settings:y("settings",60242),shield:y("shield",60243),smiley:y("smiley",60244),sortPrecedence:y("sort-precedence",60245),splitHorizontal:y("split-horizontal",60246),splitVertical:y("split-vertical",60247),squirrel:y("squirrel",60248),starFull:y("star-full",60249),starHalf:y("star-half",60250),symbolClass:y("symbol-class",60251),symbolColor:y("symbol-color",60252),symbolConstant:y("symbol-constant",60253),symbolEnumMember:y("symbol-enum-member",60254),symbolField:y("symbol-field",60255),symbolFile:y("symbol-file",60256),symbolInterface:y("symbol-interface",60257),symbolKeyword:y("symbol-keyword",60258),symbolMisc:y("symbol-misc",60259),symbolOperator:y("symbol-operator",60260),symbolProperty:y("symbol-property",60261),wrench:y("wrench",60261),wrenchSubaction:y("wrench-subaction",60261),symbolSnippet:y("symbol-snippet",60262),tasklist:y("tasklist",60263),telescope:y("telescope",60264),textSize:y("text-size",60265),threeBars:y("three-bars",60266),thumbsdown:y("thumbsdown",60267),thumbsup:y("thumbsup",60268),tools:y("tools",60269),triangleDown:y("triangle-down",60270),triangleLeft:y("triangle-left",60271),triangleRight:y("triangle-right",60272),triangleUp:y("triangle-up",60273),twitter:y("twitter",60274),unfold:y("unfold",60275),unlock:y("unlock",60276),unmute:y("unmute",60277),unverified:y("unverified",60278),verified:y("verified",60279),versions:y("versions",60280),vmActive:y("vm-active",60281),vmOutline:y("vm-outline",60282),vmRunning:y("vm-running",60283),watch:y("watch",60284),whitespace:y("whitespace",60285),wholeWord:y("whole-word",60286),window:y("window",60287),wordWrap:y("word-wrap",60288),zoomIn:y("zoom-in",60289),zoomOut:y("zoom-out",60290),listFilter:y("list-filter",60291),listFlat:y("list-flat",60292),listSelection:y("list-selection",60293),selection:y("selection",60293),listTree:y("list-tree",60294),debugBreakpointFunctionUnverified:y("debug-breakpoint-function-unverified",60295),debugBreakpointFunction:y("debug-breakpoint-function",60296),debugBreakpointFunctionDisabled:y("debug-breakpoint-function-disabled",60296),debugStackframeActive:y("debug-stackframe-active",60297),circleSmallFilled:y("circle-small-filled",60298),debugStackframeDot:y("debug-stackframe-dot",60298),terminalDecorationMark:y("terminal-decoration-mark",60298),debugStackframe:y("debug-stackframe",60299),debugStackframeFocused:y("debug-stackframe-focused",60299),debugBreakpointUnsupported:y("debug-breakpoint-unsupported",60300),symbolString:y("symbol-string",60301),debugReverseContinue:y("debug-reverse-continue",60302),debugStepBack:y("debug-step-back",60303),debugRestartFrame:y("debug-restart-frame",60304),debugAlt:y("debug-alt",60305),callIncoming:y("call-incoming",60306),callOutgoing:y("call-outgoing",60307),menu:y("menu",60308),expandAll:y("expand-all",60309),feedback:y("feedback",60310),gitPullRequestReviewer:y("git-pull-request-reviewer",60310),groupByRefType:y("group-by-ref-type",60311),ungroupByRefType:y("ungroup-by-ref-type",60312),account:y("account",60313),gitPullRequestAssignee:y("git-pull-request-assignee",60313),bellDot:y("bell-dot",60314),debugConsole:y("debug-console",60315),library:y("library",60316),output:y("output",60317),runAll:y("run-all",60318),syncIgnored:y("sync-ignored",60319),pinned:y("pinned",60320),githubInverted:y("github-inverted",60321),serverProcess:y("server-process",60322),serverEnvironment:y("server-environment",60323),pass:y("pass",60324),issueClosed:y("issue-closed",60324),stopCircle:y("stop-circle",60325),playCircle:y("play-circle",60326),record:y("record",60327),debugAltSmall:y("debug-alt-small",60328),vmConnect:y("vm-connect",60329),cloud:y("cloud",60330),merge:y("merge",60331),export:y("export",60332),graphLeft:y("graph-left",60333),magnet:y("magnet",60334),notebook:y("notebook",60335),redo:y("redo",60336),checkAll:y("check-all",60337),pinnedDirty:y("pinned-dirty",60338),passFilled:y("pass-filled",60339),circleLargeFilled:y("circle-large-filled",60340),circleLarge:y("circle-large",60341),circleLargeOutline:y("circle-large-outline",60341),combine:y("combine",60342),gather:y("gather",60342),table:y("table",60343),variableGroup:y("variable-group",60344),typeHierarchy:y("type-hierarchy",60345),typeHierarchySub:y("type-hierarchy-sub",60346),typeHierarchySuper:y("type-hierarchy-super",60347),gitPullRequestCreate:y("git-pull-request-create",60348),runAbove:y("run-above",60349),runBelow:y("run-below",60350),notebookTemplate:y("notebook-template",60351),debugRerun:y("debug-rerun",60352),workspaceTrusted:y("workspace-trusted",60353),workspaceUntrusted:y("workspace-untrusted",60354),workspaceUnknown:y("workspace-unknown",60355),terminalCmd:y("terminal-cmd",60356),terminalDebian:y("terminal-debian",60357),terminalLinux:y("terminal-linux",60358),terminalPowershell:y("terminal-powershell",60359),terminalTmux:y("terminal-tmux",60360),terminalUbuntu:y("terminal-ubuntu",60361),terminalBash:y("terminal-bash",60362),arrowSwap:y("arrow-swap",60363),copy:y("copy",60364),personAdd:y("person-add",60365),filterFilled:y("filter-filled",60366),wand:y("wand",60367),debugLineByLine:y("debug-line-by-line",60368),inspect:y("inspect",60369),layers:y("layers",60370),layersDot:y("layers-dot",60371),layersActive:y("layers-active",60372),compass:y("compass",60373),compassDot:y("compass-dot",60374),compassActive:y("compass-active",60375),azure:y("azure",60376),issueDraft:y("issue-draft",60377),gitPullRequestClosed:y("git-pull-request-closed",60378),gitPullRequestDraft:y("git-pull-request-draft",60379),debugAll:y("debug-all",60380),debugCoverage:y("debug-coverage",60381),runErrors:y("run-errors",60382),folderLibrary:y("folder-library",60383),debugContinueSmall:y("debug-continue-small",60384),beakerStop:y("beaker-stop",60385),graphLine:y("graph-line",60386),graphScatter:y("graph-scatter",60387),pieChart:y("pie-chart",60388),bracket:y("bracket",60175),bracketDot:y("bracket-dot",60389),bracketError:y("bracket-error",60390),lockSmall:y("lock-small",60391),azureDevops:y("azure-devops",60392),verifiedFilled:y("verified-filled",60393),newline:y("newline",60394),layout:y("layout",60395),layoutActivitybarLeft:y("layout-activitybar-left",60396),layoutActivitybarRight:y("layout-activitybar-right",60397),layoutPanelLeft:y("layout-panel-left",60398),layoutPanelCenter:y("layout-panel-center",60399),layoutPanelJustify:y("layout-panel-justify",60400),layoutPanelRight:y("layout-panel-right",60401),layoutPanel:y("layout-panel",60402),layoutSidebarLeft:y("layout-sidebar-left",60403),layoutSidebarRight:y("layout-sidebar-right",60404),layoutStatusbar:y("layout-statusbar",60405),layoutMenubar:y("layout-menubar",60406),layoutCentered:y("layout-centered",60407),target:y("target",60408),indent:y("indent",60409),recordSmall:y("record-small",60410),errorSmall:y("error-small",60411),terminalDecorationError:y("terminal-decoration-error",60411),arrowCircleDown:y("arrow-circle-down",60412),arrowCircleLeft:y("arrow-circle-left",60413),arrowCircleRight:y("arrow-circle-right",60414),arrowCircleUp:y("arrow-circle-up",60415),layoutSidebarRightOff:y("layout-sidebar-right-off",60416),layoutPanelOff:y("layout-panel-off",60417),layoutSidebarLeftOff:y("layout-sidebar-left-off",60418),blank:y("blank",60419),heartFilled:y("heart-filled",60420),map:y("map",60421),mapHorizontal:y("map-horizontal",60421),foldHorizontal:y("fold-horizontal",60421),mapFilled:y("map-filled",60422),mapHorizontalFilled:y("map-horizontal-filled",60422),foldHorizontalFilled:y("fold-horizontal-filled",60422),circleSmall:y("circle-small",60423),bellSlash:y("bell-slash",60424),bellSlashDot:y("bell-slash-dot",60425),commentUnresolved:y("comment-unresolved",60426),gitPullRequestGoToChanges:y("git-pull-request-go-to-changes",60427),gitPullRequestNewChanges:y("git-pull-request-new-changes",60428),searchFuzzy:y("search-fuzzy",60429),commentDraft:y("comment-draft",60430),send:y("send",60431),sparkle:y("sparkle",60432),insert:y("insert",60433),mic:y("mic",60434),thumbsdownFilled:y("thumbsdown-filled",60435),thumbsupFilled:y("thumbsup-filled",60436),coffee:y("coffee",60437),snake:y("snake",60438),game:y("game",60439),vr:y("vr",60440),chip:y("chip",60441),piano:y("piano",60442),music:y("music",60443),micFilled:y("mic-filled",60444),repoFetch:y("repo-fetch",60445),copilot:y("copilot",60446),lightbulbSparkle:y("lightbulb-sparkle",60447),robot:y("robot",60448),sparkleFilled:y("sparkle-filled",60449),diffSingle:y("diff-single",60450),diffMultiple:y("diff-multiple",60451),surroundWith:y("surround-with",60452),share:y("share",60453),gitStash:y("git-stash",60454),gitStashApply:y("git-stash-apply",60455),gitStashPop:y("git-stash-pop",60456),vscode:y("vscode",60457),vscodeInsiders:y("vscode-insiders",60458),codeOss:y("code-oss",60459),runCoverage:y("run-coverage",60460),runAllCoverage:y("run-all-coverage",60461),coverage:y("coverage",60462),githubProject:y("github-project",60463),mapVertical:y("map-vertical",60464),foldVertical:y("fold-vertical",60464),mapVerticalFilled:y("map-vertical-filled",60465),foldVerticalFilled:y("fold-vertical-filled",60465),goToSearch:y("go-to-search",60466),percentage:y("percentage",60467),sortPercentage:y("sort-percentage",60467),attach:y("attach",60468)},bee={dialogError:y("dialog-error","error"),dialogWarning:y("dialog-warning","warning"),dialogInfo:y("dialog-info","info"),dialogClose:y("dialog-close","close"),treeItemExpanded:y("tree-item-expanded","chevron-down"),treeFilterOnTypeOn:y("tree-filter-on-type-on","list-filter"),treeFilterOnTypeOff:y("tree-filter-on-type-off","list-selection"),treeFilterClear:y("tree-filter-clear","close"),treeItemLoading:y("tree-item-loading","loading"),menuSelection:y("menu-selection","check"),menuSubmenu:y("menu-submenu","chevron-right"),menuBarMore:y("menubar-more","more"),scrollbarButtonLeft:y("scrollbar-button-left","triangle-left"),scrollbarButtonRight:y("scrollbar-button-right","triangle-right"),scrollbarButtonUp:y("scrollbar-button-up","triangle-up"),scrollbarButtonDown:y("scrollbar-button-down","triangle-down"),toolBarMore:y("toolbar-more","more"),quickInputBack:y("quick-input-back","arrow-left"),dropDownButton:y("drop-down-button",60084),symbolCustomColor:y("symbol-customcolor",60252),exportIcon:y("export",60332),workspaceUnspecified:y("workspace-unspecified",60355),newLine:y("newline",60394),thumbsDownFilled:y("thumbsdown-filled",60435),thumbsUpFilled:y("thumbsup-filled",60436),gitFetch:y("git-fetch",60445),lightbulbSparkleAutofix:y("lightbulb-sparkle-autofix",60447),debugBreakpointPending:y("debug-breakpoint-pending",60377)},te={..._ee,...bee};let Zz=class{constructor(){this._tokenizationSupports=new Map,this._factories=new Map,this._onDidChange=new P,this.onDidChange=this._onDidChange.event,this._colorMap=null}handleChange(e){this._onDidChange.fire({changedLanguages:e,changedColorMap:!1})}register(e,t){return this._tokenizationSupports.set(e,t),this.handleChange([e]),ve(()=>{this._tokenizationSupports.get(e)===t&&(this._tokenizationSupports.delete(e),this.handleChange([e]))})}get(e){return this._tokenizationSupports.get(e)||null}registerFactory(e,t){var n;(n=this._factories.get(e))==null||n.dispose();const i=new Cee(this,e,t);return this._factories.set(e,i),ve(()=>{const s=this._factories.get(e);!s||s!==i||(this._factories.delete(e),s.dispose())})}async getOrCreate(e){const t=this.get(e);if(t)return t;const i=this._factories.get(e);return!i||i.isResolved?null:(await i.resolve(),this.get(e))}isResolved(e){if(this.get(e))return!0;const i=this._factories.get(e);return!!(!i||i.isResolved)}setColorMap(e){this._colorMap=e,this._onDidChange.fire({changedLanguages:Array.from(this._tokenizationSupports.keys()),changedColorMap:!0})}getColorMap(){return this._colorMap}getDefaultBackground(){return this._colorMap&&this._colorMap.length>2?this._colorMap[2]:null}};class Cee extends F{get isResolved(){return this._isResolved}constructor(e,t,i){super(),this._registry=e,this._languageId=t,this._factory=i,this._isDisposed=!1,this._resolvePromise=null,this._isResolved=!1}dispose(){this._isDisposed=!0,super.dispose()}async resolve(){return this._resolvePromise||(this._resolvePromise=this._create()),this._resolvePromise}async _create(){const e=await this._factory.tokenizationSupport;this._isResolved=!0,e&&!this._isDisposed&&this._register(this._registry.register(this._languageId,e))}}let ew=class{constructor(e,t,i){this.offset=e,this.type=t,this.language=i,this._tokenBrand=void 0}toString(){return"("+this.offset+", "+this.type+")"}};class G4{constructor(e,t){this.tokens=e,this.endState=t,this._tokenizationResultBrand=void 0}}class gI{constructor(e,t){this.tokens=e,this.endState=t,this._encodedTokenizationResultBrand=void 0}}var eo;(function(o){o[o.Increase=0]="Increase",o[o.Decrease=1]="Decrease"})(eo||(eo={}));var tw;(function(o){const e=new Map;e.set(0,te.symbolMethod),e.set(1,te.symbolFunction),e.set(2,te.symbolConstructor),e.set(3,te.symbolField),e.set(4,te.symbolVariable),e.set(5,te.symbolClass),e.set(6,te.symbolStruct),e.set(7,te.symbolInterface),e.set(8,te.symbolModule),e.set(9,te.symbolProperty),e.set(10,te.symbolEvent),e.set(11,te.symbolOperator),e.set(12,te.symbolUnit),e.set(13,te.symbolValue),e.set(15,te.symbolEnum),e.set(14,te.symbolConstant),e.set(15,te.symbolEnum),e.set(16,te.symbolEnumMember),e.set(17,te.symbolKeyword),e.set(27,te.symbolSnippet),e.set(18,te.symbolText),e.set(19,te.symbolColor),e.set(20,te.symbolFile),e.set(21,te.symbolReference),e.set(22,te.symbolCustomColor),e.set(23,te.symbolFolder),e.set(24,te.symbolTypeParameter),e.set(25,te.account),e.set(26,te.issues);function t(s){let r=e.get(s);return r||(console.info("No codicon found for CompletionItemKind "+s),r=te.symbolProperty),r}o.toIcon=t;const i=new Map;i.set("method",0),i.set("function",1),i.set("constructor",2),i.set("field",3),i.set("variable",4),i.set("class",5),i.set("struct",6),i.set("interface",7),i.set("module",8),i.set("property",9),i.set("event",10),i.set("operator",11),i.set("unit",12),i.set("value",13),i.set("constant",14),i.set("enum",15),i.set("enum-member",16),i.set("enumMember",16),i.set("keyword",17),i.set("snippet",27),i.set("text",18),i.set("color",19),i.set("file",20),i.set("reference",21),i.set("customcolor",22),i.set("folder",23),i.set("type-parameter",24),i.set("typeParameter",24),i.set("account",25),i.set("issue",26);function n(s,r){let a=i.get(s);return typeof a>"u"&&!r&&(a=9),a}o.fromString=n})(tw||(tw={}));var Ea;(function(o){o[o.Automatic=0]="Automatic",o[o.Explicit=1]="Explicit"})(Ea||(Ea={}));class Yz{constructor(e,t,i,n){this.range=e,this.text=t,this.completionKind=i,this.isSnippetText=n}equals(e){return x.lift(this.range).equalsRange(e.range)&&this.text===e.text&&this.completionKind===e.completionKind&&this.isSnippetText===e.isSnippetText}}var iw;(function(o){o[o.Automatic=0]="Automatic",o[o.PasteAs=1]="PasteAs"})(iw||(iw={}));var Rl;(function(o){o[o.Invoke=1]="Invoke",o[o.TriggerCharacter=2]="TriggerCharacter",o[o.ContentChange=3]="ContentChange"})(Rl||(Rl={}));var x_;(function(o){o[o.Text=0]="Text",o[o.Read=1]="Read",o[o.Write=2]="Write"})(x_||(x_={}));function vee(o){return o&&Se.isUri(o.uri)&&x.isIRange(o.range)&&(x.isIRange(o.originSelectionRange)||x.isIRange(o.targetSelectionRange))}const wee={17:m("Array","array"),16:m("Boolean","boolean"),4:m("Class","class"),13:m("Constant","constant"),8:m("Constructor","constructor"),9:m("Enum","enumeration"),21:m("EnumMember","enumeration member"),23:m("Event","event"),7:m("Field","field"),0:m("File","file"),11:m("Function","function"),10:m("Interface","interface"),19:m("Key","key"),5:m("Method","method"),1:m("Module","module"),2:m("Namespace","namespace"),20:m("Null","null"),15:m("Number","number"),18:m("Object","object"),24:m("Operator","operator"),3:m("Package","package"),6:m("Property","property"),14:m("String","string"),22:m("Struct","struct"),25:m("TypeParameter","type parameter"),12:m("Variable","variable")};function See(o,e){return m("symbolAriaLabel","{0} ({1})",o,wee[e])}var Oy;(function(o){const e=new Map;e.set(0,te.symbolFile),e.set(1,te.symbolModule),e.set(2,te.symbolNamespace),e.set(3,te.symbolPackage),e.set(4,te.symbolClass),e.set(5,te.symbolMethod),e.set(6,te.symbolProperty),e.set(7,te.symbolField),e.set(8,te.symbolConstructor),e.set(9,te.symbolEnum),e.set(10,te.symbolInterface),e.set(11,te.symbolFunction),e.set(12,te.symbolVariable),e.set(13,te.symbolConstant),e.set(14,te.symbolString),e.set(15,te.symbolNumber),e.set(16,te.symbolBoolean),e.set(17,te.symbolArray),e.set(18,te.symbolObject),e.set(19,te.symbolKey),e.set(20,te.symbolNull),e.set(21,te.symbolEnumMember),e.set(22,te.symbolStruct),e.set(23,te.symbolEvent),e.set(24,te.symbolOperator),e.set(25,te.symbolTypeParameter);function t(i){let n=e.get(i);return n||(console.info("No codicon found for SymbolKind "+i),n=te.symbolProperty),n}o.toIcon=t})(Oy||(Oy={}));const Rr=class Rr{static fromValue(e){switch(e){case"comment":return Rr.Comment;case"imports":return Rr.Imports;case"region":return Rr.Region}return new Rr(e)}constructor(e){this.value=e}};Rr.Comment=new Rr("comment"),Rr.Imports=new Rr("imports"),Rr.Region=new Rr("region");let Ph=Rr;var LR;(function(o){o[o.AIGenerated=1]="AIGenerated"})(LR||(LR={}));var nw;(function(o){o[o.Invoke=0]="Invoke",o[o.Automatic=1]="Automatic"})(nw||(nw={}));var xR;(function(o){function e(t){return!t||typeof t!="object"?!1:typeof t.id=="string"&&typeof t.title=="string"}o.is=e})(xR||(xR={}));var Fy;(function(o){o[o.Type=1]="Type",o[o.Parameter=2]="Parameter"})(Fy||(Fy={}));class yee{constructor(e){this.createSupport=e,this._tokenizationSupport=null}dispose(){this._tokenizationSupport&&this._tokenizationSupport.then(e=>{e&&e.dispose()})}get tokenizationSupport(){return this._tokenizationSupport||(this._tokenizationSupport=this.createSupport()),this._tokenizationSupport}}const fi=new Zz,kR=new Zz;var Wy;(function(o){o[o.Invoke=0]="Invoke",o[o.Automatic=1]="Automatic"})(Wy||(Wy={}));var DR;(function(o){o[o.Unknown=0]="Unknown",o[o.Disabled=1]="Disabled",o[o.Enabled=2]="Enabled"})(DR||(DR={}));var IR;(function(o){o[o.Invoke=1]="Invoke",o[o.Auto=2]="Auto"})(IR||(IR={}));var ER;(function(o){o[o.None=0]="None",o[o.KeepWhitespace=1]="KeepWhitespace",o[o.InsertAsSnippet=4]="InsertAsSnippet"})(ER||(ER={}));var NR;(function(o){o[o.Method=0]="Method",o[o.Function=1]="Function",o[o.Constructor=2]="Constructor",o[o.Field=3]="Field",o[o.Variable=4]="Variable",o[o.Class=5]="Class",o[o.Struct=6]="Struct",o[o.Interface=7]="Interface",o[o.Module=8]="Module",o[o.Property=9]="Property",o[o.Event=10]="Event",o[o.Operator=11]="Operator",o[o.Unit=12]="Unit",o[o.Value=13]="Value",o[o.Constant=14]="Constant",o[o.Enum=15]="Enum",o[o.EnumMember=16]="EnumMember",o[o.Keyword=17]="Keyword",o[o.Text=18]="Text",o[o.Color=19]="Color",o[o.File=20]="File",o[o.Reference=21]="Reference",o[o.Customcolor=22]="Customcolor",o[o.Folder=23]="Folder",o[o.TypeParameter=24]="TypeParameter",o[o.User=25]="User",o[o.Issue=26]="Issue",o[o.Snippet=27]="Snippet"})(NR||(NR={}));var TR;(function(o){o[o.Deprecated=1]="Deprecated"})(TR||(TR={}));var RR;(function(o){o[o.Invoke=0]="Invoke",o[o.TriggerCharacter=1]="TriggerCharacter",o[o.TriggerForIncompleteCompletions=2]="TriggerForIncompleteCompletions"})(RR||(RR={}));var MR;(function(o){o[o.EXACT=0]="EXACT",o[o.ABOVE=1]="ABOVE",o[o.BELOW=2]="BELOW"})(MR||(MR={}));var AR;(function(o){o[o.NotSet=0]="NotSet",o[o.ContentFlush=1]="ContentFlush",o[o.RecoverFromMarkers=2]="RecoverFromMarkers",o[o.Explicit=3]="Explicit",o[o.Paste=4]="Paste",o[o.Undo=5]="Undo",o[o.Redo=6]="Redo"})(AR||(AR={}));var PR;(function(o){o[o.LF=1]="LF",o[o.CRLF=2]="CRLF"})(PR||(PR={}));var OR;(function(o){o[o.Text=0]="Text",o[o.Read=1]="Read",o[o.Write=2]="Write"})(OR||(OR={}));var FR;(function(o){o[o.None=0]="None",o[o.Keep=1]="Keep",o[o.Brackets=2]="Brackets",o[o.Advanced=3]="Advanced",o[o.Full=4]="Full"})(FR||(FR={}));var WR;(function(o){o[o.acceptSuggestionOnCommitCharacter=0]="acceptSuggestionOnCommitCharacter",o[o.acceptSuggestionOnEnter=1]="acceptSuggestionOnEnter",o[o.accessibilitySupport=2]="accessibilitySupport",o[o.accessibilityPageSize=3]="accessibilityPageSize",o[o.ariaLabel=4]="ariaLabel",o[o.ariaRequired=5]="ariaRequired",o[o.autoClosingBrackets=6]="autoClosingBrackets",o[o.autoClosingComments=7]="autoClosingComments",o[o.screenReaderAnnounceInlineSuggestion=8]="screenReaderAnnounceInlineSuggestion",o[o.autoClosingDelete=9]="autoClosingDelete",o[o.autoClosingOvertype=10]="autoClosingOvertype",o[o.autoClosingQuotes=11]="autoClosingQuotes",o[o.autoIndent=12]="autoIndent",o[o.automaticLayout=13]="automaticLayout",o[o.autoSurround=14]="autoSurround",o[o.bracketPairColorization=15]="bracketPairColorization",o[o.guides=16]="guides",o[o.codeLens=17]="codeLens",o[o.codeLensFontFamily=18]="codeLensFontFamily",o[o.codeLensFontSize=19]="codeLensFontSize",o[o.colorDecorators=20]="colorDecorators",o[o.colorDecoratorsLimit=21]="colorDecoratorsLimit",o[o.columnSelection=22]="columnSelection",o[o.comments=23]="comments",o[o.contextmenu=24]="contextmenu",o[o.copyWithSyntaxHighlighting=25]="copyWithSyntaxHighlighting",o[o.cursorBlinking=26]="cursorBlinking",o[o.cursorSmoothCaretAnimation=27]="cursorSmoothCaretAnimation",o[o.cursorStyle=28]="cursorStyle",o[o.cursorSurroundingLines=29]="cursorSurroundingLines",o[o.cursorSurroundingLinesStyle=30]="cursorSurroundingLinesStyle",o[o.cursorWidth=31]="cursorWidth",o[o.disableLayerHinting=32]="disableLayerHinting",o[o.disableMonospaceOptimizations=33]="disableMonospaceOptimizations",o[o.domReadOnly=34]="domReadOnly",o[o.dragAndDrop=35]="dragAndDrop",o[o.dropIntoEditor=36]="dropIntoEditor",o[o.emptySelectionClipboard=37]="emptySelectionClipboard",o[o.experimentalWhitespaceRendering=38]="experimentalWhitespaceRendering",o[o.extraEditorClassName=39]="extraEditorClassName",o[o.fastScrollSensitivity=40]="fastScrollSensitivity",o[o.find=41]="find",o[o.fixedOverflowWidgets=42]="fixedOverflowWidgets",o[o.folding=43]="folding",o[o.foldingStrategy=44]="foldingStrategy",o[o.foldingHighlight=45]="foldingHighlight",o[o.foldingImportsByDefault=46]="foldingImportsByDefault",o[o.foldingMaximumRegions=47]="foldingMaximumRegions",o[o.unfoldOnClickAfterEndOfLine=48]="unfoldOnClickAfterEndOfLine",o[o.fontFamily=49]="fontFamily",o[o.fontInfo=50]="fontInfo",o[o.fontLigatures=51]="fontLigatures",o[o.fontSize=52]="fontSize",o[o.fontWeight=53]="fontWeight",o[o.fontVariations=54]="fontVariations",o[o.formatOnPaste=55]="formatOnPaste",o[o.formatOnType=56]="formatOnType",o[o.glyphMargin=57]="glyphMargin",o[o.gotoLocation=58]="gotoLocation",o[o.hideCursorInOverviewRuler=59]="hideCursorInOverviewRuler",o[o.hover=60]="hover",o[o.inDiffEditor=61]="inDiffEditor",o[o.inlineSuggest=62]="inlineSuggest",o[o.inlineEdit=63]="inlineEdit",o[o.letterSpacing=64]="letterSpacing",o[o.lightbulb=65]="lightbulb",o[o.lineDecorationsWidth=66]="lineDecorationsWidth",o[o.lineHeight=67]="lineHeight",o[o.lineNumbers=68]="lineNumbers",o[o.lineNumbersMinChars=69]="lineNumbersMinChars",o[o.linkedEditing=70]="linkedEditing",o[o.links=71]="links",o[o.matchBrackets=72]="matchBrackets",o[o.minimap=73]="minimap",o[o.mouseStyle=74]="mouseStyle",o[o.mouseWheelScrollSensitivity=75]="mouseWheelScrollSensitivity",o[o.mouseWheelZoom=76]="mouseWheelZoom",o[o.multiCursorMergeOverlapping=77]="multiCursorMergeOverlapping",o[o.multiCursorModifier=78]="multiCursorModifier",o[o.multiCursorPaste=79]="multiCursorPaste",o[o.multiCursorLimit=80]="multiCursorLimit",o[o.occurrencesHighlight=81]="occurrencesHighlight",o[o.overviewRulerBorder=82]="overviewRulerBorder",o[o.overviewRulerLanes=83]="overviewRulerLanes",o[o.padding=84]="padding",o[o.pasteAs=85]="pasteAs",o[o.parameterHints=86]="parameterHints",o[o.peekWidgetDefaultFocus=87]="peekWidgetDefaultFocus",o[o.placeholder=88]="placeholder",o[o.definitionLinkOpensInPeek=89]="definitionLinkOpensInPeek",o[o.quickSuggestions=90]="quickSuggestions",o[o.quickSuggestionsDelay=91]="quickSuggestionsDelay",o[o.readOnly=92]="readOnly",o[o.readOnlyMessage=93]="readOnlyMessage",o[o.renameOnType=94]="renameOnType",o[o.renderControlCharacters=95]="renderControlCharacters",o[o.renderFinalNewline=96]="renderFinalNewline",o[o.renderLineHighlight=97]="renderLineHighlight",o[o.renderLineHighlightOnlyWhenFocus=98]="renderLineHighlightOnlyWhenFocus",o[o.renderValidationDecorations=99]="renderValidationDecorations",o[o.renderWhitespace=100]="renderWhitespace",o[o.revealHorizontalRightPadding=101]="revealHorizontalRightPadding",o[o.roundedSelection=102]="roundedSelection",o[o.rulers=103]="rulers",o[o.scrollbar=104]="scrollbar",o[o.scrollBeyondLastColumn=105]="scrollBeyondLastColumn",o[o.scrollBeyondLastLine=106]="scrollBeyondLastLine",o[o.scrollPredominantAxis=107]="scrollPredominantAxis",o[o.selectionClipboard=108]="selectionClipboard",o[o.selectionHighlight=109]="selectionHighlight",o[o.selectOnLineNumbers=110]="selectOnLineNumbers",o[o.showFoldingControls=111]="showFoldingControls",o[o.showUnused=112]="showUnused",o[o.snippetSuggestions=113]="snippetSuggestions",o[o.smartSelect=114]="smartSelect",o[o.smoothScrolling=115]="smoothScrolling",o[o.stickyScroll=116]="stickyScroll",o[o.stickyTabStops=117]="stickyTabStops",o[o.stopRenderingLineAfter=118]="stopRenderingLineAfter",o[o.suggest=119]="suggest",o[o.suggestFontSize=120]="suggestFontSize",o[o.suggestLineHeight=121]="suggestLineHeight",o[o.suggestOnTriggerCharacters=122]="suggestOnTriggerCharacters",o[o.suggestSelection=123]="suggestSelection",o[o.tabCompletion=124]="tabCompletion",o[o.tabIndex=125]="tabIndex",o[o.unicodeHighlighting=126]="unicodeHighlighting",o[o.unusualLineTerminators=127]="unusualLineTerminators",o[o.useShadowDOM=128]="useShadowDOM",o[o.useTabStops=129]="useTabStops",o[o.wordBreak=130]="wordBreak",o[o.wordSegmenterLocales=131]="wordSegmenterLocales",o[o.wordSeparators=132]="wordSeparators",o[o.wordWrap=133]="wordWrap",o[o.wordWrapBreakAfterCharacters=134]="wordWrapBreakAfterCharacters",o[o.wordWrapBreakBeforeCharacters=135]="wordWrapBreakBeforeCharacters",o[o.wordWrapColumn=136]="wordWrapColumn",o[o.wordWrapOverride1=137]="wordWrapOverride1",o[o.wordWrapOverride2=138]="wordWrapOverride2",o[o.wrappingIndent=139]="wrappingIndent",o[o.wrappingStrategy=140]="wrappingStrategy",o[o.showDeprecated=141]="showDeprecated",o[o.inlayHints=142]="inlayHints",o[o.editorClassName=143]="editorClassName",o[o.pixelRatio=144]="pixelRatio",o[o.tabFocusMode=145]="tabFocusMode",o[o.layoutInfo=146]="layoutInfo",o[o.wrappingInfo=147]="wrappingInfo",o[o.defaultColorDecorators=148]="defaultColorDecorators",o[o.colorDecoratorsActivatedOn=149]="colorDecoratorsActivatedOn",o[o.inlineCompletionsAccessibilityVerbose=150]="inlineCompletionsAccessibilityVerbose"})(WR||(WR={}));var BR;(function(o){o[o.TextDefined=0]="TextDefined",o[o.LF=1]="LF",o[o.CRLF=2]="CRLF"})(BR||(BR={}));var HR;(function(o){o[o.LF=0]="LF",o[o.CRLF=1]="CRLF"})(HR||(HR={}));var VR;(function(o){o[o.Left=1]="Left",o[o.Center=2]="Center",o[o.Right=3]="Right"})(VR||(VR={}));var zR;(function(o){o[o.Increase=0]="Increase",o[o.Decrease=1]="Decrease"})(zR||(zR={}));var UR;(function(o){o[o.None=0]="None",o[o.Indent=1]="Indent",o[o.IndentOutdent=2]="IndentOutdent",o[o.Outdent=3]="Outdent"})(UR||(UR={}));var $R;(function(o){o[o.Both=0]="Both",o[o.Right=1]="Right",o[o.Left=2]="Left",o[o.None=3]="None"})($R||($R={}));var jR;(function(o){o[o.Type=1]="Type",o[o.Parameter=2]="Parameter"})(jR||(jR={}));var KR;(function(o){o[o.Automatic=0]="Automatic",o[o.Explicit=1]="Explicit"})(KR||(KR={}));var qR;(function(o){o[o.Invoke=0]="Invoke",o[o.Automatic=1]="Automatic"})(qR||(qR={}));var GR;(function(o){o[o.DependsOnKbLayout=-1]="DependsOnKbLayout",o[o.Unknown=0]="Unknown",o[o.Backspace=1]="Backspace",o[o.Tab=2]="Tab",o[o.Enter=3]="Enter",o[o.Shift=4]="Shift",o[o.Ctrl=5]="Ctrl",o[o.Alt=6]="Alt",o[o.PauseBreak=7]="PauseBreak",o[o.CapsLock=8]="CapsLock",o[o.Escape=9]="Escape",o[o.Space=10]="Space",o[o.PageUp=11]="PageUp",o[o.PageDown=12]="PageDown",o[o.End=13]="End",o[o.Home=14]="Home",o[o.LeftArrow=15]="LeftArrow",o[o.UpArrow=16]="UpArrow",o[o.RightArrow=17]="RightArrow",o[o.DownArrow=18]="DownArrow",o[o.Insert=19]="Insert",o[o.Delete=20]="Delete",o[o.Digit0=21]="Digit0",o[o.Digit1=22]="Digit1",o[o.Digit2=23]="Digit2",o[o.Digit3=24]="Digit3",o[o.Digit4=25]="Digit4",o[o.Digit5=26]="Digit5",o[o.Digit6=27]="Digit6",o[o.Digit7=28]="Digit7",o[o.Digit8=29]="Digit8",o[o.Digit9=30]="Digit9",o[o.KeyA=31]="KeyA",o[o.KeyB=32]="KeyB",o[o.KeyC=33]="KeyC",o[o.KeyD=34]="KeyD",o[o.KeyE=35]="KeyE",o[o.KeyF=36]="KeyF",o[o.KeyG=37]="KeyG",o[o.KeyH=38]="KeyH",o[o.KeyI=39]="KeyI",o[o.KeyJ=40]="KeyJ",o[o.KeyK=41]="KeyK",o[o.KeyL=42]="KeyL",o[o.KeyM=43]="KeyM",o[o.KeyN=44]="KeyN",o[o.KeyO=45]="KeyO",o[o.KeyP=46]="KeyP",o[o.KeyQ=47]="KeyQ",o[o.KeyR=48]="KeyR",o[o.KeyS=49]="KeyS",o[o.KeyT=50]="KeyT",o[o.KeyU=51]="KeyU",o[o.KeyV=52]="KeyV",o[o.KeyW=53]="KeyW",o[o.KeyX=54]="KeyX",o[o.KeyY=55]="KeyY",o[o.KeyZ=56]="KeyZ",o[o.Meta=57]="Meta",o[o.ContextMenu=58]="ContextMenu",o[o.F1=59]="F1",o[o.F2=60]="F2",o[o.F3=61]="F3",o[o.F4=62]="F4",o[o.F5=63]="F5",o[o.F6=64]="F6",o[o.F7=65]="F7",o[o.F8=66]="F8",o[o.F9=67]="F9",o[o.F10=68]="F10",o[o.F11=69]="F11",o[o.F12=70]="F12",o[o.F13=71]="F13",o[o.F14=72]="F14",o[o.F15=73]="F15",o[o.F16=74]="F16",o[o.F17=75]="F17",o[o.F18=76]="F18",o[o.F19=77]="F19",o[o.F20=78]="F20",o[o.F21=79]="F21",o[o.F22=80]="F22",o[o.F23=81]="F23",o[o.F24=82]="F24",o[o.NumLock=83]="NumLock",o[o.ScrollLock=84]="ScrollLock",o[o.Semicolon=85]="Semicolon",o[o.Equal=86]="Equal",o[o.Comma=87]="Comma",o[o.Minus=88]="Minus",o[o.Period=89]="Period",o[o.Slash=90]="Slash",o[o.Backquote=91]="Backquote",o[o.BracketLeft=92]="BracketLeft",o[o.Backslash=93]="Backslash",o[o.BracketRight=94]="BracketRight",o[o.Quote=95]="Quote",o[o.OEM_8=96]="OEM_8",o[o.IntlBackslash=97]="IntlBackslash",o[o.Numpad0=98]="Numpad0",o[o.Numpad1=99]="Numpad1",o[o.Numpad2=100]="Numpad2",o[o.Numpad3=101]="Numpad3",o[o.Numpad4=102]="Numpad4",o[o.Numpad5=103]="Numpad5",o[o.Numpad6=104]="Numpad6",o[o.Numpad7=105]="Numpad7",o[o.Numpad8=106]="Numpad8",o[o.Numpad9=107]="Numpad9",o[o.NumpadMultiply=108]="NumpadMultiply",o[o.NumpadAdd=109]="NumpadAdd",o[o.NUMPAD_SEPARATOR=110]="NUMPAD_SEPARATOR",o[o.NumpadSubtract=111]="NumpadSubtract",o[o.NumpadDecimal=112]="NumpadDecimal",o[o.NumpadDivide=113]="NumpadDivide",o[o.KEY_IN_COMPOSITION=114]="KEY_IN_COMPOSITION",o[o.ABNT_C1=115]="ABNT_C1",o[o.ABNT_C2=116]="ABNT_C2",o[o.AudioVolumeMute=117]="AudioVolumeMute",o[o.AudioVolumeUp=118]="AudioVolumeUp",o[o.AudioVolumeDown=119]="AudioVolumeDown",o[o.BrowserSearch=120]="BrowserSearch",o[o.BrowserHome=121]="BrowserHome",o[o.BrowserBack=122]="BrowserBack",o[o.BrowserForward=123]="BrowserForward",o[o.MediaTrackNext=124]="MediaTrackNext",o[o.MediaTrackPrevious=125]="MediaTrackPrevious",o[o.MediaStop=126]="MediaStop",o[o.MediaPlayPause=127]="MediaPlayPause",o[o.LaunchMediaPlayer=128]="LaunchMediaPlayer",o[o.LaunchMail=129]="LaunchMail",o[o.LaunchApp2=130]="LaunchApp2",o[o.Clear=131]="Clear",o[o.MAX_VALUE=132]="MAX_VALUE"})(GR||(GR={}));var ZR;(function(o){o[o.Hint=1]="Hint",o[o.Info=2]="Info",o[o.Warning=4]="Warning",o[o.Error=8]="Error"})(ZR||(ZR={}));var YR;(function(o){o[o.Unnecessary=1]="Unnecessary",o[o.Deprecated=2]="Deprecated"})(YR||(YR={}));var XR;(function(o){o[o.Inline=1]="Inline",o[o.Gutter=2]="Gutter"})(XR||(XR={}));var QR;(function(o){o[o.Normal=1]="Normal",o[o.Underlined=2]="Underlined"})(QR||(QR={}));var JR;(function(o){o[o.UNKNOWN=0]="UNKNOWN",o[o.TEXTAREA=1]="TEXTAREA",o[o.GUTTER_GLYPH_MARGIN=2]="GUTTER_GLYPH_MARGIN",o[o.GUTTER_LINE_NUMBERS=3]="GUTTER_LINE_NUMBERS",o[o.GUTTER_LINE_DECORATIONS=4]="GUTTER_LINE_DECORATIONS",o[o.GUTTER_VIEW_ZONE=5]="GUTTER_VIEW_ZONE",o[o.CONTENT_TEXT=6]="CONTENT_TEXT",o[o.CONTENT_EMPTY=7]="CONTENT_EMPTY",o[o.CONTENT_VIEW_ZONE=8]="CONTENT_VIEW_ZONE",o[o.CONTENT_WIDGET=9]="CONTENT_WIDGET",o[o.OVERVIEW_RULER=10]="OVERVIEW_RULER",o[o.SCROLLBAR=11]="SCROLLBAR",o[o.OVERLAY_WIDGET=12]="OVERLAY_WIDGET",o[o.OUTSIDE_EDITOR=13]="OUTSIDE_EDITOR"})(JR||(JR={}));var eM;(function(o){o[o.AIGenerated=1]="AIGenerated"})(eM||(eM={}));var tM;(function(o){o[o.Invoke=0]="Invoke",o[o.Automatic=1]="Automatic"})(tM||(tM={}));var iM;(function(o){o[o.TOP_RIGHT_CORNER=0]="TOP_RIGHT_CORNER",o[o.BOTTOM_RIGHT_CORNER=1]="BOTTOM_RIGHT_CORNER",o[o.TOP_CENTER=2]="TOP_CENTER"})(iM||(iM={}));var nM;(function(o){o[o.Left=1]="Left",o[o.Center=2]="Center",o[o.Right=4]="Right",o[o.Full=7]="Full"})(nM||(nM={}));var sM;(function(o){o[o.Word=0]="Word",o[o.Line=1]="Line",o[o.Suggest=2]="Suggest"})(sM||(sM={}));var oM;(function(o){o[o.Left=0]="Left",o[o.Right=1]="Right",o[o.None=2]="None",o[o.LeftOfInjectedText=3]="LeftOfInjectedText",o[o.RightOfInjectedText=4]="RightOfInjectedText"})(oM||(oM={}));var rM;(function(o){o[o.Off=0]="Off",o[o.On=1]="On",o[o.Relative=2]="Relative",o[o.Interval=3]="Interval",o[o.Custom=4]="Custom"})(rM||(rM={}));var aM;(function(o){o[o.None=0]="None",o[o.Text=1]="Text",o[o.Blocks=2]="Blocks"})(aM||(aM={}));var lM;(function(o){o[o.Smooth=0]="Smooth",o[o.Immediate=1]="Immediate"})(lM||(lM={}));var cM;(function(o){o[o.Auto=1]="Auto",o[o.Hidden=2]="Hidden",o[o.Visible=3]="Visible"})(cM||(cM={}));var dM;(function(o){o[o.LTR=0]="LTR",o[o.RTL=1]="RTL"})(dM||(dM={}));var hM;(function(o){o.Off="off",o.OnCode="onCode",o.On="on"})(hM||(hM={}));var uM;(function(o){o[o.Invoke=1]="Invoke",o[o.TriggerCharacter=2]="TriggerCharacter",o[o.ContentChange=3]="ContentChange"})(uM||(uM={}));var gM;(function(o){o[o.File=0]="File",o[o.Module=1]="Module",o[o.Namespace=2]="Namespace",o[o.Package=3]="Package",o[o.Class=4]="Class",o[o.Method=5]="Method",o[o.Property=6]="Property",o[o.Field=7]="Field",o[o.Constructor=8]="Constructor",o[o.Enum=9]="Enum",o[o.Interface=10]="Interface",o[o.Function=11]="Function",o[o.Variable=12]="Variable",o[o.Constant=13]="Constant",o[o.String=14]="String",o[o.Number=15]="Number",o[o.Boolean=16]="Boolean",o[o.Array=17]="Array",o[o.Object=18]="Object",o[o.Key=19]="Key",o[o.Null=20]="Null",o[o.EnumMember=21]="EnumMember",o[o.Struct=22]="Struct",o[o.Event=23]="Event",o[o.Operator=24]="Operator",o[o.TypeParameter=25]="TypeParameter"})(gM||(gM={}));var fM;(function(o){o[o.Deprecated=1]="Deprecated"})(fM||(fM={}));var mM;(function(o){o[o.Hidden=0]="Hidden",o[o.Blink=1]="Blink",o[o.Smooth=2]="Smooth",o[o.Phase=3]="Phase",o[o.Expand=4]="Expand",o[o.Solid=5]="Solid"})(mM||(mM={}));var pM;(function(o){o[o.Line=1]="Line",o[o.Block=2]="Block",o[o.Underline=3]="Underline",o[o.LineThin=4]="LineThin",o[o.BlockOutline=5]="BlockOutline",o[o.UnderlineThin=6]="UnderlineThin"})(pM||(pM={}));var _M;(function(o){o[o.AlwaysGrowsWhenTypingAtEdges=0]="AlwaysGrowsWhenTypingAtEdges",o[o.NeverGrowsWhenTypingAtEdges=1]="NeverGrowsWhenTypingAtEdges",o[o.GrowsOnlyWhenTypingBefore=2]="GrowsOnlyWhenTypingBefore",o[o.GrowsOnlyWhenTypingAfter=3]="GrowsOnlyWhenTypingAfter"})(_M||(_M={}));var bM;(function(o){o[o.None=0]="None",o[o.Same=1]="Same",o[o.Indent=2]="Indent",o[o.DeepIndent=3]="DeepIndent"})(bM||(bM={}));var Bc;let Lee=(Bc=class{static chord(e,t){return Hi(e,t)}},Bc.CtrlCmd=2048,Bc.Shift=1024,Bc.Alt=512,Bc.WinCtrl=256,Bc);function Xz(){return{editor:void 0,languages:void 0,CancellationTokenSource:li,Emitter:P,KeyCode:GR,KeyMod:Lee,Position:A,Range:x,Selection:pe,SelectionDirection:dM,MarkerSeverity:ZR,MarkerTag:YR,Uri:Se,Token:ew}}function xee(o,e){const t=o;typeof t.vscodeWindowId!="number"&&Object.defineProperty(t,"vscodeWindowId",{get:()=>e})}const Rt=window;function Qz(o){return o}class kee{constructor(e,t){this.lastCache=void 0,this.lastArgKey=void 0,typeof e=="function"?(this._fn=e,this._computeKey=Qz):(this._fn=t,this._computeKey=e.getCacheKey)}get(e){const t=this._computeKey(e);return this.lastArgKey!==t&&(this.lastArgKey=t,this.lastCache=this._fn(e)),this.lastCache}}class YB{get cachedValues(){return this._map}constructor(e,t){this._map=new Map,this._map2=new Map,typeof e=="function"?(this._fn=e,this._computeKey=Qz):(this._fn=t,this._computeKey=e.getCacheKey)}get(e){const t=this._computeKey(e);if(this._map2.has(t))return this._map2.get(t);const i=this._fn(e);return this._map.set(e,i),this._map2.set(t,i),i}}class Oa{constructor(e){this.executor=e,this._didRun=!1}get value(){if(!this._didRun)try{this._value=this.executor()}catch(e){this._error=e}finally{this._didRun=!0}if(this._error)throw this._error;return this._value}get rawValue(){return this._value}}function Jz(o){return!o||typeof o!="string"?!0:o.trim().length===0}const Dee=/{(\d+)}/g;function xf(o,...e){return e.length===0?o:o.replace(Dee,function(t,i){const n=parseInt(i,10);return isNaN(n)||n<0||n>=e.length?t:e[n]})}function Iee(o){return o.replace(/[<>"'&]/g,e=>{switch(e){case"<":return"&lt;";case">":return"&gt;";case'"':return"&quot;";case"'":return"&apos;";case"&":return"&amp;"}return e})}function jC(o){return o.replace(/[<>&]/g,function(e){switch(e){case"<":return"&lt;";case">":return"&gt;";case"&":return"&amp;";default:return e}})}function Uo(o){return o.replace(/[\\\{\}\*\+\?\|\^\$\.\[\]\(\)]/g,"\\$&")}function Eee(o,e=" "){const t=M1(o,e);return eU(t,e)}function M1(o,e){if(!o||!e)return o;const t=e.length;if(t===0||o.length===0)return o;let i=0;for(;o.indexOf(e,i)===i;)i=i+t;return o.substring(i)}function eU(o,e){if(!o||!e)return o;const t=e.length,i=o.length;if(t===0||i===0)return o;let n=i,s=-1;for(;s=o.lastIndexOf(e,n-1),!(s===-1||s+t!==n);){if(s===0)return"";n=s}return o.substring(0,n)}function Nee(o){return o.replace(/[\-\\\{\}\+\?\|\^\$\.\,\[\]\(\)\#\s]/g,"\\$&").replace(/[\*]/g,".*")}function Tee(o){return o.replace(/\*/g,"")}function tU(o,e,t={}){if(!o)throw new Error("Cannot create regex from empty string");e||(o=Uo(o)),t.wholeWord&&(/\B/.test(o.charAt(0))||(o="\\b"+o),/\B/.test(o.charAt(o.length-1))||(o=o+"\\b"));let i="";return t.global&&(i+="g"),t.matchCase||(i+="i"),t.multiline&&(i+="m"),t.unicode&&(i+="u"),new RegExp(o,i)}function Ree(o){return o.source==="^"||o.source==="^$"||o.source==="$"||o.source==="^\\s*$"?!1:!!(o.exec("")&&o.lastIndex===0)}function Kl(o){return o.split(/\r\n|\r|\n/)}function Mee(o){const e=[],t=o.split(/(\r\n|\r|\n)/);for(let i=0;i<Math.ceil(t.length/2);i++)e.push(t[2*i]+(t[2*i+1]??""));return e}function Jn(o){for(let e=0,t=o.length;e<t;e++){const i=o.charCodeAt(e);if(i!==32&&i!==9)return e}return-1}function Nt(o,e=0,t=o.length){for(let i=e;i<t;i++){const n=o.charCodeAt(i);if(n!==32&&n!==9)return o.substring(e,i)}return o.substring(e,t)}function Na(o,e=o.length-1){for(let t=e;t>=0;t--){const i=o.charCodeAt(t);if(i!==32&&i!==9)return t}return-1}function sw(o,e){return o<e?-1:o>e?1:0}function Z4(o,e,t=0,i=o.length,n=0,s=e.length){for(;t<i&&n<s;t++,n++){const l=o.charCodeAt(t),c=e.charCodeAt(n);if(l<c)return-1;if(l>c)return 1}const r=i-t,a=s-n;return r<a?-1:r>a?1:0}function CM(o,e){return A1(o,e,0,o.length,0,e.length)}function A1(o,e,t=0,i=o.length,n=0,s=e.length){for(;t<i&&n<s;t++,n++){let l=o.charCodeAt(t),c=e.charCodeAt(n);if(l===c)continue;if(l>=128||c>=128)return Z4(o.toLowerCase(),e.toLowerCase(),t,i,n,s);qd(l)&&(l-=32),qd(c)&&(c-=32);const d=l-c;if(d!==0)return d}const r=i-t,a=s-n;return r<a?-1:r>a?1:0}function IS(o){return o>=48&&o<=57}function qd(o){return o>=97&&o<=122}function bl(o){return o>=65&&o<=90}function ep(o,e){return o.length===e.length&&A1(o,e)===0}function Y4(o,e){const t=e.length;return e.length>o.length?!1:A1(o,e,0,t)===0}function Oh(o,e){const t=Math.min(o.length,e.length);let i;for(i=0;i<t;i++)if(o.charCodeAt(i)!==e.charCodeAt(i))return i;return t}function By(o,e){const t=Math.min(o.length,e.length);let i;const n=o.length-1,s=e.length-1;for(i=0;i<t;i++)if(o.charCodeAt(n-i)!==e.charCodeAt(s-i))return i;return t}function Gi(o){return 55296<=o&&o<=56319}function kf(o){return 56320<=o&&o<=57343}function X4(o,e){return(o-55296<<10)+(e-56320)+65536}function Hy(o,e,t){const i=o.charCodeAt(t);if(Gi(i)&&t+1<e){const n=o.charCodeAt(t+1);if(kf(n))return X4(i,n)}return i}function Aee(o,e){const t=o.charCodeAt(e-1);if(kf(t)&&e>1){const i=o.charCodeAt(e-2);if(Gi(i))return X4(i,t)}return t}class Q4{get offset(){return this._offset}constructor(e,t=0){this._str=e,this._len=e.length,this._offset=t}setOffset(e){this._offset=e}prevCodePoint(){const e=Aee(this._str,this._offset);return this._offset-=e>=65536?2:1,e}nextCodePoint(){const e=Hy(this._str,this._len,this._offset);return this._offset+=e>=65536?2:1,e}eol(){return this._offset>=this._len}}class Vy{get offset(){return this._iterator.offset}constructor(e,t=0){this._iterator=new Q4(e,t)}nextGraphemeLength(){const e=zy.getInstance(),t=this._iterator,i=t.offset;let n=e.getGraphemeBreakType(t.nextCodePoint());for(;!t.eol();){const s=t.offset,r=e.getGraphemeBreakType(t.nextCodePoint());if(XB(n,r)){t.setOffset(s);break}n=r}return t.offset-i}prevGraphemeLength(){const e=zy.getInstance(),t=this._iterator,i=t.offset;let n=e.getGraphemeBreakType(t.prevCodePoint());for(;t.offset>0;){const s=t.offset,r=e.getGraphemeBreakType(t.prevCodePoint());if(XB(r,n)){t.setOffset(s);break}n=r}return i-t.offset}eol(){return this._iterator.eol()}}function J4(o,e){return new Vy(o,e).nextGraphemeLength()}function iU(o,e){return new Vy(o,e).prevGraphemeLength()}function Pee(o,e){e>0&&kf(o.charCodeAt(e))&&e--;const t=e+J4(o,e);return[t-iU(o,t),t]}let oN;function Oee(){return/(?:[\u05BE\u05C0\u05C3\u05C6\u05D0-\u05F4\u0608\u060B\u060D\u061B-\u064A\u066D-\u066F\u0671-\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u0710\u0712-\u072F\u074D-\u07A5\u07B1-\u07EA\u07F4\u07F5\u07FA\u07FE-\u0815\u081A\u0824\u0828\u0830-\u0858\u085E-\u088E\u08A0-\u08C9\u200F\uFB1D\uFB1F-\uFB28\uFB2A-\uFD3D\uFD50-\uFDC7\uFDF0-\uFDFC\uFE70-\uFEFC]|\uD802[\uDC00-\uDD1B\uDD20-\uDE00\uDE10-\uDE35\uDE40-\uDEE4\uDEEB-\uDF35\uDF40-\uDFFF]|\uD803[\uDC00-\uDD23\uDE80-\uDEA9\uDEAD-\uDF45\uDF51-\uDF81\uDF86-\uDFF6]|\uD83A[\uDC00-\uDCCF\uDD00-\uDD43\uDD4B-\uDFFF]|\uD83B[\uDC00-\uDEBB])/}function k_(o){return oN||(oN=Oee()),oN.test(o)}const Fee=/^[\t\n\r\x20-\x7E]*$/;function P1(o){return Fee.test(o)}const nU=/[\u2028\u2029]/;function sU(o){return nU.test(o)}function Fh(o){return o>=11904&&o<=55215||o>=63744&&o<=64255||o>=65281&&o<=65374}function e5(o){return o>=127462&&o<=127487||o===8986||o===8987||o===9200||o===9203||o>=9728&&o<=10175||o===11088||o===11093||o>=127744&&o<=128591||o>=128640&&o<=128764||o>=128992&&o<=129008||o>=129280&&o<=129535||o>=129648&&o<=129782}const Wee="\uFEFF";function t5(o){return!!(o&&o.length>0&&o.charCodeAt(0)===65279)}function Bee(o,e=!1){return o?(e&&(o=o.replace(/\\./g,"")),o.toLowerCase()!==o):!1}function oU(o){return o=o%(2*26),o<26?String.fromCharCode(97+o):String.fromCharCode(65+o-26)}function XB(o,e){return o===0?e!==5&&e!==7:o===2&&e===3?!1:o===4||o===2||o===3||e===4||e===2||e===3?!0:!(o===8&&(e===8||e===9||e===11||e===12)||(o===11||o===9)&&(e===9||e===10)||(o===12||o===10)&&e===10||e===5||e===13||e===7||o===1||o===13&&e===14||o===6&&e===6)}const Ju=class Ju{static getInstance(){return Ju._INSTANCE||(Ju._INSTANCE=new Ju),Ju._INSTANCE}constructor(){this._data=Hee()}getGraphemeBreakType(e){if(e<32)return e===10?3:e===13?2:4;if(e<127)return 0;const t=this._data,i=t.length/3;let n=1;for(;n<=i;)if(e<t[3*n])n=2*n;else if(e>t[3*n+1])n=2*n+1;else return t[3*n+2];return 0}};Ju._INSTANCE=null;let zy=Ju;function Hee(){return JSON.parse("[0,0,0,51229,51255,12,44061,44087,12,127462,127487,6,7083,7085,5,47645,47671,12,54813,54839,12,128678,128678,14,3270,3270,5,9919,9923,14,45853,45879,12,49437,49463,12,53021,53047,12,71216,71218,7,128398,128399,14,129360,129374,14,2519,2519,5,4448,4519,9,9742,9742,14,12336,12336,14,44957,44983,12,46749,46775,12,48541,48567,12,50333,50359,12,52125,52151,12,53917,53943,12,69888,69890,5,73018,73018,5,127990,127990,14,128558,128559,14,128759,128760,14,129653,129655,14,2027,2035,5,2891,2892,7,3761,3761,5,6683,6683,5,8293,8293,4,9825,9826,14,9999,9999,14,43452,43453,5,44509,44535,12,45405,45431,12,46301,46327,12,47197,47223,12,48093,48119,12,48989,49015,12,49885,49911,12,50781,50807,12,51677,51703,12,52573,52599,12,53469,53495,12,54365,54391,12,65279,65279,4,70471,70472,7,72145,72147,7,119173,119179,5,127799,127818,14,128240,128244,14,128512,128512,14,128652,128652,14,128721,128722,14,129292,129292,14,129445,129450,14,129734,129743,14,1476,1477,5,2366,2368,7,2750,2752,7,3076,3076,5,3415,3415,5,4141,4144,5,6109,6109,5,6964,6964,5,7394,7400,5,9197,9198,14,9770,9770,14,9877,9877,14,9968,9969,14,10084,10084,14,43052,43052,5,43713,43713,5,44285,44311,12,44733,44759,12,45181,45207,12,45629,45655,12,46077,46103,12,46525,46551,12,46973,46999,12,47421,47447,12,47869,47895,12,48317,48343,12,48765,48791,12,49213,49239,12,49661,49687,12,50109,50135,12,50557,50583,12,51005,51031,12,51453,51479,12,51901,51927,12,52349,52375,12,52797,52823,12,53245,53271,12,53693,53719,12,54141,54167,12,54589,54615,12,55037,55063,12,69506,69509,5,70191,70193,5,70841,70841,7,71463,71467,5,72330,72342,5,94031,94031,5,123628,123631,5,127763,127765,14,127941,127941,14,128043,128062,14,128302,128317,14,128465,128467,14,128539,128539,14,128640,128640,14,128662,128662,14,128703,128703,14,128745,128745,14,129004,129007,14,129329,129330,14,129402,129402,14,129483,129483,14,129686,129704,14,130048,131069,14,173,173,4,1757,1757,1,2200,2207,5,2434,2435,7,2631,2632,5,2817,2817,5,3008,3008,5,3201,3201,5,3387,3388,5,3542,3542,5,3902,3903,7,4190,4192,5,6002,6003,5,6439,6440,5,6765,6770,7,7019,7027,5,7154,7155,7,8205,8205,13,8505,8505,14,9654,9654,14,9757,9757,14,9792,9792,14,9852,9853,14,9890,9894,14,9937,9937,14,9981,9981,14,10035,10036,14,11035,11036,14,42654,42655,5,43346,43347,7,43587,43587,5,44006,44007,7,44173,44199,12,44397,44423,12,44621,44647,12,44845,44871,12,45069,45095,12,45293,45319,12,45517,45543,12,45741,45767,12,45965,45991,12,46189,46215,12,46413,46439,12,46637,46663,12,46861,46887,12,47085,47111,12,47309,47335,12,47533,47559,12,47757,47783,12,47981,48007,12,48205,48231,12,48429,48455,12,48653,48679,12,48877,48903,12,49101,49127,12,49325,49351,12,49549,49575,12,49773,49799,12,49997,50023,12,50221,50247,12,50445,50471,12,50669,50695,12,50893,50919,12,51117,51143,12,51341,51367,12,51565,51591,12,51789,51815,12,52013,52039,12,52237,52263,12,52461,52487,12,52685,52711,12,52909,52935,12,53133,53159,12,53357,53383,12,53581,53607,12,53805,53831,12,54029,54055,12,54253,54279,12,54477,54503,12,54701,54727,12,54925,54951,12,55149,55175,12,68101,68102,5,69762,69762,7,70067,70069,7,70371,70378,5,70720,70721,7,71087,71087,5,71341,71341,5,71995,71996,5,72249,72249,7,72850,72871,5,73109,73109,5,118576,118598,5,121505,121519,5,127245,127247,14,127568,127569,14,127777,127777,14,127872,127891,14,127956,127967,14,128015,128016,14,128110,128172,14,128259,128259,14,128367,128368,14,128424,128424,14,128488,128488,14,128530,128532,14,128550,128551,14,128566,128566,14,128647,128647,14,128656,128656,14,128667,128673,14,128691,128693,14,128715,128715,14,128728,128732,14,128752,128752,14,128765,128767,14,129096,129103,14,129311,129311,14,129344,129349,14,129394,129394,14,129413,129425,14,129466,129471,14,129511,129535,14,129664,129666,14,129719,129722,14,129760,129767,14,917536,917631,5,13,13,2,1160,1161,5,1564,1564,4,1807,1807,1,2085,2087,5,2307,2307,7,2382,2383,7,2497,2500,5,2563,2563,7,2677,2677,5,2763,2764,7,2879,2879,5,2914,2915,5,3021,3021,5,3142,3144,5,3263,3263,5,3285,3286,5,3398,3400,7,3530,3530,5,3633,3633,5,3864,3865,5,3974,3975,5,4155,4156,7,4229,4230,5,5909,5909,7,6078,6085,7,6277,6278,5,6451,6456,7,6744,6750,5,6846,6846,5,6972,6972,5,7074,7077,5,7146,7148,7,7222,7223,5,7416,7417,5,8234,8238,4,8417,8417,5,9000,9000,14,9203,9203,14,9730,9731,14,9748,9749,14,9762,9763,14,9776,9783,14,9800,9811,14,9831,9831,14,9872,9873,14,9882,9882,14,9900,9903,14,9929,9933,14,9941,9960,14,9974,9974,14,9989,9989,14,10006,10006,14,10062,10062,14,10160,10160,14,11647,11647,5,12953,12953,14,43019,43019,5,43232,43249,5,43443,43443,5,43567,43568,7,43696,43696,5,43765,43765,7,44013,44013,5,44117,44143,12,44229,44255,12,44341,44367,12,44453,44479,12,44565,44591,12,44677,44703,12,44789,44815,12,44901,44927,12,45013,45039,12,45125,45151,12,45237,45263,12,45349,45375,12,45461,45487,12,45573,45599,12,45685,45711,12,45797,45823,12,45909,45935,12,46021,46047,12,46133,46159,12,46245,46271,12,46357,46383,12,46469,46495,12,46581,46607,12,46693,46719,12,46805,46831,12,46917,46943,12,47029,47055,12,47141,47167,12,47253,47279,12,47365,47391,12,47477,47503,12,47589,47615,12,47701,47727,12,47813,47839,12,47925,47951,12,48037,48063,12,48149,48175,12,48261,48287,12,48373,48399,12,48485,48511,12,48597,48623,12,48709,48735,12,48821,48847,12,48933,48959,12,49045,49071,12,49157,49183,12,49269,49295,12,49381,49407,12,49493,49519,12,49605,49631,12,49717,49743,12,49829,49855,12,49941,49967,12,50053,50079,12,50165,50191,12,50277,50303,12,50389,50415,12,50501,50527,12,50613,50639,12,50725,50751,12,50837,50863,12,50949,50975,12,51061,51087,12,51173,51199,12,51285,51311,12,51397,51423,12,51509,51535,12,51621,51647,12,51733,51759,12,51845,51871,12,51957,51983,12,52069,52095,12,52181,52207,12,52293,52319,12,52405,52431,12,52517,52543,12,52629,52655,12,52741,52767,12,52853,52879,12,52965,52991,12,53077,53103,12,53189,53215,12,53301,53327,12,53413,53439,12,53525,53551,12,53637,53663,12,53749,53775,12,53861,53887,12,53973,53999,12,54085,54111,12,54197,54223,12,54309,54335,12,54421,54447,12,54533,54559,12,54645,54671,12,54757,54783,12,54869,54895,12,54981,55007,12,55093,55119,12,55243,55291,10,66045,66045,5,68325,68326,5,69688,69702,5,69817,69818,5,69957,69958,7,70089,70092,5,70198,70199,5,70462,70462,5,70502,70508,5,70750,70750,5,70846,70846,7,71100,71101,5,71230,71230,7,71351,71351,5,71737,71738,5,72000,72000,7,72160,72160,5,72273,72278,5,72752,72758,5,72882,72883,5,73031,73031,5,73461,73462,7,94192,94193,7,119149,119149,7,121403,121452,5,122915,122916,5,126980,126980,14,127358,127359,14,127535,127535,14,127759,127759,14,127771,127771,14,127792,127793,14,127825,127867,14,127897,127899,14,127945,127945,14,127985,127986,14,128000,128007,14,128021,128021,14,128066,128100,14,128184,128235,14,128249,128252,14,128266,128276,14,128335,128335,14,128379,128390,14,128407,128419,14,128444,128444,14,128481,128481,14,128499,128499,14,128526,128526,14,128536,128536,14,128543,128543,14,128556,128556,14,128564,128564,14,128577,128580,14,128643,128645,14,128649,128649,14,128654,128654,14,128660,128660,14,128664,128664,14,128675,128675,14,128686,128689,14,128695,128696,14,128705,128709,14,128717,128719,14,128725,128725,14,128736,128741,14,128747,128748,14,128755,128755,14,128762,128762,14,128981,128991,14,129009,129023,14,129160,129167,14,129296,129304,14,129320,129327,14,129340,129342,14,129356,129356,14,129388,129392,14,129399,129400,14,129404,129407,14,129432,129442,14,129454,129455,14,129473,129474,14,129485,129487,14,129648,129651,14,129659,129660,14,129671,129679,14,129709,129711,14,129728,129730,14,129751,129753,14,129776,129782,14,917505,917505,4,917760,917999,5,10,10,3,127,159,4,768,879,5,1471,1471,5,1536,1541,1,1648,1648,5,1767,1768,5,1840,1866,5,2070,2073,5,2137,2139,5,2274,2274,1,2363,2363,7,2377,2380,7,2402,2403,5,2494,2494,5,2507,2508,7,2558,2558,5,2622,2624,7,2641,2641,5,2691,2691,7,2759,2760,5,2786,2787,5,2876,2876,5,2881,2884,5,2901,2902,5,3006,3006,5,3014,3016,7,3072,3072,5,3134,3136,5,3157,3158,5,3260,3260,5,3266,3266,5,3274,3275,7,3328,3329,5,3391,3392,7,3405,3405,5,3457,3457,5,3536,3537,7,3551,3551,5,3636,3642,5,3764,3772,5,3895,3895,5,3967,3967,7,3993,4028,5,4146,4151,5,4182,4183,7,4226,4226,5,4253,4253,5,4957,4959,5,5940,5940,7,6070,6070,7,6087,6088,7,6158,6158,4,6432,6434,5,6448,6449,7,6679,6680,5,6742,6742,5,6754,6754,5,6783,6783,5,6912,6915,5,6966,6970,5,6978,6978,5,7042,7042,7,7080,7081,5,7143,7143,7,7150,7150,7,7212,7219,5,7380,7392,5,7412,7412,5,8203,8203,4,8232,8232,4,8265,8265,14,8400,8412,5,8421,8432,5,8617,8618,14,9167,9167,14,9200,9200,14,9410,9410,14,9723,9726,14,9733,9733,14,9745,9745,14,9752,9752,14,9760,9760,14,9766,9766,14,9774,9774,14,9786,9786,14,9794,9794,14,9823,9823,14,9828,9828,14,9833,9850,14,9855,9855,14,9875,9875,14,9880,9880,14,9885,9887,14,9896,9897,14,9906,9916,14,9926,9927,14,9935,9935,14,9939,9939,14,9962,9962,14,9972,9972,14,9978,9978,14,9986,9986,14,9997,9997,14,10002,10002,14,10017,10017,14,10055,10055,14,10071,10071,14,10133,10135,14,10548,10549,14,11093,11093,14,12330,12333,5,12441,12442,5,42608,42610,5,43010,43010,5,43045,43046,5,43188,43203,7,43302,43309,5,43392,43394,5,43446,43449,5,43493,43493,5,43571,43572,7,43597,43597,7,43703,43704,5,43756,43757,5,44003,44004,7,44009,44010,7,44033,44059,12,44089,44115,12,44145,44171,12,44201,44227,12,44257,44283,12,44313,44339,12,44369,44395,12,44425,44451,12,44481,44507,12,44537,44563,12,44593,44619,12,44649,44675,12,44705,44731,12,44761,44787,12,44817,44843,12,44873,44899,12,44929,44955,12,44985,45011,12,45041,45067,12,45097,45123,12,45153,45179,12,45209,45235,12,45265,45291,12,45321,45347,12,45377,45403,12,45433,45459,12,45489,45515,12,45545,45571,12,45601,45627,12,45657,45683,12,45713,45739,12,45769,45795,12,45825,45851,12,45881,45907,12,45937,45963,12,45993,46019,12,46049,46075,12,46105,46131,12,46161,46187,12,46217,46243,12,46273,46299,12,46329,46355,12,46385,46411,12,46441,46467,12,46497,46523,12,46553,46579,12,46609,46635,12,46665,46691,12,46721,46747,12,46777,46803,12,46833,46859,12,46889,46915,12,46945,46971,12,47001,47027,12,47057,47083,12,47113,47139,12,47169,47195,12,47225,47251,12,47281,47307,12,47337,47363,12,47393,47419,12,47449,47475,12,47505,47531,12,47561,47587,12,47617,47643,12,47673,47699,12,47729,47755,12,47785,47811,12,47841,47867,12,47897,47923,12,47953,47979,12,48009,48035,12,48065,48091,12,48121,48147,12,48177,48203,12,48233,48259,12,48289,48315,12,48345,48371,12,48401,48427,12,48457,48483,12,48513,48539,12,48569,48595,12,48625,48651,12,48681,48707,12,48737,48763,12,48793,48819,12,48849,48875,12,48905,48931,12,48961,48987,12,49017,49043,12,49073,49099,12,49129,49155,12,49185,49211,12,49241,49267,12,49297,49323,12,49353,49379,12,49409,49435,12,49465,49491,12,49521,49547,12,49577,49603,12,49633,49659,12,49689,49715,12,49745,49771,12,49801,49827,12,49857,49883,12,49913,49939,12,49969,49995,12,50025,50051,12,50081,50107,12,50137,50163,12,50193,50219,12,50249,50275,12,50305,50331,12,50361,50387,12,50417,50443,12,50473,50499,12,50529,50555,12,50585,50611,12,50641,50667,12,50697,50723,12,50753,50779,12,50809,50835,12,50865,50891,12,50921,50947,12,50977,51003,12,51033,51059,12,51089,51115,12,51145,51171,12,51201,51227,12,51257,51283,12,51313,51339,12,51369,51395,12,51425,51451,12,51481,51507,12,51537,51563,12,51593,51619,12,51649,51675,12,51705,51731,12,51761,51787,12,51817,51843,12,51873,51899,12,51929,51955,12,51985,52011,12,52041,52067,12,52097,52123,12,52153,52179,12,52209,52235,12,52265,52291,12,52321,52347,12,52377,52403,12,52433,52459,12,52489,52515,12,52545,52571,12,52601,52627,12,52657,52683,12,52713,52739,12,52769,52795,12,52825,52851,12,52881,52907,12,52937,52963,12,52993,53019,12,53049,53075,12,53105,53131,12,53161,53187,12,53217,53243,12,53273,53299,12,53329,53355,12,53385,53411,12,53441,53467,12,53497,53523,12,53553,53579,12,53609,53635,12,53665,53691,12,53721,53747,12,53777,53803,12,53833,53859,12,53889,53915,12,53945,53971,12,54001,54027,12,54057,54083,12,54113,54139,12,54169,54195,12,54225,54251,12,54281,54307,12,54337,54363,12,54393,54419,12,54449,54475,12,54505,54531,12,54561,54587,12,54617,54643,12,54673,54699,12,54729,54755,12,54785,54811,12,54841,54867,12,54897,54923,12,54953,54979,12,55009,55035,12,55065,55091,12,55121,55147,12,55177,55203,12,65024,65039,5,65520,65528,4,66422,66426,5,68152,68154,5,69291,69292,5,69633,69633,5,69747,69748,5,69811,69814,5,69826,69826,5,69932,69932,7,70016,70017,5,70079,70080,7,70095,70095,5,70196,70196,5,70367,70367,5,70402,70403,7,70464,70464,5,70487,70487,5,70709,70711,7,70725,70725,7,70833,70834,7,70843,70844,7,70849,70849,7,71090,71093,5,71103,71104,5,71227,71228,7,71339,71339,5,71344,71349,5,71458,71461,5,71727,71735,5,71985,71989,7,71998,71998,5,72002,72002,7,72154,72155,5,72193,72202,5,72251,72254,5,72281,72283,5,72344,72345,5,72766,72766,7,72874,72880,5,72885,72886,5,73023,73029,5,73104,73105,5,73111,73111,5,92912,92916,5,94095,94098,5,113824,113827,4,119142,119142,7,119155,119162,4,119362,119364,5,121476,121476,5,122888,122904,5,123184,123190,5,125252,125258,5,127183,127183,14,127340,127343,14,127377,127386,14,127491,127503,14,127548,127551,14,127744,127756,14,127761,127761,14,127769,127769,14,127773,127774,14,127780,127788,14,127796,127797,14,127820,127823,14,127869,127869,14,127894,127895,14,127902,127903,14,127943,127943,14,127947,127950,14,127972,127972,14,127988,127988,14,127992,127994,14,128009,128011,14,128019,128019,14,128023,128041,14,128064,128064,14,128102,128107,14,128174,128181,14,128238,128238,14,128246,128247,14,128254,128254,14,128264,128264,14,128278,128299,14,128329,128330,14,128348,128359,14,128371,128377,14,128392,128393,14,128401,128404,14,128421,128421,14,128433,128434,14,128450,128452,14,128476,128478,14,128483,128483,14,128495,128495,14,128506,128506,14,128519,128520,14,128528,128528,14,128534,128534,14,128538,128538,14,128540,128542,14,128544,128549,14,128552,128555,14,128557,128557,14,128560,128563,14,128565,128565,14,128567,128576,14,128581,128591,14,128641,128642,14,128646,128646,14,128648,128648,14,128650,128651,14,128653,128653,14,128655,128655,14,128657,128659,14,128661,128661,14,128663,128663,14,128665,128666,14,128674,128674,14,128676,128677,14,128679,128685,14,128690,128690,14,128694,128694,14,128697,128702,14,128704,128704,14,128710,128714,14,128716,128716,14,128720,128720,14,128723,128724,14,128726,128727,14,128733,128735,14,128742,128744,14,128746,128746,14,128749,128751,14,128753,128754,14,128756,128758,14,128761,128761,14,128763,128764,14,128884,128895,14,128992,129003,14,129008,129008,14,129036,129039,14,129114,129119,14,129198,129279,14,129293,129295,14,129305,129310,14,129312,129319,14,129328,129328,14,129331,129338,14,129343,129343,14,129351,129355,14,129357,129359,14,129375,129387,14,129393,129393,14,129395,129398,14,129401,129401,14,129403,129403,14,129408,129412,14,129426,129431,14,129443,129444,14,129451,129453,14,129456,129465,14,129472,129472,14,129475,129482,14,129484,129484,14,129488,129510,14,129536,129647,14,129652,129652,14,129656,129658,14,129661,129663,14,129667,129670,14,129680,129685,14,129705,129708,14,129712,129718,14,129723,129727,14,129731,129733,14,129744,129750,14,129754,129759,14,129768,129775,14,129783,129791,14,917504,917504,4,917506,917535,4,917632,917759,4,918000,921599,4,0,9,4,11,12,4,14,31,4,169,169,14,174,174,14,1155,1159,5,1425,1469,5,1473,1474,5,1479,1479,5,1552,1562,5,1611,1631,5,1750,1756,5,1759,1764,5,1770,1773,5,1809,1809,5,1958,1968,5,2045,2045,5,2075,2083,5,2089,2093,5,2192,2193,1,2250,2273,5,2275,2306,5,2362,2362,5,2364,2364,5,2369,2376,5,2381,2381,5,2385,2391,5,2433,2433,5,2492,2492,5,2495,2496,7,2503,2504,7,2509,2509,5,2530,2531,5,2561,2562,5,2620,2620,5,2625,2626,5,2635,2637,5,2672,2673,5,2689,2690,5,2748,2748,5,2753,2757,5,2761,2761,7,2765,2765,5,2810,2815,5,2818,2819,7,2878,2878,5,2880,2880,7,2887,2888,7,2893,2893,5,2903,2903,5,2946,2946,5,3007,3007,7,3009,3010,7,3018,3020,7,3031,3031,5,3073,3075,7,3132,3132,5,3137,3140,7,3146,3149,5,3170,3171,5,3202,3203,7,3262,3262,7,3264,3265,7,3267,3268,7,3271,3272,7,3276,3277,5,3298,3299,5,3330,3331,7,3390,3390,5,3393,3396,5,3402,3404,7,3406,3406,1,3426,3427,5,3458,3459,7,3535,3535,5,3538,3540,5,3544,3550,7,3570,3571,7,3635,3635,7,3655,3662,5,3763,3763,7,3784,3789,5,3893,3893,5,3897,3897,5,3953,3966,5,3968,3972,5,3981,3991,5,4038,4038,5,4145,4145,7,4153,4154,5,4157,4158,5,4184,4185,5,4209,4212,5,4228,4228,7,4237,4237,5,4352,4447,8,4520,4607,10,5906,5908,5,5938,5939,5,5970,5971,5,6068,6069,5,6071,6077,5,6086,6086,5,6089,6099,5,6155,6157,5,6159,6159,5,6313,6313,5,6435,6438,7,6441,6443,7,6450,6450,5,6457,6459,5,6681,6682,7,6741,6741,7,6743,6743,7,6752,6752,5,6757,6764,5,6771,6780,5,6832,6845,5,6847,6862,5,6916,6916,7,6965,6965,5,6971,6971,7,6973,6977,7,6979,6980,7,7040,7041,5,7073,7073,7,7078,7079,7,7082,7082,7,7142,7142,5,7144,7145,5,7149,7149,5,7151,7153,5,7204,7211,7,7220,7221,7,7376,7378,5,7393,7393,7,7405,7405,5,7415,7415,7,7616,7679,5,8204,8204,5,8206,8207,4,8233,8233,4,8252,8252,14,8288,8292,4,8294,8303,4,8413,8416,5,8418,8420,5,8482,8482,14,8596,8601,14,8986,8987,14,9096,9096,14,9193,9196,14,9199,9199,14,9201,9202,14,9208,9210,14,9642,9643,14,9664,9664,14,9728,9729,14,9732,9732,14,9735,9741,14,9743,9744,14,9746,9746,14,9750,9751,14,9753,9756,14,9758,9759,14,9761,9761,14,9764,9765,14,9767,9769,14,9771,9773,14,9775,9775,14,9784,9785,14,9787,9791,14,9793,9793,14,9795,9799,14,9812,9822,14,9824,9824,14,9827,9827,14,9829,9830,14,9832,9832,14,9851,9851,14,9854,9854,14,9856,9861,14,9874,9874,14,9876,9876,14,9878,9879,14,9881,9881,14,9883,9884,14,9888,9889,14,9895,9895,14,9898,9899,14,9904,9905,14,9917,9918,14,9924,9925,14,9928,9928,14,9934,9934,14,9936,9936,14,9938,9938,14,9940,9940,14,9961,9961,14,9963,9967,14,9970,9971,14,9973,9973,14,9975,9977,14,9979,9980,14,9982,9985,14,9987,9988,14,9992,9996,14,9998,9998,14,10000,10001,14,10004,10004,14,10013,10013,14,10024,10024,14,10052,10052,14,10060,10060,14,10067,10069,14,10083,10083,14,10085,10087,14,10145,10145,14,10175,10175,14,11013,11015,14,11088,11088,14,11503,11505,5,11744,11775,5,12334,12335,5,12349,12349,14,12951,12951,14,42607,42607,5,42612,42621,5,42736,42737,5,43014,43014,5,43043,43044,7,43047,43047,7,43136,43137,7,43204,43205,5,43263,43263,5,43335,43345,5,43360,43388,8,43395,43395,7,43444,43445,7,43450,43451,7,43454,43456,7,43561,43566,5,43569,43570,5,43573,43574,5,43596,43596,5,43644,43644,5,43698,43700,5,43710,43711,5,43755,43755,7,43758,43759,7,43766,43766,5,44005,44005,5,44008,44008,5,44012,44012,7,44032,44032,11,44060,44060,11,44088,44088,11,44116,44116,11,44144,44144,11,44172,44172,11,44200,44200,11,44228,44228,11,44256,44256,11,44284,44284,11,44312,44312,11,44340,44340,11,44368,44368,11,44396,44396,11,44424,44424,11,44452,44452,11,44480,44480,11,44508,44508,11,44536,44536,11,44564,44564,11,44592,44592,11,44620,44620,11,44648,44648,11,44676,44676,11,44704,44704,11,44732,44732,11,44760,44760,11,44788,44788,11,44816,44816,11,44844,44844,11,44872,44872,11,44900,44900,11,44928,44928,11,44956,44956,11,44984,44984,11,45012,45012,11,45040,45040,11,45068,45068,11,45096,45096,11,45124,45124,11,45152,45152,11,45180,45180,11,45208,45208,11,45236,45236,11,45264,45264,11,45292,45292,11,45320,45320,11,45348,45348,11,45376,45376,11,45404,45404,11,45432,45432,11,45460,45460,11,45488,45488,11,45516,45516,11,45544,45544,11,45572,45572,11,45600,45600,11,45628,45628,11,45656,45656,11,45684,45684,11,45712,45712,11,45740,45740,11,45768,45768,11,45796,45796,11,45824,45824,11,45852,45852,11,45880,45880,11,45908,45908,11,45936,45936,11,45964,45964,11,45992,45992,11,46020,46020,11,46048,46048,11,46076,46076,11,46104,46104,11,46132,46132,11,46160,46160,11,46188,46188,11,46216,46216,11,46244,46244,11,46272,46272,11,46300,46300,11,46328,46328,11,46356,46356,11,46384,46384,11,46412,46412,11,46440,46440,11,46468,46468,11,46496,46496,11,46524,46524,11,46552,46552,11,46580,46580,11,46608,46608,11,46636,46636,11,46664,46664,11,46692,46692,11,46720,46720,11,46748,46748,11,46776,46776,11,46804,46804,11,46832,46832,11,46860,46860,11,46888,46888,11,46916,46916,11,46944,46944,11,46972,46972,11,47000,47000,11,47028,47028,11,47056,47056,11,47084,47084,11,47112,47112,11,47140,47140,11,47168,47168,11,47196,47196,11,47224,47224,11,47252,47252,11,47280,47280,11,47308,47308,11,47336,47336,11,47364,47364,11,47392,47392,11,47420,47420,11,47448,47448,11,47476,47476,11,47504,47504,11,47532,47532,11,47560,47560,11,47588,47588,11,47616,47616,11,47644,47644,11,47672,47672,11,47700,47700,11,47728,47728,11,47756,47756,11,47784,47784,11,47812,47812,11,47840,47840,11,47868,47868,11,47896,47896,11,47924,47924,11,47952,47952,11,47980,47980,11,48008,48008,11,48036,48036,11,48064,48064,11,48092,48092,11,48120,48120,11,48148,48148,11,48176,48176,11,48204,48204,11,48232,48232,11,48260,48260,11,48288,48288,11,48316,48316,11,48344,48344,11,48372,48372,11,48400,48400,11,48428,48428,11,48456,48456,11,48484,48484,11,48512,48512,11,48540,48540,11,48568,48568,11,48596,48596,11,48624,48624,11,48652,48652,11,48680,48680,11,48708,48708,11,48736,48736,11,48764,48764,11,48792,48792,11,48820,48820,11,48848,48848,11,48876,48876,11,48904,48904,11,48932,48932,11,48960,48960,11,48988,48988,11,49016,49016,11,49044,49044,11,49072,49072,11,49100,49100,11,49128,49128,11,49156,49156,11,49184,49184,11,49212,49212,11,49240,49240,11,49268,49268,11,49296,49296,11,49324,49324,11,49352,49352,11,49380,49380,11,49408,49408,11,49436,49436,11,49464,49464,11,49492,49492,11,49520,49520,11,49548,49548,11,49576,49576,11,49604,49604,11,49632,49632,11,49660,49660,11,49688,49688,11,49716,49716,11,49744,49744,11,49772,49772,11,49800,49800,11,49828,49828,11,49856,49856,11,49884,49884,11,49912,49912,11,49940,49940,11,49968,49968,11,49996,49996,11,50024,50024,11,50052,50052,11,50080,50080,11,50108,50108,11,50136,50136,11,50164,50164,11,50192,50192,11,50220,50220,11,50248,50248,11,50276,50276,11,50304,50304,11,50332,50332,11,50360,50360,11,50388,50388,11,50416,50416,11,50444,50444,11,50472,50472,11,50500,50500,11,50528,50528,11,50556,50556,11,50584,50584,11,50612,50612,11,50640,50640,11,50668,50668,11,50696,50696,11,50724,50724,11,50752,50752,11,50780,50780,11,50808,50808,11,50836,50836,11,50864,50864,11,50892,50892,11,50920,50920,11,50948,50948,11,50976,50976,11,51004,51004,11,51032,51032,11,51060,51060,11,51088,51088,11,51116,51116,11,51144,51144,11,51172,51172,11,51200,51200,11,51228,51228,11,51256,51256,11,51284,51284,11,51312,51312,11,51340,51340,11,51368,51368,11,51396,51396,11,51424,51424,11,51452,51452,11,51480,51480,11,51508,51508,11,51536,51536,11,51564,51564,11,51592,51592,11,51620,51620,11,51648,51648,11,51676,51676,11,51704,51704,11,51732,51732,11,51760,51760,11,51788,51788,11,51816,51816,11,51844,51844,11,51872,51872,11,51900,51900,11,51928,51928,11,51956,51956,11,51984,51984,11,52012,52012,11,52040,52040,11,52068,52068,11,52096,52096,11,52124,52124,11,52152,52152,11,52180,52180,11,52208,52208,11,52236,52236,11,52264,52264,11,52292,52292,11,52320,52320,11,52348,52348,11,52376,52376,11,52404,52404,11,52432,52432,11,52460,52460,11,52488,52488,11,52516,52516,11,52544,52544,11,52572,52572,11,52600,52600,11,52628,52628,11,52656,52656,11,52684,52684,11,52712,52712,11,52740,52740,11,52768,52768,11,52796,52796,11,52824,52824,11,52852,52852,11,52880,52880,11,52908,52908,11,52936,52936,11,52964,52964,11,52992,52992,11,53020,53020,11,53048,53048,11,53076,53076,11,53104,53104,11,53132,53132,11,53160,53160,11,53188,53188,11,53216,53216,11,53244,53244,11,53272,53272,11,53300,53300,11,53328,53328,11,53356,53356,11,53384,53384,11,53412,53412,11,53440,53440,11,53468,53468,11,53496,53496,11,53524,53524,11,53552,53552,11,53580,53580,11,53608,53608,11,53636,53636,11,53664,53664,11,53692,53692,11,53720,53720,11,53748,53748,11,53776,53776,11,53804,53804,11,53832,53832,11,53860,53860,11,53888,53888,11,53916,53916,11,53944,53944,11,53972,53972,11,54000,54000,11,54028,54028,11,54056,54056,11,54084,54084,11,54112,54112,11,54140,54140,11,54168,54168,11,54196,54196,11,54224,54224,11,54252,54252,11,54280,54280,11,54308,54308,11,54336,54336,11,54364,54364,11,54392,54392,11,54420,54420,11,54448,54448,11,54476,54476,11,54504,54504,11,54532,54532,11,54560,54560,11,54588,54588,11,54616,54616,11,54644,54644,11,54672,54672,11,54700,54700,11,54728,54728,11,54756,54756,11,54784,54784,11,54812,54812,11,54840,54840,11,54868,54868,11,54896,54896,11,54924,54924,11,54952,54952,11,54980,54980,11,55008,55008,11,55036,55036,11,55064,55064,11,55092,55092,11,55120,55120,11,55148,55148,11,55176,55176,11,55216,55238,9,64286,64286,5,65056,65071,5,65438,65439,5,65529,65531,4,66272,66272,5,68097,68099,5,68108,68111,5,68159,68159,5,68900,68903,5,69446,69456,5,69632,69632,7,69634,69634,7,69744,69744,5,69759,69761,5,69808,69810,7,69815,69816,7,69821,69821,1,69837,69837,1,69927,69931,5,69933,69940,5,70003,70003,5,70018,70018,7,70070,70078,5,70082,70083,1,70094,70094,7,70188,70190,7,70194,70195,7,70197,70197,7,70206,70206,5,70368,70370,7,70400,70401,5,70459,70460,5,70463,70463,7,70465,70468,7,70475,70477,7,70498,70499,7,70512,70516,5,70712,70719,5,70722,70724,5,70726,70726,5,70832,70832,5,70835,70840,5,70842,70842,5,70845,70845,5,70847,70848,5,70850,70851,5,71088,71089,7,71096,71099,7,71102,71102,7,71132,71133,5,71219,71226,5,71229,71229,5,71231,71232,5,71340,71340,7,71342,71343,7,71350,71350,7,71453,71455,5,71462,71462,7,71724,71726,7,71736,71736,7,71984,71984,5,71991,71992,7,71997,71997,7,71999,71999,1,72001,72001,1,72003,72003,5,72148,72151,5,72156,72159,7,72164,72164,7,72243,72248,5,72250,72250,1,72263,72263,5,72279,72280,7,72324,72329,1,72343,72343,7,72751,72751,7,72760,72765,5,72767,72767,5,72873,72873,7,72881,72881,7,72884,72884,7,73009,73014,5,73020,73021,5,73030,73030,1,73098,73102,7,73107,73108,7,73110,73110,7,73459,73460,5,78896,78904,4,92976,92982,5,94033,94087,7,94180,94180,5,113821,113822,5,118528,118573,5,119141,119141,5,119143,119145,5,119150,119154,5,119163,119170,5,119210,119213,5,121344,121398,5,121461,121461,5,121499,121503,5,122880,122886,5,122907,122913,5,122918,122922,5,123566,123566,5,125136,125142,5,126976,126979,14,126981,127182,14,127184,127231,14,127279,127279,14,127344,127345,14,127374,127374,14,127405,127461,14,127489,127490,14,127514,127514,14,127538,127546,14,127561,127567,14,127570,127743,14,127757,127758,14,127760,127760,14,127762,127762,14,127766,127768,14,127770,127770,14,127772,127772,14,127775,127776,14,127778,127779,14,127789,127791,14,127794,127795,14,127798,127798,14,127819,127819,14,127824,127824,14,127868,127868,14,127870,127871,14,127892,127893,14,127896,127896,14,127900,127901,14,127904,127940,14,127942,127942,14,127944,127944,14,127946,127946,14,127951,127955,14,127968,127971,14,127973,127984,14,127987,127987,14,127989,127989,14,127991,127991,14,127995,127999,5,128008,128008,14,128012,128014,14,128017,128018,14,128020,128020,14,128022,128022,14,128042,128042,14,128063,128063,14,128065,128065,14,128101,128101,14,128108,128109,14,128173,128173,14,128182,128183,14,128236,128237,14,128239,128239,14,128245,128245,14,128248,128248,14,128253,128253,14,128255,128258,14,128260,128263,14,128265,128265,14,128277,128277,14,128300,128301,14,128326,128328,14,128331,128334,14,128336,128347,14,128360,128366,14,128369,128370,14,128378,128378,14,128391,128391,14,128394,128397,14,128400,128400,14,128405,128406,14,128420,128420,14,128422,128423,14,128425,128432,14,128435,128443,14,128445,128449,14,128453,128464,14,128468,128475,14,128479,128480,14,128482,128482,14,128484,128487,14,128489,128494,14,128496,128498,14,128500,128505,14,128507,128511,14,128513,128518,14,128521,128525,14,128527,128527,14,128529,128529,14,128533,128533,14,128535,128535,14,128537,128537,14]")}function Vee(o,e){if(o===0)return 0;const t=zee(o,e);if(t!==void 0)return t;const i=new Q4(e,o);return i.prevCodePoint(),i.offset}function zee(o,e){const t=new Q4(e,o);let i=t.prevCodePoint();for(;Uee(i)||i===65039||i===8419;){if(t.offset===0)return;i=t.prevCodePoint()}if(!e5(i))return;let n=t.offset;return n>0&&t.prevCodePoint()===8205&&(n=t.offset),n}function Uee(o){return 127995<=o&&o<=127999}const rU=" ",dl=class dl{static getInstance(e){return dl.cache.get(Array.from(e))}static getLocales(){return dl._locales.value}constructor(e){this.confusableDictionary=e}isAmbiguous(e){return this.confusableDictionary.has(e)}getPrimaryConfusable(e){return this.confusableDictionary.get(e)}getConfusableCodePoints(){return new Set(this.confusableDictionary.keys())}};dl.ambiguousCharacterData=new Oa(()=>JSON.parse('{"_common":[8232,32,8233,32,5760,32,8192,32,8193,32,8194,32,8195,32,8196,32,8197,32,8198,32,8200,32,8201,32,8202,32,8287,32,8199,32,8239,32,2042,95,65101,95,65102,95,65103,95,8208,45,8209,45,8210,45,65112,45,1748,45,8259,45,727,45,8722,45,10134,45,11450,45,1549,44,1643,44,8218,44,184,44,42233,44,894,59,2307,58,2691,58,1417,58,1795,58,1796,58,5868,58,65072,58,6147,58,6153,58,8282,58,1475,58,760,58,42889,58,8758,58,720,58,42237,58,451,33,11601,33,660,63,577,63,2429,63,5038,63,42731,63,119149,46,8228,46,1793,46,1794,46,42510,46,68176,46,1632,46,1776,46,42232,46,1373,96,65287,96,8219,96,8242,96,1370,96,1523,96,8175,96,65344,96,900,96,8189,96,8125,96,8127,96,8190,96,697,96,884,96,712,96,714,96,715,96,756,96,699,96,701,96,700,96,702,96,42892,96,1497,96,2036,96,2037,96,5194,96,5836,96,94033,96,94034,96,65339,91,10088,40,10098,40,12308,40,64830,40,65341,93,10089,41,10099,41,12309,41,64831,41,10100,123,119060,123,10101,125,65342,94,8270,42,1645,42,8727,42,66335,42,5941,47,8257,47,8725,47,8260,47,9585,47,10187,47,10744,47,119354,47,12755,47,12339,47,11462,47,20031,47,12035,47,65340,92,65128,92,8726,92,10189,92,10741,92,10745,92,119311,92,119355,92,12756,92,20022,92,12034,92,42872,38,708,94,710,94,5869,43,10133,43,66203,43,8249,60,10094,60,706,60,119350,60,5176,60,5810,60,5120,61,11840,61,12448,61,42239,61,8250,62,10095,62,707,62,119351,62,5171,62,94015,62,8275,126,732,126,8128,126,8764,126,65372,124,65293,45,120784,50,120794,50,120804,50,120814,50,120824,50,130034,50,42842,50,423,50,1000,50,42564,50,5311,50,42735,50,119302,51,120785,51,120795,51,120805,51,120815,51,120825,51,130035,51,42923,51,540,51,439,51,42858,51,11468,51,1248,51,94011,51,71882,51,120786,52,120796,52,120806,52,120816,52,120826,52,130036,52,5070,52,71855,52,120787,53,120797,53,120807,53,120817,53,120827,53,130037,53,444,53,71867,53,120788,54,120798,54,120808,54,120818,54,120828,54,130038,54,11474,54,5102,54,71893,54,119314,55,120789,55,120799,55,120809,55,120819,55,120829,55,130039,55,66770,55,71878,55,2819,56,2538,56,2666,56,125131,56,120790,56,120800,56,120810,56,120820,56,120830,56,130040,56,547,56,546,56,66330,56,2663,57,2920,57,2541,57,3437,57,120791,57,120801,57,120811,57,120821,57,120831,57,130041,57,42862,57,11466,57,71884,57,71852,57,71894,57,9082,97,65345,97,119834,97,119886,97,119938,97,119990,97,120042,97,120094,97,120146,97,120198,97,120250,97,120302,97,120354,97,120406,97,120458,97,593,97,945,97,120514,97,120572,97,120630,97,120688,97,120746,97,65313,65,119808,65,119860,65,119912,65,119964,65,120016,65,120068,65,120120,65,120172,65,120224,65,120276,65,120328,65,120380,65,120432,65,913,65,120488,65,120546,65,120604,65,120662,65,120720,65,5034,65,5573,65,42222,65,94016,65,66208,65,119835,98,119887,98,119939,98,119991,98,120043,98,120095,98,120147,98,120199,98,120251,98,120303,98,120355,98,120407,98,120459,98,388,98,5071,98,5234,98,5551,98,65314,66,8492,66,119809,66,119861,66,119913,66,120017,66,120069,66,120121,66,120173,66,120225,66,120277,66,120329,66,120381,66,120433,66,42932,66,914,66,120489,66,120547,66,120605,66,120663,66,120721,66,5108,66,5623,66,42192,66,66178,66,66209,66,66305,66,65347,99,8573,99,119836,99,119888,99,119940,99,119992,99,120044,99,120096,99,120148,99,120200,99,120252,99,120304,99,120356,99,120408,99,120460,99,7428,99,1010,99,11429,99,43951,99,66621,99,128844,67,71922,67,71913,67,65315,67,8557,67,8450,67,8493,67,119810,67,119862,67,119914,67,119966,67,120018,67,120174,67,120226,67,120278,67,120330,67,120382,67,120434,67,1017,67,11428,67,5087,67,42202,67,66210,67,66306,67,66581,67,66844,67,8574,100,8518,100,119837,100,119889,100,119941,100,119993,100,120045,100,120097,100,120149,100,120201,100,120253,100,120305,100,120357,100,120409,100,120461,100,1281,100,5095,100,5231,100,42194,100,8558,68,8517,68,119811,68,119863,68,119915,68,119967,68,120019,68,120071,68,120123,68,120175,68,120227,68,120279,68,120331,68,120383,68,120435,68,5024,68,5598,68,5610,68,42195,68,8494,101,65349,101,8495,101,8519,101,119838,101,119890,101,119942,101,120046,101,120098,101,120150,101,120202,101,120254,101,120306,101,120358,101,120410,101,120462,101,43826,101,1213,101,8959,69,65317,69,8496,69,119812,69,119864,69,119916,69,120020,69,120072,69,120124,69,120176,69,120228,69,120280,69,120332,69,120384,69,120436,69,917,69,120492,69,120550,69,120608,69,120666,69,120724,69,11577,69,5036,69,42224,69,71846,69,71854,69,66182,69,119839,102,119891,102,119943,102,119995,102,120047,102,120099,102,120151,102,120203,102,120255,102,120307,102,120359,102,120411,102,120463,102,43829,102,42905,102,383,102,7837,102,1412,102,119315,70,8497,70,119813,70,119865,70,119917,70,120021,70,120073,70,120125,70,120177,70,120229,70,120281,70,120333,70,120385,70,120437,70,42904,70,988,70,120778,70,5556,70,42205,70,71874,70,71842,70,66183,70,66213,70,66853,70,65351,103,8458,103,119840,103,119892,103,119944,103,120048,103,120100,103,120152,103,120204,103,120256,103,120308,103,120360,103,120412,103,120464,103,609,103,7555,103,397,103,1409,103,119814,71,119866,71,119918,71,119970,71,120022,71,120074,71,120126,71,120178,71,120230,71,120282,71,120334,71,120386,71,120438,71,1292,71,5056,71,5107,71,42198,71,65352,104,8462,104,119841,104,119945,104,119997,104,120049,104,120101,104,120153,104,120205,104,120257,104,120309,104,120361,104,120413,104,120465,104,1211,104,1392,104,5058,104,65320,72,8459,72,8460,72,8461,72,119815,72,119867,72,119919,72,120023,72,120179,72,120231,72,120283,72,120335,72,120387,72,120439,72,919,72,120494,72,120552,72,120610,72,120668,72,120726,72,11406,72,5051,72,5500,72,42215,72,66255,72,731,105,9075,105,65353,105,8560,105,8505,105,8520,105,119842,105,119894,105,119946,105,119998,105,120050,105,120102,105,120154,105,120206,105,120258,105,120310,105,120362,105,120414,105,120466,105,120484,105,618,105,617,105,953,105,8126,105,890,105,120522,105,120580,105,120638,105,120696,105,120754,105,1110,105,42567,105,1231,105,43893,105,5029,105,71875,105,65354,106,8521,106,119843,106,119895,106,119947,106,119999,106,120051,106,120103,106,120155,106,120207,106,120259,106,120311,106,120363,106,120415,106,120467,106,1011,106,1112,106,65322,74,119817,74,119869,74,119921,74,119973,74,120025,74,120077,74,120129,74,120181,74,120233,74,120285,74,120337,74,120389,74,120441,74,42930,74,895,74,1032,74,5035,74,5261,74,42201,74,119844,107,119896,107,119948,107,120000,107,120052,107,120104,107,120156,107,120208,107,120260,107,120312,107,120364,107,120416,107,120468,107,8490,75,65323,75,119818,75,119870,75,119922,75,119974,75,120026,75,120078,75,120130,75,120182,75,120234,75,120286,75,120338,75,120390,75,120442,75,922,75,120497,75,120555,75,120613,75,120671,75,120729,75,11412,75,5094,75,5845,75,42199,75,66840,75,1472,108,8739,73,9213,73,65512,73,1633,108,1777,73,66336,108,125127,108,120783,73,120793,73,120803,73,120813,73,120823,73,130033,73,65321,73,8544,73,8464,73,8465,73,119816,73,119868,73,119920,73,120024,73,120128,73,120180,73,120232,73,120284,73,120336,73,120388,73,120440,73,65356,108,8572,73,8467,108,119845,108,119897,108,119949,108,120001,108,120053,108,120105,73,120157,73,120209,73,120261,73,120313,73,120365,73,120417,73,120469,73,448,73,120496,73,120554,73,120612,73,120670,73,120728,73,11410,73,1030,73,1216,73,1493,108,1503,108,1575,108,126464,108,126592,108,65166,108,65165,108,1994,108,11599,73,5825,73,42226,73,93992,73,66186,124,66313,124,119338,76,8556,76,8466,76,119819,76,119871,76,119923,76,120027,76,120079,76,120131,76,120183,76,120235,76,120287,76,120339,76,120391,76,120443,76,11472,76,5086,76,5290,76,42209,76,93974,76,71843,76,71858,76,66587,76,66854,76,65325,77,8559,77,8499,77,119820,77,119872,77,119924,77,120028,77,120080,77,120132,77,120184,77,120236,77,120288,77,120340,77,120392,77,120444,77,924,77,120499,77,120557,77,120615,77,120673,77,120731,77,1018,77,11416,77,5047,77,5616,77,5846,77,42207,77,66224,77,66321,77,119847,110,119899,110,119951,110,120003,110,120055,110,120107,110,120159,110,120211,110,120263,110,120315,110,120367,110,120419,110,120471,110,1400,110,1404,110,65326,78,8469,78,119821,78,119873,78,119925,78,119977,78,120029,78,120081,78,120185,78,120237,78,120289,78,120341,78,120393,78,120445,78,925,78,120500,78,120558,78,120616,78,120674,78,120732,78,11418,78,42208,78,66835,78,3074,111,3202,111,3330,111,3458,111,2406,111,2662,111,2790,111,3046,111,3174,111,3302,111,3430,111,3664,111,3792,111,4160,111,1637,111,1781,111,65359,111,8500,111,119848,111,119900,111,119952,111,120056,111,120108,111,120160,111,120212,111,120264,111,120316,111,120368,111,120420,111,120472,111,7439,111,7441,111,43837,111,959,111,120528,111,120586,111,120644,111,120702,111,120760,111,963,111,120532,111,120590,111,120648,111,120706,111,120764,111,11423,111,4351,111,1413,111,1505,111,1607,111,126500,111,126564,111,126596,111,65259,111,65260,111,65258,111,65257,111,1726,111,64428,111,64429,111,64427,111,64426,111,1729,111,64424,111,64425,111,64423,111,64422,111,1749,111,3360,111,4125,111,66794,111,71880,111,71895,111,66604,111,1984,79,2534,79,2918,79,12295,79,70864,79,71904,79,120782,79,120792,79,120802,79,120812,79,120822,79,130032,79,65327,79,119822,79,119874,79,119926,79,119978,79,120030,79,120082,79,120134,79,120186,79,120238,79,120290,79,120342,79,120394,79,120446,79,927,79,120502,79,120560,79,120618,79,120676,79,120734,79,11422,79,1365,79,11604,79,4816,79,2848,79,66754,79,42227,79,71861,79,66194,79,66219,79,66564,79,66838,79,9076,112,65360,112,119849,112,119901,112,119953,112,120005,112,120057,112,120109,112,120161,112,120213,112,120265,112,120317,112,120369,112,120421,112,120473,112,961,112,120530,112,120544,112,120588,112,120602,112,120646,112,120660,112,120704,112,120718,112,120762,112,120776,112,11427,112,65328,80,8473,80,119823,80,119875,80,119927,80,119979,80,120031,80,120083,80,120187,80,120239,80,120291,80,120343,80,120395,80,120447,80,929,80,120504,80,120562,80,120620,80,120678,80,120736,80,11426,80,5090,80,5229,80,42193,80,66197,80,119850,113,119902,113,119954,113,120006,113,120058,113,120110,113,120162,113,120214,113,120266,113,120318,113,120370,113,120422,113,120474,113,1307,113,1379,113,1382,113,8474,81,119824,81,119876,81,119928,81,119980,81,120032,81,120084,81,120188,81,120240,81,120292,81,120344,81,120396,81,120448,81,11605,81,119851,114,119903,114,119955,114,120007,114,120059,114,120111,114,120163,114,120215,114,120267,114,120319,114,120371,114,120423,114,120475,114,43847,114,43848,114,7462,114,11397,114,43905,114,119318,82,8475,82,8476,82,8477,82,119825,82,119877,82,119929,82,120033,82,120189,82,120241,82,120293,82,120345,82,120397,82,120449,82,422,82,5025,82,5074,82,66740,82,5511,82,42211,82,94005,82,65363,115,119852,115,119904,115,119956,115,120008,115,120060,115,120112,115,120164,115,120216,115,120268,115,120320,115,120372,115,120424,115,120476,115,42801,115,445,115,1109,115,43946,115,71873,115,66632,115,65331,83,119826,83,119878,83,119930,83,119982,83,120034,83,120086,83,120138,83,120190,83,120242,83,120294,83,120346,83,120398,83,120450,83,1029,83,1359,83,5077,83,5082,83,42210,83,94010,83,66198,83,66592,83,119853,116,119905,116,119957,116,120009,116,120061,116,120113,116,120165,116,120217,116,120269,116,120321,116,120373,116,120425,116,120477,116,8868,84,10201,84,128872,84,65332,84,119827,84,119879,84,119931,84,119983,84,120035,84,120087,84,120139,84,120191,84,120243,84,120295,84,120347,84,120399,84,120451,84,932,84,120507,84,120565,84,120623,84,120681,84,120739,84,11430,84,5026,84,42196,84,93962,84,71868,84,66199,84,66225,84,66325,84,119854,117,119906,117,119958,117,120010,117,120062,117,120114,117,120166,117,120218,117,120270,117,120322,117,120374,117,120426,117,120478,117,42911,117,7452,117,43854,117,43858,117,651,117,965,117,120534,117,120592,117,120650,117,120708,117,120766,117,1405,117,66806,117,71896,117,8746,85,8899,85,119828,85,119880,85,119932,85,119984,85,120036,85,120088,85,120140,85,120192,85,120244,85,120296,85,120348,85,120400,85,120452,85,1357,85,4608,85,66766,85,5196,85,42228,85,94018,85,71864,85,8744,118,8897,118,65366,118,8564,118,119855,118,119907,118,119959,118,120011,118,120063,118,120115,118,120167,118,120219,118,120271,118,120323,118,120375,118,120427,118,120479,118,7456,118,957,118,120526,118,120584,118,120642,118,120700,118,120758,118,1141,118,1496,118,71430,118,43945,118,71872,118,119309,86,1639,86,1783,86,8548,86,119829,86,119881,86,119933,86,119985,86,120037,86,120089,86,120141,86,120193,86,120245,86,120297,86,120349,86,120401,86,120453,86,1140,86,11576,86,5081,86,5167,86,42719,86,42214,86,93960,86,71840,86,66845,86,623,119,119856,119,119908,119,119960,119,120012,119,120064,119,120116,119,120168,119,120220,119,120272,119,120324,119,120376,119,120428,119,120480,119,7457,119,1121,119,1309,119,1377,119,71434,119,71438,119,71439,119,43907,119,71919,87,71910,87,119830,87,119882,87,119934,87,119986,87,120038,87,120090,87,120142,87,120194,87,120246,87,120298,87,120350,87,120402,87,120454,87,1308,87,5043,87,5076,87,42218,87,5742,120,10539,120,10540,120,10799,120,65368,120,8569,120,119857,120,119909,120,119961,120,120013,120,120065,120,120117,120,120169,120,120221,120,120273,120,120325,120,120377,120,120429,120,120481,120,5441,120,5501,120,5741,88,9587,88,66338,88,71916,88,65336,88,8553,88,119831,88,119883,88,119935,88,119987,88,120039,88,120091,88,120143,88,120195,88,120247,88,120299,88,120351,88,120403,88,120455,88,42931,88,935,88,120510,88,120568,88,120626,88,120684,88,120742,88,11436,88,11613,88,5815,88,42219,88,66192,88,66228,88,66327,88,66855,88,611,121,7564,121,65369,121,119858,121,119910,121,119962,121,120014,121,120066,121,120118,121,120170,121,120222,121,120274,121,120326,121,120378,121,120430,121,120482,121,655,121,7935,121,43866,121,947,121,8509,121,120516,121,120574,121,120632,121,120690,121,120748,121,1199,121,4327,121,71900,121,65337,89,119832,89,119884,89,119936,89,119988,89,120040,89,120092,89,120144,89,120196,89,120248,89,120300,89,120352,89,120404,89,120456,89,933,89,978,89,120508,89,120566,89,120624,89,120682,89,120740,89,11432,89,1198,89,5033,89,5053,89,42220,89,94019,89,71844,89,66226,89,119859,122,119911,122,119963,122,120015,122,120067,122,120119,122,120171,122,120223,122,120275,122,120327,122,120379,122,120431,122,120483,122,7458,122,43923,122,71876,122,66293,90,71909,90,65338,90,8484,90,8488,90,119833,90,119885,90,119937,90,119989,90,120041,90,120197,90,120249,90,120301,90,120353,90,120405,90,120457,90,918,90,120493,90,120551,90,120609,90,120667,90,120725,90,5059,90,42204,90,71849,90,65282,34,65284,36,65285,37,65286,38,65290,42,65291,43,65294,46,65295,47,65296,48,65297,49,65298,50,65299,51,65300,52,65301,53,65302,54,65303,55,65304,56,65305,57,65308,60,65309,61,65310,62,65312,64,65316,68,65318,70,65319,71,65324,76,65329,81,65330,82,65333,85,65334,86,65335,87,65343,95,65346,98,65348,100,65350,102,65355,107,65357,109,65358,110,65361,113,65362,114,65364,116,65365,117,65367,119,65370,122,65371,123,65373,125,119846,109],"_default":[160,32,8211,45,65374,126,65306,58,65281,33,8216,96,8217,96,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"cs":[65374,126,65306,58,65281,33,8216,96,8217,96,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"de":[65374,126,65306,58,65281,33,8216,96,8217,96,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"es":[8211,45,65374,126,65306,58,65281,33,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"fr":[65374,126,65306,58,65281,33,8216,96,8245,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"it":[160,32,8211,45,65374,126,65306,58,65281,33,8216,96,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"ja":[8211,45,65306,58,65281,33,8216,96,8217,96,8245,96,180,96,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65292,44,65307,59],"ko":[8211,45,65374,126,65306,58,65281,33,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"pl":[65374,126,65306,58,65281,33,8216,96,8217,96,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"pt-BR":[65374,126,65306,58,65281,33,8216,96,8217,96,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"qps-ploc":[160,32,8211,45,65374,126,65306,58,65281,33,8216,96,8217,96,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"ru":[65374,126,65306,58,65281,33,8216,96,8217,96,8245,96,180,96,12494,47,305,105,921,73,1009,112,215,120,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"tr":[160,32,8211,45,65374,126,65306,58,65281,33,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65288,40,65289,41,65292,44,65307,59,65311,63],"zh-hans":[65374,126,65306,58,65281,33,8245,96,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65288,40,65289,41],"zh-hant":[8211,45,65374,126,180,96,12494,47,1047,51,1073,54,1072,97,1040,65,1068,98,1042,66,1089,99,1057,67,1077,101,1045,69,1053,72,305,105,1050,75,921,73,1052,77,1086,111,1054,79,1009,112,1088,112,1056,80,1075,114,1058,84,215,120,1093,120,1061,88,1091,121,1059,89,65283,35,65307,59]}')),dl.cache=new kee({getCacheKey:JSON.stringify},e=>{function t(d){const h=new Map;for(let u=0;u<d.length;u+=2)h.set(d[u],d[u+1]);return h}function i(d,h){const u=new Map(d);for(const[g,f]of h)u.set(g,f);return u}function n(d,h){if(!d)return h;const u=new Map;for(const[g,f]of d)h.has(g)&&u.set(g,f);return u}const s=dl.ambiguousCharacterData.value;let r=e.filter(d=>!d.startsWith("_")&&d in s);r.length===0&&(r=["_default"]);let a;for(const d of r){const h=t(s[d]);a=n(a,h)}const l=t(s._common),c=i(l,a);return new dl(c)}),dl._locales=new Oa(()=>Object.keys(dl.ambiguousCharacterData.value).filter(e=>!e.startsWith("_")));let ow=dl;const xp=class xp{static getRawData(){return JSON.parse("[9,10,11,12,13,32,127,160,173,847,1564,4447,4448,6068,6069,6155,6156,6157,6158,7355,7356,8192,8193,8194,8195,8196,8197,8198,8199,8200,8201,8202,8203,8204,8205,8206,8207,8234,8235,8236,8237,8238,8239,8287,8288,8289,8290,8291,8292,8293,8294,8295,8296,8297,8298,8299,8300,8301,8302,8303,10240,12288,12644,65024,65025,65026,65027,65028,65029,65030,65031,65032,65033,65034,65035,65036,65037,65038,65039,65279,65440,65520,65521,65522,65523,65524,65525,65526,65527,65528,65532,78844,119155,119156,119157,119158,119159,119160,119161,119162,917504,917505,917506,917507,917508,917509,917510,917511,917512,917513,917514,917515,917516,917517,917518,917519,917520,917521,917522,917523,917524,917525,917526,917527,917528,917529,917530,917531,917532,917533,917534,917535,917536,917537,917538,917539,917540,917541,917542,917543,917544,917545,917546,917547,917548,917549,917550,917551,917552,917553,917554,917555,917556,917557,917558,917559,917560,917561,917562,917563,917564,917565,917566,917567,917568,917569,917570,917571,917572,917573,917574,917575,917576,917577,917578,917579,917580,917581,917582,917583,917584,917585,917586,917587,917588,917589,917590,917591,917592,917593,917594,917595,917596,917597,917598,917599,917600,917601,917602,917603,917604,917605,917606,917607,917608,917609,917610,917611,917612,917613,917614,917615,917616,917617,917618,917619,917620,917621,917622,917623,917624,917625,917626,917627,917628,917629,917630,917631,917760,917761,917762,917763,917764,917765,917766,917767,917768,917769,917770,917771,917772,917773,917774,917775,917776,917777,917778,917779,917780,917781,917782,917783,917784,917785,917786,917787,917788,917789,917790,917791,917792,917793,917794,917795,917796,917797,917798,917799,917800,917801,917802,917803,917804,917805,917806,917807,917808,917809,917810,917811,917812,917813,917814,917815,917816,917817,917818,917819,917820,917821,917822,917823,917824,917825,917826,917827,917828,917829,917830,917831,917832,917833,917834,917835,917836,917837,917838,917839,917840,917841,917842,917843,917844,917845,917846,917847,917848,917849,917850,917851,917852,917853,917854,917855,917856,917857,917858,917859,917860,917861,917862,917863,917864,917865,917866,917867,917868,917869,917870,917871,917872,917873,917874,917875,917876,917877,917878,917879,917880,917881,917882,917883,917884,917885,917886,917887,917888,917889,917890,917891,917892,917893,917894,917895,917896,917897,917898,917899,917900,917901,917902,917903,917904,917905,917906,917907,917908,917909,917910,917911,917912,917913,917914,917915,917916,917917,917918,917919,917920,917921,917922,917923,917924,917925,917926,917927,917928,917929,917930,917931,917932,917933,917934,917935,917936,917937,917938,917939,917940,917941,917942,917943,917944,917945,917946,917947,917948,917949,917950,917951,917952,917953,917954,917955,917956,917957,917958,917959,917960,917961,917962,917963,917964,917965,917966,917967,917968,917969,917970,917971,917972,917973,917974,917975,917976,917977,917978,917979,917980,917981,917982,917983,917984,917985,917986,917987,917988,917989,917990,917991,917992,917993,917994,917995,917996,917997,917998,917999]")}static getData(){return this._data||(this._data=new Set(xp.getRawData())),this._data}static isInvisibleCharacter(e){return xp.getData().has(e)}static get codePoints(){return xp.getData()}};xp._data=void 0;let fg=xp;const Ek=class Ek{constructor(){this.mapWindowIdToZoomFactor=new Map}getZoomFactor(e){return this.mapWindowIdToZoomFactor.get(this.getWindowId(e))??1}getWindowId(e){return e.vscodeWindowId}};Ek.INSTANCE=new Ek;let vM=Ek;function aU(o,e,t){typeof e=="string"&&(e=o.matchMedia(e)),e.addEventListener("change",t)}function $ee(o){return vM.INSTANCE.getZoomFactor(o)}const ub=navigator.userAgent,pr=ub.indexOf("Firefox")>=0,fI=ub.indexOf("AppleWebKit")>=0,O1=ub.indexOf("Chrome")>=0,Wh=!O1&&ub.indexOf("Safari")>=0,lU=!O1&&!Wh&&fI;ub.indexOf("Electron/")>=0;const QB=ub.indexOf("Android")>=0;let rN=!1;if(typeof Rt.matchMedia=="function"){const o=Rt.matchMedia("(display-mode: standalone) or (display-mode: window-controls-overlay)"),e=Rt.matchMedia("(display-mode: fullscreen)");rN=o.matches,aU(Rt,o,({matches:t})=>{rN&&e.matches||(rN=t)})}const i5={clipboard:{writeText:Pl||document.queryCommandSupported&&document.queryCommandSupported("copy")||!!(navigator&&navigator.clipboard&&navigator.clipboard.writeText),readText:Pl||!!(navigator&&navigator.clipboard&&navigator.clipboard.readText)},pointerEvents:Rt.PointerEvent&&("ontouchstart"in Rt||navigator.maxTouchPoints>0)};function wM(o,e){if(typeof o=="number"){if(o===0)return null;const t=(o&65535)>>>0,i=(o&4294901760)>>>16;return i!==0?new aN([ES(t,e),ES(i,e)]):new aN([ES(t,e)])}else{const t=[];for(let i=0;i<o.length;i++)t.push(ES(o[i],e));return new aN(t)}}function ES(o,e){const t=!!(o&2048),i=!!(o&256),n=e===2?i:t,s=!!(o&1024),r=!!(o&512),a=e===2?t:i,l=o&255;return new Jc(n,s,r,a,l)}class Jc{constructor(e,t,i,n,s){this.ctrlKey=e,this.shiftKey=t,this.altKey=i,this.metaKey=n,this.keyCode=s}equals(e){return e instanceof Jc&&this.ctrlKey===e.ctrlKey&&this.shiftKey===e.shiftKey&&this.altKey===e.altKey&&this.metaKey===e.metaKey&&this.keyCode===e.keyCode}isModifierKey(){return this.keyCode===0||this.keyCode===5||this.keyCode===57||this.keyCode===6||this.keyCode===4}isDuplicateModifierCase(){return this.ctrlKey&&this.keyCode===5||this.shiftKey&&this.keyCode===4||this.altKey&&this.keyCode===6||this.metaKey&&this.keyCode===57}}class aN{constructor(e){if(e.length===0)throw gr("chords");this.chords=e}}class jee{constructor(e,t,i,n,s,r){this.ctrlKey=e,this.shiftKey=t,this.altKey=i,this.metaKey=n,this.keyLabel=s,this.keyAriaLabel=r}}class Kee{}function qee(o){if(o.charCode){const t=String.fromCharCode(o.charCode).toUpperCase();return xc.fromString(t)}const e=o.keyCode;if(e===3)return 7;if(pr)switch(e){case 59:return 85;case 60:if(ps)return 97;break;case 61:return 86;case 107:return 109;case 109:return 111;case 173:return 88;case 224:if(Je)return 57;break}else if(fI){if(Je&&e===93)return 57;if(!Je&&e===92)return 57}return Hz[e]||0}const Gee=Je?256:2048,Zee=512,Yee=1024,Xee=Je?2048:256;class Pt{constructor(e){var i;this._standardKeyboardEventBrand=!0;const t=e;this.browserEvent=t,this.target=t.target,this.ctrlKey=t.ctrlKey,this.shiftKey=t.shiftKey,this.altKey=t.altKey,this.metaKey=t.metaKey,this.altGraphKey=(i=t.getModifierState)==null?void 0:i.call(t,"AltGraph"),this.keyCode=qee(t),this.code=t.code,this.ctrlKey=this.ctrlKey||this.keyCode===5,this.altKey=this.altKey||this.keyCode===6,this.shiftKey=this.shiftKey||this.keyCode===4,this.metaKey=this.metaKey||this.keyCode===57,this._asKeybinding=this._computeKeybinding(),this._asKeyCodeChord=this._computeKeyCodeChord()}preventDefault(){this.browserEvent&&this.browserEvent.preventDefault&&this.browserEvent.preventDefault()}stopPropagation(){this.browserEvent&&this.browserEvent.stopPropagation&&this.browserEvent.stopPropagation()}toKeyCodeChord(){return this._asKeyCodeChord}equals(e){return this._asKeybinding===e}_computeKeybinding(){let e=0;this.keyCode!==5&&this.keyCode!==4&&this.keyCode!==6&&this.keyCode!==57&&(e=this.keyCode);let t=0;return this.ctrlKey&&(t|=Gee),this.altKey&&(t|=Zee),this.shiftKey&&(t|=Yee),this.metaKey&&(t|=Xee),t|=e,t}_computeKeyCodeChord(){let e=0;return this.keyCode!==5&&this.keyCode!==4&&this.keyCode!==6&&this.keyCode!==57&&(e=this.keyCode),new Jc(this.ctrlKey,this.shiftKey,this.altKey,this.metaKey,e)}}const JB=new WeakMap;function Qee(o){if(!o.parent||o.parent===o)return null;try{const e=o.location,t=o.parent.location;if(e.origin!=="null"&&t.origin!=="null"&&e.origin!==t.origin)return null}catch{return null}return o.parent}class Jee{static getSameOriginWindowChain(e){let t=JB.get(e);if(!t){t=[],JB.set(e,t);let i=e,n;do n=Qee(i),n?t.push({window:new WeakRef(i),iframeElement:i.frameElement||null}):t.push({window:new WeakRef(i),iframeElement:null}),i=n;while(i)}return t.slice(0)}static getPositionOfChildWindowRelativeToAncestorWindow(e,t){if(!t||e===t)return{top:0,left:0};let i=0,n=0;const s=this.getSameOriginWindowChain(e);for(const r of s){const a=r.window.deref();if(i+=(a==null?void 0:a.scrollY)??0,n+=(a==null?void 0:a.scrollX)??0,a===t||!r.iframeElement)break;const l=r.iframeElement.getBoundingClientRect();i+=l.top,n+=l.left}return{top:i,left:n}}}class Vr{constructor(e,t){this.timestamp=Date.now(),this.browserEvent=t,this.leftButton=t.button===0,this.middleButton=t.button===1,this.rightButton=t.button===2,this.buttons=t.buttons,this.target=t.target,this.detail=t.detail||1,t.type==="dblclick"&&(this.detail=2),this.ctrlKey=t.ctrlKey,this.shiftKey=t.shiftKey,this.altKey=t.altKey,this.metaKey=t.metaKey,typeof t.pageX=="number"?(this.posx=t.pageX,this.posy=t.pageY):(this.posx=t.clientX+this.target.ownerDocument.body.scrollLeft+this.target.ownerDocument.documentElement.scrollLeft,this.posy=t.clientY+this.target.ownerDocument.body.scrollTop+this.target.ownerDocument.documentElement.scrollTop);const i=Jee.getPositionOfChildWindowRelativeToAncestorWindow(e,t.view);this.posx-=i.left,this.posy-=i.top}preventDefault(){this.browserEvent.preventDefault()}stopPropagation(){this.browserEvent.stopPropagation()}}class Df{constructor(e,t=0,i=0){var s;this.browserEvent=e||null,this.target=e?e.target||e.targetNode||e.srcElement:null,this.deltaY=i,this.deltaX=t;let n=!1;if(O1){const r=navigator.userAgent.match(/Chrome\/(\d+)/);n=(r?parseInt(r[1]):123)<=122}if(e){const r=e,a=e,l=((s=e.view)==null?void 0:s.devicePixelRatio)||1;if(typeof r.wheelDeltaY<"u")n?this.deltaY=r.wheelDeltaY/(120*l):this.deltaY=r.wheelDeltaY/120;else if(typeof a.VERTICAL_AXIS<"u"&&a.axis===a.VERTICAL_AXIS)this.deltaY=-a.detail/3;else if(e.type==="wheel"){const c=e;c.deltaMode===c.DOM_DELTA_LINE?pr&&!Je?this.deltaY=-e.deltaY/3:this.deltaY=-e.deltaY:this.deltaY=-e.deltaY/40}if(typeof r.wheelDeltaX<"u")Wh&&En?this.deltaX=-(r.wheelDeltaX/120):n?this.deltaX=r.wheelDeltaX/(120*l):this.deltaX=r.wheelDeltaX/120;else if(typeof a.HORIZONTAL_AXIS<"u"&&a.axis===a.HORIZONTAL_AXIS)this.deltaX=-e.detail/3;else if(e.type==="wheel"){const c=e;c.deltaMode===c.DOM_DELTA_LINE?pr&&!Je?this.deltaX=-e.deltaX/3:this.deltaX=-e.deltaX:this.deltaX=-e.deltaX/40}this.deltaY===0&&this.deltaX===0&&e.wheelDelta&&(n?this.deltaY=e.wheelDelta/(120*l):this.deltaY=e.wheelDelta/120)}}preventDefault(){var e;(e=this.browserEvent)==null||e.preventDefault()}stopPropagation(){var e;(e=this.browserEvent)==null||e.stopPropagation()}}const cU=Symbol("MicrotaskDelay");function SM(o){return!!o&&typeof o.then=="function"}function Yi(o){const e=new li,t=o(e.token),i=new Promise((n,s)=>{const r=e.token.onCancellationRequested(()=>{r.dispose(),s(new qr)});Promise.resolve(t).then(a=>{r.dispose(),e.dispose(),n(a)},a=>{r.dispose(),e.dispose(),s(a)})});return new class{cancel(){e.cancel(),e.dispose()}then(n,s){return i.then(n,s)}catch(n){return this.then(void 0,n)}finally(n){return i.finally(n)}}}function F1(o,e,t){return new Promise((i,n)=>{const s=e.onCancellationRequested(()=>{s.dispose(),i(t)});o.then(i,n).finally(()=>s.dispose())})}class ete{constructor(){this.isDisposed=!1,this.activePromise=null,this.queuedPromise=null,this.queuedPromiseFactory=null}queue(e){if(this.isDisposed)return Promise.reject(new Error("Throttler is disposed"));if(this.activePromise){if(this.queuedPromiseFactory=e,!this.queuedPromise){const t=()=>{if(this.queuedPromise=null,this.isDisposed)return;const i=this.queue(this.queuedPromiseFactory);return this.queuedPromiseFactory=null,i};this.queuedPromise=new Promise(i=>{this.activePromise.then(t,t).then(i)})}return new Promise((t,i)=>{this.queuedPromise.then(t,i)})}return this.activePromise=e(),new Promise((t,i)=>{this.activePromise.then(n=>{this.activePromise=null,t(n)},n=>{this.activePromise=null,i(n)})})}dispose(){this.isDisposed=!0}}const tte=(o,e)=>{let t=!0;const i=setTimeout(()=>{t=!1,e()},o);return{isTriggered:()=>t,dispose:()=>{clearTimeout(i),t=!1}}},ite=o=>{let e=!0;return queueMicrotask(()=>{e&&(e=!1,o())}),{isTriggered:()=>e,dispose:()=>{e=!1}}};class _r{constructor(e){this.defaultDelay=e,this.deferred=null,this.completionPromise=null,this.doResolve=null,this.doReject=null,this.task=null}trigger(e,t=this.defaultDelay){this.task=e,this.cancelTimeout(),this.completionPromise||(this.completionPromise=new Promise((n,s)=>{this.doResolve=n,this.doReject=s}).then(()=>{if(this.completionPromise=null,this.doResolve=null,this.task){const n=this.task;return this.task=null,n()}}));const i=()=>{var n;this.deferred=null,(n=this.doResolve)==null||n.call(this,null)};return this.deferred=t===cU?ite(i):tte(t,i),this.completionPromise}isTriggered(){var e;return!!((e=this.deferred)!=null&&e.isTriggered())}cancel(){var e;this.cancelTimeout(),this.completionPromise&&((e=this.doReject)==null||e.call(this,new qr),this.completionPromise=null)}cancelTimeout(){var e;(e=this.deferred)==null||e.dispose(),this.deferred=null}dispose(){this.cancel()}}class dU{constructor(e){this.delayer=new _r(e),this.throttler=new ete}trigger(e,t){return this.delayer.trigger(()=>this.throttler.queue(e),t)}cancel(){this.delayer.cancel()}dispose(){this.delayer.dispose(),this.throttler.dispose()}}function ed(o,e){return e?new Promise((t,i)=>{const n=setTimeout(()=>{s.dispose(),t()},o),s=e.onCancellationRequested(()=>{clearTimeout(n),s.dispose(),i(new qr)})}):Yi(t=>ed(o,t))}function Bh(o,e=0,t){const i=setTimeout(()=>{o(),t&&n.dispose()},e),n=ve(()=>{clearTimeout(i),t==null||t.deleteAndLeak(n)});return t==null||t.add(n),n}function n5(o,e=i=>!!i,t=null){let i=0;const n=o.length,s=()=>{if(i>=n)return Promise.resolve(t);const r=o[i++];return Promise.resolve(r()).then(l=>e(l)?Promise.resolve(l):s())};return s()}class ea{constructor(e,t){this._isDisposed=!1,this._token=-1,typeof e=="function"&&typeof t=="number"&&this.setIfNotSet(e,t)}dispose(){this.cancel(),this._isDisposed=!0}cancel(){this._token!==-1&&(clearTimeout(this._token),this._token=-1)}cancelAndSet(e,t){if(this._isDisposed)throw new bt("Calling 'cancelAndSet' on a disposed TimeoutTimer");this.cancel(),this._token=setTimeout(()=>{this._token=-1,e()},t)}setIfNotSet(e,t){if(this._isDisposed)throw new bt("Calling 'setIfNotSet' on a disposed TimeoutTimer");this._token===-1&&(this._token=setTimeout(()=>{this._token=-1,e()},t))}}class s5{constructor(){this.disposable=void 0,this.isDisposed=!1}cancel(){var e;(e=this.disposable)==null||e.dispose(),this.disposable=void 0}cancelAndSet(e,t,i=globalThis){if(this.isDisposed)throw new bt("Calling 'cancelAndSet' on a disposed IntervalTimer");this.cancel();const n=i.setInterval(()=>{e()},t);this.disposable=ve(()=>{i.clearInterval(n),this.disposable=void 0})}dispose(){this.cancel(),this.isDisposed=!0}}class It{constructor(e,t){this.timeoutToken=-1,this.runner=e,this.timeout=t,this.timeoutHandler=this.onTimeout.bind(this)}dispose(){this.cancel(),this.runner=null}cancel(){this.isScheduled()&&(clearTimeout(this.timeoutToken),this.timeoutToken=-1)}schedule(e=this.timeout){this.cancel(),this.timeoutToken=setTimeout(this.timeoutHandler,e)}get delay(){return this.timeout}set delay(e){this.timeout=e}isScheduled(){return this.timeoutToken!==-1}onTimeout(){this.timeoutToken=-1,this.runner&&this.doRun()}doRun(){var e;(e=this.runner)==null||e.call(this)}}let hU,KC;(function(){typeof globalThis.requestIdleCallback!="function"||typeof globalThis.cancelIdleCallback!="function"?KC=(o,e)=>{Nz(()=>{if(t)return;const i=Date.now()+15;e(Object.freeze({didTimeout:!0,timeRemaining(){return Math.max(0,i-Date.now())}}))});let t=!1;return{dispose(){t||(t=!0)}}}:KC=(o,e,t)=>{const i=o.requestIdleCallback(e,typeof t=="number"?{timeout:t}:void 0);let n=!1;return{dispose(){n||(n=!0,o.cancelIdleCallback(i))}}},hU=o=>KC(globalThis,o)})();class uU{constructor(e,t){this._didRun=!1,this._executor=()=>{try{this._value=t()}catch(i){this._error=i}finally{this._didRun=!0}},this._handle=KC(e,()=>this._executor())}dispose(){this._handle.dispose()}get value(){if(this._didRun||(this._handle.dispose(),this._executor()),this._error)throw this._error;return this._value}get isInitialized(){return this._didRun}}class nte extends uU{constructor(e){super(globalThis,e)}}class gb{get isRejected(){var e;return((e=this.outcome)==null?void 0:e.outcome)===1}get isSettled(){return!!this.outcome}constructor(){this.p=new Promise((e,t)=>{this.completeCallback=e,this.errorCallback=t})}complete(e){return new Promise(t=>{this.completeCallback(e),this.outcome={outcome:0,value:e},t()})}error(e){return new Promise(t=>{this.errorCallback(e),this.outcome={outcome:1,value:e},t()})}cancel(){return this.error(new qr)}}var yM;(function(o){async function e(i){let n;const s=await Promise.all(i.map(r=>r.then(a=>a,a=>{n||(n=a)})));if(typeof n<"u")throw n;return s}o.settled=e;function t(i){return new Promise(async(n,s)=>{try{await i(n,s)}catch(r){s(r)}})}o.withAsyncBody=t})(yM||(yM={}));const Zs=class Zs{static fromArray(e){return new Zs(t=>{t.emitMany(e)})}static fromPromise(e){return new Zs(async t=>{t.emitMany(await e)})}static fromPromises(e){return new Zs(async t=>{await Promise.all(e.map(async i=>t.emitOne(await i)))})}static merge(e){return new Zs(async t=>{await Promise.all(e.map(async i=>{for await(const n of i)t.emitOne(n)}))})}constructor(e,t){this._state=0,this._results=[],this._error=null,this._onReturn=t,this._onStateChanged=new P,queueMicrotask(async()=>{const i={emitOne:n=>this.emitOne(n),emitMany:n=>this.emitMany(n),reject:n=>this.reject(n)};try{await Promise.resolve(e(i)),this.resolve()}catch(n){this.reject(n)}finally{i.emitOne=void 0,i.emitMany=void 0,i.reject=void 0}})}[Symbol.asyncIterator](){let e=0;return{next:async()=>{do{if(this._state===2)throw this._error;if(e<this._results.length)return{done:!1,value:this._results[e++]};if(this._state===1)return{done:!0,value:void 0};await oe.toPromise(this._onStateChanged.event)}while(!0)},return:async()=>{var t;return(t=this._onReturn)==null||t.call(this),{done:!0,value:void 0}}}}static map(e,t){return new Zs(async i=>{for await(const n of e)i.emitOne(t(n))})}map(e){return Zs.map(this,e)}static filter(e,t){return new Zs(async i=>{for await(const n of e)t(n)&&i.emitOne(n)})}filter(e){return Zs.filter(this,e)}static coalesce(e){return Zs.filter(e,t=>!!t)}coalesce(){return Zs.coalesce(this)}static async toPromise(e){const t=[];for await(const i of e)t.push(i);return t}toPromise(){return Zs.toPromise(this)}emitOne(e){this._state===0&&(this._results.push(e),this._onStateChanged.fire())}emitMany(e){this._state===0&&(this._results=this._results.concat(e),this._onStateChanged.fire())}resolve(){this._state===0&&(this._state=1,this._onStateChanged.fire())}reject(e){this._state===0&&(this._state=2,this._error=e,this._onStateChanged.fire())}};Zs.EMPTY=Zs.fromArray([]);let Fo=Zs;class ste extends Fo{constructor(e,t){super(t),this._source=e}cancel(){this._source.cancel()}}function ote(o){const e=new li,t=o(e.token);return new ste(e,async i=>{const n=e.token.onCancellationRequested(()=>{n.dispose(),e.dispose(),i.reject(new qr)});try{for await(const s of t){if(e.token.isCancellationRequested)return;i.emitOne(s)}n.dispose(),e.dispose()}catch(s){n.dispose(),e.dispose(),i.reject(s)}})}/*! @license DOMPurify 3.1.7 | (c) Cure53 and other contributors | Released under the Apache license 2.0 and Mozilla Public License 2.0 | github.com/cure53/DOMPurify/blob/3.1.7/LICENSE */const{entries:gU,setPrototypeOf:e7,isFrozen:rte,getPrototypeOf:ate,getOwnPropertyDescriptor:lte}=Object;let{freeze:uo,seal:Gr,create:fU}=Object,{apply:LM,construct:xM}=typeof Reflect<"u"&&Reflect;uo||(uo=function(e){return e});Gr||(Gr=function(e){return e});LM||(LM=function(e,t,i){return e.apply(t,i)});xM||(xM=function(e,t){return new e(...t)});const NS=br(Array.prototype.forEach),t7=br(Array.prototype.pop),$b=br(Array.prototype.push),V0=br(String.prototype.toLowerCase),lN=br(String.prototype.toString),i7=br(String.prototype.match),jb=br(String.prototype.replace),cte=br(String.prototype.indexOf),dte=br(String.prototype.trim),da=br(Object.prototype.hasOwnProperty),Ks=br(RegExp.prototype.test),Kb=hte(TypeError);function br(o){return function(e){for(var t=arguments.length,i=new Array(t>1?t-1:0),n=1;n<t;n++)i[n-1]=arguments[n];return LM(o,e,i)}}function hte(o){return function(){for(var e=arguments.length,t=new Array(e),i=0;i<e;i++)t[i]=arguments[i];return xM(o,t)}}function Mt(o,e){let t=arguments.length>2&&arguments[2]!==void 0?arguments[2]:V0;e7&&e7(o,null);let i=e.length;for(;i--;){let n=e[i];if(typeof n=="string"){const s=t(n);s!==n&&(rte(e)||(e[i]=s),n=s)}o[n]=!0}return o}function ute(o){for(let e=0;e<o.length;e++)da(o,e)||(o[e]=null);return o}function Nu(o){const e=fU(null);for(const[t,i]of gU(o))da(o,t)&&(Array.isArray(i)?e[t]=ute(i):i&&typeof i=="object"&&i.constructor===Object?e[t]=Nu(i):e[t]=i);return e}function qb(o,e){for(;o!==null;){const i=lte(o,e);if(i){if(i.get)return br(i.get);if(typeof i.value=="function")return br(i.value)}o=ate(o)}function t(){return null}return t}const n7=uo(["a","abbr","acronym","address","area","article","aside","audio","b","bdi","bdo","big","blink","blockquote","body","br","button","canvas","caption","center","cite","code","col","colgroup","content","data","datalist","dd","decorator","del","details","dfn","dialog","dir","div","dl","dt","element","em","fieldset","figcaption","figure","font","footer","form","h1","h2","h3","h4","h5","h6","head","header","hgroup","hr","html","i","img","input","ins","kbd","label","legend","li","main","map","mark","marquee","menu","menuitem","meter","nav","nobr","ol","optgroup","option","output","p","picture","pre","progress","q","rp","rt","ruby","s","samp","section","select","shadow","small","source","spacer","span","strike","strong","style","sub","summary","sup","table","tbody","td","template","textarea","tfoot","th","thead","time","tr","track","tt","u","ul","var","video","wbr"]),cN=uo(["svg","a","altglyph","altglyphdef","altglyphitem","animatecolor","animatemotion","animatetransform","circle","clippath","defs","desc","ellipse","filter","font","g","glyph","glyphref","hkern","image","line","lineargradient","marker","mask","metadata","mpath","path","pattern","polygon","polyline","radialgradient","rect","stop","style","switch","symbol","text","textpath","title","tref","tspan","view","vkern"]),dN=uo(["feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feDistantLight","feDropShadow","feFlood","feFuncA","feFuncB","feFuncG","feFuncR","feGaussianBlur","feImage","feMerge","feMergeNode","feMorphology","feOffset","fePointLight","feSpecularLighting","feSpotLight","feTile","feTurbulence"]),gte=uo(["animate","color-profile","cursor","discard","font-face","font-face-format","font-face-name","font-face-src","font-face-uri","foreignobject","hatch","hatchpath","mesh","meshgradient","meshpatch","meshrow","missing-glyph","script","set","solidcolor","unknown","use"]),hN=uo(["math","menclose","merror","mfenced","mfrac","mglyph","mi","mlabeledtr","mmultiscripts","mn","mo","mover","mpadded","mphantom","mroot","mrow","ms","mspace","msqrt","mstyle","msub","msup","msubsup","mtable","mtd","mtext","mtr","munder","munderover","mprescripts"]),fte=uo(["maction","maligngroup","malignmark","mlongdiv","mscarries","mscarry","msgroup","mstack","msline","msrow","semantics","annotation","annotation-xml","mprescripts","none"]),s7=uo(["#text"]),o7=uo(["accept","action","align","alt","autocapitalize","autocomplete","autopictureinpicture","autoplay","background","bgcolor","border","capture","cellpadding","cellspacing","checked","cite","class","clear","color","cols","colspan","controls","controlslist","coords","crossorigin","datetime","decoding","default","dir","disabled","disablepictureinpicture","disableremoteplayback","download","draggable","enctype","enterkeyhint","face","for","headers","height","hidden","high","href","hreflang","id","inputmode","integrity","ismap","kind","label","lang","list","loading","loop","low","max","maxlength","media","method","min","minlength","multiple","muted","name","nonce","noshade","novalidate","nowrap","open","optimum","pattern","placeholder","playsinline","popover","popovertarget","popovertargetaction","poster","preload","pubdate","radiogroup","readonly","rel","required","rev","reversed","role","rows","rowspan","spellcheck","scope","selected","shape","size","sizes","span","srclang","start","src","srcset","step","style","summary","tabindex","title","translate","type","usemap","valign","value","width","wrap","xmlns","slot"]),uN=uo(["accent-height","accumulate","additive","alignment-baseline","amplitude","ascent","attributename","attributetype","azimuth","basefrequency","baseline-shift","begin","bias","by","class","clip","clippathunits","clip-path","clip-rule","color","color-interpolation","color-interpolation-filters","color-profile","color-rendering","cx","cy","d","dx","dy","diffuseconstant","direction","display","divisor","dur","edgemode","elevation","end","exponent","fill","fill-opacity","fill-rule","filter","filterunits","flood-color","flood-opacity","font-family","font-size","font-size-adjust","font-stretch","font-style","font-variant","font-weight","fx","fy","g1","g2","glyph-name","glyphref","gradientunits","gradienttransform","height","href","id","image-rendering","in","in2","intercept","k","k1","k2","k3","k4","kerning","keypoints","keysplines","keytimes","lang","lengthadjust","letter-spacing","kernelmatrix","kernelunitlength","lighting-color","local","marker-end","marker-mid","marker-start","markerheight","markerunits","markerwidth","maskcontentunits","maskunits","max","mask","media","method","mode","min","name","numoctaves","offset","operator","opacity","order","orient","orientation","origin","overflow","paint-order","path","pathlength","patterncontentunits","patterntransform","patternunits","points","preservealpha","preserveaspectratio","primitiveunits","r","rx","ry","radius","refx","refy","repeatcount","repeatdur","restart","result","rotate","scale","seed","shape-rendering","slope","specularconstant","specularexponent","spreadmethod","startoffset","stddeviation","stitchtiles","stop-color","stop-opacity","stroke-dasharray","stroke-dashoffset","stroke-linecap","stroke-linejoin","stroke-miterlimit","stroke-opacity","stroke","stroke-width","style","surfacescale","systemlanguage","tabindex","tablevalues","targetx","targety","transform","transform-origin","text-anchor","text-decoration","text-rendering","textlength","type","u1","u2","unicode","values","viewbox","visibility","version","vert-adv-y","vert-origin-x","vert-origin-y","width","word-spacing","wrap","writing-mode","xchannelselector","ychannelselector","x","x1","x2","xmlns","y","y1","y2","z","zoomandpan"]),r7=uo(["accent","accentunder","align","bevelled","close","columnsalign","columnlines","columnspan","denomalign","depth","dir","display","displaystyle","encoding","fence","frame","height","href","id","largeop","length","linethickness","lspace","lquote","mathbackground","mathcolor","mathsize","mathvariant","maxsize","minsize","movablelimits","notation","numalign","open","rowalign","rowlines","rowspacing","rowspan","rspace","rquote","scriptlevel","scriptminsize","scriptsizemultiplier","selection","separator","separators","stretchy","subscriptshift","supscriptshift","symmetric","voffset","width","xmlns"]),TS=uo(["xlink:href","xml:id","xlink:title","xml:space","xmlns:xlink"]),mte=Gr(/\{\{[\w\W]*|[\w\W]*\}\}/gm),pte=Gr(/<%[\w\W]*|[\w\W]*%>/gm),_te=Gr(/\${[\w\W]*}/gm),bte=Gr(/^data-[\-\w.\u00B7-\uFFFF]/),Cte=Gr(/^aria-[\-\w]+$/),mU=Gr(/^(?:(?:(?:f|ht)tps?|mailto|tel|callto|sms|cid|xmpp):|[^a-z]|[a-z+.\-]+(?:[^a-z+.\-:]|$))/i),vte=Gr(/^(?:\w+script|data):/i),wte=Gr(/[\u0000-\u0020\u00A0\u1680\u180E\u2000-\u2029\u205F\u3000]/g),pU=Gr(/^html$/i),Ste=Gr(/^[a-z][.\w]*(-[.\w]+)+$/i);var a7=Object.freeze({__proto__:null,MUSTACHE_EXPR:mte,ERB_EXPR:pte,TMPLIT_EXPR:_te,DATA_ATTR:bte,ARIA_ATTR:Cte,IS_ALLOWED_URI:mU,IS_SCRIPT_OR_DATA:vte,ATTR_WHITESPACE:wte,DOCTYPE_NAME:pU,CUSTOM_ELEMENT:Ste});const Gb={element:1,text:3,progressingInstruction:7,comment:8,document:9},yte=function(){return typeof window>"u"?null:window},Lte=function(e,t){if(typeof e!="object"||typeof e.createPolicy!="function")return null;let i=null;const n="data-tt-policy-suffix";t&&t.hasAttribute(n)&&(i=t.getAttribute(n));const s="dompurify"+(i?"#"+i:"");try{return e.createPolicy(s,{createHTML(r){return r},createScriptURL(r){return r}})}catch{return console.warn("TrustedTypes policy "+s+" could not be created."),null}};function _U(){let o=arguments.length>0&&arguments[0]!==void 0?arguments[0]:yte();const e=Ye=>_U(Ye);if(e.version="3.1.7",e.removed=[],!o||!o.document||o.document.nodeType!==Gb.document)return e.isSupported=!1,e;let{document:t}=o;const i=t,n=i.currentScript,{DocumentFragment:s,HTMLTemplateElement:r,Node:a,Element:l,NodeFilter:c,NamedNodeMap:d=o.NamedNodeMap||o.MozNamedAttrMap,HTMLFormElement:h,DOMParser:u,trustedTypes:g}=o,f=l.prototype,p=qb(f,"cloneNode"),_=qb(f,"remove"),b=qb(f,"nextSibling"),C=qb(f,"childNodes"),v=qb(f,"parentNode");if(typeof r=="function"){const Ye=t.createElement("template");Ye.content&&Ye.content.ownerDocument&&(t=Ye.content.ownerDocument)}let w,S="";const{implementation:L,createNodeIterator:k,createDocumentFragment:D,getElementsByTagName:N}=t,{importNode:B}=i;let W={};e.isSupported=typeof gU=="function"&&typeof v=="function"&&L&&L.createHTMLDocument!==void 0;const{MUSTACHE_EXPR:V,ERB_EXPR:$,TMPLIT_EXPR:H,DATA_ATTR:j,ARIA_ATTR:J,IS_SCRIPT_OR_DATA:he,ATTR_WHITESPACE:fe,CUSTOM_ELEMENT:ue}=a7;let{IS_ALLOWED_URI:Pe}=a7,_e=null;const it=Mt({},[...n7,...cN,...dN,...hN,...s7]);let Ne=null;const Ct=Mt({},[...o7,...uN,...r7,...TS]);let ht=Object.seal(fU(null,{tagNameCheck:{writable:!0,configurable:!1,enumerable:!0,value:null},attributeNameCheck:{writable:!0,configurable:!1,enumerable:!0,value:null},allowCustomizedBuiltInElements:{writable:!0,configurable:!1,enumerable:!0,value:!1}})),Ti=null,ts=null,xs=!0,Dr=!0,Co=!1,hu=!0,zi=!1,Ir=!0,ec=!1,Bb=!1,Hb=!1,_d=!1,pm=!1,_m=!1,uu=!0,bS=!1;const CS="user-content-";let bm=!0,gu=!1,Ga={},bd=null;const Cm=Mt({},["annotation-xml","audio","colgroup","desc","foreignobject","head","iframe","math","mi","mn","mo","ms","mtext","noembed","noframes","noscript","plaintext","script","style","svg","template","thead","title","video","xmp"]);let vS=null;const fu=Mt({},["audio","video","img","source","image","track"]);let tc=null;const vo=Mt({},["alt","class","for","id","label","name","pattern","placeholder","role","summary","title","value","style","xmlns"]),Za="http://www.w3.org/1998/Math/MathML",ic="http://www.w3.org/2000/svg",dn="http://www.w3.org/1999/xhtml";let Jo=dn,vm=!1,na=null;const si=Mt({},[Za,ic,dn],lN);let mu=null;const ZE=["application/xhtml+xml","text/html"],YE="text/html";let Cn=null,Cd=null;const XE=t.createElement("form"),wS=function(Y){return Y instanceof RegExp||Y instanceof Function},Vb=function(){let Y=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};if(!(Cd&&Cd===Y)){if((!Y||typeof Y!="object")&&(Y={}),Y=Nu(Y),mu=ZE.indexOf(Y.PARSER_MEDIA_TYPE)===-1?YE:Y.PARSER_MEDIA_TYPE,Cn=mu==="application/xhtml+xml"?lN:V0,_e=da(Y,"ALLOWED_TAGS")?Mt({},Y.ALLOWED_TAGS,Cn):it,Ne=da(Y,"ALLOWED_ATTR")?Mt({},Y.ALLOWED_ATTR,Cn):Ct,na=da(Y,"ALLOWED_NAMESPACES")?Mt({},Y.ALLOWED_NAMESPACES,lN):si,tc=da(Y,"ADD_URI_SAFE_ATTR")?Mt(Nu(vo),Y.ADD_URI_SAFE_ATTR,Cn):vo,vS=da(Y,"ADD_DATA_URI_TAGS")?Mt(Nu(fu),Y.ADD_DATA_URI_TAGS,Cn):fu,bd=da(Y,"FORBID_CONTENTS")?Mt({},Y.FORBID_CONTENTS,Cn):Cm,Ti=da(Y,"FORBID_TAGS")?Mt({},Y.FORBID_TAGS,Cn):{},ts=da(Y,"FORBID_ATTR")?Mt({},Y.FORBID_ATTR,Cn):{},Ga=da(Y,"USE_PROFILES")?Y.USE_PROFILES:!1,xs=Y.ALLOW_ARIA_ATTR!==!1,Dr=Y.ALLOW_DATA_ATTR!==!1,Co=Y.ALLOW_UNKNOWN_PROTOCOLS||!1,hu=Y.ALLOW_SELF_CLOSE_IN_ATTR!==!1,zi=Y.SAFE_FOR_TEMPLATES||!1,Ir=Y.SAFE_FOR_XML!==!1,ec=Y.WHOLE_DOCUMENT||!1,_d=Y.RETURN_DOM||!1,pm=Y.RETURN_DOM_FRAGMENT||!1,_m=Y.RETURN_TRUSTED_TYPE||!1,Hb=Y.FORCE_BODY||!1,uu=Y.SANITIZE_DOM!==!1,bS=Y.SANITIZE_NAMED_PROPS||!1,bm=Y.KEEP_CONTENT!==!1,gu=Y.IN_PLACE||!1,Pe=Y.ALLOWED_URI_REGEXP||mU,Jo=Y.NAMESPACE||dn,ht=Y.CUSTOM_ELEMENT_HANDLING||{},Y.CUSTOM_ELEMENT_HANDLING&&wS(Y.CUSTOM_ELEMENT_HANDLING.tagNameCheck)&&(ht.tagNameCheck=Y.CUSTOM_ELEMENT_HANDLING.tagNameCheck),Y.CUSTOM_ELEMENT_HANDLING&&wS(Y.CUSTOM_ELEMENT_HANDLING.attributeNameCheck)&&(ht.attributeNameCheck=Y.CUSTOM_ELEMENT_HANDLING.attributeNameCheck),Y.CUSTOM_ELEMENT_HANDLING&&typeof Y.CUSTOM_ELEMENT_HANDLING.allowCustomizedBuiltInElements=="boolean"&&(ht.allowCustomizedBuiltInElements=Y.CUSTOM_ELEMENT_HANDLING.allowCustomizedBuiltInElements),zi&&(Dr=!1),pm&&(_d=!0),Ga&&(_e=Mt({},s7),Ne=[],Ga.html===!0&&(Mt(_e,n7),Mt(Ne,o7)),Ga.svg===!0&&(Mt(_e,cN),Mt(Ne,uN),Mt(Ne,TS)),Ga.svgFilters===!0&&(Mt(_e,dN),Mt(Ne,uN),Mt(Ne,TS)),Ga.mathMl===!0&&(Mt(_e,hN),Mt(Ne,r7),Mt(Ne,TS))),Y.ADD_TAGS&&(_e===it&&(_e=Nu(_e)),Mt(_e,Y.ADD_TAGS,Cn)),Y.ADD_ATTR&&(Ne===Ct&&(Ne=Nu(Ne)),Mt(Ne,Y.ADD_ATTR,Cn)),Y.ADD_URI_SAFE_ATTR&&Mt(tc,Y.ADD_URI_SAFE_ATTR,Cn),Y.FORBID_CONTENTS&&(bd===Cm&&(bd=Nu(bd)),Mt(bd,Y.FORBID_CONTENTS,Cn)),bm&&(_e["#text"]=!0),ec&&Mt(_e,["html","head","body"]),_e.table&&(Mt(_e,["tbody"]),delete Ti.tbody),Y.TRUSTED_TYPES_POLICY){if(typeof Y.TRUSTED_TYPES_POLICY.createHTML!="function")throw Kb('TRUSTED_TYPES_POLICY configuration option must provide a "createHTML" hook.');if(typeof Y.TRUSTED_TYPES_POLICY.createScriptURL!="function")throw Kb('TRUSTED_TYPES_POLICY configuration option must provide a "createScriptURL" hook.');w=Y.TRUSTED_TYPES_POLICY,S=w.createHTML("")}else w===void 0&&(w=Lte(g,n)),w!==null&&typeof S=="string"&&(S=w.createHTML(""));uo&&uo(Y),Cd=Y}},ut=Mt({},["mi","mo","mn","ms","mtext"]),R=Mt({},["annotation-xml"]),M=Mt({},["title","style","font","a","script"]),O=Mt({},[...cN,...dN,...gte]),z=Mt({},[...hN,...fte]),q=function(Y){let Le=v(Y);(!Le||!Le.tagName)&&(Le={namespaceURI:Jo,tagName:"template"});const Ve=V0(Y.tagName),Oi=V0(Le.tagName);return na[Y.namespaceURI]?Y.namespaceURI===ic?Le.namespaceURI===dn?Ve==="svg":Le.namespaceURI===Za?Ve==="svg"&&(Oi==="annotation-xml"||ut[Oi]):!!O[Ve]:Y.namespaceURI===Za?Le.namespaceURI===dn?Ve==="math":Le.namespaceURI===ic?Ve==="math"&&R[Oi]:!!z[Ve]:Y.namespaceURI===dn?Le.namespaceURI===ic&&!R[Oi]||Le.namespaceURI===Za&&!ut[Oi]?!1:!z[Ve]&&(M[Ve]||!O[Ve]):!!(mu==="application/xhtml+xml"&&na[Y.namespaceURI]):!1},ee=function(Y){$b(e.removed,{element:Y});try{v(Y).removeChild(Y)}catch{_(Y)}},ae=function(Y,Le){try{$b(e.removed,{attribute:Le.getAttributeNode(Y),from:Le})}catch{$b(e.removed,{attribute:null,from:Le})}if(Le.removeAttribute(Y),Y==="is"&&!Ne[Y])if(_d||pm)try{ee(Le)}catch{}else try{Le.setAttribute(Y,"")}catch{}},De=function(Y){let Le=null,Ve=null;if(Hb)Y="<remove></remove>"+Y;else{const is=i7(Y,/^[\r\n\t ]+/);Ve=is&&is[0]}mu==="application/xhtml+xml"&&Jo===dn&&(Y='<html xmlns="http://www.w3.org/1999/xhtml"><head></head><body>'+Y+"</body></html>");const Oi=w?w.createHTML(Y):Y;if(Jo===dn)try{Le=new u().parseFromString(Oi,mu)}catch{}if(!Le||!Le.documentElement){Le=L.createDocument(Jo,"template",null);try{Le.documentElement.innerHTML=vm?S:Oi}catch{}}const ks=Le.body||Le.documentElement;return Y&&Ve&&ks.insertBefore(t.createTextNode(Ve),ks.childNodes[0]||null),Jo===dn?N.call(Le,ec?"html":"body")[0]:ec?Le.documentElement:ks},Me=function(Y){return k.call(Y.ownerDocument||Y,Y,c.SHOW_ELEMENT|c.SHOW_COMMENT|c.SHOW_TEXT|c.SHOW_PROCESSING_INSTRUCTION|c.SHOW_CDATA_SECTION,null)},at=function(Y){return Y instanceof h&&(typeof Y.nodeName!="string"||typeof Y.textContent!="string"||typeof Y.removeChild!="function"||!(Y.attributes instanceof d)||typeof Y.removeAttribute!="function"||typeof Y.setAttribute!="function"||typeof Y.namespaceURI!="string"||typeof Y.insertBefore!="function"||typeof Y.hasChildNodes!="function")},Et=function(Y){return typeof a=="function"&&Y instanceof a},Ft=function(Y,Le,Ve){W[Y]&&NS(W[Y],Oi=>{Oi.call(e,Le,Ve,Cd)})},yi=function(Y){let Le=null;if(Ft("beforeSanitizeElements",Y,null),at(Y))return ee(Y),!0;const Ve=Cn(Y.nodeName);if(Ft("uponSanitizeElement",Y,{tagName:Ve,allowedTags:_e}),Y.hasChildNodes()&&!Et(Y.firstElementChild)&&Ks(/<[/\w]/g,Y.innerHTML)&&Ks(/<[/\w]/g,Y.textContent)||Y.nodeType===Gb.progressingInstruction||Ir&&Y.nodeType===Gb.comment&&Ks(/<[/\w]/g,Y.data))return ee(Y),!0;if(!_e[Ve]||Ti[Ve]){if(!Ti[Ve]&&$s(Ve)&&(ht.tagNameCheck instanceof RegExp&&Ks(ht.tagNameCheck,Ve)||ht.tagNameCheck instanceof Function&&ht.tagNameCheck(Ve)))return!1;if(bm&&!bd[Ve]){const Oi=v(Y)||Y.parentNode,ks=C(Y)||Y.childNodes;if(ks&&Oi){const is=ks.length;for(let wo=is-1;wo>=0;--wo){const Ya=p(ks[wo],!0);Ya.__removalCount=(Y.__removalCount||0)+1,Oi.insertBefore(Ya,b(Y))}}}return ee(Y),!0}return Y instanceof l&&!q(Y)||(Ve==="noscript"||Ve==="noembed"||Ve==="noframes")&&Ks(/<\/no(script|embed|frames)/i,Y.innerHTML)?(ee(Y),!0):(zi&&Y.nodeType===Gb.text&&(Le=Y.textContent,NS([V,$,H],Oi=>{Le=jb(Le,Oi," ")}),Y.textContent!==Le&&($b(e.removed,{element:Y.cloneNode()}),Y.textContent=Le)),Ft("afterSanitizeElements",Y,null),!1)},Li=function(Y,Le,Ve){if(uu&&(Le==="id"||Le==="name")&&(Ve in t||Ve in XE))return!1;if(!(Dr&&!ts[Le]&&Ks(j,Le))){if(!(xs&&Ks(J,Le))){if(!Ne[Le]||ts[Le]){if(!($s(Y)&&(ht.tagNameCheck instanceof RegExp&&Ks(ht.tagNameCheck,Y)||ht.tagNameCheck instanceof Function&&ht.tagNameCheck(Y))&&(ht.attributeNameCheck instanceof RegExp&&Ks(ht.attributeNameCheck,Le)||ht.attributeNameCheck instanceof Function&&ht.attributeNameCheck(Le))||Le==="is"&&ht.allowCustomizedBuiltInElements&&(ht.tagNameCheck instanceof RegExp&&Ks(ht.tagNameCheck,Ve)||ht.tagNameCheck instanceof Function&&ht.tagNameCheck(Ve))))return!1}else if(!tc[Le]){if(!Ks(Pe,jb(Ve,fe,""))){if(!((Le==="src"||Le==="xlink:href"||Le==="href")&&Y!=="script"&&cte(Ve,"data:")===0&&vS[Y])){if(!(Co&&!Ks(he,jb(Ve,fe,"")))){if(Ve)return!1}}}}}}return!0},$s=function(Y){return Y!=="annotation-xml"&&i7(Y,ue)},er=function(Y){Ft("beforeSanitizeAttributes",Y,null);const{attributes:Le}=Y;if(!Le)return;const Ve={attrName:"",attrValue:"",keepAttr:!0,allowedAttributes:Ne};let Oi=Le.length;for(;Oi--;){const ks=Le[Oi],{name:is,namespaceURI:wo,value:Ya}=ks,Ub=Cn(is);let js=is==="value"?Ya:dte(Ya);if(Ve.attrName=Ub,Ve.attrValue=js,Ve.keepAttr=!0,Ve.forceKeepAttr=void 0,Ft("uponSanitizeAttribute",Y,Ve),js=Ve.attrValue,Ve.forceKeepAttr||(ae(is,Y),!Ve.keepAttr))continue;if(!hu&&Ks(/\/>/i,js)){ae(is,Y);continue}zi&&NS([V,$,H],NB=>{js=jb(js,NB," ")});const EB=Cn(Y.nodeName);if(Li(EB,Ub,js)){if(bS&&(Ub==="id"||Ub==="name")&&(ae(is,Y),js=CS+js),Ir&&Ks(/((--!?|])>)|<\/(style|title)/i,js)){ae(is,Y);continue}if(w&&typeof g=="object"&&typeof g.getAttributeType=="function"&&!wo)switch(g.getAttributeType(EB,Ub)){case"TrustedHTML":{js=w.createHTML(js);break}case"TrustedScriptURL":{js=w.createScriptURL(js);break}}try{wo?Y.setAttributeNS(wo,is,js):Y.setAttribute(is,js),at(Y)?ee(Y):t7(e.removed)}catch{}}}Ft("afterSanitizeAttributes",Y,null)},zb=function Ye(Y){let Le=null;const Ve=Me(Y);for(Ft("beforeSanitizeShadowDOM",Y,null);Le=Ve.nextNode();)Ft("uponSanitizeShadowNode",Le,null),!yi(Le)&&(Le.content instanceof s&&Ye(Le.content),er(Le));Ft("afterSanitizeShadowDOM",Y,null)};return e.sanitize=function(Ye){let Y=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},Le=null,Ve=null,Oi=null,ks=null;if(vm=!Ye,vm&&(Ye="<!-->"),typeof Ye!="string"&&!Et(Ye))if(typeof Ye.toString=="function"){if(Ye=Ye.toString(),typeof Ye!="string")throw Kb("dirty is not a string, aborting")}else throw Kb("toString is not a function");if(!e.isSupported)return Ye;if(Bb||Vb(Y),e.removed=[],typeof Ye=="string"&&(gu=!1),gu){if(Ye.nodeName){const Ya=Cn(Ye.nodeName);if(!_e[Ya]||Ti[Ya])throw Kb("root node is forbidden and cannot be sanitized in-place")}}else if(Ye instanceof a)Le=De("<!---->"),Ve=Le.ownerDocument.importNode(Ye,!0),Ve.nodeType===Gb.element&&Ve.nodeName==="BODY"||Ve.nodeName==="HTML"?Le=Ve:Le.appendChild(Ve);else{if(!_d&&!zi&&!ec&&Ye.indexOf("<")===-1)return w&&_m?w.createHTML(Ye):Ye;if(Le=De(Ye),!Le)return _d?null:_m?S:""}Le&&Hb&&ee(Le.firstChild);const is=Me(gu?Ye:Le);for(;Oi=is.nextNode();)yi(Oi)||(Oi.content instanceof s&&zb(Oi.content),er(Oi));if(gu)return Ye;if(_d){if(pm)for(ks=D.call(Le.ownerDocument);Le.firstChild;)ks.appendChild(Le.firstChild);else ks=Le;return(Ne.shadowroot||Ne.shadowrootmode)&&(ks=B.call(i,ks,!0)),ks}let wo=ec?Le.outerHTML:Le.innerHTML;return ec&&_e["!doctype"]&&Le.ownerDocument&&Le.ownerDocument.doctype&&Le.ownerDocument.doctype.name&&Ks(pU,Le.ownerDocument.doctype.name)&&(wo="<!DOCTYPE "+Le.ownerDocument.doctype.name+`>
`+wo),zi&&NS([V,$,H],Ya=>{wo=jb(wo,Ya," ")}),w&&_m?w.createHTML(wo):wo},e.setConfig=function(){let Ye=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};Vb(Ye),Bb=!0},e.clearConfig=function(){Cd=null,Bb=!1},e.isValidAttribute=function(Ye,Y,Le){Cd||Vb({});const Ve=Cn(Ye),Oi=Cn(Y);return Li(Ve,Oi,Le)},e.addHook=function(Ye,Y){typeof Y=="function"&&(W[Ye]=W[Ye]||[],$b(W[Ye],Y))},e.removeHook=function(Ye){if(W[Ye])return t7(W[Ye])},e.removeHooks=function(Ye){W[Ye]&&(W[Ye]=[])},e.removeAllHooks=function(){W={}},e}var ql=_U();ql.version;ql.isSupported;const bU=ql.sanitize;ql.setConfig;ql.clearConfig;ql.isValidAttribute;const CU=ql.addHook,vU=ql.removeHook;ql.removeHooks;ql.removeAllHooks;var Fe;(function(o){o.inMemory="inmemory",o.vscode="vscode",o.internal="private",o.walkThrough="walkThrough",o.walkThroughSnippet="walkThroughSnippet",o.http="http",o.https="https",o.file="file",o.mailto="mailto",o.untitled="untitled",o.data="data",o.command="command",o.vscodeRemote="vscode-remote",o.vscodeRemoteResource="vscode-remote-resource",o.vscodeManagedRemoteResource="vscode-managed-remote-resource",o.vscodeUserData="vscode-userdata",o.vscodeCustomEditor="vscode-custom-editor",o.vscodeNotebookCell="vscode-notebook-cell",o.vscodeNotebookCellMetadata="vscode-notebook-cell-metadata",o.vscodeNotebookCellMetadataDiff="vscode-notebook-cell-metadata-diff",o.vscodeNotebookCellOutput="vscode-notebook-cell-output",o.vscodeNotebookCellOutputDiff="vscode-notebook-cell-output-diff",o.vscodeNotebookMetadata="vscode-notebook-metadata",o.vscodeInteractiveInput="vscode-interactive-input",o.vscodeSettings="vscode-settings",o.vscodeWorkspaceTrust="vscode-workspace-trust",o.vscodeTerminal="vscode-terminal",o.vscodeChatCodeBlock="vscode-chat-code-block",o.vscodeChatCodeCompareBlock="vscode-chat-code-compare-block",o.vscodeChatSesssion="vscode-chat-editor",o.webviewPanel="webview-panel",o.vscodeWebview="vscode-webview",o.extension="extension",o.vscodeFileResource="vscode-file",o.tmp="tmp",o.vsls="vsls",o.vscodeSourceControl="vscode-scm",o.commentsInput="comment",o.codeSetting="code-setting",o.outputChannel="output"})(Fe||(Fe={}));function mI(o,e){return Se.isUri(o)?ep(o.scheme,e):Y4(o,e+":")}function kM(o,...e){return e.some(t=>mI(o,t))}const xte="tkn";class kte{constructor(){this._hosts=Object.create(null),this._ports=Object.create(null),this._connectionTokens=Object.create(null),this._preferredWebSchema="http",this._delegate=null,this._serverRootPath="/"}setPreferredWebSchema(e){this._preferredWebSchema=e}get _remoteResourcesPath(){return Ai.join(this._serverRootPath,Fe.vscodeRemoteResource)}rewrite(e){if(this._delegate)try{return this._delegate(e)}catch(a){return Oe(a),e}const t=e.authority;let i=this._hosts[t];i&&i.indexOf(":")!==-1&&i.indexOf("[")===-1&&(i=`[${i}]`);const n=this._ports[t],s=this._connectionTokens[t];let r=`path=${encodeURIComponent(e.path)}`;return typeof s=="string"&&(r+=`&${xte}=${encodeURIComponent(s)}`),Se.from({scheme:Qf?this._preferredWebSchema:Fe.vscodeRemoteResource,authority:`${i}:${n}`,path:this._remoteResourcesPath,query:r})}}const wU=new kte,Dte="vscode-app",Nv=class Nv{asBrowserUri(e){const t=this.toUri(e);return this.uriToBrowserUri(t)}uriToBrowserUri(e){return e.scheme===Fe.vscodeRemote?wU.rewrite(e):e.scheme===Fe.file&&(Pl||PQ===`${Fe.vscodeFileResource}://${Nv.FALLBACK_AUTHORITY}`)?e.with({scheme:Fe.vscodeFileResource,authority:e.authority||Nv.FALLBACK_AUTHORITY,query:null,fragment:null}):e}toUri(e,t){if(Se.isUri(e))return e;if(globalThis._VSCODE_FILE_ROOT){const i=globalThis._VSCODE_FILE_ROOT;if(/^\w[\w\d+.-]*:\/\//.test(i))return Se.joinPath(Se.parse(i,!0),e);const n=oee(i,e);return Se.file(n)}return Se.parse(t.toUrl(e))}};Nv.FALLBACK_AUTHORITY=Dte;let DM=Nv;const pI=new DM;var IM;(function(o){const e=new Map([["1",{"Cross-Origin-Opener-Policy":"same-origin"}],["2",{"Cross-Origin-Embedder-Policy":"require-corp"}],["3",{"Cross-Origin-Opener-Policy":"same-origin","Cross-Origin-Embedder-Policy":"require-corp"}]]);o.CoopAndCoep=Object.freeze(e.get("3"));const t="vscode-coi";function i(s){let r;typeof s=="string"?r=new URL(s).searchParams:s instanceof URL?r=s.searchParams:Se.isUri(s)&&(r=new URL(s.toString(!0)).searchParams);const a=r==null?void 0:r.get(t);if(a)return e.get(a)}o.getHeadersFromQuery=i;function n(s,r,a){if(!globalThis.crossOriginIsolated)return;const l=r&&a?"3":a?"2":"1";s instanceof URLSearchParams?s.set(t,l):s[t]=l}o.addSearchParam=n})(IM||(IM={}));function _I(o){return bI(o,0)}function bI(o,e){switch(typeof o){case"object":return o===null?Nc(349,e):Array.isArray(o)?Ete(o,e):Nte(o,e);case"string":return o5(o,e);case"boolean":return Ite(o,e);case"number":return Nc(o,e);case"undefined":return Nc(937,e);default:return Nc(617,e)}}function Nc(o,e){return(e<<5)-e+o|0}function Ite(o,e){return Nc(o?433:863,e)}function o5(o,e){e=Nc(149417,e);for(let t=0,i=o.length;t<i;t++)e=Nc(o.charCodeAt(t),e);return e}function Ete(o,e){return e=Nc(104579,e),o.reduce((t,i)=>bI(i,t),e)}function Nte(o,e){return e=Nc(181387,e),Object.keys(o).sort().reduce((t,i)=>(t=o5(i,t),bI(o[i],t)),e)}function gN(o,e,t=32){const i=t-e,n=~((1<<i)-1);return(o<<e|(n&o)>>>i)>>>0}function l7(o,e=0,t=o.byteLength,i=0){for(let n=0;n<t;n++)o[e+n]=i}function Tte(o,e,t="0"){for(;o.length<e;)o=t+o;return o}function Zb(o,e=32){return o instanceof ArrayBuffer?Array.from(new Uint8Array(o)).map(t=>t.toString(16).padStart(2,"0")).join(""):Tte((o>>>0).toString(16),e/4)}const Nk=class Nk{constructor(){this._h0=1732584193,this._h1=4023233417,this._h2=2562383102,this._h3=271733878,this._h4=3285377520,this._buff=new Uint8Array(67),this._buffDV=new DataView(this._buff.buffer),this._buffLen=0,this._totalLen=0,this._leftoverHighSurrogate=0,this._finished=!1}update(e){const t=e.length;if(t===0)return;const i=this._buff;let n=this._buffLen,s=this._leftoverHighSurrogate,r,a;for(s!==0?(r=s,a=-1,s=0):(r=e.charCodeAt(0),a=0);;){let l=r;if(Gi(r))if(a+1<t){const c=e.charCodeAt(a+1);kf(c)?(a++,l=X4(r,c)):l=65533}else{s=r;break}else kf(r)&&(l=65533);if(n=this._push(i,n,l),a++,a<t)r=e.charCodeAt(a);else break}this._buffLen=n,this._leftoverHighSurrogate=s}_push(e,t,i){return i<128?e[t++]=i:i<2048?(e[t++]=192|(i&1984)>>>6,e[t++]=128|(i&63)>>>0):i<65536?(e[t++]=224|(i&61440)>>>12,e[t++]=128|(i&4032)>>>6,e[t++]=128|(i&63)>>>0):(e[t++]=240|(i&1835008)>>>18,e[t++]=128|(i&258048)>>>12,e[t++]=128|(i&4032)>>>6,e[t++]=128|(i&63)>>>0),t>=64&&(this._step(),t-=64,this._totalLen+=64,e[0]=e[64],e[1]=e[65],e[2]=e[66]),t}digest(){return this._finished||(this._finished=!0,this._leftoverHighSurrogate&&(this._leftoverHighSurrogate=0,this._buffLen=this._push(this._buff,this._buffLen,65533)),this._totalLen+=this._buffLen,this._wrapUp()),Zb(this._h0)+Zb(this._h1)+Zb(this._h2)+Zb(this._h3)+Zb(this._h4)}_wrapUp(){this._buff[this._buffLen++]=128,l7(this._buff,this._buffLen),this._buffLen>56&&(this._step(),l7(this._buff));const e=8*this._totalLen;this._buffDV.setUint32(56,Math.floor(e/4294967296),!1),this._buffDV.setUint32(60,e%4294967296,!1),this._step()}_step(){const e=Nk._bigBlock32,t=this._buffDV;for(let h=0;h<64;h+=4)e.setUint32(h,t.getUint32(h,!1),!1);for(let h=64;h<320;h+=4)e.setUint32(h,gN(e.getUint32(h-12,!1)^e.getUint32(h-32,!1)^e.getUint32(h-56,!1)^e.getUint32(h-64,!1),1),!1);let i=this._h0,n=this._h1,s=this._h2,r=this._h3,a=this._h4,l,c,d;for(let h=0;h<80;h++)h<20?(l=n&s|~n&r,c=1518500249):h<40?(l=n^s^r,c=1859775393):h<60?(l=n&s|n&r|s&r,c=2400959708):(l=n^s^r,c=3395469782),d=gN(i,5)+l+a+c+e.getUint32(h*4,!1)&4294967295,a=r,r=s,s=gN(n,30),n=i,i=d;this._h0=this._h0+i&4294967295,this._h1=this._h1+n&4294967295,this._h2=this._h2+s&4294967295,this._h3=this._h3+r&4294967295,this._h4=this._h4+a&4294967295}};Nk._bigBlock32=new DataView(new ArrayBuffer(320));let EM=Nk;const{getWindow:xe,getWindows:SU,getWindowsCount:Rte,getWindowId:Uy,getWindowById:c7,onDidRegisterWindow:CI,onWillUnregisterWindow:Mte,onDidUnregisterWindow:Ate}=function(){const o=new Map;xee(Rt,1);const e={window:Rt,disposables:new K};o.set(Rt.vscodeWindowId,e);const t=new P,i=new P,n=new P;function s(r,a){return(typeof r=="number"?o.get(r):void 0)??(a?e:void 0)}return{onDidRegisterWindow:t.event,onWillUnregisterWindow:n.event,onDidUnregisterWindow:i.event,registerWindow(r){if(o.has(r.vscodeWindowId))return F.None;const a=new K,l={window:r,disposables:a.add(new K)};return o.set(r.vscodeWindowId,l),a.add(ve(()=>{o.delete(r.vscodeWindowId),i.fire(r)})),a.add(U(r,ie.BEFORE_UNLOAD,()=>{n.fire(r)})),t.fire(l),a},getWindows(){return o.values()},getWindowsCount(){return o.size},getWindowId(r){return r.vscodeWindowId},hasWindow(r){return o.has(r)},getWindowById:s,getWindow(r){var c;const a=r;if((c=a==null?void 0:a.ownerDocument)!=null&&c.defaultView)return a.ownerDocument.defaultView.window;const l=r;return l!=null&&l.view?l.view.window:Rt},getDocument(r){return xe(r).document}}}();function pn(o){for(;o.firstChild;)o.firstChild.remove()}class Pte{constructor(e,t,i,n){this._node=e,this._type=t,this._handler=i,this._options=n||!1,this._node.addEventListener(this._type,this._handler,this._options)}dispose(){this._handler&&(this._node.removeEventListener(this._type,this._handler,this._options),this._node=null,this._handler=null)}}function U(o,e,t,i){return new Pte(o,e,t,i)}function yU(o,e){return function(t){return e(new Vr(o,t))}}function Ote(o){return function(e){return o(new Pt(e))}}const hi=function(e,t,i,n){let s=i;return t==="click"||t==="mousedown"||t==="contextmenu"?s=yU(xe(e),i):(t==="keydown"||t==="keypress"||t==="keyup")&&(s=Ote(i)),U(e,t,s,n)},Fte=function(e,t,i){const n=yU(xe(e),t);return Wte(e,n,i)};function Wte(o,e,t){return U(o,Pa&&i5.pointerEvents?ie.POINTER_DOWN:ie.MOUSE_DOWN,e,t)}function vC(o,e,t){return KC(o,e,t)}class fN extends uU{constructor(e,t){super(e,t)}}let $y,go;class r5 extends s5{constructor(e){super(),this.defaultTarget=e&&xe(e)}cancelAndSet(e,t,i){return super.cancelAndSet(e,t,i??this.defaultTarget)}}class mN{constructor(e,t=0){this._runner=e,this.priority=t,this._canceled=!1}dispose(){this._canceled=!0}execute(){if(!this._canceled)try{this._runner()}catch(e){Oe(e)}}static sort(e,t){return t.priority-e.priority}}(function(){const o=new Map,e=new Map,t=new Map,i=new Map,n=s=>{t.set(s,!1);const r=o.get(s)??[];for(e.set(s,r),o.set(s,[]),i.set(s,!0);r.length>0;)r.sort(mN.sort),r.shift().execute();i.set(s,!1)};go=(s,r,a=0)=>{const l=Uy(s),c=new mN(r,a);let d=o.get(l);return d||(d=[],o.set(l,d)),d.push(c),t.get(l)||(t.set(l,!0),s.requestAnimationFrame(()=>n(l))),c},$y=(s,r,a)=>{const l=Uy(s);if(i.get(l)){const c=new mN(r,a);let d=e.get(l);return d||(d=[],e.set(l,d)),d.push(c),c}else return go(s,r,a)}})();function vI(o){return xe(o).getComputedStyle(o,null)}function Hh(o,e){const t=xe(o),i=t.document;if(o!==i.body)return new kt(o.clientWidth,o.clientHeight);if(Pa&&(t!=null&&t.visualViewport))return new kt(t.visualViewport.width,t.visualViewport.height);if(t!=null&&t.innerWidth&&t.innerHeight)return new kt(t.innerWidth,t.innerHeight);if(i.body&&i.body.clientWidth&&i.body.clientHeight)return new kt(i.body.clientWidth,i.body.clientHeight);if(i.documentElement&&i.documentElement.clientWidth&&i.documentElement.clientHeight)return new kt(i.documentElement.clientWidth,i.documentElement.clientHeight);throw new Error("Unable to figure out browser width and height")}class Di{static convertToPixels(e,t){return parseFloat(t)||0}static getDimension(e,t,i){const n=vI(e),s=n?n.getPropertyValue(t):"0";return Di.convertToPixels(e,s)}static getBorderLeftWidth(e){return Di.getDimension(e,"border-left-width","borderLeftWidth")}static getBorderRightWidth(e){return Di.getDimension(e,"border-right-width","borderRightWidth")}static getBorderTopWidth(e){return Di.getDimension(e,"border-top-width","borderTopWidth")}static getBorderBottomWidth(e){return Di.getDimension(e,"border-bottom-width","borderBottomWidth")}static getPaddingLeft(e){return Di.getDimension(e,"padding-left","paddingLeft")}static getPaddingRight(e){return Di.getDimension(e,"padding-right","paddingRight")}static getPaddingTop(e){return Di.getDimension(e,"padding-top","paddingTop")}static getPaddingBottom(e){return Di.getDimension(e,"padding-bottom","paddingBottom")}static getMarginLeft(e){return Di.getDimension(e,"margin-left","marginLeft")}static getMarginTop(e){return Di.getDimension(e,"margin-top","marginTop")}static getMarginRight(e){return Di.getDimension(e,"margin-right","marginRight")}static getMarginBottom(e){return Di.getDimension(e,"margin-bottom","marginBottom")}}const eg=class eg{constructor(e,t){this.width=e,this.height=t}with(e=this.width,t=this.height){return e!==this.width||t!==this.height?new eg(e,t):this}static is(e){return typeof e=="object"&&typeof e.height=="number"&&typeof e.width=="number"}static lift(e){return e instanceof eg?e:new eg(e.width,e.height)}static equals(e,t){return e===t?!0:!e||!t?!1:e.width===t.width&&e.height===t.height}};eg.None=new eg(0,0);let kt=eg;function LU(o){let e=o.offsetParent,t=o.offsetTop,i=o.offsetLeft;for(;(o=o.parentNode)!==null&&o!==o.ownerDocument.body&&o!==o.ownerDocument.documentElement;){t-=o.scrollTop;const n=kU(o)?null:vI(o);n&&(i-=n.direction!=="rtl"?o.scrollLeft:-o.scrollLeft),o===e&&(i+=Di.getBorderLeftWidth(o),t+=Di.getBorderTopWidth(o),t+=o.offsetTop,i+=o.offsetLeft,e=o.offsetParent)}return{left:i,top:t}}function Bte(o,e,t){typeof e=="number"&&(o.style.width=`${e}px`),typeof t=="number"&&(o.style.height=`${t}px`)}function wi(o){const e=o.getBoundingClientRect(),t=xe(o);return{left:e.left+t.scrollX,top:e.top+t.scrollY,width:e.width,height:e.height}}function xU(o){let e=o,t=1;do{const i=vI(e).zoom;i!=null&&i!=="1"&&(t*=i),e=e.parentElement}while(e!==null&&e!==e.ownerDocument.documentElement);return t}function Xs(o){const e=Di.getMarginLeft(o)+Di.getMarginRight(o);return o.offsetWidth+e}function pN(o){const e=Di.getBorderLeftWidth(o)+Di.getBorderRightWidth(o),t=Di.getPaddingLeft(o)+Di.getPaddingRight(o);return o.offsetWidth-e-t}function Hte(o){const e=Di.getBorderTopWidth(o)+Di.getBorderBottomWidth(o),t=Di.getPaddingTop(o)+Di.getPaddingBottom(o);return o.offsetHeight-e-t}function Tc(o){const e=Di.getMarginTop(o)+Di.getMarginBottom(o);return o.offsetHeight+e}function Zi(o,e){return!!(e!=null&&e.contains(o))}function Vte(o,e,t){for(;o&&o.nodeType===o.ELEMENT_NODE;){if(o.classList.contains(e))return o;if(t){if(typeof t=="string"){if(o.classList.contains(t))return null}else if(o===t)return null}o=o.parentNode}return null}function _N(o,e,t){return!!Vte(o,e,t)}function kU(o){return o&&!!o.host&&!!o.mode}function jy(o){return!!If(o)}function If(o){var e;for(;o.parentNode;){if(o===((e=o.ownerDocument)==null?void 0:e.body))return null;o=o.parentNode}return kU(o)?o:null}function Dn(){let o=fb().activeElement;for(;o!=null&&o.shadowRoot;)o=o.shadowRoot.activeElement;return o}function wI(o){return Dn()===o}function DU(o){return Zi(Dn(),o)}function fb(){return Rte()<=1?Rt.document:Array.from(SU()).map(({window:e})=>e.document).find(e=>e.hasFocus())??Rt.document}function wC(){var e;return((e=fb().defaultView)==null?void 0:e.window)??Rt}const a5=new Map;function IU(){return new zte}class zte{constructor(){this._currentCssStyle="",this._styleSheet=void 0}setStyle(e){e!==this._currentCssStyle&&(this._currentCssStyle=e,this._styleSheet?this._styleSheet.innerText=e:this._styleSheet=$o(Rt.document.head,t=>t.innerText=e))}dispose(){this._styleSheet&&(this._styleSheet.remove(),this._styleSheet=void 0)}}function $o(o=Rt.document.head,e,t){const i=document.createElement("style");if(i.type="text/css",i.media="screen",e==null||e(i),o.appendChild(i),t&&t.add(ve(()=>i.remove())),o===Rt.document.head){const n=new Set;a5.set(i,n);for(const{window:s,disposables:r}of SU()){if(s===Rt)continue;const a=r.add(Ute(i,n,s));t==null||t.add(a)}}return i}function Ute(o,e,t){var s,r;const i=new K,n=o.cloneNode(!0);t.document.head.appendChild(n),i.add(ve(()=>n.remove()));for(const a of NU(o))(r=n.sheet)==null||r.insertRule(a.cssText,(s=n.sheet)==null?void 0:s.cssRules.length);return i.add($te.observe(o,i,{childList:!0})(()=>{n.textContent=o.textContent})),e.add(n),i.add(ve(()=>e.delete(n))),i}const $te=new class{constructor(){this.mutationObservers=new Map}observe(o,e,t){let i=this.mutationObservers.get(o);i||(i=new Map,this.mutationObservers.set(o,i));const n=_I(t);let s=i.get(n);if(s)s.users+=1;else{const r=new P,a=new MutationObserver(c=>r.fire(c));a.observe(o,t);const l=s={users:1,observer:a,onDidMutate:r.event};e.add(ve(()=>{l.users-=1,l.users===0&&(r.dispose(),a.disconnect(),i==null||i.delete(n),(i==null?void 0:i.size)===0&&this.mutationObservers.delete(o))})),i.set(n,s)}return s.onDidMutate}};let bN=null;function EU(){return bN||(bN=$o()),bN}function NU(o){var e,t;return(e=o==null?void 0:o.sheet)!=null&&e.rules?o.sheet.rules:(t=o==null?void 0:o.sheet)!=null&&t.cssRules?o.sheet.cssRules:[]}function Ky(o,e,t=EU()){var i;if(!(!t||!e)){(i=t.sheet)==null||i.insertRule(`${o} {${e}}`,0);for(const n of a5.get(t)??[])Ky(o,e,n)}}function NM(o,e=EU()){var n;if(!e)return;const t=NU(e),i=[];for(let s=0;s<t.length;s++){const r=t[s];jte(r)&&r.selectorText.indexOf(o)!==-1&&i.push(s)}for(let s=i.length-1;s>=0;s--)(n=e.sheet)==null||n.deleteRule(i[s]);for(const s of a5.get(e)??[])NM(o,s)}function jte(o){return typeof o.selectorText=="string"}function Xi(o){return o instanceof HTMLElement||o instanceof xe(o).HTMLElement}function d7(o){return o instanceof HTMLAnchorElement||o instanceof xe(o).HTMLAnchorElement}function Kte(o){return o instanceof SVGElement||o instanceof xe(o).SVGElement}function l5(o){return o instanceof MouseEvent||o instanceof xe(o).MouseEvent}function wc(o){return o instanceof KeyboardEvent||o instanceof xe(o).KeyboardEvent}const ie={CLICK:"click",AUXCLICK:"auxclick",DBLCLICK:"dblclick",MOUSE_UP:"mouseup",MOUSE_DOWN:"mousedown",MOUSE_OVER:"mouseover",MOUSE_MOVE:"mousemove",MOUSE_OUT:"mouseout",MOUSE_ENTER:"mouseenter",MOUSE_LEAVE:"mouseleave",MOUSE_WHEEL:"wheel",POINTER_UP:"pointerup",POINTER_DOWN:"pointerdown",POINTER_MOVE:"pointermove",POINTER_LEAVE:"pointerleave",CONTEXT_MENU:"contextmenu",KEY_DOWN:"keydown",KEY_UP:"keyup",BEFORE_UNLOAD:"beforeunload",CHANGE:"change",FOCUS:"focus",FOCUS_IN:"focusin",FOCUS_OUT:"focusout",BLUR:"blur",INPUT:"input",DRAG_START:"dragstart",DRAG:"drag",DRAG_ENTER:"dragenter",DRAG_LEAVE:"dragleave",DRAG_OVER:"dragover",DROP:"drop",DRAG_END:"dragend"};function qte(o){const e=o;return!!(e&&typeof e.preventDefault=="function"&&typeof e.stopPropagation=="function")}const nt={stop:(o,e)=>(o.preventDefault(),e&&o.stopPropagation(),o)};function Gte(o){const e=[];for(let t=0;o&&o.nodeType===o.ELEMENT_NODE;t++)e[t]=o.scrollTop,o=o.parentNode;return e}function Zte(o,e){for(let t=0;o&&o.nodeType===o.ELEMENT_NODE;t++)o.scrollTop!==e[t]&&(o.scrollTop=e[t]),o=o.parentNode}class qy extends F{static hasFocusWithin(e){if(Xi(e)){const t=If(e),i=t?t.activeElement:e.ownerDocument.activeElement;return Zi(i,e)}else{const t=e;return Zi(t.document.activeElement,t.document)}}constructor(e){super(),this._onDidFocus=this._register(new P),this.onDidFocus=this._onDidFocus.event,this._onDidBlur=this._register(new P),this.onDidBlur=this._onDidBlur.event;let t=qy.hasFocusWithin(e),i=!1;const n=()=>{i=!1,t||(t=!0,this._onDidFocus.fire())},s=()=>{t&&(i=!0,(Xi(e)?xe(e):e).setTimeout(()=>{i&&(i=!1,t=!1,this._onDidBlur.fire())},0))};this._refreshStateHandler=()=>{qy.hasFocusWithin(e)!==t&&(t?s():n())},this._register(U(e,ie.FOCUS,n,!0)),this._register(U(e,ie.BLUR,s,!0)),Xi(e)&&(this._register(U(e,ie.FOCUS_IN,()=>this._refreshStateHandler())),this._register(U(e,ie.FOCUS_OUT,()=>this._refreshStateHandler())))}}function Fa(o){return new qy(o)}function Yte(o,e){return o.after(e),e}function X(o,...e){if(o.append(...e),e.length===1&&typeof e[0]!="string")return e[0]}function c5(o,e){return o.insertBefore(e,o.firstChild),e}function _n(o,...e){o.innerText="",X(o,...e)}const Xte=/([\w\-]+)?(#([\w\-]+))?((\.([\w\-]+))*)/;var rw;(function(o){o.HTML="http://www.w3.org/1999/xhtml",o.SVG="http://www.w3.org/2000/svg"})(rw||(rw={}));function TU(o,e,t,...i){const n=Xte.exec(e);if(!n)throw new Error("Bad use of emmet");const s=n[1]||"div";let r;return o!==rw.HTML?r=document.createElementNS(o,s):r=document.createElement(s),n[3]&&(r.id=n[3]),n[4]&&(r.className=n[4].replace(/\./g," ").trim()),t&&Object.entries(t).forEach(([a,l])=>{typeof l>"u"||(/^on\w+$/.test(a)?r[a]=l:a==="selected"?l&&r.setAttribute(a,"true"):r.setAttribute(a,l))}),r.append(...i),r}function ne(o,e,...t){return TU(rw.HTML,o,e,...t)}ne.SVG=function(o,e,...t){return TU(rw.SVG,o,e,...t)};function Qte(o,...e){o?to(...e):rs(...e)}function to(...o){for(const e of o)e.style.display="",e.removeAttribute("aria-hidden")}function rs(...o){for(const e of o)e.style.display="none",e.setAttribute("aria-hidden","true")}function h7(o,e){const t=o.devicePixelRatio*e;return Math.max(1,Math.floor(t))/o.devicePixelRatio}function RU(o){Rt.open(o,"_blank","noopener")}function Jte(o,e){const t=()=>{e(),i=go(o,t)};let i=go(o,t);return ve(()=>i.dispose())}wU.setPreferredWebSchema(/^https:/.test(Rt.location.href)?"https":"http");function td(o){return o?`url('${pI.uriToBrowserUri(o).toString(!0).replace(/'/g,"%27")}')`:"url('')"}function CN(o){return`'${o.replace(/'/g,"%27")}'`}function jc(o,e){if(o!==void 0){const t=o.match(/^\s*var\((.+)\)$/);if(t){const i=t[1].split(",",2);return i.length===2&&(e=jc(i[1].trim(),e)),`var(${i[0]}, ${e})`}return o}return e}function eie(o,e=!1){const t=document.createElement("a");return CU("afterSanitizeAttributes",i=>{for(const n of["href","src"])if(i.hasAttribute(n)){const s=i.getAttribute(n);if(n==="href"&&s.startsWith("#"))continue;if(t.href=s,!o.includes(t.protocol.replace(/:$/,""))){if(e&&n==="src"&&t.href.startsWith("data:"))continue;i.removeAttribute(n)}}}),ve(()=>{vU("afterSanitizeAttributes")})}const tie=Object.freeze(["a","abbr","b","bdo","blockquote","br","caption","cite","code","col","colgroup","dd","del","details","dfn","div","dl","dt","em","figcaption","figure","h1","h2","h3","h4","h5","h6","hr","i","img","input","ins","kbd","label","li","mark","ol","p","pre","q","rp","rt","ruby","samp","small","small","source","span","strike","strong","sub","summary","sup","table","tbody","td","tfoot","th","thead","time","tr","tt","u","ul","var","video","wbr"]);class Rc extends P{constructor(){super(),this._subscriptions=new K,this._keyStatus={altKey:!1,shiftKey:!1,ctrlKey:!1,metaKey:!1},this._subscriptions.add(oe.runAndSubscribe(CI,({window:e,disposables:t})=>this.registerListeners(e,t),{window:Rt,disposables:this._subscriptions}))}registerListeners(e,t){t.add(U(e,"keydown",i=>{if(i.defaultPrevented)return;const n=new Pt(i);if(!(n.keyCode===6&&i.repeat)){if(i.altKey&&!this._keyStatus.altKey)this._keyStatus.lastKeyPressed="alt";else if(i.ctrlKey&&!this._keyStatus.ctrlKey)this._keyStatus.lastKeyPressed="ctrl";else if(i.metaKey&&!this._keyStatus.metaKey)this._keyStatus.lastKeyPressed="meta";else if(i.shiftKey&&!this._keyStatus.shiftKey)this._keyStatus.lastKeyPressed="shift";else if(n.keyCode!==6)this._keyStatus.lastKeyPressed=void 0;else return;this._keyStatus.altKey=i.altKey,this._keyStatus.ctrlKey=i.ctrlKey,this._keyStatus.metaKey=i.metaKey,this._keyStatus.shiftKey=i.shiftKey,this._keyStatus.lastKeyPressed&&(this._keyStatus.event=i,this.fire(this._keyStatus))}},!0)),t.add(U(e,"keyup",i=>{i.defaultPrevented||(!i.altKey&&this._keyStatus.altKey?this._keyStatus.lastKeyReleased="alt":!i.ctrlKey&&this._keyStatus.ctrlKey?this._keyStatus.lastKeyReleased="ctrl":!i.metaKey&&this._keyStatus.metaKey?this._keyStatus.lastKeyReleased="meta":!i.shiftKey&&this._keyStatus.shiftKey?this._keyStatus.lastKeyReleased="shift":this._keyStatus.lastKeyReleased=void 0,this._keyStatus.lastKeyPressed!==this._keyStatus.lastKeyReleased&&(this._keyStatus.lastKeyPressed=void 0),this._keyStatus.altKey=i.altKey,this._keyStatus.ctrlKey=i.ctrlKey,this._keyStatus.metaKey=i.metaKey,this._keyStatus.shiftKey=i.shiftKey,this._keyStatus.lastKeyReleased&&(this._keyStatus.event=i,this.fire(this._keyStatus)))},!0)),t.add(U(e.document.body,"mousedown",()=>{this._keyStatus.lastKeyPressed=void 0},!0)),t.add(U(e.document.body,"mouseup",()=>{this._keyStatus.lastKeyPressed=void 0},!0)),t.add(U(e.document.body,"mousemove",i=>{i.buttons&&(this._keyStatus.lastKeyPressed=void 0)},!0)),t.add(U(e,"blur",()=>{this.resetKeyStatus()}))}get keyStatus(){return this._keyStatus}resetKeyStatus(){this.doResetKeyStatus(),this.fire(this._keyStatus)}doResetKeyStatus(){this._keyStatus={altKey:!1,shiftKey:!1,ctrlKey:!1,metaKey:!1}}static getInstance(){return Rc.instance||(Rc.instance=new Rc),Rc.instance}dispose(){super.dispose(),this._subscriptions.dispose()}}class iie extends F{constructor(e,t){super(),this.element=e,this.callbacks=t,this.counter=0,this.dragStartTime=0,this.registerListeners()}registerListeners(){this.callbacks.onDragStart&&this._register(U(this.element,ie.DRAG_START,e=>{var t,i;(i=(t=this.callbacks).onDragStart)==null||i.call(t,e)})),this.callbacks.onDrag&&this._register(U(this.element,ie.DRAG,e=>{var t,i;(i=(t=this.callbacks).onDrag)==null||i.call(t,e)})),this._register(U(this.element,ie.DRAG_ENTER,e=>{var t,i;this.counter++,this.dragStartTime=e.timeStamp,(i=(t=this.callbacks).onDragEnter)==null||i.call(t,e)})),this._register(U(this.element,ie.DRAG_OVER,e=>{var t,i;e.preventDefault(),(i=(t=this.callbacks).onDragOver)==null||i.call(t,e,e.timeStamp-this.dragStartTime)})),this._register(U(this.element,ie.DRAG_LEAVE,e=>{var t,i;this.counter--,this.counter===0&&(this.dragStartTime=0,(i=(t=this.callbacks).onDragLeave)==null||i.call(t,e))})),this._register(U(this.element,ie.DRAG_END,e=>{var t,i;this.counter=0,this.dragStartTime=0,(i=(t=this.callbacks).onDragEnd)==null||i.call(t,e)})),this._register(U(this.element,ie.DROP,e=>{var t,i;this.counter=0,this.dragStartTime=0,(i=(t=this.callbacks).onDrop)==null||i.call(t,e)}))}}const MU=/(?<tag>[\w\-]+)?(?:#(?<id>[\w\-]+))?(?<class>(?:\.(?:[\w\-]+))*)(?:@(?<name>(?:[\w\_])+))?/;function Xe(o,...e){let t,i;Array.isArray(e[0])?(t={},i=e[0]):(t=e[0]||{},i=e[1]);const n=MU.exec(o);if(!n||!n.groups)throw new Error("Bad use of h");const s=n.groups.tag||"div",r=document.createElement(s);n.groups.id&&(r.id=n.groups.id);const a=[];if(n.groups.class)for(const c of n.groups.class.split("."))c!==""&&a.push(c);if(t.className!==void 0)for(const c of t.className.split("."))c!==""&&a.push(c);a.length>0&&(r.className=a.join(" "));const l={};if(n.groups.name&&(l[n.groups.name]=r),i)for(const c of i)Xi(c)?r.appendChild(c):typeof c=="string"?r.append(c):"root"in c&&(Object.assign(l,c),r.appendChild(c.root));for(const[c,d]of Object.entries(t))if(c!=="className")if(c==="style")for(const[h,u]of Object.entries(d))r.style.setProperty(Gy(h),typeof u=="number"?u+"px":""+u);else c==="tabIndex"?r.tabIndex=d:r.setAttribute(Gy(c),d.toString());return l.root=r,l}function Sm(o,...e){let t,i;Array.isArray(e[0])?(t={},i=e[0]):(t=e[0]||{},i=e[1]);const n=MU.exec(o);if(!n||!n.groups)throw new Error("Bad use of h");const s=n.groups.tag||"div",r=document.createElementNS("http://www.w3.org/2000/svg",s);n.groups.id&&(r.id=n.groups.id);const a=[];if(n.groups.class)for(const c of n.groups.class.split("."))c!==""&&a.push(c);if(t.className!==void 0)for(const c of t.className.split("."))c!==""&&a.push(c);a.length>0&&(r.className=a.join(" "));const l={};if(n.groups.name&&(l[n.groups.name]=r),i)for(const c of i)Xi(c)?r.appendChild(c):typeof c=="string"?r.append(c):"root"in c&&(Object.assign(l,c),r.appendChild(c.root));for(const[c,d]of Object.entries(t))if(c!=="className")if(c==="style")for(const[h,u]of Object.entries(d))r.style.setProperty(Gy(h),typeof u=="number"?u+"px":""+u);else c==="tabIndex"?r.tabIndex=d:r.setAttribute(Gy(c),d.toString());return l.root=r,l}function Gy(o){return o.replace(/([a-z])([A-Z])/g,"$1-$2").toLowerCase()}class nie extends F{constructor(e){super(),this._onDidChange=this._register(new P),this.onDidChange=this._onDidChange.event,this._listener=()=>this._handleChange(e,!0),this._mediaQueryList=null,this._handleChange(e,!1)}_handleChange(e,t){var i;(i=this._mediaQueryList)==null||i.removeEventListener("change",this._listener),this._mediaQueryList=e.matchMedia(`(resolution: ${e.devicePixelRatio}dppx)`),this._mediaQueryList.addEventListener("change",this._listener),t&&this._onDidChange.fire()}}class sie extends F{get value(){return this._value}constructor(e){super(),this._onDidChange=this._register(new P),this.onDidChange=this._onDidChange.event,this._value=this._getPixelRatio(e);const t=this._register(new nie(e));this._register(t.onDidChange(()=>{this._value=this._getPixelRatio(e),this._onDidChange.fire(this._value)}))}_getPixelRatio(e){const t=document.createElement("canvas").getContext("2d"),i=e.devicePixelRatio||1,n=t.webkitBackingStorePixelRatio||t.mozBackingStorePixelRatio||t.msBackingStorePixelRatio||t.oBackingStorePixelRatio||t.backingStorePixelRatio||1;return i/n}}class oie{constructor(){this.mapWindowIdToPixelRatioMonitor=new Map}_getOrCreatePixelRatioMonitor(e){const t=Uy(e);let i=this.mapWindowIdToPixelRatioMonitor.get(t);return i||(i=new sie(e),this.mapWindowIdToPixelRatioMonitor.set(t,i),oe.once(Ate)(({vscodeWindowId:n})=>{n===t&&(i==null||i.dispose(),this.mapWindowIdToPixelRatioMonitor.delete(t))})),i}getInstance(e){return this._getOrCreatePixelRatioMonitor(e)}}const aw=new oie;class AU{constructor(e){this.domNode=e,this._maxWidth="",this._width="",this._height="",this._top="",this._left="",this._bottom="",this._right="",this._paddingLeft="",this._fontFamily="",this._fontWeight="",this._fontSize="",this._fontStyle="",this._fontFeatureSettings="",this._fontVariationSettings="",this._textDecoration="",this._lineHeight="",this._letterSpacing="",this._className="",this._display="",this._position="",this._visibility="",this._color="",this._backgroundColor="",this._layerHint=!1,this._contain="none",this._boxShadow=""}setMaxWidth(e){const t=sa(e);this._maxWidth!==t&&(this._maxWidth=t,this.domNode.style.maxWidth=this._maxWidth)}setWidth(e){const t=sa(e);this._width!==t&&(this._width=t,this.domNode.style.width=this._width)}setHeight(e){const t=sa(e);this._height!==t&&(this._height=t,this.domNode.style.height=this._height)}setTop(e){const t=sa(e);this._top!==t&&(this._top=t,this.domNode.style.top=this._top)}setLeft(e){const t=sa(e);this._left!==t&&(this._left=t,this.domNode.style.left=this._left)}setBottom(e){const t=sa(e);this._bottom!==t&&(this._bottom=t,this.domNode.style.bottom=this._bottom)}setRight(e){const t=sa(e);this._right!==t&&(this._right=t,this.domNode.style.right=this._right)}setPaddingLeft(e){const t=sa(e);this._paddingLeft!==t&&(this._paddingLeft=t,this.domNode.style.paddingLeft=this._paddingLeft)}setFontFamily(e){this._fontFamily!==e&&(this._fontFamily=e,this.domNode.style.fontFamily=this._fontFamily)}setFontWeight(e){this._fontWeight!==e&&(this._fontWeight=e,this.domNode.style.fontWeight=this._fontWeight)}setFontSize(e){const t=sa(e);this._fontSize!==t&&(this._fontSize=t,this.domNode.style.fontSize=this._fontSize)}setFontStyle(e){this._fontStyle!==e&&(this._fontStyle=e,this.domNode.style.fontStyle=this._fontStyle)}setFontFeatureSettings(e){this._fontFeatureSettings!==e&&(this._fontFeatureSettings=e,this.domNode.style.fontFeatureSettings=this._fontFeatureSettings)}setFontVariationSettings(e){this._fontVariationSettings!==e&&(this._fontVariationSettings=e,this.domNode.style.fontVariationSettings=this._fontVariationSettings)}setTextDecoration(e){this._textDecoration!==e&&(this._textDecoration=e,this.domNode.style.textDecoration=this._textDecoration)}setLineHeight(e){const t=sa(e);this._lineHeight!==t&&(this._lineHeight=t,this.domNode.style.lineHeight=this._lineHeight)}setLetterSpacing(e){const t=sa(e);this._letterSpacing!==t&&(this._letterSpacing=t,this.domNode.style.letterSpacing=this._letterSpacing)}setClassName(e){this._className!==e&&(this._className=e,this.domNode.className=this._className)}toggleClassName(e,t){this.domNode.classList.toggle(e,t),this._className=this.domNode.className}setDisplay(e){this._display!==e&&(this._display=e,this.domNode.style.display=this._display)}setPosition(e){this._position!==e&&(this._position=e,this.domNode.style.position=this._position)}setVisibility(e){this._visibility!==e&&(this._visibility=e,this.domNode.style.visibility=this._visibility)}setColor(e){this._color!==e&&(this._color=e,this.domNode.style.color=this._color)}setBackgroundColor(e){this._backgroundColor!==e&&(this._backgroundColor=e,this.domNode.style.backgroundColor=this._backgroundColor)}setLayerHinting(e){this._layerHint!==e&&(this._layerHint=e,this.domNode.style.transform=this._layerHint?"translate3d(0px, 0px, 0px)":"")}setBoxShadow(e){this._boxShadow!==e&&(this._boxShadow=e,this.domNode.style.boxShadow=e)}setContain(e){this._contain!==e&&(this._contain=e,this.domNode.style.contain=this._contain)}setAttribute(e,t){this.domNode.setAttribute(e,t)}removeAttribute(e){this.domNode.removeAttribute(e)}appendChild(e){this.domNode.appendChild(e.domNode)}removeChild(e){this.domNode.removeChild(e.domNode)}}function sa(o){return typeof o=="number"?`${o}px`:o}function St(o){return new AU(o)}function bn(o,e){o instanceof AU?(o.setFontFamily(e.getMassagedFontFamily()),o.setFontWeight(e.fontWeight),o.setFontSize(e.fontSize),o.setFontFeatureSettings(e.fontFeatureSettings),o.setFontVariationSettings(e.fontVariationSettings),o.setLineHeight(e.lineHeight),o.setLetterSpacing(e.letterSpacing)):(o.style.fontFamily=e.getMassagedFontFamily(),o.style.fontWeight=e.fontWeight,o.style.fontSize=e.fontSize+"px",o.style.fontFeatureSettings=e.fontFeatureSettings,o.style.fontVariationSettings=e.fontVariationSettings,o.style.lineHeight=e.lineHeight+"px",o.style.letterSpacing=e.letterSpacing+"px")}class rie{constructor(e,t){this.chr=e,this.type=t,this.width=0}fulfill(e){this.width=e}}class d5{constructor(e,t){this._bareFontInfo=e,this._requests=t,this._container=null,this._testElements=null}read(e){var t;this._createDomElements(),e.document.body.appendChild(this._container),this._readFromDomElements(),(t=this._container)==null||t.remove(),this._container=null,this._testElements=null}_createDomElements(){const e=document.createElement("div");e.style.position="absolute",e.style.top="-50000px",e.style.width="50000px";const t=document.createElement("div");bn(t,this._bareFontInfo),e.appendChild(t);const i=document.createElement("div");bn(i,this._bareFontInfo),i.style.fontWeight="bold",e.appendChild(i);const n=document.createElement("div");bn(n,this._bareFontInfo),n.style.fontStyle="italic",e.appendChild(n);const s=[];for(const r of this._requests){let a;r.type===0&&(a=t),r.type===2&&(a=i),r.type===1&&(a=n),a.appendChild(document.createElement("br"));const l=document.createElement("span");d5._render(l,r),a.appendChild(l),s.push(l)}this._container=e,this._testElements=s}static _render(e,t){if(t.chr===" "){let i=" ";for(let n=0;n<8;n++)i+=i;e.innerText=i}else{let i=t.chr;for(let n=0;n<8;n++)i+=i;e.textContent=i}}_readFromDomElements(){for(let e=0,t=this._requests.length;e<t;e++){const i=this._requests[e],n=this._testElements[e];i.fulfill(n.offsetWidth/256)}}}function aie(o,e,t){new d5(e,t).read(o)}const lr=new class{constructor(){this._zoomLevel=0,this._onDidChangeZoomLevel=new P,this.onDidChangeZoomLevel=this._onDidChangeZoomLevel.event}getZoomLevel(){return this._zoomLevel}setZoomLevel(o){o=Math.min(Math.max(-5,o),20),this._zoomLevel!==o&&(this._zoomLevel=o,this._onDidChangeZoomLevel.fire(this._zoomLevel))}},lie=Je?1.5:1.35,vN=8;class mg{static createFromValidatedSettings(e,t,i){const n=e.get(49),s=e.get(53),r=e.get(52),a=e.get(51),l=e.get(54),c=e.get(67),d=e.get(64);return mg._create(n,s,r,a,l,c,d,t,i)}static _create(e,t,i,n,s,r,a,l,c){r===0?r=lie*i:r<vN&&(r=r*i),r=Math.round(r),r<vN&&(r=vN);const d=1+(c?0:lr.getZoomLevel()*.1);return i*=d,r*=d,s===Jv.TRANSLATE&&(t==="normal"||t==="bold"?s=Jv.OFF:(s=`'wght' ${parseInt(t,10)}`,t="normal")),new mg({pixelRatio:l,fontFamily:e,fontWeight:t,fontSize:i,fontFeatureSettings:n,fontVariationSettings:s,lineHeight:r,letterSpacing:a})}constructor(e){this._bareFontInfoBrand=void 0,this.pixelRatio=e.pixelRatio,this.fontFamily=String(e.fontFamily),this.fontWeight=String(e.fontWeight),this.fontSize=e.fontSize,this.fontFeatureSettings=e.fontFeatureSettings,this.fontVariationSettings=e.fontVariationSettings,this.lineHeight=e.lineHeight|0,this.letterSpacing=e.letterSpacing}getId(){return`${this.pixelRatio}-${this.fontFamily}-${this.fontWeight}-${this.fontSize}-${this.fontFeatureSettings}-${this.fontVariationSettings}-${this.lineHeight}-${this.letterSpacing}`}getMassagedFontFamily(){const e=fs.fontFamily,t=mg._wrapInQuotes(this.fontFamily);return e&&this.fontFamily!==e?`${t}, ${e}`:t}static _wrapInQuotes(e){return/[,"']/.test(e)?e:/[+ ]/.test(e)?`"${e}"`:e}}const cie=2;class TM extends mg{constructor(e,t){super(e),this._editorStylingBrand=void 0,this.version=cie,this.isTrusted=t,this.isMonospace=e.isMonospace,this.typicalHalfwidthCharacterWidth=e.typicalHalfwidthCharacterWidth,this.typicalFullwidthCharacterWidth=e.typicalFullwidthCharacterWidth,this.canUseHalfwidthRightwardsArrow=e.canUseHalfwidthRightwardsArrow,this.spaceWidth=e.spaceWidth,this.middotWidth=e.middotWidth,this.wsmiddotWidth=e.wsmiddotWidth,this.maxDigitWidth=e.maxDigitWidth}equals(e){return this.fontFamily===e.fontFamily&&this.fontWeight===e.fontWeight&&this.fontSize===e.fontSize&&this.fontFeatureSettings===e.fontFeatureSettings&&this.fontVariationSettings===e.fontVariationSettings&&this.lineHeight===e.lineHeight&&this.letterSpacing===e.letterSpacing&&this.typicalHalfwidthCharacterWidth===e.typicalHalfwidthCharacterWidth&&this.typicalFullwidthCharacterWidth===e.typicalFullwidthCharacterWidth&&this.canUseHalfwidthRightwardsArrow===e.canUseHalfwidthRightwardsArrow&&this.spaceWidth===e.spaceWidth&&this.middotWidth===e.middotWidth&&this.wsmiddotWidth===e.wsmiddotWidth&&this.maxDigitWidth===e.maxDigitWidth}}class die extends F{constructor(){super(...arguments),this._cache=new Map,this._evictUntrustedReadingsTimeout=-1,this._onDidChange=this._register(new P),this.onDidChange=this._onDidChange.event}dispose(){this._evictUntrustedReadingsTimeout!==-1&&(clearTimeout(this._evictUntrustedReadingsTimeout),this._evictUntrustedReadingsTimeout=-1),super.dispose()}clearAllFontInfos(){this._cache.clear(),this._onDidChange.fire()}_ensureCache(e){const t=Uy(e);let i=this._cache.get(t);return i||(i=new hie,this._cache.set(t,i)),i}_writeToCache(e,t,i){this._ensureCache(e).put(t,i),!i.isTrusted&&this._evictUntrustedReadingsTimeout===-1&&(this._evictUntrustedReadingsTimeout=e.setTimeout(()=>{this._evictUntrustedReadingsTimeout=-1,this._evictUntrustedReadings(e)},5e3))}_evictUntrustedReadings(e){const t=this._ensureCache(e),i=t.getValues();let n=!1;for(const s of i)s.isTrusted||(n=!0,t.remove(s));n&&this._onDidChange.fire()}readFontInfo(e,t){const i=this._ensureCache(e);if(!i.has(t)){let n=this._actualReadFontInfo(e,t);(n.typicalHalfwidthCharacterWidth<=2||n.typicalFullwidthCharacterWidth<=2||n.spaceWidth<=2||n.maxDigitWidth<=2)&&(n=new TM({pixelRatio:aw.getInstance(e).value,fontFamily:n.fontFamily,fontWeight:n.fontWeight,fontSize:n.fontSize,fontFeatureSettings:n.fontFeatureSettings,fontVariationSettings:n.fontVariationSettings,lineHeight:n.lineHeight,letterSpacing:n.letterSpacing,isMonospace:n.isMonospace,typicalHalfwidthCharacterWidth:Math.max(n.typicalHalfwidthCharacterWidth,5),typicalFullwidthCharacterWidth:Math.max(n.typicalFullwidthCharacterWidth,5),canUseHalfwidthRightwardsArrow:n.canUseHalfwidthRightwardsArrow,spaceWidth:Math.max(n.spaceWidth,5),middotWidth:Math.max(n.middotWidth,5),wsmiddotWidth:Math.max(n.wsmiddotWidth,5),maxDigitWidth:Math.max(n.maxDigitWidth,5)},!1)),this._writeToCache(e,t,n)}return i.get(t)}_createRequest(e,t,i,n){const s=new rie(e,t);return i.push(s),n==null||n.push(s),s}_actualReadFontInfo(e,t){const i=[],n=[],s=this._createRequest("n",0,i,n),r=this._createRequest("ｍ",0,i,null),a=this._createRequest(" ",0,i,n),l=this._createRequest("0",0,i,n),c=this._createRequest("1",0,i,n),d=this._createRequest("2",0,i,n),h=this._createRequest("3",0,i,n),u=this._createRequest("4",0,i,n),g=this._createRequest("5",0,i,n),f=this._createRequest("6",0,i,n),p=this._createRequest("7",0,i,n),_=this._createRequest("8",0,i,n),b=this._createRequest("9",0,i,n),C=this._createRequest("→",0,i,n),v=this._createRequest("￫",0,i,null),w=this._createRequest("·",0,i,n),S=this._createRequest("⸱",0,i,null),L="|/-_ilm%";for(let W=0,V=L.length;W<V;W++)this._createRequest(L.charAt(W),0,i,n),this._createRequest(L.charAt(W),1,i,n),this._createRequest(L.charAt(W),2,i,n);aie(e,t,i);const k=Math.max(l.width,c.width,d.width,h.width,u.width,g.width,f.width,p.width,_.width,b.width);let D=t.fontFeatureSettings===Mh.OFF;const N=n[0].width;for(let W=1,V=n.length;D&&W<V;W++){const $=N-n[W].width;if($<-.001||$>.001){D=!1;break}}let B=!0;return D&&v.width!==N&&(B=!1),v.width>C.width&&(B=!1),new TM({pixelRatio:aw.getInstance(e).value,fontFamily:t.fontFamily,fontWeight:t.fontWeight,fontSize:t.fontSize,fontFeatureSettings:t.fontFeatureSettings,fontVariationSettings:t.fontVariationSettings,lineHeight:t.lineHeight,letterSpacing:t.letterSpacing,isMonospace:D,typicalHalfwidthCharacterWidth:s.width,typicalFullwidthCharacterWidth:r.width,canUseHalfwidthRightwardsArrow:B,spaceWidth:a.width,middotWidth:w.width,wsmiddotWidth:S.width,maxDigitWidth:k},!0)}}class hie{constructor(){this._keys=Object.create(null),this._values=Object.create(null)}has(e){const t=e.getId();return!!this._values[t]}get(e){const t=e.getId();return this._values[t]}put(e,t){const i=e.getId();this._keys[i]=e,this._values[i]=t}remove(e){const t=e.getId();delete this._keys[t],delete this._values[t]}getValues(){return Object.keys(this._keys).map(e=>this._values[e])}}const RM=new die;var xa;(function(o){o.serviceIds=new Map,o.DI_TARGET="$di$target",o.DI_DEPENDENCIES="$di$dependencies";function e(t){return t[o.DI_DEPENDENCIES]||[]}o.getServiceDependencies=e})(xa||(xa={}));const we=et("instantiationService");function uie(o,e,t){e[xa.DI_TARGET]===e?e[xa.DI_DEPENDENCIES].push({id:o,index:t}):(e[xa.DI_DEPENDENCIES]=[{id:o,index:t}],e[xa.DI_TARGET]=e)}function et(o){if(xa.serviceIds.has(o))return xa.serviceIds.get(o);const e=function(t,i,n){if(arguments.length!==3)throw new Error("@IServiceName-decorator can only be used to decorate a parameter");uie(e,t,n)};return e.toString=()=>o,xa.serviceIds.set(o,e),e}const pt=et("codeEditorService"),Kt=et("modelService"),Hs=et("textModelService");class ro extends F{constructor(e,t="",i="",n=!0,s){super(),this._onDidChange=this._register(new P),this.onDidChange=this._onDidChange.event,this._enabled=!0,this._id=e,this._label=t,this._cssClass=i,this._enabled=n,this._actionCallback=s}get id(){return this._id}get label(){return this._label}set label(e){this._setLabel(e)}_setLabel(e){this._label!==e&&(this._label=e,this._onDidChange.fire({label:e}))}get tooltip(){return this._tooltip||""}set tooltip(e){this._setTooltip(e)}_setTooltip(e){this._tooltip!==e&&(this._tooltip=e,this._onDidChange.fire({tooltip:e}))}get class(){return this._cssClass}set class(e){this._setClass(e)}_setClass(e){this._cssClass!==e&&(this._cssClass=e,this._onDidChange.fire({class:e}))}get enabled(){return this._enabled}set enabled(e){this._setEnabled(e)}_setEnabled(e){this._enabled!==e&&(this._enabled=e,this._onDidChange.fire({enabled:e}))}get checked(){return this._checked}set checked(e){this._setChecked(e)}_setChecked(e){this._checked!==e&&(this._checked=e,this._onDidChange.fire({checked:e}))}async run(e,t){this._actionCallback&&await this._actionCallback(e)}}class Ef extends F{constructor(){super(...arguments),this._onWillRun=this._register(new P),this.onWillRun=this._onWillRun.event,this._onDidRun=this._register(new P),this.onDidRun=this._onDidRun.event}async run(e,t){if(!e.enabled)return;this._onWillRun.fire({action:e});let i;try{await this.runAction(e,t)}catch(n){i=n}this._onDidRun.fire({action:e,error:i})}async runAction(e,t){await e.run(t)}}const Tv=class Tv{constructor(){this.id=Tv.ID,this.label="",this.tooltip="",this.class="separator",this.enabled=!1,this.checked=!1}static join(...e){let t=[];for(const i of e)i.length&&(t.length?t=[...t,new Tv,...i]:t=i);return t}async run(){}};Tv.ID="vs.actions.separator";let Qi=Tv;class D_{get actions(){return this._actions}constructor(e,t,i,n){this.tooltip="",this.enabled=!0,this.checked=void 0,this.id=e,this.label=t,this.class=n,this._actions=i}async run(){}}const Tk=class Tk extends ro{constructor(){super(Tk.ID,m("submenu.empty","(empty)"),void 0,!1)}};Tk.ID="vs.actions.empty";let MM=Tk;function pg(o){return{id:o.id,label:o.label,tooltip:o.tooltip??o.label,class:o.class,enabled:o.enabled??!0,checked:o.checked,run:async(...e)=>o.run(...e)}}var AM;(function(o){function e(t){return t&&typeof t=="object"&&typeof t.id=="string"}o.isThemeColor=e})(AM||(AM={}));var Ie;(function(o){o.iconNameSegment="[A-Za-z0-9]+",o.iconNameExpression="[A-Za-z0-9-]+",o.iconModifierExpression="~[A-Za-z]+",o.iconNameCharacter="[A-Za-z0-9~-]";const e=new RegExp(`^(${o.iconNameExpression})(${o.iconModifierExpression})?$`);function t(u){const g=e.exec(u.id);if(!g)return t(te.error);const[,f,p]=g,_=["codicon","codicon-"+f];return p&&_.push("codicon-modifier-"+p.substring(1)),_}o.asClassNameArray=t;function i(u){return t(u).join(" ")}o.asClassName=i;function n(u){return"."+t(u).join(".")}o.asCSSSelector=n;function s(u){return u&&typeof u=="object"&&typeof u.id=="string"&&(typeof u.color>"u"||AM.isThemeColor(u.color))}o.isThemeIcon=s;const r=new RegExp(`^\\$\\((${o.iconNameExpression}(?:${o.iconModifierExpression})?)\\)$`);function a(u){const g=r.exec(u);if(!g)return;const[,f]=g;return{id:f}}o.fromString=a;function l(u){return{id:u}}o.fromId=l;function c(u,g){let f=u.id;const p=f.lastIndexOf("~");return p!==-1&&(f=f.substring(0,p)),g&&(f=`${f}~${g}`),{id:f}}o.modify=c;function d(u){const g=u.id.lastIndexOf("~");if(g!==-1)return u.id.substring(g+1)}o.getModifier=d;function h(u,g){var f,p;return u.id===g.id&&((f=u.color)==null?void 0:f.id)===((p=g.color)==null?void 0:p.id)}o.isEqual=h})(Ie||(Ie={}));const Ht=et("commandService"),rt=new class{constructor(){this._commands=new Map,this._onDidRegisterCommand=new P,this.onDidRegisterCommand=this._onDidRegisterCommand.event}registerCommand(o,e){if(!o)throw new Error("invalid command");if(typeof o=="string"){if(!e)throw new Error("invalid command");return this.registerCommand({id:o,handler:e})}if(o.metadata&&Array.isArray(o.metadata.args)){const r=[];for(const l of o.metadata.args)r.push(l.constraint);const a=o.handler;o.handler=function(l,...c){return xQ(c,r),a(l,...c)}}const{id:t}=o;let i=this._commands.get(t);i||(i=new cs,this._commands.set(t,i));const n=i.unshift(o),s=ve(()=>{n();const r=this._commands.get(t);r!=null&&r.isEmpty()&&this._commands.delete(t)});return this._onDidRegisterCommand.fire(t),s}registerCommandAlias(o,e){return rt.registerCommand(o,(t,...i)=>t.get(Ht).executeCommand(e,...i))}getCommand(o){const e=this._commands.get(o);if(!(!e||e.isEmpty()))return ot.first(e)}getCommands(){const o=new Map;for(const e of this._commands.keys()){const t=this.getCommand(e);t&&o.set(e,t)}return o}};rt.registerCommand("noop",()=>{});function wN(...o){switch(o.length){case 1:return m("contextkey.scanner.hint.didYouMean1","Did you mean {0}?",o[0]);case 2:return m("contextkey.scanner.hint.didYouMean2","Did you mean {0} or {1}?",o[0],o[1]);case 3:return m("contextkey.scanner.hint.didYouMean3","Did you mean {0}, {1} or {2}?",o[0],o[1],o[2]);default:return}}const gie=m("contextkey.scanner.hint.didYouForgetToOpenOrCloseQuote","Did you forget to open or close the quote?"),fie=m("contextkey.scanner.hint.didYouForgetToEscapeSlash","Did you forget to escape the '/' (slash) character? Put two backslashes before it to escape, e.g., '\\\\/'.");var Hc;let Yb=(Hc=class{constructor(){this._input="",this._start=0,this._current=0,this._tokens=[],this._errors=[],this.stringRe=/[a-zA-Z0-9_<>\-\./\\:\*\?\+\[\]\^,#@;"%\$\p{L}-]+/uy}static getLexeme(e){switch(e.type){case 0:return"(";case 1:return")";case 2:return"!";case 3:return e.isTripleEq?"===":"==";case 4:return e.isTripleEq?"!==":"!=";case 5:return"<";case 6:return"<=";case 7:return">=";case 8:return">=";case 9:return"=~";case 10:return e.lexeme;case 11:return"true";case 12:return"false";case 13:return"in";case 14:return"not";case 15:return"&&";case 16:return"||";case 17:return e.lexeme;case 18:return e.lexeme;case 19:return e.lexeme;case 20:return"EOF";default:throw U4(`unhandled token type: ${JSON.stringify(e)}; have you forgotten to add a case?`)}}reset(e){return this._input=e,this._start=0,this._current=0,this._tokens=[],this._errors=[],this}scan(){for(;!this._isAtEnd();)switch(this._start=this._current,this._advance()){case 40:this._addToken(0);break;case 41:this._addToken(1);break;case 33:if(this._match(61)){const t=this._match(61);this._tokens.push({type:4,offset:this._start,isTripleEq:t})}else this._addToken(2);break;case 39:this._quotedString();break;case 47:this._regex();break;case 61:if(this._match(61)){const t=this._match(61);this._tokens.push({type:3,offset:this._start,isTripleEq:t})}else this._match(126)?this._addToken(9):this._error(wN("==","=~"));break;case 60:this._addToken(this._match(61)?6:5);break;case 62:this._addToken(this._match(61)?8:7);break;case 38:this._match(38)?this._addToken(15):this._error(wN("&&"));break;case 124:this._match(124)?this._addToken(16):this._error(wN("||"));break;case 32:case 13:case 9:case 10:case 160:break;default:this._string()}return this._start=this._current,this._addToken(20),Array.from(this._tokens)}_match(e){return this._isAtEnd()||this._input.charCodeAt(this._current)!==e?!1:(this._current++,!0)}_advance(){return this._input.charCodeAt(this._current++)}_peek(){return this._isAtEnd()?0:this._input.charCodeAt(this._current)}_addToken(e){this._tokens.push({type:e,offset:this._start})}_error(e){const t=this._start,i=this._input.substring(this._start,this._current),n={type:19,offset:this._start,lexeme:i};this._errors.push({offset:t,lexeme:i,additionalInfo:e}),this._tokens.push(n)}_string(){this.stringRe.lastIndex=this._start;const e=this.stringRe.exec(this._input);if(e){this._current=this._start+e[0].length;const t=this._input.substring(this._start,this._current),i=Hc._keywords.get(t);i?this._addToken(i):this._tokens.push({type:17,lexeme:t,offset:this._start})}}_quotedString(){for(;this._peek()!==39&&!this._isAtEnd();)this._advance();if(this._isAtEnd()){this._error(gie);return}this._advance(),this._tokens.push({type:18,lexeme:this._input.substring(this._start+1,this._current-1),offset:this._start+1})}_regex(){let e=this._current,t=!1,i=!1;for(;;){if(e>=this._input.length){this._current=e,this._error(fie);return}const s=this._input.charCodeAt(e);if(t)t=!1;else if(s===47&&!i){e++;break}else s===91?i=!0:s===92?t=!0:s===93&&(i=!1);e++}for(;e<this._input.length&&Hc._regexFlags.has(this._input.charCodeAt(e));)e++;this._current=e;const n=this._input.substring(this._start,this._current);this._tokens.push({type:10,lexeme:n,offset:this._start})}_isAtEnd(){return this._current>=this._input.length}},Hc._regexFlags=new Set(["i","g","s","m","y","u"].map(e=>e.charCodeAt(0))),Hc._keywords=new Map([["not",14],["in",13],["false",12],["true",11]]),Hc);const On=new Map;On.set("false",!1);On.set("true",!0);On.set("isMac",Je);On.set("isLinux",ps);On.set("isWindows",En);On.set("isWeb",Qf);On.set("isMacNative",Je&&!Qf);On.set("isEdge",HQ);On.set("isFirefox",WQ);On.set("isChrome",Rz);On.set("isSafari",BQ);const mie=Object.prototype.hasOwnProperty,pie={regexParsingWithErrorRecovery:!0},_ie=m("contextkey.parser.error.emptyString","Empty context key expression"),bie=m("contextkey.parser.error.emptyString.hint","Did you forget to write an expression? You can also put 'false' or 'true' to always evaluate to false or true, respectively."),Cie=m("contextkey.parser.error.noInAfterNot","'in' after 'not'."),u7=m("contextkey.parser.error.closingParenthesis","closing parenthesis ')'"),vie=m("contextkey.parser.error.unexpectedToken","Unexpected token"),wie=m("contextkey.parser.error.unexpectedToken.hint","Did you forget to put && or || before the token?"),Sie=m("contextkey.parser.error.unexpectedEOF","Unexpected end of expression"),yie=m("contextkey.parser.error.unexpectedEOF.hint","Did you forget to put a context key?");var kl;let Lie=(kl=class{constructor(e=pie){this._config=e,this._scanner=new Yb,this._tokens=[],this._current=0,this._parsingErrors=[],this._flagsGYRe=/g|y/g}parse(e){if(e===""){this._parsingErrors.push({message:_ie,offset:0,lexeme:"",additionalInfo:bie});return}this._tokens=this._scanner.reset(e).scan(),this._current=0,this._parsingErrors=[];try{const t=this._expr();if(!this._isAtEnd()){const i=this._peek(),n=i.type===17?wie:void 0;throw this._parsingErrors.push({message:vie,offset:i.offset,lexeme:Yb.getLexeme(i),additionalInfo:n}),kl._parseError}return t}catch(t){if(t!==kl._parseError)throw t;return}}_expr(){return this._or()}_or(){const e=[this._and()];for(;this._matchOne(16);){const t=this._and();e.push(t)}return e.length===1?e[0]:G.or(...e)}_and(){const e=[this._term()];for(;this._matchOne(15);){const t=this._term();e.push(t)}return e.length===1?e[0]:G.and(...e)}_term(){if(this._matchOne(2)){const e=this._peek();switch(e.type){case 11:return this._advance(),_s.INSTANCE;case 12:return this._advance(),Os.INSTANCE;case 0:{this._advance();const t=this._expr();return this._consume(1,u7),t==null?void 0:t.negate()}case 17:return this._advance(),em.create(e.lexeme);default:throw this._errExpectedButGot("KEY | true | false | '(' expression ')'",e)}}return this._primary()}_primary(){const e=this._peek();switch(e.type){case 11:return this._advance(),G.true();case 12:return this._advance(),G.false();case 0:{this._advance();const t=this._expr();return this._consume(1,u7),t}case 17:{const t=e.lexeme;if(this._advance(),this._matchOne(9)){const n=this._peek();if(!this._config.regexParsingWithErrorRecovery){if(this._advance(),n.type!==10)throw this._errExpectedButGot("REGEX",n);const s=n.lexeme,r=s.lastIndexOf("/"),a=r===s.length-1?void 0:this._removeFlagsGY(s.substring(r+1));let l;try{l=new RegExp(s.substring(1,r),a)}catch{throw this._errExpectedButGot("REGEX",n)}return lw.create(t,l)}switch(n.type){case 10:case 19:{const s=[n.lexeme];this._advance();let r=this._peek(),a=0;for(let u=0;u<n.lexeme.length;u++)n.lexeme.charCodeAt(u)===40?a++:n.lexeme.charCodeAt(u)===41&&a--;for(;!this._isAtEnd()&&r.type!==15&&r.type!==16;){switch(r.type){case 0:a++;break;case 1:a--;break;case 10:case 18:for(let u=0;u<r.lexeme.length;u++)r.lexeme.charCodeAt(u)===40?a++:n.lexeme.charCodeAt(u)===41&&a--}if(a<0)break;s.push(Yb.getLexeme(r)),this._advance(),r=this._peek()}const l=s.join(""),c=l.lastIndexOf("/"),d=c===l.length-1?void 0:this._removeFlagsGY(l.substring(c+1));let h;try{h=new RegExp(l.substring(1,c),d)}catch{throw this._errExpectedButGot("REGEX",n)}return G.regex(t,h)}case 18:{const s=n.lexeme;this._advance();let r=null;if(!Jz(s)){const a=s.indexOf("/"),l=s.lastIndexOf("/");if(a!==l&&a>=0){const c=s.slice(a+1,l),d=s[l+1]==="i"?"i":"";try{r=new RegExp(c,d)}catch{throw this._errExpectedButGot("REGEX",n)}}}if(r===null)throw this._errExpectedButGot("REGEX",n);return lw.create(t,r)}default:throw this._errExpectedButGot("REGEX",this._peek())}}if(this._matchOne(14)){this._consume(13,Cie);const n=this._value();return G.notIn(t,n)}switch(this._peek().type){case 3:{this._advance();const n=this._value();if(this._previous().type===18)return G.equals(t,n);switch(n){case"true":return G.has(t);case"false":return G.not(t);default:return G.equals(t,n)}}case 4:{this._advance();const n=this._value();if(this._previous().type===18)return G.notEquals(t,n);switch(n){case"true":return G.not(t);case"false":return G.has(t);default:return G.notEquals(t,n)}}case 5:return this._advance(),II.create(t,this._value());case 6:return this._advance(),EI.create(t,this._value());case 7:return this._advance(),kI.create(t,this._value());case 8:return this._advance(),DI.create(t,this._value());case 13:return this._advance(),G.in(t,this._value());default:return G.has(t)}}case 20:throw this._parsingErrors.push({message:Sie,offset:e.offset,lexeme:"",additionalInfo:yie}),kl._parseError;default:throw this._errExpectedButGot(`true | false | KEY 
	| KEY '=~' REGEX 
	| KEY ('==' | '!=' | '<' | '<=' | '>' | '>=' | 'in' | 'not' 'in') value`,this._peek())}}_value(){const e=this._peek();switch(e.type){case 17:case 18:return this._advance(),e.lexeme;case 11:return this._advance(),"true";case 12:return this._advance(),"false";case 13:return this._advance(),"in";default:return""}}_removeFlagsGY(e){return e.replaceAll(this._flagsGYRe,"")}_previous(){return this._tokens[this._current-1]}_matchOne(e){return this._check(e)?(this._advance(),!0):!1}_advance(){return this._isAtEnd()||this._current++,this._previous()}_consume(e,t){if(this._check(e))return this._advance();throw this._errExpectedButGot(t,this._peek())}_errExpectedButGot(e,t,i){const n=m("contextkey.parser.error.expectedButGot",`Expected: {0}
Received: '{1}'.`,e,Yb.getLexeme(t)),s=t.offset,r=Yb.getLexeme(t);return this._parsingErrors.push({message:n,offset:s,lexeme:r,additionalInfo:i}),kl._parseError}_check(e){return this._peek().type===e}_peek(){return this._tokens[this._current]}_isAtEnd(){return this._peek().type===20}},kl._parseError=new Error,kl);const wB=class wB{static false(){return _s.INSTANCE}static true(){return Os.INSTANCE}static has(e){return Jf.create(e)}static equals(e,t){return mb.create(e,t)}static notEquals(e,t){return LI.create(e,t)}static regex(e,t){return lw.create(e,t)}static in(e,t){return SI.create(e,t)}static notIn(e,t){return yI.create(e,t)}static not(e){return em.create(e)}static and(...e){return rg.create(e,null,!0)}static or(...e){return kc.create(e,null,!0)}static deserialize(e){return e==null?void 0:this._parser.parse(e)}};wB._parser=new Lie({regexParsingWithErrorRecovery:!1});let G=wB;function xie(o,e){const t=o?o.substituteConstants():void 0,i=e?e.substituteConstants():void 0;return!t&&!i?!0:!t||!i?!1:t.equals(i)}function qC(o,e){return o.cmp(e)}const Rk=class Rk{constructor(){this.type=0}cmp(e){return this.type-e.type}equals(e){return e.type===this.type}substituteConstants(){return this}evaluate(e){return!1}serialize(){return"false"}keys(){return[]}negate(){return Os.INSTANCE}};Rk.INSTANCE=new Rk;let _s=Rk;const Mk=class Mk{constructor(){this.type=1}cmp(e){return this.type-e.type}equals(e){return e.type===this.type}substituteConstants(){return this}evaluate(e){return!0}serialize(){return"true"}keys(){return[]}negate(){return _s.INSTANCE}};Mk.INSTANCE=new Mk;let Os=Mk;class Jf{static create(e,t=null){const i=On.get(e);return typeof i=="boolean"?i?Os.INSTANCE:_s.INSTANCE:new Jf(e,t)}constructor(e,t){this.key=e,this.negated=t,this.type=2}cmp(e){return e.type!==this.type?this.type-e.type:OU(this.key,e.key)}equals(e){return e.type===this.type?this.key===e.key:!1}substituteConstants(){const e=On.get(this.key);return typeof e=="boolean"?e?Os.INSTANCE:_s.INSTANCE:this}evaluate(e){return!!e.getValue(this.key)}serialize(){return this.key}keys(){return[this.key]}negate(){return this.negated||(this.negated=em.create(this.key,this)),this.negated}}class mb{static create(e,t,i=null){if(typeof t=="boolean")return t?Jf.create(e,i):em.create(e,i);const n=On.get(e);return typeof n=="boolean"?t===(n?"true":"false")?Os.INSTANCE:_s.INSTANCE:new mb(e,t,i)}constructor(e,t,i){this.key=e,this.value=t,this.negated=i,this.type=4}cmp(e){return e.type!==this.type?this.type-e.type:tm(this.key,this.value,e.key,e.value)}equals(e){return e.type===this.type?this.key===e.key&&this.value===e.value:!1}substituteConstants(){const e=On.get(this.key);if(typeof e=="boolean"){const t=e?"true":"false";return this.value===t?Os.INSTANCE:_s.INSTANCE}return this}evaluate(e){return e.getValue(this.key)==this.value}serialize(){return`${this.key} == '${this.value}'`}keys(){return[this.key]}negate(){return this.negated||(this.negated=LI.create(this.key,this.value,this)),this.negated}}class SI{static create(e,t){return new SI(e,t)}constructor(e,t){this.key=e,this.valueKey=t,this.type=10,this.negated=null}cmp(e){return e.type!==this.type?this.type-e.type:tm(this.key,this.valueKey,e.key,e.valueKey)}equals(e){return e.type===this.type?this.key===e.key&&this.valueKey===e.valueKey:!1}substituteConstants(){return this}evaluate(e){const t=e.getValue(this.valueKey),i=e.getValue(this.key);return Array.isArray(t)?t.includes(i):typeof i=="string"&&typeof t=="object"&&t!==null?mie.call(t,i):!1}serialize(){return`${this.key} in '${this.valueKey}'`}keys(){return[this.key,this.valueKey]}negate(){return this.negated||(this.negated=yI.create(this.key,this.valueKey)),this.negated}}class yI{static create(e,t){return new yI(e,t)}constructor(e,t){this.key=e,this.valueKey=t,this.type=11,this._negated=SI.create(e,t)}cmp(e){return e.type!==this.type?this.type-e.type:this._negated.cmp(e._negated)}equals(e){return e.type===this.type?this._negated.equals(e._negated):!1}substituteConstants(){return this}evaluate(e){return!this._negated.evaluate(e)}serialize(){return`${this.key} not in '${this.valueKey}'`}keys(){return this._negated.keys()}negate(){return this._negated}}class LI{static create(e,t,i=null){if(typeof t=="boolean")return t?em.create(e,i):Jf.create(e,i);const n=On.get(e);return typeof n=="boolean"?t===(n?"true":"false")?_s.INSTANCE:Os.INSTANCE:new LI(e,t,i)}constructor(e,t,i){this.key=e,this.value=t,this.negated=i,this.type=5}cmp(e){return e.type!==this.type?this.type-e.type:tm(this.key,this.value,e.key,e.value)}equals(e){return e.type===this.type?this.key===e.key&&this.value===e.value:!1}substituteConstants(){const e=On.get(this.key);if(typeof e=="boolean"){const t=e?"true":"false";return this.value===t?_s.INSTANCE:Os.INSTANCE}return this}evaluate(e){return e.getValue(this.key)!=this.value}serialize(){return`${this.key} != '${this.value}'`}keys(){return[this.key]}negate(){return this.negated||(this.negated=mb.create(this.key,this.value,this)),this.negated}}class em{static create(e,t=null){const i=On.get(e);return typeof i=="boolean"?i?_s.INSTANCE:Os.INSTANCE:new em(e,t)}constructor(e,t){this.key=e,this.negated=t,this.type=3}cmp(e){return e.type!==this.type?this.type-e.type:OU(this.key,e.key)}equals(e){return e.type===this.type?this.key===e.key:!1}substituteConstants(){const e=On.get(this.key);return typeof e=="boolean"?e?_s.INSTANCE:Os.INSTANCE:this}evaluate(e){return!e.getValue(this.key)}serialize(){return`!${this.key}`}keys(){return[this.key]}negate(){return this.negated||(this.negated=Jf.create(this.key,this)),this.negated}}function xI(o,e){if(typeof o=="string"){const t=parseFloat(o);isNaN(t)||(o=t)}return typeof o=="string"||typeof o=="number"?e(o):_s.INSTANCE}class kI{static create(e,t,i=null){return xI(t,n=>new kI(e,n,i))}constructor(e,t,i){this.key=e,this.value=t,this.negated=i,this.type=12}cmp(e){return e.type!==this.type?this.type-e.type:tm(this.key,this.value,e.key,e.value)}equals(e){return e.type===this.type?this.key===e.key&&this.value===e.value:!1}substituteConstants(){return this}evaluate(e){return typeof this.value=="string"?!1:parseFloat(e.getValue(this.key))>this.value}serialize(){return`${this.key} > ${this.value}`}keys(){return[this.key]}negate(){return this.negated||(this.negated=EI.create(this.key,this.value,this)),this.negated}}class DI{static create(e,t,i=null){return xI(t,n=>new DI(e,n,i))}constructor(e,t,i){this.key=e,this.value=t,this.negated=i,this.type=13}cmp(e){return e.type!==this.type?this.type-e.type:tm(this.key,this.value,e.key,e.value)}equals(e){return e.type===this.type?this.key===e.key&&this.value===e.value:!1}substituteConstants(){return this}evaluate(e){return typeof this.value=="string"?!1:parseFloat(e.getValue(this.key))>=this.value}serialize(){return`${this.key} >= ${this.value}`}keys(){return[this.key]}negate(){return this.negated||(this.negated=II.create(this.key,this.value,this)),this.negated}}class II{static create(e,t,i=null){return xI(t,n=>new II(e,n,i))}constructor(e,t,i){this.key=e,this.value=t,this.negated=i,this.type=14}cmp(e){return e.type!==this.type?this.type-e.type:tm(this.key,this.value,e.key,e.value)}equals(e){return e.type===this.type?this.key===e.key&&this.value===e.value:!1}substituteConstants(){return this}evaluate(e){return typeof this.value=="string"?!1:parseFloat(e.getValue(this.key))<this.value}serialize(){return`${this.key} < ${this.value}`}keys(){return[this.key]}negate(){return this.negated||(this.negated=DI.create(this.key,this.value,this)),this.negated}}class EI{static create(e,t,i=null){return xI(t,n=>new EI(e,n,i))}constructor(e,t,i){this.key=e,this.value=t,this.negated=i,this.type=15}cmp(e){return e.type!==this.type?this.type-e.type:tm(this.key,this.value,e.key,e.value)}equals(e){return e.type===this.type?this.key===e.key&&this.value===e.value:!1}substituteConstants(){return this}evaluate(e){return typeof this.value=="string"?!1:parseFloat(e.getValue(this.key))<=this.value}serialize(){return`${this.key} <= ${this.value}`}keys(){return[this.key]}negate(){return this.negated||(this.negated=kI.create(this.key,this.value,this)),this.negated}}class lw{static create(e,t){return new lw(e,t)}constructor(e,t){this.key=e,this.regexp=t,this.type=7,this.negated=null}cmp(e){if(e.type!==this.type)return this.type-e.type;if(this.key<e.key)return-1;if(this.key>e.key)return 1;const t=this.regexp?this.regexp.source:"",i=e.regexp?e.regexp.source:"";return t<i?-1:t>i?1:0}equals(e){if(e.type===this.type){const t=this.regexp?this.regexp.source:"",i=e.regexp?e.regexp.source:"";return this.key===e.key&&t===i}return!1}substituteConstants(){return this}evaluate(e){const t=e.getValue(this.key);return this.regexp?this.regexp.test(t):!1}serialize(){const e=this.regexp?`/${this.regexp.source}/${this.regexp.flags}`:"/invalid/";return`${this.key} =~ ${e}`}keys(){return[this.key]}negate(){return this.negated||(this.negated=h5.create(this)),this.negated}}class h5{static create(e){return new h5(e)}constructor(e){this._actual=e,this.type=8}cmp(e){return e.type!==this.type?this.type-e.type:this._actual.cmp(e._actual)}equals(e){return e.type===this.type?this._actual.equals(e._actual):!1}substituteConstants(){return this}evaluate(e){return!this._actual.evaluate(e)}serialize(){return`!(${this._actual.serialize()})`}keys(){return this._actual.keys()}negate(){return this._actual}}function PU(o){let e=null;for(let t=0,i=o.length;t<i;t++){const n=o[t].substituteConstants();if(o[t]!==n&&e===null){e=[];for(let s=0;s<t;s++)e[s]=o[s]}e!==null&&(e[t]=n)}return e===null?o:e}class rg{static create(e,t,i){return rg._normalizeArr(e,t,i)}constructor(e,t){this.expr=e,this.negated=t,this.type=6}cmp(e){if(e.type!==this.type)return this.type-e.type;if(this.expr.length<e.expr.length)return-1;if(this.expr.length>e.expr.length)return 1;for(let t=0,i=this.expr.length;t<i;t++){const n=qC(this.expr[t],e.expr[t]);if(n!==0)return n}return 0}equals(e){if(e.type===this.type){if(this.expr.length!==e.expr.length)return!1;for(let t=0,i=this.expr.length;t<i;t++)if(!this.expr[t].equals(e.expr[t]))return!1;return!0}return!1}substituteConstants(){const e=PU(this.expr);return e===this.expr?this:rg.create(e,this.negated,!1)}evaluate(e){for(let t=0,i=this.expr.length;t<i;t++)if(!this.expr[t].evaluate(e))return!1;return!0}static _normalizeArr(e,t,i){const n=[];let s=!1;for(const r of e)if(r){if(r.type===1){s=!0;continue}if(r.type===0)return _s.INSTANCE;if(r.type===6){n.push(...r.expr);continue}n.push(r)}if(n.length===0&&s)return Os.INSTANCE;if(n.length!==0){if(n.length===1)return n[0];n.sort(qC);for(let r=1;r<n.length;r++)n[r-1].equals(n[r])&&(n.splice(r,1),r--);if(n.length===1)return n[0];for(;n.length>1;){const r=n[n.length-1];if(r.type!==9)break;n.pop();const a=n.pop(),l=n.length===0,c=kc.create(r.expr.map(d=>rg.create([d,a],null,i)),null,l);c&&(n.push(c),n.sort(qC))}if(n.length===1)return n[0];if(i){for(let r=0;r<n.length;r++)for(let a=r+1;a<n.length;a++)if(n[r].negate().equals(n[a]))return _s.INSTANCE;if(n.length===1)return n[0]}return new rg(n,t)}}serialize(){return this.expr.map(e=>e.serialize()).join(" && ")}keys(){const e=[];for(const t of this.expr)e.push(...t.keys());return e}negate(){if(!this.negated){const e=[];for(const t of this.expr)e.push(t.negate());this.negated=kc.create(e,this,!0)}return this.negated}}class kc{static create(e,t,i){return kc._normalizeArr(e,t,i)}constructor(e,t){this.expr=e,this.negated=t,this.type=9}cmp(e){if(e.type!==this.type)return this.type-e.type;if(this.expr.length<e.expr.length)return-1;if(this.expr.length>e.expr.length)return 1;for(let t=0,i=this.expr.length;t<i;t++){const n=qC(this.expr[t],e.expr[t]);if(n!==0)return n}return 0}equals(e){if(e.type===this.type){if(this.expr.length!==e.expr.length)return!1;for(let t=0,i=this.expr.length;t<i;t++)if(!this.expr[t].equals(e.expr[t]))return!1;return!0}return!1}substituteConstants(){const e=PU(this.expr);return e===this.expr?this:kc.create(e,this.negated,!1)}evaluate(e){for(let t=0,i=this.expr.length;t<i;t++)if(this.expr[t].evaluate(e))return!0;return!1}static _normalizeArr(e,t,i){let n=[],s=!1;if(e){for(let r=0,a=e.length;r<a;r++){const l=e[r];if(l){if(l.type===0){s=!0;continue}if(l.type===1)return Os.INSTANCE;if(l.type===9){n=n.concat(l.expr);continue}n.push(l)}}if(n.length===0&&s)return _s.INSTANCE;n.sort(qC)}if(n.length!==0){if(n.length===1)return n[0];for(let r=1;r<n.length;r++)n[r-1].equals(n[r])&&(n.splice(r,1),r--);if(n.length===1)return n[0];if(i){for(let r=0;r<n.length;r++)for(let a=r+1;a<n.length;a++)if(n[r].negate().equals(n[a]))return Os.INSTANCE;if(n.length===1)return n[0]}return new kc(n,t)}}serialize(){return this.expr.map(e=>e.serialize()).join(" || ")}keys(){const e=[];for(const t of this.expr)e.push(...t.keys());return e}negate(){if(!this.negated){const e=[];for(const t of this.expr)e.push(t.negate());for(;e.length>1;){const t=e.shift(),i=e.shift(),n=[];for(const s of f7(t))for(const r of f7(i))n.push(rg.create([s,r],null,!1));e.unshift(kc.create(n,null,!1))}this.negated=kc.create(e,this,!0)}return this.negated}}const kp=class kp extends Jf{static all(){return kp._info.values()}constructor(e,t,i){super(e,null),this._defaultValue=t,typeof i=="object"?kp._info.push({...i,key:e}):i!==!0&&kp._info.push({key:e,description:i,type:t!=null?typeof t:void 0})}bindTo(e){return e.createKey(this.key,this._defaultValue)}getValue(e){return e.getContextKeyValue(this.key)}toNegated(){return this.negate()}isEqualTo(e){return mb.create(this.key,e)}};kp._info=[];let le=kp;const Ee=et("contextKeyService");function OU(o,e){return o<e?-1:o>e?1:0}function tm(o,e,t,i){return o<t?-1:o>t?1:e<i?-1:e>i?1:0}function PM(o,e){if(o.type===0||e.type===1)return!0;if(o.type===9)return e.type===9?g7(o.expr,e.expr):!1;if(e.type===9){for(const t of e.expr)if(PM(o,t))return!0;return!1}if(o.type===6){if(e.type===6)return g7(e.expr,o.expr);for(const t of o.expr)if(PM(t,e))return!0;return!1}return o.equals(e)}function g7(o,e){let t=0,i=0;for(;t<o.length&&i<e.length;){const n=o[t].cmp(e[i]);if(n<0)return!1;n===0&&t++,i++}return t===o.length}function f7(o){return o.type===9?o.expr:[o]}function SN(o,e){if(!o)throw new Error(e?`Assertion failed (${e})`:"Assertion Failed")}function NI(o,e="Unreachable"){throw new Error(e)}function m7(o){o||Oe(new bt("Soft Assertion Failed"))}function Nf(o){if(!o()){debugger;o(),Oe(new bt("Assertion Failed"))}}function u5(o,e){let t=0;for(;t<o.length-1;){const i=o[t],n=o[t+1];if(!e(i,n))return!1;t++}return!0}class kie{constructor(){this.data=new Map}add(e,t){SN(As(e)),SN(on(t)),SN(!this.data.has(e),"There is already an extension with this id"),this.data.set(e,t)}as(e){return this.data.get(e)||null}}const ci=new kie;class g5{constructor(){this._coreKeybindings=new cs,this._extensionKeybindings=[],this._cachedMergedKeybindings=null}static bindToCurrentPlatform(e){if(so===1){if(e&&e.win)return e.win}else if(so===2){if(e&&e.mac)return e.mac}else if(e&&e.linux)return e.linux;return e}registerKeybindingRule(e){const t=g5.bindToCurrentPlatform(e),i=new K;if(t&&t.primary){const n=wM(t.primary,so);n&&i.add(this._registerDefaultKeybinding(n,e.id,e.args,e.weight,0,e.when))}if(t&&Array.isArray(t.secondary))for(let n=0,s=t.secondary.length;n<s;n++){const r=t.secondary[n],a=wM(r,so);a&&i.add(this._registerDefaultKeybinding(a,e.id,e.args,e.weight,-n-1,e.when))}return i}registerCommandAndKeybindingRule(e){return jr(this.registerKeybindingRule(e),rt.registerCommand(e))}_registerDefaultKeybinding(e,t,i,n,s,r){const a=this._coreKeybindings.push({keybinding:e,command:t,commandArgs:i,when:r,weight1:n,weight2:s,extensionId:null,isBuiltinExtension:!1});return this._cachedMergedKeybindings=null,ve(()=>{a(),this._cachedMergedKeybindings=null})}getDefaultKeybindings(){return this._cachedMergedKeybindings||(this._cachedMergedKeybindings=Array.from(this._coreKeybindings).concat(this._extensionKeybindings),this._cachedMergedKeybindings.sort(Iie)),this._cachedMergedKeybindings.slice(0)}}const bs=new g5,Die={EditorModes:"platform.keybindingsRegistry"};ci.add(Die.EditorModes,bs);function Iie(o,e){if(o.weight1!==e.weight1)return o.weight1-e.weight1;if(o.command&&e.command){if(o.command<e.command)return-1;if(o.command>e.command)return 1}return o.weight2-e.weight2}var Eie=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},p7=function(o,e){return function(t,i){e(t,i,o)}},z0;function Up(o){return o.command!==void 0}function Nie(o){return o.submenu!==void 0}const T=class T{constructor(e){if(T._instances.has(e))throw new TypeError(`MenuId with identifier '${e}' already exists. Use MenuId.for(ident) or a unique identifier`);T._instances.set(e,this),this.id=e}};T._instances=new Map,T.CommandPalette=new T("CommandPalette"),T.DebugBreakpointsContext=new T("DebugBreakpointsContext"),T.DebugCallStackContext=new T("DebugCallStackContext"),T.DebugConsoleContext=new T("DebugConsoleContext"),T.DebugVariablesContext=new T("DebugVariablesContext"),T.NotebookVariablesContext=new T("NotebookVariablesContext"),T.DebugHoverContext=new T("DebugHoverContext"),T.DebugWatchContext=new T("DebugWatchContext"),T.DebugToolBar=new T("DebugToolBar"),T.DebugToolBarStop=new T("DebugToolBarStop"),T.DebugCallStackToolbar=new T("DebugCallStackToolbar"),T.DebugCreateConfiguration=new T("DebugCreateConfiguration"),T.EditorContext=new T("EditorContext"),T.SimpleEditorContext=new T("SimpleEditorContext"),T.EditorContent=new T("EditorContent"),T.EditorLineNumberContext=new T("EditorLineNumberContext"),T.EditorContextCopy=new T("EditorContextCopy"),T.EditorContextPeek=new T("EditorContextPeek"),T.EditorContextShare=new T("EditorContextShare"),T.EditorTitle=new T("EditorTitle"),T.EditorTitleRun=new T("EditorTitleRun"),T.EditorTitleContext=new T("EditorTitleContext"),T.EditorTitleContextShare=new T("EditorTitleContextShare"),T.EmptyEditorGroup=new T("EmptyEditorGroup"),T.EmptyEditorGroupContext=new T("EmptyEditorGroupContext"),T.EditorTabsBarContext=new T("EditorTabsBarContext"),T.EditorTabsBarShowTabsSubmenu=new T("EditorTabsBarShowTabsSubmenu"),T.EditorTabsBarShowTabsZenModeSubmenu=new T("EditorTabsBarShowTabsZenModeSubmenu"),T.EditorActionsPositionSubmenu=new T("EditorActionsPositionSubmenu"),T.ExplorerContext=new T("ExplorerContext"),T.ExplorerContextShare=new T("ExplorerContextShare"),T.ExtensionContext=new T("ExtensionContext"),T.GlobalActivity=new T("GlobalActivity"),T.CommandCenter=new T("CommandCenter"),T.CommandCenterCenter=new T("CommandCenterCenter"),T.LayoutControlMenuSubmenu=new T("LayoutControlMenuSubmenu"),T.LayoutControlMenu=new T("LayoutControlMenu"),T.MenubarMainMenu=new T("MenubarMainMenu"),T.MenubarAppearanceMenu=new T("MenubarAppearanceMenu"),T.MenubarDebugMenu=new T("MenubarDebugMenu"),T.MenubarEditMenu=new T("MenubarEditMenu"),T.MenubarCopy=new T("MenubarCopy"),T.MenubarFileMenu=new T("MenubarFileMenu"),T.MenubarGoMenu=new T("MenubarGoMenu"),T.MenubarHelpMenu=new T("MenubarHelpMenu"),T.MenubarLayoutMenu=new T("MenubarLayoutMenu"),T.MenubarNewBreakpointMenu=new T("MenubarNewBreakpointMenu"),T.PanelAlignmentMenu=new T("PanelAlignmentMenu"),T.PanelPositionMenu=new T("PanelPositionMenu"),T.ActivityBarPositionMenu=new T("ActivityBarPositionMenu"),T.MenubarPreferencesMenu=new T("MenubarPreferencesMenu"),T.MenubarRecentMenu=new T("MenubarRecentMenu"),T.MenubarSelectionMenu=new T("MenubarSelectionMenu"),T.MenubarShare=new T("MenubarShare"),T.MenubarSwitchEditorMenu=new T("MenubarSwitchEditorMenu"),T.MenubarSwitchGroupMenu=new T("MenubarSwitchGroupMenu"),T.MenubarTerminalMenu=new T("MenubarTerminalMenu"),T.MenubarViewMenu=new T("MenubarViewMenu"),T.MenubarHomeMenu=new T("MenubarHomeMenu"),T.OpenEditorsContext=new T("OpenEditorsContext"),T.OpenEditorsContextShare=new T("OpenEditorsContextShare"),T.ProblemsPanelContext=new T("ProblemsPanelContext"),T.SCMInputBox=new T("SCMInputBox"),T.SCMChangesSeparator=new T("SCMChangesSeparator"),T.SCMChangesContext=new T("SCMChangesContext"),T.SCMIncomingChanges=new T("SCMIncomingChanges"),T.SCMIncomingChangesContext=new T("SCMIncomingChangesContext"),T.SCMIncomingChangesSetting=new T("SCMIncomingChangesSetting"),T.SCMOutgoingChanges=new T("SCMOutgoingChanges"),T.SCMOutgoingChangesContext=new T("SCMOutgoingChangesContext"),T.SCMOutgoingChangesSetting=new T("SCMOutgoingChangesSetting"),T.SCMIncomingChangesAllChangesContext=new T("SCMIncomingChangesAllChangesContext"),T.SCMIncomingChangesHistoryItemContext=new T("SCMIncomingChangesHistoryItemContext"),T.SCMOutgoingChangesAllChangesContext=new T("SCMOutgoingChangesAllChangesContext"),T.SCMOutgoingChangesHistoryItemContext=new T("SCMOutgoingChangesHistoryItemContext"),T.SCMChangeContext=new T("SCMChangeContext"),T.SCMResourceContext=new T("SCMResourceContext"),T.SCMResourceContextShare=new T("SCMResourceContextShare"),T.SCMResourceFolderContext=new T("SCMResourceFolderContext"),T.SCMResourceGroupContext=new T("SCMResourceGroupContext"),T.SCMSourceControl=new T("SCMSourceControl"),T.SCMSourceControlInline=new T("SCMSourceControlInline"),T.SCMSourceControlTitle=new T("SCMSourceControlTitle"),T.SCMHistoryTitle=new T("SCMHistoryTitle"),T.SCMTitle=new T("SCMTitle"),T.SearchContext=new T("SearchContext"),T.SearchActionMenu=new T("SearchActionContext"),T.StatusBarWindowIndicatorMenu=new T("StatusBarWindowIndicatorMenu"),T.StatusBarRemoteIndicatorMenu=new T("StatusBarRemoteIndicatorMenu"),T.StickyScrollContext=new T("StickyScrollContext"),T.TestItem=new T("TestItem"),T.TestItemGutter=new T("TestItemGutter"),T.TestProfilesContext=new T("TestProfilesContext"),T.TestMessageContext=new T("TestMessageContext"),T.TestMessageContent=new T("TestMessageContent"),T.TestPeekElement=new T("TestPeekElement"),T.TestPeekTitle=new T("TestPeekTitle"),T.TestCallStack=new T("TestCallStack"),T.TouchBarContext=new T("TouchBarContext"),T.TitleBarContext=new T("TitleBarContext"),T.TitleBarTitleContext=new T("TitleBarTitleContext"),T.TunnelContext=new T("TunnelContext"),T.TunnelPrivacy=new T("TunnelPrivacy"),T.TunnelProtocol=new T("TunnelProtocol"),T.TunnelPortInline=new T("TunnelInline"),T.TunnelTitle=new T("TunnelTitle"),T.TunnelLocalAddressInline=new T("TunnelLocalAddressInline"),T.TunnelOriginInline=new T("TunnelOriginInline"),T.ViewItemContext=new T("ViewItemContext"),T.ViewContainerTitle=new T("ViewContainerTitle"),T.ViewContainerTitleContext=new T("ViewContainerTitleContext"),T.ViewTitle=new T("ViewTitle"),T.ViewTitleContext=new T("ViewTitleContext"),T.CommentEditorActions=new T("CommentEditorActions"),T.CommentThreadTitle=new T("CommentThreadTitle"),T.CommentThreadActions=new T("CommentThreadActions"),T.CommentThreadAdditionalActions=new T("CommentThreadAdditionalActions"),T.CommentThreadTitleContext=new T("CommentThreadTitleContext"),T.CommentThreadCommentContext=new T("CommentThreadCommentContext"),T.CommentTitle=new T("CommentTitle"),T.CommentActions=new T("CommentActions"),T.CommentsViewThreadActions=new T("CommentsViewThreadActions"),T.InteractiveToolbar=new T("InteractiveToolbar"),T.InteractiveCellTitle=new T("InteractiveCellTitle"),T.InteractiveCellDelete=new T("InteractiveCellDelete"),T.InteractiveCellExecute=new T("InteractiveCellExecute"),T.InteractiveInputExecute=new T("InteractiveInputExecute"),T.InteractiveInputConfig=new T("InteractiveInputConfig"),T.ReplInputExecute=new T("ReplInputExecute"),T.IssueReporter=new T("IssueReporter"),T.NotebookToolbar=new T("NotebookToolbar"),T.NotebookStickyScrollContext=new T("NotebookStickyScrollContext"),T.NotebookCellTitle=new T("NotebookCellTitle"),T.NotebookCellDelete=new T("NotebookCellDelete"),T.NotebookCellInsert=new T("NotebookCellInsert"),T.NotebookCellBetween=new T("NotebookCellBetween"),T.NotebookCellListTop=new T("NotebookCellTop"),T.NotebookCellExecute=new T("NotebookCellExecute"),T.NotebookCellExecuteGoTo=new T("NotebookCellExecuteGoTo"),T.NotebookCellExecutePrimary=new T("NotebookCellExecutePrimary"),T.NotebookDiffCellInputTitle=new T("NotebookDiffCellInputTitle"),T.NotebookDiffCellMetadataTitle=new T("NotebookDiffCellMetadataTitle"),T.NotebookDiffCellOutputsTitle=new T("NotebookDiffCellOutputsTitle"),T.NotebookOutputToolbar=new T("NotebookOutputToolbar"),T.NotebookOutlineFilter=new T("NotebookOutlineFilter"),T.NotebookOutlineActionMenu=new T("NotebookOutlineActionMenu"),T.NotebookEditorLayoutConfigure=new T("NotebookEditorLayoutConfigure"),T.NotebookKernelSource=new T("NotebookKernelSource"),T.BulkEditTitle=new T("BulkEditTitle"),T.BulkEditContext=new T("BulkEditContext"),T.TimelineItemContext=new T("TimelineItemContext"),T.TimelineTitle=new T("TimelineTitle"),T.TimelineTitleContext=new T("TimelineTitleContext"),T.TimelineFilterSubMenu=new T("TimelineFilterSubMenu"),T.AccountsContext=new T("AccountsContext"),T.SidebarTitle=new T("SidebarTitle"),T.PanelTitle=new T("PanelTitle"),T.AuxiliaryBarTitle=new T("AuxiliaryBarTitle"),T.AuxiliaryBarHeader=new T("AuxiliaryBarHeader"),T.TerminalInstanceContext=new T("TerminalInstanceContext"),T.TerminalEditorInstanceContext=new T("TerminalEditorInstanceContext"),T.TerminalNewDropdownContext=new T("TerminalNewDropdownContext"),T.TerminalTabContext=new T("TerminalTabContext"),T.TerminalTabEmptyAreaContext=new T("TerminalTabEmptyAreaContext"),T.TerminalStickyScrollContext=new T("TerminalStickyScrollContext"),T.WebviewContext=new T("WebviewContext"),T.InlineCompletionsActions=new T("InlineCompletionsActions"),T.InlineEditsActions=new T("InlineEditsActions"),T.InlineEditActions=new T("InlineEditActions"),T.NewFile=new T("NewFile"),T.MergeInput1Toolbar=new T("MergeToolbar1Toolbar"),T.MergeInput2Toolbar=new T("MergeToolbar2Toolbar"),T.MergeBaseToolbar=new T("MergeBaseToolbar"),T.MergeInputResultToolbar=new T("MergeToolbarResultToolbar"),T.InlineSuggestionToolbar=new T("InlineSuggestionToolbar"),T.InlineEditToolbar=new T("InlineEditToolbar"),T.ChatContext=new T("ChatContext"),T.ChatCodeBlock=new T("ChatCodeblock"),T.ChatCompareBlock=new T("ChatCompareBlock"),T.ChatMessageTitle=new T("ChatMessageTitle"),T.ChatExecute=new T("ChatExecute"),T.ChatExecuteSecondary=new T("ChatExecuteSecondary"),T.ChatInputSide=new T("ChatInputSide"),T.AccessibleView=new T("AccessibleView"),T.MultiDiffEditorFileToolbar=new T("MultiDiffEditorFileToolbar"),T.DiffEditorHunkToolbar=new T("DiffEditorHunkToolbar"),T.DiffEditorSelectionToolbar=new T("DiffEditorSelectionToolbar");let me=T;const Go=et("menuService"),Rv=class Rv{static for(e){let t=this._all.get(e);return t||(t=new Rv(e),this._all.set(e,t)),t}static merge(e){const t=new Set;for(const i of e)i instanceof Rv&&t.add(i.id);return t}constructor(e){this.id=e,this.has=t=>t===e}};Rv._all=new Map;let Hu=Rv;const rn=new class{constructor(){this._commands=new Map,this._menuItems=new Map,this._onDidChangeMenu=new KJ({merge:Hu.merge}),this.onDidChangeMenu=this._onDidChangeMenu.event}addCommand(o){return this._commands.set(o.id,o),this._onDidChangeMenu.fire(Hu.for(me.CommandPalette)),ve(()=>{this._commands.delete(o.id)&&this._onDidChangeMenu.fire(Hu.for(me.CommandPalette))})}getCommand(o){return this._commands.get(o)}getCommands(){const o=new Map;return this._commands.forEach((e,t)=>o.set(t,e)),o}appendMenuItem(o,e){let t=this._menuItems.get(o);t||(t=new cs,this._menuItems.set(o,t));const i=t.push(e);return this._onDidChangeMenu.fire(Hu.for(o)),ve(()=>{i(),this._onDidChangeMenu.fire(Hu.for(o))})}appendMenuItems(o){const e=new K;for(const{id:t,item:i}of o)e.add(this.appendMenuItem(t,i));return e}getMenuItems(o){let e;return this._menuItems.has(o)?e=[...this._menuItems.get(o)]:e=[],o===me.CommandPalette&&this._appendImplicitItems(e),e}_appendImplicitItems(o){const e=new Set;for(const t of o)Up(t)&&(e.add(t.command.id),t.alt&&e.add(t.alt.id));this._commands.forEach((t,i)=>{e.has(i)||o.push({command:t})})}};class $p extends D_{constructor(e,t,i){super(`submenuitem.${e.submenu.id}`,typeof e.title=="string"?e.title:e.title.value,i,"submenu"),this.item=e,this.hideActions=t}}let ao=z0=class{static label(e,t){return t!=null&&t.renderShortTitle&&e.shortTitle?typeof e.shortTitle=="string"?e.shortTitle:e.shortTitle.value:typeof e.title=="string"?e.title:e.title.value}constructor(e,t,i,n,s,r,a){var c;this.hideActions=n,this.menuKeybinding=s,this._commandService=a,this.id=e.id,this.label=z0.label(e,i),this.tooltip=(typeof e.tooltip=="string"?e.tooltip:(c=e.tooltip)==null?void 0:c.value)??"",this.enabled=!e.precondition||r.contextMatchesRules(e.precondition),this.checked=void 0;let l;if(e.toggled){const d=e.toggled.condition?e.toggled:{condition:e.toggled};this.checked=r.contextMatchesRules(d.condition),this.checked&&d.tooltip&&(this.tooltip=typeof d.tooltip=="string"?d.tooltip:d.tooltip.value),this.checked&&Ie.isThemeIcon(d.icon)&&(l=d.icon),this.checked&&d.title&&(this.label=typeof d.title=="string"?d.title:d.title.value)}l||(l=Ie.isThemeIcon(e.icon)?e.icon:void 0),this.item=e,this.alt=t?new z0(t,void 0,i,n,void 0,r,a):void 0,this._options=i,this.class=l&&Ie.asClassName(l)}run(...e){var i,n;let t=[];return(i=this._options)!=null&&i.arg&&(t=[...t,this._options.arg]),(n=this._options)!=null&&n.shouldForwardArgs&&(t=[...t,...e]),this._commandService.executeCommand(this.id,...t)}};ao=z0=Eie([p7(5,Ee),p7(6,Ht)],ao);class ws{constructor(e){this.desc=e}}function Ot(o){const e=[],t=new o,{f1:i,menu:n,keybinding:s,...r}=t.desc;if(rt.getCommand(r.id))throw new Error(`Cannot register two commands with the same id: ${r.id}`);if(e.push(rt.registerCommand({id:r.id,handler:(a,...l)=>t.run(a,...l),metadata:r.metadata})),Array.isArray(n))for(const a of n)e.push(rn.appendMenuItem(a.id,{command:{...r,precondition:a.precondition===null?void 0:r.precondition},...a}));else n&&e.push(rn.appendMenuItem(n.id,{command:{...r,precondition:n.precondition===null?void 0:r.precondition},...n}));if(i&&(e.push(rn.appendMenuItem(me.CommandPalette,{command:r,when:r.precondition})),e.push(rn.addCommand(r))),Array.isArray(s))for(const a of s)e.push(bs.registerKeybindingRule({...a,id:r.id,when:r.precondition?G.and(r.precondition,a.when):a.when}));else s&&e.push(bs.registerKeybindingRule({...s,id:r.id,when:r.precondition?G.and(r.precondition,s.when):s.when}));return{dispose(){Tt(e)}}}const Bn=et("telemetryService"),Nn=et("logService");var Ns;(function(o){o[o.Off=0]="Off",o[o.Trace=1]="Trace",o[o.Debug=2]="Debug",o[o.Info=3]="Info",o[o.Warning=4]="Warning",o[o.Error=5]="Error"})(Ns||(Ns={}));const FU=Ns.Info;class WU extends F{constructor(){super(...arguments),this.level=FU,this._onDidChangeLogLevel=this._register(new P),this.onDidChangeLogLevel=this._onDidChangeLogLevel.event}setLevel(e){this.level!==e&&(this.level=e,this._onDidChangeLogLevel.fire(this.level))}getLevel(){return this.level}checkLogLevel(e){return this.level!==Ns.Off&&this.level<=e}}class Tie extends WU{constructor(e=FU,t=!0){super(),this.useColors=t,this.setLevel(e)}trace(e,...t){this.checkLogLevel(Ns.Trace)&&(this.useColors?console.log("%cTRACE","color: #888",e,...t):console.log(e,...t))}debug(e,...t){this.checkLogLevel(Ns.Debug)&&(this.useColors?console.log("%cDEBUG","background: #eee; color: #888",e,...t):console.log(e,...t))}info(e,...t){this.checkLogLevel(Ns.Info)&&(this.useColors?console.log("%c INFO","color: #33f",e,...t):console.log(e,...t))}warn(e,...t){this.checkLogLevel(Ns.Warning)&&(this.useColors?console.log("%c WARN","color: #993",e,...t):console.log(e,...t))}error(e,...t){this.checkLogLevel(Ns.Error)&&(this.useColors?console.log("%c  ERR","color: #f33",e,...t):console.error(e,...t))}}class Rie extends WU{constructor(e){super(),this.loggers=e,e.length&&this.setLevel(e[0].getLevel())}setLevel(e){for(const t of this.loggers)t.setLevel(e);super.setLevel(e)}trace(e,...t){for(const i of this.loggers)i.trace(e,...t)}debug(e,...t){for(const i of this.loggers)i.debug(e,...t)}info(e,...t){for(const i of this.loggers)i.info(e,...t)}warn(e,...t){for(const i of this.loggers)i.warn(e,...t)}error(e,...t){for(const i of this.loggers)i.error(e,...t)}dispose(){for(const e of this.loggers)e.dispose();super.dispose()}}function Mie(o){switch(o){case Ns.Trace:return"trace";case Ns.Debug:return"debug";case Ns.Info:return"info";case Ns.Warning:return"warn";case Ns.Error:return"error";case Ns.Off:return"off"}}new le("logLevel",Mie(Ns.Info));class TI{constructor(e){this.id=e.id,this.precondition=e.precondition,this._kbOpts=e.kbOpts,this._menuOpts=e.menuOpts,this.metadata=e.metadata}register(){if(Array.isArray(this._menuOpts)?this._menuOpts.forEach(this._registerMenuItem,this):this._menuOpts&&this._registerMenuItem(this._menuOpts),this._kbOpts){const e=Array.isArray(this._kbOpts)?this._kbOpts:[this._kbOpts];for(const t of e){let i=t.kbExpr;this.precondition&&(i?i=G.and(i,this.precondition):i=this.precondition);const n={id:this.id,weight:t.weight,args:t.args,when:i,primary:t.primary,secondary:t.secondary,win:t.win,linux:t.linux,mac:t.mac};bs.registerKeybindingRule(n)}}rt.registerCommand({id:this.id,handler:(e,t)=>this.runCommand(e,t),metadata:this.metadata})}_registerMenuItem(e){rn.appendMenuItem(e.menuId,{group:e.group,command:{id:this.id,title:e.title,icon:e.icon,precondition:this.precondition},when:e.when,order:e.order})}}class pb extends TI{constructor(){super(...arguments),this._implementations=[]}addImplementation(e,t,i,n){return this._implementations.push({priority:e,name:t,implementation:i,when:n}),this._implementations.sort((s,r)=>r.priority-s.priority),{dispose:()=>{for(let s=0;s<this._implementations.length;s++)if(this._implementations[s].implementation===i){this._implementations.splice(s,1);return}}}}runCommand(e,t){const i=e.get(Nn),n=e.get(Ee);i.trace(`Executing Command '${this.id}' which has ${this._implementations.length} bound.`);for(const s of this._implementations){if(s.when){const a=n.getContext(Dn());if(!s.when.evaluate(a))continue}const r=s.implementation(e,t);if(r)return i.trace(`Command '${this.id}' was handled by '${s.name}'.`),typeof r=="boolean"?void 0:r}i.trace(`The Command '${this.id}' was not handled by any implementation.`)}}class BU extends TI{constructor(e,t){super(t),this.command=e}runCommand(e,t){return this.command.runCommand(e,t)}}class $i extends TI{static bindToContribution(e){return class extends $i{constructor(i){super(i),this._callback=i.handler}runEditorCommand(i,n,s){const r=e(n);r&&this._callback(r,s)}}}static runEditorCommand(e,t,i,n){const s=e.get(pt),r=s.getFocusedCodeEditor()||s.getActiveCodeEditor();if(r)return r.invokeWithinContext(a=>{if(a.get(Ee).contextMatchesRules(i??void 0))return n(a,r,t)})}runCommand(e,t){return $i.runEditorCommand(e,t,this.precondition,(i,n,s)=>this.runEditorCommand(i,n,s))}}class de extends $i{static convertOptions(e){let t;Array.isArray(e.menuOpts)?t=e.menuOpts:e.menuOpts?t=[e.menuOpts]:t=[];function i(n){return n.menuId||(n.menuId=me.EditorContext),n.title||(n.title=e.label),n.when=G.and(e.precondition,n.when),n}return Array.isArray(e.contextMenuOpts)?t.push(...e.contextMenuOpts.map(i)):e.contextMenuOpts&&t.push(i(e.contextMenuOpts)),e.menuOpts=t,e}constructor(e){super(de.convertOptions(e)),this.label=e.label,this.alias=e.alias}runEditorCommand(e,t,i){return this.reportTelemetry(e,t),this.run(e,t,i||{})}reportTelemetry(e,t){e.get(Bn).publicLog2("editorActionInvoked",{name:this.label,id:this.id})}}class HU extends de{constructor(){super(...arguments),this._implementations=[]}addImplementation(e,t){return this._implementations.push([e,t]),this._implementations.sort((i,n)=>n[0]-i[0]),{dispose:()=>{for(let i=0;i<this._implementations.length;i++)if(this._implementations[i][1]===t){this._implementations.splice(i,1);return}}}}run(e,t,i){for(const n of this._implementations){const s=n[1](e,t,i);if(s)return typeof s=="boolean"?void 0:s}}}class $a extends ws{run(e,...t){const i=e.get(pt),n=i.getFocusedCodeEditor()||i.getActiveCodeEditor();if(n)return n.invokeWithinContext(s=>{var c;const r=s.get(Ee),a=s.get(Nn);if(!r.contextMatchesRules(this.desc.precondition??void 0)){a.debug("[EditorAction2] NOT running command because its precondition is FALSE",this.desc.id,(c=this.desc.precondition)==null?void 0:c.serialize());return}return this.runEditorCommand(s,n,...t)})}}function Vs(o,e){rt.registerCommand(o,function(t,...i){const n=t.get(we),[s,r]=i;mt(Se.isUri(s)),mt(A.isIPosition(r));const a=t.get(Kt).getModel(s);if(a){const l=A.lift(r);return n.invokeFunction(e,a,l,...i.slice(2))}return t.get(Hs).createModelReference(s).then(l=>new Promise((c,d)=>{try{const h=n.invokeFunction(e,l.object.textEditorModel,A.lift(r),i.slice(2));c(h)}catch(h){d(h)}}).finally(()=>{l.dispose()}))})}function re(o){return cr.INSTANCE.registerEditorCommand(o),o}function Q(o){const e=new o;return cr.INSTANCE.registerEditorAction(e),e}function VU(o){return cr.INSTANCE.registerEditorAction(o),o}function Aie(o){cr.INSTANCE.registerEditorAction(o)}function ct(o,e,t){cr.INSTANCE.registerEditorContribution(o,e,t)}var _g;(function(o){function e(r){return cr.INSTANCE.getEditorCommand(r)}o.getEditorCommand=e;function t(){return cr.INSTANCE.getEditorActions()}o.getEditorActions=t;function i(){return cr.INSTANCE.getEditorContributions()}o.getEditorContributions=i;function n(r){return cr.INSTANCE.getEditorContributions().filter(a=>r.indexOf(a.id)>=0)}o.getSomeEditorContributions=n;function s(){return cr.INSTANCE.getDiffEditorContributions()}o.getDiffEditorContributions=s})(_g||(_g={}));const Pie={EditorCommonContributions:"editor.contributions"},Ak=class Ak{constructor(){this.editorContributions=[],this.diffEditorContributions=[],this.editorActions=[],this.editorCommands=Object.create(null)}registerEditorContribution(e,t,i){this.editorContributions.push({id:e,ctor:t,instantiation:i})}getEditorContributions(){return this.editorContributions.slice(0)}getDiffEditorContributions(){return this.diffEditorContributions.slice(0)}registerEditorAction(e){e.register(),this.editorActions.push(e)}getEditorActions(){return this.editorActions}registerEditorCommand(e){e.register(),this.editorCommands[e.id]=e}getEditorCommand(e){return this.editorCommands[e]||null}};Ak.INSTANCE=new Ak;let cr=Ak;ci.add(Pie.EditorCommonContributions,cr.INSTANCE);function W1(o){return o.register(),o}const zU=W1(new pb({id:"undo",precondition:void 0,kbOpts:{weight:0,primary:2104},menuOpts:[{menuId:me.MenubarEditMenu,group:"1_do",title:m({key:"miUndo",comment:["&& denotes a mnemonic"]},"&&Undo"),order:1},{menuId:me.CommandPalette,group:"",title:m("undo","Undo"),order:1}]}));W1(new BU(zU,{id:"default:undo",precondition:void 0}));const UU=W1(new pb({id:"redo",precondition:void 0,kbOpts:{weight:0,primary:2103,secondary:[3128],mac:{primary:3128}},menuOpts:[{menuId:me.MenubarEditMenu,group:"1_do",title:m({key:"miRedo",comment:["&& denotes a mnemonic"]},"&&Redo"),order:2},{menuId:me.CommandPalette,group:"",title:m("redo","Redo"),order:1}]}));W1(new BU(UU,{id:"default:redo",precondition:void 0}));const Oie=W1(new pb({id:"editor.action.selectAll",precondition:void 0,kbOpts:{weight:0,kbExpr:null,primary:2079},menuOpts:[{menuId:me.MenubarSelectionMenu,group:"1_basic",title:m({key:"miSelectAll",comment:["&& denotes a mnemonic"]},"&&Select All"),order:1},{menuId:me.CommandPalette,group:"",title:m("selectAll","Select All"),order:1}]})),_7="default",Fie="$initialize";let b7=!1;function OM(o){Qf&&(b7||(b7=!0,console.warn("Could not create web worker(s). Falling back to loading web worker code in main thread, which might cause UI freezes. Please see https://github.com/microsoft/monaco-editor#faq")),console.warn(o.message))}class Wie{constructor(e,t,i,n,s){this.vsWorker=e,this.req=t,this.channel=i,this.method=n,this.args=s,this.type=0}}class C7{constructor(e,t,i,n){this.vsWorker=e,this.seq=t,this.res=i,this.err=n,this.type=1}}class Bie{constructor(e,t,i,n,s){this.vsWorker=e,this.req=t,this.channel=i,this.eventName=n,this.arg=s,this.type=2}}class Hie{constructor(e,t,i){this.vsWorker=e,this.req=t,this.event=i,this.type=3}}class Vie{constructor(e,t){this.vsWorker=e,this.req=t,this.type=4}}class zie{constructor(e){this._workerId=-1,this._handler=e,this._lastSentReq=0,this._pendingReplies=Object.create(null),this._pendingEmitters=new Map,this._pendingEvents=new Map}setWorkerId(e){this._workerId=e}sendMessage(e,t,i){const n=String(++this._lastSentReq);return new Promise((s,r)=>{this._pendingReplies[n]={resolve:s,reject:r},this._send(new Wie(this._workerId,n,e,t,i))})}listen(e,t,i){let n=null;const s=new P({onWillAddFirstListener:()=>{n=String(++this._lastSentReq),this._pendingEmitters.set(n,s),this._send(new Bie(this._workerId,n,e,t,i))},onDidRemoveLastListener:()=>{this._pendingEmitters.delete(n),this._send(new Vie(this._workerId,n)),n=null}});return s.event}handleMessage(e){!e||!e.vsWorker||this._workerId!==-1&&e.vsWorker!==this._workerId||this._handleMessage(e)}createProxyToRemoteChannel(e,t){const i={get:(n,s)=>(typeof s=="string"&&!n[s]&&(jU(s)?n[s]=r=>this.listen(e,s,r):$U(s)?n[s]=this.listen(e,s,void 0):s.charCodeAt(0)===36&&(n[s]=async(...r)=>(await(t==null?void 0:t()),this.sendMessage(e,s,r)))),n[s])};return new Proxy(Object.create(null),i)}_handleMessage(e){switch(e.type){case 1:return this._handleReplyMessage(e);case 0:return this._handleRequestMessage(e);case 2:return this._handleSubscribeEventMessage(e);case 3:return this._handleEventMessage(e);case 4:return this._handleUnsubscribeEventMessage(e)}}_handleReplyMessage(e){if(!this._pendingReplies[e.seq]){console.warn("Got reply to unknown seq");return}const t=this._pendingReplies[e.seq];if(delete this._pendingReplies[e.seq],e.err){let i=e.err;e.err.$isError&&(i=new Error,i.name=e.err.name,i.message=e.err.message,i.stack=e.err.stack),t.reject(i);return}t.resolve(e.res)}_handleRequestMessage(e){const t=e.req;this._handler.handleMessage(e.channel,e.method,e.args).then(n=>{this._send(new C7(this._workerId,t,n,void 0))},n=>{n.detail instanceof Error&&(n.detail=jB(n.detail)),this._send(new C7(this._workerId,t,void 0,jB(n)))})}_handleSubscribeEventMessage(e){const t=e.req,i=this._handler.handleEvent(e.channel,e.eventName,e.arg)(n=>{this._send(new Hie(this._workerId,t,n))});this._pendingEvents.set(t,i)}_handleEventMessage(e){if(!this._pendingEmitters.has(e.req)){console.warn("Got event for unknown req");return}this._pendingEmitters.get(e.req).fire(e.event)}_handleUnsubscribeEventMessage(e){if(!this._pendingEvents.has(e.req)){console.warn("Got unsubscribe for unknown req");return}this._pendingEvents.get(e.req).dispose(),this._pendingEvents.delete(e.req)}_send(e){const t=[];if(e.type===0)for(let i=0;i<e.args.length;i++)e.args[i]instanceof ArrayBuffer&&t.push(e.args[i]);else e.type===1&&e.res instanceof ArrayBuffer&&t.push(e.res);this._handler.sendMessage(e,t)}}class Uie extends F{constructor(e,t){super(),this._localChannels=new Map,this._worker=this._register(e.create({amdModuleId:"vs/base/common/worker/simpleWorker",esmModuleLocation:t.esmModuleLocation,label:t.label},s=>{this._protocol.handleMessage(s)},s=>{Oe(s)})),this._protocol=new zie({sendMessage:(s,r)=>{this._worker.postMessage(s,r)},handleMessage:(s,r,a)=>this._handleMessage(s,r,a),handleEvent:(s,r,a)=>this._handleEvent(s,r,a)}),this._protocol.setWorkerId(this._worker.getId());let i=null;const n=globalThis.require;typeof n<"u"&&typeof n.getConfig=="function"?i=n.getConfig():typeof globalThis.requirejs<"u"&&(i=globalThis.requirejs.s.contexts._.config),this._onModuleLoaded=this._protocol.sendMessage(_7,Fie,[this._worker.getId(),JSON.parse(JSON.stringify(i)),t.amdModuleId]),this.proxy=this._protocol.createProxyToRemoteChannel(_7,async()=>{await this._onModuleLoaded}),this._onModuleLoaded.catch(s=>{this._onError("Worker failed to load "+t.amdModuleId,s)})}_handleMessage(e,t,i){const n=this._localChannels.get(e);if(!n)return Promise.reject(new Error(`Missing channel ${e} on main thread`));if(typeof n[t]!="function")return Promise.reject(new Error(`Missing method ${t} on main thread channel ${e}`));try{return Promise.resolve(n[t].apply(n,i))}catch(s){return Promise.reject(s)}}_handleEvent(e,t,i){const n=this._localChannels.get(e);if(!n)throw new Error(`Missing channel ${e} on main thread`);if(jU(t)){const s=n[t].call(n,i);if(typeof s!="function")throw new Error(`Missing dynamic event ${t} on main thread channel ${e}.`);return s}if($U(t)){const s=n[t];if(typeof s!="function")throw new Error(`Missing event ${t} on main thread channel ${e}.`);return s}throw new Error(`Malformed event name ${t}`)}setChannel(e,t){this._localChannels.set(e,t)}_onError(e,t){console.error(e),console.info(t)}}function $U(o){return o[0]==="o"&&o[1]==="n"&&bl(o.charCodeAt(2))}function jU(o){return/^onDynamic/.test(o)&&bl(o.charCodeAt(9))}function hd(o,e){var i;const t=globalThis.MonacoEnvironment;if(t!=null&&t.createTrustedTypesPolicy)try{return t.createTrustedTypesPolicy(o,e)}catch(n){Oe(n);return}try{return(i=globalThis.trustedTypes)==null?void 0:i.createPolicy(o,e)}catch(n){Oe(n);return}}let tp;typeof self=="object"&&self.constructor&&self.constructor.name==="DedicatedWorkerGlobalScope"&&globalThis.workerttPolicy!==void 0?tp=globalThis.workerttPolicy:tp=hd("defaultWorkerFactory",{createScriptURL:o=>o});function $ie(o,e){const t=globalThis.MonacoEnvironment;if(t){if(typeof t.getWorker=="function")return t.getWorker("workerMain.js",e);if(typeof t.getWorkerUrl=="function"){const i=t.getWorkerUrl("workerMain.js",e);return new Worker(tp?tp.createScriptURL(i):i,{name:e,type:"module"})}}if(o){const i=jie(e,o.toString(!0)),n=new Worker(tp?tp.createScriptURL(i):i,{name:e,type:"module"});return Kie(n)}throw new Error("You must define a function MonacoEnvironment.getWorkerUrl or MonacoEnvironment.getWorker")}function jie(o,e,t){if(!(/^((http:)|(https:)|(file:)|(vscode-file:))/.test(e)&&e.substring(0,globalThis.origin.length)!==globalThis.origin)){const s=e.lastIndexOf("?"),r=e.lastIndexOf("#",s),a=s>0?new URLSearchParams(e.substring(s+1,~r?r:void 0)):new URLSearchParams;IM.addSearchParam(a,!0,!0),a.toString()?e=`${e}?${a.toString()}#${o}`:e=`${e}#${o}`}const n=new Blob([Kr([`/*${o}*/`,void 0,`globalThis._VSCODE_NLS_MESSAGES = ${JSON.stringify(xz())};`,`globalThis._VSCODE_NLS_LANGUAGE = ${JSON.stringify(W4())};`,`globalThis._VSCODE_FILE_ROOT = '${globalThis._VSCODE_FILE_ROOT}';`,"const ttPolicy = globalThis.trustedTypes?.createPolicy('defaultWorkerFactory', { createScriptURL: value => value });","globalThis.workerttPolicy = ttPolicy;",`await import(ttPolicy?.createScriptURL('${e}') ?? '${e}');`,"globalThis.postMessage({ type: 'vscode-worker-ready' });",`/*${o}*/`]).join("")],{type:"application/javascript"});return URL.createObjectURL(n)}function Kie(o){return new Promise((e,t)=>{o.onmessage=function(i){i.data.type==="vscode-worker-ready"&&(o.onmessage=null,e(o))},o.onerror=t})}function qie(o){return typeof o.then=="function"}class Gie extends F{constructor(e,t,i,n,s,r){super(),this.id=i,this.label=n;const a=$ie(e,n);qie(a)?this.worker=a:this.worker=Promise.resolve(a),this.postMessage(t,[]),this.worker.then(l=>{l.onmessage=function(c){s(c.data)},l.onmessageerror=r,typeof l.addEventListener=="function"&&l.addEventListener("error",r)}),this._register(ve(()=>{var l;(l=this.worker)==null||l.then(c=>{c.onmessage=null,c.onmessageerror=null,c.removeEventListener("error",r),c.terminate()}),this.worker=null}))}getId(){return this.id}postMessage(e,t){var i;(i=this.worker)==null||i.then(n=>{try{n.postMessage(e,t)}catch(s){Oe(s),Oe(new Error(`FAILED to post message to '${this.label}'-worker`,{cause:s}))}})}}class Zie{constructor(e,t){this.amdModuleId=e,this.label=t,this.esmModuleLocation=pI.asBrowserUri(`${e}.esm.js`)}}const Pk=class Pk{constructor(){this._webWorkerFailedBeforeError=!1}create(e,t,i){const n=++Pk.LAST_WORKER_ID;if(this._webWorkerFailedBeforeError)throw this._webWorkerFailedBeforeError;return new Gie(e.esmModuleLocation,e.amdModuleId,n,e.label||"anonymous"+n,t,s=>{OM(s),this._webWorkerFailedBeforeError=s,i(s)})}};Pk.LAST_WORKER_ID=0;let FM=Pk;function Yie(o,e){const t=typeof o=="string"?new Zie(o,e):o;return new Uie(new FM,t)}var Bi;(function(o){o[o.None=0]="None",o[o.Indent=1]="Indent",o[o.IndentOutdent=2]="IndentOutdent",o[o.Outdent=3]="Outdent"})(Bi||(Bi={}));class yN{constructor(e){if(this._neutralCharacter=null,this._neutralCharacterSearched=!1,this.open=e.open,this.close=e.close,this._inString=!0,this._inComment=!0,this._inRegEx=!0,Array.isArray(e.notIn))for(let t=0,i=e.notIn.length;t<i;t++)switch(e.notIn[t]){case"string":this._inString=!1;break;case"comment":this._inComment=!1;break;case"regex":this._inRegEx=!1;break}}isOK(e){switch(e){case 0:return!0;case 1:return this._inComment;case 2:return this._inString;case 3:return this._inRegEx}}shouldAutoClose(e,t){if(e.getTokenCount()===0)return!0;const i=e.findTokenIndexAtOffset(t-2),n=e.getStandardTokenType(i);return this.isOK(n)}_findNeutralCharacterInRange(e,t){for(let i=e;i<=t;i++){const n=String.fromCharCode(i);if(!this.open.includes(n)&&!this.close.includes(n))return n}return null}findNeutralCharacter(){return this._neutralCharacterSearched||(this._neutralCharacterSearched=!0,this._neutralCharacter||(this._neutralCharacter=this._findNeutralCharacterInRange(48,57)),this._neutralCharacter||(this._neutralCharacter=this._findNeutralCharacterInRange(97,122)),this._neutralCharacter||(this._neutralCharacter=this._findNeutralCharacterInRange(65,90))),this._neutralCharacter}}class Xie{constructor(e){this.autoClosingPairsOpenByStart=new Map,this.autoClosingPairsOpenByEnd=new Map,this.autoClosingPairsCloseByStart=new Map,this.autoClosingPairsCloseByEnd=new Map,this.autoClosingPairsCloseSingleChar=new Map;for(const t of e)Xb(this.autoClosingPairsOpenByStart,t.open.charAt(0),t),Xb(this.autoClosingPairsOpenByEnd,t.open.charAt(t.open.length-1),t),Xb(this.autoClosingPairsCloseByStart,t.close.charAt(0),t),Xb(this.autoClosingPairsCloseByEnd,t.close.charAt(t.close.length-1),t),t.close.length===1&&t.open.length===1&&Xb(this.autoClosingPairsCloseSingleChar,t.close,t)}}function Xb(o,e,t){o.has(e)?o.get(e).push(t):o.set(e,[t])}const Dp=class Dp{constructor(e){if(e.autoClosingPairs?this._autoClosingPairs=e.autoClosingPairs.map(t=>new yN(t)):e.brackets?this._autoClosingPairs=e.brackets.map(t=>new yN({open:t[0],close:t[1]})):this._autoClosingPairs=[],e.__electricCharacterSupport&&e.__electricCharacterSupport.docComment){const t=e.__electricCharacterSupport.docComment;this._autoClosingPairs.push(new yN({open:t.open,close:t.close||""}))}this._autoCloseBeforeForQuotes=typeof e.autoCloseBefore=="string"?e.autoCloseBefore:Dp.DEFAULT_AUTOCLOSE_BEFORE_LANGUAGE_DEFINED_QUOTES,this._autoCloseBeforeForBrackets=typeof e.autoCloseBefore=="string"?e.autoCloseBefore:Dp.DEFAULT_AUTOCLOSE_BEFORE_LANGUAGE_DEFINED_BRACKETS,this._surroundingPairs=e.surroundingPairs||this._autoClosingPairs}getAutoClosingPairs(){return this._autoClosingPairs}getAutoCloseBeforeSet(e){return e?this._autoCloseBeforeForQuotes:this._autoCloseBeforeForBrackets}getSurroundingPairs(){return this._surroundingPairs}};Dp.DEFAULT_AUTOCLOSE_BEFORE_LANGUAGE_DEFINED_QUOTES=`;:.,=}])> 
	`,Dp.DEFAULT_AUTOCLOSE_BEFORE_LANGUAGE_DEFINED_BRACKETS=`'"\`;:.,=}])> 
	`;let WM=Dp;function ag(o,e){const t=o.getCount(),i=o.findTokenIndexAtOffset(e),n=o.getLanguageId(i);let s=i;for(;s+1<t&&o.getLanguageId(s+1)===n;)s++;let r=i;for(;r>0&&o.getLanguageId(r-1)===n;)r--;return new Qie(o,n,r,s+1,o.getStartOffset(r),o.getEndOffset(s))}class Qie{constructor(e,t,i,n,s,r){this._scopedLineTokensBrand=void 0,this._actual=e,this.languageId=t,this._firstTokenIndex=i,this._lastTokenIndex=n,this.firstCharOffset=s,this._lastCharOffset=r,this.languageIdCodec=e.languageIdCodec}getLineContent(){return this._actual.getLineContent().substring(this.firstCharOffset,this._lastCharOffset)}getLineLength(){return this._lastCharOffset-this.firstCharOffset}getActualLineContentBefore(e){return this._actual.getLineContent().substring(0,this.firstCharOffset+e)}getTokenCount(){return this._lastTokenIndex-this._firstTokenIndex}findTokenIndexAtOffset(e){return this._actual.findTokenIndexAtOffset(e+this.firstCharOffset)-this._firstTokenIndex}getStandardTokenType(e){return this._actual.getStandardTokenType(e+this._firstTokenIndex)}toIViewLineTokens(){return this._actual.sliceAndInflate(this.firstCharOffset,this._lastCharOffset,0)}}function ol(o){return(o&3)!==0}const v7=typeof Buffer<"u";let LN;class RI{static wrap(e){return v7&&!Buffer.isBuffer(e)&&(e=Buffer.from(e.buffer,e.byteOffset,e.byteLength)),new RI(e)}constructor(e){this.buffer=e,this.byteLength=this.buffer.byteLength}toString(){return v7?this.buffer.toString():(LN||(LN=new TextDecoder),LN.decode(this.buffer))}}function Jie(o,e){return o[e+0]<<0>>>0|o[e+1]<<8>>>0}function ene(o,e,t){o[t+0]=e&255,e=e>>>8,o[t+1]=e&255}function fa(o,e){return o[e]*2**24+o[e+1]*2**16+o[e+2]*2**8+o[e+3]}function ma(o,e,t){o[t+3]=e,e=e>>>8,o[t+2]=e,e=e>>>8,o[t+1]=e,e=e>>>8,o[t]=e}function w7(o,e){return o[e]}function S7(o,e,t){o[t]=e}let xN;function KU(){return xN||(xN=new TextDecoder("UTF-16LE")),xN}let kN;function tne(){return kN||(kN=new TextDecoder("UTF-16BE")),kN}let DN;function qU(){return DN||(DN=Tz()?KU():tne()),DN}function ine(o,e,t){const i=new Uint16Array(o.buffer,e,t);return t>0&&(i[0]===65279||i[0]===65534)?nne(o,e,t):KU().decode(i)}function nne(o,e,t){const i=[];let n=0;for(let s=0;s<t;s++){const r=Jie(o,e);e+=2,i[n++]=String.fromCharCode(r)}return i.join("")}class _b{constructor(e){this._capacity=e|0,this._buffer=new Uint16Array(this._capacity),this._completedStrings=null,this._bufferLength=0}reset(){this._completedStrings=null,this._bufferLength=0}build(){return this._completedStrings!==null?(this._flushBuffer(),this._completedStrings.join("")):this._buildBuffer()}_buildBuffer(){if(this._bufferLength===0)return"";const e=new Uint16Array(this._buffer.buffer,0,this._bufferLength);return qU().decode(e)}_flushBuffer(){const e=this._buildBuffer();this._bufferLength=0,this._completedStrings===null?this._completedStrings=[e]:this._completedStrings[this._completedStrings.length]=e}appendCharCode(e){const t=this._capacity-this._bufferLength;t<=1&&(t===0||Gi(e))&&this._flushBuffer(),this._buffer[this._bufferLength++]=e}appendASCIICharCode(e){this._bufferLength===this._capacity&&this._flushBuffer(),this._buffer[this._bufferLength++]=e}appendString(e){const t=e.length;if(this._bufferLength+t>=this._capacity){this._flushBuffer(),this._completedStrings[this._completedStrings.length]=e;return}for(let i=0;i<t;i++)this._buffer[this._bufferLength++]=e.charCodeAt(i)}}class Zy{constructor(e,t,i,n,s,r){this._richEditBracketBrand=void 0,this.languageId=e,this.index=t,this.open=i,this.close=n,this.forwardRegex=s,this.reversedRegex=r,this._openSet=Zy._toSet(this.open),this._closeSet=Zy._toSet(this.close)}isOpen(e){return this._openSet.has(e)}isClose(e){return this._closeSet.has(e)}static _toSet(e){const t=new Set;for(const i of e)t.add(i);return t}}function sne(o){const e=o.length;o=o.map(r=>[r[0].toLowerCase(),r[1].toLowerCase()]);const t=[];for(let r=0;r<e;r++)t[r]=r;const i=(r,a)=>{const[l,c]=r,[d,h]=a;return l===d||l===h||c===d||c===h},n=(r,a)=>{const l=Math.min(r,a),c=Math.max(r,a);for(let d=0;d<e;d++)t[d]===c&&(t[d]=l)};for(let r=0;r<e;r++){const a=o[r];for(let l=r+1;l<e;l++){const c=o[l];i(a,c)&&n(t[r],t[l])}}const s=[];for(let r=0;r<e;r++){const a=[],l=[];for(let c=0;c<e;c++)if(t[c]===r){const[d,h]=o[c];a.push(d),l.push(h)}a.length>0&&s.push({open:a,close:l})}return s}class one{constructor(e,t){this._richEditBracketsBrand=void 0;const i=sne(t);this.brackets=i.map((n,s)=>new Zy(e,s,n.open,n.close,rne(n.open,n.close,i,s),ane(n.open,n.close,i,s))),this.forwardRegex=lne(this.brackets),this.reversedRegex=cne(this.brackets),this.textIsBracket={},this.textIsOpenBracket={},this.maxBracketLength=0;for(const n of this.brackets){for(const s of n.open)this.textIsBracket[s]=n,this.textIsOpenBracket[s]=!0,this.maxBracketLength=Math.max(this.maxBracketLength,s.length);for(const s of n.close)this.textIsBracket[s]=n,this.textIsOpenBracket[s]=!1,this.maxBracketLength=Math.max(this.maxBracketLength,s.length)}}}function GU(o,e,t,i){for(let n=0,s=e.length;n<s;n++){if(n===t)continue;const r=e[n];for(const a of r.open)a.indexOf(o)>=0&&i.push(a);for(const a of r.close)a.indexOf(o)>=0&&i.push(a)}}function ZU(o,e){return o.length-e.length}function MI(o){if(o.length<=1)return o;const e=[],t=new Set;for(const i of o)t.has(i)||(e.push(i),t.add(i));return e}function rne(o,e,t,i){let n=[];n=n.concat(o),n=n.concat(e);for(let s=0,r=n.length;s<r;s++)GU(n[s],t,i,n);return n=MI(n),n.sort(ZU),n.reverse(),B1(n)}function ane(o,e,t,i){let n=[];n=n.concat(o),n=n.concat(e);for(let s=0,r=n.length;s<r;s++)GU(n[s],t,i,n);return n=MI(n),n.sort(ZU),n.reverse(),B1(n.map(f5))}function lne(o){let e=[];for(const t of o){for(const i of t.open)e.push(i);for(const i of t.close)e.push(i)}return e=MI(e),B1(e)}function cne(o){let e=[];for(const t of o){for(const i of t.open)e.push(i);for(const i of t.close)e.push(i)}return e=MI(e),B1(e.map(f5))}function dne(o){const e=/^[\w ]+$/.test(o);return o=Uo(o),e?`\\b${o}\\b`:o}function B1(o,e){const t=`(${o.map(dne).join(")|(")})`;return tU(t,!0,e)}const f5=function(){function o(i){const n=new Uint16Array(i.length);let s=0;for(let r=i.length-1;r>=0;r--)n[s++]=i.charCodeAt(r);return qU().decode(n)}let e=null,t=null;return function(n){return e!==n&&(e=n,t=o(e)),t}}();class Tr{static _findPrevBracketInText(e,t,i,n){const s=i.match(e);if(!s)return null;const r=i.length-(s.index||0),a=s[0].length,l=n+r;return new x(t,l-a+1,t,l+1)}static findPrevBracketInRange(e,t,i,n,s){const a=f5(i).substring(i.length-s,i.length-n);return this._findPrevBracketInText(e,t,a,n)}static findNextBracketInText(e,t,i,n){const s=i.match(e);if(!s)return null;const r=s.index||0,a=s[0].length;if(a===0)return null;const l=n+r;return new x(t,l+1,t,l+1+a)}static findNextBracketInRange(e,t,i,n,s){const r=i.substring(n,s);return this.findNextBracketInText(e,t,r,n)}}class hne{constructor(e){this._richEditBrackets=e}getElectricCharacters(){const e=[];if(this._richEditBrackets)for(const t of this._richEditBrackets.brackets)for(const i of t.close){const n=i.charAt(i.length-1);e.push(n)}return Xc(e)}onElectricCharacter(e,t,i){if(!this._richEditBrackets||this._richEditBrackets.brackets.length===0)return null;const n=t.findTokenIndexAtOffset(i-1);if(ol(t.getStandardTokenType(n)))return null;const s=this._richEditBrackets.reversedRegex,r=t.getLineContent().substring(0,i-1)+e,a=Tr.findPrevBracketInRange(s,1,r,0,r.length);if(!a)return null;const l=r.substring(a.startColumn-1,a.endColumn-1).toLowerCase();if(this._richEditBrackets.textIsOpenBracket[l])return null;const d=t.getActualLineContentBefore(a.startColumn-1);return/^\s*$/.test(d)?{matchOpenBracket:l}:null}}function RS(o){return o.global&&(o.lastIndex=0),!0}class une{constructor(e){this._indentationRules=e}shouldIncrease(e){return!!(this._indentationRules&&this._indentationRules.increaseIndentPattern&&RS(this._indentationRules.increaseIndentPattern)&&this._indentationRules.increaseIndentPattern.test(e))}shouldDecrease(e){return!!(this._indentationRules&&this._indentationRules.decreaseIndentPattern&&RS(this._indentationRules.decreaseIndentPattern)&&this._indentationRules.decreaseIndentPattern.test(e))}shouldIndentNextLine(e){return!!(this._indentationRules&&this._indentationRules.indentNextLinePattern&&RS(this._indentationRules.indentNextLinePattern)&&this._indentationRules.indentNextLinePattern.test(e))}shouldIgnore(e){return!!(this._indentationRules&&this._indentationRules.unIndentedLinePattern&&RS(this._indentationRules.unIndentedLinePattern)&&this._indentationRules.unIndentedLinePattern.test(e))}getIndentMetadata(e){let t=0;return this.shouldIncrease(e)&&(t+=1),this.shouldDecrease(e)&&(t+=2),this.shouldIndentNextLine(e)&&(t+=4),this.shouldIgnore(e)&&(t+=8),t}}class ip{constructor(e){e=e||{},e.brackets=e.brackets||[["(",")"],["{","}"],["[","]"]],this._brackets=[],e.brackets.forEach(t=>{const i=ip._createOpenBracketRegExp(t[0]),n=ip._createCloseBracketRegExp(t[1]);i&&n&&this._brackets.push({open:t[0],openRegExp:i,close:t[1],closeRegExp:n})}),this._regExpRules=e.onEnterRules||[]}onEnter(e,t,i,n){if(e>=3)for(let s=0,r=this._regExpRules.length;s<r;s++){const a=this._regExpRules[s];if([{reg:a.beforeText,text:i},{reg:a.afterText,text:n},{reg:a.previousLineText,text:t}].every(c=>c.reg?(c.reg.lastIndex=0,c.reg.test(c.text)):!0))return a.action}if(e>=2&&i.length>0&&n.length>0)for(let s=0,r=this._brackets.length;s<r;s++){const a=this._brackets[s];if(a.openRegExp.test(i)&&a.closeRegExp.test(n))return{indentAction:Bi.IndentOutdent}}if(e>=2&&i.length>0){for(let s=0,r=this._brackets.length;s<r;s++)if(this._brackets[s].openRegExp.test(i))return{indentAction:Bi.Indent}}return null}static _createOpenBracketRegExp(e){let t=Uo(e);return/\B/.test(t.charAt(0))||(t="\\b"+t),t+="\\s*$",ip._safeRegExp(t)}static _createCloseBracketRegExp(e){let t=Uo(e);return/\B/.test(t.charAt(t.length-1))||(t=t+"\\b"),t="^\\s*"+t,ip._safeRegExp(t)}static _safeRegExp(e){try{return new RegExp(e)}catch(t){return Oe(t),null}}}const qe=et("configurationService");function BM(o,e){const t=Object.create(null);for(const i in o)YU(t,i,o[i],e);return t}function YU(o,e,t,i){const n=e.split("."),s=n.pop();let r=o;for(let a=0;a<n.length;a++){const l=n[a];let c=r[l];switch(typeof c){case"undefined":c=r[l]=Object.create(null);break;case"object":if(c===null){i(`Ignoring ${e} as ${n.slice(0,a+1).join(".")} is null`);return}break;default:i(`Ignoring ${e} as ${n.slice(0,a+1).join(".")} is ${JSON.stringify(c)}`);return}r=c}if(typeof r=="object"&&r!==null)try{r[s]=t}catch{i(`Ignoring ${e} as ${n.join(".")} is ${JSON.stringify(r)}`)}else i(`Ignoring ${e} as ${n.join(".")} is ${JSON.stringify(r)}`)}function gne(o,e){const t=e.split(".");XU(o,t)}function XU(o,e){const t=e.shift();if(e.length===0){delete o[t];return}if(Object.keys(o).indexOf(t)!==-1){const i=o[t];typeof i=="object"&&!Array.isArray(i)&&(XU(i,e),Object.keys(i).length===0&&delete o[t])}}function y7(o,e,t){function i(r,a){let l=r;for(const c of a){if(typeof l!="object"||l===null)return;l=l[c]}return l}const n=e.split("."),s=i(o,n);return typeof s>"u"?t:s}function fne(o){return o.replace(/[\[\]]/g,"")}const ni=et("languageService");class Cl{constructor(e,t=[],i=!1){this.ctor=e,this.staticArguments=t,this.supportsDelayedInstantiation=i}}const QU=[];function st(o,e,t){e instanceof Cl||(e=new Cl(e,[],!!t)),QU.push([o,e])}function L7(){return QU}const vi=Object.freeze({text:"text/plain",binary:"application/octet-stream",unknown:"application/unknown",markdown:"text/markdown",latex:"text/latex",uriList:"text/uri-list"}),AI={JSONContribution:"base.contributions.json"};function mne(o){return o.length>0&&o.charAt(o.length-1)==="#"?o.substring(0,o.length-1):o}class pne{constructor(){this._onDidChangeSchema=new P,this.schemasById={}}registerSchema(e,t){this.schemasById[mne(e)]=t,this._onDidChangeSchema.fire(e)}notifySchemaChanged(e){this._onDidChangeSchema.fire(e)}}const _ne=new pne;ci.add(AI.JSONContribution,_ne);const ta={Configuration:"base.contributions.configuration"},MS="vscode://schemas/settings/resourceLanguage",x7=ci.as(AI.JSONContribution);class bne{constructor(){this.registeredConfigurationDefaults=[],this.overrideIdentifiers=new Set,this._onDidSchemaChange=new P,this._onDidUpdateConfiguration=new P,this.configurationDefaultsOverrides=new Map,this.defaultLanguageConfigurationOverridesNode={id:"defaultOverrides",title:m("defaultLanguageConfigurationOverrides.title","Default Language Configuration Overrides"),properties:{}},this.configurationContributors=[this.defaultLanguageConfigurationOverridesNode],this.resourceLanguageSettingsSchema={properties:{},patternProperties:{},additionalProperties:!0,allowTrailingCommas:!0,allowComments:!0},this.configurationProperties={},this.policyConfigurations=new Map,this.excludedConfigurationProperties={},x7.registerSchema(MS,this.resourceLanguageSettingsSchema),this.registerOverridePropertyPatternKey()}registerConfiguration(e,t=!0){this.registerConfigurations([e],t)}registerConfigurations(e,t=!0){const i=new Set;this.doRegisterConfigurations(e,t,i),x7.registerSchema(MS,this.resourceLanguageSettingsSchema),this._onDidSchemaChange.fire(),this._onDidUpdateConfiguration.fire({properties:i})}registerDefaultConfigurations(e){const t=new Set;this.doRegisterDefaultConfigurations(e,t),this._onDidSchemaChange.fire(),this._onDidUpdateConfiguration.fire({properties:t,defaultsOverrides:!0})}doRegisterDefaultConfigurations(e,t){this.registeredConfigurationDefaults.push(...e);const i=[];for(const{overrides:n,source:s}of e)for(const r in n){t.add(r);const a=this.configurationDefaultsOverrides.get(r)??this.configurationDefaultsOverrides.set(r,{configurationDefaultOverrides:[]}).get(r),l=n[r];if(a.configurationDefaultOverrides.push({value:l,source:s}),Vh.test(r)){const c=this.mergeDefaultConfigurationsForOverrideIdentifier(r,l,s,a.configurationDefaultOverrideValue);if(!c)continue;a.configurationDefaultOverrideValue=c,this.updateDefaultOverrideProperty(r,c,s),i.push(...Yy(r))}else{const c=this.mergeDefaultConfigurationsForConfigurationProperty(r,l,s,a.configurationDefaultOverrideValue);if(!c)continue;a.configurationDefaultOverrideValue=c;const d=this.configurationProperties[r];d&&(this.updatePropertyDefaultValue(r,d),this.updateSchema(r,d))}}this.doRegisterOverrideIdentifiers(i)}updateDefaultOverrideProperty(e,t,i){const n={type:"object",default:t.value,description:m("defaultLanguageConfiguration.description","Configure settings to be overridden for the {0} language.",fne(e)),$ref:MS,defaultDefaultValue:t.value,source:i,defaultValueSource:i};this.configurationProperties[e]=n,this.defaultLanguageConfigurationOverridesNode.properties[e]=n}mergeDefaultConfigurationsForOverrideIdentifier(e,t,i,n){const s=(n==null?void 0:n.value)||{},r=(n==null?void 0:n.source)??new Map;if(!(r instanceof Map)){console.error("objectConfigurationSources is not a Map");return}for(const a of Object.keys(t)){const l=t[a];if(on(l)&&(Xn(s[a])||on(s[a]))){if(s[a]={...s[a]??{},...l},i)for(const d in l)r.set(`${a}.${d}`,i)}else s[a]=l,i?r.set(a,i):r.delete(a)}return{value:s,source:r}}mergeDefaultConfigurationsForConfigurationProperty(e,t,i,n){const s=this.configurationProperties[e],r=(n==null?void 0:n.value)??(s==null?void 0:s.defaultDefaultValue);let a=i;if(on(t)&&(s!==void 0&&s.type==="object"||s===void 0&&(Xn(r)||on(r)))){if(a=(n==null?void 0:n.source)??new Map,!(a instanceof Map)){console.error("defaultValueSource is not a Map");return}for(const c in t)i&&a.set(`${e}.${c}`,i);t={...on(r)?r:{},...t}}return{value:t,source:a}}registerOverrideIdentifiers(e){this.doRegisterOverrideIdentifiers(e),this._onDidSchemaChange.fire()}doRegisterOverrideIdentifiers(e){for(const t of e)this.overrideIdentifiers.add(t);this.updateOverridePropertyPatternKey()}doRegisterConfigurations(e,t,i){e.forEach(n=>{this.validateAndRegisterProperties(n,t,n.extensionInfo,n.restrictedProperties,void 0,i),this.configurationContributors.push(n),this.registerJSONConfiguration(n)})}validateAndRegisterProperties(e,t=!0,i,n,s=3,r){var c;s=Ro(e.scope)?s:e.scope;const a=e.properties;if(a)for(const d in a){const h=a[d];if(t&&wne(d,h)){delete a[d];continue}if(h.source=i,h.defaultDefaultValue=a[d].default,this.updatePropertyDefaultValue(d,h),Vh.test(d)?h.scope=void 0:(h.scope=Ro(h.scope)?s:h.scope,h.restricted=Ro(h.restricted)?!!(n!=null&&n.includes(d)):h.restricted),a[d].hasOwnProperty("included")&&!a[d].included){this.excludedConfigurationProperties[d]=a[d],delete a[d];continue}else this.configurationProperties[d]=a[d],(c=a[d].policy)!=null&&c.name&&this.policyConfigurations.set(a[d].policy.name,d);!a[d].deprecationMessage&&a[d].markdownDeprecationMessage&&(a[d].deprecationMessage=a[d].markdownDeprecationMessage),r.add(d)}const l=e.allOf;if(l)for(const d of l)this.validateAndRegisterProperties(d,t,i,n,s,r)}getConfigurationProperties(){return this.configurationProperties}getPolicyConfigurations(){return this.policyConfigurations}registerJSONConfiguration(e){const t=i=>{const n=i.properties;if(n)for(const r in n)this.updateSchema(r,n[r]);const s=i.allOf;s==null||s.forEach(t)};t(e)}updateSchema(e,t){switch(t.scope){case 1:break;case 2:break;case 6:break;case 3:break;case 4:break;case 5:this.resourceLanguageSettingsSchema.properties[e]=t;break}}updateOverridePropertyPatternKey(){for(const e of this.overrideIdentifiers.values()){const t=`[${e}]`,i={type:"object",description:m("overrideSettings.defaultDescription","Configure editor settings to be overridden for a language."),errorMessage:m("overrideSettings.errorMessage","This setting does not support per-language configuration."),$ref:MS};this.updatePropertyDefaultValue(t,i)}}registerOverridePropertyPatternKey(){m("overrideSettings.defaultDescription","Configure editor settings to be overridden for a language."),m("overrideSettings.errorMessage","This setting does not support per-language configuration."),this._onDidSchemaChange.fire()}updatePropertyDefaultValue(e,t){var r;const i=(r=this.configurationDefaultsOverrides.get(e))==null?void 0:r.configurationDefaultOverrideValue;let n,s;i&&(!t.disallowConfigurationDefault||!i.source)&&(n=i.value,s=i.source),Xn(n)&&(n=t.defaultDefaultValue,s=void 0),Xn(n)&&(n=vne(t.type)),t.default=n,t.defaultValueSource=s}}const JU="\\[([^\\]]+)\\]",k7=new RegExp(JU,"g"),Cne=`^(${JU})+$`,Vh=new RegExp(Cne);function Yy(o){const e=[];if(Vh.test(o)){let t=k7.exec(o);for(;t!=null&&t.length;){const i=t[1].trim();i&&e.push(i),t=k7.exec(o)}}return Xc(e)}function vne(o){switch(Array.isArray(o)?o[0]:o){case"boolean":return!1;case"integer":case"number":return 0;case"string":return"";case"array":return[];case"object":return{};default:return null}}const U0=new bne;ci.add(ta.Configuration,U0);function wne(o,e){var t,i,n,s;return o.trim()?Vh.test(o)?m("config.property.languageDefault","Cannot register '{0}'. This matches property pattern '\\\\[.*\\\\]$' for describing language specific editor settings. Use 'configurationDefaults' contribution.",o):U0.getConfigurationProperties()[o]!==void 0?m("config.property.duplicate","Cannot register '{0}'. This property is already registered.",o):(t=e.policy)!=null&&t.name&&U0.getPolicyConfigurations().get((i=e.policy)==null?void 0:i.name)!==void 0?m("config.policy.duplicate","Cannot register '{0}'. The associated policy {1} is already registered with {2}.",o,(n=e.policy)==null?void 0:n.name,U0.getPolicyConfigurations().get((s=e.policy)==null?void 0:s.name)):null:m("config.property.empty","Cannot register an empty property")}const Sne={ModesRegistry:"editor.modesRegistry"};class yne{constructor(){this._onDidChangeLanguages=new P,this.onDidChangeLanguages=this._onDidChangeLanguages.event,this._languages=[]}registerLanguage(e){return this._languages.push(e),this._onDidChangeLanguages.fire(void 0),{dispose:()=>{for(let t=0,i=this._languages.length;t<i;t++)if(this._languages[t]===e){this._languages.splice(t,1);return}}}}getLanguages(){return this._languages}}const I_=new yne;ci.add(Sne.ModesRegistry,I_);const ms="plaintext",Lne=".txt";I_.registerLanguage({id:ms,extensions:[Lne],aliases:[m("plainText.alias","Plain Text"),"text"],mimetypes:[vi.text]});ci.as(ta.Configuration).registerDefaultConfigurations([{overrides:{"[plaintext]":{"editor.unicodeHighlight.ambiguousCharacters":!1,"editor.unicodeHighlight.invisibleCharacters":!1}}}]);class xne{constructor(e,t){this.languageId=e;const i=t.brackets?D7(t.brackets):[],n=new YB(a=>{const l=new Set;return{info:new kne(this,a,l),closing:l}}),s=new YB(a=>{const l=new Set,c=new Set;return{info:new Dne(this,a,l,c),opening:l,openingColorized:c}});for(const[a,l]of i){const c=n.get(a),d=s.get(l);c.closing.add(d.info),d.opening.add(c.info)}const r=t.colorizedBracketPairs?D7(t.colorizedBracketPairs):i.filter(a=>!(a[0]==="<"&&a[1]===">"));for(const[a,l]of r){const c=n.get(a),d=s.get(l);c.closing.add(d.info),d.openingColorized.add(c.info),d.opening.add(c.info)}this._openingBrackets=new Map([...n.cachedValues].map(([a,l])=>[a,l.info])),this._closingBrackets=new Map([...s.cachedValues].map(([a,l])=>[a,l.info]))}get openingBrackets(){return[...this._openingBrackets.values()]}get closingBrackets(){return[...this._closingBrackets.values()]}getOpeningBracketInfo(e){return this._openingBrackets.get(e)}getClosingBracketInfo(e){return this._closingBrackets.get(e)}getBracketInfo(e){return this.getOpeningBracketInfo(e)||this.getClosingBracketInfo(e)}getBracketRegExp(e){const t=Array.from([...this._openingBrackets.keys(),...this._closingBrackets.keys()]);return B1(t,e)}}function D7(o){return o.filter(([e,t])=>e!==""&&t!=="")}class e${constructor(e,t){this.config=e,this.bracketText=t}get languageId(){return this.config.languageId}}class kne extends e${constructor(e,t,i){super(e,t),this.openedBrackets=i,this.isOpeningBracket=!0}}class Dne extends e${constructor(e,t,i,n){super(e,t),this.openingBrackets=i,this.openingColorizedBrackets=n,this.isOpeningBracket=!1}closes(e){return e.config!==this.config?!1:this.openingBrackets.has(e)}closesColorized(e){return e.config!==this.config?!1:this.openingColorizedBrackets.has(e)}getOpeningBrackets(){return[...this.openingBrackets]}}var Ine=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},I7=function(o,e){return function(t,i){e(t,i,o)}};class IN{constructor(e){this.languageId=e}affects(e){return this.languageId?this.languageId===e:!0}}const di=et("languageConfigurationService");let HM=class extends F{constructor(e,t){super(),this.configurationService=e,this.languageService=t,this._registry=this._register(new Rne),this.onDidChangeEmitter=this._register(new P),this.onDidChange=this.onDidChangeEmitter.event,this.configurations=new Map;const i=new Set(Object.values(VM));this._register(this.configurationService.onDidChangeConfiguration(n=>{const s=n.change.keys.some(a=>i.has(a)),r=n.change.overrides.filter(([a,l])=>l.some(c=>i.has(c))).map(([a])=>a);if(s)this.configurations.clear(),this.onDidChangeEmitter.fire(new IN(void 0));else for(const a of r)this.languageService.isRegisteredLanguageId(a)&&(this.configurations.delete(a),this.onDidChangeEmitter.fire(new IN(a)))})),this._register(this._registry.onDidChange(n=>{this.configurations.delete(n.languageId),this.onDidChangeEmitter.fire(new IN(n.languageId))}))}register(e,t,i){return this._registry.register(e,t,i)}getLanguageConfiguration(e){let t=this.configurations.get(e);return t||(t=Ene(e,this._registry,this.configurationService,this.languageService),this.configurations.set(e,t)),t}};HM=Ine([I7(0,qe),I7(1,ni)],HM);function Ene(o,e,t,i){let n=e.getLanguageConfiguration(o);if(!n){if(!i.isRegisteredLanguageId(o))return new jp(o,{});n=new jp(o,{})}const s=Nne(n.languageId,t),r=i$([n.underlyingConfig,s]);return new jp(n.languageId,r)}const VM={brackets:"editor.language.brackets",colorizedBracketPairs:"editor.language.colorizedBracketPairs"};function Nne(o,e){const t=e.getValue(VM.brackets,{overrideIdentifier:o}),i=e.getValue(VM.colorizedBracketPairs,{overrideIdentifier:o});return{brackets:E7(t),colorizedBracketPairs:E7(i)}}function E7(o){if(Array.isArray(o))return o.map(e=>{if(!(!Array.isArray(e)||e.length!==2))return[e[0],e[1]]}).filter(e=>!!e)}function t$(o,e,t){const i=o.getLineContent(e);let n=Nt(i);return n.length>t-1&&(n=n.substring(0,t-1)),n}class Tne{constructor(e){this.languageId=e,this._resolved=null,this._entries=[],this._order=0,this._resolved=null}register(e,t){const i=new N7(e,t,++this._order);return this._entries.push(i),this._resolved=null,ve(()=>{for(let n=0;n<this._entries.length;n++)if(this._entries[n]===i){this._entries.splice(n,1),this._resolved=null;break}})}getResolvedConfiguration(){if(!this._resolved){const e=this._resolve();e&&(this._resolved=new jp(this.languageId,e))}return this._resolved}_resolve(){return this._entries.length===0?null:(this._entries.sort(N7.cmp),i$(this._entries.map(e=>e.configuration)))}}function i$(o){let e={comments:void 0,brackets:void 0,wordPattern:void 0,indentationRules:void 0,onEnterRules:void 0,autoClosingPairs:void 0,surroundingPairs:void 0,autoCloseBefore:void 0,folding:void 0,colorizedBracketPairs:void 0,__electricCharacterSupport:void 0};for(const t of o)e={comments:t.comments||e.comments,brackets:t.brackets||e.brackets,wordPattern:t.wordPattern||e.wordPattern,indentationRules:t.indentationRules||e.indentationRules,onEnterRules:t.onEnterRules||e.onEnterRules,autoClosingPairs:t.autoClosingPairs||e.autoClosingPairs,surroundingPairs:t.surroundingPairs||e.surroundingPairs,autoCloseBefore:t.autoCloseBefore||e.autoCloseBefore,folding:t.folding||e.folding,colorizedBracketPairs:t.colorizedBracketPairs||e.colorizedBracketPairs,__electricCharacterSupport:t.__electricCharacterSupport||e.__electricCharacterSupport};return e}class N7{constructor(e,t,i){this.configuration=e,this.priority=t,this.order=i}static cmp(e,t){return e.priority===t.priority?e.order-t.order:e.priority-t.priority}}class T7{constructor(e){this.languageId=e}}class Rne extends F{constructor(){super(),this._entries=new Map,this._onDidChange=this._register(new P),this.onDidChange=this._onDidChange.event,this._register(this.register(ms,{brackets:[["(",")"],["[","]"],["{","}"]],surroundingPairs:[{open:"{",close:"}"},{open:"[",close:"]"},{open:"(",close:")"},{open:"<",close:">"},{open:'"',close:'"'},{open:"'",close:"'"},{open:"`",close:"`"}],colorizedBracketPairs:[],folding:{offSide:!0}},0))}register(e,t,i=0){let n=this._entries.get(e);n||(n=new Tne(e),this._entries.set(e,n));const s=n.register(t,i);return this._onDidChange.fire(new T7(e)),ve(()=>{s.dispose(),this._onDidChange.fire(new T7(e))})}getLanguageConfiguration(e){const t=this._entries.get(e);return(t==null?void 0:t.getResolvedConfiguration())||null}}class jp{constructor(e,t){this.languageId=e,this.underlyingConfig=t,this._brackets=null,this._electricCharacter=null,this._onEnterSupport=this.underlyingConfig.brackets||this.underlyingConfig.indentationRules||this.underlyingConfig.onEnterRules?new ip(this.underlyingConfig):null,this.comments=jp._handleComments(this.underlyingConfig),this.characterPair=new WM(this.underlyingConfig),this.wordDefinition=this.underlyingConfig.wordPattern||V4,this.indentationRules=this.underlyingConfig.indentationRules,this.underlyingConfig.indentationRules?this.indentRulesSupport=new une(this.underlyingConfig.indentationRules):this.indentRulesSupport=null,this.foldingRules=this.underlyingConfig.folding||{},this.bracketsNew=new xne(e,this.underlyingConfig)}getWordDefinition(){return z4(this.wordDefinition)}get brackets(){return!this._brackets&&this.underlyingConfig.brackets&&(this._brackets=new one(this.languageId,this.underlyingConfig.brackets)),this._brackets}get electricCharacter(){return this._electricCharacter||(this._electricCharacter=new hne(this.brackets)),this._electricCharacter}onEnter(e,t,i,n){return this._onEnterSupport?this._onEnterSupport.onEnter(e,t,i,n):null}getAutoClosingPairs(){return new Xie(this.characterPair.getAutoClosingPairs())}getAutoCloseBeforeSet(e){return this.characterPair.getAutoCloseBeforeSet(e)}getSurroundingPairs(){return this.characterPair.getSurroundingPairs()}static _handleComments(e){const t=e.comments;if(!t)return null;const i={};if(t.lineComment&&(i.lineCommentToken=t.lineComment),t.blockComment){const[n,s]=t.blockComment;i.blockCommentStartToken=n,i.blockCommentEndToken=s}return i}}st(di,HM,1);class kd{constructor(e,t,i,n){this.originalStart=e,this.originalLength=t,this.modifiedStart=i,this.modifiedLength=n}getOriginalEnd(){return this.originalStart+this.originalLength}getModifiedEnd(){return this.modifiedStart+this.modifiedLength}}class R7{constructor(e){this.source=e}getElements(){const e=this.source,t=new Int32Array(e.length);for(let i=0,n=e.length;i<n;i++)t[i]=e.charCodeAt(i);return t}}function Mne(o,e,t){return new yl(new R7(o),new R7(e)).ComputeDiff(t).changes}class ym{static Assert(e,t){if(!e)throw new Error(t)}}class Lm{static Copy(e,t,i,n,s){for(let r=0;r<s;r++)i[n+r]=e[t+r]}static Copy2(e,t,i,n,s){for(let r=0;r<s;r++)i[n+r]=e[t+r]}}class M7{constructor(){this.m_changes=[],this.m_originalStart=1073741824,this.m_modifiedStart=1073741824,this.m_originalCount=0,this.m_modifiedCount=0}MarkNextChange(){(this.m_originalCount>0||this.m_modifiedCount>0)&&this.m_changes.push(new kd(this.m_originalStart,this.m_originalCount,this.m_modifiedStart,this.m_modifiedCount)),this.m_originalCount=0,this.m_modifiedCount=0,this.m_originalStart=1073741824,this.m_modifiedStart=1073741824}AddOriginalElement(e,t){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,t),this.m_originalCount++}AddModifiedElement(e,t){this.m_originalStart=Math.min(this.m_originalStart,e),this.m_modifiedStart=Math.min(this.m_modifiedStart,t),this.m_modifiedCount++}getChanges(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes}getReverseChanges(){return(this.m_originalCount>0||this.m_modifiedCount>0)&&this.MarkNextChange(),this.m_changes.reverse(),this.m_changes}}class yl{constructor(e,t,i=null){this.ContinueProcessingPredicate=i,this._originalSequence=e,this._modifiedSequence=t;const[n,s,r]=yl._getElements(e),[a,l,c]=yl._getElements(t);this._hasStrings=r&&c,this._originalStringElements=n,this._originalElementsOrHash=s,this._modifiedStringElements=a,this._modifiedElementsOrHash=l,this.m_forwardHistory=[],this.m_reverseHistory=[]}static _isStringArray(e){return e.length>0&&typeof e[0]=="string"}static _getElements(e){const t=e.getElements();if(yl._isStringArray(t)){const i=new Int32Array(t.length);for(let n=0,s=t.length;n<s;n++)i[n]=o5(t[n],0);return[t,i,!0]}return t instanceof Int32Array?[[],t,!1]:[[],new Int32Array(t),!1]}ElementsAreEqual(e,t){return this._originalElementsOrHash[e]!==this._modifiedElementsOrHash[t]?!1:this._hasStrings?this._originalStringElements[e]===this._modifiedStringElements[t]:!0}ElementsAreStrictEqual(e,t){if(!this.ElementsAreEqual(e,t))return!1;const i=yl._getStrictElement(this._originalSequence,e),n=yl._getStrictElement(this._modifiedSequence,t);return i===n}static _getStrictElement(e,t){return typeof e.getStrictElement=="function"?e.getStrictElement(t):null}OriginalElementsAreEqual(e,t){return this._originalElementsOrHash[e]!==this._originalElementsOrHash[t]?!1:this._hasStrings?this._originalStringElements[e]===this._originalStringElements[t]:!0}ModifiedElementsAreEqual(e,t){return this._modifiedElementsOrHash[e]!==this._modifiedElementsOrHash[t]?!1:this._hasStrings?this._modifiedStringElements[e]===this._modifiedStringElements[t]:!0}ComputeDiff(e){return this._ComputeDiff(0,this._originalElementsOrHash.length-1,0,this._modifiedElementsOrHash.length-1,e)}_ComputeDiff(e,t,i,n,s){const r=[!1];let a=this.ComputeDiffRecursive(e,t,i,n,r);return s&&(a=this.PrettifyChanges(a)),{quitEarly:r[0],changes:a}}ComputeDiffRecursive(e,t,i,n,s){for(s[0]=!1;e<=t&&i<=n&&this.ElementsAreEqual(e,i);)e++,i++;for(;t>=e&&n>=i&&this.ElementsAreEqual(t,n);)t--,n--;if(e>t||i>n){let h;return i<=n?(ym.Assert(e===t+1,"originalStart should only be one more than originalEnd"),h=[new kd(e,0,i,n-i+1)]):e<=t?(ym.Assert(i===n+1,"modifiedStart should only be one more than modifiedEnd"),h=[new kd(e,t-e+1,i,0)]):(ym.Assert(e===t+1,"originalStart should only be one more than originalEnd"),ym.Assert(i===n+1,"modifiedStart should only be one more than modifiedEnd"),h=[]),h}const r=[0],a=[0],l=this.ComputeRecursionPoint(e,t,i,n,r,a,s),c=r[0],d=a[0];if(l!==null)return l;if(!s[0]){const h=this.ComputeDiffRecursive(e,c,i,d,s);let u=[];return s[0]?u=[new kd(c+1,t-(c+1)+1,d+1,n-(d+1)+1)]:u=this.ComputeDiffRecursive(c+1,t,d+1,n,s),this.ConcatenateChanges(h,u)}return[new kd(e,t-e+1,i,n-i+1)]}WALKTRACE(e,t,i,n,s,r,a,l,c,d,h,u,g,f,p,_,b,C){let v=null,w=null,S=new M7,L=t,k=i,D=g[0]-_[0]-n,N=-1073741824,B=this.m_forwardHistory.length-1;do{const W=D+e;W===L||W<k&&c[W-1]<c[W+1]?(h=c[W+1],f=h-D-n,h<N&&S.MarkNextChange(),N=h,S.AddModifiedElement(h+1,f),D=W+1-e):(h=c[W-1]+1,f=h-D-n,h<N&&S.MarkNextChange(),N=h-1,S.AddOriginalElement(h,f+1),D=W-1-e),B>=0&&(c=this.m_forwardHistory[B],e=c[0],L=1,k=c.length-1)}while(--B>=-1);if(v=S.getReverseChanges(),C[0]){let W=g[0]+1,V=_[0]+1;if(v!==null&&v.length>0){const $=v[v.length-1];W=Math.max(W,$.getOriginalEnd()),V=Math.max(V,$.getModifiedEnd())}w=[new kd(W,u-W+1,V,p-V+1)]}else{S=new M7,L=r,k=a,D=g[0]-_[0]-l,N=1073741824,B=b?this.m_reverseHistory.length-1:this.m_reverseHistory.length-2;do{const W=D+s;W===L||W<k&&d[W-1]>=d[W+1]?(h=d[W+1]-1,f=h-D-l,h>N&&S.MarkNextChange(),N=h+1,S.AddOriginalElement(h+1,f+1),D=W+1-s):(h=d[W-1],f=h-D-l,h>N&&S.MarkNextChange(),N=h,S.AddModifiedElement(h+1,f+1),D=W-1-s),B>=0&&(d=this.m_reverseHistory[B],s=d[0],L=1,k=d.length-1)}while(--B>=-1);w=S.getChanges()}return this.ConcatenateChanges(v,w)}ComputeRecursionPoint(e,t,i,n,s,r,a){let l=0,c=0,d=0,h=0,u=0,g=0;e--,i--,s[0]=0,r[0]=0,this.m_forwardHistory=[],this.m_reverseHistory=[];const f=t-e+(n-i),p=f+1,_=new Int32Array(p),b=new Int32Array(p),C=n-i,v=t-e,w=e-i,S=t-n,k=(v-C)%2===0;_[C]=e,b[v]=t,a[0]=!1;for(let D=1;D<=f/2+1;D++){let N=0,B=0;d=this.ClipDiagonalBound(C-D,D,C,p),h=this.ClipDiagonalBound(C+D,D,C,p);for(let V=d;V<=h;V+=2){V===d||V<h&&_[V-1]<_[V+1]?l=_[V+1]:l=_[V-1]+1,c=l-(V-C)-w;const $=l;for(;l<t&&c<n&&this.ElementsAreEqual(l+1,c+1);)l++,c++;if(_[V]=l,l+c>N+B&&(N=l,B=c),!k&&Math.abs(V-v)<=D-1&&l>=b[V])return s[0]=l,r[0]=c,$<=b[V]&&D<=1448?this.WALKTRACE(C,d,h,w,v,u,g,S,_,b,l,t,s,c,n,r,k,a):null}const W=(N-e+(B-i)-D)/2;if(this.ContinueProcessingPredicate!==null&&!this.ContinueProcessingPredicate(N,W))return a[0]=!0,s[0]=N,r[0]=B,W>0&&D<=1448?this.WALKTRACE(C,d,h,w,v,u,g,S,_,b,l,t,s,c,n,r,k,a):(e++,i++,[new kd(e,t-e+1,i,n-i+1)]);u=this.ClipDiagonalBound(v-D,D,v,p),g=this.ClipDiagonalBound(v+D,D,v,p);for(let V=u;V<=g;V+=2){V===u||V<g&&b[V-1]>=b[V+1]?l=b[V+1]-1:l=b[V-1],c=l-(V-v)-S;const $=l;for(;l>e&&c>i&&this.ElementsAreEqual(l,c);)l--,c--;if(b[V]=l,k&&Math.abs(V-C)<=D&&l<=_[V])return s[0]=l,r[0]=c,$>=_[V]&&D<=1448?this.WALKTRACE(C,d,h,w,v,u,g,S,_,b,l,t,s,c,n,r,k,a):null}if(D<=1447){let V=new Int32Array(h-d+2);V[0]=C-d+1,Lm.Copy2(_,d,V,1,h-d+1),this.m_forwardHistory.push(V),V=new Int32Array(g-u+2),V[0]=v-u+1,Lm.Copy2(b,u,V,1,g-u+1),this.m_reverseHistory.push(V)}}return this.WALKTRACE(C,d,h,w,v,u,g,S,_,b,l,t,s,c,n,r,k,a)}PrettifyChanges(e){for(let t=0;t<e.length;t++){const i=e[t],n=t<e.length-1?e[t+1].originalStart:this._originalElementsOrHash.length,s=t<e.length-1?e[t+1].modifiedStart:this._modifiedElementsOrHash.length,r=i.originalLength>0,a=i.modifiedLength>0;for(;i.originalStart+i.originalLength<n&&i.modifiedStart+i.modifiedLength<s&&(!r||this.OriginalElementsAreEqual(i.originalStart,i.originalStart+i.originalLength))&&(!a||this.ModifiedElementsAreEqual(i.modifiedStart,i.modifiedStart+i.modifiedLength));){const c=this.ElementsAreStrictEqual(i.originalStart,i.modifiedStart);if(this.ElementsAreStrictEqual(i.originalStart+i.originalLength,i.modifiedStart+i.modifiedLength)&&!c)break;i.originalStart++,i.modifiedStart++}const l=[null];if(t<e.length-1&&this.ChangesOverlap(e[t],e[t+1],l)){e[t]=l[0],e.splice(t+1,1),t--;continue}}for(let t=e.length-1;t>=0;t--){const i=e[t];let n=0,s=0;if(t>0){const h=e[t-1];n=h.originalStart+h.originalLength,s=h.modifiedStart+h.modifiedLength}const r=i.originalLength>0,a=i.modifiedLength>0;let l=0,c=this._boundaryScore(i.originalStart,i.originalLength,i.modifiedStart,i.modifiedLength);for(let h=1;;h++){const u=i.originalStart-h,g=i.modifiedStart-h;if(u<n||g<s||r&&!this.OriginalElementsAreEqual(u,u+i.originalLength)||a&&!this.ModifiedElementsAreEqual(g,g+i.modifiedLength))break;const p=(u===n&&g===s?5:0)+this._boundaryScore(u,i.originalLength,g,i.modifiedLength);p>c&&(c=p,l=h)}i.originalStart-=l,i.modifiedStart-=l;const d=[null];if(t>0&&this.ChangesOverlap(e[t-1],e[t],d)){e[t-1]=d[0],e.splice(t,1),t++;continue}}if(this._hasStrings)for(let t=1,i=e.length;t<i;t++){const n=e[t-1],s=e[t],r=s.originalStart-n.originalStart-n.originalLength,a=n.originalStart,l=s.originalStart+s.originalLength,c=l-a,d=n.modifiedStart,h=s.modifiedStart+s.modifiedLength,u=h-d;if(r<5&&c<20&&u<20){const g=this._findBetterContiguousSequence(a,c,d,u,r);if(g){const[f,p]=g;(f!==n.originalStart+n.originalLength||p!==n.modifiedStart+n.modifiedLength)&&(n.originalLength=f-n.originalStart,n.modifiedLength=p-n.modifiedStart,s.originalStart=f+r,s.modifiedStart=p+r,s.originalLength=l-s.originalStart,s.modifiedLength=h-s.modifiedStart)}}}return e}_findBetterContiguousSequence(e,t,i,n,s){if(t<s||n<s)return null;const r=e+t-s+1,a=i+n-s+1;let l=0,c=0,d=0;for(let h=e;h<r;h++)for(let u=i;u<a;u++){const g=this._contiguousSequenceScore(h,u,s);g>0&&g>l&&(l=g,c=h,d=u)}return l>0?[c,d]:null}_contiguousSequenceScore(e,t,i){let n=0;for(let s=0;s<i;s++){if(!this.ElementsAreEqual(e+s,t+s))return 0;n+=this._originalStringElements[e+s].length}return n}_OriginalIsBoundary(e){return e<=0||e>=this._originalElementsOrHash.length-1?!0:this._hasStrings&&/^\s*$/.test(this._originalStringElements[e])}_OriginalRegionIsBoundary(e,t){if(this._OriginalIsBoundary(e)||this._OriginalIsBoundary(e-1))return!0;if(t>0){const i=e+t;if(this._OriginalIsBoundary(i-1)||this._OriginalIsBoundary(i))return!0}return!1}_ModifiedIsBoundary(e){return e<=0||e>=this._modifiedElementsOrHash.length-1?!0:this._hasStrings&&/^\s*$/.test(this._modifiedStringElements[e])}_ModifiedRegionIsBoundary(e,t){if(this._ModifiedIsBoundary(e)||this._ModifiedIsBoundary(e-1))return!0;if(t>0){const i=e+t;if(this._ModifiedIsBoundary(i-1)||this._ModifiedIsBoundary(i))return!0}return!1}_boundaryScore(e,t,i,n){const s=this._OriginalRegionIsBoundary(e,t)?1:0,r=this._ModifiedRegionIsBoundary(i,n)?1:0;return s+r}ConcatenateChanges(e,t){const i=[];if(e.length===0||t.length===0)return t.length>0?t:e;if(this.ChangesOverlap(e[e.length-1],t[0],i)){const n=new Array(e.length+t.length-1);return Lm.Copy(e,0,n,0,e.length-1),n[e.length-1]=i[0],Lm.Copy(t,1,n,e.length,t.length-1),n}else{const n=new Array(e.length+t.length);return Lm.Copy(e,0,n,0,e.length),Lm.Copy(t,0,n,e.length,t.length),n}}ChangesOverlap(e,t,i){if(ym.Assert(e.originalStart<=t.originalStart,"Left change is not less than or equal to right change"),ym.Assert(e.modifiedStart<=t.modifiedStart,"Left change is not less than or equal to right change"),e.originalStart+e.originalLength>=t.originalStart||e.modifiedStart+e.modifiedLength>=t.modifiedStart){const n=e.originalStart;let s=e.originalLength;const r=e.modifiedStart;let a=e.modifiedLength;return e.originalStart+e.originalLength>=t.originalStart&&(s=t.originalStart+t.originalLength-e.originalStart),e.modifiedStart+e.modifiedLength>=t.modifiedStart&&(a=t.modifiedStart+t.modifiedLength-e.modifiedStart),i[0]=new kd(n,s,r,a),!0}else return i[0]=null,!1}ClipDiagonalBound(e,t,i,n){if(e>=0&&e<n)return e;const s=i,r=n-i-1,a=t%2===0;if(e<0){const l=s%2===0;return a===l?0:1}else{const l=r%2===0;return a===l?n-1:n-2}}}function Xy(o){return o<0?0:o>255?255:o|0}function xm(o){return o<0?0:o>4294967295?4294967295:o|0}class bb{constructor(e){const t=Xy(e);this._defaultValue=t,this._asciiMap=bb._createAsciiMap(t),this._map=new Map}static _createAsciiMap(e){const t=new Uint8Array(256);return t.fill(e),t}set(e,t){const i=Xy(t);e>=0&&e<256?this._asciiMap[e]=i:this._map.set(e,i)}get(e){return e>=0&&e<256?this._asciiMap[e]:this._map.get(e)||this._defaultValue}clear(){this._asciiMap.fill(this._defaultValue),this._map.clear()}}class Qy{constructor(){this._actual=new bb(0)}add(e){this._actual.set(e,1)}has(e){return this._actual.get(e)===1}clear(){return this._actual.clear()}}class Ane{constructor(e,t,i){const n=new Uint8Array(e*t);for(let s=0,r=e*t;s<r;s++)n[s]=i;this._data=n,this.rows=e,this.cols=t}get(e,t){return this._data[e*this.cols+t]}set(e,t,i){this._data[e*this.cols+t]=i}}class Pne{constructor(e){let t=0,i=0;for(let s=0,r=e.length;s<r;s++){const[a,l,c]=e[s];l>t&&(t=l),a>i&&(i=a),c>i&&(i=c)}t++,i++;const n=new Ane(i,t,0);for(let s=0,r=e.length;s<r;s++){const[a,l,c]=e[s];n.set(a,l,c)}this._states=n,this._maxCharCode=t}nextState(e,t){return t<0||t>=this._maxCharCode?0:this._states.get(e,t)}}let EN=null;function One(){return EN===null&&(EN=new Pne([[1,104,2],[1,72,2],[1,102,6],[1,70,6],[2,116,3],[2,84,3],[3,116,4],[3,84,4],[4,112,5],[4,80,5],[5,115,9],[5,83,9],[5,58,10],[6,105,7],[6,73,7],[7,108,8],[7,76,8],[8,101,9],[8,69,9],[9,58,10],[10,47,11],[11,47,12]])),EN}let Qb=null;function Fne(){if(Qb===null){Qb=new bb(0);const o=` 	<>'"、。｡､，．：；‘〈「『〔（［｛｢｣｝］）〕』」〉’｀～…`;for(let t=0;t<o.length;t++)Qb.set(o.charCodeAt(t),1);const e=".,;:";for(let t=0;t<e.length;t++)Qb.set(e.charCodeAt(t),2)}return Qb}class Jy{static _createLink(e,t,i,n,s){let r=s-1;do{const a=t.charCodeAt(r);if(e.get(a)!==2)break;r--}while(r>n);if(n>0){const a=t.charCodeAt(n-1),l=t.charCodeAt(r);(a===40&&l===41||a===91&&l===93||a===123&&l===125)&&r--}return{range:{startLineNumber:i,startColumn:n+1,endLineNumber:i,endColumn:r+2},url:t.substring(n,r+1)}}static computeLinks(e,t=One()){const i=Fne(),n=[];for(let s=1,r=e.getLineCount();s<=r;s++){const a=e.getLineContent(s),l=a.length;let c=0,d=0,h=0,u=1,g=!1,f=!1,p=!1,_=!1;for(;c<l;){let b=!1;const C=a.charCodeAt(c);if(u===13){let v;switch(C){case 40:g=!0,v=0;break;case 41:v=g?0:1;break;case 91:p=!0,f=!0,v=0;break;case 93:p=!1,v=f?0:1;break;case 123:_=!0,v=0;break;case 125:v=_?0:1;break;case 39:case 34:case 96:h===C?v=1:h===39||h===34||h===96?v=0:v=1;break;case 42:v=h===42?1:0;break;case 124:v=h===124?1:0;break;case 32:v=p?0:1;break;default:v=i.get(C)}v===1&&(n.push(Jy._createLink(i,a,s,d,c)),b=!0)}else if(u===12){let v;C===91?(f=!0,v=0):v=i.get(C),v===1?b=!0:u=13}else u=t.nextState(u,C),u===0&&(b=!0);b&&(u=1,g=!1,f=!1,_=!1,d=c+1,h=C),c++}u===13&&n.push(Jy._createLink(i,a,s,d,l))}return n}}function Wne(o){return!o||typeof o.getLineCount!="function"||typeof o.getLineContent!="function"?[]:Jy.computeLinks(o)}const Ok=class Ok{constructor(){this._defaultValueSet=[["true","false"],["True","False"],["Private","Public","Friend","ReadOnly","Partial","Protected","WriteOnly"],["public","protected","private"]]}navigateValueSet(e,t,i,n,s){if(e&&t){const r=this.doNavigateValueSet(t,s);if(r)return{range:e,value:r}}if(i&&n){const r=this.doNavigateValueSet(n,s);if(r)return{range:i,value:r}}return null}doNavigateValueSet(e,t){const i=this.numberReplace(e,t);return i!==null?i:this.textReplace(e,t)}numberReplace(e,t){const i=Math.pow(10,e.length-(e.lastIndexOf(".")+1));let n=Number(e);const s=parseFloat(e);return!isNaN(n)&&!isNaN(s)&&n===s?n===0&&!t?null:(n=Math.floor(n*i),n+=t?i:-i,String(n/i)):null}textReplace(e,t){return this.valueSetsReplace(this._defaultValueSet,e,t)}valueSetsReplace(e,t,i){let n=null;for(let s=0,r=e.length;n===null&&s<r;s++)n=this.valueSetReplace(e[s],t,i);return n}valueSetReplace(e,t,i){let n=e.indexOf(t);return n>=0?(n+=i?1:-1,n<0?n=e.length-1:n%=e.length,e[n]):null}};Ok.INSTANCE=new Ok;let zM=Ok;const Mv=class Mv{static getChannel(e){return e.getChannel(Mv.CHANNEL_NAME)}static setChannel(e,t){e.setChannel(Mv.CHANNEL_NAME,t)}};Mv.CHANNEL_NAME="editorWorkerHost";let UM=Mv;var A7,P7;class Bne{constructor(e,t){this.uri=e,this.value=t}}function Hne(o){return Array.isArray(o)}const tg=class tg{constructor(e,t){if(this[A7]="ResourceMap",e instanceof tg)this.map=new Map(e.map),this.toKey=t??tg.defaultToKey;else if(Hne(e)){this.map=new Map,this.toKey=t??tg.defaultToKey;for(const[i,n]of e)this.set(i,n)}else this.map=new Map,this.toKey=e??tg.defaultToKey}set(e,t){return this.map.set(this.toKey(e),new Bne(e,t)),this}get(e){var t;return(t=this.map.get(this.toKey(e)))==null?void 0:t.value}has(e){return this.map.has(this.toKey(e))}get size(){return this.map.size}clear(){this.map.clear()}delete(e){return this.map.delete(this.toKey(e))}forEach(e,t){typeof t<"u"&&(e=e.bind(t));for(const[i,n]of this.map)e(n.value,n.uri,this)}*values(){for(const e of this.map.values())yield e.value}*keys(){for(const e of this.map.values())yield e.uri}*entries(){for(const e of this.map.values())yield[e.uri,e.value]}*[(A7=Symbol.toStringTag,Symbol.iterator)](){for(const[,e]of this.map)yield[e.uri,e.value]}};tg.defaultToKey=e=>e.toString();let an=tg;class Vne{constructor(){this[P7]="LinkedMap",this._map=new Map,this._head=void 0,this._tail=void 0,this._size=0,this._state=0}clear(){this._map.clear(),this._head=void 0,this._tail=void 0,this._size=0,this._state++}isEmpty(){return!this._head&&!this._tail}get size(){return this._size}get first(){var e;return(e=this._head)==null?void 0:e.value}get last(){var e;return(e=this._tail)==null?void 0:e.value}has(e){return this._map.has(e)}get(e,t=0){const i=this._map.get(e);if(i)return t!==0&&this.touch(i,t),i.value}set(e,t,i=0){let n=this._map.get(e);if(n)n.value=t,i!==0&&this.touch(n,i);else{switch(n={key:e,value:t,next:void 0,previous:void 0},i){case 0:this.addItemLast(n);break;case 1:this.addItemFirst(n);break;case 2:this.addItemLast(n);break;default:this.addItemLast(n);break}this._map.set(e,n),this._size++}return this}delete(e){return!!this.remove(e)}remove(e){const t=this._map.get(e);if(t)return this._map.delete(e),this.removeItem(t),this._size--,t.value}shift(){if(!this._head&&!this._tail)return;if(!this._head||!this._tail)throw new Error("Invalid list");const e=this._head;return this._map.delete(e.key),this.removeItem(e),this._size--,e.value}forEach(e,t){const i=this._state;let n=this._head;for(;n;){if(t?e.bind(t)(n.value,n.key,this):e(n.value,n.key,this),this._state!==i)throw new Error("LinkedMap got modified during iteration.");n=n.next}}keys(){const e=this,t=this._state;let i=this._head;const n={[Symbol.iterator](){return n},next(){if(e._state!==t)throw new Error("LinkedMap got modified during iteration.");if(i){const s={value:i.key,done:!1};return i=i.next,s}else return{value:void 0,done:!0}}};return n}values(){const e=this,t=this._state;let i=this._head;const n={[Symbol.iterator](){return n},next(){if(e._state!==t)throw new Error("LinkedMap got modified during iteration.");if(i){const s={value:i.value,done:!1};return i=i.next,s}else return{value:void 0,done:!0}}};return n}entries(){const e=this,t=this._state;let i=this._head;const n={[Symbol.iterator](){return n},next(){if(e._state!==t)throw new Error("LinkedMap got modified during iteration.");if(i){const s={value:[i.key,i.value],done:!1};return i=i.next,s}else return{value:void 0,done:!0}}};return n}[(P7=Symbol.toStringTag,Symbol.iterator)](){return this.entries()}trimOld(e){if(e>=this.size)return;if(e===0){this.clear();return}let t=this._head,i=this.size;for(;t&&i>e;)this._map.delete(t.key),t=t.next,i--;this._head=t,this._size=i,t&&(t.previous=void 0),this._state++}trimNew(e){if(e>=this.size)return;if(e===0){this.clear();return}let t=this._tail,i=this.size;for(;t&&i>e;)this._map.delete(t.key),t=t.previous,i--;this._tail=t,this._size=i,t&&(t.next=void 0),this._state++}addItemFirst(e){if(!this._head&&!this._tail)this._tail=e;else if(this._head)e.next=this._head,this._head.previous=e;else throw new Error("Invalid list");this._head=e,this._state++}addItemLast(e){if(!this._head&&!this._tail)this._head=e;else if(this._tail)e.previous=this._tail,this._tail.next=e;else throw new Error("Invalid list");this._tail=e,this._state++}removeItem(e){if(e===this._head&&e===this._tail)this._head=void 0,this._tail=void 0;else if(e===this._head){if(!e.next)throw new Error("Invalid list");e.next.previous=void 0,this._head=e.next}else if(e===this._tail){if(!e.previous)throw new Error("Invalid list");e.previous.next=void 0,this._tail=e.previous}else{const t=e.next,i=e.previous;if(!t||!i)throw new Error("Invalid list");t.previous=i,i.next=t}e.next=void 0,e.previous=void 0,this._state++}touch(e,t){if(!this._head||!this._tail)throw new Error("Invalid list");if(!(t!==1&&t!==2)){if(t===1){if(e===this._head)return;const i=e.next,n=e.previous;e===this._tail?(n.next=void 0,this._tail=n):(i.previous=n,n.next=i),e.previous=void 0,e.next=this._head,this._head.previous=e,this._head=e,this._state++}else if(t===2){if(e===this._tail)return;const i=e.next,n=e.previous;e===this._head?(i.previous=void 0,this._head=i):(i.previous=n,n.next=i),e.next=void 0,e.previous=this._tail,this._tail.next=e,this._tail=e,this._state++}}}toJSON(){const e=[];return this.forEach((t,i)=>{e.push([i,t])}),e}fromJSON(e){this.clear();for(const[t,i]of e)this.set(t,i)}}class zne extends Vne{constructor(e,t=1){super(),this._limit=e,this._ratio=Math.min(Math.max(0,t),1)}get limit(){return this._limit}set limit(e){this._limit=e,this.checkTrim()}get(e,t=2){return super.get(e,t)}peek(e){return super.get(e,0)}set(e,t){return super.set(e,t,2),this}checkTrim(){this.size>this._limit&&this.trim(Math.round(this._limit*this._ratio))}}class Gl extends zne{constructor(e,t=1){super(e,t)}trim(e){this.trimOld(e)}set(e,t){return super.set(e,t),this.checkTrim(),this}}class Une{constructor(e){if(this._m1=new Map,this._m2=new Map,e)for(const[t,i]of e)this.set(t,i)}clear(){this._m1.clear(),this._m2.clear()}set(e,t){this._m1.set(e,t),this._m2.set(t,e)}get(e){return this._m1.get(e)}getKey(e){return this._m2.get(e)}delete(e){const t=this._m1.get(e);return t===void 0?!1:(this._m1.delete(e),this._m2.delete(t),!0)}keys(){return this._m1.keys()}values(){return this._m1.values()}}class m5{constructor(){this.map=new Map}add(e,t){let i=this.map.get(e);i||(i=new Set,this.map.set(e,i)),i.add(t)}delete(e,t){const i=this.map.get(e);i&&(i.delete(t),i.size===0&&this.map.delete(e))}forEach(e,t){const i=this.map.get(e);i&&i.forEach(t)}get(e){const t=this.map.get(e);return t||new Set}}class $ne extends bb{constructor(e,t){super(0),this._segmenter=null,this._cachedLine=null,this._cachedSegments=[],this.intlSegmenterLocales=t,this.intlSegmenterLocales.length>0?this._segmenter=new Intl.Segmenter(this.intlSegmenterLocales,{granularity:"word"}):this._segmenter=null;for(let i=0,n=e.length;i<n;i++)this.set(e.charCodeAt(i),2);this.set(32,1),this.set(9,1)}findPrevIntlWordBeforeOrAtOffset(e,t){let i=null;for(const n of this._getIntlSegmenterWordsOnLine(e)){if(n.index>t)break;i=n}return i}findNextIntlWordAtOrAfterOffset(e,t){for(const i of this._getIntlSegmenterWordsOnLine(e))if(!(i.index<t))return i;return null}_getIntlSegmenterWordsOnLine(e){return this._segmenter?this._cachedLine===e?this._cachedSegments:(this._cachedLine=e,this._cachedSegments=this._filterWordSegments(this._segmenter.segment(e)),this._cachedSegments):[]}_filterWordSegments(e){const t=[];for(const i of e)this._isWordLike(i)&&t.push(i);return t}_isWordLike(e){return!!e.isWordLike}}const O7=new Gl(10);function Cr(o,e){const t=`${o}/${e.join(",")}`;let i=O7.get(t);return i||(i=new $ne(o,e),O7.set(t,i)),i}var Ho;(function(o){o[o.Left=1]="Left",o[o.Center=2]="Center",o[o.Right=4]="Right",o[o.Full=7]="Full"})(Ho||(Ho={}));var Zr;(function(o){o[o.Left=1]="Left",o[o.Center=2]="Center",o[o.Right=3]="Right"})(Zr||(Zr={}));var zr;(function(o){o[o.Both=0]="Both",o[o.Right=1]="Right",o[o.Left=2]="Left",o[o.None=3]="None"})(zr||(zr={}));class $0{get originalIndentSize(){return this._indentSizeIsTabSize?"tabSize":this.indentSize}constructor(e){this._textModelResolvedOptionsBrand=void 0,this.tabSize=Math.max(1,e.tabSize|0),e.indentSize==="tabSize"?(this.indentSize=this.tabSize,this._indentSizeIsTabSize=!0):(this.indentSize=Math.max(1,e.indentSize|0),this._indentSizeIsTabSize=!1),this.insertSpaces=!!e.insertSpaces,this.defaultEOL=e.defaultEOL|0,this.trimAutoWhitespace=!!e.trimAutoWhitespace,this.bracketPairColorizationOptions=e.bracketPairColorizationOptions}equals(e){return this.tabSize===e.tabSize&&this._indentSizeIsTabSize===e._indentSizeIsTabSize&&this.indentSize===e.indentSize&&this.insertSpaces===e.insertSpaces&&this.defaultEOL===e.defaultEOL&&this.trimAutoWhitespace===e.trimAutoWhitespace&&oo(this.bracketPairColorizationOptions,e.bracketPairColorizationOptions)}createChangeEvent(e){return{tabSize:this.tabSize!==e.tabSize,indentSize:this.indentSize!==e.indentSize,insertSpaces:this.insertSpaces!==e.insertSpaces,trimAutoWhitespace:this.trimAutoWhitespace!==e.trimAutoWhitespace}}}class cw{constructor(e,t){this._findMatchBrand=void 0,this.range=e,this.matches=t}}function jne(o){return o&&typeof o.read=="function"}class NN{constructor(e,t,i,n,s,r){this.identifier=e,this.range=t,this.text=i,this.forceMoveMarkers=n,this.isAutoWhitespaceEdit=s,this._isTracked=r}}class Kne{constructor(e,t,i){this.regex=e,this.wordSeparators=t,this.simpleSearch=i}}class qne{constructor(e,t,i){this.reverseEdits=e,this.changes=t,this.trimAutoWhitespaceLineNumbers=i}}function n$(o){return!o.isTooLargeForSyncing()&&!o.isForSimpleWidget}const Gne=999;class Tu{constructor(e,t,i,n){this.searchString=e,this.isRegex=t,this.matchCase=i,this.wordSeparators=n}parseSearchRequest(){if(this.searchString==="")return null;let e;this.isRegex?e=Zne(this.searchString):e=this.searchString.indexOf(`
`)>=0;let t=null;try{t=tU(this.searchString,this.isRegex,{matchCase:this.matchCase,wholeWord:!1,multiline:e,global:!0,unicode:!0})}catch{return null}if(!t)return null;let i=!this.isRegex&&!e;return i&&this.searchString.toLowerCase()!==this.searchString.toUpperCase()&&(i=this.matchCase),new Kne(t,this.wordSeparators?Cr(this.wordSeparators,[]):null,i?this.searchString:null)}}function Zne(o){if(!o||o.length===0)return!1;for(let e=0,t=o.length;e<t;e++){const i=o.charCodeAt(e);if(i===10)return!0;if(i===92){if(e++,e>=t)break;const n=o.charCodeAt(e);if(n===110||n===114||n===87)return!0}}return!1}function Vu(o,e,t){if(!t)return new cw(o,null);const i=[];for(let n=0,s=e.length;n<s;n++)i[n]=e[n];return new cw(o,i)}class F7{constructor(e){const t=[];let i=0;for(let n=0,s=e.length;n<s;n++)e.charCodeAt(n)===10&&(t[i++]=n);this._lineFeedsOffsets=t}findLineFeedCountBeforeOffset(e){const t=this._lineFeedsOffsets;let i=0,n=t.length-1;if(n===-1||e<=t[0])return 0;for(;i<n;){const s=i+((n-i)/2>>0);t[s]>=e?n=s-1:t[s+1]>=e?(i=s,n=s):i=s+1}return i+1}}class AS{static findMatches(e,t,i,n,s){const r=t.parseSearchRequest();return r?r.regex.multiline?this._doFindMatchesMultiline(e,i,new np(r.wordSeparators,r.regex),n,s):this._doFindMatchesLineByLine(e,i,r,n,s):[]}static _getMultilineMatchRange(e,t,i,n,s,r){let a,l=0;n?(l=n.findLineFeedCountBeforeOffset(s),a=t+s+l):a=t+s;let c;if(n){const g=n.findLineFeedCountBeforeOffset(s+r.length)-l;c=a+r.length+g}else c=a+r.length;const d=e.getPositionAt(a),h=e.getPositionAt(c);return new x(d.lineNumber,d.column,h.lineNumber,h.column)}static _doFindMatchesMultiline(e,t,i,n,s){const r=e.getOffsetAt(t.getStartPosition()),a=e.getValueInRange(t,1),l=e.getEOL()===`\r
`?new F7(a):null,c=[];let d=0,h;for(i.reset(0);h=i.next(a);)if(c[d++]=Vu(this._getMultilineMatchRange(e,r,a,l,h.index,h[0]),h,n),d>=s)return c;return c}static _doFindMatchesLineByLine(e,t,i,n,s){const r=[];let a=0;if(t.startLineNumber===t.endLineNumber){const c=e.getLineContent(t.startLineNumber).substring(t.startColumn-1,t.endColumn-1);return a=this._findMatchesInLine(i,c,t.startLineNumber,t.startColumn-1,a,r,n,s),r}const l=e.getLineContent(t.startLineNumber).substring(t.startColumn-1);a=this._findMatchesInLine(i,l,t.startLineNumber,t.startColumn-1,a,r,n,s);for(let c=t.startLineNumber+1;c<t.endLineNumber&&a<s;c++)a=this._findMatchesInLine(i,e.getLineContent(c),c,0,a,r,n,s);if(a<s){const c=e.getLineContent(t.endLineNumber).substring(0,t.endColumn-1);a=this._findMatchesInLine(i,c,t.endLineNumber,0,a,r,n,s)}return r}static _findMatchesInLine(e,t,i,n,s,r,a,l){const c=e.wordSeparators;if(!a&&e.simpleSearch){const u=e.simpleSearch,g=u.length,f=t.length;let p=-g;for(;(p=t.indexOf(u,p+g))!==-1;)if((!c||p5(c,t,f,p,g))&&(r[s++]=new cw(new x(i,p+1+n,i,p+1+g+n),null),s>=l))return s;return s}const d=new np(e.wordSeparators,e.regex);let h;d.reset(0);do if(h=d.next(t),h&&(r[s++]=Vu(new x(i,h.index+1+n,i,h.index+1+h[0].length+n),h,a),s>=l))return s;while(h);return s}static findNextMatch(e,t,i,n){const s=t.parseSearchRequest();if(!s)return null;const r=new np(s.wordSeparators,s.regex);return s.regex.multiline?this._doFindNextMatchMultiline(e,i,r,n):this._doFindNextMatchLineByLine(e,i,r,n)}static _doFindNextMatchMultiline(e,t,i,n){const s=new A(t.lineNumber,1),r=e.getOffsetAt(s),a=e.getLineCount(),l=e.getValueInRange(new x(s.lineNumber,s.column,a,e.getLineMaxColumn(a)),1),c=e.getEOL()===`\r
`?new F7(l):null;i.reset(t.column-1);const d=i.next(l);return d?Vu(this._getMultilineMatchRange(e,r,l,c,d.index,d[0]),d,n):t.lineNumber!==1||t.column!==1?this._doFindNextMatchMultiline(e,new A(1,1),i,n):null}static _doFindNextMatchLineByLine(e,t,i,n){const s=e.getLineCount(),r=t.lineNumber,a=e.getLineContent(r),l=this._findFirstMatchInLine(i,a,r,t.column,n);if(l)return l;for(let c=1;c<=s;c++){const d=(r+c-1)%s,h=e.getLineContent(d+1),u=this._findFirstMatchInLine(i,h,d+1,1,n);if(u)return u}return null}static _findFirstMatchInLine(e,t,i,n,s){e.reset(n-1);const r=e.next(t);return r?Vu(new x(i,r.index+1,i,r.index+1+r[0].length),r,s):null}static findPreviousMatch(e,t,i,n){const s=t.parseSearchRequest();if(!s)return null;const r=new np(s.wordSeparators,s.regex);return s.regex.multiline?this._doFindPreviousMatchMultiline(e,i,r,n):this._doFindPreviousMatchLineByLine(e,i,r,n)}static _doFindPreviousMatchMultiline(e,t,i,n){const s=this._doFindMatchesMultiline(e,new x(1,1,t.lineNumber,t.column),i,n,10*Gne);if(s.length>0)return s[s.length-1];const r=e.getLineCount();return t.lineNumber!==r||t.column!==e.getLineMaxColumn(r)?this._doFindPreviousMatchMultiline(e,new A(r,e.getLineMaxColumn(r)),i,n):null}static _doFindPreviousMatchLineByLine(e,t,i,n){const s=e.getLineCount(),r=t.lineNumber,a=e.getLineContent(r).substring(0,t.column-1),l=this._findLastMatchInLine(i,a,r,n);if(l)return l;for(let c=1;c<=s;c++){const d=(s+r-c-1)%s,h=e.getLineContent(d+1),u=this._findLastMatchInLine(i,h,d+1,n);if(u)return u}return null}static _findLastMatchInLine(e,t,i,n){let s=null,r;for(e.reset(0);r=e.next(t);)s=Vu(new x(i,r.index+1,i,r.index+1+r[0].length),r,n);return s}}function Yne(o,e,t,i,n){if(i===0)return!0;const s=e.charCodeAt(i-1);if(o.get(s)!==0||s===13||s===10)return!0;if(n>0){const r=e.charCodeAt(i);if(o.get(r)!==0)return!0}return!1}function Xne(o,e,t,i,n){if(i+n===t)return!0;const s=e.charCodeAt(i+n);if(o.get(s)!==0||s===13||s===10)return!0;if(n>0){const r=e.charCodeAt(i+n-1);if(o.get(r)!==0)return!0}return!1}function p5(o,e,t,i,n){return Yne(o,e,t,i,n)&&Xne(o,e,t,i,n)}class np{constructor(e,t){this._wordSeparators=e,this._searchRegex=t,this._prevMatchStartIndex=-1,this._prevMatchLength=0}reset(e){this._searchRegex.lastIndex=e,this._prevMatchStartIndex=-1,this._prevMatchLength=0}next(e){const t=e.length;let i;do{if(this._prevMatchStartIndex+this._prevMatchLength===t||(i=this._searchRegex.exec(e),!i))return null;const n=i.index,s=i[0].length;if(n===this._prevMatchStartIndex&&s===this._prevMatchLength){if(s===0){Hy(e,t,this._searchRegex.lastIndex)>65535?this._searchRegex.lastIndex+=2:this._searchRegex.lastIndex+=1;continue}return null}if(this._prevMatchStartIndex=n,this._prevMatchLength=s,!this._wordSeparators||p5(this._wordSeparators,e,t,n,s))return i}while(i);return null}}class _5{static computeUnicodeHighlights(e,t,i){const n=i?i.startLineNumber:1,s=i?i.endLineNumber:e.getLineCount(),r=new W7(t),a=r.getCandidateCodePoints();let l;a==="allNonBasicAscii"?l=new RegExp("[^\\t\\n\\r\\x20-\\x7E]","g"):l=new RegExp(`${Qne(Array.from(a))}`,"g");const c=new np(null,l),d=[];let h=!1,u,g=0,f=0,p=0;e:for(let _=n,b=s;_<=b;_++){const C=e.getLineContent(_),v=C.length;c.reset(0);do if(u=c.next(C),u){let w=u.index,S=u.index+u[0].length;if(w>0){const N=C.charCodeAt(w-1);Gi(N)&&w--}if(S+1<v){const N=C.charCodeAt(S-1);Gi(N)&&S++}const L=C.substring(w,S);let k=Qv(w+1,V4,C,0);k&&k.endColumn<=w+1&&(k=null);const D=r.shouldHighlightNonBasicASCII(L,k?k.word:null);if(D!==0){if(D===3?g++:D===2?f++:D===1?p++:NI(),d.length>=1e3){h=!0;break e}d.push(new x(_,w+1,_,S+1))}}while(u)}return{ranges:d,hasMore:h,ambiguousCharacterCount:g,invisibleCharacterCount:f,nonBasicAsciiCharacterCount:p}}static computeUnicodeHighlightReason(e,t){const i=new W7(t);switch(i.shouldHighlightNonBasicASCII(e,null)){case 0:return null;case 2:return{kind:1};case 3:{const s=e.codePointAt(0),r=i.ambiguousCharacters.getPrimaryConfusable(s),a=ow.getLocales().filter(l=>!ow.getInstance(new Set([...t.allowedLocales,l])).isAmbiguous(s));return{kind:0,confusableWith:String.fromCodePoint(r),notAmbiguousInLocales:a}}case 1:return{kind:2}}}}function Qne(o,e){return`[${Uo(o.map(i=>String.fromCodePoint(i)).join(""))}]`}class W7{constructor(e){this.options=e,this.allowedCodePoints=new Set(e.allowedCodePoints),this.ambiguousCharacters=ow.getInstance(new Set(e.allowedLocales))}getCandidateCodePoints(){if(this.options.nonBasicASCII)return"allNonBasicAscii";const e=new Set;if(this.options.invisibleCharacters)for(const t of fg.codePoints)B7(String.fromCodePoint(t))||e.add(t);if(this.options.ambiguousCharacters)for(const t of this.ambiguousCharacters.getConfusableCodePoints())e.add(t);for(const t of this.allowedCodePoints)e.delete(t);return e}shouldHighlightNonBasicASCII(e,t){const i=e.codePointAt(0);if(this.allowedCodePoints.has(i))return 0;if(this.options.nonBasicASCII)return 1;let n=!1,s=!1;if(t)for(const r of t){const a=r.codePointAt(0),l=P1(r);n=n||l,!l&&!this.ambiguousCharacters.isAmbiguous(a)&&!fg.isInvisibleCharacter(a)&&(s=!0)}return!n&&s?0:this.options.invisibleCharacters&&!B7(e)&&fg.isInvisibleCharacter(i)?2:this.options.ambiguousCharacters&&this.ambiguousCharacters.isAmbiguous(i)?3:0}}function B7(o){return o===" "||o===`
`||o==="	"}class j0{constructor(e,t,i){this.changes=e,this.moves=t,this.hitTimeout=i}}class s${constructor(e,t){this.lineRangeMapping=e,this.changes=t}}class Ge{static addRange(e,t){let i=0;for(;i<t.length&&t[i].endExclusive<e.start;)i++;let n=i;for(;n<t.length&&t[n].start<=e.endExclusive;)n++;if(i===n)t.splice(i,0,e);else{const s=Math.min(e.start,t[i].start),r=Math.max(e.endExclusive,t[n-1].endExclusive);t.splice(i,n-i,new Ge(s,r))}}static tryCreate(e,t){if(!(e>t))return new Ge(e,t)}static ofLength(e){return new Ge(0,e)}static ofStartAndLength(e,t){return new Ge(e,e+t)}constructor(e,t){if(this.start=e,this.endExclusive=t,e>t)throw new bt(`Invalid range: ${this.toString()}`)}get isEmpty(){return this.start===this.endExclusive}delta(e){return new Ge(this.start+e,this.endExclusive+e)}deltaStart(e){return new Ge(this.start+e,this.endExclusive)}deltaEnd(e){return new Ge(this.start,this.endExclusive+e)}get length(){return this.endExclusive-this.start}toString(){return`[${this.start}, ${this.endExclusive})`}contains(e){return this.start<=e&&e<this.endExclusive}join(e){return new Ge(Math.min(this.start,e.start),Math.max(this.endExclusive,e.endExclusive))}intersect(e){const t=Math.max(this.start,e.start),i=Math.min(this.endExclusive,e.endExclusive);if(t<=i)return new Ge(t,i)}intersects(e){const t=Math.max(this.start,e.start),i=Math.min(this.endExclusive,e.endExclusive);return t<i}isBefore(e){return this.endExclusive<=e.start}isAfter(e){return this.start>=e.endExclusive}slice(e){return e.slice(this.start,this.endExclusive)}substring(e){return e.substring(this.start,this.endExclusive)}clip(e){if(this.isEmpty)throw new bt(`Invalid clipping range: ${this.toString()}`);return Math.max(this.start,Math.min(this.endExclusive-1,e))}clipCyclic(e){if(this.isEmpty)throw new bt(`Invalid clipping range: ${this.toString()}`);return e<this.start?this.endExclusive-(this.start-e)%this.length:e>=this.endExclusive?this.start+(e-this.start)%this.length:e}forEach(e){for(let t=this.start;t<this.endExclusive;t++)e(t)}}class b5{constructor(){this._sortedRanges=[]}addRange(e){let t=0;for(;t<this._sortedRanges.length&&this._sortedRanges[t].endExclusive<e.start;)t++;let i=t;for(;i<this._sortedRanges.length&&this._sortedRanges[i].start<=e.endExclusive;)i++;if(t===i)this._sortedRanges.splice(t,0,e);else{const n=Math.min(e.start,this._sortedRanges[t].start),s=Math.max(e.endExclusive,this._sortedRanges[i-1].endExclusive);this._sortedRanges.splice(t,i-t,new Ge(n,s))}}toString(){return this._sortedRanges.map(e=>e.toString()).join(", ")}intersectsStrict(e){let t=0;for(;t<this._sortedRanges.length&&this._sortedRanges[t].endExclusive<=e.start;)t++;return t<this._sortedRanges.length&&this._sortedRanges[t].start<e.endExclusive}intersectWithRange(e){const t=new b5;for(const i of this._sortedRanges){const n=i.intersect(e);n&&t.addRange(n)}return t}intersectWithRangeLength(e){return this.intersectWithRange(e).length}get length(){return this._sortedRanges.reduce((e,t)=>e+t.length,0)}}function dw(o,e){const t=Jne(o,e);if(t!==-1)return o[t]}function Jne(o,e,t=o.length-1){for(let i=t;i>=0;i--){const n=o[i];if(e(n))return i}return-1}function E_(o,e){const t=hw(o,e);return t===-1?void 0:o[t]}function hw(o,e,t=0,i=o.length){let n=t,s=i;for(;n<s;){const r=Math.floor((n+s)/2);e(o[r])?n=r+1:s=r}return n-1}function ese(o,e){const t=uw(o,e);return t===o.length?void 0:o[t]}function uw(o,e,t=0,i=o.length){let n=t,s=i;for(;n<s;){const r=Math.floor((n+s)/2);e(o[r])?s=r:n=r+1}return n}const Fk=class Fk{constructor(e){this._array=e,this._findLastMonotonousLastIdx=0}findLastMonotonous(e){if(Fk.assertInvariants){if(this._prevFindLastPredicate){for(const i of this._array)if(this._prevFindLastPredicate(i)&&!e(i))throw new Error("MonotonousArray: current predicate must be weaker than (or equal to) the previous predicate.")}this._prevFindLastPredicate=e}const t=hw(this._array,e,this._findLastMonotonousLastIdx);return this._findLastMonotonousLastIdx=t+1,t===-1?void 0:this._array[t]}};Fk.assertInvariants=!1;let eL=Fk;function C5(o,e){if(o.length===0)return;let t=o[0];for(let i=1;i<o.length;i++){const n=o[i];e(n,t)>0&&(t=n)}return t}function tse(o,e){if(o.length===0)return;let t=o[0];for(let i=1;i<o.length;i++){const n=o[i];e(n,t)>=0&&(t=n)}return t}function ise(o,e){return C5(o,(t,i)=>-e(t,i))}function nse(o,e){if(o.length===0)return-1;let t=0;for(let i=1;i<o.length;i++){const n=o[i];e(n,o[t])>0&&(t=i)}return t}function sse(o,e){for(const t of o){const i=e(t);if(i!==void 0)return i}}let Be=class lc{static fromRangeInclusive(e){return new lc(e.startLineNumber,e.endLineNumber+1)}static joinMany(e){if(e.length===0)return[];let t=new dr(e[0].slice());for(let i=1;i<e.length;i++)t=t.getUnion(new dr(e[i].slice()));return t.ranges}static join(e){if(e.length===0)throw new bt("lineRanges cannot be empty");let t=e[0].startLineNumber,i=e[0].endLineNumberExclusive;for(let n=1;n<e.length;n++)t=Math.min(t,e[n].startLineNumber),i=Math.max(i,e[n].endLineNumberExclusive);return new lc(t,i)}static ofLength(e,t){return new lc(e,e+t)}static deserialize(e){return new lc(e[0],e[1])}constructor(e,t){if(e>t)throw new bt(`startLineNumber ${e} cannot be after endLineNumberExclusive ${t}`);this.startLineNumber=e,this.endLineNumberExclusive=t}contains(e){return this.startLineNumber<=e&&e<this.endLineNumberExclusive}get isEmpty(){return this.startLineNumber===this.endLineNumberExclusive}delta(e){return new lc(this.startLineNumber+e,this.endLineNumberExclusive+e)}deltaLength(e){return new lc(this.startLineNumber,this.endLineNumberExclusive+e)}get length(){return this.endLineNumberExclusive-this.startLineNumber}join(e){return new lc(Math.min(this.startLineNumber,e.startLineNumber),Math.max(this.endLineNumberExclusive,e.endLineNumberExclusive))}toString(){return`[${this.startLineNumber},${this.endLineNumberExclusive})`}intersect(e){const t=Math.max(this.startLineNumber,e.startLineNumber),i=Math.min(this.endLineNumberExclusive,e.endLineNumberExclusive);if(t<=i)return new lc(t,i)}intersectsStrict(e){return this.startLineNumber<e.endLineNumberExclusive&&e.startLineNumber<this.endLineNumberExclusive}overlapOrTouch(e){return this.startLineNumber<=e.endLineNumberExclusive&&e.startLineNumber<=this.endLineNumberExclusive}equals(e){return this.startLineNumber===e.startLineNumber&&this.endLineNumberExclusive===e.endLineNumberExclusive}toInclusiveRange(){return this.isEmpty?null:new x(this.startLineNumber,1,this.endLineNumberExclusive-1,Number.MAX_SAFE_INTEGER)}toExclusiveRange(){return new x(this.startLineNumber,1,this.endLineNumberExclusive,1)}mapToLineArray(e){const t=[];for(let i=this.startLineNumber;i<this.endLineNumberExclusive;i++)t.push(e(i));return t}forEach(e){for(let t=this.startLineNumber;t<this.endLineNumberExclusive;t++)e(t)}serialize(){return[this.startLineNumber,this.endLineNumberExclusive]}includes(e){return this.startLineNumber<=e&&e<this.endLineNumberExclusive}toOffsetRange(){return new Ge(this.startLineNumber-1,this.endLineNumberExclusive-1)}};class dr{constructor(e=[]){this._normalizedRanges=e}get ranges(){return this._normalizedRanges}addRange(e){if(e.length===0)return;const t=uw(this._normalizedRanges,n=>n.endLineNumberExclusive>=e.startLineNumber),i=hw(this._normalizedRanges,n=>n.startLineNumber<=e.endLineNumberExclusive)+1;if(t===i)this._normalizedRanges.splice(t,0,e);else if(t===i-1){const n=this._normalizedRanges[t];this._normalizedRanges[t]=n.join(e)}else{const n=this._normalizedRanges[t].join(this._normalizedRanges[i-1]).join(e);this._normalizedRanges.splice(t,i-t,n)}}contains(e){const t=E_(this._normalizedRanges,i=>i.startLineNumber<=e);return!!t&&t.endLineNumberExclusive>e}intersects(e){const t=E_(this._normalizedRanges,i=>i.startLineNumber<e.endLineNumberExclusive);return!!t&&t.endLineNumberExclusive>e.startLineNumber}getUnion(e){if(this._normalizedRanges.length===0)return e;if(e._normalizedRanges.length===0)return this;const t=[];let i=0,n=0,s=null;for(;i<this._normalizedRanges.length||n<e._normalizedRanges.length;){let r=null;if(i<this._normalizedRanges.length&&n<e._normalizedRanges.length){const a=this._normalizedRanges[i],l=e._normalizedRanges[n];a.startLineNumber<l.startLineNumber?(r=a,i++):(r=l,n++)}else i<this._normalizedRanges.length?(r=this._normalizedRanges[i],i++):(r=e._normalizedRanges[n],n++);s===null?s=r:s.endLineNumberExclusive>=r.startLineNumber?s=new Be(s.startLineNumber,Math.max(s.endLineNumberExclusive,r.endLineNumberExclusive)):(t.push(s),s=r)}return s!==null&&t.push(s),new dr(t)}subtractFrom(e){const t=uw(this._normalizedRanges,r=>r.endLineNumberExclusive>=e.startLineNumber),i=hw(this._normalizedRanges,r=>r.startLineNumber<=e.endLineNumberExclusive)+1;if(t===i)return new dr([e]);const n=[];let s=e.startLineNumber;for(let r=t;r<i;r++){const a=this._normalizedRanges[r];a.startLineNumber>s&&n.push(new Be(s,a.startLineNumber)),s=a.endLineNumberExclusive}return s<e.endLineNumberExclusive&&n.push(new Be(s,e.endLineNumberExclusive)),new dr(n)}toString(){return this._normalizedRanges.map(e=>e.toString()).join(", ")}getIntersection(e){const t=[];let i=0,n=0;for(;i<this._normalizedRanges.length&&n<e._normalizedRanges.length;){const s=this._normalizedRanges[i],r=e._normalizedRanges[n],a=s.intersect(r);a&&!a.isEmpty&&t.push(a),s.endLineNumberExclusive<r.endLineNumberExclusive?i++:n++}return new dr(t)}getWithDelta(e){return new dr(this._normalizedRanges.map(t=>t.delta(e)))}}const Rd=class Rd{static betweenPositions(e,t){return e.lineNumber===t.lineNumber?new Rd(0,t.column-e.column):new Rd(t.lineNumber-e.lineNumber,t.column-1)}static ofRange(e){return Rd.betweenPositions(e.getStartPosition(),e.getEndPosition())}static ofText(e){let t=0,i=0;for(const n of e)n===`
`?(t++,i=0):i++;return new Rd(t,i)}constructor(e,t){this.lineCount=e,this.columnCount=t}isGreaterThanOrEqualTo(e){return this.lineCount!==e.lineCount?this.lineCount>e.lineCount:this.columnCount>=e.columnCount}createRange(e){return this.lineCount===0?new x(e.lineNumber,e.column,e.lineNumber,e.column+this.columnCount):new x(e.lineNumber,e.column,e.lineNumber+this.lineCount,this.columnCount+1)}addToPosition(e){return this.lineCount===0?new A(e.lineNumber,e.column+this.columnCount):new A(e.lineNumber+this.lineCount,this.columnCount+1)}toString(){return`${this.lineCount},${this.columnCount}`}};Rd.zero=new Rd(0,0);let Yr=Rd;class ose{constructor(e){this.text=e,this.lineStartOffsetByLineIdx=[],this.lineStartOffsetByLineIdx.push(0);for(let t=0;t<e.length;t++)e.charAt(t)===`
`&&this.lineStartOffsetByLineIdx.push(t+1)}getOffset(e){return this.lineStartOffsetByLineIdx[e.lineNumber-1]+e.column-1}getOffsetRange(e){return new Ge(this.getOffset(e.getStartPosition()),this.getOffset(e.getEndPosition()))}get textLength(){const e=this.lineStartOffsetByLineIdx.length-1;return new Yr(e,this.text.length-this.lineStartOffsetByLineIdx[e])}}class v5{constructor(e){this.edits=e,Nf(()=>u5(e,(t,i)=>t.range.getEndPosition().isBeforeOrEqual(i.range.getStartPosition())))}apply(e){let t="",i=new A(1,1);for(const s of this.edits){const r=s.range,a=r.getStartPosition(),l=r.getEndPosition(),c=H7(i,a);c.isEmpty()||(t+=e.getValueOfRange(c)),t+=s.text,i=l}const n=H7(i,e.endPositionExclusive);return n.isEmpty()||(t+=e.getValueOfRange(n)),t}applyToString(e){const t=new rse(e);return this.apply(t)}getNewRanges(){const e=[];let t=0,i=0,n=0;for(const s of this.edits){const r=Yr.ofText(s.text),a=A.lift({lineNumber:s.range.startLineNumber+i,column:s.range.startColumn+(s.range.startLineNumber===t?n:0)}),l=r.createRange(a);e.push(l),i=l.endLineNumber-s.range.endLineNumber,n=l.endColumn-s.range.endColumn,t=s.range.endLineNumber}return e}}class Fl{constructor(e,t){this.range=e,this.text=t}toSingleEditOperation(){return{range:this.range,text:this.text}}}function H7(o,e){if(o.lineNumber===e.lineNumber&&o.column===Number.MAX_SAFE_INTEGER)return x.fromPositions(e,e);if(!o.isBeforeOrEqual(e))throw new bt("start must be before end");return new x(o.lineNumber,o.column,e.lineNumber,e.column)}class o${get endPositionExclusive(){return this.length.addToPosition(new A(1,1))}}class rse extends o${constructor(e){super(),this.value=e,this._t=new ose(this.value)}getValueOfRange(e){return this._t.getOffsetRange(e).substring(this.value)}get length(){return this._t.textLength}}class Yn{static inverse(e,t,i){const n=[];let s=1,r=1;for(const l of e){const c=new Yn(new Be(s,l.original.startLineNumber),new Be(r,l.modified.startLineNumber));c.modified.isEmpty||n.push(c),s=l.original.endLineNumberExclusive,r=l.modified.endLineNumberExclusive}const a=new Yn(new Be(s,t+1),new Be(r,i+1));return a.modified.isEmpty||n.push(a),n}static clip(e,t,i){const n=[];for(const s of e){const r=s.original.intersect(t),a=s.modified.intersect(i);r&&!r.isEmpty&&a&&!a.isEmpty&&n.push(new Yn(r,a))}return n}constructor(e,t){this.original=e,this.modified=t}toString(){return`{${this.original.toString()}->${this.modified.toString()}}`}flip(){return new Yn(this.modified,this.original)}join(e){return new Yn(this.original.join(e.original),this.modified.join(e.modified))}toRangeMapping(){const e=this.original.toInclusiveRange(),t=this.modified.toInclusiveRange();if(e&&t)return new Eo(e,t);if(this.original.startLineNumber===1||this.modified.startLineNumber===1){if(!(this.modified.startLineNumber===1&&this.original.startLineNumber===1))throw new bt("not a valid diff");return new Eo(new x(this.original.startLineNumber,1,this.original.endLineNumberExclusive,1),new x(this.modified.startLineNumber,1,this.modified.endLineNumberExclusive,1))}else return new Eo(new x(this.original.startLineNumber-1,Number.MAX_SAFE_INTEGER,this.original.endLineNumberExclusive-1,Number.MAX_SAFE_INTEGER),new x(this.modified.startLineNumber-1,Number.MAX_SAFE_INTEGER,this.modified.endLineNumberExclusive-1,Number.MAX_SAFE_INTEGER))}toRangeMapping2(e,t){if(V7(this.original.endLineNumberExclusive,e)&&V7(this.modified.endLineNumberExclusive,t))return new Eo(new x(this.original.startLineNumber,1,this.original.endLineNumberExclusive,1),new x(this.modified.startLineNumber,1,this.modified.endLineNumberExclusive,1));if(!this.original.isEmpty&&!this.modified.isEmpty)return new Eo(x.fromPositions(new A(this.original.startLineNumber,1),km(new A(this.original.endLineNumberExclusive-1,Number.MAX_SAFE_INTEGER),e)),x.fromPositions(new A(this.modified.startLineNumber,1),km(new A(this.modified.endLineNumberExclusive-1,Number.MAX_SAFE_INTEGER),t)));if(this.original.startLineNumber>1&&this.modified.startLineNumber>1)return new Eo(x.fromPositions(km(new A(this.original.startLineNumber-1,Number.MAX_SAFE_INTEGER),e),km(new A(this.original.endLineNumberExclusive-1,Number.MAX_SAFE_INTEGER),e)),x.fromPositions(km(new A(this.modified.startLineNumber-1,Number.MAX_SAFE_INTEGER),t),km(new A(this.modified.endLineNumberExclusive-1,Number.MAX_SAFE_INTEGER),t)));throw new bt}}function km(o,e){if(o.lineNumber<1)return new A(1,1);if(o.lineNumber>e.length)return new A(e.length,e[e.length-1].length+1);const t=e[o.lineNumber-1];return o.column>t.length+1?new A(o.lineNumber,t.length+1):o}function V7(o,e){return o>=1&&o<=e.length}class Vo extends Yn{static fromRangeMappings(e){const t=Be.join(e.map(n=>Be.fromRangeInclusive(n.originalRange))),i=Be.join(e.map(n=>Be.fromRangeInclusive(n.modifiedRange)));return new Vo(t,i,e)}constructor(e,t,i){super(e,t),this.innerChanges=i}flip(){var e;return new Vo(this.modified,this.original,(e=this.innerChanges)==null?void 0:e.map(t=>t.flip()))}withInnerChangesFromLineRanges(){return new Vo(this.original,this.modified,[this.toRangeMapping()])}}class Eo{static assertSorted(e){for(let t=1;t<e.length;t++){const i=e[t-1],n=e[t];if(!(i.originalRange.getEndPosition().isBeforeOrEqual(n.originalRange.getStartPosition())&&i.modifiedRange.getEndPosition().isBeforeOrEqual(n.modifiedRange.getStartPosition())))throw new bt("Range mappings must be sorted")}}constructor(e,t){this.originalRange=e,this.modifiedRange=t}toString(){return`{${this.originalRange.toString()}->${this.modifiedRange.toString()}}`}flip(){return new Eo(this.modifiedRange,this.originalRange)}toTextEdit(e){const t=e.getValueOfRange(this.modifiedRange);return new Fl(this.originalRange,t)}}const ase=3;class lse{computeDiff(e,t,i){var l;const s=new hse(e,t,{maxComputationTime:i.maxComputationTimeMs,shouldIgnoreTrimWhitespace:i.ignoreTrimWhitespace,shouldComputeCharChanges:!0,shouldMakePrettyDiff:!0,shouldPostProcessCharChanges:!0}).computeDiff(),r=[];let a=null;for(const c of s.changes){let d;c.originalEndLineNumber===0?d=new Be(c.originalStartLineNumber+1,c.originalStartLineNumber+1):d=new Be(c.originalStartLineNumber,c.originalEndLineNumber+1);let h;c.modifiedEndLineNumber===0?h=new Be(c.modifiedStartLineNumber+1,c.modifiedStartLineNumber+1):h=new Be(c.modifiedStartLineNumber,c.modifiedEndLineNumber+1);let u=new Vo(d,h,(l=c.charChanges)==null?void 0:l.map(g=>new Eo(new x(g.originalStartLineNumber,g.originalStartColumn,g.originalEndLineNumber,g.originalEndColumn),new x(g.modifiedStartLineNumber,g.modifiedStartColumn,g.modifiedEndLineNumber,g.modifiedEndColumn))));a&&(a.modified.endLineNumberExclusive===u.modified.startLineNumber||a.original.endLineNumberExclusive===u.original.startLineNumber)&&(u=new Vo(a.original.join(u.original),a.modified.join(u.modified),a.innerChanges&&u.innerChanges?a.innerChanges.concat(u.innerChanges):void 0),r.pop()),r.push(u),a=u}return Nf(()=>u5(r,(c,d)=>d.original.startLineNumber-c.original.endLineNumberExclusive===d.modified.startLineNumber-c.modified.endLineNumberExclusive&&c.original.endLineNumberExclusive<d.original.startLineNumber&&c.modified.endLineNumberExclusive<d.modified.startLineNumber)),new j0(r,[],s.quitEarly)}}function r$(o,e,t,i){return new yl(o,e,t).ComputeDiff(i)}let z7=class{constructor(e){const t=[],i=[];for(let n=0,s=e.length;n<s;n++)t[n]=$M(e[n],1),i[n]=jM(e[n],1);this.lines=e,this._startColumns=t,this._endColumns=i}getElements(){const e=[];for(let t=0,i=this.lines.length;t<i;t++)e[t]=this.lines[t].substring(this._startColumns[t]-1,this._endColumns[t]-1);return e}getStrictElement(e){return this.lines[e]}getStartLineNumber(e){return e+1}getEndLineNumber(e){return e+1}createCharSequence(e,t,i){const n=[],s=[],r=[];let a=0;for(let l=t;l<=i;l++){const c=this.lines[l],d=e?this._startColumns[l]:1,h=e?this._endColumns[l]:c.length+1;for(let u=d;u<h;u++)n[a]=c.charCodeAt(u-1),s[a]=l+1,r[a]=u,a++;!e&&l<i&&(n[a]=10,s[a]=l+1,r[a]=c.length+1,a++)}return new cse(n,s,r)}};class cse{constructor(e,t,i){this._charCodes=e,this._lineNumbers=t,this._columns=i}toString(){return"["+this._charCodes.map((e,t)=>(e===10?"\\n":String.fromCharCode(e))+`-(${this._lineNumbers[t]},${this._columns[t]})`).join(", ")+"]"}_assertIndex(e,t){if(e<0||e>=t.length)throw new Error("Illegal index")}getElements(){return this._charCodes}getStartLineNumber(e){return e>0&&e===this._lineNumbers.length?this.getEndLineNumber(e-1):(this._assertIndex(e,this._lineNumbers),this._lineNumbers[e])}getEndLineNumber(e){return e===-1?this.getStartLineNumber(e+1):(this._assertIndex(e,this._lineNumbers),this._charCodes[e]===10?this._lineNumbers[e]+1:this._lineNumbers[e])}getStartColumn(e){return e>0&&e===this._columns.length?this.getEndColumn(e-1):(this._assertIndex(e,this._columns),this._columns[e])}getEndColumn(e){return e===-1?this.getStartColumn(e+1):(this._assertIndex(e,this._columns),this._charCodes[e]===10?1:this._columns[e]+1)}}class Kp{constructor(e,t,i,n,s,r,a,l){this.originalStartLineNumber=e,this.originalStartColumn=t,this.originalEndLineNumber=i,this.originalEndColumn=n,this.modifiedStartLineNumber=s,this.modifiedStartColumn=r,this.modifiedEndLineNumber=a,this.modifiedEndColumn=l}static createFromDiffChange(e,t,i){const n=t.getStartLineNumber(e.originalStart),s=t.getStartColumn(e.originalStart),r=t.getEndLineNumber(e.originalStart+e.originalLength-1),a=t.getEndColumn(e.originalStart+e.originalLength-1),l=i.getStartLineNumber(e.modifiedStart),c=i.getStartColumn(e.modifiedStart),d=i.getEndLineNumber(e.modifiedStart+e.modifiedLength-1),h=i.getEndColumn(e.modifiedStart+e.modifiedLength-1);return new Kp(n,s,r,a,l,c,d,h)}}function dse(o){if(o.length<=1)return o;const e=[o[0]];let t=e[0];for(let i=1,n=o.length;i<n;i++){const s=o[i],r=s.originalStart-(t.originalStart+t.originalLength),a=s.modifiedStart-(t.modifiedStart+t.modifiedLength);Math.min(r,a)<ase?(t.originalLength=s.originalStart+s.originalLength-t.originalStart,t.modifiedLength=s.modifiedStart+s.modifiedLength-t.modifiedStart):(e.push(s),t=s)}return e}class GC{constructor(e,t,i,n,s){this.originalStartLineNumber=e,this.originalEndLineNumber=t,this.modifiedStartLineNumber=i,this.modifiedEndLineNumber=n,this.charChanges=s}static createFromDiffResult(e,t,i,n,s,r,a){let l,c,d,h,u;if(t.originalLength===0?(l=i.getStartLineNumber(t.originalStart)-1,c=0):(l=i.getStartLineNumber(t.originalStart),c=i.getEndLineNumber(t.originalStart+t.originalLength-1)),t.modifiedLength===0?(d=n.getStartLineNumber(t.modifiedStart)-1,h=0):(d=n.getStartLineNumber(t.modifiedStart),h=n.getEndLineNumber(t.modifiedStart+t.modifiedLength-1)),r&&t.originalLength>0&&t.originalLength<20&&t.modifiedLength>0&&t.modifiedLength<20&&s()){const g=i.createCharSequence(e,t.originalStart,t.originalStart+t.originalLength-1),f=n.createCharSequence(e,t.modifiedStart,t.modifiedStart+t.modifiedLength-1);if(g.getElements().length>0&&f.getElements().length>0){let p=r$(g,f,s,!0).changes;a&&(p=dse(p)),u=[];for(let _=0,b=p.length;_<b;_++)u.push(Kp.createFromDiffChange(p[_],g,f))}}return new GC(l,c,d,h,u)}}class hse{constructor(e,t,i){this.shouldComputeCharChanges=i.shouldComputeCharChanges,this.shouldPostProcessCharChanges=i.shouldPostProcessCharChanges,this.shouldIgnoreTrimWhitespace=i.shouldIgnoreTrimWhitespace,this.shouldMakePrettyDiff=i.shouldMakePrettyDiff,this.originalLines=e,this.modifiedLines=t,this.original=new z7(e),this.modified=new z7(t),this.continueLineDiff=U7(i.maxComputationTime),this.continueCharDiff=U7(i.maxComputationTime===0?0:Math.min(i.maxComputationTime,5e3))}computeDiff(){if(this.original.lines.length===1&&this.original.lines[0].length===0)return this.modified.lines.length===1&&this.modified.lines[0].length===0?{quitEarly:!1,changes:[]}:{quitEarly:!1,changes:[{originalStartLineNumber:1,originalEndLineNumber:1,modifiedStartLineNumber:1,modifiedEndLineNumber:this.modified.lines.length,charChanges:void 0}]};if(this.modified.lines.length===1&&this.modified.lines[0].length===0)return{quitEarly:!1,changes:[{originalStartLineNumber:1,originalEndLineNumber:this.original.lines.length,modifiedStartLineNumber:1,modifiedEndLineNumber:1,charChanges:void 0}]};const e=r$(this.original,this.modified,this.continueLineDiff,this.shouldMakePrettyDiff),t=e.changes,i=e.quitEarly;if(this.shouldIgnoreTrimWhitespace){const a=[];for(let l=0,c=t.length;l<c;l++)a.push(GC.createFromDiffResult(this.shouldIgnoreTrimWhitespace,t[l],this.original,this.modified,this.continueCharDiff,this.shouldComputeCharChanges,this.shouldPostProcessCharChanges));return{quitEarly:i,changes:a}}const n=[];let s=0,r=0;for(let a=-1,l=t.length;a<l;a++){const c=a+1<l?t[a+1]:null,d=c?c.originalStart:this.originalLines.length,h=c?c.modifiedStart:this.modifiedLines.length;for(;s<d&&r<h;){const u=this.originalLines[s],g=this.modifiedLines[r];if(u!==g){{let f=$M(u,1),p=$M(g,1);for(;f>1&&p>1;){const _=u.charCodeAt(f-2),b=g.charCodeAt(p-2);if(_!==b)break;f--,p--}(f>1||p>1)&&this._pushTrimWhitespaceCharChange(n,s+1,1,f,r+1,1,p)}{let f=jM(u,1),p=jM(g,1);const _=u.length+1,b=g.length+1;for(;f<_&&p<b;){const C=u.charCodeAt(f-1),v=u.charCodeAt(p-1);if(C!==v)break;f++,p++}(f<_||p<b)&&this._pushTrimWhitespaceCharChange(n,s+1,f,_,r+1,p,b)}}s++,r++}c&&(n.push(GC.createFromDiffResult(this.shouldIgnoreTrimWhitespace,c,this.original,this.modified,this.continueCharDiff,this.shouldComputeCharChanges,this.shouldPostProcessCharChanges)),s+=c.originalLength,r+=c.modifiedLength)}return{quitEarly:i,changes:n}}_pushTrimWhitespaceCharChange(e,t,i,n,s,r,a){if(this._mergeTrimWhitespaceCharChange(e,t,i,n,s,r,a))return;let l;this.shouldComputeCharChanges&&(l=[new Kp(t,i,t,n,s,r,s,a)]),e.push(new GC(t,t,s,s,l))}_mergeTrimWhitespaceCharChange(e,t,i,n,s,r,a){const l=e.length;if(l===0)return!1;const c=e[l-1];return c.originalEndLineNumber===0||c.modifiedEndLineNumber===0?!1:c.originalEndLineNumber===t&&c.modifiedEndLineNumber===s?(this.shouldComputeCharChanges&&c.charChanges&&c.charChanges.push(new Kp(t,i,t,n,s,r,s,a)),!0):c.originalEndLineNumber+1===t&&c.modifiedEndLineNumber+1===s?(c.originalEndLineNumber=t,c.modifiedEndLineNumber=s,this.shouldComputeCharChanges&&c.charChanges&&c.charChanges.push(new Kp(t,i,t,n,s,r,s,a)),!0):!1}}function $M(o,e){const t=Jn(o);return t===-1?e:t+1}function jM(o,e){const t=Na(o);return t===-1?e:t+2}function U7(o){if(o===0)return()=>!0;const e=Date.now();return()=>Date.now()-e<o}class Kc{static trivial(e,t){return new Kc([new qi(Ge.ofLength(e.length),Ge.ofLength(t.length))],!1)}static trivialTimedOut(e,t){return new Kc([new qi(Ge.ofLength(e.length),Ge.ofLength(t.length))],!0)}constructor(e,t){this.diffs=e,this.hitTimeout=t}}class qi{static invert(e,t){const i=[];return bz(e,(n,s)=>{i.push(qi.fromOffsetPairs(n?n.getEndExclusives():Mc.zero,s?s.getStarts():new Mc(t,(n?n.seq2Range.endExclusive-n.seq1Range.endExclusive:0)+t)))}),i}static fromOffsetPairs(e,t){return new qi(new Ge(e.offset1,t.offset1),new Ge(e.offset2,t.offset2))}static assertSorted(e){let t;for(const i of e){if(t&&!(t.seq1Range.endExclusive<=i.seq1Range.start&&t.seq2Range.endExclusive<=i.seq2Range.start))throw new bt("Sequence diffs must be sorted");t=i}}constructor(e,t){this.seq1Range=e,this.seq2Range=t}swap(){return new qi(this.seq2Range,this.seq1Range)}toString(){return`${this.seq1Range} <-> ${this.seq2Range}`}join(e){return new qi(this.seq1Range.join(e.seq1Range),this.seq2Range.join(e.seq2Range))}delta(e){return e===0?this:new qi(this.seq1Range.delta(e),this.seq2Range.delta(e))}deltaStart(e){return e===0?this:new qi(this.seq1Range.deltaStart(e),this.seq2Range.deltaStart(e))}deltaEnd(e){return e===0?this:new qi(this.seq1Range.deltaEnd(e),this.seq2Range.deltaEnd(e))}intersect(e){const t=this.seq1Range.intersect(e.seq1Range),i=this.seq2Range.intersect(e.seq2Range);if(!(!t||!i))return new qi(t,i)}getStarts(){return new Mc(this.seq1Range.start,this.seq2Range.start)}getEndExclusives(){return new Mc(this.seq1Range.endExclusive,this.seq2Range.endExclusive)}}const ig=class ig{constructor(e,t){this.offset1=e,this.offset2=t}toString(){return`${this.offset1} <-> ${this.offset2}`}delta(e){return e===0?this:new ig(this.offset1+e,this.offset2+e)}equals(e){return this.offset1===e.offset1&&this.offset2===e.offset2}};ig.zero=new ig(0,0),ig.max=new ig(Number.MAX_SAFE_INTEGER,Number.MAX_SAFE_INTEGER);let Mc=ig;const Wk=class Wk{isValid(){return!0}};Wk.instance=new Wk;let gw=Wk;class use{constructor(e){if(this.timeout=e,this.startTime=Date.now(),this.valid=!0,e<=0)throw new bt("timeout must be positive")}isValid(){if(!(Date.now()-this.startTime<this.timeout)&&this.valid){this.valid=!1;debugger}return this.valid}}class TN{constructor(e,t){this.width=e,this.height=t,this.array=[],this.array=new Array(e*t)}get(e,t){return this.array[e+t*this.width]}set(e,t,i){this.array[e+t*this.width]=i}}function KM(o){return o===32||o===9}const Av=class Av{static getKey(e){let t=this.chrKeys.get(e);return t===void 0&&(t=this.chrKeys.size,this.chrKeys.set(e,t)),t}constructor(e,t,i){this.range=e,this.lines=t,this.source=i,this.histogram=[];let n=0;for(let s=e.startLineNumber-1;s<e.endLineNumberExclusive-1;s++){const r=t[s];for(let l=0;l<r.length;l++){n++;const c=r[l],d=Av.getKey(c);this.histogram[d]=(this.histogram[d]||0)+1}n++;const a=Av.getKey(`
`);this.histogram[a]=(this.histogram[a]||0)+1}this.totalCount=n}computeSimilarity(e){let t=0;const i=Math.max(this.histogram.length,e.histogram.length);for(let n=0;n<i;n++)t+=Math.abs((this.histogram[n]??0)-(e.histogram[n]??0));return 1-t/(this.totalCount+e.totalCount)}};Av.chrKeys=new Map;let tL=Av;class gse{compute(e,t,i=gw.instance,n){if(e.length===0||t.length===0)return Kc.trivial(e,t);const s=new TN(e.length,t.length),r=new TN(e.length,t.length),a=new TN(e.length,t.length);for(let f=0;f<e.length;f++)for(let p=0;p<t.length;p++){if(!i.isValid())return Kc.trivialTimedOut(e,t);const _=f===0?0:s.get(f-1,p),b=p===0?0:s.get(f,p-1);let C;e.getElement(f)===t.getElement(p)?(f===0||p===0?C=0:C=s.get(f-1,p-1),f>0&&p>0&&r.get(f-1,p-1)===3&&(C+=a.get(f-1,p-1)),C+=n?n(f,p):1):C=-1;const v=Math.max(_,b,C);if(v===C){const w=f>0&&p>0?a.get(f-1,p-1):0;a.set(f,p,w+1),r.set(f,p,3)}else v===_?(a.set(f,p,0),r.set(f,p,1)):v===b&&(a.set(f,p,0),r.set(f,p,2));s.set(f,p,v)}const l=[];let c=e.length,d=t.length;function h(f,p){(f+1!==c||p+1!==d)&&l.push(new qi(new Ge(f+1,c),new Ge(p+1,d))),c=f,d=p}let u=e.length-1,g=t.length-1;for(;u>=0&&g>=0;)r.get(u,g)===3?(h(u,g),u--,g--):r.get(u,g)===1?u--:g--;return h(-1,-1),l.reverse(),new Kc(l,!1)}}class a${compute(e,t,i=gw.instance){if(e.length===0||t.length===0)return Kc.trivial(e,t);const n=e,s=t;function r(p,_){for(;p<n.length&&_<s.length&&n.getElement(p)===s.getElement(_);)p++,_++;return p}let a=0;const l=new fse;l.set(0,r(0,0));const c=new mse;c.set(0,l.get(0)===0?null:new $7(null,0,0,l.get(0)));let d=0;e:for(;;){if(a++,!i.isValid())return Kc.trivialTimedOut(n,s);const p=-Math.min(a,s.length+a%2),_=Math.min(a,n.length+a%2);for(d=p;d<=_;d+=2){const b=d===_?-1:l.get(d+1),C=d===p?-1:l.get(d-1)+1,v=Math.min(Math.max(b,C),n.length),w=v-d;if(v>n.length||w>s.length)continue;const S=r(v,w);l.set(d,S);const L=v===b?c.get(d+1):c.get(d-1);if(c.set(d,S!==v?new $7(L,v,w,S-v):L),l.get(d)===n.length&&l.get(d)-d===s.length)break e}}let h=c.get(d);const u=[];let g=n.length,f=s.length;for(;;){const p=h?h.x+h.length:0,_=h?h.y+h.length:0;if((p!==g||_!==f)&&u.push(new qi(new Ge(p,g),new Ge(_,f))),!h)break;g=h.x,f=h.y,h=h.prev}return u.reverse(),new Kc(u,!1)}}class $7{constructor(e,t,i,n){this.prev=e,this.x=t,this.y=i,this.length=n}}class fse{constructor(){this.positiveArr=new Int32Array(10),this.negativeArr=new Int32Array(10)}get(e){return e<0?(e=-e-1,this.negativeArr[e]):this.positiveArr[e]}set(e,t){if(e<0){if(e=-e-1,e>=this.negativeArr.length){const i=this.negativeArr;this.negativeArr=new Int32Array(i.length*2),this.negativeArr.set(i)}this.negativeArr[e]=t}else{if(e>=this.positiveArr.length){const i=this.positiveArr;this.positiveArr=new Int32Array(i.length*2),this.positiveArr.set(i)}this.positiveArr[e]=t}}}class mse{constructor(){this.positiveArr=[],this.negativeArr=[]}get(e){return e<0?(e=-e-1,this.negativeArr[e]):this.positiveArr[e]}set(e,t){e<0?(e=-e-1,this.negativeArr[e]=t):this.positiveArr[e]=t}}class iL{constructor(e,t,i){this.lines=e,this.range=t,this.considerWhitespaceChanges=i,this.elements=[],this.firstElementOffsetByLineIdx=[],this.lineStartOffsets=[],this.trimmedWsLengthsByLineIdx=[],this.firstElementOffsetByLineIdx.push(0);for(let n=this.range.startLineNumber;n<=this.range.endLineNumber;n++){let s=e[n-1],r=0;n===this.range.startLineNumber&&this.range.startColumn>1&&(r=this.range.startColumn-1,s=s.substring(r)),this.lineStartOffsets.push(r);let a=0;if(!i){const c=s.trimStart();a=s.length-c.length,s=c.trimEnd()}this.trimmedWsLengthsByLineIdx.push(a);const l=n===this.range.endLineNumber?Math.min(this.range.endColumn-1-r-a,s.length):s.length;for(let c=0;c<l;c++)this.elements.push(s.charCodeAt(c));n<this.range.endLineNumber&&(this.elements.push(10),this.firstElementOffsetByLineIdx.push(this.elements.length))}}toString(){return`Slice: "${this.text}"`}get text(){return this.getText(new Ge(0,this.length))}getText(e){return this.elements.slice(e.start,e.endExclusive).map(t=>String.fromCharCode(t)).join("")}getElement(e){return this.elements[e]}get length(){return this.elements.length}getBoundaryScore(e){const t=K7(e>0?this.elements[e-1]:-1),i=K7(e<this.elements.length?this.elements[e]:-1);if(t===7&&i===8)return 0;if(t===8)return 150;let n=0;return t!==i&&(n+=10,t===0&&i===1&&(n+=1)),n+=j7(t),n+=j7(i),n}translateOffset(e,t="right"){const i=hw(this.firstElementOffsetByLineIdx,s=>s<=e),n=e-this.firstElementOffsetByLineIdx[i];return new A(this.range.startLineNumber+i,1+this.lineStartOffsets[i]+n+(n===0&&t==="left"?0:this.trimmedWsLengthsByLineIdx[i]))}translateRange(e){const t=this.translateOffset(e.start,"right"),i=this.translateOffset(e.endExclusive,"left");return i.isBefore(t)?x.fromPositions(i,i):x.fromPositions(t,i)}findWordContaining(e){if(e<0||e>=this.elements.length||!RN(this.elements[e]))return;let t=e;for(;t>0&&RN(this.elements[t-1]);)t--;let i=e;for(;i<this.elements.length&&RN(this.elements[i]);)i++;return new Ge(t,i)}countLinesIn(e){return this.translateOffset(e.endExclusive).lineNumber-this.translateOffset(e.start).lineNumber}isStronglyEqual(e,t){return this.elements[e]===this.elements[t]}extendToFullLines(e){const t=E_(this.firstElementOffsetByLineIdx,n=>n<=e.start)??0,i=ese(this.firstElementOffsetByLineIdx,n=>e.endExclusive<=n)??this.elements.length;return new Ge(t,i)}}function RN(o){return o>=97&&o<=122||o>=65&&o<=90||o>=48&&o<=57}const pse={0:0,1:0,2:0,3:10,4:2,5:30,6:3,7:10,8:10};function j7(o){return pse[o]}function K7(o){return o===10?8:o===13?7:KM(o)?6:o>=97&&o<=122?0:o>=65&&o<=90?1:o>=48&&o<=57?2:o===-1?3:o===44||o===59?5:4}function _se(o,e,t,i,n,s){let{moves:r,excludedChanges:a}=Cse(o,e,t,s);if(!s.isValid())return[];const l=o.filter(d=>!a.has(d)),c=vse(l,i,n,e,t,s);return hR(r,c),r=wse(r),r=r.filter(d=>{const h=d.original.toOffsetRange().slice(e).map(g=>g.trim());return h.join(`
`).length>=15&&bse(h,g=>g.length>=2)>=2}),r=Sse(o,r),r}function bse(o,e){let t=0;for(const i of o)e(i)&&t++;return t}function Cse(o,e,t,i){const n=[],s=o.filter(l=>l.modified.isEmpty&&l.original.length>=3).map(l=>new tL(l.original,e,l)),r=new Set(o.filter(l=>l.original.isEmpty&&l.modified.length>=3).map(l=>new tL(l.modified,t,l))),a=new Set;for(const l of s){let c=-1,d;for(const h of r){const u=l.computeSimilarity(h);u>c&&(c=u,d=h)}if(c>.9&&d&&(r.delete(d),n.push(new Yn(l.range,d.range)),a.add(l.source),a.add(d.source)),!i.isValid())return{moves:n,excludedChanges:a}}return{moves:n,excludedChanges:a}}function vse(o,e,t,i,n,s){const r=[],a=new m5;for(const u of o)for(let g=u.original.startLineNumber;g<u.original.endLineNumberExclusive-2;g++){const f=`${e[g-1]}:${e[g+1-1]}:${e[g+2-1]}`;a.add(f,{range:new Be(g,g+3)})}const l=[];o.sort(gs(u=>u.modified.startLineNumber,$r));for(const u of o){let g=[];for(let f=u.modified.startLineNumber;f<u.modified.endLineNumberExclusive-2;f++){const p=`${t[f-1]}:${t[f+1-1]}:${t[f+2-1]}`,_=new Be(f,f+3),b=[];a.forEach(p,({range:C})=>{for(const w of g)if(w.originalLineRange.endLineNumberExclusive+1===C.endLineNumberExclusive&&w.modifiedLineRange.endLineNumberExclusive+1===_.endLineNumberExclusive){w.originalLineRange=new Be(w.originalLineRange.startLineNumber,C.endLineNumberExclusive),w.modifiedLineRange=new Be(w.modifiedLineRange.startLineNumber,_.endLineNumberExclusive),b.push(w);return}const v={modifiedLineRange:_,originalLineRange:C};l.push(v),b.push(v)}),g=b}if(!s.isValid())return[]}l.sort(wz(gs(u=>u.modifiedLineRange.length,$r)));const c=new dr,d=new dr;for(const u of l){const g=u.modifiedLineRange.startLineNumber-u.originalLineRange.startLineNumber,f=c.subtractFrom(u.modifiedLineRange),p=d.subtractFrom(u.originalLineRange).getWithDelta(g),_=f.getIntersection(p);for(const b of _.ranges){if(b.length<3)continue;const C=b,v=b.delta(-g);r.push(new Yn(v,C)),c.addRange(C),d.addRange(v)}}r.sort(gs(u=>u.original.startLineNumber,$r));const h=new eL(o);for(let u=0;u<r.length;u++){const g=r[u],f=h.findLastMonotonous(L=>L.original.startLineNumber<=g.original.startLineNumber),p=E_(o,L=>L.modified.startLineNumber<=g.modified.startLineNumber),_=Math.max(g.original.startLineNumber-f.original.startLineNumber,g.modified.startLineNumber-p.modified.startLineNumber),b=h.findLastMonotonous(L=>L.original.startLineNumber<g.original.endLineNumberExclusive),C=E_(o,L=>L.modified.startLineNumber<g.modified.endLineNumberExclusive),v=Math.max(b.original.endLineNumberExclusive-g.original.endLineNumberExclusive,C.modified.endLineNumberExclusive-g.modified.endLineNumberExclusive);let w;for(w=0;w<_;w++){const L=g.original.startLineNumber-w-1,k=g.modified.startLineNumber-w-1;if(L>i.length||k>n.length||c.contains(k)||d.contains(L)||!q7(i[L-1],n[k-1],s))break}w>0&&(d.addRange(new Be(g.original.startLineNumber-w,g.original.startLineNumber)),c.addRange(new Be(g.modified.startLineNumber-w,g.modified.startLineNumber)));let S;for(S=0;S<v;S++){const L=g.original.endLineNumberExclusive+S,k=g.modified.endLineNumberExclusive+S;if(L>i.length||k>n.length||c.contains(k)||d.contains(L)||!q7(i[L-1],n[k-1],s))break}S>0&&(d.addRange(new Be(g.original.endLineNumberExclusive,g.original.endLineNumberExclusive+S)),c.addRange(new Be(g.modified.endLineNumberExclusive,g.modified.endLineNumberExclusive+S))),(w>0||S>0)&&(r[u]=new Yn(new Be(g.original.startLineNumber-w,g.original.endLineNumberExclusive+S),new Be(g.modified.startLineNumber-w,g.modified.endLineNumberExclusive+S)))}return r}function q7(o,e,t){if(o.trim()===e.trim())return!0;if(o.length>300&&e.length>300)return!1;const n=new a$().compute(new iL([o],new x(1,1,1,o.length),!1),new iL([e],new x(1,1,1,e.length),!1),t);let s=0;const r=qi.invert(n.diffs,o.length);for(const d of r)d.seq1Range.forEach(h=>{KM(o.charCodeAt(h))||s++});function a(d){let h=0;for(let u=0;u<o.length;u++)KM(d.charCodeAt(u))||h++;return h}const l=a(o.length>e.length?o:e);return s/l>.6&&l>10}function wse(o){if(o.length===0)return o;o.sort(gs(t=>t.original.startLineNumber,$r));const e=[o[0]];for(let t=1;t<o.length;t++){const i=e[e.length-1],n=o[t],s=n.original.startLineNumber-i.original.endLineNumberExclusive,r=n.modified.startLineNumber-i.modified.endLineNumberExclusive;if(s>=0&&r>=0&&s+r<=2){e[e.length-1]=i.join(n);continue}e.push(n)}return e}function Sse(o,e){const t=new eL(o);return e=e.filter(i=>{const n=t.findLastMonotonous(a=>a.original.startLineNumber<i.original.endLineNumberExclusive)||new Yn(new Be(1,1),new Be(1,1)),s=E_(o,a=>a.modified.startLineNumber<i.modified.endLineNumberExclusive);return n!==s}),e}function qM(o,e,t){let i=t;return i=G7(o,e,i),i=G7(o,e,i),i=yse(o,e,i),i}function G7(o,e,t){if(t.length===0)return t;const i=[];i.push(t[0]);for(let s=1;s<t.length;s++){const r=i[i.length-1];let a=t[s];if(a.seq1Range.isEmpty||a.seq2Range.isEmpty){const l=a.seq1Range.start-r.seq1Range.endExclusive;let c;for(c=1;c<=l&&!(o.getElement(a.seq1Range.start-c)!==o.getElement(a.seq1Range.endExclusive-c)||e.getElement(a.seq2Range.start-c)!==e.getElement(a.seq2Range.endExclusive-c));c++);if(c--,c===l){i[i.length-1]=new qi(new Ge(r.seq1Range.start,a.seq1Range.endExclusive-l),new Ge(r.seq2Range.start,a.seq2Range.endExclusive-l));continue}a=a.delta(-c)}i.push(a)}const n=[];for(let s=0;s<i.length-1;s++){const r=i[s+1];let a=i[s];if(a.seq1Range.isEmpty||a.seq2Range.isEmpty){const l=r.seq1Range.start-a.seq1Range.endExclusive;let c;for(c=0;c<l&&!(!o.isStronglyEqual(a.seq1Range.start+c,a.seq1Range.endExclusive+c)||!e.isStronglyEqual(a.seq2Range.start+c,a.seq2Range.endExclusive+c));c++);if(c===l){i[s+1]=new qi(new Ge(a.seq1Range.start+l,r.seq1Range.endExclusive),new Ge(a.seq2Range.start+l,r.seq2Range.endExclusive));continue}c>0&&(a=a.delta(c))}n.push(a)}return i.length>0&&n.push(i[i.length-1]),n}function yse(o,e,t){if(!o.getBoundaryScore||!e.getBoundaryScore)return t;for(let i=0;i<t.length;i++){const n=i>0?t[i-1]:void 0,s=t[i],r=i+1<t.length?t[i+1]:void 0,a=new Ge(n?n.seq1Range.endExclusive+1:0,r?r.seq1Range.start-1:o.length),l=new Ge(n?n.seq2Range.endExclusive+1:0,r?r.seq2Range.start-1:e.length);s.seq1Range.isEmpty?t[i]=Z7(s,o,e,a,l):s.seq2Range.isEmpty&&(t[i]=Z7(s.swap(),e,o,l,a).swap())}return t}function Z7(o,e,t,i,n){let r=1;for(;o.seq1Range.start-r>=i.start&&o.seq2Range.start-r>=n.start&&t.isStronglyEqual(o.seq2Range.start-r,o.seq2Range.endExclusive-r)&&r<100;)r++;r--;let a=0;for(;o.seq1Range.start+a<i.endExclusive&&o.seq2Range.endExclusive+a<n.endExclusive&&t.isStronglyEqual(o.seq2Range.start+a,o.seq2Range.endExclusive+a)&&a<100;)a++;if(r===0&&a===0)return o;let l=0,c=-1;for(let d=-r;d<=a;d++){const h=o.seq2Range.start+d,u=o.seq2Range.endExclusive+d,g=o.seq1Range.start+d,f=e.getBoundaryScore(g)+t.getBoundaryScore(h)+t.getBoundaryScore(u);f>c&&(c=f,l=d)}return o.delta(l)}function Lse(o,e,t){const i=[];for(const n of t){const s=i[i.length-1];if(!s){i.push(n);continue}n.seq1Range.start-s.seq1Range.endExclusive<=2||n.seq2Range.start-s.seq2Range.endExclusive<=2?i[i.length-1]=new qi(s.seq1Range.join(n.seq1Range),s.seq2Range.join(n.seq2Range)):i.push(n)}return i}function xse(o,e,t){const i=qi.invert(t,o.length),n=[];let s=new Mc(0,0);function r(l,c){if(l.offset1<s.offset1||l.offset2<s.offset2)return;const d=o.findWordContaining(l.offset1),h=e.findWordContaining(l.offset2);if(!d||!h)return;let u=new qi(d,h);const g=u.intersect(c);let f=g.seq1Range.length,p=g.seq2Range.length;for(;i.length>0;){const _=i[0];if(!(_.seq1Range.intersects(u.seq1Range)||_.seq2Range.intersects(u.seq2Range)))break;const C=o.findWordContaining(_.seq1Range.start),v=e.findWordContaining(_.seq2Range.start),w=new qi(C,v),S=w.intersect(_);if(f+=S.seq1Range.length,p+=S.seq2Range.length,u=u.join(w),u.seq1Range.endExclusive>=_.seq1Range.endExclusive)i.shift();else break}f+p<(u.seq1Range.length+u.seq2Range.length)*2/3&&n.push(u),s=u.getEndExclusives()}for(;i.length>0;){const l=i.shift();l.seq1Range.isEmpty||(r(l.getStarts(),l),r(l.getEndExclusives().delta(-1),l))}return kse(t,n)}function kse(o,e){const t=[];for(;o.length>0||e.length>0;){const i=o[0],n=e[0];let s;i&&(!n||i.seq1Range.start<n.seq1Range.start)?s=o.shift():s=e.shift(),t.length>0&&t[t.length-1].seq1Range.endExclusive>=s.seq1Range.start?t[t.length-1]=t[t.length-1].join(s):t.push(s)}return t}function Dse(o,e,t){let i=t;if(i.length===0)return i;let n=0,s;do{s=!1;const r=[i[0]];for(let a=1;a<i.length;a++){let d=function(u,g){const f=new Ge(c.seq1Range.endExclusive,l.seq1Range.start);return o.getText(f).replace(/\s/g,"").length<=4&&(u.seq1Range.length+u.seq2Range.length>5||g.seq1Range.length+g.seq2Range.length>5)};const l=i[a],c=r[r.length-1];d(c,l)?(s=!0,r[r.length-1]=r[r.length-1].join(l)):r.push(l)}i=r}while(n++<10&&s);return i}function Ise(o,e,t){let i=t;if(i.length===0)return i;let n=0,s;do{s=!1;const a=[i[0]];for(let l=1;l<i.length;l++){let h=function(g,f){const p=new Ge(d.seq1Range.endExclusive,c.seq1Range.start);if(o.countLinesIn(p)>5||p.length>500)return!1;const b=o.getText(p).trim();if(b.length>20||b.split(/\r\n|\r|\n/).length>1)return!1;const C=o.countLinesIn(g.seq1Range),v=g.seq1Range.length,w=e.countLinesIn(g.seq2Range),S=g.seq2Range.length,L=o.countLinesIn(f.seq1Range),k=f.seq1Range.length,D=e.countLinesIn(f.seq2Range),N=f.seq2Range.length,B=2*40+50;function W(V){return Math.min(V,B)}return Math.pow(Math.pow(W(C*40+v),1.5)+Math.pow(W(w*40+S),1.5),1.5)+Math.pow(Math.pow(W(L*40+k),1.5)+Math.pow(W(D*40+N),1.5),1.5)>(B**1.5)**1.5*1.3};const c=i[l],d=a[a.length-1];h(d,c)?(s=!0,a[a.length-1]=a[a.length-1].join(c)):a.push(c)}i=a}while(n++<10&&s);const r=[];return vQ(i,(a,l,c)=>{let d=l;function h(b){return b.length>0&&b.trim().length<=3&&l.seq1Range.length+l.seq2Range.length>100}const u=o.extendToFullLines(l.seq1Range),g=o.getText(new Ge(u.start,l.seq1Range.start));h(g)&&(d=d.deltaStart(-g.length));const f=o.getText(new Ge(l.seq1Range.endExclusive,u.endExclusive));h(f)&&(d=d.deltaEnd(f.length));const p=qi.fromOffsetPairs(a?a.getEndExclusives():Mc.zero,c?c.getStarts():Mc.max),_=d.intersect(p);r.length>0&&_.getStarts().equals(r[r.length-1].getEndExclusives())?r[r.length-1]=r[r.length-1].join(_):r.push(_)}),r}class Y7{constructor(e,t){this.trimmedHash=e,this.lines=t}getElement(e){return this.trimmedHash[e]}get length(){return this.trimmedHash.length}getBoundaryScore(e){const t=e===0?0:X7(this.lines[e-1]),i=e===this.lines.length?0:X7(this.lines[e]);return 1e3-(t+i)}getText(e){return this.lines.slice(e.start,e.endExclusive).join(`
`)}isStronglyEqual(e,t){return this.lines[e]===this.lines[t]}}function X7(o){let e=0;for(;e<o.length&&(o.charCodeAt(e)===32||o.charCodeAt(e)===9);)e++;return e}class l${constructor(){this.dynamicProgrammingDiffing=new gse,this.myersDiffingAlgorithm=new a$}computeDiff(e,t,i){if(e.length<=1&&ii(e,t,(S,L)=>S===L))return new j0([],[],!1);if(e.length===1&&e[0].length===0||t.length===1&&t[0].length===0)return new j0([new Vo(new Be(1,e.length+1),new Be(1,t.length+1),[new Eo(new x(1,1,e.length,e[e.length-1].length+1),new x(1,1,t.length,t[t.length-1].length+1))])],[],!1);const n=i.maxComputationTimeMs===0?gw.instance:new use(i.maxComputationTimeMs),s=!i.ignoreTrimWhitespace,r=new Map;function a(S){let L=r.get(S);return L===void 0&&(L=r.size,r.set(S,L)),L}const l=e.map(S=>a(S.trim())),c=t.map(S=>a(S.trim())),d=new Y7(l,e),h=new Y7(c,t),u=d.length+h.length<1700?this.dynamicProgrammingDiffing.compute(d,h,n,(S,L)=>e[S]===t[L]?t[L].length===0?.1:1+Math.log(1+t[L].length):.99):this.myersDiffingAlgorithm.compute(d,h,n);let g=u.diffs,f=u.hitTimeout;g=qM(d,h,g),g=Dse(d,h,g);const p=[],_=S=>{if(s)for(let L=0;L<S;L++){const k=b+L,D=C+L;if(e[k]!==t[D]){const N=this.refineDiff(e,t,new qi(new Ge(k,k+1),new Ge(D,D+1)),n,s);for(const B of N.mappings)p.push(B);N.hitTimeout&&(f=!0)}}};let b=0,C=0;for(const S of g){Nf(()=>S.seq1Range.start-b===S.seq2Range.start-C);const L=S.seq1Range.start-b;_(L),b=S.seq1Range.endExclusive,C=S.seq2Range.endExclusive;const k=this.refineDiff(e,t,S,n,s);k.hitTimeout&&(f=!0);for(const D of k.mappings)p.push(D)}_(e.length-b);const v=Q7(p,e,t);let w=[];return i.computeMoves&&(w=this.computeMoves(v,e,t,l,c,n,s)),Nf(()=>{function S(k,D){if(k.lineNumber<1||k.lineNumber>D.length)return!1;const N=D[k.lineNumber-1];return!(k.column<1||k.column>N.length+1)}function L(k,D){return!(k.startLineNumber<1||k.startLineNumber>D.length+1||k.endLineNumberExclusive<1||k.endLineNumberExclusive>D.length+1)}for(const k of v){if(!k.innerChanges)return!1;for(const D of k.innerChanges)if(!(S(D.modifiedRange.getStartPosition(),t)&&S(D.modifiedRange.getEndPosition(),t)&&S(D.originalRange.getStartPosition(),e)&&S(D.originalRange.getEndPosition(),e)))return!1;if(!L(k.modified,t)||!L(k.original,e))return!1}return!0}),new j0(v,w,f)}computeMoves(e,t,i,n,s,r,a){return _se(e,t,i,n,s,r).map(d=>{const h=this.refineDiff(t,i,new qi(d.original.toOffsetRange(),d.modified.toOffsetRange()),r,a),u=Q7(h.mappings,t,i,!0);return new s$(d,u)})}refineDiff(e,t,i,n,s){const a=Nse(i).toRangeMapping2(e,t),l=new iL(e,a.originalRange,s),c=new iL(t,a.modifiedRange,s),d=l.length+c.length<500?this.dynamicProgrammingDiffing.compute(l,c,n):this.myersDiffingAlgorithm.compute(l,c,n);let h=d.diffs;return h=qM(l,c,h),h=xse(l,c,h),h=Lse(l,c,h),h=Ise(l,c,h),{mappings:h.map(g=>new Eo(l.translateRange(g.seq1Range),c.translateRange(g.seq2Range))),hitTimeout:d.hitTimeout}}}function Q7(o,e,t,i=!1){const n=[];for(const s of P4(o.map(r=>Ese(r,e,t)),(r,a)=>r.original.overlapOrTouch(a.original)||r.modified.overlapOrTouch(a.modified))){const r=s[0],a=s[s.length-1];n.push(new Vo(r.original.join(a.original),r.modified.join(a.modified),s.map(l=>l.innerChanges[0])))}return Nf(()=>!i&&n.length>0&&(n[0].modified.startLineNumber!==n[0].original.startLineNumber||t.length-n[n.length-1].modified.endLineNumberExclusive!==e.length-n[n.length-1].original.endLineNumberExclusive)?!1:u5(n,(s,r)=>r.original.startLineNumber-s.original.endLineNumberExclusive===r.modified.startLineNumber-s.modified.endLineNumberExclusive&&s.original.endLineNumberExclusive<r.original.startLineNumber&&s.modified.endLineNumberExclusive<r.modified.startLineNumber)),n}function Ese(o,e,t){let i=0,n=0;o.modifiedRange.endColumn===1&&o.originalRange.endColumn===1&&o.originalRange.startLineNumber+i<=o.originalRange.endLineNumber&&o.modifiedRange.startLineNumber+i<=o.modifiedRange.endLineNumber&&(n=-1),o.modifiedRange.startColumn-1>=t[o.modifiedRange.startLineNumber-1].length&&o.originalRange.startColumn-1>=e[o.originalRange.startLineNumber-1].length&&o.originalRange.startLineNumber<=o.originalRange.endLineNumber+n&&o.modifiedRange.startLineNumber<=o.modifiedRange.endLineNumber+n&&(i=1);const s=new Be(o.originalRange.startLineNumber+i,o.originalRange.endLineNumber+1+n),r=new Be(o.modifiedRange.startLineNumber+i,o.modifiedRange.endLineNumber+1+n);return new Vo(s,r,[o])}function Nse(o){return new Yn(new Be(o.seq1Range.start+1,o.seq1Range.endExclusive+1),new Be(o.seq2Range.start+1,o.seq2Range.endExclusive+1))}const J7={getLegacy:()=>new lse,getDefault:()=>new l$};function ch(o,e){const t=Math.pow(10,e);return Math.round(o*t)/t}class dt{constructor(e,t,i,n=1){this._rgbaBrand=void 0,this.r=Math.min(255,Math.max(0,e))|0,this.g=Math.min(255,Math.max(0,t))|0,this.b=Math.min(255,Math.max(0,i))|0,this.a=ch(Math.max(Math.min(1,n),0),3)}static equals(e,t){return e.r===t.r&&e.g===t.g&&e.b===t.b&&e.a===t.a}}class Fr{constructor(e,t,i,n){this._hslaBrand=void 0,this.h=Math.max(Math.min(360,e),0)|0,this.s=ch(Math.max(Math.min(1,t),0),3),this.l=ch(Math.max(Math.min(1,i),0),3),this.a=ch(Math.max(Math.min(1,n),0),3)}static equals(e,t){return e.h===t.h&&e.s===t.s&&e.l===t.l&&e.a===t.a}static fromRGBA(e){const t=e.r/255,i=e.g/255,n=e.b/255,s=e.a,r=Math.max(t,i,n),a=Math.min(t,i,n);let l=0,c=0;const d=(a+r)/2,h=r-a;if(h>0){switch(c=Math.min(d<=.5?h/(2*d):h/(2-2*d),1),r){case t:l=(i-n)/h+(i<n?6:0);break;case i:l=(n-t)/h+2;break;case n:l=(t-i)/h+4;break}l*=60,l=Math.round(l)}return new Fr(l,c,d,s)}static _hue2rgb(e,t,i){return i<0&&(i+=1),i>1&&(i-=1),i<1/6?e+(t-e)*6*i:i<1/2?t:i<2/3?e+(t-e)*(2/3-i)*6:e}static toRGBA(e){const t=e.h/360,{s:i,l:n,a:s}=e;let r,a,l;if(i===0)r=a=l=n;else{const c=n<.5?n*(1+i):n+i-n*i,d=2*n-c;r=Fr._hue2rgb(d,c,t+1/3),a=Fr._hue2rgb(d,c,t),l=Fr._hue2rgb(d,c,t-1/3)}return new dt(Math.round(r*255),Math.round(a*255),Math.round(l*255),s)}}class El{constructor(e,t,i,n){this._hsvaBrand=void 0,this.h=Math.max(Math.min(360,e),0)|0,this.s=ch(Math.max(Math.min(1,t),0),3),this.v=ch(Math.max(Math.min(1,i),0),3),this.a=ch(Math.max(Math.min(1,n),0),3)}static equals(e,t){return e.h===t.h&&e.s===t.s&&e.v===t.v&&e.a===t.a}static fromRGBA(e){const t=e.r/255,i=e.g/255,n=e.b/255,s=Math.max(t,i,n),r=Math.min(t,i,n),a=s-r,l=s===0?0:a/s;let c;return a===0?c=0:s===t?c=((i-n)/a%6+6)%6:s===i?c=(n-t)/a+2:c=(t-i)/a+4,new El(Math.round(c*60),l,s,e.a)}static toRGBA(e){const{h:t,s:i,v:n,a:s}=e,r=n*i,a=r*(1-Math.abs(t/60%2-1)),l=n-r;let[c,d,h]=[0,0,0];return t<60?(c=r,d=a):t<120?(c=a,d=r):t<180?(d=r,h=a):t<240?(d=a,h=r):t<300?(c=a,h=r):t<=360&&(c=r,h=a),c=Math.round((c+l)*255),d=Math.round((d+l)*255),h=Math.round((h+l)*255),new dt(c,d,h,s)}}const Ci=class Ci{static fromHex(e){return Ci.Format.CSS.parseHex(e)||Ci.red}static equals(e,t){return!e&&!t?!0:!e||!t?!1:e.equals(t)}get hsla(){return this._hsla?this._hsla:Fr.fromRGBA(this.rgba)}get hsva(){return this._hsva?this._hsva:El.fromRGBA(this.rgba)}constructor(e){if(e)if(e instanceof dt)this.rgba=e;else if(e instanceof Fr)this._hsla=e,this.rgba=Fr.toRGBA(e);else if(e instanceof El)this._hsva=e,this.rgba=El.toRGBA(e);else throw new Error("Invalid color ctor argument");else throw new Error("Color needs a value")}equals(e){return!!e&&dt.equals(this.rgba,e.rgba)&&Fr.equals(this.hsla,e.hsla)&&El.equals(this.hsva,e.hsva)}getRelativeLuminance(){const e=Ci._relativeLuminanceForComponent(this.rgba.r),t=Ci._relativeLuminanceForComponent(this.rgba.g),i=Ci._relativeLuminanceForComponent(this.rgba.b),n=.2126*e+.7152*t+.0722*i;return ch(n,4)}static _relativeLuminanceForComponent(e){const t=e/255;return t<=.03928?t/12.92:Math.pow((t+.055)/1.055,2.4)}isLighter(){return(this.rgba.r*299+this.rgba.g*587+this.rgba.b*114)/1e3>=128}isLighterThan(e){const t=this.getRelativeLuminance(),i=e.getRelativeLuminance();return t>i}isDarkerThan(e){const t=this.getRelativeLuminance(),i=e.getRelativeLuminance();return t<i}lighten(e){return new Ci(new Fr(this.hsla.h,this.hsla.s,this.hsla.l+this.hsla.l*e,this.hsla.a))}darken(e){return new Ci(new Fr(this.hsla.h,this.hsla.s,this.hsla.l-this.hsla.l*e,this.hsla.a))}transparent(e){const{r:t,g:i,b:n,a:s}=this.rgba;return new Ci(new dt(t,i,n,s*e))}isTransparent(){return this.rgba.a===0}isOpaque(){return this.rgba.a===1}opposite(){return new Ci(new dt(255-this.rgba.r,255-this.rgba.g,255-this.rgba.b,this.rgba.a))}makeOpaque(e){if(this.isOpaque()||e.rgba.a!==1)return this;const{r:t,g:i,b:n,a:s}=this.rgba;return new Ci(new dt(e.rgba.r-s*(e.rgba.r-t),e.rgba.g-s*(e.rgba.g-i),e.rgba.b-s*(e.rgba.b-n),1))}toString(){return this._toString||(this._toString=Ci.Format.CSS.format(this)),this._toString}static getLighterColor(e,t,i){if(e.isLighterThan(t))return e;i=i||.5;const n=e.getRelativeLuminance(),s=t.getRelativeLuminance();return i=i*(s-n)/s,e.lighten(i)}static getDarkerColor(e,t,i){if(e.isDarkerThan(t))return e;i=i||.5;const n=e.getRelativeLuminance(),s=t.getRelativeLuminance();return i=i*(n-s)/n,e.darken(i)}};Ci.white=new Ci(new dt(255,255,255,1)),Ci.black=new Ci(new dt(0,0,0,1)),Ci.red=new Ci(new dt(255,0,0,1)),Ci.blue=new Ci(new dt(0,0,255,1)),Ci.green=new Ci(new dt(0,255,0,1)),Ci.cyan=new Ci(new dt(0,255,255,1)),Ci.lightgrey=new Ci(new dt(211,211,211,1)),Ci.transparent=new Ci(new dt(0,0,0,0));let Z=Ci;(function(o){(function(e){(function(t){function i(g){return g.rgba.a===1?`rgb(${g.rgba.r}, ${g.rgba.g}, ${g.rgba.b})`:o.Format.CSS.formatRGBA(g)}t.formatRGB=i;function n(g){return`rgba(${g.rgba.r}, ${g.rgba.g}, ${g.rgba.b}, ${+g.rgba.a.toFixed(2)})`}t.formatRGBA=n;function s(g){return g.hsla.a===1?`hsl(${g.hsla.h}, ${(g.hsla.s*100).toFixed(2)}%, ${(g.hsla.l*100).toFixed(2)}%)`:o.Format.CSS.formatHSLA(g)}t.formatHSL=s;function r(g){return`hsla(${g.hsla.h}, ${(g.hsla.s*100).toFixed(2)}%, ${(g.hsla.l*100).toFixed(2)}%, ${g.hsla.a.toFixed(2)})`}t.formatHSLA=r;function a(g){const f=g.toString(16);return f.length!==2?"0"+f:f}function l(g){return`#${a(g.rgba.r)}${a(g.rgba.g)}${a(g.rgba.b)}`}t.formatHex=l;function c(g,f=!1){return f&&g.rgba.a===1?o.Format.CSS.formatHex(g):`#${a(g.rgba.r)}${a(g.rgba.g)}${a(g.rgba.b)}${a(Math.round(g.rgba.a*255))}`}t.formatHexA=c;function d(g){return g.isOpaque()?o.Format.CSS.formatHex(g):o.Format.CSS.formatRGBA(g)}t.format=d;function h(g){const f=g.length;if(f===0||g.charCodeAt(0)!==35)return null;if(f===7){const p=16*u(g.charCodeAt(1))+u(g.charCodeAt(2)),_=16*u(g.charCodeAt(3))+u(g.charCodeAt(4)),b=16*u(g.charCodeAt(5))+u(g.charCodeAt(6));return new o(new dt(p,_,b,1))}if(f===9){const p=16*u(g.charCodeAt(1))+u(g.charCodeAt(2)),_=16*u(g.charCodeAt(3))+u(g.charCodeAt(4)),b=16*u(g.charCodeAt(5))+u(g.charCodeAt(6)),C=16*u(g.charCodeAt(7))+u(g.charCodeAt(8));return new o(new dt(p,_,b,C/255))}if(f===4){const p=u(g.charCodeAt(1)),_=u(g.charCodeAt(2)),b=u(g.charCodeAt(3));return new o(new dt(16*p+p,16*_+_,16*b+b))}if(f===5){const p=u(g.charCodeAt(1)),_=u(g.charCodeAt(2)),b=u(g.charCodeAt(3)),C=u(g.charCodeAt(4));return new o(new dt(16*p+p,16*_+_,16*b+b,(16*C+C)/255))}return null}t.parseHex=h;function u(g){switch(g){case 48:return 0;case 49:return 1;case 50:return 2;case 51:return 3;case 52:return 4;case 53:return 5;case 54:return 6;case 55:return 7;case 56:return 8;case 57:return 9;case 97:return 10;case 65:return 10;case 98:return 11;case 66:return 11;case 99:return 12;case 67:return 12;case 100:return 13;case 68:return 13;case 101:return 14;case 69:return 14;case 102:return 15;case 70:return 15}return 0}})(e.CSS||(e.CSS={}))})(o.Format||(o.Format={}))})(Z||(Z={}));function c$(o){const e=[];for(const t of o){const i=Number(t);(i||i===0&&t.replace(/\s/g,"")!=="")&&e.push(i)}return e}function w5(o,e,t,i){return{red:o/255,blue:t/255,green:e/255,alpha:i}}function Jb(o,e){const t=e.index,i=e[0].length;if(!t)return;const n=o.positionAt(t);return{startLineNumber:n.lineNumber,startColumn:n.column,endLineNumber:n.lineNumber,endColumn:n.column+i}}function Tse(o,e){if(!o)return;const t=Z.Format.CSS.parseHex(e);if(t)return{range:o,color:w5(t.rgba.r,t.rgba.g,t.rgba.b,t.rgba.a)}}function e9(o,e,t){if(!o||e.length!==1)return;const n=e[0].values(),s=c$(n);return{range:o,color:w5(s[0],s[1],s[2],t?s[3]:1)}}function t9(o,e,t){if(!o||e.length!==1)return;const n=e[0].values(),s=c$(n),r=new Z(new Fr(s[0],s[1]/100,s[2]/100,t?s[3]:1));return{range:o,color:w5(r.rgba.r,r.rgba.g,r.rgba.b,r.rgba.a)}}function eC(o,e){return typeof o=="string"?[...o.matchAll(e)]:o.findMatches(e)}function Rse(o){const e=[],i=eC(o,/\b(rgb|rgba|hsl|hsla)(\([0-9\s,.\%]*\))|(#)([A-Fa-f0-9]{3})\b|(#)([A-Fa-f0-9]{4})\b|(#)([A-Fa-f0-9]{6})\b|(#)([A-Fa-f0-9]{8})\b/gm);if(i.length>0)for(const n of i){const s=n.filter(c=>c!==void 0),r=s[1],a=s[2];if(!a)continue;let l;if(r==="rgb"){const c=/^\(\s*(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])\s*,\s*(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])\s*,\s*(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])\s*\)$/gm;l=e9(Jb(o,n),eC(a,c),!1)}else if(r==="rgba"){const c=/^\(\s*(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])\s*,\s*(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])\s*,\s*(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])\s*,\s*(0[.][0-9]+|[.][0-9]+|[01][.]|[01])\s*\)$/gm;l=e9(Jb(o,n),eC(a,c),!0)}else if(r==="hsl"){const c=/^\(\s*(36[0]|3[0-5][0-9]|[12][0-9][0-9]|[1-9]?[0-9])\s*,\s*(100|\d{1,2}[.]\d*|\d{1,2})%\s*,\s*(100|\d{1,2}[.]\d*|\d{1,2})%\s*\)$/gm;l=t9(Jb(o,n),eC(a,c),!1)}else if(r==="hsla"){const c=/^\(\s*(36[0]|3[0-5][0-9]|[12][0-9][0-9]|[1-9]?[0-9])\s*,\s*(100|\d{1,2}[.]\d*|\d{1,2})%\s*,\s*(100|\d{1,2}[.]\d*|\d{1,2})%\s*,\s*(0[.][0-9]+|[.][0-9]+|[01][.]|[01])\s*\)$/gm;l=t9(Jb(o,n),eC(a,c),!0)}else r==="#"&&(l=Tse(Jb(o,n),r+a));l&&e.push(l)}return e}function Mse(o){return!o||typeof o.getValue!="function"||typeof o.positionAt!="function"?[]:Rse(o)}const i9=new RegExp("\\bMARK:\\s*(.*)$","d"),Ase=/^-+|-+$/g;function Pse(o,e){var i;let t=[];if(e.findRegionSectionHeaders&&((i=e.foldingRules)!=null&&i.markers)){const n=Ose(o,e);t=t.concat(n)}if(e.findMarkSectionHeaders){const n=Fse(o);t=t.concat(n)}return t}function Ose(o,e){const t=[],i=o.getLineCount();for(let n=1;n<=i;n++){const s=o.getLineContent(n),r=s.match(e.foldingRules.markers.start);if(r){const a={startLineNumber:n,startColumn:r[0].length+1,endLineNumber:n,endColumn:s.length+1};if(a.endColumn>a.startColumn){const l={range:a,...d$(s.substring(r[0].length)),shouldBeInComments:!1};(l.text||l.hasSeparatorLine)&&t.push(l)}}}return t}function Fse(o){const e=[],t=o.getLineCount();for(let i=1;i<=t;i++){const n=o.getLineContent(i);Wse(n,i,e)}return e}function Wse(o,e,t){i9.lastIndex=0;const i=i9.exec(o);if(i){const n=i.indices[1][0]+1,s=i.indices[1][1]+1,r={startLineNumber:e,startColumn:n,endLineNumber:e,endColumn:s};if(r.endColumn>r.startColumn){const a={range:r,...d$(i[1]),shouldBeInComments:!0};(a.text||a.hasSeparatorLine)&&t.push(a)}}}function d$(o){o=o.trim();const e=o.startsWith("-");return o=o.replace(Ase,""),{text:o,hasSeparatorLine:e}}class Bse{constructor(e){this.values=e,this.prefixSum=new Uint32Array(e.length),this.prefixSumValidIndex=new Int32Array(1),this.prefixSumValidIndex[0]=-1}insertValues(e,t){e=xm(e);const i=this.values,n=this.prefixSum,s=t.length;return s===0?!1:(this.values=new Uint32Array(i.length+s),this.values.set(i.subarray(0,e),0),this.values.set(i.subarray(e),e+s),this.values.set(t,e),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSum=new Uint32Array(this.values.length),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(n.subarray(0,this.prefixSumValidIndex[0]+1)),!0)}setValue(e,t){return e=xm(e),t=xm(t),this.values[e]===t?!1:(this.values[e]=t,e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),!0)}removeValues(e,t){e=xm(e),t=xm(t);const i=this.values,n=this.prefixSum;if(e>=i.length)return!1;const s=i.length-e;return t>=s&&(t=s),t===0?!1:(this.values=new Uint32Array(i.length-t),this.values.set(i.subarray(0,e),0),this.values.set(i.subarray(e+t),e),this.prefixSum=new Uint32Array(this.values.length),e-1<this.prefixSumValidIndex[0]&&(this.prefixSumValidIndex[0]=e-1),this.prefixSumValidIndex[0]>=0&&this.prefixSum.set(n.subarray(0,this.prefixSumValidIndex[0]+1)),!0)}getTotalSum(){return this.values.length===0?0:this._getPrefixSum(this.values.length-1)}getPrefixSum(e){return e<0?0:(e=xm(e),this._getPrefixSum(e))}_getPrefixSum(e){if(e<=this.prefixSumValidIndex[0])return this.prefixSum[e];let t=this.prefixSumValidIndex[0]+1;t===0&&(this.prefixSum[0]=this.values[0],t++),e>=this.values.length&&(e=this.values.length-1);for(let i=t;i<=e;i++)this.prefixSum[i]=this.prefixSum[i-1]+this.values[i];return this.prefixSumValidIndex[0]=Math.max(this.prefixSumValidIndex[0],e),this.prefixSum[e]}getIndexOf(e){e=Math.floor(e),this.getTotalSum();let t=0,i=this.values.length-1,n=0,s=0,r=0;for(;t<=i;)if(n=t+(i-t)/2|0,s=this.prefixSum[n],r=s-this.values[n],e<r)i=n-1;else if(e>=s)t=n+1;else break;return new h$(n,e-r)}}class Hse{constructor(e){this._values=e,this._isValid=!1,this._validEndIndex=-1,this._prefixSum=[],this._indexBySum=[]}getTotalSum(){return this._ensureValid(),this._indexBySum.length}getPrefixSum(e){return this._ensureValid(),e===0?0:this._prefixSum[e-1]}getIndexOf(e){this._ensureValid();const t=this._indexBySum[e],i=t>0?this._prefixSum[t-1]:0;return new h$(t,e-i)}removeValues(e,t){this._values.splice(e,t),this._invalidate(e)}insertValues(e,t){this._values=cI(this._values,e,t),this._invalidate(e)}_invalidate(e){this._isValid=!1,this._validEndIndex=Math.min(this._validEndIndex,e-1)}_ensureValid(){if(!this._isValid){for(let e=this._validEndIndex+1,t=this._values.length;e<t;e++){const i=this._values[e],n=e>0?this._prefixSum[e-1]:0;this._prefixSum[e]=n+i;for(let s=0;s<i;s++)this._indexBySum[n+s]=e}this._prefixSum.length=this._values.length,this._indexBySum.length=this._prefixSum[this._prefixSum.length-1],this._isValid=!0,this._validEndIndex=this._values.length-1}}setValue(e,t){this._values[e]!==t&&(this._values[e]=t,this._invalidate(e))}}class h${constructor(e,t){this.index=e,this.remainder=t,this._prefixSumIndexOfResultBrand=void 0,this.index=e,this.remainder=t}}class Vse{constructor(e,t,i,n){this._uri=e,this._lines=t,this._eol=i,this._versionId=n,this._lineStarts=null,this._cachedTextValue=null}dispose(){this._lines.length=0}get version(){return this._versionId}getText(){return this._cachedTextValue===null&&(this._cachedTextValue=this._lines.join(this._eol)),this._cachedTextValue}onEvents(e){e.eol&&e.eol!==this._eol&&(this._eol=e.eol,this._lineStarts=null);const t=e.changes;for(const i of t)this._acceptDeleteRange(i.range),this._acceptInsertText(new A(i.range.startLineNumber,i.range.startColumn),i.text);this._versionId=e.versionId,this._cachedTextValue=null}_ensureLineStarts(){if(!this._lineStarts){const e=this._eol.length,t=this._lines.length,i=new Uint32Array(t);for(let n=0;n<t;n++)i[n]=this._lines[n].length+e;this._lineStarts=new Bse(i)}}_setLineText(e,t){this._lines[e]=t,this._lineStarts&&this._lineStarts.setValue(e,this._lines[e].length+this._eol.length)}_acceptDeleteRange(e){if(e.startLineNumber===e.endLineNumber){if(e.startColumn===e.endColumn)return;this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.startLineNumber-1].substring(e.endColumn-1));return}this._setLineText(e.startLineNumber-1,this._lines[e.startLineNumber-1].substring(0,e.startColumn-1)+this._lines[e.endLineNumber-1].substring(e.endColumn-1)),this._lines.splice(e.startLineNumber,e.endLineNumber-e.startLineNumber),this._lineStarts&&this._lineStarts.removeValues(e.startLineNumber,e.endLineNumber-e.startLineNumber)}_acceptInsertText(e,t){if(t.length===0)return;const i=Kl(t);if(i.length===1){this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+i[0]+this._lines[e.lineNumber-1].substring(e.column-1));return}i[i.length-1]+=this._lines[e.lineNumber-1].substring(e.column-1),this._setLineText(e.lineNumber-1,this._lines[e.lineNumber-1].substring(0,e.column-1)+i[0]);const n=new Uint32Array(i.length-1);for(let s=1;s<i.length;s++)this._lines.splice(e.lineNumber+s-1,0,i[s]),n[s-1]=i[s].length+this._eol.length;this._lineStarts&&this._lineStarts.insertValues(e.lineNumber,n)}}const n9=60*1e3;class zse extends F{constructor(e,t,i=!1){if(super(),this._syncedModels=Object.create(null),this._syncedModelsLastUsedTime=Object.create(null),this._proxy=e,this._modelService=t,!i){const n=new s5;n.cancelAndSet(()=>this._checkStopModelSync(),Math.round(n9/2)),this._register(n)}}dispose(){for(const e in this._syncedModels)Tt(this._syncedModels[e]);this._syncedModels=Object.create(null),this._syncedModelsLastUsedTime=Object.create(null),super.dispose()}ensureSyncedResources(e,t=!1){for(const i of e){const n=i.toString();this._syncedModels[n]||this._beginModelSync(i,t),this._syncedModels[n]&&(this._syncedModelsLastUsedTime[n]=new Date().getTime())}}_checkStopModelSync(){const e=new Date().getTime(),t=[];for(const i in this._syncedModelsLastUsedTime)e-this._syncedModelsLastUsedTime[i]>n9&&t.push(i);for(const i of t)this._stopModelSync(i)}_beginModelSync(e,t){const i=this._modelService.getModel(e);if(!i||!t&&i.isTooLargeForSyncing())return;const n=e.toString();this._proxy.$acceptNewModel({url:i.uri.toString(),lines:i.getLinesContent(),EOL:i.getEOL(),versionId:i.getVersionId()});const s=new K;s.add(i.onDidChangeContent(r=>{this._proxy.$acceptModelChanged(n.toString(),r)})),s.add(i.onWillDispose(()=>{this._stopModelSync(n)})),s.add(ve(()=>{this._proxy.$acceptRemovedModel(n)})),this._syncedModels[n]=s}_stopModelSync(e){const t=this._syncedModels[e];delete this._syncedModels[e],delete this._syncedModelsLastUsedTime[e],Tt(t)}}class Use{constructor(){this._models=Object.create(null)}getModel(e){return this._models[e]}getModels(){const e=[];return Object.keys(this._models).forEach(t=>e.push(this._models[t])),e}$acceptNewModel(e){this._models[e.url]=new $se(Se.parse(e.url),e.lines,e.EOL,e.versionId)}$acceptModelChanged(e,t){if(!this._models[e])return;this._models[e].onEvents(t)}$acceptRemovedModel(e){this._models[e]&&delete this._models[e]}}class $se extends Vse{get uri(){return this._uri}get eol(){return this._eol}getValue(){return this.getText()}findMatches(e){const t=[];for(let i=0;i<this._lines.length;i++){const n=this._lines[i],s=this.offsetAt(new A(i+1,1)),r=n.matchAll(e);for(const a of r)(a.index||a.index===0)&&(a.index=a.index+s),t.push(a)}return t}getLinesContent(){return this._lines.slice(0)}getLineCount(){return this._lines.length}getLineContent(e){return this._lines[e-1]}getWordAtPosition(e,t){const i=Qv(e.column,z4(t),this._lines[e.lineNumber-1],0);return i?new x(e.lineNumber,i.startColumn,e.lineNumber,i.endColumn):null}words(e){const t=this._lines,i=this._wordenize.bind(this);let n=0,s="",r=0,a=[];return{*[Symbol.iterator](){for(;;)if(r<a.length){const l=s.substring(a[r].start,a[r].end);r+=1,yield l}else if(n<t.length)s=t[n],a=i(s,e),r=0,n+=1;else break}}}getLineWords(e,t){const i=this._lines[e-1],n=this._wordenize(i,t),s=[];for(const r of n)s.push({word:i.substring(r.start,r.end),startColumn:r.start+1,endColumn:r.end+1});return s}_wordenize(e,t){const i=[];let n;for(t.lastIndex=0;(n=t.exec(e))&&n[0].length!==0;)i.push({start:n.index,end:n.index+n[0].length});return i}getValueInRange(e){if(e=this._validateRange(e),e.startLineNumber===e.endLineNumber)return this._lines[e.startLineNumber-1].substring(e.startColumn-1,e.endColumn-1);const t=this._eol,i=e.startLineNumber-1,n=e.endLineNumber-1,s=[];s.push(this._lines[i].substring(e.startColumn-1));for(let r=i+1;r<n;r++)s.push(this._lines[r]);return s.push(this._lines[n].substring(0,e.endColumn-1)),s.join(t)}offsetAt(e){return e=this._validatePosition(e),this._ensureLineStarts(),this._lineStarts.getPrefixSum(e.lineNumber-2)+(e.column-1)}positionAt(e){e=Math.floor(e),e=Math.max(0,e),this._ensureLineStarts();const t=this._lineStarts.getIndexOf(e),i=this._lines[t.index].length;return{lineNumber:1+t.index,column:1+Math.min(t.remainder,i)}}_validateRange(e){const t=this._validatePosition({lineNumber:e.startLineNumber,column:e.startColumn}),i=this._validatePosition({lineNumber:e.endLineNumber,column:e.endColumn});return t.lineNumber!==e.startLineNumber||t.column!==e.startColumn||i.lineNumber!==e.endLineNumber||i.column!==e.endColumn?{startLineNumber:t.lineNumber,startColumn:t.column,endLineNumber:i.lineNumber,endColumn:i.column}:e}_validatePosition(e){if(!A.isIPosition(e))throw new Error("bad position");let{lineNumber:t,column:i}=e,n=!1;if(t<1)t=1,i=1,n=!0;else if(t>this._lines.length)t=this._lines.length,i=this._lines[t-1].length+1,n=!0;else{const s=this._lines[t-1].length+1;i<1?(i=1,n=!0):i>s&&(i=s,n=!0)}return n?{lineNumber:t,column:i}:e}}const Bk=class Bk{constructor(){this._workerTextModelSyncServer=new Use}dispose(){}_getModel(e){return this._workerTextModelSyncServer.getModel(e)}_getModels(){return this._workerTextModelSyncServer.getModels()}$acceptNewModel(e){this._workerTextModelSyncServer.$acceptNewModel(e)}$acceptModelChanged(e,t){this._workerTextModelSyncServer.$acceptModelChanged(e,t)}$acceptRemovedModel(e){this._workerTextModelSyncServer.$acceptRemovedModel(e)}async $computeUnicodeHighlights(e,t,i){const n=this._getModel(e);return n?_5.computeUnicodeHighlights(n,t,i):{ranges:[],hasMore:!1,ambiguousCharacterCount:0,invisibleCharacterCount:0,nonBasicAsciiCharacterCount:0}}async $findSectionHeaders(e,t){const i=this._getModel(e);return i?Pse(i,t):[]}async $computeDiff(e,t,i,n){const s=this._getModel(e),r=this._getModel(t);return!s||!r?null:K0.computeDiff(s,r,i,n)}static computeDiff(e,t,i,n){const s=n==="advanced"?J7.getDefault():J7.getLegacy(),r=e.getLinesContent(),a=t.getLinesContent(),l=s.computeDiff(r,a,i),c=l.changes.length>0?!1:this._modelsAreIdentical(e,t);function d(h){return h.map(u=>{var g;return[u.original.startLineNumber,u.original.endLineNumberExclusive,u.modified.startLineNumber,u.modified.endLineNumberExclusive,(g=u.innerChanges)==null?void 0:g.map(f=>[f.originalRange.startLineNumber,f.originalRange.startColumn,f.originalRange.endLineNumber,f.originalRange.endColumn,f.modifiedRange.startLineNumber,f.modifiedRange.startColumn,f.modifiedRange.endLineNumber,f.modifiedRange.endColumn])]})}return{identical:c,quitEarly:l.hitTimeout,changes:d(l.changes),moves:l.moves.map(h=>[h.lineRangeMapping.original.startLineNumber,h.lineRangeMapping.original.endLineNumberExclusive,h.lineRangeMapping.modified.startLineNumber,h.lineRangeMapping.modified.endLineNumberExclusive,d(h.changes)])}}static _modelsAreIdentical(e,t){const i=e.getLineCount(),n=t.getLineCount();if(i!==n)return!1;for(let s=1;s<=i;s++){const r=e.getLineContent(s),a=t.getLineContent(s);if(r!==a)return!1}return!0}async $computeMoreMinimalEdits(e,t,i){const n=this._getModel(e);if(!n)return t;const s=[];let r;t=t.slice(0).sort((l,c)=>{if(l.range&&c.range)return x.compareRangesUsingStarts(l.range,c.range);const d=l.range?0:1,h=c.range?0:1;return d-h});let a=0;for(let l=1;l<t.length;l++)x.getEndPosition(t[a].range).equals(x.getStartPosition(t[l].range))?(t[a].range=x.fromPositions(x.getStartPosition(t[a].range),x.getEndPosition(t[l].range)),t[a].text+=t[l].text):(a++,t[a]=t[l]);t.length=a+1;for(let{range:l,text:c,eol:d}of t){if(typeof d=="number"&&(r=d),x.isEmpty(l)&&!c)continue;const h=n.getValueInRange(l);if(c=c.replace(/\r\n|\n|\r/g,n.eol),h===c)continue;if(Math.max(c.length,h.length)>K0._diffLimit){s.push({range:l,text:c});continue}const u=Mne(h,c,i),g=n.offsetAt(x.lift(l).getStartPosition());for(const f of u){const p=n.positionAt(g+f.originalStart),_=n.positionAt(g+f.originalStart+f.originalLength),b={text:c.substr(f.modifiedStart,f.modifiedLength),range:{startLineNumber:p.lineNumber,startColumn:p.column,endLineNumber:_.lineNumber,endColumn:_.column}};n.getValueInRange(b.range)!==b.text&&s.push(b)}}return typeof r=="number"&&s.push({eol:r,text:"",range:{startLineNumber:0,startColumn:0,endLineNumber:0,endColumn:0}}),s}async $computeLinks(e){const t=this._getModel(e);return t?Wne(t):null}async $computeDefaultDocumentColors(e){const t=this._getModel(e);return t?Mse(t):null}async $textualSuggest(e,t,i,n){const s=new ln,r=new RegExp(i,n),a=new Set;e:for(const l of e){const c=this._getModel(l);if(c){for(const d of c.words(r))if(!(d===t||!isNaN(Number(d)))&&(a.add(d),a.size>K0._suggestionsLimit))break e}}return{words:Array.from(a),duration:s.elapsed()}}async $computeWordRanges(e,t,i,n){const s=this._getModel(e);if(!s)return Object.create(null);const r=new RegExp(i,n),a=Object.create(null);for(let l=t.startLineNumber;l<t.endLineNumber;l++){const c=s.getLineWords(l,r);for(const d of c){if(!isNaN(Number(d.word)))continue;let h=a[d.word];h||(h=[],a[d.word]=h),h.push({startLineNumber:l,startColumn:d.startColumn,endLineNumber:l,endColumn:d.endColumn})}}return a}async $navigateValueSet(e,t,i,n,s){const r=this._getModel(e);if(!r)return null;const a=new RegExp(n,s);t.startColumn===t.endColumn&&(t={startLineNumber:t.startLineNumber,startColumn:t.startColumn,endLineNumber:t.endLineNumber,endColumn:t.endColumn+1});const l=r.getValueInRange(t),c=r.getWordAtPosition({lineNumber:t.startLineNumber,column:t.startColumn},a);if(!c)return null;const d=r.getValueInRange(c);return zM.INSTANCE.navigateValueSet(t,l,c,d,i)}};Bk._diffLimit=1e5,Bk._suggestionsLimit=1e4;let GM=Bk;class K0 extends GM{constructor(e,t){super(),this._host=e,this._foreignModuleFactory=t,this._foreignModule=null}async $ping(){return"pong"}$loadForeignModule(e,t,i){const r={host:EQ(i,(a,l)=>this._host.$fhr(a,l)),getMirrorModels:()=>this._getModels()};return this._foreignModuleFactory?(this._foreignModule=this._foreignModuleFactory(r,t),Promise.resolve(gR(this._foreignModule))):new Promise((a,l)=>{const c=d=>{this._foreignModule=d.create(r,t),a(gR(this._foreignModule))};{const d=pI.asBrowserUri(`${e}.js`).toString(!0);ye(()=>import(`${d}`),[],import.meta.url).then(c).catch(l)}})}$fmr(e,t){if(!this._foreignModule||typeof this._foreignModule[e]!="function")return Promise.reject(new Error("Missing requestHandler or method: "+e));try{return Promise.resolve(this._foreignModule[e].apply(this._foreignModule,t))}catch(i){return Promise.reject(i)}}}typeof importScripts=="function"&&(globalThis.monaco=Xz());const PI=et("textResourceConfigurationService"),u$=et("textResourcePropertiesService"),ge=et("ILanguageFeaturesService");var S5=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},zu=function(o,e){return function(t,i){e(t,i,o)}};const s9=5*60*1e3;function Uu(o,e){const t=o.getModel(e);return!(!t||t.isTooLargeForSyncing())}let ZM=class extends F{constructor(e,t,i,n,s,r){super(),this._languageConfigurationService=s,this._modelService=t,this._workerManager=this._register(new YM(e,this._modelService)),this._logService=n,this._register(r.linkProvider.register({language:"*",hasAccessToAllModels:!0},{provideLinks:async(a,l)=>{if(!Uu(this._modelService,a.uri))return Promise.resolve({links:[]});const d=await(await this._workerWithResources([a.uri])).$computeLinks(a.uri.toString());return d&&{links:d}}})),this._register(r.completionProvider.register("*",new jse(this._workerManager,i,this._modelService,this._languageConfigurationService)))}dispose(){super.dispose()}canComputeUnicodeHighlights(e){return Uu(this._modelService,e)}async computedUnicodeHighlights(e,t,i){return(await this._workerWithResources([e])).$computeUnicodeHighlights(e.toString(),t,i)}async computeDiff(e,t,i,n){const r=await(await this._workerWithResources([e,t],!0)).$computeDiff(e.toString(),t.toString(),i,n);if(!r)return null;return{identical:r.identical,quitEarly:r.quitEarly,changes:l(r.changes),moves:r.moves.map(c=>new s$(new Yn(new Be(c[0],c[1]),new Be(c[2],c[3])),l(c[4])))};function l(c){return c.map(d=>{var h;return new Vo(new Be(d[0],d[1]),new Be(d[2],d[3]),(h=d[4])==null?void 0:h.map(u=>new Eo(new x(u[0],u[1],u[2],u[3]),new x(u[4],u[5],u[6],u[7]))))})}}async computeMoreMinimalEdits(e,t,i=!1){if(Qn(t)){if(!Uu(this._modelService,e))return Promise.resolve(t);const n=ln.create(),s=this._workerWithResources([e]).then(r=>r.$computeMoreMinimalEdits(e.toString(),t,i));return s.finally(()=>this._logService.trace("FORMAT#computeMoreMinimalEdits",e.toString(!0),n.elapsed())),Promise.race([s,ed(1e3).then(()=>t)])}else return Promise.resolve(void 0)}canNavigateValueSet(e){return Uu(this._modelService,e)}async navigateValueSet(e,t,i){const n=this._modelService.getModel(e);if(!n)return null;const s=this._languageConfigurationService.getLanguageConfiguration(n.getLanguageId()).getWordDefinition(),r=s.source,a=s.flags;return(await this._workerWithResources([e])).$navigateValueSet(e.toString(),t,i,r,a)}canComputeWordRanges(e){return Uu(this._modelService,e)}async computeWordRanges(e,t){const i=this._modelService.getModel(e);if(!i)return Promise.resolve(null);const n=this._languageConfigurationService.getLanguageConfiguration(i.getLanguageId()).getWordDefinition(),s=n.source,r=n.flags;return(await this._workerWithResources([e])).$computeWordRanges(e.toString(),t,s,r)}async findSectionHeaders(e,t){return(await this._workerWithResources([e])).$findSectionHeaders(e.toString(),t)}async computeDefaultDocumentColors(e){return(await this._workerWithResources([e])).$computeDefaultDocumentColors(e.toString())}async _workerWithResources(e,t=!1){return await(await this._workerManager.withWorker()).workerWithSyncedResources(e,t)}};ZM=S5([zu(1,Kt),zu(2,PI),zu(3,Nn),zu(4,di),zu(5,ge)],ZM);class jse{constructor(e,t,i,n){this.languageConfigurationService=n,this._debugDisplayName="wordbasedCompletions",this._workerManager=e,this._configurationService=t,this._modelService=i}async provideCompletionItems(e,t){const i=this._configurationService.getValue(e.uri,t,"editor");if(i.wordBasedSuggestions==="off")return;const n=[];if(i.wordBasedSuggestions==="currentDocument")Uu(this._modelService,e.uri)&&n.push(e.uri);else for(const h of this._modelService.getModels())Uu(this._modelService,h.uri)&&(h===e?n.unshift(h.uri):(i.wordBasedSuggestions==="allDocuments"||h.getLanguageId()===e.getLanguageId())&&n.push(h.uri));if(n.length===0)return;const s=this.languageConfigurationService.getLanguageConfiguration(e.getLanguageId()).getWordDefinition(),r=e.getWordAtPosition(t),a=r?new x(t.lineNumber,r.startColumn,t.lineNumber,r.endColumn):x.fromPositions(t),l=a.setEndPosition(t.lineNumber,t.column),d=await(await this._workerManager.withWorker()).textualSuggest(n,r==null?void 0:r.word,s);if(d)return{duration:d.duration,suggestions:d.words.map(h=>({kind:18,label:h,insertText:h,range:{insert:l,replace:a}}))}}}let YM=class extends F{constructor(e,t){super(),this._workerDescriptor=e,this._modelService=t,this._editorWorkerClient=null,this._lastWorkerUsedTime=new Date().getTime(),this._register(new r5).cancelAndSet(()=>this._checkStopIdleWorker(),Math.round(s9/2),Rt),this._register(this._modelService.onModelRemoved(n=>this._checkStopEmptyWorker()))}dispose(){this._editorWorkerClient&&(this._editorWorkerClient.dispose(),this._editorWorkerClient=null),super.dispose()}_checkStopEmptyWorker(){if(!this._editorWorkerClient)return;this._modelService.getModels().length===0&&(this._editorWorkerClient.dispose(),this._editorWorkerClient=null)}_checkStopIdleWorker(){if(!this._editorWorkerClient)return;new Date().getTime()-this._lastWorkerUsedTime>s9&&(this._editorWorkerClient.dispose(),this._editorWorkerClient=null)}withWorker(){return this._lastWorkerUsedTime=new Date().getTime(),this._editorWorkerClient||(this._editorWorkerClient=new nL(this._workerDescriptor,!1,this._modelService)),Promise.resolve(this._editorWorkerClient)}};YM=S5([zu(1,Kt)],YM);class Kse{constructor(e){this._instance=e,this.proxy=this._instance}dispose(){this._instance.dispose()}setChannel(e,t){throw new Error("Not supported")}}let nL=class extends F{constructor(e,t,i){super(),this._workerDescriptor=e,this._disposed=!1,this._modelService=i,this._keepIdleModels=t,this._worker=null,this._modelManager=null}fhr(e,t){throw new Error("Not implemented!")}_getOrCreateWorker(){if(!this._worker)try{this._worker=this._register(Yie(this._workerDescriptor)),UM.setChannel(this._worker,this._createEditorWorkerHost())}catch(e){OM(e),this._worker=this._createFallbackLocalWorker()}return this._worker}async _getProxy(){try{const e=this._getOrCreateWorker().proxy;return await e.$ping(),e}catch(e){return OM(e),this._worker=this._createFallbackLocalWorker(),this._worker.proxy}}_createFallbackLocalWorker(){return new Kse(new K0(this._createEditorWorkerHost(),null))}_createEditorWorkerHost(){return{$fhr:(e,t)=>this.fhr(e,t)}}_getOrCreateModelManager(e){return this._modelManager||(this._modelManager=this._register(new zse(e,this._modelService,this._keepIdleModels))),this._modelManager}async workerWithSyncedResources(e,t=!1){if(this._disposed)return Promise.reject(OJ());const i=await this._getProxy();return this._getOrCreateModelManager(i).ensureSyncedResources(e,t),i}async textualSuggest(e,t,i){const n=await this.workerWithSyncedResources(e),s=i.source,r=i.flags;return n.$textualSuggest(e.map(a=>a.toString()),t,s,r)}dispose(){super.dispose(),this._disposed=!0}};nL=S5([zu(2,Kt)],nL);var hr;(function(o){o.DARK="dark",o.LIGHT="light",o.HIGH_CONTRAST_DARK="hcDark",o.HIGH_CONTRAST_LIGHT="hcLight"})(hr||(hr={}));function Ta(o){return o===hr.HIGH_CONTRAST_DARK||o===hr.HIGH_CONTRAST_LIGHT}function N_(o){return o===hr.DARK||o===hr.HIGH_CONTRAST_DARK}const ji=et("themeService");function mi(o){return{id:o}}function XM(o){switch(o){case hr.DARK:return"vs-dark";case hr.HIGH_CONTRAST_DARK:return"hc-black";case hr.HIGH_CONTRAST_LIGHT:return"hc-light";default:return"vs"}}const g$={ThemingContribution:"base.contributions.theming"};class qse{constructor(){this.themingParticipants=[],this.themingParticipants=[],this.onThemingParticipantAddedEmitter=new P}onColorThemeChange(e){return this.themingParticipants.push(e),this.onThemingParticipantAddedEmitter.fire(e),ve(()=>{const t=this.themingParticipants.indexOf(e);this.themingParticipants.splice(t,1)})}getThemingParticipants(){return this.themingParticipants}}const f$=new qse;ci.add(g$.ThemingContribution,f$);function Lr(o){return f$.onColorThemeChange(o)}class Gse extends F{constructor(e){super(),this.themeService=e,this.theme=e.getColorTheme(),this._register(this.themeService.onDidColorThemeChange(t=>this.onThemeChange(t)))}onThemeChange(e){this.theme=e,this.updateStyles()}updateStyles(){}}var Zse=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Yse=function(o,e){return function(t,i){e(t,i,o)}};let QM=class extends F{constructor(e){super(),this._themeService=e,this._onWillCreateCodeEditor=this._register(new P),this._onCodeEditorAdd=this._register(new P),this.onCodeEditorAdd=this._onCodeEditorAdd.event,this._onCodeEditorRemove=this._register(new P),this.onCodeEditorRemove=this._onCodeEditorRemove.event,this._onWillCreateDiffEditor=this._register(new P),this._onDiffEditorAdd=this._register(new P),this.onDiffEditorAdd=this._onDiffEditorAdd.event,this._onDiffEditorRemove=this._register(new P),this.onDiffEditorRemove=this._onDiffEditorRemove.event,this._decorationOptionProviders=new Map,this._codeEditorOpenHandlers=new cs,this._modelProperties=new Map,this._codeEditors=Object.create(null),this._diffEditors=Object.create(null),this._globalStyleSheet=null}willCreateCodeEditor(){this._onWillCreateCodeEditor.fire()}addCodeEditor(e){this._codeEditors[e.getId()]=e,this._onCodeEditorAdd.fire(e)}removeCodeEditor(e){delete this._codeEditors[e.getId()]&&this._onCodeEditorRemove.fire(e)}listCodeEditors(){return Object.keys(this._codeEditors).map(e=>this._codeEditors[e])}willCreateDiffEditor(){this._onWillCreateDiffEditor.fire()}addDiffEditor(e){this._diffEditors[e.getId()]=e,this._onDiffEditorAdd.fire(e)}listDiffEditors(){return Object.keys(this._diffEditors).map(e=>this._diffEditors[e])}getFocusedCodeEditor(){let e=null;const t=this.listCodeEditors();for(const i of t){if(i.hasTextFocus())return i;i.hasWidgetFocus()&&(e=i)}return e}removeDecorationType(e){const t=this._decorationOptionProviders.get(e);t&&(t.refCount--,t.refCount<=0&&(this._decorationOptionProviders.delete(e),t.dispose(),this.listCodeEditors().forEach(i=>i.removeDecorationsByType(e))))}setModelProperty(e,t,i){const n=e.toString();let s;this._modelProperties.has(n)?s=this._modelProperties.get(n):(s=new Map,this._modelProperties.set(n,s)),s.set(t,i)}getModelProperty(e,t){const i=e.toString();if(this._modelProperties.has(i))return this._modelProperties.get(i).get(t)}async openCodeEditor(e,t,i){for(const n of this._codeEditorOpenHandlers){const s=await n(e,t,i);if(s!==null)return s}return null}registerCodeEditorOpenHandler(e){const t=this._codeEditorOpenHandlers.unshift(e);return ve(t)}};QM=Zse([Yse(0,ji)],QM);var Xse=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},o9=function(o,e){return function(t,i){e(t,i,o)}};let sL=class extends QM{constructor(e,t){super(t),this._register(this.onCodeEditorAdd(()=>this._checkContextKey())),this._register(this.onCodeEditorRemove(()=>this._checkContextKey())),this._editorIsOpen=e.createKey("editorIsOpen",!1),this._activeCodeEditor=null,this._register(this.registerCodeEditorOpenHandler(async(i,n,s)=>n?this.doOpenEditor(n,i):null))}_checkContextKey(){let e=!1;for(const t of this.listCodeEditors())if(!t.isSimpleWidget){e=!0;break}this._editorIsOpen.set(e)}setActiveCodeEditor(e){this._activeCodeEditor=e}getActiveCodeEditor(){return this._activeCodeEditor}doOpenEditor(e,t){if(!this.findModel(e,t.resource)){if(t.resource){const s=t.resource.scheme;if(s===Fe.http||s===Fe.https)return RU(t.resource.toString()),e}return null}const n=t.options?t.options.selection:null;if(n)if(typeof n.endLineNumber=="number"&&typeof n.endColumn=="number")e.setSelection(n),e.revealRangeInCenter(n,1);else{const s={lineNumber:n.startLineNumber,column:n.startColumn};e.setPosition(s),e.revealPositionInCenter(s,1)}return e}findModel(e,t){const i=e.getModel();return i&&i.uri.toString()!==t.toString()?null:i}};sL=Xse([o9(0,Ee),o9(1,ji)],sL);st(pt,sL,0);const tu=et("layoutService");var m$=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},p$=function(o,e){return function(t,i){e(t,i,o)}};let oL=class{get mainContainer(){var e;return((e=O4(this._codeEditorService.listCodeEditors()))==null?void 0:e.getContainerDomNode())??Rt.document.body}get activeContainer(){const e=this._codeEditorService.getFocusedCodeEditor()??this._codeEditorService.getActiveCodeEditor();return(e==null?void 0:e.getContainerDomNode())??this.mainContainer}get mainContainerDimension(){return Hh(this.mainContainer)}get activeContainerDimension(){return Hh(this.activeContainer)}get containers(){return Kr(this._codeEditorService.listCodeEditors().map(e=>e.getContainerDomNode()))}getContainer(){return this.activeContainer}whenContainerStylesLoaded(){}focus(){var e;(e=this._codeEditorService.getFocusedCodeEditor())==null||e.focus()}constructor(e){this._codeEditorService=e,this.onDidLayoutMainContainer=oe.None,this.onDidLayoutActiveContainer=oe.None,this.onDidLayoutContainer=oe.None,this.onDidChangeActiveContainer=oe.None,this.onDidAddContainer=oe.None,this.mainContainerOffset={top:0,quickPickTop:0},this.activeContainerOffset={top:0,quickPickTop:0}}};oL=m$([p$(0,pt)],oL);let JM=class extends oL{get mainContainer(){return this._container}constructor(e,t){super(t),this._container=e}};JM=m$([p$(1,pt)],JM);st(tu,oL,1);var fw;(function(o){o[o.Ignore=0]="Ignore",o[o.Info=1]="Info",o[o.Warning=2]="Warning",o[o.Error=3]="Error"})(fw||(fw={}));(function(o){const e="error",t="warning",i="warn",n="info",s="ignore";function r(l){return l?ep(e,l)?o.Error:ep(t,l)||ep(i,l)?o.Warning:ep(n,l)?o.Info:o.Ignore:o.Ignore}o.fromValue=r;function a(l){switch(l){case o.Error:return e;case o.Warning:return t;case o.Info:return n;default:return s}}o.toString=a})(fw||(fw={}));const Ii=fw,H1=et("dialogService");var OI=Ii;const Ei=et("notificationService");class Qse{}const y5=et("undoRedoService");class _${constructor(e,t){this.resource=e,this.elements=t}}const Ip=class Ip{constructor(){this.id=Ip._ID++,this.order=1}nextOrder(){return this.id===0?0:this.order++}};Ip._ID=0,Ip.None=new Ip;let e2=Ip;const Ep=class Ep{constructor(){this.id=Ep._ID++,this.order=1}nextOrder(){return this.id===0?0:this.order++}};Ep._ID=0,Ep.None=new Ep;let $u=Ep;var Jse=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},r9=function(o,e){return function(t,i){e(t,i,o)}};function PS(o){return o.scheme===Fe.file?o.fsPath:o.path}let b$=0;class OS{constructor(e,t,i,n,s,r,a){this.id=++b$,this.type=0,this.actual=e,this.label=e.label,this.confirmBeforeUndo=e.confirmBeforeUndo||!1,this.resourceLabel=t,this.strResource=i,this.resourceLabels=[this.resourceLabel],this.strResources=[this.strResource],this.groupId=n,this.groupOrder=s,this.sourceId=r,this.sourceOrder=a,this.isValid=!0}setValid(e){this.isValid=e}toString(){return`[id:${this.id}] [group:${this.groupId}] [${this.isValid?"  VALID":"INVALID"}] ${this.actual.constructor.name} - ${this.actual}`}}class a9{constructor(e,t){this.resourceLabel=e,this.reason=t}}class l9{constructor(){this.elements=new Map}createMessage(){const e=[],t=[];for(const[,n]of this.elements)(n.reason===0?e:t).push(n.resourceLabel);const i=[];return e.length>0&&i.push(m({key:"externalRemoval",comment:["{0} is a list of filenames"]},"The following files have been closed and modified on disk: {0}.",e.join(", "))),t.length>0&&i.push(m({key:"noParallelUniverses",comment:["{0} is a list of filenames"]},"The following files have been modified in an incompatible way: {0}.",t.join(", "))),i.join(`
`)}get size(){return this.elements.size}has(e){return this.elements.has(e)}set(e,t){this.elements.set(e,t)}delete(e){return this.elements.delete(e)}}class eoe{constructor(e,t,i,n,s,r,a){this.id=++b$,this.type=1,this.actual=e,this.label=e.label,this.confirmBeforeUndo=e.confirmBeforeUndo||!1,this.resourceLabels=t,this.strResources=i,this.groupId=n,this.groupOrder=s,this.sourceId=r,this.sourceOrder=a,this.removedResources=null,this.invalidatedResources=null}canSplit(){return typeof this.actual.split=="function"}removeResource(e,t,i){this.removedResources||(this.removedResources=new l9),this.removedResources.has(t)||this.removedResources.set(t,new a9(e,i))}setValid(e,t,i){i?this.invalidatedResources&&(this.invalidatedResources.delete(t),this.invalidatedResources.size===0&&(this.invalidatedResources=null)):(this.invalidatedResources||(this.invalidatedResources=new l9),this.invalidatedResources.has(t)||this.invalidatedResources.set(t,new a9(e,0)))}toString(){return`[id:${this.id}] [group:${this.groupId}] [${this.invalidatedResources?"INVALID":"  VALID"}] ${this.actual.constructor.name} - ${this.actual}`}}class C${constructor(e,t){this.resourceLabel=e,this.strResource=t,this._past=[],this._future=[],this.locked=!1,this.versionId=1}dispose(){for(const e of this._past)e.type===1&&e.removeResource(this.resourceLabel,this.strResource,0);for(const e of this._future)e.type===1&&e.removeResource(this.resourceLabel,this.strResource,0);this.versionId++}toString(){const e=[];e.push(`* ${this.strResource}:`);for(let t=0;t<this._past.length;t++)e.push(`   * [UNDO] ${this._past[t]}`);for(let t=this._future.length-1;t>=0;t--)e.push(`   * [REDO] ${this._future[t]}`);return e.join(`
`)}flushAllElements(){this._past=[],this._future=[],this.versionId++}_setElementValidFlag(e,t){e.type===1?e.setValid(this.resourceLabel,this.strResource,t):e.setValid(t)}setElementsValidFlag(e,t){for(const i of this._past)t(i.actual)&&this._setElementValidFlag(i,e);for(const i of this._future)t(i.actual)&&this._setElementValidFlag(i,e)}pushElement(e){for(const t of this._future)t.type===1&&t.removeResource(this.resourceLabel,this.strResource,1);this._future=[],this._past.push(e),this.versionId++}createSnapshot(e){const t=[];for(let i=0,n=this._past.length;i<n;i++)t.push(this._past[i].id);for(let i=this._future.length-1;i>=0;i--)t.push(this._future[i].id);return new _$(e,t)}restoreSnapshot(e){const t=e.elements.length;let i=!0,n=0,s=-1;for(let a=0,l=this._past.length;a<l;a++,n++){const c=this._past[a];i&&(n>=t||c.id!==e.elements[n])&&(i=!1,s=0),!i&&c.type===1&&c.removeResource(this.resourceLabel,this.strResource,0)}let r=-1;for(let a=this._future.length-1;a>=0;a--,n++){const l=this._future[a];i&&(n>=t||l.id!==e.elements[n])&&(i=!1,r=a),!i&&l.type===1&&l.removeResource(this.resourceLabel,this.strResource,0)}s!==-1&&(this._past=this._past.slice(0,s)),r!==-1&&(this._future=this._future.slice(r+1)),this.versionId++}getElements(){const e=[],t=[];for(const i of this._past)e.push(i.actual);for(const i of this._future)t.push(i.actual);return{past:e,future:t}}getClosestPastElement(){return this._past.length===0?null:this._past[this._past.length-1]}getSecondClosestPastElement(){return this._past.length<2?null:this._past[this._past.length-2]}getClosestFutureElement(){return this._future.length===0?null:this._future[this._future.length-1]}hasPastElements(){return this._past.length>0}hasFutureElements(){return this._future.length>0}splitPastWorkspaceElement(e,t){for(let i=this._past.length-1;i>=0;i--)if(this._past[i]===e){t.has(this.strResource)?this._past[i]=t.get(this.strResource):this._past.splice(i,1);break}this.versionId++}splitFutureWorkspaceElement(e,t){for(let i=this._future.length-1;i>=0;i--)if(this._future[i]===e){t.has(this.strResource)?this._future[i]=t.get(this.strResource):this._future.splice(i,1);break}this.versionId++}moveBackward(e){this._past.pop(),this._future.push(e),this.versionId++}moveForward(e){this._future.pop(),this._past.push(e),this.versionId++}}class MN{constructor(e){this.editStacks=e,this._versionIds=[];for(let t=0,i=this.editStacks.length;t<i;t++)this._versionIds[t]=this.editStacks[t].versionId}isValid(){for(let e=0,t=this.editStacks.length;e<t;e++)if(this._versionIds[e]!==this.editStacks[e].versionId)return!1;return!0}}const v$=new C$("","");v$.locked=!0;let t2=class{constructor(e,t){this._dialogService=e,this._notificationService=t,this._editStacks=new Map,this._uriComparisonKeyComputers=[]}getUriComparisonKey(e){for(const t of this._uriComparisonKeyComputers)if(t[0]===e.scheme)return t[1].getComparisonKey(e);return e.toString()}_print(e){console.log("------------------------------------"),console.log(`AFTER ${e}: `);const t=[];for(const i of this._editStacks)t.push(i[1].toString());console.log(t.join(`
`))}pushElement(e,t=e2.None,i=$u.None){if(e.type===0){const n=PS(e.resource),s=this.getUriComparisonKey(e.resource);this._pushElement(new OS(e,n,s,t.id,t.nextOrder(),i.id,i.nextOrder()))}else{const n=new Set,s=[],r=[];for(const a of e.resources){const l=PS(a),c=this.getUriComparisonKey(a);n.has(c)||(n.add(c),s.push(l),r.push(c))}s.length===1?this._pushElement(new OS(e,s[0],r[0],t.id,t.nextOrder(),i.id,i.nextOrder())):this._pushElement(new eoe(e,s,r,t.id,t.nextOrder(),i.id,i.nextOrder()))}}_pushElement(e){for(let t=0,i=e.strResources.length;t<i;t++){const n=e.resourceLabels[t],s=e.strResources[t];let r;this._editStacks.has(s)?r=this._editStacks.get(s):(r=new C$(n,s),this._editStacks.set(s,r)),r.pushElement(e)}}getLastElement(e){const t=this.getUriComparisonKey(e);if(this._editStacks.has(t)){const i=this._editStacks.get(t);if(i.hasFutureElements())return null;const n=i.getClosestPastElement();return n?n.actual:null}return null}_splitPastWorkspaceElement(e,t){const i=e.actual.split(),n=new Map;for(const s of i){const r=PS(s.resource),a=this.getUriComparisonKey(s.resource),l=new OS(s,r,a,0,0,0,0);n.set(l.strResource,l)}for(const s of e.strResources){if(t&&t.has(s))continue;this._editStacks.get(s).splitPastWorkspaceElement(e,n)}}_splitFutureWorkspaceElement(e,t){const i=e.actual.split(),n=new Map;for(const s of i){const r=PS(s.resource),a=this.getUriComparisonKey(s.resource),l=new OS(s,r,a,0,0,0,0);n.set(l.strResource,l)}for(const s of e.strResources){if(t&&t.has(s))continue;this._editStacks.get(s).splitFutureWorkspaceElement(e,n)}}removeElements(e){const t=typeof e=="string"?e:this.getUriComparisonKey(e);this._editStacks.has(t)&&(this._editStacks.get(t).dispose(),this._editStacks.delete(t))}setElementsValidFlag(e,t,i){const n=this.getUriComparisonKey(e);this._editStacks.has(n)&&this._editStacks.get(n).setElementsValidFlag(t,i)}createSnapshot(e){const t=this.getUriComparisonKey(e);return this._editStacks.has(t)?this._editStacks.get(t).createSnapshot(e):new _$(e,[])}restoreSnapshot(e){const t=this.getUriComparisonKey(e.resource);if(this._editStacks.has(t)){const i=this._editStacks.get(t);i.restoreSnapshot(e),!i.hasPastElements()&&!i.hasFutureElements()&&(i.dispose(),this._editStacks.delete(t))}}getElements(e){const t=this.getUriComparisonKey(e);return this._editStacks.has(t)?this._editStacks.get(t).getElements():{past:[],future:[]}}_findClosestUndoElementWithSource(e){if(!e)return[null,null];let t=null,i=null;for(const[n,s]of this._editStacks){const r=s.getClosestPastElement();r&&r.sourceId===e&&(!t||r.sourceOrder>t.sourceOrder)&&(t=r,i=n)}return[t,i]}canUndo(e){if(e instanceof $u){const[,i]=this._findClosestUndoElementWithSource(e.id);return!!i}const t=this.getUriComparisonKey(e);return this._editStacks.has(t)?this._editStacks.get(t).hasPastElements():!1}_onError(e,t){Oe(e);for(const i of t.strResources)this.removeElements(i);this._notificationService.error(e)}_acquireLocks(e){for(const t of e.editStacks)if(t.locked)throw new Error("Cannot acquire edit stack lock");for(const t of e.editStacks)t.locked=!0;return()=>{for(const t of e.editStacks)t.locked=!1}}_safeInvokeWithLocks(e,t,i,n,s){const r=this._acquireLocks(i);let a;try{a=t()}catch(l){return r(),n.dispose(),this._onError(l,e)}return a?a.then(()=>(r(),n.dispose(),s()),l=>(r(),n.dispose(),this._onError(l,e))):(r(),n.dispose(),s())}async _invokeWorkspacePrepare(e){if(typeof e.actual.prepareUndoRedo>"u")return F.None;const t=e.actual.prepareUndoRedo();return typeof t>"u"?F.None:t}_invokeResourcePrepare(e,t){if(e.actual.type!==1||typeof e.actual.prepareUndoRedo>"u")return t(F.None);const i=e.actual.prepareUndoRedo();return i?uI(i)?t(i):i.then(n=>t(n)):t(F.None)}_getAffectedEditStacks(e){const t=[];for(const i of e.strResources)t.push(this._editStacks.get(i)||v$);return new MN(t)}_tryToSplitAndUndo(e,t,i,n){if(t.canSplit())return this._splitPastWorkspaceElement(t,i),this._notificationService.warn(n),new FS(this._undo(e,0,!0));for(const s of t.strResources)this.removeElements(s);return this._notificationService.warn(n),new FS}_checkWorkspaceUndo(e,t,i,n){if(t.removedResources)return this._tryToSplitAndUndo(e,t,t.removedResources,m({key:"cannotWorkspaceUndo",comment:["{0} is a label for an operation. {1} is another message."]},"Could not undo '{0}' across all files. {1}",t.label,t.removedResources.createMessage()));if(n&&t.invalidatedResources)return this._tryToSplitAndUndo(e,t,t.invalidatedResources,m({key:"cannotWorkspaceUndo",comment:["{0} is a label for an operation. {1} is another message."]},"Could not undo '{0}' across all files. {1}",t.label,t.invalidatedResources.createMessage()));const s=[];for(const a of i.editStacks)a.getClosestPastElement()!==t&&s.push(a.resourceLabel);if(s.length>0)return this._tryToSplitAndUndo(e,t,null,m({key:"cannotWorkspaceUndoDueToChanges",comment:["{0} is a label for an operation. {1} is a list of filenames."]},"Could not undo '{0}' across all files because changes were made to {1}",t.label,s.join(", ")));const r=[];for(const a of i.editStacks)a.locked&&r.push(a.resourceLabel);return r.length>0?this._tryToSplitAndUndo(e,t,null,m({key:"cannotWorkspaceUndoDueToInProgressUndoRedo",comment:["{0} is a label for an operation. {1} is a list of filenames."]},"Could not undo '{0}' across all files because there is already an undo or redo operation running on {1}",t.label,r.join(", "))):i.isValid()?null:this._tryToSplitAndUndo(e,t,null,m({key:"cannotWorkspaceUndoDueToInMeantimeUndoRedo",comment:["{0} is a label for an operation. {1} is a list of filenames."]},"Could not undo '{0}' across all files because an undo or redo operation occurred in the meantime",t.label))}_workspaceUndo(e,t,i){const n=this._getAffectedEditStacks(t),s=this._checkWorkspaceUndo(e,t,n,!1);return s?s.returnValue:this._confirmAndExecuteWorkspaceUndo(e,t,n,i)}_isPartOfUndoGroup(e){if(!e.groupId)return!1;for(const[,t]of this._editStacks){const i=t.getClosestPastElement();if(i){if(i===e){const n=t.getSecondClosestPastElement();if(n&&n.groupId===e.groupId)return!0}if(i.groupId===e.groupId)return!0}}return!1}async _confirmAndExecuteWorkspaceUndo(e,t,i,n){if(t.canSplit()&&!this._isPartOfUndoGroup(t)){let a;(function(d){d[d.All=0]="All",d[d.This=1]="This",d[d.Cancel=2]="Cancel"})(a||(a={}));const{result:l}=await this._dialogService.prompt({type:Ii.Info,message:m("confirmWorkspace","Would you like to undo '{0}' across all files?",t.label),buttons:[{label:m({key:"ok",comment:["{0} denotes a number that is > 1, && denotes a mnemonic"]},"&&Undo in {0} Files",i.editStacks.length),run:()=>a.All},{label:m({key:"nok",comment:["&& denotes a mnemonic"]},"Undo this &&File"),run:()=>a.This}],cancelButton:{run:()=>a.Cancel}});if(l===a.Cancel)return;if(l===a.This)return this._splitPastWorkspaceElement(t,null),this._undo(e,0,!0);const c=this._checkWorkspaceUndo(e,t,i,!1);if(c)return c.returnValue;n=!0}let s;try{s=await this._invokeWorkspacePrepare(t)}catch(a){return this._onError(a,t)}const r=this._checkWorkspaceUndo(e,t,i,!0);if(r)return s.dispose(),r.returnValue;for(const a of i.editStacks)a.moveBackward(t);return this._safeInvokeWithLocks(t,()=>t.actual.undo(),i,s,()=>this._continueUndoInGroup(t.groupId,n))}_resourceUndo(e,t,i){if(!t.isValid){e.flushAllElements();return}if(e.locked){const n=m({key:"cannotResourceUndoDueToInProgressUndoRedo",comment:["{0} is a label for an operation."]},"Could not undo '{0}' because there is already an undo or redo operation running.",t.label);this._notificationService.warn(n);return}return this._invokeResourcePrepare(t,n=>(e.moveBackward(t),this._safeInvokeWithLocks(t,()=>t.actual.undo(),new MN([e]),n,()=>this._continueUndoInGroup(t.groupId,i))))}_findClosestUndoElementInGroup(e){if(!e)return[null,null];let t=null,i=null;for(const[n,s]of this._editStacks){const r=s.getClosestPastElement();r&&r.groupId===e&&(!t||r.groupOrder>t.groupOrder)&&(t=r,i=n)}return[t,i]}_continueUndoInGroup(e,t){if(!e)return;const[,i]=this._findClosestUndoElementInGroup(e);if(i)return this._undo(i,0,t)}undo(e){if(e instanceof $u){const[,t]=this._findClosestUndoElementWithSource(e.id);return t?this._undo(t,e.id,!1):void 0}return typeof e=="string"?this._undo(e,0,!1):this._undo(this.getUriComparisonKey(e),0,!1)}_undo(e,t=0,i){if(!this._editStacks.has(e))return;const n=this._editStacks.get(e),s=n.getClosestPastElement();if(!s)return;if(s.groupId){const[a,l]=this._findClosestUndoElementInGroup(s.groupId);if(s!==a&&l)return this._undo(l,t,i)}if((s.sourceId!==t||s.confirmBeforeUndo)&&!i)return this._confirmAndContinueUndo(e,t,s);try{return s.type===1?this._workspaceUndo(e,s,i):this._resourceUndo(n,s,i)}finally{}}async _confirmAndContinueUndo(e,t,i){if((await this._dialogService.confirm({message:m("confirmDifferentSource","Would you like to undo '{0}'?",i.label),primaryButton:m({key:"confirmDifferentSource.yes",comment:["&& denotes a mnemonic"]},"&&Yes"),cancelButton:m("confirmDifferentSource.no","No")})).confirmed)return this._undo(e,t,!0)}_findClosestRedoElementWithSource(e){if(!e)return[null,null];let t=null,i=null;for(const[n,s]of this._editStacks){const r=s.getClosestFutureElement();r&&r.sourceId===e&&(!t||r.sourceOrder<t.sourceOrder)&&(t=r,i=n)}return[t,i]}canRedo(e){if(e instanceof $u){const[,i]=this._findClosestRedoElementWithSource(e.id);return!!i}const t=this.getUriComparisonKey(e);return this._editStacks.has(t)?this._editStacks.get(t).hasFutureElements():!1}_tryToSplitAndRedo(e,t,i,n){if(t.canSplit())return this._splitFutureWorkspaceElement(t,i),this._notificationService.warn(n),new FS(this._redo(e));for(const s of t.strResources)this.removeElements(s);return this._notificationService.warn(n),new FS}_checkWorkspaceRedo(e,t,i,n){if(t.removedResources)return this._tryToSplitAndRedo(e,t,t.removedResources,m({key:"cannotWorkspaceRedo",comment:["{0} is a label for an operation. {1} is another message."]},"Could not redo '{0}' across all files. {1}",t.label,t.removedResources.createMessage()));if(n&&t.invalidatedResources)return this._tryToSplitAndRedo(e,t,t.invalidatedResources,m({key:"cannotWorkspaceRedo",comment:["{0} is a label for an operation. {1} is another message."]},"Could not redo '{0}' across all files. {1}",t.label,t.invalidatedResources.createMessage()));const s=[];for(const a of i.editStacks)a.getClosestFutureElement()!==t&&s.push(a.resourceLabel);if(s.length>0)return this._tryToSplitAndRedo(e,t,null,m({key:"cannotWorkspaceRedoDueToChanges",comment:["{0} is a label for an operation. {1} is a list of filenames."]},"Could not redo '{0}' across all files because changes were made to {1}",t.label,s.join(", ")));const r=[];for(const a of i.editStacks)a.locked&&r.push(a.resourceLabel);return r.length>0?this._tryToSplitAndRedo(e,t,null,m({key:"cannotWorkspaceRedoDueToInProgressUndoRedo",comment:["{0} is a label for an operation. {1} is a list of filenames."]},"Could not redo '{0}' across all files because there is already an undo or redo operation running on {1}",t.label,r.join(", "))):i.isValid()?null:this._tryToSplitAndRedo(e,t,null,m({key:"cannotWorkspaceRedoDueToInMeantimeUndoRedo",comment:["{0} is a label for an operation. {1} is a list of filenames."]},"Could not redo '{0}' across all files because an undo or redo operation occurred in the meantime",t.label))}_workspaceRedo(e,t){const i=this._getAffectedEditStacks(t),n=this._checkWorkspaceRedo(e,t,i,!1);return n?n.returnValue:this._executeWorkspaceRedo(e,t,i)}async _executeWorkspaceRedo(e,t,i){let n;try{n=await this._invokeWorkspacePrepare(t)}catch(r){return this._onError(r,t)}const s=this._checkWorkspaceRedo(e,t,i,!0);if(s)return n.dispose(),s.returnValue;for(const r of i.editStacks)r.moveForward(t);return this._safeInvokeWithLocks(t,()=>t.actual.redo(),i,n,()=>this._continueRedoInGroup(t.groupId))}_resourceRedo(e,t){if(!t.isValid){e.flushAllElements();return}if(e.locked){const i=m({key:"cannotResourceRedoDueToInProgressUndoRedo",comment:["{0} is a label for an operation."]},"Could not redo '{0}' because there is already an undo or redo operation running.",t.label);this._notificationService.warn(i);return}return this._invokeResourcePrepare(t,i=>(e.moveForward(t),this._safeInvokeWithLocks(t,()=>t.actual.redo(),new MN([e]),i,()=>this._continueRedoInGroup(t.groupId))))}_findClosestRedoElementInGroup(e){if(!e)return[null,null];let t=null,i=null;for(const[n,s]of this._editStacks){const r=s.getClosestFutureElement();r&&r.groupId===e&&(!t||r.groupOrder<t.groupOrder)&&(t=r,i=n)}return[t,i]}_continueRedoInGroup(e){if(!e)return;const[,t]=this._findClosestRedoElementInGroup(e);if(t)return this._redo(t)}redo(e){if(e instanceof $u){const[,t]=this._findClosestRedoElementWithSource(e.id);return t?this._redo(t):void 0}return typeof e=="string"?this._redo(e):this._redo(this.getUriComparisonKey(e))}_redo(e){if(!this._editStacks.has(e))return;const t=this._editStacks.get(e),i=t.getClosestFutureElement();if(i){if(i.groupId){const[n,s]=this._findClosestRedoElementInGroup(i.groupId);if(i!==n&&s)return this._redo(s)}try{return i.type===1?this._workspaceRedo(e,i):this._resourceRedo(t,i)}finally{}}}};t2=Jse([r9(0,H1),r9(1,Ei)],t2);class FS{constructor(e){this.returnValue=e}}st(y5,t2,1);function os(o,e,t){return Math.min(Math.max(o,e),t)}class w${constructor(){this._n=1,this._val=0}update(e){return this._val=this._val+(e-this._val)/this._n,this._n+=1,this._val}get value(){return this._val}}class toe{constructor(e){this._n=0,this._val=0,this._values=[],this._index=0,this._sum=0,this._values=new Array(e),this._values.fill(0,0,e)}update(e){const t=this._values[this._index];return this._values[this._index]=e,this._index=(this._index+1)%this._values.length,this._sum-=t,this._sum+=e,this._n<this._values.length&&(this._n+=1),this._val=this._sum/this._n,this._val}get value(){return this._val}}const L5=et("environmentService");var ioe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},c9=function(o,e){return function(t,i){e(t,i,o)}};const Zo=et("ILanguageFeatureDebounceService");var rL;(function(o){const e=new WeakMap;let t=0;function i(n){let s=e.get(n);return s===void 0&&(s=++t,e.set(n,s)),s}o.of=i})(rL||(rL={}));class noe{constructor(e){this._default=e}get(e){return this._default}update(e,t){return this._default}default(){return this._default}}class soe{constructor(e,t,i,n,s,r){this._logService=e,this._name=t,this._registry=i,this._default=n,this._min=s,this._max=r,this._cache=new Gl(50,.7)}_key(e){return e.id+this._registry.all(e).reduce((t,i)=>bI(rL.of(i),t),0)}get(e){const t=this._key(e),i=this._cache.get(t);return i?os(i.value,this._min,this._max):this.default()}update(e,t){const i=this._key(e);let n=this._cache.get(i);n||(n=new toe(6),this._cache.set(i,n));const s=os(n.update(t),this._min,this._max);return mI(e.uri,"output")||this._logService.trace(`[DEBOUNCE: ${this._name}] for ${e.uri.toString()} is ${s}ms`),s}_overall(){const e=new w$;for(const[,t]of this._cache)e.update(t.value);return e.value}default(){const e=this._overall()|0||this._default;return os(e,this._min,this._max)}}let i2=class{constructor(e,t){this._logService=e,this._data=new Map,this._isDev=t.isExtensionDevelopment||!t.isBuilt}for(e,t,i){const n=(i==null?void 0:i.min)??50,s=(i==null?void 0:i.max)??n**2,r=(i==null?void 0:i.key)??void 0,a=`${rL.of(e)},${n}${r?","+r:""}`;let l=this._data.get(a);return l||(this._isDev?(this._logService.debug(`[DEBOUNCE: ${t}] is disabled in developed mode`),l=new noe(n*1.5)):l=new soe(this._logService,t,e,this._overallAverage()|0||n*1.5,n,s),this._data.set(a,l)),l}_overallAverage(){const e=new w$;for(const t of this._data.values())e.update(t.default());return e.value}};i2=ioe([c9(0,Nn),c9(1,L5)],i2);st(Zo,i2,1);class Ts{static getLanguageId(e){return(e&255)>>>0}static getTokenType(e){return(e&768)>>>8}static containsBalancedBrackets(e){return(e&1024)!==0}static getFontStyle(e){return(e&30720)>>>11}static getForeground(e){return(e&16744448)>>>15}static getBackground(e){return(e&4278190080)>>>24}static getClassNameFromMetadata(e){let i="mtk"+this.getForeground(e);const n=this.getFontStyle(e);return n&1&&(i+=" mtki"),n&2&&(i+=" mtkb"),n&4&&(i+=" mtku"),n&8&&(i+=" mtks"),i}static getInlineStyleFromMetadata(e,t){const i=this.getForeground(e),n=this.getFontStyle(e);let s=`color: ${t[i]};`;n&1&&(s+="font-style: italic;"),n&2&&(s+="font-weight: bold;");let r="";return n&4&&(r+=" underline"),n&8&&(r+=" line-through"),r&&(s+=`text-decoration:${r};`),s}static getPresentationFromMetadata(e){const t=this.getForeground(e),i=this.getFontStyle(e);return{foreground:t,italic:!!(i&1),bold:!!(i&2),underline:!!(i&4),strikethrough:!!(i&8)}}}function zh(o){let e=0,t=0,i=0,n=0;for(let s=0,r=o.length;s<r;s++){const a=o.charCodeAt(s);a===13?(e===0&&(t=s),e++,s+1<r&&o.charCodeAt(s+1)===10?(n|=2,s++):n|=3,i=s+1):a===10&&(n|=1,e===0&&(t=s),e++,i=s+1)}return e===0&&(t=o.length),[e,t,o.length-i,n]}class ZC{static create(e,t){return new ZC(e,new aL(t))}get startLineNumber(){return this._startLineNumber}get endLineNumber(){return this._endLineNumber}constructor(e,t){this._startLineNumber=e,this._tokens=t,this._endLineNumber=this._startLineNumber+this._tokens.getMaxDeltaLine()}toString(){return this._tokens.toString(this._startLineNumber)}_updateEndLineNumber(){this._endLineNumber=this._startLineNumber+this._tokens.getMaxDeltaLine()}isEmpty(){return this._tokens.isEmpty()}getLineTokens(e){return this._startLineNumber<=e&&e<=this._endLineNumber?this._tokens.getLineTokens(e-this._startLineNumber):null}getRange(){const e=this._tokens.getRange();return e&&new x(this._startLineNumber+e.startLineNumber,e.startColumn,this._startLineNumber+e.endLineNumber,e.endColumn)}removeTokens(e){const t=e.startLineNumber-this._startLineNumber,i=e.endLineNumber-this._startLineNumber;this._startLineNumber+=this._tokens.removeTokens(t,e.startColumn-1,i,e.endColumn-1),this._updateEndLineNumber()}split(e){const t=e.startLineNumber-this._startLineNumber,i=e.endLineNumber-this._startLineNumber,[n,s,r]=this._tokens.split(t,e.startColumn-1,i,e.endColumn-1);return[new ZC(this._startLineNumber,n),new ZC(this._startLineNumber+r,s)]}applyEdit(e,t){const[i,n,s]=zh(t);this.acceptEdit(e,i,n,s,t.length>0?t.charCodeAt(0):0)}acceptEdit(e,t,i,n,s){this._acceptDeleteRange(e),this._acceptInsertText(new A(e.startLineNumber,e.startColumn),t,i,n,s),this._updateEndLineNumber()}_acceptDeleteRange(e){if(e.startLineNumber===e.endLineNumber&&e.startColumn===e.endColumn)return;const t=e.startLineNumber-this._startLineNumber,i=e.endLineNumber-this._startLineNumber;if(i<0){const s=i-t;this._startLineNumber-=s;return}const n=this._tokens.getMaxDeltaLine();if(!(t>=n+1)){if(t<0&&i>=n+1){this._startLineNumber=0,this._tokens.clear();return}if(t<0){const s=-t;this._startLineNumber-=s,this._tokens.acceptDeleteRange(e.startColumn-1,0,0,i,e.endColumn-1)}else this._tokens.acceptDeleteRange(0,t,e.startColumn-1,i,e.endColumn-1)}}_acceptInsertText(e,t,i,n,s){if(t===0&&i===0)return;const r=e.lineNumber-this._startLineNumber;if(r<0){this._startLineNumber+=t;return}const a=this._tokens.getMaxDeltaLine();r>=a+1||this._tokens.acceptInsertText(r,e.column-1,t,i,n,s)}}class aL{constructor(e){this._tokens=e,this._tokenCount=e.length/4}toString(e){const t=[];for(let i=0;i<this._tokenCount;i++)t.push(`(${this._getDeltaLine(i)+e},${this._getStartCharacter(i)}-${this._getEndCharacter(i)})`);return`[${t.join(",")}]`}getMaxDeltaLine(){const e=this._getTokenCount();return e===0?-1:this._getDeltaLine(e-1)}getRange(){const e=this._getTokenCount();if(e===0)return null;const t=this._getStartCharacter(0),i=this._getDeltaLine(e-1),n=this._getEndCharacter(e-1);return new x(0,t+1,i,n+1)}_getTokenCount(){return this._tokenCount}_getDeltaLine(e){return this._tokens[4*e]}_getStartCharacter(e){return this._tokens[4*e+1]}_getEndCharacter(e){return this._tokens[4*e+2]}isEmpty(){return this._getTokenCount()===0}getLineTokens(e){let t=0,i=this._getTokenCount()-1;for(;t<i;){const n=t+Math.floor((i-t)/2),s=this._getDeltaLine(n);if(s<e)t=n+1;else if(s>e)i=n-1;else{let r=n;for(;r>t&&this._getDeltaLine(r-1)===e;)r--;let a=n;for(;a<i&&this._getDeltaLine(a+1)===e;)a++;return new d9(this._tokens.subarray(4*r,4*a+4))}}return this._getDeltaLine(t)===e?new d9(this._tokens.subarray(4*t,4*t+4)):null}clear(){this._tokenCount=0}removeTokens(e,t,i,n){const s=this._tokens,r=this._tokenCount;let a=0,l=!1,c=0;for(let d=0;d<r;d++){const h=4*d,u=s[h],g=s[h+1],f=s[h+2],p=s[h+3];if((u>e||u===e&&f>=t)&&(u<i||u===i&&g<=n))l=!0;else{if(a===0&&(c=u),l){const _=4*a;s[_]=u-c,s[_+1]=g,s[_+2]=f,s[_+3]=p}a++}}return this._tokenCount=a,c}split(e,t,i,n){const s=this._tokens,r=this._tokenCount,a=[],l=[];let c=a,d=0,h=0;for(let u=0;u<r;u++){const g=4*u,f=s[g],p=s[g+1],_=s[g+2],b=s[g+3];if(f>e||f===e&&_>=t){if(f<i||f===i&&p<=n)continue;c!==l&&(c=l,d=0,h=f)}c[d++]=f-h,c[d++]=p,c[d++]=_,c[d++]=b}return[new aL(new Uint32Array(a)),new aL(new Uint32Array(l)),h]}acceptDeleteRange(e,t,i,n,s){const r=this._tokens,a=this._tokenCount,l=n-t;let c=0,d=!1;for(let h=0;h<a;h++){const u=4*h;let g=r[u],f=r[u+1],p=r[u+2];const _=r[u+3];if(g<t||g===t&&p<=i){c++;continue}else if(g===t&&f<i)g===n&&p>s?p-=s-i:p=i;else if(g===t&&f===i)if(g===n&&p>s)p-=s-i;else{d=!0;continue}else if(g<n||g===n&&f<s)if(g===n&&p>s)g=t,f=i,p=f+(p-s);else{d=!0;continue}else if(g>n){if(l===0&&!d){c=a;break}g-=l}else if(g===n&&f>=s)e&&g===0&&(f+=e,p+=e),g-=l,f-=s-i,p-=s-i;else throw new Error("Not possible!");const b=4*c;r[b]=g,r[b+1]=f,r[b+2]=p,r[b+3]=_,c++}this._tokenCount=c}acceptInsertText(e,t,i,n,s,r){const a=i===0&&n===1&&(r>=48&&r<=57||r>=65&&r<=90||r>=97&&r<=122),l=this._tokens,c=this._tokenCount;for(let d=0;d<c;d++){const h=4*d;let u=l[h],g=l[h+1],f=l[h+2];if(!(u<e||u===e&&f<t)){if(u===e&&f===t)if(a)f+=1;else continue;else if(u===e&&g<t&&t<f)i===0?f+=n:f=t;else{if(u===e&&g===t&&a)continue;if(u===e)if(u+=i,i===0)g+=n,f+=n;else{const p=f-g;g=s+(g-t),f=g+p}else u+=i}l[h]=u,l[h+1]=g,l[h+2]=f}}}}class d9{constructor(e){this._tokens=e}getCount(){return this._tokens.length/4}getStartCharacter(e){return this._tokens[4*e+1]}getEndCharacter(e){return this._tokens[4*e+2]}getMetadata(e){return this._tokens[4*e+3]}}var ooe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},AN=function(o,e){return function(t,i){e(t,i,o)}};let n2=class{constructor(e,t,i,n){this._legend=e,this._themeService=t,this._languageService=i,this._logService=n,this._hasWarnedOverlappingTokens=!1,this._hasWarnedInvalidLengthTokens=!1,this._hasWarnedInvalidEditStart=!1,this._hashTable=new s2}getMetadata(e,t,i){const n=this._languageService.languageIdCodec.encodeLanguageId(i),s=this._hashTable.get(e,t,n);let r;if(s)r=s.metadata;else{let a=this._legend.tokenTypes[e];const l=[];if(a){let c=t;for(let h=0;c>0&&h<this._legend.tokenModifiers.length;h++)c&1&&l.push(this._legend.tokenModifiers[h]),c=c>>1;const d=this._themeService.getColorTheme().getTokenStyleMetadata(a,l,i);if(typeof d>"u")r=2147483647;else{if(r=0,typeof d.italic<"u"){const h=(d.italic?1:0)<<11;r|=h|1}if(typeof d.bold<"u"){const h=(d.bold?2:0)<<11;r|=h|2}if(typeof d.underline<"u"){const h=(d.underline?4:0)<<11;r|=h|4}if(typeof d.strikethrough<"u"){const h=(d.strikethrough?8:0)<<11;r|=h|8}if(d.foreground){const h=d.foreground<<15;r|=h|16}r===0&&(r=2147483647)}}else r=2147483647,a="not-in-legend";this._hashTable.add(e,t,n,r)}return r}warnOverlappingSemanticTokens(e,t){this._hasWarnedOverlappingTokens||(this._hasWarnedOverlappingTokens=!0,this._logService.warn(`Overlapping semantic tokens detected at lineNumber ${e}, column ${t}`))}warnInvalidLengthSemanticTokens(e,t){this._hasWarnedInvalidLengthTokens||(this._hasWarnedInvalidLengthTokens=!0,this._logService.warn(`Semantic token with invalid length detected at lineNumber ${e}, column ${t}`))}warnInvalidEditStart(e,t,i,n,s){this._hasWarnedInvalidEditStart||(this._hasWarnedInvalidEditStart=!0,this._logService.warn(`Invalid semantic tokens edit detected (previousResultId: ${e}, resultId: ${t}) at edit #${i}: The provided start offset ${n} is outside the previous data (length ${s}).`))}};n2=ooe([AN(1,ji),AN(2,ni),AN(3,Nn)],n2);function S$(o,e,t){const i=o.data,n=o.data.length/5|0,s=Math.max(Math.ceil(n/1024),400),r=[];let a=0,l=1,c=0;for(;a<n;){const d=a;let h=Math.min(d+s,n);if(h<n){let C=h;for(;C-1>d&&i[5*C]===0;)C--;if(C-1===d){let v=h;for(;v+1<n&&i[5*v]===0;)v++;h=v}else h=C}let u=new Uint32Array((h-d)*4),g=0,f=0,p=0,_=0;for(;a<h;){const C=5*a,v=i[C],w=i[C+1],S=l+v|0,L=v===0?c+w|0:w,k=i[C+2],D=L+k|0,N=i[C+3],B=i[C+4];if(D<=L)e.warnInvalidLengthSemanticTokens(S,L+1);else if(p===S&&_>L)e.warnOverlappingSemanticTokens(S,L+1);else{const W=e.getMetadata(N,B,t);W!==2147483647&&(f===0&&(f=S),u[g]=S-f,u[g+1]=L,u[g+2]=D,u[g+3]=W,g+=4,p=S,_=D)}l=S,c=L,a++}g!==u.length&&(u=u.subarray(0,g));const b=ZC.create(f,u);r.push(b)}return r}class roe{constructor(e,t,i,n){this.tokenTypeIndex=e,this.tokenModifierSet=t,this.languageId=i,this.metadata=n,this.next=null}}const _c=class _c{constructor(){this._elementsCount=0,this._currentLengthIndex=0,this._currentLength=_c._SIZES[this._currentLengthIndex],this._growCount=Math.round(this._currentLengthIndex+1<_c._SIZES.length?2/3*this._currentLength:0),this._elements=[],_c._nullOutEntries(this._elements,this._currentLength)}static _nullOutEntries(e,t){for(let i=0;i<t;i++)e[i]=null}_hash2(e,t){return(e<<5)-e+t|0}_hashFunc(e,t,i){return this._hash2(this._hash2(e,t),i)%this._currentLength}get(e,t,i){const n=this._hashFunc(e,t,i);let s=this._elements[n];for(;s;){if(s.tokenTypeIndex===e&&s.tokenModifierSet===t&&s.languageId===i)return s;s=s.next}return null}add(e,t,i,n){if(this._elementsCount++,this._growCount!==0&&this._elementsCount>=this._growCount){const s=this._elements;this._currentLengthIndex++,this._currentLength=_c._SIZES[this._currentLengthIndex],this._growCount=Math.round(this._currentLengthIndex+1<_c._SIZES.length?2/3*this._currentLength:0),this._elements=[],_c._nullOutEntries(this._elements,this._currentLength);for(const r of s){let a=r;for(;a;){const l=a.next;a.next=null,this._add(a),a=l}}}this._add(new roe(e,t,i,n))}_add(e){const t=this._hashFunc(e.tokenTypeIndex,e.tokenModifierSet,e.languageId);e.next=this._elements[t],this._elements[t]=e}};_c._SIZES=[3,7,13,31,61,127,251,509,1021,2039,4093,8191,16381,32749,65521,131071,262139,524287,1048573,2097143];let s2=_c;const FI=et("semanticTokensStylingService");var aoe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},PN=function(o,e){return function(t,i){e(t,i,o)}};let o2=class extends F{constructor(e,t,i){super(),this._themeService=e,this._logService=t,this._languageService=i,this._caches=new WeakMap,this._register(this._themeService.onDidColorThemeChange(()=>{this._caches=new WeakMap}))}getStyling(e){return this._caches.has(e)||this._caches.set(e,new n2(e.getLegend(),this._themeService,this._languageService,this._logService)),this._caches.get(e)}};o2=aoe([PN(0,ji),PN(1,Nn),PN(2,ni)],o2);st(FI,o2,1);function Sd(o){return o===47||o===92}function y$(o){return o.replace(/[\\/]/g,Ai.sep)}function loe(o){return o.indexOf("/")===-1&&(o=y$(o)),/^[a-zA-Z]:(\/|$)/.test(o)&&(o="/"+o),o}function h9(o,e=Ai.sep){if(!o)return"";const t=o.length,i=o.charCodeAt(0);if(Sd(i)){if(Sd(o.charCodeAt(1))&&!Sd(o.charCodeAt(2))){let s=3;const r=s;for(;s<t&&!Sd(o.charCodeAt(s));s++);if(r!==s&&!Sd(o.charCodeAt(s+1))){for(s+=1;s<t;s++)if(Sd(o.charCodeAt(s)))return o.slice(0,s+1).replace(/[\\/]/g,e)}}return e}else if(L$(i)&&o.charCodeAt(1)===58)return Sd(o.charCodeAt(2))?o.slice(0,2)+e:o.slice(0,2);let n=o.indexOf("://");if(n!==-1){for(n+=3;n<t;n++)if(Sd(o.charCodeAt(n)))return o.slice(0,n+1)}return""}function r2(o,e,t,i=Ia){if(o===e)return!0;if(!o||!e||e.length>o.length)return!1;if(t){if(!Y4(o,e))return!1;if(e.length===o.length)return!0;let s=e.length;return e.charAt(e.length-1)===i&&s--,o.charAt(s)===i}return e.charAt(e.length-1)!==i&&(e+=i),o.indexOf(e)===0}function L$(o){return o>=65&&o<=90||o>=97&&o<=122}function coe(o,e=En){return e?L$(o.charCodeAt(0))&&o.charCodeAt(1)===58:!1}const WS="**",u9="/",q0="[/\\\\]",G0="[^/\\\\]",doe=/\//g;function g9(o,e){switch(o){case 0:return"";case 1:return`${G0}*?`;default:return`(?:${q0}|${G0}+${q0}${e?`|${q0}${G0}+`:""})*?`}}function f9(o,e){if(!o)return[];const t=[];let i=!1,n=!1,s="";for(const r of o){switch(r){case e:if(!i&&!n){t.push(s),s="";continue}break;case"{":i=!0;break;case"}":i=!1;break;case"[":n=!0;break;case"]":n=!1;break}s+=r}return s&&t.push(s),t}function x$(o){if(!o)return"";let e="";const t=f9(o,u9);if(t.every(i=>i===WS))e=".*";else{let i=!1;t.forEach((n,s)=>{if(n===WS){if(i)return;e+=g9(2,s===t.length-1)}else{let r=!1,a="",l=!1,c="";for(const d of n){if(d!=="}"&&r){a+=d;continue}if(l&&(d!=="]"||!c)){let h;d==="-"?h=d:(d==="^"||d==="!")&&!c?h="^":d===u9?h="":h=Uo(d),c+=h;continue}switch(d){case"{":r=!0;continue;case"[":l=!0;continue;case"}":{const u=`(?:${f9(a,",").map(g=>x$(g)).join("|")})`;e+=u,r=!1,a="";break}case"]":{e+="["+c+"]",l=!1,c="";break}case"?":e+=G0;continue;case"*":e+=g9(1);continue;default:e+=Uo(d)}}s<t.length-1&&(t[s+1]!==WS||s+2<t.length)&&(e+=q0)}i=n===WS})}return e}const hoe=/^\*\*\/\*\.[\w\.-]+$/,uoe=/^\*\*\/([\w\.-]+)\/?$/,goe=/^{\*\*\/\*?[\w\.-]+\/?(,\*\*\/\*?[\w\.-]+\/?)*}$/,foe=/^{\*\*\/\*?[\w\.-]+(\/(\*\*)?)?(,\*\*\/\*?[\w\.-]+(\/(\*\*)?)?)*}$/,moe=/^\*\*((\/[\w\.-]+)+)\/?$/,poe=/^([\w\.-]+(\/[\w\.-]+)*)\/?$/,m9=new Gl(1e4),p9=function(){return!1},Ml=function(){return null};function x5(o,e){if(!o)return Ml;let t;typeof o!="string"?t=o.pattern:t=o,t=t.trim();const i=`${t}_${!!e.trimForExclusions}`;let n=m9.get(i);if(n)return _9(n,o);let s;return hoe.test(t)?n=_oe(t.substr(4),t):(s=uoe.exec(ON(t,e)))?n=boe(s[1],t):(e.trimForExclusions?foe:goe).test(t)?n=Coe(t,e):(s=moe.exec(ON(t,e)))?n=b9(s[1].substr(1),t,!0):(s=poe.exec(ON(t,e)))?n=b9(s[1],t,!1):n=voe(t),m9.set(i,n),_9(n,o)}function _9(o,e){if(typeof e=="string")return o;const t=function(i,n){return r2(i,e.base,!ps)?o(M1(i.substr(e.base.length),Ia),n):null};return t.allBasenames=o.allBasenames,t.allPaths=o.allPaths,t.basenames=o.basenames,t.patterns=o.patterns,t}function ON(o,e){return e.trimForExclusions&&o.endsWith("/**")?o.substr(0,o.length-2):o}function _oe(o,e){return function(t,i){return typeof t=="string"&&t.endsWith(o)?e:null}}function boe(o,e){const t=`/${o}`,i=`\\${o}`,n=function(r,a){return typeof r!="string"?null:a?a===o?e:null:r===o||r.endsWith(t)||r.endsWith(i)?e:null},s=[o];return n.basenames=s,n.patterns=[e],n.allBasenames=s,n}function Coe(o,e){const t=D$(o.slice(1,-1).split(",").map(a=>x5(a,e)).filter(a=>a!==Ml),o),i=t.length;if(!i)return Ml;if(i===1)return t[0];const n=function(a,l){for(let c=0,d=t.length;c<d;c++)if(t[c](a,l))return o;return null},s=t.find(a=>!!a.allBasenames);s&&(n.allBasenames=s.allBasenames);const r=t.reduce((a,l)=>l.allPaths?a.concat(l.allPaths):a,[]);return r.length&&(n.allPaths=r),n}function b9(o,e,t){const i=Ia===Ai.sep,n=i?o:o.replace(doe,Ia),s=Ia+n,r=Ai.sep+o;let a;return t?a=function(l,c){return typeof l=="string"&&(l===n||l.endsWith(s)||!i&&(l===o||l.endsWith(r)))?e:null}:a=function(l,c){return typeof l=="string"&&(l===n||!i&&l===o)?e:null},a.allPaths=[(t?"*/":"./")+o],a}function voe(o){try{const e=new RegExp(`^${x$(o)}$`);return function(t){return e.lastIndex=0,typeof t=="string"&&e.test(t)?o:null}}catch{return Ml}}function woe(o,e,t){return!o||typeof e!="string"?!1:k$(o)(e,void 0,t)}function k$(o,e={}){if(!o)return p9;if(typeof o=="string"||Soe(o)){const t=x5(o,e);if(t===Ml)return p9;const i=function(n,s){return!!t(n,s)};return t.allBasenames&&(i.allBasenames=t.allBasenames),t.allPaths&&(i.allPaths=t.allPaths),i}return yoe(o,e)}function Soe(o){const e=o;return e?typeof e.base=="string"&&typeof e.pattern=="string":!1}function yoe(o,e){const t=D$(Object.getOwnPropertyNames(o).map(a=>Loe(a,o[a],e)).filter(a=>a!==Ml)),i=t.length;if(!i)return Ml;if(!t.some(a=>!!a.requiresSiblings)){if(i===1)return t[0];const a=function(d,h){let u;for(let g=0,f=t.length;g<f;g++){const p=t[g](d,h);if(typeof p=="string")return p;SM(p)&&(u||(u=[]),u.push(p))}return u?(async()=>{for(const g of u){const f=await g;if(typeof f=="string")return f}return null})():null},l=t.find(d=>!!d.allBasenames);l&&(a.allBasenames=l.allBasenames);const c=t.reduce((d,h)=>h.allPaths?d.concat(h.allPaths):d,[]);return c.length&&(a.allPaths=c),a}const n=function(a,l,c){let d,h;for(let u=0,g=t.length;u<g;u++){const f=t[u];f.requiresSiblings&&c&&(l||(l=lh(a)),d||(d=l.substr(0,l.length-lee(a).length)));const p=f(a,l,d,c);if(typeof p=="string")return p;SM(p)&&(h||(h=[]),h.push(p))}return h?(async()=>{for(const u of h){const g=await u;if(typeof g=="string")return g}return null})():null},s=t.find(a=>!!a.allBasenames);s&&(n.allBasenames=s.allBasenames);const r=t.reduce((a,l)=>l.allPaths?a.concat(l.allPaths):a,[]);return r.length&&(n.allPaths=r),n}function Loe(o,e,t){if(e===!1)return Ml;const i=x5(o,t);if(i===Ml)return Ml;if(typeof e=="boolean")return i;if(e){const n=e.when;if(typeof n=="string"){const s=(r,a,l,c)=>{if(!c||!i(r,a))return null;const d=n.replace("$(basename)",()=>l),h=c(d);return SM(h)?h.then(u=>u?o:null):h?o:null};return s.requiresSiblings=!0,s}}return i}function D$(o,e){const t=o.filter(a=>!!a.basenames);if(t.length<2)return o;const i=t.reduce((a,l)=>{const c=l.basenames;return c?a.concat(c):a},[]);let n;if(e){n=[];for(let a=0,l=i.length;a<l;a++)n.push(e)}else n=t.reduce((a,l)=>{const c=l.patterns;return c?a.concat(c):a},[]);const s=function(a,l){if(typeof a!="string")return null;if(!l){let d;for(d=a.length;d>0;d--){const h=a.charCodeAt(d-1);if(h===47||h===92)break}l=a.substr(d)}const c=i.indexOf(l);return c!==-1?n[c]:null};s.basenames=i,s.patterns=n,s.allBasenames=i;const r=o.filter(a=>!a.basenames);return r.push(s),r}function k5(o,e,t,i,n,s){if(Array.isArray(o)){let r=0;for(const a of o){const l=k5(a,e,t,i,n,s);if(l===10)return l;l>r&&(r=l)}return r}else{if(typeof o=="string")return i?o==="*"?5:o===t?10:0:0;if(o){const{language:r,pattern:a,scheme:l,hasAccessToAllModels:c,notebookType:d}=o;if(!i&&!c)return 0;d&&n&&(e=n);let h=0;if(l)if(l===e.scheme)h=10;else if(l==="*")h=5;else return 0;if(r)if(r===t)h=10;else if(r==="*")h=Math.max(h,5);else return 0;if(d)if(d===s)h=10;else if(d==="*"&&s!==void 0)h=Math.max(h,5);else return 0;if(a){let u;if(typeof a=="string"?u=a:u={...a,base:Uz(a.base)},u===e.fsPath||woe(u,e.fsPath))h=10;else return 0}return h}else return 0}}function I$(o){return typeof o=="string"?!1:Array.isArray(o)?o.every(I$):!!o.exclusive}class C9{constructor(e,t,i,n,s){this.uri=e,this.languageId=t,this.notebookUri=i,this.notebookType=n,this.recursive=s}equals(e){var t,i;return this.notebookType===e.notebookType&&this.languageId===e.languageId&&this.uri.toString()===e.uri.toString()&&((t=this.notebookUri)==null?void 0:t.toString())===((i=e.notebookUri)==null?void 0:i.toString())&&this.recursive===e.recursive}}class oi{constructor(e){this._notebookInfoResolver=e,this._clock=0,this._entries=[],this._onDidChange=new P,this.onDidChange=this._onDidChange.event}register(e,t){let i={selector:e,provider:t,_score:-1,_time:this._clock++};return this._entries.push(i),this._lastCandidate=void 0,this._onDidChange.fire(this._entries.length),ve(()=>{if(i){const n=this._entries.indexOf(i);n>=0&&(this._entries.splice(n,1),this._lastCandidate=void 0,this._onDidChange.fire(this._entries.length),i=void 0)}})}has(e){return this.all(e).length>0}all(e){if(!e)return[];this._updateScores(e,!1);const t=[];for(const i of this._entries)i._score>0&&t.push(i.provider);return t}ordered(e,t=!1){const i=[];return this._orderedForEach(e,t,n=>i.push(n.provider)),i}orderedGroups(e){const t=[];let i,n;return this._orderedForEach(e,!1,s=>{i&&n===s._score?i.push(s.provider):(n=s._score,i=[s.provider],t.push(i))}),t}_orderedForEach(e,t,i){this._updateScores(e,t);for(const n of this._entries)n._score>0&&i(n)}_updateScores(e,t){var s,r;const i=(s=this._notebookInfoResolver)==null?void 0:s.call(this,e.uri),n=i?new C9(e.uri,e.getLanguageId(),i.uri,i.type,t):new C9(e.uri,e.getLanguageId(),void 0,void 0,t);if(!((r=this._lastCandidate)!=null&&r.equals(n))){this._lastCandidate=n;for(const a of this._entries)if(a._score=k5(a.selector,n.uri,n.languageId,n$(e),n.notebookUri,n.notebookType),I$(a.selector)&&a._score>0)if(t)a._score=0;else{for(const l of this._entries)l._score=0;a._score=1e3;break}this._entries.sort(oi._compareByScoreAndTime)}}static _compareByScoreAndTime(e,t){return e._score<t._score?1:e._score>t._score?-1:SC(e.selector)&&!SC(t.selector)?1:!SC(e.selector)&&SC(t.selector)?-1:e._time<t._time?1:e._time>t._time?-1:0}}function SC(o){return typeof o=="string"?!1:Array.isArray(o)?o.some(SC):!!o.isBuiltin}class xoe{constructor(){this.referenceProvider=new oi(this._score.bind(this)),this.renameProvider=new oi(this._score.bind(this)),this.newSymbolNamesProvider=new oi(this._score.bind(this)),this.codeActionProvider=new oi(this._score.bind(this)),this.definitionProvider=new oi(this._score.bind(this)),this.typeDefinitionProvider=new oi(this._score.bind(this)),this.declarationProvider=new oi(this._score.bind(this)),this.implementationProvider=new oi(this._score.bind(this)),this.documentSymbolProvider=new oi(this._score.bind(this)),this.inlayHintsProvider=new oi(this._score.bind(this)),this.colorProvider=new oi(this._score.bind(this)),this.codeLensProvider=new oi(this._score.bind(this)),this.documentFormattingEditProvider=new oi(this._score.bind(this)),this.documentRangeFormattingEditProvider=new oi(this._score.bind(this)),this.onTypeFormattingEditProvider=new oi(this._score.bind(this)),this.signatureHelpProvider=new oi(this._score.bind(this)),this.hoverProvider=new oi(this._score.bind(this)),this.documentHighlightProvider=new oi(this._score.bind(this)),this.multiDocumentHighlightProvider=new oi(this._score.bind(this)),this.selectionRangeProvider=new oi(this._score.bind(this)),this.foldingRangeProvider=new oi(this._score.bind(this)),this.linkProvider=new oi(this._score.bind(this)),this.inlineCompletionsProvider=new oi(this._score.bind(this)),this.inlineEditProvider=new oi(this._score.bind(this)),this.completionProvider=new oi(this._score.bind(this)),this.linkedEditingRangeProvider=new oi(this._score.bind(this)),this.documentRangeSemanticTokensProvider=new oi(this._score.bind(this)),this.documentSemanticTokensProvider=new oi(this._score.bind(this)),this.documentDropEditProvider=new oi(this._score.bind(this)),this.documentPasteEditProvider=new oi(this._score.bind(this))}_score(e){var t;return(t=this._notebookTypeResolver)==null?void 0:t.call(this,e)}}st(ge,xoe,1);function D5(o){return`--vscode-${o.replace(/\./g,"-")}`}function ce(o){return`var(${D5(o)})`}function koe(o,e){return`var(${D5(o)}, ${e})`}function Doe(o){return o!==null&&typeof o=="object"&&"light"in o&&"dark"in o}const E$={ColorContribution:"base.contributions.colors"},Ioe="default";class Eoe{constructor(){this._onDidChangeSchema=new P,this.onDidChangeSchema=this._onDidChangeSchema.event,this.colorSchema={type:"object",properties:{}},this.colorReferenceSchema={type:"string",enum:[],enumDescriptions:[]},this.colorsById={}}registerColor(e,t,i,n=!1,s){const r={id:e,description:i,defaults:t,needsTransparency:n,deprecationMessage:s};this.colorsById[e]=r;const a={type:"string",format:"color-hex",defaultSnippets:[{body:"${1:#ff0000}"}]};return s&&(a.deprecationMessage=s),n&&(a.pattern="^#(?:(?<rgba>[0-9a-fA-f]{3}[0-9a-eA-E])|(?:[0-9a-fA-F]{6}(?:(?![fF]{2})(?:[0-9a-fA-F]{2}))))?$",a.patternErrorMessage=m("transparecyRequired","This color must be transparent or it will obscure content")),this.colorSchema.properties[e]={description:i,oneOf:[a,{type:"string",const:Ioe,description:m("useDefault","Use the default color.")}]},this.colorReferenceSchema.enum.push(e),this.colorReferenceSchema.enumDescriptions.push(i),this._onDidChangeSchema.fire(),e}getColors(){return Object.keys(this.colorsById).map(e=>this.colorsById[e])}resolveDefaultColor(e,t){const i=this.colorsById[e];if(i!=null&&i.defaults){const n=Doe(i.defaults)?i.defaults[t.type]:i.defaults;return ra(n,t)}}getColorSchema(){return this.colorSchema}toString(){const e=(t,i)=>{const n=t.indexOf(".")===-1?0:1,s=i.indexOf(".")===-1?0:1;return n!==s?n-s:t.localeCompare(i)};return Object.keys(this.colorsById).sort(e).map(t=>`- \`${t}\`: ${this.colorsById[t].description}`).join(`
`)}}const WI=new Eoe;ci.add(E$.ColorContribution,WI);function E(o,e,t,i,n){return WI.registerColor(o,e,t,i,n)}function Noe(o,e){var t,i,n,s;switch(o.op){case 0:return(t=ra(o.value,e))==null?void 0:t.darken(o.factor);case 1:return(i=ra(o.value,e))==null?void 0:i.lighten(o.factor);case 2:return(n=ra(o.value,e))==null?void 0:n.transparent(o.factor);case 3:{const r=ra(o.background,e);return r?(s=ra(o.value,e))==null?void 0:s.makeOpaque(r):ra(o.value,e)}case 4:for(const r of o.values){const a=ra(r,e);if(a)return a}return;case 6:return ra(e.defines(o.if)?o.then:o.else,e);case 5:{const r=ra(o.value,e);if(!r)return;const a=ra(o.background,e);return a?r.isDarkerThan(a)?Z.getLighterColor(r,a,o.factor).transparent(o.transparency):Z.getDarkerColor(r,a,o.factor).transparent(o.transparency):r.transparent(o.factor*o.transparency)}default:throw NI()}}function im(o,e){return{op:0,value:o,factor:e}}function Ra(o,e){return{op:1,value:o,factor:e}}function $e(o,e){return{op:2,value:o,factor:e}}function mw(...o){return{op:4,values:o}}function Toe(o,e,t){return{op:6,if:o,then:e,else:t}}function v9(o,e,t,i){return{op:5,value:o,background:e,factor:t,transparency:i}}function ra(o,e){if(o!==null){if(typeof o=="string")return o[0]==="#"?Z.fromHex(o):e.getColor(o);if(o instanceof Z)return o;if(typeof o=="object")return Noe(o,e)}}const N$="vscode://schemas/workbench-colors",T$=ci.as(AI.JSONContribution);T$.registerSchema(N$,WI.getColorSchema());const w9=new It(()=>T$.notifySchemaChanged(N$),200);WI.onDidChangeSchema(()=>{w9.isScheduled()||w9.schedule()});const Ze=E("foreground",{dark:"#CCCCCC",light:"#616161",hcDark:"#FFFFFF",hcLight:"#292929"},m("foreground","Overall foreground color. This color is only used if not overridden by a component."));E("disabledForeground",{dark:"#CCCCCC80",light:"#61616180",hcDark:"#A5A5A5",hcLight:"#7F7F7F"},m("disabledForeground","Overall foreground for disabled elements. This color is only used if not overridden by a component."));E("errorForeground",{dark:"#F48771",light:"#A1260D",hcDark:"#F48771",hcLight:"#B5200D"},m("errorForeground","Overall foreground color for error messages. This color is only used if not overridden by a component."));E("descriptionForeground",{light:"#717171",dark:$e(Ze,.7),hcDark:$e(Ze,.7),hcLight:$e(Ze,.7)},m("descriptionForeground","Foreground color for description text providing additional information, for example for a label."));const lL=E("icon.foreground",{dark:"#C5C5C5",light:"#424242",hcDark:"#FFFFFF",hcLight:"#292929"},m("iconForeground","The default color for icons in the workbench.")),Wl=E("focusBorder",{dark:"#007FD4",light:"#0090F1",hcDark:"#F38518",hcLight:"#006BBD"},m("focusBorder","Overall border color for focused elements. This color is only used if not overridden by a component.")),lt=E("contrastBorder",{light:null,dark:null,hcDark:"#6FC3DF",hcLight:"#0F4A85"},m("contrastBorder","An extra border around elements to separate them from others for greater contrast.")),Xt=E("contrastActiveBorder",{light:null,dark:null,hcDark:Wl,hcLight:Wl},m("activeContrastBorder","An extra border around active elements to separate them from others for greater contrast."));E("selection.background",null,m("selectionBackground","The background color of text selections in the workbench (e.g. for input fields or text areas). Note that this does not apply to selections within the editor."));const Roe=E("textLink.foreground",{light:"#006AB1",dark:"#3794FF",hcDark:"#21A6FF",hcLight:"#0F4A85"},m("textLinkForeground","Foreground color for links in text."));E("textLink.activeForeground",{light:"#006AB1",dark:"#3794FF",hcDark:"#21A6FF",hcLight:"#0F4A85"},m("textLinkActiveForeground","Foreground color for links in text when clicked on and on mouse hover."));E("textSeparator.foreground",{light:"#0000002e",dark:"#ffffff2e",hcDark:Z.black,hcLight:"#292929"},m("textSeparatorForeground","Color for text separators."));E("textPreformat.foreground",{light:"#A31515",dark:"#D7BA7D",hcDark:"#000000",hcLight:"#FFFFFF"},m("textPreformatForeground","Foreground color for preformatted text segments."));E("textPreformat.background",{light:"#0000001A",dark:"#FFFFFF1A",hcDark:"#FFFFFF",hcLight:"#09345f"},m("textPreformatBackground","Background color for preformatted text segments."));E("textBlockQuote.background",{light:"#f2f2f2",dark:"#222222",hcDark:null,hcLight:"#F2F2F2"},m("textBlockQuoteBackground","Background color for block quotes in text."));E("textBlockQuote.border",{light:"#007acc80",dark:"#007acc80",hcDark:Z.white,hcLight:"#292929"},m("textBlockQuoteBorder","Border color for block quotes in text."));E("textCodeBlock.background",{light:"#dcdcdc66",dark:"#0a0a0a66",hcDark:Z.black,hcLight:"#F2F2F2"},m("textCodeBlockBackground","Background color for code blocks in text."));E("sash.hoverBorder",Wl,m("sashActiveBorder","Border color of active sashes."));const Z0=E("badge.background",{dark:"#4D4D4D",light:"#C4C4C4",hcDark:Z.black,hcLight:"#0F4A85"},m("badgeBackground","Badge background color. Badges are small information labels, e.g. for search results count.")),Moe=E("badge.foreground",{dark:Z.white,light:"#333",hcDark:Z.white,hcLight:Z.white},m("badgeForeground","Badge foreground color. Badges are small information labels, e.g. for search results count.")),I5=E("scrollbar.shadow",{dark:"#000000",light:"#DDDDDD",hcDark:null,hcLight:null},m("scrollbarShadow","Scrollbar shadow to indicate that the view is scrolled.")),R$=E("scrollbarSlider.background",{dark:Z.fromHex("#797979").transparent(.4),light:Z.fromHex("#646464").transparent(.4),hcDark:$e(lt,.6),hcLight:$e(lt,.4)},m("scrollbarSliderBackground","Scrollbar slider background color.")),M$=E("scrollbarSlider.hoverBackground",{dark:Z.fromHex("#646464").transparent(.7),light:Z.fromHex("#646464").transparent(.7),hcDark:$e(lt,.8),hcLight:$e(lt,.8)},m("scrollbarSliderHoverBackground","Scrollbar slider background color when hovering.")),A$=E("scrollbarSlider.activeBackground",{dark:Z.fromHex("#BFBFBF").transparent(.4),light:Z.fromHex("#000000").transparent(.6),hcDark:lt,hcLight:lt},m("scrollbarSliderActiveBackground","Scrollbar slider background color when clicked on.")),Aoe=E("progressBar.background",{dark:Z.fromHex("#0E70C0"),light:Z.fromHex("#0E70C0"),hcDark:lt,hcLight:lt},m("progressBarBackground","Background color of the progress bar that can show for long running operations.")),Xr=E("editor.background",{light:"#ffffff",dark:"#1E1E1E",hcDark:Z.black,hcLight:Z.white},m("editorBackground","Editor background color.")),Zl=E("editor.foreground",{light:"#333333",dark:"#BBBBBB",hcDark:Z.white,hcLight:Ze},m("editorForeground","Editor default foreground color."));E("editorStickyScroll.background",Xr,m("editorStickyScrollBackground","Background color of sticky scroll in the editor"));E("editorStickyScrollHover.background",{dark:"#2A2D2E",light:"#F0F0F0",hcDark:null,hcLight:Z.fromHex("#0F4A85").transparent(.1)},m("editorStickyScrollHoverBackground","Background color of sticky scroll on hover in the editor"));E("editorStickyScroll.border",{dark:null,light:null,hcDark:lt,hcLight:lt},m("editorStickyScrollBorder","Border color of sticky scroll in the editor"));E("editorStickyScroll.shadow",I5,m("editorStickyScrollShadow"," Shadow color of sticky scroll in the editor"));const Wo=E("editorWidget.background",{dark:"#252526",light:"#F3F3F3",hcDark:"#0C141F",hcLight:Z.white},m("editorWidgetBackground","Background color of editor widgets, such as find/replace.")),BI=E("editorWidget.foreground",Ze,m("editorWidgetForeground","Foreground color of editor widgets, such as find/replace.")),E5=E("editorWidget.border",{dark:"#454545",light:"#C8C8C8",hcDark:lt,hcLight:lt},m("editorWidgetBorder","Border color of editor widgets. The color is only used if the widget chooses to have a border and if the color is not overridden by a widget."));E("editorWidget.resizeBorder",null,m("editorWidgetResizeBorder","Border color of the resize bar of editor widgets. The color is only used if the widget chooses to have a resize border and if the color is not overridden by a widget."));E("editorError.background",null,m("editorError.background","Background color of error text in the editor. The color must not be opaque so as not to hide underlying decorations."),!0);const HI=E("editorError.foreground",{dark:"#F14C4C",light:"#E51400",hcDark:"#F48771",hcLight:"#B5200D"},m("editorError.foreground","Foreground color of error squigglies in the editor.")),Poe=E("editorError.border",{dark:null,light:null,hcDark:Z.fromHex("#E47777").transparent(.8),hcLight:"#B5200D"},m("errorBorder","If set, color of double underlines for errors in the editor.")),Ooe=E("editorWarning.background",null,m("editorWarning.background","Background color of warning text in the editor. The color must not be opaque so as not to hide underlying decorations."),!0),id=E("editorWarning.foreground",{dark:"#CCA700",light:"#BF8803",hcDark:"#FFD370",hcLight:"#895503"},m("editorWarning.foreground","Foreground color of warning squigglies in the editor.")),pw=E("editorWarning.border",{dark:null,light:null,hcDark:Z.fromHex("#FFCC00").transparent(.8),hcLight:Z.fromHex("#FFCC00").transparent(.8)},m("warningBorder","If set, color of double underlines for warnings in the editor."));E("editorInfo.background",null,m("editorInfo.background","Background color of info text in the editor. The color must not be opaque so as not to hide underlying decorations."),!0);const Bl=E("editorInfo.foreground",{dark:"#3794FF",light:"#1a85ff",hcDark:"#3794FF",hcLight:"#1a85ff"},m("editorInfo.foreground","Foreground color of info squigglies in the editor.")),_w=E("editorInfo.border",{dark:null,light:null,hcDark:Z.fromHex("#3794FF").transparent(.8),hcLight:"#292929"},m("infoBorder","If set, color of double underlines for infos in the editor.")),Foe=E("editorHint.foreground",{dark:Z.fromHex("#eeeeee").transparent(.7),light:"#6c6c6c",hcDark:null,hcLight:null},m("editorHint.foreground","Foreground color of hint squigglies in the editor."));E("editorHint.border",{dark:null,light:null,hcDark:Z.fromHex("#eeeeee").transparent(.8),hcLight:"#292929"},m("hintBorder","If set, color of double underlines for hints in the editor."));const Woe=E("editorLink.activeForeground",{dark:"#4E94CE",light:Z.blue,hcDark:Z.cyan,hcLight:"#292929"},m("activeLinkForeground","Color of active links.")),Gd=E("editor.selectionBackground",{light:"#ADD6FF",dark:"#264F78",hcDark:"#f3f518",hcLight:"#0F4A85"},m("editorSelectionBackground","Color of the editor selection.")),Boe=E("editor.selectionForeground",{light:null,dark:null,hcDark:"#000000",hcLight:Z.white},m("editorSelectionForeground","Color of the selected text for high contrast.")),P$=E("editor.inactiveSelectionBackground",{light:$e(Gd,.5),dark:$e(Gd,.5),hcDark:$e(Gd,.7),hcLight:$e(Gd,.5)},m("editorInactiveSelection","Color of the selection in an inactive editor. The color must not be opaque so as not to hide underlying decorations."),!0),N5=E("editor.selectionHighlightBackground",{light:v9(Gd,Xr,.3,.6),dark:v9(Gd,Xr,.3,.6),hcDark:null,hcLight:null},m("editorSelectionHighlight","Color for regions with the same content as the selection. The color must not be opaque so as not to hide underlying decorations."),!0);E("editor.selectionHighlightBorder",{light:null,dark:null,hcDark:Xt,hcLight:Xt},m("editorSelectionHighlightBorder","Border color for regions with the same content as the selection."));E("editor.findMatchBackground",{light:"#A8AC94",dark:"#515C6A",hcDark:null,hcLight:null},m("editorFindMatch","Color of the current search match."));const Hoe=E("editor.findMatchForeground",null,m("editorFindMatchForeground","Text color of the current search match.")),Ac=E("editor.findMatchHighlightBackground",{light:"#EA5C0055",dark:"#EA5C0055",hcDark:null,hcLight:null},m("findMatchHighlight","Color of the other search matches. The color must not be opaque so as not to hide underlying decorations."),!0),Voe=E("editor.findMatchHighlightForeground",null,m("findMatchHighlightForeground","Foreground color of the other search matches."),!0);E("editor.findRangeHighlightBackground",{dark:"#3a3d4166",light:"#b4b4b44d",hcDark:null,hcLight:null},m("findRangeHighlight","Color of the range limiting the search. The color must not be opaque so as not to hide underlying decorations."),!0);E("editor.findMatchBorder",{light:null,dark:null,hcDark:Xt,hcLight:Xt},m("editorFindMatchBorder","Border color of the current search match."));const Zd=E("editor.findMatchHighlightBorder",{light:null,dark:null,hcDark:Xt,hcLight:Xt},m("findMatchHighlightBorder","Border color of the other search matches.")),zoe=E("editor.findRangeHighlightBorder",{dark:null,light:null,hcDark:$e(Xt,.4),hcLight:$e(Xt,.4)},m("findRangeHighlightBorder","Border color of the range limiting the search. The color must not be opaque so as not to hide underlying decorations."),!0);E("editor.hoverHighlightBackground",{light:"#ADD6FF26",dark:"#264f7840",hcDark:"#ADD6FF26",hcLight:null},m("hoverHighlight","Highlight below the word for which a hover is shown. The color must not be opaque so as not to hide underlying decorations."),!0);const cL=E("editorHoverWidget.background",Wo,m("hoverBackground","Background color of the editor hover."));E("editorHoverWidget.foreground",BI,m("hoverForeground","Foreground color of the editor hover."));const O$=E("editorHoverWidget.border",E5,m("hoverBorder","Border color of the editor hover."));E("editorHoverWidget.statusBarBackground",{dark:Ra(cL,.2),light:im(cL,.05),hcDark:Wo,hcLight:Wo},m("statusBarBackground","Background color of the editor hover status bar."));const T5=E("editorInlayHint.foreground",{dark:"#969696",light:"#969696",hcDark:Z.white,hcLight:Z.black},m("editorInlayHintForeground","Foreground color of inline hints")),R5=E("editorInlayHint.background",{dark:$e(Z0,.1),light:$e(Z0,.1),hcDark:$e(Z.white,.1),hcLight:$e(Z0,.1)},m("editorInlayHintBackground","Background color of inline hints")),Uoe=E("editorInlayHint.typeForeground",T5,m("editorInlayHintForegroundTypes","Foreground color of inline hints for types")),$oe=E("editorInlayHint.typeBackground",R5,m("editorInlayHintBackgroundTypes","Background color of inline hints for types")),joe=E("editorInlayHint.parameterForeground",T5,m("editorInlayHintForegroundParameter","Foreground color of inline hints for parameters")),Koe=E("editorInlayHint.parameterBackground",R5,m("editorInlayHintBackgroundParameter","Background color of inline hints for parameters")),qoe=E("editorLightBulb.foreground",{dark:"#FFCC00",light:"#DDB100",hcDark:"#FFCC00",hcLight:"#007ACC"},m("editorLightBulbForeground","The color used for the lightbulb actions icon."));E("editorLightBulbAutoFix.foreground",{dark:"#75BEFF",light:"#007ACC",hcDark:"#75BEFF",hcLight:"#007ACC"},m("editorLightBulbAutoFixForeground","The color used for the lightbulb auto fix actions icon."));E("editorLightBulbAi.foreground",qoe,m("editorLightBulbAiForeground","The color used for the lightbulb AI icon."));E("editor.snippetTabstopHighlightBackground",{dark:new Z(new dt(124,124,124,.3)),light:new Z(new dt(10,50,100,.2)),hcDark:new Z(new dt(124,124,124,.3)),hcLight:new Z(new dt(10,50,100,.2))},m("snippetTabstopHighlightBackground","Highlight background color of a snippet tabstop."));E("editor.snippetTabstopHighlightBorder",null,m("snippetTabstopHighlightBorder","Highlight border color of a snippet tabstop."));E("editor.snippetFinalTabstopHighlightBackground",null,m("snippetFinalTabstopHighlightBackground","Highlight background color of the final tabstop of a snippet."));E("editor.snippetFinalTabstopHighlightBorder",{dark:"#525252",light:new Z(new dt(10,50,100,.5)),hcDark:"#525252",hcLight:"#292929"},m("snippetFinalTabstopHighlightBorder","Highlight border color of the final tabstop of a snippet."));const a2=new Z(new dt(155,185,85,.2)),l2=new Z(new dt(255,0,0,.2)),Goe=E("diffEditor.insertedTextBackground",{dark:"#9ccc2c33",light:"#9ccc2c40",hcDark:null,hcLight:null},m("diffEditorInserted","Background color for text that got inserted. The color must not be opaque so as not to hide underlying decorations."),!0),Zoe=E("diffEditor.removedTextBackground",{dark:"#ff000033",light:"#ff000033",hcDark:null,hcLight:null},m("diffEditorRemoved","Background color for text that got removed. The color must not be opaque so as not to hide underlying decorations."),!0);E("diffEditor.insertedLineBackground",{dark:a2,light:a2,hcDark:null,hcLight:null},m("diffEditorInsertedLines","Background color for lines that got inserted. The color must not be opaque so as not to hide underlying decorations."),!0);E("diffEditor.removedLineBackground",{dark:l2,light:l2,hcDark:null,hcLight:null},m("diffEditorRemovedLines","Background color for lines that got removed. The color must not be opaque so as not to hide underlying decorations."),!0);E("diffEditorGutter.insertedLineBackground",null,m("diffEditorInsertedLineGutter","Background color for the margin where lines got inserted."));E("diffEditorGutter.removedLineBackground",null,m("diffEditorRemovedLineGutter","Background color for the margin where lines got removed."));const Yoe=E("diffEditorOverview.insertedForeground",null,m("diffEditorOverviewInserted","Diff overview ruler foreground for inserted content.")),Xoe=E("diffEditorOverview.removedForeground",null,m("diffEditorOverviewRemoved","Diff overview ruler foreground for removed content."));E("diffEditor.insertedTextBorder",{dark:null,light:null,hcDark:"#33ff2eff",hcLight:"#374E06"},m("diffEditorInsertedOutline","Outline color for the text that got inserted."));E("diffEditor.removedTextBorder",{dark:null,light:null,hcDark:"#FF008F",hcLight:"#AD0707"},m("diffEditorRemovedOutline","Outline color for text that got removed."));E("diffEditor.border",{dark:null,light:null,hcDark:lt,hcLight:lt},m("diffEditorBorder","Border color between the two text editors."));E("diffEditor.diagonalFill",{dark:"#cccccc33",light:"#22222233",hcDark:null,hcLight:null},m("diffDiagonalFill","Color of the diff editor's diagonal fill. The diagonal fill is used in side-by-side diff views."));E("diffEditor.unchangedRegionBackground","sideBar.background",m("diffEditor.unchangedRegionBackground","The background color of unchanged blocks in the diff editor."));E("diffEditor.unchangedRegionForeground","foreground",m("diffEditor.unchangedRegionForeground","The foreground color of unchanged blocks in the diff editor."));E("diffEditor.unchangedCodeBackground",{dark:"#74747429",light:"#b8b8b829",hcDark:null,hcLight:null},m("diffEditor.unchangedCodeBackground","The background color of unchanged code in the diff editor."));const Cb=E("widget.shadow",{dark:$e(Z.black,.36),light:$e(Z.black,.16),hcDark:null,hcLight:null},m("widgetShadow","Shadow color of widgets such as find/replace inside the editor.")),F$=E("widget.border",{dark:null,light:null,hcDark:lt,hcLight:lt},m("widgetBorder","Border color of widgets such as find/replace inside the editor.")),S9=E("toolbar.hoverBackground",{dark:"#5a5d5e50",light:"#b8b8b850",hcDark:null,hcLight:null},m("toolbarHoverBackground","Toolbar background when hovering over actions using the mouse"));E("toolbar.hoverOutline",{dark:null,light:null,hcDark:Xt,hcLight:Xt},m("toolbarHoverOutline","Toolbar outline when hovering over actions using the mouse"));E("toolbar.activeBackground",{dark:Ra(S9,.1),light:im(S9,.1),hcDark:null,hcLight:null},m("toolbarActiveBackground","Toolbar background when holding the mouse over actions"));const Qoe=E("breadcrumb.foreground",$e(Ze,.8),m("breadcrumbsFocusForeground","Color of focused breadcrumb items.")),Joe=E("breadcrumb.background",Xr,m("breadcrumbsBackground","Background color of breadcrumb items.")),y9=E("breadcrumb.focusForeground",{light:im(Ze,.2),dark:Ra(Ze,.1),hcDark:Ra(Ze,.1),hcLight:Ra(Ze,.1)},m("breadcrumbsFocusForeground","Color of focused breadcrumb items.")),ere=E("breadcrumb.activeSelectionForeground",{light:im(Ze,.2),dark:Ra(Ze,.1),hcDark:Ra(Ze,.1),hcLight:Ra(Ze,.1)},m("breadcrumbsSelectedForeground","Color of selected breadcrumb items."));E("breadcrumbPicker.background",Wo,m("breadcrumbsSelectedBackground","Background color of breadcrumb item picker."));const W$=.5,L9=Z.fromHex("#40C8AE").transparent(W$),x9=Z.fromHex("#40A6FF").transparent(W$),k9=Z.fromHex("#606060").transparent(.4),M5=.4,T_=1,c2=E("merge.currentHeaderBackground",{dark:L9,light:L9,hcDark:null,hcLight:null},m("mergeCurrentHeaderBackground","Current header background in inline merge-conflicts. The color must not be opaque so as not to hide underlying decorations."),!0);E("merge.currentContentBackground",$e(c2,M5),m("mergeCurrentContentBackground","Current content background in inline merge-conflicts. The color must not be opaque so as not to hide underlying decorations."),!0);const d2=E("merge.incomingHeaderBackground",{dark:x9,light:x9,hcDark:null,hcLight:null},m("mergeIncomingHeaderBackground","Incoming header background in inline merge-conflicts. The color must not be opaque so as not to hide underlying decorations."),!0);E("merge.incomingContentBackground",$e(d2,M5),m("mergeIncomingContentBackground","Incoming content background in inline merge-conflicts. The color must not be opaque so as not to hide underlying decorations."),!0);const h2=E("merge.commonHeaderBackground",{dark:k9,light:k9,hcDark:null,hcLight:null},m("mergeCommonHeaderBackground","Common ancestor header background in inline merge-conflicts. The color must not be opaque so as not to hide underlying decorations."),!0);E("merge.commonContentBackground",$e(h2,M5),m("mergeCommonContentBackground","Common ancestor content background in inline merge-conflicts. The color must not be opaque so as not to hide underlying decorations."),!0);const R_=E("merge.border",{dark:null,light:null,hcDark:"#C3DF6F",hcLight:"#007ACC"},m("mergeBorder","Border color on headers and the splitter in inline merge-conflicts."));E("editorOverviewRuler.currentContentForeground",{dark:$e(c2,T_),light:$e(c2,T_),hcDark:R_,hcLight:R_},m("overviewRulerCurrentContentForeground","Current overview ruler foreground for inline merge-conflicts."));E("editorOverviewRuler.incomingContentForeground",{dark:$e(d2,T_),light:$e(d2,T_),hcDark:R_,hcLight:R_},m("overviewRulerIncomingContentForeground","Incoming overview ruler foreground for inline merge-conflicts."));E("editorOverviewRuler.commonContentForeground",{dark:$e(h2,T_),light:$e(h2,T_),hcDark:R_,hcLight:R_},m("overviewRulerCommonContentForeground","Common ancestor overview ruler foreground for inline merge-conflicts."));const FN=E("editorOverviewRuler.findMatchForeground",{dark:"#d186167e",light:"#d186167e",hcDark:"#AB5A00",hcLight:"#AB5A00"},m("overviewRulerFindMatchForeground","Overview ruler marker color for find matches. The color must not be opaque so as not to hide underlying decorations."),!0),B$=E("editorOverviewRuler.selectionHighlightForeground","#A0A0A0CC",m("overviewRulerSelectionHighlightForeground","Overview ruler marker color for selection highlights. The color must not be opaque so as not to hide underlying decorations."),!0),tre=E("problemsErrorIcon.foreground",HI,m("problemsErrorIconForeground","The color used for the problems error icon.")),ire=E("problemsWarningIcon.foreground",id,m("problemsWarningIconForeground","The color used for the problems warning icon.")),nre=E("problemsInfoIcon.foreground",Bl,m("problemsInfoIconForeground","The color used for the problems info icon.")),u2=E("minimap.findMatchHighlight",{light:"#d18616",dark:"#d18616",hcDark:"#AB5A00",hcLight:"#0F4A85"},m("minimapFindMatchHighlight","Minimap marker color for find matches."),!0),VI=E("minimap.selectionOccurrenceHighlight",{light:"#c9c9c9",dark:"#676767",hcDark:"#ffffff",hcLight:"#0F4A85"},m("minimapSelectionOccurrenceHighlight","Minimap marker color for repeating editor selections."),!0),D9=E("minimap.selectionHighlight",{light:"#ADD6FF",dark:"#264F78",hcDark:"#ffffff",hcLight:"#0F4A85"},m("minimapSelectionHighlight","Minimap marker color for the editor selection."),!0),sre=E("minimap.infoHighlight",{dark:Bl,light:Bl,hcDark:_w,hcLight:_w},m("minimapInfo","Minimap marker color for infos.")),ore=E("minimap.warningHighlight",{dark:id,light:id,hcDark:pw,hcLight:pw},m("overviewRuleWarning","Minimap marker color for warnings.")),rre=E("minimap.errorHighlight",{dark:new Z(new dt(255,18,18,.7)),light:new Z(new dt(255,18,18,.7)),hcDark:new Z(new dt(255,50,50,1)),hcLight:"#B5200D"},m("minimapError","Minimap marker color for errors.")),are=E("minimap.background",null,m("minimapBackground","Minimap background color.")),lre=E("minimap.foregroundOpacity",Z.fromHex("#000f"),m("minimapForegroundOpacity",'Opacity of foreground elements rendered in the minimap. For example, "#000000c0" will render the elements with 75% opacity.'));E("minimapSlider.background",$e(R$,.5),m("minimapSliderBackground","Minimap slider background color."));E("minimapSlider.hoverBackground",$e(M$,.5),m("minimapSliderHoverBackground","Minimap slider background color when hovering."));E("minimapSlider.activeBackground",$e(A$,.5),m("minimapSliderActiveBackground","Minimap slider background color when clicked on."));E("charts.foreground",Ze,m("chartsForeground","The foreground color used in charts."));E("charts.lines",$e(Ze,.5),m("chartsLines","The color used for horizontal lines in charts."));E("charts.red",HI,m("chartsRed","The red color used in chart visualizations."));E("charts.blue",Bl,m("chartsBlue","The blue color used in chart visualizations."));E("charts.yellow",id,m("chartsYellow","The yellow color used in chart visualizations."));E("charts.orange",u2,m("chartsOrange","The orange color used in chart visualizations."));E("charts.green",{dark:"#89D185",light:"#388A34",hcDark:"#89D185",hcLight:"#374e06"},m("chartsGreen","The green color used in chart visualizations."));E("charts.purple",{dark:"#B180D7",light:"#652D90",hcDark:"#B180D7",hcLight:"#652D90"},m("chartsPurple","The purple color used in chart visualizations."));const g2=E("input.background",{dark:"#3C3C3C",light:Z.white,hcDark:Z.black,hcLight:Z.white},m("inputBoxBackground","Input box background.")),H$=E("input.foreground",Ze,m("inputBoxForeground","Input box foreground.")),V$=E("input.border",{dark:null,light:null,hcDark:lt,hcLight:lt},m("inputBoxBorder","Input box border.")),zI=E("inputOption.activeBorder",{dark:"#007ACC",light:"#007ACC",hcDark:lt,hcLight:lt},m("inputBoxActiveOptionBorder","Border color of activated options in input fields.")),cre=E("inputOption.hoverBackground",{dark:"#5a5d5e80",light:"#b8b8b850",hcDark:null,hcLight:null},m("inputOption.hoverBackground","Background color of activated options in input fields.")),V1=E("inputOption.activeBackground",{dark:$e(Wl,.4),light:$e(Wl,.2),hcDark:Z.transparent,hcLight:Z.transparent},m("inputOption.activeBackground","Background hover color of options in input fields.")),UI=E("inputOption.activeForeground",{dark:Z.white,light:Z.black,hcDark:Ze,hcLight:Ze},m("inputOption.activeForeground","Foreground color of activated options in input fields."));E("input.placeholderForeground",{light:$e(Ze,.5),dark:$e(Ze,.5),hcDark:$e(Ze,.7),hcLight:$e(Ze,.7)},m("inputPlaceholderForeground","Input box foreground color for placeholder text."));const dre=E("inputValidation.infoBackground",{dark:"#063B49",light:"#D6ECF2",hcDark:Z.black,hcLight:Z.white},m("inputValidationInfoBackground","Input validation background color for information severity.")),hre=E("inputValidation.infoForeground",{dark:null,light:null,hcDark:null,hcLight:Ze},m("inputValidationInfoForeground","Input validation foreground color for information severity.")),ure=E("inputValidation.infoBorder",{dark:"#007acc",light:"#007acc",hcDark:lt,hcLight:lt},m("inputValidationInfoBorder","Input validation border color for information severity.")),gre=E("inputValidation.warningBackground",{dark:"#352A05",light:"#F6F5D2",hcDark:Z.black,hcLight:Z.white},m("inputValidationWarningBackground","Input validation background color for warning severity.")),fre=E("inputValidation.warningForeground",{dark:null,light:null,hcDark:null,hcLight:Ze},m("inputValidationWarningForeground","Input validation foreground color for warning severity.")),mre=E("inputValidation.warningBorder",{dark:"#B89500",light:"#B89500",hcDark:lt,hcLight:lt},m("inputValidationWarningBorder","Input validation border color for warning severity.")),pre=E("inputValidation.errorBackground",{dark:"#5A1D1D",light:"#F2DEDE",hcDark:Z.black,hcLight:Z.white},m("inputValidationErrorBackground","Input validation background color for error severity.")),_re=E("inputValidation.errorForeground",{dark:null,light:null,hcDark:null,hcLight:Ze},m("inputValidationErrorForeground","Input validation foreground color for error severity.")),bre=E("inputValidation.errorBorder",{dark:"#BE1100",light:"#BE1100",hcDark:lt,hcLight:lt},m("inputValidationErrorBorder","Input validation border color for error severity.")),$I=E("dropdown.background",{dark:"#3C3C3C",light:Z.white,hcDark:Z.black,hcLight:Z.white},m("dropdownBackground","Dropdown background.")),Cre=E("dropdown.listBackground",{dark:null,light:null,hcDark:Z.black,hcLight:Z.white},m("dropdownListBackground","Dropdown list background.")),A5=E("dropdown.foreground",{dark:"#F0F0F0",light:Ze,hcDark:Z.white,hcLight:Ze},m("dropdownForeground","Dropdown foreground.")),P5=E("dropdown.border",{dark:$I,light:"#CECECE",hcDark:lt,hcLight:lt},m("dropdownBorder","Dropdown border.")),z$=E("button.foreground",Z.white,m("buttonForeground","Button foreground color.")),vre=E("button.separator",$e(z$,.4),m("buttonSeparator","Button separator color.")),yC=E("button.background",{dark:"#0E639C",light:"#007ACC",hcDark:null,hcLight:"#0F4A85"},m("buttonBackground","Button background color.")),wre=E("button.hoverBackground",{dark:Ra(yC,.2),light:im(yC,.2),hcDark:yC,hcLight:yC},m("buttonHoverBackground","Button background color when hovering.")),Sre=E("button.border",lt,m("buttonBorder","Button border color.")),yre=E("button.secondaryForeground",{dark:Z.white,light:Z.white,hcDark:Z.white,hcLight:Ze},m("buttonSecondaryForeground","Secondary button foreground color.")),f2=E("button.secondaryBackground",{dark:"#3A3D41",light:"#5F6A79",hcDark:null,hcLight:Z.white},m("buttonSecondaryBackground","Secondary button background color.")),Lre=E("button.secondaryHoverBackground",{dark:Ra(f2,.2),light:im(f2,.2),hcDark:null,hcLight:null},m("buttonSecondaryHoverBackground","Secondary button background color when hovering.")),LC=E("radio.activeForeground",UI,m("radioActiveForeground","Foreground color of active radio option.")),xre=E("radio.activeBackground",V1,m("radioBackground","Background color of active radio option.")),kre=E("radio.activeBorder",zI,m("radioActiveBorder","Border color of the active radio option.")),Dre=E("radio.inactiveForeground",null,m("radioInactiveForeground","Foreground color of inactive radio option.")),Ire=E("radio.inactiveBackground",null,m("radioInactiveBackground","Background color of inactive radio option.")),Ere=E("radio.inactiveBorder",{light:$e(LC,.2),dark:$e(LC,.2),hcDark:$e(LC,.4),hcLight:$e(LC,.2)},m("radioInactiveBorder","Border color of the inactive radio option.")),Nre=E("radio.inactiveHoverBackground",cre,m("radioHoverBackground","Background color of inactive active radio option when hovering.")),Tre=E("checkbox.background",$I,m("checkbox.background","Background color of checkbox widget."));E("checkbox.selectBackground",Wo,m("checkbox.select.background","Background color of checkbox widget when the element it's in is selected."));const Rre=E("checkbox.foreground",A5,m("checkbox.foreground","Foreground color of checkbox widget.")),Mre=E("checkbox.border",P5,m("checkbox.border","Border color of checkbox widget."));E("checkbox.selectBorder",lL,m("checkbox.select.border","Border color of checkbox widget when the element it's in is selected."));const Are=E("keybindingLabel.background",{dark:new Z(new dt(128,128,128,.17)),light:new Z(new dt(221,221,221,.4)),hcDark:Z.transparent,hcLight:Z.transparent},m("keybindingLabelBackground","Keybinding label background color. The keybinding label is used to represent a keyboard shortcut.")),Pre=E("keybindingLabel.foreground",{dark:Z.fromHex("#CCCCCC"),light:Z.fromHex("#555555"),hcDark:Z.white,hcLight:Ze},m("keybindingLabelForeground","Keybinding label foreground color. The keybinding label is used to represent a keyboard shortcut.")),Ore=E("keybindingLabel.border",{dark:new Z(new dt(51,51,51,.6)),light:new Z(new dt(204,204,204,.4)),hcDark:new Z(new dt(111,195,223)),hcLight:lt},m("keybindingLabelBorder","Keybinding label border color. The keybinding label is used to represent a keyboard shortcut.")),Fre=E("keybindingLabel.bottomBorder",{dark:new Z(new dt(68,68,68,.6)),light:new Z(new dt(187,187,187,.4)),hcDark:new Z(new dt(111,195,223)),hcLight:Ze},m("keybindingLabelBottomBorder","Keybinding label border bottom color. The keybinding label is used to represent a keyboard shortcut.")),Wre=E("list.focusBackground",null,m("listFocusBackground","List/Tree background color for the focused item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not.")),Bre=E("list.focusForeground",null,m("listFocusForeground","List/Tree foreground color for the focused item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not.")),Hre=E("list.focusOutline",{dark:Wl,light:Wl,hcDark:Xt,hcLight:Xt},m("listFocusOutline","List/Tree outline color for the focused item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not.")),Vre=E("list.focusAndSelectionOutline",null,m("listFocusAndSelectionOutline","List/Tree outline color for the focused item when the list/tree is active and selected. An active list/tree has keyboard focus, an inactive does not.")),Tf=E("list.activeSelectionBackground",{dark:"#04395E",light:"#0060C0",hcDark:null,hcLight:Z.fromHex("#0F4A85").transparent(.1)},m("listActiveSelectionBackground","List/Tree background color for the selected item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not.")),bw=E("list.activeSelectionForeground",{dark:Z.white,light:Z.white,hcDark:null,hcLight:null},m("listActiveSelectionForeground","List/Tree foreground color for the selected item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not.")),U$=E("list.activeSelectionIconForeground",null,m("listActiveSelectionIconForeground","List/Tree icon foreground color for the selected item when the list/tree is active. An active list/tree has keyboard focus, an inactive does not.")),zre=E("list.inactiveSelectionBackground",{dark:"#37373D",light:"#E4E6F1",hcDark:null,hcLight:Z.fromHex("#0F4A85").transparent(.1)},m("listInactiveSelectionBackground","List/Tree background color for the selected item when the list/tree is inactive. An active list/tree has keyboard focus, an inactive does not.")),Ure=E("list.inactiveSelectionForeground",null,m("listInactiveSelectionForeground","List/Tree foreground color for the selected item when the list/tree is inactive. An active list/tree has keyboard focus, an inactive does not.")),$re=E("list.inactiveSelectionIconForeground",null,m("listInactiveSelectionIconForeground","List/Tree icon foreground color for the selected item when the list/tree is inactive. An active list/tree has keyboard focus, an inactive does not.")),jre=E("list.inactiveFocusBackground",null,m("listInactiveFocusBackground","List/Tree background color for the focused item when the list/tree is inactive. An active list/tree has keyboard focus, an inactive does not.")),Kre=E("list.inactiveFocusOutline",null,m("listInactiveFocusOutline","List/Tree outline color for the focused item when the list/tree is inactive. An active list/tree has keyboard focus, an inactive does not.")),$$=E("list.hoverBackground",{dark:"#2A2D2E",light:"#F0F0F0",hcDark:Z.white.transparent(.1),hcLight:Z.fromHex("#0F4A85").transparent(.1)},m("listHoverBackground","List/Tree background when hovering over items using the mouse.")),j$=E("list.hoverForeground",null,m("listHoverForeground","List/Tree foreground when hovering over items using the mouse.")),qre=E("list.dropBackground",{dark:"#062F4A",light:"#D6EBFF",hcDark:null,hcLight:null},m("listDropBackground","List/Tree drag and drop background when moving items over other items when using the mouse.")),Gre=E("list.dropBetweenBackground",{dark:lL,light:lL,hcDark:null,hcLight:null},m("listDropBetweenBackground","List/Tree drag and drop border color when moving items between items when using the mouse.")),sp=E("list.highlightForeground",{dark:"#2AAAFF",light:"#0066BF",hcDark:Wl,hcLight:Wl},m("highlight","List/Tree foreground color of the match highlights when searching inside the list/tree.")),Zre=E("list.focusHighlightForeground",{dark:sp,light:Toe(Tf,sp,"#BBE7FF"),hcDark:sp,hcLight:sp},m("listFocusHighlightForeground","List/Tree foreground color of the match highlights on actively focused items when searching inside the list/tree."));E("list.invalidItemForeground",{dark:"#B89500",light:"#B89500",hcDark:"#B89500",hcLight:"#B5200D"},m("invalidItemForeground","List/Tree foreground color for invalid items, for example an unresolved root in explorer."));E("list.errorForeground",{dark:"#F88070",light:"#B01011",hcDark:null,hcLight:null},m("listErrorForeground","Foreground color of list items containing errors."));E("list.warningForeground",{dark:"#CCA700",light:"#855F00",hcDark:null,hcLight:null},m("listWarningForeground","Foreground color of list items containing warnings."));const Yre=E("listFilterWidget.background",{light:im(Wo,0),dark:Ra(Wo,0),hcDark:Wo,hcLight:Wo},m("listFilterWidgetBackground","Background color of the type filter widget in lists and trees.")),Xre=E("listFilterWidget.outline",{dark:Z.transparent,light:Z.transparent,hcDark:"#f38518",hcLight:"#007ACC"},m("listFilterWidgetOutline","Outline color of the type filter widget in lists and trees.")),Qre=E("listFilterWidget.noMatchesOutline",{dark:"#BE1100",light:"#BE1100",hcDark:lt,hcLight:lt},m("listFilterWidgetNoMatchesOutline","Outline color of the type filter widget in lists and trees, when there are no matches.")),Jre=E("listFilterWidget.shadow",Cb,m("listFilterWidgetShadow","Shadow color of the type filter widget in lists and trees."));E("list.filterMatchBackground",{dark:Ac,light:Ac,hcDark:null,hcLight:null},m("listFilterMatchHighlight","Background color of the filtered match."));E("list.filterMatchBorder",{dark:Zd,light:Zd,hcDark:lt,hcLight:Xt},m("listFilterMatchHighlightBorder","Border color of the filtered match."));E("list.deemphasizedForeground",{dark:"#8C8C8C",light:"#8E8E90",hcDark:"#A7A8A9",hcLight:"#666666"},m("listDeemphasizedForeground","List/Tree foreground color for items that are deemphasized."));const K$=E("tree.indentGuidesStroke",{dark:"#585858",light:"#a9a9a9",hcDark:"#a9a9a9",hcLight:"#a5a5a5"},m("treeIndentGuidesStroke","Tree stroke color for the indentation guides.")),eae=E("tree.inactiveIndentGuidesStroke",$e(K$,.4),m("treeInactiveIndentGuidesStroke","Tree stroke color for the indentation guides that are not active.")),tae=E("tree.tableColumnsBorder",{dark:"#CCCCCC20",light:"#61616120",hcDark:null,hcLight:null},m("tableColumnsBorder","Table border color between columns.")),iae=E("tree.tableOddRowsBackground",{dark:$e(Ze,.04),light:$e(Ze,.04),hcDark:null,hcLight:null},m("tableOddRowsBackgroundColor","Background color for odd table rows."));E("editorActionList.background",Wo,m("editorActionListBackground","Action List background color."));E("editorActionList.foreground",BI,m("editorActionListForeground","Action List foreground color."));E("editorActionList.focusForeground",bw,m("editorActionListFocusForeground","Action List foreground color for the focused item."));E("editorActionList.focusBackground",Tf,m("editorActionListFocusBackground","Action List background color for the focused item."));const nae=E("menu.border",{dark:null,light:null,hcDark:lt,hcLight:lt},m("menuBorder","Border color of menus.")),sae=E("menu.foreground",A5,m("menuForeground","Foreground color of menu items.")),oae=E("menu.background",$I,m("menuBackground","Background color of menu items.")),rae=E("menu.selectionForeground",bw,m("menuSelectionForeground","Foreground color of the selected menu item in menus.")),aae=E("menu.selectionBackground",Tf,m("menuSelectionBackground","Background color of the selected menu item in menus.")),lae=E("menu.selectionBorder",{dark:null,light:null,hcDark:Xt,hcLight:Xt},m("menuSelectionBorder","Border color of the selected menu item in menus.")),cae=E("menu.separatorBackground",{dark:"#606060",light:"#D4D4D4",hcDark:lt,hcLight:lt},m("menuSeparatorBackground","Color of a separator menu item in menus.")),I9=E("quickInput.background",Wo,m("pickerBackground","Quick picker background color. The quick picker widget is the container for pickers like the command palette.")),dae=E("quickInput.foreground",BI,m("pickerForeground","Quick picker foreground color. The quick picker widget is the container for pickers like the command palette.")),hae=E("quickInputTitle.background",{dark:new Z(new dt(255,255,255,.105)),light:new Z(new dt(0,0,0,.06)),hcDark:"#000000",hcLight:Z.white},m("pickerTitleBackground","Quick picker title background color. The quick picker widget is the container for pickers like the command palette.")),q$=E("pickerGroup.foreground",{dark:"#3794FF",light:"#0066BF",hcDark:Z.white,hcLight:"#0F4A85"},m("pickerGroupForeground","Quick picker color for grouping labels.")),uae=E("pickerGroup.border",{dark:"#3F3F46",light:"#CCCEDB",hcDark:Z.white,hcLight:"#0F4A85"},m("pickerGroupBorder","Quick picker color for grouping borders.")),E9=E("quickInput.list.focusBackground",null,"",void 0,m("quickInput.list.focusBackground deprecation","Please use quickInputList.focusBackground instead")),Cw=E("quickInputList.focusForeground",bw,m("quickInput.listFocusForeground","Quick picker foreground color for the focused item.")),O5=E("quickInputList.focusIconForeground",U$,m("quickInput.listFocusIconForeground","Quick picker icon foreground color for the focused item.")),vw=E("quickInputList.focusBackground",{dark:mw(E9,Tf),light:mw(E9,Tf),hcDark:null,hcLight:null},m("quickInput.listFocusBackground","Quick picker background color for the focused item."));E("search.resultsInfoForeground",{light:Ze,dark:$e(Ze,.65),hcDark:Ze,hcLight:Ze},m("search.resultsInfoForeground","Color of the text in the search viewlet's completion message."));E("searchEditor.findMatchBackground",{light:$e(Ac,.66),dark:$e(Ac,.66),hcDark:Ac,hcLight:Ac},m("searchEditor.queryMatch","Color of the Search Editor query matches."));E("searchEditor.findMatchBorder",{light:$e(Zd,.66),dark:$e(Zd,.66),hcDark:Zd,hcLight:Zd},m("searchEditor.editorFindMatchBorder","Border color of the Search Editor query matches."));var gae=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},N9=function(o,e){return function(t,i){e(t,i,o)}};const Yl=et("hoverService");let M_=class extends F{get delay(){return this.isInstantlyHovering()?0:this._delay}constructor(e,t,i={},n,s){super(),this.placement=e,this.instantHover=t,this.overrideOptions=i,this.configurationService=n,this.hoverService=s,this.lastHoverHideTime=0,this.timeLimit=200,this.hoverDisposables=this._register(new K),this._delay=this.configurationService.getValue("workbench.hover.delay"),this._register(this.configurationService.onDidChangeConfiguration(r=>{r.affectsConfiguration("workbench.hover.delay")&&(this._delay=this.configurationService.getValue("workbench.hover.delay"))}))}showHover(e,t){const i=typeof this.overrideOptions=="function"?this.overrideOptions(e,t):this.overrideOptions;this.hoverDisposables.clear();const n=Xi(e.target)?[e.target]:e.target.targetElements;for(const r of n)this.hoverDisposables.add(hi(r,"keydown",a=>{a.equals(9)&&this.hoverService.hideHover()}));const s=Xi(e.content)?void 0:e.content.toString();return this.hoverService.showHover({...e,...i,persistence:{hideOnKeyDown:!0,...i.persistence},id:s,appearance:{...e.appearance,compact:!0,skipFadeInAnimation:this.isInstantlyHovering(),...i.appearance}},t)}isInstantlyHovering(){return this.instantHover&&Date.now()-this.lastHoverHideTime<this.timeLimit}onDidHideHover(){this.hoverDisposables.clear(),this.instantHover&&(this.lastHoverHideTime=Date.now())}};M_=gae([N9(3,qe),N9(4,Yl)],M_);const ud=et("contextViewService"),mo=et("contextMenuService"),yt=et("keybindingService");class vb{constructor(){this._hooks=new K,this._pointerMoveCallback=null,this._onStopCallback=null}dispose(){this.stopMonitoring(!1),this._hooks.dispose()}stopMonitoring(e,t){if(!this.isMonitoring())return;this._hooks.clear(),this._pointerMoveCallback=null;const i=this._onStopCallback;this._onStopCallback=null,e&&i&&i(t)}isMonitoring(){return!!this._pointerMoveCallback}startMonitoring(e,t,i,n,s){this.isMonitoring()&&this.stopMonitoring(!1),this._pointerMoveCallback=n,this._onStopCallback=s;let r=e;try{e.setPointerCapture(t),this._hooks.add(ve(()=>{try{e.releasePointerCapture(t)}catch{}}))}catch{r=xe(e)}this._hooks.add(U(r,ie.POINTER_MOVE,a=>{if(a.buttons!==i){this.stopMonitoring(!0);return}a.preventDefault(),this._pointerMoveCallback(a)})),this._hooks.add(U(r,ie.POINTER_UP,a=>this.stopMonitoring(!0)))}}function Ni(o,e,t){let i=null,n=null;if(typeof t.value=="function"?(i="value",n=t.value,n.length!==0&&console.warn("Memoize should only be used in functions with zero parameters")):typeof t.get=="function"&&(i="get",n=t.get),!n)throw new Error("not supported");const s=`$memoize$${e}`;t[i]=function(...r){return this.hasOwnProperty(s)||Object.defineProperty(this,s,{configurable:!1,enumerable:!1,writable:!1,value:n.apply(this,r)}),this[s]}}var fae=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Bt;(function(o){o.Tap="-monaco-gesturetap",o.Change="-monaco-gesturechange",o.Start="-monaco-gesturestart",o.End="-monaco-gesturesend",o.Contextmenu="-monaco-gesturecontextmenu"})(Bt||(Bt={}));const yn=class yn extends F{constructor(){super(),this.dispatched=!1,this.targets=new cs,this.ignoreTargets=new cs,this.activeTouches={},this.handle=null,this._lastSetTapCountTime=0,this._register(oe.runAndSubscribe(CI,({window:e,disposables:t})=>{t.add(U(e.document,"touchstart",i=>this.onTouchStart(i),{passive:!1})),t.add(U(e.document,"touchend",i=>this.onTouchEnd(e,i))),t.add(U(e.document,"touchmove",i=>this.onTouchMove(i),{passive:!1}))},{window:Rt,disposables:this._store}))}static addTarget(e){if(!yn.isTouchDevice())return F.None;yn.INSTANCE||(yn.INSTANCE=new yn);const t=yn.INSTANCE.targets.push(e);return ve(t)}static ignoreTarget(e){if(!yn.isTouchDevice())return F.None;yn.INSTANCE||(yn.INSTANCE=new yn);const t=yn.INSTANCE.ignoreTargets.push(e);return ve(t)}static isTouchDevice(){return"ontouchstart"in Rt||navigator.maxTouchPoints>0}dispose(){this.handle&&(this.handle.dispose(),this.handle=null),super.dispose()}onTouchStart(e){const t=Date.now();this.handle&&(this.handle.dispose(),this.handle=null);for(let i=0,n=e.targetTouches.length;i<n;i++){const s=e.targetTouches.item(i);this.activeTouches[s.identifier]={id:s.identifier,initialTarget:s.target,initialTimeStamp:t,initialPageX:s.pageX,initialPageY:s.pageY,rollingTimestamps:[t],rollingPageX:[s.pageX],rollingPageY:[s.pageY]};const r=this.newGestureEvent(Bt.Start,s.target);r.pageX=s.pageX,r.pageY=s.pageY,this.dispatchEvent(r)}this.dispatched&&(e.preventDefault(),e.stopPropagation(),this.dispatched=!1)}onTouchEnd(e,t){const i=Date.now(),n=Object.keys(this.activeTouches).length;for(let s=0,r=t.changedTouches.length;s<r;s++){const a=t.changedTouches.item(s);if(!this.activeTouches.hasOwnProperty(String(a.identifier))){console.warn("move of an UNKNOWN touch",a);continue}const l=this.activeTouches[a.identifier],c=Date.now()-l.initialTimeStamp;if(c<yn.HOLD_DELAY&&Math.abs(l.initialPageX-tr(l.rollingPageX))<30&&Math.abs(l.initialPageY-tr(l.rollingPageY))<30){const d=this.newGestureEvent(Bt.Tap,l.initialTarget);d.pageX=tr(l.rollingPageX),d.pageY=tr(l.rollingPageY),this.dispatchEvent(d)}else if(c>=yn.HOLD_DELAY&&Math.abs(l.initialPageX-tr(l.rollingPageX))<30&&Math.abs(l.initialPageY-tr(l.rollingPageY))<30){const d=this.newGestureEvent(Bt.Contextmenu,l.initialTarget);d.pageX=tr(l.rollingPageX),d.pageY=tr(l.rollingPageY),this.dispatchEvent(d)}else if(n===1){const d=tr(l.rollingPageX),h=tr(l.rollingPageY),u=tr(l.rollingTimestamps)-l.rollingTimestamps[0],g=d-l.rollingPageX[0],f=h-l.rollingPageY[0],p=[...this.targets].filter(_=>l.initialTarget instanceof Node&&_.contains(l.initialTarget));this.inertia(e,p,i,Math.abs(g)/u,g>0?1:-1,d,Math.abs(f)/u,f>0?1:-1,h)}this.dispatchEvent(this.newGestureEvent(Bt.End,l.initialTarget)),delete this.activeTouches[a.identifier]}this.dispatched&&(t.preventDefault(),t.stopPropagation(),this.dispatched=!1)}newGestureEvent(e,t){const i=document.createEvent("CustomEvent");return i.initEvent(e,!1,!0),i.initialTarget=t,i.tapCount=0,i}dispatchEvent(e){if(e.type===Bt.Tap){const t=new Date().getTime();let i=0;t-this._lastSetTapCountTime>yn.CLEAR_TAP_COUNT_TIME?i=1:i=2,this._lastSetTapCountTime=t,e.tapCount=i}else(e.type===Bt.Change||e.type===Bt.Contextmenu)&&(this._lastSetTapCountTime=0);if(e.initialTarget instanceof Node){for(const i of this.ignoreTargets)if(i.contains(e.initialTarget))return;const t=[];for(const i of this.targets)if(i.contains(e.initialTarget)){let n=0,s=e.initialTarget;for(;s&&s!==i;)n++,s=s.parentElement;t.push([n,i])}t.sort((i,n)=>i[0]-n[0]);for(const[i,n]of t)n.dispatchEvent(e),this.dispatched=!0}}inertia(e,t,i,n,s,r,a,l,c){this.handle=go(e,()=>{const d=Date.now(),h=d-i;let u=0,g=0,f=!0;n+=yn.SCROLL_FRICTION*h,a+=yn.SCROLL_FRICTION*h,n>0&&(f=!1,u=s*n*h),a>0&&(f=!1,g=l*a*h);const p=this.newGestureEvent(Bt.Change);p.translationX=u,p.translationY=g,t.forEach(_=>_.dispatchEvent(p)),f||this.inertia(e,t,d,n,s,r+u,a,l,c+g)})}onTouchMove(e){const t=Date.now();for(let i=0,n=e.changedTouches.length;i<n;i++){const s=e.changedTouches.item(i);if(!this.activeTouches.hasOwnProperty(String(s.identifier))){console.warn("end of an UNKNOWN touch",s);continue}const r=this.activeTouches[s.identifier],a=this.newGestureEvent(Bt.Change,r.initialTarget);a.translationX=s.pageX-tr(r.rollingPageX),a.translationY=s.pageY-tr(r.rollingPageY),a.pageX=s.pageX,a.pageY=s.pageY,this.dispatchEvent(a),r.rollingPageX.length>3&&(r.rollingPageX.shift(),r.rollingPageY.shift(),r.rollingTimestamps.shift()),r.rollingPageX.push(s.pageX),r.rollingPageY.push(s.pageY),r.rollingTimestamps.push(t)}this.dispatched&&(e.preventDefault(),e.stopPropagation(),this.dispatched=!1)}};yn.SCROLL_FRICTION=-.005,yn.HOLD_DELAY=700,yn.CLEAR_TAP_COUNT_TIME=400;let Fn=yn;fae([Ni],Fn,"isTouchDevice",null);let Yo=class extends F{onclick(e,t){this._register(U(e,ie.CLICK,i=>t(new Vr(xe(e),i))))}onmousedown(e,t){this._register(U(e,ie.MOUSE_DOWN,i=>t(new Vr(xe(e),i))))}onmouseover(e,t){this._register(U(e,ie.MOUSE_OVER,i=>t(new Vr(xe(e),i))))}onmouseleave(e,t){this._register(U(e,ie.MOUSE_LEAVE,i=>t(new Vr(xe(e),i))))}onkeydown(e,t){this._register(U(e,ie.KEY_DOWN,i=>t(new Pt(i))))}onkeyup(e,t){this._register(U(e,ie.KEY_UP,i=>t(new Pt(i))))}oninput(e,t){this._register(U(e,ie.INPUT,t))}onblur(e,t){this._register(U(e,ie.BLUR,t))}onfocus(e,t){this._register(U(e,ie.FOCUS,t))}ignoreGesture(e){return Fn.ignoreTarget(e)}};const A_=11;class mae extends Yo{constructor(e){super(),this._onActivate=e.onActivate,this.bgDomNode=document.createElement("div"),this.bgDomNode.className="arrow-background",this.bgDomNode.style.position="absolute",this.bgDomNode.style.width=e.bgWidth+"px",this.bgDomNode.style.height=e.bgHeight+"px",typeof e.top<"u"&&(this.bgDomNode.style.top="0px"),typeof e.left<"u"&&(this.bgDomNode.style.left="0px"),typeof e.bottom<"u"&&(this.bgDomNode.style.bottom="0px"),typeof e.right<"u"&&(this.bgDomNode.style.right="0px"),this.domNode=document.createElement("div"),this.domNode.className=e.className,this.domNode.classList.add(...Ie.asClassNameArray(e.icon)),this.domNode.style.position="absolute",this.domNode.style.width=A_+"px",this.domNode.style.height=A_+"px",typeof e.top<"u"&&(this.domNode.style.top=e.top+"px"),typeof e.left<"u"&&(this.domNode.style.left=e.left+"px"),typeof e.bottom<"u"&&(this.domNode.style.bottom=e.bottom+"px"),typeof e.right<"u"&&(this.domNode.style.right=e.right+"px"),this._pointerMoveMonitor=this._register(new vb),this._register(hi(this.bgDomNode,ie.POINTER_DOWN,t=>this._arrowPointerDown(t))),this._register(hi(this.domNode,ie.POINTER_DOWN,t=>this._arrowPointerDown(t))),this._pointerdownRepeatTimer=this._register(new r5),this._pointerdownScheduleRepeatTimer=this._register(new ea)}_arrowPointerDown(e){if(!e.target||!(e.target instanceof Element))return;const t=()=>{this._pointerdownRepeatTimer.cancelAndSet(()=>this._onActivate(),1e3/24,xe(e))};this._onActivate(),this._pointerdownRepeatTimer.cancel(),this._pointerdownScheduleRepeatTimer.cancelAndSet(t,200),this._pointerMoveMonitor.startMonitoring(e.target,e.pointerId,e.buttons,i=>{},()=>{this._pointerdownRepeatTimer.cancel(),this._pointerdownScheduleRepeatTimer.cancel()}),e.preventDefault()}}class pae extends F{constructor(e,t,i){super(),this._visibility=e,this._visibleClassName=t,this._invisibleClassName=i,this._domNode=null,this._isVisible=!1,this._isNeeded=!1,this._rawShouldBeVisible=!1,this._shouldBeVisible=!1,this._revealTimer=this._register(new ea)}setVisibility(e){this._visibility!==e&&(this._visibility=e,this._updateShouldBeVisible())}setShouldBeVisible(e){this._rawShouldBeVisible=e,this._updateShouldBeVisible()}_applyVisibilitySetting(){return this._visibility===2?!1:this._visibility===3?!0:this._rawShouldBeVisible}_updateShouldBeVisible(){const e=this._applyVisibilitySetting();this._shouldBeVisible!==e&&(this._shouldBeVisible=e,this.ensureVisibility())}setIsNeeded(e){this._isNeeded!==e&&(this._isNeeded=e,this.ensureVisibility())}setDomNode(e){this._domNode=e,this._domNode.setClassName(this._invisibleClassName),this.setShouldBeVisible(!1)}ensureVisibility(){if(!this._isNeeded){this._hide(!1);return}this._shouldBeVisible?this._reveal():this._hide(!0)}_reveal(){this._isVisible||(this._isVisible=!0,this._revealTimer.setIfNotSet(()=>{var e;(e=this._domNode)==null||e.setClassName(this._visibleClassName)},0))}_hide(e){var t;this._revealTimer.cancel(),this._isVisible&&(this._isVisible=!1,(t=this._domNode)==null||t.setClassName(this._invisibleClassName+(e?" fade":"")))}}const _ae=140;class G$ extends Yo{constructor(e){super(),this._lazyRender=e.lazyRender,this._host=e.host,this._scrollable=e.scrollable,this._scrollByPage=e.scrollByPage,this._scrollbarState=e.scrollbarState,this._visibilityController=this._register(new pae(e.visibility,"visible scrollbar "+e.extraScrollbarClassName,"invisible scrollbar "+e.extraScrollbarClassName)),this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded()),this._pointerMoveMonitor=this._register(new vb),this._shouldRender=!0,this.domNode=St(document.createElement("div")),this.domNode.setAttribute("role","presentation"),this.domNode.setAttribute("aria-hidden","true"),this._visibilityController.setDomNode(this.domNode),this.domNode.setPosition("absolute"),this._register(U(this.domNode.domNode,ie.POINTER_DOWN,t=>this._domNodePointerDown(t)))}_createArrow(e){const t=this._register(new mae(e));this.domNode.domNode.appendChild(t.bgDomNode),this.domNode.domNode.appendChild(t.domNode)}_createSlider(e,t,i,n){this.slider=St(document.createElement("div")),this.slider.setClassName("slider"),this.slider.setPosition("absolute"),this.slider.setTop(e),this.slider.setLeft(t),typeof i=="number"&&this.slider.setWidth(i),typeof n=="number"&&this.slider.setHeight(n),this.slider.setLayerHinting(!0),this.slider.setContain("strict"),this.domNode.domNode.appendChild(this.slider.domNode),this._register(U(this.slider.domNode,ie.POINTER_DOWN,s=>{s.button===0&&(s.preventDefault(),this._sliderPointerDown(s))})),this.onclick(this.slider.domNode,s=>{s.leftButton&&s.stopPropagation()})}_onElementSize(e){return this._scrollbarState.setVisibleSize(e)&&(this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded()),this._shouldRender=!0,this._lazyRender||this.render()),this._shouldRender}_onElementScrollSize(e){return this._scrollbarState.setScrollSize(e)&&(this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded()),this._shouldRender=!0,this._lazyRender||this.render()),this._shouldRender}_onElementScrollPosition(e){return this._scrollbarState.setScrollPosition(e)&&(this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded()),this._shouldRender=!0,this._lazyRender||this.render()),this._shouldRender}beginReveal(){this._visibilityController.setShouldBeVisible(!0)}beginHide(){this._visibilityController.setShouldBeVisible(!1)}render(){this._shouldRender&&(this._shouldRender=!1,this._renderDomNode(this._scrollbarState.getRectangleLargeSize(),this._scrollbarState.getRectangleSmallSize()),this._updateSlider(this._scrollbarState.getSliderSize(),this._scrollbarState.getArrowSize()+this._scrollbarState.getSliderPosition()))}_domNodePointerDown(e){e.target===this.domNode.domNode&&this._onPointerDown(e)}delegatePointerDown(e){const t=this.domNode.domNode.getClientRects()[0].top,i=t+this._scrollbarState.getSliderPosition(),n=t+this._scrollbarState.getSliderPosition()+this._scrollbarState.getSliderSize(),s=this._sliderPointerPosition(e);i<=s&&s<=n?e.button===0&&(e.preventDefault(),this._sliderPointerDown(e)):this._onPointerDown(e)}_onPointerDown(e){let t,i;if(e.target===this.domNode.domNode&&typeof e.offsetX=="number"&&typeof e.offsetY=="number")t=e.offsetX,i=e.offsetY;else{const s=wi(this.domNode.domNode);t=e.pageX-s.left,i=e.pageY-s.top}const n=this._pointerDownRelativePosition(t,i);this._setDesiredScrollPositionNow(this._scrollByPage?this._scrollbarState.getDesiredScrollPositionFromOffsetPaged(n):this._scrollbarState.getDesiredScrollPositionFromOffset(n)),e.button===0&&(e.preventDefault(),this._sliderPointerDown(e))}_sliderPointerDown(e){if(!e.target||!(e.target instanceof Element))return;const t=this._sliderPointerPosition(e),i=this._sliderOrthogonalPointerPosition(e),n=this._scrollbarState.clone();this.slider.toggleClassName("active",!0),this._pointerMoveMonitor.startMonitoring(e.target,e.pointerId,e.buttons,s=>{const r=this._sliderOrthogonalPointerPosition(s),a=Math.abs(r-i);if(En&&a>_ae){this._setDesiredScrollPositionNow(n.getScrollPosition());return}const c=this._sliderPointerPosition(s)-t;this._setDesiredScrollPositionNow(n.getDesiredScrollPositionFromDelta(c))},()=>{this.slider.toggleClassName("active",!1),this._host.onDragEnd()}),this._host.onDragStart()}_setDesiredScrollPositionNow(e){const t={};this.writeScrollPosition(t,e),this._scrollable.setScrollPositionNow(t)}updateScrollbarSize(e){this._updateScrollbarSize(e),this._scrollbarState.setScrollbarSize(e),this._shouldRender=!0,this._lazyRender||this.render()}isNeeded(){return this._scrollbarState.isNeeded()}}const bae=20;class P_{constructor(e,t,i,n,s,r){this._scrollbarSize=Math.round(t),this._oppositeScrollbarSize=Math.round(i),this._arrowSize=Math.round(e),this._visibleSize=n,this._scrollSize=s,this._scrollPosition=r,this._computedAvailableSize=0,this._computedIsNeeded=!1,this._computedSliderSize=0,this._computedSliderRatio=0,this._computedSliderPosition=0,this._refreshComputedValues()}clone(){return new P_(this._arrowSize,this._scrollbarSize,this._oppositeScrollbarSize,this._visibleSize,this._scrollSize,this._scrollPosition)}setVisibleSize(e){const t=Math.round(e);return this._visibleSize!==t?(this._visibleSize=t,this._refreshComputedValues(),!0):!1}setScrollSize(e){const t=Math.round(e);return this._scrollSize!==t?(this._scrollSize=t,this._refreshComputedValues(),!0):!1}setScrollPosition(e){const t=Math.round(e);return this._scrollPosition!==t?(this._scrollPosition=t,this._refreshComputedValues(),!0):!1}setScrollbarSize(e){this._scrollbarSize=Math.round(e)}setOppositeScrollbarSize(e){this._oppositeScrollbarSize=Math.round(e)}static _computeValues(e,t,i,n,s){const r=Math.max(0,i-e),a=Math.max(0,r-2*t),l=n>0&&n>i;if(!l)return{computedAvailableSize:Math.round(r),computedIsNeeded:l,computedSliderSize:Math.round(a),computedSliderRatio:0,computedSliderPosition:0};const c=Math.round(Math.max(bae,Math.floor(i*a/n))),d=(a-c)/(n-i),h=s*d;return{computedAvailableSize:Math.round(r),computedIsNeeded:l,computedSliderSize:Math.round(c),computedSliderRatio:d,computedSliderPosition:Math.round(h)}}_refreshComputedValues(){const e=P_._computeValues(this._oppositeScrollbarSize,this._arrowSize,this._visibleSize,this._scrollSize,this._scrollPosition);this._computedAvailableSize=e.computedAvailableSize,this._computedIsNeeded=e.computedIsNeeded,this._computedSliderSize=e.computedSliderSize,this._computedSliderRatio=e.computedSliderRatio,this._computedSliderPosition=e.computedSliderPosition}getArrowSize(){return this._arrowSize}getScrollPosition(){return this._scrollPosition}getRectangleLargeSize(){return this._computedAvailableSize}getRectangleSmallSize(){return this._scrollbarSize}isNeeded(){return this._computedIsNeeded}getSliderSize(){return this._computedSliderSize}getSliderPosition(){return this._computedSliderPosition}getDesiredScrollPositionFromOffset(e){if(!this._computedIsNeeded)return 0;const t=e-this._arrowSize-this._computedSliderSize/2;return Math.round(t/this._computedSliderRatio)}getDesiredScrollPositionFromOffsetPaged(e){if(!this._computedIsNeeded)return 0;const t=e-this._arrowSize;let i=this._scrollPosition;return t<this._computedSliderPosition?i-=this._visibleSize:i+=this._visibleSize,i}getDesiredScrollPositionFromDelta(e){if(!this._computedIsNeeded)return 0;const t=this._computedSliderPosition+e;return Math.round(t/this._computedSliderRatio)}}class Cae extends G${constructor(e,t,i){const n=e.getScrollDimensions(),s=e.getCurrentScrollPosition();if(super({lazyRender:t.lazyRender,host:i,scrollbarState:new P_(t.horizontalHasArrows?t.arrowSize:0,t.horizontal===2?0:t.horizontalScrollbarSize,t.vertical===2?0:t.verticalScrollbarSize,n.width,n.scrollWidth,s.scrollLeft),visibility:t.horizontal,extraScrollbarClassName:"horizontal",scrollable:e,scrollByPage:t.scrollByPage}),t.horizontalHasArrows){const r=(t.arrowSize-A_)/2,a=(t.horizontalScrollbarSize-A_)/2;this._createArrow({className:"scra",icon:te.scrollbarButtonLeft,top:a,left:r,bottom:void 0,right:void 0,bgWidth:t.arrowSize,bgHeight:t.horizontalScrollbarSize,onActivate:()=>this._host.onMouseWheel(new Df(null,1,0))}),this._createArrow({className:"scra",icon:te.scrollbarButtonRight,top:a,left:void 0,bottom:void 0,right:r,bgWidth:t.arrowSize,bgHeight:t.horizontalScrollbarSize,onActivate:()=>this._host.onMouseWheel(new Df(null,-1,0))})}this._createSlider(Math.floor((t.horizontalScrollbarSize-t.horizontalSliderSize)/2),0,void 0,t.horizontalSliderSize)}_updateSlider(e,t){this.slider.setWidth(e),this.slider.setLeft(t)}_renderDomNode(e,t){this.domNode.setWidth(e),this.domNode.setHeight(t),this.domNode.setLeft(0),this.domNode.setBottom(0)}onDidScroll(e){return this._shouldRender=this._onElementScrollSize(e.scrollWidth)||this._shouldRender,this._shouldRender=this._onElementScrollPosition(e.scrollLeft)||this._shouldRender,this._shouldRender=this._onElementSize(e.width)||this._shouldRender,this._shouldRender}_pointerDownRelativePosition(e,t){return e}_sliderPointerPosition(e){return e.pageX}_sliderOrthogonalPointerPosition(e){return e.pageY}_updateScrollbarSize(e){this.slider.setHeight(e)}writeScrollPosition(e,t){e.scrollLeft=t}updateOptions(e){this.updateScrollbarSize(e.horizontal===2?0:e.horizontalScrollbarSize),this._scrollbarState.setOppositeScrollbarSize(e.vertical===2?0:e.verticalScrollbarSize),this._visibilityController.setVisibility(e.horizontal),this._scrollByPage=e.scrollByPage}}class vae extends G${constructor(e,t,i){const n=e.getScrollDimensions(),s=e.getCurrentScrollPosition();if(super({lazyRender:t.lazyRender,host:i,scrollbarState:new P_(t.verticalHasArrows?t.arrowSize:0,t.vertical===2?0:t.verticalScrollbarSize,0,n.height,n.scrollHeight,s.scrollTop),visibility:t.vertical,extraScrollbarClassName:"vertical",scrollable:e,scrollByPage:t.scrollByPage}),t.verticalHasArrows){const r=(t.arrowSize-A_)/2,a=(t.verticalScrollbarSize-A_)/2;this._createArrow({className:"scra",icon:te.scrollbarButtonUp,top:r,left:a,bottom:void 0,right:void 0,bgWidth:t.verticalScrollbarSize,bgHeight:t.arrowSize,onActivate:()=>this._host.onMouseWheel(new Df(null,0,1))}),this._createArrow({className:"scra",icon:te.scrollbarButtonDown,top:void 0,left:a,bottom:r,right:void 0,bgWidth:t.verticalScrollbarSize,bgHeight:t.arrowSize,onActivate:()=>this._host.onMouseWheel(new Df(null,0,-1))})}this._createSlider(0,Math.floor((t.verticalScrollbarSize-t.verticalSliderSize)/2),t.verticalSliderSize,void 0)}_updateSlider(e,t){this.slider.setHeight(e),this.slider.setTop(t)}_renderDomNode(e,t){this.domNode.setWidth(t),this.domNode.setHeight(e),this.domNode.setRight(0),this.domNode.setTop(0)}onDidScroll(e){return this._shouldRender=this._onElementScrollSize(e.scrollHeight)||this._shouldRender,this._shouldRender=this._onElementScrollPosition(e.scrollTop)||this._shouldRender,this._shouldRender=this._onElementSize(e.height)||this._shouldRender,this._shouldRender}_pointerDownRelativePosition(e,t){return t}_sliderPointerPosition(e){return e.pageY}_sliderOrthogonalPointerPosition(e){return e.pageX}_updateScrollbarSize(e){this.slider.setWidth(e)}writeScrollPosition(e,t){e.scrollTop=t}updateOptions(e){this.updateScrollbarSize(e.vertical===2?0:e.verticalScrollbarSize),this._scrollbarState.setOppositeScrollbarSize(0),this._visibilityController.setVisibility(e.vertical),this._scrollByPage=e.scrollByPage}}class dL{constructor(e,t,i,n,s,r,a){this._forceIntegerValues=e,this._scrollStateBrand=void 0,this._forceIntegerValues&&(t=t|0,i=i|0,n=n|0,s=s|0,r=r|0,a=a|0),this.rawScrollLeft=n,this.rawScrollTop=a,t<0&&(t=0),n+t>i&&(n=i-t),n<0&&(n=0),s<0&&(s=0),a+s>r&&(a=r-s),a<0&&(a=0),this.width=t,this.scrollWidth=i,this.scrollLeft=n,this.height=s,this.scrollHeight=r,this.scrollTop=a}equals(e){return this.rawScrollLeft===e.rawScrollLeft&&this.rawScrollTop===e.rawScrollTop&&this.width===e.width&&this.scrollWidth===e.scrollWidth&&this.scrollLeft===e.scrollLeft&&this.height===e.height&&this.scrollHeight===e.scrollHeight&&this.scrollTop===e.scrollTop}withScrollDimensions(e,t){return new dL(this._forceIntegerValues,typeof e.width<"u"?e.width:this.width,typeof e.scrollWidth<"u"?e.scrollWidth:this.scrollWidth,t?this.rawScrollLeft:this.scrollLeft,typeof e.height<"u"?e.height:this.height,typeof e.scrollHeight<"u"?e.scrollHeight:this.scrollHeight,t?this.rawScrollTop:this.scrollTop)}withScrollPosition(e){return new dL(this._forceIntegerValues,this.width,this.scrollWidth,typeof e.scrollLeft<"u"?e.scrollLeft:this.rawScrollLeft,this.height,this.scrollHeight,typeof e.scrollTop<"u"?e.scrollTop:this.rawScrollTop)}createScrollEvent(e,t){const i=this.width!==e.width,n=this.scrollWidth!==e.scrollWidth,s=this.scrollLeft!==e.scrollLeft,r=this.height!==e.height,a=this.scrollHeight!==e.scrollHeight,l=this.scrollTop!==e.scrollTop;return{inSmoothScrolling:t,oldWidth:e.width,oldScrollWidth:e.scrollWidth,oldScrollLeft:e.scrollLeft,width:this.width,scrollWidth:this.scrollWidth,scrollLeft:this.scrollLeft,oldHeight:e.height,oldScrollHeight:e.scrollHeight,oldScrollTop:e.scrollTop,height:this.height,scrollHeight:this.scrollHeight,scrollTop:this.scrollTop,widthChanged:i,scrollWidthChanged:n,scrollLeftChanged:s,heightChanged:r,scrollHeightChanged:a,scrollTopChanged:l}}}class wb extends F{constructor(e){super(),this._scrollableBrand=void 0,this._onScroll=this._register(new P),this.onScroll=this._onScroll.event,this._smoothScrollDuration=e.smoothScrollDuration,this._scheduleAtNextAnimationFrame=e.scheduleAtNextAnimationFrame,this._state=new dL(e.forceIntegerValues,0,0,0,0,0,0),this._smoothScrolling=null}dispose(){this._smoothScrolling&&(this._smoothScrolling.dispose(),this._smoothScrolling=null),super.dispose()}setSmoothScrollDuration(e){this._smoothScrollDuration=e}validateScrollPosition(e){return this._state.withScrollPosition(e)}getScrollDimensions(){return this._state}setScrollDimensions(e,t){var n;const i=this._state.withScrollDimensions(e,t);this._setState(i,!!this._smoothScrolling),(n=this._smoothScrolling)==null||n.acceptScrollDimensions(this._state)}getFutureScrollPosition(){return this._smoothScrolling?this._smoothScrolling.to:this._state}getCurrentScrollPosition(){return this._state}setScrollPositionNow(e){const t=this._state.withScrollPosition(e);this._smoothScrolling&&(this._smoothScrolling.dispose(),this._smoothScrolling=null),this._setState(t,!1)}setScrollPositionSmooth(e,t){if(this._smoothScrollDuration===0)return this.setScrollPositionNow(e);if(this._smoothScrolling){e={scrollLeft:typeof e.scrollLeft>"u"?this._smoothScrolling.to.scrollLeft:e.scrollLeft,scrollTop:typeof e.scrollTop>"u"?this._smoothScrolling.to.scrollTop:e.scrollTop};const i=this._state.withScrollPosition(e);if(this._smoothScrolling.to.scrollLeft===i.scrollLeft&&this._smoothScrolling.to.scrollTop===i.scrollTop)return;let n;t?n=new ww(this._smoothScrolling.from,i,this._smoothScrolling.startTime,this._smoothScrolling.duration):n=this._smoothScrolling.combine(this._state,i,this._smoothScrollDuration),this._smoothScrolling.dispose(),this._smoothScrolling=n}else{const i=this._state.withScrollPosition(e);this._smoothScrolling=ww.start(this._state,i,this._smoothScrollDuration)}this._smoothScrolling.animationFrameDisposable=this._scheduleAtNextAnimationFrame(()=>{this._smoothScrolling&&(this._smoothScrolling.animationFrameDisposable=null,this._performSmoothScrolling())})}hasPendingScrollAnimation(){return!!this._smoothScrolling}_performSmoothScrolling(){if(!this._smoothScrolling)return;const e=this._smoothScrolling.tick(),t=this._state.withScrollPosition(e);if(this._setState(t,!0),!!this._smoothScrolling){if(e.isDone){this._smoothScrolling.dispose(),this._smoothScrolling=null;return}this._smoothScrolling.animationFrameDisposable=this._scheduleAtNextAnimationFrame(()=>{this._smoothScrolling&&(this._smoothScrolling.animationFrameDisposable=null,this._performSmoothScrolling())})}}_setState(e,t){const i=this._state;i.equals(e)||(this._state=e,this._onScroll.fire(this._state.createScrollEvent(i,t)))}}class T9{constructor(e,t,i){this.scrollLeft=e,this.scrollTop=t,this.isDone=i}}function WN(o,e){const t=e-o;return function(i){return o+t*yae(i)}}function wae(o,e,t){return function(i){return i<t?o(i/t):e((i-t)/(1-t))}}class ww{constructor(e,t,i,n){this.from=e,this.to=t,this.duration=n,this.startTime=i,this.animationFrameDisposable=null,this._initAnimations()}_initAnimations(){this.scrollLeft=this._initAnimation(this.from.scrollLeft,this.to.scrollLeft,this.to.width),this.scrollTop=this._initAnimation(this.from.scrollTop,this.to.scrollTop,this.to.height)}_initAnimation(e,t,i){if(Math.abs(e-t)>2.5*i){let s,r;return e<t?(s=e+.75*i,r=t-.75*i):(s=e-.75*i,r=t+.75*i),wae(WN(e,s),WN(r,t),.33)}return WN(e,t)}dispose(){this.animationFrameDisposable!==null&&(this.animationFrameDisposable.dispose(),this.animationFrameDisposable=null)}acceptScrollDimensions(e){this.to=e.withScrollPosition(this.to),this._initAnimations()}tick(){return this._tick(Date.now())}_tick(e){const t=(e-this.startTime)/this.duration;if(t<1){const i=this.scrollLeft(t),n=this.scrollTop(t);return new T9(i,n,!1)}return new T9(this.to.scrollLeft,this.to.scrollTop,!0)}combine(e,t,i){return ww.start(e,t,i)}static start(e,t,i){i=i+10;const n=Date.now()-10;return new ww(e,t,n,i)}}function Sae(o){return Math.pow(o,3)}function yae(o){return 1-Sae(1-o)}const Lae=500,R9=50;class xae{constructor(e,t,i){this.timestamp=e,this.deltaX=t,this.deltaY=i,this.score=0}}const Hk=class Hk{constructor(){this._capacity=5,this._memory=[],this._front=-1,this._rear=-1}isPhysicalMouseWheel(){if(this._front===-1&&this._rear===-1)return!1;let e=1,t=0,i=1,n=this._rear;do{const s=n===this._front?e:Math.pow(2,-i);if(e-=s,t+=this._memory[n].score*s,n===this._front)break;n=(this._capacity+n-1)%this._capacity,i++}while(!0);return t<=.5}acceptStandardWheelEvent(e){if(O1){const t=xe(e.browserEvent),i=$ee(t);this.accept(Date.now(),e.deltaX*i,e.deltaY*i)}else this.accept(Date.now(),e.deltaX,e.deltaY)}accept(e,t,i){let n=null;const s=new xae(e,t,i);this._front===-1&&this._rear===-1?(this._memory[0]=s,this._front=0,this._rear=0):(n=this._memory[this._rear],this._rear=(this._rear+1)%this._capacity,this._rear===this._front&&(this._front=(this._front+1)%this._capacity),this._memory[this._rear]=s),s.score=this._computeScore(s,n)}_computeScore(e,t){if(Math.abs(e.deltaX)>0&&Math.abs(e.deltaY)>0)return 1;let i=.5;if((!this._isAlmostInt(e.deltaX)||!this._isAlmostInt(e.deltaY))&&(i+=.25),t){const n=Math.abs(e.deltaX),s=Math.abs(e.deltaY),r=Math.abs(t.deltaX),a=Math.abs(t.deltaY),l=Math.max(Math.min(n,r),1),c=Math.max(Math.min(s,a),1),d=Math.max(n,r),h=Math.max(s,a);d%l===0&&h%c===0&&(i-=.5)}return Math.min(Math.max(i,0),1)}_isAlmostInt(e){return Math.abs(Math.round(e)-e)<.01}};Hk.INSTANCE=new Hk;let hL=Hk;class F5 extends Yo{get options(){return this._options}constructor(e,t,i){super(),this._onScroll=this._register(new P),this.onScroll=this._onScroll.event,this._onWillScroll=this._register(new P),e.style.overflow="hidden",this._options=kae(t),this._scrollable=i,this._register(this._scrollable.onScroll(s=>{this._onWillScroll.fire(s),this._onDidScroll(s),this._onScroll.fire(s)}));const n={onMouseWheel:s=>this._onMouseWheel(s),onDragStart:()=>this._onDragStart(),onDragEnd:()=>this._onDragEnd()};this._verticalScrollbar=this._register(new vae(this._scrollable,this._options,n)),this._horizontalScrollbar=this._register(new Cae(this._scrollable,this._options,n)),this._domNode=document.createElement("div"),this._domNode.className="monaco-scrollable-element "+this._options.className,this._domNode.setAttribute("role","presentation"),this._domNode.style.position="relative",this._domNode.style.overflow="hidden",this._domNode.appendChild(e),this._domNode.appendChild(this._horizontalScrollbar.domNode.domNode),this._domNode.appendChild(this._verticalScrollbar.domNode.domNode),this._options.useShadows?(this._leftShadowDomNode=St(document.createElement("div")),this._leftShadowDomNode.setClassName("shadow"),this._domNode.appendChild(this._leftShadowDomNode.domNode),this._topShadowDomNode=St(document.createElement("div")),this._topShadowDomNode.setClassName("shadow"),this._domNode.appendChild(this._topShadowDomNode.domNode),this._topLeftShadowDomNode=St(document.createElement("div")),this._topLeftShadowDomNode.setClassName("shadow"),this._domNode.appendChild(this._topLeftShadowDomNode.domNode)):(this._leftShadowDomNode=null,this._topShadowDomNode=null,this._topLeftShadowDomNode=null),this._listenOnDomNode=this._options.listenOnDomNode||this._domNode,this._mouseWheelToDispose=[],this._setListeningToMouseWheel(this._options.handleMouseWheel),this.onmouseover(this._listenOnDomNode,s=>this._onMouseOver(s)),this.onmouseleave(this._listenOnDomNode,s=>this._onMouseLeave(s)),this._hideTimeout=this._register(new ea),this._isDragging=!1,this._mouseIsOver=!1,this._shouldRender=!0,this._revealOnScroll=!0}dispose(){this._mouseWheelToDispose=Tt(this._mouseWheelToDispose),super.dispose()}getDomNode(){return this._domNode}getOverviewRulerLayoutInfo(){return{parent:this._domNode,insertBefore:this._verticalScrollbar.domNode.domNode}}delegateVerticalScrollbarPointerDown(e){this._verticalScrollbar.delegatePointerDown(e)}getScrollDimensions(){return this._scrollable.getScrollDimensions()}setScrollDimensions(e){this._scrollable.setScrollDimensions(e,!1)}updateClassName(e){this._options.className=e,Je&&(this._options.className+=" mac"),this._domNode.className="monaco-scrollable-element "+this._options.className}updateOptions(e){typeof e.handleMouseWheel<"u"&&(this._options.handleMouseWheel=e.handleMouseWheel,this._setListeningToMouseWheel(this._options.handleMouseWheel)),typeof e.mouseWheelScrollSensitivity<"u"&&(this._options.mouseWheelScrollSensitivity=e.mouseWheelScrollSensitivity),typeof e.fastScrollSensitivity<"u"&&(this._options.fastScrollSensitivity=e.fastScrollSensitivity),typeof e.scrollPredominantAxis<"u"&&(this._options.scrollPredominantAxis=e.scrollPredominantAxis),typeof e.horizontal<"u"&&(this._options.horizontal=e.horizontal),typeof e.vertical<"u"&&(this._options.vertical=e.vertical),typeof e.horizontalScrollbarSize<"u"&&(this._options.horizontalScrollbarSize=e.horizontalScrollbarSize),typeof e.verticalScrollbarSize<"u"&&(this._options.verticalScrollbarSize=e.verticalScrollbarSize),typeof e.scrollByPage<"u"&&(this._options.scrollByPage=e.scrollByPage),this._horizontalScrollbar.updateOptions(this._options),this._verticalScrollbar.updateOptions(this._options),this._options.lazyRender||this._render()}delegateScrollFromMouseWheelEvent(e){this._onMouseWheel(new Df(e))}_setListeningToMouseWheel(e){if(this._mouseWheelToDispose.length>0!==e&&(this._mouseWheelToDispose=Tt(this._mouseWheelToDispose),e)){const i=n=>{this._onMouseWheel(new Df(n))};this._mouseWheelToDispose.push(U(this._listenOnDomNode,ie.MOUSE_WHEEL,i,{passive:!1}))}}_onMouseWheel(e){var s;if((s=e.browserEvent)!=null&&s.defaultPrevented)return;const t=hL.INSTANCE;t.acceptStandardWheelEvent(e);let i=!1;if(e.deltaY||e.deltaX){let r=e.deltaY*this._options.mouseWheelScrollSensitivity,a=e.deltaX*this._options.mouseWheelScrollSensitivity;this._options.scrollPredominantAxis&&(this._options.scrollYToX&&a+r===0?a=r=0:Math.abs(r)>=Math.abs(a)?a=0:r=0),this._options.flipAxes&&([r,a]=[a,r]);const l=!Je&&e.browserEvent&&e.browserEvent.shiftKey;(this._options.scrollYToX||l)&&!a&&(a=r,r=0),e.browserEvent&&e.browserEvent.altKey&&(a=a*this._options.fastScrollSensitivity,r=r*this._options.fastScrollSensitivity);const c=this._scrollable.getFutureScrollPosition();let d={};if(r){const h=R9*r,u=c.scrollTop-(h<0?Math.floor(h):Math.ceil(h));this._verticalScrollbar.writeScrollPosition(d,u)}if(a){const h=R9*a,u=c.scrollLeft-(h<0?Math.floor(h):Math.ceil(h));this._horizontalScrollbar.writeScrollPosition(d,u)}d=this._scrollable.validateScrollPosition(d),(c.scrollLeft!==d.scrollLeft||c.scrollTop!==d.scrollTop)&&(this._options.mouseWheelSmoothScroll&&t.isPhysicalMouseWheel()?this._scrollable.setScrollPositionSmooth(d):this._scrollable.setScrollPositionNow(d),i=!0)}let n=i;!n&&this._options.alwaysConsumeMouseWheel&&(n=!0),!n&&this._options.consumeMouseWheelIfScrollbarIsNeeded&&(this._verticalScrollbar.isNeeded()||this._horizontalScrollbar.isNeeded())&&(n=!0),n&&(e.preventDefault(),e.stopPropagation())}_onDidScroll(e){this._shouldRender=this._horizontalScrollbar.onDidScroll(e)||this._shouldRender,this._shouldRender=this._verticalScrollbar.onDidScroll(e)||this._shouldRender,this._options.useShadows&&(this._shouldRender=!0),this._revealOnScroll&&this._reveal(),this._options.lazyRender||this._render()}renderNow(){if(!this._options.lazyRender)throw new Error("Please use `lazyRender` together with `renderNow`!");this._render()}_render(){if(this._shouldRender&&(this._shouldRender=!1,this._horizontalScrollbar.render(),this._verticalScrollbar.render(),this._options.useShadows)){const e=this._scrollable.getCurrentScrollPosition(),t=e.scrollTop>0,i=e.scrollLeft>0,n=i?" left":"",s=t?" top":"",r=i||t?" top-left-corner":"";this._leftShadowDomNode.setClassName(`shadow${n}`),this._topShadowDomNode.setClassName(`shadow${s}`),this._topLeftShadowDomNode.setClassName(`shadow${r}${s}${n}`)}}_onDragStart(){this._isDragging=!0,this._reveal()}_onDragEnd(){this._isDragging=!1,this._hide()}_onMouseLeave(e){this._mouseIsOver=!1,this._hide()}_onMouseOver(e){this._mouseIsOver=!0,this._reveal()}_reveal(){this._verticalScrollbar.beginReveal(),this._horizontalScrollbar.beginReveal(),this._scheduleHide()}_hide(){!this._mouseIsOver&&!this._isDragging&&(this._verticalScrollbar.beginHide(),this._horizontalScrollbar.beginHide())}_scheduleHide(){!this._mouseIsOver&&!this._isDragging&&this._hideTimeout.cancelAndSet(()=>this._hide(),Lae)}}class Z$ extends F5{constructor(e,t){t=t||{},t.mouseWheelSmoothScroll=!1;const i=new wb({forceIntegerValues:!0,smoothScrollDuration:0,scheduleAtNextAnimationFrame:n=>go(xe(e),n)});super(e,t,i),this._register(i)}setScrollPosition(e){this._scrollable.setScrollPositionNow(e)}}class jI extends F5{constructor(e,t,i){super(e,t,i)}setScrollPosition(e){e.reuseAnimation?this._scrollable.setScrollPositionSmooth(e,e.reuseAnimation):this._scrollable.setScrollPositionNow(e)}getScrollPosition(){return this._scrollable.getCurrentScrollPosition()}}class z1 extends F5{constructor(e,t){t=t||{},t.mouseWheelSmoothScroll=!1;const i=new wb({forceIntegerValues:!1,smoothScrollDuration:0,scheduleAtNextAnimationFrame:n=>go(xe(e),n)});super(e,t,i),this._register(i),this._element=e,this._register(this.onScroll(n=>{n.scrollTopChanged&&(this._element.scrollTop=n.scrollTop),n.scrollLeftChanged&&(this._element.scrollLeft=n.scrollLeft)})),this.scanDomNode()}setScrollPosition(e){this._scrollable.setScrollPositionNow(e)}getScrollPosition(){return this._scrollable.getCurrentScrollPosition()}scanDomNode(){this.setScrollDimensions({width:this._element.clientWidth,scrollWidth:this._element.scrollWidth,height:this._element.clientHeight,scrollHeight:this._element.scrollHeight}),this.setScrollPosition({scrollLeft:this._element.scrollLeft,scrollTop:this._element.scrollTop})}}function kae(o){const e={lazyRender:typeof o.lazyRender<"u"?o.lazyRender:!1,className:typeof o.className<"u"?o.className:"",useShadows:typeof o.useShadows<"u"?o.useShadows:!0,handleMouseWheel:typeof o.handleMouseWheel<"u"?o.handleMouseWheel:!0,flipAxes:typeof o.flipAxes<"u"?o.flipAxes:!1,consumeMouseWheelIfScrollbarIsNeeded:typeof o.consumeMouseWheelIfScrollbarIsNeeded<"u"?o.consumeMouseWheelIfScrollbarIsNeeded:!1,alwaysConsumeMouseWheel:typeof o.alwaysConsumeMouseWheel<"u"?o.alwaysConsumeMouseWheel:!1,scrollYToX:typeof o.scrollYToX<"u"?o.scrollYToX:!1,mouseWheelScrollSensitivity:typeof o.mouseWheelScrollSensitivity<"u"?o.mouseWheelScrollSensitivity:1,fastScrollSensitivity:typeof o.fastScrollSensitivity<"u"?o.fastScrollSensitivity:5,scrollPredominantAxis:typeof o.scrollPredominantAxis<"u"?o.scrollPredominantAxis:!0,mouseWheelSmoothScroll:typeof o.mouseWheelSmoothScroll<"u"?o.mouseWheelSmoothScroll:!0,arrowSize:typeof o.arrowSize<"u"?o.arrowSize:11,listenOnDomNode:typeof o.listenOnDomNode<"u"?o.listenOnDomNode:null,horizontal:typeof o.horizontal<"u"?o.horizontal:1,horizontalScrollbarSize:typeof o.horizontalScrollbarSize<"u"?o.horizontalScrollbarSize:10,horizontalSliderSize:typeof o.horizontalSliderSize<"u"?o.horizontalSliderSize:0,horizontalHasArrows:typeof o.horizontalHasArrows<"u"?o.horizontalHasArrows:!1,vertical:typeof o.vertical<"u"?o.vertical:1,verticalScrollbarSize:typeof o.verticalScrollbarSize<"u"?o.verticalScrollbarSize:10,verticalHasArrows:typeof o.verticalHasArrows<"u"?o.verticalHasArrows:!1,verticalSliderSize:typeof o.verticalSliderSize<"u"?o.verticalSliderSize:0,scrollByPage:typeof o.scrollByPage<"u"?o.scrollByPage:!1};return e.horizontalSliderSize=typeof o.horizontalSliderSize<"u"?o.horizontalSliderSize:e.horizontalScrollbarSize,e.verticalSliderSize=typeof o.verticalSliderSize<"u"?o.verticalSliderSize:e.verticalScrollbarSize,Je&&(e.className+=" mac"),e}const BS=ne;let W5=class extends F{constructor(){super(),this.containerDomNode=document.createElement("div"),this.containerDomNode.className="monaco-hover",this.containerDomNode.tabIndex=0,this.containerDomNode.setAttribute("role","tooltip"),this.contentsDomNode=document.createElement("div"),this.contentsDomNode.className="monaco-hover-content",this.scrollbar=this._register(new z1(this.contentsDomNode,{consumeMouseWheelIfScrollbarIsNeeded:!0})),this.containerDomNode.appendChild(this.scrollbar.getDomNode())}onContentsChanged(){this.scrollbar.scanDomNode()}};class KI extends F{static render(e,t,i){return new KI(e,t,i)}constructor(e,t,i){super(),this.actionLabel=t.label,this.actionKeybindingLabel=i,this.actionContainer=X(e,BS("div.action-container")),this.actionContainer.setAttribute("tabindex","0"),this.action=X(this.actionContainer,BS("a.action")),this.action.setAttribute("role","button"),t.iconClass&&X(this.action,BS(`span.icon.${t.iconClass}`));const n=X(this.action,BS("span"));n.textContent=i?`${t.label} (${i})`:t.label,this._store.add(new X$(this.actionContainer,t.run)),this._store.add(new Q$(this.actionContainer,t.run,[3,10])),this.setEnabled(!0)}setEnabled(e){e?(this.actionContainer.classList.remove("disabled"),this.actionContainer.removeAttribute("aria-disabled")):(this.actionContainer.classList.add("disabled"),this.actionContainer.setAttribute("aria-disabled","true"))}}function Y$(o,e){return o&&e?m("acessibleViewHint","Inspect this in the accessible view with {0}.",e):o?m("acessibleViewHintNoKbOpen","Inspect this in the accessible view via the command Open Accessible View which is currently not triggerable via keybinding."):""}class X$ extends F{constructor(e,t){super(),this._register(U(e,ie.CLICK,i=>{i.stopPropagation(),i.preventDefault(),t(e)}))}}class Q$ extends F{constructor(e,t,i){super(),this._register(U(e,ie.KEY_DOWN,n=>{const s=new Pt(n);i.some(r=>s.equals(r))&&(n.stopPropagation(),n.preventDefault(),t(e))}))}}const zs=et("openerService");function Dae(o){let e;const t=/^L?(\d+)(?:,(\d+))?(-L?(\d+)(?:,(\d+))?)?/.exec(o.fragment);return t&&(e={startLineNumber:parseInt(t[1]),startColumn:t[2]?parseInt(t[2]):1,endLineNumber:t[4]?parseInt(t[4]):void 0,endColumn:t[4]?t[5]?parseInt(t[5]):1:void 0},o=o.with({fragment:""})),{selection:e,uri:o}}class tt{get event(){return this.emitter.event}constructor(e,t,i){const n=s=>this.emitter.fire(s);this.emitter=new P({onWillAddFirstListener:()=>e.addEventListener(t,n,i),onDidRemoveLastListener:()=>e.removeEventListener(t,n,i)})}dispose(){this.emitter.dispose()}}function Iae(o,e={}){const t=B5(e);return t.textContent=o,t}function Eae(o,e={}){const t=B5(e);return J$(t,Tae(o,!!e.renderCodeSegments),e.actionHandler,e.renderCodeSegments),t}function B5(o){const e=o.inline?"span":"div",t=document.createElement(e);return o.className&&(t.className=o.className),t}class Nae{constructor(e){this.source=e,this.index=0}eos(){return this.index>=this.source.length}next(){const e=this.peek();return this.advance(),e}peek(){return this.source[this.index]}advance(){this.index++}}function J$(o,e,t,i){let n;if(e.type===2)n=document.createTextNode(e.content||"");else if(e.type===3)n=document.createElement("b");else if(e.type===4)n=document.createElement("i");else if(e.type===7&&i)n=document.createElement("code");else if(e.type===5&&t){const s=document.createElement("a");t.disposables.add(hi(s,"click",r=>{t.callback(String(e.index),r)})),n=s}else e.type===8?n=document.createElement("br"):e.type===1&&(n=o);n&&o!==n&&o.appendChild(n),n&&Array.isArray(e.children)&&e.children.forEach(s=>{J$(n,s,t,i)})}function Tae(o,e){const t={type:1,children:[]};let i=0,n=t;const s=[],r=new Nae(o);for(;!r.eos();){let a=r.next();const l=a==="\\"&&m2(r.peek(),e)!==0;if(l&&(a=r.next()),!l&&Rae(a,e)&&a===r.peek()){r.advance(),n.type===2&&(n=s.pop());const c=m2(a,e);if(n.type===c||n.type===5&&c===6)n=s.pop();else{const d={type:c,children:[]};c===5&&(d.index=i,i++),n.children.push(d),s.push(n),n=d}}else if(a===`
`)n.type===2&&(n=s.pop()),n.children.push({type:8});else if(n.type!==2){const c={type:2,content:a};n.children.push(c),s.push(n),n=c}else n.content+=a}return n.type===2&&(n=s.pop()),t}function Rae(o,e){return m2(o,e)!==0}function m2(o,e){switch(o){case"*":return 3;case"_":return 4;case"[":return 5;case"]":return 6;case"`":return e?7:0;default:return 0}}const Mae=new RegExp(`(\\\\)?\\$\\((${Ie.iconNameExpression}(?:${Ie.iconModifierExpression})?)\\)`,"g");function dh(o){const e=new Array;let t,i=0,n=0;for(;(t=Mae.exec(o))!==null;){n=t.index||0,i<n&&e.push(o.substring(i,n)),i=(t.index||0)+t[0].length;const[,s,r]=t;e.push(s?`$(${r})`:Rf({id:r}))}return i<o.length&&e.push(o.substring(i)),e}function Rf(o){const e=ne("span");return e.classList.add(...Ie.asClassNameArray(o)),e}function Aae(o){const e=Pae(o);if(e&&e.length>0)return new Uint32Array(e)}let ko=0;const Wd=new Uint32Array(10);function Pae(o){if(ko=0,Xa(o,BN,4352),ko>0||(Xa(o,HN,4449),ko>0)||(Xa(o,VN,4520),ko>0)||(Xa(o,pu,12593),ko))return Wd.subarray(0,ko);if(o>=44032&&o<=55203){const e=o-44032,t=e%588,i=Math.floor(e/588),n=Math.floor(t/28),s=t%28-1;if(i<BN.length?Xa(i,BN,0):4352+i-12593<pu.length&&Xa(4352+i,pu,12593),n<HN.length?Xa(n,HN,0):4449+n-12593<pu.length&&Xa(4449+n-12593,pu,12593),s>=0&&(s<VN.length?Xa(s,VN,0):4520+s-12593<pu.length&&Xa(4520+s-12593,pu,12593)),ko>0)return Wd.subarray(0,ko)}}function Xa(o,e,t){o>=t&&o<t+e.length&&Oae(e[o-t])}function Oae(o){o!==0&&(Wd[ko++]=o&255,o>>8&&(Wd[ko++]=o>>8&255),o>>16&&(Wd[ko++]=o>>16&255))}const BN=new Uint8Array([114,82,115,101,69,102,97,113,81,116,84,100,119,87,99,122,120,118,103]),HN=new Uint16Array([107,111,105,79,106,112,117,80,104,27496,28520,27752,121,110,27246,28782,27758,98,109,27757,108]),VN=new Uint16Array([114,82,29810,115,30579,26483,101,102,29286,24934,29030,29798,30822,30310,26470,97,113,29809,116,84,100,119,99,122,120,118,103]),pu=new Uint16Array([114,82,29810,115,30579,26483,101,69,102,29286,24934,29030,29798,30822,30310,26470,97,113,81,29809,116,84,100,119,87,99,122,120,118,103,107,111,105,79,106,112,117,80,104,27496,28520,27752,121,110,27246,28782,27758,98,109,27757,108]);function H5(...o){return function(e,t){for(let i=0,n=o.length;i<n;i++){const s=o[i](e,t);if(s)return s}return null}}ej.bind(void 0,!1);const Sw=ej.bind(void 0,!0);function ej(o,e,t){if(!t||t.length<e.length)return null;let i;return o?i=Y4(t,e):i=t.indexOf(e)===0,i?e.length>0?[{start:0,end:e.length}]:[]:null}function tj(o,e){const t=e.toLowerCase().indexOf(o.toLowerCase());return t===-1?null:[{start:t,end:t+o.length}]}function ij(o,e){return p2(o.toLowerCase(),e.toLowerCase(),0,0)}function p2(o,e,t,i){if(t===o.length)return[];if(i===e.length)return null;if(o[t]===e[i]){let n=null;return(n=p2(o,e,t+1,i+1))?U5({start:i,end:i+1},n):null}return p2(o,e,t,i+1)}function V5(o){return 97<=o&&o<=122}function qI(o){return 65<=o&&o<=90}function z5(o){return 48<=o&&o<=57}function nj(o){return o===32||o===9||o===10||o===13}const sj=new Set;"()[]{}<>`'\"-/;:,.?!".split("").forEach(o=>sj.add(o.charCodeAt(0)));function uL(o){return nj(o)||sj.has(o)}function M9(o,e){return o===e||uL(o)&&uL(e)}const zN=new Map;function A9(o){if(zN.has(o))return zN.get(o);let e;const t=Aae(o);return t&&(e=t),zN.set(o,e),e}function oj(o){return V5(o)||qI(o)||z5(o)}function U5(o,e){return e.length===0?e=[o]:o.end===e[0].start?e[0].start=o.start:e.unshift(o),e}function rj(o,e){for(let t=e;t<o.length;t++){const i=o.charCodeAt(t);if(qI(i)||z5(i)||t>0&&!oj(o.charCodeAt(t-1)))return t}return o.length}function _2(o,e,t,i){if(t===o.length)return[];if(i===e.length)return null;if(o[t]!==e[i].toLowerCase())return null;{let n=null,s=i+1;for(n=_2(o,e,t+1,i+1);!n&&(s=rj(e,s))<e.length;)n=_2(o,e,t+1,s),s++;return n===null?null:U5({start:i,end:i+1},n)}}function Fae(o){let e=0,t=0,i=0,n=0,s=0;for(let d=0;d<o.length;d++)s=o.charCodeAt(d),qI(s)&&e++,V5(s)&&t++,oj(s)&&i++,z5(s)&&n++;const r=e/o.length,a=t/o.length,l=i/o.length,c=n/o.length;return{upperPercent:r,lowerPercent:a,alphaPercent:l,numericPercent:c}}function Wae(o){const{upperPercent:e,lowerPercent:t}=o;return t===0&&e>.6}function Bae(o){const{upperPercent:e,lowerPercent:t,alphaPercent:i,numericPercent:n}=o;return t>.2&&e<.8&&i>.6&&n<.2}function Hae(o){let e=0,t=0,i=0,n=0;for(let s=0;s<o.length;s++)i=o.charCodeAt(s),qI(i)&&e++,V5(i)&&t++,nj(i)&&n++;return(e===0||t===0)&&n===0?o.length<=30:e<=5}function aj(o,e){if(!e||(e=e.trim(),e.length===0)||!Hae(o))return null;e.length>60&&(e=e.substring(0,60));const t=Fae(e);if(!Bae(t)){if(!Wae(t))return null;e=e.toLowerCase()}let i=null,n=0;for(o=o.toLowerCase();n<e.length&&(i=_2(o,e,0,n))===null;)n=rj(e,n+1);return i}function Vae(o,e,t=!1){if(!e||e.length===0)return null;let i=null,n=0;for(o=o.toLowerCase(),e=e.toLowerCase();n<e.length&&(i=b2(o,e,0,n,t),i===null);)n=lj(e,n+1);return i}function b2(o,e,t,i,n){let s=0;if(t===o.length)return[];if(i===e.length)return null;if(!M9(o.charCodeAt(t),e.charCodeAt(i))){const l=A9(o.charCodeAt(t));if(!l)return null;for(let c=0;c<l.length;c++)if(!M9(l[c],e.charCodeAt(i+c)))return null;s+=l.length-1}let r=null,a=i+s+1;if(r=b2(o,e,t+1,a,n),!n)for(;!r&&(a=lj(e,a))<e.length;)r=b2(o,e,t+1,a,n),a++;if(!r)return null;if(o.charCodeAt(t)!==e.charCodeAt(i)){const l=A9(o.charCodeAt(t));if(!l)return r;for(let c=0;c<l.length;c++)if(l[c]!==e.charCodeAt(i+c))return r}return U5({start:i,end:i+s+1},r)}function lj(o,e){for(let t=e;t<o.length;t++)if(uL(o.charCodeAt(t))||t>0&&uL(o.charCodeAt(t-1)))return t;return o.length}const zae=H5(Sw,aj,tj),Uae=H5(Sw,aj,ij),P9=new Gl(1e4);function O9(o,e,t=!1){if(typeof o!="string"||typeof e!="string")return null;let i=P9.get(o);i||(i=new RegExp(Nee(o),"i"),P9.set(o,i));const n=i.exec(e);return n?[{start:n.index,end:n.index+n[0].length}]:t?Uae(o,e):zae(o,e)}function $ae(o,e){const t=Mf(o,o.toLowerCase(),0,e,e.toLowerCase(),0,{firstMatchCanBeWeak:!0,boostFullMatch:!0});return t?U1(t):null}function jae(o,e,t,i,n,s){const r=Math.min(13,o.length);for(;t<r;t++){const a=Mf(o,e,t,i,n,s,{firstMatchCanBeWeak:!0,boostFullMatch:!0});if(a)return a}return[0,s]}function U1(o){if(typeof o>"u")return[];const e=[],t=o[1];for(let i=o.length-1;i>1;i--){const n=o[i]+t,s=e[e.length-1];s&&s.end===n?s.end=n+1:e.push({start:n,end:n+1})}return e}const Yd=128;function $5(){const o=[],e=[];for(let t=0;t<=Yd;t++)e[t]=0;for(let t=0;t<=Yd;t++)o.push(e.slice(0));return o}function cj(o){const e=[];for(let t=0;t<=o;t++)e[t]=0;return e}const dj=cj(2*Yd),C2=cj(2*Yd),nc=$5(),_u=$5(),HS=$5();function VS(o,e){if(e<0||e>=o.length)return!1;const t=o.codePointAt(e);switch(t){case 95:case 45:case 46:case 32:case 47:case 92:case 39:case 34:case 58:case 36:case 60:case 62:case 40:case 41:case 91:case 93:case 123:case 125:return!0;case void 0:return!1;default:return!!e5(t)}}function F9(o,e){if(e<0||e>=o.length)return!1;switch(o.charCodeAt(e)){case 32:case 9:return!0;default:return!1}}function Y0(o,e,t){return e[o]!==t[o]}function Kae(o,e,t,i,n,s,r=!1){for(;e<t&&n<s;)o[e]===i[n]&&(r&&(dj[e]=n),e+=1),n+=1;return e===t}var Ma;(function(o){o.Default=[-100,0];function e(t){return!t||t.length===2&&t[0]===-100&&t[1]===0}o.isDefault=e})(Ma||(Ma={}));const SB=class SB{constructor(e,t){this.firstMatchCanBeWeak=e,this.boostFullMatch=t}};SB.default={boostFullMatch:!0,firstMatchCanBeWeak:!1};let yw=SB;function Mf(o,e,t,i,n,s,r=yw.default){const a=o.length>Yd?Yd:o.length,l=i.length>Yd?Yd:i.length;if(t>=a||s>=l||a-t>l-s||!Kae(e,t,a,n,s,l,!0))return;qae(a,l,t,s,e,n);let c=1,d=1,h=t,u=s;const g=[!1];for(c=1,h=t;h<a;c++,h++){const C=dj[h],v=C2[h],w=h+1<a?C2[h+1]:l;for(d=C-s+1,u=C;u<w;d++,u++){let S=Number.MIN_SAFE_INTEGER,L=!1;u<=v&&(S=Gae(o,e,h,t,i,n,u,l,s,nc[c-1][d-1]===0,g));let k=0;S!==Number.MAX_SAFE_INTEGER&&(L=!0,k=S+_u[c-1][d-1]);const D=u>C,N=D?_u[c][d-1]+(nc[c][d-1]>0?-5:0):0,B=u>C+1&&nc[c][d-1]>0,W=B?_u[c][d-2]+(nc[c][d-2]>0?-5:0):0;if(B&&(!D||W>=N)&&(!L||W>=k))_u[c][d]=W,HS[c][d]=3,nc[c][d]=0;else if(D&&(!L||N>=k))_u[c][d]=N,HS[c][d]=2,nc[c][d]=0;else if(L)_u[c][d]=k,HS[c][d]=1,nc[c][d]=nc[c-1][d-1]+1;else throw new Error("not possible")}}if(!g[0]&&!r.firstMatchCanBeWeak)return;c--,d--;const f=[_u[c][d],s];let p=0,_=0;for(;c>=1;){let C=d;do{const v=HS[c][C];if(v===3)C=C-2;else if(v===2)C=C-1;else break}while(C>=1);p>1&&e[t+c-1]===n[s+d-1]&&!Y0(C+s-1,i,n)&&p+1>nc[c][C]&&(C=d),C===d?p++:p=1,_||(_=C),c--,d=C-1,f.push(d)}l-s===a&&r.boostFullMatch&&(f[0]+=2);const b=_-a;return f[0]-=b,f}function qae(o,e,t,i,n,s){let r=o-1,a=e-1;for(;r>=t&&a>=i;)n[r]===s[a]&&(C2[r]=a,r--),a--}function Gae(o,e,t,i,n,s,r,a,l,c,d){if(e[t]!==s[r])return Number.MIN_SAFE_INTEGER;let h=1,u=!1;return r===t-i?h=o[t]===n[r]?7:5:Y0(r,n,s)&&(r===0||!Y0(r-1,n,s))?(h=o[t]===n[r]?7:5,u=!0):VS(s,r)&&(r===0||!VS(s,r-1))?h=5:(VS(s,r-1)||F9(s,r-1))&&(h=5,u=!0),h>1&&t===i&&(d[0]=!0),u||(u=Y0(r,n,s)||VS(s,r-1)||F9(s,r-1)),t===i?r>l&&(h-=u?3:5):c?h+=u?2:0:h+=u?0:1,r+1===a&&(h-=u?3:5),h}function Zae(o,e,t,i,n,s,r){return Yae(o,e,t,i,n,s,!0,r)}function Yae(o,e,t,i,n,s,r,a){let l=Mf(o,e,t,i,n,s,a);if(o.length>=3){const c=Math.min(7,o.length-1);for(let d=t+1;d<c;d++){const h=Xae(o,d);if(h){const u=Mf(h,h.toLowerCase(),t,i,n,s,a);u&&(u[0]-=3,(!l||u[0]>l[0])&&(l=u))}}}return l}function Xae(o,e){if(e+1>=o.length)return;const t=o[e],i=o[e+1];if(t!==i)return o.slice(0,e)+i+t+o.slice(e+2)}const Qae="$(",j5=new RegExp(`\\$\\(${Ie.iconNameExpression}(?:${Ie.iconModifierExpression})?\\)`,"g"),Jae=new RegExp(`(\\\\)?${j5.source}`,"g");function ele(o){return o.replace(Jae,(e,t)=>t?e:`\\${e}`)}const tle=new RegExp(`\\\\${j5.source}`,"g");function ile(o){return o.replace(tle,e=>`\\${e}`)}const nle=new RegExp(`(\\s)?(\\\\)?${j5.source}(\\s)?`,"g");function K5(o){return o.indexOf(Qae)===-1?o:o.replace(nle,(e,t,i,n)=>i?e:t||n||"")}function sle(o){return o?o.replace(/\$\((.*?)\)/g,(e,t)=>` ${t} `).trim():""}const UN=new RegExp(`\\$\\(${Ie.iconNameCharacter}+\\)`,"g");function xC(o){UN.lastIndex=0;let e="";const t=[];let i=0;for(;;){const n=UN.lastIndex,s=UN.exec(o),r=o.substring(n,s==null?void 0:s.index);if(r.length>0){e+=r;for(let a=0;a<r.length;a++)t.push(i)}if(!s)break;i+=s[0].length}return{text:e,iconOffsets:t}}function $N(o,e,t=!1){const{text:i,iconOffsets:n}=e;if(!n||n.length===0)return O9(o,i,t);const s=M1(i," "),r=i.length-s.length,a=O9(o,s,t);if(a)for(const l of a){const c=n[l.start+r]+r;l.start+=c,l.end+=c}return a}function rl(o){return Py(o,!0)}class ole{constructor(e){this._ignorePathCasing=e}compare(e,t,i=!1){return e===t?0:sw(this.getComparisonKey(e,i),this.getComparisonKey(t,i))}isEqual(e,t,i=!1){return e===t?!0:!e||!t?!1:this.getComparisonKey(e,i)===this.getComparisonKey(t,i)}getComparisonKey(e,t=!1){return e.with({path:this._ignorePathCasing(e)?e.path.toLowerCase():void 0,fragment:t?null:void 0}).toString()}isEqualOrParent(e,t,i=!1){if(e.scheme===t.scheme){if(e.scheme===Fe.file)return r2(rl(e),rl(t),this._ignorePathCasing(e))&&e.query===t.query&&(i||e.fragment===t.fragment);if(B9(e.authority,t.authority))return r2(e.path,t.path,this._ignorePathCasing(e),"/")&&e.query===t.query&&(i||e.fragment===t.fragment)}return!1}joinPath(e,...t){return Se.joinPath(e,...t)}basenameOrAuthority(e){return vr(e)||e.authority}basename(e){return Ai.basename(e.path)}extname(e){return Ai.extname(e.path)}dirname(e){if(e.path.length===0)return e;let t;return e.scheme===Fe.file?t=Se.file($z(rl(e))).path:(t=Ai.dirname(e.path),e.authority&&t.length&&t.charCodeAt(0)!==47&&(console.error(`dirname("${e.toString})) resulted in a relative path`),t="/")),e.with({path:t})}normalizePath(e){if(!e.path.length)return e;let t;return e.scheme===Fe.file?t=Se.file(Uz(rl(e))).path:t=Ai.normalize(e.path),e.with({path:t})}relativePath(e,t){if(e.scheme!==t.scheme||!B9(e.authority,t.authority))return;if(e.scheme===Fe.file){const s=aee(rl(e),rl(t));return En?y$(s):s}let i=e.path||"/";const n=t.path||"/";if(this._ignorePathCasing(e)){let s=0;for(const r=Math.min(i.length,n.length);s<r&&!(i.charCodeAt(s)!==n.charCodeAt(s)&&i.charAt(s).toLowerCase()!==n.charAt(s).toLowerCase());s++);i=n.substr(0,s)+i.substr(s)}return Ai.relative(i,n)}resolvePath(e,t){if(e.scheme===Fe.file){const i=Se.file(ree(rl(e),t));return e.with({authority:i.authority,path:i.path})}return t=loe(t),e.with({path:Ai.resolve(e.path,t)})}isAbsolutePath(e){return!!e.path&&e.path[0]==="/"}isEqualAuthority(e,t){return e===t||e!==void 0&&t!==void 0&&ep(e,t)}hasTrailingPathSeparator(e,t=Ia){if(e.scheme===Fe.file){const i=rl(e);return i.length>h9(i).length&&i[i.length-1]===t}else{const i=e.path;return i.length>1&&i.charCodeAt(i.length-1)===47&&!/^[a-zA-Z]:(\/$|\\$)/.test(e.fsPath)}}removeTrailingPathSeparator(e,t=Ia){return H9(e,t)?e.with({path:e.path.substr(0,e.path.length-1)}):e}addTrailingPathSeparator(e,t=Ia){let i=!1;if(e.scheme===Fe.file){const n=rl(e);i=n!==void 0&&n.length===h9(n).length&&n[n.length-1]===t}else{t="/";const n=e.path;i=n.length===1&&n.charCodeAt(n.length-1)===47}return!i&&!H9(e,t)?e.with({path:e.path+"/"}):e}}const Qt=new ole(()=>!1),Lw=Qt.isEqual.bind(Qt);Qt.isEqualOrParent.bind(Qt);Qt.getComparisonKey.bind(Qt);const rle=Qt.basenameOrAuthority.bind(Qt),vr=Qt.basename.bind(Qt),ale=Qt.extname.bind(Qt),GI=Qt.dirname.bind(Qt),lle=Qt.joinPath.bind(Qt),cle=Qt.normalizePath.bind(Qt),dle=Qt.relativePath.bind(Qt),W9=Qt.resolvePath.bind(Qt);Qt.isAbsolutePath.bind(Qt);const B9=Qt.isEqualAuthority.bind(Qt),H9=Qt.hasTrailingPathSeparator.bind(Qt);Qt.removeTrailingPathSeparator.bind(Qt);Qt.addTrailingPathSeparator.bind(Qt);var Uh;(function(o){o.META_DATA_LABEL="label",o.META_DATA_DESCRIPTION="description",o.META_DATA_SIZE="size",o.META_DATA_MIME="mime";function e(t){const i=new Map;t.path.substring(t.path.indexOf(";")+1,t.path.lastIndexOf(";")).split(";").forEach(r=>{const[a,l]=r.split(":");a&&l&&i.set(a,l)});const s=t.path.substring(0,t.path.indexOf(";"));return s&&i.set(o.META_DATA_MIME,s),i}o.parseMetaData=e})(Uh||(Uh={}));class Pn{constructor(e="",t=!1){if(this.value=e,typeof this.value!="string")throw gr("value");typeof t=="boolean"?(this.isTrusted=t,this.supportThemeIcons=!1,this.supportHtml=!1):(this.isTrusted=t.isTrusted??void 0,this.supportThemeIcons=t.supportThemeIcons??!1,this.supportHtml=t.supportHtml??!1)}appendText(e,t=0){return this.value+=ule(this.supportThemeIcons?ele(e):e).replace(/([ \t]+)/g,(i,n)=>"&nbsp;".repeat(n.length)).replace(/\>/gm,"\\>").replace(/\n/g,t===1?`\\
`:`

`),this}appendMarkdown(e){return this.value+=e,this}appendCodeblock(e,t){return this.value+=`
${gle(t,e)}
`,this}appendLink(e,t,i){return this.value+="[",this.value+=this._escape(t,"]"),this.value+="](",this.value+=this._escape(String(e),")"),i&&(this.value+=` "${this._escape(this._escape(i,'"'),")")}"`),this.value+=")",this}_escape(e,t){const i=new RegExp(Uo(t),"g");return e.replace(i,(n,s)=>e.charAt(s-1)!=="\\"?`\\${n}`:n)}}function O_(o){return Aa(o)?!o.value:Array.isArray(o)?o.every(O_):!0}function Aa(o){return o instanceof Pn?!0:o&&typeof o=="object"?typeof o.value=="string"&&(typeof o.isTrusted=="boolean"||typeof o.isTrusted=="object"||o.isTrusted===void 0)&&(typeof o.supportThemeIcons=="boolean"||o.supportThemeIcons===void 0):!1}function hle(o,e){return o===e?!0:!o||!e?!1:o.value===e.value&&o.isTrusted===e.isTrusted&&o.supportThemeIcons===e.supportThemeIcons&&o.supportHtml===e.supportHtml&&(o.baseUri===e.baseUri||!!o.baseUri&&!!e.baseUri&&Lw(Se.from(o.baseUri),Se.from(e.baseUri)))}function ule(o){return o.replace(/[\\`*_{}[\]()#+\-!~]/g,"\\$&")}function gle(o,e){var n;const t=((n=o.match(/^`+/gm))==null?void 0:n.reduce((s,r)=>s.length>r.length?s:r).length)??0,i=t>=3?t+1:3;return[`${"`".repeat(i)}${e}`,o,`${"`".repeat(i)}`].join(`
`)}function zS(o){return o.replace(/"/g,"&quot;")}function jN(o){return o&&o.replace(/\\([\\`*_{}[\]()#+\-.!~])/g,"$1")}function fle(o){const e=[],t=o.split("|").map(n=>n.trim());o=t[0];const i=t[1];if(i){const n=/height=(\d+)/.exec(i),s=/width=(\d+)/.exec(i),r=n?n[1]:"",a=s?s[1]:"",l=isFinite(parseInt(a)),c=isFinite(parseInt(r));l&&e.push(`width="${a}"`),c&&e.push(`height="${r}"`)}return{href:o,dimensions:e}}class q5{constructor(e){this._prefix=e,this._lastId=0}nextId(){return this._prefix+ ++this._lastId}}const v2=new q5("id#");let Hn={};(function(){function o(e,t){t(Hn)}o.amd=!0,function(e,t){typeof o=="function"&&o.amd?o(["exports"],t):typeof exports=="object"&&typeof module<"u"?t(exports):(e=typeof globalThis<"u"?globalThis:e||self,t(e.marked={}))}(this,function(e){function t(){return{async:!1,breaks:!1,extensions:null,gfm:!0,hooks:null,pedantic:!1,renderer:null,silent:!1,tokenizer:null,walkTokens:null}}e.defaults=t();function i(ut){e.defaults=ut}const n=/[&<>"']/,s=new RegExp(n.source,"g"),r=/[<>"']|&(?!(#\d{1,7}|#[Xx][a-fA-F0-9]{1,6}|\w+);)/,a=new RegExp(r.source,"g"),l={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"},c=ut=>l[ut];function d(ut,R){if(R){if(n.test(ut))return ut.replace(s,c)}else if(r.test(ut))return ut.replace(a,c);return ut}const h=/(^|[^\[])\^/g;function u(ut,R){let M=typeof ut=="string"?ut:ut.source;R=R||"";const O={replace:(z,q)=>{let ee=typeof q=="string"?q:q.source;return ee=ee.replace(h,"$1"),M=M.replace(z,ee),O},getRegex:()=>new RegExp(M,R)};return O}function g(ut){try{ut=encodeURI(ut).replace(/%25/g,"%")}catch{return null}return ut}const f={exec:()=>null};function p(ut,R){const M=ut.replace(/\|/g,(q,ee,ae)=>{let De=!1,Me=ee;for(;--Me>=0&&ae[Me]==="\\";)De=!De;return De?"|":" |"}),O=M.split(/ \|/);let z=0;if(O[0].trim()||O.shift(),O.length>0&&!O[O.length-1].trim()&&O.pop(),R)if(O.length>R)O.splice(R);else for(;O.length<R;)O.push("");for(;z<O.length;z++)O[z]=O[z].trim().replace(/\\\|/g,"|");return O}function _(ut,R,M){const O=ut.length;if(O===0)return"";let z=0;for(;z<O&&ut.charAt(O-z-1)===R;)z++;return ut.slice(0,O-z)}function b(ut,R){if(ut.indexOf(R[1])===-1)return-1;let M=0;for(let O=0;O<ut.length;O++)if(ut[O]==="\\")O++;else if(ut[O]===R[0])M++;else if(ut[O]===R[1]&&(M--,M<0))return O;return-1}function C(ut,R,M,O){const z=R.href,q=R.title?d(R.title):null,ee=ut[1].replace(/\\([\[\]])/g,"$1");if(ut[0].charAt(0)!=="!"){O.state.inLink=!0;const ae={type:"link",raw:M,href:z,title:q,text:ee,tokens:O.inlineTokens(ee)};return O.state.inLink=!1,ae}return{type:"image",raw:M,href:z,title:q,text:d(ee)}}function v(ut,R){const M=ut.match(/^(\s+)(?:```)/);if(M===null)return R;const O=M[1];return R.split(`
`).map(z=>{const q=z.match(/^\s+/);if(q===null)return z;const[ee]=q;return ee.length>=O.length?z.slice(O.length):z}).join(`
`)}class w{constructor(R){Fi(this,"options");Fi(this,"rules");Fi(this,"lexer");this.options=R||e.defaults}space(R){const M=this.rules.block.newline.exec(R);if(M&&M[0].length>0)return{type:"space",raw:M[0]}}code(R){const M=this.rules.block.code.exec(R);if(M){const O=M[0].replace(/^ {1,4}/gm,"");return{type:"code",raw:M[0],codeBlockStyle:"indented",text:this.options.pedantic?O:_(O,`
`)}}}fences(R){const M=this.rules.block.fences.exec(R);if(M){const O=M[0],z=v(O,M[3]||"");return{type:"code",raw:O,lang:M[2]?M[2].trim().replace(this.rules.inline.anyPunctuation,"$1"):M[2],text:z}}}heading(R){const M=this.rules.block.heading.exec(R);if(M){let O=M[2].trim();if(/#$/.test(O)){const z=_(O,"#");(this.options.pedantic||!z||/ $/.test(z))&&(O=z.trim())}return{type:"heading",raw:M[0],depth:M[1].length,text:O,tokens:this.lexer.inline(O)}}}hr(R){const M=this.rules.block.hr.exec(R);if(M)return{type:"hr",raw:_(M[0],`
`)}}blockquote(R){const M=this.rules.block.blockquote.exec(R);if(M){let O=_(M[0],`
`).split(`
`),z="",q="";const ee=[];for(;O.length>0;){let ae=!1;const De=[];let Me;for(Me=0;Me<O.length;Me++)if(/^ {0,3}>/.test(O[Me]))De.push(O[Me]),ae=!0;else if(!ae)De.push(O[Me]);else break;O=O.slice(Me);const at=De.join(`
`),Et=at.replace(/\n {0,3}((?:=+|-+) *)(?=\n|$)/g,`
    $1`).replace(/^ {0,3}>[ \t]?/gm,"");z=z?`${z}
${at}`:at,q=q?`${q}
${Et}`:Et;const Ft=this.lexer.state.top;if(this.lexer.state.top=!0,this.lexer.blockTokens(Et,ee,!0),this.lexer.state.top=Ft,O.length===0)break;const yi=ee[ee.length-1];if((yi==null?void 0:yi.type)==="code")break;if((yi==null?void 0:yi.type)==="blockquote"){const Li=yi,$s=Li.raw+`
`+O.join(`
`),er=this.blockquote($s);ee[ee.length-1]=er,z=z.substring(0,z.length-Li.raw.length)+er.raw,q=q.substring(0,q.length-Li.text.length)+er.text;break}else if((yi==null?void 0:yi.type)==="list"){const Li=yi,$s=Li.raw+`
`+O.join(`
`),er=this.list($s);ee[ee.length-1]=er,z=z.substring(0,z.length-yi.raw.length)+er.raw,q=q.substring(0,q.length-Li.raw.length)+er.raw,O=$s.substring(ee[ee.length-1].raw.length).split(`
`);continue}}return{type:"blockquote",raw:z,tokens:ee,text:q}}}list(R){let M=this.rules.block.list.exec(R);if(M){let O=M[1].trim();const z=O.length>1,q={type:"list",raw:"",ordered:z,start:z?+O.slice(0,-1):"",loose:!1,items:[]};O=z?`\\d{1,9}\\${O.slice(-1)}`:`\\${O}`,this.options.pedantic&&(O=z?O:"[*+-]");const ee=new RegExp(`^( {0,3}${O})((?:[	 ][^\\n]*)?(?:\\n|$))`);let ae=!1;for(;R;){let De=!1,Me="",at="";if(!(M=ee.exec(R))||this.rules.block.hr.test(R))break;Me=M[0],R=R.substring(Me.length);let Et=M[2].split(`
`,1)[0].replace(/^\t+/,zb=>" ".repeat(3*zb.length)),Ft=R.split(`
`,1)[0],yi=!Et.trim(),Li=0;if(this.options.pedantic?(Li=2,at=Et.trimStart()):yi?Li=M[1].length+1:(Li=M[2].search(/[^ ]/),Li=Li>4?1:Li,at=Et.slice(Li),Li+=M[1].length),yi&&/^ *$/.test(Ft)&&(Me+=Ft+`
`,R=R.substring(Ft.length+1),De=!0),!De){const zb=new RegExp(`^ {0,${Math.min(3,Li-1)}}(?:[*+-]|\\d{1,9}[.)])((?:[ 	][^\\n]*)?(?:\\n|$))`),Ye=new RegExp(`^ {0,${Math.min(3,Li-1)}}((?:- *){3,}|(?:_ *){3,}|(?:\\* *){3,})(?:\\n+|$)`),Y=new RegExp(`^ {0,${Math.min(3,Li-1)}}(?:\`\`\`|~~~)`),Le=new RegExp(`^ {0,${Math.min(3,Li-1)}}#`);for(;R;){const Ve=R.split(`
`,1)[0];if(Ft=Ve,this.options.pedantic&&(Ft=Ft.replace(/^ {1,4}(?=( {4})*[^ ])/g,"  ")),Y.test(Ft)||Le.test(Ft)||zb.test(Ft)||Ye.test(R))break;if(Ft.search(/[^ ]/)>=Li||!Ft.trim())at+=`
`+Ft.slice(Li);else{if(yi||Et.search(/[^ ]/)>=4||Y.test(Et)||Le.test(Et)||Ye.test(Et))break;at+=`
`+Ft}!yi&&!Ft.trim()&&(yi=!0),Me+=Ve+`
`,R=R.substring(Ve.length+1),Et=Ft.slice(Li)}}q.loose||(ae?q.loose=!0:/\n *\n *$/.test(Me)&&(ae=!0));let $s=null,er;this.options.gfm&&($s=/^\[[ xX]\] /.exec(at),$s&&(er=$s[0]!=="[ ] ",at=at.replace(/^\[[ xX]\] +/,""))),q.items.push({type:"list_item",raw:Me,task:!!$s,checked:er,loose:!1,text:at,tokens:[]}),q.raw+=Me}q.items[q.items.length-1].raw=q.items[q.items.length-1].raw.trimEnd(),q.items[q.items.length-1].text=q.items[q.items.length-1].text.trimEnd(),q.raw=q.raw.trimEnd();for(let De=0;De<q.items.length;De++)if(this.lexer.state.top=!1,q.items[De].tokens=this.lexer.blockTokens(q.items[De].text,[]),!q.loose){const Me=q.items[De].tokens.filter(Et=>Et.type==="space"),at=Me.length>0&&Me.some(Et=>/\n.*\n/.test(Et.raw));q.loose=at}if(q.loose)for(let De=0;De<q.items.length;De++)q.items[De].loose=!0;return q}}html(R){const M=this.rules.block.html.exec(R);if(M)return{type:"html",block:!0,raw:M[0],pre:M[1]==="pre"||M[1]==="script"||M[1]==="style",text:M[0]}}def(R){const M=this.rules.block.def.exec(R);if(M){const O=M[1].toLowerCase().replace(/\s+/g," "),z=M[2]?M[2].replace(/^<(.*)>$/,"$1").replace(this.rules.inline.anyPunctuation,"$1"):"",q=M[3]?M[3].substring(1,M[3].length-1).replace(this.rules.inline.anyPunctuation,"$1"):M[3];return{type:"def",tag:O,raw:M[0],href:z,title:q}}}table(R){const M=this.rules.block.table.exec(R);if(!M||!/[:|]/.test(M[2]))return;const O=p(M[1]),z=M[2].replace(/^\||\| *$/g,"").split("|"),q=M[3]&&M[3].trim()?M[3].replace(/\n[ \t]*$/,"").split(`
`):[],ee={type:"table",raw:M[0],header:[],align:[],rows:[]};if(O.length===z.length){for(const ae of z)/^ *-+: *$/.test(ae)?ee.align.push("right"):/^ *:-+: *$/.test(ae)?ee.align.push("center"):/^ *:-+ *$/.test(ae)?ee.align.push("left"):ee.align.push(null);for(let ae=0;ae<O.length;ae++)ee.header.push({text:O[ae],tokens:this.lexer.inline(O[ae]),header:!0,align:ee.align[ae]});for(const ae of q)ee.rows.push(p(ae,ee.header.length).map((De,Me)=>({text:De,tokens:this.lexer.inline(De),header:!1,align:ee.align[Me]})));return ee}}lheading(R){const M=this.rules.block.lheading.exec(R);if(M)return{type:"heading",raw:M[0],depth:M[2].charAt(0)==="="?1:2,text:M[1],tokens:this.lexer.inline(M[1])}}paragraph(R){const M=this.rules.block.paragraph.exec(R);if(M){const O=M[1].charAt(M[1].length-1)===`
`?M[1].slice(0,-1):M[1];return{type:"paragraph",raw:M[0],text:O,tokens:this.lexer.inline(O)}}}text(R){const M=this.rules.block.text.exec(R);if(M)return{type:"text",raw:M[0],text:M[0],tokens:this.lexer.inline(M[0])}}escape(R){const M=this.rules.inline.escape.exec(R);if(M)return{type:"escape",raw:M[0],text:d(M[1])}}tag(R){const M=this.rules.inline.tag.exec(R);if(M)return!this.lexer.state.inLink&&/^<a /i.test(M[0])?this.lexer.state.inLink=!0:this.lexer.state.inLink&&/^<\/a>/i.test(M[0])&&(this.lexer.state.inLink=!1),!this.lexer.state.inRawBlock&&/^<(pre|code|kbd|script)(\s|>)/i.test(M[0])?this.lexer.state.inRawBlock=!0:this.lexer.state.inRawBlock&&/^<\/(pre|code|kbd|script)(\s|>)/i.test(M[0])&&(this.lexer.state.inRawBlock=!1),{type:"html",raw:M[0],inLink:this.lexer.state.inLink,inRawBlock:this.lexer.state.inRawBlock,block:!1,text:M[0]}}link(R){const M=this.rules.inline.link.exec(R);if(M){const O=M[2].trim();if(!this.options.pedantic&&/^</.test(O)){if(!/>$/.test(O))return;const ee=_(O.slice(0,-1),"\\");if((O.length-ee.length)%2===0)return}else{const ee=b(M[2],"()");if(ee>-1){const De=(M[0].indexOf("!")===0?5:4)+M[1].length+ee;M[2]=M[2].substring(0,ee),M[0]=M[0].substring(0,De).trim(),M[3]=""}}let z=M[2],q="";if(this.options.pedantic){const ee=/^([^'"]*[^\s])\s+(['"])(.*)\2/.exec(z);ee&&(z=ee[1],q=ee[3])}else q=M[3]?M[3].slice(1,-1):"";return z=z.trim(),/^</.test(z)&&(this.options.pedantic&&!/>$/.test(O)?z=z.slice(1):z=z.slice(1,-1)),C(M,{href:z&&z.replace(this.rules.inline.anyPunctuation,"$1"),title:q&&q.replace(this.rules.inline.anyPunctuation,"$1")},M[0],this.lexer)}}reflink(R,M){let O;if((O=this.rules.inline.reflink.exec(R))||(O=this.rules.inline.nolink.exec(R))){const z=(O[2]||O[1]).replace(/\s+/g," "),q=M[z.toLowerCase()];if(!q){const ee=O[0].charAt(0);return{type:"text",raw:ee,text:ee}}return C(O,q,O[0],this.lexer)}}emStrong(R,M,O=""){let z=this.rules.inline.emStrongLDelim.exec(R);if(!z||z[3]&&O.match(/[\p{L}\p{N}]/u))return;if(!(z[1]||z[2]||"")||!O||this.rules.inline.punctuation.exec(O)){const ee=[...z[0]].length-1;let ae,De,Me=ee,at=0;const Et=z[0][0]==="*"?this.rules.inline.emStrongRDelimAst:this.rules.inline.emStrongRDelimUnd;for(Et.lastIndex=0,M=M.slice(-1*R.length+ee);(z=Et.exec(M))!=null;){if(ae=z[1]||z[2]||z[3]||z[4]||z[5]||z[6],!ae)continue;if(De=[...ae].length,z[3]||z[4]){Me+=De;continue}else if((z[5]||z[6])&&ee%3&&!((ee+De)%3)){at+=De;continue}if(Me-=De,Me>0)continue;De=Math.min(De,De+Me+at);const Ft=[...z[0]][0].length,yi=R.slice(0,ee+z.index+Ft+De);if(Math.min(ee,De)%2){const $s=yi.slice(1,-1);return{type:"em",raw:yi,text:$s,tokens:this.lexer.inlineTokens($s)}}const Li=yi.slice(2,-2);return{type:"strong",raw:yi,text:Li,tokens:this.lexer.inlineTokens(Li)}}}}codespan(R){const M=this.rules.inline.code.exec(R);if(M){let O=M[2].replace(/\n/g," ");const z=/[^ ]/.test(O),q=/^ /.test(O)&&/ $/.test(O);return z&&q&&(O=O.substring(1,O.length-1)),O=d(O,!0),{type:"codespan",raw:M[0],text:O}}}br(R){const M=this.rules.inline.br.exec(R);if(M)return{type:"br",raw:M[0]}}del(R){const M=this.rules.inline.del.exec(R);if(M)return{type:"del",raw:M[0],text:M[2],tokens:this.lexer.inlineTokens(M[2])}}autolink(R){const M=this.rules.inline.autolink.exec(R);if(M){let O,z;return M[2]==="@"?(O=d(M[1]),z="mailto:"+O):(O=d(M[1]),z=O),{type:"link",raw:M[0],text:O,href:z,tokens:[{type:"text",raw:O,text:O}]}}}url(R){var O;let M;if(M=this.rules.inline.url.exec(R)){let z,q;if(M[2]==="@")z=d(M[0]),q="mailto:"+z;else{let ee;do ee=M[0],M[0]=((O=this.rules.inline._backpedal.exec(M[0]))==null?void 0:O[0])??"";while(ee!==M[0]);z=d(M[0]),M[1]==="www."?q="http://"+M[0]:q=M[0]}return{type:"link",raw:M[0],text:z,href:q,tokens:[{type:"text",raw:z,text:z}]}}}inlineText(R){const M=this.rules.inline.text.exec(R);if(M){let O;return this.lexer.state.inRawBlock?O=M[0]:O=d(M[0]),{type:"text",raw:M[0],text:O}}}}const S=/^(?: *(?:\n|$))+/,L=/^( {4}[^\n]+(?:\n(?: *(?:\n|$))*)?)+/,k=/^ {0,3}(`{3,}(?=[^`\n]*(?:\n|$))|~{3,})([^\n]*)(?:\n|$)(?:|([\s\S]*?)(?:\n|$))(?: {0,3}\1[~`]* *(?=\n|$)|$)/,D=/^ {0,3}((?:-[\t ]*){3,}|(?:_[ \t]*){3,}|(?:\*[ \t]*){3,})(?:\n+|$)/,N=/^ {0,3}(#{1,6})(?=\s|$)(.*)(?:\n+|$)/,B=/(?:[*+-]|\d{1,9}[.)])/,W=u(/^(?!bull |blockCode|fences|blockquote|heading|html)((?:.|\n(?!\s*?\n|bull |blockCode|fences|blockquote|heading|html))+?)\n {0,3}(=+|-+) *(?:\n+|$)/).replace(/bull/g,B).replace(/blockCode/g,/ {4}/).replace(/fences/g,/ {0,3}(?:`{3,}|~{3,})/).replace(/blockquote/g,/ {0,3}>/).replace(/heading/g,/ {0,3}#{1,6}/).replace(/html/g,/ {0,3}<[^\n>]+>\n/).getRegex(),V=/^([^\n]+(?:\n(?!hr|heading|lheading|blockquote|fences|list|html|table| +\n)[^\n]+)*)/,$=/^[^\n]+/,H=/(?!\s*\])(?:\\.|[^\[\]\\])+/,j=u(/^ {0,3}\[(label)\]: *(?:\n *)?([^<\s][^\s]*|<.*?>)(?:(?: +(?:\n *)?| *\n *)(title))? *(?:\n+|$)/).replace("label",H).replace("title",/(?:"(?:\\"?|[^"\\])*"|'[^'\n]*(?:\n[^'\n]+)*\n?'|\([^()]*\))/).getRegex(),J=u(/^( {0,3}bull)([ \t][^\n]+?)?(?:\n|$)/).replace(/bull/g,B).getRegex(),he="address|article|aside|base|basefont|blockquote|body|caption|center|col|colgroup|dd|details|dialog|dir|div|dl|dt|fieldset|figcaption|figure|footer|form|frame|frameset|h[1-6]|head|header|hr|html|iframe|legend|li|link|main|menu|menuitem|meta|nav|noframes|ol|optgroup|option|p|param|search|section|summary|table|tbody|td|tfoot|th|thead|title|tr|track|ul",fe=/<!--(?:-?>|[\s\S]*?(?:-->|$))/,ue=u("^ {0,3}(?:<(script|pre|style|textarea)[\\s>][\\s\\S]*?(?:</\\1>[^\\n]*\\n+|$)|comment[^\\n]*(\\n+|$)|<\\?[\\s\\S]*?(?:\\?>\\n*|$)|<![A-Z][\\s\\S]*?(?:>\\n*|$)|<!\\[CDATA\\[[\\s\\S]*?(?:\\]\\]>\\n*|$)|</?(tag)(?: +|\\n|/?>)[\\s\\S]*?(?:(?:\\n *)+\\n|$)|<(?!script|pre|style|textarea)([a-z][\\w-]*)(?:attribute)*? */?>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:(?:\\n *)+\\n|$)|</(?!script|pre|style|textarea)[a-z][\\w-]*\\s*>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:(?:\\n *)+\\n|$))","i").replace("comment",fe).replace("tag",he).replace("attribute",/ +[a-zA-Z:_][\w.:-]*(?: *= *"[^"\n]*"| *= *'[^'\n]*'| *= *[^\s"'=<>`]+)?/).getRegex(),Pe=u(V).replace("hr",D).replace("heading"," {0,3}#{1,6}(?:\\s|$)").replace("|lheading","").replace("|table","").replace("blockquote"," {0,3}>").replace("fences"," {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list"," {0,3}(?:[*+-]|1[.)]) ").replace("html","</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|textarea|!--)").replace("tag",he).getRegex(),it={blockquote:u(/^( {0,3}> ?(paragraph|[^\n]*)(?:\n|$))+/).replace("paragraph",Pe).getRegex(),code:L,def:j,fences:k,heading:N,hr:D,html:ue,lheading:W,list:J,newline:S,paragraph:Pe,table:f,text:$},Ne=u("^ *([^\\n ].*)\\n {0,3}((?:\\| *)?:?-+:? *(?:\\| *:?-+:? *)*(?:\\| *)?)(?:\\n((?:(?! *\\n|hr|heading|blockquote|code|fences|list|html).*(?:\\n|$))*)\\n*|$)").replace("hr",D).replace("heading"," {0,3}#{1,6}(?:\\s|$)").replace("blockquote"," {0,3}>").replace("code"," {4}[^\\n]").replace("fences"," {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list"," {0,3}(?:[*+-]|1[.)]) ").replace("html","</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|textarea|!--)").replace("tag",he).getRegex(),Ct={...it,table:Ne,paragraph:u(V).replace("hr",D).replace("heading"," {0,3}#{1,6}(?:\\s|$)").replace("|lheading","").replace("table",Ne).replace("blockquote"," {0,3}>").replace("fences"," {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list"," {0,3}(?:[*+-]|1[.)]) ").replace("html","</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|textarea|!--)").replace("tag",he).getRegex()},ht={...it,html:u(`^ *(?:comment *(?:\\n|\\s*$)|<(tag)[\\s\\S]+?</\\1> *(?:\\n{2,}|\\s*$)|<tag(?:"[^"]*"|'[^']*'|\\s[^'"/>\\s]*)*?/?> *(?:\\n{2,}|\\s*$))`).replace("comment",fe).replace(/tag/g,"(?!(?:a|em|strong|small|s|cite|q|dfn|abbr|data|time|code|var|samp|kbd|sub|sup|i|b|u|mark|ruby|rt|rp|bdi|bdo|span|br|wbr|ins|del|img)\\b)\\w+(?!:|[^\\w\\s@]*@)\\b").getRegex(),def:/^ *\[([^\]]+)\]: *<?([^\s>]+)>?(?: +(["(][^\n]+[")]))? *(?:\n+|$)/,heading:/^(#{1,6})(.*)(?:\n+|$)/,fences:f,lheading:/^(.+?)\n {0,3}(=+|-+) *(?:\n+|$)/,paragraph:u(V).replace("hr",D).replace("heading",` *#{1,6} *[^
]`).replace("lheading",W).replace("|table","").replace("blockquote"," {0,3}>").replace("|fences","").replace("|list","").replace("|html","").replace("|tag","").getRegex()},Ti=/^\\([!"#$%&'()*+,\-./:;<=>?@\[\]\\^_`{|}~])/,ts=/^(`+)([^`]|[^`][\s\S]*?[^`])\1(?!`)/,xs=/^( {2,}|\\)\n(?!\s*$)/,Dr=/^(`+|[^`])(?:(?= {2,}\n)|[\s\S]*?(?:(?=[\\<!\[`*_]|\b_|$)|[^ ](?= {2,}\n)))/,Co="\\p{P}\\p{S}",hu=u(/^((?![*_])[\spunctuation])/,"u").replace(/punctuation/g,Co).getRegex(),zi=/\[[^[\]]*?\]\([^\(\)]*?\)|`[^`]*?`|<[^<>]*?>/g,Ir=u(/^(?:\*+(?:((?!\*)[punct])|[^\s*]))|^_+(?:((?!_)[punct])|([^\s_]))/,"u").replace(/punct/g,Co).getRegex(),ec=u("^[^_*]*?__[^_*]*?\\*[^_*]*?(?=__)|[^*]+(?=[^*])|(?!\\*)[punct](\\*+)(?=[\\s]|$)|[^punct\\s](\\*+)(?!\\*)(?=[punct\\s]|$)|(?!\\*)[punct\\s](\\*+)(?=[^punct\\s])|[\\s](\\*+)(?!\\*)(?=[punct])|(?!\\*)[punct](\\*+)(?!\\*)(?=[punct])|[^punct\\s](\\*+)(?=[^punct\\s])","gu").replace(/punct/g,Co).getRegex(),Bb=u("^[^_*]*?\\*\\*[^_*]*?_[^_*]*?(?=\\*\\*)|[^_]+(?=[^_])|(?!_)[punct](_+)(?=[\\s]|$)|[^punct\\s](_+)(?!_)(?=[punct\\s]|$)|(?!_)[punct\\s](_+)(?=[^punct\\s])|[\\s](_+)(?!_)(?=[punct])|(?!_)[punct](_+)(?!_)(?=[punct])","gu").replace(/punct/g,Co).getRegex(),Hb=u(/\\([punct])/,"gu").replace(/punct/g,Co).getRegex(),_d=u(/^<(scheme:[^\s\x00-\x1f<>]*|email)>/).replace("scheme",/[a-zA-Z][a-zA-Z0-9+.-]{1,31}/).replace("email",/[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+(@)[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)+(?![-_])/).getRegex(),pm=u(fe).replace("(?:-->|$)","-->").getRegex(),_m=u("^comment|^</[a-zA-Z][\\w:-]*\\s*>|^<[a-zA-Z][\\w-]*(?:attribute)*?\\s*/?>|^<\\?[\\s\\S]*?\\?>|^<![a-zA-Z]+\\s[\\s\\S]*?>|^<!\\[CDATA\\[[\\s\\S]*?\\]\\]>").replace("comment",pm).replace("attribute",/\s+[a-zA-Z:_][\w.:-]*(?:\s*=\s*"[^"]*"|\s*=\s*'[^']*'|\s*=\s*[^\s"'=<>`]+)?/).getRegex(),uu=/(?:\[(?:\\.|[^\[\]\\])*\]|\\.|`[^`]*`|[^\[\]\\`])*?/,bS=u(/^!?\[(label)\]\(\s*(href)(?:\s+(title))?\s*\)/).replace("label",uu).replace("href",/<(?:\\.|[^\n<>\\])+>|[^\s\x00-\x1f]*/).replace("title",/"(?:\\"?|[^"\\])*"|'(?:\\'?|[^'\\])*'|\((?:\\\)?|[^)\\])*\)/).getRegex(),CS=u(/^!?\[(label)\]\[(ref)\]/).replace("label",uu).replace("ref",H).getRegex(),bm=u(/^!?\[(ref)\](?:\[\])?/).replace("ref",H).getRegex(),gu=u("reflink|nolink(?!\\()","g").replace("reflink",CS).replace("nolink",bm).getRegex(),Ga={_backpedal:f,anyPunctuation:Hb,autolink:_d,blockSkip:zi,br:xs,code:ts,del:f,emStrongLDelim:Ir,emStrongRDelimAst:ec,emStrongRDelimUnd:Bb,escape:Ti,link:bS,nolink:bm,punctuation:hu,reflink:CS,reflinkSearch:gu,tag:_m,text:Dr,url:f},bd={...Ga,link:u(/^!?\[(label)\]\((.*?)\)/).replace("label",uu).getRegex(),reflink:u(/^!?\[(label)\]\s*\[([^\]]*)\]/).replace("label",uu).getRegex()},Cm={...Ga,escape:u(Ti).replace("])","~|])").getRegex(),url:u(/^((?:ftp|https?):\/\/|www\.)(?:[a-zA-Z0-9\-]+\.?)+[^\s<]*|^email/,"i").replace("email",/[A-Za-z0-9._+-]+(@)[a-zA-Z0-9-_]+(?:\.[a-zA-Z0-9-_]*[a-zA-Z0-9])+(?![-_])/).getRegex(),_backpedal:/(?:[^?!.,:;*_'"~()&]+|\([^)]*\)|&(?![a-zA-Z0-9]+;$)|[?!.,:;*_'"~)]+(?!$))+/,del:/^(~~?)(?=[^\s~])([\s\S]*?[^\s~])\1(?=[^~]|$)/,text:/^([`~]+|[^`~])(?:(?= {2,}\n)|(?=[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@)|[\s\S]*?(?:(?=[\\<!\[`*~_]|\b_|https?:\/\/|ftp:\/\/|www\.|$)|[^ ](?= {2,}\n)|[^a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-](?=[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@)))/},vS={...Cm,br:u(xs).replace("{2,}","*").getRegex(),text:u(Cm.text).replace("\\b_","\\b_| {2,}\\n").replace(/\{2,\}/g,"*").getRegex()},fu={normal:it,gfm:Ct,pedantic:ht},tc={normal:Ga,gfm:Cm,breaks:vS,pedantic:bd};class vo{constructor(R){Fi(this,"tokens");Fi(this,"options");Fi(this,"state");Fi(this,"tokenizer");Fi(this,"inlineQueue");this.tokens=[],this.tokens.links=Object.create(null),this.options=R||e.defaults,this.options.tokenizer=this.options.tokenizer||new w,this.tokenizer=this.options.tokenizer,this.tokenizer.options=this.options,this.tokenizer.lexer=this,this.inlineQueue=[],this.state={inLink:!1,inRawBlock:!1,top:!0};const M={block:fu.normal,inline:tc.normal};this.options.pedantic?(M.block=fu.pedantic,M.inline=tc.pedantic):this.options.gfm&&(M.block=fu.gfm,this.options.breaks?M.inline=tc.breaks:M.inline=tc.gfm),this.tokenizer.rules=M}static get rules(){return{block:fu,inline:tc}}static lex(R,M){return new vo(M).lex(R)}static lexInline(R,M){return new vo(M).inlineTokens(R)}lex(R){R=R.replace(/\r\n|\r/g,`
`),this.blockTokens(R,this.tokens);for(let M=0;M<this.inlineQueue.length;M++){const O=this.inlineQueue[M];this.inlineTokens(O.src,O.tokens)}return this.inlineQueue=[],this.tokens}blockTokens(R,M=[],O=!1){this.options.pedantic?R=R.replace(/\t/g,"    ").replace(/^ +$/gm,""):R=R.replace(/^( *)(\t+)/gm,(ae,De,Me)=>De+"    ".repeat(Me.length));let z,q,ee;for(;R;)if(!(this.options.extensions&&this.options.extensions.block&&this.options.extensions.block.some(ae=>(z=ae.call({lexer:this},R,M))?(R=R.substring(z.raw.length),M.push(z),!0):!1))){if(z=this.tokenizer.space(R)){R=R.substring(z.raw.length),z.raw.length===1&&M.length>0?M[M.length-1].raw+=`
`:M.push(z);continue}if(z=this.tokenizer.code(R)){R=R.substring(z.raw.length),q=M[M.length-1],q&&(q.type==="paragraph"||q.type==="text")?(q.raw+=`
`+z.raw,q.text+=`
`+z.text,this.inlineQueue[this.inlineQueue.length-1].src=q.text):M.push(z);continue}if(z=this.tokenizer.fences(R)){R=R.substring(z.raw.length),M.push(z);continue}if(z=this.tokenizer.heading(R)){R=R.substring(z.raw.length),M.push(z);continue}if(z=this.tokenizer.hr(R)){R=R.substring(z.raw.length),M.push(z);continue}if(z=this.tokenizer.blockquote(R)){R=R.substring(z.raw.length),M.push(z);continue}if(z=this.tokenizer.list(R)){R=R.substring(z.raw.length),M.push(z);continue}if(z=this.tokenizer.html(R)){R=R.substring(z.raw.length),M.push(z);continue}if(z=this.tokenizer.def(R)){R=R.substring(z.raw.length),q=M[M.length-1],q&&(q.type==="paragraph"||q.type==="text")?(q.raw+=`
`+z.raw,q.text+=`
`+z.raw,this.inlineQueue[this.inlineQueue.length-1].src=q.text):this.tokens.links[z.tag]||(this.tokens.links[z.tag]={href:z.href,title:z.title});continue}if(z=this.tokenizer.table(R)){R=R.substring(z.raw.length),M.push(z);continue}if(z=this.tokenizer.lheading(R)){R=R.substring(z.raw.length),M.push(z);continue}if(ee=R,this.options.extensions&&this.options.extensions.startBlock){let ae=1/0;const De=R.slice(1);let Me;this.options.extensions.startBlock.forEach(at=>{Me=at.call({lexer:this},De),typeof Me=="number"&&Me>=0&&(ae=Math.min(ae,Me))}),ae<1/0&&ae>=0&&(ee=R.substring(0,ae+1))}if(this.state.top&&(z=this.tokenizer.paragraph(ee))){q=M[M.length-1],O&&(q==null?void 0:q.type)==="paragraph"?(q.raw+=`
`+z.raw,q.text+=`
`+z.text,this.inlineQueue.pop(),this.inlineQueue[this.inlineQueue.length-1].src=q.text):M.push(z),O=ee.length!==R.length,R=R.substring(z.raw.length);continue}if(z=this.tokenizer.text(R)){R=R.substring(z.raw.length),q=M[M.length-1],q&&q.type==="text"?(q.raw+=`
`+z.raw,q.text+=`
`+z.text,this.inlineQueue.pop(),this.inlineQueue[this.inlineQueue.length-1].src=q.text):M.push(z);continue}if(R){const ae="Infinite loop on byte: "+R.charCodeAt(0);if(this.options.silent){console.error(ae);break}else throw new Error(ae)}}return this.state.top=!0,M}inline(R,M=[]){return this.inlineQueue.push({src:R,tokens:M}),M}inlineTokens(R,M=[]){let O,z,q,ee=R,ae,De,Me;if(this.tokens.links){const at=Object.keys(this.tokens.links);if(at.length>0)for(;(ae=this.tokenizer.rules.inline.reflinkSearch.exec(ee))!=null;)at.includes(ae[0].slice(ae[0].lastIndexOf("[")+1,-1))&&(ee=ee.slice(0,ae.index)+"["+"a".repeat(ae[0].length-2)+"]"+ee.slice(this.tokenizer.rules.inline.reflinkSearch.lastIndex))}for(;(ae=this.tokenizer.rules.inline.blockSkip.exec(ee))!=null;)ee=ee.slice(0,ae.index)+"["+"a".repeat(ae[0].length-2)+"]"+ee.slice(this.tokenizer.rules.inline.blockSkip.lastIndex);for(;(ae=this.tokenizer.rules.inline.anyPunctuation.exec(ee))!=null;)ee=ee.slice(0,ae.index)+"++"+ee.slice(this.tokenizer.rules.inline.anyPunctuation.lastIndex);for(;R;)if(De||(Me=""),De=!1,!(this.options.extensions&&this.options.extensions.inline&&this.options.extensions.inline.some(at=>(O=at.call({lexer:this},R,M))?(R=R.substring(O.raw.length),M.push(O),!0):!1))){if(O=this.tokenizer.escape(R)){R=R.substring(O.raw.length),M.push(O);continue}if(O=this.tokenizer.tag(R)){R=R.substring(O.raw.length),z=M[M.length-1],z&&O.type==="text"&&z.type==="text"?(z.raw+=O.raw,z.text+=O.text):M.push(O);continue}if(O=this.tokenizer.link(R)){R=R.substring(O.raw.length),M.push(O);continue}if(O=this.tokenizer.reflink(R,this.tokens.links)){R=R.substring(O.raw.length),z=M[M.length-1],z&&O.type==="text"&&z.type==="text"?(z.raw+=O.raw,z.text+=O.text):M.push(O);continue}if(O=this.tokenizer.emStrong(R,ee,Me)){R=R.substring(O.raw.length),M.push(O);continue}if(O=this.tokenizer.codespan(R)){R=R.substring(O.raw.length),M.push(O);continue}if(O=this.tokenizer.br(R)){R=R.substring(O.raw.length),M.push(O);continue}if(O=this.tokenizer.del(R)){R=R.substring(O.raw.length),M.push(O);continue}if(O=this.tokenizer.autolink(R)){R=R.substring(O.raw.length),M.push(O);continue}if(!this.state.inLink&&(O=this.tokenizer.url(R))){R=R.substring(O.raw.length),M.push(O);continue}if(q=R,this.options.extensions&&this.options.extensions.startInline){let at=1/0;const Et=R.slice(1);let Ft;this.options.extensions.startInline.forEach(yi=>{Ft=yi.call({lexer:this},Et),typeof Ft=="number"&&Ft>=0&&(at=Math.min(at,Ft))}),at<1/0&&at>=0&&(q=R.substring(0,at+1))}if(O=this.tokenizer.inlineText(q)){R=R.substring(O.raw.length),O.raw.slice(-1)!=="_"&&(Me=O.raw.slice(-1)),De=!0,z=M[M.length-1],z&&z.type==="text"?(z.raw+=O.raw,z.text+=O.text):M.push(O);continue}if(R){const at="Infinite loop on byte: "+R.charCodeAt(0);if(this.options.silent){console.error(at);break}else throw new Error(at)}}return M}}class Za{constructor(R){Fi(this,"options");Fi(this,"parser");this.options=R||e.defaults}space(R){return""}code({text:R,lang:M,escaped:O}){var ee;const z=(ee=(M||"").match(/^\S*/))==null?void 0:ee[0],q=R.replace(/\n$/,"")+`
`;return z?'<pre><code class="language-'+d(z)+'">'+(O?q:d(q,!0))+`</code></pre>
`:"<pre><code>"+(O?q:d(q,!0))+`</code></pre>
`}blockquote({tokens:R}){return`<blockquote>
${this.parser.parse(R)}</blockquote>
`}html({text:R}){return R}heading({tokens:R,depth:M}){return`<h${M}>${this.parser.parseInline(R)}</h${M}>
`}hr(R){return`<hr>
`}list(R){const M=R.ordered,O=R.start;let z="";for(let ae=0;ae<R.items.length;ae++){const De=R.items[ae];z+=this.listitem(De)}const q=M?"ol":"ul",ee=M&&O!==1?' start="'+O+'"':"";return"<"+q+ee+`>
`+z+"</"+q+`>
`}listitem(R){let M="";if(R.task){const O=this.checkbox({checked:!!R.checked});R.loose?R.tokens.length>0&&R.tokens[0].type==="paragraph"?(R.tokens[0].text=O+" "+R.tokens[0].text,R.tokens[0].tokens&&R.tokens[0].tokens.length>0&&R.tokens[0].tokens[0].type==="text"&&(R.tokens[0].tokens[0].text=O+" "+R.tokens[0].tokens[0].text)):R.tokens.unshift({type:"text",raw:O+" ",text:O+" "}):M+=O+" "}return M+=this.parser.parse(R.tokens,!!R.loose),`<li>${M}</li>
`}checkbox({checked:R}){return"<input "+(R?'checked="" ':"")+'disabled="" type="checkbox">'}paragraph({tokens:R}){return`<p>${this.parser.parseInline(R)}</p>
`}table(R){let M="",O="";for(let q=0;q<R.header.length;q++)O+=this.tablecell(R.header[q]);M+=this.tablerow({text:O});let z="";for(let q=0;q<R.rows.length;q++){const ee=R.rows[q];O="";for(let ae=0;ae<ee.length;ae++)O+=this.tablecell(ee[ae]);z+=this.tablerow({text:O})}return z&&(z=`<tbody>${z}</tbody>`),`<table>
<thead>
`+M+`</thead>
`+z+`</table>
`}tablerow({text:R}){return`<tr>
${R}</tr>
`}tablecell(R){const M=this.parser.parseInline(R.tokens),O=R.header?"th":"td";return(R.align?`<${O} align="${R.align}">`:`<${O}>`)+M+`</${O}>
`}strong({tokens:R}){return`<strong>${this.parser.parseInline(R)}</strong>`}em({tokens:R}){return`<em>${this.parser.parseInline(R)}</em>`}codespan({text:R}){return`<code>${R}</code>`}br(R){return"<br>"}del({tokens:R}){return`<del>${this.parser.parseInline(R)}</del>`}link({href:R,title:M,tokens:O}){const z=this.parser.parseInline(O),q=g(R);if(q===null)return z;R=q;let ee='<a href="'+R+'"';return M&&(ee+=' title="'+M+'"'),ee+=">"+z+"</a>",ee}image({href:R,title:M,text:O}){const z=g(R);if(z===null)return O;R=z;let q=`<img src="${R}" alt="${O}"`;return M&&(q+=` title="${M}"`),q+=">",q}text(R){return"tokens"in R&&R.tokens?this.parser.parseInline(R.tokens):R.text}}class ic{strong({text:R}){return R}em({text:R}){return R}codespan({text:R}){return R}del({text:R}){return R}html({text:R}){return R}text({text:R}){return R}link({text:R}){return""+R}image({text:R}){return""+R}br(){return""}}class dn{constructor(R){Fi(this,"options");Fi(this,"renderer");Fi(this,"textRenderer");this.options=R||e.defaults,this.options.renderer=this.options.renderer||new Za,this.renderer=this.options.renderer,this.renderer.options=this.options,this.renderer.parser=this,this.textRenderer=new ic}static parse(R,M){return new dn(M).parse(R)}static parseInline(R,M){return new dn(M).parseInline(R)}parse(R,M=!0){let O="";for(let z=0;z<R.length;z++){const q=R[z];if(this.options.extensions&&this.options.extensions.renderers&&this.options.extensions.renderers[q.type]){const ae=q,De=this.options.extensions.renderers[ae.type].call({parser:this},ae);if(De!==!1||!["space","hr","heading","code","table","blockquote","list","html","paragraph","text"].includes(ae.type)){O+=De||"";continue}}const ee=q;switch(ee.type){case"space":{O+=this.renderer.space(ee);continue}case"hr":{O+=this.renderer.hr(ee);continue}case"heading":{O+=this.renderer.heading(ee);continue}case"code":{O+=this.renderer.code(ee);continue}case"table":{O+=this.renderer.table(ee);continue}case"blockquote":{O+=this.renderer.blockquote(ee);continue}case"list":{O+=this.renderer.list(ee);continue}case"html":{O+=this.renderer.html(ee);continue}case"paragraph":{O+=this.renderer.paragraph(ee);continue}case"text":{let ae=ee,De=this.renderer.text(ae);for(;z+1<R.length&&R[z+1].type==="text";)ae=R[++z],De+=`
`+this.renderer.text(ae);M?O+=this.renderer.paragraph({type:"paragraph",raw:De,text:De,tokens:[{type:"text",raw:De,text:De}]}):O+=De;continue}default:{const ae='Token with "'+ee.type+'" type was not found.';if(this.options.silent)return console.error(ae),"";throw new Error(ae)}}}return O}parseInline(R,M){M=M||this.renderer;let O="";for(let z=0;z<R.length;z++){const q=R[z];if(this.options.extensions&&this.options.extensions.renderers&&this.options.extensions.renderers[q.type]){const ae=this.options.extensions.renderers[q.type].call({parser:this},q);if(ae!==!1||!["escape","html","link","image","strong","em","codespan","br","del","text"].includes(q.type)){O+=ae||"";continue}}const ee=q;switch(ee.type){case"escape":{O+=M.text(ee);break}case"html":{O+=M.html(ee);break}case"link":{O+=M.link(ee);break}case"image":{O+=M.image(ee);break}case"strong":{O+=M.strong(ee);break}case"em":{O+=M.em(ee);break}case"codespan":{O+=M.codespan(ee);break}case"br":{O+=M.br(ee);break}case"del":{O+=M.del(ee);break}case"text":{O+=M.text(ee);break}default:{const ae='Token with "'+ee.type+'" type was not found.';if(this.options.silent)return console.error(ae),"";throw new Error(ae)}}}return O}}class Jo{constructor(R){Fi(this,"options");this.options=R||e.defaults}preprocess(R){return R}postprocess(R){return R}processAllTokens(R){return R}}Fi(Jo,"passThroughHooks",new Set(["preprocess","postprocess","processAllTokens"]));class vm{constructor(...R){Fi(this,"defaults",t());Fi(this,"options",this.setOptions);Fi(this,"parse",this.parseMarkdown(vo.lex,dn.parse));Fi(this,"parseInline",this.parseMarkdown(vo.lexInline,dn.parseInline));Fi(this,"Parser",dn);Fi(this,"Renderer",Za);Fi(this,"TextRenderer",ic);Fi(this,"Lexer",vo);Fi(this,"Tokenizer",w);Fi(this,"Hooks",Jo);this.use(...R)}walkTokens(R,M){var z,q;let O=[];for(const ee of R)switch(O=O.concat(M.call(this,ee)),ee.type){case"table":{const ae=ee;for(const De of ae.header)O=O.concat(this.walkTokens(De.tokens,M));for(const De of ae.rows)for(const Me of De)O=O.concat(this.walkTokens(Me.tokens,M));break}case"list":{const ae=ee;O=O.concat(this.walkTokens(ae.items,M));break}default:{const ae=ee;(q=(z=this.defaults.extensions)==null?void 0:z.childTokens)!=null&&q[ae.type]?this.defaults.extensions.childTokens[ae.type].forEach(De=>{const Me=ae[De].flat(1/0);O=O.concat(this.walkTokens(Me,M))}):ae.tokens&&(O=O.concat(this.walkTokens(ae.tokens,M)))}}return O}use(...R){const M=this.defaults.extensions||{renderers:{},childTokens:{}};return R.forEach(O=>{const z={...O};if(z.async=this.defaults.async||z.async||!1,O.extensions&&(O.extensions.forEach(q=>{if(!q.name)throw new Error("extension name required");if("renderer"in q){const ee=M.renderers[q.name];ee?M.renderers[q.name]=function(...ae){let De=q.renderer.apply(this,ae);return De===!1&&(De=ee.apply(this,ae)),De}:M.renderers[q.name]=q.renderer}if("tokenizer"in q){if(!q.level||q.level!=="block"&&q.level!=="inline")throw new Error("extension level must be 'block' or 'inline'");const ee=M[q.level];ee?ee.unshift(q.tokenizer):M[q.level]=[q.tokenizer],q.start&&(q.level==="block"?M.startBlock?M.startBlock.push(q.start):M.startBlock=[q.start]:q.level==="inline"&&(M.startInline?M.startInline.push(q.start):M.startInline=[q.start]))}"childTokens"in q&&q.childTokens&&(M.childTokens[q.name]=q.childTokens)}),z.extensions=M),O.renderer){const q=this.defaults.renderer||new Za(this.defaults);for(const ee in O.renderer){if(!(ee in q))throw new Error(`renderer '${ee}' does not exist`);if(["options","parser"].includes(ee))continue;const ae=ee,De=O.renderer[ae],Me=q[ae];q[ae]=(...at)=>{let Et=De.apply(q,at);return Et===!1&&(Et=Me.apply(q,at)),Et||""}}z.renderer=q}if(O.tokenizer){const q=this.defaults.tokenizer||new w(this.defaults);for(const ee in O.tokenizer){if(!(ee in q))throw new Error(`tokenizer '${ee}' does not exist`);if(["options","rules","lexer"].includes(ee))continue;const ae=ee,De=O.tokenizer[ae],Me=q[ae];q[ae]=(...at)=>{let Et=De.apply(q,at);return Et===!1&&(Et=Me.apply(q,at)),Et}}z.tokenizer=q}if(O.hooks){const q=this.defaults.hooks||new Jo;for(const ee in O.hooks){if(!(ee in q))throw new Error(`hook '${ee}' does not exist`);if(ee==="options")continue;const ae=ee,De=O.hooks[ae],Me=q[ae];Jo.passThroughHooks.has(ee)?q[ae]=at=>{if(this.defaults.async)return Promise.resolve(De.call(q,at)).then(Ft=>Me.call(q,Ft));const Et=De.call(q,at);return Me.call(q,Et)}:q[ae]=(...at)=>{let Et=De.apply(q,at);return Et===!1&&(Et=Me.apply(q,at)),Et}}z.hooks=q}if(O.walkTokens){const q=this.defaults.walkTokens,ee=O.walkTokens;z.walkTokens=function(ae){let De=[];return De.push(ee.call(this,ae)),q&&(De=De.concat(q.call(this,ae))),De}}this.defaults={...this.defaults,...z}}),this}setOptions(R){return this.defaults={...this.defaults,...R},this}lexer(R,M){return vo.lex(R,M??this.defaults)}parser(R,M){return dn.parse(R,M??this.defaults)}parseMarkdown(R,M){return(z,q)=>{const ee={...q},ae={...this.defaults,...ee},De=this.onError(!!ae.silent,!!ae.async);if(this.defaults.async===!0&&ee.async===!1)return De(new Error("marked(): The async option was set to true by an extension. Remove async: false from the parse options object to return a Promise."));if(typeof z>"u"||z===null)return De(new Error("marked(): input parameter is undefined or null"));if(typeof z!="string")return De(new Error("marked(): input parameter is of type "+Object.prototype.toString.call(z)+", string expected"));if(ae.hooks&&(ae.hooks.options=ae),ae.async)return Promise.resolve(ae.hooks?ae.hooks.preprocess(z):z).then(Me=>R(Me,ae)).then(Me=>ae.hooks?ae.hooks.processAllTokens(Me):Me).then(Me=>ae.walkTokens?Promise.all(this.walkTokens(Me,ae.walkTokens)).then(()=>Me):Me).then(Me=>M(Me,ae)).then(Me=>ae.hooks?ae.hooks.postprocess(Me):Me).catch(De);try{ae.hooks&&(z=ae.hooks.preprocess(z));let Me=R(z,ae);ae.hooks&&(Me=ae.hooks.processAllTokens(Me)),ae.walkTokens&&this.walkTokens(Me,ae.walkTokens);let at=M(Me,ae);return ae.hooks&&(at=ae.hooks.postprocess(at)),at}catch(Me){return De(Me)}}}onError(R,M){return O=>{if(O.message+=`
Please report this to https://github.com/markedjs/marked.`,R){const z="<p>An error occurred:</p><pre>"+d(O.message+"",!0)+"</pre>";return M?Promise.resolve(z):z}if(M)return Promise.reject(O);throw O}}}const na=new vm;function si(ut,R){return na.parse(ut,R)}si.options=si.setOptions=function(ut){return na.setOptions(ut),si.defaults=na.defaults,i(si.defaults),si},si.getDefaults=t,si.defaults=e.defaults,si.use=function(...ut){return na.use(...ut),si.defaults=na.defaults,i(si.defaults),si},si.walkTokens=function(ut,R){return na.walkTokens(ut,R)},si.parseInline=na.parseInline,si.Parser=dn,si.parser=dn.parse,si.Renderer=Za,si.TextRenderer=ic,si.Lexer=vo,si.lexer=vo.lex,si.Tokenizer=w,si.Hooks=Jo,si.parse=si;const mu=si.options,ZE=si.setOptions,YE=si.use,Cn=si.walkTokens,Cd=si.parseInline,XE=si,wS=dn.parse,Vb=vo.lex;e.Hooks=Jo,e.Lexer=vo,e.Marked=vm,e.Parser=dn,e.Renderer=Za,e.TextRenderer=ic,e.Tokenizer=w,e.getDefaults=t,e.lexer=Vb,e.marked=si,e.options=mu,e.parse=XE,e.parseInline=Cd,e.parser=wS,e.setOptions=ZE,e.use=YE,e.walkTokens=Cn})})();Hn.Hooks||exports.Hooks;Hn.Lexer||exports.Lexer;Hn.Marked||exports.Marked;Hn.Parser||exports.Parser;var hj=Hn.Renderer||exports.Renderer;Hn.TextRenderer||exports.TextRenderer;Hn.Tokenizer||exports.Tokenizer;var mle=Hn.defaults||exports.defaults;Hn.getDefaults||exports.getDefaults;var ZI=Hn.lexer||exports.lexer;Hn.marked||exports.marked;Hn.options||exports.options;var uj=Hn.parse||exports.parse;Hn.parseInline||exports.parseInline;var ple=Hn.parser||exports.parser;Hn.setOptions||exports.setOptions;Hn.use||exports.use;Hn.walkTokens||exports.walkTokens;function _le(o){return JSON.stringify(o,ble)}function w2(o){let e=JSON.parse(o);return e=S2(e),e}function ble(o,e){return e instanceof RegExp?{$mid:2,source:e.source,flags:e.flags}:e}function S2(o,e=0){if(!o||e>200)return o;if(typeof o=="object"){switch(o.$mid){case 1:return Se.revive(o);case 2:return new RegExp(o.source,o.flags);case 17:return new Date(o.source)}if(o instanceof RI||o instanceof Uint8Array)return o;if(Array.isArray(o))for(let t=0;t<o.length;++t)o[t]=S2(o[t],e+1);else for(const t in o)Object.hasOwnProperty.call(o,t)&&(o[t]=S2(o[t],e+1))}return o}const KN=Object.freeze({image:({href:o,title:e,text:t})=>{let i=[],n=[];return o&&({href:o,dimensions:i}=fle(o),n.push(`src="${zS(o)}"`)),t&&n.push(`alt="${zS(t)}"`),e&&n.push(`title="${zS(e)}"`),i.length&&(n=n.concat(i)),"<img "+n.join(" ")+">"},paragraph({tokens:o}){return`<p>${this.parser.parseInline(o)}</p>`},link({href:o,title:e,tokens:t}){let i=this.parser.parseInline(t);return typeof o!="string"?"":(o===i&&(i=jN(i)),e=typeof e=="string"?zS(jN(e)):"",o=jN(o),o=o.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/"/g,"&quot;").replace(/'/g,"&#39;"),`<a href="${o}" title="${e||o}" draggable="false">${i}</a>`)}});function YI(o,e={},t={}){const i=new K;let n=!1;const s=B5(e),r=function(p){let _;try{_=w2(decodeURIComponent(p))}catch{}return _?(_=Lz(_,b=>{if(o.uris&&o.uris[b])return Se.revive(o.uris[b])}),encodeURIComponent(JSON.stringify(_))):p},a=function(p,_){const b=o.uris&&o.uris[p];let C=Se.revive(b);return _?p.startsWith(Fe.data+":")?p:(C||(C=Se.parse(p)),pI.uriToBrowserUri(C).toString(!0)):!C||Se.parse(p).toString()===C.toString()?p:(C.query&&(C=C.with({query:r(C.query)})),C.toString())},l=new hj;l.image=KN.image,l.link=KN.link,l.paragraph=KN.paragraph;const c=[],d=[];if(e.codeBlockRendererSync?l.code=({text:p,lang:_})=>{const b=v2.nextId(),C=e.codeBlockRendererSync(V9(_),p);return d.push([b,C]),`<div class="code" data-code="${b}">${jC(p)}</div>`}:e.codeBlockRenderer&&(l.code=({text:p,lang:_})=>{const b=v2.nextId(),C=e.codeBlockRenderer(V9(_),p);return c.push(C.then(v=>[b,v])),`<div class="code" data-code="${b}">${jC(p)}</div>`}),e.actionHandler){const p=function(C){let v=C.target;if(!(v.tagName!=="A"&&(v=v.parentElement,!v||v.tagName!=="A")))try{let w=v.dataset.href;w&&(o.baseUri&&(w=qN(Se.from(o.baseUri),w)),e.actionHandler.callback(w,C))}catch(w){Oe(w)}finally{C.preventDefault()}},_=e.actionHandler.disposables.add(new tt(s,"click")),b=e.actionHandler.disposables.add(new tt(s,"auxclick"));e.actionHandler.disposables.add(oe.any(_.event,b.event)(C=>{const v=new Vr(xe(s),C);!v.leftButton&&!v.middleButton||p(v)})),e.actionHandler.disposables.add(U(s,"keydown",C=>{const v=new Pt(C);!v.equals(10)&&!v.equals(3)||p(v)}))}o.supportHtml||(l.html=({text:p})=>{var b;return(b=e.sanitizerOptions)!=null&&b.replaceWithPlaintext?jC(p):(o.isTrusted?p.match(/^(<span[^>]+>)|(<\/\s*span>)$/):void 0)?p:""}),t.renderer=l;let h=o.value??"";h.length>1e5&&(h=`${h.substr(0,1e5)}…`),o.supportThemeIcons&&(h=ile(h));let u;if(e.fillInIncompleteTokens){const p={...mle,...t},_=ZI(h,p),b=Tle(_);u=ple(b,p)}else u=uj(h,{...t,async:!1});o.supportThemeIcons&&(u=dh(u).map(_=>typeof _=="string"?_:_.outerHTML).join(""));const f=new DOMParser().parseFromString(y2({isTrusted:o.isTrusted,...e.sanitizerOptions},u),"text/html");if(f.body.querySelectorAll("img, audio, video, source").forEach(p=>{const _=p.getAttribute("src");if(_){let b=_;try{o.baseUri&&(b=qN(Se.from(o.baseUri),b))}catch{}if(p.setAttribute("src",a(b,!0)),e.remoteImageIsAllowed){const C=Se.parse(b);C.scheme!==Fe.file&&C.scheme!==Fe.data&&!e.remoteImageIsAllowed(C)&&p.replaceWith(ne("",void 0,p.outerHTML))}}}),f.body.querySelectorAll("a").forEach(p=>{const _=p.getAttribute("href");if(p.setAttribute("href",""),!_||/^data:|javascript:/i.test(_)||/^command:/i.test(_)&&!o.isTrusted||/^command:(\/\/\/)?_workbench\.downloadResource/i.test(_))p.replaceWith(...p.childNodes);else{let b=a(_,!1);o.baseUri&&(b=qN(Se.from(o.baseUri),_)),p.dataset.href=b}}),s.innerHTML=y2({isTrusted:o.isTrusted,...e.sanitizerOptions},f.body.innerHTML),c.length>0)Promise.all(c).then(p=>{var C;if(n)return;const _=new Map(p),b=s.querySelectorAll("div[data-code]");for(const v of b){const w=_.get(v.dataset.code??"");w&&_n(v,w)}(C=e.asyncRenderCallback)==null||C.call(e)});else if(d.length>0){const p=new Map(d),_=s.querySelectorAll("div[data-code]");for(const b of _){const C=p.get(b.dataset.code??"");C&&_n(b,C)}}if(e.asyncRenderCallback)for(const p of s.getElementsByTagName("img")){const _=i.add(U(p,"load",()=>{_.dispose(),e.asyncRenderCallback()}))}return{element:s,dispose:()=>{n=!0,i.dispose()}}}function V9(o){if(!o)return"";const e=o.split(/[\s+|:|,|\{|\?]/,1);return e.length?e[0]:o}function qN(o,e){return/^\w[\w\d+.-]*:/.test(e)?e:o.path.endsWith("/")?W9(o,e).toString():W9(GI(o),e).toString()}const Cle=["area","base","br","col","command","embed","hr","img","input","keygen","link","meta","param","source","track","wbr"];function y2(o,e){const{config:t,allowedSchemes:i}=wle(o),n=new K;n.add(z9("uponSanitizeAttribute",(s,r)=>{var a;if(r.attrName==="style"||r.attrName==="class"){if(s.tagName==="SPAN"){if(r.attrName==="style"){r.keepAttr=/^(color\:(#[0-9a-fA-F]+|var\(--vscode(-[a-zA-Z]+)+\));)?(background-color\:(#[0-9a-fA-F]+|var\(--vscode(-[a-zA-Z]+)+\));)?(border-radius:[0-9]+px;)?$/.test(r.attrValue);return}else if(r.attrName==="class"){r.keepAttr=/^codicon codicon-[a-z\-]+( codicon-modifier-[a-z\-]+)?$/.test(r.attrValue);return}}r.keepAttr=!1;return}else if(s.tagName==="INPUT"&&((a=s.attributes.getNamedItem("type"))==null?void 0:a.value)==="checkbox"){if(r.attrName==="type"&&r.attrValue==="checkbox"||r.attrName==="disabled"||r.attrName==="checked"){r.keepAttr=!0;return}r.keepAttr=!1}})),n.add(z9("uponSanitizeElement",(s,r)=>{var a;if(r.tagName==="input"&&(((a=s.attributes.getNamedItem("type"))==null?void 0:a.value)==="checkbox"?s.setAttribute("disabled",""):o.replaceWithPlaintext||s.remove()),o.replaceWithPlaintext&&!r.allowedTags[r.tagName]&&r.tagName!=="body"&&s.parentElement){let l,c;if(r.tagName==="#comment")l=`<!--${s.textContent}-->`;else{const g=Cle.includes(r.tagName),f=s.attributes.length?" "+Array.from(s.attributes).map(p=>`${p.name}="${p.value}"`).join(" "):"";l=`<${r.tagName}${f}>`,g||(c=`</${r.tagName}>`)}const d=document.createDocumentFragment(),h=s.parentElement.ownerDocument.createTextNode(l);d.appendChild(h);const u=c?s.parentElement.ownerDocument.createTextNode(c):void 0;for(;s.firstChild;)d.appendChild(s.firstChild);u&&d.appendChild(u),s.nodeType===Node.COMMENT_NODE?s.parentElement.insertBefore(d,s):s.parentElement.replaceChild(d,s)}})),n.add(eie(i));try{return bU(e,{...t,RETURN_TRUSTED_TYPE:!0})}finally{n.dispose()}}const vle=["align","autoplay","alt","checked","class","colspan","controls","data-code","data-href","disabled","draggable","height","href","loop","muted","playsinline","poster","rowspan","src","style","target","title","type","width","start"];function wle(o){const e=[Fe.http,Fe.https,Fe.mailto,Fe.data,Fe.file,Fe.vscodeFileResource,Fe.vscodeRemote,Fe.vscodeRemoteResource];return o.isTrusted&&e.push(Fe.command),{config:{ALLOWED_TAGS:o.allowedTags??[...tie],ALLOWED_ATTR:vle,ALLOW_UNKNOWN_PROTOCOLS:!0},allowedSchemes:e}}function Sle(o){return typeof o=="string"?o:yle(o)}function yle(o,e){let t=o.value??"";t.length>1e5&&(t=`${t.substr(0,1e5)}…`);const i=uj(t,{async:!1,renderer:kle.value}).replace(/&(#\d+|[a-zA-Z]+);/g,n=>Lle.get(n)??n);return y2({isTrusted:!1},i).toString()}const Lle=new Map([["&quot;",'"'],["&nbsp;"," "],["&amp;","&"],["&#39;","'"],["&lt;","<"],["&gt;",">"]]);function xle(){const o=new hj;return o.code=({text:e})=>e,o.blockquote=({text:e})=>e+`
`,o.html=e=>"",o.heading=function({tokens:e}){return this.parser.parseInline(e)+`
`},o.hr=()=>"",o.list=function({items:e}){return e.map(t=>this.listitem(t)).join(`
`)+`
`},o.listitem=({text:e})=>e+`
`,o.paragraph=function({tokens:e}){return this.parser.parseInline(e)+`
`},o.table=function({header:e,rows:t}){return e.map(i=>this.tablecell(i)).join(" ")+`
`+t.map(i=>i.map(n=>this.tablecell(n)).join(" ")).join(`
`)+`
`},o.tablerow=({text:e})=>e,o.tablecell=function({tokens:e}){return this.parser.parseInline(e)},o.strong=({text:e})=>e,o.em=({text:e})=>e,o.codespan=({text:e})=>e,o.br=e=>`
`,o.del=({text:e})=>e,o.image=e=>"",o.text=({text:e})=>e,o.link=({text:e})=>e,o}const kle=new Oa(o=>xle());function gL(o){let e="";return o.forEach(t=>{e+=t.raw}),e}function gj(o){var e,t;if(o.tokens)for(let i=o.tokens.length-1;i>=0;i--){const n=o.tokens[i];if(n.type==="text"){const s=n.raw.split(`
`),r=s[s.length-1];if(r.includes("`"))return Mle(o);if(r.includes("**"))return Ble(o);if(r.match(/\*\w/))return Ale(o);if(r.match(/(^|\s)__\w/))return Hle(o);if(r.match(/(^|\s)_\w/))return Ple(o);if(Dle(r)||Ile(r)&&o.tokens.slice(0,i).some(a=>a.type==="text"&&a.raw.match(/\[[^\]]*$/))){const a=o.tokens.slice(i+1);return((e=a[0])==null?void 0:e.type)==="link"&&((t=a[1])==null?void 0:t.type)==="text"&&a[1].raw.match(/^ *"[^"]*$/)||r.match(/^[^"]* +"[^"]*$/)?Fle(o):Ole(o)}else if(r.match(/(^|\s)\[\w*/))return Wle(o)}}}function Dle(o){return!!o.match(/(^|\s)\[.*\]\(\w*/)}function Ile(o){return!!o.match(/^[^\[]*\]\([^\)]*$/)}function Ele(o){var l;const e=o.items[o.items.length-1],t=e.tokens?e.tokens[e.tokens.length-1]:void 0;let i;if((t==null?void 0:t.type)==="text"&&!("inRawBlock"in e)&&(i=gj(t)),!i||i.type!=="paragraph")return;const n=gL(o.items.slice(0,-1)),s=(l=e.raw.match(/^(\s*(-|\d+\.|\*) +)/))==null?void 0:l[0];if(!s)return;const r=s+gL(e.tokens.slice(0,-1))+i.raw,a=ZI(n+r)[0];if(a.type==="list")return a}const Nle=3;function Tle(o){for(let e=0;e<Nle;e++){const t=Rle(o);if(t)o=t;else break}return o}function Rle(o){let e,t;for(e=0;e<o.length;e++){const i=o[e];if(i.type==="paragraph"&&i.raw.match(/(\n|^)\|/)){t=Vle(o.slice(e));break}if(e===o.length-1&&i.type==="list"){const n=Ele(i);if(n){t=[n];break}}if(e===o.length-1&&i.type==="paragraph"){const n=gj(i);if(n){t=[n];break}}}if(t){const i=[...o.slice(0,e),...t];return i.links=o.links,i}return null}function Mle(o){return iu(o,"`")}function Ale(o){return iu(o,"*")}function Ple(o){return iu(o,"_")}function Ole(o){return iu(o,")")}function Fle(o){return iu(o,'")')}function Wle(o){return iu(o,"](https://microsoft.com)")}function Ble(o){return iu(o,"**")}function Hle(o){return iu(o,"__")}function iu(o,e){const t=gL(Array.isArray(o)?o:[o]);return ZI(t+e)[0]}function Vle(o){const e=gL(o),t=e.split(`
`);let i,n=!1;for(let s=0;s<t.length;s++){const r=t[s].trim();if(typeof i>"u"&&r.match(/^\s*\|/)){const a=r.match(/(\|[^\|]+)(?=\||$)/g);a&&(i=a.length)}else if(typeof i=="number")if(r.match(/^\s*\|/)){if(s!==t.length-1)return;n=!0}else return}if(typeof i=="number"&&i>0){const s=n?t.slice(0,-1).join(`
`):e,r=!!s.match(/\|\s*$/),a=s+(r?"":"|")+`
|${" --- |".repeat(i)}`;return ZI(a)}}function z9(o,e){return CU(o,e),ve(()=>vU(o))}const bc=class bc{static createEmpty(e,t){const i=bc.defaultTokenMetadata,n=new Uint32Array(2);return n[0]=e.length,n[1]=i,new bc(n,e,t)}static createFromTextAndMetadata(e,t){let i=0,n="";const s=new Array;for(const{text:r,metadata:a}of e)s.push(i+r.length,a),i+=r.length,n+=r;return new bc(new Uint32Array(s),n,t)}constructor(e,t,i){this._lineTokensBrand=void 0,this._tokens=e,this._tokensCount=this._tokens.length>>>1,this._text=t,this.languageIdCodec=i}equals(e){return e instanceof bc?this.slicedEquals(e,0,this._tokensCount):!1}slicedEquals(e,t,i){if(this._text!==e._text||this._tokensCount!==e._tokensCount)return!1;const n=t<<1,s=n+(i<<1);for(let r=n;r<s;r++)if(this._tokens[r]!==e._tokens[r])return!1;return!0}getLineContent(){return this._text}getCount(){return this._tokensCount}getStartOffset(e){return e>0?this._tokens[e-1<<1]:0}getMetadata(e){return this._tokens[(e<<1)+1]}getLanguageId(e){const t=this._tokens[(e<<1)+1],i=Ts.getLanguageId(t);return this.languageIdCodec.decodeLanguageId(i)}getStandardTokenType(e){const t=this._tokens[(e<<1)+1];return Ts.getTokenType(t)}getForeground(e){const t=this._tokens[(e<<1)+1];return Ts.getForeground(t)}getClassName(e){const t=this._tokens[(e<<1)+1];return Ts.getClassNameFromMetadata(t)}getInlineStyle(e,t){const i=this._tokens[(e<<1)+1];return Ts.getInlineStyleFromMetadata(i,t)}getPresentation(e){const t=this._tokens[(e<<1)+1];return Ts.getPresentationFromMetadata(t)}getEndOffset(e){return this._tokens[e<<1]}findTokenIndexAtOffset(e){return bc.findIndexInTokensArray(this._tokens,e)}inflate(){return this}sliceAndInflate(e,t,i){return new G5(this,e,t,i)}static convertToEndOffset(e,t){const n=(e.length>>>1)-1;for(let s=0;s<n;s++)e[s<<1]=e[s+1<<1];e[n<<1]=t}static findIndexInTokensArray(e,t){if(e.length<=2)return 0;let i=0,n=(e.length>>>1)-1;for(;i<n;){const s=i+Math.floor((n-i)/2),r=e[s<<1];if(r===t)return s+1;r<t?i=s+1:r>t&&(n=s)}return i}withInserted(e){if(e.length===0)return this;let t=0,i=0,n="";const s=new Array;let r=0;for(;;){const a=t<this._tokensCount?this._tokens[t<<1]:-1,l=i<e.length?e[i]:null;if(a!==-1&&(l===null||a<=l.offset)){n+=this._text.substring(r,a);const c=this._tokens[(t<<1)+1];s.push(n.length,c),t++,r=a}else if(l){if(l.offset>r){n+=this._text.substring(r,l.offset);const c=this._tokens[(t<<1)+1];s.push(n.length,c),r=l.offset}n+=l.text,s.push(n.length,l.tokenMetadata),i++}else break}return new bc(new Uint32Array(s),n,this.languageIdCodec)}getTokenText(e){const t=this.getStartOffset(e),i=this.getEndOffset(e);return this._text.substring(t,i)}forEach(e){const t=this.getCount();for(let i=0;i<t;i++)e(i)}};bc.defaultTokenMetadata=(32768|2<<24)>>>0;let cn=bc;class G5{constructor(e,t,i,n){this._source=e,this._startOffset=t,this._endOffset=i,this._deltaOffset=n,this._firstTokenIndex=e.findTokenIndexAtOffset(t),this.languageIdCodec=e.languageIdCodec,this._tokensCount=0;for(let s=this._firstTokenIndex,r=e.getCount();s<r&&!(e.getStartOffset(s)>=i);s++)this._tokensCount++}getMetadata(e){return this._source.getMetadata(this._firstTokenIndex+e)}getLanguageId(e){return this._source.getLanguageId(this._firstTokenIndex+e)}getLineContent(){return this._source.getLineContent().substring(this._startOffset,this._endOffset)}equals(e){return e instanceof G5?this._startOffset===e._startOffset&&this._endOffset===e._endOffset&&this._deltaOffset===e._deltaOffset&&this._source.slicedEquals(e._source,this._firstTokenIndex,this._tokensCount):!1}getCount(){return this._tokensCount}getStandardTokenType(e){return this._source.getStandardTokenType(this._firstTokenIndex+e)}getForeground(e){return this._source.getForeground(this._firstTokenIndex+e)}getEndOffset(e){const t=this._source.getEndOffset(this._firstTokenIndex+e);return Math.min(this._endOffset,t)-this._startOffset+this._deltaOffset}getClassName(e){return this._source.getClassName(this._firstTokenIndex+e)}getInlineStyle(e,t){return this._source.getInlineStyle(this._firstTokenIndex+e,t)}getPresentation(e){return this._source.getPresentation(this._firstTokenIndex+e)}findTokenIndexAtOffset(e){return this._source.findTokenIndexAtOffset(e+this._startOffset-this._deltaOffset)-this._firstTokenIndex}getTokenText(e){const t=this._firstTokenIndex+e,i=this._source.getStartOffset(t),n=this._source.getEndOffset(t);let s=this._source.getTokenText(t);return i<this._startOffset&&(s=s.substring(this._startOffset-i)),n>this._endOffset&&(s=s.substring(0,s.length-(n-this._endOffset))),s}forEach(e){for(let t=0;t<this.getCount();t++)e(t)}}function zle(o,e){const t=e.lineNumber;if(!o.tokenization.isCheapToTokenize(t))return;o.tokenization.forceTokenization(t);const i=o.tokenization.getLineTokens(t),n=i.findTokenIndexAtOffset(e.column-1);return i.getStandardTokenType(n)}const F_=new class{clone(){return this}equals(o){return this===o}};function Z5(o,e){return new G4([new ew(0,"",o)],e)}function XI(o,e){const t=new Uint32Array(2);return t[0]=0,t[1]=(o<<0|0|0|32768|2<<24)>>>0,new gI(t,e===null?F_:e)}const U9={getInitialState:()=>F_,tokenizeEncoded:(o,e,t)=>XI(0,t)};async function Ule(o,e,t){if(!t)return $9(e,o.languageIdCodec,U9);const i=await fi.getOrCreate(t);return $9(e,o.languageIdCodec,i||U9)}function $le(o,e,t,i,n,s,r){let a="<div>",l=i,c=0,d=!0;for(let h=0,u=e.getCount();h<u;h++){const g=e.getEndOffset(h);if(g<=i)continue;let f="";for(;l<g&&l<n;l++){const p=o.charCodeAt(l);switch(p){case 9:{let _=s-(l+c)%s;for(c+=_-1;_>0;)r&&d?(f+="&#160;",d=!1):(f+=" ",d=!0),_--;break}case 60:f+="&lt;",d=!1;break;case 62:f+="&gt;",d=!1;break;case 38:f+="&amp;",d=!1;break;case 0:f+="&#00;",d=!1;break;case 65279:case 8232:case 8233:case 133:f+="�",d=!1;break;case 13:f+="&#8203",d=!1;break;case 32:r&&d?(f+="&#160;",d=!1):(f+=" ",d=!0);break;default:f+=String.fromCharCode(p),d=!1}}if(a+=`<span style="${e.getInlineStyle(h,t)}">${f}</span>`,g>n||l>=n)break}return a+="</div>",a}function $9(o,e,t){let i='<div class="monaco-tokenized-source">';const n=Kl(o);let s=t.getInitialState();for(let r=0,a=n.length;r<a;r++){const l=n[r];r>0&&(i+="<br/>");const c=t.tokenizeEncoded(l,!0,s);cn.convertToEndOffset(c.tokens,l.length);const h=new cn(c.tokens,l,e).inflate();let u=0;for(let g=0,f=h.getCount();g<f;g++){const p=h.getClassName(g),_=h.getEndOffset(g);i+=`<span class="${p}">${jC(l.substring(u,_))}</span>`,u=_}s=c.endState}return i+="</div>",i}var jle=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},j9=function(o,e){return function(t,i){e(t,i,o)}},L2,Dg;let nd=(Dg=class{constructor(e,t,i){this._options=e,this._languageService=t,this._openerService=i,this._onDidRenderAsync=new P,this.onDidRenderAsync=this._onDidRenderAsync.event}dispose(){this._onDidRenderAsync.dispose()}render(e,t,i){if(!e)return{element:document.createElement("span"),dispose:()=>{}};const n=new K,s=n.add(YI(e,{...this._getRenderOptions(e,n),...t},i));return s.element.classList.add("rendered-markdown"),{element:s.element,dispose:()=>n.dispose()}}_getRenderOptions(e,t){return{codeBlockRenderer:async(i,n)=>{var l,c;let s;i?s=this._languageService.getLanguageIdByLanguageName(i):this._options.editor&&(s=(l=this._options.editor.getModel())==null?void 0:l.getLanguageId()),s||(s=ms);const r=await Ule(this._languageService,n,s),a=document.createElement("span");if(a.innerHTML=((c=L2._ttpTokenizer)==null?void 0:c.createHTML(r))??r,this._options.editor){const d=this._options.editor.getOption(50);bn(a,d)}else this._options.codeBlockFontFamily&&(a.style.fontFamily=this._options.codeBlockFontFamily);return this._options.codeBlockFontSize!==void 0&&(a.style.fontSize=this._options.codeBlockFontSize),a},asyncRenderCallback:()=>this._onDidRenderAsync.fire(),actionHandler:{callback:i=>Y5(this._openerService,i,e.isTrusted),disposables:t}}}},L2=Dg,Dg._ttpTokenizer=hd("tokenizeToString",{createHTML(e){return e}}),Dg);nd=L2=jle([j9(1,ni),j9(2,zs)],nd);async function Y5(o,e,t){try{return await o.open(e,{fromUserGesture:!0,allowContributedOpeners:!0,allowCommands:Kle(t)})}catch(i){return Oe(i),!1}}function Kle(o){return o===!0?!0:o&&Array.isArray(o.enabledCommands)?o.enabledCommands:!1}const po=et("accessibilityService"),$1=new le("accessibilityModeEnabled",!1),K9=2e4;let ju,X0,x2,Q0,k2;function qle(o){ju=document.createElement("div"),ju.className="monaco-aria-container";const e=()=>{const i=document.createElement("div");return i.className="monaco-alert",i.setAttribute("role","alert"),i.setAttribute("aria-atomic","true"),ju.appendChild(i),i};X0=e(),x2=e();const t=()=>{const i=document.createElement("div");return i.className="monaco-status",i.setAttribute("aria-live","polite"),i.setAttribute("aria-atomic","true"),ju.appendChild(i),i};Q0=t(),k2=t(),o.appendChild(ju)}function Cs(o){ju&&(X0.textContent!==o?(pn(x2),fL(X0,o)):(pn(X0),fL(x2,o)))}function Hl(o){ju&&(Q0.textContent!==o?(pn(k2),fL(Q0,o)):(pn(Q0),fL(k2,o)))}function fL(o,e){pn(o),e.length>K9&&(e=e.substr(0,K9)),o.textContent=e,o.style.visibility="hidden",o.style.visibility="visible"}var Gle=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},tC=function(o,e){return function(t,i){e(t,i,o)}};const Qa=ne;let D2=class extends Yo{get _targetWindow(){return xe(this._target.targetElements[0])}get _targetDocumentElement(){return xe(this._target.targetElements[0]).document.documentElement}get isDisposed(){return this._isDisposed}get isMouseIn(){return this._lockMouseTracker.isMouseIn}get domNode(){return this._hover.containerDomNode}get onDispose(){return this._onDispose.event}get onRequestLayout(){return this._onRequestLayout.event}get anchor(){return this._hoverPosition===2?0:1}get x(){return this._x}get y(){return this._y}get isLocked(){return this._isLocked}set isLocked(e){this._isLocked!==e&&(this._isLocked=e,this._hoverContainer.classList.toggle("locked",this._isLocked))}constructor(e,t,i,n,s,r){var u,g,f,p,_,b,C;super(),this._keybindingService=t,this._configurationService=i,this._openerService=n,this._instantiationService=s,this._accessibilityService=r,this._messageListeners=new K,this._isDisposed=!1,this._forcePosition=!1,this._x=0,this._y=0,this._isLocked=!1,this._enableFocusTraps=!1,this._addedFocusTrap=!1,this._onDispose=this._register(new P),this._onRequestLayout=this._register(new P),this._linkHandler=e.linkHandler||(v=>Y5(this._openerService,v,Aa(e.content)?e.content.isTrusted:void 0)),this._target="targetElements"in e.target?e.target:new Zle(e.target),this._hoverPointer=(u=e.appearance)!=null&&u.showPointer?Qa("div.workbench-hover-pointer"):void 0,this._hover=this._register(new W5),this._hover.containerDomNode.classList.add("workbench-hover","fadeIn"),(g=e.appearance)!=null&&g.compact&&this._hover.containerDomNode.classList.add("workbench-hover","compact"),(f=e.appearance)!=null&&f.skipFadeInAnimation&&this._hover.containerDomNode.classList.add("skip-fade-in"),e.additionalClasses&&this._hover.containerDomNode.classList.add(...e.additionalClasses),(p=e.position)!=null&&p.forcePosition&&(this._forcePosition=!0),e.trapFocus&&(this._enableFocusTraps=!0),this._hoverPosition=((_=e.position)==null?void 0:_.hoverPosition)??3,this.onmousedown(this._hover.containerDomNode,v=>v.stopPropagation()),this.onkeydown(this._hover.containerDomNode,v=>{v.equals(9)&&this.dispose()}),this._register(U(this._targetWindow,"blur",()=>this.dispose()));const a=Qa("div.hover-row.markdown-hover"),l=Qa("div.hover-contents");if(typeof e.content=="string")l.textContent=e.content,l.style.whiteSpace="pre-wrap";else if(Xi(e.content))l.appendChild(e.content),l.classList.add("html-hover-contents");else{const v=e.content,w=this._instantiationService.createInstance(nd,{codeBlockFontFamily:this._configurationService.getValue("editor").fontFamily||fs.fontFamily}),{element:S}=w.render(v,{actionHandler:{callback:L=>this._linkHandler(L),disposables:this._messageListeners},asyncRenderCallback:()=>{l.classList.add("code-hover-contents"),this.layout(),this._onRequestLayout.fire()}});l.appendChild(S)}if(a.appendChild(l),this._hover.contentsDomNode.appendChild(a),e.actions&&e.actions.length>0){const v=Qa("div.hover-row.status-bar"),w=Qa("div.actions");e.actions.forEach(S=>{const L=this._keybindingService.lookupKeybinding(S.commandId),k=L?L.getLabel():null;KI.render(w,{label:S.label,commandId:S.commandId,run:D=>{S.run(D),this.dispose()},iconClass:S.iconClass},k)}),v.appendChild(w),this._hover.containerDomNode.appendChild(v)}this._hoverContainer=Qa("div.workbench-hover-container"),this._hoverPointer&&this._hoverContainer.appendChild(this._hoverPointer),this._hoverContainer.appendChild(this._hover.containerDomNode);let c;if(e.actions&&e.actions.length>0?c=!1:((b=e.persistence)==null?void 0:b.hideOnHover)===void 0?c=typeof e.content=="string"||Aa(e.content)&&!e.content.value.includes("](")&&!e.content.value.includes("</a>"):c=e.persistence.hideOnHover,(C=e.appearance)!=null&&C.showHoverHint){const v=Qa("div.hover-row.status-bar"),w=Qa("div.info");w.textContent=m("hoverhint","Hold {0} key to mouse over",Je?"Option":"Alt"),v.appendChild(w),this._hover.containerDomNode.appendChild(v)}const d=[...this._target.targetElements];c||d.push(this._hoverContainer);const h=this._register(new q9(d));if(this._register(h.onMouseOut(()=>{this._isLocked||this.dispose()})),c){const v=[...this._target.targetElements,this._hoverContainer];this._lockMouseTracker=this._register(new q9(v)),this._register(this._lockMouseTracker.onMouseOut(()=>{this._isLocked||this.dispose()}))}else this._lockMouseTracker=h}addFocusTrap(){if(!this._enableFocusTraps||this._addedFocusTrap)return;this._addedFocusTrap=!0;const e=this._hover.containerDomNode,t=this.findLastFocusableChild(this._hover.containerDomNode);if(t){const i=c5(this._hoverContainer,Qa("div")),n=X(this._hoverContainer,Qa("div"));i.tabIndex=0,n.tabIndex=0,this._register(U(n,"focus",s=>{e.focus(),s.preventDefault()})),this._register(U(i,"focus",s=>{t.focus(),s.preventDefault()}))}}findLastFocusableChild(e){if(e.hasChildNodes())for(let t=0;t<e.childNodes.length;t++){const i=e.childNodes.item(e.childNodes.length-t-1);if(i.nodeType===i.ELEMENT_NODE){const s=i;if(typeof s.tabIndex=="number"&&s.tabIndex>=0)return s}const n=this.findLastFocusableChild(i);if(n)return n}}render(e){var n;e.appendChild(this._hoverContainer);const i=this._hoverContainer.contains(this._hoverContainer.ownerDocument.activeElement)&&Y$(this._configurationService.getValue("accessibility.verbosity.hover")===!0&&this._accessibilityService.isScreenReaderOptimized(),(n=this._keybindingService.lookupKeybinding("editor.action.accessibleView"))==null?void 0:n.getAriaLabel());i&&Hl(i),this.layout(),this.addFocusTrap()}layout(){this._hover.containerDomNode.classList.remove("right-aligned"),this._hover.contentsDomNode.style.maxHeight="";const e=d=>{const h=xU(d),u=d.getBoundingClientRect();return{top:u.top*h,bottom:u.bottom*h,right:u.right*h,left:u.left*h}},t=this._target.targetElements.map(d=>e(d)),{top:i,right:n,bottom:s,left:r}=t[0],a=n-r,l=s-i,c={top:i,right:n,bottom:s,left:r,width:a,height:l,center:{x:r+a/2,y:i+l/2}};if(this.adjustHorizontalHoverPosition(c),this.adjustVerticalHoverPosition(c),this.adjustHoverMaxHeight(c),this._hoverContainer.style.padding="",this._hoverContainer.style.margin="",this._hoverPointer){switch(this._hoverPosition){case 1:c.left+=3,c.right+=3,this._hoverContainer.style.paddingLeft="3px",this._hoverContainer.style.marginLeft="-3px";break;case 0:c.left-=3,c.right-=3,this._hoverContainer.style.paddingRight="3px",this._hoverContainer.style.marginRight="-3px";break;case 2:c.top+=3,c.bottom+=3,this._hoverContainer.style.paddingTop="3px",this._hoverContainer.style.marginTop="-3px";break;case 3:c.top-=3,c.bottom-=3,this._hoverContainer.style.paddingBottom="3px",this._hoverContainer.style.marginBottom="-3px";break}c.center.x=c.left+a/2,c.center.y=c.top+l/2}this.computeXCordinate(c),this.computeYCordinate(c),this._hoverPointer&&(this._hoverPointer.classList.remove("top"),this._hoverPointer.classList.remove("left"),this._hoverPointer.classList.remove("right"),this._hoverPointer.classList.remove("bottom"),this.setHoverPointerPosition(c)),this._hover.onContentsChanged()}computeXCordinate(e){const t=this._hover.containerDomNode.clientWidth+2;this._target.x!==void 0?this._x=this._target.x:this._hoverPosition===1?this._x=e.right:this._hoverPosition===0?this._x=e.left-t:(this._hoverPointer?this._x=e.center.x-this._hover.containerDomNode.clientWidth/2:this._x=e.left,this._x+t>=this._targetDocumentElement.clientWidth&&(this._hover.containerDomNode.classList.add("right-aligned"),this._x=Math.max(this._targetDocumentElement.clientWidth-t-2,this._targetDocumentElement.clientLeft))),this._x<this._targetDocumentElement.clientLeft&&(this._x=e.left+2)}computeYCordinate(e){this._target.y!==void 0?this._y=this._target.y:this._hoverPosition===3?this._y=e.top:this._hoverPosition===2?this._y=e.bottom-2:this._hoverPointer?this._y=e.center.y+this._hover.containerDomNode.clientHeight/2:this._y=e.bottom,this._y>this._targetWindow.innerHeight&&(this._y=e.bottom)}adjustHorizontalHoverPosition(e){if(this._target.x!==void 0)return;const t=this._hoverPointer?3:0;if(this._forcePosition){const i=t+2;this._hoverPosition===1?this._hover.containerDomNode.style.maxWidth=`${this._targetDocumentElement.clientWidth-e.right-i}px`:this._hoverPosition===0&&(this._hover.containerDomNode.style.maxWidth=`${e.left-i}px`);return}this._hoverPosition===1?this._targetDocumentElement.clientWidth-e.right<this._hover.containerDomNode.clientWidth+t&&(e.left>=this._hover.containerDomNode.clientWidth+t?this._hoverPosition=0:this._hoverPosition=2):this._hoverPosition===0&&(e.left<this._hover.containerDomNode.clientWidth+t&&(this._targetDocumentElement.clientWidth-e.right>=this._hover.containerDomNode.clientWidth+t?this._hoverPosition=1:this._hoverPosition=2),e.left-this._hover.containerDomNode.clientWidth-t<=this._targetDocumentElement.clientLeft&&(this._hoverPosition=1))}adjustVerticalHoverPosition(e){if(this._target.y!==void 0||this._forcePosition)return;const t=this._hoverPointer?3:0;this._hoverPosition===3?e.top-this._hover.containerDomNode.clientHeight-t<0&&(this._hoverPosition=2):this._hoverPosition===2&&e.bottom+this._hover.containerDomNode.clientHeight+t>this._targetWindow.innerHeight&&(this._hoverPosition=3)}adjustHoverMaxHeight(e){let t=this._targetWindow.innerHeight/2;if(this._forcePosition){const i=(this._hoverPointer?3:0)+2;this._hoverPosition===3?t=Math.min(t,e.top-i):this._hoverPosition===2&&(t=Math.min(t,this._targetWindow.innerHeight-e.bottom-i))}if(this._hover.containerDomNode.style.maxHeight=`${t}px`,this._hover.contentsDomNode.clientHeight<this._hover.contentsDomNode.scrollHeight){const i=`${this._hover.scrollbar.options.verticalScrollbarSize}px`;this._hover.contentsDomNode.style.paddingRight!==i&&(this._hover.contentsDomNode.style.paddingRight=i)}}setHoverPointerPosition(e){if(this._hoverPointer)switch(this._hoverPosition){case 0:case 1:{this._hoverPointer.classList.add(this._hoverPosition===0?"right":"left");const t=this._hover.containerDomNode.clientHeight;t>e.height?this._hoverPointer.style.top=`${e.center.y-(this._y-t)-3}px`:this._hoverPointer.style.top=`${Math.round(t/2)-3}px`;break}case 3:case 2:{this._hoverPointer.classList.add(this._hoverPosition===3?"bottom":"top");const t=this._hover.containerDomNode.clientWidth;let i=Math.round(t/2)-3;const n=this._x+i;(n<e.left||n>e.right)&&(i=e.center.x-this._x-3),this._hoverPointer.style.left=`${i}px`;break}}}focus(){this._hover.containerDomNode.focus()}dispose(){this._isDisposed||(this._onDispose.fire(),this._hoverContainer.remove(),this._messageListeners.dispose(),this._target.dispose(),super.dispose()),this._isDisposed=!0}};D2=Gle([tC(1,yt),tC(2,qe),tC(3,zs),tC(4,we),tC(5,po)],D2);class q9 extends Yo{get onMouseOut(){return this._onMouseOut.event}get isMouseIn(){return this._isMouseIn}constructor(e){super(),this._elements=e,this._isMouseIn=!0,this._onMouseOut=this._register(new P),this._elements.forEach(t=>this.onmouseover(t,()=>this._onTargetMouseOver(t))),this._elements.forEach(t=>this.onmouseleave(t,()=>this._onTargetMouseLeave(t)))}_onTargetMouseOver(e){this._isMouseIn=!0,this._clearEvaluateMouseStateTimeout(e)}_onTargetMouseLeave(e){this._isMouseIn=!1,this._evaluateMouseState(e)}_evaluateMouseState(e){this._clearEvaluateMouseStateTimeout(e),this._mouseTimeout=xe(e).setTimeout(()=>this._fireIfMouseOutside(),0)}_clearEvaluateMouseStateTimeout(e){this._mouseTimeout&&(xe(e).clearTimeout(this._mouseTimeout),this._mouseTimeout=void 0)}_fireIfMouseOutside(){this._isMouseIn||this._onMouseOut.fire()}}class Zle{constructor(e){this._element=e,this.targetElements=[this._element]}dispose(){}}var Rn;(function(o){function e(s,r){if(s.start>=r.end||r.start>=s.end)return{start:0,end:0};const a=Math.max(s.start,r.start),l=Math.min(s.end,r.end);return l-a<=0?{start:0,end:0}:{start:a,end:l}}o.intersect=e;function t(s){return s.end-s.start<=0}o.isEmpty=t;function i(s,r){return!t(e(s,r))}o.intersects=i;function n(s,r){const a=[],l={start:s.start,end:Math.min(r.start,s.end)},c={start:Math.max(r.end,s.start),end:s.end};return t(l)||a.push(l),t(c)||a.push(c),a}o.relativeComplement=n})(Rn||(Rn={}));function Yle(o){const e=o;return!!e&&typeof e.x=="number"&&typeof e.y=="number"}var Xd;(function(o){o[o.AVOID=0]="AVOID",o[o.ALIGN=1]="ALIGN"})(Xd||(Xd={}));function op(o,e,t){const i=t.mode===Xd.ALIGN?t.offset:t.offset+t.size,n=t.mode===Xd.ALIGN?t.offset+t.size:t.offset;return t.position===0?e<=o-i?i:e<=n?n-e:Math.max(o-e,0):e<=n?n-e:e<=o-i?i:0}const Np=class Np extends F{constructor(e,t){super(),this.container=null,this.useFixedPosition=!1,this.useShadowDOM=!1,this.delegate=null,this.toDisposeOnClean=F.None,this.toDisposeOnSetContainer=F.None,this.shadowRoot=null,this.shadowRootHostElement=null,this.view=ne(".context-view"),rs(this.view),this.setContainer(e,t),this._register(ve(()=>this.setContainer(null,1)))}setContainer(e,t){var n;this.useFixedPosition=t!==1;const i=this.useShadowDOM;if(this.useShadowDOM=t===3,!(e===this.container&&i===this.useShadowDOM)&&(this.container&&(this.toDisposeOnSetContainer.dispose(),this.view.remove(),this.shadowRoot&&(this.shadowRoot=null,(n=this.shadowRootHostElement)==null||n.remove(),this.shadowRootHostElement=null),this.container=null),e)){if(this.container=e,this.useShadowDOM){this.shadowRootHostElement=ne(".shadow-root-host"),this.container.appendChild(this.shadowRootHostElement),this.shadowRoot=this.shadowRootHostElement.attachShadow({mode:"open"});const r=document.createElement("style");r.textContent=Xle,this.shadowRoot.appendChild(r),this.shadowRoot.appendChild(this.view),this.shadowRoot.appendChild(ne("slot"))}else this.container.appendChild(this.view);const s=new K;Np.BUBBLE_UP_EVENTS.forEach(r=>{s.add(hi(this.container,r,a=>{this.onDOMEvent(a,!1)}))}),Np.BUBBLE_DOWN_EVENTS.forEach(r=>{s.add(hi(this.container,r,a=>{this.onDOMEvent(a,!0)},!0))}),this.toDisposeOnSetContainer=s}}show(e){var t,i;this.isVisible()&&this.hide(),pn(this.view),this.view.className="context-view monaco-component",this.view.style.top="0px",this.view.style.left="0px",this.view.style.zIndex=`${2575+(e.layer??0)}`,this.view.style.position=this.useFixedPosition?"fixed":"absolute",to(this.view),this.toDisposeOnClean=e.render(this.view)||F.None,this.delegate=e,this.doLayout(),(i=(t=this.delegate).focus)==null||i.call(t)}getViewElement(){return this.view}layout(){var e,t;if(this.isVisible()){if(this.delegate.canRelayout===!1&&!(Pa&&i5.pointerEvents)){this.hide();return}(t=(e=this.delegate)==null?void 0:e.layout)==null||t.call(e),this.doLayout()}}doLayout(){if(!this.isVisible())return;const e=this.delegate.getAnchor();let t;if(Xi(e)){const u=wi(e),g=xU(e);t={top:u.top*g,left:u.left*g,width:u.width*g,height:u.height*g}}else Yle(e)?t={top:e.y,left:e.x,width:e.width||1,height:e.height||2}:t={top:e.posy,left:e.posx,width:2,height:2};const i=Xs(this.view),n=Tc(this.view),s=this.delegate.anchorPosition||0,r=this.delegate.anchorAlignment||0,a=this.delegate.anchorAxisAlignment||0;let l,c;const d=wC();if(a===0){const u={offset:t.top-d.pageYOffset,size:t.height,position:s===0?0:1},g={offset:t.left,size:t.width,position:r===0?0:1,mode:Xd.ALIGN};l=op(d.innerHeight,n,u)+d.pageYOffset,Rn.intersects({start:l,end:l+n},{start:u.offset,end:u.offset+u.size})&&(g.mode=Xd.AVOID),c=op(d.innerWidth,i,g)}else{const u={offset:t.left,size:t.width,position:r===0?0:1},g={offset:t.top,size:t.height,position:s===0?0:1,mode:Xd.ALIGN};c=op(d.innerWidth,i,u),Rn.intersects({start:c,end:c+i},{start:u.offset,end:u.offset+u.size})&&(g.mode=Xd.AVOID),l=op(d.innerHeight,n,g)+d.pageYOffset}this.view.classList.remove("top","bottom","left","right"),this.view.classList.add(s===0?"bottom":"top"),this.view.classList.add(r===0?"left":"right"),this.view.classList.toggle("fixed",this.useFixedPosition);const h=wi(this.container);this.view.style.top=`${l-(this.useFixedPosition?wi(this.view).top:h.top)}px`,this.view.style.left=`${c-(this.useFixedPosition?wi(this.view).left:h.left)}px`,this.view.style.width="initial"}hide(e){const t=this.delegate;this.delegate=null,t!=null&&t.onHide&&t.onHide(e),this.toDisposeOnClean.dispose(),rs(this.view)}isVisible(){return!!this.delegate}onDOMEvent(e,t){this.delegate&&(this.delegate.onDOMEvent?this.delegate.onDOMEvent(e,xe(e).document.activeElement):t&&!Zi(e.target,this.container)&&this.hide())}dispose(){this.hide(),super.dispose()}};Np.BUBBLE_UP_EVENTS=["click","keydown","focus","blur"],Np.BUBBLE_DOWN_EVENTS=["click"];let I2=Np;const Xle=`
	:host {
		all: initial; /* 1st rule so subsequent properties are reset. */
	}

	.codicon[class*='codicon-'] {
		font: normal normal normal 16px/1 codicon;
		display: inline-block;
		text-decoration: none;
		text-rendering: auto;
		text-align: center;
		-webkit-font-smoothing: antialiased;
		-moz-osx-font-smoothing: grayscale;
		user-select: none;
		-webkit-user-select: none;
		-ms-user-select: none;
	}

	:host {
		font-family: -apple-system, BlinkMacSystemFont, "Segoe WPC", "Segoe UI", "HelveticaNeue-Light", system-ui, "Ubuntu", "Droid Sans", sans-serif;
	}

	:host-context(.mac) { font-family: -apple-system, BlinkMacSystemFont, sans-serif; }
	:host-context(.mac:lang(zh-Hans)) { font-family: -apple-system, BlinkMacSystemFont, "PingFang SC", "Hiragino Sans GB", sans-serif; }
	:host-context(.mac:lang(zh-Hant)) { font-family: -apple-system, BlinkMacSystemFont, "PingFang TC", sans-serif; }
	:host-context(.mac:lang(ja)) { font-family: -apple-system, BlinkMacSystemFont, "Hiragino Kaku Gothic Pro", sans-serif; }
	:host-context(.mac:lang(ko)) { font-family: -apple-system, BlinkMacSystemFont, "Nanum Gothic", "Apple SD Gothic Neo", "AppleGothic", sans-serif; }

	:host-context(.windows) { font-family: "Segoe WPC", "Segoe UI", sans-serif; }
	:host-context(.windows:lang(zh-Hans)) { font-family: "Segoe WPC", "Segoe UI", "Microsoft YaHei", sans-serif; }
	:host-context(.windows:lang(zh-Hant)) { font-family: "Segoe WPC", "Segoe UI", "Microsoft Jhenghei", sans-serif; }
	:host-context(.windows:lang(ja)) { font-family: "Segoe WPC", "Segoe UI", "Yu Gothic UI", "Meiryo UI", sans-serif; }
	:host-context(.windows:lang(ko)) { font-family: "Segoe WPC", "Segoe UI", "Malgun Gothic", "Dotom", sans-serif; }

	:host-context(.linux) { font-family: system-ui, "Ubuntu", "Droid Sans", sans-serif; }
	:host-context(.linux:lang(zh-Hans)) { font-family: system-ui, "Ubuntu", "Droid Sans", "Source Han Sans SC", "Source Han Sans CN", "Source Han Sans", sans-serif; }
	:host-context(.linux:lang(zh-Hant)) { font-family: system-ui, "Ubuntu", "Droid Sans", "Source Han Sans TC", "Source Han Sans TW", "Source Han Sans", sans-serif; }
	:host-context(.linux:lang(ja)) { font-family: system-ui, "Ubuntu", "Droid Sans", "Source Han Sans J", "Source Han Sans JP", "Source Han Sans", sans-serif; }
	:host-context(.linux:lang(ko)) { font-family: system-ui, "Ubuntu", "Droid Sans", "Source Han Sans K", "Source Han Sans JR", "Source Han Sans", "UnDotum", "FBaekmuk Gulim", sans-serif; }
`;var Qle=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Jle=function(o,e){return function(t,i){e(t,i,o)}};let mL=class extends F{constructor(e){super(),this.layoutService=e,this.contextView=this._register(new I2(this.layoutService.mainContainer,1)),this.layout(),this._register(e.onDidLayoutContainer(()=>this.layout()))}showContextView(e,t,i){let n;t?t===this.layoutService.getContainer(xe(t))?n=1:i?n=3:n=2:n=1,this.contextView.setContainer(t??this.layoutService.activeContainer,n),this.contextView.show(e);const s={close:()=>{this.openContextView===s&&this.hideContextView()}};return this.openContextView=s,s}layout(){this.contextView.layout()}hideContextView(e){this.contextView.hide(e),this.openContextView=void 0}};mL=Qle([Jle(0,tu)],mL);class ece extends mL{getContextViewElement(){return this.contextView.getViewElement()}}class tce{constructor(e,t,i){this.hoverDelegate=e,this.target=t,this.fadeInAnimation=i}async update(e,t,i){if(this._cancellationTokenSource&&(this._cancellationTokenSource.dispose(!0),this._cancellationTokenSource=void 0),this.isDisposed)return;let n;if(e===void 0||As(e)||Xi(e))n=e;else if(!Xv(e.markdown))n=e.markdown??e.markdownNotSupportedFallback;else{this._hoverWidget||this.show(m("iconLabel.loading","Loading..."),t,i),this._cancellationTokenSource=new li;const s=this._cancellationTokenSource.token;if(n=await e.markdown(s),n===void 0&&(n=e.markdownNotSupportedFallback),this.isDisposed||s.isCancellationRequested)return}this.show(n,t,i)}show(e,t,i){var s;const n=this._hoverWidget;if(this.hasContent(e)){const r={content:e,target:this.target,actions:i==null?void 0:i.actions,linkHandler:i==null?void 0:i.linkHandler,trapFocus:i==null?void 0:i.trapFocus,appearance:{showPointer:this.hoverDelegate.placement==="element",skipFadeInAnimation:!this.fadeInAnimation||!!n,showHoverHint:(s=i==null?void 0:i.appearance)==null?void 0:s.showHoverHint},position:{hoverPosition:2}};this._hoverWidget=this.hoverDelegate.showHover(r,t)}n==null||n.dispose()}hasContent(e){return e?Aa(e)?!!e.value:!0:!1}get isDisposed(){var e;return(e=this._hoverWidget)==null?void 0:e.isDisposed}dispose(){var e,t;(e=this._hoverWidget)==null||e.dispose(),(t=this._cancellationTokenSource)==null||t.dispose(!0),this._cancellationTokenSource=void 0}}var ice=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},iC=function(o,e){return function(t,i){e(t,i,o)}};let E2=class extends F{constructor(e,t,i,n,s){super(),this._instantiationService=e,this._keybindingService=i,this._layoutService=n,this._accessibilityService=s,this._managedHovers=new Map,t.onDidShowContextMenu(()=>this.hideHover()),this._contextViewHandler=this._register(new mL(this._layoutService))}showHover(e,t,i){var l,c,d,h;if(G9(this._currentHoverOptions)===G9(e)||this._currentHover&&((c=(l=this._currentHoverOptions)==null?void 0:l.persistence)!=null&&c.sticky))return;this._currentHoverOptions=e,this._lastHoverOptions=e;const n=e.trapFocus||this._accessibilityService.isScreenReaderOptimized(),s=Dn();i||(n&&s?s.classList.contains("monaco-hover")||(this._lastFocusedElementBeforeOpen=s):this._lastFocusedElementBeforeOpen=void 0);const r=new K,a=this._instantiationService.createInstance(D2,e);if((d=e.persistence)!=null&&d.sticky&&(a.isLocked=!0),a.onDispose(()=>{var g,f;((g=this._currentHover)==null?void 0:g.domNode)&&DU(this._currentHover.domNode)&&((f=this._lastFocusedElementBeforeOpen)==null||f.focus()),this._currentHoverOptions===e&&(this._currentHoverOptions=void 0),r.dispose()},void 0,r),!e.container){const u=Xi(e.target)?e.target:e.target.targetElements[0];e.container=this._layoutService.getContainer(xe(u))}if(this._contextViewHandler.showContextView(new nce(a,t),e.container),a.onRequestLayout(()=>this._contextViewHandler.layout(),void 0,r),(h=e.persistence)!=null&&h.sticky)r.add(U(xe(e.container).document,ie.MOUSE_DOWN,u=>{Zi(u.target,a.domNode)||this.doHideHover()}));else{if("targetElements"in e.target)for(const g of e.target.targetElements)r.add(U(g,ie.CLICK,()=>this.hideHover()));else r.add(U(e.target,ie.CLICK,()=>this.hideHover()));const u=Dn();if(u){const g=xe(u).document;r.add(U(u,ie.KEY_DOWN,f=>{var p;return this._keyDown(f,a,!!((p=e.persistence)!=null&&p.hideOnKeyDown))})),r.add(U(g,ie.KEY_DOWN,f=>{var p;return this._keyDown(f,a,!!((p=e.persistence)!=null&&p.hideOnKeyDown))})),r.add(U(u,ie.KEY_UP,f=>this._keyUp(f,a))),r.add(U(g,ie.KEY_UP,f=>this._keyUp(f,a)))}}if("IntersectionObserver"in Rt){const u=new IntersectionObserver(f=>this._intersectionChange(f,a),{threshold:0}),g="targetElements"in e.target?e.target.targetElements[0]:e.target;u.observe(g),r.add(ve(()=>u.disconnect()))}return this._currentHover=a,a}hideHover(){var e;(e=this._currentHover)!=null&&e.isLocked||!this._currentHoverOptions||this.doHideHover()}doHideHover(){this._currentHover=void 0,this._currentHoverOptions=void 0,this._contextViewHandler.hideContextView()}_intersectionChange(e,t){e[e.length-1].isIntersecting||t.dispose()}showAndFocusLastHover(){this._lastHoverOptions&&this.showHover(this._lastHoverOptions,!0,!0)}_keyDown(e,t,i){var r,a;if(e.key==="Alt"){t.isLocked=!0;return}const n=new Pt(e);this._keybindingService.resolveKeyboardEvent(n).getSingleModifierDispatchChords().some(l=>!!l)||this._keybindingService.softDispatch(n,n.target).kind!==0||i&&(!((r=this._currentHoverOptions)!=null&&r.trapFocus)||e.key!=="Tab")&&(this.hideHover(),(a=this._lastFocusedElementBeforeOpen)==null||a.focus())}_keyUp(e,t){var i;e.key==="Alt"&&(t.isLocked=!1,t.isMouseIn||(this.hideHover(),(i=this._lastFocusedElementBeforeOpen)==null||i.focus()))}setupManagedHover(e,t,i,n){t.setAttribute("custom-hover","true"),t.title!==""&&(console.warn("HTML element already has a title attribute, which will conflict with the custom hover. Please remove the title attribute."),console.trace("Stack trace:",t.title),t.title="");let s,r;const a=(v,w)=>{var L;const S=r!==void 0;v&&(r==null||r.dispose(),r=void 0),w&&(s==null||s.dispose(),s=void 0),S&&((L=e.onDidHideHover)==null||L.call(e),r=void 0)},l=(v,w,S,L)=>new ea(async()=>{(!r||r.isDisposed)&&(r=new tce(e,S||t,v>0),await r.update(typeof i=="function"?i():i,w,{...n,trapFocus:L}))},v);let c=!1;const d=U(t,ie.MOUSE_DOWN,()=>{c=!0,a(!0,!0)},!0),h=U(t,ie.MOUSE_UP,()=>{c=!1},!0),u=U(t,ie.MOUSE_LEAVE,v=>{c=!1,a(!1,v.fromElement===t)},!0),g=v=>{if(s)return;const w=new K,S={targetElements:[t],dispose:()=>{}};if(e.placement===void 0||e.placement==="mouse"){const L=k=>{S.x=k.x+10,Xi(k.target)&&Z9(k.target,t)!==t&&a(!0,!0)};w.add(U(t,ie.MOUSE_MOVE,L,!0))}s=w,!(Xi(v.target)&&Z9(v.target,t)!==t)&&w.add(l(e.delay,!1,S))},f=U(t,ie.MOUSE_OVER,g,!0),p=()=>{if(c||s)return;const v={targetElements:[t],dispose:()=>{}},w=new K,S=()=>a(!0,!0);w.add(U(t,ie.BLUR,S,!0)),w.add(l(e.delay,!1,v)),s=w};let _;const b=t.tagName.toLowerCase();b!=="input"&&b!=="textarea"&&(_=U(t,ie.FOCUS,p,!0));const C={show:v=>{a(!1,!0),l(0,v,void 0,v)},hide:()=>{a(!0,!0)},update:async(v,w)=>{i=v,await(r==null?void 0:r.update(i,void 0,w))},dispose:()=>{this._managedHovers.delete(t),f.dispose(),u.dispose(),d.dispose(),h.dispose(),_==null||_.dispose(),a(!0,!0)}};return this._managedHovers.set(t,C),C}showManagedHover(e){const t=this._managedHovers.get(e);t&&t.show(!0)}dispose(){this._managedHovers.forEach(e=>e.dispose()),super.dispose()}};E2=ice([iC(0,we),iC(1,mo),iC(2,yt),iC(3,tu),iC(4,po)],E2);function G9(o){if(o!==void 0)return(o==null?void 0:o.id)??o}class nce{get anchorPosition(){return this._hover.anchor}constructor(e,t=!1){this._hover=e,this._focus=t,this.layer=1}render(e){return this._hover.render(e),this._focus&&this._hover.focus(),this._hover}getAnchor(){return{x:this._hover.x,y:this._hover.y}}layout(){this._hover.layout()}}function Z9(o,e){for(e=e??xe(o).document.body;!o.hasAttribute("custom-hover")&&o!==e;)o=o.parentElement;return o}st(Yl,E2,1);Lr((o,e)=>{const t=o.getColor(O$);t&&(e.addRule(`.monaco-workbench .workbench-hover .hover-row:not(:first-child):not(:empty) { border-top: 1px solid ${t.transparent(.5)}; }`),e.addRule(`.monaco-workbench .workbench-hover hr { border-top: 1px solid ${t.transparent(.5)}; }`))});const j1=et("IWorkspaceEditService");class X5{constructor(e){this.metadata=e}static convert(e){return e.edits.map(t=>{if(hh.is(t))return hh.lift(t);if(qp.is(t))return qp.lift(t);throw new Error("Unsupported edit")})}}class hh extends X5{static is(e){return e instanceof hh?!0:on(e)&&Se.isUri(e.resource)&&on(e.textEdit)}static lift(e){return e instanceof hh?e:new hh(e.resource,e.textEdit,e.versionId,e.metadata)}constructor(e,t,i=void 0,n){super(n),this.resource=e,this.textEdit=t,this.versionId=i}}class qp extends X5{static is(e){return e instanceof qp?!0:on(e)&&(!!e.newResource||!!e.oldResource)}static lift(e){return e instanceof qp?e:new qp(e.oldResource,e.newResource,e.options,e.metadata)}constructor(e,t,i={},n){super(n),this.oldResource=e,this.newResource=t,this.options=i}}const vn={enableSplitViewResizing:!0,renderSideBySide:!0,renderMarginRevertIcon:!0,renderGutterMenu:!0,maxComputationTime:5e3,maxFileSize:50,ignoreTrimWhitespace:!0,renderIndicators:!0,originalEditable:!1,diffCodeLens:!1,renderOverviewRuler:!0,diffWordWrap:"inherit",diffAlgorithm:"advanced",accessibilityVerbose:!1,experimental:{showMoves:!1,showEmptyDecorations:!0,useTrueInlineView:!1},hideUnchangedRegions:{enabled:!1,contextLineCount:3,minimumLineCount:3,revealLineCount:20},isInEmbeddedEditor:!1,onlyShowAccessibleDiffViewer:!1,renderSideBySideInlineBreakpoint:900,useInlineViewWhenSpaceIsLimited:!0,compactMode:!1},K1=Object.freeze({id:"editor",order:5,type:"object",title:m("editorConfigurationTitle","Editor"),scope:5}),pL={...K1,properties:{"editor.tabSize":{type:"number",default:An.tabSize,minimum:1,markdownDescription:m("tabSize","The number of spaces a tab is equal to. This setting is overridden based on the file contents when {0} is on.","`#editor.detectIndentation#`")},"editor.indentSize":{anyOf:[{type:"string",enum:["tabSize"]},{type:"number",minimum:1}],default:"tabSize",markdownDescription:m("indentSize",'The number of spaces used for indentation or `"tabSize"` to use the value from `#editor.tabSize#`. This setting is overridden based on the file contents when `#editor.detectIndentation#` is on.')},"editor.insertSpaces":{type:"boolean",default:An.insertSpaces,markdownDescription:m("insertSpaces","Insert spaces when pressing `Tab`. This setting is overridden based on the file contents when {0} is on.","`#editor.detectIndentation#`")},"editor.detectIndentation":{type:"boolean",default:An.detectIndentation,markdownDescription:m("detectIndentation","Controls whether {0} and {1} will be automatically detected when a file is opened based on the file contents.","`#editor.tabSize#`","`#editor.insertSpaces#`")},"editor.trimAutoWhitespace":{type:"boolean",default:An.trimAutoWhitespace,description:m("trimAutoWhitespace","Remove trailing auto inserted whitespace.")},"editor.largeFileOptimizations":{type:"boolean",default:An.largeFileOptimizations,description:m("largeFileOptimizations","Special handling for large files to disable certain memory intensive features.")},"editor.wordBasedSuggestions":{enum:["off","currentDocument","matchingDocuments","allDocuments"],default:"matchingDocuments",enumDescriptions:[m("wordBasedSuggestions.off","Turn off Word Based Suggestions."),m("wordBasedSuggestions.currentDocument","Only suggest words from the active document."),m("wordBasedSuggestions.matchingDocuments","Suggest words from all open documents of the same language."),m("wordBasedSuggestions.allDocuments","Suggest words from all open documents.")],description:m("wordBasedSuggestions","Controls whether completions should be computed based on words in the document and from which documents they are computed.")},"editor.semanticHighlighting.enabled":{enum:[!0,!1,"configuredByTheme"],enumDescriptions:[m("semanticHighlighting.true","Semantic highlighting enabled for all color themes."),m("semanticHighlighting.false","Semantic highlighting disabled for all color themes."),m("semanticHighlighting.configuredByTheme","Semantic highlighting is configured by the current color theme's `semanticHighlighting` setting.")],default:"configuredByTheme",description:m("semanticHighlighting.enabled","Controls whether the semanticHighlighting is shown for the languages that support it.")},"editor.stablePeek":{type:"boolean",default:!1,markdownDescription:m("stablePeek","Keep peek editors open even when double-clicking their content or when hitting `Escape`.")},"editor.maxTokenizationLineLength":{type:"integer",default:2e4,description:m("maxTokenizationLineLength","Lines above this length will not be tokenized for performance reasons")},"editor.experimental.asyncTokenization":{type:"boolean",default:!0,description:m("editor.experimental.asyncTokenization","Controls whether the tokenization should happen asynchronously on a web worker."),tags:["experimental"]},"editor.experimental.asyncTokenizationLogging":{type:"boolean",default:!1,description:m("editor.experimental.asyncTokenizationLogging","Controls whether async tokenization should be logged. For debugging only.")},"editor.experimental.asyncTokenizationVerification":{type:"boolean",default:!1,description:m("editor.experimental.asyncTokenizationVerification","Controls whether async tokenization should be verified against legacy background tokenization. Might slow down tokenization. For debugging only."),tags:["experimental"]},"editor.experimental.treeSitterTelemetry":{type:"boolean",default:!1,markdownDescription:m("editor.experimental.treeSitterTelemetry","Controls whether tree sitter parsing should be turned on and telemetry collected. Setting `editor.experimental.preferTreeSitter` for specific languages will take precedence."),tags:["experimental"]},"editor.language.brackets":{type:["array","null"],default:null,description:m("schema.brackets","Defines the bracket symbols that increase or decrease the indentation."),items:{type:"array",items:[{type:"string",description:m("schema.openBracket","The opening bracket character or string sequence.")},{type:"string",description:m("schema.closeBracket","The closing bracket character or string sequence.")}]}},"editor.language.colorizedBracketPairs":{type:["array","null"],default:null,description:m("schema.colorizedBracketPairs","Defines the bracket pairs that are colorized by their nesting level if bracket pair colorization is enabled."),items:{type:"array",items:[{type:"string",description:m("schema.openBracket","The opening bracket character or string sequence.")},{type:"string",description:m("schema.closeBracket","The closing bracket character or string sequence.")}]}},"diffEditor.maxComputationTime":{type:"number",default:vn.maxComputationTime,description:m("maxComputationTime","Timeout in milliseconds after which diff computation is cancelled. Use 0 for no timeout.")},"diffEditor.maxFileSize":{type:"number",default:vn.maxFileSize,description:m("maxFileSize","Maximum file size in MB for which to compute diffs. Use 0 for no limit.")},"diffEditor.renderSideBySide":{type:"boolean",default:vn.renderSideBySide,description:m("sideBySide","Controls whether the diff editor shows the diff side by side or inline.")},"diffEditor.renderSideBySideInlineBreakpoint":{type:"number",default:vn.renderSideBySideInlineBreakpoint,description:m("renderSideBySideInlineBreakpoint","If the diff editor width is smaller than this value, the inline view is used.")},"diffEditor.useInlineViewWhenSpaceIsLimited":{type:"boolean",default:vn.useInlineViewWhenSpaceIsLimited,description:m("useInlineViewWhenSpaceIsLimited","If enabled and the editor width is too small, the inline view is used.")},"diffEditor.renderMarginRevertIcon":{type:"boolean",default:vn.renderMarginRevertIcon,description:m("renderMarginRevertIcon","When enabled, the diff editor shows arrows in its glyph margin to revert changes.")},"diffEditor.renderGutterMenu":{type:"boolean",default:vn.renderGutterMenu,description:m("renderGutterMenu","When enabled, the diff editor shows a special gutter for revert and stage actions.")},"diffEditor.ignoreTrimWhitespace":{type:"boolean",default:vn.ignoreTrimWhitespace,description:m("ignoreTrimWhitespace","When enabled, the diff editor ignores changes in leading or trailing whitespace.")},"diffEditor.renderIndicators":{type:"boolean",default:vn.renderIndicators,description:m("renderIndicators","Controls whether the diff editor shows +/- indicators for added/removed changes.")},"diffEditor.codeLens":{type:"boolean",default:vn.diffCodeLens,description:m("codeLens","Controls whether the editor shows CodeLens.")},"diffEditor.wordWrap":{type:"string",enum:["off","on","inherit"],default:vn.diffWordWrap,markdownEnumDescriptions:[m("wordWrap.off","Lines will never wrap."),m("wordWrap.on","Lines will wrap at the viewport width."),m("wordWrap.inherit","Lines will wrap according to the {0} setting.","`#editor.wordWrap#`")]},"diffEditor.diffAlgorithm":{type:"string",enum:["legacy","advanced"],default:vn.diffAlgorithm,markdownEnumDescriptions:[m("diffAlgorithm.legacy","Uses the legacy diffing algorithm."),m("diffAlgorithm.advanced","Uses the advanced diffing algorithm.")],tags:["experimental"]},"diffEditor.hideUnchangedRegions.enabled":{type:"boolean",default:vn.hideUnchangedRegions.enabled,markdownDescription:m("hideUnchangedRegions.enabled","Controls whether the diff editor shows unchanged regions.")},"diffEditor.hideUnchangedRegions.revealLineCount":{type:"integer",default:vn.hideUnchangedRegions.revealLineCount,markdownDescription:m("hideUnchangedRegions.revealLineCount","Controls how many lines are used for unchanged regions."),minimum:1},"diffEditor.hideUnchangedRegions.minimumLineCount":{type:"integer",default:vn.hideUnchangedRegions.minimumLineCount,markdownDescription:m("hideUnchangedRegions.minimumLineCount","Controls how many lines are used as a minimum for unchanged regions."),minimum:1},"diffEditor.hideUnchangedRegions.contextLineCount":{type:"integer",default:vn.hideUnchangedRegions.contextLineCount,markdownDescription:m("hideUnchangedRegions.contextLineCount","Controls how many lines are used as context when comparing unchanged regions."),minimum:1},"diffEditor.experimental.showMoves":{type:"boolean",default:vn.experimental.showMoves,markdownDescription:m("showMoves","Controls whether the diff editor should show detected code moves.")},"diffEditor.experimental.showEmptyDecorations":{type:"boolean",default:vn.experimental.showEmptyDecorations,description:m("showEmptyDecorations","Controls whether the diff editor shows empty decorations to see where characters got inserted or deleted.")},"diffEditor.experimental.useTrueInlineView":{type:"boolean",default:vn.experimental.useTrueInlineView,description:m("useTrueInlineView","If enabled and the editor uses the inline view, word changes are rendered inline.")}}};function sce(o){return typeof o.type<"u"||typeof o.anyOf<"u"}for(const o of Jm){const e=o.schema;if(typeof e<"u")if(sce(e))pL.properties[`editor.${o.name}`]=e;else for(const t in e)Object.hasOwnProperty.call(e,t)&&(pL.properties[t]=e[t])}let US=null;function fj(){return US===null&&(US=Object.create(null),Object.keys(pL.properties).forEach(o=>{US[o]=!0})),US}function oce(o){return fj()[`editor.${o}`]||!1}function rce(o){return fj()[`diffEditor.${o}`]||!1}const ace=ci.as(ta.Configuration);ace.registerConfiguration(pL);class ri{static insert(e,t){return{range:new x(e.lineNumber,e.column,e.lineNumber,e.column),text:t,forceMoveMarkers:!0}}static delete(e){return{range:e,text:null}}static replace(e,t){return{range:e,text:t}}static replaceMove(e,t){return{range:e,text:t,forceMoveMarkers:!0}}}function $S(o){return Object.isFrozen(o)?o:DQ(o)}class mn{static createEmptyModel(e){return new mn({},[],[],void 0,e)}constructor(e,t,i,n,s){this._contents=e,this._keys=t,this._overrides=i,this.raw=n,this.logService=s,this.overrideConfigurations=new Map}get rawConfiguration(){var e;if(!this._rawConfiguration)if((e=this.raw)!=null&&e.length){const t=this.raw.map(i=>{if(i instanceof mn)return i;const n=new lce("",this.logService);return n.parseRaw(i),n.configurationModel});this._rawConfiguration=t.reduce((i,n)=>n===i?n:i.merge(n),t[0])}else this._rawConfiguration=this;return this._rawConfiguration}get contents(){return this._contents}get overrides(){return this._overrides}get keys(){return this._keys}isEmpty(){return this._keys.length===0&&Object.keys(this._contents).length===0&&this._overrides.length===0}getValue(e){return e?y7(this.contents,e):this.contents}inspect(e,t){const i=this;return{get value(){return $S(i.rawConfiguration.getValue(e))},get override(){return t?$S(i.rawConfiguration.getOverrideValue(e,t)):void 0},get merged(){return $S(t?i.rawConfiguration.override(t).getValue(e):i.rawConfiguration.getValue(e))},get overrides(){const n=[];for(const{contents:s,identifiers:r,keys:a}of i.rawConfiguration.overrides){const l=new mn(s,a,[],void 0,i.logService).getValue(e);l!==void 0&&n.push({identifiers:r,value:l})}return n.length?$S(n):void 0}}}getOverrideValue(e,t){const i=this.getContentsForOverrideIdentifer(t);return i?e?y7(i,e):i:void 0}override(e){let t=this.overrideConfigurations.get(e);return t||(t=this.createOverrideConfigurationModel(e),this.overrideConfigurations.set(e,t)),t}merge(...e){var r,a;const t=vc(this.contents),i=vc(this.overrides),n=[...this.keys],s=(r=this.raw)!=null&&r.length?[...this.raw]:[this];for(const l of e)if(s.push(...(a=l.raw)!=null&&a.length?l.raw:[l]),!l.isEmpty()){this.mergeContents(t,l.contents);for(const c of l.overrides){const[d]=i.filter(h=>ii(h.identifiers,c.identifiers));d?(this.mergeContents(d.contents,c.contents),d.keys.push(...c.keys),d.keys=Xc(d.keys)):i.push(vc(c))}for(const c of l.keys)n.indexOf(c)===-1&&n.push(c)}return new mn(t,n,i,s.every(l=>l instanceof mn)?void 0:s,this.logService)}createOverrideConfigurationModel(e){const t=this.getContentsForOverrideIdentifer(e);if(!t||typeof t!="object"||!Object.keys(t).length)return this;const i={};for(const n of Xc([...Object.keys(this.contents),...Object.keys(t)])){let s=this.contents[n];const r=t[n];r&&(typeof s=="object"&&typeof r=="object"?(s=vc(s),this.mergeContents(s,r)):s=r),i[n]=s}return new mn(i,this.keys,this.overrides,void 0,this.logService)}mergeContents(e,t){for(const i of Object.keys(t)){if(i in e&&on(e[i])&&on(t[i])){this.mergeContents(e[i],t[i]);continue}e[i]=vc(t[i])}}getContentsForOverrideIdentifer(e){let t=null,i=null;const n=s=>{s&&(i?this.mergeContents(i,s):i=vc(s))};for(const s of this.overrides)s.identifiers.length===1&&s.identifiers[0]===e?t=s.contents:s.identifiers.includes(e)&&n(s.contents);return n(t),i}toJSON(){return{contents:this.contents,overrides:this.overrides,keys:this.keys}}setValue(e,t){this.updateValue(e,t,!1)}removeValue(e){const t=this.keys.indexOf(e);t!==-1&&(this.keys.splice(t,1),gne(this.contents,e),Vh.test(e)&&this.overrides.splice(this.overrides.findIndex(i=>ii(i.identifiers,Yy(e))),1))}updateValue(e,t,i){if(YU(this.contents,e,t,n=>this.logService.error(n)),i=i||this.keys.indexOf(e)===-1,i&&this.keys.push(e),Vh.test(e)){const n=Yy(e),s={identifiers:n,keys:Object.keys(this.contents[e]),contents:BM(this.contents[e],a=>this.logService.error(a))},r=this.overrides.findIndex(a=>ii(a.identifiers,n));r!==-1?this.overrides[r]=s:this.overrides.push(s)}}}class lce{constructor(e,t){this._name=e,this.logService=t,this._raw=null,this._configurationModel=null,this._restrictedConfigurations=[]}get configurationModel(){return this._configurationModel||mn.createEmptyModel(this.logService)}parseRaw(e,t){this._raw=e;const{contents:i,keys:n,overrides:s,restricted:r,hasExcludedProperties:a}=this.doParseRaw(e,t);this._configurationModel=new mn(i,n,s,a?[e]:void 0,this.logService),this._restrictedConfigurations=r||[]}doParseRaw(e,t){const i=ci.as(ta.Configuration).getConfigurationProperties(),n=this.filter(e,i,!0,t);e=n.raw;const s=BM(e,l=>this.logService.error(`Conflict in settings file ${this._name}: ${l}`)),r=Object.keys(e),a=this.toOverrides(e,l=>this.logService.error(`Conflict in settings file ${this._name}: ${l}`));return{contents:s,keys:r,overrides:a,restricted:n.restricted,hasExcludedProperties:n.hasExcludedProperties}}filter(e,t,i,n){var l,c,d;let s=!1;if(!(n!=null&&n.scopes)&&!(n!=null&&n.skipRestricted)&&!((l=n==null?void 0:n.exclude)!=null&&l.length))return{raw:e,restricted:[],hasExcludedProperties:s};const r={},a=[];for(const h in e)if(Vh.test(h)&&i){const u=this.filter(e[h],t,!1,n);r[h]=u.raw,s=s||u.hasExcludedProperties,a.push(...u.restricted)}else{const u=t[h],g=u?typeof u.scope<"u"?u.scope:3:void 0;u!=null&&u.restricted&&a.push(h),!((c=n.exclude)!=null&&c.includes(h))&&((d=n.include)!=null&&d.includes(h)||(g===void 0||n.scopes===void 0||n.scopes.includes(g))&&!(n.skipRestricted&&(u!=null&&u.restricted)))?r[h]=e[h]:s=!0}return{raw:r,restricted:a,hasExcludedProperties:s}}toOverrides(e,t){const i=[];for(const n of Object.keys(e))if(Vh.test(n)){const s={};for(const r in e[n])s[r]=e[n][r];i.push({identifiers:Yy(n),keys:Object.keys(s),contents:BM(s,t)})}return i}}class cce{constructor(e,t,i,n,s,r,a,l,c,d,h,u,g){this.key=e,this.overrides=t,this._value=i,this.overrideIdentifiers=n,this.defaultConfiguration=s,this.policyConfiguration=r,this.applicationConfiguration=a,this.userConfiguration=l,this.localUserConfiguration=c,this.remoteUserConfiguration=d,this.workspaceConfiguration=h,this.folderConfigurationModel=u,this.memoryConfigurationModel=g}toInspectValue(e){return(e==null?void 0:e.value)!==void 0||(e==null?void 0:e.override)!==void 0||(e==null?void 0:e.overrides)!==void 0?e:void 0}get userInspectValue(){return this._userInspectValue||(this._userInspectValue=this.userConfiguration.inspect(this.key,this.overrides.overrideIdentifier)),this._userInspectValue}get user(){return this.toInspectValue(this.userInspectValue)}}class QI{constructor(e,t,i,n,s,r,a,l,c,d){this._defaultConfiguration=e,this._policyConfiguration=t,this._applicationConfiguration=i,this._localUserConfiguration=n,this._remoteUserConfiguration=s,this._workspaceConfiguration=r,this._folderConfigurations=a,this._memoryConfiguration=l,this._memoryConfigurationByResource=c,this.logService=d,this._workspaceConsolidatedConfiguration=null,this._foldersConsolidatedConfigurations=new an,this._userConfiguration=null}getValue(e,t,i){return this.getConsolidatedConfigurationModel(e,t,i).getValue(e)}updateValue(e,t,i={}){let n;i.resource?(n=this._memoryConfigurationByResource.get(i.resource),n||(n=mn.createEmptyModel(this.logService),this._memoryConfigurationByResource.set(i.resource,n))):n=this._memoryConfiguration,t===void 0?n.removeValue(e):n.setValue(e,t),i.resource||(this._workspaceConsolidatedConfiguration=null)}inspect(e,t,i){const n=this.getConsolidatedConfigurationModel(e,t,i),s=this.getFolderConfigurationModelForResource(t.resource,i),r=t.resource?this._memoryConfigurationByResource.get(t.resource)||this._memoryConfiguration:this._memoryConfiguration,a=new Set;for(const l of n.overrides)for(const c of l.identifiers)n.getOverrideValue(e,c)!==void 0&&a.add(c);return new cce(e,t,n.getValue(e),a.size?[...a]:void 0,this._defaultConfiguration,this._policyConfiguration.isEmpty()?void 0:this._policyConfiguration,this.applicationConfiguration.isEmpty()?void 0:this.applicationConfiguration,this.userConfiguration,this.localUserConfiguration,this.remoteUserConfiguration,i?this._workspaceConfiguration:void 0,s||void 0,r)}get applicationConfiguration(){return this._applicationConfiguration}get userConfiguration(){return this._userConfiguration||(this._userConfiguration=this._remoteUserConfiguration.isEmpty()?this._localUserConfiguration:this._localUserConfiguration.merge(this._remoteUserConfiguration)),this._userConfiguration}get localUserConfiguration(){return this._localUserConfiguration}get remoteUserConfiguration(){return this._remoteUserConfiguration}getConsolidatedConfigurationModel(e,t,i){let n=this.getConsolidatedConfigurationModelForResource(t,i);return t.overrideIdentifier&&(n=n.override(t.overrideIdentifier)),!this._policyConfiguration.isEmpty()&&this._policyConfiguration.getValue(e)!==void 0&&(n=n.merge(this._policyConfiguration)),n}getConsolidatedConfigurationModelForResource({resource:e},t){let i=this.getWorkspaceConsolidatedConfiguration();if(t&&e){const n=t.getFolder(e);n&&(i=this.getFolderConsolidatedConfiguration(n.uri)||i);const s=this._memoryConfigurationByResource.get(e);s&&(i=i.merge(s))}return i}getWorkspaceConsolidatedConfiguration(){return this._workspaceConsolidatedConfiguration||(this._workspaceConsolidatedConfiguration=this._defaultConfiguration.merge(this.applicationConfiguration,this.userConfiguration,this._workspaceConfiguration,this._memoryConfiguration)),this._workspaceConsolidatedConfiguration}getFolderConsolidatedConfiguration(e){let t=this._foldersConsolidatedConfigurations.get(e);if(!t){const i=this.getWorkspaceConsolidatedConfiguration(),n=this._folderConfigurations.get(e);n?(t=i.merge(n),this._foldersConsolidatedConfigurations.set(e,t)):t=i}return t}getFolderConfigurationModelForResource(e,t){if(t&&e){const i=t.getFolder(e);if(i)return this._folderConfigurations.get(i.uri)}}toData(){return{defaults:{contents:this._defaultConfiguration.contents,overrides:this._defaultConfiguration.overrides,keys:this._defaultConfiguration.keys},policy:{contents:this._policyConfiguration.contents,overrides:this._policyConfiguration.overrides,keys:this._policyConfiguration.keys},application:{contents:this.applicationConfiguration.contents,overrides:this.applicationConfiguration.overrides,keys:this.applicationConfiguration.keys},user:{contents:this.userConfiguration.contents,overrides:this.userConfiguration.overrides,keys:this.userConfiguration.keys},workspace:{contents:this._workspaceConfiguration.contents,overrides:this._workspaceConfiguration.overrides,keys:this._workspaceConfiguration.keys},folders:[...this._folderConfigurations.keys()].reduce((e,t)=>{const{contents:i,overrides:n,keys:s}=this._folderConfigurations.get(t);return e.push([t,{contents:i,overrides:n,keys:s}]),e},[])}}static parse(e,t){const i=this.parseConfigurationModel(e.defaults,t),n=this.parseConfigurationModel(e.policy,t),s=this.parseConfigurationModel(e.application,t),r=this.parseConfigurationModel(e.user,t),a=this.parseConfigurationModel(e.workspace,t),l=e.folders.reduce((c,d)=>(c.set(Se.revive(d[0]),this.parseConfigurationModel(d[1],t)),c),new an);return new QI(i,n,s,r,mn.createEmptyModel(t),a,l,mn.createEmptyModel(t),new an,t)}static parseConfigurationModel(e,t){return new mn(e.contents,e.keys,e.overrides,void 0,t)}}class dce{constructor(e,t,i,n,s){this.change=e,this.previous=t,this.currentConfiguraiton=i,this.currentWorkspace=n,this.logService=s,this._marker=`
`,this._markerCode1=this._marker.charCodeAt(0),this._markerCode2=46,this.affectedKeys=new Set,this._previousConfiguration=void 0;for(const r of e.keys)this.affectedKeys.add(r);for(const[,r]of e.overrides)for(const a of r)this.affectedKeys.add(a);this._affectsConfigStr=this._marker;for(const r of this.affectedKeys)this._affectsConfigStr+=r+this._marker}get previousConfiguration(){return!this._previousConfiguration&&this.previous&&(this._previousConfiguration=QI.parse(this.previous.data,this.logService)),this._previousConfiguration}affectsConfiguration(e,t){var a;const i=this._marker+e,n=this._affectsConfigStr.indexOf(i);if(n<0)return!1;const s=n+i.length;if(s>=this._affectsConfigStr.length)return!1;const r=this._affectsConfigStr.charCodeAt(s);if(r!==this._markerCode1&&r!==this._markerCode2)return!1;if(t){const l=this.previousConfiguration?this.previousConfiguration.getValue(e,t,(a=this.previous)==null?void 0:a.workspace):void 0,c=this.currentConfiguraiton.getValue(e,t,this.currentWorkspace);return!oo(l,c)}return!0}}class hce{constructor(){this._onDidChange=new P,this.onDidChange=this._onDidChange.event,this._enabled=!0}get enabled(){return this._enabled}enable(){this._enabled=!0,this._onDidChange.fire()}disable(){this._enabled=!1,this._onDidChange.fire()}}const YC=new hce,_L={kind:0},uce={kind:1};function gce(o,e,t){return{kind:2,commandId:o,commandArgs:e,isBubble:t}}class XC{constructor(e,t,i){var n;this._log=i,this._defaultKeybindings=e,this._defaultBoundCommands=new Map;for(const s of e){const r=s.command;r&&r.charAt(0)!=="-"&&this._defaultBoundCommands.set(r,!0)}this._map=new Map,this._lookupMap=new Map,this._keybindings=XC.handleRemovals([].concat(e).concat(t));for(let s=0,r=this._keybindings.length;s<r;s++){const a=this._keybindings[s];if(a.chords.length===0)continue;const l=(n=a.when)==null?void 0:n.substituteConstants();l&&l.type===0||this._addKeyPress(a.chords[0],a)}}static _isTargetedForRemoval(e,t,i){if(t){for(let n=0;n<t.length;n++)if(t[n]!==e.chords[n])return!1}return!(i&&i.type!==1&&(!e.when||!xie(i,e.when)))}static handleRemovals(e){const t=new Map;for(let n=0,s=e.length;n<s;n++){const r=e[n];if(r.command&&r.command.charAt(0)==="-"){const a=r.command.substring(1);t.has(a)?t.get(a).push(r):t.set(a,[r])}}if(t.size===0)return e;const i=[];for(let n=0,s=e.length;n<s;n++){const r=e[n];if(!r.command||r.command.length===0){i.push(r);continue}if(r.command.charAt(0)==="-")continue;const a=t.get(r.command);if(!a||!r.isDefault){i.push(r);continue}let l=!1;for(const c of a){const d=c.when;if(this._isTargetedForRemoval(r,c.chords,d)){l=!0;break}}if(!l){i.push(r);continue}}return i}_addKeyPress(e,t){const i=this._map.get(e);if(typeof i>"u"){this._map.set(e,[t]),this._addToLookupMap(t);return}for(let n=i.length-1;n>=0;n--){const s=i[n];if(s.command===t.command)continue;let r=!0;for(let a=1;a<s.chords.length&&a<t.chords.length;a++)if(s.chords[a]!==t.chords[a]){r=!1;break}r&&XC.whenIsEntirelyIncluded(s.when,t.when)&&this._removeFromLookupMap(s)}i.push(t),this._addToLookupMap(t)}_addToLookupMap(e){if(!e.command)return;let t=this._lookupMap.get(e.command);typeof t>"u"?(t=[e],this._lookupMap.set(e.command,t)):t.push(e)}_removeFromLookupMap(e){if(!e.command)return;const t=this._lookupMap.get(e.command);if(!(typeof t>"u")){for(let i=0,n=t.length;i<n;i++)if(t[i]===e){t.splice(i,1);return}}}static whenIsEntirelyIncluded(e,t){return!t||t.type===1?!0:!e||e.type===1?!1:PM(e,t)}getKeybindings(){return this._keybindings}lookupPrimaryKeybinding(e,t){const i=this._lookupMap.get(e);if(typeof i>"u"||i.length===0)return null;if(i.length===1)return i[0];for(let n=i.length-1;n>=0;n--){const s=i[n];if(t.contextMatchesRules(s.when))return s}return i[i.length-1]}resolve(e,t,i){const n=[...t,i];this._log(`| Resolving ${n}`);const s=this._map.get(n[0]);if(s===void 0)return this._log("\\ No keybinding entries."),_L;let r=null;if(n.length<2)r=s;else{r=[];for(let l=0,c=s.length;l<c;l++){const d=s[l];if(n.length>d.chords.length)continue;let h=!0;for(let u=1;u<n.length;u++)if(d.chords[u]!==n[u]){h=!1;break}h&&r.push(d)}}const a=this._findCommand(e,r);return a?n.length<a.chords.length?(this._log(`\\ From ${r.length} keybinding entries, awaiting ${a.chords.length-n.length} more chord(s), when: ${Y9(a.when)}, source: ${X9(a)}.`),uce):(this._log(`\\ From ${r.length} keybinding entries, matched ${a.command}, when: ${Y9(a.when)}, source: ${X9(a)}.`),gce(a.command,a.commandArgs,a.bubble)):(this._log(`\\ From ${r.length} keybinding entries, no when clauses matched the context.`),_L)}_findCommand(e,t){for(let i=t.length-1;i>=0;i--){const n=t[i];if(XC._contextMatchesRules(e,n.when))return n}return null}static _contextMatchesRules(e,t){return t?t.evaluate(e):!0}}function Y9(o){return o?`${o.serialize()}`:"no when condition"}function X9(o){return o.extensionId?o.isBuiltinExtension?`built-in extension ${o.extensionId}`:`user extension ${o.extensionId}`:o.isDefault?"built-in":"user"}const fce=/^(cursor|delete|undo|redo|tab|editor\.action\.clipboard)/;class mce extends F{get onDidUpdateKeybindings(){return this._onDidUpdateKeybindings?this._onDidUpdateKeybindings.event:oe.None}get inChordMode(){return this._currentChords.length>0}constructor(e,t,i,n,s){super(),this._contextKeyService=e,this._commandService=t,this._telemetryService=i,this._notificationService=n,this._logService=s,this._onDidUpdateKeybindings=this._register(new P),this._currentChords=[],this._currentChordChecker=new s5,this._currentChordStatusMessage=null,this._ignoreSingleModifiers=rp.EMPTY,this._currentSingleModifier=null,this._currentSingleModifierClearTimeout=new ea,this._currentlyDispatchingCommandId=null,this._logging=!1}dispose(){super.dispose()}_log(e){this._logging&&this._logService.info(`[KeybindingService]: ${e}`)}getKeybindings(){return this._getResolver().getKeybindings()}lookupKeybinding(e,t){const i=this._getResolver().lookupPrimaryKeybinding(e,t||this._contextKeyService);if(i)return i.resolvedKeybinding}dispatchEvent(e,t){return this._dispatch(e,t)}softDispatch(e,t){this._log("/ Soft dispatching keyboard event");const i=this.resolveKeyboardEvent(e);if(i.hasMultipleChords())return console.warn("keyboard event should not be mapped to multiple chords"),_L;const[n]=i.getDispatchChords();if(n===null)return this._log("\\ Keyboard event cannot be dispatched"),_L;const s=this._contextKeyService.getContext(t),r=this._currentChords.map(({keypress:a})=>a);return this._getResolver().resolve(s,r,n)}_scheduleLeaveChordMode(){const e=Date.now();this._currentChordChecker.cancelAndSet(()=>{if(!this._documentHasFocus()){this._leaveChordMode();return}Date.now()-e>5e3&&this._leaveChordMode()},500)}_expectAnotherChord(e,t){switch(this._currentChords.push({keypress:e,label:t}),this._currentChords.length){case 0:throw U4("impossible");case 1:this._currentChordStatusMessage=this._notificationService.status(m("first.chord","({0}) was pressed. Waiting for second key of chord...",t));break;default:{const i=this._currentChords.map(({label:n})=>n).join(", ");this._currentChordStatusMessage=this._notificationService.status(m("next.chord","({0}) was pressed. Waiting for next key of chord...",i))}}this._scheduleLeaveChordMode(),YC.enabled&&YC.disable()}_leaveChordMode(){this._currentChordStatusMessage&&(this._currentChordStatusMessage.dispose(),this._currentChordStatusMessage=null),this._currentChordChecker.cancel(),this._currentChords=[],YC.enable()}_dispatch(e,t){return this._doDispatch(this.resolveKeyboardEvent(e),t,!1)}_singleModifierDispatch(e,t){const i=this.resolveKeyboardEvent(e),[n]=i.getSingleModifierDispatchChords();if(n)return this._ignoreSingleModifiers.has(n)?(this._log(`+ Ignoring single modifier ${n} due to it being pressed together with other keys.`),this._ignoreSingleModifiers=rp.EMPTY,this._currentSingleModifierClearTimeout.cancel(),this._currentSingleModifier=null,!1):(this._ignoreSingleModifiers=rp.EMPTY,this._currentSingleModifier===null?(this._log(`+ Storing single modifier for possible chord ${n}.`),this._currentSingleModifier=n,this._currentSingleModifierClearTimeout.cancelAndSet(()=>{this._log("+ Clearing single modifier due to 300ms elapsed."),this._currentSingleModifier=null},300),!1):n===this._currentSingleModifier?(this._log(`/ Dispatching single modifier chord ${n} ${n}`),this._currentSingleModifierClearTimeout.cancel(),this._currentSingleModifier=null,this._doDispatch(i,t,!0)):(this._log(`+ Clearing single modifier due to modifier mismatch: ${this._currentSingleModifier} ${n}`),this._currentSingleModifierClearTimeout.cancel(),this._currentSingleModifier=null,!1));const[s]=i.getChords();return this._ignoreSingleModifiers=new rp(s),this._currentSingleModifier!==null&&this._log("+ Clearing single modifier due to other key up."),this._currentSingleModifierClearTimeout.cancel(),this._currentSingleModifier=null,!1}_doDispatch(e,t,i=!1){let n=!1;if(e.hasMultipleChords())return console.warn("Unexpected keyboard event mapped to multiple chords"),!1;let s=null,r=null;if(i){const[d]=e.getSingleModifierDispatchChords();s=d,r=d?[d]:[]}else[s]=e.getDispatchChords(),r=this._currentChords.map(({keypress:d})=>d);if(s===null)return this._log("\\ Keyboard event cannot be dispatched in keydown phase."),n;const a=this._contextKeyService.getContext(t),l=e.getLabel(),c=this._getResolver().resolve(a,r,s);switch(c.kind){case 0:{if(this._logService.trace("KeybindingService#dispatch",l,"[ No matching keybinding ]"),this.inChordMode){const d=this._currentChords.map(({label:h})=>h).join(", ");this._log(`+ Leaving multi-chord mode: Nothing bound to "${d}, ${l}".`),this._notificationService.status(m("missing.chord","The key combination ({0}, {1}) is not a command.",d,l),{hideAfter:10*1e3}),this._leaveChordMode(),n=!0}return n}case 1:return this._logService.trace("KeybindingService#dispatch",l,"[ Several keybindings match - more chords needed ]"),n=!0,this._expectAnotherChord(s,l),this._log(this._currentChords.length===1?"+ Entering multi-chord mode...":"+ Continuing multi-chord mode..."),n;case 2:{if(this._logService.trace("KeybindingService#dispatch",l,`[ Will dispatch command ${c.commandId} ]`),c.commandId===null||c.commandId===""){if(this.inChordMode){const d=this._currentChords.map(({label:h})=>h).join(", ");this._log(`+ Leaving chord mode: Nothing bound to "${d}, ${l}".`),this._notificationService.status(m("missing.chord","The key combination ({0}, {1}) is not a command.",d,l),{hideAfter:10*1e3}),this._leaveChordMode(),n=!0}}else{this.inChordMode&&this._leaveChordMode(),c.isBubble||(n=!0),this._log(`+ Invoking command ${c.commandId}.`),this._currentlyDispatchingCommandId=c.commandId;try{typeof c.commandArgs>"u"?this._commandService.executeCommand(c.commandId).then(void 0,d=>this._notificationService.warn(d)):this._commandService.executeCommand(c.commandId,c.commandArgs).then(void 0,d=>this._notificationService.warn(d))}finally{this._currentlyDispatchingCommandId=null}fce.test(c.commandId)||this._telemetryService.publicLog2("workbenchActionExecuted",{id:c.commandId,from:"keybinding",detail:e.getUserSettingsLabel()??void 0})}return n}}}mightProducePrintableCharacter(e){return e.ctrlKey||e.metaKey?!1:e.keyCode>=31&&e.keyCode<=56||e.keyCode>=21&&e.keyCode<=30}}const Vk=class Vk{constructor(e){this._ctrlKey=e?e.ctrlKey:!1,this._shiftKey=e?e.shiftKey:!1,this._altKey=e?e.altKey:!1,this._metaKey=e?e.metaKey:!1}has(e){switch(e){case"ctrl":return this._ctrlKey;case"shift":return this._shiftKey;case"alt":return this._altKey;case"meta":return this._metaKey}}};Vk.EMPTY=new Vk(null);let rp=Vk;class Q9{constructor(e,t,i,n,s,r,a){this._resolvedKeybindingItemBrand=void 0,this.resolvedKeybinding=e,this.chords=e?N2(e.getDispatchChords()):[],e&&this.chords.length===0&&(this.chords=N2(e.getSingleModifierDispatchChords())),this.bubble=t?t.charCodeAt(0)===94:!1,this.command=this.bubble?t.substr(1):t,this.commandArgs=i,this.when=n,this.isDefault=s,this.extensionId=r,this.isBuiltinExtension=a}}function N2(o){const e=[];for(let t=0,i=o.length;t<i;t++){const n=o[t];if(!n)return[];e.push(n)}return e}class JI{constructor(e,t,i=t){this.modifierLabels=[null],this.modifierLabels[2]=e,this.modifierLabels[1]=t,this.modifierLabels[3]=i}toLabel(e,t,i){if(t.length===0)return null;const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s],l=i(a);if(l===null)return null;n[s]=Cce(a,l,this.modifierLabels[e])}return n.join(" ")}}const Q5=new JI({ctrlKey:"⌃",shiftKey:"⇧",altKey:"⌥",metaKey:"⌘",separator:""},{ctrlKey:m({key:"ctrlKey",comment:["This is the short form for the Control key on the keyboard"]},"Ctrl"),shiftKey:m({key:"shiftKey",comment:["This is the short form for the Shift key on the keyboard"]},"Shift"),altKey:m({key:"altKey",comment:["This is the short form for the Alt key on the keyboard"]},"Alt"),metaKey:m({key:"windowsKey",comment:["This is the short form for the Windows key on the keyboard"]},"Windows"),separator:"+"},{ctrlKey:m({key:"ctrlKey",comment:["This is the short form for the Control key on the keyboard"]},"Ctrl"),shiftKey:m({key:"shiftKey",comment:["This is the short form for the Shift key on the keyboard"]},"Shift"),altKey:m({key:"altKey",comment:["This is the short form for the Alt key on the keyboard"]},"Alt"),metaKey:m({key:"superKey",comment:["This is the short form for the Super key on the keyboard"]},"Super"),separator:"+"}),pce=new JI({ctrlKey:m({key:"ctrlKey.long",comment:["This is the long form for the Control key on the keyboard"]},"Control"),shiftKey:m({key:"shiftKey.long",comment:["This is the long form for the Shift key on the keyboard"]},"Shift"),altKey:m({key:"optKey.long",comment:["This is the long form for the Alt/Option key on the keyboard"]},"Option"),metaKey:m({key:"cmdKey.long",comment:["This is the long form for the Command key on the keyboard"]},"Command"),separator:"+"},{ctrlKey:m({key:"ctrlKey.long",comment:["This is the long form for the Control key on the keyboard"]},"Control"),shiftKey:m({key:"shiftKey.long",comment:["This is the long form for the Shift key on the keyboard"]},"Shift"),altKey:m({key:"altKey.long",comment:["This is the long form for the Alt key on the keyboard"]},"Alt"),metaKey:m({key:"windowsKey.long",comment:["This is the long form for the Windows key on the keyboard"]},"Windows"),separator:"+"},{ctrlKey:m({key:"ctrlKey.long",comment:["This is the long form for the Control key on the keyboard"]},"Control"),shiftKey:m({key:"shiftKey.long",comment:["This is the long form for the Shift key on the keyboard"]},"Shift"),altKey:m({key:"altKey.long",comment:["This is the long form for the Alt key on the keyboard"]},"Alt"),metaKey:m({key:"superKey.long",comment:["This is the long form for the Super key on the keyboard"]},"Super"),separator:"+"}),_ce=new JI({ctrlKey:"Ctrl",shiftKey:"Shift",altKey:"Alt",metaKey:"Cmd",separator:"+"},{ctrlKey:"Ctrl",shiftKey:"Shift",altKey:"Alt",metaKey:"Super",separator:"+"}),bce=new JI({ctrlKey:"ctrl",shiftKey:"shift",altKey:"alt",metaKey:"cmd",separator:"+"},{ctrlKey:"ctrl",shiftKey:"shift",altKey:"alt",metaKey:"win",separator:"+"},{ctrlKey:"ctrl",shiftKey:"shift",altKey:"alt",metaKey:"meta",separator:"+"});function Cce(o,e,t){if(e===null)return"";const i=[];return o.ctrlKey&&i.push(t.ctrlKey),o.shiftKey&&i.push(t.shiftKey),o.altKey&&i.push(t.altKey),o.metaKey&&i.push(t.metaKey),e!==""&&i.push(e),i.join(t.separator)}class vce extends Kee{constructor(e,t){if(super(),t.length===0)throw gr("chords");this._os=e,this._chords=t}getLabel(){return Q5.toLabel(this._os,this._chords,e=>this._getLabel(e))}getAriaLabel(){return pce.toLabel(this._os,this._chords,e=>this._getAriaLabel(e))}getElectronAccelerator(){return this._chords.length>1||this._chords[0].isDuplicateModifierCase()?null:_ce.toLabel(this._os,this._chords,e=>this._getElectronAccelerator(e))}getUserSettingsLabel(){return bce.toLabel(this._os,this._chords,e=>this._getUserSettingsLabel(e))}hasMultipleChords(){return this._chords.length>1}getChords(){return this._chords.map(e=>this._getChord(e))}_getChord(e){return new jee(e.ctrlKey,e.shiftKey,e.altKey,e.metaKey,this._getLabel(e),this._getAriaLabel(e))}getDispatchChords(){return this._chords.map(e=>this._getChordDispatch(e))}getSingleModifierDispatchChords(){return this._chords.map(e=>this._getSingleModifierChordDispatch(e))}}class xw extends vce{constructor(e,t){super(t,e)}_keyCodeToUILabel(e){if(this._os===2)switch(e){case 15:return"←";case 16:return"↑";case 17:return"→";case 18:return"↓"}return xc.toString(e)}_getLabel(e){return e.isDuplicateModifierCase()?"":this._keyCodeToUILabel(e.keyCode)}_getAriaLabel(e){return e.isDuplicateModifierCase()?"":xc.toString(e.keyCode)}_getElectronAccelerator(e){return xc.toElectronAccelerator(e.keyCode)}_getUserSettingsLabel(e){if(e.isDuplicateModifierCase())return"";const t=xc.toUserSettingsUS(e.keyCode);return t&&t.toLowerCase()}_getChordDispatch(e){return xw.getDispatchStr(e)}static getDispatchStr(e){if(e.isModifierKey())return null;let t="";return e.ctrlKey&&(t+="ctrl+"),e.shiftKey&&(t+="shift+"),e.altKey&&(t+="alt+"),e.metaKey&&(t+="meta+"),t+=xc.toString(e.keyCode),t}_getSingleModifierChordDispatch(e){return e.keyCode===5&&!e.shiftKey&&!e.altKey&&!e.metaKey?"ctrl":e.keyCode===4&&!e.ctrlKey&&!e.altKey&&!e.metaKey?"shift":e.keyCode===6&&!e.ctrlKey&&!e.shiftKey&&!e.metaKey?"alt":e.keyCode===57&&!e.ctrlKey&&!e.shiftKey&&!e.altKey?"meta":null}static _scanCodeToKeyCode(e){const t=q4[e];if(t!==-1)return t;switch(e){case 10:return 31;case 11:return 32;case 12:return 33;case 13:return 34;case 14:return 35;case 15:return 36;case 16:return 37;case 17:return 38;case 18:return 39;case 19:return 40;case 20:return 41;case 21:return 42;case 22:return 43;case 23:return 44;case 24:return 45;case 25:return 46;case 26:return 47;case 27:return 48;case 28:return 49;case 29:return 50;case 30:return 51;case 31:return 52;case 32:return 53;case 33:return 54;case 34:return 55;case 35:return 56;case 36:return 22;case 37:return 23;case 38:return 24;case 39:return 25;case 40:return 26;case 41:return 27;case 42:return 28;case 43:return 29;case 44:return 30;case 45:return 21;case 51:return 88;case 52:return 86;case 53:return 92;case 54:return 94;case 55:return 93;case 56:return 0;case 57:return 85;case 58:return 95;case 59:return 91;case 60:return 87;case 61:return 89;case 62:return 90;case 106:return 97}return 0}static _toKeyCodeChord(e){if(!e)return null;if(e instanceof Jc)return e;const t=this._scanCodeToKeyCode(e.scanCode);return t===0?null:new Jc(e.ctrlKey,e.shiftKey,e.altKey,e.metaKey,t)}static resolveKeybinding(e,t){const i=N2(e.chords.map(n=>this._toKeyCodeChord(n)));return i.length>0?[new xw(i,t)]:[]}}const W_=et("labelService"),mj=et("progressService"),yB=class yB{constructor(e){this.callback=e}report(e){this._value=e,this.callback(this._value)}};yB.None=Object.freeze({report(){}});let ka=yB;const nu=et("editorProgressService");class wce{constructor(){this._value="",this._pos=0}reset(e){return this._value=e,this._pos=0,this}next(){return this._pos+=1,this}hasNext(){return this._pos<this._value.length-1}cmp(e){const t=e.charCodeAt(0),i=this._value.charCodeAt(this._pos);return t-i}value(){return this._value[this._pos]}}class Sce{constructor(e=!0){this._caseSensitive=e}reset(e){return this._value=e,this._from=0,this._to=0,this.next()}hasNext(){return this._to<this._value.length}next(){this._from=this._to;let e=!0;for(;this._to<this._value.length;this._to++)if(this._value.charCodeAt(this._to)===46)if(e)this._from++;else break;else e=!1;return this}cmp(e){return this._caseSensitive?Z4(e,this._value,0,e.length,this._from,this._to):A1(e,this._value,0,e.length,this._from,this._to)}value(){return this._value.substring(this._from,this._to)}}class yce{constructor(e=!0,t=!0){this._splitOnBackslash=e,this._caseSensitive=t}reset(e){this._from=0,this._to=0,this._value=e,this._valueLen=e.length;for(let t=e.length-1;t>=0;t--,this._valueLen--){const i=this._value.charCodeAt(t);if(!(i===47||this._splitOnBackslash&&i===92))break}return this.next()}hasNext(){return this._to<this._valueLen}next(){this._from=this._to;let e=!0;for(;this._to<this._valueLen;this._to++){const t=this._value.charCodeAt(this._to);if(t===47||this._splitOnBackslash&&t===92)if(e)this._from++;else break;else e=!1}return this}cmp(e){return this._caseSensitive?Z4(e,this._value,0,e.length,this._from,this._to):A1(e,this._value,0,e.length,this._from,this._to)}value(){return this._value.substring(this._from,this._to)}}class Lce{constructor(e,t){this._ignorePathCasing=e,this._ignoreQueryAndFragment=t,this._states=[],this._stateIdx=0}reset(e){return this._value=e,this._states=[],this._value.scheme&&this._states.push(1),this._value.authority&&this._states.push(2),this._value.path&&(this._pathIterator=new yce(!1,!this._ignorePathCasing(e)),this._pathIterator.reset(e.path),this._pathIterator.value()&&this._states.push(3)),this._ignoreQueryAndFragment(e)||(this._value.query&&this._states.push(4),this._value.fragment&&this._states.push(5)),this._stateIdx=0,this}next(){return this._states[this._stateIdx]===3&&this._pathIterator.hasNext()?this._pathIterator.next():this._stateIdx+=1,this}hasNext(){return this._states[this._stateIdx]===3&&this._pathIterator.hasNext()||this._stateIdx<this._states.length-1}cmp(e){if(this._states[this._stateIdx]===1)return CM(e,this._value.scheme);if(this._states[this._stateIdx]===2)return CM(e,this._value.authority);if(this._states[this._stateIdx]===3)return this._pathIterator.cmp(e);if(this._states[this._stateIdx]===4)return sw(e,this._value.query);if(this._states[this._stateIdx]===5)return sw(e,this._value.fragment);throw new Error}value(){if(this._states[this._stateIdx]===1)return this._value.scheme;if(this._states[this._stateIdx]===2)return this._value.authority;if(this._states[this._stateIdx]===3)return this._pathIterator.value();if(this._states[this._stateIdx]===4)return this._value.query;if(this._states[this._stateIdx]===5)return this._value.fragment;throw new Error}}class jS{constructor(){this.height=1}rotateLeft(){const e=this.right;return this.right=e.left,e.left=this,this.updateHeight(),e.updateHeight(),e}rotateRight(){const e=this.left;return this.left=e.right,e.right=this,this.updateHeight(),e.updateHeight(),e}updateHeight(){this.height=1+Math.max(this.heightLeft,this.heightRight)}balanceFactor(){return this.heightRight-this.heightLeft}get heightLeft(){var e;return((e=this.left)==null?void 0:e.height)??0}get heightRight(){var e;return((e=this.right)==null?void 0:e.height)??0}}class Gp{static forUris(e=()=>!1,t=()=>!1){return new Gp(new Lce(e,t))}static forStrings(){return new Gp(new wce)}static forConfigKeys(){return new Gp(new Sce)}constructor(e){this._iter=e}clear(){this._root=void 0}set(e,t){const i=this._iter.reset(e);let n;this._root||(this._root=new jS,this._root.segment=i.value());const s=[];for(n=this._root;;){const a=i.cmp(n.segment);if(a>0)n.left||(n.left=new jS,n.left.segment=i.value()),s.push([-1,n]),n=n.left;else if(a<0)n.right||(n.right=new jS,n.right.segment=i.value()),s.push([1,n]),n=n.right;else if(i.hasNext())i.next(),n.mid||(n.mid=new jS,n.mid.segment=i.value()),s.push([0,n]),n=n.mid;else break}const r=n.value;n.value=t,n.key=e;for(let a=s.length-1;a>=0;a--){const l=s[a][1];l.updateHeight();const c=l.balanceFactor();if(c<-1||c>1){const d=s[a][0],h=s[a+1][0];if(d===1&&h===1)s[a][1]=l.rotateLeft();else if(d===-1&&h===-1)s[a][1]=l.rotateRight();else if(d===1&&h===-1)l.right=s[a+1][1]=s[a+1][1].rotateRight(),s[a][1]=l.rotateLeft();else if(d===-1&&h===1)l.left=s[a+1][1]=s[a+1][1].rotateLeft(),s[a][1]=l.rotateRight();else throw new Error;if(a>0)switch(s[a-1][0]){case-1:s[a-1][1].left=s[a][1];break;case 1:s[a-1][1].right=s[a][1];break;case 0:s[a-1][1].mid=s[a][1];break}else this._root=s[0][1]}}return r}get(e){var t;return(t=this._getNode(e))==null?void 0:t.value}_getNode(e){const t=this._iter.reset(e);let i=this._root;for(;i;){const n=t.cmp(i.segment);if(n>0)i=i.left;else if(n<0)i=i.right;else if(t.hasNext())t.next(),i=i.mid;else break}return i}has(e){const t=this._getNode(e);return!((t==null?void 0:t.value)===void 0&&(t==null?void 0:t.mid)===void 0)}delete(e){return this._delete(e,!1)}deleteSuperstr(e){return this._delete(e,!0)}_delete(e,t){const i=this._iter.reset(e),n=[];let s=this._root;for(;s;){const r=i.cmp(s.segment);if(r>0)n.push([-1,s]),s=s.left;else if(r<0)n.push([1,s]),s=s.right;else if(i.hasNext())i.next(),n.push([0,s]),s=s.mid;else break}if(s){if(t?(s.left=void 0,s.mid=void 0,s.right=void 0,s.height=1):(s.key=void 0,s.value=void 0),!s.mid&&!s.value)if(s.left&&s.right){const r=this._min(s.right);if(r.key){const{key:a,value:l,segment:c}=r;this._delete(r.key,!1),s.key=a,s.value=l,s.segment=c}}else{const r=s.left??s.right;if(n.length>0){const[a,l]=n[n.length-1];switch(a){case-1:l.left=r;break;case 0:l.mid=r;break;case 1:l.right=r;break}}else this._root=r}for(let r=n.length-1;r>=0;r--){const a=n[r][1];a.updateHeight();const l=a.balanceFactor();if(l>1?(a.right.balanceFactor()>=0||(a.right=a.right.rotateRight()),n[r][1]=a.rotateLeft()):l<-1&&(a.left.balanceFactor()<=0||(a.left=a.left.rotateLeft()),n[r][1]=a.rotateRight()),r>0)switch(n[r-1][0]){case-1:n[r-1][1].left=n[r][1];break;case 1:n[r-1][1].right=n[r][1];break;case 0:n[r-1][1].mid=n[r][1];break}else this._root=n[0][1]}}}_min(e){for(;e.left;)e=e.left;return e}findSubstr(e){const t=this._iter.reset(e);let i=this._root,n;for(;i;){const s=t.cmp(i.segment);if(s>0)i=i.left;else if(s<0)i=i.right;else if(t.hasNext())t.next(),n=i.value||n,i=i.mid;else break}return i&&i.value||n}findSuperstr(e){return this._findSuperstrOrElement(e,!1)}_findSuperstrOrElement(e,t){const i=this._iter.reset(e);let n=this._root;for(;n;){const s=i.cmp(n.segment);if(s>0)n=n.left;else if(s<0)n=n.right;else if(i.hasNext())i.next(),n=n.mid;else return n.mid?this._entries(n.mid):t?n.value:void 0}}forEach(e){for(const[t,i]of this)e(i,t)}*[Symbol.iterator](){yield*this._entries(this._root)}_entries(e){const t=[];return this._dfsEntries(e,t),t[Symbol.iterator]()}_dfsEntries(e,t){e&&(e.left&&this._dfsEntries(e.left,t),e.value&&t.push([e.key,e.value]),e.mid&&this._dfsEntries(e.mid,t),e.right&&this._dfsEntries(e.right,t))}}const Af=et("contextService");function T2(o){const e=o;return typeof(e==null?void 0:e.id)=="string"&&Se.isUri(e.uri)}function xce(o){const e=o;return typeof(e==null?void 0:e.id)=="string"&&!T2(o)&&!Ice(o)}const kce={id:"empty-window"};function Dce(o,e){if(typeof o=="string"||typeof o>"u")return typeof o=="string"?{id:lh(o)}:kce;const t=o;return t.configuration?{id:t.id,configPath:t.configuration}:t.folders.length===1?{id:t.id,uri:t.folders[0].uri}:{id:t.id}}function Ice(o){const e=o;return typeof(e==null?void 0:e.id)=="string"&&Se.isUri(e.configPath)}class Ece{constructor(e,t){this.raw=t,this.uri=e.uri,this.index=e.index,this.name=e.name}toJSON(){return{uri:this.uri,name:this.name,index:this.index}}}const R2="code-workspace";m("codeWorkspace","Code Workspace");const pj="4064f6ec-cb38-4ad0-af64-ee6467e63c82";function Nce(o){return o.id===pj}var M2;(function(o){o.inspectTokensAction=m("inspectTokens","Developer: Inspect Tokens")})(M2||(M2={}));var bL;(function(o){o.gotoLineActionLabel=m("gotoLineActionLabel","Go to Line/Column...")})(bL||(bL={}));var A2;(function(o){o.helpQuickAccessActionLabel=m("helpQuickAccess","Show all Quick Access Providers")})(A2||(A2={}));var CL;(function(o){o.quickCommandActionLabel=m("quickCommandActionLabel","Command Palette"),o.quickCommandHelp=m("quickCommandActionHelp","Show And Run Commands")})(CL||(CL={}));var kw;(function(o){o.quickOutlineActionLabel=m("quickOutlineActionLabel","Go to Symbol..."),o.quickOutlineByCategoryActionLabel=m("quickOutlineByCategoryActionLabel","Go to Symbol by Category...")})(kw||(kw={}));var P2;(function(o){o.editorViewAccessibleLabel=m("editorViewAccessibleLabel","Editor content")})(P2||(P2={}));var O2;(function(o){o.toggleHighContrast=m("toggleHighContrast","Toggle High Contrast Theme")})(O2||(O2={}));var F2;(function(o){o.bulkEditServiceSummary=m("bulkEditServiceSummary","Made {0} edits in {1} files")})(F2||(F2={}));const _j=et("workspaceTrustManagementService");let B_=[],J5=[],bj=[];function KS(o,e=!1){Tce(o,!1,e)}function Tce(o,e,t){const i=Rce(o,e);B_.push(i),i.userConfigured?bj.push(i):J5.push(i),t&&!i.userConfigured&&B_.forEach(n=>{n.mime===i.mime||n.userConfigured||(i.extension&&n.extension===i.extension&&console.warn(`Overwriting extension <<${i.extension}>> to now point to mime <<${i.mime}>>`),i.filename&&n.filename===i.filename&&console.warn(`Overwriting filename <<${i.filename}>> to now point to mime <<${i.mime}>>`),i.filepattern&&n.filepattern===i.filepattern&&console.warn(`Overwriting filepattern <<${i.filepattern}>> to now point to mime <<${i.mime}>>`),i.firstline&&n.firstline===i.firstline&&console.warn(`Overwriting firstline <<${i.firstline}>> to now point to mime <<${i.mime}>>`))})}function Rce(o,e){return{id:o.id,mime:o.mime,filename:o.filename,extension:o.extension,filepattern:o.filepattern,firstline:o.firstline,userConfigured:e,filenameLowercase:o.filename?o.filename.toLowerCase():void 0,extensionLowercase:o.extension?o.extension.toLowerCase():void 0,filepatternLowercase:o.filepattern?k$(o.filepattern.toLowerCase()):void 0,filepatternOnPath:o.filepattern?o.filepattern.indexOf(Ai.sep)>=0:!1}}function Mce(){B_=B_.filter(o=>o.userConfigured),J5=[]}function Ace(o,e){return Pce(o,e).map(t=>t.id)}function Pce(o,e){let t;if(o)switch(o.scheme){case Fe.file:t=o.fsPath;break;case Fe.data:{t=Uh.parseMetaData(o).get(Uh.META_DATA_LABEL);break}case Fe.vscodeNotebookCell:t=void 0;break;default:t=o.path}if(!t)return[{id:"unknown",mime:vi.unknown}];t=t.toLowerCase();const i=lh(t),n=J9(t,i,bj);if(n)return[n,{id:ms,mime:vi.text}];const s=J9(t,i,J5);if(s)return[s,{id:ms,mime:vi.text}];if(e){const r=Oce(e);if(r)return[r,{id:ms,mime:vi.text}]}return[{id:"unknown",mime:vi.unknown}]}function J9(o,e,t){var r;let i,n,s;for(let a=t.length-1;a>=0;a--){const l=t[a];if(e===l.filenameLowercase){i=l;break}if(l.filepattern&&(!n||l.filepattern.length>n.filepattern.length)){const c=l.filepatternOnPath?o:e;(r=l.filepatternLowercase)!=null&&r.call(l,c)&&(n=l)}l.extension&&(!s||l.extension.length>s.extension.length)&&e.endsWith(l.extensionLowercase)&&(s=l)}if(i)return i;if(n)return n;if(s)return s}function Oce(o){if(t5(o)&&(o=o.substr(1)),o.length>0)for(let e=B_.length-1;e>=0;e--){const t=B_[e];if(!t.firstline)continue;const i=o.match(t.firstline);if(i&&i.length>0)return t}}const qS=Object.prototype.hasOwnProperty,e8="vs.editor.nullLanguage";class Fce{constructor(){this._languageIdToLanguage=[],this._languageToLanguageId=new Map,this._register(e8,0),this._register(ms,1),this._nextLanguageId=2}_register(e,t){this._languageIdToLanguage[t]=e,this._languageToLanguageId.set(e,t)}register(e){if(this._languageToLanguageId.has(e))return;const t=this._nextLanguageId++;this._register(e,t)}encodeLanguageId(e){return this._languageToLanguageId.get(e)||0}decodeLanguageId(e){return this._languageIdToLanguage[e]||e8}}const Pv=class Pv extends F{constructor(e=!0,t=!1){super(),this._onDidChange=this._register(new P),this.onDidChange=this._onDidChange.event,Pv.instanceCount++,this._warnOnOverwrite=t,this.languageIdCodec=new Fce,this._dynamicLanguages=[],this._languages={},this._mimeTypesMap={},this._nameMap={},this._lowercaseNameMap={},e&&(this._initializeFromRegistry(),this._register(I_.onDidChangeLanguages(i=>{this._initializeFromRegistry()})))}dispose(){Pv.instanceCount--,super.dispose()}_initializeFromRegistry(){this._languages={},this._mimeTypesMap={},this._nameMap={},this._lowercaseNameMap={},Mce();const e=[].concat(I_.getLanguages()).concat(this._dynamicLanguages);this._registerLanguages(e)}_registerLanguages(e){for(const t of e)this._registerLanguage(t);this._mimeTypesMap={},this._nameMap={},this._lowercaseNameMap={},Object.keys(this._languages).forEach(t=>{const i=this._languages[t];i.name&&(this._nameMap[i.name]=i.identifier),i.aliases.forEach(n=>{this._lowercaseNameMap[n.toLowerCase()]=i.identifier}),i.mimetypes.forEach(n=>{this._mimeTypesMap[n]=i.identifier})}),ci.as(ta.Configuration).registerOverrideIdentifiers(this.getRegisteredLanguageIds()),this._onDidChange.fire()}_registerLanguage(e){const t=e.id;let i;qS.call(this._languages,t)?i=this._languages[t]:(this.languageIdCodec.register(t),i={identifier:t,name:null,mimetypes:[],aliases:[],extensions:[],filenames:[],configurationFiles:[],icons:[]},this._languages[t]=i),this._mergeLanguage(i,e)}_mergeLanguage(e,t){const i=t.id;let n=null;if(Array.isArray(t.mimetypes)&&t.mimetypes.length>0&&(e.mimetypes.push(...t.mimetypes),n=t.mimetypes[0]),n||(n=`text/x-${i}`,e.mimetypes.push(n)),Array.isArray(t.extensions)){t.configuration?e.extensions=t.extensions.concat(e.extensions):e.extensions=e.extensions.concat(t.extensions);for(const a of t.extensions)KS({id:i,mime:n,extension:a},this._warnOnOverwrite)}if(Array.isArray(t.filenames))for(const a of t.filenames)KS({id:i,mime:n,filename:a},this._warnOnOverwrite),e.filenames.push(a);if(Array.isArray(t.filenamePatterns))for(const a of t.filenamePatterns)KS({id:i,mime:n,filepattern:a},this._warnOnOverwrite);if(typeof t.firstLine=="string"&&t.firstLine.length>0){let a=t.firstLine;a.charAt(0)!=="^"&&(a="^"+a);try{const l=new RegExp(a);Ree(l)||KS({id:i,mime:n,firstline:l},this._warnOnOverwrite)}catch(l){console.warn(`[${t.id}]: Invalid regular expression \`${a}\`: `,l)}}e.aliases.push(i);let s=null;if(typeof t.aliases<"u"&&Array.isArray(t.aliases)&&(t.aliases.length===0?s=[null]:s=t.aliases),s!==null)for(const a of s)!a||a.length===0||e.aliases.push(a);const r=s!==null&&s.length>0;if(!(r&&s[0]===null)){const a=(r?s[0]:null)||i;(r||!e.name)&&(e.name=a)}t.configuration&&e.configurationFiles.push(t.configuration),t.icon&&e.icons.push(t.icon)}isRegisteredLanguageId(e){return e?qS.call(this._languages,e):!1}getRegisteredLanguageIds(){return Object.keys(this._languages)}getLanguageIdByLanguageName(e){const t=e.toLowerCase();return qS.call(this._lowercaseNameMap,t)?this._lowercaseNameMap[t]:null}getLanguageIdByMimeType(e){return e&&qS.call(this._mimeTypesMap,e)?this._mimeTypesMap[e]:null}guessLanguageIdByFilepathOrFirstLine(e,t){return!e&&!t?[]:Ace(e,t)}};Pv.instanceCount=0;let W2=Pv;const jo=(o,e)=>o===e;function vL(o=jo){return(e,t)=>ii(e,t,o)}function Wce(){return(o,e)=>o.equals(e)}function B2(o,e,t){if(t!==void 0){const i=o;return i==null||e===void 0||e===null?e===i:t(i,e)}else{const i=o;return(n,s)=>n==null||s===void 0||s===null?s===n:i(n,s)}}function wL(o,e){if(o===e)return!0;if(Array.isArray(o)&&Array.isArray(e)){if(o.length!==e.length)return!1;for(let t=0;t<o.length;t++)if(!wL(o[t],e[t]))return!1;return!0}if(o&&typeof o=="object"&&e&&typeof e=="object"&&Object.getPrototypeOf(o)===Object.prototype&&Object.getPrototypeOf(e)===Object.prototype){const t=o,i=e,n=Object.keys(t),s=Object.keys(i),r=new Set(s);if(n.length!==s.length)return!1;for(const a of n)if(!r.has(a)||!wL(t[a],i[a]))return!1;return!0}return!1}class Wn{constructor(e,t,i){this.owner=e,this.debugNameSource=t,this.referenceFn=i}getDebugName(e){return Bce(e,this)}}const t8=new Map,H2=new WeakMap;function Bce(o,e){const t=H2.get(o);if(t)return t;const i=Hce(o,e);if(i){let n=t8.get(i)??0;n++,t8.set(i,n);const s=n===1?i:`${i}#${n}`;return H2.set(o,s),s}}function Hce(o,e){const t=H2.get(o);if(t)return t;const i=e.owner?zce(e.owner)+".":"";let n;const s=e.debugNameSource;if(s!==void 0)if(typeof s=="function"){if(n=s(),n!==void 0)return i+n}else return i+s;const r=e.referenceFn;if(r!==void 0&&(n=e3(r),n!==void 0))return i+n;if(e.owner!==void 0){const a=Vce(e.owner,o);if(a!==void 0)return i+a}}function Vce(o,e){for(const t in o)if(o[t]===e)return t}const i8=new Map,n8=new WeakMap;function zce(o){const e=n8.get(o);if(e)return e;const t=Uce(o);let i=i8.get(t)??0;i++,i8.set(t,i);const n=i===1?t:`${t}#${i}`;return n8.set(o,n),n}function Uce(o){const e=o.constructor;return e?e.name:"Object"}function e3(o){const e=o.toString(),i=/\/\*\*\s*@description\s*([^*]*)\*\//.exec(e),n=i?i[1]:void 0;return n==null?void 0:n.trim()}let $ce;function Cj(){return $ce}let vj;function jce(o){vj=o}let wj;function Kce(o){wj=o}let V2;function qce(o){V2=o}class Sj{get TChange(){return null}reportChanges(){this.get()}read(e){return e?e.readObservable(this):this.get()}map(e,t){const i=t===void 0?void 0:e,n=t===void 0?e:t;return V2({owner:i,debugName:()=>{const s=e3(n);if(s!==void 0)return s;const a=/^\s*\(?\s*([a-zA-Z_$][a-zA-Z_$0-9]*)\s*\)?\s*=>\s*\1(?:\??)\.([a-zA-Z_$][a-zA-Z_$0-9]*)\s*$/.exec(n.toString());if(a)return`${this.debugName}.${a[2]}`;if(!i)return`${this.debugName} (mapped)`},debugReferenceFn:n},s=>n(this.read(s),s))}flatten(){return V2({owner:void 0,debugName:()=>`${this.debugName} (flattened)`},e=>this.read(e).read(e))}recomputeInitiallyAndOnChange(e,t){return e.add(vj(this,t)),this}keepObserved(e){return e.add(wj(this)),this}}class Sb extends Sj{constructor(){super(...arguments),this.observers=new Set}addObserver(e){const t=this.observers.size;this.observers.add(e),t===0&&this.onFirstObserverAdded()}removeObserver(e){this.observers.delete(e)&&this.observers.size===0&&this.onLastObserverRemoved()}onFirstObserverAdded(){}onLastObserverRemoved(){}}function ai(o,e){const t=new yb(o,e);try{o(t)}finally{t.finish()}}let GS;function kC(o){if(GS)o(GS);else{const e=new yb(o,void 0);GS=e;try{o(e)}finally{e.finish(),GS=void 0}}}async function yj(o,e){const t=new yb(o,e);try{await o(t)}finally{t.finish()}}function Pf(o,e,t){o?e(o):ai(e,t)}class yb{constructor(e,t){this._fn=e,this._getDebugName=t,this.updatingObservers=[]}getDebugName(){return this._getDebugName?this._getDebugName():e3(this._fn)}updateObserver(e,t){this.updatingObservers.push({observer:e,observable:t}),e.beginUpdate(t)}finish(){const e=this.updatingObservers;for(let t=0;t<e.length;t++){const{observer:i,observable:n}=e[t];i.endUpdate(n)}this.updatingObservers=null}}function Ke(o,e){let t;return typeof o=="string"?t=new Wn(void 0,o,void 0):t=new Wn(o,void 0,void 0),new t3(t,e,jo)}class t3 extends Sb{get debugName(){return this._debugNameData.getDebugName(this)??"ObservableValue"}constructor(e,t,i){super(),this._debugNameData=e,this._equalityComparator=i,this._value=t}get(){return this._value}set(e,t,i){var s;if(i===void 0&&this._equalityComparator(this._value,e))return;let n;t||(t=n=new yb(()=>{},()=>`Setting ${this.debugName}`));try{const r=this._value;this._setValue(e),(s=Cj())==null||s.handleObservableChanged(this,{oldValue:r,newValue:e,change:i,didChange:!0,hadValue:!0});for(const a of this.observers)t.updateObserver(a,this),a.handleChange(this,i)}finally{n&&n.finish()}}toString(){return`${this.debugName}: ${this._value}`}_setValue(e){this._value=e}}function Dw(o,e){let t;return typeof o=="string"?t=new Wn(void 0,o,void 0):t=new Wn(o,void 0,void 0),new Gce(t,e,jo)}class Gce extends t3{_setValue(e){this._value!==e&&(this._value&&this._value.dispose(),this._value=e)}dispose(){var e;(e=this._value)==null||e.dispose()}}function be(o,e){return e!==void 0?new Of(new Wn(o,void 0,e),e,void 0,void 0,void 0,jo):new Of(new Wn(void 0,void 0,o),o,void 0,void 0,void 0,jo)}function q1(o,e,t){return new Zce(new Wn(o,void 0,e),e,void 0,void 0,void 0,jo,t)}function Mo(o,e){return new Of(new Wn(o.owner,o.debugName,o.debugReferenceFn),e,void 0,void 0,o.onLastObserverRemoved,o.equalsFn??jo)}qce(Mo);function Lj(o,e){return new Of(new Wn(o.owner,o.debugName,void 0),e,o.createEmptyChangeSummary,o.handleChange,void 0,o.equalityComparer??jo)}function su(o,e){let t,i;e===void 0?(t=o,i=void 0):(i=o,t=e);const n=new K;return new Of(new Wn(i,void 0,t),s=>(n.clear(),t(s,n)),void 0,void 0,()=>n.dispose(),jo)}function Gn(o,e){let t,i;e===void 0?(t=o,i=void 0):(i=o,t=e);let n;return new Of(new Wn(i,void 0,t),s=>{n?n.clear():n=new K;const r=t(s);return r&&n.add(r),r},void 0,void 0,()=>{n&&(n.dispose(),n=void 0)},jo)}class Of extends Sb{get debugName(){return this._debugNameData.getDebugName(this)??"(anonymous)"}constructor(e,t,i,n,s=void 0,r){var a;super(),this._debugNameData=e,this._computeFn=t,this.createChangeSummary=i,this._handleChange=n,this._handleLastObserverRemoved=s,this._equalityComparator=r,this.state=0,this.value=void 0,this.updateCount=0,this.dependencies=new Set,this.dependenciesToBeRemoved=new Set,this.changeSummary=void 0,this.changeSummary=(a=this.createChangeSummary)==null?void 0:a.call(this)}onLastObserverRemoved(){var e;this.state=0,this.value=void 0;for(const t of this.dependencies)t.removeObserver(this);this.dependencies.clear(),(e=this._handleLastObserverRemoved)==null||e.call(this)}get(){var e;if(this.observers.size===0){const t=this._computeFn(this,(e=this.createChangeSummary)==null?void 0:e.call(this));return this.onLastObserverRemoved(),t}else{do{if(this.state===1){for(const t of this.dependencies)if(t.reportChanges(),this.state===2)break}this.state===1&&(this.state=3),this._recomputeIfNeeded()}while(this.state!==3);return this.value}}_recomputeIfNeeded(){var r;if(this.state===3)return;const e=this.dependenciesToBeRemoved;this.dependenciesToBeRemoved=this.dependencies,this.dependencies=e;const t=this.state!==0,i=this.value;this.state=3;const n=this.changeSummary;this.changeSummary=(r=this.createChangeSummary)==null?void 0:r.call(this);try{this.value=this._computeFn(this,n)}finally{for(const a of this.dependenciesToBeRemoved)a.removeObserver(this);this.dependenciesToBeRemoved.clear()}if(t&&!this._equalityComparator(i,this.value))for(const a of this.observers)a.handleChange(this,void 0)}toString(){return`LazyDerived<${this.debugName}>`}beginUpdate(e){this.updateCount++;const t=this.updateCount===1;if(this.state===3&&(this.state=1,!t))for(const i of this.observers)i.handlePossibleChange(this);if(t)for(const i of this.observers)i.beginUpdate(this)}endUpdate(e){if(this.updateCount--,this.updateCount===0){const t=[...this.observers];for(const i of t)i.endUpdate(this)}Nf(()=>this.updateCount>=0)}handlePossibleChange(e){if(this.state===3&&this.dependencies.has(e)&&!this.dependenciesToBeRemoved.has(e)){this.state=1;for(const t of this.observers)t.handlePossibleChange(this)}}handleChange(e,t){if(this.dependencies.has(e)&&!this.dependenciesToBeRemoved.has(e)){const i=this._handleChange?this._handleChange({changedObservable:e,change:t,didChange:s=>s===e},this.changeSummary):!0,n=this.state===3;if(i&&(this.state===1||n)&&(this.state=2,n))for(const s of this.observers)s.handlePossibleChange(this)}}readObservable(e){e.addObserver(this);const t=e.get();return this.dependencies.add(e),this.dependenciesToBeRemoved.delete(e),t}addObserver(e){const t=!this.observers.has(e)&&this.updateCount>0;super.addObserver(e),t&&e.beginUpdate(this)}removeObserver(e){const t=this.observers.has(e)&&this.updateCount>0;super.removeObserver(e),t&&e.endUpdate(this)}}class Zce extends Of{constructor(e,t,i,n,s=void 0,r,a){super(e,t,i,n,s,r),this.set=a}}function Ae(o){return new eE(new Wn(void 0,void 0,o),o,void 0,void 0)}function G1(o,e){return new eE(new Wn(o.owner,o.debugName,o.debugReferenceFn??e),e,void 0,void 0)}function Z1(o,e){return new eE(new Wn(o.owner,o.debugName,o.debugReferenceFn??e),e,o.createEmptyChangeSummary,o.handleChange)}function Yce(o,e){const t=new K,i=Z1({owner:o.owner,debugName:o.debugName,debugReferenceFn:o.debugReferenceFn??e,createEmptyChangeSummary:o.createEmptyChangeSummary,handleChange:o.handleChange},(n,s)=>{t.clear(),e(n,s,t)});return ve(()=>{i.dispose(),t.dispose()})}function Ps(o){const e=new K,t=G1({owner:void 0,debugName:void 0,debugReferenceFn:o},i=>{e.clear(),o(i,e)});return ve(()=>{t.dispose(),e.dispose()})}class eE{get debugName(){return this._debugNameData.getDebugName(this)??"(anonymous)"}constructor(e,t,i,n){var s;this._debugNameData=e,this._runFn=t,this.createChangeSummary=i,this._handleChange=n,this.state=2,this.updateCount=0,this.disposed=!1,this.dependencies=new Set,this.dependenciesToBeRemoved=new Set,this.changeSummary=(s=this.createChangeSummary)==null?void 0:s.call(this),this._runIfNeeded()}dispose(){this.disposed=!0;for(const e of this.dependencies)e.removeObserver(this);this.dependencies.clear()}_runIfNeeded(){var i,n;if(this.state===3)return;const e=this.dependenciesToBeRemoved;this.dependenciesToBeRemoved=this.dependencies,this.dependencies=e,this.state=3;const t=this.disposed;try{if(!t){(i=Cj())==null||i.handleAutorunTriggered(this);const s=this.changeSummary;this.changeSummary=(n=this.createChangeSummary)==null?void 0:n.call(this),this._runFn(this,s)}}finally{for(const s of this.dependenciesToBeRemoved)s.removeObserver(this);this.dependenciesToBeRemoved.clear()}}toString(){return`Autorun<${this.debugName}>`}beginUpdate(){this.state===3&&(this.state=1),this.updateCount++}endUpdate(){if(this.updateCount===1)do{if(this.state===1){this.state=3;for(const e of this.dependencies)if(e.reportChanges(),this.state===2)break}this._runIfNeeded()}while(this.state!==3);this.updateCount--,Nf(()=>this.updateCount>=0)}handlePossibleChange(e){this.state===3&&this.dependencies.has(e)&&!this.dependenciesToBeRemoved.has(e)&&(this.state=1)}handleChange(e,t){this.dependencies.has(e)&&!this.dependenciesToBeRemoved.has(e)&&(!this._handleChange||this._handleChange({changedObservable:e,change:t,didChange:n=>n===e},this.changeSummary))&&(this.state=2)}readObservable(e){if(this.disposed)return e.get();e.addObserver(this);const t=e.get();return this.dependencies.add(e),this.dependenciesToBeRemoved.delete(e),t}}(function(o){o.Observer=eE})(Ae||(Ae={}));function ur(o){return new Xce(o)}class Xce extends Sj{constructor(e){super(),this.value=e}get debugName(){return this.toString()}get(){return this.value}addObserver(e){}removeObserver(e){}toString(){return`Const: ${this.value}`}}function xt(...o){let e,t,i;return o.length===3?[e,t,i]=o:[t,i]=o,new Qd(new Wn(e,void 0,i),t,i,()=>Qd.globalTransaction,jo)}function Qce(o,e,t){return new Qd(new Wn(o.owner,o.debugName,o.debugReferenceFn??t),e,t,()=>Qd.globalTransaction,o.equalsFn??jo)}class Qd extends Sb{constructor(e,t,i,n,s){super(),this._debugNameData=e,this.event=t,this._getValue=i,this._getTransaction=n,this._equalityComparator=s,this.hasValue=!1,this.handleEvent=r=>{const a=this._getValue(r),l=this.value;(!this.hasValue||!this._equalityComparator(l,a))&&(this.value=a,this.hasValue&&Pf(this._getTransaction(),d=>{for(const h of this.observers)d.updateObserver(h,this),h.handleChange(this,void 0)},()=>{const d=this.getDebugName();return"Event fired"+(d?`: ${d}`:"")}),this.hasValue=!0)}}getDebugName(){return this._debugNameData.getDebugName(this)}get debugName(){const e=this.getDebugName();return"From Event"+(e?`: ${e}`:"")}onFirstObserverAdded(){this.subscription=this.event(this.handleEvent)}onLastObserverRemoved(){this.subscription.dispose(),this.subscription=void 0,this.hasValue=!1,this.value=void 0}get(){return this.subscription?(this.hasValue||this.handleEvent(void 0),this.value):this._getValue(void 0)}}(function(o){o.Observer=Qd;function e(t,i){let n=!1;Qd.globalTransaction===void 0&&(Qd.globalTransaction=t,n=!0);try{i()}finally{n&&(Qd.globalTransaction=void 0)}}o.batchEventsGlobally=e})(xt||(xt={}));function xn(o,e){return new Jce(o,e)}class Jce extends Sb{constructor(e,t){super(),this.debugName=e,this.event=t,this.handleEvent=()=>{ai(i=>{for(const n of this.observers)i.updateObserver(n,this),n.handleChange(this,void 0)},()=>this.debugName)}}onFirstObserverAdded(){this.subscription=this.event(this.handleEvent)}onLastObserverRemoved(){this.subscription.dispose(),this.subscription=void 0}get(){}}function Lb(o){return typeof o=="string"?new s8(o):new s8(void 0,o)}class s8 extends Sb{get debugName(){return new Wn(this._owner,this._debugName,void 0).getDebugName(this)??"Observable Signal"}toString(){return this.debugName}constructor(e,t){super(),this._debugName=e,this._owner=t}trigger(e,t){if(!e){ai(i=>{this.trigger(i,t)},()=>`Trigger signal ${this.debugName}`);return}for(const i of this.observers)e.updateObserver(i,this),i.handleChange(this,t)}get(){}}function ede(o){const e=new xj(!1,void 0);return o.addObserver(e),ve(()=>{o.removeObserver(e)})}Kce(ede);function xb(o,e){const t=new xj(!0,e);return o.addObserver(t),e?e(o.get()):o.reportChanges(),ve(()=>{o.removeObserver(t)})}jce(xb);class xj{constructor(e,t){this._forceRecompute=e,this._handleValue=t,this._counter=0}beginUpdate(e){this._counter++}endUpdate(e){this._counter--,this._counter===0&&this._forceRecompute&&(this._handleValue?this._handleValue(e.get()):e.reportChanges())}handlePossibleChange(e){}handleChange(e,t){}}function Y1(o,e){let t;return Mo({owner:o,debugReferenceFn:e},n=>(t=e(n,t),t))}function tde(o,e,t,i){let n=new o8(t,i);return Mo({debugReferenceFn:t,owner:o,onLastObserverRemoved:()=>{n.dispose(),n=new o8(t)}},r=>(n.setItems(e.read(r)),n.getItems()))}class o8{constructor(e,t){this._map=e,this._keySelector=t,this._cache=new Map,this._items=[]}dispose(){this._cache.forEach(e=>e.store.dispose()),this._cache.clear()}setItems(e){const t=[],i=new Set(this._cache.keys());for(const n of e){const s=this._keySelector?this._keySelector(n):n;let r=this._cache.get(s);if(r)i.delete(s);else{const a=new K;r={out:this._map(n,a),store:a},this._cache.set(s,r)}t.push(r.out)}for(const n of i)this._cache.get(n).store.dispose(),this._cache.delete(n);this._items=t}getItems(){return this._items}}function ide(o,e){return Y1(o,(t,i)=>i??e(t))}class tE{static fromFn(e){return new tE(e())}constructor(e){this._value=Ke(this,void 0),this.promiseResult=this._value,this.promise=e.then(t=>(ai(i=>{this._value.set(new r8(t,void 0),i)}),t),t=>{throw ai(i=>{this._value.set(new r8(void 0,t),i)}),t})}}class r8{constructor(e,t){this.data=e,this.error=t}}function kj(o,e,t,i){return e||(e=n=>n!=null),new Promise((n,s)=>{let r=!0,a=!1;const l=o.map(d=>({isFinished:e(d),error:t?t(d):!1,state:d})),c=Ae(d=>{const{isFinished:h,error:u,state:g}=l.read(d);(h||u)&&(r?a=!0:c.dispose(),u?s(u===!0?g:u):n(g))});if(i){const d=i.onCancellationRequested(()=>{c.dispose(),d.dispose(),s(new qr)});if(i.isCancellationRequested){c.dispose(),d.dispose(),s(new qr);return}}r=!1,a&&c.dispose()})}class nde extends Sb{get debugName(){return this._debugNameData.getDebugName(this)??"LazyObservableValue"}constructor(e,t,i){super(),this._debugNameData=e,this._equalityComparator=i,this._isUpToDate=!0,this._deltas=[],this._updateCounter=0,this._value=t}get(){return this._update(),this._value}_update(){if(!this._isUpToDate)if(this._isUpToDate=!0,this._deltas.length>0){for(const e of this.observers)for(const t of this._deltas)e.handleChange(this,t);this._deltas.length=0}else for(const e of this.observers)e.handleChange(this,void 0)}_beginUpdate(){if(this._updateCounter++,this._updateCounter===1)for(const e of this.observers)e.beginUpdate(this)}_endUpdate(){if(this._updateCounter--,this._updateCounter===0){this._update();const e=[...this.observers];for(const t of e)t.endUpdate(this)}}addObserver(e){const t=!this.observers.has(e)&&this._updateCounter>0;super.addObserver(e),t&&e.beginUpdate(this)}removeObserver(e){const t=this.observers.has(e)&&this._updateCounter>0;super.removeObserver(e),t&&e.endUpdate(this)}set(e,t,i){if(i===void 0&&this._equalityComparator(this._value,e))return;let n;t||(t=n=new yb(()=>{},()=>`Setting ${this.debugName}`));try{if(this._isUpToDate=!1,this._setValue(e),i!==void 0&&this._deltas.push(i),t.updateObserver({beginUpdate:()=>this._beginUpdate(),endUpdate:()=>this._endUpdate(),handleChange:(s,r)=>{},handlePossibleChange:s=>{}},this),this._updateCounter>1)for(const s of this.observers)s.handlePossibleChange(this)}finally{n&&n.finish()}}toString(){return`${this.debugName}: ${this._value}`}_setValue(e){this._value=e}}function z2(o,e){return o.lazy?new nde(new Wn(o.owner,o.debugName,void 0),e,o.equalsFn??jo):new t3(new Wn(o.owner,o.debugName,void 0),e,o.equalsFn??jo)}const Ov=class Ov extends F{constructor(e=!1){super(),this._onDidRequestBasicLanguageFeatures=this._register(new P),this.onDidRequestBasicLanguageFeatures=this._onDidRequestBasicLanguageFeatures.event,this._onDidRequestRichLanguageFeatures=this._register(new P),this.onDidRequestRichLanguageFeatures=this._onDidRequestRichLanguageFeatures.event,this._onDidChange=this._register(new P({leakWarningThreshold:200})),this.onDidChange=this._onDidChange.event,this._requestedBasicLanguages=new Set,this._requestedRichLanguages=new Set,Ov.instanceCount++,this._registry=this._register(new W2(!0,e)),this.languageIdCodec=this._registry.languageIdCodec,this._register(this._registry.onDidChange(()=>this._onDidChange.fire()))}dispose(){Ov.instanceCount--,super.dispose()}isRegisteredLanguageId(e){return this._registry.isRegisteredLanguageId(e)}getLanguageIdByLanguageName(e){return this._registry.getLanguageIdByLanguageName(e)}getLanguageIdByMimeType(e){return this._registry.getLanguageIdByMimeType(e)}guessLanguageIdByFilepathOrFirstLine(e,t){const i=this._registry.guessLanguageIdByFilepathOrFirstLine(e,t);return O4(i,null)}createById(e){return new a8(this.onDidChange,()=>this._createAndGetLanguageIdentifier(e))}createByFilepathOrFirstLine(e,t){return new a8(this.onDidChange,()=>{const i=this.guessLanguageIdByFilepathOrFirstLine(e,t);return this._createAndGetLanguageIdentifier(i)})}_createAndGetLanguageIdentifier(e){return(!e||!this.isRegisteredLanguageId(e))&&(e=ms),e}requestBasicLanguageFeatures(e){this._requestedBasicLanguages.has(e)||(this._requestedBasicLanguages.add(e),this._onDidRequestBasicLanguageFeatures.fire(e))}requestRichLanguageFeatures(e){this._requestedRichLanguages.has(e)||(this._requestedRichLanguages.add(e),this.requestBasicLanguageFeatures(e),fi.getOrCreate(e),this._onDidRequestRichLanguageFeatures.fire(e))}};Ov.instanceCount=0;let U2=Ov;class a8{constructor(e,t){this._value=xt(this,e,()=>t()),this.onDidChange=oe.fromObservable(this._value)}get languageId(){return this._value.get()}}const Iw={RESOURCES:"ResourceURLs",TEXT:vi.text,INTERNAL_URI_LIST:"application/vnd.code.uri-list"},sde=()=>({get delay(){return-1},dispose:()=>{},showHover:()=>{}});let iE=sde;const ode=new Oa(()=>iE("mouse",!1)),rde=new Oa(()=>iE("element",!1));function ade(o){iE=o}function Ss(o){return o==="element"?rde.value:ode.value}function H_(){return iE("element",!0)}let Dj={showHover:()=>{},hideHover:()=>{},showAndFocusLastHover:()=>{},setupManagedHover:()=>null,showManagedHover:()=>{}};function lde(o){Dj=o}function ja(){return Dj}class cde{constructor(e){this.spliceables=e}splice(e,t,i){this.spliceables.forEach(n=>n.splice(e,t,i))}}class bu extends Error{constructor(e,t){super(`ListError [${e}] ${t}`)}}function l8(o,e){const t=[];for(const i of e){if(o.start>=i.range.end)continue;if(o.end<i.range.start)break;const n=Rn.intersect(o,i.range);Rn.isEmpty(n)||t.push({range:n,size:i.size})}return t}function $2({start:o,end:e},t){return{start:o+t,end:e+t}}function dde(o){const e=[];let t=null;for(const i of o){const n=i.range.start,s=i.range.end,r=i.size;if(t&&r===t.size){t.range.end=s;continue}t={range:{start:n,end:s},size:r},e.push(t)}return e}function hde(...o){return dde(o.reduce((e,t)=>e.concat(t),[]))}class ude{get paddingTop(){return this._paddingTop}set paddingTop(e){this._size=this._size+e-this._paddingTop,this._paddingTop=e}constructor(e){this.groups=[],this._size=0,this._paddingTop=0,this._paddingTop=e??0,this._size=this._paddingTop}splice(e,t,i=[]){const n=i.length-t,s=l8({start:0,end:e},this.groups),r=l8({start:e+t,end:Number.POSITIVE_INFINITY},this.groups).map(l=>({range:$2(l.range,n),size:l.size})),a=i.map((l,c)=>({range:{start:e+c,end:e+c+1},size:l.size}));this.groups=hde(s,a,r),this._size=this._paddingTop+this.groups.reduce((l,c)=>l+c.size*(c.range.end-c.range.start),0)}get count(){const e=this.groups.length;return e?this.groups[e-1].range.end:0}get size(){return this._size}indexAt(e){if(e<0)return-1;if(e<this._paddingTop)return 0;let t=0,i=this._paddingTop;for(const n of this.groups){const s=n.range.end-n.range.start,r=i+s*n.size;if(e<r)return t+Math.floor((e-i)/n.size);t+=s,i=r}return t}indexAfter(e){return Math.min(this.indexAt(e)+1,this.count)}positionAt(e){if(e<0)return-1;let t=0,i=0;for(const n of this.groups){const s=n.range.end-n.range.start,r=i+s;if(e<r)return this._paddingTop+t+(e-i)*n.size;t+=s*n.size,i=r}return-1}}class gde{constructor(e){this.renderers=e,this.cache=new Map,this.transactionNodesPendingRemoval=new Set,this.inTransaction=!1}alloc(e){let t=this.getTemplateCache(e).pop(),i=!1;if(t)i=this.transactionNodesPendingRemoval.has(t.domNode),i&&this.transactionNodesPendingRemoval.delete(t.domNode);else{const n=ne(".monaco-list-row"),r=this.getRenderer(e).renderTemplate(n);t={domNode:n,templateId:e,templateData:r}}return{row:t,isReusingConnectedDomNode:i}}release(e){e&&this.releaseRow(e)}transact(e){if(this.inTransaction)throw new Error("Already in transaction");this.inTransaction=!0;try{e()}finally{for(const t of this.transactionNodesPendingRemoval)this.doRemoveNode(t);this.transactionNodesPendingRemoval.clear(),this.inTransaction=!1}}releaseRow(e){const{domNode:t,templateId:i}=e;t&&(this.inTransaction?this.transactionNodesPendingRemoval.add(t):this.doRemoveNode(t)),this.getTemplateCache(i).push(e)}doRemoveNode(e){e.classList.remove("scrolling"),e.remove()}getTemplateCache(e){let t=this.cache.get(e);return t||(t=[],this.cache.set(e,t)),t}dispose(){this.cache.forEach((e,t)=>{for(const i of e)this.getRenderer(t).disposeTemplate(i.templateData),i.templateData=null}),this.cache.clear(),this.transactionNodesPendingRemoval.clear()}getRenderer(e){const t=this.renderers.get(e);if(!t)throw new Error(`No renderer found for ${e}`);return t}}var gd=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s};const Cu={CurrentDragAndDropData:void 0},Ja={useShadows:!0,verticalScrollMode:1,setRowLineHeight:!0,setRowHeight:!0,supportDynamicHeights:!1,dnd:{getDragElements(o){return[o]},getDragURI(){return null},onDragStart(){},onDragOver(){return!1},drop(){},dispose(){}},horizontalScrolling:!1,transformOptimization:!0,alwaysConsumeMouseWheel:!0};class X1{constructor(e){this.elements=e}update(){}getData(){return this.elements}}class fde{constructor(e){this.elements=e}update(){}getData(){return this.elements}}class mde{constructor(){this.types=[],this.files=[]}update(e){if(e.types&&this.types.splice(0,this.types.length,...e.types),e.files){this.files.splice(0,this.files.length);for(let t=0;t<e.files.length;t++){const i=e.files.item(t);i&&(i.size||i.type)&&this.files.push(i)}}}getData(){return{types:this.types,files:this.files}}}function pde(o,e){return Array.isArray(o)&&Array.isArray(e)?ii(o,e):o===e}class _de{constructor(e){e!=null&&e.getSetSize?this.getSetSize=e.getSetSize.bind(e):this.getSetSize=(t,i,n)=>n,e!=null&&e.getPosInSet?this.getPosInSet=e.getPosInSet.bind(e):this.getPosInSet=(t,i)=>i+1,e!=null&&e.getRole?this.getRole=e.getRole.bind(e):this.getRole=t=>"listitem",e!=null&&e.isChecked?this.isChecked=e.isChecked.bind(e):this.isChecked=t=>{}}}const zk=class zk{get contentHeight(){return this.rangeMap.size}get onDidScroll(){return this.scrollableElement.onScroll}get scrollableElementDomNode(){return this.scrollableElement.getDomNode()}get horizontalScrolling(){return this._horizontalScrolling}set horizontalScrolling(e){if(e!==this._horizontalScrolling){if(e&&this.supportDynamicHeights)throw new Error("Horizontal scrolling and dynamic heights not supported simultaneously");if(this._horizontalScrolling=e,this.domNode.classList.toggle("horizontal-scrolling",this._horizontalScrolling),this._horizontalScrolling){for(const t of this.items)this.measureItemWidth(t);this.updateScrollWidth(),this.scrollableElement.setScrollDimensions({width:pN(this.domNode)}),this.rowsContainer.style.width=`${Math.max(this.scrollWidth||0,this.renderWidth)}px`}else this.scrollableElementWidthDelayer.cancel(),this.scrollableElement.setScrollDimensions({width:this.renderWidth,scrollWidth:this.renderWidth}),this.rowsContainer.style.width=""}}constructor(e,t,i,n=Ja){var r,a;if(this.virtualDelegate=t,this.domId=`list_id_${++zk.InstanceCount}`,this.renderers=new Map,this.renderWidth=0,this._scrollHeight=0,this.scrollableElementUpdateDisposable=null,this.scrollableElementWidthDelayer=new _r(50),this.splicing=!1,this.dragOverAnimationStopDisposable=F.None,this.dragOverMouseY=0,this.canDrop=!1,this.currentDragFeedbackDisposable=F.None,this.onDragLeaveTimeout=F.None,this.disposables=new K,this._onDidChangeContentHeight=new P,this._onDidChangeContentWidth=new P,this.onDidChangeContentHeight=oe.latch(this._onDidChangeContentHeight.event,void 0,this.disposables),this._horizontalScrolling=!1,n.horizontalScrolling&&n.supportDynamicHeights)throw new Error("Horizontal scrolling and dynamic heights not supported simultaneously");this.items=[],this.itemId=0,this.rangeMap=this.createRangeMap(n.paddingTop??0);for(const l of i)this.renderers.set(l.templateId,l);this.cache=this.disposables.add(new gde(this.renderers)),this.lastRenderTop=0,this.lastRenderHeight=0,this.domNode=document.createElement("div"),this.domNode.className="monaco-list",this.domNode.classList.add(this.domId),this.domNode.tabIndex=0,this.domNode.classList.toggle("mouse-support",typeof n.mouseSupport=="boolean"?n.mouseSupport:!0),this._horizontalScrolling=n.horizontalScrolling??Ja.horizontalScrolling,this.domNode.classList.toggle("horizontal-scrolling",this._horizontalScrolling),this.paddingBottom=typeof n.paddingBottom>"u"?0:n.paddingBottom,this.accessibilityProvider=new _de(n.accessibilityProvider),this.rowsContainer=document.createElement("div"),this.rowsContainer.className="monaco-list-rows",(n.transformOptimization??Ja.transformOptimization)&&(this.rowsContainer.style.transform="translate3d(0px, 0px, 0px)",this.rowsContainer.style.overflow="hidden",this.rowsContainer.style.contain="strict"),this.disposables.add(Fn.addTarget(this.rowsContainer)),this.scrollable=this.disposables.add(new wb({forceIntegerValues:!0,smoothScrollDuration:n.smoothScrolling??!1?125:0,scheduleAtNextAnimationFrame:l=>go(xe(this.domNode),l)})),this.scrollableElement=this.disposables.add(new jI(this.rowsContainer,{alwaysConsumeMouseWheel:n.alwaysConsumeMouseWheel??Ja.alwaysConsumeMouseWheel,horizontal:1,vertical:n.verticalScrollMode??Ja.verticalScrollMode,useShadows:n.useShadows??Ja.useShadows,mouseWheelScrollSensitivity:n.mouseWheelScrollSensitivity,fastScrollSensitivity:n.fastScrollSensitivity,scrollByPage:n.scrollByPage},this.scrollable)),this.domNode.appendChild(this.scrollableElement.getDomNode()),e.appendChild(this.domNode),this.scrollableElement.onScroll(this.onScroll,this,this.disposables),this.disposables.add(U(this.rowsContainer,Bt.Change,l=>this.onTouchChange(l))),this.disposables.add(U(this.scrollableElement.getDomNode(),"scroll",l=>l.target.scrollTop=0)),this.disposables.add(U(this.domNode,"dragover",l=>this.onDragOver(this.toDragEvent(l)))),this.disposables.add(U(this.domNode,"drop",l=>this.onDrop(this.toDragEvent(l)))),this.disposables.add(U(this.domNode,"dragleave",l=>this.onDragLeave(this.toDragEvent(l)))),this.disposables.add(U(this.domNode,"dragend",l=>this.onDragEnd(l))),this.setRowLineHeight=n.setRowLineHeight??Ja.setRowLineHeight,this.setRowHeight=n.setRowHeight??Ja.setRowHeight,this.supportDynamicHeights=n.supportDynamicHeights??Ja.supportDynamicHeights,this.dnd=n.dnd??this.disposables.add(Ja.dnd),this.layout((r=n.initialSize)==null?void 0:r.height,(a=n.initialSize)==null?void 0:a.width)}updateOptions(e){e.paddingBottom!==void 0&&(this.paddingBottom=e.paddingBottom,this.scrollableElement.setScrollDimensions({scrollHeight:this.scrollHeight})),e.smoothScrolling!==void 0&&this.scrollable.setSmoothScrollDuration(e.smoothScrolling?125:0),e.horizontalScrolling!==void 0&&(this.horizontalScrolling=e.horizontalScrolling);let t;if(e.scrollByPage!==void 0&&(t={...t??{},scrollByPage:e.scrollByPage}),e.mouseWheelScrollSensitivity!==void 0&&(t={...t??{},mouseWheelScrollSensitivity:e.mouseWheelScrollSensitivity}),e.fastScrollSensitivity!==void 0&&(t={...t??{},fastScrollSensitivity:e.fastScrollSensitivity}),t&&this.scrollableElement.updateOptions(t),e.paddingTop!==void 0&&e.paddingTop!==this.rangeMap.paddingTop){const i=this.getRenderRange(this.lastRenderTop,this.lastRenderHeight),n=e.paddingTop-this.rangeMap.paddingTop;this.rangeMap.paddingTop=e.paddingTop,this.render(i,Math.max(0,this.lastRenderTop+n),this.lastRenderHeight,void 0,void 0,!0),this.setScrollTop(this.lastRenderTop),this.eventuallyUpdateScrollDimensions(),this.supportDynamicHeights&&this._rerender(this.lastRenderTop,this.lastRenderHeight)}}createRangeMap(e){return new ude(e)}splice(e,t,i=[]){if(this.splicing)throw new Error("Can't run recursive splices.");this.splicing=!0;try{return this._splice(e,t,i)}finally{this.splicing=!1,this._onDidChangeContentHeight.fire(this.contentHeight)}}_splice(e,t,i=[]){const n=this.getRenderRange(this.lastRenderTop,this.lastRenderHeight),s={start:e,end:e+t},r=Rn.intersect(n,s),a=new Map;for(let S=r.end-1;S>=r.start;S--){const L=this.items[S];if(L.dragStartDisposable.dispose(),L.checkedDisposable.dispose(),L.row){let k=a.get(L.templateId);k||(k=[],a.set(L.templateId,k));const D=this.renderers.get(L.templateId);D&&D.disposeElement&&D.disposeElement(L.element,S,L.row.templateData,L.size),k.unshift(L.row)}L.row=null,L.stale=!0}const l={start:e+t,end:this.items.length},c=Rn.intersect(l,n),d=Rn.relativeComplement(l,n),h=i.map(S=>({id:String(this.itemId++),element:S,templateId:this.virtualDelegate.getTemplateId(S),size:this.virtualDelegate.getHeight(S),width:void 0,hasDynamicHeight:!!this.virtualDelegate.hasDynamicHeight&&this.virtualDelegate.hasDynamicHeight(S),lastDynamicHeightWidth:void 0,row:null,uri:void 0,dropTarget:!1,dragStartDisposable:F.None,checkedDisposable:F.None,stale:!1}));let u;e===0&&t>=this.items.length?(this.rangeMap=this.createRangeMap(this.rangeMap.paddingTop),this.rangeMap.splice(0,0,h),u=this.items,this.items=h):(this.rangeMap.splice(e,t,h),u=this.items.splice(e,t,...h));const g=i.length-t,f=this.getRenderRange(this.lastRenderTop,this.lastRenderHeight),p=$2(c,g),_=Rn.intersect(f,p);for(let S=_.start;S<_.end;S++)this.updateItemInDOM(this.items[S],S);const b=Rn.relativeComplement(p,f);for(const S of b)for(let L=S.start;L<S.end;L++)this.removeItemFromDOM(L);const C=d.map(S=>$2(S,g)),w=[{start:e,end:e+i.length},...C].map(S=>Rn.intersect(f,S)).reverse();for(const S of w)for(let L=S.end-1;L>=S.start;L--){const k=this.items[L],D=a.get(k.templateId),N=D==null?void 0:D.pop();this.insertItemInDOM(L,N)}for(const S of a.values())for(const L of S)this.cache.release(L);return this.eventuallyUpdateScrollDimensions(),this.supportDynamicHeights&&this._rerender(this.scrollTop,this.renderHeight),u.map(S=>S.element)}eventuallyUpdateScrollDimensions(){this._scrollHeight=this.contentHeight,this.rowsContainer.style.height=`${this._scrollHeight}px`,this.scrollableElementUpdateDisposable||(this.scrollableElementUpdateDisposable=go(xe(this.domNode),()=>{this.scrollableElement.setScrollDimensions({scrollHeight:this.scrollHeight}),this.updateScrollWidth(),this.scrollableElementUpdateDisposable=null}))}eventuallyUpdateScrollWidth(){if(!this.horizontalScrolling){this.scrollableElementWidthDelayer.cancel();return}this.scrollableElementWidthDelayer.trigger(()=>this.updateScrollWidth())}updateScrollWidth(){if(!this.horizontalScrolling)return;let e=0;for(const t of this.items)typeof t.width<"u"&&(e=Math.max(e,t.width));this.scrollWidth=e,this.scrollableElement.setScrollDimensions({scrollWidth:e===0?0:e+10}),this._onDidChangeContentWidth.fire(this.scrollWidth)}rerender(){if(this.supportDynamicHeights){for(const e of this.items)e.lastDynamicHeightWidth=void 0;this._rerender(this.lastRenderTop,this.lastRenderHeight)}}get length(){return this.items.length}get renderHeight(){return this.scrollableElement.getScrollDimensions().height}get firstVisibleIndex(){return this.getRenderRange(this.lastRenderTop,this.lastRenderHeight).start}element(e){return this.items[e].element}indexOf(e){return this.items.findIndex(t=>t.element===e)}domElement(e){const t=this.items[e].row;return t&&t.domNode}elementHeight(e){return this.items[e].size}elementTop(e){return this.rangeMap.positionAt(e)}indexAt(e){return this.rangeMap.indexAt(e)}indexAfter(e){return this.rangeMap.indexAfter(e)}layout(e,t){const i={height:typeof e=="number"?e:Hte(this.domNode)};this.scrollableElementUpdateDisposable&&(this.scrollableElementUpdateDisposable.dispose(),this.scrollableElementUpdateDisposable=null,i.scrollHeight=this.scrollHeight),this.scrollableElement.setScrollDimensions(i),typeof t<"u"&&(this.renderWidth=t,this.supportDynamicHeights&&this._rerender(this.scrollTop,this.renderHeight)),this.horizontalScrolling&&this.scrollableElement.setScrollDimensions({width:typeof t=="number"?t:pN(this.domNode)})}render(e,t,i,n,s,r=!1){const a=this.getRenderRange(t,i),l=Rn.relativeComplement(a,e).reverse(),c=Rn.relativeComplement(e,a);if(r){const d=Rn.intersect(e,a);for(let h=d.start;h<d.end;h++)this.updateItemInDOM(this.items[h],h)}this.cache.transact(()=>{for(const d of c)for(let h=d.start;h<d.end;h++)this.removeItemFromDOM(h);for(const d of l)for(let h=d.end-1;h>=d.start;h--)this.insertItemInDOM(h)}),n!==void 0&&(this.rowsContainer.style.left=`-${n}px`),this.rowsContainer.style.top=`-${t}px`,this.horizontalScrolling&&s!==void 0&&(this.rowsContainer.style.width=`${Math.max(s,this.renderWidth)}px`),this.lastRenderTop=t,this.lastRenderHeight=i}insertItemInDOM(e,t){var l,c;const i=this.items[e];if(!i.row)if(t)i.row=t,i.stale=!0;else{const d=this.cache.alloc(i.templateId);i.row=d.row,i.stale||(i.stale=d.isReusingConnectedDomNode)}const n=this.accessibilityProvider.getRole(i.element)||"listitem";i.row.domNode.setAttribute("role",n);const s=this.accessibilityProvider.isChecked(i.element);if(typeof s=="boolean")i.row.domNode.setAttribute("aria-checked",String(!!s));else if(s){const d=h=>i.row.domNode.setAttribute("aria-checked",String(!!h));d(s.value),i.checkedDisposable=s.onDidChange(()=>d(s.value))}if(i.stale||!i.row.domNode.parentElement){const d=((c=(l=this.items.at(e+1))==null?void 0:l.row)==null?void 0:c.domNode)??null;(i.row.domNode.parentElement!==this.rowsContainer||i.row.domNode.nextElementSibling!==d)&&this.rowsContainer.insertBefore(i.row.domNode,d),i.stale=!1}this.updateItemInDOM(i,e);const r=this.renderers.get(i.templateId);if(!r)throw new Error(`No renderer found for template id ${i.templateId}`);r==null||r.renderElement(i.element,e,i.row.templateData,i.size);const a=this.dnd.getDragURI(i.element);i.dragStartDisposable.dispose(),i.row.domNode.draggable=!!a,a&&(i.dragStartDisposable=U(i.row.domNode,"dragstart",d=>this.onDragStart(i.element,a,d))),this.horizontalScrolling&&(this.measureItemWidth(i),this.eventuallyUpdateScrollWidth())}measureItemWidth(e){if(!e.row||!e.row.domNode)return;e.row.domNode.style.width="fit-content",e.width=pN(e.row.domNode);const t=xe(e.row.domNode).getComputedStyle(e.row.domNode);t.paddingLeft&&(e.width+=parseFloat(t.paddingLeft)),t.paddingRight&&(e.width+=parseFloat(t.paddingRight)),e.row.domNode.style.width=""}updateItemInDOM(e,t){e.row.domNode.style.top=`${this.elementTop(t)}px`,this.setRowHeight&&(e.row.domNode.style.height=`${e.size}px`),this.setRowLineHeight&&(e.row.domNode.style.lineHeight=`${e.size}px`),e.row.domNode.setAttribute("data-index",`${t}`),e.row.domNode.setAttribute("data-last-element",t===this.length-1?"true":"false"),e.row.domNode.setAttribute("data-parity",t%2===0?"even":"odd"),e.row.domNode.setAttribute("aria-setsize",String(this.accessibilityProvider.getSetSize(e.element,t,this.length))),e.row.domNode.setAttribute("aria-posinset",String(this.accessibilityProvider.getPosInSet(e.element,t))),e.row.domNode.setAttribute("id",this.getElementDomId(t)),e.row.domNode.classList.toggle("drop-target",e.dropTarget)}removeItemFromDOM(e){const t=this.items[e];if(t.dragStartDisposable.dispose(),t.checkedDisposable.dispose(),t.row){const i=this.renderers.get(t.templateId);i&&i.disposeElement&&i.disposeElement(t.element,e,t.row.templateData,t.size),this.cache.release(t.row),t.row=null}this.horizontalScrolling&&this.eventuallyUpdateScrollWidth()}getScrollTop(){return this.scrollableElement.getScrollPosition().scrollTop}setScrollTop(e,t){this.scrollableElementUpdateDisposable&&(this.scrollableElementUpdateDisposable.dispose(),this.scrollableElementUpdateDisposable=null,this.scrollableElement.setScrollDimensions({scrollHeight:this.scrollHeight})),this.scrollableElement.setScrollPosition({scrollTop:e,reuseAnimation:t})}get scrollTop(){return this.getScrollTop()}set scrollTop(e){this.setScrollTop(e)}get scrollHeight(){return this._scrollHeight+(this.horizontalScrolling?10:0)+this.paddingBottom}get onMouseClick(){return oe.map(this.disposables.add(new tt(this.domNode,"click")).event,e=>this.toMouseEvent(e),this.disposables)}get onMouseDblClick(){return oe.map(this.disposables.add(new tt(this.domNode,"dblclick")).event,e=>this.toMouseEvent(e),this.disposables)}get onMouseMiddleClick(){return oe.filter(oe.map(this.disposables.add(new tt(this.domNode,"auxclick")).event,e=>this.toMouseEvent(e),this.disposables),e=>e.browserEvent.button===1,this.disposables)}get onMouseDown(){return oe.map(this.disposables.add(new tt(this.domNode,"mousedown")).event,e=>this.toMouseEvent(e),this.disposables)}get onMouseOver(){return oe.map(this.disposables.add(new tt(this.domNode,"mouseover")).event,e=>this.toMouseEvent(e),this.disposables)}get onMouseOut(){return oe.map(this.disposables.add(new tt(this.domNode,"mouseout")).event,e=>this.toMouseEvent(e),this.disposables)}get onContextMenu(){return oe.any(oe.map(this.disposables.add(new tt(this.domNode,"contextmenu")).event,e=>this.toMouseEvent(e),this.disposables),oe.map(this.disposables.add(new tt(this.domNode,Bt.Contextmenu)).event,e=>this.toGestureEvent(e),this.disposables))}get onTouchStart(){return oe.map(this.disposables.add(new tt(this.domNode,"touchstart")).event,e=>this.toTouchEvent(e),this.disposables)}get onTap(){return oe.map(this.disposables.add(new tt(this.rowsContainer,Bt.Tap)).event,e=>this.toGestureEvent(e),this.disposables)}toMouseEvent(e){const t=this.getItemIndexFromEventTarget(e.target||null),i=typeof t>"u"?void 0:this.items[t],n=i&&i.element;return{browserEvent:e,index:t,element:n}}toTouchEvent(e){const t=this.getItemIndexFromEventTarget(e.target||null),i=typeof t>"u"?void 0:this.items[t],n=i&&i.element;return{browserEvent:e,index:t,element:n}}toGestureEvent(e){const t=this.getItemIndexFromEventTarget(e.initialTarget||null),i=typeof t>"u"?void 0:this.items[t],n=i&&i.element;return{browserEvent:e,index:t,element:n}}toDragEvent(e){const t=this.getItemIndexFromEventTarget(e.target||null),i=typeof t>"u"?void 0:this.items[t],n=i&&i.element,s=this.getTargetSector(e,t);return{browserEvent:e,index:t,element:n,sector:s}}onScroll(e){try{const t=this.getRenderRange(this.lastRenderTop,this.lastRenderHeight);this.render(t,e.scrollTop,e.height,e.scrollLeft,e.scrollWidth),this.supportDynamicHeights&&this._rerender(e.scrollTop,e.height,e.inSmoothScrolling)}catch(t){throw console.error("Got bad scroll event:",e),t}}onTouchChange(e){e.preventDefault(),e.stopPropagation(),this.scrollTop-=e.translationY}onDragStart(e,t,i){var s,r;if(!i.dataTransfer)return;const n=this.dnd.getDragElements(e);if(i.dataTransfer.effectAllowed="copyMove",i.dataTransfer.setData(Iw.TEXT,t),i.dataTransfer.setDragImage){let a;this.dnd.getDragLabel&&(a=this.dnd.getDragLabel(n,i)),typeof a>"u"&&(a=String(n.length));const l=ne(".monaco-drag-image");l.textContent=a,(h=>{for(;h&&!h.classList.contains("monaco-workbench");)h=h.parentElement;return h||this.domNode.ownerDocument})(this.domNode).appendChild(l),i.dataTransfer.setDragImage(l,-10,-10),setTimeout(()=>l.remove(),0)}this.domNode.classList.add("dragging"),this.currentDragData=new X1(n),Cu.CurrentDragAndDropData=new fde(n),(r=(s=this.dnd).onDragStart)==null||r.call(s,this.currentDragData,i)}onDragOver(e){var s,r;if(e.browserEvent.preventDefault(),this.onDragLeaveTimeout.dispose(),Cu.CurrentDragAndDropData&&Cu.CurrentDragAndDropData.getData()==="vscode-ui"||(this.setupDragAndDropScrollTopAnimation(e.browserEvent),!e.browserEvent.dataTransfer))return!1;if(!this.currentDragData)if(Cu.CurrentDragAndDropData)this.currentDragData=Cu.CurrentDragAndDropData;else{if(!e.browserEvent.dataTransfer.types)return!1;this.currentDragData=new mde}const t=this.dnd.onDragOver(this.currentDragData,e.element,e.index,e.sector,e.browserEvent);if(this.canDrop=typeof t=="boolean"?t:t.accept,!this.canDrop)return this.currentDragFeedback=void 0,this.currentDragFeedbackDisposable.dispose(),!1;e.browserEvent.dataTransfer.dropEffect=typeof t!="boolean"&&((s=t.effect)==null?void 0:s.type)===0?"copy":"move";let i;typeof t!="boolean"&&t.feedback?i=t.feedback:typeof e.index>"u"?i=[-1]:i=[e.index],i=Xc(i).filter(a=>a>=-1&&a<this.length).sort((a,l)=>a-l),i=i[0]===-1?[-1]:i;let n=typeof t!="boolean"&&t.effect&&t.effect.position?t.effect.position:"drop-target";if(pde(this.currentDragFeedback,i)&&this.currentDragFeedbackPosition===n)return!0;if(this.currentDragFeedback=i,this.currentDragFeedbackPosition=n,this.currentDragFeedbackDisposable.dispose(),i[0]===-1)this.domNode.classList.add(n),this.rowsContainer.classList.add(n),this.currentDragFeedbackDisposable=ve(()=>{this.domNode.classList.remove(n),this.rowsContainer.classList.remove(n)});else{if(i.length>1&&n!=="drop-target")throw new Error("Can't use multiple feedbacks with position different than 'over'");n==="drop-target-after"&&i[0]<this.length-1&&(i[0]+=1,n="drop-target-before");for(const a of i){const l=this.items[a];l.dropTarget=!0,(r=l.row)==null||r.domNode.classList.add(n)}this.currentDragFeedbackDisposable=ve(()=>{var a;for(const l of i){const c=this.items[l];c.dropTarget=!1,(a=c.row)==null||a.domNode.classList.remove(n)}})}return!0}onDragLeave(e){var t,i;this.onDragLeaveTimeout.dispose(),this.onDragLeaveTimeout=Bh(()=>this.clearDragOverFeedback(),100,this.disposables),this.currentDragData&&((i=(t=this.dnd).onDragLeave)==null||i.call(t,this.currentDragData,e.element,e.index,e.browserEvent))}onDrop(e){if(!this.canDrop)return;const t=this.currentDragData;this.teardownDragAndDropScrollTopAnimation(),this.clearDragOverFeedback(),this.domNode.classList.remove("dragging"),this.currentDragData=void 0,Cu.CurrentDragAndDropData=void 0,!(!t||!e.browserEvent.dataTransfer)&&(e.browserEvent.preventDefault(),t.update(e.browserEvent.dataTransfer),this.dnd.drop(t,e.element,e.index,e.sector,e.browserEvent))}onDragEnd(e){var t,i;this.canDrop=!1,this.teardownDragAndDropScrollTopAnimation(),this.clearDragOverFeedback(),this.domNode.classList.remove("dragging"),this.currentDragData=void 0,Cu.CurrentDragAndDropData=void 0,(i=(t=this.dnd).onDragEnd)==null||i.call(t,e)}clearDragOverFeedback(){this.currentDragFeedback=void 0,this.currentDragFeedbackPosition=void 0,this.currentDragFeedbackDisposable.dispose(),this.currentDragFeedbackDisposable=F.None}setupDragAndDropScrollTopAnimation(e){if(!this.dragOverAnimationDisposable){const t=LU(this.domNode).top;this.dragOverAnimationDisposable=Jte(xe(this.domNode),this.animateDragAndDropScrollTop.bind(this,t))}this.dragOverAnimationStopDisposable.dispose(),this.dragOverAnimationStopDisposable=Bh(()=>{this.dragOverAnimationDisposable&&(this.dragOverAnimationDisposable.dispose(),this.dragOverAnimationDisposable=void 0)},1e3,this.disposables),this.dragOverMouseY=e.pageY}animateDragAndDropScrollTop(e){if(this.dragOverMouseY===void 0)return;const t=this.dragOverMouseY-e,i=this.renderHeight-35;t<35?this.scrollTop+=Math.max(-14,Math.floor(.3*(t-35))):t>i&&(this.scrollTop+=Math.min(14,Math.floor(.3*(t-i))))}teardownDragAndDropScrollTopAnimation(){this.dragOverAnimationStopDisposable.dispose(),this.dragOverAnimationDisposable&&(this.dragOverAnimationDisposable.dispose(),this.dragOverAnimationDisposable=void 0)}getTargetSector(e,t){if(t===void 0)return;const i=e.offsetY/this.items[t].size,n=Math.floor(i/.25);return os(n,0,3)}getItemIndexFromEventTarget(e){const t=this.scrollableElement.getDomNode();let i=e;for(;(Xi(i)||Kte(i))&&i!==this.rowsContainer&&t.contains(i);){const n=i.getAttribute("data-index");if(n){const s=Number(n);if(!isNaN(s))return s}i=i.parentElement}}getRenderRange(e,t){return{start:this.rangeMap.indexAt(e),end:this.rangeMap.indexAfter(e+t-1)}}_rerender(e,t,i){const n=this.getRenderRange(e,t);let s,r;e===this.elementTop(n.start)?(s=n.start,r=0):n.end-n.start>1&&(s=n.start+1,r=this.elementTop(s)-e);let a=0;for(;;){const l=this.getRenderRange(e,t);let c=!1;for(let d=l.start;d<l.end;d++){const h=this.probeDynamicHeight(d);h!==0&&this.rangeMap.splice(d,1,[this.items[d]]),a+=h,c=c||h!==0}if(!c){a!==0&&this.eventuallyUpdateScrollDimensions();const d=Rn.relativeComplement(n,l);for(const u of d)for(let g=u.start;g<u.end;g++)this.items[g].row&&this.removeItemFromDOM(g);const h=Rn.relativeComplement(l,n).reverse();for(const u of h)for(let g=u.end-1;g>=u.start;g--)this.insertItemInDOM(g);for(let u=l.start;u<l.end;u++)this.items[u].row&&this.updateItemInDOM(this.items[u],u);if(typeof s=="number"){const u=this.scrollable.getFutureScrollPosition().scrollTop-e,g=this.elementTop(s)-r+u;this.setScrollTop(g,i)}this._onDidChangeContentHeight.fire(this.contentHeight);return}}}probeDynamicHeight(e){var r,a,l;const t=this.items[e];if(this.virtualDelegate.getDynamicHeight){const c=this.virtualDelegate.getDynamicHeight(t.element);if(c!==null){const d=t.size;return t.size=c,t.lastDynamicHeightWidth=this.renderWidth,c-d}}if(!t.hasDynamicHeight||t.lastDynamicHeightWidth===this.renderWidth||this.virtualDelegate.hasDynamicHeight&&!this.virtualDelegate.hasDynamicHeight(t.element))return 0;const i=t.size;if(t.row)return t.row.domNode.style.height="",t.size=t.row.domNode.offsetHeight,t.size===0&&!Zi(t.row.domNode,xe(t.row.domNode).document.body)&&console.warn("Measuring item node that is not in DOM! Add ListView to the DOM before measuring row height!",new Error().stack),t.lastDynamicHeightWidth=this.renderWidth,t.size-i;const{row:n}=this.cache.alloc(t.templateId);n.domNode.style.height="",this.rowsContainer.appendChild(n.domNode);const s=this.renderers.get(t.templateId);if(!s)throw new bt("Missing renderer for templateId: "+t.templateId);return s.renderElement(t.element,e,n.templateData,void 0),t.size=n.domNode.offsetHeight,(r=s.disposeElement)==null||r.call(s,t.element,e,n.templateData,void 0),(l=(a=this.virtualDelegate).setDynamicHeight)==null||l.call(a,t.element,t.size),t.lastDynamicHeightWidth=this.renderWidth,n.domNode.remove(),this.cache.release(n),t.size-i}getElementDomId(e){return`${this.domId}_${e}`}dispose(){var e,t,i;for(const n of this.items)if(n.dragStartDisposable.dispose(),n.checkedDisposable.dispose(),n.row){const s=this.renderers.get(n.row.templateId);s&&((e=s.disposeElement)==null||e.call(s,n.element,-1,n.row.templateData,void 0),s.disposeTemplate(n.row.templateData))}this.items=[],(t=this.domNode)==null||t.remove(),(i=this.dragOverAnimationDisposable)==null||i.dispose(),this.disposables.dispose()}};zk.InstanceCount=0;let Qr=zk;gd([Ni],Qr.prototype,"onMouseClick",null);gd([Ni],Qr.prototype,"onMouseDblClick",null);gd([Ni],Qr.prototype,"onMouseMiddleClick",null);gd([Ni],Qr.prototype,"onMouseDown",null);gd([Ni],Qr.prototype,"onMouseOver",null);gd([Ni],Qr.prototype,"onMouseOut",null);gd([Ni],Qr.prototype,"onContextMenu",null);gd([Ni],Qr.prototype,"onTouchStart",null);gd([Ni],Qr.prototype,"onTap",null);var ou=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s};class bde{constructor(e){this.trait=e,this.renderedElements=[]}get templateId(){return`template:${this.trait.name}`}renderTemplate(e){return e}renderElement(e,t,i){const n=this.renderedElements.findIndex(s=>s.templateData===i);if(n>=0){const s=this.renderedElements[n];this.trait.unrender(i),s.index=t}else{const s={index:t,templateData:i};this.renderedElements.push(s)}this.trait.renderIndex(t,i)}splice(e,t,i){const n=[];for(const s of this.renderedElements)s.index<e?n.push(s):s.index>=e+t&&n.push({index:s.index+i-t,templateData:s.templateData});this.renderedElements=n}renderIndexes(e){for(const{index:t,templateData:i}of this.renderedElements)e.indexOf(t)>-1&&this.trait.renderIndex(t,i)}disposeTemplate(e){const t=this.renderedElements.findIndex(i=>i.templateData===e);t<0||this.renderedElements.splice(t,1)}}let SL=class{get name(){return this._trait}get renderer(){return new bde(this)}constructor(e){this._trait=e,this.indexes=[],this.sortedIndexes=[],this._onChange=new P,this.onChange=this._onChange.event}splice(e,t,i){const n=i.length-t,s=e+t,r=[];let a=0;for(;a<this.sortedIndexes.length&&this.sortedIndexes[a]<e;)r.push(this.sortedIndexes[a++]);for(let l=0;l<i.length;l++)i[l]&&r.push(l+e);for(;a<this.sortedIndexes.length&&this.sortedIndexes[a]>=s;)r.push(this.sortedIndexes[a++]+n);this.renderer.splice(e,t,i.length),this._set(r,r)}renderIndex(e,t){t.classList.toggle(this._trait,this.contains(e))}unrender(e){e.classList.remove(this._trait)}set(e,t){return this._set(e,[...e].sort(d8),t)}_set(e,t,i){const n=this.indexes,s=this.sortedIndexes;this.indexes=e,this.sortedIndexes=t;const r=j2(s,e);return this.renderer.renderIndexes(r),this._onChange.fire({indexes:e,browserEvent:i}),n}get(){return this.indexes}contains(e){return Zv(this.sortedIndexes,e,d8)>=0}dispose(){Tt(this._onChange)}};ou([Ni],SL.prototype,"renderer",null);class Cde extends SL{constructor(e){super("selected"),this.setAriaSelected=e}renderIndex(e,t){super.renderIndex(e,t),this.setAriaSelected&&(this.contains(e)?t.setAttribute("aria-selected","true"):t.setAttribute("aria-selected","false"))}}class GN{constructor(e,t,i){this.trait=e,this.view=t,this.identityProvider=i}splice(e,t,i){if(!this.identityProvider)return this.trait.splice(e,t,new Array(i.length).fill(!1));const n=this.trait.get().map(a=>this.identityProvider.getId(this.view.element(a)).toString());if(n.length===0)return this.trait.splice(e,t,new Array(i.length).fill(!1));const s=new Set(n),r=i.map(a=>s.has(this.identityProvider.getId(a).toString()));this.trait.splice(e,t,r)}}function uh(o){return o.tagName==="INPUT"||o.tagName==="TEXTAREA"}function Q1(o,e){return o.classList.contains(e)?!0:o.classList.contains("monaco-list")||!o.parentElement?!1:Q1(o.parentElement,e)}function DC(o){return Q1(o,"monaco-editor")}function vde(o){return Q1(o,"monaco-custom-toggle")}function wde(o){return Q1(o,"action-item")}function QC(o){return Q1(o,"monaco-tree-sticky-row")}function Ew(o){return o.classList.contains("monaco-tree-sticky-container")}function Ij(o){return o.tagName==="A"&&o.classList.contains("monaco-button")||o.tagName==="DIV"&&o.classList.contains("monaco-button-dropdown")?!0:o.classList.contains("monaco-list")||!o.parentElement?!1:Ij(o.parentElement)}class Ej{get onKeyDown(){return oe.chain(this.disposables.add(new tt(this.view.domNode,"keydown")).event,e=>e.filter(t=>!uh(t.target)).map(t=>new Pt(t)))}constructor(e,t,i){this.list=e,this.view=t,this.disposables=new K,this.multipleSelectionDisposables=new K,this.multipleSelectionSupport=i.multipleSelectionSupport,this.disposables.add(this.onKeyDown(n=>{switch(n.keyCode){case 3:return this.onEnter(n);case 16:return this.onUpArrow(n);case 18:return this.onDownArrow(n);case 11:return this.onPageUpArrow(n);case 12:return this.onPageDownArrow(n);case 9:return this.onEscape(n);case 31:this.multipleSelectionSupport&&(Je?n.metaKey:n.ctrlKey)&&this.onCtrlA(n)}}))}updateOptions(e){e.multipleSelectionSupport!==void 0&&(this.multipleSelectionSupport=e.multipleSelectionSupport)}onEnter(e){e.preventDefault(),e.stopPropagation(),this.list.setSelection(this.list.getFocus(),e.browserEvent)}onUpArrow(e){e.preventDefault(),e.stopPropagation(),this.list.focusPrevious(1,!1,e.browserEvent);const t=this.list.getFocus()[0];this.list.setAnchor(t),this.list.reveal(t),this.view.domNode.focus()}onDownArrow(e){e.preventDefault(),e.stopPropagation(),this.list.focusNext(1,!1,e.browserEvent);const t=this.list.getFocus()[0];this.list.setAnchor(t),this.list.reveal(t),this.view.domNode.focus()}onPageUpArrow(e){e.preventDefault(),e.stopPropagation(),this.list.focusPreviousPage(e.browserEvent);const t=this.list.getFocus()[0];this.list.setAnchor(t),this.list.reveal(t),this.view.domNode.focus()}onPageDownArrow(e){e.preventDefault(),e.stopPropagation(),this.list.focusNextPage(e.browserEvent);const t=this.list.getFocus()[0];this.list.setAnchor(t),this.list.reveal(t),this.view.domNode.focus()}onCtrlA(e){e.preventDefault(),e.stopPropagation(),this.list.setSelection(Is(this.list.length),e.browserEvent),this.list.setAnchor(void 0),this.view.domNode.focus()}onEscape(e){this.list.getSelection().length&&(e.preventDefault(),e.stopPropagation(),this.list.setSelection([],e.browserEvent),this.list.setAnchor(void 0),this.view.domNode.focus())}dispose(){this.disposables.dispose(),this.multipleSelectionDisposables.dispose()}}ou([Ni],Ej.prototype,"onKeyDown",null);var Ll;(function(o){o[o.Automatic=0]="Automatic",o[o.Trigger=1]="Trigger"})(Ll||(Ll={}));var ap;(function(o){o[o.Idle=0]="Idle",o[o.Typing=1]="Typing"})(ap||(ap={}));const Sde=new class{mightProducePrintableCharacter(o){return o.ctrlKey||o.metaKey||o.altKey?!1:o.keyCode>=31&&o.keyCode<=56||o.keyCode>=21&&o.keyCode<=30||o.keyCode>=98&&o.keyCode<=107||o.keyCode>=85&&o.keyCode<=95}};class yde{constructor(e,t,i,n,s){this.list=e,this.view=t,this.keyboardNavigationLabelProvider=i,this.keyboardNavigationEventFilter=n,this.delegate=s,this.enabled=!1,this.state=ap.Idle,this.mode=Ll.Automatic,this.triggered=!1,this.previouslyFocused=-1,this.enabledDisposables=new K,this.disposables=new K,this.updateOptions(e.options)}updateOptions(e){e.typeNavigationEnabled??!0?this.enable():this.disable(),this.mode=e.typeNavigationMode??Ll.Automatic}enable(){if(this.enabled)return;let e=!1;const t=oe.chain(this.enabledDisposables.add(new tt(this.view.domNode,"keydown")).event,s=>s.filter(r=>!uh(r.target)).filter(()=>this.mode===Ll.Automatic||this.triggered).map(r=>new Pt(r)).filter(r=>e||this.keyboardNavigationEventFilter(r)).filter(r=>this.delegate.mightProducePrintableCharacter(r)).forEach(r=>nt.stop(r,!0)).map(r=>r.browserEvent.key)),i=oe.debounce(t,()=>null,800,void 0,void 0,void 0,this.enabledDisposables);oe.reduce(oe.any(t,i),(s,r)=>r===null?null:(s||"")+r,void 0,this.enabledDisposables)(this.onInput,this,this.enabledDisposables),i(this.onClear,this,this.enabledDisposables),t(()=>e=!0,void 0,this.enabledDisposables),i(()=>e=!1,void 0,this.enabledDisposables),this.enabled=!0,this.triggered=!1}disable(){this.enabled&&(this.enabledDisposables.clear(),this.enabled=!1,this.triggered=!1)}onClear(){var t;const e=this.list.getFocus();if(e.length>0&&e[0]===this.previouslyFocused){const i=(t=this.list.options.accessibilityProvider)==null?void 0:t.getAriaLabel(this.list.element(e[0]));typeof i=="string"?Cs(i):i&&Cs(i.get())}this.previouslyFocused=-1}onInput(e){if(!e){this.state=ap.Idle,this.triggered=!1;return}const t=this.list.getFocus(),i=t.length>0?t[0]:0,n=this.state===ap.Idle?1:0;this.state=ap.Typing;for(let s=0;s<this.list.length;s++){const r=(i+s+n)%this.list.length,a=this.keyboardNavigationLabelProvider.getKeyboardNavigationLabel(this.view.element(r)),l=a&&a.toString();if(this.list.options.typeNavigationEnabled){if(typeof l<"u"){if(Sw(e,l)){this.previouslyFocused=i,this.list.setFocus([r]),this.list.reveal(r);return}const c=$ae(e,l);if(c&&c[0].end-c[0].start>1&&c.length===1){this.previouslyFocused=i,this.list.setFocus([r]),this.list.reveal(r);return}}}else if(typeof l>"u"||Sw(e,l)){this.previouslyFocused=i,this.list.setFocus([r]),this.list.reveal(r);return}}}dispose(){this.disable(),this.enabledDisposables.dispose(),this.disposables.dispose()}}class Lde{constructor(e,t){this.list=e,this.view=t,this.disposables=new K;const i=oe.chain(this.disposables.add(new tt(t.domNode,"keydown")).event,s=>s.filter(r=>!uh(r.target)).map(r=>new Pt(r)));oe.chain(i,s=>s.filter(r=>r.keyCode===2&&!r.ctrlKey&&!r.metaKey&&!r.shiftKey&&!r.altKey))(this.onTab,this,this.disposables)}onTab(e){if(e.target!==this.view.domNode)return;const t=this.list.getFocus();if(t.length===0)return;const i=this.view.domElement(t[0]);if(!i)return;const n=i.querySelector("[tabIndex]");if(!n||!Xi(n)||n.tabIndex===-1)return;const s=xe(n).getComputedStyle(n);s.visibility==="hidden"||s.display==="none"||(e.preventDefault(),e.stopPropagation(),n.focus())}dispose(){this.disposables.dispose()}}function Nj(o){return Je?o.browserEvent.metaKey:o.browserEvent.ctrlKey}function Tj(o){return o.browserEvent.shiftKey}function xde(o){return l5(o)&&o.button===2}const c8={isSelectionSingleChangeEvent:Nj,isSelectionRangeChangeEvent:Tj};class Rj{constructor(e){this.list=e,this.disposables=new K,this._onPointer=new P,this.onPointer=this._onPointer.event,e.options.multipleSelectionSupport!==!1&&(this.multipleSelectionController=this.list.options.multipleSelectionController||c8),this.mouseSupport=typeof e.options.mouseSupport>"u"||!!e.options.mouseSupport,this.mouseSupport&&(e.onMouseDown(this.onMouseDown,this,this.disposables),e.onContextMenu(this.onContextMenu,this,this.disposables),e.onMouseDblClick(this.onDoubleClick,this,this.disposables),e.onTouchStart(this.onMouseDown,this,this.disposables),this.disposables.add(Fn.addTarget(e.getHTMLElement()))),oe.any(e.onMouseClick,e.onMouseMiddleClick,e.onTap)(this.onViewPointer,this,this.disposables)}updateOptions(e){e.multipleSelectionSupport!==void 0&&(this.multipleSelectionController=void 0,e.multipleSelectionSupport&&(this.multipleSelectionController=this.list.options.multipleSelectionController||c8))}isSelectionSingleChangeEvent(e){return this.multipleSelectionController?this.multipleSelectionController.isSelectionSingleChangeEvent(e):!1}isSelectionRangeChangeEvent(e){return this.multipleSelectionController?this.multipleSelectionController.isSelectionRangeChangeEvent(e):!1}isSelectionChangeEvent(e){return this.isSelectionSingleChangeEvent(e)||this.isSelectionRangeChangeEvent(e)}onMouseDown(e){DC(e.browserEvent.target)||Dn()!==e.browserEvent.target&&this.list.domFocus()}onContextMenu(e){if(uh(e.browserEvent.target)||DC(e.browserEvent.target))return;const t=typeof e.index>"u"?[]:[e.index];this.list.setFocus(t,e.browserEvent)}onViewPointer(e){if(!this.mouseSupport||uh(e.browserEvent.target)||DC(e.browserEvent.target)||e.browserEvent.isHandledByList)return;e.browserEvent.isHandledByList=!0;const t=e.index;if(typeof t>"u"){this.list.setFocus([],e.browserEvent),this.list.setSelection([],e.browserEvent),this.list.setAnchor(void 0);return}if(this.isSelectionChangeEvent(e))return this.changeSelection(e);this.list.setFocus([t],e.browserEvent),this.list.setAnchor(t),xde(e.browserEvent)||this.list.setSelection([t],e.browserEvent),this._onPointer.fire(e)}onDoubleClick(e){if(uh(e.browserEvent.target)||DC(e.browserEvent.target)||this.isSelectionChangeEvent(e)||e.browserEvent.isHandledByList)return;e.browserEvent.isHandledByList=!0;const t=this.list.getFocus();this.list.setSelection(t,e.browserEvent)}changeSelection(e){const t=e.index;let i=this.list.getAnchor();if(this.isSelectionRangeChangeEvent(e)){typeof i>"u"&&(i=this.list.getFocus()[0]??t,this.list.setAnchor(i));const n=Math.min(i,t),s=Math.max(i,t),r=Is(n,s+1),a=this.list.getSelection(),l=Ide(j2(a,[i]),i);if(l.length===0)return;const c=j2(r,Ede(a,l));this.list.setSelection(c,e.browserEvent),this.list.setFocus([t],e.browserEvent)}else if(this.isSelectionSingleChangeEvent(e)){const n=this.list.getSelection(),s=n.filter(r=>r!==t);this.list.setFocus([t]),this.list.setAnchor(t),n.length===s.length?this.list.setSelection([...s,t],e.browserEvent):this.list.setSelection(s,e.browserEvent)}}dispose(){this.disposables.dispose()}}class Mj{constructor(e,t){this.styleElement=e,this.selectorSuffix=t}style(e){const t=this.selectorSuffix&&`.${this.selectorSuffix}`,i=[];e.listBackground&&i.push(`.monaco-list${t} .monaco-list-rows { background: ${e.listBackground}; }`),e.listFocusBackground&&(i.push(`.monaco-list${t}:focus .monaco-list-row.focused { background-color: ${e.listFocusBackground}; }`),i.push(`.monaco-list${t}:focus .monaco-list-row.focused:hover { background-color: ${e.listFocusBackground}; }`)),e.listFocusForeground&&i.push(`.monaco-list${t}:focus .monaco-list-row.focused { color: ${e.listFocusForeground}; }`),e.listActiveSelectionBackground&&(i.push(`.monaco-list${t}:focus .monaco-list-row.selected { background-color: ${e.listActiveSelectionBackground}; }`),i.push(`.monaco-list${t}:focus .monaco-list-row.selected:hover { background-color: ${e.listActiveSelectionBackground}; }`)),e.listActiveSelectionForeground&&i.push(`.monaco-list${t}:focus .monaco-list-row.selected { color: ${e.listActiveSelectionForeground}; }`),e.listActiveSelectionIconForeground&&i.push(`.monaco-list${t}:focus .monaco-list-row.selected .codicon { color: ${e.listActiveSelectionIconForeground}; }`),e.listFocusAndSelectionBackground&&i.push(`
				.monaco-drag-image,
				.monaco-list${t}:focus .monaco-list-row.selected.focused { background-color: ${e.listFocusAndSelectionBackground}; }
			`),e.listFocusAndSelectionForeground&&i.push(`
				.monaco-drag-image,
				.monaco-list${t}:focus .monaco-list-row.selected.focused { color: ${e.listFocusAndSelectionForeground}; }
			`),e.listInactiveFocusForeground&&(i.push(`.monaco-list${t} .monaco-list-row.focused { color:  ${e.listInactiveFocusForeground}; }`),i.push(`.monaco-list${t} .monaco-list-row.focused:hover { color:  ${e.listInactiveFocusForeground}; }`)),e.listInactiveSelectionIconForeground&&i.push(`.monaco-list${t} .monaco-list-row.focused .codicon { color:  ${e.listInactiveSelectionIconForeground}; }`),e.listInactiveFocusBackground&&(i.push(`.monaco-list${t} .monaco-list-row.focused { background-color:  ${e.listInactiveFocusBackground}; }`),i.push(`.monaco-list${t} .monaco-list-row.focused:hover { background-color:  ${e.listInactiveFocusBackground}; }`)),e.listInactiveSelectionBackground&&(i.push(`.monaco-list${t} .monaco-list-row.selected { background-color:  ${e.listInactiveSelectionBackground}; }`),i.push(`.monaco-list${t} .monaco-list-row.selected:hover { background-color:  ${e.listInactiveSelectionBackground}; }`)),e.listInactiveSelectionForeground&&i.push(`.monaco-list${t} .monaco-list-row.selected { color: ${e.listInactiveSelectionForeground}; }`),e.listHoverBackground&&i.push(`.monaco-list${t}:not(.drop-target):not(.dragging) .monaco-list-row:hover:not(.selected):not(.focused) { background-color: ${e.listHoverBackground}; }`),e.listHoverForeground&&i.push(`.monaco-list${t}:not(.drop-target):not(.dragging) .monaco-list-row:hover:not(.selected):not(.focused) { color:  ${e.listHoverForeground}; }`);const n=jc(e.listFocusAndSelectionOutline,jc(e.listSelectionOutline,e.listFocusOutline??""));n&&i.push(`.monaco-list${t}:focus .monaco-list-row.focused.selected { outline: 1px solid ${n}; outline-offset: -1px;}`),e.listFocusOutline&&i.push(`
				.monaco-drag-image,
				.monaco-list${t}:focus .monaco-list-row.focused { outline: 1px solid ${e.listFocusOutline}; outline-offset: -1px; }
				.monaco-workbench.context-menu-visible .monaco-list${t}.last-focused .monaco-list-row.focused { outline: 1px solid ${e.listFocusOutline}; outline-offset: -1px; }
			`);const s=jc(e.listSelectionOutline,e.listInactiveFocusOutline??"");s&&i.push(`.monaco-list${t} .monaco-list-row.focused.selected { outline: 1px dotted ${s}; outline-offset: -1px; }`),e.listSelectionOutline&&i.push(`.monaco-list${t} .monaco-list-row.selected { outline: 1px dotted ${e.listSelectionOutline}; outline-offset: -1px; }`),e.listInactiveFocusOutline&&i.push(`.monaco-list${t} .monaco-list-row.focused { outline: 1px dotted ${e.listInactiveFocusOutline}; outline-offset: -1px; }`),e.listHoverOutline&&i.push(`.monaco-list${t} .monaco-list-row:hover { outline: 1px dashed ${e.listHoverOutline}; outline-offset: -1px; }`),e.listDropOverBackground&&i.push(`
				.monaco-list${t}.drop-target,
				.monaco-list${t} .monaco-list-rows.drop-target,
				.monaco-list${t} .monaco-list-row.drop-target { background-color: ${e.listDropOverBackground} !important; color: inherit !important; }
			`),e.listDropBetweenBackground&&(i.push(`
			.monaco-list${t} .monaco-list-rows.drop-target-before .monaco-list-row:first-child::before,
			.monaco-list${t} .monaco-list-row.drop-target-before::before {
				content: ""; position: absolute; top: 0px; left: 0px; width: 100%; height: 1px;
				background-color: ${e.listDropBetweenBackground};
			}`),i.push(`
			.monaco-list${t} .monaco-list-rows.drop-target-after .monaco-list-row:last-child::after,
			.monaco-list${t} .monaco-list-row.drop-target-after::after {
				content: ""; position: absolute; bottom: 0px; left: 0px; width: 100%; height: 1px;
				background-color: ${e.listDropBetweenBackground};
			}`)),e.tableColumnsBorder&&i.push(`
				.monaco-table > .monaco-split-view2,
				.monaco-table > .monaco-split-view2 .monaco-sash.vertical::before,
				.monaco-workbench:not(.reduce-motion) .monaco-table:hover > .monaco-split-view2,
				.monaco-workbench:not(.reduce-motion) .monaco-table:hover > .monaco-split-view2 .monaco-sash.vertical::before {
					border-color: ${e.tableColumnsBorder};
				}

				.monaco-workbench:not(.reduce-motion) .monaco-table > .monaco-split-view2,
				.monaco-workbench:not(.reduce-motion) .monaco-table > .monaco-split-view2 .monaco-sash.vertical::before {
					border-color: transparent;
				}
			`),e.tableOddRowsBackgroundColor&&i.push(`
				.monaco-table .monaco-list-row[data-parity=odd]:not(.focused):not(.selected):not(:hover) .monaco-table-tr,
				.monaco-table .monaco-list:not(:focus) .monaco-list-row[data-parity=odd].focused:not(.selected):not(:hover) .monaco-table-tr,
				.monaco-table .monaco-list:not(.focused) .monaco-list-row[data-parity=odd].focused:not(.selected):not(:hover) .monaco-table-tr {
					background-color: ${e.tableOddRowsBackgroundColor};
				}
			`),this.styleElement.textContent=i.join(`
`)}}const kde={listFocusBackground:"#7FB0D0",listActiveSelectionBackground:"#0E639C",listActiveSelectionForeground:"#FFFFFF",listActiveSelectionIconForeground:"#FFFFFF",listFocusAndSelectionOutline:"#90C2F9",listFocusAndSelectionBackground:"#094771",listFocusAndSelectionForeground:"#FFFFFF",listInactiveSelectionBackground:"#3F3F46",listInactiveSelectionIconForeground:"#FFFFFF",listHoverBackground:"#2A2D2E",listDropOverBackground:"#383B3D",listDropBetweenBackground:"#EEEEEE",treeIndentGuidesStroke:"#a9a9a9",treeInactiveIndentGuidesStroke:Z.fromHex("#a9a9a9").transparent(.4).toString(),tableColumnsBorder:Z.fromHex("#cccccc").transparent(.2).toString(),tableOddRowsBackgroundColor:Z.fromHex("#cccccc").transparent(.04).toString(),listBackground:void 0,listFocusForeground:void 0,listInactiveSelectionForeground:void 0,listInactiveFocusForeground:void 0,listInactiveFocusBackground:void 0,listHoverForeground:void 0,listFocusOutline:void 0,listInactiveFocusOutline:void 0,listSelectionOutline:void 0,listHoverOutline:void 0,treeStickyScrollBackground:void 0,treeStickyScrollBorder:void 0,treeStickyScrollShadow:void 0},Dde={keyboardSupport:!0,mouseSupport:!0,multipleSelectionSupport:!0,dnd:{getDragURI(){return null},onDragStart(){},onDragOver(){return!1},drop(){},dispose(){}}};function Ide(o,e){const t=o.indexOf(e);if(t===-1)return[];const i=[];let n=t-1;for(;n>=0&&o[n]===e-(t-n);)i.push(o[n--]);for(i.reverse(),n=t;n<o.length&&o[n]===e+(n-t);)i.push(o[n++]);return i}function j2(o,e){const t=[];let i=0,n=0;for(;i<o.length||n<e.length;)if(i>=o.length)t.push(e[n++]);else if(n>=e.length)t.push(o[i++]);else if(o[i]===e[n]){t.push(o[i]),i++,n++;continue}else o[i]<e[n]?t.push(o[i++]):t.push(e[n++]);return t}function Ede(o,e){const t=[];let i=0,n=0;for(;i<o.length||n<e.length;)if(i>=o.length)t.push(e[n++]);else if(n>=e.length)t.push(o[i++]);else if(o[i]===e[n]){i++,n++;continue}else o[i]<e[n]?t.push(o[i++]):n++;return t}const d8=(o,e)=>o-e;class Nde{constructor(e,t){this._templateId=e,this.renderers=t}get templateId(){return this._templateId}renderTemplate(e){return this.renderers.map(t=>t.renderTemplate(e))}renderElement(e,t,i,n){let s=0;for(const r of this.renderers)r.renderElement(e,t,i[s++],n)}disposeElement(e,t,i,n){var r;let s=0;for(const a of this.renderers)(r=a.disposeElement)==null||r.call(a,e,t,i[s],n),s+=1}disposeTemplate(e){let t=0;for(const i of this.renderers)i.disposeTemplate(e[t++])}}class Tde{constructor(e){this.accessibilityProvider=e,this.templateId="a18n"}renderTemplate(e){return{container:e,disposables:new K}}renderElement(e,t,i){const n=this.accessibilityProvider.getAriaLabel(e),s=n&&typeof n!="string"?n:ur(n);i.disposables.add(Ae(a=>{this.setAriaLabel(a.readObservable(s),i.container)}));const r=this.accessibilityProvider.getAriaLevel&&this.accessibilityProvider.getAriaLevel(e);typeof r=="number"?i.container.setAttribute("aria-level",`${r}`):i.container.removeAttribute("aria-level")}setAriaLabel(e,t){e?t.setAttribute("aria-label",e):t.removeAttribute("aria-label")}disposeElement(e,t,i,n){i.disposables.clear()}disposeTemplate(e){e.disposables.dispose()}}class Rde{constructor(e,t){this.list=e,this.dnd=t}getDragElements(e){const t=this.list.getSelectedElements();return t.indexOf(e)>-1?t:[e]}getDragURI(e){return this.dnd.getDragURI(e)}getDragLabel(e,t){if(this.dnd.getDragLabel)return this.dnd.getDragLabel(e,t)}onDragStart(e,t){var i,n;(n=(i=this.dnd).onDragStart)==null||n.call(i,e,t)}onDragOver(e,t,i,n,s){return this.dnd.onDragOver(e,t,i,n,s)}onDragLeave(e,t,i,n){var s,r;(r=(s=this.dnd).onDragLeave)==null||r.call(s,e,t,i,n)}onDragEnd(e){var t,i;(i=(t=this.dnd).onDragEnd)==null||i.call(t,e)}drop(e,t,i,n,s){this.dnd.drop(e,t,i,n,s)}dispose(){this.dnd.dispose()}}class Xo{get onDidChangeFocus(){return oe.map(this.eventBufferer.wrapEvent(this.focus.onChange),e=>this.toListEvent(e),this.disposables)}get onDidChangeSelection(){return oe.map(this.eventBufferer.wrapEvent(this.selection.onChange),e=>this.toListEvent(e),this.disposables)}get domId(){return this.view.domId}get onDidScroll(){return this.view.onDidScroll}get onMouseClick(){return this.view.onMouseClick}get onMouseDblClick(){return this.view.onMouseDblClick}get onMouseMiddleClick(){return this.view.onMouseMiddleClick}get onPointer(){return this.mouseController.onPointer}get onMouseDown(){return this.view.onMouseDown}get onMouseOver(){return this.view.onMouseOver}get onMouseOut(){return this.view.onMouseOut}get onTouchStart(){return this.view.onTouchStart}get onTap(){return this.view.onTap}get onContextMenu(){let e=!1;const t=oe.chain(this.disposables.add(new tt(this.view.domNode,"keydown")).event,s=>s.map(r=>new Pt(r)).filter(r=>e=r.keyCode===58||r.shiftKey&&r.keyCode===68).map(r=>nt.stop(r,!0)).filter(()=>!1)),i=oe.chain(this.disposables.add(new tt(this.view.domNode,"keyup")).event,s=>s.forEach(()=>e=!1).map(r=>new Pt(r)).filter(r=>r.keyCode===58||r.shiftKey&&r.keyCode===68).map(r=>nt.stop(r,!0)).map(({browserEvent:r})=>{const a=this.getFocus(),l=a.length?a[0]:void 0,c=typeof l<"u"?this.view.element(l):void 0,d=typeof l<"u"?this.view.domElement(l):this.view.domNode;return{index:l,element:c,anchor:d,browserEvent:r}})),n=oe.chain(this.view.onContextMenu,s=>s.filter(r=>!e).map(({element:r,index:a,browserEvent:l})=>({element:r,index:a,anchor:new Vr(xe(this.view.domNode),l),browserEvent:l})));return oe.any(t,i,n)}get onKeyDown(){return this.disposables.add(new tt(this.view.domNode,"keydown")).event}get onDidFocus(){return oe.signal(this.disposables.add(new tt(this.view.domNode,"focus",!0)).event)}get onDidBlur(){return oe.signal(this.disposables.add(new tt(this.view.domNode,"blur",!0)).event)}constructor(e,t,i,n,s=Dde){var c,d,h;this.user=e,this._options=s,this.focus=new SL("focused"),this.anchor=new SL("anchor"),this.eventBufferer=new R1,this._ariaLabel="",this.disposables=new K,this._onDidDispose=new P,this.onDidDispose=this._onDidDispose.event;const r=this._options.accessibilityProvider&&this._options.accessibilityProvider.getWidgetRole?(c=this._options.accessibilityProvider)==null?void 0:c.getWidgetRole():"list";this.selection=new Cde(r!=="listbox");const a=[this.focus.renderer,this.selection.renderer];this.accessibilityProvider=s.accessibilityProvider,this.accessibilityProvider&&(a.push(new Tde(this.accessibilityProvider)),(h=(d=this.accessibilityProvider).onDidChangeActiveDescendant)==null||h.call(d,this.onDidChangeActiveDescendant,this,this.disposables)),n=n.map(u=>new Nde(u.templateId,[...a,u]));const l={...s,dnd:s.dnd&&new Rde(this,s.dnd)};if(this.view=this.createListView(t,i,n,l),this.view.domNode.setAttribute("role",r),s.styleController)this.styleController=s.styleController(this.view.domId);else{const u=$o(this.view.domNode);this.styleController=new Mj(u,this.view.domId)}if(this.spliceable=new cde([new GN(this.focus,this.view,s.identityProvider),new GN(this.selection,this.view,s.identityProvider),new GN(this.anchor,this.view,s.identityProvider),this.view]),this.disposables.add(this.focus),this.disposables.add(this.selection),this.disposables.add(this.anchor),this.disposables.add(this.view),this.disposables.add(this._onDidDispose),this.disposables.add(new Lde(this,this.view)),(typeof s.keyboardSupport!="boolean"||s.keyboardSupport)&&(this.keyboardController=new Ej(this,this.view,s),this.disposables.add(this.keyboardController)),s.keyboardNavigationLabelProvider){const u=s.keyboardNavigationDelegate||Sde;this.typeNavigationController=new yde(this,this.view,s.keyboardNavigationLabelProvider,s.keyboardNavigationEventFilter??(()=>!0),u),this.disposables.add(this.typeNavigationController)}this.mouseController=this.createMouseController(s),this.disposables.add(this.mouseController),this.onDidChangeFocus(this._onFocusChange,this,this.disposables),this.onDidChangeSelection(this._onSelectionChange,this,this.disposables),this.accessibilityProvider&&(this.ariaLabel=this.accessibilityProvider.getWidgetAriaLabel()),this._options.multipleSelectionSupport!==!1&&this.view.domNode.setAttribute("aria-multiselectable","true")}createListView(e,t,i,n){return new Qr(e,t,i,n)}createMouseController(e){return new Rj(this)}updateOptions(e={}){var t,i;this._options={...this._options,...e},(t=this.typeNavigationController)==null||t.updateOptions(this._options),this._options.multipleSelectionController!==void 0&&(this._options.multipleSelectionSupport?this.view.domNode.setAttribute("aria-multiselectable","true"):this.view.domNode.removeAttribute("aria-multiselectable")),this.mouseController.updateOptions(e),(i=this.keyboardController)==null||i.updateOptions(e),this.view.updateOptions(e)}get options(){return this._options}splice(e,t,i=[]){if(e<0||e>this.view.length)throw new bu(this.user,`Invalid start index: ${e}`);if(t<0)throw new bu(this.user,`Invalid delete count: ${t}`);t===0&&i.length===0||this.eventBufferer.bufferEvents(()=>this.spliceable.splice(e,t,i))}rerender(){this.view.rerender()}element(e){return this.view.element(e)}indexOf(e){return this.view.indexOf(e)}indexAt(e){return this.view.indexAt(e)}get length(){return this.view.length}get contentHeight(){return this.view.contentHeight}get onDidChangeContentHeight(){return this.view.onDidChangeContentHeight}get scrollTop(){return this.view.getScrollTop()}set scrollTop(e){this.view.setScrollTop(e)}get scrollHeight(){return this.view.scrollHeight}get renderHeight(){return this.view.renderHeight}get firstVisibleIndex(){return this.view.firstVisibleIndex}get ariaLabel(){return this._ariaLabel}set ariaLabel(e){this._ariaLabel=e,this.view.domNode.setAttribute("aria-label",e)}domFocus(){this.view.domNode.focus({preventScroll:!0})}layout(e,t){this.view.layout(e,t)}setSelection(e,t){for(const i of e)if(i<0||i>=this.length)throw new bu(this.user,`Invalid index ${i}`);this.selection.set(e,t)}getSelection(){return this.selection.get()}getSelectedElements(){return this.getSelection().map(e=>this.view.element(e))}setAnchor(e){if(typeof e>"u"){this.anchor.set([]);return}if(e<0||e>=this.length)throw new bu(this.user,`Invalid index ${e}`);this.anchor.set([e])}getAnchor(){return O4(this.anchor.get(),void 0)}getAnchorElement(){const e=this.getAnchor();return typeof e>"u"?void 0:this.element(e)}setFocus(e,t){for(const i of e)if(i<0||i>=this.length)throw new bu(this.user,`Invalid index ${i}`);this.focus.set(e,t)}focusNext(e=1,t=!1,i,n){if(this.length===0)return;const s=this.focus.get(),r=this.findNextIndex(s.length>0?s[0]+e:0,t,n);r>-1&&this.setFocus([r],i)}focusPrevious(e=1,t=!1,i,n){if(this.length===0)return;const s=this.focus.get(),r=this.findPreviousIndex(s.length>0?s[0]-e:0,t,n);r>-1&&this.setFocus([r],i)}async focusNextPage(e,t){let i=this.view.indexAt(this.view.getScrollTop()+this.view.renderHeight);i=i===0?0:i-1;const n=this.getFocus()[0];if(n!==i&&(n===void 0||i>n)){const s=this.findPreviousIndex(i,!1,t);s>-1&&n!==s?this.setFocus([s],e):this.setFocus([i],e)}else{const s=this.view.getScrollTop();let r=s+this.view.renderHeight;i>n&&(r-=this.view.elementHeight(i)),this.view.setScrollTop(r),this.view.getScrollTop()!==s&&(this.setFocus([]),await ed(0),await this.focusNextPage(e,t))}}async focusPreviousPage(e,t,i=()=>0){let n;const s=i(),r=this.view.getScrollTop()+s;r===0?n=this.view.indexAt(r):n=this.view.indexAfter(r-1);const a=this.getFocus()[0];if(a!==n&&(a===void 0||a>=n)){const l=this.findNextIndex(n,!1,t);l>-1&&a!==l?this.setFocus([l],e):this.setFocus([n],e)}else{const l=r;this.view.setScrollTop(r-this.view.renderHeight-s),this.view.getScrollTop()+i()!==l&&(this.setFocus([]),await ed(0),await this.focusPreviousPage(e,t,i))}}focusLast(e,t){if(this.length===0)return;const i=this.findPreviousIndex(this.length-1,!1,t);i>-1&&this.setFocus([i],e)}focusFirst(e,t){this.focusNth(0,e,t)}focusNth(e,t,i){if(this.length===0)return;const n=this.findNextIndex(e,!1,i);n>-1&&this.setFocus([n],t)}findNextIndex(e,t=!1,i){for(let n=0;n<this.length;n++){if(e>=this.length&&!t)return-1;if(e=e%this.length,!i||i(this.element(e)))return e;e++}return-1}findPreviousIndex(e,t=!1,i){for(let n=0;n<this.length;n++){if(e<0&&!t)return-1;if(e=(this.length+e%this.length)%this.length,!i||i(this.element(e)))return e;e--}return-1}getFocus(){return this.focus.get()}getFocusedElements(){return this.getFocus().map(e=>this.view.element(e))}reveal(e,t,i=0){if(e<0||e>=this.length)throw new bu(this.user,`Invalid index ${e}`);const n=this.view.getScrollTop(),s=this.view.elementTop(e),r=this.view.elementHeight(e);if(Rh(t)){const a=r-this.view.renderHeight+i;this.view.setScrollTop(a*os(t,0,1)+s-i)}else{const a=s+r,l=n+this.view.renderHeight;s<n+i&&a>=l||(s<n+i||a>=l&&r>=this.view.renderHeight?this.view.setScrollTop(s-i):a>=l&&this.view.setScrollTop(a-this.view.renderHeight))}}getRelativeTop(e,t=0){if(e<0||e>=this.length)throw new bu(this.user,`Invalid index ${e}`);const i=this.view.getScrollTop(),n=this.view.elementTop(e),s=this.view.elementHeight(e);if(n<i+t||n+s>i+this.view.renderHeight)return null;const r=s-this.view.renderHeight+t;return Math.abs((i+t-n)/r)}getHTMLElement(){return this.view.domNode}getScrollableElement(){return this.view.scrollableElementDomNode}getElementID(e){return this.view.getElementDomId(e)}getElementTop(e){return this.view.elementTop(e)}style(e){this.styleController.style(e)}toListEvent({indexes:e,browserEvent:t}){return{indexes:e,elements:e.map(i=>this.view.element(i)),browserEvent:t}}_onFocusChange(){const e=this.focus.get();this.view.domNode.classList.toggle("element-focused",e.length>0),this.onDidChangeActiveDescendant()}onDidChangeActiveDescendant(){var t;const e=this.focus.get();if(e.length>0){let i;(t=this.accessibilityProvider)!=null&&t.getActiveDescendantId&&(i=this.accessibilityProvider.getActiveDescendantId(this.view.element(e[0]))),this.view.domNode.setAttribute("aria-activedescendant",i||this.view.getElementDomId(e[0]))}else this.view.domNode.removeAttribute("aria-activedescendant")}_onSelectionChange(){const e=this.selection.get();this.view.domNode.classList.toggle("selection-none",e.length===0),this.view.domNode.classList.toggle("selection-single",e.length===1),this.view.domNode.classList.toggle("selection-multiple",e.length>1)}dispose(){this._onDidDispose.fire(),this.disposables.dispose(),this._onDidDispose.dispose()}}ou([Ni],Xo.prototype,"onDidChangeFocus",null);ou([Ni],Xo.prototype,"onDidChangeSelection",null);ou([Ni],Xo.prototype,"onContextMenu",null);ou([Ni],Xo.prototype,"onKeyDown",null);ou([Ni],Xo.prototype,"onDidFocus",null);ou([Ni],Xo.prototype,"onDidBlur",null);const lg=ne,Aj="selectOption.entry.template";class Mde{get templateId(){return Aj}renderTemplate(e){const t=Object.create(null);return t.root=e,t.text=X(e,lg(".option-text")),t.detail=X(e,lg(".option-detail")),t.decoratorRight=X(e,lg(".option-decorator-right")),t}renderElement(e,t,i){const n=i,s=e.text,r=e.detail,a=e.decoratorRight,l=e.isDisabled;n.text.textContent=s,n.detail.textContent=r||"",n.decoratorRight.innerText=a||"",l?n.root.classList.add("option-disabled"):n.root.classList.remove("option-disabled")}disposeTemplate(e){}}const hl=class hl extends F{constructor(e,t,i,n,s){super(),this.options=[],this._currentSelection=0,this._hasDetails=!1,this._skipLayout=!1,this._sticky=!1,this._isVisible=!1,this.styles=n,this.selectBoxOptions=s||Object.create(null),typeof this.selectBoxOptions.minBottomMargin!="number"?this.selectBoxOptions.minBottomMargin=hl.DEFAULT_DROPDOWN_MINIMUM_BOTTOM_MARGIN:this.selectBoxOptions.minBottomMargin<0&&(this.selectBoxOptions.minBottomMargin=0),this.selectElement=document.createElement("select"),this.selectElement.className="monaco-select-box monaco-select-box-dropdown-padding",typeof this.selectBoxOptions.ariaLabel=="string"&&this.selectElement.setAttribute("aria-label",this.selectBoxOptions.ariaLabel),typeof this.selectBoxOptions.ariaDescription=="string"&&this.selectElement.setAttribute("aria-description",this.selectBoxOptions.ariaDescription),this._onDidSelect=new P,this._register(this._onDidSelect),this.registerListeners(),this.constructSelectDropDown(i),this.selected=t||0,e&&this.setOptions(e,t),this.initStyleSheet()}setTitle(e){!this._hover&&e?this._hover=this._register(ja().setupManagedHover(Ss("mouse"),this.selectElement,e)):this._hover&&this._hover.update(e)}getHeight(){return 22}getTemplateId(){return Aj}constructSelectDropDown(e){this.contextViewProvider=e,this.selectDropDownContainer=ne(".monaco-select-box-dropdown-container"),this.selectDropDownContainer.classList.add("monaco-select-box-dropdown-padding"),this.selectionDetailsPane=X(this.selectDropDownContainer,lg(".select-box-details-pane"));const t=X(this.selectDropDownContainer,lg(".select-box-dropdown-container-width-control")),i=X(t,lg(".width-control-div"));this.widthControlElement=document.createElement("span"),this.widthControlElement.className="option-text-width-control",X(i,this.widthControlElement),this._dropDownPosition=0,this.styleElement=$o(this.selectDropDownContainer),this.selectDropDownContainer.setAttribute("draggable","true"),this._register(U(this.selectDropDownContainer,ie.DRAG_START,n=>{nt.stop(n,!0)}))}registerListeners(){this._register(hi(this.selectElement,"change",t=>{this.selected=t.target.selectedIndex,this._onDidSelect.fire({index:t.target.selectedIndex,selected:t.target.value}),this.options[this.selected]&&this.options[this.selected].text&&this.setTitle(this.options[this.selected].text)})),this._register(U(this.selectElement,ie.CLICK,t=>{nt.stop(t),this._isVisible?this.hideSelectDropDown(!0):this.showSelectDropDown()})),this._register(U(this.selectElement,ie.MOUSE_DOWN,t=>{nt.stop(t)}));let e;this._register(U(this.selectElement,"touchstart",t=>{e=this._isVisible})),this._register(U(this.selectElement,"touchend",t=>{nt.stop(t),e?this.hideSelectDropDown(!0):this.showSelectDropDown()})),this._register(U(this.selectElement,ie.KEY_DOWN,t=>{const i=new Pt(t);let n=!1;Je?(i.keyCode===18||i.keyCode===16||i.keyCode===10||i.keyCode===3)&&(n=!0):(i.keyCode===18&&i.altKey||i.keyCode===16&&i.altKey||i.keyCode===10||i.keyCode===3)&&(n=!0),n&&(this.showSelectDropDown(),nt.stop(t,!0))}))}get onDidSelect(){return this._onDidSelect.event}setOptions(e,t){ii(this.options,e)||(this.options=e,this.selectElement.options.length=0,this._hasDetails=!1,this._cachedMaxDetailsHeight=void 0,this.options.forEach((i,n)=>{this.selectElement.add(this.createOption(i.text,n,i.isDisabled)),typeof i.description=="string"&&(this._hasDetails=!0)})),t!==void 0&&(this.select(t),this._currentSelection=this.selected)}setOptionsList(){var e;(e=this.selectList)==null||e.splice(0,this.selectList.length,this.options)}select(e){e>=0&&e<this.options.length?this.selected=e:e>this.options.length-1?this.select(this.options.length-1):this.selected<0&&(this.selected=0),this.selectElement.selectedIndex=this.selected,this.options[this.selected]&&this.options[this.selected].text&&this.setTitle(this.options[this.selected].text)}focus(){this.selectElement&&(this.selectElement.tabIndex=0,this.selectElement.focus())}blur(){this.selectElement&&(this.selectElement.tabIndex=-1,this.selectElement.blur())}setFocusable(e){this.selectElement.tabIndex=e?0:-1}render(e){this.container=e,e.classList.add("select-container"),e.appendChild(this.selectElement),this.styleSelectElement()}initStyleSheet(){const e=[];this.styles.listFocusBackground&&e.push(`.monaco-select-box-dropdown-container > .select-box-dropdown-list-container .monaco-list .monaco-list-row.focused { background-color: ${this.styles.listFocusBackground} !important; }`),this.styles.listFocusForeground&&e.push(`.monaco-select-box-dropdown-container > .select-box-dropdown-list-container .monaco-list .monaco-list-row.focused { color: ${this.styles.listFocusForeground} !important; }`),this.styles.decoratorRightForeground&&e.push(`.monaco-select-box-dropdown-container > .select-box-dropdown-list-container .monaco-list .monaco-list-row:not(.focused) .option-decorator-right { color: ${this.styles.decoratorRightForeground}; }`),this.styles.selectBackground&&this.styles.selectBorder&&this.styles.selectBorder!==this.styles.selectBackground?(e.push(`.monaco-select-box-dropdown-container { border: 1px solid ${this.styles.selectBorder} } `),e.push(`.monaco-select-box-dropdown-container > .select-box-details-pane.border-top { border-top: 1px solid ${this.styles.selectBorder} } `),e.push(`.monaco-select-box-dropdown-container > .select-box-details-pane.border-bottom { border-bottom: 1px solid ${this.styles.selectBorder} } `)):this.styles.selectListBorder&&(e.push(`.monaco-select-box-dropdown-container > .select-box-details-pane.border-top { border-top: 1px solid ${this.styles.selectListBorder} } `),e.push(`.monaco-select-box-dropdown-container > .select-box-details-pane.border-bottom { border-bottom: 1px solid ${this.styles.selectListBorder} } `)),this.styles.listHoverForeground&&e.push(`.monaco-select-box-dropdown-container > .select-box-dropdown-list-container .monaco-list .monaco-list-row:not(.option-disabled):not(.focused):hover { color: ${this.styles.listHoverForeground} !important; }`),this.styles.listHoverBackground&&e.push(`.monaco-select-box-dropdown-container > .select-box-dropdown-list-container .monaco-list .monaco-list-row:not(.option-disabled):not(.focused):hover { background-color: ${this.styles.listHoverBackground} !important; }`),this.styles.listFocusOutline&&e.push(`.monaco-select-box-dropdown-container > .select-box-dropdown-list-container .monaco-list .monaco-list-row.focused { outline: 1.6px dotted ${this.styles.listFocusOutline} !important; outline-offset: -1.6px !important; }`),this.styles.listHoverOutline&&e.push(`.monaco-select-box-dropdown-container > .select-box-dropdown-list-container .monaco-list .monaco-list-row:not(.option-disabled):not(.focused):hover { outline: 1.6px dashed ${this.styles.listHoverOutline} !important; outline-offset: -1.6px !important; }`),e.push(".monaco-select-box-dropdown-container > .select-box-dropdown-list-container .monaco-list .monaco-list-row.option-disabled.focused { background-color: transparent !important; color: inherit !important; outline: none !important; }"),e.push(".monaco-select-box-dropdown-container > .select-box-dropdown-list-container .monaco-list .monaco-list-row.option-disabled:hover { background-color: transparent !important; color: inherit !important; outline: none !important; }"),this.styleElement.textContent=e.join(`
`)}styleSelectElement(){const e=this.styles.selectBackground??"",t=this.styles.selectForeground??"",i=this.styles.selectBorder??"";this.selectElement.style.backgroundColor=e,this.selectElement.style.color=t,this.selectElement.style.borderColor=i}styleList(){const e=this.styles.selectBackground??"",t=jc(this.styles.selectListBackground,e);this.selectDropDownListContainer.style.backgroundColor=t,this.selectionDetailsPane.style.backgroundColor=t;const i=this.styles.focusBorder??"";this.selectDropDownContainer.style.outlineColor=i,this.selectDropDownContainer.style.outlineOffset="-1px",this.selectList.style(this.styles)}createOption(e,t,i){const n=document.createElement("option");return n.value=e,n.text=e,n.disabled=!!i,n}showSelectDropDown(){this.selectionDetailsPane.innerText="",!(!this.contextViewProvider||this._isVisible)&&(this.createSelectList(this.selectDropDownContainer),this.setOptionsList(),this.contextViewProvider.showContextView({getAnchor:()=>this.selectElement,render:e=>this.renderSelectDropDown(e,!0),layout:()=>{this.layoutSelectDropDown()},onHide:()=>{this.selectDropDownContainer.classList.remove("visible"),this.selectElement.classList.remove("synthetic-focus")},anchorPosition:this._dropDownPosition},this.selectBoxOptions.optionsAsChildren?this.container:void 0),this._isVisible=!0,this.hideSelectDropDown(!1),this.contextViewProvider.showContextView({getAnchor:()=>this.selectElement,render:e=>this.renderSelectDropDown(e),layout:()=>this.layoutSelectDropDown(),onHide:()=>{this.selectDropDownContainer.classList.remove("visible"),this.selectElement.classList.remove("synthetic-focus")},anchorPosition:this._dropDownPosition},this.selectBoxOptions.optionsAsChildren?this.container:void 0),this._currentSelection=this.selected,this._isVisible=!0,this.selectElement.setAttribute("aria-expanded","true"))}hideSelectDropDown(e){!this.contextViewProvider||!this._isVisible||(this._isVisible=!1,this.selectElement.setAttribute("aria-expanded","false"),e&&this.selectElement.focus(),this.contextViewProvider.hideContextView())}renderSelectDropDown(e,t){return e.appendChild(this.selectDropDownContainer),this.layoutSelectDropDown(t),{dispose:()=>{this.selectDropDownContainer.remove()}}}measureMaxDetailsHeight(){let e=0;return this.options.forEach((t,i)=>{this.updateDetail(i),this.selectionDetailsPane.offsetHeight>e&&(e=this.selectionDetailsPane.offsetHeight)}),e}layoutSelectDropDown(e){if(this._skipLayout)return!1;if(this.selectList){this.selectDropDownContainer.classList.add("visible");const t=xe(this.selectElement),i=wi(this.selectElement),n=xe(this.selectElement).getComputedStyle(this.selectElement),s=parseFloat(n.getPropertyValue("--dropdown-padding-top"))+parseFloat(n.getPropertyValue("--dropdown-padding-bottom")),r=t.innerHeight-i.top-i.height-(this.selectBoxOptions.minBottomMargin||0),a=i.top-hl.DEFAULT_DROPDOWN_MINIMUM_TOP_MARGIN,l=this.selectElement.offsetWidth,c=this.setWidthControlElement(this.widthControlElement),d=Math.max(c,Math.round(l)).toString()+"px";this.selectDropDownContainer.style.width=d,this.selectList.getHTMLElement().style.height="",this.selectList.layout();let h=this.selectList.contentHeight;this._hasDetails&&this._cachedMaxDetailsHeight===void 0&&(this._cachedMaxDetailsHeight=this.measureMaxDetailsHeight());const u=this._hasDetails?this._cachedMaxDetailsHeight:0,g=h+s+u,f=Math.floor((r-s-u)/this.getHeight()),p=Math.floor((a-s-u)/this.getHeight());if(e)return i.top+i.height>t.innerHeight-22||i.top<hl.DEFAULT_DROPDOWN_MINIMUM_TOP_MARGIN||f<1&&p<1?!1:(f<hl.DEFAULT_MINIMUM_VISIBLE_OPTIONS&&p>f&&this.options.length>f?(this._dropDownPosition=1,this.selectDropDownListContainer.remove(),this.selectionDetailsPane.remove(),this.selectDropDownContainer.appendChild(this.selectionDetailsPane),this.selectDropDownContainer.appendChild(this.selectDropDownListContainer),this.selectionDetailsPane.classList.remove("border-top"),this.selectionDetailsPane.classList.add("border-bottom")):(this._dropDownPosition=0,this.selectDropDownListContainer.remove(),this.selectionDetailsPane.remove(),this.selectDropDownContainer.appendChild(this.selectDropDownListContainer),this.selectDropDownContainer.appendChild(this.selectionDetailsPane),this.selectionDetailsPane.classList.remove("border-bottom"),this.selectionDetailsPane.classList.add("border-top")),!0);if(i.top+i.height>t.innerHeight-22||i.top<hl.DEFAULT_DROPDOWN_MINIMUM_TOP_MARGIN||this._dropDownPosition===0&&f<1||this._dropDownPosition===1&&p<1)return this.hideSelectDropDown(!0),!1;if(this._dropDownPosition===0){if(this._isVisible&&f+p<1)return this.hideSelectDropDown(!0),!1;g>r&&(h=f*this.getHeight())}else g>a&&(h=p*this.getHeight());return this.selectList.layout(h),this.selectList.domFocus(),this.selectList.length>0&&(this.selectList.setFocus([this.selected||0]),this.selectList.reveal(this.selectList.getFocus()[0]||0)),this._hasDetails?(this.selectList.getHTMLElement().style.height=h+s+"px",this.selectDropDownContainer.style.height=""):this.selectDropDownContainer.style.height=h+s+"px",this.updateDetail(this.selected),this.selectDropDownContainer.style.width=d,this.selectDropDownListContainer.setAttribute("tabindex","0"),this.selectElement.classList.add("synthetic-focus"),this.selectDropDownContainer.classList.add("synthetic-focus"),!0}else return!1}setWidthControlElement(e){let t=0;if(e){let i=0,n=0;this.options.forEach((s,r)=>{const a=s.detail?s.detail.length:0,l=s.decoratorRight?s.decoratorRight.length:0,c=s.text.length+a+l;c>n&&(i=r,n=c)}),e.textContent=this.options[i].text+(this.options[i].decoratorRight?this.options[i].decoratorRight+" ":""),t=Xs(e)}return t}createSelectList(e){if(this.selectList)return;this.selectDropDownListContainer=X(e,lg(".select-box-dropdown-list-container")),this.listRenderer=new Mde,this.selectList=this._register(new Xo("SelectBoxCustom",this.selectDropDownListContainer,this,[this.listRenderer],{useShadows:!1,verticalScrollMode:3,keyboardSupport:!1,mouseSupport:!1,accessibilityProvider:{getAriaLabel:n=>{let s=n.text;return n.detail&&(s+=`. ${n.detail}`),n.decoratorRight&&(s+=`. ${n.decoratorRight}`),n.description&&(s+=`. ${n.description}`),s},getWidgetAriaLabel:()=>m({key:"selectBox",comment:["Behave like native select dropdown element."]},"Select Box"),getRole:()=>Je?"":"option",getWidgetRole:()=>"listbox"}})),this.selectBoxOptions.ariaLabel&&(this.selectList.ariaLabel=this.selectBoxOptions.ariaLabel);const t=this._register(new tt(this.selectDropDownListContainer,"keydown")),i=oe.chain(t.event,n=>n.filter(()=>this.selectList.length>0).map(s=>new Pt(s)));this._register(oe.chain(i,n=>n.filter(s=>s.keyCode===3))(this.onEnter,this)),this._register(oe.chain(i,n=>n.filter(s=>s.keyCode===2))(this.onEnter,this)),this._register(oe.chain(i,n=>n.filter(s=>s.keyCode===9))(this.onEscape,this)),this._register(oe.chain(i,n=>n.filter(s=>s.keyCode===16))(this.onUpArrow,this)),this._register(oe.chain(i,n=>n.filter(s=>s.keyCode===18))(this.onDownArrow,this)),this._register(oe.chain(i,n=>n.filter(s=>s.keyCode===12))(this.onPageDown,this)),this._register(oe.chain(i,n=>n.filter(s=>s.keyCode===11))(this.onPageUp,this)),this._register(oe.chain(i,n=>n.filter(s=>s.keyCode===14))(this.onHome,this)),this._register(oe.chain(i,n=>n.filter(s=>s.keyCode===13))(this.onEnd,this)),this._register(oe.chain(i,n=>n.filter(s=>s.keyCode>=21&&s.keyCode<=56||s.keyCode>=85&&s.keyCode<=113))(this.onCharacter,this)),this._register(U(this.selectList.getHTMLElement(),ie.POINTER_UP,n=>this.onPointerUp(n))),this._register(this.selectList.onMouseOver(n=>typeof n.index<"u"&&this.selectList.setFocus([n.index]))),this._register(this.selectList.onDidChangeFocus(n=>this.onListFocus(n))),this._register(U(this.selectDropDownContainer,ie.FOCUS_OUT,n=>{!this._isVisible||Zi(n.relatedTarget,this.selectDropDownContainer)||this.onListBlur()})),this.selectList.getHTMLElement().setAttribute("aria-label",this.selectBoxOptions.ariaLabel||""),this.selectList.getHTMLElement().setAttribute("aria-expanded","true"),this.styleList()}onPointerUp(e){if(!this.selectList.length)return;nt.stop(e);const t=e.target;if(!t||t.classList.contains("slider"))return;const i=t.closest(".monaco-list-row");if(!i)return;const n=Number(i.getAttribute("data-index")),s=i.classList.contains("option-disabled");n>=0&&n<this.options.length&&!s&&(this.selected=n,this.select(this.selected),this.selectList.setFocus([this.selected]),this.selectList.reveal(this.selectList.getFocus()[0]),this.selected!==this._currentSelection&&(this._currentSelection=this.selected,this._onDidSelect.fire({index:this.selectElement.selectedIndex,selected:this.options[this.selected].text}),this.options[this.selected]&&this.options[this.selected].text&&this.setTitle(this.options[this.selected].text)),this.hideSelectDropDown(!0))}onListBlur(){this._sticky||(this.selected!==this._currentSelection&&this.select(this._currentSelection),this.hideSelectDropDown(!1))}renderDescriptionMarkdown(e,t){const i=s=>{for(let r=0;r<s.childNodes.length;r++){const a=s.childNodes.item(r);(a.tagName&&a.tagName.toLowerCase())==="img"?a.remove():i(a)}},n=YI({value:e,supportThemeIcons:!0},{actionHandler:t});return n.element.classList.add("select-box-description-markdown"),i(n.element),n.element}onListFocus(e){!this._isVisible||!this._hasDetails||this.updateDetail(e.indexes[0])}updateDetail(e){this.selectionDetailsPane.innerText="";const t=this.options[e],i=(t==null?void 0:t.description)??"",n=(t==null?void 0:t.descriptionIsMarkdown)??!1;if(i){if(n){const s=t.descriptionMarkdownActionHandler;this.selectionDetailsPane.appendChild(this.renderDescriptionMarkdown(i,s))}else this.selectionDetailsPane.innerText=i;this.selectionDetailsPane.style.display="block"}else this.selectionDetailsPane.style.display="none";this._skipLayout=!0,this.contextViewProvider.layout(),this._skipLayout=!1}onEscape(e){nt.stop(e),this.select(this._currentSelection),this.hideSelectDropDown(!0)}onEnter(e){nt.stop(e),this.selected!==this._currentSelection&&(this._currentSelection=this.selected,this._onDidSelect.fire({index:this.selectElement.selectedIndex,selected:this.options[this.selected].text}),this.options[this.selected]&&this.options[this.selected].text&&this.setTitle(this.options[this.selected].text)),this.hideSelectDropDown(!0)}onDownArrow(e){if(this.selected<this.options.length-1){nt.stop(e,!0);const t=this.options[this.selected+1].isDisabled;if(t&&this.options.length>this.selected+2)this.selected+=2;else{if(t)return;this.selected++}this.select(this.selected),this.selectList.setFocus([this.selected]),this.selectList.reveal(this.selectList.getFocus()[0])}}onUpArrow(e){this.selected>0&&(nt.stop(e,!0),this.options[this.selected-1].isDisabled&&this.selected>1?this.selected-=2:this.selected--,this.select(this.selected),this.selectList.setFocus([this.selected]),this.selectList.reveal(this.selectList.getFocus()[0]))}onPageUp(e){nt.stop(e),this.selectList.focusPreviousPage(),setTimeout(()=>{this.selected=this.selectList.getFocus()[0],this.options[this.selected].isDisabled&&this.selected<this.options.length-1&&(this.selected++,this.selectList.setFocus([this.selected])),this.selectList.reveal(this.selected),this.select(this.selected)},1)}onPageDown(e){nt.stop(e),this.selectList.focusNextPage(),setTimeout(()=>{this.selected=this.selectList.getFocus()[0],this.options[this.selected].isDisabled&&this.selected>0&&(this.selected--,this.selectList.setFocus([this.selected])),this.selectList.reveal(this.selected),this.select(this.selected)},1)}onHome(e){nt.stop(e),!(this.options.length<2)&&(this.selected=0,this.options[this.selected].isDisabled&&this.selected>1&&this.selected++,this.selectList.setFocus([this.selected]),this.selectList.reveal(this.selected),this.select(this.selected))}onEnd(e){nt.stop(e),!(this.options.length<2)&&(this.selected=this.options.length-1,this.options[this.selected].isDisabled&&this.selected>1&&this.selected--,this.selectList.setFocus([this.selected]),this.selectList.reveal(this.selected),this.select(this.selected))}onCharacter(e){const t=xc.toString(e.keyCode);let i=-1;for(let n=0;n<this.options.length-1;n++)if(i=(n+this.selected+1)%this.options.length,this.options[i].text.charAt(0).toUpperCase()===t&&!this.options[i].isDisabled){this.select(i),this.selectList.setFocus([i]),this.selectList.reveal(this.selectList.getFocus()[0]),nt.stop(e);break}}dispose(){this.hideSelectDropDown(!1),super.dispose()}};hl.DEFAULT_DROPDOWN_MINIMUM_BOTTOM_MARGIN=32,hl.DEFAULT_DROPDOWN_MINIMUM_TOP_MARGIN=2,hl.DEFAULT_MINIMUM_VISIBLE_OPTIONS=3;let K2=hl;class Ade extends F{constructor(e,t,i,n){super(),this.selected=0,this.selectBoxOptions=n||Object.create(null),this.options=[],this.selectElement=document.createElement("select"),this.selectElement.className="monaco-select-box",typeof this.selectBoxOptions.ariaLabel=="string"&&this.selectElement.setAttribute("aria-label",this.selectBoxOptions.ariaLabel),typeof this.selectBoxOptions.ariaDescription=="string"&&this.selectElement.setAttribute("aria-description",this.selectBoxOptions.ariaDescription),this._onDidSelect=this._register(new P),this.styles=i,this.registerListeners(),this.setOptions(e,t)}registerListeners(){this._register(Fn.addTarget(this.selectElement)),[Bt.Tap].forEach(e=>{this._register(U(this.selectElement,e,t=>{this.selectElement.focus()}))}),this._register(hi(this.selectElement,"click",e=>{nt.stop(e,!0)})),this._register(hi(this.selectElement,"change",e=>{this.selectElement.title=e.target.value,this._onDidSelect.fire({index:e.target.selectedIndex,selected:e.target.value})})),this._register(hi(this.selectElement,"keydown",e=>{let t=!1;Je?(e.keyCode===18||e.keyCode===16||e.keyCode===10)&&(t=!0):(e.keyCode===18&&e.altKey||e.keyCode===10||e.keyCode===3)&&(t=!0),t&&e.stopPropagation()}))}get onDidSelect(){return this._onDidSelect.event}setOptions(e,t){(!this.options||!ii(this.options,e))&&(this.options=e,this.selectElement.options.length=0,this.options.forEach((i,n)=>{this.selectElement.add(this.createOption(i.text,n,i.isDisabled))})),t!==void 0&&this.select(t)}select(e){this.options.length===0?this.selected=0:e>=0&&e<this.options.length?this.selected=e:e>this.options.length-1?this.select(this.options.length-1):this.selected<0&&(this.selected=0),this.selectElement.selectedIndex=this.selected,this.selected<this.options.length&&typeof this.options[this.selected].text=="string"?this.selectElement.title=this.options[this.selected].text:this.selectElement.title=""}focus(){this.selectElement&&(this.selectElement.tabIndex=0,this.selectElement.focus())}blur(){this.selectElement&&(this.selectElement.tabIndex=-1,this.selectElement.blur())}setFocusable(e){this.selectElement.tabIndex=e?0:-1}render(e){e.classList.add("select-container"),e.appendChild(this.selectElement),this.setOptions(this.options,this.selected),this.applyStyles()}applyStyles(){this.selectElement&&(this.selectElement.style.backgroundColor=this.styles.selectBackground??"",this.selectElement.style.color=this.styles.selectForeground??"",this.selectElement.style.borderColor=this.styles.selectBorder??"")}createOption(e,t,i){const n=document.createElement("option");return n.value=e,n.text=e,n.disabled=!!i,n}}class Pde extends Yo{constructor(e,t,i,n,s){super(),Je&&!(s!=null&&s.useCustomDrawn)?this.selectBoxDelegate=new Ade(e,t,n,s):this.selectBoxDelegate=new K2(e,t,i,n,s),this._register(this.selectBoxDelegate)}get onDidSelect(){return this.selectBoxDelegate.onDidSelect}setOptions(e,t){this.selectBoxDelegate.setOptions(e,t)}select(e){this.selectBoxDelegate.select(e)}focus(){this.selectBoxDelegate.focus()}blur(){this.selectBoxDelegate.blur()}setFocusable(e){this.selectBoxDelegate.setFocusable(e)}render(e){this.selectBoxDelegate.render(e)}}class va extends F{get action(){return this._action}constructor(e,t,i={}){super(),this.options=i,this._context=e||this,this._action=t,t instanceof ro&&this._register(t.onDidChange(n=>{this.element&&this.handleActionChangeEvent(n)}))}handleActionChangeEvent(e){e.enabled!==void 0&&this.updateEnabled(),e.checked!==void 0&&this.updateChecked(),e.class!==void 0&&this.updateClass(),e.label!==void 0&&(this.updateLabel(),this.updateTooltip()),e.tooltip!==void 0&&this.updateTooltip()}get actionRunner(){return this._actionRunner||(this._actionRunner=this._register(new Ef)),this._actionRunner}set actionRunner(e){this._actionRunner=e}isEnabled(){return this._action.enabled}setActionContext(e){this._context=e}render(e){const t=this.element=e;this._register(Fn.addTarget(e));const i=this.options&&this.options.draggable;i&&(e.draggable=!0,pr&&this._register(U(e,ie.DRAG_START,n=>{var s;return(s=n.dataTransfer)==null?void 0:s.setData(Iw.TEXT,this._action.label)}))),this._register(U(t,Bt.Tap,n=>this.onClick(n,!0))),this._register(U(t,ie.MOUSE_DOWN,n=>{i||nt.stop(n,!0),this._action.enabled&&n.button===0&&t.classList.add("active")})),Je&&this._register(U(t,ie.CONTEXT_MENU,n=>{n.button===0&&n.ctrlKey===!0&&this.onClick(n)})),this._register(U(t,ie.CLICK,n=>{nt.stop(n,!0),this.options&&this.options.isMenu||this.onClick(n)})),this._register(U(t,ie.DBLCLICK,n=>{nt.stop(n,!0)})),[ie.MOUSE_UP,ie.MOUSE_OUT].forEach(n=>{this._register(U(t,n,s=>{nt.stop(s),t.classList.remove("active")}))})}onClick(e,t=!1){var n;nt.stop(e,!0);const i=Ro(this._context)?(n=this.options)!=null&&n.useEventAsContext?e:{preserveFocus:t}:this._context;this.actionRunner.run(this._action,i)}focus(){this.element&&(this.element.tabIndex=0,this.element.focus(),this.element.classList.add("focused"))}blur(){this.element&&(this.element.blur(),this.element.tabIndex=-1,this.element.classList.remove("focused"))}setFocusable(e){this.element&&(this.element.tabIndex=e?0:-1)}get trapsArrowNavigation(){return!1}updateEnabled(){}updateLabel(){}getClass(){return this.action.class}getTooltip(){return this.action.tooltip}updateTooltip(){var t;if(!this.element)return;const e=this.getTooltip()??"";if(this.updateAriaLabel(),(t=this.options.hoverDelegate)!=null&&t.showNativeHover)this.element.title=e;else if(!this.customHover&&e!==""){const i=this.options.hoverDelegate??Ss("element");this.customHover=this._store.add(ja().setupManagedHover(i,this.element,e))}else this.customHover&&this.customHover.update(e)}updateAriaLabel(){if(this.element){const e=this.getTooltip()??"";this.element.setAttribute("aria-label",e)}}updateClass(){}updateChecked(){}dispose(){this.element&&(this.element.remove(),this.element=void 0),this._context=void 0,super.dispose()}}class V_ extends va{constructor(e,t,i){super(e,t,i),this.options=i,this.options.icon=i.icon!==void 0?i.icon:!1,this.options.label=i.label!==void 0?i.label:!0,this.cssClass=""}render(e){super.render(e),mt(this.element);const t=document.createElement("a");if(t.classList.add("action-label"),t.setAttribute("role",this.getDefaultAriaRole()),this.label=t,this.element.appendChild(t),this.options.label&&this.options.keybinding){const i=document.createElement("span");i.classList.add("keybinding"),i.textContent=this.options.keybinding,this.element.appendChild(i)}this.updateClass(),this.updateLabel(),this.updateTooltip(),this.updateEnabled(),this.updateChecked()}getDefaultAriaRole(){return this._action.id===Qi.ID?"presentation":this.options.isMenu?"menuitem":this.options.isTabList?"tab":"button"}focus(){this.label&&(this.label.tabIndex=0,this.label.focus())}blur(){this.label&&(this.label.tabIndex=-1)}setFocusable(e){this.label&&(this.label.tabIndex=e?0:-1)}updateLabel(){this.options.label&&this.label&&(this.label.textContent=this.action.label)}getTooltip(){let e=null;return this.action.tooltip?e=this.action.tooltip:!this.options.label&&this.action.label&&this.options.icon&&(e=this.action.label,this.options.keybinding&&(e=m({key:"titleLabel",comment:["action title","action keybinding"]},"{0} ({1})",e,this.options.keybinding))),e??void 0}updateClass(){var e;this.cssClass&&this.label&&this.label.classList.remove(...this.cssClass.split(" ")),this.options.icon?(this.cssClass=this.getClass(),this.label&&(this.label.classList.add("codicon"),this.cssClass&&this.label.classList.add(...this.cssClass.split(" "))),this.updateEnabled()):(e=this.label)==null||e.classList.remove("codicon")}updateEnabled(){var e,t;this.action.enabled?(this.label&&(this.label.removeAttribute("aria-disabled"),this.label.classList.remove("disabled")),(e=this.element)==null||e.classList.remove("disabled")):(this.label&&(this.label.setAttribute("aria-disabled","true"),this.label.classList.add("disabled")),(t=this.element)==null||t.classList.add("disabled"))}updateAriaLabel(){if(this.label){const e=this.getTooltip()??"";this.label.setAttribute("aria-label",e)}}updateChecked(){this.label&&(this.action.checked!==void 0?(this.label.classList.toggle("checked",this.action.checked),this.options.isTabList?this.label.setAttribute("aria-selected",this.action.checked?"true":"false"):(this.label.setAttribute("aria-checked",this.action.checked?"true":"false"),this.label.setAttribute("role","checkbox"))):(this.label.classList.remove("checked"),this.label.removeAttribute(this.options.isTabList?"aria-selected":"aria-checked"),this.label.setAttribute("role",this.getDefaultAriaRole())))}}class Ode extends va{constructor(e,t,i,n,s,r,a){super(e,t),this.selectBox=new Pde(i,n,s,r,a),this.selectBox.setFocusable(!1),this._register(this.selectBox),this.registerListeners()}select(e){this.selectBox.select(e)}registerListeners(){this._register(this.selectBox.onDidSelect(e=>this.runAction(e.selected,e.index)))}runAction(e,t){this.actionRunner.run(this._action,this.getActionContext(e,t))}getActionContext(e,t){return e}setFocusable(e){this.selectBox.setFocusable(e)}focus(){var e;(e=this.selectBox)==null||e.focus()}blur(){var e;(e=this.selectBox)==null||e.blur()}render(e){this.selectBox.render(e)}}class Fde extends Ef{constructor(e,t){super(),this._onDidChangeVisibility=this._register(new P),this.onDidChangeVisibility=this._onDidChangeVisibility.event,this._element=X(e,ne(".monaco-dropdown")),this._label=X(this._element,ne(".dropdown-label"));let i=t.labelRenderer;i||(i=s=>(s.textContent=t.label||"",null));for(const s of[ie.CLICK,ie.MOUSE_DOWN,Bt.Tap])this._register(U(this.element,s,r=>nt.stop(r,!0)));for(const s of[ie.MOUSE_DOWN,Bt.Tap])this._register(U(this._label,s,r=>{l5(r)&&(r.detail>1||r.button!==0)||(this.visible?this.hide():this.show())}));this._register(U(this._label,ie.KEY_UP,s=>{const r=new Pt(s);(r.equals(3)||r.equals(10))&&(nt.stop(s,!0),this.visible?this.hide():this.show())}));const n=i(this._label);n&&this._register(n),this._register(Fn.addTarget(this._label))}get element(){return this._element}show(){this.visible||(this.visible=!0,this._onDidChangeVisibility.fire(!0))}hide(){this.visible&&(this.visible=!1,this._onDidChangeVisibility.fire(!1))}dispose(){super.dispose(),this.hide(),this.boxContainer&&(this.boxContainer.remove(),this.boxContainer=void 0),this.contents&&(this.contents.remove(),this.contents=void 0),this._label&&(this._label.remove(),this._label=void 0)}}class Wde extends Fde{constructor(e,t){super(e,t),this._options=t,this._actions=[],this.actions=t.actions||[]}set menuOptions(e){this._menuOptions=e}get menuOptions(){return this._menuOptions}get actions(){return this._options.actionProvider?this._options.actionProvider.getActions():this._actions}set actions(e){this._actions=e}show(){super.show(),this.element.classList.add("active"),this._options.contextMenuProvider.showContextMenu({getAnchor:()=>this.element,getActions:()=>this.actions,getActionsContext:()=>this.menuOptions?this.menuOptions.context:null,getActionViewItem:(e,t)=>this.menuOptions&&this.menuOptions.actionViewItemProvider?this.menuOptions.actionViewItemProvider(e,t):void 0,getKeyBinding:e=>this.menuOptions&&this.menuOptions.getKeyBinding?this.menuOptions.getKeyBinding(e):void 0,getMenuClassName:()=>this._options.menuClassName||"",onHide:()=>this.onHide(),actionRunner:this.menuOptions?this.menuOptions.actionRunner:void 0,anchorAlignment:this.menuOptions?this.menuOptions.anchorAlignment:0,domForShadowRoot:this._options.menuAsChild?this.element:void 0,skipTelemetry:this._options.skipTelemetry})}hide(){super.hide()}onHide(){this.hide(),this.element.classList.remove("active")}}class yL extends va{constructor(e,t,i,n=Object.create(null)){super(null,e,n),this.actionItem=null,this._onDidChangeVisibility=this._register(new P),this.onDidChangeVisibility=this._onDidChangeVisibility.event,this.menuActionsOrProvider=t,this.contextMenuProvider=i,this.options=n,this.options.actionRunner&&(this.actionRunner=this.options.actionRunner)}render(e){this.actionItem=e;const t=s=>{this.element=X(s,ne("a.action-label"));let r=[];return typeof this.options.classNames=="string"?r=this.options.classNames.split(/\s+/g).filter(a=>!!a):this.options.classNames&&(r=this.options.classNames),r.find(a=>a==="icon")||r.push("codicon"),this.element.classList.add(...r),this.element.setAttribute("role","button"),this.element.setAttribute("aria-haspopup","true"),this.element.setAttribute("aria-expanded","false"),this._action.label&&this._register(ja().setupManagedHover(this.options.hoverDelegate??Ss("mouse"),this.element,this._action.label)),this.element.ariaLabel=this._action.label||"",null},i=Array.isArray(this.menuActionsOrProvider),n={contextMenuProvider:this.contextMenuProvider,labelRenderer:t,menuAsChild:this.options.menuAsChild,actions:i?this.menuActionsOrProvider:void 0,actionProvider:i?void 0:this.menuActionsOrProvider,skipTelemetry:this.options.skipTelemetry};if(this.dropdownMenu=this._register(new Wde(e,n)),this._register(this.dropdownMenu.onDidChangeVisibility(s=>{var r;(r=this.element)==null||r.setAttribute("aria-expanded",`${s}`),this._onDidChangeVisibility.fire(s)})),this.dropdownMenu.menuOptions={actionViewItemProvider:this.options.actionViewItemProvider,actionRunner:this.actionRunner,getKeyBinding:this.options.keybindingProvider,context:this._context},this.options.anchorAlignmentProvider){const s=this;this.dropdownMenu.menuOptions={...this.dropdownMenu.menuOptions,get anchorAlignment(){return s.options.anchorAlignmentProvider()}}}this.updateTooltip(),this.updateEnabled()}getTooltip(){let e=null;return this.action.tooltip?e=this.action.tooltip:this.action.label&&(e=this.action.label),e??void 0}setActionContext(e){super.setActionContext(e),this.dropdownMenu&&(this.dropdownMenu.menuOptions?this.dropdownMenu.menuOptions.context=e:this.dropdownMenu.menuOptions={context:e})}show(){var e;(e=this.dropdownMenu)==null||e.show()}updateEnabled(){var t,i;const e=!this.action.enabled;(t=this.actionItem)==null||t.classList.toggle("disabled",e),(i=this.element)==null||i.classList.toggle("disabled",e)}}function Bde(o){return o&&typeof o=="object"&&typeof o.original=="string"&&typeof o.value=="string"}function Hde(o){return o?o.condition!==void 0:!1}var Zp;(function(o){o[o.STORAGE_DOES_NOT_EXIST=0]="STORAGE_DOES_NOT_EXIST",o[o.STORAGE_IN_MEMORY=1]="STORAGE_IN_MEMORY"})(Zp||(Zp={}));var lp;(function(o){o[o.None=0]="None",o[o.Initialized=1]="Initialized",o[o.Closed=2]="Closed"})(lp||(lp={}));const Uk=class Uk extends F{constructor(e,t=Object.create(null)){super(),this.database=e,this.options=t,this._onDidChangeStorage=this._register(new Lf),this.onDidChangeStorage=this._onDidChangeStorage.event,this.state=lp.None,this.cache=new Map,this.flushDelayer=this._register(new dU(Uk.DEFAULT_FLUSH_DELAY)),this.pendingDeletes=new Set,this.pendingInserts=new Map,this.whenFlushedCallbacks=[],this.registerListeners()}registerListeners(){this._register(this.database.onDidChangeItemsExternal(e=>this.onDidChangeItemsExternal(e)))}onDidChangeItemsExternal(e){var t,i;this._onDidChangeStorage.pause();try{(t=e.changed)==null||t.forEach((n,s)=>this.acceptExternal(s,n)),(i=e.deleted)==null||i.forEach(n=>this.acceptExternal(n,void 0))}finally{this._onDidChangeStorage.resume()}}acceptExternal(e,t){if(this.state===lp.Closed)return;let i=!1;Ro(t)?i=this.cache.delete(e):this.cache.get(e)!==t&&(this.cache.set(e,t),i=!0),i&&this._onDidChangeStorage.fire({key:e,external:!0})}get(e,t){const i=this.cache.get(e);return Ro(i)?t:i}getBoolean(e,t){const i=this.get(e);return Ro(i)?t:i==="true"}getNumber(e,t){const i=this.get(e);return Ro(i)?t:parseInt(i,10)}async set(e,t,i=!1){if(this.state===lp.Closed)return;if(Ro(t))return this.delete(e,i);const n=on(t)||Array.isArray(t)?_le(t):String(t);if(this.cache.get(e)!==n)return this.cache.set(e,n),this.pendingInserts.set(e,n),this.pendingDeletes.delete(e),this._onDidChangeStorage.fire({key:e,external:i}),this.doFlush()}async delete(e,t=!1){if(!(this.state===lp.Closed||!this.cache.delete(e)))return this.pendingDeletes.has(e)||this.pendingDeletes.add(e),this.pendingInserts.delete(e),this._onDidChangeStorage.fire({key:e,external:t}),this.doFlush()}get hasPending(){return this.pendingInserts.size>0||this.pendingDeletes.size>0}async flushPending(){if(!this.hasPending)return;const e={insert:this.pendingInserts,delete:this.pendingDeletes};return this.pendingDeletes=new Set,this.pendingInserts=new Map,this.database.updateItems(e).finally(()=>{var t;if(!this.hasPending)for(;this.whenFlushedCallbacks.length;)(t=this.whenFlushedCallbacks.pop())==null||t()})}async doFlush(e){return this.options.hint===Zp.STORAGE_IN_MEMORY?this.flushPending():this.flushDelayer.trigger(()=>this.flushPending(),e)}};Uk.DEFAULT_FLUSH_DELAY=100;let JC=Uk;class ZN{constructor(){this.onDidChangeItemsExternal=oe.None,this.items=new Map}async updateItems(e){var t,i;(t=e.insert)==null||t.forEach((n,s)=>this.items.set(s,n)),(i=e.delete)==null||i.forEach(n=>this.items.delete(n))}}const J0="__$__targetStorageMarker",ia=et("storageService");var Nw;(function(o){o[o.NONE=0]="NONE",o[o.SHUTDOWN=1]="SHUTDOWN"})(Nw||(Nw={}));function Vde(o){const e=o.get(J0);if(e)try{return JSON.parse(e)}catch{}return Object.create(null)}const $k=class $k extends F{constructor(e={flushInterval:$k.DEFAULT_FLUSH_INTERVAL}){super(),this.options=e,this._onDidChangeValue=this._register(new Lf),this._onDidChangeTarget=this._register(new Lf),this._onWillSaveState=this._register(new P),this.onWillSaveState=this._onWillSaveState.event,this._workspaceKeyTargets=void 0,this._profileKeyTargets=void 0,this._applicationKeyTargets=void 0}onDidChangeValue(e,t,i){return oe.filter(this._onDidChangeValue.event,n=>n.scope===e&&(t===void 0||n.key===t),i)}emitDidChangeValue(e,t){const{key:i,external:n}=t;if(i===J0){switch(e){case-1:this._applicationKeyTargets=void 0;break;case 0:this._profileKeyTargets=void 0;break;case 1:this._workspaceKeyTargets=void 0;break}this._onDidChangeTarget.fire({scope:e})}else this._onDidChangeValue.fire({scope:e,key:i,target:this.getKeyTargets(e)[i],external:n})}get(e,t,i){var n;return(n=this.getStorage(t))==null?void 0:n.get(e,i)}getBoolean(e,t,i){var n;return(n=this.getStorage(t))==null?void 0:n.getBoolean(e,i)}getNumber(e,t,i){var n;return(n=this.getStorage(t))==null?void 0:n.getNumber(e,i)}store(e,t,i,n,s=!1){if(Ro(t)){this.remove(e,i,s);return}this.withPausedEmitters(()=>{var r;this.updateKeyTarget(e,i,n),(r=this.getStorage(i))==null||r.set(e,t,s)})}remove(e,t,i=!1){this.withPausedEmitters(()=>{var n;this.updateKeyTarget(e,t,void 0),(n=this.getStorage(t))==null||n.delete(e,i)})}withPausedEmitters(e){this._onDidChangeValue.pause(),this._onDidChangeTarget.pause();try{e()}finally{this._onDidChangeValue.resume(),this._onDidChangeTarget.resume()}}updateKeyTarget(e,t,i,n=!1){var r,a;const s=this.getKeyTargets(t);typeof i=="number"?s[e]!==i&&(s[e]=i,(r=this.getStorage(t))==null||r.set(J0,JSON.stringify(s),n)):typeof s[e]=="number"&&(delete s[e],(a=this.getStorage(t))==null||a.set(J0,JSON.stringify(s),n))}get workspaceKeyTargets(){return this._workspaceKeyTargets||(this._workspaceKeyTargets=this.loadKeyTargets(1)),this._workspaceKeyTargets}get profileKeyTargets(){return this._profileKeyTargets||(this._profileKeyTargets=this.loadKeyTargets(0)),this._profileKeyTargets}get applicationKeyTargets(){return this._applicationKeyTargets||(this._applicationKeyTargets=this.loadKeyTargets(-1)),this._applicationKeyTargets}getKeyTargets(e){switch(e){case-1:return this.applicationKeyTargets;case 0:return this.profileKeyTargets;default:return this.workspaceKeyTargets}}loadKeyTargets(e){const t=this.getStorage(e);return t?Vde(t):Object.create(null)}};$k.DEFAULT_FLUSH_INTERVAL=60*1e3;let q2=$k;class zde extends q2{constructor(){super(),this.applicationStorage=this._register(new JC(new ZN,{hint:Zp.STORAGE_IN_MEMORY})),this.profileStorage=this._register(new JC(new ZN,{hint:Zp.STORAGE_IN_MEMORY})),this.workspaceStorage=this._register(new JC(new ZN,{hint:Zp.STORAGE_IN_MEMORY})),this._register(this.workspaceStorage.onDidChangeStorage(e=>this.emitDidChangeValue(1,e))),this._register(this.profileStorage.onDidChangeStorage(e=>this.emitDidChangeValue(0,e))),this._register(this.applicationStorage.onDidChangeStorage(e=>this.emitDidChangeValue(-1,e)))}getStorage(e){switch(e){case-1:return this.applicationStorage;case 0:return this.profileStorage;default:return this.workspaceStorage}}}function Ude(o,e){const t={...e};for(const i in o){const n=o[i];t[i]=n!==void 0?ce(n):void 0}return t}const $de={keybindingLabelBackground:ce(Are),keybindingLabelForeground:ce(Pre),keybindingLabelBorder:ce(Ore),keybindingLabelBottomBorder:ce(Fre),keybindingLabelShadow:ce(Cb)},jde={buttonForeground:ce(z$),buttonSeparator:ce(vre),buttonBackground:ce(yC),buttonHoverBackground:ce(wre),buttonSecondaryForeground:ce(yre),buttonSecondaryBackground:ce(f2),buttonSecondaryHoverBackground:ce(Lre),buttonBorder:ce(Sre)},Kde={progressBarBackground:ce(Aoe)},LL={inputActiveOptionBorder:ce(zI),inputActiveOptionForeground:ce(UI),inputActiveOptionBackground:ce(V1)};ce(LC),ce(xre),ce(kre),ce(Dre),ce(Ire),ce(Ere),ce(Nre);ce(Tre),ce(Mre),ce(Rre);ce(Wo),ce(BI),ce(Cb),ce(lt),ce(tre),ce(ire),ce(nre),ce(Roe);const xL={inputBackground:ce(g2),inputForeground:ce(H$),inputBorder:ce(V$),inputValidationInfoBorder:ce(ure),inputValidationInfoBackground:ce(dre),inputValidationInfoForeground:ce(hre),inputValidationWarningBorder:ce(mre),inputValidationWarningBackground:ce(gre),inputValidationWarningForeground:ce(fre),inputValidationErrorBorder:ce(bre),inputValidationErrorBackground:ce(pre),inputValidationErrorForeground:ce(_re)},qde={listFilterWidgetBackground:ce(Yre),listFilterWidgetOutline:ce(Xre),listFilterWidgetNoMatchesOutline:ce(Qre),listFilterWidgetShadow:ce(Jre),inputBoxStyles:xL,toggleStyles:LL},Pj={badgeBackground:ce(Z0),badgeForeground:ce(Moe),badgeBorder:ce(lt)};ce(Joe),ce(Qoe),ce(y9),ce(y9),ce(ere);const nm={listBackground:void 0,listInactiveFocusForeground:void 0,listFocusBackground:ce(Wre),listFocusForeground:ce(Bre),listFocusOutline:ce(Hre),listActiveSelectionBackground:ce(Tf),listActiveSelectionForeground:ce(bw),listActiveSelectionIconForeground:ce(U$),listFocusAndSelectionOutline:ce(Vre),listFocusAndSelectionBackground:ce(Tf),listFocusAndSelectionForeground:ce(bw),listInactiveSelectionBackground:ce(zre),listInactiveSelectionIconForeground:ce($re),listInactiveSelectionForeground:ce(Ure),listInactiveFocusBackground:ce(jre),listInactiveFocusOutline:ce(Kre),listHoverBackground:ce($$),listHoverForeground:ce(j$),listDropOverBackground:ce(qre),listDropBetweenBackground:ce(Gre),listSelectionOutline:ce(Xt),listHoverOutline:ce(Xt),treeIndentGuidesStroke:ce(K$),treeInactiveIndentGuidesStroke:ce(eae),treeStickyScrollBackground:void 0,treeStickyScrollBorder:void 0,treeStickyScrollShadow:ce(I5),tableColumnsBorder:ce(tae),tableOddRowsBackgroundColor:ce(iae)};function sm(o){return Ude(o,nm)}const Gde={selectBackground:ce($I),selectListBackground:ce(Cre),selectForeground:ce(A5),decoratorRightForeground:ce(q$),selectBorder:ce(P5),focusBorder:ce(Wl),listFocusBackground:ce(vw),listInactiveSelectionIconForeground:ce(O5),listFocusForeground:ce(Cw),listFocusOutline:koe(Xt,Z.transparent.toString()),listHoverBackground:ce($$),listHoverForeground:ce(j$),listHoverOutline:ce(Xt),selectListBorder:ce(E5),listBackground:void 0,listActiveSelectionBackground:void 0,listActiveSelectionForeground:void 0,listActiveSelectionIconForeground:void 0,listFocusAndSelectionBackground:void 0,listDropOverBackground:void 0,listDropBetweenBackground:void 0,listInactiveSelectionBackground:void 0,listInactiveSelectionForeground:void 0,listInactiveFocusBackground:void 0,listInactiveFocusOutline:void 0,listSelectionOutline:void 0,listFocusAndSelectionForeground:void 0,listFocusAndSelectionOutline:void 0,listInactiveFocusForeground:void 0,tableColumnsBorder:void 0,tableOddRowsBackgroundColor:void 0,treeIndentGuidesStroke:void 0,treeInactiveIndentGuidesStroke:void 0,treeStickyScrollBackground:void 0,treeStickyScrollBorder:void 0,treeStickyScrollShadow:void 0},Zde={shadowColor:ce(Cb),borderColor:ce(nae),foregroundColor:ce(sae),backgroundColor:ce(oae),selectionForegroundColor:ce(rae),selectionBackgroundColor:ce(aae),selectionBorderColor:ce(lae),separatorColor:ce(cae),scrollbarShadow:ce(I5),scrollbarSliderBackground:ce(R$),scrollbarSliderHoverBackground:ce(M$),scrollbarSliderActiveBackground:ce(A$)};var nE=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Rs=function(o,e){return function(t,i){e(t,i,o)}};function Yde(o,e,t,i){let n,s,r;if(Array.isArray(o))r=o,n=e,s=t;else{const c=e;r=o.getActions(c),n=t,s=i}const a=Rc.getInstance(),l=a.keyStatus.altKey||(En||ps)&&a.keyStatus.shiftKey;Oj(r,n,l,s?c=>c===s:c=>c==="navigation")}function sE(o,e,t,i,n,s){let r,a,l,c,d;if(Array.isArray(o))d=o,r=e,a=t,l=i,c=n;else{const u=e;d=o.getActions(u),r=t,a=i,l=n,c=s}Oj(d,r,!1,typeof a=="string"?u=>u===a:a,l,c)}function Oj(o,e,t,i=r=>r==="navigation",n=()=>!1,s=!1){let r,a;Array.isArray(e)?(r=e,a=e):(r=e.primary,a=e.secondary);const l=new Set;for(const[c,d]of o){let h;i(c)?(h=r,h.length>0&&s&&h.push(new Qi)):(h=a,h.length>0&&h.push(new Qi));for(let u of d){t&&(u=u instanceof ao&&u.alt?u.alt:u);const g=h.push(u);u instanceof D_&&l.add({group:c,action:u,index:g-1})}}for(const{group:c,action:d,index:h}of l){const u=i(c)?r:a,g=d.actions;n(d,c,u.length)&&u.splice(h,1,...g)}}let $h=class extends V_{constructor(e,t,i,n,s,r,a,l){super(void 0,e,{icon:!!(e.class||e.item.icon),label:!e.class&&!e.item.icon,draggable:t==null?void 0:t.draggable,keybinding:t==null?void 0:t.keybinding,hoverDelegate:t==null?void 0:t.hoverDelegate}),this._options=t,this._keybindingService=i,this._notificationService=n,this._contextKeyService=s,this._themeService=r,this._contextMenuService=a,this._accessibilityService=l,this._wantsAltCommand=!1,this._itemClassDispose=this._register(new Ji),this._altKey=Rc.getInstance()}get _menuItemAction(){return this._action}get _commandAction(){return this._wantsAltCommand&&this._menuItemAction.alt||this._menuItemAction}async onClick(e){e.preventDefault(),e.stopPropagation();try{await this.actionRunner.run(this._commandAction,this._context)}catch(t){this._notificationService.error(t)}}render(e){if(super.render(e),e.classList.add("menu-entry"),this.options.icon&&this._updateItemClass(this._menuItemAction.item),this._menuItemAction.alt){let t=!1;const i=()=>{var s;const n=!!((s=this._menuItemAction.alt)!=null&&s.enabled)&&(!this._accessibilityService.isMotionReduced()||t)&&(this._altKey.keyStatus.altKey||this._altKey.keyStatus.shiftKey&&t);n!==this._wantsAltCommand&&(this._wantsAltCommand=n,this.updateLabel(),this.updateTooltip(),this.updateClass())};this._register(this._altKey.event(i)),this._register(U(e,"mouseleave",n=>{t=!1,i()})),this._register(U(e,"mouseenter",n=>{t=!0,i()})),i()}}updateLabel(){this.options.label&&this.label&&(this.label.textContent=this._commandAction.label)}getTooltip(){var s;const e=this._keybindingService.lookupKeybinding(this._commandAction.id,this._contextKeyService),t=e&&e.getLabel(),i=this._commandAction.tooltip||this._commandAction.label;let n=t?m("titleAndKb","{0} ({1})",i,t):i;if(!this._wantsAltCommand&&((s=this._menuItemAction.alt)!=null&&s.enabled)){const r=this._menuItemAction.alt.tooltip||this._menuItemAction.alt.label,a=this._keybindingService.lookupKeybinding(this._menuItemAction.alt.id,this._contextKeyService),l=a&&a.getLabel(),c=l?m("titleAndKb","{0} ({1})",r,l):r;n=m("titleAndKbAndAlt",`{0}
[{1}] {2}`,n,Q5.modifierLabels[so].altKey,c)}return n}updateClass(){this.options.icon&&(this._commandAction!==this._menuItemAction?this._menuItemAction.alt&&this._updateItemClass(this._menuItemAction.alt.item):this._updateItemClass(this._menuItemAction.item))}_updateItemClass(e){this._itemClassDispose.value=void 0;const{element:t,label:i}=this;if(!t||!i)return;const n=this._commandAction.checked&&Hde(e.toggled)&&e.toggled.icon?e.toggled.icon:e.icon;if(n)if(Ie.isThemeIcon(n)){const s=Ie.asClassNameArray(n);i.classList.add(...s),this._itemClassDispose.value=ve(()=>{i.classList.remove(...s)})}else i.style.backgroundImage=N_(this._themeService.getColorTheme().type)?td(n.dark):td(n.light),i.classList.add("icon"),this._itemClassDispose.value=jr(ve(()=>{i.style.backgroundImage="",i.classList.remove("icon")}),this._themeService.onDidColorThemeChange(()=>{this.updateClass()}))}};$h=nE([Rs(2,yt),Rs(3,Ei),Rs(4,Ee),Rs(5,ji),Rs(6,mo),Rs(7,po)],$h);class i3 extends $h{render(e){var t;this.options.label=!0,this.options.icon=!1,super.render(e),e.classList.add("text-only"),e.classList.toggle("use-comma",((t=this._options)==null?void 0:t.useComma)??!1)}updateLabel(){var t;const e=this._keybindingService.lookupKeybinding(this._action.id,this._contextKeyService);if(!e)return super.updateLabel();if(this.label){const i=i3._symbolPrintEnter(e);(t=this._options)!=null&&t.conversational?this.label.textContent=m({key:"content2",comment:['A label with keybindg like "ESC to dismiss"']},"{1} to {0}",this._action.label,i):this.label.textContent=m({key:"content",comment:["A label","A keybinding"]},"{0} ({1})",this._action.label,i)}}static _symbolPrintEnter(e){var t;return(t=e.getLabel())==null?void 0:t.replace(/\benter\b/gi,"⏎").replace(/\bEscape\b/gi,"Esc")}}let G2=class extends yL{constructor(e,t,i,n,s){const r={...t,menuAsChild:(t==null?void 0:t.menuAsChild)??!1,classNames:(t==null?void 0:t.classNames)??(Ie.isThemeIcon(e.item.icon)?Ie.asClassName(e.item.icon):void 0),keybindingProvider:(t==null?void 0:t.keybindingProvider)??(a=>i.lookupKeybinding(a.id))};super(e,{getActions:()=>e.actions},n,r),this._keybindingService=i,this._contextMenuService=n,this._themeService=s}render(e){super.render(e),mt(this.element),e.classList.add("menu-entry");const t=this._action,{icon:i}=t.item;if(i&&!Ie.isThemeIcon(i)){this.element.classList.add("icon");const n=()=>{this.element&&(this.element.style.backgroundImage=N_(this._themeService.getColorTheme().type)?td(i.dark):td(i.light))};n(),this._register(this._themeService.onDidColorThemeChange(()=>{n()}))}}};G2=nE([Rs(2,yt),Rs(3,mo),Rs(4,ji)],G2);let Z2=class extends va{constructor(e,t,i,n,s,r,a,l){super(null,e),this._keybindingService=i,this._notificationService=n,this._contextMenuService=s,this._menuService=r,this._instaService=a,this._storageService=l,this._container=null,this._options=t,this._storageKey=`${e.item.submenu.id}_lastActionId`;let c;const d=t!=null&&t.persistLastActionId?l.get(this._storageKey,1):void 0;d&&(c=e.actions.find(u=>d===u.id)),c||(c=e.actions[0]),this._defaultAction=this._instaService.createInstance($h,c,{keybinding:this._getDefaultActionKeybindingLabel(c)});const h={keybindingProvider:u=>this._keybindingService.lookupKeybinding(u.id),...t,menuAsChild:(t==null?void 0:t.menuAsChild)??!0,classNames:(t==null?void 0:t.classNames)??["codicon","codicon-chevron-down"],actionRunner:(t==null?void 0:t.actionRunner)??new Ef};this._dropdown=new yL(e,e.actions,this._contextMenuService,h),this._register(this._dropdown.actionRunner.onDidRun(u=>{u.action instanceof ao&&this.update(u.action)}))}update(e){var t;(t=this._options)!=null&&t.persistLastActionId&&this._storageService.store(this._storageKey,e.id,1,1),this._defaultAction.dispose(),this._defaultAction=this._instaService.createInstance($h,e,{keybinding:this._getDefaultActionKeybindingLabel(e)}),this._defaultAction.actionRunner=new class extends Ef{async runAction(i,n){await i.run(void 0)}},this._container&&this._defaultAction.render(c5(this._container,ne(".action-container")))}_getDefaultActionKeybindingLabel(e){var i;let t;if((i=this._options)!=null&&i.renderKeybindingWithDefaultActionLabel){const n=this._keybindingService.lookupKeybinding(e.id);n&&(t=`(${n.getLabel()})`)}return t}setActionContext(e){super.setActionContext(e),this._defaultAction.setActionContext(e),this._dropdown.setActionContext(e)}render(e){this._container=e,super.render(this._container),this._container.classList.add("monaco-dropdown-with-default");const t=ne(".action-container");this._defaultAction.render(X(this._container,t)),this._register(U(t,ie.KEY_DOWN,n=>{const s=new Pt(n);s.equals(17)&&(this._defaultAction.element.tabIndex=-1,this._dropdown.focus(),s.stopPropagation())}));const i=ne(".dropdown-action-container");this._dropdown.render(X(this._container,i)),this._register(U(i,ie.KEY_DOWN,n=>{var r;const s=new Pt(n);s.equals(15)&&(this._defaultAction.element.tabIndex=0,this._dropdown.setFocusable(!1),(r=this._defaultAction.element)==null||r.focus(),s.stopPropagation())}))}focus(e){e?this._dropdown.focus():(this._defaultAction.element.tabIndex=0,this._defaultAction.element.focus())}blur(){this._defaultAction.element.tabIndex=-1,this._dropdown.blur(),this._container.blur()}setFocusable(e){e?this._defaultAction.element.tabIndex=0:(this._defaultAction.element.tabIndex=-1,this._dropdown.setFocusable(!1))}dispose(){this._defaultAction.dispose(),this._dropdown.dispose(),super.dispose()}};Z2=nE([Rs(2,yt),Rs(3,Ei),Rs(4,mo),Rs(5,Go),Rs(6,we),Rs(7,ia)],Z2);let Y2=class extends Ode{constructor(e,t){super(null,e,e.actions.map(i=>({text:i.id===Qi.ID?"─────────":i.label,isDisabled:!i.enabled})),0,t,Gde,{ariaLabel:e.tooltip,optionsAsChildren:!0}),this.select(Math.max(0,e.actions.findIndex(i=>i.checked)))}render(e){super.render(e),e.style.borderColor=ce(P5)}runAction(e,t){const i=this.action.actions[t];i&&this.actionRunner.run(i)}};Y2=nE([Rs(1,ud)],Y2);function Fj(o,e,t){return e instanceof ao?o.createInstance($h,e,t):e instanceof $p?e.item.isSelection?o.createInstance(Y2,e):e.item.rememberDefaultAction?o.createInstance(Z2,e,{...t,persistLastActionId:!0}):o.createInstance(G2,e,t):void 0}class zo extends F{constructor(e,t={}){var s,r;super(),this._actionRunnerDisposables=this._register(new K),this.viewItemDisposables=this._register(new $4),this.triggerKeyDown=!1,this.focusable=!0,this._onDidBlur=this._register(new P),this.onDidBlur=this._onDidBlur.event,this._onDidCancel=this._register(new P({onWillAddFirstListener:()=>this.cancelHasListener=!0})),this.onDidCancel=this._onDidCancel.event,this.cancelHasListener=!1,this._onDidRun=this._register(new P),this.onDidRun=this._onDidRun.event,this._onWillRun=this._register(new P),this.onWillRun=this._onWillRun.event,this.options=t,this._context=t.context??null,this._orientation=this.options.orientation??0,this._triggerKeys={keyDown:((s=this.options.triggerKeys)==null?void 0:s.keyDown)??!1,keys:((r=this.options.triggerKeys)==null?void 0:r.keys)??[3,10]},this._hoverDelegate=t.hoverDelegate??this._register(H_()),this.options.actionRunner?this._actionRunner=this.options.actionRunner:(this._actionRunner=new Ef,this._actionRunnerDisposables.add(this._actionRunner)),this._actionRunnerDisposables.add(this._actionRunner.onDidRun(a=>this._onDidRun.fire(a))),this._actionRunnerDisposables.add(this._actionRunner.onWillRun(a=>this._onWillRun.fire(a))),this.viewItems=[],this.focusedItem=void 0,this.domNode=document.createElement("div"),this.domNode.className="monaco-action-bar";let i,n;switch(this._orientation){case 0:i=[15],n=[17];break;case 1:i=[16],n=[18],this.domNode.className+=" vertical";break}this._register(U(this.domNode,ie.KEY_DOWN,a=>{const l=new Pt(a);let c=!0;const d=typeof this.focusedItem=="number"?this.viewItems[this.focusedItem]:void 0;i&&(l.equals(i[0])||l.equals(i[1]))?c=this.focusPrevious():n&&(l.equals(n[0])||l.equals(n[1]))?c=this.focusNext():l.equals(9)&&this.cancelHasListener?this._onDidCancel.fire():l.equals(14)?c=this.focusFirst():l.equals(13)?c=this.focusLast():l.equals(2)&&d instanceof va&&d.trapsArrowNavigation?c=this.focusNext(void 0,!0):this.isTriggerKeyEvent(l)?this._triggerKeys.keyDown?this.doTrigger(l):this.triggerKeyDown=!0:c=!1,c&&(l.preventDefault(),l.stopPropagation())})),this._register(U(this.domNode,ie.KEY_UP,a=>{const l=new Pt(a);this.isTriggerKeyEvent(l)?(!this._triggerKeys.keyDown&&this.triggerKeyDown&&(this.triggerKeyDown=!1,this.doTrigger(l)),l.preventDefault(),l.stopPropagation()):(l.equals(2)||l.equals(1026)||l.equals(16)||l.equals(18)||l.equals(15)||l.equals(17))&&this.updateFocusedItem()})),this.focusTracker=this._register(Fa(this.domNode)),this._register(this.focusTracker.onDidBlur(()=>{(Dn()===this.domNode||!Zi(Dn(),this.domNode))&&(this._onDidBlur.fire(),this.previouslyFocusedItem=this.focusedItem,this.focusedItem=void 0,this.triggerKeyDown=!1)})),this._register(this.focusTracker.onDidFocus(()=>this.updateFocusedItem())),this.actionsList=document.createElement("ul"),this.actionsList.className="actions-container",this.options.highlightToggledItems&&this.actionsList.classList.add("highlight-toggled"),this.actionsList.setAttribute("role",this.options.ariaRole||"toolbar"),this.options.ariaLabel&&this.actionsList.setAttribute("aria-label",this.options.ariaLabel),this.domNode.appendChild(this.actionsList),e.appendChild(this.domNode)}refreshRole(){this.length()>=1?this.actionsList.setAttribute("role",this.options.ariaRole||"toolbar"):this.actionsList.setAttribute("role","presentation")}setFocusable(e){if(this.focusable=e,this.focusable){const t=this.viewItems.find(i=>i instanceof va&&i.isEnabled());t instanceof va&&t.setFocusable(!0)}else this.viewItems.forEach(t=>{t instanceof va&&t.setFocusable(!1)})}isTriggerKeyEvent(e){let t=!1;return this._triggerKeys.keys.forEach(i=>{t=t||e.equals(i)}),t}updateFocusedItem(){var e,t;for(let i=0;i<this.actionsList.children.length;i++){const n=this.actionsList.children[i];if(Zi(Dn(),n)){this.focusedItem=i,(t=(e=this.viewItems[this.focusedItem])==null?void 0:e.showHover)==null||t.call(e);break}}}get context(){return this._context}set context(e){this._context=e,this.viewItems.forEach(t=>t.setActionContext(e))}get actionRunner(){return this._actionRunner}set actionRunner(e){this._actionRunner=e,this._actionRunnerDisposables.clear(),this._actionRunnerDisposables.add(this._actionRunner.onDidRun(t=>this._onDidRun.fire(t))),this._actionRunnerDisposables.add(this._actionRunner.onWillRun(t=>this._onWillRun.fire(t))),this.viewItems.forEach(t=>t.actionRunner=e)}getContainer(){return this.domNode}getAction(e){var t;if(typeof e=="number")return(t=this.viewItems[e])==null?void 0:t.action;if(Xi(e)){for(;e.parentElement!==this.actionsList;){if(!e.parentElement)return;e=e.parentElement}for(let i=0;i<this.actionsList.childNodes.length;i++)if(this.actionsList.childNodes[i]===e)return this.viewItems[i].action}}push(e,t={}){const i=Array.isArray(e)?e:[e];let n=Rh(t.index)?t.index:null;i.forEach(s=>{const r=document.createElement("li");r.className="action-item",r.setAttribute("role","presentation");let a;const l={hoverDelegate:this._hoverDelegate,...t,isTabList:this.options.ariaRole==="tablist"};this.options.actionViewItemProvider&&(a=this.options.actionViewItemProvider(s,l)),a||(a=new V_(this.context,s,l)),this.options.allowContextMenu||this.viewItemDisposables.set(a,U(r,ie.CONTEXT_MENU,c=>{nt.stop(c,!0)})),a.actionRunner=this._actionRunner,a.setActionContext(this.context),a.render(r),this.focusable&&a instanceof va&&this.viewItems.length===0&&a.setFocusable(!0),n===null||n<0||n>=this.actionsList.children.length?(this.actionsList.appendChild(r),this.viewItems.push(a)):(this.actionsList.insertBefore(r,this.actionsList.children[n]),this.viewItems.splice(n,0,a),n++)}),typeof this.focusedItem=="number"&&this.focus(this.focusedItem),this.refreshRole()}clear(){this.isEmpty()||(this.viewItems=Tt(this.viewItems),this.viewItemDisposables.clearAndDisposeAll(),pn(this.actionsList),this.refreshRole())}length(){return this.viewItems.length}isEmpty(){return this.viewItems.length===0}focus(e){let t=!1,i;if(e===void 0?t=!0:typeof e=="number"?i=e:typeof e=="boolean"&&(t=e),t&&typeof this.focusedItem>"u"){const n=this.viewItems.findIndex(s=>s.isEnabled());this.focusedItem=n===-1?void 0:n,this.updateFocus(void 0,void 0,!0)}else i!==void 0&&(this.focusedItem=i),this.updateFocus(void 0,void 0,!0)}focusFirst(){return this.focusedItem=this.length()-1,this.focusNext(!0)}focusLast(){return this.focusedItem=0,this.focusPrevious(!0)}focusNext(e,t){if(typeof this.focusedItem>"u")this.focusedItem=this.viewItems.length-1;else if(this.viewItems.length<=1)return!1;const i=this.focusedItem;let n;do{if(!e&&this.options.preventLoopNavigation&&this.focusedItem+1>=this.viewItems.length)return this.focusedItem=i,!1;this.focusedItem=(this.focusedItem+1)%this.viewItems.length,n=this.viewItems[this.focusedItem]}while(this.focusedItem!==i&&(this.options.focusOnlyEnabledItems&&!n.isEnabled()||n.action.id===Qi.ID));return this.updateFocus(void 0,void 0,t),!0}focusPrevious(e){if(typeof this.focusedItem>"u")this.focusedItem=0;else if(this.viewItems.length<=1)return!1;const t=this.focusedItem;let i;do{if(this.focusedItem=this.focusedItem-1,this.focusedItem<0){if(!e&&this.options.preventLoopNavigation)return this.focusedItem=t,!1;this.focusedItem=this.viewItems.length-1}i=this.viewItems[this.focusedItem]}while(this.focusedItem!==t&&(this.options.focusOnlyEnabledItems&&!i.isEnabled()||i.action.id===Qi.ID));return this.updateFocus(!0),!0}updateFocus(e,t,i=!1){var s,r;typeof this.focusedItem>"u"&&this.actionsList.focus({preventScroll:t}),this.previouslyFocusedItem!==void 0&&this.previouslyFocusedItem!==this.focusedItem&&((s=this.viewItems[this.previouslyFocusedItem])==null||s.blur());const n=this.focusedItem!==void 0?this.viewItems[this.focusedItem]:void 0;if(n){let a=!0;Xv(n.focus)||(a=!1),this.options.focusOnlyEnabledItems&&Xv(n.isEnabled)&&!n.isEnabled()&&(a=!1),n.action.id===Qi.ID&&(a=!1),a?(i||this.previouslyFocusedItem!==this.focusedItem)&&(n.focus(e),this.previouslyFocusedItem=this.focusedItem):(this.actionsList.focus({preventScroll:t}),this.previouslyFocusedItem=void 0),a&&((r=n.showHover)==null||r.call(n))}}doTrigger(e){if(typeof this.focusedItem>"u")return;const t=this.viewItems[this.focusedItem];if(t instanceof va){const i=t._context===null||t._context===void 0?e:t._context;this.run(t._action,i)}}async run(e,t){await this._actionRunner.run(e,t)}dispose(){this._context=void 0,this.viewItems=Tt(this.viewItems),this.getContainer().remove(),super.dispose()}}const X2=/\(&([^\s&])\)|(^|[^&])&([^\s&])/,YN=/(&amp;)?(&amp;)([^\s&])/g;var kL;(function(o){o[o.Right=0]="Right",o[o.Left=1]="Left"})(kL||(kL={}));var Q2;(function(o){o[o.Above=0]="Above",o[o.Below=1]="Below"})(Q2||(Q2={}));class Yp extends zo{constructor(e,t,i,n){e.classList.add("monaco-menu-container"),e.setAttribute("role","presentation");const s=document.createElement("div");s.classList.add("monaco-menu"),s.setAttribute("role","presentation"),super(s,{orientation:1,actionViewItemProvider:c=>this.doGetActionViewItem(c,i,r),context:i.context,actionRunner:i.actionRunner,ariaLabel:i.ariaLabel,ariaRole:"menu",focusOnlyEnabledItems:!0,triggerKeys:{keys:[3,...Je||ps?[10]:[]],keyDown:!0}}),this.menuStyles=n,this.menuElement=s,this.actionsList.tabIndex=0,this.initializeOrUpdateStyleSheet(e,n),this._register(Fn.addTarget(s)),this._register(U(s,ie.KEY_DOWN,c=>{new Pt(c).equals(2)&&c.preventDefault()})),i.enableMnemonics&&this._register(U(s,ie.KEY_DOWN,c=>{const d=c.key.toLocaleLowerCase();if(this.mnemonics.has(d)){nt.stop(c,!0);const h=this.mnemonics.get(d);if(h.length===1&&(h[0]instanceof h8&&h[0].container&&this.focusItemByElement(h[0].container),h[0].onClick(c)),h.length>1){const u=h.shift();u&&u.container&&(this.focusItemByElement(u.container),h.push(u)),this.mnemonics.set(d,h)}}})),ps&&this._register(U(s,ie.KEY_DOWN,c=>{const d=new Pt(c);d.equals(14)||d.equals(11)?(this.focusedItem=this.viewItems.length-1,this.focusNext(),nt.stop(c,!0)):(d.equals(13)||d.equals(12))&&(this.focusedItem=0,this.focusPrevious(),nt.stop(c,!0))})),this._register(U(this.domNode,ie.MOUSE_OUT,c=>{const d=c.relatedTarget;Zi(d,this.domNode)||(this.focusedItem=void 0,this.updateFocus(),c.stopPropagation())})),this._register(U(this.actionsList,ie.MOUSE_OVER,c=>{let d=c.target;if(!(!d||!Zi(d,this.actionsList)||d===this.actionsList)){for(;d.parentElement!==this.actionsList&&d.parentElement!==null;)d=d.parentElement;if(d.classList.contains("action-item")){const h=this.focusedItem;this.setFocusedItem(d),h!==this.focusedItem&&this.updateFocus()}}})),this._register(Fn.addTarget(this.actionsList)),this._register(U(this.actionsList,Bt.Tap,c=>{let d=c.initialTarget;if(!(!d||!Zi(d,this.actionsList)||d===this.actionsList)){for(;d.parentElement!==this.actionsList&&d.parentElement!==null;)d=d.parentElement;if(d.classList.contains("action-item")){const h=this.focusedItem;this.setFocusedItem(d),h!==this.focusedItem&&this.updateFocus()}}}));const r={parent:this};this.mnemonics=new Map,this.scrollableElement=this._register(new z1(s,{alwaysConsumeMouseWheel:!0,horizontal:2,vertical:3,verticalScrollbarSize:7,handleMouseWheel:!0,useShadows:!0}));const a=this.scrollableElement.getDomNode();a.style.position="",this.styleScrollElement(a,n),this._register(U(s,Bt.Change,c=>{nt.stop(c,!0);const d=this.scrollableElement.getScrollPosition().scrollTop;this.scrollableElement.setScrollPosition({scrollTop:d-c.translationY})})),this._register(U(a,ie.MOUSE_UP,c=>{c.preventDefault()}));const l=xe(e);s.style.maxHeight=`${Math.max(10,l.innerHeight-e.getBoundingClientRect().top-35)}px`,t=t.filter((c,d)=>{var h;return(h=i.submenuIds)!=null&&h.has(c.id)?(console.warn(`Found submenu cycle: ${c.id}`),!1):!(c instanceof Qi&&(d===t.length-1||d===0||t[d-1]instanceof Qi))}),this.push(t,{icon:!0,label:!0,isMenu:!0}),e.appendChild(this.scrollableElement.getDomNode()),this.scrollableElement.scanDomNode(),this.viewItems.filter(c=>!(c instanceof u8)).forEach((c,d,h)=>{c.updatePositionInSet(d+1,h.length)})}initializeOrUpdateStyleSheet(e,t){this.styleSheet||(jy(e)?this.styleSheet=$o(e):(Yp.globalStyleSheet||(Yp.globalStyleSheet=$o()),this.styleSheet=Yp.globalStyleSheet)),this.styleSheet.textContent=Qde(t,jy(e))}styleScrollElement(e,t){const i=t.foregroundColor??"",n=t.backgroundColor??"",s=t.borderColor?`1px solid ${t.borderColor}`:"",r="5px",a=t.shadowColor?`0 2px 8px ${t.shadowColor}`:"";e.style.outline=s,e.style.borderRadius=r,e.style.color=i,e.style.backgroundColor=n,e.style.boxShadow=a}getContainer(){return this.scrollableElement.getDomNode()}get onScroll(){return this.scrollableElement.onScroll}focusItemByElement(e){const t=this.focusedItem;this.setFocusedItem(e),t!==this.focusedItem&&this.updateFocus()}setFocusedItem(e){for(let t=0;t<this.actionsList.children.length;t++){const i=this.actionsList.children[t];if(e===i){this.focusedItem=t;break}}}updateFocus(e){super.updateFocus(e,!0,!0),typeof this.focusedItem<"u"&&this.scrollableElement.setScrollPosition({scrollTop:Math.round(this.menuElement.scrollTop)})}doGetActionViewItem(e,t,i){if(e instanceof Qi)return new u8(t.context,e,{icon:!0},this.menuStyles);if(e instanceof D_){const n=new h8(e,e.actions,i,{...t,submenuIds:new Set([...t.submenuIds||[],e.id])},this.menuStyles);if(t.enableMnemonics){const s=n.getMnemonic();if(s&&n.isEnabled()){let r=[];this.mnemonics.has(s)&&(r=this.mnemonics.get(s)),r.push(n),this.mnemonics.set(s,r)}}return n}else{const n={enableMnemonics:t.enableMnemonics,useEventAsContext:t.useEventAsContext};if(t.getKeyBinding){const r=t.getKeyBinding(e);if(r){const a=r.getLabel();a&&(n.keybinding=a)}}const s=new Wj(t.context,e,n,this.menuStyles);if(t.enableMnemonics){const r=s.getMnemonic();if(r&&s.isEnabled()){let a=[];this.mnemonics.has(r)&&(a=this.mnemonics.get(r)),a.push(s),this.mnemonics.set(r,a)}}return s}}}class Wj extends va{constructor(e,t,i,n){if(i.isMenu=!0,super(t,t,i),this.menuStyle=n,this.options=i,this.options.icon=i.icon!==void 0?i.icon:!1,this.options.label=i.label!==void 0?i.label:!0,this.cssClass="",this.options.label&&i.enableMnemonics){const s=this.action.label;if(s){const r=X2.exec(s);r&&(this.mnemonic=(r[1]?r[1]:r[3]).toLocaleLowerCase())}}this.runOnceToEnableMouseUp=new It(()=>{this.element&&(this._register(U(this.element,ie.MOUSE_UP,s=>{if(nt.stop(s,!0),pr){if(new Vr(xe(this.element),s).rightButton)return;this.onClick(s)}else setTimeout(()=>{this.onClick(s)},0)})),this._register(U(this.element,ie.CONTEXT_MENU,s=>{nt.stop(s,!0)})))},100),this._register(this.runOnceToEnableMouseUp)}render(e){super.render(e),this.element&&(this.container=e,this.item=X(this.element,ne("a.action-menu-item")),this._action.id===Qi.ID?this.item.setAttribute("role","presentation"):(this.item.setAttribute("role","menuitem"),this.mnemonic&&this.item.setAttribute("aria-keyshortcuts",`${this.mnemonic}`)),this.check=X(this.item,ne("span.menu-item-check"+Ie.asCSSSelector(te.menuSelection))),this.check.setAttribute("role","none"),this.label=X(this.item,ne("span.action-label")),this.options.label&&this.options.keybinding&&(X(this.item,ne("span.keybinding")).textContent=this.options.keybinding),this.runOnceToEnableMouseUp.schedule(),this.updateClass(),this.updateLabel(),this.updateTooltip(),this.updateEnabled(),this.updateChecked(),this.applyStyle())}blur(){super.blur(),this.applyStyle()}focus(){var e;super.focus(),(e=this.item)==null||e.focus(),this.applyStyle()}updatePositionInSet(e,t){this.item&&(this.item.setAttribute("aria-posinset",`${e}`),this.item.setAttribute("aria-setsize",`${t}`))}updateLabel(){var e;if(this.label&&this.options.label){pn(this.label);let t=K5(this.action.label);if(t){const i=Xde(t);this.options.enableMnemonics||(t=i),this.label.setAttribute("aria-label",i.replace(/&&/g,"&"));const n=X2.exec(t);if(n){t=jC(t),YN.lastIndex=0;let s=YN.exec(t);for(;s&&s[1];)s=YN.exec(t);const r=a=>a.replace(/&amp;&amp;/g,"&amp;");s?this.label.append(M1(r(t.substr(0,s.index))," "),ne("u",{"aria-hidden":"true"},s[3]),eU(r(t.substr(s.index+s[0].length))," ")):this.label.innerText=r(t).trim(),(e=this.item)==null||e.setAttribute("aria-keyshortcuts",(n[1]?n[1]:n[3]).toLocaleLowerCase())}else this.label.innerText=t.replace(/&&/g,"&").trim()}}}updateTooltip(){}updateClass(){this.cssClass&&this.item&&this.item.classList.remove(...this.cssClass.split(" ")),this.options.icon&&this.label?(this.cssClass=this.action.class||"",this.label.classList.add("icon"),this.cssClass&&this.label.classList.add(...this.cssClass.split(" ")),this.updateEnabled()):this.label&&this.label.classList.remove("icon")}updateEnabled(){this.action.enabled?(this.element&&(this.element.classList.remove("disabled"),this.element.removeAttribute("aria-disabled")),this.item&&(this.item.classList.remove("disabled"),this.item.removeAttribute("aria-disabled"),this.item.tabIndex=0)):(this.element&&(this.element.classList.add("disabled"),this.element.setAttribute("aria-disabled","true")),this.item&&(this.item.classList.add("disabled"),this.item.setAttribute("aria-disabled","true")))}updateChecked(){if(!this.item)return;const e=this.action.checked;this.item.classList.toggle("checked",!!e),e!==void 0?(this.item.setAttribute("role","menuitemcheckbox"),this.item.setAttribute("aria-checked",e?"true":"false")):(this.item.setAttribute("role","menuitem"),this.item.setAttribute("aria-checked",""))}getMnemonic(){return this.mnemonic}applyStyle(){const e=this.element&&this.element.classList.contains("focused"),t=e&&this.menuStyle.selectionForegroundColor?this.menuStyle.selectionForegroundColor:this.menuStyle.foregroundColor,i=e&&this.menuStyle.selectionBackgroundColor?this.menuStyle.selectionBackgroundColor:void 0,n=e&&this.menuStyle.selectionBorderColor?`1px solid ${this.menuStyle.selectionBorderColor}`:"",s=e&&this.menuStyle.selectionBorderColor?"-1px":"";this.item&&(this.item.style.color=t??"",this.item.style.backgroundColor=i??"",this.item.style.outline=n,this.item.style.outlineOffset=s),this.check&&(this.check.style.color=t??"")}}class h8 extends Wj{constructor(e,t,i,n,s){super(e,e,n,s),this.submenuActions=t,this.parentData=i,this.submenuOptions=n,this.mysubmenu=null,this.submenuDisposables=this._register(new K),this.mouseOver=!1,this.expandDirection=n&&n.expandDirection!==void 0?n.expandDirection:{horizontal:kL.Right,vertical:Q2.Below},this.showScheduler=new It(()=>{this.mouseOver&&(this.cleanupExistingSubmenu(!1),this.createSubmenu(!1))},250),this.hideScheduler=new It(()=>{this.element&&!Zi(Dn(),this.element)&&this.parentData.submenu===this.mysubmenu&&(this.parentData.parent.focus(!1),this.cleanupExistingSubmenu(!0))},750)}render(e){super.render(e),this.element&&(this.item&&(this.item.classList.add("monaco-submenu-item"),this.item.tabIndex=0,this.item.setAttribute("aria-haspopup","true"),this.updateAriaExpanded("false"),this.submenuIndicator=X(this.item,ne("span.submenu-indicator"+Ie.asCSSSelector(te.menuSubmenu))),this.submenuIndicator.setAttribute("aria-hidden","true")),this._register(U(this.element,ie.KEY_UP,t=>{const i=new Pt(t);(i.equals(17)||i.equals(3))&&(nt.stop(t,!0),this.createSubmenu(!0))})),this._register(U(this.element,ie.KEY_DOWN,t=>{const i=new Pt(t);Dn()===this.item&&(i.equals(17)||i.equals(3))&&nt.stop(t,!0)})),this._register(U(this.element,ie.MOUSE_OVER,t=>{this.mouseOver||(this.mouseOver=!0,this.showScheduler.schedule())})),this._register(U(this.element,ie.MOUSE_LEAVE,t=>{this.mouseOver=!1})),this._register(U(this.element,ie.FOCUS_OUT,t=>{this.element&&!Zi(Dn(),this.element)&&this.hideScheduler.schedule()})),this._register(this.parentData.parent.onScroll(()=>{this.parentData.submenu===this.mysubmenu&&(this.parentData.parent.focus(!1),this.cleanupExistingSubmenu(!0))})))}updateEnabled(){}onClick(e){nt.stop(e,!0),this.cleanupExistingSubmenu(!1),this.createSubmenu(!0)}cleanupExistingSubmenu(e){if(this.parentData.submenu&&(e||this.parentData.submenu!==this.mysubmenu)){try{this.parentData.submenu.dispose()}catch{}this.parentData.submenu=void 0,this.updateAriaExpanded("false"),this.submenuContainer&&(this.submenuDisposables.clear(),this.submenuContainer=void 0)}}calculateSubmenuMenuLayout(e,t,i,n){const s={top:0,left:0};return s.left=op(e.width,t.width,{position:n.horizontal===kL.Right?0:1,offset:i.left,size:i.width}),s.left>=i.left&&s.left<i.left+i.width&&(i.left+10+t.width<=e.width&&(s.left=i.left+10),i.top+=10,i.height=0),s.top=op(e.height,t.height,{position:0,offset:i.top,size:0}),s.top+t.height===i.top&&s.top+i.height+t.height<=e.height&&(s.top+=i.height),s}createSubmenu(e=!0){if(this.element)if(this.parentData.submenu)this.parentData.submenu.focus(!1);else{this.updateAriaExpanded("true"),this.submenuContainer=X(this.element,ne("div.monaco-submenu")),this.submenuContainer.classList.add("menubar-menu-items-holder","context-view");const t=xe(this.parentData.parent.domNode).getComputedStyle(this.parentData.parent.domNode),i=parseFloat(t.paddingTop||"0")||0;this.submenuContainer.style.zIndex="1",this.submenuContainer.style.position="fixed",this.submenuContainer.style.top="0",this.submenuContainer.style.left="0",this.parentData.submenu=new Yp(this.submenuContainer,this.submenuActions.length?this.submenuActions:[new MM],this.submenuOptions,this.menuStyle);const n=this.element.getBoundingClientRect(),s={top:n.top-i,left:n.left,height:n.height+2*i,width:n.width},r=this.submenuContainer.getBoundingClientRect(),a=xe(this.element),{top:l,left:c}=this.calculateSubmenuMenuLayout(new kt(a.innerWidth,a.innerHeight),kt.lift(r),s,this.expandDirection);this.submenuContainer.style.left=`${c-r.left}px`,this.submenuContainer.style.top=`${l-r.top}px`,this.submenuDisposables.add(U(this.submenuContainer,ie.KEY_UP,d=>{new Pt(d).equals(15)&&(nt.stop(d,!0),this.parentData.parent.focus(),this.cleanupExistingSubmenu(!0))})),this.submenuDisposables.add(U(this.submenuContainer,ie.KEY_DOWN,d=>{new Pt(d).equals(15)&&nt.stop(d,!0)})),this.submenuDisposables.add(this.parentData.submenu.onDidCancel(()=>{this.parentData.parent.focus(),this.cleanupExistingSubmenu(!0)})),this.parentData.submenu.focus(e),this.mysubmenu=this.parentData.submenu}}updateAriaExpanded(e){var t;this.item&&((t=this.item)==null||t.setAttribute("aria-expanded",e))}applyStyle(){super.applyStyle();const t=this.element&&this.element.classList.contains("focused")&&this.menuStyle.selectionForegroundColor?this.menuStyle.selectionForegroundColor:this.menuStyle.foregroundColor;this.submenuIndicator&&(this.submenuIndicator.style.color=t??"")}dispose(){super.dispose(),this.hideScheduler.dispose(),this.mysubmenu&&(this.mysubmenu.dispose(),this.mysubmenu=null),this.submenuContainer&&(this.submenuContainer=void 0)}}class u8 extends V_{constructor(e,t,i,n){super(e,t,i),this.menuStyles=n}render(e){super.render(e),this.label&&(this.label.style.borderBottomColor=this.menuStyles.separatorColor?`${this.menuStyles.separatorColor}`:"")}}function Xde(o){const e=X2,t=e.exec(o);if(!t)return o;const i=!t[1];return o.replace(e,i?"$2$3":"").trim()}function g8(o){const e=Gz()[o.id];return`.codicon-${o.id}:before { content: '\\${e.toString(16)}'; }`}function Qde(o,e){let t=`
.monaco-menu {
	font-size: 13px;
	border-radius: 5px;
	min-width: 160px;
}

${g8(te.menuSelection)}
${g8(te.menuSubmenu)}

.monaco-menu .monaco-action-bar {
	text-align: right;
	overflow: hidden;
	white-space: nowrap;
}

.monaco-menu .monaco-action-bar .actions-container {
	display: flex;
	margin: 0 auto;
	padding: 0;
	width: 100%;
	justify-content: flex-end;
}

.monaco-menu .monaco-action-bar.vertical .actions-container {
	display: inline-block;
}

.monaco-menu .monaco-action-bar.reverse .actions-container {
	flex-direction: row-reverse;
}

.monaco-menu .monaco-action-bar .action-item {
	cursor: pointer;
	display: inline-block;
	transition: transform 50ms ease;
	position: relative;  /* DO NOT REMOVE - this is the key to preventing the ghosting icon bug in Chrome 42 */
}

.monaco-menu .monaco-action-bar .action-item.disabled {
	cursor: default;
}

.monaco-menu .monaco-action-bar .action-item .icon,
.monaco-menu .monaco-action-bar .action-item .codicon {
	display: inline-block;
}

.monaco-menu .monaco-action-bar .action-item .codicon {
	display: flex;
	align-items: center;
}

.monaco-menu .monaco-action-bar .action-label {
	font-size: 11px;
	margin-right: 4px;
}

.monaco-menu .monaco-action-bar .action-item.disabled .action-label,
.monaco-menu .monaco-action-bar .action-item.disabled .action-label:hover {
	color: var(--vscode-disabledForeground);
}

/* Vertical actions */

.monaco-menu .monaco-action-bar.vertical {
	text-align: left;
}

.monaco-menu .monaco-action-bar.vertical .action-item {
	display: block;
}

.monaco-menu .monaco-action-bar.vertical .action-label.separator {
	display: block;
	border-bottom: 1px solid var(--vscode-menu-separatorBackground);
	padding-top: 1px;
	padding: 30px;
}

.monaco-menu .secondary-actions .monaco-action-bar .action-label {
	margin-left: 6px;
}

/* Action Items */
.monaco-menu .monaco-action-bar .action-item.select-container {
	overflow: hidden; /* somehow the dropdown overflows its container, we prevent it here to not push */
	flex: 1;
	max-width: 170px;
	min-width: 60px;
	display: flex;
	align-items: center;
	justify-content: center;
	margin-right: 10px;
}

.monaco-menu .monaco-action-bar.vertical {
	margin-left: 0;
	overflow: visible;
}

.monaco-menu .monaco-action-bar.vertical .actions-container {
	display: block;
}

.monaco-menu .monaco-action-bar.vertical .action-item {
	padding: 0;
	transform: none;
	display: flex;
}

.monaco-menu .monaco-action-bar.vertical .action-item.active {
	transform: none;
}

.monaco-menu .monaco-action-bar.vertical .action-menu-item {
	flex: 1 1 auto;
	display: flex;
	height: 2em;
	align-items: center;
	position: relative;
	margin: 0 4px;
	border-radius: 4px;
}

.monaco-menu .monaco-action-bar.vertical .action-menu-item:hover .keybinding,
.monaco-menu .monaco-action-bar.vertical .action-menu-item:focus .keybinding {
	opacity: unset;
}

.monaco-menu .monaco-action-bar.vertical .action-label {
	flex: 1 1 auto;
	text-decoration: none;
	padding: 0 1em;
	background: none;
	font-size: 12px;
	line-height: 1;
}

.monaco-menu .monaco-action-bar.vertical .keybinding,
.monaco-menu .monaco-action-bar.vertical .submenu-indicator {
	display: inline-block;
	flex: 2 1 auto;
	padding: 0 1em;
	text-align: right;
	font-size: 12px;
	line-height: 1;
}

.monaco-menu .monaco-action-bar.vertical .submenu-indicator {
	height: 100%;
}

.monaco-menu .monaco-action-bar.vertical .submenu-indicator.codicon {
	font-size: 16px !important;
	display: flex;
	align-items: center;
}

.monaco-menu .monaco-action-bar.vertical .submenu-indicator.codicon::before {
	margin-left: auto;
	margin-right: -20px;
}

.monaco-menu .monaco-action-bar.vertical .action-item.disabled .keybinding,
.monaco-menu .monaco-action-bar.vertical .action-item.disabled .submenu-indicator {
	opacity: 0.4;
}

.monaco-menu .monaco-action-bar.vertical .action-label:not(.separator) {
	display: inline-block;
	box-sizing: border-box;
	margin: 0;
}

.monaco-menu .monaco-action-bar.vertical .action-item {
	position: static;
	overflow: visible;
}

.monaco-menu .monaco-action-bar.vertical .action-item .monaco-submenu {
	position: absolute;
}

.monaco-menu .monaco-action-bar.vertical .action-label.separator {
	width: 100%;
	height: 0px !important;
	opacity: 1;
}

.monaco-menu .monaco-action-bar.vertical .action-label.separator.text {
	padding: 0.7em 1em 0.1em 1em;
	font-weight: bold;
	opacity: 1;
}

.monaco-menu .monaco-action-bar.vertical .action-label:hover {
	color: inherit;
}

.monaco-menu .monaco-action-bar.vertical .menu-item-check {
	position: absolute;
	visibility: hidden;
	width: 1em;
	height: 100%;
}

.monaco-menu .monaco-action-bar.vertical .action-menu-item.checked .menu-item-check {
	visibility: visible;
	display: flex;
	align-items: center;
	justify-content: center;
}

/* Context Menu */

.context-view.monaco-menu-container {
	outline: 0;
	border: none;
	animation: fadeIn 0.083s linear;
	-webkit-app-region: no-drag;
}

.context-view.monaco-menu-container :focus,
.context-view.monaco-menu-container .monaco-action-bar.vertical:focus,
.context-view.monaco-menu-container .monaco-action-bar.vertical :focus {
	outline: 0;
}

.hc-black .context-view.monaco-menu-container,
.hc-light .context-view.monaco-menu-container,
:host-context(.hc-black) .context-view.monaco-menu-container,
:host-context(.hc-light) .context-view.monaco-menu-container {
	box-shadow: none;
}

.hc-black .monaco-menu .monaco-action-bar.vertical .action-item.focused,
.hc-light .monaco-menu .monaco-action-bar.vertical .action-item.focused,
:host-context(.hc-black) .monaco-menu .monaco-action-bar.vertical .action-item.focused,
:host-context(.hc-light) .monaco-menu .monaco-action-bar.vertical .action-item.focused {
	background: none;
}

/* Vertical Action Bar Styles */

.monaco-menu .monaco-action-bar.vertical {
	padding: 4px 0;
}

.monaco-menu .monaco-action-bar.vertical .action-menu-item {
	height: 2em;
}

.monaco-menu .monaco-action-bar.vertical .action-label:not(.separator),
.monaco-menu .monaco-action-bar.vertical .keybinding {
	font-size: inherit;
	padding: 0 2em;
	max-height: 100%;
}

.monaco-menu .monaco-action-bar.vertical .menu-item-check {
	font-size: inherit;
	width: 2em;
}

.monaco-menu .monaco-action-bar.vertical .action-label.separator {
	font-size: inherit;
	margin: 5px 0 !important;
	padding: 0;
	border-radius: 0;
}

.linux .monaco-menu .monaco-action-bar.vertical .action-label.separator,
:host-context(.linux) .monaco-menu .monaco-action-bar.vertical .action-label.separator {
	margin-left: 0;
	margin-right: 0;
}

.monaco-menu .monaco-action-bar.vertical .submenu-indicator {
	font-size: 60%;
	padding: 0 1.8em;
}

.linux .monaco-menu .monaco-action-bar.vertical .submenu-indicator,
:host-context(.linux) .monaco-menu .monaco-action-bar.vertical .submenu-indicator {
	height: 100%;
	mask-size: 10px 10px;
	-webkit-mask-size: 10px 10px;
}

.monaco-menu .action-item {
	cursor: default;
}`;if(e){t+=`
			/* Arrows */
			.monaco-scrollable-element > .scrollbar > .scra {
				cursor: pointer;
				font-size: 11px !important;
			}

			.monaco-scrollable-element > .visible {
				opacity: 1;

				/* Background rule added for IE9 - to allow clicks on dom node */
				background:rgba(0,0,0,0);

				transition: opacity 100ms linear;
			}
			.monaco-scrollable-element > .invisible {
				opacity: 0;
				pointer-events: none;
			}
			.monaco-scrollable-element > .invisible.fade {
				transition: opacity 800ms linear;
			}

			/* Scrollable Content Inset Shadow */
			.monaco-scrollable-element > .shadow {
				position: absolute;
				display: none;
			}
			.monaco-scrollable-element > .shadow.top {
				display: block;
				top: 0;
				left: 3px;
				height: 3px;
				width: 100%;
			}
			.monaco-scrollable-element > .shadow.left {
				display: block;
				top: 3px;
				left: 0;
				height: 100%;
				width: 3px;
			}
			.monaco-scrollable-element > .shadow.top-left-corner {
				display: block;
				top: 0;
				left: 0;
				height: 3px;
				width: 3px;
			}
		`;const i=o.scrollbarShadow;i&&(t+=`
				.monaco-scrollable-element > .shadow.top {
					box-shadow: ${i} 0 6px 6px -6px inset;
				}

				.monaco-scrollable-element > .shadow.left {
					box-shadow: ${i} 6px 0 6px -6px inset;
				}

				.monaco-scrollable-element > .shadow.top.left {
					box-shadow: ${i} 6px 6px 6px -6px inset;
				}
			`);const n=o.scrollbarSliderBackground;n&&(t+=`
				.monaco-scrollable-element > .scrollbar > .slider {
					background: ${n};
				}
			`);const s=o.scrollbarSliderHoverBackground;s&&(t+=`
				.monaco-scrollable-element > .scrollbar > .slider:hover {
					background: ${s};
				}
			`);const r=o.scrollbarSliderActiveBackground;r&&(t+=`
				.monaco-scrollable-element > .scrollbar > .slider.active {
					background: ${r};
				}
			`)}return t}class Jde{constructor(e,t,i,n){this.contextViewService=e,this.telemetryService=t,this.notificationService=i,this.keybindingService=n,this.focusToReturn=null,this.lastContainer=null,this.block=null,this.blockDisposable=null,this.options={blockMouse:!0}}configure(e){this.options=e}showContextMenu(e){const t=e.getActions();if(!t.length)return;this.focusToReturn=Dn();let i;const n=Xi(e.domForShadowRoot)?e.domForShadowRoot:void 0;this.contextViewService.showContextView({getAnchor:()=>e.getAnchor(),canRelayout:!1,anchorAlignment:e.anchorAlignment,anchorAxisAlignment:e.anchorAxisAlignment,render:s=>{var d;this.lastContainer=s;const r=e.getMenuClassName?e.getMenuClassName():"";r&&(s.className+=" "+r),this.options.blockMouse&&(this.block=s.appendChild(ne(".context-view-block")),this.block.style.position="fixed",this.block.style.cursor="initial",this.block.style.left="0",this.block.style.top="0",this.block.style.width="100%",this.block.style.height="100%",this.block.style.zIndex="-1",(d=this.blockDisposable)==null||d.dispose(),this.blockDisposable=U(this.block,ie.MOUSE_DOWN,h=>h.stopPropagation()));const a=new K,l=e.actionRunner||new Ef;l.onWillRun(h=>this.onActionRun(h,!e.skipTelemetry),this,a),l.onDidRun(this.onDidActionRun,this,a),i=new Yp(s,t,{actionViewItemProvider:e.getActionViewItem,context:e.getActionsContext?e.getActionsContext():null,actionRunner:l,getKeyBinding:e.getKeyBinding?e.getKeyBinding:h=>this.keybindingService.lookupKeybinding(h.id)},Zde),i.onDidCancel(()=>this.contextViewService.hideContextView(!0),null,a),i.onDidBlur(()=>this.contextViewService.hideContextView(!0),null,a);const c=xe(s);return a.add(U(c,ie.BLUR,()=>this.contextViewService.hideContextView(!0))),a.add(U(c,ie.MOUSE_DOWN,h=>{if(h.defaultPrevented)return;const u=new Vr(c,h);let g=u.target;if(!u.rightButton){for(;g;){if(g===s)return;g=g.parentElement}this.contextViewService.hideContextView(!0)}})),jr(a,i)},focus:()=>{i==null||i.focus(!!e.autoSelectFirstItem)},onHide:s=>{var r,a,l;(r=e.onHide)==null||r.call(e,!!s),this.block&&(this.block.remove(),this.block=null),(a=this.blockDisposable)==null||a.dispose(),this.blockDisposable=null,this.lastContainer&&(Dn()===this.lastContainer||Zi(Dn(),this.lastContainer))&&((l=this.focusToReturn)==null||l.focus()),this.lastContainer=null}},n,!!n)}onActionRun(e,t){t&&this.telemetryService.publicLog2("workbenchActionExecuted",{id:e.action.id,from:"contextMenu"}),this.contextViewService.hideContextView(!1)}onDidActionRun(e){e.error&&!yr(e.error)&&this.notificationService.error(e.error)}}var ehe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Dm=function(o,e){return function(t,i){e(t,i,o)}};let J2=class extends F{get contextMenuHandler(){return this._contextMenuHandler||(this._contextMenuHandler=new Jde(this.contextViewService,this.telemetryService,this.notificationService,this.keybindingService)),this._contextMenuHandler}constructor(e,t,i,n,s,r){super(),this.telemetryService=e,this.notificationService=t,this.contextViewService=i,this.keybindingService=n,this.menuService=s,this.contextKeyService=r,this._contextMenuHandler=void 0,this._onDidShowContextMenu=this._store.add(new P),this.onDidShowContextMenu=this._onDidShowContextMenu.event,this._onDidHideContextMenu=this._store.add(new P)}configure(e){this.contextMenuHandler.configure(e)}showContextMenu(e){e=eA.transform(e,this.menuService,this.contextKeyService),this.contextMenuHandler.showContextMenu({...e,onHide:t=>{var i;(i=e.onHide)==null||i.call(e,t),this._onDidHideContextMenu.fire()}}),Rc.getInstance().resetKeyStatus(),this._onDidShowContextMenu.fire()}};J2=ehe([Dm(0,Bn),Dm(1,Ei),Dm(2,ud),Dm(3,yt),Dm(4,Go),Dm(5,Ee)],J2);var eA;(function(o){function e(i){return i&&i.menuId instanceof me}function t(i,n,s){if(!e(i))return i;const{menuId:r,menuActionOptions:a,contextKeyService:l}=i;return{...i,getActions:()=>{const c=[];if(r){const d=n.getMenuActions(r,l??s,a);Yde(d,c)}return i.getActions?Qi.join(i.getActions(),c):c}}}o.transform=t})(eA||(eA={}));var DL;(function(o){o[o.API=0]="API",o[o.USER=1]="USER"})(DL||(DL={}));var n3=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},IL=function(o,e){return function(t,i){e(t,i,o)}};let tA=class{constructor(e){this._commandService=e}async open(e,t){if(!mI(e,Fe.command))return!1;if(!(t!=null&&t.allowCommands)||(typeof e=="string"&&(e=Se.parse(e)),Array.isArray(t.allowCommands)&&!t.allowCommands.includes(e.path)))return!0;let i=[];try{i=w2(decodeURIComponent(e.query))}catch{try{i=w2(e.query)}catch{}}return Array.isArray(i)||(i=[i]),await this._commandService.executeCommand(e.path,...i),!0}};tA=n3([IL(0,Ht)],tA);let iA=class{constructor(e){this._editorService=e}async open(e,t){typeof e=="string"&&(e=Se.parse(e));const{selection:i,uri:n}=Dae(e);return e=n,e.scheme===Fe.file&&(e=cle(e)),await this._editorService.openCodeEditor({resource:e,options:{selection:i,source:t!=null&&t.fromUserGesture?DL.USER:DL.API,...t==null?void 0:t.editorOptions}},this._editorService.getFocusedCodeEditor(),t==null?void 0:t.openToSide),!0}};iA=n3([IL(0,pt)],iA);let nA=class{constructor(e,t){this._openers=new cs,this._validators=new cs,this._resolvers=new cs,this._resolvedUriTargets=new an(i=>i.with({path:null,fragment:null,query:null}).toString()),this._externalOpeners=new cs,this._defaultExternalOpener={openExternal:async i=>(kM(i,Fe.http,Fe.https)?RU(i):Rt.location.href=i,!0)},this._openers.push({open:async(i,n)=>n!=null&&n.openExternal||kM(i,Fe.mailto,Fe.http,Fe.https,Fe.vsls)?(await this._doOpenExternal(i,n),!0):!1}),this._openers.push(new tA(t)),this._openers.push(new iA(e))}registerOpener(e){return{dispose:this._openers.unshift(e)}}async open(e,t){const i=typeof e=="string"?Se.parse(e):e,n=this._resolvedUriTargets.get(i)??e;for(const s of this._validators)if(!await s.shouldOpen(n,t))return!1;for(const s of this._openers)if(await s.open(e,t))return!0;return!1}async resolveExternalUri(e,t){for(const i of this._resolvers)try{const n=await i.resolveExternalUri(e,t);if(n)return this._resolvedUriTargets.has(n.resolved)||this._resolvedUriTargets.set(n.resolved,e),n}catch{}throw new Error("Could not resolve external URI: "+e.toString())}async _doOpenExternal(e,t){const i=typeof e=="string"?Se.parse(e):e;let n;try{n=(await this.resolveExternalUri(i,t)).resolved}catch{n=i}let s;if(typeof e=="string"&&i.toString()===n.toString()?s=e:s=encodeURI(n.toString(!0)),t!=null&&t.allowContributedOpeners){const r=typeof(t==null?void 0:t.allowContributedOpeners)=="string"?t==null?void 0:t.allowContributedOpeners:void 0;for(const a of this._externalOpeners)if(await a.openExternal(s,{sourceUri:i,preferredOpenerId:r},ze.None))return!0}return this._defaultExternalOpener.openExternal(s,{sourceUri:i},ze.None)}dispose(){this._validators.clear()}};nA=n3([IL(0,pt),IL(1,Ht)],nA);const Us=et("editorWorkerService");var pi;(function(o){o[o.Hint=1]="Hint",o[o.Info=2]="Info",o[o.Warning=4]="Warning",o[o.Error=8]="Error"})(pi||(pi={}));(function(o){function e(r,a){return a-r}o.compare=e;const t=Object.create(null);t[o.Error]=m("sev.error","Error"),t[o.Warning]=m("sev.warning","Warning"),t[o.Info]=m("sev.info","Info");function i(r){return t[r]||""}o.toString=i;function n(r){switch(r){case Ii.Error:return o.Error;case Ii.Warning:return o.Warning;case Ii.Info:return o.Info;case Ii.Ignore:return o.Hint}}o.fromSeverity=n;function s(r){switch(r){case o.Error:return Ii.Error;case o.Warning:return Ii.Warning;case o.Info:return Ii.Info;case o.Hint:return Ii.Ignore}}o.toSeverity=s})(pi||(pi={}));var EL;(function(o){const e="";function t(n){return i(n,!0)}o.makeKey=t;function i(n,s){const r=[e];return n.source?r.push(n.source.replace("¦","\\¦")):r.push(e),n.code?typeof n.code=="string"?r.push(n.code.replace("¦","\\¦")):r.push(n.code.value.replace("¦","\\¦")):r.push(e),n.severity!==void 0&&n.severity!==null?r.push(pi.toString(n.severity)):r.push(e),n.message&&s?r.push(n.message.replace("¦","\\¦")):r.push(e),n.startLineNumber!==void 0&&n.startLineNumber!==null?r.push(n.startLineNumber.toString()):r.push(e),n.startColumn!==void 0&&n.startColumn!==null?r.push(n.startColumn.toString()):r.push(e),n.endLineNumber!==void 0&&n.endLineNumber!==null?r.push(n.endLineNumber.toString()):r.push(e),n.endColumn!==void 0&&n.endColumn!==null?r.push(n.endColumn.toString()):r.push(e),r.push(e),r.join("¦")}o.makeKeyOptionalMessage=i})(EL||(EL={}));const Xl=et("markerService"),Bj=E("editor.lineHighlightBackground",null,m("lineHighlight","Background color for the highlight of line at the cursor position.")),f8=E("editor.lineHighlightBorder",{dark:"#282828",light:"#eeeeee",hcDark:"#f38518",hcLight:lt},m("lineHighlightBorderBox","Background color for the border around the line at the cursor position."));E("editor.rangeHighlightBackground",{dark:"#ffffff0b",light:"#fdff0033",hcDark:null,hcLight:null},m("rangeHighlight","Background color of highlighted ranges, like by quick open and find features. The color must not be opaque so as not to hide underlying decorations."),!0);E("editor.rangeHighlightBorder",{dark:null,light:null,hcDark:Xt,hcLight:Xt},m("rangeHighlightBorder","Background color of the border around highlighted ranges."));E("editor.symbolHighlightBackground",{dark:Ac,light:Ac,hcDark:null,hcLight:null},m("symbolHighlight","Background color of highlighted symbol, like for go to definition or go next/previous symbol. The color must not be opaque so as not to hide underlying decorations."),!0);E("editor.symbolHighlightBorder",{dark:null,light:null,hcDark:Xt,hcLight:Xt},m("symbolHighlightBorder","Background color of the border around highlighted symbols."));const oE=E("editorCursor.foreground",{dark:"#AEAFAD",light:Z.black,hcDark:Z.white,hcLight:"#0F4A85"},m("caret","Color of the editor cursor.")),s3=E("editorCursor.background",null,m("editorCursorBackground","The background color of the editor cursor. Allows customizing the color of a character overlapped by a block cursor.")),Hj=E("editorMultiCursor.primary.foreground",oE,m("editorMultiCursorPrimaryForeground","Color of the primary editor cursor when multiple cursors are present.")),the=E("editorMultiCursor.primary.background",s3,m("editorMultiCursorPrimaryBackground","The background color of the primary editor cursor when multiple cursors are present. Allows customizing the color of a character overlapped by a block cursor.")),Vj=E("editorMultiCursor.secondary.foreground",oE,m("editorMultiCursorSecondaryForeground","Color of secondary editor cursors when multiple cursors are present.")),ihe=E("editorMultiCursor.secondary.background",s3,m("editorMultiCursorSecondaryBackground","The background color of secondary editor cursors when multiple cursors are present. Allows customizing the color of a character overlapped by a block cursor.")),o3=E("editorWhitespace.foreground",{dark:"#e3e4e229",light:"#33333333",hcDark:"#e3e4e229",hcLight:"#CCCCCC"},m("editorWhitespaces","Color of whitespace characters in the editor.")),nhe=E("editorLineNumber.foreground",{dark:"#858585",light:"#237893",hcDark:Z.white,hcLight:"#292929"},m("editorLineNumbers","Color of editor line numbers.")),she=E("editorIndentGuide.background",o3,m("editorIndentGuides","Color of the editor indentation guides."),!1,m("deprecatedEditorIndentGuides","'editorIndentGuide.background' is deprecated. Use 'editorIndentGuide.background1' instead.")),ohe=E("editorIndentGuide.activeBackground",o3,m("editorActiveIndentGuide","Color of the active editor indentation guides."),!1,m("deprecatedEditorActiveIndentGuide","'editorIndentGuide.activeBackground' is deprecated. Use 'editorIndentGuide.activeBackground1' instead.")),J1=E("editorIndentGuide.background1",she,m("editorIndentGuides1","Color of the editor indentation guides (1).")),rhe=E("editorIndentGuide.background2","#00000000",m("editorIndentGuides2","Color of the editor indentation guides (2).")),ahe=E("editorIndentGuide.background3","#00000000",m("editorIndentGuides3","Color of the editor indentation guides (3).")),lhe=E("editorIndentGuide.background4","#00000000",m("editorIndentGuides4","Color of the editor indentation guides (4).")),che=E("editorIndentGuide.background5","#00000000",m("editorIndentGuides5","Color of the editor indentation guides (5).")),dhe=E("editorIndentGuide.background6","#00000000",m("editorIndentGuides6","Color of the editor indentation guides (6).")),eS=E("editorIndentGuide.activeBackground1",ohe,m("editorActiveIndentGuide1","Color of the active editor indentation guides (1).")),hhe=E("editorIndentGuide.activeBackground2","#00000000",m("editorActiveIndentGuide2","Color of the active editor indentation guides (2).")),uhe=E("editorIndentGuide.activeBackground3","#00000000",m("editorActiveIndentGuide3","Color of the active editor indentation guides (3).")),ghe=E("editorIndentGuide.activeBackground4","#00000000",m("editorActiveIndentGuide4","Color of the active editor indentation guides (4).")),fhe=E("editorIndentGuide.activeBackground5","#00000000",m("editorActiveIndentGuide5","Color of the active editor indentation guides (5).")),mhe=E("editorIndentGuide.activeBackground6","#00000000",m("editorActiveIndentGuide6","Color of the active editor indentation guides (6).")),phe=E("editorActiveLineNumber.foreground",{dark:"#c6c6c6",light:"#0B216F",hcDark:Xt,hcLight:Xt},m("editorActiveLineNumber","Color of editor active line number"),!1,m("deprecatedEditorActiveLineNumber","Id is deprecated. Use 'editorLineNumber.activeForeground' instead."));E("editorLineNumber.activeForeground",phe,m("editorActiveLineNumber","Color of editor active line number"));const _he=E("editorLineNumber.dimmedForeground",null,m("editorDimmedLineNumber","Color of the final editor line when editor.renderFinalNewline is set to dimmed."));E("editorRuler.foreground",{dark:"#5A5A5A",light:Z.lightgrey,hcDark:Z.white,hcLight:"#292929"},m("editorRuler","Color of the editor rulers."));E("editorCodeLens.foreground",{dark:"#999999",light:"#919191",hcDark:"#999999",hcLight:"#292929"},m("editorCodeLensForeground","Foreground color of editor CodeLens"));E("editorBracketMatch.background",{dark:"#0064001a",light:"#0064001a",hcDark:"#0064001a",hcLight:"#0000"},m("editorBracketMatchBackground","Background color behind matching brackets"));E("editorBracketMatch.border",{dark:"#888",light:"#B9B9B9",hcDark:lt,hcLight:lt},m("editorBracketMatchBorder","Color for matching brackets boxes"));const bhe=E("editorOverviewRuler.border",{dark:"#7f7f7f4d",light:"#7f7f7f4d",hcDark:"#7f7f7f4d",hcLight:"#666666"},m("editorOverviewRulerBorder","Color of the overview ruler border.")),Che=E("editorOverviewRuler.background",null,m("editorOverviewRulerBackground","Background color of the editor overview ruler."));E("editorGutter.background",Xr,m("editorGutter","Background color of the editor gutter. The gutter contains the glyph margins and the line numbers."));E("editorUnnecessaryCode.border",{dark:null,light:null,hcDark:Z.fromHex("#fff").transparent(.8),hcLight:lt},m("unnecessaryCodeBorder","Border color of unnecessary (unused) source code in the editor."));const vhe=E("editorUnnecessaryCode.opacity",{dark:Z.fromHex("#000a"),light:Z.fromHex("#0007"),hcDark:null,hcLight:null},m("unnecessaryCodeOpacity",`Opacity of unnecessary (unused) source code in the editor. For example, "#000000c0" will render the code with 75% opacity. For high contrast themes, use the  'editorUnnecessaryCode.border' theme color to underline unnecessary code instead of fading it out.`));E("editorGhostText.border",{dark:null,light:null,hcDark:Z.fromHex("#fff").transparent(.8),hcLight:Z.fromHex("#292929").transparent(.8)},m("editorGhostTextBorder","Border color of ghost text in the editor."));const whe=E("editorGhostText.foreground",{dark:Z.fromHex("#ffffff56"),light:Z.fromHex("#0007"),hcDark:null,hcLight:null},m("editorGhostTextForeground","Foreground color of the ghost text in the editor."));E("editorGhostText.background",null,m("editorGhostTextBackground","Background color of the ghost text in the editor."));const She=new Z(new dt(0,122,204,.6)),zj=E("editorOverviewRuler.rangeHighlightForeground",She,m("overviewRulerRangeHighlight","Overview ruler marker color for range highlights. The color must not be opaque so as not to hide underlying decorations."),!0),yhe=E("editorOverviewRuler.errorForeground",{dark:new Z(new dt(255,18,18,.7)),light:new Z(new dt(255,18,18,.7)),hcDark:new Z(new dt(255,50,50,1)),hcLight:"#B5200D"},m("overviewRuleError","Overview ruler marker color for errors.")),Lhe=E("editorOverviewRuler.warningForeground",{dark:id,light:id,hcDark:pw,hcLight:pw},m("overviewRuleWarning","Overview ruler marker color for warnings.")),xhe=E("editorOverviewRuler.infoForeground",{dark:Bl,light:Bl,hcDark:_w,hcLight:_w},m("overviewRuleInfo","Overview ruler marker color for infos.")),Uj=E("editorBracketHighlight.foreground1",{dark:"#FFD700",light:"#0431FAFF",hcDark:"#FFD700",hcLight:"#0431FAFF"},m("editorBracketHighlightForeground1","Foreground color of brackets (1). Requires enabling bracket pair colorization.")),$j=E("editorBracketHighlight.foreground2",{dark:"#DA70D6",light:"#319331FF",hcDark:"#DA70D6",hcLight:"#319331FF"},m("editorBracketHighlightForeground2","Foreground color of brackets (2). Requires enabling bracket pair colorization.")),jj=E("editorBracketHighlight.foreground3",{dark:"#179FFF",light:"#7B3814FF",hcDark:"#87CEFA",hcLight:"#7B3814FF"},m("editorBracketHighlightForeground3","Foreground color of brackets (3). Requires enabling bracket pair colorization.")),Kj=E("editorBracketHighlight.foreground4","#00000000",m("editorBracketHighlightForeground4","Foreground color of brackets (4). Requires enabling bracket pair colorization.")),qj=E("editorBracketHighlight.foreground5","#00000000",m("editorBracketHighlightForeground5","Foreground color of brackets (5). Requires enabling bracket pair colorization.")),Gj=E("editorBracketHighlight.foreground6","#00000000",m("editorBracketHighlightForeground6","Foreground color of brackets (6). Requires enabling bracket pair colorization.")),khe=E("editorBracketHighlight.unexpectedBracket.foreground",{dark:new Z(new dt(255,18,18,.8)),light:new Z(new dt(255,18,18,.8)),hcDark:"new Color(new RGBA(255, 50, 50, 1))",hcLight:"#B5200D"},m("editorBracketHighlightUnexpectedBracketForeground","Foreground color of unexpected brackets.")),Dhe=E("editorBracketPairGuide.background1","#00000000",m("editorBracketPairGuide.background1","Background color of inactive bracket pair guides (1). Requires enabling bracket pair guides.")),Ihe=E("editorBracketPairGuide.background2","#00000000",m("editorBracketPairGuide.background2","Background color of inactive bracket pair guides (2). Requires enabling bracket pair guides.")),Ehe=E("editorBracketPairGuide.background3","#00000000",m("editorBracketPairGuide.background3","Background color of inactive bracket pair guides (3). Requires enabling bracket pair guides.")),Nhe=E("editorBracketPairGuide.background4","#00000000",m("editorBracketPairGuide.background4","Background color of inactive bracket pair guides (4). Requires enabling bracket pair guides.")),The=E("editorBracketPairGuide.background5","#00000000",m("editorBracketPairGuide.background5","Background color of inactive bracket pair guides (5). Requires enabling bracket pair guides.")),Rhe=E("editorBracketPairGuide.background6","#00000000",m("editorBracketPairGuide.background6","Background color of inactive bracket pair guides (6). Requires enabling bracket pair guides.")),Mhe=E("editorBracketPairGuide.activeBackground1","#00000000",m("editorBracketPairGuide.activeBackground1","Background color of active bracket pair guides (1). Requires enabling bracket pair guides.")),Ahe=E("editorBracketPairGuide.activeBackground2","#00000000",m("editorBracketPairGuide.activeBackground2","Background color of active bracket pair guides (2). Requires enabling bracket pair guides.")),Phe=E("editorBracketPairGuide.activeBackground3","#00000000",m("editorBracketPairGuide.activeBackground3","Background color of active bracket pair guides (3). Requires enabling bracket pair guides.")),Ohe=E("editorBracketPairGuide.activeBackground4","#00000000",m("editorBracketPairGuide.activeBackground4","Background color of active bracket pair guides (4). Requires enabling bracket pair guides.")),Fhe=E("editorBracketPairGuide.activeBackground5","#00000000",m("editorBracketPairGuide.activeBackground5","Background color of active bracket pair guides (5). Requires enabling bracket pair guides.")),Whe=E("editorBracketPairGuide.activeBackground6","#00000000",m("editorBracketPairGuide.activeBackground6","Background color of active bracket pair guides (6). Requires enabling bracket pair guides."));E("editorUnicodeHighlight.border",id,m("editorUnicodeHighlight.border","Border color used to highlight unicode characters."));E("editorUnicodeHighlight.background",Ooe,m("editorUnicodeHighlight.background","Background color used to highlight unicode characters."));Lr((o,e)=>{const t=o.getColor(Xr),i=o.getColor(Bj),n=i&&!i.isTransparent()?i:t;n&&e.addRule(`.monaco-editor .inputarea.ime-input { background-color: ${n}; }`)});function Bhe(o,e){const t=[],i=[];for(const n of o)e.has(n)||t.push(n);for(const n of e)o.has(n)||i.push(n);return{removed:t,added:i}}function Hhe(o,e){const t=new Set;for(const i of e)o.has(i)&&t.add(i);return t}var Vhe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},m8=function(o,e){return function(t,i){e(t,i,o)}};let sA=class extends F{constructor(e,t){super(),this._markerService=t,this._onDidChangeMarker=this._register(new P),this._markerDecorations=new an,e.getModels().forEach(i=>this._onModelAdded(i)),this._register(e.onModelAdded(this._onModelAdded,this)),this._register(e.onModelRemoved(this._onModelRemoved,this)),this._register(this._markerService.onMarkerChanged(this._handleMarkerChange,this))}dispose(){super.dispose(),this._markerDecorations.forEach(e=>e.dispose()),this._markerDecorations.clear()}getMarker(e,t){const i=this._markerDecorations.get(e);return i&&i.getMarker(t)||null}_handleMarkerChange(e){e.forEach(t=>{const i=this._markerDecorations.get(t);i&&this._updateDecorations(i)})}_onModelAdded(e){const t=new zhe(e);this._markerDecorations.set(e.uri,t),this._updateDecorations(t)}_onModelRemoved(e){var i;const t=this._markerDecorations.get(e.uri);t&&(t.dispose(),this._markerDecorations.delete(e.uri)),(e.uri.scheme===Fe.inMemory||e.uri.scheme===Fe.internal||e.uri.scheme===Fe.vscode)&&((i=this._markerService)==null||i.read({resource:e.uri}).map(n=>n.owner).forEach(n=>this._markerService.remove(n,[e.uri])))}_updateDecorations(e){const t=this._markerService.read({resource:e.model.uri,take:500});e.update(t)&&this._onDidChangeMarker.fire(e.model)}};sA=Vhe([m8(0,Kt),m8(1,Xl)],sA);class zhe extends F{constructor(e){super(),this.model=e,this._map=new Une,this._register(ve(()=>{this.model.deltaDecorations([...this._map.values()],[]),this._map.clear()}))}update(e){const{added:t,removed:i}=Bhe(new Set(this._map.keys()),new Set(e));if(t.length===0&&i.length===0)return!1;const n=i.map(a=>this._map.get(a)),s=t.map(a=>({range:this._createDecorationRange(this.model,a),options:this._createDecorationOption(a)})),r=this.model.deltaDecorations(n,s);for(const a of i)this._map.delete(a);for(let a=0;a<r.length;a++)this._map.set(t[a],r[a]);return!0}getMarker(e){return this._map.getKey(e.id)}_createDecorationRange(e,t){let i=x.lift(t);if(t.severity===pi.Hint&&!this._hasMarkerTag(t,1)&&!this._hasMarkerTag(t,2)&&(i=i.setEndPosition(i.startLineNumber,i.startColumn+2)),i=e.validateRange(i),i.isEmpty()){const n=e.getLineLastNonWhitespaceColumn(i.startLineNumber)||e.getLineMaxColumn(i.startLineNumber);if(n===1||i.endColumn>=n)return i;const s=e.getWordAtPosition(i.getStartPosition());s&&(i=new x(i.startLineNumber,s.startColumn,i.endLineNumber,s.endColumn))}else if(t.endColumn===Number.MAX_VALUE&&t.startColumn===1&&i.startLineNumber===i.endLineNumber){const n=e.getLineFirstNonWhitespaceColumn(t.startLineNumber);n<i.endColumn&&(i=new x(i.startLineNumber,n,i.endLineNumber,i.endColumn),t.startColumn=n)}return i}_createDecorationOption(e){let t,i,n,s,r;switch(e.severity){case pi.Hint:this._hasMarkerTag(e,2)?t=void 0:this._hasMarkerTag(e,1)?t="squiggly-unnecessary":t="squiggly-hint",n=0;break;case pi.Info:t="squiggly-info",i=mi(xhe),n=10,r={color:mi(sre),position:1};break;case pi.Warning:t="squiggly-warning",i=mi(Lhe),n=20,r={color:mi(ore),position:1};break;case pi.Error:default:t="squiggly-error",i=mi(yhe),n=30,r={color:mi(rre),position:1};break}return e.tags&&(e.tags.indexOf(1)!==-1&&(s="squiggly-inline-unnecessary"),e.tags.indexOf(2)!==-1&&(s="squiggly-inline-deprecated")),{description:"marker-decoration",stickiness:1,className:t,showIfCollapsed:!0,overviewRuler:{color:i,position:Ho.Right},minimap:r,zIndex:n,inlineClassName:s}}_hasMarkerTag(e,t){return e.tags?e.tags.indexOf(t)>=0:!1}}const r3=et("markerDecorationsService");class Ui{static _nextVisibleColumn(e,t,i){return e===9?Ui.nextRenderTabStop(t,i):Fh(e)||e5(e)?t+2:t+1}static visibleColumnFromColumn(e,t,i){const n=Math.min(t-1,e.length),s=e.substring(0,n),r=new Vy(s);let a=0;for(;!r.eol();){const l=Hy(s,n,r.offset);r.nextGraphemeLength(),a=this._nextVisibleColumn(l,a,i)}return a}static columnFromVisibleColumn(e,t,i){if(t<=0)return 1;const n=e.length,s=new Vy(e);let r=0,a=1;for(;!s.eol();){const l=Hy(e,n,s.offset);s.nextGraphemeLength();const c=this._nextVisibleColumn(l,r,i),d=s.offset+1;if(c>=t){const h=t-r;return c-t<h?d:a}r=c,a=d}return n+1}static nextRenderTabStop(e,t){return e+t-e%t}static nextIndentTabStop(e,t){return e+t-e%t}static prevRenderTabStop(e,t){return Math.max(0,e-1-(e-1)%t)}static prevIndentTabStop(e,t){return Math.max(0,e-1-(e-1)%t)}}function Uhe(o,e,t){let i=0;for(let s=0;s<o.length;s++)o.charAt(s)==="	"?i=Ui.nextIndentTabStop(i,e):i++;let n="";if(!t){const s=Math.floor(i/e);i=i%e;for(let r=0;r<s;r++)n+="	"}for(let s=0;s<i;s++)n+=" ";return n}function a3(o,e,t){let i=Jn(o);return i===-1&&(i=o.length),Uhe(o.substring(0,i),e,t)+o.substring(i)}class p8{constructor(e,t,i,n){this.range=e,this.nestingLevel=t,this.nestingLevelOfEqualBracketType=i,this.isInvalid=n}}class $he{constructor(e,t,i,n,s,r){this.range=e,this.openingBracketRange=t,this.closingBracketRange=i,this.nestingLevel=n,this.nestingLevelOfEqualBracketType=s,this.bracketPairNode=r}get openingBracketInfo(){return this.bracketPairNode.openingBracket.bracketInfo}}class jhe extends $he{constructor(e,t,i,n,s,r,a){super(e,t,i,n,s,r),this.minVisibleColumnIndentation=a}}function Khe(o,e,t,i){return o!==t?Mi(t-o,i):Mi(0,i-e)}const ds=0;function NL(o){return o===0}const Bo=2**26;function Mi(o,e){return o*Bo+e}function fr(o){const e=o,t=Math.floor(e/Bo),i=e-t*Bo;return new Yr(t,i)}function qhe(o){return Math.floor(o/Bo)}function _i(o,e){let t=o+e;return e>=Bo&&(t=t-o%Bo),t}function Ghe(o,e){return o.reduce((t,i)=>_i(t,e(i)),ds)}function Zj(o,e){return o===e}function Tw(o,e){const t=o,i=e;if(i-t<=0)return ds;const s=Math.floor(t/Bo),r=Math.floor(i/Bo),a=i-r*Bo;if(s===r){const l=t-s*Bo;return Mi(0,a-l)}else return Mi(r-s,a)}function Xp(o,e){return o<e}function Qp(o,e){return o<=e}function IC(o,e){return o>=e}function cp(o){return Mi(o.lineNumber-1,o.column-1)}function bg(o,e){const t=o,i=Math.floor(t/Bo),n=t-i*Bo,s=e,r=Math.floor(s/Bo),a=s-r*Bo;return new x(i+1,n+1,r+1,a+1)}function Zhe(o){const e=Kl(o);return Mi(e.length-1,e[e.length-1].length)}class Pc{static fromModelContentChanges(e){return e.map(i=>{const n=x.lift(i.range);return new Pc(cp(n.getStartPosition()),cp(n.getEndPosition()),Zhe(i.text))}).reverse()}constructor(e,t,i){this.startOffset=e,this.endOffset=t,this.newLength=i}toString(){return`[${fr(this.startOffset)}...${fr(this.endOffset)}) -> ${fr(this.newLength)}`}}class Yhe{constructor(e){this.nextEditIdx=0,this.deltaOldToNewLineCount=0,this.deltaOldToNewColumnCount=0,this.deltaLineIdxInOld=-1,this.edits=e.map(t=>l3.from(t))}getOffsetBeforeChange(e){return this.adjustNextEdit(e),this.translateCurToOld(e)}getDistanceToNextChange(e){this.adjustNextEdit(e);const t=this.edits[this.nextEditIdx],i=t?this.translateOldToCur(t.offsetObj):null;return i===null?null:Tw(e,i)}translateOldToCur(e){return e.lineCount===this.deltaLineIdxInOld?Mi(e.lineCount+this.deltaOldToNewLineCount,e.columnCount+this.deltaOldToNewColumnCount):Mi(e.lineCount+this.deltaOldToNewLineCount,e.columnCount)}translateCurToOld(e){const t=fr(e);return t.lineCount-this.deltaOldToNewLineCount===this.deltaLineIdxInOld?Mi(t.lineCount-this.deltaOldToNewLineCount,t.columnCount-this.deltaOldToNewColumnCount):Mi(t.lineCount-this.deltaOldToNewLineCount,t.columnCount)}adjustNextEdit(e){for(;this.nextEditIdx<this.edits.length;){const t=this.edits[this.nextEditIdx],i=this.translateOldToCur(t.endOffsetAfterObj);if(Qp(i,e)){this.nextEditIdx++;const n=fr(i),s=fr(this.translateOldToCur(t.endOffsetBeforeObj)),r=n.lineCount-s.lineCount;this.deltaOldToNewLineCount+=r;const a=this.deltaLineIdxInOld===t.endOffsetBeforeObj.lineCount?this.deltaOldToNewColumnCount:0,l=n.columnCount-s.columnCount;this.deltaOldToNewColumnCount=a+l,this.deltaLineIdxInOld=t.endOffsetBeforeObj.lineCount}else break}}}class l3{static from(e){return new l3(e.startOffset,e.endOffset,e.newLength)}constructor(e,t,i){this.endOffsetBeforeObj=fr(t),this.endOffsetAfterObj=fr(_i(e,i)),this.offsetObj=fr(e)}}const ZS=[],ir=class ir{static create(e,t){if(e<=128&&t.length===0){let i=ir.cache[e];return i||(i=new ir(e,t),ir.cache[e]=i),i}return new ir(e,t)}static getEmpty(){return this.empty}constructor(e,t){this.items=e,this.additionalItems=t}add(e,t){const i=t.getKey(e);let n=i>>5;if(n===0){const r=1<<i|this.items;return r===this.items?this:ir.create(r,this.additionalItems)}n--;const s=this.additionalItems.slice(0);for(;s.length<n;)s.push(0);return s[n]|=1<<(i&31),ir.create(this.items,s)}merge(e){const t=this.items|e.items;if(this.additionalItems===ZS&&e.additionalItems===ZS)return t===this.items?this:t===e.items?e:ir.create(t,ZS);const i=[];for(let n=0;n<Math.max(this.additionalItems.length,e.additionalItems.length);n++){const s=this.additionalItems[n]||0,r=e.additionalItems[n]||0;i.push(s|r)}return ir.create(t,i)}intersects(e){if((this.items&e.items)!==0)return!0;for(let t=0;t<Math.min(this.additionalItems.length,e.additionalItems.length);t++)if((this.additionalItems[t]&e.additionalItems[t])!==0)return!0;return!1}};ir.cache=new Array(129),ir.empty=ir.create(0,ZS);let io=ir;const _8={getKey(o){return o}};class Yj{constructor(){this.items=new Map}getKey(e){let t=this.items.get(e);return t===void 0&&(t=this.items.size,this.items.set(e,t)),t}}class c3{get length(){return this._length}constructor(e){this._length=e}}class Rw extends c3{static create(e,t,i){let n=e.length;return t&&(n=_i(n,t.length)),i&&(n=_i(n,i.length)),new Rw(n,e,t,i,t?t.missingOpeningBracketIds:io.getEmpty())}get kind(){return 2}get listHeight(){return 0}get childrenLength(){return 3}getChild(e){switch(e){case 0:return this.openingBracket;case 1:return this.child;case 2:return this.closingBracket}throw new Error("Invalid child index")}get children(){const e=[];return e.push(this.openingBracket),this.child&&e.push(this.child),this.closingBracket&&e.push(this.closingBracket),e}constructor(e,t,i,n,s){super(e),this.openingBracket=t,this.child=i,this.closingBracket=n,this.missingOpeningBracketIds=s}canBeReused(e){return!(this.closingBracket===null||e.intersects(this.missingOpeningBracketIds))}deepClone(){return new Rw(this.length,this.openingBracket.deepClone(),this.child&&this.child.deepClone(),this.closingBracket&&this.closingBracket.deepClone(),this.missingOpeningBracketIds)}computeMinIndentation(e,t){return this.child?this.child.computeMinIndentation(_i(e,this.openingBracket.length),t):Number.MAX_SAFE_INTEGER}}class Vl extends c3{static create23(e,t,i,n=!1){let s=e.length,r=e.missingOpeningBracketIds;if(e.listHeight!==t.listHeight)throw new Error("Invalid list heights");if(s=_i(s,t.length),r=r.merge(t.missingOpeningBracketIds),i){if(e.listHeight!==i.listHeight)throw new Error("Invalid list heights");s=_i(s,i.length),r=r.merge(i.missingOpeningBracketIds)}return n?new Xhe(s,e.listHeight+1,e,t,i,r):new Mw(s,e.listHeight+1,e,t,i,r)}static getEmpty(){return new Qhe(ds,0,[],io.getEmpty())}get kind(){return 4}get missingOpeningBracketIds(){return this._missingOpeningBracketIds}constructor(e,t,i){super(e),this.listHeight=t,this._missingOpeningBracketIds=i,this.cachedMinIndentation=-1}throwIfImmutable(){}makeLastElementMutable(){this.throwIfImmutable();const e=this.childrenLength;if(e===0)return;const t=this.getChild(e-1),i=t.kind===4?t.toMutable():t;return t!==i&&this.setChild(e-1,i),i}makeFirstElementMutable(){if(this.throwIfImmutable(),this.childrenLength===0)return;const t=this.getChild(0),i=t.kind===4?t.toMutable():t;return t!==i&&this.setChild(0,i),i}canBeReused(e){if(e.intersects(this.missingOpeningBracketIds)||this.childrenLength===0)return!1;let t=this;for(;t.kind===4;){const i=t.childrenLength;if(i===0)throw new bt;t=t.getChild(i-1)}return t.canBeReused(e)}handleChildrenChanged(){this.throwIfImmutable();const e=this.childrenLength;let t=this.getChild(0).length,i=this.getChild(0).missingOpeningBracketIds;for(let n=1;n<e;n++){const s=this.getChild(n);t=_i(t,s.length),i=i.merge(s.missingOpeningBracketIds)}this._length=t,this._missingOpeningBracketIds=i,this.cachedMinIndentation=-1}computeMinIndentation(e,t){if(this.cachedMinIndentation!==-1)return this.cachedMinIndentation;let i=Number.MAX_SAFE_INTEGER,n=e;for(let s=0;s<this.childrenLength;s++){const r=this.getChild(s);r&&(i=Math.min(i,r.computeMinIndentation(n,t)),n=_i(n,r.length))}return this.cachedMinIndentation=i,i}}class Mw extends Vl{get childrenLength(){return this._item3!==null?3:2}getChild(e){switch(e){case 0:return this._item1;case 1:return this._item2;case 2:return this._item3}throw new Error("Invalid child index")}setChild(e,t){switch(e){case 0:this._item1=t;return;case 1:this._item2=t;return;case 2:this._item3=t;return}throw new Error("Invalid child index")}get children(){return this._item3?[this._item1,this._item2,this._item3]:[this._item1,this._item2]}get item1(){return this._item1}get item2(){return this._item2}get item3(){return this._item3}constructor(e,t,i,n,s,r){super(e,t,r),this._item1=i,this._item2=n,this._item3=s}deepClone(){return new Mw(this.length,this.listHeight,this._item1.deepClone(),this._item2.deepClone(),this._item3?this._item3.deepClone():null,this.missingOpeningBracketIds)}appendChildOfSameHeight(e){if(this._item3)throw new Error("Cannot append to a full (2,3) tree node");this.throwIfImmutable(),this._item3=e,this.handleChildrenChanged()}unappendChild(){if(!this._item3)throw new Error("Cannot remove from a non-full (2,3) tree node");this.throwIfImmutable();const e=this._item3;return this._item3=null,this.handleChildrenChanged(),e}prependChildOfSameHeight(e){if(this._item3)throw new Error("Cannot prepend to a full (2,3) tree node");this.throwIfImmutable(),this._item3=this._item2,this._item2=this._item1,this._item1=e,this.handleChildrenChanged()}unprependChild(){if(!this._item3)throw new Error("Cannot remove from a non-full (2,3) tree node");this.throwIfImmutable();const e=this._item1;return this._item1=this._item2,this._item2=this._item3,this._item3=null,this.handleChildrenChanged(),e}toMutable(){return this}}class Xhe extends Mw{toMutable(){return new Mw(this.length,this.listHeight,this.item1,this.item2,this.item3,this.missingOpeningBracketIds)}throwIfImmutable(){throw new Error("this instance is immutable")}}class TL extends Vl{get childrenLength(){return this._children.length}getChild(e){return this._children[e]}setChild(e,t){this._children[e]=t}get children(){return this._children}constructor(e,t,i,n){super(e,t,n),this._children=i}deepClone(){const e=new Array(this._children.length);for(let t=0;t<this._children.length;t++)e[t]=this._children[t].deepClone();return new TL(this.length,this.listHeight,e,this.missingOpeningBracketIds)}appendChildOfSameHeight(e){this.throwIfImmutable(),this._children.push(e),this.handleChildrenChanged()}unappendChild(){this.throwIfImmutable();const e=this._children.pop();return this.handleChildrenChanged(),e}prependChildOfSameHeight(e){this.throwIfImmutable(),this._children.unshift(e),this.handleChildrenChanged()}unprependChild(){this.throwIfImmutable();const e=this._children.shift();return this.handleChildrenChanged(),e}toMutable(){return this}}class Qhe extends TL{toMutable(){return new TL(this.length,this.listHeight,[...this.children],this.missingOpeningBracketIds)}throwIfImmutable(){throw new Error("this instance is immutable")}}const Jhe=[];class d3 extends c3{get listHeight(){return 0}get childrenLength(){return 0}getChild(e){return null}get children(){return Jhe}deepClone(){return this}}class Ku extends d3{get kind(){return 0}get missingOpeningBracketIds(){return io.getEmpty()}canBeReused(e){return!0}computeMinIndentation(e,t){const i=fr(e),n=(i.columnCount===0?i.lineCount:i.lineCount+1)+1,s=qhe(_i(e,this.length))+1;let r=Number.MAX_SAFE_INTEGER;for(let a=n;a<=s;a++){const l=t.getLineFirstNonWhitespaceColumn(a),c=t.getLineContent(a);if(l===0)continue;const d=Ui.visibleColumnFromColumn(c,l,t.getOptions().tabSize);r=Math.min(r,d)}return r}}class RL extends d3{static create(e,t,i){return new RL(e,t,i)}get kind(){return 1}get missingOpeningBracketIds(){return io.getEmpty()}constructor(e,t,i){super(e),this.bracketInfo=t,this.bracketIds=i}get text(){return this.bracketInfo.bracketText}get languageId(){return this.bracketInfo.languageId}canBeReused(e){return!1}computeMinIndentation(e,t){return Number.MAX_SAFE_INTEGER}}class eue extends d3{get kind(){return 3}constructor(e,t){super(t),this.missingOpeningBracketIds=e}canBeReused(e){return!e.intersects(this.missingOpeningBracketIds)}computeMinIndentation(e,t){return Number.MAX_SAFE_INTEGER}}let Bd=class{constructor(e,t,i,n,s){this.length=e,this.kind=t,this.bracketId=i,this.bracketIds=n,this.astNode=s}};class Xj{constructor(e,t){this.textModel=e,this.bracketTokens=t,this.reader=new tue(this.textModel,this.bracketTokens),this._offset=ds,this.didPeek=!1,this.peeked=null,this.textBufferLineCount=e.getLineCount(),this.textBufferLastLineLength=e.getLineLength(this.textBufferLineCount)}get offset(){return this._offset}get length(){return Mi(this.textBufferLineCount-1,this.textBufferLastLineLength)}skip(e){this.didPeek=!1,this._offset=_i(this._offset,e);const t=fr(this._offset);this.reader.setPosition(t.lineCount,t.columnCount)}read(){let e;return this.peeked?(this.didPeek=!1,e=this.peeked):e=this.reader.read(),e&&(this._offset=_i(this._offset,e.length)),e}peek(){return this.didPeek||(this.peeked=this.reader.read(),this.didPeek=!0),this.peeked}}class tue{constructor(e,t){this.textModel=e,this.bracketTokens=t,this.lineIdx=0,this.line=null,this.lineCharOffset=0,this.lineTokens=null,this.lineTokenOffset=0,this.peekedToken=null,this.textBufferLineCount=e.getLineCount(),this.textBufferLastLineLength=e.getLineLength(this.textBufferLineCount)}setPosition(e,t){e===this.lineIdx?(this.lineCharOffset=t,this.line!==null&&(this.lineTokenOffset=this.lineCharOffset===0?0:this.lineTokens.findTokenIndexAtOffset(this.lineCharOffset))):(this.lineIdx=e,this.lineCharOffset=t,this.line=null),this.peekedToken=null}read(){if(this.peekedToken){const s=this.peekedToken;return this.peekedToken=null,this.lineCharOffset+=s.length,s}if(this.lineIdx>this.textBufferLineCount-1||this.lineIdx===this.textBufferLineCount-1&&this.lineCharOffset>=this.textBufferLastLineLength)return null;this.line===null&&(this.lineTokens=this.textModel.tokenization.getLineTokens(this.lineIdx+1),this.line=this.lineTokens.getLineContent(),this.lineTokenOffset=this.lineCharOffset===0?0:this.lineTokens.findTokenIndexAtOffset(this.lineCharOffset));const e=this.lineIdx,t=this.lineCharOffset;let i=0;for(;;){const s=this.lineTokens,r=s.getCount();let a=null;if(this.lineTokenOffset<r){const l=s.getMetadata(this.lineTokenOffset);for(;this.lineTokenOffset+1<r&&l===s.getMetadata(this.lineTokenOffset+1);)this.lineTokenOffset++;const c=Ts.getTokenType(l)===0,d=Ts.containsBalancedBrackets(l),h=s.getEndOffset(this.lineTokenOffset);if(d&&c&&this.lineCharOffset<h){const u=s.getLanguageId(this.lineTokenOffset),g=this.line.substring(this.lineCharOffset,h),f=this.bracketTokens.getSingleLanguageBracketTokens(u),p=f.regExpGlobal;if(p){p.lastIndex=0;const _=p.exec(g);_&&(a=f.getToken(_[0]),a&&(this.lineCharOffset+=_.index))}}if(i+=h-this.lineCharOffset,a)if(e!==this.lineIdx||t!==this.lineCharOffset){this.peekedToken=a;break}else return this.lineCharOffset+=a.length,a;else this.lineTokenOffset++,this.lineCharOffset=h}else if(this.lineIdx===this.textBufferLineCount-1||(this.lineIdx++,this.lineTokens=this.textModel.tokenization.getLineTokens(this.lineIdx+1),this.lineTokenOffset=0,this.line=this.lineTokens.getLineContent(),this.lineCharOffset=0,i+=33,i>1e3))break;if(i>1500)break}const n=Khe(e,t,this.lineIdx,this.lineCharOffset);return new Bd(n,0,-1,io.getEmpty(),new Ku(n))}}class iue{constructor(e,t){this.text=e,this._offset=ds,this.idx=0;const i=t.getRegExpStr(),n=i?new RegExp(i+`|
`,"gi"):null,s=[];let r,a=0,l=0,c=0,d=0;const h=[];for(let f=0;f<60;f++)h.push(new Bd(Mi(0,f),0,-1,io.getEmpty(),new Ku(Mi(0,f))));const u=[];for(let f=0;f<60;f++)u.push(new Bd(Mi(1,f),0,-1,io.getEmpty(),new Ku(Mi(1,f))));if(n)for(n.lastIndex=0;(r=n.exec(e))!==null;){const f=r.index,p=r[0];if(p===`
`)a++,l=f+1;else{if(c!==f){let _;if(d===a){const b=f-c;if(b<h.length)_=h[b];else{const C=Mi(0,b);_=new Bd(C,0,-1,io.getEmpty(),new Ku(C))}}else{const b=a-d,C=f-l;if(b===1&&C<u.length)_=u[C];else{const v=Mi(b,C);_=new Bd(v,0,-1,io.getEmpty(),new Ku(v))}}s.push(_)}s.push(t.getToken(p)),c=f+p.length,d=a}}const g=e.length;if(c!==g){const f=d===a?Mi(0,g-c):Mi(a-d,g-l);s.push(new Bd(f,0,-1,io.getEmpty(),new Ku(f)))}this.length=Mi(a,g-l),this.tokens=s}get offset(){return this._offset}read(){return this.tokens[this.idx++]||null}peek(){return this.tokens[this.idx]||null}skip(e){throw new FJ}}class h3{static createFromLanguage(e,t){function i(s){return t.getKey(`${s.languageId}:::${s.bracketText}`)}const n=new Map;for(const s of e.bracketsNew.openingBrackets){const r=Mi(0,s.bracketText.length),a=i(s),l=io.getEmpty().add(a,_8);n.set(s.bracketText,new Bd(r,1,a,l,RL.create(r,s,l)))}for(const s of e.bracketsNew.closingBrackets){const r=Mi(0,s.bracketText.length);let a=io.getEmpty();const l=s.getOpeningBrackets();for(const c of l)a=a.add(i(c),_8);n.set(s.bracketText,new Bd(r,2,i(l[0]),a,RL.create(r,s,a)))}return new h3(n)}constructor(e){this.map=e,this.hasRegExp=!1,this._regExpGlobal=null}getRegExpStr(){if(this.isEmpty)return null;{const e=[...this.map.keys()];return e.sort(),e.reverse(),e.map(t=>nue(t)).join("|")}}get regExpGlobal(){if(!this.hasRegExp){const e=this.getRegExpStr();this._regExpGlobal=e?new RegExp(e,"gi"):null,this.hasRegExp=!0}return this._regExpGlobal}getToken(e){return this.map.get(e.toLowerCase())}findClosingTokenText(e){for(const[t,i]of this.map)if(i.kind===2&&i.bracketIds.intersects(e))return t}get isEmpty(){return this.map.size===0}}function nue(o){let e=Uo(o);return/^[\w ]+/.test(o)&&(e=`\\b${e}`),/[\w ]+$/.test(o)&&(e=`${e}\\b`),e}class Qj{constructor(e,t){this.denseKeyProvider=e,this.getLanguageConfiguration=t,this.languageIdToBracketTokens=new Map}didLanguageChange(e){return this.languageIdToBracketTokens.has(e)}getSingleLanguageBracketTokens(e){let t=this.languageIdToBracketTokens.get(e);return t||(t=h3.createFromLanguage(this.getLanguageConfiguration(e),this.denseKeyProvider),this.languageIdToBracketTokens.set(e,t)),t}}function sue(o){if(o.length===0)return null;if(o.length===1)return o[0];let e=0;function t(){if(e>=o.length)return null;const r=e,a=o[r].listHeight;for(e++;e<o.length&&o[e].listHeight===a;)e++;return e-r>=2?Jj(r===0&&e===o.length?o:o.slice(r,e),!1):o[r]}let i=t(),n=t();if(!n)return i;for(let r=t();r;r=t())b8(i,n)<=b8(n,r)?(i=XN(i,n),n=r):n=XN(n,r);return XN(i,n)}function Jj(o,e=!1){if(o.length===0)return null;if(o.length===1)return o[0];let t=o.length;for(;t>3;){const i=t>>1;for(let n=0;n<i;n++){const s=n<<1;o[n]=Vl.create23(o[s],o[s+1],s+3===t?o[s+2]:null,e)}t=i}return Vl.create23(o[0],o[1],t>=3?o[2]:null,e)}function b8(o,e){return Math.abs(o.listHeight-e.listHeight)}function XN(o,e){return o.listHeight===e.listHeight?Vl.create23(o,e,null,!1):o.listHeight>e.listHeight?oue(o,e):rue(e,o)}function oue(o,e){o=o.toMutable();let t=o;const i=[];let n;for(;;){if(e.listHeight===t.listHeight){n=e;break}if(t.kind!==4)throw new Error("unexpected");i.push(t),t=t.makeLastElementMutable()}for(let s=i.length-1;s>=0;s--){const r=i[s];n?r.childrenLength>=3?n=Vl.create23(r.unappendChild(),n,null,!1):(r.appendChildOfSameHeight(n),n=void 0):r.handleChildrenChanged()}return n?Vl.create23(o,n,null,!1):o}function rue(o,e){o=o.toMutable();let t=o;const i=[];for(;e.listHeight!==t.listHeight;){if(t.kind!==4)throw new Error("unexpected");i.push(t),t=t.makeFirstElementMutable()}let n=e;for(let s=i.length-1;s>=0;s--){const r=i[s];n?r.childrenLength>=3?n=Vl.create23(n,r.unprependChild(),null,!1):(r.prependChildOfSameHeight(n),n=void 0):r.handleChildrenChanged()}return n?Vl.create23(n,o,null,!1):o}class aue{constructor(e){this.lastOffset=ds,this.nextNodes=[e],this.offsets=[ds],this.idxs=[]}readLongestNodeAt(e,t){if(Xp(e,this.lastOffset))throw new Error("Invalid offset");for(this.lastOffset=e;;){const i=nC(this.nextNodes);if(!i)return;const n=nC(this.offsets);if(Xp(e,n))return;if(Xp(n,e))if(_i(n,i.length)<=e)this.nextNodeAfterCurrent();else{const s=QN(i);s!==-1?(this.nextNodes.push(i.getChild(s)),this.offsets.push(n),this.idxs.push(s)):this.nextNodeAfterCurrent()}else{if(t(i))return this.nextNodeAfterCurrent(),i;{const s=QN(i);if(s===-1){this.nextNodeAfterCurrent();return}else this.nextNodes.push(i.getChild(s)),this.offsets.push(n),this.idxs.push(s)}}}}nextNodeAfterCurrent(){for(;;){const e=nC(this.offsets),t=nC(this.nextNodes);if(this.nextNodes.pop(),this.offsets.pop(),this.idxs.length===0)break;const i=nC(this.nextNodes),n=QN(i,this.idxs[this.idxs.length-1]);if(n!==-1){this.nextNodes.push(i.getChild(n)),this.offsets.push(_i(e,t.length)),this.idxs[this.idxs.length-1]=n;break}else this.idxs.pop()}}}function QN(o,e=-1){for(;;){if(e++,e>=o.childrenLength)return-1;if(o.getChild(e))return e}}function nC(o){return o.length>0?o[o.length-1]:void 0}function oA(o,e,t,i){return new lue(o,e,t,i).parseDocument()}class lue{constructor(e,t,i,n){if(this.tokenizer=e,this.createImmutableLists=n,this._itemsConstructed=0,this._itemsFromCache=0,i&&n)throw new Error("Not supported");this.oldNodeReader=i?new aue(i):void 0,this.positionMapper=new Yhe(t)}parseDocument(){this._itemsConstructed=0,this._itemsFromCache=0;let e=this.parseList(io.getEmpty(),0);return e||(e=Vl.getEmpty()),e}parseList(e,t){const i=[];for(;;){let s=this.tryReadChildFromCache(e);if(!s){const r=this.tokenizer.peek();if(!r||r.kind===2&&r.bracketIds.intersects(e))break;s=this.parseChild(e,t+1)}s.kind===4&&s.childrenLength===0||i.push(s)}return this.oldNodeReader?sue(i):Jj(i,this.createImmutableLists)}tryReadChildFromCache(e){if(this.oldNodeReader){const t=this.positionMapper.getDistanceToNextChange(this.tokenizer.offset);if(t===null||!NL(t)){const i=this.oldNodeReader.readLongestNodeAt(this.positionMapper.getOffsetBeforeChange(this.tokenizer.offset),n=>t!==null&&!Xp(n.length,t)?!1:n.canBeReused(e));if(i)return this._itemsFromCache++,this.tokenizer.skip(i.length),i}}}parseChild(e,t){this._itemsConstructed++;const i=this.tokenizer.read();switch(i.kind){case 2:return new eue(i.bracketIds,i.length);case 0:return i.astNode;case 1:{if(t>300)return new Ku(i.length);const n=e.merge(i.bracketIds),s=this.parseList(n,t+1),r=this.tokenizer.peek();return r&&r.kind===2&&(r.bracketId===i.bracketId||r.bracketIds.intersects(i.bracketIds))?(this.tokenizer.read(),Rw.create(i.astNode,s,r.astNode)):Rw.create(i.astNode,s,null)}default:throw new Error("unexpected")}}}function ML(o,e){if(o.length===0)return e;if(e.length===0)return o;const t=new Qc(C8(o)),i=C8(e);i.push({modified:!1,lengthBefore:void 0,lengthAfter:void 0});let n=t.dequeue();function s(c){if(c===void 0){const h=t.takeWhile(u=>!0)||[];return n&&h.unshift(n),h}const d=[];for(;n&&!NL(c);){const[h,u]=n.splitAt(c);d.push(h),c=Tw(h.lengthAfter,c),n=u??t.dequeue()}return NL(c)||d.push(new Jd(!1,c,c)),d}const r=[];function a(c,d,h){if(r.length>0&&Zj(r[r.length-1].endOffset,c)){const u=r[r.length-1];r[r.length-1]=new Pc(u.startOffset,d,_i(u.newLength,h))}else r.push({startOffset:c,endOffset:d,newLength:h})}let l=ds;for(const c of i){const d=s(c.lengthBefore);if(c.modified){const h=Ghe(d,g=>g.lengthBefore),u=_i(l,h);a(l,u,c.lengthAfter),l=u}else for(const h of d){const u=l;l=_i(l,h.lengthBefore),h.modified&&a(u,l,h.lengthAfter)}}return r}class Jd{constructor(e,t,i){this.modified=e,this.lengthBefore=t,this.lengthAfter=i}splitAt(e){const t=Tw(e,this.lengthAfter);return Zj(t,ds)?[this,void 0]:this.modified?[new Jd(this.modified,this.lengthBefore,e),new Jd(this.modified,ds,t)]:[new Jd(this.modified,e,e),new Jd(this.modified,t,t)]}toString(){return`${this.modified?"M":"U"}:${fr(this.lengthBefore)} -> ${fr(this.lengthAfter)}`}}function C8(o){const e=[];let t=ds;for(const i of o){const n=Tw(t,i.startOffset);NL(n)||e.push(new Jd(!1,n,n));const s=Tw(i.startOffset,i.endOffset);e.push(new Jd(!0,s,i.newLength)),t=i.endOffset}return e}class cue extends F{didLanguageChange(e){return this.brackets.didLanguageChange(e)}constructor(e,t){if(super(),this.textModel=e,this.getLanguageConfiguration=t,this.didChangeEmitter=new P,this.denseKeyProvider=new Yj,this.brackets=new Qj(this.denseKeyProvider,this.getLanguageConfiguration),this.onDidChange=this.didChangeEmitter.event,this.queuedTextEditsForInitialAstWithoutTokens=[],this.queuedTextEdits=[],e.tokenization.hasTokens)e.tokenization.backgroundTokenizationState===2?(this.initialAstWithoutTokens=void 0,this.astWithTokens=this.parseDocumentFromTextBuffer([],void 0,!1)):(this.initialAstWithoutTokens=this.parseDocumentFromTextBuffer([],void 0,!0),this.astWithTokens=this.initialAstWithoutTokens);else{const i=this.brackets.getSingleLanguageBracketTokens(this.textModel.getLanguageId()),n=new iue(this.textModel.getValue(),i);this.initialAstWithoutTokens=oA(n,[],void 0,!0),this.astWithTokens=this.initialAstWithoutTokens}}handleDidChangeBackgroundTokenizationState(){if(this.textModel.tokenization.backgroundTokenizationState===2){const e=this.initialAstWithoutTokens===void 0;this.initialAstWithoutTokens=void 0,e||this.didChangeEmitter.fire()}}handleDidChangeTokens({ranges:e}){const t=e.map(i=>new Pc(Mi(i.fromLineNumber-1,0),Mi(i.toLineNumber,0),Mi(i.toLineNumber-i.fromLineNumber+1,0)));this.handleEdits(t,!0),this.initialAstWithoutTokens||this.didChangeEmitter.fire()}handleContentChanged(e){const t=Pc.fromModelContentChanges(e.changes);this.handleEdits(t,!1)}handleEdits(e,t){const i=ML(this.queuedTextEdits,e);this.queuedTextEdits=i,this.initialAstWithoutTokens&&!t&&(this.queuedTextEditsForInitialAstWithoutTokens=ML(this.queuedTextEditsForInitialAstWithoutTokens,e))}flushQueue(){this.queuedTextEdits.length>0&&(this.astWithTokens=this.parseDocumentFromTextBuffer(this.queuedTextEdits,this.astWithTokens,!1),this.queuedTextEdits=[]),this.queuedTextEditsForInitialAstWithoutTokens.length>0&&(this.initialAstWithoutTokens&&(this.initialAstWithoutTokens=this.parseDocumentFromTextBuffer(this.queuedTextEditsForInitialAstWithoutTokens,this.initialAstWithoutTokens,!1)),this.queuedTextEditsForInitialAstWithoutTokens=[])}parseDocumentFromTextBuffer(e,t,i){const n=t,s=new Xj(this.textModel,this.brackets);return oA(s,e,n,i)}getBracketsInRange(e,t){this.flushQueue();const i=Mi(e.startLineNumber-1,e.startColumn-1),n=Mi(e.endLineNumber-1,e.endColumn-1);return new gg(s=>{const r=this.initialAstWithoutTokens||this.astWithTokens;rA(r,ds,r.length,i,n,s,0,0,new Map,t)})}getBracketPairsInRange(e,t){this.flushQueue();const i=cp(e.getStartPosition()),n=cp(e.getEndPosition());return new gg(s=>{const r=this.initialAstWithoutTokens||this.astWithTokens,a=new due(s,t,this.textModel);aA(r,ds,r.length,i,n,a,0,new Map)})}getFirstBracketAfter(e){this.flushQueue();const t=this.initialAstWithoutTokens||this.astWithTokens;return tK(t,ds,t.length,cp(e))}getFirstBracketBefore(e){this.flushQueue();const t=this.initialAstWithoutTokens||this.astWithTokens;return eK(t,ds,t.length,cp(e))}}function eK(o,e,t,i){if(o.kind===4||o.kind===2){const n=[];for(const s of o.children)t=_i(e,s.length),n.push({nodeOffsetStart:e,nodeOffsetEnd:t}),e=t;for(let s=n.length-1;s>=0;s--){const{nodeOffsetStart:r,nodeOffsetEnd:a}=n[s];if(Xp(r,i)){const l=eK(o.children[s],r,a,i);if(l)return l}}return null}else{if(o.kind===3)return null;if(o.kind===1){const n=bg(e,t);return{bracketInfo:o.bracketInfo,range:n}}}return null}function tK(o,e,t,i){if(o.kind===4||o.kind===2){for(const n of o.children){if(t=_i(e,n.length),Xp(i,t)){const s=tK(n,e,t,i);if(s)return s}e=t}return null}else{if(o.kind===3)return null;if(o.kind===1){const n=bg(e,t);return{bracketInfo:o.bracketInfo,range:n}}}return null}function rA(o,e,t,i,n,s,r,a,l,c,d=!1){if(r>200)return!0;e:for(;;)switch(o.kind){case 4:{const h=o.childrenLength;for(let u=0;u<h;u++){const g=o.getChild(u);if(g){if(t=_i(e,g.length),Qp(e,n)&&IC(t,i)){if(IC(t,n)){o=g;continue e}if(!rA(g,e,t,i,n,s,r,0,l,c))return!1}e=t}}return!0}case 2:{const h=!c||!o.closingBracket||o.closingBracket.bracketInfo.closesColorized(o.openingBracket.bracketInfo);let u=0;if(l){let f=l.get(o.openingBracket.text);f===void 0&&(f=0),u=f,h&&(f++,l.set(o.openingBracket.text,f))}const g=o.childrenLength;for(let f=0;f<g;f++){const p=o.getChild(f);if(p){if(t=_i(e,p.length),Qp(e,n)&&IC(t,i)){if(IC(t,n)&&p.kind!==1){o=p,h?(r++,a=u+1):a=u;continue e}if((h||p.kind!==1||!o.closingBracket)&&!rA(p,e,t,i,n,s,h?r+1:r,h?u+1:u,l,c,!o.closingBracket))return!1}e=t}}return l==null||l.set(o.openingBracket.text,u),!0}case 3:{const h=bg(e,t);return s(new p8(h,r-1,0,!0))}case 1:{const h=bg(e,t);return s(new p8(h,r-1,a-1,d))}case 0:return!0}}class due{constructor(e,t,i){this.push=e,this.includeMinIndentation=t,this.textModel=i}}function aA(o,e,t,i,n,s,r,a){var c;if(r>200)return!0;let l=!0;if(o.kind===2){let d=0;if(a){let g=a.get(o.openingBracket.text);g===void 0&&(g=0),d=g,g++,a.set(o.openingBracket.text,g)}const h=_i(e,o.openingBracket.length);let u=-1;if(s.includeMinIndentation&&(u=o.computeMinIndentation(e,s.textModel)),l=s.push(new jhe(bg(e,t),bg(e,h),o.closingBracket?bg(_i(h,((c=o.child)==null?void 0:c.length)||ds),t):void 0,r,d,o,u)),e=h,l&&o.child){const g=o.child;if(t=_i(e,g.length),Qp(e,n)&&IC(t,i)&&(l=aA(g,e,t,i,n,s,r+1,a),!l))return!1}a==null||a.set(o.openingBracket.text,d)}else{let d=e;for(const h of o.children){const u=d;if(d=_i(d,h.length),Qp(u,n)&&Qp(i,d)&&(l=aA(h,u,d,i,n,s,r,a),!l))return!1}}return l}class hue extends F{get canBuildAST(){return this.textModel.getValueLength()<=5e6}constructor(e,t){super(),this.textModel=e,this.languageConfigurationService=t,this.bracketPairsTree=this._register(new Ji),this.onDidChangeEmitter=new P,this.onDidChange=this.onDidChangeEmitter.event,this.bracketsRequested=!1}handleLanguageConfigurationServiceChange(e){var t;(!e.languageId||(t=this.bracketPairsTree.value)!=null&&t.object.didLanguageChange(e.languageId))&&(this.bracketPairsTree.clear(),this.updateBracketPairsTree())}handleDidChangeOptions(e){this.bracketPairsTree.clear(),this.updateBracketPairsTree()}handleDidChangeLanguage(e){this.bracketPairsTree.clear(),this.updateBracketPairsTree()}handleDidChangeContent(e){var t;(t=this.bracketPairsTree.value)==null||t.object.handleContentChanged(e)}handleDidChangeBackgroundTokenizationState(){var e;(e=this.bracketPairsTree.value)==null||e.object.handleDidChangeBackgroundTokenizationState()}handleDidChangeTokens(e){var t;(t=this.bracketPairsTree.value)==null||t.object.handleDidChangeTokens(e)}updateBracketPairsTree(){if(this.bracketsRequested&&this.canBuildAST){if(!this.bracketPairsTree.value){const e=new K;this.bracketPairsTree.value=uue(e.add(new cue(this.textModel,t=>this.languageConfigurationService.getLanguageConfiguration(t))),e),e.add(this.bracketPairsTree.value.object.onDidChange(t=>this.onDidChangeEmitter.fire(t))),this.onDidChangeEmitter.fire()}}else this.bracketPairsTree.value&&(this.bracketPairsTree.clear(),this.onDidChangeEmitter.fire())}getBracketPairsInRange(e){var t;return this.bracketsRequested=!0,this.updateBracketPairsTree(),((t=this.bracketPairsTree.value)==null?void 0:t.object.getBracketPairsInRange(e,!1))||gg.empty}getBracketPairsInRangeWithMinIndentation(e){var t;return this.bracketsRequested=!0,this.updateBracketPairsTree(),((t=this.bracketPairsTree.value)==null?void 0:t.object.getBracketPairsInRange(e,!0))||gg.empty}getBracketsInRange(e,t=!1){var i;return this.bracketsRequested=!0,this.updateBracketPairsTree(),((i=this.bracketPairsTree.value)==null?void 0:i.object.getBracketsInRange(e,t))||gg.empty}findMatchingBracketUp(e,t,i){const n=this.textModel.validatePosition(t),s=this.textModel.getLanguageIdAtPosition(n.lineNumber,n.column);if(this.canBuildAST){const r=this.languageConfigurationService.getLanguageConfiguration(s).bracketsNew.getClosingBracketInfo(e);if(!r)return null;const a=this.getBracketPairsInRange(x.fromPositions(t,t)).findLast(l=>r.closes(l.openingBracketInfo));return a?a.openingBracketRange:null}else{const r=e.toLowerCase(),a=this.languageConfigurationService.getLanguageConfiguration(s).brackets;if(!a)return null;const l=a.textIsBracket[r];return l?YS(this._findMatchingBracketUp(l,n,JN(i))):null}}matchBracket(e,t){if(this.canBuildAST){const i=this.getBracketPairsInRange(x.fromPositions(e,e)).filter(n=>n.closingBracketRange!==void 0&&(n.openingBracketRange.containsPosition(e)||n.closingBracketRange.containsPosition(e))).findLastMaxBy(gs(n=>n.openingBracketRange.containsPosition(e)?n.openingBracketRange:n.closingBracketRange,x.compareRangesUsingStarts));return i?[i.openingBracketRange,i.closingBracketRange]:null}else{const i=JN(t);return this._matchBracket(this.textModel.validatePosition(e),i)}}_establishBracketSearchOffsets(e,t,i,n){const s=t.getCount(),r=t.getLanguageId(n);let a=Math.max(0,e.column-1-i.maxBracketLength);for(let c=n-1;c>=0;c--){const d=t.getEndOffset(c);if(d<=a)break;if(ol(t.getStandardTokenType(c))||t.getLanguageId(c)!==r){a=d;break}}let l=Math.min(t.getLineContent().length,e.column-1+i.maxBracketLength);for(let c=n+1;c<s;c++){const d=t.getStartOffset(c);if(d>=l)break;if(ol(t.getStandardTokenType(c))||t.getLanguageId(c)!==r){l=d;break}}return{searchStartOffset:a,searchEndOffset:l}}_matchBracket(e,t){const i=e.lineNumber,n=this.textModel.tokenization.getLineTokens(i),s=this.textModel.getLineContent(i),r=n.findTokenIndexAtOffset(e.column-1);if(r<0)return null;const a=this.languageConfigurationService.getLanguageConfiguration(n.getLanguageId(r)).brackets;if(a&&!ol(n.getStandardTokenType(r))){let{searchStartOffset:l,searchEndOffset:c}=this._establishBracketSearchOffsets(e,n,a,r),d=null;for(;;){const h=Tr.findNextBracketInRange(a.forwardRegex,i,s,l,c);if(!h)break;if(h.startColumn<=e.column&&e.column<=h.endColumn){const u=s.substring(h.startColumn-1,h.endColumn-1).toLowerCase(),g=this._matchFoundBracket(h,a.textIsBracket[u],a.textIsOpenBracket[u],t);if(g){if(g instanceof Sc)return null;d=g}}l=h.endColumn-1}if(d)return d}if(r>0&&n.getStartOffset(r)===e.column-1){const l=r-1,c=this.languageConfigurationService.getLanguageConfiguration(n.getLanguageId(l)).brackets;if(c&&!ol(n.getStandardTokenType(l))){const{searchStartOffset:d,searchEndOffset:h}=this._establishBracketSearchOffsets(e,n,c,l),u=Tr.findPrevBracketInRange(c.reversedRegex,i,s,d,h);if(u&&u.startColumn<=e.column&&e.column<=u.endColumn){const g=s.substring(u.startColumn-1,u.endColumn-1).toLowerCase(),f=this._matchFoundBracket(u,c.textIsBracket[g],c.textIsOpenBracket[g],t);if(f)return f instanceof Sc?null:f}}}return null}_matchFoundBracket(e,t,i,n){if(!t)return null;const s=i?this._findMatchingBracketDown(t,e.getEndPosition(),n):this._findMatchingBracketUp(t,e.getStartPosition(),n);return s?s instanceof Sc?s:[e,s]:null}_findMatchingBracketUp(e,t,i){const n=e.languageId,s=e.reversedRegex;let r=-1,a=0;const l=(c,d,h,u)=>{for(;;){if(i&&++a%100===0&&!i())return Sc.INSTANCE;const g=Tr.findPrevBracketInRange(s,c,d,h,u);if(!g)break;const f=d.substring(g.startColumn-1,g.endColumn-1).toLowerCase();if(e.isOpen(f)?r++:e.isClose(f)&&r--,r===0)return g;u=g.startColumn-1}return null};for(let c=t.lineNumber;c>=1;c--){const d=this.textModel.tokenization.getLineTokens(c),h=d.getCount(),u=this.textModel.getLineContent(c);let g=h-1,f=u.length,p=u.length;c===t.lineNumber&&(g=d.findTokenIndexAtOffset(t.column-1),f=t.column-1,p=t.column-1);let _=!0;for(;g>=0;g--){const b=d.getLanguageId(g)===n&&!ol(d.getStandardTokenType(g));if(b)_?f=d.getStartOffset(g):(f=d.getStartOffset(g),p=d.getEndOffset(g));else if(_&&f!==p){const C=l(c,u,f,p);if(C)return C}_=b}if(_&&f!==p){const b=l(c,u,f,p);if(b)return b}}return null}_findMatchingBracketDown(e,t,i){const n=e.languageId,s=e.forwardRegex;let r=1,a=0;const l=(d,h,u,g)=>{for(;;){if(i&&++a%100===0&&!i())return Sc.INSTANCE;const f=Tr.findNextBracketInRange(s,d,h,u,g);if(!f)break;const p=h.substring(f.startColumn-1,f.endColumn-1).toLowerCase();if(e.isOpen(p)?r++:e.isClose(p)&&r--,r===0)return f;u=f.endColumn-1}return null},c=this.textModel.getLineCount();for(let d=t.lineNumber;d<=c;d++){const h=this.textModel.tokenization.getLineTokens(d),u=h.getCount(),g=this.textModel.getLineContent(d);let f=0,p=0,_=0;d===t.lineNumber&&(f=h.findTokenIndexAtOffset(t.column-1),p=t.column-1,_=t.column-1);let b=!0;for(;f<u;f++){const C=h.getLanguageId(f)===n&&!ol(h.getStandardTokenType(f));if(C)b||(p=h.getStartOffset(f)),_=h.getEndOffset(f);else if(b&&p!==_){const v=l(d,g,p,_);if(v)return v}b=C}if(b&&p!==_){const C=l(d,g,p,_);if(C)return C}}return null}findPrevBracket(e){var r;const t=this.textModel.validatePosition(e);if(this.canBuildAST)return this.bracketsRequested=!0,this.updateBracketPairsTree(),((r=this.bracketPairsTree.value)==null?void 0:r.object.getFirstBracketBefore(t))||null;let i=null,n=null,s=null;for(let a=t.lineNumber;a>=1;a--){const l=this.textModel.tokenization.getLineTokens(a),c=l.getCount(),d=this.textModel.getLineContent(a);let h=c-1,u=d.length,g=d.length;if(a===t.lineNumber){h=l.findTokenIndexAtOffset(t.column-1),u=t.column-1,g=t.column-1;const p=l.getLanguageId(h);i!==p&&(i=p,n=this.languageConfigurationService.getLanguageConfiguration(i).brackets,s=this.languageConfigurationService.getLanguageConfiguration(i).bracketsNew)}let f=!0;for(;h>=0;h--){const p=l.getLanguageId(h);if(i!==p){if(n&&s&&f&&u!==g){const b=Tr.findPrevBracketInRange(n.reversedRegex,a,d,u,g);if(b)return this._toFoundBracket(s,b);f=!1}i=p,n=this.languageConfigurationService.getLanguageConfiguration(i).brackets,s=this.languageConfigurationService.getLanguageConfiguration(i).bracketsNew}const _=!!n&&!ol(l.getStandardTokenType(h));if(_)f?u=l.getStartOffset(h):(u=l.getStartOffset(h),g=l.getEndOffset(h));else if(s&&n&&f&&u!==g){const b=Tr.findPrevBracketInRange(n.reversedRegex,a,d,u,g);if(b)return this._toFoundBracket(s,b)}f=_}if(s&&n&&f&&u!==g){const p=Tr.findPrevBracketInRange(n.reversedRegex,a,d,u,g);if(p)return this._toFoundBracket(s,p)}}return null}findNextBracket(e){var a;const t=this.textModel.validatePosition(e);if(this.canBuildAST)return this.bracketsRequested=!0,this.updateBracketPairsTree(),((a=this.bracketPairsTree.value)==null?void 0:a.object.getFirstBracketAfter(t))||null;const i=this.textModel.getLineCount();let n=null,s=null,r=null;for(let l=t.lineNumber;l<=i;l++){const c=this.textModel.tokenization.getLineTokens(l),d=c.getCount(),h=this.textModel.getLineContent(l);let u=0,g=0,f=0;if(l===t.lineNumber){u=c.findTokenIndexAtOffset(t.column-1),g=t.column-1,f=t.column-1;const _=c.getLanguageId(u);n!==_&&(n=_,s=this.languageConfigurationService.getLanguageConfiguration(n).brackets,r=this.languageConfigurationService.getLanguageConfiguration(n).bracketsNew)}let p=!0;for(;u<d;u++){const _=c.getLanguageId(u);if(n!==_){if(r&&s&&p&&g!==f){const C=Tr.findNextBracketInRange(s.forwardRegex,l,h,g,f);if(C)return this._toFoundBracket(r,C);p=!1}n=_,s=this.languageConfigurationService.getLanguageConfiguration(n).brackets,r=this.languageConfigurationService.getLanguageConfiguration(n).bracketsNew}const b=!!s&&!ol(c.getStandardTokenType(u));if(b)p||(g=c.getStartOffset(u)),f=c.getEndOffset(u);else if(r&&s&&p&&g!==f){const C=Tr.findNextBracketInRange(s.forwardRegex,l,h,g,f);if(C)return this._toFoundBracket(r,C)}p=b}if(r&&s&&p&&g!==f){const _=Tr.findNextBracketInRange(s.forwardRegex,l,h,g,f);if(_)return this._toFoundBracket(r,_)}}return null}findEnclosingBrackets(e,t){const i=this.textModel.validatePosition(e);if(this.canBuildAST){const g=x.fromPositions(i),f=this.getBracketPairsInRange(x.fromPositions(i,i)).findLast(p=>p.closingBracketRange!==void 0&&p.range.strictContainsRange(g));return f?[f.openingBracketRange,f.closingBracketRange]:null}const n=JN(t),s=this.textModel.getLineCount(),r=new Map;let a=[];const l=(g,f)=>{if(!r.has(g)){const p=[];for(let _=0,b=f?f.brackets.length:0;_<b;_++)p[_]=0;r.set(g,p)}a=r.get(g)};let c=0;const d=(g,f,p,_,b)=>{for(;;){if(n&&++c%100===0&&!n())return Sc.INSTANCE;const C=Tr.findNextBracketInRange(g.forwardRegex,f,p,_,b);if(!C)break;const v=p.substring(C.startColumn-1,C.endColumn-1).toLowerCase(),w=g.textIsBracket[v];if(w&&(w.isOpen(v)?a[w.index]++:w.isClose(v)&&a[w.index]--,a[w.index]===-1))return this._matchFoundBracket(C,w,!1,n);_=C.endColumn-1}return null};let h=null,u=null;for(let g=i.lineNumber;g<=s;g++){const f=this.textModel.tokenization.getLineTokens(g),p=f.getCount(),_=this.textModel.getLineContent(g);let b=0,C=0,v=0;if(g===i.lineNumber){b=f.findTokenIndexAtOffset(i.column-1),C=i.column-1,v=i.column-1;const S=f.getLanguageId(b);h!==S&&(h=S,u=this.languageConfigurationService.getLanguageConfiguration(h).brackets,l(h,u))}let w=!0;for(;b<p;b++){const S=f.getLanguageId(b);if(h!==S){if(u&&w&&C!==v){const k=d(u,g,_,C,v);if(k)return YS(k);w=!1}h=S,u=this.languageConfigurationService.getLanguageConfiguration(h).brackets,l(h,u)}const L=!!u&&!ol(f.getStandardTokenType(b));if(L)w||(C=f.getStartOffset(b)),v=f.getEndOffset(b);else if(u&&w&&C!==v){const k=d(u,g,_,C,v);if(k)return YS(k)}w=L}if(u&&w&&C!==v){const S=d(u,g,_,C,v);if(S)return YS(S)}}return null}_toFoundBracket(e,t){if(!t)return null;let i=this.textModel.getValueInRange(t);i=i.toLowerCase();const n=e.getBracketInfo(i);return n?{range:t,bracketInfo:n}:null}}function uue(o,e){return{object:o,dispose:()=>e==null?void 0:e.dispose()}}function JN(o){if(typeof o>"u")return()=>!0;{const e=Date.now();return()=>Date.now()-e<=o}}const jk=class jk{constructor(){this._searchCanceledBrand=void 0}};jk.INSTANCE=new jk;let Sc=jk;function YS(o){return o instanceof Sc?null:o}class gue extends F{constructor(e){super(),this.textModel=e,this.colorProvider=new iK,this.onDidChangeEmitter=new P,this.onDidChange=this.onDidChangeEmitter.event,this.colorizationOptions=e.getOptions().bracketPairColorizationOptions,this._register(e.bracketPairs.onDidChange(t=>{this.onDidChangeEmitter.fire()}))}handleDidChangeOptions(e){this.colorizationOptions=this.textModel.getOptions().bracketPairColorizationOptions}getDecorationsInRange(e,t,i,n){return n?[]:t===void 0?[]:this.colorizationOptions.enabled?this.textModel.bracketPairs.getBracketsInRange(e,!0).map(r=>({id:`bracket${r.range.toString()}-${r.nestingLevel}`,options:{description:"BracketPairColorization",inlineClassName:this.colorProvider.getInlineClassName(r,this.colorizationOptions.independentColorPoolPerBracketType)},ownerId:0,range:r.range})).toArray():[]}getAllDecorations(e,t){return e===void 0?[]:this.colorizationOptions.enabled?this.getDecorationsInRange(new x(1,1,this.textModel.getLineCount(),1),e,t):[]}}class iK{constructor(){this.unexpectedClosingBracketClassName="unexpected-closing-bracket"}getInlineClassName(e,t){return e.isInvalid?this.unexpectedClosingBracketClassName:this.getInlineClassNameOfLevel(t?e.nestingLevelOfEqualBracketType:e.nestingLevel)}getInlineClassNameOfLevel(e){return`bracket-highlighting-${e%30}`}}Lr((o,e)=>{const t=[Uj,$j,jj,Kj,qj,Gj],i=new iK;e.addRule(`.monaco-editor .${i.unexpectedClosingBracketClassName} { color: ${o.getColor(khe)}; }`);const n=t.map(s=>o.getColor(s)).filter(s=>!!s).filter(s=>!s.isTransparent());for(let s=0;s<30;s++){const r=n[s%n.length];e.addRule(`.monaco-editor .${i.getInlineClassNameOfLevel(s)} { color: ${r}; }`)}});function XS(o){return o.replace(/\n/g,"\\n").replace(/\r/g,"\\r")}class Ln{get oldLength(){return this.oldText.length}get oldEnd(){return this.oldPosition+this.oldText.length}get newLength(){return this.newText.length}get newEnd(){return this.newPosition+this.newText.length}constructor(e,t,i,n){this.oldPosition=e,this.oldText=t,this.newPosition=i,this.newText=n}toString(){return this.oldText.length===0?`(insert@${this.oldPosition} "${XS(this.newText)}")`:this.newText.length===0?`(delete@${this.oldPosition} "${XS(this.oldText)}")`:`(replace@${this.oldPosition} "${XS(this.oldText)}" with "${XS(this.newText)}")`}static _writeStringSize(e){return 4+2*e.length}static _writeString(e,t,i){const n=t.length;ma(e,n,i),i+=4;for(let s=0;s<n;s++)ene(e,t.charCodeAt(s),i),i+=2;return i}static _readString(e,t){const i=fa(e,t);return t+=4,ine(e,t,i)}writeSize(){return 8+Ln._writeStringSize(this.oldText)+Ln._writeStringSize(this.newText)}write(e,t){return ma(e,this.oldPosition,t),t+=4,ma(e,this.newPosition,t),t+=4,t=Ln._writeString(e,this.oldText,t),t=Ln._writeString(e,this.newText,t),t}static read(e,t,i){const n=fa(e,t);t+=4;const s=fa(e,t);t+=4;const r=Ln._readString(e,t);t+=Ln._writeStringSize(r);const a=Ln._readString(e,t);return t+=Ln._writeStringSize(a),i.push(new Ln(n,r,s,a)),t}}function fue(o,e){return o===null||o.length===0?e:new ll(o,e).compress()}class ll{constructor(e,t){this._prevEdits=e,this._currEdits=t,this._result=[],this._resultLen=0,this._prevLen=this._prevEdits.length,this._prevDeltaOffset=0,this._currLen=this._currEdits.length,this._currDeltaOffset=0}compress(){let e=0,t=0,i=this._getPrev(e),n=this._getCurr(t);for(;e<this._prevLen||t<this._currLen;){if(i===null){this._acceptCurr(n),n=this._getCurr(++t);continue}if(n===null){this._acceptPrev(i),i=this._getPrev(++e);continue}if(n.oldEnd<=i.newPosition){this._acceptCurr(n),n=this._getCurr(++t);continue}if(i.newEnd<=n.oldPosition){this._acceptPrev(i),i=this._getPrev(++e);continue}if(n.oldPosition<i.newPosition){const[c,d]=ll._splitCurr(n,i.newPosition-n.oldPosition);this._acceptCurr(c),n=d;continue}if(i.newPosition<n.oldPosition){const[c,d]=ll._splitPrev(i,n.oldPosition-i.newPosition);this._acceptPrev(c),i=d;continue}let a,l;if(n.oldEnd===i.newEnd)a=i,l=n,i=this._getPrev(++e),n=this._getCurr(++t);else if(n.oldEnd<i.newEnd){const[c,d]=ll._splitPrev(i,n.oldLength);a=c,l=n,i=d,n=this._getCurr(++t)}else{const[c,d]=ll._splitCurr(n,i.newLength);a=i,l=c,i=this._getPrev(++e),n=d}this._result[this._resultLen++]=new Ln(a.oldPosition,a.oldText,l.newPosition,l.newText),this._prevDeltaOffset+=a.newLength-a.oldLength,this._currDeltaOffset+=l.newLength-l.oldLength}const s=ll._merge(this._result);return ll._removeNoOps(s)}_acceptCurr(e){this._result[this._resultLen++]=ll._rebaseCurr(this._prevDeltaOffset,e),this._currDeltaOffset+=e.newLength-e.oldLength}_getCurr(e){return e<this._currLen?this._currEdits[e]:null}_acceptPrev(e){this._result[this._resultLen++]=ll._rebasePrev(this._currDeltaOffset,e),this._prevDeltaOffset+=e.newLength-e.oldLength}_getPrev(e){return e<this._prevLen?this._prevEdits[e]:null}static _rebaseCurr(e,t){return new Ln(t.oldPosition-e,t.oldText,t.newPosition,t.newText)}static _rebasePrev(e,t){return new Ln(t.oldPosition,t.oldText,t.newPosition+e,t.newText)}static _splitPrev(e,t){const i=e.newText.substr(0,t),n=e.newText.substr(t);return[new Ln(e.oldPosition,e.oldText,e.newPosition,i),new Ln(e.oldEnd,"",e.newPosition+t,n)]}static _splitCurr(e,t){const i=e.oldText.substr(0,t),n=e.oldText.substr(t);return[new Ln(e.oldPosition,i,e.newPosition,e.newText),new Ln(e.oldPosition+t,n,e.newEnd,"")]}static _merge(e){if(e.length===0)return e;const t=[];let i=0,n=e[0];for(let s=1;s<e.length;s++){const r=e[s];n.oldEnd===r.oldPosition?n=new Ln(n.oldPosition,n.oldText+r.oldText,n.newPosition,n.newText+r.newText):(t[i++]=n,n=r)}return t[i++]=n,t}static _removeNoOps(e){if(e.length===0)return e;const t=[];let i=0;for(let n=0;n<e.length;n++){const s=e[n];s.oldText!==s.newText&&(t[i++]=s)}return t}}function Im(o){return o.toString()}class tn{static create(e,t){const i=e.getAlternativeVersionId(),n=lA(e);return new tn(i,i,n,n,t,t,[])}constructor(e,t,i,n,s,r,a){this.beforeVersionId=e,this.afterVersionId=t,this.beforeEOL=i,this.afterEOL=n,this.beforeCursorState=s,this.afterCursorState=r,this.changes=a}append(e,t,i,n,s){t.length>0&&(this.changes=fue(this.changes,t)),this.afterEOL=i,this.afterVersionId=n,this.afterCursorState=s}static _writeSelectionsSize(e){return 4+4*4*(e?e.length:0)}static _writeSelections(e,t,i){if(ma(e,t?t.length:0,i),i+=4,t)for(const n of t)ma(e,n.selectionStartLineNumber,i),i+=4,ma(e,n.selectionStartColumn,i),i+=4,ma(e,n.positionLineNumber,i),i+=4,ma(e,n.positionColumn,i),i+=4;return i}static _readSelections(e,t,i){const n=fa(e,t);t+=4;for(let s=0;s<n;s++){const r=fa(e,t);t+=4;const a=fa(e,t);t+=4;const l=fa(e,t);t+=4;const c=fa(e,t);t+=4,i.push(new pe(r,a,l,c))}return t}serialize(){let e=10+tn._writeSelectionsSize(this.beforeCursorState)+tn._writeSelectionsSize(this.afterCursorState)+4;for(const n of this.changes)e+=n.writeSize();const t=new Uint8Array(e);let i=0;ma(t,this.beforeVersionId,i),i+=4,ma(t,this.afterVersionId,i),i+=4,S7(t,this.beforeEOL,i),i+=1,S7(t,this.afterEOL,i),i+=1,i=tn._writeSelections(t,this.beforeCursorState,i),i=tn._writeSelections(t,this.afterCursorState,i),ma(t,this.changes.length,i),i+=4;for(const n of this.changes)i=n.write(t,i);return t.buffer}static deserialize(e){const t=new Uint8Array(e);let i=0;const n=fa(t,i);i+=4;const s=fa(t,i);i+=4;const r=w7(t,i);i+=1;const a=w7(t,i);i+=1;const l=[];i=tn._readSelections(t,i,l);const c=[];i=tn._readSelections(t,i,c);const d=fa(t,i);i+=4;const h=[];for(let u=0;u<d;u++)i=Ln.read(t,i,h);return new tn(n,s,r,a,l,c,h)}}class nK{get type(){return 0}get resource(){return Se.isUri(this.model)?this.model:this.model.uri}constructor(e,t,i,n){this.label=e,this.code=t,this.model=i,this._data=tn.create(i,n)}toString(){return(this._data instanceof tn?this._data:tn.deserialize(this._data)).changes.map(t=>t.toString()).join(", ")}matchesResource(e){return(Se.isUri(this.model)?this.model:this.model.uri).toString()===e.toString()}setModel(e){this.model=e}canAppend(e){return this.model===e&&this._data instanceof tn}append(e,t,i,n,s){this._data instanceof tn&&this._data.append(e,t,i,n,s)}close(){this._data instanceof tn&&(this._data=this._data.serialize())}open(){this._data instanceof tn||(this._data=tn.deserialize(this._data))}undo(){if(Se.isUri(this.model))throw new Error("Invalid SingleModelEditStackElement");this._data instanceof tn&&(this._data=this._data.serialize());const e=tn.deserialize(this._data);this.model._applyUndo(e.changes,e.beforeEOL,e.beforeVersionId,e.beforeCursorState)}redo(){if(Se.isUri(this.model))throw new Error("Invalid SingleModelEditStackElement");this._data instanceof tn&&(this._data=this._data.serialize());const e=tn.deserialize(this._data);this.model._applyRedo(e.changes,e.afterEOL,e.afterVersionId,e.afterCursorState)}heapSize(){return this._data instanceof tn&&(this._data=this._data.serialize()),this._data.byteLength+168}}class mue{get resources(){return this._editStackElementsArr.map(e=>e.resource)}constructor(e,t,i){this.label=e,this.code=t,this.type=1,this._isOpen=!0,this._editStackElementsArr=i.slice(0),this._editStackElementsMap=new Map;for(const n of this._editStackElementsArr){const s=Im(n.resource);this._editStackElementsMap.set(s,n)}this._delegate=null}prepareUndoRedo(){if(this._delegate)return this._delegate.prepareUndoRedo(this)}matchesResource(e){const t=Im(e);return this._editStackElementsMap.has(t)}setModel(e){const t=Im(Se.isUri(e)?e:e.uri);this._editStackElementsMap.has(t)&&this._editStackElementsMap.get(t).setModel(e)}canAppend(e){if(!this._isOpen)return!1;const t=Im(e.uri);return this._editStackElementsMap.has(t)?this._editStackElementsMap.get(t).canAppend(e):!1}append(e,t,i,n,s){const r=Im(e.uri);this._editStackElementsMap.get(r).append(e,t,i,n,s)}close(){this._isOpen=!1}open(){}undo(){this._isOpen=!1;for(const e of this._editStackElementsArr)e.undo()}redo(){for(const e of this._editStackElementsArr)e.redo()}heapSize(e){const t=Im(e);return this._editStackElementsMap.has(t)?this._editStackElementsMap.get(t).heapSize():0}split(){return this._editStackElementsArr}toString(){const e=[];for(const t of this._editStackElementsArr)e.push(`${vr(t.resource)}: ${t}`);return`{${e.join(", ")}}`}}function lA(o){return o.getEOL()===`
`?0:1}function yc(o){return o?o instanceof nK||o instanceof mue:!1}class u3{constructor(e,t){this._model=e,this._undoRedoService=t}pushStackElement(){const e=this._undoRedoService.getLastElement(this._model.uri);yc(e)&&e.close()}popStackElement(){const e=this._undoRedoService.getLastElement(this._model.uri);yc(e)&&e.open()}clear(){this._undoRedoService.removeElements(this._model.uri)}_getOrCreateEditStackElement(e,t){const i=this._undoRedoService.getLastElement(this._model.uri);if(yc(i)&&i.canAppend(this._model))return i;const n=new nK(m("edit","Typing"),"undoredo.textBufferEdit",this._model,e);return this._undoRedoService.pushElement(n,t),n}pushEOL(e){const t=this._getOrCreateEditStackElement(null,void 0);this._model.setEOL(e),t.append(this._model,[],lA(this._model),this._model.getAlternativeVersionId(),null)}pushEditOperation(e,t,i,n){const s=this._getOrCreateEditStackElement(e,n),r=this._model.applyEdits(t,!0),a=u3._computeCursorState(i,r),l=r.map((c,d)=>({index:d,textChange:c.textChange}));return l.sort((c,d)=>c.textChange.oldPosition===d.textChange.oldPosition?c.index-d.index:c.textChange.oldPosition-d.textChange.oldPosition),s.append(this._model,l.map(c=>c.textChange),lA(this._model),this._model.getAlternativeVersionId(),a),a}static _computeCursorState(e,t){try{return e?e(t):null}catch(i){return Oe(i),null}}}class sK extends F{constructor(){super(...arguments),this._isDisposed=!1}dispose(){super.dispose(),this._isDisposed=!0}assertNotDisposed(){if(this._isDisposed)throw new Error("TextModelPart is disposed!")}}function rE(o,e){let t=0,i=0;const n=o.length;for(;i<n;){const s=o.charCodeAt(i);if(s===32)t++;else if(s===9)t=t-t%e+e;else break;i++}return i===n?-1:t}var Cg;(function(o){o[o.Disabled=0]="Disabled",o[o.EnabledForActive=1]="EnabledForActive",o[o.Enabled=2]="Enabled"})(Cg||(Cg={}));class cg{constructor(e,t,i,n,s,r){if(this.visibleColumn=e,this.column=t,this.className=i,this.horizontalLine=n,this.forWrappedLinesAfterColumn=s,this.forWrappedLinesBeforeOrAtColumn=r,e!==-1==(t!==-1))throw new Error}}class ev{constructor(e,t){this.top=e,this.endColumn=t}}class pue extends sK{constructor(e,t){super(),this.textModel=e,this.languageConfigurationService=t}getLanguageConfiguration(e){return this.languageConfigurationService.getLanguageConfiguration(e)}_computeIndentLevel(e){return rE(this.textModel.getLineContent(e+1),this.textModel.getOptions().tabSize)}getActiveIndentGuide(e,t,i){this.assertNotDisposed();const n=this.textModel.getLineCount();if(e<1||e>n)throw new bt("Illegal value for lineNumber");const s=this.getLanguageConfiguration(this.textModel.getLanguageId()).foldingRules,r=!!(s&&s.offSide);let a=-2,l=-1,c=-2,d=-1;const h=k=>{if(a!==-1&&(a===-2||a>k-1)){a=-1,l=-1;for(let D=k-2;D>=0;D--){const N=this._computeIndentLevel(D);if(N>=0){a=D,l=N;break}}}if(c===-2){c=-1,d=-1;for(let D=k;D<n;D++){const N=this._computeIndentLevel(D);if(N>=0){c=D,d=N;break}}}};let u=-2,g=-1,f=-2,p=-1;const _=k=>{if(u===-2){u=-1,g=-1;for(let D=k-2;D>=0;D--){const N=this._computeIndentLevel(D);if(N>=0){u=D,g=N;break}}}if(f!==-1&&(f===-2||f<k-1)){f=-1,p=-1;for(let D=k;D<n;D++){const N=this._computeIndentLevel(D);if(N>=0){f=D,p=N;break}}}};let b=0,C=!0,v=0,w=!0,S=0,L=0;for(let k=0;C||w;k++){const D=e-k,N=e+k;k>1&&(D<1||D<t)&&(C=!1),k>1&&(N>n||N>i)&&(w=!1),k>5e4&&(C=!1,w=!1);let B=-1;if(C&&D>=1){const V=this._computeIndentLevel(D-1);V>=0?(c=D-1,d=V,B=Math.ceil(V/this.textModel.getOptions().indentSize)):(h(D),B=this._getIndentLevelForWhitespaceLine(r,l,d))}let W=-1;if(w&&N<=n){const V=this._computeIndentLevel(N-1);V>=0?(u=N-1,g=V,W=Math.ceil(V/this.textModel.getOptions().indentSize)):(_(N),W=this._getIndentLevelForWhitespaceLine(r,g,p))}if(k===0){L=B;continue}if(k===1){if(N<=n&&W>=0&&L+1===W){C=!1,b=N,v=N,S=W;continue}if(D>=1&&B>=0&&B-1===L){w=!1,b=D,v=D,S=B;continue}if(b=e,v=e,S=L,S===0)return{startLineNumber:b,endLineNumber:v,indent:S}}C&&(B>=S?b=D:C=!1),w&&(W>=S?v=N:w=!1)}return{startLineNumber:b,endLineNumber:v,indent:S}}getLinesBracketGuides(e,t,i,n){var h;const s=[];for(let u=e;u<=t;u++)s.push([]);const r=!0,a=this.textModel.bracketPairs.getBracketPairsInRangeWithMinIndentation(new x(e,1,t,this.textModel.getLineMaxColumn(t))).toArray();let l;if(i&&a.length>0){const u=(e<=i.lineNumber&&i.lineNumber<=t?a:this.textModel.bracketPairs.getBracketPairsInRange(x.fromPositions(i)).toArray()).filter(g=>x.strictContainsPosition(g.range,i));l=(h=dw(u,g=>r))==null?void 0:h.range}const c=this.textModel.getOptions().bracketPairColorizationOptions.independentColorPoolPerBracketType,d=new oK;for(const u of a){if(!u.closingBracketRange)continue;const g=l&&u.range.equalsRange(l);if(!g&&!n.includeInactive)continue;const f=d.getInlineClassName(u.nestingLevel,u.nestingLevelOfEqualBracketType,c)+(n.highlightActive&&g?" "+d.activeClassName:""),p=u.openingBracketRange.getStartPosition(),_=u.closingBracketRange.getStartPosition(),b=n.horizontalGuides===Cg.Enabled||n.horizontalGuides===Cg.EnabledForActive&&g;if(u.range.startLineNumber===u.range.endLineNumber){b&&s[u.range.startLineNumber-e].push(new cg(-1,u.openingBracketRange.getEndPosition().column,f,new ev(!1,_.column),-1,-1));continue}const C=this.getVisibleColumnFromPosition(_),v=this.getVisibleColumnFromPosition(u.openingBracketRange.getStartPosition()),w=Math.min(v,C,u.minVisibleColumnIndentation+1);let S=!1;Jn(this.textModel.getLineContent(u.closingBracketRange.startLineNumber))<u.closingBracketRange.startColumn-1&&(S=!0);const D=Math.max(p.lineNumber,e),N=Math.min(_.lineNumber,t),B=S?1:0;for(let W=D;W<N+B;W++)s[W-e].push(new cg(w,-1,f,null,W===p.lineNumber?p.column:-1,W===_.lineNumber?_.column:-1));b&&(p.lineNumber>=e&&v>w&&s[p.lineNumber-e].push(new cg(w,-1,f,new ev(!1,p.column),-1,-1)),_.lineNumber<=t&&C>w&&s[_.lineNumber-e].push(new cg(w,-1,f,new ev(!S,_.column),-1,-1)))}for(const u of s)u.sort((g,f)=>g.visibleColumn-f.visibleColumn);return s}getVisibleColumnFromPosition(e){return Ui.visibleColumnFromColumn(this.textModel.getLineContent(e.lineNumber),e.column,this.textModel.getOptions().tabSize)+1}getLinesIndentGuides(e,t){this.assertNotDisposed();const i=this.textModel.getLineCount();if(e<1||e>i)throw new Error("Illegal value for startLineNumber");if(t<1||t>i)throw new Error("Illegal value for endLineNumber");const n=this.textModel.getOptions(),s=this.getLanguageConfiguration(this.textModel.getLanguageId()).foldingRules,r=!!(s&&s.offSide),a=new Array(t-e+1);let l=-2,c=-1,d=-2,h=-1;for(let u=e;u<=t;u++){const g=u-e,f=this._computeIndentLevel(u-1);if(f>=0){l=u-1,c=f,a[g]=Math.ceil(f/n.indentSize);continue}if(l===-2){l=-1,c=-1;for(let p=u-2;p>=0;p--){const _=this._computeIndentLevel(p);if(_>=0){l=p,c=_;break}}}if(d!==-1&&(d===-2||d<u-1)){d=-1,h=-1;for(let p=u;p<i;p++){const _=this._computeIndentLevel(p);if(_>=0){d=p,h=_;break}}}a[g]=this._getIndentLevelForWhitespaceLine(r,c,h)}return a}_getIndentLevelForWhitespaceLine(e,t,i){const n=this.textModel.getOptions();return t===-1||i===-1?0:t<i?1+Math.floor(t/n.indentSize):t===i||e?Math.ceil(i/n.indentSize):1+Math.floor(i/n.indentSize)}}class oK{constructor(){this.activeClassName="indent-active"}getInlineClassName(e,t,i){return this.getInlineClassNameOfLevel(i?t:e)}getInlineClassNameOfLevel(e){return`bracket-indent-guide lvl-${e%30}`}}class _ue{constructor(){this.spacesDiff=0,this.looksLikeAlignment=!1}}function bue(o,e,t,i,n){n.spacesDiff=0,n.looksLikeAlignment=!1;let s;for(s=0;s<e&&s<i;s++){const u=o.charCodeAt(s),g=t.charCodeAt(s);if(u!==g)break}let r=0,a=0;for(let u=s;u<e;u++)o.charCodeAt(u)===32?r++:a++;let l=0,c=0;for(let u=s;u<i;u++)t.charCodeAt(u)===32?l++:c++;if(r>0&&a>0||l>0&&c>0)return;const d=Math.abs(a-c),h=Math.abs(r-l);if(d===0){n.spacesDiff=h,h>0&&0<=l-1&&l-1<o.length&&l<t.length&&t.charCodeAt(l)!==32&&o.charCodeAt(l-1)===32&&o.charCodeAt(o.length-1)===44&&(n.looksLikeAlignment=!0);return}if(h%d===0){n.spacesDiff=h/d;return}}function v8(o,e,t){const i=Math.min(o.getLineCount(),1e4);let n=0,s=0,r="",a=0;const l=[2,4,6,8,3,5,7],c=8,d=[0,0,0,0,0,0,0,0,0],h=new _ue;for(let f=1;f<=i;f++){const p=o.getLineLength(f),_=o.getLineContent(f),b=p<=65536;let C=!1,v=0,w=0,S=0;for(let k=0,D=p;k<D;k++){const N=b?_.charCodeAt(k):o.getLineCharCode(f,k);if(N===9)S++;else if(N===32)w++;else{C=!0,v=k;break}}if(!C||(S>0?n++:w>1&&s++,bue(r,a,_,v,h),h.looksLikeAlignment&&!(t&&e===h.spacesDiff)))continue;const L=h.spacesDiff;L<=c&&d[L]++,r=_,a=v}let u=t;n!==s&&(u=n<s);let g=e;if(u){let f=u?0:.1*i;l.forEach(p=>{const _=d[p];_>f&&(f=_,g=p)}),g===4&&d[4]>0&&d[2]>0&&d[2]>=d[4]/2&&(g=2)}return{insertSpaces:u,tabSize:g}}function Ds(o){return(o.metadata&1)>>>0}function Gt(o,e){o.metadata=o.metadata&254|e<<0}function In(o){return(o.metadata&2)>>>1===1}function $t(o,e){o.metadata=o.metadata&253|(e?1:0)<<1}function rK(o){return(o.metadata&4)>>>2===1}function w8(o,e){o.metadata=o.metadata&251|(e?1:0)<<2}function aK(o){return(o.metadata&64)>>>6===1}function S8(o,e){o.metadata=o.metadata&191|(e?1:0)<<6}function Cue(o){return(o.metadata&24)>>>3}function y8(o,e){o.metadata=o.metadata&231|e<<3}function vue(o){return(o.metadata&32)>>>5===1}function L8(o,e){o.metadata=o.metadata&223|(e?1:0)<<5}class lK{constructor(e,t,i){this.metadata=0,this.parent=this,this.left=this,this.right=this,Gt(this,1),this.start=t,this.end=i,this.delta=0,this.maxEnd=i,this.id=e,this.ownerId=0,this.options=null,w8(this,!1),S8(this,!1),y8(this,1),L8(this,!1),this.cachedVersionId=0,this.cachedAbsoluteStart=t,this.cachedAbsoluteEnd=i,this.range=null,$t(this,!1)}reset(e,t,i,n){this.start=t,this.end=i,this.maxEnd=i,this.cachedVersionId=e,this.cachedAbsoluteStart=t,this.cachedAbsoluteEnd=i,this.range=n}setOptions(e){this.options=e;const t=this.options.className;w8(this,t==="squiggly-error"||t==="squiggly-warning"||t==="squiggly-info"),S8(this,this.options.glyphMarginClassName!==null),y8(this,this.options.stickiness),L8(this,this.options.collapseOnReplaceEdit)}setCachedOffsets(e,t,i){this.cachedVersionId!==i&&(this.range=null),this.cachedVersionId=i,this.cachedAbsoluteStart=e,this.cachedAbsoluteEnd=t}detach(){this.parent=null,this.left=null,this.right=null}}const Qe=new lK(null,0,0);Qe.parent=Qe;Qe.left=Qe;Qe.right=Qe;Gt(Qe,0);class eT{constructor(){this.root=Qe,this.requestNormalizeDelta=!1}intervalSearch(e,t,i,n,s,r){return this.root===Qe?[]:Iue(this,e,t,i,n,s,r)}search(e,t,i,n){return this.root===Qe?[]:Due(this,e,t,i,n)}collectNodesFromOwner(e){return xue(this,e)}collectNodesPostOrder(){return kue(this)}insert(e){x8(this,e),this._normalizeDeltaIfNecessary()}delete(e){k8(this,e),this._normalizeDeltaIfNecessary()}resolveNode(e,t){const i=e;let n=0;for(;e!==this.root;)e===e.parent.right&&(n+=e.parent.delta),e=e.parent;const s=i.start+n,r=i.end+n;i.setCachedOffsets(s,r,t)}acceptReplace(e,t,i,n){const s=yue(this,e,e+t);for(let r=0,a=s.length;r<a;r++){const l=s[r];k8(this,l)}this._normalizeDeltaIfNecessary(),Lue(this,e,e+t,i),this._normalizeDeltaIfNecessary();for(let r=0,a=s.length;r<a;r++){const l=s[r];l.start=l.cachedAbsoluteStart,l.end=l.cachedAbsoluteEnd,Sue(l,e,e+t,i,n),l.maxEnd=l.end,x8(this,l)}this._normalizeDeltaIfNecessary()}_normalizeDeltaIfNecessary(){this.requestNormalizeDelta&&(this.requestNormalizeDelta=!1,wue(this))}}function wue(o){let e=o.root,t=0;for(;e!==Qe;){if(e.left!==Qe&&!In(e.left)){e=e.left;continue}if(e.right!==Qe&&!In(e.right)){t+=e.delta,e=e.right;continue}e.start=t+e.start,e.end=t+e.end,e.delta=0,jh(e),$t(e,!0),$t(e.left,!1),$t(e.right,!1),e===e.parent.right&&(t-=e.parent.delta),e=e.parent}$t(o.root,!1)}function Em(o,e,t,i){return o<t?!0:o>t||i===1?!1:i===2?!0:e}function Sue(o,e,t,i,n){const s=Cue(o),r=s===0||s===2,a=s===1||s===2,l=t-e,c=i,d=Math.min(l,c),h=o.start;let u=!1;const g=o.end;let f=!1;e<=h&&g<=t&&vue(o)&&(o.start=e,u=!0,o.end=e,f=!0);{const _=n?1:l>0?2:0;!u&&Em(h,r,e,_)&&(u=!0),!f&&Em(g,a,e,_)&&(f=!0)}if(d>0&&!n){const _=l>c?2:0;!u&&Em(h,r,e+d,_)&&(u=!0),!f&&Em(g,a,e+d,_)&&(f=!0)}{const _=n?1:0;!u&&Em(h,r,t,_)&&(o.start=e+c,u=!0),!f&&Em(g,a,t,_)&&(o.end=e+c,f=!0)}const p=c-l;u||(o.start=Math.max(0,h+p)),f||(o.end=Math.max(0,g+p)),o.start>o.end&&(o.end=o.start)}function yue(o,e,t){let i=o.root,n=0,s=0,r=0,a=0;const l=[];let c=0;for(;i!==Qe;){if(In(i)){$t(i.left,!1),$t(i.right,!1),i===i.parent.right&&(n-=i.parent.delta),i=i.parent;continue}if(!In(i.left)){if(s=n+i.maxEnd,s<e){$t(i,!0);continue}if(i.left!==Qe){i=i.left;continue}}if(r=n+i.start,r>t){$t(i,!0);continue}if(a=n+i.end,a>=e&&(i.setCachedOffsets(r,a,0),l[c++]=i),$t(i,!0),i.right!==Qe&&!In(i.right)){n+=i.delta,i=i.right;continue}}return $t(o.root,!1),l}function Lue(o,e,t,i){let n=o.root,s=0,r=0,a=0;const l=i-(t-e);for(;n!==Qe;){if(In(n)){$t(n.left,!1),$t(n.right,!1),n===n.parent.right&&(s-=n.parent.delta),jh(n),n=n.parent;continue}if(!In(n.left)){if(r=s+n.maxEnd,r<e){$t(n,!0);continue}if(n.left!==Qe){n=n.left;continue}}if(a=s+n.start,a>t){n.start+=l,n.end+=l,n.delta+=l,(n.delta<-1073741824||n.delta>1073741824)&&(o.requestNormalizeDelta=!0),$t(n,!0);continue}if($t(n,!0),n.right!==Qe&&!In(n.right)){s+=n.delta,n=n.right;continue}}$t(o.root,!1)}function xue(o,e){let t=o.root;const i=[];let n=0;for(;t!==Qe;){if(In(t)){$t(t.left,!1),$t(t.right,!1),t=t.parent;continue}if(t.left!==Qe&&!In(t.left)){t=t.left;continue}if(t.ownerId===e&&(i[n++]=t),$t(t,!0),t.right!==Qe&&!In(t.right)){t=t.right;continue}}return $t(o.root,!1),i}function kue(o){let e=o.root;const t=[];let i=0;for(;e!==Qe;){if(In(e)){$t(e.left,!1),$t(e.right,!1),e=e.parent;continue}if(e.left!==Qe&&!In(e.left)){e=e.left;continue}if(e.right!==Qe&&!In(e.right)){e=e.right;continue}t[i++]=e,$t(e,!0)}return $t(o.root,!1),t}function Due(o,e,t,i,n){let s=o.root,r=0,a=0,l=0;const c=[];let d=0;for(;s!==Qe;){if(In(s)){$t(s.left,!1),$t(s.right,!1),s===s.parent.right&&(r-=s.parent.delta),s=s.parent;continue}if(s.left!==Qe&&!In(s.left)){s=s.left;continue}a=r+s.start,l=r+s.end,s.setCachedOffsets(a,l,i);let h=!0;if(e&&s.ownerId&&s.ownerId!==e&&(h=!1),t&&rK(s)&&(h=!1),n&&!aK(s)&&(h=!1),h&&(c[d++]=s),$t(s,!0),s.right!==Qe&&!In(s.right)){r+=s.delta,s=s.right;continue}}return $t(o.root,!1),c}function Iue(o,e,t,i,n,s,r){let a=o.root,l=0,c=0,d=0,h=0;const u=[];let g=0;for(;a!==Qe;){if(In(a)){$t(a.left,!1),$t(a.right,!1),a===a.parent.right&&(l-=a.parent.delta),a=a.parent;continue}if(!In(a.left)){if(c=l+a.maxEnd,c<e){$t(a,!0);continue}if(a.left!==Qe){a=a.left;continue}}if(d=l+a.start,d>t){$t(a,!0);continue}if(h=l+a.end,h>=e){a.setCachedOffsets(d,h,s);let f=!0;i&&a.ownerId&&a.ownerId!==i&&(f=!1),n&&rK(a)&&(f=!1),r&&!aK(a)&&(f=!1),f&&(u[g++]=a)}if($t(a,!0),a.right!==Qe&&!In(a.right)){l+=a.delta,a=a.right;continue}}return $t(o.root,!1),u}function x8(o,e){if(o.root===Qe)return e.parent=Qe,e.left=Qe,e.right=Qe,Gt(e,0),o.root=e,o.root;Eue(o,e),Dd(e.parent);let t=e;for(;t!==o.root&&Ds(t.parent)===1;)if(t.parent===t.parent.parent.left){const i=t.parent.parent.right;Ds(i)===1?(Gt(t.parent,0),Gt(i,0),Gt(t.parent.parent,1),t=t.parent.parent):(t===t.parent.right&&(t=t.parent,tv(o,t)),Gt(t.parent,0),Gt(t.parent.parent,1),iv(o,t.parent.parent))}else{const i=t.parent.parent.left;Ds(i)===1?(Gt(t.parent,0),Gt(i,0),Gt(t.parent.parent,1),t=t.parent.parent):(t===t.parent.left&&(t=t.parent,iv(o,t)),Gt(t.parent,0),Gt(t.parent.parent,1),tv(o,t.parent.parent))}return Gt(o.root,0),e}function Eue(o,e){let t=0,i=o.root;const n=e.start,s=e.end;for(;;)if(Tue(n,s,i.start+t,i.end+t)<0)if(i.left===Qe){e.start-=t,e.end-=t,e.maxEnd-=t,i.left=e;break}else i=i.left;else if(i.right===Qe){e.start-=t+i.delta,e.end-=t+i.delta,e.maxEnd-=t+i.delta,i.right=e;break}else t+=i.delta,i=i.right;e.parent=i,e.left=Qe,e.right=Qe,Gt(e,1)}function k8(o,e){let t,i;if(e.left===Qe?(t=e.right,i=e,t.delta+=e.delta,(t.delta<-1073741824||t.delta>1073741824)&&(o.requestNormalizeDelta=!0),t.start+=e.delta,t.end+=e.delta):e.right===Qe?(t=e.left,i=e):(i=Nue(e.right),t=i.right,t.start+=i.delta,t.end+=i.delta,t.delta+=i.delta,(t.delta<-1073741824||t.delta>1073741824)&&(o.requestNormalizeDelta=!0),i.start+=e.delta,i.end+=e.delta,i.delta=e.delta,(i.delta<-1073741824||i.delta>1073741824)&&(o.requestNormalizeDelta=!0)),i===o.root){o.root=t,Gt(t,0),e.detach(),tT(),jh(t),o.root.parent=Qe;return}const n=Ds(i)===1;if(i===i.parent.left?i.parent.left=t:i.parent.right=t,i===e?t.parent=i.parent:(i.parent===e?t.parent=i:t.parent=i.parent,i.left=e.left,i.right=e.right,i.parent=e.parent,Gt(i,Ds(e)),e===o.root?o.root=i:e===e.parent.left?e.parent.left=i:e.parent.right=i,i.left!==Qe&&(i.left.parent=i),i.right!==Qe&&(i.right.parent=i)),e.detach(),n){Dd(t.parent),i!==e&&(Dd(i),Dd(i.parent)),tT();return}Dd(t),Dd(t.parent),i!==e&&(Dd(i),Dd(i.parent));let s;for(;t!==o.root&&Ds(t)===0;)t===t.parent.left?(s=t.parent.right,Ds(s)===1&&(Gt(s,0),Gt(t.parent,1),tv(o,t.parent),s=t.parent.right),Ds(s.left)===0&&Ds(s.right)===0?(Gt(s,1),t=t.parent):(Ds(s.right)===0&&(Gt(s.left,0),Gt(s,1),iv(o,s),s=t.parent.right),Gt(s,Ds(t.parent)),Gt(t.parent,0),Gt(s.right,0),tv(o,t.parent),t=o.root)):(s=t.parent.left,Ds(s)===1&&(Gt(s,0),Gt(t.parent,1),iv(o,t.parent),s=t.parent.left),Ds(s.left)===0&&Ds(s.right)===0?(Gt(s,1),t=t.parent):(Ds(s.left)===0&&(Gt(s.right,0),Gt(s,1),tv(o,s),s=t.parent.left),Gt(s,Ds(t.parent)),Gt(t.parent,0),Gt(s.left,0),iv(o,t.parent),t=o.root));Gt(t,0),tT()}function Nue(o){for(;o.left!==Qe;)o=o.left;return o}function tT(){Qe.parent=Qe,Qe.delta=0,Qe.start=0,Qe.end=0}function tv(o,e){const t=e.right;t.delta+=e.delta,(t.delta<-1073741824||t.delta>1073741824)&&(o.requestNormalizeDelta=!0),t.start+=e.delta,t.end+=e.delta,e.right=t.left,t.left!==Qe&&(t.left.parent=e),t.parent=e.parent,e.parent===Qe?o.root=t:e===e.parent.left?e.parent.left=t:e.parent.right=t,t.left=e,e.parent=t,jh(e),jh(t)}function iv(o,e){const t=e.left;e.delta-=t.delta,(e.delta<-1073741824||e.delta>1073741824)&&(o.requestNormalizeDelta=!0),e.start-=t.delta,e.end-=t.delta,e.left=t.right,t.right!==Qe&&(t.right.parent=e),t.parent=e.parent,e.parent===Qe?o.root=t:e===e.parent.right?e.parent.right=t:e.parent.left=t,t.right=e,e.parent=t,jh(e),jh(t)}function cK(o){let e=o.end;if(o.left!==Qe){const t=o.left.maxEnd;t>e&&(e=t)}if(o.right!==Qe){const t=o.right.maxEnd+o.delta;t>e&&(e=t)}return e}function jh(o){o.maxEnd=cK(o)}function Dd(o){for(;o!==Qe;){const e=cK(o);if(o.maxEnd===e)return;o.maxEnd=e,o=o.parent}}function Tue(o,e,t,i){return o===t?e-i:o-t}class cA{constructor(e,t){this.piece=e,this.color=t,this.size_left=0,this.lf_left=0,this.parent=this,this.left=this,this.right=this}next(){if(this.right!==He)return g3(this.right);let e=this;for(;e.parent!==He&&e.parent.left!==e;)e=e.parent;return e.parent===He?He:e.parent}prev(){if(this.left!==He)return dK(this.left);let e=this;for(;e.parent!==He&&e.parent.right!==e;)e=e.parent;return e.parent===He?He:e.parent}detach(){this.parent=null,this.left=null,this.right=null}}const He=new cA(null,0);He.parent=He;He.left=He;He.right=He;He.color=0;function g3(o){for(;o.left!==He;)o=o.left;return o}function dK(o){for(;o.right!==He;)o=o.right;return o}function f3(o){return o===He?0:o.size_left+o.piece.length+f3(o.right)}function m3(o){return o===He?0:o.lf_left+o.piece.lineFeedCnt+m3(o.right)}function iT(){He.parent=He}function nv(o,e){const t=e.right;t.size_left+=e.size_left+(e.piece?e.piece.length:0),t.lf_left+=e.lf_left+(e.piece?e.piece.lineFeedCnt:0),e.right=t.left,t.left!==He&&(t.left.parent=e),t.parent=e.parent,e.parent===He?o.root=t:e.parent.left===e?e.parent.left=t:e.parent.right=t,t.left=e,e.parent=t}function sv(o,e){const t=e.left;e.left=t.right,t.right!==He&&(t.right.parent=e),t.parent=e.parent,e.size_left-=t.size_left+(t.piece?t.piece.length:0),e.lf_left-=t.lf_left+(t.piece?t.piece.lineFeedCnt:0),e.parent===He?o.root=t:e===e.parent.right?e.parent.right=t:e.parent.left=t,t.right=e,e.parent=t}function QS(o,e){let t,i;if(e.left===He?(i=e,t=i.right):e.right===He?(i=e,t=i.left):(i=g3(e.right),t=i.right),i===o.root){o.root=t,t.color=0,e.detach(),iT(),o.root.parent=He;return}const n=i.color===1;if(i===i.parent.left?i.parent.left=t:i.parent.right=t,i===e?(t.parent=i.parent,EC(o,t)):(i.parent===e?t.parent=i:t.parent=i.parent,EC(o,t),i.left=e.left,i.right=e.right,i.parent=e.parent,i.color=e.color,e===o.root?o.root=i:e===e.parent.left?e.parent.left=i:e.parent.right=i,i.left!==He&&(i.left.parent=i),i.right!==He&&(i.right.parent=i),i.size_left=e.size_left,i.lf_left=e.lf_left,EC(o,i)),e.detach(),t.parent.left===t){const r=f3(t),a=m3(t);if(r!==t.parent.size_left||a!==t.parent.lf_left){const l=r-t.parent.size_left,c=a-t.parent.lf_left;t.parent.size_left=r,t.parent.lf_left=a,cc(o,t.parent,l,c)}}if(EC(o,t.parent),n){iT();return}let s;for(;t!==o.root&&t.color===0;)t===t.parent.left?(s=t.parent.right,s.color===1&&(s.color=0,t.parent.color=1,nv(o,t.parent),s=t.parent.right),s.left.color===0&&s.right.color===0?(s.color=1,t=t.parent):(s.right.color===0&&(s.left.color=0,s.color=1,sv(o,s),s=t.parent.right),s.color=t.parent.color,t.parent.color=0,s.right.color=0,nv(o,t.parent),t=o.root)):(s=t.parent.left,s.color===1&&(s.color=0,t.parent.color=1,sv(o,t.parent),s=t.parent.left),s.left.color===0&&s.right.color===0?(s.color=1,t=t.parent):(s.left.color===0&&(s.right.color=0,s.color=1,nv(o,s),s=t.parent.left),s.color=t.parent.color,t.parent.color=0,s.left.color=0,sv(o,t.parent),t=o.root));t.color=0,iT()}function D8(o,e){for(EC(o,e);e!==o.root&&e.parent.color===1;)if(e.parent===e.parent.parent.left){const t=e.parent.parent.right;t.color===1?(e.parent.color=0,t.color=0,e.parent.parent.color=1,e=e.parent.parent):(e===e.parent.right&&(e=e.parent,nv(o,e)),e.parent.color=0,e.parent.parent.color=1,sv(o,e.parent.parent))}else{const t=e.parent.parent.left;t.color===1?(e.parent.color=0,t.color=0,e.parent.parent.color=1,e=e.parent.parent):(e===e.parent.left&&(e=e.parent,sv(o,e)),e.parent.color=0,e.parent.parent.color=1,nv(o,e.parent.parent))}o.root.color=0}function cc(o,e,t,i){for(;e!==o.root&&e!==He;)e.parent.left===e&&(e.parent.size_left+=t,e.parent.lf_left+=i),e=e.parent}function EC(o,e){let t=0,i=0;if(e!==o.root){for(;e!==o.root&&e===e.parent.right;)e=e.parent;if(e!==o.root)for(e=e.parent,t=f3(e.left)-e.size_left,i=m3(e.left)-e.lf_left,e.size_left+=t,e.lf_left+=i;e!==o.root&&(t!==0||i!==0);)e.parent.left===e&&(e.parent.size_left+=t,e.parent.lf_left+=i),e=e.parent}}const sc=65535;function hK(o){let e;return o[o.length-1]<65536?e=new Uint16Array(o.length):e=new Uint32Array(o.length),e.set(o,0),e}class Rue{constructor(e,t,i,n,s){this.lineStarts=e,this.cr=t,this.lf=i,this.crlf=n,this.isBasicASCII=s}}function dc(o,e=!0){const t=[0];let i=1;for(let n=0,s=o.length;n<s;n++){const r=o.charCodeAt(n);r===13?n+1<s&&o.charCodeAt(n+1)===10?(t[i++]=n+2,n++):t[i++]=n+1:r===10&&(t[i++]=n+1)}return e?hK(t):t}function Mue(o,e){o.length=0,o[0]=0;let t=1,i=0,n=0,s=0,r=!0;for(let l=0,c=e.length;l<c;l++){const d=e.charCodeAt(l);d===13?l+1<c&&e.charCodeAt(l+1)===10?(s++,o[t++]=l+2,l++):(i++,o[t++]=l+1):d===10?(n++,o[t++]=l+1):r&&d!==9&&(d<32||d>126)&&(r=!1)}const a=new Rue(hK(o),i,n,s,r);return o.length=0,a}class qs{constructor(e,t,i,n,s){this.bufferIndex=e,this.start=t,this.end=i,this.lineFeedCnt=n,this.length=s}}class qu{constructor(e,t){this.buffer=e,this.lineStarts=t}}class Aue{constructor(e,t){this._pieces=[],this._tree=e,this._BOM=t,this._index=0,e.root!==He&&e.iterate(e.root,i=>(i!==He&&this._pieces.push(i.piece),!0))}read(){return this._pieces.length===0?this._index===0?(this._index++,this._BOM):null:this._index>this._pieces.length-1?null:this._index===0?this._BOM+this._tree.getPieceContent(this._pieces[this._index++]):this._tree.getPieceContent(this._pieces[this._index++])}}class Pue{constructor(e){this._limit=e,this._cache=[]}get(e){for(let t=this._cache.length-1;t>=0;t--){const i=this._cache[t];if(i.nodeStartOffset<=e&&i.nodeStartOffset+i.node.piece.length>=e)return i}return null}get2(e){for(let t=this._cache.length-1;t>=0;t--){const i=this._cache[t];if(i.nodeStartLineNumber&&i.nodeStartLineNumber<e&&i.nodeStartLineNumber+i.node.piece.lineFeedCnt>=e)return i}return null}set(e){this._cache.length>=this._limit&&this._cache.shift(),this._cache.push(e)}validate(e){let t=!1;const i=this._cache;for(let n=0;n<i.length;n++){const s=i[n];if(s.node.parent===null||s.nodeStartOffset>=e){i[n]=null,t=!0;continue}}if(t){const n=[];for(const s of i)s!==null&&n.push(s);this._cache=n}}}class Oue{constructor(e,t,i){this.create(e,t,i)}create(e,t,i){this._buffers=[new qu("",[0])],this._lastChangeBufferPos={line:0,column:0},this.root=He,this._lineCnt=1,this._length=0,this._EOL=t,this._EOLLength=t.length,this._EOLNormalized=i;let n=null;for(let s=0,r=e.length;s<r;s++)if(e[s].buffer.length>0){e[s].lineStarts||(e[s].lineStarts=dc(e[s].buffer));const a=new qs(s+1,{line:0,column:0},{line:e[s].lineStarts.length-1,column:e[s].buffer.length-e[s].lineStarts[e[s].lineStarts.length-1]},e[s].lineStarts.length-1,e[s].buffer.length);this._buffers.push(e[s]),n=this.rbInsertRight(n,a)}this._searchCache=new Pue(1),this._lastVisitedLine={lineNumber:0,value:""},this.computeBufferMetadata()}normalizeEOL(e){const t=sc,i=t-Math.floor(t/3),n=i*2;let s="",r=0;const a=[];if(this.iterate(this.root,l=>{const c=this.getNodeContent(l),d=c.length;if(r<=i||r+d<n)return s+=c,r+=d,!0;const h=s.replace(/\r\n|\r|\n/g,e);return a.push(new qu(h,dc(h))),s=c,r=d,!0}),r>0){const l=s.replace(/\r\n|\r|\n/g,e);a.push(new qu(l,dc(l)))}this.create(a,e,!0)}getEOL(){return this._EOL}setEOL(e){this._EOL=e,this._EOLLength=this._EOL.length,this.normalizeEOL(e)}createSnapshot(e){return new Aue(this,e)}getOffsetAt(e,t){let i=0,n=this.root;for(;n!==He;)if(n.left!==He&&n.lf_left+1>=e)n=n.left;else if(n.lf_left+n.piece.lineFeedCnt+1>=e){i+=n.size_left;const s=this.getAccumulatedValue(n,e-n.lf_left-2);return i+=s+t-1}else e-=n.lf_left+n.piece.lineFeedCnt,i+=n.size_left+n.piece.length,n=n.right;return i}getPositionAt(e){e=Math.floor(e),e=Math.max(0,e);let t=this.root,i=0;const n=e;for(;t!==He;)if(t.size_left!==0&&t.size_left>=e)t=t.left;else if(t.size_left+t.piece.length>=e){const s=this.getIndexOf(t,e-t.size_left);if(i+=t.lf_left+s.index,s.index===0){const r=this.getOffsetAt(i+1,1),a=n-r;return new A(i+1,a+1)}return new A(i+1,s.remainder+1)}else if(e-=t.size_left+t.piece.length,i+=t.lf_left+t.piece.lineFeedCnt,t.right===He){const s=this.getOffsetAt(i+1,1),r=n-e-s;return new A(i+1,r+1)}else t=t.right;return new A(1,1)}getValueInRange(e,t){if(e.startLineNumber===e.endLineNumber&&e.startColumn===e.endColumn)return"";const i=this.nodeAt2(e.startLineNumber,e.startColumn),n=this.nodeAt2(e.endLineNumber,e.endColumn),s=this.getValueInRange2(i,n);return t?t!==this._EOL||!this._EOLNormalized?s.replace(/\r\n|\r|\n/g,t):t===this.getEOL()&&this._EOLNormalized?s:s.replace(/\r\n|\r|\n/g,t):s}getValueInRange2(e,t){if(e.node===t.node){const a=e.node,l=this._buffers[a.piece.bufferIndex].buffer,c=this.offsetInBuffer(a.piece.bufferIndex,a.piece.start);return l.substring(c+e.remainder,c+t.remainder)}let i=e.node;const n=this._buffers[i.piece.bufferIndex].buffer,s=this.offsetInBuffer(i.piece.bufferIndex,i.piece.start);let r=n.substring(s+e.remainder,s+i.piece.length);for(i=i.next();i!==He;){const a=this._buffers[i.piece.bufferIndex].buffer,l=this.offsetInBuffer(i.piece.bufferIndex,i.piece.start);if(i===t.node){r+=a.substring(l,l+t.remainder);break}else r+=a.substr(l,i.piece.length);i=i.next()}return r}getLinesContent(){const e=[];let t=0,i="",n=!1;return this.iterate(this.root,s=>{if(s===He)return!0;const r=s.piece;let a=r.length;if(a===0)return!0;const l=this._buffers[r.bufferIndex].buffer,c=this._buffers[r.bufferIndex].lineStarts,d=r.start.line,h=r.end.line;let u=c[d]+r.start.column;if(n&&(l.charCodeAt(u)===10&&(u++,a--),e[t++]=i,i="",n=!1,a===0))return!0;if(d===h)return!this._EOLNormalized&&l.charCodeAt(u+a-1)===13?(n=!0,i+=l.substr(u,a-1)):i+=l.substr(u,a),!0;i+=this._EOLNormalized?l.substring(u,Math.max(u,c[d+1]-this._EOLLength)):l.substring(u,c[d+1]).replace(/(\r\n|\r|\n)$/,""),e[t++]=i;for(let g=d+1;g<h;g++)i=this._EOLNormalized?l.substring(c[g],c[g+1]-this._EOLLength):l.substring(c[g],c[g+1]).replace(/(\r\n|\r|\n)$/,""),e[t++]=i;return!this._EOLNormalized&&l.charCodeAt(c[h]+r.end.column-1)===13?(n=!0,r.end.column===0?t--:i=l.substr(c[h],r.end.column-1)):i=l.substr(c[h],r.end.column),!0}),n&&(e[t++]=i,i=""),e[t++]=i,e}getLength(){return this._length}getLineCount(){return this._lineCnt}getLineContent(e){return this._lastVisitedLine.lineNumber===e?this._lastVisitedLine.value:(this._lastVisitedLine.lineNumber=e,e===this._lineCnt?this._lastVisitedLine.value=this.getLineRawContent(e):this._EOLNormalized?this._lastVisitedLine.value=this.getLineRawContent(e,this._EOLLength):this._lastVisitedLine.value=this.getLineRawContent(e).replace(/(\r\n|\r|\n)$/,""),this._lastVisitedLine.value)}_getCharCode(e){if(e.remainder===e.node.piece.length){const t=e.node.next();if(!t)return 0;const i=this._buffers[t.piece.bufferIndex],n=this.offsetInBuffer(t.piece.bufferIndex,t.piece.start);return i.buffer.charCodeAt(n)}else{const t=this._buffers[e.node.piece.bufferIndex],n=this.offsetInBuffer(e.node.piece.bufferIndex,e.node.piece.start)+e.remainder;return t.buffer.charCodeAt(n)}}getLineCharCode(e,t){const i=this.nodeAt2(e,t+1);return this._getCharCode(i)}getLineLength(e){if(e===this.getLineCount()){const t=this.getOffsetAt(e,1);return this.getLength()-t}return this.getOffsetAt(e+1,1)-this.getOffsetAt(e,1)-this._EOLLength}findMatchesInNode(e,t,i,n,s,r,a,l,c,d,h){const u=this._buffers[e.piece.bufferIndex],g=this.offsetInBuffer(e.piece.bufferIndex,e.piece.start),f=this.offsetInBuffer(e.piece.bufferIndex,s),p=this.offsetInBuffer(e.piece.bufferIndex,r);let _;const b={line:0,column:0};let C,v;t._wordSeparators?(C=u.buffer.substring(f,p),v=w=>w+f,t.reset(0)):(C=u.buffer,v=w=>w,t.reset(f));do if(_=t.next(C),_){if(v(_.index)>=p)return d;this.positionInBuffer(e,v(_.index)-g,b);const w=this.getLineFeedCnt(e.piece.bufferIndex,s,b),S=b.line===s.line?b.column-s.column+n:b.column+1,L=S+_[0].length;if(h[d++]=Vu(new x(i+w,S,i+w,L),_,l),v(_.index)+_[0].length>=p||d>=c)return d}while(_);return d}findMatchesLineByLine(e,t,i,n){const s=[];let r=0;const a=new np(t.wordSeparators,t.regex);let l=this.nodeAt2(e.startLineNumber,e.startColumn);if(l===null)return[];const c=this.nodeAt2(e.endLineNumber,e.endColumn);if(c===null)return[];let d=this.positionInBuffer(l.node,l.remainder);const h=this.positionInBuffer(c.node,c.remainder);if(l.node===c.node)return this.findMatchesInNode(l.node,a,e.startLineNumber,e.startColumn,d,h,t,i,n,r,s),s;let u=e.startLineNumber,g=l.node;for(;g!==c.node;){const p=this.getLineFeedCnt(g.piece.bufferIndex,d,g.piece.end);if(p>=1){const b=this._buffers[g.piece.bufferIndex].lineStarts,C=this.offsetInBuffer(g.piece.bufferIndex,g.piece.start),v=b[d.line+p],w=u===e.startLineNumber?e.startColumn:1;if(r=this.findMatchesInNode(g,a,u,w,d,this.positionInBuffer(g,v-C),t,i,n,r,s),r>=n)return s;u+=p}const _=u===e.startLineNumber?e.startColumn-1:0;if(u===e.endLineNumber){const b=this.getLineContent(u).substring(_,e.endColumn-1);return r=this._findMatchesInLine(t,a,b,e.endLineNumber,_,r,s,i,n),s}if(r=this._findMatchesInLine(t,a,this.getLineContent(u).substr(_),u,_,r,s,i,n),r>=n)return s;u++,l=this.nodeAt2(u,1),g=l.node,d=this.positionInBuffer(l.node,l.remainder)}if(u===e.endLineNumber){const p=u===e.startLineNumber?e.startColumn-1:0,_=this.getLineContent(u).substring(p,e.endColumn-1);return r=this._findMatchesInLine(t,a,_,e.endLineNumber,p,r,s,i,n),s}const f=u===e.startLineNumber?e.startColumn:1;return r=this.findMatchesInNode(c.node,a,u,f,d,h,t,i,n,r,s),s}_findMatchesInLine(e,t,i,n,s,r,a,l,c){const d=e.wordSeparators;if(!l&&e.simpleSearch){const u=e.simpleSearch,g=u.length,f=i.length;let p=-g;for(;(p=i.indexOf(u,p+g))!==-1;)if((!d||p5(d,i,f,p,g))&&(a[r++]=new cw(new x(n,p+1+s,n,p+1+g+s),null),r>=c))return r;return r}let h;t.reset(0);do if(h=t.next(i),h&&(a[r++]=Vu(new x(n,h.index+1+s,n,h.index+1+h[0].length+s),h,l),r>=c))return r;while(h);return r}insert(e,t,i=!1){if(this._EOLNormalized=this._EOLNormalized&&i,this._lastVisitedLine.lineNumber=0,this._lastVisitedLine.value="",this.root!==He){const{node:n,remainder:s,nodeStartOffset:r}=this.nodeAt(e),a=n.piece,l=a.bufferIndex,c=this.positionInBuffer(n,s);if(n.piece.bufferIndex===0&&a.end.line===this._lastChangeBufferPos.line&&a.end.column===this._lastChangeBufferPos.column&&r+a.length===e&&t.length<sc){this.appendToNode(n,t),this.computeBufferMetadata();return}if(r===e)this.insertContentToNodeLeft(t,n),this._searchCache.validate(e);else if(r+n.piece.length>e){const d=[];let h=new qs(a.bufferIndex,c,a.end,this.getLineFeedCnt(a.bufferIndex,c,a.end),this.offsetInBuffer(l,a.end)-this.offsetInBuffer(l,c));if(this.shouldCheckCRLF()&&this.endWithCR(t)&&this.nodeCharCodeAt(n,s)===10){const p={line:h.start.line+1,column:0};h=new qs(h.bufferIndex,p,h.end,this.getLineFeedCnt(h.bufferIndex,p,h.end),h.length-1),t+=`
`}if(this.shouldCheckCRLF()&&this.startWithLF(t))if(this.nodeCharCodeAt(n,s-1)===13){const p=this.positionInBuffer(n,s-1);this.deleteNodeTail(n,p),t="\r"+t,n.piece.length===0&&d.push(n)}else this.deleteNodeTail(n,c);else this.deleteNodeTail(n,c);const u=this.createNewPieces(t);h.length>0&&this.rbInsertRight(n,h);let g=n;for(let f=0;f<u.length;f++)g=this.rbInsertRight(g,u[f]);this.deleteNodes(d)}else this.insertContentToNodeRight(t,n)}else{const n=this.createNewPieces(t);let s=this.rbInsertLeft(null,n[0]);for(let r=1;r<n.length;r++)s=this.rbInsertRight(s,n[r])}this.computeBufferMetadata()}delete(e,t){if(this._lastVisitedLine.lineNumber=0,this._lastVisitedLine.value="",t<=0||this.root===He)return;const i=this.nodeAt(e),n=this.nodeAt(e+t),s=i.node,r=n.node;if(s===r){const u=this.positionInBuffer(s,i.remainder),g=this.positionInBuffer(s,n.remainder);if(i.nodeStartOffset===e){if(t===s.piece.length){const f=s.next();QS(this,s),this.validateCRLFWithPrevNode(f),this.computeBufferMetadata();return}this.deleteNodeHead(s,g),this._searchCache.validate(e),this.validateCRLFWithPrevNode(s),this.computeBufferMetadata();return}if(i.nodeStartOffset+s.piece.length===e+t){this.deleteNodeTail(s,u),this.validateCRLFWithNextNode(s),this.computeBufferMetadata();return}this.shrinkNode(s,u,g),this.computeBufferMetadata();return}const a=[],l=this.positionInBuffer(s,i.remainder);this.deleteNodeTail(s,l),this._searchCache.validate(e),s.piece.length===0&&a.push(s);const c=this.positionInBuffer(r,n.remainder);this.deleteNodeHead(r,c),r.piece.length===0&&a.push(r);const d=s.next();for(let u=d;u!==He&&u!==r;u=u.next())a.push(u);const h=s.piece.length===0?s.prev():s;this.deleteNodes(a),this.validateCRLFWithNextNode(h),this.computeBufferMetadata()}insertContentToNodeLeft(e,t){const i=[];if(this.shouldCheckCRLF()&&this.endWithCR(e)&&this.startWithLF(t)){const r=t.piece,a={line:r.start.line+1,column:0},l=new qs(r.bufferIndex,a,r.end,this.getLineFeedCnt(r.bufferIndex,a,r.end),r.length-1);t.piece=l,e+=`
`,cc(this,t,-1,-1),t.piece.length===0&&i.push(t)}const n=this.createNewPieces(e);let s=this.rbInsertLeft(t,n[n.length-1]);for(let r=n.length-2;r>=0;r--)s=this.rbInsertLeft(s,n[r]);this.validateCRLFWithPrevNode(s),this.deleteNodes(i)}insertContentToNodeRight(e,t){this.adjustCarriageReturnFromNext(e,t)&&(e+=`
`);const i=this.createNewPieces(e),n=this.rbInsertRight(t,i[0]);let s=n;for(let r=1;r<i.length;r++)s=this.rbInsertRight(s,i[r]);this.validateCRLFWithPrevNode(n)}positionInBuffer(e,t,i){const n=e.piece,s=e.piece.bufferIndex,r=this._buffers[s].lineStarts,l=r[n.start.line]+n.start.column+t;let c=n.start.line,d=n.end.line,h=0,u=0,g=0;for(;c<=d&&(h=c+(d-c)/2|0,g=r[h],h!==d);)if(u=r[h+1],l<g)d=h-1;else if(l>=u)c=h+1;else break;return i?(i.line=h,i.column=l-g,null):{line:h,column:l-g}}getLineFeedCnt(e,t,i){if(i.column===0)return i.line-t.line;const n=this._buffers[e].lineStarts;if(i.line===n.length-1)return i.line-t.line;const s=n[i.line+1],r=n[i.line]+i.column;if(s>r+1)return i.line-t.line;const a=r-1;return this._buffers[e].buffer.charCodeAt(a)===13?i.line-t.line+1:i.line-t.line}offsetInBuffer(e,t){return this._buffers[e].lineStarts[t.line]+t.column}deleteNodes(e){for(let t=0;t<e.length;t++)QS(this,e[t])}createNewPieces(e){if(e.length>sc){const d=[];for(;e.length>sc;){const u=e.charCodeAt(sc-1);let g;u===13||u>=55296&&u<=56319?(g=e.substring(0,sc-1),e=e.substring(sc-1)):(g=e.substring(0,sc),e=e.substring(sc));const f=dc(g);d.push(new qs(this._buffers.length,{line:0,column:0},{line:f.length-1,column:g.length-f[f.length-1]},f.length-1,g.length)),this._buffers.push(new qu(g,f))}const h=dc(e);return d.push(new qs(this._buffers.length,{line:0,column:0},{line:h.length-1,column:e.length-h[h.length-1]},h.length-1,e.length)),this._buffers.push(new qu(e,h)),d}let t=this._buffers[0].buffer.length;const i=dc(e,!1);let n=this._lastChangeBufferPos;if(this._buffers[0].lineStarts[this._buffers[0].lineStarts.length-1]===t&&t!==0&&this.startWithLF(e)&&this.endWithCR(this._buffers[0].buffer)){this._lastChangeBufferPos={line:this._lastChangeBufferPos.line,column:this._lastChangeBufferPos.column+1},n=this._lastChangeBufferPos;for(let d=0;d<i.length;d++)i[d]+=t+1;this._buffers[0].lineStarts=this._buffers[0].lineStarts.concat(i.slice(1)),this._buffers[0].buffer+="_"+e,t+=1}else{if(t!==0)for(let d=0;d<i.length;d++)i[d]+=t;this._buffers[0].lineStarts=this._buffers[0].lineStarts.concat(i.slice(1)),this._buffers[0].buffer+=e}const s=this._buffers[0].buffer.length,r=this._buffers[0].lineStarts.length-1,a=s-this._buffers[0].lineStarts[r],l={line:r,column:a},c=new qs(0,n,l,this.getLineFeedCnt(0,n,l),s-t);return this._lastChangeBufferPos=l,[c]}getLineRawContent(e,t=0){let i=this.root,n="";const s=this._searchCache.get2(e);if(s){i=s.node;const r=this.getAccumulatedValue(i,e-s.nodeStartLineNumber-1),a=this._buffers[i.piece.bufferIndex].buffer,l=this.offsetInBuffer(i.piece.bufferIndex,i.piece.start);if(s.nodeStartLineNumber+i.piece.lineFeedCnt===e)n=a.substring(l+r,l+i.piece.length);else{const c=this.getAccumulatedValue(i,e-s.nodeStartLineNumber);return a.substring(l+r,l+c-t)}}else{let r=0;const a=e;for(;i!==He;)if(i.left!==He&&i.lf_left>=e-1)i=i.left;else if(i.lf_left+i.piece.lineFeedCnt>e-1){const l=this.getAccumulatedValue(i,e-i.lf_left-2),c=this.getAccumulatedValue(i,e-i.lf_left-1),d=this._buffers[i.piece.bufferIndex].buffer,h=this.offsetInBuffer(i.piece.bufferIndex,i.piece.start);return r+=i.size_left,this._searchCache.set({node:i,nodeStartOffset:r,nodeStartLineNumber:a-(e-1-i.lf_left)}),d.substring(h+l,h+c-t)}else if(i.lf_left+i.piece.lineFeedCnt===e-1){const l=this.getAccumulatedValue(i,e-i.lf_left-2),c=this._buffers[i.piece.bufferIndex].buffer,d=this.offsetInBuffer(i.piece.bufferIndex,i.piece.start);n=c.substring(d+l,d+i.piece.length);break}else e-=i.lf_left+i.piece.lineFeedCnt,r+=i.size_left+i.piece.length,i=i.right}for(i=i.next();i!==He;){const r=this._buffers[i.piece.bufferIndex].buffer;if(i.piece.lineFeedCnt>0){const a=this.getAccumulatedValue(i,0),l=this.offsetInBuffer(i.piece.bufferIndex,i.piece.start);return n+=r.substring(l,l+a-t),n}else{const a=this.offsetInBuffer(i.piece.bufferIndex,i.piece.start);n+=r.substr(a,i.piece.length)}i=i.next()}return n}computeBufferMetadata(){let e=this.root,t=1,i=0;for(;e!==He;)t+=e.lf_left+e.piece.lineFeedCnt,i+=e.size_left+e.piece.length,e=e.right;this._lineCnt=t,this._length=i,this._searchCache.validate(this._length)}getIndexOf(e,t){const i=e.piece,n=this.positionInBuffer(e,t),s=n.line-i.start.line;if(this.offsetInBuffer(i.bufferIndex,i.end)-this.offsetInBuffer(i.bufferIndex,i.start)===t){const r=this.getLineFeedCnt(e.piece.bufferIndex,i.start,n);if(r!==s)return{index:r,remainder:0}}return{index:s,remainder:n.column}}getAccumulatedValue(e,t){if(t<0)return 0;const i=e.piece,n=this._buffers[i.bufferIndex].lineStarts,s=i.start.line+t+1;return s>i.end.line?n[i.end.line]+i.end.column-n[i.start.line]-i.start.column:n[s]-n[i.start.line]-i.start.column}deleteNodeTail(e,t){const i=e.piece,n=i.lineFeedCnt,s=this.offsetInBuffer(i.bufferIndex,i.end),r=t,a=this.offsetInBuffer(i.bufferIndex,r),l=this.getLineFeedCnt(i.bufferIndex,i.start,r),c=l-n,d=a-s,h=i.length+d;e.piece=new qs(i.bufferIndex,i.start,r,l,h),cc(this,e,d,c)}deleteNodeHead(e,t){const i=e.piece,n=i.lineFeedCnt,s=this.offsetInBuffer(i.bufferIndex,i.start),r=t,a=this.getLineFeedCnt(i.bufferIndex,r,i.end),l=this.offsetInBuffer(i.bufferIndex,r),c=a-n,d=s-l,h=i.length+d;e.piece=new qs(i.bufferIndex,r,i.end,a,h),cc(this,e,d,c)}shrinkNode(e,t,i){const n=e.piece,s=n.start,r=n.end,a=n.length,l=n.lineFeedCnt,c=t,d=this.getLineFeedCnt(n.bufferIndex,n.start,c),h=this.offsetInBuffer(n.bufferIndex,t)-this.offsetInBuffer(n.bufferIndex,s);e.piece=new qs(n.bufferIndex,n.start,c,d,h),cc(this,e,h-a,d-l);const u=new qs(n.bufferIndex,i,r,this.getLineFeedCnt(n.bufferIndex,i,r),this.offsetInBuffer(n.bufferIndex,r)-this.offsetInBuffer(n.bufferIndex,i)),g=this.rbInsertRight(e,u);this.validateCRLFWithPrevNode(g)}appendToNode(e,t){this.adjustCarriageReturnFromNext(t,e)&&(t+=`
`);const i=this.shouldCheckCRLF()&&this.startWithLF(t)&&this.endWithCR(e),n=this._buffers[0].buffer.length;this._buffers[0].buffer+=t;const s=dc(t,!1);for(let g=0;g<s.length;g++)s[g]+=n;if(i){const g=this._buffers[0].lineStarts[this._buffers[0].lineStarts.length-2];this._buffers[0].lineStarts.pop(),this._lastChangeBufferPos={line:this._lastChangeBufferPos.line-1,column:n-g}}this._buffers[0].lineStarts=this._buffers[0].lineStarts.concat(s.slice(1));const r=this._buffers[0].lineStarts.length-1,a=this._buffers[0].buffer.length-this._buffers[0].lineStarts[r],l={line:r,column:a},c=e.piece.length+t.length,d=e.piece.lineFeedCnt,h=this.getLineFeedCnt(0,e.piece.start,l),u=h-d;e.piece=new qs(e.piece.bufferIndex,e.piece.start,l,h,c),this._lastChangeBufferPos=l,cc(this,e,t.length,u)}nodeAt(e){let t=this.root;const i=this._searchCache.get(e);if(i)return{node:i.node,nodeStartOffset:i.nodeStartOffset,remainder:e-i.nodeStartOffset};let n=0;for(;t!==He;)if(t.size_left>e)t=t.left;else if(t.size_left+t.piece.length>=e){n+=t.size_left;const s={node:t,remainder:e-t.size_left,nodeStartOffset:n};return this._searchCache.set(s),s}else e-=t.size_left+t.piece.length,n+=t.size_left+t.piece.length,t=t.right;return null}nodeAt2(e,t){let i=this.root,n=0;for(;i!==He;)if(i.left!==He&&i.lf_left>=e-1)i=i.left;else if(i.lf_left+i.piece.lineFeedCnt>e-1){const s=this.getAccumulatedValue(i,e-i.lf_left-2),r=this.getAccumulatedValue(i,e-i.lf_left-1);return n+=i.size_left,{node:i,remainder:Math.min(s+t-1,r),nodeStartOffset:n}}else if(i.lf_left+i.piece.lineFeedCnt===e-1){const s=this.getAccumulatedValue(i,e-i.lf_left-2);if(s+t-1<=i.piece.length)return{node:i,remainder:s+t-1,nodeStartOffset:n};t-=i.piece.length-s;break}else e-=i.lf_left+i.piece.lineFeedCnt,n+=i.size_left+i.piece.length,i=i.right;for(i=i.next();i!==He;){if(i.piece.lineFeedCnt>0){const s=this.getAccumulatedValue(i,0),r=this.offsetOfNode(i);return{node:i,remainder:Math.min(t-1,s),nodeStartOffset:r}}else if(i.piece.length>=t-1){const s=this.offsetOfNode(i);return{node:i,remainder:t-1,nodeStartOffset:s}}else t-=i.piece.length;i=i.next()}return null}nodeCharCodeAt(e,t){if(e.piece.lineFeedCnt<1)return-1;const i=this._buffers[e.piece.bufferIndex],n=this.offsetInBuffer(e.piece.bufferIndex,e.piece.start)+t;return i.buffer.charCodeAt(n)}offsetOfNode(e){if(!e)return 0;let t=e.size_left;for(;e!==this.root;)e.parent.right===e&&(t+=e.parent.size_left+e.parent.piece.length),e=e.parent;return t}shouldCheckCRLF(){return!(this._EOLNormalized&&this._EOL===`
`)}startWithLF(e){if(typeof e=="string")return e.charCodeAt(0)===10;if(e===He||e.piece.lineFeedCnt===0)return!1;const t=e.piece,i=this._buffers[t.bufferIndex].lineStarts,n=t.start.line,s=i[n]+t.start.column;return n===i.length-1||i[n+1]>s+1?!1:this._buffers[t.bufferIndex].buffer.charCodeAt(s)===10}endWithCR(e){return typeof e=="string"?e.charCodeAt(e.length-1)===13:e===He||e.piece.lineFeedCnt===0?!1:this.nodeCharCodeAt(e,e.piece.length-1)===13}validateCRLFWithPrevNode(e){if(this.shouldCheckCRLF()&&this.startWithLF(e)){const t=e.prev();this.endWithCR(t)&&this.fixCRLF(t,e)}}validateCRLFWithNextNode(e){if(this.shouldCheckCRLF()&&this.endWithCR(e)){const t=e.next();this.startWithLF(t)&&this.fixCRLF(e,t)}}fixCRLF(e,t){const i=[],n=this._buffers[e.piece.bufferIndex].lineStarts;let s;e.piece.end.column===0?s={line:e.piece.end.line-1,column:n[e.piece.end.line]-n[e.piece.end.line-1]-1}:s={line:e.piece.end.line,column:e.piece.end.column-1};const r=e.piece.length-1,a=e.piece.lineFeedCnt-1;e.piece=new qs(e.piece.bufferIndex,e.piece.start,s,a,r),cc(this,e,-1,-1),e.piece.length===0&&i.push(e);const l={line:t.piece.start.line+1,column:0},c=t.piece.length-1,d=this.getLineFeedCnt(t.piece.bufferIndex,l,t.piece.end);t.piece=new qs(t.piece.bufferIndex,l,t.piece.end,d,c),cc(this,t,-1,-1),t.piece.length===0&&i.push(t);const h=this.createNewPieces(`\r
`);this.rbInsertRight(e,h[0]);for(let u=0;u<i.length;u++)QS(this,i[u])}adjustCarriageReturnFromNext(e,t){if(this.shouldCheckCRLF()&&this.endWithCR(e)){const i=t.next();if(this.startWithLF(i)){if(e+=`
`,i.piece.length===1)QS(this,i);else{const n=i.piece,s={line:n.start.line+1,column:0},r=n.length-1,a=this.getLineFeedCnt(n.bufferIndex,s,n.end);i.piece=new qs(n.bufferIndex,s,n.end,a,r),cc(this,i,-1,-1)}return!0}}return!1}iterate(e,t){if(e===He)return t(He);const i=this.iterate(e.left,t);return i&&t(e)&&this.iterate(e.right,t)}getNodeContent(e){if(e===He)return"";const t=this._buffers[e.piece.bufferIndex],i=e.piece,n=this.offsetInBuffer(i.bufferIndex,i.start),s=this.offsetInBuffer(i.bufferIndex,i.end);return t.buffer.substring(n,s)}getPieceContent(e){const t=this._buffers[e.bufferIndex],i=this.offsetInBuffer(e.bufferIndex,e.start),n=this.offsetInBuffer(e.bufferIndex,e.end);return t.buffer.substring(i,n)}rbInsertRight(e,t){const i=new cA(t,1);if(i.left=He,i.right=He,i.parent=He,i.size_left=0,i.lf_left=0,this.root===He)this.root=i,i.color=0;else if(e.right===He)e.right=i,i.parent=e;else{const s=g3(e.right);s.left=i,i.parent=s}return D8(this,i),i}rbInsertLeft(e,t){const i=new cA(t,1);if(i.left=He,i.right=He,i.parent=He,i.size_left=0,i.lf_left=0,this.root===He)this.root=i,i.color=0;else if(e.left===He)e.left=i,i.parent=e;else{const n=dK(e.left);n.right=i,i.parent=n}return D8(this,i),i}}class Jp extends F{constructor(e,t,i,n,s,r,a){super(),this._onDidChangeContent=this._register(new P),this._BOM=t,this._mightContainNonBasicASCII=!r,this._mightContainRTL=n,this._mightContainUnusualLineTerminators=s,this._pieceTree=new Oue(e,i,a)}mightContainRTL(){return this._mightContainRTL}mightContainUnusualLineTerminators(){return this._mightContainUnusualLineTerminators}resetMightContainUnusualLineTerminators(){this._mightContainUnusualLineTerminators=!1}mightContainNonBasicASCII(){return this._mightContainNonBasicASCII}getBOM(){return this._BOM}getEOL(){return this._pieceTree.getEOL()}createSnapshot(e){return this._pieceTree.createSnapshot(e?this._BOM:"")}getOffsetAt(e,t){return this._pieceTree.getOffsetAt(e,t)}getPositionAt(e){return this._pieceTree.getPositionAt(e)}getRangeAt(e,t){const i=e+t,n=this.getPositionAt(e),s=this.getPositionAt(i);return new x(n.lineNumber,n.column,s.lineNumber,s.column)}getValueInRange(e,t=0){if(e.isEmpty())return"";const i=this._getEndOfLine(t);return this._pieceTree.getValueInRange(e,i)}getValueLengthInRange(e,t=0){if(e.isEmpty())return 0;if(e.startLineNumber===e.endLineNumber)return e.endColumn-e.startColumn;const i=this.getOffsetAt(e.startLineNumber,e.startColumn),n=this.getOffsetAt(e.endLineNumber,e.endColumn);let s=0;const r=this._getEndOfLine(t),a=this.getEOL();if(r.length!==a.length){const l=r.length-a.length,c=e.endLineNumber-e.startLineNumber;s=l*c}return n-i+s}getCharacterCountInRange(e,t=0){if(this._mightContainNonBasicASCII){let i=0;const n=e.startLineNumber,s=e.endLineNumber;for(let r=n;r<=s;r++){const a=this.getLineContent(r),l=r===n?e.startColumn-1:0,c=r===s?e.endColumn-1:a.length;for(let d=l;d<c;d++)Gi(a.charCodeAt(d))?(i=i+1,d=d+1):i=i+1}return i+=this._getEndOfLine(t).length*(s-n),i}return this.getValueLengthInRange(e,t)}getLength(){return this._pieceTree.getLength()}getLineCount(){return this._pieceTree.getLineCount()}getLinesContent(){return this._pieceTree.getLinesContent()}getLineContent(e){return this._pieceTree.getLineContent(e)}getLineCharCode(e,t){return this._pieceTree.getLineCharCode(e,t)}getLineLength(e){return this._pieceTree.getLineLength(e)}getLineFirstNonWhitespaceColumn(e){const t=Jn(this.getLineContent(e));return t===-1?0:t+1}getLineLastNonWhitespaceColumn(e){const t=Na(this.getLineContent(e));return t===-1?0:t+2}_getEndOfLine(e){switch(e){case 1:return`
`;case 2:return`\r
`;case 0:return this.getEOL();default:throw new Error("Unknown EOL preference")}}setEOL(e){this._pieceTree.setEOL(e)}applyEdits(e,t,i){let n=this._mightContainRTL,s=this._mightContainUnusualLineTerminators,r=this._mightContainNonBasicASCII,a=!0,l=[];for(let p=0;p<e.length;p++){const _=e[p];a&&_._isTracked&&(a=!1);const b=_.range;if(_.text){let L=!0;r||(L=!P1(_.text),r=L),!n&&L&&(n=k_(_.text)),!s&&L&&(s=sU(_.text))}let C="",v=0,w=0,S=0;if(_.text){let L;[v,w,S,L]=zh(_.text);const k=this.getEOL();L===0||L===(k===`\r
`?2:1)?C=_.text:C=_.text.replace(/\r\n|\r|\n/g,k)}l[p]={sortIndex:p,identifier:_.identifier||null,range:b,rangeOffset:this.getOffsetAt(b.startLineNumber,b.startColumn),rangeLength:this.getValueLengthInRange(b),text:C,eolCount:v,firstLineLength:w,lastLineLength:S,forceMoveMarkers:!!_.forceMoveMarkers,isAutoWhitespaceEdit:_.isAutoWhitespaceEdit||!1}}l.sort(Jp._sortOpsAscending);let c=!1;for(let p=0,_=l.length-1;p<_;p++){const b=l[p].range.getEndPosition(),C=l[p+1].range.getStartPosition();if(C.isBeforeOrEqual(b)){if(C.isBefore(b))throw new Error("Overlapping ranges are not allowed!");c=!0}}a&&(l=this._reduceOperations(l));const d=i||t?Jp._getInverseEditRanges(l):[],h=[];if(t)for(let p=0;p<l.length;p++){const _=l[p],b=d[p];if(_.isAutoWhitespaceEdit&&_.range.isEmpty())for(let C=b.startLineNumber;C<=b.endLineNumber;C++){let v="";C===b.startLineNumber&&(v=this.getLineContent(_.range.startLineNumber),Jn(v)!==-1)||h.push({lineNumber:C,oldContent:v})}}let u=null;if(i){let p=0;u=[];for(let _=0;_<l.length;_++){const b=l[_],C=d[_],v=this.getValueInRange(b.range),w=b.rangeOffset+p;p+=b.text.length-v.length,u[_]={sortIndex:b.sortIndex,identifier:b.identifier,range:C,text:v,textChange:new Ln(b.rangeOffset,v,w,b.text)}}c||u.sort((_,b)=>_.sortIndex-b.sortIndex)}this._mightContainRTL=n,this._mightContainUnusualLineTerminators=s,this._mightContainNonBasicASCII=r;const g=this._doApplyEdits(l);let f=null;if(t&&h.length>0){h.sort((p,_)=>_.lineNumber-p.lineNumber),f=[];for(let p=0,_=h.length;p<_;p++){const b=h[p].lineNumber;if(p>0&&h[p-1].lineNumber===b)continue;const C=h[p].oldContent,v=this.getLineContent(b);v.length===0||v===C||Jn(v)!==-1||f.push(b)}}return this._onDidChangeContent.fire(),new qne(u,g,f)}_reduceOperations(e){return e.length<1e3?e:[this._toSingleEditOperation(e)]}_toSingleEditOperation(e){let t=!1;const i=e[0].range,n=e[e.length-1].range,s=new x(i.startLineNumber,i.startColumn,n.endLineNumber,n.endColumn);let r=i.startLineNumber,a=i.startColumn;const l=[];for(let g=0,f=e.length;g<f;g++){const p=e[g],_=p.range;t=t||p.forceMoveMarkers,l.push(this.getValueInRange(new x(r,a,_.startLineNumber,_.startColumn))),p.text.length>0&&l.push(p.text),r=_.endLineNumber,a=_.endColumn}const c=l.join(""),[d,h,u]=zh(c);return{sortIndex:0,identifier:e[0].identifier,range:s,rangeOffset:this.getOffsetAt(s.startLineNumber,s.startColumn),rangeLength:this.getValueLengthInRange(s,0),text:c,eolCount:d,firstLineLength:h,lastLineLength:u,forceMoveMarkers:t,isAutoWhitespaceEdit:!1}}_doApplyEdits(e){e.sort(Jp._sortOpsDescending);const t=[];for(let i=0;i<e.length;i++){const n=e[i],s=n.range.startLineNumber,r=n.range.startColumn,a=n.range.endLineNumber,l=n.range.endColumn;if(s===a&&r===l&&n.text.length===0)continue;n.text?(this._pieceTree.delete(n.rangeOffset,n.rangeLength),this._pieceTree.insert(n.rangeOffset,n.text,!0)):this._pieceTree.delete(n.rangeOffset,n.rangeLength);const c=new x(s,r,a,l);t.push({range:c,rangeLength:n.rangeLength,text:n.text,rangeOffset:n.rangeOffset,forceMoveMarkers:n.forceMoveMarkers})}return t}findMatchesLineByLine(e,t,i,n){return this._pieceTree.findMatchesLineByLine(e,t,i,n)}static _getInverseEditRanges(e){const t=[];let i=0,n=0,s=null;for(let r=0,a=e.length;r<a;r++){const l=e[r];let c,d;s?s.range.endLineNumber===l.range.startLineNumber?(c=i,d=n+(l.range.startColumn-s.range.endColumn)):(c=i+(l.range.startLineNumber-s.range.endLineNumber),d=l.range.startColumn):(c=l.range.startLineNumber,d=l.range.startColumn);let h;if(l.text.length>0){const u=l.eolCount+1;u===1?h=new x(c,d,c,d+l.firstLineLength):h=new x(c,d,c+u-1,l.lastLineLength+1)}else h=new x(c,d,c,d);i=h.endLineNumber,n=h.endColumn,t.push(h),s=l}return t}static _sortOpsAscending(e,t){const i=x.compareRangesUsingEnds(e.range,t.range);return i===0?e.sortIndex-t.sortIndex:i}static _sortOpsDescending(e,t){const i=x.compareRangesUsingEnds(e.range,t.range);return i===0?t.sortIndex-e.sortIndex:-i}}class Fue{constructor(e,t,i,n,s,r,a,l,c){this._chunks=e,this._bom=t,this._cr=i,this._lf=n,this._crlf=s,this._containsRTL=r,this._containsUnusualLineTerminators=a,this._isBasicASCII=l,this._normalizeEOL=c}_getEOL(e){const t=this._cr+this._lf+this._crlf,i=this._cr+this._crlf;return t===0?e===1?`
`:`\r
`:i>t/2?`\r
`:`
`}create(e){const t=this._getEOL(e),i=this._chunks;if(this._normalizeEOL&&(t===`\r
`&&(this._cr>0||this._lf>0)||t===`
`&&(this._cr>0||this._crlf>0)))for(let s=0,r=i.length;s<r;s++){const a=i[s].buffer.replace(/\r\n|\r|\n/g,t),l=dc(a);i[s]=new qu(a,l)}const n=new Jp(i,this._bom,t,this._containsRTL,this._containsUnusualLineTerminators,this._isBasicASCII,this._normalizeEOL);return{textBuffer:n,disposable:n}}}class uK{constructor(){this.chunks=[],this.BOM="",this._hasPreviousChar=!1,this._previousChar=0,this._tmpLineStarts=[],this.cr=0,this.lf=0,this.crlf=0,this.containsRTL=!1,this.containsUnusualLineTerminators=!1,this.isBasicASCII=!0}acceptChunk(e){if(e.length===0)return;this.chunks.length===0&&t5(e)&&(this.BOM=Wee,e=e.substr(1));const t=e.charCodeAt(e.length-1);t===13||t>=55296&&t<=56319?(this._acceptChunk1(e.substr(0,e.length-1),!1),this._hasPreviousChar=!0,this._previousChar=t):(this._acceptChunk1(e,!1),this._hasPreviousChar=!1,this._previousChar=t)}_acceptChunk1(e,t){!t&&e.length===0||(this._hasPreviousChar?this._acceptChunk2(String.fromCharCode(this._previousChar)+e):this._acceptChunk2(e))}_acceptChunk2(e){const t=Mue(this._tmpLineStarts,e);this.chunks.push(new qu(e,t.lineStarts)),this.cr+=t.cr,this.lf+=t.lf,this.crlf+=t.crlf,t.isBasicASCII||(this.isBasicASCII=!1,this.containsRTL||(this.containsRTL=k_(e)),this.containsUnusualLineTerminators||(this.containsUnusualLineTerminators=sU(e)))}finish(e=!0){return this._finish(),new Fue(this.chunks,this.BOM,this.cr,this.lf,this.crlf,this.containsRTL,this.containsUnusualLineTerminators,this.isBasicASCII,e)}_finish(){if(this.chunks.length===0&&this._acceptChunk1("",!0),this._hasPreviousChar){this._hasPreviousChar=!1;const e=this.chunks[this.chunks.length-1];e.buffer+=String.fromCharCode(this._previousChar);const t=dc(e.buffer);e.lineStarts=t,this._previousChar===13&&this.cr++}}}class Wue{constructor(e){this._default=e,this._store=[]}get(e){return e<this._store.length?this._store[e]:this._default}set(e,t){for(;e>=this._store.length;)this._store[this._store.length]=this._default;this._store[e]=t}replace(e,t,i){if(e>=this._store.length)return;if(t===0){this.insert(e,i);return}else if(i===0){this.delete(e,t);return}const n=this._store.slice(0,e),s=this._store.slice(e+t),r=Bue(i,this._default);this._store=n.concat(r,s)}delete(e,t){t===0||e>=this._store.length||this._store.splice(e,t)}insert(e,t){if(t===0||e>=this._store.length)return;const i=[];for(let n=0;n<t;n++)i[n]=this._default;this._store=cI(this._store,e,i)}}function Bue(o,e){const t=[];for(let i=0;i<o;i++)t[i]=e;return t}class Hue{get startLineNumber(){return this._startLineNumber}get endLineNumber(){return this._startLineNumber+this._tokens.length-1}constructor(e,t){this._startLineNumber=e,this._tokens=t}getLineTokens(e){return this._tokens[e-this._startLineNumber]}appendLineTokens(e){this._tokens.push(e)}}class dA{constructor(){this._tokens=[]}add(e,t){if(this._tokens.length>0){const i=this._tokens[this._tokens.length-1];if(i.endLineNumber+1===e){i.appendLineTokens(t);return}}this._tokens.push(new Hue(e,[t]))}finalize(){return this._tokens}}class Vue{constructor(e,t){this.tokenizationSupport=t,this.initialState=this.tokenizationSupport.getInitialState(),this.store=new hA(e)}getStartState(e){return this.store.getStartState(e,this.initialState)}getFirstInvalidLine(){return this.store.getFirstInvalidLine(this.initialState)}}class zue extends Vue{constructor(e,t,i,n){super(e,t),this._textModel=i,this._languageIdCodec=n}updateTokensUntilLine(e,t){const i=this._textModel.getLanguageId();for(;;){const n=this.getFirstInvalidLine();if(!n||n.lineNumber>t)break;const s=this._textModel.getLineContent(n.lineNumber),r=sC(this._languageIdCodec,i,this.tokenizationSupport,s,!0,n.startState);e.add(n.lineNumber,r.tokens),this.store.setEndState(n.lineNumber,r.endState)}}getTokenTypeIfInsertingCharacter(e,t){const i=this.getStartState(e.lineNumber);if(!i)return 0;const n=this._textModel.getLanguageId(),s=this._textModel.getLineContent(e.lineNumber),r=s.substring(0,e.column-1)+t+s.substring(e.column-1),a=sC(this._languageIdCodec,n,this.tokenizationSupport,r,!0,i),l=new cn(a.tokens,r,this._languageIdCodec);if(l.getCount()===0)return 0;const c=l.findTokenIndexAtOffset(e.column-1);return l.getStandardTokenType(c)}tokenizeLineWithEdit(e,t,i){const n=e.lineNumber,s=e.column,r=this.getStartState(n);if(!r)return null;const a=this._textModel.getLineContent(n),l=a.substring(0,s-1)+i+a.substring(s-1+t),c=this._textModel.getLanguageIdAtPosition(n,0),d=sC(this._languageIdCodec,c,this.tokenizationSupport,l,!0,r);return new cn(d.tokens,l,this._languageIdCodec)}hasAccurateTokensForLine(e){const t=this.store.getFirstInvalidEndStateLineNumberOrMax();return e<t}isCheapToTokenize(e){const t=this.store.getFirstInvalidEndStateLineNumberOrMax();return e<t||e===t&&this._textModel.getLineLength(e)<2048}tokenizeHeuristically(e,t,i){if(i<=this.store.getFirstInvalidEndStateLineNumberOrMax())return{heuristicTokens:!1};if(t<=this.store.getFirstInvalidEndStateLineNumberOrMax())return this.updateTokensUntilLine(e,i),{heuristicTokens:!1};let n=this.guessStartState(t);const s=this._textModel.getLanguageId();for(let r=t;r<=i;r++){const a=this._textModel.getLineContent(r),l=sC(this._languageIdCodec,s,this.tokenizationSupport,a,!0,n);e.add(r,l.tokens),n=l.endState}return{heuristicTokens:!0}}guessStartState(e){let t=this._textModel.getLineFirstNonWhitespaceColumn(e);const i=[];let n=null;for(let a=e-1;t>1&&a>=1;a--){const l=this._textModel.getLineFirstNonWhitespaceColumn(a);if(l!==0&&l<t&&(i.push(this._textModel.getLineContent(a)),t=l,n=this.getStartState(a),n))break}n||(n=this.tokenizationSupport.getInitialState()),i.reverse();const s=this._textModel.getLanguageId();let r=n;for(const a of i)r=sC(this._languageIdCodec,s,this.tokenizationSupport,a,!1,r).endState;return r}}class hA{constructor(e){this.lineCount=e,this._tokenizationStateStore=new Uue,this._invalidEndStatesLineNumbers=new $ue,this._invalidEndStatesLineNumbers.addRange(new Ge(1,e+1))}getEndState(e){return this._tokenizationStateStore.getEndState(e)}setEndState(e,t){if(!t)throw new bt("Cannot set null/undefined state");this._invalidEndStatesLineNumbers.delete(e);const i=this._tokenizationStateStore.setEndState(e,t);return i&&e<this.lineCount&&this._invalidEndStatesLineNumbers.addRange(new Ge(e+1,e+2)),i}acceptChange(e,t){this.lineCount+=t-e.length,this._tokenizationStateStore.acceptChange(e,t),this._invalidEndStatesLineNumbers.addRangeAndResize(new Ge(e.startLineNumber,e.endLineNumberExclusive),t)}acceptChanges(e){for(const t of e){const[i]=zh(t.text);this.acceptChange(new Be(t.range.startLineNumber,t.range.endLineNumber+1),i+1)}}invalidateEndStateRange(e){this._invalidEndStatesLineNumbers.addRange(new Ge(e.startLineNumber,e.endLineNumberExclusive))}getFirstInvalidEndStateLineNumber(){return this._invalidEndStatesLineNumbers.min}getFirstInvalidEndStateLineNumberOrMax(){return this.getFirstInvalidEndStateLineNumber()||Number.MAX_SAFE_INTEGER}allStatesValid(){return this._invalidEndStatesLineNumbers.min===null}getStartState(e,t){return e===1?t:this.getEndState(e-1)}getFirstInvalidLine(e){const t=this.getFirstInvalidEndStateLineNumber();if(t===null)return null;const i=this.getStartState(t,e);if(!i)throw new bt("Start state must be defined");return{lineNumber:t,startState:i}}}class Uue{constructor(){this._lineEndStates=new Wue(null)}getEndState(e){return this._lineEndStates.get(e)}setEndState(e,t){const i=this._lineEndStates.get(e);return i&&i.equals(t)?!1:(this._lineEndStates.set(e,t),!0)}acceptChange(e,t){let i=e.length;t>0&&i>0&&(i--,t--),this._lineEndStates.replace(e.startLineNumber,i,t)}}class $ue{constructor(){this._ranges=[]}get min(){return this._ranges.length===0?null:this._ranges[0].start}delete(e){const t=this._ranges.findIndex(i=>i.contains(e));if(t!==-1){const i=this._ranges[t];i.start===e?i.endExclusive===e+1?this._ranges.splice(t,1):this._ranges[t]=new Ge(e+1,i.endExclusive):i.endExclusive===e+1?this._ranges[t]=new Ge(i.start,e):this._ranges.splice(t,1,new Ge(i.start,e),new Ge(e+1,i.endExclusive))}}addRange(e){Ge.addRange(e,this._ranges)}addRangeAndResize(e,t){let i=0;for(;!(i>=this._ranges.length||e.start<=this._ranges[i].endExclusive);)i++;let n=i;for(;!(n>=this._ranges.length||e.endExclusive<this._ranges[n].start);)n++;const s=t-e.length;for(let r=n;r<this._ranges.length;r++)this._ranges[r]=this._ranges[r].delta(s);if(i===n){const r=new Ge(e.start,e.start+t);r.isEmpty||this._ranges.splice(i,0,r)}else{const r=Math.min(e.start,this._ranges[i].start),a=Math.max(e.endExclusive,this._ranges[n-1].endExclusive),l=new Ge(r,a+s);l.isEmpty?this._ranges.splice(i,n-i):this._ranges.splice(i,n-i,l)}}toString(){return this._ranges.map(e=>e.toString()).join(" + ")}}function sC(o,e,t,i,n,s){let r=null;if(t)try{r=t.tokenizeEncoded(i,n,s.clone())}catch(a){Oe(a)}return r||(r=XI(o.encodeLanguageId(e),s)),cn.convertToEndOffset(r.tokens,i.length),r}class jue{constructor(e,t){this._tokenizerWithStateStore=e,this._backgroundTokenStore=t,this._isDisposed=!1,this._isScheduled=!1}dispose(){this._isDisposed=!0}handleChanges(){this._beginBackgroundTokenization()}_beginBackgroundTokenization(){this._isScheduled||!this._tokenizerWithStateStore._textModel.isAttachedToEditor()||!this._hasLinesToTokenize()||(this._isScheduled=!0,hU(e=>{this._isScheduled=!1,this._backgroundTokenizeWithDeadline(e)}))}_backgroundTokenizeWithDeadline(e){const t=Date.now()+e.timeRemaining(),i=()=>{this._isDisposed||!this._tokenizerWithStateStore._textModel.isAttachedToEditor()||!this._hasLinesToTokenize()||(this._backgroundTokenizeForAtLeast1ms(),Date.now()<t?Nz(i):this._beginBackgroundTokenization())};i()}_backgroundTokenizeForAtLeast1ms(){const e=this._tokenizerWithStateStore._textModel.getLineCount(),t=new dA,i=ln.create(!1);do if(i.elapsed()>1||this._tokenizeOneInvalidLine(t)>=e)break;while(this._hasLinesToTokenize());this._backgroundTokenStore.setTokens(t.finalize()),this.checkFinished()}_hasLinesToTokenize(){return this._tokenizerWithStateStore?!this._tokenizerWithStateStore.store.allStatesValid():!1}_tokenizeOneInvalidLine(e){var i;const t=(i=this._tokenizerWithStateStore)==null?void 0:i.getFirstInvalidLine();return t?(this._tokenizerWithStateStore.updateTokensUntilLine(e,t.lineNumber),t.lineNumber):this._tokenizerWithStateStore._textModel.getLineCount()+1}checkFinished(){this._isDisposed||this._tokenizerWithStateStore.store.allStatesValid()&&this._backgroundTokenStore.backgroundTokenizationFinished()}requestTokens(e,t){this._tokenizerWithStateStore.store.invalidateEndStateRange(new Be(e,t))}}class Kue{constructor(){this._onDidChangeVisibleRanges=new P,this.onDidChangeVisibleRanges=this._onDidChangeVisibleRanges.event,this._views=new Set}attachView(){const e=new que(t=>{this._onDidChangeVisibleRanges.fire({view:e,state:t})});return this._views.add(e),e}detachView(e){this._views.delete(e),this._onDidChangeVisibleRanges.fire({view:e,state:void 0})}}class que{constructor(e){this.handleStateChange=e}setVisibleLines(e,t){const i=e.map(n=>new Be(n.startLineNumber,n.endLineNumber+1));this.handleStateChange({visibleLineRanges:i,stabilized:t})}}class Gue extends F{get lineRanges(){return this._lineRanges}constructor(e){super(),this._refreshTokens=e,this.runner=this._register(new It(()=>this.update(),50)),this._computedLineRanges=[],this._lineRanges=[]}update(){ii(this._computedLineRanges,this._lineRanges,(e,t)=>e.equals(t))||(this._computedLineRanges=this._lineRanges,this._refreshTokens())}handleStateChange(e){this._lineRanges=e.visibleLineRanges,e.stabilized?(this.runner.cancel(),this.update()):this.runner.schedule()}}class gK extends F{get backgroundTokenizationState(){return this._backgroundTokenizationState}constructor(e,t,i){super(),this._languageIdCodec=e,this._textModel=t,this.getLanguageId=i,this._backgroundTokenizationState=1,this._onDidChangeBackgroundTokenizationState=this._register(new P),this.onDidChangeBackgroundTokenizationState=this._onDidChangeBackgroundTokenizationState.event,this._onDidChangeTokens=this._register(new P),this.onDidChangeTokens=this._onDidChangeTokens.event}tokenizeIfCheap(e){this.isCheapToTokenize(e)&&this.forceTokenization(e)}}class I8 extends gK{constructor(e,t,i,n){super(t,i,n),this._treeSitterService=e,this._tokenizationSupport=null,this._initialize()}_initialize(){const e=this.getLanguageId();(!this._tokenizationSupport||this._lastLanguageId!==e)&&(this._lastLanguageId=e,this._tokenizationSupport=kR.get(e))}getLineTokens(e){const t=this._textModel.getLineContent(e);if(this._tokenizationSupport){const i=this._tokenizationSupport.tokenizeEncoded(e,this._textModel);if(i)return new cn(i,t,this._languageIdCodec)}return cn.createEmpty(t,this._languageIdCodec)}resetTokenization(e=!0){e&&this._onDidChangeTokens.fire({semanticTokensApplied:!1,ranges:[{fromLineNumber:1,toLineNumber:this._textModel.getLineCount()}]}),this._initialize()}handleDidChangeAttached(){}handleDidChangeContent(e){e.isFlush&&this.resetTokenization(!1)}forceTokenization(e){}hasAccurateTokensForLine(e){return!0}isCheapToTokenize(e){return!0}getTokenTypeIfInsertingCharacter(e,t,i){return 0}tokenizeLineWithEdit(e,t,i){return null}get hasTokens(){return this._treeSitterService.getParseResult(this._textModel)!==void 0}}const fK=et("treeSitterParserService"),hc=new Uint32Array(0).buffer;class pl{static deleteBeginning(e,t){return e===null||e===hc?e:pl.delete(e,0,t)}static deleteEnding(e,t){if(e===null||e===hc)return e;const i=Dc(e),n=i[i.length-2];return pl.delete(e,t,n)}static delete(e,t,i){if(e===null||e===hc||t===i)return e;const n=Dc(e),s=n.length>>>1;if(t===0&&n[n.length-2]===i)return hc;const r=cn.findIndexInTokensArray(n,t),a=r>0?n[r-1<<1]:0,l=n[r<<1];if(i<l){const g=i-t;for(let f=r;f<s;f++)n[f<<1]-=g;return e}let c,d;a!==t?(n[r<<1]=t,c=r+1<<1,d=t):(c=r<<1,d=a);const h=i-t;for(let g=r+1;g<s;g++){const f=n[g<<1]-h;f>d&&(n[c++]=f,n[c++]=n[(g<<1)+1],d=f)}if(c===n.length)return e;const u=new Uint32Array(c);return u.set(n.subarray(0,c),0),u.buffer}static append(e,t){if(t===hc)return e;if(e===hc)return t;if(e===null)return e;if(t===null)return null;const i=Dc(e),n=Dc(t),s=n.length>>>1,r=new Uint32Array(i.length+n.length);r.set(i,0);let a=i.length;const l=i[i.length-2];for(let c=0;c<s;c++)r[a++]=n[c<<1]+l,r[a++]=n[(c<<1)+1];return r.buffer}static insert(e,t,i){if(e===null||e===hc)return e;const n=Dc(e),s=n.length>>>1;let r=cn.findIndexInTokensArray(n,t);r>0&&n[r-1<<1]===t&&r--;for(let a=r;a<s;a++)n[a<<1]+=i;return e}}function Dc(o){return o instanceof Uint32Array?o:new Uint32Array(o)}class Aw{constructor(e){this._lineTokens=[],this._len=0,this._languageIdCodec=e}flush(){this._lineTokens=[],this._len=0}get hasTokens(){return this._lineTokens.length>0}getTokens(e,t,i){let n=null;if(t<this._len&&(n=this._lineTokens[t]),n!==null&&n!==hc)return new cn(Dc(n),i,this._languageIdCodec);const s=new Uint32Array(2);return s[0]=i.length,s[1]=E8(this._languageIdCodec.encodeLanguageId(e)),new cn(s,i,this._languageIdCodec)}static _massageTokens(e,t,i){const n=i?Dc(i):null;if(t===0){let s=!1;if(n&&n.length>1&&(s=Ts.getLanguageId(n[1])!==e),!s)return hc}if(!n||n.length===0){const s=new Uint32Array(2);return s[0]=t,s[1]=E8(e),s.buffer}return n[n.length-2]=t,n.byteOffset===0&&n.byteLength===n.buffer.byteLength?n.buffer:n}_ensureLine(e){for(;e>=this._len;)this._lineTokens[this._len]=null,this._len++}_deleteLines(e,t){t!==0&&(e+t>this._len&&(t=this._len-e),this._lineTokens.splice(e,t),this._len-=t)}_insertLines(e,t){if(t===0)return;const i=[];for(let n=0;n<t;n++)i[n]=null;this._lineTokens=cI(this._lineTokens,e,i),this._len+=t}setTokens(e,t,i,n,s){const r=Aw._massageTokens(this._languageIdCodec.encodeLanguageId(e),i,n);this._ensureLine(t);const a=this._lineTokens[t];return this._lineTokens[t]=r,s?!Aw._equals(a,r):!1}static _equals(e,t){if(!e||!t)return!e&&!t;const i=Dc(e),n=Dc(t);if(i.length!==n.length)return!1;for(let s=0,r=i.length;s<r;s++)if(i[s]!==n[s])return!1;return!0}acceptEdit(e,t,i){this._acceptDeleteRange(e),this._acceptInsertText(new A(e.startLineNumber,e.startColumn),t,i)}_acceptDeleteRange(e){const t=e.startLineNumber-1;if(t>=this._len)return;if(e.startLineNumber===e.endLineNumber){if(e.startColumn===e.endColumn)return;this._lineTokens[t]=pl.delete(this._lineTokens[t],e.startColumn-1,e.endColumn-1);return}this._lineTokens[t]=pl.deleteEnding(this._lineTokens[t],e.startColumn-1);const i=e.endLineNumber-1;let n=null;i<this._len&&(n=pl.deleteBeginning(this._lineTokens[i],e.endColumn-1)),this._lineTokens[t]=pl.append(this._lineTokens[t],n),this._deleteLines(e.startLineNumber,e.endLineNumber-e.startLineNumber)}_acceptInsertText(e,t,i){if(t===0&&i===0)return;const n=e.lineNumber-1;if(!(n>=this._len)){if(t===0){this._lineTokens[n]=pl.insert(this._lineTokens[n],e.column-1,i);return}this._lineTokens[n]=pl.deleteEnding(this._lineTokens[n],e.column-1),this._lineTokens[n]=pl.insert(this._lineTokens[n],e.column-1,i),this._insertLines(e.lineNumber,t)}}setMultilineTokens(e,t){if(e.length===0)return{changes:[]};const i=[];for(let n=0,s=e.length;n<s;n++){const r=e[n];let a=0,l=0,c=!1;for(let d=r.startLineNumber;d<=r.endLineNumber;d++)c?(this.setTokens(t.getLanguageId(),d-1,t.getLineLength(d),r.getLineTokens(d),!1),l=d):this.setTokens(t.getLanguageId(),d-1,t.getLineLength(d),r.getLineTokens(d),!0)&&(c=!0,a=d,l=d);c&&i.push({fromLineNumber:a,toLineNumber:l})}return{changes:i}}}function E8(o){return(o<<0|0|0|32768|2<<24|1024)>>>0}class p3{constructor(e){this._pieces=[],this._isComplete=!1,this._languageIdCodec=e}flush(){this._pieces=[],this._isComplete=!1}isEmpty(){return this._pieces.length===0}set(e,t){this._pieces=e||[],this._isComplete=t}setPartial(e,t){let i=e;if(t.length>0){const s=t[0].getRange(),r=t[t.length-1].getRange();if(!s||!r)return e;i=e.plusRange(s).plusRange(r)}let n=null;for(let s=0,r=this._pieces.length;s<r;s++){const a=this._pieces[s];if(a.endLineNumber<i.startLineNumber)continue;if(a.startLineNumber>i.endLineNumber){n=n||{index:s};break}if(a.removeTokens(i),a.isEmpty()){this._pieces.splice(s,1),s--,r--;continue}if(a.endLineNumber<i.startLineNumber)continue;if(a.startLineNumber>i.endLineNumber){n=n||{index:s};continue}const[l,c]=a.split(i);if(l.isEmpty()){n=n||{index:s};continue}c.isEmpty()||(this._pieces.splice(s,1,l,c),s++,r++,n=n||{index:s})}return n=n||{index:this._pieces.length},t.length>0&&(this._pieces=cI(this._pieces,n.index,t)),i}isComplete(){return this._isComplete}addSparseTokens(e,t){if(t.getLineContent().length===0)return t;const i=this._pieces;if(i.length===0)return t;const n=p3._findFirstPieceWithLine(i,e),s=i[n].getLineTokens(e);if(!s)return t;const r=t.getCount(),a=s.getCount();let l=0;const c=[];let d=0,h=0;const u=(g,f)=>{g!==h&&(h=g,c[d++]=g,c[d++]=f)};for(let g=0;g<a;g++){const f=s.getStartCharacter(g),p=s.getEndCharacter(g),_=s.getMetadata(g),b=((_&1?2048:0)|(_&2?4096:0)|(_&4?8192:0)|(_&8?16384:0)|(_&16?16744448:0)|(_&32?4278190080:0))>>>0,C=~b>>>0;for(;l<r&&t.getEndOffset(l)<=f;)u(t.getEndOffset(l),t.getMetadata(l)),l++;for(l<r&&t.getStartOffset(l)<f&&u(f,t.getMetadata(l));l<r&&t.getEndOffset(l)<p;)u(t.getEndOffset(l),t.getMetadata(l)&C|_&b),l++;if(l<r)u(p,t.getMetadata(l)&C|_&b),t.getEndOffset(l)===p&&l++;else{const v=Math.min(Math.max(0,l-1),r-1);u(p,t.getMetadata(v)&C|_&b)}}for(;l<r;)u(t.getEndOffset(l),t.getMetadata(l)),l++;return new cn(new Uint32Array(c),t.getLineContent(),this._languageIdCodec)}static _findFirstPieceWithLine(e,t){let i=0,n=e.length-1;for(;i<n;){let s=i+Math.floor((n-i)/2);if(e[s].endLineNumber<t)i=s+1;else if(e[s].startLineNumber>t)n=s-1;else{for(;s>i&&e[s-1].startLineNumber<=t&&t<=e[s-1].endLineNumber;)s--;return s}}return i}acceptEdit(e,t,i,n,s){for(const r of this._pieces)r.acceptEdit(e,t,i,n,s)}}var Zue=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},nT=function(o,e){return function(t,i){e(t,i,o)}},ey;let uA=ey=class extends sK{constructor(e,t,i,n,s,r,a){super(),this._textModel=e,this._bracketPairsTextModelPart=t,this._languageId=i,this._attachedViews=n,this._languageService=s,this._languageConfigurationService=r,this._treeSitterService=a,this._semanticTokens=new p3(this._languageService.languageIdCodec),this._onDidChangeLanguage=this._register(new P),this.onDidChangeLanguage=this._onDidChangeLanguage.event,this._onDidChangeLanguageConfiguration=this._register(new P),this.onDidChangeLanguageConfiguration=this._onDidChangeLanguageConfiguration.event,this._onDidChangeTokens=this._register(new P),this.onDidChangeTokens=this._onDidChangeTokens.event,this._tokensDisposables=this._register(new K),this._register(this._languageConfigurationService.onDidChange(l=>{l.affects(this._languageId)&&this._onDidChangeLanguageConfiguration.fire({})})),this._register(oe.filter(kR.onDidChange,l=>l.changedLanguages.includes(this._languageId))(()=>{this.createPreferredTokenProvider()})),this.createPreferredTokenProvider()}createGrammarTokens(){return this._register(new N8(this._languageService.languageIdCodec,this._textModel,()=>this._languageId,this._attachedViews))}createTreeSitterTokens(){return this._register(new I8(this._treeSitterService,this._languageService.languageIdCodec,this._textModel,()=>this._languageId))}createTokens(e){var i;const t=this._tokens!==void 0;(i=this._tokens)==null||i.dispose(),this._tokens=e?this.createTreeSitterTokens():this.createGrammarTokens(),this._tokensDisposables.clear(),this._tokensDisposables.add(this._tokens.onDidChangeTokens(n=>{this._emitModelTokensChangedEvent(n)})),this._tokensDisposables.add(this._tokens.onDidChangeBackgroundTokenizationState(n=>{this._bracketPairsTextModelPart.handleDidChangeBackgroundTokenizationState()})),t&&this._tokens.resetTokenization()}createPreferredTokenProvider(){kR.get(this._languageId)?this._tokens instanceof I8||this.createTokens(!0):this._tokens instanceof N8||this.createTokens(!1)}handleLanguageConfigurationServiceChange(e){e.affects(this._languageId)&&this._onDidChangeLanguageConfiguration.fire({})}handleDidChangeContent(e){if(e.isFlush)this._semanticTokens.flush();else if(!e.isEolChange)for(const t of e.changes){const[i,n,s]=zh(t.text);this._semanticTokens.acceptEdit(t.range,i,n,s,t.text.length>0?t.text.charCodeAt(0):0)}this._tokens.handleDidChangeContent(e)}handleDidChangeAttached(){this._tokens.handleDidChangeAttached()}getLineTokens(e){this.validateLineNumber(e);const t=this._tokens.getLineTokens(e);return this._semanticTokens.addSparseTokens(e,t)}_emitModelTokensChangedEvent(e){this._textModel._isDisposing()||(this._bracketPairsTextModelPart.handleDidChangeTokens(e),this._onDidChangeTokens.fire(e))}validateLineNumber(e){if(e<1||e>this._textModel.getLineCount())throw new bt("Illegal value for lineNumber")}get hasTokens(){return this._tokens.hasTokens}resetTokenization(){this._tokens.resetTokenization()}get backgroundTokenizationState(){return this._tokens.backgroundTokenizationState}forceTokenization(e){this.validateLineNumber(e),this._tokens.forceTokenization(e)}hasAccurateTokensForLine(e){return this.validateLineNumber(e),this._tokens.hasAccurateTokensForLine(e)}isCheapToTokenize(e){return this.validateLineNumber(e),this._tokens.isCheapToTokenize(e)}tokenizeIfCheap(e){this.validateLineNumber(e),this._tokens.tokenizeIfCheap(e)}getTokenTypeIfInsertingCharacter(e,t,i){return this._tokens.getTokenTypeIfInsertingCharacter(e,t,i)}tokenizeLineWithEdit(e,t,i){return this._tokens.tokenizeLineWithEdit(e,t,i)}setSemanticTokens(e,t){this._semanticTokens.set(e,t),this._emitModelTokensChangedEvent({semanticTokensApplied:e!==null,ranges:[{fromLineNumber:1,toLineNumber:this._textModel.getLineCount()}]})}hasCompleteSemanticTokens(){return this._semanticTokens.isComplete()}hasSomeSemanticTokens(){return!this._semanticTokens.isEmpty()}setPartialSemanticTokens(e,t){if(this.hasCompleteSemanticTokens())return;const i=this._textModel.validateRange(this._semanticTokens.setPartial(e,t));this._emitModelTokensChangedEvent({semanticTokensApplied:!0,ranges:[{fromLineNumber:i.startLineNumber,toLineNumber:i.endLineNumber}]})}getWordAtPosition(e){this.assertNotDisposed();const t=this._textModel.validatePosition(e),i=this._textModel.getLineContent(t.lineNumber),n=this.getLineTokens(t.lineNumber),s=n.findTokenIndexAtOffset(t.column-1),[r,a]=ey._findLanguageBoundaries(n,s),l=Qv(t.column,this.getLanguageConfiguration(n.getLanguageId(s)).getWordDefinition(),i.substring(r,a),r);if(l&&l.startColumn<=e.column&&e.column<=l.endColumn)return l;if(s>0&&r===t.column-1){const[c,d]=ey._findLanguageBoundaries(n,s-1),h=Qv(t.column,this.getLanguageConfiguration(n.getLanguageId(s-1)).getWordDefinition(),i.substring(c,d),c);if(h&&h.startColumn<=e.column&&e.column<=h.endColumn)return h}return null}getLanguageConfiguration(e){return this._languageConfigurationService.getLanguageConfiguration(e)}static _findLanguageBoundaries(e,t){const i=e.getLanguageId(t);let n=0;for(let r=t;r>=0&&e.getLanguageId(r)===i;r--)n=e.getStartOffset(r);let s=e.getLineContent().length;for(let r=t,a=e.getCount();r<a&&e.getLanguageId(r)===i;r++)s=e.getEndOffset(r);return[n,s]}getWordUntilPosition(e){const t=this.getWordAtPosition(e);return t?{word:t.word.substr(0,e.column-t.startColumn),startColumn:t.startColumn,endColumn:e.column}:{word:"",startColumn:e.column,endColumn:e.column}}getLanguageId(){return this._languageId}getLanguageIdAtPosition(e,t){const i=this._textModel.validatePosition(new A(e,t)),n=this.getLineTokens(i.lineNumber);return n.getLanguageId(n.findTokenIndexAtOffset(i.column-1))}setLanguageId(e,t="api"){if(this._languageId===e)return;const i={oldLanguage:this._languageId,newLanguage:e,source:t};this._languageId=e,this._bracketPairsTextModelPart.handleDidChangeLanguage(i),this._tokens.resetTokenization(),this.createPreferredTokenProvider(),this._onDidChangeLanguage.fire(i),this._onDidChangeLanguageConfiguration.fire({})}};uA=ey=Zue([nT(4,ni),nT(5,di),nT(6,fK)],uA);class N8 extends gK{constructor(e,t,i,n){super(e,t,i),this._tokenizer=null,this._defaultBackgroundTokenizer=null,this._backgroundTokenizer=this._register(new Ji),this._tokens=new Aw(this._languageIdCodec),this._debugBackgroundTokenizer=this._register(new Ji),this._attachedViewStates=this._register(new $4),this._register(fi.onDidChange(s=>{const r=this.getLanguageId();s.changedLanguages.indexOf(r)!==-1&&this.resetTokenization()})),this.resetTokenization(),this._register(n.onDidChangeVisibleRanges(({view:s,state:r})=>{if(r){let a=this._attachedViewStates.get(s);a||(a=new Gue(()=>this.refreshRanges(a.lineRanges)),this._attachedViewStates.set(s,a)),a.handleStateChange(r)}else this._attachedViewStates.deleteAndDispose(s)}))}resetTokenization(e=!0){var s;this._tokens.flush(),(s=this._debugBackgroundTokens)==null||s.flush(),this._debugBackgroundStates&&(this._debugBackgroundStates=new hA(this._textModel.getLineCount())),e&&this._onDidChangeTokens.fire({semanticTokensApplied:!1,ranges:[{fromLineNumber:1,toLineNumber:this._textModel.getLineCount()}]});const t=()=>{if(this._textModel.isTooLargeForTokenization())return[null,null];const r=fi.get(this.getLanguageId());if(!r)return[null,null];let a;try{a=r.getInitialState()}catch(l){return Oe(l),[null,null]}return[r,a]},[i,n]=t();if(i&&n?this._tokenizer=new zue(this._textModel.getLineCount(),i,this._textModel,this._languageIdCodec):this._tokenizer=null,this._backgroundTokenizer.clear(),this._defaultBackgroundTokenizer=null,this._tokenizer){const r={setTokens:a=>{this.setTokens(a)},backgroundTokenizationFinished:()=>{if(this._backgroundTokenizationState===2)return;const a=2;this._backgroundTokenizationState=a,this._onDidChangeBackgroundTokenizationState.fire()},setEndState:(a,l)=>{var d;if(!this._tokenizer)return;const c=this._tokenizer.store.getFirstInvalidEndStateLineNumber();c!==null&&a>=c&&((d=this._tokenizer)==null||d.store.setEndState(a,l))}};i&&i.createBackgroundTokenizer&&!i.backgroundTokenizerShouldOnlyVerifyTokens&&(this._backgroundTokenizer.value=i.createBackgroundTokenizer(this._textModel,r)),!this._backgroundTokenizer.value&&!this._textModel.isTooLargeForTokenization()&&(this._backgroundTokenizer.value=this._defaultBackgroundTokenizer=new jue(this._tokenizer,r),this._defaultBackgroundTokenizer.handleChanges()),i!=null&&i.backgroundTokenizerShouldOnlyVerifyTokens&&i.createBackgroundTokenizer?(this._debugBackgroundTokens=new Aw(this._languageIdCodec),this._debugBackgroundStates=new hA(this._textModel.getLineCount()),this._debugBackgroundTokenizer.clear(),this._debugBackgroundTokenizer.value=i.createBackgroundTokenizer(this._textModel,{setTokens:a=>{var l;(l=this._debugBackgroundTokens)==null||l.setMultilineTokens(a,this._textModel)},backgroundTokenizationFinished(){},setEndState:(a,l)=>{var c;(c=this._debugBackgroundStates)==null||c.setEndState(a,l)}})):(this._debugBackgroundTokens=void 0,this._debugBackgroundStates=void 0,this._debugBackgroundTokenizer.value=void 0)}this.refreshAllVisibleLineTokens()}handleDidChangeAttached(){var e;(e=this._defaultBackgroundTokenizer)==null||e.handleChanges()}handleDidChangeContent(e){var t,i,n;if(e.isFlush)this.resetTokenization(!1);else if(!e.isEolChange){for(const s of e.changes){const[r,a]=zh(s.text);this._tokens.acceptEdit(s.range,r,a),(t=this._debugBackgroundTokens)==null||t.acceptEdit(s.range,r,a)}(i=this._debugBackgroundStates)==null||i.acceptChanges(e.changes),this._tokenizer&&this._tokenizer.store.acceptChanges(e.changes),(n=this._defaultBackgroundTokenizer)==null||n.handleChanges()}}setTokens(e){const{changes:t}=this._tokens.setMultilineTokens(e,this._textModel);return t.length>0&&this._onDidChangeTokens.fire({semanticTokensApplied:!1,ranges:t}),{changes:t}}refreshAllVisibleLineTokens(){const e=Be.joinMany([...this._attachedViewStates].map(([t,i])=>i.lineRanges));this.refreshRanges(e)}refreshRanges(e){for(const t of e)this.refreshRange(t.startLineNumber,t.endLineNumberExclusive-1)}refreshRange(e,t){var r,a;if(!this._tokenizer)return;e=Math.max(1,Math.min(this._textModel.getLineCount(),e)),t=Math.min(this._textModel.getLineCount(),t);const i=new dA,{heuristicTokens:n}=this._tokenizer.tokenizeHeuristically(i,e,t),s=this.setTokens(i.finalize());if(n)for(const l of s.changes)(r=this._backgroundTokenizer.value)==null||r.requestTokens(l.fromLineNumber,l.toLineNumber+1);(a=this._defaultBackgroundTokenizer)==null||a.checkFinished()}forceTokenization(e){var i,n;const t=new dA;(i=this._tokenizer)==null||i.updateTokensUntilLine(t,e),this.setTokens(t.finalize()),(n=this._defaultBackgroundTokenizer)==null||n.checkFinished()}hasAccurateTokensForLine(e){return this._tokenizer?this._tokenizer.hasAccurateTokensForLine(e):!0}isCheapToTokenize(e){return this._tokenizer?this._tokenizer.isCheapToTokenize(e):!0}getLineTokens(e){var n;const t=this._textModel.getLineContent(e),i=this._tokens.getTokens(this._textModel.getLanguageId(),e-1,t);if(this._debugBackgroundTokens&&this._debugBackgroundStates&&this._tokenizer&&this._debugBackgroundStates.getFirstInvalidEndStateLineNumberOrMax()>e&&this._tokenizer.store.getFirstInvalidEndStateLineNumberOrMax()>e){const s=this._debugBackgroundTokens.getTokens(this._textModel.getLanguageId(),e-1,t);!i.equals(s)&&((n=this._debugBackgroundTokenizer.value)!=null&&n.reportMismatchingTokens)&&this._debugBackgroundTokenizer.value.reportMismatchingTokens(e)}return i}getTokenTypeIfInsertingCharacter(e,t,i){if(!this._tokenizer)return 0;const n=this._textModel.validatePosition(new A(e,t));return this.forceTokenization(n.lineNumber),this._tokenizer.getTokenTypeIfInsertingCharacter(n,i)}tokenizeLineWithEdit(e,t,i){if(!this._tokenizer)return null;const n=this._textModel.validatePosition(e);return this.forceTokenization(n.lineNumber),this._tokenizer.tokenizeLineWithEdit(n,t,i)}get hasTokens(){return this._tokens.hasTokens}}class Yue{constructor(){this.changeType=1}}class Wa{static applyInjectedText(e,t){if(!t||t.length===0)return e;let i="",n=0;for(const s of t)i+=e.substring(n,s.column-1),n=s.column-1,i+=s.options.content;return i+=e.substring(n),i}static fromDecorations(e){const t=[];for(const i of e)i.options.before&&i.options.before.content.length>0&&t.push(new Wa(i.ownerId,i.range.startLineNumber,i.range.startColumn,i.options.before,0)),i.options.after&&i.options.after.content.length>0&&t.push(new Wa(i.ownerId,i.range.endLineNumber,i.range.endColumn,i.options.after,1));return t.sort((i,n)=>i.lineNumber===n.lineNumber?i.column===n.column?i.order-n.order:i.column-n.column:i.lineNumber-n.lineNumber),t}constructor(e,t,i,n,s){this.ownerId=e,this.lineNumber=t,this.column=i,this.options=n,this.order=s}}class T8{constructor(e,t,i){this.changeType=2,this.lineNumber=e,this.detail=t,this.injectedText=i}}class Xue{constructor(e,t){this.changeType=3,this.fromLineNumber=e,this.toLineNumber=t}}class Que{constructor(e,t,i,n){this.changeType=4,this.injectedTexts=n,this.fromLineNumber=e,this.toLineNumber=t,this.detail=i}}class Jue{constructor(){this.changeType=5}}class e_{constructor(e,t,i,n){this.changes=e,this.versionId=t,this.isUndoing=i,this.isRedoing=n,this.resultingSelection=null}containsEvent(e){for(let t=0,i=this.changes.length;t<i;t++)if(this.changes[t].changeType===e)return!0;return!1}static merge(e,t){const i=[].concat(e.changes).concat(t.changes),n=t.versionId,s=e.isUndoing||t.isUndoing,r=e.isRedoing||t.isRedoing;return new e_(i,n,s,r)}}class mK{constructor(e){this.changes=e}}class vg{constructor(e,t){this.rawContentChangedEvent=e,this.contentChangedEvent=t}merge(e){const t=e_.merge(this.rawContentChangedEvent,e.rawContentChangedEvent),i=vg._mergeChangeEvents(this.contentChangedEvent,e.contentChangedEvent);return new vg(t,i)}static _mergeChangeEvents(e,t){const i=[].concat(e.changes).concat(t.changes),n=t.eol,s=t.versionId,r=e.isUndoing||t.isUndoing,a=e.isRedoing||t.isRedoing,l=e.isFlush||t.isFlush,c=e.isEolChange&&t.isEolChange;return{changes:i,eol:n,isEolChange:c,versionId:s,isUndoing:r,isRedoing:a,isFlush:l}}}var ege=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},JS=function(o,e){return function(t,i){e(t,i,o)}},Ru;function tge(o){const e=new uK;return e.acceptChunk(o),e.finish()}function ige(o){const e=new uK;let t;for(;typeof(t=o.read())=="string";)e.acceptChunk(t);return e.finish()}function R8(o,e){let t;return typeof o=="string"?t=tge(o):jne(o)?t=ige(o):t=o,t.create(e)}let e0=0;const nge=999,sge=1e4;class oge{constructor(e){this._source=e,this._eos=!1}read(){if(this._eos)return null;const e=[];let t=0,i=0;do{const n=this._source.read();if(n===null)return this._eos=!0,t===0?null:e.join("");if(n.length>0&&(e[t++]=n,i+=n.length),i>=64*1024)return e.join("")}while(!0)}}const oC=()=>{throw new Error("Invalid change accessor")};var wa;let Da=(wa=class extends F{static resolveOptions(e,t){if(t.detectIndentation){const i=v8(e,t.tabSize,t.insertSpaces);return new $0({tabSize:i.tabSize,indentSize:"tabSize",insertSpaces:i.insertSpaces,trimAutoWhitespace:t.trimAutoWhitespace,defaultEOL:t.defaultEOL,bracketPairColorizationOptions:t.bracketPairColorizationOptions})}return new $0(t)}get onDidChangeLanguage(){return this._tokenizationTextModelPart.onDidChangeLanguage}get onDidChangeLanguageConfiguration(){return this._tokenizationTextModelPart.onDidChangeLanguageConfiguration}get onDidChangeTokens(){return this._tokenizationTextModelPart.onDidChangeTokens}onDidChangeContent(e){return this._eventEmitter.slowEvent(t=>e(t.contentChangedEvent))}onDidChangeContentOrInjectedText(e){return jr(this._eventEmitter.fastEvent(t=>e(t)),this._onDidChangeInjectedText.event(t=>e(t)))}_isDisposing(){return this.__isDisposing}get tokenization(){return this._tokenizationTextModelPart}get bracketPairs(){return this._bracketPairs}get guides(){return this._guidesTextModelPart}constructor(e,t,i,n=null,s,r,a,l){super(),this._undoRedoService=s,this._languageService=r,this._languageConfigurationService=a,this.instantiationService=l,this._onWillDispose=this._register(new P),this.onWillDispose=this._onWillDispose.event,this._onDidChangeDecorations=this._register(new hge(f=>this.handleBeforeFireDecorationsChangedEvent(f))),this.onDidChangeDecorations=this._onDidChangeDecorations.event,this._onDidChangeOptions=this._register(new P),this.onDidChangeOptions=this._onDidChangeOptions.event,this._onDidChangeAttached=this._register(new P),this.onDidChangeAttached=this._onDidChangeAttached.event,this._onDidChangeInjectedText=this._register(new P),this._eventEmitter=this._register(new uge),this._languageSelectionListener=this._register(new Ji),this._deltaDecorationCallCnt=0,this._attachedViews=new Kue,e0++,this.id="$model"+e0,this.isForSimpleWidget=i.isForSimpleWidget,typeof n>"u"||n===null?this._associatedResource=Se.parse("inmemory://model/"+e0):this._associatedResource=n,this._attachedEditorCount=0;const{textBuffer:c,disposable:d}=R8(e,i.defaultEOL);this._buffer=c,this._bufferDisposable=d,this._options=Ru.resolveOptions(this._buffer,i);const h=typeof t=="string"?t:t.languageId;typeof t!="string"&&(this._languageSelectionListener.value=t.onDidChange(()=>this._setLanguage(t.languageId))),this._bracketPairs=this._register(new hue(this,this._languageConfigurationService)),this._guidesTextModelPart=this._register(new pue(this,this._languageConfigurationService)),this._decorationProvider=this._register(new gue(this)),this._tokenizationTextModelPart=this.instantiationService.createInstance(uA,this,this._bracketPairs,h,this._attachedViews);const u=this._buffer.getLineCount(),g=this._buffer.getValueLengthInRange(new x(1,1,u,this._buffer.getLineLength(u)+1),0);i.largeFileOptimizations?(this._isTooLargeForTokenization=g>Ru.LARGE_FILE_SIZE_THRESHOLD||u>Ru.LARGE_FILE_LINE_COUNT_THRESHOLD,this._isTooLargeForHeapOperation=g>Ru.LARGE_FILE_HEAP_OPERATION_THRESHOLD):(this._isTooLargeForTokenization=!1,this._isTooLargeForHeapOperation=!1),this._isTooLargeForSyncing=g>Ru._MODEL_SYNC_LIMIT,this._versionId=1,this._alternativeVersionId=1,this._initialUndoRedoSnapshot=null,this._isDisposed=!1,this.__isDisposing=!1,this._instanceId=oU(e0),this._lastDecorationId=0,this._decorations=Object.create(null),this._decorationsTree=new M8,this._commandManager=new u3(this,this._undoRedoService),this._isUndoing=!1,this._isRedoing=!1,this._trimAutoWhitespaceLines=null,this._register(this._decorationProvider.onDidChange(()=>{this._onDidChangeDecorations.beginDeferredEmit(),this._onDidChangeDecorations.fire(),this._onDidChangeDecorations.endDeferredEmit()})),this._languageService.requestRichLanguageFeatures(h),this._register(this._languageConfigurationService.onDidChange(f=>{this._bracketPairs.handleLanguageConfigurationServiceChange(f),this._tokenizationTextModelPart.handleLanguageConfigurationServiceChange(f)}))}dispose(){this.__isDisposing=!0,this._onWillDispose.fire(),this._tokenizationTextModelPart.dispose(),this._isDisposed=!0,super.dispose(),this._bufferDisposable.dispose(),this.__isDisposing=!1;const e=new Jp([],"",`
`,!1,!1,!0,!0);e.dispose(),this._buffer=e,this._bufferDisposable=F.None}_assertNotDisposed(){if(this._isDisposed)throw new bt("Model is disposed!")}_emitContentChangedEvent(e,t){this.__isDisposing||(this._tokenizationTextModelPart.handleDidChangeContent(t),this._bracketPairs.handleDidChangeContent(t),this._eventEmitter.fire(new vg(e,t)))}setValue(e){if(this._assertNotDisposed(),e==null)throw gr();const{textBuffer:t,disposable:i}=R8(e,this._options.defaultEOL);this._setValueFromTextBuffer(t,i)}_createContentChanged2(e,t,i,n,s,r,a,l){return{changes:[{range:e,rangeOffset:t,rangeLength:i,text:n}],eol:this._buffer.getEOL(),isEolChange:l,versionId:this.getVersionId(),isUndoing:s,isRedoing:r,isFlush:a}}_setValueFromTextBuffer(e,t){this._assertNotDisposed();const i=this.getFullModelRange(),n=this.getValueLengthInRange(i),s=this.getLineCount(),r=this.getLineMaxColumn(s);this._buffer=e,this._bufferDisposable.dispose(),this._bufferDisposable=t,this._increaseVersionId(),this._decorations=Object.create(null),this._decorationsTree=new M8,this._commandManager.clear(),this._trimAutoWhitespaceLines=null,this._emitContentChangedEvent(new e_([new Yue],this._versionId,!1,!1),this._createContentChanged2(new x(1,1,s,r),0,n,this.getValue(),!1,!1,!0,!1))}setEOL(e){this._assertNotDisposed();const t=e===1?`\r
`:`
`;if(this._buffer.getEOL()===t)return;const i=this.getFullModelRange(),n=this.getValueLengthInRange(i),s=this.getLineCount(),r=this.getLineMaxColumn(s);this._onBeforeEOLChange(),this._buffer.setEOL(t),this._increaseVersionId(),this._onAfterEOLChange(),this._emitContentChangedEvent(new e_([new Jue],this._versionId,!1,!1),this._createContentChanged2(new x(1,1,s,r),0,n,this.getValue(),!1,!1,!1,!0))}_onBeforeEOLChange(){this._decorationsTree.ensureAllNodesHaveRanges(this)}_onAfterEOLChange(){const e=this.getVersionId(),t=this._decorationsTree.collectNodesPostOrder();for(let i=0,n=t.length;i<n;i++){const s=t[i],r=s.range,a=s.cachedAbsoluteStart-s.start,l=this._buffer.getOffsetAt(r.startLineNumber,r.startColumn),c=this._buffer.getOffsetAt(r.endLineNumber,r.endColumn);s.cachedAbsoluteStart=l,s.cachedAbsoluteEnd=c,s.cachedVersionId=e,s.start=l-a,s.end=c-a,jh(s)}}onBeforeAttached(){return this._attachedEditorCount++,this._attachedEditorCount===1&&(this._tokenizationTextModelPart.handleDidChangeAttached(),this._onDidChangeAttached.fire(void 0)),this._attachedViews.attachView()}onBeforeDetached(e){this._attachedEditorCount--,this._attachedEditorCount===0&&(this._tokenizationTextModelPart.handleDidChangeAttached(),this._onDidChangeAttached.fire(void 0)),this._attachedViews.detachView(e)}isAttachedToEditor(){return this._attachedEditorCount>0}getAttachedEditorCount(){return this._attachedEditorCount}isTooLargeForSyncing(){return this._isTooLargeForSyncing}isTooLargeForTokenization(){return this._isTooLargeForTokenization}isTooLargeForHeapOperation(){return this._isTooLargeForHeapOperation}isDisposed(){return this._isDisposed}isDominatedByLongLines(){if(this._assertNotDisposed(),this.isTooLargeForTokenization())return!1;let e=0,t=0;const i=this._buffer.getLineCount();for(let n=1;n<=i;n++){const s=this._buffer.getLineLength(n);s>=sge?t+=s:e+=s}return t>e}get uri(){return this._associatedResource}getOptions(){return this._assertNotDisposed(),this._options}getFormattingOptions(){return{tabSize:this._options.indentSize,insertSpaces:this._options.insertSpaces}}updateOptions(e){this._assertNotDisposed();const t=typeof e.tabSize<"u"?e.tabSize:this._options.tabSize,i=typeof e.indentSize<"u"?e.indentSize:this._options.originalIndentSize,n=typeof e.insertSpaces<"u"?e.insertSpaces:this._options.insertSpaces,s=typeof e.trimAutoWhitespace<"u"?e.trimAutoWhitespace:this._options.trimAutoWhitespace,r=typeof e.bracketColorizationOptions<"u"?e.bracketColorizationOptions:this._options.bracketPairColorizationOptions,a=new $0({tabSize:t,indentSize:i,insertSpaces:n,defaultEOL:this._options.defaultEOL,trimAutoWhitespace:s,bracketPairColorizationOptions:r});if(this._options.equals(a))return;const l=this._options.createChangeEvent(a);this._options=a,this._bracketPairs.handleDidChangeOptions(l),this._decorationProvider.handleDidChangeOptions(l),this._onDidChangeOptions.fire(l)}detectIndentation(e,t){this._assertNotDisposed();const i=v8(this._buffer,t,e);this.updateOptions({insertSpaces:i.insertSpaces,tabSize:i.tabSize,indentSize:i.tabSize})}normalizeIndentation(e){return this._assertNotDisposed(),a3(e,this._options.indentSize,this._options.insertSpaces)}getVersionId(){return this._assertNotDisposed(),this._versionId}mightContainRTL(){return this._buffer.mightContainRTL()}mightContainUnusualLineTerminators(){return this._buffer.mightContainUnusualLineTerminators()}removeUnusualLineTerminators(e=null){const t=this.findMatches(nU.source,!1,!0,!1,null,!1,1073741824);this._buffer.resetMightContainUnusualLineTerminators(),this.pushEditOperations(e,t.map(i=>({range:i.range,text:null})),()=>null)}mightContainNonBasicASCII(){return this._buffer.mightContainNonBasicASCII()}getAlternativeVersionId(){return this._assertNotDisposed(),this._alternativeVersionId}getInitialUndoRedoSnapshot(){return this._assertNotDisposed(),this._initialUndoRedoSnapshot}getOffsetAt(e){this._assertNotDisposed();const t=this._validatePosition(e.lineNumber,e.column,0);return this._buffer.getOffsetAt(t.lineNumber,t.column)}getPositionAt(e){this._assertNotDisposed();const t=Math.min(this._buffer.getLength(),Math.max(0,e));return this._buffer.getPositionAt(t)}_increaseVersionId(){this._versionId=this._versionId+1,this._alternativeVersionId=this._versionId}_overwriteVersionId(e){this._versionId=e}_overwriteAlternativeVersionId(e){this._alternativeVersionId=e}_overwriteInitialUndoRedoSnapshot(e){this._initialUndoRedoSnapshot=e}getValue(e,t=!1){if(this._assertNotDisposed(),this.isTooLargeForHeapOperation())throw new bt("Operation would exceed heap memory limits");const i=this.getFullModelRange(),n=this.getValueInRange(i,e);return t?this._buffer.getBOM()+n:n}createSnapshot(e=!1){return new oge(this._buffer.createSnapshot(e))}getValueLength(e,t=!1){this._assertNotDisposed();const i=this.getFullModelRange(),n=this.getValueLengthInRange(i,e);return t?this._buffer.getBOM().length+n:n}getValueInRange(e,t=0){return this._assertNotDisposed(),this._buffer.getValueInRange(this.validateRange(e),t)}getValueLengthInRange(e,t=0){return this._assertNotDisposed(),this._buffer.getValueLengthInRange(this.validateRange(e),t)}getCharacterCountInRange(e,t=0){return this._assertNotDisposed(),this._buffer.getCharacterCountInRange(this.validateRange(e),t)}getLineCount(){return this._assertNotDisposed(),this._buffer.getLineCount()}getLineContent(e){if(this._assertNotDisposed(),e<1||e>this.getLineCount())throw new bt("Illegal value for lineNumber");return this._buffer.getLineContent(e)}getLineLength(e){if(this._assertNotDisposed(),e<1||e>this.getLineCount())throw new bt("Illegal value for lineNumber");return this._buffer.getLineLength(e)}getLinesContent(){if(this._assertNotDisposed(),this.isTooLargeForHeapOperation())throw new bt("Operation would exceed heap memory limits");return this._buffer.getLinesContent()}getEOL(){return this._assertNotDisposed(),this._buffer.getEOL()}getEndOfLineSequence(){return this._assertNotDisposed(),this._buffer.getEOL()===`
`?0:1}getLineMinColumn(e){return this._assertNotDisposed(),1}getLineMaxColumn(e){if(this._assertNotDisposed(),e<1||e>this.getLineCount())throw new bt("Illegal value for lineNumber");return this._buffer.getLineLength(e)+1}getLineFirstNonWhitespaceColumn(e){if(this._assertNotDisposed(),e<1||e>this.getLineCount())throw new bt("Illegal value for lineNumber");return this._buffer.getLineFirstNonWhitespaceColumn(e)}getLineLastNonWhitespaceColumn(e){if(this._assertNotDisposed(),e<1||e>this.getLineCount())throw new bt("Illegal value for lineNumber");return this._buffer.getLineLastNonWhitespaceColumn(e)}_validateRangeRelaxedNoAllocations(e){const t=this._buffer.getLineCount(),i=e.startLineNumber,n=e.startColumn;let s=Math.floor(typeof i=="number"&&!isNaN(i)?i:1),r=Math.floor(typeof n=="number"&&!isNaN(n)?n:1);if(s<1)s=1,r=1;else if(s>t)s=t,r=this.getLineMaxColumn(s);else if(r<=1)r=1;else{const h=this.getLineMaxColumn(s);r>=h&&(r=h)}const a=e.endLineNumber,l=e.endColumn;let c=Math.floor(typeof a=="number"&&!isNaN(a)?a:1),d=Math.floor(typeof l=="number"&&!isNaN(l)?l:1);if(c<1)c=1,d=1;else if(c>t)c=t,d=this.getLineMaxColumn(c);else if(d<=1)d=1;else{const h=this.getLineMaxColumn(c);d>=h&&(d=h)}return i===s&&n===r&&a===c&&l===d&&e instanceof x&&!(e instanceof pe)?e:new x(s,r,c,d)}_isValidPosition(e,t,i){if(typeof e!="number"||typeof t!="number"||isNaN(e)||isNaN(t)||e<1||t<1||(e|0)!==e||(t|0)!==t)return!1;const n=this._buffer.getLineCount();if(e>n)return!1;if(t===1)return!0;const s=this.getLineMaxColumn(e);if(t>s)return!1;if(i===1){const r=this._buffer.getLineCharCode(e,t-2);if(Gi(r))return!1}return!0}_validatePosition(e,t,i){const n=Math.floor(typeof e=="number"&&!isNaN(e)?e:1),s=Math.floor(typeof t=="number"&&!isNaN(t)?t:1),r=this._buffer.getLineCount();if(n<1)return new A(1,1);if(n>r)return new A(r,this.getLineMaxColumn(r));if(s<=1)return new A(n,1);const a=this.getLineMaxColumn(n);if(s>=a)return new A(n,a);if(i===1){const l=this._buffer.getLineCharCode(n,s-2);if(Gi(l))return new A(n,s-1)}return new A(n,s)}validatePosition(e){return this._assertNotDisposed(),e instanceof A&&this._isValidPosition(e.lineNumber,e.column,1)?e:this._validatePosition(e.lineNumber,e.column,1)}_isValidRange(e,t){const i=e.startLineNumber,n=e.startColumn,s=e.endLineNumber,r=e.endColumn;if(!this._isValidPosition(i,n,0)||!this._isValidPosition(s,r,0))return!1;if(t===1){const a=n>1?this._buffer.getLineCharCode(i,n-2):0,l=r>1&&r<=this._buffer.getLineLength(s)?this._buffer.getLineCharCode(s,r-2):0,c=Gi(a),d=Gi(l);return!c&&!d}return!0}validateRange(e){if(this._assertNotDisposed(),e instanceof x&&!(e instanceof pe)&&this._isValidRange(e,1))return e;const i=this._validatePosition(e.startLineNumber,e.startColumn,0),n=this._validatePosition(e.endLineNumber,e.endColumn,0),s=i.lineNumber,r=i.column,a=n.lineNumber,l=n.column;{const c=r>1?this._buffer.getLineCharCode(s,r-2):0,d=l>1&&l<=this._buffer.getLineLength(a)?this._buffer.getLineCharCode(a,l-2):0,h=Gi(c),u=Gi(d);return!h&&!u?new x(s,r,a,l):s===a&&r===l?new x(s,r-1,a,l-1):h&&u?new x(s,r-1,a,l+1):h?new x(s,r-1,a,l):new x(s,r,a,l+1)}}modifyPosition(e,t){this._assertNotDisposed();const i=this.getOffsetAt(e)+t;return this.getPositionAt(Math.min(this._buffer.getLength(),Math.max(0,i)))}getFullModelRange(){this._assertNotDisposed();const e=this.getLineCount();return new x(1,1,e,this.getLineMaxColumn(e))}findMatchesLineByLine(e,t,i,n){return this._buffer.findMatchesLineByLine(e,t,i,n)}findMatches(e,t,i,n,s,r,a=nge){this._assertNotDisposed();let l=null;t!==null&&(Array.isArray(t)||(t=[t]),t.every(h=>x.isIRange(h))&&(l=t.map(h=>this.validateRange(h)))),l===null&&(l=[this.getFullModelRange()]),l=l.sort((h,u)=>h.startLineNumber-u.startLineNumber||h.startColumn-u.startColumn);const c=[];c.push(l.reduce((h,u)=>x.areIntersecting(h,u)?h.plusRange(u):(c.push(h),u)));let d;if(!i&&e.indexOf(`
`)<0){const u=new Tu(e,i,n,s).parseSearchRequest();if(!u)return[];d=g=>this.findMatchesLineByLine(g,u,r,a)}else d=h=>AS.findMatches(this,new Tu(e,i,n,s),h,r,a);return c.map(d).reduce((h,u)=>h.concat(u),[])}findNextMatch(e,t,i,n,s,r){this._assertNotDisposed();const a=this.validatePosition(t);if(!i&&e.indexOf(`
`)<0){const c=new Tu(e,i,n,s).parseSearchRequest();if(!c)return null;const d=this.getLineCount();let h=new x(a.lineNumber,a.column,d,this.getLineMaxColumn(d)),u=this.findMatchesLineByLine(h,c,r,1);return AS.findNextMatch(this,new Tu(e,i,n,s),a,r),u.length>0||(h=new x(1,1,a.lineNumber,this.getLineMaxColumn(a.lineNumber)),u=this.findMatchesLineByLine(h,c,r,1),u.length>0)?u[0]:null}return AS.findNextMatch(this,new Tu(e,i,n,s),a,r)}findPreviousMatch(e,t,i,n,s,r){this._assertNotDisposed();const a=this.validatePosition(t);return AS.findPreviousMatch(this,new Tu(e,i,n,s),a,r)}pushStackElement(){this._commandManager.pushStackElement()}popStackElement(){this._commandManager.popStackElement()}pushEOL(e){if((this.getEOL()===`
`?0:1)!==e)try{this._onDidChangeDecorations.beginDeferredEmit(),this._eventEmitter.beginDeferredEmit(),this._initialUndoRedoSnapshot===null&&(this._initialUndoRedoSnapshot=this._undoRedoService.createSnapshot(this.uri)),this._commandManager.pushEOL(e)}finally{this._eventEmitter.endDeferredEmit(),this._onDidChangeDecorations.endDeferredEmit()}}_validateEditOperation(e){return e instanceof NN?e:new NN(e.identifier||null,this.validateRange(e.range),e.text,e.forceMoveMarkers||!1,e.isAutoWhitespaceEdit||!1,e._isTracked||!1)}_validateEditOperations(e){const t=[];for(let i=0,n=e.length;i<n;i++)t[i]=this._validateEditOperation(e[i]);return t}pushEditOperations(e,t,i,n){try{return this._onDidChangeDecorations.beginDeferredEmit(),this._eventEmitter.beginDeferredEmit(),this._pushEditOperations(e,this._validateEditOperations(t),i,n)}finally{this._eventEmitter.endDeferredEmit(),this._onDidChangeDecorations.endDeferredEmit()}}_pushEditOperations(e,t,i,n){if(this._options.trimAutoWhitespace&&this._trimAutoWhitespaceLines){const s=t.map(a=>({range:this.validateRange(a.range),text:a.text}));let r=!0;if(e)for(let a=0,l=e.length;a<l;a++){const c=e[a];let d=!1;for(let h=0,u=s.length;h<u;h++){const g=s[h].range,f=g.startLineNumber>c.endLineNumber,p=c.startLineNumber>g.endLineNumber;if(!f&&!p){d=!0;break}}if(!d){r=!1;break}}if(r)for(let a=0,l=this._trimAutoWhitespaceLines.length;a<l;a++){const c=this._trimAutoWhitespaceLines[a],d=this.getLineMaxColumn(c);let h=!0;for(let u=0,g=s.length;u<g;u++){const f=s[u].range,p=s[u].text;if(!(c<f.startLineNumber||c>f.endLineNumber)&&!(c===f.startLineNumber&&f.startColumn===d&&f.isEmpty()&&p&&p.length>0&&p.charAt(0)===`
`)&&!(c===f.startLineNumber&&f.startColumn===1&&f.isEmpty()&&p&&p.length>0&&p.charAt(p.length-1)===`
`)){h=!1;break}}if(h){const u=new x(c,1,c,d);t.push(new NN(null,u,null,!1,!1,!1))}}this._trimAutoWhitespaceLines=null}return this._initialUndoRedoSnapshot===null&&(this._initialUndoRedoSnapshot=this._undoRedoService.createSnapshot(this.uri)),this._commandManager.pushEditOperation(e,t,i,n)}_applyUndo(e,t,i,n){const s=e.map(r=>{const a=this.getPositionAt(r.newPosition),l=this.getPositionAt(r.newEnd);return{range:new x(a.lineNumber,a.column,l.lineNumber,l.column),text:r.oldText}});this._applyUndoRedoEdits(s,t,!0,!1,i,n)}_applyRedo(e,t,i,n){const s=e.map(r=>{const a=this.getPositionAt(r.oldPosition),l=this.getPositionAt(r.oldEnd);return{range:new x(a.lineNumber,a.column,l.lineNumber,l.column),text:r.newText}});this._applyUndoRedoEdits(s,t,!1,!0,i,n)}_applyUndoRedoEdits(e,t,i,n,s,r){try{this._onDidChangeDecorations.beginDeferredEmit(),this._eventEmitter.beginDeferredEmit(),this._isUndoing=i,this._isRedoing=n,this.applyEdits(e,!1),this.setEOL(t),this._overwriteAlternativeVersionId(s)}finally{this._isUndoing=!1,this._isRedoing=!1,this._eventEmitter.endDeferredEmit(r),this._onDidChangeDecorations.endDeferredEmit()}}applyEdits(e,t=!1){try{this._onDidChangeDecorations.beginDeferredEmit(),this._eventEmitter.beginDeferredEmit();const i=this._validateEditOperations(e);return this._doApplyEdits(i,t)}finally{this._eventEmitter.endDeferredEmit(),this._onDidChangeDecorations.endDeferredEmit()}}_doApplyEdits(e,t){const i=this._buffer.getLineCount(),n=this._buffer.applyEdits(e,this._options.trimAutoWhitespace,t),s=this._buffer.getLineCount(),r=n.changes;if(this._trimAutoWhitespaceLines=n.trimAutoWhitespaceLineNumbers,r.length!==0){for(let c=0,d=r.length;c<d;c++){const h=r[c];this._decorationsTree.acceptReplace(h.rangeOffset,h.rangeLength,h.text.length,h.forceMoveMarkers)}const a=[];this._increaseVersionId();let l=i;for(let c=0,d=r.length;c<d;c++){const h=r[c],[u]=zh(h.text);this._onDidChangeDecorations.fire();const g=h.range.startLineNumber,f=h.range.endLineNumber,p=f-g,_=u,b=Math.min(p,_),C=_-p,v=s-l-C+g,w=v,S=v+_,L=this._decorationsTree.getInjectedTextInInterval(this,this.getOffsetAt(new A(w,1)),this.getOffsetAt(new A(S,this.getLineMaxColumn(S))),0),k=Wa.fromDecorations(L),D=new Qc(k);for(let N=b;N>=0;N--){const B=g+N,W=v+N;D.takeFromEndWhile($=>$.lineNumber>W);const V=D.takeFromEndWhile($=>$.lineNumber===W);a.push(new T8(B,this.getLineContent(W),V))}if(b<p){const N=g+b;a.push(new Xue(N+1,f))}if(b<_){const N=new Qc(k),B=g+b,W=_-b,V=s-l-W+B+1,$=[],H=[];for(let j=0;j<W;j++){const J=V+j;H[j]=this.getLineContent(J),N.takeWhile(he=>he.lineNumber<J),$[j]=N.takeWhile(he=>he.lineNumber===J)}a.push(new Que(B+1,g+_,H,$))}l+=C}this._emitContentChangedEvent(new e_(a,this.getVersionId(),this._isUndoing,this._isRedoing),{changes:r,eol:this._buffer.getEOL(),isEolChange:!1,versionId:this.getVersionId(),isUndoing:this._isUndoing,isRedoing:this._isRedoing,isFlush:!1})}return n.reverseEdits===null?void 0:n.reverseEdits}undo(){return this._undoRedoService.undo(this.uri)}canUndo(){return this._undoRedoService.canUndo(this.uri)}redo(){return this._undoRedoService.redo(this.uri)}canRedo(){return this._undoRedoService.canRedo(this.uri)}handleBeforeFireDecorationsChangedEvent(e){if(e===null||e.size===0)return;const i=Array.from(e).map(n=>new T8(n,this.getLineContent(n),this._getInjectedTextInLine(n)));this._onDidChangeInjectedText.fire(new mK(i))}changeDecorations(e,t=0){this._assertNotDisposed();try{return this._onDidChangeDecorations.beginDeferredEmit(),this._changeDecorations(t,e)}finally{this._onDidChangeDecorations.endDeferredEmit()}}_changeDecorations(e,t){const i={addDecoration:(s,r)=>this._deltaDecorationsImpl(e,[],[{range:s,options:r}])[0],changeDecoration:(s,r)=>{this._changeDecorationImpl(s,r)},changeDecorationOptions:(s,r)=>{this._changeDecorationOptionsImpl(s,P8(r))},removeDecoration:s=>{this._deltaDecorationsImpl(e,[s],[])},deltaDecorations:(s,r)=>s.length===0&&r.length===0?[]:this._deltaDecorationsImpl(e,s,r)};let n=null;try{n=t(i)}catch(s){Oe(s)}return i.addDecoration=oC,i.changeDecoration=oC,i.changeDecorationOptions=oC,i.removeDecoration=oC,i.deltaDecorations=oC,n}deltaDecorations(e,t,i=0){if(this._assertNotDisposed(),e||(e=[]),e.length===0&&t.length===0)return[];try{return this._deltaDecorationCallCnt++,this._deltaDecorationCallCnt>1&&(console.warn("Invoking deltaDecorations recursively could lead to leaking decorations."),Oe(new Error("Invoking deltaDecorations recursively could lead to leaking decorations."))),this._onDidChangeDecorations.beginDeferredEmit(),this._deltaDecorationsImpl(i,e,t)}finally{this._onDidChangeDecorations.endDeferredEmit(),this._deltaDecorationCallCnt--}}_getTrackedRange(e){return this.getDecorationRange(e)}_setTrackedRange(e,t,i){const n=e?this._decorations[e]:null;if(!n)return t?this._deltaDecorationsImpl(0,[],[{range:t,options:A8[i]}],!0)[0]:null;if(!t)return this._decorationsTree.delete(n),delete this._decorations[n.id],null;const s=this._validateRangeRelaxedNoAllocations(t),r=this._buffer.getOffsetAt(s.startLineNumber,s.startColumn),a=this._buffer.getOffsetAt(s.endLineNumber,s.endColumn);return this._decorationsTree.delete(n),n.reset(this.getVersionId(),r,a,s),n.setOptions(A8[i]),this._decorationsTree.insert(n),n.id}removeAllDecorationsWithOwnerId(e){if(this._isDisposed)return;const t=this._decorationsTree.collectNodesFromOwner(e);for(let i=0,n=t.length;i<n;i++){const s=t[i];this._decorationsTree.delete(s),delete this._decorations[s.id]}}getDecorationOptions(e){const t=this._decorations[e];return t?t.options:null}getDecorationRange(e){const t=this._decorations[e];return t?this._decorationsTree.getNodeRange(this,t):null}getLineDecorations(e,t=0,i=!1){return e<1||e>this.getLineCount()?[]:this.getLinesDecorations(e,e,t,i)}getLinesDecorations(e,t,i=0,n=!1,s=!1){const r=this.getLineCount(),a=Math.min(r,Math.max(1,e)),l=Math.min(r,Math.max(1,t)),c=this.getLineMaxColumn(l),d=new x(a,1,l,c),h=this._getDecorationsInRange(d,i,n,s);return hR(h,this._decorationProvider.getDecorationsInRange(d,i,n)),h}getDecorationsInRange(e,t=0,i=!1,n=!1,s=!1){const r=this.validateRange(e),a=this._getDecorationsInRange(r,t,i,s);return hR(a,this._decorationProvider.getDecorationsInRange(r,t,i,n)),a}getOverviewRulerDecorations(e=0,t=!1){return this._decorationsTree.getAll(this,e,t,!0,!1)}getInjectedTextDecorations(e=0){return this._decorationsTree.getAllInjectedText(this,e)}_getInjectedTextInLine(e){const t=this._buffer.getOffsetAt(e,1),i=t+this._buffer.getLineLength(e),n=this._decorationsTree.getInjectedTextInInterval(this,t,i,0);return Wa.fromDecorations(n).filter(s=>s.lineNumber===e)}getAllDecorations(e=0,t=!1){let i=this._decorationsTree.getAll(this,e,t,!1,!1);return i=i.concat(this._decorationProvider.getAllDecorations(e,t)),i}getAllMarginDecorations(e=0){return this._decorationsTree.getAll(this,e,!1,!1,!0)}_getDecorationsInRange(e,t,i,n){const s=this._buffer.getOffsetAt(e.startLineNumber,e.startColumn),r=this._buffer.getOffsetAt(e.endLineNumber,e.endColumn);return this._decorationsTree.getAllInInterval(this,s,r,t,i,n)}getRangeAt(e,t){return this._buffer.getRangeAt(e,t-e)}_changeDecorationImpl(e,t){const i=this._decorations[e];if(!i)return;if(i.options.after){const a=this.getDecorationRange(e);this._onDidChangeDecorations.recordLineAffectedByInjectedText(a.endLineNumber)}if(i.options.before){const a=this.getDecorationRange(e);this._onDidChangeDecorations.recordLineAffectedByInjectedText(a.startLineNumber)}const n=this._validateRangeRelaxedNoAllocations(t),s=this._buffer.getOffsetAt(n.startLineNumber,n.startColumn),r=this._buffer.getOffsetAt(n.endLineNumber,n.endColumn);this._decorationsTree.delete(i),i.reset(this.getVersionId(),s,r,n),this._decorationsTree.insert(i),this._onDidChangeDecorations.checkAffectedAndFire(i.options),i.options.after&&this._onDidChangeDecorations.recordLineAffectedByInjectedText(n.endLineNumber),i.options.before&&this._onDidChangeDecorations.recordLineAffectedByInjectedText(n.startLineNumber)}_changeDecorationOptionsImpl(e,t){const i=this._decorations[e];if(!i)return;const n=!!(i.options.overviewRuler&&i.options.overviewRuler.color),s=!!(t.overviewRuler&&t.overviewRuler.color);if(this._onDidChangeDecorations.checkAffectedAndFire(i.options),this._onDidChangeDecorations.checkAffectedAndFire(t),i.options.after||t.after){const l=this._decorationsTree.getNodeRange(this,i);this._onDidChangeDecorations.recordLineAffectedByInjectedText(l.endLineNumber)}if(i.options.before||t.before){const l=this._decorationsTree.getNodeRange(this,i);this._onDidChangeDecorations.recordLineAffectedByInjectedText(l.startLineNumber)}const r=n!==s,a=age(t)!==ty(i);r||a?(this._decorationsTree.delete(i),i.setOptions(t),this._decorationsTree.insert(i)):i.setOptions(t)}_deltaDecorationsImpl(e,t,i,n=!1){const s=this.getVersionId(),r=t.length;let a=0;const l=i.length;let c=0;this._onDidChangeDecorations.beginDeferredEmit();try{const d=new Array(l);for(;a<r||c<l;){let h=null;if(a<r){do h=this._decorations[t[a++]];while(!h&&a<r);if(h){if(h.options.after){const u=this._decorationsTree.getNodeRange(this,h);this._onDidChangeDecorations.recordLineAffectedByInjectedText(u.endLineNumber)}if(h.options.before){const u=this._decorationsTree.getNodeRange(this,h);this._onDidChangeDecorations.recordLineAffectedByInjectedText(u.startLineNumber)}this._decorationsTree.delete(h),n||this._onDidChangeDecorations.checkAffectedAndFire(h.options)}}if(c<l){if(!h){const b=++this._lastDecorationId,C=`${this._instanceId};${b}`;h=new lK(C,0,0),this._decorations[C]=h}const u=i[c],g=this._validateRangeRelaxedNoAllocations(u.range),f=P8(u.options),p=this._buffer.getOffsetAt(g.startLineNumber,g.startColumn),_=this._buffer.getOffsetAt(g.endLineNumber,g.endColumn);h.ownerId=e,h.reset(s,p,_,g),h.setOptions(f),h.options.after&&this._onDidChangeDecorations.recordLineAffectedByInjectedText(g.endLineNumber),h.options.before&&this._onDidChangeDecorations.recordLineAffectedByInjectedText(g.startLineNumber),n||this._onDidChangeDecorations.checkAffectedAndFire(f),this._decorationsTree.insert(h),d[c]=h.id,c++}else h&&delete this._decorations[h.id]}return d}finally{this._onDidChangeDecorations.endDeferredEmit()}}getLanguageId(){return this.tokenization.getLanguageId()}setLanguage(e,t){typeof e=="string"?(this._languageSelectionListener.clear(),this._setLanguage(e,t)):(this._languageSelectionListener.value=e.onDidChange(()=>this._setLanguage(e.languageId,t)),this._setLanguage(e.languageId,t))}_setLanguage(e,t){this.tokenization.setLanguageId(e,t),this._languageService.requestRichLanguageFeatures(e)}getLanguageIdAtPosition(e,t){return this.tokenization.getLanguageIdAtPosition(e,t)}getWordAtPosition(e){return this._tokenizationTextModelPart.getWordAtPosition(e)}getWordUntilPosition(e){return this._tokenizationTextModelPart.getWordUntilPosition(e)}normalizePosition(e,t){return e}getLineIndentColumn(e){return rge(this.getLineContent(e))+1}},Ru=wa,wa._MODEL_SYNC_LIMIT=50*1024*1024,wa.LARGE_FILE_SIZE_THRESHOLD=20*1024*1024,wa.LARGE_FILE_LINE_COUNT_THRESHOLD=300*1e3,wa.LARGE_FILE_HEAP_OPERATION_THRESHOLD=256*1024*1024,wa.DEFAULT_CREATION_OPTIONS={isForSimpleWidget:!1,tabSize:An.tabSize,indentSize:An.indentSize,insertSpaces:An.insertSpaces,detectIndentation:!1,defaultEOL:1,trimAutoWhitespace:An.trimAutoWhitespace,largeFileOptimizations:An.largeFileOptimizations,bracketPairColorizationOptions:An.bracketPairColorizationOptions},wa);Da=Ru=ege([JS(4,y5),JS(5,ni),JS(6,di),JS(7,we)],Da);function rge(o){let e=0;for(const t of o)if(t===" "||t==="	")e++;else break;return e}function sT(o){return!!(o.options.overviewRuler&&o.options.overviewRuler.color)}function age(o){return!!o.after||!!o.before}function ty(o){return!!o.options.after||!!o.options.before}class M8{constructor(){this._decorationsTree0=new eT,this._decorationsTree1=new eT,this._injectedTextDecorationsTree=new eT}ensureAllNodesHaveRanges(e){this.getAll(e,0,!1,!1,!1)}_ensureNodesHaveRanges(e,t){for(const i of t)i.range===null&&(i.range=e.getRangeAt(i.cachedAbsoluteStart,i.cachedAbsoluteEnd));return t}getAllInInterval(e,t,i,n,s,r){const a=e.getVersionId(),l=this._intervalSearch(t,i,n,s,a,r);return this._ensureNodesHaveRanges(e,l)}_intervalSearch(e,t,i,n,s,r){const a=this._decorationsTree0.intervalSearch(e,t,i,n,s,r),l=this._decorationsTree1.intervalSearch(e,t,i,n,s,r),c=this._injectedTextDecorationsTree.intervalSearch(e,t,i,n,s,r);return a.concat(l).concat(c)}getInjectedTextInInterval(e,t,i,n){const s=e.getVersionId(),r=this._injectedTextDecorationsTree.intervalSearch(t,i,n,!1,s,!1);return this._ensureNodesHaveRanges(e,r).filter(a=>a.options.showIfCollapsed||!a.range.isEmpty())}getAllInjectedText(e,t){const i=e.getVersionId(),n=this._injectedTextDecorationsTree.search(t,!1,i,!1);return this._ensureNodesHaveRanges(e,n).filter(s=>s.options.showIfCollapsed||!s.range.isEmpty())}getAll(e,t,i,n,s){const r=e.getVersionId(),a=this._search(t,i,n,r,s);return this._ensureNodesHaveRanges(e,a)}_search(e,t,i,n,s){if(i)return this._decorationsTree1.search(e,t,n,s);{const r=this._decorationsTree0.search(e,t,n,s),a=this._decorationsTree1.search(e,t,n,s),l=this._injectedTextDecorationsTree.search(e,t,n,s);return r.concat(a).concat(l)}}collectNodesFromOwner(e){const t=this._decorationsTree0.collectNodesFromOwner(e),i=this._decorationsTree1.collectNodesFromOwner(e),n=this._injectedTextDecorationsTree.collectNodesFromOwner(e);return t.concat(i).concat(n)}collectNodesPostOrder(){const e=this._decorationsTree0.collectNodesPostOrder(),t=this._decorationsTree1.collectNodesPostOrder(),i=this._injectedTextDecorationsTree.collectNodesPostOrder();return e.concat(t).concat(i)}insert(e){ty(e)?this._injectedTextDecorationsTree.insert(e):sT(e)?this._decorationsTree1.insert(e):this._decorationsTree0.insert(e)}delete(e){ty(e)?this._injectedTextDecorationsTree.delete(e):sT(e)?this._decorationsTree1.delete(e):this._decorationsTree0.delete(e)}getNodeRange(e,t){const i=e.getVersionId();return t.cachedVersionId!==i&&this._resolveNode(t,i),t.range===null&&(t.range=e.getRangeAt(t.cachedAbsoluteStart,t.cachedAbsoluteEnd)),t.range}_resolveNode(e,t){ty(e)?this._injectedTextDecorationsTree.resolveNode(e,t):sT(e)?this._decorationsTree1.resolveNode(e,t):this._decorationsTree0.resolveNode(e,t)}acceptReplace(e,t,i,n){this._decorationsTree0.acceptReplace(e,t,i,n),this._decorationsTree1.acceptReplace(e,t,i,n),this._injectedTextDecorationsTree.acceptReplace(e,t,i,n)}}function el(o){return o.replace(/[^a-z0-9\-_]/gi," ")}class pK{constructor(e){this.color=e.color||"",this.darkColor=e.darkColor||""}}class lge extends pK{constructor(e){super(e),this._resolvedColor=null,this.position=typeof e.position=="number"?e.position:Ho.Center}getColor(e){return this._resolvedColor||(e.type!=="light"&&this.darkColor?this._resolvedColor=this._resolveColor(this.darkColor,e):this._resolvedColor=this._resolveColor(this.color,e)),this._resolvedColor}invalidateCachedColor(){this._resolvedColor=null}_resolveColor(e,t){if(typeof e=="string")return e;const i=e?t.getColor(e.id):null;return i?i.toString():""}}class cge{constructor(e){this.position=(e==null?void 0:e.position)??Zr.Center,this.persistLane=e==null?void 0:e.persistLane}}class dge extends pK{constructor(e){super(e),this.position=e.position,this.sectionHeaderStyle=e.sectionHeaderStyle??null,this.sectionHeaderText=e.sectionHeaderText??null}getColor(e){return this._resolvedColor||(e.type!=="light"&&this.darkColor?this._resolvedColor=this._resolveColor(this.darkColor,e):this._resolvedColor=this._resolveColor(this.color,e)),this._resolvedColor}invalidateCachedColor(){this._resolvedColor=void 0}_resolveColor(e,t){return typeof e=="string"?Z.fromHex(e):t.getColor(e.id)}}class Kh{static from(e){return e instanceof Kh?e:new Kh(e)}constructor(e){this.content=e.content||"",this.inlineClassName=e.inlineClassName||null,this.inlineClassNameAffectsLetterSpacing=e.inlineClassNameAffectsLetterSpacing||!1,this.attachedData=e.attachedData||null,this.cursorStops=e.cursorStops||null}}class We{static register(e){return new We(e)}static createDynamic(e){return new We(e)}constructor(e){this.description=e.description,this.blockClassName=e.blockClassName?el(e.blockClassName):null,this.blockDoesNotCollapse=e.blockDoesNotCollapse??null,this.blockIsAfterEnd=e.blockIsAfterEnd??null,this.blockPadding=e.blockPadding??null,this.stickiness=e.stickiness||0,this.zIndex=e.zIndex||0,this.className=e.className?el(e.className):null,this.shouldFillLineOnLineBreak=e.shouldFillLineOnLineBreak??null,this.hoverMessage=e.hoverMessage||null,this.glyphMarginHoverMessage=e.glyphMarginHoverMessage||null,this.lineNumberHoverMessage=e.lineNumberHoverMessage||null,this.isWholeLine=e.isWholeLine||!1,this.showIfCollapsed=e.showIfCollapsed||!1,this.collapseOnReplaceEdit=e.collapseOnReplaceEdit||!1,this.overviewRuler=e.overviewRuler?new lge(e.overviewRuler):null,this.minimap=e.minimap?new dge(e.minimap):null,this.glyphMargin=e.glyphMarginClassName?new cge(e.glyphMargin):null,this.glyphMarginClassName=e.glyphMarginClassName?el(e.glyphMarginClassName):null,this.linesDecorationsClassName=e.linesDecorationsClassName?el(e.linesDecorationsClassName):null,this.lineNumberClassName=e.lineNumberClassName?el(e.lineNumberClassName):null,this.linesDecorationsTooltip=e.linesDecorationsTooltip?Iee(e.linesDecorationsTooltip):null,this.firstLineDecorationClassName=e.firstLineDecorationClassName?el(e.firstLineDecorationClassName):null,this.marginClassName=e.marginClassName?el(e.marginClassName):null,this.inlineClassName=e.inlineClassName?el(e.inlineClassName):null,this.inlineClassNameAffectsLetterSpacing=e.inlineClassNameAffectsLetterSpacing||!1,this.beforeContentClassName=e.beforeContentClassName?el(e.beforeContentClassName):null,this.afterContentClassName=e.afterContentClassName?el(e.afterContentClassName):null,this.after=e.after?Kh.from(e.after):null,this.before=e.before?Kh.from(e.before):null,this.hideInCommentTokens=e.hideInCommentTokens??!1,this.hideInStringTokens=e.hideInStringTokens??!1}}We.EMPTY=We.register({description:"empty"});const A8=[We.register({description:"tracked-range-always-grows-when-typing-at-edges",stickiness:0}),We.register({description:"tracked-range-never-grows-when-typing-at-edges",stickiness:1}),We.register({description:"tracked-range-grows-only-when-typing-before",stickiness:2}),We.register({description:"tracked-range-grows-only-when-typing-after",stickiness:3})];function P8(o){return o instanceof We?o:We.createDynamic(o)}class hge extends F{constructor(e){super(),this.handleBeforeFire=e,this._actual=this._register(new P),this.event=this._actual.event,this._affectedInjectedTextLines=null,this._deferredCnt=0,this._shouldFireDeferred=!1,this._affectsMinimap=!1,this._affectsOverviewRuler=!1,this._affectsGlyphMargin=!1,this._affectsLineNumber=!1}beginDeferredEmit(){this._deferredCnt++}endDeferredEmit(){var e;this._deferredCnt--,this._deferredCnt===0&&(this._shouldFireDeferred&&this.doFire(),(e=this._affectedInjectedTextLines)==null||e.clear(),this._affectedInjectedTextLines=null)}recordLineAffectedByInjectedText(e){this._affectedInjectedTextLines||(this._affectedInjectedTextLines=new Set),this._affectedInjectedTextLines.add(e)}checkAffectedAndFire(e){var t,i;this._affectsMinimap||(this._affectsMinimap=!!((t=e.minimap)!=null&&t.position)),this._affectsOverviewRuler||(this._affectsOverviewRuler=!!((i=e.overviewRuler)!=null&&i.color)),this._affectsGlyphMargin||(this._affectsGlyphMargin=!!e.glyphMarginClassName),this._affectsLineNumber||(this._affectsLineNumber=!!e.lineNumberClassName),this.tryFire()}fire(){this._affectsMinimap=!0,this._affectsOverviewRuler=!0,this._affectsGlyphMargin=!0,this.tryFire()}tryFire(){this._deferredCnt===0?this.doFire():this._shouldFireDeferred=!0}doFire(){this.handleBeforeFire(this._affectedInjectedTextLines);const e={affectsMinimap:this._affectsMinimap,affectsOverviewRuler:this._affectsOverviewRuler,affectsGlyphMargin:this._affectsGlyphMargin,affectsLineNumber:this._affectsLineNumber};this._shouldFireDeferred=!1,this._affectsMinimap=!1,this._affectsOverviewRuler=!1,this._affectsGlyphMargin=!1,this._actual.fire(e)}}class uge extends F{constructor(){super(),this._fastEmitter=this._register(new P),this.fastEvent=this._fastEmitter.event,this._slowEmitter=this._register(new P),this.slowEvent=this._slowEmitter.event,this._deferredCnt=0,this._deferredEvent=null}beginDeferredEmit(){this._deferredCnt++}endDeferredEmit(e=null){if(this._deferredCnt--,this._deferredCnt===0&&this._deferredEvent!==null){this._deferredEvent.rawContentChangedEvent.resultingSelection=e;const t=this._deferredEvent;this._deferredEvent=null,this._fastEmitter.fire(t),this._slowEmitter.fire(t)}}fire(e){if(this._deferredCnt>0){this._deferredEvent?this._deferredEvent=this._deferredEvent.merge(e):this._deferredEvent=e;return}this._fastEmitter.fire(e),this._slowEmitter.fire(e)}}var gge=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},t0=function(o,e){return function(t,i){e(t,i,o)}},$m;function vu(o){return o.toString()}let fge=class{constructor(e,t,i){this.model=e,this._modelEventListeners=new K,this.model=e,this._modelEventListeners.add(e.onWillDispose(()=>t(e))),this._modelEventListeners.add(e.onDidChangeLanguage(n=>i(e,n)))}dispose(){this._modelEventListeners.dispose()}};const mge=ps||Je?1:2;class pge{constructor(e,t,i,n,s,r,a,l){this.uri=e,this.initialUndoRedoSnapshot=t,this.time=i,this.sharesUndoRedoStack=n,this.heapSize=s,this.sha1=r,this.versionId=a,this.alternativeVersionId=l}}var Ig;let gA=(Ig=class extends F{constructor(e,t,i,n){super(),this._configurationService=e,this._resourcePropertiesService=t,this._undoRedoService=i,this._instantiationService=n,this._onModelAdded=this._register(new P),this.onModelAdded=this._onModelAdded.event,this._onModelRemoved=this._register(new P),this.onModelRemoved=this._onModelRemoved.event,this._onModelModeChanged=this._register(new P),this.onModelLanguageChanged=this._onModelModeChanged.event,this._modelCreationOptionsByLanguageAndResource=Object.create(null),this._models={},this._disposedModels=new Map,this._disposedModelsHeapSize=0,this._register(this._configurationService.onDidChangeConfiguration(s=>this._updateModelOptions(s))),this._updateModelOptions(void 0)}static _readModelOptions(e,t){var u;let i=An.tabSize;if(e.editor&&typeof e.editor.tabSize<"u"){const g=parseInt(e.editor.tabSize,10);isNaN(g)||(i=g),i<1&&(i=1)}let n="tabSize";if(e.editor&&typeof e.editor.indentSize<"u"&&e.editor.indentSize!=="tabSize"){const g=parseInt(e.editor.indentSize,10);isNaN(g)||(n=Math.max(g,1))}let s=An.insertSpaces;e.editor&&typeof e.editor.insertSpaces<"u"&&(s=e.editor.insertSpaces==="false"?!1:!!e.editor.insertSpaces);let r=mge;const a=e.eol;a===`\r
`?r=2:a===`
`&&(r=1);let l=An.trimAutoWhitespace;e.editor&&typeof e.editor.trimAutoWhitespace<"u"&&(l=e.editor.trimAutoWhitespace==="false"?!1:!!e.editor.trimAutoWhitespace);let c=An.detectIndentation;e.editor&&typeof e.editor.detectIndentation<"u"&&(c=e.editor.detectIndentation==="false"?!1:!!e.editor.detectIndentation);let d=An.largeFileOptimizations;e.editor&&typeof e.editor.largeFileOptimizations<"u"&&(d=e.editor.largeFileOptimizations==="false"?!1:!!e.editor.largeFileOptimizations);let h=An.bracketPairColorizationOptions;return(u=e.editor)!=null&&u.bracketPairColorization&&typeof e.editor.bracketPairColorization=="object"&&(h={enabled:!!e.editor.bracketPairColorization.enabled,independentColorPoolPerBracketType:!!e.editor.bracketPairColorization.independentColorPoolPerBracketType}),{isForSimpleWidget:t,tabSize:i,indentSize:n,insertSpaces:s,detectIndentation:c,defaultEOL:r,trimAutoWhitespace:l,largeFileOptimizations:d,bracketPairColorizationOptions:h}}_getEOL(e,t){if(e)return this._resourcePropertiesService.getEOL(e,t);const i=this._configurationService.getValue("files.eol",{overrideIdentifier:t});return i&&typeof i=="string"&&i!=="auto"?i:so===3||so===2?`
`:`\r
`}_shouldRestoreUndoStack(){const e=this._configurationService.getValue("files.restoreUndoStack");return typeof e=="boolean"?e:!0}getCreationOptions(e,t,i){const n=typeof e=="string"?e:e.languageId;let s=this._modelCreationOptionsByLanguageAndResource[n+t];if(!s){const r=this._configurationService.getValue("editor",{overrideIdentifier:n,resource:t}),a=this._getEOL(t,n);s=$m._readModelOptions({editor:r,eol:a},i),this._modelCreationOptionsByLanguageAndResource[n+t]=s}return s}_updateModelOptions(e){const t=this._modelCreationOptionsByLanguageAndResource;this._modelCreationOptionsByLanguageAndResource=Object.create(null);const i=Object.keys(this._models);for(let n=0,s=i.length;n<s;n++){const r=i[n],a=this._models[r],l=a.model.getLanguageId(),c=a.model.uri;if(e&&!e.affectsConfiguration("editor",{overrideIdentifier:l,resource:c})&&!e.affectsConfiguration("files.eol",{overrideIdentifier:l,resource:c}))continue;const d=t[l+c],h=this.getCreationOptions(l,c,a.model.isForSimpleWidget);$m._setModelOptionsForModel(a.model,h,d)}}static _setModelOptionsForModel(e,t,i){i&&i.defaultEOL!==t.defaultEOL&&e.getLineCount()===1&&e.setEOL(t.defaultEOL===1?0:1),!(i&&i.detectIndentation===t.detectIndentation&&i.insertSpaces===t.insertSpaces&&i.tabSize===t.tabSize&&i.indentSize===t.indentSize&&i.trimAutoWhitespace===t.trimAutoWhitespace&&oo(i.bracketPairColorizationOptions,t.bracketPairColorizationOptions))&&(t.detectIndentation?(e.detectIndentation(t.insertSpaces,t.tabSize),e.updateOptions({trimAutoWhitespace:t.trimAutoWhitespace,bracketColorizationOptions:t.bracketPairColorizationOptions})):e.updateOptions({insertSpaces:t.insertSpaces,tabSize:t.tabSize,indentSize:t.indentSize,trimAutoWhitespace:t.trimAutoWhitespace,bracketColorizationOptions:t.bracketPairColorizationOptions}))}_insertDisposedModel(e){this._disposedModels.set(vu(e.uri),e),this._disposedModelsHeapSize+=e.heapSize}_removeDisposedModel(e){const t=this._disposedModels.get(vu(e));return t&&(this._disposedModelsHeapSize-=t.heapSize),this._disposedModels.delete(vu(e)),t}_ensureDisposedModelsHeapSize(e){if(this._disposedModelsHeapSize>e){const t=[];for(this._disposedModels.forEach(i=>{i.sharesUndoRedoStack||t.push(i)}),t.sort((i,n)=>i.time-n.time);t.length>0&&this._disposedModelsHeapSize>e;){const i=t.shift();this._removeDisposedModel(i.uri),i.initialUndoRedoSnapshot!==null&&this._undoRedoService.restoreSnapshot(i.initialUndoRedoSnapshot)}}}_createModelData(e,t,i,n){const s=this.getCreationOptions(t,i,n),r=this._instantiationService.createInstance(Da,e,t,s,i);if(i&&this._disposedModels.has(vu(i))){const c=this._removeDisposedModel(i),d=this._undoRedoService.getElements(i),h=this._getSHA1Computer(),u=h.canComputeSHA1(r)?h.computeSHA1(r)===c.sha1:!1;if(u||c.sharesUndoRedoStack){for(const g of d.past)yc(g)&&g.matchesResource(i)&&g.setModel(r);for(const g of d.future)yc(g)&&g.matchesResource(i)&&g.setModel(r);this._undoRedoService.setElementsValidFlag(i,!0,g=>yc(g)&&g.matchesResource(i)),u&&(r._overwriteVersionId(c.versionId),r._overwriteAlternativeVersionId(c.alternativeVersionId),r._overwriteInitialUndoRedoSnapshot(c.initialUndoRedoSnapshot))}else c.initialUndoRedoSnapshot!==null&&this._undoRedoService.restoreSnapshot(c.initialUndoRedoSnapshot)}const a=vu(r.uri);if(this._models[a])throw new Error("ModelService: Cannot add model because it already exists!");const l=new fge(r,c=>this._onWillDispose(c),(c,d)=>this._onDidChangeLanguage(c,d));return this._models[a]=l,l}createModel(e,t,i,n=!1){let s;return t?s=this._createModelData(e,t,i,n):s=this._createModelData(e,ms,i,n),this._onModelAdded.fire(s.model),s.model}getModels(){const e=[],t=Object.keys(this._models);for(let i=0,n=t.length;i<n;i++){const s=t[i];e.push(this._models[s].model)}return e}getModel(e){const t=vu(e),i=this._models[t];return i?i.model:null}_schemaShouldMaintainUndoRedoElements(e){return e.scheme===Fe.file||e.scheme===Fe.vscodeRemote||e.scheme===Fe.vscodeUserData||e.scheme===Fe.vscodeNotebookCell||e.scheme==="fake-fs"}_onWillDispose(e){const t=vu(e.uri),i=this._models[t],n=this._undoRedoService.getUriComparisonKey(e.uri)!==e.uri.toString();let s=!1,r=0;if(n||this._shouldRestoreUndoStack()&&this._schemaShouldMaintainUndoRedoElements(e.uri)){const c=this._undoRedoService.getElements(e.uri);if(c.past.length>0||c.future.length>0){for(const d of c.past)yc(d)&&d.matchesResource(e.uri)&&(s=!0,r+=d.heapSize(e.uri),d.setModel(e.uri));for(const d of c.future)yc(d)&&d.matchesResource(e.uri)&&(s=!0,r+=d.heapSize(e.uri),d.setModel(e.uri))}}const a=$m.MAX_MEMORY_FOR_CLOSED_FILES_UNDO_STACK,l=this._getSHA1Computer();if(s)if(!n&&(r>a||!l.canComputeSHA1(e))){const c=i.model.getInitialUndoRedoSnapshot();c!==null&&this._undoRedoService.restoreSnapshot(c)}else this._ensureDisposedModelsHeapSize(a-r),this._undoRedoService.setElementsValidFlag(e.uri,!1,c=>yc(c)&&c.matchesResource(e.uri)),this._insertDisposedModel(new pge(e.uri,i.model.getInitialUndoRedoSnapshot(),Date.now(),n,r,l.computeSHA1(e),e.getVersionId(),e.getAlternativeVersionId()));else if(!n){const c=i.model.getInitialUndoRedoSnapshot();c!==null&&this._undoRedoService.restoreSnapshot(c)}delete this._models[t],i.dispose(),delete this._modelCreationOptionsByLanguageAndResource[e.getLanguageId()+e.uri],this._onModelRemoved.fire(e)}_onDidChangeLanguage(e,t){const i=t.oldLanguage,n=e.getLanguageId(),s=this.getCreationOptions(i,e.uri,e.isForSimpleWidget),r=this.getCreationOptions(n,e.uri,e.isForSimpleWidget);$m._setModelOptionsForModel(e,r,s),this._onModelModeChanged.fire({model:e,oldLanguageId:i})}_getSHA1Computer(){return new fA}},$m=Ig,Ig.MAX_MEMORY_FOR_CLOSED_FILES_UNDO_STACK=20*1024*1024,Ig);gA=$m=gge([t0(0,qe),t0(1,u$),t0(2,y5),t0(3,we)],gA);const Kk=class Kk{canComputeSHA1(e){return e.getValueLength()<=Kk.MAX_MODEL_SIZE}computeSHA1(e){const t=new EM,i=e.createSnapshot();let n;for(;n=i.read();)t.update(n);return t.digest()}};Kk.MAX_MODEL_SIZE=10*1024*1024;let fA=Kk;var mA;(function(o){o[o.PRESERVE=0]="PRESERVE",o[o.LAST=1]="LAST"})(mA||(mA={}));const om={Quickaccess:"workbench.contributions.quickaccess"};class _ge{constructor(){this.providers=[],this.defaultProvider=void 0}registerQuickAccessProvider(e){return e.prefix.length===0?this.defaultProvider=e:this.providers.push(e),this.providers.sort((t,i)=>i.prefix.length-t.prefix.length),ve(()=>{this.providers.splice(this.providers.indexOf(e),1),this.defaultProvider===e&&(this.defaultProvider=void 0)})}getQuickAccessProviders(){return Kr([this.defaultProvider,...this.providers])}getQuickAccessProvider(e){return e&&this.providers.find(i=>e.startsWith(i.prefix))||void 0||this.defaultProvider}}ci.add(om.Quickaccess,new _ge);const bge={ctrlCmd:!1,alt:!1};var z_;(function(o){o[o.Blur=1]="Blur",o[o.Gesture=2]="Gesture",o[o.Other=3]="Other"})(z_||(z_={}));var pa;(function(o){o[o.NONE=0]="NONE",o[o.FIRST=1]="FIRST",o[o.SECOND=2]="SECOND",o[o.LAST=3]="LAST"})(pa||(pa={}));var zt;(function(o){o[o.First=1]="First",o[o.Second=2]="Second",o[o.Last=3]="Last",o[o.Next=4]="Next",o[o.Previous=5]="Previous",o[o.NextPage=6]="NextPage",o[o.PreviousPage=7]="PreviousPage",o[o.NextSeparator=8]="NextSeparator",o[o.PreviousSeparator=9]="PreviousSeparator"})(zt||(zt={}));var AL;(function(o){o[o.Title=1]="Title",o[o.Inline=2]="Inline"})(AL||(AL={}));const xr=et("quickInputService");var Cge=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},O8=function(o,e){return function(t,i){e(t,i,o)}};let pA=class extends F{constructor(e,t){super(),this.quickInputService=e,this.instantiationService=t,this.registry=ci.as(om.Quickaccess),this.mapProviderToDescriptor=new Map,this.lastAcceptedPickerValues=new Map,this.visibleQuickAccess=void 0}show(e="",t){this.doShowOrPick(e,!1,t)}doShowOrPick(e,t,i){var f,p;const[n,s]=this.getOrInstantiateProvider(e,i==null?void 0:i.enabledProviderPrefixes),r=this.visibleQuickAccess,a=r==null?void 0:r.descriptor;if(r&&s&&a===s){e!==s.prefix&&!(i!=null&&i.preserveValue)&&(r.picker.value=e),this.adjustValueSelection(r.picker,s,i);return}if(s&&!(i!=null&&i.preserveValue)){let _;if(r&&a&&a!==s){const b=r.value.substr(a.prefix.length);b&&(_=`${s.prefix}${b}`)}if(!_){const b=n==null?void 0:n.defaultFilterValue;b===mA.LAST?_=this.lastAcceptedPickerValues.get(s):typeof b=="string"&&(_=`${s.prefix}${b}`)}typeof _=="string"&&(e=_)}const l=(f=r==null?void 0:r.picker)==null?void 0:f.valueSelection,c=(p=r==null?void 0:r.picker)==null?void 0:p.value,d=new K,h=d.add(this.quickInputService.createQuickPick({useSeparators:!0}));h.value=e,this.adjustValueSelection(h,s,i),h.placeholder=(i==null?void 0:i.placeholder)??(s==null?void 0:s.placeholder),h.quickNavigate=i==null?void 0:i.quickNavigateConfiguration,h.hideInput=!!h.quickNavigate&&!r,(typeof(i==null?void 0:i.itemActivation)=="number"||i!=null&&i.quickNavigateConfiguration)&&(h.itemActivation=(i==null?void 0:i.itemActivation)??pa.SECOND),h.contextKey=s==null?void 0:s.contextKey,h.filterValue=_=>_.substring(s?s.prefix.length:0);let u;t&&(u=new gb,d.add(oe.once(h.onWillAccept)(_=>{_.veto(),h.hide()}))),d.add(this.registerPickerListeners(h,n,s,e,i));const g=d.add(new li);if(n&&d.add(n.provide(h,g.token,i==null?void 0:i.providerOptions)),oe.once(h.onDidHide)(()=>{h.selectedItems.length===0&&g.cancel(),d.dispose(),u==null||u.complete(h.selectedItems.slice(0))}),h.show(),l&&c===e&&(h.valueSelection=l),t)return u==null?void 0:u.p}adjustValueSelection(e,t,i){let n;i!=null&&i.preserveValue?n=[e.value.length,e.value.length]:n=[(t==null?void 0:t.prefix.length)??0,e.value.length],e.valueSelection=n}registerPickerListeners(e,t,i,n,s){const r=new K,a=this.visibleQuickAccess={picker:e,descriptor:i,value:n};return r.add(ve(()=>{a===this.visibleQuickAccess&&(this.visibleQuickAccess=void 0)})),r.add(e.onDidChangeValue(l=>{const[c]=this.getOrInstantiateProvider(l,s==null?void 0:s.enabledProviderPrefixes);c!==t?this.show(l,{enabledProviderPrefixes:s==null?void 0:s.enabledProviderPrefixes,preserveValue:!0,providerOptions:s==null?void 0:s.providerOptions}):a.value=l})),i&&r.add(e.onDidAccept(()=>{this.lastAcceptedPickerValues.set(i,e.value)})),r}getOrInstantiateProvider(e,t){const i=this.registry.getQuickAccessProvider(e);if(!i||t&&!(t!=null&&t.includes(i.prefix)))return[void 0,void 0];let n=this.mapProviderToDescriptor.get(i);return n||(n=this.instantiationService.createInstance(i.ctor),this.mapProviderToDescriptor.set(i,n)),[n,i]}};pA=Cge([O8(0,xr),O8(1,we)],pA);class kb extends Yo{constructor(e){super(),this._onChange=this._register(new P),this.onChange=this._onChange.event,this._onKeyDown=this._register(new P),this.onKeyDown=this._onKeyDown.event,this._opts=e,this._checked=this._opts.isChecked;const t=["monaco-custom-toggle"];this._opts.icon&&(this._icon=this._opts.icon,t.push(...Ie.asClassNameArray(this._icon))),this._opts.actionClassName&&t.push(...this._opts.actionClassName.split(" ")),this._checked&&t.push("checked"),this.domNode=document.createElement("div"),this._hover=this._register(ja().setupManagedHover(e.hoverDelegate??Ss("mouse"),this.domNode,this._opts.title)),this.domNode.classList.add(...t),this._opts.notFocusable||(this.domNode.tabIndex=0),this.domNode.setAttribute("role","checkbox"),this.domNode.setAttribute("aria-checked",String(this._checked)),this.domNode.setAttribute("aria-label",this._opts.title),this.applyStyles(),this.onclick(this.domNode,i=>{this.enabled&&(this.checked=!this._checked,this._onChange.fire(!1),i.preventDefault())}),this._register(this.ignoreGesture(this.domNode)),this.onkeydown(this.domNode,i=>{if(i.keyCode===10||i.keyCode===3){this.checked=!this._checked,this._onChange.fire(!0),i.preventDefault(),i.stopPropagation();return}this._onKeyDown.fire(i)})}get enabled(){return this.domNode.getAttribute("aria-disabled")!=="true"}focus(){this.domNode.focus()}get checked(){return this._checked}set checked(e){this._checked=e,this.domNode.setAttribute("aria-checked",String(this._checked)),this.domNode.classList.toggle("checked",this._checked),this.applyStyles()}width(){return 22}applyStyles(){this.domNode&&(this.domNode.style.borderColor=this._checked&&this._opts.inputActiveOptionBorder||"",this.domNode.style.color=this._checked&&this._opts.inputActiveOptionForeground||"inherit",this.domNode.style.backgroundColor=this._checked&&this._opts.inputActiveOptionBackground||"")}enable(){this.domNode.setAttribute("aria-disabled",String(!1))}disable(){this.domNode.setAttribute("aria-disabled",String(!0))}}var vge=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s};class _K{constructor(e){this.nodes=e}toString(){return this.nodes.map(e=>typeof e=="string"?e:e.label).join("")}}vge([Ni],_K.prototype,"toString",null);const wge=/\[([^\]]+)\]\(((?:https?:\/\/|command:|file:)[^\)\s]+)(?: (["'])(.+?)(\3))?\)/gi;function Sge(o){const e=[];let t=0,i;for(;i=wge.exec(o);){i.index-t>0&&e.push(o.substring(t,i.index));const[,n,s,,r]=i;r?e.push({label:n,href:s,title:r}):e.push({label:n,href:s}),t=i.index+i[0].length}return t<o.length&&e.push(o.substring(t)),new _K(e)}const oT={},yge=new q5("quick-input-button-icon-");function Lge(o){if(!o)return;let e;const t=o.dark.toString();return oT[t]?e=oT[t]:(e=yge.nextId(),Ky(`.${e}, .hc-light .${e}`,`background-image: ${td(o.light||o.dark)}`),Ky(`.vs-dark .${e}, .hc-black .${e}`,`background-image: ${td(o.dark)}`),oT[t]=e),e}function ov(o,e,t){let i=o.iconClass||Lge(o.iconPath);return o.alwaysVisible&&(i=i?`${i} always-visible`:"always-visible"),{id:e,label:"",tooltip:o.tooltip||"",class:i,enabled:!0,run:t}}function xge(o,e,t){_n(e);const i=Sge(o);let n=0;for(const s of i.nodes)if(typeof s=="string")e.append(...dh(s));else{let r=s.title;!r&&s.href.startsWith("command:")?r=m("executeCommand","Click to execute command '{0}'",s.href.substring(8)):r||(r=s.href);const a=ne("a",{href:s.href,title:r,tabIndex:n++},s.label);a.style.textDecoration="underline";const l=g=>{qte(g)&&nt.stop(g,!0),t.callback(s.href)},c=t.disposables.add(new tt(a,ie.CLICK)).event,d=t.disposables.add(new tt(a,ie.KEY_DOWN)).event,h=oe.chain(d,g=>g.filter(f=>{const p=new Pt(f);return p.equals(10)||p.equals(3)}));t.disposables.add(Fn.addTarget(a));const u=t.disposables.add(new tt(a,Bt.Tap)).event;oe.any(c,u,h)(l,null,t.disposables),e.appendChild(a)}}var kge=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},F8=function(o,e){return function(t,i){e(t,i,o)}};const bK="inQuickInput",Dge=new le(bK,!1,m("inQuickInput","Whether keyboard focus is inside the quick input control")),Ige=G.has(bK),CK="quickInputType",Ege=new le(CK,void 0,m("quickInputType","The type of the currently visible quick input")),vK="cursorAtEndOfQuickInputBox",Nge=new le(vK,!1,m("cursorAtEndOfQuickInputBox","Whether the cursor in the quick input is at the end of the input box")),Tge=G.has(vK),_A={iconClass:Ie.asClassName(te.quickInputBack),tooltip:m("quickInput.back","Back")},qk=class qk extends F{constructor(e){super(),this.ui=e,this._widgetUpdated=!1,this.visible=!1,this._enabled=!0,this._busy=!1,this._ignoreFocusOut=!1,this._leftButtons=[],this._rightButtons=[],this._inlineButtons=[],this.buttonsUpdated=!1,this._toggles=[],this.togglesUpdated=!1,this.noValidationMessage=qk.noPromptMessage,this._severity=Ii.Ignore,this.onDidTriggerButtonEmitter=this._register(new P),this.onDidHideEmitter=this._register(new P),this.onWillHideEmitter=this._register(new P),this.onDisposeEmitter=this._register(new P),this.visibleDisposables=this._register(new K),this.onDidHide=this.onDidHideEmitter.event}get title(){return this._title}set title(e){this._title=e,this.update()}get description(){return this._description}set description(e){this._description=e,this.update()}get step(){return this._steps}set step(e){this._steps=e,this.update()}get totalSteps(){return this._totalSteps}set totalSteps(e){this._totalSteps=e,this.update()}get enabled(){return this._enabled}set enabled(e){this._enabled=e,this.update()}get contextKey(){return this._contextKey}set contextKey(e){this._contextKey=e,this.update()}get busy(){return this._busy}set busy(e){this._busy=e,this.update()}get ignoreFocusOut(){return this._ignoreFocusOut}set ignoreFocusOut(e){const t=this._ignoreFocusOut!==e&&!Pa;this._ignoreFocusOut=e&&!Pa,t&&this.update()}get titleButtons(){return this._leftButtons.length?[...this._leftButtons,this._rightButtons]:this._rightButtons}get buttons(){return[...this._leftButtons,...this._rightButtons,...this._inlineButtons]}set buttons(e){this._leftButtons=e.filter(t=>t===_A),this._rightButtons=e.filter(t=>t!==_A&&t.location!==AL.Inline),this._inlineButtons=e.filter(t=>t.location===AL.Inline),this.buttonsUpdated=!0,this.update()}get toggles(){return this._toggles}set toggles(e){this._toggles=e??[],this.togglesUpdated=!0,this.update()}get validationMessage(){return this._validationMessage}set validationMessage(e){this._validationMessage=e,this.update()}get severity(){return this._severity}set severity(e){this._severity=e,this.update()}show(){this.visible||(this.visibleDisposables.add(this.ui.onDidTriggerButton(e=>{this.buttons.indexOf(e)!==-1&&this.onDidTriggerButtonEmitter.fire(e)})),this.ui.show(this),this.visible=!0,this._lastValidationMessage=void 0,this._lastSeverity=void 0,this.buttons.length&&(this.buttonsUpdated=!0),this.toggles.length&&(this.togglesUpdated=!0),this.update())}hide(){this.visible&&this.ui.hide()}didHide(e=z_.Other){this.visible=!1,this.visibleDisposables.clear(),this.onDidHideEmitter.fire({reason:e})}willHide(e=z_.Other){this.onWillHideEmitter.fire({reason:e})}update(){var n;if(!this.visible)return;const e=this.getTitle();e&&this.ui.title.textContent!==e?this.ui.title.textContent=e:!e&&this.ui.title.innerHTML!=="&nbsp;"&&(this.ui.title.innerText=" ");const t=this.getDescription();if(this.ui.description1.textContent!==t&&(this.ui.description1.textContent=t),this.ui.description2.textContent!==t&&(this.ui.description2.textContent=t),this._widgetUpdated&&(this._widgetUpdated=!1,this._widget?_n(this.ui.widget,this._widget):_n(this.ui.widget)),this.busy&&!this.busyDelay&&(this.busyDelay=new ea,this.busyDelay.setIfNotSet(()=>{this.visible&&this.ui.progressBar.infinite()},800)),!this.busy&&this.busyDelay&&(this.ui.progressBar.stop(),this.busyDelay.cancel(),this.busyDelay=void 0),this.buttonsUpdated){this.buttonsUpdated=!1,this.ui.leftActionBar.clear();const s=this._leftButtons.map((l,c)=>ov(l,`id-${c}`,async()=>this.onDidTriggerButtonEmitter.fire(l)));this.ui.leftActionBar.push(s,{icon:!0,label:!1}),this.ui.rightActionBar.clear();const r=this._rightButtons.map((l,c)=>ov(l,`id-${c}`,async()=>this.onDidTriggerButtonEmitter.fire(l)));this.ui.rightActionBar.push(r,{icon:!0,label:!1}),this.ui.inlineActionBar.clear();const a=this._inlineButtons.map((l,c)=>ov(l,`id-${c}`,async()=>this.onDidTriggerButtonEmitter.fire(l)));this.ui.inlineActionBar.push(a,{icon:!0,label:!1})}if(this.togglesUpdated){this.togglesUpdated=!1;const s=((n=this.toggles)==null?void 0:n.filter(r=>r instanceof kb))??[];this.ui.inputBox.toggles=s}this.ui.ignoreFocusOut=this.ignoreFocusOut,this.ui.setEnabled(this.enabled),this.ui.setContextKey(this.contextKey);const i=this.validationMessage||this.noValidationMessage;this._lastValidationMessage!==i&&(this._lastValidationMessage=i,_n(this.ui.message),xge(i,this.ui.message,{callback:s=>{this.ui.linkOpenerDelegate(s)},disposables:this.visibleDisposables})),this._lastSeverity!==this.severity&&(this._lastSeverity=this.severity,this.showMessageDecoration(this.severity))}getTitle(){return this.title&&this.step?`${this.title} (${this.getSteps()})`:this.title?this.title:this.step?this.getSteps():""}getDescription(){return this.description||""}getSteps(){return this.step&&this.totalSteps?m("quickInput.steps","{0}/{1}",this.step,this.totalSteps):this.step?String(this.step):""}showMessageDecoration(e){if(this.ui.inputBox.showDecoration(e),e!==Ii.Ignore){const t=this.ui.inputBox.stylesForType(e);this.ui.message.style.color=t.foreground?`${t.foreground}`:"",this.ui.message.style.backgroundColor=t.background?`${t.background}`:"",this.ui.message.style.border=t.border?`1px solid ${t.border}`:"",this.ui.message.style.marginBottom="-2px"}else this.ui.message.style.color="",this.ui.message.style.backgroundColor="",this.ui.message.style.border="",this.ui.message.style.marginBottom=""}dispose(){this.hide(),this.onDisposeEmitter.fire(),super.dispose()}};qk.noPromptMessage=m("inputModeEntry","Press 'Enter' to confirm your input or 'Escape' to cancel");let PL=qk;const Gk=class Gk extends PL{constructor(){super(...arguments),this._value="",this.onDidChangeValueEmitter=this._register(new P),this.onWillAcceptEmitter=this._register(new P),this.onDidAcceptEmitter=this._register(new P),this.onDidCustomEmitter=this._register(new P),this._items=[],this.itemsUpdated=!1,this._canSelectMany=!1,this._canAcceptInBackground=!1,this._matchOnDescription=!1,this._matchOnDetail=!1,this._matchOnLabel=!0,this._matchOnLabelMode="fuzzy",this._sortByLabel=!0,this._keepScrollPosition=!1,this._itemActivation=pa.FIRST,this._activeItems=[],this.activeItemsUpdated=!1,this.activeItemsToConfirm=[],this.onDidChangeActiveEmitter=this._register(new P),this._selectedItems=[],this.selectedItemsUpdated=!1,this.selectedItemsToConfirm=[],this.onDidChangeSelectionEmitter=this._register(new P),this.onDidTriggerItemButtonEmitter=this._register(new P),this.onDidTriggerSeparatorButtonEmitter=this._register(new P),this.valueSelectionUpdated=!0,this._ok="default",this._customButton=!1,this._focusEventBufferer=new R1,this.type="quickPick",this.filterValue=e=>e,this.onDidChangeValue=this.onDidChangeValueEmitter.event,this.onWillAccept=this.onWillAcceptEmitter.event,this.onDidAccept=this.onDidAcceptEmitter.event,this.onDidChangeActive=this.onDidChangeActiveEmitter.event,this.onDidChangeSelection=this.onDidChangeSelectionEmitter.event,this.onDidTriggerItemButton=this.onDidTriggerItemButtonEmitter.event,this.onDidTriggerSeparatorButton=this.onDidTriggerSeparatorButtonEmitter.event}get quickNavigate(){return this._quickNavigate}set quickNavigate(e){this._quickNavigate=e,this.update()}get value(){return this._value}set value(e){this.doSetValue(e)}doSetValue(e,t){this._value!==e&&(this._value=e,t||this.update(),this.visible&&this.ui.list.filter(this.filterValue(this._value))&&this.trySelectFirst(),this.onDidChangeValueEmitter.fire(this._value))}set ariaLabel(e){this._ariaLabel=e,this.update()}get ariaLabel(){return this._ariaLabel}get placeholder(){return this._placeholder}set placeholder(e){this._placeholder=e,this.update()}get items(){return this._items}get scrollTop(){return this.ui.list.scrollTop}set scrollTop(e){this.ui.list.scrollTop=e}set items(e){this._items=e,this.itemsUpdated=!0,this.update()}get canSelectMany(){return this._canSelectMany}set canSelectMany(e){this._canSelectMany=e,this.update()}get canAcceptInBackground(){return this._canAcceptInBackground}set canAcceptInBackground(e){this._canAcceptInBackground=e}get matchOnDescription(){return this._matchOnDescription}set matchOnDescription(e){this._matchOnDescription=e,this.update()}get matchOnDetail(){return this._matchOnDetail}set matchOnDetail(e){this._matchOnDetail=e,this.update()}get matchOnLabel(){return this._matchOnLabel}set matchOnLabel(e){this._matchOnLabel=e,this.update()}get matchOnLabelMode(){return this._matchOnLabelMode}set matchOnLabelMode(e){this._matchOnLabelMode=e,this.update()}get sortByLabel(){return this._sortByLabel}set sortByLabel(e){this._sortByLabel=e,this.update()}get keepScrollPosition(){return this._keepScrollPosition}set keepScrollPosition(e){this._keepScrollPosition=e}get itemActivation(){return this._itemActivation}set itemActivation(e){this._itemActivation=e}get activeItems(){return this._activeItems}set activeItems(e){this._activeItems=e,this.activeItemsUpdated=!0,this.update()}get selectedItems(){return this._selectedItems}set selectedItems(e){this._selectedItems=e,this.selectedItemsUpdated=!0,this.update()}get keyMods(){return this._quickNavigate?bge:this.ui.keyMods}get valueSelection(){const e=this.ui.inputBox.getSelection();if(e)return[e.start,e.end]}set valueSelection(e){this._valueSelection=e,this.valueSelectionUpdated=!0,this.update()}get customButton(){return this._customButton}set customButton(e){this._customButton=e,this.update()}get customLabel(){return this._customButtonLabel}set customLabel(e){this._customButtonLabel=e,this.update()}get customHover(){return this._customButtonHover}set customHover(e){this._customButtonHover=e,this.update()}get ok(){return this._ok}set ok(e){this._ok=e,this.update()}get hideInput(){return!!this._hideInput}set hideInput(e){this._hideInput=e,this.update()}trySelectFirst(){this.canSelectMany||this.ui.list.focus(zt.First)}show(){this.visible||(this.visibleDisposables.add(this.ui.inputBox.onDidChange(e=>{this.doSetValue(e,!0)})),this.visibleDisposables.add(this.ui.onDidAccept(()=>{this.canSelectMany?this.ui.list.getCheckedElements().length||(this._selectedItems=[],this.onDidChangeSelectionEmitter.fire(this.selectedItems)):this.activeItems[0]&&(this._selectedItems=[this.activeItems[0]],this.onDidChangeSelectionEmitter.fire(this.selectedItems)),this.handleAccept(!1)})),this.visibleDisposables.add(this.ui.onDidCustom(()=>{this.onDidCustomEmitter.fire()})),this.visibleDisposables.add(this._focusEventBufferer.wrapEvent(this.ui.list.onDidChangeFocus,(e,t)=>t)(e=>{this.activeItemsUpdated||this.activeItemsToConfirm!==this._activeItems&&ii(e,this._activeItems,(t,i)=>t===i)||(this._activeItems=e,this.onDidChangeActiveEmitter.fire(e))})),this.visibleDisposables.add(this.ui.list.onDidChangeSelection(({items:e,event:t})=>{if(this.canSelectMany){e.length&&this.ui.list.setSelectedElements([]);return}this.selectedItemsToConfirm!==this._selectedItems&&ii(e,this._selectedItems,(i,n)=>i===n)||(this._selectedItems=e,this.onDidChangeSelectionEmitter.fire(e),e.length&&this.handleAccept(l5(t)&&t.button===1))})),this.visibleDisposables.add(this.ui.list.onChangedCheckedElements(e=>{!this.canSelectMany||!this.visible||this.selectedItemsToConfirm!==this._selectedItems&&ii(e,this._selectedItems,(t,i)=>t===i)||(this._selectedItems=e,this.onDidChangeSelectionEmitter.fire(e))})),this.visibleDisposables.add(this.ui.list.onButtonTriggered(e=>this.onDidTriggerItemButtonEmitter.fire(e))),this.visibleDisposables.add(this.ui.list.onSeparatorButtonTriggered(e=>this.onDidTriggerSeparatorButtonEmitter.fire(e))),this.visibleDisposables.add(this.registerQuickNavigation()),this.valueSelectionUpdated=!0),super.show()}handleAccept(e){let t=!1;this.onWillAcceptEmitter.fire({veto:()=>t=!0}),t||this.onDidAcceptEmitter.fire({inBackground:e})}registerQuickNavigation(){return U(this.ui.container,ie.KEY_UP,e=>{if(this.canSelectMany||!this._quickNavigate)return;const t=new Pt(e),i=t.keyCode;this._quickNavigate.keybindings.some(r=>{const a=r.getChords();return a.length>1?!1:a[0].shiftKey&&i===4?!(t.ctrlKey||t.altKey||t.metaKey):!!(a[0].altKey&&i===6||a[0].ctrlKey&&i===5||a[0].metaKey&&i===57)})&&(this.activeItems[0]&&(this._selectedItems=[this.activeItems[0]],this.onDidChangeSelectionEmitter.fire(this.selectedItems),this.handleAccept(!1)),this._quickNavigate=void 0)})}update(){if(!this.visible)return;const e=this.keepScrollPosition?this.scrollTop:0,t=!!this.description,i={title:!!this.title||!!this.step||!!this.titleButtons.length,description:t,checkAll:this.canSelectMany&&!this._hideCheckAll,checkBox:this.canSelectMany,inputBox:!this._hideInput,progressBar:!this._hideInput||t,visibleCount:!0,count:this.canSelectMany&&!this._hideCountBadge,ok:this.ok==="default"?this.canSelectMany:this.ok,list:!0,message:!!this.validationMessage,customButton:this.customButton};this.ui.setVisibilities(i),super.update(),this.ui.inputBox.value!==this.value&&(this.ui.inputBox.value=this.value),this.valueSelectionUpdated&&(this.valueSelectionUpdated=!1,this.ui.inputBox.select(this._valueSelection&&{start:this._valueSelection[0],end:this._valueSelection[1]})),this.ui.inputBox.placeholder!==(this.placeholder||"")&&(this.ui.inputBox.placeholder=this.placeholder||"");let n=this.ariaLabel;!n&&i.inputBox&&(n=this.placeholder||Gk.DEFAULT_ARIA_LABEL,this.title&&(n+=` - ${this.title}`)),this.ui.list.ariaLabel!==n&&(this.ui.list.ariaLabel=n??null),this.ui.list.matchOnDescription=this.matchOnDescription,this.ui.list.matchOnDetail=this.matchOnDetail,this.ui.list.matchOnLabel=this.matchOnLabel,this.ui.list.matchOnLabelMode=this.matchOnLabelMode,this.ui.list.sortByLabel=this.sortByLabel,this.itemsUpdated&&(this.itemsUpdated=!1,this._focusEventBufferer.bufferEvents(()=>{switch(this.ui.list.setElements(this.items),this.ui.list.shouldLoop=!this.canSelectMany,this.ui.list.filter(this.filterValue(this.ui.inputBox.value)),this._itemActivation){case pa.NONE:this._itemActivation=pa.FIRST;break;case pa.SECOND:this.ui.list.focus(zt.Second),this._itemActivation=pa.FIRST;break;case pa.LAST:this.ui.list.focus(zt.Last),this._itemActivation=pa.FIRST;break;default:this.trySelectFirst();break}})),this.ui.container.classList.contains("show-checkboxes")!==!!this.canSelectMany&&(this.canSelectMany?this.ui.list.clearFocus():this.trySelectFirst()),this.activeItemsUpdated&&(this.activeItemsUpdated=!1,this.activeItemsToConfirm=this._activeItems,this.ui.list.setFocusedElements(this.activeItems),this.activeItemsToConfirm===this._activeItems&&(this.activeItemsToConfirm=null)),this.selectedItemsUpdated&&(this.selectedItemsUpdated=!1,this.selectedItemsToConfirm=this._selectedItems,this.canSelectMany?this.ui.list.setCheckedElements(this.selectedItems):this.ui.list.setSelectedElements(this.selectedItems),this.selectedItemsToConfirm===this._selectedItems&&(this.selectedItemsToConfirm=null)),this.ui.customButton.label=this.customLabel||"",this.ui.customButton.element.title=this.customHover||"",i.inputBox||(this.ui.list.domFocus(),this.canSelectMany&&this.ui.list.focus(zt.First)),this.keepScrollPosition&&(this.scrollTop=e)}focus(e){this.ui.list.focus(e),this.canSelectMany&&this.ui.list.domFocus()}accept(e){e&&!this._canAcceptInBackground||this.activeItems[0]&&(this._selectedItems=[this.activeItems[0]],this.onDidChangeSelectionEmitter.fire(this.selectedItems),this.handleAccept(e??!1))}};Gk.DEFAULT_ARIA_LABEL=m("quickInputBox.ariaLabel","Type to narrow down results.");let OL=Gk,Rge=class extends PL{constructor(){super(...arguments),this._value="",this.valueSelectionUpdated=!0,this._password=!1,this.onDidValueChangeEmitter=this._register(new P),this.onDidAcceptEmitter=this._register(new P),this.type="inputBox",this.onDidChangeValue=this.onDidValueChangeEmitter.event,this.onDidAccept=this.onDidAcceptEmitter.event}get value(){return this._value}set value(e){this._value=e||"",this.update()}get placeholder(){return this._placeholder}set placeholder(e){this._placeholder=e,this.update()}get password(){return this._password}set password(e){this._password=e,this.update()}show(){this.visible||(this.visibleDisposables.add(this.ui.inputBox.onDidChange(e=>{e!==this.value&&(this._value=e,this.onDidValueChangeEmitter.fire(e))})),this.visibleDisposables.add(this.ui.onDidAccept(()=>this.onDidAcceptEmitter.fire())),this.valueSelectionUpdated=!0),super.show()}update(){if(!this.visible)return;this.ui.container.classList.remove("hidden-input");const e={title:!!this.title||!!this.step||!!this.titleButtons.length,description:!!this.description||!!this.step,inputBox:!0,message:!0,progressBar:!0};this.ui.setVisibilities(e),super.update(),this.ui.inputBox.value!==this.value&&(this.ui.inputBox.value=this.value),this.valueSelectionUpdated&&(this.valueSelectionUpdated=!1,this.ui.inputBox.select(this._valueSelection&&{start:this._valueSelection[0],end:this._valueSelection[1]})),this.ui.inputBox.placeholder!==(this.placeholder||"")&&(this.ui.inputBox.placeholder=this.placeholder||""),this.ui.inputBox.password!==this.password&&(this.ui.inputBox.password=this.password)}},bA=class extends M_{constructor(e,t){super("element",!1,i=>this.getOverrideOptions(i),e,t)}getOverrideOptions(e){const t=(Xi(e.content)?e.content.textContent??"":typeof e.content=="string"?e.content:e.content.value).includes(`
`);return{persistence:{hideOnKeyDown:!1},appearance:{showHoverHint:t,skipFadeInAnimation:!0}}}};bA=kge([F8(0,qe),F8(1,Yl)],bA);Z.white.toString(),Z.white.toString();class FL extends F{get onDidClick(){return this._onDidClick.event}constructor(e,t){super(),this._label="",this._onDidClick=this._register(new P),this._onDidEscape=this._register(new P),this.options=t,this._element=document.createElement("a"),this._element.classList.add("monaco-button"),this._element.tabIndex=0,this._element.setAttribute("role","button"),this._element.classList.toggle("secondary",!!t.secondary);const i=t.secondary?t.buttonSecondaryBackground:t.buttonBackground,n=t.secondary?t.buttonSecondaryForeground:t.buttonForeground;this._element.style.color=n||"",this._element.style.backgroundColor=i||"",t.supportShortLabel&&(this._labelShortElement=document.createElement("div"),this._labelShortElement.classList.add("monaco-button-label-short"),this._element.appendChild(this._labelShortElement),this._labelElement=document.createElement("div"),this._labelElement.classList.add("monaco-button-label"),this._element.appendChild(this._labelElement),this._element.classList.add("monaco-text-button-with-short-label")),typeof t.title=="string"&&this.setTitle(t.title),typeof t.ariaLabel=="string"&&this._element.setAttribute("aria-label",t.ariaLabel),e.appendChild(this._element),this._register(Fn.addTarget(this._element)),[ie.CLICK,Bt.Tap].forEach(s=>{this._register(U(this._element,s,r=>{if(!this.enabled){nt.stop(r);return}this._onDidClick.fire(r)}))}),this._register(U(this._element,ie.KEY_DOWN,s=>{const r=new Pt(s);let a=!1;this.enabled&&(r.equals(3)||r.equals(10))?(this._onDidClick.fire(s),a=!0):r.equals(9)&&(this._onDidEscape.fire(s),this._element.blur(),a=!0),a&&nt.stop(r,!0)})),this._register(U(this._element,ie.MOUSE_OVER,s=>{this._element.classList.contains("disabled")||this.updateBackground(!0)})),this._register(U(this._element,ie.MOUSE_OUT,s=>{this.updateBackground(!1)})),this.focusTracker=this._register(Fa(this._element)),this._register(this.focusTracker.onDidFocus(()=>{this.enabled&&this.updateBackground(!0)})),this._register(this.focusTracker.onDidBlur(()=>{this.enabled&&this.updateBackground(!1)}))}dispose(){super.dispose(),this._element.remove()}getContentElements(e){const t=[];for(let i of dh(e))if(typeof i=="string"){if(i=i.trim(),i==="")continue;const n=document.createElement("span");n.textContent=i,t.push(n)}else t.push(i);return t}updateBackground(e){let t;this.options.secondary?t=e?this.options.buttonSecondaryHoverBackground:this.options.buttonSecondaryBackground:t=e?this.options.buttonHoverBackground:this.options.buttonBackground,t&&(this._element.style.backgroundColor=t)}get element(){return this._element}set label(e){var n;if(this._label===e||Aa(this._label)&&Aa(e)&&hle(this._label,e))return;this._element.classList.add("monaco-text-button");const t=this.options.supportShortLabel?this._labelElement:this._element;if(Aa(e)){const s=YI(e,{inline:!0});s.dispose();const r=(n=s.element.querySelector("p"))==null?void 0:n.innerHTML;if(r){const a=bU(r,{ADD_TAGS:["b","i","u","code","span"],ALLOWED_ATTR:["class"],RETURN_TRUSTED_TYPE:!0});t.innerHTML=a}else _n(t)}else this.options.supportIcons?_n(t,...this.getContentElements(e)):t.textContent=e;let i="";typeof this.options.title=="string"?i=this.options.title:this.options.title&&(i=Sle(e)),this.setTitle(i),typeof this.options.ariaLabel=="string"?this._element.setAttribute("aria-label",this.options.ariaLabel):this.options.ariaLabel&&this._element.setAttribute("aria-label",i),this._label=e}get label(){return this._label}set icon(e){this._element.classList.add(...Ie.asClassNameArray(e))}set enabled(e){e?(this._element.classList.remove("disabled"),this._element.setAttribute("aria-disabled",String(!1)),this._element.tabIndex=0):(this._element.classList.add("disabled"),this._element.setAttribute("aria-disabled",String(!0)))}get enabled(){return!this._element.classList.contains("disabled")}setTitle(e){!this._hover&&e!==""?this._hover=this._register(ja().setupManagedHover(this.options.hoverDelegate??Ss("mouse"),this._element,e)):this._hover&&this._hover.update(e)}}class CA{constructor(e,t,i){this.options=t,this.styles=i,this.count=0,this.element=X(e,ne(".monaco-count-badge")),this.countFormat=this.options.countFormat||"{0}",this.titleFormat=this.options.titleFormat||"",this.setCount(this.options.count||0)}setCount(e){this.count=e,this.render()}setTitleFormat(e){this.titleFormat=e,this.render()}render(){this.element.textContent=xf(this.countFormat,this.count),this.element.title=xf(this.titleFormat,this.count),this.element.style.backgroundColor=this.styles.badgeBackground??"",this.element.style.color=this.styles.badgeForeground??"",this.styles.badgeBorder&&(this.element.style.border=`1px solid ${this.styles.badgeBorder}`)}}const W8="done",B8="active",rT="infinite",aT="infinite-long-running",H8="discrete",Zk=class Zk extends F{constructor(e,t){super(),this.progressSignal=this._register(new Ji),this.workedVal=0,this.showDelayedScheduler=this._register(new It(()=>to(this.element),0)),this.longRunningScheduler=this._register(new It(()=>this.infiniteLongRunning(),Zk.LONG_RUNNING_INFINITE_THRESHOLD)),this.create(e,t)}create(e,t){this.element=document.createElement("div"),this.element.classList.add("monaco-progress-container"),this.element.setAttribute("role","progressbar"),this.element.setAttribute("aria-valuemin","0"),e.appendChild(this.element),this.bit=document.createElement("div"),this.bit.classList.add("progress-bit"),this.bit.style.backgroundColor=(t==null?void 0:t.progressBarBackground)||"#0E70C0",this.element.appendChild(this.bit)}off(){this.bit.style.width="inherit",this.bit.style.opacity="1",this.element.classList.remove(B8,rT,aT,H8),this.workedVal=0,this.totalWork=void 0,this.longRunningScheduler.cancel(),this.progressSignal.clear()}stop(){return this.doDone(!1)}doDone(e){return this.element.classList.add(W8),this.element.classList.contains(rT)?(this.bit.style.opacity="0",e?setTimeout(()=>this.off(),200):this.off()):(this.bit.style.width="inherit",e?setTimeout(()=>this.off(),200):this.off()),this}infinite(){return this.bit.style.width="2%",this.bit.style.opacity="1",this.element.classList.remove(H8,W8,aT),this.element.classList.add(B8,rT),this.longRunningScheduler.schedule(),this}infiniteLongRunning(){this.element.classList.add(aT)}getContainer(){return this.element}};Zk.LONG_RUNNING_INFINITE_THRESHOLD=1e4;let vA=Zk;const Mge=m("caseDescription","Match Case"),Age=m("wordsDescription","Match Whole Word"),Pge=m("regexDescription","Use Regular Expression");class wK extends kb{constructor(e){super({icon:te.caseSensitive,title:Mge+e.appendTitle,isChecked:e.isChecked,hoverDelegate:e.hoverDelegate??Ss("element"),inputActiveOptionBorder:e.inputActiveOptionBorder,inputActiveOptionForeground:e.inputActiveOptionForeground,inputActiveOptionBackground:e.inputActiveOptionBackground})}}class SK extends kb{constructor(e){super({icon:te.wholeWord,title:Age+e.appendTitle,isChecked:e.isChecked,hoverDelegate:e.hoverDelegate??Ss("element"),inputActiveOptionBorder:e.inputActiveOptionBorder,inputActiveOptionForeground:e.inputActiveOptionForeground,inputActiveOptionBackground:e.inputActiveOptionBackground})}}class yK extends kb{constructor(e){super({icon:te.regex,title:Pge+e.appendTitle,isChecked:e.isChecked,hoverDelegate:e.hoverDelegate??Ss("element"),inputActiveOptionBorder:e.inputActiveOptionBorder,inputActiveOptionForeground:e.inputActiveOptionForeground,inputActiveOptionBackground:e.inputActiveOptionBackground})}}class Oge{constructor(e,t=0,i=e.length,n=t-1){this.items=e,this.start=t,this.end=i,this.index=n}current(){return this.index===this.start-1||this.index===this.end?null:this.items[this.index]}next(){return this.index=Math.min(this.index+1,this.end),this.current()}previous(){return this.index=Math.max(this.index-1,this.start-1),this.current()}first(){return this.index=this.start,this.current()}last(){return this.index=this.end-1,this.current()}}class Fge{constructor(e=[],t=10){this._initialize(e),this._limit=t,this._onChange()}getHistory(){return this._elements}add(e){this._history.delete(e),this._history.add(e),this._onChange()}next(){return this._navigator.next()}previous(){return this._currentPosition()!==0?this._navigator.previous():null}current(){return this._navigator.current()}first(){return this._navigator.first()}last(){return this._navigator.last()}isLast(){return this._currentPosition()>=this._elements.length-1}isNowhere(){return this._navigator.current()===null}has(e){return this._history.has(e)}_onChange(){this._reduceToLimit();const e=this._elements;this._navigator=new Oge(e,0,e.length,e.length)}_reduceToLimit(){const e=this._elements;e.length>this._limit&&this._initialize(e.slice(e.length-this._limit))}_currentPosition(){const e=this._navigator.current();return e?this._elements.indexOf(e):-1}_initialize(e){this._history=new Set;for(const t of e)this._history.add(t)}get _elements(){const e=[];return this._history.forEach(t=>e.push(t)),e}}const rC=ne;class Wge extends Yo{constructor(e,t,i){super(),this.state="idle",this.maxHeight=Number.POSITIVE_INFINITY,this._onDidChange=this._register(new P),this.onDidChange=this._onDidChange.event,this._onDidHeightChange=this._register(new P),this.onDidHeightChange=this._onDidHeightChange.event,this.contextViewProvider=t,this.options=i,this.message=null,this.placeholder=this.options.placeholder||"",this.tooltip=this.options.tooltip??(this.placeholder||""),this.ariaLabel=this.options.ariaLabel||"",this.options.validationOptions&&(this.validation=this.options.validationOptions.validation),this.element=X(e,rC(".monaco-inputbox.idle"));const n=this.options.flexibleHeight?"textarea":"input",s=X(this.element,rC(".ibwrapper"));if(this.input=X(s,rC(n+".input.empty")),this.input.setAttribute("autocorrect","off"),this.input.setAttribute("autocapitalize","off"),this.input.setAttribute("spellcheck","false"),this.onfocus(this.input,()=>this.element.classList.add("synthetic-focus")),this.onblur(this.input,()=>this.element.classList.remove("synthetic-focus")),this.options.flexibleHeight){this.maxHeight=typeof this.options.flexibleMaxHeight=="number"?this.options.flexibleMaxHeight:Number.POSITIVE_INFINITY,this.mirror=X(s,rC("div.mirror")),this.mirror.innerText=" ",this.scrollableElement=new Z$(this.element,{vertical:1}),this.options.flexibleWidth&&(this.input.setAttribute("wrap","off"),this.mirror.style.whiteSpace="pre",this.mirror.style.wordWrap="initial"),X(e,this.scrollableElement.getDomNode()),this._register(this.scrollableElement),this._register(this.scrollableElement.onScroll(l=>this.input.scrollTop=l.scrollTop));const r=this._register(new tt(e.ownerDocument,"selectionchange")),a=oe.filter(r.event,()=>{const l=e.ownerDocument.getSelection();return(l==null?void 0:l.anchorNode)===s});this._register(a(this.updateScrollDimensions,this)),this._register(this.onDidHeightChange(this.updateScrollDimensions,this))}else this.input.type=this.options.type||"text",this.input.setAttribute("wrap","off");this.ariaLabel&&this.input.setAttribute("aria-label",this.ariaLabel),this.placeholder&&!this.options.showPlaceholderOnFocus&&this.setPlaceHolder(this.placeholder),this.tooltip&&this.setTooltip(this.tooltip),this.oninput(this.input,()=>this.onValueChange()),this.onblur(this.input,()=>this.onBlur()),this.onfocus(this.input,()=>this.onFocus()),this._register(this.ignoreGesture(this.input)),setTimeout(()=>this.updateMirror(),0),this.options.actions&&(this.actionbar=this._register(new zo(this.element)),this.actionbar.push(this.options.actions,{icon:!0,label:!1})),this.applyStyles()}onBlur(){this._hideMessage(),this.options.showPlaceholderOnFocus&&this.input.setAttribute("placeholder","")}onFocus(){this._showMessage(),this.options.showPlaceholderOnFocus&&this.input.setAttribute("placeholder",this.placeholder||"")}setPlaceHolder(e){this.placeholder=e,this.input.setAttribute("placeholder",e)}setTooltip(e){this.tooltip=e,this.hover?this.hover.update(e):this.hover=this._register(ja().setupManagedHover(Ss("mouse"),this.input,e))}get inputElement(){return this.input}get value(){return this.input.value}set value(e){this.input.value!==e&&(this.input.value=e,this.onValueChange())}get height(){return typeof this.cachedHeight=="number"?this.cachedHeight:Tc(this.element)}focus(){this.input.focus()}blur(){this.input.blur()}hasFocus(){return wI(this.input)}select(e=null){this.input.select(),e&&(this.input.setSelectionRange(e.start,e.end),e.end===this.input.value.length&&(this.input.scrollLeft=this.input.scrollWidth))}isSelectionAtEnd(){return this.input.selectionEnd===this.input.value.length&&this.input.selectionStart===this.input.selectionEnd}getSelection(){const e=this.input.selectionStart;if(e===null)return null;const t=this.input.selectionEnd??e;return{start:e,end:t}}enable(){this.input.removeAttribute("disabled")}disable(){this.blur(),this.input.disabled=!0,this._hideMessage()}set paddingRight(e){this.input.style.width=`calc(100% - ${e}px)`,this.mirror&&(this.mirror.style.paddingRight=e+"px")}updateScrollDimensions(){if(typeof this.cachedContentHeight!="number"||typeof this.cachedHeight!="number"||!this.scrollableElement)return;const e=this.cachedContentHeight,t=this.cachedHeight,i=this.input.scrollTop;this.scrollableElement.setScrollDimensions({scrollHeight:e,height:t}),this.scrollableElement.setScrollPosition({scrollTop:i})}showMessage(e,t){if(this.state==="open"&&oo(this.message,e))return;this.message=e,this.element.classList.remove("idle"),this.element.classList.remove("info"),this.element.classList.remove("warning"),this.element.classList.remove("error"),this.element.classList.add(this.classForType(e.type));const i=this.stylesForType(this.message.type);this.element.style.border=`1px solid ${jc(i.border,"transparent")}`,this.message.content&&(this.hasFocus()||t)&&this._showMessage()}hideMessage(){this.message=null,this.element.classList.remove("info"),this.element.classList.remove("warning"),this.element.classList.remove("error"),this.element.classList.add("idle"),this._hideMessage(),this.applyStyles()}validate(){let e=null;return this.validation&&(e=this.validation(this.value),e?(this.inputElement.setAttribute("aria-invalid","true"),this.showMessage(e)):this.inputElement.hasAttribute("aria-invalid")&&(this.inputElement.removeAttribute("aria-invalid"),this.hideMessage())),e==null?void 0:e.type}stylesForType(e){const t=this.options.inputBoxStyles;switch(e){case 1:return{border:t.inputValidationInfoBorder,background:t.inputValidationInfoBackground,foreground:t.inputValidationInfoForeground};case 2:return{border:t.inputValidationWarningBorder,background:t.inputValidationWarningBackground,foreground:t.inputValidationWarningForeground};default:return{border:t.inputValidationErrorBorder,background:t.inputValidationErrorBackground,foreground:t.inputValidationErrorForeground}}}classForType(e){switch(e){case 1:return"info";case 2:return"warning";default:return"error"}}_showMessage(){if(!this.contextViewProvider||!this.message)return;let e;const t=()=>e.style.width=Xs(this.element)+"px";this.contextViewProvider.showContextView({getAnchor:()=>this.element,anchorAlignment:1,render:n=>{if(!this.message)return null;e=X(n,rC(".monaco-inputbox-container")),t();const s={inline:!0,className:"monaco-inputbox-message"},r=this.message.formatContent?Eae(this.message.content,s):Iae(this.message.content,s);r.classList.add(this.classForType(this.message.type));const a=this.stylesForType(this.message.type);return r.style.backgroundColor=a.background??"",r.style.color=a.foreground??"",r.style.border=a.border?`1px solid ${a.border}`:"",X(e,r),null},onHide:()=>{this.state="closed"},layout:t});let i;this.message.type===3?i=m("alertErrorMessage","Error: {0}",this.message.content):this.message.type===2?i=m("alertWarningMessage","Warning: {0}",this.message.content):i=m("alertInfoMessage","Info: {0}",this.message.content),Cs(i),this.state="open"}_hideMessage(){this.contextViewProvider&&(this.state==="open"&&this.contextViewProvider.hideContextView(),this.state="idle")}onValueChange(){this._onDidChange.fire(this.value),this.validate(),this.updateMirror(),this.input.classList.toggle("empty",!this.value),this.state==="open"&&this.contextViewProvider&&this.contextViewProvider.layout()}updateMirror(){if(!this.mirror)return;const e=this.value,i=e.charCodeAt(e.length-1)===10?" ":"";(e+i).replace(/\u000c/g,"")?this.mirror.textContent=e+i:this.mirror.innerText=" ",this.layout()}applyStyles(){const e=this.options.inputBoxStyles,t=e.inputBackground??"",i=e.inputForeground??"",n=e.inputBorder??"";this.element.style.backgroundColor=t,this.element.style.color=i,this.input.style.backgroundColor="inherit",this.input.style.color=i,this.element.style.border=`1px solid ${jc(n,"transparent")}`}layout(){if(!this.mirror)return;const e=this.cachedContentHeight;this.cachedContentHeight=Tc(this.mirror),e!==this.cachedContentHeight&&(this.cachedHeight=Math.min(this.cachedContentHeight,this.maxHeight),this.input.style.height=this.cachedHeight+"px",this._onDidHeightChange.fire(this.cachedContentHeight))}insertAtCursor(e){const t=this.inputElement,i=t.selectionStart,n=t.selectionEnd,s=t.value;i!==null&&n!==null&&(this.value=s.substr(0,i)+e+s.substr(n),t.setSelectionRange(i+1,i+1),this.layout())}dispose(){var e;this._hideMessage(),this.message=null,(e=this.actionbar)==null||e.dispose(),super.dispose()}}class LK extends Wge{constructor(e,t,i){const n=m({key:"history.inputbox.hint.suffix.noparens",comment:['Text is the suffix of an input field placeholder coming after the action the input field performs, this will be used when the input field ends in a closing parenthesis ")", for example "Filter (e.g. text, !exclude)". The character inserted into the final string is ⇅ to represent the up and down arrow keys.']}," or {0} for history","⇅"),s=m({key:"history.inputbox.hint.suffix.inparens",comment:['Text is the suffix of an input field placeholder coming after the action the input field performs, this will be used when the input field does NOT end in a closing parenthesis (eg. "Find"). The character inserted into the final string is ⇅ to represent the up and down arrow keys.']}," ({0} for history)","⇅");super(e,t,i),this._onDidFocus=this._register(new P),this.onDidFocus=this._onDidFocus.event,this._onDidBlur=this._register(new P),this.onDidBlur=this._onDidBlur.event,this.history=new Fge(i.history,100);const r=()=>{if(i.showHistoryHint&&i.showHistoryHint()&&!this.placeholder.endsWith(n)&&!this.placeholder.endsWith(s)&&this.history.getHistory().length){const a=this.placeholder.endsWith(")")?n:s,l=this.placeholder+a;i.showPlaceholderOnFocus&&!wI(this.input)?this.placeholder=l:this.setPlaceHolder(l)}};this.observer=new MutationObserver((a,l)=>{a.forEach(c=>{c.target.textContent||r()})}),this.observer.observe(this.input,{attributeFilter:["class"]}),this.onfocus(this.input,()=>r()),this.onblur(this.input,()=>{const a=l=>{if(this.placeholder.endsWith(l)){const c=this.placeholder.slice(0,this.placeholder.length-l.length);return i.showPlaceholderOnFocus?this.placeholder=c:this.setPlaceHolder(c),!0}else return!1};a(s)||a(n)})}dispose(){super.dispose(),this.observer&&(this.observer.disconnect(),this.observer=void 0)}addToHistory(e){this.value&&(e||this.value!==this.getCurrentValue())&&this.history.add(this.value)}isAtLastInHistory(){return this.history.isLast()}isNowhereInHistory(){return this.history.isNowhere()}showNextValue(){this.history.has(this.value)||this.addToHistory();let e=this.getNextValue();e&&(e=e===this.value?this.getNextValue():e),this.value=e??"",Hl(this.value?this.value:m("clearedInput","Cleared Input"))}showPreviousValue(){this.history.has(this.value)||this.addToHistory();let e=this.getPreviousValue();e&&(e=e===this.value?this.getPreviousValue():e),e&&(this.value=e,Hl(this.value))}setPlaceHolder(e){super.setPlaceHolder(e),this.setTooltip(e)}onBlur(){super.onBlur(),this._onDidBlur.fire()}onFocus(){super.onFocus(),this._onDidFocus.fire()}getCurrentValue(){let e=this.history.current();return e||(e=this.history.last(),this.history.next()),e}getPreviousValue(){return this.history.previous()||this.history.first()}getNextValue(){return this.history.next()}}const Bge=m("defaultLabel","input");class xK extends Yo{constructor(e,t,i){super(),this.fixFocusOnOptionClickEnabled=!0,this.imeSessionInProgress=!1,this.additionalTogglesDisposables=this._register(new Ji),this.additionalToggles=[],this._onDidOptionChange=this._register(new P),this.onDidOptionChange=this._onDidOptionChange.event,this._onKeyDown=this._register(new P),this.onKeyDown=this._onKeyDown.event,this._onMouseDown=this._register(new P),this.onMouseDown=this._onMouseDown.event,this._onInput=this._register(new P),this._onKeyUp=this._register(new P),this._onCaseSensitiveKeyDown=this._register(new P),this.onCaseSensitiveKeyDown=this._onCaseSensitiveKeyDown.event,this._onRegexKeyDown=this._register(new P),this.onRegexKeyDown=this._onRegexKeyDown.event,this._lastHighlightFindOptions=0,this.placeholder=i.placeholder||"",this.validation=i.validation,this.label=i.label||Bge,this.showCommonFindToggles=!!i.showCommonFindToggles;const n=i.appendCaseSensitiveLabel||"",s=i.appendWholeWordsLabel||"",r=i.appendRegexLabel||"",a=i.history||[],l=!!i.flexibleHeight,c=!!i.flexibleWidth,d=i.flexibleMaxHeight;this.domNode=document.createElement("div"),this.domNode.classList.add("monaco-findInput"),this.inputBox=this._register(new LK(this.domNode,t,{placeholder:this.placeholder||"",ariaLabel:this.label||"",validationOptions:{validation:this.validation},history:a,showHistoryHint:i.showHistoryHint,flexibleHeight:l,flexibleWidth:c,flexibleMaxHeight:d,inputBoxStyles:i.inputBoxStyles}));const h=this._register(H_());if(this.showCommonFindToggles){this.regex=this._register(new yK({appendTitle:r,isChecked:!1,hoverDelegate:h,...i.toggleStyles})),this._register(this.regex.onChange(g=>{this._onDidOptionChange.fire(g),!g&&this.fixFocusOnOptionClickEnabled&&this.inputBox.focus(),this.validate()})),this._register(this.regex.onKeyDown(g=>{this._onRegexKeyDown.fire(g)})),this.wholeWords=this._register(new SK({appendTitle:s,isChecked:!1,hoverDelegate:h,...i.toggleStyles})),this._register(this.wholeWords.onChange(g=>{this._onDidOptionChange.fire(g),!g&&this.fixFocusOnOptionClickEnabled&&this.inputBox.focus(),this.validate()})),this.caseSensitive=this._register(new wK({appendTitle:n,isChecked:!1,hoverDelegate:h,...i.toggleStyles})),this._register(this.caseSensitive.onChange(g=>{this._onDidOptionChange.fire(g),!g&&this.fixFocusOnOptionClickEnabled&&this.inputBox.focus(),this.validate()})),this._register(this.caseSensitive.onKeyDown(g=>{this._onCaseSensitiveKeyDown.fire(g)}));const u=[this.caseSensitive.domNode,this.wholeWords.domNode,this.regex.domNode];this.onkeydown(this.domNode,g=>{if(g.equals(15)||g.equals(17)||g.equals(9)){const f=u.indexOf(this.domNode.ownerDocument.activeElement);if(f>=0){let p=-1;g.equals(17)?p=(f+1)%u.length:g.equals(15)&&(f===0?p=u.length-1:p=f-1),g.equals(9)?(u[f].blur(),this.inputBox.focus()):p>=0&&u[p].focus(),nt.stop(g,!0)}}})}this.controls=document.createElement("div"),this.controls.className="controls",this.controls.style.display=this.showCommonFindToggles?"":"none",this.caseSensitive&&this.controls.append(this.caseSensitive.domNode),this.wholeWords&&this.controls.appendChild(this.wholeWords.domNode),this.regex&&this.controls.appendChild(this.regex.domNode),this.setAdditionalToggles(i==null?void 0:i.additionalToggles),this.controls&&this.domNode.appendChild(this.controls),e==null||e.appendChild(this.domNode),this._register(U(this.inputBox.inputElement,"compositionstart",u=>{this.imeSessionInProgress=!0})),this._register(U(this.inputBox.inputElement,"compositionend",u=>{this.imeSessionInProgress=!1,this._onInput.fire()})),this.onkeydown(this.inputBox.inputElement,u=>this._onKeyDown.fire(u)),this.onkeyup(this.inputBox.inputElement,u=>this._onKeyUp.fire(u)),this.oninput(this.inputBox.inputElement,u=>this._onInput.fire()),this.onmousedown(this.inputBox.inputElement,u=>this._onMouseDown.fire(u))}get onDidChange(){return this.inputBox.onDidChange}layout(e){this.inputBox.layout(),this.updateInputBoxPadding(e.collapsedFindWidget)}enable(){var e,t,i;this.domNode.classList.remove("disabled"),this.inputBox.enable(),(e=this.regex)==null||e.enable(),(t=this.wholeWords)==null||t.enable(),(i=this.caseSensitive)==null||i.enable();for(const n of this.additionalToggles)n.enable()}disable(){var e,t,i;this.domNode.classList.add("disabled"),this.inputBox.disable(),(e=this.regex)==null||e.disable(),(t=this.wholeWords)==null||t.disable(),(i=this.caseSensitive)==null||i.disable();for(const n of this.additionalToggles)n.disable()}setFocusInputOnOptionClick(e){this.fixFocusOnOptionClickEnabled=e}setEnabled(e){e?this.enable():this.disable()}setAdditionalToggles(e){for(const t of this.additionalToggles)t.domNode.remove();this.additionalToggles=[],this.additionalTogglesDisposables.value=new K;for(const t of e??[])this.additionalTogglesDisposables.value.add(t),this.controls.appendChild(t.domNode),this.additionalTogglesDisposables.value.add(t.onChange(i=>{this._onDidOptionChange.fire(i),!i&&this.fixFocusOnOptionClickEnabled&&this.inputBox.focus()})),this.additionalToggles.push(t);this.additionalToggles.length>0&&(this.controls.style.display=""),this.updateInputBoxPadding()}updateInputBoxPadding(e=!1){var t,i,n;e?this.inputBox.paddingRight=0:this.inputBox.paddingRight=(((t=this.caseSensitive)==null?void 0:t.width())??0)+(((i=this.wholeWords)==null?void 0:i.width())??0)+(((n=this.regex)==null?void 0:n.width())??0)+this.additionalToggles.reduce((s,r)=>s+r.width(),0)}getValue(){return this.inputBox.value}setValue(e){this.inputBox.value!==e&&(this.inputBox.value=e)}select(){this.inputBox.select()}focus(){this.inputBox.focus()}getCaseSensitive(){var e;return((e=this.caseSensitive)==null?void 0:e.checked)??!1}setCaseSensitive(e){this.caseSensitive&&(this.caseSensitive.checked=e)}getWholeWords(){var e;return((e=this.wholeWords)==null?void 0:e.checked)??!1}setWholeWords(e){this.wholeWords&&(this.wholeWords.checked=e)}getRegex(){var e;return((e=this.regex)==null?void 0:e.checked)??!1}setRegex(e){this.regex&&(this.regex.checked=e,this.validate())}focusOnCaseSensitive(){var e;(e=this.caseSensitive)==null||e.focus()}highlightFindOptions(){this.domNode.classList.remove("highlight-"+this._lastHighlightFindOptions),this._lastHighlightFindOptions=1-this._lastHighlightFindOptions,this.domNode.classList.add("highlight-"+this._lastHighlightFindOptions)}validate(){this.inputBox.validate()}showMessage(e){this.inputBox.showMessage(e)}clearMessage(){this.inputBox.hideMessage()}}const Hge=ne;class Vge extends F{constructor(e,t,i){super(),this.parent=e,this.onKeyDown=s=>hi(this.findInput.inputBox.inputElement,ie.KEY_DOWN,s),this.onDidChange=s=>this.findInput.onDidChange(s),this.container=X(this.parent,Hge(".quick-input-box")),this.findInput=this._register(new xK(this.container,void 0,{label:"",inputBoxStyles:t,toggleStyles:i}));const n=this.findInput.inputBox.inputElement;n.role="combobox",n.ariaHasPopup="menu",n.ariaAutoComplete="list",n.ariaExpanded="true"}get value(){return this.findInput.getValue()}set value(e){this.findInput.setValue(e)}select(e=null){this.findInput.inputBox.select(e)}getSelection(){return this.findInput.inputBox.getSelection()}isSelectionAtEnd(){return this.findInput.inputBox.isSelectionAtEnd()}get placeholder(){return this.findInput.inputBox.inputElement.getAttribute("placeholder")||""}set placeholder(e){this.findInput.inputBox.setPlaceHolder(e)}get password(){return this.findInput.inputBox.inputElement.type==="password"}set password(e){this.findInput.inputBox.inputElement.type=e?"password":"text"}set enabled(e){this.findInput.inputBox.inputElement.toggleAttribute("readonly",!e)}set toggles(e){this.findInput.setAdditionalToggles(e)}setAttribute(e,t){this.findInput.inputBox.inputElement.setAttribute(e,t)}showDecoration(e){e===Ii.Ignore?this.findInput.clearMessage():this.findInput.showMessage({type:e===Ii.Info?1:e===Ii.Warning?2:3,content:""})}stylesForType(e){return this.findInput.inputBox.stylesForType(e===Ii.Info?1:e===Ii.Warning?2:3)}setFocus(){this.findInput.focus()}layout(){this.findInput.inputBox.layout()}}class zge{get templateId(){return this.renderer.templateId}constructor(e,t){this.renderer=e,this.modelProvider=t}renderTemplate(e){return{data:this.renderer.renderTemplate(e),disposable:F.None}}renderElement(e,t,i,n){var l;if((l=i.disposable)==null||l.dispose(),!i.data)return;const s=this.modelProvider();if(s.isResolved(e))return this.renderer.renderElement(s.get(e),e,i.data,n);const r=new li,a=s.resolve(e,r.token);i.disposable={dispose:()=>r.cancel()},this.renderer.renderPlaceholder(e,i.data),a.then(c=>this.renderer.renderElement(c,e,i.data,n))}disposeTemplate(e){e.disposable&&(e.disposable.dispose(),e.disposable=void 0),e.data&&(this.renderer.disposeTemplate(e.data),e.data=void 0)}}class Uge{constructor(e,t){this.modelProvider=e,this.accessibilityProvider=t}getWidgetAriaLabel(){return this.accessibilityProvider.getWidgetAriaLabel()}getAriaLabel(e){const t=this.modelProvider();return t.isResolved(e)?this.accessibilityProvider.getAriaLabel(t.get(e)):null}}function $ge(o,e){return{...e,accessibilityProvider:e.accessibilityProvider&&new Uge(o,e.accessibilityProvider)}}class jge{constructor(e,t,i,n,s={}){const r=()=>this.model,a=n.map(l=>new zge(l,r));this.list=new Xo(e,t,i,a,$ge(r,s))}updateOptions(e){this.list.updateOptions(e)}getHTMLElement(){return this.list.getHTMLElement()}get onDidFocus(){return this.list.onDidFocus}get widget(){return this.list}get onDidDispose(){return this.list.onDidDispose}get onMouseDblClick(){return oe.map(this.list.onMouseDblClick,({element:e,index:t,browserEvent:i})=>({element:e===void 0?void 0:this._model.get(e),index:t,browserEvent:i}))}get onPointer(){return oe.map(this.list.onPointer,({element:e,index:t,browserEvent:i})=>({element:e===void 0?void 0:this._model.get(e),index:t,browserEvent:i}))}get onDidChangeSelection(){return oe.map(this.list.onDidChangeSelection,({elements:e,indexes:t,browserEvent:i})=>({elements:e.map(n=>this._model.get(n)),indexes:t,browserEvent:i}))}get model(){return this._model}set model(e){this._model=e,this.list.splice(0,this.list.length,Is(e.length))}getFocus(){return this.list.getFocus()}getSelection(){return this.list.getSelection()}getSelectedElements(){return this.getSelection().map(e=>this.model.get(e))}style(e){this.list.style(e)}dispose(){this.list.dispose()}}var Db=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s};const Kge=!1;var WL;(function(o){o.North="north",o.South="south",o.East="east",o.West="west"})(WL||(WL={}));let qge=4;const Gge=new P;let Zge=300;const Yge=new P;class _3{constructor(e){this.el=e,this.disposables=new K}get onPointerMove(){return this.disposables.add(new tt(xe(this.el),"mousemove")).event}get onPointerUp(){return this.disposables.add(new tt(xe(this.el),"mouseup")).event}dispose(){this.disposables.dispose()}}Db([Ni],_3.prototype,"onPointerMove",null);Db([Ni],_3.prototype,"onPointerUp",null);class b3{get onPointerMove(){return this.disposables.add(new tt(this.el,Bt.Change)).event}get onPointerUp(){return this.disposables.add(new tt(this.el,Bt.End)).event}constructor(e){this.el=e,this.disposables=new K}dispose(){this.disposables.dispose()}}Db([Ni],b3.prototype,"onPointerMove",null);Db([Ni],b3.prototype,"onPointerUp",null);class BL{get onPointerMove(){return this.factory.onPointerMove}get onPointerUp(){return this.factory.onPointerUp}constructor(e){this.factory=e}dispose(){}}Db([Ni],BL.prototype,"onPointerMove",null);Db([Ni],BL.prototype,"onPointerUp",null);const V8="pointer-events-disabled";class Mn extends F{get state(){return this._state}get orthogonalStartSash(){return this._orthogonalStartSash}get orthogonalEndSash(){return this._orthogonalEndSash}set state(e){this._state!==e&&(this.el.classList.toggle("disabled",e===0),this.el.classList.toggle("minimum",e===1),this.el.classList.toggle("maximum",e===2),this._state=e,this.onDidEnablementChange.fire(e))}set orthogonalStartSash(e){if(this._orthogonalStartSash!==e){if(this.orthogonalStartDragHandleDisposables.clear(),this.orthogonalStartSashDisposables.clear(),e){const t=i=>{this.orthogonalStartDragHandleDisposables.clear(),i!==0&&(this._orthogonalStartDragHandle=X(this.el,ne(".orthogonal-drag-handle.start")),this.orthogonalStartDragHandleDisposables.add(ve(()=>this._orthogonalStartDragHandle.remove())),this.orthogonalStartDragHandleDisposables.add(new tt(this._orthogonalStartDragHandle,"mouseenter")).event(()=>Mn.onMouseEnter(e),void 0,this.orthogonalStartDragHandleDisposables),this.orthogonalStartDragHandleDisposables.add(new tt(this._orthogonalStartDragHandle,"mouseleave")).event(()=>Mn.onMouseLeave(e),void 0,this.orthogonalStartDragHandleDisposables))};this.orthogonalStartSashDisposables.add(e.onDidEnablementChange.event(t,this)),t(e.state)}this._orthogonalStartSash=e}}set orthogonalEndSash(e){if(this._orthogonalEndSash!==e){if(this.orthogonalEndDragHandleDisposables.clear(),this.orthogonalEndSashDisposables.clear(),e){const t=i=>{this.orthogonalEndDragHandleDisposables.clear(),i!==0&&(this._orthogonalEndDragHandle=X(this.el,ne(".orthogonal-drag-handle.end")),this.orthogonalEndDragHandleDisposables.add(ve(()=>this._orthogonalEndDragHandle.remove())),this.orthogonalEndDragHandleDisposables.add(new tt(this._orthogonalEndDragHandle,"mouseenter")).event(()=>Mn.onMouseEnter(e),void 0,this.orthogonalEndDragHandleDisposables),this.orthogonalEndDragHandleDisposables.add(new tt(this._orthogonalEndDragHandle,"mouseleave")).event(()=>Mn.onMouseLeave(e),void 0,this.orthogonalEndDragHandleDisposables))};this.orthogonalEndSashDisposables.add(e.onDidEnablementChange.event(t,this)),t(e.state)}this._orthogonalEndSash=e}}constructor(e,t,i){super(),this.hoverDelay=Zge,this.hoverDelayer=this._register(new _r(this.hoverDelay)),this._state=3,this.onDidEnablementChange=this._register(new P),this._onDidStart=this._register(new P),this._onDidChange=this._register(new P),this._onDidReset=this._register(new P),this._onDidEnd=this._register(new P),this.orthogonalStartSashDisposables=this._register(new K),this.orthogonalStartDragHandleDisposables=this._register(new K),this.orthogonalEndSashDisposables=this._register(new K),this.orthogonalEndDragHandleDisposables=this._register(new K),this.onDidStart=this._onDidStart.event,this.onDidChange=this._onDidChange.event,this.onDidReset=this._onDidReset.event,this.onDidEnd=this._onDidEnd.event,this.linkedSash=void 0,this.el=X(e,ne(".monaco-sash")),i.orthogonalEdge&&this.el.classList.add(`orthogonal-edge-${i.orthogonalEdge}`),Je&&this.el.classList.add("mac");const n=this._register(new tt(this.el,"mousedown")).event;this._register(n(h=>this.onPointerStart(h,new _3(e)),this));const s=this._register(new tt(this.el,"dblclick")).event;this._register(s(this.onPointerDoublePress,this));const r=this._register(new tt(this.el,"mouseenter")).event;this._register(r(()=>Mn.onMouseEnter(this)));const a=this._register(new tt(this.el,"mouseleave")).event;this._register(a(()=>Mn.onMouseLeave(this))),this._register(Fn.addTarget(this.el));const l=this._register(new tt(this.el,Bt.Start)).event;this._register(l(h=>this.onPointerStart(h,new b3(this.el)),this));const c=this._register(new tt(this.el,Bt.Tap)).event;let d;this._register(c(h=>{if(d){clearTimeout(d),d=void 0,this.onPointerDoublePress(h);return}clearTimeout(d),d=setTimeout(()=>d=void 0,250)},this)),typeof i.size=="number"?(this.size=i.size,i.orientation===0?this.el.style.width=`${this.size}px`:this.el.style.height=`${this.size}px`):(this.size=qge,this._register(Gge.event(h=>{this.size=h,this.layout()}))),this._register(Yge.event(h=>this.hoverDelay=h)),this.layoutProvider=t,this.orthogonalStartSash=i.orthogonalStartSash,this.orthogonalEndSash=i.orthogonalEndSash,this.orientation=i.orientation||0,this.orientation===1?(this.el.classList.add("horizontal"),this.el.classList.remove("vertical")):(this.el.classList.remove("horizontal"),this.el.classList.add("vertical")),this.el.classList.toggle("debug",Kge),this.layout()}onPointerStart(e,t){nt.stop(e);let i=!1;if(!e.__orthogonalSashEvent){const f=this.getOrthogonalSash(e);f&&(i=!0,e.__orthogonalSashEvent=!0,f.onPointerStart(e,new BL(t)))}if(this.linkedSash&&!e.__linkedSashEvent&&(e.__linkedSashEvent=!0,this.linkedSash.onPointerStart(e,new BL(t))),!this.state)return;const n=this.el.ownerDocument.getElementsByTagName("iframe");for(const f of n)f.classList.add(V8);const s=e.pageX,r=e.pageY,a=e.altKey,l={startX:s,currentX:s,startY:r,currentY:r,altKey:a};this.el.classList.add("active"),this._onDidStart.fire(l);const c=$o(this.el),d=()=>{let f="";i?f="all-scroll":this.orientation===1?this.state===1?f="s-resize":this.state===2?f="n-resize":f=Je?"row-resize":"ns-resize":this.state===1?f="e-resize":this.state===2?f="w-resize":f=Je?"col-resize":"ew-resize",c.textContent=`* { cursor: ${f} !important; }`},h=new K;d(),i||this.onDidEnablementChange.event(d,null,h);const u=f=>{nt.stop(f,!1);const p={startX:s,currentX:f.pageX,startY:r,currentY:f.pageY,altKey:a};this._onDidChange.fire(p)},g=f=>{nt.stop(f,!1),c.remove(),this.el.classList.remove("active"),this._onDidEnd.fire(),h.dispose();for(const p of n)p.classList.remove(V8)};t.onPointerMove(u,null,h),t.onPointerUp(g,null,h),h.add(t)}onPointerDoublePress(e){const t=this.getOrthogonalSash(e);t&&t._onDidReset.fire(),this.linkedSash&&this.linkedSash._onDidReset.fire(),this._onDidReset.fire()}static onMouseEnter(e,t=!1){e.el.classList.contains("active")?(e.hoverDelayer.cancel(),e.el.classList.add("hover")):e.hoverDelayer.trigger(()=>e.el.classList.add("hover"),e.hoverDelay).then(void 0,()=>{}),!t&&e.linkedSash&&Mn.onMouseEnter(e.linkedSash,!0)}static onMouseLeave(e,t=!1){e.hoverDelayer.cancel(),e.el.classList.remove("hover"),!t&&e.linkedSash&&Mn.onMouseLeave(e.linkedSash,!0)}clearSashHoverState(){Mn.onMouseLeave(this)}layout(){if(this.orientation===0){const e=this.layoutProvider;this.el.style.left=e.getVerticalSashLeft(this)-this.size/2+"px",e.getVerticalSashTop&&(this.el.style.top=e.getVerticalSashTop(this)+"px"),e.getVerticalSashHeight&&(this.el.style.height=e.getVerticalSashHeight(this)+"px")}else{const e=this.layoutProvider;this.el.style.top=e.getHorizontalSashTop(this)-this.size/2+"px",e.getHorizontalSashLeft&&(this.el.style.left=e.getHorizontalSashLeft(this)+"px"),e.getHorizontalSashWidth&&(this.el.style.width=e.getHorizontalSashWidth(this)+"px")}}getOrthogonalSash(e){const t=e.initialTarget??e.target;if(!(!t||!Xi(t))&&t.classList.contains("orthogonal-drag-handle"))return t.classList.contains("start")?this.orthogonalStartSash:this.orthogonalEndSash}dispose(){super.dispose(),this.el.remove()}}const Xge={separatorBorder:Z.transparent};class kK{set size(e){this._size=e}get size(){return this._size}get visible(){return typeof this._cachedVisibleSize>"u"}setVisible(e,t){var i,n;if(e!==this.visible){e?(this.size=os(this._cachedVisibleSize,this.viewMinimumSize,this.viewMaximumSize),this._cachedVisibleSize=void 0):(this._cachedVisibleSize=typeof t=="number"?t:this.size,this.size=0),this.container.classList.toggle("visible",e);try{(n=(i=this.view).setVisible)==null||n.call(i,e)}catch(s){console.error("Splitview: Failed to set visible view"),console.error(s)}}}get minimumSize(){return this.visible?this.view.minimumSize:0}get viewMinimumSize(){return this.view.minimumSize}get maximumSize(){return this.visible?this.view.maximumSize:0}get viewMaximumSize(){return this.view.maximumSize}get priority(){return this.view.priority}get proportionalLayout(){return this.view.proportionalLayout??!0}get snap(){return!!this.view.snap}set enabled(e){this.container.style.pointerEvents=e?"":"none"}constructor(e,t,i,n){this.container=e,this.view=t,this.disposable=n,this._cachedVisibleSize=void 0,typeof i=="number"?(this._size=i,this._cachedVisibleSize=void 0,e.classList.add("visible")):(this._size=0,this._cachedVisibleSize=i.cachedVisibleSize)}layout(e,t){this.layoutContainer(e);try{this.view.layout(this.size,e,t)}catch(i){console.error("Splitview: Failed to layout view"),console.error(i)}}dispose(){this.disposable.dispose()}}class Qge extends kK{layoutContainer(e){this.container.style.top=`${e}px`,this.container.style.height=`${this.size}px`}}class Jge extends kK{layoutContainer(e){this.container.style.left=`${e}px`,this.container.style.width=`${this.size}px`}}var uc;(function(o){o[o.Idle=0]="Idle",o[o.Busy=1]="Busy"})(uc||(uc={}));var HL;(function(o){o.Distribute={type:"distribute"};function e(n){return{type:"split",index:n}}o.Split=e;function t(n){return{type:"auto",index:n}}o.Auto=t;function i(n){return{type:"invisible",cachedVisibleSize:n}}o.Invisible=i})(HL||(HL={}));class DK extends F{get orthogonalStartSash(){return this._orthogonalStartSash}get orthogonalEndSash(){return this._orthogonalEndSash}get startSnappingEnabled(){return this._startSnappingEnabled}get endSnappingEnabled(){return this._endSnappingEnabled}set orthogonalStartSash(e){for(const t of this.sashItems)t.sash.orthogonalStartSash=e;this._orthogonalStartSash=e}set orthogonalEndSash(e){for(const t of this.sashItems)t.sash.orthogonalEndSash=e;this._orthogonalEndSash=e}set startSnappingEnabled(e){this._startSnappingEnabled!==e&&(this._startSnappingEnabled=e,this.updateSashEnablement())}set endSnappingEnabled(e){this._endSnappingEnabled!==e&&(this._endSnappingEnabled=e,this.updateSashEnablement())}constructor(e,t={}){super(),this.size=0,this._contentSize=0,this.proportions=void 0,this.viewItems=[],this.sashItems=[],this.state=uc.Idle,this._onDidSashChange=this._register(new P),this._onDidSashReset=this._register(new P),this._startSnappingEnabled=!0,this._endSnappingEnabled=!0,this.onDidSashChange=this._onDidSashChange.event,this.onDidSashReset=this._onDidSashReset.event,this.orientation=t.orientation??0,this.inverseAltBehavior=t.inverseAltBehavior??!1,this.proportionalLayout=t.proportionalLayout??!0,this.getSashOrthogonalSize=t.getSashOrthogonalSize,this.el=document.createElement("div"),this.el.classList.add("monaco-split-view2"),this.el.classList.add(this.orientation===0?"vertical":"horizontal"),e.appendChild(this.el),this.sashContainer=X(this.el,ne(".sash-container")),this.viewContainer=ne(".split-view-container"),this.scrollable=this._register(new wb({forceIntegerValues:!0,smoothScrollDuration:125,scheduleAtNextAnimationFrame:n=>go(xe(this.el),n)})),this.scrollableElement=this._register(new jI(this.viewContainer,{vertical:this.orientation===0?t.scrollbarVisibility??1:2,horizontal:this.orientation===1?t.scrollbarVisibility??1:2},this.scrollable));const i=this._register(new tt(this.viewContainer,"scroll")).event;this._register(i(n=>{const s=this.scrollableElement.getScrollPosition(),r=Math.abs(this.viewContainer.scrollLeft-s.scrollLeft)<=1?void 0:this.viewContainer.scrollLeft,a=Math.abs(this.viewContainer.scrollTop-s.scrollTop)<=1?void 0:this.viewContainer.scrollTop;(r!==void 0||a!==void 0)&&this.scrollableElement.setScrollPosition({scrollLeft:r,scrollTop:a})})),this.onDidScroll=this.scrollableElement.onScroll,this._register(this.onDidScroll(n=>{n.scrollTopChanged&&(this.viewContainer.scrollTop=n.scrollTop),n.scrollLeftChanged&&(this.viewContainer.scrollLeft=n.scrollLeft)})),X(this.el,this.scrollableElement.getDomNode()),this.style(t.styles||Xge),t.descriptor&&(this.size=t.descriptor.size,t.descriptor.views.forEach((n,s)=>{const r=Xn(n.visible)||n.visible?n.size:{type:"invisible",cachedVisibleSize:n.size},a=n.view;this.doAddView(a,r,s,!0)}),this._contentSize=this.viewItems.reduce((n,s)=>n+s.size,0),this.saveProportions())}style(e){e.separatorBorder.isTransparent()?(this.el.classList.remove("separator-border"),this.el.style.removeProperty("--separator-border")):(this.el.classList.add("separator-border"),this.el.style.setProperty("--separator-border",e.separatorBorder.toString()))}addView(e,t,i=this.viewItems.length,n){this.doAddView(e,t,i,n)}layout(e,t){const i=Math.max(this.size,this._contentSize);if(this.size=e,this.layoutContext=t,this.proportions){let n=0;for(let s=0;s<this.viewItems.length;s++){const r=this.viewItems[s],a=this.proportions[s];typeof a=="number"?n+=a:e-=r.size}for(let s=0;s<this.viewItems.length;s++){const r=this.viewItems[s],a=this.proportions[s];typeof a=="number"&&n>0&&(r.size=os(Math.round(a*e/n),r.minimumSize,r.maximumSize))}}else{const n=Is(this.viewItems.length),s=n.filter(a=>this.viewItems[a].priority===1),r=n.filter(a=>this.viewItems[a].priority===2);this.resize(this.viewItems.length-1,e-i,void 0,s,r)}this.distributeEmptySpace(),this.layoutViews()}saveProportions(){this.proportionalLayout&&this._contentSize>0&&(this.proportions=this.viewItems.map(e=>e.proportionalLayout&&e.visible?e.size/this._contentSize:void 0))}onSashStart({sash:e,start:t,alt:i}){for(const a of this.viewItems)a.enabled=!1;const n=this.sashItems.findIndex(a=>a.sash===e),s=jr(U(this.el.ownerDocument.body,"keydown",a=>r(this.sashDragState.current,a.altKey)),U(this.el.ownerDocument.body,"keyup",()=>r(this.sashDragState.current,!1))),r=(a,l)=>{const c=this.viewItems.map(f=>f.size);let d=Number.NEGATIVE_INFINITY,h=Number.POSITIVE_INFINITY;if(this.inverseAltBehavior&&(l=!l),l)if(n===this.sashItems.length-1){const p=this.viewItems[n];d=(p.minimumSize-p.size)/2,h=(p.maximumSize-p.size)/2}else{const p=this.viewItems[n+1];d=(p.size-p.maximumSize)/2,h=(p.size-p.minimumSize)/2}let u,g;if(!l){const f=Is(n,-1),p=Is(n+1,this.viewItems.length),_=f.reduce((D,N)=>D+(this.viewItems[N].minimumSize-c[N]),0),b=f.reduce((D,N)=>D+(this.viewItems[N].viewMaximumSize-c[N]),0),C=p.length===0?Number.POSITIVE_INFINITY:p.reduce((D,N)=>D+(c[N]-this.viewItems[N].minimumSize),0),v=p.length===0?Number.NEGATIVE_INFINITY:p.reduce((D,N)=>D+(c[N]-this.viewItems[N].viewMaximumSize),0),w=Math.max(_,v),S=Math.min(C,b),L=this.findFirstSnapIndex(f),k=this.findFirstSnapIndex(p);if(typeof L=="number"){const D=this.viewItems[L],N=Math.floor(D.viewMinimumSize/2);u={index:L,limitDelta:D.visible?w-N:w+N,size:D.size}}if(typeof k=="number"){const D=this.viewItems[k],N=Math.floor(D.viewMinimumSize/2);g={index:k,limitDelta:D.visible?S+N:S-N,size:D.size}}}this.sashDragState={start:a,current:a,index:n,sizes:c,minDelta:d,maxDelta:h,alt:l,snapBefore:u,snapAfter:g,disposable:s}};r(t,i)}onSashChange({current:e}){const{index:t,start:i,sizes:n,alt:s,minDelta:r,maxDelta:a,snapBefore:l,snapAfter:c}=this.sashDragState;this.sashDragState.current=e;const d=e-i,h=this.resize(t,d,n,void 0,void 0,r,a,l,c);if(s){const u=t===this.sashItems.length-1,g=this.viewItems.map(v=>v.size),f=u?t:t+1,p=this.viewItems[f],_=p.size-p.maximumSize,b=p.size-p.minimumSize,C=u?t-1:t+1;this.resize(C,-h,g,void 0,void 0,_,b)}this.distributeEmptySpace(),this.layoutViews()}onSashEnd(e){this._onDidSashChange.fire(e),this.sashDragState.disposable.dispose(),this.saveProportions();for(const t of this.viewItems)t.enabled=!0}onViewChange(e,t){const i=this.viewItems.indexOf(e);i<0||i>=this.viewItems.length||(t=typeof t=="number"?t:e.size,t=os(t,e.minimumSize,e.maximumSize),this.inverseAltBehavior&&i>0?(this.resize(i-1,Math.floor((e.size-t)/2)),this.distributeEmptySpace(),this.layoutViews()):(e.size=t,this.relayout([i],void 0)))}resizeView(e,t){if(!(e<0||e>=this.viewItems.length)){if(this.state!==uc.Idle)throw new Error("Cant modify splitview");this.state=uc.Busy;try{const i=Is(this.viewItems.length).filter(a=>a!==e),n=[...i.filter(a=>this.viewItems[a].priority===1),e],s=i.filter(a=>this.viewItems[a].priority===2),r=this.viewItems[e];t=Math.round(t),t=os(t,r.minimumSize,Math.min(r.maximumSize,this.size)),r.size=t,this.relayout(n,s)}finally{this.state=uc.Idle}}}distributeViewSizes(){const e=[];let t=0;for(const a of this.viewItems)a.maximumSize-a.minimumSize>0&&(e.push(a),t+=a.size);const i=Math.floor(t/e.length);for(const a of e)a.size=os(i,a.minimumSize,a.maximumSize);const n=Is(this.viewItems.length),s=n.filter(a=>this.viewItems[a].priority===1),r=n.filter(a=>this.viewItems[a].priority===2);this.relayout(s,r)}getViewSize(e){return e<0||e>=this.viewItems.length?-1:this.viewItems[e].size}doAddView(e,t,i=this.viewItems.length,n){if(this.state!==uc.Idle)throw new Error("Cant modify splitview");this.state=uc.Busy;try{const s=ne(".split-view-view");i===this.viewItems.length?this.viewContainer.appendChild(s):this.viewContainer.insertBefore(s,this.viewContainer.children.item(i));const r=e.onDidChange(u=>this.onViewChange(d,u)),a=ve(()=>s.remove()),l=jr(r,a);let c;typeof t=="number"?c=t:(t.type==="auto"&&(this.areViewsDistributed()?t={type:"distribute"}:t={type:"split",index:t.index}),t.type==="split"?c=this.getViewSize(t.index)/2:t.type==="invisible"?c={cachedVisibleSize:t.cachedVisibleSize}:c=e.minimumSize);const d=this.orientation===0?new Qge(s,e,c,l):new Jge(s,e,c,l);if(this.viewItems.splice(i,0,d),this.viewItems.length>1){const u={orthogonalStartSash:this.orthogonalStartSash,orthogonalEndSash:this.orthogonalEndSash},g=this.orientation===0?new Mn(this.sashContainer,{getHorizontalSashTop:D=>this.getSashPosition(D),getHorizontalSashWidth:this.getSashOrthogonalSize},{...u,orientation:1}):new Mn(this.sashContainer,{getVerticalSashLeft:D=>this.getSashPosition(D),getVerticalSashHeight:this.getSashOrthogonalSize},{...u,orientation:0}),f=this.orientation===0?D=>({sash:g,start:D.startY,current:D.currentY,alt:D.altKey}):D=>({sash:g,start:D.startX,current:D.currentX,alt:D.altKey}),_=oe.map(g.onDidStart,f)(this.onSashStart,this),C=oe.map(g.onDidChange,f)(this.onSashChange,this),w=oe.map(g.onDidEnd,()=>this.sashItems.findIndex(D=>D.sash===g))(this.onSashEnd,this),S=g.onDidReset(()=>{const D=this.sashItems.findIndex($=>$.sash===g),N=Is(D,-1),B=Is(D+1,this.viewItems.length),W=this.findFirstSnapIndex(N),V=this.findFirstSnapIndex(B);typeof W=="number"&&!this.viewItems[W].visible||typeof V=="number"&&!this.viewItems[V].visible||this._onDidSashReset.fire(D)}),L=jr(_,C,w,S,g),k={sash:g,disposable:L};this.sashItems.splice(i-1,0,k)}s.appendChild(e.element);let h;typeof t!="number"&&t.type==="split"&&(h=[t.index]),n||this.relayout([i],h),!n&&typeof t!="number"&&t.type==="distribute"&&this.distributeViewSizes()}finally{this.state=uc.Idle}}relayout(e,t){const i=this.viewItems.reduce((n,s)=>n+s.size,0);this.resize(this.viewItems.length-1,this.size-i,void 0,e,t),this.distributeEmptySpace(),this.layoutViews(),this.saveProportions()}resize(e,t,i=this.viewItems.map(d=>d.size),n,s,r=Number.NEGATIVE_INFINITY,a=Number.POSITIVE_INFINITY,l,c){if(e<0||e>=this.viewItems.length)return 0;const d=Is(e,-1),h=Is(e+1,this.viewItems.length);if(s)for(const k of s)iN(d,k),iN(h,k);if(n)for(const k of n)LS(d,k),LS(h,k);const u=d.map(k=>this.viewItems[k]),g=d.map(k=>i[k]),f=h.map(k=>this.viewItems[k]),p=h.map(k=>i[k]),_=d.reduce((k,D)=>k+(this.viewItems[D].minimumSize-i[D]),0),b=d.reduce((k,D)=>k+(this.viewItems[D].maximumSize-i[D]),0),C=h.length===0?Number.POSITIVE_INFINITY:h.reduce((k,D)=>k+(i[D]-this.viewItems[D].minimumSize),0),v=h.length===0?Number.NEGATIVE_INFINITY:h.reduce((k,D)=>k+(i[D]-this.viewItems[D].maximumSize),0),w=Math.max(_,v,r),S=Math.min(C,b,a);let L=!1;if(l){const k=this.viewItems[l.index],D=t>=l.limitDelta;L=D!==k.visible,k.setVisible(D,l.size)}if(!L&&c){const k=this.viewItems[c.index],D=t<c.limitDelta;L=D!==k.visible,k.setVisible(D,c.size)}if(L)return this.resize(e,t,i,n,s,r,a);t=os(t,w,S);for(let k=0,D=t;k<u.length;k++){const N=u[k],B=os(g[k]+D,N.minimumSize,N.maximumSize),W=B-g[k];D-=W,N.size=B}for(let k=0,D=t;k<f.length;k++){const N=f[k],B=os(p[k]-D,N.minimumSize,N.maximumSize),W=B-p[k];D+=W,N.size=B}return t}distributeEmptySpace(e){const t=this.viewItems.reduce((a,l)=>a+l.size,0);let i=this.size-t;const n=Is(this.viewItems.length-1,-1),s=n.filter(a=>this.viewItems[a].priority===1),r=n.filter(a=>this.viewItems[a].priority===2);for(const a of r)iN(n,a);for(const a of s)LS(n,a);typeof e=="number"&&LS(n,e);for(let a=0;i!==0&&a<n.length;a++){const l=this.viewItems[n[a]],c=os(l.size+i,l.minimumSize,l.maximumSize),d=c-l.size;i-=d,l.size=c}}layoutViews(){this._contentSize=this.viewItems.reduce((t,i)=>t+i.size,0);let e=0;for(const t of this.viewItems)t.layout(e,this.layoutContext),e+=t.size;this.sashItems.forEach(t=>t.sash.layout()),this.updateSashEnablement(),this.updateScrollableElement()}updateScrollableElement(){this.orientation===0?this.scrollableElement.setScrollDimensions({height:this.size,scrollHeight:this._contentSize}):this.scrollableElement.setScrollDimensions({width:this.size,scrollWidth:this._contentSize})}updateSashEnablement(){let e=!1;const t=this.viewItems.map(l=>e=l.size-l.minimumSize>0||e);e=!1;const i=this.viewItems.map(l=>e=l.maximumSize-l.size>0||e),n=[...this.viewItems].reverse();e=!1;const s=n.map(l=>e=l.size-l.minimumSize>0||e).reverse();e=!1;const r=n.map(l=>e=l.maximumSize-l.size>0||e).reverse();let a=0;for(let l=0;l<this.sashItems.length;l++){const{sash:c}=this.sashItems[l],d=this.viewItems[l];a+=d.size;const h=!(t[l]&&r[l+1]),u=!(i[l]&&s[l+1]);if(h&&u){const g=Is(l,-1),f=Is(l+1,this.viewItems.length),p=this.findFirstSnapIndex(g),_=this.findFirstSnapIndex(f),b=typeof p=="number"&&!this.viewItems[p].visible,C=typeof _=="number"&&!this.viewItems[_].visible;b&&s[l]&&(a>0||this.startSnappingEnabled)?c.state=1:C&&t[l]&&(a<this._contentSize||this.endSnappingEnabled)?c.state=2:c.state=0}else h&&!u?c.state=1:!h&&u?c.state=2:c.state=3}}getSashPosition(e){let t=0;for(let i=0;i<this.sashItems.length;i++)if(t+=this.viewItems[i].size,this.sashItems[i].sash===e)return t;return 0}findFirstSnapIndex(e){for(const t of e){const i=this.viewItems[t];if(i.visible&&i.snap)return t}for(const t of e){const i=this.viewItems[t];if(i.visible&&i.maximumSize-i.minimumSize>0)return;if(!i.visible&&i.snap)return t}}areViewsDistributed(){let e,t;for(const i of this.viewItems)if(e=e===void 0?i.size:Math.min(e,i.size),t=t===void 0?i.size:Math.max(t,i.size),t-e>2)return!1;return!0}dispose(){var e;(e=this.sashDragState)==null||e.disposable.dispose(),Tt(this.viewItems),this.viewItems=[],this.sashItems.forEach(t=>t.disposable.dispose()),this.sashItems=[],super.dispose()}}const Yk=class Yk{constructor(e,t,i){this.columns=e,this.getColumnSize=i,this.templateId=Yk.TemplateId,this.renderedTemplates=new Set;const n=new Map(t.map(s=>[s.templateId,s]));this.renderers=[];for(const s of e){const r=n.get(s.templateId);if(!r)throw new Error(`Table cell renderer for template id ${s.templateId} not found.`);this.renderers.push(r)}}renderTemplate(e){const t=X(e,ne(".monaco-table-tr")),i=[],n=[];for(let r=0;r<this.columns.length;r++){const a=this.renderers[r],l=X(t,ne(".monaco-table-td",{"data-col-index":r}));l.style.width=`${this.getColumnSize(r)}px`,i.push(l),n.push(a.renderTemplate(l))}const s={container:e,cellContainers:i,cellTemplateData:n};return this.renderedTemplates.add(s),s}renderElement(e,t,i,n){for(let s=0;s<this.columns.length;s++){const a=this.columns[s].project(e);this.renderers[s].renderElement(a,t,i.cellTemplateData[s],n)}}disposeElement(e,t,i,n){for(let s=0;s<this.columns.length;s++){const r=this.renderers[s];if(r.disposeElement){const l=this.columns[s].project(e);r.disposeElement(l,t,i.cellTemplateData[s],n)}}}disposeTemplate(e){for(let t=0;t<this.columns.length;t++)this.renderers[t].disposeTemplate(e.cellTemplateData[t]);pn(e.container),this.renderedTemplates.delete(e)}layoutColumn(e,t){for(const{cellContainers:i}of this.renderedTemplates)i[e].style.width=`${t}px`}};Yk.TemplateId="row";let VL=Yk;function efe(o){return{getHeight(e){return o.getHeight(e)},getTemplateId(){return VL.TemplateId}}}class tfe extends F{get minimumSize(){return this.column.minimumWidth??120}get maximumSize(){return this.column.maximumWidth??Number.POSITIVE_INFINITY}get onDidChange(){return this.column.onDidChangeWidthConstraints??oe.None}constructor(e,t){super(),this.column=e,this.index=t,this._onDidLayout=new P,this.onDidLayout=this._onDidLayout.event,this.element=ne(".monaco-table-th",{"data-col-index":t},e.label),e.tooltip&&this._register(ja().setupManagedHover(Ss("mouse"),this.element,e.tooltip))}layout(e){this._onDidLayout.fire([this.index,e])}}const Xk=class Xk{get onDidChangeFocus(){return this.list.onDidChangeFocus}get onDidChangeSelection(){return this.list.onDidChangeSelection}get onDidScroll(){return this.list.onDidScroll}get onMouseDblClick(){return this.list.onMouseDblClick}get onPointer(){return this.list.onPointer}get onDidFocus(){return this.list.onDidFocus}get scrollTop(){return this.list.scrollTop}set scrollTop(e){this.list.scrollTop=e}get scrollHeight(){return this.list.scrollHeight}get renderHeight(){return this.list.renderHeight}get onDidDispose(){return this.list.onDidDispose}constructor(e,t,i,n,s,r){this.virtualDelegate=i,this.columns=n,this.domId=`table_id_${++Xk.InstanceCount}`,this.disposables=new K,this.cachedWidth=0,this.cachedHeight=0,this.domNode=X(t,ne(`.monaco-table.${this.domId}`));const a=n.map((d,h)=>this.disposables.add(new tfe(d,h))),l={size:a.reduce((d,h)=>d+h.column.weight,0),views:a.map(d=>({size:d.column.weight,view:d}))};this.splitview=this.disposables.add(new DK(this.domNode,{orientation:1,scrollbarVisibility:2,getSashOrthogonalSize:()=>this.cachedHeight,descriptor:l})),this.splitview.el.style.height=`${i.headerRowHeight}px`,this.splitview.el.style.lineHeight=`${i.headerRowHeight}px`;const c=new VL(n,s,d=>this.splitview.getViewSize(d));this.list=this.disposables.add(new Xo(e,this.domNode,efe(i),[c],r)),oe.any(...a.map(d=>d.onDidLayout))(([d,h])=>c.layoutColumn(d,h),null,this.disposables),this.splitview.onDidSashReset(d=>{const h=n.reduce((g,f)=>g+f.weight,0),u=n[d].weight/h*this.cachedWidth;this.splitview.resizeView(d,u)},null,this.disposables),this.styleElement=$o(this.domNode),this.style(kde)}updateOptions(e){this.list.updateOptions(e)}splice(e,t,i=[]){this.list.splice(e,t,i)}getHTMLElement(){return this.domNode}style(e){const t=[];t.push(`.monaco-table.${this.domId} > .monaco-split-view2 .monaco-sash.vertical::before {
			top: ${this.virtualDelegate.headerRowHeight+1}px;
			height: calc(100% - ${this.virtualDelegate.headerRowHeight}px);
		}`),this.styleElement.textContent=t.join(`
`),this.list.style(e)}getSelectedElements(){return this.list.getSelectedElements()}getSelection(){return this.list.getSelection()}getFocus(){return this.list.getFocus()}dispose(){this.disposables.dispose()}};Xk.InstanceCount=0;let wA=Xk;var Do;(function(o){o[o.Expanded=0]="Expanded",o[o.Collapsed=1]="Collapsed",o[o.PreserveOrExpanded=2]="PreserveOrExpanded",o[o.PreserveOrCollapsed=3]="PreserveOrCollapsed"})(Do||(Do={}));var dg;(function(o){o[o.Unknown=0]="Unknown",o[o.Twistie=1]="Twistie",o[o.Element=2]="Element",o[o.Filter=3]="Filter"})(dg||(dg={}));class Ao extends Error{constructor(e,t){super(`TreeError [${e}] ${t}`)}}class C3{constructor(e){this.fn=e,this._map=new WeakMap}map(e){let t=this._map.get(e);return t||(t=this.fn(e),this._map.set(e,t)),t}}function v3(o){return typeof o=="object"&&"visibility"in o&&"data"in o}function Pw(o){switch(o){case!0:return 1;case!1:return 0;default:return o}}function lT(o){return typeof o.collapsible=="boolean"}class ife{constructor(e,t,i,n={}){this.user=e,this.list=t,this.rootRef=[],this.eventBufferer=new R1,this._onDidChangeCollapseState=new P,this.onDidChangeCollapseState=this.eventBufferer.wrapEvent(this._onDidChangeCollapseState.event),this._onDidChangeRenderNodeCount=new P,this.onDidChangeRenderNodeCount=this.eventBufferer.wrapEvent(this._onDidChangeRenderNodeCount.event),this._onDidSplice=new P,this.onDidSplice=this._onDidSplice.event,this.refilterDelayer=new _r(cU),this.collapseByDefault=typeof n.collapseByDefault>"u"?!1:n.collapseByDefault,this.allowNonCollapsibleParents=n.allowNonCollapsibleParents??!1,this.filter=n.filter,this.autoExpandSingleChildren=typeof n.autoExpandSingleChildren>"u"?!1:n.autoExpandSingleChildren,this.root={parent:void 0,element:i,children:[],depth:0,visibleChildrenCount:0,visibleChildIndex:-1,collapsible:!1,collapsed:!1,renderNodeCount:0,visibility:1,visible:!0,filterData:void 0}}splice(e,t,i=ot.empty(),n={}){if(e.length===0)throw new Ao(this.user,"Invalid tree location");n.diffIdentityProvider?this.spliceSmart(n.diffIdentityProvider,e,t,i,n):this.spliceSimple(e,t,i,n)}spliceSmart(e,t,i,n=ot.empty(),s,r=s.diffDepth??0){const{parentNode:a}=this.getParentNodeWithListIndex(t);if(!a.lastDiffIds)return this.spliceSimple(t,i,n,s);const l=[...n],c=t[t.length-1],d=new yl({getElements:()=>a.lastDiffIds},{getElements:()=>[...a.children.slice(0,c),...l,...a.children.slice(c+i)].map(p=>e.getId(p.element).toString())}).ComputeDiff(!1);if(d.quitEarly)return a.lastDiffIds=void 0,this.spliceSimple(t,i,l,s);const h=t.slice(0,-1),u=(p,_,b)=>{if(r>0)for(let C=0;C<b;C++)p--,_--,this.spliceSmart(e,[...h,p,0],Number.MAX_SAFE_INTEGER,l[_].children,s,r-1)};let g=Math.min(a.children.length,c+i),f=l.length;for(const p of d.changes.sort((_,b)=>b.originalStart-_.originalStart))u(g,f,g-(p.originalStart+p.originalLength)),g=p.originalStart,f=p.modifiedStart-c,this.spliceSimple([...h,g],p.originalLength,ot.slice(l,f,f+p.modifiedLength),s);u(g,f,g)}spliceSimple(e,t,i=ot.empty(),{onDidCreateNode:n,onDidDeleteNode:s,diffIdentityProvider:r}){const{parentNode:a,listIndex:l,revealed:c,visible:d}=this.getParentNodeWithListIndex(e),h=[],u=ot.map(i,S=>this.createTreeNode(S,a,a.visible?1:0,c,h,n)),g=e[e.length-1];let f=0;for(let S=g;S>=0&&S<a.children.length;S--){const L=a.children[S];if(L.visible){f=L.visibleChildIndex;break}}const p=[];let _=0,b=0;for(const S of u)p.push(S),b+=S.renderNodeCount,S.visible&&(S.visibleChildIndex=f+_++);const C=BB(a.children,g,t,p);r?a.lastDiffIds?BB(a.lastDiffIds,g,t,p.map(S=>r.getId(S.element).toString())):a.lastDiffIds=a.children.map(S=>r.getId(S.element).toString()):a.lastDiffIds=void 0;let v=0;for(const S of C)S.visible&&v++;if(v!==0)for(let S=g+p.length;S<a.children.length;S++){const L=a.children[S];L.visible&&(L.visibleChildIndex-=v)}if(a.visibleChildrenCount+=_-v,c&&d){const S=C.reduce((L,k)=>L+(k.visible?k.renderNodeCount:0),0);this._updateAncestorsRenderNodeCount(a,b-S),this.list.splice(l,S,h)}if(C.length>0&&s){const S=L=>{s(L),L.children.forEach(S)};C.forEach(S)}this._onDidSplice.fire({insertedNodes:p,deletedNodes:C});let w=a;for(;w;){if(w.visibility===2){this.refilterDelayer.trigger(()=>this.refilter());break}w=w.parent}}rerender(e){if(e.length===0)throw new Ao(this.user,"Invalid tree location");const{node:t,listIndex:i,revealed:n}=this.getTreeNodeWithListIndex(e);t.visible&&n&&this.list.splice(i,1,[t])}has(e){return this.hasTreeNode(e)}getListIndex(e){const{listIndex:t,visible:i,revealed:n}=this.getTreeNodeWithListIndex(e);return i&&n?t:-1}getListRenderCount(e){return this.getTreeNode(e).renderNodeCount}isCollapsible(e){return this.getTreeNode(e).collapsible}setCollapsible(e,t){const i=this.getTreeNode(e);typeof t>"u"&&(t=!i.collapsible);const n={collapsible:t};return this.eventBufferer.bufferEvents(()=>this._setCollapseState(e,n))}isCollapsed(e){return this.getTreeNode(e).collapsed}setCollapsed(e,t,i){const n=this.getTreeNode(e);typeof t>"u"&&(t=!n.collapsed);const s={collapsed:t,recursive:i||!1};return this.eventBufferer.bufferEvents(()=>this._setCollapseState(e,s))}_setCollapseState(e,t){const{node:i,listIndex:n,revealed:s}=this.getTreeNodeWithListIndex(e),r=this._setListNodeCollapseState(i,n,s,t);if(i!==this.root&&this.autoExpandSingleChildren&&r&&!lT(t)&&i.collapsible&&!i.collapsed&&!t.recursive){let a=-1;for(let l=0;l<i.children.length;l++)if(i.children[l].visible)if(a>-1){a=-1;break}else a=l;a>-1&&this._setCollapseState([...e,a],t)}return r}_setListNodeCollapseState(e,t,i,n){const s=this._setNodeCollapseState(e,n,!1);if(!i||!e.visible||!s)return s;const r=e.renderNodeCount,a=this.updateNodeAfterCollapseChange(e),l=r-(t===-1?0:1);return this.list.splice(t+1,l,a.slice(1)),s}_setNodeCollapseState(e,t,i){let n;if(e===this.root?n=!1:(lT(t)?(n=e.collapsible!==t.collapsible,e.collapsible=t.collapsible):e.collapsible?(n=e.collapsed!==t.collapsed,e.collapsed=t.collapsed):n=!1,n&&this._onDidChangeCollapseState.fire({node:e,deep:i})),!lT(t)&&t.recursive)for(const s of e.children)n=this._setNodeCollapseState(s,t,!0)||n;return n}expandTo(e){this.eventBufferer.bufferEvents(()=>{let t=this.getTreeNode(e);for(;t.parent;)t=t.parent,e=e.slice(0,e.length-1),t.collapsed&&this._setCollapseState(e,{collapsed:!1,recursive:!1})})}refilter(){const e=this.root.renderNodeCount,t=this.updateNodeAfterFilterChange(this.root);this.list.splice(0,e,t),this.refilterDelayer.cancel()}createTreeNode(e,t,i,n,s,r){const a={parent:t,element:e.element,children:[],depth:t.depth+1,visibleChildrenCount:0,visibleChildIndex:-1,collapsible:typeof e.collapsible=="boolean"?e.collapsible:typeof e.collapsed<"u",collapsed:typeof e.collapsed>"u"?this.collapseByDefault:e.collapsed,renderNodeCount:1,visibility:1,visible:!0,filterData:void 0},l=this._filterNode(a,i);a.visibility=l,n&&s.push(a);const c=e.children||ot.empty(),d=n&&l!==0&&!a.collapsed;let h=0,u=1;for(const g of c){const f=this.createTreeNode(g,a,l,d,s,r);a.children.push(f),u+=f.renderNodeCount,f.visible&&(f.visibleChildIndex=h++)}return this.allowNonCollapsibleParents||(a.collapsible=a.collapsible||a.children.length>0),a.visibleChildrenCount=h,a.visible=l===2?h>0:l===1,a.visible?a.collapsed||(a.renderNodeCount=u):(a.renderNodeCount=0,n&&s.pop()),r==null||r(a),a}updateNodeAfterCollapseChange(e){const t=e.renderNodeCount,i=[];return this._updateNodeAfterCollapseChange(e,i),this._updateAncestorsRenderNodeCount(e.parent,i.length-t),i}_updateNodeAfterCollapseChange(e,t){if(e.visible===!1)return 0;if(t.push(e),e.renderNodeCount=1,!e.collapsed)for(const i of e.children)e.renderNodeCount+=this._updateNodeAfterCollapseChange(i,t);return this._onDidChangeRenderNodeCount.fire(e),e.renderNodeCount}updateNodeAfterFilterChange(e){const t=e.renderNodeCount,i=[];return this._updateNodeAfterFilterChange(e,e.visible?1:0,i),this._updateAncestorsRenderNodeCount(e.parent,i.length-t),i}_updateNodeAfterFilterChange(e,t,i,n=!0){let s;if(e!==this.root){if(s=this._filterNode(e,t),s===0)return e.visible=!1,e.renderNodeCount=0,!1;n&&i.push(e)}const r=i.length;e.renderNodeCount=e===this.root?0:1;let a=!1;if(!e.collapsed||s!==0){let l=0;for(const c of e.children)a=this._updateNodeAfterFilterChange(c,s,i,n&&!e.collapsed)||a,c.visible&&(c.visibleChildIndex=l++);e.visibleChildrenCount=l}else e.visibleChildrenCount=0;return e!==this.root&&(e.visible=s===2?a:s===1,e.visibility=s),e.visible?e.collapsed||(e.renderNodeCount+=i.length-r):(e.renderNodeCount=0,n&&i.pop()),this._onDidChangeRenderNodeCount.fire(e),e.visible}_updateAncestorsRenderNodeCount(e,t){if(t!==0)for(;e;)e.renderNodeCount+=t,this._onDidChangeRenderNodeCount.fire(e),e=e.parent}_filterNode(e,t){const i=this.filter?this.filter.filter(e.element,t):1;return typeof i=="boolean"?(e.filterData=void 0,i?1:0):v3(i)?(e.filterData=i.data,Pw(i.visibility)):(e.filterData=void 0,Pw(i))}hasTreeNode(e,t=this.root){if(!e||e.length===0)return!0;const[i,...n]=e;return i<0||i>t.children.length?!1:this.hasTreeNode(n,t.children[i])}getTreeNode(e,t=this.root){if(!e||e.length===0)return t;const[i,...n]=e;if(i<0||i>t.children.length)throw new Ao(this.user,"Invalid tree location");return this.getTreeNode(n,t.children[i])}getTreeNodeWithListIndex(e){if(e.length===0)return{node:this.root,listIndex:-1,revealed:!0,visible:!1};const{parentNode:t,listIndex:i,revealed:n,visible:s}=this.getParentNodeWithListIndex(e),r=e[e.length-1];if(r<0||r>t.children.length)throw new Ao(this.user,"Invalid tree location");const a=t.children[r];return{node:a,listIndex:i,revealed:n,visible:s&&a.visible}}getParentNodeWithListIndex(e,t=this.root,i=0,n=!0,s=!0){const[r,...a]=e;if(r<0||r>t.children.length)throw new Ao(this.user,"Invalid tree location");for(let l=0;l<r;l++)i+=t.children[l].renderNodeCount;return n=n&&!t.collapsed,s=s&&t.visible,a.length===0?{parentNode:t,listIndex:i,revealed:n,visible:s}:this.getParentNodeWithListIndex(a,t.children[r],i+1,n,s)}getNode(e=[]){return this.getTreeNode(e)}getNodeLocation(e){const t=[];let i=e;for(;i.parent;)t.push(i.parent.children.indexOf(i)),i=i.parent;return t.reverse()}getParentNodeLocation(e){if(e.length!==0)return e.length===1?[]:_Q(e)[0]}getFirstElementChild(e){const t=this.getTreeNode(e);if(t.children.length!==0)return t.children[0].element}}class nfe extends X1{constructor(e){super(e.elements.map(t=>t.element)),this.data=e}}function cT(o){return o instanceof X1?new nfe(o):o}class sfe{constructor(e,t){this.modelProvider=e,this.dnd=t,this.autoExpandDisposable=F.None,this.disposables=new K}getDragURI(e){return this.dnd.getDragURI(e.element)}getDragLabel(e,t){if(this.dnd.getDragLabel)return this.dnd.getDragLabel(e.map(i=>i.element),t)}onDragStart(e,t){var i,n;(n=(i=this.dnd).onDragStart)==null||n.call(i,cT(e),t)}onDragOver(e,t,i,n,s,r=!0){const a=this.dnd.onDragOver(cT(e),t&&t.element,i,n,s),l=this.autoExpandNode!==t;if(l&&(this.autoExpandDisposable.dispose(),this.autoExpandNode=t),typeof t>"u")return a;if(l&&typeof a!="boolean"&&a.autoExpand&&(this.autoExpandDisposable=Bh(()=>{const g=this.modelProvider(),f=g.getNodeLocation(t);g.isCollapsed(f)&&g.setCollapsed(f,!1),this.autoExpandNode=void 0},500,this.disposables)),typeof a=="boolean"||!a.accept||typeof a.bubble>"u"||a.feedback){if(!r){const g=typeof a=="boolean"?a:a.accept,f=typeof a=="boolean"?void 0:a.effect;return{accept:g,effect:f,feedback:[i]}}return a}if(a.bubble===1){const g=this.modelProvider(),f=g.getNodeLocation(t),p=g.getParentNodeLocation(f),_=g.getNode(p),b=p&&g.getListIndex(p);return this.onDragOver(e,_,b,n,s,!1)}const c=this.modelProvider(),d=c.getNodeLocation(t),h=c.getListIndex(d),u=c.getListRenderCount(d);return{...a,feedback:Is(h,h+u)}}drop(e,t,i,n,s){this.autoExpandDisposable.dispose(),this.autoExpandNode=void 0,this.dnd.drop(cT(e),t&&t.element,i,n,s)}onDragEnd(e){var t,i;(i=(t=this.dnd).onDragEnd)==null||i.call(t,e)}dispose(){this.disposables.dispose(),this.dnd.dispose()}}function ofe(o,e){return e&&{...e,identityProvider:e.identityProvider&&{getId(t){return e.identityProvider.getId(t.element)}},dnd:e.dnd&&new sfe(o,e.dnd),multipleSelectionController:e.multipleSelectionController&&{isSelectionSingleChangeEvent(t){return e.multipleSelectionController.isSelectionSingleChangeEvent({...t,element:t.element})},isSelectionRangeChangeEvent(t){return e.multipleSelectionController.isSelectionRangeChangeEvent({...t,element:t.element})}},accessibilityProvider:e.accessibilityProvider&&{...e.accessibilityProvider,getSetSize(t){const i=o(),n=i.getNodeLocation(t),s=i.getParentNodeLocation(n);return i.getNode(s).visibleChildrenCount},getPosInSet(t){return t.visibleChildIndex+1},isChecked:e.accessibilityProvider&&e.accessibilityProvider.isChecked?t=>e.accessibilityProvider.isChecked(t.element):void 0,getRole:e.accessibilityProvider&&e.accessibilityProvider.getRole?t=>e.accessibilityProvider.getRole(t.element):()=>"treeitem",getAriaLabel(t){return e.accessibilityProvider.getAriaLabel(t.element)},getWidgetAriaLabel(){return e.accessibilityProvider.getWidgetAriaLabel()},getWidgetRole:e.accessibilityProvider&&e.accessibilityProvider.getWidgetRole?()=>e.accessibilityProvider.getWidgetRole():()=>"tree",getAriaLevel:e.accessibilityProvider&&e.accessibilityProvider.getAriaLevel?t=>e.accessibilityProvider.getAriaLevel(t.element):t=>t.depth,getActiveDescendantId:e.accessibilityProvider.getActiveDescendantId&&(t=>e.accessibilityProvider.getActiveDescendantId(t.element))},keyboardNavigationLabelProvider:e.keyboardNavigationLabelProvider&&{...e.keyboardNavigationLabelProvider,getKeyboardNavigationLabel(t){return e.keyboardNavigationLabelProvider.getKeyboardNavigationLabel(t.element)}}}}class w3{constructor(e){this.delegate=e}getHeight(e){return this.delegate.getHeight(e.element)}getTemplateId(e){return this.delegate.getTemplateId(e.element)}hasDynamicHeight(e){return!!this.delegate.hasDynamicHeight&&this.delegate.hasDynamicHeight(e.element)}setDynamicHeight(e,t){var i,n;(n=(i=this.delegate).setDynamicHeight)==null||n.call(i,e.element,t)}}var U_;(function(o){o.None="none",o.OnHover="onHover",o.Always="always"})(U_||(U_={}));class rfe{get elements(){return this._elements}constructor(e,t=[]){this._elements=t,this.disposables=new K,this.onDidChange=oe.forEach(e,i=>this._elements=i,this.disposables)}dispose(){this.disposables.dispose()}}const Fv=class Fv{constructor(e,t,i,n,s,r={}){var a;this.renderer=e,this.modelProvider=t,this.activeNodes=n,this.renderedIndentGuides=s,this.renderedElements=new Map,this.renderedNodes=new Map,this.indent=Fv.DefaultIndent,this.hideTwistiesOfChildlessElements=!1,this.shouldRenderIndentGuides=!1,this.activeIndentNodes=new Set,this.indentGuidesDisposable=F.None,this.disposables=new K,this.templateId=e.templateId,this.updateOptions(r),oe.map(i,l=>l.node)(this.onDidChangeNodeTwistieState,this,this.disposables),(a=e.onDidChangeTwistieState)==null||a.call(e,this.onDidChangeTwistieState,this,this.disposables)}updateOptions(e={}){if(typeof e.indent<"u"){const t=os(e.indent,0,40);if(t!==this.indent){this.indent=t;for(const[i,n]of this.renderedNodes)this.renderTreeElement(i,n)}}if(typeof e.renderIndentGuides<"u"){const t=e.renderIndentGuides!==U_.None;if(t!==this.shouldRenderIndentGuides){this.shouldRenderIndentGuides=t;for(const[i,n]of this.renderedNodes)this._renderIndentGuides(i,n);if(this.indentGuidesDisposable.dispose(),t){const i=new K;this.activeNodes.onDidChange(this._onDidChangeActiveNodes,this,i),this.indentGuidesDisposable=i,this._onDidChangeActiveNodes(this.activeNodes.elements)}}}typeof e.hideTwistiesOfChildlessElements<"u"&&(this.hideTwistiesOfChildlessElements=e.hideTwistiesOfChildlessElements)}renderTemplate(e){const t=X(e,ne(".monaco-tl-row")),i=X(t,ne(".monaco-tl-indent")),n=X(t,ne(".monaco-tl-twistie")),s=X(t,ne(".monaco-tl-contents")),r=this.renderer.renderTemplate(s);return{container:e,indent:i,twistie:n,indentGuidesDisposable:F.None,templateData:r}}renderElement(e,t,i,n){this.renderedNodes.set(e,i),this.renderedElements.set(e.element,e),this.renderTreeElement(e,i),this.renderer.renderElement(e,t,i.templateData,n)}disposeElement(e,t,i,n){var s,r;i.indentGuidesDisposable.dispose(),(r=(s=this.renderer).disposeElement)==null||r.call(s,e,t,i.templateData,n),typeof n=="number"&&(this.renderedNodes.delete(e),this.renderedElements.delete(e.element))}disposeTemplate(e){this.renderer.disposeTemplate(e.templateData)}onDidChangeTwistieState(e){const t=this.renderedElements.get(e);t&&this.onDidChangeNodeTwistieState(t)}onDidChangeNodeTwistieState(e){const t=this.renderedNodes.get(e);t&&(this._onDidChangeActiveNodes(this.activeNodes.elements),this.renderTreeElement(e,t))}renderTreeElement(e,t){const i=Fv.DefaultIndent+(e.depth-1)*this.indent;t.twistie.style.paddingLeft=`${i}px`,t.indent.style.width=`${i+this.indent-16}px`,e.collapsible?t.container.setAttribute("aria-expanded",String(!e.collapsed)):t.container.removeAttribute("aria-expanded"),t.twistie.classList.remove(...Ie.asClassNameArray(te.treeItemExpanded));let n=!1;this.renderer.renderTwistie&&(n=this.renderer.renderTwistie(e.element,t.twistie)),e.collapsible&&(!this.hideTwistiesOfChildlessElements||e.visibleChildrenCount>0)?(n||t.twistie.classList.add(...Ie.asClassNameArray(te.treeItemExpanded)),t.twistie.classList.add("collapsible"),t.twistie.classList.toggle("collapsed",e.collapsed)):t.twistie.classList.remove("collapsible","collapsed"),this._renderIndentGuides(e,t)}_renderIndentGuides(e,t){if(pn(t.indent),t.indentGuidesDisposable.dispose(),!this.shouldRenderIndentGuides)return;const i=new K,n=this.modelProvider();for(;;){const s=n.getNodeLocation(e),r=n.getParentNodeLocation(s);if(!r)break;const a=n.getNode(r),l=ne(".indent-guide",{style:`width: ${this.indent}px`});this.activeIndentNodes.has(a)&&l.classList.add("active"),t.indent.childElementCount===0?t.indent.appendChild(l):t.indent.insertBefore(l,t.indent.firstElementChild),this.renderedIndentGuides.add(a,l),i.add(ve(()=>this.renderedIndentGuides.delete(a,l))),e=a}t.indentGuidesDisposable=i}_onDidChangeActiveNodes(e){if(!this.shouldRenderIndentGuides)return;const t=new Set,i=this.modelProvider();e.forEach(n=>{const s=i.getNodeLocation(n);try{const r=i.getParentNodeLocation(s);n.collapsible&&n.children.length>0&&!n.collapsed?t.add(n):r&&t.add(i.getNode(r))}catch{}}),this.activeIndentNodes.forEach(n=>{t.has(n)||this.renderedIndentGuides.forEach(n,s=>s.classList.remove("active"))}),t.forEach(n=>{this.activeIndentNodes.has(n)||this.renderedIndentGuides.forEach(n,s=>s.classList.add("active"))}),this.activeIndentNodes=t}dispose(){this.renderedNodes.clear(),this.renderedElements.clear(),this.indentGuidesDisposable.dispose(),Tt(this.disposables)}};Fv.DefaultIndent=8;let SA=Fv;class afe{get totalCount(){return this._totalCount}get matchCount(){return this._matchCount}constructor(e,t,i){this.tree=e,this.keyboardNavigationLabelProvider=t,this._filter=i,this._totalCount=0,this._matchCount=0,this._pattern="",this._lowercasePattern="",this.disposables=new K,e.onWillRefilter(this.reset,this,this.disposables)}filter(e,t){let i=1;if(this._filter){const r=this._filter.filter(e,t);if(typeof r=="boolean"?i=r?1:0:v3(r)?i=Pw(r.visibility):i=r,i===0)return!1}if(this._totalCount++,!this._pattern)return this._matchCount++,{data:Ma.Default,visibility:i};const n=this.keyboardNavigationLabelProvider.getKeyboardNavigationLabel(e),s=Array.isArray(n)?n:[n];for(const r of s){const a=r&&r.toString();if(typeof a>"u")return{data:Ma.Default,visibility:i};let l;if(this.tree.findMatchType===Ff.Contiguous){const c=a.toLowerCase().indexOf(this._lowercasePattern);if(c>-1){l=[Number.MAX_SAFE_INTEGER,0];for(let d=this._lowercasePattern.length;d>0;d--)l.push(c+d-1)}}else l=Mf(this._pattern,this._lowercasePattern,0,a,a.toLowerCase(),0,{firstMatchCanBeWeak:!0,boostFullMatch:!0});if(l)return this._matchCount++,s.length===1?{data:l,visibility:i}:{data:{label:a,score:l},visibility:i}}return this.tree.findMode===Oc.Filter?typeof this.tree.options.defaultFindVisibility=="number"?this.tree.options.defaultFindVisibility:this.tree.options.defaultFindVisibility?this.tree.options.defaultFindVisibility(e):2:{data:Ma.Default,visibility:i}}reset(){this._totalCount=0,this._matchCount=0}dispose(){Tt(this.disposables)}}var Oc;(function(o){o[o.Highlight=0]="Highlight",o[o.Filter=1]="Filter"})(Oc||(Oc={}));var Ff;(function(o){o[o.Fuzzy=0]="Fuzzy",o[o.Contiguous=1]="Contiguous"})(Ff||(Ff={}));let lfe=class{get pattern(){return this._pattern}get mode(){return this._mode}set mode(e){e!==this._mode&&(this._mode=e,this.widget&&(this.widget.mode=this._mode),this.tree.refilter(),this.render(),this._onDidChangeMode.fire(e))}get matchType(){return this._matchType}set matchType(e){e!==this._matchType&&(this._matchType=e,this.widget&&(this.widget.matchType=this._matchType),this.tree.refilter(),this.render(),this._onDidChangeMatchType.fire(e))}constructor(e,t,i,n,s,r={}){this.tree=e,this.view=i,this.filter=n,this.contextViewProvider=s,this.options=r,this._pattern="",this.width=0,this._onDidChangeMode=new P,this.onDidChangeMode=this._onDidChangeMode.event,this._onDidChangeMatchType=new P,this.onDidChangeMatchType=this._onDidChangeMatchType.event,this._onDidChangePattern=new P,this._onDidChangeOpenState=new P,this.onDidChangeOpenState=this._onDidChangeOpenState.event,this.enabledDisposables=new K,this.disposables=new K,this._mode=e.options.defaultFindMode??Oc.Highlight,this._matchType=e.options.defaultFindMatchType??Ff.Fuzzy,t.onDidSplice(this.onDidSpliceModel,this,this.disposables)}updateOptions(e={}){e.defaultFindMode!==void 0&&(this.mode=e.defaultFindMode),e.defaultFindMatchType!==void 0&&(this.matchType=e.defaultFindMatchType)}onDidSpliceModel(){!this.widget||this.pattern.length===0||(this.tree.refilter(),this.render())}render(){var t,i,n;const e=this.filter.totalCount>0&&this.filter.matchCount===0;this.pattern&&e?(Cs(m("replFindNoResults","No results")),this.tree.options.showNotFoundMessage??!0?(t=this.widget)==null||t.showMessage({type:2,content:m("not found","No elements found.")}):(i=this.widget)==null||i.showMessage({type:2})):((n=this.widget)==null||n.clearMessage(),this.pattern&&Cs(m("replFindResults","{0} results",this.filter.matchCount)))}shouldAllowFocus(e){return!this.widget||!this.pattern||this.filter.totalCount>0&&this.filter.matchCount<=1?!0:!Ma.isDefault(e.filterData)}layout(e){var t;this.width=e,(t=this.widget)==null||t.layout(e)}dispose(){this._history=void 0,this._onDidChangePattern.dispose(),this.enabledDisposables.dispose(),this.disposables.dispose()}};function cfe(o,e){return o.position===e.position&&IK(o,e)}function IK(o,e){return o.node.element===e.node.element&&o.startIndex===e.startIndex&&o.height===e.height&&o.endIndex===e.endIndex}class dfe{constructor(e=[]){this.stickyNodes=e}get count(){return this.stickyNodes.length}equal(e){return ii(this.stickyNodes,e.stickyNodes,cfe)}lastNodePartiallyVisible(){if(this.count===0)return!1;const e=this.stickyNodes[this.count-1];if(this.count===1)return e.position!==0;const t=this.stickyNodes[this.count-2];return t.position+t.height!==e.position}animationStateChanged(e){if(!ii(this.stickyNodes,e.stickyNodes,IK)||this.count===0)return!1;const t=this.stickyNodes[this.count-1],i=e.stickyNodes[e.count-1];return t.position!==i.position}}class hfe{constrainStickyScrollNodes(e,t,i){for(let n=0;n<e.length;n++){const s=e[n];if(s.position+s.height>i||n>=t)return e.slice(0,n)}return e}}let z8=class extends F{constructor(e,t,i,n,s,r={}){super(),this.tree=e,this.model=t,this.view=i,this.treeDelegate=s,this.maxWidgetViewRatio=.4;const a=this.validateStickySettings(r);this.stickyScrollMaxItemCount=a.stickyScrollMaxItemCount,this.stickyScrollDelegate=r.stickyScrollDelegate??new hfe,this._widget=this._register(new ufe(i.getScrollableElement(),i,e,n,s,r.accessibilityProvider)),this.onDidChangeHasFocus=this._widget.onDidChangeHasFocus,this.onContextMenu=this._widget.onContextMenu,this._register(i.onDidScroll(()=>this.update())),this._register(i.onDidChangeContentHeight(()=>this.update())),this._register(e.onDidChangeCollapseState(()=>this.update())),this.update()}get height(){return this._widget.height}getNodeAtHeight(e){let t;if(e===0?t=this.view.firstVisibleIndex:t=this.view.indexAt(e+this.view.scrollTop),!(t<0||t>=this.view.length))return this.view.element(t)}update(){const e=this.getNodeAtHeight(0);if(!e||this.tree.scrollTop===0){this._widget.setState(void 0);return}const t=this.findStickyState(e);this._widget.setState(t)}findStickyState(e){const t=[];let i=e,n=0,s=this.getNextStickyNode(i,void 0,n);for(;s&&(t.push(s),n+=s.height,!(t.length<=this.stickyScrollMaxItemCount&&(i=this.getNextVisibleNode(s),!i)));)s=this.getNextStickyNode(i,s.node,n);const r=this.constrainStickyNodes(t);return r.length?new dfe(r):void 0}getNextVisibleNode(e){return this.getNodeAtHeight(e.position+e.height)}getNextStickyNode(e,t,i){const n=this.getAncestorUnderPrevious(e,t);if(n&&!(n===e&&(!this.nodeIsUncollapsedParent(e)||this.nodeTopAlignsWithStickyNodesBottom(e,i))))return this.createStickyScrollNode(n,i)}nodeTopAlignsWithStickyNodesBottom(e,t){const i=this.getNodeIndex(e),n=this.view.getElementTop(i),s=t;return this.view.scrollTop===n-s}createStickyScrollNode(e,t){const i=this.treeDelegate.getHeight(e),{startIndex:n,endIndex:s}=this.getNodeRange(e),r=this.calculateStickyNodePosition(s,t,i);return{node:e,position:r,height:i,startIndex:n,endIndex:s}}getAncestorUnderPrevious(e,t=void 0){let i=e,n=this.getParentNode(i);for(;n;){if(n===t)return i;i=n,n=this.getParentNode(i)}if(t===void 0)return i}calculateStickyNodePosition(e,t,i){let n=this.view.getRelativeTop(e);if(n===null&&this.view.firstVisibleIndex===e&&e+1<this.view.length){const c=this.treeDelegate.getHeight(this.view.element(e)),d=this.view.getRelativeTop(e+1);n=d?d-c/this.view.renderHeight:null}if(n===null)return t;const s=this.view.element(e),r=this.treeDelegate.getHeight(s),l=n*this.view.renderHeight+r;return t+i>l&&t<=l?l-i:t}constrainStickyNodes(e){if(e.length===0)return[];const t=this.view.renderHeight*this.maxWidgetViewRatio,i=e[e.length-1];if(e.length<=this.stickyScrollMaxItemCount&&i.position+i.height<=t)return e;const n=this.stickyScrollDelegate.constrainStickyScrollNodes(e,this.stickyScrollMaxItemCount,t);if(!n.length)return[];const s=n[n.length-1];if(n.length>this.stickyScrollMaxItemCount||s.position+s.height>t)throw new Error("stickyScrollDelegate violates constraints");return n}getParentNode(e){const t=this.model.getNodeLocation(e),i=this.model.getParentNodeLocation(t);return i?this.model.getNode(i):void 0}nodeIsUncollapsedParent(e){const t=this.model.getNodeLocation(e);return this.model.getListRenderCount(t)>1}getNodeIndex(e){const t=this.model.getNodeLocation(e);return this.model.getListIndex(t)}getNodeRange(e){const t=this.model.getNodeLocation(e),i=this.model.getListIndex(t);if(i<0)throw new Error("Node not found in tree");const n=this.model.getListRenderCount(t),s=i+n-1;return{startIndex:i,endIndex:s}}nodePositionTopBelowWidget(e){const t=[];let i=this.getParentNode(e);for(;i;)t.push(i),i=this.getParentNode(i);let n=0;for(let s=0;s<t.length&&s<this.stickyScrollMaxItemCount;s++)n+=this.treeDelegate.getHeight(t[s]);return n}domFocus(){this._widget.domFocus()}focusedLast(){return this._widget.focusedLast()}updateOptions(e={}){if(!e.stickyScrollMaxItemCount)return;const t=this.validateStickySettings(e);this.stickyScrollMaxItemCount!==t.stickyScrollMaxItemCount&&(this.stickyScrollMaxItemCount=t.stickyScrollMaxItemCount,this.update())}validateStickySettings(e){let t=7;return typeof e.stickyScrollMaxItemCount=="number"&&(t=Math.max(e.stickyScrollMaxItemCount,1)),{stickyScrollMaxItemCount:t}}},ufe=class{constructor(e,t,i,n,s,r){this.view=t,this.tree=i,this.treeRenderers=n,this.treeDelegate=s,this.accessibilityProvider=r,this._previousElements=[],this._previousStateDisposables=new K,this._rootDomNode=ne(".monaco-tree-sticky-container.empty"),e.appendChild(this._rootDomNode);const a=ne(".monaco-tree-sticky-container-shadow");this._rootDomNode.appendChild(a),this.stickyScrollFocus=new gfe(this._rootDomNode,t),this.onDidChangeHasFocus=this.stickyScrollFocus.onDidChangeHasFocus,this.onContextMenu=this.stickyScrollFocus.onContextMenu}get height(){if(!this._previousState)return 0;const e=this._previousState.stickyNodes[this._previousState.count-1];return e.position+e.height}setState(e){const t=!!this._previousState&&this._previousState.count>0,i=!!e&&e.count>0;if(!t&&!i||t&&i&&this._previousState.equal(e))return;if(t!==i&&this.setVisible(i),!i){this._previousState=void 0,this._previousElements=[],this._previousStateDisposables.clear();return}const n=e.stickyNodes[e.count-1];if(this._previousState&&e.animationStateChanged(this._previousState))this._previousElements[this._previousState.count-1].style.top=`${n.position}px`;else{this._previousStateDisposables.clear();const s=Array(e.count);for(let r=e.count-1;r>=0;r--){const a=e.stickyNodes[r],{element:l,disposable:c}=this.createElement(a,r,e.count);s[r]=l,this._rootDomNode.appendChild(l),this._previousStateDisposables.add(c)}this.stickyScrollFocus.updateElements(s,e),this._previousElements=s}this._previousState=e,this._rootDomNode.style.height=`${n.position+n.height}px`}createElement(e,t,i){const n=e.startIndex,s=document.createElement("div");s.style.top=`${e.position}px`,this.tree.options.setRowHeight!==!1&&(s.style.height=`${e.height}px`),this.tree.options.setRowLineHeight!==!1&&(s.style.lineHeight=`${e.height}px`),s.classList.add("monaco-tree-sticky-row"),s.classList.add("monaco-list-row"),s.setAttribute("data-index",`${n}`),s.setAttribute("data-parity",n%2===0?"even":"odd"),s.setAttribute("id",this.view.getElementID(n));const r=this.setAccessibilityAttributes(s,e.node.element,t,i),a=this.treeDelegate.getTemplateId(e.node),l=this.treeRenderers.find(u=>u.templateId===a);if(!l)throw new Error(`No renderer found for template id ${a}`);let c=e.node;c===this.tree.getNode(this.tree.getNodeLocation(e.node))&&(c=new Proxy(e.node,{}));const d=l.renderTemplate(s);l.renderElement(c,e.startIndex,d,e.height);const h=ve(()=>{r.dispose(),l.disposeElement(c,e.startIndex,d,e.height),l.disposeTemplate(d),s.remove()});return{element:s,disposable:h}}setAccessibilityAttributes(e,t,i,n){if(!this.accessibilityProvider)return F.None;this.accessibilityProvider.getSetSize&&e.setAttribute("aria-setsize",String(this.accessibilityProvider.getSetSize(t,i,n))),this.accessibilityProvider.getPosInSet&&e.setAttribute("aria-posinset",String(this.accessibilityProvider.getPosInSet(t,i))),this.accessibilityProvider.getRole&&e.setAttribute("role",this.accessibilityProvider.getRole(t)??"treeitem");const s=this.accessibilityProvider.getAriaLabel(t),r=s&&typeof s!="string"?s:ur(s),a=Ae(c=>{const d=c.readObservable(r);d?e.setAttribute("aria-label",d):e.removeAttribute("aria-label")});typeof s=="string"||s&&e.setAttribute("aria-label",s.get());const l=this.accessibilityProvider.getAriaLevel&&this.accessibilityProvider.getAriaLevel(t);return typeof l=="number"&&e.setAttribute("aria-level",`${l}`),e.setAttribute("aria-selected",String(!1)),a}setVisible(e){this._rootDomNode.classList.toggle("empty",!e),e||this.stickyScrollFocus.updateElements([],void 0)}domFocus(){this.stickyScrollFocus.domFocus()}focusedLast(){return this.stickyScrollFocus.focusedLast()}dispose(){this.stickyScrollFocus.dispose(),this._previousStateDisposables.dispose(),this._rootDomNode.remove()}};class gfe extends F{get domHasFocus(){return this._domHasFocus}set domHasFocus(e){e!==this._domHasFocus&&(this._onDidChangeHasFocus.fire(e),this._domHasFocus=e)}constructor(e,t){super(),this.container=e,this.view=t,this.focusedIndex=-1,this.elements=[],this._onDidChangeHasFocus=new P,this.onDidChangeHasFocus=this._onDidChangeHasFocus.event,this._onContextMenu=new P,this.onContextMenu=this._onContextMenu.event,this._domHasFocus=!1,this._register(U(this.container,"focus",()=>this.onFocus())),this._register(U(this.container,"blur",()=>this.onBlur())),this._register(this.view.onDidFocus(()=>this.toggleStickyScrollFocused(!1))),this._register(this.view.onKeyDown(i=>this.onKeyDown(i))),this._register(this.view.onMouseDown(i=>this.onMouseDown(i))),this._register(this.view.onContextMenu(i=>this.handleContextMenu(i)))}handleContextMenu(e){const t=e.browserEvent.target;if(!Ew(t)&&!QC(t)){this.focusedLast()&&this.view.domFocus();return}if(!wc(e.browserEvent)){if(!this.state)throw new Error("Context menu should not be triggered when state is undefined");const r=this.state.stickyNodes.findIndex(a=>{var l;return a.node.element===((l=e.element)==null?void 0:l.element)});if(r===-1)throw new Error("Context menu should not be triggered when element is not in sticky scroll widget");this.container.focus(),this.setFocus(r);return}if(!this.state||this.focusedIndex<0)throw new Error("Context menu key should not be triggered when focus is not in sticky scroll widget");const n=this.state.stickyNodes[this.focusedIndex].node.element,s=this.elements[this.focusedIndex];this._onContextMenu.fire({element:n,anchor:s,browserEvent:e.browserEvent,isStickyScroll:!0})}onKeyDown(e){if(this.domHasFocus&&this.state){if(e.key==="ArrowUp")this.setFocusedElement(Math.max(0,this.focusedIndex-1)),e.preventDefault(),e.stopPropagation();else if(e.key==="ArrowDown"||e.key==="ArrowRight"){if(this.focusedIndex>=this.state.count-1){const t=this.state.stickyNodes[this.state.count-1].startIndex+1;this.view.domFocus(),this.view.setFocus([t]),this.scrollNodeUnderWidget(t,this.state)}else this.setFocusedElement(this.focusedIndex+1);e.preventDefault(),e.stopPropagation()}}}onMouseDown(e){const t=e.browserEvent.target;!Ew(t)&&!QC(t)||(e.browserEvent.preventDefault(),e.browserEvent.stopPropagation())}updateElements(e,t){if(t&&t.count===0)throw new Error("Sticky scroll state must be undefined when there are no sticky nodes");if(t&&t.count!==e.length)throw new Error("Sticky scroll focus received illigel state");const i=this.focusedIndex;if(this.removeFocus(),this.elements=e,this.state=t,t){const n=os(i,0,t.count-1);this.setFocus(n)}else this.domHasFocus&&this.view.domFocus();this.container.tabIndex=t?0:-1}setFocusedElement(e){const t=this.state;if(!t)throw new Error("Cannot set focus when state is undefined");if(this.setFocus(e),!(e<t.count-1)&&t.lastNodePartiallyVisible()){const i=t.stickyNodes[e];this.scrollNodeUnderWidget(i.endIndex+1,t)}}scrollNodeUnderWidget(e,t){const i=t.stickyNodes[t.count-1],n=t.count>1?t.stickyNodes[t.count-2]:void 0,s=this.view.getElementTop(e),r=n?n.position+n.height+i.height:i.height;this.view.scrollTop=s-r}domFocus(){if(!this.state)throw new Error("Cannot focus when state is undefined");this.container.focus()}focusedLast(){return this.state?this.view.getHTMLElement().classList.contains("sticky-scroll-focused"):!1}removeFocus(){this.focusedIndex!==-1&&(this.toggleElementFocus(this.elements[this.focusedIndex],!1),this.focusedIndex=-1)}setFocus(e){if(0>e)throw new Error("addFocus() can not remove focus");if(!this.state&&e>=0)throw new Error("Cannot set focus index when state is undefined");if(this.state&&e>=this.state.count)throw new Error("Cannot set focus index to an index that does not exist");const t=this.focusedIndex;t>=0&&this.toggleElementFocus(this.elements[t],!1),e>=0&&this.toggleElementFocus(this.elements[e],!0),this.focusedIndex=e}toggleElementFocus(e,t){this.toggleElementActiveFocus(e,t&&this.domHasFocus),this.toggleElementPassiveFocus(e,t)}toggleCurrentElementActiveFocus(e){this.focusedIndex!==-1&&this.toggleElementActiveFocus(this.elements[this.focusedIndex],e)}toggleElementActiveFocus(e,t){e.classList.toggle("focused",t)}toggleElementPassiveFocus(e,t){e.classList.toggle("passive-focused",t)}toggleStickyScrollFocused(e){this.view.getHTMLElement().classList.toggle("sticky-scroll-focused",e)}onFocus(){if(!this.state||this.elements.length===0)throw new Error("Cannot focus when state is undefined or elements are empty");this.domHasFocus=!0,this.toggleStickyScrollFocused(!0),this.toggleCurrentElementActiveFocus(!0),this.focusedIndex===-1&&this.setFocus(0)}onBlur(){this.domHasFocus=!1,this.toggleCurrentElementActiveFocus(!1)}dispose(){this.toggleStickyScrollFocused(!1),this._onDidChangeHasFocus.fire(!1),super.dispose()}}function i0(o){let e=dg.Unknown;return _N(o.browserEvent.target,"monaco-tl-twistie","monaco-tl-row")?e=dg.Twistie:_N(o.browserEvent.target,"monaco-tl-contents","monaco-tl-row")?e=dg.Element:_N(o.browserEvent.target,"monaco-tree-type-filter","monaco-list")&&(e=dg.Filter),{browserEvent:o.browserEvent,element:o.element?o.element.element:null,target:e}}function ffe(o){const e=Ew(o.browserEvent.target);return{element:o.element?o.element.element:null,browserEvent:o.browserEvent,anchor:o.anchor,isStickyScroll:e}}function iy(o,e){e(o),o.children.forEach(t=>iy(t,e))}class dT{get nodeSet(){return this._nodeSet||(this._nodeSet=this.createNodeSet()),this._nodeSet}constructor(e,t){this.getFirstViewElementWithTrait=e,this.identityProvider=t,this.nodes=[],this._onDidChange=new P,this.onDidChange=this._onDidChange.event}set(e,t){!(t!=null&&t.__forceEvent)&&ii(this.nodes,e)||this._set(e,!1,t)}_set(e,t,i){if(this.nodes=[...e],this.elements=void 0,this._nodeSet=void 0,!t){const n=this;this._onDidChange.fire({get elements(){return n.get()},browserEvent:i})}}get(){return this.elements||(this.elements=this.nodes.map(e=>e.element)),[...this.elements]}getNodes(){return this.nodes}has(e){return this.nodeSet.has(e)}onDidModelSplice({insertedNodes:e,deletedNodes:t}){if(!this.identityProvider){const l=this.createNodeSet(),c=d=>l.delete(d);t.forEach(d=>iy(d,c)),this.set([...l.values()]);return}const i=new Set,n=l=>i.add(this.identityProvider.getId(l.element).toString());t.forEach(l=>iy(l,n));const s=new Map,r=l=>s.set(this.identityProvider.getId(l.element).toString(),l);e.forEach(l=>iy(l,r));const a=[];for(const l of this.nodes){const c=this.identityProvider.getId(l.element).toString();if(!i.has(c))a.push(l);else{const h=s.get(c);h&&h.visible&&a.push(h)}}if(this.nodes.length>0&&a.length===0){const l=this.getFirstViewElementWithTrait();l&&a.push(l)}this._set(a,!0)}createNodeSet(){const e=new Set;for(const t of this.nodes)e.add(t);return e}}class mfe extends Rj{constructor(e,t,i){super(e),this.tree=t,this.stickyScrollProvider=i}onViewPointer(e){if(Ij(e.browserEvent.target)||uh(e.browserEvent.target)||DC(e.browserEvent.target)||e.browserEvent.isHandledByList)return;const t=e.element;if(!t)return super.onViewPointer(e);if(this.isSelectionRangeChangeEvent(e)||this.isSelectionSingleChangeEvent(e))return super.onViewPointer(e);const i=e.browserEvent.target,n=i.classList.contains("monaco-tl-twistie")||i.classList.contains("monaco-icon-label")&&i.classList.contains("folder-icon")&&e.browserEvent.offsetX<16,s=QC(e.browserEvent.target);let r=!1;if(s?r=!0:typeof this.tree.expandOnlyOnTwistieClick=="function"?r=this.tree.expandOnlyOnTwistieClick(t.element):r=!!this.tree.expandOnlyOnTwistieClick,s)this.handleStickyScrollMouseEvent(e,t);else{if(r&&!n&&e.browserEvent.detail!==2)return super.onViewPointer(e);if(!this.tree.expandOnDoubleClick&&e.browserEvent.detail===2)return super.onViewPointer(e)}if(t.collapsible&&(!s||n)){const a=this.tree.getNodeLocation(t),l=e.browserEvent.altKey;if(this.tree.setFocus([a]),this.tree.toggleCollapsed(a,l),n){e.browserEvent.isHandledByList=!0;return}}s||super.onViewPointer(e)}handleStickyScrollMouseEvent(e,t){if(vde(e.browserEvent.target)||wde(e.browserEvent.target))return;const i=this.stickyScrollProvider();if(!i)throw new Error("Sticky scroll controller not found");const n=this.list.indexOf(t),s=this.list.getElementTop(n),r=i.nodePositionTopBelowWidget(t);this.tree.scrollTop=s-r,this.list.domFocus(),this.list.setFocus([n]),this.list.setSelection([n])}onDoubleClick(e){e.browserEvent.target.classList.contains("monaco-tl-twistie")||!this.tree.expandOnDoubleClick||e.browserEvent.isHandledByList||super.onDoubleClick(e)}onMouseDown(e){const t=e.browserEvent.target;if(!Ew(t)&&!QC(t)){super.onMouseDown(e);return}}onContextMenu(e){const t=e.browserEvent.target;if(!Ew(t)&&!QC(t)){super.onContextMenu(e);return}}}class pfe extends Xo{constructor(e,t,i,n,s,r,a,l){super(e,t,i,n,l),this.focusTrait=s,this.selectionTrait=r,this.anchorTrait=a}createMouseController(e){return new mfe(this,e.tree,e.stickyScrollProvider)}splice(e,t,i=[]){if(super.splice(e,t,i),i.length===0)return;const n=[],s=[];let r;i.forEach((a,l)=>{this.focusTrait.has(a)&&n.push(e+l),this.selectionTrait.has(a)&&s.push(e+l),this.anchorTrait.has(a)&&(r=e+l)}),n.length>0&&super.setFocus(Xc([...super.getFocus(),...n])),s.length>0&&super.setSelection(Xc([...super.getSelection(),...s])),typeof r=="number"&&super.setAnchor(r)}setFocus(e,t,i=!1){super.setFocus(e,t),i||this.focusTrait.set(e.map(n=>this.element(n)),t)}setSelection(e,t,i=!1){super.setSelection(e,t),i||this.selectionTrait.set(e.map(n=>this.element(n)),t)}setAnchor(e,t=!1){super.setAnchor(e),t||(typeof e>"u"?this.anchorTrait.set([]):this.anchorTrait.set([this.element(e)]))}}class EK{get onDidScroll(){return this.view.onDidScroll}get onDidChangeFocus(){return this.eventBufferer.wrapEvent(this.focus.onDidChange)}get onDidChangeSelection(){return this.eventBufferer.wrapEvent(this.selection.onDidChange)}get onMouseDblClick(){return oe.filter(oe.map(this.view.onMouseDblClick,i0),e=>e.target!==dg.Filter)}get onMouseOver(){return oe.map(this.view.onMouseOver,i0)}get onMouseOut(){return oe.map(this.view.onMouseOut,i0)}get onContextMenu(){var e;return oe.any(oe.filter(oe.map(this.view.onContextMenu,ffe),t=>!t.isStickyScroll),((e=this.stickyScrollController)==null?void 0:e.onContextMenu)??oe.None)}get onPointer(){return oe.map(this.view.onPointer,i0)}get onKeyDown(){return this.view.onKeyDown}get onDidFocus(){return this.view.onDidFocus}get onDidChangeModel(){return oe.signal(this.model.onDidSplice)}get onDidChangeCollapseState(){return this.model.onDidChangeCollapseState}get findMode(){var e;return((e=this.findController)==null?void 0:e.mode)??Oc.Highlight}set findMode(e){this.findController&&(this.findController.mode=e)}get findMatchType(){var e;return((e=this.findController)==null?void 0:e.matchType)??Ff.Fuzzy}set findMatchType(e){this.findController&&(this.findController.matchType=e)}get expandOnDoubleClick(){return typeof this._options.expandOnDoubleClick>"u"?!0:this._options.expandOnDoubleClick}get expandOnlyOnTwistieClick(){return typeof this._options.expandOnlyOnTwistieClick>"u"?!0:this._options.expandOnlyOnTwistieClick}get onDidDispose(){return this.view.onDidDispose}constructor(e,t,i,n,s={}){this._user=e,this._options=s,this.eventBufferer=new R1,this.onDidChangeFindOpenState=oe.None,this.onDidChangeStickyScrollFocused=oe.None,this.disposables=new K,this._onWillRefilter=new P,this.onWillRefilter=this._onWillRefilter.event,this._onDidUpdateOptions=new P,this.treeDelegate=new w3(i);const r=new KB,a=new KB,l=this.disposables.add(new rfe(a.event)),c=new m5;this.renderers=n.map(f=>new SA(f,()=>this.model,r.event,l,c,s));for(const f of this.renderers)this.disposables.add(f);let d;s.keyboardNavigationLabelProvider&&(d=new afe(this,s.keyboardNavigationLabelProvider,s.filter),s={...s,filter:d},this.disposables.add(d)),this.focus=new dT(()=>this.view.getFocusedElements()[0],s.identityProvider),this.selection=new dT(()=>this.view.getSelectedElements()[0],s.identityProvider),this.anchor=new dT(()=>this.view.getAnchorElement(),s.identityProvider),this.view=new pfe(e,t,this.treeDelegate,this.renderers,this.focus,this.selection,this.anchor,{...ofe(()=>this.model,s),tree:this,stickyScrollProvider:()=>this.stickyScrollController}),this.model=this.createModel(e,this.view,s),r.input=this.model.onDidChangeCollapseState;const h=oe.forEach(this.model.onDidSplice,f=>{this.eventBufferer.bufferEvents(()=>{this.focus.onDidModelSplice(f),this.selection.onDidModelSplice(f)})},this.disposables);h(()=>null,null,this.disposables);const u=this.disposables.add(new P),g=this.disposables.add(new _r(0));if(this.disposables.add(oe.any(h,this.focus.onDidChange,this.selection.onDidChange)(()=>{g.trigger(()=>{const f=new Set;for(const p of this.focus.getNodes())f.add(p);for(const p of this.selection.getNodes())f.add(p);u.fire([...f.values()])})})),a.input=u.event,s.keyboardSupport!==!1){const f=oe.chain(this.view.onKeyDown,p=>p.filter(_=>!uh(_.target)).map(_=>new Pt(_)));oe.chain(f,p=>p.filter(_=>_.keyCode===15))(this.onLeftArrow,this,this.disposables),oe.chain(f,p=>p.filter(_=>_.keyCode===17))(this.onRightArrow,this,this.disposables),oe.chain(f,p=>p.filter(_=>_.keyCode===10))(this.onSpace,this,this.disposables)}if((s.findWidgetEnabled??!0)&&s.keyboardNavigationLabelProvider&&s.contextViewProvider){const f=this.options.findWidgetStyles?{styles:this.options.findWidgetStyles}:void 0;this.findController=new lfe(this,this.model,this.view,d,s.contextViewProvider,f),this.focusNavigationFilter=p=>this.findController.shouldAllowFocus(p),this.onDidChangeFindOpenState=this.findController.onDidChangeOpenState,this.disposables.add(this.findController),this.onDidChangeFindMode=this.findController.onDidChangeMode,this.onDidChangeFindMatchType=this.findController.onDidChangeMatchType}else this.onDidChangeFindMode=oe.None,this.onDidChangeFindMatchType=oe.None;s.enableStickyScroll&&(this.stickyScrollController=new z8(this,this.model,this.view,this.renderers,this.treeDelegate,s),this.onDidChangeStickyScrollFocused=this.stickyScrollController.onDidChangeHasFocus),this.styleElement=$o(this.view.getHTMLElement()),this.getHTMLElement().classList.toggle("always",this._options.renderIndentGuides===U_.Always)}updateOptions(e={}){var t;this._options={...this._options,...e};for(const i of this.renderers)i.updateOptions(e);this.view.updateOptions(this._options),(t=this.findController)==null||t.updateOptions(e),this.updateStickyScroll(e),this._onDidUpdateOptions.fire(this._options),this.getHTMLElement().classList.toggle("always",this._options.renderIndentGuides===U_.Always)}get options(){return this._options}updateStickyScroll(e){var t;!this.stickyScrollController&&this._options.enableStickyScroll?(this.stickyScrollController=new z8(this,this.model,this.view,this.renderers,this.treeDelegate,this._options),this.onDidChangeStickyScrollFocused=this.stickyScrollController.onDidChangeHasFocus):this.stickyScrollController&&!this._options.enableStickyScroll&&(this.onDidChangeStickyScrollFocused=oe.None,this.stickyScrollController.dispose(),this.stickyScrollController=void 0),(t=this.stickyScrollController)==null||t.updateOptions(e)}getHTMLElement(){return this.view.getHTMLElement()}get scrollTop(){return this.view.scrollTop}set scrollTop(e){this.view.scrollTop=e}get scrollHeight(){return this.view.scrollHeight}get renderHeight(){return this.view.renderHeight}get ariaLabel(){return this.view.ariaLabel}set ariaLabel(e){this.view.ariaLabel=e}domFocus(){var e;(e=this.stickyScrollController)!=null&&e.focusedLast()?this.stickyScrollController.domFocus():this.view.domFocus()}layout(e,t){var i;this.view.layout(e,t),Rh(t)&&((i=this.findController)==null||i.layout(t))}style(e){const t=`.${this.view.domId}`,i=[];e.treeIndentGuidesStroke&&(i.push(`.monaco-list${t}:hover .monaco-tl-indent > .indent-guide, .monaco-list${t}.always .monaco-tl-indent > .indent-guide  { border-color: ${e.treeInactiveIndentGuidesStroke}; }`),i.push(`.monaco-list${t} .monaco-tl-indent > .indent-guide.active { border-color: ${e.treeIndentGuidesStroke}; }`));const n=e.treeStickyScrollBackground??e.listBackground;n&&(i.push(`.monaco-list${t} .monaco-scrollable-element .monaco-tree-sticky-container { background-color: ${n}; }`),i.push(`.monaco-list${t} .monaco-scrollable-element .monaco-tree-sticky-container .monaco-tree-sticky-row { background-color: ${n}; }`)),e.treeStickyScrollBorder&&i.push(`.monaco-list${t} .monaco-scrollable-element .monaco-tree-sticky-container { border-bottom: 1px solid ${e.treeStickyScrollBorder}; }`),e.treeStickyScrollShadow&&i.push(`.monaco-list${t} .monaco-scrollable-element .monaco-tree-sticky-container .monaco-tree-sticky-container-shadow { box-shadow: ${e.treeStickyScrollShadow} 0 6px 6px -6px inset; height: 3px; }`),e.listFocusForeground&&(i.push(`.monaco-list${t}.sticky-scroll-focused .monaco-scrollable-element .monaco-tree-sticky-container:focus .monaco-list-row.focused { color: ${e.listFocusForeground}; }`),i.push(`.monaco-list${t}:not(.sticky-scroll-focused) .monaco-scrollable-element .monaco-tree-sticky-container .monaco-list-row.focused { color: inherit; }`));const s=jc(e.listFocusAndSelectionOutline,jc(e.listSelectionOutline,e.listFocusOutline??""));s&&(i.push(`.monaco-list${t}.sticky-scroll-focused .monaco-scrollable-element .monaco-tree-sticky-container:focus .monaco-list-row.focused.selected { outline: 1px solid ${s}; outline-offset: -1px;}`),i.push(`.monaco-list${t}:not(.sticky-scroll-focused) .monaco-scrollable-element .monaco-tree-sticky-container .monaco-list-row.focused.selected { outline: inherit;}`)),e.listFocusOutline&&(i.push(`.monaco-list${t}.sticky-scroll-focused .monaco-scrollable-element .monaco-tree-sticky-container:focus .monaco-list-row.focused { outline: 1px solid ${e.listFocusOutline}; outline-offset: -1px; }`),i.push(`.monaco-list${t}:not(.sticky-scroll-focused) .monaco-scrollable-element .monaco-tree-sticky-container .monaco-list-row.focused { outline: inherit; }`),i.push(`.monaco-workbench.context-menu-visible .monaco-list${t}.last-focused.sticky-scroll-focused .monaco-scrollable-element .monaco-tree-sticky-container .monaco-list-row.passive-focused { outline: 1px solid ${e.listFocusOutline}; outline-offset: -1px; }`),i.push(`.monaco-workbench.context-menu-visible .monaco-list${t}.last-focused.sticky-scroll-focused .monaco-list-rows .monaco-list-row.focused { outline: inherit; }`),i.push(`.monaco-workbench.context-menu-visible .monaco-list${t}.last-focused:not(.sticky-scroll-focused) .monaco-tree-sticky-container .monaco-list-rows .monaco-list-row.focused { outline: inherit; }`)),this.styleElement.textContent=i.join(`
`),this.view.style(e)}getParentElement(e){const t=this.model.getParentNodeLocation(e);return this.model.getNode(t).element}getFirstElementChild(e){return this.model.getFirstElementChild(e)}getNode(e){return this.model.getNode(e)}getNodeLocation(e){return this.model.getNodeLocation(e)}collapse(e,t=!1){return this.model.setCollapsed(e,!0,t)}expand(e,t=!1){return this.model.setCollapsed(e,!1,t)}toggleCollapsed(e,t=!1){return this.model.setCollapsed(e,void 0,t)}isCollapsible(e){return this.model.isCollapsible(e)}setCollapsible(e,t){return this.model.setCollapsible(e,t)}isCollapsed(e){return this.model.isCollapsed(e)}refilter(){this._onWillRefilter.fire(void 0),this.model.refilter()}setSelection(e,t){this.eventBufferer.bufferEvents(()=>{const i=e.map(s=>this.model.getNode(s));this.selection.set(i,t);const n=e.map(s=>this.model.getListIndex(s)).filter(s=>s>-1);this.view.setSelection(n,t,!0)})}getSelection(){return this.selection.get()}setFocus(e,t){this.eventBufferer.bufferEvents(()=>{const i=e.map(s=>this.model.getNode(s));this.focus.set(i,t);const n=e.map(s=>this.model.getListIndex(s)).filter(s=>s>-1);this.view.setFocus(n,t,!0)})}focusNext(e=1,t=!1,i,n=wc(i)&&i.altKey?void 0:this.focusNavigationFilter){this.view.focusNext(e,t,i,n)}focusPrevious(e=1,t=!1,i,n=wc(i)&&i.altKey?void 0:this.focusNavigationFilter){this.view.focusPrevious(e,t,i,n)}focusNextPage(e,t=wc(e)&&e.altKey?void 0:this.focusNavigationFilter){return this.view.focusNextPage(e,t)}focusPreviousPage(e,t=wc(e)&&e.altKey?void 0:this.focusNavigationFilter){return this.view.focusPreviousPage(e,t,()=>{var i;return((i=this.stickyScrollController)==null?void 0:i.height)??0})}focusLast(e,t=wc(e)&&e.altKey?void 0:this.focusNavigationFilter){this.view.focusLast(e,t)}focusFirst(e,t=wc(e)&&e.altKey?void 0:this.focusNavigationFilter){this.view.focusFirst(e,t)}getFocus(){return this.focus.get()}reveal(e,t){this.model.expandTo(e);const i=this.model.getListIndex(e);if(i!==-1)if(!this.stickyScrollController)this.view.reveal(i,t);else{const n=this.stickyScrollController.nodePositionTopBelowWidget(this.getNode(e));this.view.reveal(i,t,n)}}onLeftArrow(e){e.preventDefault(),e.stopPropagation();const t=this.view.getFocusedElements();if(t.length===0)return;const i=t[0],n=this.model.getNodeLocation(i);if(!this.model.setCollapsed(n,!0)){const r=this.model.getParentNodeLocation(n);if(!r)return;const a=this.model.getListIndex(r);this.view.reveal(a),this.view.setFocus([a])}}onRightArrow(e){e.preventDefault(),e.stopPropagation();const t=this.view.getFocusedElements();if(t.length===0)return;const i=t[0],n=this.model.getNodeLocation(i);if(!this.model.setCollapsed(n,!1)){if(!i.children.some(l=>l.visible))return;const[r]=this.view.getFocus(),a=r+1;this.view.reveal(a),this.view.setFocus([a])}}onSpace(e){e.preventDefault(),e.stopPropagation();const t=this.view.getFocusedElements();if(t.length===0)return;const i=t[0],n=this.model.getNodeLocation(i),s=e.browserEvent.altKey;this.model.setCollapsed(n,void 0,s)}dispose(){var e;Tt(this.disposables),(e=this.stickyScrollController)==null||e.dispose(),this.view.dispose()}}class S3{constructor(e,t,i={}){this.user=e,this.rootRef=null,this.nodes=new Map,this.nodesByIdentity=new Map,this.model=new ife(e,t,null,i),this.onDidSplice=this.model.onDidSplice,this.onDidChangeCollapseState=this.model.onDidChangeCollapseState,this.onDidChangeRenderNodeCount=this.model.onDidChangeRenderNodeCount,i.sorter&&(this.sorter={compare(n,s){return i.sorter.compare(n.element,s.element)}}),this.identityProvider=i.identityProvider}setChildren(e,t=ot.empty(),i={}){const n=this.getElementLocation(e);this._setChildren(n,this.preserveCollapseState(t),i)}_setChildren(e,t=ot.empty(),i){const n=new Set,s=new Set,r=l=>{var d;if(l.element===null)return;const c=l;if(n.add(c.element),this.nodes.set(c.element,c),this.identityProvider){const h=this.identityProvider.getId(c.element).toString();s.add(h),this.nodesByIdentity.set(h,c)}(d=i.onDidCreateNode)==null||d.call(i,c)},a=l=>{var d;if(l.element===null)return;const c=l;if(n.has(c.element)||this.nodes.delete(c.element),this.identityProvider){const h=this.identityProvider.getId(c.element).toString();s.has(h)||this.nodesByIdentity.delete(h)}(d=i.onDidDeleteNode)==null||d.call(i,c)};this.model.splice([...e,0],Number.MAX_VALUE,t,{...i,onDidCreateNode:r,onDidDeleteNode:a})}preserveCollapseState(e=ot.empty()){return this.sorter&&(e=[...e].sort(this.sorter.compare.bind(this.sorter))),ot.map(e,t=>{let i=this.nodes.get(t.element);if(!i&&this.identityProvider){const r=this.identityProvider.getId(t.element).toString();i=this.nodesByIdentity.get(r)}if(!i){let r;return typeof t.collapsed>"u"?r=void 0:t.collapsed===Do.Collapsed||t.collapsed===Do.PreserveOrCollapsed?r=!0:t.collapsed===Do.Expanded||t.collapsed===Do.PreserveOrExpanded?r=!1:r=!!t.collapsed,{...t,children:this.preserveCollapseState(t.children),collapsed:r}}const n=typeof t.collapsible=="boolean"?t.collapsible:i.collapsible;let s;return typeof t.collapsed>"u"||t.collapsed===Do.PreserveOrCollapsed||t.collapsed===Do.PreserveOrExpanded?s=i.collapsed:t.collapsed===Do.Collapsed?s=!0:t.collapsed===Do.Expanded?s=!1:s=!!t.collapsed,{...t,collapsible:n,collapsed:s,children:this.preserveCollapseState(t.children)}})}rerender(e){const t=this.getElementLocation(e);this.model.rerender(t)}getFirstElementChild(e=null){const t=this.getElementLocation(e);return this.model.getFirstElementChild(t)}has(e){return this.nodes.has(e)}getListIndex(e){const t=this.getElementLocation(e);return this.model.getListIndex(t)}getListRenderCount(e){const t=this.getElementLocation(e);return this.model.getListRenderCount(t)}isCollapsible(e){const t=this.getElementLocation(e);return this.model.isCollapsible(t)}setCollapsible(e,t){const i=this.getElementLocation(e);return this.model.setCollapsible(i,t)}isCollapsed(e){const t=this.getElementLocation(e);return this.model.isCollapsed(t)}setCollapsed(e,t,i){const n=this.getElementLocation(e);return this.model.setCollapsed(n,t,i)}expandTo(e){const t=this.getElementLocation(e);this.model.expandTo(t)}refilter(){this.model.refilter()}getNode(e=null){if(e===null)return this.model.getNode(this.model.rootRef);const t=this.nodes.get(e);if(!t)throw new Ao(this.user,`Tree element not found: ${e}`);return t}getNodeLocation(e){return e.element}getParentNodeLocation(e){if(e===null)throw new Ao(this.user,"Invalid getParentNodeLocation call");const t=this.nodes.get(e);if(!t)throw new Ao(this.user,`Tree element not found: ${e}`);const i=this.model.getNodeLocation(t),n=this.model.getParentNodeLocation(i);return this.model.getNode(n).element}getElementLocation(e){if(e===null)return[];const t=this.nodes.get(e);if(!t)throw new Ao(this.user,`Tree element not found: ${e}`);return this.model.getNodeLocation(t)}}function ny(o){const e=[o.element],t=o.incompressible||!1;return{element:{elements:e,incompressible:t},children:ot.map(ot.from(o.children),ny),collapsible:o.collapsible,collapsed:o.collapsed}}function sy(o){const e=[o.element],t=o.incompressible||!1;let i,n;for(;[n,i]=ot.consume(ot.from(o.children),2),!(n.length!==1||n[0].incompressible);)o=n[0],e.push(o.element);return{element:{elements:e,incompressible:t},children:ot.map(ot.concat(n,i),sy),collapsible:o.collapsible,collapsed:o.collapsed}}function yA(o,e=0){let t;return e<o.element.elements.length-1?t=[yA(o,e+1)]:t=ot.map(ot.from(o.children),i=>yA(i,0)),e===0&&o.element.incompressible?{element:o.element.elements[e],children:t,incompressible:!0,collapsible:o.collapsible,collapsed:o.collapsed}:{element:o.element.elements[e],children:t,collapsible:o.collapsible,collapsed:o.collapsed}}function U8(o){return yA(o,0)}function NK(o,e,t){return o.element===e?{...o,children:t}:{...o,children:ot.map(ot.from(o.children),i=>NK(i,e,t))}}const _fe=o=>({getId(e){return e.elements.map(t=>o.getId(t).toString()).join("\0")}});class bfe{get onDidSplice(){return this.model.onDidSplice}get onDidChangeCollapseState(){return this.model.onDidChangeCollapseState}get onDidChangeRenderNodeCount(){return this.model.onDidChangeRenderNodeCount}constructor(e,t,i={}){this.user=e,this.rootRef=null,this.nodes=new Map,this.model=new S3(e,t,i),this.enabled=typeof i.compressionEnabled>"u"?!0:i.compressionEnabled,this.identityProvider=i.identityProvider}setChildren(e,t=ot.empty(),i){const n=i.diffIdentityProvider&&_fe(i.diffIdentityProvider);if(e===null){const f=ot.map(t,this.enabled?sy:ny);this._setChildren(null,f,{diffIdentityProvider:n,diffDepth:1/0});return}const s=this.nodes.get(e);if(!s)throw new Ao(this.user,"Unknown compressed tree node");const r=this.model.getNode(s),a=this.model.getParentNodeLocation(s),l=this.model.getNode(a),c=U8(r),d=NK(c,e,t),h=(this.enabled?sy:ny)(d),u=i.diffIdentityProvider?(f,p)=>i.diffIdentityProvider.getId(f)===i.diffIdentityProvider.getId(p):void 0;if(ii(h.element.elements,r.element.elements,u)){this._setChildren(s,h.children||ot.empty(),{diffIdentityProvider:n,diffDepth:1});return}const g=l.children.map(f=>f===r?h:f);this._setChildren(l.element,g,{diffIdentityProvider:n,diffDepth:r.depth-l.depth})}isCompressionEnabled(){return this.enabled}setCompressionEnabled(e){if(e===this.enabled)return;this.enabled=e;const i=this.model.getNode().children,n=ot.map(i,U8),s=ot.map(n,e?sy:ny);this._setChildren(null,s,{diffIdentityProvider:this.identityProvider,diffDepth:1/0})}_setChildren(e,t,i){const n=new Set,s=a=>{for(const l of a.element.elements)n.add(l),this.nodes.set(l,a.element)},r=a=>{for(const l of a.element.elements)n.has(l)||this.nodes.delete(l)};this.model.setChildren(e,t,{...i,onDidCreateNode:s,onDidDeleteNode:r})}has(e){return this.nodes.has(e)}getListIndex(e){const t=this.getCompressedNode(e);return this.model.getListIndex(t)}getListRenderCount(e){const t=this.getCompressedNode(e);return this.model.getListRenderCount(t)}getNode(e){if(typeof e>"u")return this.model.getNode();const t=this.getCompressedNode(e);return this.model.getNode(t)}getNodeLocation(e){const t=this.model.getNodeLocation(e);return t===null?null:t.elements[t.elements.length-1]}getParentNodeLocation(e){const t=this.getCompressedNode(e),i=this.model.getParentNodeLocation(t);return i===null?null:i.elements[i.elements.length-1]}getFirstElementChild(e){const t=this.getCompressedNode(e);return this.model.getFirstElementChild(t)}isCollapsible(e){const t=this.getCompressedNode(e);return this.model.isCollapsible(t)}setCollapsible(e,t){const i=this.getCompressedNode(e);return this.model.setCollapsible(i,t)}isCollapsed(e){const t=this.getCompressedNode(e);return this.model.isCollapsed(t)}setCollapsed(e,t,i){const n=this.getCompressedNode(e);return this.model.setCollapsed(n,t,i)}expandTo(e){const t=this.getCompressedNode(e);this.model.expandTo(t)}rerender(e){const t=this.getCompressedNode(e);this.model.rerender(t)}refilter(){this.model.refilter()}getCompressedNode(e){if(e===null)return null;const t=this.nodes.get(e);if(!t)throw new Ao(this.user,`Tree element not found: ${e}`);return t}}const Cfe=o=>o[o.length-1];class y3{get element(){return this.node.element===null?null:this.unwrapper(this.node.element)}get children(){return this.node.children.map(e=>new y3(this.unwrapper,e))}get depth(){return this.node.depth}get visibleChildrenCount(){return this.node.visibleChildrenCount}get visibleChildIndex(){return this.node.visibleChildIndex}get collapsible(){return this.node.collapsible}get collapsed(){return this.node.collapsed}get visible(){return this.node.visible}get filterData(){return this.node.filterData}constructor(e,t){this.unwrapper=e,this.node=t}}function vfe(o,e){return{splice(t,i,n){e.splice(t,i,n.map(s=>o.map(s)))},updateElementHeight(t,i){e.updateElementHeight(t,i)}}}function wfe(o,e){return{...e,identityProvider:e.identityProvider&&{getId(t){return e.identityProvider.getId(o(t))}},sorter:e.sorter&&{compare(t,i){return e.sorter.compare(t.elements[0],i.elements[0])}},filter:e.filter&&{filter(t,i){return e.filter.filter(o(t),i)}}}}class Sfe{get onDidSplice(){return oe.map(this.model.onDidSplice,({insertedNodes:e,deletedNodes:t})=>({insertedNodes:e.map(i=>this.nodeMapper.map(i)),deletedNodes:t.map(i=>this.nodeMapper.map(i))}))}get onDidChangeCollapseState(){return oe.map(this.model.onDidChangeCollapseState,({node:e,deep:t})=>({node:this.nodeMapper.map(e),deep:t}))}get onDidChangeRenderNodeCount(){return oe.map(this.model.onDidChangeRenderNodeCount,e=>this.nodeMapper.map(e))}constructor(e,t,i={}){this.rootRef=null,this.elementMapper=i.elementMapper||Cfe;const n=s=>this.elementMapper(s.elements);this.nodeMapper=new C3(s=>new y3(n,s)),this.model=new bfe(e,vfe(this.nodeMapper,t),wfe(n,i))}setChildren(e,t=ot.empty(),i={}){this.model.setChildren(e,t,i)}isCompressionEnabled(){return this.model.isCompressionEnabled()}setCompressionEnabled(e){this.model.setCompressionEnabled(e)}has(e){return this.model.has(e)}getListIndex(e){return this.model.getListIndex(e)}getListRenderCount(e){return this.model.getListRenderCount(e)}getNode(e){return this.nodeMapper.map(this.model.getNode(e))}getNodeLocation(e){return e.element}getParentNodeLocation(e){return this.model.getParentNodeLocation(e)}getFirstElementChild(e){const t=this.model.getFirstElementChild(e);return t===null||typeof t>"u"?t:this.elementMapper(t.elements)}isCollapsible(e){return this.model.isCollapsible(e)}setCollapsible(e,t){return this.model.setCollapsible(e,t)}isCollapsed(e){return this.model.isCollapsed(e)}setCollapsed(e,t,i){return this.model.setCollapsed(e,t,i)}expandTo(e){return this.model.expandTo(e)}rerender(e){return this.model.rerender(e)}refilter(){return this.model.refilter()}getCompressedTreeNode(e=null){return this.model.getNode(e)}}var yfe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s};class L3 extends EK{get onDidChangeCollapseState(){return this.model.onDidChangeCollapseState}constructor(e,t,i,n,s={}){super(e,t,i,n,s),this.user=e}setChildren(e,t=ot.empty(),i){this.model.setChildren(e,t,i)}rerender(e){if(e===void 0){this.view.rerender();return}this.model.rerender(e)}hasElement(e){return this.model.has(e)}createModel(e,t,i){return new S3(e,t,i)}}class TK{get compressedTreeNodeProvider(){return this._compressedTreeNodeProvider()}constructor(e,t,i){this._compressedTreeNodeProvider=e,this.stickyScrollDelegate=t,this.renderer=i,this.templateId=i.templateId,i.onDidChangeTwistieState&&(this.onDidChangeTwistieState=i.onDidChangeTwistieState)}renderTemplate(e){return{compressedTreeNode:void 0,data:this.renderer.renderTemplate(e)}}renderElement(e,t,i,n){let s=this.stickyScrollDelegate.getCompressedNode(e);s||(s=this.compressedTreeNodeProvider.getCompressedTreeNode(e.element)),s.element.elements.length===1?(i.compressedTreeNode=void 0,this.renderer.renderElement(e,t,i.data,n)):(i.compressedTreeNode=s,this.renderer.renderCompressedElements(s,t,i.data,n))}disposeElement(e,t,i,n){var s,r,a,l;i.compressedTreeNode?(r=(s=this.renderer).disposeCompressedElements)==null||r.call(s,i.compressedTreeNode,t,i.data,n):(l=(a=this.renderer).disposeElement)==null||l.call(a,e,t,i.data,n)}disposeTemplate(e){this.renderer.disposeTemplate(e.data)}renderTwistie(e,t){return this.renderer.renderTwistie?this.renderer.renderTwistie(e,t):!1}}yfe([Ni],TK.prototype,"compressedTreeNodeProvider",null);class Lfe{constructor(e){this.modelProvider=e,this.compressedStickyNodes=new Map}getCompressedNode(e){return this.compressedStickyNodes.get(e)}constrainStickyScrollNodes(e,t,i){if(this.compressedStickyNodes.clear(),e.length===0)return[];for(let n=0;n<e.length;n++){const s=e[n],r=s.position+s.height;if(n+1<e.length&&r+e[n+1].height>i||n>=t-1&&t<e.length){const l=e.slice(0,n),c=e.slice(n),d=this.compressStickyNodes(c);return[...l,d]}}return e}compressStickyNodes(e){if(e.length===0)throw new Error("Can't compress empty sticky nodes");const t=this.modelProvider();if(!t.isCompressionEnabled())return e[0];const i=[];for(let c=0;c<e.length;c++){const d=e[c],h=t.getCompressedTreeNode(d.node.element);if(h.element){if(c!==0&&h.element.incompressible)break;i.push(...h.element.elements)}}if(i.length<2)return e[0];const n=e[e.length-1],s={elements:i,incompressible:!1},r={...n.node,children:[],element:s},a=new Proxy(e[0].node,{}),l={node:a,startIndex:e[0].startIndex,endIndex:n.endIndex,position:e[0].position,height:e[0].height};return this.compressedStickyNodes.set(a,r),l}}function xfe(o,e){return e&&{...e,keyboardNavigationLabelProvider:e.keyboardNavigationLabelProvider&&{getKeyboardNavigationLabel(t){let i;try{i=o().getCompressedTreeNode(t)}catch{return e.keyboardNavigationLabelProvider.getKeyboardNavigationLabel(t)}return i.element.elements.length===1?e.keyboardNavigationLabelProvider.getKeyboardNavigationLabel(t):e.keyboardNavigationLabelProvider.getCompressedNodeKeyboardNavigationLabel(i.element.elements)}}}}class RK extends L3{constructor(e,t,i,n,s={}){const r=()=>this,a=new Lfe(()=>this.model),l=n.map(c=>new TK(r,a,c));super(e,t,i,l,{...xfe(r,s),stickyScrollDelegate:a})}setChildren(e,t=ot.empty(),i){this.model.setChildren(e,t,i)}createModel(e,t,i){return new Sfe(e,t,i)}updateOptions(e={}){super.updateOptions(e),typeof e.compressionEnabled<"u"&&this.model.setCompressionEnabled(e.compressionEnabled)}getCompressedTreeNode(e=null){return this.model.getCompressedTreeNode(e)}}function hT(o){return{...o,children:[],refreshPromise:void 0,stale:!0,slow:!1,forceExpanded:!1}}function LA(o,e){return e.parent?e.parent===o?!0:LA(o,e.parent):!1}function kfe(o,e){return o===e||LA(o,e)||LA(e,o)}class x3{get element(){return this.node.element.element}get children(){return this.node.children.map(e=>new x3(e))}get depth(){return this.node.depth}get visibleChildrenCount(){return this.node.visibleChildrenCount}get visibleChildIndex(){return this.node.visibleChildIndex}get collapsible(){return this.node.collapsible}get collapsed(){return this.node.collapsed}get visible(){return this.node.visible}get filterData(){return this.node.filterData}constructor(e){this.node=e}}class Dfe{constructor(e,t,i){this.renderer=e,this.nodeMapper=t,this.onDidChangeTwistieState=i,this.renderedNodes=new Map,this.templateId=e.templateId}renderTemplate(e){return{templateData:this.renderer.renderTemplate(e)}}renderElement(e,t,i,n){this.renderer.renderElement(this.nodeMapper.map(e),t,i.templateData,n)}renderTwistie(e,t){return e.slow?(t.classList.add(...Ie.asClassNameArray(te.treeItemLoading)),!0):(t.classList.remove(...Ie.asClassNameArray(te.treeItemLoading)),!1)}disposeElement(e,t,i,n){var s,r;(r=(s=this.renderer).disposeElement)==null||r.call(s,this.nodeMapper.map(e),t,i.templateData,n)}disposeTemplate(e){this.renderer.disposeTemplate(e.templateData)}dispose(){this.renderedNodes.clear()}}function $8(o){return{browserEvent:o.browserEvent,elements:o.elements.map(e=>e.element)}}function j8(o){return{browserEvent:o.browserEvent,element:o.element&&o.element.element,target:o.target}}class Ife extends X1{constructor(e){super(e.elements.map(t=>t.element)),this.data=e}}function uT(o){return o instanceof X1?new Ife(o):o}class Efe{constructor(e){this.dnd=e}getDragURI(e){return this.dnd.getDragURI(e.element)}getDragLabel(e,t){if(this.dnd.getDragLabel)return this.dnd.getDragLabel(e.map(i=>i.element),t)}onDragStart(e,t){var i,n;(n=(i=this.dnd).onDragStart)==null||n.call(i,uT(e),t)}onDragOver(e,t,i,n,s,r=!0){return this.dnd.onDragOver(uT(e),t&&t.element,i,n,s)}drop(e,t,i,n,s){this.dnd.drop(uT(e),t&&t.element,i,n,s)}onDragEnd(e){var t,i;(i=(t=this.dnd).onDragEnd)==null||i.call(t,e)}dispose(){this.dnd.dispose()}}function MK(o){return o&&{...o,collapseByDefault:!0,identityProvider:o.identityProvider&&{getId(e){return o.identityProvider.getId(e.element)}},dnd:o.dnd&&new Efe(o.dnd),multipleSelectionController:o.multipleSelectionController&&{isSelectionSingleChangeEvent(e){return o.multipleSelectionController.isSelectionSingleChangeEvent({...e,element:e.element})},isSelectionRangeChangeEvent(e){return o.multipleSelectionController.isSelectionRangeChangeEvent({...e,element:e.element})}},accessibilityProvider:o.accessibilityProvider&&{...o.accessibilityProvider,getPosInSet:void 0,getSetSize:void 0,getRole:o.accessibilityProvider.getRole?e=>o.accessibilityProvider.getRole(e.element):()=>"treeitem",isChecked:o.accessibilityProvider.isChecked?e=>{var t;return!!((t=o.accessibilityProvider)!=null&&t.isChecked(e.element))}:void 0,getAriaLabel(e){return o.accessibilityProvider.getAriaLabel(e.element)},getWidgetAriaLabel(){return o.accessibilityProvider.getWidgetAriaLabel()},getWidgetRole:o.accessibilityProvider.getWidgetRole?()=>o.accessibilityProvider.getWidgetRole():()=>"tree",getAriaLevel:o.accessibilityProvider.getAriaLevel&&(e=>o.accessibilityProvider.getAriaLevel(e.element)),getActiveDescendantId:o.accessibilityProvider.getActiveDescendantId&&(e=>o.accessibilityProvider.getActiveDescendantId(e.element))},filter:o.filter&&{filter(e,t){return o.filter.filter(e.element,t)}},keyboardNavigationLabelProvider:o.keyboardNavigationLabelProvider&&{...o.keyboardNavigationLabelProvider,getKeyboardNavigationLabel(e){return o.keyboardNavigationLabelProvider.getKeyboardNavigationLabel(e.element)}},sorter:void 0,expandOnlyOnTwistieClick:typeof o.expandOnlyOnTwistieClick>"u"?void 0:typeof o.expandOnlyOnTwistieClick!="function"?o.expandOnlyOnTwistieClick:e=>o.expandOnlyOnTwistieClick(e.element),defaultFindVisibility:e=>e.hasChildren&&e.stale?1:typeof o.defaultFindVisibility=="number"?o.defaultFindVisibility:typeof o.defaultFindVisibility>"u"?2:o.defaultFindVisibility(e.element)}}function xA(o,e){e(o),o.children.forEach(t=>xA(t,e))}class AK{get onDidScroll(){return this.tree.onDidScroll}get onDidChangeFocus(){return oe.map(this.tree.onDidChangeFocus,$8)}get onDidChangeSelection(){return oe.map(this.tree.onDidChangeSelection,$8)}get onMouseDblClick(){return oe.map(this.tree.onMouseDblClick,j8)}get onPointer(){return oe.map(this.tree.onPointer,j8)}get onDidFocus(){return this.tree.onDidFocus}get onDidChangeModel(){return this.tree.onDidChangeModel}get onDidChangeCollapseState(){return this.tree.onDidChangeCollapseState}get onDidChangeFindOpenState(){return this.tree.onDidChangeFindOpenState}get onDidChangeStickyScrollFocused(){return this.tree.onDidChangeStickyScrollFocused}get onDidDispose(){return this.tree.onDidDispose}constructor(e,t,i,n,s,r={}){this.user=e,this.dataSource=s,this.nodes=new Map,this.subTreeRefreshPromises=new Map,this.refreshPromises=new Map,this._onDidRender=new P,this._onDidChangeNodeSlowState=new P,this.nodeMapper=new C3(a=>new x3(a)),this.disposables=new K,this.identityProvider=r.identityProvider,this.autoExpandSingleChildren=typeof r.autoExpandSingleChildren>"u"?!1:r.autoExpandSingleChildren,this.sorter=r.sorter,this.getDefaultCollapseState=a=>r.collapseByDefault?r.collapseByDefault(a)?Do.PreserveOrCollapsed:Do.PreserveOrExpanded:void 0,this.tree=this.createTree(e,t,i,n,r),this.onDidChangeFindMode=this.tree.onDidChangeFindMode,this.onDidChangeFindMatchType=this.tree.onDidChangeFindMatchType,this.root=hT({element:void 0,parent:null,hasChildren:!0,defaultCollapseState:void 0}),this.identityProvider&&(this.root={...this.root,id:null}),this.nodes.set(null,this.root),this.tree.onDidChangeCollapseState(this._onDidChangeCollapseState,this,this.disposables)}createTree(e,t,i,n,s){const r=new w3(i),a=n.map(c=>new Dfe(c,this.nodeMapper,this._onDidChangeNodeSlowState.event)),l=MK(s)||{};return new L3(e,t,r,a,l)}updateOptions(e={}){this.tree.updateOptions(e)}getHTMLElement(){return this.tree.getHTMLElement()}get scrollTop(){return this.tree.scrollTop}set scrollTop(e){this.tree.scrollTop=e}get scrollHeight(){return this.tree.scrollHeight}get renderHeight(){return this.tree.renderHeight}domFocus(){this.tree.domFocus()}layout(e,t){this.tree.layout(e,t)}style(e){this.tree.style(e)}getInput(){return this.root.element}async setInput(e,t){this.refreshPromises.forEach(n=>n.cancel()),this.refreshPromises.clear(),this.root.element=e;const i=t&&{viewState:t,focus:[],selection:[]};await this._updateChildren(e,!0,!1,i),i&&(this.tree.setFocus(i.focus),this.tree.setSelection(i.selection)),t&&typeof t.scrollTop=="number"&&(this.scrollTop=t.scrollTop)}async _updateChildren(e=this.root.element,t=!0,i=!1,n,s){if(typeof this.root.element>"u")throw new Ao(this.user,"Tree input not set");this.root.refreshPromise&&(await this.root.refreshPromise,await oe.toPromise(this._onDidRender.event));const r=this.getDataNode(e);if(await this.refreshAndRenderNode(r,t,n,s),i)try{this.tree.rerender(r)}catch{}}rerender(e){if(e===void 0||e===this.root.element){this.tree.rerender();return}const t=this.getDataNode(e);this.tree.rerender(t)}getNode(e=this.root.element){const t=this.getDataNode(e),i=this.tree.getNode(t===this.root?null:t);return this.nodeMapper.map(i)}collapse(e,t=!1){const i=this.getDataNode(e);return this.tree.collapse(i===this.root?null:i,t)}async expand(e,t=!1){if(typeof this.root.element>"u")throw new Ao(this.user,"Tree input not set");this.root.refreshPromise&&(await this.root.refreshPromise,await oe.toPromise(this._onDidRender.event));const i=this.getDataNode(e);if(this.tree.hasElement(i)&&!this.tree.isCollapsible(i)||(i.refreshPromise&&(await this.root.refreshPromise,await oe.toPromise(this._onDidRender.event)),i!==this.root&&!i.refreshPromise&&!this.tree.isCollapsed(i)))return!1;const n=this.tree.expand(i===this.root?null:i,t);return i.refreshPromise&&(await this.root.refreshPromise,await oe.toPromise(this._onDidRender.event)),n}setSelection(e,t){const i=e.map(n=>this.getDataNode(n));this.tree.setSelection(i,t)}getSelection(){return this.tree.getSelection().map(t=>t.element)}setFocus(e,t){const i=e.map(n=>this.getDataNode(n));this.tree.setFocus(i,t)}getFocus(){return this.tree.getFocus().map(t=>t.element)}reveal(e,t){this.tree.reveal(this.getDataNode(e),t)}getParentElement(e){const t=this.tree.getParentElement(this.getDataNode(e));return t&&t.element}getFirstElementChild(e=this.root.element){const t=this.getDataNode(e),i=this.tree.getFirstElementChild(t===this.root?null:t);return i&&i.element}getDataNode(e){const t=this.nodes.get(e===this.root.element?null:e);if(!t)throw new Ao(this.user,`Data tree node not found: ${e}`);return t}async refreshAndRenderNode(e,t,i,n){await this.refreshNode(e,t,i),!this.disposables.isDisposed&&this.render(e,i,n)}async refreshNode(e,t,i){let n;if(this.subTreeRefreshPromises.forEach((s,r)=>{!n&&kfe(r,e)&&(n=s.then(()=>this.refreshNode(e,t,i)))}),n)return n;if(e!==this.root&&this.tree.getNode(e).collapsed){e.hasChildren=!!this.dataSource.hasChildren(e.element),e.stale=!0,this.setChildren(e,[],t,i);return}return this.doRefreshSubTree(e,t,i)}async doRefreshSubTree(e,t,i){let n;e.refreshPromise=new Promise(s=>n=s),this.subTreeRefreshPromises.set(e,e.refreshPromise),e.refreshPromise.finally(()=>{e.refreshPromise=void 0,this.subTreeRefreshPromises.delete(e)});try{const s=await this.doRefreshNode(e,t,i);e.stale=!1,await yM.settled(s.map(r=>this.doRefreshSubTree(r,t,i)))}finally{n()}}async doRefreshNode(e,t,i){e.hasChildren=!!this.dataSource.hasChildren(e.element);let n;if(!e.hasChildren)n=Promise.resolve(ot.empty());else{const s=this.doGetChildren(e);if(HB(s))n=Promise.resolve(s);else{const r=ed(800);r.then(()=>{e.slow=!0,this._onDidChangeNodeSlowState.fire(e)},a=>null),n=s.finally(()=>r.cancel())}}try{const s=await n;return this.setChildren(e,s,t,i)}catch(s){if(e!==this.root&&this.tree.hasElement(e)&&this.tree.collapse(e),yr(s))return[];throw s}finally{e.slow&&(e.slow=!1,this._onDidChangeNodeSlowState.fire(e))}}doGetChildren(e){let t=this.refreshPromises.get(e);if(t)return t;const i=this.dataSource.getChildren(e.element);return HB(i)?this.processChildren(i):(t=Yi(async()=>this.processChildren(await i)),this.refreshPromises.set(e,t),t.finally(()=>{this.refreshPromises.delete(e)}))}_onDidChangeCollapseState({node:e,deep:t}){e.element!==null&&!e.collapsed&&e.element.stale&&(t?this.collapse(e.element.element):this.refreshAndRenderNode(e.element,!1).catch(Oe))}setChildren(e,t,i,n){const s=[...t];if(e.children.length===0&&s.length===0)return[];const r=new Map,a=new Map;for(const d of e.children)r.set(d.element,d),this.identityProvider&&a.set(d.id,{node:d,collapsed:this.tree.hasElement(d)&&this.tree.isCollapsed(d)});const l=[],c=s.map(d=>{const h=!!this.dataSource.hasChildren(d);if(!this.identityProvider){const p=hT({element:d,parent:e,hasChildren:h,defaultCollapseState:this.getDefaultCollapseState(d)});return h&&p.defaultCollapseState===Do.PreserveOrExpanded&&l.push(p),p}const u=this.identityProvider.getId(d).toString(),g=a.get(u);if(g){const p=g.node;return r.delete(p.element),this.nodes.delete(p.element),this.nodes.set(d,p),p.element=d,p.hasChildren=h,i?g.collapsed?(p.children.forEach(_=>xA(_,b=>this.nodes.delete(b.element))),p.children.splice(0,p.children.length),p.stale=!0):l.push(p):h&&!g.collapsed&&l.push(p),p}const f=hT({element:d,parent:e,id:u,hasChildren:h,defaultCollapseState:this.getDefaultCollapseState(d)});return n&&n.viewState.focus&&n.viewState.focus.indexOf(u)>-1&&n.focus.push(f),n&&n.viewState.selection&&n.viewState.selection.indexOf(u)>-1&&n.selection.push(f),(n&&n.viewState.expanded&&n.viewState.expanded.indexOf(u)>-1||h&&f.defaultCollapseState===Do.PreserveOrExpanded)&&l.push(f),f});for(const d of r.values())xA(d,h=>this.nodes.delete(h.element));for(const d of c)this.nodes.set(d.element,d);return e.children.splice(0,e.children.length,...c),e!==this.root&&this.autoExpandSingleChildren&&c.length===1&&l.length===0&&(c[0].forceExpanded=!0,l.push(c[0])),l}render(e,t,i){const n=e.children.map(r=>this.asTreeElement(r,t)),s=i&&{...i,diffIdentityProvider:i.diffIdentityProvider&&{getId(r){return i.diffIdentityProvider.getId(r.element)}}};this.tree.setChildren(e===this.root?null:e,n,s),e!==this.root&&this.tree.setCollapsible(e,e.hasChildren),this._onDidRender.fire()}asTreeElement(e,t){if(e.stale)return{element:e,collapsible:e.hasChildren,collapsed:!0};let i;return t&&t.viewState.expanded&&e.id&&t.viewState.expanded.indexOf(e.id)>-1?i=!1:e.forceExpanded?(i=!1,e.forceExpanded=!1):i=e.defaultCollapseState,{element:e,children:e.hasChildren?ot.map(e.children,n=>this.asTreeElement(n,t)):[],collapsible:e.hasChildren,collapsed:i}}processChildren(e){return this.sorter&&(e=[...e].sort(this.sorter.compare.bind(this.sorter))),e}dispose(){this.disposables.dispose(),this.tree.dispose()}}class k3{get element(){return{elements:this.node.element.elements.map(e=>e.element),incompressible:this.node.element.incompressible}}get children(){return this.node.children.map(e=>new k3(e))}get depth(){return this.node.depth}get visibleChildrenCount(){return this.node.visibleChildrenCount}get visibleChildIndex(){return this.node.visibleChildIndex}get collapsible(){return this.node.collapsible}get collapsed(){return this.node.collapsed}get visible(){return this.node.visible}get filterData(){return this.node.filterData}constructor(e){this.node=e}}class Nfe{constructor(e,t,i,n){this.renderer=e,this.nodeMapper=t,this.compressibleNodeMapperProvider=i,this.onDidChangeTwistieState=n,this.renderedNodes=new Map,this.disposables=[],this.templateId=e.templateId}renderTemplate(e){return{templateData:this.renderer.renderTemplate(e)}}renderElement(e,t,i,n){this.renderer.renderElement(this.nodeMapper.map(e),t,i.templateData,n)}renderCompressedElements(e,t,i,n){this.renderer.renderCompressedElements(this.compressibleNodeMapperProvider().map(e),t,i.templateData,n)}renderTwistie(e,t){return e.slow?(t.classList.add(...Ie.asClassNameArray(te.treeItemLoading)),!0):(t.classList.remove(...Ie.asClassNameArray(te.treeItemLoading)),!1)}disposeElement(e,t,i,n){var s,r;(r=(s=this.renderer).disposeElement)==null||r.call(s,this.nodeMapper.map(e),t,i.templateData,n)}disposeCompressedElements(e,t,i,n){var s,r;(r=(s=this.renderer).disposeCompressedElements)==null||r.call(s,this.compressibleNodeMapperProvider().map(e),t,i.templateData,n)}disposeTemplate(e){this.renderer.disposeTemplate(e.templateData)}dispose(){this.renderedNodes.clear(),this.disposables=Tt(this.disposables)}}function Tfe(o){const e=o&&MK(o);return e&&{...e,keyboardNavigationLabelProvider:e.keyboardNavigationLabelProvider&&{...e.keyboardNavigationLabelProvider,getCompressedNodeKeyboardNavigationLabel(t){return o.keyboardNavigationLabelProvider.getCompressedNodeKeyboardNavigationLabel(t.map(i=>i.element))}}}}class Rfe extends AK{constructor(e,t,i,n,s,r,a={}){super(e,t,i,s,r,a),this.compressionDelegate=n,this.compressibleNodeMapper=new C3(l=>new k3(l)),this.filter=a.filter}createTree(e,t,i,n,s){const r=new w3(i),a=n.map(c=>new Nfe(c,this.nodeMapper,()=>this.compressibleNodeMapper,this._onDidChangeNodeSlowState.event)),l=Tfe(s)||{};return new RK(e,t,r,a,l)}asTreeElement(e,t){return{incompressible:this.compressionDelegate.isIncompressible(e.element),...super.asTreeElement(e,t)}}updateOptions(e={}){this.tree.updateOptions(e)}render(e,t,i){if(!this.identityProvider)return super.render(e,t);const n=g=>this.identityProvider.getId(g).toString(),s=g=>{const f=new Set;for(const p of g){const _=this.tree.getCompressedTreeNode(p===this.root?null:p);if(_.element)for(const b of _.element.elements)f.add(n(b.element))}return f},r=s(this.tree.getSelection()),a=s(this.tree.getFocus());super.render(e,t,i);const l=this.getSelection();let c=!1;const d=this.getFocus();let h=!1;const u=g=>{const f=g.element;if(f)for(let p=0;p<f.elements.length;p++){const _=n(f.elements[p].element),b=f.elements[f.elements.length-1].element;r.has(_)&&l.indexOf(b)===-1&&(l.push(b),c=!0),a.has(_)&&d.indexOf(b)===-1&&(d.push(b),h=!0)}g.children.forEach(u)};u(this.tree.getCompressedTreeNode(e===this.root?null:e)),c&&this.setSelection(l),h&&this.setFocus(d)}processChildren(e){return this.filter&&(e=ot.filter(e,t=>{const i=this.filter.filter(t,1),n=Mfe(i);if(n===2)throw new Error("Recursive tree visibility not supported in async data compressed trees");return n===1})),super.processChildren(e)}}function Mfe(o){return typeof o=="boolean"?o?1:0:v3(o)?Pw(o.visibility):Pw(o)}class Afe extends EK{constructor(e,t,i,n,s,r={}){super(e,t,i,n,r),this.user=e,this.dataSource=s,this.identityProvider=r.identityProvider}createModel(e,t,i){return new S3(e,t,i)}}new le("isMac",Je,m("isMac","Whether the operating system is macOS"));new le("isLinux",ps,m("isLinux","Whether the operating system is Linux"));const aE=new le("isWindows",En,m("isWindows","Whether the operating system is Windows")),PK=new le("isWeb",Qf,m("isWeb","Whether the platform is a web browser"));new le("isMacNative",Je&&!Qf,m("isMacNative","Whether the operating system is macOS on a non-browser platform"));new le("isIOS",Pa,m("isIOS","Whether the operating system is iOS"));new le("isMobile",Ez,m("isMobile","Whether the platform is a mobile web browser"));new le("isDevelopment",!1,!0);new le("productQualityType","",m("productQualityType","Quality type of VS Code"));const OK="inputFocus",FK=new le(OK,!1,m("inputFocus","Whether keyboard focus is inside an input box"));var fd=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},jt=function(o,e){return function(t,i){e(t,i,o)}};const kr=et("listService");class Pfe{get lastFocusedList(){return this._lastFocusedWidget}constructor(){this.disposables=new K,this.lists=[],this._lastFocusedWidget=void 0,this._hasCreatedStyleController=!1}setLastFocusedList(e){var t,i;e!==this._lastFocusedWidget&&((t=this._lastFocusedWidget)==null||t.getHTMLElement().classList.remove("last-focused"),this._lastFocusedWidget=e,(i=this._lastFocusedWidget)==null||i.getHTMLElement().classList.add("last-focused"))}register(e,t){if(this._hasCreatedStyleController||(this._hasCreatedStyleController=!0,new Mj($o(),"").style(nm)),this.lists.some(n=>n.widget===e))throw new Error("Cannot register the same widget multiple times");const i={widget:e,extraContextKeys:t};return this.lists.push(i),wI(e.getHTMLElement())&&this.setLastFocusedList(e),jr(e.onDidFocus(()=>this.setLastFocusedList(e)),ve(()=>this.lists.splice(this.lists.indexOf(i),1)),e.onDidDispose(()=>{this.lists=this.lists.filter(n=>n!==i),this._lastFocusedWidget===e&&this.setLastFocusedList(void 0)}))}dispose(){this.disposables.dispose()}}const Ow=new le("listScrollAtBoundary","none");G.or(Ow.isEqualTo("top"),Ow.isEqualTo("both"));G.or(Ow.isEqualTo("bottom"),Ow.isEqualTo("both"));const WK=new le("listFocus",!0),BK=new le("treestickyScrollFocused",!1),lE=new le("listSupportsMultiselect",!0),HK=G.and(WK,G.not(OK),BK.negate()),D3=new le("listHasSelectionOrFocus",!1),I3=new le("listDoubleSelection",!1),E3=new le("listMultiSelection",!1),cE=new le("listSelectionNavigation",!1),Ofe=new le("listSupportsFind",!0),N3=new le("treeElementCanCollapse",!1),Ffe=new le("treeElementHasParent",!1),T3=new le("treeElementCanExpand",!1),Wfe=new le("treeElementHasChild",!1),Bfe=new le("treeFindOpen",!1),VK="listTypeNavigationMode",zK="listAutomaticKeyboardNavigation";function dE(o,e){const t=o.createScoped(e.getHTMLElement());return WK.bindTo(t),t}function hE(o,e){const t=Ow.bindTo(o),i=()=>{const n=e.scrollTop===0,s=e.scrollHeight-e.renderHeight-e.scrollTop<1;n&&s?t.set("both"):n?t.set("top"):s?t.set("bottom"):t.set("none")};return i(),e.onDidScroll(i)}const rm="workbench.list.multiSelectModifier",oy="workbench.list.openMode",mr="workbench.list.horizontalScrolling",R3="workbench.list.defaultFindMode",M3="workbench.list.typeNavigationMode",zL="workbench.list.keyboardNavigation",Ba="workbench.list.scrollByPage",A3="workbench.list.defaultFindMatchType",Fw="workbench.tree.indent",UL="workbench.tree.renderIndentGuides",Ha="workbench.list.smoothScrolling",zl="workbench.list.mouseWheelScrollSensitivity",Ul="workbench.list.fastScrollSensitivity",$L="workbench.tree.expandMode",jL="workbench.tree.enableStickyScroll",KL="workbench.tree.stickyScrollMaxItemCount";function $l(o){return o.getValue(rm)==="alt"}class Hfe extends F{constructor(e){super(),this.configurationService=e,this.useAltAsMultipleSelectionModifier=$l(e),this.registerListeners()}registerListeners(){this._register(this.configurationService.onDidChangeConfiguration(e=>{e.affectsConfiguration(rm)&&(this.useAltAsMultipleSelectionModifier=$l(this.configurationService))}))}isSelectionSingleChangeEvent(e){return this.useAltAsMultipleSelectionModifier?e.browserEvent.altKey:Nj(e)}isSelectionRangeChangeEvent(e){return Tj(e)}}function uE(o,e){const t=o.get(qe),i=o.get(yt),n=new K;return[{...e,keyboardNavigationDelegate:{mightProducePrintableCharacter(r){return i.mightProducePrintableCharacter(r)}},smoothScrolling:!!t.getValue(Ha),mouseWheelScrollSensitivity:t.getValue(zl),fastScrollSensitivity:t.getValue(Ul),multipleSelectionController:e.multipleSelectionController??n.add(new Hfe(t)),keyboardNavigationEventFilter:Ufe(i),scrollByPage:!!t.getValue(Ba)},n]}let K8=class extends Xo{constructor(e,t,i,n,s,r,a,l,c){const d=typeof s.horizontalScrolling<"u"?s.horizontalScrolling:!!l.getValue(mr),[h,u]=c.invokeFunction(uE,s);super(e,t,i,n,{keyboardSupport:!1,...h,horizontalScrolling:d}),this.disposables.add(u),this.contextKeyService=dE(r,this),this.disposables.add(hE(this.contextKeyService,this)),this.listSupportsMultiSelect=lE.bindTo(this.contextKeyService),this.listSupportsMultiSelect.set(s.multipleSelectionSupport!==!1),cE.bindTo(this.contextKeyService).set(!!s.selectionNavigation),this.listHasSelectionOrFocus=D3.bindTo(this.contextKeyService),this.listDoubleSelection=I3.bindTo(this.contextKeyService),this.listMultiSelection=E3.bindTo(this.contextKeyService),this.horizontalScrolling=s.horizontalScrolling,this._useAltAsMultipleSelectionModifier=$l(l),this.disposables.add(this.contextKeyService),this.disposables.add(a.register(this)),this.updateStyles(s.overrideStyles),this.disposables.add(this.onDidChangeSelection(()=>{const f=this.getSelection(),p=this.getFocus();this.contextKeyService.bufferChangeEvents(()=>{this.listHasSelectionOrFocus.set(f.length>0||p.length>0),this.listMultiSelection.set(f.length>1),this.listDoubleSelection.set(f.length===2)})})),this.disposables.add(this.onDidChangeFocus(()=>{const f=this.getSelection(),p=this.getFocus();this.listHasSelectionOrFocus.set(f.length>0||p.length>0)})),this.disposables.add(l.onDidChangeConfiguration(f=>{f.affectsConfiguration(rm)&&(this._useAltAsMultipleSelectionModifier=$l(l));let p={};if(f.affectsConfiguration(mr)&&this.horizontalScrolling===void 0){const _=!!l.getValue(mr);p={...p,horizontalScrolling:_}}if(f.affectsConfiguration(Ba)){const _=!!l.getValue(Ba);p={...p,scrollByPage:_}}if(f.affectsConfiguration(Ha)){const _=!!l.getValue(Ha);p={...p,smoothScrolling:_}}if(f.affectsConfiguration(zl)){const _=l.getValue(zl);p={...p,mouseWheelScrollSensitivity:_}}if(f.affectsConfiguration(Ul)){const _=l.getValue(Ul);p={...p,fastScrollSensitivity:_}}Object.keys(p).length>0&&this.updateOptions(p)})),this.navigator=new UK(this,{configurationService:l,...s}),this.disposables.add(this.navigator)}updateOptions(e){super.updateOptions(e),e.overrideStyles!==void 0&&this.updateStyles(e.overrideStyles),e.multipleSelectionSupport!==void 0&&this.listSupportsMultiSelect.set(!!e.multipleSelectionSupport)}updateStyles(e){this.style(e?sm(e):nm)}};K8=fd([jt(5,Ee),jt(6,kr),jt(7,qe),jt(8,we)],K8);let q8=class extends jge{constructor(e,t,i,n,s,r,a,l,c){const d=typeof s.horizontalScrolling<"u"?s.horizontalScrolling:!!l.getValue(mr),[h,u]=c.invokeFunction(uE,s);super(e,t,i,n,{keyboardSupport:!1,...h,horizontalScrolling:d}),this.disposables=new K,this.disposables.add(u),this.contextKeyService=dE(r,this),this.disposables.add(hE(this.contextKeyService,this.widget)),this.horizontalScrolling=s.horizontalScrolling,this.listSupportsMultiSelect=lE.bindTo(this.contextKeyService),this.listSupportsMultiSelect.set(s.multipleSelectionSupport!==!1),cE.bindTo(this.contextKeyService).set(!!s.selectionNavigation),this._useAltAsMultipleSelectionModifier=$l(l),this.disposables.add(this.contextKeyService),this.disposables.add(a.register(this)),this.updateStyles(s.overrideStyles),this.disposables.add(l.onDidChangeConfiguration(f=>{f.affectsConfiguration(rm)&&(this._useAltAsMultipleSelectionModifier=$l(l));let p={};if(f.affectsConfiguration(mr)&&this.horizontalScrolling===void 0){const _=!!l.getValue(mr);p={...p,horizontalScrolling:_}}if(f.affectsConfiguration(Ba)){const _=!!l.getValue(Ba);p={...p,scrollByPage:_}}if(f.affectsConfiguration(Ha)){const _=!!l.getValue(Ha);p={...p,smoothScrolling:_}}if(f.affectsConfiguration(zl)){const _=l.getValue(zl);p={...p,mouseWheelScrollSensitivity:_}}if(f.affectsConfiguration(Ul)){const _=l.getValue(Ul);p={...p,fastScrollSensitivity:_}}Object.keys(p).length>0&&this.updateOptions(p)})),this.navigator=new UK(this,{configurationService:l,...s}),this.disposables.add(this.navigator)}updateOptions(e){super.updateOptions(e),e.overrideStyles!==void 0&&this.updateStyles(e.overrideStyles),e.multipleSelectionSupport!==void 0&&this.listSupportsMultiSelect.set(!!e.multipleSelectionSupport)}updateStyles(e){this.style(e?sm(e):nm)}dispose(){this.disposables.dispose(),super.dispose()}};q8=fd([jt(5,Ee),jt(6,kr),jt(7,qe),jt(8,we)],q8);let G8=class extends wA{constructor(e,t,i,n,s,r,a,l,c,d){const h=typeof r.horizontalScrolling<"u"?r.horizontalScrolling:!!c.getValue(mr),[u,g]=d.invokeFunction(uE,r);super(e,t,i,n,s,{keyboardSupport:!1,...u,horizontalScrolling:h}),this.disposables.add(g),this.contextKeyService=dE(a,this),this.disposables.add(hE(this.contextKeyService,this)),this.listSupportsMultiSelect=lE.bindTo(this.contextKeyService),this.listSupportsMultiSelect.set(r.multipleSelectionSupport!==!1),cE.bindTo(this.contextKeyService).set(!!r.selectionNavigation),this.listHasSelectionOrFocus=D3.bindTo(this.contextKeyService),this.listDoubleSelection=I3.bindTo(this.contextKeyService),this.listMultiSelection=E3.bindTo(this.contextKeyService),this.horizontalScrolling=r.horizontalScrolling,this._useAltAsMultipleSelectionModifier=$l(c),this.disposables.add(this.contextKeyService),this.disposables.add(l.register(this)),this.updateStyles(r.overrideStyles),this.disposables.add(this.onDidChangeSelection(()=>{const p=this.getSelection(),_=this.getFocus();this.contextKeyService.bufferChangeEvents(()=>{this.listHasSelectionOrFocus.set(p.length>0||_.length>0),this.listMultiSelection.set(p.length>1),this.listDoubleSelection.set(p.length===2)})})),this.disposables.add(this.onDidChangeFocus(()=>{const p=this.getSelection(),_=this.getFocus();this.listHasSelectionOrFocus.set(p.length>0||_.length>0)})),this.disposables.add(c.onDidChangeConfiguration(p=>{p.affectsConfiguration(rm)&&(this._useAltAsMultipleSelectionModifier=$l(c));let _={};if(p.affectsConfiguration(mr)&&this.horizontalScrolling===void 0){const b=!!c.getValue(mr);_={..._,horizontalScrolling:b}}if(p.affectsConfiguration(Ba)){const b=!!c.getValue(Ba);_={..._,scrollByPage:b}}if(p.affectsConfiguration(Ha)){const b=!!c.getValue(Ha);_={..._,smoothScrolling:b}}if(p.affectsConfiguration(zl)){const b=c.getValue(zl);_={..._,mouseWheelScrollSensitivity:b}}if(p.affectsConfiguration(Ul)){const b=c.getValue(Ul);_={..._,fastScrollSensitivity:b}}Object.keys(_).length>0&&this.updateOptions(_)})),this.navigator=new Vfe(this,{configurationService:c,...r}),this.disposables.add(this.navigator)}updateOptions(e){super.updateOptions(e),e.overrideStyles!==void 0&&this.updateStyles(e.overrideStyles),e.multipleSelectionSupport!==void 0&&this.listSupportsMultiSelect.set(!!e.multipleSelectionSupport)}updateStyles(e){this.style(e?sm(e):nm)}dispose(){this.disposables.dispose(),super.dispose()}};G8=fd([jt(6,Ee),jt(7,kr),jt(8,qe),jt(9,we)],G8);class P3 extends F{constructor(e,t){super(),this.widget=e,this._onDidOpen=this._register(new P),this.onDidOpen=this._onDidOpen.event,this._register(oe.filter(this.widget.onDidChangeSelection,i=>wc(i.browserEvent))(i=>this.onSelectionFromKeyboard(i))),this._register(this.widget.onPointer(i=>this.onPointer(i.element,i.browserEvent))),this._register(this.widget.onMouseDblClick(i=>this.onMouseDblClick(i.element,i.browserEvent))),typeof(t==null?void 0:t.openOnSingleClick)!="boolean"&&(t!=null&&t.configurationService)?(this.openOnSingleClick=(t==null?void 0:t.configurationService.getValue(oy))!=="doubleClick",this._register(t==null?void 0:t.configurationService.onDidChangeConfiguration(i=>{i.affectsConfiguration(oy)&&(this.openOnSingleClick=(t==null?void 0:t.configurationService.getValue(oy))!=="doubleClick")}))):this.openOnSingleClick=(t==null?void 0:t.openOnSingleClick)??!0}onSelectionFromKeyboard(e){if(e.elements.length!==1)return;const t=e.browserEvent,i=typeof t.preserveFocus=="boolean"?t.preserveFocus:!0,n=typeof t.pinned=="boolean"?t.pinned:!i;this._open(this.getSelectedElement(),i,n,!1,e.browserEvent)}onPointer(e,t){if(!this.openOnSingleClick||t.detail===2)return;const n=t.button===1,s=!0,r=n,a=t.ctrlKey||t.metaKey||t.altKey;this._open(e,s,r,a,t)}onMouseDblClick(e,t){if(!t)return;const i=t.target;if(i.classList.contains("monaco-tl-twistie")||i.classList.contains("monaco-icon-label")&&i.classList.contains("folder-icon")&&t.offsetX<16)return;const s=!1,r=!0,a=t.ctrlKey||t.metaKey||t.altKey;this._open(e,s,r,a,t)}_open(e,t,i,n,s){e&&this._onDidOpen.fire({editorOptions:{preserveFocus:t,pinned:i,revealIfVisible:!0},sideBySide:n,element:e,browserEvent:s})}}class UK extends P3{constructor(e,t){super(e,t),this.widget=e}getSelectedElement(){return this.widget.getSelectedElements()[0]}}class Vfe extends P3{constructor(e,t){super(e,t)}getSelectedElement(){return this.widget.getSelectedElements()[0]}}class zfe extends P3{constructor(e,t){super(e,t)}getSelectedElement(){return this.widget.getSelection()[0]??void 0}}function Ufe(o){let e=!1;return t=>{if(t.toKeyCodeChord().isModifierKey())return!1;if(e)return e=!1,!1;const i=o.softDispatch(t,t.target);return i.kind===1?(e=!0,!1):(e=!1,i.kind===0)}}let kA=class extends L3{constructor(e,t,i,n,s,r,a,l,c){const{options:d,getTypeNavigationMode:h,disposable:u}=r.invokeFunction(tS,s);super(e,t,i,n,d),this.disposables.add(u),this.internals=new Wf(this,s,h,s.overrideStyles,a,l,c),this.disposables.add(this.internals)}updateOptions(e){super.updateOptions(e),this.internals.updateOptions(e)}};kA=fd([jt(5,we),jt(6,Ee),jt(7,kr),jt(8,qe)],kA);let Z8=class extends RK{constructor(e,t,i,n,s,r,a,l,c){const{options:d,getTypeNavigationMode:h,disposable:u}=r.invokeFunction(tS,s);super(e,t,i,n,d),this.disposables.add(u),this.internals=new Wf(this,s,h,s.overrideStyles,a,l,c),this.disposables.add(this.internals)}updateOptions(e={}){super.updateOptions(e),e.overrideStyles&&this.internals.updateStyleOverrides(e.overrideStyles),this.internals.updateOptions(e)}};Z8=fd([jt(5,we),jt(6,Ee),jt(7,kr),jt(8,qe)],Z8);let Y8=class extends Afe{constructor(e,t,i,n,s,r,a,l,c,d){const{options:h,getTypeNavigationMode:u,disposable:g}=a.invokeFunction(tS,r);super(e,t,i,n,s,h),this.disposables.add(g),this.internals=new Wf(this,r,u,r.overrideStyles,l,c,d),this.disposables.add(this.internals)}updateOptions(e={}){super.updateOptions(e),e.overrideStyles!==void 0&&this.internals.updateStyleOverrides(e.overrideStyles),this.internals.updateOptions(e)}};Y8=fd([jt(6,we),jt(7,Ee),jt(8,kr),jt(9,qe)],Y8);let DA=class extends AK{get onDidOpen(){return this.internals.onDidOpen}constructor(e,t,i,n,s,r,a,l,c,d){const{options:h,getTypeNavigationMode:u,disposable:g}=a.invokeFunction(tS,r);super(e,t,i,n,s,h),this.disposables.add(g),this.internals=new Wf(this,r,u,r.overrideStyles,l,c,d),this.disposables.add(this.internals)}updateOptions(e={}){super.updateOptions(e),e.overrideStyles&&this.internals.updateStyleOverrides(e.overrideStyles),this.internals.updateOptions(e)}};DA=fd([jt(6,we),jt(7,Ee),jt(8,kr),jt(9,qe)],DA);let X8=class extends Rfe{constructor(e,t,i,n,s,r,a,l,c,d,h){const{options:u,getTypeNavigationMode:g,disposable:f}=l.invokeFunction(tS,a);super(e,t,i,n,s,r,u),this.disposables.add(f),this.internals=new Wf(this,a,g,a.overrideStyles,c,d,h),this.disposables.add(this.internals)}updateOptions(e){super.updateOptions(e),this.internals.updateOptions(e)}};X8=fd([jt(7,we),jt(8,Ee),jt(9,kr),jt(10,qe)],X8);function $K(o){const e=o.getValue(R3);if(e==="highlight")return Oc.Highlight;if(e==="filter")return Oc.Filter;const t=o.getValue(zL);if(t==="simple"||t==="highlight")return Oc.Highlight;if(t==="filter")return Oc.Filter}function jK(o){const e=o.getValue(A3);if(e==="fuzzy")return Ff.Fuzzy;if(e==="contiguous")return Ff.Contiguous}function tS(o,e){const t=o.get(qe),i=o.get(ud),n=o.get(Ee),s=o.get(we),r=()=>{const u=n.getContextKeyValue(VK);if(u==="automatic")return Ll.Automatic;if(u==="trigger"||n.getContextKeyValue(zK)===!1)return Ll.Trigger;const f=t.getValue(M3);if(f==="automatic")return Ll.Automatic;if(f==="trigger")return Ll.Trigger},a=e.horizontalScrolling!==void 0?e.horizontalScrolling:!!t.getValue(mr),[l,c]=s.invokeFunction(uE,e),d=e.paddingBottom,h=e.renderIndentGuides!==void 0?e.renderIndentGuides:t.getValue(UL);return{getTypeNavigationMode:r,disposable:c,options:{keyboardSupport:!1,...l,indent:typeof t.getValue(Fw)=="number"?t.getValue(Fw):void 0,renderIndentGuides:h,smoothScrolling:!!t.getValue(Ha),defaultFindMode:$K(t),defaultFindMatchType:jK(t),horizontalScrolling:a,scrollByPage:!!t.getValue(Ba),paddingBottom:d,hideTwistiesOfChildlessElements:e.hideTwistiesOfChildlessElements,expandOnlyOnTwistieClick:e.expandOnlyOnTwistieClick??t.getValue($L)==="doubleClick",contextViewProvider:i,findWidgetStyles:qde,enableStickyScroll:!!t.getValue(jL),stickyScrollMaxItemCount:Number(t.getValue(KL))}}}let Wf=class{get onDidOpen(){return this.navigator.onDidOpen}constructor(e,t,i,n,s,r,a){this.tree=e,this.disposables=[],this.contextKeyService=dE(s,e),this.disposables.push(hE(this.contextKeyService,e)),this.listSupportsMultiSelect=lE.bindTo(this.contextKeyService),this.listSupportsMultiSelect.set(t.multipleSelectionSupport!==!1),cE.bindTo(this.contextKeyService).set(!!t.selectionNavigation),this.listSupportFindWidget=Ofe.bindTo(this.contextKeyService),this.listSupportFindWidget.set(t.findWidgetEnabled??!0),this.hasSelectionOrFocus=D3.bindTo(this.contextKeyService),this.hasDoubleSelection=I3.bindTo(this.contextKeyService),this.hasMultiSelection=E3.bindTo(this.contextKeyService),this.treeElementCanCollapse=N3.bindTo(this.contextKeyService),this.treeElementHasParent=Ffe.bindTo(this.contextKeyService),this.treeElementCanExpand=T3.bindTo(this.contextKeyService),this.treeElementHasChild=Wfe.bindTo(this.contextKeyService),this.treeFindOpen=Bfe.bindTo(this.contextKeyService),this.treeStickyScrollFocused=BK.bindTo(this.contextKeyService),this._useAltAsMultipleSelectionModifier=$l(a),this.updateStyleOverrides(n);const c=()=>{const h=e.getFocus()[0];if(!h)return;const u=e.getNode(h);this.treeElementCanCollapse.set(u.collapsible&&!u.collapsed),this.treeElementHasParent.set(!!e.getParentElement(h)),this.treeElementCanExpand.set(u.collapsible&&u.collapsed),this.treeElementHasChild.set(!!e.getFirstElementChild(h))},d=new Set;d.add(VK),d.add(zK),this.disposables.push(this.contextKeyService,r.register(e),e.onDidChangeSelection(()=>{const h=e.getSelection(),u=e.getFocus();this.contextKeyService.bufferChangeEvents(()=>{this.hasSelectionOrFocus.set(h.length>0||u.length>0),this.hasMultiSelection.set(h.length>1),this.hasDoubleSelection.set(h.length===2)})}),e.onDidChangeFocus(()=>{const h=e.getSelection(),u=e.getFocus();this.hasSelectionOrFocus.set(h.length>0||u.length>0),c()}),e.onDidChangeCollapseState(c),e.onDidChangeModel(c),e.onDidChangeFindOpenState(h=>this.treeFindOpen.set(h)),e.onDidChangeStickyScrollFocused(h=>this.treeStickyScrollFocused.set(h)),a.onDidChangeConfiguration(h=>{let u={};if(h.affectsConfiguration(rm)&&(this._useAltAsMultipleSelectionModifier=$l(a)),h.affectsConfiguration(Fw)){const g=a.getValue(Fw);u={...u,indent:g}}if(h.affectsConfiguration(UL)&&t.renderIndentGuides===void 0){const g=a.getValue(UL);u={...u,renderIndentGuides:g}}if(h.affectsConfiguration(Ha)){const g=!!a.getValue(Ha);u={...u,smoothScrolling:g}}if(h.affectsConfiguration(R3)||h.affectsConfiguration(zL)){const g=$K(a);u={...u,defaultFindMode:g}}if(h.affectsConfiguration(M3)||h.affectsConfiguration(zL)){const g=i();u={...u,typeNavigationMode:g}}if(h.affectsConfiguration(A3)){const g=jK(a);u={...u,defaultFindMatchType:g}}if(h.affectsConfiguration(mr)&&t.horizontalScrolling===void 0){const g=!!a.getValue(mr);u={...u,horizontalScrolling:g}}if(h.affectsConfiguration(Ba)){const g=!!a.getValue(Ba);u={...u,scrollByPage:g}}if(h.affectsConfiguration($L)&&t.expandOnlyOnTwistieClick===void 0&&(u={...u,expandOnlyOnTwistieClick:a.getValue($L)==="doubleClick"}),h.affectsConfiguration(jL)){const g=a.getValue(jL);u={...u,enableStickyScroll:g}}if(h.affectsConfiguration(KL)){const g=Math.max(1,a.getValue(KL));u={...u,stickyScrollMaxItemCount:g}}if(h.affectsConfiguration(zl)){const g=a.getValue(zl);u={...u,mouseWheelScrollSensitivity:g}}if(h.affectsConfiguration(Ul)){const g=a.getValue(Ul);u={...u,fastScrollSensitivity:g}}Object.keys(u).length>0&&e.updateOptions(u)}),this.contextKeyService.onDidChangeContext(h=>{h.affectsSome(d)&&e.updateOptions({typeNavigationMode:i()})})),this.navigator=new zfe(e,{configurationService:a,...t}),this.disposables.push(this.navigator)}updateOptions(e){e.multipleSelectionSupport!==void 0&&this.listSupportsMultiSelect.set(!!e.multipleSelectionSupport)}updateStyleOverrides(e){this.tree.style(e?sm(e):nm)}dispose(){this.disposables=Tt(this.disposables)}};Wf=fd([jt(4,Ee),jt(5,kr),jt(6,qe)],Wf);const $fe=ci.as(ta.Configuration);$fe.registerConfiguration({id:"workbench",order:7,title:m("workbenchConfigurationTitle","Workbench"),type:"object",properties:{[rm]:{type:"string",enum:["ctrlCmd","alt"],markdownEnumDescriptions:[m("multiSelectModifier.ctrlCmd","Maps to `Control` on Windows and Linux and to `Command` on macOS."),m("multiSelectModifier.alt","Maps to `Alt` on Windows and Linux and to `Option` on macOS.")],default:"ctrlCmd",description:m({key:"multiSelectModifier",comment:["- `ctrlCmd` refers to a value the setting can take and should not be localized.","- `Control` and `Command` refer to the modifier keys Ctrl or Cmd on the keyboard and can be localized."]},"The modifier to be used to add an item in trees and lists to a multi-selection with the mouse (for example in the explorer, open editors and scm view). The 'Open to Side' mouse gestures - if supported - will adapt such that they do not conflict with the multiselect modifier.")},[oy]:{type:"string",enum:["singleClick","doubleClick"],default:"singleClick",description:m({key:"openModeModifier",comment:["`singleClick` and `doubleClick` refers to a value the setting can take and should not be localized."]},"Controls how to open items in trees and lists using the mouse (if supported). Note that some trees and lists might choose to ignore this setting if it is not applicable.")},[mr]:{type:"boolean",default:!1,description:m("horizontalScrolling setting","Controls whether lists and trees support horizontal scrolling in the workbench. Warning: turning on this setting has a performance implication.")},[Ba]:{type:"boolean",default:!1,description:m("list.scrollByPage","Controls whether clicks in the scrollbar scroll page by page.")},[Fw]:{type:"number",default:8,minimum:4,maximum:40,description:m("tree indent setting","Controls tree indentation in pixels.")},[UL]:{type:"string",enum:["none","onHover","always"],default:"onHover",description:m("render tree indent guides","Controls whether the tree should render indent guides.")},[Ha]:{type:"boolean",default:!1,description:m("list smoothScrolling setting","Controls whether lists and trees have smooth scrolling.")},[zl]:{type:"number",default:1,markdownDescription:m("Mouse Wheel Scroll Sensitivity","A multiplier to be used on the `deltaX` and `deltaY` of mouse wheel scroll events.")},[Ul]:{type:"number",default:5,markdownDescription:m("Fast Scroll Sensitivity","Scrolling speed multiplier when pressing `Alt`.")},[R3]:{type:"string",enum:["highlight","filter"],enumDescriptions:[m("defaultFindModeSettingKey.highlight","Highlight elements when searching. Further up and down navigation will traverse only the highlighted elements."),m("defaultFindModeSettingKey.filter","Filter elements when searching.")],default:"highlight",description:m("defaultFindModeSettingKey","Controls the default find mode for lists and trees in the workbench.")},[zL]:{type:"string",enum:["simple","highlight","filter"],enumDescriptions:[m("keyboardNavigationSettingKey.simple","Simple keyboard navigation focuses elements which match the keyboard input. Matching is done only on prefixes."),m("keyboardNavigationSettingKey.highlight","Highlight keyboard navigation highlights elements which match the keyboard input. Further up and down navigation will traverse only the highlighted elements."),m("keyboardNavigationSettingKey.filter","Filter keyboard navigation will filter out and hide all the elements which do not match the keyboard input.")],default:"highlight",description:m("keyboardNavigationSettingKey","Controls the keyboard navigation style for lists and trees in the workbench. Can be simple, highlight and filter."),deprecated:!0,deprecationMessage:m("keyboardNavigationSettingKeyDeprecated","Please use 'workbench.list.defaultFindMode' and	'workbench.list.typeNavigationMode' instead.")},[A3]:{type:"string",enum:["fuzzy","contiguous"],enumDescriptions:[m("defaultFindMatchTypeSettingKey.fuzzy","Use fuzzy matching when searching."),m("defaultFindMatchTypeSettingKey.contiguous","Use contiguous matching when searching.")],default:"fuzzy",description:m("defaultFindMatchTypeSettingKey","Controls the type of matching used when searching lists and trees in the workbench.")},[$L]:{type:"string",enum:["singleClick","doubleClick"],default:"singleClick",description:m("expand mode","Controls how tree folders are expanded when clicking the folder names. Note that some trees and lists might choose to ignore this setting if it is not applicable.")},[jL]:{type:"boolean",default:!0,description:m("sticky scroll","Controls whether sticky scrolling is enabled in trees.")},[KL]:{type:"number",minimum:1,default:7,markdownDescription:m("sticky scroll maximum items","Controls the number of sticky elements displayed in the tree when {0} is enabled.","`#workbench.tree.enableStickyScroll#`")},[M3]:{type:"string",enum:["automatic","trigger"],default:"automatic",markdownDescription:m("typeNavigationMode2","Controls how type navigation works in lists and trees in the workbench. When set to `trigger`, type navigation begins once the `list.triggerTypeNavigation` command is run.")}}});class gh extends F{constructor(e,t){super(),this.options=t,this.text="",this.title="",this.highlights=[],this.didEverRender=!1,this.supportIcons=(t==null?void 0:t.supportIcons)??!1,this.domNode=X(e,ne("span.monaco-highlighted-label"))}get element(){return this.domNode}set(e,t=[],i="",n){e||(e=""),n&&(e=gh.escapeNewLines(e,t)),!(this.didEverRender&&this.text===e&&this.title===i&&oo(this.highlights,t))&&(this.text=e,this.title=i,this.highlights=t,this.render())}render(){var i,n,s;const e=[];let t=0;for(const r of this.highlights){if(r.end===r.start)continue;if(t<r.start){const c=this.text.substring(t,r.start);this.supportIcons?e.push(...dh(c)):e.push(c),t=r.start}const a=this.text.substring(t,r.end),l=ne("span.highlight",void 0,...this.supportIcons?dh(a):[a]);r.extraClasses&&l.classList.add(...r.extraClasses),e.push(l),t=r.end}if(t<this.text.length){const r=this.text.substring(t);this.supportIcons?e.push(...dh(r)):e.push(r)}if(_n(this.domNode,...e),(n=(i=this.options)==null?void 0:i.hoverDelegate)!=null&&n.showNativeHover)this.domNode.title=this.title;else if(!this.customHover&&this.title!==""){const r=((s=this.options)==null?void 0:s.hoverDelegate)??Ss("mouse");this.customHover=this._register(ja().setupManagedHover(r,this.domNode,this.title))}else this.customHover&&this.customHover.update(this.title);this.didEverRender=!0}static escapeNewLines(e,t){let i=0,n=0;return e.replace(/\r\n|\r|\n/g,(s,r)=>{n=s===`\r
`?-1:0,r+=i;for(const a of t)a.end<=r||(a.start>=r&&(a.start+=n),a.end>=r&&(a.end+=n));return i+=n,"⏎"})}}class aC{constructor(e){this._element=e}get element(){return this._element}set textContent(e){this.disposed||e===this._textContent||(this._textContent=e,this._element.textContent=e)}set classNames(e){this.disposed||oo(e,this._classNames)||(this._classNames=e,this._element.classList.value="",this._element.classList.add(...e))}set empty(e){this.disposed||e===this._empty||(this._empty=e,this._element.style.marginLeft=e?"0":"")}dispose(){this.disposed=!0}}class qL extends F{constructor(e,t){super(),this.customHovers=new Map,this.creationOptions=t,this.domNode=this._register(new aC(X(e,ne(".monaco-icon-label")))),this.labelContainer=X(this.domNode.element,ne(".monaco-icon-label-container")),this.nameContainer=X(this.labelContainer,ne("span.monaco-icon-name-container")),t!=null&&t.supportHighlights||t!=null&&t.supportIcons?this.nameNode=this._register(new qfe(this.nameContainer,!!t.supportIcons)):this.nameNode=new jfe(this.nameContainer),this.hoverDelegate=(t==null?void 0:t.hoverDelegate)??Ss("mouse")}get element(){return this.domNode.element}setLabel(e,t,i){const n=["monaco-icon-label"],s=["monaco-icon-label-container"];let r="";i&&(i.extraClasses&&n.push(...i.extraClasses),i.italic&&n.push("italic"),i.strikethrough&&n.push("strikethrough"),i.disabledCommand&&s.push("disabled"),i.title&&(typeof i.title=="string"?r+=i.title:r+=e));const a=this.domNode.element.querySelector(".monaco-icon-label-iconpath");if(i!=null&&i.iconPath){let l;!a||!Xi(a)?(l=ne(".monaco-icon-label-iconpath"),this.domNode.element.prepend(l)):l=a,l.style.backgroundImage=td(i==null?void 0:i.iconPath)}else a&&a.remove();if(this.domNode.classNames=n,this.domNode.element.setAttribute("aria-label",r),this.labelContainer.classList.value="",this.labelContainer.classList.add(...s),this.setupHover(i!=null&&i.descriptionTitle?this.labelContainer:this.element,i==null?void 0:i.title),this.nameNode.setLabel(e,i),t||this.descriptionNode){const l=this.getOrCreateDescriptionNode();l instanceof gh?(l.set(t||"",i?i.descriptionMatches:void 0,void 0,i==null?void 0:i.labelEscapeNewLines),this.setupHover(l.element,i==null?void 0:i.descriptionTitle)):(l.textContent=t&&(i!=null&&i.labelEscapeNewLines)?gh.escapeNewLines(t,[]):t||"",this.setupHover(l.element,(i==null?void 0:i.descriptionTitle)||""),l.empty=!t)}if(i!=null&&i.suffix||this.suffixNode){const l=this.getOrCreateSuffixNode();l.textContent=(i==null?void 0:i.suffix)??""}}setupHover(e,t){const i=this.customHovers.get(e);if(i&&(i.dispose(),this.customHovers.delete(e)),!t){e.removeAttribute("title");return}if(this.hoverDelegate.showNativeHover)(function(s,r){As(r)?s.title=K5(r):r!=null&&r.markdownNotSupportedFallback?s.title=r.markdownNotSupportedFallback:s.removeAttribute("title")})(e,t);else{const n=ja().setupManagedHover(this.hoverDelegate,e,t);n&&this.customHovers.set(e,n)}}dispose(){super.dispose();for(const e of this.customHovers.values())e.dispose();this.customHovers.clear()}getOrCreateSuffixNode(){if(!this.suffixNode){const e=this._register(new aC(Yte(this.nameContainer,ne("span.monaco-icon-suffix-container"))));this.suffixNode=this._register(new aC(X(e.element,ne("span.label-suffix"))))}return this.suffixNode}getOrCreateDescriptionNode(){var e;if(!this.descriptionNode){const t=this._register(new aC(X(this.labelContainer,ne("span.monaco-icon-description-container"))));(e=this.creationOptions)!=null&&e.supportDescriptionHighlights?this.descriptionNode=this._register(new gh(X(t.element,ne("span.label-description")),{supportIcons:!!this.creationOptions.supportIcons})):this.descriptionNode=this._register(new aC(X(t.element,ne("span.label-description"))))}return this.descriptionNode}}class jfe{constructor(e){this.container=e,this.label=void 0,this.singleLabel=void 0}setLabel(e,t){if(!(this.label===e&&oo(this.options,t)))if(this.label=e,this.options=t,typeof e=="string")this.singleLabel||(this.container.innerText="",this.container.classList.remove("multiple"),this.singleLabel=X(this.container,ne("a.label-name",{id:t==null?void 0:t.domId}))),this.singleLabel.textContent=e;else{this.container.innerText="",this.container.classList.add("multiple"),this.singleLabel=void 0;for(let i=0;i<e.length;i++){const n=e[i],s=(t==null?void 0:t.domId)&&`${t==null?void 0:t.domId}_${i}`;X(this.container,ne("a.label-name",{id:s,"data-icon-label-count":e.length,"data-icon-label-index":i,role:"treeitem"},n)),i<e.length-1&&X(this.container,ne("span.label-separator",void 0,(t==null?void 0:t.separator)||"/"))}}}}function Kfe(o,e,t){if(!t)return;let i=0;return o.map(n=>{const s={start:i,end:i+n.length},r=t.map(a=>Rn.intersect(s,a)).filter(a=>!Rn.isEmpty(a)).map(({start:a,end:l})=>({start:a-i,end:l-i}));return i=s.end+e.length,r})}class qfe extends F{constructor(e,t){super(),this.container=e,this.supportIcons=t,this.label=void 0,this.singleLabel=void 0}setLabel(e,t){if(!(this.label===e&&oo(this.options,t)))if(this.label=e,this.options=t,typeof e=="string")this.singleLabel||(this.container.innerText="",this.container.classList.remove("multiple"),this.singleLabel=this._register(new gh(X(this.container,ne("a.label-name",{id:t==null?void 0:t.domId})),{supportIcons:this.supportIcons}))),this.singleLabel.set(e,t==null?void 0:t.matches,void 0,t==null?void 0:t.labelEscapeNewLines);else{this.container.innerText="",this.container.classList.add("multiple"),this.singleLabel=void 0;const i=(t==null?void 0:t.separator)||"/",n=Kfe(e,i,t==null?void 0:t.matches);for(let s=0;s<e.length;s++){const r=e[s],a=n?n[s]:void 0,l=(t==null?void 0:t.domId)&&`${t==null?void 0:t.domId}_${s}`,c=ne("a.label-name",{id:l,"data-icon-label-count":e.length,"data-icon-label-index":s,role:"treeitem"});this._register(new gh(X(this.container,c),{supportIcons:this.supportIcons})).set(r,a,void 0,t==null?void 0:t.labelEscapeNewLines),s<e.length-1&&X(c,ne("span.label-separator",void 0,i))}}}}const n0=ne,KK={keybindingLabelBackground:void 0,keybindingLabelForeground:void 0,keybindingLabelBorder:void 0,keybindingLabelBottomBorder:void 0,keybindingLabelShadow:void 0};class Ib extends F{constructor(e,t,i){super(),this.os=t,this.keyElements=new Set,this.options=i||Object.create(null);const n=this.options.keybindingLabelForeground;this.domNode=X(e,n0(".monaco-keybinding")),n&&(this.domNode.style.color=n),this.hover=this._register(ja().setupManagedHover(Ss("mouse"),this.domNode,"")),this.didEverRender=!1,e.appendChild(this.domNode)}get element(){return this.domNode}set(e,t){this.didEverRender&&this.keybinding===e&&Ib.areSame(this.matches,t)||(this.keybinding=e,this.matches=t,this.render())}render(){if(this.clear(),this.keybinding){const e=this.keybinding.getChords();e[0]&&this.renderChord(this.domNode,e[0],this.matches?this.matches.firstPart:null);for(let i=1;i<e.length;i++)X(this.domNode,n0("span.monaco-keybinding-key-chord-separator",void 0," ")),this.renderChord(this.domNode,e[i],this.matches?this.matches.chordPart:null);const t=this.options.disableTitle??!1?void 0:this.keybinding.getAriaLabel()||void 0;this.hover.update(t),this.domNode.setAttribute("aria-label",t||"")}else this.options&&this.options.renderUnboundKeybindings&&this.renderUnbound(this.domNode);this.didEverRender=!0}clear(){pn(this.domNode),this.keyElements.clear()}renderChord(e,t,i){const n=Q5.modifierLabels[this.os];t.ctrlKey&&this.renderKey(e,n.ctrlKey,!!(i!=null&&i.ctrlKey),n.separator),t.shiftKey&&this.renderKey(e,n.shiftKey,!!(i!=null&&i.shiftKey),n.separator),t.altKey&&this.renderKey(e,n.altKey,!!(i!=null&&i.altKey),n.separator),t.metaKey&&this.renderKey(e,n.metaKey,!!(i!=null&&i.metaKey),n.separator);const s=t.keyLabel;s&&this.renderKey(e,s,!!(i!=null&&i.keyCode),"")}renderKey(e,t,i,n){X(e,this.createKeyElement(t,i?".highlight":"")),n&&X(e,n0("span.monaco-keybinding-key-separator",void 0,n))}renderUnbound(e){X(e,this.createKeyElement(m("unbound","Unbound")))}createKeyElement(e,t=""){const i=n0("span.monaco-keybinding-key"+t,void 0,e);return this.keyElements.add(i),this.options.keybindingLabelBackground&&(i.style.backgroundColor=this.options.keybindingLabelBackground),this.options.keybindingLabelBorder&&(i.style.borderColor=this.options.keybindingLabelBorder),this.options.keybindingLabelBottomBorder&&(i.style.borderBottomColor=this.options.keybindingLabelBottomBorder),this.options.keybindingLabelShadow&&(i.style.boxShadow=`inset 0 -1px 0 ${this.options.keybindingLabelShadow}`),i}static areSame(e,t){return e===t||!e&&!t?!0:!!e&&!!t&&oo(e.firstPart,t.firstPart)&&oo(e.chordPart,t.chordPart)}}const Q8=new Oa(()=>{const o=new Intl.Collator(void 0,{numeric:!0,sensitivity:"base"});return{collator:o,collatorIsNumeric:o.resolvedOptions().numeric}});function Gfe(o,e,t=!1){const i=o||"",n=e||"",s=Q8.value.collator.compare(i,n);return Q8.value.collatorIsNumeric&&s===0&&i!==n?i<n?-1:1:s}function Zfe(o,e,t){const i=o.toLowerCase(),n=e.toLowerCase(),s=Yfe(o,e,t);if(s)return s;const r=i.endsWith(t),a=n.endsWith(t);if(r!==a)return r?-1:1;const l=Gfe(i,n);return l!==0?l:i.localeCompare(n)}function Yfe(o,e,t){const i=o.toLowerCase(),n=e.toLowerCase(),s=i.startsWith(t),r=n.startsWith(t);if(s!==r)return s?-1:1;if(s&&r){if(i.length<n.length)return-1;if(i.length>n.length)return 1}return 0}var gE=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},IA=function(o,e){return function(t,i){e(t,i,o)}},EA;const aa=ne;class qK{constructor(e,t,i){this.index=e,this.hasCheckbox=t,this._hidden=!1,this._init=new Oa(()=>{const n=i.label??"",s=xC(n).text.trim(),r=i.ariaLabel||[n,this.saneDescription,this.saneDetail].map(a=>sle(a)).filter(a=>!!a).join(", ");return{saneLabel:n,saneSortLabel:s,saneAriaLabel:r}}),this._saneDescription=i.description,this._saneTooltip=i.tooltip}get saneLabel(){return this._init.value.saneLabel}get saneSortLabel(){return this._init.value.saneSortLabel}get saneAriaLabel(){return this._init.value.saneAriaLabel}get element(){return this._element}set element(e){this._element=e}get hidden(){return this._hidden}set hidden(e){this._hidden=e}get saneDescription(){return this._saneDescription}set saneDescription(e){this._saneDescription=e}get saneDetail(){return this._saneDetail}set saneDetail(e){this._saneDetail=e}get saneTooltip(){return this._saneTooltip}set saneTooltip(e){this._saneTooltip=e}get labelHighlights(){return this._labelHighlights}set labelHighlights(e){this._labelHighlights=e}get descriptionHighlights(){return this._descriptionHighlights}set descriptionHighlights(e){this._descriptionHighlights=e}get detailHighlights(){return this._detailHighlights}set detailHighlights(e){this._detailHighlights=e}}class wn extends qK{constructor(e,t,i,n,s,r){var a,l,c;super(e,t,s),this.fireButtonTriggered=i,this._onChecked=n,this.item=s,this._separator=r,this._checked=!1,this.onChecked=t?oe.map(oe.filter(this._onChecked.event,d=>d.element===this),d=>d.checked):oe.None,this._saneDetail=s.detail,this._labelHighlights=(a=s.highlights)==null?void 0:a.label,this._descriptionHighlights=(l=s.highlights)==null?void 0:l.description,this._detailHighlights=(c=s.highlights)==null?void 0:c.detail}get separator(){return this._separator}set separator(e){this._separator=e}get checked(){return this._checked}set checked(e){e!==this._checked&&(this._checked=e,this._onChecked.fire({element:this,checked:e}))}get checkboxDisabled(){return!!this.item.disabled}}var _l;(function(o){o[o.NONE=0]="NONE",o[o.MOUSE_HOVER=1]="MOUSE_HOVER",o[o.ACTIVE_ITEM=2]="ACTIVE_ITEM"})(_l||(_l={}));class Mu extends qK{constructor(e,t,i){super(e,!1,i),this.fireSeparatorButtonTriggered=t,this.separator=i,this.children=new Array,this.focusInsideSeparator=_l.NONE}}class Xfe{getHeight(e){return e instanceof Mu?30:e.saneDetail?44:22}getTemplateId(e){return e instanceof wn?GL.ID:ZL.ID}}class Qfe{getWidgetAriaLabel(){return m("quickInput","Quick Input")}getAriaLabel(e){var t;return(t=e.separator)!=null&&t.label?`${e.saneAriaLabel}, ${e.separator.label}`:e.saneAriaLabel}getWidgetRole(){return"listbox"}getRole(e){return e.hasCheckbox?"checkbox":"option"}isChecked(e){if(!(!e.hasCheckbox||!(e instanceof wn)))return{get value(){return e.checked},onDidChange:t=>e.onChecked(()=>t())}}}class GK{constructor(e){this.hoverDelegate=e}renderTemplate(e){const t=Object.create(null);t.toDisposeElement=new K,t.toDisposeTemplate=new K,t.entry=X(e,aa(".quick-input-list-entry"));const i=X(t.entry,aa("label.quick-input-list-label"));t.toDisposeTemplate.add(hi(i,ie.CLICK,c=>{t.checkbox.offsetParent||c.preventDefault()})),t.checkbox=X(i,aa("input.quick-input-list-checkbox")),t.checkbox.type="checkbox";const n=X(i,aa(".quick-input-list-rows")),s=X(n,aa(".quick-input-list-row")),r=X(n,aa(".quick-input-list-row"));t.label=new qL(s,{supportHighlights:!0,supportDescriptionHighlights:!0,supportIcons:!0,hoverDelegate:this.hoverDelegate}),t.toDisposeTemplate.add(t.label),t.icon=c5(t.label.element,aa(".quick-input-list-icon"));const a=X(s,aa(".quick-input-list-entry-keybinding"));t.keybinding=new Ib(a,so),t.toDisposeTemplate.add(t.keybinding);const l=X(r,aa(".quick-input-list-label-meta"));return t.detail=new qL(l,{supportHighlights:!0,supportIcons:!0,hoverDelegate:this.hoverDelegate}),t.toDisposeTemplate.add(t.detail),t.separator=X(t.entry,aa(".quick-input-list-separator")),t.actionBar=new zo(t.entry,this.hoverDelegate?{hoverDelegate:this.hoverDelegate}:void 0),t.actionBar.domNode.classList.add("quick-input-list-entry-action-bar"),t.toDisposeTemplate.add(t.actionBar),t}disposeTemplate(e){e.toDisposeElement.dispose(),e.toDisposeTemplate.dispose()}disposeElement(e,t,i){i.toDisposeElement.clear(),i.actionBar.clear()}}var Eg;let GL=(Eg=class extends GK{constructor(e,t){super(e),this.themeService=t,this._itemsWithSeparatorsFrequency=new Map}get templateId(){return EA.ID}renderTemplate(e){const t=super.renderTemplate(e);return t.toDisposeTemplate.add(hi(t.checkbox,ie.CHANGE,i=>{t.element.checked=t.checkbox.checked})),t}renderElement(e,t,i){var u;const n=e.element;i.element=n,n.element=i.entry??void 0;const s=n.item;i.checkbox.checked=n.checked,i.toDisposeElement.add(n.onChecked(g=>i.checkbox.checked=g)),i.checkbox.disabled=n.checkboxDisabled;const{labelHighlights:r,descriptionHighlights:a,detailHighlights:l}=n;if(s.iconPath){const g=N_(this.themeService.getColorTheme().type)?s.iconPath.dark:s.iconPath.light??s.iconPath.dark,f=Se.revive(g);i.icon.className="quick-input-list-icon",i.icon.style.backgroundImage=td(f)}else i.icon.style.backgroundImage="",i.icon.className=s.iconClass?`quick-input-list-icon ${s.iconClass}`:"";let c;!n.saneTooltip&&n.saneDescription&&(c={markdown:{value:n.saneDescription,supportThemeIcons:!0},markdownNotSupportedFallback:n.saneDescription});const d={matches:r||[],descriptionTitle:c,descriptionMatches:a||[],labelEscapeNewLines:!0};if(d.extraClasses=s.iconClasses,d.italic=s.italic,d.strikethrough=s.strikethrough,i.entry.classList.remove("quick-input-list-separator-as-item"),i.label.setLabel(n.saneLabel,n.saneDescription,d),i.keybinding.set(s.keybinding),n.saneDetail){let g;n.saneTooltip||(g={markdown:{value:n.saneDetail,supportThemeIcons:!0},markdownNotSupportedFallback:n.saneDetail}),i.detail.element.style.display="",i.detail.setLabel(n.saneDetail,void 0,{matches:l,title:g,labelEscapeNewLines:!0})}else i.detail.element.style.display="none";(u=n.separator)!=null&&u.label?(i.separator.textContent=n.separator.label,i.separator.style.display="",this.addItemWithSeparator(n)):i.separator.style.display="none",i.entry.classList.toggle("quick-input-list-separator-border",!!n.separator);const h=s.buttons;h&&h.length?(i.actionBar.push(h.map((g,f)=>ov(g,`id-${f}`,()=>n.fireButtonTriggered({button:g,item:n.item}))),{icon:!0,label:!1}),i.entry.classList.add("has-actions")):i.entry.classList.remove("has-actions")}disposeElement(e,t,i){this.removeItemWithSeparator(e.element),super.disposeElement(e,t,i)}isItemWithSeparatorVisible(e){return this._itemsWithSeparatorsFrequency.has(e)}addItemWithSeparator(e){this._itemsWithSeparatorsFrequency.set(e,(this._itemsWithSeparatorsFrequency.get(e)||0)+1)}removeItemWithSeparator(e){const t=this._itemsWithSeparatorsFrequency.get(e)||0;t>1?this._itemsWithSeparatorsFrequency.set(e,t-1):this._itemsWithSeparatorsFrequency.delete(e)}},EA=Eg,Eg.ID="quickpickitem",Eg);GL=EA=gE([IA(1,ji)],GL);const Qk=class Qk extends GK{constructor(){super(...arguments),this._visibleSeparatorsFrequency=new Map}get templateId(){return Qk.ID}get visibleSeparators(){return[...this._visibleSeparatorsFrequency.keys()]}isSeparatorVisible(e){return this._visibleSeparatorsFrequency.has(e)}renderTemplate(e){const t=super.renderTemplate(e);return t.checkbox.style.display="none",t}renderElement(e,t,i){const n=e.element;i.element=n,n.element=i.entry??void 0,n.element.classList.toggle("focus-inside",!!n.focusInsideSeparator);const s=n.separator,{labelHighlights:r,descriptionHighlights:a,detailHighlights:l}=n;i.icon.style.backgroundImage="",i.icon.className="";let c;!n.saneTooltip&&n.saneDescription&&(c={markdown:{value:n.saneDescription,supportThemeIcons:!0},markdownNotSupportedFallback:n.saneDescription});const d={matches:r||[],descriptionTitle:c,descriptionMatches:a||[],labelEscapeNewLines:!0};if(i.entry.classList.add("quick-input-list-separator-as-item"),i.label.setLabel(n.saneLabel,n.saneDescription,d),n.saneDetail){let u;n.saneTooltip||(u={markdown:{value:n.saneDetail,supportThemeIcons:!0},markdownNotSupportedFallback:n.saneDetail}),i.detail.element.style.display="",i.detail.setLabel(n.saneDetail,void 0,{matches:l,title:u,labelEscapeNewLines:!0})}else i.detail.element.style.display="none";i.separator.style.display="none",i.entry.classList.add("quick-input-list-separator-border");const h=s.buttons;h&&h.length?(i.actionBar.push(h.map((u,g)=>ov(u,`id-${g}`,()=>n.fireSeparatorButtonTriggered({button:u,separator:n.separator}))),{icon:!0,label:!1}),i.entry.classList.add("has-actions")):i.entry.classList.remove("has-actions"),this.addSeparator(n)}disposeElement(e,t,i){var n;this.removeSeparator(e.element),this.isSeparatorVisible(e.element)||(n=e.element.element)==null||n.classList.remove("focus-inside"),super.disposeElement(e,t,i)}addSeparator(e){this._visibleSeparatorsFrequency.set(e,(this._visibleSeparatorsFrequency.get(e)||0)+1)}removeSeparator(e){const t=this._visibleSeparatorsFrequency.get(e)||0;t>1?this._visibleSeparatorsFrequency.set(e,t-1):this._visibleSeparatorsFrequency.delete(e)}};Qk.ID="quickpickseparator";let ZL=Qk,Ww=class extends F{constructor(e,t,i,n,s,r){super(),this.parent=e,this.hoverDelegate=t,this.linkOpenerDelegate=i,this.accessibilityService=r,this._onKeyDown=new P,this._onLeave=new P,this.onLeave=this._onLeave.event,this._visibleCountObservable=Ke("VisibleCount",0),this.onChangedVisibleCount=oe.fromObservable(this._visibleCountObservable,this._store),this._allVisibleCheckedObservable=Ke("AllVisibleChecked",!1),this.onChangedAllVisibleChecked=oe.fromObservable(this._allVisibleCheckedObservable,this._store),this._checkedCountObservable=Ke("CheckedCount",0),this.onChangedCheckedCount=oe.fromObservable(this._checkedCountObservable,this._store),this._checkedElementsObservable=z2({equalsFn:ii},new Array),this.onChangedCheckedElements=oe.fromObservable(this._checkedElementsObservable,this._store),this._onButtonTriggered=new P,this.onButtonTriggered=this._onButtonTriggered.event,this._onSeparatorButtonTriggered=new P,this.onSeparatorButtonTriggered=this._onSeparatorButtonTriggered.event,this._elementChecked=new P,this._elementCheckedEventBufferer=new R1,this._hasCheckboxes=!1,this._inputElements=new Array,this._elementTree=new Array,this._itemElements=new Array,this._elementDisposable=this._register(new K),this._matchOnDescription=!1,this._matchOnDetail=!1,this._matchOnLabel=!0,this._matchOnLabelMode="fuzzy",this._sortByLabel=!0,this._shouldLoop=!0,this._container=X(this.parent,aa(".quick-input-list")),this._separatorRenderer=new ZL(t),this._itemRenderer=s.createInstance(GL,t),this._tree=this._register(s.createInstance(kA,"QuickInput",this._container,new Xfe,[this._itemRenderer,this._separatorRenderer],{filter:{filter(a){return a.hidden?0:a instanceof Mu?2:1}},sorter:{compare:(a,l)=>{if(!this.sortByLabel||!this._lastQueryString)return 0;const c=this._lastQueryString.toLowerCase();return eme(a,l,c)}},accessibilityProvider:new Qfe,setRowLineHeight:!1,multipleSelectionSupport:!1,hideTwistiesOfChildlessElements:!0,renderIndentGuides:U_.None,findWidgetEnabled:!1,indent:0,horizontalScrolling:!1,allowNonCollapsibleParents:!0,alwaysConsumeMouseWheel:!0})),this._tree.getHTMLElement().id=n,this._registerListeners()}get onDidChangeFocus(){return oe.map(this._tree.onDidChangeFocus,e=>e.elements.filter(t=>t instanceof wn).map(t=>t.item),this._store)}get onDidChangeSelection(){return oe.map(this._tree.onDidChangeSelection,e=>({items:e.elements.filter(t=>t instanceof wn).map(t=>t.item),event:e.browserEvent}),this._store)}get displayed(){return this._container.style.display!=="none"}set displayed(e){this._container.style.display=e?"":"none"}get scrollTop(){return this._tree.scrollTop}set scrollTop(e){this._tree.scrollTop=e}get ariaLabel(){return this._tree.ariaLabel}set ariaLabel(e){this._tree.ariaLabel=e??""}set enabled(e){this._tree.getHTMLElement().style.pointerEvents=e?"":"none"}get matchOnDescription(){return this._matchOnDescription}set matchOnDescription(e){this._matchOnDescription=e}get matchOnDetail(){return this._matchOnDetail}set matchOnDetail(e){this._matchOnDetail=e}get matchOnLabel(){return this._matchOnLabel}set matchOnLabel(e){this._matchOnLabel=e}get matchOnLabelMode(){return this._matchOnLabelMode}set matchOnLabelMode(e){this._matchOnLabelMode=e}get sortByLabel(){return this._sortByLabel}set sortByLabel(e){this._sortByLabel=e}get shouldLoop(){return this._shouldLoop}set shouldLoop(e){this._shouldLoop=e}_registerListeners(){this._registerOnKeyDown(),this._registerOnContainerClick(),this._registerOnMouseMiddleClick(),this._registerOnTreeModelChanged(),this._registerOnElementChecked(),this._registerOnContextMenu(),this._registerHoverListeners(),this._registerSelectionChangeListener(),this._registerSeparatorActionShowingListeners()}_registerOnKeyDown(){this._register(this._tree.onKeyDown(e=>{const t=new Pt(e);switch(t.keyCode){case 10:this.toggleCheckbox();break}this._onKeyDown.fire(t)}))}_registerOnContainerClick(){this._register(U(this._container,ie.CLICK,e=>{(e.x||e.y)&&this._onLeave.fire()}))}_registerOnMouseMiddleClick(){this._register(U(this._container,ie.AUXCLICK,e=>{e.button===1&&this._onLeave.fire()}))}_registerOnTreeModelChanged(){this._register(this._tree.onDidChangeModel(()=>{const e=this._itemElements.filter(t=>!t.hidden).length;this._visibleCountObservable.set(e,void 0),this._hasCheckboxes&&this._updateCheckedObservables()}))}_registerOnElementChecked(){this._register(this._elementCheckedEventBufferer.wrapEvent(this._elementChecked.event,(e,t)=>t)(e=>this._updateCheckedObservables()))}_registerOnContextMenu(){this._register(this._tree.onContextMenu(e=>{e.element&&(e.browserEvent.preventDefault(),this._tree.setSelection([e.element]))}))}_registerHoverListeners(){const e=this._register(new dU(this.hoverDelegate.delay));this._register(this._tree.onMouseOver(async t=>{var i;if(d7(t.browserEvent.target)){e.cancel();return}if(!(!d7(t.browserEvent.relatedTarget)&&Zi(t.browserEvent.relatedTarget,(i=t.element)==null?void 0:i.element)))try{await e.trigger(async()=>{t.element instanceof wn&&this.showHover(t.element)})}catch(n){if(!yr(n))throw n}})),this._register(this._tree.onMouseOut(t=>{var i;Zi(t.browserEvent.relatedTarget,(i=t.element)==null?void 0:i.element)||e.cancel()}))}_registerSeparatorActionShowingListeners(){this._register(this._tree.onDidChangeFocus(e=>{const t=e.elements[0]?this._tree.getParentElement(e.elements[0]):null;for(const i of this._separatorRenderer.visibleSeparators){const n=i===t;!!(i.focusInsideSeparator&_l.ACTIVE_ITEM)!==n&&(n?i.focusInsideSeparator|=_l.ACTIVE_ITEM:i.focusInsideSeparator&=~_l.ACTIVE_ITEM,this._tree.rerender(i))}})),this._register(this._tree.onMouseOver(e=>{const t=e.element?this._tree.getParentElement(e.element):null;for(const i of this._separatorRenderer.visibleSeparators){if(i!==t)continue;!!(i.focusInsideSeparator&_l.MOUSE_HOVER)||(i.focusInsideSeparator|=_l.MOUSE_HOVER,this._tree.rerender(i))}})),this._register(this._tree.onMouseOut(e=>{const t=e.element?this._tree.getParentElement(e.element):null;for(const i of this._separatorRenderer.visibleSeparators){if(i!==t)continue;!!(i.focusInsideSeparator&_l.MOUSE_HOVER)&&(i.focusInsideSeparator&=~_l.MOUSE_HOVER,this._tree.rerender(i))}}))}_registerSelectionChangeListener(){this._register(this._tree.onDidChangeSelection(e=>{const t=e.elements.filter(i=>i instanceof wn);t.length!==e.elements.length&&(e.elements.length===1&&e.elements[0]instanceof Mu&&(this._tree.setFocus([e.elements[0].children[0]]),this._tree.reveal(e.elements[0],0)),this._tree.setSelection(t))}))}setAllVisibleChecked(e){this._elementCheckedEventBufferer.bufferEvents(()=>{this._itemElements.forEach(t=>{!t.hidden&&!t.checkboxDisabled&&(t.checked=e)})})}setElements(e){this._elementDisposable.clear(),this._lastQueryString=void 0,this._inputElements=e,this._hasCheckboxes=this.parent.classList.contains("show-checkboxes");let t;this._itemElements=new Array,this._elementTree=e.reduce((i,n,s)=>{let r;if(n.type==="separator"){if(!n.buttons)return i;t=new Mu(s,a=>this._onSeparatorButtonTriggered.fire(a),n),r=t}else{const a=s>0?e[s-1]:void 0;let l;a&&a.type==="separator"&&!a.buttons&&(t=void 0,l=a);const c=new wn(s,this._hasCheckboxes,d=>this._onButtonTriggered.fire(d),this._elementChecked,n,l);if(this._itemElements.push(c),t)return t.children.push(c),i;r=c}return i.push(r),i},new Array),this._setElementsToTree(this._elementTree),this.accessibilityService.isScreenReaderOptimized()&&setTimeout(()=>{const i=this._tree.getHTMLElement().querySelector(".monaco-list-row.focused"),n=i==null?void 0:i.parentNode;if(i&&n){const s=i.nextSibling;i.remove(),n.insertBefore(i,s)}},0)}setFocusedElements(e){const t=e.map(i=>this._itemElements.find(n=>n.item===i)).filter(i=>!!i).filter(i=>!i.hidden);if(this._tree.setFocus(t),e.length>0){const i=this._tree.getFocus()[0];i&&this._tree.reveal(i)}}getActiveDescendant(){return this._tree.getHTMLElement().getAttribute("aria-activedescendant")}setSelectedElements(e){const t=e.map(i=>this._itemElements.find(n=>n.item===i)).filter(i=>!!i);this._tree.setSelection(t)}getCheckedElements(){return this._itemElements.filter(e=>e.checked).map(e=>e.item)}setCheckedElements(e){this._elementCheckedEventBufferer.bufferEvents(()=>{const t=new Set;for(const i of e)t.add(i);for(const i of this._itemElements)i.checked=t.has(i.item)})}focus(e){var t;if(this._itemElements.length)switch(e===zt.Second&&this._itemElements.length<2&&(e=zt.First),e){case zt.First:this._tree.scrollTop=0,this._tree.focusFirst(void 0,i=>i.element instanceof wn);break;case zt.Second:{this._tree.scrollTop=0;let i=!1;this._tree.focusFirst(void 0,n=>n.element instanceof wn?i?!0:(i=!i,!1):!1);break}case zt.Last:this._tree.scrollTop=this._tree.scrollHeight,this._tree.focusLast(void 0,i=>i.element instanceof wn);break;case zt.Next:{const i=this._tree.getFocus();this._tree.focusNext(void 0,this._shouldLoop,void 0,s=>s.element instanceof wn?(this._tree.reveal(s.element),!0):!1);const n=this._tree.getFocus();i.length&&i[0]===n[0]&&i[0]===this._itemElements[this._itemElements.length-1]&&this._onLeave.fire();break}case zt.Previous:{const i=this._tree.getFocus();this._tree.focusPrevious(void 0,this._shouldLoop,void 0,s=>{if(!(s.element instanceof wn))return!1;const r=this._tree.getParentElement(s.element);return r===null||r.children[0]!==s.element?this._tree.reveal(s.element):this._tree.reveal(r),!0});const n=this._tree.getFocus();i.length&&i[0]===n[0]&&i[0]===this._itemElements[0]&&this._onLeave.fire();break}case zt.NextPage:this._tree.focusNextPage(void 0,i=>i.element instanceof wn?(this._tree.reveal(i.element),!0):!1);break;case zt.PreviousPage:this._tree.focusPreviousPage(void 0,i=>{if(!(i.element instanceof wn))return!1;const n=this._tree.getParentElement(i.element);return n===null||n.children[0]!==i.element?this._tree.reveal(i.element):this._tree.reveal(n),!0});break;case zt.NextSeparator:{let i=!1;const n=this._tree.getFocus()[0];this._tree.focusNext(void 0,!0,void 0,r=>{if(i)return!0;if(r.element instanceof Mu)i=!0,this._separatorRenderer.isSeparatorVisible(r.element)?this._tree.reveal(r.element.children[0]):this._tree.reveal(r.element,0);else if(r.element instanceof wn){if(r.element.separator)return this._itemRenderer.isItemWithSeparatorVisible(r.element)?this._tree.reveal(r.element):this._tree.reveal(r.element,0),!0;if(r.element===this._elementTree[0])return this._tree.reveal(r.element,0),!0}return!1});const s=this._tree.getFocus()[0];n===s&&(this._tree.scrollTop=this._tree.scrollHeight,this._tree.focusLast(void 0,r=>r.element instanceof wn));break}case zt.PreviousSeparator:{let i,n=!!((t=this._tree.getFocus()[0])!=null&&t.separator);this._tree.focusPrevious(void 0,!0,void 0,s=>{if(s.element instanceof Mu)n?i||(this._separatorRenderer.isSeparatorVisible(s.element)?this._tree.reveal(s.element):this._tree.reveal(s.element,0),i=s.element.children[0]):n=!0;else if(s.element instanceof wn&&!i){if(s.element.separator)this._itemRenderer.isItemWithSeparatorVisible(s.element)?this._tree.reveal(s.element):this._tree.reveal(s.element,0),i=s.element;else if(s.element===this._elementTree[0])return this._tree.reveal(s.element,0),!0}return!1}),i&&this._tree.setFocus([i]);break}}}clearFocus(){this._tree.setFocus([])}domFocus(){this._tree.domFocus()}layout(e){this._tree.getHTMLElement().style.maxHeight=e?`${Math.floor(e/44)*44+6}px`:"",this._tree.layout()}filter(e){if(this._lastQueryString=e,!(this._sortByLabel||this._matchOnLabel||this._matchOnDescription||this._matchOnDetail))return this._tree.layout(),!1;const t=e;if(e=e.trim(),!e||!(this.matchOnLabel||this.matchOnDescription||this.matchOnDetail))this._itemElements.forEach(i=>{i.labelHighlights=void 0,i.descriptionHighlights=void 0,i.detailHighlights=void 0,i.hidden=!1;const n=i.index&&this._inputElements[i.index-1];i.item&&(i.separator=n&&n.type==="separator"&&!n.buttons?n:void 0)});else{let i;this._itemElements.forEach(n=>{let s;this.matchOnLabelMode==="fuzzy"?s=this.matchOnLabel?$N(e,xC(n.saneLabel))??void 0:void 0:s=this.matchOnLabel?Jfe(t,xC(n.saneLabel))??void 0:void 0;const r=this.matchOnDescription?$N(e,xC(n.saneDescription||""))??void 0:void 0,a=this.matchOnDetail?$N(e,xC(n.saneDetail||""))??void 0:void 0;if(s||r||a?(n.labelHighlights=s,n.descriptionHighlights=r,n.detailHighlights=a,n.hidden=!1):(n.labelHighlights=void 0,n.descriptionHighlights=void 0,n.detailHighlights=void 0,n.hidden=n.item?!n.item.alwaysShow:!0),n.item?n.separator=void 0:n.separator&&(n.hidden=!0),!this.sortByLabel){const l=n.index&&this._inputElements[n.index-1]||void 0;(l==null?void 0:l.type)==="separator"&&!l.buttons&&(i=l),i&&!n.hidden&&(n.separator=i,i=void 0)}})}return this._setElementsToTree(this._sortByLabel&&e?this._itemElements:this._elementTree),this._tree.layout(),!0}toggleCheckbox(){this._elementCheckedEventBufferer.bufferEvents(()=>{const e=this._tree.getFocus().filter(i=>i instanceof wn),t=this._allVisibleChecked(e);for(const i of e)i.checkboxDisabled||(i.checked=!t)})}style(e){this._tree.style(e)}toggleHover(){const e=this._tree.getFocus()[0];if(!(e!=null&&e.saneTooltip)||!(e instanceof wn))return;if(this._lastHover&&!this._lastHover.isDisposed){this._lastHover.dispose();return}this.showHover(e);const t=new K;t.add(this._tree.onDidChangeFocus(i=>{i.elements[0]instanceof wn&&this.showHover(i.elements[0])})),this._lastHover&&t.add(this._lastHover),this._elementDisposable.add(t)}_setElementsToTree(e){const t=new Array;for(const i of e)i instanceof Mu?t.push({element:i,collapsible:!1,collapsed:!1,children:i.children.map(n=>({element:n,collapsible:!1,collapsed:!1}))}):t.push({element:i,collapsible:!1,collapsed:!1});this._tree.setChildren(null,t)}_allVisibleChecked(e,t=!0){for(let i=0,n=e.length;i<n;i++){const s=e[i];if(!s.hidden)if(s.checked)t=!0;else return!1}return t}_updateCheckedObservables(){ai(e=>{this._allVisibleCheckedObservable.set(this._allVisibleChecked(this._itemElements,!1),e);const t=this._itemElements.filter(i=>i.checked).length;this._checkedCountObservable.set(t,e),this._checkedElementsObservable.set(this.getCheckedElements(),e)})}showHover(e){var t,i,n;this._lastHover&&!this._lastHover.isDisposed&&((i=(t=this.hoverDelegate).onDidHideHover)==null||i.call(t),(n=this._lastHover)==null||n.dispose()),!(!e.element||!e.saneTooltip)&&(this._lastHover=this.hoverDelegate.showHover({content:e.saneTooltip,target:e.element,linkHandler:s=>{this.linkOpenerDelegate(s)},appearance:{showPointer:!0},container:this._container,position:{hoverPosition:1}},!1))}};gE([Ni],Ww.prototype,"onDidChangeFocus",null);gE([Ni],Ww.prototype,"onDidChangeSelection",null);Ww=gE([IA(4,we),IA(5,po)],Ww);function Jfe(o,e){const{text:t,iconOffsets:i}=e;if(!i||i.length===0)return J8(o,t);const n=M1(t," "),s=t.length-n.length,r=J8(o,n);if(r)for(const a of r){const l=i[a.start+s]+s;a.start+=l,a.end+=l}return r}function J8(o,e){const t=e.toLowerCase().indexOf(o.toLowerCase());return t!==-1?[{start:t,end:t+o.length}]:null}function eme(o,e,t){const i=o.labelHighlights||[],n=e.labelHighlights||[];return i.length&&!n.length?-1:!i.length&&n.length?1:i.length===0&&n.length===0?0:Zfe(o.saneSortLabel,e.saneSortLabel,t)}const ZK={weight:200,when:G.and(G.equals(CK,"quickPick"),Ige),metadata:{description:m("quickPick","Used while in the context of the quick pick. If you change one keybinding for this command, you should change all of the other keybindings (modifier variants) of this command as well.")}};function Qs(o,e={}){bs.registerCommandAndKeybindingRule({...ZK,...o,secondary:tme(o.primary,o.secondary??[],e)})}const YL=Je?256:2048;function tme(o,e,t={}){return t.withAltMod&&e.push(512+o),t.withCtrlMod&&(e.push(YL+o),t.withAltMod&&e.push(512+YL+o)),t.withCmdMod&&Je&&(e.push(2048+o),t.withCtrlMod&&e.push(2304+o),t.withAltMod&&(e.push(2560+o),t.withCtrlMod&&e.push(2816+o))),e}function Io(o,e){return t=>{const i=t.get(xr).currentQuickInput;if(i)return e&&i.quickNavigate?i.focus(e):i.focus(o)}}Qs({id:"quickInput.pageNext",primary:12,handler:Io(zt.NextPage)},{withAltMod:!0,withCtrlMod:!0,withCmdMod:!0});Qs({id:"quickInput.pagePrevious",primary:11,handler:Io(zt.PreviousPage)},{withAltMod:!0,withCtrlMod:!0,withCmdMod:!0});Qs({id:"quickInput.first",primary:YL+14,handler:Io(zt.First)},{withAltMod:!0,withCmdMod:!0});Qs({id:"quickInput.last",primary:YL+13,handler:Io(zt.Last)},{withAltMod:!0,withCmdMod:!0});Qs({id:"quickInput.next",primary:18,handler:Io(zt.Next)},{withCtrlMod:!0});Qs({id:"quickInput.previous",primary:16,handler:Io(zt.Previous)},{withCtrlMod:!0});const eH=m("quickInput.nextSeparatorWithQuickAccessFallback","If we're in quick access mode, this will navigate to the next item. If we are not in quick access mode, this will navigate to the next separator."),tH=m("quickInput.previousSeparatorWithQuickAccessFallback","If we're in quick access mode, this will navigate to the previous item. If we are not in quick access mode, this will navigate to the previous separator.");Je?(Qs({id:"quickInput.nextSeparatorWithQuickAccessFallback",primary:2066,handler:Io(zt.NextSeparator,zt.Next),metadata:{description:eH}}),Qs({id:"quickInput.nextSeparator",primary:2578,secondary:[2322],handler:Io(zt.NextSeparator)},{withCtrlMod:!0}),Qs({id:"quickInput.previousSeparatorWithQuickAccessFallback",primary:2064,handler:Io(zt.PreviousSeparator,zt.Previous),metadata:{description:tH}}),Qs({id:"quickInput.previousSeparator",primary:2576,secondary:[2320],handler:Io(zt.PreviousSeparator)},{withCtrlMod:!0})):(Qs({id:"quickInput.nextSeparatorWithQuickAccessFallback",primary:530,handler:Io(zt.NextSeparator,zt.Next),metadata:{description:eH}}),Qs({id:"quickInput.nextSeparator",primary:2578,handler:Io(zt.NextSeparator)}),Qs({id:"quickInput.previousSeparatorWithQuickAccessFallback",primary:528,handler:Io(zt.PreviousSeparator,zt.Previous),metadata:{description:tH}}),Qs({id:"quickInput.previousSeparator",primary:2576,handler:Io(zt.PreviousSeparator)}));Qs({id:"quickInput.acceptInBackground",when:G.and(ZK.when,G.or(FK.negate(),Tge)),primary:17,weight:250,handler:o=>{const e=o.get(xr).currentQuickInput;e==null||e.accept(!0)}},{withAltMod:!0,withCtrlMod:!0,withCmdMod:!0});var ime=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},gT=function(o,e){return function(t,i){e(t,i,o)}},NA;const Gs=ne;var Ng;let TA=(Ng=class extends F{get currentQuickInput(){return this.controller??void 0}get container(){return this._container}constructor(e,t,i,n){super(),this.options=e,this.layoutService=t,this.instantiationService=i,this.contextKeyService=n,this.enabled=!0,this.onDidAcceptEmitter=this._register(new P),this.onDidCustomEmitter=this._register(new P),this.onDidTriggerButtonEmitter=this._register(new P),this.keyMods={ctrlCmd:!1,alt:!1},this.controller=null,this.onShowEmitter=this._register(new P),this.onShow=this.onShowEmitter.event,this.onHideEmitter=this._register(new P),this.onHide=this.onHideEmitter.event,this.inQuickInputContext=Dge.bindTo(this.contextKeyService),this.quickInputTypeContext=Ege.bindTo(this.contextKeyService),this.endOfQuickInputBoxContext=Nge.bindTo(this.contextKeyService),this.idPrefix=e.idPrefix,this._container=e.container,this.styles=e.styles,this._register(oe.runAndSubscribe(CI,({window:s,disposables:r})=>this.registerKeyModsListeners(s,r),{window:Rt,disposables:this._store})),this._register(Mte(s=>{this.ui&&xe(this.ui.container)===s&&(this.reparentUI(this.layoutService.mainContainer),this.layout(this.layoutService.mainContainerDimension,this.layoutService.mainContainerOffset.quickPickTop))}))}registerKeyModsListeners(e,t){const i=n=>{this.keyMods.ctrlCmd=n.ctrlKey||n.metaKey,this.keyMods.alt=n.altKey};for(const n of[ie.KEY_DOWN,ie.KEY_UP,ie.MOUSE_DOWN])t.add(U(e,n,i,!0))}getUI(e){if(this.ui)return e&&xe(this._container)!==xe(this.layoutService.activeContainer)&&(this.reparentUI(this.layoutService.activeContainer),this.layout(this.layoutService.activeContainerDimension,this.layoutService.activeContainerOffset.quickPickTop)),this.ui;const t=X(this._container,Gs(".quick-input-widget.show-file-icons"));t.tabIndex=-1,t.style.display="none";const i=$o(t),n=X(t,Gs(".quick-input-titlebar")),s=this._register(new zo(n,{hoverDelegate:this.options.hoverDelegate}));s.domNode.classList.add("quick-input-left-action-bar");const r=X(n,Gs(".quick-input-title")),a=this._register(new zo(n,{hoverDelegate:this.options.hoverDelegate}));a.domNode.classList.add("quick-input-right-action-bar");const l=X(t,Gs(".quick-input-header")),c=X(l,Gs("input.quick-input-check-all"));c.type="checkbox",c.setAttribute("aria-label",m("quickInput.checkAll","Toggle all checkboxes")),this._register(hi(c,ie.CHANGE,H=>{const j=c.checked;V.setAllVisibleChecked(j)})),this._register(U(c,ie.CLICK,H=>{(H.x||H.y)&&g.setFocus()}));const d=X(l,Gs(".quick-input-description")),h=X(l,Gs(".quick-input-and-message")),u=X(h,Gs(".quick-input-filter")),g=this._register(new Vge(u,this.styles.inputBox,this.styles.toggle));g.setAttribute("aria-describedby",`${this.idPrefix}message`);const f=X(u,Gs(".quick-input-visible-count"));f.setAttribute("aria-live","polite"),f.setAttribute("aria-atomic","true");const p=new CA(f,{countFormat:m({key:"quickInput.visibleCount",comment:["This tells the user how many items are shown in a list of items to select from. The items can be anything. Currently not visible, but read by screen readers."]},"{0} Results")},this.styles.countBadge),_=X(u,Gs(".quick-input-count"));_.setAttribute("aria-live","polite");const b=new CA(_,{countFormat:m({key:"quickInput.countSelected",comment:["This tells the user how many items are selected in a list of items to select from. The items can be anything."]},"{0} Selected")},this.styles.countBadge),C=this._register(new zo(l,{hoverDelegate:this.options.hoverDelegate}));C.domNode.classList.add("quick-input-inline-action-bar");const v=X(l,Gs(".quick-input-action")),w=this._register(new FL(v,this.styles.button));w.label=m("ok","OK"),this._register(w.onDidClick(H=>{this.onDidAcceptEmitter.fire()}));const S=X(l,Gs(".quick-input-action")),L=this._register(new FL(S,{...this.styles.button,supportIcons:!0}));L.label=m("custom","Custom"),this._register(L.onDidClick(H=>{this.onDidCustomEmitter.fire()}));const k=X(h,Gs(`#${this.idPrefix}message.quick-input-message`)),D=this._register(new vA(t,this.styles.progressBar));D.getContainer().classList.add("quick-input-progress");const N=X(t,Gs(".quick-input-html-widget"));N.tabIndex=-1;const B=X(t,Gs(".quick-input-description")),W=this.idPrefix+"list",V=this._register(this.instantiationService.createInstance(Ww,t,this.options.hoverDelegate,this.options.linkOpenerDelegate,W));g.setAttribute("aria-controls",W),this._register(V.onDidChangeFocus(()=>{g.setAttribute("aria-activedescendant",V.getActiveDescendant()??"")})),this._register(V.onChangedAllVisibleChecked(H=>{c.checked=H})),this._register(V.onChangedVisibleCount(H=>{p.setCount(H)})),this._register(V.onChangedCheckedCount(H=>{b.setCount(H)})),this._register(V.onLeave(()=>{setTimeout(()=>{this.controller&&(g.setFocus(),this.controller instanceof OL&&this.controller.canSelectMany&&V.clearFocus())},0)}));const $=Fa(t);return this._register($),this._register(U(t,ie.FOCUS,H=>{const j=this.getUI();if(Zi(H.relatedTarget,j.inputContainer)){const J=j.inputBox.isSelectionAtEnd();this.endOfQuickInputBoxContext.get()!==J&&this.endOfQuickInputBoxContext.set(J)}Zi(H.relatedTarget,j.container)||(this.inQuickInputContext.set(!0),this.previousFocusElement=Xi(H.relatedTarget)?H.relatedTarget:void 0)},!0)),this._register($.onDidBlur(()=>{!this.getUI().ignoreFocusOut&&!this.options.ignoreFocusOut()&&this.hide(z_.Blur),this.inQuickInputContext.set(!1),this.endOfQuickInputBoxContext.set(!1),this.previousFocusElement=void 0})),this._register(g.onKeyDown(H=>{const j=this.getUI().inputBox.isSelectionAtEnd();this.endOfQuickInputBoxContext.get()!==j&&this.endOfQuickInputBoxContext.set(j)})),this._register(U(t,ie.FOCUS,H=>{g.setFocus()})),this._register(hi(t,ie.KEY_DOWN,H=>{if(!Zi(H.target,N))switch(H.keyCode){case 3:nt.stop(H,!0),this.enabled&&this.onDidAcceptEmitter.fire();break;case 9:nt.stop(H,!0),this.hide(z_.Gesture);break;case 2:if(!H.altKey&&!H.ctrlKey&&!H.metaKey){const j=[".quick-input-list .monaco-action-bar .always-visible",".quick-input-list-entry:hover .monaco-action-bar",".monaco-list-row.focused .monaco-action-bar"];if(t.classList.contains("show-checkboxes")?j.push("input"):j.push("input[type=text]"),this.getUI().list.displayed&&j.push(".monaco-list"),this.getUI().message&&j.push(".quick-input-message a"),this.getUI().widget){if(Zi(H.target,this.getUI().widget))break;j.push(".quick-input-html-widget")}const J=t.querySelectorAll(j.join(", "));H.shiftKey&&H.target===J[0]?(nt.stop(H,!0),V.clearFocus()):!H.shiftKey&&Zi(H.target,J[J.length-1])&&(nt.stop(H,!0),J[0].focus())}break;case 10:H.ctrlKey&&(nt.stop(H,!0),this.getUI().list.toggleHover());break}})),this.ui={container:t,styleSheet:i,leftActionBar:s,titleBar:n,title:r,description1:B,description2:d,widget:N,rightActionBar:a,inlineActionBar:C,checkAll:c,inputContainer:h,filterContainer:u,inputBox:g,visibleCountContainer:f,visibleCount:p,countContainer:_,count:b,okContainer:v,ok:w,message:k,customButtonContainer:S,customButton:L,list:V,progressBar:D,onDidAccept:this.onDidAcceptEmitter.event,onDidCustom:this.onDidCustomEmitter.event,onDidTriggerButton:this.onDidTriggerButtonEmitter.event,ignoreFocusOut:!1,keyMods:this.keyMods,show:H=>this.show(H),hide:()=>this.hide(),setVisibilities:H=>this.setVisibilities(H),setEnabled:H=>this.setEnabled(H),setContextKey:H=>this.options.setContextKey(H),linkOpenerDelegate:H=>this.options.linkOpenerDelegate(H)},this.updateStyles(),this.ui}reparentUI(e){this.ui&&(this._container=e,X(this._container,this.ui.container))}pick(e,t={},i=ze.None){return new Promise((n,s)=>{let r=d=>{var h;r=n,(h=t.onKeyMods)==null||h.call(t,a.keyMods),n(d)};if(i.isCancellationRequested){r(void 0);return}const a=this.createQuickPick({useSeparators:!0});let l;const c=[a,a.onDidAccept(()=>{if(a.canSelectMany)r(a.selectedItems.slice()),a.hide();else{const d=a.activeItems[0];d&&(r(d),a.hide())}}),a.onDidChangeActive(d=>{const h=d[0];h&&t.onDidFocus&&t.onDidFocus(h)}),a.onDidChangeSelection(d=>{if(!a.canSelectMany){const h=d[0];h&&(r(h),a.hide())}}),a.onDidTriggerItemButton(d=>t.onDidTriggerItemButton&&t.onDidTriggerItemButton({...d,removeItem:()=>{const h=a.items.indexOf(d.item);if(h!==-1){const u=a.items.slice(),g=u.splice(h,1),f=a.activeItems.filter(_=>_!==g[0]),p=a.keepScrollPosition;a.keepScrollPosition=!0,a.items=u,f&&(a.activeItems=f),a.keepScrollPosition=p}}})),a.onDidTriggerSeparatorButton(d=>{var h;return(h=t.onDidTriggerSeparatorButton)==null?void 0:h.call(t,d)}),a.onDidChangeValue(d=>{l&&!d&&(a.activeItems.length!==1||a.activeItems[0]!==l)&&(a.activeItems=[l])}),i.onCancellationRequested(()=>{a.hide()}),a.onDidHide(()=>{Tt(c),r(void 0)})];a.title=t.title,t.value&&(a.value=t.value),a.canSelectMany=!!t.canPickMany,a.placeholder=t.placeHolder,a.ignoreFocusOut=!!t.ignoreFocusLost,a.matchOnDescription=!!t.matchOnDescription,a.matchOnDetail=!!t.matchOnDetail,a.matchOnLabel=t.matchOnLabel===void 0||t.matchOnLabel,a.quickNavigate=t.quickNavigate,a.hideInput=!!t.hideInput,a.contextKey=t.contextKey,a.busy=!0,Promise.all([e,t.activeItem]).then(([d,h])=>{l=h,a.busy=!1,a.items=d,a.canSelectMany&&(a.selectedItems=d.filter(u=>u.type!=="separator"&&u.picked)),l&&(a.activeItems=[l])}),a.show(),Promise.resolve(e).then(void 0,d=>{s(d),a.hide()})})}createQuickPick(e={useSeparators:!1}){const t=this.getUI(!0);return new OL(t)}createInputBox(){const e=this.getUI(!0);return new Rge(e)}show(e){const t=this.getUI(!0);this.onShowEmitter.fire();const i=this.controller;this.controller=e,i==null||i.didHide(),this.setEnabled(!0),t.leftActionBar.clear(),t.title.textContent="",t.description1.textContent="",t.description2.textContent="",_n(t.widget),t.rightActionBar.clear(),t.inlineActionBar.clear(),t.checkAll.checked=!1,t.inputBox.placeholder="",t.inputBox.password=!1,t.inputBox.showDecoration(Ii.Ignore),t.visibleCount.setCount(0),t.count.setCount(0),_n(t.message),t.progressBar.stop(),t.list.setElements([]),t.list.matchOnDescription=!1,t.list.matchOnDetail=!1,t.list.matchOnLabel=!0,t.list.sortByLabel=!0,t.ignoreFocusOut=!1,t.inputBox.toggles=void 0;const n=this.options.backKeybindingLabel();_A.tooltip=n?m("quickInput.backWithKeybinding","Back ({0})",n):m("quickInput.back","Back"),t.container.style.display="",this.updateLayout(),t.inputBox.setFocus(),this.quickInputTypeContext.set(e.type)}isVisible(){return!!this.ui&&this.ui.container.style.display!=="none"}setVisibilities(e){const t=this.getUI();t.title.style.display=e.title?"":"none",t.description1.style.display=e.description&&(e.inputBox||e.checkAll)?"":"none",t.description2.style.display=e.description&&!(e.inputBox||e.checkAll)?"":"none",t.checkAll.style.display=e.checkAll?"":"none",t.inputContainer.style.display=e.inputBox?"":"none",t.filterContainer.style.display=e.inputBox?"":"none",t.visibleCountContainer.style.display=e.visibleCount?"":"none",t.countContainer.style.display=e.count?"":"none",t.okContainer.style.display=e.ok?"":"none",t.customButtonContainer.style.display=e.customButton?"":"none",t.message.style.display=e.message?"":"none",t.progressBar.getContainer().style.display=e.progressBar?"":"none",t.list.displayed=!!e.list,t.container.classList.toggle("show-checkboxes",!!e.checkBox),t.container.classList.toggle("hidden-input",!e.inputBox&&!e.description),this.updateLayout()}setEnabled(e){if(e!==this.enabled){this.enabled=e;for(const t of this.getUI().leftActionBar.viewItems)t.action.enabled=e;for(const t of this.getUI().rightActionBar.viewItems)t.action.enabled=e;this.getUI().checkAll.disabled=!e,this.getUI().inputBox.enabled=e,this.getUI().ok.enabled=e,this.getUI().list.enabled=e}}hide(e){var s;const t=this.controller;if(!t)return;t.willHide(e);const i=(s=this.ui)==null?void 0:s.container,n=i&&!DU(i);if(this.controller=null,this.onHideEmitter.fire(),i&&(i.style.display="none"),!n){let r=this.previousFocusElement;for(;r&&!r.offsetParent;)r=r.parentElement??void 0;r!=null&&r.offsetParent?(r.focus(),this.previousFocusElement=void 0):this.options.returnFocus()}t.didHide(e)}layout(e,t){this.dimension=e,this.titleBarOffset=t,this.updateLayout()}updateLayout(){if(this.ui&&this.isVisible()){this.ui.container.style.top=`${this.titleBarOffset}px`;const e=this.ui.container.style,t=Math.min(this.dimension.width*.62,NA.MAX_WIDTH);e.width=t+"px",e.marginLeft="-"+t/2+"px",this.ui.inputBox.layout(),this.ui.list.layout(this.dimension&&this.dimension.height*.4)}}applyStyles(e){this.styles=e,this.updateStyles()}updateStyles(){if(this.ui){const{quickInputTitleBackground:e,quickInputBackground:t,quickInputForeground:i,widgetBorder:n,widgetShadow:s}=this.styles.widget;this.ui.titleBar.style.backgroundColor=e??"",this.ui.container.style.backgroundColor=t??"",this.ui.container.style.color=i??"",this.ui.container.style.border=n?`1px solid ${n}`:"",this.ui.container.style.boxShadow=s?`0 0 8px 2px ${s}`:"",this.ui.list.style(this.styles.list);const r=[];this.styles.pickerGroup.pickerGroupBorder&&r.push(`.quick-input-list .quick-input-list-entry { border-top-color:  ${this.styles.pickerGroup.pickerGroupBorder}; }`),this.styles.pickerGroup.pickerGroupForeground&&r.push(`.quick-input-list .quick-input-list-separator { color:  ${this.styles.pickerGroup.pickerGroupForeground}; }`),this.styles.pickerGroup.pickerGroupForeground&&r.push(".quick-input-list .quick-input-list-separator-as-item { color: var(--vscode-descriptionForeground); }"),(this.styles.keybindingLabel.keybindingLabelBackground||this.styles.keybindingLabel.keybindingLabelBorder||this.styles.keybindingLabel.keybindingLabelBottomBorder||this.styles.keybindingLabel.keybindingLabelShadow||this.styles.keybindingLabel.keybindingLabelForeground)&&(r.push(".quick-input-list .monaco-keybinding > .monaco-keybinding-key {"),this.styles.keybindingLabel.keybindingLabelBackground&&r.push(`background-color: ${this.styles.keybindingLabel.keybindingLabelBackground};`),this.styles.keybindingLabel.keybindingLabelBorder&&r.push(`border-color: ${this.styles.keybindingLabel.keybindingLabelBorder};`),this.styles.keybindingLabel.keybindingLabelBottomBorder&&r.push(`border-bottom-color: ${this.styles.keybindingLabel.keybindingLabelBottomBorder};`),this.styles.keybindingLabel.keybindingLabelShadow&&r.push(`box-shadow: inset 0 -1px 0 ${this.styles.keybindingLabel.keybindingLabelShadow};`),this.styles.keybindingLabel.keybindingLabelForeground&&r.push(`color: ${this.styles.keybindingLabel.keybindingLabelForeground};`),r.push("}"));const a=r.join(`
`);a!==this.ui.styleSheet.textContent&&(this.ui.styleSheet.textContent=a)}}},NA=Ng,Ng.MAX_WIDTH=600,Ng);TA=NA=ime([gT(1,tu),gT(2,we),gT(3,Ee)],TA);var nme=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},lC=function(o,e){return function(t,i){e(t,i,o)}};let RA=class extends Gse{get controller(){return this._controller||(this._controller=this._register(this.createController())),this._controller}get hasController(){return!!this._controller}get currentQuickInput(){return this.controller.currentQuickInput}get quickAccess(){return this._quickAccess||(this._quickAccess=this._register(this.instantiationService.createInstance(pA))),this._quickAccess}constructor(e,t,i,n,s){super(i),this.instantiationService=e,this.contextKeyService=t,this.layoutService=n,this.configurationService=s,this._onShow=this._register(new P),this._onHide=this._register(new P),this.contexts=new Map}createController(e=this.layoutService,t){const i={idPrefix:"quickInput_",container:e.activeContainer,ignoreFocusOut:()=>!1,backKeybindingLabel:()=>{},setContextKey:s=>this.setContextKey(s),linkOpenerDelegate:s=>{this.instantiationService.invokeFunction(r=>{r.get(zs).open(s,{allowCommands:!0,fromUserGesture:!0})})},returnFocus:()=>e.focus(),styles:this.computeStyles(),hoverDelegate:this._register(this.instantiationService.createInstance(bA))},n=this._register(this.instantiationService.createInstance(TA,{...i,...t}));return n.layout(e.activeContainerDimension,e.activeContainerOffset.quickPickTop),this._register(e.onDidLayoutActiveContainer(s=>{xe(e.activeContainer)===xe(n.container)&&n.layout(s,e.activeContainerOffset.quickPickTop)})),this._register(e.onDidChangeActiveContainer(()=>{n.isVisible()||n.layout(e.activeContainerDimension,e.activeContainerOffset.quickPickTop)})),this._register(n.onShow(()=>{this.resetContextKeys(),this._onShow.fire()})),this._register(n.onHide(()=>{this.resetContextKeys(),this._onHide.fire()})),n}setContextKey(e){let t;e&&(t=this.contexts.get(e),t||(t=new le(e,!1).bindTo(this.contextKeyService),this.contexts.set(e,t))),!(t&&t.get())&&(this.resetContextKeys(),t==null||t.set(!0))}resetContextKeys(){this.contexts.forEach(e=>{e.get()&&e.reset()})}pick(e,t,i=ze.None){return this.controller.pick(e,t,i)}createQuickPick(e={useSeparators:!1}){return this.controller.createQuickPick(e)}createInputBox(){return this.controller.createInputBox()}updateStyles(){this.hasController&&this.controller.applyStyles(this.computeStyles())}computeStyles(){return{widget:{quickInputBackground:ce(I9),quickInputForeground:ce(dae),quickInputTitleBackground:ce(hae),widgetBorder:ce(F$),widgetShadow:ce(Cb)},inputBox:xL,toggle:LL,countBadge:Pj,button:jde,progressBar:Kde,keybindingLabel:$de,list:sm({listBackground:I9,listFocusBackground:vw,listFocusForeground:Cw,listInactiveFocusForeground:Cw,listInactiveSelectionIconForeground:O5,listInactiveFocusBackground:vw,listFocusOutline:Xt,listInactiveFocusOutline:Xt}),pickerGroup:{pickerGroupBorder:ce(uae),pickerGroupForeground:ce(q$)}}}};RA=nme([lC(0,we),lC(1,Ee),lC(2,ji),lC(3,tu),lC(4,qe)],RA);var YK=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Gu=function(o,e){return function(t,i){e(t,i,o)}};let MA=class extends RA{constructor(e,t,i,n,s,r){super(t,i,n,new JM(e.getContainerDomNode(),s),r),this.host=void 0;const a=Bw.get(e);if(a){const l=a.widget;this.host={_serviceBrand:void 0,get mainContainer(){return l.getDomNode()},getContainer(){return l.getDomNode()},whenContainerStylesLoaded(){},get containers(){return[l.getDomNode()]},get activeContainer(){return l.getDomNode()},get mainContainerDimension(){return e.getLayoutInfo()},get activeContainerDimension(){return e.getLayoutInfo()},get onDidLayoutMainContainer(){return e.onDidLayoutChange},get onDidLayoutActiveContainer(){return e.onDidLayoutChange},get onDidLayoutContainer(){return oe.map(e.onDidLayoutChange,c=>({container:l.getDomNode(),dimension:c}))},get onDidChangeActiveContainer(){return oe.None},get onDidAddContainer(){return oe.None},get mainContainerOffset(){return{top:0,quickPickTop:0}},get activeContainerOffset(){return{top:0,quickPickTop:0}},focus:()=>e.focus()}}else this.host=void 0}createController(){return super.createController(this.host)}};MA=YK([Gu(1,we),Gu(2,Ee),Gu(3,ji),Gu(4,pt),Gu(5,qe)],MA);let AA=class{get activeService(){const e=this.codeEditorService.getFocusedCodeEditor();if(!e)throw new Error("Quick input service needs a focused editor to work.");let t=this.mapEditorToService.get(e);if(!t){const i=t=this.instantiationService.createInstance(MA,e);this.mapEditorToService.set(e,t),Ah(e.onDidDispose)(()=>{i.dispose(),this.mapEditorToService.delete(e)})}return t}get currentQuickInput(){return this.activeService.currentQuickInput}get quickAccess(){return this.activeService.quickAccess}constructor(e,t){this.instantiationService=e,this.codeEditorService=t,this.mapEditorToService=new Map}pick(e,t,i=ze.None){return this.activeService.pick(e,t,i)}createQuickPick(e={useSeparators:!1}){return this.activeService.createQuickPick(e)}createInputBox(){return this.activeService.createInputBox()}};AA=YK([Gu(0,we),Gu(1,pt)],AA);const Jk=class Jk{static get(e){return e.getContribution(Jk.ID)}constructor(e){this.editor=e,this.widget=new PA(this.editor)}dispose(){this.widget.dispose()}};Jk.ID="editor.controller.quickInput";let Bw=Jk;const eD=class eD{constructor(e){this.codeEditor=e,this.domNode=document.createElement("div"),this.codeEditor.addOverlayWidget(this)}getId(){return eD.ID}getDomNode(){return this.domNode}getPosition(){return{preference:2}}dispose(){this.codeEditor.removeOverlayWidget(this)}};eD.ID="editor.contrib.quickInputWidget";let PA=eD;ct(Bw.ID,Bw,4);class sme{constructor(e,t,i,n,s){this._parsedThemeRuleBrand=void 0,this.token=e,this.index=t,this.fontStyle=i,this.foreground=n,this.background=s}}function ome(o){if(!o||!Array.isArray(o))return[];const e=[];let t=0;for(let i=0,n=o.length;i<n;i++){const s=o[i];let r=-1;if(typeof s.fontStyle=="string"){r=0;const c=s.fontStyle.split(" ");for(let d=0,h=c.length;d<h;d++)switch(c[d]){case"italic":r=r|1;break;case"bold":r=r|2;break;case"underline":r=r|4;break;case"strikethrough":r=r|8;break}}let a=null;typeof s.foreground=="string"&&(a=s.foreground);let l=null;typeof s.background=="string"&&(l=s.background),e[t++]=new sme(s.token||"",i,r,a,l)}return e}function rme(o,e){o.sort((d,h)=>{const u=hme(d.token,h.token);return u!==0?u:d.index-h.index});let t=0,i="000000",n="ffffff";for(;o.length>=1&&o[0].token==="";){const d=o.shift();d.fontStyle!==-1&&(t=d.fontStyle),d.foreground!==null&&(i=d.foreground),d.background!==null&&(n=d.background)}const s=new lme;for(const d of e)s.getId(d);const r=s.getId(i),a=s.getId(n),l=new O3(t,r,a),c=new F3(l);for(let d=0,h=o.length;d<h;d++){const u=o[d];c.insert(u.token,u.fontStyle,s.getId(u.foreground),s.getId(u.background))}return new XK(s,c)}const ame=/^#?([0-9A-Fa-f]{6})([0-9A-Fa-f]{2})?$/;class lme{constructor(){this._lastColorId=0,this._id2color=[],this._color2id=new Map}getId(e){if(e===null)return 0;const t=e.match(ame);if(!t)throw new Error("Illegal value for token color: "+e);e=t[1].toUpperCase();let i=this._color2id.get(e);return i||(i=++this._lastColorId,this._color2id.set(e,i),this._id2color[i]=Z.fromHex("#"+e),i)}getColorMap(){return this._id2color.slice(0)}}class XK{static createFromRawTokenTheme(e,t){return this.createFromParsedTokenTheme(ome(e),t)}static createFromParsedTokenTheme(e,t){return rme(e,t)}constructor(e,t){this._colorMap=e,this._root=t,this._cache=new Map}getColorMap(){return this._colorMap.getColorMap()}_match(e){return this._root.match(e)}match(e,t){let i=this._cache.get(t);if(typeof i>"u"){const n=this._match(t),s=dme(t);i=(n.metadata|s<<8)>>>0,this._cache.set(t,i)}return(i|e<<0)>>>0}}const cme=/\b(comment|string|regex|regexp)\b/;function dme(o){const e=o.match(cme);if(!e)return 0;switch(e[1]){case"comment":return 1;case"string":return 2;case"regex":return 3;case"regexp":return 3}throw new Error("Unexpected match for standard token type!")}function hme(o,e){return o<e?-1:o>e?1:0}class O3{constructor(e,t,i){this._themeTrieElementRuleBrand=void 0,this._fontStyle=e,this._foreground=t,this._background=i,this.metadata=(this._fontStyle<<11|this._foreground<<15|this._background<<24)>>>0}clone(){return new O3(this._fontStyle,this._foreground,this._background)}acceptOverwrite(e,t,i){e!==-1&&(this._fontStyle=e),t!==0&&(this._foreground=t),i!==0&&(this._background=i),this.metadata=(this._fontStyle<<11|this._foreground<<15|this._background<<24)>>>0}}class F3{constructor(e){this._themeTrieElementBrand=void 0,this._mainRule=e,this._children=new Map}match(e){if(e==="")return this._mainRule;const t=e.indexOf(".");let i,n;t===-1?(i=e,n=""):(i=e.substring(0,t),n=e.substring(t+1));const s=this._children.get(i);return typeof s<"u"?s.match(n):this._mainRule}insert(e,t,i,n){if(e===""){this._mainRule.acceptOverwrite(t,i,n);return}const s=e.indexOf(".");let r,a;s===-1?(r=e,a=""):(r=e.substring(0,s),a=e.substring(s+1));let l=this._children.get(r);typeof l>"u"&&(l=new F3(this._mainRule.clone()),this._children.set(r,l)),l.insert(a,t,i,n)}}function ume(o){const e=[];for(let t=1,i=o.length;t<i;t++){const n=o[t];e[t]=`.mtk${t} { color: ${n}; }`}return e.push(".mtki { font-style: italic; }"),e.push(".mtkb { font-weight: bold; }"),e.push(".mtku { text-decoration: underline; text-underline-position: under; }"),e.push(".mtks { text-decoration: line-through; }"),e.push(".mtks.mtku { text-decoration: underline line-through; text-underline-position: under; }"),e.join(`
`)}const gme={base:"vs",inherit:!1,rules:[{token:"",foreground:"000000",background:"fffffe"},{token:"invalid",foreground:"cd3131"},{token:"emphasis",fontStyle:"italic"},{token:"strong",fontStyle:"bold"},{token:"variable",foreground:"001188"},{token:"variable.predefined",foreground:"4864AA"},{token:"constant",foreground:"dd0000"},{token:"comment",foreground:"008000"},{token:"number",foreground:"098658"},{token:"number.hex",foreground:"3030c0"},{token:"regexp",foreground:"800000"},{token:"annotation",foreground:"808080"},{token:"type",foreground:"008080"},{token:"delimiter",foreground:"000000"},{token:"delimiter.html",foreground:"383838"},{token:"delimiter.xml",foreground:"0000FF"},{token:"tag",foreground:"800000"},{token:"tag.id.pug",foreground:"4F76AC"},{token:"tag.class.pug",foreground:"4F76AC"},{token:"meta.scss",foreground:"800000"},{token:"metatag",foreground:"e00000"},{token:"metatag.content.html",foreground:"FF0000"},{token:"metatag.html",foreground:"808080"},{token:"metatag.xml",foreground:"808080"},{token:"metatag.php",fontStyle:"bold"},{token:"key",foreground:"863B00"},{token:"string.key.json",foreground:"A31515"},{token:"string.value.json",foreground:"0451A5"},{token:"attribute.name",foreground:"FF0000"},{token:"attribute.value",foreground:"0451A5"},{token:"attribute.value.number",foreground:"098658"},{token:"attribute.value.unit",foreground:"098658"},{token:"attribute.value.html",foreground:"0000FF"},{token:"attribute.value.xml",foreground:"0000FF"},{token:"string",foreground:"A31515"},{token:"string.html",foreground:"0000FF"},{token:"string.sql",foreground:"FF0000"},{token:"string.yaml",foreground:"0451A5"},{token:"keyword",foreground:"0000FF"},{token:"keyword.json",foreground:"0451A5"},{token:"keyword.flow",foreground:"AF00DB"},{token:"keyword.flow.scss",foreground:"0000FF"},{token:"operator.scss",foreground:"666666"},{token:"operator.sql",foreground:"778899"},{token:"operator.swift",foreground:"666666"},{token:"predefined.sql",foreground:"C700C7"}],colors:{[Xr]:"#FFFFFE",[Zl]:"#000000",[P$]:"#E5EBF1",[J1]:"#D3D3D3",[eS]:"#939393",[N5]:"#ADD6FF4D"}},fme={base:"vs-dark",inherit:!1,rules:[{token:"",foreground:"D4D4D4",background:"1E1E1E"},{token:"invalid",foreground:"f44747"},{token:"emphasis",fontStyle:"italic"},{token:"strong",fontStyle:"bold"},{token:"variable",foreground:"74B0DF"},{token:"variable.predefined",foreground:"4864AA"},{token:"variable.parameter",foreground:"9CDCFE"},{token:"constant",foreground:"569CD6"},{token:"comment",foreground:"608B4E"},{token:"number",foreground:"B5CEA8"},{token:"number.hex",foreground:"5BB498"},{token:"regexp",foreground:"B46695"},{token:"annotation",foreground:"cc6666"},{token:"type",foreground:"3DC9B0"},{token:"delimiter",foreground:"DCDCDC"},{token:"delimiter.html",foreground:"808080"},{token:"delimiter.xml",foreground:"808080"},{token:"tag",foreground:"569CD6"},{token:"tag.id.pug",foreground:"4F76AC"},{token:"tag.class.pug",foreground:"4F76AC"},{token:"meta.scss",foreground:"A79873"},{token:"meta.tag",foreground:"CE9178"},{token:"metatag",foreground:"DD6A6F"},{token:"metatag.content.html",foreground:"9CDCFE"},{token:"metatag.html",foreground:"569CD6"},{token:"metatag.xml",foreground:"569CD6"},{token:"metatag.php",fontStyle:"bold"},{token:"key",foreground:"9CDCFE"},{token:"string.key.json",foreground:"9CDCFE"},{token:"string.value.json",foreground:"CE9178"},{token:"attribute.name",foreground:"9CDCFE"},{token:"attribute.value",foreground:"CE9178"},{token:"attribute.value.number.css",foreground:"B5CEA8"},{token:"attribute.value.unit.css",foreground:"B5CEA8"},{token:"attribute.value.hex.css",foreground:"D4D4D4"},{token:"string",foreground:"CE9178"},{token:"string.sql",foreground:"FF0000"},{token:"keyword",foreground:"569CD6"},{token:"keyword.flow",foreground:"C586C0"},{token:"keyword.json",foreground:"CE9178"},{token:"keyword.flow.scss",foreground:"569CD6"},{token:"operator.scss",foreground:"909090"},{token:"operator.sql",foreground:"778899"},{token:"operator.swift",foreground:"909090"},{token:"predefined.sql",foreground:"FF00FF"}],colors:{[Xr]:"#1E1E1E",[Zl]:"#D4D4D4",[P$]:"#3A3D41",[J1]:"#404040",[eS]:"#707070",[N5]:"#ADD6FF26"}},mme={base:"hc-black",inherit:!1,rules:[{token:"",foreground:"FFFFFF",background:"000000"},{token:"invalid",foreground:"f44747"},{token:"emphasis",fontStyle:"italic"},{token:"strong",fontStyle:"bold"},{token:"variable",foreground:"1AEBFF"},{token:"variable.parameter",foreground:"9CDCFE"},{token:"constant",foreground:"569CD6"},{token:"comment",foreground:"608B4E"},{token:"number",foreground:"FFFFFF"},{token:"regexp",foreground:"C0C0C0"},{token:"annotation",foreground:"569CD6"},{token:"type",foreground:"3DC9B0"},{token:"delimiter",foreground:"FFFF00"},{token:"delimiter.html",foreground:"FFFF00"},{token:"tag",foreground:"569CD6"},{token:"tag.id.pug",foreground:"4F76AC"},{token:"tag.class.pug",foreground:"4F76AC"},{token:"meta",foreground:"D4D4D4"},{token:"meta.tag",foreground:"CE9178"},{token:"metatag",foreground:"569CD6"},{token:"metatag.content.html",foreground:"1AEBFF"},{token:"metatag.html",foreground:"569CD6"},{token:"metatag.xml",foreground:"569CD6"},{token:"metatag.php",fontStyle:"bold"},{token:"key",foreground:"9CDCFE"},{token:"string.key",foreground:"9CDCFE"},{token:"string.value",foreground:"CE9178"},{token:"attribute.name",foreground:"569CD6"},{token:"attribute.value",foreground:"3FF23F"},{token:"string",foreground:"CE9178"},{token:"string.sql",foreground:"FF0000"},{token:"keyword",foreground:"569CD6"},{token:"keyword.flow",foreground:"C586C0"},{token:"operator.sql",foreground:"778899"},{token:"operator.swift",foreground:"909090"},{token:"predefined.sql",foreground:"FF00FF"}],colors:{[Xr]:"#000000",[Zl]:"#FFFFFF",[J1]:"#FFFFFF",[eS]:"#FFFFFF"}},pme={base:"hc-light",inherit:!1,rules:[{token:"",foreground:"292929",background:"FFFFFF"},{token:"invalid",foreground:"B5200D"},{token:"emphasis",fontStyle:"italic"},{token:"strong",fontStyle:"bold"},{token:"variable",foreground:"264F70"},{token:"variable.predefined",foreground:"4864AA"},{token:"constant",foreground:"dd0000"},{token:"comment",foreground:"008000"},{token:"number",foreground:"098658"},{token:"number.hex",foreground:"3030c0"},{token:"regexp",foreground:"800000"},{token:"annotation",foreground:"808080"},{token:"type",foreground:"008080"},{token:"delimiter",foreground:"000000"},{token:"delimiter.html",foreground:"383838"},{token:"tag",foreground:"800000"},{token:"tag.id.pug",foreground:"4F76AC"},{token:"tag.class.pug",foreground:"4F76AC"},{token:"meta.scss",foreground:"800000"},{token:"metatag",foreground:"e00000"},{token:"metatag.content.html",foreground:"B5200D"},{token:"metatag.html",foreground:"808080"},{token:"metatag.xml",foreground:"808080"},{token:"metatag.php",fontStyle:"bold"},{token:"key",foreground:"863B00"},{token:"string.key.json",foreground:"A31515"},{token:"string.value.json",foreground:"0451A5"},{token:"attribute.name",foreground:"264F78"},{token:"attribute.value",foreground:"0451A5"},{token:"string",foreground:"A31515"},{token:"string.sql",foreground:"B5200D"},{token:"keyword",foreground:"0000FF"},{token:"keyword.flow",foreground:"AF00DB"},{token:"operator.sql",foreground:"778899"},{token:"operator.swift",foreground:"666666"},{token:"predefined.sql",foreground:"C700C7"}],colors:{[Xr]:"#FFFFFF",[Zl]:"#292929",[J1]:"#292929",[eS]:"#292929"}},_me={IconContribution:"base.contributions.icons"};var iH;(function(o){function e(t,i){let n=t.defaults;for(;Ie.isThemeIcon(n);){const s=am.getIcon(n.id);if(!s)return;n=s.defaults}return n}o.getDefinition=e})(iH||(iH={}));var nH;(function(o){function e(i){return{weight:i.weight,style:i.style,src:i.src.map(n=>({format:n.format,location:n.location.toString()}))}}o.toJSONObject=e;function t(i){const n=s=>As(s)?s:void 0;if(i&&Array.isArray(i.src)&&i.src.every(s=>As(s.format)&&As(s.location)))return{weight:n(i.weight),style:n(i.style),src:i.src.map(s=>({format:s.format,location:Se.parse(s.location)}))}}o.fromJSONObject=t})(nH||(nH={}));class bme{constructor(){this._onDidChange=new P,this.onDidChange=this._onDidChange.event,this.iconSchema={definitions:{icons:{type:"object",properties:{fontId:{type:"string",description:m("iconDefinition.fontId","The id of the font to use. If not set, the font that is defined first is used.")},fontCharacter:{type:"string",description:m("iconDefinition.fontCharacter","The font character associated with the icon definition.")}},additionalProperties:!1,defaultSnippets:[{body:{fontCharacter:"\\\\e030"}}]}},type:"object",properties:{}},this.iconReferenceSchema={type:"string",pattern:`^${Ie.iconNameExpression}$`,enum:[],enumDescriptions:[]},this.iconsById={},this.iconFontsById={}}registerIcon(e,t,i,n){const s=this.iconsById[e];if(s){if(i&&!s.description){s.description=i,this.iconSchema.properties[e].markdownDescription=`${i} $(${e})`;const l=this.iconReferenceSchema.enum.indexOf(e);l!==-1&&(this.iconReferenceSchema.enumDescriptions[l]=i),this._onDidChange.fire()}return s}const r={id:e,description:i,defaults:t,deprecationMessage:n};this.iconsById[e]=r;const a={$ref:"#/definitions/icons"};return n&&(a.deprecationMessage=n),i&&(a.markdownDescription=`${i}: $(${e})`),this.iconSchema.properties[e]=a,this.iconReferenceSchema.enum.push(e),this.iconReferenceSchema.enumDescriptions.push(i||""),this._onDidChange.fire(),{id:e}}getIcons(){return Object.keys(this.iconsById).map(e=>this.iconsById[e])}getIcon(e){return this.iconsById[e]}getIconSchema(){return this.iconSchema}toString(){const e=(s,r)=>s.id.localeCompare(r.id),t=s=>{for(;Ie.isThemeIcon(s.defaults);)s=this.iconsById[s.defaults.id];return`codicon codicon-${s?s.id:""}`},i=[];i.push("| preview     | identifier                        | default codicon ID                | description"),i.push("| ----------- | --------------------------------- | --------------------------------- | --------------------------------- |");const n=Object.keys(this.iconsById).map(s=>this.iconsById[s]);for(const s of n.filter(r=>!!r.description).sort(e))i.push(`|<i class="${t(s)}"></i>|${s.id}|${Ie.isThemeIcon(s.defaults)?s.defaults.id:s.id}|${s.description||""}|`);i.push("| preview     | identifier                        "),i.push("| ----------- | --------------------------------- |");for(const s of n.filter(r=>!Ie.isThemeIcon(r.defaults)).sort(e))i.push(`|<i class="${t(s)}"></i>|${s.id}|`);return i.join(`
`)}}const am=new bme;ci.add(_me.IconContribution,am);function qt(o,e,t,i){return am.registerIcon(o,e,t,i)}function QK(){return am}function Cme(){const o=Gz();for(const e in o){const t="\\"+o[e].toString(16);am.registerIcon(e,{fontCharacter:t})}}Cme();const JK="vscode://schemas/icons",eq=ci.as(AI.JSONContribution);eq.registerSchema(JK,am.getIconSchema());const sH=new It(()=>eq.notifySchemaChanged(JK),200);am.onDidChange(()=>{sH.isScheduled()||sH.schedule()});const tq=qt("widget-close",te.close,m("widgetClose","Icon for the close action in widgets."));qt("goto-previous-location",te.arrowUp,m("previousChangeIcon","Icon for goto previous editor location."));qt("goto-next-location",te.arrowDown,m("nextChangeIcon","Icon for goto next editor location."));Ie.modify(te.sync,"spin");Ie.modify(te.loading,"spin");function vme(o){const e=new K,t=e.add(new P),i=QK();return e.add(i.onDidChange(()=>t.fire())),o&&e.add(o.onDidProductIconThemeChange(()=>t.fire())),{dispose:()=>e.dispose(),onDidChange:t.event,getCSS(){const n=o?o.getProductIconTheme():new iq,s={},r=[],a=[];for(const l of i.getIcons()){const c=n.getIcon(l);if(!c)continue;const d=c.font,h=`--vscode-icon-${l.id}-font-family`,u=`--vscode-icon-${l.id}-content`;d?(s[d.id]=d.definition,a.push(`${h}: ${CN(d.id)};`,`${u}: '${c.fontCharacter}';`),r.push(`.codicon-${l.id}:before { content: '${c.fontCharacter}'; font-family: ${CN(d.id)}; }`)):(a.push(`${u}: '${c.fontCharacter}'; ${h}: 'codicon';`),r.push(`.codicon-${l.id}:before { content: '${c.fontCharacter}'; }`))}for(const l in s){const c=s[l],d=c.weight?`font-weight: ${c.weight};`:"",h=c.style?`font-style: ${c.style};`:"",u=c.src.map(g=>`${td(g.location)} format('${g.format}')`).join(", ");r.push(`@font-face { src: ${u}; font-family: ${CN(l)};${d}${h} font-display: block; }`)}return r.push(`:root { ${a.join(" ")} }`),r.join(`
`)}}}class iq{getIcon(e){const t=QK();let i=e.defaults;for(;Ie.isThemeIcon(i);){const n=t.getIcon(i.id);if(!n)return;i=n.defaults}return i}}const Ic="vs",t_="vs-dark",wg="hc-black",Sg="hc-light",nq=ci.as(E$.ColorContribution),wme=ci.as(g$.ThemingContribution);class sq{constructor(e,t){this.semanticHighlighting=!1,this.themeData=t;const i=t.base;e.length>0?(ry(e)?this.id=e:this.id=i+" "+e,this.themeName=e):(this.id=i,this.themeName=i),this.colors=null,this.defaultColors=Object.create(null),this._tokenTheme=null}get base(){return this.themeData.base}notifyBaseUpdated(){this.themeData.inherit&&(this.colors=null,this._tokenTheme=null)}getColors(){if(!this.colors){const e=new Map;for(const t in this.themeData.colors)e.set(t,Z.fromHex(this.themeData.colors[t]));if(this.themeData.inherit){const t=OA(this.themeData.base);for(const i in t.colors)e.has(i)||e.set(i,Z.fromHex(t.colors[i]))}this.colors=e}return this.colors}getColor(e,t){const i=this.getColors().get(e);if(i)return i;if(t!==!1)return this.getDefault(e)}getDefault(e){let t=this.defaultColors[e];return t||(t=nq.resolveDefaultColor(e,this),this.defaultColors[e]=t,t)}defines(e){return this.getColors().has(e)}get type(){switch(this.base){case Ic:return hr.LIGHT;case wg:return hr.HIGH_CONTRAST_DARK;case Sg:return hr.HIGH_CONTRAST_LIGHT;default:return hr.DARK}}get tokenTheme(){if(!this._tokenTheme){let e=[],t=[];if(this.themeData.inherit){const s=OA(this.themeData.base);e=s.rules,s.encodedTokensColors&&(t=s.encodedTokensColors)}const i=this.themeData.colors["editor.foreground"],n=this.themeData.colors["editor.background"];if(i||n){const s={token:""};i&&(s.foreground=i),n&&(s.background=n),e.push(s)}e=e.concat(this.themeData.rules),this.themeData.encodedTokensColors&&(t=this.themeData.encodedTokensColors),this._tokenTheme=XK.createFromRawTokenTheme(e,t)}return this._tokenTheme}getTokenStyleMetadata(e,t,i){const s=this.tokenTheme._match([e].concat(t).join(".")).metadata,r=Ts.getForeground(s),a=Ts.getFontStyle(s);return{foreground:r,italic:!!(a&1),bold:!!(a&2),underline:!!(a&4),strikethrough:!!(a&8)}}}function ry(o){return o===Ic||o===t_||o===wg||o===Sg}function OA(o){switch(o){case Ic:return gme;case t_:return fme;case wg:return mme;case Sg:return pme}}function s0(o){const e=OA(o);return new sq(o,e)}class Sme extends F{constructor(){super(),this._onColorThemeChange=this._register(new P),this.onDidColorThemeChange=this._onColorThemeChange.event,this._onProductIconThemeChange=this._register(new P),this.onDidProductIconThemeChange=this._onProductIconThemeChange.event,this._environment=Object.create(null),this._builtInProductIconTheme=new iq,this._autoDetectHighContrast=!0,this._knownThemes=new Map,this._knownThemes.set(Ic,s0(Ic)),this._knownThemes.set(t_,s0(t_)),this._knownThemes.set(wg,s0(wg)),this._knownThemes.set(Sg,s0(Sg));const e=this._register(vme(this));this._codiconCSS=e.getCSS(),this._themeCSS="",this._allCSS=`${this._codiconCSS}
${this._themeCSS}`,this._globalStyleElement=null,this._styleElements=[],this._colorMapOverride=null,this.setTheme(Ic),this._onOSSchemeChanged(),this._register(e.onDidChange(()=>{this._codiconCSS=e.getCSS(),this._updateCSS()})),aU(Rt,"(forced-colors: active)",()=>{this._onOSSchemeChanged()})}registerEditorContainer(e){return jy(e)?this._registerShadowDomContainer(e):this._registerRegularEditorContainer()}_registerRegularEditorContainer(){return this._globalStyleElement||(this._globalStyleElement=$o(void 0,e=>{e.className="monaco-colors",e.textContent=this._allCSS}),this._styleElements.push(this._globalStyleElement)),F.None}_registerShadowDomContainer(e){const t=$o(e,i=>{i.className="monaco-colors",i.textContent=this._allCSS});return this._styleElements.push(t),{dispose:()=>{for(let i=0;i<this._styleElements.length;i++)if(this._styleElements[i]===t){this._styleElements.splice(i,1);return}}}}defineTheme(e,t){if(!/^[a-z0-9\-]+$/i.test(e))throw new Error("Illegal theme name!");if(!ry(t.base)&&!ry(e))throw new Error("Illegal theme base!");this._knownThemes.set(e,new sq(e,t)),ry(e)&&this._knownThemes.forEach(i=>{i.base===e&&i.notifyBaseUpdated()}),this._theme.themeName===e&&this.setTheme(e)}getColorTheme(){return this._theme}setColorMapOverride(e){this._colorMapOverride=e,this._updateThemeOrColorMap()}setTheme(e){let t;this._knownThemes.has(e)?t=this._knownThemes.get(e):t=this._knownThemes.get(Ic),this._updateActualTheme(t)}_updateActualTheme(e){!e||this._theme===e||(this._theme=e,this._updateThemeOrColorMap())}_onOSSchemeChanged(){if(this._autoDetectHighContrast){const e=Rt.matchMedia("(forced-colors: active)").matches;if(e!==Ta(this._theme.type)){let t;N_(this._theme.type)?t=e?wg:t_:t=e?Sg:Ic,this._updateActualTheme(this._knownThemes.get(t))}}}setAutoDetectHighContrast(e){this._autoDetectHighContrast=e,this._onOSSchemeChanged()}_updateThemeOrColorMap(){const e=[],t={},i={addRule:r=>{t[r]||(e.push(r),t[r]=!0)}};wme.getThemingParticipants().forEach(r=>r(this._theme,i,this._environment));const n=[];for(const r of nq.getColors()){const a=this._theme.getColor(r.id,!0);a&&n.push(`${D5(r.id)}: ${a.toString()};`)}i.addRule(`.monaco-editor, .monaco-diff-editor, .monaco-component { ${n.join(`
`)} }`);const s=this._colorMapOverride||this._theme.tokenTheme.getColorMap();i.addRule(ume(s)),this._themeCSS=e.join(`
`),this._updateCSS(),fi.setColorMap(s),this._onColorThemeChange.fire(this._theme)}_updateCSS(){this._allCSS=`${this._codiconCSS}
${this._themeCSS}`,this._styleElements.forEach(e=>e.textContent=this._allCSS)}getFileIconTheme(){return{hasFileIcons:!1,hasFolderIcons:!1,hidesExplorerArrows:!1}}getProductIconTheme(){return this._builtInProductIconTheme}}const Qo=et("themeService");var yme=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},fT=function(o,e){return function(t,i){e(t,i,o)}};let FA=class extends F{constructor(e,t,i){super(),this._contextKeyService=e,this._layoutService=t,this._configurationService=i,this._accessibilitySupport=0,this._onDidChangeScreenReaderOptimized=new P,this._onDidChangeReducedMotion=new P,this._onDidChangeLinkUnderline=new P,this._accessibilityModeEnabledContext=$1.bindTo(this._contextKeyService);const n=()=>this._accessibilityModeEnabledContext.set(this.isScreenReaderOptimized());this._register(this._configurationService.onDidChangeConfiguration(r=>{r.affectsConfiguration("editor.accessibilitySupport")&&(n(),this._onDidChangeScreenReaderOptimized.fire()),r.affectsConfiguration("workbench.reduceMotion")&&(this._configMotionReduced=this._configurationService.getValue("workbench.reduceMotion"),this._onDidChangeReducedMotion.fire())})),n(),this._register(this.onDidChangeScreenReaderOptimized(()=>n()));const s=Rt.matchMedia("(prefers-reduced-motion: reduce)");this._systemMotionReduced=s.matches,this._configMotionReduced=this._configurationService.getValue("workbench.reduceMotion"),this._linkUnderlinesEnabled=this._configurationService.getValue("accessibility.underlineLinks"),this.initReducedMotionListeners(s),this.initLinkUnderlineListeners()}initReducedMotionListeners(e){this._register(U(e,"change",()=>{this._systemMotionReduced=e.matches,this._configMotionReduced==="auto"&&this._onDidChangeReducedMotion.fire()}));const t=()=>{const i=this.isMotionReduced();this._layoutService.mainContainer.classList.toggle("reduce-motion",i),this._layoutService.mainContainer.classList.toggle("enable-motion",!i)};t(),this._register(this.onDidChangeReducedMotion(()=>t()))}initLinkUnderlineListeners(){this._register(this._configurationService.onDidChangeConfiguration(t=>{if(t.affectsConfiguration("accessibility.underlineLinks")){const i=this._configurationService.getValue("accessibility.underlineLinks");this._linkUnderlinesEnabled=i,this._onDidChangeLinkUnderline.fire()}}));const e=()=>{const t=this._linkUnderlinesEnabled;this._layoutService.mainContainer.classList.toggle("underline-links",t)};e(),this._register(this.onDidChangeLinkUnderlines(()=>e()))}onDidChangeLinkUnderlines(e){return this._onDidChangeLinkUnderline.event(e)}get onDidChangeScreenReaderOptimized(){return this._onDidChangeScreenReaderOptimized.event}isScreenReaderOptimized(){const e=this._configurationService.getValue("editor.accessibilitySupport");return e==="on"||e==="auto"&&this._accessibilitySupport===2}get onDidChangeReducedMotion(){return this._onDidChangeReducedMotion.event}isMotionReduced(){const e=this._configMotionReduced;return e==="on"||e==="auto"&&this._systemMotionReduced}getAccessibilitySupport(){return this._accessibilitySupport}};FA=yme([fT(0,Ee),fT(1,tu),fT(2,qe)],FA);var fE=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Al=function(o,e){return function(t,i){e(t,i,o)}},jm,NC;let WA=class{constructor(e,t,i){this._commandService=e,this._keybindingService=t,this._hiddenStates=new BA(i)}createMenu(e,t,i){return new XL(e,this._hiddenStates,{emitEventsForSubmenuChanges:!1,eventDebounceDelay:50,...i},this._commandService,this._keybindingService,t)}getMenuActions(e,t,i){const n=new XL(e,this._hiddenStates,{emitEventsForSubmenuChanges:!1,eventDebounceDelay:50,...i},this._commandService,this._keybindingService,t),s=n.getActions(i);return n.dispose(),s}resetHiddenStates(e){this._hiddenStates.reset(e)}};WA=fE([Al(0,Ht),Al(1,yt),Al(2,ia)],WA);var Tg;let BA=(Tg=class{constructor(e){this._storageService=e,this._disposables=new K,this._onDidChange=new P,this.onDidChange=this._onDidChange.event,this._ignoreChangeEvent=!1,this._hiddenByDefaultCache=new Map;try{const t=e.get(jm._key,0,"{}");this._data=JSON.parse(t)}catch{this._data=Object.create(null)}this._disposables.add(e.onDidChangeValue(0,jm._key,this._disposables)(()=>{if(!this._ignoreChangeEvent)try{const t=e.get(jm._key,0,"{}");this._data=JSON.parse(t)}catch(t){console.log("FAILED to read storage after UPDATE",t)}this._onDidChange.fire()}))}dispose(){this._onDidChange.dispose(),this._disposables.dispose()}_isHiddenByDefault(e,t){return this._hiddenByDefaultCache.get(`${e.id}/${t}`)??!1}setDefaultState(e,t,i){this._hiddenByDefaultCache.set(`${e.id}/${t}`,i)}isHidden(e,t){var s;const i=this._isHiddenByDefault(e,t),n=((s=this._data[e.id])==null?void 0:s.includes(t))??!1;return i?!n:n}updateHidden(e,t,i){this._isHiddenByDefault(e,t)&&(i=!i);const s=this._data[e.id];if(i)s?s.indexOf(t)<0&&s.push(t):this._data[e.id]=[t];else if(s){const r=s.indexOf(t);r>=0&&bQ(s,r),s.length===0&&delete this._data[e.id]}this._persist()}reset(e){if(e===void 0)this._data=Object.create(null),this._persist();else{for(const{id:t}of e)this._data[t]&&delete this._data[t];this._persist()}}_persist(){try{this._ignoreChangeEvent=!0;const e=JSON.stringify(this._data);this._storageService.store(jm._key,e,0,0)}finally{this._ignoreChangeEvent=!1}}},jm=Tg,Tg._key="menu.hiddenCommands",Tg);BA=jm=fE([Al(0,ia)],BA);class rv{constructor(e,t){this._id=e,this._collectContextKeysForSubmenus=t,this._menuGroups=[],this._allMenuIds=new Set,this._structureContextKeys=new Set,this._preconditionContextKeys=new Set,this._toggledContextKeys=new Set,this.refresh()}get allMenuIds(){return this._allMenuIds}get structureContextKeys(){return this._structureContextKeys}get preconditionContextKeys(){return this._preconditionContextKeys}get toggledContextKeys(){return this._toggledContextKeys}refresh(){this._menuGroups.length=0,this._allMenuIds.clear(),this._structureContextKeys.clear(),this._preconditionContextKeys.clear(),this._toggledContextKeys.clear();const e=this._sort(rn.getMenuItems(this._id));let t;for(const i of e){const n=i.group||"";(!t||t[0]!==n)&&(t=[n,[]],this._menuGroups.push(t)),t[1].push(i),this._collectContextKeysAndSubmenuIds(i)}this._allMenuIds.add(this._id)}_sort(e){return e}_collectContextKeysAndSubmenuIds(e){if(rv._fillInKbExprKeys(e.when,this._structureContextKeys),Up(e)){if(e.command.precondition&&rv._fillInKbExprKeys(e.command.precondition,this._preconditionContextKeys),e.command.toggled){const t=e.command.toggled.condition||e.command.toggled;rv._fillInKbExprKeys(t,this._toggledContextKeys)}}else this._collectContextKeysForSubmenus&&(rn.getMenuItems(e.submenu).forEach(this._collectContextKeysAndSubmenuIds,this),this._allMenuIds.add(e.submenu))}static _fillInKbExprKeys(e,t){if(e)for(const i of e.keys())t.add(i)}}let HA=NC=class extends rv{constructor(e,t,i,n,s,r){super(e,i),this._hiddenStates=t,this._commandService=n,this._keybindingService=s,this._contextKeyService=r,this.refresh()}createActionGroups(e){const t=[];for(const i of this._menuGroups){const[n,s]=i;let r;for(const a of s)if(this._contextKeyService.contextMatchesRules(a.when)){const l=Up(a);l&&this._hiddenStates.setDefaultState(this._id,a.command.id,!!a.isHiddenByDefault);const c=Lme(this._id,l?a.command:a,this._hiddenStates);if(l){const d=oq(this._commandService,this._keybindingService,a.command.id,a.when);(r??(r=[])).push(new ao(a.command,a.alt,e,c,d,this._contextKeyService,this._commandService))}else{const d=new NC(a.submenu,this._hiddenStates,this._collectContextKeysForSubmenus,this._commandService,this._keybindingService,this._contextKeyService).createActionGroups(e),h=Qi.join(...d.map(u=>u[1]));h.length>0&&(r??(r=[])).push(new $p(a,c,h))}}r&&r.length>0&&t.push([n,r])}return t}_sort(e){return e.sort(NC._compareMenuItems)}static _compareMenuItems(e,t){const i=e.group,n=t.group;if(i!==n){if(i){if(!n)return-1}else return 1;if(i==="navigation")return-1;if(n==="navigation")return 1;const a=i.localeCompare(n);if(a!==0)return a}const s=e.order||0,r=t.order||0;return s<r?-1:s>r?1:NC._compareTitles(Up(e)?e.command.title:e.title,Up(t)?t.command.title:t.title)}static _compareTitles(e,t){const i=typeof e=="string"?e:e.original,n=typeof t=="string"?t:t.original;return i.localeCompare(n)}};HA=NC=fE([Al(3,Ht),Al(4,yt),Al(5,Ee)],HA);let XL=class{constructor(e,t,i,n,s,r){this._disposables=new K,this._menuInfo=new HA(e,t,i.emitEventsForSubmenuChanges,n,s,r);const a=new It(()=>{this._menuInfo.refresh(),this._onDidChange.fire({menu:this,isStructuralChange:!0,isEnablementChange:!0,isToggleChange:!0})},i.eventDebounceDelay);this._disposables.add(a),this._disposables.add(rn.onDidChangeMenu(h=>{for(const u of this._menuInfo.allMenuIds)if(h.has(u)){a.schedule();break}}));const l=this._disposables.add(new K),c=h=>{let u=!1,g=!1,f=!1;for(const p of h)if(u=u||p.isStructuralChange,g=g||p.isEnablementChange,f=f||p.isToggleChange,u&&g&&f)break;return{menu:this,isStructuralChange:u,isEnablementChange:g,isToggleChange:f}},d=()=>{l.add(r.onDidChangeContext(h=>{const u=h.affectsSome(this._menuInfo.structureContextKeys),g=h.affectsSome(this._menuInfo.preconditionContextKeys),f=h.affectsSome(this._menuInfo.toggledContextKeys);(u||g||f)&&this._onDidChange.fire({menu:this,isStructuralChange:u,isEnablementChange:g,isToggleChange:f})})),l.add(t.onDidChange(h=>{this._onDidChange.fire({menu:this,isStructuralChange:!0,isEnablementChange:!1,isToggleChange:!1})}))};this._onDidChange=new Wz({onWillAddFirstListener:d,onDidRemoveLastListener:l.clear.bind(l),delay:i.eventDebounceDelay,merge:c}),this.onDidChange=this._onDidChange.event}getActions(e){return this._menuInfo.createActionGroups(e)}dispose(){this._disposables.dispose(),this._onDidChange.dispose()}};XL=fE([Al(3,Ht),Al(4,yt),Al(5,Ee)],XL);function Lme(o,e,t){const i=Nie(e)?e.submenu.id:e.id,n=typeof e.title=="string"?e.title:e.title.value,s=pg({id:`hide/${o.id}/${i}`,label:m("hide.label","Hide '{0}'",n),run(){t.updateHidden(o,i,!0)}}),r=pg({id:`toggle/${o.id}/${i}`,label:n,get checked(){return!t.isHidden(o,i)},run(){t.updateHidden(o,i,!!this.checked)}});return{hide:s,toggle:r,get isHidden(){return!r.checked}}}function oq(o,e,t,i=void 0,n=!0){return pg({id:`configureKeybinding/${t}`,label:m("configure keybinding","Configure Keybinding"),enabled:n,run(){const r=!!!e.lookupKeybinding(t)&&i?i.serialize():void 0;o.executeCommand("workbench.action.openGlobalKeybindings",`@command:${t}`+(r?` +when:${r}`:""))}})}var xme=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},oH=function(o,e){return function(t,i){e(t,i,o)}},VA;const rH="application/vnd.code.resources";var Rg;let zA=(Rg=class extends F{constructor(e,t){super(),this.layoutService=e,this.logService=t,this.mapTextToType=new Map,this.findText="",this.resources=[],this.resourcesStateHash=void 0,(Wh||lU)&&this.installWebKitWriteTextWorkaround(),this._register(oe.runAndSubscribe(CI,({window:i,disposables:n})=>{n.add(U(i.document,"copy",()=>this.clearResourcesState()))},{window:Rt,disposables:this._store}))}installWebKitWriteTextWorkaround(){const e=()=>{const t=new gb;this.webKitPendingClipboardWritePromise&&!this.webKitPendingClipboardWritePromise.isSettled&&this.webKitPendingClipboardWritePromise.cancel(),this.webKitPendingClipboardWritePromise=t,wC().navigator.clipboard.write([new ClipboardItem({"text/plain":t.p})]).catch(async i=>{(!(i instanceof Error)||i.name!=="NotAllowedError"||!t.isRejected)&&this.logService.error(i)})};this._register(oe.runAndSubscribe(this.layoutService.onDidAddContainer,({container:t,disposables:i})=>{i.add(U(t,"click",e)),i.add(U(t,"keydown",e))},{container:this.layoutService.mainContainer,disposables:this._store}))}async writeText(e,t){if(this.clearResourcesState(),t){this.mapTextToType.set(t,e);return}if(this.webKitPendingClipboardWritePromise)return this.webKitPendingClipboardWritePromise.complete(e);try{return await wC().navigator.clipboard.writeText(e)}catch(i){console.error(i)}this.fallbackWriteText(e)}fallbackWriteText(e){const t=fb(),i=t.activeElement,n=t.body.appendChild(ne("textarea",{"aria-hidden":!0}));n.style.height="1px",n.style.width="1px",n.style.position="absolute",n.value=e,n.focus(),n.select(),t.execCommand("copy"),Xi(i)&&i.focus(),n.remove()}async readText(e){if(e)return this.mapTextToType.get(e)||"";try{return await wC().navigator.clipboard.readText()}catch(t){console.error(t)}return""}async readFindText(){return this.findText}async writeFindText(e){this.findText=e}async readResources(){try{const t=await wC().navigator.clipboard.read();for(const i of t)if(i.types.includes(`web ${rH}`)){const n=await i.getType(`web ${rH}`);return JSON.parse(await n.text()).map(r=>Se.from(r))}}catch{}const e=await this.computeResourcesStateHash();return this.resourcesStateHash!==e&&this.clearResourcesState(),this.resources}async computeResourcesStateHash(){if(this.resources.length===0)return;const e=await this.readText();return _I(e.substring(0,VA.MAX_RESOURCE_STATE_SOURCE_LENGTH))}clearInternalState(){this.clearResourcesState()}clearResourcesState(){this.resources=[],this.resourcesStateHash=void 0}},VA=Rg,Rg.MAX_RESOURCE_STATE_SOURCE_LENGTH=1e3,Rg);zA=VA=xme([oH(0,tu),oH(1,Nn)],zA);const md=et("clipboardService");var kme=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Dme=function(o,e){return function(t,i){e(t,i,o)}};const av="data-keybinding-context";let W3=class{constructor(e,t){this._id=e,this._parent=t,this._value=Object.create(null),this._value._contextId=e}get value(){return{...this._value}}setValue(e,t){return this._value[e]!==t?(this._value[e]=t,!0):!1}removeValue(e){return e in this._value?(delete this._value[e],!0):!1}getValue(e){const t=this._value[e];return typeof t>"u"&&this._parent?this._parent.getValue(e):t}};const tD=class tD extends W3{constructor(){super(-1,null)}setValue(e,t){return!1}removeValue(e){return!1}getValue(e){}};tD.INSTANCE=new tD;let $_=tD;const Wv=class Wv extends W3{constructor(e,t,i){super(e,null),this._configurationService=t,this._values=Gp.forConfigKeys(),this._listener=this._configurationService.onDidChangeConfiguration(n=>{if(n.source===7){const s=Array.from(this._values,([r])=>r);this._values.clear(),i.fire(new lH(s))}else{const s=[];for(const r of n.affectedKeys){const a=`config.${r}`,l=this._values.findSuperstr(a);l!==void 0&&(s.push(...ot.map(l,([c])=>c)),this._values.deleteSuperstr(a)),this._values.has(a)&&(s.push(a),this._values.delete(a))}i.fire(new lH(s))}})}dispose(){this._listener.dispose()}getValue(e){if(e.indexOf(Wv._keyPrefix)!==0)return super.getValue(e);if(this._values.has(e))return this._values.get(e);const t=e.substr(Wv._keyPrefix.length),i=this._configurationService.getValue(t);let n;switch(typeof i){case"number":case"boolean":case"string":n=i;break;default:Array.isArray(i)?n=JSON.stringify(i):n=i}return this._values.set(e,n),n}setValue(e,t){return super.setValue(e,t)}removeValue(e){return super.removeValue(e)}};Wv._keyPrefix="config.";let UA=Wv;class Ime{constructor(e,t,i){this._service=e,this._key=t,this._defaultValue=i,this.reset()}set(e){this._service.setContext(this._key,e)}reset(){typeof this._defaultValue>"u"?this._service.removeContext(this._key):this._service.setContext(this._key,this._defaultValue)}get(){return this._service.getContextKeyValue(this._key)}}class aH{constructor(e){this.key=e}affectsSome(e){return e.has(this.key)}allKeysContainedIn(e){return this.affectsSome(e)}}class lH{constructor(e){this.keys=e}affectsSome(e){for(const t of this.keys)if(e.has(t))return!0;return!1}allKeysContainedIn(e){return this.keys.every(t=>e.has(t))}}class Eme{constructor(e){this.events=e}affectsSome(e){for(const t of this.events)if(t.affectsSome(e))return!0;return!1}allKeysContainedIn(e){return this.events.every(t=>t.allKeysContainedIn(e))}}function Nme(o,e){return o.allKeysContainedIn(new Set(Object.keys(e)))}class rq extends F{constructor(e){super(),this._onDidChangeContext=this._register(new Lf({merge:t=>new Eme(t)})),this.onDidChangeContext=this._onDidChangeContext.event,this._isDisposed=!1,this._myContextId=e}createKey(e,t){if(this._isDisposed)throw new Error("AbstractContextKeyService has been disposed");return new Ime(this,e,t)}bufferChangeEvents(e){this._onDidChangeContext.pause();try{e()}finally{this._onDidChangeContext.resume()}}createScoped(e){if(this._isDisposed)throw new Error("AbstractContextKeyService has been disposed");return new Tme(this,e)}contextMatchesRules(e){if(this._isDisposed)throw new Error("AbstractContextKeyService has been disposed");const t=this.getContextValuesContainer(this._myContextId);return e?e.evaluate(t):!0}getContextKeyValue(e){if(!this._isDisposed)return this.getContextValuesContainer(this._myContextId).getValue(e)}setContext(e,t){if(this._isDisposed)return;const i=this.getContextValuesContainer(this._myContextId);i&&i.setValue(e,t)&&this._onDidChangeContext.fire(new aH(e))}removeContext(e){this._isDisposed||this.getContextValuesContainer(this._myContextId).removeValue(e)&&this._onDidChangeContext.fire(new aH(e))}getContext(e){return this._isDisposed?$_.INSTANCE:this.getContextValuesContainer(Rme(e))}dispose(){super.dispose(),this._isDisposed=!0}}let $A=class extends rq{constructor(e){super(0),this._contexts=new Map,this._lastContextId=0;const t=this._register(new UA(this._myContextId,e,this._onDidChangeContext));this._contexts.set(this._myContextId,t)}getContextValuesContainer(e){return this._isDisposed?$_.INSTANCE:this._contexts.get(e)||$_.INSTANCE}createChildContext(e=this._myContextId){if(this._isDisposed)throw new Error("ContextKeyService has been disposed");const t=++this._lastContextId;return this._contexts.set(t,new W3(t,this.getContextValuesContainer(e))),t}disposeContext(e){this._isDisposed||this._contexts.delete(e)}};$A=kme([Dme(0,qe)],$A);class Tme extends rq{constructor(e,t){if(super(e.createChildContext()),this._parentChangeListener=this._register(new Ji),this._parent=e,this._updateParentChangeListener(),this._domNode=t,this._domNode.hasAttribute(av)){let i="";this._domNode.classList&&(i=Array.from(this._domNode.classList.values()).join(", ")),console.error(`Element already has context attribute${i?": "+i:""}`)}this._domNode.setAttribute(av,String(this._myContextId))}_updateParentChangeListener(){this._parentChangeListener.value=this._parent.onDidChangeContext(e=>{const i=this._parent.getContextValuesContainer(this._myContextId).value;Nme(e,i)||this._onDidChangeContext.fire(e)})}dispose(){this._isDisposed||(this._parent.disposeContext(this._myContextId),this._domNode.removeAttribute(av),super.dispose())}getContextValuesContainer(e){return this._isDisposed?$_.INSTANCE:this._parent.getContextValuesContainer(e)}createChildContext(e=this._myContextId){if(this._isDisposed)throw new Error("ScopedContextKeyService has been disposed");return this._parent.createChildContext(e)}disposeContext(e){this._isDisposed||this._parent.disposeContext(e)}}function Rme(o){for(;o;){if(o.hasAttribute(av)){const e=o.getAttribute(av);return e?parseInt(e,10):NaN}o=o.parentElement}return 0}function Mme(o,e,t){o.get(Ee).createKey(String(e),Ame(t))}function Ame(o){return Lz(o,e=>{if(typeof e=="object"&&e.$mid===1)return Se.revive(e).toString();if(e instanceof Se)return e.toString()})}rt.registerCommand("_setContext",Mme);rt.registerCommand({id:"getContextKeyInfo",handler(){return[...le.all()].sort((o,e)=>o.key.localeCompare(e.key))},metadata:{description:m("getContextKeyInfo","A command that returns information about context keys"),args:[]}});rt.registerCommand("_generateContextKeyInfo",function(){const o=[],e=new Set;for(const t of le.all())e.has(t.key)||(e.add(t.key),o.push(t));o.sort((t,i)=>t.key.localeCompare(i.key)),console.log(JSON.stringify(o,void 0,2))});let Pme=class{constructor(e,t){this.key=e,this.data=t,this.incoming=new Map,this.outgoing=new Map}};class cH{constructor(e){this._hashFn=e,this._nodes=new Map}roots(){const e=[];for(const t of this._nodes.values())t.outgoing.size===0&&e.push(t);return e}insertEdge(e,t){const i=this.lookupOrInsertNode(e),n=this.lookupOrInsertNode(t);i.outgoing.set(n.key,n),n.incoming.set(i.key,i)}removeNode(e){const t=this._hashFn(e);this._nodes.delete(t);for(const i of this._nodes.values())i.outgoing.delete(t),i.incoming.delete(t)}lookupOrInsertNode(e){const t=this._hashFn(e);let i=this._nodes.get(t);return i||(i=new Pme(t,e),this._nodes.set(t,i)),i}isEmpty(){return this._nodes.size===0}toString(){const e=[];for(const[t,i]of this._nodes)e.push(`${t}
	(-> incoming)[${[...i.incoming.keys()].join(", ")}]
	(outgoing ->)[${[...i.outgoing.keys()].join(",")}]
`);return e.join(`
`)}findCycleSlow(){for(const[e,t]of this._nodes){const i=new Set([e]),n=this._findCycle(t,i);if(n)return n}}_findCycle(e,t){for(const[i,n]of e.outgoing){if(t.has(i))return[...t,i].join(" -> ");t.add(i);const s=this._findCycle(n,t);if(s)return s;t.delete(i)}}}class Eb{constructor(...e){this._entries=new Map;for(const[t,i]of e)this.set(t,i)}set(e,t){const i=this._entries.get(e);return this._entries.set(e,t),i}get(e){return this._entries.get(e)}}const Ome=!1;class dH extends Error{constructor(e){super("cyclic dependency between services"),this.message=e.findCycleSlow()??`UNABLE to detect cycle, dumping graph: 
${e.toString()}`}}class QL{constructor(e=new Eb,t=!1,i,n=Ome){this._services=e,this._strict=t,this._parent=i,this._enableTracing=n,this._isDisposed=!1,this._servicesToMaybeDispose=new Set,this._children=new Set,this._activeInstantiations=new Set,this._services.set(we,this),this._globalGraph=n?(i==null?void 0:i._globalGraph)??new cH(s=>s):void 0}dispose(){if(!this._isDisposed){this._isDisposed=!0,Tt(this._children),this._children.clear();for(const e of this._servicesToMaybeDispose)uI(e)&&e.dispose();this._servicesToMaybeDispose.clear()}}_throwIfDisposed(){if(this._isDisposed)throw new Error("InstantiationService has been disposed")}createChild(e,t){this._throwIfDisposed();const i=this,n=new class extends QL{dispose(){i._children.delete(n),super.dispose()}}(e,this._strict,this,this._enableTracing);return this._children.add(n),t==null||t.add(n),n}invokeFunction(e,...t){this._throwIfDisposed();const i=lv.traceInvocation(this._enableTracing,e);let n=!1;try{return e({get:r=>{if(n)throw U4("service accessor is only valid during the invocation of its target method");const a=this._getOrCreateServiceInstance(r,i);if(!a)throw new Error(`[invokeFunction] unknown service '${r}'`);return a}},...t)}finally{n=!0,i.stop()}}createInstance(e,...t){this._throwIfDisposed();let i,n;return e instanceof Cl?(i=lv.traceCreation(this._enableTracing,e.ctor),n=this._createInstance(e.ctor,e.staticArguments.concat(t),i)):(i=lv.traceCreation(this._enableTracing,e),n=this._createInstance(e,t,i)),i.stop(),n}_createInstance(e,t=[],i){const n=xa.getServiceDependencies(e).sort((a,l)=>a.index-l.index),s=[];for(const a of n){const l=this._getOrCreateServiceInstance(a.id,i);l||this._throwIfStrict(`[createInstance] ${e.name} depends on UNKNOWN service ${a.id}.`,!1),s.push(l)}const r=n.length>0?n[0].index:t.length;if(t.length!==r){console.trace(`[createInstance] First service dependency of ${e.name} at position ${r+1} conflicts with ${t.length} static arguments`);const a=r-t.length;a>0?t=t.concat(new Array(a)):t=t.slice(0,r)}return Reflect.construct(e,t.concat(s))}_setCreatedServiceInstance(e,t){if(this._services.get(e)instanceof Cl)this._services.set(e,t);else if(this._parent)this._parent._setCreatedServiceInstance(e,t);else throw new Error("illegalState - setting UNKNOWN service instance")}_getServiceInstanceOrDescriptor(e){const t=this._services.get(e);return!t&&this._parent?this._parent._getServiceInstanceOrDescriptor(e):t}_getOrCreateServiceInstance(e,t){this._globalGraph&&this._globalGraphImplicitDependency&&this._globalGraph.insertEdge(this._globalGraphImplicitDependency,String(e));const i=this._getServiceInstanceOrDescriptor(e);return i instanceof Cl?this._safeCreateAndCacheServiceInstance(e,i,t.branch(e,!0)):(t.branch(e,!1),i)}_safeCreateAndCacheServiceInstance(e,t,i){if(this._activeInstantiations.has(e))throw new Error(`illegal state - RECURSIVELY instantiating service '${e}'`);this._activeInstantiations.add(e);try{return this._createAndCacheServiceInstance(e,t,i)}finally{this._activeInstantiations.delete(e)}}_createAndCacheServiceInstance(e,t,i){var l;const n=new cH(c=>c.id.toString());let s=0;const r=[{id:e,desc:t,_trace:i}],a=new Set;for(;r.length;){const c=r.pop();if(!a.has(String(c.id))){if(a.add(String(c.id)),n.lookupOrInsertNode(c),s++>1e3)throw new dH(n);for(const d of xa.getServiceDependencies(c.desc.ctor)){const h=this._getServiceInstanceOrDescriptor(d.id);if(h||this._throwIfStrict(`[createInstance] ${e} depends on ${d.id} which is NOT registered.`,!0),(l=this._globalGraph)==null||l.insertEdge(String(c.id),String(d.id)),h instanceof Cl){const u={id:d.id,desc:h,_trace:c._trace.branch(d.id,!0)};n.insertEdge(c,u),r.push(u)}}}}for(;;){const c=n.roots();if(c.length===0){if(!n.isEmpty())throw new dH(n);break}for(const{data:d}of c){if(this._getServiceInstanceOrDescriptor(d.id)instanceof Cl){const u=this._createServiceInstanceWithOwner(d.id,d.desc.ctor,d.desc.staticArguments,d.desc.supportsDelayedInstantiation,d._trace);this._setCreatedServiceInstance(d.id,u)}n.removeNode(d)}}return this._getServiceInstanceOrDescriptor(e)}_createServiceInstanceWithOwner(e,t,i=[],n,s){if(this._services.get(e)instanceof Cl)return this._createServiceInstance(e,t,i,n,s,this._servicesToMaybeDispose);if(this._parent)return this._parent._createServiceInstanceWithOwner(e,t,i,n,s);throw new Error(`illegalState - creating UNKNOWN service instance ${t.name}`)}_createServiceInstance(e,t,i=[],n,s,r){if(n){const a=new QL(void 0,this._strict,this,this._enableTracing);a._globalGraphImplicitDependency=String(e);const l=new Map,c=new nte(()=>{const d=a._createInstance(t,i,s);for(const[h,u]of l){const g=d[h];if(typeof g=="function")for(const f of u)f.disposable=g.apply(d,f.listener)}return l.clear(),r.add(d),d});return new Proxy(Object.create(null),{get(d,h){if(!c.isInitialized&&typeof h=="string"&&(h.startsWith("onDid")||h.startsWith("onWill"))){let f=l.get(h);return f||(f=new cs,l.set(h,f)),(_,b,C)=>{if(c.isInitialized)return c.value[h](_,b,C);{const v={listener:[_,b,C],disposable:void 0},w=f.push(v);return ve(()=>{var L;w(),(L=v.disposable)==null||L.dispose()})}}}if(h in d)return d[h];const u=c.value;let g=u[h];return typeof g!="function"||(g=g.bind(u),d[h]=g),g},set(d,h,u){return c.value[h]=u,!0},getPrototypeOf(d){return t.prototype}})}else{const a=this._createInstance(t,i,s);return r.add(a),a}}_throwIfStrict(e,t){if(t&&console.warn(e),this._strict)throw new Error(e)}}const Lo=class Lo{static traceInvocation(e,t){return e?new Lo(2,t.name||new Error().stack.split(`
`).slice(3,4).join(`
`)):Lo._None}static traceCreation(e,t){return e?new Lo(1,t.name):Lo._None}constructor(e,t){this.type=e,this.name=t,this._start=Date.now(),this._dep=[]}branch(e,t){const i=new Lo(3,e.toString());return this._dep.push([e,t,i]),i}stop(){const e=Date.now()-this._start;Lo._totals+=e;let t=!1;function i(s,r){const a=[],l=new Array(s+1).join("	");for(const[c,d,h]of r._dep)if(d&&h){t=!0,a.push(`${l}CREATES -> ${c}`);const u=i(s+1,h);u&&a.push(u)}else a.push(`${l}uses -> ${c}`);return a.join(`
`)}const n=[`${this.type===1?"CREATE":"CALL"} ${this.name}`,`${i(1,this)}`,`DONE, took ${e.toFixed(2)}ms (grand total ${Lo._totals.toFixed(2)}ms)`];(e>2||t)&&Lo.all.add(n.join(`
`))}};Lo.all=new Set,Lo._None=new class extends Lo{constructor(){super(0,null)}stop(){}branch(){return this}},Lo._totals=0;let lv=Lo;const Fme=new Set([Fe.inMemory,Fe.vscodeSourceControl,Fe.walkThrough,Fe.walkThroughSnippet,Fe.vscodeChatCodeBlock]);class Wme{constructor(){this._byResource=new an,this._byOwner=new Map}set(e,t,i){let n=this._byResource.get(e);n||(n=new Map,this._byResource.set(e,n)),n.set(t,i);let s=this._byOwner.get(t);s||(s=new an,this._byOwner.set(t,s)),s.set(e,i)}get(e,t){const i=this._byResource.get(e);return i==null?void 0:i.get(t)}delete(e,t){let i=!1,n=!1;const s=this._byResource.get(e);s&&(i=s.delete(t));const r=this._byOwner.get(t);if(r&&(n=r.delete(e)),i!==n)throw new Error("illegal state");return i&&n}values(e){var t,i;return typeof e=="string"?((t=this._byOwner.get(e))==null?void 0:t.values())??ot.empty():Se.isUri(e)?((i=this._byResource.get(e))==null?void 0:i.values())??ot.empty():ot.map(ot.concat(...this._byOwner.values()),n=>n[1])}}class Bme{constructor(e){this.errors=0,this.infos=0,this.warnings=0,this.unknowns=0,this._data=new an,this._service=e,this._subscription=e.onMarkerChanged(this._update,this)}dispose(){this._subscription.dispose()}_update(e){for(const t of e){const i=this._data.get(t);i&&this._substract(i);const n=this._resourceStats(t);this._add(n),this._data.set(t,n)}}_resourceStats(e){const t={errors:0,warnings:0,infos:0,unknowns:0};if(Fme.has(e.scheme))return t;for(const{severity:i}of this._service.read({resource:e}))i===pi.Error?t.errors+=1:i===pi.Warning?t.warnings+=1:i===pi.Info?t.infos+=1:t.unknowns+=1;return t}_substract(e){this.errors-=e.errors,this.warnings-=e.warnings,this.infos-=e.infos,this.unknowns-=e.unknowns}_add(e){this.errors+=e.errors,this.warnings+=e.warnings,this.infos+=e.infos,this.unknowns+=e.unknowns}}class Nd{constructor(){this._onMarkerChanged=new Wz({delay:0,merge:Nd._merge}),this.onMarkerChanged=this._onMarkerChanged.event,this._data=new Wme,this._stats=new Bme(this)}dispose(){this._stats.dispose(),this._onMarkerChanged.dispose()}remove(e,t){for(const i of t||[])this.changeOne(e,i,[])}changeOne(e,t,i){if(Cz(i))this._data.delete(t,e)&&this._onMarkerChanged.fire([t]);else{const n=[];for(const s of i){const r=Nd._toMarker(e,t,s);r&&n.push(r)}this._data.set(t,e,n),this._onMarkerChanged.fire([t])}}static _toMarker(e,t,i){let{code:n,severity:s,message:r,source:a,startLineNumber:l,startColumn:c,endLineNumber:d,endColumn:h,relatedInformation:u,tags:g}=i;if(r)return l=l>0?l:1,c=c>0?c:1,d=d>=l?d:l,h=h>0?h:c,{resource:t,owner:e,code:n,severity:s,message:r,source:a,startLineNumber:l,startColumn:c,endLineNumber:d,endColumn:h,relatedInformation:u,tags:g}}changeAll(e,t){const i=[],n=this._data.values(e);if(n)for(const s of n){const r=ot.first(s);r&&(i.push(r.resource),this._data.delete(r.resource,e))}if(Qn(t)){const s=new an;for(const{resource:r,marker:a}of t){const l=Nd._toMarker(e,r,a);if(!l)continue;const c=s.get(r);c?c.push(l):(s.set(r,[l]),i.push(r))}for(const[r,a]of s)this._data.set(r,e,a)}i.length>0&&this._onMarkerChanged.fire(i)}read(e=Object.create(null)){let{owner:t,resource:i,severities:n,take:s}=e;if((!s||s<0)&&(s=-1),t&&i){const r=this._data.get(i,t);if(r){const a=[];for(const l of r)if(Nd._accept(l,n)){const c=a.push(l);if(s>0&&c===s)break}return a}else return[]}else if(!t&&!i){const r=[];for(const a of this._data.values())for(const l of a)if(Nd._accept(l,n)){const c=r.push(l);if(s>0&&c===s)return r}return r}else{const r=this._data.values(i??t),a=[];for(const l of r)for(const c of l)if(Nd._accept(c,n)){const d=a.push(c);if(s>0&&d===s)return a}return a}}static _accept(e,t){return t===void 0||(t&e.severity)===e.severity}static _merge(e){const t=new an;for(const i of e)for(const n of i)t.set(n,!0);return Array.from(t.keys())}}class Hme extends F{get configurationModel(){return this._configurationModel}constructor(e){super(),this.logService=e,this._configurationModel=mn.createEmptyModel(this.logService)}reload(){return this.resetConfigurationModel(),this.configurationModel}getConfigurationDefaultOverrides(){return{}}resetConfigurationModel(){this._configurationModel=mn.createEmptyModel(this.logService);const e=ci.as(ta.Configuration).getConfigurationProperties();this.updateConfigurationModel(Object.keys(e),e)}updateConfigurationModel(e,t){const i=this.getConfigurationDefaultOverrides();for(const n of e){const s=i[n],r=t[n];s!==void 0?this._configurationModel.setValue(n,s):r?this._configurationModel.setValue(n,r.default):this._configurationModel.removeValue(n)}}}const ru=et("accessibilitySignalService"),_t=class _t{static register(e){return new _t(e.fileName)}constructor(e){this.fileName=e}};_t.error=_t.register({fileName:"error.mp3"}),_t.warning=_t.register({fileName:"warning.mp3"}),_t.success=_t.register({fileName:"success.mp3"}),_t.foldedArea=_t.register({fileName:"foldedAreas.mp3"}),_t.break=_t.register({fileName:"break.mp3"}),_t.quickFixes=_t.register({fileName:"quickFixes.mp3"}),_t.taskCompleted=_t.register({fileName:"taskCompleted.mp3"}),_t.taskFailed=_t.register({fileName:"taskFailed.mp3"}),_t.terminalBell=_t.register({fileName:"terminalBell.mp3"}),_t.diffLineInserted=_t.register({fileName:"diffLineInserted.mp3"}),_t.diffLineDeleted=_t.register({fileName:"diffLineDeleted.mp3"}),_t.diffLineModified=_t.register({fileName:"diffLineModified.mp3"}),_t.chatRequestSent=_t.register({fileName:"chatRequestSent.mp3"}),_t.chatResponseReceived1=_t.register({fileName:"chatResponseReceived1.mp3"}),_t.chatResponseReceived2=_t.register({fileName:"chatResponseReceived2.mp3"}),_t.chatResponseReceived3=_t.register({fileName:"chatResponseReceived3.mp3"}),_t.chatResponseReceived4=_t.register({fileName:"chatResponseReceived4.mp3"}),_t.clear=_t.register({fileName:"clear.mp3"}),_t.save=_t.register({fileName:"save.mp3"}),_t.format=_t.register({fileName:"format.mp3"}),_t.voiceRecordingStarted=_t.register({fileName:"voiceRecordingStarted.mp3"}),_t.voiceRecordingStopped=_t.register({fileName:"voiceRecordingStopped.mp3"}),_t.progress=_t.register({fileName:"progress.mp3"});let ei=_t;class Vme{constructor(e){this.randomOneOf=e}}const je=class je{constructor(e,t,i,n,s,r){this.sound=e,this.name=t,this.legacySoundSettingsKey=i,this.settingsKey=n,this.legacyAnnouncementSettingsKey=s,this.announcementMessage=r}static register(e){const t=new Vme("randomOneOf"in e.sound?e.sound.randomOneOf:[e.sound]),i=new je(t,e.name,e.legacySoundSettingsKey,e.settingsKey,e.legacyAnnouncementSettingsKey,e.announcementMessage);return je._signals.add(i),i}};je._signals=new Set,je.errorAtPosition=je.register({name:m("accessibilitySignals.positionHasError.name","Error at Position"),sound:ei.error,announcementMessage:m("accessibility.signals.positionHasError","Error"),settingsKey:"accessibility.signals.positionHasError",delaySettingsKey:"accessibility.signalOptions.delays.errorAtPosition"}),je.warningAtPosition=je.register({name:m("accessibilitySignals.positionHasWarning.name","Warning at Position"),sound:ei.warning,announcementMessage:m("accessibility.signals.positionHasWarning","Warning"),settingsKey:"accessibility.signals.positionHasWarning",delaySettingsKey:"accessibility.signalOptions.delays.warningAtPosition"}),je.errorOnLine=je.register({name:m("accessibilitySignals.lineHasError.name","Error on Line"),sound:ei.error,legacySoundSettingsKey:"audioCues.lineHasError",legacyAnnouncementSettingsKey:"accessibility.alert.error",announcementMessage:m("accessibility.signals.lineHasError","Error on Line"),settingsKey:"accessibility.signals.lineHasError"}),je.warningOnLine=je.register({name:m("accessibilitySignals.lineHasWarning.name","Warning on Line"),sound:ei.warning,legacySoundSettingsKey:"audioCues.lineHasWarning",legacyAnnouncementSettingsKey:"accessibility.alert.warning",announcementMessage:m("accessibility.signals.lineHasWarning","Warning on Line"),settingsKey:"accessibility.signals.lineHasWarning"}),je.foldedArea=je.register({name:m("accessibilitySignals.lineHasFoldedArea.name","Folded Area on Line"),sound:ei.foldedArea,legacySoundSettingsKey:"audioCues.lineHasFoldedArea",legacyAnnouncementSettingsKey:"accessibility.alert.foldedArea",announcementMessage:m("accessibility.signals.lineHasFoldedArea","Folded"),settingsKey:"accessibility.signals.lineHasFoldedArea"}),je.break=je.register({name:m("accessibilitySignals.lineHasBreakpoint.name","Breakpoint on Line"),sound:ei.break,legacySoundSettingsKey:"audioCues.lineHasBreakpoint",legacyAnnouncementSettingsKey:"accessibility.alert.breakpoint",announcementMessage:m("accessibility.signals.lineHasBreakpoint","Breakpoint"),settingsKey:"accessibility.signals.lineHasBreakpoint"}),je.inlineSuggestion=je.register({name:m("accessibilitySignals.lineHasInlineSuggestion.name","Inline Suggestion on Line"),sound:ei.quickFixes,legacySoundSettingsKey:"audioCues.lineHasInlineSuggestion",settingsKey:"accessibility.signals.lineHasInlineSuggestion"}),je.terminalQuickFix=je.register({name:m("accessibilitySignals.terminalQuickFix.name","Terminal Quick Fix"),sound:ei.quickFixes,legacySoundSettingsKey:"audioCues.terminalQuickFix",legacyAnnouncementSettingsKey:"accessibility.alert.terminalQuickFix",announcementMessage:m("accessibility.signals.terminalQuickFix","Quick Fix"),settingsKey:"accessibility.signals.terminalQuickFix"}),je.onDebugBreak=je.register({name:m("accessibilitySignals.onDebugBreak.name","Debugger Stopped on Breakpoint"),sound:ei.break,legacySoundSettingsKey:"audioCues.onDebugBreak",legacyAnnouncementSettingsKey:"accessibility.alert.onDebugBreak",announcementMessage:m("accessibility.signals.onDebugBreak","Breakpoint"),settingsKey:"accessibility.signals.onDebugBreak"}),je.noInlayHints=je.register({name:m("accessibilitySignals.noInlayHints","No Inlay Hints on Line"),sound:ei.error,legacySoundSettingsKey:"audioCues.noInlayHints",legacyAnnouncementSettingsKey:"accessibility.alert.noInlayHints",announcementMessage:m("accessibility.signals.noInlayHints","No Inlay Hints"),settingsKey:"accessibility.signals.noInlayHints"}),je.taskCompleted=je.register({name:m("accessibilitySignals.taskCompleted","Task Completed"),sound:ei.taskCompleted,legacySoundSettingsKey:"audioCues.taskCompleted",legacyAnnouncementSettingsKey:"accessibility.alert.taskCompleted",announcementMessage:m("accessibility.signals.taskCompleted","Task Completed"),settingsKey:"accessibility.signals.taskCompleted"}),je.taskFailed=je.register({name:m("accessibilitySignals.taskFailed","Task Failed"),sound:ei.taskFailed,legacySoundSettingsKey:"audioCues.taskFailed",legacyAnnouncementSettingsKey:"accessibility.alert.taskFailed",announcementMessage:m("accessibility.signals.taskFailed","Task Failed"),settingsKey:"accessibility.signals.taskFailed"}),je.terminalCommandFailed=je.register({name:m("accessibilitySignals.terminalCommandFailed","Terminal Command Failed"),sound:ei.error,legacySoundSettingsKey:"audioCues.terminalCommandFailed",legacyAnnouncementSettingsKey:"accessibility.alert.terminalCommandFailed",announcementMessage:m("accessibility.signals.terminalCommandFailed","Command Failed"),settingsKey:"accessibility.signals.terminalCommandFailed"}),je.terminalCommandSucceeded=je.register({name:m("accessibilitySignals.terminalCommandSucceeded","Terminal Command Succeeded"),sound:ei.success,announcementMessage:m("accessibility.signals.terminalCommandSucceeded","Command Succeeded"),settingsKey:"accessibility.signals.terminalCommandSucceeded"}),je.terminalBell=je.register({name:m("accessibilitySignals.terminalBell","Terminal Bell"),sound:ei.terminalBell,legacySoundSettingsKey:"audioCues.terminalBell",legacyAnnouncementSettingsKey:"accessibility.alert.terminalBell",announcementMessage:m("accessibility.signals.terminalBell","Terminal Bell"),settingsKey:"accessibility.signals.terminalBell"}),je.notebookCellCompleted=je.register({name:m("accessibilitySignals.notebookCellCompleted","Notebook Cell Completed"),sound:ei.taskCompleted,legacySoundSettingsKey:"audioCues.notebookCellCompleted",legacyAnnouncementSettingsKey:"accessibility.alert.notebookCellCompleted",announcementMessage:m("accessibility.signals.notebookCellCompleted","Notebook Cell Completed"),settingsKey:"accessibility.signals.notebookCellCompleted"}),je.notebookCellFailed=je.register({name:m("accessibilitySignals.notebookCellFailed","Notebook Cell Failed"),sound:ei.taskFailed,legacySoundSettingsKey:"audioCues.notebookCellFailed",legacyAnnouncementSettingsKey:"accessibility.alert.notebookCellFailed",announcementMessage:m("accessibility.signals.notebookCellFailed","Notebook Cell Failed"),settingsKey:"accessibility.signals.notebookCellFailed"}),je.diffLineInserted=je.register({name:m("accessibilitySignals.diffLineInserted","Diff Line Inserted"),sound:ei.diffLineInserted,legacySoundSettingsKey:"audioCues.diffLineInserted",settingsKey:"accessibility.signals.diffLineInserted"}),je.diffLineDeleted=je.register({name:m("accessibilitySignals.diffLineDeleted","Diff Line Deleted"),sound:ei.diffLineDeleted,legacySoundSettingsKey:"audioCues.diffLineDeleted",settingsKey:"accessibility.signals.diffLineDeleted"}),je.diffLineModified=je.register({name:m("accessibilitySignals.diffLineModified","Diff Line Modified"),sound:ei.diffLineModified,legacySoundSettingsKey:"audioCues.diffLineModified",settingsKey:"accessibility.signals.diffLineModified"}),je.chatRequestSent=je.register({name:m("accessibilitySignals.chatRequestSent","Chat Request Sent"),sound:ei.chatRequestSent,legacySoundSettingsKey:"audioCues.chatRequestSent",legacyAnnouncementSettingsKey:"accessibility.alert.chatRequestSent",announcementMessage:m("accessibility.signals.chatRequestSent","Chat Request Sent"),settingsKey:"accessibility.signals.chatRequestSent"}),je.chatResponseReceived=je.register({name:m("accessibilitySignals.chatResponseReceived","Chat Response Received"),legacySoundSettingsKey:"audioCues.chatResponseReceived",sound:{randomOneOf:[ei.chatResponseReceived1,ei.chatResponseReceived2,ei.chatResponseReceived3,ei.chatResponseReceived4]},settingsKey:"accessibility.signals.chatResponseReceived"}),je.progress=je.register({name:m("accessibilitySignals.progress","Progress"),sound:ei.progress,legacySoundSettingsKey:"audioCues.chatResponsePending",legacyAnnouncementSettingsKey:"accessibility.alert.progress",announcementMessage:m("accessibility.signals.progress","Progress"),settingsKey:"accessibility.signals.progress"}),je.clear=je.register({name:m("accessibilitySignals.clear","Clear"),sound:ei.clear,legacySoundSettingsKey:"audioCues.clear",legacyAnnouncementSettingsKey:"accessibility.alert.clear",announcementMessage:m("accessibility.signals.clear","Clear"),settingsKey:"accessibility.signals.clear"}),je.save=je.register({name:m("accessibilitySignals.save","Save"),sound:ei.save,legacySoundSettingsKey:"audioCues.save",legacyAnnouncementSettingsKey:"accessibility.alert.save",announcementMessage:m("accessibility.signals.save","Save"),settingsKey:"accessibility.signals.save"}),je.format=je.register({name:m("accessibilitySignals.format","Format"),sound:ei.format,legacySoundSettingsKey:"audioCues.format",legacyAnnouncementSettingsKey:"accessibility.alert.format",announcementMessage:m("accessibility.signals.format","Format"),settingsKey:"accessibility.signals.format"}),je.voiceRecordingStarted=je.register({name:m("accessibilitySignals.voiceRecordingStarted","Voice Recording Started"),sound:ei.voiceRecordingStarted,legacySoundSettingsKey:"audioCues.voiceRecordingStarted",settingsKey:"accessibility.signals.voiceRecordingStarted"}),je.voiceRecordingStopped=je.register({name:m("accessibilitySignals.voiceRecordingStopped","Voice Recording Stopped"),sound:ei.voiceRecordingStopped,legacySoundSettingsKey:"audioCues.voiceRecordingStopped",settingsKey:"accessibility.signals.voiceRecordingStopped"});let No=je;class zme extends F{constructor(e,t=[]){super(),this.logger=new Rie([e,...t]),this._register(e.onDidChangeLogLevel(i=>this.setLevel(i)))}get onDidChangeLogLevel(){return this.logger.onDidChangeLogLevel}setLevel(e){this.logger.setLevel(e)}getLevel(){return this.logger.getLevel()}trace(e,...t){this.logger.trace(e,...t)}debug(e,...t){this.logger.debug(e,...t)}info(e,...t){this.logger.info(e,...t)}warn(e,...t){this.logger.warn(e,...t)}error(e,...t){this.logger.error(e,...t)}}const aq=[];function Nb(o){aq.push(o)}function Ume(){return aq.slice(0)}class $me{getParseResult(e){}}var Ql=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Pi=function(o,e){return function(t,i){e(t,i,o)}};class jme{constructor(e){this.disposed=!1,this.model=e,this._onWillDispose=new P}get textEditorModel(){return this.model}dispose(){this.disposed=!0,this._onWillDispose.fire()}}let jA=class{constructor(e){this.modelService=e}createModelReference(e){const t=this.modelService.getModel(e);return t?Promise.resolve(new BJ(new jme(t))):Promise.reject(new Error("Model not found"))}};jA=Ql([Pi(0,Kt)],jA);const iD=class iD{show(){return iD.NULL_PROGRESS_RUNNER}async showWhile(e,t){await e}};iD.NULL_PROGRESS_RUNNER={done:()=>{},total:()=>{},worked:()=>{}};let KA=iD;class Kme{withProgress(e,t,i){return t({report:()=>{}})}}class qme{constructor(){this.isExtensionDevelopment=!1,this.isBuilt=!1}}class Gme{async confirm(e){return{confirmed:this.doConfirm(e.message,e.detail),checkboxChecked:!1}}doConfirm(e,t){let i=e;return t&&(i=i+`

`+t),Rt.confirm(i)}async prompt(e){var n;let t;if(this.doConfirm(e.message,e.detail)){const s=[...e.buttons??[]];e.cancelButton&&typeof e.cancelButton!="string"&&typeof e.cancelButton!="boolean"&&s.push(e.cancelButton),t=await((n=s[0])==null?void 0:n.run({checkboxChecked:!1}))}return{result:t}}async error(e,t){await this.prompt({type:Ii.Error,message:e,detail:t})}}const Bv=class Bv{info(e){return this.notify({severity:Ii.Info,message:e})}warn(e){return this.notify({severity:Ii.Warning,message:e})}error(e){return this.notify({severity:Ii.Error,message:e})}notify(e){switch(e.severity){case Ii.Error:console.error(e.message);break;case Ii.Warning:console.warn(e.message);break;default:console.log(e.message);break}return Bv.NO_OP}prompt(e,t,i,n){return Bv.NO_OP}status(e,t){return F.None}};Bv.NO_OP=new Qse;let qA=Bv,GA=class{constructor(e){this._onWillExecuteCommand=new P,this._onDidExecuteCommand=new P,this.onDidExecuteCommand=this._onDidExecuteCommand.event,this._instantiationService=e}executeCommand(e,...t){const i=rt.getCommand(e);if(!i)return Promise.reject(new Error(`command '${e}' not found`));try{this._onWillExecuteCommand.fire({commandId:e,args:t});const n=this._instantiationService.invokeFunction.apply(this._instantiationService,[i.handler,...t]);return this._onDidExecuteCommand.fire({commandId:e,args:t}),Promise.resolve(n)}catch(n){return Promise.reject(n)}}};GA=Ql([Pi(0,we)],GA);let j_=class extends mce{constructor(e,t,i,n,s,r){super(e,t,i,n,s),this._cachedResolver=null,this._dynamicKeybindings=[],this._domNodeListeners=[];const a=g=>{const f=new K;f.add(U(g,ie.KEY_DOWN,p=>{const _=new Pt(p);this._dispatch(_,_.target)&&(_.preventDefault(),_.stopPropagation())})),f.add(U(g,ie.KEY_UP,p=>{const _=new Pt(p);this._singleModifierDispatch(_,_.target)&&_.preventDefault()})),this._domNodeListeners.push(new Zme(g,f))},l=g=>{for(let f=0;f<this._domNodeListeners.length;f++){const p=this._domNodeListeners[f];p.domNode===g&&(this._domNodeListeners.splice(f,1),p.dispose())}},c=g=>{g.getOption(61)||a(g.getContainerDomNode())},d=g=>{g.getOption(61)||l(g.getContainerDomNode())};this._register(r.onCodeEditorAdd(c)),this._register(r.onCodeEditorRemove(d)),r.listCodeEditors().forEach(c);const h=g=>{a(g.getContainerDomNode())},u=g=>{l(g.getContainerDomNode())};this._register(r.onDiffEditorAdd(h)),this._register(r.onDiffEditorRemove(u)),r.listDiffEditors().forEach(h)}addDynamicKeybinding(e,t,i,n){return jr(rt.registerCommand(e,i),this.addDynamicKeybindings([{keybinding:t,command:e,when:n}]))}addDynamicKeybindings(e){const t=e.map(i=>({keybinding:wM(i.keybinding,so),command:i.command??null,commandArgs:i.commandArgs,when:i.when,weight1:1e3,weight2:0,extensionId:null,isBuiltinExtension:!1}));return this._dynamicKeybindings=this._dynamicKeybindings.concat(t),this.updateResolver(),ve(()=>{for(let i=0;i<this._dynamicKeybindings.length;i++)if(this._dynamicKeybindings[i]===t[0]){this._dynamicKeybindings.splice(i,t.length),this.updateResolver();return}})}updateResolver(){this._cachedResolver=null,this._onDidUpdateKeybindings.fire()}_getResolver(){if(!this._cachedResolver){const e=this._toNormalizedKeybindingItems(bs.getDefaultKeybindings(),!0),t=this._toNormalizedKeybindingItems(this._dynamicKeybindings,!1);this._cachedResolver=new XC(e,t,i=>this._log(i))}return this._cachedResolver}_documentHasFocus(){return Rt.document.hasFocus()}_toNormalizedKeybindingItems(e,t){const i=[];let n=0;for(const s of e){const r=s.when||void 0,a=s.keybinding;if(!a)i[n++]=new Q9(void 0,s.command,s.commandArgs,r,t,null,!1);else{const l=xw.resolveKeybinding(a,so);for(const c of l)i[n++]=new Q9(c,s.command,s.commandArgs,r,t,null,!1)}}return i}resolveKeyboardEvent(e){const t=new Jc(e.ctrlKey,e.shiftKey,e.altKey,e.metaKey,e.keyCode);return new xw([t],so)}};j_=Ql([Pi(0,Ee),Pi(1,Ht),Pi(2,Bn),Pi(3,Ei),Pi(4,Nn),Pi(5,pt)],j_);class Zme extends F{constructor(e,t){super(),this.domNode=e,this._register(t)}}function hH(o){return o&&typeof o=="object"&&(!o.overrideIdentifier||typeof o.overrideIdentifier=="string")&&(!o.resource||o.resource instanceof Se)}let JL=class{constructor(e){this.logService=e,this._onDidChangeConfiguration=new P,this.onDidChangeConfiguration=this._onDidChangeConfiguration.event;const t=new Hme(e);this._configuration=new QI(t.reload(),mn.createEmptyModel(e),mn.createEmptyModel(e),mn.createEmptyModel(e),mn.createEmptyModel(e),mn.createEmptyModel(e),new an,mn.createEmptyModel(e),new an,e),t.dispose()}getValue(e,t){const i=typeof e=="string"?e:void 0,n=hH(e)?e:hH(t)?t:{};return this._configuration.getValue(i,n,void 0)}updateValues(e){const t={data:this._configuration.toData()},i=[];for(const n of e){const[s,r]=n;this.getValue(s)!==r&&(this._configuration.updateValue(s,r),i.push(s))}if(i.length>0){const n=new dce({keys:i,overrides:[]},t,this._configuration,void 0,this.logService);n.source=8,this._onDidChangeConfiguration.fire(n)}return Promise.resolve()}updateValue(e,t,i,n){return this.updateValues([[e,t]])}inspect(e,t={}){return this._configuration.inspect(e,t,void 0)}};JL=Ql([Pi(0,Nn)],JL);let ZA=class{constructor(e,t,i){this.configurationService=e,this.modelService=t,this.languageService=i,this._onDidChangeConfiguration=new P,this.configurationService.onDidChangeConfiguration(n=>{this._onDidChangeConfiguration.fire({affectedKeys:n.affectedKeys,affectsConfiguration:(s,r)=>n.affectsConfiguration(r)})})}getValue(e,t,i){const n=A.isIPosition(t)?t:null,s=n?typeof i=="string"?i:void 0:typeof t=="string"?t:void 0,r=e?this.getLanguage(e,n):void 0;return typeof s>"u"?this.configurationService.getValue({resource:e,overrideIdentifier:r}):this.configurationService.getValue(s,{resource:e,overrideIdentifier:r})}getLanguage(e,t){const i=this.modelService.getModel(e);return i?t?i.getLanguageIdAtPosition(t.lineNumber,t.column):i.getLanguageId():this.languageService.guessLanguageIdByFilepathOrFirstLine(e)}};ZA=Ql([Pi(0,qe),Pi(1,Kt),Pi(2,ni)],ZA);let YA=class{constructor(e){this.configurationService=e}getEOL(e,t){const i=this.configurationService.getValue("files.eol",{overrideIdentifier:t,resource:e});return i&&typeof i=="string"&&i!=="auto"?i:ps||Je?`
`:`\r
`}};YA=Ql([Pi(0,qe)],YA);class Yme{publicLog2(){}}const Hv=class Hv{constructor(){const e=Se.from({scheme:Hv.SCHEME,authority:"model",path:"/"});this.workspace={id:pj,folders:[new Ece({uri:e,name:"",index:0})]}}getWorkspace(){return this.workspace}getWorkspaceFolder(e){return e&&e.scheme===Hv.SCHEME?this.workspace.folders[0]:null}};Hv.SCHEME="inmemory";let XA=Hv;function ex(o,e,t){if(!e||!(o instanceof JL))return;const i=[];Object.keys(e).forEach(n=>{oce(n)&&i.push([`editor.${n}`,e[n]]),t&&rce(n)&&i.push([`diffEditor.${n}`,e[n]])}),i.length>0&&o.updateValues(i)}let QA=class{constructor(e){this._modelService=e}hasPreviewHandler(){return!1}async apply(e,t){const i=Array.isArray(e)?e:X5.convert(e),n=new Map;for(const a of i){if(!(a instanceof hh))throw new Error("bad edit - only text edits are supported");const l=this._modelService.getModel(a.resource);if(!l)throw new Error("bad edit - model not found");if(typeof a.versionId=="number"&&l.getVersionId()!==a.versionId)throw new Error("bad state - model changed in the meantime");let c=n.get(l);c||(c=[],n.set(l,c)),c.push(ri.replaceMove(x.lift(a.textEdit.range),a.textEdit.text))}let s=0,r=0;for(const[a,l]of n)a.pushStackElement(),a.pushEditOperations([],l,()=>[]),a.pushStackElement(),r+=1,s+=l.length;return{ariaSummary:xf(F2.bulkEditServiceSummary,s,r),isApplied:s>0}}};QA=Ql([Pi(0,Kt)],QA);class Xme{getUriLabel(e,t){return e.scheme==="file"?e.fsPath:e.path}getUriBasenameLabel(e){return vr(e)}}let JA=class extends ece{constructor(e,t){super(e),this._codeEditorService=t}showContextView(e,t,i){if(!t){const n=this._codeEditorService.getFocusedCodeEditor()||this._codeEditorService.getActiveCodeEditor();n&&(t=n.getContainerDomNode())}return super.showContextView(e,t,i)}};JA=Ql([Pi(0,tu),Pi(1,pt)],JA);class Qme{constructor(){this._neverEmitter=new P,this.onDidChangeTrust=this._neverEmitter.event}isWorkspaceTrusted(){return!0}}class Jme extends U2{constructor(){super()}}class epe extends zme{constructor(){super(new Tie)}}let eP=class extends J2{constructor(e,t,i,n,s,r){super(e,t,i,n,s,r),this.configure({blockMouse:!1})}};eP=Ql([Pi(0,Bn),Pi(1,Ei),Pi(2,ud),Pi(3,yt),Pi(4,Go),Pi(5,Ee)],eP);const tP={amdModuleId:"vs/editor/common/services/editorSimpleWorker",esmModuleLocation:void 0,label:"editorWorkerService"};let iP=class extends ZM{constructor(e,t,i,n,s){super(tP,e,t,i,n,s)}};iP=Ql([Pi(0,Kt),Pi(1,PI),Pi(2,Nn),Pi(3,di),Pi(4,ge)],iP);class tpe{async playSignal(e,t){}}st(Nn,epe,0);st(qe,JL,0);st(PI,ZA,0);st(u$,YA,0);st(Af,XA,0);st(W_,Xme,0);st(Bn,Yme,0);st(H1,Gme,0);st(L5,qme,0);st(Ei,qA,0);st(Xl,Nd,0);st(ni,Jme,0);st(Qo,Sme,0);st(Kt,gA,0);st(r3,sA,0);st(Ee,$A,0);st(mj,Kme,0);st(nu,KA,0);st(ia,zde,0);st(Us,iP,0);st(j1,QA,0);st(_j,Qme,0);st(Hs,jA,0);st(po,FA,0);st(kr,Pfe,0);st(Ht,GA,0);st(yt,j_,0);st(xr,AA,0);st(ud,JA,0);st(zs,nA,0);st(md,zA,0);st(mo,eP,0);st(Go,WA,0);st(ru,tpe,0);st(fK,$me,0);var Re;(function(o){const e=new Eb;for(const[l,c]of L7())e.set(l,c);const t=new QL(e,!0);e.set(we,t);function i(l){n||r({});const c=e.get(l);if(!c)throw new Error("Missing service "+l);return c instanceof Cl?t.invokeFunction(d=>d.get(l)):c}o.get=i;let n=!1;const s=new P;function r(l){if(n)return t;n=!0;for(const[d,h]of L7())e.get(d)||e.set(d,h);for(const d in l)if(l.hasOwnProperty(d)){const h=et(d);e.get(h)instanceof Cl&&e.set(h,l[d])}const c=Ume();for(const d of c)try{t.createInstance(d)}catch(h){Oe(h)}return s.fire(),t}o.initialize=r;function a(l){if(n)return l();const c=new K,d=c.add(s.event(()=>{d.dispose(),c.add(l())}));return c}o.withServices=a})(Re||(Re={}));function ipe(o,e){return new npe(o,e)}class npe extends nL{constructor(e,t){const i={amdModuleId:tP.amdModuleId,esmModuleLocation:tP.esmModuleLocation,label:t.label};super(i,t.keepIdleModels||!1,e),this._foreignModuleId=t.moduleId,this._foreignModuleCreateData=t.createData||null,this._foreignModuleHost=t.host||null,this._foreignProxy=null}fhr(e,t){if(!this._foreignModuleHost||typeof this._foreignModuleHost[e]!="function")return Promise.reject(new Error("Missing method "+e+" or missing main thread foreign host."));try{return Promise.resolve(this._foreignModuleHost[e].apply(this._foreignModuleHost,t))}catch(i){return Promise.reject(i)}}_getForeignProxy(){return this._foreignProxy||(this._foreignProxy=this._getProxy().then(e=>{const t=this._foreignModuleHost?gR(this._foreignModuleHost):[];return e.$loadForeignModule(this._foreignModuleId,this._foreignModuleCreateData,t).then(i=>{this._foreignModuleCreateData=null;const n=(a,l)=>e.$fmr(a,l),s=(a,l)=>function(){const c=Array.prototype.slice.call(arguments,0);return l(a,c)},r={};for(const a of i)r[a]=s(a,n);return r})})),this._foreignProxy}getProxy(){return this._getForeignProxy()}withSyncedResources(e){return this.workerWithSyncedResources(e).then(t=>this.getProxy())}}const iS={ICodeEditor:"vs.editor.ICodeEditor",IDiffEditor:"vs.editor.IDiffEditor"};class hs{constructor(e,t,i,n){this.startColumn=e,this.endColumn=t,this.className=i,this.type=n,this._lineDecorationBrand=void 0}static _equals(e,t){return e.startColumn===t.startColumn&&e.endColumn===t.endColumn&&e.className===t.className&&e.type===t.type}static equalsArr(e,t){const i=e.length,n=t.length;if(i!==n)return!1;for(let s=0;s<i;s++)if(!hs._equals(e[s],t[s]))return!1;return!0}static extractWrapped(e,t,i){if(e.length===0)return e;const n=t+1,s=i+1,r=i-t,a=[];let l=0;for(const c of e)c.endColumn<=n||c.startColumn>=s||(a[l++]=new hs(Math.max(1,c.startColumn-n+1),Math.min(r+1,c.endColumn-n+1),c.className,c.type));return a}static filter(e,t,i,n){if(e.length===0)return[];const s=[];let r=0;for(let a=0,l=e.length;a<l;a++){const c=e[a],d=c.range;if(d.endLineNumber<t||d.startLineNumber>t||d.isEmpty()&&(c.type===0||c.type===3))continue;const h=d.startLineNumber===t?d.startColumn:i,u=d.endLineNumber===t?d.endColumn:n;s[r++]=new hs(h,u,c.inlineClassName,c.type)}return s}static _typeCompare(e,t){const i=[2,0,1,3];return i[e]-i[t]}static compare(e,t){if(e.startColumn!==t.startColumn)return e.startColumn-t.startColumn;if(e.endColumn!==t.endColumn)return e.endColumn-t.endColumn;const i=hs._typeCompare(e.type,t.type);return i!==0?i:e.className!==t.className?e.className<t.className?-1:1:0}}class uH{constructor(e,t,i,n){this.startOffset=e,this.endOffset=t,this.className=i,this.metadata=n}}class tx{constructor(){this.stopOffsets=[],this.classNames=[],this.metadata=[],this.count=0}static _metadata(e){let t=0;for(let i=0,n=e.length;i<n;i++)t|=e[i];return t}consumeLowerThan(e,t,i){for(;this.count>0&&this.stopOffsets[0]<e;){let n=0;for(;n+1<this.count&&this.stopOffsets[n]===this.stopOffsets[n+1];)n++;i.push(new uH(t,this.stopOffsets[n],this.classNames.join(" "),tx._metadata(this.metadata))),t=this.stopOffsets[n]+1,this.stopOffsets.splice(0,n+1),this.classNames.splice(0,n+1),this.metadata.splice(0,n+1),this.count-=n+1}return this.count>0&&t<e&&(i.push(new uH(t,e-1,this.classNames.join(" "),tx._metadata(this.metadata))),t=e),t}insert(e,t,i){if(this.count===0||this.stopOffsets[this.count-1]<=e)this.stopOffsets.push(e),this.classNames.push(t),this.metadata.push(i);else for(let n=0;n<this.count;n++)if(this.stopOffsets[n]>=e){this.stopOffsets.splice(n,0,e),this.classNames.splice(n,0,t),this.metadata.splice(n,0,i);break}this.count++}}class spe{static normalize(e,t){if(t.length===0)return[];const i=[],n=new tx;let s=0;for(let r=0,a=t.length;r<a;r++){const l=t[r];let c=l.startColumn,d=l.endColumn;const h=l.className,u=l.type===1?2:l.type===2?4:0;if(c>1){const p=e.charCodeAt(c-2);Gi(p)&&c--}if(d>1){const p=e.charCodeAt(d-2);Gi(p)&&d--}const g=c-1,f=d-2;s=n.consumeLowerThan(g,s,i),n.count===0&&(s=g),n.insert(f,h,u)}return n.consumeLowerThan(1073741824,s,i),i}}class sn{constructor(e,t,i,n){this.endIndex=e,this.type=t,this.metadata=i,this.containsRTL=n,this._linePartBrand=void 0}isWhitespace(){return!!(this.metadata&1)}isPseudoAfter(){return!!(this.metadata&4)}}class lq{constructor(e,t){this.startOffset=e,this.endOffset=t}equals(e){return this.startOffset===e.startOffset&&this.endOffset===e.endOffset}}class au{constructor(e,t,i,n,s,r,a,l,c,d,h,u,g,f,p,_,b,C,v){this.useMonospaceOptimizations=e,this.canUseHalfwidthRightwardsArrow=t,this.lineContent=i,this.continuesWithWrappedLine=n,this.isBasicASCII=s,this.containsRTL=r,this.fauxIndentLength=a,this.lineTokens=l,this.lineDecorations=c.sort(hs.compare),this.tabSize=d,this.startVisibleColumn=h,this.spaceWidth=u,this.stopRenderingLineAfter=p,this.renderWhitespace=_==="all"?4:_==="boundary"?1:_==="selection"?2:_==="trailing"?3:0,this.renderControlCharacters=b,this.fontLigatures=C,this.selectionsOnLine=v&&v.sort((L,k)=>L.startOffset<k.startOffset?-1:1);const w=Math.abs(f-u),S=Math.abs(g-u);w<S?(this.renderSpaceWidth=f,this.renderSpaceCharCode=11825):(this.renderSpaceWidth=g,this.renderSpaceCharCode=183)}sameSelection(e){if(this.selectionsOnLine===null)return e===null;if(e===null||e.length!==this.selectionsOnLine.length)return!1;for(let t=0;t<this.selectionsOnLine.length;t++)if(!this.selectionsOnLine[t].equals(e[t]))return!1;return!0}equals(e){return this.useMonospaceOptimizations===e.useMonospaceOptimizations&&this.canUseHalfwidthRightwardsArrow===e.canUseHalfwidthRightwardsArrow&&this.lineContent===e.lineContent&&this.continuesWithWrappedLine===e.continuesWithWrappedLine&&this.isBasicASCII===e.isBasicASCII&&this.containsRTL===e.containsRTL&&this.fauxIndentLength===e.fauxIndentLength&&this.tabSize===e.tabSize&&this.startVisibleColumn===e.startVisibleColumn&&this.spaceWidth===e.spaceWidth&&this.renderSpaceWidth===e.renderSpaceWidth&&this.renderSpaceCharCode===e.renderSpaceCharCode&&this.stopRenderingLineAfter===e.stopRenderingLineAfter&&this.renderWhitespace===e.renderWhitespace&&this.renderControlCharacters===e.renderControlCharacters&&this.fontLigatures===e.fontLigatures&&hs.equalsArr(this.lineDecorations,e.lineDecorations)&&this.lineTokens.equals(e.lineTokens)&&this.sameSelection(e.selectionsOnLine)}}class cq{constructor(e,t){this.partIndex=e,this.charIndex=t}}class vl{static getPartIndex(e){return(e&4294901760)>>>16}static getCharIndex(e){return(e&65535)>>>0}constructor(e,t){this.length=e,this._data=new Uint32Array(this.length),this._horizontalOffset=new Uint32Array(this.length)}setColumnInfo(e,t,i,n){const s=(t<<16|i<<0)>>>0;this._data[e-1]=s,this._horizontalOffset[e-1]=n}getHorizontalOffset(e){return this._horizontalOffset.length===0?0:this._horizontalOffset[e-1]}charOffsetToPartData(e){return this.length===0?0:e<0?this._data[0]:e>=this.length?this._data[this.length-1]:this._data[e]}getDomPosition(e){const t=this.charOffsetToPartData(e-1),i=vl.getPartIndex(t),n=vl.getCharIndex(t);return new cq(i,n)}getColumn(e,t){return this.partDataToCharOffset(e.partIndex,t,e.charIndex)+1}partDataToCharOffset(e,t,i){if(this.length===0)return 0;const n=(e<<16|i<<0)>>>0;let s=0,r=this.length-1;for(;s+1<r;){const p=s+r>>>1,_=this._data[p];if(_===n)return p;_>n?r=p:s=p}if(s===r)return s;const a=this._data[s],l=this._data[r];if(a===n)return s;if(l===n)return r;const c=vl.getPartIndex(a),d=vl.getCharIndex(a),h=vl.getPartIndex(l);let u;c!==h?u=t:u=vl.getCharIndex(l);const g=i-d,f=u-i;return g<=f?s:r}}class nP{constructor(e,t,i){this._renderLineOutputBrand=void 0,this.characterMapping=e,this.containsRTL=t,this.containsForeignElements=i}}function nS(o,e){if(o.lineContent.length===0){if(o.lineDecorations.length>0){e.appendString("<span>");let t=0,i=0,n=0;for(const r of o.lineDecorations)(r.type===1||r.type===2)&&(e.appendString('<span class="'),e.appendString(r.className),e.appendString('"></span>'),r.type===1&&(n|=1,t++),r.type===2&&(n|=2,i++));e.appendString("</span>");const s=new vl(1,t+i);return s.setColumnInfo(1,t,0,0),new nP(s,!1,n)}return e.appendString("<span><span></span></span>"),new nP(new vl(0,0),!1,0)}return gpe(ape(o),e)}class ope{constructor(e,t,i,n){this.characterMapping=e,this.html=t,this.containsRTL=i,this.containsForeignElements=n}}function mE(o){const e=new _b(1e4),t=nS(o,e);return new ope(t.characterMapping,e.build(),t.containsRTL,t.containsForeignElements)}class rpe{constructor(e,t,i,n,s,r,a,l,c,d,h,u,g,f,p,_){this.fontIsMonospace=e,this.canUseHalfwidthRightwardsArrow=t,this.lineContent=i,this.len=n,this.isOverflowing=s,this.overflowingCharCount=r,this.parts=a,this.containsForeignElements=l,this.fauxIndentLength=c,this.tabSize=d,this.startVisibleColumn=h,this.containsRTL=u,this.spaceWidth=g,this.renderSpaceCharCode=f,this.renderWhitespace=p,this.renderControlCharacters=_}}function ape(o){const e=o.lineContent;let t,i,n;o.stopRenderingLineAfter!==-1&&o.stopRenderingLineAfter<e.length?(t=!0,i=e.length-o.stopRenderingLineAfter,n=o.stopRenderingLineAfter):(t=!1,i=0,n=e.length);let s=lpe(e,o.containsRTL,o.lineTokens,o.fauxIndentLength,n);o.renderControlCharacters&&!o.isBasicASCII&&(s=dpe(e,s)),(o.renderWhitespace===4||o.renderWhitespace===1||o.renderWhitespace===2&&o.selectionsOnLine||o.renderWhitespace===3&&!o.continuesWithWrappedLine)&&(s=hpe(o,e,n,s));let r=0;if(o.lineDecorations.length>0){for(let a=0,l=o.lineDecorations.length;a<l;a++){const c=o.lineDecorations[a];c.type===3||c.type===1?r|=1:c.type===2&&(r|=2)}s=upe(e,n,s,o.lineDecorations)}return o.containsRTL||(s=cpe(e,s,!o.isBasicASCII||o.fontLigatures)),new rpe(o.useMonospaceOptimizations,o.canUseHalfwidthRightwardsArrow,e,n,t,i,s,r,o.fauxIndentLength,o.tabSize,o.startVisibleColumn,o.containsRTL,o.spaceWidth,o.renderSpaceCharCode,o.renderWhitespace,o.renderControlCharacters)}function lpe(o,e,t,i,n){const s=[];let r=0;i>0&&(s[r++]=new sn(i,"",0,!1));let a=i;for(let l=0,c=t.getCount();l<c;l++){const d=t.getEndOffset(l);if(d<=i)continue;const h=t.getClassName(l);if(d>=n){const g=e?k_(o.substring(a,n)):!1;s[r++]=new sn(n,h,0,g);break}const u=e?k_(o.substring(a,d)):!1;s[r++]=new sn(d,h,0,u),a=d}return s}function cpe(o,e,t){let i=0;const n=[];let s=0;if(t)for(let r=0,a=e.length;r<a;r++){const l=e[r],c=l.endIndex;if(i+50<c){const d=l.type,h=l.metadata,u=l.containsRTL;let g=-1,f=i;for(let p=i;p<c;p++)o.charCodeAt(p)===32&&(g=p),g!==-1&&p-f>=50&&(n[s++]=new sn(g+1,d,h,u),f=g+1,g=-1);f!==c&&(n[s++]=new sn(c,d,h,u))}else n[s++]=l;i=c}else for(let r=0,a=e.length;r<a;r++){const l=e[r],c=l.endIndex,d=c-i;if(d>50){const h=l.type,u=l.metadata,g=l.containsRTL,f=Math.ceil(d/50);for(let p=1;p<f;p++){const _=i+p*50;n[s++]=new sn(_,h,u,g)}n[s++]=new sn(c,h,u,g)}else n[s++]=l;i=c}return n}function dq(o){return o<32?o!==9:o===127||o>=8234&&o<=8238||o>=8294&&o<=8297||o>=8206&&o<=8207||o===1564}function dpe(o,e){const t=[];let i=new sn(0,"",0,!1),n=0;for(const s of e){const r=s.endIndex;for(;n<r;n++){const a=o.charCodeAt(n);dq(a)&&(n>i.endIndex&&(i=new sn(n,s.type,s.metadata,s.containsRTL),t.push(i)),i=new sn(n+1,"mtkcontrol",s.metadata,!1),t.push(i))}n>i.endIndex&&(i=new sn(r,s.type,s.metadata,s.containsRTL),t.push(i))}return t}function hpe(o,e,t,i){const n=o.continuesWithWrappedLine,s=o.fauxIndentLength,r=o.tabSize,a=o.startVisibleColumn,l=o.useMonospaceOptimizations,c=o.selectionsOnLine,d=o.renderWhitespace===1,h=o.renderWhitespace===3,u=o.renderSpaceWidth!==o.spaceWidth,g=[];let f=0,p=0,_=i[p].type,b=i[p].containsRTL,C=i[p].endIndex;const v=i.length;let w=!1,S=Jn(e),L;S===-1?(w=!0,S=t,L=t):L=Na(e);let k=!1,D=0,N=c&&c[D],B=a%r;for(let V=s;V<t;V++){const $=e.charCodeAt(V);N&&V>=N.endOffset&&(D++,N=c&&c[D]);let H;if(V<S||V>L)H=!0;else if($===9)H=!0;else if($===32)if(d)if(k)H=!0;else{const j=V+1<t?e.charCodeAt(V+1):0;H=j===32||j===9}else H=!0;else H=!1;if(H&&c&&(H=!!N&&N.startOffset<=V&&N.endOffset>V),H&&h&&(H=w||V>L),H&&b&&V>=S&&V<=L&&(H=!1),k){if(!H||!l&&B>=r){if(u){const j=f>0?g[f-1].endIndex:s;for(let J=j+1;J<=V;J++)g[f++]=new sn(J,"mtkw",1,!1)}else g[f++]=new sn(V,"mtkw",1,!1);B=B%r}}else(V===C||H&&V>s)&&(g[f++]=new sn(V,_,0,b),B=B%r);for($===9?B=r:Fh($)?B+=2:B++,k=H;V===C&&(p++,p<v);)_=i[p].type,b=i[p].containsRTL,C=i[p].endIndex}let W=!1;if(k)if(n&&d){const V=t>0?e.charCodeAt(t-1):0,$=t>1?e.charCodeAt(t-2):0;V===32&&$!==32&&$!==9||(W=!0)}else W=!0;if(W)if(u){const V=f>0?g[f-1].endIndex:s;for(let $=V+1;$<=t;$++)g[f++]=new sn($,"mtkw",1,!1)}else g[f++]=new sn(t,"mtkw",1,!1);else g[f++]=new sn(t,_,0,b);return g}function upe(o,e,t,i){i.sort(hs.compare);const n=spe.normalize(o,i),s=n.length;let r=0;const a=[];let l=0,c=0;for(let h=0,u=t.length;h<u;h++){const g=t[h],f=g.endIndex,p=g.type,_=g.metadata,b=g.containsRTL;for(;r<s&&n[r].startOffset<f;){const C=n[r];if(C.startOffset>c&&(c=C.startOffset,a[l++]=new sn(c,p,_,b)),C.endOffset+1<=f)c=C.endOffset+1,a[l++]=new sn(c,p+" "+C.className,_|C.metadata,b),r++;else{c=f,a[l++]=new sn(c,p+" "+C.className,_|C.metadata,b);break}}f>c&&(c=f,a[l++]=new sn(c,p,_,b))}const d=t[t.length-1].endIndex;if(r<s&&n[r].startOffset===d)for(;r<s&&n[r].startOffset===d;){const h=n[r];a[l++]=new sn(c,h.className,h.metadata,!1),r++}return a}function gpe(o,e){const t=o.fontIsMonospace,i=o.canUseHalfwidthRightwardsArrow,n=o.containsForeignElements,s=o.lineContent,r=o.len,a=o.isOverflowing,l=o.overflowingCharCount,c=o.parts,d=o.fauxIndentLength,h=o.tabSize,u=o.startVisibleColumn,g=o.containsRTL,f=o.spaceWidth,p=o.renderSpaceCharCode,_=o.renderWhitespace,b=o.renderControlCharacters,C=new vl(r+1,c.length);let v=!1,w=0,S=u,L=0,k=0,D=0;g?e.appendString('<span dir="ltr">'):e.appendString("<span>");for(let N=0,B=c.length;N<B;N++){const W=c[N],V=W.endIndex,$=W.type,H=W.containsRTL,j=_!==0&&W.isWhitespace(),J=j&&!t&&($==="mtkw"||!n),he=w===V&&W.isPseudoAfter();if(L=0,e.appendString("<span "),H&&e.appendString('style="unicode-bidi:isolate" '),e.appendString('class="'),e.appendString(J?"mtkz":$),e.appendASCIICharCode(34),j){let fe=0;{let ue=w,Pe=S;for(;ue<V;ue++){const it=(s.charCodeAt(ue)===9?h-Pe%h:1)|0;fe+=it,ue>=d&&(Pe+=it)}}for(J&&(e.appendString(' style="width:'),e.appendString(String(f*fe)),e.appendString('px"')),e.appendASCIICharCode(62);w<V;w++){C.setColumnInfo(w+1,N-D,L,k),D=0;const ue=s.charCodeAt(w);let Pe,_e;if(ue===9){Pe=h-S%h|0,_e=Pe,!i||_e>1?e.appendCharCode(8594):e.appendCharCode(65515);for(let it=2;it<=_e;it++)e.appendCharCode(160)}else Pe=2,_e=1,e.appendCharCode(p),e.appendCharCode(8204);L+=Pe,k+=_e,w>=d&&(S+=_e)}}else for(e.appendASCIICharCode(62);w<V;w++){C.setColumnInfo(w+1,N-D,L,k),D=0;const fe=s.charCodeAt(w);let ue=1,Pe=1;switch(fe){case 9:ue=h-S%h,Pe=ue;for(let _e=1;_e<=ue;_e++)e.appendCharCode(160);break;case 32:e.appendCharCode(160);break;case 60:e.appendString("&lt;");break;case 62:e.appendString("&gt;");break;case 38:e.appendString("&amp;");break;case 0:b?e.appendCharCode(9216):e.appendString("&#00;");break;case 65279:case 8232:case 8233:case 133:e.appendCharCode(65533);break;default:Fh(fe)&&Pe++,b&&fe<32?e.appendCharCode(9216+fe):b&&fe===127?e.appendCharCode(9249):b&&dq(fe)?(e.appendString("[U+"),e.appendString(fpe(fe)),e.appendString("]"),ue=8,Pe=ue):e.appendCharCode(fe)}L+=ue,k+=Pe,w>=d&&(S+=Pe)}he?D++:D=0,w>=r&&!v&&W.isPseudoAfter()&&(v=!0,C.setColumnInfo(w+1,N,L,k)),e.appendString("</span>")}return v||C.setColumnInfo(r+1,c.length-1,L,k),a&&(e.appendString('<span class="mtkoverflow">'),e.appendString(m("showMore","Show more ({0})",mpe(l))),e.appendString("</span>")),e.appendString("</span>"),new nP(C,g,n)}function fpe(o){return o.toString(16).toUpperCase().padStart(4,"0")}function mpe(o){return o<1024?m("overflow.chars","{0} chars",o):o<1024*1024?`${(o/1024).toFixed(1)} KB`:`${(o/1024/1024).toFixed(1)} MB`}class gH{constructor(e,t,i,n){this._viewportBrand=void 0,this.top=e|0,this.left=t|0,this.width=i|0,this.height=n|0}}class ppe{constructor(e,t){this.tabSize=e,this.data=t}}class B3{constructor(e,t,i,n,s,r,a){this._viewLineDataBrand=void 0,this.content=e,this.continuesWithWrappedLine=t,this.minColumn=i,this.maxColumn=n,this.startVisibleColumn=s,this.tokens=r,this.inlineDecorations=a}}class Ko{constructor(e,t,i,n,s,r,a,l,c,d){this.minColumn=e,this.maxColumn=t,this.content=i,this.continuesWithWrappedLine=n,this.isBasicASCII=Ko.isBasicASCII(i,r),this.containsRTL=Ko.containsRTL(i,this.isBasicASCII,s),this.tokens=a,this.inlineDecorations=l,this.tabSize=c,this.startVisibleColumn=d}static isBasicASCII(e,t){return t?P1(e):!0}static containsRTL(e,t,i){return!t&&i?k_(e):!1}}class cv{constructor(e,t,i){this.range=e,this.inlineClassName=t,this.type=i}}class _pe{constructor(e,t,i,n){this.startOffset=e,this.endOffset=t,this.inlineClassName=i,this.inlineClassNameAffectsLetterSpacing=n}toInlineDecoration(e){return new cv(new x(e,this.startOffset+1,e,this.endOffset+1),this.inlineClassName,this.inlineClassNameAffectsLetterSpacing?3:0)}}class hq{constructor(e,t){this._viewModelDecorationBrand=void 0,this.range=e,this.options=t}}class Hw{constructor(e,t,i){this.color=e,this.zIndex=t,this.data=i}static compareByRenderingProps(e,t){return e.zIndex===t.zIndex?e.color<t.color?-1:e.color>t.color?1:0:e.zIndex-t.zIndex}static equals(e,t){return e.color===t.color&&e.zIndex===t.zIndex&&ii(e.data,t.data)}static equalsArr(e,t){return ii(e,t,Hw.equals)}}function bpe(o){return Array.isArray(o)}function Cpe(o){return!bpe(o)}function uq(o){return typeof o=="string"}function fH(o){return!uq(o)}function Zu(o){return!o}function qc(o,e){return o.ignoreCase&&e?e.toLowerCase():e}function mH(o){return o.replace(/[&<>'"_]/g,"-")}function vpe(o,e){console.log(`${o.languageId}: ${e}`)}function Zt(o,e){return new Error(`${o.languageId}: ${e}`)}function Hd(o,e,t,i,n){const s=/\$((\$)|(#)|(\d\d?)|[sS](\d\d?)|@(\w+))/g;let r=null;return e.replace(s,function(a,l,c,d,h,u,g,f,p){return Zu(c)?Zu(d)?!Zu(h)&&h<i.length?qc(o,i[h]):!Zu(g)&&o&&typeof o[g]=="string"?o[g]:(r===null&&(r=n.split("."),r.unshift(n)),!Zu(u)&&u<r.length?qc(o,r[u]):""):qc(o,t):"$"})}function wpe(o,e,t){const i=/\$[sS](\d\d?)/g;let n=null;return e.replace(i,function(s,r){return n===null&&(n=t.split("."),n.unshift(t)),!Zu(r)&&r<n.length?qc(o,n[r]):""})}function o0(o,e){let t=e;for(;t&&t.length>0;){const i=o.tokenizer[t];if(i)return i;const n=t.lastIndexOf(".");n<0?t=null:t=t.substr(0,n)}return null}function Spe(o,e){let t=e;for(;t&&t.length>0;){if(o.stateNames[t])return!0;const n=t.lastIndexOf(".");n<0?t=null:t=t.substr(0,n)}return!1}var ype=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Lpe=function(o,e){return function(t,i){e(t,i,o)}},sP;const gq=5,nD=class nD{static create(e,t){return this._INSTANCE.create(e,t)}constructor(e){this._maxCacheDepth=e,this._entries=Object.create(null)}create(e,t){if(e!==null&&e.depth>=this._maxCacheDepth)return new i_(e,t);let i=i_.getStackElementId(e);i.length>0&&(i+="|"),i+=t;let n=this._entries[i];return n||(n=new i_(e,t),this._entries[i]=n,n)}};nD._INSTANCE=new nD(gq);let Vw=nD;class i_{constructor(e,t){this.parent=e,this.state=t,this.depth=(this.parent?this.parent.depth:0)+1}static getStackElementId(e){let t="";for(;e!==null;)t.length>0&&(t+="|"),t+=e.state,e=e.parent;return t}static _equals(e,t){for(;e!==null&&t!==null;){if(e===t)return!0;if(e.state!==t.state)return!1;e=e.parent,t=t.parent}return e===null&&t===null}equals(e){return i_._equals(this,e)}push(e){return Vw.create(this,e)}pop(){return this.parent}popall(){let e=this;for(;e.parent;)e=e.parent;return e}switchTo(e){return Vw.create(this.parent,e)}}class dp{constructor(e,t){this.languageId=e,this.state=t}equals(e){return this.languageId===e.languageId&&this.state.equals(e.state)}clone(){return this.state.clone()===this.state?this:new dp(this.languageId,this.state)}}const sD=class sD{static create(e,t){return this._INSTANCE.create(e,t)}constructor(e){this._maxCacheDepth=e,this._entries=Object.create(null)}create(e,t){if(t!==null)return new dv(e,t);if(e!==null&&e.depth>=this._maxCacheDepth)return new dv(e,t);const i=i_.getStackElementId(e);let n=this._entries[i];return n||(n=new dv(e,null),this._entries[i]=n,n)}};sD._INSTANCE=new sD(gq);let Vd=sD;class dv{constructor(e,t){this.stack=e,this.embeddedLanguageData=t}clone(){return(this.embeddedLanguageData?this.embeddedLanguageData.clone():null)===this.embeddedLanguageData?this:Vd.create(this.stack,this.embeddedLanguageData)}equals(e){return!(e instanceof dv)||!this.stack.equals(e.stack)?!1:this.embeddedLanguageData===null&&e.embeddedLanguageData===null?!0:this.embeddedLanguageData===null||e.embeddedLanguageData===null?!1:this.embeddedLanguageData.equals(e.embeddedLanguageData)}}class xpe{constructor(){this._tokens=[],this._languageId=null,this._lastTokenType=null,this._lastTokenLanguage=null}enterLanguage(e){this._languageId=e}emit(e,t){this._lastTokenType===t&&this._lastTokenLanguage===this._languageId||(this._lastTokenType=t,this._lastTokenLanguage=this._languageId,this._tokens.push(new ew(e,t,this._languageId)))}nestedLanguageTokenize(e,t,i,n){const s=i.languageId,r=i.state,a=fi.get(s);if(!a)return this.enterLanguage(s),this.emit(n,""),r;const l=a.tokenize(e,t,r);if(n!==0)for(const c of l.tokens)this._tokens.push(new ew(c.offset+n,c.type,c.language));else this._tokens=this._tokens.concat(l.tokens);return this._lastTokenType=null,this._lastTokenLanguage=null,this._languageId=null,l.endState}finalize(e){return new G4(this._tokens,e)}}class ix{constructor(e,t){this._languageService=e,this._theme=t,this._prependTokens=null,this._tokens=[],this._currentLanguageId=0,this._lastTokenMetadata=0}enterLanguage(e){this._currentLanguageId=this._languageService.languageIdCodec.encodeLanguageId(e)}emit(e,t){const i=this._theme.match(this._currentLanguageId,t)|1024;this._lastTokenMetadata!==i&&(this._lastTokenMetadata=i,this._tokens.push(e),this._tokens.push(i))}static _merge(e,t,i){const n=e!==null?e.length:0,s=t.length,r=i!==null?i.length:0;if(n===0&&s===0&&r===0)return new Uint32Array(0);if(n===0&&s===0)return i;if(s===0&&r===0)return e;const a=new Uint32Array(n+s+r);e!==null&&a.set(e);for(let l=0;l<s;l++)a[n+l]=t[l];return i!==null&&a.set(i,n+s),a}nestedLanguageTokenize(e,t,i,n){const s=i.languageId,r=i.state,a=fi.get(s);if(!a)return this.enterLanguage(s),this.emit(n,""),r;const l=a.tokenizeEncoded(e,t,r);if(n!==0)for(let c=0,d=l.tokens.length;c<d;c+=2)l.tokens[c]+=n;return this._prependTokens=ix._merge(this._prependTokens,this._tokens,l.tokens),this._tokens=[],this._currentLanguageId=0,this._lastTokenMetadata=0,l.endState}finalize(e){return new gI(ix._merge(this._prependTokens,this._tokens,null),e)}}let zw=sP=class extends F{constructor(e,t,i,n,s){super(),this._configurationService=s,this._languageService=e,this._standaloneThemeService=t,this._languageId=i,this._lexer=n,this._embeddedLanguages=Object.create(null),this.embeddedLoaded=Promise.resolve(void 0);let r=!1;this._register(fi.onDidChange(a=>{if(r)return;let l=!1;for(let c=0,d=a.changedLanguages.length;c<d;c++){const h=a.changedLanguages[c];if(this._embeddedLanguages[h]){l=!0;break}}l&&(r=!0,fi.handleChange([this._languageId]),r=!1)})),this._maxTokenizationLineLength=this._configurationService.getValue("editor.maxTokenizationLineLength",{overrideIdentifier:this._languageId}),this._register(this._configurationService.onDidChangeConfiguration(a=>{a.affectsConfiguration("editor.maxTokenizationLineLength")&&(this._maxTokenizationLineLength=this._configurationService.getValue("editor.maxTokenizationLineLength",{overrideIdentifier:this._languageId}))}))}getLoadStatus(){const e=[];for(const t in this._embeddedLanguages){const i=fi.get(t);if(i){if(i instanceof sP){const n=i.getLoadStatus();n.loaded===!1&&e.push(n.promise)}continue}fi.isResolved(t)||e.push(fi.getOrCreate(t))}return e.length===0?{loaded:!0}:{loaded:!1,promise:Promise.all(e).then(t=>{})}}getInitialState(){const e=Vw.create(null,this._lexer.start);return Vd.create(e,null)}tokenize(e,t,i){if(e.length>=this._maxTokenizationLineLength)return Z5(this._languageId,i);const n=new xpe,s=this._tokenize(e,t,i,n);return n.finalize(s)}tokenizeEncoded(e,t,i){if(e.length>=this._maxTokenizationLineLength)return XI(this._languageService.languageIdCodec.encodeLanguageId(this._languageId),i);const n=new ix(this._languageService,this._standaloneThemeService.getColorTheme().tokenTheme),s=this._tokenize(e,t,i,n);return n.finalize(s)}_tokenize(e,t,i,n){return i.embeddedLanguageData?this._nestedTokenize(e,t,i,0,n):this._myTokenize(e,t,i,0,n)}_findLeavingNestedLanguageOffset(e,t){let i=this._lexer.tokenizer[t.stack.state];if(!i&&(i=o0(this._lexer,t.stack.state),!i))throw Zt(this._lexer,"tokenizer state is not defined: "+t.stack.state);let n=-1,s=!1;for(const r of i){if(!fH(r.action)||r.action.nextEmbedded!=="@pop")continue;s=!0;let a=r.resolveRegex(t.stack.state);const l=a.source;if(l.substr(0,4)==="^(?:"&&l.substr(l.length-1,1)===")"){const d=(a.ignoreCase?"i":"")+(a.unicode?"u":"");a=new RegExp(l.substr(4,l.length-5),d)}const c=e.search(a);c===-1||c!==0&&r.matchOnlyAtLineStart||(n===-1||c<n)&&(n=c)}if(!s)throw Zt(this._lexer,'no rule containing nextEmbedded: "@pop" in tokenizer embedded state: '+t.stack.state);return n}_nestedTokenize(e,t,i,n,s){const r=this._findLeavingNestedLanguageOffset(e,i);if(r===-1){const c=s.nestedLanguageTokenize(e,t,i.embeddedLanguageData,n);return Vd.create(i.stack,new dp(i.embeddedLanguageData.languageId,c))}const a=e.substring(0,r);a.length>0&&s.nestedLanguageTokenize(a,!1,i.embeddedLanguageData,n);const l=e.substring(r);return this._myTokenize(l,t,i,n+r,s)}_safeRuleName(e){return e?e.name:"(unknown)"}_myTokenize(e,t,i,n,s){s.enterLanguage(this._languageId);const r=e.length,a=t&&this._lexer.includeLF?e+`
`:e,l=a.length;let c=i.embeddedLanguageData,d=i.stack,h=0,u=null,g=!0;for(;g||h<l;){const f=h,p=d.depth,_=u?u.groups.length:0,b=d.state;let C=null,v=null,w=null,S=null,L=null;if(u){C=u.matches;const N=u.groups.shift();v=N.matched,w=N.action,S=u.rule,u.groups.length===0&&(u=null)}else{if(!g&&h>=l)break;g=!1;let N=this._lexer.tokenizer[b];if(!N&&(N=o0(this._lexer,b),!N))throw Zt(this._lexer,"tokenizer state is not defined: "+b);const B=a.substr(h);for(const W of N)if((h===0||!W.matchOnlyAtLineStart)&&(C=B.match(W.resolveRegex(b)),C)){v=C[0],w=W.action;break}}if(C||(C=[""],v=""),w||(h<l&&(C=[a.charAt(h)],v=C[0]),w=this._lexer.defaultToken),v===null)break;for(h+=v.length;Cpe(w)&&fH(w)&&w.test;)w=w.test(v,C,b,h===l);let k=null;if(typeof w=="string"||Array.isArray(w))k=w;else if(w.group)k=w.group;else if(w.token!==null&&w.token!==void 0){if(w.tokenSubst?k=Hd(this._lexer,w.token,v,C,b):k=w.token,w.nextEmbedded)if(w.nextEmbedded==="@pop"){if(!c)throw Zt(this._lexer,"cannot pop embedded language if not inside one");c=null}else{if(c)throw Zt(this._lexer,"cannot enter embedded language from within an embedded language");L=Hd(this._lexer,w.nextEmbedded,v,C,b)}if(w.goBack&&(h=Math.max(0,h-w.goBack)),w.switchTo&&typeof w.switchTo=="string"){let N=Hd(this._lexer,w.switchTo,v,C,b);if(N[0]==="@"&&(N=N.substr(1)),o0(this._lexer,N))d=d.switchTo(N);else throw Zt(this._lexer,"trying to switch to a state '"+N+"' that is undefined in rule: "+this._safeRuleName(S))}else{if(w.transform&&typeof w.transform=="function")throw Zt(this._lexer,"action.transform not supported");if(w.next)if(w.next==="@push"){if(d.depth>=this._lexer.maxStack)throw Zt(this._lexer,"maximum tokenizer stack size reached: ["+d.state+","+d.parent.state+",...]");d=d.push(b)}else if(w.next==="@pop"){if(d.depth<=1)throw Zt(this._lexer,"trying to pop an empty stack in rule: "+this._safeRuleName(S));d=d.pop()}else if(w.next==="@popall")d=d.popall();else{let N=Hd(this._lexer,w.next,v,C,b);if(N[0]==="@"&&(N=N.substr(1)),o0(this._lexer,N))d=d.push(N);else throw Zt(this._lexer,"trying to set a next state '"+N+"' that is undefined in rule: "+this._safeRuleName(S))}}w.log&&typeof w.log=="string"&&vpe(this._lexer,this._lexer.languageId+": "+Hd(this._lexer,w.log,v,C,b))}if(k===null)throw Zt(this._lexer,"lexer rule has no well-defined action in rule: "+this._safeRuleName(S));const D=N=>{const B=this._languageService.getLanguageIdByLanguageName(N)||this._languageService.getLanguageIdByMimeType(N)||N,W=this._getNestedEmbeddedLanguageData(B);if(h<l){const V=e.substr(h);return this._nestedTokenize(V,t,Vd.create(d,W),n+h,s)}else return Vd.create(d,W)};if(Array.isArray(k)){if(u&&u.groups.length>0)throw Zt(this._lexer,"groups cannot be nested: "+this._safeRuleName(S));if(C.length!==k.length+1)throw Zt(this._lexer,"matched number of groups does not match the number of actions in rule: "+this._safeRuleName(S));let N=0;for(let B=1;B<C.length;B++)N+=C[B].length;if(N!==v.length)throw Zt(this._lexer,"with groups, all characters should be matched in consecutive groups in rule: "+this._safeRuleName(S));u={rule:S,matches:C,groups:[]};for(let B=0;B<k.length;B++)u.groups[B]={action:k[B],matched:C[B+1]};h-=v.length;continue}else{if(k==="@rematch"&&(h-=v.length,v="",C=null,k="",L!==null))return D(L);if(v.length===0){if(l===0||p!==d.depth||b!==d.state||(u?u.groups.length:0)!==_)continue;throw Zt(this._lexer,"no progress in tokenizer in rule: "+this._safeRuleName(S))}let N=null;if(uq(k)&&k.indexOf("@brackets")===0){const B=k.substr(9),W=kpe(this._lexer,v);if(!W)throw Zt(this._lexer,"@brackets token returned but no bracket defined as: "+v);N=mH(W.token+B)}else{const B=k===""?"":k+this._lexer.tokenPostfix;N=mH(B)}f<r&&s.emit(f+n,N)}if(L!==null)return D(L)}return Vd.create(d,c)}_getNestedEmbeddedLanguageData(e){if(!this._languageService.isRegisteredLanguageId(e))return new dp(e,F_);e!==this._languageId&&(this._languageService.requestBasicLanguageFeatures(e),fi.getOrCreate(e),this._embeddedLanguages[e]=!0);const t=fi.get(e);return t?new dp(e,t.getInitialState()):new dp(e,F_)}};zw=sP=ype([Lpe(4,qe)],zw);function kpe(o,e){if(!e)return null;e=qc(o,e);const t=o.brackets;for(const i of t){if(i.open===e)return{token:i.token,bracketType:1};if(i.close===e)return{token:i.token,bracketType:-1}}return null}const mT=hd("standaloneColorizer",{createHTML:o=>o});class H3{static colorizeElement(e,t,i,n){n=n||{};const s=n.theme||"vs",r=n.mimeType||i.getAttribute("lang")||i.getAttribute("data-lang");if(!r)return console.error("Mode not detected"),Promise.resolve();const a=t.getLanguageIdByMimeType(r)||r;e.setTheme(s);const l=i.firstChild?i.firstChild.nodeValue:"";i.className+=" "+s;const c=d=>{const h=(mT==null?void 0:mT.createHTML(d))??d;i.innerHTML=h};return this.colorize(t,l||"",a,n).then(c,d=>console.error(d))}static async colorize(e,t,i,n){const s=e.languageIdCodec;let r=4;n&&typeof n.tabSize=="number"&&(r=n.tabSize),t5(t)&&(t=t.substr(1));const a=Kl(t);if(!e.isRegisteredLanguageId(i))return pH(a,r,s);const l=await fi.getOrCreate(i);return l?Dpe(a,r,l,s):pH(a,r,s)}static colorizeLine(e,t,i,n,s=4){const r=Ko.isBasicASCII(e,t),a=Ko.containsRTL(e,r,i);return mE(new au(!1,!0,e,!1,r,a,0,n,[],s,0,0,0,0,-1,"none",!1,!1,null)).html}static colorizeModelLine(e,t,i=4){const n=e.getLineContent(t);e.tokenization.forceTokenization(t);const r=e.tokenization.getLineTokens(t).inflate();return this.colorizeLine(n,e.mightContainNonBasicASCII(),e.mightContainRTL(),r,i)}}function Dpe(o,e,t,i){return new Promise((n,s)=>{const r=()=>{const a=Ipe(o,e,t,i);if(t instanceof zw){const l=t.getLoadStatus();if(l.loaded===!1){l.promise.then(r,s);return}}n(a)};r()})}function pH(o,e,t){let i=[];const s=new Uint32Array(2);s[0]=0,s[1]=33587200;for(let r=0,a=o.length;r<a;r++){const l=o[r];s[0]=l.length;const c=new cn(s,l,t),d=Ko.isBasicASCII(l,!0),h=Ko.containsRTL(l,d,!0),u=mE(new au(!1,!0,l,!1,d,h,0,c,[],e,0,0,0,0,-1,"none",!1,!1,null));i=i.concat(u.html),i.push("<br/>")}return i.join("")}function Ipe(o,e,t,i){let n=[],s=t.getInitialState();for(let r=0,a=o.length;r<a;r++){const l=o[r],c=t.tokenizeEncoded(l,!0,s);cn.convertToEndOffset(c.tokens,l.length);const d=new cn(c.tokens,l,i),h=Ko.isBasicASCII(l,!0),u=Ko.containsRTL(l,h,!0),g=mE(new au(!1,!0,l,!1,h,u,0,d.inflate(),[],e,0,0,0,0,-1,"none",!1,!1,null));n=n.concat(g.html),n.push("<br/>"),s=c.endState}return n.join("")}var Epe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Npe=function(o,e){return function(t,i){e(t,i,o)}},u_;let nx=(u_=class{constructor(e,t){}dispose(){}},u_.ID="editor.contrib.markerDecorations",u_);nx=Epe([Npe(1,r3)],nx);ct(nx.ID,nx,0);class fq extends F{constructor(e,t){super(),this._onDidChange=this._register(new P),this.onDidChange=this._onDidChange.event,this._referenceDomElement=e,this._width=-1,this._height=-1,this._resizeObserver=null,this.measureReferenceDomElement(!1,t)}dispose(){this.stopObserving(),super.dispose()}getWidth(){return this._width}getHeight(){return this._height}startObserving(){if(!this._resizeObserver&&this._referenceDomElement){let e=null;const t=()=>{e?this.observe({width:e.width,height:e.height}):this.observe()};let i=!1,n=!1;const s=()=>{if(i&&!n)try{i=!1,n=!0,t()}finally{go(xe(this._referenceDomElement),()=>{n=!1,s()})}};this._resizeObserver=new ResizeObserver(r=>{r&&r[0]&&r[0].contentRect?e={width:r[0].contentRect.width,height:r[0].contentRect.height}:e=null,i=!0,s()}),this._resizeObserver.observe(this._referenceDomElement)}}stopObserving(){this._resizeObserver&&(this._resizeObserver.disconnect(),this._resizeObserver=null)}observe(e){this.measureReferenceDomElement(!0,e)}measureReferenceDomElement(e,t){let i=0,n=0;t?(i=t.width,n=t.height):this._referenceDomElement&&(i=this._referenceDomElement.clientWidth,n=this._referenceDomElement.clientHeight),i=Math.max(5,i),n=Math.max(5,n),(this._width!==i||this._height!==n)&&(this._width=i,this._height=n,e&&this._onDidChange.fire())}}const Tp=class Tp{constructor(e,t){this.key=e,this.migrate=t}apply(e){const t=Tp._read(e,this.key),i=s=>Tp._read(e,s),n=(s,r)=>Tp._write(e,s,r);this.migrate(t,i,n)}static _read(e,t){if(typeof e>"u")return;const i=t.indexOf(".");if(i>=0){const n=t.substring(0,i);return this._read(e[n],t.substring(i+1))}return e[t]}static _write(e,t,i){const n=t.indexOf(".");if(n>=0){const s=t.substring(0,n);e[s]=e[s]||{},this._write(e[s],t.substring(n+1),i);return}e[t]=i}};Tp.items=[];let Uw=Tp;function Ka(o,e){Uw.items.push(new Uw(o,e))}function _o(o,e){Ka(o,(t,i,n)=>{if(typeof t<"u"){for(const[s,r]of e)if(t===s){n(o,r);return}}})}function Tpe(o){Uw.items.forEach(e=>e.apply(o))}_o("wordWrap",[[!0,"on"],[!1,"off"]]);_o("lineNumbers",[[!0,"on"],[!1,"off"]]);_o("cursorBlinking",[["visible","solid"]]);_o("renderWhitespace",[[!0,"boundary"],[!1,"none"]]);_o("renderLineHighlight",[[!0,"line"],[!1,"none"]]);_o("acceptSuggestionOnEnter",[[!0,"on"],[!1,"off"]]);_o("tabCompletion",[[!1,"off"],[!0,"onlySnippets"]]);_o("hover",[[!0,{enabled:!0}],[!1,{enabled:!1}]]);_o("parameterHints",[[!0,{enabled:!0}],[!1,{enabled:!1}]]);_o("autoIndent",[[!1,"advanced"],[!0,"full"]]);_o("matchBrackets",[[!0,"always"],[!1,"never"]]);_o("renderFinalNewline",[[!0,"on"],[!1,"off"]]);_o("cursorSmoothCaretAnimation",[[!0,"on"],[!1,"off"]]);_o("occurrencesHighlight",[[!0,"singleFile"],[!1,"off"]]);_o("wordBasedSuggestions",[[!0,"matchingDocuments"],[!1,"off"]]);Ka("autoClosingBrackets",(o,e,t)=>{o===!1&&(t("autoClosingBrackets","never"),typeof e("autoClosingQuotes")>"u"&&t("autoClosingQuotes","never"),typeof e("autoSurround")>"u"&&t("autoSurround","never"))});Ka("renderIndentGuides",(o,e,t)=>{typeof o<"u"&&(t("renderIndentGuides",void 0),typeof e("guides.indentation")>"u"&&t("guides.indentation",!!o))});Ka("highlightActiveIndentGuide",(o,e,t)=>{typeof o<"u"&&(t("highlightActiveIndentGuide",void 0),typeof e("guides.highlightActiveIndentation")>"u"&&t("guides.highlightActiveIndentation",!!o))});const Rpe={method:"showMethods",function:"showFunctions",constructor:"showConstructors",deprecated:"showDeprecated",field:"showFields",variable:"showVariables",class:"showClasses",struct:"showStructs",interface:"showInterfaces",module:"showModules",property:"showProperties",event:"showEvents",operator:"showOperators",unit:"showUnits",value:"showValues",constant:"showConstants",enum:"showEnums",enumMember:"showEnumMembers",keyword:"showKeywords",text:"showWords",color:"showColors",file:"showFiles",reference:"showReferences",folder:"showFolders",typeParameter:"showTypeParameters",snippet:"showSnippets"};Ka("suggest.filteredTypes",(o,e,t)=>{if(o&&typeof o=="object"){for(const i of Object.entries(Rpe))o[i[0]]===!1&&typeof e(`suggest.${i[1]}`)>"u"&&t(`suggest.${i[1]}`,!1);t("suggest.filteredTypes",void 0)}});Ka("quickSuggestions",(o,e,t)=>{if(typeof o=="boolean"){const i=o?"on":"off";t("quickSuggestions",{comments:i,strings:i,other:i})}});Ka("experimental.stickyScroll.enabled",(o,e,t)=>{typeof o=="boolean"&&(t("experimental.stickyScroll.enabled",void 0),typeof e("stickyScroll.enabled")>"u"&&t("stickyScroll.enabled",o))});Ka("experimental.stickyScroll.maxLineCount",(o,e,t)=>{typeof o=="number"&&(t("experimental.stickyScroll.maxLineCount",void 0),typeof e("stickyScroll.maxLineCount")>"u"&&t("stickyScroll.maxLineCount",o))});Ka("codeActionsOnSave",(o,e,t)=>{if(o&&typeof o=="object"){let i=!1;const n={};for(const s of Object.entries(o))typeof s[1]=="boolean"?(i=!0,n[s[0]]=s[1]?"explicit":"never"):n[s[0]]=s[1];i&&t("codeActionsOnSave",n)}});Ka("codeActionWidget.includeNearbyQuickfixes",(o,e,t)=>{typeof o=="boolean"&&(t("codeActionWidget.includeNearbyQuickfixes",void 0),typeof e("codeActionWidget.includeNearbyQuickFixes")>"u"&&t("codeActionWidget.includeNearbyQuickFixes",o))});Ka("lightbulb.enabled",(o,e,t)=>{typeof o=="boolean"&&t("lightbulb.enabled",o?void 0:"off")});class Mpe{constructor(){this._tabFocus=!1,this._onDidChangeTabFocus=new P,this.onDidChangeTabFocus=this._onDidChangeTabFocus.event}getTabFocusMode(){return this._tabFocus}setTabFocusMode(e){this._tabFocus=e,this._onDidChangeTabFocus.fire(this._tabFocus)}}const K_=new Mpe;var Ape=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Ppe=function(o,e){return function(t,i){e(t,i,o)}};let oP=class extends F{constructor(e,t,i,n,s){super(),this._accessibilityService=s,this._onDidChange=this._register(new P),this.onDidChange=this._onDidChange.event,this._onDidChangeFast=this._register(new P),this.onDidChangeFast=this._onDidChangeFast.event,this._isDominatedByLongLines=!1,this._viewLineCount=1,this._lineNumbersDigitCount=1,this._reservedHeight=0,this._glyphMarginDecorationLaneCount=1,this._computeOptionsMemory=new Pz,this.isSimpleWidget=e,this.contextMenuId=t,this._containerObserver=this._register(new fq(n,i.dimension)),this._targetWindowId=xe(n).vscodeWindowId,this._rawOptions=_H(i),this._validatedOptions=zd.validateOptions(this._rawOptions),this.options=this._computeOptions(),this.options.get(13)&&this._containerObserver.startObserving(),this._register(lr.onDidChangeZoomLevel(()=>this._recomputeOptions())),this._register(K_.onDidChangeTabFocus(()=>this._recomputeOptions())),this._register(this._containerObserver.onDidChange(()=>this._recomputeOptions())),this._register(RM.onDidChange(()=>this._recomputeOptions())),this._register(aw.getInstance(xe(n)).onDidChange(()=>this._recomputeOptions())),this._register(this._accessibilityService.onDidChangeScreenReaderOptimized(()=>this._recomputeOptions()))}_recomputeOptions(){const e=this._computeOptions(),t=zd.checkEquals(this.options,e);t!==null&&(this.options=e,this._onDidChangeFast.fire(t),this._onDidChange.fire(t))}_computeOptions(){const e=this._readEnvConfiguration(),t=mg.createFromValidatedSettings(this._validatedOptions,e.pixelRatio,this.isSimpleWidget),i=this._readFontInfo(t),n={memory:this._computeOptionsMemory,outerWidth:e.outerWidth,outerHeight:e.outerHeight-this._reservedHeight,fontInfo:i,extraEditorClassName:e.extraEditorClassName,isDominatedByLongLines:this._isDominatedByLongLines,viewLineCount:this._viewLineCount,lineNumbersDigitCount:this._lineNumbersDigitCount,emptySelectionClipboard:e.emptySelectionClipboard,pixelRatio:e.pixelRatio,tabFocusMode:K_.getTabFocusMode(),accessibilitySupport:e.accessibilitySupport,glyphMarginDecorationLaneCount:this._glyphMarginDecorationLaneCount};return zd.computeOptions(this._validatedOptions,n)}_readEnvConfiguration(){return{extraEditorClassName:Fpe(),outerWidth:this._containerObserver.getWidth(),outerHeight:this._containerObserver.getHeight(),emptySelectionClipboard:fI||pr,pixelRatio:aw.getInstance(c7(this._targetWindowId,!0).window).value,accessibilitySupport:this._accessibilityService.isScreenReaderOptimized()?2:this._accessibilityService.getAccessibilitySupport()}}_readFontInfo(e){return RM.readFontInfo(c7(this._targetWindowId,!0).window,e)}getRawOptions(){return this._rawOptions}updateOptions(e){const t=_H(e);zd.applyUpdate(this._rawOptions,t)&&(this._validatedOptions=zd.validateOptions(this._rawOptions),this._recomputeOptions())}observeContainer(e){this._containerObserver.observe(e)}setIsDominatedByLongLines(e){this._isDominatedByLongLines!==e&&(this._isDominatedByLongLines=e,this._recomputeOptions())}setModelLineCount(e){const t=Ope(e);this._lineNumbersDigitCount!==t&&(this._lineNumbersDigitCount=t,this._recomputeOptions())}setViewLineCount(e){this._viewLineCount!==e&&(this._viewLineCount=e,this._recomputeOptions())}setReservedHeight(e){this._reservedHeight!==e&&(this._reservedHeight=e,this._recomputeOptions())}setGlyphMarginDecorationLaneCount(e){this._glyphMarginDecorationLaneCount!==e&&(this._glyphMarginDecorationLaneCount=e,this._recomputeOptions())}};oP=Ape([Ppe(4,po)],oP);function Ope(o){let e=0;for(;o;)o=Math.floor(o/10),e++;return e||1}function Fpe(){let o="";return!Wh&&!lU&&(o+="no-user-select "),Wh&&(o+="no-minimap-shadow ",o+="enable-user-select "),Je&&(o+="mac "),o}class Wpe{constructor(){this._values=[]}_read(e){return this._values[e]}get(e){return this._values[e]}_write(e,t){this._values[e]=t}}class Bpe{constructor(){this._values=[]}_read(e){if(e>=this._values.length)throw new Error("Cannot read uninitialized value");return this._values[e]}get(e){return this._read(e)}_write(e,t){this._values[e]=t}}class zd{static validateOptions(e){const t=new Wpe;for(const i of Jm){const n=i.name==="_never_"?void 0:e[i.name];t._write(i.id,i.validate(n))}return t}static computeOptions(e,t){const i=new Bpe;for(const n of Jm)i._write(n.id,n.compute(t,i,e._read(n.id)));return i}static _deepEquals(e,t){if(typeof e!="object"||typeof t!="object"||!e||!t)return e===t;if(Array.isArray(e)||Array.isArray(t))return Array.isArray(e)&&Array.isArray(t)?ii(e,t):!1;if(Object.keys(e).length!==Object.keys(t).length)return!1;for(const i in e)if(!zd._deepEquals(e[i],t[i]))return!1;return!0}static checkEquals(e,t){const i=[];let n=!1;for(const s of Jm){const r=!zd._deepEquals(e._read(s.id),t._read(s.id));i[s.id]=r,r&&(n=!0)}return n?new Az(i):null}static applyUpdate(e,t){let i=!1;for(const n of Jm)if(t.hasOwnProperty(n.name)){const s=n.applyUpdate(e[n.name],t[n.name]);e[n.name]=s.newValue,i=i||s.didChange}return i}}function _H(o){const e=vc(o);return Tpe(e),e}var eh;(function(o){const e={total:0,min:Number.MAX_VALUE,max:0},t={...e},i={...e},n={...e};let s=0;const r={keydown:0,input:0,render:0};function a(){b(),performance.mark("inputlatency/start"),performance.mark("keydown/start"),r.keydown=1,queueMicrotask(l)}o.onKeyDown=a;function l(){r.keydown===1&&(performance.mark("keydown/end"),r.keydown=2)}function c(){performance.mark("input/start"),r.input=1,_()}o.onBeforeInput=c;function d(){r.input===0&&c(),queueMicrotask(h)}o.onInput=d;function h(){r.input===1&&(performance.mark("input/end"),r.input=2)}function u(){b()}o.onKeyUp=u;function g(){b()}o.onSelectionChange=g;function f(){r.keydown===2&&r.input===2&&r.render===0&&(performance.mark("render/start"),r.render=1,queueMicrotask(p),_())}o.onRenderStart=f;function p(){r.render===1&&(performance.mark("render/end"),r.render=2)}function _(){setTimeout(b)}function b(){r.keydown===2&&r.input===2&&r.render===2&&(performance.mark("inputlatency/end"),performance.measure("keydown","keydown/start","keydown/end"),performance.measure("input","input/start","input/end"),performance.measure("render","render/start","render/end"),performance.measure("inputlatency","inputlatency/start","inputlatency/end"),C("keydown",e),C("input",t),C("render",i),C("inputlatency",n),s++,v())}function C(k,D){const N=performance.getEntriesByName(k)[0].duration;D.total+=N,D.min=Math.min(D.min,N),D.max=Math.max(D.max,N)}function v(){performance.clearMarks("keydown/start"),performance.clearMarks("keydown/end"),performance.clearMarks("input/start"),performance.clearMarks("input/end"),performance.clearMarks("render/start"),performance.clearMarks("render/end"),performance.clearMarks("inputlatency/start"),performance.clearMarks("inputlatency/end"),performance.clearMeasures("keydown"),performance.clearMeasures("input"),performance.clearMeasures("render"),performance.clearMeasures("inputlatency"),r.keydown=0,r.input=0,r.render=0}function w(){if(s===0)return;const k={keydown:S(e),input:S(t),render:S(i),total:S(n),sampleCount:s};return L(e),L(t),L(i),L(n),s=0,k}o.getAndClearMeasurements=w;function S(k){return{average:k.total/s,max:k.max,min:k.min}}function L(k){k.total=0,k.min=Number.MAX_VALUE,k.max=0}})(eh||(eh={}));class pE{constructor(e,t){this.x=e,this.y=t,this._pageCoordinatesBrand=void 0}toClientCoordinates(e){return new mq(this.x-e.scrollX,this.y-e.scrollY)}}class mq{constructor(e,t){this.clientX=e,this.clientY=t,this._clientCoordinatesBrand=void 0}toPageCoordinates(e){return new pE(this.clientX+e.scrollX,this.clientY+e.scrollY)}}class Hpe{constructor(e,t,i,n){this.x=e,this.y=t,this.width=i,this.height=n,this._editorPagePositionBrand=void 0}}class Vpe{constructor(e,t){this.x=e,this.y=t,this._positionRelativeToEditorBrand=void 0}}function V3(o){const e=wi(o);return new Hpe(e.left,e.top,e.width,e.height)}function z3(o,e,t){const i=e.width/o.offsetWidth,n=e.height/o.offsetHeight,s=(t.x-e.x)/i,r=(t.y-e.y)/n;return new Vpe(s,r)}class qh extends Vr{constructor(e,t,i){super(xe(i),e),this._editorMouseEventBrand=void 0,this.isFromPointerCapture=t,this.pos=new pE(this.posx,this.posy),this.editorPos=V3(i),this.relativePos=z3(i,this.editorPos,this.pos)}}class zpe{constructor(e){this._editorViewDomNode=e}_create(e){return new qh(e,!1,this._editorViewDomNode)}onContextMenu(e,t){return U(e,"contextmenu",i=>{t(this._create(i))})}onMouseUp(e,t){return U(e,"mouseup",i=>{t(this._create(i))})}onMouseDown(e,t){return U(e,ie.MOUSE_DOWN,i=>{t(this._create(i))})}onPointerDown(e,t){return U(e,ie.POINTER_DOWN,i=>{t(this._create(i),i.pointerId)})}onMouseLeave(e,t){return U(e,ie.MOUSE_LEAVE,i=>{t(this._create(i))})}onMouseMove(e,t){return U(e,"mousemove",i=>t(this._create(i)))}}class Upe{constructor(e){this._editorViewDomNode=e}_create(e){return new qh(e,!1,this._editorViewDomNode)}onPointerUp(e,t){return U(e,"pointerup",i=>{t(this._create(i))})}onPointerDown(e,t){return U(e,ie.POINTER_DOWN,i=>{t(this._create(i),i.pointerId)})}onPointerLeave(e,t){return U(e,ie.POINTER_LEAVE,i=>{t(this._create(i))})}onPointerMove(e,t){return U(e,"pointermove",i=>t(this._create(i)))}}class $pe extends F{constructor(e){super(),this._editorViewDomNode=e,this._globalPointerMoveMonitor=this._register(new vb),this._keydownListener=null}startMonitoring(e,t,i,n,s){this._keydownListener=hi(e.ownerDocument,"keydown",r=>{r.toKeyCodeChord().isModifierKey()||this._globalPointerMoveMonitor.stopMonitoring(!0,r.browserEvent)},!0),this._globalPointerMoveMonitor.startMonitoring(e,t,i,r=>{n(new qh(r,!0,this._editorViewDomNode))},r=>{this._keydownListener.dispose(),s(r)})}stopMonitoring(){this._globalPointerMoveMonitor.stopMonitoring(!0)}}const oD=class oD{constructor(e){this._editor=e,this._instanceId=++oD._idPool,this._counter=0,this._rules=new Map,this._garbageCollectionScheduler=new It(()=>this.garbageCollect(),1e3)}createClassNameRef(e){const t=this.getOrCreateRule(e);return t.increaseRefCount(),{className:t.className,dispose:()=>{t.decreaseRefCount(),this._garbageCollectionScheduler.schedule()}}}getOrCreateRule(e){const t=this.computeUniqueKey(e);let i=this._rules.get(t);if(!i){const n=this._counter++;i=new jpe(t,`dyn-rule-${this._instanceId}-${n}`,jy(this._editor.getContainerDomNode())?this._editor.getContainerDomNode():void 0,e),this._rules.set(t,i)}return i}computeUniqueKey(e){return JSON.stringify(e)}garbageCollect(){for(const e of this._rules.values())e.hasReferences()||(this._rules.delete(e.key),e.dispose())}};oD._idPool=0;let sx=oD;class jpe{constructor(e,t,i,n){this.key=e,this.className=t,this.properties=n,this._referenceCount=0,this._styleElementDisposables=new K,this._styleElement=$o(i,void 0,this._styleElementDisposables),this._styleElement.textContent=this.getCssText(this.className,this.properties)}getCssText(e,t){let i=`.${e} {`;for(const n in t){const s=t[n];let r;typeof s=="object"?r=ce(s.id):r=s;const a=Kpe(n);i+=`
	${a}: ${r};`}return i+=`
}`,i}dispose(){this._styleElementDisposables.dispose(),this._styleElement=void 0}increaseRefCount(){this._referenceCount++}decreaseRefCount(){this._referenceCount--}hasReferences(){return this._referenceCount>0}}function Kpe(o){return o.replace(/(^[A-Z])/,([e])=>e.toLowerCase()).replace(/([A-Z])/g,([e])=>`-${e.toLowerCase()}`)}class sS extends F{constructor(){super(),this._shouldRender=!0}shouldRender(){return this._shouldRender}forceShouldRender(){this._shouldRender=!0}setShouldRender(){this._shouldRender=!0}onDidRender(){this._shouldRender=!1}onCompositionStart(e){return!1}onCompositionEnd(e){return!1}onConfigurationChanged(e){return!1}onCursorStateChanged(e){return!1}onDecorationsChanged(e){return!1}onFlushed(e){return!1}onFocusChanged(e){return!1}onLanguageConfigurationChanged(e){return!1}onLineMappingChanged(e){return!1}onLinesChanged(e){return!1}onLinesDeleted(e){return!1}onLinesInserted(e){return!1}onRevealRangeRequest(e){return!1}onScrollChanged(e){return!1}onThemeChanged(e){return!1}onTokensChanged(e){return!1}onTokensColorsChanged(e){return!1}onZonesChanged(e){return!1}handleEvents(e){let t=!1;for(let i=0,n=e.length;i<n;i++){const s=e[i];switch(s.type){case 0:this.onCompositionStart(s)&&(t=!0);break;case 1:this.onCompositionEnd(s)&&(t=!0);break;case 2:this.onConfigurationChanged(s)&&(t=!0);break;case 3:this.onCursorStateChanged(s)&&(t=!0);break;case 4:this.onDecorationsChanged(s)&&(t=!0);break;case 5:this.onFlushed(s)&&(t=!0);break;case 6:this.onFocusChanged(s)&&(t=!0);break;case 7:this.onLanguageConfigurationChanged(s)&&(t=!0);break;case 8:this.onLineMappingChanged(s)&&(t=!0);break;case 9:this.onLinesChanged(s)&&(t=!0);break;case 10:this.onLinesDeleted(s)&&(t=!0);break;case 11:this.onLinesInserted(s)&&(t=!0);break;case 12:this.onRevealRangeRequest(s)&&(t=!0);break;case 13:this.onScrollChanged(s)&&(t=!0);break;case 15:this.onTokensChanged(s)&&(t=!0);break;case 14:this.onThemeChanged(s)&&(t=!0);break;case 16:this.onTokensColorsChanged(s)&&(t=!0);break;case 17:this.onZonesChanged(s)&&(t=!0);break;default:console.info("View received unknown event: "),console.info(s)}}t&&(this._shouldRender=!0)}}class bo extends sS{constructor(e){super(),this._context=e,this._context.addEventHandler(this)}dispose(){this._context.removeEventHandler(this),super.dispose()}}class Va{static write(e,t){e.setAttribute("data-mprt",String(t))}static read(e){const t=e.getAttribute("data-mprt");return t===null?0:parseInt(t,10)}static collect(e,t){const i=[];let n=0;for(;e&&e!==e.ownerDocument.body&&e!==t;)e.nodeType===e.ELEMENT_NODE&&(i[n++]=this.read(e)),e=e.parentElement;const s=new Uint8Array(n);for(let r=0;r<n;r++)s[r]=i[n-r-1];return s}}class qpe{constructor(e,t){this._restrictedRenderingContextBrand=void 0,this._viewLayout=e,this.viewportData=t,this.scrollWidth=this._viewLayout.getScrollWidth(),this.scrollHeight=this._viewLayout.getScrollHeight(),this.visibleRange=this.viewportData.visibleRange,this.bigNumbersDelta=this.viewportData.bigNumbersDelta;const i=this._viewLayout.getCurrentViewport();this.scrollTop=i.top,this.scrollLeft=i.left,this.viewportWidth=i.width,this.viewportHeight=i.height}getScrolledTopFromAbsoluteTop(e){return e-this.scrollTop}getVerticalOffsetForLineNumber(e,t){return this._viewLayout.getVerticalOffsetForLineNumber(e,t)}getVerticalOffsetAfterLineNumber(e,t){return this._viewLayout.getVerticalOffsetAfterLineNumber(e,t)}getDecorationsInViewport(){return this.viewportData.getDecorationsInViewport()}}class Gpe extends qpe{constructor(e,t,i){super(e,t),this._renderingContextBrand=void 0,this._viewLines=i}linesVisibleRangesForRange(e,t){return this._viewLines.linesVisibleRangesForRange(e,t)}visibleRangeForPosition(e){return this._viewLines.visibleRangeForPosition(e)}}class Zpe{constructor(e,t,i,n){this.outsideRenderedLine=e,this.lineNumber=t,this.ranges=i,this.continuesOnNextLine=n}}class _E{static from(e){const t=new Array(e.length);for(let i=0,n=e.length;i<n;i++){const s=e[i];t[i]=new _E(s.left,s.width)}return t}constructor(e,t){this._horizontalRangeBrand=void 0,this.left=Math.round(e),this.width=Math.round(t)}toString(){return`[${this.left},${this.width}]`}}class yg{constructor(e,t){this._floatHorizontalRangeBrand=void 0,this.left=e,this.width=t}toString(){return`[${this.left},${this.width}]`}static compare(e,t){return e.left-t.left}}class Ype{constructor(e,t){this.outsideRenderedLine=e,this.originalLeft=t,this.left=Math.round(this.originalLeft)}}class bH{constructor(e,t){this.outsideRenderedLine=e,this.ranges=t}}class ay{static _createRange(){return this._handyReadyRange||(this._handyReadyRange=document.createRange()),this._handyReadyRange}static _detachRange(e,t){e.selectNodeContents(t)}static _readClientRects(e,t,i,n,s){const r=this._createRange();try{return r.setStart(e,t),r.setEnd(i,n),r.getClientRects()}catch{return null}finally{this._detachRange(r,s)}}static _mergeAdjacentRanges(e){if(e.length===1)return e;e.sort(yg.compare);const t=[];let i=0,n=e[0];for(let s=1,r=e.length;s<r;s++){const a=e[s];n.left+n.width+.9>=a.left?n.width=Math.max(n.width,a.left+a.width-n.left):(t[i++]=n,n=a)}return t[i++]=n,t}static _createHorizontalRangesFromClientRects(e,t,i){if(!e||e.length===0)return null;const n=[];for(let s=0,r=e.length;s<r;s++){const a=e[s];n[s]=new yg(Math.max(0,(a.left-t)/i),a.width/i)}return this._mergeAdjacentRanges(n)}static readHorizontalRanges(e,t,i,n,s,r){const l=e.children.length-1;if(0>l)return null;if(t=Math.min(l,Math.max(0,t)),n=Math.min(l,Math.max(0,n)),t===n&&i===s&&i===0&&!e.children[t].firstChild){const u=e.children[t].getClientRects();return r.markDidDomLayout(),this._createHorizontalRangesFromClientRects(u,r.clientRectDeltaLeft,r.clientRectScale)}t!==n&&n>0&&s===0&&(n--,s=1073741824);let c=e.children[t].firstChild,d=e.children[n].firstChild;if((!c||!d)&&(!c&&i===0&&t>0&&(c=e.children[t-1].firstChild,i=1073741824),!d&&s===0&&n>0&&(d=e.children[n-1].firstChild,s=1073741824)),!c||!d)return null;i=Math.min(c.textContent.length,Math.max(0,i)),s=Math.min(d.textContent.length,Math.max(0,s));const h=this._readClientRects(c,i,d,s,r.endNode);return r.markDidDomLayout(),this._createHorizontalRangesFromClientRects(h,r.clientRectDeltaLeft,r.clientRectScale)}}const Xpe=function(){return Pl?!0:!(ps||pr||Wh)}();let n_=!0;class CH{constructor(e,t){this.themeType=t;const i=e.options,n=i.get(50);i.get(38)==="off"?this.renderWhitespace=i.get(100):this.renderWhitespace="none",this.renderControlCharacters=i.get(95),this.spaceWidth=n.spaceWidth,this.middotWidth=n.middotWidth,this.wsmiddotWidth=n.wsmiddotWidth,this.useMonospaceOptimizations=n.isMonospace&&!i.get(33),this.canUseHalfwidthRightwardsArrow=n.canUseHalfwidthRightwardsArrow,this.lineHeight=i.get(67),this.stopRenderingLineAfter=i.get(118),this.fontLigatures=i.get(51)}equals(e){return this.themeType===e.themeType&&this.renderWhitespace===e.renderWhitespace&&this.renderControlCharacters===e.renderControlCharacters&&this.spaceWidth===e.spaceWidth&&this.middotWidth===e.middotWidth&&this.wsmiddotWidth===e.wsmiddotWidth&&this.useMonospaceOptimizations===e.useMonospaceOptimizations&&this.canUseHalfwidthRightwardsArrow===e.canUseHalfwidthRightwardsArrow&&this.lineHeight===e.lineHeight&&this.stopRenderingLineAfter===e.stopRenderingLineAfter&&this.fontLigatures===e.fontLigatures}}const rD=class rD{constructor(e){this._options=e,this._isMaybeInvalid=!0,this._renderedViewLine=null}getDomNode(){return this._renderedViewLine&&this._renderedViewLine.domNode?this._renderedViewLine.domNode.domNode:null}setDomNode(e){if(this._renderedViewLine)this._renderedViewLine.domNode=St(e);else throw new Error("I have no rendered view line to set the dom node to...")}onContentChanged(){this._isMaybeInvalid=!0}onTokensChanged(){this._isMaybeInvalid=!0}onDecorationsChanged(){this._isMaybeInvalid=!0}onOptionsChanged(e){this._isMaybeInvalid=!0,this._options=e}onSelectionChanged(){return Ta(this._options.themeType)||this._options.renderWhitespace==="selection"?(this._isMaybeInvalid=!0,!0):!1}renderLine(e,t,i,n,s){if(this._isMaybeInvalid===!1)return!1;this._isMaybeInvalid=!1;const r=n.getViewLineRenderingData(e),a=this._options,l=hs.filter(r.inlineDecorations,e,r.minColumn,r.maxColumn);let c=null;if(Ta(a.themeType)||this._options.renderWhitespace==="selection"){const g=n.selections;for(const f of g){if(f.endLineNumber<e||f.startLineNumber>e)continue;const p=f.startLineNumber===e?f.startColumn:r.minColumn,_=f.endLineNumber===e?f.endColumn:r.maxColumn;p<_&&(Ta(a.themeType)&&l.push(new hs(p,_,"inline-selected-text",0)),this._options.renderWhitespace==="selection"&&(c||(c=[]),c.push(new lq(p-1,_-1))))}}const d=new au(a.useMonospaceOptimizations,a.canUseHalfwidthRightwardsArrow,r.content,r.continuesWithWrappedLine,r.isBasicASCII,r.containsRTL,r.minColumn-1,r.tokens,l,r.tabSize,r.startVisibleColumn,a.spaceWidth,a.middotWidth,a.wsmiddotWidth,a.stopRenderingLineAfter,a.renderWhitespace,a.renderControlCharacters,a.fontLigatures!==Mh.OFF,c);if(this._renderedViewLine&&this._renderedViewLine.input.equals(d))return!1;s.appendString('<div style="top:'),s.appendString(String(t)),s.appendString("px;height:"),s.appendString(String(i)),s.appendString('px;" class="'),s.appendString(rD.CLASS_NAME),s.appendString('">');const h=nS(d,s);s.appendString("</div>");let u=null;return n_&&Xpe&&r.isBasicASCII&&a.useMonospaceOptimizations&&h.containsForeignElements===0&&(u=new r0(this._renderedViewLine?this._renderedViewLine.domNode:null,d,h.characterMapping)),u||(u=_q(this._renderedViewLine?this._renderedViewLine.domNode:null,d,h.characterMapping,h.containsRTL,h.containsForeignElements)),this._renderedViewLine=u,!0}layoutLine(e,t,i){this._renderedViewLine&&this._renderedViewLine.domNode&&(this._renderedViewLine.domNode.setTop(t),this._renderedViewLine.domNode.setHeight(i))}getWidth(e){return this._renderedViewLine?this._renderedViewLine.getWidth(e):0}getWidthIsFast(){return this._renderedViewLine?this._renderedViewLine.getWidthIsFast():!0}needsMonospaceFontCheck(){return this._renderedViewLine?this._renderedViewLine instanceof r0:!1}monospaceAssumptionsAreValid(){return this._renderedViewLine&&this._renderedViewLine instanceof r0?this._renderedViewLine.monospaceAssumptionsAreValid():n_}onMonospaceAssumptionsInvalidated(){this._renderedViewLine&&this._renderedViewLine instanceof r0&&(this._renderedViewLine=this._renderedViewLine.toSlowRenderedLine())}getVisibleRangesForRange(e,t,i,n){if(!this._renderedViewLine)return null;t=Math.min(this._renderedViewLine.input.lineContent.length+1,Math.max(1,t)),i=Math.min(this._renderedViewLine.input.lineContent.length+1,Math.max(1,i));const s=this._renderedViewLine.input.stopRenderingLineAfter;if(s!==-1&&t>s+1&&i>s+1)return new bH(!0,[new yg(this.getWidth(n),0)]);s!==-1&&t>s+1&&(t=s+1),s!==-1&&i>s+1&&(i=s+1);const r=this._renderedViewLine.getVisibleRangesForRange(e,t,i,n);return r&&r.length>0?new bH(!1,r):null}getColumnOfNodeOffset(e,t){return this._renderedViewLine?this._renderedViewLine.getColumnOfNodeOffset(e,t):1}};rD.CLASS_NAME="view-line";let Ec=rD;class r0{constructor(e,t,i){this._cachedWidth=-1,this.domNode=e,this.input=t;const n=Math.floor(t.lineContent.length/300);if(n>0){this._keyColumnPixelOffsetCache=new Float32Array(n);for(let s=0;s<n;s++)this._keyColumnPixelOffsetCache[s]=-1}else this._keyColumnPixelOffsetCache=null;this._characterMapping=i,this._charWidth=t.spaceWidth}getWidth(e){if(!this.domNode||this.input.lineContent.length<300){const t=this._characterMapping.getHorizontalOffset(this._characterMapping.length);return Math.round(this._charWidth*t)}return this._cachedWidth===-1&&(this._cachedWidth=this._getReadingTarget(this.domNode).offsetWidth,e==null||e.markDidDomLayout()),this._cachedWidth}getWidthIsFast(){return this.input.lineContent.length<300||this._cachedWidth!==-1}monospaceAssumptionsAreValid(){if(!this.domNode)return n_;if(this.input.lineContent.length<300){const e=this.getWidth(null),t=this.domNode.domNode.firstChild.offsetWidth;Math.abs(e-t)>=2&&(console.warn("monospace assumptions have been violated, therefore disabling monospace optimizations!"),n_=!1)}return n_}toSlowRenderedLine(){return _q(this.domNode,this.input,this._characterMapping,!1,0)}getVisibleRangesForRange(e,t,i,n){const s=this._getColumnPixelOffset(e,t,n),r=this._getColumnPixelOffset(e,i,n);return[new yg(s,r-s)]}_getColumnPixelOffset(e,t,i){if(t<=300){const c=this._characterMapping.getHorizontalOffset(t);return this._charWidth*c}const n=Math.floor((t-1)/300)-1,s=(n+1)*300+1;let r=-1;if(this._keyColumnPixelOffsetCache&&(r=this._keyColumnPixelOffsetCache[n],r===-1&&(r=this._actualReadPixelOffset(e,s,i),this._keyColumnPixelOffsetCache[n]=r)),r===-1){const c=this._characterMapping.getHorizontalOffset(t);return this._charWidth*c}const a=this._characterMapping.getHorizontalOffset(s),l=this._characterMapping.getHorizontalOffset(t);return r+this._charWidth*(l-a)}_getReadingTarget(e){return e.domNode.firstChild}_actualReadPixelOffset(e,t,i){if(!this.domNode)return-1;const n=this._characterMapping.getDomPosition(t),s=ay.readHorizontalRanges(this._getReadingTarget(this.domNode),n.partIndex,n.charIndex,n.partIndex,n.charIndex,i);return!s||s.length===0?-1:s[0].left}getColumnOfNodeOffset(e,t){return U3(this._characterMapping,e,t)}}class pq{constructor(e,t,i,n,s){if(this.domNode=e,this.input=t,this._characterMapping=i,this._isWhitespaceOnly=/^\s*$/.test(t.lineContent),this._containsForeignElements=s,this._cachedWidth=-1,this._pixelOffsetCache=null,!n||this._characterMapping.length===0){this._pixelOffsetCache=new Float32Array(Math.max(2,this._characterMapping.length+1));for(let r=0,a=this._characterMapping.length;r<=a;r++)this._pixelOffsetCache[r]=-1}}_getReadingTarget(e){return e.domNode.firstChild}getWidth(e){return this.domNode?(this._cachedWidth===-1&&(this._cachedWidth=this._getReadingTarget(this.domNode).offsetWidth,e==null||e.markDidDomLayout()),this._cachedWidth):0}getWidthIsFast(){return this._cachedWidth!==-1}getVisibleRangesForRange(e,t,i,n){if(!this.domNode)return null;if(this._pixelOffsetCache!==null){const s=this._readPixelOffset(this.domNode,e,t,n);if(s===-1)return null;const r=this._readPixelOffset(this.domNode,e,i,n);return r===-1?null:[new yg(s,r-s)]}return this._readVisibleRangesForRange(this.domNode,e,t,i,n)}_readVisibleRangesForRange(e,t,i,n,s){if(i===n){const r=this._readPixelOffset(e,t,i,s);return r===-1?null:[new yg(r,0)]}else return this._readRawVisibleRangesForRange(e,i,n,s)}_readPixelOffset(e,t,i,n){if(this._characterMapping.length===0){if(this._containsForeignElements===0||this._containsForeignElements===2)return 0;if(this._containsForeignElements===1)return this.getWidth(n);const s=this._getReadingTarget(e);return s.firstChild?(n.markDidDomLayout(),s.firstChild.offsetWidth):0}if(this._pixelOffsetCache!==null){const s=this._pixelOffsetCache[i];if(s!==-1)return s;const r=this._actualReadPixelOffset(e,t,i,n);return this._pixelOffsetCache[i]=r,r}return this._actualReadPixelOffset(e,t,i,n)}_actualReadPixelOffset(e,t,i,n){if(this._characterMapping.length===0){const l=ay.readHorizontalRanges(this._getReadingTarget(e),0,0,0,0,n);return!l||l.length===0?-1:l[0].left}if(i===this._characterMapping.length&&this._isWhitespaceOnly&&this._containsForeignElements===0)return this.getWidth(n);const s=this._characterMapping.getDomPosition(i),r=ay.readHorizontalRanges(this._getReadingTarget(e),s.partIndex,s.charIndex,s.partIndex,s.charIndex,n);if(!r||r.length===0)return-1;const a=r[0].left;if(this.input.isBasicASCII){const l=this._characterMapping.getHorizontalOffset(i),c=Math.round(this.input.spaceWidth*l);if(Math.abs(c-a)<=1)return c}return a}_readRawVisibleRangesForRange(e,t,i,n){if(t===1&&i===this._characterMapping.length)return[new yg(0,this.getWidth(n))];const s=this._characterMapping.getDomPosition(t),r=this._characterMapping.getDomPosition(i);return ay.readHorizontalRanges(this._getReadingTarget(e),s.partIndex,s.charIndex,r.partIndex,r.charIndex,n)}getColumnOfNodeOffset(e,t){return U3(this._characterMapping,e,t)}}class Qpe extends pq{_readVisibleRangesForRange(e,t,i,n,s){const r=super._readVisibleRangesForRange(e,t,i,n,s);if(!r||r.length===0||i===n||i===1&&n===this._characterMapping.length)return r;if(!this.input.containsRTL){const a=this._readPixelOffset(e,t,n,s);if(a!==-1){const l=r[r.length-1];l.left<a&&(l.width=a-l.left)}}return r}}const _q=function(){return fI?Jpe:e_e}();function Jpe(o,e,t,i,n){return new Qpe(o,e,t,i,n)}function e_e(o,e,t,i,n){return new pq(o,e,t,i,n)}function U3(o,e,t){const i=e.textContent.length;let n=-1;for(;e;)e=e.previousSibling,n++;return o.getColumn(new cq(n,t),i)}class $w{static whitespaceVisibleColumn(e,t,i){const n=e.length;let s=0,r=-1,a=-1;for(let l=0;l<n;l++){if(l===t)return[r,a,s];switch(s%i===0&&(r=l,a=s),e.charCodeAt(l)){case 32:s+=1;break;case 9:s=Ui.nextRenderTabStop(s,i);break;default:return[-1,-1,-1]}}return t===n?[r,a,s]:[-1,-1,-1]}static atomicPosition(e,t,i,n){const s=e.length,[r,a,l]=$w.whitespaceVisibleColumn(e,t,i);if(l===-1)return-1;let c;switch(n){case 0:c=!0;break;case 1:c=!1;break;case 2:if(l%i===0)return t;c=l%i<=i/2;break}if(c){if(r===-1)return-1;let u=a;for(let g=r;g<s;++g){if(u===a+i)return r;switch(e.charCodeAt(g)){case 32:u+=1;break;case 9:u=Ui.nextRenderTabStop(u,i);break;default:return-1}}return u===a+i?r:-1}const d=Ui.nextRenderTabStop(l,i);let h=l;for(let u=t;u<s;u++){if(h===d)return u;switch(e.charCodeAt(u)){case 32:h+=1;break;case 9:h=Ui.nextRenderTabStop(h,i);break;default:return-1}}return h===d?s:-1}}class Id{constructor(e=null){this.hitTarget=e,this.type=0}}class bq{get hitTarget(){return this.spanNode}constructor(e,t,i){this.position=e,this.spanNode=t,this.injectedText=i,this.type=1}}var Yu;(function(o){function e(t,i,n){const s=t.getPositionFromDOMInfo(i,n);return s?new bq(s,i,null):new Id(i)}o.createFromDOMInfo=e})(Yu||(Yu={}));class t_e{constructor(e,t){this.lastViewCursorsRenderData=e,this.lastTextareaPosition=t}}class Kn{static _deduceRage(e,t=null){return!t&&e?new x(e.lineNumber,e.column,e.lineNumber,e.column):t??null}static createUnknown(e,t,i){return{type:0,element:e,mouseColumn:t,position:i,range:this._deduceRage(i)}}static createTextarea(e,t){return{type:1,element:e,mouseColumn:t,position:null,range:null}}static createMargin(e,t,i,n,s,r){return{type:e,element:t,mouseColumn:i,position:n,range:s,detail:r}}static createViewZone(e,t,i,n,s){return{type:e,element:t,mouseColumn:i,position:n,range:this._deduceRage(n),detail:s}}static createContentText(e,t,i,n,s){return{type:6,element:e,mouseColumn:t,position:i,range:this._deduceRage(i,n),detail:s}}static createContentEmpty(e,t,i,n){return{type:7,element:e,mouseColumn:t,position:i,range:this._deduceRage(i),detail:n}}static createContentWidget(e,t,i){return{type:9,element:e,mouseColumn:t,position:null,range:null,detail:i}}static createScrollbar(e,t,i){return{type:11,element:e,mouseColumn:t,position:i,range:this._deduceRage(i)}}static createOverlayWidget(e,t,i){return{type:12,element:e,mouseColumn:t,position:null,range:null,detail:i}}static createOutsideEditor(e,t,i,n){return{type:13,element:null,mouseColumn:e,position:t,range:this._deduceRage(t),outsidePosition:i,outsideDistance:n}}static _typeToString(e){return e===1?"TEXTAREA":e===2?"GUTTER_GLYPH_MARGIN":e===3?"GUTTER_LINE_NUMBERS":e===4?"GUTTER_LINE_DECORATIONS":e===5?"GUTTER_VIEW_ZONE":e===6?"CONTENT_TEXT":e===7?"CONTENT_EMPTY":e===8?"CONTENT_VIEW_ZONE":e===9?"CONTENT_WIDGET":e===10?"OVERVIEW_RULER":e===11?"SCROLLBAR":e===12?"OVERLAY_WIDGET":"UNKNOWN"}static toString(e){return this._typeToString(e.type)+": "+e.position+" - "+e.range+" - "+JSON.stringify(e.detail)}}class ns{static isTextArea(e){return e.length===2&&e[0]===3&&e[1]===7}static isChildOfViewLines(e){return e.length>=4&&e[0]===3&&e[3]===8}static isStrictChildOfViewLines(e){return e.length>4&&e[0]===3&&e[3]===8}static isChildOfScrollableElement(e){return e.length>=2&&e[0]===3&&e[1]===6}static isChildOfMinimap(e){return e.length>=2&&e[0]===3&&e[1]===9}static isChildOfContentWidgets(e){return e.length>=4&&e[0]===3&&e[3]===1}static isChildOfOverflowGuard(e){return e.length>=1&&e[0]===3}static isChildOfOverflowingContentWidgets(e){return e.length>=1&&e[0]===2}static isChildOfOverlayWidgets(e){return e.length>=2&&e[0]===3&&e[1]===4}static isChildOfOverflowingOverlayWidgets(e){return e.length>=1&&e[0]===5}}class q_{constructor(e,t,i){this.viewModel=e.viewModel;const n=e.configuration.options;this.layoutInfo=n.get(146),this.viewDomNode=t.viewDomNode,this.lineHeight=n.get(67),this.stickyTabStops=n.get(117),this.typicalHalfwidthCharacterWidth=n.get(50).typicalHalfwidthCharacterWidth,this.lastRenderData=i,this._context=e,this._viewHelper=t}getZoneAtCoord(e){return q_.getZoneAtCoord(this._context,e)}static getZoneAtCoord(e,t){const i=e.viewLayout.getWhitespaceAtVerticalOffset(t);if(i){const n=i.verticalOffset+i.height/2,s=e.viewModel.getLineCount();let r=null,a,l=null;return i.afterLineNumber!==s&&(l=new A(i.afterLineNumber+1,1)),i.afterLineNumber>0&&(r=new A(i.afterLineNumber,e.viewModel.getLineMaxColumn(i.afterLineNumber))),l===null?a=r:r===null?a=l:t<n?a=r:a=l,{viewZoneId:i.id,afterLineNumber:i.afterLineNumber,positionBefore:r,positionAfter:l,position:a}}return null}getFullLineRangeAtCoord(e){if(this._context.viewLayout.isAfterLines(e)){const n=this._context.viewModel.getLineCount(),s=this._context.viewModel.getLineMaxColumn(n);return{range:new x(n,s,n,s),isAfterLines:!0}}const t=this._context.viewLayout.getLineNumberAtVerticalOffset(e),i=this._context.viewModel.getLineMaxColumn(t);return{range:new x(t,1,t,i),isAfterLines:!1}}getLineNumberAtVerticalOffset(e){return this._context.viewLayout.getLineNumberAtVerticalOffset(e)}isAfterLines(e){return this._context.viewLayout.isAfterLines(e)}isInTopPadding(e){return this._context.viewLayout.isInTopPadding(e)}isInBottomPadding(e){return this._context.viewLayout.isInBottomPadding(e)}getVerticalOffsetForLineNumber(e){return this._context.viewLayout.getVerticalOffsetForLineNumber(e)}findAttribute(e,t){return q_._findAttribute(e,t,this._viewHelper.viewDomNode)}static _findAttribute(e,t,i){for(;e&&e!==e.ownerDocument.body;){if(e.hasAttribute&&e.hasAttribute(t))return e.getAttribute(t);if(e===i)return null;e=e.parentNode}return null}getLineWidth(e){return this._viewHelper.getLineWidth(e)}visibleRangeForPosition(e,t){return this._viewHelper.visibleRangeForPosition(e,t)}getPositionFromDOMInfo(e,t){return this._viewHelper.getPositionFromDOMInfo(e,t)}getCurrentScrollTop(){return this._context.viewLayout.getCurrentScrollTop()}getCurrentScrollLeft(){return this._context.viewLayout.getCurrentScrollLeft()}}class i_e{constructor(e,t,i,n){this.editorPos=t,this.pos=i,this.relativePos=n,this.mouseVerticalOffset=Math.max(0,e.getCurrentScrollTop()+this.relativePos.y),this.mouseContentHorizontalOffset=e.getCurrentScrollLeft()+this.relativePos.x-e.layoutInfo.contentLeft,this.isInMarginArea=this.relativePos.x<e.layoutInfo.contentLeft&&this.relativePos.x>=e.layoutInfo.glyphMarginLeft,this.isInContentArea=!this.isInMarginArea,this.mouseColumn=Math.max(0,jn._getMouseColumn(this.mouseContentHorizontalOffset,e.typicalHalfwidthCharacterWidth))}}class n_e extends i_e{get target(){return this._useHitTestTarget?this.hitTestResult.value.hitTarget:this._eventTarget}get targetPath(){return this._targetPathCacheElement!==this.target&&(this._targetPathCacheElement=this.target,this._targetPathCacheValue=Va.collect(this.target,this._ctx.viewDomNode)),this._targetPathCacheValue}constructor(e,t,i,n,s){super(e,t,i,n),this.hitTestResult=new Oa(()=>jn.doHitTest(this._ctx,this)),this._targetPathCacheElement=null,this._targetPathCacheValue=new Uint8Array(0),this._ctx=e,this._eventTarget=s;const r=!!this._eventTarget;this._useHitTestTarget=!r}toString(){return`pos(${this.pos.x},${this.pos.y}), editorPos(${this.editorPos.x},${this.editorPos.y}), relativePos(${this.relativePos.x},${this.relativePos.y}), mouseVerticalOffset: ${this.mouseVerticalOffset}, mouseContentHorizontalOffset: ${this.mouseContentHorizontalOffset}
	target: ${this.target?this.target.outerHTML:null}`}get wouldBenefitFromHitTestTargetSwitch(){return!this._useHitTestTarget&&this.hitTestResult.value.hitTarget!==null&&this.target!==this.hitTestResult.value.hitTarget}switchToHitTestTarget(){this._useHitTestTarget=!0}_getMouseColumn(e=null){return e&&e.column<this._ctx.viewModel.getLineMaxColumn(e.lineNumber)?Ui.visibleColumnFromColumn(this._ctx.viewModel.getLineContent(e.lineNumber),e.column,this._ctx.viewModel.model.getOptions().tabSize)+1:this.mouseColumn}fulfillUnknown(e=null){return Kn.createUnknown(this.target,this._getMouseColumn(e),e)}fulfillTextarea(){return Kn.createTextarea(this.target,this._getMouseColumn())}fulfillMargin(e,t,i,n){return Kn.createMargin(e,this.target,this._getMouseColumn(t),t,i,n)}fulfillViewZone(e,t,i){return Kn.createViewZone(e,this.target,this._getMouseColumn(t),t,i)}fulfillContentText(e,t,i){return Kn.createContentText(this.target,this._getMouseColumn(e),e,t,i)}fulfillContentEmpty(e,t){return Kn.createContentEmpty(this.target,this._getMouseColumn(e),e,t)}fulfillContentWidget(e){return Kn.createContentWidget(this.target,this._getMouseColumn(),e)}fulfillScrollbar(e){return Kn.createScrollbar(this.target,this._getMouseColumn(e),e)}fulfillOverlayWidget(e){return Kn.createOverlayWidget(this.target,this._getMouseColumn(),e)}}const vH={isAfterLines:!0};function pT(o){return{isAfterLines:!1,horizontalDistanceToText:o}}class jn{constructor(e,t){this._context=e,this._viewHelper=t}mouseTargetIsWidget(e){const t=e.target,i=Va.collect(t,this._viewHelper.viewDomNode);return!!(ns.isChildOfContentWidgets(i)||ns.isChildOfOverflowingContentWidgets(i)||ns.isChildOfOverlayWidgets(i)||ns.isChildOfOverflowingOverlayWidgets(i))}createMouseTarget(e,t,i,n,s){const r=new q_(this._context,this._viewHelper,e),a=new n_e(r,t,i,n,s);try{const l=jn._createMouseTarget(r,a);if(l.type===6&&r.stickyTabStops&&l.position!==null){const c=jn._snapToSoftTabBoundary(l.position,r.viewModel),d=x.fromPositions(c,c).plusRange(l.range);return a.fulfillContentText(c,d,l.detail)}return l}catch{return a.fulfillUnknown()}}static _createMouseTarget(e,t){if(t.target===null)return t.fulfillUnknown();const i=t;let n=null;return!ns.isChildOfOverflowGuard(t.targetPath)&&!ns.isChildOfOverflowingContentWidgets(t.targetPath)&&!ns.isChildOfOverflowingOverlayWidgets(t.targetPath)&&(n=n||t.fulfillUnknown()),n=n||jn._hitTestContentWidget(e,i),n=n||jn._hitTestOverlayWidget(e,i),n=n||jn._hitTestMinimap(e,i),n=n||jn._hitTestScrollbarSlider(e,i),n=n||jn._hitTestViewZone(e,i),n=n||jn._hitTestMargin(e,i),n=n||jn._hitTestViewCursor(e,i),n=n||jn._hitTestTextArea(e,i),n=n||jn._hitTestViewLines(e,i),n=n||jn._hitTestScrollbar(e,i),n||t.fulfillUnknown()}static _hitTestContentWidget(e,t){if(ns.isChildOfContentWidgets(t.targetPath)||ns.isChildOfOverflowingContentWidgets(t.targetPath)){const i=e.findAttribute(t.target,"widgetId");return i?t.fulfillContentWidget(i):t.fulfillUnknown()}return null}static _hitTestOverlayWidget(e,t){if(ns.isChildOfOverlayWidgets(t.targetPath)||ns.isChildOfOverflowingOverlayWidgets(t.targetPath)){const i=e.findAttribute(t.target,"widgetId");return i?t.fulfillOverlayWidget(i):t.fulfillUnknown()}return null}static _hitTestViewCursor(e,t){if(t.target){const i=e.lastRenderData.lastViewCursorsRenderData;for(const n of i)if(t.target===n.domNode)return t.fulfillContentText(n.position,null,{mightBeForeignElement:!1,injectedText:null})}if(t.isInContentArea){const i=e.lastRenderData.lastViewCursorsRenderData,n=t.mouseContentHorizontalOffset,s=t.mouseVerticalOffset;for(const r of i){if(n<r.contentLeft||n>r.contentLeft+r.width)continue;const a=e.getVerticalOffsetForLineNumber(r.position.lineNumber);if(a<=s&&s<=a+r.height)return t.fulfillContentText(r.position,null,{mightBeForeignElement:!1,injectedText:null})}}return null}static _hitTestViewZone(e,t){const i=e.getZoneAtCoord(t.mouseVerticalOffset);if(i){const n=t.isInContentArea?8:5;return t.fulfillViewZone(n,i.position,i)}return null}static _hitTestTextArea(e,t){return ns.isTextArea(t.targetPath)?e.lastRenderData.lastTextareaPosition?t.fulfillContentText(e.lastRenderData.lastTextareaPosition,null,{mightBeForeignElement:!1,injectedText:null}):t.fulfillTextarea():null}static _hitTestMargin(e,t){if(t.isInMarginArea){const i=e.getFullLineRangeAtCoord(t.mouseVerticalOffset),n=i.range.getStartPosition();let s=Math.abs(t.relativePos.x);const r={isAfterLines:i.isAfterLines,glyphMarginLeft:e.layoutInfo.glyphMarginLeft,glyphMarginWidth:e.layoutInfo.glyphMarginWidth,lineNumbersWidth:e.layoutInfo.lineNumbersWidth,offsetX:s};if(s-=e.layoutInfo.glyphMarginLeft,s<=e.layoutInfo.glyphMarginWidth){const a=e.viewModel.coordinatesConverter.convertViewPositionToModelPosition(i.range.getStartPosition()),l=e.viewModel.glyphLanes.getLanesAtLine(a.lineNumber);return r.glyphMarginLane=l[Math.floor(s/e.lineHeight)],t.fulfillMargin(2,n,i.range,r)}return s-=e.layoutInfo.glyphMarginWidth,s<=e.layoutInfo.lineNumbersWidth?t.fulfillMargin(3,n,i.range,r):(s-=e.layoutInfo.lineNumbersWidth,t.fulfillMargin(4,n,i.range,r))}return null}static _hitTestViewLines(e,t){if(!ns.isChildOfViewLines(t.targetPath))return null;if(e.isInTopPadding(t.mouseVerticalOffset))return t.fulfillContentEmpty(new A(1,1),vH);if(e.isAfterLines(t.mouseVerticalOffset)||e.isInBottomPadding(t.mouseVerticalOffset)){const n=e.viewModel.getLineCount(),s=e.viewModel.getLineMaxColumn(n);return t.fulfillContentEmpty(new A(n,s),vH)}if(ns.isStrictChildOfViewLines(t.targetPath)){const n=e.getLineNumberAtVerticalOffset(t.mouseVerticalOffset);if(e.viewModel.getLineLength(n)===0){const r=e.getLineWidth(n),a=pT(t.mouseContentHorizontalOffset-r);return t.fulfillContentEmpty(new A(n,1),a)}const s=e.getLineWidth(n);if(t.mouseContentHorizontalOffset>=s){const r=pT(t.mouseContentHorizontalOffset-s),a=new A(n,e.viewModel.getLineMaxColumn(n));return t.fulfillContentEmpty(a,r)}}const i=t.hitTestResult.value;return i.type===1?jn.createMouseTargetFromHitTestPosition(e,t,i.spanNode,i.position,i.injectedText):t.wouldBenefitFromHitTestTargetSwitch?(t.switchToHitTestTarget(),this._createMouseTarget(e,t)):t.fulfillUnknown()}static _hitTestMinimap(e,t){if(ns.isChildOfMinimap(t.targetPath)){const i=e.getLineNumberAtVerticalOffset(t.mouseVerticalOffset),n=e.viewModel.getLineMaxColumn(i);return t.fulfillScrollbar(new A(i,n))}return null}static _hitTestScrollbarSlider(e,t){if(ns.isChildOfScrollableElement(t.targetPath)&&t.target&&t.target.nodeType===1){const i=t.target.className;if(i&&/\b(slider|scrollbar)\b/.test(i)){const n=e.getLineNumberAtVerticalOffset(t.mouseVerticalOffset),s=e.viewModel.getLineMaxColumn(n);return t.fulfillScrollbar(new A(n,s))}}return null}static _hitTestScrollbar(e,t){if(ns.isChildOfScrollableElement(t.targetPath)){const i=e.getLineNumberAtVerticalOffset(t.mouseVerticalOffset),n=e.viewModel.getLineMaxColumn(i);return t.fulfillScrollbar(new A(i,n))}return null}getMouseColumn(e){const t=this._context.configuration.options,i=t.get(146),n=this._context.viewLayout.getCurrentScrollLeft()+e.x-i.contentLeft;return jn._getMouseColumn(n,t.get(50).typicalHalfwidthCharacterWidth)}static _getMouseColumn(e,t){return e<0?1:Math.round(e/t)+1}static createMouseTargetFromHitTestPosition(e,t,i,n,s){const r=n.lineNumber,a=n.column,l=e.getLineWidth(r);if(t.mouseContentHorizontalOffset>l){const b=pT(t.mouseContentHorizontalOffset-l);return t.fulfillContentEmpty(n,b)}const c=e.visibleRangeForPosition(r,a);if(!c)return t.fulfillUnknown(n);const d=c.left;if(Math.abs(t.mouseContentHorizontalOffset-d)<1)return t.fulfillContentText(n,null,{mightBeForeignElement:!!s,injectedText:s});const h=[];if(h.push({offset:c.left,column:a}),a>1){const b=e.visibleRangeForPosition(r,a-1);b&&h.push({offset:b.left,column:a-1})}const u=e.viewModel.getLineMaxColumn(r);if(a<u){const b=e.visibleRangeForPosition(r,a+1);b&&h.push({offset:b.left,column:a+1})}h.sort((b,C)=>b.offset-C.offset);const g=t.pos.toClientCoordinates(xe(e.viewDomNode)),f=i.getBoundingClientRect(),p=f.left<=g.clientX&&g.clientX<=f.right;let _=null;for(let b=1;b<h.length;b++){const C=h[b-1],v=h[b];if(C.offset<=t.mouseContentHorizontalOffset&&t.mouseContentHorizontalOffset<=v.offset){_=new x(r,C.column,r,v.column);const w=Math.abs(C.offset-t.mouseContentHorizontalOffset),S=Math.abs(v.offset-t.mouseContentHorizontalOffset);n=w<S?new A(r,C.column):new A(r,v.column);break}}return t.fulfillContentText(n,_,{mightBeForeignElement:!p||!!s,injectedText:s})}static _doHitTestWithCaretRangeFromPoint(e,t){const i=e.getLineNumberAtVerticalOffset(t.mouseVerticalOffset),n=e.getVerticalOffsetForLineNumber(i),s=n+e.lineHeight;if(!(i===e.viewModel.getLineCount()&&t.mouseVerticalOffset>s)){const a=Math.floor((n+s)/2);let l=t.pos.y+(a-t.mouseVerticalOffset);l<=t.editorPos.y&&(l=t.editorPos.y+1),l>=t.editorPos.y+t.editorPos.height&&(l=t.editorPos.y+t.editorPos.height-1);const c=new pE(t.pos.x,l),d=this._actualDoHitTestWithCaretRangeFromPoint(e,c.toClientCoordinates(xe(e.viewDomNode)));if(d.type===1)return d}return this._actualDoHitTestWithCaretRangeFromPoint(e,t.pos.toClientCoordinates(xe(e.viewDomNode)))}static _actualDoHitTestWithCaretRangeFromPoint(e,t){const i=If(e.viewDomNode);let n;if(i?typeof i.caretRangeFromPoint>"u"?n=s_e(i,t.clientX,t.clientY):n=i.caretRangeFromPoint(t.clientX,t.clientY):n=e.viewDomNode.ownerDocument.caretRangeFromPoint(t.clientX,t.clientY),!n||!n.startContainer)return new Id;const s=n.startContainer;if(s.nodeType===s.TEXT_NODE){const r=s.parentNode,a=r?r.parentNode:null,l=a?a.parentNode:null;return(l&&l.nodeType===l.ELEMENT_NODE?l.className:null)===Ec.CLASS_NAME?Yu.createFromDOMInfo(e,r,n.startOffset):new Id(s.parentNode)}else if(s.nodeType===s.ELEMENT_NODE){const r=s.parentNode,a=r?r.parentNode:null;return(a&&a.nodeType===a.ELEMENT_NODE?a.className:null)===Ec.CLASS_NAME?Yu.createFromDOMInfo(e,s,s.textContent.length):new Id(s)}return new Id}static _doHitTestWithCaretPositionFromPoint(e,t){const i=e.viewDomNode.ownerDocument.caretPositionFromPoint(t.clientX,t.clientY);if(i.offsetNode.nodeType===i.offsetNode.TEXT_NODE){const n=i.offsetNode.parentNode,s=n?n.parentNode:null,r=s?s.parentNode:null;return(r&&r.nodeType===r.ELEMENT_NODE?r.className:null)===Ec.CLASS_NAME?Yu.createFromDOMInfo(e,i.offsetNode.parentNode,i.offset):new Id(i.offsetNode.parentNode)}if(i.offsetNode.nodeType===i.offsetNode.ELEMENT_NODE){const n=i.offsetNode.parentNode,s=n&&n.nodeType===n.ELEMENT_NODE?n.className:null,r=n?n.parentNode:null,a=r&&r.nodeType===r.ELEMENT_NODE?r.className:null;if(s===Ec.CLASS_NAME){const l=i.offsetNode.childNodes[Math.min(i.offset,i.offsetNode.childNodes.length-1)];if(l)return Yu.createFromDOMInfo(e,l,0)}else if(a===Ec.CLASS_NAME)return Yu.createFromDOMInfo(e,i.offsetNode,0)}return new Id(i.offsetNode)}static _snapToSoftTabBoundary(e,t){const i=t.getLineContent(e.lineNumber),{tabSize:n}=t.model.getOptions(),s=$w.atomicPosition(i,e.column-1,n,2);return s!==-1?new A(e.lineNumber,s+1):e}static doHitTest(e,t){let i=new Id;if(typeof e.viewDomNode.ownerDocument.caretRangeFromPoint=="function"?i=this._doHitTestWithCaretRangeFromPoint(e,t):e.viewDomNode.ownerDocument.caretPositionFromPoint&&(i=this._doHitTestWithCaretPositionFromPoint(e,t.pos.toClientCoordinates(xe(e.viewDomNode)))),i.type===1){const n=e.viewModel.getInjectedTextAt(i.position),s=e.viewModel.normalizePosition(i.position,2);(n||!s.equals(i.position))&&(i=new bq(s,i.spanNode,n))}return i}}function s_e(o,e,t){const i=document.createRange();let n=o.elementFromPoint(e,t);if(n!==null){for(;n&&n.firstChild&&n.firstChild.nodeType!==n.firstChild.TEXT_NODE&&n.lastChild&&n.lastChild.firstChild;)n=n.lastChild;const s=n.getBoundingClientRect(),r=xe(n),a=r.getComputedStyle(n,null).getPropertyValue("font-style"),l=r.getComputedStyle(n,null).getPropertyValue("font-variant"),c=r.getComputedStyle(n,null).getPropertyValue("font-weight"),d=r.getComputedStyle(n,null).getPropertyValue("font-size"),h=r.getComputedStyle(n,null).getPropertyValue("line-height"),u=r.getComputedStyle(n,null).getPropertyValue("font-family"),g=`${a} ${l} ${c} ${d}/${h} ${u}`,f=n.innerText;let p=s.left,_=0,b;if(e>s.left+s.width)_=f.length;else{const C=rP.getInstance();for(let v=0;v<f.length+1;v++){if(b=C.getCharWidth(f.charAt(v),g)/2,p+=b,e<p){_=v;break}p+=b}}i.setStart(n.firstChild,_),i.setEnd(n.firstChild,_)}return i}const ng=class ng{static getInstance(){return ng._INSTANCE||(ng._INSTANCE=new ng),ng._INSTANCE}constructor(){this._cache={},this._canvas=document.createElement("canvas")}getCharWidth(e,t){const i=e+t;if(this._cache[i])return this._cache[i];const n=this._canvas.getContext("2d");n.font=t;const r=n.measureText(e).width;return this._cache[i]=r,r}};ng._INSTANCE=null;let rP=ng;class $3 extends sS{constructor(e,t,i){super(),this._mouseLeaveMonitor=null,this._context=e,this.viewController=t,this.viewHelper=i,this.mouseTargetFactory=new jn(this._context,i),this._mouseDownOperation=this._register(new o_e(this._context,this.viewController,this.viewHelper,this.mouseTargetFactory,(r,a)=>this._createMouseTarget(r,a),r=>this._getMouseColumn(r))),this.lastMouseLeaveTime=-1,this._height=this._context.configuration.options.get(146).height;const n=new zpe(this.viewHelper.viewDomNode);this._register(n.onContextMenu(this.viewHelper.viewDomNode,r=>this._onContextMenu(r,!0))),this._register(n.onMouseMove(this.viewHelper.viewDomNode,r=>{this._onMouseMove(r),this._mouseLeaveMonitor||(this._mouseLeaveMonitor=U(this.viewHelper.viewDomNode.ownerDocument,"mousemove",a=>{this.viewHelper.viewDomNode.contains(a.target)||this._onMouseLeave(new qh(a,!1,this.viewHelper.viewDomNode))}))})),this._register(n.onMouseUp(this.viewHelper.viewDomNode,r=>this._onMouseUp(r))),this._register(n.onMouseLeave(this.viewHelper.viewDomNode,r=>this._onMouseLeave(r)));let s=0;this._register(n.onPointerDown(this.viewHelper.viewDomNode,(r,a)=>{s=a})),this._register(U(this.viewHelper.viewDomNode,ie.POINTER_UP,r=>{this._mouseDownOperation.onPointerUp()})),this._register(n.onMouseDown(this.viewHelper.viewDomNode,r=>this._onMouseDown(r,s))),this._setupMouseWheelZoomListener(),this._context.addEventHandler(this)}_setupMouseWheelZoomListener(){const e=hL.INSTANCE;let t=0,i=lr.getZoomLevel(),n=!1,s=0;const r=l=>{if(this.viewController.emitMouseWheel(l),!this._context.configuration.options.get(76))return;const c=new Df(l);if(e.acceptStandardWheelEvent(c),e.isPhysicalMouseWheel()){if(a(l)){const d=lr.getZoomLevel(),h=c.deltaY>0?1:-1;lr.setZoomLevel(d+h),c.preventDefault(),c.stopPropagation()}}else Date.now()-t>50&&(i=lr.getZoomLevel(),n=a(l),s=0),t=Date.now(),s+=c.deltaY,n&&(lr.setZoomLevel(i+s/5),c.preventDefault(),c.stopPropagation())};this._register(U(this.viewHelper.viewDomNode,ie.MOUSE_WHEEL,r,{capture:!0,passive:!1}));function a(l){return Je?(l.metaKey||l.ctrlKey)&&!l.shiftKey&&!l.altKey:l.ctrlKey&&!l.metaKey&&!l.shiftKey&&!l.altKey}}dispose(){this._context.removeEventHandler(this),this._mouseLeaveMonitor&&(this._mouseLeaveMonitor.dispose(),this._mouseLeaveMonitor=null),super.dispose()}onConfigurationChanged(e){if(e.hasChanged(146)){const t=this._context.configuration.options.get(146).height;this._height!==t&&(this._height=t,this._mouseDownOperation.onHeightChanged())}return!1}onCursorStateChanged(e){return this._mouseDownOperation.onCursorStateChanged(e),!1}onFocusChanged(e){return!1}getTargetAtClientPoint(e,t){const n=new mq(e,t).toPageCoordinates(xe(this.viewHelper.viewDomNode)),s=V3(this.viewHelper.viewDomNode);if(n.y<s.y||n.y>s.y+s.height||n.x<s.x||n.x>s.x+s.width)return null;const r=z3(this.viewHelper.viewDomNode,s,n);return this.mouseTargetFactory.createMouseTarget(this.viewHelper.getLastRenderData(),s,n,r,null)}_createMouseTarget(e,t){let i=e.target;if(!this.viewHelper.viewDomNode.contains(i)){const n=If(this.viewHelper.viewDomNode);n&&(i=n.elementsFromPoint(e.posx,e.posy).find(s=>this.viewHelper.viewDomNode.contains(s)))}return this.mouseTargetFactory.createMouseTarget(this.viewHelper.getLastRenderData(),e.editorPos,e.pos,e.relativePos,t?i:null)}_getMouseColumn(e){return this.mouseTargetFactory.getMouseColumn(e.relativePos)}_onContextMenu(e,t){this.viewController.emitContextMenu({event:e,target:this._createMouseTarget(e,t)})}_onMouseMove(e){this.mouseTargetFactory.mouseTargetIsWidget(e)||e.preventDefault(),!(this._mouseDownOperation.isActive()||e.timestamp<this.lastMouseLeaveTime)&&this.viewController.emitMouseMove({event:e,target:this._createMouseTarget(e,!0)})}_onMouseLeave(e){this._mouseLeaveMonitor&&(this._mouseLeaveMonitor.dispose(),this._mouseLeaveMonitor=null),this.lastMouseLeaveTime=new Date().getTime(),this.viewController.emitMouseLeave({event:e,target:null})}_onMouseUp(e){this.viewController.emitMouseUp({event:e,target:this._createMouseTarget(e,!0)})}_onMouseDown(e,t){const i=this._createMouseTarget(e,!0),n=i.type===6||i.type===7,s=i.type===2||i.type===3||i.type===4,r=i.type===3,a=this._context.configuration.options.get(110),l=i.type===8||i.type===5,c=i.type===9;let d=e.leftButton||e.middleButton;Je&&e.leftButton&&e.ctrlKey&&(d=!1);const h=()=>{e.preventDefault(),this.viewHelper.focusTextArea()};if(d&&(n||r&&a))h(),this._mouseDownOperation.start(i.type,e,t);else if(s)e.preventDefault();else if(l){const u=i.detail;d&&this.viewHelper.shouldSuppressMouseDownOnViewZone(u.viewZoneId)&&(h(),this._mouseDownOperation.start(i.type,e,t),e.preventDefault())}else c&&this.viewHelper.shouldSuppressMouseDownOnWidget(i.detail)&&(h(),e.preventDefault());this.viewController.emitMouseDown({event:e,target:i})}}class o_e extends F{constructor(e,t,i,n,s,r){super(),this._context=e,this._viewController=t,this._viewHelper=i,this._mouseTargetFactory=n,this._createMouseTarget=s,this._getMouseColumn=r,this._mouseMoveMonitor=this._register(new $pe(this._viewHelper.viewDomNode)),this._topBottomDragScrolling=this._register(new r_e(this._context,this._viewHelper,this._mouseTargetFactory,(a,l,c)=>this._dispatchMouse(a,l,c))),this._mouseState=new aP,this._currentSelection=new pe(1,1,1,1),this._isActive=!1,this._lastMouseEvent=null}dispose(){super.dispose()}isActive(){return this._isActive}_onMouseDownThenMove(e){this._lastMouseEvent=e,this._mouseState.setModifiers(e);const t=this._findMousePosition(e,!1);t&&(this._mouseState.isDragAndDrop?this._viewController.emitMouseDrag({event:e,target:t}):t.type===13&&(t.outsidePosition==="above"||t.outsidePosition==="below")?this._topBottomDragScrolling.start(t,e):(this._topBottomDragScrolling.stop(),this._dispatchMouse(t,!0,1)))}start(e,t,i){this._lastMouseEvent=t,this._mouseState.setStartedOnLineNumbers(e===3),this._mouseState.setStartButtons(t),this._mouseState.setModifiers(t);const n=this._findMousePosition(t,!0);if(!n||!n.position)return;this._mouseState.trySetCount(t.detail,n.position),t.detail=this._mouseState.count;const s=this._context.configuration.options;if(!s.get(92)&&s.get(35)&&!s.get(22)&&!this._mouseState.altKey&&t.detail<2&&!this._isActive&&!this._currentSelection.isEmpty()&&n.type===6&&n.position&&this._currentSelection.containsPosition(n.position)){this._mouseState.isDragAndDrop=!0,this._isActive=!0,this._mouseMoveMonitor.startMonitoring(this._viewHelper.viewLinesDomNode,i,t.buttons,r=>this._onMouseDownThenMove(r),r=>{const a=this._findMousePosition(this._lastMouseEvent,!1);wc(r)?this._viewController.emitMouseDropCanceled():this._viewController.emitMouseDrop({event:this._lastMouseEvent,target:a?this._createMouseTarget(this._lastMouseEvent,!0):null}),this._stop()});return}this._mouseState.isDragAndDrop=!1,this._dispatchMouse(n,t.shiftKey,1),this._isActive||(this._isActive=!0,this._mouseMoveMonitor.startMonitoring(this._viewHelper.viewLinesDomNode,i,t.buttons,r=>this._onMouseDownThenMove(r),()=>this._stop()))}_stop(){this._isActive=!1,this._topBottomDragScrolling.stop()}onHeightChanged(){this._mouseMoveMonitor.stopMonitoring()}onPointerUp(){this._mouseMoveMonitor.stopMonitoring()}onCursorStateChanged(e){this._currentSelection=e.selections[0]}_getPositionOutsideEditor(e){const t=e.editorPos,i=this._context.viewModel,n=this._context.viewLayout,s=this._getMouseColumn(e);if(e.posy<t.y){const a=t.y-e.posy,l=Math.max(n.getCurrentScrollTop()-a,0),c=q_.getZoneAtCoord(this._context,l);if(c){const h=this._helpPositionJumpOverViewZone(c);if(h)return Kn.createOutsideEditor(s,h,"above",a)}const d=n.getLineNumberAtVerticalOffset(l);return Kn.createOutsideEditor(s,new A(d,1),"above",a)}if(e.posy>t.y+t.height){const a=e.posy-t.y-t.height,l=n.getCurrentScrollTop()+e.relativePos.y,c=q_.getZoneAtCoord(this._context,l);if(c){const h=this._helpPositionJumpOverViewZone(c);if(h)return Kn.createOutsideEditor(s,h,"below",a)}const d=n.getLineNumberAtVerticalOffset(l);return Kn.createOutsideEditor(s,new A(d,i.getLineMaxColumn(d)),"below",a)}const r=n.getLineNumberAtVerticalOffset(n.getCurrentScrollTop()+e.relativePos.y);if(e.posx<t.x){const a=t.x-e.posx;return Kn.createOutsideEditor(s,new A(r,1),"left",a)}if(e.posx>t.x+t.width){const a=e.posx-t.x-t.width;return Kn.createOutsideEditor(s,new A(r,i.getLineMaxColumn(r)),"right",a)}return null}_findMousePosition(e,t){const i=this._getPositionOutsideEditor(e);if(i)return i;const n=this._createMouseTarget(e,t);if(!n.position)return null;if(n.type===8||n.type===5){const r=this._helpPositionJumpOverViewZone(n.detail);if(r)return Kn.createViewZone(n.type,n.element,n.mouseColumn,r,n.detail)}return n}_helpPositionJumpOverViewZone(e){const t=new A(this._currentSelection.selectionStartLineNumber,this._currentSelection.selectionStartColumn),i=e.positionBefore,n=e.positionAfter;return i&&n?i.isBefore(t)?i:n:null}_dispatchMouse(e,t,i){e.position&&this._viewController.dispatchMouse({position:e.position,mouseColumn:e.mouseColumn,startedOnLineNumbers:this._mouseState.startedOnLineNumbers,revealType:i,inSelectionMode:t,mouseDownCount:this._mouseState.count,altKey:this._mouseState.altKey,ctrlKey:this._mouseState.ctrlKey,metaKey:this._mouseState.metaKey,shiftKey:this._mouseState.shiftKey,leftButton:this._mouseState.leftButton,middleButton:this._mouseState.middleButton,onInjectedText:e.type===6&&e.detail.injectedText!==null})}}class r_e extends F{constructor(e,t,i,n){super(),this._context=e,this._viewHelper=t,this._mouseTargetFactory=i,this._dispatchMouse=n,this._operation=null}dispose(){super.dispose(),this.stop()}start(e,t){this._operation?this._operation.setPosition(e,t):this._operation=new a_e(this._context,this._viewHelper,this._mouseTargetFactory,this._dispatchMouse,e,t)}stop(){this._operation&&(this._operation.dispose(),this._operation=null)}}class a_e extends F{constructor(e,t,i,n,s,r){super(),this._context=e,this._viewHelper=t,this._mouseTargetFactory=i,this._dispatchMouse=n,this._position=s,this._mouseEvent=r,this._lastTime=Date.now(),this._animationFrameDisposable=go(xe(r.browserEvent),()=>this._execute())}dispose(){this._animationFrameDisposable.dispose(),super.dispose()}setPosition(e,t){this._position=e,this._mouseEvent=t}_tick(){const e=Date.now(),t=e-this._lastTime;return this._lastTime=e,t}_getScrollSpeed(){const e=this._context.configuration.options.get(67),t=this._context.configuration.options.get(146).height/e,i=this._position.outsideDistance/e;return i<=1.5?Math.max(30,t*(1+i)):i<=3?Math.max(60,t*(2+i)):Math.max(200,t*(7+i))}_execute(){const e=this._context.configuration.options.get(67),t=this._getScrollSpeed(),i=this._tick(),n=t*(i/1e3)*e,s=this._position.outsidePosition==="above"?-n:n;this._context.viewModel.viewLayout.deltaScrollNow(0,s),this._viewHelper.renderNow();const r=this._context.viewLayout.getLinesViewportData(),a=this._position.outsidePosition==="above"?r.startLineNumber:r.endLineNumber;let l;{const c=V3(this._viewHelper.viewDomNode),d=this._context.configuration.options.get(146).horizontalScrollbarHeight,h=new pE(this._mouseEvent.pos.x,c.y+c.height-d-.1),u=z3(this._viewHelper.viewDomNode,c,h);l=this._mouseTargetFactory.createMouseTarget(this._viewHelper.getLastRenderData(),c,h,u,null)}(!l.position||l.position.lineNumber!==a)&&(this._position.outsidePosition==="above"?l=Kn.createOutsideEditor(this._position.mouseColumn,new A(a,1),"above",this._position.outsideDistance):l=Kn.createOutsideEditor(this._position.mouseColumn,new A(a,this._context.viewModel.getLineMaxColumn(a)),"below",this._position.outsideDistance)),this._dispatchMouse(l,!0,2),this._animationFrameDisposable=go(xe(l.element),()=>this._execute())}}const aD=class aD{get altKey(){return this._altKey}get ctrlKey(){return this._ctrlKey}get metaKey(){return this._metaKey}get shiftKey(){return this._shiftKey}get leftButton(){return this._leftButton}get middleButton(){return this._middleButton}get startedOnLineNumbers(){return this._startedOnLineNumbers}constructor(){this._altKey=!1,this._ctrlKey=!1,this._metaKey=!1,this._shiftKey=!1,this._leftButton=!1,this._middleButton=!1,this._startedOnLineNumbers=!1,this._lastMouseDownPosition=null,this._lastMouseDownPositionEqualCount=0,this._lastMouseDownCount=0,this._lastSetMouseDownCountTime=0,this.isDragAndDrop=!1}get count(){return this._lastMouseDownCount}setModifiers(e){this._altKey=e.altKey,this._ctrlKey=e.ctrlKey,this._metaKey=e.metaKey,this._shiftKey=e.shiftKey}setStartButtons(e){this._leftButton=e.leftButton,this._middleButton=e.middleButton}setStartedOnLineNumbers(e){this._startedOnLineNumbers=e}trySetCount(e,t){const i=new Date().getTime();i-this._lastSetMouseDownCountTime>aD.CLEAR_MOUSE_DOWN_COUNT_TIME&&(e=1),this._lastSetMouseDownCountTime=i,e>this._lastMouseDownCount+1&&(e=this._lastMouseDownCount+1),this._lastMouseDownPosition&&this._lastMouseDownPosition.equals(t)?this._lastMouseDownPositionEqualCount++:this._lastMouseDownPositionEqualCount=1,this._lastMouseDownPosition=t,this._lastMouseDownCount=Math.min(e,this._lastMouseDownPositionEqualCount)}};aD.CLEAR_MOUSE_DOWN_COUNT_TIME=400;let aP=aD;const Rp=class Rp{constructor(e,t,i,n,s){this.value=e,this.selectionStart=t,this.selectionEnd=i,this.selection=n,this.newlineCountBeforeSelection=s}toString(){return`[ <${this.value}>, selectionStart: ${this.selectionStart}, selectionEnd: ${this.selectionEnd}]`}static readFromTextArea(e,t){const i=e.getValue(),n=e.getSelectionStart(),s=e.getSelectionEnd();let r;if(t){const a=i.substring(0,n),l=t.value.substring(0,t.selectionStart);a===l&&(r=t.newlineCountBeforeSelection)}return new Rp(i,n,s,null,r)}collapseSelection(){return this.selectionStart===this.value.length?this:new Rp(this.value,this.value.length,this.value.length,null,void 0)}writeToTextArea(e,t,i){t.setValue(e,this.value),i&&t.setSelectionRange(e,this.selectionStart,this.selectionEnd)}deduceEditorPosition(e){var n,s,r,a;if(e<=this.selectionStart){const l=this.value.substring(e,this.selectionStart);return this._finishDeduceEditorPosition(((n=this.selection)==null?void 0:n.getStartPosition())??null,l,-1)}if(e>=this.selectionEnd){const l=this.value.substring(this.selectionEnd,e);return this._finishDeduceEditorPosition(((s=this.selection)==null?void 0:s.getEndPosition())??null,l,1)}const t=this.value.substring(this.selectionStart,e);if(t.indexOf("…")===-1)return this._finishDeduceEditorPosition(((r=this.selection)==null?void 0:r.getStartPosition())??null,t,1);const i=this.value.substring(e,this.selectionEnd);return this._finishDeduceEditorPosition(((a=this.selection)==null?void 0:a.getEndPosition())??null,i,-1)}_finishDeduceEditorPosition(e,t,i){let n=0,s=-1;for(;(s=t.indexOf(`
`,s+1))!==-1;)n++;return[e,i*t.length,n]}static deduceInput(e,t,i){if(!e)return{text:"",replacePrevCharCnt:0,replaceNextCharCnt:0,positionDelta:0};const n=Math.min(Oh(e.value,t.value),e.selectionStart,t.selectionStart),s=Math.min(By(e.value,t.value),e.value.length-e.selectionEnd,t.value.length-t.selectionEnd);e.value.substring(n,e.value.length-s);const r=t.value.substring(n,t.value.length-s),a=e.selectionStart-n,l=e.selectionEnd-n,c=t.selectionStart-n,d=t.selectionEnd-n;if(c===d){const u=e.selectionStart-n;return{text:r,replacePrevCharCnt:u,replaceNextCharCnt:0,positionDelta:0}}const h=l-a;return{text:r,replacePrevCharCnt:h,replaceNextCharCnt:0,positionDelta:0}}static deduceAndroidCompositionInput(e,t){if(!e)return{text:"",replacePrevCharCnt:0,replaceNextCharCnt:0,positionDelta:0};if(e.value===t.value)return{text:"",replacePrevCharCnt:0,replaceNextCharCnt:0,positionDelta:t.selectionEnd-e.selectionEnd};const i=Math.min(Oh(e.value,t.value),e.selectionEnd),n=Math.min(By(e.value,t.value),e.value.length-e.selectionEnd),s=e.value.substring(i,e.value.length-n),r=t.value.substring(i,t.value.length-n);e.selectionStart-i;const a=e.selectionEnd-i;t.selectionStart-i;const l=t.selectionEnd-i;return{text:r,replacePrevCharCnt:a,replaceNextCharCnt:s.length-a,positionDelta:l-r.length}}};Rp.EMPTY=new Rp("",0,0,null,void 0);let qn=Rp;class hp{static _getPageOfLine(e,t){return Math.floor((e-1)/t)}static _getRangeForPage(e,t){const i=e*t,n=i+1,s=i+t;return new x(n,1,s+1,1)}static fromEditorSelection(e,t,i,n){const r=hp._getPageOfLine(t.startLineNumber,i),a=hp._getRangeForPage(r,i),l=hp._getPageOfLine(t.endLineNumber,i),c=hp._getRangeForPage(l,i);let d=a.intersectRanges(new x(1,1,t.startLineNumber,t.startColumn));if(n&&e.getValueLengthInRange(d,1)>500){const b=e.modifyPosition(d.getEndPosition(),-500);d=x.fromPositions(b,d.getEndPosition())}const h=e.getValueInRange(d,1),u=e.getLineCount(),g=e.getLineMaxColumn(u);let f=c.intersectRanges(new x(t.endLineNumber,t.endColumn,u,g));if(n&&e.getValueLengthInRange(f,1)>500){const b=e.modifyPosition(f.getStartPosition(),500);f=x.fromPositions(f.getStartPosition(),b)}const p=e.getValueInRange(f,1);let _;if(r===l||r+1===l)_=e.getValueInRange(t,1);else{const b=a.intersectRanges(t),C=c.intersectRanges(t);_=e.getValueInRange(b,1)+"…"+e.getValueInRange(C,1)}return n&&_.length>2*500&&(_=_.substring(0,500)+"…"+_.substring(_.length-500,_.length)),new qn(h+_+p,h.length,h.length+_.length,t,d.endLineNumber-d.startLineNumber)}}var l_e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},wH=function(o,e){return function(t,i){e(t,i,o)}},ox;(function(o){o.Tap="-monaco-textarea-synthetic-tap"})(ox||(ox={}));const lP={forceCopyWithSyntaxHighlighting:!1},lD=class lD{constructor(){this._lastState=null}set(e,t){this._lastState={lastCopiedValue:e,data:t}}get(e){return this._lastState&&this._lastState.lastCopiedValue===e?this._lastState.data:(this._lastState=null,null)}};lD.INSTANCE=new lD;let jw=lD;class c_e{constructor(){this._lastTypeTextLength=0}handleCompositionUpdate(e){e=e||"";const t={text:e,replacePrevCharCnt:this._lastTypeTextLength,replaceNextCharCnt:0,positionDelta:0};return this._lastTypeTextLength=e.length,t}}let cP=class extends F{get textAreaState(){return this._textAreaState}constructor(e,t,i,n,s,r){super(),this._host=e,this._textArea=t,this._OS=i,this._browser=n,this._accessibilityService=s,this._logService=r,this._onFocus=this._register(new P),this.onFocus=this._onFocus.event,this._onBlur=this._register(new P),this.onBlur=this._onBlur.event,this._onKeyDown=this._register(new P),this.onKeyDown=this._onKeyDown.event,this._onKeyUp=this._register(new P),this.onKeyUp=this._onKeyUp.event,this._onCut=this._register(new P),this.onCut=this._onCut.event,this._onPaste=this._register(new P),this.onPaste=this._onPaste.event,this._onType=this._register(new P),this.onType=this._onType.event,this._onCompositionStart=this._register(new P),this.onCompositionStart=this._onCompositionStart.event,this._onCompositionUpdate=this._register(new P),this.onCompositionUpdate=this._onCompositionUpdate.event,this._onCompositionEnd=this._register(new P),this.onCompositionEnd=this._onCompositionEnd.event,this._onSelectionChangeRequest=this._register(new P),this.onSelectionChangeRequest=this._onSelectionChangeRequest.event,this._asyncFocusGainWriteScreenReaderContent=this._register(new Ji),this._asyncTriggerCut=this._register(new It(()=>this._onCut.fire(),0)),this._textAreaState=qn.EMPTY,this._selectionChangeListener=null,this._accessibilityService.isScreenReaderOptimized()&&this.writeNativeTextAreaContent("ctor"),this._register(oe.runAndSubscribe(this._accessibilityService.onDidChangeScreenReaderOptimized,()=>{this._accessibilityService.isScreenReaderOptimized()&&!this._asyncFocusGainWriteScreenReaderContent.value?this._asyncFocusGainWriteScreenReaderContent.value=this._register(new It(()=>this.writeNativeTextAreaContent("asyncFocusGain"),0)):this._asyncFocusGainWriteScreenReaderContent.clear()})),this._hasFocus=!1,this._currentComposition=null;let a=null;this._register(this._textArea.onKeyDown(l=>{const c=new Pt(l);(c.keyCode===114||this._currentComposition&&c.keyCode===1)&&c.stopPropagation(),c.equals(9)&&c.preventDefault(),a=c,this._onKeyDown.fire(c)})),this._register(this._textArea.onKeyUp(l=>{const c=new Pt(l);this._onKeyUp.fire(c)})),this._register(this._textArea.onCompositionStart(l=>{const c=new c_e;if(this._currentComposition){this._currentComposition=c;return}if(this._currentComposition=c,this._OS===2&&a&&a.equals(114)&&this._textAreaState.selectionStart===this._textAreaState.selectionEnd&&this._textAreaState.selectionStart>0&&this._textAreaState.value.substr(this._textAreaState.selectionStart-1,1)===l.data&&(a.code==="ArrowRight"||a.code==="ArrowLeft")){c.handleCompositionUpdate("x"),this._onCompositionStart.fire({data:l.data});return}if(this._browser.isAndroid){this._onCompositionStart.fire({data:l.data});return}this._onCompositionStart.fire({data:l.data})})),this._register(this._textArea.onCompositionUpdate(l=>{const c=this._currentComposition;if(!c)return;if(this._browser.isAndroid){const h=qn.readFromTextArea(this._textArea,this._textAreaState),u=qn.deduceAndroidCompositionInput(this._textAreaState,h);this._textAreaState=h,this._onType.fire(u),this._onCompositionUpdate.fire(l);return}const d=c.handleCompositionUpdate(l.data);this._textAreaState=qn.readFromTextArea(this._textArea,this._textAreaState),this._onType.fire(d),this._onCompositionUpdate.fire(l)})),this._register(this._textArea.onCompositionEnd(l=>{const c=this._currentComposition;if(!c)return;if(this._currentComposition=null,this._browser.isAndroid){const h=qn.readFromTextArea(this._textArea,this._textAreaState),u=qn.deduceAndroidCompositionInput(this._textAreaState,h);this._textAreaState=h,this._onType.fire(u),this._onCompositionEnd.fire();return}const d=c.handleCompositionUpdate(l.data);this._textAreaState=qn.readFromTextArea(this._textArea,this._textAreaState),this._onType.fire(d),this._onCompositionEnd.fire()})),this._register(this._textArea.onInput(l=>{if(this._textArea.setIgnoreSelectionChangeTime("received input event"),this._currentComposition)return;const c=qn.readFromTextArea(this._textArea,this._textAreaState),d=qn.deduceInput(this._textAreaState,c,this._OS===2);d.replacePrevCharCnt===0&&d.text.length===1&&(Gi(d.text.charCodeAt(0))||d.text.charCodeAt(0)===127)||(this._textAreaState=c,(d.text!==""||d.replacePrevCharCnt!==0||d.replaceNextCharCnt!==0||d.positionDelta!==0)&&this._onType.fire(d))})),this._register(this._textArea.onCut(l=>{this._textArea.setIgnoreSelectionChangeTime("received cut event"),this._ensureClipboardGetsEditorSelection(l),this._asyncTriggerCut.schedule()})),this._register(this._textArea.onCopy(l=>{this._ensureClipboardGetsEditorSelection(l)})),this._register(this._textArea.onPaste(l=>{if(this._textArea.setIgnoreSelectionChangeTime("received paste event"),l.preventDefault(),!l.clipboardData)return;let[c,d]=dP.getTextData(l.clipboardData);c&&(d=d||jw.INSTANCE.get(c),this._onPaste.fire({text:c,metadata:d}))})),this._register(this._textArea.onFocus(()=>{const l=this._hasFocus;this._setHasFocus(!0),this._accessibilityService.isScreenReaderOptimized()&&this._browser.isSafari&&!l&&this._hasFocus&&(this._asyncFocusGainWriteScreenReaderContent.value||(this._asyncFocusGainWriteScreenReaderContent.value=new It(()=>this.writeNativeTextAreaContent("asyncFocusGain"),0)),this._asyncFocusGainWriteScreenReaderContent.value.schedule())})),this._register(this._textArea.onBlur(()=>{this._currentComposition&&(this._currentComposition=null,this.writeNativeTextAreaContent("blurWithoutCompositionEnd"),this._onCompositionEnd.fire()),this._setHasFocus(!1)})),this._register(this._textArea.onSyntheticTap(()=>{this._browser.isAndroid&&this._currentComposition&&(this._currentComposition=null,this.writeNativeTextAreaContent("tapWithoutCompositionEnd"),this._onCompositionEnd.fire())}))}_installSelectionChangeListener(){let e=0;return U(this._textArea.ownerDocument,"selectionchange",t=>{if(eh.onSelectionChange(),!this._hasFocus||this._currentComposition||!this._browser.isChrome)return;const i=Date.now(),n=i-e;if(e=i,n<5)return;const s=i-this._textArea.getIgnoreSelectionChangeTime();if(this._textArea.resetSelectionChangeTime(),s<100||!this._textAreaState.selection)return;const r=this._textArea.getValue();if(this._textAreaState.value!==r)return;const a=this._textArea.getSelectionStart(),l=this._textArea.getSelectionEnd();if(this._textAreaState.selectionStart===a&&this._textAreaState.selectionEnd===l)return;const c=this._textAreaState.deduceEditorPosition(a),d=this._host.deduceModelPosition(c[0],c[1],c[2]),h=this._textAreaState.deduceEditorPosition(l),u=this._host.deduceModelPosition(h[0],h[1],h[2]),g=new pe(d.lineNumber,d.column,u.lineNumber,u.column);this._onSelectionChangeRequest.fire(g)})}dispose(){super.dispose(),this._selectionChangeListener&&(this._selectionChangeListener.dispose(),this._selectionChangeListener=null)}focusTextArea(){this._setHasFocus(!0),this.refreshFocusState()}isFocused(){return this._hasFocus}refreshFocusState(){this._setHasFocus(this._textArea.hasFocus())}_setHasFocus(e){this._hasFocus!==e&&(this._hasFocus=e,this._selectionChangeListener&&(this._selectionChangeListener.dispose(),this._selectionChangeListener=null),this._hasFocus&&(this._selectionChangeListener=this._installSelectionChangeListener()),this._hasFocus&&this.writeNativeTextAreaContent("focusgain"),this._hasFocus?this._onFocus.fire():this._onBlur.fire())}_setAndWriteTextAreaState(e,t){this._hasFocus||(t=t.collapseSelection()),t.writeToTextArea(e,this._textArea,this._hasFocus),this._textAreaState=t}writeNativeTextAreaContent(e){!this._accessibilityService.isScreenReaderOptimized()&&e==="render"||this._currentComposition||(this._logService.trace(`writeTextAreaState(reason: ${e})`),this._setAndWriteTextAreaState(e,this._host.getScreenReaderContent()))}_ensureClipboardGetsEditorSelection(e){const t=this._host.getDataToCopy(),i={version:1,isFromEmptySelection:t.isFromEmptySelection,multicursorText:t.multicursorText,mode:t.mode};jw.INSTANCE.set(this._browser.isFirefox?t.text.replace(/\r\n/g,`
`):t.text,i),e.preventDefault(),e.clipboardData&&dP.setTextData(e.clipboardData,t.text,t.html,i)}};cP=l_e([wH(4,po),wH(5,Nn)],cP);const dP={getTextData(o){const e=o.getData(vi.text);let t=null;const i=o.getData("vscode-editor-data");if(typeof i=="string")try{t=JSON.parse(i),t.version!==1&&(t=null)}catch{}return e.length===0&&t===null&&o.files.length>0?[Array.prototype.slice.call(o.files,0).map(s=>s.name).join(`
`),null]:[e,t]},setTextData(o,e,t,i){o.setData(vi.text,e),typeof t=="string"&&o.setData("text/html",t),o.setData("vscode-editor-data",JSON.stringify(i))}};class d_e extends F{get ownerDocument(){return this._actual.ownerDocument}constructor(e){super(),this._actual=e,this.onKeyDown=this._register(new tt(this._actual,"keydown")).event,this.onKeyUp=this._register(new tt(this._actual,"keyup")).event,this.onCompositionStart=this._register(new tt(this._actual,"compositionstart")).event,this.onCompositionUpdate=this._register(new tt(this._actual,"compositionupdate")).event,this.onCompositionEnd=this._register(new tt(this._actual,"compositionend")).event,this.onBeforeInput=this._register(new tt(this._actual,"beforeinput")).event,this.onInput=this._register(new tt(this._actual,"input")).event,this.onCut=this._register(new tt(this._actual,"cut")).event,this.onCopy=this._register(new tt(this._actual,"copy")).event,this.onPaste=this._register(new tt(this._actual,"paste")).event,this.onFocus=this._register(new tt(this._actual,"focus")).event,this.onBlur=this._register(new tt(this._actual,"blur")).event,this._onSyntheticTap=this._register(new P),this.onSyntheticTap=this._onSyntheticTap.event,this._ignoreSelectionChangeTime=0,this._register(this.onKeyDown(()=>eh.onKeyDown())),this._register(this.onBeforeInput(()=>eh.onBeforeInput())),this._register(this.onInput(()=>eh.onInput())),this._register(this.onKeyUp(()=>eh.onKeyUp())),this._register(U(this._actual,ox.Tap,()=>this._onSyntheticTap.fire()))}hasFocus(){const e=If(this._actual);return e?e.activeElement===this._actual:this._actual.isConnected?Dn()===this._actual:!1}setIgnoreSelectionChangeTime(e){this._ignoreSelectionChangeTime=Date.now()}getIgnoreSelectionChangeTime(){return this._ignoreSelectionChangeTime}resetSelectionChangeTime(){this._ignoreSelectionChangeTime=0}getValue(){return this._actual.value}setValue(e,t){const i=this._actual;i.value!==t&&(this.setIgnoreSelectionChangeTime("setValue"),i.value=t)}getSelectionStart(){return this._actual.selectionDirection==="backward"?this._actual.selectionEnd:this._actual.selectionStart}getSelectionEnd(){return this._actual.selectionDirection==="backward"?this._actual.selectionStart:this._actual.selectionEnd}setSelectionRange(e,t,i){const n=this._actual;let s=null;const r=If(n);r?s=r.activeElement:s=Dn();const a=xe(s),l=s===n,c=n.selectionStart,d=n.selectionEnd;if(l&&c===t&&d===i){pr&&a.parent!==a&&n.focus();return}if(l){this.setIgnoreSelectionChangeTime("setSelectionRange"),n.setSelectionRange(t,i),pr&&a.parent!==a&&n.focus();return}try{const h=Gte(n);this.setIgnoreSelectionChangeTime("setSelectionRange"),n.focus(),n.setSelectionRange(t,i),Zte(n,h)}catch{}}}class h_e extends $3{constructor(e,t,i){super(e,t,i),this._register(Fn.addTarget(this.viewHelper.linesContentDomNode)),this._register(U(this.viewHelper.linesContentDomNode,Bt.Tap,s=>this.onTap(s))),this._register(U(this.viewHelper.linesContentDomNode,Bt.Change,s=>this.onChange(s))),this._register(U(this.viewHelper.linesContentDomNode,Bt.Contextmenu,s=>this._onContextMenu(new qh(s,!1,this.viewHelper.viewDomNode),!1))),this._lastPointerType="mouse",this._register(U(this.viewHelper.linesContentDomNode,"pointerdown",s=>{const r=s.pointerType;if(r==="mouse"){this._lastPointerType="mouse";return}else r==="touch"?this._lastPointerType="touch":this._lastPointerType="pen"}));const n=new Upe(this.viewHelper.viewDomNode);this._register(n.onPointerMove(this.viewHelper.viewDomNode,s=>this._onMouseMove(s))),this._register(n.onPointerUp(this.viewHelper.viewDomNode,s=>this._onMouseUp(s))),this._register(n.onPointerLeave(this.viewHelper.viewDomNode,s=>this._onMouseLeave(s))),this._register(n.onPointerDown(this.viewHelper.viewDomNode,(s,r)=>this._onMouseDown(s,r)))}onTap(e){!e.initialTarget||!this.viewHelper.linesContentDomNode.contains(e.initialTarget)||(e.preventDefault(),this.viewHelper.focusTextArea(),this._dispatchGesture(e,!1))}onChange(e){this._lastPointerType==="touch"&&this._context.viewModel.viewLayout.deltaScrollNow(-e.translationX,-e.translationY),this._lastPointerType==="pen"&&this._dispatchGesture(e,!0)}_dispatchGesture(e,t){const i=this._createMouseTarget(new qh(e,!1,this.viewHelper.viewDomNode),!1);i.position&&this.viewController.dispatchMouse({position:i.position,mouseColumn:i.position.column,startedOnLineNumbers:!1,revealType:1,mouseDownCount:e.tapCount,inSelectionMode:t,altKey:!1,ctrlKey:!1,metaKey:!1,shiftKey:!1,leftButton:!1,middleButton:!1,onInjectedText:i.type===6&&i.detail.injectedText!==null})}_onMouseDown(e,t){e.browserEvent.pointerType!=="touch"&&super._onMouseDown(e,t)}}class u_e extends $3{constructor(e,t,i){super(e,t,i),this._register(Fn.addTarget(this.viewHelper.linesContentDomNode)),this._register(U(this.viewHelper.linesContentDomNode,Bt.Tap,n=>this.onTap(n))),this._register(U(this.viewHelper.linesContentDomNode,Bt.Change,n=>this.onChange(n))),this._register(U(this.viewHelper.linesContentDomNode,Bt.Contextmenu,n=>this._onContextMenu(new qh(n,!1,this.viewHelper.viewDomNode),!1)))}onTap(e){e.preventDefault(),this.viewHelper.focusTextArea();const t=this._createMouseTarget(new qh(e,!1,this.viewHelper.viewDomNode),!1);if(t.position){const i=document.createEvent("CustomEvent");i.initEvent(ox.Tap,!1,!0),this.viewHelper.dispatchTextAreaEvent(i),this.viewController.moveTo(t.position,1)}}onChange(e){this._context.viewModel.viewLayout.deltaScrollNow(-e.translationX,-e.translationY)}}class g_e extends F{constructor(e,t,i){super(),(Pa||VQ&&Ez)&&i5.pointerEvents?this.handler=this._register(new h_e(e,t,i)):Rt.TouchEvent?this.handler=this._register(new u_e(e,t,i)):this.handler=this._register(new $3(e,t,i))}getTargetAtClientPoint(e,t){return this.handler.getTargetAtClientPoint(e,t)}}class lm extends sS{}const cD=class cD extends lm{constructor(e){super(),this._context=e,this._readConfig(),this._lastCursorModelPosition=new A(1,1),this._renderResult=null,this._activeLineNumber=1,this._context.addEventHandler(this)}_readConfig(){const e=this._context.configuration.options;this._lineHeight=e.get(67);const t=e.get(68);this._renderLineNumbers=t.renderType,this._renderCustomLineNumbers=t.renderFn,this._renderFinalNewline=e.get(96);const i=e.get(146);this._lineNumbersLeft=i.lineNumbersLeft,this._lineNumbersWidth=i.lineNumbersWidth}dispose(){this._context.removeEventHandler(this),this._renderResult=null,super.dispose()}onConfigurationChanged(e){return this._readConfig(),!0}onCursorStateChanged(e){const t=e.selections[0].getPosition();this._lastCursorModelPosition=this._context.viewModel.coordinatesConverter.convertViewPositionToModelPosition(t);let i=!1;return this._activeLineNumber!==t.lineNumber&&(this._activeLineNumber=t.lineNumber,i=!0),(this._renderLineNumbers===2||this._renderLineNumbers===3)&&(i=!0),i}onFlushed(e){return!0}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return e.scrollTopChanged}onZonesChanged(e){return!0}onDecorationsChanged(e){return e.affectsLineNumber}_getLineRenderLineNumber(e){const t=this._context.viewModel.coordinatesConverter.convertViewPositionToModelPosition(new A(e,1));if(t.column!==1)return"";const i=t.lineNumber;if(this._renderCustomLineNumbers)return this._renderCustomLineNumbers(i);if(this._renderLineNumbers===2){const n=Math.abs(this._lastCursorModelPosition.lineNumber-i);return n===0?'<span class="relative-current-line-number">'+i+"</span>":String(n)}if(this._renderLineNumbers===3){if(this._lastCursorModelPosition.lineNumber===i||i%10===0)return String(i);const n=this._context.viewModel.getLineCount();return i===n?String(i):""}return String(i)}prepareRender(e){if(this._renderLineNumbers===0){this._renderResult=null;return}const t=ps?this._lineHeight%2===0?" lh-even":" lh-odd":"",i=e.visibleRange.startLineNumber,n=e.visibleRange.endLineNumber,s=this._context.viewModel.getDecorationsInViewport(e.visibleRange).filter(c=>!!c.options.lineNumberClassName);s.sort((c,d)=>x.compareRangesUsingEnds(c.range,d.range));let r=0;const a=this._context.viewModel.getLineCount(),l=[];for(let c=i;c<=n;c++){const d=c-i;let h=this._getLineRenderLineNumber(c),u="";for(;r<s.length&&s[r].range.endLineNumber<c;)r++;for(let g=r;g<s.length;g++){const{range:f,options:p}=s[g];f.startLineNumber<=c&&(u+=" "+p.lineNumberClassName)}if(!h&&!u){l[d]="";continue}c===a&&this._context.viewModel.getLineLength(c)===0&&(this._renderFinalNewline==="off"&&(h=""),this._renderFinalNewline==="dimmed"&&(u+=" dimmed-line-number")),c===this._activeLineNumber&&(u+=" active-line-number"),l[d]=`<div class="${cD.CLASS_NAME}${t}${u}" style="left:${this._lineNumbersLeft}px;width:${this._lineNumbersWidth}px;">${h}</div>`}this._renderResult=l}render(e,t){if(!this._renderResult)return"";const i=t-e;return i<0||i>=this._renderResult.length?"":this._renderResult[i]}};cD.CLASS_NAME="line-numbers";let rx=cD;Lr((o,e)=>{const t=o.getColor(nhe),i=o.getColor(_he);i?e.addRule(`.monaco-editor .line-numbers.dimmed-line-number { color: ${i}; }`):t&&e.addRule(`.monaco-editor .line-numbers.dimmed-line-number { color: ${t.transparent(.4)}; }`)});const Mp=class Mp extends bo{constructor(e){super(e);const t=this._context.configuration.options,i=t.get(146);this._canUseLayerHinting=!t.get(32),this._contentLeft=i.contentLeft,this._glyphMarginLeft=i.glyphMarginLeft,this._glyphMarginWidth=i.glyphMarginWidth,this._domNode=St(document.createElement("div")),this._domNode.setClassName(Mp.OUTER_CLASS_NAME),this._domNode.setPosition("absolute"),this._domNode.setAttribute("role","presentation"),this._domNode.setAttribute("aria-hidden","true"),this._glyphMarginBackgroundDomNode=St(document.createElement("div")),this._glyphMarginBackgroundDomNode.setClassName(Mp.CLASS_NAME),this._domNode.appendChild(this._glyphMarginBackgroundDomNode)}dispose(){super.dispose()}getDomNode(){return this._domNode}onConfigurationChanged(e){const t=this._context.configuration.options,i=t.get(146);return this._canUseLayerHinting=!t.get(32),this._contentLeft=i.contentLeft,this._glyphMarginLeft=i.glyphMarginLeft,this._glyphMarginWidth=i.glyphMarginWidth,!0}onScrollChanged(e){return super.onScrollChanged(e)||e.scrollTopChanged}prepareRender(e){}render(e){this._domNode.setLayerHinting(this._canUseLayerHinting),this._domNode.setContain("strict");const t=e.scrollTop-e.bigNumbersDelta;this._domNode.setTop(-t);const i=Math.min(e.scrollHeight,1e6);this._domNode.setHeight(i),this._domNode.setWidth(this._contentLeft),this._glyphMarginBackgroundDomNode.setLeft(this._glyphMarginLeft),this._glyphMarginBackgroundDomNode.setWidth(this._glyphMarginWidth),this._glyphMarginBackgroundDomNode.setHeight(i)}};Mp.CLASS_NAME="glyph-margin",Mp.OUTER_CLASS_NAME="margin";let ax=Mp;const s_="monaco-mouse-cursor-text";var f_e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},SH=function(o,e){return function(t,i){e(t,i,o)}};class m_e{constructor(e,t,i,n,s){this._context=e,this.modelLineNumber=t,this.distanceToModelLineStart=i,this.widthOfHiddenLineTextBefore=n,this.distanceToModelLineEnd=s,this._visibleTextAreaBrand=void 0,this.startPosition=null,this.endPosition=null,this.visibleTextareaStart=null,this.visibleTextareaEnd=null,this._previousPresentation=null}prepareRender(e){const t=new A(this.modelLineNumber,this.distanceToModelLineStart+1),i=new A(this.modelLineNumber,this._context.viewModel.model.getLineMaxColumn(this.modelLineNumber)-this.distanceToModelLineEnd);this.startPosition=this._context.viewModel.coordinatesConverter.convertModelPositionToViewPosition(t),this.endPosition=this._context.viewModel.coordinatesConverter.convertModelPositionToViewPosition(i),this.startPosition.lineNumber===this.endPosition.lineNumber?(this.visibleTextareaStart=e.visibleRangeForPosition(this.startPosition),this.visibleTextareaEnd=e.visibleRangeForPosition(this.endPosition)):(this.visibleTextareaStart=null,this.visibleTextareaEnd=null)}definePresentation(e){return this._previousPresentation||(e?this._previousPresentation=e:this._previousPresentation={foreground:1,italic:!1,bold:!1,underline:!1,strikethrough:!1}),this._previousPresentation}}const _T=pr;let hP=class extends bo{constructor(e,t,i,n,s){super(e),this._keybindingService=n,this._instantiationService=s,this._primaryCursorPosition=new A(1,1),this._primaryCursorVisibleRange=null,this._viewController=t,this._visibleRangeProvider=i,this._scrollLeft=0,this._scrollTop=0;const r=this._context.configuration.options,a=r.get(146);this._setAccessibilityOptions(r),this._contentLeft=a.contentLeft,this._contentWidth=a.contentWidth,this._contentHeight=a.height,this._fontInfo=r.get(50),this._lineHeight=r.get(67),this._emptySelectionClipboard=r.get(37),this._copyWithSyntaxHighlighting=r.get(25),this._visibleTextArea=null,this._selections=[new pe(1,1,1,1)],this._modelSelections=[new pe(1,1,1,1)],this._lastRenderPosition=null,this.textArea=St(document.createElement("textarea")),Va.write(this.textArea,7),this.textArea.setClassName(`inputarea ${s_}`),this.textArea.setAttribute("wrap",this._textAreaWrapping&&!this._visibleTextArea?"on":"off");const{tabSize:l}=this._context.viewModel.model.getOptions();this.textArea.domNode.style.tabSize=`${l*this._fontInfo.spaceWidth}px`,this.textArea.setAttribute("autocorrect","off"),this.textArea.setAttribute("autocapitalize","off"),this.textArea.setAttribute("autocomplete","off"),this.textArea.setAttribute("spellcheck","false"),this.textArea.setAttribute("aria-label",this._getAriaLabel(r)),this.textArea.setAttribute("aria-required",r.get(5)?"true":"false"),this.textArea.setAttribute("tabindex",String(r.get(125))),this.textArea.setAttribute("role","textbox"),this.textArea.setAttribute("aria-roledescription",m("editor","editor")),this.textArea.setAttribute("aria-multiline","true"),this.textArea.setAttribute("aria-autocomplete",r.get(92)?"none":"both"),this._ensureReadOnlyAttribute(),this.textAreaCover=St(document.createElement("div")),this.textAreaCover.setPosition("absolute");const c={getLineCount:()=>this._context.viewModel.getLineCount(),getLineMaxColumn:u=>this._context.viewModel.getLineMaxColumn(u),getValueInRange:(u,g)=>this._context.viewModel.getValueInRange(u,g),getValueLengthInRange:(u,g)=>this._context.viewModel.getValueLengthInRange(u,g),modifyPosition:(u,g)=>this._context.viewModel.modifyPosition(u,g)},d={getDataToCopy:()=>{const u=this._context.viewModel.getPlainTextToCopy(this._modelSelections,this._emptySelectionClipboard,En),g=this._context.viewModel.model.getEOL(),f=this._emptySelectionClipboard&&this._modelSelections.length===1&&this._modelSelections[0].isEmpty(),p=Array.isArray(u)?u:null,_=Array.isArray(u)?u.join(g):u;let b,C=null;if(lP.forceCopyWithSyntaxHighlighting||this._copyWithSyntaxHighlighting&&_.length<65536){const v=this._context.viewModel.getRichTextToCopy(this._modelSelections,this._emptySelectionClipboard);v&&(b=v.html,C=v.mode)}return{isFromEmptySelection:f,multicursorText:p,text:_,html:b,mode:C}},getScreenReaderContent:()=>{if(this._accessibilitySupport===1){const u=this._selections[0];if(Je&&u.isEmpty()){const f=u.getStartPosition();let p=this._getWordBeforePosition(f);if(p.length===0&&(p=this._getCharacterBeforePosition(f)),p.length>0)return new qn(p,p.length,p.length,x.fromPositions(f),0)}if(Je&&!u.isEmpty()&&c.getValueLengthInRange(u,0)<500){const f=c.getValueInRange(u,0);return new qn(f,0,f.length,u,0)}if(Wh&&!u.isEmpty()){const f="vscode-placeholder";return new qn(f,0,f.length,null,void 0)}return qn.EMPTY}if(QB){const u=this._selections[0];if(u.isEmpty()){const g=u.getStartPosition(),[f,p]=this._getAndroidWordAtPosition(g);if(f.length>0)return new qn(f,p,p,x.fromPositions(g),0)}return qn.EMPTY}return hp.fromEditorSelection(c,this._selections[0],this._accessibilityPageSize,this._accessibilitySupport===0)},deduceModelPosition:(u,g,f)=>this._context.viewModel.deduceModelPositionRelativeToViewPosition(u,g,f)},h=this._register(new d_e(this.textArea.domNode));this._textAreaInput=this._register(this._instantiationService.createInstance(cP,d,h,so,{isAndroid:QB,isChrome:O1,isFirefox:pr,isSafari:Wh})),this._register(this._textAreaInput.onKeyDown(u=>{this._viewController.emitKeyDown(u)})),this._register(this._textAreaInput.onKeyUp(u=>{this._viewController.emitKeyUp(u)})),this._register(this._textAreaInput.onPaste(u=>{let g=!1,f=null,p=null;u.metadata&&(g=this._emptySelectionClipboard&&!!u.metadata.isFromEmptySelection,f=typeof u.metadata.multicursorText<"u"?u.metadata.multicursorText:null,p=u.metadata.mode),this._viewController.paste(u.text,g,f,p)})),this._register(this._textAreaInput.onCut(()=>{this._viewController.cut()})),this._register(this._textAreaInput.onType(u=>{u.replacePrevCharCnt||u.replaceNextCharCnt||u.positionDelta?this._viewController.compositionType(u.text,u.replacePrevCharCnt,u.replaceNextCharCnt,u.positionDelta):this._viewController.type(u.text)})),this._register(this._textAreaInput.onSelectionChangeRequest(u=>{this._viewController.setSelection(u)})),this._register(this._textAreaInput.onCompositionStart(u=>{const g=this.textArea.domNode,f=this._modelSelections[0],{distanceToModelLineStart:p,widthOfHiddenTextBefore:_}=(()=>{const C=g.value.substring(0,Math.min(g.selectionStart,g.selectionEnd)),v=C.lastIndexOf(`
`),w=C.substring(v+1),S=w.lastIndexOf("	"),L=w.length-S-1,k=f.getStartPosition(),D=Math.min(k.column-1,L),N=k.column-1-D,B=w.substring(0,w.length-D),{tabSize:W}=this._context.viewModel.model.getOptions(),V=p_e(this.textArea.domNode.ownerDocument,B,this._fontInfo,W);return{distanceToModelLineStart:N,widthOfHiddenTextBefore:V}})(),{distanceToModelLineEnd:b}=(()=>{const C=g.value.substring(Math.max(g.selectionStart,g.selectionEnd)),v=C.indexOf(`
`),w=v===-1?C:C.substring(0,v),S=w.indexOf("	"),L=S===-1?w.length:w.length-S-1,k=f.getEndPosition(),D=Math.min(this._context.viewModel.model.getLineMaxColumn(k.lineNumber)-k.column,L);return{distanceToModelLineEnd:this._context.viewModel.model.getLineMaxColumn(k.lineNumber)-k.column-D}})();this._context.viewModel.revealRange("keyboard",!0,x.fromPositions(this._selections[0].getStartPosition()),0,1),this._visibleTextArea=new m_e(this._context,f.startLineNumber,p,_,b),this.textArea.setAttribute("wrap",this._textAreaWrapping&&!this._visibleTextArea?"on":"off"),this._visibleTextArea.prepareRender(this._visibleRangeProvider),this._render(),this.textArea.setClassName(`inputarea ${s_} ime-input`),this._viewController.compositionStart(),this._context.viewModel.onCompositionStart()})),this._register(this._textAreaInput.onCompositionUpdate(u=>{this._visibleTextArea&&(this._visibleTextArea.prepareRender(this._visibleRangeProvider),this._render())})),this._register(this._textAreaInput.onCompositionEnd(()=>{this._visibleTextArea=null,this.textArea.setAttribute("wrap",this._textAreaWrapping&&!this._visibleTextArea?"on":"off"),this._render(),this.textArea.setClassName(`inputarea ${s_}`),this._viewController.compositionEnd(),this._context.viewModel.onCompositionEnd()})),this._register(this._textAreaInput.onFocus(()=>{this._context.viewModel.setHasFocus(!0)})),this._register(this._textAreaInput.onBlur(()=>{this._context.viewModel.setHasFocus(!1)})),this._register(YC.onDidChange(()=>{this._ensureReadOnlyAttribute()}))}writeScreenReaderContent(e){this._textAreaInput.writeNativeTextAreaContent(e)}dispose(){super.dispose()}_getAndroidWordAtPosition(e){const t='`~!@#$%^&*()-=+[{]}\\|;:",.<>/?',i=this._context.viewModel.getLineContent(e.lineNumber),n=Cr(t,[]);let s=!0,r=e.column,a=!0,l=e.column,c=0;for(;c<50&&(s||a);){if(s&&r<=1&&(s=!1),s){const d=i.charCodeAt(r-2);n.get(d)!==0?s=!1:r--}if(a&&l>i.length&&(a=!1),a){const d=i.charCodeAt(l-1);n.get(d)!==0?a=!1:l++}c++}return[i.substring(r-1,l-1),e.column-r]}_getWordBeforePosition(e){const t=this._context.viewModel.getLineContent(e.lineNumber),i=Cr(this._context.configuration.options.get(132),[]);let n=e.column,s=0;for(;n>1;){const r=t.charCodeAt(n-2);if(i.get(r)!==0||s>50)return t.substring(n-1,e.column-1);s++,n--}return t.substring(0,e.column-1)}_getCharacterBeforePosition(e){if(e.column>1){const i=this._context.viewModel.getLineContent(e.lineNumber).charAt(e.column-2);if(!Gi(i.charCodeAt(0)))return i}return""}_getAriaLabel(e){var i,n,s;if(e.get(2)===1){const r=(i=this._keybindingService.lookupKeybinding("editor.action.toggleScreenReaderAccessibilityMode"))==null?void 0:i.getAriaLabel(),a=(n=this._keybindingService.lookupKeybinding("workbench.action.showCommands"))==null?void 0:n.getAriaLabel(),l=(s=this._keybindingService.lookupKeybinding("workbench.action.openGlobalKeybindings"))==null?void 0:s.getAriaLabel(),c=m("accessibilityModeOff","The editor is not accessible at this time.");return r?m("accessibilityOffAriaLabel","{0} To enable screen reader optimized mode, use {1}",c,r):a?m("accessibilityOffAriaLabelNoKb","{0} To enable screen reader optimized mode, open the quick pick with {1} and run the command Toggle Screen Reader Accessibility Mode, which is currently not triggerable via keyboard.",c,a):l?m("accessibilityOffAriaLabelNoKbs","{0} Please assign a keybinding for the command Toggle Screen Reader Accessibility Mode by accessing the keybindings editor with {1} and run it.",c,l):c}return e.get(4)}_setAccessibilityOptions(e){this._accessibilitySupport=e.get(2);const t=e.get(3);this._accessibilitySupport===2&&t===Ua.accessibilityPageSize.defaultValue?this._accessibilityPageSize=500:this._accessibilityPageSize=t;const n=e.get(146).wrappingColumn;if(n!==-1&&this._accessibilitySupport!==1){const s=e.get(50);this._textAreaWrapping=!0,this._textAreaWidth=Math.round(n*s.typicalHalfwidthCharacterWidth)}else this._textAreaWrapping=!1,this._textAreaWidth=_T?0:1}onConfigurationChanged(e){const t=this._context.configuration.options,i=t.get(146);this._setAccessibilityOptions(t),this._contentLeft=i.contentLeft,this._contentWidth=i.contentWidth,this._contentHeight=i.height,this._fontInfo=t.get(50),this._lineHeight=t.get(67),this._emptySelectionClipboard=t.get(37),this._copyWithSyntaxHighlighting=t.get(25),this.textArea.setAttribute("wrap",this._textAreaWrapping&&!this._visibleTextArea?"on":"off");const{tabSize:n}=this._context.viewModel.model.getOptions();return this.textArea.domNode.style.tabSize=`${n*this._fontInfo.spaceWidth}px`,this.textArea.setAttribute("aria-label",this._getAriaLabel(t)),this.textArea.setAttribute("aria-required",t.get(5)?"true":"false"),this.textArea.setAttribute("tabindex",String(t.get(125))),(e.hasChanged(34)||e.hasChanged(92))&&this._ensureReadOnlyAttribute(),e.hasChanged(2)&&this._textAreaInput.writeNativeTextAreaContent("strategy changed"),!0}onCursorStateChanged(e){return this._selections=e.selections.slice(0),this._modelSelections=e.modelSelections.slice(0),this._textAreaInput.writeNativeTextAreaContent("selection changed"),!0}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return this._scrollLeft=e.scrollLeft,this._scrollTop=e.scrollTop,!0}onZonesChanged(e){return!0}isFocused(){return this._textAreaInput.isFocused()}focusTextArea(){this._textAreaInput.focusTextArea()}getLastRenderData(){return this._lastRenderPosition}setAriaOptions(e){e.activeDescendant?(this.textArea.setAttribute("aria-haspopup","true"),this.textArea.setAttribute("aria-autocomplete","list"),this.textArea.setAttribute("aria-activedescendant",e.activeDescendant)):(this.textArea.setAttribute("aria-haspopup","false"),this.textArea.setAttribute("aria-autocomplete","both"),this.textArea.removeAttribute("aria-activedescendant")),e.role&&this.textArea.setAttribute("role",e.role)}_ensureReadOnlyAttribute(){const e=this._context.configuration.options;!YC.enabled||e.get(34)&&e.get(92)?this.textArea.setAttribute("readonly","true"):this.textArea.removeAttribute("readonly")}prepareRender(e){var t;this._primaryCursorPosition=new A(this._selections[0].positionLineNumber,this._selections[0].positionColumn),this._primaryCursorVisibleRange=e.visibleRangeForPosition(this._primaryCursorPosition),(t=this._visibleTextArea)==null||t.prepareRender(e)}render(e){this._textAreaInput.writeNativeTextAreaContent("render"),this._render()}_render(){if(this._visibleTextArea){const i=this._visibleTextArea.visibleTextareaStart,n=this._visibleTextArea.visibleTextareaEnd,s=this._visibleTextArea.startPosition,r=this._visibleTextArea.endPosition;if(s&&r&&i&&n&&n.left>=this._scrollLeft&&i.left<=this._scrollLeft+this._contentWidth){const a=this._context.viewLayout.getVerticalOffsetForLineNumber(this._primaryCursorPosition.lineNumber)-this._scrollTop,l=this._newlinecount(this.textArea.domNode.value.substr(0,this.textArea.domNode.selectionStart));let c=this._visibleTextArea.widthOfHiddenLineTextBefore,d=this._contentLeft+i.left-this._scrollLeft,h=n.left-i.left+1;if(d<this._contentLeft){const b=this._contentLeft-d;d+=b,c+=b,h-=b}h>this._contentWidth&&(h=this._contentWidth);const u=this._context.viewModel.getViewLineData(s.lineNumber),g=u.tokens.findTokenIndexAtOffset(s.column-1),f=u.tokens.findTokenIndexAtOffset(r.column-1),p=g===f,_=this._visibleTextArea.definePresentation(p?u.tokens.getPresentation(g):null);this.textArea.domNode.scrollTop=l*this._lineHeight,this.textArea.domNode.scrollLeft=c,this._doRender({lastRenderPosition:null,top:a,left:d,width:h,height:this._lineHeight,useCover:!1,color:(fi.getColorMap()||[])[_.foreground],italic:_.italic,bold:_.bold,underline:_.underline,strikethrough:_.strikethrough})}return}if(!this._primaryCursorVisibleRange){this._renderAtTopLeft();return}const e=this._contentLeft+this._primaryCursorVisibleRange.left-this._scrollLeft;if(e<this._contentLeft||e>this._contentLeft+this._contentWidth){this._renderAtTopLeft();return}const t=this._context.viewLayout.getVerticalOffsetForLineNumber(this._selections[0].positionLineNumber)-this._scrollTop;if(t<0||t>this._contentHeight){this._renderAtTopLeft();return}if(Je||this._accessibilitySupport===2){this._doRender({lastRenderPosition:this._primaryCursorPosition,top:t,left:this._textAreaWrapping?this._contentLeft:e,width:this._textAreaWidth,height:this._lineHeight,useCover:!1}),this.textArea.domNode.scrollLeft=this._primaryCursorVisibleRange.left;const i=this._textAreaInput.textAreaState.newlineCountBeforeSelection??this._newlinecount(this.textArea.domNode.value.substr(0,this.textArea.domNode.selectionStart));this.textArea.domNode.scrollTop=i*this._lineHeight;return}this._doRender({lastRenderPosition:this._primaryCursorPosition,top:t,left:this._textAreaWrapping?this._contentLeft:e,width:this._textAreaWidth,height:_T?0:1,useCover:!1})}_newlinecount(e){let t=0,i=-1;do{if(i=e.indexOf(`
`,i+1),i===-1)break;t++}while(!0);return t}_renderAtTopLeft(){this._doRender({lastRenderPosition:null,top:0,left:0,width:this._textAreaWidth,height:_T?0:1,useCover:!0})}_doRender(e){this._lastRenderPosition=e.lastRenderPosition;const t=this.textArea,i=this.textAreaCover;bn(t,this._fontInfo),t.setTop(e.top),t.setLeft(e.left),t.setWidth(e.width),t.setHeight(e.height),t.setColor(e.color?Z.Format.CSS.formatHex(e.color):""),t.setFontStyle(e.italic?"italic":""),e.bold&&t.setFontWeight("bold"),t.setTextDecoration(`${e.underline?" underline":""}${e.strikethrough?" line-through":""}`),i.setTop(e.useCover?e.top:0),i.setLeft(e.useCover?e.left:0),i.setWidth(e.useCover?e.width:0),i.setHeight(e.useCover?e.height:0);const n=this._context.configuration.options;n.get(57)?i.setClassName("monaco-editor-background textAreaCover "+ax.OUTER_CLASS_NAME):n.get(68).renderType!==0?i.setClassName("monaco-editor-background textAreaCover "+rx.CLASS_NAME):i.setClassName("monaco-editor-background textAreaCover")}};hP=f_e([SH(3,yt),SH(4,we)],hP);function p_e(o,e,t,i){if(e.length===0)return 0;const n=o.createElement("div");n.style.position="absolute",n.style.top="-50000px",n.style.width="50000px";const s=o.createElement("span");bn(s,t),s.style.whiteSpace="pre",s.style.tabSize=`${i*t.spaceWidth}px`,s.append(e),n.appendChild(s),o.body.appendChild(n);const r=s.offsetWidth;return n.remove(),r}const __e=()=>!0,b_e=()=>!1,C_e=o=>o===" "||o==="	";class Nm{static shouldRecreate(e){return e.hasChanged(146)||e.hasChanged(132)||e.hasChanged(37)||e.hasChanged(77)||e.hasChanged(79)||e.hasChanged(80)||e.hasChanged(6)||e.hasChanged(7)||e.hasChanged(11)||e.hasChanged(9)||e.hasChanged(10)||e.hasChanged(14)||e.hasChanged(129)||e.hasChanged(50)||e.hasChanged(92)||e.hasChanged(131)}constructor(e,t,i,n){this.languageConfigurationService=n,this._cursorMoveConfigurationBrand=void 0,this._languageId=e;const s=i.options,r=s.get(146),a=s.get(50);this.readOnly=s.get(92),this.tabSize=t.tabSize,this.indentSize=t.indentSize,this.insertSpaces=t.insertSpaces,this.stickyTabStops=s.get(117),this.lineHeight=a.lineHeight,this.typicalHalfwidthCharacterWidth=a.typicalHalfwidthCharacterWidth,this.pageSize=Math.max(1,Math.floor(r.height/this.lineHeight)-2),this.useTabStops=s.get(129),this.wordSeparators=s.get(132),this.emptySelectionClipboard=s.get(37),this.copyWithSyntaxHighlighting=s.get(25),this.multiCursorMergeOverlapping=s.get(77),this.multiCursorPaste=s.get(79),this.multiCursorLimit=s.get(80),this.autoClosingBrackets=s.get(6),this.autoClosingComments=s.get(7),this.autoClosingQuotes=s.get(11),this.autoClosingDelete=s.get(9),this.autoClosingOvertype=s.get(10),this.autoSurround=s.get(14),this.autoIndent=s.get(12),this.wordSegmenterLocales=s.get(131),this.surroundingPairs={},this._electricChars=null,this.shouldAutoCloseBefore={quote:this._getShouldAutoClose(e,this.autoClosingQuotes,!0),comment:this._getShouldAutoClose(e,this.autoClosingComments,!1),bracket:this._getShouldAutoClose(e,this.autoClosingBrackets,!1)},this.autoClosingPairs=this.languageConfigurationService.getLanguageConfiguration(e).getAutoClosingPairs();const l=this.languageConfigurationService.getLanguageConfiguration(e).getSurroundingPairs();if(l)for(const d of l)this.surroundingPairs[d.open]=d.close;const c=this.languageConfigurationService.getLanguageConfiguration(e).comments;this.blockCommentStartToken=(c==null?void 0:c.blockCommentStartToken)??null}get electricChars(){var e;if(!this._electricChars){this._electricChars={};const t=(e=this.languageConfigurationService.getLanguageConfiguration(this._languageId).electricCharacter)==null?void 0:e.getElectricCharacters();if(t)for(const i of t)this._electricChars[i]=!0}return this._electricChars}onElectricCharacter(e,t,i){const n=ag(t,i-1),s=this.languageConfigurationService.getLanguageConfiguration(n.languageId).electricCharacter;return s?s.onElectricCharacter(e,n,i-n.firstCharOffset):null}normalizeIndentation(e){return a3(e,this.indentSize,this.insertSpaces)}_getShouldAutoClose(e,t,i){switch(t){case"beforeWhitespace":return C_e;case"languageDefined":return this._getLanguageDefinedShouldAutoClose(e,i);case"always":return __e;case"never":return b_e}}_getLanguageDefinedShouldAutoClose(e,t){const i=this.languageConfigurationService.getLanguageConfiguration(e).getAutoCloseBeforeSet(t);return n=>i.indexOf(n)!==-1}visibleColumnFromColumn(e,t){return Ui.visibleColumnFromColumn(e.getLineContent(t.lineNumber),t.column,this.tabSize)}columnFromVisibleColumn(e,t,i){const n=Ui.columnFromVisibleColumn(e.getLineContent(t),i,this.tabSize),s=e.getLineMinColumn(t);if(n<s)return s;const r=e.getLineMaxColumn(t);return n>r?r:n}}let ft=class Cq{static fromModelState(e){return new v_e(e)}static fromViewState(e){return new w_e(e)}static fromModelSelection(e){const t=pe.liftSelection(e),i=new gn(x.fromPositions(t.getSelectionStart()),0,0,t.getPosition(),0);return Cq.fromModelState(i)}static fromModelSelections(e){const t=[];for(let i=0,n=e.length;i<n;i++)t[i]=this.fromModelSelection(e[i]);return t}constructor(e,t){this._cursorStateBrand=void 0,this.modelState=e,this.viewState=t}equals(e){return this.viewState.equals(e.viewState)&&this.modelState.equals(e.modelState)}};class v_e{constructor(e){this.modelState=e,this.viewState=null}}class w_e{constructor(e){this.modelState=null,this.viewState=e}}class gn{constructor(e,t,i,n,s){this.selectionStart=e,this.selectionStartKind=t,this.selectionStartLeftoverVisibleColumns=i,this.position=n,this.leftoverVisibleColumns=s,this._singleCursorStateBrand=void 0,this.selection=gn._computeSelection(this.selectionStart,this.position)}equals(e){return this.selectionStartLeftoverVisibleColumns===e.selectionStartLeftoverVisibleColumns&&this.leftoverVisibleColumns===e.leftoverVisibleColumns&&this.selectionStartKind===e.selectionStartKind&&this.position.equals(e.position)&&this.selectionStart.equalsRange(e.selectionStart)}hasSelection(){return!this.selection.isEmpty()||!this.selectionStart.isEmpty()}move(e,t,i,n){return e?new gn(this.selectionStart,this.selectionStartKind,this.selectionStartLeftoverVisibleColumns,new A(t,i),n):new gn(new x(t,i,t,i),0,n,new A(t,i),n)}static _computeSelection(e,t){return e.isEmpty()||!t.isBeforeOrEqual(e.getStartPosition())?pe.fromPositions(e.getStartPosition(),t):pe.fromPositions(e.getEndPosition(),t)}}class Fs{constructor(e,t,i){this._editOperationResultBrand=void 0,this.type=e,this.commands=t,this.shouldPushStackElementBefore=i.shouldPushStackElementBefore,this.shouldPushStackElementAfter=i.shouldPushStackElementAfter}}function Gh(o){return o==="'"||o==='"'||o==="`"}class Xu{static columnSelect(e,t,i,n,s,r){const a=Math.abs(s-i)+1,l=i>s,c=n>r,d=n<r,h=[];for(let u=0;u<a;u++){const g=i+(l?-u:u),f=e.columnFromVisibleColumn(t,g,n),p=e.columnFromVisibleColumn(t,g,r),_=e.visibleColumnFromColumn(t,new A(g,f)),b=e.visibleColumnFromColumn(t,new A(g,p));d&&(_>r||b<n)||c&&(b>n||_<r)||h.push(new gn(new x(g,f,g,f),0,0,new A(g,p),0))}if(h.length===0)for(let u=0;u<a;u++){const g=i+(l?-u:u),f=t.getLineMaxColumn(g);h.push(new gn(new x(g,f,g,f),0,0,new A(g,f),0))}return{viewStates:h,reversed:l,fromLineNumber:i,fromVisualColumn:n,toLineNumber:s,toVisualColumn:r}}static columnSelectLeft(e,t,i){let n=i.toViewVisualColumn;return n>0&&n--,Xu.columnSelect(e,t,i.fromViewLineNumber,i.fromViewVisualColumn,i.toViewLineNumber,n)}static columnSelectRight(e,t,i){let n=0;const s=Math.min(i.fromViewLineNumber,i.toViewLineNumber),r=Math.max(i.fromViewLineNumber,i.toViewLineNumber);for(let l=s;l<=r;l++){const c=t.getLineMaxColumn(l),d=e.visibleColumnFromColumn(t,new A(l,c));n=Math.max(n,d)}let a=i.toViewVisualColumn;return a<n&&a++,this.columnSelect(e,t,i.fromViewLineNumber,i.fromViewVisualColumn,i.toViewLineNumber,a)}static columnSelectUp(e,t,i,n){const s=n?e.pageSize:1,r=Math.max(1,i.toViewLineNumber-s);return this.columnSelect(e,t,i.fromViewLineNumber,i.fromViewVisualColumn,r,i.toViewVisualColumn)}static columnSelectDown(e,t,i,n){const s=n?e.pageSize:1,r=Math.min(t.getLineCount(),i.toViewLineNumber+s);return this.columnSelect(e,t,i.fromViewLineNumber,i.fromViewVisualColumn,r,i.toViewVisualColumn)}}class kn{constructor(e,t,i=!1){this._range=e,this._text=t,this.insertsAutoWhitespace=i}getEditOperations(e,t){t.addTrackedEditOperation(this._range,this._text)}computeCursorState(e,t){const n=t.getInverseEditOperations()[0].range;return pe.fromPositions(n.getEndPosition())}}class S_e{constructor(e,t){this._range=e,this._text=t}getEditOperations(e,t){t.addTrackedEditOperation(this._range,this._text)}computeCursorState(e,t){const n=t.getInverseEditOperations()[0].range;return pe.fromRange(n,0)}}class ly{constructor(e,t,i=!1){this._range=e,this._text=t,this.insertsAutoWhitespace=i}getEditOperations(e,t){t.addTrackedEditOperation(this._range,this._text)}computeCursorState(e,t){const n=t.getInverseEditOperations()[0].range;return pe.fromPositions(n.getStartPosition())}}class lx{constructor(e,t,i,n,s=!1){this._range=e,this._text=t,this._columnDeltaOffset=n,this._lineNumberDeltaOffset=i,this.insertsAutoWhitespace=s}getEditOperations(e,t){t.addTrackedEditOperation(this._range,this._text)}computeCursorState(e,t){const n=t.getInverseEditOperations()[0].range;return pe.fromPositions(n.getEndPosition().delta(this._lineNumberDeltaOffset,this._columnDeltaOffset))}}class j3{constructor(e,t,i,n=!1){this._range=e,this._text=t,this._initialSelection=i,this._forceMoveMarkers=n,this._selectionId=null}getEditOperations(e,t){t.addTrackedEditOperation(this._range,this._text,this._forceMoveMarkers),this._selectionId=t.trackSelection(this._initialSelection)}computeCursorState(e,t){return t.getTrackedSelection(this._selectionId)}}class bT{constructor(e,t,i){this._cursorPositionBrand=void 0,this.lineNumber=e,this.column=t,this.leftoverVisibleColumns=i}}class Lt{static leftPosition(e,t){if(t.column>e.getLineMinColumn(t.lineNumber))return t.delta(void 0,-iU(e.getLineContent(t.lineNumber),t.column-1));if(t.lineNumber>1){const i=t.lineNumber-1;return new A(i,e.getLineMaxColumn(i))}else return t}static leftPositionAtomicSoftTabs(e,t,i){if(t.column<=e.getLineIndentColumn(t.lineNumber)){const n=e.getLineMinColumn(t.lineNumber),s=e.getLineContent(t.lineNumber),r=$w.atomicPosition(s,t.column-1,i,0);if(r!==-1&&r+1>=n)return new A(t.lineNumber,r+1)}return this.leftPosition(e,t)}static left(e,t,i){const n=e.stickyTabStops?Lt.leftPositionAtomicSoftTabs(t,i,e.tabSize):Lt.leftPosition(t,i);return new bT(n.lineNumber,n.column,0)}static moveLeft(e,t,i,n,s){let r,a;if(i.hasSelection()&&!n)r=i.selection.startLineNumber,a=i.selection.startColumn;else{const l=i.position.delta(void 0,-(s-1)),c=t.normalizePosition(Lt.clipPositionColumn(l,t),0),d=Lt.left(e,t,c);r=d.lineNumber,a=d.column}return i.move(n,r,a,0)}static clipPositionColumn(e,t){return new A(e.lineNumber,Lt.clipRange(e.column,t.getLineMinColumn(e.lineNumber),t.getLineMaxColumn(e.lineNumber)))}static clipRange(e,t,i){return e<t?t:e>i?i:e}static rightPosition(e,t,i){return i<e.getLineMaxColumn(t)?i=i+J4(e.getLineContent(t),i-1):t<e.getLineCount()&&(t=t+1,i=e.getLineMinColumn(t)),new A(t,i)}static rightPositionAtomicSoftTabs(e,t,i,n,s){if(i<e.getLineIndentColumn(t)){const r=e.getLineContent(t),a=$w.atomicPosition(r,i-1,n,1);if(a!==-1)return new A(t,a+1)}return this.rightPosition(e,t,i)}static right(e,t,i){const n=e.stickyTabStops?Lt.rightPositionAtomicSoftTabs(t,i.lineNumber,i.column,e.tabSize,e.indentSize):Lt.rightPosition(t,i.lineNumber,i.column);return new bT(n.lineNumber,n.column,0)}static moveRight(e,t,i,n,s){let r,a;if(i.hasSelection()&&!n)r=i.selection.endLineNumber,a=i.selection.endColumn;else{const l=i.position.delta(void 0,s-1),c=t.normalizePosition(Lt.clipPositionColumn(l,t),1),d=Lt.right(e,t,c);r=d.lineNumber,a=d.column}return i.move(n,r,a,0)}static vertical(e,t,i,n,s,r,a,l){const c=Ui.visibleColumnFromColumn(t.getLineContent(i),n,e.tabSize)+s,d=t.getLineCount(),h=i===1&&n===1,u=i===d&&n===t.getLineMaxColumn(i),g=r<i?h:u;if(i=r,i<1?(i=1,a?n=t.getLineMinColumn(i):n=Math.min(t.getLineMaxColumn(i),n)):i>d?(i=d,a?n=t.getLineMaxColumn(i):n=Math.min(t.getLineMaxColumn(i),n)):n=e.columnFromVisibleColumn(t,i,c),g?s=0:s=c-Ui.visibleColumnFromColumn(t.getLineContent(i),n,e.tabSize),l!==void 0){const f=new A(i,n),p=t.normalizePosition(f,l);s=s+(n-p.column),i=p.lineNumber,n=p.column}return new bT(i,n,s)}static down(e,t,i,n,s,r,a){return this.vertical(e,t,i,n,s,i+r,a,4)}static moveDown(e,t,i,n,s){let r,a;i.hasSelection()&&!n?(r=i.selection.endLineNumber,a=i.selection.endColumn):(r=i.position.lineNumber,a=i.position.column);let l=0,c;do if(c=Lt.down(e,t,r+l,a,i.leftoverVisibleColumns,s,!0),t.normalizePosition(new A(c.lineNumber,c.column),2).lineNumber>r)break;while(l++<10&&r+l<t.getLineCount());return i.move(n,c.lineNumber,c.column,c.leftoverVisibleColumns)}static translateDown(e,t,i){const n=i.selection,s=Lt.down(e,t,n.selectionStartLineNumber,n.selectionStartColumn,i.selectionStartLeftoverVisibleColumns,1,!1),r=Lt.down(e,t,n.positionLineNumber,n.positionColumn,i.leftoverVisibleColumns,1,!1);return new gn(new x(s.lineNumber,s.column,s.lineNumber,s.column),0,s.leftoverVisibleColumns,new A(r.lineNumber,r.column),r.leftoverVisibleColumns)}static up(e,t,i,n,s,r,a){return this.vertical(e,t,i,n,s,i-r,a,3)}static moveUp(e,t,i,n,s){let r,a;i.hasSelection()&&!n?(r=i.selection.startLineNumber,a=i.selection.startColumn):(r=i.position.lineNumber,a=i.position.column);const l=Lt.up(e,t,r,a,i.leftoverVisibleColumns,s,!0);return i.move(n,l.lineNumber,l.column,l.leftoverVisibleColumns)}static translateUp(e,t,i){const n=i.selection,s=Lt.up(e,t,n.selectionStartLineNumber,n.selectionStartColumn,i.selectionStartLeftoverVisibleColumns,1,!1),r=Lt.up(e,t,n.positionLineNumber,n.positionColumn,i.leftoverVisibleColumns,1,!1);return new gn(new x(s.lineNumber,s.column,s.lineNumber,s.column),0,s.leftoverVisibleColumns,new A(r.lineNumber,r.column),r.leftoverVisibleColumns)}static _isBlankLine(e,t){return e.getLineFirstNonWhitespaceColumn(t)===0}static moveToPrevBlankLine(e,t,i,n){let s=i.position.lineNumber;for(;s>1&&this._isBlankLine(t,s);)s--;for(;s>1&&!this._isBlankLine(t,s);)s--;return i.move(n,s,t.getLineMinColumn(s),0)}static moveToNextBlankLine(e,t,i,n){const s=t.getLineCount();let r=i.position.lineNumber;for(;r<s&&this._isBlankLine(t,r);)r++;for(;r<s&&!this._isBlankLine(t,r);)r++;return i.move(n,r,t.getLineMinColumn(r),0)}static moveToBeginningOfLine(e,t,i,n){const s=i.position.lineNumber,r=t.getLineMinColumn(s),a=t.getLineFirstNonWhitespaceColumn(s)||r;let l;return i.position.column===a?l=r:l=a,i.move(n,s,l,0)}static moveToEndOfLine(e,t,i,n,s){const r=i.position.lineNumber,a=t.getLineMaxColumn(r);return i.move(n,r,a,s?1073741824-a:0)}static moveToBeginningOfBuffer(e,t,i,n){return i.move(n,1,1,0)}static moveToEndOfBuffer(e,t,i,n){const s=t.getLineCount(),r=t.getLineMaxColumn(s);return i.move(n,s,r,0)}}class Bf{static deleteRight(e,t,i,n){const s=[];let r=e!==3;for(let a=0,l=n.length;a<l;a++){const c=n[a];let d=c;if(d.isEmpty()){const h=c.getPosition(),u=Lt.right(t,i,h);d=new x(u.lineNumber,u.column,h.lineNumber,h.column)}if(d.isEmpty()){s[a]=null;continue}d.startLineNumber!==d.endLineNumber&&(r=!0),s[a]=new kn(d,"")}return[r,s]}static isAutoClosingPairDelete(e,t,i,n,s,r,a){if(t==="never"&&i==="never"||e==="never")return!1;for(let l=0,c=r.length;l<c;l++){const d=r[l],h=d.getPosition();if(!d.isEmpty())return!1;const u=s.getLineContent(h.lineNumber);if(h.column<2||h.column>=u.length+1)return!1;const g=u.charAt(h.column-2),f=n.get(g);if(!f)return!1;if(Gh(g)){if(i==="never")return!1}else if(t==="never")return!1;const p=u.charAt(h.column-1);let _=!1;for(const b of f)b.open===g&&b.close===p&&(_=!0);if(!_)return!1;if(e==="auto"){let b=!1;for(let C=0,v=a.length;C<v;C++){const w=a[C];if(h.lineNumber===w.startLineNumber&&h.column===w.startColumn){b=!0;break}}if(!b)return!1}}return!0}static _runAutoClosingPairDelete(e,t,i){const n=[];for(let s=0,r=i.length;s<r;s++){const a=i[s].getPosition(),l=new x(a.lineNumber,a.column-1,a.lineNumber,a.column+1);n[s]=new kn(l,"")}return[!0,n]}static deleteLeft(e,t,i,n,s){if(this.isAutoClosingPairDelete(t.autoClosingDelete,t.autoClosingBrackets,t.autoClosingQuotes,t.autoClosingPairs.autoClosingPairsOpenByEnd,i,n,s))return this._runAutoClosingPairDelete(t,i,n);const r=[];let a=e!==2;for(let l=0,c=n.length;l<c;l++){const d=Bf.getDeleteRange(n[l],i,t);if(d.isEmpty()){r[l]=null;continue}d.startLineNumber!==d.endLineNumber&&(a=!0),r[l]=new kn(d,"")}return[a,r]}static getDeleteRange(e,t,i){if(!e.isEmpty())return e;const n=e.getPosition();if(i.useTabStops&&n.column>1){const s=t.getLineContent(n.lineNumber),r=Jn(s),a=r===-1?s.length+1:r+1;if(n.column<=a){const l=i.visibleColumnFromColumn(t,n),c=Ui.prevIndentTabStop(l,i.indentSize),d=i.columnFromVisibleColumn(t,n.lineNumber,c);return new x(n.lineNumber,d,n.lineNumber,n.column)}}return x.fromPositions(Bf.getPositionAfterDeleteLeft(n,t),n)}static getPositionAfterDeleteLeft(e,t){if(e.column>1){const i=Vee(e.column-1,t.getLineContent(e.lineNumber));return e.with(void 0,i+1)}else if(e.lineNumber>1){const i=e.lineNumber-1;return new A(i,t.getLineMaxColumn(i))}else return e}static cut(e,t,i){const n=[];let s=null;i.sort((r,a)=>A.compare(r.getStartPosition(),a.getEndPosition()));for(let r=0,a=i.length;r<a;r++){const l=i[r];if(l.isEmpty())if(e.emptySelectionClipboard){const c=l.getPosition();let d,h,u,g;c.lineNumber<t.getLineCount()?(d=c.lineNumber,h=1,u=c.lineNumber+1,g=1):c.lineNumber>1&&(s==null?void 0:s.endLineNumber)!==c.lineNumber?(d=c.lineNumber-1,h=t.getLineMaxColumn(c.lineNumber-1),u=c.lineNumber,g=t.getLineMaxColumn(c.lineNumber)):(d=c.lineNumber,h=1,u=c.lineNumber,g=t.getLineMaxColumn(c.lineNumber));const f=new x(d,h,u,g);s=f,f.isEmpty()?n[r]=null:n[r]=new kn(f,"")}else n[r]=null;else n[r]=new kn(l,"")}return new Fs(0,n,{shouldPushStackElementBefore:!0,shouldPushStackElementAfter:!0})}}class wt{static _createWord(e,t,i,n,s){return{start:n,end:s,wordType:t,nextCharClass:i}}static _createIntlWord(e,t){return{start:e.index,end:e.index+e.segment.length,wordType:1,nextCharClass:t}}static _findPreviousWordOnLine(e,t,i){const n=t.getLineContent(i.lineNumber);return this._doFindPreviousWordOnLine(n,e,i)}static _doFindPreviousWordOnLine(e,t,i){let n=0;const s=t.findPrevIntlWordBeforeOrAtOffset(e,i.column-2);for(let r=i.column-2;r>=0;r--){const a=e.charCodeAt(r),l=t.get(a);if(s&&r===s.index)return this._createIntlWord(s,l);if(l===0){if(n===2)return this._createWord(e,n,l,r+1,this._findEndOfWord(e,t,n,r+1));n=1}else if(l===2){if(n===1)return this._createWord(e,n,l,r+1,this._findEndOfWord(e,t,n,r+1));n=2}else if(l===1&&n!==0)return this._createWord(e,n,l,r+1,this._findEndOfWord(e,t,n,r+1))}return n!==0?this._createWord(e,n,1,0,this._findEndOfWord(e,t,n,0)):null}static _findEndOfWord(e,t,i,n){const s=t.findNextIntlWordAtOrAfterOffset(e,n),r=e.length;for(let a=n;a<r;a++){const l=e.charCodeAt(a),c=t.get(l);if(s&&a===s.index+s.segment.length||c===1||i===1&&c===2||i===2&&c===0)return a}return r}static _findNextWordOnLine(e,t,i){const n=t.getLineContent(i.lineNumber);return this._doFindNextWordOnLine(n,e,i)}static _doFindNextWordOnLine(e,t,i){let n=0;const s=e.length,r=t.findNextIntlWordAtOrAfterOffset(e,i.column-1);for(let a=i.column-1;a<s;a++){const l=e.charCodeAt(a),c=t.get(l);if(r&&a===r.index)return this._createIntlWord(r,c);if(c===0){if(n===2)return this._createWord(e,n,c,this._findStartOfWord(e,t,n,a-1),a);n=1}else if(c===2){if(n===1)return this._createWord(e,n,c,this._findStartOfWord(e,t,n,a-1),a);n=2}else if(c===1&&n!==0)return this._createWord(e,n,c,this._findStartOfWord(e,t,n,a-1),a)}return n!==0?this._createWord(e,n,1,this._findStartOfWord(e,t,n,s-1),s):null}static _findStartOfWord(e,t,i,n){const s=t.findPrevIntlWordBeforeOrAtOffset(e,n);for(let r=n;r>=0;r--){const a=e.charCodeAt(r),l=t.get(a);if(s&&r===s.index)return r;if(l===1||i===1&&l===2||i===2&&l===0)return r+1}return 0}static moveWordLeft(e,t,i,n,s){let r=i.lineNumber,a=i.column;a===1&&r>1&&(r=r-1,a=t.getLineMaxColumn(r));let l=wt._findPreviousWordOnLine(e,t,new A(r,a));if(n===0)return new A(r,l?l.start+1:1);if(n===1)return!s&&l&&l.wordType===2&&l.end-l.start===1&&l.nextCharClass===0&&(l=wt._findPreviousWordOnLine(e,t,new A(r,l.start+1))),new A(r,l?l.start+1:1);if(n===3){for(;l&&l.wordType===2;)l=wt._findPreviousWordOnLine(e,t,new A(r,l.start+1));return new A(r,l?l.start+1:1)}return l&&a<=l.end+1&&(l=wt._findPreviousWordOnLine(e,t,new A(r,l.start+1))),new A(r,l?l.end+1:1)}static _moveWordPartLeft(e,t){const i=t.lineNumber,n=e.getLineMaxColumn(i);if(t.column===1)return i>1?new A(i-1,e.getLineMaxColumn(i-1)):t;const s=e.getLineContent(i);for(let r=t.column-1;r>1;r--){const a=s.charCodeAt(r-2),l=s.charCodeAt(r-1);if(a===95&&l!==95)return new A(i,r);if(a===45&&l!==45)return new A(i,r);if((qd(a)||IS(a))&&bl(l))return new A(i,r);if(bl(a)&&bl(l)&&r+1<n){const c=s.charCodeAt(r);if(qd(c)||IS(c))return new A(i,r)}}return new A(i,1)}static moveWordRight(e,t,i,n){let s=i.lineNumber,r=i.column,a=!1;r===t.getLineMaxColumn(s)&&s<t.getLineCount()&&(a=!0,s=s+1,r=1);let l=wt._findNextWordOnLine(e,t,new A(s,r));if(n===2)l&&l.wordType===2&&l.end-l.start===1&&l.nextCharClass===0&&(l=wt._findNextWordOnLine(e,t,new A(s,l.end+1))),l?r=l.end+1:r=t.getLineMaxColumn(s);else if(n===3){for(a&&(r=0);l&&(l.wordType===2||l.start+1<=r);)l=wt._findNextWordOnLine(e,t,new A(s,l.end+1));l?r=l.start+1:r=t.getLineMaxColumn(s)}else l&&!a&&r>=l.start+1&&(l=wt._findNextWordOnLine(e,t,new A(s,l.end+1))),l?r=l.start+1:r=t.getLineMaxColumn(s);return new A(s,r)}static _moveWordPartRight(e,t){const i=t.lineNumber,n=e.getLineMaxColumn(i);if(t.column===n)return i<e.getLineCount()?new A(i+1,1):t;const s=e.getLineContent(i);for(let r=t.column+1;r<n;r++){const a=s.charCodeAt(r-2),l=s.charCodeAt(r-1);if(a!==95&&l===95)return new A(i,r);if(a!==45&&l===45)return new A(i,r);if((qd(a)||IS(a))&&bl(l))return new A(i,r);if(bl(a)&&bl(l)&&r+1<n){const c=s.charCodeAt(r);if(qd(c)||IS(c))return new A(i,r)}}return new A(i,n)}static _deleteWordLeftWhitespace(e,t){const i=e.getLineContent(t.lineNumber),n=t.column-2,s=Na(i,n);return s+1<n?new x(t.lineNumber,s+2,t.lineNumber,t.column):null}static deleteWordLeft(e,t){const i=e.wordSeparators,n=e.model,s=e.selection,r=e.whitespaceHeuristics;if(!s.isEmpty())return s;if(Bf.isAutoClosingPairDelete(e.autoClosingDelete,e.autoClosingBrackets,e.autoClosingQuotes,e.autoClosingPairs.autoClosingPairsOpenByEnd,e.model,[e.selection],e.autoClosedCharacters)){const h=e.selection.getPosition();return new x(h.lineNumber,h.column-1,h.lineNumber,h.column+1)}const a=new A(s.positionLineNumber,s.positionColumn);let l=a.lineNumber,c=a.column;if(l===1&&c===1)return null;if(r){const h=this._deleteWordLeftWhitespace(n,a);if(h)return h}let d=wt._findPreviousWordOnLine(i,n,a);return t===0?d?c=d.start+1:c>1?c=1:(l--,c=n.getLineMaxColumn(l)):(d&&c<=d.end+1&&(d=wt._findPreviousWordOnLine(i,n,new A(l,d.start+1))),d?c=d.end+1:c>1?c=1:(l--,c=n.getLineMaxColumn(l))),new x(l,c,a.lineNumber,a.column)}static deleteInsideWord(e,t,i){if(!i.isEmpty())return i;const n=new A(i.positionLineNumber,i.positionColumn),s=this._deleteInsideWordWhitespace(t,n);return s||this._deleteInsideWordDetermineDeleteRange(e,t,n)}static _charAtIsWhitespace(e,t){const i=e.charCodeAt(t);return i===32||i===9}static _deleteInsideWordWhitespace(e,t){const i=e.getLineContent(t.lineNumber),n=i.length;if(n===0)return null;let s=Math.max(t.column-2,0);if(!this._charAtIsWhitespace(i,s))return null;let r=Math.min(t.column-1,n-1);if(!this._charAtIsWhitespace(i,r))return null;for(;s>0&&this._charAtIsWhitespace(i,s-1);)s--;for(;r+1<n&&this._charAtIsWhitespace(i,r+1);)r++;return new x(t.lineNumber,s+1,t.lineNumber,r+2)}static _deleteInsideWordDetermineDeleteRange(e,t,i){const n=t.getLineContent(i.lineNumber),s=n.length;if(s===0)return i.lineNumber>1?new x(i.lineNumber-1,t.getLineMaxColumn(i.lineNumber-1),i.lineNumber,1):i.lineNumber<t.getLineCount()?new x(i.lineNumber,1,i.lineNumber+1,1):new x(i.lineNumber,1,i.lineNumber,1);const r=h=>h.start+1<=i.column&&i.column<=h.end+1,a=(h,u)=>(h=Math.min(h,i.column),u=Math.max(u,i.column),new x(i.lineNumber,h,i.lineNumber,u)),l=h=>{let u=h.start+1,g=h.end+1,f=!1;for(;g-1<s&&this._charAtIsWhitespace(n,g-1);)f=!0,g++;if(!f)for(;u>1&&this._charAtIsWhitespace(n,u-2);)u--;return a(u,g)},c=wt._findPreviousWordOnLine(e,t,i);if(c&&r(c))return l(c);const d=wt._findNextWordOnLine(e,t,i);return d&&r(d)?l(d):c&&d?a(c.end+1,d.start+1):c?a(c.start+1,c.end+1):d?a(d.start+1,d.end+1):a(1,s+1)}static _deleteWordPartLeft(e,t){if(!t.isEmpty())return t;const i=t.getPosition(),n=wt._moveWordPartLeft(e,i);return new x(i.lineNumber,i.column,n.lineNumber,n.column)}static _findFirstNonWhitespaceChar(e,t){const i=e.length;for(let n=t;n<i;n++){const s=e.charAt(n);if(s!==" "&&s!=="	")return n}return i}static _deleteWordRightWhitespace(e,t){const i=e.getLineContent(t.lineNumber),n=t.column-1,s=this._findFirstNonWhitespaceChar(i,n);return n+1<s?new x(t.lineNumber,t.column,t.lineNumber,s+1):null}static deleteWordRight(e,t){const i=e.wordSeparators,n=e.model,s=e.selection,r=e.whitespaceHeuristics;if(!s.isEmpty())return s;const a=new A(s.positionLineNumber,s.positionColumn);let l=a.lineNumber,c=a.column;const d=n.getLineCount(),h=n.getLineMaxColumn(l);if(l===d&&c===h)return null;if(r){const g=this._deleteWordRightWhitespace(n,a);if(g)return g}let u=wt._findNextWordOnLine(i,n,a);return t===2?u?c=u.end+1:c<h||l===d?c=h:(l++,u=wt._findNextWordOnLine(i,n,new A(l,1)),u?c=u.start+1:c=n.getLineMaxColumn(l)):(u&&c>=u.start+1&&(u=wt._findNextWordOnLine(i,n,new A(l,u.end+1))),u?c=u.start+1:c<h||l===d?c=h:(l++,u=wt._findNextWordOnLine(i,n,new A(l,1)),u?c=u.start+1:c=n.getLineMaxColumn(l))),new x(l,c,a.lineNumber,a.column)}static _deleteWordPartRight(e,t){if(!t.isEmpty())return t;const i=t.getPosition(),n=wt._moveWordPartRight(e,i);return new x(i.lineNumber,i.column,n.lineNumber,n.column)}static _createWordAtPosition(e,t,i){const n=new x(t,i.start+1,t,i.end+1);return{word:e.getValueInRange(n),startColumn:n.startColumn,endColumn:n.endColumn}}static getWordAtPosition(e,t,i,n){const s=Cr(t,i),r=wt._findPreviousWordOnLine(s,e,n);if(r&&r.wordType===1&&r.start<=n.column-1&&n.column-1<=r.end)return wt._createWordAtPosition(e,n.lineNumber,r);const a=wt._findNextWordOnLine(s,e,n);return a&&a.wordType===1&&a.start<=n.column-1&&n.column-1<=a.end?wt._createWordAtPosition(e,n.lineNumber,a):null}static word(e,t,i,n,s){const r=Cr(e.wordSeparators,e.wordSegmenterLocales),a=wt._findPreviousWordOnLine(r,t,s),l=wt._findNextWordOnLine(r,t,s);if(!n){let g,f;return a&&a.wordType===1&&a.start<=s.column-1&&s.column-1<=a.end?(g=a.start+1,f=a.end+1):l&&l.wordType===1&&l.start<=s.column-1&&s.column-1<=l.end?(g=l.start+1,f=l.end+1):(a?g=a.end+1:g=1,l?f=l.start+1:f=t.getLineMaxColumn(s.lineNumber)),new gn(new x(s.lineNumber,g,s.lineNumber,f),1,0,new A(s.lineNumber,f),0)}let c,d;a&&a.wordType===1&&a.start<s.column-1&&s.column-1<a.end?(c=a.start+1,d=a.end+1):l&&l.wordType===1&&l.start<s.column-1&&s.column-1<l.end?(c=l.start+1,d=l.end+1):(c=s.column,d=s.column);const h=s.lineNumber;let u;if(i.selectionStart.containsPosition(s))u=i.selectionStart.endColumn;else if(s.isBeforeOrEqual(i.selectionStart.getStartPosition())){u=c;const g=new A(h,u);i.selectionStart.containsPosition(g)&&(u=i.selectionStart.endColumn)}else{u=d;const g=new A(h,u);i.selectionStart.containsPosition(g)&&(u=i.selectionStart.startColumn)}return i.move(!0,h,u,0)}}class bE extends wt{static deleteWordPartLeft(e){const t=a0([wt.deleteWordLeft(e,0),wt.deleteWordLeft(e,2),wt._deleteWordPartLeft(e.model,e.selection)]);return t.sort(x.compareRangesUsingEnds),t[2]}static deleteWordPartRight(e){const t=a0([wt.deleteWordRight(e,0),wt.deleteWordRight(e,2),wt._deleteWordPartRight(e.model,e.selection)]);return t.sort(x.compareRangesUsingStarts),t[0]}static moveWordPartLeft(e,t,i,n){const s=a0([wt.moveWordLeft(e,t,i,0,n),wt.moveWordLeft(e,t,i,2,n),wt._moveWordPartLeft(t,i)]);return s.sort(A.compare),s[2]}static moveWordPartRight(e,t,i){const n=a0([wt.moveWordRight(e,t,i,0),wt.moveWordRight(e,t,i,2),wt._moveWordPartRight(t,i)]);return n.sort(A.compare),n[0]}}function a0(o){return o.filter(e=>!!e)}class un{static addCursorDown(e,t,i){const n=[];let s=0;for(let r=0,a=t.length;r<a;r++){const l=t[r];n[s++]=new ft(l.modelState,l.viewState),i?n[s++]=ft.fromModelState(Lt.translateDown(e.cursorConfig,e.model,l.modelState)):n[s++]=ft.fromViewState(Lt.translateDown(e.cursorConfig,e,l.viewState))}return n}static addCursorUp(e,t,i){const n=[];let s=0;for(let r=0,a=t.length;r<a;r++){const l=t[r];n[s++]=new ft(l.modelState,l.viewState),i?n[s++]=ft.fromModelState(Lt.translateUp(e.cursorConfig,e.model,l.modelState)):n[s++]=ft.fromViewState(Lt.translateUp(e.cursorConfig,e,l.viewState))}return n}static moveToBeginningOfLine(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s];n[s]=this._moveToLineStart(e,a,i)}return n}static _moveToLineStart(e,t,i){const n=t.viewState.position.column,s=t.modelState.position.column,r=n===s,a=t.viewState.position.lineNumber,l=e.getLineFirstNonWhitespaceColumn(a);return!r&&!(n===l)?this._moveToLineStartByView(e,t,i):this._moveToLineStartByModel(e,t,i)}static _moveToLineStartByView(e,t,i){return ft.fromViewState(Lt.moveToBeginningOfLine(e.cursorConfig,e,t.viewState,i))}static _moveToLineStartByModel(e,t,i){return ft.fromModelState(Lt.moveToBeginningOfLine(e.cursorConfig,e.model,t.modelState,i))}static moveToEndOfLine(e,t,i,n){const s=[];for(let r=0,a=t.length;r<a;r++){const l=t[r];s[r]=this._moveToLineEnd(e,l,i,n)}return s}static _moveToLineEnd(e,t,i,n){const s=t.viewState.position,r=e.getLineMaxColumn(s.lineNumber),a=s.column===r,l=t.modelState.position,c=e.model.getLineMaxColumn(l.lineNumber),d=r-s.column===c-l.column;return a||d?this._moveToLineEndByModel(e,t,i,n):this._moveToLineEndByView(e,t,i,n)}static _moveToLineEndByView(e,t,i,n){return ft.fromViewState(Lt.moveToEndOfLine(e.cursorConfig,e,t.viewState,i,n))}static _moveToLineEndByModel(e,t,i,n){return ft.fromModelState(Lt.moveToEndOfLine(e.cursorConfig,e.model,t.modelState,i,n))}static expandLineSelection(e,t){const i=[];for(let n=0,s=t.length;n<s;n++){const r=t[n],a=r.modelState.selection.startLineNumber,l=e.model.getLineCount();let c=r.modelState.selection.endLineNumber,d;c===l?d=e.model.getLineMaxColumn(l):(c++,d=1),i[n]=ft.fromModelState(new gn(new x(a,1,a,1),0,0,new A(c,d),0))}return i}static moveToBeginningOfBuffer(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s];n[s]=ft.fromModelState(Lt.moveToBeginningOfBuffer(e.cursorConfig,e.model,a.modelState,i))}return n}static moveToEndOfBuffer(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s];n[s]=ft.fromModelState(Lt.moveToEndOfBuffer(e.cursorConfig,e.model,a.modelState,i))}return n}static selectAll(e,t){const i=e.model.getLineCount(),n=e.model.getLineMaxColumn(i);return ft.fromModelState(new gn(new x(1,1,1,1),0,0,new A(i,n),0))}static line(e,t,i,n,s){const r=e.model.validatePosition(n),a=s?e.coordinatesConverter.validateViewPosition(new A(s.lineNumber,s.column),r):e.coordinatesConverter.convertModelPositionToViewPosition(r);if(!i){const c=e.model.getLineCount();let d=r.lineNumber+1,h=1;return d>c&&(d=c,h=e.model.getLineMaxColumn(d)),ft.fromModelState(new gn(new x(r.lineNumber,1,d,h),2,0,new A(d,h),0))}const l=t.modelState.selectionStart.getStartPosition().lineNumber;if(r.lineNumber<l)return ft.fromViewState(t.viewState.move(!0,a.lineNumber,1,0));if(r.lineNumber>l){const c=e.getLineCount();let d=a.lineNumber+1,h=1;return d>c&&(d=c,h=e.getLineMaxColumn(d)),ft.fromViewState(t.viewState.move(!0,d,h,0))}else{const c=t.modelState.selectionStart.getEndPosition();return ft.fromModelState(t.modelState.move(!0,c.lineNumber,c.column,0))}}static word(e,t,i,n){const s=e.model.validatePosition(n);return ft.fromModelState(wt.word(e.cursorConfig,e.model,t.modelState,i,s))}static cancelSelection(e,t){if(!t.modelState.hasSelection())return new ft(t.modelState,t.viewState);const i=t.viewState.position.lineNumber,n=t.viewState.position.column;return ft.fromViewState(new gn(new x(i,n,i,n),0,0,new A(i,n),0))}static moveTo(e,t,i,n,s){if(i){if(t.modelState.selectionStartKind===1)return this.word(e,t,i,n);if(t.modelState.selectionStartKind===2)return this.line(e,t,i,n,s)}const r=e.model.validatePosition(n),a=s?e.coordinatesConverter.validateViewPosition(new A(s.lineNumber,s.column),r):e.coordinatesConverter.convertModelPositionToViewPosition(r);return ft.fromViewState(t.viewState.move(i,a.lineNumber,a.column,0))}static simpleMove(e,t,i,n,s,r){switch(i){case 0:return r===4?this._moveHalfLineLeft(e,t,n):this._moveLeft(e,t,n,s);case 1:return r===4?this._moveHalfLineRight(e,t,n):this._moveRight(e,t,n,s);case 2:return r===2?this._moveUpByViewLines(e,t,n,s):this._moveUpByModelLines(e,t,n,s);case 3:return r===2?this._moveDownByViewLines(e,t,n,s):this._moveDownByModelLines(e,t,n,s);case 4:return r===2?t.map(a=>ft.fromViewState(Lt.moveToPrevBlankLine(e.cursorConfig,e,a.viewState,n))):t.map(a=>ft.fromModelState(Lt.moveToPrevBlankLine(e.cursorConfig,e.model,a.modelState,n)));case 5:return r===2?t.map(a=>ft.fromViewState(Lt.moveToNextBlankLine(e.cursorConfig,e,a.viewState,n))):t.map(a=>ft.fromModelState(Lt.moveToNextBlankLine(e.cursorConfig,e.model,a.modelState,n)));case 6:return this._moveToViewMinColumn(e,t,n);case 7:return this._moveToViewFirstNonWhitespaceColumn(e,t,n);case 8:return this._moveToViewCenterColumn(e,t,n);case 9:return this._moveToViewMaxColumn(e,t,n);case 10:return this._moveToViewLastNonWhitespaceColumn(e,t,n);default:return null}}static viewportMove(e,t,i,n,s){const r=e.getCompletelyVisibleViewRange(),a=e.coordinatesConverter.convertViewRangeToModelRange(r);switch(i){case 11:{const l=this._firstLineNumberInRange(e.model,a,s),c=e.model.getLineFirstNonWhitespaceColumn(l);return[this._moveToModelPosition(e,t[0],n,l,c)]}case 13:{const l=this._lastLineNumberInRange(e.model,a,s),c=e.model.getLineFirstNonWhitespaceColumn(l);return[this._moveToModelPosition(e,t[0],n,l,c)]}case 12:{const l=Math.round((a.startLineNumber+a.endLineNumber)/2),c=e.model.getLineFirstNonWhitespaceColumn(l);return[this._moveToModelPosition(e,t[0],n,l,c)]}case 14:{const l=[];for(let c=0,d=t.length;c<d;c++){const h=t[c];l[c]=this.findPositionInViewportIfOutside(e,h,r,n)}return l}default:return null}}static findPositionInViewportIfOutside(e,t,i,n){const s=t.viewState.position.lineNumber;if(i.startLineNumber<=s&&s<=i.endLineNumber-1)return new ft(t.modelState,t.viewState);{let r;s>i.endLineNumber-1?r=i.endLineNumber-1:s<i.startLineNumber?r=i.startLineNumber:r=s;const a=Lt.vertical(e.cursorConfig,e,s,t.viewState.position.column,t.viewState.leftoverVisibleColumns,r,!1);return ft.fromViewState(t.viewState.move(n,a.lineNumber,a.column,a.leftoverVisibleColumns))}}static _firstLineNumberInRange(e,t,i){let n=t.startLineNumber;return t.startColumn!==e.getLineMinColumn(n)&&n++,Math.min(t.endLineNumber,n+i-1)}static _lastLineNumberInRange(e,t,i){let n=t.startLineNumber;return t.startColumn!==e.getLineMinColumn(n)&&n++,Math.max(n,t.endLineNumber-i+1)}static _moveLeft(e,t,i,n){return t.map(s=>ft.fromViewState(Lt.moveLeft(e.cursorConfig,e,s.viewState,i,n)))}static _moveHalfLineLeft(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s],l=a.viewState.position.lineNumber,c=Math.round(e.getLineLength(l)/2);n[s]=ft.fromViewState(Lt.moveLeft(e.cursorConfig,e,a.viewState,i,c))}return n}static _moveRight(e,t,i,n){return t.map(s=>ft.fromViewState(Lt.moveRight(e.cursorConfig,e,s.viewState,i,n)))}static _moveHalfLineRight(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s],l=a.viewState.position.lineNumber,c=Math.round(e.getLineLength(l)/2);n[s]=ft.fromViewState(Lt.moveRight(e.cursorConfig,e,a.viewState,i,c))}return n}static _moveDownByViewLines(e,t,i,n){const s=[];for(let r=0,a=t.length;r<a;r++){const l=t[r];s[r]=ft.fromViewState(Lt.moveDown(e.cursorConfig,e,l.viewState,i,n))}return s}static _moveDownByModelLines(e,t,i,n){const s=[];for(let r=0,a=t.length;r<a;r++){const l=t[r];s[r]=ft.fromModelState(Lt.moveDown(e.cursorConfig,e.model,l.modelState,i,n))}return s}static _moveUpByViewLines(e,t,i,n){const s=[];for(let r=0,a=t.length;r<a;r++){const l=t[r];s[r]=ft.fromViewState(Lt.moveUp(e.cursorConfig,e,l.viewState,i,n))}return s}static _moveUpByModelLines(e,t,i,n){const s=[];for(let r=0,a=t.length;r<a;r++){const l=t[r];s[r]=ft.fromModelState(Lt.moveUp(e.cursorConfig,e.model,l.modelState,i,n))}return s}static _moveToViewPosition(e,t,i,n,s){return ft.fromViewState(t.viewState.move(i,n,s,0))}static _moveToModelPosition(e,t,i,n,s){return ft.fromModelState(t.modelState.move(i,n,s,0))}static _moveToViewMinColumn(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s],l=a.viewState.position.lineNumber,c=e.getLineMinColumn(l);n[s]=this._moveToViewPosition(e,a,i,l,c)}return n}static _moveToViewFirstNonWhitespaceColumn(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s],l=a.viewState.position.lineNumber,c=e.getLineFirstNonWhitespaceColumn(l);n[s]=this._moveToViewPosition(e,a,i,l,c)}return n}static _moveToViewCenterColumn(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s],l=a.viewState.position.lineNumber,c=Math.round((e.getLineMaxColumn(l)+e.getLineMinColumn(l))/2);n[s]=this._moveToViewPosition(e,a,i,l,c)}return n}static _moveToViewMaxColumn(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s],l=a.viewState.position.lineNumber,c=e.getLineMaxColumn(l);n[s]=this._moveToViewPosition(e,a,i,l,c)}return n}static _moveToViewLastNonWhitespaceColumn(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s],l=a.viewState.position.lineNumber,c=e.getLineLastNonWhitespaceColumn(l);n[s]=this._moveToViewPosition(e,a,i,l,c)}return n}}var cx;(function(o){const e=function(i){if(!on(i))return!1;const n=i;return!(!As(n.to)||!Xn(n.select)&&!Sz(n.select)||!Xn(n.by)&&!As(n.by)||!Xn(n.value)&&!Rh(n.value))};o.metadata={description:"Move cursor to a logical position in the view",args:[{name:"Cursor move argument object",description:`Property-value pairs that can be passed through this argument:
					* 'to': A mandatory logical position value providing where to move the cursor.
						\`\`\`
						'left', 'right', 'up', 'down', 'prevBlankLine', 'nextBlankLine',
						'wrappedLineStart', 'wrappedLineEnd', 'wrappedLineColumnCenter'
						'wrappedLineFirstNonWhitespaceCharacter', 'wrappedLineLastNonWhitespaceCharacter'
						'viewPortTop', 'viewPortCenter', 'viewPortBottom', 'viewPortIfOutside'
						\`\`\`
					* 'by': Unit to move. Default is computed based on 'to' value.
						\`\`\`
						'line', 'wrappedLine', 'character', 'halfLine'
						\`\`\`
					* 'value': Number of units to move. Default is '1'.
					* 'select': If 'true' makes the selection. Default is 'false'.
				`,constraint:e,schema:{type:"object",required:["to"],properties:{to:{type:"string",enum:["left","right","up","down","prevBlankLine","nextBlankLine","wrappedLineStart","wrappedLineEnd","wrappedLineColumnCenter","wrappedLineFirstNonWhitespaceCharacter","wrappedLineLastNonWhitespaceCharacter","viewPortTop","viewPortCenter","viewPortBottom","viewPortIfOutside"]},by:{type:"string",enum:["line","wrappedLine","character","halfLine"]},value:{type:"number",default:1},select:{type:"boolean",default:!1}}}}]},o.RawDirection={Left:"left",Right:"right",Up:"up",Down:"down",PrevBlankLine:"prevBlankLine",NextBlankLine:"nextBlankLine",WrappedLineStart:"wrappedLineStart",WrappedLineFirstNonWhitespaceCharacter:"wrappedLineFirstNonWhitespaceCharacter",WrappedLineColumnCenter:"wrappedLineColumnCenter",WrappedLineEnd:"wrappedLineEnd",WrappedLineLastNonWhitespaceCharacter:"wrappedLineLastNonWhitespaceCharacter",ViewPortTop:"viewPortTop",ViewPortCenter:"viewPortCenter",ViewPortBottom:"viewPortBottom",ViewPortIfOutside:"viewPortIfOutside"},o.RawUnit={Line:"line",WrappedLine:"wrappedLine",Character:"character",HalfLine:"halfLine"};function t(i){if(!i.to)return null;let n;switch(i.to){case o.RawDirection.Left:n=0;break;case o.RawDirection.Right:n=1;break;case o.RawDirection.Up:n=2;break;case o.RawDirection.Down:n=3;break;case o.RawDirection.PrevBlankLine:n=4;break;case o.RawDirection.NextBlankLine:n=5;break;case o.RawDirection.WrappedLineStart:n=6;break;case o.RawDirection.WrappedLineFirstNonWhitespaceCharacter:n=7;break;case o.RawDirection.WrappedLineColumnCenter:n=8;break;case o.RawDirection.WrappedLineEnd:n=9;break;case o.RawDirection.WrappedLineLastNonWhitespaceCharacter:n=10;break;case o.RawDirection.ViewPortTop:n=11;break;case o.RawDirection.ViewPortBottom:n=13;break;case o.RawDirection.ViewPortCenter:n=12;break;case o.RawDirection.ViewPortIfOutside:n=14;break;default:return null}let s=0;switch(i.by){case o.RawUnit.Line:s=1;break;case o.RawUnit.WrappedLine:s=2;break;case o.RawUnit.Character:s=3;break;case o.RawUnit.HalfLine:s=4;break}return{direction:n,unit:s,select:!!i.select,value:i.value||1}}o.parse=t})(cx||(cx={}));class K3{constructor(e,t,i){this._indentRulesSupport=t,this._indentationLineProcessor=new vq(e,i)}shouldIncrease(e,t){const i=this._indentationLineProcessor.getProcessedLine(e,t);return this._indentRulesSupport.shouldIncrease(i)}shouldDecrease(e,t){const i=this._indentationLineProcessor.getProcessedLine(e,t);return this._indentRulesSupport.shouldDecrease(i)}shouldIgnore(e,t){const i=this._indentationLineProcessor.getProcessedLine(e,t);return this._indentRulesSupport.shouldIgnore(i)}shouldIndentNextLine(e,t){const i=this._indentationLineProcessor.getProcessedLine(e,t);return this._indentRulesSupport.shouldIndentNextLine(i)}}class q3{constructor(e,t){this.model=e,this.indentationLineProcessor=new vq(e,t)}getProcessedTokenContextAroundRange(e){const t=this._getProcessedTokensBeforeRange(e),i=this._getProcessedTokensAfterRange(e),n=this._getProcessedPreviousLineTokens(e);return{beforeRangeProcessedTokens:t,afterRangeProcessedTokens:i,previousLineProcessedTokens:n}}_getProcessedTokensBeforeRange(e){this.model.tokenization.forceTokenization(e.startLineNumber);const t=this.model.tokenization.getLineTokens(e.startLineNumber),i=ag(t,e.startColumn-1);let n;if(G3(this.model,e.getStartPosition())){const r=e.startColumn-1-i.firstCharOffset,a=i.firstCharOffset,l=a+r;n=t.sliceAndInflate(a,l,0)}else{const r=e.startColumn-1;n=t.sliceAndInflate(0,r,0)}return this.indentationLineProcessor.getProcessedTokens(n)}_getProcessedTokensAfterRange(e){const t=e.isEmpty()?e.getStartPosition():e.getEndPosition();this.model.tokenization.forceTokenization(t.lineNumber);const i=this.model.tokenization.getLineTokens(t.lineNumber),n=ag(i,t.column-1),s=t.column-1-n.firstCharOffset,r=n.firstCharOffset+s,a=n.firstCharOffset+n.getLineLength(),l=i.sliceAndInflate(r,a,0);return this.indentationLineProcessor.getProcessedTokens(l)}_getProcessedPreviousLineTokens(e){const t=g=>{this.model.tokenization.forceTokenization(g);const f=this.model.tokenization.getLineTokens(g),p=this.model.getLineMaxColumn(g)-1;return ag(f,p)};this.model.tokenization.forceTokenization(e.startLineNumber);const i=this.model.tokenization.getLineTokens(e.startLineNumber),n=ag(i,e.startColumn-1),s=cn.createEmpty("",n.languageIdCodec),r=e.startLineNumber-1;if(r===0||!(n.firstCharOffset===0))return s;const c=t(r);if(!(n.languageId===c.languageId))return s;const h=c.toIViewLineTokens();return this.indentationLineProcessor.getProcessedTokens(h)}}class vq{constructor(e,t){this.model=e,this.languageConfigurationService=t}getProcessedLine(e,t){var r,a;const i=(l,c)=>{const d=Nt(l);return c+l.substring(d.length)};(a=(r=this.model.tokenization).forceTokenization)==null||a.call(r,e);const n=this.model.tokenization.getLineTokens(e);let s=this.getProcessedTokens(n).getLineContent();return t!==void 0&&(s=i(s,t)),s}getProcessedTokens(e){const t=l=>l===2||l===3||l===1,i=e.getLanguageId(0),s=this.languageConfigurationService.getLanguageConfiguration(i).bracketsNew.getBracketRegExp({global:!0}),r=[];return e.forEach(l=>{const c=e.getStandardTokenType(l);let d=e.getTokenText(l);t(c)&&(d=d.replace(s,""));const h=e.getMetadata(l);r.push({text:d,metadata:h})}),cn.createFromTextAndMetadata(r,e.languageIdCodec)}}function G3(o,e){o.tokenization.forceTokenization(e.lineNumber);const t=o.tokenization.getLineTokens(e.lineNumber),i=ag(t,e.column-1),n=i.firstCharOffset===0,s=t.getLanguageId(0)===i.languageId;return!n&&!s}function o_(o,e,t,i){e.tokenization.forceTokenization(t.startLineNumber);const n=e.getLanguageIdAtPosition(t.startLineNumber,t.startColumn),s=i.getLanguageConfiguration(n);if(!s)return null;const a=new q3(e,i).getProcessedTokenContextAroundRange(t),l=a.previousLineProcessedTokens.getLineContent(),c=a.beforeRangeProcessedTokens.getLineContent(),d=a.afterRangeProcessedTokens.getLineContent(),h=s.onEnter(o,l,c,d);if(!h)return null;const u=h.indentAction;let g=h.appendText;const f=h.removeText||0;g?u===Bi.Indent&&(g="	"+g):u===Bi.Indent||u===Bi.IndentOutdent?g="	":g="";let p=t$(e,t.startLineNumber,t.startColumn);return f&&(p=p.substring(0,p.length-f)),{indentAction:u,appendText:g,removeText:f,indentation:p}}var y_e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},L_e=function(o,e){return function(t,i){e(t,i,o)}},cy;const CT=Object.create(null);function wu(o,e){if(e<=0)return"";CT[o]||(CT[o]=["",o]);const t=CT[o];for(let i=t.length;i<=e;i++)t[i]=t[i-1]+o;return t[e]}let wr=cy=class{static unshiftIndent(e,t,i,n,s){const r=Ui.visibleColumnFromColumn(e,t,i);if(s){const a=wu(" ",n),c=Ui.prevIndentTabStop(r,n)/n;return wu(a,c)}else{const a="	",c=Ui.prevRenderTabStop(r,i)/i;return wu(a,c)}}static shiftIndent(e,t,i,n,s){const r=Ui.visibleColumnFromColumn(e,t,i);if(s){const a=wu(" ",n),c=Ui.nextIndentTabStop(r,n)/n;return wu(a,c)}else{const a="	",c=Ui.nextRenderTabStop(r,i)/i;return wu(a,c)}}constructor(e,t,i){this._languageConfigurationService=i,this._opts=t,this._selection=e,this._selectionId=null,this._useLastEditRangeForCursorEndPosition=!1,this._selectionStartColumnStaysPut=!1}_addEditOperation(e,t,i){this._useLastEditRangeForCursorEndPosition?e.addTrackedEditOperation(t,i):e.addEditOperation(t,i)}getEditOperations(e,t){const i=this._selection.startLineNumber;let n=this._selection.endLineNumber;this._selection.endColumn===1&&i!==n&&(n=n-1);const{tabSize:s,indentSize:r,insertSpaces:a}=this._opts,l=i===n;if(this._opts.useTabStops){this._selection.isEmpty()&&/^\s*$/.test(e.getLineContent(i))&&(this._useLastEditRangeForCursorEndPosition=!0);let c=0,d=0;for(let h=i;h<=n;h++,c=d){d=0;const u=e.getLineContent(h);let g=Jn(u);if(this._opts.isUnshift&&(u.length===0||g===0)||!l&&!this._opts.isUnshift&&u.length===0)continue;if(g===-1&&(g=u.length),h>1&&Ui.visibleColumnFromColumn(u,g+1,s)%r!==0&&e.tokenization.isCheapToTokenize(h-1)){const _=o_(this._opts.autoIndent,e,new x(h-1,e.getLineMaxColumn(h-1),h-1,e.getLineMaxColumn(h-1)),this._languageConfigurationService);if(_){if(d=c,_.appendText)for(let b=0,C=_.appendText.length;b<C&&d<r&&_.appendText.charCodeAt(b)===32;b++)d++;_.removeText&&(d=Math.max(0,d-_.removeText));for(let b=0;b<d&&!(g===0||u.charCodeAt(g-1)!==32);b++)g--}}if(this._opts.isUnshift&&g===0)continue;let f;this._opts.isUnshift?f=cy.unshiftIndent(u,g+1,s,r,a):f=cy.shiftIndent(u,g+1,s,r,a),this._addEditOperation(t,new x(h,1,h,g+1),f),h===i&&!this._selection.isEmpty()&&(this._selectionStartColumnStaysPut=this._selection.startColumn<=g+1)}}else{!this._opts.isUnshift&&this._selection.isEmpty()&&e.getLineLength(i)===0&&(this._useLastEditRangeForCursorEndPosition=!0);const c=a?wu(" ",r):"	";for(let d=i;d<=n;d++){const h=e.getLineContent(d);let u=Jn(h);if(!(this._opts.isUnshift&&(h.length===0||u===0))&&!(!l&&!this._opts.isUnshift&&h.length===0)&&(u===-1&&(u=h.length),!(this._opts.isUnshift&&u===0)))if(this._opts.isUnshift){u=Math.min(u,r);for(let g=0;g<u;g++)if(h.charCodeAt(g)===9){u=g+1;break}this._addEditOperation(t,new x(d,1,d,u+1),"")}else this._addEditOperation(t,new x(d,1,d,1),c),d===i&&!this._selection.isEmpty()&&(this._selectionStartColumnStaysPut=this._selection.startColumn===1)}}this._selectionId=t.trackSelection(this._selection)}computeCursorState(e,t){if(this._useLastEditRangeForCursorEndPosition){const n=t.getInverseEditOperations()[0];return new pe(n.range.endLineNumber,n.range.endColumn,n.range.endLineNumber,n.range.endColumn)}const i=t.getTrackedSelection(this._selectionId);if(this._selectionStartColumnStaysPut){const n=this._selection.startColumn;return i.startColumn<=n?i:i.getDirection()===0?new pe(i.startLineNumber,n,i.endLineNumber,i.endColumn):new pe(i.endLineNumber,i.endColumn,i.startLineNumber,n)}return i}};wr=cy=y_e([L_e(2,di)],wr);class x_e{constructor(e,t,i){this._range=e,this._charBeforeSelection=t,this._charAfterSelection=i}getEditOperations(e,t){t.addTrackedEditOperation(new x(this._range.startLineNumber,this._range.startColumn,this._range.startLineNumber,this._range.startColumn),this._charBeforeSelection),t.addTrackedEditOperation(new x(this._range.endLineNumber,this._range.endColumn,this._range.endLineNumber,this._range.endColumn),this._charAfterSelection)}computeCursorState(e,t){const i=t.getInverseEditOperations(),n=i[0].range,s=i[1].range;return new pe(n.endLineNumber,n.endColumn,s.endLineNumber,s.endColumn-this._charAfterSelection.length)}}class k_e{constructor(e,t,i){this._position=e,this._text=t,this._charAfter=i}getEditOperations(e,t){t.addTrackedEditOperation(new x(this._position.lineNumber,this._position.column,this._position.lineNumber,this._position.column),this._text+this._charAfter)}computeCursorState(e,t){const n=t.getInverseEditOperations()[0].range;return new pe(n.endLineNumber,n.startColumn,n.endLineNumber,n.endColumn-this._charAfter.length)}}function D_e(o,e,t){const i=o.tokenization.getLanguageIdAtPosition(e,0);if(e>1){let n,s=-1;for(n=e-1;n>=1;n--){if(o.tokenization.getLanguageIdAtPosition(n,0)!==i)return s;const r=o.getLineContent(n);if(t.shouldIgnore(n)||/^\s+$/.test(r)||r===""){s=n;continue}return n}}return-1}function Kw(o,e,t,i=!0,n){if(o<4)return null;const s=n.getLanguageConfiguration(e.tokenization.getLanguageId()).indentRulesSupport;if(!s)return null;const r=new K3(e,s,n);if(t<=1)return{indentation:"",action:null};for(let l=t-1;l>0&&e.getLineContent(l)==="";l--)if(l===1)return{indentation:"",action:null};const a=D_e(e,t,r);if(a<0)return null;if(a<1)return{indentation:"",action:null};if(r.shouldIncrease(a)||r.shouldIndentNextLine(a)){const l=e.getLineContent(a);return{indentation:Nt(l),action:Bi.Indent,line:a}}else if(r.shouldDecrease(a)){const l=e.getLineContent(a);return{indentation:Nt(l),action:null,line:a}}else{if(a===1)return{indentation:Nt(e.getLineContent(a)),action:null,line:a};const l=a-1,c=s.getIndentMetadata(e.getLineContent(l));if(!(c&3)&&c&4){let d=0;for(let h=l-1;h>0;h--)if(!r.shouldIndentNextLine(h)){d=h;break}return{indentation:Nt(e.getLineContent(d+1)),action:null,line:d+1}}if(i)return{indentation:Nt(e.getLineContent(a)),action:null,line:a};for(let d=a;d>0;d--){if(r.shouldIncrease(d))return{indentation:Nt(e.getLineContent(d)),action:Bi.Indent,line:d};if(r.shouldIndentNextLine(d)){let h=0;for(let u=d-1;u>0;u--)if(!r.shouldIndentNextLine(d)){h=u;break}return{indentation:Nt(e.getLineContent(h+1)),action:null,line:h+1}}else if(r.shouldDecrease(d))return{indentation:Nt(e.getLineContent(d)),action:null,line:d}}return{indentation:Nt(e.getLineContent(1)),action:null,line:1}}}function hv(o,e,t,i,n,s){if(o<4)return null;const r=s.getLanguageConfiguration(t);if(!r)return null;const a=s.getLanguageConfiguration(t).indentRulesSupport;if(!a)return null;const l=new K3(e,a,s),c=Kw(o,e,i,void 0,s);if(c){const d=c.line;if(d!==void 0){let h=!0;for(let u=d;u<i-1;u++)if(!/^\s*$/.test(e.getLineContent(u))){h=!1;break}if(h){const u=r.onEnter(o,"",e.getLineContent(d),"");if(u){let g=Nt(e.getLineContent(d));return u.removeText&&(g=g.substring(0,g.length-u.removeText)),u.indentAction===Bi.Indent||u.indentAction===Bi.IndentOutdent?g=n.shiftIndent(g):u.indentAction===Bi.Outdent&&(g=n.unshiftIndent(g)),l.shouldDecrease(i)&&(g=n.unshiftIndent(g)),u.appendText&&(g+=u.appendText),Nt(g)}}}return l.shouldDecrease(i)?c.action===Bi.Indent?c.indentation:n.unshiftIndent(c.indentation):c.action===Bi.Indent?n.shiftIndent(c.indentation):c.indentation}return null}function I_e(o,e,t,i,n){if(o<4)return null;const s=e.getLanguageIdAtPosition(t.startLineNumber,t.startColumn),r=n.getLanguageConfiguration(s).indentRulesSupport;if(!r)return null;e.tokenization.forceTokenization(t.startLineNumber);const l=new q3(e,n).getProcessedTokenContextAroundRange(t),c=l.afterRangeProcessedTokens,d=l.beforeRangeProcessedTokens,h=Nt(d.getLineContent()),u=N_e(e,t.startLineNumber,d),g=G3(e,t.getStartPosition()),f=e.getLineContent(t.startLineNumber),p=Nt(f),_=Kw(o,u,t.startLineNumber+1,void 0,n);if(!_){const C=g?p:h;return{beforeEnter:C,afterEnter:C}}let b=g?p:_.indentation;return _.action===Bi.Indent&&(b=i.shiftIndent(b)),r.shouldDecrease(c.getLineContent())&&(b=i.unshiftIndent(b)),{beforeEnter:g?p:h,afterEnter:b}}function E_e(o,e,t,i,n,s){const r=o.autoIndent;if(r<4||G3(e,t.getStartPosition()))return null;const l=e.getLanguageIdAtPosition(t.startLineNumber,t.startColumn),c=s.getLanguageConfiguration(l).indentRulesSupport;if(!c)return null;const h=new q3(e,s).getProcessedTokenContextAroundRange(t),u=h.beforeRangeProcessedTokens.getLineContent(),g=h.afterRangeProcessedTokens.getLineContent(),f=u+g,p=u+i+g;if(!c.shouldDecrease(f)&&c.shouldDecrease(p)){const b=Kw(r,e,t.startLineNumber,!1,s);if(!b)return null;let C=b.indentation;return b.action!==Bi.Indent&&(C=n.unshiftIndent(C)),C}const _=t.startLineNumber-1;if(_>0){const b=e.getLineContent(_);if(c.shouldIndentNextLine(b)&&c.shouldIncrease(p)){const C=Kw(r,e,t.startLineNumber,!1,s),v=C==null?void 0:C.indentation;if(v!==void 0){const w=e.getLineContent(t.startLineNumber),S=Nt(w),k=n.shiftIndent(v)===S,D=/^\s*$/.test(f),N=o.autoClosingPairs.autoClosingPairsOpenByEnd.get(i),W=N&&N.length>0&&D;if(k&&W)return v}}}return null}function wq(o,e,t){const i=t.getLanguageConfiguration(o.getLanguageId()).indentRulesSupport;return!i||e<1||e>o.getLineCount()?null:i.getIndentMetadata(o.getLineContent(e))}function N_e(o,e,t){return{tokenization:{getLineTokens:n=>n===e?t:o.tokenization.getLineTokens(n),getLanguageId:()=>o.getLanguageId(),getLanguageIdAtPosition:(n,s)=>o.getLanguageIdAtPosition(n,s)},getLineContent:n=>n===e?t.getLineContent():o.getLineContent(n)}}class T_e{static getEdits(e,t,i,n,s){if(!s&&this._isAutoIndentType(e,t,i)){const r=[];for(const l of i){const c=this._findActualIndentationForSelection(e,t,l,n);if(c===null)return;r.push({selection:l,indentation:c})}const a=uP.getAutoClosingPairClose(e,t,i,n,!1);return this._getIndentationAndAutoClosingPairEdits(e,t,r,n,a)}}static _isAutoIndentType(e,t,i){if(e.autoIndent<4)return!1;for(let n=0,s=i.length;n<s;n++)if(!t.tokenization.isCheapToTokenize(i[n].getEndPosition().lineNumber))return!1;return!0}static _findActualIndentationForSelection(e,t,i,n){const s=E_e(e,t,i,n,{shiftIndent:a=>X3(e,a),unshiftIndent:a=>dx(e,a)},e.languageConfigurationService);if(s===null)return null;const r=t$(t,i.startLineNumber,i.startColumn);return s===e.normalizeIndentation(r)?null:s}static _getIndentationAndAutoClosingPairEdits(e,t,i,n,s){const r=i.map(({selection:l,indentation:c})=>{if(s!==null){const d=this._getEditFromIndentationAndSelection(e,t,c,l,n,!1);return new z_e(d,l,n,s)}else{const d=this._getEditFromIndentationAndSelection(e,t,c,l,n,!0);return Au(d.range,d.text,!1)}}),a={shouldPushStackElementBefore:!0,shouldPushStackElementAfter:!1};return new Fs(4,r,a)}static _getEditFromIndentationAndSelection(e,t,i,n,s,r=!0){const a=n.startLineNumber,l=t.getLineFirstNonWhitespaceColumn(a);let c=e.normalizeIndentation(i);if(l!==0){const h=t.getLineContent(a);c+=h.substring(l-1,n.startColumn-1)}return c+=r?s:"",{range:new x(a,1,n.endLineNumber,n.endColumn),text:c}}}class R_e{static getEdits(e,t,i,n,s,r){if(Sq(t,i,n,s,r))return this._runAutoClosingOvertype(e,n,r)}static _runAutoClosingOvertype(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const l=t[s].getPosition(),c=new x(l.lineNumber,l.column,l.lineNumber,l.column+1);n[s]=new kn(c,i)}return new Fs(4,n,{shouldPushStackElementBefore:vE(e,4),shouldPushStackElementAfter:!1})}}class M_e{static getEdits(e,t,i,n,s){if(Sq(e,t,i,n,s)){const r=i.map(a=>new kn(new x(a.positionLineNumber,a.positionColumn,a.positionLineNumber,a.positionColumn+1),"",!1));return new Fs(4,r,{shouldPushStackElementBefore:!0,shouldPushStackElementAfter:!1})}}}class uP{static getEdits(e,t,i,n,s,r){if(!r){const a=this.getAutoClosingPairClose(e,t,i,n,s);if(a!==null)return this._runAutoClosingOpenCharType(i,n,s,a)}}static _runAutoClosingOpenCharType(e,t,i,n){const s=[];for(let r=0,a=e.length;r<a;r++){const l=e[r];s[r]=new V_e(l,t,!i,n)}return new Fs(4,s,{shouldPushStackElementBefore:!0,shouldPushStackElementAfter:!1})}static getAutoClosingPairClose(e,t,i,n,s){for(const f of i)if(!f.isEmpty())return null;const r=i.map(f=>{const p=f.getPosition();return s?{lineNumber:p.lineNumber,beforeColumn:p.column-n.length,afterColumn:p.column}:{lineNumber:p.lineNumber,beforeColumn:p.column,afterColumn:p.column}}),a=this._findAutoClosingPairOpen(e,t,r.map(f=>new A(f.lineNumber,f.beforeColumn)),n);if(!a)return null;let l,c;if(Gh(n)?(l=e.autoClosingQuotes,c=e.shouldAutoCloseBefore.quote):(e.blockCommentStartToken?a.open.includes(e.blockCommentStartToken):!1)?(l=e.autoClosingComments,c=e.shouldAutoCloseBefore.comment):(l=e.autoClosingBrackets,c=e.shouldAutoCloseBefore.bracket),l==="never")return null;const h=this._findContainedAutoClosingPair(e,a),u=h?h.close:"";let g=!0;for(const f of r){const{lineNumber:p,beforeColumn:_,afterColumn:b}=f,C=t.getLineContent(p),v=C.substring(0,_-1),w=C.substring(b-1);if(w.startsWith(u)||(g=!1),w.length>0){const D=w.charAt(0);if(!this._isBeforeClosingBrace(e,w)&&!c(D))return null}if(a.open.length===1&&(n==="'"||n==='"')&&l!=="always"){const D=Cr(e.wordSeparators,[]);if(v.length>0){const N=v.charCodeAt(v.length-1);if(D.get(N)===0)return null}}if(!t.tokenization.isCheapToTokenize(p))return null;t.tokenization.forceTokenization(p);const S=t.tokenization.getLineTokens(p),L=ag(S,_-1);if(!a.shouldAutoClose(L,_-L.firstCharOffset))return null;const k=a.findNeutralCharacter();if(k){const D=t.tokenization.getTokenTypeIfInsertingCharacter(p,_,k);if(!a.isOK(D))return null}}return g?a.close.substring(0,a.close.length-u.length):a.close}static _findContainedAutoClosingPair(e,t){if(t.open.length<=1)return null;const i=t.close.charAt(t.close.length-1),n=e.autoClosingPairs.autoClosingPairsCloseByEnd.get(i)||[];let s=null;for(const r of n)r.open!==t.open&&t.open.includes(r.open)&&t.close.endsWith(r.close)&&(!s||r.open.length>s.open.length)&&(s=r);return s}static _findAutoClosingPairOpen(e,t,i,n){const s=e.autoClosingPairs.autoClosingPairsOpenByEnd.get(n);if(!s)return null;let r=null;for(const a of s)if(r===null||a.open.length>r.open.length){let l=!0;for(const c of i)if(t.getValueInRange(new x(c.lineNumber,c.column-a.open.length+1,c.lineNumber,c.column))+n!==a.open){l=!1;break}l&&(r=a)}return r}static _isBeforeClosingBrace(e,t){const i=t.charAt(0),n=e.autoClosingPairs.autoClosingPairsOpenByStart.get(i)||[],s=e.autoClosingPairs.autoClosingPairsCloseByStart.get(i)||[],r=n.some(l=>t.startsWith(l.open)),a=s.some(l=>t.startsWith(l.close));return!r&&a}}class A_e{static getEdits(e,t,i,n,s){if(!s&&this._isSurroundSelectionType(e,t,i,n))return this._runSurroundSelectionType(e,i,n)}static _runSurroundSelectionType(e,t,i){const n=[];for(let s=0,r=t.length;s<r;s++){const a=t[s],l=e.surroundingPairs[i];n[s]=new x_e(a,i,l)}return new Fs(0,n,{shouldPushStackElementBefore:!0,shouldPushStackElementAfter:!0})}static _isSurroundSelectionType(e,t,i,n){if(!yq(e,n)||!e.surroundingPairs.hasOwnProperty(n))return!1;const s=Gh(n);for(const r of i){if(r.isEmpty())return!1;let a=!0;for(let l=r.startLineNumber;l<=r.endLineNumber;l++){const c=t.getLineContent(l),d=l===r.startLineNumber?r.startColumn-1:0,h=l===r.endLineNumber?r.endColumn-1:c.length,u=c.substring(d,h);if(/[^ \t]/.test(u)){a=!1;break}}if(a)return!1;if(s&&r.startLineNumber===r.endLineNumber&&r.startColumn+1===r.endColumn){const l=t.getValueInRange(r);if(Gh(l))return!1}}return!0}}class P_e{static getEdits(e,t,i,n,s,r){if(!r&&this._isTypeInterceptorElectricChar(t,i,n)){const a=this._typeInterceptorElectricChar(e,t,i,n[0],s);if(a)return a}}static _isTypeInterceptorElectricChar(e,t,i){return!!(i.length===1&&t.tokenization.isCheapToTokenize(i[0].getEndPosition().lineNumber))}static _typeInterceptorElectricChar(e,t,i,n,s){if(!t.electricChars.hasOwnProperty(s)||!n.isEmpty())return null;const r=n.getPosition();i.tokenization.forceTokenization(r.lineNumber);const a=i.tokenization.getLineTokens(r.lineNumber);let l;try{l=t.onElectricCharacter(s,a,r.column)}catch(c){return Oe(c),null}if(!l)return null;if(l.matchOpenBracket){const c=(a.getLineContent()+s).lastIndexOf(l.matchOpenBracket)+1,d=i.bracketPairs.findMatchingBracketUp(l.matchOpenBracket,{lineNumber:r.lineNumber,column:c},500);if(d){if(d.startLineNumber===r.lineNumber)return null;const h=i.getLineContent(d.startLineNumber),u=Nt(h),g=t.normalizeIndentation(u),f=i.getLineContent(r.lineNumber),p=i.getLineFirstNonWhitespaceColumn(r.lineNumber)||r.column,_=f.substring(p-1,r.column-1),b=g+_+s,C=new x(r.lineNumber,1,r.lineNumber,r.column),v=new kn(C,b);return new Fs(Y3(b,e),[v],{shouldPushStackElementBefore:!1,shouldPushStackElementAfter:!0})}}return null}}class O_e{static getEdits(e,t,i){const n=[];for(let r=0,a=t.length;r<a;r++)n[r]=new kn(t[r],i);const s=Y3(i,e);return new Fs(s,n,{shouldPushStackElementBefore:vE(e,s),shouldPushStackElementAfter:!1})}}class CE{static getEdits(e,t,i,n,s){if(!s&&n===`
`){const r=[];for(let a=0,l=i.length;a<l;a++)r[a]=this._enter(e,t,!1,i[a]);return new Fs(4,r,{shouldPushStackElementBefore:!0,shouldPushStackElementAfter:!1})}}static _enter(e,t,i,n){if(e.autoIndent===0)return Au(n,`
`,i);if(!t.tokenization.isCheapToTokenize(n.getStartPosition().lineNumber)||e.autoIndent===1){const l=t.getLineContent(n.startLineNumber),c=Nt(l).substring(0,n.startColumn-1);return Au(n,`
`+e.normalizeIndentation(c),i)}const s=o_(e.autoIndent,t,n,e.languageConfigurationService);if(s){if(s.indentAction===Bi.None)return Au(n,`
`+e.normalizeIndentation(s.indentation+s.appendText),i);if(s.indentAction===Bi.Indent)return Au(n,`
`+e.normalizeIndentation(s.indentation+s.appendText),i);if(s.indentAction===Bi.IndentOutdent){const l=e.normalizeIndentation(s.indentation),c=e.normalizeIndentation(s.indentation+s.appendText),d=`
`+c+`
`+l;return i?new ly(n,d,!0):new lx(n,d,-1,c.length-l.length,!0)}else if(s.indentAction===Bi.Outdent){const l=dx(e,s.indentation);return Au(n,`
`+e.normalizeIndentation(l+s.appendText),i)}}const r=t.getLineContent(n.startLineNumber),a=Nt(r).substring(0,n.startColumn-1);if(e.autoIndent>=4){const l=I_e(e.autoIndent,t,n,{unshiftIndent:c=>dx(e,c),shiftIndent:c=>X3(e,c),normalizeIndentation:c=>e.normalizeIndentation(c)},e.languageConfigurationService);if(l){let c=e.visibleColumnFromColumn(t,n.getEndPosition());const d=n.endColumn,h=t.getLineContent(n.endLineNumber),u=Jn(h);if(u>=0?n=n.setEndPosition(n.endLineNumber,Math.max(n.endColumn,u+1)):n=n.setEndPosition(n.endLineNumber,t.getLineMaxColumn(n.endLineNumber)),i)return new ly(n,`
`+e.normalizeIndentation(l.afterEnter),!0);{let g=0;return d<=u+1&&(e.insertSpaces||(c=Math.ceil(c/e.indentSize)),g=Math.min(c+1-e.normalizeIndentation(l.afterEnter).length-1,0)),new lx(n,`
`+e.normalizeIndentation(l.afterEnter),0,g,!0)}}}return Au(n,`
`+e.normalizeIndentation(a),i)}static lineInsertBefore(e,t,i){if(t===null||i===null)return[];const n=[];for(let s=0,r=i.length;s<r;s++){let a=i[s].positionLineNumber;if(a===1)n[s]=new ly(new x(1,1,1,1),`
`);else{a--;const l=t.getLineMaxColumn(a);n[s]=this._enter(e,t,!1,new x(a,l,a,l))}}return n}static lineInsertAfter(e,t,i){if(t===null||i===null)return[];const n=[];for(let s=0,r=i.length;s<r;s++){const a=i[s].positionLineNumber,l=t.getLineMaxColumn(a);n[s]=this._enter(e,t,!1,new x(a,l,a,l))}return n}static lineBreakInsert(e,t,i){const n=[];for(let s=0,r=i.length;s<r;s++)n[s]=this._enter(e,t,!0,i[s]);return n}}class F_e{static getEdits(e,t,i,n,s,r){const a=this._distributePasteToCursors(e,i,n,s,r);return a?(i=i.sort(x.compareRangesUsingStarts),this._distributedPaste(e,t,i,a)):this._simplePaste(e,t,i,n,s)}static _distributePasteToCursors(e,t,i,n,s){if(n||t.length===1)return null;if(s&&s.length===t.length)return s;if(e.multiCursorPaste==="spread"){i.charCodeAt(i.length-1)===10&&(i=i.substring(0,i.length-1)),i.charCodeAt(i.length-1)===13&&(i=i.substring(0,i.length-1));const r=Kl(i);if(r.length===t.length)return r}return null}static _distributedPaste(e,t,i,n){const s=[];for(let r=0,a=i.length;r<a;r++)s[r]=new kn(i[r],n[r]);return new Fs(0,s,{shouldPushStackElementBefore:!0,shouldPushStackElementAfter:!0})}static _simplePaste(e,t,i,n,s){const r=[];for(let a=0,l=i.length;a<l;a++){const c=i[a],d=c.getPosition();if(s&&!c.isEmpty()&&(s=!1),s&&n.indexOf(`
`)!==n.length-1&&(s=!1),s){const h=new x(d.lineNumber,1,d.lineNumber,1);r[a]=new j3(h,n,c,!0)}else r[a]=new kn(c,n)}return new Fs(0,r,{shouldPushStackElementBefore:!0,shouldPushStackElementAfter:!0})}}class W_e{static getEdits(e,t,i,n,s,r,a,l){const c=n.map(d=>this._compositionType(i,d,s,r,a,l));return new Fs(4,c,{shouldPushStackElementBefore:vE(e,4),shouldPushStackElementAfter:!1})}static _compositionType(e,t,i,n,s,r){if(!t.isEmpty())return null;const a=t.getPosition(),l=Math.max(1,a.column-n),c=Math.min(e.getLineMaxColumn(a.lineNumber),a.column+s),d=new x(a.lineNumber,l,a.lineNumber,c);return e.getValueInRange(d)===i&&r===0?null:new lx(d,i,0,r)}}class B_e{static getEdits(e,t,i){const n=[];for(let r=0,a=t.length;r<a;r++)n[r]=new kn(t[r],i);const s=Y3(i,e);return new Fs(s,n,{shouldPushStackElementBefore:vE(e,s),shouldPushStackElementAfter:!1})}}class H_e{static getCommands(e,t,i){const n=[];for(let s=0,r=i.length;s<r;s++){const a=i[s];if(a.isEmpty()){const l=t.getLineContent(a.startLineNumber);if(/^\s*$/.test(l)&&t.tokenization.isCheapToTokenize(a.startLineNumber)){let c=this._goodIndentForLine(e,t,a.startLineNumber);c=c||"	";const d=e.normalizeIndentation(c);if(!l.startsWith(d)){n[s]=new kn(new x(a.startLineNumber,1,a.startLineNumber,l.length+1),d,!0);continue}}n[s]=this._replaceJumpToNextIndent(e,t,a,!0)}else{if(a.startLineNumber===a.endLineNumber){const l=t.getLineMaxColumn(a.startLineNumber);if(a.startColumn!==1||a.endColumn!==l){n[s]=this._replaceJumpToNextIndent(e,t,a,!1);continue}}n[s]=new wr(a,{isUnshift:!1,tabSize:e.tabSize,indentSize:e.indentSize,insertSpaces:e.insertSpaces,useTabStops:e.useTabStops,autoIndent:e.autoIndent},e.languageConfigurationService)}}return n}static _goodIndentForLine(e,t,i){let n=null,s="";const r=Kw(e.autoIndent,t,i,!1,e.languageConfigurationService);if(r)n=r.action,s=r.indentation;else if(i>1){let a;for(a=i-1;a>=1;a--){const d=t.getLineContent(a);if(Na(d)>=0)break}if(a<1)return null;const l=t.getLineMaxColumn(a),c=o_(e.autoIndent,t,new x(a,l,a,l),e.languageConfigurationService);c&&(s=c.indentation+c.appendText)}return n&&(n===Bi.Indent&&(s=X3(e,s)),n===Bi.Outdent&&(s=dx(e,s)),s=e.normalizeIndentation(s)),s||null}static _replaceJumpToNextIndent(e,t,i,n){let s="";const r=i.getStartPosition();if(e.insertSpaces){const a=e.visibleColumnFromColumn(t,r),l=e.indentSize,c=l-a%l;for(let d=0;d<c;d++)s+=" "}else s="	";return new kn(i,s,n)}}class Z3 extends lx{constructor(e,t,i,n,s,r){super(e,t,i,n),this._openCharacter=s,this._closeCharacter=r,this.closeCharacterRange=null,this.enclosingRange=null}_computeCursorStateWithRange(e,t,i){return this.closeCharacterRange=new x(t.startLineNumber,t.endColumn-this._closeCharacter.length,t.endLineNumber,t.endColumn),this.enclosingRange=new x(t.startLineNumber,t.endColumn-this._openCharacter.length-this._closeCharacter.length,t.endLineNumber,t.endColumn),super.computeCursorState(e,i)}}class V_e extends Z3{constructor(e,t,i,n){const s=(i?t:"")+n,r=0,a=-n.length;super(e,s,r,a,t,n)}computeCursorState(e,t){const n=t.getInverseEditOperations()[0].range;return this._computeCursorStateWithRange(e,n,t)}}class z_e extends Z3{constructor(e,t,i,n){const s=i+n,r=0,a=i.length;super(t,s,r,a,i,n),this._autoIndentationEdit=e,this._autoClosingEdit={range:t,text:s}}getEditOperations(e,t){t.addTrackedEditOperation(this._autoIndentationEdit.range,this._autoIndentationEdit.text),t.addTrackedEditOperation(this._autoClosingEdit.range,this._autoClosingEdit.text)}computeCursorState(e,t){const i=t.getInverseEditOperations();if(i.length!==2)throw new Error("There should be two inverse edit operations!");const n=i[0].range,s=i[1].range,r=n.plusRange(s);return this._computeCursorStateWithRange(e,r,t)}}function Y3(o,e){return o===" "?e===5||e===6?6:5:4}function vE(o,e){return LH(o)&&!LH(e)?!0:o===5?!1:yH(o)!==yH(e)}function yH(o){return o===6||o===5?"space":o}function LH(o){return o===4||o===5||o===6}function Sq(o,e,t,i,n){if(o.autoClosingOvertype==="never"||!o.autoClosingPairs.autoClosingPairsCloseSingleChar.has(n))return!1;for(let s=0,r=t.length;s<r;s++){const a=t[s];if(!a.isEmpty())return!1;const l=a.getPosition(),c=e.getLineContent(l.lineNumber);if(c.charAt(l.column-1)!==n)return!1;const h=Gh(n);if((l.column>2?c.charCodeAt(l.column-2):0)===92&&h)return!1;if(o.autoClosingOvertype==="auto"){let g=!1;for(let f=0,p=i.length;f<p;f++){const _=i[f];if(l.lineNumber===_.startLineNumber&&l.column===_.startColumn){g=!0;break}}if(!g)return!1}}return!0}function Au(o,e,t){return t?new ly(o,e,!0):new kn(o,e,!0)}function X3(o,e,t){return t=t||1,wr.shiftIndent(e,e.length+t,o.tabSize,o.indentSize,o.insertSpaces)}function dx(o,e,t){return t=t||1,wr.unshiftIndent(e,e.length+t,o.tabSize,o.indentSize,o.insertSpaces)}function yq(o,e){return Gh(e)?o.autoSurround==="quotes"||o.autoSurround==="languageDefined":o.autoSurround==="brackets"||o.autoSurround==="languageDefined"}class Ud{static indent(e,t,i){if(t===null||i===null)return[];const n=[];for(let s=0,r=i.length;s<r;s++)n[s]=new wr(i[s],{isUnshift:!1,tabSize:e.tabSize,indentSize:e.indentSize,insertSpaces:e.insertSpaces,useTabStops:e.useTabStops,autoIndent:e.autoIndent},e.languageConfigurationService);return n}static outdent(e,t,i){const n=[];for(let s=0,r=i.length;s<r;s++)n[s]=new wr(i[s],{isUnshift:!0,tabSize:e.tabSize,indentSize:e.indentSize,insertSpaces:e.insertSpaces,useTabStops:e.useTabStops,autoIndent:e.autoIndent},e.languageConfigurationService);return n}static paste(e,t,i,n,s,r){return F_e.getEdits(e,t,i,n,s,r)}static tab(e,t,i){return H_e.getCommands(e,t,i)}static compositionType(e,t,i,n,s,r,a,l){return W_e.getEdits(e,t,i,n,s,r,a,l)}static compositionEndWithInterceptors(e,t,i,n,s,r){if(!n)return null;let a=null;for(const u of n)if(a===null)a=u.insertedText;else if(a!==u.insertedText)return null;if(!a||a.length!==1)return null;const l=a;let c=!1;for(const u of n)if(u.deletedText.length!==0){c=!0;break}if(c){if(!yq(t,l)||!t.surroundingPairs.hasOwnProperty(l))return null;const u=Gh(l);for(const p of n)if(p.deletedSelectionStart!==0||p.deletedSelectionEnd!==p.deletedText.length||/^[ \t]+$/.test(p.deletedText)||u&&Gh(p.deletedText))return null;const g=[];for(const p of s){if(!p.isEmpty())return null;g.push(p.getPosition())}if(g.length!==n.length)return null;const f=[];for(let p=0,_=g.length;p<_;p++)f.push(new k_e(g[p],n[p].deletedText,t.surroundingPairs[l]));return new Fs(4,f,{shouldPushStackElementBefore:!0,shouldPushStackElementAfter:!1})}const d=M_e.getEdits(t,i,s,r,l);if(d!==void 0)return d;const h=uP.getEdits(t,i,s,l,!0,!1);return h!==void 0?h:null}static typeWithInterceptors(e,t,i,n,s,r,a){const l=CE.getEdits(i,n,s,a,e);if(l!==void 0)return l;const c=T_e.getEdits(i,n,s,a,e);if(c!==void 0)return c;const d=R_e.getEdits(t,i,n,s,r,a);if(d!==void 0)return d;const h=uP.getEdits(i,n,s,a,!1,e);if(h!==void 0)return h;const u=A_e.getEdits(i,n,s,a,e);if(u!==void 0)return u;const g=P_e.getEdits(t,i,n,s,a,e);return g!==void 0?g:O_e.getEdits(t,s,a)}static typeWithoutInterceptors(e,t,i,n,s){return B_e.getEdits(e,n,s)}}class U_e{constructor(e,t,i,n,s,r){this.deletedText=e,this.deletedSelectionStart=t,this.deletedSelectionEnd=i,this.insertedText=n,this.insertedSelectionStart=s,this.insertedSelectionEnd=r}}var I;(function(o){o.editorSimpleInput=new le("editorSimpleInput",!1,!0),o.editorTextFocus=new le("editorTextFocus",!1,m("editorTextFocus","Whether the editor text has focus (cursor is blinking)")),o.focus=new le("editorFocus",!1,m("editorFocus","Whether the editor or an editor widget has focus (e.g. focus is in the find widget)")),o.textInputFocus=new le("textInputFocus",!1,m("textInputFocus","Whether an editor or a rich text input has focus (cursor is blinking)")),o.readOnly=new le("editorReadonly",!1,m("editorReadonly","Whether the editor is read-only")),o.inDiffEditor=new le("inDiffEditor",!1,m("inDiffEditor","Whether the context is a diff editor")),o.isEmbeddedDiffEditor=new le("isEmbeddedDiffEditor",!1,m("isEmbeddedDiffEditor","Whether the context is an embedded diff editor")),o.inMultiDiffEditor=new le("inMultiDiffEditor",!1,m("inMultiDiffEditor","Whether the context is a multi diff editor")),o.multiDiffEditorAllCollapsed=new le("multiDiffEditorAllCollapsed",void 0,m("multiDiffEditorAllCollapsed","Whether all files in multi diff editor are collapsed")),o.hasChanges=new le("diffEditorHasChanges",!1,m("diffEditorHasChanges","Whether the diff editor has changes")),o.comparingMovedCode=new le("comparingMovedCode",!1,m("comparingMovedCode","Whether a moved code block is selected for comparison")),o.accessibleDiffViewerVisible=new le("accessibleDiffViewerVisible",!1,m("accessibleDiffViewerVisible","Whether the accessible diff viewer is visible")),o.diffEditorRenderSideBySideInlineBreakpointReached=new le("diffEditorRenderSideBySideInlineBreakpointReached",!1,m("diffEditorRenderSideBySideInlineBreakpointReached","Whether the diff editor render side by side inline breakpoint is reached")),o.diffEditorInlineMode=new le("diffEditorInlineMode",!1,m("diffEditorInlineMode","Whether inline mode is active")),o.diffEditorOriginalWritable=new le("diffEditorOriginalWritable",!1,m("diffEditorOriginalWritable","Whether modified is writable in the diff editor")),o.diffEditorModifiedWritable=new le("diffEditorModifiedWritable",!1,m("diffEditorModifiedWritable","Whether modified is writable in the diff editor")),o.diffEditorOriginalUri=new le("diffEditorOriginalUri","",m("diffEditorOriginalUri","The uri of the original document")),o.diffEditorModifiedUri=new le("diffEditorModifiedUri","",m("diffEditorModifiedUri","The uri of the modified document")),o.columnSelection=new le("editorColumnSelection",!1,m("editorColumnSelection","Whether `editor.columnSelection` is enabled")),o.writable=o.readOnly.toNegated(),o.hasNonEmptySelection=new le("editorHasSelection",!1,m("editorHasSelection","Whether the editor has text selected")),o.hasOnlyEmptySelection=o.hasNonEmptySelection.toNegated(),o.hasMultipleSelections=new le("editorHasMultipleSelections",!1,m("editorHasMultipleSelections","Whether the editor has multiple selections")),o.hasSingleSelection=o.hasMultipleSelections.toNegated(),o.tabMovesFocus=new le("editorTabMovesFocus",!1,m("editorTabMovesFocus","Whether `Tab` will move focus out of the editor")),o.tabDoesNotMoveFocus=o.tabMovesFocus.toNegated(),o.isInEmbeddedEditor=new le("isInEmbeddedEditor",!1,!0),o.canUndo=new le("canUndo",!1,!0),o.canRedo=new le("canRedo",!1,!0),o.hoverVisible=new le("editorHoverVisible",!1,m("editorHoverVisible","Whether the editor hover is visible")),o.hoverFocused=new le("editorHoverFocused",!1,m("editorHoverFocused","Whether the editor hover is focused")),o.stickyScrollFocused=new le("stickyScrollFocused",!1,m("stickyScrollFocused","Whether the sticky scroll is focused")),o.stickyScrollVisible=new le("stickyScrollVisible",!1,m("stickyScrollVisible","Whether the sticky scroll is visible")),o.standaloneColorPickerVisible=new le("standaloneColorPickerVisible",!1,m("standaloneColorPickerVisible","Whether the standalone color picker is visible")),o.standaloneColorPickerFocused=new le("standaloneColorPickerFocused",!1,m("standaloneColorPickerFocused","Whether the standalone color picker is focused")),o.inCompositeEditor=new le("inCompositeEditor",void 0,m("inCompositeEditor","Whether the editor is part of a larger editor (e.g. notebooks)")),o.notInCompositeEditor=o.inCompositeEditor.toNegated(),o.languageId=new le("editorLangId","",m("editorLangId","The language identifier of the editor")),o.hasCompletionItemProvider=new le("editorHasCompletionItemProvider",!1,m("editorHasCompletionItemProvider","Whether the editor has a completion item provider")),o.hasCodeActionsProvider=new le("editorHasCodeActionsProvider",!1,m("editorHasCodeActionsProvider","Whether the editor has a code actions provider")),o.hasCodeLensProvider=new le("editorHasCodeLensProvider",!1,m("editorHasCodeLensProvider","Whether the editor has a code lens provider")),o.hasDefinitionProvider=new le("editorHasDefinitionProvider",!1,m("editorHasDefinitionProvider","Whether the editor has a definition provider")),o.hasDeclarationProvider=new le("editorHasDeclarationProvider",!1,m("editorHasDeclarationProvider","Whether the editor has a declaration provider")),o.hasImplementationProvider=new le("editorHasImplementationProvider",!1,m("editorHasImplementationProvider","Whether the editor has an implementation provider")),o.hasTypeDefinitionProvider=new le("editorHasTypeDefinitionProvider",!1,m("editorHasTypeDefinitionProvider","Whether the editor has a type definition provider")),o.hasHoverProvider=new le("editorHasHoverProvider",!1,m("editorHasHoverProvider","Whether the editor has a hover provider")),o.hasDocumentHighlightProvider=new le("editorHasDocumentHighlightProvider",!1,m("editorHasDocumentHighlightProvider","Whether the editor has a document highlight provider")),o.hasDocumentSymbolProvider=new le("editorHasDocumentSymbolProvider",!1,m("editorHasDocumentSymbolProvider","Whether the editor has a document symbol provider")),o.hasReferenceProvider=new le("editorHasReferenceProvider",!1,m("editorHasReferenceProvider","Whether the editor has a reference provider")),o.hasRenameProvider=new le("editorHasRenameProvider",!1,m("editorHasRenameProvider","Whether the editor has a rename provider")),o.hasSignatureHelpProvider=new le("editorHasSignatureHelpProvider",!1,m("editorHasSignatureHelpProvider","Whether the editor has a signature help provider")),o.hasInlayHintsProvider=new le("editorHasInlayHintsProvider",!1,m("editorHasInlayHintsProvider","Whether the editor has an inline hints provider")),o.hasDocumentFormattingProvider=new le("editorHasDocumentFormattingProvider",!1,m("editorHasDocumentFormattingProvider","Whether the editor has a document formatting provider")),o.hasDocumentSelectionFormattingProvider=new le("editorHasDocumentSelectionFormattingProvider",!1,m("editorHasDocumentSelectionFormattingProvider","Whether the editor has a document selection formatting provider")),o.hasMultipleDocumentFormattingProvider=new le("editorHasMultipleDocumentFormattingProvider",!1,m("editorHasMultipleDocumentFormattingProvider","Whether the editor has multiple document formatting providers")),o.hasMultipleDocumentSelectionFormattingProvider=new le("editorHasMultipleDocumentSelectionFormattingProvider",!1,m("editorHasMultipleDocumentSelectionFormattingProvider","Whether the editor has multiple document selection formatting providers"))})(I||(I={}));const vt=0;class bi extends $i{runEditorCommand(e,t,i){const n=t._getViewModel();n&&this.runCoreEditorCommand(n,i||{})}}var Sn;(function(o){const e=function(i){if(!on(i))return!1;const n=i;return!(!As(n.to)||!Xn(n.by)&&!As(n.by)||!Xn(n.value)&&!Rh(n.value)||!Xn(n.revealCursor)&&!Sz(n.revealCursor))};o.metadata={description:"Scroll editor in the given direction",args:[{name:"Editor scroll argument object",description:"Property-value pairs that can be passed through this argument:\n					* 'to': A mandatory direction value.\n						```\n						'up', 'down'\n						```\n					* 'by': Unit to move. Default is computed based on 'to' value.\n						```\n						'line', 'wrappedLine', 'page', 'halfPage', 'editor'\n						```\n					* 'value': Number of units to move. Default is '1'.\n					* 'revealCursor': If 'true' reveals the cursor if it is outside view port.\n				",constraint:e,schema:{type:"object",required:["to"],properties:{to:{type:"string",enum:["up","down"]},by:{type:"string",enum:["line","wrappedLine","page","halfPage","editor"]},value:{type:"number",default:1},revealCursor:{type:"boolean"}}}}]},o.RawDirection={Up:"up",Right:"right",Down:"down",Left:"left"},o.RawUnit={Line:"line",WrappedLine:"wrappedLine",Page:"page",HalfPage:"halfPage",Editor:"editor",Column:"column"};function t(i){let n;switch(i.to){case o.RawDirection.Up:n=1;break;case o.RawDirection.Right:n=2;break;case o.RawDirection.Down:n=3;break;case o.RawDirection.Left:n=4;break;default:return null}let s;switch(i.by){case o.RawUnit.Line:s=1;break;case o.RawUnit.WrappedLine:s=2;break;case o.RawUnit.Page:s=3;break;case o.RawUnit.HalfPage:s=4;break;case o.RawUnit.Editor:s=5;break;case o.RawUnit.Column:s=6;break;default:s=2}const r=Math.floor(i.value||1),a=!!i.revealCursor;return{direction:n,unit:s,value:r,revealCursor:a,select:!!i.select}}o.parse=t})(Sn||(Sn={}));var up;(function(o){const e=function(t){if(!on(t))return!1;const i=t;return!(!Rh(i.lineNumber)&&!As(i.lineNumber)||!Xn(i.at)&&!As(i.at))};o.metadata={description:"Reveal the given line at the given logical position",args:[{name:"Reveal line argument object",description:"Property-value pairs that can be passed through this argument:\n					* 'lineNumber': A mandatory line number value.\n					* 'at': Logical position at which line has to be revealed.\n						```\n						'top', 'center', 'bottom'\n						```\n				",constraint:e,schema:{type:"object",required:["lineNumber"],properties:{lineNumber:{type:["number","string"]},at:{type:"string",enum:["top","center","bottom"]}}}}]},o.RawAtArgument={Top:"top",Center:"center",Bottom:"bottom"}})(up||(up={}));class gP{constructor(e){e.addImplementation(1e4,"code-editor",(t,i)=>{const n=t.get(pt).getFocusedCodeEditor();return n&&n.hasTextFocus()?this._runEditorCommand(t,n,i):!1}),e.addImplementation(1e3,"generic-dom-input-textarea",(t,i)=>{const n=Dn();return n&&["input","textarea"].indexOf(n.tagName.toLowerCase())>=0?(this.runDOMCommand(n),!0):!1}),e.addImplementation(0,"generic-dom",(t,i)=>{const n=t.get(pt).getActiveCodeEditor();return n?(n.focus(),this._runEditorCommand(t,n,i)):!1})}_runEditorCommand(e,t,i){const n=this.runEditorCommand(e,t,i);return n||!0}}var en;(function(o){class e extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){if(!v.position)return;C.model.pushStackElement(),C.setCursorStates(v.source,3,[un.moveTo(C,C.getPrimaryCursorState(),this._inSelectionMode,v.position,v.viewPosition)])&&v.revealType!==2&&C.revealAllCursors(v.source,!0,!0)}}o.MoveTo=re(new e({id:"_moveTo",inSelectionMode:!1,precondition:void 0})),o.MoveToSelect=re(new e({id:"_moveToSelect",inSelectionMode:!0,precondition:void 0}));class t extends bi{runCoreEditorCommand(C,v){C.model.pushStackElement();const w=this._getColumnSelectResult(C,C.getPrimaryCursorState(),C.getCursorColumnSelectData(),v);w!==null&&(C.setCursorStates(v.source,3,w.viewStates.map(S=>ft.fromViewState(S))),C.setCursorColumnSelectData({isReal:!0,fromViewLineNumber:w.fromLineNumber,fromViewVisualColumn:w.fromVisualColumn,toViewLineNumber:w.toLineNumber,toViewVisualColumn:w.toVisualColumn}),w.reversed?C.revealTopMostCursor(v.source):C.revealBottomMostCursor(v.source))}}o.ColumnSelect=re(new class extends t{constructor(){super({id:"columnSelect",precondition:void 0})}_getColumnSelectResult(b,C,v,w){if(typeof w.position>"u"||typeof w.viewPosition>"u"||typeof w.mouseColumn>"u")return null;const S=b.model.validatePosition(w.position),L=b.coordinatesConverter.validateViewPosition(new A(w.viewPosition.lineNumber,w.viewPosition.column),S),k=w.doColumnSelect?v.fromViewLineNumber:L.lineNumber,D=w.doColumnSelect?v.fromViewVisualColumn:w.mouseColumn-1;return Xu.columnSelect(b.cursorConfig,b,k,D,L.lineNumber,w.mouseColumn-1)}}),o.CursorColumnSelectLeft=re(new class extends t{constructor(){super({id:"cursorColumnSelectLeft",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:3599,linux:{primary:0}}})}_getColumnSelectResult(b,C,v,w){return Xu.columnSelectLeft(b.cursorConfig,b,v)}}),o.CursorColumnSelectRight=re(new class extends t{constructor(){super({id:"cursorColumnSelectRight",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:3601,linux:{primary:0}}})}_getColumnSelectResult(b,C,v,w){return Xu.columnSelectRight(b.cursorConfig,b,v)}});class i extends t{constructor(C){super(C),this._isPaged=C.isPaged}_getColumnSelectResult(C,v,w,S){return Xu.columnSelectUp(C.cursorConfig,C,w,this._isPaged)}}o.CursorColumnSelectUp=re(new i({isPaged:!1,id:"cursorColumnSelectUp",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:3600,linux:{primary:0}}})),o.CursorColumnSelectPageUp=re(new i({isPaged:!0,id:"cursorColumnSelectPageUp",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:3595,linux:{primary:0}}}));class n extends t{constructor(C){super(C),this._isPaged=C.isPaged}_getColumnSelectResult(C,v,w,S){return Xu.columnSelectDown(C.cursorConfig,C,w,this._isPaged)}}o.CursorColumnSelectDown=re(new n({isPaged:!1,id:"cursorColumnSelectDown",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:3602,linux:{primary:0}}})),o.CursorColumnSelectPageDown=re(new n({isPaged:!0,id:"cursorColumnSelectPageDown",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:3596,linux:{primary:0}}}));class s extends bi{constructor(){super({id:"cursorMove",precondition:void 0,metadata:cx.metadata})}runCoreEditorCommand(C,v){const w=cx.parse(v);w&&this._runCursorMove(C,v.source,w)}_runCursorMove(C,v,w){C.model.pushStackElement(),C.setCursorStates(v,3,s._move(C,C.getCursorStates(),w)),C.revealAllCursors(v,!0)}static _move(C,v,w){const S=w.select,L=w.value;switch(w.direction){case 0:case 1:case 2:case 3:case 4:case 5:case 6:case 7:case 8:case 9:case 10:return un.simpleMove(C,v,w.direction,S,L,w.unit);case 11:case 13:case 12:case 14:return un.viewportMove(C,v,w.direction,S,L);default:return null}}}o.CursorMoveImpl=s,o.CursorMove=re(new s);class r extends bi{constructor(C){super(C),this._staticArgs=C.args}runCoreEditorCommand(C,v){let w=this._staticArgs;this._staticArgs.value===-1&&(w={direction:this._staticArgs.direction,unit:this._staticArgs.unit,select:this._staticArgs.select,value:v.pageSize||C.cursorConfig.pageSize}),C.model.pushStackElement(),C.setCursorStates(v.source,3,un.simpleMove(C,C.getCursorStates(),w.direction,w.select,w.value,w.unit)),C.revealAllCursors(v.source,!0)}}o.CursorLeft=re(new r({args:{direction:0,unit:0,select:!1,value:1},id:"cursorLeft",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:15,mac:{primary:15,secondary:[288]}}})),o.CursorLeftSelect=re(new r({args:{direction:0,unit:0,select:!0,value:1},id:"cursorLeftSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:1039}})),o.CursorRight=re(new r({args:{direction:1,unit:0,select:!1,value:1},id:"cursorRight",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:17,mac:{primary:17,secondary:[292]}}})),o.CursorRightSelect=re(new r({args:{direction:1,unit:0,select:!0,value:1},id:"cursorRightSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:1041}})),o.CursorUp=re(new r({args:{direction:2,unit:2,select:!1,value:1},id:"cursorUp",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:16,mac:{primary:16,secondary:[302]}}})),o.CursorUpSelect=re(new r({args:{direction:2,unit:2,select:!0,value:1},id:"cursorUpSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:1040,secondary:[3088],mac:{primary:1040},linux:{primary:1040}}})),o.CursorPageUp=re(new r({args:{direction:2,unit:2,select:!1,value:-1},id:"cursorPageUp",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:11}})),o.CursorPageUpSelect=re(new r({args:{direction:2,unit:2,select:!0,value:-1},id:"cursorPageUpSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:1035}})),o.CursorDown=re(new r({args:{direction:3,unit:2,select:!1,value:1},id:"cursorDown",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:18,mac:{primary:18,secondary:[300]}}})),o.CursorDownSelect=re(new r({args:{direction:3,unit:2,select:!0,value:1},id:"cursorDownSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:1042,secondary:[3090],mac:{primary:1042},linux:{primary:1042}}})),o.CursorPageDown=re(new r({args:{direction:3,unit:2,select:!1,value:-1},id:"cursorPageDown",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:12}})),o.CursorPageDownSelect=re(new r({args:{direction:3,unit:2,select:!0,value:-1},id:"cursorPageDownSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:1036}})),o.CreateCursor=re(new class extends bi{constructor(){super({id:"createCursor",precondition:void 0})}runCoreEditorCommand(b,C){if(!C.position)return;let v;C.wholeLine?v=un.line(b,b.getPrimaryCursorState(),!1,C.position,C.viewPosition):v=un.moveTo(b,b.getPrimaryCursorState(),!1,C.position,C.viewPosition);const w=b.getCursorStates();if(w.length>1){const S=v.modelState?v.modelState.position:null,L=v.viewState?v.viewState.position:null;for(let k=0,D=w.length;k<D;k++){const N=w[k];if(!(S&&!N.modelState.selection.containsPosition(S))&&!(L&&!N.viewState.selection.containsPosition(L))){w.splice(k,1),b.model.pushStackElement(),b.setCursorStates(C.source,3,w);return}}}w.push(v),b.model.pushStackElement(),b.setCursorStates(C.source,3,w)}}),o.LastCursorMoveToSelect=re(new class extends bi{constructor(){super({id:"_lastCursorMoveToSelect",precondition:void 0})}runCoreEditorCommand(b,C){if(!C.position)return;const v=b.getLastAddedCursorIndex(),w=b.getCursorStates(),S=w.slice(0);S[v]=un.moveTo(b,w[v],!0,C.position,C.viewPosition),b.model.pushStackElement(),b.setCursorStates(C.source,3,S)}});class a extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){C.model.pushStackElement(),C.setCursorStates(v.source,3,un.moveToBeginningOfLine(C,C.getCursorStates(),this._inSelectionMode)),C.revealAllCursors(v.source,!0)}}o.CursorHome=re(new a({inSelectionMode:!1,id:"cursorHome",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:14,mac:{primary:14,secondary:[2063]}}})),o.CursorHomeSelect=re(new a({inSelectionMode:!0,id:"cursorHomeSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:1038,mac:{primary:1038,secondary:[3087]}}}));class l extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){C.model.pushStackElement(),C.setCursorStates(v.source,3,this._exec(C.getCursorStates())),C.revealAllCursors(v.source,!0)}_exec(C){const v=[];for(let w=0,S=C.length;w<S;w++){const L=C[w],k=L.modelState.position.lineNumber;v[w]=ft.fromModelState(L.modelState.move(this._inSelectionMode,k,1,0))}return v}}o.CursorLineStart=re(new l({inSelectionMode:!1,id:"cursorLineStart",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:0,mac:{primary:287}}})),o.CursorLineStartSelect=re(new l({inSelectionMode:!0,id:"cursorLineStartSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:0,mac:{primary:1311}}}));class c extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){C.model.pushStackElement(),C.setCursorStates(v.source,3,un.moveToEndOfLine(C,C.getCursorStates(),this._inSelectionMode,v.sticky||!1)),C.revealAllCursors(v.source,!0)}}o.CursorEnd=re(new c({inSelectionMode:!1,id:"cursorEnd",precondition:void 0,kbOpts:{args:{sticky:!1},weight:vt,kbExpr:I.textInputFocus,primary:13,mac:{primary:13,secondary:[2065]}},metadata:{description:"Go to End",args:[{name:"args",schema:{type:"object",properties:{sticky:{description:m("stickydesc","Stick to the end even when going to longer lines"),type:"boolean",default:!1}}}}]}})),o.CursorEndSelect=re(new c({inSelectionMode:!0,id:"cursorEndSelect",precondition:void 0,kbOpts:{args:{sticky:!1},weight:vt,kbExpr:I.textInputFocus,primary:1037,mac:{primary:1037,secondary:[3089]}},metadata:{description:"Select to End",args:[{name:"args",schema:{type:"object",properties:{sticky:{description:m("stickydesc","Stick to the end even when going to longer lines"),type:"boolean",default:!1}}}}]}}));class d extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){C.model.pushStackElement(),C.setCursorStates(v.source,3,this._exec(C,C.getCursorStates())),C.revealAllCursors(v.source,!0)}_exec(C,v){const w=[];for(let S=0,L=v.length;S<L;S++){const k=v[S],D=k.modelState.position.lineNumber,N=C.model.getLineMaxColumn(D);w[S]=ft.fromModelState(k.modelState.move(this._inSelectionMode,D,N,0))}return w}}o.CursorLineEnd=re(new d({inSelectionMode:!1,id:"cursorLineEnd",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:0,mac:{primary:291}}})),o.CursorLineEndSelect=re(new d({inSelectionMode:!0,id:"cursorLineEndSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:0,mac:{primary:1315}}}));class h extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){C.model.pushStackElement(),C.setCursorStates(v.source,3,un.moveToBeginningOfBuffer(C,C.getCursorStates(),this._inSelectionMode)),C.revealAllCursors(v.source,!0)}}o.CursorTop=re(new h({inSelectionMode:!1,id:"cursorTop",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:2062,mac:{primary:2064}}})),o.CursorTopSelect=re(new h({inSelectionMode:!0,id:"cursorTopSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:3086,mac:{primary:3088}}}));class u extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){C.model.pushStackElement(),C.setCursorStates(v.source,3,un.moveToEndOfBuffer(C,C.getCursorStates(),this._inSelectionMode)),C.revealAllCursors(v.source,!0)}}o.CursorBottom=re(new u({inSelectionMode:!1,id:"cursorBottom",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:2061,mac:{primary:2066}}})),o.CursorBottomSelect=re(new u({inSelectionMode:!0,id:"cursorBottomSelect",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:3085,mac:{primary:3090}}}));class g extends bi{constructor(){super({id:"editorScroll",precondition:void 0,metadata:Sn.metadata})}determineScrollMethod(C){const v=[6],w=[1,2,3,4,5,6],S=[4,2],L=[1,3];return v.includes(C.unit)&&S.includes(C.direction)?this._runHorizontalEditorScroll.bind(this):w.includes(C.unit)&&L.includes(C.direction)?this._runVerticalEditorScroll.bind(this):null}runCoreEditorCommand(C,v){const w=Sn.parse(v);if(!w)return;const S=this.determineScrollMethod(w);S&&S(C,v.source,w)}_runVerticalEditorScroll(C,v,w){const S=this._computeDesiredScrollTop(C,w);if(w.revealCursor){const L=C.getCompletelyVisibleViewRangeAtScrollTop(S);C.setCursorStates(v,3,[un.findPositionInViewportIfOutside(C,C.getPrimaryCursorState(),L,w.select)])}C.viewLayout.setScrollPosition({scrollTop:S},0)}_computeDesiredScrollTop(C,v){if(v.unit===1){const L=C.viewLayout.getFutureViewport(),k=C.getCompletelyVisibleViewRangeAtScrollTop(L.top),D=C.coordinatesConverter.convertViewRangeToModelRange(k);let N;v.direction===1?N=Math.max(1,D.startLineNumber-v.value):N=Math.min(C.model.getLineCount(),D.startLineNumber+v.value);const B=C.coordinatesConverter.convertModelPositionToViewPosition(new A(N,1));return C.viewLayout.getVerticalOffsetForLineNumber(B.lineNumber)}if(v.unit===5){let L=0;return v.direction===3&&(L=C.model.getLineCount()-C.cursorConfig.pageSize),C.viewLayout.getVerticalOffsetForLineNumber(L)}let w;v.unit===3?w=C.cursorConfig.pageSize*v.value:v.unit===4?w=Math.round(C.cursorConfig.pageSize/2)*v.value:w=v.value;const S=(v.direction===1?-1:1)*w;return C.viewLayout.getCurrentScrollTop()+S*C.cursorConfig.lineHeight}_runHorizontalEditorScroll(C,v,w){const S=this._computeDesiredScrollLeft(C,w);C.viewLayout.setScrollPosition({scrollLeft:S},0)}_computeDesiredScrollLeft(C,v){const w=(v.direction===4?-1:1)*v.value;return C.viewLayout.getCurrentScrollLeft()+w*C.cursorConfig.typicalHalfwidthCharacterWidth}}o.EditorScrollImpl=g,o.EditorScroll=re(new g),o.ScrollLineUp=re(new class extends bi{constructor(){super({id:"scrollLineUp",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:2064,mac:{primary:267}}})}runCoreEditorCommand(b,C){o.EditorScroll.runCoreEditorCommand(b,{to:Sn.RawDirection.Up,by:Sn.RawUnit.WrappedLine,value:1,revealCursor:!1,select:!1,source:C.source})}}),o.ScrollPageUp=re(new class extends bi{constructor(){super({id:"scrollPageUp",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:2059,win:{primary:523},linux:{primary:523}}})}runCoreEditorCommand(b,C){o.EditorScroll.runCoreEditorCommand(b,{to:Sn.RawDirection.Up,by:Sn.RawUnit.Page,value:1,revealCursor:!1,select:!1,source:C.source})}}),o.ScrollEditorTop=re(new class extends bi{constructor(){super({id:"scrollEditorTop",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus}})}runCoreEditorCommand(b,C){o.EditorScroll.runCoreEditorCommand(b,{to:Sn.RawDirection.Up,by:Sn.RawUnit.Editor,value:1,revealCursor:!1,select:!1,source:C.source})}}),o.ScrollLineDown=re(new class extends bi{constructor(){super({id:"scrollLineDown",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:2066,mac:{primary:268}}})}runCoreEditorCommand(b,C){o.EditorScroll.runCoreEditorCommand(b,{to:Sn.RawDirection.Down,by:Sn.RawUnit.WrappedLine,value:1,revealCursor:!1,select:!1,source:C.source})}}),o.ScrollPageDown=re(new class extends bi{constructor(){super({id:"scrollPageDown",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:2060,win:{primary:524},linux:{primary:524}}})}runCoreEditorCommand(b,C){o.EditorScroll.runCoreEditorCommand(b,{to:Sn.RawDirection.Down,by:Sn.RawUnit.Page,value:1,revealCursor:!1,select:!1,source:C.source})}}),o.ScrollEditorBottom=re(new class extends bi{constructor(){super({id:"scrollEditorBottom",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus}})}runCoreEditorCommand(b,C){o.EditorScroll.runCoreEditorCommand(b,{to:Sn.RawDirection.Down,by:Sn.RawUnit.Editor,value:1,revealCursor:!1,select:!1,source:C.source})}}),o.ScrollLeft=re(new class extends bi{constructor(){super({id:"scrollLeft",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus}})}runCoreEditorCommand(b,C){o.EditorScroll.runCoreEditorCommand(b,{to:Sn.RawDirection.Left,by:Sn.RawUnit.Column,value:2,revealCursor:!1,select:!1,source:C.source})}}),o.ScrollRight=re(new class extends bi{constructor(){super({id:"scrollRight",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus}})}runCoreEditorCommand(b,C){o.EditorScroll.runCoreEditorCommand(b,{to:Sn.RawDirection.Right,by:Sn.RawUnit.Column,value:2,revealCursor:!1,select:!1,source:C.source})}});class f extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){v.position&&(C.model.pushStackElement(),C.setCursorStates(v.source,3,[un.word(C,C.getPrimaryCursorState(),this._inSelectionMode,v.position)]),v.revealType!==2&&C.revealAllCursors(v.source,!0,!0))}}o.WordSelect=re(new f({inSelectionMode:!1,id:"_wordSelect",precondition:void 0})),o.WordSelectDrag=re(new f({inSelectionMode:!0,id:"_wordSelectDrag",precondition:void 0})),o.LastCursorWordSelect=re(new class extends bi{constructor(){super({id:"lastCursorWordSelect",precondition:void 0})}runCoreEditorCommand(b,C){if(!C.position)return;const v=b.getLastAddedCursorIndex(),w=b.getCursorStates(),S=w.slice(0),L=w[v];S[v]=un.word(b,L,L.modelState.hasSelection(),C.position),b.model.pushStackElement(),b.setCursorStates(C.source,3,S)}});class p extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){v.position&&(C.model.pushStackElement(),C.setCursorStates(v.source,3,[un.line(C,C.getPrimaryCursorState(),this._inSelectionMode,v.position,v.viewPosition)]),v.revealType!==2&&C.revealAllCursors(v.source,!1,!0))}}o.LineSelect=re(new p({inSelectionMode:!1,id:"_lineSelect",precondition:void 0})),o.LineSelectDrag=re(new p({inSelectionMode:!0,id:"_lineSelectDrag",precondition:void 0}));class _ extends bi{constructor(C){super(C),this._inSelectionMode=C.inSelectionMode}runCoreEditorCommand(C,v){if(!v.position)return;const w=C.getLastAddedCursorIndex(),S=C.getCursorStates(),L=S.slice(0);L[w]=un.line(C,S[w],this._inSelectionMode,v.position,v.viewPosition),C.model.pushStackElement(),C.setCursorStates(v.source,3,L)}}o.LastCursorLineSelect=re(new _({inSelectionMode:!1,id:"lastCursorLineSelect",precondition:void 0})),o.LastCursorLineSelectDrag=re(new _({inSelectionMode:!0,id:"lastCursorLineSelectDrag",precondition:void 0})),o.CancelSelection=re(new class extends bi{constructor(){super({id:"cancelSelection",precondition:I.hasNonEmptySelection,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:9,secondary:[1033]}})}runCoreEditorCommand(b,C){b.model.pushStackElement(),b.setCursorStates(C.source,3,[un.cancelSelection(b,b.getPrimaryCursorState())]),b.revealAllCursors(C.source,!0)}}),o.RemoveSecondaryCursors=re(new class extends bi{constructor(){super({id:"removeSecondaryCursors",precondition:I.hasMultipleSelections,kbOpts:{weight:vt+1,kbExpr:I.textInputFocus,primary:9,secondary:[1033]}})}runCoreEditorCommand(b,C){b.model.pushStackElement(),b.setCursorStates(C.source,3,[b.getPrimaryCursorState()]),b.revealAllCursors(C.source,!0),Hl(m("removedCursor","Removed secondary cursors"))}}),o.RevealLine=re(new class extends bi{constructor(){super({id:"revealLine",precondition:void 0,metadata:up.metadata})}runCoreEditorCommand(b,C){const v=C,w=v.lineNumber||0;let S=typeof w=="number"?w+1:parseInt(w)+1;S<1&&(S=1);const L=b.model.getLineCount();S>L&&(S=L);const k=new x(S,1,S,b.model.getLineMaxColumn(S));let D=0;if(v.at)switch(v.at){case up.RawAtArgument.Top:D=3;break;case up.RawAtArgument.Center:D=1;break;case up.RawAtArgument.Bottom:D=4;break}const N=b.coordinatesConverter.convertModelRangeToViewRange(k);b.revealRange(C.source,!1,N,D,0)}}),o.SelectAll=new class extends gP{constructor(){super(Oie)}runDOMCommand(b){pr&&(b.focus(),b.select()),b.ownerDocument.execCommand("selectAll")}runEditorCommand(b,C,v){const w=C._getViewModel();w&&this.runCoreEditorCommand(w,v)}runCoreEditorCommand(b,C){b.model.pushStackElement(),b.setCursorStates("keyboard",3,[un.selectAll(b,b.getPrimaryCursorState())])}},o.SetSelection=re(new class extends bi{constructor(){super({id:"setSelection",precondition:void 0})}runCoreEditorCommand(b,C){C.selection&&(b.model.pushStackElement(),b.setCursorStates(C.source,3,[ft.fromModelSelection(C.selection)]))}})})(en||(en={}));const $_e=G.and(I.textInputFocus,I.columnSelection);function Tb(o,e){bs.registerKeybindingRule({id:o,primary:e,when:$_e,weight:vt+1})}Tb(en.CursorColumnSelectLeft.id,1039);Tb(en.CursorColumnSelectRight.id,1041);Tb(en.CursorColumnSelectUp.id,1040);Tb(en.CursorColumnSelectPageUp.id,1035);Tb(en.CursorColumnSelectDown.id,1042);Tb(en.CursorColumnSelectPageDown.id,1036);function xH(o){return o.register(),o}var r_;(function(o){class e extends $i{runEditorCommand(i,n,s){const r=n._getViewModel();r&&this.runCoreEditingCommand(n,r,s||{})}}o.CoreEditingCommand=e,o.LineBreakInsert=re(new class extends e{constructor(){super({id:"lineBreakInsert",precondition:I.writable,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:0,mac:{primary:301}}})}runCoreEditingCommand(t,i,n){t.pushUndoStop(),t.executeCommands(this.id,CE.lineBreakInsert(i.cursorConfig,i.model,i.getCursorStates().map(s=>s.modelState.selection)))}}),o.Outdent=re(new class extends e{constructor(){super({id:"outdent",precondition:I.writable,kbOpts:{weight:vt,kbExpr:G.and(I.editorTextFocus,I.tabDoesNotMoveFocus),primary:1026}})}runCoreEditingCommand(t,i,n){t.pushUndoStop(),t.executeCommands(this.id,Ud.outdent(i.cursorConfig,i.model,i.getCursorStates().map(s=>s.modelState.selection))),t.pushUndoStop()}}),o.Tab=re(new class extends e{constructor(){super({id:"tab",precondition:I.writable,kbOpts:{weight:vt,kbExpr:G.and(I.editorTextFocus,I.tabDoesNotMoveFocus),primary:2}})}runCoreEditingCommand(t,i,n){t.pushUndoStop(),t.executeCommands(this.id,Ud.tab(i.cursorConfig,i.model,i.getCursorStates().map(s=>s.modelState.selection))),t.pushUndoStop()}}),o.DeleteLeft=re(new class extends e{constructor(){super({id:"deleteLeft",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:1,secondary:[1025],mac:{primary:1,secondary:[1025,294,257]}}})}runCoreEditingCommand(t,i,n){const[s,r]=Bf.deleteLeft(i.getPrevEditOperationType(),i.cursorConfig,i.model,i.getCursorStates().map(a=>a.modelState.selection),i.getCursorAutoClosedCharacters());s&&t.pushUndoStop(),t.executeCommands(this.id,r),i.setPrevEditOperationType(2)}}),o.DeleteRight=re(new class extends e{constructor(){super({id:"deleteRight",precondition:void 0,kbOpts:{weight:vt,kbExpr:I.textInputFocus,primary:20,mac:{primary:20,secondary:[290,276]}}})}runCoreEditingCommand(t,i,n){const[s,r]=Bf.deleteRight(i.getPrevEditOperationType(),i.cursorConfig,i.model,i.getCursorStates().map(a=>a.modelState.selection));s&&t.pushUndoStop(),t.executeCommands(this.id,r),i.setPrevEditOperationType(3)}}),o.Undo=new class extends gP{constructor(){super(zU)}runDOMCommand(t){t.ownerDocument.execCommand("undo")}runEditorCommand(t,i,n){if(!(!i.hasModel()||i.getOption(92)===!0))return i.getModel().undo()}},o.Redo=new class extends gP{constructor(){super(UU)}runDOMCommand(t){t.ownerDocument.execCommand("redo")}runEditorCommand(t,i,n){if(!(!i.hasModel()||i.getOption(92)===!0))return i.getModel().redo()}}})(r_||(r_={}));class kH extends TI{constructor(e,t,i){super({id:e,precondition:void 0,metadata:i}),this._handlerId=t}runCommand(e,t){const i=e.get(pt).getFocusedCodeEditor();i&&i.trigger("keyboard",this._handlerId,t)}}function cm(o,e){xH(new kH("default:"+o,o)),xH(new kH(o,o,e))}cm("type",{description:"Type",args:[{name:"args",schema:{type:"object",required:["text"],properties:{text:{type:"string"}}}}]});cm("replacePreviousChar");cm("compositionType");cm("compositionStart");cm("compositionEnd");cm("paste");cm("cut");class j_e{constructor(e,t,i,n){this.configuration=e,this.viewModel=t,this.userInputEvents=i,this.commandDelegate=n}paste(e,t,i,n){this.commandDelegate.paste(e,t,i,n)}type(e){this.commandDelegate.type(e)}compositionType(e,t,i,n){this.commandDelegate.compositionType(e,t,i,n)}compositionStart(){this.commandDelegate.startComposition()}compositionEnd(){this.commandDelegate.endComposition()}cut(){this.commandDelegate.cut()}setSelection(e){en.SetSelection.runCoreEditorCommand(this.viewModel,{source:"keyboard",selection:e})}_validateViewColumn(e){const t=this.viewModel.getLineMinColumn(e.lineNumber);return e.column<t?new A(e.lineNumber,t):e}_hasMulticursorModifier(e){switch(this.configuration.options.get(78)){case"altKey":return e.altKey;case"ctrlKey":return e.ctrlKey;case"metaKey":return e.metaKey;default:return!1}}_hasNonMulticursorModifier(e){switch(this.configuration.options.get(78)){case"altKey":return e.ctrlKey||e.metaKey;case"ctrlKey":return e.altKey||e.metaKey;case"metaKey":return e.ctrlKey||e.altKey;default:return!1}}dispatchMouse(e){const t=this.configuration.options,i=ps&&t.get(108),n=t.get(22);e.middleButton&&!i?this._columnSelect(e.position,e.mouseColumn,e.inSelectionMode):e.startedOnLineNumbers?this._hasMulticursorModifier(e)?e.inSelectionMode?this._lastCursorLineSelect(e.position,e.revealType):this._createCursor(e.position,!0):e.inSelectionMode?this._lineSelectDrag(e.position,e.revealType):this._lineSelect(e.position,e.revealType):e.mouseDownCount>=4?this._selectAll():e.mouseDownCount===3?this._hasMulticursorModifier(e)?e.inSelectionMode?this._lastCursorLineSelectDrag(e.position,e.revealType):this._lastCursorLineSelect(e.position,e.revealType):e.inSelectionMode?this._lineSelectDrag(e.position,e.revealType):this._lineSelect(e.position,e.revealType):e.mouseDownCount===2?e.onInjectedText||(this._hasMulticursorModifier(e)?this._lastCursorWordSelect(e.position,e.revealType):e.inSelectionMode?this._wordSelectDrag(e.position,e.revealType):this._wordSelect(e.position,e.revealType)):this._hasMulticursorModifier(e)?this._hasNonMulticursorModifier(e)||(e.shiftKey?this._columnSelect(e.position,e.mouseColumn,!0):e.inSelectionMode?this._lastCursorMoveToSelect(e.position,e.revealType):this._createCursor(e.position,!1)):e.inSelectionMode?e.altKey?this._columnSelect(e.position,e.mouseColumn,!0):n?this._columnSelect(e.position,e.mouseColumn,!0):this._moveToSelect(e.position,e.revealType):this.moveTo(e.position,e.revealType)}_usualArgs(e,t){return e=this._validateViewColumn(e),{source:"mouse",position:this._convertViewToModelPosition(e),viewPosition:e,revealType:t}}moveTo(e,t){en.MoveTo.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_moveToSelect(e,t){en.MoveToSelect.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_columnSelect(e,t,i){e=this._validateViewColumn(e),en.ColumnSelect.runCoreEditorCommand(this.viewModel,{source:"mouse",position:this._convertViewToModelPosition(e),viewPosition:e,mouseColumn:t,doColumnSelect:i})}_createCursor(e,t){e=this._validateViewColumn(e),en.CreateCursor.runCoreEditorCommand(this.viewModel,{source:"mouse",position:this._convertViewToModelPosition(e),viewPosition:e,wholeLine:t})}_lastCursorMoveToSelect(e,t){en.LastCursorMoveToSelect.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_wordSelect(e,t){en.WordSelect.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_wordSelectDrag(e,t){en.WordSelectDrag.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_lastCursorWordSelect(e,t){en.LastCursorWordSelect.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_lineSelect(e,t){en.LineSelect.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_lineSelectDrag(e,t){en.LineSelectDrag.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_lastCursorLineSelect(e,t){en.LastCursorLineSelect.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_lastCursorLineSelectDrag(e,t){en.LastCursorLineSelectDrag.runCoreEditorCommand(this.viewModel,this._usualArgs(e,t))}_selectAll(){en.SelectAll.runCoreEditorCommand(this.viewModel,{source:"mouse"})}_convertViewToModelPosition(e){return this.viewModel.coordinatesConverter.convertViewPositionToModelPosition(e)}emitKeyDown(e){this.userInputEvents.emitKeyDown(e)}emitKeyUp(e){this.userInputEvents.emitKeyUp(e)}emitContextMenu(e){this.userInputEvents.emitContextMenu(e)}emitMouseMove(e){this.userInputEvents.emitMouseMove(e)}emitMouseLeave(e){this.userInputEvents.emitMouseLeave(e)}emitMouseUp(e){this.userInputEvents.emitMouseUp(e)}emitMouseDown(e){this.userInputEvents.emitMouseDown(e)}emitMouseDrag(e){this.userInputEvents.emitMouseDrag(e)}emitMouseDrop(e){this.userInputEvents.emitMouseDrop(e)}emitMouseDropCanceled(){this.userInputEvents.emitMouseDropCanceled()}emitMouseWheel(e){this.userInputEvents.emitMouseWheel(e)}}class Lq{constructor(e){this._lineFactory=e,this._set(1,[])}flush(){this._set(1,[])}_set(e,t){this._lines=t,this._rendLineNumberStart=e}_get(){return{rendLineNumberStart:this._rendLineNumberStart,lines:this._lines}}getStartLineNumber(){return this._rendLineNumberStart}getEndLineNumber(){return this._rendLineNumberStart+this._lines.length-1}getCount(){return this._lines.length}getLine(e){const t=e-this._rendLineNumberStart;if(t<0||t>=this._lines.length)throw new bt("Illegal value for lineNumber");return this._lines[t]}onLinesDeleted(e,t){if(this.getCount()===0)return null;const i=this.getStartLineNumber(),n=this.getEndLineNumber();if(t<i){const l=t-e+1;return this._rendLineNumberStart-=l,null}if(e>n)return null;let s=0,r=0;for(let l=i;l<=n;l++){const c=l-this._rendLineNumberStart;e<=l&&l<=t&&(r===0?(s=c,r=1):r++)}if(e<i){let l=0;t<i?l=t-e+1:l=i-e,this._rendLineNumberStart-=l}return this._lines.splice(s,r)}onLinesChanged(e,t){const i=e+t-1;if(this.getCount()===0)return!1;const n=this.getStartLineNumber(),s=this.getEndLineNumber();let r=!1;for(let a=e;a<=i;a++)a>=n&&a<=s&&(this._lines[a-this._rendLineNumberStart].onContentChanged(),r=!0);return r}onLinesInserted(e,t){if(this.getCount()===0)return null;const i=t-e+1,n=this.getStartLineNumber(),s=this.getEndLineNumber();if(e<=n)return this._rendLineNumberStart+=i,null;if(e>s)return null;if(i+e>s)return this._lines.splice(e-this._rendLineNumberStart,s-e+1);const r=[];for(let h=0;h<i;h++)r[h]=this._lineFactory.createLine();const a=e-this._rendLineNumberStart,l=this._lines.slice(0,a),c=this._lines.slice(a,this._lines.length-i),d=this._lines.slice(this._lines.length-i,this._lines.length);return this._lines=l.concat(r).concat(c),d}onTokensChanged(e){if(this.getCount()===0)return!1;const t=this.getStartLineNumber(),i=this.getEndLineNumber();let n=!1;for(let s=0,r=e.length;s<r;s++){const a=e[s];if(a.toLineNumber<t||a.fromLineNumber>i)continue;const l=Math.max(t,a.fromLineNumber),c=Math.min(i,a.toLineNumber);for(let d=l;d<=c;d++){const h=d-this._rendLineNumberStart;this._lines[h].onTokensChanged(),n=!0}}return n}}class xq{constructor(e){this._lineFactory=e,this.domNode=this._createDomNode(),this._linesCollection=new Lq(this._lineFactory)}_createDomNode(){const e=St(document.createElement("div"));return e.setClassName("view-layer"),e.setPosition("absolute"),e.domNode.setAttribute("role","presentation"),e.domNode.setAttribute("aria-hidden","true"),e}onConfigurationChanged(e){return!!e.hasChanged(146)}onFlushed(e){return this._linesCollection.flush(),!0}onLinesChanged(e){return this._linesCollection.onLinesChanged(e.fromLineNumber,e.count)}onLinesDeleted(e){const t=this._linesCollection.onLinesDeleted(e.fromLineNumber,e.toLineNumber);if(t)for(let i=0,n=t.length;i<n;i++){const s=t[i].getDomNode();s==null||s.remove()}return!0}onLinesInserted(e){const t=this._linesCollection.onLinesInserted(e.fromLineNumber,e.toLineNumber);if(t)for(let i=0,n=t.length;i<n;i++){const s=t[i].getDomNode();s==null||s.remove()}return!0}onScrollChanged(e){return e.scrollTopChanged}onTokensChanged(e){return this._linesCollection.onTokensChanged(e.ranges)}onZonesChanged(e){return!0}getStartLineNumber(){return this._linesCollection.getStartLineNumber()}getEndLineNumber(){return this._linesCollection.getEndLineNumber()}getVisibleLine(e){return this._linesCollection.getLine(e)}renderLines(e){const t=this._linesCollection._get(),i=new fP(this.domNode.domNode,this._lineFactory,e),n={rendLineNumberStart:t.rendLineNumberStart,lines:t.lines,linesLength:t.lines.length},s=i.render(n,e.startLineNumber,e.endLineNumber,e.relativeVerticalOffset);this._linesCollection._set(s.rendLineNumberStart,s.lines)}}const Cc=class Cc{constructor(e,t,i){this._domNode=e,this._lineFactory=t,this._viewportData=i}render(e,t,i,n){const s={rendLineNumberStart:e.rendLineNumberStart,lines:e.lines.slice(0),linesLength:e.linesLength};if(s.rendLineNumberStart+s.linesLength-1<t||i<s.rendLineNumberStart){s.rendLineNumberStart=t,s.linesLength=i-t+1,s.lines=[];for(let r=t;r<=i;r++)s.lines[r-t]=this._lineFactory.createLine();return this._finishRendering(s,!0,n),s}if(this._renderUntouchedLines(s,Math.max(t-s.rendLineNumberStart,0),Math.min(i-s.rendLineNumberStart,s.linesLength-1),n,t),s.rendLineNumberStart>t){const r=t,a=Math.min(i,s.rendLineNumberStart-1);r<=a&&(this._insertLinesBefore(s,r,a,n,t),s.linesLength+=a-r+1)}else if(s.rendLineNumberStart<t){const r=Math.min(s.linesLength,t-s.rendLineNumberStart);r>0&&(this._removeLinesBefore(s,r),s.linesLength-=r)}if(s.rendLineNumberStart=t,s.rendLineNumberStart+s.linesLength-1<i){const r=s.rendLineNumberStart+s.linesLength,a=i;r<=a&&(this._insertLinesAfter(s,r,a,n,t),s.linesLength+=a-r+1)}else if(s.rendLineNumberStart+s.linesLength-1>i){const r=Math.max(0,i-s.rendLineNumberStart+1),l=s.linesLength-1-r+1;l>0&&(this._removeLinesAfter(s,l),s.linesLength-=l)}return this._finishRendering(s,!1,n),s}_renderUntouchedLines(e,t,i,n,s){const r=e.rendLineNumberStart,a=e.lines;for(let l=t;l<=i;l++){const c=r+l;a[l].layoutLine(c,n[c-s],this._viewportData.lineHeight)}}_insertLinesBefore(e,t,i,n,s){const r=[];let a=0;for(let l=t;l<=i;l++)r[a++]=this._lineFactory.createLine();e.lines=r.concat(e.lines)}_removeLinesBefore(e,t){for(let i=0;i<t;i++){const n=e.lines[i].getDomNode();n==null||n.remove()}e.lines.splice(0,t)}_insertLinesAfter(e,t,i,n,s){const r=[];let a=0;for(let l=t;l<=i;l++)r[a++]=this._lineFactory.createLine();e.lines=e.lines.concat(r)}_removeLinesAfter(e,t){const i=e.linesLength-t;for(let n=0;n<t;n++){const s=e.lines[i+n].getDomNode();s==null||s.remove()}e.lines.splice(i,t)}_finishRenderingNewLines(e,t,i,n){Cc._ttPolicy&&(i=Cc._ttPolicy.createHTML(i));const s=this._domNode.lastChild;t||!s?this._domNode.innerHTML=i:s.insertAdjacentHTML("afterend",i);let r=this._domNode.lastChild;for(let a=e.linesLength-1;a>=0;a--){const l=e.lines[a];n[a]&&(l.setDomNode(r),r=r.previousSibling)}}_finishRenderingInvalidLines(e,t,i){const n=document.createElement("div");Cc._ttPolicy&&(t=Cc._ttPolicy.createHTML(t)),n.innerHTML=t;for(let s=0;s<e.linesLength;s++){const r=e.lines[s];if(i[s]){const a=n.firstChild,l=r.getDomNode();l.parentNode.replaceChild(a,l),r.setDomNode(a)}}}_finishRendering(e,t,i){const n=Cc._sb,s=e.linesLength,r=e.lines,a=e.rendLineNumberStart,l=[];{n.reset();let c=!1;for(let d=0;d<s;d++){const h=r[d];l[d]=!1,!(h.getDomNode()||!h.renderLine(d+a,i[d],this._viewportData.lineHeight,this._viewportData,n))&&(l[d]=!0,c=!0)}c&&this._finishRenderingNewLines(e,t,n.build(),l)}{n.reset();let c=!1;const d=[];for(let h=0;h<s;h++){const u=r[h];d[h]=!1,!(l[h]||!u.renderLine(h+a,i[h],this._viewportData.lineHeight,this._viewportData,n))&&(d[h]=!0,c=!0)}c&&this._finishRenderingInvalidLines(e,n.build(),d)}}};Cc._ttPolicy=hd("editorViewLayer",{createHTML:e=>e}),Cc._sb=new _b(1e5);let fP=Cc;class kq extends bo{constructor(e){super(e),this._dynamicOverlays=[],this._isFocused=!1,this._visibleLines=new xq({createLine:()=>new K_e(this._dynamicOverlays)}),this.domNode=this._visibleLines.domNode;const i=this._context.configuration.options.get(50);bn(this.domNode,i),this.domNode.setClassName("view-overlays")}shouldRender(){if(super.shouldRender())return!0;for(let e=0,t=this._dynamicOverlays.length;e<t;e++)if(this._dynamicOverlays[e].shouldRender())return!0;return!1}dispose(){super.dispose();for(let e=0,t=this._dynamicOverlays.length;e<t;e++)this._dynamicOverlays[e].dispose();this._dynamicOverlays=[]}getDomNode(){return this.domNode}addDynamicOverlay(e){this._dynamicOverlays.push(e)}onConfigurationChanged(e){this._visibleLines.onConfigurationChanged(e);const i=this._context.configuration.options.get(50);return bn(this.domNode,i),!0}onFlushed(e){return this._visibleLines.onFlushed(e)}onFocusChanged(e){return this._isFocused=e.isFocused,!0}onLinesChanged(e){return this._visibleLines.onLinesChanged(e)}onLinesDeleted(e){return this._visibleLines.onLinesDeleted(e)}onLinesInserted(e){return this._visibleLines.onLinesInserted(e)}onScrollChanged(e){return this._visibleLines.onScrollChanged(e)||!0}onTokensChanged(e){return this._visibleLines.onTokensChanged(e)}onZonesChanged(e){return this._visibleLines.onZonesChanged(e)}prepareRender(e){const t=this._dynamicOverlays.filter(i=>i.shouldRender());for(let i=0,n=t.length;i<n;i++){const s=t[i];s.prepareRender(e),s.onDidRender()}}render(e){this._viewOverlaysRender(e),this.domNode.toggleClassName("focused",this._isFocused)}_viewOverlaysRender(e){this._visibleLines.renderLines(e.viewportData)}}class K_e{constructor(e){this._dynamicOverlays=e,this._domNode=null,this._renderedContent=null}getDomNode(){return this._domNode?this._domNode.domNode:null}setDomNode(e){this._domNode=St(e)}onContentChanged(){}onTokensChanged(){}renderLine(e,t,i,n,s){let r="";for(let a=0,l=this._dynamicOverlays.length;a<l;a++){const c=this._dynamicOverlays[a];r+=c.render(n.startLineNumber,e)}return this._renderedContent===r?!1:(this._renderedContent=r,s.appendString('<div style="top:'),s.appendString(String(t)),s.appendString("px;height:"),s.appendString(String(i)),s.appendString('px;">'),s.appendString(r),s.appendString("</div>"),!0)}layoutLine(e,t,i){this._domNode&&(this._domNode.setTop(t),this._domNode.setHeight(i))}}class q_e extends kq{constructor(e){super(e);const i=this._context.configuration.options.get(146);this._contentWidth=i.contentWidth,this.domNode.setHeight(0)}onConfigurationChanged(e){const i=this._context.configuration.options.get(146);return this._contentWidth=i.contentWidth,super.onConfigurationChanged(e)||!0}onScrollChanged(e){return super.onScrollChanged(e)||e.scrollWidthChanged}_viewOverlaysRender(e){super._viewOverlaysRender(e),this.domNode.setWidth(Math.max(e.scrollWidth,this._contentWidth))}}class G_e extends kq{constructor(e){super(e);const t=this._context.configuration.options,i=t.get(146);this._contentLeft=i.contentLeft,this.domNode.setClassName("margin-view-overlays"),this.domNode.setWidth(1),bn(this.domNode,t.get(50))}onConfigurationChanged(e){const t=this._context.configuration.options;bn(this.domNode,t.get(50));const i=t.get(146);return this._contentLeft=i.contentLeft,super.onConfigurationChanged(e)||!0}onScrollChanged(e){return super.onScrollChanged(e)||e.scrollHeightChanged}_viewOverlaysRender(e){super._viewOverlaysRender(e);const t=Math.min(e.scrollHeight,1e6);this.domNode.setHeight(t),this.domNode.setWidth(this._contentLeft)}}class wE{constructor(e){this.onKeyDown=null,this.onKeyUp=null,this.onContextMenu=null,this.onMouseMove=null,this.onMouseLeave=null,this.onMouseDown=null,this.onMouseUp=null,this.onMouseDrag=null,this.onMouseDrop=null,this.onMouseDropCanceled=null,this.onMouseWheel=null,this._coordinatesConverter=e}emitKeyDown(e){var t;(t=this.onKeyDown)==null||t.call(this,e)}emitKeyUp(e){var t;(t=this.onKeyUp)==null||t.call(this,e)}emitContextMenu(e){var t;(t=this.onContextMenu)==null||t.call(this,this._convertViewToModelMouseEvent(e))}emitMouseMove(e){var t;(t=this.onMouseMove)==null||t.call(this,this._convertViewToModelMouseEvent(e))}emitMouseLeave(e){var t;(t=this.onMouseLeave)==null||t.call(this,this._convertViewToModelMouseEvent(e))}emitMouseDown(e){var t;(t=this.onMouseDown)==null||t.call(this,this._convertViewToModelMouseEvent(e))}emitMouseUp(e){var t;(t=this.onMouseUp)==null||t.call(this,this._convertViewToModelMouseEvent(e))}emitMouseDrag(e){var t;(t=this.onMouseDrag)==null||t.call(this,this._convertViewToModelMouseEvent(e))}emitMouseDrop(e){var t;(t=this.onMouseDrop)==null||t.call(this,this._convertViewToModelMouseEvent(e))}emitMouseDropCanceled(){var e;(e=this.onMouseDropCanceled)==null||e.call(this)}emitMouseWheel(e){var t;(t=this.onMouseWheel)==null||t.call(this,e)}_convertViewToModelMouseEvent(e){return e.target?{event:e.event,target:this._convertViewToModelMouseTarget(e.target)}:e}_convertViewToModelMouseTarget(e){return wE.convertViewToModelMouseTarget(e,this._coordinatesConverter)}static convertViewToModelMouseTarget(e,t){const i={...e};return i.position&&(i.position=t.convertViewPositionToModelPosition(i.position)),i.range&&(i.range=t.convertViewRangeToModelRange(i.range)),(i.type===5||i.type===8)&&(i.detail=this.convertViewToModelViewZoneData(i.detail,t)),i}static convertViewToModelViewZoneData(e,t){return{viewZoneId:e.viewZoneId,positionBefore:e.positionBefore?t.convertViewPositionToModelPosition(e.positionBefore):e.positionBefore,positionAfter:e.positionAfter?t.convertViewPositionToModelPosition(e.positionAfter):e.positionAfter,position:t.convertViewPositionToModelPosition(e.position),afterLineNumber:t.convertViewPositionToModelPosition(new A(e.afterLineNumber,1)).lineNumber}}}class Z_e extends bo{constructor(e){super(e),this.blocks=[],this.contentWidth=-1,this.contentLeft=0,this.domNode=St(document.createElement("div")),this.domNode.setAttribute("role","presentation"),this.domNode.setAttribute("aria-hidden","true"),this.domNode.setClassName("blockDecorations-container"),this.update()}update(){let e=!1;const i=this._context.configuration.options.get(146),n=i.contentWidth-i.verticalScrollbarWidth;this.contentWidth!==n&&(this.contentWidth=n,e=!0);const s=i.contentLeft;return this.contentLeft!==s&&(this.contentLeft=s,e=!0),e}dispose(){super.dispose()}onConfigurationChanged(e){return this.update()}onScrollChanged(e){return e.scrollTopChanged||e.scrollLeftChanged}onDecorationsChanged(e){return!0}onZonesChanged(e){return!0}prepareRender(e){}render(e){let t=0;const i=e.getDecorationsInViewport();for(const n of i){if(!n.options.blockClassName)continue;let s=this.blocks[t];s||(s=this.blocks[t]=St(document.createElement("div")),this.domNode.appendChild(s));let r,a;n.options.blockIsAfterEnd?(r=e.getVerticalOffsetAfterLineNumber(n.range.endLineNumber,!1),a=e.getVerticalOffsetAfterLineNumber(n.range.endLineNumber,!0)):(r=e.getVerticalOffsetForLineNumber(n.range.startLineNumber,!0),a=n.range.isEmpty()&&!n.options.blockDoesNotCollapse?e.getVerticalOffsetForLineNumber(n.range.startLineNumber,!1):e.getVerticalOffsetAfterLineNumber(n.range.endLineNumber,!0));const[l,c,d,h]=n.options.blockPadding??[0,0,0,0];s.setClassName("blockDecorations-block "+n.options.blockClassName),s.setLeft(this.contentLeft-h),s.setWidth(this.contentWidth+h+c),s.setTop(r-e.scrollTop-l),s.setHeight(a-r+l+d),t++}for(let n=t;n<this.blocks.length;n++)this.blocks[n].domNode.remove();this.blocks.length=t}}class Y_e extends bo{constructor(e,t){super(e),this._viewDomNode=t,this._widgets={},this.domNode=St(document.createElement("div")),Va.write(this.domNode,1),this.domNode.setClassName("contentWidgets"),this.domNode.setPosition("absolute"),this.domNode.setTop(0),this.overflowingContentWidgetsDomNode=St(document.createElement("div")),Va.write(this.overflowingContentWidgetsDomNode,2),this.overflowingContentWidgetsDomNode.setClassName("overflowingContentWidgets")}dispose(){super.dispose(),this._widgets={}}onConfigurationChanged(e){const t=Object.keys(this._widgets);for(const i of t)this._widgets[i].onConfigurationChanged(e);return!0}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onLineMappingChanged(e){return this._updateAnchorsViewPositions(),!0}onLinesChanged(e){return this._updateAnchorsViewPositions(),!0}onLinesDeleted(e){return this._updateAnchorsViewPositions(),!0}onLinesInserted(e){return this._updateAnchorsViewPositions(),!0}onScrollChanged(e){return!0}onZonesChanged(e){return!0}_updateAnchorsViewPositions(){const e=Object.keys(this._widgets);for(const t of e)this._widgets[t].updateAnchorViewPosition()}addWidget(e){const t=new X_e(this._context,this._viewDomNode,e);this._widgets[t.id]=t,t.allowEditorOverflow?this.overflowingContentWidgetsDomNode.appendChild(t.domNode):this.domNode.appendChild(t.domNode),this.setShouldRender()}setWidgetPosition(e,t,i,n,s){this._widgets[e.getId()].setPosition(t,i,n,s),this.setShouldRender()}removeWidget(e){const t=e.getId();if(this._widgets.hasOwnProperty(t)){const i=this._widgets[t];delete this._widgets[t];const n=i.domNode.domNode;n.remove(),n.removeAttribute("monaco-visible-content-widget"),this.setShouldRender()}}shouldSuppressMouseDownOnWidget(e){return this._widgets.hasOwnProperty(e)?this._widgets[e].suppressMouseDown:!1}onBeforeRender(e){const t=Object.keys(this._widgets);for(const i of t)this._widgets[i].onBeforeRender(e)}prepareRender(e){const t=Object.keys(this._widgets);for(const i of t)this._widgets[i].prepareRender(e)}render(e){const t=Object.keys(this._widgets);for(const i of t)this._widgets[i].render(e)}}class X_e{constructor(e,t,i){this._primaryAnchor=new cC(null,null),this._secondaryAnchor=new cC(null,null),this._context=e,this._viewDomNode=t,this._actual=i,this.domNode=St(this._actual.getDomNode()),this.id=this._actual.getId(),this.allowEditorOverflow=this._actual.allowEditorOverflow||!1,this.suppressMouseDown=this._actual.suppressMouseDown||!1;const n=this._context.configuration.options,s=n.get(146);this._fixedOverflowWidgets=n.get(42),this._contentWidth=s.contentWidth,this._contentLeft=s.contentLeft,this._lineHeight=n.get(67),this._affinity=null,this._preference=[],this._cachedDomNodeOffsetWidth=-1,this._cachedDomNodeOffsetHeight=-1,this._maxWidth=this._getMaxWidth(),this._isVisible=!1,this._renderData=null,this.domNode.setPosition(this._fixedOverflowWidgets&&this.allowEditorOverflow?"fixed":"absolute"),this.domNode.setDisplay("none"),this.domNode.setVisibility("hidden"),this.domNode.setAttribute("widgetId",this.id),this.domNode.setMaxWidth(this._maxWidth)}onConfigurationChanged(e){const t=this._context.configuration.options;if(this._lineHeight=t.get(67),e.hasChanged(146)){const i=t.get(146);this._contentLeft=i.contentLeft,this._contentWidth=i.contentWidth,this._maxWidth=this._getMaxWidth()}}updateAnchorViewPosition(){this._setPosition(this._affinity,this._primaryAnchor.modelPosition,this._secondaryAnchor.modelPosition)}_setPosition(e,t,i){this._affinity=e,this._primaryAnchor=n(t,this._context.viewModel,this._affinity),this._secondaryAnchor=n(i,this._context.viewModel,this._affinity);function n(s,r,a){if(!s)return new cC(null,null);const l=r.model.validatePosition(s);if(r.coordinatesConverter.modelPositionIsVisible(l)){const c=r.coordinatesConverter.convertModelPositionToViewPosition(l,a??void 0);return new cC(s,c)}return new cC(s,null)}}_getMaxWidth(){const e=this.domNode.domNode.ownerDocument,t=e.defaultView;return this.allowEditorOverflow?(t==null?void 0:t.innerWidth)||e.documentElement.offsetWidth||e.body.offsetWidth:this._contentWidth}setPosition(e,t,i,n){this._setPosition(n,e,t),this._preference=i,this._primaryAnchor.viewPosition&&this._preference&&this._preference.length>0?this.domNode.setDisplay("block"):this.domNode.setDisplay("none"),this._cachedDomNodeOffsetWidth=-1,this._cachedDomNodeOffsetHeight=-1}_layoutBoxInViewport(e,t,i,n){const s=e.top,r=s,a=e.top+e.height,l=n.viewportHeight-a,c=s-i,d=r>=i,h=a,u=l>=i;let g=e.left;return g+t>n.scrollLeft+n.viewportWidth&&(g=n.scrollLeft+n.viewportWidth-t),g<n.scrollLeft&&(g=n.scrollLeft),{fitsAbove:d,aboveTop:c,fitsBelow:u,belowTop:h,left:g}}_layoutHorizontalSegmentInPage(e,t,i,n){const a=Math.max(15,t.left-n),l=Math.min(t.left+t.width+n,e.width-15),d=this._viewDomNode.domNode.ownerDocument.defaultView;let h=t.left+i-((d==null?void 0:d.scrollX)??0);if(h+n>l){const u=h-(l-n);h-=u,i-=u}if(h<a){const u=h-a;h-=u,i-=u}return[i,h]}_layoutBoxInPage(e,t,i,n){const s=e.top-i,r=e.top+e.height,a=wi(this._viewDomNode.domNode),l=this._viewDomNode.domNode.ownerDocument,c=l.defaultView,d=a.top+s-((c==null?void 0:c.scrollY)??0),h=a.top+r-((c==null?void 0:c.scrollY)??0),u=Hh(l.body),[g,f]=this._layoutHorizontalSegmentInPage(u,a,e.left-n.scrollLeft+this._contentLeft,t),p=22,_=22,b=d>=p,C=h+i<=u.height-_;return this._fixedOverflowWidgets?{fitsAbove:b,aboveTop:Math.max(d,p),fitsBelow:C,belowTop:h,left:f}:{fitsAbove:b,aboveTop:s,fitsBelow:C,belowTop:r,left:g}}_prepareRenderWidgetAtExactPositionOverflowing(e){return new dC(e.top,e.left+this._contentLeft)}_getAnchorsCoordinates(e){var r,a;const t=s(this._primaryAnchor.viewPosition,this._affinity,this._lineHeight),i=((r=this._secondaryAnchor.viewPosition)==null?void 0:r.lineNumber)===((a=this._primaryAnchor.viewPosition)==null?void 0:a.lineNumber)?this._secondaryAnchor.viewPosition:null,n=s(i,this._affinity,this._lineHeight);return{primary:t,secondary:n};function s(l,c,d){if(!l)return null;const h=e.visibleRangeForPosition(l);if(!h)return null;const u=l.column===1&&c===3?0:h.left,g=e.getVerticalOffsetForLineNumber(l.lineNumber)-e.scrollTop;return new DH(g,u,d)}}_reduceAnchorCoordinates(e,t,i){if(!t)return e;const n=this._context.configuration.options.get(50);let s=t.left;return s<e.left?s=Math.max(s,e.left-i+n.typicalFullwidthCharacterWidth):s=Math.min(s,e.left+i-n.typicalFullwidthCharacterWidth),new DH(e.top,s,e.height)}_prepareRenderWidget(e){if(!this._preference||this._preference.length===0)return null;const{primary:t,secondary:i}=this._getAnchorsCoordinates(e);if(!t)return{kind:"offViewport",preserveFocus:this.domNode.domNode.contains(this.domNode.domNode.ownerDocument.activeElement)};if(this._cachedDomNodeOffsetWidth===-1||this._cachedDomNodeOffsetHeight===-1){let r=null;if(typeof this._actual.beforeRender=="function"&&(r=vT(this._actual.beforeRender,this._actual)),r)this._cachedDomNodeOffsetWidth=r.width,this._cachedDomNodeOffsetHeight=r.height;else{const l=this.domNode.domNode.getBoundingClientRect();this._cachedDomNodeOffsetWidth=Math.round(l.width),this._cachedDomNodeOffsetHeight=Math.round(l.height)}}const n=this._reduceAnchorCoordinates(t,i,this._cachedDomNodeOffsetWidth);let s;this.allowEditorOverflow?s=this._layoutBoxInPage(n,this._cachedDomNodeOffsetWidth,this._cachedDomNodeOffsetHeight,e):s=this._layoutBoxInViewport(n,this._cachedDomNodeOffsetWidth,this._cachedDomNodeOffsetHeight,e);for(let r=1;r<=2;r++)for(const a of this._preference)if(a===1){if(!s)return null;if(r===2||s.fitsAbove)return{kind:"inViewport",coordinate:new dC(s.aboveTop,s.left),position:1}}else if(a===2){if(!s)return null;if(r===2||s.fitsBelow)return{kind:"inViewport",coordinate:new dC(s.belowTop,s.left),position:2}}else return this.allowEditorOverflow?{kind:"inViewport",coordinate:this._prepareRenderWidgetAtExactPositionOverflowing(new dC(n.top,n.left)),position:0}:{kind:"inViewport",coordinate:new dC(n.top,n.left),position:0};return null}onBeforeRender(e){!this._primaryAnchor.viewPosition||!this._preference||this._primaryAnchor.viewPosition.lineNumber<e.startLineNumber||this._primaryAnchor.viewPosition.lineNumber>e.endLineNumber||this.domNode.setMaxWidth(this._maxWidth)}prepareRender(e){this._renderData=this._prepareRenderWidget(e)}render(e){var t;if(!this._renderData||this._renderData.kind==="offViewport"){this._isVisible&&(this.domNode.removeAttribute("monaco-visible-content-widget"),this._isVisible=!1,((t=this._renderData)==null?void 0:t.kind)==="offViewport"&&this._renderData.preserveFocus?this.domNode.setTop(-1e3):this.domNode.setVisibility("hidden")),typeof this._actual.afterRender=="function"&&vT(this._actual.afterRender,this._actual,null);return}this.allowEditorOverflow?(this.domNode.setTop(this._renderData.coordinate.top),this.domNode.setLeft(this._renderData.coordinate.left)):(this.domNode.setTop(this._renderData.coordinate.top+e.scrollTop-e.bigNumbersDelta),this.domNode.setLeft(this._renderData.coordinate.left)),this._isVisible||(this.domNode.setVisibility("inherit"),this.domNode.setAttribute("monaco-visible-content-widget","true"),this._isVisible=!0),typeof this._actual.afterRender=="function"&&vT(this._actual.afterRender,this._actual,this._renderData.position)}}class cC{constructor(e,t){this.modelPosition=e,this.viewPosition=t}}class dC{constructor(e,t){this.top=e,this.left=t,this._coordinateBrand=void 0}}class DH{constructor(e,t,i){this.top=e,this.left=t,this.height=i,this._anchorCoordinateBrand=void 0}}function vT(o,e,...t){try{return o.call(e,...t)}catch{return null}}class Dq extends lm{constructor(e){super(),this._context=e;const t=this._context.configuration.options,i=t.get(146);this._renderLineHighlight=t.get(97),this._renderLineHighlightOnlyWhenFocus=t.get(98),this._wordWrap=i.isViewportWrapping,this._contentLeft=i.contentLeft,this._contentWidth=i.contentWidth,this._selectionIsEmpty=!0,this._focused=!1,this._cursorLineNumbers=[1],this._selections=[new pe(1,1,1,1)],this._renderData=null,this._context.addEventHandler(this)}dispose(){this._context.removeEventHandler(this),super.dispose()}_readFromSelections(){let e=!1;const t=new Set;for(const s of this._selections)t.add(s.positionLineNumber);const i=Array.from(t);i.sort((s,r)=>s-r),ii(this._cursorLineNumbers,i)||(this._cursorLineNumbers=i,e=!0);const n=this._selections.every(s=>s.isEmpty());return this._selectionIsEmpty!==n&&(this._selectionIsEmpty=n,e=!0),e}onThemeChanged(e){return this._readFromSelections()}onConfigurationChanged(e){const t=this._context.configuration.options,i=t.get(146);return this._renderLineHighlight=t.get(97),this._renderLineHighlightOnlyWhenFocus=t.get(98),this._wordWrap=i.isViewportWrapping,this._contentLeft=i.contentLeft,this._contentWidth=i.contentWidth,!0}onCursorStateChanged(e){return this._selections=e.selections,this._readFromSelections()}onFlushed(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return e.scrollWidthChanged||e.scrollTopChanged}onZonesChanged(e){return!0}onFocusChanged(e){return this._renderLineHighlightOnlyWhenFocus?(this._focused=e.isFocused,!0):!1}prepareRender(e){if(!this._shouldRenderThis()){this._renderData=null;return}const t=e.visibleRange.startLineNumber,i=e.visibleRange.endLineNumber,n=[];for(let r=t;r<=i;r++){const a=r-t;n[a]=""}if(this._wordWrap){const r=this._renderOne(e,!1);for(const a of this._cursorLineNumbers){const l=this._context.viewModel.coordinatesConverter,c=l.convertViewPositionToModelPosition(new A(a,1)).lineNumber,d=l.convertModelPositionToViewPosition(new A(c,1)).lineNumber,h=l.convertModelPositionToViewPosition(new A(c,this._context.viewModel.model.getLineMaxColumn(c))).lineNumber,u=Math.max(d,t),g=Math.min(h,i);for(let f=u;f<=g;f++){const p=f-t;n[p]=r}}}const s=this._renderOne(e,!0);for(const r of this._cursorLineNumbers){if(r<t||r>i)continue;const a=r-t;n[a]=s}this._renderData=n}render(e,t){if(!this._renderData)return"";const i=t-e;return i>=this._renderData.length?"":this._renderData[i]}_shouldRenderInMargin(){return(this._renderLineHighlight==="gutter"||this._renderLineHighlight==="all")&&(!this._renderLineHighlightOnlyWhenFocus||this._focused)}_shouldRenderInContent(){return(this._renderLineHighlight==="line"||this._renderLineHighlight==="all")&&this._selectionIsEmpty&&(!this._renderLineHighlightOnlyWhenFocus||this._focused)}}class Q_e extends Dq{_renderOne(e,t){return`<div class="${"current-line"+(this._shouldRenderInMargin()?" current-line-both":"")+(t?" current-line-exact":"")}" style="width:${Math.max(e.scrollWidth,this._contentWidth)}px;"></div>`}_shouldRenderThis(){return this._shouldRenderInContent()}_shouldRenderOther(){return this._shouldRenderInMargin()}}class J_e extends Dq{_renderOne(e,t){return`<div class="${"current-line"+(this._shouldRenderInMargin()?" current-line-margin":"")+(this._shouldRenderOther()?" current-line-margin-both":"")+(this._shouldRenderInMargin()&&t?" current-line-exact-margin":"")}" style="width:${this._contentLeft}px"></div>`}_shouldRenderThis(){return!0}_shouldRenderOther(){return this._shouldRenderInContent()}}Lr((o,e)=>{const t=o.getColor(Bj);if(t&&(e.addRule(`.monaco-editor .view-overlays .current-line { background-color: ${t}; }`),e.addRule(`.monaco-editor .margin-view-overlays .current-line-margin { background-color: ${t}; border: none; }`)),!t||t.isTransparent()||o.defines(f8)){const i=o.getColor(f8);i&&(e.addRule(`.monaco-editor .view-overlays .current-line-exact { border: 2px solid ${i}; }`),e.addRule(`.monaco-editor .margin-view-overlays .current-line-exact-margin { border: 2px solid ${i}; }`),Ta(o.type)&&(e.addRule(".monaco-editor .view-overlays .current-line-exact { border-width: 1px; }"),e.addRule(".monaco-editor .margin-view-overlays .current-line-exact-margin { border-width: 1px; }")))}});class ebe extends lm{constructor(e){super(),this._context=e;const t=this._context.configuration.options;this._typicalHalfwidthCharacterWidth=t.get(50).typicalHalfwidthCharacterWidth,this._renderResult=null,this._context.addEventHandler(this)}dispose(){this._context.removeEventHandler(this),this._renderResult=null,super.dispose()}onConfigurationChanged(e){const t=this._context.configuration.options;return this._typicalHalfwidthCharacterWidth=t.get(50).typicalHalfwidthCharacterWidth,!0}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return e.scrollTopChanged||e.scrollWidthChanged}onZonesChanged(e){return!0}prepareRender(e){const t=e.getDecorationsInViewport();let i=[],n=0;for(let l=0,c=t.length;l<c;l++){const d=t[l];d.options.className&&(i[n++]=d)}i=i.sort((l,c)=>{if(l.options.zIndex<c.options.zIndex)return-1;if(l.options.zIndex>c.options.zIndex)return 1;const d=l.options.className,h=c.options.className;return d<h?-1:d>h?1:x.compareRangesUsingStarts(l.range,c.range)});const s=e.visibleRange.startLineNumber,r=e.visibleRange.endLineNumber,a=[];for(let l=s;l<=r;l++){const c=l-s;a[c]=""}this._renderWholeLineDecorations(e,i,a),this._renderNormalDecorations(e,i,a),this._renderResult=a}_renderWholeLineDecorations(e,t,i){const n=e.visibleRange.startLineNumber,s=e.visibleRange.endLineNumber;for(let r=0,a=t.length;r<a;r++){const l=t[r];if(!l.options.isWholeLine)continue;const c='<div class="cdr '+l.options.className+'" style="left:0;width:100%;"></div>',d=Math.max(l.range.startLineNumber,n),h=Math.min(l.range.endLineNumber,s);for(let u=d;u<=h;u++){const g=u-n;i[g]+=c}}}_renderNormalDecorations(e,t,i){const n=e.visibleRange.startLineNumber;let s=null,r=!1,a=null,l=!1;for(let c=0,d=t.length;c<d;c++){const h=t[c];if(h.options.isWholeLine)continue;const u=h.options.className,g=!!h.options.showIfCollapsed;let f=h.range;if(g&&f.endColumn===1&&f.endLineNumber!==f.startLineNumber&&(f=new x(f.startLineNumber,f.startColumn,f.endLineNumber-1,this._context.viewModel.getLineMaxColumn(f.endLineNumber-1))),s===u&&r===g&&x.areIntersectingOrTouching(a,f)){a=x.plusRange(a,f);continue}s!==null&&this._renderNormalDecoration(e,a,s,l,r,n,i),s=u,r=g,a=f,l=h.options.shouldFillLineOnLineBreak??!1}s!==null&&this._renderNormalDecoration(e,a,s,l,r,n,i)}_renderNormalDecoration(e,t,i,n,s,r,a){const l=e.linesVisibleRangesForRange(t,i==="findMatch");if(l)for(let c=0,d=l.length;c<d;c++){const h=l[c];if(h.outsideRenderedLine)continue;const u=h.lineNumber-r;if(s&&h.ranges.length===1){const g=h.ranges[0];if(g.width<this._typicalHalfwidthCharacterWidth){const f=Math.round(g.left+g.width/2),p=Math.max(0,Math.round(f-this._typicalHalfwidthCharacterWidth/2));h.ranges[0]=new _E(p,this._typicalHalfwidthCharacterWidth)}}for(let g=0,f=h.ranges.length;g<f;g++){const p=n&&h.continuesOnNextLine&&f===1,_=h.ranges[g],b='<div class="cdr '+i+'" style="left:'+String(_.left)+"px;width:"+(p?"100%;":String(_.width)+"px;")+'"></div>';a[u]+=b}}}render(e,t){if(!this._renderResult)return"";const i=t-e;return i<0||i>=this._renderResult.length?"":this._renderResult[i]}}class tbe extends bo{constructor(e,t,i,n){super(e);const s=this._context.configuration.options,r=s.get(104),a=s.get(75),l=s.get(40),c=s.get(107),d={listenOnDomNode:i.domNode,className:"editor-scrollable "+XM(e.theme.type),useShadows:!1,lazyRender:!0,vertical:r.vertical,horizontal:r.horizontal,verticalHasArrows:r.verticalHasArrows,horizontalHasArrows:r.horizontalHasArrows,verticalScrollbarSize:r.verticalScrollbarSize,verticalSliderSize:r.verticalSliderSize,horizontalScrollbarSize:r.horizontalScrollbarSize,horizontalSliderSize:r.horizontalSliderSize,handleMouseWheel:r.handleMouseWheel,alwaysConsumeMouseWheel:r.alwaysConsumeMouseWheel,arrowSize:r.arrowSize,mouseWheelScrollSensitivity:a,fastScrollSensitivity:l,scrollPredominantAxis:c,scrollByPage:r.scrollByPage};this.scrollbar=this._register(new jI(t.domNode,d,this._context.viewLayout.getScrollable())),Va.write(this.scrollbar.getDomNode(),6),this.scrollbarDomNode=St(this.scrollbar.getDomNode()),this.scrollbarDomNode.setPosition("absolute"),this._setLayout();const h=(u,g,f)=>{const p={};{const _=u.scrollTop;_&&(p.scrollTop=this._context.viewLayout.getCurrentScrollTop()+_,u.scrollTop=0)}if(f){const _=u.scrollLeft;_&&(p.scrollLeft=this._context.viewLayout.getCurrentScrollLeft()+_,u.scrollLeft=0)}this._context.viewModel.viewLayout.setScrollPosition(p,1)};this._register(U(i.domNode,"scroll",u=>h(i.domNode,!0,!0))),this._register(U(t.domNode,"scroll",u=>h(t.domNode,!0,!1))),this._register(U(n.domNode,"scroll",u=>h(n.domNode,!0,!1))),this._register(U(this.scrollbarDomNode.domNode,"scroll",u=>h(this.scrollbarDomNode.domNode,!0,!1)))}dispose(){super.dispose()}_setLayout(){const e=this._context.configuration.options,t=e.get(146);this.scrollbarDomNode.setLeft(t.contentLeft),e.get(73).side==="right"?this.scrollbarDomNode.setWidth(t.contentWidth+t.minimap.minimapWidth):this.scrollbarDomNode.setWidth(t.contentWidth),this.scrollbarDomNode.setHeight(t.height)}getOverviewRulerLayoutInfo(){return this.scrollbar.getOverviewRulerLayoutInfo()}getDomNode(){return this.scrollbarDomNode}delegateVerticalScrollbarPointerDown(e){this.scrollbar.delegateVerticalScrollbarPointerDown(e)}delegateScrollFromMouseWheelEvent(e){this.scrollbar.delegateScrollFromMouseWheelEvent(e)}onConfigurationChanged(e){if(e.hasChanged(104)||e.hasChanged(75)||e.hasChanged(40)){const t=this._context.configuration.options,i=t.get(104),n=t.get(75),s=t.get(40),r=t.get(107),a={vertical:i.vertical,horizontal:i.horizontal,verticalScrollbarSize:i.verticalScrollbarSize,horizontalScrollbarSize:i.horizontalScrollbarSize,scrollByPage:i.scrollByPage,handleMouseWheel:i.handleMouseWheel,mouseWheelScrollSensitivity:n,fastScrollSensitivity:s,scrollPredominantAxis:r};this.scrollbar.updateOptions(a)}return e.hasChanged(146)&&this._setLayout(),!0}onScrollChanged(e){return!0}onThemeChanged(e){return this.scrollbar.updateClassName("editor-scrollable "+XM(this._context.theme.type)),!0}prepareRender(e){}render(e){this.scrollbar.renderNow()}}class mP{constructor(e,t,i,n,s){this.startLineNumber=e,this.endLineNumber=t,this.className=i,this.tooltip=n,this._decorationToRenderBrand=void 0,this.zIndex=s??0}}class ibe{constructor(e,t,i){this.className=e,this.zIndex=t,this.tooltip=i}}class nbe{constructor(){this.decorations=[]}add(e){this.decorations.push(e)}getDecorations(){return this.decorations}}class Iq extends lm{_render(e,t,i){const n=[];for(let a=e;a<=t;a++){const l=a-e;n[l]=new nbe}if(i.length===0)return n;i.sort((a,l)=>a.className===l.className?a.startLineNumber===l.startLineNumber?a.endLineNumber-l.endLineNumber:a.startLineNumber-l.startLineNumber:a.className<l.className?-1:1);let s=null,r=0;for(let a=0,l=i.length;a<l;a++){const c=i[a],d=c.className,h=c.zIndex;let u=Math.max(c.startLineNumber,e)-e;const g=Math.min(c.endLineNumber,t)-e;s===d?(u=Math.max(r+1,u),r=Math.max(r,g)):(s=d,r=g);for(let f=u;f<=r;f++)n[f].add(new ibe(d,h,c.tooltip))}return n}}class sbe extends bo{constructor(e){super(e),this._widgets={},this._context=e;const t=this._context.configuration.options,i=t.get(146);this.domNode=St(document.createElement("div")),this.domNode.setClassName("glyph-margin-widgets"),this.domNode.setPosition("absolute"),this.domNode.setTop(0),this._lineHeight=t.get(67),this._glyphMargin=t.get(57),this._glyphMarginLeft=i.glyphMarginLeft,this._glyphMarginWidth=i.glyphMarginWidth,this._glyphMarginDecorationLaneCount=i.glyphMarginDecorationLaneCount,this._managedDomNodes=[],this._decorationGlyphsToRender=[]}dispose(){this._managedDomNodes=[],this._decorationGlyphsToRender=[],this._widgets={},super.dispose()}getWidgets(){return Object.values(this._widgets)}onConfigurationChanged(e){const t=this._context.configuration.options,i=t.get(146);return this._lineHeight=t.get(67),this._glyphMargin=t.get(57),this._glyphMarginLeft=i.glyphMarginLeft,this._glyphMarginWidth=i.glyphMarginWidth,this._glyphMarginDecorationLaneCount=i.glyphMarginDecorationLaneCount,!0}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return e.scrollTopChanged}onZonesChanged(e){return!0}addWidget(e){const t=St(e.getDomNode());this._widgets[e.getId()]={widget:e,preference:e.getPosition(),domNode:t,renderInfo:null},t.setPosition("absolute"),t.setDisplay("none"),t.setAttribute("widgetId",e.getId()),this.domNode.appendChild(t),this.setShouldRender()}setWidgetPosition(e,t){const i=this._widgets[e.getId()];return i.preference.lane===t.lane&&i.preference.zIndex===t.zIndex&&x.equalsRange(i.preference.range,t.range)?!1:(i.preference=t,this.setShouldRender(),!0)}removeWidget(e){const t=e.getId();if(this._widgets[t]){const n=this._widgets[t].domNode.domNode;delete this._widgets[t],n.remove(),this.setShouldRender()}}_collectDecorationBasedGlyphRenderRequest(e,t){var r;const i=e.visibleRange.startLineNumber,n=e.visibleRange.endLineNumber,s=e.getDecorationsInViewport();for(const a of s){const l=a.options.glyphMarginClassName;if(!l)continue;const c=Math.max(a.range.startLineNumber,i),d=Math.min(a.range.endLineNumber,n),h=((r=a.options.glyphMargin)==null?void 0:r.position)??Zr.Center,u=a.options.zIndex??0;for(let g=c;g<=d;g++){const f=this._context.viewModel.coordinatesConverter.convertViewPositionToModelPosition(new A(g,0)),p=this._context.viewModel.glyphLanes.getLanesAtLine(f.lineNumber).indexOf(h);t.push(new obe(g,p,u,l))}}}_collectWidgetBasedGlyphRenderRequest(e,t){const i=e.visibleRange.startLineNumber,n=e.visibleRange.endLineNumber;for(const s of Object.values(this._widgets)){const r=s.preference.range,{startLineNumber:a,endLineNumber:l}=this._context.viewModel.coordinatesConverter.convertModelRangeToViewRange(x.lift(r));if(!a||!l||l<i||a>n)continue;const c=Math.max(a,i),d=this._context.viewModel.coordinatesConverter.convertViewPositionToModelPosition(new A(c,0)),h=this._context.viewModel.glyphLanes.getLanesAtLine(d.lineNumber).indexOf(s.preference.lane);t.push(new rbe(c,h,s.preference.zIndex,s))}}_collectSortedGlyphRenderRequests(e){const t=[];return this._collectDecorationBasedGlyphRenderRequest(e,t),this._collectWidgetBasedGlyphRenderRequest(e,t),t.sort((i,n)=>i.lineNumber===n.lineNumber?i.laneIndex===n.laneIndex?i.zIndex===n.zIndex?n.type===i.type?i.type===0&&n.type===0?i.className<n.className?-1:1:0:n.type-i.type:n.zIndex-i.zIndex:i.laneIndex-n.laneIndex:i.lineNumber-n.lineNumber),t}prepareRender(e){if(!this._glyphMargin){this._decorationGlyphsToRender=[];return}for(const n of Object.values(this._widgets))n.renderInfo=null;const t=new Qc(this._collectSortedGlyphRenderRequests(e)),i=[];for(;t.length>0;){const n=t.peek();if(!n)break;const s=t.takeWhile(a=>a.lineNumber===n.lineNumber&&a.laneIndex===n.laneIndex);if(!s||s.length===0)break;const r=s[0];if(r.type===0){const a=[];for(const l of s){if(l.zIndex!==r.zIndex||l.type!==r.type)break;(a.length===0||a[a.length-1]!==l.className)&&a.push(l.className)}i.push(r.accept(a.join(" ")))}else r.widget.renderInfo={lineNumber:r.lineNumber,laneIndex:r.laneIndex}}this._decorationGlyphsToRender=i}render(e){if(!this._glyphMargin){for(const i of Object.values(this._widgets))i.domNode.setDisplay("none");for(;this._managedDomNodes.length>0;){const i=this._managedDomNodes.pop();i==null||i.domNode.remove()}return}const t=Math.round(this._glyphMarginWidth/this._glyphMarginDecorationLaneCount);for(const i of Object.values(this._widgets))if(!i.renderInfo)i.domNode.setDisplay("none");else{const n=e.viewportData.relativeVerticalOffset[i.renderInfo.lineNumber-e.viewportData.startLineNumber],s=this._glyphMarginLeft+i.renderInfo.laneIndex*this._lineHeight;i.domNode.setDisplay("block"),i.domNode.setTop(n),i.domNode.setLeft(s),i.domNode.setWidth(t),i.domNode.setHeight(this._lineHeight)}for(let i=0;i<this._decorationGlyphsToRender.length;i++){const n=this._decorationGlyphsToRender[i],s=e.viewportData.relativeVerticalOffset[n.lineNumber-e.viewportData.startLineNumber],r=this._glyphMarginLeft+n.laneIndex*this._lineHeight;let a;i<this._managedDomNodes.length?a=this._managedDomNodes[i]:(a=St(document.createElement("div")),this._managedDomNodes.push(a),this.domNode.appendChild(a)),a.setClassName("cgmr codicon "+n.combinedClassName),a.setPosition("absolute"),a.setTop(s),a.setLeft(r),a.setWidth(t),a.setHeight(this._lineHeight)}for(;this._managedDomNodes.length>this._decorationGlyphsToRender.length;){const i=this._managedDomNodes.pop();i==null||i.domNode.remove()}}}class obe{constructor(e,t,i,n){this.lineNumber=e,this.laneIndex=t,this.zIndex=i,this.className=n,this.type=0}accept(e){return new abe(this.lineNumber,this.laneIndex,e)}}class rbe{constructor(e,t,i,n){this.lineNumber=e,this.laneIndex=t,this.zIndex=i,this.widget=n,this.type=1}}class abe{constructor(e,t,i){this.lineNumber=e,this.laneIndex=t,this.combinedClassName=i}}class lbe extends lm{constructor(e){super(),this._context=e,this._primaryPosition=null;const t=this._context.configuration.options,i=t.get(147),n=t.get(50);this._spaceWidth=n.spaceWidth,this._maxIndentLeft=i.wrappingColumn===-1?-1:i.wrappingColumn*n.typicalHalfwidthCharacterWidth,this._bracketPairGuideOptions=t.get(16),this._renderResult=null,this._context.addEventHandler(this)}dispose(){this._context.removeEventHandler(this),this._renderResult=null,super.dispose()}onConfigurationChanged(e){const t=this._context.configuration.options,i=t.get(147),n=t.get(50);return this._spaceWidth=n.spaceWidth,this._maxIndentLeft=i.wrappingColumn===-1?-1:i.wrappingColumn*n.typicalHalfwidthCharacterWidth,this._bracketPairGuideOptions=t.get(16),!0}onCursorStateChanged(e){var n;const i=e.selections[0].getPosition();return(n=this._primaryPosition)!=null&&n.equals(i)?!1:(this._primaryPosition=i,!0)}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return e.scrollTopChanged}onZonesChanged(e){return!0}onLanguageConfigurationChanged(e){return!0}prepareRender(e){var l,c;if(!this._bracketPairGuideOptions.indentation&&this._bracketPairGuideOptions.bracketPairs===!1){this._renderResult=null;return}const t=e.visibleRange.startLineNumber,i=e.visibleRange.endLineNumber,n=e.scrollWidth,s=this._primaryPosition,r=this.getGuidesByLine(t,Math.min(i+1,this._context.viewModel.getLineCount()),s),a=[];for(let d=t;d<=i;d++){const h=d-t,u=r[h];let g="";const f=((l=e.visibleRangeForPosition(new A(d,1)))==null?void 0:l.left)??0;for(const p of u){const _=p.column===-1?f+(p.visibleColumn-1)*this._spaceWidth:e.visibleRangeForPosition(new A(d,p.column)).left;if(_>n||this._maxIndentLeft>0&&_>this._maxIndentLeft)break;const b=p.horizontalLine?p.horizontalLine.top?"horizontal-top":"horizontal-bottom":"vertical",C=p.horizontalLine?(((c=e.visibleRangeForPosition(new A(d,p.horizontalLine.endColumn)))==null?void 0:c.left)??_+this._spaceWidth)-_:this._spaceWidth;g+=`<div class="core-guide ${p.className} ${b}" style="left:${_}px;width:${C}px"></div>`}a[h]=g}this._renderResult=a}getGuidesByLine(e,t,i){const n=this._bracketPairGuideOptions.bracketPairs!==!1?this._context.viewModel.getBracketGuidesInRangeByLine(e,t,i,{highlightActive:this._bracketPairGuideOptions.highlightActiveBracketPair,horizontalGuides:this._bracketPairGuideOptions.bracketPairsHorizontal===!0?Cg.Enabled:this._bracketPairGuideOptions.bracketPairsHorizontal==="active"?Cg.EnabledForActive:Cg.Disabled,includeInactive:this._bracketPairGuideOptions.bracketPairs===!0}):null,s=this._bracketPairGuideOptions.indentation?this._context.viewModel.getLinesIndentGuides(e,t):null;let r=0,a=0,l=0;if(this._bracketPairGuideOptions.highlightActiveIndentation!==!1&&i){const h=this._context.viewModel.getActiveIndentGuide(i.lineNumber,e,t);r=h.startLineNumber,a=h.endLineNumber,l=h.indent}const{indentSize:c}=this._context.viewModel.model.getOptions(),d=[];for(let h=e;h<=t;h++){const u=new Array;d.push(u);const g=n?n[h-e]:[],f=new Qc(g),p=s?s[h-e]:0;for(let _=1;_<=p;_++){const b=(_-1)*c+1,C=(this._bracketPairGuideOptions.highlightActiveIndentation==="always"||g.length===0)&&r<=h&&h<=a&&_===l;u.push(...f.takeWhile(w=>w.visibleColumn<b)||[]);const v=f.peek();(!v||v.visibleColumn!==b||v.horizontalLine)&&u.push(new cg(b,-1,`core-guide-indent lvl-${(_-1)%30}`+(C?" indent-active":""),null,-1,-1))}u.push(...f.takeWhile(_=>!0)||[])}return d}render(e,t){if(!this._renderResult)return"";const i=t-e;return i<0||i>=this._renderResult.length?"":this._renderResult[i]}}function Tm(o){if(!(o&&o.isTransparent()))return o}Lr((o,e)=>{const t=[{bracketColor:Uj,guideColor:Dhe,guideColorActive:Mhe},{bracketColor:$j,guideColor:Ihe,guideColorActive:Ahe},{bracketColor:jj,guideColor:Ehe,guideColorActive:Phe},{bracketColor:Kj,guideColor:Nhe,guideColorActive:Ohe},{bracketColor:qj,guideColor:The,guideColorActive:Fhe},{bracketColor:Gj,guideColor:Rhe,guideColorActive:Whe}],i=new oK,n=[{indentColor:J1,indentColorActive:eS},{indentColor:rhe,indentColorActive:hhe},{indentColor:ahe,indentColorActive:uhe},{indentColor:lhe,indentColorActive:ghe},{indentColor:che,indentColorActive:fhe},{indentColor:dhe,indentColorActive:mhe}],s=t.map(a=>{const l=o.getColor(a.bracketColor),c=o.getColor(a.guideColor),d=o.getColor(a.guideColorActive),h=Tm(Tm(c)??(l==null?void 0:l.transparent(.3))),u=Tm(Tm(d)??l);if(!(!h||!u))return{guideColor:h,guideColorActive:u}}).filter(Tl),r=n.map(a=>{const l=o.getColor(a.indentColor),c=o.getColor(a.indentColorActive),d=Tm(l),h=Tm(c);if(!(!d||!h))return{indentColor:d,indentColorActive:h}}).filter(Tl);if(s.length>0){for(let a=0;a<30;a++){const l=s[a%s.length];e.addRule(`.monaco-editor .${i.getInlineClassNameOfLevel(a).replace(/ /g,".")} { --guide-color: ${l.guideColor}; --guide-color-active: ${l.guideColorActive}; }`)}e.addRule(".monaco-editor .vertical { box-shadow: 1px 0 0 0 var(--guide-color) inset; }"),e.addRule(".monaco-editor .horizontal-top { border-top: 1px solid var(--guide-color); }"),e.addRule(".monaco-editor .horizontal-bottom { border-bottom: 1px solid var(--guide-color); }"),e.addRule(`.monaco-editor .vertical.${i.activeClassName} { box-shadow: 1px 0 0 0 var(--guide-color-active) inset; }`),e.addRule(`.monaco-editor .horizontal-top.${i.activeClassName} { border-top: 1px solid var(--guide-color-active); }`),e.addRule(`.monaco-editor .horizontal-bottom.${i.activeClassName} { border-bottom: 1px solid var(--guide-color-active); }`)}if(r.length>0){for(let a=0;a<30;a++){const l=r[a%r.length];e.addRule(`.monaco-editor .lines-content .core-guide-indent.lvl-${a} { --indent-color: ${l.indentColor}; --indent-color-active: ${l.indentColorActive}; }`)}e.addRule(".monaco-editor .lines-content .core-guide-indent { box-shadow: 1px 0 0 0 var(--indent-color) inset; }"),e.addRule(".monaco-editor .lines-content .core-guide-indent.indent-active { box-shadow: 1px 0 0 0 var(--indent-color-active) inset; }")}});class wT{get didDomLayout(){return this._didDomLayout}readClientRect(){if(!this._clientRectRead){this._clientRectRead=!0;const e=this._domNode.getBoundingClientRect();this.markDidDomLayout(),this._clientRectDeltaLeft=e.left,this._clientRectScale=e.width/this._domNode.offsetWidth}}get clientRectDeltaLeft(){return this._clientRectRead||this.readClientRect(),this._clientRectDeltaLeft}get clientRectScale(){return this._clientRectRead||this.readClientRect(),this._clientRectScale}constructor(e,t){this._domNode=e,this.endNode=t,this._didDomLayout=!1,this._clientRectDeltaLeft=0,this._clientRectScale=1,this._clientRectRead=!1}markDidDomLayout(){this._didDomLayout=!0}}class cbe{constructor(){this._currentVisibleRange=new x(1,1,1,1)}getCurrentVisibleRange(){return this._currentVisibleRange}setCurrentVisibleRange(e){this._currentVisibleRange=e}}class dbe{constructor(e,t,i,n,s,r,a){this.minimalReveal=e,this.lineNumber=t,this.startColumn=i,this.endColumn=n,this.startScrollTop=s,this.stopScrollTop=r,this.scrollType=a,this.type="range",this.minLineNumber=t,this.maxLineNumber=t}}class hbe{constructor(e,t,i,n,s){this.minimalReveal=e,this.selections=t,this.startScrollTop=i,this.stopScrollTop=n,this.scrollType=s,this.type="selections";let r=t[0].startLineNumber,a=t[0].endLineNumber;for(let l=1,c=t.length;l<c;l++){const d=t[l];r=Math.min(r,d.startLineNumber),a=Math.max(a,d.endLineNumber)}this.minLineNumber=r,this.maxLineNumber=a}}const dD=class dD extends bo{constructor(e,t){super(e);const i=this._context.configuration,n=this._context.configuration.options,s=n.get(50),r=n.get(147);this._lineHeight=n.get(67),this._typicalHalfwidthCharacterWidth=s.typicalHalfwidthCharacterWidth,this._isViewportWrapping=r.isViewportWrapping,this._revealHorizontalRightPadding=n.get(101),this._cursorSurroundingLines=n.get(29),this._cursorSurroundingLinesStyle=n.get(30),this._canUseLayerHinting=!n.get(32),this._viewLineOptions=new CH(i,this._context.theme.type),this._linesContent=t,this._textRangeRestingSpot=document.createElement("div"),this._visibleLines=new xq({createLine:()=>new Ec(this._viewLineOptions)}),this.domNode=this._visibleLines.domNode,Va.write(this.domNode,8),this.domNode.setClassName(`view-lines ${s_}`),bn(this.domNode,s),this._maxLineWidth=0,this._asyncUpdateLineWidths=new It(()=>{this._updateLineWidthsSlow()},200),this._asyncCheckMonospaceFontAssumptions=new It(()=>{this._checkMonospaceFontAssumptions()},2e3),this._lastRenderedData=new cbe,this._horizontalRevealRequest=null,this._stickyScrollEnabled=n.get(116).enabled,this._maxNumberStickyLines=n.get(116).maxLineCount}dispose(){this._asyncUpdateLineWidths.dispose(),this._asyncCheckMonospaceFontAssumptions.dispose(),super.dispose()}getDomNode(){return this.domNode}onConfigurationChanged(e){this._visibleLines.onConfigurationChanged(e),e.hasChanged(147)&&(this._maxLineWidth=0);const t=this._context.configuration.options,i=t.get(50),n=t.get(147);return this._lineHeight=t.get(67),this._typicalHalfwidthCharacterWidth=i.typicalHalfwidthCharacterWidth,this._isViewportWrapping=n.isViewportWrapping,this._revealHorizontalRightPadding=t.get(101),this._cursorSurroundingLines=t.get(29),this._cursorSurroundingLinesStyle=t.get(30),this._canUseLayerHinting=!t.get(32),this._stickyScrollEnabled=t.get(116).enabled,this._maxNumberStickyLines=t.get(116).maxLineCount,bn(this.domNode,i),this._onOptionsMaybeChanged(),e.hasChanged(146)&&(this._maxLineWidth=0),!0}_onOptionsMaybeChanged(){const e=this._context.configuration,t=new CH(e,this._context.theme.type);if(!this._viewLineOptions.equals(t)){this._viewLineOptions=t;const i=this._visibleLines.getStartLineNumber(),n=this._visibleLines.getEndLineNumber();for(let s=i;s<=n;s++)this._visibleLines.getVisibleLine(s).onOptionsChanged(this._viewLineOptions);return!0}return!1}onCursorStateChanged(e){const t=this._visibleLines.getStartLineNumber(),i=this._visibleLines.getEndLineNumber();let n=!1;for(let s=t;s<=i;s++)n=this._visibleLines.getVisibleLine(s).onSelectionChanged()||n;return n}onDecorationsChanged(e){{const t=this._visibleLines.getStartLineNumber(),i=this._visibleLines.getEndLineNumber();for(let n=t;n<=i;n++)this._visibleLines.getVisibleLine(n).onDecorationsChanged()}return!0}onFlushed(e){const t=this._visibleLines.onFlushed(e);return this._maxLineWidth=0,t}onLinesChanged(e){return this._visibleLines.onLinesChanged(e)}onLinesDeleted(e){return this._visibleLines.onLinesDeleted(e)}onLinesInserted(e){return this._visibleLines.onLinesInserted(e)}onRevealRangeRequest(e){const t=this._computeScrollTopToRevealRange(this._context.viewLayout.getFutureViewport(),e.source,e.minimalReveal,e.range,e.selections,e.verticalType);if(t===-1)return!1;let i=this._context.viewLayout.validateScrollPosition({scrollTop:t});e.revealHorizontal?e.range&&e.range.startLineNumber!==e.range.endLineNumber?i={scrollTop:i.scrollTop,scrollLeft:0}:e.range?this._horizontalRevealRequest=new dbe(e.minimalReveal,e.range.startLineNumber,e.range.startColumn,e.range.endColumn,this._context.viewLayout.getCurrentScrollTop(),i.scrollTop,e.scrollType):e.selections&&e.selections.length>0&&(this._horizontalRevealRequest=new hbe(e.minimalReveal,e.selections,this._context.viewLayout.getCurrentScrollTop(),i.scrollTop,e.scrollType)):this._horizontalRevealRequest=null;const s=Math.abs(this._context.viewLayout.getCurrentScrollTop()-i.scrollTop)<=this._lineHeight?1:e.scrollType;return this._context.viewModel.viewLayout.setScrollPosition(i,s),!0}onScrollChanged(e){if(this._horizontalRevealRequest&&e.scrollLeftChanged&&(this._horizontalRevealRequest=null),this._horizontalRevealRequest&&e.scrollTopChanged){const t=Math.min(this._horizontalRevealRequest.startScrollTop,this._horizontalRevealRequest.stopScrollTop),i=Math.max(this._horizontalRevealRequest.startScrollTop,this._horizontalRevealRequest.stopScrollTop);(e.scrollTop<t||e.scrollTop>i)&&(this._horizontalRevealRequest=null)}return this.domNode.setWidth(e.scrollWidth),this._visibleLines.onScrollChanged(e)||!0}onTokensChanged(e){return this._visibleLines.onTokensChanged(e)}onZonesChanged(e){return this._context.viewModel.viewLayout.setMaxLineWidth(this._maxLineWidth),this._visibleLines.onZonesChanged(e)}onThemeChanged(e){return this._onOptionsMaybeChanged()}getPositionFromDOMInfo(e,t){const i=this._getViewLineDomNode(e);if(i===null)return null;const n=this._getLineNumberFor(i);if(n===-1||n<1||n>this._context.viewModel.getLineCount())return null;if(this._context.viewModel.getLineMaxColumn(n)===1)return new A(n,1);const s=this._visibleLines.getStartLineNumber(),r=this._visibleLines.getEndLineNumber();if(n<s||n>r)return null;let a=this._visibleLines.getVisibleLine(n).getColumnOfNodeOffset(e,t);const l=this._context.viewModel.getLineMinColumn(n);return a<l&&(a=l),new A(n,a)}_getViewLineDomNode(e){for(;e&&e.nodeType===1;){if(e.className===Ec.CLASS_NAME)return e;e=e.parentElement}return null}_getLineNumberFor(e){const t=this._visibleLines.getStartLineNumber(),i=this._visibleLines.getEndLineNumber();for(let n=t;n<=i;n++){const s=this._visibleLines.getVisibleLine(n);if(e===s.getDomNode())return n}return-1}getLineWidth(e){const t=this._visibleLines.getStartLineNumber(),i=this._visibleLines.getEndLineNumber();if(e<t||e>i)return-1;const n=new wT(this.domNode.domNode,this._textRangeRestingSpot),s=this._visibleLines.getVisibleLine(e).getWidth(n);return this._updateLineWidthsSlowIfDomDidLayout(n),s}linesVisibleRangesForRange(e,t){if(this.shouldRender())return null;const i=e.endLineNumber,n=x.intersectRanges(e,this._lastRenderedData.getCurrentVisibleRange());if(!n)return null;const s=[];let r=0;const a=new wT(this.domNode.domNode,this._textRangeRestingSpot);let l=0;t&&(l=this._context.viewModel.coordinatesConverter.convertViewPositionToModelPosition(new A(n.startLineNumber,1)).lineNumber);const c=this._visibleLines.getStartLineNumber(),d=this._visibleLines.getEndLineNumber();for(let h=n.startLineNumber;h<=n.endLineNumber;h++){if(h<c||h>d)continue;const u=h===n.startLineNumber?n.startColumn:1,g=h!==n.endLineNumber,f=g?this._context.viewModel.getLineMaxColumn(h):n.endColumn,p=this._visibleLines.getVisibleLine(h).getVisibleRangesForRange(h,u,f,a);if(p){if(t&&h<i){const _=l;l=this._context.viewModel.coordinatesConverter.convertViewPositionToModelPosition(new A(h+1,1)).lineNumber,_!==l&&(p.ranges[p.ranges.length-1].width+=this._typicalHalfwidthCharacterWidth)}s[r++]=new Zpe(p.outsideRenderedLine,h,_E.from(p.ranges),g)}}return this._updateLineWidthsSlowIfDomDidLayout(a),r===0?null:s}_visibleRangesForLineRange(e,t,i){if(this.shouldRender()||e<this._visibleLines.getStartLineNumber()||e>this._visibleLines.getEndLineNumber())return null;const n=new wT(this.domNode.domNode,this._textRangeRestingSpot),s=this._visibleLines.getVisibleLine(e).getVisibleRangesForRange(e,t,i,n);return this._updateLineWidthsSlowIfDomDidLayout(n),s}visibleRangeForPosition(e){const t=this._visibleRangesForLineRange(e.lineNumber,e.column,e.column);return t?new Ype(t.outsideRenderedLine,t.ranges[0].left):null}_updateLineWidthsFast(){return this._updateLineWidths(!0)}_updateLineWidthsSlow(){this._updateLineWidths(!1)}_updateLineWidthsSlowIfDomDidLayout(e){e.didDomLayout&&(this._asyncUpdateLineWidths.isScheduled()||(this._asyncUpdateLineWidths.cancel(),this._updateLineWidthsSlow()))}_updateLineWidths(e){const t=this._visibleLines.getStartLineNumber(),i=this._visibleLines.getEndLineNumber();let n=1,s=!0;for(let r=t;r<=i;r++){const a=this._visibleLines.getVisibleLine(r);if(e&&!a.getWidthIsFast()){s=!1;continue}n=Math.max(n,a.getWidth(null))}return s&&t===1&&i===this._context.viewModel.getLineCount()&&(this._maxLineWidth=0),this._ensureMaxLineWidth(n),s}_checkMonospaceFontAssumptions(){let e=-1,t=-1;const i=this._visibleLines.getStartLineNumber(),n=this._visibleLines.getEndLineNumber();for(let s=i;s<=n;s++){const r=this._visibleLines.getVisibleLine(s);if(r.needsMonospaceFontCheck()){const a=r.getWidth(null);a>t&&(t=a,e=s)}}if(e!==-1&&!this._visibleLines.getVisibleLine(e).monospaceAssumptionsAreValid())for(let s=i;s<=n;s++)this._visibleLines.getVisibleLine(s).onMonospaceAssumptionsInvalidated()}prepareRender(){throw new Error("Not supported")}render(){throw new Error("Not supported")}renderText(e){if(this._visibleLines.renderLines(e),this._lastRenderedData.setCurrentVisibleRange(e.visibleRange),this.domNode.setWidth(this._context.viewLayout.getScrollWidth()),this.domNode.setHeight(Math.min(this._context.viewLayout.getScrollHeight(),1e6)),this._horizontalRevealRequest){const i=this._horizontalRevealRequest;if(e.startLineNumber<=i.minLineNumber&&i.maxLineNumber<=e.endLineNumber){this._horizontalRevealRequest=null,this.onDidRender();const n=this._computeScrollLeftToReveal(i);n&&(this._isViewportWrapping||this._ensureMaxLineWidth(n.maxHorizontalOffset),this._context.viewModel.viewLayout.setScrollPosition({scrollLeft:n.scrollLeft},i.scrollType))}}if(this._updateLineWidthsFast()?this._asyncUpdateLineWidths.cancel():this._asyncUpdateLineWidths.schedule(),ps&&!this._asyncCheckMonospaceFontAssumptions.isScheduled()){const i=this._visibleLines.getStartLineNumber(),n=this._visibleLines.getEndLineNumber();for(let s=i;s<=n;s++)if(this._visibleLines.getVisibleLine(s).needsMonospaceFontCheck()){this._asyncCheckMonospaceFontAssumptions.schedule();break}}this._linesContent.setLayerHinting(this._canUseLayerHinting),this._linesContent.setContain("strict");const t=this._context.viewLayout.getCurrentScrollTop()-e.bigNumbersDelta;this._linesContent.setTop(-t),this._linesContent.setLeft(-this._context.viewLayout.getCurrentScrollLeft())}_ensureMaxLineWidth(e){const t=Math.ceil(e);this._maxLineWidth<t&&(this._maxLineWidth=t,this._context.viewModel.viewLayout.setMaxLineWidth(this._maxLineWidth))}_computeScrollTopToRevealRange(e,t,i,n,s,r){const a=e.top,l=e.height,c=a+l;let d,h,u;if(s&&s.length>0){let b=s[0].startLineNumber,C=s[0].endLineNumber;for(let v=1,w=s.length;v<w;v++){const S=s[v];b=Math.min(b,S.startLineNumber),C=Math.max(C,S.endLineNumber)}d=!1,h=this._context.viewLayout.getVerticalOffsetForLineNumber(b),u=this._context.viewLayout.getVerticalOffsetForLineNumber(C)+this._lineHeight}else if(n)d=!0,h=this._context.viewLayout.getVerticalOffsetForLineNumber(n.startLineNumber),u=this._context.viewLayout.getVerticalOffsetForLineNumber(n.endLineNumber)+this._lineHeight;else return-1;const g=(t==="mouse"||i)&&this._cursorSurroundingLinesStyle==="default";let f=0,p=0;if(g)i||(f=this._lineHeight);else{const b=l/this._lineHeight,C=Math.max(this._cursorSurroundingLines,this._stickyScrollEnabled?this._maxNumberStickyLines:0),v=Math.min(b/2,C);f=v*this._lineHeight,p=Math.max(0,v-1)*this._lineHeight}i||(r===0||r===4)&&(p+=this._lineHeight),h-=f,u+=p;let _;if(u-h>l){if(!d)return-1;_=h}else if(r===5||r===6)if(r===6&&a<=h&&u<=c)_=a;else{const b=Math.max(5*this._lineHeight,l*.2),C=h-b,v=u-l;_=Math.max(v,C)}else if(r===1||r===2)if(r===2&&a<=h&&u<=c)_=a;else{const b=(h+u)/2;_=Math.max(0,b-l/2)}else _=this._computeMinimumScrolling(a,c,h,u,r===3,r===4);return _}_computeScrollLeftToReveal(e){const t=this._context.viewLayout.getCurrentViewport(),i=this._context.configuration.options.get(146),n=t.left,s=n+t.width-i.verticalScrollbarWidth;let r=1073741824,a=0;if(e.type==="range"){const c=this._visibleRangesForLineRange(e.lineNumber,e.startColumn,e.endColumn);if(!c)return null;for(const d of c.ranges)r=Math.min(r,Math.round(d.left)),a=Math.max(a,Math.round(d.left+d.width))}else for(const c of e.selections){if(c.startLineNumber!==c.endLineNumber)return null;const d=this._visibleRangesForLineRange(c.startLineNumber,c.startColumn,c.endColumn);if(!d)return null;for(const h of d.ranges)r=Math.min(r,Math.round(h.left)),a=Math.max(a,Math.round(h.left+h.width))}return e.minimalReveal||(r=Math.max(0,r-dD.HORIZONTAL_EXTRA_PX),a+=this._revealHorizontalRightPadding),e.type==="selections"&&a-r>t.width?null:{scrollLeft:this._computeMinimumScrolling(n,s,r,a),maxHorizontalOffset:a}}_computeMinimumScrolling(e,t,i,n,s,r){e=e|0,t=t|0,i=i|0,n=n|0,s=!!s,r=!!r;const a=t-e;if(n-i<a){if(s)return i;if(r)return Math.max(0,n-a);if(i<e)return i;if(n>t)return Math.max(0,n-a)}else return i;return e}};dD.HORIZONTAL_EXTRA_PX=30;let pP=dD;class ube extends Iq{constructor(e){super(),this._context=e;const i=this._context.configuration.options.get(146);this._decorationsLeft=i.decorationsLeft,this._decorationsWidth=i.decorationsWidth,this._renderResult=null,this._context.addEventHandler(this)}dispose(){this._context.removeEventHandler(this),this._renderResult=null,super.dispose()}onConfigurationChanged(e){const i=this._context.configuration.options.get(146);return this._decorationsLeft=i.decorationsLeft,this._decorationsWidth=i.decorationsWidth,!0}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return e.scrollTopChanged}onZonesChanged(e){return!0}_getDecorations(e){const t=e.getDecorationsInViewport(),i=[];let n=0;for(let s=0,r=t.length;s<r;s++){const a=t[s],l=a.options.linesDecorationsClassName,c=a.options.zIndex;l&&(i[n++]=new mP(a.range.startLineNumber,a.range.endLineNumber,l,a.options.linesDecorationsTooltip??null,c));const d=a.options.firstLineDecorationClassName;d&&(i[n++]=new mP(a.range.startLineNumber,a.range.startLineNumber,d,a.options.linesDecorationsTooltip??null,c))}return i}prepareRender(e){const t=e.visibleRange.startLineNumber,i=e.visibleRange.endLineNumber,n=this._render(t,i,this._getDecorations(e)),s=this._decorationsLeft.toString(),r=this._decorationsWidth.toString(),a='" style="left:'+s+"px;width:"+r+'px;"></div>',l=[];for(let c=t;c<=i;c++){const d=c-t,h=n[d].getDecorations();let u="";for(const g of h){let f='<div class="cldr '+g.className;g.tooltip!==null&&(f+='" title="'+g.tooltip),f+=a,u+=f}l[d]=u}this._renderResult=l}render(e,t){return this._renderResult?this._renderResult[t-e]:""}}class gbe extends Iq{constructor(e){super(),this._context=e,this._renderResult=null,this._context.addEventHandler(this)}dispose(){this._context.removeEventHandler(this),this._renderResult=null,super.dispose()}onConfigurationChanged(e){return!0}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return e.scrollTopChanged}onZonesChanged(e){return!0}_getDecorations(e){const t=e.getDecorationsInViewport(),i=[];let n=0;for(let s=0,r=t.length;s<r;s++){const a=t[s],l=a.options.marginClassName,c=a.options.zIndex;l&&(i[n++]=new mP(a.range.startLineNumber,a.range.endLineNumber,l,null,c))}return i}prepareRender(e){const t=e.visibleRange.startLineNumber,i=e.visibleRange.endLineNumber,n=this._render(t,i,this._getDecorations(e)),s=[];for(let r=t;r<=i;r++){const a=r-t,l=n[a].getDecorations();let c="";for(const d of l)c+='<div class="cmdr '+d.className+'" style=""></div>';s[a]=c}this._renderResult=s}render(e,t){return this._renderResult?this._renderResult[t-e]:""}}const Md=class Md{constructor(e,t,i,n){this._rgba8Brand=void 0,this.r=Md._clamp(e),this.g=Md._clamp(t),this.b=Md._clamp(i),this.a=Md._clamp(n)}equals(e){return this.r===e.r&&this.g===e.g&&this.b===e.b&&this.a===e.a}static _clamp(e){return e<0?0:e>255?255:e|0}};Md.Empty=new Md(0,0,0,0);let Gc=Md;const hD=class hD extends F{static getInstance(){return this._INSTANCE||(this._INSTANCE=new hD),this._INSTANCE}constructor(){super(),this._onDidChange=new P,this.onDidChange=this._onDidChange.event,this._updateColorMap(),this._register(fi.onDidChange(e=>{e.changedColorMap&&this._updateColorMap()}))}_updateColorMap(){const e=fi.getColorMap();if(!e){this._colors=[Gc.Empty],this._backgroundIsLight=!0;return}this._colors=[Gc.Empty];for(let i=1;i<e.length;i++){const n=e[i].rgba;this._colors[i]=new Gc(n.r,n.g,n.b,Math.round(n.a*255))}const t=e[2].getRelativeLuminance();this._backgroundIsLight=t>=.5,this._onDidChange.fire(void 0)}getColor(e){return(e<1||e>=this._colors.length)&&(e=2),this._colors[e]}backgroundIsLight(){return this._backgroundIsLight}};hD._INSTANCE=null;let hx=hD;const fbe=(()=>{const o=[];for(let e=32;e<=126;e++)o.push(e);return o.push(65533),o})(),mbe=(o,e)=>(o-=32,o<0||o>96?e<=2?(o+96)%96:95:o);class qw{constructor(e,t){this.scale=t,this._minimapCharRendererBrand=void 0,this.charDataNormal=qw.soften(e,12/15),this.charDataLight=qw.soften(e,50/60)}static soften(e,t){const i=new Uint8ClampedArray(e.length);for(let n=0,s=e.length;n<s;n++)i[n]=Xy(e[n]*t);return i}renderChar(e,t,i,n,s,r,a,l,c,d,h){const u=1*this.scale,g=2*this.scale,f=h?1:g;if(t+u>e.width||i+f>e.height){console.warn("bad render request outside image data");return}const p=d?this.charDataLight:this.charDataNormal,_=mbe(n,c),b=e.width*4,C=a.r,v=a.g,w=a.b,S=s.r-C,L=s.g-v,k=s.b-w,D=Math.max(r,l),N=e.data;let B=_*u*g,W=i*b+t*4;for(let V=0;V<f;V++){let $=W;for(let H=0;H<u;H++){const j=p[B++]/255*(r/255);N[$++]=C+S*j,N[$++]=v+L*j,N[$++]=w+k*j,N[$++]=D}W+=b}}blockRenderChar(e,t,i,n,s,r,a,l){const c=1*this.scale,d=2*this.scale,h=l?1:d;if(t+c>e.width||i+h>e.height){console.warn("bad render request outside image data");return}const u=e.width*4,g=.5*(s/255),f=r.r,p=r.g,_=r.b,b=n.r-f,C=n.g-p,v=n.b-_,w=f+b*g,S=p+C*g,L=_+v*g,k=Math.max(s,a),D=e.data;let N=i*u+t*4;for(let B=0;B<h;B++){let W=N;for(let V=0;V<c;V++)D[W++]=w,D[W++]=S,D[W++]=L,D[W++]=k;N+=u}}}const IH={0:0,1:1,2:2,3:3,4:4,5:5,6:6,7:7,8:8,9:9,A:10,B:11,C:12,D:13,E:14,F:15},EH=o=>{const e=new Uint8ClampedArray(o.length/2);for(let t=0;t<o.length;t+=2)e[t>>1]=IH[o[t]]<<4|IH[o[t+1]]&15;return e},NH={1:Ah(()=>EH("0000511D6300CF609C709645A78432005642574171487021003C451900274D35D762755E8B629C5BA856AF57BA649530C167D1512A272A3F6038604460398526BCA2A968DB6F8957C768BE5FBE2FB467CF5D8D5B795DC7625B5DFF50DE64C466DB2FC47CD860A65E9A2EB96CB54CE06DA763AB2EA26860524D3763536601005116008177A8705E53AB738E6A982F88BAA35B5F5B626D9C636B449B737E5B7B678598869A662F6B5B8542706C704C80736A607578685B70594A49715A4522E792")),2:Ah(()=>EH("000000000000000055394F383D2800008B8B1F210002000081B1CBCBCC820000847AAF6B9AAF2119BE08B8881AD60000A44FD07DCCF107015338130C00000000385972265F390B406E2437634B4B48031B12B8A0847000001E15B29A402F0000000000004B33460B00007A752C2A0000000000004D3900000084394B82013400ABA5CFC7AD9C0302A45A3E5A98AB000089A43382D97900008BA54AA087A70A0248A6A7AE6DBE0000BF6F94987EA40A01A06DCFA7A7A9030496C32F77891D0000A99FB1A0AFA80603B29AB9CA75930D010C0948354D3900000C0948354F37460D0028BE673D8400000000AF9D7B6E00002B007AA8933400007AA642675C2700007984CFB9C3985B768772A8A6B7B20000CAAECAAFC4B700009F94A6009F840009D09F9BA4CA9C0000CC8FC76DC87F0000C991C472A2000000A894A48CA7B501079BA2C9C69BA20000B19A5D3FA89000005CA6009DA2960901B0A7F0669FB200009D009E00B7890000DAD0F5D092820000D294D4C48BD10000B5A7A4A3B1A50402CAB6CBA6A2000000B5A7A4A3B1A8044FCDADD19D9CB00000B7778F7B8AAE0803C9AB5D3F5D3F00009EA09EA0BAB006039EA0989A8C7900009B9EF4D6B7C00000A9A7816CACA80000ABAC84705D3F000096DA635CDC8C00006F486F266F263D4784006124097B00374F6D2D6D2D6D4A3A95872322000000030000000000008D8939130000000000002E22A5C9CBC70600AB25C0B5C9B400061A2DB04CA67001082AA6BEBEBFC606002321DACBC19E03087AA08B6768380000282FBAC0B8CA7A88AD25BBA5A29900004C396C5894A6000040485A6E356E9442A32CD17EADA70000B4237923628600003E2DE9C1D7B500002F25BBA5A2990000231DB6AFB4A804023025C0B5CAB588062B2CBDBEC0C706882435A75CA20000002326BD6A82A908048B4B9A5A668000002423A09CB4BB060025259C9D8A7900001C1FCAB2C7C700002A2A9387ABA200002626A4A47D6E9D14333163A0C87500004B6F9C2D643A257049364936493647358A34438355497F1A0000A24C1D590000D38DFFBDD4CD3126"))};class uv{static create(e,t){if(this.lastCreated&&e===this.lastCreated.scale&&t===this.lastFontFamily)return this.lastCreated;let i;return NH[e]?i=new qw(NH[e](),e):i=uv.createFromSampleData(uv.createSampleData(t).data,e),this.lastFontFamily=t,this.lastCreated=i,i}static createSampleData(e){const t=document.createElement("canvas"),i=t.getContext("2d");t.style.height="16px",t.height=16,t.width=96*10,t.style.width=96*10+"px",i.fillStyle="#ffffff",i.font=`bold 16px ${e}`,i.textBaseline="middle";let n=0;for(const s of fbe)i.fillText(String.fromCharCode(s),n,16/2),n+=10;return i.getImageData(0,0,96*10,16)}static createFromSampleData(e,t){if(e.length!==61440)throw new Error("Unexpected source in MinimapCharRenderer");const n=uv._downsample(e,t);return new qw(n,t)}static _downsampleChar(e,t,i,n,s){const r=1*s,a=2*s;let l=n,c=0;for(let d=0;d<a;d++){const h=d/a*16,u=(d+1)/a*16;for(let g=0;g<r;g++){const f=g/r*10,p=(g+1)/r*10;let _=0,b=0;for(let v=h;v<u;v++){const w=t+Math.floor(v)*3840,S=1-(v-Math.floor(v));for(let L=f;L<p;L++){const k=1-(L-Math.floor(L)),D=w+Math.floor(L)*4,N=k*S;b+=N,_+=e[D]*e[D+3]/255*N}}const C=_/b;c=Math.max(c,C),i[l++]=Xy(C)}}return c}static _downsample(e,t){const i=2*t*1*t,n=i*96,s=new Uint8ClampedArray(n);let r=0,a=0,l=0;for(let c=0;c<96;c++)l=Math.max(l,this._downsampleChar(e,a,s,r,t)),r+=i,a+=10*4;if(l>0){const c=255/l;for(let d=0;d<n;d++)s[d]*=c}return s}}const Eq=En?'"Segoe WPC", "Segoe UI", sans-serif':Je?"-apple-system, BlinkMacSystemFont, sans-serif":'system-ui, "Ubuntu", "Droid Sans", sans-serif',pbe=140,_be=2;class a_{constructor(e,t,i){const n=e.options,s=n.get(144),r=n.get(146),a=r.minimap,l=n.get(50),c=n.get(73);this.renderMinimap=a.renderMinimap,this.size=c.size,this.minimapHeightIsEditorHeight=a.minimapHeightIsEditorHeight,this.scrollBeyondLastLine=n.get(106),this.paddingTop=n.get(84).top,this.paddingBottom=n.get(84).bottom,this.showSlider=c.showSlider,this.autohide=c.autohide,this.pixelRatio=s,this.typicalHalfwidthCharacterWidth=l.typicalHalfwidthCharacterWidth,this.lineHeight=n.get(67),this.minimapLeft=a.minimapLeft,this.minimapWidth=a.minimapWidth,this.minimapHeight=r.height,this.canvasInnerWidth=a.minimapCanvasInnerWidth,this.canvasInnerHeight=a.minimapCanvasInnerHeight,this.canvasOuterWidth=a.minimapCanvasOuterWidth,this.canvasOuterHeight=a.minimapCanvasOuterHeight,this.isSampling=a.minimapIsSampling,this.editorHeight=r.height,this.fontScale=a.minimapScale,this.minimapLineHeight=a.minimapLineHeight,this.minimapCharWidth=1*this.fontScale,this.sectionHeaderFontFamily=Eq,this.sectionHeaderFontSize=c.sectionHeaderFontSize*s,this.sectionHeaderLetterSpacing=c.sectionHeaderLetterSpacing,this.sectionHeaderFontColor=a_._getSectionHeaderColor(t,i.getColor(1)),this.charRenderer=Ah(()=>uv.create(this.fontScale,l.fontFamily)),this.defaultBackgroundColor=i.getColor(2),this.backgroundColor=a_._getMinimapBackground(t,this.defaultBackgroundColor),this.foregroundAlpha=a_._getMinimapForegroundOpacity(t)}static _getMinimapBackground(e,t){const i=e.getColor(are);return i?new Gc(i.rgba.r,i.rgba.g,i.rgba.b,Math.round(255*i.rgba.a)):t}static _getMinimapForegroundOpacity(e){const t=e.getColor(lre);return t?Gc._clamp(Math.round(255*t.rgba.a)):255}static _getSectionHeaderColor(e,t){const i=e.getColor(Zl);return i?new Gc(i.rgba.r,i.rgba.g,i.rgba.b,Math.round(255*i.rgba.a)):t}equals(e){return this.renderMinimap===e.renderMinimap&&this.size===e.size&&this.minimapHeightIsEditorHeight===e.minimapHeightIsEditorHeight&&this.scrollBeyondLastLine===e.scrollBeyondLastLine&&this.paddingTop===e.paddingTop&&this.paddingBottom===e.paddingBottom&&this.showSlider===e.showSlider&&this.autohide===e.autohide&&this.pixelRatio===e.pixelRatio&&this.typicalHalfwidthCharacterWidth===e.typicalHalfwidthCharacterWidth&&this.lineHeight===e.lineHeight&&this.minimapLeft===e.minimapLeft&&this.minimapWidth===e.minimapWidth&&this.minimapHeight===e.minimapHeight&&this.canvasInnerWidth===e.canvasInnerWidth&&this.canvasInnerHeight===e.canvasInnerHeight&&this.canvasOuterWidth===e.canvasOuterWidth&&this.canvasOuterHeight===e.canvasOuterHeight&&this.isSampling===e.isSampling&&this.editorHeight===e.editorHeight&&this.fontScale===e.fontScale&&this.minimapLineHeight===e.minimapLineHeight&&this.minimapCharWidth===e.minimapCharWidth&&this.sectionHeaderFontSize===e.sectionHeaderFontSize&&this.sectionHeaderLetterSpacing===e.sectionHeaderLetterSpacing&&this.defaultBackgroundColor&&this.defaultBackgroundColor.equals(e.defaultBackgroundColor)&&this.backgroundColor&&this.backgroundColor.equals(e.backgroundColor)&&this.foregroundAlpha===e.foregroundAlpha}}class gv{constructor(e,t,i,n,s,r,a,l,c){this.scrollTop=e,this.scrollHeight=t,this.sliderNeeded=i,this._computedSliderRatio=n,this.sliderTop=s,this.sliderHeight=r,this.topPaddingLineCount=a,this.startLineNumber=l,this.endLineNumber=c}getDesiredScrollTopFromDelta(e){return Math.round(this.scrollTop+e/this._computedSliderRatio)}getDesiredScrollTopFromTouchLocation(e){return Math.round((e-this.sliderHeight/2)/this._computedSliderRatio)}intersectWithViewport(e){const t=Math.max(this.startLineNumber,e.startLineNumber),i=Math.min(this.endLineNumber,e.endLineNumber);return t>i?null:[t,i]}getYForLineNumber(e,t){return+(e-this.startLineNumber+this.topPaddingLineCount)*t}static create(e,t,i,n,s,r,a,l,c,d,h){const u=e.pixelRatio,g=e.minimapLineHeight,f=Math.floor(e.canvasInnerHeight/g),p=e.lineHeight;if(e.minimapHeightIsEditorHeight){let L=l*e.lineHeight+e.paddingTop+e.paddingBottom;e.scrollBeyondLastLine&&(L+=Math.max(0,s-e.lineHeight-e.paddingBottom));const k=Math.max(1,Math.floor(s*s/L)),D=Math.max(0,e.minimapHeight-k),N=D/(d-s),B=c*N,W=D>0,V=Math.floor(e.canvasInnerHeight/e.minimapLineHeight),$=Math.floor(e.paddingTop/e.lineHeight);return new gv(c,d,W,N,B,k,$,1,Math.min(a,V))}let _;if(r&&i!==a){const L=i-t+1;_=Math.floor(L*g/u)}else{const L=s/p;_=Math.floor(L*g/u)}const b=Math.floor(e.paddingTop/p);let C=Math.floor(e.paddingBottom/p);if(e.scrollBeyondLastLine){const L=s/p;C=Math.max(C,L-1)}let v;if(C>0){const L=s/p;v=(b+a+C-L-1)*g/u}else v=Math.max(0,(b+a)*g/u-_);v=Math.min(e.minimapHeight-_,v);const w=v/(d-s),S=c*w;if(f>=b+a+C){const L=v>0;return new gv(c,d,L,w,S,_,b,1,a)}else{let L;t>1?L=t+b:L=Math.max(1,c/p);let k,D=Math.max(1,Math.floor(L-S*u/g));D<b?(k=b-D+1,D=1):(k=0,D=Math.max(1,D-b)),h&&h.scrollHeight===d&&(h.scrollTop>c&&(D=Math.min(D,h.startLineNumber),k=Math.max(k,h.topPaddingLineCount)),h.scrollTop<c&&(D=Math.max(D,h.startLineNumber),k=Math.min(k,h.topPaddingLineCount)));const N=Math.min(a,D-k+f-1),B=(c-n)/p;let W;return c>=e.paddingTop?W=(t-D+k+B)*g/u:W=c/e.paddingTop*(k+B)*g/u,new gv(c,d,!0,w,W,_,k,D,N)}}}const uD=class uD{constructor(e){this.dy=e}onContentChanged(){this.dy=-1}onTokensChanged(){this.dy=-1}};uD.INVALID=new uD(-1);let ux=uD;class TH{constructor(e,t,i){this.renderedLayout=e,this._imageData=t,this._renderedLines=new Lq({createLine:()=>ux.INVALID}),this._renderedLines._set(e.startLineNumber,i)}linesEquals(e){if(!this.scrollEquals(e))return!1;const i=this._renderedLines._get().lines;for(let n=0,s=i.length;n<s;n++)if(i[n].dy===-1)return!1;return!0}scrollEquals(e){return this.renderedLayout.startLineNumber===e.startLineNumber&&this.renderedLayout.endLineNumber===e.endLineNumber}_get(){const e=this._renderedLines._get();return{imageData:this._imageData,rendLineNumberStart:e.rendLineNumberStart,lines:e.lines}}onLinesChanged(e,t){return this._renderedLines.onLinesChanged(e,t)}onLinesDeleted(e,t){this._renderedLines.onLinesDeleted(e,t)}onLinesInserted(e,t){this._renderedLines.onLinesInserted(e,t)}onTokensChanged(e){return this._renderedLines.onTokensChanged(e)}}class Q3{constructor(e,t,i,n){this._backgroundFillData=Q3._createBackgroundFillData(t,i,n),this._buffers=[e.createImageData(t,i),e.createImageData(t,i)],this._lastUsedBuffer=0}getBuffer(){this._lastUsedBuffer=1-this._lastUsedBuffer;const e=this._buffers[this._lastUsedBuffer];return e.data.set(this._backgroundFillData),e}static _createBackgroundFillData(e,t,i){const n=i.r,s=i.g,r=i.b,a=i.a,l=new Uint8ClampedArray(e*t*4);let c=0;for(let d=0;d<t;d++)for(let h=0;h<e;h++)l[c]=n,l[c+1]=s,l[c+2]=r,l[c+3]=a,c+=4;return l}}class Gw{static compute(e,t,i){if(e.renderMinimap===0||!e.isSampling)return[null,[]];const{minimapLineCount:n}=Hp.computeContainedMinimapLineCount({viewLineCount:t,scrollBeyondLastLine:e.scrollBeyondLastLine,paddingTop:e.paddingTop,paddingBottom:e.paddingBottom,height:e.editorHeight,lineHeight:e.lineHeight,pixelRatio:e.pixelRatio}),s=t/n,r=s/2;if(!i||i.minimapLines.length===0){const _=[];if(_[0]=1,n>1){for(let b=0,C=n-1;b<C;b++)_[b]=Math.round(b*s+r);_[n-1]=t}return[new Gw(s,_),[]]}const a=i.minimapLines,l=a.length,c=[];let d=0,h=0,u=1;const g=10;let f=[],p=null;for(let _=0;_<n;_++){const b=Math.max(u,Math.round(_*s)),C=Math.max(b,Math.round((_+1)*s));for(;d<l&&a[d]<b;){if(f.length<g){const w=d+1+h;p&&p.type==="deleted"&&p._oldIndex===d-1?p.deleteToLineNumber++:(p={type:"deleted",_oldIndex:d,deleteFromLineNumber:w,deleteToLineNumber:w},f.push(p)),h--}d++}let v;if(d<l&&a[d]<=C)v=a[d],d++;else if(_===0?v=1:_+1===n?v=t:v=Math.round(_*s+r),f.length<g){const w=d+1+h;p&&p.type==="inserted"&&p._i===_-1?p.insertToLineNumber++:(p={type:"inserted",_i:_,insertFromLineNumber:w,insertToLineNumber:w},f.push(p)),h++}c[_]=v,u=v}if(f.length<g)for(;d<l;){const _=d+1+h;p&&p.type==="deleted"&&p._oldIndex===d-1?p.deleteToLineNumber++:(p={type:"deleted",_oldIndex:d,deleteFromLineNumber:_,deleteToLineNumber:_},f.push(p)),h--,d++}else f=[{type:"flush"}];return[new Gw(s,c),f]}constructor(e,t){this.samplingRatio=e,this.minimapLines=t}modelLineToMinimapLine(e){return Math.min(this.minimapLines.length,Math.max(1,Math.round(e/this.samplingRatio)))}modelLineRangeToMinimapLineRange(e,t){let i=this.modelLineToMinimapLine(e)-1;for(;i>0&&this.minimapLines[i-1]>=e;)i--;let n=this.modelLineToMinimapLine(t)-1;for(;n+1<this.minimapLines.length&&this.minimapLines[n+1]<=t;)n++;if(i===n){const s=this.minimapLines[i];if(s<e||s>t)return null}return[i+1,n+1]}decorationLineRangeToMinimapLineRange(e,t){let i=this.modelLineToMinimapLine(e),n=this.modelLineToMinimapLine(t);return e!==t&&n===i&&(n===this.minimapLines.length?i>1&&i--:n++),[i,n]}onLinesDeleted(e){const t=e.toLineNumber-e.fromLineNumber+1;let i=this.minimapLines.length,n=0;for(let s=this.minimapLines.length-1;s>=0&&!(this.minimapLines[s]<e.fromLineNumber);s--)this.minimapLines[s]<=e.toLineNumber?(this.minimapLines[s]=Math.max(1,e.fromLineNumber-1),i=Math.min(i,s),n=Math.max(n,s)):this.minimapLines[s]-=t;return[i,n]}onLinesInserted(e){const t=e.toLineNumber-e.fromLineNumber+1;for(let i=this.minimapLines.length-1;i>=0&&!(this.minimapLines[i]<e.fromLineNumber);i--)this.minimapLines[i]+=t}}class bbe extends bo{constructor(e){super(e),this._sectionHeaderCache=new Gl(10,1.5),this.tokensColorTracker=hx.getInstance(),this._selections=[],this._minimapSelections=null,this.options=new a_(this._context.configuration,this._context.theme,this.tokensColorTracker);const[t]=Gw.compute(this.options,this._context.viewModel.getLineCount(),null);this._samplingState=t,this._shouldCheckSampling=!1,this._actual=new gp(e.theme,this)}dispose(){this._actual.dispose(),super.dispose()}getDomNode(){return this._actual.getDomNode()}_onOptionsMaybeChanged(){const e=new a_(this._context.configuration,this._context.theme,this.tokensColorTracker);return this.options.equals(e)?!1:(this.options=e,this._recreateLineSampling(),this._actual.onDidChangeOptions(),!0)}onConfigurationChanged(e){return this._onOptionsMaybeChanged()}onCursorStateChanged(e){return this._selections=e.selections,this._minimapSelections=null,this._actual.onSelectionChanged()}onDecorationsChanged(e){return e.affectsMinimap?this._actual.onDecorationsChanged():!1}onFlushed(e){return this._samplingState&&(this._shouldCheckSampling=!0),this._actual.onFlushed()}onLinesChanged(e){if(this._samplingState){const t=this._samplingState.modelLineRangeToMinimapLineRange(e.fromLineNumber,e.fromLineNumber+e.count-1);return t?this._actual.onLinesChanged(t[0],t[1]-t[0]+1):!1}else return this._actual.onLinesChanged(e.fromLineNumber,e.count)}onLinesDeleted(e){if(this._samplingState){const[t,i]=this._samplingState.onLinesDeleted(e);return t<=i&&this._actual.onLinesChanged(t+1,i-t+1),this._shouldCheckSampling=!0,!0}else return this._actual.onLinesDeleted(e.fromLineNumber,e.toLineNumber)}onLinesInserted(e){return this._samplingState?(this._samplingState.onLinesInserted(e),this._shouldCheckSampling=!0,!0):this._actual.onLinesInserted(e.fromLineNumber,e.toLineNumber)}onScrollChanged(e){return this._actual.onScrollChanged()}onThemeChanged(e){return this._actual.onThemeChanged(),this._onOptionsMaybeChanged(),!0}onTokensChanged(e){if(this._samplingState){const t=[];for(const i of e.ranges){const n=this._samplingState.modelLineRangeToMinimapLineRange(i.fromLineNumber,i.toLineNumber);n&&t.push({fromLineNumber:n[0],toLineNumber:n[1]})}return t.length?this._actual.onTokensChanged(t):!1}else return this._actual.onTokensChanged(e.ranges)}onTokensColorsChanged(e){return this._onOptionsMaybeChanged(),this._actual.onTokensColorsChanged()}onZonesChanged(e){return this._actual.onZonesChanged()}prepareRender(e){this._shouldCheckSampling&&(this._shouldCheckSampling=!1,this._recreateLineSampling())}render(e){let t=e.visibleRange.startLineNumber,i=e.visibleRange.endLineNumber;this._samplingState&&(t=this._samplingState.modelLineToMinimapLine(t),i=this._samplingState.modelLineToMinimapLine(i));const n={viewportContainsWhitespaceGaps:e.viewportData.whitespaceViewportData.length>0,scrollWidth:e.scrollWidth,scrollHeight:e.scrollHeight,viewportStartLineNumber:t,viewportEndLineNumber:i,viewportStartLineNumberVerticalOffset:e.getVerticalOffsetForLineNumber(t),scrollTop:e.scrollTop,scrollLeft:e.scrollLeft,viewportWidth:e.viewportWidth,viewportHeight:e.viewportHeight};this._actual.render(n)}_recreateLineSampling(){this._minimapSelections=null;const e=!!this._samplingState,[t,i]=Gw.compute(this.options,this._context.viewModel.getLineCount(),this._samplingState);if(this._samplingState=t,e&&this._samplingState)for(const n of i)switch(n.type){case"deleted":this._actual.onLinesDeleted(n.deleteFromLineNumber,n.deleteToLineNumber);break;case"inserted":this._actual.onLinesInserted(n.insertFromLineNumber,n.insertToLineNumber);break;case"flush":this._actual.onFlushed();break}}getLineCount(){return this._samplingState?this._samplingState.minimapLines.length:this._context.viewModel.getLineCount()}getRealLineCount(){return this._context.viewModel.getLineCount()}getLineContent(e){return this._samplingState?this._context.viewModel.getLineContent(this._samplingState.minimapLines[e-1]):this._context.viewModel.getLineContent(e)}getLineMaxColumn(e){return this._samplingState?this._context.viewModel.getLineMaxColumn(this._samplingState.minimapLines[e-1]):this._context.viewModel.getLineMaxColumn(e)}getMinimapLinesRenderingData(e,t,i){if(this._samplingState){const n=[];for(let s=0,r=t-e+1;s<r;s++)i[s]?n[s]=this._context.viewModel.getViewLineData(this._samplingState.minimapLines[e+s-1]):n[s]=null;return n}return this._context.viewModel.getMinimapLinesRenderingData(e,t,i).data}getSelections(){if(this._minimapSelections===null)if(this._samplingState){this._minimapSelections=[];for(const e of this._selections){const[t,i]=this._samplingState.decorationLineRangeToMinimapLineRange(e.startLineNumber,e.endLineNumber);this._minimapSelections.push(new pe(t,e.startColumn,i,e.endColumn))}}else this._minimapSelections=this._selections;return this._minimapSelections}getMinimapDecorationsInViewport(e,t){const i=this._getMinimapDecorationsInViewport(e,t).filter(n=>{var s;return!((s=n.options.minimap)!=null&&s.sectionHeaderStyle)});if(this._samplingState){const n=[];for(const s of i){if(!s.options.minimap)continue;const r=s.range,a=this._samplingState.modelLineToMinimapLine(r.startLineNumber),l=this._samplingState.modelLineToMinimapLine(r.endLineNumber);n.push(new hq(new x(a,r.startColumn,l,r.endColumn),s.options))}return n}return i}getSectionHeaderDecorationsInViewport(e,t){const i=this.options.minimapLineHeight,s=this.options.sectionHeaderFontSize/i;return e=Math.floor(Math.max(1,e-s)),this._getMinimapDecorationsInViewport(e,t).filter(r=>{var a;return!!((a=r.options.minimap)!=null&&a.sectionHeaderStyle)})}_getMinimapDecorationsInViewport(e,t){let i;if(this._samplingState){const n=this._samplingState.minimapLines[e-1],s=this._samplingState.minimapLines[t-1];i=new x(n,1,s,this._context.viewModel.getLineMaxColumn(s))}else i=new x(e,1,t,this._context.viewModel.getLineMaxColumn(t));return this._context.viewModel.getMinimapDecorationsInRange(i)}getSectionHeaderText(e,t){var r;const i=(r=e.options.minimap)==null?void 0:r.sectionHeaderText;if(!i)return null;const n=this._sectionHeaderCache.get(i);if(n)return n;const s=t(i);return this._sectionHeaderCache.set(i,s),s}getOptions(){return this._context.viewModel.model.getOptions()}revealLineNumber(e){this._samplingState&&(e=this._samplingState.minimapLines[e-1]),this._context.viewModel.revealRange("mouse",!1,new x(e,1,e,1),1,0)}setScrollTop(e){this._context.viewModel.viewLayout.setScrollPosition({scrollTop:e},1)}}class gp extends F{constructor(e,t){super(),this._renderDecorations=!1,this._gestureInProgress=!1,this._theme=e,this._model=t,this._lastRenderData=null,this._buffers=null,this._selectionColor=this._theme.getColor(D9),this._domNode=St(document.createElement("div")),Va.write(this._domNode,9),this._domNode.setClassName(this._getMinimapDomNodeClassName()),this._domNode.setPosition("absolute"),this._domNode.setAttribute("role","presentation"),this._domNode.setAttribute("aria-hidden","true"),this._shadow=St(document.createElement("div")),this._shadow.setClassName("minimap-shadow-hidden"),this._domNode.appendChild(this._shadow),this._canvas=St(document.createElement("canvas")),this._canvas.setPosition("absolute"),this._canvas.setLeft(0),this._domNode.appendChild(this._canvas),this._decorationsCanvas=St(document.createElement("canvas")),this._decorationsCanvas.setPosition("absolute"),this._decorationsCanvas.setClassName("minimap-decorations-layer"),this._decorationsCanvas.setLeft(0),this._domNode.appendChild(this._decorationsCanvas),this._slider=St(document.createElement("div")),this._slider.setPosition("absolute"),this._slider.setClassName("minimap-slider"),this._slider.setLayerHinting(!0),this._slider.setContain("strict"),this._domNode.appendChild(this._slider),this._sliderHorizontal=St(document.createElement("div")),this._sliderHorizontal.setPosition("absolute"),this._sliderHorizontal.setClassName("minimap-slider-horizontal"),this._slider.appendChild(this._sliderHorizontal),this._applyLayout(),this._pointerDownListener=hi(this._domNode.domNode,ie.POINTER_DOWN,i=>{if(i.preventDefault(),this._model.options.renderMinimap===0||!this._lastRenderData)return;if(this._model.options.size!=="proportional"){if(i.button===0&&this._lastRenderData){const c=wi(this._slider.domNode),d=c.top+c.height/2;this._startSliderDragging(i,d,this._lastRenderData.renderedLayout)}return}const s=this._model.options.minimapLineHeight,r=this._model.options.canvasInnerHeight/this._model.options.canvasOuterHeight*i.offsetY;let l=Math.floor(r/s)+this._lastRenderData.renderedLayout.startLineNumber-this._lastRenderData.renderedLayout.topPaddingLineCount;l=Math.min(l,this._model.getLineCount()),this._model.revealLineNumber(l)}),this._sliderPointerMoveMonitor=new vb,this._sliderPointerDownListener=hi(this._slider.domNode,ie.POINTER_DOWN,i=>{i.preventDefault(),i.stopPropagation(),i.button===0&&this._lastRenderData&&this._startSliderDragging(i,i.pageY,this._lastRenderData.renderedLayout)}),this._gestureDisposable=Fn.addTarget(this._domNode.domNode),this._sliderTouchStartListener=U(this._domNode.domNode,Bt.Start,i=>{i.preventDefault(),i.stopPropagation(),this._lastRenderData&&(this._slider.toggleClassName("active",!0),this._gestureInProgress=!0,this.scrollDueToTouchEvent(i))},{passive:!1}),this._sliderTouchMoveListener=U(this._domNode.domNode,Bt.Change,i=>{i.preventDefault(),i.stopPropagation(),this._lastRenderData&&this._gestureInProgress&&this.scrollDueToTouchEvent(i)},{passive:!1}),this._sliderTouchEndListener=hi(this._domNode.domNode,Bt.End,i=>{i.preventDefault(),i.stopPropagation(),this._gestureInProgress=!1,this._slider.toggleClassName("active",!1)})}_startSliderDragging(e,t,i){if(!e.target||!(e.target instanceof Element))return;const n=e.pageX;this._slider.toggleClassName("active",!0);const s=(r,a)=>{const l=wi(this._domNode.domNode),c=Math.min(Math.abs(a-n),Math.abs(a-l.left),Math.abs(a-l.left-l.width));if(En&&c>pbe){this._model.setScrollTop(i.scrollTop);return}const d=r-t;this._model.setScrollTop(i.getDesiredScrollTopFromDelta(d))};e.pageY!==t&&s(e.pageY,n),this._sliderPointerMoveMonitor.startMonitoring(e.target,e.pointerId,e.buttons,r=>s(r.pageY,r.pageX),()=>{this._slider.toggleClassName("active",!1)})}scrollDueToTouchEvent(e){const t=this._domNode.domNode.getBoundingClientRect().top,i=this._lastRenderData.renderedLayout.getDesiredScrollTopFromTouchLocation(e.pageY-t);this._model.setScrollTop(i)}dispose(){this._pointerDownListener.dispose(),this._sliderPointerMoveMonitor.dispose(),this._sliderPointerDownListener.dispose(),this._gestureDisposable.dispose(),this._sliderTouchStartListener.dispose(),this._sliderTouchMoveListener.dispose(),this._sliderTouchEndListener.dispose(),super.dispose()}_getMinimapDomNodeClassName(){const e=["minimap"];return this._model.options.showSlider==="always"?e.push("slider-always"):e.push("slider-mouseover"),this._model.options.autohide&&e.push("autohide"),e.join(" ")}getDomNode(){return this._domNode}_applyLayout(){this._domNode.setLeft(this._model.options.minimapLeft),this._domNode.setWidth(this._model.options.minimapWidth),this._domNode.setHeight(this._model.options.minimapHeight),this._shadow.setHeight(this._model.options.minimapHeight),this._canvas.setWidth(this._model.options.canvasOuterWidth),this._canvas.setHeight(this._model.options.canvasOuterHeight),this._canvas.domNode.width=this._model.options.canvasInnerWidth,this._canvas.domNode.height=this._model.options.canvasInnerHeight,this._decorationsCanvas.setWidth(this._model.options.canvasOuterWidth),this._decorationsCanvas.setHeight(this._model.options.canvasOuterHeight),this._decorationsCanvas.domNode.width=this._model.options.canvasInnerWidth,this._decorationsCanvas.domNode.height=this._model.options.canvasInnerHeight,this._slider.setWidth(this._model.options.minimapWidth)}_getBuffer(){return this._buffers||this._model.options.canvasInnerWidth>0&&this._model.options.canvasInnerHeight>0&&(this._buffers=new Q3(this._canvas.domNode.getContext("2d"),this._model.options.canvasInnerWidth,this._model.options.canvasInnerHeight,this._model.options.backgroundColor)),this._buffers?this._buffers.getBuffer():null}onDidChangeOptions(){this._lastRenderData=null,this._buffers=null,this._applyLayout(),this._domNode.setClassName(this._getMinimapDomNodeClassName())}onSelectionChanged(){return this._renderDecorations=!0,!0}onDecorationsChanged(){return this._renderDecorations=!0,!0}onFlushed(){return this._lastRenderData=null,!0}onLinesChanged(e,t){return this._lastRenderData?this._lastRenderData.onLinesChanged(e,t):!1}onLinesDeleted(e,t){var i;return(i=this._lastRenderData)==null||i.onLinesDeleted(e,t),!0}onLinesInserted(e,t){var i;return(i=this._lastRenderData)==null||i.onLinesInserted(e,t),!0}onScrollChanged(){return this._renderDecorations=!0,!0}onThemeChanged(){return this._selectionColor=this._theme.getColor(D9),this._renderDecorations=!0,!0}onTokensChanged(e){return this._lastRenderData?this._lastRenderData.onTokensChanged(e):!1}onTokensColorsChanged(){return this._lastRenderData=null,this._buffers=null,!0}onZonesChanged(){return this._lastRenderData=null,!0}render(e){if(this._model.options.renderMinimap===0){this._shadow.setClassName("minimap-shadow-hidden"),this._sliderHorizontal.setWidth(0),this._sliderHorizontal.setHeight(0);return}e.scrollLeft+e.viewportWidth>=e.scrollWidth?this._shadow.setClassName("minimap-shadow-hidden"):this._shadow.setClassName("minimap-shadow-visible");const i=gv.create(this._model.options,e.viewportStartLineNumber,e.viewportEndLineNumber,e.viewportStartLineNumberVerticalOffset,e.viewportHeight,e.viewportContainsWhitespaceGaps,this._model.getLineCount(),this._model.getRealLineCount(),e.scrollTop,e.scrollHeight,this._lastRenderData?this._lastRenderData.renderedLayout:null);this._slider.setDisplay(i.sliderNeeded?"block":"none"),this._slider.setTop(i.sliderTop),this._slider.setHeight(i.sliderHeight),this._sliderHorizontal.setLeft(0),this._sliderHorizontal.setWidth(this._model.options.minimapWidth),this._sliderHorizontal.setTop(0),this._sliderHorizontal.setHeight(i.sliderHeight),this.renderDecorations(i),this._lastRenderData=this.renderLines(i)}renderDecorations(e){if(this._renderDecorations){this._renderDecorations=!1;const t=this._model.getSelections();t.sort(x.compareRangesUsingStarts);const i=this._model.getMinimapDecorationsInViewport(e.startLineNumber,e.endLineNumber);i.sort((u,g)=>(u.options.zIndex||0)-(g.options.zIndex||0));const{canvasInnerWidth:n,canvasInnerHeight:s}=this._model.options,r=this._model.options.minimapLineHeight,a=this._model.options.minimapCharWidth,l=this._model.getOptions().tabSize,c=this._decorationsCanvas.domNode.getContext("2d");c.clearRect(0,0,n,s);const d=new RH(e.startLineNumber,e.endLineNumber,!1);this._renderSelectionLineHighlights(c,t,d,e,r),this._renderDecorationsLineHighlights(c,i,d,e,r);const h=new RH(e.startLineNumber,e.endLineNumber,null);this._renderSelectionsHighlights(c,t,h,e,r,l,a,n),this._renderDecorationsHighlights(c,i,h,e,r,l,a,n),this._renderSectionHeaders(e)}}_renderSelectionLineHighlights(e,t,i,n,s){if(!this._selectionColor||this._selectionColor.isTransparent())return;e.fillStyle=this._selectionColor.transparent(.5).toString();let r=0,a=0;for(const l of t){const c=n.intersectWithViewport(l);if(!c)continue;const[d,h]=c;for(let f=d;f<=h;f++)i.set(f,!0);const u=n.getYForLineNumber(d,s),g=n.getYForLineNumber(h,s);a>=u||(a>r&&e.fillRect(sl,r,e.canvas.width,a-r),r=u),a=g}a>r&&e.fillRect(sl,r,e.canvas.width,a-r)}_renderDecorationsLineHighlights(e,t,i,n,s){const r=new Map;for(let a=t.length-1;a>=0;a--){const l=t[a],c=l.options.minimap;if(!c||c.position!==1)continue;const d=n.intersectWithViewport(l.range);if(!d)continue;const[h,u]=d,g=c.getColor(this._theme.value);if(!g||g.isTransparent())continue;let f=r.get(g.toString());f||(f=g.transparent(.5).toString(),r.set(g.toString(),f)),e.fillStyle=f;for(let p=h;p<=u;p++){if(i.has(p))continue;i.set(p,!0);const _=n.getYForLineNumber(h,s);e.fillRect(sl,_,e.canvas.width,s)}}}_renderSelectionsHighlights(e,t,i,n,s,r,a,l){if(!(!this._selectionColor||this._selectionColor.isTransparent()))for(const c of t){const d=n.intersectWithViewport(c);if(!d)continue;const[h,u]=d;for(let g=h;g<=u;g++)this.renderDecorationOnLine(e,i,c,this._selectionColor,n,g,s,s,r,a,l)}}_renderDecorationsHighlights(e,t,i,n,s,r,a,l){for(const c of t){const d=c.options.minimap;if(!d)continue;const h=n.intersectWithViewport(c.range);if(!h)continue;const[u,g]=h,f=d.getColor(this._theme.value);if(!(!f||f.isTransparent()))for(let p=u;p<=g;p++)switch(d.position){case 1:this.renderDecorationOnLine(e,i,c.range,f,n,p,s,s,r,a,l);continue;case 2:{const _=n.getYForLineNumber(p,s);this.renderDecoration(e,f,2,_,_be,s);continue}}}}renderDecorationOnLine(e,t,i,n,s,r,a,l,c,d,h){const u=s.getYForLineNumber(r,l);if(u+a<0||u>this._model.options.canvasInnerHeight)return;const{startLineNumber:g,endLineNumber:f}=i,p=g===r?i.startColumn:1,_=f===r?i.endColumn:this._model.getLineMaxColumn(r),b=this.getXOffsetForPosition(t,r,p,c,d,h),C=this.getXOffsetForPosition(t,r,_,c,d,h);this.renderDecoration(e,n,b,u,C-b,a)}getXOffsetForPosition(e,t,i,n,s,r){if(i===1)return sl;if((i-1)*s>=r)return r;let l=e.get(t);if(!l){const c=this._model.getLineContent(t);l=[sl];let d=sl;for(let h=1;h<c.length+1;h++){const u=c.charCodeAt(h-1),g=u===9?n*s:Fh(u)?2*s:s,f=d+g;if(f>=r){l[h]=r;break}l[h]=f,d=f}e.set(t,l)}return i-1<l.length?l[i-1]:r}renderDecoration(e,t,i,n,s,r){e.fillStyle=t&&t.toString()||"",e.fillRect(i,n,s,r)}_renderSectionHeaders(e){var p;const t=this._model.options.minimapLineHeight,i=this._model.options.sectionHeaderFontSize,n=this._model.options.sectionHeaderLetterSpacing,s=i*1.5,{canvasInnerWidth:r}=this._model.options,a=this._model.options.backgroundColor,l=`rgb(${a.r} ${a.g} ${a.b} / .7)`,c=this._model.options.sectionHeaderFontColor,d=`rgb(${c.r} ${c.g} ${c.b})`,h=d,u=this._decorationsCanvas.domNode.getContext("2d");u.letterSpacing=n+"px",u.font="500 "+i+"px "+this._model.options.sectionHeaderFontFamily,u.strokeStyle=h,u.lineWidth=.2;const g=this._model.getSectionHeaderDecorationsInViewport(e.startLineNumber,e.endLineNumber);g.sort((_,b)=>_.range.startLineNumber-b.range.startLineNumber);const f=gp._fitSectionHeader.bind(null,u,r-sl);for(const _ of g){const b=e.getYForLineNumber(_.range.startLineNumber,t)+i,C=b-i,v=C+2,w=this._model.getSectionHeaderText(_,f);gp._renderSectionLabel(u,w,((p=_.options.minimap)==null?void 0:p.sectionHeaderStyle)===2,l,d,r,C,s,b,v)}}static _fitSectionHeader(e,t,i){if(!i)return i;const n="…",s=e.measureText(i).width,r=e.measureText(n).width;if(s<=t||s<=r)return i;const a=i.length,l=s/i.length,c=Math.floor((t-r)/l)-1;let d=Math.ceil(c/2);for(;d>0&&/\s/.test(i[d-1]);)--d;return i.substring(0,d)+n+i.substring(a-(c-d))}static _renderSectionLabel(e,t,i,n,s,r,a,l,c,d){t&&(e.fillStyle=n,e.fillRect(0,a,r,l),e.fillStyle=s,e.fillText(t,sl,c)),i&&(e.beginPath(),e.moveTo(0,d),e.lineTo(r,d),e.closePath(),e.stroke())}renderLines(e){const t=e.startLineNumber,i=e.endLineNumber,n=this._model.options.minimapLineHeight;if(this._lastRenderData&&this._lastRenderData.linesEquals(e)){const j=this._lastRenderData._get();return new TH(e,j.imageData,j.lines)}const s=this._getBuffer();if(!s)return null;const[r,a,l]=gp._renderUntouchedLines(s,e.topPaddingLineCount,t,i,n,this._lastRenderData),c=this._model.getMinimapLinesRenderingData(t,i,l),d=this._model.getOptions().tabSize,h=this._model.options.defaultBackgroundColor,u=this._model.options.backgroundColor,g=this._model.options.foregroundAlpha,f=this._model.tokensColorTracker,p=f.backgroundIsLight(),_=this._model.options.renderMinimap,b=this._model.options.charRenderer(),C=this._model.options.fontScale,v=this._model.options.minimapCharWidth,S=(_===1?2:3)*C,L=n>S?Math.floor((n-S)/2):0,k=u.a/255,D=new Gc(Math.round((u.r-h.r)*k+h.r),Math.round((u.g-h.g)*k+h.g),Math.round((u.b-h.b)*k+h.b),255);let N=e.topPaddingLineCount*n;const B=[];for(let j=0,J=i-t+1;j<J;j++)l[j]&&gp._renderLine(s,D,u.a,p,_,v,f,g,b,N,L,d,c[j],C,n),B[j]=new ux(N),N+=n;const W=r===-1?0:r,$=(a===-1?s.height:a)-W;return this._canvas.domNode.getContext("2d").putImageData(s,0,0,0,W,s.width,$),new TH(e,s,B)}static _renderUntouchedLines(e,t,i,n,s,r){const a=[];if(!r){for(let N=0,B=n-i+1;N<B;N++)a[N]=!0;return[-1,-1,a]}const l=r._get(),c=l.imageData.data,d=l.rendLineNumberStart,h=l.lines,u=h.length,g=e.width,f=e.data,p=(n-i+1)*s*g*4;let _=-1,b=-1,C=-1,v=-1,w=-1,S=-1,L=t*s;for(let N=i;N<=n;N++){const B=N-i,W=N-d,V=W>=0&&W<u?h[W].dy:-1;if(V===-1){a[B]=!0,L+=s;continue}const $=V*g*4,H=(V+s)*g*4,j=L*g*4,J=(L+s)*g*4;v===$&&S===j?(v=H,S=J):(C!==-1&&(f.set(c.subarray(C,v),w),_===-1&&C===0&&C===w&&(_=v),b===-1&&v===p&&C===w&&(b=C)),C=$,v=H,w=j,S=J),a[B]=!1,L+=s}C!==-1&&(f.set(c.subarray(C,v),w),_===-1&&C===0&&C===w&&(_=v),b===-1&&v===p&&C===w&&(b=C));const k=_===-1?-1:_/(g*4),D=b===-1?-1:b/(g*4);return[k,D,a]}static _renderLine(e,t,i,n,s,r,a,l,c,d,h,u,g,f,p){const _=g.content,b=g.tokens,C=e.width-r,v=p===1;let w=sl,S=0,L=0;for(let k=0,D=b.getCount();k<D;k++){const N=b.getEndOffset(k),B=b.getForeground(k),W=a.getColor(B);for(;S<N;S++){if(w>C)return;const V=_.charCodeAt(S);if(V===9){const $=u-(S+L)%u;L+=$-1,w+=$*r}else if(V===32)w+=r;else{const $=Fh(V)?2:1;for(let H=0;H<$;H++)if(s===2?c.blockRenderChar(e,w,d+h,W,l,t,i,v):c.renderChar(e,w,d+h,V,W,l,t,i,f,n,v),w+=r,w>C)return}}}}}class RH{constructor(e,t,i){this._startLineNumber=e,this._endLineNumber=t,this._defaultValue=i,this._values=[];for(let n=0,s=this._endLineNumber-this._startLineNumber+1;n<s;n++)this._values[n]=i}has(e){return this.get(e)!==this._defaultValue}set(e,t){e<this._startLineNumber||e>this._endLineNumber||(this._values[e-this._startLineNumber]=t)}get(e){return e<this._startLineNumber||e>this._endLineNumber?this._defaultValue:this._values[e-this._startLineNumber]}}class Cbe extends bo{constructor(e,t){super(e),this._viewDomNode=t;const n=this._context.configuration.options.get(146);this._widgets={},this._verticalScrollbarWidth=n.verticalScrollbarWidth,this._minimapWidth=n.minimap.minimapWidth,this._horizontalScrollbarHeight=n.horizontalScrollbarHeight,this._editorHeight=n.height,this._editorWidth=n.width,this._viewDomNodeRect={top:0,left:0,width:0,height:0},this._domNode=St(document.createElement("div")),Va.write(this._domNode,4),this._domNode.setClassName("overlayWidgets"),this.overflowingOverlayWidgetsDomNode=St(document.createElement("div")),Va.write(this.overflowingOverlayWidgetsDomNode,5),this.overflowingOverlayWidgetsDomNode.setClassName("overflowingOverlayWidgets")}dispose(){super.dispose(),this._widgets={}}getDomNode(){return this._domNode}onConfigurationChanged(e){const i=this._context.configuration.options.get(146);return this._verticalScrollbarWidth=i.verticalScrollbarWidth,this._minimapWidth=i.minimap.minimapWidth,this._horizontalScrollbarHeight=i.horizontalScrollbarHeight,this._editorHeight=i.height,this._editorWidth=i.width,!0}addWidget(e){const t=St(e.getDomNode());this._widgets[e.getId()]={widget:e,preference:null,domNode:t},t.setPosition("absolute"),t.setAttribute("widgetId",e.getId()),e.allowEditorOverflow?this.overflowingOverlayWidgetsDomNode.appendChild(t):this._domNode.appendChild(t),this.setShouldRender(),this._updateMaxMinWidth()}setWidgetPosition(e,t){const i=this._widgets[e.getId()],n=t?t.preference:null,s=t==null?void 0:t.stackOridinal;return i.preference===n&&i.stack===s?(this._updateMaxMinWidth(),!1):(i.preference=n,i.stack=s,this.setShouldRender(),this._updateMaxMinWidth(),!0)}removeWidget(e){const t=e.getId();if(this._widgets.hasOwnProperty(t)){const n=this._widgets[t].domNode.domNode;delete this._widgets[t],n.remove(),this.setShouldRender(),this._updateMaxMinWidth()}}_updateMaxMinWidth(){var i,n;let e=0;const t=Object.keys(this._widgets);for(let s=0,r=t.length;s<r;s++){const a=t[s],c=(n=(i=this._widgets[a].widget).getMinContentWidthInPx)==null?void 0:n.call(i);typeof c<"u"&&(e=Math.max(e,c))}this._context.viewLayout.setOverlayWidgetsMinWidth(e)}_renderWidget(e,t){const i=e.domNode;if(e.preference===null){i.setTop("");return}const n=2*this._verticalScrollbarWidth+this._minimapWidth;if(e.preference===0||e.preference===1){if(e.preference===1){const s=i.domNode.clientHeight;i.setTop(this._editorHeight-s-2*this._horizontalScrollbarHeight)}else i.setTop(0);e.stack!==void 0?(i.setTop(t[e.preference]),t[e.preference]+=i.domNode.clientWidth):i.setRight(n)}else if(e.preference===2)i.domNode.style.right="50%",e.stack!==void 0?(i.setTop(t[2]),t[2]+=i.domNode.clientHeight):i.setTop(0);else{const{top:s,left:r}=e.preference;if(this._context.configuration.options.get(42)&&e.widget.allowEditorOverflow){const l=this._viewDomNodeRect;i.setTop(s+l.top),i.setLeft(r+l.left),i.setPosition("fixed")}else i.setTop(s),i.setLeft(r),i.setPosition("absolute")}}prepareRender(e){this._viewDomNodeRect=wi(this._viewDomNode.domNode)}render(e){this._domNode.setWidth(this._editorWidth);const t=Object.keys(this._widgets),i=Array.from({length:3},()=>0);t.sort((n,s)=>(this._widgets[n].stack||0)-(this._widgets[s].stack||0));for(let n=0,s=t.length;n<s;n++){const r=t[n];this._renderWidget(this._widgets[r],i)}}}class vbe{constructor(e,t){const i=e.options;this.lineHeight=i.get(67),this.pixelRatio=i.get(144),this.overviewRulerLanes=i.get(83),this.renderBorder=i.get(82);const n=t.getColor(bhe);this.borderColor=n?n.toString():null,this.hideCursor=i.get(59);const s=t.getColor(oE);this.cursorColorSingle=s?s.transparent(.7).toString():null;const r=t.getColor(Hj);this.cursorColorPrimary=r?r.transparent(.7).toString():null;const a=t.getColor(Vj);this.cursorColorSecondary=a?a.transparent(.7).toString():null,this.themeType=t.type;const l=i.get(73),c=l.enabled,d=l.side,h=t.getColor(Che),u=fi.getDefaultBackground();h?this.backgroundColor=h:c&&d==="right"?this.backgroundColor=u:this.backgroundColor=null;const f=i.get(146).overviewRuler;this.top=f.top,this.right=f.right,this.domWidth=f.width,this.domHeight=f.height,this.overviewRulerLanes===0?(this.canvasWidth=0,this.canvasHeight=0):(this.canvasWidth=this.domWidth*this.pixelRatio|0,this.canvasHeight=this.domHeight*this.pixelRatio|0);const[p,_]=this._initLanes(1,this.canvasWidth,this.overviewRulerLanes);this.x=p,this.w=_}_initLanes(e,t,i){const n=t-e;if(i>=3){const s=Math.floor(n/3),r=Math.floor(n/3),a=n-s-r,l=e,c=l+s,d=l+s+a;return[[0,l,c,l,d,l,c,l],[0,s,a,s+a,r,s+a+r,a+r,s+a+r]]}else if(i===2){const s=Math.floor(n/2),r=n-s,a=e,l=a+s;return[[0,a,a,a,l,a,a,a],[0,s,s,s,r,s+r,s+r,s+r]]}else{const s=e,r=n;return[[0,s,s,s,s,s,s,s],[0,r,r,r,r,r,r,r]]}}equals(e){return this.lineHeight===e.lineHeight&&this.pixelRatio===e.pixelRatio&&this.overviewRulerLanes===e.overviewRulerLanes&&this.renderBorder===e.renderBorder&&this.borderColor===e.borderColor&&this.hideCursor===e.hideCursor&&this.cursorColorSingle===e.cursorColorSingle&&this.cursorColorPrimary===e.cursorColorPrimary&&this.cursorColorSecondary===e.cursorColorSecondary&&this.themeType===e.themeType&&Z.equals(this.backgroundColor,e.backgroundColor)&&this.top===e.top&&this.right===e.right&&this.domWidth===e.domWidth&&this.domHeight===e.domHeight&&this.canvasWidth===e.canvasWidth&&this.canvasHeight===e.canvasHeight}}class wbe extends bo{constructor(e){super(e),this._actualShouldRender=0,this._renderedDecorations=[],this._renderedCursorPositions=[],this._domNode=St(document.createElement("canvas")),this._domNode.setClassName("decorationsOverviewRuler"),this._domNode.setPosition("absolute"),this._domNode.setLayerHinting(!0),this._domNode.setContain("strict"),this._domNode.setAttribute("aria-hidden","true"),this._updateSettings(!1),this._tokensColorTrackerListener=fi.onDidChange(t=>{t.changedColorMap&&this._updateSettings(!0)}),this._cursorPositions=[{position:new A(1,1),color:this._settings.cursorColorSingle}]}dispose(){super.dispose(),this._tokensColorTrackerListener.dispose()}_updateSettings(e){const t=new vbe(this._context.configuration,this._context.theme);return this._settings&&this._settings.equals(t)?!1:(this._settings=t,this._domNode.setTop(this._settings.top),this._domNode.setRight(this._settings.right),this._domNode.setWidth(this._settings.domWidth),this._domNode.setHeight(this._settings.domHeight),this._domNode.domNode.width=this._settings.canvasWidth,this._domNode.domNode.height=this._settings.canvasHeight,e&&this._render(),!0)}_markRenderingIsNeeded(){return this._actualShouldRender=2,!0}_markRenderingIsMaybeNeeded(){return this._actualShouldRender=1,!0}onConfigurationChanged(e){return this._updateSettings(!1)?this._markRenderingIsNeeded():!1}onCursorStateChanged(e){this._cursorPositions=[];for(let t=0,i=e.selections.length;t<i;t++){let n=this._settings.cursorColorSingle;i>1&&(n=t===0?this._settings.cursorColorPrimary:this._settings.cursorColorSecondary),this._cursorPositions.push({position:e.selections[t].getPosition(),color:n})}return this._cursorPositions.sort((t,i)=>A.compare(t.position,i.position)),this._markRenderingIsMaybeNeeded()}onDecorationsChanged(e){return e.affectsOverviewRuler?this._markRenderingIsMaybeNeeded():!1}onFlushed(e){return this._markRenderingIsNeeded()}onScrollChanged(e){return e.scrollHeightChanged?this._markRenderingIsNeeded():!1}onZonesChanged(e){return this._markRenderingIsNeeded()}onThemeChanged(e){return this._updateSettings(!1)?this._markRenderingIsNeeded():!1}getDomNode(){return this._domNode.domNode}prepareRender(e){}render(e){this._render(),this._actualShouldRender=0}_render(){const e=this._settings.backgroundColor;if(this._settings.overviewRulerLanes===0){this._domNode.setBackgroundColor(e?Z.Format.CSS.formatHexA(e):""),this._domNode.setDisplay("none");return}const t=this._context.viewModel.getAllOverviewRulerDecorations(this._context.theme);if(t.sort(Hw.compareByRenderingProps),this._actualShouldRender===1&&!Hw.equalsArr(this._renderedDecorations,t)&&(this._actualShouldRender=2),this._actualShouldRender===1&&!ii(this._renderedCursorPositions,this._cursorPositions,(f,p)=>f.position.lineNumber===p.position.lineNumber&&f.color===p.color)&&(this._actualShouldRender=2),this._actualShouldRender===1)return;this._renderedDecorations=t,this._renderedCursorPositions=this._cursorPositions,this._domNode.setDisplay("block");const i=this._settings.canvasWidth,n=this._settings.canvasHeight,s=this._settings.lineHeight,r=this._context.viewLayout,a=this._context.viewLayout.getScrollHeight(),l=n/a,c=6*this._settings.pixelRatio|0,d=c/2|0,h=this._domNode.domNode.getContext("2d");e?e.isOpaque()?(h.fillStyle=Z.Format.CSS.formatHexA(e),h.fillRect(0,0,i,n)):(h.clearRect(0,0,i,n),h.fillStyle=Z.Format.CSS.formatHexA(e),h.fillRect(0,0,i,n)):h.clearRect(0,0,i,n);const u=this._settings.x,g=this._settings.w;for(const f of t){const p=f.color,_=f.data;h.fillStyle=p;let b=0,C=0,v=0;for(let w=0,S=_.length/3;w<S;w++){const L=_[3*w],k=_[3*w+1],D=_[3*w+2];let N=r.getVerticalOffsetForLineNumber(k)*l|0,B=(r.getVerticalOffsetForLineNumber(D)+s)*l|0;if(B-N<c){let V=(N+B)/2|0;V<d?V=d:V+d>n&&(V=n-d),N=V-d,B=V+d}N>v+1||L!==b?(w!==0&&h.fillRect(u[b],C,g[b],v-C),b=L,C=N,v=B):B>v&&(v=B)}h.fillRect(u[b],C,g[b],v-C)}if(!this._settings.hideCursor){const f=2*this._settings.pixelRatio|0,p=f/2|0,_=this._settings.x[7],b=this._settings.w[7];let C=-100,v=-100,w=null;for(let S=0,L=this._cursorPositions.length;S<L;S++){const k=this._cursorPositions[S].color;if(!k)continue;const D=this._cursorPositions[S].position;let N=r.getVerticalOffsetForLineNumber(D.lineNumber)*l|0;N<p?N=p:N+p>n&&(N=n-p);const B=N-p,W=B+f;B>v+1||k!==w?(S!==0&&w&&h.fillRect(_,C,b,v-C),C=B,v=W):W>v&&(v=W),w=k,h.fillStyle=k}w&&h.fillRect(_,C,b,v-C)}this._settings.renderBorder&&this._settings.borderColor&&this._settings.overviewRulerLanes>0&&(h.beginPath(),h.lineWidth=1,h.strokeStyle=this._settings.borderColor,h.moveTo(0,0),h.lineTo(0,n),h.moveTo(1,0),h.lineTo(i,0),h.stroke())}}class MH{constructor(e,t,i){this._colorZoneBrand=void 0,this.from=e|0,this.to=t|0,this.colorId=i|0}static compare(e,t){return e.colorId===t.colorId?e.from===t.from?e.to-t.to:e.from-t.from:e.colorId-t.colorId}}class Nq{constructor(e,t,i,n){this._overviewRulerZoneBrand=void 0,this.startLineNumber=e,this.endLineNumber=t,this.heightInLines=i,this.color=n,this._colorZone=null}static compare(e,t){return e.color===t.color?e.startLineNumber===t.startLineNumber?e.heightInLines===t.heightInLines?e.endLineNumber-t.endLineNumber:e.heightInLines-t.heightInLines:e.startLineNumber-t.startLineNumber:e.color<t.color?-1:1}setColorZone(e){this._colorZone=e}getColorZones(){return this._colorZone}}class Sbe{constructor(e){this._getVerticalOffsetForLine=e,this._zones=[],this._colorZonesInvalid=!1,this._lineHeight=0,this._domWidth=0,this._domHeight=0,this._outerHeight=0,this._pixelRatio=1,this._lastAssignedId=0,this._color2Id=Object.create(null),this._id2Color=[]}getId2Color(){return this._id2Color}setZones(e){this._zones=e,this._zones.sort(Nq.compare)}setLineHeight(e){return this._lineHeight===e?!1:(this._lineHeight=e,this._colorZonesInvalid=!0,!0)}setPixelRatio(e){this._pixelRatio=e,this._colorZonesInvalid=!0}getDOMWidth(){return this._domWidth}getCanvasWidth(){return this._domWidth*this._pixelRatio}setDOMWidth(e){return this._domWidth===e?!1:(this._domWidth=e,this._colorZonesInvalid=!0,!0)}getDOMHeight(){return this._domHeight}getCanvasHeight(){return this._domHeight*this._pixelRatio}setDOMHeight(e){return this._domHeight===e?!1:(this._domHeight=e,this._colorZonesInvalid=!0,!0)}getOuterHeight(){return this._outerHeight}setOuterHeight(e){return this._outerHeight===e?!1:(this._outerHeight=e,this._colorZonesInvalid=!0,!0)}resolveColorZones(){const e=this._colorZonesInvalid,t=Math.floor(this._lineHeight),i=Math.floor(this.getCanvasHeight()),n=Math.floor(this._outerHeight),s=i/n,r=Math.floor(4*this._pixelRatio/2),a=[];for(let l=0,c=this._zones.length;l<c;l++){const d=this._zones[l];if(!e){const w=d.getColorZones();if(w){a.push(w);continue}}const h=this._getVerticalOffsetForLine(d.startLineNumber),u=d.heightInLines===0?this._getVerticalOffsetForLine(d.endLineNumber)+t:h+d.heightInLines*t,g=Math.floor(s*h),f=Math.floor(s*u);let p=Math.floor((g+f)/2),_=f-p;_<r&&(_=r),p-_<0&&(p=_),p+_>i&&(p=i-_);const b=d.color;let C=this._color2Id[b];C||(C=++this._lastAssignedId,this._color2Id[b]=C,this._id2Color[C]=b);const v=new MH(p-_,p+_,C);d.setColorZone(v),a.push(v)}return this._colorZonesInvalid=!1,a.sort(MH.compare),a}}class ybe extends sS{constructor(e,t){super(),this._context=e;const i=this._context.configuration.options;this._domNode=St(document.createElement("canvas")),this._domNode.setClassName(t),this._domNode.setPosition("absolute"),this._domNode.setLayerHinting(!0),this._domNode.setContain("strict"),this._zoneManager=new Sbe(n=>this._context.viewLayout.getVerticalOffsetForLineNumber(n)),this._zoneManager.setDOMWidth(0),this._zoneManager.setDOMHeight(0),this._zoneManager.setOuterHeight(this._context.viewLayout.getScrollHeight()),this._zoneManager.setLineHeight(i.get(67)),this._zoneManager.setPixelRatio(i.get(144)),this._context.addEventHandler(this)}dispose(){this._context.removeEventHandler(this),super.dispose()}onConfigurationChanged(e){const t=this._context.configuration.options;return e.hasChanged(67)&&(this._zoneManager.setLineHeight(t.get(67)),this._render()),e.hasChanged(144)&&(this._zoneManager.setPixelRatio(t.get(144)),this._domNode.setWidth(this._zoneManager.getDOMWidth()),this._domNode.setHeight(this._zoneManager.getDOMHeight()),this._domNode.domNode.width=this._zoneManager.getCanvasWidth(),this._domNode.domNode.height=this._zoneManager.getCanvasHeight(),this._render()),!0}onFlushed(e){return this._render(),!0}onScrollChanged(e){return e.scrollHeightChanged&&(this._zoneManager.setOuterHeight(e.scrollHeight),this._render()),!0}onZonesChanged(e){return this._render(),!0}getDomNode(){return this._domNode.domNode}setLayout(e){this._domNode.setTop(e.top),this._domNode.setRight(e.right);let t=!1;t=this._zoneManager.setDOMWidth(e.width)||t,t=this._zoneManager.setDOMHeight(e.height)||t,t&&(this._domNode.setWidth(this._zoneManager.getDOMWidth()),this._domNode.setHeight(this._zoneManager.getDOMHeight()),this._domNode.domNode.width=this._zoneManager.getCanvasWidth(),this._domNode.domNode.height=this._zoneManager.getCanvasHeight(),this._render())}setZones(e){this._zoneManager.setZones(e),this._render()}_render(){if(this._zoneManager.getOuterHeight()===0)return!1;const e=this._zoneManager.getCanvasWidth(),t=this._zoneManager.getCanvasHeight(),i=this._zoneManager.resolveColorZones(),n=this._zoneManager.getId2Color(),s=this._domNode.domNode.getContext("2d");return s.clearRect(0,0,e,t),i.length>0&&this._renderOneLane(s,i,n,e),!0}_renderOneLane(e,t,i,n){let s=0,r=0,a=0;for(const l of t){const c=l.colorId,d=l.from,h=l.to;c!==s?(e.fillRect(0,r,n,a-r),s=c,e.fillStyle=i[s],r=d,a=h):a>=d?a=Math.max(a,h):(e.fillRect(0,r,n,a-r),r=d,a=h)}e.fillRect(0,r,n,a-r)}}class Lbe extends bo{constructor(e){super(e),this.domNode=St(document.createElement("div")),this.domNode.setAttribute("role","presentation"),this.domNode.setAttribute("aria-hidden","true"),this.domNode.setClassName("view-rulers"),this._renderedRulers=[];const t=this._context.configuration.options;this._rulers=t.get(103),this._typicalHalfwidthCharacterWidth=t.get(50).typicalHalfwidthCharacterWidth}dispose(){super.dispose()}onConfigurationChanged(e){const t=this._context.configuration.options;return this._rulers=t.get(103),this._typicalHalfwidthCharacterWidth=t.get(50).typicalHalfwidthCharacterWidth,!0}onScrollChanged(e){return e.scrollHeightChanged}prepareRender(e){}_ensureRulersCount(){const e=this._renderedRulers.length,t=this._rulers.length;if(e===t)return;if(e<t){const{tabSize:n}=this._context.viewModel.model.getOptions(),s=n;let r=t-e;for(;r>0;){const a=St(document.createElement("div"));a.setClassName("view-ruler"),a.setWidth(s),this.domNode.appendChild(a),this._renderedRulers.push(a),r--}return}let i=e-t;for(;i>0;){const n=this._renderedRulers.pop();this.domNode.removeChild(n),i--}}render(e){this._ensureRulersCount();for(let t=0,i=this._rulers.length;t<i;t++){const n=this._renderedRulers[t],s=this._rulers[t];n.setBoxShadow(s.color?`1px 0 0 0 ${s.color} inset`:""),n.setHeight(Math.min(e.scrollHeight,1e6)),n.setLeft(s.column*this._typicalHalfwidthCharacterWidth)}}}class xbe extends bo{constructor(e){super(e),this._scrollTop=0,this._width=0,this._updateWidth(),this._shouldShow=!1;const i=this._context.configuration.options.get(104);this._useShadows=i.useShadows,this._domNode=St(document.createElement("div")),this._domNode.setAttribute("role","presentation"),this._domNode.setAttribute("aria-hidden","true")}dispose(){super.dispose()}_updateShouldShow(){const e=this._useShadows&&this._scrollTop>0;return this._shouldShow!==e?(this._shouldShow=e,!0):!1}getDomNode(){return this._domNode}_updateWidth(){const t=this._context.configuration.options.get(146);t.minimap.renderMinimap===0||t.minimap.minimapWidth>0&&t.minimap.minimapLeft===0?this._width=t.width:this._width=t.width-t.verticalScrollbarWidth}onConfigurationChanged(e){const i=this._context.configuration.options.get(104);return this._useShadows=i.useShadows,this._updateWidth(),this._updateShouldShow(),!0}onScrollChanged(e){return this._scrollTop=e.scrollTop,this._updateShouldShow()}prepareRender(e){}render(e){this._domNode.setWidth(this._width),this._domNode.setClassName(this._shouldShow?"scroll-decoration":"")}}class kbe{constructor(e){this.left=e.left,this.width=e.width,this.startStyle=null,this.endStyle=null}}class Dbe{constructor(e,t){this.lineNumber=e,this.ranges=t}}function Ibe(o){return new kbe(o)}function Ebe(o){return new Dbe(o.lineNumber,o.ranges.map(Ibe))}const ki=class ki extends lm{constructor(e){super(),this._previousFrameVisibleRangesWithStyle=[],this._context=e;const t=this._context.configuration.options;this._roundedSelection=t.get(102),this._typicalHalfwidthCharacterWidth=t.get(50).typicalHalfwidthCharacterWidth,this._selections=[],this._renderResult=null,this._context.addEventHandler(this)}dispose(){this._context.removeEventHandler(this),this._renderResult=null,super.dispose()}onConfigurationChanged(e){const t=this._context.configuration.options;return this._roundedSelection=t.get(102),this._typicalHalfwidthCharacterWidth=t.get(50).typicalHalfwidthCharacterWidth,!0}onCursorStateChanged(e){return this._selections=e.selections.slice(0),!0}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return e.scrollTopChanged}onZonesChanged(e){return!0}_visibleRangesHaveGaps(e){for(let t=0,i=e.length;t<i;t++)if(e[t].ranges.length>1)return!0;return!1}_enrichVisibleRangesWithStyle(e,t,i){const n=this._typicalHalfwidthCharacterWidth/4;let s=null,r=null;if(i&&i.length>0&&t.length>0){const a=t[0].lineNumber;if(a===e.startLineNumber)for(let c=0;!s&&c<i.length;c++)i[c].lineNumber===a&&(s=i[c].ranges[0]);const l=t[t.length-1].lineNumber;if(l===e.endLineNumber)for(let c=i.length-1;!r&&c>=0;c--)i[c].lineNumber===l&&(r=i[c].ranges[0]);s&&!s.startStyle&&(s=null),r&&!r.startStyle&&(r=null)}for(let a=0,l=t.length;a<l;a++){const c=t[a].ranges[0],d=c.left,h=c.left+c.width,u={top:0,bottom:0},g={top:0,bottom:0};if(a>0){const f=t[a-1].ranges[0].left,p=t[a-1].ranges[0].left+t[a-1].ranges[0].width;l0(d-f)<n?u.top=2:d>f&&(u.top=1),l0(h-p)<n?g.top=2:f<h&&h<p&&(g.top=1)}else s&&(u.top=s.startStyle.top,g.top=s.endStyle.top);if(a+1<l){const f=t[a+1].ranges[0].left,p=t[a+1].ranges[0].left+t[a+1].ranges[0].width;l0(d-f)<n?u.bottom=2:f<d&&d<p&&(u.bottom=1),l0(h-p)<n?g.bottom=2:h<p&&(g.bottom=1)}else r&&(u.bottom=r.startStyle.bottom,g.bottom=r.endStyle.bottom);c.startStyle=u,c.endStyle=g}}_getVisibleRangesWithStyle(e,t,i){const s=(t.linesVisibleRangesForRange(e,!0)||[]).map(Ebe);return!this._visibleRangesHaveGaps(s)&&this._roundedSelection&&this._enrichVisibleRangesWithStyle(t.visibleRange,s,i),s}_createSelectionPiece(e,t,i,n,s){return'<div class="cslr '+i+'" style="top:'+e.toString()+"px;bottom:"+t.toString()+"px;left:"+n.toString()+"px;width:"+s.toString()+'px;"></div>'}_actualRenderOneSelection(e,t,i,n){if(n.length===0)return;const s=!!n[0].ranges[0].startStyle,r=n[0].lineNumber,a=n[n.length-1].lineNumber;for(let l=0,c=n.length;l<c;l++){const d=n[l],h=d.lineNumber,u=h-t,g=i&&h===r?1:0,f=i&&h!==r&&h===a?1:0;let p="",_="";for(let b=0,C=d.ranges.length;b<C;b++){const v=d.ranges[b];if(s){const S=v.startStyle,L=v.endStyle;if(S.top===1||S.bottom===1){p+=this._createSelectionPiece(g,f,ki.SELECTION_CLASS_NAME,v.left-ki.ROUNDED_PIECE_WIDTH,ki.ROUNDED_PIECE_WIDTH);let k=ki.EDITOR_BACKGROUND_CLASS_NAME;S.top===1&&(k+=" "+ki.SELECTION_TOP_RIGHT),S.bottom===1&&(k+=" "+ki.SELECTION_BOTTOM_RIGHT),p+=this._createSelectionPiece(g,f,k,v.left-ki.ROUNDED_PIECE_WIDTH,ki.ROUNDED_PIECE_WIDTH)}if(L.top===1||L.bottom===1){p+=this._createSelectionPiece(g,f,ki.SELECTION_CLASS_NAME,v.left+v.width,ki.ROUNDED_PIECE_WIDTH);let k=ki.EDITOR_BACKGROUND_CLASS_NAME;L.top===1&&(k+=" "+ki.SELECTION_TOP_LEFT),L.bottom===1&&(k+=" "+ki.SELECTION_BOTTOM_LEFT),p+=this._createSelectionPiece(g,f,k,v.left+v.width,ki.ROUNDED_PIECE_WIDTH)}}let w=ki.SELECTION_CLASS_NAME;if(s){const S=v.startStyle,L=v.endStyle;S.top===0&&(w+=" "+ki.SELECTION_TOP_LEFT),S.bottom===0&&(w+=" "+ki.SELECTION_BOTTOM_LEFT),L.top===0&&(w+=" "+ki.SELECTION_TOP_RIGHT),L.bottom===0&&(w+=" "+ki.SELECTION_BOTTOM_RIGHT)}_+=this._createSelectionPiece(g,f,w,v.left,v.width)}e[u][0]+=p,e[u][1]+=_}}prepareRender(e){const t=[],i=e.visibleRange.startLineNumber,n=e.visibleRange.endLineNumber;for(let r=i;r<=n;r++){const a=r-i;t[a]=["",""]}const s=[];for(let r=0,a=this._selections.length;r<a;r++){const l=this._selections[r];if(l.isEmpty()){s[r]=null;continue}const c=this._getVisibleRangesWithStyle(l,e,this._previousFrameVisibleRangesWithStyle[r]);s[r]=c,this._actualRenderOneSelection(t,i,this._selections.length>1,c)}this._previousFrameVisibleRangesWithStyle=s,this._renderResult=t.map(([r,a])=>r+a)}render(e,t){if(!this._renderResult)return"";const i=t-e;return i<0||i>=this._renderResult.length?"":this._renderResult[i]}};ki.SELECTION_CLASS_NAME="selected-text",ki.SELECTION_TOP_LEFT="top-left-radius",ki.SELECTION_BOTTOM_LEFT="bottom-left-radius",ki.SELECTION_TOP_RIGHT="top-right-radius",ki.SELECTION_BOTTOM_RIGHT="bottom-right-radius",ki.EDITOR_BACKGROUND_CLASS_NAME="monaco-editor-background",ki.ROUNDED_PIECE_WIDTH=10;let _P=ki;Lr((o,e)=>{const t=o.getColor(Boe);t&&!t.isTransparent()&&e.addRule(`.monaco-editor .view-line span.inline-selected-text { color: ${t}; }`)});function l0(o){return o<0?-o:o}class AH{constructor(e,t,i,n,s,r,a){this.top=e,this.left=t,this.paddingLeft=i,this.width=n,this.height=s,this.textContent=r,this.textContentClassName=a}}var Fc;(function(o){o[o.Single=0]="Single",o[o.MultiPrimary=1]="MultiPrimary",o[o.MultiSecondary=2]="MultiSecondary"})(Fc||(Fc={}));class PH{constructor(e,t){this._context=e;const i=this._context.configuration.options,n=i.get(50);this._cursorStyle=i.get(28),this._lineHeight=i.get(67),this._typicalHalfwidthCharacterWidth=n.typicalHalfwidthCharacterWidth,this._lineCursorWidth=Math.min(i.get(31),this._typicalHalfwidthCharacterWidth),this._isVisible=!0,this._domNode=St(document.createElement("div")),this._domNode.setClassName(`cursor ${s_}`),this._domNode.setHeight(this._lineHeight),this._domNode.setTop(0),this._domNode.setLeft(0),bn(this._domNode,n),this._domNode.setDisplay("none"),this._position=new A(1,1),this._pluralityClass="",this.setPlurality(t),this._lastRenderedContent="",this._renderData=null}getDomNode(){return this._domNode}getPosition(){return this._position}setPlurality(e){switch(e){default:case Fc.Single:this._pluralityClass="";break;case Fc.MultiPrimary:this._pluralityClass="cursor-primary";break;case Fc.MultiSecondary:this._pluralityClass="cursor-secondary";break}}show(){this._isVisible||(this._domNode.setVisibility("inherit"),this._isVisible=!0)}hide(){this._isVisible&&(this._domNode.setVisibility("hidden"),this._isVisible=!1)}onConfigurationChanged(e){const t=this._context.configuration.options,i=t.get(50);return this._cursorStyle=t.get(28),this._lineHeight=t.get(67),this._typicalHalfwidthCharacterWidth=i.typicalHalfwidthCharacterWidth,this._lineCursorWidth=Math.min(t.get(31),this._typicalHalfwidthCharacterWidth),bn(this._domNode,i),!0}onCursorPositionChanged(e,t){return t?this._domNode.domNode.style.transitionProperty="none":this._domNode.domNode.style.transitionProperty="",this._position=e,!0}_getGraphemeAwarePosition(){const{lineNumber:e,column:t}=this._position,i=this._context.viewModel.getLineContent(e),[n,s]=Pee(i,t-1);return[new A(e,n+1),i.substring(n,s)]}_prepareRender(e){let t="",i="";const[n,s]=this._getGraphemeAwarePosition();if(this._cursorStyle===fn.Line||this._cursorStyle===fn.LineThin){const u=e.visibleRangeForPosition(n);if(!u||u.outsideRenderedLine)return null;const g=xe(this._domNode.domNode);let f;this._cursorStyle===fn.Line?(f=h7(g,this._lineCursorWidth>0?this._lineCursorWidth:2),f>2&&(t=s,i=this._getTokenClassName(n))):f=h7(g,1);let p=u.left,_=0;f>=2&&p>=1&&(_=1,p-=_);const b=e.getVerticalOffsetForLineNumber(n.lineNumber)-e.bigNumbersDelta;return new AH(b,p,_,f,this._lineHeight,t,i)}const r=e.linesVisibleRangesForRange(new x(n.lineNumber,n.column,n.lineNumber,n.column+s.length),!1);if(!r||r.length===0)return null;const a=r[0];if(a.outsideRenderedLine||a.ranges.length===0)return null;const l=a.ranges[0],c=s==="	"?this._typicalHalfwidthCharacterWidth:l.width<1?this._typicalHalfwidthCharacterWidth:l.width;this._cursorStyle===fn.Block&&(t=s,i=this._getTokenClassName(n));let d=e.getVerticalOffsetForLineNumber(n.lineNumber)-e.bigNumbersDelta,h=this._lineHeight;return(this._cursorStyle===fn.Underline||this._cursorStyle===fn.UnderlineThin)&&(d+=this._lineHeight-2,h=2),new AH(d,l.left,0,c,h,t,i)}_getTokenClassName(e){const t=this._context.viewModel.getViewLineData(e.lineNumber),i=t.tokens.findTokenIndexAtOffset(e.column-1);return t.tokens.getClassName(i)}prepareRender(e){this._renderData=this._prepareRender(e)}render(e){return this._renderData?(this._lastRenderedContent!==this._renderData.textContent&&(this._lastRenderedContent=this._renderData.textContent,this._domNode.domNode.textContent=this._lastRenderedContent),this._domNode.setClassName(`cursor ${this._pluralityClass} ${s_} ${this._renderData.textContentClassName}`),this._domNode.setDisplay("block"),this._domNode.setTop(this._renderData.top),this._domNode.setLeft(this._renderData.left),this._domNode.setPaddingLeft(this._renderData.paddingLeft),this._domNode.setWidth(this._renderData.width),this._domNode.setLineHeight(this._renderData.height),this._domNode.setHeight(this._renderData.height),{domNode:this._domNode.domNode,position:this._position,contentLeft:this._renderData.left,height:this._renderData.height,width:2}):(this._domNode.setDisplay("none"),null)}}const Vv=class Vv extends bo{constructor(e){super(e);const t=this._context.configuration.options;this._readOnly=t.get(92),this._cursorBlinking=t.get(26),this._cursorStyle=t.get(28),this._cursorSmoothCaretAnimation=t.get(27),this._selectionIsEmpty=!0,this._isComposingInput=!1,this._isVisible=!1,this._primaryCursor=new PH(this._context,Fc.Single),this._secondaryCursors=[],this._renderData=[],this._domNode=St(document.createElement("div")),this._domNode.setAttribute("role","presentation"),this._domNode.setAttribute("aria-hidden","true"),this._updateDomClassName(),this._domNode.appendChild(this._primaryCursor.getDomNode()),this._startCursorBlinkAnimation=new ea,this._cursorFlatBlinkInterval=new r5,this._blinkingEnabled=!1,this._editorHasFocus=!1,this._updateBlinking()}dispose(){super.dispose(),this._startCursorBlinkAnimation.dispose(),this._cursorFlatBlinkInterval.dispose()}getDomNode(){return this._domNode}onCompositionStart(e){return this._isComposingInput=!0,this._updateBlinking(),!0}onCompositionEnd(e){return this._isComposingInput=!1,this._updateBlinking(),!0}onConfigurationChanged(e){const t=this._context.configuration.options;this._readOnly=t.get(92),this._cursorBlinking=t.get(26),this._cursorStyle=t.get(28),this._cursorSmoothCaretAnimation=t.get(27),this._updateBlinking(),this._updateDomClassName(),this._primaryCursor.onConfigurationChanged(e);for(let i=0,n=this._secondaryCursors.length;i<n;i++)this._secondaryCursors[i].onConfigurationChanged(e);return!0}_onCursorPositionChanged(e,t,i){const n=this._secondaryCursors.length!==t.length||this._cursorSmoothCaretAnimation==="explicit"&&i!==3;if(this._primaryCursor.setPlurality(t.length?Fc.MultiPrimary:Fc.Single),this._primaryCursor.onCursorPositionChanged(e,n),this._updateBlinking(),this._secondaryCursors.length<t.length){const s=t.length-this._secondaryCursors.length;for(let r=0;r<s;r++){const a=new PH(this._context,Fc.MultiSecondary);this._domNode.domNode.insertBefore(a.getDomNode().domNode,this._primaryCursor.getDomNode().domNode.nextSibling),this._secondaryCursors.push(a)}}else if(this._secondaryCursors.length>t.length){const s=this._secondaryCursors.length-t.length;for(let r=0;r<s;r++)this._domNode.removeChild(this._secondaryCursors[0].getDomNode()),this._secondaryCursors.splice(0,1)}for(let s=0;s<t.length;s++)this._secondaryCursors[s].onCursorPositionChanged(t[s],n)}onCursorStateChanged(e){const t=[];for(let n=0,s=e.selections.length;n<s;n++)t[n]=e.selections[n].getPosition();this._onCursorPositionChanged(t[0],t.slice(1),e.reason);const i=e.selections[0].isEmpty();return this._selectionIsEmpty!==i&&(this._selectionIsEmpty=i,this._updateDomClassName()),!0}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onFocusChanged(e){return this._editorHasFocus=e.isFocused,this._updateBlinking(),!1}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return!0}onTokensChanged(e){const t=i=>{for(let n=0,s=e.ranges.length;n<s;n++)if(e.ranges[n].fromLineNumber<=i.lineNumber&&i.lineNumber<=e.ranges[n].toLineNumber)return!0;return!1};if(t(this._primaryCursor.getPosition()))return!0;for(const i of this._secondaryCursors)if(t(i.getPosition()))return!0;return!1}onZonesChanged(e){return!0}_getCursorBlinking(){return this._isComposingInput||!this._editorHasFocus?0:this._readOnly?5:this._cursorBlinking}_updateBlinking(){this._startCursorBlinkAnimation.cancel(),this._cursorFlatBlinkInterval.cancel();const e=this._getCursorBlinking(),t=e===0,i=e===5;t?this._hide():this._show(),this._blinkingEnabled=!1,this._updateDomClassName(),!t&&!i&&(e===1?this._cursorFlatBlinkInterval.cancelAndSet(()=>{this._isVisible?this._hide():this._show()},Vv.BLINK_INTERVAL,xe(this._domNode.domNode)):this._startCursorBlinkAnimation.setIfNotSet(()=>{this._blinkingEnabled=!0,this._updateDomClassName()},Vv.BLINK_INTERVAL))}_updateDomClassName(){this._domNode.setClassName(this._getClassName())}_getClassName(){let e="cursors-layer";switch(this._selectionIsEmpty||(e+=" has-selection"),this._cursorStyle){case fn.Line:e+=" cursor-line-style";break;case fn.Block:e+=" cursor-block-style";break;case fn.Underline:e+=" cursor-underline-style";break;case fn.LineThin:e+=" cursor-line-thin-style";break;case fn.BlockOutline:e+=" cursor-block-outline-style";break;case fn.UnderlineThin:e+=" cursor-underline-thin-style";break;default:e+=" cursor-line-style"}if(this._blinkingEnabled)switch(this._getCursorBlinking()){case 1:e+=" cursor-blink";break;case 2:e+=" cursor-smooth";break;case 3:e+=" cursor-phase";break;case 4:e+=" cursor-expand";break;case 5:e+=" cursor-solid";break;default:e+=" cursor-solid"}else e+=" cursor-solid";return(this._cursorSmoothCaretAnimation==="on"||this._cursorSmoothCaretAnimation==="explicit")&&(e+=" cursor-smooth-caret-animation"),e}_show(){this._primaryCursor.show();for(let e=0,t=this._secondaryCursors.length;e<t;e++)this._secondaryCursors[e].show();this._isVisible=!0}_hide(){this._primaryCursor.hide();for(let e=0,t=this._secondaryCursors.length;e<t;e++)this._secondaryCursors[e].hide();this._isVisible=!1}prepareRender(e){this._primaryCursor.prepareRender(e);for(let t=0,i=this._secondaryCursors.length;t<i;t++)this._secondaryCursors[t].prepareRender(e)}render(e){const t=[];let i=0;const n=this._primaryCursor.render(e);n&&(t[i++]=n);for(let s=0,r=this._secondaryCursors.length;s<r;s++){const a=this._secondaryCursors[s].render(e);a&&(t[i++]=a)}this._renderData=t}getLastRenderData(){return this._renderData}};Vv.BLINK_INTERVAL=500;let bP=Vv;Lr((o,e)=>{const t=[{class:".cursor",foreground:oE,background:s3},{class:".cursor-primary",foreground:Hj,background:the},{class:".cursor-secondary",foreground:Vj,background:ihe}];for(const i of t){const n=o.getColor(i.foreground);if(n){let s=o.getColor(i.background);s||(s=n.opposite()),e.addRule(`.monaco-editor .cursors-layer ${i.class} { background-color: ${n}; border-color: ${n}; color: ${s}; }`),Ta(o.type)&&e.addRule(`.monaco-editor .cursors-layer.has-selection ${i.class} { border-left: 1px solid ${s}; border-right: 1px solid ${s}; }`)}}});const ST=()=>{throw new Error("Invalid change accessor")};class Nbe extends bo{constructor(e){super(e);const t=this._context.configuration.options,i=t.get(146);this._lineHeight=t.get(67),this._contentWidth=i.contentWidth,this._contentLeft=i.contentLeft,this.domNode=St(document.createElement("div")),this.domNode.setClassName("view-zones"),this.domNode.setPosition("absolute"),this.domNode.setAttribute("role","presentation"),this.domNode.setAttribute("aria-hidden","true"),this.marginDomNode=St(document.createElement("div")),this.marginDomNode.setClassName("margin-view-zones"),this.marginDomNode.setPosition("absolute"),this.marginDomNode.setAttribute("role","presentation"),this.marginDomNode.setAttribute("aria-hidden","true"),this._zones={}}dispose(){super.dispose(),this._zones={}}_recomputeWhitespacesProps(){const e=this._context.viewLayout.getWhitespaces(),t=new Map;for(const n of e)t.set(n.id,n);let i=!1;return this._context.viewModel.changeWhitespace(n=>{const s=Object.keys(this._zones);for(let r=0,a=s.length;r<a;r++){const l=s[r],c=this._zones[l],d=this._computeWhitespaceProps(c.delegate);c.isInHiddenArea=d.isInHiddenArea;const h=t.get(l);h&&(h.afterLineNumber!==d.afterViewLineNumber||h.height!==d.heightInPx)&&(n.changeOneWhitespace(l,d.afterViewLineNumber,d.heightInPx),this._safeCallOnComputedHeight(c.delegate,d.heightInPx),i=!0)}}),i}onConfigurationChanged(e){const t=this._context.configuration.options,i=t.get(146);return this._lineHeight=t.get(67),this._contentWidth=i.contentWidth,this._contentLeft=i.contentLeft,e.hasChanged(67)&&this._recomputeWhitespacesProps(),!0}onLineMappingChanged(e){return this._recomputeWhitespacesProps()}onLinesDeleted(e){return!0}onScrollChanged(e){return e.scrollTopChanged||e.scrollWidthChanged}onZonesChanged(e){return!0}onLinesInserted(e){return!0}_getZoneOrdinal(e){return e.ordinal??e.afterColumn??1e4}_computeWhitespaceProps(e){if(e.afterLineNumber===0)return{isInHiddenArea:!1,afterViewLineNumber:0,heightInPx:this._heightInPixels(e),minWidthInPx:this._minWidthInPixels(e)};let t;if(typeof e.afterColumn<"u")t=this._context.viewModel.model.validatePosition({lineNumber:e.afterLineNumber,column:e.afterColumn});else{const r=this._context.viewModel.model.validatePosition({lineNumber:e.afterLineNumber,column:1}).lineNumber;t=new A(r,this._context.viewModel.model.getLineMaxColumn(r))}let i;t.column===this._context.viewModel.model.getLineMaxColumn(t.lineNumber)?i=this._context.viewModel.model.validatePosition({lineNumber:t.lineNumber+1,column:1}):i=this._context.viewModel.model.validatePosition({lineNumber:t.lineNumber,column:t.column+1});const n=this._context.viewModel.coordinatesConverter.convertModelPositionToViewPosition(t,e.afterColumnAffinity,!0),s=e.showInHiddenAreas||this._context.viewModel.coordinatesConverter.modelPositionIsVisible(i);return{isInHiddenArea:!s,afterViewLineNumber:n.lineNumber,heightInPx:s?this._heightInPixels(e):0,minWidthInPx:this._minWidthInPixels(e)}}changeViewZones(e){let t=!1;return this._context.viewModel.changeWhitespace(i=>{const n={addZone:s=>(t=!0,this._addZone(i,s)),removeZone:s=>{s&&(t=this._removeZone(i,s)||t)},layoutZone:s=>{s&&(t=this._layoutZone(i,s)||t)}};Tbe(e,n),n.addZone=ST,n.removeZone=ST,n.layoutZone=ST}),t}_addZone(e,t){const i=this._computeWhitespaceProps(t),s={whitespaceId:e.insertWhitespace(i.afterViewLineNumber,this._getZoneOrdinal(t),i.heightInPx,i.minWidthInPx),delegate:t,isInHiddenArea:i.isInHiddenArea,isVisible:!1,domNode:St(t.domNode),marginDomNode:t.marginDomNode?St(t.marginDomNode):null};return this._safeCallOnComputedHeight(s.delegate,i.heightInPx),s.domNode.setPosition("absolute"),s.domNode.domNode.style.width="100%",s.domNode.setDisplay("none"),s.domNode.setAttribute("monaco-view-zone",s.whitespaceId),this.domNode.appendChild(s.domNode),s.marginDomNode&&(s.marginDomNode.setPosition("absolute"),s.marginDomNode.domNode.style.width="100%",s.marginDomNode.setDisplay("none"),s.marginDomNode.setAttribute("monaco-view-zone",s.whitespaceId),this.marginDomNode.appendChild(s.marginDomNode)),this._zones[s.whitespaceId]=s,this.setShouldRender(),s.whitespaceId}_removeZone(e,t){if(this._zones.hasOwnProperty(t)){const i=this._zones[t];return delete this._zones[t],e.removeWhitespace(i.whitespaceId),i.domNode.removeAttribute("monaco-visible-view-zone"),i.domNode.removeAttribute("monaco-view-zone"),i.domNode.domNode.remove(),i.marginDomNode&&(i.marginDomNode.removeAttribute("monaco-visible-view-zone"),i.marginDomNode.removeAttribute("monaco-view-zone"),i.marginDomNode.domNode.remove()),this.setShouldRender(),!0}return!1}_layoutZone(e,t){if(this._zones.hasOwnProperty(t)){const i=this._zones[t],n=this._computeWhitespaceProps(i.delegate);return i.isInHiddenArea=n.isInHiddenArea,e.changeOneWhitespace(i.whitespaceId,n.afterViewLineNumber,n.heightInPx),this._safeCallOnComputedHeight(i.delegate,n.heightInPx),this.setShouldRender(),!0}return!1}shouldSuppressMouseDownOnViewZone(e){return this._zones.hasOwnProperty(e)?!!this._zones[e].delegate.suppressMouseDown:!1}_heightInPixels(e){return typeof e.heightInPx=="number"?e.heightInPx:typeof e.heightInLines=="number"?this._lineHeight*e.heightInLines:this._lineHeight}_minWidthInPixels(e){return typeof e.minWidthInPx=="number"?e.minWidthInPx:0}_safeCallOnComputedHeight(e,t){if(typeof e.onComputedHeight=="function")try{e.onComputedHeight(t)}catch(i){Oe(i)}}_safeCallOnDomNodeTop(e,t){if(typeof e.onDomNodeTop=="function")try{e.onDomNodeTop(t)}catch(i){Oe(i)}}prepareRender(e){}render(e){const t=e.viewportData.whitespaceViewportData,i={};let n=!1;for(const r of t)this._zones[r.id].isInHiddenArea||(i[r.id]=r,n=!0);const s=Object.keys(this._zones);for(let r=0,a=s.length;r<a;r++){const l=s[r],c=this._zones[l];let d=0,h=0,u="none";i.hasOwnProperty(l)?(d=i[l].verticalOffset-e.bigNumbersDelta,h=i[l].height,u="block",c.isVisible||(c.domNode.setAttribute("monaco-visible-view-zone","true"),c.isVisible=!0),this._safeCallOnDomNodeTop(c.delegate,e.getScrolledTopFromAbsoluteTop(i[l].verticalOffset))):(c.isVisible&&(c.domNode.removeAttribute("monaco-visible-view-zone"),c.isVisible=!1),this._safeCallOnDomNodeTop(c.delegate,e.getScrolledTopFromAbsoluteTop(-1e6))),c.domNode.setTop(d),c.domNode.setHeight(h),c.domNode.setDisplay(u),c.marginDomNode&&(c.marginDomNode.setTop(d),c.marginDomNode.setHeight(h),c.marginDomNode.setDisplay(u))}n&&(this.domNode.setWidth(Math.max(e.scrollWidth,this._contentWidth)),this.marginDomNode.setWidth(this._contentLeft))}}function Tbe(o,e){try{return o(e)}catch(t){Oe(t)}}class Rbe extends lm{constructor(e){super(),this._context=e,this._options=new OH(this._context.configuration),this._selection=[],this._renderResult=null,this._context.addEventHandler(this)}dispose(){this._context.removeEventHandler(this),this._renderResult=null,super.dispose()}onConfigurationChanged(e){const t=new OH(this._context.configuration);return this._options.equals(t)?e.hasChanged(146):(this._options=t,!0)}onCursorStateChanged(e){return this._selection=e.selections,this._options.renderWhitespace==="selection"}onDecorationsChanged(e){return!0}onFlushed(e){return!0}onLinesChanged(e){return!0}onLinesDeleted(e){return!0}onLinesInserted(e){return!0}onScrollChanged(e){return e.scrollTopChanged}onZonesChanged(e){return!0}prepareRender(e){if(this._options.renderWhitespace==="none"){this._renderResult=null;return}const t=e.visibleRange.startLineNumber,n=e.visibleRange.endLineNumber-t+1,s=new Array(n);for(let a=0;a<n;a++)s[a]=!0;const r=this._context.viewModel.getMinimapLinesRenderingData(e.viewportData.startLineNumber,e.viewportData.endLineNumber,s);this._renderResult=[];for(let a=e.viewportData.startLineNumber;a<=e.viewportData.endLineNumber;a++){const l=a-e.viewportData.startLineNumber,c=r.data[l];let d=null;if(this._options.renderWhitespace==="selection"){const h=this._selection;for(const u of h){if(u.endLineNumber<a||u.startLineNumber>a)continue;const g=u.startLineNumber===a?u.startColumn:c.minColumn,f=u.endLineNumber===a?u.endColumn:c.maxColumn;g<f&&(d||(d=[]),d.push(new lq(g-1,f-1)))}}this._renderResult[l]=this._applyRenderWhitespace(e,a,d,c)}}_applyRenderWhitespace(e,t,i,n){if(this._options.renderWhitespace==="selection"&&!i||this._options.renderWhitespace==="trailing"&&n.continuesWithWrappedLine)return"";const s=this._context.theme.getColor(o3),r=this._options.renderWithSVG,a=n.content,l=this._options.stopRenderingLineAfter===-1?a.length:Math.min(this._options.stopRenderingLineAfter,a.length),c=n.continuesWithWrappedLine,d=n.minColumn-1,h=this._options.renderWhitespace==="boundary",u=this._options.renderWhitespace==="trailing",g=this._options.lineHeight,f=this._options.middotWidth,p=this._options.wsmiddotWidth,_=this._options.spaceWidth,b=Math.abs(p-_),C=Math.abs(f-_),v=b<C?11825:183,w=this._options.canUseHalfwidthRightwardsArrow;let S="",L=!1,k=Jn(a),D;k===-1?(L=!0,k=l,D=l):D=Na(a);let N=0,B=i&&i[N],W=0;for(let V=d;V<l;V++){const $=a.charCodeAt(V);if(B&&V>=B.endOffset&&(N++,B=i&&i[N]),$!==9&&$!==32||u&&!L&&V<=D)continue;if(h&&V>=k&&V<=D&&$===32){const j=V-1>=0?a.charCodeAt(V-1):0,J=V+1<l?a.charCodeAt(V+1):0;if(j!==32&&J!==32)continue}if(h&&c&&V===l-1){const j=V-1>=0?a.charCodeAt(V-1):0;if($===32&&j!==32&&j!==9)continue}if(i&&(!B||B.startOffset>V||B.endOffset<=V))continue;const H=e.visibleRangeForPosition(new A(t,V+1));H&&(r?(W=Math.max(W,H.left),$===9?S+=this._renderArrow(g,_,H.left):S+=`<circle cx="${(H.left+_/2).toFixed(2)}" cy="${(g/2).toFixed(2)}" r="${(_/7).toFixed(2)}" />`):$===9?S+=`<div class="mwh" style="left:${H.left}px;height:${g}px;">${w?"￫":"→"}</div>`:S+=`<div class="mwh" style="left:${H.left}px;height:${g}px;">${String.fromCharCode(v)}</div>`)}return r?(W=Math.round(W+_),`<svg style="bottom:0;position:absolute;width:${W}px;height:${g}px" viewBox="0 0 ${W} ${g}" xmlns="http://www.w3.org/2000/svg" fill="${s}">`+S+"</svg>"):S}_renderArrow(e,t,i){const n=t/7,s=t,r=e/2,a=i,l={x:0,y:n/2},c={x:100/125*s,y:l.y},d={x:c.x-.2*c.x,y:c.y+.2*c.x},h={x:d.x+.1*c.x,y:d.y+.1*c.x},u={x:h.x+.35*c.x,y:h.y-.35*c.x},g={x:u.x,y:-u.y},f={x:h.x,y:-h.y},p={x:d.x,y:-d.y},_={x:c.x,y:-c.y},b={x:l.x,y:-l.y};return`<path d="M ${[l,c,d,h,u,g,f,p,_,b].map(w=>`${(a+w.x).toFixed(2)} ${(r+w.y).toFixed(2)}`).join(" L ")}" />`}render(e,t){if(!this._renderResult)return"";const i=t-e;return i<0||i>=this._renderResult.length?"":this._renderResult[i]}}class OH{constructor(e){const t=e.options,i=t.get(50),n=t.get(38);n==="off"?(this.renderWhitespace="none",this.renderWithSVG=!1):n==="svg"?(this.renderWhitespace=t.get(100),this.renderWithSVG=!0):(this.renderWhitespace=t.get(100),this.renderWithSVG=!1),this.spaceWidth=i.spaceWidth,this.middotWidth=i.middotWidth,this.wsmiddotWidth=i.wsmiddotWidth,this.canUseHalfwidthRightwardsArrow=i.canUseHalfwidthRightwardsArrow,this.lineHeight=t.get(67),this.stopRenderingLineAfter=t.get(118)}equals(e){return this.renderWhitespace===e.renderWhitespace&&this.renderWithSVG===e.renderWithSVG&&this.spaceWidth===e.spaceWidth&&this.middotWidth===e.middotWidth&&this.wsmiddotWidth===e.wsmiddotWidth&&this.canUseHalfwidthRightwardsArrow===e.canUseHalfwidthRightwardsArrow&&this.lineHeight===e.lineHeight&&this.stopRenderingLineAfter===e.stopRenderingLineAfter}}class Mbe{constructor(e,t,i,n){this.selections=e,this.startLineNumber=t.startLineNumber|0,this.endLineNumber=t.endLineNumber|0,this.relativeVerticalOffset=t.relativeVerticalOffset,this.bigNumbersDelta=t.bigNumbersDelta|0,this.lineHeight=t.lineHeight|0,this.whitespaceViewportData=i,this._model=n,this.visibleRange=new x(t.startLineNumber,this._model.getLineMinColumn(t.startLineNumber),t.endLineNumber,this._model.getLineMaxColumn(t.endLineNumber))}getViewLineRenderingData(e){return this._model.getViewportViewLineRenderingData(this.visibleRange,e)}getDecorationsInViewport(){return this._model.getDecorationsInViewport(this.visibleRange)}}class Abe{get type(){return this._theme.type}get value(){return this._theme}constructor(e){this._theme=e}update(e){this._theme=e}getColor(e){return this._theme.getColor(e)}}class Pbe{constructor(e,t,i){this.configuration=e,this.theme=new Abe(t),this.viewModel=i,this.viewLayout=i.viewLayout}addEventHandler(e){this.viewModel.addViewEventHandler(e)}removeEventHandler(e){this.viewModel.removeViewEventHandler(e)}}var Obe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Fbe=function(o,e){return function(t,i){e(t,i,o)}};let CP=class extends sS{constructor(e,t,i,n,s,r,a){super(),this._instantiationService=a,this._shouldRecomputeGlyphMarginLanes=!1,this._selections=[new pe(1,1,1,1)],this._renderAnimationFrame=null;const l=new j_e(t,n,s,e);this._context=new Pbe(t,i,n),this._context.addEventHandler(this),this._viewParts=[],this._textAreaHandler=this._instantiationService.createInstance(hP,this._context,l,this._createTextAreaHandlerHelper()),this._viewParts.push(this._textAreaHandler),this._linesContent=St(document.createElement("div")),this._linesContent.setClassName("lines-content monaco-editor-background"),this._linesContent.setPosition("absolute"),this.domNode=St(document.createElement("div")),this.domNode.setClassName(this._getEditorClassName()),this.domNode.setAttribute("role","code"),this._overflowGuardContainer=St(document.createElement("div")),Va.write(this._overflowGuardContainer,3),this._overflowGuardContainer.setClassName("overflow-guard"),this._scrollbar=new tbe(this._context,this._linesContent,this.domNode,this._overflowGuardContainer),this._viewParts.push(this._scrollbar),this._viewLines=new pP(this._context,this._linesContent),this._viewZones=new Nbe(this._context),this._viewParts.push(this._viewZones);const c=new wbe(this._context);this._viewParts.push(c);const d=new xbe(this._context);this._viewParts.push(d);const h=new q_e(this._context);this._viewParts.push(h),h.addDynamicOverlay(new Q_e(this._context)),h.addDynamicOverlay(new _P(this._context)),h.addDynamicOverlay(new lbe(this._context)),h.addDynamicOverlay(new ebe(this._context)),h.addDynamicOverlay(new Rbe(this._context));const u=new G_e(this._context);this._viewParts.push(u),u.addDynamicOverlay(new J_e(this._context)),u.addDynamicOverlay(new gbe(this._context)),u.addDynamicOverlay(new ube(this._context)),u.addDynamicOverlay(new rx(this._context)),this._glyphMarginWidgets=new sbe(this._context),this._viewParts.push(this._glyphMarginWidgets);const g=new ax(this._context);g.getDomNode().appendChild(this._viewZones.marginDomNode),g.getDomNode().appendChild(u.getDomNode()),g.getDomNode().appendChild(this._glyphMarginWidgets.domNode),this._viewParts.push(g),this._contentWidgets=new Y_e(this._context,this.domNode),this._viewParts.push(this._contentWidgets),this._viewCursors=new bP(this._context),this._viewParts.push(this._viewCursors),this._overlayWidgets=new Cbe(this._context,this.domNode),this._viewParts.push(this._overlayWidgets);const f=new Lbe(this._context);this._viewParts.push(f);const p=new Z_e(this._context);this._viewParts.push(p);const _=new bbe(this._context);if(this._viewParts.push(_),c){const b=this._scrollbar.getOverviewRulerLayoutInfo();b.parent.insertBefore(c.getDomNode(),b.insertBefore)}this._linesContent.appendChild(h.getDomNode()),this._linesContent.appendChild(f.domNode),this._linesContent.appendChild(this._viewZones.domNode),this._linesContent.appendChild(this._viewLines.getDomNode()),this._linesContent.appendChild(this._contentWidgets.domNode),this._linesContent.appendChild(this._viewCursors.getDomNode()),this._overflowGuardContainer.appendChild(g.getDomNode()),this._overflowGuardContainer.appendChild(this._scrollbar.getDomNode()),this._overflowGuardContainer.appendChild(d.getDomNode()),this._overflowGuardContainer.appendChild(this._textAreaHandler.textArea),this._overflowGuardContainer.appendChild(this._textAreaHandler.textAreaCover),this._overflowGuardContainer.appendChild(this._overlayWidgets.getDomNode()),this._overflowGuardContainer.appendChild(_.getDomNode()),this._overflowGuardContainer.appendChild(p.domNode),this.domNode.appendChild(this._overflowGuardContainer),r?(r.appendChild(this._contentWidgets.overflowingContentWidgetsDomNode.domNode),r.appendChild(this._overlayWidgets.overflowingOverlayWidgetsDomNode.domNode)):(this.domNode.appendChild(this._contentWidgets.overflowingContentWidgetsDomNode),this.domNode.appendChild(this._overlayWidgets.overflowingOverlayWidgetsDomNode)),this._applyLayout(),this._pointerHandler=this._register(new g_e(this._context,l,this._createPointerHandlerHelper()))}_computeGlyphMarginLanes(){const e=this._context.viewModel.model,t=this._context.viewModel.glyphLanes;let i=[],n=0;i=i.concat(e.getAllMarginDecorations().map(s=>{var a,l;const r=((a=s.options.glyphMargin)==null?void 0:a.position)??Zr.Center;return n=Math.max(n,s.range.endLineNumber),{range:s.range,lane:r,persist:(l=s.options.glyphMargin)==null?void 0:l.persistLane}})),i=i.concat(this._glyphMarginWidgets.getWidgets().map(s=>{const r=e.validateRange(s.preference.range);return n=Math.max(n,r.endLineNumber),{range:r,lane:s.preference.lane}})),i.sort((s,r)=>x.compareRangesUsingStarts(s.range,r.range)),t.reset(n);for(const s of i)t.push(s.lane,s.range,s.persist);return t}_createPointerHandlerHelper(){return{viewDomNode:this.domNode.domNode,linesContentDomNode:this._linesContent.domNode,viewLinesDomNode:this._viewLines.getDomNode().domNode,focusTextArea:()=>{this.focus()},dispatchTextAreaEvent:e=>{this._textAreaHandler.textArea.domNode.dispatchEvent(e)},getLastRenderData:()=>{const e=this._viewCursors.getLastRenderData()||[],t=this._textAreaHandler.getLastRenderData();return new t_e(e,t)},renderNow:()=>{this.render(!0,!1)},shouldSuppressMouseDownOnViewZone:e=>this._viewZones.shouldSuppressMouseDownOnViewZone(e),shouldSuppressMouseDownOnWidget:e=>this._contentWidgets.shouldSuppressMouseDownOnWidget(e),getPositionFromDOMInfo:(e,t)=>(this._flushAccumulatedAndRenderNow(),this._viewLines.getPositionFromDOMInfo(e,t)),visibleRangeForPosition:(e,t)=>(this._flushAccumulatedAndRenderNow(),this._viewLines.visibleRangeForPosition(new A(e,t))),getLineWidth:e=>(this._flushAccumulatedAndRenderNow(),this._viewLines.getLineWidth(e))}}_createTextAreaHandlerHelper(){return{visibleRangeForPosition:e=>(this._flushAccumulatedAndRenderNow(),this._viewLines.visibleRangeForPosition(e))}}_applyLayout(){const t=this._context.configuration.options.get(146);this.domNode.setWidth(t.width),this.domNode.setHeight(t.height),this._overflowGuardContainer.setWidth(t.width),this._overflowGuardContainer.setHeight(t.height),this._linesContent.setWidth(16777216),this._linesContent.setHeight(16777216)}_getEditorClassName(){const e=this._textAreaHandler.isFocused()?" focused":"";return this._context.configuration.options.get(143)+" "+XM(this._context.theme.type)+e}handleEvents(e){super.handleEvents(e),this._scheduleRender()}onConfigurationChanged(e){return this.domNode.setClassName(this._getEditorClassName()),this._applyLayout(),!1}onCursorStateChanged(e){return this._selections=e.selections,!1}onDecorationsChanged(e){return e.affectsGlyphMargin&&(this._shouldRecomputeGlyphMarginLanes=!0),!1}onFocusChanged(e){return this.domNode.setClassName(this._getEditorClassName()),!1}onThemeChanged(e){return this._context.theme.update(e.theme),this.domNode.setClassName(this._getEditorClassName()),!1}dispose(){this._renderAnimationFrame!==null&&(this._renderAnimationFrame.dispose(),this._renderAnimationFrame=null),this._contentWidgets.overflowingContentWidgetsDomNode.domNode.remove(),this._context.removeEventHandler(this),this._viewLines.dispose();for(const e of this._viewParts)e.dispose();super.dispose()}_scheduleRender(){var e;if(this._store.isDisposed)throw new bt;if(this._renderAnimationFrame===null){const t=this._createCoordinatedRendering();this._renderAnimationFrame=vP.INSTANCE.scheduleCoordinatedRendering({window:xe((e=this.domNode)==null?void 0:e.domNode),prepareRenderText:()=>{if(this._store.isDisposed)throw new bt;try{return t.prepareRenderText()}finally{this._renderAnimationFrame=null}},renderText:()=>{if(this._store.isDisposed)throw new bt;return t.renderText()},prepareRender:(i,n)=>{if(this._store.isDisposed)throw new bt;return t.prepareRender(i,n)},render:(i,n)=>{if(this._store.isDisposed)throw new bt;return t.render(i,n)}})}}_flushAccumulatedAndRenderNow(){const e=this._createCoordinatedRendering();th(()=>e.prepareRenderText());const t=th(()=>e.renderText());if(t){const[i,n]=t;th(()=>e.prepareRender(i,n)),th(()=>e.render(i,n))}}_getViewPartsToRender(){const e=[];let t=0;for(const i of this._viewParts)i.shouldRender()&&(e[t++]=i);return e}_createCoordinatedRendering(){return{prepareRenderText:()=>{if(this._shouldRecomputeGlyphMarginLanes){this._shouldRecomputeGlyphMarginLanes=!1;const e=this._computeGlyphMarginLanes();this._context.configuration.setGlyphMarginDecorationLaneCount(e.requiredLanes)}eh.onRenderStart()},renderText:()=>{if(!this.domNode.domNode.isConnected)return null;let e=this._getViewPartsToRender();if(!this._viewLines.shouldRender()&&e.length===0)return null;const t=this._context.viewLayout.getLinesViewportData();this._context.viewModel.setViewport(t.startLineNumber,t.endLineNumber,t.centeredLineNumber);const i=new Mbe(this._selections,t,this._context.viewLayout.getWhitespaceViewportData(),this._context.viewModel);return this._contentWidgets.shouldRender()&&this._contentWidgets.onBeforeRender(i),this._viewLines.shouldRender()&&(this._viewLines.renderText(i),this._viewLines.onDidRender(),e=this._getViewPartsToRender()),[e,new Gpe(this._context.viewLayout,i,this._viewLines)]},prepareRender:(e,t)=>{for(const i of e)i.prepareRender(t)},render:(e,t)=>{for(const i of e)i.render(t),i.onDidRender()}}}delegateVerticalScrollbarPointerDown(e){this._scrollbar.delegateVerticalScrollbarPointerDown(e)}delegateScrollFromMouseWheelEvent(e){this._scrollbar.delegateScrollFromMouseWheelEvent(e)}restoreState(e){this._context.viewModel.viewLayout.setScrollPosition({scrollTop:e.scrollTop,scrollLeft:e.scrollLeft},1),this._context.viewModel.visibleLinesStabilized()}getOffsetForColumn(e,t){const i=this._context.viewModel.model.validatePosition({lineNumber:e,column:t}),n=this._context.viewModel.coordinatesConverter.convertModelPositionToViewPosition(i);this._flushAccumulatedAndRenderNow();const s=this._viewLines.visibleRangeForPosition(new A(n.lineNumber,n.column));return s?s.left:-1}getTargetAtClientPoint(e,t){const i=this._pointerHandler.getTargetAtClientPoint(e,t);return i?wE.convertViewToModelMouseTarget(i,this._context.viewModel.coordinatesConverter):null}createOverviewRuler(e){return new ybe(this._context,e)}change(e){this._viewZones.changeViewZones(e),this._scheduleRender()}render(e,t){if(t){this._viewLines.forceShouldRender();for(const i of this._viewParts)i.forceShouldRender()}e?this._flushAccumulatedAndRenderNow():this._scheduleRender()}writeScreenReaderContent(e){this._textAreaHandler.writeScreenReaderContent(e)}focus(){this._textAreaHandler.focusTextArea()}isFocused(){return this._textAreaHandler.isFocused()}setAriaOptions(e){this._textAreaHandler.setAriaOptions(e)}addContentWidget(e){this._contentWidgets.addWidget(e.widget),this.layoutContentWidget(e),this._scheduleRender()}layoutContentWidget(e){var t,i,n,s;this._contentWidgets.setWidgetPosition(e.widget,((t=e.position)==null?void 0:t.position)??null,((i=e.position)==null?void 0:i.secondaryPosition)??null,((n=e.position)==null?void 0:n.preference)??null,((s=e.position)==null?void 0:s.positionAffinity)??null),this._scheduleRender()}removeContentWidget(e){this._contentWidgets.removeWidget(e.widget),this._scheduleRender()}addOverlayWidget(e){this._overlayWidgets.addWidget(e.widget),this.layoutOverlayWidget(e),this._scheduleRender()}layoutOverlayWidget(e){this._overlayWidgets.setWidgetPosition(e.widget,e.position)&&this._scheduleRender()}removeOverlayWidget(e){this._overlayWidgets.removeWidget(e.widget),this._scheduleRender()}addGlyphMarginWidget(e){this._glyphMarginWidgets.addWidget(e.widget),this._shouldRecomputeGlyphMarginLanes=!0,this._scheduleRender()}layoutGlyphMarginWidget(e){const t=e.position;this._glyphMarginWidgets.setWidgetPosition(e.widget,t)&&(this._shouldRecomputeGlyphMarginLanes=!0,this._scheduleRender())}removeGlyphMarginWidget(e){this._glyphMarginWidgets.removeWidget(e.widget),this._shouldRecomputeGlyphMarginLanes=!0,this._scheduleRender()}};CP=Obe([Fbe(6,we)],CP);function th(o){try{return o()}catch(e){return Oe(e),null}}const gD=class gD{constructor(){this._coordinatedRenderings=[],this._animationFrameRunners=new Map}scheduleCoordinatedRendering(e){return this._coordinatedRenderings.push(e),this._scheduleRender(e.window),{dispose:()=>{const t=this._coordinatedRenderings.indexOf(e);if(t!==-1&&(this._coordinatedRenderings.splice(t,1),this._coordinatedRenderings.length===0)){for(const[i,n]of this._animationFrameRunners)n.dispose();this._animationFrameRunners.clear()}}}}_scheduleRender(e){if(!this._animationFrameRunners.has(e)){const t=()=>{this._animationFrameRunners.delete(e),this._onRenderScheduled()};this._animationFrameRunners.set(e,$y(e,t,100))}}_onRenderScheduled(){const e=this._coordinatedRenderings.slice(0);this._coordinatedRenderings=[];for(const i of e)th(()=>i.prepareRenderText());const t=[];for(let i=0,n=e.length;i<n;i++){const s=e[i];t[i]=th(()=>s.renderText())}for(let i=0,n=e.length;i<n;i++){const s=e[i],r=t[i];if(!r)continue;const[a,l]=r;th(()=>s.prepareRender(a,l))}for(let i=0,n=e.length;i<n;i++){const s=e[i],r=t[i];if(!r)continue;const[a,l]=r;th(()=>s.render(a,l))}}};gD.INSTANCE=new gD;let vP=gD;class fv{constructor(e,t,i,n,s){this.injectionOffsets=e,this.injectionOptions=t,this.breakOffsets=i,this.breakOffsetsVisibleColumn=n,this.wrappedTextIndentLength=s}getOutputLineCount(){return this.breakOffsets.length}getMinOutputOffset(e){return e>0?this.wrappedTextIndentLength:0}getLineLength(e){const t=e>0?this.breakOffsets[e-1]:0;let n=this.breakOffsets[e]-t;return e>0&&(n+=this.wrappedTextIndentLength),n}getMaxOutputOffset(e){return this.getLineLength(e)}translateToInputOffset(e,t){e>0&&(t=Math.max(0,t-this.wrappedTextIndentLength));let n=e===0?t:this.breakOffsets[e-1]+t;if(this.injectionOffsets!==null)for(let s=0;s<this.injectionOffsets.length&&n>this.injectionOffsets[s];s++)n<this.injectionOffsets[s]+this.injectionOptions[s].content.length?n=this.injectionOffsets[s]:n-=this.injectionOptions[s].content.length;return n}translateToOutputPosition(e,t=2){let i=e;if(this.injectionOffsets!==null)for(let n=0;n<this.injectionOffsets.length&&!(e<this.injectionOffsets[n]||t!==1&&e===this.injectionOffsets[n]);n++)i+=this.injectionOptions[n].content.length;return this.offsetInInputWithInjectionsToOutputPosition(i,t)}offsetInInputWithInjectionsToOutputPosition(e,t=2){let i=0,n=this.breakOffsets.length-1,s=0,r=0;for(;i<=n;){s=i+(n-i)/2|0;const l=this.breakOffsets[s];if(r=s>0?this.breakOffsets[s-1]:0,t===0)if(e<=r)n=s-1;else if(e>l)i=s+1;else break;else if(e<r)n=s-1;else if(e>=l)i=s+1;else break}let a=e-r;return s>0&&(a+=this.wrappedTextIndentLength),new c0(s,a)}normalizeOutputPosition(e,t,i){if(this.injectionOffsets!==null){const n=this.outputPositionToOffsetInInputWithInjections(e,t),s=this.normalizeOffsetInInputWithInjectionsAroundInjections(n,i);if(s!==n)return this.offsetInInputWithInjectionsToOutputPosition(s,i)}if(i===0){if(e>0&&t===this.getMinOutputOffset(e))return new c0(e-1,this.getMaxOutputOffset(e-1))}else if(i===1){const n=this.getOutputLineCount()-1;if(e<n&&t===this.getMaxOutputOffset(e))return new c0(e+1,this.getMinOutputOffset(e+1))}return new c0(e,t)}outputPositionToOffsetInInputWithInjections(e,t){return e>0&&(t=Math.max(0,t-this.wrappedTextIndentLength)),(e>0?this.breakOffsets[e-1]:0)+t}normalizeOffsetInInputWithInjectionsAroundInjections(e,t){const i=this.getInjectedTextAtOffset(e);if(!i)return e;if(t===2){if(e===i.offsetInInputWithInjections+i.length&&FH(this.injectionOptions[i.injectedTextIndex].cursorStops))return i.offsetInInputWithInjections+i.length;{let n=i.offsetInInputWithInjections;if(WH(this.injectionOptions[i.injectedTextIndex].cursorStops))return n;let s=i.injectedTextIndex-1;for(;s>=0&&this.injectionOffsets[s]===this.injectionOffsets[i.injectedTextIndex]&&!(FH(this.injectionOptions[s].cursorStops)||(n-=this.injectionOptions[s].content.length,WH(this.injectionOptions[s].cursorStops)));)s--;return n}}else if(t===1||t===4){let n=i.offsetInInputWithInjections+i.length,s=i.injectedTextIndex;for(;s+1<this.injectionOffsets.length&&this.injectionOffsets[s+1]===this.injectionOffsets[s];)n+=this.injectionOptions[s+1].content.length,s++;return n}else if(t===0||t===3){let n=i.offsetInInputWithInjections,s=i.injectedTextIndex;for(;s-1>=0&&this.injectionOffsets[s-1]===this.injectionOffsets[s];)n-=this.injectionOptions[s-1].content.length,s--;return n}NI()}getInjectedText(e,t){const i=this.outputPositionToOffsetInInputWithInjections(e,t),n=this.getInjectedTextAtOffset(i);return n?{options:this.injectionOptions[n.injectedTextIndex]}:null}getInjectedTextAtOffset(e){const t=this.injectionOffsets,i=this.injectionOptions;if(t!==null){let n=0;for(let s=0;s<t.length;s++){const r=i[s].content.length,a=t[s]+n,l=t[s]+n+r;if(a>e)break;if(e<=l)return{injectedTextIndex:s,offsetInInputWithInjections:a,length:r};n+=r}}}}function FH(o){return o==null?!0:o===zr.Right||o===zr.Both}function WH(o){return o==null?!0:o===zr.Left||o===zr.Both}class c0{constructor(e,t){this.outputLineIndex=e,this.outputOffset=t}toString(){return`${this.outputLineIndex}:${this.outputOffset}`}toPosition(e){return new A(e+this.outputLineIndex,this.outputOffset+1)}}const yT=hd("domLineBreaksComputer",{createHTML:o=>o});class J3{static create(e){return new J3(new WeakRef(e))}constructor(e){this.targetWindow=e}createLineBreaksComputer(e,t,i,n,s){const r=[],a=[];return{addRequest:(l,c,d)=>{r.push(l),a.push(c)},finalize:()=>Wbe(Kd(this.targetWindow.deref()),r,e,t,i,n,s,a)}}}function Wbe(o,e,t,i,n,s,r,a){function l(N){const B=a[N];if(B){const W=Wa.applyInjectedText(e[N],B),V=B.map(H=>H.options),$=B.map(H=>H.column-1);return new fv($,V,[W.length],[],0)}else return null}if(n===-1){const N=[];for(let B=0,W=e.length;B<W;B++)N[B]=l(B);return N}const c=Math.round(n*t.typicalHalfwidthCharacterWidth),h=Math.round(i*(s===3?2:s===2?1:0)),u=Math.ceil(t.spaceWidth*h),g=document.createElement("div");bn(g,t);const f=new _b(1e4),p=[],_=[],b=[],C=[],v=[];for(let N=0;N<e.length;N++){const B=Wa.applyInjectedText(e[N],a[N]);let W=0,V=0,$=c;if(s!==0)if(W=Jn(B),W===-1)W=0;else{for(let he=0;he<W;he++){const fe=B.charCodeAt(he)===9?i-V%i:1;V+=fe}const J=Math.ceil(t.spaceWidth*V);J+t.typicalFullwidthCharacterWidth>c?(W=0,V=0):$=c-J}const H=B.substr(W),j=Bbe(H,V,i,$,f,u);p[N]=W,_[N]=V,b[N]=H,C[N]=j[0],v[N]=j[1]}const w=f.build(),S=(yT==null?void 0:yT.createHTML(w))??w;g.innerHTML=S,g.style.position="absolute",g.style.top="10000",r==="keepAll"?(g.style.wordBreak="keep-all",g.style.overflowWrap="anywhere"):(g.style.wordBreak="inherit",g.style.overflowWrap="break-word"),o.document.body.appendChild(g);const L=document.createRange(),k=Array.prototype.slice.call(g.children,0),D=[];for(let N=0;N<e.length;N++){const B=k[N],W=Hbe(L,B,b[N],C[N]);if(W===null){D[N]=l(N);continue}const V=p[N],$=_[N]+h,H=v[N],j=[];for(let ue=0,Pe=W.length;ue<Pe;ue++)j[ue]=H[W[ue]];if(V!==0)for(let ue=0,Pe=W.length;ue<Pe;ue++)W[ue]+=V;let J,he;const fe=a[N];fe?(J=fe.map(ue=>ue.options),he=fe.map(ue=>ue.column-1)):(J=null,he=null),D[N]=new fv(he,J,W,j,$)}return g.remove(),D}function Bbe(o,e,t,i,n,s){if(s!==0){const u=String(s);n.appendString('<div style="text-indent: -'),n.appendString(u),n.appendString("px; padding-left: "),n.appendString(u),n.appendString("px; box-sizing: border-box; width:")}else n.appendString('<div style="width:');n.appendString(String(i)),n.appendString('px;">');const r=o.length;let a=e,l=0;const c=[],d=[];let h=0<r?o.charCodeAt(0):0;n.appendString("<span>");for(let u=0;u<r;u++){u!==0&&u%16384===0&&n.appendString("</span><span>"),c[u]=l,d[u]=a;const g=h;h=u+1<r?o.charCodeAt(u+1):0;let f=1,p=1;switch(g){case 9:f=t-a%t,p=f;for(let _=1;_<=f;_++)_<f?n.appendCharCode(160):n.appendASCIICharCode(32);break;case 32:h===32?n.appendCharCode(160):n.appendASCIICharCode(32);break;case 60:n.appendString("&lt;");break;case 62:n.appendString("&gt;");break;case 38:n.appendString("&amp;");break;case 0:n.appendString("&#00;");break;case 65279:case 8232:case 8233:case 133:n.appendCharCode(65533);break;default:Fh(g)&&p++,g<32?n.appendCharCode(9216+g):n.appendCharCode(g)}l+=f,a+=p}return n.appendString("</span>"),c[o.length]=l,d[o.length]=a,n.appendString("</div>"),[c,d]}function Hbe(o,e,t,i){if(t.length<=1)return null;const n=Array.prototype.slice.call(e.children,0),s=[];try{wP(o,n,i,0,null,t.length-1,null,s)}catch(r){return console.log(r),null}return s.length===0?null:(s.push(t.length),s)}function wP(o,e,t,i,n,s,r,a){if(i===s||(n=n||LT(o,e,t[i],t[i+1]),r=r||LT(o,e,t[s],t[s+1]),Math.abs(n[0].top-r[0].top)<=.1))return;if(i+1===s){a.push(s);return}const l=i+(s-i)/2|0,c=LT(o,e,t[l],t[l+1]);wP(o,e,t,i,n,l,c,a),wP(o,e,t,l,c,s,r,a)}function LT(o,e,t,i){return o.setStart(e[t/16384|0].firstChild,t%16384),o.setEnd(e[i/16384|0].firstChild,i%16384),o.getClientRects()}class Vbe extends F{constructor(){super(),this._editor=null,this._instantiationService=null,this._instances=this._register(new $4),this._pending=new Map,this._finishedInstantiation=[],this._finishedInstantiation[0]=!1,this._finishedInstantiation[1]=!1,this._finishedInstantiation[2]=!1,this._finishedInstantiation[3]=!1}initialize(e,t,i){this._editor=e,this._instantiationService=i;for(const n of t){if(this._pending.has(n.id)){Oe(new Error(`Cannot have two contributions with the same id ${n.id}`));continue}this._pending.set(n.id,n)}this._instantiateSome(0),this._register(vC(xe(this._editor.getDomNode()),()=>{this._instantiateSome(1)})),this._register(vC(xe(this._editor.getDomNode()),()=>{this._instantiateSome(2)})),this._register(vC(xe(this._editor.getDomNode()),()=>{this._instantiateSome(3)},5e3))}saveViewState(){const e={};for(const[t,i]of this._instances)typeof i.saveViewState=="function"&&(e[t]=i.saveViewState());return e}restoreViewState(e){for(const[t,i]of this._instances)typeof i.restoreViewState=="function"&&i.restoreViewState(e[t])}get(e){return this._instantiateById(e),this._instances.get(e)||null}onBeforeInteractionEvent(){this._instantiateSome(2)}onAfterModelAttached(){var e;return vC(xe((e=this._editor)==null?void 0:e.getDomNode()),()=>{this._instantiateSome(1)},50)}_instantiateSome(e){if(this._finishedInstantiation[e])return;this._finishedInstantiation[e]=!0;const t=this._findPendingContributionsByInstantiation(e);for(const i of t)this._instantiateById(i.id)}_findPendingContributionsByInstantiation(e){const t=[];for(const[,i]of this._pending)i.instantiation===e&&t.push(i);return t}_instantiateById(e){const t=this._pending.get(e);if(t){if(this._pending.delete(e),!this._instantiationService||!this._editor)throw new Error("Cannot instantiate contributions before being initialized!");try{const i=this._instantiationService.createInstance(t.ctor,this._editor);this._instances.set(t.id,i),typeof i.restoreViewState=="function"&&t.instantiation!==0&&console.warn(`Editor contribution '${t.id}' should be eager instantiated because it uses saveViewState / restoreViewState.`)}catch(i){Oe(i)}}}}class Tq{constructor(e,t,i,n,s,r,a){this.id=e,this.label=t,this.alias=i,this.metadata=n,this._precondition=s,this._run=r,this._contextKeyService=a}isSupported(){return this._contextKeyService.contextMatchesRules(this._precondition)}run(e){return this.isSupported()?this._run(e):Promise.resolve(void 0)}}class eW{static create(e){return new eW(e.get(135),e.get(134))}constructor(e,t){this.classifier=new zbe(e,t)}createLineBreaksComputer(e,t,i,n,s){const r=[],a=[],l=[];return{addRequest:(c,d,h)=>{r.push(c),a.push(d),l.push(h)},finalize:()=>{const c=e.typicalFullwidthCharacterWidth/e.typicalHalfwidthCharacterWidth,d=[];for(let h=0,u=r.length;h<u;h++){const g=a[h],f=l[h];f&&!f.injectionOptions&&!g?d[h]=Ube(this.classifier,f,r[h],t,i,c,n,s):d[h]=$be(this.classifier,r[h],g,t,i,c,n,s)}return SP.length=0,yP.length=0,d}}}}class zbe extends bb{constructor(e,t){super(0);for(let i=0;i<e.length;i++)this.set(e.charCodeAt(i),1);for(let i=0;i<t.length;i++)this.set(t.charCodeAt(i),2)}get(e){return e>=0&&e<256?this._asciiMap[e]:e>=12352&&e<=12543||e>=13312&&e<=19903||e>=19968&&e<=40959?3:this._map.get(e)||this._defaultValue}}let SP=[],yP=[];function Ube(o,e,t,i,n,s,r,a){if(n===-1)return null;const l=t.length;if(l<=1)return null;const c=a==="keepAll",d=e.breakOffsets,h=e.breakOffsetsVisibleColumn,u=Rq(t,i,n,s,r),g=n-u,f=SP,p=yP;let _=0,b=0,C=0,v=n;const w=d.length;let S=0;if(S>=0){let L=Math.abs(h[S]-v);for(;S+1<w;){const k=Math.abs(h[S+1]-v);if(k>=L)break;L=k,S++}}for(;S<w;){let L=S<0?0:d[S],k=S<0?0:h[S];b>L&&(L=b,k=C);let D=0,N=0,B=0,W=0;if(k<=v){let $=k,H=L===0?0:t.charCodeAt(L-1),j=L===0?0:o.get(H),J=!0;for(let he=L;he<l;he++){const fe=he,ue=t.charCodeAt(he);let Pe,_e;if(Gi(ue)?(he++,Pe=0,_e=2):(Pe=o.get(ue),_e=mv(ue,$,i,s)),fe>b&&LP(H,j,ue,Pe,c)&&(D=fe,N=$),$+=_e,$>v){fe>b?(B=fe,W=$-_e):(B=he+1,W=$),$-N>g&&(D=0),J=!1;break}H=ue,j=Pe}if(J){_>0&&(f[_]=d[d.length-1],p[_]=h[d.length-1],_++);break}}if(D===0){let $=k,H=t.charCodeAt(L),j=o.get(H),J=!1;for(let he=L-1;he>=b;he--){const fe=he+1,ue=t.charCodeAt(he);if(ue===9){J=!0;break}let Pe,_e;if(kf(ue)?(he--,Pe=0,_e=2):(Pe=o.get(ue),_e=Fh(ue)?s:1),$<=v){if(B===0&&(B=fe,W=$),$<=v-g)break;if(LP(ue,Pe,H,j,c)){D=fe,N=$;break}}$-=_e,H=ue,j=Pe}if(D!==0){const he=g-(W-N);if(he<=i){const fe=t.charCodeAt(B);let ue;Gi(fe)?ue=2:ue=mv(fe,W,i,s),he-ue<0&&(D=0)}}if(J){S--;continue}}if(D===0&&(D=B,N=W),D<=b){const $=t.charCodeAt(b);Gi($)?(D=b+2,N=C+2):(D=b+1,N=C+mv($,C,i,s))}for(b=D,f[_]=D,C=N,p[_]=N,_++,v=N+g;S<0||S<w&&h[S]<N;)S++;let V=Math.abs(h[S]-v);for(;S+1<w;){const $=Math.abs(h[S+1]-v);if($>=V)break;V=$,S++}}return _===0?null:(f.length=_,p.length=_,SP=e.breakOffsets,yP=e.breakOffsetsVisibleColumn,e.breakOffsets=f,e.breakOffsetsVisibleColumn=p,e.wrappedTextIndentLength=u,e)}function $be(o,e,t,i,n,s,r,a){const l=Wa.applyInjectedText(e,t);let c,d;if(t&&t.length>0?(c=t.map(N=>N.options),d=t.map(N=>N.column-1)):(c=null,d=null),n===-1)return c?new fv(d,c,[l.length],[],0):null;const h=l.length;if(h<=1)return c?new fv(d,c,[l.length],[],0):null;const u=a==="keepAll",g=Rq(l,i,n,s,r),f=n-g,p=[],_=[];let b=0,C=0,v=0,w=n,S=l.charCodeAt(0),L=o.get(S),k=mv(S,0,i,s),D=1;Gi(S)&&(k+=1,S=l.charCodeAt(1),L=o.get(S),D++);for(let N=D;N<h;N++){const B=N,W=l.charCodeAt(N);let V,$;Gi(W)?(N++,V=0,$=2):(V=o.get(W),$=mv(W,k,i,s)),LP(S,L,W,V,u)&&(C=B,v=k),k+=$,k>w&&((C===0||k-v>f)&&(C=B,v=k-$),p[b]=C,_[b]=v,b++,w=v+f,C=0),S=W,L=V}return b===0&&(!t||t.length===0)?null:(p[b]=h,_[b]=k,new fv(d,c,p,_,g))}function mv(o,e,t,i){return o===9?t-e%t:Fh(o)||o<32?i:1}function BH(o,e){return e-o%e}function LP(o,e,t,i,n){return t!==32&&(e===2&&i!==2||e!==1&&i===1||!n&&e===3&&i!==2||!n&&i===3&&e!==1)}function Rq(o,e,t,i,n){let s=0;if(n!==0){const r=Jn(o);if(r!==-1){for(let l=0;l<r;l++){const c=o.charCodeAt(l)===9?BH(s,e):1;s+=c}const a=n===3?2:n===2?1:0;for(let l=0;l<a;l++){const c=BH(s,e);s+=c}s+i>t&&(s=0)}}return s}class gx{constructor(e){this._selTrackedRange=null,this._trackSelection=!0,this._setState(e,new gn(new x(1,1,1,1),0,0,new A(1,1),0),new gn(new x(1,1,1,1),0,0,new A(1,1),0))}dispose(e){this._removeTrackedRange(e)}startTrackingSelection(e){this._trackSelection=!0,this._updateTrackedRange(e)}stopTrackingSelection(e){this._trackSelection=!1,this._removeTrackedRange(e)}_updateTrackedRange(e){this._trackSelection&&(this._selTrackedRange=e.model._setTrackedRange(this._selTrackedRange,this.modelState.selection,0))}_removeTrackedRange(e){this._selTrackedRange=e.model._setTrackedRange(this._selTrackedRange,null,0)}asCursorState(){return new ft(this.modelState,this.viewState)}readSelectionFromMarkers(e){const t=e.model._getTrackedRange(this._selTrackedRange);return this.modelState.selection.isEmpty()&&!t.isEmpty()?pe.fromRange(t.collapseToEnd(),this.modelState.selection.getDirection()):pe.fromRange(t,this.modelState.selection.getDirection())}ensureValidState(e){this._setState(e,this.modelState,this.viewState)}setState(e,t,i){this._setState(e,t,i)}static _validatePositionWithCache(e,t,i,n){return t.equals(i)?n:e.normalizePosition(t,2)}static _validateViewState(e,t){const i=t.position,n=t.selectionStart.getStartPosition(),s=t.selectionStart.getEndPosition(),r=e.normalizePosition(i,2),a=this._validatePositionWithCache(e,n,i,r),l=this._validatePositionWithCache(e,s,n,a);return i.equals(r)&&n.equals(a)&&s.equals(l)?t:new gn(x.fromPositions(a,l),t.selectionStartKind,t.selectionStartLeftoverVisibleColumns+n.column-a.column,r,t.leftoverVisibleColumns+i.column-r.column)}_setState(e,t,i){if(i&&(i=gx._validateViewState(e.viewModel,i)),t){const n=e.model.validateRange(t.selectionStart),s=t.selectionStart.equalsRange(n)?t.selectionStartLeftoverVisibleColumns:0,r=e.model.validatePosition(t.position),a=t.position.equals(r)?t.leftoverVisibleColumns:0;t=new gn(n,t.selectionStartKind,s,r,a)}else{if(!i)return;const n=e.model.validateRange(e.coordinatesConverter.convertViewRangeToModelRange(i.selectionStart)),s=e.model.validatePosition(e.coordinatesConverter.convertViewPositionToModelPosition(i.position));t=new gn(n,i.selectionStartKind,i.selectionStartLeftoverVisibleColumns,s,i.leftoverVisibleColumns)}if(i){const n=e.coordinatesConverter.validateViewRange(i.selectionStart,t.selectionStart),s=e.coordinatesConverter.validateViewPosition(i.position,t.position);i=new gn(n,t.selectionStartKind,t.selectionStartLeftoverVisibleColumns,s,t.leftoverVisibleColumns)}else{const n=e.coordinatesConverter.convertModelPositionToViewPosition(new A(t.selectionStart.startLineNumber,t.selectionStart.startColumn)),s=e.coordinatesConverter.convertModelPositionToViewPosition(new A(t.selectionStart.endLineNumber,t.selectionStart.endColumn)),r=new x(n.lineNumber,n.column,s.lineNumber,s.column),a=e.coordinatesConverter.convertModelPositionToViewPosition(t.position);i=new gn(r,t.selectionStartKind,t.selectionStartLeftoverVisibleColumns,a,t.leftoverVisibleColumns)}this.modelState=t,this.viewState=i,this._updateTrackedRange(e)}}class HH{constructor(e){this.context=e,this.cursors=[new gx(e)],this.lastAddedCursorIndex=0}dispose(){for(const e of this.cursors)e.dispose(this.context)}startTrackingSelections(){for(const e of this.cursors)e.startTrackingSelection(this.context)}stopTrackingSelections(){for(const e of this.cursors)e.stopTrackingSelection(this.context)}updateContext(e){this.context=e}ensureValidState(){for(const e of this.cursors)e.ensureValidState(this.context)}readSelectionFromMarkers(){return this.cursors.map(e=>e.readSelectionFromMarkers(this.context))}getAll(){return this.cursors.map(e=>e.asCursorState())}getViewPositions(){return this.cursors.map(e=>e.viewState.position)}getTopMostViewPosition(){return ise(this.cursors,gs(e=>e.viewState.position,A.compare)).viewState.position}getBottomMostViewPosition(){return tse(this.cursors,gs(e=>e.viewState.position,A.compare)).viewState.position}getSelections(){return this.cursors.map(e=>e.modelState.selection)}getViewSelections(){return this.cursors.map(e=>e.viewState.selection)}setSelections(e){this.setStates(ft.fromModelSelections(e))}getPrimaryCursor(){return this.cursors[0].asCursorState()}setStates(e){e!==null&&(this.cursors[0].setState(this.context,e[0].modelState,e[0].viewState),this._setSecondaryStates(e.slice(1)))}_setSecondaryStates(e){const t=this.cursors.length-1,i=e.length;if(t<i){const n=i-t;for(let s=0;s<n;s++)this._addSecondaryCursor()}else if(t>i){const n=t-i;for(let s=0;s<n;s++)this._removeSecondaryCursor(this.cursors.length-2)}for(let n=0;n<i;n++)this.cursors[n+1].setState(this.context,e[n].modelState,e[n].viewState)}killSecondaryCursors(){this._setSecondaryStates([])}_addSecondaryCursor(){this.cursors.push(new gx(this.context)),this.lastAddedCursorIndex=this.cursors.length-1}getLastAddedCursorIndex(){return this.cursors.length===1||this.lastAddedCursorIndex===0?0:this.lastAddedCursorIndex}_removeSecondaryCursor(e){this.lastAddedCursorIndex>=e+1&&this.lastAddedCursorIndex--,this.cursors[e+1].dispose(this.context),this.cursors.splice(e+1,1)}normalize(){if(this.cursors.length===1)return;const e=this.cursors.slice(0),t=[];for(let i=0,n=e.length;i<n;i++)t.push({index:i,selection:e[i].modelState.selection});t.sort(gs(i=>i.selection,x.compareRangesUsingStarts));for(let i=0;i<t.length-1;i++){const n=t[i],s=t[i+1],r=n.selection,a=s.selection;if(!this.context.cursorConfig.multiCursorMergeOverlapping)continue;let l;if(a.isEmpty()||r.isEmpty()?l=a.getStartPosition().isBeforeOrEqual(r.getEndPosition()):l=a.getStartPosition().isBefore(r.getEndPosition()),l){const c=n.index<s.index?i:i+1,d=n.index<s.index?i+1:i,h=t[d].index,u=t[c].index,g=t[d].selection,f=t[c].selection;if(!g.equalsSelection(f)){const p=g.plusRange(f),_=g.selectionStartLineNumber===g.startLineNumber&&g.selectionStartColumn===g.startColumn,b=f.selectionStartLineNumber===f.startLineNumber&&f.selectionStartColumn===f.startColumn;let C;h===this.lastAddedCursorIndex?(C=_,this.lastAddedCursorIndex=u):C=b;let v;C?v=new pe(p.startLineNumber,p.startColumn,p.endLineNumber,p.endColumn):v=new pe(p.endLineNumber,p.endColumn,p.startLineNumber,p.startColumn),t[c].selection=v;const w=ft.fromModelSelection(v);e[u].setState(this.context,w.modelState,w.viewState)}for(const p of t)p.index>h&&p.index--;e.splice(h,1),t.splice(d,1),this._removeSecondaryCursor(h-1),i--}}}}class VH{constructor(e,t,i,n){this._cursorContextBrand=void 0,this.model=e,this.viewModel=t,this.coordinatesConverter=i,this.cursorConfig=n}}class jbe{constructor(){this.type=0}}class Kbe{constructor(){this.type=1}}class qbe{constructor(e){this.type=2,this._source=e}hasChanged(e){return this._source.hasChanged(e)}}class Gbe{constructor(e,t,i){this.selections=e,this.modelSelections=t,this.reason=i,this.type=3}}class Su{constructor(e){this.type=4,e?(this.affectsMinimap=e.affectsMinimap,this.affectsOverviewRuler=e.affectsOverviewRuler,this.affectsGlyphMargin=e.affectsGlyphMargin,this.affectsLineNumber=e.affectsLineNumber):(this.affectsMinimap=!0,this.affectsOverviewRuler=!0,this.affectsGlyphMargin=!0,this.affectsLineNumber=!0)}}class d0{constructor(){this.type=5}}class Zbe{constructor(e){this.type=6,this.isFocused=e}}class Ybe{constructor(){this.type=7}}class h0{constructor(){this.type=8}}class Mq{constructor(e,t){this.fromLineNumber=e,this.count=t,this.type=9}}class xP{constructor(e,t){this.type=10,this.fromLineNumber=e,this.toLineNumber=t}}class kP{constructor(e,t){this.type=11,this.fromLineNumber=e,this.toLineNumber=t}}class pv{constructor(e,t,i,n,s,r,a){this.source=e,this.minimalReveal=t,this.range=i,this.selections=n,this.verticalType=s,this.revealHorizontal=r,this.scrollType=a,this.type=12}}class Xbe{constructor(e){this.type=13,this.scrollWidth=e.scrollWidth,this.scrollLeft=e.scrollLeft,this.scrollHeight=e.scrollHeight,this.scrollTop=e.scrollTop,this.scrollWidthChanged=e.scrollWidthChanged,this.scrollLeftChanged=e.scrollLeftChanged,this.scrollHeightChanged=e.scrollHeightChanged,this.scrollTopChanged=e.scrollTopChanged}}class Qbe{constructor(e){this.theme=e,this.type=14}}class Jbe{constructor(e){this.type=15,this.ranges=e}}class eCe{constructor(){this.type=16}}let tCe=class{constructor(){this.type=17}};class iCe extends F{constructor(){super(),this._onEvent=this._register(new P),this.onEvent=this._onEvent.event,this._eventHandlers=[],this._viewEventQueue=null,this._isConsumingViewEventQueue=!1,this._collector=null,this._collectorCnt=0,this._outgoingEvents=[]}emitOutgoingEvent(e){this._addOutgoingEvent(e),this._emitOutgoingEvents()}_addOutgoingEvent(e){for(let t=0,i=this._outgoingEvents.length;t<i;t++){const n=this._outgoingEvents[t].kind===e.kind?this._outgoingEvents[t].attemptToMerge(e):null;if(n){this._outgoingEvents[t]=n;return}}this._outgoingEvents.push(e)}_emitOutgoingEvents(){for(;this._outgoingEvents.length>0;){if(this._collector||this._isConsumingViewEventQueue)return;const e=this._outgoingEvents.shift();e.isNoOp()||this._onEvent.fire(e)}}addViewEventHandler(e){for(let t=0,i=this._eventHandlers.length;t<i;t++)this._eventHandlers[t]===e&&console.warn("Detected duplicate listener in ViewEventDispatcher",e);this._eventHandlers.push(e)}removeViewEventHandler(e){for(let t=0;t<this._eventHandlers.length;t++)if(this._eventHandlers[t]===e){this._eventHandlers.splice(t,1);break}}beginEmitViewEvents(){return this._collectorCnt++,this._collectorCnt===1&&(this._collector=new nCe),this._collector}endEmitViewEvents(){if(this._collectorCnt--,this._collectorCnt===0){const e=this._collector.outgoingEvents,t=this._collector.viewEvents;this._collector=null;for(const i of e)this._addOutgoingEvent(i);t.length>0&&this._emitMany(t)}this._emitOutgoingEvents()}emitSingleViewEvent(e){try{this.beginEmitViewEvents().emitViewEvent(e)}finally{this.endEmitViewEvents()}}_emitMany(e){this._viewEventQueue?this._viewEventQueue=this._viewEventQueue.concat(e):this._viewEventQueue=e,this._isConsumingViewEventQueue||this._consumeViewEventQueue()}_consumeViewEventQueue(){try{this._isConsumingViewEventQueue=!0,this._doConsumeQueue()}finally{this._isConsumingViewEventQueue=!1}}_doConsumeQueue(){for(;this._viewEventQueue;){const e=this._viewEventQueue;this._viewEventQueue=null;const t=this._eventHandlers.slice(0);for(const i of t)i.handleEvents(e)}}}class nCe{constructor(){this.viewEvents=[],this.outgoingEvents=[]}emitViewEvent(e){this.viewEvents.push(e)}emitOutgoingEvent(e){this.outgoingEvents.push(e)}}class tW{constructor(e,t,i,n){this.kind=0,this._oldContentWidth=e,this._oldContentHeight=t,this.contentWidth=i,this.contentHeight=n,this.contentWidthChanged=this._oldContentWidth!==this.contentWidth,this.contentHeightChanged=this._oldContentHeight!==this.contentHeight}isNoOp(){return!this.contentWidthChanged&&!this.contentHeightChanged}attemptToMerge(e){return e.kind!==this.kind?null:new tW(this._oldContentWidth,this._oldContentHeight,e.contentWidth,e.contentHeight)}}class iW{constructor(e,t){this.kind=1,this.oldHasFocus=e,this.hasFocus=t}isNoOp(){return this.oldHasFocus===this.hasFocus}attemptToMerge(e){return e.kind!==this.kind?null:new iW(this.oldHasFocus,e.hasFocus)}}class nW{constructor(e,t,i,n,s,r,a,l){this.kind=2,this._oldScrollWidth=e,this._oldScrollLeft=t,this._oldScrollHeight=i,this._oldScrollTop=n,this.scrollWidth=s,this.scrollLeft=r,this.scrollHeight=a,this.scrollTop=l,this.scrollWidthChanged=this._oldScrollWidth!==this.scrollWidth,this.scrollLeftChanged=this._oldScrollLeft!==this.scrollLeft,this.scrollHeightChanged=this._oldScrollHeight!==this.scrollHeight,this.scrollTopChanged=this._oldScrollTop!==this.scrollTop}isNoOp(){return!this.scrollWidthChanged&&!this.scrollLeftChanged&&!this.scrollHeightChanged&&!this.scrollTopChanged}attemptToMerge(e){return e.kind!==this.kind?null:new nW(this._oldScrollWidth,this._oldScrollLeft,this._oldScrollHeight,this._oldScrollTop,e.scrollWidth,e.scrollLeft,e.scrollHeight,e.scrollTop)}}class sCe{constructor(){this.kind=3}isNoOp(){return!1}attemptToMerge(e){return e.kind!==this.kind?null:this}}class oCe{constructor(){this.kind=4}isNoOp(){return!1}attemptToMerge(e){return e.kind!==this.kind?null:this}}class fx{constructor(e,t,i,n,s,r,a){this.kind=6,this.oldSelections=e,this.selections=t,this.oldModelVersionId=i,this.modelVersionId=n,this.source=s,this.reason=r,this.reachedMaxCursorCount=a}static _selectionsAreEqual(e,t){if(!e&&!t)return!0;if(!e||!t)return!1;const i=e.length,n=t.length;if(i!==n)return!1;for(let s=0;s<i;s++)if(!e[s].equalsSelection(t[s]))return!1;return!0}isNoOp(){return fx._selectionsAreEqual(this.oldSelections,this.selections)&&this.oldModelVersionId===this.modelVersionId}attemptToMerge(e){return e.kind!==this.kind?null:new fx(this.oldSelections,e.selections,this.oldModelVersionId,e.modelVersionId,e.source,e.reason,this.reachedMaxCursorCount||e.reachedMaxCursorCount)}}class rCe{constructor(){this.kind=5}isNoOp(){return!1}attemptToMerge(e){return e.kind!==this.kind?null:this}}class aCe{constructor(e){this.event=e,this.kind=7}isNoOp(){return!1}attemptToMerge(e){return null}}class lCe{constructor(e){this.event=e,this.kind=8}isNoOp(){return!1}attemptToMerge(e){return null}}class cCe{constructor(e){this.event=e,this.kind=9}isNoOp(){return!1}attemptToMerge(e){return null}}class dCe{constructor(e){this.event=e,this.kind=10}isNoOp(){return!1}attemptToMerge(e){return null}}class hCe{constructor(e){this.event=e,this.kind=11}isNoOp(){return!1}attemptToMerge(e){return null}}class uCe{constructor(e){this.event=e,this.kind=12}isNoOp(){return!1}attemptToMerge(e){return null}}class gCe extends F{constructor(e,t,i,n){super(),this._model=e,this._knownModelVersionId=this._model.getVersionId(),this._viewModel=t,this._coordinatesConverter=i,this.context=new VH(this._model,this._viewModel,this._coordinatesConverter,n),this._cursors=new HH(this.context),this._hasFocus=!1,this._isHandling=!1,this._compositionState=null,this._columnSelectData=null,this._autoClosedActions=[],this._prevEditOperationType=0}dispose(){this._cursors.dispose(),this._autoClosedActions=Tt(this._autoClosedActions),super.dispose()}updateConfiguration(e){this.context=new VH(this._model,this._viewModel,this._coordinatesConverter,e),this._cursors.updateContext(this.context)}onLineMappingChanged(e){this._knownModelVersionId===this._model.getVersionId()&&this.setStates(e,"viewModel",0,this.getCursorStates())}setHasFocus(e){this._hasFocus=e}_validateAutoClosedActions(){if(this._autoClosedActions.length>0){const e=this._cursors.getSelections();for(let t=0;t<this._autoClosedActions.length;t++){const i=this._autoClosedActions[t];i.isValid(e)||(i.dispose(),this._autoClosedActions.splice(t,1),t--)}}}getPrimaryCursorState(){return this._cursors.getPrimaryCursor()}getLastAddedCursorIndex(){return this._cursors.getLastAddedCursorIndex()}getCursorStates(){return this._cursors.getAll()}setStates(e,t,i,n){let s=!1;const r=this.context.cursorConfig.multiCursorLimit;n!==null&&n.length>r&&(n=n.slice(0,r),s=!0);const a=_v.from(this._model,this);return this._cursors.setStates(n),this._cursors.normalize(),this._columnSelectData=null,this._validateAutoClosedActions(),this._emitStateChangedIfNecessary(e,t,i,a,s)}setCursorColumnSelectData(e){this._columnSelectData=e}revealAll(e,t,i,n,s,r){const a=this._cursors.getViewPositions();let l=null,c=null;a.length>1?c=this._cursors.getViewSelections():l=x.fromPositions(a[0],a[0]),e.emitViewEvent(new pv(t,i,l,c,n,s,r))}revealPrimary(e,t,i,n,s,r){const l=[this._cursors.getPrimaryCursor().viewState.selection];e.emitViewEvent(new pv(t,i,null,l,n,s,r))}saveState(){const e=[],t=this._cursors.getSelections();for(let i=0,n=t.length;i<n;i++){const s=t[i];e.push({inSelectionMode:!s.isEmpty(),selectionStart:{lineNumber:s.selectionStartLineNumber,column:s.selectionStartColumn},position:{lineNumber:s.positionLineNumber,column:s.positionColumn}})}return e}restoreState(e,t){const i=[];for(let n=0,s=t.length;n<s;n++){const r=t[n];let a=1,l=1;r.position&&r.position.lineNumber&&(a=r.position.lineNumber),r.position&&r.position.column&&(l=r.position.column);let c=a,d=l;r.selectionStart&&r.selectionStart.lineNumber&&(c=r.selectionStart.lineNumber),r.selectionStart&&r.selectionStart.column&&(d=r.selectionStart.column),i.push({selectionStartLineNumber:c,selectionStartColumn:d,positionLineNumber:a,positionColumn:l})}this.setStates(e,"restoreState",0,ft.fromModelSelections(i)),this.revealAll(e,"restoreState",!1,0,!0,1)}onModelContentChanged(e,t){if(t instanceof mK){if(this._isHandling)return;this._isHandling=!0;try{this.setStates(e,"modelChange",0,this.getCursorStates())}finally{this._isHandling=!1}}else{const i=t.rawContentChangedEvent;if(this._knownModelVersionId=i.versionId,this._isHandling)return;const n=i.containsEvent(1);if(this._prevEditOperationType=0,n)this._cursors.dispose(),this._cursors=new HH(this.context),this._validateAutoClosedActions(),this._emitStateChangedIfNecessary(e,"model",1,null,!1);else if(this._hasFocus&&i.resultingSelection&&i.resultingSelection.length>0){const s=ft.fromModelSelections(i.resultingSelection);this.setStates(e,"modelChange",i.isUndoing?5:i.isRedoing?6:2,s)&&this.revealAll(e,"modelChange",!1,0,!0,0)}else{const s=this._cursors.readSelectionFromMarkers();this.setStates(e,"modelChange",2,ft.fromModelSelections(s))}}}getSelection(){return this._cursors.getPrimaryCursor().modelState.selection}getTopMostViewPosition(){return this._cursors.getTopMostViewPosition()}getBottomMostViewPosition(){return this._cursors.getBottomMostViewPosition()}getCursorColumnSelectData(){if(this._columnSelectData)return this._columnSelectData;const e=this._cursors.getPrimaryCursor(),t=e.viewState.selectionStart.getStartPosition(),i=e.viewState.position;return{isReal:!1,fromViewLineNumber:t.lineNumber,fromViewVisualColumn:this.context.cursorConfig.visibleColumnFromColumn(this._viewModel,t),toViewLineNumber:i.lineNumber,toViewVisualColumn:this.context.cursorConfig.visibleColumnFromColumn(this._viewModel,i)}}getSelections(){return this._cursors.getSelections()}setSelections(e,t,i,n){this.setStates(e,t,n,ft.fromModelSelections(i))}getPrevEditOperationType(){return this._prevEditOperationType}setPrevEditOperationType(e){this._prevEditOperationType=e}_pushAutoClosedAction(e,t){const i=[],n=[];for(let a=0,l=e.length;a<l;a++)i.push({range:e[a],options:{description:"auto-closed-character",inlineClassName:"auto-closed-character",stickiness:1}}),n.push({range:t[a],options:{description:"auto-closed-enclosing",stickiness:1}});const s=this._model.deltaDecorations([],i),r=this._model.deltaDecorations([],n);this._autoClosedActions.push(new zH(this._model,s,r))}_executeEditOperation(e){if(!e)return;e.shouldPushStackElementBefore&&this._model.pushStackElement();const t=fCe.executeCommands(this._model,this._cursors.getSelections(),e.commands);if(t){this._interpretCommandResult(t);const i=[],n=[];for(let s=0;s<e.commands.length;s++){const r=e.commands[s];r instanceof Z3&&r.enclosingRange&&r.closeCharacterRange&&(i.push(r.closeCharacterRange),n.push(r.enclosingRange))}i.length>0&&this._pushAutoClosedAction(i,n),this._prevEditOperationType=e.type}e.shouldPushStackElementAfter&&this._model.pushStackElement()}_interpretCommandResult(e){(!e||e.length===0)&&(e=this._cursors.readSelectionFromMarkers()),this._columnSelectData=null,this._cursors.setSelections(e),this._cursors.normalize()}_emitStateChangedIfNecessary(e,t,i,n,s){const r=_v.from(this._model,this);if(r.equals(n))return!1;const a=this._cursors.getSelections(),l=this._cursors.getViewSelections();if(e.emitViewEvent(new Gbe(l,a,i)),!n||n.cursorState.length!==r.cursorState.length||r.cursorState.some((c,d)=>!c.modelState.equals(n.cursorState[d].modelState))){const c=n?n.cursorState.map(h=>h.modelState.selection):null,d=n?n.modelVersionId:0;e.emitOutgoingEvent(new fx(c,a,d,r.modelVersionId,t||"keyboard",i,s))}return!0}_findAutoClosingPairs(e){if(!e.length)return null;const t=[];for(let i=0,n=e.length;i<n;i++){const s=e[i];if(!s.text||s.text.indexOf(`
`)>=0)return null;const r=s.text.match(/([)\]}>'"`])([^)\]}>'"`]*)$/);if(!r)return null;const a=r[1],l=this.context.cursorConfig.autoClosingPairs.autoClosingPairsCloseSingleChar.get(a);if(!l||l.length!==1)return null;const c=l[0].open,d=s.text.length-r[2].length-1,h=s.text.lastIndexOf(c,d-1);if(h===-1)return null;t.push([h,d])}return t}executeEdits(e,t,i,n){let s=null;t==="snippet"&&(s=this._findAutoClosingPairs(i)),s&&(i[0]._isTracked=!0);const r=[],a=[],l=this._model.pushEditOperations(this.getSelections(),i,c=>{if(s)for(let h=0,u=s.length;h<u;h++){const[g,f]=s[h],p=c[h],_=p.range.startLineNumber,b=p.range.startColumn-1+g,C=p.range.startColumn-1+f;r.push(new x(_,C+1,_,C+2)),a.push(new x(_,b+1,_,C+2))}const d=n(c);return d&&(this._isHandling=!0),d});l&&(this._isHandling=!1,this.setSelections(e,t,l,0)),r.length>0&&this._pushAutoClosedAction(r,a)}_executeEdit(e,t,i,n=0){if(this.context.cursorConfig.readOnly)return;const s=_v.from(this._model,this);this._cursors.stopTrackingSelections(),this._isHandling=!0;try{this._cursors.ensureValidState(),e()}catch(r){Oe(r)}this._isHandling=!1,this._cursors.startTrackingSelections(),this._validateAutoClosedActions(),this._emitStateChangedIfNecessary(t,i,n,s,!1)&&this.revealAll(t,i,!1,0,!0,0)}getAutoClosedCharacters(){return zH.getAllAutoClosedCharacters(this._autoClosedActions)}startComposition(e){this._compositionState=new bv(this._model,this.getSelections())}endComposition(e,t){const i=this._compositionState?this._compositionState.deduceOutcome(this._model,this.getSelections()):null;this._compositionState=null,this._executeEdit(()=>{t==="keyboard"&&this._executeEditOperation(Ud.compositionEndWithInterceptors(this._prevEditOperationType,this.context.cursorConfig,this._model,i,this.getSelections(),this.getAutoClosedCharacters()))},e,t)}type(e,t,i){this._executeEdit(()=>{if(i==="keyboard"){const n=t.length;let s=0;for(;s<n;){const r=J4(t,s),a=t.substr(s,r);this._executeEditOperation(Ud.typeWithInterceptors(!!this._compositionState,this._prevEditOperationType,this.context.cursorConfig,this._model,this.getSelections(),this.getAutoClosedCharacters(),a)),s+=r}}else this._executeEditOperation(Ud.typeWithoutInterceptors(this._prevEditOperationType,this.context.cursorConfig,this._model,this.getSelections(),t))},e,i)}compositionType(e,t,i,n,s,r){if(t.length===0&&i===0&&n===0){if(s!==0){const a=this.getSelections().map(l=>{const c=l.getPosition();return new pe(c.lineNumber,c.column+s,c.lineNumber,c.column+s)});this.setSelections(e,r,a,0)}return}this._executeEdit(()=>{this._executeEditOperation(Ud.compositionType(this._prevEditOperationType,this.context.cursorConfig,this._model,this.getSelections(),t,i,n,s))},e,r)}paste(e,t,i,n,s){this._executeEdit(()=>{this._executeEditOperation(Ud.paste(this.context.cursorConfig,this._model,this.getSelections(),t,i,n||[]))},e,s,4)}cut(e,t){this._executeEdit(()=>{this._executeEditOperation(Bf.cut(this.context.cursorConfig,this._model,this.getSelections()))},e,t)}executeCommand(e,t,i){this._executeEdit(()=>{this._cursors.killSecondaryCursors(),this._executeEditOperation(new Fs(0,[t],{shouldPushStackElementBefore:!1,shouldPushStackElementAfter:!1}))},e,i)}executeCommands(e,t,i){this._executeEdit(()=>{this._executeEditOperation(new Fs(0,t,{shouldPushStackElementBefore:!1,shouldPushStackElementAfter:!1}))},e,i)}}class _v{static from(e,t){return new _v(e.getVersionId(),t.getCursorStates())}constructor(e,t){this.modelVersionId=e,this.cursorState=t}equals(e){if(!e||this.modelVersionId!==e.modelVersionId||this.cursorState.length!==e.cursorState.length)return!1;for(let t=0,i=this.cursorState.length;t<i;t++)if(!this.cursorState[t].equals(e.cursorState[t]))return!1;return!0}}class zH{static getAllAutoClosedCharacters(e){let t=[];for(const i of e)t=t.concat(i.getAutoClosedCharactersRanges());return t}constructor(e,t,i){this._model=e,this._autoClosedCharactersDecorations=t,this._autoClosedEnclosingDecorations=i}dispose(){this._autoClosedCharactersDecorations=this._model.deltaDecorations(this._autoClosedCharactersDecorations,[]),this._autoClosedEnclosingDecorations=this._model.deltaDecorations(this._autoClosedEnclosingDecorations,[])}getAutoClosedCharactersRanges(){const e=[];for(let t=0;t<this._autoClosedCharactersDecorations.length;t++){const i=this._model.getDecorationRange(this._autoClosedCharactersDecorations[t]);i&&e.push(i)}return e}isValid(e){const t=[];for(let i=0;i<this._autoClosedEnclosingDecorations.length;i++){const n=this._model.getDecorationRange(this._autoClosedEnclosingDecorations[i]);if(n&&(t.push(n),n.startLineNumber!==n.endLineNumber))return!1}t.sort(x.compareRangesUsingStarts),e.sort(x.compareRangesUsingStarts);for(let i=0;i<e.length;i++)if(i>=t.length||!t[i].strictContainsRange(e[i]))return!1;return!0}}class fCe{static executeCommands(e,t,i){const n={model:e,selectionsBefore:t,trackedRanges:[],trackedRangesDirection:[]},s=this._innerExecuteCommands(n,i);for(let r=0,a=n.trackedRanges.length;r<a;r++)n.model._setTrackedRange(n.trackedRanges[r],null,0);return s}static _innerExecuteCommands(e,t){if(this._arrayIsEmpty(t))return null;const i=this._getEditOperations(e,t);if(i.operations.length===0)return null;const n=i.operations,s=this._getLoserCursorMap(n);if(s.hasOwnProperty("0"))return console.warn("Ignoring commands"),null;const r=[];for(let c=0,d=n.length;c<d;c++)s.hasOwnProperty(n[c].identifier.major.toString())||r.push(n[c]);i.hadTrackedEditOperation&&r.length>0&&(r[0]._isTracked=!0);let a=e.model.pushEditOperations(e.selectionsBefore,r,c=>{const d=[];for(let g=0;g<e.selectionsBefore.length;g++)d[g]=[];for(const g of c)g.identifier&&d[g.identifier.major].push(g);const h=(g,f)=>g.identifier.minor-f.identifier.minor,u=[];for(let g=0;g<e.selectionsBefore.length;g++)d[g].length>0?(d[g].sort(h),u[g]=t[g].computeCursorState(e.model,{getInverseEditOperations:()=>d[g],getTrackedSelection:f=>{const p=parseInt(f,10),_=e.model._getTrackedRange(e.trackedRanges[p]);return e.trackedRangesDirection[p]===0?new pe(_.startLineNumber,_.startColumn,_.endLineNumber,_.endColumn):new pe(_.endLineNumber,_.endColumn,_.startLineNumber,_.startColumn)}})):u[g]=e.selectionsBefore[g];return u});a||(a=e.selectionsBefore);const l=[];for(const c in s)s.hasOwnProperty(c)&&l.push(parseInt(c,10));l.sort((c,d)=>d-c);for(const c of l)a.splice(c,1);return a}static _arrayIsEmpty(e){for(let t=0,i=e.length;t<i;t++)if(e[t])return!1;return!0}static _getEditOperations(e,t){let i=[],n=!1;for(let s=0,r=t.length;s<r;s++){const a=t[s];if(a){const l=this._getEditOperationsFromCommand(e,s,a);i=i.concat(l.operations),n=n||l.hadTrackedEditOperation}}return{operations:i,hadTrackedEditOperation:n}}static _getEditOperationsFromCommand(e,t,i){const n=[];let s=0;const r=(h,u,g=!1)=>{x.isEmpty(h)&&u===""||n.push({identifier:{major:t,minor:s++},range:h,text:u,forceMoveMarkers:g,isAutoWhitespaceEdit:i.insertsAutoWhitespace})};let a=!1;const d={addEditOperation:r,addTrackedEditOperation:(h,u,g)=>{a=!0,r(h,u,g)},trackSelection:(h,u)=>{const g=pe.liftSelection(h);let f;if(g.isEmpty())if(typeof u=="boolean")u?f=2:f=3;else{const b=e.model.getLineMaxColumn(g.startLineNumber);g.startColumn===b?f=2:f=3}else f=1;const p=e.trackedRanges.length,_=e.model._setTrackedRange(null,g,f);return e.trackedRanges[p]=_,e.trackedRangesDirection[p]=g.getDirection(),p.toString()}};try{i.getEditOperations(e.model,d)}catch(h){return Oe(h),{operations:[],hadTrackedEditOperation:!1}}return{operations:n,hadTrackedEditOperation:a}}static _getLoserCursorMap(e){e=e.slice(0),e.sort((i,n)=>-x.compareRangesUsingEnds(i.range,n.range));const t={};for(let i=1;i<e.length;i++){const n=e[i-1],s=e[i];if(x.getStartPosition(n.range).isBefore(x.getEndPosition(s.range))){let r;n.identifier.major>s.identifier.major?r=n.identifier.major:r=s.identifier.major,t[r.toString()]=!0;for(let a=0;a<e.length;a++)e[a].identifier.major===r&&(e.splice(a,1),a<i&&i--,a--);i>0&&i--}}return t}}class mCe{constructor(e,t,i){this.text=e,this.startSelection=t,this.endSelection=i}}class bv{static _capture(e,t){const i=[];for(const n of t){if(n.startLineNumber!==n.endLineNumber)return null;i.push(new mCe(e.getLineContent(n.startLineNumber),n.startColumn-1,n.endColumn-1))}return i}constructor(e,t){this._original=bv._capture(e,t)}deduceOutcome(e,t){if(!this._original)return null;const i=bv._capture(e,t);if(!i||this._original.length!==i.length)return null;const n=[];for(let s=0,r=this._original.length;s<r;s++)n.push(bv._deduceOutcome(this._original[s],i[s]));return n}static _deduceOutcome(e,t){const i=Math.min(e.startSelection,t.startSelection,Oh(e.text,t.text)),n=Math.min(e.text.length-e.endSelection,t.text.length-t.endSelection,By(e.text,t.text)),s=e.text.substring(i,e.text.length-n),r=t.text.substring(i,t.text.length-n);return new U_e(s,e.startSelection-i,e.endSelection-i,r,t.startSelection-i,t.endSelection-i)}}class pCe{constructor(){this._hasPending=!1,this._inserts=[],this._changes=[],this._removes=[]}insert(e){this._hasPending=!0,this._inserts.push(e)}change(e){this._hasPending=!0,this._changes.push(e)}remove(e){this._hasPending=!0,this._removes.push(e)}mustCommit(){return this._hasPending}commit(e){if(!this._hasPending)return;const t=this._inserts,i=this._changes,n=this._removes;this._hasPending=!1,this._inserts=[],this._changes=[],this._removes=[],e._commitPendingChanges(t,i,n)}}class _Ce{constructor(e,t,i,n,s){this.id=e,this.afterLineNumber=t,this.ordinal=i,this.height=n,this.minWidth=s,this.prefixSum=0}}var mh;let bCe=(mh=class{constructor(e,t,i,n){this._instanceId=oU(++mh.INSTANCE_COUNT),this._pendingChanges=new pCe,this._lastWhitespaceId=0,this._arr=[],this._prefixSumValidIndex=-1,this._minWidth=-1,this._lineCount=e,this._lineHeight=t,this._paddingTop=i,this._paddingBottom=n}static findInsertionIndex(e,t,i){let n=0,s=e.length;for(;n<s;){const r=n+s>>>1;t===e[r].afterLineNumber?i<e[r].ordinal?s=r:n=r+1:t<e[r].afterLineNumber?s=r:n=r+1}return n}setLineHeight(e){this._checkPendingChanges(),this._lineHeight=e}setPadding(e,t){this._paddingTop=e,this._paddingBottom=t}onFlushed(e){this._checkPendingChanges(),this._lineCount=e}changeWhitespace(e){let t=!1;try{e({insertWhitespace:(n,s,r,a)=>{t=!0,n=n|0,s=s|0,r=r|0,a=a|0;const l=this._instanceId+ ++this._lastWhitespaceId;return this._pendingChanges.insert(new _Ce(l,n,s,r,a)),l},changeOneWhitespace:(n,s,r)=>{t=!0,s=s|0,r=r|0,this._pendingChanges.change({id:n,newAfterLineNumber:s,newHeight:r})},removeWhitespace:n=>{t=!0,this._pendingChanges.remove({id:n})}})}finally{this._pendingChanges.commit(this)}return t}_commitPendingChanges(e,t,i){if((e.length>0||i.length>0)&&(this._minWidth=-1),e.length+t.length+i.length<=1){for(const l of e)this._insertWhitespace(l);for(const l of t)this._changeOneWhitespace(l.id,l.newAfterLineNumber,l.newHeight);for(const l of i){const c=this._findWhitespaceIndex(l.id);c!==-1&&this._removeWhitespace(c)}return}const n=new Set;for(const l of i)n.add(l.id);const s=new Map;for(const l of t)s.set(l.id,l);const r=l=>{const c=[];for(const d of l)if(!n.has(d.id)){if(s.has(d.id)){const h=s.get(d.id);d.afterLineNumber=h.newAfterLineNumber,d.height=h.newHeight}c.push(d)}return c},a=r(this._arr).concat(r(e));a.sort((l,c)=>l.afterLineNumber===c.afterLineNumber?l.ordinal-c.ordinal:l.afterLineNumber-c.afterLineNumber),this._arr=a,this._prefixSumValidIndex=-1}_checkPendingChanges(){this._pendingChanges.mustCommit()&&this._pendingChanges.commit(this)}_insertWhitespace(e){const t=mh.findInsertionIndex(this._arr,e.afterLineNumber,e.ordinal);this._arr.splice(t,0,e),this._prefixSumValidIndex=Math.min(this._prefixSumValidIndex,t-1)}_findWhitespaceIndex(e){const t=this._arr;for(let i=0,n=t.length;i<n;i++)if(t[i].id===e)return i;return-1}_changeOneWhitespace(e,t,i){const n=this._findWhitespaceIndex(e);if(n!==-1&&(this._arr[n].height!==i&&(this._arr[n].height=i,this._prefixSumValidIndex=Math.min(this._prefixSumValidIndex,n-1)),this._arr[n].afterLineNumber!==t)){const s=this._arr[n];this._removeWhitespace(n),s.afterLineNumber=t,this._insertWhitespace(s)}}_removeWhitespace(e){this._arr.splice(e,1),this._prefixSumValidIndex=Math.min(this._prefixSumValidIndex,e-1)}onLinesDeleted(e,t){this._checkPendingChanges(),e=e|0,t=t|0,this._lineCount-=t-e+1;for(let i=0,n=this._arr.length;i<n;i++){const s=this._arr[i].afterLineNumber;e<=s&&s<=t?this._arr[i].afterLineNumber=e-1:s>t&&(this._arr[i].afterLineNumber-=t-e+1)}}onLinesInserted(e,t){this._checkPendingChanges(),e=e|0,t=t|0,this._lineCount+=t-e+1;for(let i=0,n=this._arr.length;i<n;i++){const s=this._arr[i].afterLineNumber;e<=s&&(this._arr[i].afterLineNumber+=t-e+1)}}getWhitespacesTotalHeight(){return this._checkPendingChanges(),this._arr.length===0?0:this.getWhitespacesAccumulatedHeight(this._arr.length-1)}getWhitespacesAccumulatedHeight(e){this._checkPendingChanges(),e=e|0;let t=Math.max(0,this._prefixSumValidIndex+1);t===0&&(this._arr[0].prefixSum=this._arr[0].height,t++);for(let i=t;i<=e;i++)this._arr[i].prefixSum=this._arr[i-1].prefixSum+this._arr[i].height;return this._prefixSumValidIndex=Math.max(this._prefixSumValidIndex,e),this._arr[e].prefixSum}getLinesTotalHeight(){this._checkPendingChanges();const e=this._lineHeight*this._lineCount,t=this.getWhitespacesTotalHeight();return e+t+this._paddingTop+this._paddingBottom}getWhitespaceAccumulatedHeightBeforeLineNumber(e){this._checkPendingChanges(),e=e|0;const t=this._findLastWhitespaceBeforeLineNumber(e);return t===-1?0:this.getWhitespacesAccumulatedHeight(t)}_findLastWhitespaceBeforeLineNumber(e){e=e|0;const t=this._arr;let i=0,n=t.length-1;for(;i<=n;){const r=(n-i|0)/2|0,a=i+r|0;if(t[a].afterLineNumber<e){if(a+1>=t.length||t[a+1].afterLineNumber>=e)return a;i=a+1|0}else n=a-1|0}return-1}_findFirstWhitespaceAfterLineNumber(e){e=e|0;const i=this._findLastWhitespaceBeforeLineNumber(e)+1;return i<this._arr.length?i:-1}getFirstWhitespaceIndexAfterLineNumber(e){return this._checkPendingChanges(),e=e|0,this._findFirstWhitespaceAfterLineNumber(e)}getVerticalOffsetForLineNumber(e,t=!1){this._checkPendingChanges(),e=e|0;let i;e>1?i=this._lineHeight*(e-1):i=0;const n=this.getWhitespaceAccumulatedHeightBeforeLineNumber(e-(t?1:0));return i+n+this._paddingTop}getVerticalOffsetAfterLineNumber(e,t=!1){this._checkPendingChanges(),e=e|0;const i=this._lineHeight*e,n=this.getWhitespaceAccumulatedHeightBeforeLineNumber(e+(t?1:0));return i+n+this._paddingTop}getWhitespaceMinWidth(){if(this._checkPendingChanges(),this._minWidth===-1){let e=0;for(let t=0,i=this._arr.length;t<i;t++)e=Math.max(e,this._arr[t].minWidth);this._minWidth=e}return this._minWidth}isAfterLines(e){this._checkPendingChanges();const t=this.getLinesTotalHeight();return e>t}isInTopPadding(e){return this._paddingTop===0?!1:(this._checkPendingChanges(),e<this._paddingTop)}isInBottomPadding(e){if(this._paddingBottom===0)return!1;this._checkPendingChanges();const t=this.getLinesTotalHeight();return e>=t-this._paddingBottom}getLineNumberAtOrAfterVerticalOffset(e){if(this._checkPendingChanges(),e=e|0,e<0)return 1;const t=this._lineCount|0,i=this._lineHeight;let n=1,s=t;for(;n<s;){const r=(n+s)/2|0,a=this.getVerticalOffsetForLineNumber(r)|0;if(e>=a+i)n=r+1;else{if(e>=a)return r;s=r}}return n>t?t:n}getLinesViewportData(e,t){this._checkPendingChanges(),e=e|0,t=t|0;const i=this._lineHeight,n=this.getLineNumberAtOrAfterVerticalOffset(e)|0,s=this.getVerticalOffsetForLineNumber(n)|0;let r=this._lineCount|0,a=this.getFirstWhitespaceIndexAfterLineNumber(n)|0;const l=this.getWhitespacesCount()|0;let c,d;a===-1?(a=l,d=r+1,c=0):(d=this.getAfterLineNumberForWhitespaceIndex(a)|0,c=this.getHeightForWhitespaceIndex(a)|0);let h=s,u=h;const g=5e5;let f=0;s>=g&&(f=Math.floor(s/g)*g,f=Math.floor(f/i)*i,u-=f);const p=[],_=e+(t-e)/2;let b=-1;for(let S=n;S<=r;S++){if(b===-1){const L=h,k=h+i;(L<=_&&_<k||L>_)&&(b=S)}for(h+=i,p[S-n]=u,u+=i;d===S;)u+=c,h+=c,a++,a>=l?d=r+1:(d=this.getAfterLineNumberForWhitespaceIndex(a)|0,c=this.getHeightForWhitespaceIndex(a)|0);if(h>=t){r=S;break}}b===-1&&(b=r);const C=this.getVerticalOffsetForLineNumber(r)|0;let v=n,w=r;return v<w&&s<e&&v++,v<w&&C+i>t&&w--,{bigNumbersDelta:f,startLineNumber:n,endLineNumber:r,relativeVerticalOffset:p,centeredLineNumber:b,completelyVisibleStartLineNumber:v,completelyVisibleEndLineNumber:w,lineHeight:this._lineHeight}}getVerticalOffsetForWhitespaceIndex(e){this._checkPendingChanges(),e=e|0;const t=this.getAfterLineNumberForWhitespaceIndex(e);let i;t>=1?i=this._lineHeight*t:i=0;let n;return e>0?n=this.getWhitespacesAccumulatedHeight(e-1):n=0,i+n+this._paddingTop}getWhitespaceIndexAtOrAfterVerticallOffset(e){this._checkPendingChanges(),e=e|0;let t=0,i=this.getWhitespacesCount()-1;if(i<0)return-1;const n=this.getVerticalOffsetForWhitespaceIndex(i),s=this.getHeightForWhitespaceIndex(i);if(e>=n+s)return-1;for(;t<i;){const r=Math.floor((t+i)/2),a=this.getVerticalOffsetForWhitespaceIndex(r),l=this.getHeightForWhitespaceIndex(r);if(e>=a+l)t=r+1;else{if(e>=a)return r;i=r}}return t}getWhitespaceAtVerticalOffset(e){this._checkPendingChanges(),e=e|0;const t=this.getWhitespaceIndexAtOrAfterVerticallOffset(e);if(t<0||t>=this.getWhitespacesCount())return null;const i=this.getVerticalOffsetForWhitespaceIndex(t);if(i>e)return null;const n=this.getHeightForWhitespaceIndex(t),s=this.getIdForWhitespaceIndex(t),r=this.getAfterLineNumberForWhitespaceIndex(t);return{id:s,afterLineNumber:r,verticalOffset:i,height:n}}getWhitespaceViewportData(e,t){this._checkPendingChanges(),e=e|0,t=t|0;const i=this.getWhitespaceIndexAtOrAfterVerticallOffset(e),n=this.getWhitespacesCount()-1;if(i<0)return[];const s=[];for(let r=i;r<=n;r++){const a=this.getVerticalOffsetForWhitespaceIndex(r),l=this.getHeightForWhitespaceIndex(r);if(a>=t)break;s.push({id:this.getIdForWhitespaceIndex(r),afterLineNumber:this.getAfterLineNumberForWhitespaceIndex(r),verticalOffset:a,height:l})}return s}getWhitespaces(){return this._checkPendingChanges(),this._arr.slice(0)}getWhitespacesCount(){return this._checkPendingChanges(),this._arr.length}getIdForWhitespaceIndex(e){return this._checkPendingChanges(),e=e|0,this._arr[e].id}getAfterLineNumberForWhitespaceIndex(e){return this._checkPendingChanges(),e=e|0,this._arr[e].afterLineNumber}getHeightForWhitespaceIndex(e){return this._checkPendingChanges(),e=e|0,this._arr[e].height}},mh.INSTANCE_COUNT=0,mh);const CCe=125;class TC{constructor(e,t,i,n){e=e|0,t=t|0,i=i|0,n=n|0,e<0&&(e=0),t<0&&(t=0),i<0&&(i=0),n<0&&(n=0),this.width=e,this.contentWidth=t,this.scrollWidth=Math.max(e,t),this.height=i,this.contentHeight=n,this.scrollHeight=Math.max(i,n)}equals(e){return this.width===e.width&&this.contentWidth===e.contentWidth&&this.height===e.height&&this.contentHeight===e.contentHeight}}class vCe extends F{constructor(e,t){super(),this._onDidContentSizeChange=this._register(new P),this.onDidContentSizeChange=this._onDidContentSizeChange.event,this._dimensions=new TC(0,0,0,0),this._scrollable=this._register(new wb({forceIntegerValues:!0,smoothScrollDuration:e,scheduleAtNextAnimationFrame:t})),this.onDidScroll=this._scrollable.onScroll}getScrollable(){return this._scrollable}setSmoothScrollDuration(e){this._scrollable.setSmoothScrollDuration(e)}validateScrollPosition(e){return this._scrollable.validateScrollPosition(e)}getScrollDimensions(){return this._dimensions}setScrollDimensions(e){if(this._dimensions.equals(e))return;const t=this._dimensions;this._dimensions=e,this._scrollable.setScrollDimensions({width:e.width,scrollWidth:e.scrollWidth,height:e.height,scrollHeight:e.scrollHeight},!0);const i=t.contentWidth!==e.contentWidth,n=t.contentHeight!==e.contentHeight;(i||n)&&this._onDidContentSizeChange.fire(new tW(t.contentWidth,t.contentHeight,e.contentWidth,e.contentHeight))}getFutureScrollPosition(){return this._scrollable.getFutureScrollPosition()}getCurrentScrollPosition(){return this._scrollable.getCurrentScrollPosition()}setScrollPositionNow(e){this._scrollable.setScrollPositionNow(e)}setScrollPositionSmooth(e){this._scrollable.setScrollPositionSmooth(e)}hasPendingScrollAnimation(){return this._scrollable.hasPendingScrollAnimation()}}class wCe extends F{constructor(e,t,i){super(),this._configuration=e;const n=this._configuration.options,s=n.get(146),r=n.get(84);this._linesLayout=new bCe(t,n.get(67),r.top,r.bottom),this._maxLineWidth=0,this._overlayWidgetsMinWidth=0,this._scrollable=this._register(new vCe(0,i)),this._configureSmoothScrollDuration(),this._scrollable.setScrollDimensions(new TC(s.contentWidth,0,s.height,0)),this.onDidScroll=this._scrollable.onDidScroll,this.onDidContentSizeChange=this._scrollable.onDidContentSizeChange,this._updateHeight()}dispose(){super.dispose()}getScrollable(){return this._scrollable.getScrollable()}onHeightMaybeChanged(){this._updateHeight()}_configureSmoothScrollDuration(){this._scrollable.setSmoothScrollDuration(this._configuration.options.get(115)?CCe:0)}onConfigurationChanged(e){const t=this._configuration.options;if(e.hasChanged(67)&&this._linesLayout.setLineHeight(t.get(67)),e.hasChanged(84)){const i=t.get(84);this._linesLayout.setPadding(i.top,i.bottom)}if(e.hasChanged(146)){const i=t.get(146),n=i.contentWidth,s=i.height,r=this._scrollable.getScrollDimensions(),a=r.contentWidth;this._scrollable.setScrollDimensions(new TC(n,r.contentWidth,s,this._getContentHeight(n,s,a)))}else this._updateHeight();e.hasChanged(115)&&this._configureSmoothScrollDuration()}onFlushed(e){this._linesLayout.onFlushed(e)}onLinesDeleted(e,t){this._linesLayout.onLinesDeleted(e,t)}onLinesInserted(e,t){this._linesLayout.onLinesInserted(e,t)}_getHorizontalScrollbarHeight(e,t){const n=this._configuration.options.get(104);return n.horizontal===2||e>=t?0:n.horizontalScrollbarSize}_getContentHeight(e,t,i){const n=this._configuration.options;let s=this._linesLayout.getLinesTotalHeight();return n.get(106)?s+=Math.max(0,t-n.get(67)-n.get(84).bottom):n.get(104).ignoreHorizontalScrollbarInContentHeight||(s+=this._getHorizontalScrollbarHeight(e,i)),s}_updateHeight(){const e=this._scrollable.getScrollDimensions(),t=e.width,i=e.height,n=e.contentWidth;this._scrollable.setScrollDimensions(new TC(t,e.contentWidth,i,this._getContentHeight(t,i,n)))}getCurrentViewport(){const e=this._scrollable.getScrollDimensions(),t=this._scrollable.getCurrentScrollPosition();return new gH(t.scrollTop,t.scrollLeft,e.width,e.height)}getFutureViewport(){const e=this._scrollable.getScrollDimensions(),t=this._scrollable.getFutureScrollPosition();return new gH(t.scrollTop,t.scrollLeft,e.width,e.height)}_computeContentWidth(){const e=this._configuration.options,t=this._maxLineWidth,i=e.get(147),n=e.get(50),s=e.get(146);if(i.isViewportWrapping){const r=e.get(73);return t>s.contentWidth+n.typicalHalfwidthCharacterWidth&&r.enabled&&r.side==="right"?t+s.verticalScrollbarWidth:t}else{const r=e.get(105)*n.typicalHalfwidthCharacterWidth,a=this._linesLayout.getWhitespaceMinWidth();return Math.max(t+r+s.verticalScrollbarWidth,a,this._overlayWidgetsMinWidth)}}setMaxLineWidth(e){this._maxLineWidth=e,this._updateContentWidth()}setOverlayWidgetsMinWidth(e){this._overlayWidgetsMinWidth=e,this._updateContentWidth()}_updateContentWidth(){const e=this._scrollable.getScrollDimensions();this._scrollable.setScrollDimensions(new TC(e.width,this._computeContentWidth(),e.height,e.contentHeight)),this._updateHeight()}saveState(){const e=this._scrollable.getFutureScrollPosition(),t=e.scrollTop,i=this._linesLayout.getLineNumberAtOrAfterVerticalOffset(t),n=this._linesLayout.getWhitespaceAccumulatedHeightBeforeLineNumber(i);return{scrollTop:t,scrollTopWithoutViewZones:t-n,scrollLeft:e.scrollLeft}}changeWhitespace(e){const t=this._linesLayout.changeWhitespace(e);return t&&this.onHeightMaybeChanged(),t}getVerticalOffsetForLineNumber(e,t=!1){return this._linesLayout.getVerticalOffsetForLineNumber(e,t)}getVerticalOffsetAfterLineNumber(e,t=!1){return this._linesLayout.getVerticalOffsetAfterLineNumber(e,t)}isAfterLines(e){return this._linesLayout.isAfterLines(e)}isInTopPadding(e){return this._linesLayout.isInTopPadding(e)}isInBottomPadding(e){return this._linesLayout.isInBottomPadding(e)}getLineNumberAtVerticalOffset(e){return this._linesLayout.getLineNumberAtOrAfterVerticalOffset(e)}getWhitespaceAtVerticalOffset(e){return this._linesLayout.getWhitespaceAtVerticalOffset(e)}getLinesViewportData(){const e=this.getCurrentViewport();return this._linesLayout.getLinesViewportData(e.top,e.top+e.height)}getLinesViewportDataAtScrollTop(e){const t=this._scrollable.getScrollDimensions();return e+t.height>t.scrollHeight&&(e=t.scrollHeight-t.height),e<0&&(e=0),this._linesLayout.getLinesViewportData(e,e+t.height)}getWhitespaceViewportData(){const e=this.getCurrentViewport();return this._linesLayout.getWhitespaceViewportData(e.top,e.top+e.height)}getWhitespaces(){return this._linesLayout.getWhitespaces()}getContentWidth(){return this._scrollable.getScrollDimensions().contentWidth}getScrollWidth(){return this._scrollable.getScrollDimensions().scrollWidth}getContentHeight(){return this._scrollable.getScrollDimensions().contentHeight}getScrollHeight(){return this._scrollable.getScrollDimensions().scrollHeight}getCurrentScrollLeft(){return this._scrollable.getCurrentScrollPosition().scrollLeft}getCurrentScrollTop(){return this._scrollable.getCurrentScrollPosition().scrollTop}validateScrollPosition(e){return this._scrollable.validateScrollPosition(e)}setScrollPosition(e,t){t===1?this._scrollable.setScrollPositionNow(e):this._scrollable.setScrollPositionSmooth(e)}hasPendingScrollAnimation(){return this._scrollable.hasPendingScrollAnimation()}deltaScrollNow(e,t){const i=this._scrollable.getCurrentScrollPosition();this._scrollable.setScrollPositionNow({scrollLeft:i.scrollLeft+e,scrollTop:i.scrollTop+t})}}class SCe{constructor(e,t,i,n,s){this.editorId=e,this.model=t,this.configuration=i,this._linesCollection=n,this._coordinatesConverter=s,this._decorationsCache=Object.create(null),this._cachedModelDecorationsResolver=null,this._cachedModelDecorationsResolverViewRange=null}_clearCachedModelDecorationsResolver(){this._cachedModelDecorationsResolver=null,this._cachedModelDecorationsResolverViewRange=null}dispose(){this._decorationsCache=Object.create(null),this._clearCachedModelDecorationsResolver()}reset(){this._decorationsCache=Object.create(null),this._clearCachedModelDecorationsResolver()}onModelDecorationsChanged(){this._decorationsCache=Object.create(null),this._clearCachedModelDecorationsResolver()}onLineMappingChanged(){this._decorationsCache=Object.create(null),this._clearCachedModelDecorationsResolver()}_getOrCreateViewModelDecoration(e){const t=e.id;let i=this._decorationsCache[t];if(!i){const n=e.range,s=e.options;let r;if(s.isWholeLine){const a=this._coordinatesConverter.convertModelPositionToViewPosition(new A(n.startLineNumber,1),0,!1,!0),l=this._coordinatesConverter.convertModelPositionToViewPosition(new A(n.endLineNumber,this.model.getLineMaxColumn(n.endLineNumber)),1);r=new x(a.lineNumber,a.column,l.lineNumber,l.column)}else r=this._coordinatesConverter.convertModelRangeToViewRange(n,1);i=new hq(r,s),this._decorationsCache[t]=i}return i}getMinimapDecorationsInRange(e){return this._getDecorationsInRange(e,!0,!1).decorations}getDecorationsViewportData(e){let t=this._cachedModelDecorationsResolver!==null;return t=t&&e.equalsRange(this._cachedModelDecorationsResolverViewRange),t||(this._cachedModelDecorationsResolver=this._getDecorationsInRange(e,!1,!1),this._cachedModelDecorationsResolverViewRange=e),this._cachedModelDecorationsResolver}getInlineDecorationsOnLine(e,t=!1,i=!1){const n=new x(e,this._linesCollection.getViewLineMinColumn(e),e,this._linesCollection.getViewLineMaxColumn(e));return this._getDecorationsInRange(n,t,i).inlineDecorations[0]}_getDecorationsInRange(e,t,i){const n=this._linesCollection.getDecorationsInRange(e,this.editorId,Ty(this.configuration.options),t,i),s=e.startLineNumber,r=e.endLineNumber,a=[];let l=0;const c=[];for(let d=s;d<=r;d++)c[d-s]=[];for(let d=0,h=n.length;d<h;d++){const u=n[d],g=u.options;if(!sW(this.model,u))continue;const f=this._getOrCreateViewModelDecoration(u),p=f.range;if(a[l++]=f,g.inlineClassName){const _=new cv(p,g.inlineClassName,g.inlineClassNameAffectsLetterSpacing?3:0),b=Math.max(s,p.startLineNumber),C=Math.min(r,p.endLineNumber);for(let v=b;v<=C;v++)c[v-s].push(_)}if(g.beforeContentClassName&&s<=p.startLineNumber&&p.startLineNumber<=r){const _=new cv(new x(p.startLineNumber,p.startColumn,p.startLineNumber,p.startColumn),g.beforeContentClassName,1);c[p.startLineNumber-s].push(_)}if(g.afterContentClassName&&s<=p.endLineNumber&&p.endLineNumber<=r){const _=new cv(new x(p.endLineNumber,p.endColumn,p.endLineNumber,p.endColumn),g.afterContentClassName,2);c[p.endLineNumber-s].push(_)}}return{decorations:a,inlineDecorations:c}}}function sW(o,e){return!(e.options.hideInCommentTokens&&oW(o,e)||e.options.hideInStringTokens&&rW(o,e))}function oW(o,e){return Aq(o,e.range,t=>t===1)}function rW(o,e){return Aq(o,e.range,t=>t===2)}function Aq(o,e,t){for(let i=e.startLineNumber;i<=e.endLineNumber;i++){const n=o.tokenization.getLineTokens(i),s=i===e.startLineNumber,r=i===e.endLineNumber;let a=s?n.findTokenIndexAtOffset(e.startColumn-1):0;for(;a<n.getCount()&&!(r&&n.getStartOffset(a)>e.endColumn-1);){if(!t(n.getStandardTokenType(a)))return!1;a++}}return!0}function xT(o,e){return o===null?e?mx.INSTANCE:px.INSTANCE:new yCe(o,e)}class yCe{constructor(e,t){this._projectionData=e,this._isVisible=t}isVisible(){return this._isVisible}setVisible(e){return this._isVisible=e,this}getProjectionData(){return this._projectionData}getViewLineCount(){return this._isVisible?this._projectionData.getOutputLineCount():0}getViewLineContent(e,t,i){this._assertVisible();const n=i>0?this._projectionData.breakOffsets[i-1]:0,s=this._projectionData.breakOffsets[i];let r;if(this._projectionData.injectionOffsets!==null){const a=this._projectionData.injectionOffsets.map((c,d)=>new Wa(0,0,c+1,this._projectionData.injectionOptions[d],0));r=Wa.applyInjectedText(e.getLineContent(t),a).substring(n,s)}else r=e.getValueInRange({startLineNumber:t,startColumn:n+1,endLineNumber:t,endColumn:s+1});return i>0&&(r=UH(this._projectionData.wrappedTextIndentLength)+r),r}getViewLineLength(e,t,i){return this._assertVisible(),this._projectionData.getLineLength(i)}getViewLineMinColumn(e,t,i){return this._assertVisible(),this._projectionData.getMinOutputOffset(i)+1}getViewLineMaxColumn(e,t,i){return this._assertVisible(),this._projectionData.getMaxOutputOffset(i)+1}getViewLineData(e,t,i){const n=new Array;return this.getViewLinesData(e,t,i,1,0,[!0],n),n[0]}getViewLinesData(e,t,i,n,s,r,a){this._assertVisible();const l=this._projectionData,c=l.injectionOffsets,d=l.injectionOptions;let h=null;if(c){h=[];let g=0,f=0;for(let p=0;p<l.getOutputLineCount();p++){const _=new Array;h[p]=_;const b=p>0?l.breakOffsets[p-1]:0,C=l.breakOffsets[p];for(;f<c.length;){const v=d[f].content.length,w=c[f]+g,S=w+v;if(w>C)break;if(b<S){const L=d[f];if(L.inlineClassName){const k=p>0?l.wrappedTextIndentLength:0,D=k+Math.max(w-b,0),N=k+Math.min(S-b,C-b);D!==N&&_.push(new _pe(D,N,L.inlineClassName,L.inlineClassNameAffectsLetterSpacing))}}if(S<=C)g+=v,f++;else break}}}let u;c?u=e.tokenization.getLineTokens(t).withInserted(c.map((g,f)=>({offset:g,text:d[f].content,tokenMetadata:cn.defaultTokenMetadata}))):u=e.tokenization.getLineTokens(t);for(let g=i;g<i+n;g++){const f=s+g-i;if(!r[f]){a[f]=null;continue}a[f]=this._getViewLineData(u,h?h[g]:null,g)}}_getViewLineData(e,t,i){this._assertVisible();const n=this._projectionData,s=i>0?n.wrappedTextIndentLength:0,r=i>0?n.breakOffsets[i-1]:0,a=n.breakOffsets[i],l=e.sliceAndInflate(r,a,s);let c=l.getLineContent();i>0&&(c=UH(n.wrappedTextIndentLength)+c);const d=this._projectionData.getMinOutputOffset(i)+1,h=c.length+1,u=i+1<this.getViewLineCount(),g=i===0?0:n.breakOffsetsVisibleColumn[i-1];return new B3(c,u,d,h,g,l,t)}getModelColumnOfViewPosition(e,t){return this._assertVisible(),this._projectionData.translateToInputOffset(e,t-1)+1}getViewPositionOfModelPosition(e,t,i=2){return this._assertVisible(),this._projectionData.translateToOutputPosition(t-1,i).toPosition(e)}getViewLineNumberOfModelPosition(e,t){this._assertVisible();const i=this._projectionData.translateToOutputPosition(t-1);return e+i.outputLineIndex}normalizePosition(e,t,i){const n=t.lineNumber-e;return this._projectionData.normalizeOutputPosition(e,t.column-1,i).toPosition(n)}getInjectedTextAt(e,t){return this._projectionData.getInjectedText(e,t-1)}_assertVisible(){if(!this._isVisible)throw new Error("Not supported")}}const fD=class fD{constructor(){}isVisible(){return!0}setVisible(e){return e?this:px.INSTANCE}getProjectionData(){return null}getViewLineCount(){return 1}getViewLineContent(e,t,i){return e.getLineContent(t)}getViewLineLength(e,t,i){return e.getLineLength(t)}getViewLineMinColumn(e,t,i){return e.getLineMinColumn(t)}getViewLineMaxColumn(e,t,i){return e.getLineMaxColumn(t)}getViewLineData(e,t,i){const n=e.tokenization.getLineTokens(t),s=n.getLineContent();return new B3(s,!1,1,s.length+1,0,n.inflate(),null)}getViewLinesData(e,t,i,n,s,r,a){if(!r[s]){a[s]=null;return}a[s]=this.getViewLineData(e,t,0)}getModelColumnOfViewPosition(e,t){return t}getViewPositionOfModelPosition(e,t){return new A(e,t)}getViewLineNumberOfModelPosition(e,t){return e}normalizePosition(e,t,i){return t}getInjectedTextAt(e,t){return null}};fD.INSTANCE=new fD;let mx=fD;const mD=class mD{constructor(){}isVisible(){return!1}setVisible(e){return e?mx.INSTANCE:this}getProjectionData(){return null}getViewLineCount(){return 0}getViewLineContent(e,t,i){throw new Error("Not supported")}getViewLineLength(e,t,i){throw new Error("Not supported")}getViewLineMinColumn(e,t,i){throw new Error("Not supported")}getViewLineMaxColumn(e,t,i){throw new Error("Not supported")}getViewLineData(e,t,i){throw new Error("Not supported")}getViewLinesData(e,t,i,n,s,r,a){throw new Error("Not supported")}getModelColumnOfViewPosition(e,t){throw new Error("Not supported")}getViewPositionOfModelPosition(e,t){throw new Error("Not supported")}getViewLineNumberOfModelPosition(e,t){throw new Error("Not supported")}normalizePosition(e,t,i){throw new Error("Not supported")}getInjectedTextAt(e,t){throw new Error("Not supported")}};mD.INSTANCE=new mD;let px=mD;const kT=[""];function UH(o){if(o>=kT.length)for(let e=1;e<=o;e++)kT[e]=LCe(e);return kT[o]}function LCe(o){return new Array(o+1).join(" ")}class xCe{constructor(e,t,i,n,s,r,a,l,c,d){this._editorId=e,this.model=t,this._validModelVersionId=-1,this._domLineBreaksComputerFactory=i,this._monospaceLineBreaksComputerFactory=n,this.fontInfo=s,this.tabSize=r,this.wrappingStrategy=a,this.wrappingColumn=l,this.wrappingIndent=c,this.wordBreak=d,this._constructLines(!0,null)}dispose(){this.hiddenAreasDecorationIds=this.model.deltaDecorations(this.hiddenAreasDecorationIds,[])}createCoordinatesConverter(){return new DCe(this)}_constructLines(e,t){this.modelLineProjections=[],e&&(this.hiddenAreasDecorationIds=this.model.deltaDecorations(this.hiddenAreasDecorationIds,[]));const i=this.model.getLinesContent(),n=this.model.getInjectedTextDecorations(this._editorId),s=i.length,r=this.createLineBreaksComputer(),a=new Qc(Wa.fromDecorations(n));for(let p=0;p<s;p++){const _=a.takeWhile(b=>b.lineNumber===p+1);r.addRequest(i[p],_,t?t[p]:null)}const l=r.finalize(),c=[],d=this.hiddenAreasDecorationIds.map(p=>this.model.getDecorationRange(p)).sort(x.compareRangesUsingStarts);let h=1,u=0,g=-1,f=g+1<d.length?u+1:s+2;for(let p=0;p<s;p++){const _=p+1;_===f&&(g++,h=d[g].startLineNumber,u=d[g].endLineNumber,f=g+1<d.length?u+1:s+2);const b=_>=h&&_<=u,C=xT(l[p],!b);c[p]=C.getViewLineCount(),this.modelLineProjections[p]=C}this._validModelVersionId=this.model.getVersionId(),this.projectedModelLineLineCounts=new Hse(c)}getHiddenAreas(){return this.hiddenAreasDecorationIds.map(e=>this.model.getDecorationRange(e))}setHiddenAreas(e){const t=e.map(u=>this.model.validateRange(u)),i=kCe(t),n=this.hiddenAreasDecorationIds.map(u=>this.model.getDecorationRange(u)).sort(x.compareRangesUsingStarts);if(i.length===n.length){let u=!1;for(let g=0;g<i.length;g++)if(!i[g].equalsRange(n[g])){u=!0;break}if(!u)return!1}const s=i.map(u=>({range:u,options:We.EMPTY}));this.hiddenAreasDecorationIds=this.model.deltaDecorations(this.hiddenAreasDecorationIds,s);const r=i;let a=1,l=0,c=-1,d=c+1<r.length?l+1:this.modelLineProjections.length+2,h=!1;for(let u=0;u<this.modelLineProjections.length;u++){const g=u+1;g===d&&(c++,a=r[c].startLineNumber,l=r[c].endLineNumber,d=c+1<r.length?l+1:this.modelLineProjections.length+2);let f=!1;if(g>=a&&g<=l?this.modelLineProjections[u].isVisible()&&(this.modelLineProjections[u]=this.modelLineProjections[u].setVisible(!1),f=!0):(h=!0,this.modelLineProjections[u].isVisible()||(this.modelLineProjections[u]=this.modelLineProjections[u].setVisible(!0),f=!0)),f){const p=this.modelLineProjections[u].getViewLineCount();this.projectedModelLineLineCounts.setValue(u,p)}}return h||this.setHiddenAreas([]),!0}modelPositionIsVisible(e,t){return e<1||e>this.modelLineProjections.length?!1:this.modelLineProjections[e-1].isVisible()}getModelLineViewLineCount(e){return e<1||e>this.modelLineProjections.length?1:this.modelLineProjections[e-1].getViewLineCount()}setTabSize(e){return this.tabSize===e?!1:(this.tabSize=e,this._constructLines(!1,null),!0)}setWrappingSettings(e,t,i,n,s){const r=this.fontInfo.equals(e),a=this.wrappingStrategy===t,l=this.wrappingColumn===i,c=this.wrappingIndent===n,d=this.wordBreak===s;if(r&&a&&l&&c&&d)return!1;const h=r&&a&&!l&&c&&d;this.fontInfo=e,this.wrappingStrategy=t,this.wrappingColumn=i,this.wrappingIndent=n,this.wordBreak=s;let u=null;if(h){u=[];for(let g=0,f=this.modelLineProjections.length;g<f;g++)u[g]=this.modelLineProjections[g].getProjectionData()}return this._constructLines(!1,u),!0}createLineBreaksComputer(){return(this.wrappingStrategy==="advanced"?this._domLineBreaksComputerFactory:this._monospaceLineBreaksComputerFactory).createLineBreaksComputer(this.fontInfo,this.tabSize,this.wrappingColumn,this.wrappingIndent,this.wordBreak)}onModelFlushed(){this._constructLines(!0,null)}onModelLinesDeleted(e,t,i){if(!e||e<=this._validModelVersionId)return null;const n=t===1?1:this.projectedModelLineLineCounts.getPrefixSum(t-1)+1,s=this.projectedModelLineLineCounts.getPrefixSum(i);return this.modelLineProjections.splice(t-1,i-t+1),this.projectedModelLineLineCounts.removeValues(t-1,i-t+1),new xP(n,s)}onModelLinesInserted(e,t,i,n){if(!e||e<=this._validModelVersionId)return null;const s=t>2&&!this.modelLineProjections[t-2].isVisible(),r=t===1?1:this.projectedModelLineLineCounts.getPrefixSum(t-1)+1;let a=0;const l=[],c=[];for(let d=0,h=n.length;d<h;d++){const u=xT(n[d],!s);l.push(u);const g=u.getViewLineCount();a+=g,c[d]=g}return this.modelLineProjections=this.modelLineProjections.slice(0,t-1).concat(l).concat(this.modelLineProjections.slice(t-1)),this.projectedModelLineLineCounts.insertValues(t-1,c),new kP(r,r+a-1)}onModelLineChanged(e,t,i){if(e!==null&&e<=this._validModelVersionId)return[!1,null,null,null];const n=t-1,s=this.modelLineProjections[n].getViewLineCount(),r=this.modelLineProjections[n].isVisible(),a=xT(i,r);this.modelLineProjections[n]=a;const l=this.modelLineProjections[n].getViewLineCount();let c=!1,d=0,h=-1,u=0,g=-1,f=0,p=-1;s>l?(d=this.projectedModelLineLineCounts.getPrefixSum(t-1)+1,h=d+l-1,f=h+1,p=f+(s-l)-1,c=!0):s<l?(d=this.projectedModelLineLineCounts.getPrefixSum(t-1)+1,h=d+s-1,u=h+1,g=u+(l-s)-1,c=!0):(d=this.projectedModelLineLineCounts.getPrefixSum(t-1)+1,h=d+l-1),this.projectedModelLineLineCounts.setValue(n,l);const _=d<=h?new Mq(d,h-d+1):null,b=u<=g?new kP(u,g):null,C=f<=p?new xP(f,p):null;return[c,_,b,C]}acceptVersionId(e){this._validModelVersionId=e,this.modelLineProjections.length===1&&!this.modelLineProjections[0].isVisible()&&this.setHiddenAreas([])}getViewLineCount(){return this.projectedModelLineLineCounts.getTotalSum()}_toValidViewLineNumber(e){if(e<1)return 1;const t=this.getViewLineCount();return e>t?t:e|0}getActiveIndentGuide(e,t,i){e=this._toValidViewLineNumber(e),t=this._toValidViewLineNumber(t),i=this._toValidViewLineNumber(i);const n=this.convertViewPositionToModelPosition(e,this.getViewLineMinColumn(e)),s=this.convertViewPositionToModelPosition(t,this.getViewLineMinColumn(t)),r=this.convertViewPositionToModelPosition(i,this.getViewLineMinColumn(i)),a=this.model.guides.getActiveIndentGuide(n.lineNumber,s.lineNumber,r.lineNumber),l=this.convertModelPositionToViewPosition(a.startLineNumber,1),c=this.convertModelPositionToViewPosition(a.endLineNumber,this.model.getLineMaxColumn(a.endLineNumber));return{startLineNumber:l.lineNumber,endLineNumber:c.lineNumber,indent:a.indent}}getViewLineInfo(e){e=this._toValidViewLineNumber(e);const t=this.projectedModelLineLineCounts.getIndexOf(e-1),i=t.index,n=t.remainder;return new $H(i+1,n)}getMinColumnOfViewLine(e){return this.modelLineProjections[e.modelLineNumber-1].getViewLineMinColumn(this.model,e.modelLineNumber,e.modelLineWrappedLineIdx)}getMaxColumnOfViewLine(e){return this.modelLineProjections[e.modelLineNumber-1].getViewLineMaxColumn(this.model,e.modelLineNumber,e.modelLineWrappedLineIdx)}getModelStartPositionOfViewLine(e){const t=this.modelLineProjections[e.modelLineNumber-1],i=t.getViewLineMinColumn(this.model,e.modelLineNumber,e.modelLineWrappedLineIdx),n=t.getModelColumnOfViewPosition(e.modelLineWrappedLineIdx,i);return new A(e.modelLineNumber,n)}getModelEndPositionOfViewLine(e){const t=this.modelLineProjections[e.modelLineNumber-1],i=t.getViewLineMaxColumn(this.model,e.modelLineNumber,e.modelLineWrappedLineIdx),n=t.getModelColumnOfViewPosition(e.modelLineWrappedLineIdx,i);return new A(e.modelLineNumber,n)}getViewLineInfosGroupedByModelRanges(e,t){const i=this.getViewLineInfo(e),n=this.getViewLineInfo(t),s=new Array;let r=this.getModelStartPositionOfViewLine(i),a=new Array;for(let l=i.modelLineNumber;l<=n.modelLineNumber;l++){const c=this.modelLineProjections[l-1];if(c.isVisible()){const d=l===i.modelLineNumber?i.modelLineWrappedLineIdx:0,h=l===n.modelLineNumber?n.modelLineWrappedLineIdx+1:c.getViewLineCount();for(let u=d;u<h;u++)a.push(new $H(l,u))}if(!c.isVisible()&&r){const d=new A(l-1,this.model.getLineMaxColumn(l-1)+1),h=x.fromPositions(r,d);s.push(new jH(h,a)),a=[],r=null}else c.isVisible()&&!r&&(r=new A(l,1))}if(r){const l=x.fromPositions(r,this.getModelEndPositionOfViewLine(n));s.push(new jH(l,a))}return s}getViewLinesBracketGuides(e,t,i,n){const s=i?this.convertViewPositionToModelPosition(i.lineNumber,i.column):null,r=[];for(const a of this.getViewLineInfosGroupedByModelRanges(e,t)){const l=a.modelRange.startLineNumber,c=this.model.guides.getLinesBracketGuides(l,a.modelRange.endLineNumber,s,n);for(const d of a.viewLines){const u=c[d.modelLineNumber-l].map(g=>{if(g.forWrappedLinesAfterColumn!==-1&&this.modelLineProjections[d.modelLineNumber-1].getViewPositionOfModelPosition(0,g.forWrappedLinesAfterColumn).lineNumber>=d.modelLineWrappedLineIdx||g.forWrappedLinesBeforeOrAtColumn!==-1&&this.modelLineProjections[d.modelLineNumber-1].getViewPositionOfModelPosition(0,g.forWrappedLinesBeforeOrAtColumn).lineNumber<d.modelLineWrappedLineIdx)return;if(!g.horizontalLine)return g;let f=-1;if(g.column!==-1){const b=this.modelLineProjections[d.modelLineNumber-1].getViewPositionOfModelPosition(0,g.column);if(b.lineNumber===d.modelLineWrappedLineIdx)f=b.column;else if(b.lineNumber<d.modelLineWrappedLineIdx)f=this.getMinColumnOfViewLine(d);else if(b.lineNumber>d.modelLineWrappedLineIdx)return}const p=this.convertModelPositionToViewPosition(d.modelLineNumber,g.horizontalLine.endColumn),_=this.modelLineProjections[d.modelLineNumber-1].getViewPositionOfModelPosition(0,g.horizontalLine.endColumn);return _.lineNumber===d.modelLineWrappedLineIdx?new cg(g.visibleColumn,f,g.className,new ev(g.horizontalLine.top,p.column),-1,-1):_.lineNumber<d.modelLineWrappedLineIdx||g.visibleColumn!==-1?void 0:new cg(g.visibleColumn,f,g.className,new ev(g.horizontalLine.top,this.getMaxColumnOfViewLine(d)),-1,-1)});r.push(u.filter(g=>!!g))}}return r}getViewLinesIndentGuides(e,t){e=this._toValidViewLineNumber(e),t=this._toValidViewLineNumber(t);const i=this.convertViewPositionToModelPosition(e,this.getViewLineMinColumn(e)),n=this.convertViewPositionToModelPosition(t,this.getViewLineMaxColumn(t));let s=[];const r=[],a=[],l=i.lineNumber-1,c=n.lineNumber-1;let d=null;for(let f=l;f<=c;f++){const p=this.modelLineProjections[f];if(p.isVisible()){const _=p.getViewLineNumberOfModelPosition(0,f===l?i.column:1),b=p.getViewLineNumberOfModelPosition(0,this.model.getLineMaxColumn(f+1)),C=b-_+1;let v=0;C>1&&p.getViewLineMinColumn(this.model,f+1,b)===1&&(v=_===0?1:2),r.push(C),a.push(v),d===null&&(d=new A(f+1,0))}else d!==null&&(s=s.concat(this.model.guides.getLinesIndentGuides(d.lineNumber,f)),d=null)}d!==null&&(s=s.concat(this.model.guides.getLinesIndentGuides(d.lineNumber,n.lineNumber)),d=null);const h=t-e+1,u=new Array(h);let g=0;for(let f=0,p=s.length;f<p;f++){let _=s[f];const b=Math.min(h-g,r[f]),C=a[f];let v;C===2?v=0:C===1?v=1:v=b;for(let w=0;w<b;w++)w===v&&(_=0),u[g++]=_}return u}getViewLineContent(e){const t=this.getViewLineInfo(e);return this.modelLineProjections[t.modelLineNumber-1].getViewLineContent(this.model,t.modelLineNumber,t.modelLineWrappedLineIdx)}getViewLineLength(e){const t=this.getViewLineInfo(e);return this.modelLineProjections[t.modelLineNumber-1].getViewLineLength(this.model,t.modelLineNumber,t.modelLineWrappedLineIdx)}getViewLineMinColumn(e){const t=this.getViewLineInfo(e);return this.modelLineProjections[t.modelLineNumber-1].getViewLineMinColumn(this.model,t.modelLineNumber,t.modelLineWrappedLineIdx)}getViewLineMaxColumn(e){const t=this.getViewLineInfo(e);return this.modelLineProjections[t.modelLineNumber-1].getViewLineMaxColumn(this.model,t.modelLineNumber,t.modelLineWrappedLineIdx)}getViewLineData(e){const t=this.getViewLineInfo(e);return this.modelLineProjections[t.modelLineNumber-1].getViewLineData(this.model,t.modelLineNumber,t.modelLineWrappedLineIdx)}getViewLinesData(e,t,i){e=this._toValidViewLineNumber(e),t=this._toValidViewLineNumber(t);const n=this.projectedModelLineLineCounts.getIndexOf(e-1);let s=e;const r=n.index,a=n.remainder,l=[];for(let c=r,d=this.model.getLineCount();c<d;c++){const h=this.modelLineProjections[c];if(!h.isVisible())continue;const u=c===r?a:0;let g=h.getViewLineCount()-u,f=!1;if(s+g>t&&(f=!0,g=t-s+1),h.getViewLinesData(this.model,c+1,u,g,s-e,i,l),s+=g,f)break}return l}validateViewPosition(e,t,i){e=this._toValidViewLineNumber(e);const n=this.projectedModelLineLineCounts.getIndexOf(e-1),s=n.index,r=n.remainder,a=this.modelLineProjections[s],l=a.getViewLineMinColumn(this.model,s+1,r),c=a.getViewLineMaxColumn(this.model,s+1,r);t<l&&(t=l),t>c&&(t=c);const d=a.getModelColumnOfViewPosition(r,t);return this.model.validatePosition(new A(s+1,d)).equals(i)?new A(e,t):this.convertModelPositionToViewPosition(i.lineNumber,i.column)}validateViewRange(e,t){const i=this.validateViewPosition(e.startLineNumber,e.startColumn,t.getStartPosition()),n=this.validateViewPosition(e.endLineNumber,e.endColumn,t.getEndPosition());return new x(i.lineNumber,i.column,n.lineNumber,n.column)}convertViewPositionToModelPosition(e,t){const i=this.getViewLineInfo(e),n=this.modelLineProjections[i.modelLineNumber-1].getModelColumnOfViewPosition(i.modelLineWrappedLineIdx,t);return this.model.validatePosition(new A(i.modelLineNumber,n))}convertViewRangeToModelRange(e){const t=this.convertViewPositionToModelPosition(e.startLineNumber,e.startColumn),i=this.convertViewPositionToModelPosition(e.endLineNumber,e.endColumn);return new x(t.lineNumber,t.column,i.lineNumber,i.column)}convertModelPositionToViewPosition(e,t,i=2,n=!1,s=!1){const r=this.model.validatePosition(new A(e,t)),a=r.lineNumber,l=r.column;let c=a-1,d=!1;if(s)for(;c<this.modelLineProjections.length&&!this.modelLineProjections[c].isVisible();)c++,d=!0;else for(;c>0&&!this.modelLineProjections[c].isVisible();)c--,d=!0;if(c===0&&!this.modelLineProjections[c].isVisible())return new A(n?0:1,1);const h=1+this.projectedModelLineLineCounts.getPrefixSum(c);let u;return d?s?u=this.modelLineProjections[c].getViewPositionOfModelPosition(h,1,i):u=this.modelLineProjections[c].getViewPositionOfModelPosition(h,this.model.getLineMaxColumn(c+1),i):u=this.modelLineProjections[a-1].getViewPositionOfModelPosition(h,l,i),u}convertModelRangeToViewRange(e,t=0){if(e.isEmpty()){const i=this.convertModelPositionToViewPosition(e.startLineNumber,e.startColumn,t);return x.fromPositions(i)}else{const i=this.convertModelPositionToViewPosition(e.startLineNumber,e.startColumn,1),n=this.convertModelPositionToViewPosition(e.endLineNumber,e.endColumn,0);return new x(i.lineNumber,i.column,n.lineNumber,n.column)}}getViewLineNumberOfModelPosition(e,t){let i=e-1;if(this.modelLineProjections[i].isVisible()){const s=1+this.projectedModelLineLineCounts.getPrefixSum(i);return this.modelLineProjections[i].getViewLineNumberOfModelPosition(s,t)}for(;i>0&&!this.modelLineProjections[i].isVisible();)i--;if(i===0&&!this.modelLineProjections[i].isVisible())return 1;const n=1+this.projectedModelLineLineCounts.getPrefixSum(i);return this.modelLineProjections[i].getViewLineNumberOfModelPosition(n,this.model.getLineMaxColumn(i+1))}getDecorationsInRange(e,t,i,n,s){const r=this.convertViewPositionToModelPosition(e.startLineNumber,e.startColumn),a=this.convertViewPositionToModelPosition(e.endLineNumber,e.endColumn);if(a.lineNumber-r.lineNumber<=e.endLineNumber-e.startLineNumber)return this.model.getDecorationsInRange(new x(r.lineNumber,1,a.lineNumber,a.column),t,i,n,s);let l=[];const c=r.lineNumber-1,d=a.lineNumber-1;let h=null;for(let p=c;p<=d;p++)if(this.modelLineProjections[p].isVisible())h===null&&(h=new A(p+1,p===c?r.column:1));else if(h!==null){const b=this.model.getLineMaxColumn(p);l=l.concat(this.model.getDecorationsInRange(new x(h.lineNumber,h.column,p,b),t,i,n)),h=null}h!==null&&(l=l.concat(this.model.getDecorationsInRange(new x(h.lineNumber,h.column,a.lineNumber,a.column),t,i,n)),h=null),l.sort((p,_)=>{const b=x.compareRangesUsingStarts(p.range,_.range);return b===0?p.id<_.id?-1:p.id>_.id?1:0:b});const u=[];let g=0,f=null;for(const p of l){const _=p.id;f!==_&&(f=_,u[g++]=p)}return u}getInjectedTextAt(e){const t=this.getViewLineInfo(e.lineNumber);return this.modelLineProjections[t.modelLineNumber-1].getInjectedTextAt(t.modelLineWrappedLineIdx,e.column)}normalizePosition(e,t){const i=this.getViewLineInfo(e.lineNumber);return this.modelLineProjections[i.modelLineNumber-1].normalizePosition(i.modelLineWrappedLineIdx,e,t)}getLineIndentColumn(e){const t=this.getViewLineInfo(e);return t.modelLineWrappedLineIdx===0?this.model.getLineIndentColumn(t.modelLineNumber):0}}function kCe(o){if(o.length===0)return[];const e=o.slice();e.sort(x.compareRangesUsingStarts);const t=[];let i=e[0].startLineNumber,n=e[0].endLineNumber;for(let s=1,r=e.length;s<r;s++){const a=e[s];a.startLineNumber>n+1?(t.push(new x(i,1,n,1)),i=a.startLineNumber,n=a.endLineNumber):a.endLineNumber>n&&(n=a.endLineNumber)}return t.push(new x(i,1,n,1)),t}class $H{constructor(e,t){this.modelLineNumber=e,this.modelLineWrappedLineIdx=t}}class jH{constructor(e,t){this.modelRange=e,this.viewLines=t}}class DCe{constructor(e){this._lines=e}convertViewPositionToModelPosition(e){return this._lines.convertViewPositionToModelPosition(e.lineNumber,e.column)}convertViewRangeToModelRange(e){return this._lines.convertViewRangeToModelRange(e)}validateViewPosition(e,t){return this._lines.validateViewPosition(e.lineNumber,e.column,t)}validateViewRange(e,t){return this._lines.validateViewRange(e,t)}convertModelPositionToViewPosition(e,t,i,n){return this._lines.convertModelPositionToViewPosition(e.lineNumber,e.column,t,i,n)}convertModelRangeToViewRange(e,t){return this._lines.convertModelRangeToViewRange(e,t)}modelPositionIsVisible(e){return this._lines.modelPositionIsVisible(e.lineNumber,e.column)}getModelLineViewLineCount(e){return this._lines.getModelLineViewLineCount(e)}getViewLineNumberOfModelPosition(e,t){return this._lines.getViewLineNumberOfModelPosition(e,t)}}class ICe{constructor(e){this.model=e}dispose(){}createCoordinatesConverter(){return new ECe(this)}getHiddenAreas(){return[]}setHiddenAreas(e){return!1}setTabSize(e){return!1}setWrappingSettings(e,t,i,n){return!1}createLineBreaksComputer(){const e=[];return{addRequest:(t,i,n)=>{e.push(null)},finalize:()=>e}}onModelFlushed(){}onModelLinesDeleted(e,t,i){return new xP(t,i)}onModelLinesInserted(e,t,i,n){return new kP(t,i)}onModelLineChanged(e,t,i){return[!1,new Mq(t,1),null,null]}acceptVersionId(e){}getViewLineCount(){return this.model.getLineCount()}getActiveIndentGuide(e,t,i){return{startLineNumber:e,endLineNumber:e,indent:0}}getViewLinesBracketGuides(e,t,i){return new Array(t-e+1).fill([])}getViewLinesIndentGuides(e,t){const i=t-e+1,n=new Array(i);for(let s=0;s<i;s++)n[s]=0;return n}getViewLineContent(e){return this.model.getLineContent(e)}getViewLineLength(e){return this.model.getLineLength(e)}getViewLineMinColumn(e){return this.model.getLineMinColumn(e)}getViewLineMaxColumn(e){return this.model.getLineMaxColumn(e)}getViewLineData(e){const t=this.model.tokenization.getLineTokens(e),i=t.getLineContent();return new B3(i,!1,1,i.length+1,0,t.inflate(),null)}getViewLinesData(e,t,i){const n=this.model.getLineCount();e=Math.min(Math.max(1,e),n),t=Math.min(Math.max(1,t),n);const s=[];for(let r=e;r<=t;r++){const a=r-e;s[a]=i[a]?this.getViewLineData(r):null}return s}getDecorationsInRange(e,t,i,n,s){return this.model.getDecorationsInRange(e,t,i,n,s)}normalizePosition(e,t){return this.model.normalizePosition(e,t)}getLineIndentColumn(e){return this.model.getLineIndentColumn(e)}getInjectedTextAt(e){return null}}class ECe{constructor(e){this._lines=e}_validPosition(e){return this._lines.model.validatePosition(e)}_validRange(e){return this._lines.model.validateRange(e)}convertViewPositionToModelPosition(e){return this._validPosition(e)}convertViewRangeToModelRange(e){return this._validRange(e)}validateViewPosition(e,t){return this._validPosition(t)}validateViewRange(e,t){return this._validRange(t)}convertModelPositionToViewPosition(e){return this._validPosition(e)}convertModelRangeToViewRange(e){return this._validRange(e)}modelPositionIsVisible(e){const t=this._lines.model.getLineCount();return!(e.lineNumber<1||e.lineNumber>t)}getModelLineViewLineCount(e){return 1}getViewLineNumberOfModelPosition(e,t){return e}}const yu=Zr.Right;class NCe{constructor(e){this.persist=0,this._requiredLanes=1,this.lanes=new Uint8Array(Math.ceil((e+1)*yu/8))}reset(e){const t=Math.ceil((e+1)*yu/8);this.lanes.length<t?this.lanes=new Uint8Array(t):this.lanes.fill(0),this._requiredLanes=1}get requiredLanes(){return this._requiredLanes}push(e,t,i){i&&(this.persist|=1<<e-1);for(let n=t.startLineNumber;n<=t.endLineNumber;n++){const s=yu*n+(e-1);this.lanes[s>>>3]|=1<<s%8,this._requiredLanes=Math.max(this._requiredLanes,this.countAtLine(n))}}getLanesAtLine(e){const t=[];let i=yu*e;for(let n=0;n<yu;n++)(this.persist&1<<n||this.lanes[i>>>3]&1<<i%8)&&t.push(n+1),i++;return t.length?t:[Zr.Center]}countAtLine(e){let t=yu*e,i=0;for(let n=0;n<yu;n++)(this.persist&1<<n||this.lanes[t>>>3]&1<<t%8)&&i++,t++;return i}}let TCe=class extends F{constructor(e,t,i,n,s,r,a,l,c,d){if(super(),this.languageConfigurationService=a,this._themeService=l,this._attachedView=c,this._transactionalTarget=d,this.hiddenAreasModel=new MCe,this.previousHiddenAreas=[],this._editorId=e,this._configuration=t,this.model=i,this._eventDispatcher=new iCe,this.onEvent=this._eventDispatcher.onEvent,this.cursorConfig=new Nm(this.model.getLanguageId(),this.model.getOptions(),this._configuration,this.languageConfigurationService),this._updateConfigurationViewLineCount=this._register(new It(()=>this._updateConfigurationViewLineCountNow(),0)),this._hasFocus=!1,this._viewportStart=aW.create(this.model),this.glyphLanes=new NCe(0),this.model.isTooLargeForTokenization())this._lines=new ICe(this.model);else{const h=this._configuration.options,u=h.get(50),g=h.get(140),f=h.get(147),p=h.get(139),_=h.get(130);this._lines=new xCe(this._editorId,this.model,n,s,u,this.model.getOptions().tabSize,g,f.wrappingColumn,p,_)}this.coordinatesConverter=this._lines.createCoordinatesConverter(),this._cursor=this._register(new gCe(i,this,this.coordinatesConverter,this.cursorConfig)),this.viewLayout=this._register(new wCe(this._configuration,this.getLineCount(),r)),this._register(this.viewLayout.onDidScroll(h=>{h.scrollTopChanged&&this._handleVisibleLinesChanged(),h.scrollTopChanged&&this._viewportStart.invalidate(),this._eventDispatcher.emitSingleViewEvent(new Xbe(h)),this._eventDispatcher.emitOutgoingEvent(new nW(h.oldScrollWidth,h.oldScrollLeft,h.oldScrollHeight,h.oldScrollTop,h.scrollWidth,h.scrollLeft,h.scrollHeight,h.scrollTop))})),this._register(this.viewLayout.onDidContentSizeChange(h=>{this._eventDispatcher.emitOutgoingEvent(h)})),this._decorations=new SCe(this._editorId,this.model,this._configuration,this._lines,this.coordinatesConverter),this._registerModelEvents(),this._register(this._configuration.onDidChangeFast(h=>{try{const u=this._eventDispatcher.beginEmitViewEvents();this._onConfigurationChanged(u,h)}finally{this._eventDispatcher.endEmitViewEvents()}})),this._register(hx.getInstance().onDidChange(()=>{this._eventDispatcher.emitSingleViewEvent(new eCe)})),this._register(this._themeService.onDidColorThemeChange(h=>{this._invalidateDecorationsColorCache(),this._eventDispatcher.emitSingleViewEvent(new Qbe(h))})),this._updateConfigurationViewLineCountNow()}dispose(){super.dispose(),this._decorations.dispose(),this._lines.dispose(),this._viewportStart.dispose(),this._eventDispatcher.dispose()}createLineBreaksComputer(){return this._lines.createLineBreaksComputer()}addViewEventHandler(e){this._eventDispatcher.addViewEventHandler(e)}removeViewEventHandler(e){this._eventDispatcher.removeViewEventHandler(e)}_updateConfigurationViewLineCountNow(){this._configuration.setViewLineCount(this._lines.getViewLineCount())}getModelVisibleRanges(){const e=this.viewLayout.getLinesViewportData(),t=new x(e.startLineNumber,this.getLineMinColumn(e.startLineNumber),e.endLineNumber,this.getLineMaxColumn(e.endLineNumber));return this._toModelVisibleRanges(t)}visibleLinesStabilized(){const e=this.getModelVisibleRanges();this._attachedView.setVisibleLines(e,!0)}_handleVisibleLinesChanged(){const e=this.getModelVisibleRanges();this._attachedView.setVisibleLines(e,!1)}setHasFocus(e){this._hasFocus=e,this._cursor.setHasFocus(e),this._eventDispatcher.emitSingleViewEvent(new Zbe(e)),this._eventDispatcher.emitOutgoingEvent(new iW(!e,e))}onCompositionStart(){this._eventDispatcher.emitSingleViewEvent(new jbe)}onCompositionEnd(){this._eventDispatcher.emitSingleViewEvent(new Kbe)}_captureStableViewport(){if(this._viewportStart.isValid&&this.viewLayout.getCurrentScrollTop()>0){const e=new A(this._viewportStart.viewLineNumber,this.getLineMinColumn(this._viewportStart.viewLineNumber)),t=this.coordinatesConverter.convertViewPositionToModelPosition(e);return new qH(t,this._viewportStart.startLineDelta)}return new qH(null,0)}_onConfigurationChanged(e,t){const i=this._captureStableViewport(),n=this._configuration.options,s=n.get(50),r=n.get(140),a=n.get(147),l=n.get(139),c=n.get(130);this._lines.setWrappingSettings(s,r,a.wrappingColumn,l,c)&&(e.emitViewEvent(new d0),e.emitViewEvent(new h0),e.emitViewEvent(new Su(null)),this._cursor.onLineMappingChanged(e),this._decorations.onLineMappingChanged(),this.viewLayout.onFlushed(this.getLineCount()),this._updateConfigurationViewLineCount.schedule()),t.hasChanged(92)&&(this._decorations.reset(),e.emitViewEvent(new Su(null))),t.hasChanged(99)&&(this._decorations.reset(),e.emitViewEvent(new Su(null))),e.emitViewEvent(new qbe(t)),this.viewLayout.onConfigurationChanged(t),i.recoverViewportStart(this.coordinatesConverter,this.viewLayout),Nm.shouldRecreate(t)&&(this.cursorConfig=new Nm(this.model.getLanguageId(),this.model.getOptions(),this._configuration,this.languageConfigurationService),this._cursor.updateConfiguration(this.cursorConfig))}_registerModelEvents(){this._register(this.model.onDidChangeContentOrInjectedText(e=>{try{const i=this._eventDispatcher.beginEmitViewEvents();let n=!1,s=!1;const r=e instanceof vg?e.rawContentChangedEvent.changes:e.changes,a=e instanceof vg?e.rawContentChangedEvent.versionId:null,l=this._lines.createLineBreaksComputer();for(const h of r)switch(h.changeType){case 4:{for(let u=0;u<h.detail.length;u++){const g=h.detail[u];let f=h.injectedTexts[u];f&&(f=f.filter(p=>!p.ownerId||p.ownerId===this._editorId)),l.addRequest(g,f,null)}break}case 2:{let u=null;h.injectedText&&(u=h.injectedText.filter(g=>!g.ownerId||g.ownerId===this._editorId)),l.addRequest(h.detail,u,null);break}}const c=l.finalize(),d=new Qc(c);for(const h of r)switch(h.changeType){case 1:{this._lines.onModelFlushed(),i.emitViewEvent(new d0),this._decorations.reset(),this.viewLayout.onFlushed(this.getLineCount()),n=!0;break}case 3:{const u=this._lines.onModelLinesDeleted(a,h.fromLineNumber,h.toLineNumber);u!==null&&(i.emitViewEvent(u),this.viewLayout.onLinesDeleted(u.fromLineNumber,u.toLineNumber)),n=!0;break}case 4:{const u=d.takeCount(h.detail.length),g=this._lines.onModelLinesInserted(a,h.fromLineNumber,h.toLineNumber,u);g!==null&&(i.emitViewEvent(g),this.viewLayout.onLinesInserted(g.fromLineNumber,g.toLineNumber)),n=!0;break}case 2:{const u=d.dequeue(),[g,f,p,_]=this._lines.onModelLineChanged(a,h.lineNumber,u);s=g,f&&i.emitViewEvent(f),p&&(i.emitViewEvent(p),this.viewLayout.onLinesInserted(p.fromLineNumber,p.toLineNumber)),_&&(i.emitViewEvent(_),this.viewLayout.onLinesDeleted(_.fromLineNumber,_.toLineNumber));break}case 5:break}a!==null&&this._lines.acceptVersionId(a),this.viewLayout.onHeightMaybeChanged(),!n&&s&&(i.emitViewEvent(new h0),i.emitViewEvent(new Su(null)),this._cursor.onLineMappingChanged(i),this._decorations.onLineMappingChanged())}finally{this._eventDispatcher.endEmitViewEvents()}const t=this._viewportStart.isValid;if(this._viewportStart.invalidate(),this._configuration.setModelLineCount(this.model.getLineCount()),this._updateConfigurationViewLineCountNow(),!this._hasFocus&&this.model.getAttachedEditorCount()>=2&&t){const i=this.model._getTrackedRange(this._viewportStart.modelTrackedRange);if(i){const n=this.coordinatesConverter.convertModelPositionToViewPosition(i.getStartPosition()),s=this.viewLayout.getVerticalOffsetForLineNumber(n.lineNumber);this.viewLayout.setScrollPosition({scrollTop:s+this._viewportStart.startLineDelta},1)}}try{const i=this._eventDispatcher.beginEmitViewEvents();e instanceof vg&&i.emitOutgoingEvent(new dCe(e.contentChangedEvent)),this._cursor.onModelContentChanged(i,e)}finally{this._eventDispatcher.endEmitViewEvents()}this._handleVisibleLinesChanged()})),this._register(this.model.onDidChangeTokens(e=>{const t=[];for(let i=0,n=e.ranges.length;i<n;i++){const s=e.ranges[i],r=this.coordinatesConverter.convertModelPositionToViewPosition(new A(s.fromLineNumber,1)).lineNumber,a=this.coordinatesConverter.convertModelPositionToViewPosition(new A(s.toLineNumber,this.model.getLineMaxColumn(s.toLineNumber))).lineNumber;t[i]={fromLineNumber:r,toLineNumber:a}}this._eventDispatcher.emitSingleViewEvent(new Jbe(t)),this._eventDispatcher.emitOutgoingEvent(new uCe(e))})),this._register(this.model.onDidChangeLanguageConfiguration(e=>{this._eventDispatcher.emitSingleViewEvent(new Ybe),this.cursorConfig=new Nm(this.model.getLanguageId(),this.model.getOptions(),this._configuration,this.languageConfigurationService),this._cursor.updateConfiguration(this.cursorConfig),this._eventDispatcher.emitOutgoingEvent(new cCe(e))})),this._register(this.model.onDidChangeLanguage(e=>{this.cursorConfig=new Nm(this.model.getLanguageId(),this.model.getOptions(),this._configuration,this.languageConfigurationService),this._cursor.updateConfiguration(this.cursorConfig),this._eventDispatcher.emitOutgoingEvent(new lCe(e))})),this._register(this.model.onDidChangeOptions(e=>{if(this._lines.setTabSize(this.model.getOptions().tabSize)){try{const t=this._eventDispatcher.beginEmitViewEvents();t.emitViewEvent(new d0),t.emitViewEvent(new h0),t.emitViewEvent(new Su(null)),this._cursor.onLineMappingChanged(t),this._decorations.onLineMappingChanged(),this.viewLayout.onFlushed(this.getLineCount())}finally{this._eventDispatcher.endEmitViewEvents()}this._updateConfigurationViewLineCount.schedule()}this.cursorConfig=new Nm(this.model.getLanguageId(),this.model.getOptions(),this._configuration,this.languageConfigurationService),this._cursor.updateConfiguration(this.cursorConfig),this._eventDispatcher.emitOutgoingEvent(new hCe(e))})),this._register(this.model.onDidChangeDecorations(e=>{this._decorations.onModelDecorationsChanged(),this._eventDispatcher.emitSingleViewEvent(new Su(e)),this._eventDispatcher.emitOutgoingEvent(new aCe(e))}))}setHiddenAreas(e,t){var r;this.hiddenAreasModel.setHiddenAreas(t,e);const i=this.hiddenAreasModel.getMergedRanges();if(i===this.previousHiddenAreas)return;this.previousHiddenAreas=i;const n=this._captureStableViewport();let s=!1;try{const a=this._eventDispatcher.beginEmitViewEvents();s=this._lines.setHiddenAreas(i),s&&(a.emitViewEvent(new d0),a.emitViewEvent(new h0),a.emitViewEvent(new Su(null)),this._cursor.onLineMappingChanged(a),this._decorations.onLineMappingChanged(),this.viewLayout.onFlushed(this.getLineCount()),this.viewLayout.onHeightMaybeChanged());const l=(r=n.viewportStartModelPosition)==null?void 0:r.lineNumber;l&&i.some(d=>d.startLineNumber<=l&&l<=d.endLineNumber)||n.recoverViewportStart(this.coordinatesConverter,this.viewLayout)}finally{this._eventDispatcher.endEmitViewEvents()}this._updateConfigurationViewLineCount.schedule(),s&&this._eventDispatcher.emitOutgoingEvent(new oCe)}getVisibleRangesPlusViewportAboveBelow(){const e=this._configuration.options.get(146),t=this._configuration.options.get(67),i=Math.max(20,Math.round(e.height/t)),n=this.viewLayout.getLinesViewportData(),s=Math.max(1,n.completelyVisibleStartLineNumber-i),r=Math.min(this.getLineCount(),n.completelyVisibleEndLineNumber+i);return this._toModelVisibleRanges(new x(s,this.getLineMinColumn(s),r,this.getLineMaxColumn(r)))}getVisibleRanges(){const e=this.getCompletelyVisibleViewRange();return this._toModelVisibleRanges(e)}getHiddenAreas(){return this._lines.getHiddenAreas()}_toModelVisibleRanges(e){const t=this.coordinatesConverter.convertViewRangeToModelRange(e),i=this._lines.getHiddenAreas();if(i.length===0)return[t];const n=[];let s=0,r=t.startLineNumber,a=t.startColumn;const l=t.endLineNumber,c=t.endColumn;for(let d=0,h=i.length;d<h;d++){const u=i[d].startLineNumber,g=i[d].endLineNumber;g<r||u>l||(r<u&&(n[s++]=new x(r,a,u-1,this.model.getLineMaxColumn(u-1))),r=g+1,a=1)}return(r<l||r===l&&a<c)&&(n[s++]=new x(r,a,l,c)),n}getCompletelyVisibleViewRange(){const e=this.viewLayout.getLinesViewportData(),t=e.completelyVisibleStartLineNumber,i=e.completelyVisibleEndLineNumber;return new x(t,this.getLineMinColumn(t),i,this.getLineMaxColumn(i))}getCompletelyVisibleViewRangeAtScrollTop(e){const t=this.viewLayout.getLinesViewportDataAtScrollTop(e),i=t.completelyVisibleStartLineNumber,n=t.completelyVisibleEndLineNumber;return new x(i,this.getLineMinColumn(i),n,this.getLineMaxColumn(n))}saveState(){const e=this.viewLayout.saveState(),t=e.scrollTop,i=this.viewLayout.getLineNumberAtVerticalOffset(t),n=this.coordinatesConverter.convertViewPositionToModelPosition(new A(i,this.getLineMinColumn(i))),s=this.viewLayout.getVerticalOffsetForLineNumber(i)-t;return{scrollLeft:e.scrollLeft,firstPosition:n,firstPositionDeltaTop:s}}reduceRestoreState(e){if(typeof e.firstPosition>"u")return this._reduceRestoreStateCompatibility(e);const t=this.model.validatePosition(e.firstPosition),i=this.coordinatesConverter.convertModelPositionToViewPosition(t),n=this.viewLayout.getVerticalOffsetForLineNumber(i.lineNumber)-e.firstPositionDeltaTop;return{scrollLeft:e.scrollLeft,scrollTop:n}}_reduceRestoreStateCompatibility(e){return{scrollLeft:e.scrollLeft,scrollTop:e.scrollTopWithoutViewZones}}getTabSize(){return this.model.getOptions().tabSize}getLineCount(){return this._lines.getViewLineCount()}setViewport(e,t,i){this._viewportStart.update(this,e)}getActiveIndentGuide(e,t,i){return this._lines.getActiveIndentGuide(e,t,i)}getLinesIndentGuides(e,t){return this._lines.getViewLinesIndentGuides(e,t)}getBracketGuidesInRangeByLine(e,t,i,n){return this._lines.getViewLinesBracketGuides(e,t,i,n)}getLineContent(e){return this._lines.getViewLineContent(e)}getLineLength(e){return this._lines.getViewLineLength(e)}getLineMinColumn(e){return this._lines.getViewLineMinColumn(e)}getLineMaxColumn(e){return this._lines.getViewLineMaxColumn(e)}getLineFirstNonWhitespaceColumn(e){const t=Jn(this.getLineContent(e));return t===-1?0:t+1}getLineLastNonWhitespaceColumn(e){const t=Na(this.getLineContent(e));return t===-1?0:t+2}getMinimapDecorationsInRange(e){return this._decorations.getMinimapDecorationsInRange(e)}getDecorationsInViewport(e){return this._decorations.getDecorationsViewportData(e).decorations}getInjectedTextAt(e){return this._lines.getInjectedTextAt(e)}getViewportViewLineRenderingData(e,t){const n=this._decorations.getDecorationsViewportData(e).inlineDecorations[t-e.startLineNumber];return this._getViewLineRenderingData(t,n)}getViewLineRenderingData(e){const t=this._decorations.getInlineDecorationsOnLine(e);return this._getViewLineRenderingData(e,t)}_getViewLineRenderingData(e,t){const i=this.model.mightContainRTL(),n=this.model.mightContainNonBasicASCII(),s=this.getTabSize(),r=this._lines.getViewLineData(e);return r.inlineDecorations&&(t=[...t,...r.inlineDecorations.map(a=>a.toInlineDecoration(e))]),new Ko(r.minColumn,r.maxColumn,r.content,r.continuesWithWrappedLine,i,n,r.tokens,t,s,r.startVisibleColumn)}getViewLineData(e){return this._lines.getViewLineData(e)}getMinimapLinesRenderingData(e,t,i){const n=this._lines.getViewLinesData(e,t,i);return new ppe(this.getTabSize(),n)}getAllOverviewRulerDecorations(e){const t=this.model.getOverviewRulerDecorations(this._editorId,Ty(this._configuration.options)),i=new RCe;for(const n of t){const s=n.options,r=s.overviewRuler;if(!r)continue;const a=r.position;if(a===0)continue;const l=r.getColor(e.value),c=this.coordinatesConverter.getViewLineNumberOfModelPosition(n.range.startLineNumber,n.range.startColumn),d=this.coordinatesConverter.getViewLineNumberOfModelPosition(n.range.endLineNumber,n.range.endColumn);i.accept(l,s.zIndex,c,d,a)}return i.asArray}_invalidateDecorationsColorCache(){const e=this.model.getOverviewRulerDecorations();for(const t of e){const i=t.options.overviewRuler;i==null||i.invalidateCachedColor();const n=t.options.minimap;n==null||n.invalidateCachedColor()}}getValueInRange(e,t){const i=this.coordinatesConverter.convertViewRangeToModelRange(e);return this.model.getValueInRange(i,t)}getValueLengthInRange(e,t){const i=this.coordinatesConverter.convertViewRangeToModelRange(e);return this.model.getValueLengthInRange(i,t)}modifyPosition(e,t){const i=this.coordinatesConverter.convertViewPositionToModelPosition(e),n=this.model.modifyPosition(i,t);return this.coordinatesConverter.convertModelPositionToViewPosition(n)}deduceModelPositionRelativeToViewPosition(e,t,i){const n=this.coordinatesConverter.convertViewPositionToModelPosition(e);this.model.getEOL().length===2&&(t<0?t-=i:t+=i);const r=this.model.getOffsetAt(n)+t;return this.model.getPositionAt(r)}getPlainTextToCopy(e,t,i){const n=i?`\r
`:this.model.getEOL();e=e.slice(0),e.sort(x.compareRangesUsingStarts);let s=!1,r=!1;for(const l of e)l.isEmpty()?s=!0:r=!0;if(!r){if(!t)return"";const l=e.map(d=>d.startLineNumber);let c="";for(let d=0;d<l.length;d++)d>0&&l[d-1]===l[d]||(c+=this.model.getLineContent(l[d])+n);return c}if(s&&t){const l=[];let c=0;for(const d of e){const h=d.startLineNumber;d.isEmpty()?h!==c&&l.push(this.model.getLineContent(h)):l.push(this.model.getValueInRange(d,i?2:0)),c=h}return l.length===1?l[0]:l}const a=[];for(const l of e)l.isEmpty()||a.push(this.model.getValueInRange(l,i?2:0));return a.length===1?a[0]:a}getRichTextToCopy(e,t){const i=this.model.getLanguageId();if(i===ms||e.length!==1)return null;let n=e[0];if(n.isEmpty()){if(!t)return null;const d=n.startLineNumber;n=new x(d,this.model.getLineMinColumn(d),d,this.model.getLineMaxColumn(d))}const s=this._configuration.options.get(50),r=this._getColorMap(),l=/[:;\\\/<>]/.test(s.fontFamily)||s.fontFamily===fs.fontFamily;let c;return l?c=fs.fontFamily:(c=s.fontFamily,c=c.replace(/"/g,"'"),/[,']/.test(c)||/[+ ]/.test(c)&&(c=`'${c}'`),c=`${c}, ${fs.fontFamily}`),{mode:i,html:`<div style="color: ${r[1]};background-color: ${r[2]};font-family: ${c};font-weight: ${s.fontWeight};font-size: ${s.fontSize}px;line-height: ${s.lineHeight}px;white-space: pre;">`+this._getHTMLToCopy(n,r)+"</div>"}}_getHTMLToCopy(e,t){const i=e.startLineNumber,n=e.startColumn,s=e.endLineNumber,r=e.endColumn,a=this.getTabSize();let l="";for(let c=i;c<=s;c++){const d=this.model.tokenization.getLineTokens(c),h=d.getLineContent(),u=c===i?n-1:0,g=c===s?r-1:h.length;h===""?l+="<br>":l+=$le(h,d.inflate(),t,u,g,a,En)}return l}_getColorMap(){const e=fi.getColorMap(),t=["#000000"];if(e)for(let i=1,n=e.length;i<n;i++)t[i]=Z.Format.CSS.formatHex(e[i]);return t}getPrimaryCursorState(){return this._cursor.getPrimaryCursorState()}getLastAddedCursorIndex(){return this._cursor.getLastAddedCursorIndex()}getCursorStates(){return this._cursor.getCursorStates()}setCursorStates(e,t,i){return this._withViewEventsCollector(n=>this._cursor.setStates(n,e,t,i))}getCursorColumnSelectData(){return this._cursor.getCursorColumnSelectData()}getCursorAutoClosedCharacters(){return this._cursor.getAutoClosedCharacters()}setCursorColumnSelectData(e){this._cursor.setCursorColumnSelectData(e)}getPrevEditOperationType(){return this._cursor.getPrevEditOperationType()}setPrevEditOperationType(e){this._cursor.setPrevEditOperationType(e)}getSelection(){return this._cursor.getSelection()}getSelections(){return this._cursor.getSelections()}getPosition(){return this._cursor.getPrimaryCursorState().modelState.position}setSelections(e,t,i=0){this._withViewEventsCollector(n=>this._cursor.setSelections(n,e,t,i))}saveCursorState(){return this._cursor.saveState()}restoreCursorState(e){this._withViewEventsCollector(t=>this._cursor.restoreState(t,e))}_executeCursorEdit(e){if(this._cursor.context.cursorConfig.readOnly){this._eventDispatcher.emitOutgoingEvent(new rCe);return}this._withViewEventsCollector(e)}executeEdits(e,t,i){this._executeCursorEdit(n=>this._cursor.executeEdits(n,e,t,i))}startComposition(){this._executeCursorEdit(e=>this._cursor.startComposition(e))}endComposition(e){this._executeCursorEdit(t=>this._cursor.endComposition(t,e))}type(e,t){this._executeCursorEdit(i=>this._cursor.type(i,e,t))}compositionType(e,t,i,n,s){this._executeCursorEdit(r=>this._cursor.compositionType(r,e,t,i,n,s))}paste(e,t,i,n){this._executeCursorEdit(s=>this._cursor.paste(s,e,t,i,n))}cut(e){this._executeCursorEdit(t=>this._cursor.cut(t,e))}executeCommand(e,t){this._executeCursorEdit(i=>this._cursor.executeCommand(i,e,t))}executeCommands(e,t){this._executeCursorEdit(i=>this._cursor.executeCommands(i,e,t))}revealAllCursors(e,t,i=!1){this._withViewEventsCollector(n=>this._cursor.revealAll(n,e,i,0,t,0))}revealPrimaryCursor(e,t,i=!1){this._withViewEventsCollector(n=>this._cursor.revealPrimary(n,e,i,0,t,0))}revealTopMostCursor(e){const t=this._cursor.getTopMostViewPosition(),i=new x(t.lineNumber,t.column,t.lineNumber,t.column);this._withViewEventsCollector(n=>n.emitViewEvent(new pv(e,!1,i,null,0,!0,0)))}revealBottomMostCursor(e){const t=this._cursor.getBottomMostViewPosition(),i=new x(t.lineNumber,t.column,t.lineNumber,t.column);this._withViewEventsCollector(n=>n.emitViewEvent(new pv(e,!1,i,null,0,!0,0)))}revealRange(e,t,i,n,s){this._withViewEventsCollector(r=>r.emitViewEvent(new pv(e,!1,i,null,n,t,s)))}changeWhitespace(e){this.viewLayout.changeWhitespace(e)&&(this._eventDispatcher.emitSingleViewEvent(new tCe),this._eventDispatcher.emitOutgoingEvent(new sCe))}_withViewEventsCollector(e){return this._transactionalTarget.batchChanges(()=>{try{const t=this._eventDispatcher.beginEmitViewEvents();return e(t)}finally{this._eventDispatcher.endEmitViewEvents()}})}batchEvents(e){this._withViewEventsCollector(()=>{e()})}normalizePosition(e,t){return this._lines.normalizePosition(e,t)}getLineIndentColumn(e){return this._lines.getLineIndentColumn(e)}};class aW{static create(e){const t=e._setTrackedRange(null,new x(1,1,1,1),1);return new aW(e,1,!1,t,0)}get viewLineNumber(){return this._viewLineNumber}get isValid(){return this._isValid}get modelTrackedRange(){return this._modelTrackedRange}get startLineDelta(){return this._startLineDelta}constructor(e,t,i,n,s){this._model=e,this._viewLineNumber=t,this._isValid=i,this._modelTrackedRange=n,this._startLineDelta=s}dispose(){this._model._setTrackedRange(this._modelTrackedRange,null,1)}update(e,t){const i=e.coordinatesConverter.convertViewPositionToModelPosition(new A(t,e.getLineMinColumn(t))),n=e.model._setTrackedRange(this._modelTrackedRange,new x(i.lineNumber,i.column,i.lineNumber,i.column),1),s=e.viewLayout.getVerticalOffsetForLineNumber(t),r=e.viewLayout.getCurrentScrollTop();this._viewLineNumber=t,this._isValid=!0,this._modelTrackedRange=n,this._startLineDelta=r-s}invalidate(){this._isValid=!1}}class RCe{constructor(){this._asMap=Object.create(null),this.asArray=[]}accept(e,t,i,n,s){const r=this._asMap[e];if(r){const a=r.data,l=a[a.length-3],c=a[a.length-1];if(l===s&&c+1>=i){n>c&&(a[a.length-1]=n);return}a.push(s,i,n)}else{const a=new Hw(e,t,[s,i,n]);this._asMap[e]=a,this.asArray.push(a)}}}class MCe{constructor(){this.hiddenAreas=new Map,this.shouldRecompute=!1,this.ranges=[]}setHiddenAreas(e,t){const i=this.hiddenAreas.get(e);i&&KH(i,t)||(this.hiddenAreas.set(e,t),this.shouldRecompute=!0)}getMergedRanges(){if(!this.shouldRecompute)return this.ranges;this.shouldRecompute=!1;const e=Array.from(this.hiddenAreas.values()).reduce((t,i)=>ACe(t,i),[]);return KH(this.ranges,e)?this.ranges:(this.ranges=e,this.ranges)}}function ACe(o,e){const t=[];let i=0,n=0;for(;i<o.length&&n<e.length;){const s=o[i],r=e[n];if(s.endLineNumber<r.startLineNumber-1)t.push(o[i++]);else if(r.endLineNumber<s.startLineNumber-1)t.push(e[n++]);else{const a=Math.min(s.startLineNumber,r.startLineNumber),l=Math.max(s.endLineNumber,r.endLineNumber);t.push(new x(a,1,l,1)),i++,n++}}for(;i<o.length;)t.push(o[i++]);for(;n<e.length;)t.push(e[n++]);return t}function KH(o,e){if(o.length!==e.length)return!1;for(let t=0;t<o.length;t++)if(!o[t].equalsRange(e[t]))return!1;return!0}class qH{constructor(e,t){this.viewportStartModelPosition=e,this.startLineDelta=t}recoverViewportStart(e,t){if(!this.viewportStartModelPosition)return;const i=e.convertModelPositionToViewPosition(this.viewportStartModelPosition),n=t.getVerticalOffsetForLineNumber(i.lineNumber);t.setScrollPosition({scrollTop:n+this.startLineDelta},1)}}var PCe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},oc=function(o,e){return function(t,i){e(t,i,o)}},Pu,Mg;let Zw=(Mg=class extends F{get isSimpleWidget(){return this._configuration.isSimpleWidget}get contextMenuId(){return this._configuration.contextMenuId}constructor(e,t,i,n,s,r,a,l,c,d,h,u){super(),this.languageConfigurationService=h,this._deliveryQueue=jJ(),this._contributions=this._register(new Vbe),this._onDidDispose=this._register(new P),this.onDidDispose=this._onDidDispose.event,this._onDidChangeModelContent=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeModelContent=this._onDidChangeModelContent.event,this._onDidChangeModelLanguage=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeModelLanguage=this._onDidChangeModelLanguage.event,this._onDidChangeModelLanguageConfiguration=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeModelLanguageConfiguration=this._onDidChangeModelLanguageConfiguration.event,this._onDidChangeModelOptions=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeModelOptions=this._onDidChangeModelOptions.event,this._onDidChangeModelDecorations=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeModelDecorations=this._onDidChangeModelDecorations.event,this._onDidChangeModelTokens=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeModelTokens=this._onDidChangeModelTokens.event,this._onDidChangeConfiguration=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeConfiguration=this._onDidChangeConfiguration.event,this._onWillChangeModel=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onWillChangeModel=this._onWillChangeModel.event,this._onDidChangeModel=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeModel=this._onDidChangeModel.event,this._onDidChangeCursorPosition=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeCursorPosition=this._onDidChangeCursorPosition.event,this._onDidChangeCursorSelection=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeCursorSelection=this._onDidChangeCursorSelection.event,this._onDidAttemptReadOnlyEdit=this._register(new zn(this._contributions,this._deliveryQueue)),this.onDidAttemptReadOnlyEdit=this._onDidAttemptReadOnlyEdit.event,this._onDidLayoutChange=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidLayoutChange=this._onDidLayoutChange.event,this._editorTextFocus=this._register(new GH({deliveryQueue:this._deliveryQueue})),this.onDidFocusEditorText=this._editorTextFocus.onDidChangeToTrue,this.onDidBlurEditorText=this._editorTextFocus.onDidChangeToFalse,this._editorWidgetFocus=this._register(new GH({deliveryQueue:this._deliveryQueue})),this.onDidFocusEditorWidget=this._editorWidgetFocus.onDidChangeToTrue,this.onDidBlurEditorWidget=this._editorWidgetFocus.onDidChangeToFalse,this._onWillType=this._register(new zn(this._contributions,this._deliveryQueue)),this.onWillType=this._onWillType.event,this._onDidType=this._register(new zn(this._contributions,this._deliveryQueue)),this.onDidType=this._onDidType.event,this._onDidCompositionStart=this._register(new zn(this._contributions,this._deliveryQueue)),this.onDidCompositionStart=this._onDidCompositionStart.event,this._onDidCompositionEnd=this._register(new zn(this._contributions,this._deliveryQueue)),this.onDidCompositionEnd=this._onDidCompositionEnd.event,this._onDidPaste=this._register(new zn(this._contributions,this._deliveryQueue)),this.onDidPaste=this._onDidPaste.event,this._onMouseUp=this._register(new zn(this._contributions,this._deliveryQueue)),this.onMouseUp=this._onMouseUp.event,this._onMouseDown=this._register(new zn(this._contributions,this._deliveryQueue)),this.onMouseDown=this._onMouseDown.event,this._onMouseDrag=this._register(new zn(this._contributions,this._deliveryQueue)),this.onMouseDrag=this._onMouseDrag.event,this._onMouseDrop=this._register(new zn(this._contributions,this._deliveryQueue)),this.onMouseDrop=this._onMouseDrop.event,this._onMouseDropCanceled=this._register(new zn(this._contributions,this._deliveryQueue)),this.onMouseDropCanceled=this._onMouseDropCanceled.event,this._onDropIntoEditor=this._register(new zn(this._contributions,this._deliveryQueue)),this.onDropIntoEditor=this._onDropIntoEditor.event,this._onContextMenu=this._register(new zn(this._contributions,this._deliveryQueue)),this.onContextMenu=this._onContextMenu.event,this._onMouseMove=this._register(new zn(this._contributions,this._deliveryQueue)),this.onMouseMove=this._onMouseMove.event,this._onMouseLeave=this._register(new zn(this._contributions,this._deliveryQueue)),this.onMouseLeave=this._onMouseLeave.event,this._onMouseWheel=this._register(new zn(this._contributions,this._deliveryQueue)),this.onMouseWheel=this._onMouseWheel.event,this._onKeyUp=this._register(new zn(this._contributions,this._deliveryQueue)),this.onKeyUp=this._onKeyUp.event,this._onKeyDown=this._register(new zn(this._contributions,this._deliveryQueue)),this.onKeyDown=this._onKeyDown.event,this._onDidContentSizeChange=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidContentSizeChange=this._onDidContentSizeChange.event,this._onDidScrollChange=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidScrollChange=this._onDidScrollChange.event,this._onDidChangeViewZones=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeViewZones=this._onDidChangeViewZones.event,this._onDidChangeHiddenAreas=this._register(new P({deliveryQueue:this._deliveryQueue})),this.onDidChangeHiddenAreas=this._onDidChangeHiddenAreas.event,this._updateCounter=0,this._onBeginUpdate=this._register(new P),this.onBeginUpdate=this._onBeginUpdate.event,this._onEndUpdate=this._register(new P),this.onEndUpdate=this._onEndUpdate.event,this._actions=new Map,this._bannerDomNode=null,this._dropIntoEditorDecorations=this.createDecorationsCollection(),s.willCreateCodeEditor();const g={...t};this._domElement=e,this._overflowWidgetsDomNode=g.overflowWidgetsDomNode,delete g.overflowWidgetsDomNode,this._id=++OCe,this._decorationTypeKeysToIds={},this._decorationTypeSubtypes={},this._telemetryData=i.telemetryData,this._configuration=this._register(this._createConfiguration(i.isSimpleWidget||!1,i.contextMenuId??(i.isSimpleWidget?me.SimpleEditorContext:me.EditorContext),g,d)),this._register(this._configuration.onDidChange(_=>{this._onDidChangeConfiguration.fire(_);const b=this._configuration.options;if(_.hasChanged(146)){const C=b.get(146);this._onDidLayoutChange.fire(C)}})),this._contextKeyService=this._register(a.createScoped(this._domElement)),this._notificationService=c,this._codeEditorService=s,this._commandService=r,this._themeService=l,this._register(new WCe(this,this._contextKeyService)),this._register(new BCe(this,this._contextKeyService,u)),this._instantiationService=this._register(n.createChild(new Eb([Ee,this._contextKeyService]))),this._modelData=null,this._focusTracker=new HCe(e,this._overflowWidgetsDomNode),this._register(this._focusTracker.onChange(()=>{this._editorWidgetFocus.setValue(this._focusTracker.hasFocus())})),this._contentWidgets={},this._overlayWidgets={},this._glyphMarginWidgets={};let f;Array.isArray(i.contributions)?f=i.contributions:f=_g.getEditorContributions(),this._contributions.initialize(this,f,this._instantiationService);for(const _ of _g.getEditorActions()){if(this._actions.has(_.id)){Oe(new Error(`Cannot have two actions with the same id ${_.id}`));continue}const b=new Tq(_.id,_.label,_.alias,_.metadata,_.precondition??void 0,C=>this._instantiationService.invokeFunction(v=>Promise.resolve(_.runEditorCommand(v,this,C))),this._contextKeyService);this._actions.set(b.id,b)}const p=()=>!this._configuration.options.get(92)&&this._configuration.options.get(36).enabled;this._register(new iie(this._domElement,{onDragOver:_=>{if(!p())return;const b=this.getTargetAtClientPoint(_.clientX,_.clientY);b!=null&&b.position&&this.showDropIndicatorAt(b.position)},onDrop:async _=>{if(!p()||(this.removeDropIndicator(),!_.dataTransfer))return;const b=this.getTargetAtClientPoint(_.clientX,_.clientY);b!=null&&b.position&&this._onDropIntoEditor.fire({position:b.position,event:_})},onDragLeave:()=>{this.removeDropIndicator()},onDragEnd:()=>{this.removeDropIndicator()}})),this._codeEditorService.addCodeEditor(this)}writeScreenReaderContent(e){var t;(t=this._modelData)==null||t.view.writeScreenReaderContent(e)}_createConfiguration(e,t,i,n){return new oP(e,t,i,this._domElement,n)}getId(){return this.getEditorType()+":"+this._id}getEditorType(){return iS.ICodeEditor}dispose(){this._codeEditorService.removeCodeEditor(this),this._focusTracker.dispose(),this._actions.clear(),this._contentWidgets={},this._overlayWidgets={},this._removeDecorationTypes(),this._postDetachModelCleanup(this._detachModel()),this._onDidDispose.fire(),super.dispose()}invokeWithinContext(e){return this._instantiationService.invokeFunction(e)}updateOptions(e){this._configuration.updateOptions(e||{})}getOptions(){return this._configuration.options}getOption(e){return this._configuration.options.get(e)}getRawOptions(){return this._configuration.getRawOptions()}getOverflowWidgetsDomNode(){return this._overflowWidgetsDomNode}getConfiguredWordAtPosition(e){return this._modelData?wt.getWordAtPosition(this._modelData.model,this._configuration.options.get(132),this._configuration.options.get(131),e):null}getValue(e=null){if(!this._modelData)return"";const t=!!(e&&e.preserveBOM);let i=0;return e&&e.lineEnding&&e.lineEnding===`
`?i=1:e&&e.lineEnding&&e.lineEnding===`\r
`&&(i=2),this._modelData.model.getValue(i,t)}setValue(e){try{if(this._beginUpdate(),!this._modelData)return;this._modelData.model.setValue(e)}finally{this._endUpdate()}}getModel(){return this._modelData?this._modelData.model:null}setModel(e=null){var t;try{this._beginUpdate();const i=e;if(this._modelData===null&&i===null||this._modelData&&this._modelData.model===i)return;const n={oldModelUrl:((t=this._modelData)==null?void 0:t.model.uri)||null,newModelUrl:(i==null?void 0:i.uri)||null};this._onWillChangeModel.fire(n);const s=this.hasTextFocus(),r=this._detachModel();this._attachModel(i),s&&this.hasModel()&&this.focus(),this._removeDecorationTypes(),this._onDidChangeModel.fire(n),this._postDetachModelCleanup(r),this._contributionsDisposable=this._contributions.onAfterModelAttached()}finally{this._endUpdate()}}_removeDecorationTypes(){if(this._decorationTypeKeysToIds={},this._decorationTypeSubtypes){for(const e in this._decorationTypeSubtypes){const t=this._decorationTypeSubtypes[e];for(const i in t)this._removeDecorationType(e+"-"+i)}this._decorationTypeSubtypes={}}}getVisibleRanges(){return this._modelData?this._modelData.viewModel.getVisibleRanges():[]}getVisibleRangesPlusViewportAboveBelow(){return this._modelData?this._modelData.viewModel.getVisibleRangesPlusViewportAboveBelow():[]}getWhitespaces(){return this._modelData?this._modelData.viewModel.viewLayout.getWhitespaces():[]}static _getVerticalOffsetAfterPosition(e,t,i,n){const s=e.model.validatePosition({lineNumber:t,column:i}),r=e.viewModel.coordinatesConverter.convertModelPositionToViewPosition(s);return e.viewModel.viewLayout.getVerticalOffsetAfterLineNumber(r.lineNumber,n)}getTopForLineNumber(e,t=!1){return this._modelData?Pu._getVerticalOffsetForPosition(this._modelData,e,1,t):-1}getTopForPosition(e,t){return this._modelData?Pu._getVerticalOffsetForPosition(this._modelData,e,t,!1):-1}static _getVerticalOffsetForPosition(e,t,i,n=!1){const s=e.model.validatePosition({lineNumber:t,column:i}),r=e.viewModel.coordinatesConverter.convertModelPositionToViewPosition(s);return e.viewModel.viewLayout.getVerticalOffsetForLineNumber(r.lineNumber,n)}getBottomForLineNumber(e,t=!1){if(!this._modelData)return-1;const i=this._modelData.model.getLineMaxColumn(e);return Pu._getVerticalOffsetAfterPosition(this._modelData,e,i,t)}setHiddenAreas(e,t){var i;(i=this._modelData)==null||i.viewModel.setHiddenAreas(e.map(n=>x.lift(n)),t)}getVisibleColumnFromPosition(e){if(!this._modelData)return e.column;const t=this._modelData.model.validatePosition(e),i=this._modelData.model.getOptions().tabSize;return Ui.visibleColumnFromColumn(this._modelData.model.getLineContent(t.lineNumber),t.column,i)+1}getPosition(){return this._modelData?this._modelData.viewModel.getPosition():null}setPosition(e,t="api"){if(this._modelData){if(!A.isIPosition(e))throw new Error("Invalid arguments");this._modelData.viewModel.setSelections(t,[{selectionStartLineNumber:e.lineNumber,selectionStartColumn:e.column,positionLineNumber:e.lineNumber,positionColumn:e.column}])}}_sendRevealRange(e,t,i,n){if(!this._modelData)return;if(!x.isIRange(e))throw new Error("Invalid arguments");const s=this._modelData.model.validateRange(e),r=this._modelData.viewModel.coordinatesConverter.convertModelRangeToViewRange(s);this._modelData.viewModel.revealRange("api",i,r,t,n)}revealLine(e,t=0){this._revealLine(e,0,t)}revealLineInCenter(e,t=0){this._revealLine(e,1,t)}revealLineInCenterIfOutsideViewport(e,t=0){this._revealLine(e,2,t)}revealLineNearTop(e,t=0){this._revealLine(e,5,t)}_revealLine(e,t,i){if(typeof e!="number")throw new Error("Invalid arguments");this._sendRevealRange(new x(e,1,e,1),t,!1,i)}revealPosition(e,t=0){this._revealPosition(e,0,!0,t)}revealPositionInCenter(e,t=0){this._revealPosition(e,1,!0,t)}revealPositionInCenterIfOutsideViewport(e,t=0){this._revealPosition(e,2,!0,t)}revealPositionNearTop(e,t=0){this._revealPosition(e,5,!0,t)}_revealPosition(e,t,i,n){if(!A.isIPosition(e))throw new Error("Invalid arguments");this._sendRevealRange(new x(e.lineNumber,e.column,e.lineNumber,e.column),t,i,n)}getSelection(){return this._modelData?this._modelData.viewModel.getSelection():null}getSelections(){return this._modelData?this._modelData.viewModel.getSelections():null}setSelection(e,t="api"){const i=pe.isISelection(e),n=x.isIRange(e);if(!i&&!n)throw new Error("Invalid arguments");if(i)this._setSelectionImpl(e,t);else if(n){const s={selectionStartLineNumber:e.startLineNumber,selectionStartColumn:e.startColumn,positionLineNumber:e.endLineNumber,positionColumn:e.endColumn};this._setSelectionImpl(s,t)}}_setSelectionImpl(e,t){if(!this._modelData)return;const i=new pe(e.selectionStartLineNumber,e.selectionStartColumn,e.positionLineNumber,e.positionColumn);this._modelData.viewModel.setSelections(t,[i])}revealLines(e,t,i=0){this._revealLines(e,t,0,i)}revealLinesInCenter(e,t,i=0){this._revealLines(e,t,1,i)}revealLinesInCenterIfOutsideViewport(e,t,i=0){this._revealLines(e,t,2,i)}revealLinesNearTop(e,t,i=0){this._revealLines(e,t,5,i)}_revealLines(e,t,i,n){if(typeof e!="number"||typeof t!="number")throw new Error("Invalid arguments");this._sendRevealRange(new x(e,1,t,1),i,!1,n)}revealRange(e,t=0,i=!1,n=!0){this._revealRange(e,i?1:0,n,t)}revealRangeInCenter(e,t=0){this._revealRange(e,1,!0,t)}revealRangeInCenterIfOutsideViewport(e,t=0){this._revealRange(e,2,!0,t)}revealRangeNearTop(e,t=0){this._revealRange(e,5,!0,t)}revealRangeNearTopIfOutsideViewport(e,t=0){this._revealRange(e,6,!0,t)}revealRangeAtTop(e,t=0){this._revealRange(e,3,!0,t)}_revealRange(e,t,i,n){if(!x.isIRange(e))throw new Error("Invalid arguments");this._sendRevealRange(x.lift(e),t,i,n)}setSelections(e,t="api",i=0){if(this._modelData){if(!e||e.length===0)throw new Error("Invalid arguments");for(let n=0,s=e.length;n<s;n++)if(!pe.isISelection(e[n]))throw new Error("Invalid arguments");this._modelData.viewModel.setSelections(t,e,i)}}getContentWidth(){return this._modelData?this._modelData.viewModel.viewLayout.getContentWidth():-1}getScrollWidth(){return this._modelData?this._modelData.viewModel.viewLayout.getScrollWidth():-1}getScrollLeft(){return this._modelData?this._modelData.viewModel.viewLayout.getCurrentScrollLeft():-1}getContentHeight(){return this._modelData?this._modelData.viewModel.viewLayout.getContentHeight():-1}getScrollHeight(){return this._modelData?this._modelData.viewModel.viewLayout.getScrollHeight():-1}getScrollTop(){return this._modelData?this._modelData.viewModel.viewLayout.getCurrentScrollTop():-1}setScrollLeft(e,t=1){if(this._modelData){if(typeof e!="number")throw new Error("Invalid arguments");this._modelData.viewModel.viewLayout.setScrollPosition({scrollLeft:e},t)}}setScrollTop(e,t=1){if(this._modelData){if(typeof e!="number")throw new Error("Invalid arguments");this._modelData.viewModel.viewLayout.setScrollPosition({scrollTop:e},t)}}setScrollPosition(e,t=1){this._modelData&&this._modelData.viewModel.viewLayout.setScrollPosition(e,t)}hasPendingScrollAnimation(){return this._modelData?this._modelData.viewModel.viewLayout.hasPendingScrollAnimation():!1}saveViewState(){if(!this._modelData)return null;const e=this._contributions.saveViewState(),t=this._modelData.viewModel.saveCursorState(),i=this._modelData.viewModel.saveState();return{cursorState:t,viewState:i,contributionsState:e}}restoreViewState(e){if(!this._modelData||!this._modelData.hasRealView)return;const t=e;if(t&&t.cursorState&&t.viewState){const i=t.cursorState;Array.isArray(i)?i.length>0&&this._modelData.viewModel.restoreCursorState(i):this._modelData.viewModel.restoreCursorState([i]),this._contributions.restoreViewState(t.contributionsState||{});const n=this._modelData.viewModel.reduceRestoreState(t.viewState);this._modelData.view.restoreState(n)}}handleInitialized(){var e;(e=this._getViewModel())==null||e.visibleLinesStabilized()}getContribution(e){return this._contributions.get(e)}getActions(){return Array.from(this._actions.values())}getSupportedActions(){let e=this.getActions();return e=e.filter(t=>t.isSupported()),e}getAction(e){return this._actions.get(e)||null}trigger(e,t,i){i=i||{};try{switch(this._beginUpdate(),t){case"compositionStart":this._startComposition();return;case"compositionEnd":this._endComposition(e);return;case"type":{const s=i;this._type(e,s.text||"");return}case"replacePreviousChar":{const s=i;this._compositionType(e,s.text||"",s.replaceCharCnt||0,0,0);return}case"compositionType":{const s=i;this._compositionType(e,s.text||"",s.replacePrevCharCnt||0,s.replaceNextCharCnt||0,s.positionDelta||0);return}case"paste":{const s=i;this._paste(e,s.text||"",s.pasteOnNewLine||!1,s.multicursorText||null,s.mode||null,s.clipboardEvent);return}case"cut":this._cut(e);return}const n=this.getAction(t);if(n){Promise.resolve(n.run(i)).then(void 0,Oe);return}if(!this._modelData||this._triggerEditorCommand(e,t,i))return;this._triggerCommand(t,i)}finally{this._endUpdate()}}_triggerCommand(e,t){this._commandService.executeCommand(e,t)}_startComposition(){this._modelData&&(this._modelData.viewModel.startComposition(),this._onDidCompositionStart.fire())}_endComposition(e){this._modelData&&(this._modelData.viewModel.endComposition(e),this._onDidCompositionEnd.fire())}_type(e,t){!this._modelData||t.length===0||(e==="keyboard"&&this._onWillType.fire(t),this._modelData.viewModel.type(t,e),e==="keyboard"&&this._onDidType.fire(t))}_compositionType(e,t,i,n,s){this._modelData&&this._modelData.viewModel.compositionType(t,i,n,s,e)}_paste(e,t,i,n,s,r){if(!this._modelData)return;const a=this._modelData.viewModel,l=a.getSelection().getStartPosition();a.paste(t,i,n,e);const c=a.getSelection().getStartPosition();e==="keyboard"&&this._onDidPaste.fire({clipboardEvent:r,range:new x(l.lineNumber,l.column,c.lineNumber,c.column),languageId:s})}_cut(e){this._modelData&&this._modelData.viewModel.cut(e)}_triggerEditorCommand(e,t,i){const n=_g.getEditorCommand(t);return n?(i=i||{},i.source=e,this._instantiationService.invokeFunction(s=>{Promise.resolve(n.runEditorCommand(s,this,i)).then(void 0,Oe)}),!0):!1}_getViewModel(){return this._modelData?this._modelData.viewModel:null}pushUndoStop(){return!this._modelData||this._configuration.options.get(92)?!1:(this._modelData.model.pushStackElement(),!0)}popUndoStop(){return!this._modelData||this._configuration.options.get(92)?!1:(this._modelData.model.popStackElement(),!0)}executeEdits(e,t,i){if(!this._modelData||this._configuration.options.get(92))return!1;let n;return i?Array.isArray(i)?n=()=>i:n=i:n=()=>null,this._modelData.viewModel.executeEdits(e,t,n),!0}executeCommand(e,t){this._modelData&&this._modelData.viewModel.executeCommand(t,e)}executeCommands(e,t){this._modelData&&this._modelData.viewModel.executeCommands(t,e)}createDecorationsCollection(e){return new VCe(this,e)}changeDecorations(e){return this._modelData?this._modelData.model.changeDecorations(e,this._id):null}getLineDecorations(e){return this._modelData?this._modelData.model.getLineDecorations(e,this._id,Ty(this._configuration.options)):null}getDecorationsInRange(e){return this._modelData?this._modelData.model.getDecorationsInRange(e,this._id,Ty(this._configuration.options)):null}deltaDecorations(e,t){return this._modelData?e.length===0&&t.length===0?e:this._modelData.model.deltaDecorations(e,t,this._id):[]}removeDecorations(e){!this._modelData||e.length===0||this._modelData.model.changeDecorations(t=>{t.deltaDecorations(e,[])})}removeDecorationsByType(e){const t=this._decorationTypeKeysToIds[e];t&&this.changeDecorations(i=>i.deltaDecorations(t,[])),this._decorationTypeKeysToIds.hasOwnProperty(e)&&delete this._decorationTypeKeysToIds[e],this._decorationTypeSubtypes.hasOwnProperty(e)&&delete this._decorationTypeSubtypes[e]}getLayoutInfo(){return this._configuration.options.get(146)}createOverviewRuler(e){return!this._modelData||!this._modelData.hasRealView?null:this._modelData.view.createOverviewRuler(e)}getContainerDomNode(){return this._domElement}getDomNode(){return!this._modelData||!this._modelData.hasRealView?null:this._modelData.view.domNode.domNode}delegateVerticalScrollbarPointerDown(e){!this._modelData||!this._modelData.hasRealView||this._modelData.view.delegateVerticalScrollbarPointerDown(e)}delegateScrollFromMouseWheelEvent(e){!this._modelData||!this._modelData.hasRealView||this._modelData.view.delegateScrollFromMouseWheelEvent(e)}layout(e,t=!1){this._configuration.observeContainer(e),t||this.render()}focus(){!this._modelData||!this._modelData.hasRealView||this._modelData.view.focus()}hasTextFocus(){return!this._modelData||!this._modelData.hasRealView?!1:this._modelData.view.isFocused()}hasWidgetFocus(){return this._focusTracker&&this._focusTracker.hasFocus()}addContentWidget(e){const t={widget:e,position:e.getPosition()};this._contentWidgets.hasOwnProperty(e.getId())&&console.warn("Overwriting a content widget with the same id:"+e.getId()),this._contentWidgets[e.getId()]=t,this._modelData&&this._modelData.hasRealView&&this._modelData.view.addContentWidget(t)}layoutContentWidget(e){const t=e.getId();if(this._contentWidgets.hasOwnProperty(t)){const i=this._contentWidgets[t];i.position=e.getPosition(),this._modelData&&this._modelData.hasRealView&&this._modelData.view.layoutContentWidget(i)}}removeContentWidget(e){const t=e.getId();if(this._contentWidgets.hasOwnProperty(t)){const i=this._contentWidgets[t];delete this._contentWidgets[t],this._modelData&&this._modelData.hasRealView&&this._modelData.view.removeContentWidget(i)}}addOverlayWidget(e){const t={widget:e,position:e.getPosition()};this._overlayWidgets.hasOwnProperty(e.getId())&&console.warn("Overwriting an overlay widget with the same id."),this._overlayWidgets[e.getId()]=t,this._modelData&&this._modelData.hasRealView&&this._modelData.view.addOverlayWidget(t)}layoutOverlayWidget(e){const t=e.getId();if(this._overlayWidgets.hasOwnProperty(t)){const i=this._overlayWidgets[t];i.position=e.getPosition(),this._modelData&&this._modelData.hasRealView&&this._modelData.view.layoutOverlayWidget(i)}}removeOverlayWidget(e){const t=e.getId();if(this._overlayWidgets.hasOwnProperty(t)){const i=this._overlayWidgets[t];delete this._overlayWidgets[t],this._modelData&&this._modelData.hasRealView&&this._modelData.view.removeOverlayWidget(i)}}addGlyphMarginWidget(e){const t={widget:e,position:e.getPosition()};this._glyphMarginWidgets.hasOwnProperty(e.getId())&&console.warn("Overwriting a glyph margin widget with the same id."),this._glyphMarginWidgets[e.getId()]=t,this._modelData&&this._modelData.hasRealView&&this._modelData.view.addGlyphMarginWidget(t)}layoutGlyphMarginWidget(e){const t=e.getId();if(this._glyphMarginWidgets.hasOwnProperty(t)){const i=this._glyphMarginWidgets[t];i.position=e.getPosition(),this._modelData&&this._modelData.hasRealView&&this._modelData.view.layoutGlyphMarginWidget(i)}}removeGlyphMarginWidget(e){const t=e.getId();if(this._glyphMarginWidgets.hasOwnProperty(t)){const i=this._glyphMarginWidgets[t];delete this._glyphMarginWidgets[t],this._modelData&&this._modelData.hasRealView&&this._modelData.view.removeGlyphMarginWidget(i)}}changeViewZones(e){!this._modelData||!this._modelData.hasRealView||this._modelData.view.change(e)}getTargetAtClientPoint(e,t){return!this._modelData||!this._modelData.hasRealView?null:this._modelData.view.getTargetAtClientPoint(e,t)}getScrolledVisiblePosition(e){if(!this._modelData||!this._modelData.hasRealView)return null;const t=this._modelData.model.validatePosition(e),i=this._configuration.options,n=i.get(146),s=Pu._getVerticalOffsetForPosition(this._modelData,t.lineNumber,t.column)-this.getScrollTop(),r=this._modelData.view.getOffsetForColumn(t.lineNumber,t.column)+n.glyphMarginWidth+n.lineNumbersWidth+n.decorationsWidth-this.getScrollLeft();return{top:s,left:r,height:i.get(67)}}getOffsetForColumn(e,t){return!this._modelData||!this._modelData.hasRealView?-1:this._modelData.view.getOffsetForColumn(e,t)}render(e=!1){!this._modelData||!this._modelData.hasRealView||this._modelData.viewModel.batchEvents(()=>{this._modelData.view.render(!0,e)})}setAriaOptions(e){!this._modelData||!this._modelData.hasRealView||this._modelData.view.setAriaOptions(e)}applyFontInfo(e){bn(e,this._configuration.options.get(50))}setBanner(e,t){this._bannerDomNode&&this._domElement.contains(this._bannerDomNode)&&this._bannerDomNode.remove(),this._bannerDomNode=e,this._configuration.setReservedHeight(e?t:0),this._bannerDomNode&&this._domElement.prepend(this._bannerDomNode)}_attachModel(e){if(!e){this._modelData=null;return}const t=[];this._domElement.setAttribute("data-mode-id",e.getLanguageId()),this._configuration.setIsDominatedByLongLines(e.isDominatedByLongLines()),this._configuration.setModelLineCount(e.getLineCount());const i=e.onBeforeAttached(),n=new TCe(this._id,this._configuration,e,J3.create(xe(this._domElement)),eW.create(this._configuration.options),a=>go(xe(this._domElement),a),this.languageConfigurationService,this._themeService,i,{batchChanges:a=>{try{return this._beginUpdate(),a()}finally{this._endUpdate()}}});t.push(e.onWillDispose(()=>this.setModel(null))),t.push(n.onEvent(a=>{switch(a.kind){case 0:this._onDidContentSizeChange.fire(a);break;case 1:this._editorTextFocus.setValue(a.hasFocus);break;case 2:this._onDidScrollChange.fire(a);break;case 3:this._onDidChangeViewZones.fire();break;case 4:this._onDidChangeHiddenAreas.fire();break;case 5:this._onDidAttemptReadOnlyEdit.fire();break;case 6:{if(a.reachedMaxCursorCount){const h=this.getOption(80),u=m("cursors.maximum","The number of cursors has been limited to {0}. Consider using [find and replace](https://code.visualstudio.com/docs/editor/codebasics#_find-and-replace) for larger changes or increase the editor multi cursor limit setting.",h);this._notificationService.prompt(OI.Warning,u,[{label:"Find and Replace",run:()=>{this._commandService.executeCommand("editor.action.startFindReplaceAction")}},{label:m("goToSetting","Increase Multi Cursor Limit"),run:()=>{this._commandService.executeCommand("workbench.action.openSettings2",{query:"editor.multiCursorLimit"})}}])}const l=[];for(let h=0,u=a.selections.length;h<u;h++)l[h]=a.selections[h].getPosition();const c={position:l[0],secondaryPositions:l.slice(1),reason:a.reason,source:a.source};this._onDidChangeCursorPosition.fire(c);const d={selection:a.selections[0],secondarySelections:a.selections.slice(1),modelVersionId:a.modelVersionId,oldSelections:a.oldSelections,oldModelVersionId:a.oldModelVersionId,source:a.source,reason:a.reason};this._onDidChangeCursorSelection.fire(d);break}case 7:this._onDidChangeModelDecorations.fire(a.event);break;case 8:this._domElement.setAttribute("data-mode-id",e.getLanguageId()),this._onDidChangeModelLanguage.fire(a.event);break;case 9:this._onDidChangeModelLanguageConfiguration.fire(a.event);break;case 10:this._onDidChangeModelContent.fire(a.event);break;case 11:this._onDidChangeModelOptions.fire(a.event);break;case 12:this._onDidChangeModelTokens.fire(a.event);break}}));const[s,r]=this._createView(n);if(r){this._domElement.appendChild(s.domNode.domNode);let a=Object.keys(this._contentWidgets);for(let l=0,c=a.length;l<c;l++){const d=a[l];s.addContentWidget(this._contentWidgets[d])}a=Object.keys(this._overlayWidgets);for(let l=0,c=a.length;l<c;l++){const d=a[l];s.addOverlayWidget(this._overlayWidgets[d])}a=Object.keys(this._glyphMarginWidgets);for(let l=0,c=a.length;l<c;l++){const d=a[l];s.addGlyphMarginWidget(this._glyphMarginWidgets[d])}s.render(!1,!0),s.domNode.domNode.setAttribute("data-uri",e.uri.toString())}this._modelData=new FCe(e,n,s,r,t,i)}_createView(e){let t;this.isSimpleWidget?t={paste:(s,r,a,l)=>{this._paste("keyboard",s,r,a,l)},type:s=>{this._type("keyboard",s)},compositionType:(s,r,a,l)=>{this._compositionType("keyboard",s,r,a,l)},startComposition:()=>{this._startComposition()},endComposition:()=>{this._endComposition("keyboard")},cut:()=>{this._cut("keyboard")}}:t={paste:(s,r,a,l)=>{const c={text:s,pasteOnNewLine:r,multicursorText:a,mode:l};this._commandService.executeCommand("paste",c)},type:s=>{const r={text:s};this._commandService.executeCommand("type",r)},compositionType:(s,r,a,l)=>{if(a||l){const c={text:s,replacePrevCharCnt:r,replaceNextCharCnt:a,positionDelta:l};this._commandService.executeCommand("compositionType",c)}else{const c={text:s,replaceCharCnt:r};this._commandService.executeCommand("replacePreviousChar",c)}},startComposition:()=>{this._commandService.executeCommand("compositionStart",{})},endComposition:()=>{this._commandService.executeCommand("compositionEnd",{})},cut:()=>{this._commandService.executeCommand("cut",{})}};const i=new wE(e.coordinatesConverter);return i.onKeyDown=s=>this._onKeyDown.fire(s),i.onKeyUp=s=>this._onKeyUp.fire(s),i.onContextMenu=s=>this._onContextMenu.fire(s),i.onMouseMove=s=>this._onMouseMove.fire(s),i.onMouseLeave=s=>this._onMouseLeave.fire(s),i.onMouseDown=s=>this._onMouseDown.fire(s),i.onMouseUp=s=>this._onMouseUp.fire(s),i.onMouseDrag=s=>this._onMouseDrag.fire(s),i.onMouseDrop=s=>this._onMouseDrop.fire(s),i.onMouseDropCanceled=s=>this._onMouseDropCanceled.fire(s),i.onMouseWheel=s=>this._onMouseWheel.fire(s),[new CP(t,this._configuration,this._themeService.getColorTheme(),e,i,this._overflowWidgetsDomNode,this._instantiationService),!0]}_postDetachModelCleanup(e){e==null||e.removeAllDecorationsWithOwnerId(this._id)}_detachModel(){var i;if((i=this._contributionsDisposable)==null||i.dispose(),this._contributionsDisposable=void 0,!this._modelData)return null;const e=this._modelData.model,t=this._modelData.hasRealView?this._modelData.view.domNode.domNode:null;return this._modelData.dispose(),this._modelData=null,this._domElement.removeAttribute("data-mode-id"),t&&this._domElement.contains(t)&&t.remove(),this._bannerDomNode&&this._domElement.contains(this._bannerDomNode)&&this._bannerDomNode.remove(),e}_removeDecorationType(e){this._codeEditorService.removeDecorationType(e)}hasModel(){return this._modelData!==null}showDropIndicatorAt(e){const t=[{range:new x(e.lineNumber,e.column,e.lineNumber,e.column),options:Pu.dropIntoEditorDecorationOptions}];this._dropIntoEditorDecorations.set(t),this.revealPosition(e,1)}removeDropIndicator(){this._dropIntoEditorDecorations.clear()}setContextValue(e,t){this._contextKeyService.createKey(e,t)}_beginUpdate(){this._updateCounter++,this._updateCounter===1&&this._onBeginUpdate.fire()}_endUpdate(){this._updateCounter--,this._updateCounter===0&&this._onEndUpdate.fire()}},Pu=Mg,Mg.dropIntoEditorDecorationOptions=We.register({description:"workbench-dnd-target",className:"dnd-target"}),Mg);Zw=Pu=PCe([oc(3,we),oc(4,pt),oc(5,Ht),oc(6,Ee),oc(7,ji),oc(8,Ei),oc(9,po),oc(10,di),oc(11,ge)],Zw);let OCe=0;class FCe{constructor(e,t,i,n,s,r){this.model=e,this.viewModel=t,this.view=i,this.hasRealView=n,this.listenersToRemove=s,this.attachedView=r}dispose(){Tt(this.listenersToRemove),this.model.onBeforeDetached(this.attachedView),this.hasRealView&&this.view.dispose(),this.viewModel.dispose()}}class GH extends F{constructor(e){super(),this._emitterOptions=e,this._onDidChangeToTrue=this._register(new P(this._emitterOptions)),this.onDidChangeToTrue=this._onDidChangeToTrue.event,this._onDidChangeToFalse=this._register(new P(this._emitterOptions)),this.onDidChangeToFalse=this._onDidChangeToFalse.event,this._value=0}setValue(e){const t=e?2:1;this._value!==t&&(this._value=t,this._value===2?this._onDidChangeToTrue.fire():this._value===1&&this._onDidChangeToFalse.fire())}}class zn extends P{constructor(e,t){super({deliveryQueue:t}),this._contributions=e}fire(e){this._contributions.onBeforeInteractionEvent(),super.fire(e)}}class WCe extends F{constructor(e,t){super(),this._editor=e,t.createKey("editorId",e.getId()),this._editorSimpleInput=I.editorSimpleInput.bindTo(t),this._editorFocus=I.focus.bindTo(t),this._textInputFocus=I.textInputFocus.bindTo(t),this._editorTextFocus=I.editorTextFocus.bindTo(t),this._tabMovesFocus=I.tabMovesFocus.bindTo(t),this._editorReadonly=I.readOnly.bindTo(t),this._inDiffEditor=I.inDiffEditor.bindTo(t),this._editorColumnSelection=I.columnSelection.bindTo(t),this._hasMultipleSelections=I.hasMultipleSelections.bindTo(t),this._hasNonEmptySelection=I.hasNonEmptySelection.bindTo(t),this._canUndo=I.canUndo.bindTo(t),this._canRedo=I.canRedo.bindTo(t),this._register(this._editor.onDidChangeConfiguration(()=>this._updateFromConfig())),this._register(this._editor.onDidChangeCursorSelection(()=>this._updateFromSelection())),this._register(this._editor.onDidFocusEditorWidget(()=>this._updateFromFocus())),this._register(this._editor.onDidBlurEditorWidget(()=>this._updateFromFocus())),this._register(this._editor.onDidFocusEditorText(()=>this._updateFromFocus())),this._register(this._editor.onDidBlurEditorText(()=>this._updateFromFocus())),this._register(this._editor.onDidChangeModel(()=>this._updateFromModel())),this._register(this._editor.onDidChangeConfiguration(()=>this._updateFromModel())),this._register(K_.onDidChangeTabFocus(i=>this._tabMovesFocus.set(i))),this._updateFromConfig(),this._updateFromSelection(),this._updateFromFocus(),this._updateFromModel(),this._editorSimpleInput.set(this._editor.isSimpleWidget)}_updateFromConfig(){const e=this._editor.getOptions();this._tabMovesFocus.set(K_.getTabFocusMode()),this._editorReadonly.set(e.get(92)),this._inDiffEditor.set(e.get(61)),this._editorColumnSelection.set(e.get(22))}_updateFromSelection(){const e=this._editor.getSelections();e?(this._hasMultipleSelections.set(e.length>1),this._hasNonEmptySelection.set(e.some(t=>!t.isEmpty()))):(this._hasMultipleSelections.reset(),this._hasNonEmptySelection.reset())}_updateFromFocus(){this._editorFocus.set(this._editor.hasWidgetFocus()&&!this._editor.isSimpleWidget),this._editorTextFocus.set(this._editor.hasTextFocus()&&!this._editor.isSimpleWidget),this._textInputFocus.set(this._editor.hasTextFocus())}_updateFromModel(){const e=this._editor.getModel();this._canUndo.set(!!(e&&e.canUndo())),this._canRedo.set(!!(e&&e.canRedo()))}}class BCe extends F{constructor(e,t,i){super(),this._editor=e,this._contextKeyService=t,this._languageFeaturesService=i,this._langId=I.languageId.bindTo(t),this._hasCompletionItemProvider=I.hasCompletionItemProvider.bindTo(t),this._hasCodeActionsProvider=I.hasCodeActionsProvider.bindTo(t),this._hasCodeLensProvider=I.hasCodeLensProvider.bindTo(t),this._hasDefinitionProvider=I.hasDefinitionProvider.bindTo(t),this._hasDeclarationProvider=I.hasDeclarationProvider.bindTo(t),this._hasImplementationProvider=I.hasImplementationProvider.bindTo(t),this._hasTypeDefinitionProvider=I.hasTypeDefinitionProvider.bindTo(t),this._hasHoverProvider=I.hasHoverProvider.bindTo(t),this._hasDocumentHighlightProvider=I.hasDocumentHighlightProvider.bindTo(t),this._hasDocumentSymbolProvider=I.hasDocumentSymbolProvider.bindTo(t),this._hasReferenceProvider=I.hasReferenceProvider.bindTo(t),this._hasRenameProvider=I.hasRenameProvider.bindTo(t),this._hasSignatureHelpProvider=I.hasSignatureHelpProvider.bindTo(t),this._hasInlayHintsProvider=I.hasInlayHintsProvider.bindTo(t),this._hasDocumentFormattingProvider=I.hasDocumentFormattingProvider.bindTo(t),this._hasDocumentSelectionFormattingProvider=I.hasDocumentSelectionFormattingProvider.bindTo(t),this._hasMultipleDocumentFormattingProvider=I.hasMultipleDocumentFormattingProvider.bindTo(t),this._hasMultipleDocumentSelectionFormattingProvider=I.hasMultipleDocumentSelectionFormattingProvider.bindTo(t),this._isInEmbeddedEditor=I.isInEmbeddedEditor.bindTo(t);const n=()=>this._update();this._register(e.onDidChangeModel(n)),this._register(e.onDidChangeModelLanguage(n)),this._register(i.completionProvider.onDidChange(n)),this._register(i.codeActionProvider.onDidChange(n)),this._register(i.codeLensProvider.onDidChange(n)),this._register(i.definitionProvider.onDidChange(n)),this._register(i.declarationProvider.onDidChange(n)),this._register(i.implementationProvider.onDidChange(n)),this._register(i.typeDefinitionProvider.onDidChange(n)),this._register(i.hoverProvider.onDidChange(n)),this._register(i.documentHighlightProvider.onDidChange(n)),this._register(i.documentSymbolProvider.onDidChange(n)),this._register(i.referenceProvider.onDidChange(n)),this._register(i.renameProvider.onDidChange(n)),this._register(i.documentFormattingEditProvider.onDidChange(n)),this._register(i.documentRangeFormattingEditProvider.onDidChange(n)),this._register(i.signatureHelpProvider.onDidChange(n)),this._register(i.inlayHintsProvider.onDidChange(n)),n()}dispose(){super.dispose()}reset(){this._contextKeyService.bufferChangeEvents(()=>{this._langId.reset(),this._hasCompletionItemProvider.reset(),this._hasCodeActionsProvider.reset(),this._hasCodeLensProvider.reset(),this._hasDefinitionProvider.reset(),this._hasDeclarationProvider.reset(),this._hasImplementationProvider.reset(),this._hasTypeDefinitionProvider.reset(),this._hasHoverProvider.reset(),this._hasDocumentHighlightProvider.reset(),this._hasDocumentSymbolProvider.reset(),this._hasReferenceProvider.reset(),this._hasRenameProvider.reset(),this._hasDocumentFormattingProvider.reset(),this._hasDocumentSelectionFormattingProvider.reset(),this._hasSignatureHelpProvider.reset(),this._isInEmbeddedEditor.reset()})}_update(){const e=this._editor.getModel();if(!e){this.reset();return}this._contextKeyService.bufferChangeEvents(()=>{this._langId.set(e.getLanguageId()),this._hasCompletionItemProvider.set(this._languageFeaturesService.completionProvider.has(e)),this._hasCodeActionsProvider.set(this._languageFeaturesService.codeActionProvider.has(e)),this._hasCodeLensProvider.set(this._languageFeaturesService.codeLensProvider.has(e)),this._hasDefinitionProvider.set(this._languageFeaturesService.definitionProvider.has(e)),this._hasDeclarationProvider.set(this._languageFeaturesService.declarationProvider.has(e)),this._hasImplementationProvider.set(this._languageFeaturesService.implementationProvider.has(e)),this._hasTypeDefinitionProvider.set(this._languageFeaturesService.typeDefinitionProvider.has(e)),this._hasHoverProvider.set(this._languageFeaturesService.hoverProvider.has(e)),this._hasDocumentHighlightProvider.set(this._languageFeaturesService.documentHighlightProvider.has(e)),this._hasDocumentSymbolProvider.set(this._languageFeaturesService.documentSymbolProvider.has(e)),this._hasReferenceProvider.set(this._languageFeaturesService.referenceProvider.has(e)),this._hasRenameProvider.set(this._languageFeaturesService.renameProvider.has(e)),this._hasSignatureHelpProvider.set(this._languageFeaturesService.signatureHelpProvider.has(e)),this._hasInlayHintsProvider.set(this._languageFeaturesService.inlayHintsProvider.has(e)),this._hasDocumentFormattingProvider.set(this._languageFeaturesService.documentFormattingEditProvider.has(e)||this._languageFeaturesService.documentRangeFormattingEditProvider.has(e)),this._hasDocumentSelectionFormattingProvider.set(this._languageFeaturesService.documentRangeFormattingEditProvider.has(e)),this._hasMultipleDocumentFormattingProvider.set(this._languageFeaturesService.documentFormattingEditProvider.all(e).length+this._languageFeaturesService.documentRangeFormattingEditProvider.all(e).length>1),this._hasMultipleDocumentSelectionFormattingProvider.set(this._languageFeaturesService.documentRangeFormattingEditProvider.all(e).length>1),this._isInEmbeddedEditor.set(e.uri.scheme===Fe.walkThroughSnippet||e.uri.scheme===Fe.vscodeChatCodeBlock)})}}class HCe extends F{constructor(e,t){super(),this._onChange=this._register(new P),this.onChange=this._onChange.event,this._hadFocus=void 0,this._hasDomElementFocus=!1,this._domFocusTracker=this._register(Fa(e)),this._overflowWidgetsDomNodeHasFocus=!1,this._register(this._domFocusTracker.onDidFocus(()=>{this._hasDomElementFocus=!0,this._update()})),this._register(this._domFocusTracker.onDidBlur(()=>{this._hasDomElementFocus=!1,this._update()})),t&&(this._overflowWidgetsDomNode=this._register(Fa(t)),this._register(this._overflowWidgetsDomNode.onDidFocus(()=>{this._overflowWidgetsDomNodeHasFocus=!0,this._update()})),this._register(this._overflowWidgetsDomNode.onDidBlur(()=>{this._overflowWidgetsDomNodeHasFocus=!1,this._update()})))}_update(){const e=this._hasDomElementFocus||this._overflowWidgetsDomNodeHasFocus;this._hadFocus!==e&&(this._hadFocus=e,this._onChange.fire(void 0))}hasFocus(){return this._hadFocus??!1}}class VCe{get length(){return this._decorationIds.length}constructor(e,t){this._editor=e,this._decorationIds=[],this._isChangingDecorations=!1,Array.isArray(t)&&t.length>0&&this.set(t)}onDidChange(e,t,i){return this._editor.onDidChangeModelDecorations(n=>{this._isChangingDecorations||e.call(t,n)},i)}getRange(e){return!this._editor.hasModel()||e>=this._decorationIds.length?null:this._editor.getModel().getDecorationRange(this._decorationIds[e])}getRanges(){if(!this._editor.hasModel())return[];const e=this._editor.getModel(),t=[];for(const i of this._decorationIds){const n=e.getDecorationRange(i);n&&t.push(n)}return t}has(e){return this._decorationIds.includes(e.id)}clear(){this._decorationIds.length!==0&&this.set([])}set(e){try{this._isChangingDecorations=!0,this._editor.changeDecorations(t=>{this._decorationIds=t.deltaDecorations(this._decorationIds,e)})}finally{this._isChangingDecorations=!1}return this._decorationIds}append(e){let t=[];try{this._isChangingDecorations=!0,this._editor.changeDecorations(i=>{t=i.deltaDecorations([],e),this._decorationIds=this._decorationIds.concat(t)})}finally{this._isChangingDecorations=!1}return t}}const zCe=encodeURIComponent("<svg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 6 3' enable-background='new 0 0 6 3' height='3' width='6'><g fill='"),UCe=encodeURIComponent("'><polygon points='5.5,0 2.5,3 1.1,3 4.1,0'/><polygon points='4,0 6,2 6,0.6 5.4,0'/><polygon points='0,2 1,3 2.4,3 0,0.6'/></g></svg>");function DT(o){return zCe+encodeURIComponent(o.toString())+UCe}const $Ce=encodeURIComponent('<svg xmlns="http://www.w3.org/2000/svg" height="3" width="12"><g fill="'),jCe=encodeURIComponent('"><circle cx="1" cy="1" r="1"/><circle cx="5" cy="1" r="1"/><circle cx="9" cy="1" r="1"/></g></svg>');function KCe(o){return $Ce+encodeURIComponent(o.toString())+jCe}Lr((o,e)=>{const t=o.getColor(HI);t&&e.addRule(`.monaco-editor .squiggly-error { background: url("data:image/svg+xml,${DT(t)}") repeat-x bottom left; }`);const i=o.getColor(id);i&&e.addRule(`.monaco-editor .squiggly-warning { background: url("data:image/svg+xml,${DT(i)}") repeat-x bottom left; }`);const n=o.getColor(Bl);n&&e.addRule(`.monaco-editor .squiggly-info { background: url("data:image/svg+xml,${DT(n)}") repeat-x bottom left; }`);const s=o.getColor(Foe);s&&e.addRule(`.monaco-editor .squiggly-hint { background: url("data:image/svg+xml,${KCe(s)}") no-repeat bottom left; }`);const r=o.getColor(vhe);r&&e.addRule(`.monaco-editor.showUnused .squiggly-inline-unnecessary { opacity: ${r.rgba.a}; }`)});class za{static capture(e){if(e.getScrollTop()===0||e.hasPendingScrollAnimation())return new za(e.getScrollTop(),e.getContentHeight(),null,0,null);let t=null,i=0;const n=e.getVisibleRanges();if(n.length>0){t=n[0].getStartPosition();const s=e.getTopForPosition(t.lineNumber,t.column);i=e.getScrollTop()-s}return new za(e.getScrollTop(),e.getContentHeight(),t,i,e.getPosition())}constructor(e,t,i,n,s){this._initialScrollTop=e,this._initialContentHeight=t,this._visiblePosition=i,this._visiblePositionScrollDelta=n,this._cursorPosition=s}restore(e){if(!(this._initialContentHeight===e.getContentHeight()&&this._initialScrollTop===e.getScrollTop())&&this._visiblePosition){const t=e.getTopForPosition(this._visiblePosition.lineNumber,this._visiblePosition.column);e.setScrollTop(t+this._visiblePositionScrollDelta)}}restoreRelativeVerticalPositionOfCursor(e){if(this._initialContentHeight===e.getContentHeight()&&this._initialScrollTop===e.getScrollTop())return;const t=e.getPosition();if(!this._cursorPosition||!t)return;const i=e.getTopForLineNumber(t.lineNumber)-e.getTopForLineNumber(this._cursorPosition.lineNumber);e.setScrollTop(e.getScrollTop()+i,1)}}function qCe(o,e,t,i){if(o.length===0)return e;if(e.length===0)return o;const n=[];let s=0,r=0;for(;s<o.length&&r<e.length;){const a=o[s],l=e[r],c=t(a),d=t(l);c<d?(n.push(a),s++):c>d?(n.push(l),r++):(n.push(i(a,l)),s++,r++)}for(;s<o.length;)n.push(o[s]),s++;for(;r<e.length;)n.push(e[r]),r++;return n}function _x(o,e){const t=new K,i=o.createDecorationsCollection();return t.add(G1({debugName:()=>`Apply decorations from ${e.debugName}`},n=>{const s=e.read(n);i.set(s)})),t.add({dispose:()=>{i.clear()}}),t}function fp(o,e){return o.appendChild(e),ve(()=>{e.remove()})}function GCe(o,e){return o.prepend(e),ve(()=>{e.remove()})}class Pq extends F{get width(){return this._width}get height(){return this._height}get automaticLayout(){return this._automaticLayout}constructor(e,t){super(),this._automaticLayout=!1,this.elementSizeObserver=this._register(new fq(e,t)),this._width=Ke(this,this.elementSizeObserver.getWidth()),this._height=Ke(this,this.elementSizeObserver.getHeight()),this._register(this.elementSizeObserver.onDidChange(i=>ai(n=>{this._width.set(this.elementSizeObserver.getWidth(),n),this._height.set(this.elementSizeObserver.getHeight(),n)})))}observe(e){this.elementSizeObserver.observe(e)}setAutomaticLayout(e){this._automaticLayout=e,e?this.elementSizeObserver.startObserving():this.elementSizeObserver.stopObserving()}}function ZH(o,e,t){let i=e.get(),n=i,s=i;const r=Ke("animatedValue",i);let a=-1;const l=300;let c;t.add(Z1({createEmptyChangeSummary:()=>({animate:!1}),handleChange:(h,u)=>(h.didChange(e)&&(u.animate=u.animate||h.change),!0)},(h,u)=>{c!==void 0&&(o.cancelAnimationFrame(c),c=void 0),n=s,i=e.read(h),a=Date.now()-(u.animate?0:l),d()}));function d(){const h=Date.now()-a;s=Math.floor(ZCe(h,n,i-n,l)),h<l?c=o.requestAnimationFrame(d):s=i,r.set(s,void 0)}return r}function ZCe(o,e,t,i){return o===i?e+t:t*(-Math.pow(2,-10*o/i)+1)+e}class lW extends F{constructor(e,t,i){super(),this._register(new DP(e,i)),this._register(sd(i,{height:t.actualHeight,top:t.actualTop}))}}class mp{get afterLineNumber(){return this._afterLineNumber.get()}constructor(e,t){this._afterLineNumber=e,this.heightInPx=t,this.domNode=document.createElement("div"),this._actualTop=Ke(this,void 0),this._actualHeight=Ke(this,void 0),this.actualTop=this._actualTop,this.actualHeight=this._actualHeight,this.showInHiddenAreas=!0,this.onChange=this._afterLineNumber,this.onDomNodeTop=i=>{this._actualTop.set(i,void 0)},this.onComputedHeight=i=>{this._actualHeight.set(i,void 0)}}}const pD=class pD{constructor(e,t){this._editor=e,this._domElement=t,this._overlayWidgetId=`managedOverlayWidget-${pD._counter++}`,this._overlayWidget={getId:()=>this._overlayWidgetId,getDomNode:()=>this._domElement,getPosition:()=>null},this._editor.addOverlayWidget(this._overlayWidget)}dispose(){this._editor.removeOverlayWidget(this._overlayWidget)}};pD._counter=0;let DP=pD;function sd(o,e){return Ae(t=>{for(let[i,n]of Object.entries(e))n&&typeof n=="object"&&"read"in n&&(n=n.read(t)),typeof n=="number"&&(n=`${n}px`),i=i.replace(/[A-Z]/g,s=>"-"+s.toLowerCase()),o.style[i]=n})}function bx(o,e,t,i){const n=new K,s=[];return n.add(Ps((r,a)=>{const l=e.read(r),c=new Map,d=new Map;t&&t(!0),o.changeViewZones(h=>{for(const u of s)h.removeZone(u),i==null||i.delete(u);s.length=0;for(const u of l){const g=h.addZone(u);u.setZoneId&&u.setZoneId(g),s.push(g),i==null||i.add(g),c.set(u,g)}}),t&&t(!1),a.add(Z1({createEmptyChangeSummary(){return{zoneIds:[]}},handleChange(h,u){const g=d.get(h.changedObservable);return g!==void 0&&u.zoneIds.push(g),!0}},(h,u)=>{for(const g of l)g.onChange&&(d.set(g.onChange,c.get(g)),g.onChange.read(h));t&&t(!0),o.changeViewZones(g=>{for(const f of u.zoneIds)g.layoutZone(f)}),t&&t(!1)}))})),n.add({dispose(){t&&t(!0),o.changeViewZones(r=>{for(const a of s)r.removeZone(a)}),i==null||i.clear(),t&&t(!1)}}),n}class YCe extends li{dispose(){super.dispose(!0)}}function YH(o,e){const t=dw(e,n=>n.original.startLineNumber<=o.lineNumber);if(!t)return x.fromPositions(o);if(t.original.endLineNumberExclusive<=o.lineNumber){const n=o.lineNumber-t.original.endLineNumberExclusive+t.modified.endLineNumberExclusive;return x.fromPositions(new A(n,o.column))}if(!t.innerChanges)return x.fromPositions(new A(t.modified.startLineNumber,1));const i=dw(t.innerChanges,n=>n.originalRange.getStartPosition().isBeforeOrEqual(o));if(!i){const n=o.lineNumber-t.original.startLineNumber+t.modified.startLineNumber;return x.fromPositions(new A(n,o.column))}if(i.originalRange.containsPosition(o))return i.modifiedRange;{const n=XCe(i.originalRange.getEndPosition(),o);return x.fromPositions(n.addToPosition(i.modifiedRange.getEndPosition()))}}function XCe(o,e){return o.lineNumber===e.lineNumber?new Yr(0,e.column-o.column):new Yr(e.lineNumber-o.lineNumber,e.column-1)}function QCe(o,e){let t;return o.filter(i=>{const n=e(i,t);return t=i,n})}class Cx{static create(e,t=void 0){return new XH(e,e,t)}static createWithDisposable(e,t,i=void 0){const n=new K;return n.add(t),n.add(e),new XH(e,n,i)}}class XH extends Cx{constructor(e,t,i){super(),this.object=e,this._disposable=t,this._debugOwner=i,this._refCount=1,this._isDisposed=!1,this._owners=[],i&&this._addOwner(i)}_addOwner(e){e&&this._owners.push(e)}createNewRef(e){return this._refCount++,e&&this._addOwner(e),new JCe(this,e)}dispose(){this._isDisposed||(this._isDisposed=!0,this._decreaseRefCount(this._debugOwner))}_decreaseRefCount(e){if(this._refCount--,this._refCount===0&&this._disposable.dispose(),e){const t=this._owners.indexOf(e);t!==-1&&this._owners.splice(t,1)}}}class JCe extends Cx{constructor(e,t){super(),this._base=e,this._debugOwner=t,this._isDisposed=!1}get object(){return this._base.object}createNewRef(e){return this._base.createNewRef(e)}dispose(){this._isDisposed||(this._isDisposed=!0,this._base._decreaseRefCount(this._debugOwner))}}var cW=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},dW=function(o,e){return function(t,i){e(t,i,o)}};const eve=qt("diff-review-insert",te.add,m("accessibleDiffViewerInsertIcon","Icon for 'Insert' in accessible diff viewer.")),tve=qt("diff-review-remove",te.remove,m("accessibleDiffViewerRemoveIcon","Icon for 'Remove' in accessible diff viewer.")),ive=qt("diff-review-close",te.close,m("accessibleDiffViewerCloseIcon","Icon for 'Close' in accessible diff viewer."));var g_;let hg=(g_=class extends F{constructor(e,t,i,n,s,r,a,l,c){super(),this._parentNode=e,this._visible=t,this._setVisible=i,this._canClose=n,this._width=s,this._height=r,this._diffs=a,this._models=l,this._instantiationService=c,this._state=su(this,(d,h)=>{const u=this._visible.read(d);if(this._parentNode.style.visibility=u?"visible":"hidden",!u)return null;const g=h.add(this._instantiationService.createInstance(IP,this._diffs,this._models,this._setVisible,this._canClose)),f=h.add(this._instantiationService.createInstance(EP,this._parentNode,g,this._width,this._height,this._models));return{model:g,view:f}}).recomputeInitiallyAndOnChange(this._store)}next(){ai(e=>{const t=this._visible.get();this._setVisible(!0,e),t&&this._state.get().model.nextGroup(e)})}prev(){ai(e=>{this._setVisible(!0,e),this._state.get().model.previousGroup(e)})}close(){ai(e=>{this._setVisible(!1,e)})}},g_._ttPolicy=hd("diffReview",{createHTML:e=>e}),g_);hg=cW([dW(8,we)],hg);let IP=class extends F{constructor(e,t,i,n,s){super(),this._diffs=e,this._models=t,this._setVisible=i,this.canClose=n,this._accessibilitySignalService=s,this._groups=Ke(this,[]),this._currentGroupIdx=Ke(this,0),this._currentElementIdx=Ke(this,0),this.groups=this._groups,this.currentGroup=this._currentGroupIdx.map((r,a)=>this._groups.read(a)[r]),this.currentGroupIndex=this._currentGroupIdx,this.currentElement=this._currentElementIdx.map((r,a)=>{var l;return(l=this.currentGroup.read(a))==null?void 0:l.lines[r]}),this._register(Ae(r=>{const a=this._diffs.read(r);if(!a){this._groups.set([],void 0);return}const l=nve(a,this._models.getOriginalModel().getLineCount(),this._models.getModifiedModel().getLineCount());ai(c=>{const d=this._models.getModifiedPosition();if(d){const h=l.findIndex(u=>(d==null?void 0:d.lineNumber)<u.range.modified.endLineNumberExclusive);h!==-1&&this._currentGroupIdx.set(h,c)}this._groups.set(l,c)})})),this._register(Ae(r=>{const a=this.currentElement.read(r);(a==null?void 0:a.type)===as.Deleted?this._accessibilitySignalService.playSignal(No.diffLineDeleted,{source:"accessibleDiffViewer.currentElementChanged"}):(a==null?void 0:a.type)===as.Added&&this._accessibilitySignalService.playSignal(No.diffLineInserted,{source:"accessibleDiffViewer.currentElementChanged"})})),this._register(Ae(r=>{const a=this.currentElement.read(r);if(a&&a.type!==as.Header){const l=a.modifiedLineNumber??a.diff.modified.startLineNumber;this._models.modifiedSetSelection(x.fromPositions(new A(l,1)))}}))}_goToGroupDelta(e,t){const i=this.groups.get();!i||i.length<=1||Pf(t,n=>{this._currentGroupIdx.set(Ge.ofLength(i.length).clipCyclic(this._currentGroupIdx.get()+e),n),this._currentElementIdx.set(0,n)})}nextGroup(e){this._goToGroupDelta(1,e)}previousGroup(e){this._goToGroupDelta(-1,e)}_goToLineDelta(e){const t=this.currentGroup.get();!t||t.lines.length<=1||ai(i=>{this._currentElementIdx.set(Ge.ofLength(t.lines.length).clip(this._currentElementIdx.get()+e),i)})}goToNextLine(){this._goToLineDelta(1)}goToPreviousLine(){this._goToLineDelta(-1)}goToLine(e){const t=this.currentGroup.get();if(!t)return;const i=t.lines.indexOf(e);i!==-1&&ai(n=>{this._currentElementIdx.set(i,n)})}revealCurrentElementInEditor(){if(!this.canClose.get())return;this._setVisible(!1,void 0);const e=this.currentElement.get();e&&(e.type===as.Deleted?this._models.originalReveal(x.fromPositions(new A(e.originalLineNumber,1))):this._models.modifiedReveal(e.type!==as.Header?x.fromPositions(new A(e.modifiedLineNumber,1)):void 0))}close(){this.canClose.get()&&(this._setVisible(!1,void 0),this._models.modifiedFocus())}};IP=cW([dW(4,ru)],IP);const hC=3;function nve(o,e,t){const i=[];for(const n of P4(o,(s,r)=>r.modified.startLineNumber-s.modified.endLineNumberExclusive<2*hC)){const s=[];s.push(new ove);const r=new Be(Math.max(1,n[0].original.startLineNumber-hC),Math.min(n[n.length-1].original.endLineNumberExclusive+hC,e+1)),a=new Be(Math.max(1,n[0].modified.startLineNumber-hC),Math.min(n[n.length-1].modified.endLineNumberExclusive+hC,t+1));bz(n,(d,h)=>{const u=new Be(d?d.original.endLineNumberExclusive:r.startLineNumber,h?h.original.startLineNumber:r.endLineNumberExclusive),g=new Be(d?d.modified.endLineNumberExclusive:a.startLineNumber,h?h.modified.startLineNumber:a.endLineNumberExclusive);u.forEach(f=>{s.push(new lve(f,g.startLineNumber+(f-u.startLineNumber)))}),h&&(h.original.forEach(f=>{s.push(new rve(h,f))}),h.modified.forEach(f=>{s.push(new ave(h,f))}))});const l=n[0].modified.join(n[n.length-1].modified),c=n[0].original.join(n[n.length-1].original);i.push(new sve(new Yn(l,c),s))}return i}var as;(function(o){o[o.Header=0]="Header",o[o.Unchanged=1]="Unchanged",o[o.Deleted=2]="Deleted",o[o.Added=3]="Added"})(as||(as={}));class sve{constructor(e,t){this.range=e,this.lines=t}}class ove{constructor(){this.type=as.Header}}class rve{constructor(e,t){this.diff=e,this.originalLineNumber=t,this.type=as.Deleted,this.modifiedLineNumber=void 0}}class ave{constructor(e,t){this.diff=e,this.modifiedLineNumber=t,this.type=as.Added,this.originalLineNumber=void 0}}class lve{constructor(e,t){this.originalLineNumber=e,this.modifiedLineNumber=t,this.type=as.Unchanged}}let EP=class extends F{constructor(e,t,i,n,s,r){super(),this._element=e,this._model=t,this._width=i,this._height=n,this._models=s,this._languageService=r,this.domNode=this._element,this.domNode.className="monaco-component diff-review monaco-editor-background";const a=document.createElement("div");a.className="diff-review-actions",this._actionBar=this._register(new zo(a)),this._register(Ae(l=>{this._actionBar.clear(),this._model.canClose.read(l)&&this._actionBar.push(new ro("diffreview.close",m("label.close","Close"),"close-diff-review "+Ie.asClassName(ive),!0,async()=>t.close()),{label:!1,icon:!0})})),this._content=document.createElement("div"),this._content.className="diff-review-content",this._content.setAttribute("role","code"),this._scrollbar=this._register(new z1(this._content,{})),_n(this.domNode,this._scrollbar.getDomNode(),a),this._register(Ae(l=>{this._height.read(l),this._width.read(l),this._scrollbar.scanDomNode()})),this._register(ve(()=>{_n(this.domNode)})),this._register(sd(this.domNode,{width:this._width,height:this._height})),this._register(sd(this._content,{width:this._width,height:this._height})),this._register(Ps((l,c)=>{this._model.currentGroup.read(l),this._render(c)})),this._register(hi(this.domNode,"keydown",l=>{(l.equals(18)||l.equals(2066)||l.equals(530))&&(l.preventDefault(),this._model.goToNextLine()),(l.equals(16)||l.equals(2064)||l.equals(528))&&(l.preventDefault(),this._model.goToPreviousLine()),(l.equals(9)||l.equals(2057)||l.equals(521)||l.equals(1033))&&(l.preventDefault(),this._model.close()),(l.equals(10)||l.equals(3))&&(l.preventDefault(),this._model.revealCurrentElementInEditor())}))}_render(e){const t=this._models.getOriginalOptions(),i=this._models.getModifiedOptions(),n=document.createElement("div");n.className="diff-review-table",n.setAttribute("role","list"),n.setAttribute("aria-label",m("ariaLabel","Accessible Diff Viewer. Use arrow up and down to navigate.")),bn(n,i.get(50)),_n(this._content,n);const s=this._models.getOriginalModel(),r=this._models.getModifiedModel();if(!s||!r)return;const a=s.getOptions(),l=r.getOptions(),c=i.get(67),d=this._model.currentGroup.get();for(const h of(d==null?void 0:d.lines)||[]){if(!d)break;let u;if(h.type===as.Header){const f=document.createElement("div");f.className="diff-review-row",f.setAttribute("role","listitem");const p=d.range,_=this._model.currentGroupIndex.get(),b=this._model.groups.get().length,C=L=>L===0?m("no_lines_changed","no lines changed"):L===1?m("one_line_changed","1 line changed"):m("more_lines_changed","{0} lines changed",L),v=C(p.original.length),w=C(p.modified.length);f.setAttribute("aria-label",m({key:"header",comment:["This is the ARIA label for a git diff header.","A git diff header looks like this: @@ -154,12 +159,39 @@.","That encodes that at original line 154 (which is now line 159), 12 lines were removed/changed with 39 lines.","Variables 0 and 1 refer to the diff index out of total number of diffs.","Variables 2 and 4 will be numbers (a line number).",'Variables 3 and 5 will be "no lines changed", "1 line changed" or "X lines changed", localized separately.']},"Difference {0} of {1}: original line {2}, {3}, modified line {4}, {5}",_+1,b,p.original.startLineNumber,v,p.modified.startLineNumber,w));const S=document.createElement("div");S.className="diff-review-cell diff-review-summary",S.appendChild(document.createTextNode(`${_+1}/${b}: @@ -${p.original.startLineNumber},${p.original.length} +${p.modified.startLineNumber},${p.modified.length} @@`)),f.appendChild(S),u=f}else u=this._createRow(h,c,this._width.get(),t,s,a,i,r,l);n.appendChild(u);const g=be(f=>this._model.currentElement.read(f)===h);e.add(Ae(f=>{const p=g.read(f);u.tabIndex=p?0:-1,p&&u.focus()})),e.add(U(u,"focus",()=>{this._model.goToLine(h)}))}this._scrollbar.scanDomNode()}_createRow(e,t,i,n,s,r,a,l,c){const d=n.get(146),h=d.glyphMarginWidth+d.lineNumbersWidth,u=a.get(146),g=10+u.glyphMarginWidth+u.lineNumbersWidth;let f="diff-review-row",p="";const _="diff-review-spacer";let b=null;switch(e.type){case as.Added:f="diff-review-row line-insert",p=" char-insert",b=eve;break;case as.Deleted:f="diff-review-row line-delete",p=" char-delete",b=tve;break}const C=document.createElement("div");C.style.minWidth=i+"px",C.className=f,C.setAttribute("role","listitem"),C.ariaLevel="";const v=document.createElement("div");v.className="diff-review-cell",v.style.height=`${t}px`,C.appendChild(v);const w=document.createElement("span");w.style.width=h+"px",w.style.minWidth=h+"px",w.className="diff-review-line-number"+p,e.originalLineNumber!==void 0?w.appendChild(document.createTextNode(String(e.originalLineNumber))):w.innerText=" ",v.appendChild(w);const S=document.createElement("span");S.style.width=g+"px",S.style.minWidth=g+"px",S.style.paddingRight="10px",S.className="diff-review-line-number"+p,e.modifiedLineNumber!==void 0?S.appendChild(document.createTextNode(String(e.modifiedLineNumber))):S.innerText=" ",v.appendChild(S);const L=document.createElement("span");if(L.className=_,b){const N=document.createElement("span");N.className=Ie.asClassName(b),N.innerText="  ",L.appendChild(N)}else L.innerText="  ";v.appendChild(L);let k;if(e.modifiedLineNumber!==void 0){let N=this._getLineHtml(l,a,c.tabSize,e.modifiedLineNumber,this._languageService.languageIdCodec);hg._ttPolicy&&(N=hg._ttPolicy.createHTML(N)),v.insertAdjacentHTML("beforeend",N),k=l.getLineContent(e.modifiedLineNumber)}else{let N=this._getLineHtml(s,n,r.tabSize,e.originalLineNumber,this._languageService.languageIdCodec);hg._ttPolicy&&(N=hg._ttPolicy.createHTML(N)),v.insertAdjacentHTML("beforeend",N),k=s.getLineContent(e.originalLineNumber)}k.length===0&&(k=m("blankLine","blank"));let D="";switch(e.type){case as.Unchanged:e.originalLineNumber===e.modifiedLineNumber?D=m({key:"unchangedLine",comment:["The placeholders are contents of the line and should not be translated."]},"{0} unchanged line {1}",k,e.originalLineNumber):D=m("equalLine","{0} original line {1} modified line {2}",k,e.originalLineNumber,e.modifiedLineNumber);break;case as.Added:D=m("insertLine","+ {0} modified line {1}",k,e.modifiedLineNumber);break;case as.Deleted:D=m("deleteLine","- {0} original line {1}",k,e.originalLineNumber);break}return C.setAttribute("aria-label",D),C}_getLineHtml(e,t,i,n,s){const r=e.getLineContent(n),a=t.get(50),l=cn.createEmpty(r,s),c=Ko.isBasicASCII(r,e.mightContainNonBasicASCII()),d=Ko.containsRTL(r,c,e.mightContainRTL());return mE(new au(a.isMonospace&&!t.get(33),a.canUseHalfwidthRightwardsArrow,r,!1,c,d,0,l,[],i,0,a.spaceWidth,a.middotWidth,a.wsmiddotWidth,t.get(118),t.get(100),t.get(95),t.get(51)!==Mh.OFF,null)).html}};EP=cW([dW(5,ni)],EP);class cve{constructor(e){this.editors=e}getOriginalModel(){return this.editors.original.getModel()}getOriginalOptions(){return this.editors.original.getOptions()}originalReveal(e){this.editors.original.revealRange(e),this.editors.original.setSelection(e),this.editors.original.focus()}getModifiedModel(){return this.editors.modified.getModel()}getModifiedOptions(){return this.editors.modified.getOptions()}modifiedReveal(e){e&&(this.editors.modified.revealRange(e),this.editors.modified.setSelection(e)),this.editors.modified.focus()}modifiedSetSelection(e){this.editors.modified.setSelection(e)}modifiedFocus(){this.editors.modified.focus()}getModifiedPosition(){return this.editors.modified.getPosition()??void 0}}E("diffEditor.move.border","#8b8b8b9c",m("diffEditor.move.border","The border color for text that got moved in the diff editor."));E("diffEditor.moveActive.border","#FFA500",m("diffEditor.moveActive.border","The active border color for text that got moved in the diff editor."));E("diffEditor.unchangedRegionShadow",{dark:"#000000",light:"#737373BF",hcDark:"#000000",hcLight:"#737373BF"},m("diffEditor.unchangedRegionShadow","The color of the shadow around unchanged region widgets."));const dve=qt("diff-insert",te.add,m("diffInsertIcon","Line decoration for inserts in the diff editor.")),Oq=qt("diff-remove",te.remove,m("diffRemoveIcon","Line decoration for removals in the diff editor.")),vx=We.register({className:"line-insert",description:"line-insert",isWholeLine:!0,linesDecorationsClassName:"insert-sign "+Ie.asClassName(dve),marginClassName:"gutter-insert"}),Yw=We.register({className:"line-delete",description:"line-delete",isWholeLine:!0,linesDecorationsClassName:"delete-sign "+Ie.asClassName(Oq),marginClassName:"gutter-delete"}),QH=We.register({className:"line-insert",description:"line-insert",isWholeLine:!0,marginClassName:"gutter-insert"}),JH=We.register({className:"line-delete",description:"line-delete",isWholeLine:!0,marginClassName:"gutter-delete"}),wx=We.register({className:"char-insert",description:"char-insert",shouldFillLineOnLineBreak:!0}),hW=We.register({className:"char-insert",description:"char-insert",isWholeLine:!0}),uW=We.register({className:"char-insert diff-range-empty",description:"char-insert diff-range-empty"}),G_=We.register({className:"char-delete",description:"char-delete",shouldFillLineOnLineBreak:!0}),gW=We.register({className:"char-delete",description:"char-delete",isWholeLine:!0}),fW=We.register({className:"char-delete diff-range-empty",description:"char-delete diff-range-empty"});var Fq=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},NP=function(o,e){return function(t,i){e(t,i,o)}},Ou;const oS=et("diffProviderFactoryService");let TP=class{constructor(e){this.instantiationService=e}createDiffProvider(e){return this.instantiationService.createInstance(RP,e)}};TP=Fq([NP(0,we)],TP);st(oS,TP,1);var Ag;let RP=(Ag=class{constructor(e,t,i){this.editorWorkerService=t,this.telemetryService=i,this.onDidChangeEventEmitter=new P,this.onDidChange=this.onDidChangeEventEmitter.event,this.diffAlgorithm="advanced",this.diffAlgorithmOnDidChangeSubscription=void 0,this.setOptions(e)}dispose(){var e;(e=this.diffAlgorithmOnDidChangeSubscription)==null||e.dispose()}async computeDiff(e,t,i,n){if(typeof this.diffAlgorithm!="string")return this.diffAlgorithm.computeDiff(e,t,i,n);if(e.isDisposed()||t.isDisposed())return{changes:[],identical:!0,quitEarly:!1,moves:[]};if(e.getLineCount()===1&&e.getLineMaxColumn(1)===1)return t.getLineCount()===1&&t.getLineMaxColumn(1)===1?{changes:[],identical:!0,quitEarly:!1,moves:[]}:{changes:[new Vo(new Be(1,2),new Be(1,t.getLineCount()+1),[new Eo(e.getFullModelRange(),t.getFullModelRange())])],identical:!1,quitEarly:!1,moves:[]};const s=JSON.stringify([e.uri.toString(),t.uri.toString()]),r=JSON.stringify([e.id,t.id,e.getAlternativeVersionId(),t.getAlternativeVersionId(),JSON.stringify(i)]),a=Ou.diffCache.get(s);if(a&&a.context===r)return a.result;const l=ln.create(),c=await this.editorWorkerService.computeDiff(e.uri,t.uri,i,this.diffAlgorithm),d=l.elapsed();if(this.telemetryService.publicLog2("diffEditor.computeDiff",{timeMs:d,timedOut:(c==null?void 0:c.quitEarly)??!0,detectedMoves:i.computeMoves?(c==null?void 0:c.moves.length)??0:-1}),n.isCancellationRequested)return{changes:[],identical:!1,quitEarly:!0,moves:[]};if(!c)throw new Error("no diff result available");return Ou.diffCache.size>10&&Ou.diffCache.delete(Ou.diffCache.keys().next().value),Ou.diffCache.set(s,{result:c,context:r}),c}setOptions(e){var i;let t=!1;e.diffAlgorithm&&this.diffAlgorithm!==e.diffAlgorithm&&((i=this.diffAlgorithmOnDidChangeSubscription)==null||i.dispose(),this.diffAlgorithmOnDidChangeSubscription=void 0,this.diffAlgorithm=e.diffAlgorithm,typeof e.diffAlgorithm!="string"&&(this.diffAlgorithmOnDidChangeSubscription=e.diffAlgorithm.onDidChange(()=>this.onDidChangeEventEmitter.fire())),t=!0),t&&this.onDidChangeEventEmitter.fire()}},Ou=Ag,Ag.diffCache=new Map,Ag);RP=Ou=Fq([NP(1,Us),NP(2,Bn)],RP);function SE(){return vR&&!!vR.VSCODE_DEV}function Wq(o){if(SE()){const e=hve();return e.add(o),{dispose(){e.delete(o)}}}else return{dispose(){}}}function hve(){u0||(u0=new Set);const o=globalThis;return o.$hotReload_applyNewExports||(o.$hotReload_applyNewExports=e=>{const t={config:{mode:void 0},...e},i=[];for(const n of u0){const s=n(t);s&&i.push(s)}if(i.length>0)return n=>{let s=!1;for(const r of i)r(n)&&(s=!0);return s}}),u0}let u0;SE()&&Wq(({oldExports:o,newSrc:e,config:t})=>{if(t.mode==="patch-prototype")return i=>{var n,s;for(const r in i){const a=i[r];if(console.log(`[hot-reload] Patching prototype methods of '${r}'`,{exportedItem:a}),typeof a=="function"&&a.prototype){const l=o[r];if(l){for(const c of Object.getOwnPropertyNames(a.prototype)){const d=Object.getOwnPropertyDescriptor(a.prototype,c),h=Object.getOwnPropertyDescriptor(l.prototype,c);((n=d==null?void 0:d.value)==null?void 0:n.toString())!==((s=h==null?void 0:h.value)==null?void 0:s.toString())&&console.log(`[hot-reload] Patching prototype method '${r}.${c}'`),Object.defineProperty(l.prototype,c,d)}i[r]=l}}}return!0}});function Js(o,e){return uve([o],e),o}function uve(o,e){SE()&&xn("reload",i=>Wq(({oldExports:n})=>{if([...Object.values(n)].some(s=>o.includes(s)))return s=>(i(void 0),!0)})).read(e)}var gve=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},fve=function(o,e){return function(t,i){e(t,i,o)}};let MP=class extends F{setActiveMovedText(e){this._activeMovedText.set(e,void 0)}constructor(e,t,i){super(),this.model=e,this._options=t,this._diffProviderFactoryService=i,this._isDiffUpToDate=Ke(this,!1),this.isDiffUpToDate=this._isDiffUpToDate,this._diff=Ke(this,void 0),this.diff=this._diff,this._unchangedRegions=Ke(this,void 0),this.unchangedRegions=be(this,a=>{var l;return this._options.hideUnchangedRegions.read(a)?((l=this._unchangedRegions.read(a))==null?void 0:l.regions)??[]:(ai(c=>{var d;for(const h of((d=this._unchangedRegions.get())==null?void 0:d.regions)||[])h.collapseAll(c)}),[])}),this.movedTextToCompare=Ke(this,void 0),this._activeMovedText=Ke(this,void 0),this._hoveredMovedText=Ke(this,void 0),this.activeMovedText=be(this,a=>this.movedTextToCompare.read(a)??this._hoveredMovedText.read(a)??this._activeMovedText.read(a)),this._cancellationTokenSource=new li,this._diffProvider=be(this,a=>{const l=this._diffProviderFactoryService.createDiffProvider({diffAlgorithm:this._options.diffAlgorithm.read(a)}),c=xn("onDidChange",l.onDidChange);return{diffProvider:l,onChangeSignal:c}}),this._register(ve(()=>this._cancellationTokenSource.cancel()));const n=Lb("contentChangedSignal"),s=this._register(new It(()=>n.trigger(void 0),200));this._register(Ae(a=>{const l=this._unchangedRegions.read(a);if(!l||l.regions.some(f=>f.isDragged.read(a)))return;const c=l.originalDecorationIds.map(f=>e.original.getDecorationRange(f)).map(f=>f?Be.fromRangeInclusive(f):void 0),d=l.modifiedDecorationIds.map(f=>e.modified.getDecorationRange(f)).map(f=>f?Be.fromRangeInclusive(f):void 0),h=l.regions.map((f,p)=>!c[p]||!d[p]?void 0:new ih(c[p].startLineNumber,d[p].startLineNumber,c[p].length,f.visibleLineCountTop.read(a),f.visibleLineCountBottom.read(a))).filter(Tl),u=[];let g=!1;for(const f of P4(h,(p,_)=>p.getHiddenModifiedRange(a).endLineNumberExclusive===_.getHiddenModifiedRange(a).startLineNumber))if(f.length>1){g=!0;const p=f.reduce((b,C)=>b+C.lineCount,0),_=new ih(f[0].originalLineNumber,f[0].modifiedLineNumber,p,f[0].visibleLineCountTop.get(),f[f.length-1].visibleLineCountBottom.get());u.push(_)}else u.push(f[0]);if(g){const f=e.original.deltaDecorations(l.originalDecorationIds,u.map(_=>({range:_.originalUnchangedRange.toInclusiveRange(),options:{description:"unchanged"}}))),p=e.modified.deltaDecorations(l.modifiedDecorationIds,u.map(_=>({range:_.modifiedUnchangedRange.toInclusiveRange(),options:{description:"unchanged"}})));ai(_=>{this._unchangedRegions.set({regions:u,originalDecorationIds:f,modifiedDecorationIds:p},_)})}}));const r=(a,l,c)=>{const d=ih.fromDiffs(a.changes,e.original.getLineCount(),e.modified.getLineCount(),this._options.hideUnchangedRegionsMinimumLineCount.read(c),this._options.hideUnchangedRegionsContextLineCount.read(c));let h;const u=this._unchangedRegions.get();if(u){const _=u.originalDecorationIds.map(w=>e.original.getDecorationRange(w)).map(w=>w?Be.fromRangeInclusive(w):void 0),b=u.modifiedDecorationIds.map(w=>e.modified.getDecorationRange(w)).map(w=>w?Be.fromRangeInclusive(w):void 0);let v=QCe(u.regions.map((w,S)=>{if(!_[S]||!b[S])return;const L=_[S].length;return new ih(_[S].startLineNumber,b[S].startLineNumber,L,Math.min(w.visibleLineCountTop.get(),L),Math.min(w.visibleLineCountBottom.get(),L-w.visibleLineCountTop.get()))}).filter(Tl),(w,S)=>!S||w.modifiedLineNumber>=S.modifiedLineNumber+S.lineCount&&w.originalLineNumber>=S.originalLineNumber+S.lineCount).map(w=>new Yn(w.getHiddenOriginalRange(c),w.getHiddenModifiedRange(c)));v=Yn.clip(v,Be.ofLength(1,e.original.getLineCount()),Be.ofLength(1,e.modified.getLineCount())),h=Yn.inverse(v,e.original.getLineCount(),e.modified.getLineCount())}const g=[];if(h)for(const _ of d){const b=h.filter(C=>C.original.intersectsStrict(_.originalUnchangedRange)&&C.modified.intersectsStrict(_.modifiedUnchangedRange));g.push(..._.setVisibleRanges(b,l))}else g.push(...d);const f=e.original.deltaDecorations((u==null?void 0:u.originalDecorationIds)||[],g.map(_=>({range:_.originalUnchangedRange.toInclusiveRange(),options:{description:"unchanged"}}))),p=e.modified.deltaDecorations((u==null?void 0:u.modifiedDecorationIds)||[],g.map(_=>({range:_.modifiedUnchangedRange.toInclusiveRange(),options:{description:"unchanged"}})));this._unchangedRegions.set({regions:g,originalDecorationIds:f,modifiedDecorationIds:p},l)};this._register(e.modified.onDidChangeContent(a=>{if(this._diff.get()){const c=Pc.fromModelContentChanges(a.changes);this._lastDiff,e.original,e.modified}this._isDiffUpToDate.set(!1,void 0),s.schedule()})),this._register(e.original.onDidChangeContent(a=>{if(this._diff.get()){const c=Pc.fromModelContentChanges(a.changes);this._lastDiff,e.original,e.modified}this._isDiffUpToDate.set(!1,void 0),s.schedule()})),this._register(Ps(async(a,l)=>{this._options.hideUnchangedRegionsMinimumLineCount.read(a),this._options.hideUnchangedRegionsContextLineCount.read(a),s.cancel(),n.read(a);const c=this._diffProvider.read(a);c.onChangeSignal.read(a),Js(l$,a),Js(qM,a),this._isDiffUpToDate.set(!1,void 0);let d=[];l.add(e.original.onDidChangeContent(g=>{const f=Pc.fromModelContentChanges(g.changes);d=ML(d,f)}));let h=[];l.add(e.modified.onDidChangeContent(g=>{const f=Pc.fromModelContentChanges(g.changes);h=ML(h,f)}));let u=await c.diffProvider.computeDiff(e.original,e.modified,{ignoreTrimWhitespace:this._options.ignoreTrimWhitespace.read(a),maxComputationTimeMs:this._options.maxComputationTimeMs.read(a),computeMoves:this._options.showMoves.read(a)},this._cancellationTokenSource.token);this._cancellationTokenSource.token.isCancellationRequested||e.original.isDisposed()||e.modified.isDisposed()||(u=mve(u,e.original,e.modified),u=(e.original,e.modified,void 0)??u,u=(e.original,e.modified,void 0)??u,ai(g=>{r(u,g),this._lastDiff=u;const f=mW.fromDiffResult(u);this._diff.set(f,g),this._isDiffUpToDate.set(!0,g);const p=this.movedTextToCompare.get();this.movedTextToCompare.set(p?this._lastDiff.moves.find(_=>_.lineRangeMapping.modified.intersect(p.lineRangeMapping.modified)):void 0,g)}))}))}ensureModifiedLineIsVisible(e,t,i){var s,r;if(((s=this.diff.get())==null?void 0:s.mappings.length)===0)return;const n=((r=this._unchangedRegions.get())==null?void 0:r.regions)||[];for(const a of n)if(a.getHiddenModifiedRange(void 0).contains(e)){a.showModifiedLine(e,t,i);return}}ensureOriginalLineIsVisible(e,t,i){var s,r;if(((s=this.diff.get())==null?void 0:s.mappings.length)===0)return;const n=((r=this._unchangedRegions.get())==null?void 0:r.regions)||[];for(const a of n)if(a.getHiddenOriginalRange(void 0).contains(e)){a.showOriginalLine(e,t,i);return}}async waitForDiff(){await kj(this.isDiffUpToDate,e=>e)}serializeState(){const e=this._unchangedRegions.get();return{collapsedRegions:e==null?void 0:e.regions.map(t=>({range:t.getHiddenModifiedRange(void 0).serialize()}))}}restoreSerializedState(e){var n;const t=(n=e.collapsedRegions)==null?void 0:n.map(s=>Be.deserialize(s.range)),i=this._unchangedRegions.get();!i||!t||ai(s=>{for(const r of i.regions)for(const a of t)if(r.modifiedUnchangedRange.intersect(a)){r.setHiddenModifiedRange(a,s);break}})}};MP=gve([fve(2,oS)],MP);function mve(o,e,t){return{changes:o.changes.map(i=>new Vo(i.original,i.modified,i.innerChanges?i.innerChanges.map(n=>pve(n,e,t)):void 0)),moves:o.moves,identical:o.identical,quitEarly:o.quitEarly}}function pve(o,e,t){let i=o.originalRange,n=o.modifiedRange;return i.startColumn===1&&n.startColumn===1&&(i.endColumn!==1||n.endColumn!==1)&&i.endColumn===e.getLineMaxColumn(i.endLineNumber)&&n.endColumn===t.getLineMaxColumn(n.endLineNumber)&&i.endLineNumber<e.getLineCount()&&n.endLineNumber<t.getLineCount()&&(i=i.setEndPosition(i.endLineNumber+1,1),n=n.setEndPosition(n.endLineNumber+1,1)),new Eo(i,n)}class mW{static fromDiffResult(e){return new mW(e.changes.map(t=>new Bq(t)),e.moves||[],e.identical,e.quitEarly)}constructor(e,t,i,n){this.mappings=e,this.movedTexts=t,this.identical=i,this.quitEarly=n}}class Bq{constructor(e){this.lineRangeMapping=e}}class ih{static fromDiffs(e,t,i,n,s){const r=Vo.inverse(e,t,i),a=[];for(const l of r){let c=l.original.startLineNumber,d=l.modified.startLineNumber,h=l.original.length;const u=c===1&&d===1,g=c+h===t+1&&d+h===i+1;(u||g)&&h>=s+n?(u&&!g&&(h-=s),g&&!u&&(c+=s,d+=s,h-=s),a.push(new ih(c,d,h,0,0))):h>=s*2+n&&(c+=s,d+=s,h-=s*2,a.push(new ih(c,d,h,0,0)))}return a}get originalUnchangedRange(){return Be.ofLength(this.originalLineNumber,this.lineCount)}get modifiedUnchangedRange(){return Be.ofLength(this.modifiedLineNumber,this.lineCount)}constructor(e,t,i,n,s){this.originalLineNumber=e,this.modifiedLineNumber=t,this.lineCount=i,this._visibleLineCountTop=Ke(this,0),this.visibleLineCountTop=this._visibleLineCountTop,this._visibleLineCountBottom=Ke(this,0),this.visibleLineCountBottom=this._visibleLineCountBottom,this._shouldHideControls=be(this,l=>this.visibleLineCountTop.read(l)+this.visibleLineCountBottom.read(l)===this.lineCount&&!this.isDragged.read(l)),this.isDragged=Ke(this,void 0);const r=Math.max(Math.min(n,this.lineCount),0),a=Math.max(Math.min(s,this.lineCount-n),0);m7(n===r),m7(s===a),this._visibleLineCountTop.set(r,void 0),this._visibleLineCountBottom.set(a,void 0)}setVisibleRanges(e,t){const i=[],n=new dr(e.map(l=>l.modified)).subtractFrom(this.modifiedUnchangedRange);let s=this.originalLineNumber,r=this.modifiedLineNumber;const a=this.modifiedLineNumber+this.lineCount;if(n.ranges.length===0)this.showAll(t),i.push(this);else{let l=0;for(const c of n.ranges){const d=l===n.ranges.length-1;l++;const h=(d?a:c.endLineNumberExclusive)-r,u=new ih(s,r,h,0,0);u.setHiddenModifiedRange(c,t),i.push(u),s=u.originalUnchangedRange.endLineNumberExclusive,r=u.modifiedUnchangedRange.endLineNumberExclusive}}return i}shouldHideControls(e){return this._shouldHideControls.read(e)}getHiddenOriginalRange(e){return Be.ofLength(this.originalLineNumber+this._visibleLineCountTop.read(e),this.lineCount-this._visibleLineCountTop.read(e)-this._visibleLineCountBottom.read(e))}getHiddenModifiedRange(e){return Be.ofLength(this.modifiedLineNumber+this._visibleLineCountTop.read(e),this.lineCount-this._visibleLineCountTop.read(e)-this._visibleLineCountBottom.read(e))}setHiddenModifiedRange(e,t){const i=e.startLineNumber-this.modifiedLineNumber,n=this.modifiedLineNumber+this.lineCount-e.endLineNumberExclusive;this.setState(i,n,t)}getMaxVisibleLineCountTop(){return this.lineCount-this._visibleLineCountBottom.get()}getMaxVisibleLineCountBottom(){return this.lineCount-this._visibleLineCountTop.get()}showMoreAbove(e=10,t){const i=this.getMaxVisibleLineCountTop();this._visibleLineCountTop.set(Math.min(this._visibleLineCountTop.get()+e,i),t)}showMoreBelow(e=10,t){const i=this.lineCount-this._visibleLineCountTop.get();this._visibleLineCountBottom.set(Math.min(this._visibleLineCountBottom.get()+e,i),t)}showAll(e){this._visibleLineCountBottom.set(this.lineCount-this._visibleLineCountTop.get(),e)}showModifiedLine(e,t,i){const n=e+1-(this.modifiedLineNumber+this._visibleLineCountTop.get()),s=this.modifiedLineNumber-this._visibleLineCountBottom.get()+this.lineCount-e;t===0&&n<s||t===1?this._visibleLineCountTop.set(this._visibleLineCountTop.get()+n,i):this._visibleLineCountBottom.set(this._visibleLineCountBottom.get()+s,i)}showOriginalLine(e,t,i){const n=e-this.originalLineNumber,s=this.originalLineNumber+this.lineCount-e;t===0&&n<s||t===1?this._visibleLineCountTop.set(Math.min(this._visibleLineCountTop.get()+s-n,this.getMaxVisibleLineCountTop()),i):this._visibleLineCountBottom.set(Math.min(this._visibleLineCountBottom.get()+n-s,this.getMaxVisibleLineCountBottom()),i)}collapseAll(e){this._visibleLineCountTop.set(0,e),this._visibleLineCountBottom.set(0,e)}setState(e,t,i){e=Math.max(Math.min(e,this.lineCount),0),t=Math.max(Math.min(t,this.lineCount-e),0),this._visibleLineCountTop.set(e,i),this._visibleLineCountBottom.set(t,i)}}class _ve extends F{get visibility(){return this._visibility}set visibility(e){this._visibility!==e&&(this._visibility=e,this._diffActions.style.visibility=e?"visible":"hidden")}constructor(e,t,i,n,s,r,a,l,c){super(),this._getViewZoneId=e,this._marginDomNode=t,this._modifiedEditor=i,this._diff=n,this._editor=s,this._viewLineCounts=r,this._originalTextModel=a,this._contextMenuService=l,this._clipboardService=c,this._visibility=!1,this._marginDomNode.style.zIndex="10",this._diffActions=document.createElement("div"),this._diffActions.className=Ie.asClassName(te.lightBulb)+" lightbulb-glyph",this._diffActions.style.position="absolute";const d=this._modifiedEditor.getOption(67);this._diffActions.style.right="0px",this._diffActions.style.visibility="hidden",this._diffActions.style.height=`${d}px`,this._diffActions.style.lineHeight=`${d}px`,this._marginDomNode.appendChild(this._diffActions);let h=0;const u=i.getOption(128)&&!Pa,g=(f,p)=>{this._contextMenuService.showContextMenu({domForShadowRoot:u?i.getDomNode()??void 0:void 0,getAnchor:()=>({x:f,y:p}),getActions:()=>{const _=[],b=n.modified.isEmpty;return _.push(new ro("diff.clipboard.copyDeletedContent",b?n.original.length>1?m("diff.clipboard.copyDeletedLinesContent.label","Copy deleted lines"):m("diff.clipboard.copyDeletedLinesContent.single.label","Copy deleted line"):n.original.length>1?m("diff.clipboard.copyChangedLinesContent.label","Copy changed lines"):m("diff.clipboard.copyChangedLinesContent.single.label","Copy changed line"),void 0,!0,async()=>{const v=this._originalTextModel.getValueInRange(n.original.toExclusiveRange());await this._clipboardService.writeText(v)})),n.original.length>1&&_.push(new ro("diff.clipboard.copyDeletedLineContent",b?m("diff.clipboard.copyDeletedLineContent.label","Copy deleted line ({0})",n.original.startLineNumber+h):m("diff.clipboard.copyChangedLineContent.label","Copy changed line ({0})",n.original.startLineNumber+h),void 0,!0,async()=>{let v=this._originalTextModel.getLineContent(n.original.startLineNumber+h);v===""&&(v=this._originalTextModel.getEndOfLineSequence()===0?`
`:`\r
`),await this._clipboardService.writeText(v)})),i.getOption(92)||_.push(new ro("diff.inline.revertChange",m("diff.inline.revertChange.label","Revert this change"),void 0,!0,async()=>{this._editor.revert(this._diff)})),_},autoSelectFirstItem:!0})};this._register(hi(this._diffActions,"mousedown",f=>{if(!f.leftButton)return;const{top:p,height:_}=wi(this._diffActions),b=Math.floor(d/3);f.preventDefault(),g(f.posx,p+_+b)})),this._register(i.onMouseMove(f=>{(f.target.type===8||f.target.type===5)&&f.target.detail.viewZoneId===this._getViewZoneId()?(h=this._updateLightBulbPosition(this._marginDomNode,f.event.browserEvent.y,d),this.visibility=!0):this.visibility=!1})),this._register(i.onMouseDown(f=>{f.event.leftButton&&(f.target.type===8||f.target.type===5)&&f.target.detail.viewZoneId===this._getViewZoneId()&&(f.event.preventDefault(),h=this._updateLightBulbPosition(this._marginDomNode,f.event.browserEvent.y,d),g(f.event.posx,f.event.posy+d))}))}_updateLightBulbPosition(e,t,i){const{top:n}=wi(e),s=t-n,r=Math.floor(s/i),a=r*i;if(this._diffActions.style.top=`${a}px`,this._viewLineCounts){let l=0;for(let c=0;c<this._viewLineCounts.length;c++)if(l+=this._viewLineCounts[c],r<l)return c}return r}}const eV=hd("diffEditorWidget",{createHTML:o=>o});function bve(o,e,t,i){bn(i,e.fontInfo);const n=t.length>0,s=new _b(1e4);let r=0,a=0;const l=[];for(let u=0;u<o.lineTokens.length;u++){const g=u+1,f=o.lineTokens[u],p=o.lineBreakData[u],_=hs.filter(t,g,1,Number.MAX_SAFE_INTEGER);if(p){let b=0;for(const C of p.breakOffsets){const v=f.sliceAndInflate(b,C,0);r=Math.max(r,tV(a,v,hs.extractWrapped(_,b,C),n,o.mightContainNonBasicASCII,o.mightContainRTL,e,s)),a++,b=C}l.push(p.breakOffsets.length)}else l.push(1),r=Math.max(r,tV(a,f,_,n,o.mightContainNonBasicASCII,o.mightContainRTL,e,s)),a++}r+=e.scrollBeyondLastColumn;const c=s.build(),d=eV?eV.createHTML(c):c;i.innerHTML=d;const h=r*e.typicalHalfwidthCharacterWidth;return{heightInLines:a,minWidthInPx:h,viewLineCounts:l}}class Cve{constructor(e,t,i,n){this.lineTokens=e,this.lineBreakData=t,this.mightContainNonBasicASCII=i,this.mightContainRTL=n}}class pW{static fromEditor(e){var s;const t=e.getOptions(),i=t.get(50),n=t.get(146);return new pW(((s=e.getModel())==null?void 0:s.getOptions().tabSize)||0,i,t.get(33),i.typicalHalfwidthCharacterWidth,t.get(105),t.get(67),n.decorationsWidth,t.get(118),t.get(100),t.get(95),t.get(51))}constructor(e,t,i,n,s,r,a,l,c,d,h){this.tabSize=e,this.fontInfo=t,this.disableMonospaceOptimizations=i,this.typicalHalfwidthCharacterWidth=n,this.scrollBeyondLastColumn=s,this.lineHeight=r,this.lineDecorationsWidth=a,this.stopRenderingLineAfter=l,this.renderWhitespace=c,this.renderControlCharacters=d,this.fontLigatures=h}}function tV(o,e,t,i,n,s,r,a){a.appendString('<div class="view-line'),i||a.appendString(" char-delete"),a.appendString('" style="top:'),a.appendString(String(o*r.lineHeight)),a.appendString('px;width:1000000px;">');const l=e.getLineContent(),c=Ko.isBasicASCII(l,n),d=Ko.containsRTL(l,c,s),h=nS(new au(r.fontInfo.isMonospace&&!r.disableMonospaceOptimizations,r.fontInfo.canUseHalfwidthRightwardsArrow,l,!1,c,d,0,e,t,r.tabSize,0,r.fontInfo.spaceWidth,r.fontInfo.middotWidth,r.fontInfo.wsmiddotWidth,r.stopRenderingLineAfter,r.renderWhitespace,r.renderControlCharacters,r.fontLigatures!==Mh.OFF,null),a);return a.appendString("</div>"),h.characterMapping.getHorizontalOffset(h.characterMapping.length)}var vve=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},iV=function(o,e){return function(t,i){e(t,i,o)}};let AP=class extends F{constructor(e,t,i,n,s,r,a,l,c,d){super(),this._targetWindow=e,this._editors=t,this._diffModel=i,this._options=n,this._diffEditorWidget=s,this._canIgnoreViewZoneUpdateEvent=r,this._origViewZonesToIgnore=a,this._modViewZonesToIgnore=l,this._clipboardService=c,this._contextMenuService=d,this._originalTopPadding=Ke(this,0),this._originalScrollOffset=Ke(this,0),this._originalScrollOffsetAnimated=ZH(this._targetWindow,this._originalScrollOffset,this._store),this._modifiedTopPadding=Ke(this,0),this._modifiedScrollOffset=Ke(this,0),this._modifiedScrollOffsetAnimated=ZH(this._targetWindow,this._modifiedScrollOffset,this._store);const h=Ke("invalidateAlignmentsState",0),u=this._register(new It(()=>{h.set(h.get()+1,void 0)},0));this._register(this._editors.original.onDidChangeViewZones(v=>{this._canIgnoreViewZoneUpdateEvent()||u.schedule()})),this._register(this._editors.modified.onDidChangeViewZones(v=>{this._canIgnoreViewZoneUpdateEvent()||u.schedule()})),this._register(this._editors.original.onDidChangeConfiguration(v=>{(v.hasChanged(147)||v.hasChanged(67))&&u.schedule()})),this._register(this._editors.modified.onDidChangeConfiguration(v=>{(v.hasChanged(147)||v.hasChanged(67))&&u.schedule()}));const g=this._diffModel.map(v=>v?xt(this,v.model.original.onDidChangeTokens,()=>v.model.original.tokenization.backgroundTokenizationState===2):void 0).map((v,w)=>v==null?void 0:v.read(w)),f=be(v=>{const w=this._diffModel.read(v),S=w==null?void 0:w.diff.read(v);if(!w||!S)return null;h.read(v);const k=this._options.renderSideBySide.read(v);return nV(this._editors.original,this._editors.modified,S.mappings,this._origViewZonesToIgnore,this._modViewZonesToIgnore,k)}),p=be(v=>{var L;const w=(L=this._diffModel.read(v))==null?void 0:L.movedTextToCompare.read(v);if(!w)return null;h.read(v);const S=w.changes.map(k=>new Bq(k));return nV(this._editors.original,this._editors.modified,S,this._origViewZonesToIgnore,this._modViewZonesToIgnore,!0)});function _(){const v=document.createElement("div");return v.className="diagonal-fill",v}const b=this._register(new K);this.viewZones=su(this,(v,w)=>{var ue,Pe,_e,it;b.clear();const S=f.read(v)||[],L=[],k=[],D=this._modifiedTopPadding.read(v);D>0&&k.push({afterLineNumber:0,domNode:document.createElement("div"),heightInPx:D,showInHiddenAreas:!0,suppressMouseDown:!0});const N=this._originalTopPadding.read(v);N>0&&L.push({afterLineNumber:0,domNode:document.createElement("div"),heightInPx:N,showInHiddenAreas:!0,suppressMouseDown:!0});const B=this._options.renderSideBySide.read(v),W=B||(ue=this._editors.modified._getViewModel())==null?void 0:ue.createLineBreaksComputer();if(W){const Ne=this._editors.original.getModel();for(const Ct of S)if(Ct.diff)for(let ht=Ct.originalRange.startLineNumber;ht<Ct.originalRange.endLineNumberExclusive;ht++){if(ht>Ne.getLineCount())return{orig:L,mod:k};W==null||W.addRequest(Ne.getLineContent(ht),null,null)}}const V=(W==null?void 0:W.finalize())??[];let $=0;const H=this._editors.modified.getOption(67),j=(Pe=this._diffModel.read(v))==null?void 0:Pe.movedTextToCompare.read(v),J=((_e=this._editors.original.getModel())==null?void 0:_e.mightContainNonBasicASCII())??!1,he=((it=this._editors.original.getModel())==null?void 0:it.mightContainRTL())??!1,fe=pW.fromEditor(this._editors.modified);for(const Ne of S)if(Ne.diff&&!B&&(!this._options.useTrueInlineDiffRendering.read(v)||!_W(Ne.diff))){if(!Ne.originalRange.isEmpty){g.read(v);const ht=document.createElement("div");ht.classList.add("view-lines","line-delete","monaco-mouse-cursor-text");const Ti=this._editors.original.getModel();if(Ne.originalRange.endLineNumberExclusive-1>Ti.getLineCount())return{orig:L,mod:k};const ts=new Cve(Ne.originalRange.mapToLineArray(zi=>Ti.tokenization.getLineTokens(zi)),Ne.originalRange.mapToLineArray(zi=>V[$++]),J,he),xs=[];for(const zi of Ne.diff.innerChanges||[])xs.push(new cv(zi.originalRange.delta(-(Ne.diff.original.startLineNumber-1)),G_.className,0));const Dr=bve(ts,fe,xs,ht),Co=document.createElement("div");if(Co.className="inline-deleted-margin-view-zone",bn(Co,fe.fontInfo),this._options.renderIndicators.read(v))for(let zi=0;zi<Dr.heightInLines;zi++){const Ir=document.createElement("div");Ir.className=`delete-sign ${Ie.asClassName(Oq)}`,Ir.setAttribute("style",`position:absolute;top:${zi*H}px;width:${fe.lineDecorationsWidth}px;height:${H}px;right:0;`),Co.appendChild(Ir)}let hu;b.add(new _ve(()=>Kd(hu),Co,this._editors.modified,Ne.diff,this._diffEditorWidget,Dr.viewLineCounts,this._editors.original.getModel(),this._contextMenuService,this._clipboardService));for(let zi=0;zi<Dr.viewLineCounts.length;zi++){const Ir=Dr.viewLineCounts[zi];Ir>1&&L.push({afterLineNumber:Ne.originalRange.startLineNumber+zi,domNode:_(),heightInPx:(Ir-1)*H,showInHiddenAreas:!0,suppressMouseDown:!0})}k.push({afterLineNumber:Ne.modifiedRange.startLineNumber-1,domNode:ht,heightInPx:Dr.heightInLines*H,minWidthInPx:Dr.minWidthInPx,marginDomNode:Co,setZoneId(zi){hu=zi},showInHiddenAreas:!0,suppressMouseDown:!0})}const Ct=document.createElement("div");Ct.className="gutter-delete",L.push({afterLineNumber:Ne.originalRange.endLineNumberExclusive-1,domNode:_(),heightInPx:Ne.modifiedHeightInPx,marginDomNode:Ct,showInHiddenAreas:!0,suppressMouseDown:!0})}else{const Ct=Ne.modifiedHeightInPx-Ne.originalHeightInPx;if(Ct>0){if(j!=null&&j.lineRangeMapping.original.delta(-1).deltaLength(2).contains(Ne.originalRange.endLineNumberExclusive-1))continue;L.push({afterLineNumber:Ne.originalRange.endLineNumberExclusive-1,domNode:_(),heightInPx:Ct,showInHiddenAreas:!0,suppressMouseDown:!0})}else{let ht=function(){const ts=document.createElement("div");return ts.className="arrow-revert-change "+Ie.asClassName(te.arrowRight),w.add(U(ts,"mousedown",xs=>xs.stopPropagation())),w.add(U(ts,"click",xs=>{xs.stopPropagation(),s.revert(Ne.diff)})),ne("div",{},ts)};if(j!=null&&j.lineRangeMapping.modified.delta(-1).deltaLength(2).contains(Ne.modifiedRange.endLineNumberExclusive-1))continue;let Ti;Ne.diff&&Ne.diff.modified.isEmpty&&this._options.shouldRenderOldRevertArrows.read(v)&&(Ti=ht()),k.push({afterLineNumber:Ne.modifiedRange.endLineNumberExclusive-1,domNode:_(),heightInPx:-Ct,marginDomNode:Ti,showInHiddenAreas:!0,suppressMouseDown:!0})}}for(const Ne of p.read(v)??[]){if(!(j!=null&&j.lineRangeMapping.original.intersect(Ne.originalRange))||!(j!=null&&j.lineRangeMapping.modified.intersect(Ne.modifiedRange)))continue;const Ct=Ne.modifiedHeightInPx-Ne.originalHeightInPx;Ct>0?L.push({afterLineNumber:Ne.originalRange.endLineNumberExclusive-1,domNode:_(),heightInPx:Ct,showInHiddenAreas:!0,suppressMouseDown:!0}):k.push({afterLineNumber:Ne.modifiedRange.endLineNumberExclusive-1,domNode:_(),heightInPx:-Ct,showInHiddenAreas:!0,suppressMouseDown:!0})}return{orig:L,mod:k}});let C=!1;this._register(this._editors.original.onDidScrollChange(v=>{v.scrollLeftChanged&&!C&&(C=!0,this._editors.modified.setScrollLeft(v.scrollLeft),C=!1)})),this._register(this._editors.modified.onDidScrollChange(v=>{v.scrollLeftChanged&&!C&&(C=!0,this._editors.original.setScrollLeft(v.scrollLeft),C=!1)})),this._originalScrollTop=xt(this._editors.original.onDidScrollChange,()=>this._editors.original.getScrollTop()),this._modifiedScrollTop=xt(this._editors.modified.onDidScrollChange,()=>this._editors.modified.getScrollTop()),this._register(Ae(v=>{const w=this._originalScrollTop.read(v)-(this._originalScrollOffsetAnimated.get()-this._modifiedScrollOffsetAnimated.read(v))-(this._originalTopPadding.get()-this._modifiedTopPadding.read(v));w!==this._editors.modified.getScrollTop()&&this._editors.modified.setScrollTop(w,1)})),this._register(Ae(v=>{const w=this._modifiedScrollTop.read(v)-(this._modifiedScrollOffsetAnimated.get()-this._originalScrollOffsetAnimated.read(v))-(this._modifiedTopPadding.get()-this._originalTopPadding.read(v));w!==this._editors.original.getScrollTop()&&this._editors.original.setScrollTop(w,1)})),this._register(Ae(v=>{var L;const w=(L=this._diffModel.read(v))==null?void 0:L.movedTextToCompare.read(v);let S=0;if(w){const k=this._editors.original.getTopForLineNumber(w.lineRangeMapping.original.startLineNumber,!0)-this._originalTopPadding.get();S=this._editors.modified.getTopForLineNumber(w.lineRangeMapping.modified.startLineNumber,!0)-this._modifiedTopPadding.get()-k}S>0?(this._modifiedTopPadding.set(0,void 0),this._originalTopPadding.set(S,void 0)):S<0?(this._modifiedTopPadding.set(-S,void 0),this._originalTopPadding.set(0,void 0)):setTimeout(()=>{this._modifiedTopPadding.set(0,void 0),this._originalTopPadding.set(0,void 0)},400),this._editors.modified.hasTextFocus()?this._originalScrollOffset.set(this._modifiedScrollOffset.get()-S,void 0,!0):this._modifiedScrollOffset.set(this._originalScrollOffset.get()+S,void 0,!0)}))}};AP=vve([iV(8,md),iV(9,mo)],AP);function nV(o,e,t,i,n,s){const r=new Qc(sV(o,i)),a=new Qc(sV(e,n)),l=o.getOption(67),c=e.getOption(67),d=[];let h=0,u=0;function g(f,p){for(;;){let _=r.peek(),b=a.peek();if(_&&_.lineNumber>=f&&(_=void 0),b&&b.lineNumber>=p&&(b=void 0),!_&&!b)break;const C=_?_.lineNumber-h:Number.MAX_VALUE,v=b?b.lineNumber-u:Number.MAX_VALUE;C<v?(r.dequeue(),b={lineNumber:_.lineNumber-h+u,heightInPx:0}):C>v?(a.dequeue(),_={lineNumber:b.lineNumber-u+h,heightInPx:0}):(r.dequeue(),a.dequeue()),d.push({originalRange:Be.ofLength(_.lineNumber,1),modifiedRange:Be.ofLength(b.lineNumber,1),originalHeightInPx:l+_.heightInPx,modifiedHeightInPx:c+b.heightInPx,diff:void 0})}}for(const f of t){let v=function(w,S,L=!1){var W,V;if(w<C||S<b)return;if(_)_=!1;else if(!L&&(w===C||S===b))return;const k=new Be(C,w),D=new Be(b,S);if(k.isEmpty&&D.isEmpty)return;const N=((W=r.takeWhile($=>$.lineNumber<w))==null?void 0:W.reduce(($,H)=>$+H.heightInPx,0))??0,B=((V=a.takeWhile($=>$.lineNumber<S))==null?void 0:V.reduce(($,H)=>$+H.heightInPx,0))??0;d.push({originalRange:k,modifiedRange:D,originalHeightInPx:k.length*l+N,modifiedHeightInPx:D.length*c+B,diff:f.lineRangeMapping}),C=w,b=S};const p=f.lineRangeMapping;g(p.original.startLineNumber,p.modified.startLineNumber);let _=!0,b=p.modified.startLineNumber,C=p.original.startLineNumber;if(s)for(const w of p.innerChanges||[]){w.originalRange.startColumn>1&&w.modifiedRange.startColumn>1&&v(w.originalRange.startLineNumber,w.modifiedRange.startLineNumber);const S=o.getModel(),L=w.originalRange.endLineNumber<=S.getLineCount()?S.getLineMaxColumn(w.originalRange.endLineNumber):Number.MAX_SAFE_INTEGER;w.originalRange.endColumn<L&&v(w.originalRange.endLineNumber,w.modifiedRange.endLineNumber)}v(p.original.endLineNumberExclusive,p.modified.endLineNumberExclusive,!0),h=p.original.endLineNumberExclusive,u=p.modified.endLineNumberExclusive}return g(Number.MAX_VALUE,Number.MAX_VALUE),d}function sV(o,e){const t=[],i=[],n=o.getOption(147).wrappingColumn!==-1,s=o._getViewModel().coordinatesConverter,r=o.getOption(67);if(n)for(let l=1;l<=o.getModel().getLineCount();l++){const c=s.getModelLineViewLineCount(l);c>1&&i.push({lineNumber:l,heightInPx:r*(c-1)})}for(const l of o.getWhitespaces()){if(e.has(l.id))continue;const c=l.afterLineNumber===0?0:s.convertViewPositionToModelPosition(new A(l.afterLineNumber,1)).lineNumber;t.push({lineNumber:c,heightInPx:l.height})}return qCe(t,i,l=>l.lineNumber,(l,c)=>({lineNumber:l.lineNumber,heightInPx:l.heightInPx+c.heightInPx}))}function _W(o){return o.innerChanges?o.innerChanges.every(e=>oV(e.modifiedRange)&&oV(e.originalRange)||e.originalRange.equalsRange(new x(1,1,1,1))):!1}function oV(o){return o.startLineNumber===o.endLineNumber}const zv=class zv extends F{constructor(e,t,i,n,s){super(),this._rootElement=e,this._diffModel=t,this._originalEditorLayoutInfo=i,this._modifiedEditorLayoutInfo=n,this._editors=s,this._originalScrollTop=xt(this,this._editors.original.onDidScrollChange,()=>this._editors.original.getScrollTop()),this._modifiedScrollTop=xt(this,this._editors.modified.onDidScrollChange,()=>this._editors.modified.getScrollTop()),this._viewZonesChanged=xn("onDidChangeViewZones",this._editors.modified.onDidChangeViewZones),this.width=Ke(this,0),this._modifiedViewZonesChangedSignal=xn("modified.onDidChangeViewZones",this._editors.modified.onDidChangeViewZones),this._originalViewZonesChangedSignal=xn("original.onDidChangeViewZones",this._editors.original.onDidChangeViewZones),this._state=su(this,(d,h)=>{var k;this._element.replaceChildren();const u=this._diffModel.read(d),g=(k=u==null?void 0:u.diff.read(d))==null?void 0:k.movedTexts;if(!g||g.length===0){this.width.set(0,void 0);return}this._viewZonesChanged.read(d);const f=this._originalEditorLayoutInfo.read(d),p=this._modifiedEditorLayoutInfo.read(d);if(!f||!p){this.width.set(0,void 0);return}this._modifiedViewZonesChangedSignal.read(d),this._originalViewZonesChangedSignal.read(d);const _=g.map(D=>{function N(fe,ue){const Pe=ue.getTopForLineNumber(fe.startLineNumber,!0),_e=ue.getTopForLineNumber(fe.endLineNumberExclusive,!0);return(Pe+_e)/2}const B=N(D.lineRangeMapping.original,this._editors.original),W=this._originalScrollTop.read(d),V=N(D.lineRangeMapping.modified,this._editors.modified),$=this._modifiedScrollTop.read(d),H=B-W,j=V-$,J=Math.min(B,V),he=Math.max(B,V);return{range:new Ge(J,he),from:H,to:j,fromWithoutScroll:B,toWithoutScroll:V,move:D}});_.sort(SQ(gs(D=>D.fromWithoutScroll>D.toWithoutScroll,yQ),gs(D=>D.fromWithoutScroll>D.toWithoutScroll?D.fromWithoutScroll:-D.toWithoutScroll,$r)));const b=bW.compute(_.map(D=>D.range)),C=10,v=f.verticalScrollbarWidth,w=(b.getTrackCount()-1)*10+C*2,S=v+w+(p.contentLeft-zv.movedCodeBlockPadding);let L=0;for(const D of _){const N=b.getTrack(L),B=v+C+N*10,W=15,V=15,$=S,H=p.glyphMarginWidth+p.lineNumbersWidth,j=18,J=document.createElementNS("http://www.w3.org/2000/svg","rect");J.classList.add("arrow-rectangle"),J.setAttribute("x",`${$-H}`),J.setAttribute("y",`${D.to-j/2}`),J.setAttribute("width",`${H}`),J.setAttribute("height",`${j}`),this._element.appendChild(J);const he=document.createElementNS("http://www.w3.org/2000/svg","g"),fe=document.createElementNS("http://www.w3.org/2000/svg","path");fe.setAttribute("d",`M 0 ${D.from} L ${B} ${D.from} L ${B} ${D.to} L ${$-V} ${D.to}`),fe.setAttribute("fill","none"),he.appendChild(fe);const ue=document.createElementNS("http://www.w3.org/2000/svg","polygon");ue.classList.add("arrow"),h.add(Ae(Pe=>{fe.classList.toggle("currentMove",D.move===u.activeMovedText.read(Pe)),ue.classList.toggle("currentMove",D.move===u.activeMovedText.read(Pe))})),ue.setAttribute("points",`${$-V},${D.to-W/2} ${$},${D.to} ${$-V},${D.to+W/2}`),he.appendChild(ue),this._element.appendChild(he),L++}this.width.set(w,void 0)}),this._element=document.createElementNS("http://www.w3.org/2000/svg","svg"),this._element.setAttribute("class","moved-blocks-lines"),this._rootElement.appendChild(this._element),this._register(ve(()=>this._element.remove())),this._register(Ae(d=>{const h=this._originalEditorLayoutInfo.read(d),u=this._modifiedEditorLayoutInfo.read(d);!h||!u||(this._element.style.left=`${h.width-h.verticalScrollbarWidth}px`,this._element.style.height=`${h.height}px`,this._element.style.width=`${h.verticalScrollbarWidth+h.contentLeft-zv.movedCodeBlockPadding+this.width.read(d)}px`)})),this._register(xb(this._state));const r=be(d=>{const h=this._diffModel.read(d),u=h==null?void 0:h.diff.read(d);return u?u.movedTexts.map(g=>({move:g,original:new mp(ur(g.lineRangeMapping.original.startLineNumber-1),18),modified:new mp(ur(g.lineRangeMapping.modified.startLineNumber-1),18)})):[]});this._register(bx(this._editors.original,r.map(d=>d.map(h=>h.original)))),this._register(bx(this._editors.modified,r.map(d=>d.map(h=>h.modified)))),this._register(Ps((d,h)=>{const u=r.read(d);for(const g of u)h.add(new rV(this._editors.original,g.original,g.move,"original",this._diffModel.get())),h.add(new rV(this._editors.modified,g.modified,g.move,"modified",this._diffModel.get()))}));const a=xn("original.onDidFocusEditorWidget",d=>this._editors.original.onDidFocusEditorWidget(()=>setTimeout(()=>d(void 0),0))),l=xn("modified.onDidFocusEditorWidget",d=>this._editors.modified.onDidFocusEditorWidget(()=>setTimeout(()=>d(void 0),0)));let c="modified";this._register(Z1({createEmptyChangeSummary:()=>{},handleChange:(d,h)=>(d.didChange(a)&&(c="original"),d.didChange(l)&&(c="modified"),!0)},d=>{a.read(d),l.read(d);const h=this._diffModel.read(d);if(!h)return;const u=h.diff.read(d);let g;if(u&&c==="original"){const f=this._editors.originalCursor.read(d);f&&(g=u.movedTexts.find(p=>p.lineRangeMapping.original.contains(f.lineNumber)))}if(u&&c==="modified"){const f=this._editors.modifiedCursor.read(d);f&&(g=u.movedTexts.find(p=>p.lineRangeMapping.modified.contains(f.lineNumber)))}g!==h.movedTextToCompare.get()&&h.movedTextToCompare.set(void 0,void 0),h.setActiveMovedText(g)}))}};zv.movedCodeBlockPadding=4;let l_=zv;class bW{static compute(e){const t=[],i=[];for(const n of e){let s=t.findIndex(r=>!r.intersectsStrict(n));s===-1&&(t.length>=6?s=nse(t,gs(a=>a.intersectWithRangeLength(n),$r)):(s=t.length,t.push(new b5))),t[s].addRange(n),i.push(s)}return new bW(t.length,i)}constructor(e,t){this._trackCount=e,this.trackPerLineIdx=t}getTrack(e){return this.trackPerLineIdx[e]}getTrackCount(){return this._trackCount}}class rV extends lW{constructor(e,t,i,n,s){const r=Xe("div.diff-hidden-lines-widget");super(e,t,r.root),this._editor=e,this._move=i,this._kind=n,this._diffModel=s,this._nodes=Xe("div.diff-moved-code-block",{style:{marginRight:"4px"}},[Xe("div.text-content@textContent"),Xe("div.action-bar@actionBar")]),r.root.appendChild(this._nodes.root);const a=xt(this._editor.onDidLayoutChange,()=>this._editor.getLayoutInfo());this._register(sd(this._nodes.root,{paddingRight:a.map(u=>u.verticalScrollbarWidth)}));let l;i.changes.length>0?l=this._kind==="original"?m("codeMovedToWithChanges","Code moved with changes to line {0}-{1}",this._move.lineRangeMapping.modified.startLineNumber,this._move.lineRangeMapping.modified.endLineNumberExclusive-1):m("codeMovedFromWithChanges","Code moved with changes from line {0}-{1}",this._move.lineRangeMapping.original.startLineNumber,this._move.lineRangeMapping.original.endLineNumberExclusive-1):l=this._kind==="original"?m("codeMovedTo","Code moved to line {0}-{1}",this._move.lineRangeMapping.modified.startLineNumber,this._move.lineRangeMapping.modified.endLineNumberExclusive-1):m("codeMovedFrom","Code moved from line {0}-{1}",this._move.lineRangeMapping.original.startLineNumber,this._move.lineRangeMapping.original.endLineNumberExclusive-1);const c=this._register(new zo(this._nodes.actionBar,{highlightToggledItems:!0})),d=new ro("",l,"",!1);c.push(d,{icon:!1,label:!0});const h=new ro("","Compare",Ie.asClassName(te.compareChanges),!0,()=>{this._editor.focus(),this._diffModel.movedTextToCompare.set(this._diffModel.movedTextToCompare.get()===i?void 0:this._move,void 0)});this._register(Ae(u=>{const g=this._diffModel.movedTextToCompare.read(u)===i;h.checked=g})),c.push(h,{icon:!1,label:!0})}}class wve extends F{constructor(e,t,i,n){super(),this._editors=e,this._diffModel=t,this._options=i,this._decorations=be(this,s=>{const r=this._diffModel.read(s),a=r==null?void 0:r.diff.read(s);if(!a)return null;const l=this._diffModel.read(s).movedTextToCompare.read(s),c=this._options.renderIndicators.read(s),d=this._options.showEmptyDecorations.read(s),h=[],u=[];if(!l)for(const f of a.mappings)if(f.lineRangeMapping.original.isEmpty||h.push({range:f.lineRangeMapping.original.toInclusiveRange(),options:c?Yw:JH}),f.lineRangeMapping.modified.isEmpty||u.push({range:f.lineRangeMapping.modified.toInclusiveRange(),options:c?vx:QH}),f.lineRangeMapping.modified.isEmpty||f.lineRangeMapping.original.isEmpty)f.lineRangeMapping.original.isEmpty||h.push({range:f.lineRangeMapping.original.toInclusiveRange(),options:gW}),f.lineRangeMapping.modified.isEmpty||u.push({range:f.lineRangeMapping.modified.toInclusiveRange(),options:hW});else{const p=this._options.useTrueInlineDiffRendering.read(s)&&_W(f.lineRangeMapping);for(const _ of f.lineRangeMapping.innerChanges||[])if(f.lineRangeMapping.original.contains(_.originalRange.startLineNumber)&&h.push({range:_.originalRange,options:_.originalRange.isEmpty()&&d?fW:G_}),f.lineRangeMapping.modified.contains(_.modifiedRange.startLineNumber)&&u.push({range:_.modifiedRange,options:_.modifiedRange.isEmpty()&&d&&!p?uW:wx}),p){const b=r.model.original.getValueInRange(_.originalRange);u.push({range:_.modifiedRange,options:{description:"deleted-text",before:{content:b,inlineClassName:"inline-deleted-text"},zIndex:1e5,showIfCollapsed:!0}})}}if(l)for(const f of l.changes){const p=f.original.toInclusiveRange();p&&h.push({range:p,options:c?Yw:JH});const _=f.modified.toInclusiveRange();_&&u.push({range:_,options:c?vx:QH});for(const b of f.innerChanges||[])h.push({range:b.originalRange,options:G_}),u.push({range:b.modifiedRange,options:wx})}const g=this._diffModel.read(s).activeMovedText.read(s);for(const f of a.movedTexts)h.push({range:f.lineRangeMapping.original.toInclusiveRange(),options:{description:"moved",blockClassName:"movedOriginal"+(f===g?" currentMove":""),blockPadding:[l_.movedCodeBlockPadding,0,l_.movedCodeBlockPadding,l_.movedCodeBlockPadding]}}),u.push({range:f.lineRangeMapping.modified.toInclusiveRange(),options:{description:"moved",blockClassName:"movedModified"+(f===g?" currentMove":""),blockPadding:[4,0,4,4]}});return{originalDecorations:h,modifiedDecorations:u}}),this._register(_x(this._editors.original,this._decorations.map(s=>(s==null?void 0:s.originalDecorations)||[]))),this._register(_x(this._editors.modified,this._decorations.map(s=>(s==null?void 0:s.modifiedDecorations)||[])))}}class Sve{resetSash(){this._sashRatio.set(void 0,void 0)}constructor(e,t){this._options=e,this.dimensions=t,this.sashLeft=q1(this,i=>{const n=this._sashRatio.read(i)??this._options.splitViewDefaultRatio.read(i);return this._computeSashLeft(n,i)},(i,n)=>{const s=this.dimensions.width.get();this._sashRatio.set(i/s,n)}),this._sashRatio=Ke(this,void 0)}_computeSashLeft(e,t){const i=this.dimensions.width.read(t),n=Math.floor(this._options.splitViewDefaultRatio.read(t)*i),s=this._options.enableSplitViewResizing.read(t)?Math.floor(e*i):n,r=100;return i<=r*2?n:s<r?r:s>i-r?i-r:s}}class Hq extends F{constructor(e,t,i,n,s,r){super(),this._domNode=e,this._dimensions=t,this._enabled=i,this._boundarySashes=n,this.sashLeft=s,this._resetSash=r,this._sash=this._register(new Mn(this._domNode,{getVerticalSashTop:a=>0,getVerticalSashLeft:a=>this.sashLeft.get(),getVerticalSashHeight:a=>this._dimensions.height.get()},{orientation:0})),this._startSashPosition=void 0,this._register(this._sash.onDidStart(()=>{this._startSashPosition=this.sashLeft.get()})),this._register(this._sash.onDidChange(a=>{this.sashLeft.set(this._startSashPosition+(a.currentX-a.startX),void 0)})),this._register(this._sash.onDidEnd(()=>this._sash.layout())),this._register(this._sash.onDidReset(()=>this._resetSash())),this._register(Ae(a=>{const l=this._boundarySashes.read(a);l&&(this._sash.orthogonalEndSash=l.bottom)})),this._register(Ae(a=>{const l=this._enabled.read(a);this._sash.state=l?3:0,this.sashLeft.read(a),this._dimensions.height.read(a),this._sash.layout()}))}}class yve extends F{constructor(e,t,i){super(),this._editor=e,this._domNode=t,this.itemProvider=i,this.scrollTop=xt(this,this._editor.onDidScrollChange,r=>this._editor.getScrollTop()),this.isScrollTopZero=this.scrollTop.map(r=>r===0),this.modelAttached=xt(this,this._editor.onDidChangeModel,r=>this._editor.hasModel()),this.editorOnDidChangeViewZones=xn("onDidChangeViewZones",this._editor.onDidChangeViewZones),this.editorOnDidContentSizeChange=xn("onDidContentSizeChange",this._editor.onDidContentSizeChange),this.domNodeSizeChanged=Lb("domNodeSizeChanged"),this.views=new Map,this._domNode.className="gutter monaco-editor";const n=this._domNode.appendChild(Xe("div.scroll-decoration",{role:"presentation",ariaHidden:"true",style:{width:"100%"}}).root),s=new ResizeObserver(()=>{ai(r=>{this.domNodeSizeChanged.trigger(r)})});s.observe(this._domNode),this._register(ve(()=>s.disconnect())),this._register(Ae(r=>{n.className=this.isScrollTopZero.read(r)?"":"scroll-decoration"})),this._register(Ae(r=>this.render(r)))}dispose(){super.dispose(),_n(this._domNode)}render(e){if(!this.modelAttached.read(e))return;this.domNodeSizeChanged.read(e),this.editorOnDidChangeViewZones.read(e),this.editorOnDidContentSizeChange.read(e);const t=this.scrollTop.read(e),i=this._editor.getVisibleRanges(),n=new Set(this.views.keys()),s=Ge.ofStartAndLength(0,this._domNode.clientHeight);if(!s.isEmpty)for(const r of i){const a=new Be(r.startLineNumber,r.endLineNumber+1),l=this.itemProvider.getIntersectingGutterItems(a,e);ai(c=>{for(const d of l){if(!d.range.intersect(a))continue;n.delete(d.id);let h=this.views.get(d.id);if(h)h.item.set(d,c);else{const p=document.createElement("div");this._domNode.appendChild(p);const _=Ke("item",d),b=this.itemProvider.createView(_,p);h=new Lve(_,b,p),this.views.set(d.id,h)}const u=d.range.startLineNumber<=this._editor.getModel().getLineCount()?this._editor.getTopForLineNumber(d.range.startLineNumber,!0)-t:this._editor.getBottomForLineNumber(d.range.startLineNumber-1,!1)-t,f=(d.range.endLineNumberExclusive===1?Math.max(u,this._editor.getTopForLineNumber(d.range.startLineNumber,!1)-t):Math.max(u,this._editor.getBottomForLineNumber(d.range.endLineNumberExclusive-1,!0)-t))-u;h.domNode.style.top=`${u}px`,h.domNode.style.height=`${f}px`,h.gutterItemView.layout(Ge.ofStartAndLength(u,f),s)}})}for(const r of n){const a=this.views.get(r);a.gutterItemView.dispose(),a.domNode.remove(),this.views.delete(r)}}}class Lve{constructor(e,t,i){this.item=e,this.gutterItemView=t,this.domNode=i}}class Vq extends Ef{constructor(e){super(),this._getContext=e}runAction(e,t){const i=this._getContext();return super.runAction(e,i)}}class aV extends o${constructor(e){super(),this._textModel=e}getValueOfRange(e){return this._textModel.getValueInRange(e)}get length(){const e=this._textModel.getLineCount(),t=this._textModel.getLineLength(e);return new Yr(e-1,t)}}class xve extends F{constructor(e,t,i={orientation:0}){super(),this.submenuActionViewItems=[],this.hasSecondaryActions=!1,this._onDidChangeDropdownVisibility=this._register(new qJ),this.onDidChangeDropdownVisibility=this._onDidChangeDropdownVisibility.event,this.disposables=this._register(new K),i.hoverDelegate=i.hoverDelegate??this._register(H_()),this.options=i,this.toggleMenuAction=this._register(new Xw(()=>{var n;return(n=this.toggleMenuActionViewItem)==null?void 0:n.show()},i.toggleMenuTitle)),this.element=document.createElement("div"),this.element.className="monaco-toolbar",e.appendChild(this.element),this.actionBar=this._register(new zo(this.element,{orientation:i.orientation,ariaLabel:i.ariaLabel,actionRunner:i.actionRunner,allowContextMenu:i.allowContextMenu,highlightToggledItems:i.highlightToggledItems,hoverDelegate:i.hoverDelegate,actionViewItemProvider:(n,s)=>{if(n.id===Xw.ID)return this.toggleMenuActionViewItem=new yL(n,n.menuActions,t,{actionViewItemProvider:this.options.actionViewItemProvider,actionRunner:this.actionRunner,keybindingProvider:this.options.getKeyBinding,classNames:Ie.asClassNameArray(i.moreIcon??te.toolBarMore),anchorAlignmentProvider:this.options.anchorAlignmentProvider,menuAsChild:!!this.options.renderDropdownAsChildElement,skipTelemetry:this.options.skipTelemetry,isMenu:!0,hoverDelegate:this.options.hoverDelegate}),this.toggleMenuActionViewItem.setActionContext(this.actionBar.context),this.disposables.add(this._onDidChangeDropdownVisibility.add(this.toggleMenuActionViewItem.onDidChangeVisibility)),this.toggleMenuActionViewItem;if(i.actionViewItemProvider){const r=i.actionViewItemProvider(n,s);if(r)return r}if(n instanceof D_){const r=new yL(n,n.actions,t,{actionViewItemProvider:this.options.actionViewItemProvider,actionRunner:this.actionRunner,keybindingProvider:this.options.getKeyBinding,classNames:n.class,anchorAlignmentProvider:this.options.anchorAlignmentProvider,menuAsChild:!!this.options.renderDropdownAsChildElement,skipTelemetry:this.options.skipTelemetry,hoverDelegate:this.options.hoverDelegate});return r.setActionContext(this.actionBar.context),this.submenuActionViewItems.push(r),this.disposables.add(this._onDidChangeDropdownVisibility.add(r.onDidChangeVisibility)),r}}}))}set actionRunner(e){this.actionBar.actionRunner=e}get actionRunner(){return this.actionBar.actionRunner}getElement(){return this.element}getItemAction(e){return this.actionBar.getAction(e)}setActions(e,t){this.clear();const i=e?e.slice(0):[];this.hasSecondaryActions=!!(t&&t.length>0),this.hasSecondaryActions&&t&&(this.toggleMenuAction.menuActions=t.slice(0),i.push(this.toggleMenuAction)),i.forEach(n=>{this.actionBar.push(n,{icon:this.options.icon??!0,label:this.options.label??!1,keybinding:this.getKeybindingLabel(n)})})}getKeybindingLabel(e){var i,n;const t=(n=(i=this.options).getKeyBinding)==null?void 0:n.call(i,e);return(t==null?void 0:t.getLabel())??void 0}clear(){this.submenuActionViewItems=[],this.disposables.clear(),this.actionBar.clear()}dispose(){this.clear(),this.disposables.dispose(),super.dispose()}}const _D=class _D extends ro{constructor(e,t){t=t||m("moreActions","More Actions..."),super(_D.ID,t,void 0,!0),this._menuActions=[],this.toggleDropdownMenu=e}async run(){this.toggleDropdownMenu()}get menuActions(){return this._menuActions}set menuActions(e){this._menuActions=e}};_D.ID="toolbar.toggle.more";let Xw=_D;var zq=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Wr=function(o,e){return function(t,i){e(t,i,o)}};let Qw=class extends xve{constructor(e,t,i,n,s,r,a,l){super(e,s,{getKeyBinding:d=>r.lookupKeybinding(d.id)??void 0,...t,allowContextMenu:!0,skipTelemetry:typeof(t==null?void 0:t.telemetrySource)=="string"}),this._options=t,this._menuService=i,this._contextKeyService=n,this._contextMenuService=s,this._keybindingService=r,this._commandService=a,this._sessionDisposables=this._store.add(new K);const c=t==null?void 0:t.telemetrySource;c&&this._store.add(this.actionBar.onDidRun(d=>l.publicLog2("workbenchActionExecuted",{id:d.action.id,from:c})))}setActions(e,t=[],i){var d,h,u;this._sessionDisposables.clear();const n=e.slice(),s=t.slice(),r=[];let a=0;const l=[];let c=!1;if(((d=this._options)==null?void 0:d.hiddenItemStrategy)!==-1)for(let g=0;g<n.length;g++){const f=n[g];!(f instanceof ao)&&!(f instanceof $p)||f.hideActions&&(r.push(f.hideActions.toggle),f.hideActions.toggle.checked&&a++,f.hideActions.isHidden&&(c=!0,n[g]=void 0,((h=this._options)==null?void 0:h.hiddenItemStrategy)!==0&&(l[g]=f)))}if(((u=this._options)==null?void 0:u.overflowBehavior)!==void 0){const g=Hhe(new Set(this._options.overflowBehavior.exempted),ot.map(n,_=>_==null?void 0:_.id)),f=this._options.overflowBehavior.maxItems-g.size;let p=0;for(let _=0;_<n.length;_++){const b=n[_];b&&(p++,!g.has(b.id)&&p>=f&&(n[_]=void 0,l[_]=b))}}WB(n),WB(l),super.setActions(n,Qi.join(l,s)),(r.length>0||n.length>0)&&this._sessionDisposables.add(U(this.getElement(),"contextmenu",g=>{var C,v,w,S,L;const f=new Vr(xe(this.getElement()),g),p=this.getItemAction(f.target);if(!p)return;f.preventDefault(),f.stopPropagation();const _=[];if(p instanceof ao&&p.menuKeybinding)_.push(p.menuKeybinding);else if(!(p instanceof $p||p instanceof Xw)){const k=!!this._keybindingService.lookupKeybinding(p.id);_.push(oq(this._commandService,this._keybindingService,p.id,void 0,k))}if(r.length>0){let k=!1;if(a===1&&((C=this._options)==null?void 0:C.hiddenItemStrategy)===0){k=!0;for(let D=0;D<r.length;D++)if(r[D].checked){r[D]=pg({id:p.id,label:p.label,checked:!0,enabled:!1,run(){}});break}}if(!k&&(p instanceof ao||p instanceof $p)){if(!p.hideActions)return;_.push(p.hideActions.hide)}else _.push(pg({id:"label",label:m("hide","Hide"),enabled:!1,run(){}}))}const b=Qi.join(_,r);(v=this._options)!=null&&v.resetMenu&&!i&&(i=[this._options.resetMenu]),c&&i&&(b.push(new Qi),b.push(pg({id:"resetThisMenu",label:m("resetThisMenu","Reset Menu"),run:()=>this._menuService.resetHiddenStates(i)}))),b.length!==0&&this._contextMenuService.showContextMenu({getAnchor:()=>f,getActions:()=>b,menuId:(w=this._options)==null?void 0:w.contextMenu,menuActionOptions:{renderShortTitle:!0,...(S=this._options)==null?void 0:S.menuOptions},skipTelemetry:typeof((L=this._options)==null?void 0:L.telemetrySource)=="string",contextKeyService:this._contextKeyService})}))}};Qw=zq([Wr(2,Go),Wr(3,Ee),Wr(4,mo),Wr(5,yt),Wr(6,Ht),Wr(7,Bn)],Qw);let Sx=class extends Qw{constructor(e,t,i,n,s,r,a,l,c){super(e,{resetMenu:t,...i},n,s,r,a,l,c),this._onDidChangeMenuItems=this._store.add(new P),this.onDidChangeMenuItems=this._onDidChangeMenuItems.event;const d=this._store.add(n.createMenu(t,s,{emitEventsForSubmenuChanges:!0})),h=()=>{var f,p,_;const u=[],g=[];sE(d,i==null?void 0:i.menuOptions,{primary:u,secondary:g},(f=i==null?void 0:i.toolbarOptions)==null?void 0:f.primaryGroup,(p=i==null?void 0:i.toolbarOptions)==null?void 0:p.shouldInlineSubmenu,(_=i==null?void 0:i.toolbarOptions)==null?void 0:_.useSeparatorsInPrimaryActions),e.classList.toggle("has-no-actions",u.length===0&&g.length===0),super.setActions(u,g)};this._store.add(d.onDidChange(()=>{h(),this._onDidChangeMenuItems.fire(this)})),h()}setActions(){throw new bt("This toolbar is populated from a menu.")}};Sx=zq([Wr(3,Go),Wr(4,Ee),Wr(5,mo),Wr(6,yt),Wr(7,Ht),Wr(8,Bn)],Sx);var Uq=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},dy=function(o,e){return function(t,i){e(t,i,o)}};const IT=[],g0=35;let PP=class extends F{constructor(e,t,i,n,s,r,a,l,c){super(),this._diffModel=t,this._editors=i,this._options=n,this._sashLayout=s,this._boundarySashes=r,this._instantiationService=a,this._contextKeyService=l,this._menuService=c,this._menu=this._register(this._menuService.createMenu(me.DiffEditorHunkToolbar,this._contextKeyService)),this._actions=xt(this,this._menu.onDidChange,()=>this._menu.getActions()),this._hasActions=this._actions.map(d=>d.length>0),this._showSash=be(this,d=>this._options.renderSideBySide.read(d)&&this._hasActions.read(d)),this.width=be(this,d=>this._hasActions.read(d)?g0:0),this.elements=Xe("div.gutter@gutter",{style:{position:"absolute",height:"100%",width:g0+"px"}},[]),this._currentDiff=be(this,d=>{var f;const h=this._diffModel.read(d);if(!h)return;const u=(f=h.diff.read(d))==null?void 0:f.mappings,g=this._editors.modifiedCursor.read(d);if(g)return u==null?void 0:u.find(p=>p.lineRangeMapping.modified.contains(g.lineNumber))}),this._selectedDiffs=be(this,d=>{const h=this._diffModel.read(d),u=h==null?void 0:h.diff.read(d);if(!u)return IT;const g=this._editors.modifiedSelections.read(d);if(g.every(b=>b.isEmpty()))return IT;const f=new dr(g.map(b=>Be.fromRangeInclusive(b))),_=u.mappings.filter(b=>b.lineRangeMapping.innerChanges&&f.intersects(b.lineRangeMapping.modified)).map(b=>({mapping:b,rangeMappings:b.lineRangeMapping.innerChanges.filter(C=>g.some(v=>x.areIntersecting(C.modifiedRange,v)))}));return _.length===0||_.every(b=>b.rangeMappings.length===0)?IT:_}),this._register(GCe(e,this.elements.root)),this._register(U(this.elements.root,"click",()=>{this._editors.modified.focus()})),this._register(sd(this.elements.root,{display:this._hasActions.map(d=>d?"block":"none")})),Gn(this,d=>this._showSash.read(d)?new Hq(e,this._sashLayout.dimensions,this._options.enableSplitViewResizing,this._boundarySashes,q1(this,u=>this._sashLayout.sashLeft.read(u)-g0,(u,g)=>this._sashLayout.sashLeft.set(u+g0,g)),()=>this._sashLayout.resetSash()):void 0).recomputeInitiallyAndOnChange(this._store),this._register(new yve(this._editors.modified,this.elements.root,{getIntersectingGutterItems:(d,h)=>{const u=this._diffModel.read(h);if(!u)return[];const g=u.diff.read(h);if(!g)return[];const f=this._selectedDiffs.read(h);if(f.length>0){const _=Vo.fromRangeMappings(f.flatMap(b=>b.rangeMappings));return[new lV(_,!0,me.DiffEditorSelectionToolbar,void 0,u.model.original.uri,u.model.modified.uri)]}const p=this._currentDiff.read(h);return g.mappings.map(_=>new lV(_.lineRangeMapping.withInnerChangesFromLineRanges(),_.lineRangeMapping===(p==null?void 0:p.lineRangeMapping),me.DiffEditorHunkToolbar,void 0,u.model.original.uri,u.model.modified.uri))},createView:(d,h)=>this._instantiationService.createInstance(OP,d,h,this)})),this._register(U(this.elements.gutter,ie.MOUSE_WHEEL,d=>{this._editors.modified.getOption(104).handleMouseWheel&&this._editors.modified.delegateScrollFromMouseWheelEvent(d)},{passive:!1}))}computeStagedValue(e){const t=e.innerChanges??[],i=new aV(this._editors.modifiedModel.get()),n=new aV(this._editors.original.getModel());return new v5(t.map(a=>a.toTextEdit(i))).apply(n)}layout(e){this.elements.gutter.style.left=e+"px"}};PP=Uq([dy(6,we),dy(7,Ee),dy(8,Go)],PP);class lV{constructor(e,t,i,n,s,r){this.mapping=e,this.showAlways=t,this.menuId=i,this.rangeOverride=n,this.originalUri=s,this.modifiedUri=r}get id(){return this.mapping.modified.toString()}get range(){return this.rangeOverride??this.mapping.modified}}let OP=class extends F{constructor(e,t,i,n){super(),this._item=e,this._elements=Xe("div.gutterItem",{style:{height:"20px",width:"34px"}},[Xe("div.background@background",{},[]),Xe("div.buttons@buttons",{},[])]),this._showAlways=this._item.map(this,r=>r.showAlways),this._menuId=this._item.map(this,r=>r.menuId),this._isSmall=Ke(this,!1),this._lastItemRange=void 0,this._lastViewRange=void 0;const s=this._register(n.createInstance(M_,"element",!0,{position:{hoverPosition:1}}));this._register(fp(t,this._elements.root)),this._register(Ae(r=>{const a=this._showAlways.read(r);this._elements.root.classList.toggle("noTransition",!0),this._elements.root.classList.toggle("showAlways",a),setTimeout(()=>{this._elements.root.classList.toggle("noTransition",!1)},0)})),this._register(Ps((r,a)=>{this._elements.buttons.replaceChildren();const l=a.add(n.createInstance(Sx,this._elements.buttons,this._menuId.read(r),{orientation:1,hoverDelegate:s,toolbarOptions:{primaryGroup:c=>c.startsWith("primary")},overflowBehavior:{maxItems:this._isSmall.read(r)?1:3},hiddenItemStrategy:0,actionRunner:new Vq(()=>{const c=this._item.get(),d=c.mapping;return{mapping:d,originalWithModifiedChanges:i.computeStagedValue(d),originalUri:c.originalUri,modifiedUri:c.modifiedUri}}),menuOptions:{shouldForwardArgs:!0}}));a.add(l.onDidChangeMenuItems(()=>{this._lastItemRange&&this.layout(this._lastItemRange,this._lastViewRange)}))}))}layout(e,t){this._lastItemRange=e,this._lastViewRange=t;let i=this._elements.buttons.clientHeight;this._isSmall.set(this._item.get().mapping.original.startLineNumber===1&&e.length<30,void 0),i=this._elements.buttons.clientHeight;const n=e.length/2-i/2,s=i;let r=e.start+n;const a=Ge.tryCreate(s,t.endExclusive-s-i),l=Ge.tryCreate(e.start+s,e.endExclusive-i-s);l&&a&&l.start<l.endExclusive&&(r=a.clip(r),r=l.clip(r)),this._elements.buttons.style.top=`${r-e.start}px`}};OP=Uq([dy(3,we)],OP);function no(o){return FP.get(o)}const Ad=class Ad extends F{static get(e){let t=Ad._map.get(e);if(!t){t=new Ad(e),Ad._map.set(e,t);const i=e.onDidDispose(()=>{const n=Ad._map.get(e);n&&(Ad._map.delete(e),n.dispose(),i.dispose())})}return t}_beginUpdate(){this._updateCounter++,this._updateCounter===1&&(this._currentTransaction=new yb(()=>{}))}_endUpdate(){if(this._updateCounter--,this._updateCounter===0){const e=this._currentTransaction;this._currentTransaction=void 0,e.finish()}}constructor(e){var t;super(),this.editor=e,this._updateCounter=0,this._currentTransaction=void 0,this._model=Ke(this,this.editor.getModel()),this.model=this._model,this.isReadonly=xt(this,this.editor.onDidChangeConfiguration,()=>this.editor.getOption(92)),this._versionId=z2({owner:this,lazy:!0},((t=this.editor.getModel())==null?void 0:t.getVersionId())??null),this.versionId=this._versionId,this._selections=z2({owner:this,equalsFn:B2(vL(pe.selectionsEqual)),lazy:!0},this.editor.getSelections()??null),this.selections=this._selections,this.isFocused=xt(this,i=>{const n=this.editor.onDidFocusEditorWidget(i),s=this.editor.onDidBlurEditorWidget(i);return{dispose(){n.dispose(),s.dispose()}}},()=>this.editor.hasWidgetFocus()),this.value=q1(this,i=>{var n;return this.versionId.read(i),((n=this.model.read(i))==null?void 0:n.getValue())??""},(i,n)=>{const s=this.model.get();s!==null&&i!==s.getValue()&&s.setValue(i)}),this.valueIsEmpty=be(this,i=>{var n;return this.versionId.read(i),((n=this.editor.getModel())==null?void 0:n.getValueLength())===0}),this.cursorSelection=Mo({owner:this,equalsFn:B2(pe.selectionsEqual)},i=>{var n;return((n=this.selections.read(i))==null?void 0:n[0])??null}),this.onDidType=Lb(this),this.scrollTop=xt(this.editor.onDidScrollChange,()=>this.editor.getScrollTop()),this.scrollLeft=xt(this.editor.onDidScrollChange,()=>this.editor.getScrollLeft()),this.layoutInfo=xt(this.editor.onDidLayoutChange,()=>this.editor.getLayoutInfo()),this.layoutInfoContentLeft=this.layoutInfo.map(i=>i.contentLeft),this.layoutInfoDecorationsLeft=this.layoutInfo.map(i=>i.decorationsLeft),this.contentWidth=xt(this.editor.onDidContentSizeChange,()=>this.editor.getContentWidth()),this._overlayWidgetCounter=0,this._register(this.editor.onBeginUpdate(()=>this._beginUpdate())),this._register(this.editor.onEndUpdate(()=>this._endUpdate())),this._register(this.editor.onDidChangeModel(()=>{this._beginUpdate();try{this._model.set(this.editor.getModel(),this._currentTransaction),this._forceUpdate()}finally{this._endUpdate()}})),this._register(this.editor.onDidType(i=>{this._beginUpdate();try{this._forceUpdate(),this.onDidType.trigger(this._currentTransaction,i)}finally{this._endUpdate()}})),this._register(this.editor.onDidChangeModelContent(i=>{var n;this._beginUpdate();try{this._versionId.set(((n=this.editor.getModel())==null?void 0:n.getVersionId())??null,this._currentTransaction,i),this._forceUpdate()}finally{this._endUpdate()}})),this._register(this.editor.onDidChangeCursorSelection(i=>{this._beginUpdate();try{this._selections.set(this.editor.getSelections(),this._currentTransaction,i),this._forceUpdate()}finally{this._endUpdate()}}))}forceUpdate(e){this._beginUpdate();try{return this._forceUpdate(),e?e(this._currentTransaction):void 0}finally{this._endUpdate()}}_forceUpdate(){var e;this._beginUpdate();try{this._model.set(this.editor.getModel(),this._currentTransaction),this._versionId.set(((e=this.editor.getModel())==null?void 0:e.getVersionId())??null,this._currentTransaction,void 0),this._selections.set(this.editor.getSelections(),this._currentTransaction,void 0)}finally{this._endUpdate()}}getOption(e){return xt(this,t=>this.editor.onDidChangeConfiguration(i=>{i.hasChanged(e)&&t(void 0)}),()=>this.editor.getOption(e))}setDecorations(e){const t=new K,i=this.editor.createDecorationsCollection();return t.add(G1({owner:this,debugName:()=>`Apply decorations from ${e.debugName}`},n=>{const s=e.read(n);i.set(s)})),t.add({dispose:()=>{i.clear()}}),t}createOverlayWidget(e){const t="observableOverlayWidget"+this._overlayWidgetCounter++,i={getDomNode:()=>e.domNode,getPosition:()=>e.position.get(),getId:()=>t,allowEditorOverflow:e.allowEditorOverflow,getMinContentWidthInPx:()=>e.minContentWidthInPx.get()};this.editor.addOverlayWidget(i);const n=Ae(s=>{e.position.read(s),e.minContentWidthInPx.read(s),this.editor.layoutOverlayWidget(i)});return ve(()=>{n.dispose(),this.editor.removeOverlayWidget(i)})}};Ad._map=new Map;let FP=Ad;function WP(o,e){return Yce({createEmptyChangeSummary:()=>({deltas:[],didChange:!1}),handleChange:(t,i)=>{if(t.didChange(o)){const n=t.change;n!==void 0&&i.deltas.push(n),i.didChange=!0}return!0}},(t,i)=>{const n=o.read(t);i.didChange&&e(n,i.deltas)})}function kve(o,e){const t=new K,i=WP(o,(n,s)=>{t.clear(),e(n,s,t)});return{dispose(){i.dispose(),t.dispose()}}}var Dve=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Ive=function(o,e){return function(t,i){e(t,i,o)}},hy,Pg;let yx=(Pg=class extends F{static setBreadcrumbsSourceFactory(e){this._breadcrumbsSourceFactory.set(e,void 0)}get isUpdatingHiddenAreas(){return this._isUpdatingHiddenAreas}constructor(e,t,i,n){super(),this._editors=e,this._diffModel=t,this._options=i,this._instantiationService=n,this._modifiedOutlineSource=Gn(this,l=>{const c=this._editors.modifiedModel.read(l),d=hy._breadcrumbsSourceFactory.read(l);return!c||!d?void 0:d(c,this._instantiationService)}),this._isUpdatingHiddenAreas=!1,this._register(this._editors.original.onDidChangeCursorPosition(l=>{if(l.reason===1)return;const c=this._diffModel.get();ai(d=>{for(const h of this._editors.original.getSelections()||[])c==null||c.ensureOriginalLineIsVisible(h.getStartPosition().lineNumber,0,d),c==null||c.ensureOriginalLineIsVisible(h.getEndPosition().lineNumber,0,d)})})),this._register(this._editors.modified.onDidChangeCursorPosition(l=>{if(l.reason===1)return;const c=this._diffModel.get();ai(d=>{for(const h of this._editors.modified.getSelections()||[])c==null||c.ensureModifiedLineIsVisible(h.getStartPosition().lineNumber,0,d),c==null||c.ensureModifiedLineIsVisible(h.getEndPosition().lineNumber,0,d)})}));const s=this._diffModel.map((l,c)=>{var h;const d=(l==null?void 0:l.unchangedRegions.read(c))??[];return d.length===1&&d[0].modifiedLineNumber===1&&d[0].lineCount===((h=this._editors.modifiedModel.read(c))==null?void 0:h.getLineCount())?[]:d});this.viewZones=su(this,(l,c)=>{const d=this._modifiedOutlineSource.read(l);if(!d)return{origViewZones:[],modViewZones:[]};const h=[],u=[],g=this._options.renderSideBySide.read(l),f=this._options.compactMode.read(l),p=s.read(l);for(let _=0;_<p.length;_++){const b=p[_];if(!b.shouldHideControls(l)&&!(f&&(_===0||_===p.length-1)))if(f){{const C=be(this,w=>b.getHiddenOriginalRange(w).startLineNumber-1),v=new mp(C,12);h.push(v),c.add(new cV(this._editors.original,v,b,!g))}{const C=be(this,w=>b.getHiddenModifiedRange(w).startLineNumber-1),v=new mp(C,12);u.push(v),c.add(new cV(this._editors.modified,v,b))}}else{{const C=be(this,w=>b.getHiddenOriginalRange(w).startLineNumber-1),v=new mp(C,24);h.push(v),c.add(new dV(this._editors.original,v,b,b.originalUnchangedRange,!g,d,w=>this._diffModel.get().ensureModifiedLineIsVisible(w,2,void 0),this._options))}{const C=be(this,w=>b.getHiddenModifiedRange(w).startLineNumber-1),v=new mp(C,24);u.push(v),c.add(new dV(this._editors.modified,v,b,b.modifiedUnchangedRange,!1,d,w=>this._diffModel.get().ensureModifiedLineIsVisible(w,2,void 0),this._options))}}}return{origViewZones:h,modViewZones:u}});const r={description:"unchanged lines",className:"diff-unchanged-lines",isWholeLine:!0},a={description:"Fold Unchanged",glyphMarginHoverMessage:new Pn(void 0,{isTrusted:!0,supportThemeIcons:!0}).appendMarkdown(m("foldUnchanged","Fold Unchanged Region")),glyphMarginClassName:"fold-unchanged "+Ie.asClassName(te.fold),zIndex:10001};this._register(_x(this._editors.original,be(this,l=>{const c=s.read(l),d=c.map(h=>({range:h.originalUnchangedRange.toInclusiveRange(),options:r}));for(const h of c)h.shouldHideControls(l)&&d.push({range:x.fromPositions(new A(h.originalLineNumber,1)),options:a});return d}))),this._register(_x(this._editors.modified,be(this,l=>{const c=s.read(l),d=c.map(h=>({range:h.modifiedUnchangedRange.toInclusiveRange(),options:r}));for(const h of c)h.shouldHideControls(l)&&d.push({range:Be.ofLength(h.modifiedLineNumber,1).toInclusiveRange(),options:a});return d}))),this._register(Ae(l=>{const c=s.read(l);this._isUpdatingHiddenAreas=!0;try{this._editors.original.setHiddenAreas(c.map(d=>d.getHiddenOriginalRange(l).toInclusiveRange()).filter(Tl)),this._editors.modified.setHiddenAreas(c.map(d=>d.getHiddenModifiedRange(l).toInclusiveRange()).filter(Tl))}finally{this._isUpdatingHiddenAreas=!1}})),this._register(this._editors.modified.onMouseUp(l=>{var c;if(!l.event.rightButton&&l.target.position&&((c=l.target.element)!=null&&c.className.includes("fold-unchanged"))){const d=l.target.position.lineNumber,h=this._diffModel.get();if(!h)return;const u=h.unchangedRegions.get().find(g=>g.modifiedUnchangedRange.includes(d));if(!u)return;u.collapseAll(void 0),l.event.stopPropagation(),l.event.preventDefault()}})),this._register(this._editors.original.onMouseUp(l=>{var c;if(!l.event.rightButton&&l.target.position&&((c=l.target.element)!=null&&c.className.includes("fold-unchanged"))){const d=l.target.position.lineNumber,h=this._diffModel.get();if(!h)return;const u=h.unchangedRegions.get().find(g=>g.originalUnchangedRange.includes(d));if(!u)return;u.collapseAll(void 0),l.event.stopPropagation(),l.event.preventDefault()}}))}},hy=Pg,Pg._breadcrumbsSourceFactory=Ke(hy,()=>({dispose(){},getBreadcrumbItems(e,t){return[]}})),Pg);yx=hy=Dve([Ive(3,we)],yx);class cV extends lW{constructor(e,t,i,n=!1){const s=Xe("div.diff-hidden-lines-widget");super(e,t,s.root),this._unchangedRegion=i,this._hide=n,this._nodes=Xe("div.diff-hidden-lines-compact",[Xe("div.line-left",[]),Xe("div.text@text",[]),Xe("div.line-right",[])]),s.root.appendChild(this._nodes.root),this._hide&&this._nodes.root.replaceChildren(),this._register(Ae(r=>{if(!this._hide){const a=this._unchangedRegion.getHiddenModifiedRange(r).length,l=m("hiddenLines","{0} hidden lines",a);this._nodes.text.innerText=l}}))}}class dV extends lW{constructor(e,t,i,n,s,r,a,l){const c=Xe("div.diff-hidden-lines-widget");super(e,t,c.root),this._editor=e,this._unchangedRegion=i,this._unchangedRegionRange=n,this._hide=s,this._modifiedOutlineSource=r,this._revealModifiedHiddenLine=a,this._options=l,this._nodes=Xe("div.diff-hidden-lines",[Xe("div.top@top",{title:m("diff.hiddenLines.top","Click or drag to show more above")}),Xe("div.center@content",{style:{display:"flex"}},[Xe("div@first",{style:{display:"flex",justifyContent:"center",alignItems:"center",flexShrink:"0"}},[ne("a",{title:m("showUnchangedRegion","Show Unchanged Region"),role:"button",onclick:()=>{this._unchangedRegion.showAll(void 0)}},...dh("$(unfold)"))]),Xe("div@others",{style:{display:"flex",justifyContent:"center",alignItems:"center"}})]),Xe("div.bottom@bottom",{title:m("diff.bottom","Click or drag to show more below"),role:"button"})]),c.root.appendChild(this._nodes.root),this._hide?_n(this._nodes.first):this._register(sd(this._nodes.first,{width:no(this._editor).layoutInfoContentLeft})),this._register(Ae(h=>{const u=this._unchangedRegion.visibleLineCountTop.read(h)+this._unchangedRegion.visibleLineCountBottom.read(h)===this._unchangedRegion.lineCount;this._nodes.bottom.classList.toggle("canMoveTop",!u),this._nodes.bottom.classList.toggle("canMoveBottom",this._unchangedRegion.visibleLineCountBottom.read(h)>0),this._nodes.top.classList.toggle("canMoveTop",this._unchangedRegion.visibleLineCountTop.read(h)>0),this._nodes.top.classList.toggle("canMoveBottom",!u);const g=this._unchangedRegion.isDragged.read(h),f=this._editor.getDomNode();f&&(f.classList.toggle("draggingUnchangedRegion",!!g),g==="top"?(f.classList.toggle("canMoveTop",this._unchangedRegion.visibleLineCountTop.read(h)>0),f.classList.toggle("canMoveBottom",!u)):g==="bottom"?(f.classList.toggle("canMoveTop",!u),f.classList.toggle("canMoveBottom",this._unchangedRegion.visibleLineCountBottom.read(h)>0)):(f.classList.toggle("canMoveTop",!1),f.classList.toggle("canMoveBottom",!1)))}));const d=this._editor;this._register(U(this._nodes.top,"mousedown",h=>{if(h.button!==0)return;this._nodes.top.classList.toggle("dragging",!0),this._nodes.root.classList.toggle("dragging",!0),h.preventDefault();const u=h.clientY;let g=!1;const f=this._unchangedRegion.visibleLineCountTop.get();this._unchangedRegion.isDragged.set("top",void 0);const p=xe(this._nodes.top),_=U(p,"mousemove",C=>{const w=C.clientY-u;g=g||Math.abs(w)>2;const S=Math.round(w/d.getOption(67)),L=Math.max(0,Math.min(f+S,this._unchangedRegion.getMaxVisibleLineCountTop()));this._unchangedRegion.visibleLineCountTop.set(L,void 0)}),b=U(p,"mouseup",C=>{g||this._unchangedRegion.showMoreAbove(this._options.hideUnchangedRegionsRevealLineCount.get(),void 0),this._nodes.top.classList.toggle("dragging",!1),this._nodes.root.classList.toggle("dragging",!1),this._unchangedRegion.isDragged.set(void 0,void 0),_.dispose(),b.dispose()})})),this._register(U(this._nodes.bottom,"mousedown",h=>{if(h.button!==0)return;this._nodes.bottom.classList.toggle("dragging",!0),this._nodes.root.classList.toggle("dragging",!0),h.preventDefault();const u=h.clientY;let g=!1;const f=this._unchangedRegion.visibleLineCountBottom.get();this._unchangedRegion.isDragged.set("bottom",void 0);const p=xe(this._nodes.bottom),_=U(p,"mousemove",C=>{const w=C.clientY-u;g=g||Math.abs(w)>2;const S=Math.round(w/d.getOption(67)),L=Math.max(0,Math.min(f-S,this._unchangedRegion.getMaxVisibleLineCountBottom())),k=this._unchangedRegionRange.endLineNumberExclusive>d.getModel().getLineCount()?d.getContentHeight():d.getTopForLineNumber(this._unchangedRegionRange.endLineNumberExclusive);this._unchangedRegion.visibleLineCountBottom.set(L,void 0);const D=this._unchangedRegionRange.endLineNumberExclusive>d.getModel().getLineCount()?d.getContentHeight():d.getTopForLineNumber(this._unchangedRegionRange.endLineNumberExclusive);d.setScrollTop(d.getScrollTop()+(D-k))}),b=U(p,"mouseup",C=>{if(this._unchangedRegion.isDragged.set(void 0,void 0),!g){const v=d.getTopForLineNumber(this._unchangedRegionRange.endLineNumberExclusive);this._unchangedRegion.showMoreBelow(this._options.hideUnchangedRegionsRevealLineCount.get(),void 0);const w=d.getTopForLineNumber(this._unchangedRegionRange.endLineNumberExclusive);d.setScrollTop(d.getScrollTop()+(w-v))}this._nodes.bottom.classList.toggle("dragging",!1),this._nodes.root.classList.toggle("dragging",!1),_.dispose(),b.dispose()})})),this._register(Ae(h=>{const u=[];if(!this._hide){const g=i.getHiddenModifiedRange(h).length,f=m("hiddenLines","{0} hidden lines",g),p=ne("span",{title:m("diff.hiddenLines.expandAll","Double click to unfold")},f);p.addEventListener("dblclick",C=>{C.button===0&&(C.preventDefault(),this._unchangedRegion.showAll(void 0))}),u.push(p);const _=this._unchangedRegion.getHiddenModifiedRange(h),b=this._modifiedOutlineSource.getBreadcrumbItems(_,h);if(b.length>0){u.push(ne("span",void 0,"  |  "));for(let C=0;C<b.length;C++){const v=b[C],w=Oy.toIcon(v.kind),S=Xe("div.breadcrumb-item",{style:{display:"flex",alignItems:"center"}},[Rf(w)," ",v.name,...C===b.length-1?[]:[Rf(te.chevronRight)]]).root;u.push(S),S.onclick=()=>{this._revealModifiedHiddenLine(v.startLineNumber)}}}}_n(this._nodes.others,...u)}))}}var Eve=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Nve=function(o,e){return function(t,i){e(t,i,o)}},la,Vc;let Jw=(Vc=class extends F{constructor(e,t,i,n,s,r,a){super(),this._editors=e,this._rootElement=t,this._diffModel=i,this._rootWidth=n,this._rootHeight=s,this._modifiedEditorLayoutInfo=r,this._themeService=a,this.width=la.ENTIRE_DIFF_OVERVIEW_WIDTH;const l=xt(this._themeService.onDidColorThemeChange,()=>this._themeService.getColorTheme()),c=be(u=>{const g=l.read(u),f=g.getColor(Yoe)||(g.getColor(Goe)||a2).transparent(2),p=g.getColor(Xoe)||(g.getColor(Zoe)||l2).transparent(2);return{insertColor:f,removeColor:p}}),d=St(document.createElement("div"));d.setClassName("diffViewport"),d.setPosition("absolute");const h=Xe("div.diffOverview",{style:{position:"absolute",top:"0px",width:la.ENTIRE_DIFF_OVERVIEW_WIDTH+"px"}}).root;this._register(fp(h,d.domNode)),this._register(hi(h,ie.POINTER_DOWN,u=>{this._editors.modified.delegateVerticalScrollbarPointerDown(u)})),this._register(U(h,ie.MOUSE_WHEEL,u=>{this._editors.modified.delegateScrollFromMouseWheelEvent(u)},{passive:!1})),this._register(fp(this._rootElement,h)),this._register(Ps((u,g)=>{const f=this._diffModel.read(u),p=this._editors.original.createOverviewRuler("original diffOverviewRuler");p&&(g.add(p),g.add(fp(h,p.getDomNode())));const _=this._editors.modified.createOverviewRuler("modified diffOverviewRuler");if(_&&(g.add(_),g.add(fp(h,_.getDomNode()))),!p||!_)return;const b=xn("viewZoneChanged",this._editors.original.onDidChangeViewZones),C=xn("viewZoneChanged",this._editors.modified.onDidChangeViewZones),v=xn("hiddenRangesChanged",this._editors.original.onDidChangeHiddenAreas),w=xn("hiddenRangesChanged",this._editors.modified.onDidChangeHiddenAreas);g.add(Ae(S=>{var W;b.read(S),C.read(S),v.read(S),w.read(S);const L=c.read(S),k=(W=f==null?void 0:f.diff.read(S))==null?void 0:W.mappings;function D(V,$,H){const j=H._getViewModel();return j?V.filter(J=>J.length>0).map(J=>{const he=j.coordinatesConverter.convertModelPositionToViewPosition(new A(J.startLineNumber,1)),fe=j.coordinatesConverter.convertModelPositionToViewPosition(new A(J.endLineNumberExclusive,1)),ue=fe.lineNumber-he.lineNumber;return new Nq(he.lineNumber,fe.lineNumber,ue,$.toString())}):[]}const N=D((k||[]).map(V=>V.lineRangeMapping.original),L.removeColor,this._editors.original),B=D((k||[]).map(V=>V.lineRangeMapping.modified),L.insertColor,this._editors.modified);p==null||p.setZones(N),_==null||_.setZones(B)})),g.add(Ae(S=>{const L=this._rootHeight.read(S),k=this._rootWidth.read(S),D=this._modifiedEditorLayoutInfo.read(S);if(D){const N=la.ENTIRE_DIFF_OVERVIEW_WIDTH-2*la.ONE_OVERVIEW_WIDTH;p.setLayout({top:0,height:L,right:N+la.ONE_OVERVIEW_WIDTH,width:la.ONE_OVERVIEW_WIDTH}),_.setLayout({top:0,height:L,right:0,width:la.ONE_OVERVIEW_WIDTH});const B=this._editors.modifiedScrollTop.read(S),W=this._editors.modifiedScrollHeight.read(S),V=this._editors.modified.getOption(104),$=new P_(V.verticalHasArrows?V.arrowSize:0,V.verticalScrollbarSize,0,D.height,W,B);d.setTop($.getSliderPosition()),d.setHeight($.getSliderSize())}else d.setTop(0),d.setHeight(0);h.style.height=L+"px",h.style.left=k-la.ENTIRE_DIFF_OVERVIEW_WIDTH+"px",d.setWidth(la.ENTIRE_DIFF_OVERVIEW_WIDTH)}))}))}},la=Vc,Vc.ONE_OVERVIEW_WIDTH=15,Vc.ENTIRE_DIFF_OVERVIEW_WIDTH=Vc.ONE_OVERVIEW_WIDTH*2,Vc);Jw=la=Eve([Nve(6,ji)],Jw);const ET=[];class Tve extends F{constructor(e,t,i,n){super(),this._editors=e,this._diffModel=t,this._options=i,this._widget=n,this._selectedDiffs=be(this,s=>{const r=this._diffModel.read(s),a=r==null?void 0:r.diff.read(s);if(!a)return ET;const l=this._editors.modifiedSelections.read(s);if(l.every(u=>u.isEmpty()))return ET;const c=new dr(l.map(u=>Be.fromRangeInclusive(u))),h=a.mappings.filter(u=>u.lineRangeMapping.innerChanges&&c.intersects(u.lineRangeMapping.modified)).map(u=>({mapping:u,rangeMappings:u.lineRangeMapping.innerChanges.filter(g=>l.some(f=>x.areIntersecting(g.modifiedRange,f)))}));return h.length===0||h.every(u=>u.rangeMappings.length===0)?ET:h}),this._register(Ps((s,r)=>{if(!this._options.shouldRenderOldRevertArrows.read(s))return;const a=this._diffModel.read(s),l=a==null?void 0:a.diff.read(s);if(!a||!l||a.movedTextToCompare.read(s))return;const c=[],d=this._selectedDiffs.read(s),h=new Set(d.map(u=>u.mapping));if(d.length>0){const u=this._editors.modifiedSelections.read(s),g=r.add(new Lx(u[u.length-1].positionLineNumber,this._widget,d.flatMap(f=>f.rangeMappings),!0));this._editors.modified.addGlyphMarginWidget(g),c.push(g)}for(const u of l.mappings)if(!h.has(u)&&!u.lineRangeMapping.modified.isEmpty&&u.lineRangeMapping.innerChanges){const g=r.add(new Lx(u.lineRangeMapping.modified.startLineNumber,this._widget,u.lineRangeMapping,!1));this._editors.modified.addGlyphMarginWidget(g),c.push(g)}r.add(ve(()=>{for(const u of c)this._editors.modified.removeGlyphMarginWidget(u)}))}))}}const bD=class bD extends F{getId(){return this._id}constructor(e,t,i,n){super(),this._lineNumber=e,this._widget=t,this._diffs=i,this._revertSelection=n,this._id=`revertButton${bD.counter++}`,this._domNode=Xe("div.revertButton",{title:this._revertSelection?m("revertSelectedChanges","Revert Selected Changes"):m("revertChange","Revert Change")},[Rf(te.arrowRight)]).root,this._register(U(this._domNode,ie.MOUSE_DOWN,s=>{s.button!==2&&(s.stopPropagation(),s.preventDefault())})),this._register(U(this._domNode,ie.MOUSE_UP,s=>{s.stopPropagation(),s.preventDefault()})),this._register(U(this._domNode,ie.CLICK,s=>{this._diffs instanceof Yn?this._widget.revert(this._diffs):this._widget.revertRangeMappings(this._diffs),s.stopPropagation(),s.preventDefault()}))}getDomNode(){return this._domNode}getPosition(){return{lane:Zr.Right,range:{startColumn:1,startLineNumber:this._lineNumber,endColumn:1,endLineNumber:this._lineNumber},zIndex:10001}}};bD.counter=0;let Lx=bD;function Rve(o,e,t){return Qce({debugName:()=>`Configuration Key "${o}"`},i=>t.onDidChangeConfiguration(n=>{n.affectsConfiguration(o)&&i(n)}),()=>t.getValue(o)??e)}function ha(o,e,t){const i=o.bindTo(e);return G1({debugName:()=>`Set Context Key "${o.key}"`},n=>{i.set(t(n))})}var Mve=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},hV=function(o,e){return function(t,i){e(t,i,o)}};let BP=class extends F{get onDidContentSizeChange(){return this._onDidContentSizeChange.event}constructor(e,t,i,n,s,r,a){super(),this.originalEditorElement=e,this.modifiedEditorElement=t,this._options=i,this._argCodeEditorWidgetOptions=n,this._createInnerEditor=s,this._instantiationService=r,this._keybindingService=a,this.original=this._register(this._createLeftHandSideEditor(this._options.editorOptions.get(),this._argCodeEditorWidgetOptions.originalEditor||{})),this.modified=this._register(this._createRightHandSideEditor(this._options.editorOptions.get(),this._argCodeEditorWidgetOptions.modifiedEditor||{})),this._onDidContentSizeChange=this._register(new P),this.modifiedScrollTop=xt(this,this.modified.onDidScrollChange,()=>this.modified.getScrollTop()),this.modifiedScrollHeight=xt(this,this.modified.onDidScrollChange,()=>this.modified.getScrollHeight()),this.modifiedObs=no(this.modified),this.originalObs=no(this.original),this.modifiedModel=this.modifiedObs.model,this.modifiedSelections=xt(this,this.modified.onDidChangeCursorSelection,()=>this.modified.getSelections()??[]),this.modifiedCursor=Mo({owner:this,equalsFn:A.equals},l=>{var c;return((c=this.modifiedSelections.read(l)[0])==null?void 0:c.getPosition())??new A(1,1)}),this.originalCursor=xt(this,this.original.onDidChangeCursorPosition,()=>this.original.getPosition()??new A(1,1)),this._argCodeEditorWidgetOptions=null,this._register(Z1({createEmptyChangeSummary:()=>({}),handleChange:(l,c)=>(l.didChange(i.editorOptions)&&Object.assign(c,l.change.changedOptions),!0)},(l,c)=>{i.editorOptions.read(l),this._options.renderSideBySide.read(l),this.modified.updateOptions(this._adjustOptionsForRightHandSide(l,c)),this.original.updateOptions(this._adjustOptionsForLeftHandSide(l,c))}))}_createLeftHandSideEditor(e,t){const i=this._adjustOptionsForLeftHandSide(void 0,e),n=this._constructInnerEditor(this._instantiationService,this.originalEditorElement,i,t);return n.setContextValue("isInDiffLeftEditor",!0),n}_createRightHandSideEditor(e,t){const i=this._adjustOptionsForRightHandSide(void 0,e),n=this._constructInnerEditor(this._instantiationService,this.modifiedEditorElement,i,t);return n.setContextValue("isInDiffRightEditor",!0),n}_constructInnerEditor(e,t,i,n){const s=this._createInnerEditor(e,t,i,n);return this._register(s.onDidContentSizeChange(r=>{const a=this.original.getContentWidth()+this.modified.getContentWidth()+Jw.ENTIRE_DIFF_OVERVIEW_WIDTH,l=Math.max(this.modified.getContentHeight(),this.original.getContentHeight());this._onDidContentSizeChange.fire({contentHeight:l,contentWidth:a,contentHeightChanged:r.contentHeightChanged,contentWidthChanged:r.contentWidthChanged})})),s}_adjustOptionsForLeftHandSide(e,t){const i=this._adjustOptionsForSubEditor(t);return this._options.renderSideBySide.get()?(i.unicodeHighlight=this._options.editorOptions.get().unicodeHighlight||{},i.wordWrapOverride1=this._options.diffWordWrap.get()):(i.wordWrapOverride1="off",i.wordWrapOverride2="off",i.stickyScroll={enabled:!1},i.unicodeHighlight={nonBasicASCII:!1,ambiguousCharacters:!1,invisibleCharacters:!1}),i.glyphMargin=this._options.renderSideBySide.get(),t.originalAriaLabel&&(i.ariaLabel=t.originalAriaLabel),i.ariaLabel=this._updateAriaLabel(i.ariaLabel),i.readOnly=!this._options.originalEditable.get(),i.dropIntoEditor={enabled:!i.readOnly},i.extraEditorClassName="original-in-monaco-diff-editor",i}_adjustOptionsForRightHandSide(e,t){const i=this._adjustOptionsForSubEditor(t);return t.modifiedAriaLabel&&(i.ariaLabel=t.modifiedAriaLabel),i.ariaLabel=this._updateAriaLabel(i.ariaLabel),i.wordWrapOverride1=this._options.diffWordWrap.get(),i.revealHorizontalRightPadding=Ua.revealHorizontalRightPadding.defaultValue+Jw.ENTIRE_DIFF_OVERVIEW_WIDTH,i.scrollbar.verticalHasArrows=!1,i.extraEditorClassName="modified-in-monaco-diff-editor",i}_adjustOptionsForSubEditor(e){const t={...e,dimension:{height:0,width:0}};return t.inDiffEditor=!0,t.automaticLayout=!1,t.scrollbar={...t.scrollbar||{}},t.folding=!1,t.codeLens=this._options.diffCodeLens.get(),t.fixedOverflowWidgets=!0,t.minimap={...t.minimap||{}},t.minimap.enabled=!1,this._options.hideUnchangedRegions.get()?t.stickyScroll={enabled:!1}:t.stickyScroll=this._options.editorOptions.get().stickyScroll,t}_updateAriaLabel(e){var i;e||(e="");const t=m("diff-aria-navigation-tip"," use {0} to open the accessibility help.",(i=this._keybindingService.lookupKeybinding("editor.action.accessibilityHelp"))==null?void 0:i.getAriaLabel());return this._options.accessibilityVerbose.get()?e+t:e?e.replaceAll(t,""):""}};BP=Mve([hV(5,we),hV(6,yt)],BP);const CD=class CD extends F{constructor(){super(...arguments),this._id=++CD.idCounter,this._onDidDispose=this._register(new P),this.onDidDispose=this._onDidDispose.event}getId(){return this.getEditorType()+":v2:"+this._id}getVisibleColumnFromPosition(e){return this._targetEditor.getVisibleColumnFromPosition(e)}getPosition(){return this._targetEditor.getPosition()}setPosition(e,t="api"){this._targetEditor.setPosition(e,t)}revealLine(e,t=0){this._targetEditor.revealLine(e,t)}revealLineInCenter(e,t=0){this._targetEditor.revealLineInCenter(e,t)}revealLineInCenterIfOutsideViewport(e,t=0){this._targetEditor.revealLineInCenterIfOutsideViewport(e,t)}revealLineNearTop(e,t=0){this._targetEditor.revealLineNearTop(e,t)}revealPosition(e,t=0){this._targetEditor.revealPosition(e,t)}revealPositionInCenter(e,t=0){this._targetEditor.revealPositionInCenter(e,t)}revealPositionInCenterIfOutsideViewport(e,t=0){this._targetEditor.revealPositionInCenterIfOutsideViewport(e,t)}revealPositionNearTop(e,t=0){this._targetEditor.revealPositionNearTop(e,t)}getSelection(){return this._targetEditor.getSelection()}getSelections(){return this._targetEditor.getSelections()}setSelection(e,t="api"){this._targetEditor.setSelection(e,t)}setSelections(e,t="api"){this._targetEditor.setSelections(e,t)}revealLines(e,t,i=0){this._targetEditor.revealLines(e,t,i)}revealLinesInCenter(e,t,i=0){this._targetEditor.revealLinesInCenter(e,t,i)}revealLinesInCenterIfOutsideViewport(e,t,i=0){this._targetEditor.revealLinesInCenterIfOutsideViewport(e,t,i)}revealLinesNearTop(e,t,i=0){this._targetEditor.revealLinesNearTop(e,t,i)}revealRange(e,t=0,i=!1,n=!0){this._targetEditor.revealRange(e,t,i,n)}revealRangeInCenter(e,t=0){this._targetEditor.revealRangeInCenter(e,t)}revealRangeInCenterIfOutsideViewport(e,t=0){this._targetEditor.revealRangeInCenterIfOutsideViewport(e,t)}revealRangeNearTop(e,t=0){this._targetEditor.revealRangeNearTop(e,t)}revealRangeNearTopIfOutsideViewport(e,t=0){this._targetEditor.revealRangeNearTopIfOutsideViewport(e,t)}revealRangeAtTop(e,t=0){this._targetEditor.revealRangeAtTop(e,t)}getSupportedActions(){return this._targetEditor.getSupportedActions()}focus(){this._targetEditor.focus()}trigger(e,t,i){this._targetEditor.trigger(e,t,i)}createDecorationsCollection(e){return this._targetEditor.createDecorationsCollection(e)}changeDecorations(e){return this._targetEditor.changeDecorations(e)}};CD.idCounter=0;let HP=CD;var Ave=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Pve=function(o,e){return function(t,i){e(t,i,o)}};let VP=class{get editorOptions(){return this._options}constructor(e,t){this._accessibilityService=t,this._diffEditorWidth=Ke(this,0),this._screenReaderMode=xt(this,this._accessibilityService.onDidChangeScreenReaderOptimized,()=>this._accessibilityService.isScreenReaderOptimized()),this.couldShowInlineViewBecauseOfSize=be(this,n=>this._options.read(n).renderSideBySide&&this._diffEditorWidth.read(n)<=this._options.read(n).renderSideBySideInlineBreakpoint),this.renderOverviewRuler=be(this,n=>this._options.read(n).renderOverviewRuler),this.renderSideBySide=be(this,n=>this.compactMode.read(n)&&this.shouldRenderInlineViewInSmartMode.read(n)?!1:this._options.read(n).renderSideBySide&&!(this._options.read(n).useInlineViewWhenSpaceIsLimited&&this.couldShowInlineViewBecauseOfSize.read(n)&&!this._screenReaderMode.read(n))),this.readOnly=be(this,n=>this._options.read(n).readOnly),this.shouldRenderOldRevertArrows=be(this,n=>!(!this._options.read(n).renderMarginRevertIcon||!this.renderSideBySide.read(n)||this.readOnly.read(n)||this.shouldRenderGutterMenu.read(n))),this.shouldRenderGutterMenu=be(this,n=>this._options.read(n).renderGutterMenu),this.renderIndicators=be(this,n=>this._options.read(n).renderIndicators),this.enableSplitViewResizing=be(this,n=>this._options.read(n).enableSplitViewResizing),this.splitViewDefaultRatio=be(this,n=>this._options.read(n).splitViewDefaultRatio),this.ignoreTrimWhitespace=be(this,n=>this._options.read(n).ignoreTrimWhitespace),this.maxComputationTimeMs=be(this,n=>this._options.read(n).maxComputationTime),this.showMoves=be(this,n=>this._options.read(n).experimental.showMoves&&this.renderSideBySide.read(n)),this.isInEmbeddedEditor=be(this,n=>this._options.read(n).isInEmbeddedEditor),this.diffWordWrap=be(this,n=>this._options.read(n).diffWordWrap),this.originalEditable=be(this,n=>this._options.read(n).originalEditable),this.diffCodeLens=be(this,n=>this._options.read(n).diffCodeLens),this.accessibilityVerbose=be(this,n=>this._options.read(n).accessibilityVerbose),this.diffAlgorithm=be(this,n=>this._options.read(n).diffAlgorithm),this.showEmptyDecorations=be(this,n=>this._options.read(n).experimental.showEmptyDecorations),this.onlyShowAccessibleDiffViewer=be(this,n=>this._options.read(n).onlyShowAccessibleDiffViewer),this.compactMode=be(this,n=>this._options.read(n).compactMode),this.trueInlineDiffRenderingEnabled=be(this,n=>this._options.read(n).experimental.useTrueInlineView),this.useTrueInlineDiffRendering=be(this,n=>!this.renderSideBySide.read(n)&&this.trueInlineDiffRenderingEnabled.read(n)),this.hideUnchangedRegions=be(this,n=>this._options.read(n).hideUnchangedRegions.enabled),this.hideUnchangedRegionsRevealLineCount=be(this,n=>this._options.read(n).hideUnchangedRegions.revealLineCount),this.hideUnchangedRegionsContextLineCount=be(this,n=>this._options.read(n).hideUnchangedRegions.contextLineCount),this.hideUnchangedRegionsMinimumLineCount=be(this,n=>this._options.read(n).hideUnchangedRegions.minimumLineCount),this._model=Ke(this,void 0),this.shouldRenderInlineViewInSmartMode=this._model.map(this,n=>ide(this,s=>{const r=n==null?void 0:n.diff.read(s);return r?Ove(r,this.trueInlineDiffRenderingEnabled.read(s)):void 0})).flatten().map(this,n=>!!n),this.inlineViewHideOriginalLineNumbers=this.compactMode;const i={...e,...uV(e,vn)};this._options=Ke(this,i)}updateOptions(e){const t=uV(e,this._options.get()),i={...this._options.get(),...e,...t};this._options.set(i,void 0,{changedOptions:e})}setWidth(e){this._diffEditorWidth.set(e,void 0)}setModel(e){this._model.set(e,void 0)}};VP=Ave([Pve(1,po)],VP);function Ove(o,e){return o.mappings.every(t=>Fve(t.lineRangeMapping)||Wve(t.lineRangeMapping)||e&&_W(t.lineRangeMapping))}function Fve(o){return o.original.length===0}function Wve(o){return o.modified.length===0}function uV(o,e){var t,i,n,s,r,a,l,c;return{enableSplitViewResizing:Ce(o.enableSplitViewResizing,e.enableSplitViewResizing),splitViewDefaultRatio:$Q(o.splitViewDefaultRatio,.5,.1,.9),renderSideBySide:Ce(o.renderSideBySide,e.renderSideBySide),renderMarginRevertIcon:Ce(o.renderMarginRevertIcon,e.renderMarginRevertIcon),maxComputationTime:Iu(o.maxComputationTime,e.maxComputationTime,0,1073741824),maxFileSize:Iu(o.maxFileSize,e.maxFileSize,0,1073741824),ignoreTrimWhitespace:Ce(o.ignoreTrimWhitespace,e.ignoreTrimWhitespace),renderIndicators:Ce(o.renderIndicators,e.renderIndicators),originalEditable:Ce(o.originalEditable,e.originalEditable),diffCodeLens:Ce(o.diffCodeLens,e.diffCodeLens),renderOverviewRuler:Ce(o.renderOverviewRuler,e.renderOverviewRuler),diffWordWrap:gi(o.diffWordWrap,e.diffWordWrap,["off","on","inherit"]),diffAlgorithm:gi(o.diffAlgorithm,e.diffAlgorithm,["legacy","advanced"],{smart:"legacy",experimental:"advanced"}),accessibilityVerbose:Ce(o.accessibilityVerbose,e.accessibilityVerbose),experimental:{showMoves:Ce((t=o.experimental)==null?void 0:t.showMoves,e.experimental.showMoves),showEmptyDecorations:Ce((i=o.experimental)==null?void 0:i.showEmptyDecorations,e.experimental.showEmptyDecorations),useTrueInlineView:Ce((n=o.experimental)==null?void 0:n.useTrueInlineView,e.experimental.useTrueInlineView)},hideUnchangedRegions:{enabled:Ce(((s=o.hideUnchangedRegions)==null?void 0:s.enabled)??((r=o.experimental)==null?void 0:r.collapseUnchangedRegions),e.hideUnchangedRegions.enabled),contextLineCount:Iu((a=o.hideUnchangedRegions)==null?void 0:a.contextLineCount,e.hideUnchangedRegions.contextLineCount,0,1073741824),minimumLineCount:Iu((l=o.hideUnchangedRegions)==null?void 0:l.minimumLineCount,e.hideUnchangedRegions.minimumLineCount,0,1073741824),revealLineCount:Iu((c=o.hideUnchangedRegions)==null?void 0:c.revealLineCount,e.hideUnchangedRegions.revealLineCount,0,1073741824)},isInEmbeddedEditor:Ce(o.isInEmbeddedEditor,e.isInEmbeddedEditor),onlyShowAccessibleDiffViewer:Ce(o.onlyShowAccessibleDiffViewer,e.onlyShowAccessibleDiffViewer),renderSideBySideInlineBreakpoint:Iu(o.renderSideBySideInlineBreakpoint,e.renderSideBySideInlineBreakpoint,0,1073741824),useInlineViewWhenSpaceIsLimited:Ce(o.useInlineViewWhenSpaceIsLimited,e.useInlineViewWhenSpaceIsLimited),renderGutterMenu:Ce(o.renderGutterMenu,e.renderGutterMenu),compactMode:Ce(o.compactMode,e.compactMode)}}var Bve=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},uC=function(o,e){return function(t,i){e(t,i,o)}};let od=class extends HP{get onDidContentSizeChange(){return this._editors.onDidContentSizeChange}constructor(e,t,i,n,s,r,a,l){super(),this._domElement=e,this._parentContextKeyService=n,this._parentInstantiationService=s,this._accessibilitySignalService=a,this._editorProgressService=l,this.elements=Xe("div.monaco-diff-editor.side-by-side",{style:{position:"relative",height:"100%"}},[Xe("div.editor.original@original",{style:{position:"absolute",height:"100%"}}),Xe("div.editor.modified@modified",{style:{position:"absolute",height:"100%"}}),Xe("div.accessibleDiffViewer@accessibleDiffViewer",{style:{position:"absolute",height:"100%"}})]),this._diffModelSrc=this._register(Dw(this,void 0)),this._diffModel=be(this,w=>{var S;return(S=this._diffModelSrc.read(w))==null?void 0:S.object}),this.onDidChangeModel=oe.fromObservableLight(this._diffModel),this._contextKeyService=this._register(this._parentContextKeyService.createScoped(this._domElement)),this._instantiationService=this._register(this._parentInstantiationService.createChild(new Eb([Ee,this._contextKeyService]))),this._boundarySashes=Ke(this,void 0),this._accessibleDiffViewerShouldBeVisible=Ke(this,!1),this._accessibleDiffViewerVisible=be(this,w=>this._options.onlyShowAccessibleDiffViewer.read(w)?!0:this._accessibleDiffViewerShouldBeVisible.read(w)),this._movedBlocksLinesPart=Ke(this,void 0),this._layoutInfo=be(this,w=>{var he,fe;const S=this._rootSizeObserver.width.read(w),L=this._rootSizeObserver.height.read(w);this._rootSizeObserver.automaticLayout?this.elements.root.style.height="100%":this.elements.root.style.height=L+"px";const k=this._sash.read(w),D=this._gutter.read(w),N=(D==null?void 0:D.width.read(w))??0,B=((he=this._overviewRulerPart.read(w))==null?void 0:he.width)??0;let W,V,$,H,j;if(!!k){const ue=k.sashLeft.read(w),Pe=((fe=this._movedBlocksLinesPart.read(w))==null?void 0:fe.width.read(w))??0;W=0,V=ue-N-Pe,j=ue-N,$=ue,H=S-$-B}else{j=0;const ue=this._options.inlineViewHideOriginalLineNumbers.read(w);W=N,ue?V=0:V=Math.max(5,this._editors.originalObs.layoutInfoDecorationsLeft.read(w)),$=N+V,H=S-$-B}return this.elements.original.style.left=W+"px",this.elements.original.style.width=V+"px",this._editors.original.layout({width:V,height:L},!0),D==null||D.layout(j),this.elements.modified.style.left=$+"px",this.elements.modified.style.width=H+"px",this._editors.modified.layout({width:H,height:L},!0),{modifiedEditor:this._editors.modified.getLayoutInfo(),originalEditor:this._editors.original.getLayoutInfo()}}),this._diffValue=this._diffModel.map((w,S)=>w==null?void 0:w.diff.read(S)),this.onDidUpdateDiff=oe.fromObservableLight(this._diffValue),r.willCreateDiffEditor(),this._contextKeyService.createKey("isInDiffEditor",!0),this._domElement.appendChild(this.elements.root),this._register(ve(()=>this.elements.root.remove())),this._rootSizeObserver=this._register(new Pq(this.elements.root,t.dimension)),this._rootSizeObserver.setAutomaticLayout(t.automaticLayout??!1),this._options=this._instantiationService.createInstance(VP,t),this._register(Ae(w=>{this._options.setWidth(this._rootSizeObserver.width.read(w))})),this._contextKeyService.createKey(I.isEmbeddedDiffEditor.key,!1),this._register(ha(I.isEmbeddedDiffEditor,this._contextKeyService,w=>this._options.isInEmbeddedEditor.read(w))),this._register(ha(I.comparingMovedCode,this._contextKeyService,w=>{var S;return!!((S=this._diffModel.read(w))!=null&&S.movedTextToCompare.read(w))})),this._register(ha(I.diffEditorRenderSideBySideInlineBreakpointReached,this._contextKeyService,w=>this._options.couldShowInlineViewBecauseOfSize.read(w))),this._register(ha(I.diffEditorInlineMode,this._contextKeyService,w=>!this._options.renderSideBySide.read(w))),this._register(ha(I.hasChanges,this._contextKeyService,w=>{var S,L;return(((L=(S=this._diffModel.read(w))==null?void 0:S.diff.read(w))==null?void 0:L.mappings.length)??0)>0})),this._editors=this._register(this._instantiationService.createInstance(BP,this.elements.original,this.elements.modified,this._options,i,(w,S,L,k)=>this._createInnerEditor(w,S,L,k))),this._register(ha(I.diffEditorOriginalWritable,this._contextKeyService,w=>this._options.originalEditable.read(w))),this._register(ha(I.diffEditorModifiedWritable,this._contextKeyService,w=>!this._options.readOnly.read(w))),this._register(ha(I.diffEditorOriginalUri,this._contextKeyService,w=>{var S;return((S=this._diffModel.read(w))==null?void 0:S.model.original.uri.toString())??""})),this._register(ha(I.diffEditorModifiedUri,this._contextKeyService,w=>{var S;return((S=this._diffModel.read(w))==null?void 0:S.model.modified.uri.toString())??""})),this._overviewRulerPart=Gn(this,w=>this._options.renderOverviewRuler.read(w)?this._instantiationService.createInstance(Js(Jw,w),this._editors,this.elements.root,this._diffModel,this._rootSizeObserver.width,this._rootSizeObserver.height,this._layoutInfo.map(S=>S.modifiedEditor)):void 0).recomputeInitiallyAndOnChange(this._store);const c={height:this._rootSizeObserver.height,width:this._rootSizeObserver.width.map((w,S)=>{var L;return w-(((L=this._overviewRulerPart.read(S))==null?void 0:L.width)??0)})};this._sashLayout=new Sve(this._options,c),this._sash=Gn(this,w=>{const S=this._options.renderSideBySide.read(w);return this.elements.root.classList.toggle("side-by-side",S),S?new Hq(this.elements.root,c,this._options.enableSplitViewResizing,this._boundarySashes,this._sashLayout.sashLeft,()=>this._sashLayout.resetSash()):void 0}).recomputeInitiallyAndOnChange(this._store);const d=Gn(this,w=>this._instantiationService.createInstance(Js(yx,w),this._editors,this._diffModel,this._options)).recomputeInitiallyAndOnChange(this._store);Gn(this,w=>this._instantiationService.createInstance(Js(wve,w),this._editors,this._diffModel,this._options,this)).recomputeInitiallyAndOnChange(this._store);const h=new Set,u=new Set;let g=!1;const f=Gn(this,w=>this._instantiationService.createInstance(Js(AP,w),xe(this._domElement),this._editors,this._diffModel,this._options,this,()=>g||d.get().isUpdatingHiddenAreas,h,u)).recomputeInitiallyAndOnChange(this._store),p=be(this,w=>{const S=f.read(w).viewZones.read(w).orig,L=d.read(w).viewZones.read(w).origViewZones;return S.concat(L)}),_=be(this,w=>{const S=f.read(w).viewZones.read(w).mod,L=d.read(w).viewZones.read(w).modViewZones;return S.concat(L)});this._register(bx(this._editors.original,p,w=>{g=w},h));let b;this._register(bx(this._editors.modified,_,w=>{g=w,g?b=za.capture(this._editors.modified):(b==null||b.restore(this._editors.modified),b=void 0)},u)),this._accessibleDiffViewer=Gn(this,w=>this._instantiationService.createInstance(Js(hg,w),this.elements.accessibleDiffViewer,this._accessibleDiffViewerVisible,(S,L)=>this._accessibleDiffViewerShouldBeVisible.set(S,L),this._options.onlyShowAccessibleDiffViewer.map(S=>!S),this._rootSizeObserver.width,this._rootSizeObserver.height,this._diffModel.map((S,L)=>{var k;return(k=S==null?void 0:S.diff.read(L))==null?void 0:k.mappings.map(D=>D.lineRangeMapping)}),new cve(this._editors))).recomputeInitiallyAndOnChange(this._store);const C=this._accessibleDiffViewerVisible.map(w=>w?"hidden":"visible");this._register(sd(this.elements.modified,{visibility:C})),this._register(sd(this.elements.original,{visibility:C})),this._createDiffEditorContributions(),r.addDiffEditor(this),this._gutter=Gn(this,w=>this._options.shouldRenderGutterMenu.read(w)?this._instantiationService.createInstance(Js(PP,w),this.elements.root,this._diffModel,this._editors,this._options,this._sashLayout,this._boundarySashes):void 0),this._register(xb(this._layoutInfo)),Gn(this,w=>new(Js(l_,w))(this.elements.root,this._diffModel,this._layoutInfo.map(S=>S.originalEditor),this._layoutInfo.map(S=>S.modifiedEditor),this._editors)).recomputeInitiallyAndOnChange(this._store,w=>{this._movedBlocksLinesPart.set(w,void 0)}),this._register(oe.runAndSubscribe(this._editors.modified.onDidChangeCursorPosition,w=>this._handleCursorPositionChange(w,!0))),this._register(oe.runAndSubscribe(this._editors.original.onDidChangeCursorPosition,w=>this._handleCursorPositionChange(w,!1)));const v=this._diffModel.map(this,(w,S)=>{if(w)return w.diff.read(S)===void 0&&!w.isDiffUpToDate.read(S)});this._register(Ps((w,S)=>{if(v.read(w)===!0){const L=this._editorProgressService.show(!0,1e3);S.add(ve(()=>L.done()))}})),this._register(Ps((w,S)=>{S.add(new(Js(Tve,w))(this._editors,this._diffModel,this._options,this))})),this._register(Ps((w,S)=>{const L=this._diffModel.read(w);if(L)for(const k of[L.model.original,L.model.modified])S.add(k.onWillDispose(D=>{Oe(new bt("TextModel got disposed before DiffEditorWidget model got reset")),this.setModel(null)}))})),this._register(Ae(w=>{this._options.setModel(this._diffModel.read(w))}))}_createInnerEditor(e,t,i,n){return e.createInstance(Zw,t,i,n)}_createDiffEditorContributions(){const e=_g.getDiffEditorContributions();for(const t of e)try{this._register(this._instantiationService.createInstance(t.ctor,this))}catch(i){Oe(i)}}get _targetEditor(){return this._editors.modified}getEditorType(){return iS.IDiffEditor}layout(e){this._rootSizeObserver.observe(e)}hasTextFocus(){return this._editors.original.hasTextFocus()||this._editors.modified.hasTextFocus()}saveViewState(){var i;const e=this._editors.original.saveViewState(),t=this._editors.modified.saveViewState();return{original:e,modified:t,modelState:(i=this._diffModel.get())==null?void 0:i.serializeState()}}restoreViewState(e){var t;if(e&&e.original&&e.modified){const i=e;this._editors.original.restoreViewState(i.original),this._editors.modified.restoreViewState(i.modified),i.modelState&&((t=this._diffModel.get())==null||t.restoreSerializedState(i.modelState))}}handleInitialized(){this._editors.original.handleInitialized(),this._editors.modified.handleInitialized()}createViewModel(e){return this._instantiationService.createInstance(MP,e,this._options)}getModel(){var e;return((e=this._diffModel.get())==null?void 0:e.model)??null}setModel(e){const t=e?"model"in e?Cx.create(e).createNewRef(this):Cx.create(this.createViewModel(e),this):null;this.setDiffModel(t)}setDiffModel(e,t){const i=this._diffModel.get();!e&&i&&this._accessibleDiffViewer.get().close(),this._diffModel.get()!==(e==null?void 0:e.object)&&Pf(t,n=>{var a;const s=e==null?void 0:e.object;xt.batchEventsGlobally(n,()=>{this._editors.original.setModel(s?s.model.original:null),this._editors.modified.setModel(s?s.model.modified:null)});const r=(a=this._diffModelSrc.get())==null?void 0:a.createNewRef(this);this._diffModelSrc.set(e==null?void 0:e.createNewRef(this),n),setTimeout(()=>{r==null||r.dispose()},0)})}updateOptions(e){this._options.updateOptions(e)}getContainerDomNode(){return this._domElement}getOriginalEditor(){return this._editors.original}getModifiedEditor(){return this._editors.modified}getLineChanges(){var t;const e=(t=this._diffModel.get())==null?void 0:t.diff.get();return e?Hve(e):null}revert(e){const t=this._diffModel.get();!t||!t.isDiffUpToDate.get()||this._editors.modified.executeEdits("diffEditor",[{range:e.modified.toExclusiveRange(),text:t.model.original.getValueInRange(e.original.toExclusiveRange())}])}revertRangeMappings(e){const t=this._diffModel.get();if(!t||!t.isDiffUpToDate.get())return;const i=e.map(n=>({range:n.modifiedRange,text:t.model.original.getValueInRange(n.originalRange)}));this._editors.modified.executeEdits("diffEditor",i)}_goTo(e){this._editors.modified.setPosition(new A(e.lineRangeMapping.modified.startLineNumber,1)),this._editors.modified.revealRangeInCenter(e.lineRangeMapping.modified.toExclusiveRange())}goToDiff(e){var s,r;const t=(r=(s=this._diffModel.get())==null?void 0:s.diff.get())==null?void 0:r.mappings;if(!t||t.length===0)return;const i=this._editors.modified.getPosition().lineNumber;let n;e==="next"?n=t.find(a=>a.lineRangeMapping.modified.startLineNumber>i)??t[0]:n=dw(t,a=>a.lineRangeMapping.modified.startLineNumber<i)??t[t.length-1],this._goTo(n),n.lineRangeMapping.modified.isEmpty?this._accessibilitySignalService.playSignal(No.diffLineDeleted,{source:"diffEditor.goToDiff"}):n.lineRangeMapping.original.isEmpty?this._accessibilitySignalService.playSignal(No.diffLineInserted,{source:"diffEditor.goToDiff"}):n&&this._accessibilitySignalService.playSignal(No.diffLineModified,{source:"diffEditor.goToDiff"})}revealFirstDiff(){const e=this._diffModel.get();e&&this.waitForDiff().then(()=>{var i;const t=(i=e.diff.get())==null?void 0:i.mappings;!t||t.length===0||this._goTo(t[0])})}accessibleDiffViewerNext(){this._accessibleDiffViewer.get().next()}accessibleDiffViewerPrev(){this._accessibleDiffViewer.get().prev()}async waitForDiff(){const e=this._diffModel.get();e&&await e.waitForDiff()}mapToOtherSide(){var r,a;const e=this._editors.modified.hasWidgetFocus(),t=e?this._editors.modified:this._editors.original,i=e?this._editors.original:this._editors.modified;let n;const s=t.getSelection();if(s){const l=(a=(r=this._diffModel.get())==null?void 0:r.diff.get())==null?void 0:a.mappings.map(c=>e?c.lineRangeMapping.flip():c.lineRangeMapping);if(l){const c=YH(s.getStartPosition(),l),d=YH(s.getEndPosition(),l);n=x.plusRange(c,d)}}return{destination:i,destinationSelection:n}}switchSide(){const{destination:e,destinationSelection:t}=this.mapToOtherSide();e.focus(),t&&e.setSelection(t)}exitCompareMove(){const e=this._diffModel.get();e&&e.movedTextToCompare.set(void 0,void 0)}collapseAllUnchangedRegions(){var t;const e=(t=this._diffModel.get())==null?void 0:t.unchangedRegions.get();e&&ai(i=>{for(const n of e)n.collapseAll(i)})}showAllUnchangedRegions(){var t;const e=(t=this._diffModel.get())==null?void 0:t.unchangedRegions.get();e&&ai(i=>{for(const n of e)n.showAll(i)})}_handleCursorPositionChange(e,t){var i,n;if((e==null?void 0:e.reason)===3){const s=(n=(i=this._diffModel.get())==null?void 0:i.diff.get())==null?void 0:n.mappings.find(r=>t?r.lineRangeMapping.modified.contains(e.position.lineNumber):r.lineRangeMapping.original.contains(e.position.lineNumber));s!=null&&s.lineRangeMapping.modified.isEmpty?this._accessibilitySignalService.playSignal(No.diffLineDeleted,{source:"diffEditor.cursorPositionChanged"}):s!=null&&s.lineRangeMapping.original.isEmpty?this._accessibilitySignalService.playSignal(No.diffLineInserted,{source:"diffEditor.cursorPositionChanged"}):s&&this._accessibilitySignalService.playSignal(No.diffLineModified,{source:"diffEditor.cursorPositionChanged"})}}};od=Bve([uC(3,Ee),uC(4,we),uC(5,pt),uC(6,ru),uC(7,nu)],od);function Hve(o){return o.mappings.map(e=>{const t=e.lineRangeMapping;let i,n,s,r,a=t.innerChanges;return t.original.isEmpty?(i=t.original.startLineNumber-1,n=0,a=void 0):(i=t.original.startLineNumber,n=t.original.endLineNumberExclusive-1),t.modified.isEmpty?(s=t.modified.startLineNumber-1,r=0,a=void 0):(s=t.modified.startLineNumber,r=t.modified.endLineNumberExclusive-1),{originalStartLineNumber:i,originalEndLineNumber:n,modifiedStartLineNumber:s,modifiedEndLineNumber:r,charChanges:a==null?void 0:a.map(l=>({originalStartLineNumber:l.originalRange.startLineNumber,originalStartColumn:l.originalRange.startColumn,originalEndLineNumber:l.originalRange.endLineNumber,originalEndColumn:l.originalRange.endColumn,modifiedStartLineNumber:l.modifiedRange.startLineNumber,modifiedStartColumn:l.modifiedRange.startColumn,modifiedEndLineNumber:l.modifiedRange.endLineNumber,modifiedEndColumn:l.modifiedRange.endColumn}))}})}var CW=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Vt=function(o,e){return function(t,i){e(t,i,o)}};let Vve=0,gV=!1;function zve(o){if(!o){if(gV)return;gV=!0}qle(o||Rt.document.body)}let xx=class extends Zw{constructor(e,t,i,n,s,r,a,l,c,d,h,u,g){const f={...t};f.ariaLabel=f.ariaLabel||P2.editorViewAccessibleLabel,super(e,f,{},i,n,s,r,c,d,h,u,g),l instanceof j_?this._standaloneKeybindingService=l:this._standaloneKeybindingService=null,zve(f.ariaContainerElement),ade((p,_)=>i.createInstance(M_,p,_,{})),lde(a)}addCommand(e,t,i){if(!this._standaloneKeybindingService)return console.warn("Cannot add command because the editor is configured with an unrecognized KeybindingService"),null;const n="DYNAMIC_"+ ++Vve,s=G.deserialize(i);return this._standaloneKeybindingService.addDynamicKeybinding(n,e,t,s),n}createContextKey(e,t){return this._contextKeyService.createKey(e,t)}addAction(e){if(typeof e.id!="string"||typeof e.label!="string"||typeof e.run!="function")throw new Error("Invalid action descriptor, `id`, `label` and `run` are required properties!");if(!this._standaloneKeybindingService)return console.warn("Cannot add keybinding because the editor is configured with an unrecognized KeybindingService"),F.None;const t=e.id,i=e.label,n=G.and(G.equals("editorId",this.getId()),G.deserialize(e.precondition)),s=e.keybindings,r=G.and(n,G.deserialize(e.keybindingContext)),a=e.contextMenuGroupId||null,l=e.contextMenuOrder||0,c=(g,...f)=>Promise.resolve(e.run(this,...f)),d=new K,h=this.getId()+":"+t;if(d.add(rt.registerCommand(h,c)),a){const g={command:{id:h,title:i},when:n,group:a,order:l};d.add(rn.appendMenuItem(me.EditorContext,g))}if(Array.isArray(s))for(const g of s)d.add(this._standaloneKeybindingService.addDynamicKeybinding(h,g,c,r));const u=new Tq(h,i,i,void 0,n,(...g)=>Promise.resolve(e.run(this,...g)),this._contextKeyService);return this._actions.set(t,u),d.add(ve(()=>{this._actions.delete(t)})),d}_triggerCommand(e,t){if(this._codeEditorService instanceof sL)try{this._codeEditorService.setActiveCodeEditor(this),super._triggerCommand(e,t)}finally{this._codeEditorService.setActiveCodeEditor(null)}else super._triggerCommand(e,t)}};xx=CW([Vt(2,we),Vt(3,pt),Vt(4,Ht),Vt(5,Ee),Vt(6,Yl),Vt(7,yt),Vt(8,ji),Vt(9,Ei),Vt(10,po),Vt(11,di),Vt(12,ge)],xx);let zP=class extends xx{constructor(e,t,i,n,s,r,a,l,c,d,h,u,g,f,p,_){const b={...t};ex(h,b,!1);const C=c.registerEditorContainer(e);typeof b.theme=="string"&&c.setTheme(b.theme),typeof b.autoDetectHighContrast<"u"&&c.setAutoDetectHighContrast(!!b.autoDetectHighContrast);const v=b.model;delete b.model,super(e,b,i,n,s,r,a,l,c,d,u,p,_),this._configurationService=h,this._standaloneThemeService=c,this._register(C);let w;if(typeof v>"u"){const S=f.getLanguageIdByMimeType(b.language)||b.language||ms;w=$q(g,f,b.value||"",S,void 0),this._ownsModel=!0}else w=v,this._ownsModel=!1;if(this._attachModel(w),w){const S={oldModelUrl:null,newModelUrl:w.uri};this._onDidChangeModel.fire(S)}}dispose(){super.dispose()}updateOptions(e){ex(this._configurationService,e,!1),typeof e.theme=="string"&&this._standaloneThemeService.setTheme(e.theme),typeof e.autoDetectHighContrast<"u"&&this._standaloneThemeService.setAutoDetectHighContrast(!!e.autoDetectHighContrast),super.updateOptions(e)}_postDetachModelCleanup(e){super._postDetachModelCleanup(e),e&&this._ownsModel&&(e.dispose(),this._ownsModel=!1)}};zP=CW([Vt(2,we),Vt(3,pt),Vt(4,Ht),Vt(5,Ee),Vt(6,Yl),Vt(7,yt),Vt(8,Qo),Vt(9,Ei),Vt(10,qe),Vt(11,po),Vt(12,Kt),Vt(13,ni),Vt(14,di),Vt(15,ge)],zP);let UP=class extends od{constructor(e,t,i,n,s,r,a,l,c,d,h,u){const g={...t};ex(l,g,!0);const f=r.registerEditorContainer(e);typeof g.theme=="string"&&r.setTheme(g.theme),typeof g.autoDetectHighContrast<"u"&&r.setAutoDetectHighContrast(!!g.autoDetectHighContrast),super(e,g,{},n,i,s,u,d),this._configurationService=l,this._standaloneThemeService=r,this._register(f)}dispose(){super.dispose()}updateOptions(e){ex(this._configurationService,e,!0),typeof e.theme=="string"&&this._standaloneThemeService.setTheme(e.theme),typeof e.autoDetectHighContrast<"u"&&this._standaloneThemeService.setAutoDetectHighContrast(!!e.autoDetectHighContrast),super.updateOptions(e)}_createInnerEditor(e,t,i){return e.createInstance(xx,t,i)}getOriginalEditor(){return super.getOriginalEditor()}getModifiedEditor(){return super.getModifiedEditor()}addCommand(e,t,i){return this.getModifiedEditor().addCommand(e,t,i)}createContextKey(e,t){return this.getModifiedEditor().createContextKey(e,t)}addAction(e){return this.getModifiedEditor().addAction(e)}};UP=CW([Vt(2,we),Vt(3,Ee),Vt(4,pt),Vt(5,Qo),Vt(6,Ei),Vt(7,qe),Vt(8,mo),Vt(9,nu),Vt(10,md),Vt(11,ru)],UP);function $q(o,e,t,i,n){if(t=t||"",!i){const s=t.indexOf(`
`);let r=t;return s!==-1&&(r=t.substring(0,s)),fV(o,t,e.createByFilepathOrFirstLine(n||null,r),n)}return fV(o,t,e.createById(i),n)}function fV(o,e,t,i){return o.createModel(e,t,i)}var Uve=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},mV=function(o,e){return function(t,i){e(t,i,o)}};class $ve{constructor(e,t){this.viewModel=e,this.deltaScrollVertical=t}getId(){return this.viewModel}}let kx=class extends F{constructor(e,t,i,n,s){super(),this._container=e,this._overflowWidgetsDomNode=t,this._workbenchUIElementFactory=i,this._instantiationService=n,this._viewModel=Ke(this,void 0),this._collapsed=be(this,l=>{var c;return(c=this._viewModel.read(l))==null?void 0:c.collapsed.read(l)}),this._editorContentHeight=Ke(this,500),this.contentHeight=be(this,l=>(this._collapsed.read(l)?0:this._editorContentHeight.read(l))+this._outerEditorHeight),this._modifiedContentWidth=Ke(this,0),this._modifiedWidth=Ke(this,0),this._originalContentWidth=Ke(this,0),this._originalWidth=Ke(this,0),this.maxScroll=be(this,l=>{const c=this._modifiedContentWidth.read(l)-this._modifiedWidth.read(l),d=this._originalContentWidth.read(l)-this._originalWidth.read(l);return c>d?{maxScroll:c,width:this._modifiedWidth.read(l)}:{maxScroll:d,width:this._originalWidth.read(l)}}),this._elements=Xe("div.multiDiffEntry",[Xe("div.header@header",[Xe("div.header-content",[Xe("div.collapse-button@collapseButton"),Xe("div.file-path",[Xe("div.title.modified.show-file-icons@primaryPath",[]),Xe("div.status.deleted@status",["R"]),Xe("div.title.original.show-file-icons@secondaryPath",[])]),Xe("div.actions@actions")])]),Xe("div.editorParent",[Xe("div.editorContainer@editor")])]),this.editor=this._register(this._instantiationService.createInstance(od,this._elements.editor,{overflowWidgetsDomNode:this._overflowWidgetsDomNode},{})),this.isModifedFocused=no(this.editor.getModifiedEditor()).isFocused,this.isOriginalFocused=no(this.editor.getOriginalEditor()).isFocused,this.isFocused=be(this,l=>this.isModifedFocused.read(l)||this.isOriginalFocused.read(l)),this._resourceLabel=this._workbenchUIElementFactory.createResourceLabel?this._register(this._workbenchUIElementFactory.createResourceLabel(this._elements.primaryPath)):void 0,this._resourceLabel2=this._workbenchUIElementFactory.createResourceLabel?this._register(this._workbenchUIElementFactory.createResourceLabel(this._elements.secondaryPath)):void 0,this._dataStore=this._register(new K),this._headerHeight=40,this._lastScrollTop=-1,this._isSettingScrollTop=!1;const r=new FL(this._elements.collapseButton,{});this._register(Ae(l=>{r.element.className="",r.icon=this._collapsed.read(l)?te.chevronRight:te.chevronDown})),this._register(r.onDidClick(()=>{var l;(l=this._viewModel.get())==null||l.collapsed.set(!this._collapsed.get(),void 0)})),this._register(Ae(l=>{this._elements.editor.style.display=this._collapsed.read(l)?"none":"block"})),this._register(this.editor.getModifiedEditor().onDidLayoutChange(l=>{const c=this.editor.getModifiedEditor().getLayoutInfo().contentWidth;this._modifiedWidth.set(c,void 0)})),this._register(this.editor.getOriginalEditor().onDidLayoutChange(l=>{const c=this.editor.getOriginalEditor().getLayoutInfo().contentWidth;this._originalWidth.set(c,void 0)})),this._register(this.editor.onDidContentSizeChange(l=>{kC(c=>{this._editorContentHeight.set(l.contentHeight,c),this._modifiedContentWidth.set(this.editor.getModifiedEditor().getContentWidth(),c),this._originalContentWidth.set(this.editor.getOriginalEditor().getContentWidth(),c)})})),this._register(this.editor.getOriginalEditor().onDidScrollChange(l=>{if(this._isSettingScrollTop||!l.scrollTopChanged||!this._data)return;const c=l.scrollTop-this._lastScrollTop;this._data.deltaScrollVertical(c)})),this._register(Ae(l=>{var d;const c=(d=this._viewModel.read(l))==null?void 0:d.isActive.read(l);this._elements.root.classList.toggle("active",c)})),this._container.appendChild(this._elements.root),this._outerEditorHeight=this._headerHeight,this._contextKeyService=this._register(s.createScoped(this._elements.actions));const a=this._register(this._instantiationService.createChild(new Eb([Ee,this._contextKeyService])));this._register(a.createInstance(Sx,this._elements.actions,me.MultiDiffEditorFileToolbar,{actionRunner:this._register(new Vq(()=>{var l;return(l=this._viewModel.get())==null?void 0:l.modifiedUri})),menuOptions:{shouldForwardArgs:!0},toolbarOptions:{primaryGroup:l=>l.startsWith("navigation")},actionViewItemProvider:(l,c)=>Fj(a,l,c)}))}setScrollLeft(e){this._modifiedContentWidth.get()-this._modifiedWidth.get()>this._originalContentWidth.get()-this._originalWidth.get()?this.editor.getModifiedEditor().setScrollLeft(e):this.editor.getOriginalEditor().setScrollLeft(e)}setData(e){this._data=e;function t(n){return{...n,scrollBeyondLastLine:!1,hideUnchangedRegions:{enabled:!0},scrollbar:{vertical:"hidden",horizontal:"hidden",handleMouseWheel:!1,useShadows:!1},renderOverviewRuler:!1,fixedOverflowWidgets:!0,overviewRulerBorder:!1}}if(!e){kC(n=>{this._viewModel.set(void 0,n),this.editor.setDiffModel(null,n),this._dataStore.clear()});return}const i=e.viewModel.documentDiffItem;if(kC(n=>{var c,d;(c=this._resourceLabel)==null||c.setUri(e.viewModel.modifiedUri??e.viewModel.originalUri,{strikethrough:e.viewModel.modifiedUri===void 0});let s=!1,r=!1,a=!1,l="";e.viewModel.modifiedUri&&e.viewModel.originalUri&&e.viewModel.modifiedUri.path!==e.viewModel.originalUri.path?(l="R",s=!0):e.viewModel.modifiedUri?e.viewModel.originalUri||(l="A",a=!0):(l="D",r=!0),this._elements.status.classList.toggle("renamed",s),this._elements.status.classList.toggle("deleted",r),this._elements.status.classList.toggle("added",a),this._elements.status.innerText=l,(d=this._resourceLabel2)==null||d.setUri(s?e.viewModel.originalUri:void 0,{strikethrough:!0}),this._dataStore.clear(),this._viewModel.set(e.viewModel,n),this.editor.setDiffModel(e.viewModel.diffEditorViewModelRef,n),this.editor.updateOptions(t(i.options??{}))}),i.onOptionsDidChange&&this._dataStore.add(i.onOptionsDidChange(()=>{this.editor.updateOptions(t(i.options??{}))})),e.viewModel.isAlive.recomputeInitiallyAndOnChange(this._dataStore,n=>{n||this.setData(void 0)}),e.viewModel.documentDiffItem.contextKeys)for(const[n,s]of Object.entries(e.viewModel.documentDiffItem.contextKeys))this._contextKeyService.createKey(n,s)}render(e,t,i,n){this._elements.root.style.visibility="visible",this._elements.root.style.top=`${e.start}px`,this._elements.root.style.height=`${e.length}px`,this._elements.root.style.width=`${t}px`,this._elements.root.style.position="absolute";const s=e.length-this._headerHeight,r=Math.max(0,Math.min(n.start-e.start,s));this._elements.header.style.transform=`translateY(${r}px)`,kC(a=>{this.editor.layout({width:t-2*8-2*1,height:e.length-this._outerEditorHeight})});try{this._isSettingScrollTop=!0,this._lastScrollTop=i,this.editor.getOriginalEditor().setScrollTop(i)}finally{this._isSettingScrollTop=!1}this._elements.header.classList.toggle("shadow",r>0||i>0),this._elements.header.classList.toggle("collapsed",r===s)}hide(){this._elements.root.style.top="-100000px",this._elements.root.style.visibility="hidden"}};kx=Uve([mV(3,we),mV(4,Ee)],kx);class jve{constructor(e){this._create=e,this._unused=new Set,this._used=new Set,this._itemData=new Map}getUnusedObj(e){let t;if(this._unused.size===0)t=this._create(e),this._itemData.set(t,e);else{const i=[...this._unused.values()];t=i.find(n=>this._itemData.get(n).getId()===e.getId())??i[0],this._unused.delete(t),this._itemData.set(t,e),t.setData(e)}return this._used.add(t),{object:t,dispose:()=>{this._used.delete(t),this._unused.size>5?t.dispose():this._unused.add(t)}}}dispose(){for(const e of this._used)e.dispose();for(const e of this._unused)e.dispose();this._used.clear(),this._unused.clear()}}var Kve=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},pV=function(o,e){return function(t,i){e(t,i,o)}};let $P=class extends F{constructor(e,t,i,n,s,r){super(),this._element=e,this._dimension=t,this._viewModel=i,this._workbenchUIElementFactory=n,this._parentContextKeyService=s,this._parentInstantiationService=r,this._scrollableElements=Xe("div.scrollContent",[Xe("div@content",{style:{overflow:"hidden"}}),Xe("div.monaco-editor@overflowWidgetsDomNode",{})]),this._scrollable=this._register(new wb({forceIntegerValues:!1,scheduleAtNextAnimationFrame:l=>go(xe(this._element),l),smoothScrollDuration:100})),this._scrollableElement=this._register(new jI(this._scrollableElements.root,{vertical:1,horizontal:1,useShadows:!1},this._scrollable)),this._elements=Xe("div.monaco-component.multiDiffEditor",{},[Xe("div",{},[this._scrollableElement.getDomNode()]),Xe("div.placeholder@placeholder",{},[Xe("div",[m("noChangedFiles","No Changed Files")])])]),this._sizeObserver=this._register(new Pq(this._element,void 0)),this._objectPool=this._register(new jve(l=>{const c=this._instantiationService.createInstance(kx,this._scrollableElements.content,this._scrollableElements.overflowWidgetsDomNode,this._workbenchUIElementFactory);return c.setData(l),c})),this.scrollTop=xt(this,this._scrollableElement.onScroll,()=>this._scrollableElement.getScrollPosition().scrollTop),this.scrollLeft=xt(this,this._scrollableElement.onScroll,()=>this._scrollableElement.getScrollPosition().scrollLeft),this._viewItemsInfo=su(this,(l,c)=>{const d=this._viewModel.read(l);if(!d)return{items:[],getItem:f=>{throw new bt}};const h=d.items.read(l),u=new Map;return{items:h.map(f=>{var b;const p=c.add(new qve(f,this._objectPool,this.scrollLeft,C=>{this._scrollableElement.setScrollPosition({scrollTop:this._scrollableElement.getScrollPosition().scrollTop+C})})),_=(b=this._lastDocStates)==null?void 0:b[p.getKey()];return _&&ai(C=>{p.setViewState(_,C)}),u.set(f,p),p}),getItem:f=>u.get(f)}}),this._viewItems=this._viewItemsInfo.map(this,l=>l.items),this._spaceBetweenPx=0,this._totalHeight=this._viewItems.map(this,(l,c)=>l.reduce((d,h)=>d+h.contentHeight.read(c)+this._spaceBetweenPx,0)),this._contextKeyService=this._register(this._parentContextKeyService.createScoped(this._element)),this._instantiationService=this._register(this._parentInstantiationService.createChild(new Eb([Ee,this._contextKeyService]))),this._lastDocStates={},this._contextKeyService.createKey(I.inMultiDiffEditor.key,!0),this._register(Ps((l,c)=>{const d=this._viewModel.read(l);if(d&&d.contextKeys)for(const[h,u]of Object.entries(d.contextKeys)){const g=this._contextKeyService.createKey(h,void 0);g.set(u),c.add(ve(()=>g.reset()))}}));const a=this._parentContextKeyService.createKey(I.multiDiffEditorAllCollapsed.key,!1);this._register(Ae(l=>{const c=this._viewModel.read(l);if(c){const d=c.items.read(l).every(h=>h.collapsed.read(l));a.set(d)}})),this._register(Ae(l=>{const c=this._dimension.read(l);this._sizeObserver.observe(c)})),this._register(Ae(l=>{const c=this._viewItems.read(l);this._elements.placeholder.classList.toggle("visible",c.length===0)})),this._scrollableElements.content.style.position="relative",this._register(Ae(l=>{const c=this._sizeObserver.height.read(l);this._scrollableElements.root.style.height=`${c}px`;const d=this._totalHeight.read(l);this._scrollableElements.content.style.height=`${d}px`;const h=this._sizeObserver.width.read(l);let u=h;const g=this._viewItems.read(l),f=C5(g,gs(p=>p.maxScroll.read(l).maxScroll,$r));if(f){const p=f.maxScroll.read(l);u=h+p.maxScroll}this._scrollableElement.setScrollDimensions({width:h,height:c,scrollHeight:d,scrollWidth:u})})),e.replaceChildren(this._elements.root),this._register(ve(()=>{e.replaceChildren()})),this._register(this._register(Ae(l=>{kC(c=>{this.render(l)})})))}render(e){const t=this.scrollTop.read(e);let i=0,n=0,s=0;const r=this._sizeObserver.height.read(e),a=Ge.ofStartAndLength(t,r),l=this._sizeObserver.width.read(e);for(const c of this._viewItems.read(e)){const d=c.contentHeight.read(e),h=Math.min(d,r),u=Ge.ofStartAndLength(n,h),g=Ge.ofStartAndLength(s,d);if(g.isBefore(a))i-=d-h,c.hide();else if(g.isAfter(a))c.hide();else{const f=Math.max(0,Math.min(a.start-g.start,d-h));i-=f;const p=Ge.ofStartAndLength(t+i,r);c.render(u,f,l,p)}n+=h+this._spaceBetweenPx,s+=d+this._spaceBetweenPx}this._scrollableElements.content.style.transform=`translateY(${-(t+i)}px)`}};$P=Kve([pV(4,Ee),pV(5,we)],$P);class qve extends F{constructor(e,t,i,n){super(),this.viewModel=e,this._objectPool=t,this._scrollLeft=i,this._deltaScrollVertical=n,this._templateRef=this._register(Dw(this,void 0)),this.contentHeight=be(this,s=>{var r,a;return((a=(r=this._templateRef.read(s))==null?void 0:r.object.contentHeight)==null?void 0:a.read(s))??this.viewModel.lastTemplateData.read(s).contentHeight}),this.maxScroll=be(this,s=>{var r;return((r=this._templateRef.read(s))==null?void 0:r.object.maxScroll.read(s))??{maxScroll:0,scrollWidth:0}}),this.template=be(this,s=>{var r;return(r=this._templateRef.read(s))==null?void 0:r.object}),this._isHidden=Ke(this,!1),this._isFocused=be(this,s=>{var r;return((r=this.template.read(s))==null?void 0:r.isFocused.read(s))??!1}),this.viewModel.setIsFocused(this._isFocused,void 0),this._register(Ae(s=>{var a;const r=this._scrollLeft.read(s);(a=this._templateRef.read(s))==null||a.object.setScrollLeft(r)})),this._register(Ae(s=>{const r=this._templateRef.read(s);!r||!this._isHidden.read(s)||r.object.isFocused.read(s)||this._clear()}))}dispose(){this._clear(),super.dispose()}toString(){var e;return`VirtualViewItem(${(e=this.viewModel.documentDiffItem.modified)==null?void 0:e.uri.toString()})`}getKey(){return this.viewModel.getKey()}setViewState(e,t){var r;this.viewModel.collapsed.set(e.collapsed,t),this._updateTemplateData(t);const i=this.viewModel.lastTemplateData.get(),n=(r=e.selections)==null?void 0:r.map(pe.liftSelection);this.viewModel.lastTemplateData.set({...i,selections:n},t);const s=this._templateRef.get();s&&n&&s.object.editor.setSelections(n)}_updateTemplateData(e){const t=this._templateRef.get();t&&this.viewModel.lastTemplateData.set({contentHeight:t.object.contentHeight.get(),selections:t.object.editor.getSelections()??void 0},e)}_clear(){const e=this._templateRef.get();e&&ai(t=>{this._updateTemplateData(t),e.object.hide(),this._templateRef.set(void 0,t)})}hide(){this._isHidden.set(!0,void 0)}render(e,t,i,n){this._isHidden.set(!1,void 0);let s=this._templateRef.get();if(!s){s=this._objectPool.getUnusedObj(new $ve(this.viewModel,this._deltaScrollVertical)),this._templateRef.set(s,void 0);const r=this.viewModel.lastTemplateData.get().selections;r&&s.object.editor.setSelections(r)}s.object.render(e,i,t,n)}}E("multiDiffEditor.headerBackground",{dark:"#262626",light:"tab.inactiveBackground",hcDark:"tab.inactiveBackground",hcLight:"tab.inactiveBackground"},m("multiDiffEditor.headerBackground","The background color of the diff editor's header"));E("multiDiffEditor.background",Xr,m("multiDiffEditor.background","The background color of the multi file diff editor"));E("multiDiffEditor.border",{dark:"sideBarSectionHeader.border",light:"#cccccc",hcDark:"sideBarSectionHeader.border",hcLight:"#cccccc"},m("multiDiffEditor.border","The border color of the multi file diff editor"));var Gve=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Zve=function(o,e){return function(t,i){e(t,i,o)}};let jP=class extends F{constructor(e,t,i){super(),this._element=e,this._workbenchUIElementFactory=t,this._instantiationService=i,this._dimension=Ke(this,void 0),this._viewModel=Ke(this,void 0),this._widgetImpl=su(this,(n,s)=>(Js(kx,n),s.add(this._instantiationService.createInstance(Js($P,n),this._element,this._dimension,this._viewModel,this._workbenchUIElementFactory)))),this._register(xb(this._widgetImpl))}};jP=Gve([Zve(2,we)],jP);function Yve(o,e,t){return Re.initialize(t||{}).createInstance(zP,o,e)}function Xve(o){return Re.get(pt).onCodeEditorAdd(t=>{o(t)})}function Qve(o){return Re.get(pt).onDiffEditorAdd(t=>{o(t)})}function Jve(){return Re.get(pt).listCodeEditors()}function ewe(){return Re.get(pt).listDiffEditors()}function twe(o,e,t){return Re.initialize(t||{}).createInstance(UP,o,e)}function iwe(o,e){const t=Re.initialize(e||{});return new jP(o,{},t)}function nwe(o){if(typeof o.id!="string"||typeof o.run!="function")throw new Error("Invalid command descriptor, `id` and `run` are required properties!");return rt.registerCommand(o.id,o.run)}function swe(o){if(typeof o.id!="string"||typeof o.label!="string"||typeof o.run!="function")throw new Error("Invalid action descriptor, `id`, `label` and `run` are required properties!");const e=G.deserialize(o.precondition),t=(n,...s)=>$i.runEditorCommand(n,s,e,(r,a,l)=>Promise.resolve(o.run(a,...l))),i=new K;if(i.add(rt.registerCommand(o.id,t)),o.contextMenuGroupId){const n={command:{id:o.id,title:o.label},when:e,group:o.contextMenuGroupId,order:o.contextMenuOrder||0};i.add(rn.appendMenuItem(me.EditorContext,n))}if(Array.isArray(o.keybindings)){const n=Re.get(yt);if(!(n instanceof j_))console.warn("Cannot add keybinding because the editor is configured with an unrecognized KeybindingService");else{const s=G.and(e,G.deserialize(o.keybindingContext));i.add(n.addDynamicKeybindings(o.keybindings.map(r=>({keybinding:r,command:o.id,when:s}))))}}return i}function owe(o){return jq([o])}function jq(o){const e=Re.get(yt);return e instanceof j_?e.addDynamicKeybindings(o.map(t=>({keybinding:t.keybinding,command:t.command,commandArgs:t.commandArgs,when:G.deserialize(t.when)}))):(console.warn("Cannot add keybinding because the editor is configured with an unrecognized KeybindingService"),F.None)}function rwe(o,e,t){const i=Re.get(ni),n=i.getLanguageIdByMimeType(e)||e;return $q(Re.get(Kt),i,o,n,t)}function awe(o,e){const t=Re.get(ni),i=t.getLanguageIdByMimeType(e)||e||ms;o.setLanguage(t.createById(i))}function lwe(o,e,t){o&&Re.get(Xl).changeOne(e,o.uri,t)}function cwe(o){Re.get(Xl).changeAll(o,[])}function dwe(o){return Re.get(Xl).read(o)}function hwe(o){return Re.get(Xl).onMarkerChanged(o)}function uwe(o){return Re.get(Kt).getModel(o)}function gwe(){return Re.get(Kt).getModels()}function fwe(o){return Re.get(Kt).onModelAdded(o)}function mwe(o){return Re.get(Kt).onModelRemoved(o)}function pwe(o){return Re.get(Kt).onModelLanguageChanged(t=>{o({model:t.model,oldLanguage:t.oldLanguageId})})}function _we(o){return ipe(Re.get(Kt),o)}function bwe(o,e){const t=Re.get(ni),i=Re.get(Qo);return H3.colorizeElement(i,t,o,e).then(()=>{i.registerEditorContainer(o)})}function Cwe(o,e,t){const i=Re.get(ni);return Re.get(Qo).registerEditorContainer(Rt.document.body),H3.colorize(i,o,e,t)}function vwe(o,e,t=4){return Re.get(Qo).registerEditorContainer(Rt.document.body),H3.colorizeModelLine(o,e,t)}function wwe(o){const e=fi.get(o);return e||{getInitialState:()=>F_,tokenize:(t,i,n)=>Z5(o,n)}}function Swe(o,e){fi.getOrCreate(e);const t=wwe(e),i=Kl(o),n=[];let s=t.getInitialState();for(let r=0,a=i.length;r<a;r++){const l=i[r],c=t.tokenize(l,!0,s);n[r]=c.tokens,s=c.endState}return n}function ywe(o,e){Re.get(Qo).defineTheme(o,e)}function Lwe(o){Re.get(Qo).setTheme(o)}function xwe(){RM.clearAllFontInfos()}function kwe(o,e){return rt.registerCommand({id:o,handler:e})}function Dwe(o){return Re.get(zs).registerOpener({async open(t){return typeof t=="string"&&(t=Se.parse(t)),o.open(t)}})}function Iwe(o){return Re.get(pt).registerCodeEditorOpenHandler(async(t,i,n)=>{var a;if(!i)return null;const s=(a=t.options)==null?void 0:a.selection;let r;return s&&typeof s.endLineNumber=="number"&&typeof s.endColumn=="number"?r=s:s&&(r={lineNumber:s.startLineNumber,column:s.startColumn}),await o.openCodeEditor(i,t.resource,r)?i:null})}function Ewe(){return{create:Yve,getEditors:Jve,getDiffEditors:ewe,onDidCreateEditor:Xve,onDidCreateDiffEditor:Qve,createDiffEditor:twe,addCommand:nwe,addEditorAction:swe,addKeybindingRule:owe,addKeybindingRules:jq,createModel:rwe,setModelLanguage:awe,setModelMarkers:lwe,getModelMarkers:dwe,removeAllMarkers:cwe,onDidChangeMarkers:hwe,getModels:gwe,getModel:uwe,onDidCreateModel:fwe,onWillDisposeModel:mwe,onDidChangeModelLanguage:pwe,createWebWorker:_we,colorizeElement:bwe,colorize:Cwe,colorizeModelLine:vwe,tokenize:Swe,defineTheme:ywe,setTheme:Lwe,remeasureFonts:xwe,registerCommand:kwe,registerLinkOpener:Dwe,registerEditorOpener:Iwe,AccessibilitySupport:DR,ContentWidgetPositionPreference:MR,CursorChangeReason:AR,DefaultEndOfLine:PR,EditorAutoIndentStrategy:FR,EditorOption:WR,EndOfLinePreference:BR,EndOfLineSequence:HR,MinimapPosition:XR,MinimapSectionHeaderStyle:QR,MouseTargetType:JR,OverlayWidgetPositionPreference:iM,OverviewRulerLane:nM,GlyphMarginLane:VR,RenderLineNumbersType:rM,RenderMinimap:aM,ScrollbarVisibility:cM,ScrollType:lM,TextEditorCursorBlinkingStyle:mM,TextEditorCursorStyle:pM,TrackedRangeStickiness:_M,WrappingIndent:bM,InjectedTextCursorStops:$R,PositionAffinity:oM,ShowLightbulbIconMode:hM,ConfigurationChangedEvent:Az,BareFontInfo:mg,FontInfo:TM,TextModelResolvedOptions:$0,FindMatch:cw,ApplyUpdateResult:$C,EditorZoom:lr,createMultiFileDiffEditor:iwe,EditorType:iS,EditorOptions:Ua}}function Nwe(o,e){if(!e||!Array.isArray(e))return!1;for(const t of e)if(!o(t))return!1;return!0}function f0(o,e){return typeof o=="boolean"?o:e}function _V(o,e){return typeof o=="string"?o:e}function Twe(o){const e={};for(const t of o)e[t]=!0;return e}function bV(o,e=!1){e&&(o=o.map(function(i){return i.toLowerCase()}));const t=Twe(o);return e?function(i){return t[i.toLowerCase()]!==void 0&&t.hasOwnProperty(i.toLowerCase())}:function(i){return t[i]!==void 0&&t.hasOwnProperty(i)}}function KP(o,e,t){e=e.replace(/@@/g,"");let i=0,n;do n=!1,e=e.replace(/@(\w+)/g,function(r,a){n=!0;let l="";if(typeof o[a]=="string")l=o[a];else if(o[a]&&o[a]instanceof RegExp)l=o[a].source;else throw o[a]===void 0?Zt(o,"language definition does not contain attribute '"+a+"', used at: "+e):Zt(o,"attribute reference '"+a+"' must be a string, used at: "+e);return Zu(l)?"":"(?:"+l+")"}),i++;while(n&&i<5);e=e.replace(/\x01/g,"@");const s=(o.ignoreCase?"i":"")+(o.unicode?"u":"");if(t&&e.match(/\$[sS](\d\d?)/g)){let a=null,l=null;return c=>(l&&a===c||(a=c,l=new RegExp(wpe(o,e,c),s)),l)}return new RegExp(e,s)}function Rwe(o,e,t,i){if(i<0)return o;if(i<e.length)return e[i];if(i>=100){i=i-100;const n=t.split(".");if(n.unshift(t),i<n.length)return n[i]}return null}function Mwe(o,e,t,i){let n=-1,s=t,r=t.match(/^\$(([sS]?)(\d\d?)|#)(.*)$/);r&&(r[3]&&(n=parseInt(r[3]),r[2]&&(n=n+100)),s=r[4]);let a="~",l=s;!s||s.length===0?(a="!=",l=""):/^\w*$/.test(l)?a="==":(r=s.match(/^(@|!@|~|!~|==|!=)(.*)$/),r&&(a=r[1],l=r[2]));let c;if((a==="~"||a==="!~")&&/^(\w|\|)*$/.test(l)){const d=bV(l.split("|"),o.ignoreCase);c=function(h){return a==="~"?d(h):!d(h)}}else if(a==="@"||a==="!@"){const d=o[l];if(!d)throw Zt(o,"the @ match target '"+l+"' is not defined, in rule: "+e);if(!Nwe(function(u){return typeof u=="string"},d))throw Zt(o,"the @ match target '"+l+"' must be an array of strings, in rule: "+e);const h=bV(d,o.ignoreCase);c=function(u){return a==="@"?h(u):!h(u)}}else if(a==="~"||a==="!~")if(l.indexOf("$")<0){const d=KP(o,"^"+l+"$",!1);c=function(h){return a==="~"?d.test(h):!d.test(h)}}else c=function(d,h,u,g){return KP(o,"^"+Hd(o,l,h,u,g)+"$",!1).test(d)};else if(l.indexOf("$")<0){const d=qc(o,l);c=function(h){return a==="=="?h===d:h!==d}}else{const d=qc(o,l);c=function(h,u,g,f,p){const _=Hd(o,d,u,g,f);return a==="=="?h===_:h!==_}}return n===-1?{name:t,value:i,test:function(d,h,u,g){return c(d,d,h,u,g)}}:{name:t,value:i,test:function(d,h,u,g){const f=Rwe(d,h,u,n);return c(f||"",d,h,u,g)}}}function qP(o,e,t){if(t){if(typeof t=="string")return t;if(t.token||t.token===""){if(typeof t.token!="string")throw Zt(o,"a 'token' attribute must be of type string, in rule: "+e);{const i={token:t.token};if(t.token.indexOf("$")>=0&&(i.tokenSubst=!0),typeof t.bracket=="string")if(t.bracket==="@open")i.bracket=1;else if(t.bracket==="@close")i.bracket=-1;else throw Zt(o,"a 'bracket' attribute must be either '@open' or '@close', in rule: "+e);if(t.next){if(typeof t.next!="string")throw Zt(o,"the next state must be a string value in rule: "+e);{let n=t.next;if(!/^(@pop|@push|@popall)$/.test(n)&&(n[0]==="@"&&(n=n.substr(1)),n.indexOf("$")<0&&!Spe(o,Hd(o,n,"",[],""))))throw Zt(o,"the next state '"+t.next+"' is not defined in rule: "+e);i.next=n}}return typeof t.goBack=="number"&&(i.goBack=t.goBack),typeof t.switchTo=="string"&&(i.switchTo=t.switchTo),typeof t.log=="string"&&(i.log=t.log),typeof t.nextEmbedded=="string"&&(i.nextEmbedded=t.nextEmbedded,o.usesEmbedded=!0),i}}else if(Array.isArray(t)){const i=[];for(let n=0,s=t.length;n<s;n++)i[n]=qP(o,e,t[n]);return{group:i}}else if(t.cases){const i=[];for(const s in t.cases)if(t.cases.hasOwnProperty(s)){const r=qP(o,e,t.cases[s]);s==="@default"||s==="@"||s===""?i.push({test:void 0,value:r,name:s}):s==="@eos"?i.push({test:function(a,l,c,d){return d},value:r,name:s}):i.push(Mwe(o,e,s,r))}const n=o.defaultToken;return{test:function(s,r,a,l){for(const c of i)if(!c.test||c.test(s,r,a,l))return c.value;return n}}}else throw Zt(o,"an action must be a string, an object with a 'token' or 'cases' attribute, or an array of actions; in rule: "+e)}else return{token:""}}class Awe{constructor(e){this.regex=new RegExp(""),this.action={token:""},this.matchOnlyAtLineStart=!1,this.name="",this.name=e}setRegex(e,t){let i;if(typeof t=="string")i=t;else if(t instanceof RegExp)i=t.source;else throw Zt(e,"rules must start with a match string or regular expression: "+this.name);this.matchOnlyAtLineStart=i.length>0&&i[0]==="^",this.name=this.name+": "+i,this.regex=KP(e,"^(?:"+(this.matchOnlyAtLineStart?i.substr(1):i)+")",!0)}setAction(e,t){this.action=qP(e,this.name,t)}resolveRegex(e){return this.regex instanceof RegExp?this.regex:this.regex(e)}}function Kq(o,e){if(!e||typeof e!="object")throw new Error("Monarch: expecting a language definition object");const t={languageId:o,includeLF:f0(e.includeLF,!1),noThrow:!1,maxStack:100,start:typeof e.start=="string"?e.start:null,ignoreCase:f0(e.ignoreCase,!1),unicode:f0(e.unicode,!1),tokenPostfix:_V(e.tokenPostfix,"."+o),defaultToken:_V(e.defaultToken,"source"),usesEmbedded:!1,stateNames:{},tokenizer:{},brackets:[]},i=e;i.languageId=o,i.includeLF=t.includeLF,i.ignoreCase=t.ignoreCase,i.unicode=t.unicode,i.noThrow=t.noThrow,i.usesEmbedded=t.usesEmbedded,i.stateNames=e.tokenizer,i.defaultToken=t.defaultToken;function n(r,a,l){for(const c of l){let d=c.include;if(d){if(typeof d!="string")throw Zt(t,"an 'include' attribute must be a string at: "+r);if(d[0]==="@"&&(d=d.substr(1)),!e.tokenizer[d])throw Zt(t,"include target '"+d+"' is not defined at: "+r);n(r+"."+d,a,e.tokenizer[d])}else{const h=new Awe(r);if(Array.isArray(c)&&c.length>=1&&c.length<=3)if(h.setRegex(i,c[0]),c.length>=3)if(typeof c[1]=="string")h.setAction(i,{token:c[1],next:c[2]});else if(typeof c[1]=="object"){const u=c[1];u.next=c[2],h.setAction(i,u)}else throw Zt(t,"a next state as the last element of a rule can only be given if the action is either an object or a string, at: "+r);else h.setAction(i,c[1]);else{if(!c.regex)throw Zt(t,"a rule must either be an array, or an object with a 'regex' or 'include' field at: "+r);c.name&&typeof c.name=="string"&&(h.name=c.name),c.matchOnlyAtStart&&(h.matchOnlyAtLineStart=f0(c.matchOnlyAtLineStart,!1)),h.setRegex(i,c.regex),h.setAction(i,c.action)}a.push(h)}}}if(!e.tokenizer||typeof e.tokenizer!="object")throw Zt(t,"a language definition must define the 'tokenizer' attribute as an object");t.tokenizer=[];for(const r in e.tokenizer)if(e.tokenizer.hasOwnProperty(r)){t.start||(t.start=r);const a=e.tokenizer[r];t.tokenizer[r]=new Array,n("tokenizer."+r,t.tokenizer[r],a)}if(t.usesEmbedded=i.usesEmbedded,e.brackets){if(!Array.isArray(e.brackets))throw Zt(t,"the 'brackets' attribute must be defined as an array")}else e.brackets=[{open:"{",close:"}",token:"delimiter.curly"},{open:"[",close:"]",token:"delimiter.square"},{open:"(",close:")",token:"delimiter.parenthesis"},{open:"<",close:">",token:"delimiter.angle"}];const s=[];for(const r of e.brackets){let a=r;if(a&&Array.isArray(a)&&a.length===3&&(a={token:a[2],open:a[0],close:a[1]}),a.open===a.close)throw Zt(t,"open and close brackets in a 'brackets' attribute must be different: "+a.open+`
 hint: use the 'bracket' attribute if matching on equal brackets is required.`);if(typeof a.open=="string"&&typeof a.token=="string"&&typeof a.close=="string")s.push({token:a.token+t.tokenPostfix,open:qc(t,a.open),close:qc(t,a.close)});else throw Zt(t,"every element in the 'brackets' array must be a '{open,close,token}' object or array")}return t.brackets=s,t.noThrow=!0,t}function Pwe(o){I_.registerLanguage(o)}function Owe(){let o=[];return o=o.concat(I_.getLanguages()),o}function Fwe(o){return Re.get(ni).languageIdCodec.encodeLanguageId(o)}function Wwe(o,e){return Re.withServices(()=>{const i=Re.get(ni).onDidRequestRichLanguageFeatures(n=>{n===o&&(i.dispose(),e())});return i})}function Bwe(o,e){return Re.withServices(()=>{const i=Re.get(ni).onDidRequestBasicLanguageFeatures(n=>{n===o&&(i.dispose(),e())});return i})}function Hwe(o,e){if(!Re.get(ni).isRegisteredLanguageId(o))throw new Error(`Cannot set configuration for unknown language ${o}`);return Re.get(di).register(o,e,100)}class Vwe{constructor(e,t){this._languageId=e,this._actual=t}dispose(){}getInitialState(){return this._actual.getInitialState()}tokenize(e,t,i){if(typeof this._actual.tokenize=="function")return e1.adaptTokenize(this._languageId,this._actual,e,i);throw new Error("Not supported!")}tokenizeEncoded(e,t,i){const n=this._actual.tokenizeEncoded(e,i);return new gI(n.tokens,n.endState)}}class e1{constructor(e,t,i,n){this._languageId=e,this._actual=t,this._languageService=i,this._standaloneThemeService=n}dispose(){}getInitialState(){return this._actual.getInitialState()}static _toClassicTokens(e,t){const i=[];let n=0;for(let s=0,r=e.length;s<r;s++){const a=e[s];let l=a.startIndex;s===0?l=0:l<n&&(l=n),i[s]=new ew(l,a.scopes,t),n=l}return i}static adaptTokenize(e,t,i,n){const s=t.tokenize(i,n),r=e1._toClassicTokens(s.tokens,e);let a;return s.endState.equals(n)?a=n:a=s.endState,new G4(r,a)}tokenize(e,t,i){return e1.adaptTokenize(this._languageId,this._actual,e,i)}_toBinaryTokens(e,t){const i=e.encodeLanguageId(this._languageId),n=this._standaloneThemeService.getColorTheme().tokenTheme,s=[];let r=0,a=0;for(let c=0,d=t.length;c<d;c++){const h=t[c],u=n.match(i,h.scopes)|1024;if(r>0&&s[r-1]===u)continue;let g=h.startIndex;c===0?g=0:g<a&&(g=a),s[r++]=g,s[r++]=u,a=g}const l=new Uint32Array(r);for(let c=0;c<r;c++)l[c]=s[c];return l}tokenizeEncoded(e,t,i){const n=this._actual.tokenize(e,i),s=this._toBinaryTokens(this._languageService.languageIdCodec,n.tokens);let r;return n.endState.equals(i)?r=i:r=n.endState,new gI(s,r)}}function zwe(o){return typeof o.getInitialState=="function"}function Uwe(o){return"tokenizeEncoded"in o}function qq(o){return o&&typeof o.then=="function"}function $we(o){const e=Re.get(Qo);if(o){const t=[null];for(let i=1,n=o.length;i<n;i++)t[i]=Z.fromHex(o[i]);e.setColorMapOverride(t)}else e.setColorMapOverride(null)}function Gq(o,e){return Uwe(e)?new Vwe(o,e):new e1(o,e,Re.get(ni),Re.get(Qo))}function vW(o,e){const t=new yee(async()=>{const i=await Promise.resolve(e.create());return i?zwe(i)?Gq(o,i):new zw(Re.get(ni),Re.get(Qo),o,Kq(o,i),Re.get(qe)):null});return fi.registerFactory(o,t)}function jwe(o,e){if(!Re.get(ni).isRegisteredLanguageId(o))throw new Error(`Cannot set tokens provider for unknown language ${o}`);return qq(e)?vW(o,{create:()=>e}):fi.register(o,Gq(o,e))}function Kwe(o,e){const t=i=>new zw(Re.get(ni),Re.get(Qo),o,Kq(o,i),Re.get(qe));return qq(e)?vW(o,{create:()=>e}):fi.register(o,t(e))}function qwe(o,e){return Re.get(ge).referenceProvider.register(o,e)}function Gwe(o,e){return Re.get(ge).renameProvider.register(o,e)}function Zwe(o,e){return Re.get(ge).newSymbolNamesProvider.register(o,e)}function Ywe(o,e){return Re.get(ge).signatureHelpProvider.register(o,e)}function Xwe(o,e){return Re.get(ge).hoverProvider.register(o,{provideHover:async(i,n,s,r)=>{const a=i.getWordAtPosition(n);return Promise.resolve(e.provideHover(i,n,s,r)).then(l=>{if(l)return!l.range&&a&&(l.range=new x(n.lineNumber,a.startColumn,n.lineNumber,a.endColumn)),l.range||(l.range=new x(n.lineNumber,n.column,n.lineNumber,n.column)),l})}})}function Qwe(o,e){return Re.get(ge).documentSymbolProvider.register(o,e)}function Jwe(o,e){return Re.get(ge).documentHighlightProvider.register(o,e)}function e1e(o,e){return Re.get(ge).linkedEditingRangeProvider.register(o,e)}function t1e(o,e){return Re.get(ge).definitionProvider.register(o,e)}function i1e(o,e){return Re.get(ge).implementationProvider.register(o,e)}function n1e(o,e){return Re.get(ge).typeDefinitionProvider.register(o,e)}function s1e(o,e){return Re.get(ge).codeLensProvider.register(o,e)}function o1e(o,e,t){return Re.get(ge).codeActionProvider.register(o,{providedCodeActionKinds:t==null?void 0:t.providedCodeActionKinds,documentation:t==null?void 0:t.documentation,provideCodeActions:(n,s,r,a)=>{const c=Re.get(Xl).read({resource:n.uri}).filter(d=>x.areIntersectingOrTouching(d,s));return e.provideCodeActions(n,s,{markers:c,only:r.only,trigger:r.trigger},a)},resolveCodeAction:e.resolveCodeAction})}function r1e(o,e){return Re.get(ge).documentFormattingEditProvider.register(o,e)}function a1e(o,e){return Re.get(ge).documentRangeFormattingEditProvider.register(o,e)}function l1e(o,e){return Re.get(ge).onTypeFormattingEditProvider.register(o,e)}function c1e(o,e){return Re.get(ge).linkProvider.register(o,e)}function d1e(o,e){return Re.get(ge).completionProvider.register(o,e)}function h1e(o,e){return Re.get(ge).colorProvider.register(o,e)}function u1e(o,e){return Re.get(ge).foldingRangeProvider.register(o,e)}function g1e(o,e){return Re.get(ge).declarationProvider.register(o,e)}function f1e(o,e){return Re.get(ge).selectionRangeProvider.register(o,e)}function m1e(o,e){return Re.get(ge).documentSemanticTokensProvider.register(o,e)}function p1e(o,e){return Re.get(ge).documentRangeSemanticTokensProvider.register(o,e)}function _1e(o,e){return Re.get(ge).inlineCompletionsProvider.register(o,e)}function b1e(o,e){return Re.get(ge).inlineEditProvider.register(o,e)}function C1e(o,e){return Re.get(ge).inlayHintsProvider.register(o,e)}function v1e(){return{register:Pwe,getLanguages:Owe,onLanguage:Wwe,onLanguageEncountered:Bwe,getEncodedLanguageId:Fwe,setLanguageConfiguration:Hwe,setColorMap:$we,registerTokensProviderFactory:vW,setTokensProvider:jwe,setMonarchTokensProvider:Kwe,registerReferenceProvider:qwe,registerRenameProvider:Gwe,registerNewSymbolNameProvider:Zwe,registerCompletionItemProvider:d1e,registerSignatureHelpProvider:Ywe,registerHoverProvider:Xwe,registerDocumentSymbolProvider:Qwe,registerDocumentHighlightProvider:Jwe,registerLinkedEditingRangeProvider:e1e,registerDefinitionProvider:t1e,registerImplementationProvider:i1e,registerTypeDefinitionProvider:n1e,registerCodeLensProvider:s1e,registerCodeActionProvider:o1e,registerDocumentFormattingEditProvider:r1e,registerDocumentRangeFormattingEditProvider:a1e,registerOnTypeFormattingEditProvider:l1e,registerLinkProvider:c1e,registerColorProvider:h1e,registerFoldingRangeProvider:u1e,registerDeclarationProvider:g1e,registerSelectionRangeProvider:f1e,registerDocumentSemanticTokensProvider:m1e,registerDocumentRangeSemanticTokensProvider:p1e,registerInlineCompletionsProvider:_1e,registerInlineEditProvider:b1e,registerInlayHintsProvider:C1e,DocumentHighlightKind:OR,CompletionItemKind:NR,CompletionItemTag:TR,CompletionItemInsertTextRule:ER,SymbolKind:gM,SymbolTag:fM,IndentAction:UR,CompletionTriggerKind:RR,SignatureHelpTriggerKind:uM,InlayHintKind:jR,InlineCompletionTriggerKind:KR,InlineEditTriggerKind:qR,CodeActionTriggerType:IR,NewSymbolNameTag:eM,NewSymbolNameTriggerKind:tM,PartialAcceptTriggerKind:sM,HoverVerbosityAction:zR,FoldingRangeKind:Ph,SelectedSuggestionInfo:Yz}}const wW=et("IEditorCancelService"),Zq=new le("cancellableOperation",!1,m("cancellableOperation","Whether the editor runs a cancellable operation, e.g. like 'Peek References'"));st(wW,class{constructor(){this._tokens=new WeakMap}add(o,e){let t=this._tokens.get(o);t||(t=o.invokeWithinContext(n=>{const s=Zq.bindTo(n.get(Ee)),r=new cs;return{key:s,tokens:r}}),this._tokens.set(o,t));let i;return t.key.set(!0),i=t.tokens.push(e),()=>{i&&(i(),t.key.set(!t.tokens.isEmpty()),i=void 0)}}cancel(o){const e=this._tokens.get(o);if(!e)return;const t=e.tokens.pop();t&&(t.cancel(),e.key.set(!e.tokens.isEmpty()))}},1);class w1e extends li{constructor(e,t){super(t),this.editor=e,this._unregister=e.invokeWithinContext(i=>i.get(wW).add(e,this))}dispose(){this._unregister(),super.dispose()}}re(new class extends $i{constructor(){super({id:"editor.cancelOperation",kbOpts:{weight:100,primary:9},precondition:Zq})}runEditorCommand(o,e){o.get(wW).cancel(e)}});let Yq=class GP{constructor(e,t){if(this.flags=t,(this.flags&1)!==0){const i=e.getModel();this.modelVersionId=i?xf("{0}#{1}",i.uri.toString(),i.getVersionId()):null}else this.modelVersionId=null;(this.flags&4)!==0?this.position=e.getPosition():this.position=null,(this.flags&2)!==0?this.selection=e.getSelection():this.selection=null,(this.flags&8)!==0?(this.scrollLeft=e.getScrollLeft(),this.scrollTop=e.getScrollTop()):(this.scrollLeft=-1,this.scrollTop=-1)}_equals(e){if(!(e instanceof GP))return!1;const t=e;return!(this.modelVersionId!==t.modelVersionId||this.scrollLeft!==t.scrollLeft||this.scrollTop!==t.scrollTop||!this.position&&t.position||this.position&&!t.position||this.position&&t.position&&!this.position.equals(t.position)||!this.selection&&t.selection||this.selection&&!t.selection||this.selection&&t.selection&&!this.selection.equalsRange(t.selection))}validate(e){return this._equals(new GP(e,this.flags))}};class Zh extends w1e{constructor(e,t,i,n){super(e,n),this._listener=new K,t&4&&this._listener.add(e.onDidChangeCursorPosition(s=>{(!i||!x.containsPosition(i,s.position))&&this.cancel()})),t&2&&this._listener.add(e.onDidChangeCursorSelection(s=>{(!i||!x.containsRange(i,s.selection))&&this.cancel()})),t&8&&this._listener.add(e.onDidScrollChange(s=>this.cancel())),t&1&&(this._listener.add(e.onDidChangeModel(s=>this.cancel())),this._listener.add(e.onDidChangeModelContent(s=>this.cancel())))}dispose(){this._listener.dispose(),super.dispose()}}class SW extends li{constructor(e,t){super(t),this._listener=e.onDidChangeContent(()=>this.cancel())}dispose(){this._listener.dispose(),super.dispose()}}function jl(o){return o&&typeof o.getEditorType=="function"?o.getEditorType()===iS.ICodeEditor:!1}function yW(o){return o&&typeof o.getEditorType=="function"?o.getEditorType()===iS.IDiffEditor:!1}function S1e(o){return!!o&&typeof o=="object"&&typeof o.onDidChangeActiveEditor=="function"}function Xq(o){return jl(o)?o:yW(o)?o.getModifiedEditor():S1e(o)&&jl(o.activeCodeEditor)?o.activeCodeEditor:null}class Z_{static _handleEolEdits(e,t){let i;const n=[];for(const s of t)typeof s.eol=="number"&&(i=s.eol),s.range&&typeof s.text=="string"&&n.push(s);return typeof i=="number"&&e.hasModel()&&e.getModel().pushEOL(i),n}static _isFullModelReplaceEdit(e,t){if(!e.hasModel())return!1;const i=e.getModel(),n=i.validateRange(t.range);return i.getFullModelRange().equalsRange(n)}static execute(e,t,i){i&&e.pushUndoStop();const n=za.capture(e),s=Z_._handleEolEdits(e,t);s.length===1&&Z_._isFullModelReplaceEdit(e,s[0])?e.executeEdits("formatEditsCommand",s.map(r=>ri.replace(x.lift(r.range),r.text))):e.executeEdits("formatEditsCommand",s.map(r=>ri.replaceMove(x.lift(r.range),r.text))),i&&e.pushUndoStop(),n.restoreRelativeVerticalPositionOfCursor(e)}}class CV{constructor(e){this.value=e,this._lower=e.toLowerCase()}static toKey(e){return typeof e=="string"?e.toLowerCase():e._lower}}class y1e{constructor(e){if(this._set=new Set,e)for(const t of e)this.add(t)}add(e){this._set.add(CV.toKey(e))}has(e){return this._set.has(CV.toKey(e))}}function Qq(o,e,t){const i=[],n=new y1e,s=o.ordered(t);for(const a of s)i.push(a),a.extensionId&&n.add(a.extensionId);const r=e.ordered(t);for(const a of r){if(a.extensionId){if(n.has(a.extensionId))continue;n.add(a.extensionId)}i.push({displayName:a.displayName,extensionId:a.extensionId,provideDocumentFormattingEdits(l,c,d){return a.provideDocumentRangeFormattingEdits(l,l.getFullModelRange(),c,d)}})}return i}const Uv=class Uv{static setFormatterSelector(e){return{dispose:Uv._selectors.unshift(e)}}static async select(e,t,i,n){if(e.length===0)return;const s=ot.first(Uv._selectors);if(s)return await s(e,t,i,n)}};Uv._selectors=new cs;let t1=Uv;async function Jq(o,e,t,i,n,s,r){const a=o.get(we),{documentRangeFormattingEditProvider:l}=o.get(ge),c=jl(e)?e.getModel():e,d=l.ordered(c),h=await t1.select(d,c,i,2);h&&(n.report(h),await a.invokeFunction(L1e,h,e,t,s,r))}async function L1e(o,e,t,i,n,s){var b,C;const r=o.get(Us),a=o.get(Nn),l=o.get(ru);let c,d;jl(t)?(c=t.getModel(),d=new Zh(t,5,void 0,n)):(c=t,d=new SW(t,n));const h=[];let u=0;for(const v of F4(i).sort(x.compareRangesUsingStarts))u>0&&x.areIntersectingOrTouching(h[u-1],v)?h[u-1]=x.fromPositions(h[u-1].getStartPosition(),v.getEndPosition()):u=h.push(v);const g=async v=>{var S,L;a.trace("[format][provideDocumentRangeFormattingEdits] (request)",(S=e.extensionId)==null?void 0:S.value,v);const w=await e.provideDocumentRangeFormattingEdits(c,v,c.getFormattingOptions(),d.token)||[];return a.trace("[format][provideDocumentRangeFormattingEdits] (response)",(L=e.extensionId)==null?void 0:L.value,w),w},f=(v,w)=>{if(!v.length||!w.length)return!1;const S=v.reduce((L,k)=>x.plusRange(L,k.range),v[0].range);if(!w.some(L=>x.intersectRanges(S,L.range)))return!1;for(const L of v)for(const k of w)if(x.intersectRanges(L.range,k.range))return!0;return!1},p=[],_=[];try{if(typeof e.provideDocumentRangesFormattingEdits=="function"){a.trace("[format][provideDocumentRangeFormattingEdits] (request)",(b=e.extensionId)==null?void 0:b.value,h);const v=await e.provideDocumentRangesFormattingEdits(c,h,c.getFormattingOptions(),d.token)||[];a.trace("[format][provideDocumentRangeFormattingEdits] (response)",(C=e.extensionId)==null?void 0:C.value,v),_.push(v)}else{for(const v of h){if(d.token.isCancellationRequested)return!0;_.push(await g(v))}for(let v=0;v<h.length;++v)for(let w=v+1;w<h.length;++w){if(d.token.isCancellationRequested)return!0;if(f(_[v],_[w])){const S=x.plusRange(h[v],h[w]),L=await g(S);h.splice(w,1),h.splice(v,1),h.push(S),_.splice(w,1),_.splice(v,1),_.push(L),v=0,w=0}}}for(const v of _){if(d.token.isCancellationRequested)return!0;const w=await r.computeMoreMinimalEdits(c.uri,v);w&&p.push(...w)}}finally{d.dispose()}if(p.length===0)return!1;if(jl(t))Z_.execute(t,p,!0),t.revealPositionInCenterIfOutsideViewport(t.getPosition(),1);else{const[{range:v}]=p,w=new pe(v.startLineNumber,v.startColumn,v.endLineNumber,v.endColumn);c.pushEditOperations([w],p.map(S=>({text:S.text,range:x.lift(S.range),forceMoveMarkers:!0})),S=>{for(const{range:L}of S)if(x.areIntersectingOrTouching(L,w))return[new pe(L.startLineNumber,L.startColumn,L.endLineNumber,L.endColumn)];return null})}return l.playSignal(No.format,{userGesture:s}),!0}async function x1e(o,e,t,i,n,s){const r=o.get(we),a=o.get(ge),l=jl(e)?e.getModel():e,c=Qq(a.documentFormattingEditProvider,a.documentRangeFormattingEditProvider,l),d=await t1.select(c,l,t,1);d&&(i.report(d),await r.invokeFunction(k1e,d,e,t,n,s))}async function k1e(o,e,t,i,n,s){const r=o.get(Us),a=o.get(ru);let l,c;jl(t)?(l=t.getModel(),c=new Zh(t,5,void 0,n)):(l=t,c=new SW(t,n));let d;try{const h=await e.provideDocumentFormattingEdits(l,l.getFormattingOptions(),c.token);if(d=await r.computeMoreMinimalEdits(l.uri,h),c.token.isCancellationRequested)return!0}finally{c.dispose()}if(!d||d.length===0)return!1;if(jl(t))Z_.execute(t,d,i!==2),i!==2&&t.revealPositionInCenterIfOutsideViewport(t.getPosition(),1);else{const[{range:h}]=d,u=new pe(h.startLineNumber,h.startColumn,h.endLineNumber,h.endColumn);l.pushEditOperations([u],d.map(g=>({text:g.text,range:x.lift(g.range),forceMoveMarkers:!0})),g=>{for(const{range:f}of g)if(x.areIntersectingOrTouching(f,u))return[new pe(f.startLineNumber,f.startColumn,f.endLineNumber,f.endColumn)];return null})}return a.playSignal(No.format,{userGesture:s}),!0}async function D1e(o,e,t,i,n,s){const r=e.documentRangeFormattingEditProvider.ordered(t);for(const a of r){const l=await Promise.resolve(a.provideDocumentRangeFormattingEdits(t,i,n,s)).catch(Si);if(Qn(l))return await o.computeMoreMinimalEdits(t.uri,l)}}async function I1e(o,e,t,i,n){const s=Qq(e.documentFormattingEditProvider,e.documentRangeFormattingEditProvider,t);for(const r of s){const a=await Promise.resolve(r.provideDocumentFormattingEdits(t,i,n)).catch(Si);if(Qn(a))return await o.computeMoreMinimalEdits(t.uri,a)}}function eG(o,e,t,i,n,s,r){const a=e.onTypeFormattingEditProvider.ordered(t);return a.length===0||a[0].autoFormatTriggerCharacters.indexOf(n)<0?Promise.resolve(void 0):Promise.resolve(a[0].provideOnTypeFormattingEdits(t,i,n,s,r)).catch(Si).then(l=>o.computeMoreMinimalEdits(t.uri,l))}rt.registerCommand("_executeFormatRangeProvider",async function(o,...e){const[t,i,n]=e;mt(Se.isUri(t)),mt(x.isIRange(i));const s=o.get(Hs),r=o.get(Us),a=o.get(ge),l=await s.createModelReference(t);try{return D1e(r,a,l.object.textEditorModel,x.lift(i),n,ze.None)}finally{l.dispose()}});rt.registerCommand("_executeFormatDocumentProvider",async function(o,...e){const[t,i]=e;mt(Se.isUri(t));const n=o.get(Hs),s=o.get(Us),r=o.get(ge),a=await n.createModelReference(t);try{return I1e(s,r,a.object.textEditorModel,i,ze.None)}finally{a.dispose()}});rt.registerCommand("_executeFormatOnTypeProvider",async function(o,...e){const[t,i,n,s]=e;mt(Se.isUri(t)),mt(A.isIPosition(i)),mt(typeof n=="string");const r=o.get(Hs),a=o.get(Us),l=o.get(ge),c=await r.createModelReference(t);try{return eG(a,l,c.object.textEditorModel,A.lift(i),n,s,ze.None)}finally{c.dispose()}});Ua.wrappingIndent.defaultValue=0;Ua.glyphMargin.defaultValue=!1;Ua.autoIndent.defaultValue=3;Ua.overviewRulerLanes.defaultValue=2;t1.setFormatterSelector((o,e,t)=>Promise.resolve(o[0]));const ys=Xz();ys.editor=Ewe();ys.languages=v1e();const tG=ys.CancellationTokenSource,iG=ys.Emitter,nG=ys.KeyCode,sG=ys.KeyMod,oG=ys.Position,rG=ys.Range,aG=ys.Selection,lG=ys.SelectionDirection,cG=ys.MarkerSeverity,dG=ys.MarkerTag,hG=ys.Uri,uG=ys.Token,gG=ys.editor,fG=ys.languages,NT=globalThis.MonacoEnvironment;(NT!=null&&NT.globalAPI||typeof define=="function"&&define.amd)&&(globalThis.monaco=ys);typeof globalThis.require<"u"&&typeof globalThis.require.config=="function"&&globalThis.require.config({ignoreDuplicateModules:["vscode-languageserver-types","vscode-languageserver-types/main","vscode-languageserver-textdocument","vscode-languageserver-textdocument/main","vscode-nls","vscode-nls/vscode-nls","jsonc-parser","jsonc-parser/main","vscode-uri","vscode-uri/index","vs/basic-languages/typescript/typescript"]});const rS=Object.freeze(Object.defineProperty({__proto__:null,CancellationTokenSource:tG,Emitter:iG,KeyCode:nG,KeyMod:sG,MarkerSeverity:cG,MarkerTag:dG,Position:oG,Range:rG,Selection:aG,SelectionDirection:lG,Token:uG,Uri:hG,editor:gG,languages:fG},Symbol.toStringTag,{value:"Module"}));/*!-----------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Version: 0.52.2(404545bded1df6ffa41ea0af4e8ddb219018c6c1)
 * Released under the MIT license
 * https://github.com/microsoft/monaco-editor/blob/main/LICENSE.txt
 *-----------------------------------------------------------------------------*/var E1e=Object.defineProperty,N1e=Object.getOwnPropertyDescriptor,T1e=Object.getOwnPropertyNames,R1e=Object.prototype.hasOwnProperty,M1e=(o,e,t,i)=>{if(e&&typeof e=="object"||typeof e=="function")for(let n of T1e(e))!R1e.call(o,n)&&n!==t&&E1e(o,n,{get:()=>e[n],enumerable:!(i=N1e(e,n))||i.enumerable});return o},A1e=(o,e,t)=>(M1e(o,e,"default"),t),RC={};A1e(RC,rS);var mG={},TT={},P1e=class pG{static getOrCreate(e){return TT[e]||(TT[e]=new pG(e)),TT[e]}constructor(e){this._languageId=e,this._loadingTriggered=!1,this._lazyLoadPromise=new Promise((t,i)=>{this._lazyLoadPromiseResolve=t,this._lazyLoadPromiseReject=i})}load(){return this._loadingTriggered||(this._loadingTriggered=!0,mG[this._languageId].loader().then(e=>this._lazyLoadPromiseResolve(e),e=>this._lazyLoadPromiseReject(e))),this._lazyLoadPromise}};function ke(o){const e=o.id;mG[e]=o,RC.languages.register(o);const t=P1e.getOrCreate(e);RC.languages.registerTokensProviderFactory(e,{create:async()=>(await t.load()).language}),RC.languages.onLanguageEncountered(e,async()=>{const i=await t.load();RC.languages.setLanguageConfiguration(e,i.conf)})}ke({id:"abap",extensions:[".abap"],aliases:["abap","ABAP"],loader:()=>ye(()=>import("./abap-BrgZPUOV.js"),[],import.meta.url)});ke({id:"apex",extensions:[".cls"],aliases:["Apex","apex"],mimetypes:["text/x-apex-source","text/x-apex"],loader:()=>ye(()=>import("./apex-DyP6w7ZV.js"),[],import.meta.url)});ke({id:"azcli",extensions:[".azcli"],aliases:["Azure CLI","azcli"],loader:()=>ye(()=>import("./azcli-BaLxmfj-.js"),[],import.meta.url)});ke({id:"bat",extensions:[".bat",".cmd"],aliases:["Batch","bat"],loader:()=>ye(()=>import("./bat-CFOPXBzS.js"),[],import.meta.url)});ke({id:"bicep",extensions:[".bicep"],aliases:["Bicep"],loader:()=>ye(()=>import("./bicep-BfEKNvv3.js"),[],import.meta.url)});ke({id:"cameligo",extensions:[".mligo"],aliases:["Cameligo"],loader:()=>ye(()=>import("./cameligo-BFG1Mk7z.js"),[],import.meta.url)});ke({id:"clojure",extensions:[".clj",".cljs",".cljc",".edn"],aliases:["clojure","Clojure"],loader:()=>ye(()=>import("./clojure-DTECt2xU.js"),[],import.meta.url)});ke({id:"coffeescript",extensions:[".coffee"],aliases:["CoffeeScript","coffeescript","coffee"],mimetypes:["text/x-coffeescript","text/coffeescript"],loader:()=>ye(()=>import("./coffee-CDGzqUPQ.js"),[],import.meta.url)});ke({id:"c",extensions:[".c",".h"],aliases:["C","c"],loader:()=>ye(()=>import("./cpp-CLLBncYj.js"),[],import.meta.url)});ke({id:"cpp",extensions:[".cpp",".cc",".cxx",".hpp",".hh",".hxx"],aliases:["C++","Cpp","cpp"],loader:()=>ye(()=>import("./cpp-CLLBncYj.js"),[],import.meta.url)});ke({id:"csharp",extensions:[".cs",".csx",".cake"],aliases:["C#","csharp"],loader:()=>ye(()=>import("./csharp-dUCx_-0o.js"),[],import.meta.url)});ke({id:"csp",extensions:[".csp"],aliases:["CSP","csp"],loader:()=>ye(()=>import("./csp-5Rap-vPy.js"),[],import.meta.url)});ke({id:"css",extensions:[".css"],aliases:["CSS","css"],mimetypes:["text/css"],loader:()=>ye(()=>import("./css-D3h14YRZ.js"),[],import.meta.url)});ke({id:"cypher",extensions:[".cypher",".cyp"],aliases:["Cypher","OpenCypher"],loader:()=>ye(()=>import("./cypher-DrQuvNYM.js"),[],import.meta.url)});ke({id:"dart",extensions:[".dart"],aliases:["Dart","dart"],mimetypes:["text/x-dart-source","text/x-dart"],loader:()=>ye(()=>import("./dart-CFKIUWau.js"),[],import.meta.url)});ke({id:"dockerfile",extensions:[".dockerfile"],filenames:["Dockerfile"],aliases:["Dockerfile"],loader:()=>ye(()=>import("./dockerfile-Zznr-cwX.js"),[],import.meta.url)});ke({id:"ecl",extensions:[".ecl"],aliases:["ECL","Ecl","ecl"],loader:()=>ye(()=>import("./ecl-Ce3n6wWz.js"),[],import.meta.url)});ke({id:"elixir",extensions:[".ex",".exs"],aliases:["Elixir","elixir","ex"],loader:()=>ye(()=>import("./elixir-deUWdS0T.js"),[],import.meta.url)});ke({id:"flow9",extensions:[".flow"],aliases:["Flow9","Flow","flow9","flow"],loader:()=>ye(()=>import("./flow9-i9-g7ZhI.js"),[],import.meta.url)});ke({id:"fsharp",extensions:[".fs",".fsi",".ml",".mli",".fsx",".fsscript"],aliases:["F#","FSharp","fsharp"],loader:()=>ye(()=>import("./fsharp-CzKuDChf.js"),[],import.meta.url)});ke({id:"freemarker2",extensions:[".ftl",".ftlh",".ftlx"],aliases:["FreeMarker2","Apache FreeMarker2"],loader:()=>ye(()=>import("./freemarker2-ipp5yHNJ.js"),__vite__mapDeps([0,1,2]),import.meta.url).then(o=>o.TagAutoInterpolationDollar)});ke({id:"freemarker2.tag-angle.interpolation-dollar",aliases:["FreeMarker2 (Angle/Dollar)","Apache FreeMarker2 (Angle/Dollar)"],loader:()=>ye(()=>import("./freemarker2-ipp5yHNJ.js"),__vite__mapDeps([0,1,2]),import.meta.url).then(o=>o.TagAngleInterpolationDollar)});ke({id:"freemarker2.tag-bracket.interpolation-dollar",aliases:["FreeMarker2 (Bracket/Dollar)","Apache FreeMarker2 (Bracket/Dollar)"],loader:()=>ye(()=>import("./freemarker2-ipp5yHNJ.js"),__vite__mapDeps([0,1,2]),import.meta.url).then(o=>o.TagBracketInterpolationDollar)});ke({id:"freemarker2.tag-angle.interpolation-bracket",aliases:["FreeMarker2 (Angle/Bracket)","Apache FreeMarker2 (Angle/Bracket)"],loader:()=>ye(()=>import("./freemarker2-ipp5yHNJ.js"),__vite__mapDeps([0,1,2]),import.meta.url).then(o=>o.TagAngleInterpolationBracket)});ke({id:"freemarker2.tag-bracket.interpolation-bracket",aliases:["FreeMarker2 (Bracket/Bracket)","Apache FreeMarker2 (Bracket/Bracket)"],loader:()=>ye(()=>import("./freemarker2-ipp5yHNJ.js"),__vite__mapDeps([0,1,2]),import.meta.url).then(o=>o.TagBracketInterpolationBracket)});ke({id:"freemarker2.tag-auto.interpolation-dollar",aliases:["FreeMarker2 (Auto/Dollar)","Apache FreeMarker2 (Auto/Dollar)"],loader:()=>ye(()=>import("./freemarker2-ipp5yHNJ.js"),__vite__mapDeps([0,1,2]),import.meta.url).then(o=>o.TagAutoInterpolationDollar)});ke({id:"freemarker2.tag-auto.interpolation-bracket",aliases:["FreeMarker2 (Auto/Bracket)","Apache FreeMarker2 (Auto/Bracket)"],loader:()=>ye(()=>import("./freemarker2-ipp5yHNJ.js"),__vite__mapDeps([0,1,2]),import.meta.url).then(o=>o.TagAutoInterpolationBracket)});ke({id:"go",extensions:[".go"],aliases:["Go"],loader:()=>ye(()=>import("./go-Cphgjts3.js"),[],import.meta.url)});ke({id:"graphql",extensions:[".graphql",".gql"],aliases:["GraphQL","graphql","gql"],mimetypes:["application/graphql"],loader:()=>ye(()=>import("./graphql-Cg7bfA9N.js"),[],import.meta.url)});ke({id:"handlebars",extensions:[".handlebars",".hbs"],aliases:["Handlebars","handlebars","hbs"],mimetypes:["text/x-handlebars-template"],loader:()=>ye(()=>import("./handlebars-CcWMc5kR.js"),__vite__mapDeps([3,1,2]),import.meta.url)});ke({id:"hcl",extensions:[".tf",".tfvars",".hcl"],aliases:["Terraform","tf","HCL","hcl"],loader:()=>ye(()=>import("./hcl-0cvrggvQ.js"),[],import.meta.url)});ke({id:"html",extensions:[".html",".htm",".shtml",".xhtml",".mdoc",".jsp",".asp",".aspx",".jshtm"],aliases:["HTML","htm","html","xhtml"],mimetypes:["text/html","text/x-jshtm","text/template","text/ng-template"],loader:()=>ye(()=>import("./html-DhzDJZot.js"),__vite__mapDeps([4,1,2]),import.meta.url)});ke({id:"ini",extensions:[".ini",".properties",".gitconfig"],filenames:["config",".gitattributes",".gitconfig",".editorconfig"],aliases:["Ini","ini"],loader:()=>ye(()=>import("./ini-Drc7WvVn.js"),[],import.meta.url)});ke({id:"java",extensions:[".java",".jav"],aliases:["Java","java"],mimetypes:["text/x-java-source","text/x-java"],loader:()=>ye(()=>import("./java-B_fMsGYe.js"),[],import.meta.url)});ke({id:"javascript",extensions:[".js",".es6",".jsx",".mjs",".cjs"],firstLine:"^#!.*\\bnode",filenames:["jakefile"],aliases:["JavaScript","javascript","js"],mimetypes:["text/javascript"],loader:()=>ye(()=>import("./javascript-vcrMQ9kK.js"),__vite__mapDeps([5,6,1,2]),import.meta.url)});ke({id:"julia",extensions:[".jl"],aliases:["julia","Julia"],loader:()=>ye(()=>import("./julia-Bqgm2twL.js"),[],import.meta.url)});ke({id:"kotlin",extensions:[".kt",".kts"],aliases:["Kotlin","kotlin"],mimetypes:["text/x-kotlin-source","text/x-kotlin"],loader:()=>ye(()=>import("./kotlin-BSkB5QuD.js"),[],import.meta.url)});ke({id:"less",extensions:[".less"],aliases:["Less","less"],mimetypes:["text/x-less","text/less"],loader:()=>ye(()=>import("./less-BsTHnhdd.js"),[],import.meta.url)});ke({id:"lexon",extensions:[".lex"],aliases:["Lexon"],loader:()=>ye(()=>import("./lexon-YWi4-JPR.js"),[],import.meta.url)});ke({id:"lua",extensions:[".lua"],aliases:["Lua","lua"],loader:()=>ye(()=>import("./lua-nf6ki56Z.js"),[],import.meta.url)});ke({id:"liquid",extensions:[".liquid",".html.liquid"],aliases:["Liquid","liquid"],mimetypes:["application/liquid"],loader:()=>ye(()=>import("./liquid-wPFVW8nU.js"),__vite__mapDeps([7,1,2]),import.meta.url)});ke({id:"m3",extensions:[".m3",".i3",".mg",".ig"],aliases:["Modula-3","Modula3","modula3","m3"],loader:()=>ye(()=>import("./m3-Cpb6xl2v.js"),[],import.meta.url)});ke({id:"markdown",extensions:[".md",".markdown",".mdown",".mkdn",".mkd",".mdwn",".mdtxt",".mdtext"],aliases:["Markdown","markdown"],loader:()=>ye(()=>import("./markdown-DSZPf7rp.js"),[],import.meta.url)});ke({id:"mdx",extensions:[".mdx"],aliases:["MDX","mdx"],loader:()=>ye(()=>import("./mdx-D3eN1vrz.js"),__vite__mapDeps([8,1,2]),import.meta.url)});ke({id:"mips",extensions:[".s"],aliases:["MIPS","MIPS-V"],mimetypes:["text/x-mips","text/mips","text/plaintext"],loader:()=>ye(()=>import("./mips-B_c3zf-v.js"),[],import.meta.url)});ke({id:"msdax",extensions:[".dax",".msdax"],aliases:["DAX","MSDAX"],loader:()=>ye(()=>import("./msdax-rUNN04Wq.js"),[],import.meta.url)});ke({id:"mysql",extensions:[],aliases:["MySQL","mysql"],loader:()=>ye(()=>import("./mysql-DDwshQtU.js"),[],import.meta.url)});ke({id:"objective-c",extensions:[".m"],aliases:["Objective-C"],loader:()=>ye(()=>import("./objective-c-B5zXfXm9.js"),[],import.meta.url)});ke({id:"pascal",extensions:[".pas",".p",".pp"],aliases:["Pascal","pas"],mimetypes:["text/x-pascal-source","text/x-pascal"],loader:()=>ye(()=>import("./pascal-CXOwvkN_.js"),[],import.meta.url)});ke({id:"pascaligo",extensions:[".ligo"],aliases:["Pascaligo","ligo"],loader:()=>ye(()=>import("./pascaligo-Bc-ZgV77.js"),[],import.meta.url)});ke({id:"perl",extensions:[".pl",".pm"],aliases:["Perl","pl"],loader:()=>ye(()=>import("./perl-CwNk8-XU.js"),[],import.meta.url)});ke({id:"pgsql",extensions:[],aliases:["PostgreSQL","postgres","pg","postgre"],loader:()=>ye(()=>import("./pgsql-tGk8EFnU.js"),[],import.meta.url)});ke({id:"php",extensions:[".php",".php4",".php5",".phtml",".ctp"],aliases:["PHP","php"],mimetypes:["application/x-php"],loader:()=>ye(()=>import("./php-CpIb_Oan.js"),[],import.meta.url)});ke({id:"pla",extensions:[".pla"],loader:()=>ye(()=>import("./pla-B03wrqEc.js"),[],import.meta.url)});ke({id:"postiats",extensions:[".dats",".sats",".hats"],aliases:["ATS","ATS/Postiats"],loader:()=>ye(()=>import("./postiats-BKlk5iyT.js"),[],import.meta.url)});ke({id:"powerquery",extensions:[".pq",".pqm"],aliases:["PQ","M","Power Query","Power Query M"],loader:()=>ye(()=>import("./powerquery-Bhzvs7bI.js"),[],import.meta.url)});ke({id:"powershell",extensions:[".ps1",".psm1",".psd1"],aliases:["PowerShell","powershell","ps","ps1"],loader:()=>ye(()=>import("./powershell-Dd3NCNK9.js"),[],import.meta.url)});ke({id:"proto",extensions:[".proto"],aliases:["protobuf","Protocol Buffers"],loader:()=>ye(()=>import("./protobuf-COyEY5Pt.js"),[],import.meta.url)});ke({id:"pug",extensions:[".jade",".pug"],aliases:["Pug","Jade","jade"],loader:()=>ye(()=>import("./pug-BaJupSGV.js"),[],import.meta.url)});ke({id:"python",extensions:[".py",".rpy",".pyw",".cpy",".gyp",".gypi"],aliases:["Python","py"],firstLine:"^#!/.*\\bpython[0-9.-]*\\b",loader:()=>ye(()=>import("./python-DG5AD6tt.js"),__vite__mapDeps([9,1,2]),import.meta.url)});ke({id:"qsharp",extensions:[".qs"],aliases:["Q#","qsharp"],loader:()=>ye(()=>import("./qsharp-DXyYeYxl.js"),[],import.meta.url)});ke({id:"r",extensions:[".r",".rhistory",".rmd",".rprofile",".rt"],aliases:["R","r"],loader:()=>ye(()=>import("./r-CdQndTaG.js"),[],import.meta.url)});ke({id:"razor",extensions:[".cshtml"],aliases:["Razor","razor"],mimetypes:["text/x-cshtml"],loader:()=>ye(()=>import("./razor-D9mSmZ6d.js"),__vite__mapDeps([10,1,2]),import.meta.url)});ke({id:"redis",extensions:[".redis"],aliases:["redis"],loader:()=>ye(()=>import("./redis-CVwtpugi.js"),[],import.meta.url)});ke({id:"redshift",extensions:[],aliases:["Redshift","redshift"],loader:()=>ye(()=>import("./redshift-25W9uPmb.js"),[],import.meta.url)});ke({id:"restructuredtext",extensions:[".rst"],aliases:["reStructuredText","restructuredtext"],loader:()=>ye(()=>import("./restructuredtext-DfzH4Xui.js"),[],import.meta.url)});ke({id:"ruby",extensions:[".rb",".rbx",".rjs",".gemspec",".pp"],filenames:["rakefile","Gemfile"],aliases:["Ruby","rb"],loader:()=>ye(()=>import("./ruby-Cp1zYvxS.js"),[],import.meta.url)});ke({id:"rust",extensions:[".rs",".rlib"],aliases:["Rust","rust"],loader:()=>ye(()=>import("./rust-D5C2fndG.js"),[],import.meta.url)});ke({id:"sb",extensions:[".sb"],aliases:["Small Basic","sb"],loader:()=>ye(()=>import("./sb-CDntyWJ8.js"),[],import.meta.url)});ke({id:"scala",extensions:[".scala",".sc",".sbt"],aliases:["Scala","scala","SBT","Sbt","sbt","Dotty","dotty"],mimetypes:["text/x-scala-source","text/x-scala","text/x-sbt","text/x-dotty"],loader:()=>ye(()=>import("./scala-BoFRg7Ot.js"),[],import.meta.url)});ke({id:"scheme",extensions:[".scm",".ss",".sch",".rkt"],aliases:["scheme","Scheme"],loader:()=>ye(()=>import("./scheme-Bio4gycK.js"),[],import.meta.url)});ke({id:"scss",extensions:[".scss"],aliases:["Sass","sass","scss"],mimetypes:["text/x-scss","text/scss"],loader:()=>ye(()=>import("./scss-4Ik7cdeQ.js"),[],import.meta.url)});ke({id:"shell",extensions:[".sh",".bash"],aliases:["Shell","sh"],loader:()=>ye(()=>import("./shell-CX-rkNHf.js"),[],import.meta.url)});ke({id:"sol",extensions:[".sol"],aliases:["sol","solidity","Solidity"],loader:()=>ye(()=>import("./solidity-Tw7wswEv.js"),[],import.meta.url)});ke({id:"aes",extensions:[".aes"],aliases:["aes","sophia","Sophia"],loader:()=>ye(()=>import("./sophia-C5WLch3f.js"),[],import.meta.url)});ke({id:"sparql",extensions:[".rq"],aliases:["sparql","SPARQL"],loader:()=>ye(()=>import("./sparql-DHaeiCBh.js"),[],import.meta.url)});ke({id:"sql",extensions:[".sql"],aliases:["SQL"],loader:()=>ye(()=>import("./sql-CCSDG5nI.js"),[],import.meta.url)});ke({id:"st",extensions:[".st",".iecst",".iecplc",".lc3lib",".TcPOU",".TcDUT",".TcGVL",".TcIO"],aliases:["StructuredText","scl","stl"],loader:()=>ye(()=>import("./st-pnP8ivHi.js"),[],import.meta.url)});ke({id:"swift",aliases:["Swift","swift"],extensions:[".swift"],mimetypes:["text/swift"],loader:()=>ye(()=>import("./swift-DwJ7jVG9.js"),[],import.meta.url)});ke({id:"systemverilog",extensions:[".sv",".svh"],aliases:["SV","sv","SystemVerilog","systemverilog"],loader:()=>ye(()=>import("./systemverilog-B9Xyijhd.js"),[],import.meta.url)});ke({id:"verilog",extensions:[".v",".vh"],aliases:["V","v","Verilog","verilog"],loader:()=>ye(()=>import("./systemverilog-B9Xyijhd.js"),[],import.meta.url)});ke({id:"tcl",extensions:[".tcl"],aliases:["tcl","Tcl","tcltk","TclTk","tcl/tk","Tcl/Tk"],loader:()=>ye(()=>import("./tcl-DnHyzjbg.js"),[],import.meta.url)});ke({id:"twig",extensions:[".twig"],aliases:["Twig","twig"],mimetypes:["text/x-twig"],loader:()=>ye(()=>import("./twig-CPajHgWi.js"),[],import.meta.url)});ke({id:"typescript",extensions:[".ts",".tsx",".cts",".mts"],aliases:["TypeScript","ts","typescript"],mimetypes:["text/typescript"],loader:()=>ye(()=>import("./typescript-B2FbScha.js"),__vite__mapDeps([6,1,2]),import.meta.url)});ke({id:"typespec",extensions:[".tsp"],aliases:["TypeSpec"],loader:()=>ye(()=>import("./typespec-D-MeaMDU.js"),[],import.meta.url)});ke({id:"vb",extensions:[".vb"],aliases:["Visual Basic","vb"],loader:()=>ye(()=>import("./vb-DgyLZaXg.js"),[],import.meta.url)});ke({id:"wgsl",extensions:[".wgsl"],aliases:["WebGPU Shading Language","WGSL","wgsl"],loader:()=>ye(()=>import("./wgsl-BIv9DU6q.js"),[],import.meta.url)});ke({id:"xml",extensions:[".xml",".xsd",".dtd",".ascx",".csproj",".config",".props",".targets",".wxi",".wxl",".wxs",".xaml",".svg",".svgz",".opf",".xslt",".xsl"],firstLine:"(\\<\\?xml.*)|(\\<svg)|(\\<\\!doctype\\s+svg)",aliases:["XML","xml"],mimetypes:["text/xml","application/xml","application/xaml+xml","application/xml-dtd"],loader:()=>ye(()=>import("./xml-DCi87pgG.js"),__vite__mapDeps([11,1,2]),import.meta.url)});ke({id:"yaml",extensions:[".yaml",".yml"],aliases:["YAML","yaml","YML","yml"],mimetypes:["application/x-yaml","text/x-yaml"],loader:()=>ye(()=>import("./yaml-BhpQBQ82.js"),__vite__mapDeps([12,1,2]),import.meta.url)});/*!-----------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Version: 0.52.2(404545bded1df6ffa41ea0af4e8ddb219018c6c1)
 * Released under the MIT license
 * https://github.com/microsoft/monaco-editor/blob/main/LICENSE.txt
 *-----------------------------------------------------------------------------*/var O1e=Object.defineProperty,F1e=Object.getOwnPropertyDescriptor,W1e=Object.getOwnPropertyNames,B1e=Object.prototype.hasOwnProperty,H1e=(o,e,t,i)=>{if(e&&typeof e=="object"||typeof e=="function")for(let n of W1e(e))!B1e.call(o,n)&&n!==t&&O1e(o,n,{get:()=>e[n],enumerable:!(i=F1e(e,n))||i.enumerable});return o},V1e=(o,e,t)=>(H1e(o,e,"default"),t),Rb={};V1e(Rb,rS);var LW=class{constructor(e,t,i){this._onDidChange=new Rb.Emitter,this._languageId=e,this.setOptions(t),this.setModeConfiguration(i)}get onDidChange(){return this._onDidChange.event}get languageId(){return this._languageId}get modeConfiguration(){return this._modeConfiguration}get diagnosticsOptions(){return this.options}get options(){return this._options}setOptions(e){this._options=e||Object.create(null),this._onDidChange.fire(this)}setDiagnosticsOptions(e){this.setOptions(e)}setModeConfiguration(e){this._modeConfiguration=e||Object.create(null),this._onDidChange.fire(this)}},xW={validate:!0,lint:{compatibleVendorPrefixes:"ignore",vendorPrefix:"warning",duplicateProperties:"warning",emptyRules:"warning",importStatement:"ignore",boxModel:"ignore",universalSelector:"ignore",zeroUnits:"ignore",fontFaceProperties:"warning",hexColorLength:"error",argumentsInColorFunction:"error",unknownProperties:"warning",ieHack:"ignore",unknownVendorSpecificProperties:"ignore",propertyIgnoredDueToDisplay:"warning",important:"ignore",float:"ignore",idSelector:"ignore"},data:{useDefaultDataProvider:!0},format:{newlineBetweenSelectors:!0,newlineBetweenRules:!0,spaceAroundSelectorSeparator:!1,braceStyle:"collapse",maxPreserveNewLines:void 0,preserveNewLines:!0}},kW={completionItems:!0,hovers:!0,documentSymbols:!0,definitions:!0,references:!0,documentHighlights:!0,rename:!0,colors:!0,foldingRanges:!0,diagnostics:!0,selectionRanges:!0,documentFormattingEdits:!0,documentRangeFormattingEdits:!0},_G=new LW("css",xW,kW),bG=new LW("scss",xW,kW),CG=new LW("less",xW,kW);Rb.languages.css={cssDefaults:_G,lessDefaults:CG,scssDefaults:bG};function DW(){return ye(()=>import("./cssMode-BhkQQmYI.js"),__vite__mapDeps([13,1,2]),import.meta.url)}Rb.languages.onLanguage("less",()=>{DW().then(o=>o.setupMode(CG))});Rb.languages.onLanguage("scss",()=>{DW().then(o=>o.setupMode(bG))});Rb.languages.onLanguage("css",()=>{DW().then(o=>o.setupMode(_G))});/*!-----------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Version: 0.52.2(404545bded1df6ffa41ea0af4e8ddb219018c6c1)
 * Released under the MIT license
 * https://github.com/microsoft/monaco-editor/blob/main/LICENSE.txt
 *-----------------------------------------------------------------------------*/var z1e=Object.defineProperty,U1e=Object.getOwnPropertyDescriptor,$1e=Object.getOwnPropertyNames,j1e=Object.prototype.hasOwnProperty,K1e=(o,e,t,i)=>{if(e&&typeof e=="object"||typeof e=="function")for(let n of $1e(e))!j1e.call(o,n)&&n!==t&&z1e(o,n,{get:()=>e[n],enumerable:!(i=U1e(e,n))||i.enumerable});return o},q1e=(o,e,t)=>(K1e(o,e,"default"),t),yE={};q1e(yE,rS);var G1e=class{constructor(e,t,i){this._onDidChange=new yE.Emitter,this._languageId=e,this.setOptions(t),this.setModeConfiguration(i)}get onDidChange(){return this._onDidChange.event}get languageId(){return this._languageId}get options(){return this._options}get modeConfiguration(){return this._modeConfiguration}setOptions(e){this._options=e||Object.create(null),this._onDidChange.fire(this)}setModeConfiguration(e){this._modeConfiguration=e||Object.create(null),this._onDidChange.fire(this)}},Z1e={tabSize:4,insertSpaces:!1,wrapLineLength:120,unformatted:'default": "a, abbr, acronym, b, bdo, big, br, button, cite, code, dfn, em, i, img, input, kbd, label, map, object, q, samp, select, small, span, strong, sub, sup, textarea, tt, var',contentUnformatted:"pre",indentInnerHtml:!1,preserveNewLines:!0,maxPreserveNewLines:void 0,indentHandlebars:!1,endWithNewline:!1,extraLiners:"head, body, /html",wrapAttributes:"auto"},LE={format:Z1e,suggest:{},data:{useDefaultDataProvider:!0}};function xE(o){return{completionItems:!0,hovers:!0,documentSymbols:!0,links:!0,documentHighlights:!0,rename:!0,colors:!0,foldingRanges:!0,selectionRanges:!0,diagnostics:o===Cv,documentFormattingEdits:o===Cv,documentRangeFormattingEdits:o===Cv}}var Cv="html",vV="handlebars",wV="razor",vG=kE(Cv,LE,xE(Cv)),Y1e=vG.defaults,wG=kE(vV,LE,xE(vV)),X1e=wG.defaults,SG=kE(wV,LE,xE(wV)),Q1e=SG.defaults;yE.languages.html={htmlDefaults:Y1e,razorDefaults:Q1e,handlebarDefaults:X1e,htmlLanguageService:vG,handlebarLanguageService:wG,razorLanguageService:SG,registerHTMLLanguageService:kE};function J1e(){return ye(()=>import("./htmlMode-DlDHmIFJ.js"),__vite__mapDeps([14,1,2]),import.meta.url)}function kE(o,e=LE,t=xE(o)){const i=new G1e(o,e,t);let n;const s=yE.languages.onLanguage(o,async()=>{n=(await J1e()).setupMode(i)});return{defaults:i,dispose(){s.dispose(),n==null||n.dispose(),n=void 0}}}/*!-----------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Version: 0.52.2(404545bded1df6ffa41ea0af4e8ddb219018c6c1)
 * Released under the MIT license
 * https://github.com/microsoft/monaco-editor/blob/main/LICENSE.txt
 *-----------------------------------------------------------------------------*/var eSe=Object.defineProperty,tSe=Object.getOwnPropertyDescriptor,iSe=Object.getOwnPropertyNames,nSe=Object.prototype.hasOwnProperty,sSe=(o,e,t,i)=>{if(e&&typeof e=="object"||typeof e=="function")for(let n of iSe(e))!nSe.call(o,n)&&n!==t&&eSe(o,n,{get:()=>e[n],enumerable:!(i=tSe(e,n))||i.enumerable});return o},oSe=(o,e,t)=>(sSe(o,e,"default"),t),aS={};oSe(aS,rS);var rSe=class{constructor(e,t,i){this._onDidChange=new aS.Emitter,this._languageId=e,this.setDiagnosticsOptions(t),this.setModeConfiguration(i)}get onDidChange(){return this._onDidChange.event}get languageId(){return this._languageId}get modeConfiguration(){return this._modeConfiguration}get diagnosticsOptions(){return this._diagnosticsOptions}setDiagnosticsOptions(e){this._diagnosticsOptions=e||Object.create(null),this._onDidChange.fire(this)}setModeConfiguration(e){this._modeConfiguration=e||Object.create(null),this._onDidChange.fire(this)}},aSe={validate:!0,allowComments:!0,schemas:[],enableSchemaRequest:!1,schemaRequest:"warning",schemaValidation:"warning",comments:"error",trailingCommas:"error"},lSe={documentFormattingEdits:!0,documentRangeFormattingEdits:!0,completionItems:!0,hovers:!0,documentSymbols:!0,tokens:!0,colors:!0,foldingRanges:!0,diagnostics:!0,selectionRanges:!0},yG=new rSe("json",aSe,lSe),cSe=()=>LG().then(o=>o.getWorker());aS.languages.json={jsonDefaults:yG,getWorker:cSe};function LG(){return ye(()=>import("./jsonMode-syxg-436.js"),__vite__mapDeps([15,1,2]),import.meta.url)}aS.languages.register({id:"json",extensions:[".json",".bowerrc",".jshintrc",".jscsrc",".eslintrc",".babelrc",".har"],aliases:["JSON","json"],mimetypes:["application/json"]});aS.languages.onLanguage("json",()=>{LG().then(o=>o.setupMode(yG))});/*!-----------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Version: 0.52.2(404545bded1df6ffa41ea0af4e8ddb219018c6c1)
 * Released under the MIT license
 * https://github.com/microsoft/monaco-editor/blob/main/LICENSE.txt
 *-----------------------------------------------------------------------------*/var dSe=Object.defineProperty,hSe=Object.getOwnPropertyDescriptor,uSe=Object.getOwnPropertyNames,gSe=Object.prototype.hasOwnProperty,fSe=(o,e,t,i)=>{if(e&&typeof e=="object"||typeof e=="function")for(let n of uSe(e))!gSe.call(o,n)&&n!==t&&dSe(o,n,{get:()=>e[n],enumerable:!(i=hSe(e,n))||i.enumerable});return o},mSe=(o,e,t)=>(fSe(o,e,"default"),t),pSe="5.4.5",Y_={};mSe(Y_,rS);var xG=(o=>(o[o.None=0]="None",o[o.CommonJS=1]="CommonJS",o[o.AMD=2]="AMD",o[o.UMD=3]="UMD",o[o.System=4]="System",o[o.ES2015=5]="ES2015",o[o.ESNext=99]="ESNext",o))(xG||{}),kG=(o=>(o[o.None=0]="None",o[o.Preserve=1]="Preserve",o[o.React=2]="React",o[o.ReactNative=3]="ReactNative",o[o.ReactJSX=4]="ReactJSX",o[o.ReactJSXDev=5]="ReactJSXDev",o))(kG||{}),DG=(o=>(o[o.CarriageReturnLineFeed=0]="CarriageReturnLineFeed",o[o.LineFeed=1]="LineFeed",o))(DG||{}),IG=(o=>(o[o.ES3=0]="ES3",o[o.ES5=1]="ES5",o[o.ES2015=2]="ES2015",o[o.ES2016=3]="ES2016",o[o.ES2017=4]="ES2017",o[o.ES2018=5]="ES2018",o[o.ES2019=6]="ES2019",o[o.ES2020=7]="ES2020",o[o.ESNext=99]="ESNext",o[o.JSON=100]="JSON",o[o.Latest=99]="Latest",o))(IG||{}),EG=(o=>(o[o.Classic=1]="Classic",o[o.NodeJs=2]="NodeJs",o))(EG||{}),NG=class{constructor(o,e,t,i,n){this._onDidChange=new Y_.Emitter,this._onDidExtraLibsChange=new Y_.Emitter,this._extraLibs=Object.create(null),this._removedExtraLibs=Object.create(null),this._eagerModelSync=!1,this.setCompilerOptions(o),this.setDiagnosticsOptions(e),this.setWorkerOptions(t),this.setInlayHintsOptions(i),this.setModeConfiguration(n),this._onDidExtraLibsChangeTimeout=-1}get onDidChange(){return this._onDidChange.event}get onDidExtraLibsChange(){return this._onDidExtraLibsChange.event}get modeConfiguration(){return this._modeConfiguration}get workerOptions(){return this._workerOptions}get inlayHintsOptions(){return this._inlayHintsOptions}getExtraLibs(){return this._extraLibs}addExtraLib(o,e){let t;if(typeof e>"u"?t=`ts:extralib-${Math.random().toString(36).substring(2,15)}`:t=e,this._extraLibs[t]&&this._extraLibs[t].content===o)return{dispose:()=>{}};let i=1;return this._removedExtraLibs[t]&&(i=this._removedExtraLibs[t]+1),this._extraLibs[t]&&(i=this._extraLibs[t].version+1),this._extraLibs[t]={content:o,version:i},this._fireOnDidExtraLibsChangeSoon(),{dispose:()=>{let n=this._extraLibs[t];n&&n.version===i&&(delete this._extraLibs[t],this._removedExtraLibs[t]=i,this._fireOnDidExtraLibsChangeSoon())}}}setExtraLibs(o){for(const e in this._extraLibs)this._removedExtraLibs[e]=this._extraLibs[e].version;if(this._extraLibs=Object.create(null),o&&o.length>0)for(const e of o){const t=e.filePath||`ts:extralib-${Math.random().toString(36).substring(2,15)}`,i=e.content;let n=1;this._removedExtraLibs[t]&&(n=this._removedExtraLibs[t]+1),this._extraLibs[t]={content:i,version:n}}this._fireOnDidExtraLibsChangeSoon()}_fireOnDidExtraLibsChangeSoon(){this._onDidExtraLibsChangeTimeout===-1&&(this._onDidExtraLibsChangeTimeout=window.setTimeout(()=>{this._onDidExtraLibsChangeTimeout=-1,this._onDidExtraLibsChange.fire(void 0)},0))}getCompilerOptions(){return this._compilerOptions}setCompilerOptions(o){this._compilerOptions=o||Object.create(null),this._onDidChange.fire(void 0)}getDiagnosticsOptions(){return this._diagnosticsOptions}setDiagnosticsOptions(o){this._diagnosticsOptions=o||Object.create(null),this._onDidChange.fire(void 0)}setWorkerOptions(o){this._workerOptions=o||Object.create(null),this._onDidChange.fire(void 0)}setInlayHintsOptions(o){this._inlayHintsOptions=o||Object.create(null),this._onDidChange.fire(void 0)}setMaximumWorkerIdleTime(o){}setEagerModelSync(o){this._eagerModelSync=o}getEagerModelSync(){return this._eagerModelSync}setModeConfiguration(o){this._modeConfiguration=o||Object.create(null),this._onDidChange.fire(void 0)}},_Se=pSe,TG={completionItems:!0,hovers:!0,documentSymbols:!0,definitions:!0,references:!0,documentHighlights:!0,rename:!0,diagnostics:!0,documentRangeFormattingEdits:!0,signatureHelp:!0,onTypeFormattingEdits:!0,codeActions:!0,inlayHints:!0},RG=new NG({allowNonTsExtensions:!0,target:99},{noSemanticValidation:!1,noSyntaxValidation:!1,onlyVisible:!1},{},{},TG),MG=new NG({allowNonTsExtensions:!0,allowJs:!0,target:99},{noSemanticValidation:!0,noSyntaxValidation:!1,onlyVisible:!1},{},{},TG),bSe=()=>DE().then(o=>o.getTypeScriptWorker()),CSe=()=>DE().then(o=>o.getJavaScriptWorker());Y_.languages.typescript={ModuleKind:xG,JsxEmit:kG,NewLineKind:DG,ScriptTarget:IG,ModuleResolutionKind:EG,typescriptVersion:_Se,typescriptDefaults:RG,javascriptDefaults:MG,getTypeScriptWorker:bSe,getJavaScriptWorker:CSe};function DE(){return ye(()=>import("./tsMode-DMNyIhlm.js"),__vite__mapDeps([16,1,2]),import.meta.url)}Y_.languages.onLanguage("typescript",()=>DE().then(o=>o.setupTypeScript(RG)));Y_.languages.onLanguage("javascript",()=>DE().then(o=>o.setupJavaScript(MG)));class vSe extends ws{constructor(){super({id:"diffEditor.toggleCollapseUnchangedRegions",title:Te("toggleCollapseUnchangedRegions","Toggle Collapse Unchanged Regions"),icon:te.map,toggled:G.has("config.diffEditor.hideUnchangedRegions.enabled"),precondition:G.has("isInDiffEditor"),menu:{when:G.has("isInDiffEditor"),id:me.EditorTitle,order:22,group:"navigation"}})}run(e,...t){const i=e.get(qe),n=!i.getValue("diffEditor.hideUnchangedRegions.enabled");i.updateValue("diffEditor.hideUnchangedRegions.enabled",n)}}class AG extends ws{constructor(){super({id:"diffEditor.toggleShowMovedCodeBlocks",title:Te("toggleShowMovedCodeBlocks","Toggle Show Moved Code Blocks"),precondition:G.has("isInDiffEditor")})}run(e,...t){const i=e.get(qe),n=!i.getValue("diffEditor.experimental.showMoves");i.updateValue("diffEditor.experimental.showMoves",n)}}class PG extends ws{constructor(){super({id:"diffEditor.toggleUseInlineViewWhenSpaceIsLimited",title:Te("toggleUseInlineViewWhenSpaceIsLimited","Toggle Use Inline View When Space Is Limited"),precondition:G.has("isInDiffEditor")})}run(e,...t){const i=e.get(qe),n=!i.getValue("diffEditor.useInlineViewWhenSpaceIsLimited");i.updateValue("diffEditor.useInlineViewWhenSpaceIsLimited",n)}}const lS=Te("diffEditor","Diff Editor");class wSe extends $a{constructor(){super({id:"diffEditor.switchSide",title:Te("switchSide","Switch Side"),icon:te.arrowSwap,precondition:G.has("isInDiffEditor"),f1:!0,category:lS})}runEditorCommand(e,t,i){const n=Mb(e);if(n instanceof od){if(i&&i.dryRun)return{destinationSelection:n.mapToOtherSide().destinationSelection};n.switchSide()}}}class SSe extends $a{constructor(){super({id:"diffEditor.exitCompareMove",title:Te("exitCompareMove","Exit Compare Move"),icon:te.close,precondition:I.comparingMovedCode,f1:!1,category:lS,keybinding:{weight:1e4,primary:9}})}runEditorCommand(e,t,...i){const n=Mb(e);n instanceof od&&n.exitCompareMove()}}class ySe extends $a{constructor(){super({id:"diffEditor.collapseAllUnchangedRegions",title:Te("collapseAllUnchangedRegions","Collapse All Unchanged Regions"),icon:te.fold,precondition:G.has("isInDiffEditor"),f1:!0,category:lS})}runEditorCommand(e,t,...i){const n=Mb(e);n instanceof od&&n.collapseAllUnchangedRegions()}}class LSe extends $a{constructor(){super({id:"diffEditor.showAllUnchangedRegions",title:Te("showAllUnchangedRegions","Show All Unchanged Regions"),icon:te.unfold,precondition:G.has("isInDiffEditor"),f1:!0,category:lS})}runEditorCommand(e,t,...i){const n=Mb(e);n instanceof od&&n.showAllUnchangedRegions()}}class ZP extends ws{constructor(){super({id:"diffEditor.revert",title:Te("revert","Revert"),f1:!1,category:lS})}run(e,t){const i=xSe(e,t.originalUri,t.modifiedUri);i instanceof od&&i.revertRangeMappings(t.mapping.innerChanges??[])}}const OG=Te("accessibleDiffViewer","Accessible Diff Viewer"),vD=class vD extends ws{constructor(){super({id:vD.id,title:Te("editor.action.accessibleDiffViewer.next","Go to Next Difference"),category:OG,precondition:G.has("isInDiffEditor"),keybinding:{primary:65,weight:100},f1:!0})}run(e){const t=Mb(e);t==null||t.accessibleDiffViewerNext()}};vD.id="editor.action.accessibleDiffViewer.next";let i1=vD;const wD=class wD extends ws{constructor(){super({id:wD.id,title:Te("editor.action.accessibleDiffViewer.prev","Go to Previous Difference"),category:OG,precondition:G.has("isInDiffEditor"),keybinding:{primary:1089,weight:100},f1:!0})}run(e){const t=Mb(e);t==null||t.accessibleDiffViewerPrev()}};wD.id="editor.action.accessibleDiffViewer.prev";let Dx=wD;function xSe(o,e,t){return o.get(pt).listDiffEditors().find(s=>{var l,c;const r=s.getModifiedEditor(),a=s.getOriginalEditor();return r&&((l=r.getModel())==null?void 0:l.uri.toString())===t.toString()&&a&&((c=a.getModel())==null?void 0:c.uri.toString())===e.toString()})||null}function Mb(o){const t=o.get(pt).listDiffEditors(),i=Dn();if(i)for(const n of t){const s=n.getContainerDomNode();if(kSe(s,i))return n}return null}function kSe(o,e){let t=e;for(;t;){if(t===o)return!0;t=t.parentElement}return!1}Ot(vSe);Ot(AG);Ot(PG);rn.appendMenuItem(me.EditorTitle,{command:{id:new PG().desc.id,title:m("useInlineViewWhenSpaceIsLimited","Use Inline View When Space Is Limited"),toggled:G.has("config.diffEditor.useInlineViewWhenSpaceIsLimited"),precondition:G.has("isInDiffEditor")},order:11,group:"1_diff",when:G.and(I.diffEditorRenderSideBySideInlineBreakpointReached,G.has("isInDiffEditor"))});rn.appendMenuItem(me.EditorTitle,{command:{id:new AG().desc.id,title:m("showMoves","Show Moved Code Blocks"),icon:te.move,toggled:mb.create("config.diffEditor.experimental.showMoves",!0),precondition:G.has("isInDiffEditor")},order:10,group:"1_diff",when:G.has("isInDiffEditor")});Ot(ZP);for(const o of[{icon:te.arrowRight,key:I.diffEditorInlineMode.toNegated()},{icon:te.discard,key:I.diffEditorInlineMode}])rn.appendMenuItem(me.DiffEditorHunkToolbar,{command:{id:new ZP().desc.id,title:m("revertHunk","Revert Block"),icon:o.icon},when:G.and(I.diffEditorModifiedWritable,o.key),order:5,group:"primary"}),rn.appendMenuItem(me.DiffEditorSelectionToolbar,{command:{id:new ZP().desc.id,title:m("revertSelection","Revert Selection"),icon:o.icon},when:G.and(I.diffEditorModifiedWritable,o.key),order:5,group:"primary"});Ot(wSe);Ot(SSe);Ot(ySe);Ot(LSe);rn.appendMenuItem(me.EditorTitle,{command:{id:i1.id,title:m("Open Accessible Diff Viewer","Open Accessible Diff Viewer"),precondition:G.has("isInDiffEditor")},order:10,group:"2_diff",when:G.and(I.accessibleDiffViewerVisible.negate(),G.has("isInDiffEditor"))});rt.registerCommandAlias("editor.action.diffReview.next",i1.id);Ot(i1);rt.registerCommandAlias("editor.action.diffReview.prev",Dx.id);Ot(Dx);var DSe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},ISe=function(o,e){return function(t,i){e(t,i,o)}},YP;const IE=new le("selectionAnchorSet",!1);var Og;let Yh=(Og=class{static get(e){return e.getContribution(YP.ID)}constructor(e,t){this.editor=e,this.selectionAnchorSetContextKey=IE.bindTo(t),this.modelChangeListener=e.onDidChangeModel(()=>this.selectionAnchorSetContextKey.reset())}setSelectionAnchor(){if(this.editor.hasModel()){const e=this.editor.getPosition();this.editor.changeDecorations(t=>{this.decorationId&&t.removeDecoration(this.decorationId),this.decorationId=t.addDecoration(pe.fromPositions(e,e),{description:"selection-anchor",stickiness:1,hoverMessage:new Pn().appendText(m("selectionAnchor","Selection Anchor")),className:"selection-anchor"})}),this.selectionAnchorSetContextKey.set(!!this.decorationId),Cs(m("anchorSet","Anchor set at {0}:{1}",e.lineNumber,e.column))}}goToSelectionAnchor(){if(this.editor.hasModel()&&this.decorationId){const e=this.editor.getModel().getDecorationRange(this.decorationId);e&&this.editor.setPosition(e.getStartPosition())}}selectFromAnchorToCursor(){if(this.editor.hasModel()&&this.decorationId){const e=this.editor.getModel().getDecorationRange(this.decorationId);if(e){const t=this.editor.getPosition();this.editor.setSelection(pe.fromPositions(e.getStartPosition(),t)),this.cancelSelectionAnchor()}}}cancelSelectionAnchor(){if(this.decorationId){const e=this.decorationId;this.editor.changeDecorations(t=>{t.removeDecoration(e),this.decorationId=void 0}),this.selectionAnchorSetContextKey.set(!1)}}dispose(){this.cancelSelectionAnchor(),this.modelChangeListener.dispose()}},YP=Og,Og.ID="editor.contrib.selectionAnchorController",Og);Yh=YP=DSe([ISe(1,Ee)],Yh);class ESe extends de{constructor(){super({id:"editor.action.setSelectionAnchor",label:m("setSelectionAnchor","Set Selection Anchor"),alias:"Set Selection Anchor",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2080),weight:100}})}async run(e,t){var i;(i=Yh.get(t))==null||i.setSelectionAnchor()}}class NSe extends de{constructor(){super({id:"editor.action.goToSelectionAnchor",label:m("goToSelectionAnchor","Go to Selection Anchor"),alias:"Go to Selection Anchor",precondition:IE})}async run(e,t){var i;(i=Yh.get(t))==null||i.goToSelectionAnchor()}}class TSe extends de{constructor(){super({id:"editor.action.selectFromAnchorToCursor",label:m("selectFromAnchorToCursor","Select from Anchor to Cursor"),alias:"Select from Anchor to Cursor",precondition:IE,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2089),weight:100}})}async run(e,t){var i;(i=Yh.get(t))==null||i.selectFromAnchorToCursor()}}class RSe extends de{constructor(){super({id:"editor.action.cancelSelectionAnchor",label:m("cancelSelectionAnchor","Cancel Selection Anchor"),alias:"Cancel Selection Anchor",precondition:IE,kbOpts:{kbExpr:I.editorTextFocus,primary:9,weight:100}})}async run(e,t){var i;(i=Yh.get(t))==null||i.cancelSelectionAnchor()}}ct(Yh.ID,Yh,4);Q(ESe);Q(NSe);Q(TSe);Q(RSe);const MSe=E("editorOverviewRuler.bracketMatchForeground","#A0A0A0",m("overviewRulerBracketMatchForeground","Overview ruler marker color for matching brackets."));class ASe extends de{constructor(){super({id:"editor.action.jumpToBracket",label:m("smartSelect.jumpBracket","Go to Bracket"),alias:"Go to Bracket",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:3165,weight:100}})}run(e,t){var i;(i=Hf.get(t))==null||i.jumpToBracket()}}class PSe extends de{constructor(){super({id:"editor.action.selectToBracket",label:m("smartSelect.selectToBracket","Select to Bracket"),alias:"Select to Bracket",precondition:void 0,metadata:{description:Te("smartSelect.selectToBracketDescription","Select the text inside and including the brackets or curly braces"),args:[{name:"args",schema:{type:"object",properties:{selectBrackets:{type:"boolean",default:!0}}}}]}})}run(e,t,i){var s;let n=!0;i&&i.selectBrackets===!1&&(n=!1),(s=Hf.get(t))==null||s.selectToBracket(n)}}class OSe extends de{constructor(){super({id:"editor.action.removeBrackets",label:m("smartSelect.removeBrackets","Remove Brackets"),alias:"Remove Brackets",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:2561,weight:100}})}run(e,t){var i;(i=Hf.get(t))==null||i.removeBrackets(this.id)}}class FSe{constructor(e,t,i){this.position=e,this.brackets=t,this.options=i}}const Pd=class Pd extends F{static get(e){return e.getContribution(Pd.ID)}constructor(e){super(),this._editor=e,this._lastBracketsData=[],this._lastVersionId=0,this._decorations=this._editor.createDecorationsCollection(),this._updateBracketsSoon=this._register(new It(()=>this._updateBrackets(),50)),this._matchBrackets=this._editor.getOption(72),this._updateBracketsSoon.schedule(),this._register(e.onDidChangeCursorPosition(t=>{this._matchBrackets!=="never"&&this._updateBracketsSoon.schedule()})),this._register(e.onDidChangeModelContent(t=>{this._updateBracketsSoon.schedule()})),this._register(e.onDidChangeModel(t=>{this._lastBracketsData=[],this._updateBracketsSoon.schedule()})),this._register(e.onDidChangeModelLanguageConfiguration(t=>{this._lastBracketsData=[],this._updateBracketsSoon.schedule()})),this._register(e.onDidChangeConfiguration(t=>{t.hasChanged(72)&&(this._matchBrackets=this._editor.getOption(72),this._decorations.clear(),this._lastBracketsData=[],this._lastVersionId=0,this._updateBracketsSoon.schedule())})),this._register(e.onDidBlurEditorWidget(()=>{this._updateBracketsSoon.schedule()})),this._register(e.onDidFocusEditorWidget(()=>{this._updateBracketsSoon.schedule()}))}jumpToBracket(){if(!this._editor.hasModel())return;const e=this._editor.getModel(),t=this._editor.getSelections().map(i=>{const n=i.getStartPosition(),s=e.bracketPairs.matchBracket(n);let r=null;if(s)s[0].containsPosition(n)&&!s[1].containsPosition(n)?r=s[1].getStartPosition():s[1].containsPosition(n)&&(r=s[0].getStartPosition());else{const a=e.bracketPairs.findEnclosingBrackets(n);if(a)r=a[1].getStartPosition();else{const l=e.bracketPairs.findNextBracket(n);l&&l.range&&(r=l.range.getStartPosition())}}return r?new pe(r.lineNumber,r.column,r.lineNumber,r.column):new pe(n.lineNumber,n.column,n.lineNumber,n.column)});this._editor.setSelections(t),this._editor.revealRange(t[0])}selectToBracket(e){if(!this._editor.hasModel())return;const t=this._editor.getModel(),i=[];this._editor.getSelections().forEach(n=>{const s=n.getStartPosition();let r=t.bracketPairs.matchBracket(s);if(!r&&(r=t.bracketPairs.findEnclosingBrackets(s),!r)){const c=t.bracketPairs.findNextBracket(s);c&&c.range&&(r=t.bracketPairs.matchBracket(c.range.getStartPosition()))}let a=null,l=null;if(r){r.sort(x.compareRangesUsingStarts);const[c,d]=r;if(a=e?c.getStartPosition():c.getEndPosition(),l=e?d.getEndPosition():d.getStartPosition(),d.containsPosition(s)){const h=a;a=l,l=h}}a&&l&&i.push(new pe(a.lineNumber,a.column,l.lineNumber,l.column))}),i.length>0&&(this._editor.setSelections(i),this._editor.revealRange(i[0]))}removeBrackets(e){if(!this._editor.hasModel())return;const t=this._editor.getModel();this._editor.getSelections().forEach(i=>{const n=i.getPosition();let s=t.bracketPairs.matchBracket(n);s||(s=t.bracketPairs.findEnclosingBrackets(n)),s&&(this._editor.pushUndoStop(),this._editor.executeEdits(e,[{range:s[0],text:""},{range:s[1],text:""}]),this._editor.pushUndoStop())})}_updateBrackets(){if(this._matchBrackets==="never")return;this._recomputeBrackets();const e=[];let t=0;for(const i of this._lastBracketsData){const n=i.brackets;n&&(e[t++]={range:n[0],options:i.options},e[t++]={range:n[1],options:i.options})}this._decorations.set(e)}_recomputeBrackets(){if(!this._editor.hasModel()||!this._editor.hasWidgetFocus()){this._lastBracketsData=[],this._lastVersionId=0;return}const e=this._editor.getSelections();if(e.length>100){this._lastBracketsData=[],this._lastVersionId=0;return}const t=this._editor.getModel(),i=t.getVersionId();let n=[];this._lastVersionId===i&&(n=this._lastBracketsData);const s=[];let r=0;for(let h=0,u=e.length;h<u;h++){const g=e[h];g.isEmpty()&&(s[r++]=g.getStartPosition())}s.length>1&&s.sort(A.compare);const a=[];let l=0,c=0;const d=n.length;for(let h=0,u=s.length;h<u;h++){const g=s[h];for(;c<d&&n[c].position.isBefore(g);)c++;if(c<d&&n[c].position.equals(g))a[l++]=n[c];else{let f=t.bracketPairs.matchBracket(g,20),p=Pd._DECORATION_OPTIONS_WITH_OVERVIEW_RULER;!f&&this._matchBrackets==="always"&&(f=t.bracketPairs.findEnclosingBrackets(g,20),p=Pd._DECORATION_OPTIONS_WITHOUT_OVERVIEW_RULER),a[l++]=new FSe(g,f,p)}}this._lastBracketsData=a,this._lastVersionId=i}};Pd.ID="editor.contrib.bracketMatchingController",Pd._DECORATION_OPTIONS_WITH_OVERVIEW_RULER=We.register({description:"bracket-match-overview",stickiness:1,className:"bracket-match",overviewRuler:{color:mi(MSe),position:Ho.Center}}),Pd._DECORATION_OPTIONS_WITHOUT_OVERVIEW_RULER=We.register({description:"bracket-match-no-overview",stickiness:1,className:"bracket-match"});let Hf=Pd;ct(Hf.ID,Hf,1);Q(PSe);Q(ASe);Q(OSe);rn.appendMenuItem(me.MenubarGoMenu,{group:"5_infile_nav",command:{id:"editor.action.jumpToBracket",title:m({key:"miGoToBracket",comment:["&& denotes a mnemonic"]},"Go to &&Bracket")},order:2});class WSe{constructor(e,t){this._selection=e,this._isMovingLeft=t}getEditOperations(e,t){if(this._selection.startLineNumber!==this._selection.endLineNumber||this._selection.isEmpty())return;const i=this._selection.startLineNumber,n=this._selection.startColumn,s=this._selection.endColumn;if(!(this._isMovingLeft&&n===1)&&!(!this._isMovingLeft&&s===e.getLineMaxColumn(i)))if(this._isMovingLeft){const r=new x(i,n-1,i,n),a=e.getValueInRange(r);t.addEditOperation(r,null),t.addEditOperation(new x(i,s,i,s),a)}else{const r=new x(i,s,i,s+1),a=e.getValueInRange(r);t.addEditOperation(r,null),t.addEditOperation(new x(i,n,i,n),a)}}computeCursorState(e,t){return this._isMovingLeft?new pe(this._selection.startLineNumber,this._selection.startColumn-1,this._selection.endLineNumber,this._selection.endColumn-1):new pe(this._selection.startLineNumber,this._selection.startColumn+1,this._selection.endLineNumber,this._selection.endColumn+1)}}class FG extends de{constructor(e,t){super(t),this.left=e}run(e,t){if(!t.hasModel())return;const i=[],n=t.getSelections();for(const s of n)i.push(new WSe(s,this.left));t.pushUndoStop(),t.executeCommands(this.id,i),t.pushUndoStop()}}class BSe extends FG{constructor(){super(!0,{id:"editor.action.moveCarretLeftAction",label:m("caret.moveLeft","Move Selected Text Left"),alias:"Move Selected Text Left",precondition:I.writable})}}class HSe extends FG{constructor(){super(!1,{id:"editor.action.moveCarretRightAction",label:m("caret.moveRight","Move Selected Text Right"),alias:"Move Selected Text Right",precondition:I.writable})}}Q(BSe);Q(HSe);class VSe extends de{constructor(){super({id:"editor.action.transposeLetters",label:m("transposeLetters.label","Transpose Letters"),alias:"Transpose Letters",precondition:I.writable,kbOpts:{kbExpr:I.textInputFocus,primary:0,mac:{primary:306},weight:100}})}run(e,t){if(!t.hasModel())return;const i=t.getModel(),n=[],s=t.getSelections();for(const r of s){if(!r.isEmpty())continue;const a=r.startLineNumber,l=r.startColumn,c=i.getLineMaxColumn(a);if(a===1&&(l===1||l===2&&c===2))continue;const d=l===c?r.getPosition():Lt.rightPosition(i,r.getPosition().lineNumber,r.getPosition().column),h=Lt.leftPosition(i,d),u=Lt.leftPosition(i,h),g=i.getValueInRange(x.fromPositions(u,h)),f=i.getValueInRange(x.fromPositions(h,d)),p=x.fromPositions(u,d);n.push(new kn(p,f+g))}n.length>0&&(t.pushUndoStop(),t.executeCommands(this.id,n),t.pushUndoStop())}}Q(VSe);const EE=function(){if(typeof crypto=="object"&&typeof crypto.randomUUID=="function")return crypto.randomUUID.bind(crypto);let o;typeof crypto=="object"&&typeof crypto.getRandomValues=="function"?o=crypto.getRandomValues.bind(crypto):o=function(i){for(let n=0;n<i.length;n++)i[n]=Math.floor(Math.random()*256);return i};const e=new Uint8Array(16),t=[];for(let i=0;i<256;i++)t.push(i.toString(16).padStart(2,"0"));return function(){o(e),e[6]=e[6]&15|64,e[8]=e[8]&63|128;let n=0,s="";return s+=t[e[n++]],s+=t[e[n++]],s+=t[e[n++]],s+=t[e[n++]],s+="-",s+=t[e[n++]],s+=t[e[n++]],s+="-",s+=t[e[n++]],s+=t[e[n++]],s+="-",s+=t[e[n++]],s+=t[e[n++]],s+="-",s+=t[e[n++]],s+=t[e[n++]],s+=t[e[n++]],s+=t[e[n++]],s+=t[e[n++]],s+=t[e[n++]],s}}();function IW(o){return{asString:async()=>o,asFile:()=>{},value:typeof o=="string"?o:void 0}}function zSe(o,e,t){const i={id:EE(),name:o,uri:e,data:t};return{asString:async()=>"",asFile:()=>i,value:void 0}}class WG{constructor(){this._entries=new Map}get size(){let e=0;for(const t of this._entries)e++;return e}has(e){return this._entries.has(this.toKey(e))}matches(e){const t=[...this._entries.keys()];return ot.some(this,([i,n])=>n.asFile())&&t.push("files"),HG(Ix(e),t)}get(e){var t;return(t=this._entries.get(this.toKey(e)))==null?void 0:t[0]}append(e,t){const i=this._entries.get(e);i?i.push(t):this._entries.set(this.toKey(e),[t])}replace(e,t){this._entries.set(this.toKey(e),[t])}delete(e){this._entries.delete(this.toKey(e))}*[Symbol.iterator](){for(const[e,t]of this._entries)for(const i of t)yield[e,i]}toKey(e){return Ix(e)}}function Ix(o){return o.toLowerCase()}function BG(o,e){return HG(Ix(o),e.map(Ix))}function HG(o,e){if(o==="*/*")return e.length>0;if(e.includes(o))return!0;const t=o.match(/^([a-z]+)\/([a-z]+|\*)$/i);if(!t)return!1;const[i,n,s]=t;return s==="*"?e.some(r=>r.startsWith(n+"/")):!1}const NE=Object.freeze({create:o=>Xc(o.map(e=>e.toString())).join(`\r
`),split:o=>o.split(`\r
`),parse:o=>NE.split(o).filter(e=>!e.startsWith("#"))}),ul=class ul{constructor(e){this.value=e}equals(e){return this.value===e.value}contains(e){return this.equals(e)||this.value===""||e.value.startsWith(this.value+ul.sep)}intersects(e){return this.contains(e)||e.contains(this)}append(...e){return new ul((this.value?[this.value,...e]:e).join(ul.sep))}};ul.sep=".",ul.None=new ul("@@none@@"),ul.Empty=new ul("");let ti=ul;const SV={EDITORS:"CodeEditors",FILES:"CodeFiles"};class USe{}const $Se={DragAndDropContribution:"workbench.contributions.dragAndDrop"};ci.add($Se.DragAndDropContribution,new USe);const $v=class $v{constructor(){}static getInstance(){return $v.INSTANCE}hasData(e){return e&&e===this.proto}getData(e){if(this.hasData(e))return this.data}};$v.INSTANCE=new $v;let XP=$v;function VG(o){const e=new WG;for(const t of o.items){const i=t.type;if(t.kind==="string"){const n=new Promise(s=>t.getAsString(s));e.append(i,IW(n))}else if(t.kind==="file"){const n=t.getAsFile();n&&e.append(i,jSe(n))}}return e}function jSe(o){const e=o.path?Se.parse(o.path):void 0;return zSe(o.name,e,async()=>new Uint8Array(await o.arrayBuffer()))}const KSe=Object.freeze([SV.EDITORS,SV.FILES,Iw.RESOURCES,Iw.INTERNAL_URI_LIST]);function zG(o,e=!1){const t=VG(o),i=t.get(Iw.INTERNAL_URI_LIST);if(i)t.replace(vi.uriList,i);else if(e||!t.has(vi.uriList)){const n=[];for(const s of o.items){const r=s.getAsFile();if(r){const a=r.path;try{a?n.push(Se.file(a).toString()):n.push(Se.parse(r.name,!0).toString())}catch{}}}n.length&&t.replace(vi.uriList,IW(NE.create(n)))}for(const n of KSe)t.delete(n);return t}var EW=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},n1=function(o,e){return function(t,i){e(t,i,o)}};class NW{async provideDocumentPasteEdits(e,t,i,n,s){const r=await this.getEdit(i,s);if(r)return{edits:[{insertText:r.insertText,title:r.title,kind:r.kind,handledMimeType:r.handledMimeType,yieldTo:r.yieldTo}],dispose(){}}}async provideDocumentDropEdits(e,t,i,n){const s=await this.getEdit(i,n);if(s)return{edits:[{insertText:s.insertText,title:s.title,kind:s.kind,handledMimeType:s.handledMimeType,yieldTo:s.yieldTo}],dispose(){}}}}const jv=class jv extends NW{constructor(){super(...arguments),this.kind=jv.kind,this.dropMimeTypes=[vi.text],this.pasteMimeTypes=[vi.text]}async getEdit(e,t){const i=e.get(vi.text);if(!i||e.has(vi.uriList))return;const n=await i.asString();return{handledMimeType:vi.text,title:m("text.label","Insert Plain Text"),insertText:n,kind:this.kind}}};jv.id="text",jv.kind=new ti("text.plain");let Vf=jv;class UG extends NW{constructor(){super(...arguments),this.kind=new ti("uri.absolute"),this.dropMimeTypes=[vi.uriList],this.pasteMimeTypes=[vi.uriList]}async getEdit(e,t){const i=await $G(e);if(!i.length||t.isCancellationRequested)return;let n=0;const s=i.map(({uri:a,originalText:l})=>a.scheme===Fe.file?a.fsPath:(n++,l)).join(" ");let r;return n>0?r=i.length>1?m("defaultDropProvider.uriList.uris","Insert Uris"):m("defaultDropProvider.uriList.uri","Insert Uri"):r=i.length>1?m("defaultDropProvider.uriList.paths","Insert Paths"):m("defaultDropProvider.uriList.path","Insert Path"),{handledMimeType:vi.uriList,insertText:s,title:r,kind:this.kind}}}let Ex=class extends NW{constructor(e){super(),this._workspaceContextService=e,this.kind=new ti("uri.relative"),this.dropMimeTypes=[vi.uriList],this.pasteMimeTypes=[vi.uriList]}async getEdit(e,t){const i=await $G(e);if(!i.length||t.isCancellationRequested)return;const n=Kr(i.map(({uri:s})=>{const r=this._workspaceContextService.getWorkspaceFolder(s);return r?dle(r.uri,s):void 0}));if(n.length)return{handledMimeType:vi.uriList,insertText:n.join(" "),title:i.length>1?m("defaultDropProvider.uriList.relativePaths","Insert Relative Paths"):m("defaultDropProvider.uriList.relativePath","Insert Relative Path"),kind:this.kind}}};Ex=EW([n1(0,Af)],Ex);class qSe{constructor(){this.kind=new ti("html"),this.pasteMimeTypes=["text/html"],this._yieldTo=[{mimeType:vi.text}]}async provideDocumentPasteEdits(e,t,i,n,s){var l;if(n.triggerKind!==iw.PasteAs&&!((l=n.only)!=null&&l.contains(this.kind)))return;const r=i.get("text/html"),a=await(r==null?void 0:r.asString());if(!(!a||s.isCancellationRequested))return{dispose(){},edits:[{insertText:a,yieldTo:this._yieldTo,title:m("pasteHtmlLabel","Insert HTML"),kind:this.kind}]}}}async function $G(o){const e=o.get(vi.uriList);if(!e)return[];const t=await e.asString(),i=[];for(const n of NE.parse(t))try{i.push({uri:Se.parse(n),originalText:n})}catch{}return i}let QP=class extends F{constructor(e,t){super(),this._register(e.documentDropEditProvider.register("*",new Vf)),this._register(e.documentDropEditProvider.register("*",new UG)),this._register(e.documentDropEditProvider.register("*",new Ex(t)))}};QP=EW([n1(0,ge),n1(1,Af)],QP);let JP=class extends F{constructor(e,t){super(),this._register(e.documentPasteEditProvider.register("*",new Vf)),this._register(e.documentPasteEditProvider.register("*",new UG)),this._register(e.documentPasteEditProvider.register("*",new Ex(t))),this._register(e.documentPasteEditProvider.register("*",new qSe))}};JP=EW([n1(0,ge),n1(1,Af)],JP);const Mr=class Mr{constructor(){this.value="",this.pos=0}static isDigitCharacter(e){return e>=48&&e<=57}static isVariableCharacter(e){return e===95||e>=97&&e<=122||e>=65&&e<=90}text(e){this.value=e,this.pos=0}tokenText(e){return this.value.substr(e.pos,e.len)}next(){if(this.pos>=this.value.length)return{type:14,pos:this.pos,len:0};const e=this.pos;let t=0,i=this.value.charCodeAt(e),n;if(n=Mr._table[i],typeof n=="number")return this.pos+=1,{type:n,pos:e,len:1};if(Mr.isDigitCharacter(i)){n=8;do t+=1,i=this.value.charCodeAt(e+t);while(Mr.isDigitCharacter(i));return this.pos+=t,{type:n,pos:e,len:t}}if(Mr.isVariableCharacter(i)){n=9;do i=this.value.charCodeAt(e+ ++t);while(Mr.isVariableCharacter(i)||Mr.isDigitCharacter(i));return this.pos+=t,{type:n,pos:e,len:t}}n=10;do t+=1,i=this.value.charCodeAt(e+t);while(!isNaN(i)&&typeof Mr._table[i]>"u"&&!Mr.isDigitCharacter(i)&&!Mr.isVariableCharacter(i));return this.pos+=t,{type:n,pos:e,len:t}}};Mr._table={36:0,58:1,44:2,123:3,125:4,92:5,47:6,124:7,43:11,45:12,63:13};let eO=Mr;class Ab{constructor(){this._children=[]}appendChild(e){return e instanceof ls&&this._children[this._children.length-1]instanceof ls?this._children[this._children.length-1].value+=e.value:(e.parent=this,this._children.push(e)),this}replace(e,t){const{parent:i}=e,n=i.children.indexOf(e),s=i.children.slice(0);s.splice(n,1,...t),i._children=s,function r(a,l){for(const c of a)c.parent=l,r(c.children,c)}(t,i)}get children(){return this._children}get rightMostDescendant(){return this._children.length>0?this._children[this._children.length-1].rightMostDescendant:this}get snippet(){let e=this;for(;;){if(!e)return;if(e instanceof cS)return e;e=e.parent}}toString(){return this.children.reduce((e,t)=>e+t.toString(),"")}len(){return 0}}class ls extends Ab{constructor(e){super(),this.value=e}toString(){return this.value}len(){return this.value.length}clone(){return new ls(this.value)}}class jG extends Ab{}class ar extends jG{static compareByIndex(e,t){return e.index===t.index?0:e.isFinalTabstop?1:t.isFinalTabstop||e.index<t.index?-1:e.index>t.index?1:0}constructor(e){super(),this.index=e}get isFinalTabstop(){return this.index===0}get choice(){return this._children.length===1&&this._children[0]instanceof Pb?this._children[0]:void 0}clone(){const e=new ar(this.index);return this.transform&&(e.transform=this.transform.clone()),e._children=this.children.map(t=>t.clone()),e}}class Pb extends Ab{constructor(){super(...arguments),this.options=[]}appendChild(e){return e instanceof ls&&(e.parent=this,this.options.push(e)),this}toString(){return this.options[0].value}len(){return this.options[0].len()}clone(){const e=new Pb;return this.options.forEach(e.appendChild,e),e}}class TW extends Ab{constructor(){super(...arguments),this.regexp=new RegExp("")}resolve(e){const t=this;let i=!1,n=e.replace(this.regexp,function(){return i=!0,t._replace(Array.prototype.slice.call(arguments,0,-2))});return!i&&this._children.some(s=>s instanceof _a&&!!s.elseValue)&&(n=this._replace([])),n}_replace(e){let t="";for(const i of this._children)if(i instanceof _a){let n=e[i.index]||"";n=i.resolve(n),t+=n}else t+=i.toString();return t}toString(){return""}clone(){const e=new TW;return e.regexp=new RegExp(this.regexp.source,(this.regexp.ignoreCase?"i":"")+(this.regexp.global?"g":"")),e._children=this.children.map(t=>t.clone()),e}}class _a extends Ab{constructor(e,t,i,n){super(),this.index=e,this.shorthandName=t,this.ifValue=i,this.elseValue=n}resolve(e){return this.shorthandName==="upcase"?e?e.toLocaleUpperCase():"":this.shorthandName==="downcase"?e?e.toLocaleLowerCase():"":this.shorthandName==="capitalize"?e?e[0].toLocaleUpperCase()+e.substr(1):"":this.shorthandName==="pascalcase"?e?this._toPascalCase(e):"":this.shorthandName==="camelcase"?e?this._toCamelCase(e):"":e&&typeof this.ifValue=="string"?this.ifValue:!e&&typeof this.elseValue=="string"?this.elseValue:e||""}_toPascalCase(e){const t=e.match(/[a-z0-9]+/gi);return t?t.map(i=>i.charAt(0).toUpperCase()+i.substr(1)).join(""):e}_toCamelCase(e){const t=e.match(/[a-z0-9]+/gi);return t?t.map((i,n)=>n===0?i.charAt(0).toLowerCase()+i.substr(1):i.charAt(0).toUpperCase()+i.substr(1)).join(""):e}clone(){return new _a(this.index,this.shorthandName,this.ifValue,this.elseValue)}}class s1 extends jG{constructor(e){super(),this.name=e}resolve(e){let t=e.resolve(this);return this.transform&&(t=this.transform.resolve(t||"")),t!==void 0?(this._children=[new ls(t)],!0):!1}clone(){const e=new s1(this.name);return this.transform&&(e.transform=this.transform.clone()),e._children=this.children.map(t=>t.clone()),e}}function yV(o,e){const t=[...o];for(;t.length>0;){const i=t.shift();if(!e(i))break;t.unshift(...i.children)}}class cS extends Ab{get placeholderInfo(){if(!this._placeholders){const e=[];let t;this.walk(function(i){return i instanceof ar&&(e.push(i),t=!t||t.index<i.index?i:t),!0}),this._placeholders={all:e,last:t}}return this._placeholders}get placeholders(){const{all:e}=this.placeholderInfo;return e}offset(e){let t=0,i=!1;return this.walk(n=>n===e?(i=!0,!1):(t+=n.len(),!0)),i?t:-1}fullLen(e){let t=0;return yV([e],i=>(t+=i.len(),!0)),t}enclosingPlaceholders(e){const t=[];let{parent:i}=e;for(;i;)i instanceof ar&&t.push(i),i=i.parent;return t}resolveVariables(e){return this.walk(t=>(t instanceof s1&&t.resolve(e)&&(this._placeholders=void 0),!0)),this}appendChild(e){return this._placeholders=void 0,super.appendChild(e)}replace(e,t){return this._placeholders=void 0,super.replace(e,t)}clone(){const e=new cS;return this._children=this.children.map(t=>t.clone()),e}walk(e){yV(this.children,e)}}class zf{constructor(){this._scanner=new eO,this._token={type:14,pos:0,len:0}}static escape(e){return e.replace(/\$|}|\\/g,"\\$&")}static guessNeedsClipboard(e){return/\${?CLIPBOARD/.test(e)}parse(e,t,i){const n=new cS;return this.parseFragment(e,n),this.ensureFinalTabstop(n,i??!1,t??!1),n}parseFragment(e,t){const i=t.children.length;for(this._scanner.text(e),this._token=this._scanner.next();this._parse(t););const n=new Map,s=[];t.walk(l=>(l instanceof ar&&(l.isFinalTabstop?n.set(0,void 0):!n.has(l.index)&&l.children.length>0?n.set(l.index,l.children):s.push(l)),!0));const r=(l,c)=>{const d=n.get(l.index);if(!d)return;const h=new ar(l.index);h.transform=l.transform;for(const u of d){const g=u.clone();h.appendChild(g),g instanceof ar&&n.has(g.index)&&!c.has(g.index)&&(c.add(g.index),r(g,c),c.delete(g.index))}t.replace(l,[h])},a=new Set;for(const l of s)r(l,a);return t.children.slice(i)}ensureFinalTabstop(e,t,i){(t||i&&e.placeholders.length>0)&&(e.placeholders.find(s=>s.index===0)||e.appendChild(new ar(0)))}_accept(e,t){if(e===void 0||this._token.type===e){const i=t?this._scanner.tokenText(this._token):!0;return this._token=this._scanner.next(),i}return!1}_backTo(e){return this._scanner.pos=e.pos+e.len,this._token=e,!1}_until(e){const t=this._token;for(;this._token.type!==e;){if(this._token.type===14)return!1;if(this._token.type===5){const n=this._scanner.next();if(n.type!==0&&n.type!==4&&n.type!==5)return!1}this._token=this._scanner.next()}const i=this._scanner.value.substring(t.pos,this._token.pos).replace(/\\(\$|}|\\)/g,"$1");return this._token=this._scanner.next(),i}_parse(e){return this._parseEscaped(e)||this._parseTabstopOrVariableName(e)||this._parseComplexPlaceholder(e)||this._parseComplexVariable(e)||this._parseAnything(e)}_parseEscaped(e){let t;return(t=this._accept(5,!0))?(t=this._accept(0,!0)||this._accept(4,!0)||this._accept(5,!0)||t,e.appendChild(new ls(t)),!0):!1}_parseTabstopOrVariableName(e){let t;const i=this._token;return this._accept(0)&&(t=this._accept(9,!0)||this._accept(8,!0))?(e.appendChild(/^\d+$/.test(t)?new ar(Number(t)):new s1(t)),!0):this._backTo(i)}_parseComplexPlaceholder(e){let t;const i=this._token;if(!(this._accept(0)&&this._accept(3)&&(t=this._accept(8,!0))))return this._backTo(i);const s=new ar(Number(t));if(this._accept(1))for(;;){if(this._accept(4))return e.appendChild(s),!0;if(!this._parse(s))return e.appendChild(new ls("${"+t+":")),s.children.forEach(e.appendChild,e),!0}else if(s.index>0&&this._accept(7)){const r=new Pb;for(;;){if(this._parseChoiceElement(r)){if(this._accept(2))continue;if(this._accept(7)&&(s.appendChild(r),this._accept(4)))return e.appendChild(s),!0}return this._backTo(i),!1}}else return this._accept(6)?this._parseTransform(s)?(e.appendChild(s),!0):(this._backTo(i),!1):this._accept(4)?(e.appendChild(s),!0):this._backTo(i)}_parseChoiceElement(e){const t=this._token,i=[];for(;!(this._token.type===2||this._token.type===7);){let n;if((n=this._accept(5,!0))?n=this._accept(2,!0)||this._accept(7,!0)||this._accept(5,!0)||n:n=this._accept(void 0,!0),!n)return this._backTo(t),!1;i.push(n)}return i.length===0?(this._backTo(t),!1):(e.appendChild(new ls(i.join(""))),!0)}_parseComplexVariable(e){let t;const i=this._token;if(!(this._accept(0)&&this._accept(3)&&(t=this._accept(9,!0))))return this._backTo(i);const s=new s1(t);if(this._accept(1))for(;;){if(this._accept(4))return e.appendChild(s),!0;if(!this._parse(s))return e.appendChild(new ls("${"+t+":")),s.children.forEach(e.appendChild,e),!0}else return this._accept(6)?this._parseTransform(s)?(e.appendChild(s),!0):(this._backTo(i),!1):this._accept(4)?(e.appendChild(s),!0):this._backTo(i)}_parseTransform(e){const t=new TW;let i="",n="";for(;!this._accept(6);){let s;if(s=this._accept(5,!0)){s=this._accept(6,!0)||s,i+=s;continue}if(this._token.type!==14){i+=this._accept(void 0,!0);continue}return!1}for(;!this._accept(6);){let s;if(s=this._accept(5,!0)){s=this._accept(5,!0)||this._accept(6,!0)||s,t.appendChild(new ls(s));continue}if(!(this._parseFormatString(t)||this._parseAnything(t)))return!1}for(;!this._accept(4);){if(this._token.type!==14){n+=this._accept(void 0,!0);continue}return!1}try{t.regexp=new RegExp(i,n)}catch{return!1}return e.transform=t,!0}_parseFormatString(e){const t=this._token;if(!this._accept(0))return!1;let i=!1;this._accept(3)&&(i=!0);const n=this._accept(8,!0);if(n)if(i){if(this._accept(4))return e.appendChild(new _a(Number(n))),!0;if(!this._accept(1))return this._backTo(t),!1}else return e.appendChild(new _a(Number(n))),!0;else return this._backTo(t),!1;if(this._accept(6)){const s=this._accept(9,!0);return!s||!this._accept(4)?(this._backTo(t),!1):(e.appendChild(new _a(Number(n),s)),!0)}else if(this._accept(11)){const s=this._until(4);if(s)return e.appendChild(new _a(Number(n),void 0,s,void 0)),!0}else if(this._accept(12)){const s=this._until(4);if(s)return e.appendChild(new _a(Number(n),void 0,void 0,s)),!0}else if(this._accept(13)){const s=this._until(1);if(s){const r=this._until(4);if(r)return e.appendChild(new _a(Number(n),void 0,s,r)),!0}}else{const s=this._until(4);if(s)return e.appendChild(new _a(Number(n),void 0,void 0,s)),!0}return this._backTo(t),!1}_parseAnything(e){return this._token.type!==14?(e.appendChild(new ls(this._scanner.tokenText(this._token))),this._accept(void 0),!0):!1}}function KG(o,e,t){var i,n;return(typeof t.insertText=="string"?t.insertText==="":t.insertText.snippet==="")?{edits:((i=t.additionalEdit)==null?void 0:i.edits)??[]}:{edits:[...e.map(s=>new hh(o,{range:s,text:typeof t.insertText=="string"?zf.escape(t.insertText)+"$0":t.insertText.snippet,insertAsSnippet:!0})),...((n=t.additionalEdit)==null?void 0:n.edits)??[]]}}function qG(o){function e(r,a){return"mimeType"in r?r.mimeType===a.handledMimeType:!!a.kind&&r.kind.contains(a.kind)}const t=new Map;for(const r of o)for(const a of r.yieldTo??[])for(const l of o)if(l!==r&&e(a,l)){let c=t.get(r);c||(c=[],t.set(r,c)),c.push(l)}if(!t.size)return Array.from(o);const i=new Set,n=[];function s(r){if(!r.length)return[];const a=r[0];if(n.includes(a))return console.warn("Yield to cycle detected",a),r;if(i.has(a))return s(r.slice(1));let l=[];const c=t.get(a);return c&&(n.push(a),l=s(c),n.pop()),i.add(a),[...l,a,...s(r.slice(1))]}return s(Array.from(o))}var GSe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},ZSe=function(o,e){return function(t,i){e(t,i,o)}};const YSe=We.register({description:"inline-progress-widget",stickiness:1,showIfCollapsed:!0,after:{content:rU,inlineClassName:"inline-editor-progress-decoration",inlineClassNameAffectsLetterSpacing:!0}}),SD=class SD extends F{constructor(e,t,i,n,s){super(),this.typeId=e,this.editor=t,this.range=i,this.delegate=s,this.allowEditorOverflow=!1,this.suppressMouseDown=!0,this.create(n),this.editor.addContentWidget(this),this.editor.layoutContentWidget(this)}create(e){this.domNode=ne(".inline-progress-widget"),this.domNode.role="button",this.domNode.title=e;const t=ne("span.icon");this.domNode.append(t),t.classList.add(...Ie.asClassNameArray(te.loading),"codicon-modifier-spin");const i=()=>{const n=this.editor.getOption(67);this.domNode.style.height=`${n}px`,this.domNode.style.width=`${Math.ceil(.8*n)}px`};i(),this._register(this.editor.onDidChangeConfiguration(n=>{(n.hasChanged(52)||n.hasChanged(67))&&i()})),this._register(U(this.domNode,ie.CLICK,n=>{this.delegate.cancel()}))}getId(){return SD.baseId+"."+this.typeId}getDomNode(){return this.domNode}getPosition(){return{position:{lineNumber:this.range.startLineNumber,column:this.range.startColumn},preference:[0]}}dispose(){super.dispose(),this.editor.removeContentWidget(this)}};SD.baseId="editor.widget.inlineProgressWidget";let tO=SD,Nx=class extends F{constructor(e,t,i){super(),this.id=e,this._editor=t,this._instantiationService=i,this._showDelay=500,this._showPromise=this._register(new Ji),this._currentWidget=this._register(new Ji),this._operationIdPool=0,this._currentDecorations=t.createDecorationsCollection()}dispose(){super.dispose(),this._currentDecorations.clear()}async showWhile(e,t,i,n,s){const r=this._operationIdPool++;this._currentOperation=r,this.clear(),this._showPromise.value=Bh(()=>{const a=x.fromPositions(e);this._currentDecorations.set([{range:a,options:YSe}]).length>0&&(this._currentWidget.value=this._instantiationService.createInstance(tO,this.id,this._editor,a,t,n))},s??this._showDelay);try{return await i}finally{this._currentOperation===r&&(this.clear(),this._currentOperation=void 0)}}clear(){this._showPromise.clear(),this._currentDecorations.clear(),this._currentWidget.clear()}};Nx=GSe([ZSe(2,we)],Nx);var XSe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},LV=function(o,e){return function(t,i){e(t,i,o)}},uy,ph;let lo=(ph=class{static get(e){return e.getContribution(uy.ID)}constructor(e,t,i){this._openerService=i,this._messageWidget=new Ji,this._messageListeners=new K,this._mouseOverMessage=!1,this._editor=e,this._visible=uy.MESSAGE_VISIBLE.bindTo(t)}dispose(){var e;(e=this._message)==null||e.dispose(),this._messageListeners.dispose(),this._messageWidget.dispose(),this._visible.reset()}showMessage(e,t){Cs(Aa(e)?e.value:e),this._visible.set(!0),this._messageWidget.clear(),this._messageListeners.clear(),this._message=Aa(e)?YI(e,{actionHandler:{callback:n=>{this.closeMessage(),Y5(this._openerService,n,Aa(e)?e.isTrusted:void 0)},disposables:this._messageListeners}}):void 0,this._messageWidget.value=new xV(this._editor,t,typeof e=="string"?e:this._message.element),this._messageListeners.add(oe.debounce(this._editor.onDidBlurEditorText,(n,s)=>s,0)(()=>{this._mouseOverMessage||this._messageWidget.value&&Zi(Dn(),this._messageWidget.value.getDomNode())||this.closeMessage()})),this._messageListeners.add(this._editor.onDidChangeCursorPosition(()=>this.closeMessage())),this._messageListeners.add(this._editor.onDidDispose(()=>this.closeMessage())),this._messageListeners.add(this._editor.onDidChangeModel(()=>this.closeMessage())),this._messageListeners.add(U(this._messageWidget.value.getDomNode(),ie.MOUSE_ENTER,()=>this._mouseOverMessage=!0,!0)),this._messageListeners.add(U(this._messageWidget.value.getDomNode(),ie.MOUSE_LEAVE,()=>this._mouseOverMessage=!1,!0));let i;this._messageListeners.add(this._editor.onMouseMove(n=>{n.target.position&&(i?i.containsPosition(n.target.position)||this.closeMessage():i=new x(t.lineNumber-3,1,n.target.position.lineNumber+3,1))}))}closeMessage(){this._visible.reset(),this._messageListeners.clear(),this._messageWidget.value&&this._messageListeners.add(xV.fadeOut(this._messageWidget.value))}},uy=ph,ph.ID="editor.contrib.messageController",ph.MESSAGE_VISIBLE=new le("messageVisible",!1,m("messageVisible","Whether the editor is currently showing an inline message")),ph);lo=uy=XSe([LV(1,Ee),LV(2,zs)],lo);const QSe=$i.bindToContribution(lo.get);re(new QSe({id:"leaveEditorMessage",precondition:lo.MESSAGE_VISIBLE,handler:o=>o.closeMessage(),kbOpts:{weight:130,primary:9}}));let xV=class{static fadeOut(e){const t=()=>{e.dispose(),clearTimeout(i),e.getDomNode().removeEventListener("animationend",t)},i=setTimeout(t,110);return e.getDomNode().addEventListener("animationend",t),e.getDomNode().classList.add("fadeOut"),{dispose:t}}constructor(e,{lineNumber:t,column:i},n){this.allowEditorOverflow=!0,this.suppressMouseDown=!1,this._editor=e,this._editor.revealLinesInCenterIfOutsideViewport(t,t,0),this._position={lineNumber:t,column:i},this._domNode=document.createElement("div"),this._domNode.classList.add("monaco-editor-overlaymessage"),this._domNode.style.marginLeft="-6px";const s=document.createElement("div");s.classList.add("anchor","top"),this._domNode.appendChild(s);const r=document.createElement("div");typeof n=="string"?(r.classList.add("message"),r.textContent=n):(n.classList.add("message"),r.appendChild(n)),this._domNode.appendChild(r);const a=document.createElement("div");a.classList.add("anchor","below"),this._domNode.appendChild(a),this._editor.addContentWidget(this),this._domNode.classList.add("fadeIn")}dispose(){this._editor.removeContentWidget(this)}getId(){return"messageoverlay"}getDomNode(){return this._domNode}getPosition(){return{position:this._position,preference:[1,2],positionAffinity:1}}afterRender(e){this._domNode.classList.toggle("below",e===2)}};ct(lo.ID,lo,4);function RT(o,e){return e&&(o.stack||o.stacktrace)?m("stackTrace.format","{0}: {1}",DV(o),kV(o.stack)||kV(o.stacktrace)):DV(o)}function kV(o){return Array.isArray(o)?o.join(`
`):o}function DV(o){return o.code==="ERR_UNC_HOST_NOT_ALLOWED"?`${o.message}. Please update the 'security.allowedUNCHosts' setting if you want to allow this host.`:typeof o.code=="string"&&typeof o.errno=="number"&&typeof o.syscall=="string"?m("nodeExceptionMessage","A system error occurred ({0})",o.message):o.message||m("error.defaultMessage","An unknown error occurred. Please consult the log for more details.")}function Tx(o=null,e=!1){if(!o)return m("error.defaultMessage","An unknown error occurred. Please consult the log for more details.");if(Array.isArray(o)){const t=Kr(o),i=Tx(t[0],e);return t.length>1?m("error.moreErrors","{0} ({1} errors in total)",i,t.length):i}if(As(o))return o;if(o.detail){const t=o.detail;if(t.error)return RT(t.error,e);if(t.exception)return RT(t.exception,e)}return o.stack?RT(o,e):o.message?o.message:m("error.defaultMessage","An unknown error occurred. Please consult the log for more details.")}var GG=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},c_=function(o,e){return function(t,i){e(t,i,o)}},iO,Fg;let nO=(Fg=class extends F{constructor(e,t,i,n,s,r,a,l,c,d){super(),this.typeId=e,this.editor=t,this.showCommand=n,this.range=s,this.edits=r,this.onSelectNewEdit=a,this._contextMenuService=l,this._keybindingService=d,this.allowEditorOverflow=!0,this.suppressMouseDown=!0,this.create(),this.visibleContext=i.bindTo(c),this.visibleContext.set(!0),this._register(ve(()=>this.visibleContext.reset())),this.editor.addContentWidget(this),this.editor.layoutContentWidget(this),this._register(ve(()=>this.editor.removeContentWidget(this))),this._register(this.editor.onDidChangeCursorPosition(h=>{s.containsPosition(h.position)||this.dispose()})),this._register(oe.runAndSubscribe(d.onDidUpdateKeybindings,()=>{this._updateButtonTitle()}))}_updateButtonTitle(){var t;const e=(t=this._keybindingService.lookupKeybinding(this.showCommand.id))==null?void 0:t.getLabel();this.button.element.title=this.showCommand.label+(e?` (${e})`:"")}create(){this.domNode=ne(".post-edit-widget"),this.button=this._register(new FL(this.domNode,{supportIcons:!0})),this.button.label="$(insert)",this._register(U(this.domNode,ie.CLICK,()=>this.showSelector()))}getId(){return iO.baseId+"."+this.typeId}getDomNode(){return this.domNode}getPosition(){return{position:this.range.getEndPosition(),preference:[2]}}showSelector(){this._contextMenuService.showContextMenu({getAnchor:()=>{const e=wi(this.button.element);return{x:e.left+e.width,y:e.top+e.height}},getActions:()=>this.edits.allEdits.map((e,t)=>pg({id:"",label:e.title,checked:t===this.edits.activeEditIndex,run:()=>{if(t!==this.edits.activeEditIndex)return this.onSelectNewEdit(t)}}))})}},iO=Fg,Fg.baseId="editor.widget.postEditWidget",Fg);nO=iO=GG([c_(7,mo),c_(8,Ee),c_(9,yt)],nO);let Rx=class extends F{constructor(e,t,i,n,s,r,a){super(),this._id=e,this._editor=t,this._visibleContext=i,this._showCommand=n,this._instantiationService=s,this._bulkEditService=r,this._notificationService=a,this._currentWidget=this._register(new Ji),this._register(oe.any(t.onDidChangeModel,t.onDidChangeModelContent)(()=>this.clear()))}async applyEditAndShowIfNeeded(e,t,i,n,s){const r=this._editor.getModel();if(!r||!e.length)return;const a=t.allEdits.at(t.activeEditIndex);if(!a)return;const l=async _=>{const b=this._editor.getModel();b&&(await b.undo(),this.applyEditAndShowIfNeeded(e,{activeEditIndex:_,allEdits:t.allEdits},i,n,s))},c=(_,b)=>{yr(_)||(this._notificationService.error(b),i&&this.show(e[0],t,l))};let d;try{d=await n(a,s)}catch(_){return c(_,m("resolveError",`Error resolving edit '{0}':
{1}`,a.title,Tx(_)))}if(s.isCancellationRequested)return;const h=KG(r.uri,e,d),u=e[0],g=r.deltaDecorations([],[{range:u,options:{description:"paste-line-suffix",stickiness:0}}]);this._editor.focus();let f,p;try{f=await this._bulkEditService.apply(h,{editor:this._editor,token:s}),p=r.getDecorationRange(g[0])}catch(_){return c(_,m("applyError",`Error applying edit '{0}':
{1}`,a.title,Tx(_)))}finally{r.deltaDecorations(g,[])}s.isCancellationRequested||i&&f.isApplied&&t.allEdits.length>1&&this.show(p??u,t,l)}show(e,t,i){this.clear(),this._editor.hasModel()&&(this._currentWidget.value=this._instantiationService.createInstance(nO,this._id,this._editor,this._visibleContext,this._showCommand,e,t,i))}clear(){this._currentWidget.clear()}tryShowSelector(){var e;(e=this._currentWidget.value)==null||e.showSelector()}};Rx=GG([c_(4,we),c_(5,j1),c_(6,Ei)],Rx);var JSe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Rm=function(o,e){return function(t,i){e(t,i,o)}},Fu;const ZG="editor.changePasteType",RW=new le("pasteWidgetVisible",!1,m("pasteWidgetVisible","Whether the paste widget is showing")),MT="application/vnd.code.copyMetadata";var Wg;let rd=(Wg=class extends F{static get(e){return e.getContribution(Fu.ID)}constructor(e,t,i,n,s,r,a){super(),this._bulkEditService=i,this._clipboardService=n,this._languageFeaturesService=s,this._quickInputService=r,this._progressService=a,this._editor=e;const l=e.getContainerDomNode();this._register(U(l,"copy",c=>this.handleCopy(c))),this._register(U(l,"cut",c=>this.handleCopy(c))),this._register(U(l,"paste",c=>this.handlePaste(c),!0)),this._pasteProgressManager=this._register(new Nx("pasteIntoEditor",e,t)),this._postPasteWidgetManager=this._register(t.createInstance(Rx,"pasteIntoEditor",e,RW,{id:ZG,label:m("postPasteWidgetTitle","Show paste options...")}))}changePasteType(){this._postPasteWidgetManager.tryShowSelector()}pasteAs(e){this._editor.focus();try{this._pasteAsActionContext={preferred:e},fb().execCommand("paste")}finally{this._pasteAsActionContext=void 0}}clearWidgets(){this._postPasteWidgetManager.clear()}isPasteAsEnabled(){return this._editor.getOption(85).enabled}async finishedPaste(){await this._currentPasteOperation}handleCopy(e){var p,_,b,C;if(!this._editor.hasTextFocus()||((_=(p=this._clipboardService).clearInternalState)==null||_.call(p),!e.clipboardData||!this.isPasteAsEnabled()))return;const t=this._editor.getModel(),i=this._editor.getSelections();if(!t||!(i!=null&&i.length))return;const n=this._editor.getOption(37);let s=i;const r=i.length===1&&i[0].isEmpty();if(r){if(!n)return;s=[new x(s[0].startLineNumber,1,s[0].startLineNumber,1+t.getLineLength(s[0].startLineNumber))]}const a=(b=this._editor._getViewModel())==null?void 0:b.getPlainTextToCopy(i,n,En),c={multicursorText:Array.isArray(a)?a:null,pasteOnNewLine:r,mode:null},d=this._languageFeaturesService.documentPasteEditProvider.ordered(t).filter(v=>!!v.prepareDocumentPaste);if(!d.length){this.setCopyMetadata(e.clipboardData,{defaultPastePayload:c});return}const h=VG(e.clipboardData),u=d.flatMap(v=>v.copyMimeTypes??[]),g=EE();this.setCopyMetadata(e.clipboardData,{id:g,providerCopyMimeTypes:u,defaultPastePayload:c});const f=Yi(async v=>{const w=Kr(await Promise.all(d.map(async S=>{try{return await S.prepareDocumentPaste(t,s,h,v)}catch(L){console.error(L);return}})));w.reverse();for(const S of w)for(const[L,k]of S)h.replace(L,k);return h});(C=Fu._currentCopyOperation)==null||C.dataTransferPromise.cancel(),Fu._currentCopyOperation={handle:g,dataTransferPromise:f}}async handlePaste(e){var l,c,d;if(!e.clipboardData||!this._editor.hasTextFocus())return;(l=lo.get(this._editor))==null||l.closeMessage(),(c=this._currentPasteOperation)==null||c.cancel(),this._currentPasteOperation=void 0;const t=this._editor.getModel(),i=this._editor.getSelections();if(!(i!=null&&i.length)||!t||this._editor.getOption(92)||!this.isPasteAsEnabled()&&!this._pasteAsActionContext)return;const n=this.fetchCopyMetadata(e),s=zG(e.clipboardData);s.delete(MT);const r=[...e.clipboardData.types,...(n==null?void 0:n.providerCopyMimeTypes)??[],vi.uriList],a=this._languageFeaturesService.documentPasteEditProvider.ordered(t).filter(h=>{var g,f;const u=(g=this._pasteAsActionContext)==null?void 0:g.preferred;return u&&h.providedPasteEditKinds&&!this.providerMatchesPreference(h,u)?!1:(f=h.pasteMimeTypes)==null?void 0:f.some(p=>BG(p,r))});if(!a.length){(d=this._pasteAsActionContext)!=null&&d.preferred&&this.showPasteAsNoEditMessage(i,this._pasteAsActionContext.preferred);return}e.preventDefault(),e.stopImmediatePropagation(),this._pasteAsActionContext?this.showPasteAsPick(this._pasteAsActionContext.preferred,a,i,s,n):this.doPasteInline(a,i,s,n,e)}showPasteAsNoEditMessage(e,t){var i;(i=lo.get(this._editor))==null||i.showMessage(m("pasteAsError","No paste edits for '{0}' found",t instanceof ti?t.value:t.providerId),e[0].getStartPosition())}doPasteInline(e,t,i,n,s){const r=this._editor;if(!r.hasModel())return;const a=new Zh(r,3,void 0),l=Yi(async c=>{const d=this._editor;if(!d.hasModel())return;const h=d.getModel(),u=new K,g=u.add(new li(c));u.add(a.token.onCancellationRequested(()=>g.cancel()));const f=g.token;try{if(await this.mergeInDataFromCopy(i,n,f),f.isCancellationRequested)return;const p=e.filter(C=>this.isSupportedPasteProvider(C,i));if(!p.length||p.length===1&&p[0]instanceof Vf)return this.applyDefaultPasteHandler(i,n,f,s);const _={triggerKind:iw.Automatic},b=await this.getPasteEdits(p,i,h,t,_,f);if(u.add(b),f.isCancellationRequested)return;if(b.edits.length===1&&b.edits[0].provider instanceof Vf)return this.applyDefaultPasteHandler(i,n,f,s);if(b.edits.length){const C=d.getOption(85).showPasteSelector==="afterPaste";return this._postPasteWidgetManager.applyEditAndShowIfNeeded(t,{activeEditIndex:0,allEdits:b.edits},C,(v,w)=>new Promise((S,L)=>{(async()=>{var k,D;try{const N=(D=(k=v.provider).resolveDocumentPasteEdit)==null?void 0:D.call(k,v,w),B=new gb,W=N&&await this._pasteProgressManager.showWhile(t[0].getEndPosition(),m("resolveProcess","Resolving paste edit. Click to cancel"),Promise.race([B.p,N]),{cancel:()=>(B.cancel(),L(new qr))},0);return W&&(v.additionalEdit=W.additionalEdit),S(v)}catch(N){return L(N)}})()}),f)}await this.applyDefaultPasteHandler(i,n,f,s)}finally{u.dispose(),this._currentPasteOperation===l&&(this._currentPasteOperation=void 0)}});this._pasteProgressManager.showWhile(t[0].getEndPosition(),m("pasteIntoEditorProgress","Running paste handlers. Click to cancel and do basic paste"),l,{cancel:async()=>{try{if(l.cancel(),a.token.isCancellationRequested)return;await this.applyDefaultPasteHandler(i,n,a.token,s)}finally{a.dispose()}}}).then(()=>{a.dispose()}),this._currentPasteOperation=l}showPasteAsPick(e,t,i,n,s){const r=Yi(async a=>{const l=this._editor;if(!l.hasModel())return;const c=l.getModel(),d=new K,h=d.add(new Zh(l,3,void 0,a));try{if(await this.mergeInDataFromCopy(n,s,h.token),h.token.isCancellationRequested)return;let u=t.filter(b=>this.isSupportedPasteProvider(b,n,e));e&&(u=u.filter(b=>this.providerMatchesPreference(b,e)));const g={triggerKind:iw.PasteAs,only:e&&e instanceof ti?e:void 0};let f=d.add(await this.getPasteEdits(u,n,c,i,g,h.token));if(h.token.isCancellationRequested)return;if(e&&(f={edits:f.edits.filter(b=>e instanceof ti?e.contains(b.kind):e.providerId===b.provider.id),dispose:f.dispose}),!f.edits.length){g.only&&this.showPasteAsNoEditMessage(i,g.only);return}let p;if(e)p=f.edits.at(0);else{const b=await this._quickInputService.pick(f.edits.map(C=>{var v;return{label:C.title,description:(v=C.kind)==null?void 0:v.value,edit:C}}),{placeHolder:m("pasteAsPickerPlaceholder","Select Paste Action")});p=b==null?void 0:b.edit}if(!p)return;const _=KG(c.uri,i,p);await this._bulkEditService.apply(_,{editor:this._editor})}finally{d.dispose(),this._currentPasteOperation===r&&(this._currentPasteOperation=void 0)}});this._progressService.withProgress({location:10,title:m("pasteAsProgress","Running paste handlers")},()=>r)}setCopyMetadata(e,t){e.setData(MT,JSON.stringify(t))}fetchCopyMetadata(e){if(!e.clipboardData)return;const t=e.clipboardData.getData(MT);if(t)try{return JSON.parse(t)}catch{return}const[i,n]=dP.getTextData(e.clipboardData);if(n)return{defaultPastePayload:{mode:n.mode,multicursorText:n.multicursorText??null,pasteOnNewLine:!!n.isFromEmptySelection}}}async mergeInDataFromCopy(e,t,i){var n;if(t!=null&&t.id&&((n=Fu._currentCopyOperation)==null?void 0:n.handle)===t.id){const s=await Fu._currentCopyOperation.dataTransferPromise;if(i.isCancellationRequested)return;for(const[r,a]of s)e.replace(r,a)}if(!e.has(vi.uriList)){const s=await this._clipboardService.readResources();if(i.isCancellationRequested)return;s.length&&e.append(vi.uriList,IW(NE.create(s)))}}async getPasteEdits(e,t,i,n,s,r){const a=new K,l=await F1(Promise.all(e.map(async d=>{var h,u;try{const g=await((h=d.provideDocumentPasteEdits)==null?void 0:h.call(d,i,n,t,s,r));return g&&a.add(g),(u=g==null?void 0:g.edits)==null?void 0:u.map(f=>({...f,provider:d}))}catch(g){yr(g)||console.error(g);return}})),r),c=Kr(l??[]).flat().filter(d=>!s.only||s.only.contains(d.kind));return{edits:qG(c),dispose:()=>a.dispose()}}async applyDefaultPasteHandler(e,t,i,n){const s=e.get(vi.text)??e.get("text"),r=await(s==null?void 0:s.asString())??"";if(i.isCancellationRequested)return;const a={clipboardEvent:n,text:r,pasteOnNewLine:(t==null?void 0:t.defaultPastePayload.pasteOnNewLine)??!1,multicursorText:(t==null?void 0:t.defaultPastePayload.multicursorText)??null,mode:null};this._editor.trigger("keyboard","paste",a)}isSupportedPasteProvider(e,t,i){var n;return(n=e.pasteMimeTypes)!=null&&n.some(s=>t.matches(s))?!i||this.providerMatchesPreference(e,i):!1}providerMatchesPreference(e,t){return t instanceof ti?e.providedPasteEditKinds?e.providedPasteEditKinds.some(i=>t.contains(i)):!0:e.id===t.providerId}},Fu=Wg,Wg.ID="editor.contrib.copyPasteActionController",Wg);rd=Fu=JSe([Rm(1,we),Rm(2,j1),Rm(3,md),Rm(4,ge),Rm(5,xr),Rm(6,mj)],rd);const Uf="9_cutcopypaste",e0e=Pl||document.queryCommandSupported("cut"),YG=Pl||document.queryCommandSupported("copy"),t0e=typeof navigator.clipboard>"u"||pr?document.queryCommandSupported("paste"):!0;function MW(o){return o.register(),o}const i0e=e0e?MW(new pb({id:"editor.action.clipboardCutAction",precondition:void 0,kbOpts:Pl?{primary:2102,win:{primary:2102,secondary:[1044]},weight:100}:void 0,menuOpts:[{menuId:me.MenubarEditMenu,group:"2_ccp",title:m({key:"miCut",comment:["&& denotes a mnemonic"]},"Cu&&t"),order:1},{menuId:me.EditorContext,group:Uf,title:m("actions.clipboard.cutLabel","Cut"),when:I.writable,order:1},{menuId:me.CommandPalette,group:"",title:m("actions.clipboard.cutLabel","Cut"),order:1},{menuId:me.SimpleEditorContext,group:Uf,title:m("actions.clipboard.cutLabel","Cut"),when:I.writable,order:1}]})):void 0,n0e=YG?MW(new pb({id:"editor.action.clipboardCopyAction",precondition:void 0,kbOpts:Pl?{primary:2081,win:{primary:2081,secondary:[2067]},weight:100}:void 0,menuOpts:[{menuId:me.MenubarEditMenu,group:"2_ccp",title:m({key:"miCopy",comment:["&& denotes a mnemonic"]},"&&Copy"),order:2},{menuId:me.EditorContext,group:Uf,title:m("actions.clipboard.copyLabel","Copy"),order:2},{menuId:me.CommandPalette,group:"",title:m("actions.clipboard.copyLabel","Copy"),order:1},{menuId:me.SimpleEditorContext,group:Uf,title:m("actions.clipboard.copyLabel","Copy"),order:2}]})):void 0;rn.appendMenuItem(me.MenubarEditMenu,{submenu:me.MenubarCopy,title:Te("copy as","Copy As"),group:"2_ccp",order:3});rn.appendMenuItem(me.EditorContext,{submenu:me.EditorContextCopy,title:Te("copy as","Copy As"),group:Uf,order:3});rn.appendMenuItem(me.EditorContext,{submenu:me.EditorContextShare,title:Te("share","Share"),group:"11_share",order:-1,when:G.and(G.notEquals("resourceScheme","output"),I.editorTextFocus)});rn.appendMenuItem(me.ExplorerContext,{submenu:me.ExplorerContextShare,title:Te("share","Share"),group:"11_share",order:-1});const AT=t0e?MW(new pb({id:"editor.action.clipboardPasteAction",precondition:void 0,kbOpts:Pl?{primary:2100,win:{primary:2100,secondary:[1043]},linux:{primary:2100,secondary:[1043]},weight:100}:void 0,menuOpts:[{menuId:me.MenubarEditMenu,group:"2_ccp",title:m({key:"miPaste",comment:["&& denotes a mnemonic"]},"&&Paste"),order:4},{menuId:me.EditorContext,group:Uf,title:m("actions.clipboard.pasteLabel","Paste"),when:I.writable,order:4},{menuId:me.CommandPalette,group:"",title:m("actions.clipboard.pasteLabel","Paste"),order:1},{menuId:me.SimpleEditorContext,group:Uf,title:m("actions.clipboard.pasteLabel","Paste"),when:I.writable,order:4}]})):void 0;class s0e extends de{constructor(){super({id:"editor.action.clipboardCopyWithSyntaxHighlightingAction",label:m("actions.clipboard.copyWithSyntaxHighlightingLabel","Copy With Syntax Highlighting"),alias:"Copy With Syntax Highlighting",precondition:void 0,kbOpts:{kbExpr:I.textInputFocus,primary:0,weight:100}})}run(e,t){!t.hasModel()||!t.getOption(37)&&t.getSelection().isEmpty()||(lP.forceCopyWithSyntaxHighlighting=!0,t.focus(),t.getContainerDomNode().ownerDocument.execCommand("copy"),lP.forceCopyWithSyntaxHighlighting=!1)}}function XG(o,e){o&&(o.addImplementation(1e4,"code-editor",(t,i)=>{const n=t.get(pt).getFocusedCodeEditor();if(n&&n.hasTextFocus()){const s=n.getOption(37),r=n.getSelection();return r&&r.isEmpty()&&!s||n.getContainerDomNode().ownerDocument.execCommand(e),!0}return!1}),o.addImplementation(0,"generic-dom",(t,i)=>(fb().execCommand(e),!0)))}XG(i0e,"cut");XG(n0e,"copy");AT&&(AT.addImplementation(1e4,"code-editor",(o,e)=>{var s;const t=o.get(pt),i=o.get(md),n=t.getFocusedCodeEditor();return n&&n.hasTextFocus()?n.getContainerDomNode().ownerDocument.execCommand("paste")?((s=rd.get(n))==null?void 0:s.finishedPaste())??Promise.resolve():Qf?(async()=>{const a=await i.readText();if(a!==""){const l=jw.INSTANCE.get(a);let c=!1,d=null,h=null;l&&(c=n.getOption(37)&&!!l.isFromEmptySelection,d=typeof l.multicursorText<"u"?l.multicursorText:null,h=l.mode),n.trigger("keyboard","paste",{text:a,pasteOnNewLine:c,multicursorText:d,mode:h})}})():!0:!1}),AT.addImplementation(0,"generic-dom",(o,e)=>(fb().execCommand("paste"),!0)));YG&&Q(s0e);const Yt=new class{constructor(){this.QuickFix=new ti("quickfix"),this.Refactor=new ti("refactor"),this.RefactorExtract=this.Refactor.append("extract"),this.RefactorInline=this.Refactor.append("inline"),this.RefactorMove=this.Refactor.append("move"),this.RefactorRewrite=this.Refactor.append("rewrite"),this.Notebook=new ti("notebook"),this.Source=new ti("source"),this.SourceOrganizeImports=this.Source.append("organizeImports"),this.SourceFixAll=this.Source.append("fixAll"),this.SurroundWith=this.Refactor.append("surround")}};var fo;(function(o){o.Refactor="refactor",o.RefactorPreview="refactor preview",o.Lightbulb="lightbulb",o.Default="other (default)",o.SourceAction="source action",o.QuickFix="quick fix action",o.FixAll="fix all",o.OrganizeImports="organize imports",o.AutoFix="auto fix",o.QuickFixHover="quick fix hover window",o.OnSave="save participants",o.ProblemsView="problems view"})(fo||(fo={}));function o0e(o,e){return!(o.include&&!o.include.intersects(e)||o.excludes&&o.excludes.some(t=>QG(e,t,o.include))||!o.includeSourceActions&&Yt.Source.contains(e))}function r0e(o,e){const t=e.kind?new ti(e.kind):void 0;return!(o.include&&(!t||!o.include.contains(t))||o.excludes&&t&&o.excludes.some(i=>QG(t,i,o.include))||!o.includeSourceActions&&t&&Yt.Source.contains(t)||o.onlyIncludePreferredActions&&!e.isPreferred)}function QG(o,e,t){return!(!e.contains(o)||t&&e.contains(t))}class xl{static fromUser(e,t){return!e||typeof e!="object"?new xl(t.kind,t.apply,!1):new xl(xl.getKindFromUser(e,t.kind),xl.getApplyFromUser(e,t.apply),xl.getPreferredUser(e))}static getApplyFromUser(e,t){switch(typeof e.apply=="string"?e.apply.toLowerCase():""){case"first":return"first";case"never":return"never";case"ifsingle":return"ifSingle";default:return t}}static getKindFromUser(e,t){return typeof e.kind=="string"?new ti(e.kind):t}static getPreferredUser(e){return typeof e.preferred=="boolean"?e.preferred:!1}constructor(e,t,i){this.kind=e,this.apply=t,this.preferred=i}}class a0e{constructor(e,t,i){this.action=e,this.provider=t,this.highlightRange=i}async resolve(e){var t;if((t=this.provider)!=null&&t.resolveCodeAction&&!this.action.edit){let i;try{i=await this.provider.resolveCodeAction(this.action,e)}catch(n){Si(n)}i&&(this.action.edit=i.edit)}return this}}const JG="editor.action.codeAction",AW="editor.action.quickFix",eZ="editor.action.autoFix",tZ="editor.action.refactor",iZ="editor.action.sourceAction",sO="editor.action.organizeImports",oO="editor.action.fixAll";class vv extends F{static codeActionsPreferredComparator(e,t){return e.isPreferred&&!t.isPreferred?-1:!e.isPreferred&&t.isPreferred?1:0}static codeActionsComparator({action:e},{action:t}){return e.isAI&&!t.isAI?1:!e.isAI&&t.isAI?-1:Qn(e.diagnostics)?Qn(t.diagnostics)?vv.codeActionsPreferredComparator(e,t):-1:Qn(t.diagnostics)?1:vv.codeActionsPreferredComparator(e,t)}constructor(e,t,i){super(),this.documentation=t,this._register(i),this.allActions=[...e].sort(vv.codeActionsComparator),this.validActions=this.allActions.filter(({action:n})=>!n.disabled)}get hasAutoFix(){return this.validActions.some(({action:e})=>!!e.kind&&Yt.QuickFix.contains(new ti(e.kind))&&!!e.isPreferred)}get hasAIFix(){return this.validActions.some(({action:e})=>!!e.isAI)}get allAIFixes(){return this.validActions.every(({action:e})=>!!e.isAI)}}const IV={actions:[],documentation:void 0};async function pp(o,e,t,i,n,s){var p;const r=i.filter||{},a={...r,excludes:[...r.excludes||[],Yt.Notebook]},l={only:(p=r.include)==null?void 0:p.value,trigger:i.type},c=new SW(e,s),d=i.type===2,h=l0e(o,e,d?a:r),u=new K,g=h.map(async _=>{try{n.report(_);const b=await _.provideCodeActions(e,t,l,c.token);if(b&&u.add(b),c.token.isCancellationRequested)return IV;const C=((b==null?void 0:b.actions)||[]).filter(w=>w&&r0e(r,w)),v=d0e(_,C,r.include);return{actions:C.map(w=>new a0e(w,_)),documentation:v}}catch(b){if(yr(b))throw b;return Si(b),IV}}),f=o.onDidChange(()=>{const _=o.all(e);ii(_,h)||c.cancel()});try{const _=await Promise.all(g),b=_.map(v=>v.actions).flat(),C=[...Kr(_.map(v=>v.documentation)),...c0e(o,e,i,b)];return new vv(b,C,u)}finally{f.dispose(),c.dispose()}}function l0e(o,e,t){return o.all(e).filter(i=>i.providedCodeActionKinds?i.providedCodeActionKinds.some(n=>o0e(t,new ti(n))):!0)}function*c0e(o,e,t,i){var n,s,r;if(e&&i.length)for(const a of o.all(e))a._getAdditionalMenuItems&&(yield*(r=a._getAdditionalMenuItems)==null?void 0:r.call(a,{trigger:t.type,only:(s=(n=t.filter)==null?void 0:n.include)==null?void 0:s.value},i.map(l=>l.action)))}function d0e(o,e,t){if(!o.documentation)return;const i=o.documentation.map(n=>({kind:new ti(n.kind),command:n.command}));if(t){let n;for(const s of i)s.kind.contains(t)&&(n?n.kind.contains(s.kind)&&(n=s):n=s);if(n)return n==null?void 0:n.command}for(const n of e)if(n.kind){for(const s of i)if(s.kind.contains(new ti(n.kind)))return s.command}}var ug;(function(o){o.OnSave="onSave",o.FromProblemsView="fromProblemsView",o.FromCodeActions="fromCodeActions",o.FromAILightbulb="fromAILightbulb"})(ug||(ug={}));async function h0e(o,e,t,i,n=ze.None){var c;const s=o.get(j1),r=o.get(Ht),a=o.get(Bn),l=o.get(Ei);if(a.publicLog2("codeAction.applyCodeAction",{codeActionTitle:e.action.title,codeActionKind:e.action.kind,codeActionIsPreferred:!!e.action.isPreferred,reason:t}),await e.resolve(n),!n.isCancellationRequested&&!((c=e.action.edit)!=null&&c.edits.length&&!(await s.apply(e.action.edit,{editor:i==null?void 0:i.editor,label:e.action.title,quotableLabel:e.action.title,code:"undoredo.codeAction",respectAutoSaveConfig:t!==ug.OnSave,showPreview:i==null?void 0:i.preview})).isApplied)&&e.action.command)try{await r.executeCommand(e.action.command.id,...e.action.command.arguments||[])}catch(d){const h=u0e(d);l.error(typeof h=="string"?h:m("applyCodeActionFailed","An unknown error occurred while applying the code action"))}}function u0e(o){return typeof o=="string"?o:o instanceof Error&&typeof o.message=="string"?o.message:void 0}rt.registerCommand("_executeCodeActionProvider",async function(o,e,t,i,n){if(!(e instanceof Se))throw gr();const{codeActionProvider:s}=o.get(ge),r=o.get(Kt).getModel(e);if(!r)throw gr();const a=pe.isISelection(t)?pe.liftSelection(t):x.isIRange(t)?r.validateRange(t):void 0;if(!a)throw gr();const l=typeof i=="string"?new ti(i):void 0,c=await pp(s,r,a,{type:1,triggerAction:fo.Default,filter:{includeSourceActions:!0,include:l}},ka.None,ze.None),d=[],h=Math.min(c.validActions.length,typeof n=="number"?n:0);for(let u=0;u<h;u++)d.push(c.validActions[u].resolve(ze.None));try{return await Promise.all(d),c.validActions.map(u=>u.action)}finally{setTimeout(()=>c.dispose(),100)}});var g0e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},f0e=function(o,e){return function(t,i){e(t,i,o)}},rO,Bg;let aO=(Bg=class{constructor(e){this.keybindingService=e}getResolver(){const e=new Oa(()=>this.keybindingService.getKeybindings().filter(t=>rO.codeActionCommands.indexOf(t.command)>=0).filter(t=>t.resolvedKeybinding).map(t=>{let i=t.commandArgs;return t.command===sO?i={kind:Yt.SourceOrganizeImports.value}:t.command===oO&&(i={kind:Yt.SourceFixAll.value}),{resolvedKeybinding:t.resolvedKeybinding,...xl.fromUser(i,{kind:ti.None,apply:"never"})}}));return t=>{if(t.kind){const i=this.bestKeybindingForCodeAction(t,e.value);return i==null?void 0:i.resolvedKeybinding}}}bestKeybindingForCodeAction(e,t){if(!e.kind)return;const i=new ti(e.kind);return t.filter(n=>n.kind.contains(i)).filter(n=>n.preferred?e.isPreferred:!0).reduceRight((n,s)=>n?n.kind.contains(s.kind)?s:n:s,void 0)}},rO=Bg,Bg.codeActionCommands=[tZ,JG,iZ,sO,oO],Bg);aO=rO=g0e([f0e(0,yt)],aO);E("symbolIcon.arrayForeground",Ze,m("symbolIcon.arrayForeground","The foreground color for array symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.booleanForeground",Ze,m("symbolIcon.booleanForeground","The foreground color for boolean symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.classForeground",{dark:"#EE9D28",light:"#D67E00",hcDark:"#EE9D28",hcLight:"#D67E00"},m("symbolIcon.classForeground","The foreground color for class symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.colorForeground",Ze,m("symbolIcon.colorForeground","The foreground color for color symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.constantForeground",Ze,m("symbolIcon.constantForeground","The foreground color for constant symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.constructorForeground",{dark:"#B180D7",light:"#652D90",hcDark:"#B180D7",hcLight:"#652D90"},m("symbolIcon.constructorForeground","The foreground color for constructor symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.enumeratorForeground",{dark:"#EE9D28",light:"#D67E00",hcDark:"#EE9D28",hcLight:"#D67E00"},m("symbolIcon.enumeratorForeground","The foreground color for enumerator symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.enumeratorMemberForeground",{dark:"#75BEFF",light:"#007ACC",hcDark:"#75BEFF",hcLight:"#007ACC"},m("symbolIcon.enumeratorMemberForeground","The foreground color for enumerator member symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.eventForeground",{dark:"#EE9D28",light:"#D67E00",hcDark:"#EE9D28",hcLight:"#D67E00"},m("symbolIcon.eventForeground","The foreground color for event symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.fieldForeground",{dark:"#75BEFF",light:"#007ACC",hcDark:"#75BEFF",hcLight:"#007ACC"},m("symbolIcon.fieldForeground","The foreground color for field symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.fileForeground",Ze,m("symbolIcon.fileForeground","The foreground color for file symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.folderForeground",Ze,m("symbolIcon.folderForeground","The foreground color for folder symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.functionForeground",{dark:"#B180D7",light:"#652D90",hcDark:"#B180D7",hcLight:"#652D90"},m("symbolIcon.functionForeground","The foreground color for function symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.interfaceForeground",{dark:"#75BEFF",light:"#007ACC",hcDark:"#75BEFF",hcLight:"#007ACC"},m("symbolIcon.interfaceForeground","The foreground color for interface symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.keyForeground",Ze,m("symbolIcon.keyForeground","The foreground color for key symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.keywordForeground",Ze,m("symbolIcon.keywordForeground","The foreground color for keyword symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.methodForeground",{dark:"#B180D7",light:"#652D90",hcDark:"#B180D7",hcLight:"#652D90"},m("symbolIcon.methodForeground","The foreground color for method symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.moduleForeground",Ze,m("symbolIcon.moduleForeground","The foreground color for module symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.namespaceForeground",Ze,m("symbolIcon.namespaceForeground","The foreground color for namespace symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.nullForeground",Ze,m("symbolIcon.nullForeground","The foreground color for null symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.numberForeground",Ze,m("symbolIcon.numberForeground","The foreground color for number symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.objectForeground",Ze,m("symbolIcon.objectForeground","The foreground color for object symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.operatorForeground",Ze,m("symbolIcon.operatorForeground","The foreground color for operator symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.packageForeground",Ze,m("symbolIcon.packageForeground","The foreground color for package symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.propertyForeground",Ze,m("symbolIcon.propertyForeground","The foreground color for property symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.referenceForeground",Ze,m("symbolIcon.referenceForeground","The foreground color for reference symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.snippetForeground",Ze,m("symbolIcon.snippetForeground","The foreground color for snippet symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.stringForeground",Ze,m("symbolIcon.stringForeground","The foreground color for string symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.structForeground",Ze,m("symbolIcon.structForeground","The foreground color for struct symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.textForeground",Ze,m("symbolIcon.textForeground","The foreground color for text symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.typeParameterForeground",Ze,m("symbolIcon.typeParameterForeground","The foreground color for type parameter symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.unitForeground",Ze,m("symbolIcon.unitForeground","The foreground color for unit symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));E("symbolIcon.variableForeground",{dark:"#75BEFF",light:"#007ACC",hcDark:"#75BEFF",hcLight:"#007ACC"},m("symbolIcon.variableForeground","The foreground color for variable symbols. These symbols appear in the outline, breadcrumb, and suggest widget."));const nZ=Object.freeze({kind:ti.Empty,title:m("codeAction.widget.id.more","More Actions...")}),m0e=Object.freeze([{kind:Yt.QuickFix,title:m("codeAction.widget.id.quickfix","Quick Fix")},{kind:Yt.RefactorExtract,title:m("codeAction.widget.id.extract","Extract"),icon:te.wrench},{kind:Yt.RefactorInline,title:m("codeAction.widget.id.inline","Inline"),icon:te.wrench},{kind:Yt.RefactorRewrite,title:m("codeAction.widget.id.convert","Rewrite"),icon:te.wrench},{kind:Yt.RefactorMove,title:m("codeAction.widget.id.move","Move"),icon:te.wrench},{kind:Yt.SurroundWith,title:m("codeAction.widget.id.surround","Surround With"),icon:te.surroundWith},{kind:Yt.Source,title:m("codeAction.widget.id.source","Source Action"),icon:te.symbolFile},nZ]);function p0e(o,e,t){if(!e)return o.map(s=>{var r;return{kind:"action",item:s,group:nZ,disabled:!!s.action.disabled,label:s.action.disabled||s.action.title,canPreview:!!((r=s.action.edit)!=null&&r.edits.length)}});const i=m0e.map(s=>({group:s,actions:[]}));for(const s of o){const r=s.action.kind?new ti(s.action.kind):ti.None;for(const a of i)if(a.group.kind.contains(r)){a.actions.push(s);break}}const n=[];for(const s of i)if(s.actions.length){n.push({kind:"header",group:s.group});for(const r of s.actions){const a=s.group;n.push({kind:"action",item:r,group:r.action.isAI?{title:a.title,kind:a.kind,icon:te.sparkle}:a,label:r.action.title,disabled:!!r.action.disabled,keybinding:t(r.action)})}}return n}var _0e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},b0e=function(o,e){return function(t,i){e(t,i,o)}},Km;const EV=qt("gutter-lightbulb",te.lightBulb,m("gutterLightbulbWidget","Icon which spawns code actions menu from the gutter when there is no space in the editor.")),NV=qt("gutter-lightbulb-auto-fix",te.lightbulbAutofix,m("gutterLightbulbAutoFixWidget","Icon which spawns code actions menu from the gutter when there is no space in the editor and a quick fix is available.")),TV=qt("gutter-lightbulb-sparkle",te.lightbulbSparkle,m("gutterLightbulbAIFixWidget","Icon which spawns code actions menu from the gutter when there is no space in the editor and an AI fix is available.")),RV=qt("gutter-lightbulb-aifix-auto-fix",te.lightbulbSparkleAutofix,m("gutterLightbulbAIFixAutoFixWidget","Icon which spawns code actions menu from the gutter when there is no space in the editor and an AI fix and a quick fix is available.")),MV=qt("gutter-lightbulb-sparkle-filled",te.sparkleFilled,m("gutterLightbulbSparkleFilledWidget","Icon which spawns code actions menu from the gutter when there is no space in the editor and an AI fix and a quick fix is available."));var ua;(function(o){o.Hidden={type:0};class e{constructor(i,n,s,r){this.actions=i,this.trigger=n,this.editorPosition=s,this.widgetPosition=r,this.type=1}}o.Showing=e})(ua||(ua={}));var zc;let o1=(zc=class extends F{constructor(e,t){super(),this._editor=e,this._keybindingService=t,this._onClick=this._register(new P),this.onClick=this._onClick.event,this._state=ua.Hidden,this._gutterState=ua.Hidden,this._iconClasses=[],this.lightbulbClasses=["codicon-"+EV.id,"codicon-"+RV.id,"codicon-"+NV.id,"codicon-"+TV.id,"codicon-"+MV.id],this.gutterDecoration=Km.GUTTER_DECORATION,this._domNode=ne("div.lightBulbWidget"),this._domNode.role="listbox",this._register(Fn.ignoreTarget(this._domNode)),this._editor.addContentWidget(this),this._register(this._editor.onDidChangeModelContent(i=>{const n=this._editor.getModel();(this.state.type!==1||!n||this.state.editorPosition.lineNumber>=n.getLineCount())&&this.hide(),(this.gutterState.type!==1||!n||this.gutterState.editorPosition.lineNumber>=n.getLineCount())&&this.gutterHide()})),this._register(Fte(this._domNode,i=>{if(this.state.type!==1)return;this._editor.focus(),i.preventDefault();const{top:n,height:s}=wi(this._domNode),r=this._editor.getOption(67);let a=Math.floor(r/3);this.state.widgetPosition.position!==null&&this.state.widgetPosition.position.lineNumber<this.state.editorPosition.lineNumber&&(a+=r),this._onClick.fire({x:i.posx,y:n+s+a,actions:this.state.actions,trigger:this.state.trigger})})),this._register(U(this._domNode,"mouseenter",i=>{(i.buttons&1)===1&&this.hide()})),this._register(oe.runAndSubscribe(this._keybindingService.onDidUpdateKeybindings,()=>{var i,n;this._preferredKbLabel=((i=this._keybindingService.lookupKeybinding(eZ))==null?void 0:i.getLabel())??void 0,this._quickFixKbLabel=((n=this._keybindingService.lookupKeybinding(AW))==null?void 0:n.getLabel())??void 0,this._updateLightBulbTitleAndIcon()})),this._register(this._editor.onMouseDown(async i=>{if(!i.target.element||!this.lightbulbClasses.some(l=>i.target.element&&i.target.element.classList.contains(l))||this.gutterState.type!==1)return;this._editor.focus();const{top:n,height:s}=wi(i.target.element),r=this._editor.getOption(67);let a=Math.floor(r/3);this.gutterState.widgetPosition.position!==null&&this.gutterState.widgetPosition.position.lineNumber<this.gutterState.editorPosition.lineNumber&&(a+=r),this._onClick.fire({x:i.event.posx,y:n+s+a,actions:this.gutterState.actions,trigger:this.gutterState.trigger})}))}dispose(){super.dispose(),this._editor.removeContentWidget(this),this._gutterDecorationID&&this._removeGutterDecoration(this._gutterDecorationID)}getId(){return"LightBulbWidget"}getDomNode(){return this._domNode}getPosition(){return this._state.type===1?this._state.widgetPosition:null}update(e,t,i){if(e.validActions.length<=0)return this.gutterHide(),this.hide();if(!this._editor.hasTextFocus())return this.gutterHide(),this.hide();if(!this._editor.getOptions().get(65).enabled)return this.gutterHide(),this.hide();const r=this._editor.getModel();if(!r)return this.gutterHide(),this.hide();const{lineNumber:a,column:l}=r.validatePosition(i),c=r.getOptions().tabSize,d=this._editor.getOptions().get(50),h=r.getLineContent(a),u=rE(h,c),g=d.spaceWidth*u>22,f=S=>S>2&&this._editor.getTopForLineNumber(S)===this._editor.getTopForLineNumber(S-1),p=this._editor.getLineDecorations(a);let _=!1;if(p)for(const S of p){const L=S.options.glyphMarginClassName;if(L&&!this.lightbulbClasses.some(k=>L.includes(k))){_=!0;break}}let b=a,C=1;if(!g){const S=L=>{const k=r.getLineContent(L);return/^\s*$|^\s+/.test(k)||k.length<=C};if(a>1&&!f(a-1)){const L=r.getLineCount(),k=a===L,D=a>1&&S(a-1),N=!k&&S(a+1),B=S(a),W=!N&&!D;if(!N&&!D&&!_)return this.gutterState=new ua.Showing(e,t,i,{position:{lineNumber:b,column:C},preference:Km._posPref}),this.renderGutterLightbub(),this.hide();D||k||D&&!B?b-=1:(N||W&&B)&&(b+=1)}else if(a===1&&(a===r.getLineCount()||!S(a+1)&&!S(a)))if(this.gutterState=new ua.Showing(e,t,i,{position:{lineNumber:b,column:C},preference:Km._posPref}),_)this.gutterHide();else return this.renderGutterLightbub(),this.hide();else if(a<r.getLineCount()&&!f(a+1))b+=1;else if(l*d.spaceWidth<22)return this.hide();C=/^\S\s*$/.test(r.getLineContent(b))?2:1}this.state=new ua.Showing(e,t,i,{position:{lineNumber:b,column:C},preference:Km._posPref}),this._gutterDecorationID&&(this._removeGutterDecoration(this._gutterDecorationID),this.gutterHide());const v=e.validActions,w=e.validActions[0].action.kind;if(v.length!==1||!w){this._editor.layoutContentWidget(this);return}this._editor.layoutContentWidget(this)}hide(){this.state!==ua.Hidden&&(this.state=ua.Hidden,this._editor.layoutContentWidget(this))}gutterHide(){this.gutterState!==ua.Hidden&&(this._gutterDecorationID&&this._removeGutterDecoration(this._gutterDecorationID),this.gutterState=ua.Hidden)}get state(){return this._state}set state(e){this._state=e,this._updateLightBulbTitleAndIcon()}get gutterState(){return this._gutterState}set gutterState(e){this._gutterState=e,this._updateGutterLightBulbTitleAndIcon()}_updateLightBulbTitleAndIcon(){if(this._domNode.classList.remove(...this._iconClasses),this._iconClasses=[],this.state.type!==1)return;let e,t=!1;this.state.actions.allAIFixes?(e=te.sparkleFilled,this.state.actions.validActions.length===1&&(t=!0)):this.state.actions.hasAutoFix?this.state.actions.hasAIFix?e=te.lightbulbSparkleAutofix:e=te.lightbulbAutofix:this.state.actions.hasAIFix?e=te.lightbulbSparkle:e=te.lightBulb,this._updateLightbulbTitle(this.state.actions.hasAutoFix,t),this._iconClasses=Ie.asClassNameArray(e),this._domNode.classList.add(...this._iconClasses)}_updateGutterLightBulbTitleAndIcon(){if(this.gutterState.type!==1)return;let e,t=!1;this.gutterState.actions.allAIFixes?(e=MV,this.gutterState.actions.validActions.length===1&&(t=!0)):this.gutterState.actions.hasAutoFix?this.gutterState.actions.hasAIFix?e=RV:e=NV:this.gutterState.actions.hasAIFix?e=TV:e=EV,this._updateLightbulbTitle(this.gutterState.actions.hasAutoFix,t);const i=We.register({description:"codicon-gutter-lightbulb-decoration",glyphMarginClassName:Ie.asClassName(e),glyphMargin:{position:Zr.Left},stickiness:1});this.gutterDecoration=i}renderGutterLightbub(){const e=this._editor.getSelection();e&&(this._gutterDecorationID===void 0?this._addGutterDecoration(e.startLineNumber):this._updateGutterDecoration(this._gutterDecorationID,e.startLineNumber))}_addGutterDecoration(e){this._editor.changeDecorations(t=>{this._gutterDecorationID=t.addDecoration(new x(e,0,e,0),this.gutterDecoration)})}_removeGutterDecoration(e){this._editor.changeDecorations(t=>{t.removeDecoration(e),this._gutterDecorationID=void 0})}_updateGutterDecoration(e,t){this._editor.changeDecorations(i=>{i.changeDecoration(e,new x(t,0,t,0)),i.changeDecorationOptions(e,this.gutterDecoration)})}_updateLightbulbTitle(e,t){this.state.type===1&&(t?this.title=m("codeActionAutoRun","Run: {0}",this.state.actions.validActions[0].action.title):e&&this._preferredKbLabel?this.title=m("preferredcodeActionWithKb","Show Code Actions. Preferred Quick Fix Available ({0})",this._preferredKbLabel):!e&&this._quickFixKbLabel?this.title=m("codeActionWithKb","Show Code Actions ({0})",this._quickFixKbLabel):e||(this.title=m("codeAction","Show Code Actions")))}set title(e){this._domNode.title=e}},Km=zc,zc.GUTTER_DECORATION=We.register({description:"codicon-gutter-lightbulb-decoration",glyphMarginClassName:Ie.asClassName(te.lightBulb),glyphMargin:{position:Zr.Left},stickiness:1}),zc.ID="editor.contrib.lightbulbWidget",zc._posPref=[0],zc);o1=Km=_0e([b0e(1,yt)],o1);var sZ=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},lO=function(o,e){return function(t,i){e(t,i,o)}};const oZ="acceptSelectedCodeAction",rZ="previewSelectedCodeAction";class C0e{get templateId(){return"header"}renderTemplate(e){e.classList.add("group-header");const t=document.createElement("span");return e.append(t),{container:e,text:t}}renderElement(e,t,i){var n;i.text.textContent=((n=e.group)==null?void 0:n.title)??""}disposeTemplate(e){}}let cO=class{get templateId(){return"action"}constructor(e,t){this._supportsPreview=e,this._keybindingService=t}renderTemplate(e){e.classList.add(this.templateId);const t=document.createElement("div");t.className="icon",e.append(t);const i=document.createElement("span");i.className="title",e.append(i);const n=new Ib(e,so);return{container:e,icon:t,text:i,keybinding:n}}renderElement(e,t,i){var r,a,l;if((r=e.group)!=null&&r.icon?(i.icon.className=Ie.asClassName(e.group.icon),e.group.icon.color&&(i.icon.style.color=ce(e.group.icon.color.id))):(i.icon.className=Ie.asClassName(te.lightBulb),i.icon.style.color="var(--vscode-editorLightBulb-foreground)"),!e.item||!e.label)return;i.text.textContent=aZ(e.label),i.keybinding.set(e.keybinding),Qte(!!e.keybinding,i.keybinding.element);const n=(a=this._keybindingService.lookupKeybinding(oZ))==null?void 0:a.getLabel(),s=(l=this._keybindingService.lookupKeybinding(rZ))==null?void 0:l.getLabel();i.container.classList.toggle("option-disabled",e.disabled),e.disabled?i.container.title=e.label:n&&s?this._supportsPreview&&e.canPreview?i.container.title=m({key:"label-preview",comment:['placeholders are keybindings, e.g "F2 to Apply, Shift+F2 to Preview"']},"{0} to Apply, {1} to Preview",n,s):i.container.title=m({key:"label",comment:['placeholder is a keybinding, e.g "F2 to Apply"']},"{0} to Apply",n):i.container.title=""}disposeTemplate(e){e.keybinding.dispose()}};cO=sZ([lO(1,yt)],cO);class v0e extends UIEvent{constructor(){super("acceptSelectedAction")}}class AV extends UIEvent{constructor(){super("previewSelectedAction")}}function w0e(o){if(o.kind==="action")return o.label}let dO=class extends F{constructor(e,t,i,n,s,r){super(),this._delegate=n,this._contextViewService=s,this._keybindingService=r,this._actionLineHeight=24,this._headerLineHeight=26,this.cts=this._register(new li),this.domNode=document.createElement("div"),this.domNode.classList.add("actionList");const a={getHeight:l=>l.kind==="header"?this._headerLineHeight:this._actionLineHeight,getTemplateId:l=>l.kind};this._list=this._register(new Xo(e,this.domNode,a,[new cO(t,this._keybindingService),new C0e],{keyboardSupport:!1,typeNavigationEnabled:!0,keyboardNavigationLabelProvider:{getKeyboardNavigationLabel:w0e},accessibilityProvider:{getAriaLabel:l=>{if(l.kind==="action"){let c=l.label?aZ(l==null?void 0:l.label):"";return l.disabled&&(c=m({key:"customQuickFixWidget.labels",comment:["Action widget labels for accessibility."]},"{0}, Disabled Reason: {1}",c,l.disabled)),c}return null},getWidgetAriaLabel:()=>m({key:"customQuickFixWidget",comment:["An action widget option"]},"Action Widget"),getRole:l=>l.kind==="action"?"option":"separator",getWidgetRole:()=>"listbox"}})),this._list.style(nm),this._register(this._list.onMouseClick(l=>this.onListClick(l))),this._register(this._list.onMouseOver(l=>this.onListHover(l))),this._register(this._list.onDidChangeFocus(()=>this.onFocus())),this._register(this._list.onDidChangeSelection(l=>this.onListSelection(l))),this._allMenuItems=i,this._list.splice(0,this._list.length,this._allMenuItems),this._list.length&&this.focusNext()}focusCondition(e){return!e.disabled&&e.kind==="action"}hide(e){this._delegate.onHide(e),this.cts.cancel(),this._contextViewService.hideContextView()}layout(e){const t=this._allMenuItems.filter(l=>l.kind==="header").length,n=this._allMenuItems.length*this._actionLineHeight+t*this._headerLineHeight-t*this._actionLineHeight;this._list.layout(n);let s=e;if(this._allMenuItems.length>=50)s=380;else{const l=this._allMenuItems.map((c,d)=>{const h=this.domNode.ownerDocument.getElementById(this._list.getElementID(d));if(h){h.style.width="auto";const u=h.getBoundingClientRect().width;return h.style.width="",u}return 0});s=Math.max(...l,e)}const a=Math.min(n,this.domNode.ownerDocument.body.clientHeight*.7);return this._list.layout(a,s),this.domNode.style.height=`${a}px`,this._list.domFocus(),s}focusPrevious(){this._list.focusPrevious(1,!0,void 0,this.focusCondition)}focusNext(){this._list.focusNext(1,!0,void 0,this.focusCondition)}acceptSelected(e){const t=this._list.getFocus();if(t.length===0)return;const i=t[0],n=this._list.element(i);if(!this.focusCondition(n))return;const s=e?new AV:new v0e;this._list.setSelection([i],s)}onListSelection(e){if(!e.elements.length)return;const t=e.elements[0];t.item&&this.focusCondition(t)?this._delegate.onSelect(t.item,e.browserEvent instanceof AV):this._list.setSelection([])}onFocus(){var n,s;const e=this._list.getFocus();if(e.length===0)return;const t=e[0],i=this._list.element(t);(s=(n=this._delegate).onFocus)==null||s.call(n,i.item)}async onListHover(e){const t=e.element;if(t&&t.item&&this.focusCondition(t)){if(this._delegate.onHover&&!t.disabled&&t.kind==="action"){const i=await this._delegate.onHover(t.item,this.cts.token);t.canPreview=i?i.canPreview:void 0}e.index&&this._list.splice(e.index,1,[t])}this._list.setFocus(typeof e.index=="number"?[e.index]:[])}onListClick(e){e.element&&this.focusCondition(e.element)&&this._list.setFocus([])}};dO=sZ([lO(4,ud),lO(5,yt)],dO);function aZ(o){return o.replace(/\r\n|\r|\n/g," ")}var S0e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},PT=function(o,e){return function(t,i){e(t,i,o)}};E("actionBar.toggledBackground",V1,m("actionBar.toggledBackground","Background color for toggled action items in action bar."));const $f={Visible:new le("codeActionMenuVisible",!1,m("codeActionMenuVisible","Whether the action widget list is visible"))},dm=et("actionWidgetService");let jf=class extends F{get isVisible(){return $f.Visible.getValue(this._contextKeyService)||!1}constructor(e,t,i){super(),this._contextViewService=e,this._contextKeyService=t,this._instantiationService=i,this._list=this._register(new Ji)}show(e,t,i,n,s,r,a){const l=$f.Visible.bindTo(this._contextKeyService),c=this._instantiationService.createInstance(dO,e,t,i,n);this._contextViewService.showContextView({getAnchor:()=>s,render:d=>(l.set(!0),this._renderWidget(d,c,a??[])),onHide:d=>{l.reset(),this._onWidgetClosed(d)}},r,!1)}acceptSelected(e){var t;(t=this._list.value)==null||t.acceptSelected(e)}focusPrevious(){var e,t;(t=(e=this._list)==null?void 0:e.value)==null||t.focusPrevious()}focusNext(){var e,t;(t=(e=this._list)==null?void 0:e.value)==null||t.focusNext()}hide(e){var t;(t=this._list.value)==null||t.hide(e),this._list.clear()}_renderWidget(e,t,i){var g;const n=document.createElement("div");if(n.classList.add("action-widget"),e.appendChild(n),this._list.value=t,this._list.value)n.appendChild(this._list.value.domNode);else throw new Error("List has no value");const s=new K,r=document.createElement("div"),a=e.appendChild(r);a.classList.add("context-view-block"),s.add(U(a,ie.MOUSE_DOWN,f=>f.stopPropagation()));const l=document.createElement("div"),c=e.appendChild(l);c.classList.add("context-view-pointerBlock"),s.add(U(c,ie.POINTER_MOVE,()=>c.remove())),s.add(U(c,ie.MOUSE_DOWN,()=>c.remove()));let d=0;if(i.length){const f=this._createActionBar(".action-widget-action-bar",i);f&&(n.appendChild(f.getContainer().parentElement),s.add(f),d=f.getContainer().offsetWidth)}const h=(g=this._list.value)==null?void 0:g.layout(d);n.style.width=`${h}px`;const u=s.add(Fa(e));return s.add(u.onDidBlur(()=>this.hide(!0))),s}_createActionBar(e,t){if(!t.length)return;const i=ne(e),n=new zo(i);return n.push(t,{icon:!1,label:!0}),n}_onWidgetClosed(e){var t;(t=this._list.value)==null||t.hide(e)}};jf=S0e([PT(0,ud),PT(1,Ee),PT(2,we)],jf);st(dm,jf,1);const dS=1100;Ot(class extends ws{constructor(){super({id:"hideCodeActionWidget",title:Te("hideCodeActionWidget.title","Hide action widget"),precondition:$f.Visible,keybinding:{weight:dS,primary:9,secondary:[1033]}})}run(o){o.get(dm).hide(!0)}});Ot(class extends ws{constructor(){super({id:"selectPrevCodeAction",title:Te("selectPrevCodeAction.title","Select previous action"),precondition:$f.Visible,keybinding:{weight:dS,primary:16,secondary:[2064],mac:{primary:16,secondary:[2064,302]}}})}run(o){const e=o.get(dm);e instanceof jf&&e.focusPrevious()}});Ot(class extends ws{constructor(){super({id:"selectNextCodeAction",title:Te("selectNextCodeAction.title","Select next action"),precondition:$f.Visible,keybinding:{weight:dS,primary:18,secondary:[2066],mac:{primary:18,secondary:[2066,300]}}})}run(o){const e=o.get(dm);e instanceof jf&&e.focusNext()}});Ot(class extends ws{constructor(){super({id:oZ,title:Te("acceptSelected.title","Accept selected action"),precondition:$f.Visible,keybinding:{weight:dS,primary:3,secondary:[2137]}})}run(o){const e=o.get(dm);e instanceof jf&&e.acceptSelected()}});Ot(class extends ws{constructor(){super({id:rZ,title:Te("previewSelected.title","Preview selected action"),precondition:$f.Visible,keybinding:{weight:dS,primary:2051}})}run(o){const e=o.get(dm);e instanceof jf&&e.acceptSelected(!0)}});const lZ=new le("supportedCodeAction",""),PV="_typescript.applyFixAllCodeAction";class y0e extends F{constructor(e,t,i,n=250){super(),this._editor=e,this._markerService=t,this._signalChange=i,this._delay=n,this._autoTriggerTimer=this._register(new ea),this._register(this._markerService.onMarkerChanged(s=>this._onMarkerChanges(s))),this._register(this._editor.onDidChangeCursorPosition(()=>this._tryAutoTrigger()))}trigger(e){const t=this._getRangeOfSelectionUnlessWhitespaceEnclosed(e);this._signalChange(t?{trigger:e,selection:t}:void 0)}_onMarkerChanges(e){const t=this._editor.getModel();t&&e.some(i=>Lw(i,t.uri))&&this._tryAutoTrigger()}_tryAutoTrigger(){this._autoTriggerTimer.cancelAndSet(()=>{this.trigger({type:2,triggerAction:fo.Default})},this._delay)}_getRangeOfSelectionUnlessWhitespaceEnclosed(e){if(!this._editor.hasModel())return;const t=this._editor.getSelection();if(e.type===1)return t;const i=this._editor.getOption(65).enabled;if(i!==Or.Off){{if(i===Or.On)return t;if(i===Or.OnCode){if(!t.isEmpty())return t;const s=this._editor.getModel(),{lineNumber:r,column:a}=t.getPosition(),l=s.getLineContent(r);if(l.length===0)return;if(a===1){if(/\s/.test(l[0]))return}else if(a===s.getLineMaxColumn(r)){if(/\s/.test(l[l.length-1]))return}else if(/\s/.test(l[a-2])&&/\s/.test(l[a-1]))return}}return t}}}var Qu;(function(o){o.Empty={type:0};class e{constructor(i,n,s){this.trigger=i,this.position=n,this._cancellablePromise=s,this.type=1,this.actions=s.catch(r=>{if(yr(r))return cZ;throw r})}cancel(){this._cancellablePromise.cancel()}}o.Triggered=e})(Qu||(Qu={}));const cZ=Object.freeze({allActions:[],validActions:[],dispose:()=>{},documentation:[],hasAutoFix:!1,hasAIFix:!1,allAIFixes:!1});class L0e extends F{constructor(e,t,i,n,s,r,a){super(),this._editor=e,this._registry=t,this._markerService=i,this._progressService=s,this._configurationService=r,this._telemetryService=a,this._codeActionOracle=this._register(new Ji),this._state=Qu.Empty,this._onDidChangeState=this._register(new P),this.onDidChangeState=this._onDidChangeState.event,this._disposed=!1,this._supportedCodeActions=lZ.bindTo(n),this._register(this._editor.onDidChangeModel(()=>this._update())),this._register(this._editor.onDidChangeModelLanguage(()=>this._update())),this._register(this._registry.onDidChange(()=>this._update())),this._register(this._editor.onDidChangeConfiguration(l=>{l.hasChanged(65)&&this._update()})),this._update()}dispose(){this._disposed||(this._disposed=!0,super.dispose(),this.setState(Qu.Empty,!0))}_settingEnabledNearbyQuickfixes(){var t;const e=(t=this._editor)==null?void 0:t.getModel();return this._configurationService?this._configurationService.getValue("editor.codeActionWidget.includeNearbyQuickFixes",{resource:e==null?void 0:e.uri}):!1}_update(){if(this._disposed)return;this._codeActionOracle.value=void 0,this.setState(Qu.Empty);const e=this._editor.getModel();if(e&&this._registry.has(e)&&!this._editor.getOption(92)){const t=this._registry.all(e).flatMap(i=>i.providedCodeActionKinds??[]);this._supportedCodeActions.set(t.join(" ")),this._codeActionOracle.value=new y0e(this._editor,this._markerService,i=>{var l;if(!i){this.setState(Qu.Empty);return}const n=i.selection.getStartPosition(),s=Yi(async c=>{var d,h,u,g,f,p,_,b,C,v;if(this._settingEnabledNearbyQuickfixes()&&i.trigger.type===1&&(i.trigger.triggerAction===fo.QuickFix||(h=(d=i.trigger.filter)==null?void 0:d.include)!=null&&h.contains(Yt.QuickFix))){const w=await pp(this._registry,e,i.selection,i.trigger,ka.None,c),S=[...w.allActions];if(c.isCancellationRequested)return cZ;const L=(u=w.validActions)==null?void 0:u.some(D=>D.action.kind?Yt.QuickFix.contains(new ti(D.action.kind)):!1),k=this._markerService.read({resource:e.uri});if(L){for(const D of w.validActions)(f=(g=D.action.command)==null?void 0:g.arguments)!=null&&f.some(N=>typeof N=="string"&&N.includes(PV))&&(D.action.diagnostics=[...k.filter(N=>N.relatedInformation)]);return{validActions:w.validActions,allActions:S,documentation:w.documentation,hasAutoFix:w.hasAutoFix,hasAIFix:w.hasAIFix,allAIFixes:w.allAIFixes,dispose:()=>{w.dispose()}}}else if(!L&&k.length>0){const D=i.selection.getPosition();let N=D,B=Number.MAX_VALUE;const W=[...w.validActions];for(const $ of k){const H=$.endColumn,j=$.endLineNumber,J=$.startLineNumber;if(j===D.lineNumber||J===D.lineNumber){N=new A(j,H);const he={type:i.trigger.type,triggerAction:i.trigger.triggerAction,filter:{include:(p=i.trigger.filter)!=null&&p.include?(_=i.trigger.filter)==null?void 0:_.include:Yt.QuickFix},autoApply:i.trigger.autoApply,context:{notAvailableMessage:((b=i.trigger.context)==null?void 0:b.notAvailableMessage)||"",position:N}},fe=new pe(N.lineNumber,N.column,N.lineNumber,N.column),ue=await pp(this._registry,e,fe,he,ka.None,c);if(ue.validActions.length!==0){for(const Pe of ue.validActions)(v=(C=Pe.action.command)==null?void 0:C.arguments)!=null&&v.some(_e=>typeof _e=="string"&&_e.includes(PV))&&(Pe.action.diagnostics=[...k.filter(_e=>_e.relatedInformation)]);w.allActions.length===0&&S.push(...ue.allActions),Math.abs(D.column-H)<B?W.unshift(...ue.validActions):W.push(...ue.validActions)}B=Math.abs(D.column-H)}}const V=W.filter(($,H,j)=>j.findIndex(J=>J.action.title===$.action.title)===H);return V.sort(($,H)=>$.action.isPreferred&&!H.action.isPreferred?-1:!$.action.isPreferred&&H.action.isPreferred||$.action.isAI&&!H.action.isAI?1:!$.action.isAI&&H.action.isAI?-1:0),{validActions:V,allActions:S,documentation:w.documentation,hasAutoFix:w.hasAutoFix,hasAIFix:w.hasAIFix,allAIFixes:w.allAIFixes,dispose:()=>{w.dispose()}}}}if(i.trigger.type===1){const w=new ln,S=await pp(this._registry,e,i.selection,i.trigger,ka.None,c);return this._telemetryService&&this._telemetryService.publicLog2("codeAction.invokedDurations",{codeActions:S.validActions.length,duration:w.elapsed()}),S}return pp(this._registry,e,i.selection,i.trigger,ka.None,c)});i.trigger.type===1&&((l=this._progressService)==null||l.showWhile(s,250));const r=new Qu.Triggered(i.trigger,n,s);let a=!1;this._state.type===1&&(a=this._state.trigger.type===1&&r.type===1&&r.trigger.type===2&&this._state.position!==r.position),a?setTimeout(()=>{this.setState(r)},500):this.setState(r)},void 0),this._codeActionOracle.value.trigger({type:2,triggerAction:fo.Default})}else this._supportedCodeActions.reset()}trigger(e){var t;(t=this._codeActionOracle.value)==null||t.trigger(e)}setState(e,t){e!==this._state&&(this._state.type===1&&this._state.cancel(),this._state=e,!t&&!this._disposed&&this._onDidChangeState.fire(e))}}var x0e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},tl=function(o,e){return function(t,i){e(t,i,o)}},qm;const k0e="quickfix-edit-highlight";var _h;let X_=(_h=class extends F{static get(e){return e.getContribution(qm.ID)}constructor(e,t,i,n,s,r,a,l,c,d,h){super(),this._commandService=a,this._configurationService=l,this._actionWidgetService=c,this._instantiationService=d,this._telemetryService=h,this._activeCodeActions=this._register(new Ji),this._showDisabled=!1,this._disposed=!1,this._editor=e,this._model=this._register(new L0e(this._editor,s.codeActionProvider,t,i,r,l,this._telemetryService)),this._register(this._model.onDidChangeState(u=>this.update(u))),this._lightBulbWidget=new Oa(()=>{const u=this._editor.getContribution(o1.ID);return u&&this._register(u.onClick(g=>this.showCodeActionsFromLightbulb(g.actions,g))),u}),this._resolver=n.createInstance(aO),this._register(this._editor.onDidLayoutChange(()=>this._actionWidgetService.hide()))}dispose(){this._disposed=!0,super.dispose()}async showCodeActionsFromLightbulb(e,t){if(e.allAIFixes&&e.validActions.length===1){const i=e.validActions[0],n=i.action.command;n&&n.id==="inlineChat.start"&&n.arguments&&n.arguments.length>=1&&(n.arguments[0]={...n.arguments[0],autoSend:!1}),await this._applyCodeAction(i,!1,!1,ug.FromAILightbulb);return}await this.showCodeActionList(e,t,{includeDisabledActions:!1,fromLightbulb:!0})}showCodeActions(e,t,i){return this.showCodeActionList(t,i,{includeDisabledActions:!1,fromLightbulb:!1})}manualTriggerAtCurrentPosition(e,t,i,n){var r;if(!this._editor.hasModel())return;(r=lo.get(this._editor))==null||r.closeMessage();const s=this._editor.getPosition();this._trigger({type:1,triggerAction:t,filter:i,autoApply:n,context:{notAvailableMessage:e,position:s}})}_trigger(e){return this._model.trigger(e)}async _applyCodeAction(e,t,i,n){try{await this._instantiationService.invokeFunction(h0e,e,n,{preview:i,editor:this._editor})}finally{t&&this._trigger({type:2,triggerAction:fo.QuickFix,filter:{}})}}hideLightBulbWidget(){var e,t;(e=this._lightBulbWidget.rawValue)==null||e.hide(),(t=this._lightBulbWidget.rawValue)==null||t.gutterHide()}async update(e){var n,s,r,a,l;if(e.type!==1){this.hideLightBulbWidget();return}let t;try{t=await e.actions}catch(c){Oe(c);return}if(this._disposed)return;const i=this._editor.getSelection();if((i==null?void 0:i.startLineNumber)===e.position.lineNumber)if((n=this._lightBulbWidget.value)==null||n.update(t,e.trigger,e.position),e.trigger.type===1){if((s=e.trigger.filter)!=null&&s.include){const d=this.tryGetValidActionToApply(e.trigger,t);if(d){try{this.hideLightBulbWidget(),await this._applyCodeAction(d,!1,!1,ug.FromCodeActions)}finally{t.dispose()}return}if(e.trigger.context){const h=this.getInvalidActionThatWouldHaveBeenApplied(e.trigger,t);if(h&&h.action.disabled){(r=lo.get(this._editor))==null||r.showMessage(h.action.disabled,e.trigger.context.position),t.dispose();return}}}const c=!!((a=e.trigger.filter)!=null&&a.include);if(e.trigger.context&&(!t.allActions.length||!c&&!t.validActions.length)){(l=lo.get(this._editor))==null||l.showMessage(e.trigger.context.notAvailableMessage,e.trigger.context.position),this._activeCodeActions.value=t,t.dispose();return}this._activeCodeActions.value=t,this.showCodeActionList(t,this.toCoords(e.position),{includeDisabledActions:c,fromLightbulb:!1})}else this._actionWidgetService.isVisible?t.dispose():this._activeCodeActions.value=t}getInvalidActionThatWouldHaveBeenApplied(e,t){if(t.allActions.length&&(e.autoApply==="first"&&t.validActions.length===0||e.autoApply==="ifSingle"&&t.allActions.length===1))return t.allActions.find(({action:i})=>i.disabled)}tryGetValidActionToApply(e,t){if(t.validActions.length&&(e.autoApply==="first"&&t.validActions.length>0||e.autoApply==="ifSingle"&&t.validActions.length===1))return t.validActions[0]}async showCodeActionList(e,t,i){const n=this._editor.createDecorationsCollection(),s=this._editor.getDomNode();if(!s)return;const r=i.includeDisabledActions&&(this._showDisabled||e.validActions.length===0)?e.allActions:e.validActions;if(!r.length)return;const a=A.isIPosition(t)?this.toCoords(t):t,l={onSelect:async(c,d)=>{this._applyCodeAction(c,!0,!!d,i.fromLightbulb?ug.FromAILightbulb:ug.FromCodeActions),this._actionWidgetService.hide(!1),n.clear()},onHide:c=>{var d;(d=this._editor)==null||d.focus(),n.clear()},onHover:async(c,d)=>{var g;if(d.isCancellationRequested)return;let h=!1;const u=c.action.kind;if(u){const f=new ti(u);h=[Yt.RefactorExtract,Yt.RefactorInline,Yt.RefactorRewrite,Yt.RefactorMove,Yt.Source].some(_=>_.contains(f))}return{canPreview:h||!!((g=c.action.edit)!=null&&g.edits.length)}},onFocus:c=>{var d,h;if(c&&c.action){const u=c.action.ranges,g=c.action.diagnostics;if(n.clear(),u&&u.length>0){const f=g&&(g==null?void 0:g.length)>1?g.map(p=>({range:p,options:qm.DECORATION})):u.map(p=>({range:p,options:qm.DECORATION}));n.set(f)}else if(g&&g.length>0){const f=g.map(_=>({range:_,options:qm.DECORATION}));n.set(f);const p=g[0];if(p.startLineNumber&&p.startColumn){const _=(h=(d=this._editor.getModel())==null?void 0:d.getWordAtPosition({lineNumber:p.startLineNumber,column:p.startColumn}))==null?void 0:h.word;Hl(m("editingNewSelection","Context: {0} at line {1} and column {2}.",_,p.startLineNumber,p.startColumn))}}}else n.clear()}};this._actionWidgetService.show("codeActionWidget",!0,p0e(r,this._shouldShowHeaders(),this._resolver.getResolver()),l,a,s,this._getActionBarActions(e,t,i))}toCoords(e){if(!this._editor.hasModel())return{x:0,y:0};this._editor.revealPosition(e,1),this._editor.render();const t=this._editor.getScrolledVisiblePosition(e),i=wi(this._editor.getDomNode()),n=i.left+t.left,s=i.top+t.top+t.height;return{x:n,y:s}}_shouldShowHeaders(){var t;const e=(t=this._editor)==null?void 0:t.getModel();return this._configurationService.getValue("editor.codeActionWidget.showHeaders",{resource:e==null?void 0:e.uri})}_getActionBarActions(e,t,i){if(i.fromLightbulb)return[];const n=e.documentation.map(s=>({id:s.id,label:s.title,tooltip:s.tooltip??"",class:void 0,enabled:!0,run:()=>this._commandService.executeCommand(s.id,...s.arguments??[])}));return i.includeDisabledActions&&e.validActions.length>0&&e.allActions.length!==e.validActions.length&&n.push(this._showDisabled?{id:"hideMoreActions",label:m("hideMoreActions","Hide Disabled"),enabled:!0,tooltip:"",class:void 0,run:()=>(this._showDisabled=!1,this.showCodeActionList(e,t,i))}:{id:"showMoreActions",label:m("showMoreActions","Show Disabled"),enabled:!0,tooltip:"",class:void 0,run:()=>(this._showDisabled=!0,this.showCodeActionList(e,t,i))}),n}},qm=_h,_h.ID="editor.contrib.codeActionController",_h.DECORATION=We.register({description:"quickfix-highlight",className:k0e}),_h);X_=qm=x0e([tl(1,Xl),tl(2,Ee),tl(3,we),tl(4,ge),tl(5,nu),tl(6,Ht),tl(7,qe),tl(8,dm),tl(9,we),tl(10,Bn)],X_);Lr((o,e)=>{((n,s)=>{s&&e.addRule(`.monaco-editor ${n} { background-color: ${s}; }`)})(".quickfix-edit-highlight",o.getColor(Ac));const i=o.getColor(Zd);i&&e.addRule(`.monaco-editor .quickfix-edit-highlight { border: 1px ${Ta(o.type)?"dotted":"solid"} ${i}; box-sizing: border-box; }`)});function hS(o){return G.regex(lZ.keys()[0],new RegExp("(\\s|^)"+Uo(o.value)+"\\b"))}const PW={type:"object",defaultSnippets:[{body:{kind:""}}],properties:{kind:{type:"string",description:m("args.schema.kind","Kind of the code action to run.")},apply:{type:"string",description:m("args.schema.apply","Controls when the returned actions are applied."),default:"ifSingle",enum:["first","ifSingle","never"],enumDescriptions:[m("args.schema.apply.first","Always apply the first returned code action."),m("args.schema.apply.ifSingle","Apply the first returned code action if it is the only one."),m("args.schema.apply.never","Do not apply the returned code actions.")]},preferred:{type:"boolean",default:!1,description:m("args.schema.preferred","Controls if only preferred code actions should be returned.")}}};function hm(o,e,t,i,n=fo.Default){if(o.hasModel()){const s=X_.get(o);s==null||s.manualTriggerAtCurrentPosition(e,n,t,i)}}class D0e extends de{constructor(){super({id:AW,label:m("quickfix.trigger.label","Quick Fix..."),alias:"Quick Fix...",precondition:G.and(I.writable,I.hasCodeActionsProvider),kbOpts:{kbExpr:I.textInputFocus,primary:2137,weight:100}})}run(e,t){return hm(t,m("editor.action.quickFix.noneMessage","No code actions available"),void 0,void 0,fo.QuickFix)}}class I0e extends $i{constructor(){super({id:JG,precondition:G.and(I.writable,I.hasCodeActionsProvider),metadata:{description:"Trigger a code action",args:[{name:"args",schema:PW}]}})}runEditorCommand(e,t,i){const n=xl.fromUser(i,{kind:ti.Empty,apply:"ifSingle"});return hm(t,typeof(i==null?void 0:i.kind)=="string"?n.preferred?m("editor.action.codeAction.noneMessage.preferred.kind","No preferred code actions for '{0}' available",i.kind):m("editor.action.codeAction.noneMessage.kind","No code actions for '{0}' available",i.kind):n.preferred?m("editor.action.codeAction.noneMessage.preferred","No preferred code actions available"):m("editor.action.codeAction.noneMessage","No code actions available"),{include:n.kind,includeSourceActions:!0,onlyIncludePreferredActions:n.preferred},n.apply)}}class E0e extends de{constructor(){super({id:tZ,label:m("refactor.label","Refactor..."),alias:"Refactor...",precondition:G.and(I.writable,I.hasCodeActionsProvider),kbOpts:{kbExpr:I.textInputFocus,primary:3120,mac:{primary:1328},weight:100},contextMenuOpts:{group:"1_modification",order:2,when:G.and(I.writable,hS(Yt.Refactor))},metadata:{description:"Refactor...",args:[{name:"args",schema:PW}]}})}run(e,t,i){const n=xl.fromUser(i,{kind:Yt.Refactor,apply:"never"});return hm(t,typeof(i==null?void 0:i.kind)=="string"?n.preferred?m("editor.action.refactor.noneMessage.preferred.kind","No preferred refactorings for '{0}' available",i.kind):m("editor.action.refactor.noneMessage.kind","No refactorings for '{0}' available",i.kind):n.preferred?m("editor.action.refactor.noneMessage.preferred","No preferred refactorings available"):m("editor.action.refactor.noneMessage","No refactorings available"),{include:Yt.Refactor.contains(n.kind)?n.kind:ti.None,onlyIncludePreferredActions:n.preferred},n.apply,fo.Refactor)}}class N0e extends de{constructor(){super({id:iZ,label:m("source.label","Source Action..."),alias:"Source Action...",precondition:G.and(I.writable,I.hasCodeActionsProvider),contextMenuOpts:{group:"1_modification",order:2.1,when:G.and(I.writable,hS(Yt.Source))},metadata:{description:"Source Action...",args:[{name:"args",schema:PW}]}})}run(e,t,i){const n=xl.fromUser(i,{kind:Yt.Source,apply:"never"});return hm(t,typeof(i==null?void 0:i.kind)=="string"?n.preferred?m("editor.action.source.noneMessage.preferred.kind","No preferred source actions for '{0}' available",i.kind):m("editor.action.source.noneMessage.kind","No source actions for '{0}' available",i.kind):n.preferred?m("editor.action.source.noneMessage.preferred","No preferred source actions available"):m("editor.action.source.noneMessage","No source actions available"),{include:Yt.Source.contains(n.kind)?n.kind:ti.None,includeSourceActions:!0,onlyIncludePreferredActions:n.preferred},n.apply,fo.SourceAction)}}class T0e extends de{constructor(){super({id:sO,label:m("organizeImports.label","Organize Imports"),alias:"Organize Imports",precondition:G.and(I.writable,hS(Yt.SourceOrganizeImports)),kbOpts:{kbExpr:I.textInputFocus,primary:1581,weight:100}})}run(e,t){return hm(t,m("editor.action.organize.noneMessage","No organize imports action available"),{include:Yt.SourceOrganizeImports,includeSourceActions:!0},"ifSingle",fo.OrganizeImports)}}class R0e extends de{constructor(){super({id:oO,label:m("fixAll.label","Fix All"),alias:"Fix All",precondition:G.and(I.writable,hS(Yt.SourceFixAll))})}run(e,t){return hm(t,m("fixAll.noneMessage","No fix all action available"),{include:Yt.SourceFixAll,includeSourceActions:!0},"ifSingle",fo.FixAll)}}class M0e extends de{constructor(){super({id:eZ,label:m("autoFix.label","Auto Fix..."),alias:"Auto Fix...",precondition:G.and(I.writable,hS(Yt.QuickFix)),kbOpts:{kbExpr:I.textInputFocus,primary:1625,mac:{primary:2649},weight:100}})}run(e,t){return hm(t,m("editor.action.autoFix.noneMessage","No auto fixes available"),{include:Yt.QuickFix,onlyIncludePreferredActions:!0},"ifSingle",fo.AutoFix)}}ct(X_.ID,X_,3);ct(o1.ID,o1,4);Q(D0e);Q(E0e);Q(N0e);Q(T0e);Q(M0e);Q(R0e);re(new I0e);ci.as(ta.Configuration).registerConfiguration({...K1,properties:{"editor.codeActionWidget.showHeaders":{type:"boolean",scope:5,description:m("showCodeActionHeaders","Enable/disable showing group headers in the Code Action menu."),default:!0}}});ci.as(ta.Configuration).registerConfiguration({...K1,properties:{"editor.codeActionWidget.includeNearbyQuickFixes":{type:"boolean",scope:5,description:m("includeNearbyQuickFixes","Enable/disable showing nearest Quick Fix within a line when not currently on a diagnostic."),default:!0}}});ci.as(ta.Configuration).registerConfiguration({...K1,properties:{"editor.codeActions.triggerOnFocusChange":{type:"boolean",scope:5,markdownDescription:m("triggerOnFocusChange","Enable triggering {0} when {1} is set to {2}. Code Actions must be set to {3} to be triggered for window and focus changes.","`#editor.codeActionsOnSave#`","`#files.autoSave#`","`afterDelay`","`always`"),default:!1}}});class hO{constructor(){this.lenses=[],this._disposables=new K}dispose(){this._disposables.dispose()}get isDisposed(){return this._disposables.isDisposed}add(e,t){this._disposables.add(e);for(const i of e.lenses)this.lenses.push({symbol:i,provider:t})}}async function dZ(o,e,t){const i=o.ordered(e),n=new Map,s=new hO,r=i.map(async(a,l)=>{n.set(a,l);try{const c=await Promise.resolve(a.provideCodeLenses(e,t));c&&s.add(c,a)}catch(c){Si(c)}});return await Promise.all(r),s.lenses=s.lenses.sort((a,l)=>a.symbol.range.startLineNumber<l.symbol.range.startLineNumber?-1:a.symbol.range.startLineNumber>l.symbol.range.startLineNumber?1:n.get(a.provider)<n.get(l.provider)?-1:n.get(a.provider)>n.get(l.provider)?1:a.symbol.range.startColumn<l.symbol.range.startColumn?-1:a.symbol.range.startColumn>l.symbol.range.startColumn?1:0),s}rt.registerCommand("_executeCodeLensProvider",function(o,...e){let[t,i]=e;mt(Se.isUri(t)),mt(typeof i=="number"||!i);const{codeLensProvider:n}=o.get(ge),s=o.get(Kt).getModel(t);if(!s)throw gr();const r=[],a=new K;return dZ(n,s,ze.None).then(l=>{a.add(l);const c=[];for(const d of l.lenses)i==null||d.symbol.command?r.push(d.symbol):i-- >0&&d.provider.resolveCodeLens&&c.push(Promise.resolve(d.provider.resolveCodeLens(s,d.symbol,ze.None)).then(h=>r.push(h||d.symbol)));return Promise.all(c)}).then(()=>r).finally(()=>{setTimeout(()=>a.dispose(),100)})});var A0e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},P0e=function(o,e){return function(t,i){e(t,i,o)}};const hZ=et("ICodeLensCache");class OV{constructor(e,t){this.lineCount=e,this.data=t}}let uO=class{constructor(e){this._fakeProvider=new class{provideCodeLenses(){throw new Error("not supported")}},this._cache=new Gl(20,.75);const t="codelens/cache";vC(Rt,()=>e.remove(t,1));const i="codelens/cache2",n=e.get(i,1,"{}");this._deserialize(n);const s=oe.filter(e.onWillSaveState,r=>r.reason===Nw.SHUTDOWN);oe.once(s)(r=>{e.store(i,this._serialize(),1,1)})}put(e,t){const i=t.lenses.map(r=>{var a;return{range:r.symbol.range,command:r.symbol.command&&{id:"",title:(a=r.symbol.command)==null?void 0:a.title}}}),n=new hO;n.add({lenses:i,dispose:()=>{}},this._fakeProvider);const s=new OV(e.getLineCount(),n);this._cache.set(e.uri.toString(),s)}get(e){const t=this._cache.get(e.uri.toString());return t&&t.lineCount===e.getLineCount()?t.data:void 0}delete(e){this._cache.delete(e.uri.toString())}_serialize(){const e=Object.create(null);for(const[t,i]of this._cache){const n=new Set;for(const s of i.data.lenses)n.add(s.symbol.range.startLineNumber);e[t]={lineCount:i.lineCount,lines:[...n.values()]}}return JSON.stringify(e)}_deserialize(e){try{const t=JSON.parse(e);for(const i in t){const n=t[i],s=[];for(const a of n.lines)s.push({range:new x(a,1,a,11)});const r=new hO;r.add({lenses:s,dispose(){}},this._fakeProvider),this._cache.set(i,new OV(n.lineCount,r))}}catch{}}};uO=A0e([P0e(0,ia)],uO);st(hZ,uO,1);class O0e{constructor(e,t,i){this.afterColumn=1073741824,this.afterLineNumber=e,this.heightInPx=t,this._onHeight=i,this.suppressMouseDown=!0,this.domNode=document.createElement("div")}onComputedHeight(e){this._lastHeight===void 0?this._lastHeight=e:this._lastHeight!==e&&(this._lastHeight=e,this._onHeight())}isVisible(){return this._lastHeight!==0&&this.domNode.hasAttribute("monaco-visible-view-zone")}}const Kv=class Kv{constructor(e,t){this.allowEditorOverflow=!1,this.suppressMouseDown=!0,this._commands=new Map,this._isEmpty=!0,this._editor=e,this._id=`codelens.widget-${Kv._idPool++}`,this.updatePosition(t),this._domNode=document.createElement("span"),this._domNode.className="codelens-decoration"}withCommands(e,t){this._commands.clear();const i=[];let n=!1;for(let s=0;s<e.length;s++){const r=e[s];if(r&&(n=!0,r.command)){const a=dh(r.command.title.trim());if(r.command.id){const l=`c${Kv._idPool++}`;i.push(ne("a",{id:l,title:r.command.tooltip,role:"button"},...a)),this._commands.set(l,r.command)}else i.push(ne("span",{title:r.command.tooltip},...a));s+1<e.length&&i.push(ne("span",void 0," | "))}}n?(_n(this._domNode,...i),this._isEmpty&&t&&this._domNode.classList.add("fadein"),this._isEmpty=!1):_n(this._domNode,ne("span",void 0,"no commands"))}getCommand(e){return e.parentElement===this._domNode?this._commands.get(e.id):void 0}getId(){return this._id}getDomNode(){return this._domNode}updatePosition(e){const t=this._editor.getModel().getLineFirstNonWhitespaceColumn(e);this._widgetPosition={position:{lineNumber:e,column:t},preference:[1]}}getPosition(){return this._widgetPosition||null}};Kv._idPool=0;let gO=Kv;class OT{constructor(){this._removeDecorations=[],this._addDecorations=[],this._addDecorationsCallbacks=[]}addDecoration(e,t){this._addDecorations.push(e),this._addDecorationsCallbacks.push(t)}removeDecoration(e){this._removeDecorations.push(e)}commit(e){const t=e.deltaDecorations(this._removeDecorations,this._addDecorations);for(let i=0,n=t.length;i<n;i++)this._addDecorationsCallbacks[i](t[i])}}const FV=We.register({collapseOnReplaceEdit:!0,description:"codelens"});class WV{constructor(e,t,i,n,s,r){this._isDisposed=!1,this._editor=t,this._data=e,this._decorationIds=[];let a;const l=[];this._data.forEach((c,d)=>{c.symbol.command&&l.push(c.symbol),i.addDecoration({range:c.symbol.range,options:FV},h=>this._decorationIds[d]=h),a?a=x.plusRange(a,c.symbol.range):a=x.lift(c.symbol.range)}),this._viewZone=new O0e(a.startLineNumber-1,s,r),this._viewZoneId=n.addZone(this._viewZone),l.length>0&&(this._createContentWidgetIfNecessary(),this._contentWidget.withCommands(l,!1))}_createContentWidgetIfNecessary(){this._contentWidget?this._editor.layoutContentWidget(this._contentWidget):(this._contentWidget=new gO(this._editor,this._viewZone.afterLineNumber+1),this._editor.addContentWidget(this._contentWidget))}dispose(e,t){this._decorationIds.forEach(e.removeDecoration,e),this._decorationIds=[],t==null||t.removeZone(this._viewZoneId),this._contentWidget&&(this._editor.removeContentWidget(this._contentWidget),this._contentWidget=void 0),this._isDisposed=!0}isDisposed(){return this._isDisposed}isValid(){return this._decorationIds.some((e,t)=>{const i=this._editor.getModel().getDecorationRange(e),n=this._data[t].symbol;return!!(i&&x.isEmpty(n.range)===i.isEmpty())})}updateCodeLensSymbols(e,t){this._decorationIds.forEach(t.removeDecoration,t),this._decorationIds=[],this._data=e,this._data.forEach((i,n)=>{t.addDecoration({range:i.symbol.range,options:FV},s=>this._decorationIds[n]=s)})}updateHeight(e,t){this._viewZone.heightInPx=e,t.layoutZone(this._viewZoneId),this._contentWidget&&this._editor.layoutContentWidget(this._contentWidget)}computeIfNecessary(e){if(!this._viewZone.isVisible())return null;for(let t=0;t<this._decorationIds.length;t++){const i=e.getDecorationRange(this._decorationIds[t]);i&&(this._data[t].symbol.range=i)}return this._data}updateCommands(e){this._createContentWidgetIfNecessary(),this._contentWidget.withCommands(e,!0);for(let t=0;t<this._data.length;t++){const i=e[t];if(i){const{symbol:n}=this._data[t];n.command=i.command||n.command}}}getCommand(e){var t;return(t=this._contentWidget)==null?void 0:t.getCommand(e)}getLineNumber(){const e=this._editor.getModel().getDecorationRange(this._decorationIds[0]);return e?e.startLineNumber:-1}update(e){if(this.isValid()){const t=this._editor.getModel().getDecorationRange(this._decorationIds[0]);t&&(this._viewZone.afterLineNumber=t.startLineNumber-1,e.layoutZone(this._viewZoneId),this._contentWidget&&(this._contentWidget.updatePosition(t.startLineNumber),this._editor.layoutContentWidget(this._contentWidget)))}}}var F0e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},gC=function(o,e){return function(t,i){e(t,i,o)}},f_;let r1=(f_=class{constructor(e,t,i,n,s,r){this._editor=e,this._languageFeaturesService=t,this._commandService=n,this._notificationService=s,this._codeLensCache=r,this._disposables=new K,this._localToDispose=new K,this._lenses=[],this._oldCodeLensModels=new K,this._provideCodeLensDebounce=i.for(t.codeLensProvider,"CodeLensProvide",{min:250}),this._resolveCodeLensesDebounce=i.for(t.codeLensProvider,"CodeLensResolve",{min:250,salt:"resolve"}),this._resolveCodeLensesScheduler=new It(()=>this._resolveCodeLensesInViewport(),this._resolveCodeLensesDebounce.default()),this._disposables.add(this._editor.onDidChangeModel(()=>this._onModelChange())),this._disposables.add(this._editor.onDidChangeModelLanguage(()=>this._onModelChange())),this._disposables.add(this._editor.onDidChangeConfiguration(a=>{(a.hasChanged(50)||a.hasChanged(19)||a.hasChanged(18))&&this._updateLensStyle(),a.hasChanged(17)&&this._onModelChange()})),this._disposables.add(t.codeLensProvider.onDidChange(this._onModelChange,this)),this._onModelChange(),this._updateLensStyle()}dispose(){var e;this._localDispose(),this._disposables.dispose(),this._oldCodeLensModels.dispose(),(e=this._currentCodeLensModel)==null||e.dispose()}_getLayoutInfo(){const e=Math.max(1.3,this._editor.getOption(67)/this._editor.getOption(52));let t=this._editor.getOption(19);return(!t||t<5)&&(t=this._editor.getOption(52)*.9|0),{fontSize:t,codeLensHeight:t*e|0}}_updateLensStyle(){const{codeLensHeight:e,fontSize:t}=this._getLayoutInfo(),i=this._editor.getOption(18),n=this._editor.getOption(50),{style:s}=this._editor.getContainerDomNode();s.setProperty("--vscode-editorCodeLens-lineHeight",`${e}px`),s.setProperty("--vscode-editorCodeLens-fontSize",`${t}px`),s.setProperty("--vscode-editorCodeLens-fontFeatureSettings",n.fontFeatureSettings),i&&(s.setProperty("--vscode-editorCodeLens-fontFamily",i),s.setProperty("--vscode-editorCodeLens-fontFamilyDefault",fs.fontFamily)),this._editor.changeViewZones(r=>{for(const a of this._lenses)a.updateHeight(e,r)})}_localDispose(){var e,t,i;(e=this._getCodeLensModelPromise)==null||e.cancel(),this._getCodeLensModelPromise=void 0,(t=this._resolveCodeLensesPromise)==null||t.cancel(),this._resolveCodeLensesPromise=void 0,this._localToDispose.clear(),this._oldCodeLensModels.clear(),(i=this._currentCodeLensModel)==null||i.dispose()}_onModelChange(){this._localDispose();const e=this._editor.getModel();if(!e||!this._editor.getOption(17)||e.isTooLargeForTokenization())return;const t=this._codeLensCache.get(e);if(t&&this._renderCodeLensSymbols(t),!this._languageFeaturesService.codeLensProvider.has(e)){t&&Bh(()=>{const n=this._codeLensCache.get(e);t===n&&(this._codeLensCache.delete(e),this._onModelChange())},30*1e3,this._localToDispose);return}for(const n of this._languageFeaturesService.codeLensProvider.all(e))if(typeof n.onDidChange=="function"){const s=n.onDidChange(()=>i.schedule());this._localToDispose.add(s)}const i=new It(()=>{var s;const n=Date.now();(s=this._getCodeLensModelPromise)==null||s.cancel(),this._getCodeLensModelPromise=Yi(r=>dZ(this._languageFeaturesService.codeLensProvider,e,r)),this._getCodeLensModelPromise.then(r=>{this._currentCodeLensModel&&this._oldCodeLensModels.add(this._currentCodeLensModel),this._currentCodeLensModel=r,this._codeLensCache.put(e,r);const a=this._provideCodeLensDebounce.update(e,Date.now()-n);i.delay=a,this._renderCodeLensSymbols(r),this._resolveCodeLensesInViewportSoon()},Oe)},this._provideCodeLensDebounce.get(e));this._localToDispose.add(i),this._localToDispose.add(ve(()=>this._resolveCodeLensesScheduler.cancel())),this._localToDispose.add(this._editor.onDidChangeModelContent(()=>{var n;this._editor.changeDecorations(s=>{this._editor.changeViewZones(r=>{const a=[];let l=-1;this._lenses.forEach(d=>{!d.isValid()||l===d.getLineNumber()?a.push(d):(d.update(r),l=d.getLineNumber())});const c=new OT;a.forEach(d=>{d.dispose(c,r),this._lenses.splice(this._lenses.indexOf(d),1)}),c.commit(s)})}),i.schedule(),this._resolveCodeLensesScheduler.cancel(),(n=this._resolveCodeLensesPromise)==null||n.cancel(),this._resolveCodeLensesPromise=void 0})),this._localToDispose.add(this._editor.onDidFocusEditorText(()=>{i.schedule()})),this._localToDispose.add(this._editor.onDidBlurEditorText(()=>{i.cancel()})),this._localToDispose.add(this._editor.onDidScrollChange(n=>{n.scrollTopChanged&&this._lenses.length>0&&this._resolveCodeLensesInViewportSoon()})),this._localToDispose.add(this._editor.onDidLayoutChange(()=>{this._resolveCodeLensesInViewportSoon()})),this._localToDispose.add(ve(()=>{if(this._editor.getModel()){const n=za.capture(this._editor);this._editor.changeDecorations(s=>{this._editor.changeViewZones(r=>{this._disposeAllLenses(s,r)})}),n.restore(this._editor)}else this._disposeAllLenses(void 0,void 0)})),this._localToDispose.add(this._editor.onMouseDown(n=>{if(n.target.type!==9)return;let s=n.target.element;if((s==null?void 0:s.tagName)==="SPAN"&&(s=s.parentElement),(s==null?void 0:s.tagName)==="A")for(const r of this._lenses){const a=r.getCommand(s);if(a){this._commandService.executeCommand(a.id,...a.arguments||[]).catch(l=>this._notificationService.error(l));break}}})),i.schedule()}_disposeAllLenses(e,t){const i=new OT;for(const n of this._lenses)n.dispose(i,t);e&&i.commit(e),this._lenses.length=0}_renderCodeLensSymbols(e){if(!this._editor.hasModel())return;const t=this._editor.getModel().getLineCount(),i=[];let n;for(const a of e.lenses){const l=a.symbol.range.startLineNumber;l<1||l>t||(n&&n[n.length-1].symbol.range.startLineNumber===l?n.push(a):(n=[a],i.push(n)))}if(!i.length&&!this._lenses.length)return;const s=za.capture(this._editor),r=this._getLayoutInfo();this._editor.changeDecorations(a=>{this._editor.changeViewZones(l=>{const c=new OT;let d=0,h=0;for(;h<i.length&&d<this._lenses.length;){const u=i[h][0].symbol.range.startLineNumber,g=this._lenses[d].getLineNumber();g<u?(this._lenses[d].dispose(c,l),this._lenses.splice(d,1)):g===u?(this._lenses[d].updateCodeLensSymbols(i[h],c),h++,d++):(this._lenses.splice(d,0,new WV(i[h],this._editor,c,l,r.codeLensHeight,()=>this._resolveCodeLensesInViewportSoon())),d++,h++)}for(;d<this._lenses.length;)this._lenses[d].dispose(c,l),this._lenses.splice(d,1);for(;h<i.length;)this._lenses.push(new WV(i[h],this._editor,c,l,r.codeLensHeight,()=>this._resolveCodeLensesInViewportSoon())),h++;c.commit(a)})}),s.restore(this._editor)}_resolveCodeLensesInViewportSoon(){this._editor.getModel()&&this._resolveCodeLensesScheduler.schedule()}_resolveCodeLensesInViewport(){var r;(r=this._resolveCodeLensesPromise)==null||r.cancel(),this._resolveCodeLensesPromise=void 0;const e=this._editor.getModel();if(!e)return;const t=[],i=[];if(this._lenses.forEach(a=>{const l=a.computeIfNecessary(e);l&&(t.push(l),i.push(a))}),t.length===0)return;const n=Date.now(),s=Yi(a=>{const l=t.map((c,d)=>{const h=new Array(c.length),u=c.map((g,f)=>!g.symbol.command&&typeof g.provider.resolveCodeLens=="function"?Promise.resolve(g.provider.resolveCodeLens(e,g.symbol,a)).then(p=>{h[f]=p},Si):(h[f]=g.symbol,Promise.resolve(void 0)));return Promise.all(u).then(()=>{!a.isCancellationRequested&&!i[d].isDisposed()&&i[d].updateCommands(h)})});return Promise.all(l)});this._resolveCodeLensesPromise=s,this._resolveCodeLensesPromise.then(()=>{const a=this._resolveCodeLensesDebounce.update(e,Date.now()-n);this._resolveCodeLensesScheduler.delay=a,this._currentCodeLensModel&&this._codeLensCache.put(e,this._currentCodeLensModel),this._oldCodeLensModels.clear(),s===this._resolveCodeLensesPromise&&(this._resolveCodeLensesPromise=void 0)},a=>{Oe(a),s===this._resolveCodeLensesPromise&&(this._resolveCodeLensesPromise=void 0)})}async getModel(){var e;return await this._getCodeLensModelPromise,await this._resolveCodeLensesPromise,(e=this._currentCodeLensModel)!=null&&e.isDisposed?void 0:this._currentCodeLensModel}},f_.ID="css.editor.codeLens",f_);r1=F0e([gC(1,ge),gC(2,Zo),gC(3,Ht),gC(4,Ei),gC(5,hZ)],r1);ct(r1.ID,r1,1);Q(class extends de{constructor(){super({id:"codelens.showLensesInCurrentLine",precondition:I.hasCodeLensProvider,label:m("showLensOnLine","Show CodeLens Commands For Current Line"),alias:"Show CodeLens Commands For Current Line"})}async run(e,t){if(!t.hasModel())return;const i=e.get(xr),n=e.get(Ht),s=e.get(Ei),r=t.getSelection().positionLineNumber,a=t.getContribution(r1.ID);if(!a)return;const l=await a.getModel();if(!l)return;const c=[];for(const u of l.lenses)u.symbol.command&&u.symbol.range.startLineNumber===r&&c.push({label:u.symbol.command.title,command:u.symbol.command});if(c.length===0)return;const d=await i.pick(c,{canPickMany:!1,placeHolder:m("placeHolder","Select a command")});if(!d)return;let h=d.command;if(l.isDisposed){const u=await a.getModel(),g=u==null?void 0:u.lenses.find(f=>{var p;return f.symbol.range.startLineNumber===r&&((p=f.symbol.command)==null?void 0:p.title)===h.title});if(!g||!g.symbol.command)return;h=g.symbol.command}try{await n.executeCommand(h.id,...h.arguments||[])}catch(u){s.error(u)}}});var uZ=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},fO=function(o,e){return function(t,i){e(t,i,o)}};let a1=class{constructor(e){this._editorWorkerService=e}async provideDocumentColors(e,t){return this._editorWorkerService.computeDefaultDocumentColors(e.uri)}provideColorPresentations(e,t,i){const n=t.range,s=t.color,r=s.alpha,a=new Z(new dt(Math.round(255*s.red),Math.round(255*s.green),Math.round(255*s.blue),r)),l=r?Z.Format.CSS.formatRGB(a):Z.Format.CSS.formatRGBA(a),c=r?Z.Format.CSS.formatHSL(a):Z.Format.CSS.formatHSLA(a),d=r?Z.Format.CSS.formatHex(a):Z.Format.CSS.formatHexA(a),h=[];return h.push({label:l,textEdit:{range:n,text:l}}),h.push({label:c,textEdit:{range:n,text:c}}),h.push({label:d,textEdit:{range:n,text:d}}),h}};a1=uZ([fO(0,Us)],a1);let mO=class extends F{constructor(e,t){super(),this._register(e.colorProvider.register("*",new a1(t)))}};mO=uZ([fO(0,ge),fO(1,Us)],mO);Nb(mO);async function gZ(o,e,t,i=!0){return OW(new W0e,o,e,t,i)}function fZ(o,e,t,i){return Promise.resolve(t.provideColorPresentations(o,e,i))}class W0e{constructor(){}async compute(e,t,i,n){const s=await e.provideDocumentColors(t,i);if(Array.isArray(s))for(const r of s)n.push({colorInfo:r,provider:e});return Array.isArray(s)}}class B0e{constructor(){}async compute(e,t,i,n){const s=await e.provideDocumentColors(t,i);if(Array.isArray(s))for(const r of s)n.push({range:r.range,color:[r.color.red,r.color.green,r.color.blue,r.color.alpha]});return Array.isArray(s)}}class H0e{constructor(e){this.colorInfo=e}async compute(e,t,i,n){const s=await e.provideColorPresentations(t,this.colorInfo,ze.None);return Array.isArray(s)&&n.push(...s),Array.isArray(s)}}async function OW(o,e,t,i,n){let s=!1,r;const a=[],l=e.ordered(t);for(let c=l.length-1;c>=0;c--){const d=l[c];if(d instanceof a1)r=d;else try{await o.compute(d,t,i,a)&&(s=!0)}catch(h){Si(h)}}return s?a:r&&n?(await o.compute(r,t,i,a),a):[]}function mZ(o,e){const{colorProvider:t}=o.get(ge),i=o.get(Kt).getModel(e);if(!i)throw gr();const n=o.get(qe).getValue("editor.defaultColorDecorators",{resource:e});return{model:i,colorProviderRegistry:t,isDefaultColorDecoratorsEnabled:n}}rt.registerCommand("_executeDocumentColorProvider",function(o,...e){const[t]=e;if(!(t instanceof Se))throw gr();const{model:i,colorProviderRegistry:n,isDefaultColorDecoratorsEnabled:s}=mZ(o,t);return OW(new B0e,n,i,ze.None,s)});rt.registerCommand("_executeColorPresentationProvider",function(o,...e){const[t,i]=e,{uri:n,range:s}=i;if(!(n instanceof Se)||!Array.isArray(t)||t.length!==4||!x.isIRange(s))throw gr();const{model:r,colorProviderRegistry:a,isDefaultColorDecoratorsEnabled:l}=mZ(o,n),[c,d,h,u]=t;return OW(new H0e({range:s,color:{red:c,green:d,blue:h,alpha:u}}),a,r,ze.None,l)});var V0e=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},FT=function(o,e){return function(t,i){e(t,i,o)}},pO;const pZ=Object.create({});var bh;let Q_=(bh=class extends F{constructor(e,t,i,n){super(),this._editor=e,this._configurationService=t,this._languageFeaturesService=i,this._localToDispose=this._register(new K),this._decorationsIds=[],this._colorDatas=new Map,this._colorDecoratorIds=this._editor.createDecorationsCollection(),this._ruleFactory=new sx(this._editor),this._decoratorLimitReporter=new z0e,this._colorDecorationClassRefs=this._register(new K),this._debounceInformation=n.for(i.colorProvider,"Document Colors",{min:pO.RECOMPUTE_TIME}),this._register(e.onDidChangeModel(()=>{this._isColorDecoratorsEnabled=this.isEnabled(),this.updateColors()})),this._register(e.onDidChangeModelLanguage(()=>this.updateColors())),this._register(i.colorProvider.onDidChange(()=>this.updateColors())),this._register(e.onDidChangeConfiguration(s=>{const r=this._isColorDecoratorsEnabled;this._isColorDecoratorsEnabled=this.isEnabled(),this._isDefaultColorDecoratorsEnabled=this._editor.getOption(148);const a=r!==this._isColorDecoratorsEnabled||s.hasChanged(21),l=s.hasChanged(148);(a||l)&&(this._isColorDecoratorsEnabled?this.updateColors():this.removeAllDecorations())})),this._timeoutTimer=null,this._computePromise=null,this._isColorDecoratorsEnabled=this.isEnabled(),this._isDefaultColorDecoratorsEnabled=this._editor.getOption(148),this.updateColors()}isEnabled(){const e=this._editor.getModel();if(!e)return!1;const t=e.getLanguageId(),i=this._configurationService.getValue(t);if(i&&typeof i=="object"){const n=i.colorDecorators;if(n&&n.enable!==void 0&&!n.enable)return n.enable}return this._editor.getOption(20)}static get(e){return e.getContribution(this.ID)}dispose(){this.stop(),this.removeAllDecorations(),super.dispose()}updateColors(){if(this.stop(),!this._isColorDecoratorsEnabled)return;const e=this._editor.getModel();!e||!this._languageFeaturesService.colorProvider.has(e)||(this._localToDispose.add(this._editor.onDidChangeModelContent(()=>{this._timeoutTimer||(this._timeoutTimer=new ea,this._timeoutTimer.cancelAndSet(()=>{this._timeoutTimer=null,this.beginCompute()},this._debounceInformation.get(e)))})),this.beginCompute())}async beginCompute(){this._computePromise=Yi(async e=>{const t=this._editor.getModel();if(!t)return[];const i=new ln(!1),n=await gZ(this._languageFeaturesService.colorProvider,t,e,this._isDefaultColorDecoratorsEnabled);return this._debounceInformation.update(t,i.elapsed()),n});try{const e=await this._computePromise;this.updateDecorations(e),this.updateColorDecorators(e),this._computePromise=null}catch(e){Oe(e)}}stop(){this._timeoutTimer&&(this._timeoutTimer.cancel(),this._timeoutTimer=null),this._computePromise&&(this._computePromise.cancel(),this._computePromise=null),this._localToDispose.clear()}updateDecorations(e){const t=e.map(i=>({range:{startLineNumber:i.colorInfo.range.startLineNumber,startColumn:i.colorInfo.range.startColumn,endLineNumber:i.colorInfo.range.endLineNumber,endColumn:i.colorInfo.range.endColumn},options:We.EMPTY}));this._editor.changeDecorations(i=>{this._decorationsIds=i.deltaDecorations(this._decorationsIds,t),this._colorDatas=new Map,this._decorationsIds.forEach((n,s)=>this._colorDatas.set(n,e[s]))})}updateColorDecorators(e){this._colorDecorationClassRefs.clear();const t=[],i=this._editor.getOption(21);for(let s=0;s<e.length&&t.length<i;s++){const{red:r,green:a,blue:l,alpha:c}=e[s].colorInfo.color,d=new dt(Math.round(r*255),Math.round(a*255),Math.round(l*255),c),h=`rgba(${d.r}, ${d.g}, ${d.b}, ${d.a})`,u=this._colorDecorationClassRefs.add(this._ruleFactory.createClassNameRef({backgroundColor:h}));t.push({range:{startLineNumber:e[s].colorInfo.range.startLineNumber,startColumn:e[s].colorInfo.range.startColumn,endLineNumber:e[s].colorInfo.range.endLineNumber,endColumn:e[s].colorInfo.range.endColumn},options:{description:"colorDetector",before:{content:rU,inlineClassName:`${u.className} colorpicker-color-decoration`,inlineClassNameAffectsLetterSpacing:!0,attachedData:pZ}}})}const n=i<e.length?i:!1;this._decoratorLimitReporter.update(e.length,n),this._colorDecoratorIds.set(t)}removeAllDecorations(){this._editor.removeDecorations(this._decorationsIds),this._decorationsIds=[],this._colorDecoratorIds.clear(),this._colorDecorationClassRefs.clear()}getColorData(e){const t=this._editor.getModel();if(!t)return null;const i=t.getDecorationsInRange(x.fromPositions(e,e)).filter(n=>this._colorDatas.has(n.id));return i.length===0?null:this._colorDatas.get(i[0].id)}isColorDecoration(e){return this._colorDecoratorIds.has(e)}},pO=bh,bh.ID="editor.contrib.colorDetector",bh.RECOMPUTE_TIME=1e3,bh);Q_=pO=V0e([FT(1,qe),FT(2,ge),FT(3,Zo)],Q_);class z0e{constructor(){this._onDidChange=new P,this._computed=0,this._limited=!1}update(e,t){(e!==this._computed||t!==this._limited)&&(this._computed=e,this._limited=t,this._onDidChange.fire())}}ct(Q_.ID,Q_,1);class U0e{get color(){return this._color}set color(e){this._color.equals(e)||(this._color=e,this._onDidChangeColor.fire(e))}get presentation(){return this.colorPresentations[this.presentationIndex]}get colorPresentations(){return this._colorPresentations}set colorPresentations(e){this._colorPresentations=e,this.presentationIndex>e.length-1&&(this.presentationIndex=0),this._onDidChangePresentation.fire(this.presentation)}constructor(e,t,i){this.presentationIndex=i,this._onColorFlushed=new P,this.onColorFlushed=this._onColorFlushed.event,this._onDidChangeColor=new P,this.onDidChangeColor=this._onDidChangeColor.event,this._onDidChangePresentation=new P,this.onDidChangePresentation=this._onDidChangePresentation.event,this.originalColor=e,this._color=e,this._colorPresentations=t}selectNextColorPresentation(){this.presentationIndex=(this.presentationIndex+1)%this.colorPresentations.length,this.flushColor(),this._onDidChangePresentation.fire(this.presentation)}guessColorPresentation(e,t){let i=-1;for(let n=0;n<this.colorPresentations.length;n++)if(t.toLowerCase()===this.colorPresentations[n].label){i=n;break}if(i===-1){const n=t.split("(")[0].toLowerCase();for(let s=0;s<this.colorPresentations.length;s++)if(this.colorPresentations[s].label.toLowerCase().startsWith(n)){i=s;break}}i!==-1&&i!==this.presentationIndex&&(this.presentationIndex=i,this._onDidChangePresentation.fire(this.presentation))}flushColor(){this._onColorFlushed.fire(this._color)}}const Po=ne;class $0e extends F{constructor(e,t,i,n=!1){super(),this.model=t,this.showingStandaloneColorPicker=n,this._closeButton=null,this._domNode=Po(".colorpicker-header"),X(e,this._domNode),this._pickedColorNode=X(this._domNode,Po(".picked-color")),X(this._pickedColorNode,Po("span.codicon.codicon-color-mode")),this._pickedColorPresentation=X(this._pickedColorNode,document.createElement("span")),this._pickedColorPresentation.classList.add("picked-color-presentation");const s=m("clickToToggleColorOptions","Click to toggle color options (rgb/hsl/hex)");this._pickedColorNode.setAttribute("title",s),this._originalColorNode=X(this._domNode,Po(".original-color")),this._originalColorNode.style.backgroundColor=Z.Format.CSS.format(this.model.originalColor)||"",this.backgroundColor=i.getColorTheme().getColor(cL)||Z.white,this._register(i.onDidColorThemeChange(r=>{this.backgroundColor=r.getColor(cL)||Z.white})),this._register(U(this._pickedColorNode,ie.CLICK,()=>this.model.selectNextColorPresentation())),this._register(U(this._originalColorNode,ie.CLICK,()=>{this.model.color=this.model.originalColor,this.model.flushColor()})),this._register(t.onDidChangeColor(this.onDidChangeColor,this)),this._register(t.onDidChangePresentation(this.onDidChangePresentation,this)),this._pickedColorNode.style.backgroundColor=Z.Format.CSS.format(t.color)||"",this._pickedColorNode.classList.toggle("light",t.color.rgba.a<.5?this.backgroundColor.isLighter():t.color.isLighter()),this.onDidChangeColor(this.model.color),this.showingStandaloneColorPicker&&(this._domNode.classList.add("standalone-colorpicker"),this._closeButton=this._register(new j0e(this._domNode)))}get closeButton(){return this._closeButton}get pickedColorNode(){return this._pickedColorNode}get originalColorNode(){return this._originalColorNode}onDidChangeColor(e){this._pickedColorNode.style.backgroundColor=Z.Format.CSS.format(e)||"",this._pickedColorNode.classList.toggle("light",e.rgba.a<.5?this.backgroundColor.isLighter():e.isLighter()),this.onDidChangePresentation()}onDidChangePresentation(){this._pickedColorPresentation.textContent=this.model.presentation?this.model.presentation.label:""}}class j0e extends F{constructor(e){super(),this._onClicked=this._register(new P),this.onClicked=this._onClicked.event,this._button=document.createElement("div"),this._button.classList.add("close-button"),X(e,this._button);const t=document.createElement("div");t.classList.add("close-button-inner-div"),X(this._button,t),X(t,Po(".button"+Ie.asCSSSelector(qt("color-picker-close",te.close,m("closeIcon","Icon to close the color picker"))))).classList.add("close-icon"),this._register(U(this._button,ie.CLICK,()=>{this._onClicked.fire()}))}}class K0e extends F{constructor(e,t,i,n=!1){super(),this.model=t,this.pixelRatio=i,this._insertButton=null,this._domNode=Po(".colorpicker-body"),X(e,this._domNode),this._saturationBox=new q0e(this._domNode,this.model,this.pixelRatio),this._register(this._saturationBox),this._register(this._saturationBox.onDidChange(this.onDidSaturationValueChange,this)),this._register(this._saturationBox.onColorFlushed(this.flushColor,this)),this._opacityStrip=new G0e(this._domNode,this.model,n),this._register(this._opacityStrip),this._register(this._opacityStrip.onDidChange(this.onDidOpacityChange,this)),this._register(this._opacityStrip.onColorFlushed(this.flushColor,this)),this._hueStrip=new Z0e(this._domNode,this.model,n),this._register(this._hueStrip),this._register(this._hueStrip.onDidChange(this.onDidHueChange,this)),this._register(this._hueStrip.onColorFlushed(this.flushColor,this)),n&&(this._insertButton=this._register(new Y0e(this._domNode)),this._domNode.classList.add("standalone-colorpicker"))}flushColor(){this.model.flushColor()}onDidSaturationValueChange({s:e,v:t}){const i=this.model.color.hsva;this.model.color=new Z(new El(i.h,e,t,i.a))}onDidOpacityChange(e){const t=this.model.color.hsva;this.model.color=new Z(new El(t.h,t.s,t.v,e))}onDidHueChange(e){const t=this.model.color.hsva,i=(1-e)*360;this.model.color=new Z(new El(i===360?0:i,t.s,t.v,t.a))}get domNode(){return this._domNode}get saturationBox(){return this._saturationBox}get enterButton(){return this._insertButton}layout(){this._saturationBox.layout(),this._opacityStrip.layout(),this._hueStrip.layout()}}class q0e extends F{constructor(e,t,i){super(),this.model=t,this.pixelRatio=i,this._onDidChange=new P,this.onDidChange=this._onDidChange.event,this._onColorFlushed=new P,this.onColorFlushed=this._onColorFlushed.event,this._domNode=Po(".saturation-wrap"),X(e,this._domNode),this._canvas=document.createElement("canvas"),this._canvas.className="saturation-box",X(this._domNode,this._canvas),this.selection=Po(".saturation-selection"),X(this._domNode,this.selection),this.layout(),this._register(U(this._domNode,ie.POINTER_DOWN,n=>this.onPointerDown(n))),this._register(this.model.onDidChangeColor(this.onDidChangeColor,this)),this.monitor=null}get domNode(){return this._domNode}onPointerDown(e){if(!e.target||!(e.target instanceof Element))return;this.monitor=this._register(new vb);const t=wi(this._domNode);e.target!==this.selection&&this.onDidChangePosition(e.offsetX,e.offsetY),this.monitor.startMonitoring(e.target,e.pointerId,e.buttons,n=>this.onDidChangePosition(n.pageX-t.left,n.pageY-t.top),()=>null);const i=U(e.target.ownerDocument,ie.POINTER_UP,()=>{this._onColorFlushed.fire(),i.dispose(),this.monitor&&(this.monitor.stopMonitoring(!0),this.monitor=null)},!0)}onDidChangePosition(e,t){const i=Math.max(0,Math.min(1,e/this.width)),n=Math.max(0,Math.min(1,1-t/this.height));this.paintSelection(i,n),this._onDidChange.fire({s:i,v:n})}layout(){this.width=this._domNode.offsetWidth,this.height=this._domNode.offsetHeight,this._canvas.width=this.width*this.pixelRatio,this._canvas.height=this.height*this.pixelRatio,this.paint();const e=this.model.color.hsva;this.paintSelection(e.s,e.v)}paint(){const e=this.model.color.hsva,t=new Z(new El(e.h,1,1,1)),i=this._canvas.getContext("2d"),n=i.createLinearGradient(0,0,this._canvas.width,0);n.addColorStop(0,"rgba(255, 255, 255, 1)"),n.addColorStop(.5,"rgba(255, 255, 255, 0.5)"),n.addColorStop(1,"rgba(255, 255, 255, 0)");const s=i.createLinearGradient(0,0,0,this._canvas.height);s.addColorStop(0,"rgba(0, 0, 0, 0)"),s.addColorStop(1,"rgba(0, 0, 0, 1)"),i.rect(0,0,this._canvas.width,this._canvas.height),i.fillStyle=Z.Format.CSS.format(t),i.fill(),i.fillStyle=n,i.fill(),i.fillStyle=s,i.fill()}paintSelection(e,t){this.selection.style.left=`${e*this.width}px`,this.selection.style.top=`${this.height-t*this.height}px`}onDidChangeColor(e){if(this.monitor&&this.monitor.isMonitoring())return;this.paint();const t=e.hsva;this.paintSelection(t.s,t.v)}}class _Z extends F{constructor(e,t,i=!1){super(),this.model=t,this._onDidChange=new P,this.onDidChange=this._onDidChange.event,this._onColorFlushed=new P,this.onColorFlushed=this._onColorFlushed.event,i?(this.domNode=X(e,Po(".standalone-strip")),this.overlay=X(this.domNode,Po(".standalone-overlay"))):(this.domNode=X(e,Po(".strip")),this.overlay=X(this.domNode,Po(".overlay"))),this.slider=X(this.domNode,Po(".slider")),this.slider.style.top="0px",this._register(U(this.domNode,ie.POINTER_DOWN,n=>this.onPointerDown(n))),this._register(t.onDidChangeColor(this.onDidChangeColor,this)),this.layout()}layout(){this.height=this.domNode.offsetHeight-this.slider.offsetHeight;const e=this.getValue(this.model.color);this.updateSliderPosition(e)}onDidChangeColor(e){const t=this.getValue(e);this.updateSliderPosition(t)}onPointerDown(e){if(!e.target||!(e.target instanceof Element))return;const t=this._register(new vb),i=wi(this.domNode);this.domNode.classList.add("grabbing"),e.target!==this.slider&&this.onDidChangeTop(e.offsetY),t.startMonitoring(e.target,e.pointerId,e.buttons,s=>this.onDidChangeTop(s.pageY-i.top),()=>null);const n=U(e.target.ownerDocument,ie.POINTER_UP,()=>{this._onColorFlushed.fire(),n.dispose(),t.stopMonitoring(!0),this.domNode.classList.remove("grabbing")},!0)}onDidChangeTop(e){const t=Math.max(0,Math.min(1,1-e/this.height));this.updateSliderPosition(t),this._onDidChange.fire(t)}updateSliderPosition(e){this.slider.style.top=`${(1-e)*this.height}px`}}class G0e extends _Z{constructor(e,t,i=!1){super(e,t,i),this.domNode.classList.add("opacity-strip"),this.onDidChangeColor(this.model.color)}onDidChangeColor(e){super.onDidChangeColor(e);const{r:t,g:i,b:n}=e.rgba,s=new Z(new dt(t,i,n,1)),r=new Z(new dt(t,i,n,0));this.overlay.style.background=`linear-gradient(to bottom, ${s} 0%, ${r} 100%)`}getValue(e){return e.hsva.a}}class Z0e extends _Z{constructor(e,t,i=!1){super(e,t,i),this.domNode.classList.add("hue-strip")}getValue(e){return 1-e.hsva.h/360}}class Y0e extends F{constructor(e){super(),this._onClicked=this._register(new P),this.onClicked=this._onClicked.event,this._button=X(e,document.createElement("button")),this._button.classList.add("insert-button"),this._button.textContent="Insert",this._register(U(this._button,ie.CLICK,()=>{this._onClicked.fire()}))}get button(){return this._button}}class X0e extends Yo{constructor(e,t,i,n,s=!1){super(),this.model=t,this.pixelRatio=i,this._register(aw.getInstance(xe(e)).onDidChange(()=>this.layout())),this._domNode=Po(".colorpicker-widget"),e.appendChild(this._domNode),this.header=this._register(new $0e(this._domNode,this.model,n,s)),this.body=this._register(new K0e(this._domNode,this.model,this.pixelRatio,s))}layout(){this.body.layout()}get domNode(){return this._domNode}}class WT{constructor(e,t,i,n){this.priority=e,this.range=t,this.initialMousePosX=i,this.initialMousePosY=n,this.type=1}equals(e){return e.type===1&&this.range.equalsRange(e.range)}canAdoptVisibleHover(e,t){return e.type===1&&t.lineNumber===this.range.startLineNumber}}class gy{constructor(e,t,i,n,s,r){this.priority=e,this.owner=t,this.range=i,this.initialMousePosX=n,this.initialMousePosY=s,this.supportsMarkerHover=r,this.type=2}equals(e){return e.type===2&&this.owner===e.owner}canAdoptVisibleHover(e,t){return e.type===2&&this.owner===e.owner}}class Kf{constructor(e){this.renderedHoverParts=e}dispose(){for(const e of this.renderedHoverParts)e.dispose()}}const um=new class{constructor(){this._participants=[]}register(e){this._participants.push(e)}getAll(){return this._participants}};var bZ=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},CZ=function(o,e){return function(t,i){e(t,i,o)}};class Q0e{constructor(e,t,i,n){this.owner=e,this.range=t,this.model=i,this.provider=n,this.forceShowAtRange=!0}isValidForHoverAnchor(e){return e.type===1&&this.range.startColumn<=e.range.startColumn&&this.range.endColumn>=e.range.endColumn}}let l1=class{constructor(e,t){this._editor=e,this._themeService=t,this.hoverOrdinal=2}computeSync(e,t){return[]}computeAsync(e,t,i){return Fo.fromPromise(this._computeAsync(e,t,i))}async _computeAsync(e,t,i){if(!this._editor.hasModel())return[];const n=Q_.get(this._editor);if(!n)return[];for(const s of t){if(!n.isColorDecoration(s))continue;const r=n.getColorData(s.range.getStartPosition());if(r)return[await vZ(this,this._editor.getModel(),r.colorInfo,r.provider)]}return[]}renderHoverParts(e,t){const i=wZ(this,this._editor,this._themeService,t,e);if(!i)return new Kf([]);this._colorPicker=i.colorPicker;const n={hoverPart:i.hoverPart,hoverElement:this._colorPicker.domNode,dispose(){i.disposables.dispose()}};return new Kf([n])}handleResize(){var e;(e=this._colorPicker)==null||e.layout()}isColorPickerVisible(){return!!this._colorPicker}};l1=bZ([CZ(1,ji)],l1);class J0e{constructor(e,t,i,n){this.owner=e,this.range=t,this.model=i,this.provider=n}}let c1=class{constructor(e,t){this._editor=e,this._themeService=t,this._color=null}async createColorHover(e,t,i){if(!this._editor.hasModel()||!Q_.get(this._editor))return null;const s=await gZ(i,this._editor.getModel(),ze.None);let r=null,a=null;for(const h of s){const u=h.colorInfo;x.containsRange(u.range,e.range)&&(r=u,a=h.provider)}const l=r??e,c=a??t,d=!!r;return{colorHover:await vZ(this,this._editor.getModel(),l,c),foundInEditor:d}}async updateEditorModel(e){if(!this._editor.hasModel())return;const t=e.model;let i=new x(e.range.startLineNumber,e.range.startColumn,e.range.endLineNumber,e.range.endColumn);this._color&&(await fy(this._editor.getModel(),t,this._color,i,e),i=SZ(this._editor,i,t))}renderHoverParts(e,t){return wZ(this,this._editor,this._themeService,t,e)}set color(e){this._color=e}get color(){return this._color}};c1=bZ([CZ(1,ji)],c1);async function vZ(o,e,t,i){const n=e.getValueInRange(t.range),{red:s,green:r,blue:a,alpha:l}=t.color,c=new dt(Math.round(s*255),Math.round(r*255),Math.round(a*255),l),d=new Z(c),h=await fZ(e,t,i,ze.None),u=new U0e(d,[],0);return u.colorPresentations=h||[],u.guessColorPresentation(d,n),o instanceof l1?new Q0e(o,x.lift(t.range),u,i):new J0e(o,x.lift(t.range),u,i)}function wZ(o,e,t,i,n){if(i.length===0||!e.hasModel())return;if(n.setMinimumDimensions){const u=e.getOption(67)+8;n.setMinimumDimensions(new kt(302,u))}const s=new K,r=i[0],a=e.getModel(),l=r.model,c=s.add(new X0e(n.fragment,l,e.getOption(144),t,o instanceof c1));let d=!1,h=new x(r.range.startLineNumber,r.range.startColumn,r.range.endLineNumber,r.range.endColumn);if(o instanceof c1){const u=r.model.color;o.color=u,fy(a,l,u,h,r),s.add(l.onColorFlushed(g=>{o.color=g}))}else s.add(l.onColorFlushed(async u=>{await fy(a,l,u,h,r),d=!0,h=SZ(e,h,l)}));return s.add(l.onDidChangeColor(u=>{fy(a,l,u,h,r)})),s.add(e.onDidChangeModelContent(u=>{d?d=!1:(n.hide(),e.focus())})),{hoverPart:r,colorPicker:c,disposables:s}}function SZ(o,e,t){const i=[],n=t.presentation.textEdit??{range:e,text:t.presentation.label,forceMoveMarkers:!1};i.push(n),t.presentation.additionalTextEdits&&i.push(...t.presentation.additionalTextEdits);const s=x.lift(n.range),r=o.getModel()._setTrackedRange(null,s,3);return o.executeEdits("colorpicker",i),o.pushUndoStop(),o.getModel()._getTrackedRange(r)??s}async function fy(o,e,t,i,n){const s=await fZ(o,{range:i,color:{red:t.rgba.r/255,green:t.rgba.g/255,blue:t.rgba.b/255,alpha:t.rgba.a}},n.provider,ze.None);e.colorPresentations=s||[]}const yZ="editor.action.showHover",eye="editor.action.showDefinitionPreviewHover",tye="editor.action.scrollUpHover",iye="editor.action.scrollDownHover",nye="editor.action.scrollLeftHover",sye="editor.action.scrollRightHover",oye="editor.action.pageUpHover",rye="editor.action.pageDownHover",aye="editor.action.goToTopHover",lye="editor.action.goToBottomHover",TE="editor.action.increaseHoverVerbosityLevel",cye=m({key:"increaseHoverVerbosityLevel",comment:["Label for action that will increase the hover verbosity level."]},"Increase Hover Verbosity Level"),RE="editor.action.decreaseHoverVerbosityLevel",dye=m({key:"decreaseHoverVerbosityLevel",comment:["Label for action that will decrease the hover verbosity level."]},"Decrease Hover Verbosity Level"),LZ="editor.action.inlineSuggest.commit",xZ="editor.action.inlineSuggest.showPrevious",kZ="editor.action.inlineSuggest.showNext";var FW=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Br=function(o,e){return function(t,i){e(t,i,o)}},my;let _O=class extends F{constructor(e,t,i){super(),this.editor=e,this.model=t,this.instantiationService=i,this.alwaysShowToolbar=xt(this,this.editor.onDidChangeConfiguration,()=>this.editor.getOption(62).showToolbar==="always"),this.sessionPosition=void 0,this.position=be(this,n=>{var l,c;const s=(l=this.model.read(n))==null?void 0:l.primaryGhostText.read(n);if(!this.alwaysShowToolbar.read(n)||!s||s.parts.length===0)return this.sessionPosition=void 0,null;const r=s.parts[0].column;this.sessionPosition&&this.sessionPosition.lineNumber!==s.lineNumber&&(this.sessionPosition=void 0);const a=new A(s.lineNumber,Math.min(r,((c=this.sessionPosition)==null?void 0:c.column)??Number.MAX_SAFE_INTEGER));return this.sessionPosition=a,a}),this._register(Ps((n,s)=>{const r=this.model.read(n);if(!r||!this.alwaysShowToolbar.read(n))return;const a=su((c,d)=>{const h=d.add(this.instantiationService.createInstance(J_,this.editor,!0,this.position,r.selectedInlineCompletionIndex,r.inlineCompletionsCount,r.activeCommands));return e.addContentWidget(h),d.add(ve(()=>e.removeContentWidget(h))),d.add(Ae(u=>{this.position.read(u)&&r.lastTriggerKind.read(u)!==Ea.Explicit&&r.triggerExplicitly()})),h}),l=Y1(this,(c,d)=>!!this.position.read(c)||!!d);s.add(Ae(c=>{l.read(c)&&a.read(c)}))}))}};_O=FW([Br(2,we)],_O);const hye=qt("inline-suggestion-hints-next",te.chevronRight,m("parameterHintsNextIcon","Icon for show next parameter hint.")),uye=qt("inline-suggestion-hints-previous",te.chevronLeft,m("parameterHintsPreviousIcon","Icon for show previous parameter hint."));var Ch;let J_=(Ch=class extends F{static get dropDownVisible(){return this._dropDownVisible}createCommandAction(e,t,i){const n=new ro(e,t,i,!0,()=>this._commandService.executeCommand(e)),s=this.keybindingService.lookupKeybinding(e,this._contextKeyService);let r=t;return s&&(r=m({key:"content",comment:["A label","A keybinding"]},"{0} ({1})",t,s.getLabel())),n.tooltip=r,n}constructor(e,t,i,n,s,r,a,l,c,d,h){super(),this.editor=e,this.withBorder=t,this._position=i,this._currentSuggestionIdx=n,this._suggestionCount=s,this._extraCommands=r,this._commandService=a,this.keybindingService=c,this._contextKeyService=d,this._menuService=h,this.id=`InlineSuggestionHintsContentWidget${my.id++}`,this.allowEditorOverflow=!0,this.suppressMouseDown=!1,this.nodes=Xe("div.inlineSuggestionsHints",{className:this.withBorder?".withBorder":""},[Xe("div@toolBar")]),this.previousAction=this.createCommandAction(xZ,m("previous","Previous"),Ie.asClassName(uye)),this.availableSuggestionCountAction=new ro("inlineSuggestionHints.availableSuggestionCount","",void 0,!1),this.nextAction=this.createCommandAction(kZ,m("next","Next"),Ie.asClassName(hye)),this.inlineCompletionsActionsMenus=this._register(this._menuService.createMenu(me.InlineCompletionsActions,this._contextKeyService)),this.clearAvailableSuggestionCountLabelDebounced=this._register(new It(()=>{this.availableSuggestionCountAction.label=""},100)),this.disableButtonsDebounced=this._register(new It(()=>{this.previousAction.enabled=this.nextAction.enabled=!1},100)),this.toolBar=this._register(l.createInstance(bO,this.nodes.toolBar,me.InlineSuggestionToolbar,{menuOptions:{renderShortTitle:!0},toolbarOptions:{primaryGroup:u=>u.startsWith("primary")},actionViewItemProvider:(u,g)=>{if(u instanceof ao)return l.createInstance(fye,u,void 0);if(u===this.availableSuggestionCountAction){const f=new gye(void 0,u,{label:!0,icon:!1});return f.setClass("availableSuggestionCount"),f}},telemetrySource:"InlineSuggestionToolbar"})),this.toolBar.setPrependedPrimaryActions([this.previousAction,this.availableSuggestionCountAction,this.nextAction]),this._register(this.toolBar.onDidChangeDropdownVisibility(u=>{my._dropDownVisible=u})),this._register(Ae(u=>{this._position.read(u),this.editor.layoutContentWidget(this)})),this._register(Ae(u=>{const g=this._suggestionCount.read(u),f=this._currentSuggestionIdx.read(u);g!==void 0?(this.clearAvailableSuggestionCountLabelDebounced.cancel(),this.availableSuggestionCountAction.label=`${f+1}/${g}`):this.clearAvailableSuggestionCountLabelDebounced.schedule(),g!==void 0&&g>1?(this.disableButtonsDebounced.cancel(),this.previousAction.enabled=this.nextAction.enabled=!0):this.disableButtonsDebounced.schedule()})),this._register(Ae(u=>{const f=this._extraCommands.read(u).map(p=>({class:void 0,id:p.id,enabled:!0,tooltip:p.tooltip||"",label:p.title,run:_=>this._commandService.executeCommand(p.id)}));for(const[p,_]of this.inlineCompletionsActionsMenus.getActions())for(const b of _)b instanceof ao&&f.push(b);f.length>0&&f.unshift(new Qi),this.toolBar.setAdditionalSecondaryActions(f)}))}getId(){return this.id}getDomNode(){return this.nodes.root}getPosition(){return{position:this._position.get(),preference:[1,2],positionAffinity:3}}},my=Ch,Ch._dropDownVisible=!1,Ch.id=0,Ch);J_=my=FW([Br(6,Ht),Br(7,we),Br(8,yt),Br(9,Ee),Br(10,Go)],J_);class gye extends V_{constructor(){super(...arguments),this._className=void 0}setClass(e){this._className=e}render(e){super.render(e),this._className&&e.classList.add(this._className)}updateTooltip(){}}let fye=class extends $h{updateLabel(){const e=this._keybindingService.lookupKeybinding(this._action.id,this._contextKeyService);if(!e)return super.updateLabel();if(this.label){const t=Xe("div.keybinding").root;this._register(new Ib(t,so,{disableTitle:!0,...KK})).set(e),this.label.textContent=this._action.label,this.label.appendChild(t),this.label.classList.add("inlineSuggestionStatusBarItemLabel")}}updateTooltip(){}},bO=class extends Qw{constructor(e,t,i,n,s,r,a,l,c){super(e,{resetMenu:t,...i},n,s,r,a,l,c),this.menuId=t,this.options2=i,this.menuService=n,this.contextKeyService=s,this.menu=this._store.add(this.menuService.createMenu(this.menuId,this.contextKeyService,{emitEventsForSubmenuChanges:!0})),this.additionalActions=[],this.prependedPrimaryActions=[],this._store.add(this.menu.onDidChange(()=>this.updateToolbar())),this.updateToolbar()}updateToolbar(){var i,n,s,r,a,l,c;const e=[],t=[];sE(this.menu,(i=this.options2)==null?void 0:i.menuOptions,{primary:e,secondary:t},(s=(n=this.options2)==null?void 0:n.toolbarOptions)==null?void 0:s.primaryGroup,(a=(r=this.options2)==null?void 0:r.toolbarOptions)==null?void 0:a.shouldInlineSubmenu,(c=(l=this.options2)==null?void 0:l.toolbarOptions)==null?void 0:c.useSeparatorsInPrimaryActions),t.push(...this.additionalActions),e.unshift(...this.prependedPrimaryActions),this.setActions(e,t)}setPrependedPrimaryActions(e){ii(this.prependedPrimaryActions,e,(t,i)=>t===i)||(this.prependedPrimaryActions=e,this.updateToolbar())}setAdditionalSecondaryActions(e){ii(this.additionalActions,e,(t,i)=>t===i)||(this.additionalActions=e,this.updateToolbar())}};bO=FW([Br(3,Go),Br(4,Ee),Br(5,mo),Br(6,yt),Br(7,Ht),Br(8,Bn)],bO);function ME(o,e,t){const i=wi(o);return!(e<i.left||e>i.left+i.width||t<i.top||t>i.top+i.height)}let mye=class{constructor(e,t,i){this.value=e,this.isComplete=t,this.hasLoadingMessage=i}};class DZ extends F{constructor(e,t){super(),this._editor=e,this._computer=t,this._onResult=this._register(new P),this.onResult=this._onResult.event,this._firstWaitScheduler=this._register(new It(()=>this._triggerAsyncComputation(),0)),this._secondWaitScheduler=this._register(new It(()=>this._triggerSyncComputation(),0)),this._loadingMessageScheduler=this._register(new It(()=>this._triggerLoadingMessage(),0)),this._state=0,this._asyncIterable=null,this._asyncIterableDone=!1,this._result=[]}dispose(){this._asyncIterable&&(this._asyncIterable.cancel(),this._asyncIterable=null),super.dispose()}get _hoverTime(){return this._editor.getOption(60).delay}get _firstWaitTime(){return this._hoverTime/2}get _secondWaitTime(){return this._hoverTime-this._firstWaitTime}get _loadingMessageTime(){return 3*this._hoverTime}_setState(e,t=!0){this._state=e,t&&this._fireResult()}_triggerAsyncComputation(){this._setState(2),this._secondWaitScheduler.schedule(this._secondWaitTime),this._computer.computeAsync?(this._asyncIterableDone=!1,this._asyncIterable=ote(e=>this._computer.computeAsync(e)),(async()=>{try{for await(const e of this._asyncIterable)e&&(this._result.push(e),this._fireResult());this._asyncIterableDone=!0,(this._state===3||this._state===4)&&this._setState(0)}catch(e){Oe(e)}})()):this._asyncIterableDone=!0}_triggerSyncComputation(){this._computer.computeSync&&(this._result=this._result.concat(this._computer.computeSync())),this._setState(this._asyncIterableDone?0:3)}_triggerLoadingMessage(){this._state===3&&this._setState(4)}_fireResult(){if(this._state===1||this._state===2)return;const e=this._state===0,t=this._state===4;this._onResult.fire(new mye(this._result.slice(0),e,t))}start(e){if(e===0)this._state===0&&(this._setState(1),this._firstWaitScheduler.schedule(this._firstWaitTime),this._loadingMessageScheduler.schedule(this._loadingMessageTime));else switch(this._state){case 0:this._triggerAsyncComputation(),this._secondWaitScheduler.cancel(),this._triggerSyncComputation();break;case 2:this._secondWaitScheduler.cancel(),this._triggerSyncComputation();break}}cancel(){this._firstWaitScheduler.cancel(),this._secondWaitScheduler.cancel(),this._loadingMessageScheduler.cancel(),this._asyncIterable&&(this._asyncIterable.cancel(),this._asyncIterable=null),this._result=[],this._setState(0,!1)}}class WW{constructor(){this._onDidWillResize=new P,this.onDidWillResize=this._onDidWillResize.event,this._onDidResize=new P,this.onDidResize=this._onDidResize.event,this._sashListener=new K,this._size=new kt(0,0),this._minSize=new kt(0,0),this._maxSize=new kt(Number.MAX_SAFE_INTEGER,Number.MAX_SAFE_INTEGER),this.domNode=document.createElement("div"),this._eastSash=new Mn(this.domNode,{getVerticalSashLeft:()=>this._size.width},{orientation:0}),this._westSash=new Mn(this.domNode,{getVerticalSashLeft:()=>0},{orientation:0}),this._northSash=new Mn(this.domNode,{getHorizontalSashTop:()=>0},{orientation:1,orthogonalEdge:WL.North}),this._southSash=new Mn(this.domNode,{getHorizontalSashTop:()=>this._size.height},{orientation:1,orthogonalEdge:WL.South}),this._northSash.orthogonalStartSash=this._westSash,this._northSash.orthogonalEndSash=this._eastSash,this._southSash.orthogonalStartSash=this._westSash,this._southSash.orthogonalEndSash=this._eastSash;let e,t=0,i=0;this._sashListener.add(oe.any(this._northSash.onDidStart,this._eastSash.onDidStart,this._southSash.onDidStart,this._westSash.onDidStart)(()=>{e===void 0&&(this._onDidWillResize.fire(),e=this._size,t=0,i=0)})),this._sashListener.add(oe.any(this._northSash.onDidEnd,this._eastSash.onDidEnd,this._southSash.onDidEnd,this._westSash.onDidEnd)(()=>{e!==void 0&&(e=void 0,t=0,i=0,this._onDidResize.fire({dimension:this._size,done:!0}))})),this._sashListener.add(this._eastSash.onDidChange(n=>{e&&(i=n.currentX-n.startX,this.layout(e.height+t,e.width+i),this._onDidResize.fire({dimension:this._size,done:!1,east:!0}))})),this._sashListener.add(this._westSash.onDidChange(n=>{e&&(i=-(n.currentX-n.startX),this.layout(e.height+t,e.width+i),this._onDidResize.fire({dimension:this._size,done:!1,west:!0}))})),this._sashListener.add(this._northSash.onDidChange(n=>{e&&(t=-(n.currentY-n.startY),this.layout(e.height+t,e.width+i),this._onDidResize.fire({dimension:this._size,done:!1,north:!0}))})),this._sashListener.add(this._southSash.onDidChange(n=>{e&&(t=n.currentY-n.startY,this.layout(e.height+t,e.width+i),this._onDidResize.fire({dimension:this._size,done:!1,south:!0}))})),this._sashListener.add(oe.any(this._eastSash.onDidReset,this._westSash.onDidReset)(n=>{this._preferredSize&&(this.layout(this._size.height,this._preferredSize.width),this._onDidResize.fire({dimension:this._size,done:!0}))})),this._sashListener.add(oe.any(this._northSash.onDidReset,this._southSash.onDidReset)(n=>{this._preferredSize&&(this.layout(this._preferredSize.height,this._size.width),this._onDidResize.fire({dimension:this._size,done:!0}))}))}dispose(){this._northSash.dispose(),this._southSash.dispose(),this._eastSash.dispose(),this._westSash.dispose(),this._sashListener.dispose(),this._onDidResize.dispose(),this._onDidWillResize.dispose(),this.domNode.remove()}enableSashes(e,t,i,n){this._northSash.state=e?3:0,this._eastSash.state=t?3:0,this._southSash.state=i?3:0,this._westSash.state=n?3:0}layout(e=this.size.height,t=this.size.width){const{height:i,width:n}=this._minSize,{height:s,width:r}=this._maxSize;e=Math.max(i,Math.min(s,e)),t=Math.max(n,Math.min(r,t));const a=new kt(t,e);kt.equals(a,this._size)||(this.domNode.style.height=e+"px",this.domNode.style.width=t+"px",this._size=a,this._northSash.layout(),this._eastSash.layout(),this._southSash.layout(),this._westSash.layout())}clearSashHoverState(){this._eastSash.clearSashHoverState(),this._westSash.clearSashHoverState(),this._northSash.clearSashHoverState(),this._southSash.clearSashHoverState()}get size(){return this._size}set maxSize(e){this._maxSize=e}get maxSize(){return this._maxSize}set minSize(e){this._minSize=e}get minSize(){return this._minSize}set preferredSize(e){this._preferredSize=e}get preferredSize(){return this._preferredSize}}const pye=30,_ye=24;class bye extends F{constructor(e,t=new kt(10,10)){super(),this._editor=e,this.allowEditorOverflow=!0,this.suppressMouseDown=!1,this._resizableNode=this._register(new WW),this._contentPosition=null,this._isResizing=!1,this._resizableNode.domNode.style.position="absolute",this._resizableNode.minSize=kt.lift(t),this._resizableNode.layout(t.height,t.width),this._resizableNode.enableSashes(!0,!0,!0,!0),this._register(this._resizableNode.onDidResize(i=>{this._resize(new kt(i.dimension.width,i.dimension.height)),i.done&&(this._isResizing=!1)})),this._register(this._resizableNode.onDidWillResize(()=>{this._isResizing=!0}))}get isResizing(){return this._isResizing}getDomNode(){return this._resizableNode.domNode}getPosition(){return this._contentPosition}get position(){var e;return(e=this._contentPosition)!=null&&e.position?A.lift(this._contentPosition.position):void 0}_availableVerticalSpaceAbove(e){const t=this._editor.getDomNode(),i=this._editor.getScrolledVisiblePosition(e);return!t||!i?void 0:wi(t).top+i.top-pye}_availableVerticalSpaceBelow(e){const t=this._editor.getDomNode(),i=this._editor.getScrolledVisiblePosition(e);if(!t||!i)return;const n=wi(t),s=Hh(t.ownerDocument.body),r=n.top+i.top+i.height;return s.height-r-_ye}_findPositionPreference(e,t){const i=Math.min(this._availableVerticalSpaceBelow(t)??1/0,e),n=Math.min(this._availableVerticalSpaceAbove(t)??1/0,e),s=Math.min(Math.max(n,i),e),r=Math.min(e,s);let a;return this._editor.getOption(60).above?a=r<=n?1:2:a=r<=i?2:1,a===1?this._resizableNode.enableSashes(!0,!0,!1,!1):this._resizableNode.enableSashes(!1,!0,!0,!1),a}_resize(e){this._resizableNode.layout(e.height,e.width)}}var Cye=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},m0=function(o,e){return function(t,i){e(t,i,o)}},al;const BV=30,vye=6;var vh;let CO=(vh=class extends bye{get isVisibleFromKeyboard(){var e;return((e=this._renderedHover)==null?void 0:e.source)===1}get isVisible(){return this._hoverVisibleKey.get()??!1}get isFocused(){return this._hoverFocusedKey.get()??!1}constructor(e,t,i,n,s){const r=e.getOption(67)+8,a=150,l=new kt(a,r);super(e,l),this._configurationService=i,this._accessibilityService=n,this._keybindingService=s,this._hover=this._register(new W5),this._onDidResize=this._register(new P),this.onDidResize=this._onDidResize.event,this._minimumSize=l,this._hoverVisibleKey=I.hoverVisible.bindTo(t),this._hoverFocusedKey=I.hoverFocused.bindTo(t),X(this._resizableNode.domNode,this._hover.containerDomNode),this._resizableNode.domNode.style.zIndex="50",this._register(this._editor.onDidLayoutChange(()=>{this.isVisible&&this._updateMaxDimensions()})),this._register(this._editor.onDidChangeConfiguration(d=>{d.hasChanged(50)&&this._updateFont()}));const c=this._register(Fa(this._resizableNode.domNode));this._register(c.onDidFocus(()=>{this._hoverFocusedKey.set(!0)})),this._register(c.onDidBlur(()=>{this._hoverFocusedKey.set(!1)})),this._setRenderedHover(void 0),this._editor.addContentWidget(this)}dispose(){var e;super.dispose(),(e=this._renderedHover)==null||e.dispose(),this._editor.removeContentWidget(this)}getId(){return al.ID}static _applyDimensions(e,t,i){const n=typeof t=="number"?`${t}px`:t,s=typeof i=="number"?`${i}px`:i;e.style.width=n,e.style.height=s}_setContentsDomNodeDimensions(e,t){const i=this._hover.contentsDomNode;return al._applyDimensions(i,e,t)}_setContainerDomNodeDimensions(e,t){const i=this._hover.containerDomNode;return al._applyDimensions(i,e,t)}_setHoverWidgetDimensions(e,t){this._setContentsDomNodeDimensions(e,t),this._setContainerDomNodeDimensions(e,t),this._layoutContentWidget()}static _applyMaxDimensions(e,t,i){const n=typeof t=="number"?`${t}px`:t,s=typeof i=="number"?`${i}px`:i;e.style.maxWidth=n,e.style.maxHeight=s}_setHoverWidgetMaxDimensions(e,t){al._applyMaxDimensions(this._hover.contentsDomNode,e,t),al._applyMaxDimensions(this._hover.containerDomNode,e,t),this._hover.containerDomNode.style.setProperty("--vscode-hover-maxWidth",typeof e=="number"?`${e}px`:e),this._layoutContentWidget()}_setAdjustedHoverWidgetDimensions(e){this._setHoverWidgetMaxDimensions("none","none");const t=e.width,i=e.height;this._setHoverWidgetDimensions(t,i)}_updateResizableNodeMaxDimensions(){const e=this._findMaximumRenderingWidth()??1/0,t=this._findMaximumRenderingHeight()??1/0;this._resizableNode.maxSize=new kt(e,t),this._setHoverWidgetMaxDimensions(e,t)}_resize(e){al._lastDimensions=new kt(e.width,e.height),this._setAdjustedHoverWidgetDimensions(e),this._resizableNode.layout(e.height,e.width),this._updateResizableNodeMaxDimensions(),this._hover.scrollbar.scanDomNode(),this._editor.layoutContentWidget(this),this._onDidResize.fire()}_findAvailableSpaceVertically(){var t;const e=(t=this._renderedHover)==null?void 0:t.showAtPosition;if(e)return this._positionPreference===1?this._availableVerticalSpaceAbove(e):this._availableVerticalSpaceBelow(e)}_findMaximumRenderingHeight(){const e=this._findAvailableSpaceVertically();if(!e)return;let t=vye;return Array.from(this._hover.contentsDomNode.children).forEach(i=>{t+=i.clientHeight}),Math.min(e,t)}_isHoverTextOverflowing(){this._hover.containerDomNode.style.setProperty("--vscode-hover-whiteSpace","nowrap"),this._hover.containerDomNode.style.setProperty("--vscode-hover-sourceWhiteSpace","nowrap");const e=Array.from(this._hover.contentsDomNode.children).some(t=>t.scrollWidth>t.clientWidth);return this._hover.containerDomNode.style.removeProperty("--vscode-hover-whiteSpace"),this._hover.containerDomNode.style.removeProperty("--vscode-hover-sourceWhiteSpace"),e}_findMaximumRenderingWidth(){if(!this._editor||!this._editor.hasModel())return;const e=this._isHoverTextOverflowing(),t=typeof this._contentWidth>"u"?0:this._contentWidth-2;return e||this._hover.containerDomNode.clientWidth<t?Hh(this._hover.containerDomNode.ownerDocument.body).width-14:this._hover.containerDomNode.clientWidth+2}isMouseGettingCloser(e,t){if(!this._renderedHover)return!1;if(this._renderedHover.initialMousePosX===void 0||this._renderedHover.initialMousePosY===void 0)return this._renderedHover.initialMousePosX=e,this._renderedHover.initialMousePosY=t,!1;const i=wi(this.getDomNode());this._renderedHover.closestMouseDistance===void 0&&(this._renderedHover.closestMouseDistance=HV(this._renderedHover.initialMousePosX,this._renderedHover.initialMousePosY,i.left,i.top,i.width,i.height));const n=HV(e,t,i.left,i.top,i.width,i.height);return n>this._renderedHover.closestMouseDistance+4?!1:(this._renderedHover.closestMouseDistance=Math.min(this._renderedHover.closestMouseDistance,n),!0)}_setRenderedHover(e){var t;(t=this._renderedHover)==null||t.dispose(),this._renderedHover=e,this._hoverVisibleKey.set(!!e),this._hover.containerDomNode.classList.toggle("hidden",!e)}_updateFont(){const{fontSize:e,lineHeight:t}=this._editor.getOption(50),i=this._hover.contentsDomNode;i.style.fontSize=`${e}px`,i.style.lineHeight=`${t/e}`,Array.prototype.slice.call(this._hover.contentsDomNode.getElementsByClassName("code")).forEach(s=>this._editor.applyFontInfo(s))}_updateContent(e){const t=this._hover.contentsDomNode;t.style.paddingBottom="",t.textContent="",t.appendChild(e)}_layoutContentWidget(){this._editor.layoutContentWidget(this),this._hover.onContentsChanged()}_updateMaxDimensions(){const e=Math.max(this._editor.getLayoutInfo().height/4,250,al._lastDimensions.height),t=Math.max(this._editor.getLayoutInfo().width*.66,500,al._lastDimensions.width);this._setHoverWidgetMaxDimensions(t,e)}_render(e){this._setRenderedHover(e),this._updateFont(),this._updateContent(e.domNode),this._updateMaxDimensions(),this.onContentsChanged(),this._editor.render()}getPosition(){return this._renderedHover?{position:this._renderedHover.showAtPosition,secondaryPosition:this._renderedHover.showAtSecondaryPosition,positionAffinity:this._renderedHover.shouldAppearBeforeContent?3:void 0,preference:[this._positionPreference??1]}:null}show(e){var r;if(!this._editor||!this._editor.hasModel())return;this._render(e);const t=Tc(this._hover.containerDomNode),i=e.showAtPosition;this._positionPreference=this._findPositionPreference(t,i)??1,this.onContentsChanged(),e.shouldFocus&&this._hover.containerDomNode.focus(),this._onDidResize.fire();const s=this._hover.containerDomNode.ownerDocument.activeElement===this._hover.containerDomNode&&Y$(this._configurationService.getValue("accessibility.verbosity.hover")===!0&&this._accessibilityService.isScreenReaderOptimized(),((r=this._keybindingService.lookupKeybinding("editor.action.accessibleView"))==null?void 0:r.getAriaLabel())??"");s&&(this._hover.contentsDomNode.ariaLabel=this._hover.contentsDomNode.textContent+", "+s)}hide(){if(!this._renderedHover)return;const e=this._renderedHover.shouldFocus||this._hoverFocusedKey.get();this._setRenderedHover(void 0),this._resizableNode.maxSize=new kt(1/0,1/0),this._resizableNode.clearSashHoverState(),this._hoverFocusedKey.set(!1),this._editor.layoutContentWidget(this),e&&this._editor.focus()}_removeConstraintsRenderNormally(){const e=this._editor.getLayoutInfo();this._resizableNode.layout(e.height,e.width),this._setHoverWidgetDimensions("auto","auto")}setMinimumDimensions(e){this._minimumSize=new kt(Math.max(this._minimumSize.width,e.width),Math.max(this._minimumSize.height,e.height)),this._updateMinimumWidth()}_updateMinimumWidth(){const e=typeof this._contentWidth>"u"?this._minimumSize.width:Math.min(this._contentWidth,this._minimumSize.width);this._resizableNode.minSize=new kt(e,this._minimumSize.height)}onContentsChanged(){var n;this._removeConstraintsRenderNormally();const e=this._hover.containerDomNode;let t=Tc(e),i=Xs(e);if(this._resizableNode.layout(t,i),this._setHoverWidgetDimensions(i,t),t=Tc(e),i=Xs(e),this._contentWidth=i,this._updateMinimumWidth(),this._resizableNode.layout(t,i),(n=this._renderedHover)!=null&&n.showAtPosition){const s=Tc(this._hover.containerDomNode);this._positionPreference=this._findPositionPreference(s,this._renderedHover.showAtPosition)}this._layoutContentWidget()}focus(){this._hover.containerDomNode.focus()}scrollUp(){const e=this._hover.scrollbar.getScrollPosition().scrollTop,t=this._editor.getOption(50);this._hover.scrollbar.setScrollPosition({scrollTop:e-t.lineHeight})}scrollDown(){const e=this._hover.scrollbar.getScrollPosition().scrollTop,t=this._editor.getOption(50);this._hover.scrollbar.setScrollPosition({scrollTop:e+t.lineHeight})}scrollLeft(){const e=this._hover.scrollbar.getScrollPosition().scrollLeft;this._hover.scrollbar.setScrollPosition({scrollLeft:e-BV})}scrollRight(){const e=this._hover.scrollbar.getScrollPosition().scrollLeft;this._hover.scrollbar.setScrollPosition({scrollLeft:e+BV})}pageUp(){const e=this._hover.scrollbar.getScrollPosition().scrollTop,t=this._hover.scrollbar.getScrollDimensions().height;this._hover.scrollbar.setScrollPosition({scrollTop:e-t})}pageDown(){const e=this._hover.scrollbar.getScrollPosition().scrollTop,t=this._hover.scrollbar.getScrollDimensions().height;this._hover.scrollbar.setScrollPosition({scrollTop:e+t})}goToTop(){this._hover.scrollbar.setScrollPosition({scrollTop:0})}goToBottom(){this._hover.scrollbar.setScrollPosition({scrollTop:this._hover.scrollbar.getScrollDimensions().scrollHeight})}},al=vh,vh.ID="editor.contrib.resizableContentHoverWidget",vh._lastDimensions=new kt(0,0),vh);CO=al=Cye([m0(1,Ee),m0(2,qe),m0(3,po),m0(4,yt)],CO);function HV(o,e,t,i,n,s){const r=t+n/2,a=i+s/2,l=Math.max(Math.abs(o-r)-n/2,0),c=Math.max(Math.abs(e-a)-s/2,0);return Math.sqrt(l*l+c*c)}class Mx{get anchor(){return this._anchor}set anchor(e){this._anchor=e}get shouldFocus(){return this._shouldFocus}set shouldFocus(e){this._shouldFocus=e}get source(){return this._source}set source(e){this._source=e}get insistOnKeepingHoverVisible(){return this._insistOnKeepingHoverVisible}set insistOnKeepingHoverVisible(e){this._insistOnKeepingHoverVisible=e}constructor(e,t){this._editor=e,this._participants=t,this._anchor=null,this._shouldFocus=!1,this._source=0,this._insistOnKeepingHoverVisible=!1}static _getLineDecorations(e,t){if(t.type!==1&&!t.supportsMarkerHover)return[];const i=e.getModel(),n=t.range.startLineNumber;if(n>i.getLineCount())return[];const s=i.getLineMaxColumn(n);return e.getLineDecorations(n).filter(r=>{if(r.options.isWholeLine)return!0;const a=r.range.startLineNumber===n?r.range.startColumn:1,l=r.range.endLineNumber===n?r.range.endColumn:s;if(r.options.showIfCollapsed){if(a>t.range.startColumn+1||t.range.endColumn-1>l)return!1}else if(a>t.range.startColumn||t.range.endColumn>l)return!1;return!0})}computeAsync(e){const t=this._anchor;if(!this._editor.hasModel()||!t)return Fo.EMPTY;const i=Mx._getLineDecorations(this._editor,t);return Fo.merge(this._participants.map(n=>n.computeAsync?n.computeAsync(t,i,e):Fo.EMPTY))}computeSync(){if(!this._editor.hasModel()||!this._anchor)return[];const e=Mx._getLineDecorations(this._editor,this._anchor);let t=[];for(const i of this._participants)t=t.concat(i.computeSync(this._anchor,e));return Kr(t)}}class IZ{constructor(e,t,i){this.anchor=e,this.hoverParts=t,this.isComplete=i}filter(e){const t=this.hoverParts.filter(i=>i.isValidForHoverAnchor(e));return t.length===this.hoverParts.length?this:new wye(this,this.anchor,t,this.isComplete)}}class wye extends IZ{constructor(e,t,i,n){super(t,i,n),this.original=e}filter(e){return this.original.filter(e)}}var Sye=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},yye=function(o,e){return function(t,i){e(t,i,o)}};const VV=ne;let Ax=class extends F{get hasContent(){return this._hasContent}constructor(e){super(),this._keybindingService=e,this.actions=[],this._hasContent=!1,this.hoverElement=VV("div.hover-row.status-bar"),this.hoverElement.tabIndex=0,this.actionsElement=X(this.hoverElement,VV("div.actions"))}addAction(e){const t=this._keybindingService.lookupKeybinding(e.commandId),i=t?t.getLabel():null;this._hasContent=!0;const n=this._register(KI.render(this.actionsElement,e,i));return this.actions.push(n),n}append(e){const t=X(this.actionsElement,e);return this._hasContent=!0,t}};Ax=Sye([yye(0,yt)],Ax);class Lye{constructor(e,t,i){this.provider=e,this.hover=t,this.ordinal=i}}async function xye(o,e,t,i,n){const s=await Promise.resolve(o.provideHover(t,i,n)).catch(Si);if(!(!s||!kye(s)))return new Lye(o,s,e)}function BW(o,e,t,i,n=!1){const r=o.ordered(e,n).map((a,l)=>xye(a,l,e,t,i));return Fo.fromPromises(r).coalesce()}function EZ(o,e,t,i,n=!1){return BW(o,e,t,i,n).map(s=>s.hover).toPromise()}Vs("_executeHoverProvider",(o,e,t)=>{const i=o.get(ge);return EZ(i.hoverProvider,e,t,ze.None)});Vs("_executeHoverProvider_recursive",(o,e,t)=>{const i=o.get(ge);return EZ(i.hoverProvider,e,t,ze.None,!0)});function kye(o){const e=typeof o.range<"u",t=typeof o.contents<"u"&&o.contents&&o.contents.length>0;return e&&t}var Dye=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Lu=function(o,e){return function(t,i){e(t,i,o)}};const _p=ne,Iye=qt("hover-increase-verbosity",te.add,m("increaseHoverVerbosity","Icon for increaseing hover verbosity.")),Eye=qt("hover-decrease-verbosity",te.remove,m("decreaseHoverVerbosity","Icon for decreasing hover verbosity."));class Hr{constructor(e,t,i,n,s,r=void 0){this.owner=e,this.range=t,this.contents=i,this.isBeforeContent=n,this.ordinal=s,this.source=r}isValidForHoverAnchor(e){return e.type===1&&this.range.startColumn<=e.range.startColumn&&this.range.endColumn>=e.range.endColumn}}class NZ{constructor(e,t,i){this.hover=e,this.hoverProvider=t,this.hoverPosition=i}supportsVerbosityAction(e){switch(e){case eo.Increase:return this.hover.canIncreaseVerbosity??!1;case eo.Decrease:return this.hover.canDecreaseVerbosity??!1}}}let d1=class{constructor(e,t,i,n,s,r,a,l){this._editor=e,this._languageService=t,this._openerService=i,this._configurationService=n,this._languageFeaturesService=s,this._keybindingService=r,this._hoverService=a,this._commandService=l,this.hoverOrdinal=3}createLoadingMessage(e){return new Hr(this,e.range,[new Pn().appendText(m("modesContentHover.loading","Loading..."))],!1,2e3)}computeSync(e,t){if(!this._editor.hasModel()||e.type!==1)return[];const i=this._editor.getModel(),n=e.range.startLineNumber,s=i.getLineMaxColumn(n),r=[];let a=1e3;const l=i.getLineLength(n),c=i.getLanguageIdAtPosition(e.range.startLineNumber,e.range.startColumn),d=this._editor.getOption(118),h=this._configurationService.getValue("editor.maxTokenizationLineLength",{overrideIdentifier:c});let u=!1;d>=0&&l>d&&e.range.startColumn>=d&&(u=!0,r.push(new Hr(this,e.range,[{value:m("stopped rendering","Rendering paused for long line for performance reasons. This can be configured via `editor.stopRenderingLineAfter`.")}],!1,a++))),!u&&typeof h=="number"&&l>=h&&r.push(new Hr(this,e.range,[{value:m("too many characters","Tokenization is skipped for long lines for performance reasons. This can be configured via `editor.maxTokenizationLineLength`.")}],!1,a++));let g=!1;for(const f of t){const p=f.range.startLineNumber===n?f.range.startColumn:1,_=f.range.endLineNumber===n?f.range.endColumn:s,b=f.options.hoverMessage;if(!b||O_(b))continue;f.options.beforeContentClassName&&(g=!0);const C=new x(e.range.startLineNumber,p,e.range.startLineNumber,_);r.push(new Hr(this,C,F4(b),g,a++))}return r}computeAsync(e,t,i){if(!this._editor.hasModel()||e.type!==1)return Fo.EMPTY;const n=this._editor.getModel(),s=this._languageFeaturesService.hoverProvider;return s.has(n)?this._getMarkdownHovers(s,n,e,i):Fo.EMPTY}_getMarkdownHovers(e,t,i,n){const s=i.range.getStartPosition();return BW(e,t,s,n).filter(l=>!O_(l.hover.contents)).map(l=>{const c=l.hover.range?x.lift(l.hover.range):i.range,d=new NZ(l.hover,l.provider,s);return new Hr(this,c,l.hover.contents,!1,l.ordinal,d)})}renderHoverParts(e,t){return this._renderedHoverParts=new Nye(t,e.fragment,this,this._editor,this._languageService,this._openerService,this._commandService,this._keybindingService,this._hoverService,this._configurationService,e.onContentsChanged),this._renderedHoverParts}updateMarkdownHoverVerbosityLevel(e,t,i){var n;return Promise.resolve((n=this._renderedHoverParts)==null?void 0:n.updateMarkdownHoverPartVerbosityLevel(e,t,i))}};d1=Dye([Lu(1,ni),Lu(2,zs),Lu(3,qe),Lu(4,ge),Lu(5,yt),Lu(6,Yl),Lu(7,Ht)],d1);class p0{constructor(e,t,i){this.hoverPart=e,this.hoverElement=t,this.disposables=i}dispose(){this.disposables.dispose()}}class Nye{constructor(e,t,i,n,s,r,a,l,c,d,h){this._hoverParticipant=i,this._editor=n,this._languageService=s,this._openerService=r,this._commandService=a,this._keybindingService=l,this._hoverService=c,this._configurationService=d,this._onFinishedRendering=h,this._ongoingHoverOperations=new Map,this._disposables=new K,this.renderedHoverParts=this._renderHoverParts(e,t,this._onFinishedRendering),this._disposables.add(ve(()=>{this.renderedHoverParts.forEach(u=>{u.dispose()}),this._ongoingHoverOperations.forEach(u=>{u.tokenSource.dispose(!0)})}))}_renderHoverParts(e,t,i){return e.sort(gs(n=>n.ordinal,$r)),e.map(n=>{const s=this._renderHoverPart(n,i);return t.appendChild(s.hoverElement),s})}_renderHoverPart(e,t){const i=this._renderMarkdownHover(e,t),n=i.hoverElement,s=e.source,r=new K;if(r.add(i),!s)return new p0(e,n,r);const a=s.supportsVerbosityAction(eo.Increase),l=s.supportsVerbosityAction(eo.Decrease);if(!a&&!l)return new p0(e,n,r);const c=_p("div.verbosity-actions");return n.prepend(c),r.add(this._renderHoverExpansionAction(c,eo.Increase,a)),r.add(this._renderHoverExpansionAction(c,eo.Decrease,l)),new p0(e,n,r)}_renderMarkdownHover(e,t){return TZ(this._editor,e,this._languageService,this._openerService,t)}_renderHoverExpansionAction(e,t,i){const n=new K,s=t===eo.Increase,r=X(e,_p(Ie.asCSSSelector(s?Iye:Eye)));r.tabIndex=0;const a=new M_("mouse",!1,{target:e,position:{hoverPosition:0}},this._configurationService,this._hoverService);if(n.add(this._hoverService.setupManagedHover(a,r,Rye(this._keybindingService,t))),!i)return r.classList.add("disabled"),n;r.classList.add("enabled");const l=()=>this._commandService.executeCommand(t===eo.Increase?TE:RE);return n.add(new X$(r,l)),n.add(new Q$(r,l,[3,10])),n}async updateMarkdownHoverPartVerbosityLevel(e,t,i=!0){const n=this._editor.getModel();if(!n)return;const s=this._getRenderedHoverPartAtIndex(t),r=s==null?void 0:s.hoverPart.source;if(!s||!(r!=null&&r.supportsVerbosityAction(e)))return;const a=await this._fetchHover(r,n,e);if(!a)return;const l=new NZ(a,r.hoverProvider,r.hoverPosition),c=s.hoverPart,d=new Hr(this._hoverParticipant,c.range,a.contents,c.isBeforeContent,c.ordinal,l),h=this._renderHoverPart(d,this._onFinishedRendering);return this._replaceRenderedHoverPartAtIndex(t,h,d),i&&this._focusOnHoverPartWithIndex(t),{hoverPart:d,hoverElement:h.hoverElement}}async _fetchHover(e,t,i){let n=i===eo.Increase?1:-1;const s=e.hoverProvider,r=this._ongoingHoverOperations.get(s);r&&(r.tokenSource.cancel(),n+=r.verbosityDelta);const a=new li;this._ongoingHoverOperations.set(s,{verbosityDelta:n,tokenSource:a});const l={verbosityRequest:{verbosityDelta:n,previousHover:e.hover}};let c;try{c=await Promise.resolve(s.provideHover(t,e.hoverPosition,a.token,l))}catch(d){Si(d)}return a.dispose(),this._ongoingHoverOperations.delete(s),c}_replaceRenderedHoverPartAtIndex(e,t,i){if(e>=this.renderedHoverParts.length||e<0)return;const n=this.renderedHoverParts[e],s=n.hoverElement,r=t.hoverElement,a=Array.from(r.children);s.replaceChildren(...a);const l=new p0(i,s,t.disposables);s.focus(),n.dispose(),this.renderedHoverParts[e]=l}_focusOnHoverPartWithIndex(e){this.renderedHoverParts[e].hoverElement.focus()}_getRenderedHoverPartAtIndex(e){return this.renderedHoverParts[e]}dispose(){this._disposables.dispose()}}function Tye(o,e,t,i,n){e.sort(gs(r=>r.ordinal,$r));const s=[];for(const r of e)s.push(TZ(t,r,i,n,o.onContentsChanged));return new Kf(s)}function TZ(o,e,t,i,n){const s=new K,r=_p("div.hover-row"),a=_p("div.hover-row-contents");r.appendChild(a);const l=e.contents;for(const d of l){if(O_(d))continue;const h=_p("div.markdown-hover"),u=X(h,_p("div.hover-contents")),g=s.add(new nd({editor:o},t,i));s.add(g.onDidRenderAsync(()=>{u.className="hover-contents code-hover-contents",n()}));const f=s.add(g.render(d));u.appendChild(f.element),a.appendChild(h)}return{hoverPart:e,hoverElement:r,dispose(){s.dispose()}}}function Rye(o,e){switch(e){case eo.Increase:{const t=o.lookupKeybinding(TE);return t?m("increaseVerbosityWithKb","Increase Hover Verbosity ({0})",t.getLabel()):m("increaseVerbosity","Increase Hover Verbosity")}case eo.Decrease:{const t=o.lookupKeybinding(RE);return t?m("decreaseVerbosityWithKb","Decrease Hover Verbosity ({0})",t.getLabel()):m("decreaseVerbosity","Decrease Hover Verbosity")}}}function vO(o,e){return!!o[e]}class BT{constructor(e,t){this.target=e.target,this.isLeftClick=e.event.leftButton,this.isMiddleClick=e.event.middleButton,this.isRightClick=e.event.rightButton,this.hasTriggerModifier=vO(e.event,t.triggerModifier),this.hasSideBySideModifier=vO(e.event,t.triggerSideBySideModifier),this.isNoneOrSingleMouseDown=e.event.detail<=1}}class zV{constructor(e,t){this.keyCodeIsTriggerKey=e.keyCode===t.triggerKey,this.keyCodeIsSideBySideKey=e.keyCode===t.triggerSideBySideKey,this.hasTriggerModifier=vO(e,t.triggerModifier)}}class _0{constructor(e,t,i,n){this.triggerKey=e,this.triggerModifier=t,this.triggerSideBySideKey=i,this.triggerSideBySideModifier=n}equals(e){return this.triggerKey===e.triggerKey&&this.triggerModifier===e.triggerModifier&&this.triggerSideBySideKey===e.triggerSideBySideKey&&this.triggerSideBySideModifier===e.triggerSideBySideModifier}}function UV(o){return o==="altKey"?Je?new _0(57,"metaKey",6,"altKey"):new _0(5,"ctrlKey",6,"altKey"):Je?new _0(6,"altKey",57,"metaKey"):new _0(6,"altKey",5,"ctrlKey")}class AE extends F{constructor(e,t){super(),this._onMouseMoveOrRelevantKeyDown=this._register(new P),this.onMouseMoveOrRelevantKeyDown=this._onMouseMoveOrRelevantKeyDown.event,this._onExecute=this._register(new P),this.onExecute=this._onExecute.event,this._onCancel=this._register(new P),this.onCancel=this._onCancel.event,this._editor=e,this._extractLineNumberFromMouseEvent=(t==null?void 0:t.extractLineNumberFromMouseEvent)??(i=>i.target.position?i.target.position.lineNumber:0),this._opts=UV(this._editor.getOption(78)),this._lastMouseMoveEvent=null,this._hasTriggerKeyOnMouseDown=!1,this._lineNumberOnMouseDown=0,this._register(this._editor.onDidChangeConfiguration(i=>{if(i.hasChanged(78)){const n=UV(this._editor.getOption(78));if(this._opts.equals(n))return;this._opts=n,this._lastMouseMoveEvent=null,this._hasTriggerKeyOnMouseDown=!1,this._lineNumberOnMouseDown=0,this._onCancel.fire()}})),this._register(this._editor.onMouseMove(i=>this._onEditorMouseMove(new BT(i,this._opts)))),this._register(this._editor.onMouseDown(i=>this._onEditorMouseDown(new BT(i,this._opts)))),this._register(this._editor.onMouseUp(i=>this._onEditorMouseUp(new BT(i,this._opts)))),this._register(this._editor.onKeyDown(i=>this._onEditorKeyDown(new zV(i,this._opts)))),this._register(this._editor.onKeyUp(i=>this._onEditorKeyUp(new zV(i,this._opts)))),this._register(this._editor.onMouseDrag(()=>this._resetHandler())),this._register(this._editor.onDidChangeCursorSelection(i=>this._onDidChangeCursorSelection(i))),this._register(this._editor.onDidChangeModel(i=>this._resetHandler())),this._register(this._editor.onDidChangeModelContent(()=>this._resetHandler())),this._register(this._editor.onDidScrollChange(i=>{(i.scrollTopChanged||i.scrollLeftChanged)&&this._resetHandler()}))}_onDidChangeCursorSelection(e){e.selection&&e.selection.startColumn!==e.selection.endColumn&&this._resetHandler()}_onEditorMouseMove(e){this._lastMouseMoveEvent=e,this._onMouseMoveOrRelevantKeyDown.fire([e,null])}_onEditorMouseDown(e){this._hasTriggerKeyOnMouseDown=e.hasTriggerModifier,this._lineNumberOnMouseDown=this._extractLineNumberFromMouseEvent(e)}_onEditorMouseUp(e){const t=this._extractLineNumberFromMouseEvent(e);this._hasTriggerKeyOnMouseDown&&this._lineNumberOnMouseDown&&this._lineNumberOnMouseDown===t&&this._onExecute.fire(e)}_onEditorKeyDown(e){this._lastMouseMoveEvent&&(e.keyCodeIsTriggerKey||e.keyCodeIsSideBySideKey&&e.hasTriggerModifier)?this._onMouseMoveOrRelevantKeyDown.fire([this._lastMouseMoveEvent,e]):e.hasTriggerModifier&&this._onCancel.fire()}_onEditorKeyUp(e){e.keyCodeIsTriggerKey&&this._onCancel.fire()}_resetHandler(){this._lastMouseMoveEvent=null,this._hasTriggerKeyOnMouseDown=!1,this._onCancel.fire()}}class RZ{constructor(e,t){this.range=e,this.direction=t}}class HW{constructor(e,t,i){this.hint=e,this.anchor=t,this.provider=i,this._isResolved=!1}with(e){const t=new HW(this.hint,e.anchor,this.provider);return t._isResolved=this._isResolved,t._currentResolve=this._currentResolve,t}async resolve(e){if(typeof this.provider.resolveInlayHint=="function"){if(this._currentResolve)return await this._currentResolve,e.isCancellationRequested?void 0:this.resolve(e);this._isResolved||(this._currentResolve=this._doResolve(e).finally(()=>this._currentResolve=void 0)),await this._currentResolve}}async _doResolve(e){try{const t=await Promise.resolve(this.provider.resolveInlayHint(this.hint,e));this.hint.tooltip=(t==null?void 0:t.tooltip)??this.hint.tooltip,this.hint.label=(t==null?void 0:t.label)??this.hint.label,this.hint.textEdits=(t==null?void 0:t.textEdits)??this.hint.textEdits,this._isResolved=!0}catch(t){Si(t),this._isResolved=!1}}}const Ap=class Ap{static async create(e,t,i,n){const s=[],r=e.ordered(t).reverse().map(a=>i.map(async l=>{try{const c=await a.provideInlayHints(t,l,n);(c!=null&&c.hints.length||a.onDidChangeInlayHints)&&s.push([c??Ap._emptyInlayHintList,a])}catch(c){Si(c)}}));if(await Promise.all(r.flat()),n.isCancellationRequested||t.isDisposed())throw new qr;return new Ap(i,s,t)}constructor(e,t,i){this._disposables=new K,this.ranges=e,this.provider=new Set;const n=[];for(const[s,r]of t){this._disposables.add(s),this.provider.add(r);for(const a of s.hints){const l=i.validatePosition(a.position);let c="before";const d=Ap._getRangeAtPosition(i,l);let h;d.getStartPosition().isBefore(l)?(h=x.fromPositions(d.getStartPosition(),l),c="after"):(h=x.fromPositions(l,d.getEndPosition()),c="before"),n.push(new HW(a,new RZ(h,c),r))}}this.items=n.sort((s,r)=>A.compare(s.hint.position,r.hint.position))}dispose(){this._disposables.dispose()}static _getRangeAtPosition(e,t){const i=t.lineNumber,n=e.getWordAtPosition(t);if(n)return new x(i,n.startColumn,i,n.endColumn);e.tokenization.tokenizeIfCheap(i);const s=e.tokenization.getLineTokens(i),r=t.column-1,a=s.findTokenIndexAtOffset(r);let l=s.getStartOffset(a),c=s.getEndOffset(a);return c-l===1&&(l===r&&a>1?(l=s.getStartOffset(a-1),c=s.getEndOffset(a-1)):c===r&&a<s.getCount()-1&&(l=s.getStartOffset(a+1),c=s.getEndOffset(a+1))),new x(i,l+1,i,c+1)}};Ap._emptyInlayHintList=Object.freeze({dispose(){},hints:[]});let Px=Ap;function Mye(o){return Se.from({scheme:Fe.command,path:o.id,query:o.arguments&&encodeURIComponent(JSON.stringify(o.arguments))}).toString()}var Aye=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},rc=function(o,e){return function(t,i){e(t,i,o)}};let Jr=class extends Zw{constructor(e,t,i,n,s,r,a,l,c,d,h,u,g){super(e,{...n.getRawOptions(),overflowWidgetsDomNode:n.getOverflowWidgetsDomNode()},i,s,r,a,l,c,d,h,u,g),this._parentEditor=n,this._overwriteOptions=t,super.updateOptions(this._overwriteOptions),this._register(n.onDidChangeConfiguration(f=>this._onParentConfigurationChanged(f)))}getParentEditor(){return this._parentEditor}_onParentConfigurationChanged(e){super.updateOptions(this._parentEditor.getRawOptions()),super.updateOptions(this._overwriteOptions)}updateOptions(e){dI(this._overwriteOptions,e,!0),super.updateOptions(this._overwriteOptions)}};Jr=Aye([rc(4,we),rc(5,pt),rc(6,Ht),rc(7,Ee),rc(8,ji),rc(9,Ei),rc(10,po),rc(11,di),rc(12,ge)],Jr);const $V=new Z(new dt(0,122,204)),Pye={showArrow:!0,showFrame:!0,className:"",frameColor:$V,arrowColor:$V,keepEditorSelection:!1},Oye="vs.editor.contrib.zoneWidget";class Fye{constructor(e,t,i,n,s,r,a,l){this.id="",this.domNode=e,this.afterLineNumber=t,this.afterColumn=i,this.heightInLines=n,this.showInHiddenAreas=a,this.ordinal=l,this._onDomNodeTop=s,this._onComputedHeight=r}onDomNodeTop(e){this._onDomNodeTop(e)}onComputedHeight(e){this._onComputedHeight(e)}}class Wye{constructor(e,t){this._id=e,this._domNode=t}getId(){return this._id}getDomNode(){return this._domNode}getPosition(){return null}}const yD=class yD{constructor(e){this._editor=e,this._ruleName=yD._IdGenerator.nextId(),this._decorations=this._editor.createDecorationsCollection(),this._color=null,this._height=-1}dispose(){this.hide(),NM(this._ruleName)}set color(e){this._color!==e&&(this._color=e,this._updateStyle())}set height(e){this._height!==e&&(this._height=e,this._updateStyle())}_updateStyle(){NM(this._ruleName),Ky(`.monaco-editor ${this._ruleName}`,`border-style: solid; border-color: transparent; border-bottom-color: ${this._color}; border-width: ${this._height}px; bottom: -${this._height}px !important; margin-left: -${this._height}px; `)}show(e){e.column===1&&(e={lineNumber:e.lineNumber,column:2}),this._decorations.set([{range:x.fromPositions(e),options:{description:"zone-widget-arrow",className:this._ruleName,stickiness:1}}])}hide(){this._decorations.clear()}};yD._IdGenerator=new q5(".arrow-decoration-");let wO=yD;class Bye{constructor(e,t={}){this._arrow=null,this._overlayWidget=null,this._resizeSash=null,this._viewZone=null,this._disposables=new K,this.container=null,this._isShowing=!1,this.editor=e,this._positionMarkerId=this.editor.createDecorationsCollection(),this.options=vc(t),dI(this.options,Pye,!1),this.domNode=document.createElement("div"),this.options.isAccessible||(this.domNode.setAttribute("aria-hidden","true"),this.domNode.setAttribute("role","presentation")),this._disposables.add(this.editor.onDidLayoutChange(i=>{const n=this._getWidth(i);this.domNode.style.width=n+"px",this.domNode.style.left=this._getLeft(i)+"px",this._onWidth(n)}))}dispose(){this._overlayWidget&&(this.editor.removeOverlayWidget(this._overlayWidget),this._overlayWidget=null),this._viewZone&&this.editor.changeViewZones(e=>{this._viewZone&&e.removeZone(this._viewZone.id),this._viewZone=null}),this._positionMarkerId.clear(),this._disposables.dispose()}create(){this.domNode.classList.add("zone-widget"),this.options.className&&this.domNode.classList.add(this.options.className),this.container=document.createElement("div"),this.container.classList.add("zone-widget-container"),this.domNode.appendChild(this.container),this.options.showArrow&&(this._arrow=new wO(this.editor),this._disposables.add(this._arrow)),this._fillContainer(this.container),this._initSash(),this._applyStyles()}style(e){e.frameColor&&(this.options.frameColor=e.frameColor),e.arrowColor&&(this.options.arrowColor=e.arrowColor),this._applyStyles()}_applyStyles(){if(this.container&&this.options.frameColor){const e=this.options.frameColor.toString();this.container.style.borderTopColor=e,this.container.style.borderBottomColor=e}if(this._arrow&&this.options.arrowColor){const e=this.options.arrowColor.toString();this._arrow.color=e}}_getWidth(e){return e.width-e.minimap.minimapWidth-e.verticalScrollbarWidth}_getLeft(e){return e.minimap.minimapWidth>0&&e.minimap.minimapLeft===0?e.minimap.minimapWidth:0}_onViewZoneTop(e){this.domNode.style.top=e+"px"}_onViewZoneHeight(e){var t;if(this.domNode.style.height=`${e}px`,this.container){const i=e-this._decoratingElementsHeight();this.container.style.height=`${i}px`;const n=this.editor.getLayoutInfo();this._doLayout(i,this._getWidth(n))}(t=this._resizeSash)==null||t.layout()}get position(){const e=this._positionMarkerId.getRange(0);if(e)return e.getStartPosition()}show(e,t){const i=x.isIRange(e)?x.lift(e):x.fromPositions(e);this._isShowing=!0,this._showImpl(i,t),this._isShowing=!1,this._positionMarkerId.set([{range:i,options:We.EMPTY}])}hide(){var e;this._viewZone&&(this.editor.changeViewZones(t=>{this._viewZone&&t.removeZone(this._viewZone.id)}),this._viewZone=null),this._overlayWidget&&(this.editor.removeOverlayWidget(this._overlayWidget),this._overlayWidget=null),(e=this._arrow)==null||e.hide(),this._positionMarkerId.clear()}_decoratingElementsHeight(){const e=this.editor.getOption(67);let t=0;if(this.options.showArrow){const i=Math.round(e/3);t+=2*i}if(this.options.showFrame){const i=Math.round(e/9);t+=2*i}return t}_showImpl(e,t){const i=e.getStartPosition(),n=this.editor.getLayoutInfo(),s=this._getWidth(n);this.domNode.style.width=`${s}px`,this.domNode.style.left=this._getLeft(n)+"px";const r=document.createElement("div");r.style.overflow="hidden";const a=this.editor.getOption(67);if(!this.options.allowUnlimitedHeight){const u=Math.max(12,this.editor.getLayoutInfo().height/a*.8);t=Math.min(t,u)}let l=0,c=0;if(this._arrow&&this.options.showArrow&&(l=Math.round(a/3),this._arrow.height=l,this._arrow.show(i)),this.options.showFrame&&(c=Math.round(a/9)),this.editor.changeViewZones(u=>{this._viewZone&&u.removeZone(this._viewZone.id),this._overlayWidget&&(this.editor.removeOverlayWidget(this._overlayWidget),this._overlayWidget=null),this.domNode.style.top="-1000px",this._viewZone=new Fye(r,i.lineNumber,i.column,t,g=>this._onViewZoneTop(g),g=>this._onViewZoneHeight(g),this.options.showInHiddenAreas,this.options.ordinal),this._viewZone.id=u.addZone(this._viewZone),this._overlayWidget=new Wye(Oye+this._viewZone.id,this.domNode),this.editor.addOverlayWidget(this._overlayWidget)}),this.container&&this.options.showFrame){const u=this.options.frameWidth?this.options.frameWidth:c;this.container.style.borderTopWidth=u+"px",this.container.style.borderBottomWidth=u+"px"}const d=t*a-this._decoratingElementsHeight();this.container&&(this.container.style.top=l+"px",this.container.style.height=d+"px",this.container.style.overflow="hidden"),this._doLayout(d,s),this.options.keepEditorSelection||this.editor.setSelection(e);const h=this.editor.getModel();if(h){const u=h.validateRange(new x(e.startLineNumber,1,e.endLineNumber+1,1));this.revealRange(u,u.startLineNumber===h.getLineCount())}}revealRange(e,t){t?this.editor.revealLineNearTop(e.endLineNumber,0):this.editor.revealRange(e,0)}setCssClass(e,t){this.container&&(t&&this.container.classList.remove(t),this.container.classList.add(e))}_onWidth(e){}_doLayout(e,t){}_relayout(e){this._viewZone&&this._viewZone.heightInLines!==e&&this.editor.changeViewZones(t=>{this._viewZone&&(this._viewZone.heightInLines=e,t.layoutZone(this._viewZone.id))})}_initSash(){if(this._resizeSash)return;this._resizeSash=this._disposables.add(new Mn(this.domNode,this,{orientation:1})),this.options.isResizeable||(this._resizeSash.state=0);let e;this._disposables.add(this._resizeSash.onDidStart(t=>{this._viewZone&&(e={startY:t.startY,heightInLines:this._viewZone.heightInLines})})),this._disposables.add(this._resizeSash.onDidEnd(()=>{e=void 0})),this._disposables.add(this._resizeSash.onDidChange(t=>{if(e){const i=(t.currentY-e.startY)/this.editor.getOption(67),n=i<0?Math.ceil(i):Math.floor(i),s=e.heightInLines+n;s>5&&s<35&&this._relayout(s)}}))}getHorizontalSashLeft(){return 0}getHorizontalSashTop(){return(this.domNode.style.height===null?0:parseInt(this.domNode.style.height))-this._decoratingElementsHeight()/2}getHorizontalSashWidth(){const e=this.editor.getLayoutInfo();return e.width-e.minimap.minimapWidth}}var MZ=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},AZ=function(o,e){return function(t,i){e(t,i,o)}};const PZ=et("IPeekViewService");st(PZ,class{constructor(){this._widgets=new Map}addExclusiveWidget(o,e){const t=this._widgets.get(o);t&&(t.listener.dispose(),t.widget.dispose());const i=()=>{const n=this._widgets.get(o);n&&n.widget===e&&(n.listener.dispose(),this._widgets.delete(o))};this._widgets.set(o,{widget:e,listener:e.onDidClose(i)})}},1);var Ws;(function(o){o.inPeekEditor=new le("inReferenceSearchEditor",!0,m("inReferenceSearchEditor","Whether the current code editor is embedded inside peek")),o.notInPeekEditor=o.inPeekEditor.toNegated()})(Ws||(Ws={}));var m_;let Ox=(m_=class{constructor(e,t){e instanceof Jr&&Ws.inPeekEditor.bindTo(t)}dispose(){}},m_.ID="editor.contrib.referenceController",m_);Ox=MZ([AZ(1,Ee)],Ox);ct(Ox.ID,Ox,0);function Hye(o){const e=o.get(pt).getFocusedCodeEditor();return e instanceof Jr?e.getParentEditor():e}const Vye={headerBackgroundColor:Z.white,primaryHeadingColor:Z.fromHex("#333333"),secondaryHeadingColor:Z.fromHex("#6c6c6cb3")};let Fx=class extends Bye{constructor(e,t,i){super(e,t),this.instantiationService=i,this._onDidClose=new P,this.onDidClose=this._onDidClose.event,dI(this.options,Vye,!1)}dispose(){this.disposed||(this.disposed=!0,super.dispose(),this._onDidClose.fire(this))}style(e){const t=this.options;e.headerBackgroundColor&&(t.headerBackgroundColor=e.headerBackgroundColor),e.primaryHeadingColor&&(t.primaryHeadingColor=e.primaryHeadingColor),e.secondaryHeadingColor&&(t.secondaryHeadingColor=e.secondaryHeadingColor),super.style(e)}_applyStyles(){super._applyStyles();const e=this.options;this._headElement&&e.headerBackgroundColor&&(this._headElement.style.backgroundColor=e.headerBackgroundColor.toString()),this._primaryHeading&&e.primaryHeadingColor&&(this._primaryHeading.style.color=e.primaryHeadingColor.toString()),this._secondaryHeading&&e.secondaryHeadingColor&&(this._secondaryHeading.style.color=e.secondaryHeadingColor.toString()),this._bodyElement&&e.frameColor&&(this._bodyElement.style.borderColor=e.frameColor.toString())}_fillContainer(e){this.setCssClass("peekview-widget"),this._headElement=ne(".head"),this._bodyElement=ne(".body"),this._fillHead(this._headElement),this._fillBody(this._bodyElement),e.appendChild(this._headElement),e.appendChild(this._bodyElement)}_fillHead(e,t){this._titleElement=ne(".peekview-title"),this.options.supportOnTitleClick&&(this._titleElement.classList.add("clickable"),hi(this._titleElement,"click",s=>this._onTitleClick(s))),X(this._headElement,this._titleElement),this._fillTitleIcon(this._titleElement),this._primaryHeading=ne("span.filename"),this._secondaryHeading=ne("span.dirname"),this._metaHeading=ne("span.meta"),X(this._titleElement,this._primaryHeading,this._secondaryHeading,this._metaHeading);const i=ne(".peekview-actions");X(this._headElement,i);const n=this._getActionBarOptions();this._actionbarWidget=new zo(i,n),this._disposables.add(this._actionbarWidget),t||this._actionbarWidget.push(new ro("peekview.close",m("label.close","Close"),Ie.asClassName(te.close),!0,()=>(this.dispose(),Promise.resolve())),{label:!1,icon:!0})}_fillTitleIcon(e){}_getActionBarOptions(){return{actionViewItemProvider:Fj.bind(void 0,this.instantiationService),orientation:0}}_onTitleClick(e){}setTitle(e,t){this._primaryHeading&&this._secondaryHeading&&(this._primaryHeading.innerText=e,this._primaryHeading.setAttribute("title",e),t?this._secondaryHeading.innerText=t:pn(this._secondaryHeading))}setMetaTitle(e){this._metaHeading&&(e?(this._metaHeading.innerText=e,to(this._metaHeading)):rs(this._metaHeading))}_doLayout(e,t){if(!this._isShowing&&e<0){this.dispose();return}const i=Math.ceil(this.editor.getOption(67)*1.2),n=Math.round(e-(i+2));this._doLayoutHead(i,t),this._doLayoutBody(n,t)}_doLayoutHead(e,t){this._headElement&&(this._headElement.style.height=`${e}px`,this._headElement.style.lineHeight=this._headElement.style.height)}_doLayoutBody(e,t){this._bodyElement&&(this._bodyElement.style.height=`${e}px`)}};Fx=MZ([AZ(2,we)],Fx);const zye=E("peekViewTitle.background",{dark:"#252526",light:"#F3F3F3",hcDark:Z.black,hcLight:Z.white},m("peekViewTitleBackground","Background color of the peek view title area.")),OZ=E("peekViewTitleLabel.foreground",{dark:Z.white,light:Z.black,hcDark:Z.white,hcLight:Zl},m("peekViewTitleForeground","Color of the peek view title.")),FZ=E("peekViewTitleDescription.foreground",{dark:"#ccccccb3",light:"#616161",hcDark:"#FFFFFF99",hcLight:"#292929"},m("peekViewTitleInfoForeground","Color of the peek view title info.")),Uye=E("peekView.border",{dark:Bl,light:Bl,hcDark:lt,hcLight:lt},m("peekViewBorder","Color of the peek view borders and arrow.")),$ye=E("peekViewResult.background",{dark:"#252526",light:"#F3F3F3",hcDark:Z.black,hcLight:Z.white},m("peekViewResultsBackground","Background color of the peek view result list."));E("peekViewResult.lineForeground",{dark:"#bbbbbb",light:"#646465",hcDark:Z.white,hcLight:Zl},m("peekViewResultsMatchForeground","Foreground color for line nodes in the peek view result list."));E("peekViewResult.fileForeground",{dark:Z.white,light:"#1E1E1E",hcDark:Z.white,hcLight:Zl},m("peekViewResultsFileForeground","Foreground color for file nodes in the peek view result list."));E("peekViewResult.selectionBackground",{dark:"#3399ff33",light:"#3399ff33",hcDark:null,hcLight:null},m("peekViewResultsSelectionBackground","Background color of the selected entry in the peek view result list."));E("peekViewResult.selectionForeground",{dark:Z.white,light:"#6C6C6C",hcDark:Z.white,hcLight:Zl},m("peekViewResultsSelectionForeground","Foreground color of the selected entry in the peek view result list."));const WZ=E("peekViewEditor.background",{dark:"#001F33",light:"#F2F8FC",hcDark:Z.black,hcLight:Z.white},m("peekViewEditorBackground","Background color of the peek view editor."));E("peekViewEditorGutter.background",WZ,m("peekViewEditorGutterBackground","Background color of the gutter in the peek view editor."));E("peekViewEditorStickyScroll.background",WZ,m("peekViewEditorStickScrollBackground","Background color of sticky scroll in the peek view editor."));E("peekViewResult.matchHighlightBackground",{dark:"#ea5c004d",light:"#ea5c004d",hcDark:null,hcLight:null},m("peekViewResultsMatchHighlight","Match highlight color in the peek view result list."));E("peekViewEditor.matchHighlightBackground",{dark:"#ff8f0099",light:"#f5d802de",hcDark:null,hcLight:null},m("peekViewEditorMatchHighlight","Match highlight color in the peek view editor."));E("peekViewEditor.matchHighlightBorder",{dark:null,light:null,hcDark:Xt,hcLight:Xt},m("peekViewEditorMatchHighlightBorder","Match highlight border in the peek view editor."));class Xh{constructor(e,t,i,n){this.isProviderFirst=e,this.parent=t,this.link=i,this._rangeCallback=n,this.id=v2.nextId()}get uri(){return this.link.uri}get range(){return this._range??this.link.targetSelectionRange??this.link.range}set range(e){this._range=e,this._rangeCallback(this)}get ariaMessage(){var t;const e=(t=this.parent.getPreview(this))==null?void 0:t.preview(this.range);return e?m({key:"aria.oneReference.preview",comment:["Placeholders are: 0: filename, 1:line number, 2: column number, 3: preview snippet of source code"]},"{0} in {1} on line {2} at column {3}",e.value,vr(this.uri),this.range.startLineNumber,this.range.startColumn):m("aria.oneReference","in {0} on line {1} at column {2}",vr(this.uri),this.range.startLineNumber,this.range.startColumn)}}class jye{constructor(e){this._modelReference=e}dispose(){this._modelReference.dispose()}preview(e,t=8){const i=this._modelReference.object.textEditorModel;if(!i)return;const{startLineNumber:n,startColumn:s,endLineNumber:r,endColumn:a}=e,l=i.getWordUntilPosition({lineNumber:n,column:s-t}),c=new x(n,l.startColumn,n,s),d=new x(r,a,r,1073741824),h=i.getValueInRange(c).replace(/^\s+/,""),u=i.getValueInRange(e),g=i.getValueInRange(d).replace(/\s+$/,"");return{value:h+u+g,highlight:{start:h.length,end:h.length+u.length}}}}class h1{constructor(e,t){this.parent=e,this.uri=t,this.children=[],this._previews=new an}dispose(){Tt(this._previews.values()),this._previews.clear()}getPreview(e){return this._previews.get(e.uri)}get ariaMessage(){const e=this.children.length;return e===1?m("aria.fileReferences.1","1 symbol in {0}, full path {1}",vr(this.uri),this.uri.fsPath):m("aria.fileReferences.N","{0} symbols in {1}, full path {2}",e,vr(this.uri),this.uri.fsPath)}async resolve(e){if(this._previews.size!==0)return this;for(const t of this.children)if(!this._previews.has(t.uri))try{const i=await e.createModelReference(t.uri);this._previews.set(t.uri,new jye(i))}catch(i){Oe(i)}return this}}class co{constructor(e,t){this.groups=[],this.references=[],this._onDidChangeReferenceRange=new P,this.onDidChangeReferenceRange=this._onDidChangeReferenceRange.event,this._links=e,this._title=t;const[i]=e;e.sort(co._compareReferences);let n;for(const s of e)if((!n||!Qt.isEqual(n.uri,s.uri,!0))&&(n=new h1(this,s.uri),this.groups.push(n)),n.children.length===0||co._compareReferences(s,n.children[n.children.length-1])!==0){const r=new Xh(i===s,n,s,a=>this._onDidChangeReferenceRange.fire(a));this.references.push(r),n.children.push(r)}}dispose(){Tt(this.groups),this._onDidChangeReferenceRange.dispose(),this.groups.length=0}clone(){return new co(this._links,this._title)}get title(){return this._title}get isEmpty(){return this.groups.length===0}get ariaMessage(){return this.isEmpty?m("aria.result.0","No results found"):this.references.length===1?m("aria.result.1","Found 1 symbol in {0}",this.references[0].uri.fsPath):this.groups.length===1?m("aria.result.n1","Found {0} symbols in {1}",this.references.length,this.groups[0].uri.fsPath):m("aria.result.nm","Found {0} symbols in {1} files",this.references.length,this.groups.length)}nextOrPreviousReference(e,t){const{parent:i}=e;let n=i.children.indexOf(e);const s=i.children.length,r=i.parent.groups.length;return r===1||t&&n+1<s||!t&&n>0?(t?n=(n+1)%s:n=(n+s-1)%s,i.children[n]):(n=i.parent.groups.indexOf(i),t?(n=(n+1)%r,i.parent.groups[n].children[0]):(n=(n+r-1)%r,i.parent.groups[n].children[i.parent.groups[n].children.length-1]))}nearestReference(e,t){const i=this.references.map((n,s)=>({idx:s,prefixLen:Oh(n.uri.toString(),e.toString()),offsetDist:Math.abs(n.range.startLineNumber-t.lineNumber)*100+Math.abs(n.range.startColumn-t.column)})).sort((n,s)=>n.prefixLen>s.prefixLen?-1:n.prefixLen<s.prefixLen?1:n.offsetDist<s.offsetDist?-1:n.offsetDist>s.offsetDist?1:0)[0];if(i)return this.references[i.idx]}referenceAt(e,t){for(const i of this.references)if(i.uri.toString()===e.toString()&&x.containsPosition(i.range,t))return i}firstReference(){for(const e of this.references)if(e.isProviderFirst)return e;return this.references[0]}static _compareReferences(e,t){return Qt.compare(e.uri,t.uri)||x.compareRangesUsingStarts(e.range,t.range)}}var PE=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},OE=function(o,e){return function(t,i){e(t,i,o)}},SO;let yO=class{constructor(e){this._resolverService=e}hasChildren(e){return e instanceof co||e instanceof h1}getChildren(e){if(e instanceof co)return e.groups;if(e instanceof h1)return e.resolve(this._resolverService).then(t=>t.children);throw new Error("bad tree")}};yO=PE([OE(0,Hs)],yO);class Kye{getHeight(){return 23}getTemplateId(e){return e instanceof h1?Wx.id:Bx.id}}let LO=class{constructor(e){this._keybindingService=e}getKeyboardNavigationLabel(e){var t;if(e instanceof Xh){const i=(t=e.parent.getPreview(e))==null?void 0:t.preview(e.range);if(i)return i.value}return vr(e.uri)}};LO=PE([OE(0,yt)],LO);class qye{getId(e){return e instanceof Xh?e.id:e.uri}}let xO=class extends F{constructor(e,t){super(),this._labelService=t;const i=document.createElement("div");i.classList.add("reference-file"),this.file=this._register(new qL(i,{supportHighlights:!0})),this.badge=new CA(X(i,ne(".count")),{},Pj),e.appendChild(i)}set(e,t){const i=GI(e.uri);this.file.setLabel(this._labelService.getUriBasenameLabel(e.uri),this._labelService.getUriLabel(i,{relative:!0}),{title:this._labelService.getUriLabel(e.uri),matches:t});const n=e.children.length;this.badge.setCount(n),n>1?this.badge.setTitleFormat(m("referencesCount","{0} references",n)):this.badge.setTitleFormat(m("referenceCount","{0} reference",n))}};xO=PE([OE(1,W_)],xO);var Hg;let Wx=(Hg=class{constructor(e){this._instantiationService=e,this.templateId=SO.id}renderTemplate(e){return this._instantiationService.createInstance(xO,e)}renderElement(e,t,i){i.set(e.element,U1(e.filterData))}disposeTemplate(e){e.dispose()}},SO=Hg,Hg.id="FileReferencesRenderer",Hg);Wx=SO=PE([OE(0,we)],Wx);class Gye extends F{constructor(e){super(),this.label=this._register(new gh(e))}set(e,t){var n;const i=(n=e.parent.getPreview(e))==null?void 0:n.preview(e.range);if(!i||!i.value)this.label.set(`${vr(e.uri)}:${e.range.startLineNumber+1}:${e.range.startColumn+1}`);else{const{value:s,highlight:r}=i;t&&!Ma.isDefault(t)?(this.label.element.classList.toggle("referenceMatch",!1),this.label.set(s,U1(t))):(this.label.element.classList.toggle("referenceMatch",!0),this.label.set(s,[r]))}}}const LD=class LD{constructor(){this.templateId=LD.id}renderTemplate(e){return new Gye(e)}renderElement(e,t,i){i.set(e.element,e.filterData)}disposeTemplate(e){e.dispose()}};LD.id="OneReferenceRenderer";let Bx=LD;class Zye{getWidgetAriaLabel(){return m("treeAriaLabel","References")}getAriaLabel(e){return e.ariaMessage}}var Yye=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Mm=function(o,e){return function(t,i){e(t,i,o)}};const xD=class xD{constructor(e,t){this._editor=e,this._model=t,this._decorations=new Map,this._decorationIgnoreSet=new Set,this._callOnDispose=new K,this._callOnModelChange=new K,this._callOnDispose.add(this._editor.onDidChangeModel(()=>this._onModelChanged())),this._onModelChanged()}dispose(){this._callOnModelChange.dispose(),this._callOnDispose.dispose(),this.removeDecorations()}_onModelChanged(){this._callOnModelChange.clear();const e=this._editor.getModel();if(e){for(const t of this._model.references)if(t.uri.toString()===e.uri.toString()){this._addDecorations(t.parent);return}}}_addDecorations(e){if(!this._editor.hasModel())return;this._callOnModelChange.add(this._editor.getModel().onDidChangeDecorations(()=>this._onDecorationChanged()));const t=[],i=[];for(let n=0,s=e.children.length;n<s;n++){const r=e.children[n];this._decorationIgnoreSet.has(r.id)||r.uri.toString()===this._editor.getModel().uri.toString()&&(t.push({range:r.range,options:xD.DecorationOptions}),i.push(n))}this._editor.changeDecorations(n=>{const s=n.deltaDecorations([],t);for(let r=0;r<s.length;r++)this._decorations.set(s[r],e.children[i[r]])})}_onDecorationChanged(){const e=[],t=this._editor.getModel();if(t){for(const[i,n]of this._decorations){const s=t.getDecorationRange(i);if(!s)continue;let r=!1;if(!x.equalsRange(s,n.range)){if(x.spansMultipleLines(s))r=!0;else{const a=n.range.endColumn-n.range.startColumn,l=s.endColumn-s.startColumn;a!==l&&(r=!0)}r?(this._decorationIgnoreSet.add(n.id),e.push(i)):n.range=s}}for(let i=0,n=e.length;i<n;i++)this._decorations.delete(e[i]);this._editor.removeDecorations(e)}}removeDecorations(){this._editor.removeDecorations([...this._decorations.keys()]),this._decorations.clear()}};xD.DecorationOptions=We.register({description:"reference-decoration",stickiness:1,className:"reference-decoration"});let kO=xD;class Xye{constructor(){this.ratio=.7,this.heightInLines=18}static fromJSON(e){let t,i;try{const n=JSON.parse(e);t=n.ratio,i=n.heightInLines}catch{}return{ratio:t||.7,heightInLines:i||18}}}class Qye extends DA{}let DO=class extends Fx{constructor(e,t,i,n,s,r,a,l,c){super(e,{showFrame:!1,showArrow:!0,isResizeable:!0,isAccessible:!0,supportOnTitleClick:!0},r),this._defaultTreeKeyboardSupport=t,this.layoutData=i,this._textModelResolverService=s,this._instantiationService=r,this._peekViewService=a,this._uriLabel=l,this._keybindingService=c,this._disposeOnNewModel=new K,this._callOnDispose=new K,this._onDidSelectReference=new P,this.onDidSelectReference=this._onDidSelectReference.event,this._dim=new kt(0,0),this._isClosing=!1,this._applyTheme(n.getColorTheme()),this._callOnDispose.add(n.onDidColorThemeChange(this._applyTheme.bind(this))),this._peekViewService.addExclusiveWidget(e,this),this.create()}get isClosing(){return this._isClosing}dispose(){this._isClosing=!0,this.setModel(void 0),this._callOnDispose.dispose(),this._disposeOnNewModel.dispose(),Tt(this._preview),Tt(this._previewNotAvailableMessage),Tt(this._tree),Tt(this._previewModelReference),this._splitView.dispose(),super.dispose()}_applyTheme(e){const t=e.getColor(Uye)||Z.transparent;this.style({arrowColor:t,frameColor:t,headerBackgroundColor:e.getColor(zye)||Z.transparent,primaryHeadingColor:e.getColor(OZ),secondaryHeadingColor:e.getColor(FZ)})}show(e){super.show(e,this.layoutData.heightInLines||18)}focusOnReferenceTree(){this._tree.domFocus()}focusOnPreviewEditor(){this._preview.focus()}isPreviewEditorFocused(){return this._preview.hasTextFocus()}_onTitleClick(e){this._preview&&this._preview.getModel()&&this._onDidSelectReference.fire({element:this._getFocusedReference(),kind:e.ctrlKey||e.metaKey||e.altKey?"side":"open",source:"title"})}_fillBody(e){this.setCssClass("reference-zone-widget"),this._messageContainer=X(e,ne("div.messages")),rs(this._messageContainer),this._splitView=new DK(e,{orientation:1}),this._previewContainer=X(e,ne("div.preview.inline"));const t={scrollBeyondLastLine:!1,scrollbar:{verticalScrollbarSize:14,horizontal:"auto",useShadows:!0,verticalHasArrows:!1,horizontalHasArrows:!1,alwaysConsumeMouseWheel:!0},overviewRulerLanes:2,fixedOverflowWidgets:!0,minimap:{enabled:!1}};this._preview=this._instantiationService.createInstance(Jr,this._previewContainer,t,{},this.editor),rs(this._previewContainer),this._previewNotAvailableMessage=this._instantiationService.createInstance(Da,m("missingPreviewMessage","no preview available"),ms,Da.DEFAULT_CREATION_OPTIONS,null),this._treeContainer=X(e,ne("div.ref-tree.inline"));const i={keyboardSupport:this._defaultTreeKeyboardSupport,accessibilityProvider:new Zye,keyboardNavigationLabelProvider:this._instantiationService.createInstance(LO),identityProvider:new qye,openOnSingleClick:!0,selectionNavigation:!0,overrideStyles:{listBackground:$ye}};this._defaultTreeKeyboardSupport&&this._callOnDispose.add(hi(this._treeContainer,"keydown",s=>{s.equals(9)&&(this._keybindingService.dispatchEvent(s,s.target),s.stopPropagation())},!0)),this._tree=this._instantiationService.createInstance(Qye,"ReferencesWidget",this._treeContainer,new Kye,[this._instantiationService.createInstance(Wx),this._instantiationService.createInstance(Bx)],this._instantiationService.createInstance(yO),i),this._splitView.addView({onDidChange:oe.None,element:this._previewContainer,minimumSize:200,maximumSize:Number.MAX_VALUE,layout:s=>{this._preview.layout({height:this._dim.height,width:s})}},HL.Distribute),this._splitView.addView({onDidChange:oe.None,element:this._treeContainer,minimumSize:100,maximumSize:Number.MAX_VALUE,layout:s=>{this._treeContainer.style.height=`${this._dim.height}px`,this._treeContainer.style.width=`${s}px`,this._tree.layout(this._dim.height,s)}},HL.Distribute),this._disposables.add(this._splitView.onDidSashChange(()=>{this._dim.width&&(this.layoutData.ratio=this._splitView.getViewSize(0)/this._dim.width)},void 0));const n=(s,r)=>{s instanceof Xh&&(r==="show"&&this._revealReference(s,!1),this._onDidSelectReference.fire({element:s,kind:r,source:"tree"}))};this._disposables.add(this._tree.onDidOpen(s=>{s.sideBySide?n(s.element,"side"):s.editorOptions.pinned?n(s.element,"goto"):n(s.element,"show")})),rs(this._treeContainer)}_onWidth(e){this._dim&&this._doLayoutBody(this._dim.height,e)}_doLayoutBody(e,t){super._doLayoutBody(e,t),this._dim=new kt(t,e),this.layoutData.heightInLines=this._viewZone?this._viewZone.heightInLines:this.layoutData.heightInLines,this._splitView.layout(t),this._splitView.resizeView(0,t*this.layoutData.ratio)}setSelection(e){return this._revealReference(e,!0).then(()=>{this._model&&(this._tree.setSelection([e]),this._tree.setFocus([e]))})}setModel(e){return this._disposeOnNewModel.clear(),this._model=e,this._model?this._onNewModel():Promise.resolve()}_onNewModel(){return this._model?this._model.isEmpty?(this.setTitle(""),this._messageContainer.innerText=m("noResults","No results"),to(this._messageContainer),Promise.resolve(void 0)):(rs(this._messageContainer),this._decorationsManager=new kO(this._preview,this._model),this._disposeOnNewModel.add(this._decorationsManager),this._disposeOnNewModel.add(this._model.onDidChangeReferenceRange(e=>this._tree.rerender(e))),this._disposeOnNewModel.add(this._preview.onMouseDown(e=>{const{event:t,target:i}=e;if(t.detail!==2)return;const n=this._getFocusedReference();n&&this._onDidSelectReference.fire({element:{uri:n.uri,range:i.range},kind:t.ctrlKey||t.metaKey||t.altKey?"side":"open",source:"editor"})})),this.container.classList.add("results-loaded"),to(this._treeContainer),to(this._previewContainer),this._splitView.layout(this._dim.width),this.focusOnReferenceTree(),this._tree.setInput(this._model.groups.length===1?this._model.groups[0]:this._model)):Promise.resolve(void 0)}_getFocusedReference(){const[e]=this._tree.getFocus();if(e instanceof Xh)return e;if(e instanceof h1&&e.children.length>0)return e.children[0]}async revealReference(e){await this._revealReference(e,!1),this._onDidSelectReference.fire({element:e,kind:"goto",source:"tree"})}async _revealReference(e,t){if(this._revealedReference===e)return;this._revealedReference=e,e.uri.scheme!==Fe.inMemory?this.setTitle(rle(e.uri),this._uriLabel.getUriLabel(GI(e.uri))):this.setTitle(m("peekView.alternateTitle","References"));const i=this._textModelResolverService.createModelReference(e.uri);this._tree.getInput()===e.parent?this._tree.reveal(e):(t&&this._tree.reveal(e.parent),await this._tree.expand(e.parent),this._tree.reveal(e));const n=await i;if(!this._model){n.dispose();return}Tt(this._previewModelReference);const s=n.object;if(s){const r=this._preview.getModel()===s.textEditorModel?0:1,a=x.lift(e.range).collapseToStart();this._previewModelReference=n,this._preview.setModel(s.textEditorModel),this._preview.setSelection(a),this._preview.revealRangeInCenter(a,r)}else this._preview.setModel(this._previewNotAvailableMessage),n.dispose()}};DO=Yye([Mm(3,ji),Mm(4,Hs),Mm(5,we),Mm(6,PZ),Mm(7,W_),Mm(8,yt)],DO);var Jye=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Am=function(o,e){return function(t,i){e(t,i,o)}},py;const gm=new le("referenceSearchVisible",!1,m("referenceSearchVisible","Whether reference peek is visible, like 'Peek References' or 'Peek Definition'"));var Vg;let qf=(Vg=class{static get(e){return e.getContribution(py.ID)}constructor(e,t,i,n,s,r,a,l){this._defaultTreeKeyboardSupport=e,this._editor=t,this._editorService=n,this._notificationService=s,this._instantiationService=r,this._storageService=a,this._configurationService=l,this._disposables=new K,this._requestIdPool=0,this._ignoreModelChangeEvent=!1,this._referenceSearchVisible=gm.bindTo(i)}dispose(){var e,t;this._referenceSearchVisible.reset(),this._disposables.dispose(),(e=this._widget)==null||e.dispose(),(t=this._model)==null||t.dispose(),this._widget=void 0,this._model=void 0}toggleWidget(e,t,i){let n;if(this._widget&&(n=this._widget.position),this.closeWidget(),n&&e.containsPosition(n))return;this._peekMode=i,this._referenceSearchVisible.set(!0),this._disposables.add(this._editor.onDidChangeModelLanguage(()=>{this.closeWidget()})),this._disposables.add(this._editor.onDidChangeModel(()=>{this._ignoreModelChangeEvent||this.closeWidget()}));const s="peekViewLayout",r=Xye.fromJSON(this._storageService.get(s,0,"{}"));this._widget=this._instantiationService.createInstance(DO,this._editor,this._defaultTreeKeyboardSupport,r),this._widget.setTitle(m("labelLoading","Loading...")),this._widget.show(e),this._disposables.add(this._widget.onDidClose(()=>{t.cancel(),this._widget?(this._storageService.store(s,JSON.stringify(this._widget.layoutData),0,1),this._widget.isClosing||this.closeWidget(),this._widget=void 0):this.closeWidget()})),this._disposables.add(this._widget.onDidSelectReference(l=>{const{element:c,kind:d}=l;if(c)switch(d){case"open":(l.source!=="editor"||!this._configurationService.getValue("editor.stablePeek"))&&this.openReference(c,!1,!1);break;case"side":this.openReference(c,!0,!1);break;case"goto":i?this._gotoReference(c,!0):this.openReference(c,!1,!0);break}}));const a=++this._requestIdPool;t.then(l=>{var c;if(a!==this._requestIdPool||!this._widget){l.dispose();return}return(c=this._model)==null||c.dispose(),this._model=l,this._widget.setModel(this._model).then(()=>{if(this._widget&&this._model&&this._editor.hasModel()){this._model.isEmpty?this._widget.setMetaTitle(""):this._widget.setMetaTitle(m("metaTitle.N","{0} ({1})",this._model.title,this._model.references.length));const d=this._editor.getModel().uri,h=new A(e.startLineNumber,e.startColumn),u=this._model.nearestReference(d,h);if(u)return this._widget.setSelection(u).then(()=>{this._widget&&this._editor.getOption(87)==="editor"&&this._widget.focusOnPreviewEditor()})}})},l=>{this._notificationService.error(l)})}changeFocusBetweenPreviewAndReferences(){this._widget&&(this._widget.isPreviewEditorFocused()?this._widget.focusOnReferenceTree():this._widget.focusOnPreviewEditor())}async goToNextOrPreviousReference(e){if(!this._editor.hasModel()||!this._model||!this._widget)return;const t=this._widget.position;if(!t)return;const i=this._model.nearestReference(this._editor.getModel().uri,t);if(!i)return;const n=this._model.nextOrPreviousReference(i,e),s=this._editor.hasTextFocus(),r=this._widget.isPreviewEditorFocused();await this._widget.setSelection(n),await this._gotoReference(n,!1),s?this._editor.focus():this._widget&&r&&this._widget.focusOnPreviewEditor()}async revealReference(e){!this._editor.hasModel()||!this._model||!this._widget||await this._widget.revealReference(e)}closeWidget(e=!0){var t,i;(t=this._widget)==null||t.dispose(),(i=this._model)==null||i.dispose(),this._referenceSearchVisible.reset(),this._disposables.clear(),this._widget=void 0,this._model=void 0,e&&this._editor.focus(),this._requestIdPool+=1}_gotoReference(e,t){var n;(n=this._widget)==null||n.hide(),this._ignoreModelChangeEvent=!0;const i=x.lift(e.range).collapseToStart();return this._editorService.openCodeEditor({resource:e.uri,options:{selection:i,selectionSource:"code.jump",pinned:t}},this._editor).then(s=>{if(this._ignoreModelChangeEvent=!1,!s||!this._widget){this.closeWidget();return}if(this._editor===s)this._widget.show(i),this._widget.focusOnReferenceTree();else{const r=py.get(s),a=this._model.clone();this.closeWidget(),s.focus(),r==null||r.toggleWidget(i,Yi(l=>Promise.resolve(a)),this._peekMode??!1)}},s=>{this._ignoreModelChangeEvent=!1,Oe(s)})}openReference(e,t,i){t||this.closeWidget();const{uri:n,range:s}=e;this._editorService.openCodeEditor({resource:n,options:{selection:s,selectionSource:"code.jump",pinned:i}},this._editor,t)}},py=Vg,Vg.ID="editor.contrib.referencesController",Vg);qf=py=Jye([Am(2,Ee),Am(3,pt),Am(4,Ei),Am(5,we),Am(6,ia),Am(7,qe)],qf);function fm(o,e){const t=Hye(o);if(!t)return;const i=qf.get(t);i&&e(i)}bs.registerCommandAndKeybindingRule({id:"togglePeekWidgetFocus",weight:100,primary:Hi(2089,60),when:G.or(gm,Ws.inPeekEditor),handler(o){fm(o,e=>{e.changeFocusBetweenPreviewAndReferences()})}});bs.registerCommandAndKeybindingRule({id:"goToNextReference",weight:90,primary:62,secondary:[70],when:G.or(gm,Ws.inPeekEditor),handler(o){fm(o,e=>{e.goToNextOrPreviousReference(!0)})}});bs.registerCommandAndKeybindingRule({id:"goToPreviousReference",weight:90,primary:1086,secondary:[1094],when:G.or(gm,Ws.inPeekEditor),handler(o){fm(o,e=>{e.goToNextOrPreviousReference(!1)})}});rt.registerCommandAlias("goToNextReferenceFromEmbeddedEditor","goToNextReference");rt.registerCommandAlias("goToPreviousReferenceFromEmbeddedEditor","goToPreviousReference");rt.registerCommandAlias("closeReferenceSearchEditor","closeReferenceSearch");rt.registerCommand("closeReferenceSearch",o=>fm(o,e=>e.closeWidget()));bs.registerKeybindingRule({id:"closeReferenceSearch",weight:-1,primary:9,secondary:[1033],when:G.and(Ws.inPeekEditor,G.not("config.editor.stablePeek"))});bs.registerKeybindingRule({id:"closeReferenceSearch",weight:250,primary:9,secondary:[1033],when:G.and(gm,G.not("config.editor.stablePeek"),G.or(I.editorTextFocus,FK.negate()))});bs.registerCommandAndKeybindingRule({id:"revealReference",weight:200,primary:3,mac:{primary:3,secondary:[2066]},when:G.and(gm,HK,N3.negate(),T3.negate()),handler(o){var i;const t=(i=o.get(kr).lastFocusedList)==null?void 0:i.getFocus();Array.isArray(t)&&t[0]instanceof Xh&&fm(o,n=>n.revealReference(t[0]))}});bs.registerCommandAndKeybindingRule({id:"openReferenceToSide",weight:100,primary:2051,mac:{primary:259},when:G.and(gm,HK,N3.negate(),T3.negate()),handler(o){var i;const t=(i=o.get(kr).lastFocusedList)==null?void 0:i.getFocus();Array.isArray(t)&&t[0]instanceof Xh&&fm(o,n=>n.openReference(t[0],!0,!0))}});rt.registerCommand("openReference",o=>{var i;const t=(i=o.get(kr).lastFocusedList)==null?void 0:i.getFocus();Array.isArray(t)&&t[0]instanceof Xh&&fm(o,n=>n.openReference(t[0],!1,!0))});var BZ=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},MC=function(o,e){return function(t,i){e(t,i,o)}};const VW=new le("hasSymbols",!1,m("hasSymbols","Whether there are symbol locations that can be navigated via keyboard-only.")),FE=et("ISymbolNavigationService");let IO=class{constructor(e,t,i,n){this._editorService=t,this._notificationService=i,this._keybindingService=n,this._currentModel=void 0,this._currentIdx=-1,this._ignoreEditorChange=!1,this._ctxHasSymbols=VW.bindTo(e)}reset(){var e,t;this._ctxHasSymbols.reset(),(e=this._currentState)==null||e.dispose(),(t=this._currentMessage)==null||t.dispose(),this._currentModel=void 0,this._currentIdx=-1}put(e){const t=e.parent.parent;if(t.references.length<=1){this.reset();return}this._currentModel=t,this._currentIdx=t.references.indexOf(e),this._ctxHasSymbols.set(!0),this._showMessage();const i=new EO(this._editorService),n=i.onDidChange(s=>{if(this._ignoreEditorChange)return;const r=this._editorService.getActiveCodeEditor();if(!r)return;const a=r.getModel(),l=r.getPosition();if(!a||!l)return;let c=!1,d=!1;for(const h of t.references)if(Lw(h.uri,a.uri))c=!0,d=d||x.containsPosition(h.range,l);else if(c)break;(!c||!d)&&this.reset()});this._currentState=jr(i,n)}revealNext(e){if(!this._currentModel)return Promise.resolve();this._currentIdx+=1,this._currentIdx%=this._currentModel.references.length;const t=this._currentModel.references[this._currentIdx];return this._showMessage(),this._ignoreEditorChange=!0,this._editorService.openCodeEditor({resource:t.uri,options:{selection:x.collapseToStart(t.range),selectionRevealType:3}},e).finally(()=>{this._ignoreEditorChange=!1})}_showMessage(){var i;(i=this._currentMessage)==null||i.dispose();const e=this._keybindingService.lookupKeybinding("editor.gotoNextSymbolFromResult"),t=e?m("location.kb","Symbol {0} of {1}, {2} for next",this._currentIdx+1,this._currentModel.references.length,e.getLabel()):m("location","Symbol {0} of {1}",this._currentIdx+1,this._currentModel.references.length);this._currentMessage=this._notificationService.status(t)}};IO=BZ([MC(0,Ee),MC(1,pt),MC(2,Ei),MC(3,yt)],IO);st(FE,IO,1);re(new class extends $i{constructor(){super({id:"editor.gotoNextSymbolFromResult",precondition:VW,kbOpts:{weight:100,primary:70}})}runEditorCommand(o,e){return o.get(FE).revealNext(e)}});bs.registerCommandAndKeybindingRule({id:"editor.gotoNextSymbolFromResult.cancel",weight:100,when:VW,primary:9,handler(o){o.get(FE).reset()}});let EO=class{constructor(e){this._listener=new Map,this._disposables=new K,this._onDidChange=new P,this.onDidChange=this._onDidChange.event,this._disposables.add(e.onCodeEditorRemove(this._onDidRemoveEditor,this)),this._disposables.add(e.onCodeEditorAdd(this._onDidAddEditor,this)),e.listCodeEditors().forEach(this._onDidAddEditor,this)}dispose(){this._disposables.dispose(),this._onDidChange.dispose(),Tt(this._listener.values())}_onDidAddEditor(e){this._listener.set(e,jr(e.onDidChangeCursorPosition(t=>this._onDidChange.fire({editor:e})),e.onDidChangeModelContent(t=>this._onDidChange.fire({editor:e}))))}_onDidRemoveEditor(e){var t;(t=this._listener.get(e))==null||t.dispose(),this._listener.delete(e)}};EO=BZ([MC(0,pt)],EO);function NO(o,e){return e.uri.scheme===o.uri.scheme?!0:!kM(e.uri,Fe.walkThroughSnippet,Fe.vscodeChatCodeBlock,Fe.vscodeChatCodeCompareBlock)}async function uS(o,e,t,i,n){const r=t.ordered(o,i).map(l=>Promise.resolve(n(l,o,e)).then(void 0,c=>{Si(c)})),a=await Promise.all(r);return Kr(a.flat()).filter(l=>NO(o,l))}function gS(o,e,t,i,n){return uS(e,t,o,i,(s,r,a)=>s.provideDefinition(r,a,n))}function zW(o,e,t,i,n){return uS(e,t,o,i,(s,r,a)=>s.provideDeclaration(r,a,n))}function UW(o,e,t,i,n){return uS(e,t,o,i,(s,r,a)=>s.provideImplementation(r,a,n))}function $W(o,e,t,i,n){return uS(e,t,o,i,(s,r,a)=>s.provideTypeDefinition(r,a,n))}function fS(o,e,t,i,n,s){return uS(e,t,o,n,async(r,a,l)=>{var h,u;const c=(h=await r.provideReferences(a,l,{includeDeclaration:!0},s))==null?void 0:h.filter(g=>NO(a,g));if(!i||!c||c.length!==2)return c;const d=(u=await r.provideReferences(a,l,{includeDeclaration:!1},s))==null?void 0:u.filter(g=>NO(a,g));return d&&d.length===1?d:c})}async function Jl(o){const e=await o(),t=new co(e,""),i=t.references.map(n=>n.link);return t.dispose(),i}Vs("_executeDefinitionProvider",(o,e,t)=>{const i=o.get(ge),n=gS(i.definitionProvider,e,t,!1,ze.None);return Jl(()=>n)});Vs("_executeDefinitionProvider_recursive",(o,e,t)=>{const i=o.get(ge),n=gS(i.definitionProvider,e,t,!0,ze.None);return Jl(()=>n)});Vs("_executeTypeDefinitionProvider",(o,e,t)=>{const i=o.get(ge),n=$W(i.typeDefinitionProvider,e,t,!1,ze.None);return Jl(()=>n)});Vs("_executeTypeDefinitionProvider_recursive",(o,e,t)=>{const i=o.get(ge),n=$W(i.typeDefinitionProvider,e,t,!0,ze.None);return Jl(()=>n)});Vs("_executeDeclarationProvider",(o,e,t)=>{const i=o.get(ge),n=zW(i.declarationProvider,e,t,!1,ze.None);return Jl(()=>n)});Vs("_executeDeclarationProvider_recursive",(o,e,t)=>{const i=o.get(ge),n=zW(i.declarationProvider,e,t,!0,ze.None);return Jl(()=>n)});Vs("_executeReferenceProvider",(o,e,t)=>{const i=o.get(ge),n=fS(i.referenceProvider,e,t,!1,!1,ze.None);return Jl(()=>n)});Vs("_executeReferenceProvider_recursive",(o,e,t)=>{const i=o.get(ge),n=fS(i.referenceProvider,e,t,!1,!0,ze.None);return Jl(()=>n)});Vs("_executeImplementationProvider",(o,e,t)=>{const i=o.get(ge),n=UW(i.implementationProvider,e,t,!1,ze.None);return Jl(()=>n)});Vs("_executeImplementationProvider_recursive",(o,e,t)=>{const i=o.get(ge),n=UW(i.implementationProvider,e,t,!0,ze.None);return Jl(()=>n)});rn.appendMenuItem(me.EditorContext,{submenu:me.EditorContextPeek,title:m("peek.submenu","Peek"),group:"navigation",order:100});class eb{static is(e){return!e||typeof e!="object"?!1:!!(e instanceof eb||A.isIPosition(e.position)&&e.model)}constructor(e,t){this.model=e,this.position=t}}const Ar=class Ar extends $a{static all(){return Ar._allSymbolNavigationCommands.values()}static _patchConfig(e){const t={...e,f1:!0};if(t.menu)for(const i of ot.wrap(t.menu))(i.id===me.EditorContext||i.id===me.EditorContextPeek)&&(i.when=G.and(e.precondition,i.when));return t}constructor(e,t){super(Ar._patchConfig(t)),this.configuration=e,Ar._allSymbolNavigationCommands.set(t.id,this)}runEditorCommand(e,t,i,n){if(!t.hasModel())return Promise.resolve(void 0);const s=e.get(Ei),r=e.get(pt),a=e.get(nu),l=e.get(FE),c=e.get(ge),d=e.get(we),h=t.getModel(),u=t.getPosition(),g=eb.is(i)?i:new eb(h,u),f=new Zh(t,5),p=F1(this._getLocationModel(c,g.model,g.position,f.token),f.token).then(async _=>{var v;if(!_||f.token.isCancellationRequested)return;Cs(_.ariaMessage);let b;if(_.referenceAt(h.uri,u)){const w=this._getAlternativeCommand(t);!Ar._activeAlternativeCommands.has(w)&&Ar._allSymbolNavigationCommands.has(w)&&(b=Ar._allSymbolNavigationCommands.get(w))}const C=_.references.length;if(C===0){if(!this.configuration.muteMessage){const w=h.getWordAtPosition(u);(v=lo.get(t))==null||v.showMessage(this._getNoResultFoundMessage(w),u)}}else if(C===1&&b)Ar._activeAlternativeCommands.add(this.desc.id),d.invokeFunction(w=>b.runEditorCommand(w,t,i,n).finally(()=>{Ar._activeAlternativeCommands.delete(this.desc.id)}));else return this._onResult(r,l,t,_,n)},_=>{s.error(_)}).finally(()=>{f.dispose()});return a.showWhile(p,250),p}async _onResult(e,t,i,n,s){const r=this._getGoToPreference(i);if(!(i instanceof Jr)&&(this.configuration.openInPeek||r==="peek"&&n.references.length>1))this._openInPeek(i,n,s);else{const a=n.firstReference(),l=n.references.length>1&&r==="gotoAndPeek",c=await this._openReference(i,e,a,this.configuration.openToSide,!l);l&&c?this._openInPeek(c,n,s):n.dispose(),r==="goto"&&t.put(a)}}async _openReference(e,t,i,n,s){let r;if(vee(i)&&(r=i.targetSelectionRange),r||(r=i.range),!r)return;const a=await t.openCodeEditor({resource:i.uri,options:{selection:x.collapseToStart(r),selectionRevealType:3,selectionSource:"code.jump"}},e,n);if(a){if(s){const l=a.getModel(),c=a.createDecorationsCollection([{range:r,options:{description:"symbol-navigate-action-highlight",className:"symbolHighlight"}}]);setTimeout(()=>{a.getModel()===l&&c.clear()},350)}return a}}_openInPeek(e,t,i){const n=qf.get(e);n&&e.hasModel()?n.toggleWidget(i??e.getSelection(),Yi(s=>Promise.resolve(t)),this.configuration.openInPeek):t.dispose()}};Ar._allSymbolNavigationCommands=new Map,Ar._activeAlternativeCommands=new Set;let ad=Ar;class mS extends ad{async _getLocationModel(e,t,i,n){return new co(await gS(e.definitionProvider,t,i,!1,n),m("def.title","Definitions"))}_getNoResultFoundMessage(e){return e&&e.word?m("noResultWord","No definition found for '{0}'",e.word):m("generic.noResults","No definition found")}_getAlternativeCommand(e){return e.getOption(58).alternativeDefinitionCommand}_getGoToPreference(e){return e.getOption(58).multipleDefinitions}}var wh;Ot((wh=class extends mS{constructor(){super({openToSide:!1,openInPeek:!1,muteMessage:!1},{id:wh.id,title:{...Te("actions.goToDecl.label","Go to Definition"),mnemonicTitle:m({key:"miGotoDefinition",comment:["&& denotes a mnemonic"]},"Go to &&Definition")},precondition:I.hasDefinitionProvider,keybinding:[{when:I.editorTextFocus,primary:70,weight:100},{when:G.and(I.editorTextFocus,PK),primary:2118,weight:100}],menu:[{id:me.EditorContext,group:"navigation",order:1.1},{id:me.MenubarGoMenu,precondition:null,group:"4_symbol_nav",order:2}]}),rt.registerCommandAlias("editor.action.goToDeclaration",wh.id)}},wh.id="editor.action.revealDefinition",wh));var Sh;Ot((Sh=class extends mS{constructor(){super({openToSide:!0,openInPeek:!1,muteMessage:!1},{id:Sh.id,title:Te("actions.goToDeclToSide.label","Open Definition to the Side"),precondition:G.and(I.hasDefinitionProvider,I.isInEmbeddedEditor.toNegated()),keybinding:[{when:I.editorTextFocus,primary:Hi(2089,70),weight:100},{when:G.and(I.editorTextFocus,PK),primary:Hi(2089,2118),weight:100}]}),rt.registerCommandAlias("editor.action.openDeclarationToTheSide",Sh.id)}},Sh.id="editor.action.revealDefinitionAside",Sh));var yh;Ot((yh=class extends mS{constructor(){super({openToSide:!1,openInPeek:!0,muteMessage:!1},{id:yh.id,title:Te("actions.previewDecl.label","Peek Definition"),precondition:G.and(I.hasDefinitionProvider,Ws.notInPeekEditor,I.isInEmbeddedEditor.toNegated()),keybinding:{when:I.editorTextFocus,primary:582,linux:{primary:3140},weight:100},menu:{id:me.EditorContextPeek,group:"peek",order:2}}),rt.registerCommandAlias("editor.action.previewDeclaration",yh.id)}},yh.id="editor.action.peekDefinition",yh));class HZ extends ad{async _getLocationModel(e,t,i,n){return new co(await zW(e.declarationProvider,t,i,!1,n),m("decl.title","Declarations"))}_getNoResultFoundMessage(e){return e&&e.word?m("decl.noResultWord","No declaration found for '{0}'",e.word):m("decl.generic.noResults","No declaration found")}_getAlternativeCommand(e){return e.getOption(58).alternativeDeclarationCommand}_getGoToPreference(e){return e.getOption(58).multipleDeclarations}}var zg;Ot((zg=class extends HZ{constructor(){super({openToSide:!1,openInPeek:!1,muteMessage:!1},{id:zg.id,title:{...Te("actions.goToDeclaration.label","Go to Declaration"),mnemonicTitle:m({key:"miGotoDeclaration",comment:["&& denotes a mnemonic"]},"Go to &&Declaration")},precondition:G.and(I.hasDeclarationProvider,I.isInEmbeddedEditor.toNegated()),menu:[{id:me.EditorContext,group:"navigation",order:1.3},{id:me.MenubarGoMenu,precondition:null,group:"4_symbol_nav",order:3}]})}_getNoResultFoundMessage(e){return e&&e.word?m("decl.noResultWord","No declaration found for '{0}'",e.word):m("decl.generic.noResults","No declaration found")}},zg.id="editor.action.revealDeclaration",zg));Ot(class extends HZ{constructor(){super({openToSide:!1,openInPeek:!0,muteMessage:!1},{id:"editor.action.peekDeclaration",title:Te("actions.peekDecl.label","Peek Declaration"),precondition:G.and(I.hasDeclarationProvider,Ws.notInPeekEditor,I.isInEmbeddedEditor.toNegated()),menu:{id:me.EditorContextPeek,group:"peek",order:3}})}});class VZ extends ad{async _getLocationModel(e,t,i,n){return new co(await $W(e.typeDefinitionProvider,t,i,!1,n),m("typedef.title","Type Definitions"))}_getNoResultFoundMessage(e){return e&&e.word?m("goToTypeDefinition.noResultWord","No type definition found for '{0}'",e.word):m("goToTypeDefinition.generic.noResults","No type definition found")}_getAlternativeCommand(e){return e.getOption(58).alternativeTypeDefinitionCommand}_getGoToPreference(e){return e.getOption(58).multipleTypeDefinitions}}var Ug;Ot((Ug=class extends VZ{constructor(){super({openToSide:!1,openInPeek:!1,muteMessage:!1},{id:Ug.ID,title:{...Te("actions.goToTypeDefinition.label","Go to Type Definition"),mnemonicTitle:m({key:"miGotoTypeDefinition",comment:["&& denotes a mnemonic"]},"Go to &&Type Definition")},precondition:I.hasTypeDefinitionProvider,keybinding:{when:I.editorTextFocus,primary:0,weight:100},menu:[{id:me.EditorContext,group:"navigation",order:1.4},{id:me.MenubarGoMenu,precondition:null,group:"4_symbol_nav",order:3}]})}},Ug.ID="editor.action.goToTypeDefinition",Ug));var $g;Ot(($g=class extends VZ{constructor(){super({openToSide:!1,openInPeek:!0,muteMessage:!1},{id:$g.ID,title:Te("actions.peekTypeDefinition.label","Peek Type Definition"),precondition:G.and(I.hasTypeDefinitionProvider,Ws.notInPeekEditor,I.isInEmbeddedEditor.toNegated()),menu:{id:me.EditorContextPeek,group:"peek",order:4}})}},$g.ID="editor.action.peekTypeDefinition",$g));class zZ extends ad{async _getLocationModel(e,t,i,n){return new co(await UW(e.implementationProvider,t,i,!1,n),m("impl.title","Implementations"))}_getNoResultFoundMessage(e){return e&&e.word?m("goToImplementation.noResultWord","No implementation found for '{0}'",e.word):m("goToImplementation.generic.noResults","No implementation found")}_getAlternativeCommand(e){return e.getOption(58).alternativeImplementationCommand}_getGoToPreference(e){return e.getOption(58).multipleImplementations}}var jg;Ot((jg=class extends zZ{constructor(){super({openToSide:!1,openInPeek:!1,muteMessage:!1},{id:jg.ID,title:{...Te("actions.goToImplementation.label","Go to Implementations"),mnemonicTitle:m({key:"miGotoImplementation",comment:["&& denotes a mnemonic"]},"Go to &&Implementations")},precondition:I.hasImplementationProvider,keybinding:{when:I.editorTextFocus,primary:2118,weight:100},menu:[{id:me.EditorContext,group:"navigation",order:1.45},{id:me.MenubarGoMenu,precondition:null,group:"4_symbol_nav",order:4}]})}},jg.ID="editor.action.goToImplementation",jg));var Kg;Ot((Kg=class extends zZ{constructor(){super({openToSide:!1,openInPeek:!0,muteMessage:!1},{id:Kg.ID,title:Te("actions.peekImplementation.label","Peek Implementations"),precondition:G.and(I.hasImplementationProvider,Ws.notInPeekEditor,I.isInEmbeddedEditor.toNegated()),keybinding:{when:I.editorTextFocus,primary:3142,weight:100},menu:{id:me.EditorContextPeek,group:"peek",order:5}})}},Kg.ID="editor.action.peekImplementation",Kg));class UZ extends ad{_getNoResultFoundMessage(e){return e?m("references.no","No references found for '{0}'",e.word):m("references.noGeneric","No references found")}_getAlternativeCommand(e){return e.getOption(58).alternativeReferenceCommand}_getGoToPreference(e){return e.getOption(58).multipleReferences}}Ot(class extends UZ{constructor(){super({openToSide:!1,openInPeek:!1,muteMessage:!1},{id:"editor.action.goToReferences",title:{...Te("goToReferences.label","Go to References"),mnemonicTitle:m({key:"miGotoReference",comment:["&& denotes a mnemonic"]},"Go to &&References")},precondition:G.and(I.hasReferenceProvider,Ws.notInPeekEditor,I.isInEmbeddedEditor.toNegated()),keybinding:{when:I.editorTextFocus,primary:1094,weight:100},menu:[{id:me.EditorContext,group:"navigation",order:1.45},{id:me.MenubarGoMenu,precondition:null,group:"4_symbol_nav",order:5}]})}async _getLocationModel(e,t,i,n){return new co(await fS(e.referenceProvider,t,i,!0,!1,n),m("ref.title","References"))}});Ot(class extends UZ{constructor(){super({openToSide:!1,openInPeek:!0,muteMessage:!1},{id:"editor.action.referenceSearch.trigger",title:Te("references.action.label","Peek References"),precondition:G.and(I.hasReferenceProvider,Ws.notInPeekEditor,I.isInEmbeddedEditor.toNegated()),menu:{id:me.EditorContextPeek,group:"peek",order:6}})}async _getLocationModel(e,t,i,n){return new co(await fS(e.referenceProvider,t,i,!1,!1,n),m("ref.title","References"))}});class eLe extends ad{constructor(e,t,i){super(e,{id:"editor.action.goToLocation",title:Te("label.generic","Go to Any Symbol"),precondition:G.and(Ws.notInPeekEditor,I.isInEmbeddedEditor.toNegated())}),this._references=t,this._gotoMultipleBehaviour=i}async _getLocationModel(e,t,i,n){return new co(this._references,m("generic.title","Locations"))}_getNoResultFoundMessage(e){return e&&m("generic.noResult","No results for '{0}'",e.word)||""}_getGoToPreference(e){return this._gotoMultipleBehaviour??e.getOption(58).multipleReferences}_getAlternativeCommand(){return""}}rt.registerCommand({id:"editor.action.goToLocations",metadata:{description:"Go to locations from a position in a file",args:[{name:"uri",description:"The text document in which to start",constraint:Se},{name:"position",description:"The position at which to start",constraint:A.isIPosition},{name:"locations",description:"An array of locations.",constraint:Array},{name:"multiple",description:"Define what to do when having multiple results, either `peek`, `gotoAndPeek`, or `goto`"},{name:"noResultsMessage",description:"Human readable message that shows when locations is empty."}]},handler:async(o,e,t,i,n,s,r)=>{mt(Se.isUri(e)),mt(A.isIPosition(t)),mt(Array.isArray(i)),mt(typeof n>"u"||typeof n=="string"),mt(typeof r>"u"||typeof r=="boolean");const a=o.get(pt),l=await a.openCodeEditor({resource:e},a.getFocusedCodeEditor());if(jl(l))return l.setPosition(t),l.revealPositionInCenterIfOutsideViewport(t,0),l.invokeWithinContext(c=>{const d=new class extends eLe{_getNoResultFoundMessage(h){return s||super._getNoResultFoundMessage(h)}}({muteMessage:!s,openInPeek:!!r,openToSide:!1},i,n);c.get(we).invokeFunction(d.run.bind(d),l)})}});rt.registerCommand({id:"editor.action.peekLocations",metadata:{description:"Peek locations from a position in a file",args:[{name:"uri",description:"The text document in which to start",constraint:Se},{name:"position",description:"The position at which to start",constraint:A.isIPosition},{name:"locations",description:"An array of locations.",constraint:Array},{name:"multiple",description:"Define what to do when having multiple results, either `peek`, `gotoAndPeek`, or `goto`"}]},handler:async(o,e,t,i,n)=>{o.get(Ht).executeCommand("editor.action.goToLocations",e,t,i,n,void 0,!0)}});rt.registerCommand({id:"editor.action.findReferences",handler:(o,e,t)=>{mt(Se.isUri(e)),mt(A.isIPosition(t));const i=o.get(ge),n=o.get(pt);return n.openCodeEditor({resource:e},n.getFocusedCodeEditor()).then(s=>{if(!jl(s)||!s.hasModel())return;const r=qf.get(s);if(!r)return;const a=Yi(c=>fS(i.referenceProvider,s.getModel(),A.lift(t),!1,!1,c).then(d=>new co(d,m("ref.title","References")))),l=new x(t.lineNumber,t.column,t.lineNumber,t.column);return Promise.resolve(r.toggleWidget(l,a,!1))})}});rt.registerCommandAlias("editor.action.showReferences","editor.action.peekLocations");async function tLe(o,e,t,i){const n=o.get(Hs),s=o.get(mo),r=o.get(Ht),a=o.get(we),l=o.get(Ei);if(await i.item.resolve(ze.None),!i.part.location)return;const c=i.part.location,d=[],h=new Set(rn.getMenuItems(me.EditorContext).map(g=>Up(g)?g.command.id:EE()));for(const g of ad.all())h.has(g.desc.id)&&d.push(new ro(g.desc.id,ao.label(g.desc,{renderShortTitle:!0}),void 0,!0,async()=>{const f=await n.createModelReference(c.uri);try{const p=new eb(f.object.textEditorModel,x.getStartPosition(c.range)),_=i.item.anchor.range;await a.invokeFunction(g.runEditorCommand.bind(g),e,p,_)}finally{f.dispose()}}));if(i.part.command){const{command:g}=i.part;d.push(new Qi),d.push(new ro(g.id,g.title,void 0,!0,async()=>{try{await r.executeCommand(g.id,...g.arguments??[])}catch(f){l.notify({severity:OI.Error,source:i.item.provider.displayName,message:f})}}))}const u=e.getOption(128);s.showContextMenu({domForShadowRoot:u?e.getDomNode()??void 0:void 0,getAnchor:()=>{const g=wi(t);return{x:g.left,y:g.top+g.height+8}},getActions:()=>d,onHide:()=>{e.focus()},autoSelectFirstItem:!0})}async function $Z(o,e,t,i){const s=await o.get(Hs).createModelReference(i.uri);await t.invokeWithinContext(async r=>{const a=e.hasSideBySideModifier,l=r.get(Ee),c=Ws.inPeekEditor.getValue(l),d=!a&&t.getOption(89)&&!c;return new mS({openToSide:a,openInPeek:d,muteMessage:!0},{title:{value:"",original:""},id:"",precondition:void 0}).run(r,new eb(s.object.textEditorModel,x.getStartPosition(i.range)),x.lift(i.range))}),s.dispose()}var iLe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Pm=function(o,e){return function(t,i){e(t,i,o)}},Gm;class Hx{constructor(){this._entries=new Gl(50)}get(e){const t=Hx._key(e);return this._entries.get(t)}set(e,t){const i=Hx._key(e);this._entries.set(i,t)}static _key(e){return`${e.uri.toString()}/${e.getVersionId()}`}}const jZ=et("IInlayHintsCache");st(jZ,Hx,1);class TO{constructor(e,t){this.item=e,this.index=t}get part(){const e=this.item.hint.label;return typeof e=="string"?{label:e}:e[this.index]}}class nLe{constructor(e,t){this.part=e,this.hasTriggerModifier=t}}var Uc;let u1=(Uc=class{static get(e){return e.getContribution(Gm.ID)??void 0}constructor(e,t,i,n,s,r,a){this._editor=e,this._languageFeaturesService=t,this._inlayHintsCache=n,this._commandService=s,this._notificationService=r,this._instaService=a,this._disposables=new K,this._sessionDisposables=new K,this._decorationsMetadata=new Map,this._ruleFactory=new sx(this._editor),this._activeRenderMode=0,this._debounceInfo=i.for(t.inlayHintsProvider,"InlayHint",{min:25}),this._disposables.add(t.inlayHintsProvider.onDidChange(()=>this._update())),this._disposables.add(e.onDidChangeModel(()=>this._update())),this._disposables.add(e.onDidChangeModelLanguage(()=>this._update())),this._disposables.add(e.onDidChangeConfiguration(l=>{l.hasChanged(142)&&this._update()})),this._update()}dispose(){this._sessionDisposables.dispose(),this._removeAllDecorations(),this._disposables.dispose()}_update(){this._sessionDisposables.clear(),this._removeAllDecorations();const e=this._editor.getOption(142);if(e.enabled==="off")return;const t=this._editor.getModel();if(!t||!this._languageFeaturesService.inlayHintsProvider.has(t))return;if(e.enabled==="on")this._activeRenderMode=0;else{let a,l;e.enabled==="onUnlessPressed"?(a=0,l=1):(a=1,l=0),this._activeRenderMode=a,this._sessionDisposables.add(Rc.getInstance().event(c=>{if(!this._editor.hasModel())return;const d=c.altKey&&c.ctrlKey&&!(c.shiftKey||c.metaKey)?l:a;if(d!==this._activeRenderMode){this._activeRenderMode=d;const h=this._editor.getModel(),u=this._copyInlayHintsWithCurrentAnchor(h);this._updateHintsDecorators([h.getFullModelRange()],u),r.schedule(0)}}))}const i=this._inlayHintsCache.get(t);i&&this._updateHintsDecorators([t.getFullModelRange()],i),this._sessionDisposables.add(ve(()=>{t.isDisposed()||this._cacheHintsForFastRestore(t)}));let n;const s=new Set,r=new It(async()=>{const a=Date.now();n==null||n.dispose(!0),n=new li;const l=t.onWillDispose(()=>n==null?void 0:n.cancel());try{const c=n.token,d=await Px.create(this._languageFeaturesService.inlayHintsProvider,t,this._getHintsRanges(),c);if(r.delay=this._debounceInfo.update(t,Date.now()-a),c.isCancellationRequested){d.dispose();return}for(const h of d.provider)typeof h.onDidChangeInlayHints=="function"&&!s.has(h)&&(s.add(h),this._sessionDisposables.add(h.onDidChangeInlayHints(()=>{r.isScheduled()||r.schedule()})));this._sessionDisposables.add(d),this._updateHintsDecorators(d.ranges,d.items),this._cacheHintsForFastRestore(t)}catch(c){Oe(c)}finally{n.dispose(),l.dispose()}},this._debounceInfo.get(t));this._sessionDisposables.add(r),this._sessionDisposables.add(ve(()=>n==null?void 0:n.dispose(!0))),r.schedule(0),this._sessionDisposables.add(this._editor.onDidScrollChange(a=>{(a.scrollTopChanged||!r.isScheduled())&&r.schedule()})),this._sessionDisposables.add(this._editor.onDidChangeModelContent(a=>{n==null||n.cancel();const l=Math.max(r.delay,1250);r.schedule(l)})),this._sessionDisposables.add(this._installDblClickGesture(()=>r.schedule(0))),this._sessionDisposables.add(this._installLinkGesture()),this._sessionDisposables.add(this._installContextMenu())}_installLinkGesture(){const e=new K,t=e.add(new AE(this._editor)),i=new K;return e.add(i),e.add(t.onMouseMoveOrRelevantKeyDown(n=>{const[s]=n,r=this._getInlayHintLabelPart(s),a=this._editor.getModel();if(!r||!a){i.clear();return}const l=new li;i.add(ve(()=>l.dispose(!0))),r.item.resolve(l.token),this._activeInlayHintPart=r.part.command||r.part.location?new nLe(r,s.hasTriggerModifier):void 0;const c=a.validatePosition(r.item.hint.position).lineNumber,d=new x(c,1,c,a.getLineMaxColumn(c)),h=this._getInlineHintsForRange(d);this._updateHintsDecorators([d],h),i.add(ve(()=>{this._activeInlayHintPart=void 0,this._updateHintsDecorators([d],h)}))})),e.add(t.onCancel(()=>i.clear())),e.add(t.onExecute(async n=>{const s=this._getInlayHintLabelPart(n);if(s){const r=s.part;r.location?this._instaService.invokeFunction($Z,n,this._editor,r.location):xR.is(r.command)&&await this._invokeCommand(r.command,s.item)}})),e}_getInlineHintsForRange(e){const t=new Set;for(const i of this._decorationsMetadata.values())e.containsRange(i.item.anchor.range)&&t.add(i.item);return Array.from(t)}_installDblClickGesture(e){return this._editor.onMouseUp(async t=>{if(t.event.detail!==2)return;const i=this._getInlayHintLabelPart(t);if(i&&(t.event.preventDefault(),await i.item.resolve(ze.None),Qn(i.item.hint.textEdits))){const n=i.item.hint.textEdits.map(s=>ri.replace(x.lift(s.range),s.text));this._editor.executeEdits("inlayHint.default",n),e()}})}_installContextMenu(){return this._editor.onContextMenu(async e=>{if(!Xi(e.event.target))return;const t=this._getInlayHintLabelPart(e);t&&await this._instaService.invokeFunction(tLe,this._editor,e.event.target,t)})}_getInlayHintLabelPart(e){var i;if(e.target.type!==6)return;const t=(i=e.target.detail.injectedText)==null?void 0:i.options;if(t instanceof Kh&&(t==null?void 0:t.attachedData)instanceof TO)return t.attachedData}async _invokeCommand(e,t){try{await this._commandService.executeCommand(e.id,...e.arguments??[])}catch(i){this._notificationService.notify({severity:OI.Error,source:t.provider.displayName,message:i})}}_cacheHintsForFastRestore(e){const t=this._copyInlayHintsWithCurrentAnchor(e);this._inlayHintsCache.set(e,t)}_copyInlayHintsWithCurrentAnchor(e){const t=new Map;for(const[i,n]of this._decorationsMetadata){if(t.has(n.item))continue;const s=e.getDecorationRange(i);if(s){const r=new RZ(s,n.item.anchor.direction),a=n.item.with({anchor:r});t.set(n.item,a)}}return Array.from(t.values())}_getHintsRanges(){const t=this._editor.getModel(),i=this._editor.getVisibleRangesPlusViewportAboveBelow(),n=[];for(const s of i.sort(x.compareRangesUsingStarts)){const r=t.validateRange(new x(s.startLineNumber-30,s.startColumn,s.endLineNumber+30,s.endColumn));n.length===0||!x.areIntersectingOrTouching(n[n.length-1],r)?n.push(r):n[n.length-1]=x.plusRange(n[n.length-1],r)}return n}_updateHintsDecorators(e,t){var f,p;const i=[],n=(_,b,C,v,w)=>{const S={content:C,inlineClassNameAffectsLetterSpacing:!0,inlineClassName:b.className,cursorStops:v,attachedData:w};i.push({item:_,classNameRef:b,decoration:{range:_.anchor.range,options:{description:"InlayHint",showIfCollapsed:_.anchor.range.isEmpty(),collapseOnReplaceEdit:!_.anchor.range.isEmpty(),stickiness:0,[_.anchor.direction]:this._activeRenderMode===0?S:void 0}}})},s=(_,b)=>{const C=this._ruleFactory.createClassNameRef({width:`${r/3|0}px`,display:"inline-block"});n(_,C," ",b?zr.Right:zr.None)},{fontSize:r,fontFamily:a,padding:l,isUniform:c}=this._getLayoutInfo(),d="--code-editorInlayHintsFontFamily";this._editor.getContainerDomNode().style.setProperty(d,a);let h={line:0,totalLen:0};for(const _ of t){if(h.line!==_.anchor.range.startLineNumber&&(h={line:_.anchor.range.startLineNumber,totalLen:0}),h.totalLen>Gm._MAX_LABEL_LEN)continue;_.hint.paddingLeft&&s(_,!1);const b=typeof _.hint.label=="string"?[{label:_.hint.label}]:_.hint.label;for(let C=0;C<b.length;C++){const v=b[C],w=C===0,S=C===b.length-1,L={fontSize:`${r}px`,fontFamily:`var(${d}), ${fs.fontFamily}`,verticalAlign:c?"baseline":"middle",unicodeBidi:"isolate"};Qn(_.hint.textEdits)&&(L.cursor="default"),this._fillInColors(L,_.hint),(v.command||v.location)&&((f=this._activeInlayHintPart)==null?void 0:f.part.item)===_&&this._activeInlayHintPart.part.index===C&&(L.textDecoration="underline",this._activeInlayHintPart.hasTriggerModifier&&(L.color=mi(Woe),L.cursor="pointer")),l&&(w&&S?(L.padding=`1px ${Math.max(1,r/4)|0}px`,L.borderRadius=`${r/4|0}px`):w?(L.padding=`1px 0 1px ${Math.max(1,r/4)|0}px`,L.borderRadius=`${r/4|0}px 0 0 ${r/4|0}px`):S?(L.padding=`1px ${Math.max(1,r/4)|0}px 1px 0`,L.borderRadius=`0 ${r/4|0}px ${r/4|0}px 0`):L.padding="1px 0 1px 0");let k=v.label;h.totalLen+=k.length;let D=!1;const N=h.totalLen-Gm._MAX_LABEL_LEN;if(N>0&&(k=k.slice(0,-N)+"…",D=!0),n(_,this._ruleFactory.createClassNameRef(L),sLe(k),S&&!_.hint.paddingRight?zr.Right:zr.None,new TO(_,C)),D)break}if(_.hint.paddingRight&&s(_,!0),i.length>Gm._MAX_DECORATORS)break}const u=[];for(const[_,b]of this._decorationsMetadata){const C=(p=this._editor.getModel())==null?void 0:p.getDecorationRange(_);C&&e.some(v=>v.containsRange(C))&&(u.push(_),b.classNameRef.dispose(),this._decorationsMetadata.delete(_))}const g=za.capture(this._editor);this._editor.changeDecorations(_=>{const b=_.deltaDecorations(u,i.map(C=>C.decoration));for(let C=0;C<b.length;C++){const v=i[C];this._decorationsMetadata.set(b[C],v)}}),g.restore(this._editor)}_fillInColors(e,t){t.kind===Fy.Parameter?(e.backgroundColor=mi(Koe),e.color=mi(joe)):t.kind===Fy.Type?(e.backgroundColor=mi($oe),e.color=mi(Uoe)):(e.backgroundColor=mi(R5),e.color=mi(T5))}_getLayoutInfo(){const e=this._editor.getOption(142),t=e.padding,i=this._editor.getOption(52),n=this._editor.getOption(49);let s=e.fontSize;(!s||s<5||s>i)&&(s=i);const r=e.fontFamily||n;return{fontSize:s,fontFamily:r,padding:t,isUniform:!t&&r===n&&s===i}}_removeAllDecorations(){this._editor.removeDecorations(Array.from(this._decorationsMetadata.keys()));for(const e of this._decorationsMetadata.values())e.classNameRef.dispose();this._decorationsMetadata.clear()}},Gm=Uc,Uc.ID="editor.contrib.InlayHints",Uc._MAX_DECORATORS=1500,Uc._MAX_LABEL_LEN=43,Uc);u1=Gm=iLe([Pm(1,ge),Pm(2,Zo),Pm(3,jZ),Pm(4,Ht),Pm(5,Ei),Pm(6,we)],u1);function sLe(o){return o.replace(/[ \t]/g," ")}rt.registerCommand("_executeInlayHintProvider",async(o,...e)=>{const[t,i]=e;mt(Se.isUri(t)),mt(x.isIRange(i));const{inlayHintsProvider:n}=o.get(ge),s=await o.get(Hs).createModelReference(t);try{const r=await Px.create(n,s.object.textEditorModel,[x.lift(i)],ze.None),a=r.items.map(l=>l.hint);return setTimeout(()=>r.dispose(),0),a}finally{s.dispose()}});var oLe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},yd=function(o,e){return function(t,i){e(t,i,o)}};class jV extends gy{constructor(e,t,i,n){super(10,t,e.item.anchor.range,i,n,!0),this.part=e}}let Vx=class extends d1{constructor(e,t,i,n,s,r,a,l,c){super(e,t,i,r,l,n,s,c),this._resolverService=a,this.hoverOrdinal=6}suggestHoverAnchor(e){var n;if(!u1.get(this._editor)||e.target.type!==6)return null;const i=(n=e.target.detail.injectedText)==null?void 0:n.options;return i instanceof Kh&&i.attachedData instanceof TO?new jV(i.attachedData,this,e.event.posx,e.event.posy):null}computeSync(){return[]}computeAsync(e,t,i){return e instanceof jV?new Fo(async n=>{const{part:s}=e;if(await s.item.resolve(i),i.isCancellationRequested)return;let r;typeof s.item.hint.tooltip=="string"?r=new Pn().appendText(s.item.hint.tooltip):s.item.hint.tooltip&&(r=s.item.hint.tooltip),r&&n.emitOne(new Hr(this,e.range,[r],!1,0)),Qn(s.item.hint.textEdits)&&n.emitOne(new Hr(this,e.range,[new Pn().appendText(m("hint.dbl","Double-click to insert"))],!1,10001));let a;if(typeof s.part.tooltip=="string"?a=new Pn().appendText(s.part.tooltip):s.part.tooltip&&(a=s.part.tooltip),a&&n.emitOne(new Hr(this,e.range,[a],!1,1)),s.part.location||s.part.command){let c;const h=this._editor.getOption(78)==="altKey"?Je?m("links.navigate.kb.meta.mac","cmd + click"):m("links.navigate.kb.meta","ctrl + click"):Je?m("links.navigate.kb.alt.mac","option + click"):m("links.navigate.kb.alt","alt + click");s.part.location&&s.part.command?c=new Pn().appendText(m("hint.defAndCommand","Go to Definition ({0}), right click for more",h)):s.part.location?c=new Pn().appendText(m("hint.def","Go to Definition ({0})",h)):s.part.command&&(c=new Pn(`[${m("hint.cmd","Execute Command")}](${Mye(s.part.command)} "${s.part.command.title}") (${h})`,{isTrusted:!0})),c&&n.emitOne(new Hr(this,e.range,[c],!1,1e4))}const l=await this._resolveInlayHintLabelPartHover(s,i);for await(const c of l)n.emitOne(c)}):Fo.EMPTY}async _resolveInlayHintLabelPartHover(e,t){if(!e.part.location)return Fo.EMPTY;const{uri:i,range:n}=e.part.location,s=await this._resolverService.createModelReference(i);try{const r=s.object.textEditorModel;return this._languageFeaturesService.hoverProvider.has(r)?BW(this._languageFeaturesService.hoverProvider,r,new A(n.startLineNumber,n.startColumn),t).filter(a=>!O_(a.hover.contents)).map(a=>new Hr(this,e.item.anchor.range,a.hover.contents,!1,2+a.ordinal)):Fo.EMPTY}finally{s.dispose()}}};Vx=oLe([yd(1,ni),yd(2,zs),yd(3,yt),yd(4,Yl),yd(5,qe),yd(6,Hs),yd(7,ge),yd(8,Ht)],Vx);class jW extends F{constructor(e,t,i,n,s,r){super();const a=t.anchor,l=t.hoverParts;this._renderedHoverParts=this._register(new RO(e,i,l,r,s));const{showAtPosition:c,showAtSecondaryPosition:d}=jW.computeHoverPositions(e,a.range,l);this.shouldAppearBeforeContent=l.some(h=>h.isBeforeContent),this.showAtPosition=c,this.showAtSecondaryPosition=d,this.initialMousePosX=a.initialMousePosX,this.initialMousePosY=a.initialMousePosY,this.shouldFocus=n.shouldFocus,this.source=n.source}get domNode(){return this._renderedHoverParts.domNode}get domNodeHasChildren(){return this._renderedHoverParts.domNodeHasChildren}get focusedHoverPartIndex(){return this._renderedHoverParts.focusedHoverPartIndex}async updateHoverVerbosityLevel(e,t,i){this._renderedHoverParts.updateHoverVerbosityLevel(e,t,i)}isColorPickerVisible(){return this._renderedHoverParts.isColorPickerVisible()}static computeHoverPositions(e,t,i){let n=1;if(e.hasModel()){const d=e._getViewModel(),h=d.coordinatesConverter,u=h.convertModelRangeToViewRange(t),g=d.getLineMinColumn(u.startLineNumber),f=new A(u.startLineNumber,g);n=h.convertViewPositionToModelPosition(f).column}const s=t.startLineNumber;let r=t.startColumn,a;for(const d of i){const h=d.range,u=h.startLineNumber===s,g=h.endLineNumber===s;if(u&&g){const p=h.startColumn,_=Math.min(r,p);r=Math.max(_,n)}d.forceShowAtRange&&(a=h)}let l,c;if(a){const d=a.getStartPosition();l=d,c=d}else l=t.getStartPosition(),c=new A(s,r);return{showAtPosition:l,showAtSecondaryPosition:c}}}class rLe{constructor(e,t){this._statusBar=t,e.appendChild(this._statusBar.hoverElement)}get hoverElement(){return this._statusBar.hoverElement}get actions(){return this._statusBar.actions}dispose(){this._statusBar.dispose()}}const kD=class kD extends F{constructor(e,t,i,n,s){super(),this._renderedParts=[],this._focusedHoverPartIndex=-1,this._context=s,this._fragment=document.createDocumentFragment(),this._register(this._renderParts(t,i,s,n)),this._register(this._registerListenersOnRenderedParts()),this._register(this._createEditorDecorations(e,i)),this._updateMarkdownAndColorParticipantInfo(t)}_createEditorDecorations(e,t){if(t.length===0)return F.None;let i=t[0].range;for(const s of t){const r=s.range;i=x.plusRange(i,r)}const n=e.createDecorationsCollection();return n.set([{range:i,options:kD._DECORATION_OPTIONS}]),ve(()=>{n.clear()})}_renderParts(e,t,i,n){const s=new Ax(n),r={fragment:this._fragment,statusBar:s,...i},a=new K;for(const c of e){const d=this._renderHoverPartsForParticipant(t,c,r);a.add(d);for(const h of d.renderedHoverParts)this._renderedParts.push({type:"hoverPart",participant:c,hoverPart:h.hoverPart,hoverElement:h.hoverElement})}const l=this._renderStatusBar(this._fragment,s);return l&&(a.add(l),this._renderedParts.push({type:"statusBar",hoverElement:l.hoverElement,actions:l.actions})),ve(()=>{a.dispose()})}_renderHoverPartsForParticipant(e,t,i){const n=e.filter(r=>r.owner===t);return n.length>0?t.renderHoverParts(i,n):new Kf([])}_renderStatusBar(e,t){if(t.hasContent)return new rLe(e,t)}_registerListenersOnRenderedParts(){const e=new K;return this._renderedParts.forEach((t,i)=>{const n=t.hoverElement;n.tabIndex=0,e.add(U(n,ie.FOCUS_IN,s=>{s.stopPropagation(),this._focusedHoverPartIndex=i})),e.add(U(n,ie.FOCUS_OUT,s=>{s.stopPropagation(),this._focusedHoverPartIndex=-1}))}),e}_updateMarkdownAndColorParticipantInfo(e){const t=e.find(i=>i instanceof d1&&!(i instanceof Vx));t&&(this._markdownHoverParticipant=t),this._colorHoverParticipant=e.find(i=>i instanceof l1)}async updateHoverVerbosityLevel(e,t,i){if(!this._markdownHoverParticipant)return;const n=this._normalizedIndexToMarkdownHoverIndexRange(this._markdownHoverParticipant,t);if(n===void 0)return;const s=await this._markdownHoverParticipant.updateMarkdownHoverVerbosityLevel(e,n,i);s&&(this._renderedParts[t]={type:"hoverPart",participant:this._markdownHoverParticipant,hoverPart:s.hoverPart,hoverElement:s.hoverElement},this._context.onContentsChanged())}isColorPickerVisible(){var e;return((e=this._colorHoverParticipant)==null?void 0:e.isColorPickerVisible())??!1}_normalizedIndexToMarkdownHoverIndexRange(e,t){const i=this._renderedParts[t];if(!i||i.type!=="hoverPart"||!(i.participant===e))return;const s=this._renderedParts.findIndex(r=>r.type==="hoverPart"&&r.participant===e);if(s===-1)throw new bt;return t-s}get domNode(){return this._fragment}get domNodeHasChildren(){return this._fragment.hasChildNodes()}get focusedHoverPartIndex(){return this._focusedHoverPartIndex}};kD._DECORATION_OPTIONS=We.register({description:"content-hover-highlight",className:"hoverHighlight"});let RO=kD;var aLe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},KV=function(o,e){return function(t,i){e(t,i,o)}};let MO=class extends F{constructor(e,t,i){super(),this._editor=e,this._instantiationService=t,this._keybindingService=i,this._currentResult=null,this._onContentsChanged=this._register(new P),this.onContentsChanged=this._onContentsChanged.event,this._contentHoverWidget=this._register(this._instantiationService.createInstance(CO,this._editor)),this._participants=this._initializeHoverParticipants(),this._computer=new Mx(this._editor,this._participants),this._hoverOperation=this._register(new DZ(this._editor,this._computer)),this._registerListeners()}_initializeHoverParticipants(){const e=[];for(const t of um.getAll()){const i=this._instantiationService.createInstance(t,this._editor);e.push(i)}return e.sort((t,i)=>t.hoverOrdinal-i.hoverOrdinal),this._register(this._contentHoverWidget.onDidResize(()=>{this._participants.forEach(t=>{var i;return(i=t.handleResize)==null?void 0:i.call(t)})})),e}_registerListeners(){this._register(this._hoverOperation.onResult(t=>{if(!this._computer.anchor)return;const i=t.hasLoadingMessage?this._addLoadingMessage(t.value):t.value;this._withResult(new IZ(this._computer.anchor,i,t.isComplete))}));const e=this._contentHoverWidget.getDomNode();this._register(hi(e,"keydown",t=>{t.equals(9)&&this.hide()})),this._register(hi(e,"mouseleave",t=>{this._onMouseLeave(t)})),this._register(fi.onDidChange(()=>{this._contentHoverWidget.position&&this._currentResult&&this._setCurrentResult(this._currentResult)}))}_startShowingOrUpdateHover(e,t,i,n,s){if(!(this._contentHoverWidget.position&&this._currentResult))return e?(this._startHoverOperationIfNecessary(e,t,i,n,!1),!0):!1;const a=this._editor.getOption(60).sticky,l=s&&this._contentHoverWidget.isMouseGettingCloser(s.event.posx,s.event.posy);return a&&l?(e&&this._startHoverOperationIfNecessary(e,t,i,n,!0),!0):e?this._currentResult.anchor.equals(e)?!0:e.canAdoptVisibleHover(this._currentResult.anchor,this._contentHoverWidget.position)?(this._setCurrentResult(this._currentResult.filter(e)),this._startHoverOperationIfNecessary(e,t,i,n,!1),!0):(this._setCurrentResult(null),this._startHoverOperationIfNecessary(e,t,i,n,!1),!0):(this._setCurrentResult(null),!1)}_startHoverOperationIfNecessary(e,t,i,n,s){this._computer.anchor&&this._computer.anchor.equals(e)||(this._hoverOperation.cancel(),this._computer.anchor=e,this._computer.shouldFocus=n,this._computer.source=i,this._computer.insistOnKeepingHoverVisible=s,this._hoverOperation.start(t))}_setCurrentResult(e){let t=e;if(this._currentResult===t)return;t&&t.hoverParts.length===0&&(t=null),this._currentResult=t,this._currentResult?this._showHover(this._currentResult):this._hideHover()}_addLoadingMessage(e){if(!this._computer.anchor)return e;for(const t of this._participants){if(!t.createLoadingMessage)continue;const i=t.createLoadingMessage(this._computer.anchor);if(i)return e.slice(0).concat([i])}return e}_withResult(e){if(this._contentHoverWidget.position&&this._currentResult&&this._currentResult.isComplete||this._setCurrentResult(e),!e.isComplete)return;const n=e.hoverParts.length===0,s=this._computer.insistOnKeepingHoverVisible;n&&s||this._setCurrentResult(e)}_showHover(e){const t=this._getHoverContext();this._renderedContentHover=new jW(this._editor,e,this._participants,this._computer,t,this._keybindingService),this._renderedContentHover.domNodeHasChildren?this._contentHoverWidget.show(this._renderedContentHover):this._renderedContentHover.dispose()}_hideHover(){this._contentHoverWidget.hide()}_getHoverContext(){return{hide:()=>{this.hide()},onContentsChanged:()=>{this._onContentsChanged.fire(),this._contentHoverWidget.onContentsChanged()},setMinimumDimensions:n=>{this._contentHoverWidget.setMinimumDimensions(n)}}}showsOrWillShow(e){if(this._contentHoverWidget.isResizing)return!0;const i=this._findHoverAnchorCandidates(e);if(!(i.length>0))return this._startShowingOrUpdateHover(null,0,0,!1,e);const s=i[0];return this._startShowingOrUpdateHover(s,0,0,!1,e)}_findHoverAnchorCandidates(e){const t=[];for(const n of this._participants){if(!n.suggestHoverAnchor)continue;const s=n.suggestHoverAnchor(e);s&&t.push(s)}const i=e.target;switch(i.type){case 6:{t.push(new WT(0,i.range,e.event.posx,e.event.posy));break}case 7:{const n=this._editor.getOption(50).typicalHalfwidthCharacterWidth/2;if(!(!i.detail.isAfterLines&&typeof i.detail.horizontalDistanceToText=="number"&&i.detail.horizontalDistanceToText<n))break;t.push(new WT(0,i.range,e.event.posx,e.event.posy));break}}return t.sort((n,s)=>s.priority-n.priority),t}_onMouseLeave(e){const t=this._editor.getDomNode();(!t||!ME(t,e.x,e.y))&&this.hide()}startShowingAtRange(e,t,i,n){this._startShowingOrUpdateHover(new WT(0,e,void 0,void 0),t,i,n,null)}async updateHoverVerbosityLevel(e,t,i){var n;(n=this._renderedContentHover)==null||n.updateHoverVerbosityLevel(e,t,i)}focusedHoverPartIndex(){var e;return((e=this._renderedContentHover)==null?void 0:e.focusedHoverPartIndex)??-1}containsNode(e){return e?this._contentHoverWidget.getDomNode().contains(e):!1}focus(){this._contentHoverWidget.focus()}scrollUp(){this._contentHoverWidget.scrollUp()}scrollDown(){this._contentHoverWidget.scrollDown()}scrollLeft(){this._contentHoverWidget.scrollLeft()}scrollRight(){this._contentHoverWidget.scrollRight()}pageUp(){this._contentHoverWidget.pageUp()}pageDown(){this._contentHoverWidget.pageDown()}goToTop(){this._contentHoverWidget.goToTop()}goToBottom(){this._contentHoverWidget.goToBottom()}hide(){this._computer.anchor=null,this._hoverOperation.cancel(),this._setCurrentResult(null)}getDomNode(){return this._contentHoverWidget.getDomNode()}get isColorPickerVisible(){var e;return((e=this._renderedContentHover)==null?void 0:e.isColorPickerVisible())??!1}get isVisibleFromKeyboard(){return this._contentHoverWidget.isVisibleFromKeyboard}get isVisible(){return this._contentHoverWidget.isVisible}get isFocused(){return this._contentHoverWidget.isFocused}get isResizing(){return this._contentHoverWidget.isResizing}get widget(){return this._contentHoverWidget}};MO=aLe([KV(1,we),KV(2,yt)],MO);var lLe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},qV=function(o,e){return function(t,i){e(t,i,o)}},AO,qg;let es=(qg=class extends F{constructor(e,t,i){super(),this._editor=e,this._instantiationService=t,this._keybindingService=i,this._onHoverContentsChanged=this._register(new P),this.shouldKeepOpenOnEditorMouseMoveOrLeave=!1,this._listenersStore=new K,this._hoverState={mouseDown:!1,activatedByDecoratorClick:!1},this._reactToEditorMouseMoveRunner=this._register(new It(()=>this._reactToEditorMouseMove(this._mouseMoveEvent),0)),this._hookListeners(),this._register(this._editor.onDidChangeConfiguration(n=>{n.hasChanged(60)&&(this._unhookListeners(),this._hookListeners())}))}static get(e){return e.getContribution(AO.ID)}_hookListeners(){const e=this._editor.getOption(60);this._hoverSettings={enabled:e.enabled,sticky:e.sticky,hidingDelay:e.hidingDelay},e.enabled?(this._listenersStore.add(this._editor.onMouseDown(t=>this._onEditorMouseDown(t))),this._listenersStore.add(this._editor.onMouseUp(()=>this._onEditorMouseUp())),this._listenersStore.add(this._editor.onMouseMove(t=>this._onEditorMouseMove(t))),this._listenersStore.add(this._editor.onKeyDown(t=>this._onKeyDown(t)))):(this._listenersStore.add(this._editor.onMouseMove(t=>this._onEditorMouseMove(t))),this._listenersStore.add(this._editor.onKeyDown(t=>this._onKeyDown(t)))),this._listenersStore.add(this._editor.onMouseLeave(t=>this._onEditorMouseLeave(t))),this._listenersStore.add(this._editor.onDidChangeModel(()=>{this._cancelScheduler(),this._hideWidgets()})),this._listenersStore.add(this._editor.onDidChangeModelContent(()=>this._cancelScheduler())),this._listenersStore.add(this._editor.onDidScrollChange(t=>this._onEditorScrollChanged(t)))}_unhookListeners(){this._listenersStore.clear()}_cancelScheduler(){this._mouseMoveEvent=void 0,this._reactToEditorMouseMoveRunner.cancel()}_onEditorScrollChanged(e){(e.scrollTopChanged||e.scrollLeftChanged)&&this._hideWidgets()}_onEditorMouseDown(e){this._hoverState.mouseDown=!0,!this._shouldNotHideCurrentHoverWidget(e)&&this._hideWidgets()}_shouldNotHideCurrentHoverWidget(e){return this._isMouseOnContentHoverWidget(e)||this._isContentWidgetResizing()}_isMouseOnContentHoverWidget(e){var i;const t=(i=this._contentWidget)==null?void 0:i.getDomNode();return t?ME(t,e.event.posx,e.event.posy):!1}_onEditorMouseUp(){this._hoverState.mouseDown=!1}_onEditorMouseLeave(e){this.shouldKeepOpenOnEditorMouseMoveOrLeave||(this._cancelScheduler(),this._shouldNotHideCurrentHoverWidget(e))||this._hideWidgets()}_shouldNotRecomputeCurrentHoverWidget(e){const t=this._hoverSettings.sticky,i=(r,a)=>{const l=this._isMouseOnContentHoverWidget(r);return a&&l},n=r=>{var c;const a=this._isMouseOnContentHoverWidget(r),l=((c=this._contentWidget)==null?void 0:c.isColorPickerVisible)??!1;return a&&l},s=(r,a)=>{var l,c,d,h;return(a&&((c=this._contentWidget)==null?void 0:c.containsNode((l=r.event.browserEvent.view)==null?void 0:l.document.activeElement))&&!((h=(d=r.event.browserEvent.view)==null?void 0:d.getSelection())!=null&&h.isCollapsed))??!1};return i(e,t)||n(e)||s(e,t)}_onEditorMouseMove(e){var a,l,c,d;if(this.shouldKeepOpenOnEditorMouseMoveOrLeave||(this._mouseMoveEvent=e,(a=this._contentWidget)!=null&&a.isFocused||(l=this._contentWidget)!=null&&l.isResizing))return;const t=this._hoverSettings.sticky;if(t&&((c=this._contentWidget)!=null&&c.isVisibleFromKeyboard))return;if(this._shouldNotRecomputeCurrentHoverWidget(e)){this._reactToEditorMouseMoveRunner.cancel();return}const n=this._hoverSettings.hidingDelay;if(((d=this._contentWidget)==null?void 0:d.isVisible)&&t&&n>0){this._reactToEditorMouseMoveRunner.isScheduled()||this._reactToEditorMouseMoveRunner.schedule(n);return}this._reactToEditorMouseMove(e)}_reactToEditorMouseMove(e){var l;if(!e)return;const i=(l=e.target.element)==null?void 0:l.classList.contains("colorpicker-color-decoration"),n=this._editor.getOption(149),s=this._hoverSettings.enabled,r=this._hoverState.activatedByDecoratorClick;if(i&&(n==="click"&&!r||n==="hover"&&!s||n==="clickAndHover"&&!s&&!r)||!i&&!s&&!r){this._hideWidgets();return}this._tryShowHoverWidget(e)||this._hideWidgets()}_tryShowHoverWidget(e){return this._getOrCreateContentWidget().showsOrWillShow(e)}_onKeyDown(e){var n;if(!this._editor.hasModel())return;const t=this._keybindingService.softDispatch(e,this._editor.getDomNode()),i=t.kind===1||t.kind===2&&(t.commandId===yZ||t.commandId===TE||t.commandId===RE)&&((n=this._contentWidget)==null?void 0:n.isVisible);e.keyCode===5||e.keyCode===6||e.keyCode===57||e.keyCode===4||i||this._hideWidgets()}_hideWidgets(){var e,t;this._hoverState.mouseDown&&((e=this._contentWidget)!=null&&e.isColorPickerVisible)||J_.dropDownVisible||(this._hoverState.activatedByDecoratorClick=!1,(t=this._contentWidget)==null||t.hide())}_getOrCreateContentWidget(){return this._contentWidget||(this._contentWidget=this._instantiationService.createInstance(MO,this._editor),this._listenersStore.add(this._contentWidget.onContentsChanged(()=>this._onHoverContentsChanged.fire()))),this._contentWidget}showContentHover(e,t,i,n,s=!1){this._hoverState.activatedByDecoratorClick=s,this._getOrCreateContentWidget().startShowingAtRange(e,t,i,n)}_isContentWidgetResizing(){var e;return((e=this._contentWidget)==null?void 0:e.widget.isResizing)||!1}focusedHoverPartIndex(){return this._getOrCreateContentWidget().focusedHoverPartIndex()}updateHoverVerbosityLevel(e,t,i){this._getOrCreateContentWidget().updateHoverVerbosityLevel(e,t,i)}focus(){var e;(e=this._contentWidget)==null||e.focus()}scrollUp(){var e;(e=this._contentWidget)==null||e.scrollUp()}scrollDown(){var e;(e=this._contentWidget)==null||e.scrollDown()}scrollLeft(){var e;(e=this._contentWidget)==null||e.scrollLeft()}scrollRight(){var e;(e=this._contentWidget)==null||e.scrollRight()}pageUp(){var e;(e=this._contentWidget)==null||e.pageUp()}pageDown(){var e;(e=this._contentWidget)==null||e.pageDown()}goToTop(){var e;(e=this._contentWidget)==null||e.goToTop()}goToBottom(){var e;(e=this._contentWidget)==null||e.goToBottom()}get isColorPickerVisible(){var e;return(e=this._contentWidget)==null?void 0:e.isColorPickerVisible}get isHoverVisible(){var e;return(e=this._contentWidget)==null?void 0:e.isVisible}dispose(){var e;super.dispose(),this._unhookListeners(),this._listenersStore.dispose(),(e=this._contentWidget)==null||e.dispose()}},AO=qg,qg.ID="editor.contrib.contentHover",qg);es=AO=lLe([qV(1,we),qV(2,yt)],es);const LB=class LB extends F{constructor(e){super(),this._editor=e,this._register(e.onMouseDown(t=>this.onMouseDown(t)))}dispose(){super.dispose()}onMouseDown(e){const t=this._editor.getOption(149);if(t!=="click"&&t!=="clickAndHover")return;const i=e.target;if(i.type!==6||!i.detail.injectedText||i.detail.injectedText.options.attachedData!==pZ||!i.range)return;const n=this._editor.getContribution(es.ID);if(n&&!n.isColorPickerVisible){const s=new x(i.range.startLineNumber,i.range.startColumn+1,i.range.endLineNumber,i.range.endColumn+1);n.showContentHover(s,1,0,!1,!0)}}};LB.ID="editor.contrib.colorContribution";let zx=LB;ct(zx.ID,zx,2);um.register(l1);var KZ=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},bp=function(o,e){return function(t,i){e(t,i,o)}},PO,OO,Gg;let Gf=(Gg=class extends F{constructor(e,t,i){super(),this._editor=e,this._instantiationService=i,this._standaloneColorPickerWidget=null,this._standaloneColorPickerVisible=I.standaloneColorPickerVisible.bindTo(t),this._standaloneColorPickerFocused=I.standaloneColorPickerFocused.bindTo(t)}showOrFocus(){var e;this._editor.hasModel()&&(this._standaloneColorPickerVisible.get()?this._standaloneColorPickerFocused.get()||(e=this._standaloneColorPickerWidget)==null||e.focus():this._standaloneColorPickerWidget=this._instantiationService.createInstance(FO,this._editor,this._standaloneColorPickerVisible,this._standaloneColorPickerFocused))}hide(){var e;this._standaloneColorPickerFocused.set(!1),this._standaloneColorPickerVisible.set(!1),(e=this._standaloneColorPickerWidget)==null||e.hide(),this._editor.focus()}insertColor(){var e;(e=this._standaloneColorPickerWidget)==null||e.updateEditor(),this.hide()}static get(e){return e.getContribution(PO.ID)}},PO=Gg,Gg.ID="editor.contrib.standaloneColorPickerController",Gg);Gf=PO=KZ([bp(1,Ee),bp(2,we)],Gf);ct(Gf.ID,Gf,1);const GV=8,cLe=22;var Zg;let FO=(Zg=class extends F{constructor(e,t,i,n,s,r,a){var h;super(),this._editor=e,this._standaloneColorPickerVisible=t,this._standaloneColorPickerFocused=i,this._keybindingService=s,this._languageFeaturesService=r,this._editorWorkerService=a,this.allowEditorOverflow=!0,this._position=void 0,this._body=document.createElement("div"),this._colorHover=null,this._selectionSetInEditor=!1,this._onResult=this._register(new P),this.onResult=this._onResult.event,this._standaloneColorPickerVisible.set(!0),this._standaloneColorPickerParticipant=n.createInstance(c1,this._editor),this._position=(h=this._editor._getViewModel())==null?void 0:h.getPrimaryCursorState().modelState.position;const l=this._editor.getSelection(),c=l?{startLineNumber:l.startLineNumber,startColumn:l.startColumn,endLineNumber:l.endLineNumber,endColumn:l.endColumn}:{startLineNumber:0,endLineNumber:0,endColumn:0,startColumn:0},d=this._register(Fa(this._body));this._register(d.onDidBlur(u=>{this.hide()})),this._register(d.onDidFocus(u=>{this.focus()})),this._register(this._editor.onDidChangeCursorPosition(()=>{this._selectionSetInEditor?this._selectionSetInEditor=!1:this.hide()})),this._register(this._editor.onMouseMove(u=>{var f;const g=(f=u.target.element)==null?void 0:f.classList;g&&g.contains("colorpicker-color-decoration")&&this.hide()})),this._register(this.onResult(u=>{this._render(u.value,u.foundInEditor)})),this._start(c),this._body.style.zIndex="50",this._editor.addContentWidget(this)}updateEditor(){this._colorHover&&this._standaloneColorPickerParticipant.updateEditorModel(this._colorHover)}getId(){return OO.ID}getDomNode(){return this._body}getPosition(){if(!this._position)return null;const e=this._editor.getOption(60).above;return{position:this._position,secondaryPosition:this._position,preference:e?[1,2]:[2,1],positionAffinity:2}}hide(){this.dispose(),this._standaloneColorPickerVisible.set(!1),this._standaloneColorPickerFocused.set(!1),this._editor.removeContentWidget(this),this._editor.focus()}focus(){this._standaloneColorPickerFocused.set(!0),this._body.focus()}async _start(e){const t=await this._computeAsync(e);t&&this._onResult.fire(new dLe(t.result,t.foundInEditor))}async _computeAsync(e){if(!this._editor.hasModel())return null;const t={range:e,color:{red:0,green:0,blue:0,alpha:1}},i=await this._standaloneColorPickerParticipant.createColorHover(t,new a1(this._editorWorkerService),this._languageFeaturesService.colorProvider);return i?{result:i.colorHover,foundInEditor:i.foundInEditor}:null}_render(e,t){const i=document.createDocumentFragment(),n=this._register(new Ax(this._keybindingService)),s={fragment:i,statusBar:n,onContentsChanged:()=>{},hide:()=>this.hide()};this._colorHover=e;const r=this._standaloneColorPickerParticipant.renderHoverParts(s,[e]);if(!r)return;this._register(r.disposables);const a=r.colorPicker;this._body.classList.add("standalone-colorpicker-body"),this._body.style.maxHeight=Math.max(this._editor.getLayoutInfo().height/4,250)+"px",this._body.style.maxWidth=Math.max(this._editor.getLayoutInfo().width*.66,500)+"px",this._body.tabIndex=0,this._body.appendChild(i),a.layout();const l=a.body,c=l.saturationBox.domNode.clientWidth,d=l.domNode.clientWidth-c-cLe-GV,h=a.body.enterButton;h==null||h.onClicked(()=>{this.updateEditor(),this.hide()});const u=a.header,g=u.pickedColorNode;g.style.width=c+GV+"px";const f=u.originalColorNode;f.style.width=d+"px";const p=a.header.closeButton;p==null||p.onClicked(()=>{this.hide()}),t&&(h&&(h.button.textContent="Replace"),this._selectionSetInEditor=!0,this._editor.setSelection(e.range)),this._editor.layoutContentWidget(this)}},OO=Zg,Zg.ID="editor.contrib.standaloneColorPickerWidget",Zg);FO=OO=KZ([bp(3,we),bp(4,yt),bp(5,ge),bp(6,Us)],FO);class dLe{constructor(e,t){this.value=e,this.foundInEditor=t}}class hLe extends $a{constructor(){super({id:"editor.action.showOrFocusStandaloneColorPicker",title:{...Te("showOrFocusStandaloneColorPicker","Show or Focus Standalone Color Picker"),mnemonicTitle:m({key:"mishowOrFocusStandaloneColorPicker",comment:["&& denotes a mnemonic"]},"&&Show or Focus Standalone Color Picker")},precondition:void 0,menu:[{id:me.CommandPalette}],metadata:{description:Te("showOrFocusStandaloneColorPickerDescription","Show or focus a standalone color picker which uses the default color provider. It displays hex/rgb/hsl colors.")}})}runEditorCommand(e,t){var i;(i=Gf.get(t))==null||i.showOrFocus()}}class uLe extends de{constructor(){super({id:"editor.action.hideColorPicker",label:m({key:"hideColorPicker",comment:["Action that hides the color picker"]},"Hide the Color Picker"),alias:"Hide the Color Picker",precondition:I.standaloneColorPickerVisible.isEqualTo(!0),kbOpts:{primary:9,weight:100},metadata:{description:Te("hideColorPickerDescription","Hide the standalone color picker.")}})}run(e,t){var i;(i=Gf.get(t))==null||i.hide()}}class gLe extends de{constructor(){super({id:"editor.action.insertColorWithStandaloneColorPicker",label:m({key:"insertColorWithStandaloneColorPicker",comment:["Action that inserts color with standalone color picker"]},"Insert Color with Standalone Color Picker"),alias:"Insert Color with Standalone Color Picker",precondition:I.standaloneColorPickerFocused.isEqualTo(!0),kbOpts:{primary:3,weight:100},metadata:{description:Te("insertColorWithStandaloneColorPickerDescription","Insert hex/rgb/hsl colors with the focused standalone color picker.")}})}run(e,t){var i;(i=Gf.get(t))==null||i.insertColor()}}Q(uLe);Q(gLe);Ot(hLe);class nh{constructor(e,t,i){this.languageConfigurationService=i,this._selection=e,this._insertSpace=t,this._usedEndToken=null}static _haystackHasNeedleAtOffset(e,t,i){if(i<0)return!1;const n=t.length,s=e.length;if(i+n>s)return!1;for(let r=0;r<n;r++){const a=e.charCodeAt(i+r),l=t.charCodeAt(r);if(a!==l&&!(a>=65&&a<=90&&a+32===l)&&!(l>=65&&l<=90&&l+32===a))return!1}return!0}_createOperationsForBlockComment(e,t,i,n,s,r){const a=e.startLineNumber,l=e.startColumn,c=e.endLineNumber,d=e.endColumn,h=s.getLineContent(a),u=s.getLineContent(c);let g=h.lastIndexOf(t,l-1+t.length),f=u.indexOf(i,d-1-i.length);if(g!==-1&&f!==-1)if(a===c)h.substring(g+t.length,f).indexOf(i)>=0&&(g=-1,f=-1);else{const _=h.substring(g+t.length),b=u.substring(0,f);(_.indexOf(i)>=0||b.indexOf(i)>=0)&&(g=-1,f=-1)}let p;g!==-1&&f!==-1?(n&&g+t.length<h.length&&h.charCodeAt(g+t.length)===32&&(t=t+" "),n&&f>0&&u.charCodeAt(f-1)===32&&(i=" "+i,f-=1),p=nh._createRemoveBlockCommentOperations(new x(a,g+t.length+1,c,f+1),t,i)):(p=nh._createAddBlockCommentOperations(e,t,i,this._insertSpace),this._usedEndToken=p.length===1?i:null);for(const _ of p)r.addTrackedEditOperation(_.range,_.text)}static _createRemoveBlockCommentOperations(e,t,i){const n=[];return x.isEmpty(e)?n.push(ri.delete(new x(e.startLineNumber,e.startColumn-t.length,e.endLineNumber,e.endColumn+i.length))):(n.push(ri.delete(new x(e.startLineNumber,e.startColumn-t.length,e.startLineNumber,e.startColumn))),n.push(ri.delete(new x(e.endLineNumber,e.endColumn,e.endLineNumber,e.endColumn+i.length)))),n}static _createAddBlockCommentOperations(e,t,i,n){const s=[];return x.isEmpty(e)?s.push(ri.replace(new x(e.startLineNumber,e.startColumn,e.endLineNumber,e.endColumn),t+"  "+i)):(s.push(ri.insert(new A(e.startLineNumber,e.startColumn),t+(n?" ":""))),s.push(ri.insert(new A(e.endLineNumber,e.endColumn),(n?" ":"")+i))),s}getEditOperations(e,t){const i=this._selection.startLineNumber,n=this._selection.startColumn;e.tokenization.tokenizeIfCheap(i);const s=e.getLanguageIdAtPosition(i,n),r=this.languageConfigurationService.getLanguageConfiguration(s).comments;!r||!r.blockCommentStartToken||!r.blockCommentEndToken||this._createOperationsForBlockComment(this._selection,r.blockCommentStartToken,r.blockCommentEndToken,this._insertSpace,e,t)}computeCursorState(e,t){const i=t.getInverseEditOperations();if(i.length===2){const n=i[0],s=i[1];return new pe(n.range.endLineNumber,n.range.endColumn,s.range.startLineNumber,s.range.startColumn)}else{const n=i[0].range,s=this._usedEndToken?-this._usedEndToken.length-1:0;return new pe(n.endLineNumber,n.endColumn+s,n.endLineNumber,n.endColumn+s)}}}class gc{constructor(e,t,i,n,s,r,a){this.languageConfigurationService=e,this._selection=t,this._indentSize=i,this._type=n,this._insertSpace=s,this._selectionId=null,this._deltaColumn=0,this._moveEndPositionDown=!1,this._ignoreEmptyLines=r,this._ignoreFirstLine=a||!1}static _gatherPreflightCommentStrings(e,t,i,n){e.tokenization.tokenizeIfCheap(t);const s=e.getLanguageIdAtPosition(t,1),r=n.getLanguageConfiguration(s).comments,a=r?r.lineCommentToken:null;if(!a)return null;const l=[];for(let c=0,d=i-t+1;c<d;c++)l[c]={ignore:!1,commentStr:a,commentStrOffset:0,commentStrLength:a.length};return l}static _analyzeLines(e,t,i,n,s,r,a,l){let c=!0,d;e===0?d=!0:e===1?d=!1:d=!0;for(let h=0,u=n.length;h<u;h++){const g=n[h],f=s+h;if(f===s&&a){g.ignore=!0;continue}const p=i.getLineContent(f),_=Jn(p);if(_===-1){g.ignore=r,g.commentStrOffset=p.length;continue}if(c=!1,g.ignore=!1,g.commentStrOffset=_,d&&!nh._haystackHasNeedleAtOffset(p,g.commentStr,_)&&(e===0?d=!1:e===1||(g.ignore=!0)),d&&t){const b=_+g.commentStrLength;b<p.length&&p.charCodeAt(b)===32&&(g.commentStrLength+=1)}}if(e===0&&c){d=!1;for(let h=0,u=n.length;h<u;h++)n[h].ignore=!1}return{supported:!0,shouldRemoveComments:d,lines:n}}static _gatherPreflightData(e,t,i,n,s,r,a,l){const c=gc._gatherPreflightCommentStrings(i,n,s,l);return c===null?{supported:!1}:gc._analyzeLines(e,t,i,c,n,r,a,l)}_executeLineComments(e,t,i,n){let s;i.shouldRemoveComments?s=gc._createRemoveLineCommentsOperations(i.lines,n.startLineNumber):(gc._normalizeInsertionPoint(e,i.lines,n.startLineNumber,this._indentSize),s=this._createAddLineCommentsOperations(i.lines,n.startLineNumber));const r=new A(n.positionLineNumber,n.positionColumn);for(let a=0,l=s.length;a<l;a++)t.addEditOperation(s[a].range,s[a].text),x.isEmpty(s[a].range)&&x.getStartPosition(s[a].range).equals(r)&&e.getLineContent(r.lineNumber).length+1===r.column&&(this._deltaColumn=(s[a].text||"").length);this._selectionId=t.trackSelection(n)}_attemptRemoveBlockComment(e,t,i,n){let s=t.startLineNumber,r=t.endLineNumber;const a=n.length+Math.max(e.getLineFirstNonWhitespaceColumn(t.startLineNumber),t.startColumn);let l=e.getLineContent(s).lastIndexOf(i,a-1),c=e.getLineContent(r).indexOf(n,t.endColumn-1-i.length);return l!==-1&&c===-1&&(c=e.getLineContent(s).indexOf(n,l+i.length),r=s),l===-1&&c!==-1&&(l=e.getLineContent(r).lastIndexOf(i,c),s=r),t.isEmpty()&&(l===-1||c===-1)&&(l=e.getLineContent(s).indexOf(i),l!==-1&&(c=e.getLineContent(s).indexOf(n,l+i.length))),l!==-1&&e.getLineContent(s).charCodeAt(l+i.length)===32&&(i+=" "),c!==-1&&e.getLineContent(r).charCodeAt(c-1)===32&&(n=" "+n,c-=1),l!==-1&&c!==-1?nh._createRemoveBlockCommentOperations(new x(s,l+i.length+1,r,c+1),i,n):null}_executeBlockComment(e,t,i){e.tokenization.tokenizeIfCheap(i.startLineNumber);const n=e.getLanguageIdAtPosition(i.startLineNumber,1),s=this.languageConfigurationService.getLanguageConfiguration(n).comments;if(!s||!s.blockCommentStartToken||!s.blockCommentEndToken)return;const r=s.blockCommentStartToken,a=s.blockCommentEndToken;let l=this._attemptRemoveBlockComment(e,i,r,a);if(!l){if(i.isEmpty()){const c=e.getLineContent(i.startLineNumber);let d=Jn(c);d===-1&&(d=c.length),l=nh._createAddBlockCommentOperations(new x(i.startLineNumber,d+1,i.startLineNumber,c.length+1),r,a,this._insertSpace)}else l=nh._createAddBlockCommentOperations(new x(i.startLineNumber,e.getLineFirstNonWhitespaceColumn(i.startLineNumber),i.endLineNumber,e.getLineMaxColumn(i.endLineNumber)),r,a,this._insertSpace);l.length===1&&(this._deltaColumn=r.length+1)}this._selectionId=t.trackSelection(i);for(const c of l)t.addEditOperation(c.range,c.text)}getEditOperations(e,t){let i=this._selection;if(this._moveEndPositionDown=!1,i.startLineNumber===i.endLineNumber&&this._ignoreFirstLine){t.addEditOperation(new x(i.startLineNumber,e.getLineMaxColumn(i.startLineNumber),i.startLineNumber+1,1),i.startLineNumber===e.getLineCount()?"":`
`),this._selectionId=t.trackSelection(i);return}i.startLineNumber<i.endLineNumber&&i.endColumn===1&&(this._moveEndPositionDown=!0,i=i.setEndPosition(i.endLineNumber-1,e.getLineMaxColumn(i.endLineNumber-1)));const n=gc._gatherPreflightData(this._type,this._insertSpace,e,i.startLineNumber,i.endLineNumber,this._ignoreEmptyLines,this._ignoreFirstLine,this.languageConfigurationService);return n.supported?this._executeLineComments(e,t,n,i):this._executeBlockComment(e,t,i)}computeCursorState(e,t){let i=t.getTrackedSelection(this._selectionId);return this._moveEndPositionDown&&(i=i.setEndPosition(i.endLineNumber+1,1)),new pe(i.selectionStartLineNumber,i.selectionStartColumn+this._deltaColumn,i.positionLineNumber,i.positionColumn+this._deltaColumn)}static _createRemoveLineCommentsOperations(e,t){const i=[];for(let n=0,s=e.length;n<s;n++){const r=e[n];r.ignore||i.push(ri.delete(new x(t+n,r.commentStrOffset+1,t+n,r.commentStrOffset+r.commentStrLength+1)))}return i}_createAddLineCommentsOperations(e,t){const i=[],n=this._insertSpace?" ":"";for(let s=0,r=e.length;s<r;s++){const a=e[s];a.ignore||i.push(ri.insert(new A(t+s,a.commentStrOffset+1),a.commentStr+n))}return i}static nextVisibleColumn(e,t,i,n){return i?e+(t-e%t):e+n}static _normalizeInsertionPoint(e,t,i,n){let s=1073741824,r,a;for(let l=0,c=t.length;l<c;l++){if(t[l].ignore)continue;const d=e.getLineContent(i+l);let h=0;for(let u=0,g=t[l].commentStrOffset;h<s&&u<g;u++)h=gc.nextVisibleColumn(h,n,d.charCodeAt(u)===9,1);h<s&&(s=h)}s=Math.floor(s/n)*n;for(let l=0,c=t.length;l<c;l++){if(t[l].ignore)continue;const d=e.getLineContent(i+l);let h=0;for(r=0,a=t[l].commentStrOffset;h<s&&r<a;r++)h=gc.nextVisibleColumn(h,n,d.charCodeAt(r)===9,1);h>s?t[l].commentStrOffset=r-1:t[l].commentStrOffset=r}}}class KW extends de{constructor(e,t){super(t),this._type=e}run(e,t){const i=e.get(di);if(!t.hasModel())return;const n=t.getModel(),s=[],r=n.getOptions(),a=t.getOption(23),l=t.getSelections().map((d,h)=>({selection:d,index:h,ignoreFirstLine:!1}));l.sort((d,h)=>x.compareRangesUsingStarts(d.selection,h.selection));let c=l[0];for(let d=1;d<l.length;d++){const h=l[d];c.selection.endLineNumber===h.selection.startLineNumber&&(c.index<h.index?h.ignoreFirstLine=!0:(c.ignoreFirstLine=!0,c=h))}for(const d of l)s.push(new gc(i,d.selection,r.indentSize,this._type,a.insertSpace,a.ignoreEmptyLines,d.ignoreFirstLine));t.pushUndoStop(),t.executeCommands(this.id,s),t.pushUndoStop()}}class fLe extends KW{constructor(){super(0,{id:"editor.action.commentLine",label:m("comment.line","Toggle Line Comment"),alias:"Toggle Line Comment",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:2138,weight:100},menuOpts:{menuId:me.MenubarEditMenu,group:"5_insert",title:m({key:"miToggleLineComment",comment:["&& denotes a mnemonic"]},"&&Toggle Line Comment"),order:1}})}}class mLe extends KW{constructor(){super(1,{id:"editor.action.addCommentLine",label:m("comment.line.add","Add Line Comment"),alias:"Add Line Comment",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2081),weight:100}})}}class pLe extends KW{constructor(){super(2,{id:"editor.action.removeCommentLine",label:m("comment.line.remove","Remove Line Comment"),alias:"Remove Line Comment",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2099),weight:100}})}}class _Le extends de{constructor(){super({id:"editor.action.blockComment",label:m("comment.block","Toggle Block Comment"),alias:"Toggle Block Comment",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:1567,linux:{primary:3103},weight:100},menuOpts:{menuId:me.MenubarEditMenu,group:"5_insert",title:m({key:"miToggleBlockComment",comment:["&& denotes a mnemonic"]},"Toggle &&Block Comment"),order:2}})}run(e,t){const i=e.get(di);if(!t.hasModel())return;const n=t.getOption(23),s=[],r=t.getSelections();for(const a of r)s.push(new nh(a,n.insertSpace,i));t.pushUndoStop(),t.executeCommands(this.id,s),t.pushUndoStop()}}Q(fLe);Q(mLe);Q(pLe);Q(_Le);var bLe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},xu=function(o,e){return function(t,i){e(t,i,o)}},WO,Yg;let tb=(Yg=class{static get(e){return e.getContribution(WO.ID)}constructor(e,t,i,n,s,r,a,l){this._contextMenuService=t,this._contextViewService=i,this._contextKeyService=n,this._keybindingService=s,this._menuService=r,this._configurationService=a,this._workspaceContextService=l,this._toDispose=new K,this._contextMenuIsBeingShownCount=0,this._editor=e,this._toDispose.add(this._editor.onContextMenu(c=>this._onContextMenu(c))),this._toDispose.add(this._editor.onMouseWheel(c=>{if(this._contextMenuIsBeingShownCount>0){const d=this._contextViewService.getContextViewElement(),h=c.srcElement;h.shadowRoot&&If(d)===h.shadowRoot||this._contextViewService.hideContextView()}})),this._toDispose.add(this._editor.onKeyDown(c=>{this._editor.getOption(24)&&c.keyCode===58&&(c.preventDefault(),c.stopPropagation(),this.showContextMenu())}))}_onContextMenu(e){if(!this._editor.hasModel())return;if(!this._editor.getOption(24)){this._editor.focus(),e.target.position&&!this._editor.getSelection().containsPosition(e.target.position)&&this._editor.setPosition(e.target.position);return}if(e.target.type===12||e.target.type===6&&e.target.detail.injectedText)return;if(e.event.preventDefault(),e.event.stopPropagation(),e.target.type===11)return this._showScrollbarContextMenu(e.event);if(e.target.type!==6&&e.target.type!==7&&e.target.type!==1)return;if(this._editor.focus(),e.target.position){let i=!1;for(const n of this._editor.getSelections())if(n.containsPosition(e.target.position)){i=!0;break}i||this._editor.setPosition(e.target.position)}let t=null;e.target.type!==1&&(t=e.event),this.showContextMenu(t)}showContextMenu(e){if(!this._editor.getOption(24)||!this._editor.hasModel())return;const t=this._getMenuActions(this._editor.getModel(),this._editor.contextMenuId);t.length>0&&this._doShowContextMenu(t,e)}_getMenuActions(e,t){const i=[],n=this._menuService.getMenuActions(t,this._contextKeyService,{arg:e.uri});for(const s of n){const[,r]=s;let a=0;for(const l of r)if(l instanceof $p){const c=this._getMenuActions(e,l.item.submenu);c.length>0&&(i.push(new D_(l.id,l.label,c)),a++)}else i.push(l),a++;a&&i.push(new Qi)}return i.length&&i.pop(),i}_doShowContextMenu(e,t=null){if(!this._editor.hasModel())return;const i=this._editor.getOption(60);this._editor.updateOptions({hover:{enabled:!1}});let n=t;if(!n){this._editor.revealPosition(this._editor.getPosition(),1),this._editor.render();const r=this._editor.getScrolledVisiblePosition(this._editor.getPosition()),a=wi(this._editor.getDomNode()),l=a.left+r.left,c=a.top+r.top+r.height;n={x:l,y:c}}const s=this._editor.getOption(128)&&!Pa;this._contextMenuIsBeingShownCount++,this._contextMenuService.showContextMenu({domForShadowRoot:s?this._editor.getOverflowWidgetsDomNode()??this._editor.getDomNode():void 0,getAnchor:()=>n,getActions:()=>e,getActionViewItem:r=>{const a=this._keybindingFor(r);if(a)return new V_(r,r,{label:!0,keybinding:a.getLabel(),isMenu:!0});const l=r;return typeof l.getActionViewItem=="function"?l.getActionViewItem():new V_(r,r,{icon:!0,label:!0,isMenu:!0})},getKeyBinding:r=>this._keybindingFor(r),onHide:r=>{this._contextMenuIsBeingShownCount--,this._editor.updateOptions({hover:i})}})}_showScrollbarContextMenu(e){if(!this._editor.hasModel()||Nce(this._workspaceContextService.getWorkspace()))return;const t=this._editor.getOption(73);let i=0;const n=c=>({id:`menu-action-${++i}`,label:c.label,tooltip:"",class:void 0,enabled:typeof c.enabled>"u"?!0:c.enabled,checked:c.checked,run:c.run}),s=(c,d)=>new D_(`menu-action-${++i}`,c,d,void 0),r=(c,d,h,u,g)=>{if(!d)return n({label:c,enabled:d,run:()=>{}});const f=_=>()=>{this._configurationService.updateValue(h,_)},p=[];for(const _ of g)p.push(n({label:_.label,checked:u===_.value,run:f(_.value)}));return s(c,p)},a=[];a.push(n({label:m("context.minimap.minimap","Minimap"),checked:t.enabled,run:()=>{this._configurationService.updateValue("editor.minimap.enabled",!t.enabled)}})),a.push(new Qi),a.push(n({label:m("context.minimap.renderCharacters","Render Characters"),enabled:t.enabled,checked:t.renderCharacters,run:()=>{this._configurationService.updateValue("editor.minimap.renderCharacters",!t.renderCharacters)}})),a.push(r(m("context.minimap.size","Vertical size"),t.enabled,"editor.minimap.size",t.size,[{label:m("context.minimap.size.proportional","Proportional"),value:"proportional"},{label:m("context.minimap.size.fill","Fill"),value:"fill"},{label:m("context.minimap.size.fit","Fit"),value:"fit"}])),a.push(r(m("context.minimap.slider","Slider"),t.enabled,"editor.minimap.showSlider",t.showSlider,[{label:m("context.minimap.slider.mouseover","Mouse Over"),value:"mouseover"},{label:m("context.minimap.slider.always","Always"),value:"always"}]));const l=this._editor.getOption(128)&&!Pa;this._contextMenuIsBeingShownCount++,this._contextMenuService.showContextMenu({domForShadowRoot:l?this._editor.getDomNode():void 0,getAnchor:()=>e,getActions:()=>a,onHide:c=>{this._contextMenuIsBeingShownCount--,this._editor.focus()}})}_keybindingFor(e){return this._keybindingService.lookupKeybinding(e.id)}dispose(){this._contextMenuIsBeingShownCount>0&&this._contextViewService.hideContextView(),this._toDispose.dispose()}},WO=Yg,Yg.ID="editor.contrib.contextmenu",Yg);tb=WO=bLe([xu(1,mo),xu(2,ud),xu(3,Ee),xu(4,yt),xu(5,Go),xu(6,qe),xu(7,Af)],tb);class CLe extends de{constructor(){super({id:"editor.action.showContextMenu",label:m("action.showContextMenu.label","Show Editor Context Menu"),alias:"Show Editor Context Menu",precondition:void 0,kbOpts:{kbExpr:I.textInputFocus,primary:1092,weight:100}})}run(e,t){var i;(i=tb.get(t))==null||i.showContextMenu()}}ct(tb.ID,tb,2);Q(CLe);class HT{constructor(e){this.selections=e}equals(e){const t=this.selections.length,i=e.selections.length;if(t!==i)return!1;for(let n=0;n<t;n++)if(!this.selections[n].equalsSelection(e.selections[n]))return!1;return!0}}class VT{constructor(e,t,i){this.cursorState=e,this.scrollTop=t,this.scrollLeft=i}}const DD=class DD extends F{static get(e){return e.getContribution(DD.ID)}constructor(e){super(),this._editor=e,this._isCursorUndoRedo=!1,this._undoStack=[],this._redoStack=[],this._register(e.onDidChangeModel(t=>{this._undoStack=[],this._redoStack=[]})),this._register(e.onDidChangeModelContent(t=>{this._undoStack=[],this._redoStack=[]})),this._register(e.onDidChangeCursorSelection(t=>{if(this._isCursorUndoRedo||!t.oldSelections||t.oldModelVersionId!==t.modelVersionId)return;const i=new HT(t.oldSelections);this._undoStack.length>0&&this._undoStack[this._undoStack.length-1].cursorState.equals(i)||(this._undoStack.push(new VT(i,e.getScrollTop(),e.getScrollLeft())),this._redoStack=[],this._undoStack.length>50&&this._undoStack.shift())}))}cursorUndo(){!this._editor.hasModel()||this._undoStack.length===0||(this._redoStack.push(new VT(new HT(this._editor.getSelections()),this._editor.getScrollTop(),this._editor.getScrollLeft())),this._applyState(this._undoStack.pop()))}cursorRedo(){!this._editor.hasModel()||this._redoStack.length===0||(this._undoStack.push(new VT(new HT(this._editor.getSelections()),this._editor.getScrollTop(),this._editor.getScrollLeft())),this._applyState(this._redoStack.pop()))}_applyState(e){this._isCursorUndoRedo=!0,this._editor.setSelections(e.cursorState.selections),this._editor.setScrollPosition({scrollTop:e.scrollTop,scrollLeft:e.scrollLeft}),this._isCursorUndoRedo=!1}};DD.ID="editor.contrib.cursorUndoRedoController";let ib=DD;class vLe extends de{constructor(){super({id:"cursorUndo",label:m("cursor.undo","Cursor Undo"),alias:"Cursor Undo",precondition:void 0,kbOpts:{kbExpr:I.textInputFocus,primary:2099,weight:100}})}run(e,t,i){var n;(n=ib.get(t))==null||n.cursorUndo()}}class wLe extends de{constructor(){super({id:"cursorRedo",label:m("cursor.redo","Cursor Redo"),alias:"Cursor Redo",precondition:void 0})}run(e,t,i){var n;(n=ib.get(t))==null||n.cursorRedo()}}ct(ib.ID,ib,0);Q(vLe);Q(wLe);class SLe{constructor(e,t,i){this.selection=e,this.targetPosition=t,this.copy=i,this.targetSelection=null}getEditOperations(e,t){const i=e.getValueInRange(this.selection);if(this.copy||t.addEditOperation(this.selection,null),t.addEditOperation(new x(this.targetPosition.lineNumber,this.targetPosition.column,this.targetPosition.lineNumber,this.targetPosition.column),i),this.selection.containsPosition(this.targetPosition)&&!(this.copy&&(this.selection.getEndPosition().equals(this.targetPosition)||this.selection.getStartPosition().equals(this.targetPosition)))){this.targetSelection=this.selection;return}if(this.copy){this.targetSelection=new pe(this.targetPosition.lineNumber,this.targetPosition.column,this.selection.endLineNumber-this.selection.startLineNumber+this.targetPosition.lineNumber,this.selection.startLineNumber===this.selection.endLineNumber?this.targetPosition.column+this.selection.endColumn-this.selection.startColumn:this.selection.endColumn);return}if(this.targetPosition.lineNumber>this.selection.endLineNumber){this.targetSelection=new pe(this.targetPosition.lineNumber-this.selection.endLineNumber+this.selection.startLineNumber,this.targetPosition.column,this.targetPosition.lineNumber,this.selection.startLineNumber===this.selection.endLineNumber?this.targetPosition.column+this.selection.endColumn-this.selection.startColumn:this.selection.endColumn);return}if(this.targetPosition.lineNumber<this.selection.endLineNumber){this.targetSelection=new pe(this.targetPosition.lineNumber,this.targetPosition.column,this.targetPosition.lineNumber+this.selection.endLineNumber-this.selection.startLineNumber,this.selection.startLineNumber===this.selection.endLineNumber?this.targetPosition.column+this.selection.endColumn-this.selection.startColumn:this.selection.endColumn);return}this.selection.endColumn<=this.targetPosition.column?this.targetSelection=new pe(this.targetPosition.lineNumber-this.selection.endLineNumber+this.selection.startLineNumber,this.selection.startLineNumber===this.selection.endLineNumber?this.targetPosition.column-this.selection.endColumn+this.selection.startColumn:this.targetPosition.column-this.selection.endColumn+this.selection.startColumn,this.targetPosition.lineNumber,this.selection.startLineNumber===this.selection.endLineNumber?this.targetPosition.column:this.selection.endColumn):this.targetSelection=new pe(this.targetPosition.lineNumber-this.selection.endLineNumber+this.selection.startLineNumber,this.targetPosition.column,this.targetPosition.lineNumber,this.targetPosition.column+this.selection.endColumn-this.selection.startColumn)}computeCursorState(e,t){return this.targetSelection}}function Om(o){return Je?o.altKey:o.ctrlKey}const Od=class Od extends F{constructor(e){super(),this._editor=e,this._dndDecorationIds=this._editor.createDecorationsCollection(),this._register(this._editor.onMouseDown(t=>this._onEditorMouseDown(t))),this._register(this._editor.onMouseUp(t=>this._onEditorMouseUp(t))),this._register(this._editor.onMouseDrag(t=>this._onEditorMouseDrag(t))),this._register(this._editor.onMouseDrop(t=>this._onEditorMouseDrop(t))),this._register(this._editor.onMouseDropCanceled(()=>this._onEditorMouseDropCanceled())),this._register(this._editor.onKeyDown(t=>this.onEditorKeyDown(t))),this._register(this._editor.onKeyUp(t=>this.onEditorKeyUp(t))),this._register(this._editor.onDidBlurEditorWidget(()=>this.onEditorBlur())),this._register(this._editor.onDidBlurEditorText(()=>this.onEditorBlur())),this._mouseDown=!1,this._modifierPressed=!1,this._dragSelection=null}onEditorBlur(){this._removeDecoration(),this._dragSelection=null,this._mouseDown=!1,this._modifierPressed=!1}onEditorKeyDown(e){!this._editor.getOption(35)||this._editor.getOption(22)||(Om(e)&&(this._modifierPressed=!0),this._mouseDown&&Om(e)&&this._editor.updateOptions({mouseStyle:"copy"}))}onEditorKeyUp(e){!this._editor.getOption(35)||this._editor.getOption(22)||(Om(e)&&(this._modifierPressed=!1),this._mouseDown&&e.keyCode===Od.TRIGGER_KEY_VALUE&&this._editor.updateOptions({mouseStyle:"default"}))}_onEditorMouseDown(e){this._mouseDown=!0}_onEditorMouseUp(e){this._mouseDown=!1,this._editor.updateOptions({mouseStyle:"text"})}_onEditorMouseDrag(e){const t=e.target;if(this._dragSelection===null){const n=(this._editor.getSelections()||[]).filter(s=>t.position&&s.containsPosition(t.position));if(n.length===1)this._dragSelection=n[0];else return}Om(e.event)?this._editor.updateOptions({mouseStyle:"copy"}):this._editor.updateOptions({mouseStyle:"default"}),t.position&&(this._dragSelection.containsPosition(t.position)?this._removeDecoration():this.showAt(t.position))}_onEditorMouseDropCanceled(){this._editor.updateOptions({mouseStyle:"text"}),this._removeDecoration(),this._dragSelection=null,this._mouseDown=!1}_onEditorMouseDrop(e){if(e.target&&(this._hitContent(e.target)||this._hitMargin(e.target))&&e.target.position){const t=new A(e.target.position.lineNumber,e.target.position.column);if(this._dragSelection===null){let i=null;if(e.event.shiftKey){const n=this._editor.getSelection();if(n){const{selectionStartLineNumber:s,selectionStartColumn:r}=n;i=[new pe(s,r,t.lineNumber,t.column)]}}else i=(this._editor.getSelections()||[]).map(n=>n.containsPosition(t)?new pe(t.lineNumber,t.column,t.lineNumber,t.column):n);this._editor.setSelections(i||[],"mouse",3)}else(!this._dragSelection.containsPosition(t)||(Om(e.event)||this._modifierPressed)&&(this._dragSelection.getEndPosition().equals(t)||this._dragSelection.getStartPosition().equals(t)))&&(this._editor.pushUndoStop(),this._editor.executeCommand(Od.ID,new SLe(this._dragSelection,t,Om(e.event)||this._modifierPressed)),this._editor.pushUndoStop())}this._editor.updateOptions({mouseStyle:"text"}),this._removeDecoration(),this._dragSelection=null,this._mouseDown=!1}showAt(e){this._dndDecorationIds.set([{range:new x(e.lineNumber,e.column,e.lineNumber,e.column),options:Od._DECORATION_OPTIONS}]),this._editor.revealPosition(e,1)}_removeDecoration(){this._dndDecorationIds.clear()}_hitContent(e){return e.type===6||e.type===7}_hitMargin(e){return e.type===2||e.type===3||e.type===4}dispose(){this._removeDecoration(),this._dragSelection=null,this._mouseDown=!1,this._modifierPressed=!1,super.dispose()}};Od.ID="editor.contrib.dragAndDrop",Od.TRIGGER_KEY_VALUE=Je?6:5,Od._DECORATION_OPTIONS=We.register({description:"dnd-target",className:"dnd-target"});let Ux=Od;ct(Ux.ID,Ux,2);ct(rd.ID,rd,0);Nb(JP);re(new class extends $i{constructor(){super({id:ZG,precondition:RW,kbOpts:{weight:100,primary:2137}})}runEditorCommand(o,e){var t;return(t=rd.get(e))==null?void 0:t.changePasteType()}});re(new class extends $i{constructor(){super({id:"editor.hidePasteWidget",precondition:RW,kbOpts:{weight:100,primary:9}})}runEditorCommand(o,e){var t;(t=rd.get(e))==null||t.clearWidgets()}});var Xg;Q((Xg=class extends de{constructor(){super({id:"editor.action.pasteAs",label:m("pasteAs","Paste As..."),alias:"Paste As...",precondition:I.writable,metadata:{description:"Paste as",args:[{name:"args",schema:Xg.argsSchema}]}})}run(e,t,i){var s;let n=typeof(i==null?void 0:i.kind)=="string"?i.kind:void 0;return!n&&i&&(n=typeof i.id=="string"?i.id:void 0),(s=rd.get(t))==null?void 0:s.pasteAs(n?new ti(n):void 0)}},Xg.argsSchema={type:"object",properties:{kind:{type:"string",description:m("pasteAs.kind","The kind of the paste edit to try applying. If not provided or there are multiple edits for this kind, the editor will show a picker.")}}},Xg));Q(class extends de{constructor(){super({id:"editor.action.pasteAsText",label:m("pasteAsText","Paste as Text"),alias:"Paste as Text",precondition:I.writable})}run(o,e){var t;return(t=rd.get(e))==null?void 0:t.pasteAs({providerId:Vf.id})}});class yLe{constructor(){this._dragOperations=new Map}removeDragOperationTransfer(e){if(e&&this._dragOperations.has(e)){const t=this._dragOperations.get(e);return this._dragOperations.delete(e),t}}}class ZV{constructor(e){this.identifier=e}}const qZ=et("treeViewsDndService");st(qZ,yLe,1);var LLe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},b0=function(o,e){return function(t,i){e(t,i,o)}},BO;const GZ="editor.experimental.dropIntoEditor.defaultProvider",ZZ="editor.changeDropType",qW=new le("dropWidgetVisible",!1,m("dropWidgetVisible","Whether the drop widget is showing"));var Qg;let nb=(Qg=class extends F{static get(e){return e.getContribution(BO.ID)}constructor(e,t,i,n,s){super(),this._configService=i,this._languageFeaturesService=n,this._treeViewsDragAndDropService=s,this.treeItemsTransfer=XP.getInstance(),this._dropProgressManager=this._register(t.createInstance(Nx,"dropIntoEditor",e)),this._postDropWidgetManager=this._register(t.createInstance(Rx,"dropIntoEditor",e,qW,{id:ZZ,label:m("postDropWidgetTitle","Show drop options...")})),this._register(e.onDropIntoEditor(r=>this.onDropIntoEditor(e,r.position,r.event)))}clearWidgets(){this._postDropWidgetManager.clear()}changeDropType(){this._postDropWidgetManager.tryShowSelector()}async onDropIntoEditor(e,t,i){var s;if(!i.dataTransfer||!e.hasModel())return;(s=this._currentOperation)==null||s.cancel(),e.focus(),e.setPosition(t);const n=Yi(async r=>{const a=new K,l=a.add(new Zh(e,1,void 0,r));try{const c=await this.extractDataTransferData(i);if(c.size===0||l.token.isCancellationRequested)return;const d=e.getModel();if(!d)return;const h=this._languageFeaturesService.documentDropEditProvider.ordered(d).filter(g=>g.dropMimeTypes?g.dropMimeTypes.some(f=>c.matches(f)):!0),u=a.add(await this.getDropEdits(h,d,t,c,l));if(l.token.isCancellationRequested)return;if(u.edits.length){const g=this.getInitialActiveEditIndex(d,u.edits),f=e.getOption(36).showDropSelector==="afterDrop";await this._postDropWidgetManager.applyEditAndShowIfNeeded([x.fromPositions(t)],{activeEditIndex:g,allEdits:u.edits},f,async p=>p,r)}}finally{a.dispose(),this._currentOperation===n&&(this._currentOperation=void 0)}});this._dropProgressManager.showWhile(t,m("dropIntoEditorProgress","Running drop handlers. Click to cancel"),n,{cancel:()=>n.cancel()}),this._currentOperation=n}async getDropEdits(e,t,i,n,s){const r=new K,a=await F1(Promise.all(e.map(async c=>{try{const d=await c.provideDocumentDropEdits(t,i,n,s.token);return d&&r.add(d),d==null?void 0:d.edits.map(h=>({...h,providerId:c.id}))}catch(d){console.error(d)}})),s.token),l=Kr(a??[]).flat();return{edits:qG(l),dispose:()=>r.dispose()}}getInitialActiveEditIndex(e,t){const i=this._configService.getValue(GZ,{resource:e.uri});for(const[n,s]of Object.entries(i)){const r=new ti(s),a=t.findIndex(l=>r.value===l.providerId&&l.handledMimeType&&BG(n,[l.handledMimeType]));if(a>=0)return a}return 0}async extractDataTransferData(e){if(!e.dataTransfer)return new WG;const t=zG(e.dataTransfer);if(this.treeItemsTransfer.hasData(ZV.prototype)){const i=this.treeItemsTransfer.getData(ZV.prototype);if(Array.isArray(i))for(const n of i){const s=await this._treeViewsDragAndDropService.removeDragOperationTransfer(n.identifier);if(s)for(const[r,a]of s)t.replace(r,a)}}return t}},BO=Qg,Qg.ID="editor.contrib.dropIntoEditorController",Qg);nb=BO=LLe([b0(1,we),b0(2,qe),b0(3,ge),b0(4,qZ)],nb);ct(nb.ID,nb,2);Nb(QP);re(new class extends $i{constructor(){super({id:ZZ,precondition:qW,kbOpts:{weight:100,primary:2137}})}runEditorCommand(o,e,t){var i;(i=nb.get(e))==null||i.changeDropType()}});re(new class extends $i{constructor(){super({id:"editor.hideDropWidget",precondition:qW,kbOpts:{weight:100,primary:9}})}runEditorCommand(o,e,t){var i;(i=nb.get(e))==null||i.clearWidgets()}});ci.as(ta.Configuration).registerConfiguration({...K1,properties:{[GZ]:{type:"object",scope:5,description:m("defaultProviderDescription","Configures the default drop provider to use for content of a given mime type."),default:{},additionalProperties:{type:"string"}}}});const Un=class Un{constructor(e){this._editor=e,this._decorations=[],this._overviewRulerApproximateDecorations=[],this._findScopeDecorationIds=[],this._rangeHighlightDecorationId=null,this._highlightedDecorationId=null,this._startPosition=this._editor.getPosition()}dispose(){this._editor.removeDecorations(this._allDecorations()),this._decorations=[],this._overviewRulerApproximateDecorations=[],this._findScopeDecorationIds=[],this._rangeHighlightDecorationId=null,this._highlightedDecorationId=null}reset(){this._decorations=[],this._overviewRulerApproximateDecorations=[],this._findScopeDecorationIds=[],this._rangeHighlightDecorationId=null,this._highlightedDecorationId=null}getCount(){return this._decorations.length}getFindScope(){return this._findScopeDecorationIds[0]?this._editor.getModel().getDecorationRange(this._findScopeDecorationIds[0]):null}getFindScopes(){if(this._findScopeDecorationIds.length){const e=this._findScopeDecorationIds.map(t=>this._editor.getModel().getDecorationRange(t)).filter(t=>!!t);if(e.length)return e}return null}getStartPosition(){return this._startPosition}setStartPosition(e){this._startPosition=e,this.setCurrentFindMatch(null)}_getDecorationIndex(e){const t=this._decorations.indexOf(e);return t>=0?t+1:1}getDecorationRangeAt(e){const t=e<this._decorations.length?this._decorations[e]:null;return t?this._editor.getModel().getDecorationRange(t):null}getCurrentMatchesPosition(e){const t=this._editor.getModel().getDecorationsInRange(e);for(const i of t){const n=i.options;if(n===Un._FIND_MATCH_DECORATION||n===Un._CURRENT_FIND_MATCH_DECORATION)return this._getDecorationIndex(i.id)}return 0}setCurrentFindMatch(e){let t=null,i=0;if(e)for(let n=0,s=this._decorations.length;n<s;n++){const r=this._editor.getModel().getDecorationRange(this._decorations[n]);if(e.equalsRange(r)){t=this._decorations[n],i=n+1;break}}return(this._highlightedDecorationId!==null||t!==null)&&this._editor.changeDecorations(n=>{if(this._highlightedDecorationId!==null&&(n.changeDecorationOptions(this._highlightedDecorationId,Un._FIND_MATCH_DECORATION),this._highlightedDecorationId=null),t!==null&&(this._highlightedDecorationId=t,n.changeDecorationOptions(this._highlightedDecorationId,Un._CURRENT_FIND_MATCH_DECORATION)),this._rangeHighlightDecorationId!==null&&(n.removeDecoration(this._rangeHighlightDecorationId),this._rangeHighlightDecorationId=null),t!==null){let s=this._editor.getModel().getDecorationRange(t);if(s.startLineNumber!==s.endLineNumber&&s.endColumn===1){const r=s.endLineNumber-1,a=this._editor.getModel().getLineMaxColumn(r);s=new x(s.startLineNumber,s.startColumn,r,a)}this._rangeHighlightDecorationId=n.addDecoration(s,Un._RANGE_HIGHLIGHT_DECORATION)}}),i}set(e,t){this._editor.changeDecorations(i=>{let n=Un._FIND_MATCH_DECORATION;const s=[];if(e.length>1e3){n=Un._FIND_MATCH_NO_OVERVIEW_DECORATION;const a=this._editor.getModel().getLineCount(),c=this._editor.getLayoutInfo().height/a,d=Math.max(2,Math.ceil(3/c));let h=e[0].range.startLineNumber,u=e[0].range.endLineNumber;for(let g=1,f=e.length;g<f;g++){const p=e[g].range;u+d>=p.startLineNumber?p.endLineNumber>u&&(u=p.endLineNumber):(s.push({range:new x(h,1,u,1),options:Un._FIND_MATCH_ONLY_OVERVIEW_DECORATION}),h=p.startLineNumber,u=p.endLineNumber)}s.push({range:new x(h,1,u,1),options:Un._FIND_MATCH_ONLY_OVERVIEW_DECORATION})}const r=new Array(e.length);for(let a=0,l=e.length;a<l;a++)r[a]={range:e[a].range,options:n};this._decorations=i.deltaDecorations(this._decorations,r),this._overviewRulerApproximateDecorations=i.deltaDecorations(this._overviewRulerApproximateDecorations,s),this._rangeHighlightDecorationId&&(i.removeDecoration(this._rangeHighlightDecorationId),this._rangeHighlightDecorationId=null),this._findScopeDecorationIds.length&&(this._findScopeDecorationIds.forEach(a=>i.removeDecoration(a)),this._findScopeDecorationIds=[]),t!=null&&t.length&&(this._findScopeDecorationIds=t.map(a=>i.addDecoration(a,Un._FIND_SCOPE_DECORATION)))})}matchBeforePosition(e){if(this._decorations.length===0)return null;for(let t=this._decorations.length-1;t>=0;t--){const i=this._decorations[t],n=this._editor.getModel().getDecorationRange(i);if(!(!n||n.endLineNumber>e.lineNumber)){if(n.endLineNumber<e.lineNumber)return n;if(!(n.endColumn>e.column))return n}}return this._editor.getModel().getDecorationRange(this._decorations[this._decorations.length-1])}matchAfterPosition(e){if(this._decorations.length===0)return null;for(let t=0,i=this._decorations.length;t<i;t++){const n=this._decorations[t],s=this._editor.getModel().getDecorationRange(n);if(!(!s||s.startLineNumber<e.lineNumber)){if(s.startLineNumber>e.lineNumber)return s;if(!(s.startColumn<e.column))return s}}return this._editor.getModel().getDecorationRange(this._decorations[0])}_allDecorations(){let e=[];return e=e.concat(this._decorations),e=e.concat(this._overviewRulerApproximateDecorations),this._findScopeDecorationIds.length&&e.push(...this._findScopeDecorationIds),this._rangeHighlightDecorationId&&e.push(this._rangeHighlightDecorationId),e}};Un._CURRENT_FIND_MATCH_DECORATION=We.register({description:"current-find-match",stickiness:1,zIndex:13,className:"currentFindMatch",inlineClassName:"currentFindMatchInline",showIfCollapsed:!0,overviewRuler:{color:mi(FN),position:Ho.Center},minimap:{color:mi(u2),position:1}}),Un._FIND_MATCH_DECORATION=We.register({description:"find-match",stickiness:1,zIndex:10,className:"findMatch",inlineClassName:"findMatchInline",showIfCollapsed:!0,overviewRuler:{color:mi(FN),position:Ho.Center},minimap:{color:mi(u2),position:1}}),Un._FIND_MATCH_NO_OVERVIEW_DECORATION=We.register({description:"find-match-no-overview",stickiness:1,className:"findMatch",showIfCollapsed:!0}),Un._FIND_MATCH_ONLY_OVERVIEW_DECORATION=We.register({description:"find-match-only-overview",stickiness:1,overviewRuler:{color:mi(FN),position:Ho.Center}}),Un._RANGE_HIGHLIGHT_DECORATION=We.register({description:"find-range-highlight",stickiness:1,className:"rangeHighlight",isWholeLine:!0}),Un._FIND_SCOPE_DECORATION=We.register({description:"find-scope",className:"findScope",isWholeLine:!0});let HO=Un;class xLe{constructor(e,t,i){this._editorSelection=e,this._ranges=t,this._replaceStrings=i,this._trackedEditorSelectionId=null}getEditOperations(e,t){if(this._ranges.length>0){const i=[];for(let r=0;r<this._ranges.length;r++)i.push({range:this._ranges[r],text:this._replaceStrings[r]});i.sort((r,a)=>x.compareRangesUsingStarts(r.range,a.range));const n=[];let s=i[0];for(let r=1;r<i.length;r++)s.range.endLineNumber===i[r].range.startLineNumber&&s.range.endColumn===i[r].range.startColumn?(s.range=s.range.plusRange(i[r].range),s.text=s.text+i[r].text):(n.push(s),s=i[r]);n.push(s);for(const r of n)t.addEditOperation(r.range,r.text)}this._trackedEditorSelectionId=t.trackSelection(this._editorSelection)}computeCursorState(e,t){return t.getTrackedSelection(this._trackedEditorSelectionId)}}function YZ(o,e){if(o&&o[0]!==""){const t=YV(o,e,"-"),i=YV(o,e,"_");return t&&!i?XV(o,e,"-"):!t&&i?XV(o,e,"_"):o[0].toUpperCase()===o[0]?e.toUpperCase():o[0].toLowerCase()===o[0]?e.toLowerCase():Bee(o[0][0])&&e.length>0?e[0].toUpperCase()+e.substr(1):o[0][0].toUpperCase()!==o[0][0]&&e.length>0?e[0].toLowerCase()+e.substr(1):e}else return e}function YV(o,e,t){return o[0].indexOf(t)!==-1&&e.indexOf(t)!==-1&&o[0].split(t).length===e.split(t).length}function XV(o,e,t){const i=e.split(t),n=o[0].split(t);let s="";return i.forEach((r,a)=>{s+=YZ([n[a]],r)+t}),s.slice(0,-1)}class QV{constructor(e){this.staticValue=e,this.kind=0}}class kLe{constructor(e){this.pieces=e,this.kind=1}}class sb{static fromStaticValue(e){return new sb([Lg.staticValue(e)])}get hasReplacementPatterns(){return this._state.kind===1}constructor(e){!e||e.length===0?this._state=new QV(""):e.length===1&&e[0].staticValue!==null?this._state=new QV(e[0].staticValue):this._state=new kLe(e)}buildReplaceString(e,t){if(this._state.kind===0)return t?YZ(e,this._state.staticValue):this._state.staticValue;let i="";for(let n=0,s=this._state.pieces.length;n<s;n++){const r=this._state.pieces[n];if(r.staticValue!==null){i+=r.staticValue;continue}let a=sb._substitute(r.matchIndex,e);if(r.caseOps!==null&&r.caseOps.length>0){const l=[],c=r.caseOps.length;let d=0;for(let h=0,u=a.length;h<u;h++){if(d>=c){l.push(a.slice(h));break}switch(r.caseOps[d]){case"U":l.push(a[h].toUpperCase());break;case"u":l.push(a[h].toUpperCase()),d++;break;case"L":l.push(a[h].toLowerCase());break;case"l":l.push(a[h].toLowerCase()),d++;break;default:l.push(a[h])}}a=l.join("")}i+=a}return i}static _substitute(e,t){if(t===null)return"";if(e===0)return t[0];let i="";for(;e>0;){if(e<t.length)return(t[e]||"")+i;i=String(e%10)+i,e=Math.floor(e/10)}return"$"+i}}class Lg{static staticValue(e){return new Lg(e,-1,null)}static caseOps(e,t){return new Lg(null,e,t)}constructor(e,t,i){this.staticValue=e,this.matchIndex=t,!i||i.length===0?this.caseOps=null:this.caseOps=i.slice(0)}}class DLe{constructor(e){this._source=e,this._lastCharIndex=0,this._result=[],this._resultLen=0,this._currentStaticPiece=""}emitUnchanged(e){this._emitStatic(this._source.substring(this._lastCharIndex,e)),this._lastCharIndex=e}emitStatic(e,t){this._emitStatic(e),this._lastCharIndex=t}_emitStatic(e){e.length!==0&&(this._currentStaticPiece+=e)}emitMatchIndex(e,t,i){this._currentStaticPiece.length!==0&&(this._result[this._resultLen++]=Lg.staticValue(this._currentStaticPiece),this._currentStaticPiece=""),this._result[this._resultLen++]=Lg.caseOps(e,i),this._lastCharIndex=t}finalize(){return this.emitUnchanged(this._source.length),this._currentStaticPiece.length!==0&&(this._result[this._resultLen++]=Lg.staticValue(this._currentStaticPiece),this._currentStaticPiece=""),new sb(this._result)}}function ILe(o){if(!o||o.length===0)return new sb(null);const e=[],t=new DLe(o);for(let i=0,n=o.length;i<n;i++){const s=o.charCodeAt(i);if(s===92){if(i++,i>=n)break;const r=o.charCodeAt(i);switch(r){case 92:t.emitUnchanged(i-1),t.emitStatic("\\",i+1);break;case 110:t.emitUnchanged(i-1),t.emitStatic(`
`,i+1);break;case 116:t.emitUnchanged(i-1),t.emitStatic("	",i+1);break;case 117:case 85:case 108:case 76:t.emitUnchanged(i-1),t.emitStatic("",i+1),e.push(String.fromCharCode(r));break}continue}if(s===36){if(i++,i>=n)break;const r=o.charCodeAt(i);if(r===36){t.emitUnchanged(i-1),t.emitStatic("$",i+1);continue}if(r===48||r===38){t.emitUnchanged(i-1),t.emitMatchIndex(0,i+1,e),e.length=0;continue}if(49<=r&&r<=57){let a=r-48;if(i+1<n){const l=o.charCodeAt(i+1);if(48<=l&&l<=57){i++,a=a*10+(l-48),t.emitUnchanged(i-2),t.emitMatchIndex(a,i+1,e),e.length=0;continue}}t.emitUnchanged(i-1),t.emitMatchIndex(a,i+1,e),e.length=0;continue}}}return t.finalize()}const pd=new le("findWidgetVisible",!1);pd.toNegated();const WE=new le("findInputFocussed",!1),GW=new le("replaceInputFocussed",!1),C0={primary:545,mac:{primary:2593}},v0={primary:565,mac:{primary:2613}},w0={primary:560,mac:{primary:2608}},S0={primary:554,mac:{primary:2602}},y0={primary:558,mac:{primary:2606}},Ut={StartFindAction:"actions.find",StartFindWithSelection:"actions.findWithSelection",StartFindWithArgs:"editor.actions.findWithArgs",NextMatchFindAction:"editor.action.nextMatchFindAction",PreviousMatchFindAction:"editor.action.previousMatchFindAction",GoToMatchFindAction:"editor.action.goToMatchFindAction",NextSelectionMatchFindAction:"editor.action.nextSelectionMatchFindAction",PreviousSelectionMatchFindAction:"editor.action.previousSelectionMatchFindAction",StartFindReplaceAction:"editor.action.startFindReplaceAction",CloseFindWidgetCommand:"closeFindWidget",ToggleCaseSensitiveCommand:"toggleFindCaseSensitive",ToggleWholeWordCommand:"toggleFindWholeWord",ToggleRegexCommand:"toggleFindRegex",ToggleSearchScopeCommand:"toggleFindInSelection",TogglePreserveCaseCommand:"togglePreserveCase",ReplaceOneAction:"editor.action.replaceOne",ReplaceAllAction:"editor.action.replaceAll",SelectAllMatchesAction:"editor.action.selectAllMatches"},sh=19999,ELe=240;class wv{constructor(e,t){this._toDispose=new K,this._editor=e,this._state=t,this._isDisposed=!1,this._startSearchingTimer=new ea,this._decorations=new HO(e),this._toDispose.add(this._decorations),this._updateDecorationsScheduler=new It(()=>{if(this._editor.hasModel())return this.research(!1)},100),this._toDispose.add(this._updateDecorationsScheduler),this._toDispose.add(this._editor.onDidChangeCursorPosition(i=>{(i.reason===3||i.reason===5||i.reason===6)&&this._decorations.setStartPosition(this._editor.getPosition())})),this._ignoreModelContentChanged=!1,this._toDispose.add(this._editor.onDidChangeModelContent(i=>{this._ignoreModelContentChanged||(i.isFlush&&this._decorations.reset(),this._decorations.setStartPosition(this._editor.getPosition()),this._updateDecorationsScheduler.schedule())})),this._toDispose.add(this._state.onFindReplaceStateChange(i=>this._onStateChanged(i))),this.research(!1,this._state.searchScope)}dispose(){this._isDisposed=!0,Tt(this._startSearchingTimer),this._toDispose.dispose()}_onStateChanged(e){this._isDisposed||this._editor.hasModel()&&(e.searchString||e.isReplaceRevealed||e.isRegex||e.wholeWord||e.matchCase||e.searchScope)&&(this._editor.getModel().isTooLargeForSyncing()?(this._startSearchingTimer.cancel(),this._startSearchingTimer.setIfNotSet(()=>{e.searchScope?this.research(e.moveCursor,this._state.searchScope):this.research(e.moveCursor)},ELe)):e.searchScope?this.research(e.moveCursor,this._state.searchScope):this.research(e.moveCursor))}static _getSearchRange(e,t){return t||e.getFullModelRange()}research(e,t){let i=null;typeof t<"u"?t!==null&&(Array.isArray(t)?i=t:i=[t]):i=this._decorations.getFindScopes(),i!==null&&(i=i.map(a=>{if(a.startLineNumber!==a.endLineNumber){let l=a.endLineNumber;return a.endColumn===1&&(l=l-1),new x(a.startLineNumber,1,l,this._editor.getModel().getLineMaxColumn(l))}return a}));const n=this._findMatches(i,!1,sh);this._decorations.set(n,i);const s=this._editor.getSelection();let r=this._decorations.getCurrentMatchesPosition(s);if(r===0&&n.length>0){const a=uw(n.map(l=>l.range),l=>x.compareRangesUsingStarts(l,s)>=0);r=a>0?a-1+1:r}this._state.changeMatchInfo(r,this._decorations.getCount(),void 0),e&&this._editor.getOption(41).cursorMoveOnType&&this._moveToNextMatch(this._decorations.getStartPosition())}_hasMatches(){return this._state.matchesCount>0}_cannotFind(){if(!this._hasMatches()){const e=this._decorations.getFindScope();return e&&this._editor.revealRangeInCenterIfOutsideViewport(e,0),!0}return!1}_setCurrentFindMatch(e){const t=this._decorations.setCurrentFindMatch(e);this._state.changeMatchInfo(t,this._decorations.getCount(),e),this._editor.setSelection(e),this._editor.revealRangeInCenterIfOutsideViewport(e,0)}_prevSearchPosition(e){const t=this._state.isRegex&&(this._state.searchString.indexOf("^")>=0||this._state.searchString.indexOf("$")>=0);let{lineNumber:i,column:n}=e;const s=this._editor.getModel();return t||n===1?(i===1?i=s.getLineCount():i--,n=s.getLineMaxColumn(i)):n--,new A(i,n)}_moveToPrevMatch(e,t=!1){if(!this._state.canNavigateBack()){const d=this._decorations.matchAfterPosition(e);d&&this._setCurrentFindMatch(d);return}if(this._decorations.getCount()<sh){let d=this._decorations.matchBeforePosition(e);d&&d.isEmpty()&&d.getStartPosition().equals(e)&&(e=this._prevSearchPosition(e),d=this._decorations.matchBeforePosition(e)),d&&this._setCurrentFindMatch(d);return}if(this._cannotFind())return;const i=this._decorations.getFindScope(),n=wv._getSearchRange(this._editor.getModel(),i);n.getEndPosition().isBefore(e)&&(e=n.getEndPosition()),e.isBefore(n.getStartPosition())&&(e=n.getEndPosition());const{lineNumber:s,column:r}=e,a=this._editor.getModel();let l=new A(s,r),c=a.findPreviousMatch(this._state.searchString,l,this._state.isRegex,this._state.matchCase,this._state.wholeWord?this._editor.getOption(132):null,!1);if(c&&c.range.isEmpty()&&c.range.getStartPosition().equals(l)&&(l=this._prevSearchPosition(l),c=a.findPreviousMatch(this._state.searchString,l,this._state.isRegex,this._state.matchCase,this._state.wholeWord?this._editor.getOption(132):null,!1)),!!c){if(!t&&!n.containsRange(c.range))return this._moveToPrevMatch(c.range.getStartPosition(),!0);this._setCurrentFindMatch(c.range)}}moveToPrevMatch(){this._moveToPrevMatch(this._editor.getSelection().getStartPosition())}_nextSearchPosition(e){const t=this._state.isRegex&&(this._state.searchString.indexOf("^")>=0||this._state.searchString.indexOf("$")>=0);let{lineNumber:i,column:n}=e;const s=this._editor.getModel();return t||n===s.getLineMaxColumn(i)?(i===s.getLineCount()?i=1:i++,n=1):n++,new A(i,n)}_moveToNextMatch(e){if(!this._state.canNavigateForward()){const i=this._decorations.matchBeforePosition(e);i&&this._setCurrentFindMatch(i);return}if(this._decorations.getCount()<sh){let i=this._decorations.matchAfterPosition(e);i&&i.isEmpty()&&i.getStartPosition().equals(e)&&(e=this._nextSearchPosition(e),i=this._decorations.matchAfterPosition(e)),i&&this._setCurrentFindMatch(i);return}const t=this._getNextMatch(e,!1,!0);t&&this._setCurrentFindMatch(t.range)}_getNextMatch(e,t,i,n=!1){if(this._cannotFind())return null;const s=this._decorations.getFindScope(),r=wv._getSearchRange(this._editor.getModel(),s);r.getEndPosition().isBefore(e)&&(e=r.getStartPosition()),e.isBefore(r.getStartPosition())&&(e=r.getStartPosition());const{lineNumber:a,column:l}=e,c=this._editor.getModel();let d=new A(a,l),h=c.findNextMatch(this._state.searchString,d,this._state.isRegex,this._state.matchCase,this._state.wholeWord?this._editor.getOption(132):null,t);return i&&h&&h.range.isEmpty()&&h.range.getStartPosition().equals(d)&&(d=this._nextSearchPosition(d),h=c.findNextMatch(this._state.searchString,d,this._state.isRegex,this._state.matchCase,this._state.wholeWord?this._editor.getOption(132):null,t)),h?!n&&!r.containsRange(h.range)?this._getNextMatch(h.range.getEndPosition(),t,i,!0):h:null}moveToNextMatch(){this._moveToNextMatch(this._editor.getSelection().getEndPosition())}_moveToMatch(e){const t=this._decorations.getDecorationRangeAt(e);t&&this._setCurrentFindMatch(t)}moveToMatch(e){this._moveToMatch(e)}_getReplacePattern(){return this._state.isRegex?ILe(this._state.replaceString):sb.fromStaticValue(this._state.replaceString)}replace(){if(!this._hasMatches())return;const e=this._getReplacePattern(),t=this._editor.getSelection(),i=this._getNextMatch(t.getStartPosition(),!0,!1);if(i)if(t.equalsRange(i.range)){const n=e.buildReplaceString(i.matches,this._state.preserveCase),s=new kn(t,n);this._executeEditorCommand("replace",s),this._decorations.setStartPosition(new A(t.startLineNumber,t.startColumn+n.length)),this.research(!0)}else this._decorations.setStartPosition(this._editor.getPosition()),this._setCurrentFindMatch(i.range)}_findMatches(e,t,i){const n=(e||[null]).map(s=>wv._getSearchRange(this._editor.getModel(),s));return this._editor.getModel().findMatches(this._state.searchString,n,this._state.isRegex,this._state.matchCase,this._state.wholeWord?this._editor.getOption(132):null,t,i)}replaceAll(){if(!this._hasMatches())return;const e=this._decorations.getFindScopes();e===null&&this._state.matchesCount>=sh?this._largeReplaceAll():this._regularReplaceAll(e),this.research(!1)}_largeReplaceAll(){const t=new Tu(this._state.searchString,this._state.isRegex,this._state.matchCase,this._state.wholeWord?this._editor.getOption(132):null).parseSearchRequest();if(!t)return;let i=t.regex;if(!i.multiline){let h="mu";i.ignoreCase&&(h+="i"),i.global&&(h+="g"),i=new RegExp(i.source,h)}const n=this._editor.getModel(),s=n.getValue(1),r=n.getFullModelRange(),a=this._getReplacePattern();let l;const c=this._state.preserveCase;a.hasReplacementPatterns||c?l=s.replace(i,function(){return a.buildReplaceString(arguments,c)}):l=s.replace(i,a.buildReplaceString(null,c));const d=new j3(r,l,this._editor.getSelection());this._executeEditorCommand("replaceAll",d)}_regularReplaceAll(e){const t=this._getReplacePattern(),i=this._findMatches(e,t.hasReplacementPatterns||this._state.preserveCase,1073741824),n=[];for(let r=0,a=i.length;r<a;r++)n[r]=t.buildReplaceString(i[r].matches,this._state.preserveCase);const s=new xLe(this._editor.getSelection(),i.map(r=>r.range),n);this._executeEditorCommand("replaceAll",s)}selectAllMatches(){if(!this._hasMatches())return;const e=this._decorations.getFindScopes();let i=this._findMatches(e,!1,1073741824).map(s=>new pe(s.range.startLineNumber,s.range.startColumn,s.range.endLineNumber,s.range.endColumn));const n=this._editor.getSelection();for(let s=0,r=i.length;s<r;s++)if(i[s].equalsRange(n)){i=[n].concat(i.slice(0,s)).concat(i.slice(s+1));break}this._editor.setSelections(i)}_executeEditorCommand(e,t){try{this._ignoreModelContentChanged=!0,this._editor.pushUndoStop(),this._editor.executeCommand(e,t),this._editor.pushUndoStop()}finally{this._ignoreModelContentChanged=!1}}}const ID=class ID extends Yo{constructor(e,t,i){super(),this._hideSoon=this._register(new It(()=>this._hide(),2e3)),this._isVisible=!1,this._editor=e,this._state=t,this._keybindingService=i,this._domNode=document.createElement("div"),this._domNode.className="findOptionsWidget",this._domNode.style.display="none",this._domNode.style.top="10px",this._domNode.style.zIndex="12",this._domNode.setAttribute("role","presentation"),this._domNode.setAttribute("aria-hidden","true");const n={inputActiveOptionBorder:ce(zI),inputActiveOptionForeground:ce(UI),inputActiveOptionBackground:ce(V1)},s=this._register(H_());this.caseSensitive=this._register(new wK({appendTitle:this._keybindingLabelFor(Ut.ToggleCaseSensitiveCommand),isChecked:this._state.matchCase,hoverDelegate:s,...n})),this._domNode.appendChild(this.caseSensitive.domNode),this._register(this.caseSensitive.onChange(()=>{this._state.change({matchCase:this.caseSensitive.checked},!1)})),this.wholeWords=this._register(new SK({appendTitle:this._keybindingLabelFor(Ut.ToggleWholeWordCommand),isChecked:this._state.wholeWord,hoverDelegate:s,...n})),this._domNode.appendChild(this.wholeWords.domNode),this._register(this.wholeWords.onChange(()=>{this._state.change({wholeWord:this.wholeWords.checked},!1)})),this.regex=this._register(new yK({appendTitle:this._keybindingLabelFor(Ut.ToggleRegexCommand),isChecked:this._state.isRegex,hoverDelegate:s,...n})),this._domNode.appendChild(this.regex.domNode),this._register(this.regex.onChange(()=>{this._state.change({isRegex:this.regex.checked},!1)})),this._editor.addOverlayWidget(this),this._register(this._state.onFindReplaceStateChange(r=>{let a=!1;r.isRegex&&(this.regex.checked=this._state.isRegex,a=!0),r.wholeWord&&(this.wholeWords.checked=this._state.wholeWord,a=!0),r.matchCase&&(this.caseSensitive.checked=this._state.matchCase,a=!0),!this._state.isRevealed&&a&&this._revealTemporarily()})),this._register(U(this._domNode,ie.MOUSE_LEAVE,r=>this._onMouseLeave())),this._register(U(this._domNode,"mouseover",r=>this._onMouseOver()))}_keybindingLabelFor(e){const t=this._keybindingService.lookupKeybinding(e);return t?` (${t.getLabel()})`:""}dispose(){this._editor.removeOverlayWidget(this),super.dispose()}getId(){return ID.ID}getDomNode(){return this._domNode}getPosition(){return{preference:0}}highlightFindOptions(){this._revealTemporarily()}_revealTemporarily(){this._show(),this._hideSoon.schedule()}_onMouseLeave(){this._hideSoon.schedule()}_onMouseOver(){this._hideSoon.cancel()}_show(){this._isVisible||(this._isVisible=!0,this._domNode.style.display="block")}_hide(){this._isVisible&&(this._isVisible=!1,this._domNode.style.display="none")}};ID.ID="editor.contrib.findOptionsWidget";let VO=ID;function L0(o,e){return o===1?!0:o===2?!1:e}class NLe extends F{get searchString(){return this._searchString}get replaceString(){return this._replaceString}get isRevealed(){return this._isRevealed}get isReplaceRevealed(){return this._isReplaceRevealed}get isRegex(){return L0(this._isRegexOverride,this._isRegex)}get wholeWord(){return L0(this._wholeWordOverride,this._wholeWord)}get matchCase(){return L0(this._matchCaseOverride,this._matchCase)}get preserveCase(){return L0(this._preserveCaseOverride,this._preserveCase)}get actualIsRegex(){return this._isRegex}get actualWholeWord(){return this._wholeWord}get actualMatchCase(){return this._matchCase}get actualPreserveCase(){return this._preserveCase}get searchScope(){return this._searchScope}get matchesPosition(){return this._matchesPosition}get matchesCount(){return this._matchesCount}get currentMatch(){return this._currentMatch}constructor(){super(),this._onFindReplaceStateChange=this._register(new P),this.onFindReplaceStateChange=this._onFindReplaceStateChange.event,this._searchString="",this._replaceString="",this._isRevealed=!1,this._isReplaceRevealed=!1,this._isRegex=!1,this._isRegexOverride=0,this._wholeWord=!1,this._wholeWordOverride=0,this._matchCase=!1,this._matchCaseOverride=0,this._preserveCase=!1,this._preserveCaseOverride=0,this._searchScope=null,this._matchesPosition=0,this._matchesCount=0,this._currentMatch=null,this._loop=!0,this._isSearching=!1,this._filters=null}changeMatchInfo(e,t,i){const n={moveCursor:!1,updateHistory:!1,searchString:!1,replaceString:!1,isRevealed:!1,isReplaceRevealed:!1,isRegex:!1,wholeWord:!1,matchCase:!1,preserveCase:!1,searchScope:!1,matchesPosition:!1,matchesCount:!1,currentMatch:!1,loop:!1,isSearching:!1,filters:!1};let s=!1;t===0&&(e=0),e>t&&(e=t),this._matchesPosition!==e&&(this._matchesPosition=e,n.matchesPosition=!0,s=!0),this._matchesCount!==t&&(this._matchesCount=t,n.matchesCount=!0,s=!0),typeof i<"u"&&(x.equalsRange(this._currentMatch,i)||(this._currentMatch=i,n.currentMatch=!0,s=!0)),s&&this._onFindReplaceStateChange.fire(n)}change(e,t,i=!0){var d;const n={moveCursor:t,updateHistory:i,searchString:!1,replaceString:!1,isRevealed:!1,isReplaceRevealed:!1,isRegex:!1,wholeWord:!1,matchCase:!1,preserveCase:!1,searchScope:!1,matchesPosition:!1,matchesCount:!1,currentMatch:!1,loop:!1,isSearching:!1,filters:!1};let s=!1;const r=this.isRegex,a=this.wholeWord,l=this.matchCase,c=this.preserveCase;typeof e.searchString<"u"&&this._searchString!==e.searchString&&(this._searchString=e.searchString,n.searchString=!0,s=!0),typeof e.replaceString<"u"&&this._replaceString!==e.replaceString&&(this._replaceString=e.replaceString,n.replaceString=!0,s=!0),typeof e.isRevealed<"u"&&this._isRevealed!==e.isRevealed&&(this._isRevealed=e.isRevealed,n.isRevealed=!0,s=!0),typeof e.isReplaceRevealed<"u"&&this._isReplaceRevealed!==e.isReplaceRevealed&&(this._isReplaceRevealed=e.isReplaceRevealed,n.isReplaceRevealed=!0,s=!0),typeof e.isRegex<"u"&&(this._isRegex=e.isRegex),typeof e.wholeWord<"u"&&(this._wholeWord=e.wholeWord),typeof e.matchCase<"u"&&(this._matchCase=e.matchCase),typeof e.preserveCase<"u"&&(this._preserveCase=e.preserveCase),typeof e.searchScope<"u"&&((d=e.searchScope)!=null&&d.every(h=>{var u;return(u=this._searchScope)==null?void 0:u.some(g=>!x.equalsRange(g,h))})||(this._searchScope=e.searchScope,n.searchScope=!0,s=!0)),typeof e.loop<"u"&&this._loop!==e.loop&&(this._loop=e.loop,n.loop=!0,s=!0),typeof e.isSearching<"u"&&this._isSearching!==e.isSearching&&(this._isSearching=e.isSearching,n.isSearching=!0,s=!0),typeof e.filters<"u"&&(this._filters?this._filters.update(e.filters):this._filters=e.filters,n.filters=!0,s=!0),this._isRegexOverride=typeof e.isRegexOverride<"u"?e.isRegexOverride:0,this._wholeWordOverride=typeof e.wholeWordOverride<"u"?e.wholeWordOverride:0,this._matchCaseOverride=typeof e.matchCaseOverride<"u"?e.matchCaseOverride:0,this._preserveCaseOverride=typeof e.preserveCaseOverride<"u"?e.preserveCaseOverride:0,r!==this.isRegex&&(s=!0,n.isRegex=!0),a!==this.wholeWord&&(s=!0,n.wholeWord=!0),l!==this.matchCase&&(s=!0,n.matchCase=!0),c!==this.preserveCase&&(s=!0,n.preserveCase=!0),s&&this._onFindReplaceStateChange.fire(n)}canNavigateBack(){return this.canNavigateInLoop()||this.matchesPosition!==1}canNavigateForward(){return this.canNavigateInLoop()||this.matchesPosition<this.matchesCount}canNavigateInLoop(){return this._loop||this.matchesCount>=sh}}const TLe=m("defaultLabel","input"),RLe=m("label.preserveCaseToggle","Preserve Case");class MLe extends kb{constructor(e){super({icon:te.preserveCase,title:RLe+e.appendTitle,isChecked:e.isChecked,hoverDelegate:e.hoverDelegate??Ss("element"),inputActiveOptionBorder:e.inputActiveOptionBorder,inputActiveOptionForeground:e.inputActiveOptionForeground,inputActiveOptionBackground:e.inputActiveOptionBackground})}}class ALe extends Yo{constructor(e,t,i,n){super(),this._showOptionButtons=i,this.fixFocusOnOptionClickEnabled=!0,this.cachedOptionsWidth=0,this._onDidOptionChange=this._register(new P),this.onDidOptionChange=this._onDidOptionChange.event,this._onKeyDown=this._register(new P),this.onKeyDown=this._onKeyDown.event,this._onMouseDown=this._register(new P),this._onInput=this._register(new P),this._onKeyUp=this._register(new P),this._onPreserveCaseKeyDown=this._register(new P),this.onPreserveCaseKeyDown=this._onPreserveCaseKeyDown.event,this.contextViewProvider=t,this.placeholder=n.placeholder||"",this.validation=n.validation,this.label=n.label||TLe;const s=n.appendPreserveCaseLabel||"",r=n.history||[],a=!!n.flexibleHeight,l=!!n.flexibleWidth,c=n.flexibleMaxHeight;this.domNode=document.createElement("div"),this.domNode.classList.add("monaco-findInput"),this.inputBox=this._register(new LK(this.domNode,this.contextViewProvider,{ariaLabel:this.label||"",placeholder:this.placeholder||"",validationOptions:{validation:this.validation},history:r,showHistoryHint:n.showHistoryHint,flexibleHeight:a,flexibleWidth:l,flexibleMaxHeight:c,inputBoxStyles:n.inputBoxStyles})),this.preserveCase=this._register(new MLe({appendTitle:s,isChecked:!1,...n.toggleStyles})),this._register(this.preserveCase.onChange(u=>{this._onDidOptionChange.fire(u),!u&&this.fixFocusOnOptionClickEnabled&&this.inputBox.focus(),this.validate()})),this._register(this.preserveCase.onKeyDown(u=>{this._onPreserveCaseKeyDown.fire(u)})),this._showOptionButtons?this.cachedOptionsWidth=this.preserveCase.width():this.cachedOptionsWidth=0;const d=[this.preserveCase.domNode];this.onkeydown(this.domNode,u=>{if(u.equals(15)||u.equals(17)||u.equals(9)){const g=d.indexOf(this.domNode.ownerDocument.activeElement);if(g>=0){let f=-1;u.equals(17)?f=(g+1)%d.length:u.equals(15)&&(g===0?f=d.length-1:f=g-1),u.equals(9)?(d[g].blur(),this.inputBox.focus()):f>=0&&d[f].focus(),nt.stop(u,!0)}}});const h=document.createElement("div");h.className="controls",h.style.display=this._showOptionButtons?"block":"none",h.appendChild(this.preserveCase.domNode),this.domNode.appendChild(h),e==null||e.appendChild(this.domNode),this.onkeydown(this.inputBox.inputElement,u=>this._onKeyDown.fire(u)),this.onkeyup(this.inputBox.inputElement,u=>this._onKeyUp.fire(u)),this.oninput(this.inputBox.inputElement,u=>this._onInput.fire()),this.onmousedown(this.inputBox.inputElement,u=>this._onMouseDown.fire(u))}enable(){this.domNode.classList.remove("disabled"),this.inputBox.enable(),this.preserveCase.enable()}disable(){this.domNode.classList.add("disabled"),this.inputBox.disable(),this.preserveCase.disable()}setEnabled(e){e?this.enable():this.disable()}select(){this.inputBox.select()}focus(){this.inputBox.focus()}getPreserveCase(){return this.preserveCase.checked}setPreserveCase(e){this.preserveCase.checked=e}focusOnPreserve(){this.preserveCase.focus()}validate(){var e;(e=this.inputBox)==null||e.validate()}set width(e){this.inputBox.paddingRight=this.cachedOptionsWidth,this.domNode.style.width=e+"px"}dispose(){super.dispose()}}var XZ=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},QZ=function(o,e){return function(t,i){e(t,i,o)}};const ZW=new le("suggestWidgetVisible",!1,m("suggestWidgetVisible","Whether suggestion are visible")),YW="historyNavigationWidgetFocus",JZ="historyNavigationForwardsEnabled",eY="historyNavigationBackwardsEnabled";let Zc;const x0=[];function tY(o,e){if(x0.includes(e))throw new Error("Cannot register the same widget multiple times");x0.push(e);const t=new K,i=new le(YW,!1).bindTo(o),n=new le(JZ,!0).bindTo(o),s=new le(eY,!0).bindTo(o),r=()=>{i.set(!0),Zc=e},a=()=>{i.set(!1),Zc===e&&(Zc=void 0)};return wI(e.element)&&r(),t.add(e.onDidFocus(()=>r())),t.add(e.onDidBlur(()=>a())),t.add(ve(()=>{x0.splice(x0.indexOf(e),1),a()})),{historyNavigationForwardsEnablement:n,historyNavigationBackwardsEnablement:s,dispose(){t.dispose()}}}let zO=class extends xK{constructor(e,t,i,n){super(e,t,i);const s=this._register(n.createScoped(this.inputBox.element));this._register(tY(s,this.inputBox))}};zO=XZ([QZ(3,Ee)],zO);let UO=class extends ALe{constructor(e,t,i,n,s=!1){super(e,t,s,i);const r=this._register(n.createScoped(this.inputBox.element));this._register(tY(r,this.inputBox))}};UO=XZ([QZ(3,Ee)],UO);bs.registerCommandAndKeybindingRule({id:"history.showPrevious",weight:200,when:G.and(G.has(YW),G.equals(eY,!0),G.not("isComposing"),ZW.isEqualTo(!1)),primary:16,secondary:[528],handler:o=>{Zc==null||Zc.showPreviousValue()}});bs.registerCommandAndKeybindingRule({id:"history.showNext",weight:200,when:G.and(G.has(YW),G.equals(JZ,!0),G.not("isComposing"),ZW.isEqualTo(!1)),primary:18,secondary:[530],handler:o=>{Zc==null||Zc.showNextValue()}});function JV(o){var e,t;return((e=o.lookupKeybinding("history.showPrevious"))==null?void 0:e.getElectronAccelerator())==="Up"&&((t=o.lookupKeybinding("history.showNext"))==null?void 0:t.getElectronAccelerator())==="Down"}const e6=qt("find-collapsed",te.chevronRight,m("findCollapsedIcon","Icon to indicate that the editor find widget is collapsed.")),t6=qt("find-expanded",te.chevronDown,m("findExpandedIcon","Icon to indicate that the editor find widget is expanded.")),PLe=qt("find-selection",te.selection,m("findSelectionIcon","Icon for 'Find in Selection' in the editor find widget.")),OLe=qt("find-replace",te.replace,m("findReplaceIcon","Icon for 'Replace' in the editor find widget.")),FLe=qt("find-replace-all",te.replaceAll,m("findReplaceAllIcon","Icon for 'Replace All' in the editor find widget.")),WLe=qt("find-previous-match",te.arrowUp,m("findPreviousMatchIcon","Icon for 'Find Previous' in the editor find widget.")),BLe=qt("find-next-match",te.arrowDown,m("findNextMatchIcon","Icon for 'Find Next' in the editor find widget.")),HLe=m("label.findDialog","Find / Replace"),VLe=m("label.find","Find"),zLe=m("placeholder.find","Find"),ULe=m("label.previousMatchButton","Previous Match"),$Le=m("label.nextMatchButton","Next Match"),jLe=m("label.toggleSelectionFind","Find in Selection"),KLe=m("label.closeButton","Close"),qLe=m("label.replace","Replace"),GLe=m("placeholder.replace","Replace"),ZLe=m("label.replaceButton","Replace"),YLe=m("label.replaceAllButton","Replace All"),XLe=m("label.toggleReplaceButton","Toggle Replace"),QLe=m("title.matchesCountLimit","Only the first {0} results are highlighted, but all find operations work on the entire text.",sh),JLe=m("label.matchesLocation","{0} of {1}"),i6=m("label.noResults","No results"),il=419,exe=275,txe=exe-54;let fC=69;const ixe=33,n6="ctrlEnterReplaceAll.windows.donotask",s6=Je?256:2048;class zT{constructor(e){this.afterLineNumber=e,this.heightInPx=ixe,this.suppressMouseDown=!1,this.domNode=document.createElement("div"),this.domNode.className="dock-find-viewzone"}}function o6(o,e,t){const i=!!e.match(/\n/);if(t&&i&&t.selectionStart>0){o.stopPropagation();return}}function r6(o,e,t){const i=!!e.match(/\n/);if(t&&i&&t.selectionEnd<t.value.length){o.stopPropagation();return}}const ED=class ED extends Yo{constructor(e,t,i,n,s,r,a,l,c,d){super(),this._hoverService=d,this._cachedHeight=null,this._revealTimeouts=[],this._codeEditor=e,this._controller=t,this._state=i,this._contextViewProvider=n,this._keybindingService=s,this._contextKeyService=r,this._storageService=l,this._notificationService=c,this._ctrlEnterReplaceAllWarningPrompted=!!l.getBoolean(n6,0),this._isVisible=!1,this._isReplaceVisible=!1,this._ignoreChangeEvent=!1,this._updateHistoryDelayer=new _r(500),this._register(ve(()=>this._updateHistoryDelayer.cancel())),this._register(this._state.onFindReplaceStateChange(h=>this._onStateChanged(h))),this._buildDomNode(),this._updateButtons(),this._tryUpdateWidgetWidth(),this._findInput.inputBox.layout(),this._register(this._codeEditor.onDidChangeConfiguration(h=>{if(h.hasChanged(92)&&(this._codeEditor.getOption(92)&&this._state.change({isReplaceRevealed:!1},!1),this._updateButtons()),h.hasChanged(146)&&this._tryUpdateWidgetWidth(),h.hasChanged(2)&&this.updateAccessibilitySupport(),h.hasChanged(41)){const u=this._codeEditor.getOption(41).loop;this._state.change({loop:u},!1);const g=this._codeEditor.getOption(41).addExtraSpaceOnTop;g&&!this._viewZone&&(this._viewZone=new zT(0),this._showViewZone()),!g&&this._viewZone&&this._removeViewZone()}})),this.updateAccessibilitySupport(),this._register(this._codeEditor.onDidChangeCursorSelection(()=>{this._isVisible&&this._updateToggleSelectionFindButton()})),this._register(this._codeEditor.onDidFocusEditorWidget(async()=>{if(this._isVisible){const h=await this._controller.getGlobalBufferTerm();h&&h!==this._state.searchString&&(this._state.change({searchString:h},!1),this._findInput.select())}})),this._findInputFocused=WE.bindTo(r),this._findFocusTracker=this._register(Fa(this._findInput.inputBox.inputElement)),this._register(this._findFocusTracker.onDidFocus(()=>{this._findInputFocused.set(!0),this._updateSearchScope()})),this._register(this._findFocusTracker.onDidBlur(()=>{this._findInputFocused.set(!1)})),this._replaceInputFocused=GW.bindTo(r),this._replaceFocusTracker=this._register(Fa(this._replaceInput.inputBox.inputElement)),this._register(this._replaceFocusTracker.onDidFocus(()=>{this._replaceInputFocused.set(!0),this._updateSearchScope()})),this._register(this._replaceFocusTracker.onDidBlur(()=>{this._replaceInputFocused.set(!1)})),this._codeEditor.addOverlayWidget(this),this._codeEditor.getOption(41).addExtraSpaceOnTop&&(this._viewZone=new zT(0)),this._register(this._codeEditor.onDidChangeModel(()=>{this._isVisible&&(this._viewZoneId=void 0)})),this._register(this._codeEditor.onDidScrollChange(h=>{if(h.scrollTopChanged){this._layoutViewZone();return}setTimeout(()=>{this._layoutViewZone()},0)}))}getId(){return ED.ID}getDomNode(){return this._domNode}getPosition(){return this._isVisible?{preference:0}:null}_onStateChanged(e){if(e.searchString){try{this._ignoreChangeEvent=!0,this._findInput.setValue(this._state.searchString)}finally{this._ignoreChangeEvent=!1}this._updateButtons()}if(e.replaceString&&(this._replaceInput.inputBox.value=this._state.replaceString),e.isRevealed&&(this._state.isRevealed?this._reveal():this._hide(!0)),e.isReplaceRevealed&&(this._state.isReplaceRevealed?!this._codeEditor.getOption(92)&&!this._isReplaceVisible&&(this._isReplaceVisible=!0,this._replaceInput.width=Xs(this._findInput.domNode),this._updateButtons(),this._replaceInput.inputBox.layout()):this._isReplaceVisible&&(this._isReplaceVisible=!1,this._updateButtons())),(e.isRevealed||e.isReplaceRevealed)&&(this._state.isRevealed||this._state.isReplaceRevealed)&&this._tryUpdateHeight()&&this._showViewZone(),e.isRegex&&this._findInput.setRegex(this._state.isRegex),e.wholeWord&&this._findInput.setWholeWords(this._state.wholeWord),e.matchCase&&this._findInput.setCaseSensitive(this._state.matchCase),e.preserveCase&&this._replaceInput.setPreserveCase(this._state.preserveCase),e.searchScope&&(this._state.searchScope?this._toggleSelectionFind.checked=!0:this._toggleSelectionFind.checked=!1,this._updateToggleSelectionFindButton()),e.searchString||e.matchesCount||e.matchesPosition){const t=this._state.searchString.length>0&&this._state.matchesCount===0;this._domNode.classList.toggle("no-results",t),this._updateMatchesCount(),this._updateButtons()}(e.searchString||e.currentMatch)&&this._layoutViewZone(),e.updateHistory&&this._delayedUpdateHistory(),e.loop&&this._updateButtons()}_delayedUpdateHistory(){this._updateHistoryDelayer.trigger(this._updateHistory.bind(this)).then(void 0,Oe)}_updateHistory(){this._state.searchString&&this._findInput.inputBox.addToHistory(),this._state.replaceString&&this._replaceInput.inputBox.addToHistory()}_updateMatchesCount(){var t;this._matchesCount.style.minWidth=fC+"px",this._state.matchesCount>=sh?this._matchesCount.title=QLe:this._matchesCount.title="",(t=this._matchesCount.firstChild)==null||t.remove();let e;if(this._state.matchesCount>0){let i=String(this._state.matchesCount);this._state.matchesCount>=sh&&(i+="+");let n=String(this._state.matchesPosition);n==="0"&&(n="?"),e=xf(JLe,n,i)}else e=i6;this._matchesCount.appendChild(document.createTextNode(e)),Cs(this._getAriaLabel(e,this._state.currentMatch,this._state.searchString)),fC=Math.max(fC,this._matchesCount.clientWidth)}_getAriaLabel(e,t,i){if(e===i6)return i===""?m("ariaSearchNoResultEmpty","{0} found",e):m("ariaSearchNoResult","{0} found for '{1}'",e,i);if(t){const n=m("ariaSearchNoResultWithLineNum","{0} found for '{1}', at {2}",e,i,t.startLineNumber+":"+t.startColumn),s=this._codeEditor.getModel();return s&&t.startLineNumber<=s.getLineCount()&&t.startLineNumber>=1?`${s.getLineContent(t.startLineNumber)}, ${n}`:n}return m("ariaSearchNoResultWithLineNumNoCurrentMatch","{0} found for '{1}'",e,i)}_updateToggleSelectionFindButton(){const e=this._codeEditor.getSelection(),t=e?e.startLineNumber!==e.endLineNumber||e.startColumn!==e.endColumn:!1,i=this._toggleSelectionFind.checked;this._isVisible&&(i||t)?this._toggleSelectionFind.enable():this._toggleSelectionFind.disable()}_updateButtons(){this._findInput.setEnabled(this._isVisible),this._replaceInput.setEnabled(this._isVisible&&this._isReplaceVisible),this._updateToggleSelectionFindButton(),this._closeBtn.setEnabled(this._isVisible);const e=this._state.searchString.length>0,t=!!this._state.matchesCount;this._prevBtn.setEnabled(this._isVisible&&e&&t&&this._state.canNavigateBack()),this._nextBtn.setEnabled(this._isVisible&&e&&t&&this._state.canNavigateForward()),this._replaceBtn.setEnabled(this._isVisible&&this._isReplaceVisible&&e),this._replaceAllBtn.setEnabled(this._isVisible&&this._isReplaceVisible&&e),this._domNode.classList.toggle("replaceToggled",this._isReplaceVisible),this._toggleReplaceBtn.setExpanded(this._isReplaceVisible);const i=!this._codeEditor.getOption(92);this._toggleReplaceBtn.setEnabled(this._isVisible&&i)}_reveal(){if(this._revealTimeouts.forEach(e=>{clearTimeout(e)}),this._revealTimeouts=[],!this._isVisible){this._isVisible=!0;const e=this._codeEditor.getSelection();switch(this._codeEditor.getOption(41).autoFindInSelection){case"always":this._toggleSelectionFind.checked=!0;break;case"never":this._toggleSelectionFind.checked=!1;break;case"multiline":{const i=!!e&&e.startLineNumber!==e.endLineNumber;this._toggleSelectionFind.checked=i;break}}this._tryUpdateWidgetWidth(),this._updateButtons(),this._revealTimeouts.push(setTimeout(()=>{this._domNode.classList.add("visible"),this._domNode.setAttribute("aria-hidden","false")},0)),this._revealTimeouts.push(setTimeout(()=>{this._findInput.validate()},200)),this._codeEditor.layoutOverlayWidget(this);let t=!0;if(this._codeEditor.getOption(41).seedSearchStringFromSelection&&e){const i=this._codeEditor.getDomNode();if(i){const n=wi(i),s=this._codeEditor.getScrolledVisiblePosition(e.getStartPosition()),r=n.left+(s?s.left:0),a=s?s.top:0;if(this._viewZone&&a<this._viewZone.heightInPx){e.endLineNumber>e.startLineNumber&&(t=!1);const l=LU(this._domNode).left;r>l&&(t=!1);const c=this._codeEditor.getScrolledVisiblePosition(e.getEndPosition());n.left+(c?c.left:0)>l&&(t=!1)}}}this._showViewZone(t)}}_hide(e){this._revealTimeouts.forEach(t=>{clearTimeout(t)}),this._revealTimeouts=[],this._isVisible&&(this._isVisible=!1,this._updateButtons(),this._domNode.classList.remove("visible"),this._domNode.setAttribute("aria-hidden","true"),this._findInput.clearMessage(),e&&this._codeEditor.focus(),this._codeEditor.layoutOverlayWidget(this),this._removeViewZone())}_layoutViewZone(e){if(!this._codeEditor.getOption(41).addExtraSpaceOnTop){this._removeViewZone();return}if(!this._isVisible)return;const i=this._viewZone;this._viewZoneId!==void 0||!i||this._codeEditor.changeViewZones(n=>{i.heightInPx=this._getHeight(),this._viewZoneId=n.addZone(i),this._codeEditor.setScrollTop(e||this._codeEditor.getScrollTop()+i.heightInPx)})}_showViewZone(e=!0){if(!this._isVisible||!this._codeEditor.getOption(41).addExtraSpaceOnTop)return;this._viewZone===void 0&&(this._viewZone=new zT(0));const i=this._viewZone;this._codeEditor.changeViewZones(n=>{if(this._viewZoneId!==void 0){const s=this._getHeight();if(s===i.heightInPx)return;const r=s-i.heightInPx;i.heightInPx=s,n.layoutZone(this._viewZoneId),e&&this._codeEditor.setScrollTop(this._codeEditor.getScrollTop()+r);return}else{let s=this._getHeight();if(s-=this._codeEditor.getOption(84).top,s<=0)return;i.heightInPx=s,this._viewZoneId=n.addZone(i),e&&this._codeEditor.setScrollTop(this._codeEditor.getScrollTop()+s)}})}_removeViewZone(){this._codeEditor.changeViewZones(e=>{this._viewZoneId!==void 0&&(e.removeZone(this._viewZoneId),this._viewZoneId=void 0,this._viewZone&&(this._codeEditor.setScrollTop(this._codeEditor.getScrollTop()-this._viewZone.heightInPx),this._viewZone=void 0))})}_tryUpdateWidgetWidth(){if(!this._isVisible||!this._domNode.isConnected)return;const e=this._codeEditor.getLayoutInfo();if(e.contentWidth<=0){this._domNode.classList.add("hiddenEditor");return}else this._domNode.classList.contains("hiddenEditor")&&this._domNode.classList.remove("hiddenEditor");const i=e.width,n=e.minimap.minimapWidth;let s=!1,r=!1,a=!1;if(this._resized&&Xs(this._domNode)>il){this._domNode.style.maxWidth=`${i-28-n-15}px`,this._replaceInput.width=Xs(this._findInput.domNode);return}if(il+28+n>=i&&(r=!0),il+28+n-fC>=i&&(a=!0),il+28+n-fC>=i+50&&(s=!0),this._domNode.classList.toggle("collapsed-find-widget",s),this._domNode.classList.toggle("narrow-find-widget",a),this._domNode.classList.toggle("reduced-find-widget",r),!a&&!s&&(this._domNode.style.maxWidth=`${i-28-n-15}px`),this._findInput.layout({collapsedFindWidget:s,narrowFindWidget:a,reducedFindWidget:r}),this._resized){const l=this._findInput.inputBox.element.clientWidth;l>0&&(this._replaceInput.width=l)}else this._isReplaceVisible&&(this._replaceInput.width=Xs(this._findInput.domNode))}_getHeight(){let e=0;return e+=4,e+=this._findInput.inputBox.height+2,this._isReplaceVisible&&(e+=4,e+=this._replaceInput.inputBox.height+2),e+=4,e}_tryUpdateHeight(){const e=this._getHeight();return this._cachedHeight!==null&&this._cachedHeight===e?!1:(this._cachedHeight=e,this._domNode.style.height=`${e}px`,!0)}focusFindInput(){this._findInput.select(),this._findInput.focus()}focusReplaceInput(){this._replaceInput.select(),this._replaceInput.focus()}highlightFindOptions(){this._findInput.highlightFindOptions()}_updateSearchScope(){if(this._codeEditor.hasModel()&&this._toggleSelectionFind.checked){const e=this._codeEditor.getSelections();e.map(t=>{t.endColumn===1&&t.endLineNumber>t.startLineNumber&&(t=t.setEndPosition(t.endLineNumber-1,this._codeEditor.getModel().getLineMaxColumn(t.endLineNumber-1)));const i=this._state.currentMatch;return t.startLineNumber!==t.endLineNumber&&!x.equalsRange(t,i)?t:null}).filter(t=>!!t),e.length&&this._state.change({searchScope:e},!0)}}_onFindInputMouseDown(e){e.middleButton&&e.stopPropagation()}_onFindInputKeyDown(e){if(e.equals(s6|3))if(this._keybindingService.dispatchEvent(e,e.target)){e.preventDefault();return}else{this._findInput.inputBox.insertAtCursor(`
`),e.preventDefault();return}if(e.equals(2)){this._isReplaceVisible?this._replaceInput.focus():this._findInput.focusOnCaseSensitive(),e.preventDefault();return}if(e.equals(2066)){this._codeEditor.focus(),e.preventDefault();return}if(e.equals(16))return o6(e,this._findInput.getValue(),this._findInput.domNode.querySelector("textarea"));if(e.equals(18))return r6(e,this._findInput.getValue(),this._findInput.domNode.querySelector("textarea"))}_onReplaceInputKeyDown(e){if(e.equals(s6|3))if(this._keybindingService.dispatchEvent(e,e.target)){e.preventDefault();return}else{En&&Pl&&!this._ctrlEnterReplaceAllWarningPrompted&&(this._notificationService.info(m("ctrlEnter.keybindingChanged","Ctrl+Enter now inserts line break instead of replacing all. You can modify the keybinding for editor.action.replaceAll to override this behavior.")),this._ctrlEnterReplaceAllWarningPrompted=!0,this._storageService.store(n6,!0,0,0)),this._replaceInput.inputBox.insertAtCursor(`
`),e.preventDefault();return}if(e.equals(2)){this._findInput.focusOnCaseSensitive(),e.preventDefault();return}if(e.equals(1026)){this._findInput.focus(),e.preventDefault();return}if(e.equals(2066)){this._codeEditor.focus(),e.preventDefault();return}if(e.equals(16))return o6(e,this._replaceInput.inputBox.value,this._replaceInput.inputBox.element.querySelector("textarea"));if(e.equals(18))return r6(e,this._replaceInput.inputBox.value,this._replaceInput.inputBox.element.querySelector("textarea"))}getVerticalSashLeft(e){return 0}_keybindingLabelFor(e){const t=this._keybindingService.lookupKeybinding(e);return t?` (${t.getLabel()})`:""}_buildDomNode(){this._findInput=this._register(new zO(null,this._contextViewProvider,{width:txe,label:VLe,placeholder:zLe,appendCaseSensitiveLabel:this._keybindingLabelFor(Ut.ToggleCaseSensitiveCommand),appendWholeWordsLabel:this._keybindingLabelFor(Ut.ToggleWholeWordCommand),appendRegexLabel:this._keybindingLabelFor(Ut.ToggleRegexCommand),validation:d=>{if(d.length===0||!this._findInput.getRegex())return null;try{return new RegExp(d,"gu"),null}catch(h){return{content:h.message}}},flexibleHeight:!0,flexibleWidth:!0,flexibleMaxHeight:118,showCommonFindToggles:!0,showHistoryHint:()=>JV(this._keybindingService),inputBoxStyles:xL,toggleStyles:LL},this._contextKeyService)),this._findInput.setRegex(!!this._state.isRegex),this._findInput.setCaseSensitive(!!this._state.matchCase),this._findInput.setWholeWords(!!this._state.wholeWord),this._register(this._findInput.onKeyDown(d=>this._onFindInputKeyDown(d))),this._register(this._findInput.inputBox.onDidChange(()=>{this._ignoreChangeEvent||this._state.change({searchString:this._findInput.getValue()},!0)})),this._register(this._findInput.onDidOptionChange(()=>{this._state.change({isRegex:this._findInput.getRegex(),wholeWord:this._findInput.getWholeWords(),matchCase:this._findInput.getCaseSensitive()},!0)})),this._register(this._findInput.onCaseSensitiveKeyDown(d=>{d.equals(1026)&&this._isReplaceVisible&&(this._replaceInput.focus(),d.preventDefault())})),this._register(this._findInput.onRegexKeyDown(d=>{d.equals(2)&&this._isReplaceVisible&&(this._replaceInput.focusOnPreserve(),d.preventDefault())})),this._register(this._findInput.inputBox.onDidHeightChange(d=>{this._tryUpdateHeight()&&this._showViewZone()})),ps&&this._register(this._findInput.onMouseDown(d=>this._onFindInputMouseDown(d))),this._matchesCount=document.createElement("div"),this._matchesCount.className="matchesCount",this._updateMatchesCount();const i=this._register(H_());this._prevBtn=this._register(new Fm({label:ULe+this._keybindingLabelFor(Ut.PreviousMatchFindAction),icon:WLe,hoverDelegate:i,onTrigger:()=>{Kd(this._codeEditor.getAction(Ut.PreviousMatchFindAction)).run().then(void 0,Oe)}},this._hoverService)),this._nextBtn=this._register(new Fm({label:$Le+this._keybindingLabelFor(Ut.NextMatchFindAction),icon:BLe,hoverDelegate:i,onTrigger:()=>{Kd(this._codeEditor.getAction(Ut.NextMatchFindAction)).run().then(void 0,Oe)}},this._hoverService));const n=document.createElement("div");n.className="find-part",n.appendChild(this._findInput.domNode);const s=document.createElement("div");s.className="find-actions",n.appendChild(s),s.appendChild(this._matchesCount),s.appendChild(this._prevBtn.domNode),s.appendChild(this._nextBtn.domNode),this._toggleSelectionFind=this._register(new kb({icon:PLe,title:jLe+this._keybindingLabelFor(Ut.ToggleSearchScopeCommand),isChecked:!1,hoverDelegate:i,inputActiveOptionBackground:ce(V1),inputActiveOptionBorder:ce(zI),inputActiveOptionForeground:ce(UI)})),this._register(this._toggleSelectionFind.onChange(()=>{if(this._toggleSelectionFind.checked){if(this._codeEditor.hasModel()){let d=this._codeEditor.getSelections();d=d.map(h=>(h.endColumn===1&&h.endLineNumber>h.startLineNumber&&(h=h.setEndPosition(h.endLineNumber-1,this._codeEditor.getModel().getLineMaxColumn(h.endLineNumber-1))),h.isEmpty()?null:h)).filter(h=>!!h),d.length&&this._state.change({searchScope:d},!0)}}else this._state.change({searchScope:null},!0)})),s.appendChild(this._toggleSelectionFind.domNode),this._closeBtn=this._register(new Fm({label:KLe+this._keybindingLabelFor(Ut.CloseFindWidgetCommand),icon:tq,hoverDelegate:i,onTrigger:()=>{this._state.change({isRevealed:!1,searchScope:null},!1)},onKeyDown:d=>{d.equals(2)&&this._isReplaceVisible&&(this._replaceBtn.isEnabled()?this._replaceBtn.focus():this._codeEditor.focus(),d.preventDefault())}},this._hoverService)),this._replaceInput=this._register(new UO(null,void 0,{label:qLe,placeholder:GLe,appendPreserveCaseLabel:this._keybindingLabelFor(Ut.TogglePreserveCaseCommand),history:[],flexibleHeight:!0,flexibleWidth:!0,flexibleMaxHeight:118,showHistoryHint:()=>JV(this._keybindingService),inputBoxStyles:xL,toggleStyles:LL},this._contextKeyService,!0)),this._replaceInput.setPreserveCase(!!this._state.preserveCase),this._register(this._replaceInput.onKeyDown(d=>this._onReplaceInputKeyDown(d))),this._register(this._replaceInput.inputBox.onDidChange(()=>{this._state.change({replaceString:this._replaceInput.inputBox.value},!1)})),this._register(this._replaceInput.inputBox.onDidHeightChange(d=>{this._isReplaceVisible&&this._tryUpdateHeight()&&this._showViewZone()})),this._register(this._replaceInput.onDidOptionChange(()=>{this._state.change({preserveCase:this._replaceInput.getPreserveCase()},!0)})),this._register(this._replaceInput.onPreserveCaseKeyDown(d=>{d.equals(2)&&(this._prevBtn.isEnabled()?this._prevBtn.focus():this._nextBtn.isEnabled()?this._nextBtn.focus():this._toggleSelectionFind.enabled?this._toggleSelectionFind.focus():this._closeBtn.isEnabled()&&this._closeBtn.focus(),d.preventDefault())}));const r=this._register(H_());this._replaceBtn=this._register(new Fm({label:ZLe+this._keybindingLabelFor(Ut.ReplaceOneAction),icon:OLe,hoverDelegate:r,onTrigger:()=>{this._controller.replace()},onKeyDown:d=>{d.equals(1026)&&(this._closeBtn.focus(),d.preventDefault())}},this._hoverService)),this._replaceAllBtn=this._register(new Fm({label:YLe+this._keybindingLabelFor(Ut.ReplaceAllAction),icon:FLe,hoverDelegate:r,onTrigger:()=>{this._controller.replaceAll()}},this._hoverService));const a=document.createElement("div");a.className="replace-part",a.appendChild(this._replaceInput.domNode);const l=document.createElement("div");l.className="replace-actions",a.appendChild(l),l.appendChild(this._replaceBtn.domNode),l.appendChild(this._replaceAllBtn.domNode),this._toggleReplaceBtn=this._register(new Fm({label:XLe,className:"codicon toggle left",onTrigger:()=>{this._state.change({isReplaceRevealed:!this._isReplaceVisible},!1),this._isReplaceVisible&&(this._replaceInput.width=Xs(this._findInput.domNode),this._replaceInput.inputBox.layout()),this._showViewZone()}},this._hoverService)),this._toggleReplaceBtn.setExpanded(this._isReplaceVisible),this._domNode=document.createElement("div"),this._domNode.className="editor-widget find-widget",this._domNode.setAttribute("aria-hidden","true"),this._domNode.ariaLabel=HLe,this._domNode.role="dialog",this._domNode.style.width=`${il}px`,this._domNode.appendChild(this._toggleReplaceBtn.domNode),this._domNode.appendChild(n),this._domNode.appendChild(this._closeBtn.domNode),this._domNode.appendChild(a),this._resizeSash=this._register(new Mn(this._domNode,this,{orientation:0,size:2})),this._resized=!1;let c=il;this._register(this._resizeSash.onDidStart(()=>{c=Xs(this._domNode)})),this._register(this._resizeSash.onDidChange(d=>{this._resized=!0;const h=c+d.startX-d.currentX;if(h<il)return;const u=parseFloat(vI(this._domNode).maxWidth)||0;h>u||(this._domNode.style.width=`${h}px`,this._isReplaceVisible&&(this._replaceInput.width=Xs(this._findInput.domNode)),this._findInput.inputBox.layout(),this._tryUpdateHeight())})),this._register(this._resizeSash.onDidReset(()=>{const d=Xs(this._domNode);if(d<il)return;let h=il;if(!this._resized||d===il){const u=this._codeEditor.getLayoutInfo();h=u.width-28-u.minimap.minimapWidth-15,this._resized=!0}this._domNode.style.width=`${h}px`,this._isReplaceVisible&&(this._replaceInput.width=Xs(this._findInput.domNode)),this._findInput.inputBox.layout()}))}updateAccessibilitySupport(){const e=this._codeEditor.getOption(2);this._findInput.setFocusInputOnOptionClick(e!==2)}};ED.ID="editor.contrib.findWidget";let $O=ED;class Fm extends Yo{constructor(e,t){super(),this._opts=e;let i="button";this._opts.className&&(i=i+" "+this._opts.className),this._opts.icon&&(i=i+" "+Ie.asClassName(this._opts.icon)),this._domNode=document.createElement("div"),this._domNode.tabIndex=0,this._domNode.className=i,this._domNode.setAttribute("role","button"),this._domNode.setAttribute("aria-label",this._opts.label),this._register(t.setupManagedHover(e.hoverDelegate??Ss("element"),this._domNode,this._opts.label)),this.onclick(this._domNode,n=>{this._opts.onTrigger(),n.preventDefault()}),this.onkeydown(this._domNode,n=>{var s,r;if(n.equals(10)||n.equals(3)){this._opts.onTrigger(),n.preventDefault();return}(r=(s=this._opts).onKeyDown)==null||r.call(s,n)})}get domNode(){return this._domNode}isEnabled(){return this._domNode.tabIndex>=0}focus(){this._domNode.focus()}setEnabled(e){this._domNode.classList.toggle("disabled",!e),this._domNode.setAttribute("aria-disabled",String(!e)),this._domNode.tabIndex=e?0:-1}setExpanded(e){this._domNode.setAttribute("aria-expanded",String(!!e)),e?(this._domNode.classList.remove(...Ie.asClassNameArray(e6)),this._domNode.classList.add(...Ie.asClassNameArray(t6))):(this._domNode.classList.remove(...Ie.asClassNameArray(t6)),this._domNode.classList.add(...Ie.asClassNameArray(e6)))}}Lr((o,e)=>{const t=o.getColor(Zd);t&&e.addRule(`.monaco-editor .findMatch { border: 1px ${Ta(o.type)?"dotted":"solid"} ${t}; box-sizing: border-box; }`);const i=o.getColor(zoe);i&&e.addRule(`.monaco-editor .findScope { border: 1px ${Ta(o.type)?"dashed":"solid"} ${i}; }`);const n=o.getColor(lt);n&&e.addRule(`.monaco-editor .find-widget { border: 1px solid ${n}; }`);const s=o.getColor(Hoe);s&&e.addRule(`.monaco-editor .findMatchInline { color: ${s}; }`);const r=o.getColor(Voe);r&&e.addRule(`.monaco-editor .currentFindMatchInline { color: ${r}; }`)});var iY=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},sr=function(o,e){return function(t,i){e(t,i,o)}},jO;const nxe=524288;function KO(o,e="single",t=!1){if(!o.hasModel())return null;const i=o.getSelection();if(e==="single"&&i.startLineNumber===i.endLineNumber||e==="multiple"){if(i.isEmpty()){const n=o.getConfiguredWordAtPosition(i.getStartPosition());if(n&&t===!1)return n.word}else if(o.getModel().getValueLengthInRange(i)<nxe)return o.getModel().getValueInRange(i)}return null}var Jg;let Bs=(Jg=class extends F{get editor(){return this._editor}static get(e){return e.getContribution(jO.ID)}constructor(e,t,i,n,s,r){super(),this._editor=e,this._findWidgetVisible=pd.bindTo(t),this._contextKeyService=t,this._storageService=i,this._clipboardService=n,this._notificationService=s,this._hoverService=r,this._updateHistoryDelayer=new _r(500),this._state=this._register(new NLe),this.loadQueryState(),this._register(this._state.onFindReplaceStateChange(a=>this._onStateChanged(a))),this._model=null,this._register(this._editor.onDidChangeModel(()=>{const a=this._editor.getModel()&&this._state.isRevealed;this.disposeModel(),this._state.change({searchScope:null,matchCase:this._storageService.getBoolean("editor.matchCase",1,!1),wholeWord:this._storageService.getBoolean("editor.wholeWord",1,!1),isRegex:this._storageService.getBoolean("editor.isRegex",1,!1),preserveCase:this._storageService.getBoolean("editor.preserveCase",1,!1)},!1),a&&this._start({forceRevealReplace:!1,seedSearchStringFromSelection:"none",seedSearchStringFromNonEmptySelection:!1,seedSearchStringFromGlobalClipboard:!1,shouldFocus:0,shouldAnimate:!1,updateSearchScope:!1,loop:this._editor.getOption(41).loop})}))}dispose(){this.disposeModel(),super.dispose()}disposeModel(){this._model&&(this._model.dispose(),this._model=null)}_onStateChanged(e){this.saveQueryState(e),e.isRevealed&&(this._state.isRevealed?this._findWidgetVisible.set(!0):(this._findWidgetVisible.reset(),this.disposeModel())),e.searchString&&this.setGlobalBufferTerm(this._state.searchString)}saveQueryState(e){e.isRegex&&this._storageService.store("editor.isRegex",this._state.actualIsRegex,1,1),e.wholeWord&&this._storageService.store("editor.wholeWord",this._state.actualWholeWord,1,1),e.matchCase&&this._storageService.store("editor.matchCase",this._state.actualMatchCase,1,1),e.preserveCase&&this._storageService.store("editor.preserveCase",this._state.actualPreserveCase,1,1)}loadQueryState(){this._state.change({matchCase:this._storageService.getBoolean("editor.matchCase",1,this._state.matchCase),wholeWord:this._storageService.getBoolean("editor.wholeWord",1,this._state.wholeWord),isRegex:this._storageService.getBoolean("editor.isRegex",1,this._state.isRegex),preserveCase:this._storageService.getBoolean("editor.preserveCase",1,this._state.preserveCase)},!1)}isFindInputFocused(){return!!WE.getValue(this._contextKeyService)}getState(){return this._state}closeFindWidget(){this._state.change({isRevealed:!1,searchScope:null},!1),this._editor.focus()}toggleCaseSensitive(){this._state.change({matchCase:!this._state.matchCase},!1),this._state.isRevealed||this.highlightFindOptions()}toggleWholeWords(){this._state.change({wholeWord:!this._state.wholeWord},!1),this._state.isRevealed||this.highlightFindOptions()}toggleRegex(){this._state.change({isRegex:!this._state.isRegex},!1),this._state.isRevealed||this.highlightFindOptions()}togglePreserveCase(){this._state.change({preserveCase:!this._state.preserveCase},!1),this._state.isRevealed||this.highlightFindOptions()}toggleSearchScope(){if(this._state.searchScope)this._state.change({searchScope:null},!0);else if(this._editor.hasModel()){let e=this._editor.getSelections();e=e.map(t=>(t.endColumn===1&&t.endLineNumber>t.startLineNumber&&(t=t.setEndPosition(t.endLineNumber-1,this._editor.getModel().getLineMaxColumn(t.endLineNumber-1))),t.isEmpty()?null:t)).filter(t=>!!t),e.length&&this._state.change({searchScope:e},!0)}}setSearchString(e){this._state.isRegex&&(e=Uo(e)),this._state.change({searchString:e},!1)}highlightFindOptions(e=!1){}async _start(e,t){if(this.disposeModel(),!this._editor.hasModel())return;const i={...t,isRevealed:!0};if(e.seedSearchStringFromSelection==="single"){const n=KO(this._editor,e.seedSearchStringFromSelection,e.seedSearchStringFromNonEmptySelection);n&&(this._state.isRegex?i.searchString=Uo(n):i.searchString=n)}else if(e.seedSearchStringFromSelection==="multiple"&&!e.updateSearchScope){const n=KO(this._editor,e.seedSearchStringFromSelection);n&&(i.searchString=n)}if(!i.searchString&&e.seedSearchStringFromGlobalClipboard){const n=await this.getGlobalBufferTerm();if(!this._editor.hasModel())return;n&&(i.searchString=n)}if(e.forceRevealReplace||i.isReplaceRevealed?i.isReplaceRevealed=!0:this._findWidgetVisible.get()||(i.isReplaceRevealed=!1),e.updateSearchScope){const n=this._editor.getSelections();n.some(s=>!s.isEmpty())&&(i.searchScope=n)}i.loop=e.loop,this._state.change(i,!1),this._model||(this._model=new wv(this._editor,this._state))}start(e,t){return this._start(e,t)}moveToNextMatch(){return this._model?(this._model.moveToNextMatch(),!0):!1}moveToPrevMatch(){return this._model?(this._model.moveToPrevMatch(),!0):!1}goToMatch(e){return this._model?(this._model.moveToMatch(e),!0):!1}replace(){return this._model?(this._model.replace(),!0):!1}replaceAll(){var e;return this._model?(e=this._editor.getModel())!=null&&e.isTooLargeForHeapOperation()?(this._notificationService.warn(m("too.large.for.replaceall","The file is too large to perform a replace all operation.")),!1):(this._model.replaceAll(),!0):!1}selectAllMatches(){return this._model?(this._model.selectAllMatches(),this._editor.focus(),!0):!1}async getGlobalBufferTerm(){return this._editor.getOption(41).globalFindClipboard&&this._editor.hasModel()&&!this._editor.getModel().isTooLargeForSyncing()?this._clipboardService.readFindText():""}setGlobalBufferTerm(e){this._editor.getOption(41).globalFindClipboard&&this._editor.hasModel()&&!this._editor.getModel().isTooLargeForSyncing()&&this._clipboardService.writeFindText(e)}},jO=Jg,Jg.ID="editor.contrib.findController",Jg);Bs=jO=iY([sr(1,Ee),sr(2,ia),sr(3,md),sr(4,Ei),sr(5,Yl)],Bs);let qO=class extends Bs{constructor(e,t,i,n,s,r,a,l,c){super(e,i,a,l,r,c),this._contextViewService=t,this._keybindingService=n,this._themeService=s,this._widget=null,this._findOptionsWidget=null}async _start(e,t){this._widget||this._createFindWidget();const i=this._editor.getSelection();let n=!1;switch(this._editor.getOption(41).autoFindInSelection){case"always":n=!0;break;case"never":n=!1;break;case"multiline":{n=!!i&&i.startLineNumber!==i.endLineNumber;break}}e.updateSearchScope=e.updateSearchScope||n,await super._start(e,t),this._widget&&(e.shouldFocus===2?this._widget.focusReplaceInput():e.shouldFocus===1&&this._widget.focusFindInput())}highlightFindOptions(e=!1){this._widget||this._createFindWidget(),this._state.isRevealed&&!e?this._widget.highlightFindOptions():this._findOptionsWidget.highlightFindOptions()}_createFindWidget(){this._widget=this._register(new $O(this._editor,this,this._state,this._contextViewService,this._keybindingService,this._contextKeyService,this._themeService,this._storageService,this._notificationService,this._hoverService)),this._findOptionsWidget=this._register(new VO(this._editor,this._state,this._keybindingService))}};qO=iY([sr(1,ud),sr(2,Ee),sr(3,yt),sr(4,ji),sr(5,Ei),sr(6,ia),sr(7,md),sr(8,Yl)],qO);const sxe=VU(new HU({id:Ut.StartFindAction,label:m("startFindAction","Find"),alias:"Find",precondition:G.or(I.focus,G.has("editorIsOpen")),kbOpts:{kbExpr:null,primary:2084,weight:100},menuOpts:{menuId:me.MenubarEditMenu,group:"3_find",title:m({key:"miFind",comment:["&& denotes a mnemonic"]},"&&Find"),order:1}}));sxe.addImplementation(0,(o,e,t)=>{const i=Bs.get(e);return i?i.start({forceRevealReplace:!1,seedSearchStringFromSelection:e.getOption(41).seedSearchStringFromSelection!=="never"?"single":"none",seedSearchStringFromNonEmptySelection:e.getOption(41).seedSearchStringFromSelection==="selection",seedSearchStringFromGlobalClipboard:e.getOption(41).globalFindClipboard,shouldFocus:1,shouldAnimate:!0,updateSearchScope:!1,loop:e.getOption(41).loop}):!1});const oxe={description:"Open a new In-Editor Find Widget.",args:[{name:"Open a new In-Editor Find Widget args",schema:{properties:{searchString:{type:"string"},replaceString:{type:"string"},isRegex:{type:"boolean"},matchWholeWord:{type:"boolean"},isCaseSensitive:{type:"boolean"},preserveCase:{type:"boolean"},findInSelection:{type:"boolean"}}}}]};class rxe extends de{constructor(){super({id:Ut.StartFindWithArgs,label:m("startFindWithArgsAction","Find With Arguments"),alias:"Find With Arguments",precondition:void 0,kbOpts:{kbExpr:null,primary:0,weight:100},metadata:oxe})}async run(e,t,i){const n=Bs.get(t);if(n){const s=i?{searchString:i.searchString,replaceString:i.replaceString,isReplaceRevealed:i.replaceString!==void 0,isRegex:i.isRegex,wholeWord:i.matchWholeWord,matchCase:i.isCaseSensitive,preserveCase:i.preserveCase}:{};await n.start({forceRevealReplace:!1,seedSearchStringFromSelection:n.getState().searchString.length===0&&t.getOption(41).seedSearchStringFromSelection!=="never"?"single":"none",seedSearchStringFromNonEmptySelection:t.getOption(41).seedSearchStringFromSelection==="selection",seedSearchStringFromGlobalClipboard:!0,shouldFocus:1,shouldAnimate:!0,updateSearchScope:(i==null?void 0:i.findInSelection)||!1,loop:t.getOption(41).loop},s),n.setGlobalBufferTerm(n.getState().searchString)}}}class axe extends de{constructor(){super({id:Ut.StartFindWithSelection,label:m("startFindWithSelectionAction","Find With Selection"),alias:"Find With Selection",precondition:void 0,kbOpts:{kbExpr:null,primary:0,mac:{primary:2083},weight:100}})}async run(e,t){const i=Bs.get(t);i&&(await i.start({forceRevealReplace:!1,seedSearchStringFromSelection:"multiple",seedSearchStringFromNonEmptySelection:!1,seedSearchStringFromGlobalClipboard:!1,shouldFocus:0,shouldAnimate:!0,updateSearchScope:!1,loop:t.getOption(41).loop}),i.setGlobalBufferTerm(i.getState().searchString))}}class nY extends de{async run(e,t){const i=Bs.get(t);i&&!this._run(i)&&(await i.start({forceRevealReplace:!1,seedSearchStringFromSelection:i.getState().searchString.length===0&&t.getOption(41).seedSearchStringFromSelection!=="never"?"single":"none",seedSearchStringFromNonEmptySelection:t.getOption(41).seedSearchStringFromSelection==="selection",seedSearchStringFromGlobalClipboard:!0,shouldFocus:0,shouldAnimate:!0,updateSearchScope:!1,loop:t.getOption(41).loop}),this._run(i))}}class lxe extends nY{constructor(){super({id:Ut.NextMatchFindAction,label:m("findNextMatchAction","Find Next"),alias:"Find Next",precondition:void 0,kbOpts:[{kbExpr:I.focus,primary:61,mac:{primary:2085,secondary:[61]},weight:100},{kbExpr:G.and(I.focus,WE),primary:3,weight:100}]})}_run(e){return e.moveToNextMatch()?(e.editor.pushUndoStop(),!0):!1}}class cxe extends nY{constructor(){super({id:Ut.PreviousMatchFindAction,label:m("findPreviousMatchAction","Find Previous"),alias:"Find Previous",precondition:void 0,kbOpts:[{kbExpr:I.focus,primary:1085,mac:{primary:3109,secondary:[1085]},weight:100},{kbExpr:G.and(I.focus,WE),primary:1027,weight:100}]})}_run(e){return e.moveToPrevMatch()}}class dxe extends de{constructor(){super({id:Ut.GoToMatchFindAction,label:m("findMatchAction.goToMatch","Go to Match..."),alias:"Go to Match...",precondition:pd}),this._highlightDecorations=[]}run(e,t,i){const n=Bs.get(t);if(!n)return;const s=n.getState().matchesCount;if(s<1){e.get(Ei).notify({severity:OI.Warning,message:m("findMatchAction.noResults","No matches. Try searching for something else.")});return}const r=e.get(xr),a=new K,l=a.add(r.createInputBox());l.placeholder=m("findMatchAction.inputPlaceHolder","Type a number to go to a specific match (between 1 and {0})",s);const c=h=>{const u=parseInt(h);if(isNaN(u))return;const g=n.getState().matchesCount;if(u>0&&u<=g)return u-1;if(u<0&&u>=-g)return g+u},d=h=>{const u=c(h);if(typeof u=="number"){l.validationMessage=void 0,n.goToMatch(u);const g=n.getState().currentMatch;g&&this.addDecorations(t,g)}else l.validationMessage=m("findMatchAction.inputValidationMessage","Please type a number between 1 and {0}",n.getState().matchesCount),this.clearDecorations(t)};a.add(l.onDidChangeValue(h=>{d(h)})),a.add(l.onDidAccept(()=>{const h=c(l.value);typeof h=="number"?(n.goToMatch(h),l.hide()):l.validationMessage=m("findMatchAction.inputValidationMessage","Please type a number between 1 and {0}",n.getState().matchesCount)})),a.add(l.onDidHide(()=>{this.clearDecorations(t),a.dispose()})),l.show()}clearDecorations(e){e.changeDecorations(t=>{this._highlightDecorations=t.deltaDecorations(this._highlightDecorations,[])})}addDecorations(e,t){e.changeDecorations(i=>{this._highlightDecorations=i.deltaDecorations(this._highlightDecorations,[{range:t,options:{description:"find-match-quick-access-range-highlight",className:"rangeHighlight",isWholeLine:!0}},{range:t,options:{description:"find-match-quick-access-range-highlight-overview",overviewRuler:{color:mi(zj),position:Ho.Full}}}])})}}class sY extends de{async run(e,t){const i=Bs.get(t);if(!i)return;const n=KO(t,"single",!1);n&&i.setSearchString(n),this._run(i)||(await i.start({forceRevealReplace:!1,seedSearchStringFromSelection:"none",seedSearchStringFromNonEmptySelection:!1,seedSearchStringFromGlobalClipboard:!1,shouldFocus:0,shouldAnimate:!0,updateSearchScope:!1,loop:t.getOption(41).loop}),this._run(i))}}class hxe extends sY{constructor(){super({id:Ut.NextSelectionMatchFindAction,label:m("nextSelectionMatchFindAction","Find Next Selection"),alias:"Find Next Selection",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:2109,weight:100}})}_run(e){return e.moveToNextMatch()}}class uxe extends sY{constructor(){super({id:Ut.PreviousSelectionMatchFindAction,label:m("previousSelectionMatchFindAction","Find Previous Selection"),alias:"Find Previous Selection",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:3133,weight:100}})}_run(e){return e.moveToPrevMatch()}}const gxe=VU(new HU({id:Ut.StartFindReplaceAction,label:m("startReplace","Replace"),alias:"Replace",precondition:G.or(I.focus,G.has("editorIsOpen")),kbOpts:{kbExpr:null,primary:2086,mac:{primary:2596},weight:100},menuOpts:{menuId:me.MenubarEditMenu,group:"3_find",title:m({key:"miReplace",comment:["&& denotes a mnemonic"]},"&&Replace"),order:2}}));gxe.addImplementation(0,(o,e,t)=>{if(!e.hasModel()||e.getOption(92))return!1;const i=Bs.get(e);if(!i)return!1;const n=e.getSelection(),s=i.isFindInputFocused(),r=!n.isEmpty()&&n.startLineNumber===n.endLineNumber&&e.getOption(41).seedSearchStringFromSelection!=="never"&&!s,a=s||r?2:1;return i.start({forceRevealReplace:!0,seedSearchStringFromSelection:r?"single":"none",seedSearchStringFromNonEmptySelection:e.getOption(41).seedSearchStringFromSelection==="selection",seedSearchStringFromGlobalClipboard:e.getOption(41).seedSearchStringFromSelection!=="never",shouldFocus:a,shouldAnimate:!0,updateSearchScope:!1,loop:e.getOption(41).loop})});ct(Bs.ID,qO,0);Q(rxe);Q(axe);Q(lxe);Q(cxe);Q(dxe);Q(hxe);Q(uxe);const qa=$i.bindToContribution(Bs.get);re(new qa({id:Ut.CloseFindWidgetCommand,precondition:pd,handler:o=>o.closeFindWidget(),kbOpts:{weight:105,kbExpr:G.and(I.focus,G.not("isComposing")),primary:9,secondary:[1033]}}));re(new qa({id:Ut.ToggleCaseSensitiveCommand,precondition:void 0,handler:o=>o.toggleCaseSensitive(),kbOpts:{weight:105,kbExpr:I.focus,primary:C0.primary,mac:C0.mac,win:C0.win,linux:C0.linux}}));re(new qa({id:Ut.ToggleWholeWordCommand,precondition:void 0,handler:o=>o.toggleWholeWords(),kbOpts:{weight:105,kbExpr:I.focus,primary:v0.primary,mac:v0.mac,win:v0.win,linux:v0.linux}}));re(new qa({id:Ut.ToggleRegexCommand,precondition:void 0,handler:o=>o.toggleRegex(),kbOpts:{weight:105,kbExpr:I.focus,primary:w0.primary,mac:w0.mac,win:w0.win,linux:w0.linux}}));re(new qa({id:Ut.ToggleSearchScopeCommand,precondition:void 0,handler:o=>o.toggleSearchScope(),kbOpts:{weight:105,kbExpr:I.focus,primary:S0.primary,mac:S0.mac,win:S0.win,linux:S0.linux}}));re(new qa({id:Ut.TogglePreserveCaseCommand,precondition:void 0,handler:o=>o.togglePreserveCase(),kbOpts:{weight:105,kbExpr:I.focus,primary:y0.primary,mac:y0.mac,win:y0.win,linux:y0.linux}}));re(new qa({id:Ut.ReplaceOneAction,precondition:pd,handler:o=>o.replace(),kbOpts:{weight:105,kbExpr:I.focus,primary:3094}}));re(new qa({id:Ut.ReplaceOneAction,precondition:pd,handler:o=>o.replace(),kbOpts:{weight:105,kbExpr:G.and(I.focus,GW),primary:3}}));re(new qa({id:Ut.ReplaceAllAction,precondition:pd,handler:o=>o.replaceAll(),kbOpts:{weight:105,kbExpr:I.focus,primary:2563}}));re(new qa({id:Ut.ReplaceAllAction,precondition:pd,handler:o=>o.replaceAll(),kbOpts:{weight:105,kbExpr:G.and(I.focus,GW),primary:void 0,mac:{primary:2051}}}));re(new qa({id:Ut.SelectAllMatchesAction,precondition:pd,handler:o=>o.selectAllMatches(),kbOpts:{weight:105,kbExpr:I.focus,primary:515}}));const fxe={0:" ",1:"u",2:"r"},a6=65535,ba=16777215,l6=4278190080;class UT{constructor(e){const t=Math.ceil(e/32);this._states=new Uint32Array(t)}get(e){const t=e/32|0,i=e%32;return(this._states[t]&1<<i)!==0}set(e,t){const i=e/32|0,n=e%32,s=this._states[i];t?this._states[i]=s|1<<n:this._states[i]=s&~(1<<n)}}class To{constructor(e,t,i){if(e.length!==t.length||e.length>a6)throw new Error("invalid startIndexes or endIndexes size");this._startIndexes=e,this._endIndexes=t,this._collapseStates=new UT(e.length),this._userDefinedStates=new UT(e.length),this._recoveredStates=new UT(e.length),this._types=i,this._parentsComputed=!1}ensureParentIndices(){if(!this._parentsComputed){this._parentsComputed=!0;const e=[],t=(i,n)=>{const s=e[e.length-1];return this.getStartLineNumber(s)<=i&&this.getEndLineNumber(s)>=n};for(let i=0,n=this._startIndexes.length;i<n;i++){const s=this._startIndexes[i],r=this._endIndexes[i];if(s>ba||r>ba)throw new Error("startLineNumber or endLineNumber must not exceed "+ba);for(;e.length>0&&!t(s,r);)e.pop();const a=e.length>0?e[e.length-1]:-1;e.push(i),this._startIndexes[i]=s+((a&255)<<24),this._endIndexes[i]=r+((a&65280)<<16)}}}get length(){return this._startIndexes.length}getStartLineNumber(e){return this._startIndexes[e]&ba}getEndLineNumber(e){return this._endIndexes[e]&ba}getType(e){return this._types?this._types[e]:void 0}hasTypes(){return!!this._types}isCollapsed(e){return this._collapseStates.get(e)}setCollapsed(e,t){this._collapseStates.set(e,t)}isUserDefined(e){return this._userDefinedStates.get(e)}setUserDefined(e,t){return this._userDefinedStates.set(e,t)}isRecovered(e){return this._recoveredStates.get(e)}setRecovered(e,t){return this._recoveredStates.set(e,t)}getSource(e){return this.isUserDefined(e)?1:this.isRecovered(e)?2:0}setSource(e,t){t===1?(this.setUserDefined(e,!0),this.setRecovered(e,!1)):t===2?(this.setUserDefined(e,!1),this.setRecovered(e,!0)):(this.setUserDefined(e,!1),this.setRecovered(e,!1))}setCollapsedAllOfType(e,t){let i=!1;if(this._types)for(let n=0;n<this._types.length;n++)this._types[n]===e&&(this.setCollapsed(n,t),i=!0);return i}toRegion(e){return new mxe(this,e)}getParentIndex(e){this.ensureParentIndices();const t=((this._startIndexes[e]&l6)>>>24)+((this._endIndexes[e]&l6)>>>16);return t===a6?-1:t}contains(e,t){return this.getStartLineNumber(e)<=t&&this.getEndLineNumber(e)>=t}findIndex(e){let t=0,i=this._startIndexes.length;if(i===0)return-1;for(;t<i;){const n=Math.floor((t+i)/2);e<this.getStartLineNumber(n)?i=n:t=n+1}return t-1}findRange(e){let t=this.findIndex(e);if(t>=0){if(this.getEndLineNumber(t)>=e)return t;for(t=this.getParentIndex(t);t!==-1;){if(this.contains(t,e))return t;t=this.getParentIndex(t)}}return-1}toString(){const e=[];for(let t=0;t<this.length;t++)e[t]=`[${fxe[this.getSource(t)]}${this.isCollapsed(t)?"+":"-"}] ${this.getStartLineNumber(t)}/${this.getEndLineNumber(t)}`;return e.join(", ")}toFoldRange(e){return{startLineNumber:this._startIndexes[e]&ba,endLineNumber:this._endIndexes[e]&ba,type:this._types?this._types[e]:void 0,isCollapsed:this.isCollapsed(e),source:this.getSource(e)}}static fromFoldRanges(e){const t=e.length,i=new Uint32Array(t),n=new Uint32Array(t);let s=[],r=!1;for(let l=0;l<t;l++){const c=e[l];i[l]=c.startLineNumber,n[l]=c.endLineNumber,s.push(c.type),c.type&&(r=!0)}r||(s=void 0);const a=new To(i,n,s);for(let l=0;l<t;l++)e[l].isCollapsed&&a.setCollapsed(l,!0),a.setSource(l,e[l].source);return a}static sanitizeAndMerge(e,t,i,n){i=i??Number.MAX_VALUE;const s=(_,b)=>Array.isArray(_)?C=>C<b?_[C]:void 0:C=>C<b?_.toFoldRange(C):void 0,r=s(e,e.length),a=s(t,t.length);let l=0,c=0,d=r(0),h=a(0);const u=[];let g,f=0;const p=[];for(;d||h;){let _;if(h&&(!d||d.startLineNumber>=h.startLineNumber))d&&d.startLineNumber===h.startLineNumber?(h.source===1?_=h:(_=d,_.isCollapsed=h.isCollapsed&&(d.endLineNumber===h.endLineNumber||!(n!=null&&n.startsInside(d.startLineNumber+1,d.endLineNumber+1))),_.source=0),d=r(++l)):(_=h,h.isCollapsed&&h.source===0&&(_.source=2)),h=a(++c);else{let b=c,C=h;for(;;){if(!C||C.startLineNumber>d.endLineNumber){_=d;break}if(C.source===1&&C.endLineNumber>d.endLineNumber)break;C=a(++b)}d=r(++l)}if(_){for(;g&&g.endLineNumber<_.startLineNumber;)g=u.pop();_.endLineNumber>_.startLineNumber&&_.startLineNumber>f&&_.endLineNumber<=i&&(!g||g.endLineNumber>=_.endLineNumber)&&(p.push(_),f=_.startLineNumber,g&&u.push(g),g=_)}}return p}}class mxe{constructor(e,t){this.ranges=e,this.index=t}get startLineNumber(){return this.ranges.getStartLineNumber(this.index)}get endLineNumber(){return this.ranges.getEndLineNumber(this.index)}get regionIndex(){return this.index}get parentIndex(){return this.ranges.getParentIndex(this.index)}get isCollapsed(){return this.ranges.isCollapsed(this.index)}containedBy(e){return e.startLineNumber<=this.startLineNumber&&e.endLineNumber>=this.endLineNumber}containsLine(e){return this.startLineNumber<=e&&e<=this.endLineNumber}}class pxe{get regions(){return this._regions}get textModel(){return this._textModel}constructor(e,t){this._updateEventEmitter=new P,this.onDidChange=this._updateEventEmitter.event,this._textModel=e,this._decorationProvider=t,this._regions=new To(new Uint32Array(0),new Uint32Array(0)),this._editorDecorationIds=[]}toggleCollapseState(e){if(!e.length)return;e=e.sort((i,n)=>i.regionIndex-n.regionIndex);const t={};this._decorationProvider.changeDecorations(i=>{let n=0,s=-1,r=-1;const a=l=>{for(;n<l;){const c=this._regions.getEndLineNumber(n),d=this._regions.isCollapsed(n);if(c<=s){const h=this.regions.getSource(n)!==0;i.changeDecorationOptions(this._editorDecorationIds[n],this._decorationProvider.getDecorationOption(d,c<=r,h))}d&&c>r&&(r=c),n++}};for(const l of e){const c=l.regionIndex,d=this._editorDecorationIds[c];if(d&&!t[d]){t[d]=!0,a(c);const h=!this._regions.isCollapsed(c);this._regions.setCollapsed(c,h),s=Math.max(s,this._regions.getEndLineNumber(c))}}a(this._regions.length)}),this._updateEventEmitter.fire({model:this,collapseStateChanged:e})}removeManualRanges(e){const t=new Array,i=n=>{for(const s of e)if(!(s.startLineNumber>n.endLineNumber||n.startLineNumber>s.endLineNumber))return!0;return!1};for(let n=0;n<this._regions.length;n++){const s=this._regions.toFoldRange(n);(s.source===0||!i(s))&&t.push(s)}this.updatePost(To.fromFoldRanges(t))}update(e,t){const i=this._currentFoldedOrManualRanges(t),n=To.sanitizeAndMerge(e,i,this._textModel.getLineCount(),t);this.updatePost(To.fromFoldRanges(n))}updatePost(e){const t=[];let i=-1;for(let n=0,s=e.length;n<s;n++){const r=e.getStartLineNumber(n),a=e.getEndLineNumber(n),l=e.isCollapsed(n),c=e.getSource(n)!==0,d={startLineNumber:r,startColumn:this._textModel.getLineMaxColumn(r),endLineNumber:a,endColumn:this._textModel.getLineMaxColumn(a)+1};t.push({range:d,options:this._decorationProvider.getDecorationOption(l,a<=i,c)}),l&&a>i&&(i=a)}this._decorationProvider.changeDecorations(n=>this._editorDecorationIds=n.deltaDecorations(this._editorDecorationIds,t)),this._regions=e,this._updateEventEmitter.fire({model:this})}_currentFoldedOrManualRanges(e){const t=[];for(let i=0,n=this._regions.length;i<n;i++){let s=this.regions.isCollapsed(i);const r=this.regions.getSource(i);if(s||r!==0){const a=this._regions.toFoldRange(i),l=this._textModel.getDecorationRange(this._editorDecorationIds[i]);l&&(s&&(e!=null&&e.startsInside(l.startLineNumber+1,l.endLineNumber))&&(s=!1),t.push({startLineNumber:l.startLineNumber,endLineNumber:l.endLineNumber,type:a.type,isCollapsed:s,source:r}))}}return t}getMemento(){const e=this._currentFoldedOrManualRanges(),t=[],i=this._textModel.getLineCount();for(let n=0,s=e.length;n<s;n++){const r=e[n];if(r.startLineNumber>=r.endLineNumber||r.startLineNumber<1||r.endLineNumber>i)continue;const a=this._getLinesChecksum(r.startLineNumber+1,r.endLineNumber);t.push({startLineNumber:r.startLineNumber,endLineNumber:r.endLineNumber,isCollapsed:r.isCollapsed,source:r.source,checksum:a})}return t.length>0?t:void 0}applyMemento(e){if(!Array.isArray(e))return;const t=[],i=this._textModel.getLineCount();for(const s of e){if(s.startLineNumber>=s.endLineNumber||s.startLineNumber<1||s.endLineNumber>i)continue;const r=this._getLinesChecksum(s.startLineNumber+1,s.endLineNumber);(!s.checksum||r===s.checksum)&&t.push({startLineNumber:s.startLineNumber,endLineNumber:s.endLineNumber,type:void 0,isCollapsed:s.isCollapsed??!0,source:s.source??0})}const n=To.sanitizeAndMerge(this._regions,t,i);this.updatePost(To.fromFoldRanges(n))}_getLinesChecksum(e,t){return _I(this._textModel.getLineContent(e)+this._textModel.getLineContent(t))%1e6}dispose(){this._decorationProvider.removeDecorations(this._editorDecorationIds)}getAllRegionsAtLine(e,t){const i=[];if(this._regions){let n=this._regions.findRange(e),s=1;for(;n>=0;){const r=this._regions.toRegion(n);(!t||t(r,s))&&i.push(r),s++,n=r.parentIndex}}return i}getRegionAtLine(e){if(this._regions){const t=this._regions.findRange(e);if(t>=0)return this._regions.toRegion(t)}return null}getRegionsInside(e,t){const i=[],n=e?e.regionIndex+1:0,s=e?e.endLineNumber:Number.MAX_VALUE;if(t&&t.length===2){const r=[];for(let a=n,l=this._regions.length;a<l;a++){const c=this._regions.toRegion(a);if(this._regions.getStartLineNumber(a)<s){for(;r.length>0&&!c.containedBy(r[r.length-1]);)r.pop();r.push(c),t(c,r.length)&&i.push(c)}else break}}else for(let r=n,a=this._regions.length;r<a;r++){const l=this._regions.toRegion(r);if(this._regions.getStartLineNumber(r)<s)(!t||t(l))&&i.push(l);else break}return i}}function XW(o,e,t){const i=[];for(const n of t){const s=o.getRegionAtLine(n);if(s){const r=!s.isCollapsed;if(i.push(s),e>1){const a=o.getRegionsInside(s,(l,c)=>l.isCollapsed!==r&&c<e);i.push(...a)}}}o.toggleCollapseState(i)}function Ob(o,e,t=Number.MAX_VALUE,i){const n=[];if(i&&i.length>0)for(const s of i){const r=o.getRegionAtLine(s);if(r&&(r.isCollapsed!==e&&n.push(r),t>1)){const a=o.getRegionsInside(r,(l,c)=>l.isCollapsed!==e&&c<t);n.push(...a)}}else{const s=o.getRegionsInside(null,(r,a)=>r.isCollapsed!==e&&a<t);n.push(...s)}o.toggleCollapseState(n)}function oY(o,e,t,i){const n=[];for(const s of i){const r=o.getAllRegionsAtLine(s,(a,l)=>a.isCollapsed!==e&&l<=t);n.push(...r)}o.toggleCollapseState(n)}function _xe(o,e,t){const i=[];for(const n of t){const s=o.getAllRegionsAtLine(n,r=>r.isCollapsed!==e);s.length>0&&i.push(s[0])}o.toggleCollapseState(i)}function bxe(o,e,t,i){const n=(r,a)=>a===e&&r.isCollapsed!==t&&!i.some(l=>r.containsLine(l)),s=o.getRegionsInside(null,n);o.toggleCollapseState(s)}function rY(o,e,t){const i=[];for(const r of t){const a=o.getAllRegionsAtLine(r,void 0);a.length>0&&i.push(a[0])}const n=r=>i.every(a=>!a.containedBy(r)&&!r.containedBy(a))&&r.isCollapsed!==e,s=o.getRegionsInside(null,n);o.toggleCollapseState(s)}function QW(o,e,t){const i=o.textModel,n=o.regions,s=[];for(let r=n.length-1;r>=0;r--)if(t!==n.isCollapsed(r)){const a=n.getStartLineNumber(r);e.test(i.getLineContent(a))&&s.push(n.toRegion(r))}o.toggleCollapseState(s)}function JW(o,e,t){const i=o.regions,n=[];for(let s=i.length-1;s>=0;s--)t!==i.isCollapsed(s)&&e===i.getType(s)&&n.push(i.toRegion(s));o.toggleCollapseState(n)}function Cxe(o,e){let t=null;const i=e.getRegionAtLine(o);if(i!==null&&(t=i.startLineNumber,o===t)){const n=i.parentIndex;n!==-1?t=e.regions.getStartLineNumber(n):t=null}return t}function vxe(o,e){let t=e.getRegionAtLine(o);if(t!==null&&t.startLineNumber===o){if(o!==t.startLineNumber)return t.startLineNumber;{const i=t.parentIndex;let n=0;for(i!==-1&&(n=e.regions.getStartLineNumber(t.parentIndex));t!==null;)if(t.regionIndex>0){if(t=e.regions.toRegion(t.regionIndex-1),t.startLineNumber<=n)return null;if(t.parentIndex===i)return t.startLineNumber}else return null}}else if(e.regions.length>0)for(t=e.regions.toRegion(e.regions.length-1);t!==null;){if(t.startLineNumber<o)return t.startLineNumber;t.regionIndex>0?t=e.regions.toRegion(t.regionIndex-1):t=null}return null}function wxe(o,e){let t=e.getRegionAtLine(o);if(t!==null&&t.startLineNumber===o){const i=t.parentIndex;let n=0;if(i!==-1)n=e.regions.getEndLineNumber(t.parentIndex);else{if(e.regions.length===0)return null;n=e.regions.getEndLineNumber(e.regions.length-1)}for(;t!==null;)if(t.regionIndex<e.regions.length){if(t=e.regions.toRegion(t.regionIndex+1),t.startLineNumber>=n)return null;if(t.parentIndex===i)return t.startLineNumber}else return null}else if(e.regions.length>0)for(t=e.regions.toRegion(0);t!==null;){if(t.startLineNumber>o)return t.startLineNumber;t.regionIndex<e.regions.length?t=e.regions.toRegion(t.regionIndex+1):t=null}return null}class Sxe{get onDidChange(){return this._updateEventEmitter.event}get hiddenRanges(){return this._hiddenRanges}constructor(e){this._updateEventEmitter=new P,this._hasLineChanges=!1,this._foldingModel=e,this._foldingModelListener=e.onDidChange(t=>this.updateHiddenRanges()),this._hiddenRanges=[],e.regions.length&&this.updateHiddenRanges()}notifyChangeModelContent(e){this._hiddenRanges.length&&!this._hasLineChanges&&(this._hasLineChanges=e.changes.some(t=>t.range.endLineNumber!==t.range.startLineNumber||zh(t.text)[0]!==0))}updateHiddenRanges(){let e=!1;const t=[];let i=0,n=0,s=Number.MAX_VALUE,r=-1;const a=this._foldingModel.regions;for(;i<a.length;i++){if(!a.isCollapsed(i))continue;const l=a.getStartLineNumber(i)+1,c=a.getEndLineNumber(i);s<=l&&c<=r||(!e&&n<this._hiddenRanges.length&&this._hiddenRanges[n].startLineNumber===l&&this._hiddenRanges[n].endLineNumber===c?(t.push(this._hiddenRanges[n]),n++):(e=!0,t.push(new x(l,1,c,1))),s=l,r=c)}(this._hasLineChanges||e||n<this._hiddenRanges.length)&&this.applyHiddenRanges(t)}applyHiddenRanges(e){this._hiddenRanges=e,this._hasLineChanges=!1,this._updateEventEmitter.fire(e)}hasRanges(){return this._hiddenRanges.length>0}isHidden(e){return c6(this._hiddenRanges,e)!==null}adjustSelections(e){let t=!1;const i=this._foldingModel.textModel;let n=null;const s=r=>((!n||!yxe(r,n))&&(n=c6(this._hiddenRanges,r)),n?n.startLineNumber-1:null);for(let r=0,a=e.length;r<a;r++){let l=e[r];const c=s(l.startLineNumber);c&&(l=l.setStartPosition(c,i.getLineMaxColumn(c)),t=!0);const d=s(l.endLineNumber);d&&(l=l.setEndPosition(d,i.getLineMaxColumn(d)),t=!0),e[r]=l}return t}dispose(){this.hiddenRanges.length>0&&(this._hiddenRanges=[],this._updateEventEmitter.fire(this._hiddenRanges)),this._foldingModelListener&&(this._foldingModelListener.dispose(),this._foldingModelListener=null)}}function yxe(o,e){return o>=e.startLineNumber&&o<=e.endLineNumber}function c6(o,e){const t=uw(o,i=>e<i.startLineNumber)-1;return t>=0&&o[t].endLineNumber>=e?o[t]:null}const Lxe=5e3,xxe="indent";class eB{constructor(e,t,i){this.editorModel=e,this.languageConfigurationService=t,this.foldingRangesLimit=i,this.id=xxe}dispose(){}compute(e){const t=this.languageConfigurationService.getLanguageConfiguration(this.editorModel.getLanguageId()).foldingRules,i=t&&!!t.offSide,n=t&&t.markers;return Promise.resolve(Ixe(this.editorModel,i,n,this.foldingRangesLimit))}}let kxe=class{constructor(e){this._startIndexes=[],this._endIndexes=[],this._indentOccurrences=[],this._length=0,this._foldingRangesLimit=e}insertFirst(e,t,i){if(e>ba||t>ba)return;const n=this._length;this._startIndexes[n]=e,this._endIndexes[n]=t,this._length++,i<1e3&&(this._indentOccurrences[i]=(this._indentOccurrences[i]||0)+1)}toIndentRanges(e){const t=this._foldingRangesLimit.limit;if(this._length<=t){this._foldingRangesLimit.update(this._length,!1);const i=new Uint32Array(this._length),n=new Uint32Array(this._length);for(let s=this._length-1,r=0;s>=0;s--,r++)i[r]=this._startIndexes[s],n[r]=this._endIndexes[s];return new To(i,n)}else{this._foldingRangesLimit.update(this._length,t);let i=0,n=this._indentOccurrences.length;for(let l=0;l<this._indentOccurrences.length;l++){const c=this._indentOccurrences[l];if(c){if(c+i>t){n=l;break}i+=c}}const s=e.getOptions().tabSize,r=new Uint32Array(t),a=new Uint32Array(t);for(let l=this._length-1,c=0;l>=0;l--){const d=this._startIndexes[l],h=e.getLineContent(d),u=rE(h,s);(u<n||u===n&&i++<t)&&(r[c]=d,a[c]=this._endIndexes[l],c++)}return new To(r,a)}}};const Dxe={limit:Lxe,update:()=>{}};function Ixe(o,e,t,i=Dxe){const n=o.getOptions().tabSize,s=new kxe(i);let r;t&&(r=new RegExp(`(${t.start.source})|(?:${t.end.source})`));const a=[],l=o.getLineCount()+1;a.push({indent:-1,endAbove:l,line:l});for(let c=o.getLineCount();c>0;c--){const d=o.getLineContent(c),h=rE(d,n);let u=a[a.length-1];if(h===-1){e&&(u.endAbove=c);continue}let g;if(r&&(g=d.match(r)))if(g[1]){let f=a.length-1;for(;f>0&&a[f].indent!==-2;)f--;if(f>0){a.length=f+1,u=a[f],s.insertFirst(c,u.line,h),u.line=c,u.indent=h,u.endAbove=c;continue}}else{a.push({indent:-2,endAbove:c,line:c});continue}if(u.indent>h){do a.pop(),u=a[a.length-1];while(u.indent>h);const f=u.endAbove-1;f-c>=1&&s.insertFirst(c,f,h)}u.indent===h?u.endAbove=c:a.push({indent:h,endAbove:c,line:c})}return s.toIndentRanges(o)}const Exe=E("editor.foldBackground",{light:$e(Gd,.3),dark:$e(Gd,.3),hcDark:null,hcLight:null},m("foldBackgroundBackground","Background color behind folded ranges. The color must not be opaque so as not to hide underlying decorations."),!0);E("editor.foldPlaceholderForeground",{light:"#808080",dark:"#808080",hcDark:null,hcLight:null},m("collapsedTextColor","Color of the collapsed text after the first line of a folded range."));E("editorGutter.foldingControlForeground",lL,m("editorGutter.foldingControlForeground","Color of the folding control in the editor gutter."));const $x=qt("folding-expanded",te.chevronDown,m("foldingExpandedIcon","Icon for expanded ranges in the editor glyph margin.")),jx=qt("folding-collapsed",te.chevronRight,m("foldingCollapsedIcon","Icon for collapsed ranges in the editor glyph margin.")),d6=qt("folding-manual-collapsed",jx,m("foldingManualCollapedIcon","Icon for manually collapsed ranges in the editor glyph margin.")),h6=qt("folding-manual-expanded",$x,m("foldingManualExpandedIcon","Icon for manually expanded ranges in the editor glyph margin.")),$T={color:mi(Exe),position:1},Wm=m("linesCollapsed","Click to expand the range."),k0=m("linesExpanded","Click to collapse the range."),Wi=class Wi{constructor(e){this.editor=e,this.showFoldingControls="mouseover",this.showFoldingHighlights=!0}getDecorationOption(e,t,i){return t?Wi.HIDDEN_RANGE_DECORATION:this.showFoldingControls==="never"?e?this.showFoldingHighlights?Wi.NO_CONTROLS_COLLAPSED_HIGHLIGHTED_RANGE_DECORATION:Wi.NO_CONTROLS_COLLAPSED_RANGE_DECORATION:Wi.NO_CONTROLS_EXPANDED_RANGE_DECORATION:e?i?this.showFoldingHighlights?Wi.MANUALLY_COLLAPSED_HIGHLIGHTED_VISUAL_DECORATION:Wi.MANUALLY_COLLAPSED_VISUAL_DECORATION:this.showFoldingHighlights?Wi.COLLAPSED_HIGHLIGHTED_VISUAL_DECORATION:Wi.COLLAPSED_VISUAL_DECORATION:this.showFoldingControls==="mouseover"?i?Wi.MANUALLY_EXPANDED_AUTO_HIDE_VISUAL_DECORATION:Wi.EXPANDED_AUTO_HIDE_VISUAL_DECORATION:i?Wi.MANUALLY_EXPANDED_VISUAL_DECORATION:Wi.EXPANDED_VISUAL_DECORATION}changeDecorations(e){return this.editor.changeDecorations(e)}removeDecorations(e){this.editor.removeDecorations(e)}};Wi.COLLAPSED_VISUAL_DECORATION=We.register({description:"folding-collapsed-visual-decoration",stickiness:0,afterContentClassName:"inline-folded",isWholeLine:!0,linesDecorationsTooltip:Wm,firstLineDecorationClassName:Ie.asClassName(jx)}),Wi.COLLAPSED_HIGHLIGHTED_VISUAL_DECORATION=We.register({description:"folding-collapsed-highlighted-visual-decoration",stickiness:0,afterContentClassName:"inline-folded",className:"folded-background",minimap:$T,isWholeLine:!0,linesDecorationsTooltip:Wm,firstLineDecorationClassName:Ie.asClassName(jx)}),Wi.MANUALLY_COLLAPSED_VISUAL_DECORATION=We.register({description:"folding-manually-collapsed-visual-decoration",stickiness:0,afterContentClassName:"inline-folded",isWholeLine:!0,linesDecorationsTooltip:Wm,firstLineDecorationClassName:Ie.asClassName(d6)}),Wi.MANUALLY_COLLAPSED_HIGHLIGHTED_VISUAL_DECORATION=We.register({description:"folding-manually-collapsed-highlighted-visual-decoration",stickiness:0,afterContentClassName:"inline-folded",className:"folded-background",minimap:$T,isWholeLine:!0,linesDecorationsTooltip:Wm,firstLineDecorationClassName:Ie.asClassName(d6)}),Wi.NO_CONTROLS_COLLAPSED_RANGE_DECORATION=We.register({description:"folding-no-controls-range-decoration",stickiness:0,afterContentClassName:"inline-folded",isWholeLine:!0,linesDecorationsTooltip:Wm}),Wi.NO_CONTROLS_COLLAPSED_HIGHLIGHTED_RANGE_DECORATION=We.register({description:"folding-no-controls-range-decoration",stickiness:0,afterContentClassName:"inline-folded",className:"folded-background",minimap:$T,isWholeLine:!0,linesDecorationsTooltip:Wm}),Wi.EXPANDED_VISUAL_DECORATION=We.register({description:"folding-expanded-visual-decoration",stickiness:1,isWholeLine:!0,firstLineDecorationClassName:"alwaysShowFoldIcons "+Ie.asClassName($x),linesDecorationsTooltip:k0}),Wi.EXPANDED_AUTO_HIDE_VISUAL_DECORATION=We.register({description:"folding-expanded-auto-hide-visual-decoration",stickiness:1,isWholeLine:!0,firstLineDecorationClassName:Ie.asClassName($x),linesDecorationsTooltip:k0}),Wi.MANUALLY_EXPANDED_VISUAL_DECORATION=We.register({description:"folding-manually-expanded-visual-decoration",stickiness:0,isWholeLine:!0,firstLineDecorationClassName:"alwaysShowFoldIcons "+Ie.asClassName(h6),linesDecorationsTooltip:k0}),Wi.MANUALLY_EXPANDED_AUTO_HIDE_VISUAL_DECORATION=We.register({description:"folding-manually-expanded-auto-hide-visual-decoration",stickiness:0,isWholeLine:!0,firstLineDecorationClassName:Ie.asClassName(h6),linesDecorationsTooltip:k0}),Wi.NO_CONTROLS_EXPANDED_RANGE_DECORATION=We.register({description:"folding-no-controls-range-decoration",stickiness:0,isWholeLine:!0}),Wi.HIDDEN_RANGE_DECORATION=We.register({description:"folding-hidden-range-decoration",stickiness:1});let GO=Wi;const Nxe={},Txe="syntax";class tB{constructor(e,t,i,n,s){this.editorModel=e,this.providers=t,this.handleFoldingRangesChange=i,this.foldingRangesLimit=n,this.fallbackRangeProvider=s,this.id=Txe,this.disposables=new K,s&&this.disposables.add(s);for(const r of t)typeof r.onDidChange=="function"&&this.disposables.add(r.onDidChange(i))}compute(e){return Rxe(this.providers,this.editorModel,e).then(t=>{var i;return t?Axe(t,this.foldingRangesLimit):((i=this.fallbackRangeProvider)==null?void 0:i.compute(e))??null})}dispose(){this.disposables.dispose()}}function Rxe(o,e,t){let i=null;const n=o.map((s,r)=>Promise.resolve(s.provideFoldingRanges(e,Nxe,t)).then(a=>{if(!t.isCancellationRequested&&Array.isArray(a)){Array.isArray(i)||(i=[]);const l=e.getLineCount();for(const c of a)c.start>0&&c.end>c.start&&c.end<=l&&i.push({start:c.start,end:c.end,rank:r,kind:c.kind})}},Si));return Promise.all(n).then(s=>i)}class Mxe{constructor(e){this._startIndexes=[],this._endIndexes=[],this._nestingLevels=[],this._nestingLevelCounts=[],this._types=[],this._length=0,this._foldingRangesLimit=e}add(e,t,i,n){if(e>ba||t>ba)return;const s=this._length;this._startIndexes[s]=e,this._endIndexes[s]=t,this._nestingLevels[s]=n,this._types[s]=i,this._length++,n<30&&(this._nestingLevelCounts[n]=(this._nestingLevelCounts[n]||0)+1)}toIndentRanges(){const e=this._foldingRangesLimit.limit;if(this._length<=e){this._foldingRangesLimit.update(this._length,!1);const t=new Uint32Array(this._length),i=new Uint32Array(this._length);for(let n=0;n<this._length;n++)t[n]=this._startIndexes[n],i[n]=this._endIndexes[n];return new To(t,i,this._types)}else{this._foldingRangesLimit.update(this._length,e);let t=0,i=this._nestingLevelCounts.length;for(let a=0;a<this._nestingLevelCounts.length;a++){const l=this._nestingLevelCounts[a];if(l){if(l+t>e){i=a;break}t+=l}}const n=new Uint32Array(e),s=new Uint32Array(e),r=[];for(let a=0,l=0;a<this._length;a++){const c=this._nestingLevels[a];(c<i||c===i&&t++<e)&&(n[l]=this._startIndexes[a],s[l]=this._endIndexes[a],r[l]=this._types[a],l++)}return new To(n,s,r)}}}function Axe(o,e){const t=o.sort((r,a)=>{let l=r.start-a.start;return l===0&&(l=r.rank-a.rank),l}),i=new Mxe(e);let n;const s=[];for(const r of t)if(!n)n=r,i.add(r.start,r.end,r.kind&&r.kind.value,s.length);else if(r.start>n.start)if(r.end<=n.end)s.push(n),n=r,i.add(r.start,r.end,r.kind&&r.kind.value,s.length);else{if(r.start>n.end){do n=s.pop();while(n&&r.start>n.end);n&&s.push(n),n=r}i.add(r.start,r.end,r.kind&&r.kind.value,s.length)}return i.toIndentRanges()}var Pxe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},mC=function(o,e){return function(t,i){e(t,i,o)}},Wu;const Tn=new le("foldingEnabled",!1);var ef;let Qh=(ef=class extends F{static get(e){return e.getContribution(Wu.ID)}static getFoldingRangeProviders(e,t){var n;const i=e.foldingRangeProvider.ordered(t);return((n=Wu._foldingRangeSelector)==null?void 0:n.call(Wu,i,t))??i}constructor(e,t,i,n,s,r){super(),this.contextKeyService=t,this.languageConfigurationService=i,this.languageFeaturesService=r,this.localToDispose=this._register(new K),this.editor=e,this._foldingLimitReporter=new aY(e);const a=this.editor.getOptions();this._isEnabled=a.get(43),this._useFoldingProviders=a.get(44)!=="indentation",this._unfoldOnClickAfterEndOfLine=a.get(48),this._restoringViewState=!1,this._currentModelHasFoldedImports=!1,this._foldingImportsByDefault=a.get(46),this.updateDebounceInfo=s.for(r.foldingRangeProvider,"Folding",{min:200}),this.foldingModel=null,this.hiddenRangeModel=null,this.rangeProvider=null,this.foldingRegionPromise=null,this.foldingModelPromise=null,this.updateScheduler=null,this.cursorChangedScheduler=null,this.mouseDownInfo=null,this.foldingDecorationProvider=new GO(e),this.foldingDecorationProvider.showFoldingControls=a.get(111),this.foldingDecorationProvider.showFoldingHighlights=a.get(45),this.foldingEnabled=Tn.bindTo(this.contextKeyService),this.foldingEnabled.set(this._isEnabled),this._register(this.editor.onDidChangeModel(()=>this.onModelChanged())),this._register(this.editor.onDidChangeConfiguration(l=>{if(l.hasChanged(43)&&(this._isEnabled=this.editor.getOptions().get(43),this.foldingEnabled.set(this._isEnabled),this.onModelChanged()),l.hasChanged(47)&&this.onModelChanged(),l.hasChanged(111)||l.hasChanged(45)){const c=this.editor.getOptions();this.foldingDecorationProvider.showFoldingControls=c.get(111),this.foldingDecorationProvider.showFoldingHighlights=c.get(45),this.triggerFoldingModelChanged()}l.hasChanged(44)&&(this._useFoldingProviders=this.editor.getOptions().get(44)!=="indentation",this.onFoldingStrategyChanged()),l.hasChanged(48)&&(this._unfoldOnClickAfterEndOfLine=this.editor.getOptions().get(48)),l.hasChanged(46)&&(this._foldingImportsByDefault=this.editor.getOptions().get(46))})),this.onModelChanged()}saveViewState(){const e=this.editor.getModel();if(!e||!this._isEnabled||e.isTooLargeForTokenization())return{};if(this.foldingModel){const t=this.foldingModel.getMemento(),i=this.rangeProvider?this.rangeProvider.id:void 0;return{collapsedRegions:t,lineCount:e.getLineCount(),provider:i,foldedImports:this._currentModelHasFoldedImports}}}restoreViewState(e){const t=this.editor.getModel();if(!(!t||!this._isEnabled||t.isTooLargeForTokenization()||!this.hiddenRangeModel)&&e&&(this._currentModelHasFoldedImports=!!e.foldedImports,e.collapsedRegions&&e.collapsedRegions.length>0&&this.foldingModel)){this._restoringViewState=!0;try{this.foldingModel.applyMemento(e.collapsedRegions)}finally{this._restoringViewState=!1}}}onModelChanged(){this.localToDispose.clear();const e=this.editor.getModel();!this._isEnabled||!e||e.isTooLargeForTokenization()||(this._currentModelHasFoldedImports=!1,this.foldingModel=new pxe(e,this.foldingDecorationProvider),this.localToDispose.add(this.foldingModel),this.hiddenRangeModel=new Sxe(this.foldingModel),this.localToDispose.add(this.hiddenRangeModel),this.localToDispose.add(this.hiddenRangeModel.onDidChange(t=>this.onHiddenRangesChanges(t))),this.updateScheduler=new _r(this.updateDebounceInfo.get(e)),this.cursorChangedScheduler=new It(()=>this.revealCursor(),200),this.localToDispose.add(this.cursorChangedScheduler),this.localToDispose.add(this.languageFeaturesService.foldingRangeProvider.onDidChange(()=>this.onFoldingStrategyChanged())),this.localToDispose.add(this.editor.onDidChangeModelLanguageConfiguration(()=>this.onFoldingStrategyChanged())),this.localToDispose.add(this.editor.onDidChangeModelContent(t=>this.onDidChangeModelContent(t))),this.localToDispose.add(this.editor.onDidChangeCursorPosition(()=>this.onCursorPositionChanged())),this.localToDispose.add(this.editor.onMouseDown(t=>this.onEditorMouseDown(t))),this.localToDispose.add(this.editor.onMouseUp(t=>this.onEditorMouseUp(t))),this.localToDispose.add({dispose:()=>{var t,i;this.foldingRegionPromise&&(this.foldingRegionPromise.cancel(),this.foldingRegionPromise=null),(t=this.updateScheduler)==null||t.cancel(),this.updateScheduler=null,this.foldingModel=null,this.foldingModelPromise=null,this.hiddenRangeModel=null,this.cursorChangedScheduler=null,(i=this.rangeProvider)==null||i.dispose(),this.rangeProvider=null}}),this.triggerFoldingModelChanged())}onFoldingStrategyChanged(){var e;(e=this.rangeProvider)==null||e.dispose(),this.rangeProvider=null,this.triggerFoldingModelChanged()}getRangeProvider(e){if(this.rangeProvider)return this.rangeProvider;const t=new eB(e,this.languageConfigurationService,this._foldingLimitReporter);if(this.rangeProvider=t,this._useFoldingProviders&&this.foldingModel){const i=Wu.getFoldingRangeProviders(this.languageFeaturesService,e);i.length>0&&(this.rangeProvider=new tB(e,i,()=>this.triggerFoldingModelChanged(),this._foldingLimitReporter,t))}return this.rangeProvider}getFoldingModel(){return this.foldingModelPromise}onDidChangeModelContent(e){var t;(t=this.hiddenRangeModel)==null||t.notifyChangeModelContent(e),this.triggerFoldingModelChanged()}triggerFoldingModelChanged(){this.updateScheduler&&(this.foldingRegionPromise&&(this.foldingRegionPromise.cancel(),this.foldingRegionPromise=null),this.foldingModelPromise=this.updateScheduler.trigger(()=>{const e=this.foldingModel;if(!e)return null;const t=new ln,i=this.getRangeProvider(e.textModel),n=this.foldingRegionPromise=Yi(s=>i.compute(s));return n.then(s=>{if(s&&n===this.foldingRegionPromise){let r;if(this._foldingImportsByDefault&&!this._currentModelHasFoldedImports){const c=s.setCollapsedAllOfType(Ph.Imports.value,!0);c&&(r=za.capture(this.editor),this._currentModelHasFoldedImports=c)}const a=this.editor.getSelections();e.update(s,Oxe(a)),r==null||r.restore(this.editor);const l=this.updateDebounceInfo.update(e.textModel,t.elapsed());this.updateScheduler&&(this.updateScheduler.defaultDelay=l)}return e})}).then(void 0,e=>(Oe(e),null)))}onHiddenRangesChanges(e){if(this.hiddenRangeModel&&e.length&&!this._restoringViewState){const t=this.editor.getSelections();t&&this.hiddenRangeModel.adjustSelections(t)&&this.editor.setSelections(t)}this.editor.setHiddenAreas(e,this)}onCursorPositionChanged(){this.hiddenRangeModel&&this.hiddenRangeModel.hasRanges()&&this.cursorChangedScheduler.schedule()}revealCursor(){const e=this.getFoldingModel();e&&e.then(t=>{if(t){const i=this.editor.getSelections();if(i&&i.length>0){const n=[];for(const s of i){const r=s.selectionStartLineNumber;this.hiddenRangeModel&&this.hiddenRangeModel.isHidden(r)&&n.push(...t.getAllRegionsAtLine(r,a=>a.isCollapsed&&r>a.startLineNumber))}n.length&&(t.toggleCollapseState(n),this.reveal(i[0].getPosition()))}}}).then(void 0,Oe)}onEditorMouseDown(e){if(this.mouseDownInfo=null,!this.hiddenRangeModel||!e.target||!e.target.range||!e.event.leftButton&&!e.event.middleButton)return;const t=e.target.range;let i=!1;switch(e.target.type){case 4:{const n=e.target.detail,s=e.target.element.offsetLeft;if(n.offsetX-s<4)return;i=!0;break}case 7:{if(this._unfoldOnClickAfterEndOfLine&&this.hiddenRangeModel.hasRanges()&&!e.target.detail.isAfterLines)break;return}case 6:{if(this.hiddenRangeModel.hasRanges()){const n=this.editor.getModel();if(n&&t.startColumn===n.getLineMaxColumn(t.startLineNumber))break}return}default:return}this.mouseDownInfo={lineNumber:t.startLineNumber,iconClicked:i}}onEditorMouseUp(e){const t=this.foldingModel;if(!t||!this.mouseDownInfo||!e.target)return;const i=this.mouseDownInfo.lineNumber,n=this.mouseDownInfo.iconClicked,s=e.target.range;if(!s||s.startLineNumber!==i)return;if(n){if(e.target.type!==4)return}else{const a=this.editor.getModel();if(!a||s.startColumn!==a.getLineMaxColumn(i))return}const r=t.getRegionAtLine(i);if(r&&r.startLineNumber===i){const a=r.isCollapsed;if(n||a){const l=e.event.altKey;let c=[];if(l){const d=u=>!u.containedBy(r)&&!r.containedBy(u),h=t.getRegionsInside(null,d);for(const u of h)u.isCollapsed&&c.push(u);c.length===0&&(c=h)}else{const d=e.event.middleButton||e.event.shiftKey;if(d)for(const h of t.getRegionsInside(r))h.isCollapsed===a&&c.push(h);(a||!d||c.length===0)&&c.push(r)}t.toggleCollapseState(c),this.reveal({lineNumber:i,column:1})}}}reveal(e){this.editor.revealPositionInCenterIfOutsideViewport(e,0)}},Wu=ef,ef.ID="editor.contrib.folding",ef);Qh=Wu=Pxe([mC(1,Ee),mC(2,di),mC(3,Ei),mC(4,Zo),mC(5,ge)],Qh);class aY{constructor(e){this.editor=e,this._onDidChange=new P,this._computed=0,this._limited=!1}get limit(){return this.editor.getOptions().get(47)}update(e,t){(e!==this._computed||t!==this._limited)&&(this._computed=e,this._limited=t,this._onDidChange.fire())}}class Vn extends de{runEditorCommand(e,t,i){const n=e.get(di),s=Qh.get(t);if(!s)return;const r=s.getFoldingModel();if(r)return this.reportTelemetry(e,t),r.then(a=>{if(a){this.invoke(s,a,t,i,n);const l=t.getSelection();l&&s.reveal(l.getStartPosition())}})}getSelectedLines(e){const t=e.getSelections();return t?t.map(i=>i.startLineNumber):[]}getLineNumbers(e,t){return e&&e.selectionLines?e.selectionLines.map(i=>i+1):this.getSelectedLines(t)}run(e,t){}}function Oxe(o){return!o||o.length===0?{startsInside:()=>!1}:{startsInside(e,t){for(const i of o){const n=i.startLineNumber;if(n>=e&&n<=t)return!0}return!1}}}function lY(o){if(!Xn(o)){if(!on(o))return!1;const e=o;if(!Xn(e.levels)&&!Rh(e.levels)||!Xn(e.direction)&&!As(e.direction)||!Xn(e.selectionLines)&&(!Array.isArray(e.selectionLines)||!e.selectionLines.every(Rh)))return!1}return!0}class Fxe extends Vn{constructor(){super({id:"editor.unfold",label:m("unfoldAction.label","Unfold"),alias:"Unfold",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:3166,mac:{primary:2654},weight:100},metadata:{description:"Unfold the content in the editor",args:[{name:"Unfold editor argument",description:`Property-value pairs that can be passed through this argument:
						* 'levels': Number of levels to unfold. If not set, defaults to 1.
						* 'direction': If 'up', unfold given number of levels up otherwise unfolds down.
						* 'selectionLines': Array of the start lines (0-based) of the editor selections to apply the unfold action to. If not set, the active selection(s) will be used.
						`,constraint:lY,schema:{type:"object",properties:{levels:{type:"number",default:1},direction:{type:"string",enum:["up","down"],default:"down"},selectionLines:{type:"array",items:{type:"number"}}}}}]}})}invoke(e,t,i,n){const s=n&&n.levels||1,r=this.getLineNumbers(n,i);n&&n.direction==="up"?oY(t,!1,s,r):Ob(t,!1,s,r)}}class Wxe extends Vn{constructor(){super({id:"editor.unfoldRecursively",label:m("unFoldRecursivelyAction.label","Unfold Recursively"),alias:"Unfold Recursively",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2142),weight:100}})}invoke(e,t,i,n){Ob(t,!1,Number.MAX_VALUE,this.getSelectedLines(i))}}class Bxe extends Vn{constructor(){super({id:"editor.fold",label:m("foldAction.label","Fold"),alias:"Fold",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:3164,mac:{primary:2652},weight:100},metadata:{description:"Fold the content in the editor",args:[{name:"Fold editor argument",description:`Property-value pairs that can be passed through this argument:
							* 'levels': Number of levels to fold.
							* 'direction': If 'up', folds given number of levels up otherwise folds down.
							* 'selectionLines': Array of the start lines (0-based) of the editor selections to apply the fold action to. If not set, the active selection(s) will be used.
							If no levels or direction is set, folds the region at the locations or if already collapsed, the first uncollapsed parent instead.
						`,constraint:lY,schema:{type:"object",properties:{levels:{type:"number"},direction:{type:"string",enum:["up","down"]},selectionLines:{type:"array",items:{type:"number"}}}}}]}})}invoke(e,t,i,n){const s=this.getLineNumbers(n,i),r=n&&n.levels,a=n&&n.direction;typeof r!="number"&&typeof a!="string"?_xe(t,!0,s):a==="up"?oY(t,!0,r||1,s):Ob(t,!0,r||1,s)}}class Hxe extends Vn{constructor(){super({id:"editor.toggleFold",label:m("toggleFoldAction.label","Toggle Fold"),alias:"Toggle Fold",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2090),weight:100}})}invoke(e,t,i){const n=this.getSelectedLines(i);XW(t,1,n)}}class Vxe extends Vn{constructor(){super({id:"editor.foldRecursively",label:m("foldRecursivelyAction.label","Fold Recursively"),alias:"Fold Recursively",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2140),weight:100}})}invoke(e,t,i){const n=this.getSelectedLines(i);Ob(t,!0,Number.MAX_VALUE,n)}}class zxe extends Vn{constructor(){super({id:"editor.toggleFoldRecursively",label:m("toggleFoldRecursivelyAction.label","Toggle Fold Recursively"),alias:"Toggle Fold Recursively",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,3114),weight:100}})}invoke(e,t,i){const n=this.getSelectedLines(i);XW(t,Number.MAX_VALUE,n)}}class Uxe extends Vn{constructor(){super({id:"editor.foldAllBlockComments",label:m("foldAllBlockComments.label","Fold All Block Comments"),alias:"Fold All Block Comments",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2138),weight:100}})}invoke(e,t,i,n,s){if(t.regions.hasTypes())JW(t,Ph.Comment.value,!0);else{const r=i.getModel();if(!r)return;const a=s.getLanguageConfiguration(r.getLanguageId()).comments;if(a&&a.blockCommentStartToken){const l=new RegExp("^\\s*"+Uo(a.blockCommentStartToken));QW(t,l,!0)}}}}class $xe extends Vn{constructor(){super({id:"editor.foldAllMarkerRegions",label:m("foldAllMarkerRegions.label","Fold All Regions"),alias:"Fold All Regions",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2077),weight:100}})}invoke(e,t,i,n,s){if(t.regions.hasTypes())JW(t,Ph.Region.value,!0);else{const r=i.getModel();if(!r)return;const a=s.getLanguageConfiguration(r.getLanguageId()).foldingRules;if(a&&a.markers&&a.markers.start){const l=new RegExp(a.markers.start);QW(t,l,!0)}}}}class jxe extends Vn{constructor(){super({id:"editor.unfoldAllMarkerRegions",label:m("unfoldAllMarkerRegions.label","Unfold All Regions"),alias:"Unfold All Regions",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2078),weight:100}})}invoke(e,t,i,n,s){if(t.regions.hasTypes())JW(t,Ph.Region.value,!1);else{const r=i.getModel();if(!r)return;const a=s.getLanguageConfiguration(r.getLanguageId()).foldingRules;if(a&&a.markers&&a.markers.start){const l=new RegExp(a.markers.start);QW(t,l,!1)}}}}class Kxe extends Vn{constructor(){super({id:"editor.foldAllExcept",label:m("foldAllExcept.label","Fold All Except Selected"),alias:"Fold All Except Selected",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2136),weight:100}})}invoke(e,t,i){const n=this.getSelectedLines(i);rY(t,!0,n)}}class qxe extends Vn{constructor(){super({id:"editor.unfoldAllExcept",label:m("unfoldAllExcept.label","Unfold All Except Selected"),alias:"Unfold All Except Selected",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2134),weight:100}})}invoke(e,t,i){const n=this.getSelectedLines(i);rY(t,!1,n)}}class Gxe extends Vn{constructor(){super({id:"editor.foldAll",label:m("foldAllAction.label","Fold All"),alias:"Fold All",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2069),weight:100}})}invoke(e,t,i){Ob(t,!0)}}class Zxe extends Vn{constructor(){super({id:"editor.unfoldAll",label:m("unfoldAllAction.label","Unfold All"),alias:"Unfold All",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2088),weight:100}})}invoke(e,t,i){Ob(t,!1)}}const Pp=class Pp extends Vn{getFoldingLevel(){return parseInt(this.id.substr(Pp.ID_PREFIX.length))}invoke(e,t,i){bxe(t,this.getFoldingLevel(),!0,this.getSelectedLines(i))}};Pp.ID_PREFIX="editor.foldLevel",Pp.ID=e=>Pp.ID_PREFIX+e;let Kx=Pp;class Yxe extends Vn{constructor(){super({id:"editor.gotoParentFold",label:m("gotoParentFold.label","Go to Parent Fold"),alias:"Go to Parent Fold",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,weight:100}})}invoke(e,t,i){const n=this.getSelectedLines(i);if(n.length>0){const s=Cxe(n[0],t);s!==null&&i.setSelection({startLineNumber:s,startColumn:1,endLineNumber:s,endColumn:1})}}}class Xxe extends Vn{constructor(){super({id:"editor.gotoPreviousFold",label:m("gotoPreviousFold.label","Go to Previous Folding Range"),alias:"Go to Previous Folding Range",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,weight:100}})}invoke(e,t,i){const n=this.getSelectedLines(i);if(n.length>0){const s=vxe(n[0],t);s!==null&&i.setSelection({startLineNumber:s,startColumn:1,endLineNumber:s,endColumn:1})}}}class Qxe extends Vn{constructor(){super({id:"editor.gotoNextFold",label:m("gotoNextFold.label","Go to Next Folding Range"),alias:"Go to Next Folding Range",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,weight:100}})}invoke(e,t,i){const n=this.getSelectedLines(i);if(n.length>0){const s=wxe(n[0],t);s!==null&&i.setSelection({startLineNumber:s,startColumn:1,endLineNumber:s,endColumn:1})}}}class Jxe extends Vn{constructor(){super({id:"editor.createFoldingRangeFromSelection",label:m("createManualFoldRange.label","Create Folding Range from Selection"),alias:"Create Folding Range from Selection",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2135),weight:100}})}invoke(e,t,i){var r;const n=[],s=i.getSelections();if(s){for(const a of s){let l=a.endLineNumber;a.endColumn===1&&--l,l>a.startLineNumber&&(n.push({startLineNumber:a.startLineNumber,endLineNumber:l,type:void 0,isCollapsed:!0,source:1}),i.setSelection({startLineNumber:a.startLineNumber,startColumn:1,endLineNumber:a.startLineNumber,endColumn:1}))}if(n.length>0){n.sort((l,c)=>l.startLineNumber-c.startLineNumber);const a=To.sanitizeAndMerge(t.regions,n,(r=i.getModel())==null?void 0:r.getLineCount());t.updatePost(To.fromFoldRanges(a))}}}}class eke extends Vn{constructor(){super({id:"editor.removeManualFoldingRanges",label:m("removeManualFoldingRanges.label","Remove Manual Folding Ranges"),alias:"Remove Manual Folding Ranges",precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2137),weight:100}})}invoke(e,t,i){const n=i.getSelections();if(n){const s=[];for(const r of n){const{startLineNumber:a,endLineNumber:l}=r;s.push(l>=a?{startLineNumber:a,endLineNumber:l}:{endLineNumber:l,startLineNumber:a})}t.removeManualRanges(s),e.triggerFoldingModelChanged()}}}ct(Qh.ID,Qh,0);Q(Fxe);Q(Wxe);Q(Bxe);Q(Vxe);Q(zxe);Q(Gxe);Q(Zxe);Q(Uxe);Q($xe);Q(jxe);Q(Kxe);Q(qxe);Q(Hxe);Q(Yxe);Q(Xxe);Q(Qxe);Q(Jxe);Q(eke);for(let o=1;o<=7;o++)Aie(new Kx({id:Kx.ID(o),label:m("foldLevelAction.label","Fold Level {0}",o),alias:`Fold Level ${o}`,precondition:Tn,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2048|21+o),weight:100}}));rt.registerCommand("_executeFoldingRangeProvider",async function(o,...e){const[t]=e;if(!(t instanceof Se))throw gr();const i=o.get(ge),n=o.get(Kt).getModel(t);if(!n)throw gr();const s=o.get(qe);if(!s.getValue("editor.folding",{resource:t}))return[];const r=o.get(di),a=s.getValue("editor.foldingStrategy",{resource:t}),l={get limit(){return s.getValue("editor.foldingMaximumRegions",{resource:t})},update:(g,f)=>{}},c=new eB(n,r,l);let d=c;if(a!=="indentation"){const g=Qh.getFoldingRangeProviders(i,n);g.length&&(d=new tB(n,g,()=>{},l,c))}const h=await d.compute(ze.None),u=[];try{if(h)for(let g=0;g<h.length;g++){const f=h.getType(g);u.push({start:h.getStartLineNumber(g),end:h.getEndLineNumber(g),kind:f?Ph.fromValue(f):void 0})}return u}finally{d.dispose()}});class tke extends de{constructor(){super({id:"editor.action.fontZoomIn",label:m("EditorFontZoomIn.label","Increase Editor Font Size"),alias:"Increase Editor Font Size",precondition:void 0})}run(e,t){lr.setZoomLevel(lr.getZoomLevel()+1)}}class ike extends de{constructor(){super({id:"editor.action.fontZoomOut",label:m("EditorFontZoomOut.label","Decrease Editor Font Size"),alias:"Decrease Editor Font Size",precondition:void 0})}run(e,t){lr.setZoomLevel(lr.getZoomLevel()-1)}}class nke extends de{constructor(){super({id:"editor.action.fontZoomReset",label:m("EditorFontZoomReset.label","Reset Editor Font Size"),alias:"Reset Editor Font Size",precondition:void 0})}run(e,t){lr.setZoomLevel(0)}}Q(tke);Q(ike);Q(nke);var cY=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Sv=function(o,e){return function(t,i){e(t,i,o)}},p_;let qx=(p_=class{constructor(e,t,i,n){this._editor=e,this._languageFeaturesService=t,this._workerService=i,this._accessibilitySignalService=n,this._disposables=new K,this._sessionDisposables=new K,this._disposables.add(t.onTypeFormattingEditProvider.onDidChange(this._update,this)),this._disposables.add(e.onDidChangeModel(()=>this._update())),this._disposables.add(e.onDidChangeModelLanguage(()=>this._update())),this._disposables.add(e.onDidChangeConfiguration(s=>{s.hasChanged(56)&&this._update()})),this._update()}dispose(){this._disposables.dispose(),this._sessionDisposables.dispose()}_update(){if(this._sessionDisposables.clear(),!this._editor.getOption(56)||!this._editor.hasModel())return;const e=this._editor.getModel(),[t]=this._languageFeaturesService.onTypeFormattingEditProvider.ordered(e);if(!t||!t.autoFormatTriggerCharacters)return;const i=new Qy;for(const n of t.autoFormatTriggerCharacters)i.add(n.charCodeAt(0));this._sessionDisposables.add(this._editor.onDidType(n=>{const s=n.charCodeAt(n.length-1);i.has(s)&&this._trigger(String.fromCharCode(s))}))}_trigger(e){if(!this._editor.hasModel()||this._editor.getSelections().length>1||!this._editor.getSelection().isEmpty())return;const t=this._editor.getModel(),i=this._editor.getPosition(),n=new li,s=this._editor.onDidChangeModelContent(r=>{if(r.isFlush){n.cancel(),s.dispose();return}for(let a=0,l=r.changes.length;a<l;a++)if(r.changes[a].range.endLineNumber<=i.lineNumber){n.cancel(),s.dispose();return}});eG(this._workerService,this._languageFeaturesService,t,i,e,t.getFormattingOptions(),n.token).then(r=>{n.token.isCancellationRequested||Qn(r)&&(this._accessibilitySignalService.playSignal(No.format,{userGesture:!1}),Z_.execute(this._editor,r,!0))}).finally(()=>{s.dispose()})}},p_.ID="editor.contrib.autoFormat",p_);qx=cY([Sv(1,ge),Sv(2,Us),Sv(3,ru)],qx);var __;let Gx=(__=class{constructor(e,t,i){this.editor=e,this._languageFeaturesService=t,this._instantiationService=i,this._callOnDispose=new K,this._callOnModel=new K,this._callOnDispose.add(e.onDidChangeConfiguration(()=>this._update())),this._callOnDispose.add(e.onDidChangeModel(()=>this._update())),this._callOnDispose.add(e.onDidChangeModelLanguage(()=>this._update())),this._callOnDispose.add(t.documentRangeFormattingEditProvider.onDidChange(this._update,this))}dispose(){this._callOnDispose.dispose(),this._callOnModel.dispose()}_update(){this._callOnModel.clear(),this.editor.getOption(55)&&this.editor.hasModel()&&this._languageFeaturesService.documentRangeFormattingEditProvider.has(this.editor.getModel())&&this._callOnModel.add(this.editor.onDidPaste(({range:e})=>this._trigger(e)))}_trigger(e){this.editor.hasModel()&&(this.editor.getSelections().length>1||this._instantiationService.invokeFunction(Jq,this.editor,e,2,ka.None,ze.None,!1).catch(Oe))}},__.ID="editor.contrib.formatOnPaste",__);Gx=cY([Sv(1,ge),Sv(2,we)],Gx);class ske extends de{constructor(){super({id:"editor.action.formatDocument",label:m("formatDocument.label","Format Document"),alias:"Format Document",precondition:G.and(I.notInCompositeEditor,I.writable,I.hasDocumentFormattingProvider),kbOpts:{kbExpr:I.editorTextFocus,primary:1572,linux:{primary:3111},weight:100},contextMenuOpts:{group:"1_modification",order:1.3}})}async run(e,t){if(t.hasModel()){const i=e.get(we);await e.get(nu).showWhile(i.invokeFunction(x1e,t,1,ka.None,ze.None,!0),250)}}}class oke extends de{constructor(){super({id:"editor.action.formatSelection",label:m("formatSelection.label","Format Selection"),alias:"Format Selection",precondition:G.and(I.writable,I.hasDocumentSelectionFormattingProvider),kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2084),weight:100},contextMenuOpts:{when:I.hasNonEmptySelection,group:"1_modification",order:1.31}})}async run(e,t){if(!t.hasModel())return;const i=e.get(we),n=t.getModel(),s=t.getSelections().map(a=>a.isEmpty()?new x(a.startLineNumber,1,a.startLineNumber,n.getLineMaxColumn(a.startLineNumber)):a);await e.get(nu).showWhile(i.invokeFunction(Jq,t,s,1,ka.None,ze.None,!0),250)}}ct(qx.ID,qx,2);ct(Gx.ID,Gx,2);Q(ske);Q(oke);rt.registerCommand("editor.action.format",async o=>{const e=o.get(pt).getFocusedCodeEditor();if(!e||!e.hasModel())return;const t=o.get(Ht);e.getSelection().isEmpty()?await t.executeCommand("editor.action.formatDocument"):await t.executeCommand("editor.action.formatSelection")});var rke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},jT=function(o,e){return function(t,i){e(t,i,o)}};class Cp{remove(){var e;(e=this.parent)==null||e.children.delete(this.id)}static findId(e,t){let i;typeof e=="string"?i=`${t.id}/${e}`:(i=`${t.id}/${e.name}`,t.children.get(i)!==void 0&&(i=`${t.id}/${e.name}_${e.range.startLineNumber}_${e.range.startColumn}`));let n=i;for(let s=0;t.children.get(n)!==void 0;s++)n=`${i}_${s}`;return n}static empty(e){return e.children.size===0}}class ZO extends Cp{constructor(e,t,i){super(),this.id=e,this.parent=t,this.symbol=i,this.children=new Map}}class dY extends Cp{constructor(e,t,i,n){super(),this.id=e,this.parent=t,this.label=i,this.order=n,this.children=new Map}}class Lc extends Cp{static create(e,t,i){const n=new li(i),s=new Lc(t.uri),r=e.ordered(t),a=r.map((c,d)=>{const h=Cp.findId(`provider_${d}`,s),u=new dY(h,s,c.displayName??"Unknown Outline Provider",d);return Promise.resolve(c.provideDocumentSymbols(t,n.token)).then(g=>{for(const f of g||[])Lc._makeOutlineElement(f,u);return u},g=>(Si(g),u)).then(g=>{Cp.empty(g)?g.remove():s._groups.set(h,g)})}),l=e.onDidChange(()=>{const c=e.ordered(t);ii(c,r)||n.cancel()});return Promise.all(a).then(()=>n.token.isCancellationRequested&&!i.isCancellationRequested?Lc.create(e,t,i):s._compact()).finally(()=>{n.dispose(),l.dispose(),n.dispose()})}static _makeOutlineElement(e,t){const i=Cp.findId(e,t),n=new ZO(i,t,e);if(e.children)for(const s of e.children)Lc._makeOutlineElement(s,n);t.children.set(n.id,n)}constructor(e){super(),this.uri=e,this.id="root",this.parent=void 0,this._groups=new Map,this.children=new Map,this.id="root",this.parent=void 0}_compact(){let e=0;for(const[t,i]of this._groups)i.children.size===0?this._groups.delete(t):e+=1;if(e!==1)this.children=this._groups;else{const t=ot.first(this._groups.values());for(const[,i]of t.children)i.parent=this,this.children.set(i.id,i)}return this}getTopLevelSymbols(){const e=[];for(const t of this.children.values())t instanceof ZO?e.push(t.symbol):e.push(...ot.map(t.children.values(),i=>i.symbol));return e.sort((t,i)=>x.compareRangesUsingStarts(t.range,i.range))}asListOfDocumentSymbols(){const e=this.getTopLevelSymbols(),t=[];return Lc._flattenDocumentSymbols(t,e,""),t.sort((i,n)=>A.compare(x.getStartPosition(i.range),x.getStartPosition(n.range))||A.compare(x.getEndPosition(n.range),x.getEndPosition(i.range)))}static _flattenDocumentSymbols(e,t,i){for(const n of t)e.push({kind:n.kind,tags:n.tags,name:n.name,detail:n.detail,containerName:n.containerName||i,range:n.range,selectionRange:n.selectionRange,children:void 0}),n.children&&Lc._flattenDocumentSymbols(e,n.children,n.name)}}const pS=et("IOutlineModelService");let YO=class{constructor(e,t,i){this._languageFeaturesService=e,this._disposables=new K,this._cache=new Gl(10,.7),this._debounceInformation=t.for(e.documentSymbolProvider,"DocumentSymbols",{min:350}),this._disposables.add(i.onModelRemoved(n=>{this._cache.delete(n.id)}))}dispose(){this._disposables.dispose()}async getOrCreate(e,t){const i=this._languageFeaturesService.documentSymbolProvider,n=i.ordered(e);let s=this._cache.get(e.id);if(!s||s.versionId!==e.getVersionId()||!ii(s.provider,n)){const a=new li;s={versionId:e.getVersionId(),provider:n,promiseCnt:0,source:a,promise:Lc.create(i,e,a.token),model:void 0},this._cache.set(e.id,s);const l=Date.now();s.promise.then(c=>{s.model=c,this._debounceInformation.update(e,Date.now()-l)}).catch(c=>{this._cache.delete(e.id)})}if(s.model)return s.model;s.promiseCnt+=1;const r=t.onCancellationRequested(()=>{--s.promiseCnt===0&&(s.source.cancel(),this._cache.delete(e.id))});try{return await s.promise}finally{r.dispose()}}};YO=rke([jT(0,ge),jT(1,Zo),jT(2,Kt)],YO);st(pS,YO,1);rt.registerCommand("_executeDocumentSymbolProvider",async function(o,...e){const[t]=e;mt(Se.isUri(t));const i=o.get(pS),s=await o.get(Hs).createModelReference(t);try{return(await i.getOrCreate(s.object.textEditorModel,ze.None)).getTopLevelSymbols()}finally{s.dispose()}});const gl=class gl extends F{constructor(e,t){super(),this.contextKeyService=e,this.model=t,this.inlineCompletionVisible=gl.inlineSuggestionVisible.bindTo(this.contextKeyService),this.inlineCompletionSuggestsIndentation=gl.inlineSuggestionHasIndentation.bindTo(this.contextKeyService),this.inlineCompletionSuggestsIndentationLessThanTabSize=gl.inlineSuggestionHasIndentationLessThanTabSize.bindTo(this.contextKeyService),this.suppressSuggestions=gl.suppressSuggestions.bindTo(this.contextKeyService),this._register(Ae(i=>{const n=this.model.read(i),s=n==null?void 0:n.state.read(i),r=!!(s!=null&&s.inlineCompletion)&&(s==null?void 0:s.primaryGhostText)!==void 0&&!(s!=null&&s.primaryGhostText.isEmpty());this.inlineCompletionVisible.set(r),s!=null&&s.primaryGhostText&&(s!=null&&s.inlineCompletion)&&this.suppressSuggestions.set(s.inlineCompletion.inlineCompletion.source.inlineCompletions.suppressSuggestions)})),this._register(Ae(i=>{const n=this.model.read(i);let s=!1,r=!0;const a=n==null?void 0:n.primaryGhostText.read(i);if(n!=null&&n.selectedSuggestItem&&a&&a.parts.length>0){const{column:l,lines:c}=a.parts[0],d=c[0],h=n.textModel.getLineIndentColumn(a.lineNumber);if(l<=h){let g=Jn(d);g===-1&&(g=d.length-1),s=g>0;const f=n.textModel.getOptions().tabSize;r=Ui.visibleColumnFromColumn(d,g+1,f)<f}}this.inlineCompletionSuggestsIndentation.set(s),this.inlineCompletionSuggestsIndentationLessThanTabSize.set(r)}))}};gl.inlineSuggestionVisible=new le("inlineSuggestionVisible",!1,m("inlineSuggestionVisible","Whether an inline suggestion is visible")),gl.inlineSuggestionHasIndentation=new le("inlineSuggestionHasIndentation",!1,m("inlineSuggestionHasIndentation","Whether the inline suggestion starts with whitespace")),gl.inlineSuggestionHasIndentationLessThanTabSize=new le("inlineSuggestionHasIndentationLessThanTabSize",!0,m("inlineSuggestionHasIndentationLessThanTabSize","Whether the inline suggestion starts with whitespace that is less than what would be inserted by tab")),gl.suppressSuggestions=new le("inlineSuggestionSuppressSuggestions",void 0,m("suppressSuggestions","Whether suggestions should be suppressed for the current suggestion"));let ho=gl;function ake(o){const e=new K,t=e.add(IU());return e.add(Ae(i=>{t.setStyle(o.read(i))})),e}class g1{constructor(e,t){this.lineNumber=e,this.parts=t}equals(e){return this.lineNumber===e.lineNumber&&this.parts.length===e.parts.length&&this.parts.every((t,i)=>t.equals(e.parts[i]))}renderForScreenReader(e){if(this.parts.length===0)return"";const t=this.parts[this.parts.length-1],i=e.substr(0,t.column-1);return new v5([...this.parts.map(s=>new Fl(x.fromPositions(new A(1,s.column)),s.lines.join(`
`)))]).applyToString(i).substring(this.parts[0].column-1)}isEmpty(){return this.parts.every(e=>e.lines.length===0)}get lineCount(){return 1+this.parts.reduce((e,t)=>e+t.lines.length-1,0)}}class Zx{constructor(e,t,i){this.column=e,this.text=t,this.preview=i,this.lines=Kl(this.text)}equals(e){return this.column===e.column&&this.lines.length===e.lines.length&&this.lines.every((t,i)=>t===e.lines[i])}}class XO{constructor(e,t,i,n=0){this.lineNumber=e,this.columnRange=t,this.text=i,this.additionalReservedLineCount=n,this.parts=[new Zx(this.columnRange.endColumnExclusive,this.text,!1)],this.newLines=Kl(this.text)}renderForScreenReader(e){return this.newLines.join(`
`)}get lineCount(){return this.newLines.length}isEmpty(){return this.parts.every(e=>e.lines.length===0)}equals(e){return this.lineNumber===e.lineNumber&&this.columnRange.equals(e.columnRange)&&this.newLines.length===e.newLines.length&&this.newLines.every((t,i)=>t===e.newLines[i])&&this.additionalReservedLineCount===e.additionalReservedLineCount}}function u6(o,e){return ii(o,e,hY)}function hY(o,e){return o===e?!0:!o||!e?!1:o instanceof g1&&e instanceof g1||o instanceof XO&&e instanceof XO?o.equals(e):!1}const lke=[];function cke(){return lke}class uY{constructor(e,t){if(this.startColumn=e,this.endColumnExclusive=t,e>t)throw new bt(`startColumn ${e} cannot be after endColumnExclusive ${t}`)}toRange(e){return new x(e,this.startColumn,e,this.endColumnExclusive)}equals(e){return this.startColumn===e.startColumn&&this.endColumnExclusive===e.endColumnExclusive}}function gY(o,e){const t=new K,i=o.createDecorationsCollection();return t.add(G1({debugName:()=>`Apply decorations from ${e.debugName}`},n=>{const s=e.read(n);i.set(s)})),t.add({dispose:()=>{i.clear()}}),t}function dke(o,e){return new A(o.lineNumber+e.lineNumber-1,e.lineNumber===1?o.column+e.column-1:e.column)}function g6(o,e){return new A(o.lineNumber-e.lineNumber+1,o.lineNumber-e.lineNumber===0?o.column-e.column+1:o.column)}var hke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},uke=function(o,e){return function(t,i){e(t,i,o)}};const f6="ghost-text";let QO=class extends F{constructor(e,t,i){super(),this.editor=e,this.model=t,this.languageService=i,this.isDisposed=Ke(this,!1),this.currentTextModel=xt(this,this.editor.onDidChangeModel,()=>this.editor.getModel()),this.uiState=be(this,n=>{if(this.isDisposed.read(n))return;const s=this.currentTextModel.read(n);if(s!==this.model.targetTextModel.read(n))return;const r=this.model.ghostText.read(n);if(!r)return;const a=r instanceof XO?r.columnRange:void 0,l=[],c=[];function d(p,_){if(c.length>0){const b=c[c.length-1];_&&b.decorations.push(new hs(b.content.length+1,b.content.length+1+p[0].length,_,0)),b.content+=p[0],p=p.slice(1)}for(const b of p)c.push({content:b,decorations:_?[new hs(1,b.length+1,_,0)]:[]})}const h=s.getLineContent(r.lineNumber);let u,g=0;for(const p of r.parts){let _=p.lines;u===void 0?(l.push({column:p.column,text:_[0],preview:p.preview}),_=_.slice(1)):d([h.substring(g,p.column-1)],void 0),_.length>0&&(d(_,f6),u===void 0&&p.column<=h.length&&(u=p.column)),g=p.column-1}u!==void 0&&d([h.substring(g)],void 0);const f=u!==void 0?new uY(u,h.length+1):void 0;return{replacedRange:a,inlineTexts:l,additionalLines:c,hiddenRange:f,lineNumber:r.lineNumber,additionalReservedLineCount:this.model.minReservedLineCount.read(n),targetTextModel:s}}),this.decorations=be(this,n=>{const s=this.uiState.read(n);if(!s)return[];const r=[];s.replacedRange&&r.push({range:s.replacedRange.toRange(s.lineNumber),options:{inlineClassName:"inline-completion-text-to-replace",description:"GhostTextReplacement"}}),s.hiddenRange&&r.push({range:s.hiddenRange.toRange(s.lineNumber),options:{inlineClassName:"ghost-text-hidden",description:"ghost-text-hidden"}});for(const a of s.inlineTexts)r.push({range:x.fromPositions(new A(s.lineNumber,a.column)),options:{description:f6,after:{content:a.text,inlineClassName:a.preview?"ghost-text-decoration-preview":"ghost-text-decoration",cursorStops:zr.Left},showIfCollapsed:!0}});return r}),this.additionalLinesWidget=this._register(new gke(this.editor,this.languageService.languageIdCodec,be(n=>{const s=this.uiState.read(n);return s?{lineNumber:s.lineNumber,additionalLines:s.additionalLines,minReservedLineCount:s.additionalReservedLineCount,targetTextModel:s.targetTextModel}:void 0}))),this._register(ve(()=>{this.isDisposed.set(!0,void 0)})),this._register(gY(this.editor,this.decorations))}ownsViewZone(e){return this.additionalLinesWidget.viewZoneId===e}};QO=hke([uke(2,ni)],QO);class gke extends F{get viewZoneId(){return this._viewZoneId}constructor(e,t,i){super(),this.editor=e,this.languageIdCodec=t,this.lines=i,this._viewZoneId=void 0,this.editorOptionsChanged=xn("editorOptionChanged",oe.filter(this.editor.onDidChangeConfiguration,n=>n.hasChanged(33)||n.hasChanged(118)||n.hasChanged(100)||n.hasChanged(95)||n.hasChanged(51)||n.hasChanged(50)||n.hasChanged(67))),this._register(Ae(n=>{const s=this.lines.read(n);this.editorOptionsChanged.read(n),s?this.updateLines(s.lineNumber,s.additionalLines,s.minReservedLineCount):this.clear()}))}dispose(){super.dispose(),this.clear()}clear(){this.editor.changeViewZones(e=>{this._viewZoneId&&(e.removeZone(this._viewZoneId),this._viewZoneId=void 0)})}updateLines(e,t,i){const n=this.editor.getModel();if(!n)return;const{tabSize:s}=n.getOptions();this.editor.changeViewZones(r=>{this._viewZoneId&&(r.removeZone(this._viewZoneId),this._viewZoneId=void 0);const a=Math.max(t.length,i);if(a>0){const l=document.createElement("div");fke(l,s,t,this.editor.getOptions(),this.languageIdCodec),this._viewZoneId=r.addZone({afterLineNumber:e,heightInLines:a,domNode:l,afterColumnAffinity:1})}})}}function fke(o,e,t,i,n){const s=i.get(33),r=i.get(118),a="none",l=i.get(95),c=i.get(51),d=i.get(50),h=i.get(67),u=new _b(1e4);u.appendString('<div class="suggest-preview-text">');for(let p=0,_=t.length;p<_;p++){const b=t[p],C=b.content;u.appendString('<div class="view-line'),u.appendString('" style="top:'),u.appendString(String(p*h)),u.appendString('px;width:1000000px;">');const v=P1(C),w=k_(C),S=cn.createEmpty(C,n);nS(new au(d.isMonospace&&!s,d.canUseHalfwidthRightwardsArrow,C,!1,v,w,0,S,b.decorations,e,0,d.spaceWidth,d.middotWidth,d.wsmiddotWidth,r,a,l,c!==Mh.OFF,null),u),u.appendString("</div>")}u.appendString("</div>"),bn(o,d);const g=u.build(),f=m6?m6.createHTML(g):g;o.innerHTML=f}const m6=hd("editorGhostText",{createHTML:o=>o});function mke(o,e){const t=new Yj,i=new Qj(t,c=>e.getLanguageConfiguration(c)),n=new Xj(new pke([o]),i),s=oA(n,[],void 0,!0);let r="";const a=o.getLineContent();function l(c,d){if(c.kind===2)if(l(c.openingBracket,d),d=_i(d,c.openingBracket.length),c.child&&(l(c.child,d),d=_i(d,c.child.length)),c.closingBracket)l(c.closingBracket,d),d=_i(d,c.closingBracket.length);else{const u=i.getSingleLanguageBracketTokens(c.openingBracket.languageId).findClosingTokenText(c.openingBracket.bracketIds);r+=u}else if(c.kind!==3){if(c.kind===0||c.kind===1)r+=a.substring(d,_i(d,c.length));else if(c.kind===4)for(const h of c.children)l(h,d),d=_i(d,h.length)}}return l(s,ds),r}class pke{constructor(e){this.lines=e,this.tokenization={getLineTokens:t=>this.lines[t-1]}}getLineCount(){return this.lines.length}getLineLength(e){return this.lines[e-1].getLineContent().length}}async function fY(o,e,t,i,n=ze.None,s){const r=e instanceof A?Cke(e,t):e,a=o.all(t),l=new m5;for(const b of a)b.groupId&&l.add(b.groupId,b);function c(b){if(!b.yieldsToGroupIds)return[];const C=[];for(const v of b.yieldsToGroupIds||[]){const w=l.get(v);for(const S of w)C.push(S)}return C}const d=new Map,h=new Set;function u(b,C){if(C=[...C,b],h.has(b))return C;h.add(b);try{const v=c(b);for(const w of v){const S=u(w,C);if(S)return S}}finally{h.delete(b)}}function g(b){const C=d.get(b);if(C)return C;const v=u(b,[]);v&&Si(new Error(`Inline completions: cyclic yield-to dependency detected. Path: ${v.map(S=>S.toString?S.toString():""+S).join(" -> ")}`));const w=new gb;return d.set(b,w.p),(async()=>{var S;if(!v){const L=c(b);for(const k of L){const D=await g(k);if(D&&D.items.length>0)return}}try{return e instanceof A?await b.provideInlineCompletions(t,e,i,n):await((S=b.provideInlineEdits)==null?void 0:S.call(b,t,e,i,n))}catch(L){Si(L);return}})().then(S=>w.complete(S),S=>w.error(S)),w.p}const f=await Promise.all(a.map(async b=>({provider:b,completions:await g(b)}))),p=new Map,_=[];for(const b of f){const C=b.completions;if(!C)continue;const v=new bke(C,b.provider);_.push(v);for(const w of C.items){const S=Yx.from(w,v,r,t,s);p.set(S.hash(),S)}}return new _ke(Array.from(p.values()),new Set(p.keys()),_)}class _ke{constructor(e,t,i){this.completions=e,this.hashs=t,this.providerResults=i}has(e){return this.hashs.has(e.hash())}dispose(){for(const e of this.providerResults)e.removeRef()}}class bke{constructor(e,t){this.inlineCompletions=e,this.provider=t,this.refCount=1}addRef(){this.refCount++}removeRef(){this.refCount--,this.refCount===0&&this.provider.freeInlineCompletions(this.inlineCompletions)}}class Yx{static from(e,t,i,n,s){let r,a,l=e.range?x.lift(e.range):i;if(typeof e.insertText=="string"){if(r=e.insertText,s&&e.completeBracketPairs){r=p6(r,l.getStartPosition(),n,s);const c=r.length-e.insertText.length;c!==0&&(l=new x(l.startLineNumber,l.startColumn,l.endLineNumber,l.endColumn+c))}a=void 0}else if("snippet"in e.insertText){const c=e.insertText.snippet.length;if(s&&e.completeBracketPairs){e.insertText.snippet=p6(e.insertText.snippet,l.getStartPosition(),n,s);const h=e.insertText.snippet.length-c;h!==0&&(l=new x(l.startLineNumber,l.startColumn,l.endLineNumber,l.endColumn+h))}const d=new zf().parse(e.insertText.snippet);d.children.length===1&&d.children[0]instanceof ls?(r=d.children[0].value,a=void 0):(r=d.toString(),a={snippet:e.insertText.snippet,range:l})}else NI(e.insertText);return new Yx(r,e.command,l,r,a,e.additionalTextEdits||cke(),e,t)}constructor(e,t,i,n,s,r,a,l){this.filterText=e,this.command=t,this.range=i,this.insertText=n,this.snippetInfo=s,this.additionalTextEdits=r,this.sourceInlineCompletion=a,this.source=l,e=e.replace(/\r\n|\r/g,`
`),n=e.replace(/\r\n|\r/g,`
`)}withRange(e){return new Yx(this.filterText,this.command,e,this.insertText,this.snippetInfo,this.additionalTextEdits,this.sourceInlineCompletion,this.source)}hash(){return JSON.stringify({insertText:this.insertText,range:this.range.toString()})}toSingleTextEdit(){return new Fl(this.range,this.insertText)}}function Cke(o,e){const t=e.getWordAtPosition(o),i=e.getLineMaxColumn(o.lineNumber);return t?new x(o.lineNumber,t.startColumn,o.lineNumber,i):x.fromPositions(o,o.with(void 0,i))}function p6(o,e,t,i){const s=t.getLineContent(e.lineNumber).substring(0,e.column-1)+o,r=t.tokenization.tokenizeLineWithEdit(e,s.length-(e.column-1),o),a=r==null?void 0:r.sliceAndInflate(e.column-1,s.length,0);return a?mke(a,i):o}function xg(o,e,t){const i=t?o.range.intersectRanges(t):o.range;if(!i)return o;const n=e.getValueInRange(i,1),s=Oh(n,o.text),r=Yr.ofText(n.substring(0,s)).addToPosition(o.range.getStartPosition()),a=o.text.substring(s),l=x.fromPositions(r,o.range.getEndPosition());return new Fl(l,a)}function mY(o,e){return o.text.startsWith(e.text)&&vke(o.range,e.range)}function _6(o,e,t,i,n=0){let s=xg(o,e);if(s.range.endLineNumber!==s.range.startLineNumber)return;const r=e.getLineContent(s.range.startLineNumber),a=Nt(r).length;if(s.range.startColumn-1<=a){const f=Nt(s.text).length,p=r.substring(s.range.startColumn-1,a),[_,b]=[s.range.getStartPosition(),s.range.getEndPosition()],C=_.column+p.length<=b.column?_.delta(0,p.length):b,v=x.fromPositions(C,b),w=s.text.startsWith(p)?s.text.substring(p.length):s.text.substring(f);s=new Fl(v,w)}const c=e.getValueInRange(s.range),d=wke(c,s.text);if(!d)return;const h=s.range.startLineNumber,u=new Array;if(t==="prefix"){const f=d.filter(p=>p.originalLength===0);if(f.length>1||f.length===1&&f[0].originalStart!==c.length)return}const g=s.text.length-n;for(const f of d){const p=s.range.startColumn+f.originalStart+f.originalLength;if(t==="subwordSmart"&&i&&i.lineNumber===s.range.startLineNumber&&p<i.column||f.originalLength>0)return;if(f.modifiedLength===0)continue;const _=f.modifiedStart+f.modifiedLength,b=Math.max(f.modifiedStart,Math.min(_,g)),C=s.text.substring(f.modifiedStart,b),v=s.text.substring(b,Math.max(f.modifiedStart,_));C.length>0&&u.push(new Zx(p,C,!1)),v.length>0&&u.push(new Zx(p,v,!0))}return new g1(h,u)}function vke(o,e){return e.getStartPosition().equals(o.getStartPosition())&&e.getEndPosition().isBeforeOrEqual(o.getEndPosition())}let nl;function wke(o,e){if((nl==null?void 0:nl.originalValue)===o&&(nl==null?void 0:nl.newValue)===e)return nl==null?void 0:nl.changes;{let t=C6(o,e,!0);if(t){const i=b6(t);if(i>0){const n=C6(o,e,!1);n&&b6(n)<i&&(t=n)}}return nl={originalValue:o,newValue:e,changes:t},t}}function b6(o){let e=0;for(const t of o)e+=t.originalLength;return e}function C6(o,e,t){if(o.length>5e3||e.length>5e3)return;function i(c){let d=0;for(let h=0,u=c.length;h<u;h++){const g=c.charCodeAt(h);g>d&&(d=g)}return d}const n=Math.max(i(o),i(e));function s(c){if(c<0)throw new Error("unexpected");return n+c+1}function r(c){let d=0,h=0;const u=new Int32Array(c.length);for(let g=0,f=c.length;g<f;g++)if(t&&c[g]==="("){const p=h*100+d;u[g]=s(2*p),d++}else if(t&&c[g]===")"){d=Math.max(d-1,0);const p=h*100+d;u[g]=s(2*p+1),d===0&&h++}else u[g]=c.charCodeAt(g);return u}const a=r(o),l=r(e);return new yl({getElements:()=>a},{getElements:()=>l}).ComputeDiff(!1).changes}var Ske=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},v6=function(o,e){return function(t,i){e(t,i,o)}};let JO=class extends F{constructor(e,t,i,n,s){super(),this.textModel=e,this.versionId=t,this._debounceValue=i,this.languageFeaturesService=n,this.languageConfigurationService=s,this._updateOperation=this._register(new Ji),this.inlineCompletions=Dw("inlineCompletions",void 0),this.suggestWidgetInlineCompletions=Dw("suggestWidgetInlineCompletions",void 0),this._register(this.textModel.onDidChangeContent(()=>{this._updateOperation.clear()}))}fetch(e,t,i){var d,h;const n=new Lke(e,t,this.textModel.getVersionId()),s=t.selectedSuggestionInfo?this.suggestWidgetInlineCompletions:this.inlineCompletions;if((d=this._updateOperation.value)!=null&&d.request.satisfies(n))return this._updateOperation.value.promise;if((h=s.get())!=null&&h.request.satisfies(n))return Promise.resolve(!0);const r=!!this._updateOperation.value;this._updateOperation.clear();const a=new li,l=(async()=>{if((r||t.triggerKind===Ea.Automatic)&&await yke(this._debounceValue.get(this.textModel),a.token),a.token.isCancellationRequested||this._store.isDisposed||this.textModel.getVersionId()!==n.versionId)return!1;const g=new Date,f=await fY(this.languageFeaturesService.inlineCompletionsProvider,e,this.textModel,t,a.token,this.languageConfigurationService);if(a.token.isCancellationRequested||this._store.isDisposed||this.textModel.getVersionId()!==n.versionId)return!1;const p=new Date;this._debounceValue.update(this.textModel,p.getTime()-g.getTime());const _=new kke(f,n,this.textModel,this.versionId);if(i){const b=i.toInlineCompletion(void 0);i.canBeReused(this.textModel,e)&&!f.has(b)&&_.prepend(i.inlineCompletion,b.range,!0)}return this._updateOperation.clear(),ai(b=>{s.set(_,b)}),!0})(),c=new xke(n,a,l);return this._updateOperation.value=c,l}clear(e){this._updateOperation.clear(),this.inlineCompletions.set(void 0,e),this.suggestWidgetInlineCompletions.set(void 0,e)}clearSuggestWidgetInlineCompletions(e){var t;(t=this._updateOperation.value)!=null&&t.request.context.selectedSuggestionInfo&&this._updateOperation.clear(),this.suggestWidgetInlineCompletions.set(void 0,e)}cancelUpdate(){this._updateOperation.clear()}};JO=Ske([v6(3,ge),v6(4,di)],JO);function yke(o,e){return new Promise(t=>{let i;const n=setTimeout(()=>{i&&i.dispose(),t()},o);e&&(i=e.onCancellationRequested(()=>{clearTimeout(n),i&&i.dispose(),t()}))})}class Lke{constructor(e,t,i){this.position=e,this.context=t,this.versionId=i}satisfies(e){return this.position.equals(e.position)&&B2(this.context.selectedSuggestionInfo,e.context.selectedSuggestionInfo,Wce())&&(e.context.triggerKind===Ea.Automatic||this.context.triggerKind===Ea.Explicit)&&this.versionId===e.versionId}}class xke{constructor(e,t,i){this.request=e,this.cancellationTokenSource=t,this.promise=i}dispose(){this.cancellationTokenSource.cancel()}}class kke{get inlineCompletions(){return this._inlineCompletions}constructor(e,t,i,n){this.inlineCompletionProviderResult=e,this.request=t,this._textModel=i,this._versionId=n,this._refCount=1,this._prependedInlineCompletionItems=[];const s=i.deltaDecorations([],e.completions.map(r=>({range:r.range,options:{description:"inline-completion-tracking-range"}})));this._inlineCompletions=e.completions.map((r,a)=>new w6(r,s[a],this._textModel,this._versionId))}clone(){return this._refCount++,this}dispose(){if(this._refCount--,this._refCount===0){setTimeout(()=>{this._textModel.isDisposed()||this._textModel.deltaDecorations(this._inlineCompletions.map(e=>e.decorationId),[])},0),this.inlineCompletionProviderResult.dispose();for(const e of this._prependedInlineCompletionItems)e.source.removeRef()}}prepend(e,t,i){i&&e.source.addRef();const n=this._textModel.deltaDecorations([],[{range:t,options:{description:"inline-completion-tracking-range"}}])[0];this._inlineCompletions.unshift(new w6(e,n,this._textModel,this._versionId)),this._prependedInlineCompletionItems.push(e)}}class w6{get forwardStable(){return this.inlineCompletion.source.inlineCompletions.enableForwardStability??!1}constructor(e,t,i,n){this.inlineCompletion=e,this.decorationId=t,this._textModel=i,this._modelVersion=n,this.semanticId=JSON.stringify([this.inlineCompletion.filterText,this.inlineCompletion.insertText,this.inlineCompletion.range.getStartPosition().toString()]),this._updatedRange=Mo({owner:this,equalsFn:x.equalsRange},s=>(this._modelVersion.read(s),this._textModel.getDecorationRange(this.decorationId)))}toInlineCompletion(e){return this.inlineCompletion.withRange(this._updatedRange.read(e)??KT)}toSingleTextEdit(e){return new Fl(this._updatedRange.read(e)??KT,this.inlineCompletion.insertText)}isVisible(e,t,i){const n=xg(this._toFilterTextReplacement(i),e),s=this._updatedRange.read(i);if(!s||!this.inlineCompletion.range.getStartPosition().equals(s.getStartPosition())||t.lineNumber!==n.range.startLineNumber)return!1;const r=e.getValueInRange(n.range,1),a=n.text,l=Math.max(0,t.column-n.range.startColumn);let c=a.substring(0,l),d=a.substring(l),h=r.substring(0,l),u=r.substring(l);const g=e.getLineIndentColumn(n.range.startLineNumber);return n.range.startColumn<=g&&(h=h.trimStart(),h.length===0&&(u=u.trimStart()),c=c.trimStart(),c.length===0&&(d=d.trimStart())),c.startsWith(h)&&!!ij(u,d)}canBeReused(e,t){const i=this._updatedRange.read(void 0);return!!i&&i.containsPosition(t)&&this.isVisible(e,t,void 0)&&Yr.ofRange(i).isGreaterThanOrEqualTo(Yr.ofRange(this.inlineCompletion.range))}_toFilterTextReplacement(e){return new Fl(this._updatedRange.read(e)??KT,this.inlineCompletion.filterText)}}const KT=new x(1,1,1,1),Ue={Visible:ZW,HasFocusedSuggestion:new le("suggestWidgetHasFocusedSuggestion",!1,m("suggestWidgetHasSelection","Whether any suggestion is focused")),DetailsVisible:new le("suggestWidgetDetailsVisible",!1,m("suggestWidgetDetailsVisible","Whether suggestion details are visible")),MultipleSuggestions:new le("suggestWidgetMultipleSuggestions",!1,m("suggestWidgetMultipleSuggestions","Whether there are multiple suggestions to pick from")),MakesTextEdit:new le("suggestionMakesTextEdit",!0,m("suggestionMakesTextEdit","Whether inserting the current suggestion yields in a change or has everything already been typed")),AcceptSuggestionsOnEnter:new le("acceptSuggestionOnEnter",!0,m("acceptSuggestionOnEnter","Whether suggestions are inserted when pressing Enter")),HasInsertAndReplaceRange:new le("suggestionHasInsertAndReplaceRange",!1,m("suggestionHasInsertAndReplaceRange","Whether the current suggestion has insert and replace behaviour")),InsertMode:new le("suggestionInsertMode",void 0,{type:"string",description:m("suggestionInsertMode","Whether the default behaviour is to insert or replace")}),CanResolve:new le("suggestionCanResolve",!1,m("suggestionCanResolve","Whether the current suggestion supports to resolve further details"))},fh=new me("suggestWidgetStatusBar");class Dke{constructor(e,t,i,n){var s;this.position=e,this.completion=t,this.container=i,this.provider=n,this.isInvalid=!1,this.score=Ma.Default,this.distance=0,this.textLabel=typeof t.label=="string"?t.label:(s=t.label)==null?void 0:s.label,this.labelLow=this.textLabel.toLowerCase(),this.isInvalid=!this.textLabel,this.sortTextLow=t.sortText&&t.sortText.toLowerCase(),this.filterTextLow=t.filterText&&t.filterText.toLowerCase(),this.extensionId=t.extensionId,x.isIRange(t.range)?(this.editStart=new A(t.range.startLineNumber,t.range.startColumn),this.editInsertEnd=new A(t.range.endLineNumber,t.range.endColumn),this.editReplaceEnd=new A(t.range.endLineNumber,t.range.endColumn),this.isInvalid=this.isInvalid||x.spansMultipleLines(t.range)||t.range.startLineNumber!==e.lineNumber):(this.editStart=new A(t.range.insert.startLineNumber,t.range.insert.startColumn),this.editInsertEnd=new A(t.range.insert.endLineNumber,t.range.insert.endColumn),this.editReplaceEnd=new A(t.range.replace.endLineNumber,t.range.replace.endColumn),this.isInvalid=this.isInvalid||x.spansMultipleLines(t.range.insert)||x.spansMultipleLines(t.range.replace)||t.range.insert.startLineNumber!==e.lineNumber||t.range.replace.startLineNumber!==e.lineNumber||t.range.insert.startColumn!==t.range.replace.startColumn),typeof n.resolveCompletionItem!="function"&&(this._resolveCache=Promise.resolve(),this._resolveDuration=0)}get isResolved(){return this._resolveDuration!==void 0}get resolveDuration(){return this._resolveDuration!==void 0?this._resolveDuration:-1}async resolve(e){if(!this._resolveCache){const t=e.onCancellationRequested(()=>{this._resolveCache=void 0,this._resolveDuration=void 0}),i=new ln(!0);this._resolveCache=Promise.resolve(this.provider.resolveCompletionItem(this.completion,e)).then(n=>{Object.assign(this.completion,n),this._resolveDuration=i.elapsed()},n=>{yr(n)&&(this._resolveCache=void 0,this._resolveDuration=void 0)}).finally(()=>{t.dispose()})}return this._resolveCache}}const ND=class ND{constructor(e=2,t=new Set,i=new Set,n=new Map,s=!0){this.snippetSortOrder=e,this.kindFilter=t,this.providerFilter=i,this.providerItemsToReuse=n,this.showDeprecated=s}};ND.default=new ND;let f1=ND;class Ike{constructor(e,t,i,n){this.items=e,this.needsClipboard=t,this.durations=i,this.disposable=n}}async function iB(o,e,t,i=f1.default,n={triggerKind:0},s=ze.None){const r=new ln;t=t.clone();const a=e.getWordAtPosition(t),l=a?new x(t.lineNumber,a.startColumn,t.lineNumber,a.endColumn):x.fromPositions(t),c={replace:l,insert:l.setEndPosition(t.lineNumber,t.column)},d=[],h=new K,u=[];let g=!1;const f=(_,b,C)=>{var w;let v=!1;if(!b)return v;for(const S of b.suggestions)if(!i.kindFilter.has(S.kind)){if(!i.showDeprecated&&((w=S==null?void 0:S.tags)!=null&&w.includes(1)))continue;S.range||(S.range=c),S.sortText||(S.sortText=typeof S.label=="string"?S.label:S.label.label),!g&&S.insertTextRules&&S.insertTextRules&4&&(g=zf.guessNeedsClipboard(S.insertText)),d.push(new Dke(t,S,b,_)),v=!0}return uI(b)&&h.add(b),u.push({providerName:_._debugDisplayName??"unknown_provider",elapsedProvider:b.duration??-1,elapsedOverall:C.elapsed()}),v},p=(async()=>{})();for(const _ of o.orderedGroups(e)){let b=!1;if(await Promise.all(_.map(async C=>{if(i.providerItemsToReuse.has(C)){const v=i.providerItemsToReuse.get(C);v.forEach(w=>d.push(w)),b=b||v.length>0;return}if(!(i.providerFilter.size>0&&!i.providerFilter.has(C)))try{const v=new ln,w=await C.provideCompletionItems(e,t,n,s);b=f(C,w,v)||b}catch(v){Si(v)}})),b||s.isCancellationRequested)break}return await p,s.isCancellationRequested?(h.dispose(),Promise.reject(new qr)):new Ike(d.sort(Tke(i.snippetSortOrder)),g,{entries:u,elapsed:r.elapsed()},h)}function nB(o,e){if(o.sortTextLow&&e.sortTextLow){if(o.sortTextLow<e.sortTextLow)return-1;if(o.sortTextLow>e.sortTextLow)return 1}return o.textLabel<e.textLabel?-1:o.textLabel>e.textLabel?1:o.completion.kind-e.completion.kind}function Eke(o,e){if(o.completion.kind!==e.completion.kind){if(o.completion.kind===27)return-1;if(e.completion.kind===27)return 1}return nB(o,e)}function Nke(o,e){if(o.completion.kind!==e.completion.kind){if(o.completion.kind===27)return 1;if(e.completion.kind===27)return-1}return nB(o,e)}const BE=new Map;BE.set(0,Eke);BE.set(2,Nke);BE.set(1,nB);function Tke(o){return BE.get(o)}rt.registerCommand("_executeCompletionItemProvider",async(o,...e)=>{const[t,i,n,s]=e;mt(Se.isUri(t)),mt(A.isIPosition(i)),mt(typeof n=="string"||!n),mt(typeof s=="number"||!s);const{completionProvider:r}=o.get(ge),a=await o.get(Hs).createModelReference(t);try{const l={incomplete:!1,suggestions:[]},c=[],d=a.object.textEditorModel.validatePosition(i),h=await iB(r,a.object.textEditorModel,d,void 0,{triggerCharacter:n??void 0,triggerKind:n?1:0});for(const u of h.items)c.length<(s??0)&&c.push(u.resolve(ze.None)),l.incomplete=l.incomplete||u.container.incomplete,l.suggestions.push(u.completion);try{return await Promise.all(c),l}finally{setTimeout(()=>h.disposable.dispose(),100)}}finally{a.dispose()}});function Rke(o,e){var t;(t=o.getContribution("editor.contrib.suggestController"))==null||t.triggerSuggest(new Set().add(e),void 0,!0)}class vp{static isAllOff(e){return e.other==="off"&&e.comments==="off"&&e.strings==="off"}static isAllOn(e){return e.other==="on"&&e.comments==="on"&&e.strings==="on"}static valueFor(e,t){switch(t){case 1:return e.comments;case 2:return e.strings;default:return e.other}}}function S6(o,e=En){return coe(o,e)?o.charAt(0).toUpperCase()+o.slice(1):o}var Mke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Ake=function(o,e){return function(t,i){e(t,i,o)}};class y6{constructor(e){this._delegates=e}resolve(e){for(const t of this._delegates){const i=t.resolve(e);if(i!==void 0)return i}}}class L6{constructor(e,t,i,n){this._model=e,this._selection=t,this._selectionIdx=i,this._overtypingCapturer=n}resolve(e){const{name:t}=e;if(t==="SELECTION"||t==="TM_SELECTED_TEXT"){let i=this._model.getValueInRange(this._selection)||void 0,n=this._selection.startLineNumber!==this._selection.endLineNumber;if(!i&&this._overtypingCapturer){const s=this._overtypingCapturer.getLastOvertypedInfo(this._selectionIdx);s&&(i=s.value,n=s.multiline)}if(i&&n&&e.snippet){const s=this._model.getLineContent(this._selection.startLineNumber),r=Nt(s,0,this._selection.startColumn-1);let a=r;e.snippet.walk(c=>c===e?!1:(c instanceof ls&&(a=Nt(Kl(c.value).pop())),!0));const l=Oh(a,r);i=i.replace(/(\r\n|\r|\n)(.*)/g,(c,d,h)=>`${d}${a.substr(l)}${h}`)}return i}else{if(t==="TM_CURRENT_LINE")return this._model.getLineContent(this._selection.positionLineNumber);if(t==="TM_CURRENT_WORD"){const i=this._model.getWordAtPosition({lineNumber:this._selection.positionLineNumber,column:this._selection.positionColumn});return i&&i.word||void 0}else{if(t==="TM_LINE_INDEX")return String(this._selection.positionLineNumber-1);if(t==="TM_LINE_NUMBER")return String(this._selection.positionLineNumber);if(t==="CURSOR_INDEX")return String(this._selectionIdx);if(t==="CURSOR_NUMBER")return String(this._selectionIdx+1)}}}}class x6{constructor(e,t){this._labelService=e,this._model=t}resolve(e){const{name:t}=e;if(t==="TM_FILENAME")return lh(this._model.uri.fsPath);if(t==="TM_FILENAME_BASE"){const i=lh(this._model.uri.fsPath),n=i.lastIndexOf(".");return n<=0?i:i.slice(0,n)}else{if(t==="TM_DIRECTORY")return $z(this._model.uri.fsPath)==="."?"":this._labelService.getUriLabel(GI(this._model.uri));if(t==="TM_FILEPATH")return this._labelService.getUriLabel(this._model.uri);if(t==="RELATIVE_FILEPATH")return this._labelService.getUriLabel(this._model.uri,{relative:!0,noPrefix:!0})}}}class k6{constructor(e,t,i,n){this._readClipboardText=e,this._selectionIdx=t,this._selectionCount=i,this._spread=n}resolve(e){if(e.name!=="CLIPBOARD")return;const t=this._readClipboardText();if(t){if(this._spread){const i=t.split(/\r\n|\n|\r/).filter(n=>!Jz(n));if(i.length===this._selectionCount)return i[this._selectionIdx]}return t}}}let Xx=class{constructor(e,t,i){this._model=e,this._selection=t,this._languageConfigurationService=i}resolve(e){const{name:t}=e,i=this._model.getLanguageIdAtPosition(this._selection.selectionStartLineNumber,this._selection.selectionStartColumn),n=this._languageConfigurationService.getLanguageConfiguration(i).comments;if(n){if(t==="LINE_COMMENT")return n.lineCommentToken||void 0;if(t==="BLOCK_COMMENT_START")return n.blockCommentStartToken||void 0;if(t==="BLOCK_COMMENT_END")return n.blockCommentEndToken||void 0}}};Xx=Mke([Ake(2,di)],Xx);const fl=class fl{constructor(){this._date=new Date}resolve(e){const{name:t}=e;if(t==="CURRENT_YEAR")return String(this._date.getFullYear());if(t==="CURRENT_YEAR_SHORT")return String(this._date.getFullYear()).slice(-2);if(t==="CURRENT_MONTH")return String(this._date.getMonth().valueOf()+1).padStart(2,"0");if(t==="CURRENT_DATE")return String(this._date.getDate().valueOf()).padStart(2,"0");if(t==="CURRENT_HOUR")return String(this._date.getHours().valueOf()).padStart(2,"0");if(t==="CURRENT_MINUTE")return String(this._date.getMinutes().valueOf()).padStart(2,"0");if(t==="CURRENT_SECOND")return String(this._date.getSeconds().valueOf()).padStart(2,"0");if(t==="CURRENT_DAY_NAME")return fl.dayNames[this._date.getDay()];if(t==="CURRENT_DAY_NAME_SHORT")return fl.dayNamesShort[this._date.getDay()];if(t==="CURRENT_MONTH_NAME")return fl.monthNames[this._date.getMonth()];if(t==="CURRENT_MONTH_NAME_SHORT")return fl.monthNamesShort[this._date.getMonth()];if(t==="CURRENT_SECONDS_UNIX")return String(Math.floor(this._date.getTime()/1e3));if(t==="CURRENT_TIMEZONE_OFFSET"){const i=this._date.getTimezoneOffset(),n=i>0?"-":"+",s=Math.trunc(Math.abs(i/60)),r=s<10?"0"+s:s,a=Math.abs(i)-s*60,l=a<10?"0"+a:a;return n+r+":"+l}}};fl.dayNames=[m("Sunday","Sunday"),m("Monday","Monday"),m("Tuesday","Tuesday"),m("Wednesday","Wednesday"),m("Thursday","Thursday"),m("Friday","Friday"),m("Saturday","Saturday")],fl.dayNamesShort=[m("SundayShort","Sun"),m("MondayShort","Mon"),m("TuesdayShort","Tue"),m("WednesdayShort","Wed"),m("ThursdayShort","Thu"),m("FridayShort","Fri"),m("SaturdayShort","Sat")],fl.monthNames=[m("January","January"),m("February","February"),m("March","March"),m("April","April"),m("May","May"),m("June","June"),m("July","July"),m("August","August"),m("September","September"),m("October","October"),m("November","November"),m("December","December")],fl.monthNamesShort=[m("JanuaryShort","Jan"),m("FebruaryShort","Feb"),m("MarchShort","Mar"),m("AprilShort","Apr"),m("MayShort","May"),m("JuneShort","Jun"),m("JulyShort","Jul"),m("AugustShort","Aug"),m("SeptemberShort","Sep"),m("OctoberShort","Oct"),m("NovemberShort","Nov"),m("DecemberShort","Dec")];let Qx=fl;class D6{constructor(e){this._workspaceService=e}resolve(e){if(!this._workspaceService)return;const t=Dce(this._workspaceService.getWorkspace());if(!xce(t)){if(e.name==="WORKSPACE_NAME")return this._resolveWorkspaceName(t);if(e.name==="WORKSPACE_FOLDER")return this._resoveWorkspacePath(t)}}_resolveWorkspaceName(e){if(T2(e))return lh(e.uri.path);let t=lh(e.configPath.path);return t.endsWith(R2)&&(t=t.substr(0,t.length-R2.length-1)),t}_resoveWorkspacePath(e){if(T2(e))return S6(e.uri.fsPath);const t=lh(e.configPath.path);let i=e.configPath.fsPath;return i.endsWith(t)&&(i=i.substr(0,i.length-t.length-1)),i?S6(i):"/"}}class I6{resolve(e){const{name:t}=e;if(t==="RANDOM")return Math.random().toString().slice(-6);if(t==="RANDOM_HEX")return Math.random().toString(16).slice(-6);if(t==="UUID")return EE()}}var Pke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Oke=function(o,e){return function(t,i){e(t,i,o)}},ca;const Pr=class Pr{constructor(e,t,i){this._editor=e,this._snippet=t,this._snippetLineLeadingWhitespace=i,this._offset=-1,this._nestingLevel=1,this._placeholderGroups=FB(t.placeholders,ar.compareByIndex),this._placeholderGroupsIdx=-1}initialize(e){this._offset=e.newPosition}dispose(){this._placeholderDecorations&&this._editor.removeDecorations([...this._placeholderDecorations.values()]),this._placeholderGroups.length=0}_initDecorations(){if(this._offset===-1)throw new Error("Snippet not initialized!");if(this._placeholderDecorations)return;this._placeholderDecorations=new Map;const e=this._editor.getModel();this._editor.changeDecorations(t=>{for(const i of this._snippet.placeholders){const n=this._snippet.offset(i),s=this._snippet.fullLen(i),r=x.fromPositions(e.getPositionAt(this._offset+n),e.getPositionAt(this._offset+n+s)),a=i.isFinalTabstop?Pr._decor.inactiveFinal:Pr._decor.inactive,l=t.addDecoration(r,a);this._placeholderDecorations.set(i,l)}})}move(e){if(!this._editor.hasModel())return[];if(this._initDecorations(),this._placeholderGroupsIdx>=0){const n=[];for(const s of this._placeholderGroups[this._placeholderGroupsIdx])if(s.transform){const r=this._placeholderDecorations.get(s),a=this._editor.getModel().getDecorationRange(r),l=this._editor.getModel().getValueInRange(a),c=s.transform.resolve(l).split(/\r\n|\r|\n/);for(let d=1;d<c.length;d++)c[d]=this._editor.getModel().normalizeIndentation(this._snippetLineLeadingWhitespace+c[d]);n.push(ri.replace(a,c.join(this._editor.getModel().getEOL())))}n.length>0&&this._editor.executeEdits("snippet.placeholderTransform",n)}let t=!1;e===!0&&this._placeholderGroupsIdx<this._placeholderGroups.length-1?(this._placeholderGroupsIdx+=1,t=!0):e===!1&&this._placeholderGroupsIdx>0&&(this._placeholderGroupsIdx-=1,t=!0);const i=this._editor.getModel().changeDecorations(n=>{const s=new Set,r=[];for(const a of this._placeholderGroups[this._placeholderGroupsIdx]){const l=this._placeholderDecorations.get(a),c=this._editor.getModel().getDecorationRange(l);r.push(new pe(c.startLineNumber,c.startColumn,c.endLineNumber,c.endColumn)),t=t&&this._hasPlaceholderBeenCollapsed(a),n.changeDecorationOptions(l,a.isFinalTabstop?Pr._decor.activeFinal:Pr._decor.active),s.add(a);for(const d of this._snippet.enclosingPlaceholders(a)){const h=this._placeholderDecorations.get(d);n.changeDecorationOptions(h,d.isFinalTabstop?Pr._decor.activeFinal:Pr._decor.active),s.add(d)}}for(const[a,l]of this._placeholderDecorations)s.has(a)||n.changeDecorationOptions(l,a.isFinalTabstop?Pr._decor.inactiveFinal:Pr._decor.inactive);return r});return t?this.move(e):i??[]}_hasPlaceholderBeenCollapsed(e){let t=e;for(;t;){if(t instanceof ar){const i=this._placeholderDecorations.get(t);if(this._editor.getModel().getDecorationRange(i).isEmpty()&&t.toString().length>0)return!0}t=t.parent}return!1}get isAtFirstPlaceholder(){return this._placeholderGroupsIdx<=0||this._placeholderGroups.length===0}get isAtLastPlaceholder(){return this._placeholderGroupsIdx===this._placeholderGroups.length-1}get hasPlaceholder(){return this._snippet.placeholders.length>0}get isTrivialSnippet(){if(this._snippet.placeholders.length===0)return!0;if(this._snippet.placeholders.length===1){const[e]=this._snippet.placeholders;if(e.isFinalTabstop&&this._snippet.rightMostDescendant===e)return!0}return!1}computePossibleSelections(){const e=new Map;for(const t of this._placeholderGroups){let i;for(const n of t){if(n.isFinalTabstop)break;i||(i=[],e.set(n.index,i));const s=this._placeholderDecorations.get(n),r=this._editor.getModel().getDecorationRange(s);if(!r){e.delete(n.index);break}i.push(r)}}return e}get activeChoice(){if(!this._placeholderDecorations)return;const e=this._placeholderGroups[this._placeholderGroupsIdx][0];if(!(e!=null&&e.choice))return;const t=this._placeholderDecorations.get(e);if(!t)return;const i=this._editor.getModel().getDecorationRange(t);if(i)return{range:i,choice:e.choice}}get hasChoice(){let e=!1;return this._snippet.walk(t=>(e=t instanceof Pb,!e)),e}merge(e){const t=this._editor.getModel();this._nestingLevel*=10,this._editor.changeDecorations(i=>{for(const n of this._placeholderGroups[this._placeholderGroupsIdx]){const s=e.shift();console.assert(s._offset!==-1),console.assert(!s._placeholderDecorations);const r=s._snippet.placeholderInfo.last.index;for(const l of s._snippet.placeholderInfo.all)l.isFinalTabstop?l.index=n.index+(r+1)/this._nestingLevel:l.index=n.index+l.index/this._nestingLevel;this._snippet.replace(n,s._snippet.children);const a=this._placeholderDecorations.get(n);i.removeDecoration(a),this._placeholderDecorations.delete(n);for(const l of s._snippet.placeholders){const c=s._snippet.offset(l),d=s._snippet.fullLen(l),h=x.fromPositions(t.getPositionAt(s._offset+c),t.getPositionAt(s._offset+c+d)),u=i.addDecoration(h,Pr._decor.inactive);this._placeholderDecorations.set(l,u)}}this._placeholderGroups=FB(this._snippet.placeholders,ar.compareByIndex)})}};Pr._decor={active:We.register({description:"snippet-placeholder-1",stickiness:0,className:"snippet-placeholder"}),inactive:We.register({description:"snippet-placeholder-2",stickiness:1,className:"snippet-placeholder"}),activeFinal:We.register({description:"snippet-placeholder-3",stickiness:1,className:"finish-snippet-placeholder"}),inactiveFinal:We.register({description:"snippet-placeholder-4",stickiness:1,className:"finish-snippet-placeholder"})};let Jx=Pr;const E6={overwriteBefore:0,overwriteAfter:0,adjustWhitespace:!0,clipboardText:void 0,overtypingCapturer:void 0};let ek=ca=class{static adjustWhitespace(e,t,i,n,s){const r=e.getLineContent(t.lineNumber),a=Nt(r,0,t.column-1);let l;return n.walk(c=>{if(!(c instanceof ls)||c.parent instanceof Pb||s&&!s.has(c))return!0;const d=c.value.split(/\r\n|\r|\n/);if(i){const u=n.offset(c);if(u===0)d[0]=e.normalizeIndentation(d[0]);else{l=l??n.toString();const g=l.charCodeAt(u-1);(g===10||g===13)&&(d[0]=e.normalizeIndentation(a+d[0]))}for(let g=1;g<d.length;g++)d[g]=e.normalizeIndentation(a+d[g])}const h=d.join(e.getEOL());return h!==c.value&&(c.parent.replace(c,[new ls(h)]),l=void 0),!0}),a}static adjustSelection(e,t,i,n){if(i!==0||n!==0){const{positionLineNumber:s,positionColumn:r}=t,a=r-i,l=r+n,c=e.validateRange({startLineNumber:s,startColumn:a,endLineNumber:s,endColumn:l});t=pe.createWithDirection(c.startLineNumber,c.startColumn,c.endLineNumber,c.endColumn,t.getDirection())}return t}static createEditsAndSnippetsFromSelections(e,t,i,n,s,r,a,l,c){const d=[],h=[];if(!e.hasModel())return{edits:d,snippets:h};const u=e.getModel(),g=e.invokeWithinContext(w=>w.get(Af)),f=e.invokeWithinContext(w=>new x6(w.get(W_),u)),p=()=>a,_=u.getValueInRange(ca.adjustSelection(u,e.getSelection(),i,0)),b=u.getValueInRange(ca.adjustSelection(u,e.getSelection(),0,n)),C=u.getLineFirstNonWhitespaceColumn(e.getSelection().positionLineNumber),v=e.getSelections().map((w,S)=>({selection:w,idx:S})).sort((w,S)=>x.compareRangesUsingStarts(w.selection,S.selection));for(const{selection:w,idx:S}of v){let L=ca.adjustSelection(u,w,i,0),k=ca.adjustSelection(u,w,0,n);_!==u.getValueInRange(L)&&(L=w),b!==u.getValueInRange(k)&&(k=w);const D=w.setStartPosition(L.startLineNumber,L.startColumn).setEndPosition(k.endLineNumber,k.endColumn),N=new zf().parse(t,!0,s),B=D.getStartPosition(),W=ca.adjustWhitespace(u,B,r||S>0&&C!==u.getLineFirstNonWhitespaceColumn(w.positionLineNumber),N);N.resolveVariables(new y6([f,new k6(p,S,v.length,e.getOption(79)==="spread"),new L6(u,w,S,l),new Xx(u,w,c),new Qx,new D6(g),new I6])),d[S]=ri.replace(D,N.toString()),d[S].identifier={major:S,minor:0},d[S]._isTracked=!0,h[S]=new Jx(e,N,W)}return{edits:d,snippets:h}}static createEditsAndSnippetsFromEdits(e,t,i,n,s,r,a){if(!e.hasModel()||t.length===0)return{edits:[],snippets:[]};const l=[],c=e.getModel(),d=new zf,h=new cS,u=new y6([e.invokeWithinContext(f=>new x6(f.get(W_),c)),new k6(()=>s,0,e.getSelections().length,e.getOption(79)==="spread"),new L6(c,e.getSelection(),0,r),new Xx(c,e.getSelection(),a),new Qx,new D6(e.invokeWithinContext(f=>f.get(Af))),new I6]);t=t.sort((f,p)=>x.compareRangesUsingStarts(f.range,p.range));let g=0;for(let f=0;f<t.length;f++){const{range:p,template:_}=t[f];if(f>0){const S=t[f-1].range,L=x.fromPositions(S.getEndPosition(),p.getStartPosition()),k=new ls(c.getValueInRange(L));h.appendChild(k),g+=k.value.length}const b=d.parseFragment(_,h);ca.adjustWhitespace(c,p.getStartPosition(),!0,h,new Set(b)),h.resolveVariables(u);const C=h.toString(),v=C.slice(g);g=C.length;const w=ri.replace(p,v);w.identifier={major:f,minor:0},w._isTracked=!0,l.push(w)}return d.ensureFinalTabstop(h,i,!0),{edits:l,snippets:[new Jx(e,h,"")]}}constructor(e,t,i=E6,n){this._editor=e,this._template=t,this._options=i,this._languageConfigurationService=n,this._templateMerges=[],this._snippets=[]}dispose(){Tt(this._snippets)}_logInfo(){return`template="${this._template}", merged_templates="${this._templateMerges.join(" -> ")}"`}insert(){if(!this._editor.hasModel())return;const{edits:e,snippets:t}=typeof this._template=="string"?ca.createEditsAndSnippetsFromSelections(this._editor,this._template,this._options.overwriteBefore,this._options.overwriteAfter,!1,this._options.adjustWhitespace,this._options.clipboardText,this._options.overtypingCapturer,this._languageConfigurationService):ca.createEditsAndSnippetsFromEdits(this._editor,this._template,!1,this._options.adjustWhitespace,this._options.clipboardText,this._options.overtypingCapturer,this._languageConfigurationService);this._snippets=t,this._editor.executeEdits("snippet",e,i=>{const n=i.filter(s=>!!s.identifier);for(let s=0;s<t.length;s++)t[s].initialize(n[s].textChange);return this._snippets[0].hasPlaceholder?this._move(!0):n.map(s=>pe.fromPositions(s.range.getEndPosition()))}),this._editor.revealRange(this._editor.getSelections()[0])}merge(e,t=E6){if(!this._editor.hasModel())return;this._templateMerges.push([this._snippets[0]._nestingLevel,this._snippets[0]._placeholderGroupsIdx,e]);const{edits:i,snippets:n}=ca.createEditsAndSnippetsFromSelections(this._editor,e,t.overwriteBefore,t.overwriteAfter,!0,t.adjustWhitespace,t.clipboardText,t.overtypingCapturer,this._languageConfigurationService);this._editor.executeEdits("snippet",i,s=>{const r=s.filter(l=>!!l.identifier);for(let l=0;l<n.length;l++)n[l].initialize(r[l].textChange);const a=n[0].isTrivialSnippet;if(!a){for(const l of this._snippets)l.merge(n);console.assert(n.length===0)}return this._snippets[0].hasPlaceholder&&!a?this._move(void 0):r.map(l=>pe.fromPositions(l.range.getEndPosition()))})}next(){const e=this._move(!0);this._editor.setSelections(e),this._editor.revealPositionInCenterIfOutsideViewport(e[0].getPosition())}prev(){const e=this._move(!1);this._editor.setSelections(e),this._editor.revealPositionInCenterIfOutsideViewport(e[0].getPosition())}_move(e){const t=[];for(const i of this._snippets){const n=i.move(e);t.push(...n)}return t}get isAtFirstPlaceholder(){return this._snippets[0].isAtFirstPlaceholder}get isAtLastPlaceholder(){return this._snippets[0].isAtLastPlaceholder}get hasPlaceholder(){return this._snippets[0].hasPlaceholder}get hasChoice(){return this._snippets[0].hasChoice}get activeChoice(){return this._snippets[0].activeChoice}isSelectionWithinPlaceholders(){if(!this.hasPlaceholder)return!1;const e=this._editor.getSelections();if(e.length<this._snippets.length)return!1;const t=new Map;for(const i of this._snippets){const n=i.computePossibleSelections();if(t.size===0)for(const[s,r]of n){r.sort(x.compareRangesUsingStarts);for(const a of e)if(r[0].containsRange(a)){t.set(s,[]);break}}if(t.size===0)return!1;t.forEach((s,r)=>{s.push(...n.get(r))})}e.sort(x.compareRangesUsingStarts);for(const[i,n]of t){if(n.length!==e.length){t.delete(i);continue}n.sort(x.compareRangesUsingStarts);for(let s=0;s<n.length;s++)if(!n[s].containsRange(e[s])){t.delete(i);continue}}return t.size>0}};ek=ca=Pke([Oke(3,di)],ek);var Fke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},D0=function(o,e){return function(t,i){e(t,i,o)}},Zm;const N6={overwriteBefore:0,overwriteAfter:0,undoStopBefore:!0,undoStopAfter:!0,adjustWhitespace:!0,clipboardText:void 0,overtypingCapturer:void 0};var Dl;let vs=(Dl=class{static get(e){return e.getContribution(Zm.ID)}constructor(e,t,i,n,s){this._editor=e,this._logService=t,this._languageFeaturesService=i,this._languageConfigurationService=s,this._snippetListener=new K,this._modelVersionId=-1,this._inSnippet=Zm.InSnippetMode.bindTo(n),this._hasNextTabstop=Zm.HasNextTabstop.bindTo(n),this._hasPrevTabstop=Zm.HasPrevTabstop.bindTo(n)}dispose(){var e;this._inSnippet.reset(),this._hasPrevTabstop.reset(),this._hasNextTabstop.reset(),(e=this._session)==null||e.dispose(),this._snippetListener.dispose()}insert(e,t){try{this._doInsert(e,typeof t>"u"?N6:{...N6,...t})}catch(i){this.cancel(),this._logService.error(i),this._logService.error("snippet_error"),this._logService.error("insert_template=",e),this._logService.error("existing_template=",this._session?this._session._logInfo():"<no_session>")}}_doInsert(e,t){var i;if(this._editor.hasModel()){if(this._snippetListener.clear(),t.undoStopBefore&&this._editor.getModel().pushStackElement(),this._session&&typeof e!="string"&&this.cancel(),this._session?(mt(typeof e=="string"),this._session.merge(e,t)):(this._modelVersionId=this._editor.getModel().getAlternativeVersionId(),this._session=new ek(this._editor,e,t,this._languageConfigurationService),this._session.insert()),t.undoStopAfter&&this._editor.getModel().pushStackElement(),(i=this._session)!=null&&i.hasChoice){const n={_debugDisplayName:"snippetChoiceCompletions",provideCompletionItems:(d,h)=>{if(!this._session||d!==this._editor.getModel()||!A.equals(this._editor.getPosition(),h))return;const{activeChoice:u}=this._session;if(!u||u.choice.options.length===0)return;const g=d.getValueInRange(u.range),f=!!u.choice.options.find(_=>_.value===g),p=[];for(let _=0;_<u.choice.options.length;_++){const b=u.choice.options[_];p.push({kind:13,label:b.value,insertText:b.value,sortText:"a".repeat(_+1),range:u.range,filterText:f?`${g}_${b.value}`:void 0,command:{id:"jumpToNextSnippetPlaceholder",title:m("next","Go to next placeholder...")}})}return{suggestions:p}}},s=this._editor.getModel();let r,a=!1;const l=()=>{r==null||r.dispose(),a=!1},c=()=>{a||(r=this._languageFeaturesService.completionProvider.register({language:s.getLanguageId(),pattern:s.uri.fsPath,scheme:s.uri.scheme,exclusive:!0},n),this._snippetListener.add(r),a=!0)};this._choiceCompletions={provider:n,enable:c,disable:l}}this._updateState(),this._snippetListener.add(this._editor.onDidChangeModelContent(n=>n.isFlush&&this.cancel())),this._snippetListener.add(this._editor.onDidChangeModel(()=>this.cancel())),this._snippetListener.add(this._editor.onDidChangeCursorSelection(()=>this._updateState()))}}_updateState(){if(!(!this._session||!this._editor.hasModel())){if(this._modelVersionId===this._editor.getModel().getAlternativeVersionId())return this.cancel();if(!this._session.hasPlaceholder)return this.cancel();if(this._session.isAtLastPlaceholder||!this._session.isSelectionWithinPlaceholders())return this._editor.getModel().pushStackElement(),this.cancel();this._inSnippet.set(!0),this._hasPrevTabstop.set(!this._session.isAtFirstPlaceholder),this._hasNextTabstop.set(!this._session.isAtLastPlaceholder),this._handleChoice()}}_handleChoice(){var t;if(!this._session||!this._editor.hasModel()){this._currentChoice=void 0;return}const{activeChoice:e}=this._session;if(!e||!this._choiceCompletions){(t=this._choiceCompletions)==null||t.disable(),this._currentChoice=void 0;return}this._currentChoice!==e.choice&&(this._currentChoice=e.choice,this._choiceCompletions.enable(),queueMicrotask(()=>{Rke(this._editor,this._choiceCompletions.provider)}))}finish(){for(;this._inSnippet.get();)this.next()}cancel(e=!1){var t;this._inSnippet.reset(),this._hasPrevTabstop.reset(),this._hasNextTabstop.reset(),this._snippetListener.clear(),this._currentChoice=void 0,(t=this._session)==null||t.dispose(),this._session=void 0,this._modelVersionId=-1,e&&this._editor.setSelections([this._editor.getSelection()])}prev(){var e;(e=this._session)==null||e.prev(),this._updateState()}next(){var e;(e=this._session)==null||e.next(),this._updateState()}isInSnippet(){return!!this._inSnippet.get()}},Zm=Dl,Dl.ID="snippetController2",Dl.InSnippetMode=new le("inSnippetMode",!1,m("inSnippetMode","Whether the editor in current in snippet mode")),Dl.HasNextTabstop=new le("hasNextTabstop",!1,m("hasNextTabstop","Whether there is a next tab stop when in snippet mode")),Dl.HasPrevTabstop=new le("hasPrevTabstop",!1,m("hasPrevTabstop","Whether there is a previous tab stop when in snippet mode")),Dl);vs=Zm=Fke([D0(1,Nn),D0(2,ge),D0(3,Ee),D0(4,di)],vs);ct(vs.ID,vs,4);const HE=$i.bindToContribution(vs.get);re(new HE({id:"jumpToNextSnippetPlaceholder",precondition:G.and(vs.InSnippetMode,vs.HasNextTabstop),handler:o=>o.next(),kbOpts:{weight:130,kbExpr:I.textInputFocus,primary:2}}));re(new HE({id:"jumpToPrevSnippetPlaceholder",precondition:G.and(vs.InSnippetMode,vs.HasPrevTabstop),handler:o=>o.prev(),kbOpts:{weight:130,kbExpr:I.textInputFocus,primary:1026}}));re(new HE({id:"leaveSnippet",precondition:vs.InSnippetMode,handler:o=>o.cancel(!0),kbOpts:{weight:130,kbExpr:I.textInputFocus,primary:9,secondary:[1033]}}));re(new HE({id:"acceptSnippet",precondition:vs.InSnippetMode,handler:o=>o.finish()}));var Wke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},qT=function(o,e){return function(t,i){e(t,i,o)}};let eF=class extends F{get isAcceptingPartially(){return this._isAcceptingPartially}constructor(e,t,i,n,s,r,a,l,c,d,h,u){super(),this.textModel=e,this.selectedSuggestItem=t,this._textModelVersionId=i,this._positions=n,this._debounceValue=s,this._suggestPreviewEnabled=r,this._suggestPreviewMode=a,this._inlineSuggestMode=l,this._enabled=c,this._instantiationService=d,this._commandService=h,this._languageConfigurationService=u,this._source=this._register(this._instantiationService.createInstance(JO,this.textModel,this._textModelVersionId,this._debounceValue)),this._isActive=Ke(this,!1),this._forceUpdateExplicitlySignal=Lb(this),this._selectedInlineCompletionId=Ke(this,void 0),this._primaryPosition=be(this,f=>this._positions.read(f)[0]??new A(1,1)),this._isAcceptingPartially=!1,this._preserveCurrentCompletionReasons=new Set([fc.Redo,fc.Undo,fc.AcceptWord]),this._fetchInlineCompletionsPromise=Lj({owner:this,createEmptyChangeSummary:()=>({preserveCurrentCompletion:!1,inlineCompletionTriggerKind:Ea.Automatic}),handleChange:(f,p)=>(f.didChange(this._textModelVersionId)&&this._preserveCurrentCompletionReasons.has(this._getReason(f.change))?p.preserveCurrentCompletion=!0:f.didChange(this._forceUpdateExplicitlySignal)&&(p.inlineCompletionTriggerKind=Ea.Explicit),!0)},(f,p)=>{if(this._forceUpdateExplicitlySignal.read(f),!(this._enabled.read(f)&&this.selectedSuggestItem.read(f)||this._isActive.read(f))){this._source.cancelUpdate();return}this._textModelVersionId.read(f);const b=this._source.suggestWidgetInlineCompletions.get(),C=this.selectedSuggestItem.read(f);if(b&&!C){const k=this._source.inlineCompletions.get();ai(D=>{(!k||b.request.versionId>k.request.versionId)&&this._source.inlineCompletions.set(b.clone(),D),this._source.clearSuggestWidgetInlineCompletions(D)})}const v=this._primaryPosition.read(f),w={triggerKind:p.inlineCompletionTriggerKind,selectedSuggestionInfo:C==null?void 0:C.toSelectedSuggestionInfo()},S=this.selectedInlineCompletion.get(),L=p.preserveCurrentCompletion||S!=null&&S.forwardStable?S:void 0;return this._source.fetch(v,w,L)}),this._filteredInlineCompletionItems=Mo({owner:this,equalsFn:vL()},f=>{const p=this._source.inlineCompletions.read(f);if(!p)return[];const _=this._primaryPosition.read(f);return p.inlineCompletions.filter(C=>C.isVisible(this.textModel,_,f))}),this.selectedInlineCompletionIndex=be(this,f=>{const p=this._selectedInlineCompletionId.read(f),_=this._filteredInlineCompletionItems.read(f),b=this._selectedInlineCompletionId===void 0?-1:_.findIndex(C=>C.semanticId===p);return b===-1?(this._selectedInlineCompletionId.set(void 0,void 0),0):b}),this.selectedInlineCompletion=be(this,f=>{const p=this._filteredInlineCompletionItems.read(f),_=this.selectedInlineCompletionIndex.read(f);return p[_]}),this.activeCommands=Mo({owner:this,equalsFn:vL()},f=>{var p;return((p=this.selectedInlineCompletion.read(f))==null?void 0:p.inlineCompletion.source.inlineCompletions.commands)??[]}),this.lastTriggerKind=this._source.inlineCompletions.map(this,f=>f==null?void 0:f.request.context.triggerKind),this.inlineCompletionsCount=be(this,f=>{if(this.lastTriggerKind.read(f)===Ea.Explicit)return this._filteredInlineCompletionItems.read(f).length}),this.state=Mo({owner:this,equalsFn:(f,p)=>!f||!p?f===p:u6(f.ghostTexts,p.ghostTexts)&&f.inlineCompletion===p.inlineCompletion&&f.suggestItem===p.suggestItem},f=>{const p=this.textModel,_=this.selectedSuggestItem.read(f);if(_){const b=xg(_.toSingleTextEdit(),p),C=this._computeAugmentation(b,f);if(!this._suggestPreviewEnabled.read(f)&&!C)return;const w=(C==null?void 0:C.edit)??b,S=C?C.edit.text.length-b.text.length:0,L=this._suggestPreviewMode.read(f),k=this._positions.read(f),D=[w,...GT(this.textModel,k,w)],N=D.map((W,V)=>_6(W,p,L,k[V],S)).filter(Tl),B=N[0]??new g1(w.range.endLineNumber,[]);return{edits:D,primaryGhostText:B,ghostTexts:N,inlineCompletion:C==null?void 0:C.completion,suggestItem:_}}else{if(!this._isActive.read(f))return;const b=this.selectedInlineCompletion.read(f);if(!b)return;const C=b.toSingleTextEdit(f),v=this._inlineSuggestMode.read(f),w=this._positions.read(f),S=[C,...GT(this.textModel,w,C)],L=S.map((k,D)=>_6(k,p,v,w[D],0)).filter(Tl);return L[0]?{edits:S,primaryGhostText:L[0],ghostTexts:L,inlineCompletion:b,suggestItem:void 0}:void 0}}),this.ghostTexts=Mo({owner:this,equalsFn:u6},f=>{const p=this.state.read(f);if(p)return p.ghostTexts}),this.primaryGhostText=Mo({owner:this,equalsFn:hY},f=>{const p=this.state.read(f);if(p)return p==null?void 0:p.primaryGhostText}),this._register(xb(this._fetchInlineCompletionsPromise));let g;this._register(Ae(f=>{var b,C;const p=this.state.read(f),_=p==null?void 0:p.inlineCompletion;if((_==null?void 0:_.semanticId)!==(g==null?void 0:g.semanticId)&&(g=_,_)){const v=_.inlineCompletion,w=v.source;(C=(b=w.provider).handleItemDidShow)==null||C.call(b,w.inlineCompletions,v.sourceInlineCompletion,v.insertText)}}))}_getReason(e){return e!=null&&e.isUndoing?fc.Undo:e!=null&&e.isRedoing?fc.Redo:this.isAcceptingPartially?fc.AcceptWord:fc.Other}async trigger(e){this._isActive.set(!0,e),await this._fetchInlineCompletionsPromise.get()}async triggerExplicitly(e){Pf(e,t=>{this._isActive.set(!0,t),this._forceUpdateExplicitlySignal.trigger(t)}),await this._fetchInlineCompletionsPromise.get()}stop(e){Pf(e,t=>{this._isActive.set(!1,t),this._source.clear(t)})}_computeAugmentation(e,t){const i=this.textModel,n=this._source.suggestWidgetInlineCompletions.read(t),s=n?n.inlineCompletions:[this.selectedInlineCompletion.read(t)].filter(Tl);return sse(s,a=>{let l=a.toSingleTextEdit(t);return l=xg(l,i,x.fromPositions(l.range.getStartPosition(),e.range.getEndPosition())),mY(l,e)?{completion:a,edit:l}:void 0})}async _deltaSelectedInlineCompletionIndex(e){await this.triggerExplicitly();const t=this._filteredInlineCompletionItems.get()||[];if(t.length>0){const i=(this.selectedInlineCompletionIndex.get()+e+t.length)%t.length;this._selectedInlineCompletionId.set(t[i].semanticId,void 0)}else this._selectedInlineCompletionId.set(void 0,void 0)}async next(){await this._deltaSelectedInlineCompletionIndex(1)}async previous(){await this._deltaSelectedInlineCompletionIndex(-1)}async accept(e){var n;if(e.getModel()!==this.textModel)throw new bt;const t=this.state.get();if(!t||t.primaryGhostText.isEmpty()||!t.inlineCompletion)return;const i=t.inlineCompletion.toInlineCompletion(void 0);if(i.command&&i.source.addRef(),e.pushUndoStop(),i.snippetInfo)e.executeEdits("inlineSuggestion.accept",[ri.replace(i.range,""),...i.additionalTextEdits]),e.setPosition(i.snippetInfo.range.getStartPosition(),"inlineCompletionAccept"),(n=vs.get(e))==null||n.insert(i.snippetInfo.snippet,{undoStopBefore:!1});else{const s=t.edits,r=T6(s).map(a=>pe.fromPositions(a));e.executeEdits("inlineSuggestion.accept",[...s.map(a=>ri.replace(a.range,a.text)),...i.additionalTextEdits]),e.setSelections(r,"inlineCompletionAccept")}this.stop(),i.command&&(await this._commandService.executeCommand(i.command.id,...i.command.arguments||[]).then(void 0,Si),i.source.removeRef())}async acceptNextWord(e){await this._acceptNext(e,(t,i)=>{const n=this.textModel.getLanguageIdAtPosition(t.lineNumber,t.column),s=this._languageConfigurationService.getLanguageConfiguration(n),r=new RegExp(s.wordDefinition.source,s.wordDefinition.flags.replace("g","")),a=i.match(r);let l=0;a&&a.index!==void 0?a.index===0?l=a[0].length:l=a.index:l=i.length;const d=/\s+/g.exec(i);return d&&d.index!==void 0&&d.index+d[0].length<l&&(l=d.index+d[0].length),l},0)}async acceptNextLine(e){await this._acceptNext(e,(t,i)=>{const n=i.match(/\n/);return n&&n.index!==void 0?n.index+1:i.length},1)}async _acceptNext(e,t,i){if(e.getModel()!==this.textModel)throw new bt;const n=this.state.get();if(!n||n.primaryGhostText.isEmpty()||!n.inlineCompletion)return;const s=n.primaryGhostText,r=n.inlineCompletion.toInlineCompletion(void 0);if(r.snippetInfo||r.filterText!==r.insertText){await this.accept(e);return}const a=s.parts[0],l=new A(s.lineNumber,a.column),c=a.text,d=t(l,c);if(d===c.length&&s.parts.length===1){this.accept(e);return}const h=c.substring(0,d),u=this._positions.get(),g=u[0];r.source.addRef();try{this._isAcceptingPartially=!0;try{e.pushUndoStop();const f=x.fromPositions(g,l),p=e.getModel().getValueInRange(f)+h,_=new Fl(f,p),b=[_,...GT(this.textModel,u,_)],C=T6(b).map(v=>pe.fromPositions(v));e.executeEdits("inlineSuggestion.accept",b.map(v=>ri.replace(v.range,v.text))),e.setSelections(C,"inlineCompletionPartialAccept"),e.revealPositionInCenterIfOutsideViewport(e.getPosition(),1)}finally{this._isAcceptingPartially=!1}if(r.source.provider.handlePartialAccept){const f=x.fromPositions(r.range.getStartPosition(),Yr.ofText(h).addToPosition(l)),p=e.getModel().getValueInRange(f,1);r.source.provider.handlePartialAccept(r.source.inlineCompletions,r.sourceInlineCompletion,p.length,{kind:i})}}finally{r.source.removeRef()}}handleSuggestAccepted(e){var s,r;const t=xg(e.toSingleTextEdit(),this.textModel),i=this._computeAugmentation(t,void 0);if(!i)return;const n=i.completion.inlineCompletion;(r=(s=n.source.provider).handlePartialAccept)==null||r.call(s,n.source.inlineCompletions,n.sourceInlineCompletion,t.text.length,{kind:2})}};eF=Wke([qT(9,we),qT(10,Ht),qT(11,di)],eF);var fc;(function(o){o[o.Undo=0]="Undo",o[o.Redo=1]="Redo",o[o.AcceptWord=2]="AcceptWord",o[o.Other=3]="Other"})(fc||(fc={}));function GT(o,e,t){if(e.length===1)return[];const i=e[0],n=e.slice(1),s=t.range.getStartPosition(),r=t.range.getEndPosition(),a=o.getValueInRange(x.fromPositions(i,r)),l=g6(i,s);if(l.lineNumber<1)return Oe(new bt(`positionWithinTextEdit line number should be bigger than 0.
			Invalid subtraction between ${i.toString()} and ${s.toString()}`)),[];const c=Bke(t.text,l);return n.map(d=>{const h=dke(g6(d,s),r),u=o.getValueInRange(x.fromPositions(d,h)),g=Oh(a,u),f=x.fromPositions(d,d.delta(0,g));return new Fl(f,c)})}function Bke(o,e){let t="";const i=Mee(o);for(let n=e.lineNumber-1;n<i.length;n++)t+=i[n].substring(n===e.lineNumber-1?e.column-1:0);return t}function T6(o){const e=ky.createSortPermutation(o,gs(s=>s.range,x.compareRangesUsingStarts)),i=new v5(e.apply(o)).getNewRanges();return e.inverse().apply(i).map(s=>s.getEndPosition())}var Hke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},R6=function(o,e){return function(t,i){e(t,i,o)}},AC;class sB{constructor(e){this.name=e}select(e,t,i){if(i.length===0)return 0;const n=i[0].score[0];for(let s=0;s<i.length;s++){const{score:r,completion:a}=i[s];if(r[0]!==n)break;if(a.preselect)return s}return 0}}class M6 extends sB{constructor(){super("first")}memorize(e,t,i){}toJSON(){}fromJSON(){}}class Vke extends sB{constructor(){super("recentlyUsed"),this._cache=new Gl(300,.66),this._seq=0}memorize(e,t,i){const n=`${e.getLanguageId()}/${i.textLabel}`;this._cache.set(n,{touch:this._seq++,type:i.completion.kind,insertText:i.completion.insertText})}select(e,t,i){if(i.length===0)return 0;const n=e.getLineContent(t.lineNumber).substr(t.column-10,t.column-1);if(/\s$/.test(n))return super.select(e,t,i);const s=i[0].score[0];let r=-1,a=-1,l=-1;for(let c=0;c<i.length&&i[c].score[0]===s;c++){const d=`${e.getLanguageId()}/${i[c].textLabel}`,h=this._cache.peek(d);if(h&&h.touch>l&&h.type===i[c].completion.kind&&h.insertText===i[c].completion.insertText&&(l=h.touch,a=c),i[c].completion.preselect&&r===-1)return r=c}return a!==-1?a:r!==-1?r:0}toJSON(){return this._cache.toJSON()}fromJSON(e){this._cache.clear();const t=0;for(const[i,n]of e)n.touch=t,n.type=typeof n.type=="number"?n.type:tw.fromString(n.type),this._cache.set(i,n);this._seq=this._cache.size}}class zke extends sB{constructor(){super("recentlyUsedByPrefix"),this._trie=Gp.forStrings(),this._seq=0}memorize(e,t,i){const{word:n}=e.getWordUntilPosition(t),s=`${e.getLanguageId()}/${n}`;this._trie.set(s,{type:i.completion.kind,insertText:i.completion.insertText,touch:this._seq++})}select(e,t,i){const{word:n}=e.getWordUntilPosition(t);if(!n)return super.select(e,t,i);const s=`${e.getLanguageId()}/${n}`;let r=this._trie.get(s);if(r||(r=this._trie.findSubstr(s)),r)for(let a=0;a<i.length;a++){const{kind:l,insertText:c}=i[a].completion;if(l===r.type&&c===r.insertText)return a}return super.select(e,t,i)}toJSON(){const e=[];return this._trie.forEach((t,i)=>e.push([i,t])),e.sort((t,i)=>-(t[1].touch-i[1].touch)).forEach((t,i)=>t[1].touch=i),e.slice(0,200)}fromJSON(e){if(this._trie.clear(),e.length>0){this._seq=e[0][1].touch+1;for(const[t,i]of e)i.type=typeof i.type=="number"?i.type:tw.fromString(i.type),this._trie.set(t,i)}}}var Lh;let tF=(Lh=class{constructor(e,t){this._storageService=e,this._configService=t,this._disposables=new K,this._persistSoon=new It(()=>this._saveState(),500),this._disposables.add(e.onWillSaveState(i=>{i.reason===Nw.SHUTDOWN&&this._saveState()}))}dispose(){this._disposables.dispose(),this._persistSoon.dispose()}memorize(e,t,i){this._withStrategy(e,t).memorize(e,t,i),this._persistSoon.schedule()}select(e,t,i){return this._withStrategy(e,t).select(e,t,i)}_withStrategy(e,t){var n;const i=this._configService.getValue("editor.suggestSelection",{overrideIdentifier:e.getLanguageIdAtPosition(t.lineNumber,t.column),resource:e.uri});if(((n=this._strategy)==null?void 0:n.name)!==i){this._saveState();const s=AC._strategyCtors.get(i)||M6;this._strategy=new s;try{const a=this._configService.getValue("editor.suggest.shareSuggestSelections")?0:1,l=this._storageService.get(`${AC._storagePrefix}/${i}`,a);l&&this._strategy.fromJSON(JSON.parse(l))}catch{}}return this._strategy}_saveState(){if(this._strategy){const t=this._configService.getValue("editor.suggest.shareSuggestSelections")?0:1,i=JSON.stringify(this._strategy);this._storageService.store(`${AC._storagePrefix}/${this._strategy.name}`,i,t,1)}}},AC=Lh,Lh._strategyCtors=new Map([["recentlyUsedByPrefix",zke],["recentlyUsed",Vke],["first",M6]]),Lh._storagePrefix="suggest/memories",Lh);tF=AC=Hke([R6(0,ia),R6(1,qe)],tF);const VE=et("ISuggestMemories");st(VE,tF,1);var Uke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},$ke=function(o,e){return function(t,i){e(t,i,o)}},iF,tf;let tk=(tf=class{constructor(e,t){this._editor=e,this._enabled=!1,this._ckAtEnd=iF.AtEnd.bindTo(t),this._configListener=this._editor.onDidChangeConfiguration(i=>i.hasChanged(124)&&this._update()),this._update()}dispose(){var e;this._configListener.dispose(),(e=this._selectionListener)==null||e.dispose(),this._ckAtEnd.reset()}_update(){const e=this._editor.getOption(124)==="on";if(this._enabled!==e)if(this._enabled=e,this._enabled){const t=()=>{if(!this._editor.hasModel()){this._ckAtEnd.set(!1);return}const i=this._editor.getModel(),n=this._editor.getSelection(),s=i.getWordAtPosition(n.getStartPosition());if(!s){this._ckAtEnd.set(!1);return}this._ckAtEnd.set(s.endColumn===n.getStartPosition().column)};this._selectionListener=this._editor.onDidChangeCursorSelection(t),t()}else this._selectionListener&&(this._ckAtEnd.reset(),this._selectionListener.dispose(),this._selectionListener=void 0)}},iF=tf,tf.AtEnd=new le("atEndOfWord",!1),tf);tk=iF=Uke([$ke(1,Ee)],tk);var jke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Kke=function(o,e){return function(t,i){e(t,i,o)}},PC,nf;let ob=(nf=class{constructor(e,t){this._editor=e,this._index=0,this._ckOtherSuggestions=PC.OtherSuggestions.bindTo(t)}dispose(){this.reset()}reset(){var e;this._ckOtherSuggestions.reset(),(e=this._listener)==null||e.dispose(),this._model=void 0,this._acceptNext=void 0,this._ignore=!1}set({model:e,index:t},i){if(e.items.length===0){this.reset();return}if(PC._moveIndex(!0,e,t)===t){this.reset();return}this._acceptNext=i,this._model=e,this._index=t,this._listener=this._editor.onDidChangeCursorPosition(()=>{this._ignore||this.reset()}),this._ckOtherSuggestions.set(!0)}static _moveIndex(e,t,i){let n=i;for(let s=t.items.length;s>0&&(n=(n+t.items.length+(e?1:-1))%t.items.length,!(n===i||!t.items[n].completion.additionalTextEdits));s--);return n}next(){this._move(!0)}prev(){this._move(!1)}_move(e){if(this._model)try{this._ignore=!0,this._index=PC._moveIndex(e,this._model,this._index),this._acceptNext({index:this._index,item:this._model.items[this._index],model:this._model})}finally{this._ignore=!1}}},PC=nf,nf.OtherSuggestions=new le("hasOtherSuggestions",!1),nf);ob=PC=jke([Kke(1,Ee)],ob);class qke{constructor(e,t,i,n){this._disposables=new K,this._disposables.add(i.onDidSuggest(s=>{s.completionModel.items.length===0&&this.reset()})),this._disposables.add(i.onDidCancel(s=>{this.reset()})),this._disposables.add(t.onDidShow(()=>this._onItem(t.getFocusedItem()))),this._disposables.add(t.onDidFocus(this._onItem,this)),this._disposables.add(t.onDidHide(this.reset,this)),this._disposables.add(e.onWillType(s=>{if(this._active&&!t.isFrozen()&&i.state!==0){const r=s.charCodeAt(s.length-1);this._active.acceptCharacters.has(r)&&e.getOption(0)&&n(this._active.item)}}))}_onItem(e){if(!e||!Qn(e.item.completion.commitCharacters)){this.reset();return}if(this._active&&this._active.item.item===e.item)return;const t=new Qy;for(const i of e.item.completion.commitCharacters)i.length>0&&t.add(i.charCodeAt(0));this._active={acceptCharacters:t,item:e}}reset(){this._active=void 0}dispose(){this._disposables.dispose()}}const nr=class nr{async provideSelectionRanges(e,t){const i=[];for(const n of t){const s=[];i.push(s);const r=new Map;await new Promise(a=>nr._bracketsRightYield(a,0,e,n,r)),await new Promise(a=>nr._bracketsLeftYield(a,0,e,n,r,s))}return i}static _bracketsRightYield(e,t,i,n,s){const r=new Map,a=Date.now();for(;;){if(t>=nr._maxRounds){e();break}if(!n){e();break}const l=i.bracketPairs.findNextBracket(n);if(!l){e();break}if(Date.now()-a>nr._maxDuration){setTimeout(()=>nr._bracketsRightYield(e,t+1,i,n,s));break}if(l.bracketInfo.isOpeningBracket){const d=l.bracketInfo.bracketText,h=r.has(d)?r.get(d):0;r.set(d,h+1)}else{const d=l.bracketInfo.getOpeningBrackets()[0].bracketText;let h=r.has(d)?r.get(d):0;if(h-=1,r.set(d,Math.max(0,h)),h<0){let u=s.get(d);u||(u=new cs,s.set(d,u)),u.push(l.range)}}n=l.range.getEndPosition()}}static _bracketsLeftYield(e,t,i,n,s,r){const a=new Map,l=Date.now();for(;;){if(t>=nr._maxRounds&&s.size===0){e();break}if(!n){e();break}const c=i.bracketPairs.findPrevBracket(n);if(!c){e();break}if(Date.now()-l>nr._maxDuration){setTimeout(()=>nr._bracketsLeftYield(e,t+1,i,n,s,r));break}if(c.bracketInfo.isOpeningBracket){const h=c.bracketInfo.bracketText;let u=a.has(h)?a.get(h):0;if(u-=1,a.set(h,Math.max(0,u)),u<0){const g=s.get(h);if(g){const f=g.shift();g.size===0&&s.delete(h);const p=x.fromPositions(c.range.getEndPosition(),f.getStartPosition()),_=x.fromPositions(c.range.getStartPosition(),f.getEndPosition());r.push({range:p}),r.push({range:_}),nr._addBracketLeading(i,_,r)}}}else{const h=c.bracketInfo.getOpeningBrackets()[0].bracketText,u=a.has(h)?a.get(h):0;a.set(h,u+1)}n=c.range.getStartPosition()}}static _addBracketLeading(e,t,i){if(t.startLineNumber===t.endLineNumber)return;const n=t.startLineNumber,s=e.getLineFirstNonWhitespaceColumn(n);s!==0&&s!==t.startColumn&&(i.push({range:x.fromPositions(new A(n,s),t.getEndPosition())}),i.push({range:x.fromPositions(new A(n,1),t.getEndPosition())}));const r=n-1;if(r>0){const a=e.getLineFirstNonWhitespaceColumn(r);a===t.startColumn&&a!==e.getLineLastNonWhitespaceColumn(r)&&(i.push({range:x.fromPositions(new A(r,a),t.getEndPosition())}),i.push({range:x.fromPositions(new A(r,1),t.getEndPosition())}))}}};nr._maxDuration=30,nr._maxRounds=2;let ik=nr;const ml=class ml{static async create(e,t){if(!t.getOption(119).localityBonus||!t.hasModel())return ml.None;const i=t.getModel(),n=t.getPosition();if(!e.canComputeWordRanges(i.uri))return ml.None;const[s]=await new ik().provideSelectionRanges(i,[n]);if(s.length===0)return ml.None;const r=await e.computeWordRanges(i.uri,s[0].range);if(!r)return ml.None;const a=i.getWordUntilPosition(n);return delete r[a.word],new class extends ml{distance(l,c){if(!n.equals(t.getPosition()))return 0;if(c.kind===17)return 2<<20;const d=typeof c.label=="string"?c.label:c.label.label,h=r[d];if(Cz(h))return 2<<20;const u=Zv(h,x.fromPositions(l),x.compareRangesUsingStarts),g=u>=0?h[u]:h[Math.max(0,~u-1)];let f=s.length;for(const p of s){if(!x.containsRange(p.range,g))break;f-=1}return f}}}};ml.None=new class extends ml{distance(){return 0}};let nk=ml,A6=class{constructor(e,t){this.leadingLineContent=e,this.characterCountDelta=t}};class $d{constructor(e,t,i,n,s,r,a=yw.default,l=void 0){this.clipboardText=l,this._snippetCompareFn=$d._compareCompletionItems,this._items=e,this._column=t,this._wordDistance=n,this._options=s,this._refilterKind=1,this._lineContext=i,this._fuzzyScoreOptions=a,r==="top"?this._snippetCompareFn=$d._compareCompletionItemsSnippetsUp:r==="bottom"&&(this._snippetCompareFn=$d._compareCompletionItemsSnippetsDown)}get lineContext(){return this._lineContext}set lineContext(e){(this._lineContext.leadingLineContent!==e.leadingLineContent||this._lineContext.characterCountDelta!==e.characterCountDelta)&&(this._refilterKind=this._lineContext.characterCountDelta<e.characterCountDelta&&this._filteredItems?2:1,this._lineContext=e)}get items(){return this._ensureCachedState(),this._filteredItems}getItemsByProvider(){return this._ensureCachedState(),this._itemsByProvider}getIncompleteProvider(){this._ensureCachedState();const e=new Set;for(const[t,i]of this.getItemsByProvider())i.length>0&&i[0].container.incomplete&&e.add(t);return e}get stats(){return this._ensureCachedState(),this._stats}_ensureCachedState(){this._refilterKind!==0&&this._createCachedState()}_createCachedState(){this._itemsByProvider=new Map;const e=[],{leadingLineContent:t,characterCountDelta:i}=this._lineContext;let n="",s="";const r=this._refilterKind===1?this._items:this._filteredItems,a=[],l=!this._options.filterGraceful||r.length>2e3?Mf:Zae;for(let c=0;c<r.length;c++){const d=r[c];if(d.isInvalid)continue;const h=this._itemsByProvider.get(d.provider);h?h.push(d):this._itemsByProvider.set(d.provider,[d]);const u=d.position.column-d.editStart.column,g=u+i-(d.position.column-this._column);if(n.length!==g&&(n=g===0?"":t.slice(-g),s=n.toLowerCase()),d.word=n,g===0)d.score=Ma.Default;else{let f=0;for(;f<u;){const p=n.charCodeAt(f);if(p===32||p===9)f+=1;else break}if(f>=g)d.score=Ma.Default;else if(typeof d.completion.filterText=="string"){const p=l(n,s,f,d.completion.filterText,d.filterTextLow,0,this._fuzzyScoreOptions);if(!p)continue;CM(d.completion.filterText,d.textLabel)===0?d.score=p:(d.score=jae(n,s,f,d.textLabel,d.labelLow,0),d.score[0]=p[0])}else{const p=l(n,s,f,d.textLabel,d.labelLow,0,this._fuzzyScoreOptions);if(!p)continue;d.score=p}}d.idx=c,d.distance=this._wordDistance.distance(d.position,d.completion),a.push(d),e.push(d.textLabel.length)}this._filteredItems=a.sort(this._snippetCompareFn),this._refilterKind=0,this._stats={pLabelLen:e.length?dR(e.length-.85,e,(c,d)=>c-d):0}}static _compareCompletionItems(e,t){return e.score[0]>t.score[0]?-1:e.score[0]<t.score[0]?1:e.distance<t.distance?-1:e.distance>t.distance?1:e.idx<t.idx?-1:e.idx>t.idx?1:0}static _compareCompletionItemsSnippetsDown(e,t){if(e.completion.kind!==t.completion.kind){if(e.completion.kind===27)return 1;if(t.completion.kind===27)return-1}return $d._compareCompletionItems(e,t)}static _compareCompletionItemsSnippetsUp(e,t){if(e.completion.kind!==t.completion.kind){if(e.completion.kind===27)return-1;if(t.completion.kind===27)return 1}return $d._compareCompletionItems(e,t)}}var Gke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Ld=function(o,e){return function(t,i){e(t,i,o)}},nF;class ku{static shouldAutoTrigger(e){if(!e.hasModel())return!1;const t=e.getModel(),i=e.getPosition();t.tokenization.tokenizeIfCheap(i.lineNumber);const n=t.getWordAtPosition(i);return!(!n||n.endColumn!==i.column&&n.startColumn+1!==i.column||!isNaN(Number(n.word)))}constructor(e,t,i){this.leadingLineContent=e.getLineContent(t.lineNumber).substr(0,t.column-1),this.leadingWord=e.getWordUntilPosition(t),this.lineNumber=t.lineNumber,this.column=t.column,this.triggerOptions=i}}function Zke(o,e,t){if(!e.getContextKeyValue(ho.inlineSuggestionVisible.key))return!0;const i=e.getContextKeyValue(ho.suppressSuggestions.key);return i!==void 0?!i:!o.getOption(62).suppressSuggestions}function Yke(o,e,t){if(!e.getContextKeyValue("inlineSuggestionVisible"))return!0;const i=e.getContextKeyValue(ho.suppressSuggestions.key);return i!==void 0?!i:!o.getOption(62).suppressSuggestions}let sk=nF=class{constructor(e,t,i,n,s,r,a,l,c){this._editor=e,this._editorWorkerService=t,this._clipboardService=i,this._telemetryService=n,this._logService=s,this._contextKeyService=r,this._configurationService=a,this._languageFeaturesService=l,this._envService=c,this._toDispose=new K,this._triggerCharacterListener=new K,this._triggerQuickSuggest=new ea,this._triggerState=void 0,this._completionDisposables=new K,this._onDidCancel=new P,this._onDidTrigger=new P,this._onDidSuggest=new P,this.onDidCancel=this._onDidCancel.event,this.onDidTrigger=this._onDidTrigger.event,this.onDidSuggest=this._onDidSuggest.event,this._telemetryGate=0,this._currentSelection=this._editor.getSelection()||new pe(1,1,1,1),this._toDispose.add(this._editor.onDidChangeModel(()=>{this._updateTriggerCharacters(),this.cancel()})),this._toDispose.add(this._editor.onDidChangeModelLanguage(()=>{this._updateTriggerCharacters(),this.cancel()})),this._toDispose.add(this._editor.onDidChangeConfiguration(()=>{this._updateTriggerCharacters()})),this._toDispose.add(this._languageFeaturesService.completionProvider.onDidChange(()=>{this._updateTriggerCharacters(),this._updateActiveSuggestSession()}));let d=!1;this._toDispose.add(this._editor.onDidCompositionStart(()=>{d=!0})),this._toDispose.add(this._editor.onDidCompositionEnd(()=>{d=!1,this._onCompositionEnd()})),this._toDispose.add(this._editor.onDidChangeCursorSelection(h=>{d||this._onCursorChange(h)})),this._toDispose.add(this._editor.onDidChangeModelContent(()=>{!d&&this._triggerState!==void 0&&this._refilterCompletionItems()})),this._updateTriggerCharacters()}dispose(){Tt(this._triggerCharacterListener),Tt([this._onDidCancel,this._onDidSuggest,this._onDidTrigger,this._triggerQuickSuggest]),this._toDispose.dispose(),this._completionDisposables.dispose(),this.cancel()}_updateTriggerCharacters(){if(this._triggerCharacterListener.clear(),this._editor.getOption(92)||!this._editor.hasModel()||!this._editor.getOption(122))return;const e=new Map;for(const i of this._languageFeaturesService.completionProvider.all(this._editor.getModel()))for(const n of i.triggerCharacters||[]){let s=e.get(n);s||(s=new Set,e.set(n,s)),s.add(i)}const t=i=>{var r;if(!Yke(this._editor,this._contextKeyService,this._configurationService)||ku.shouldAutoTrigger(this._editor))return;if(!i){const a=this._editor.getPosition();i=this._editor.getModel().getLineContent(a.lineNumber).substr(0,a.column-1)}let n="";kf(i.charCodeAt(i.length-1))?Gi(i.charCodeAt(i.length-2))&&(n=i.substr(i.length-2)):n=i.charAt(i.length-1);const s=e.get(n);if(s){const a=new Map;if(this._completionModel)for(const[l,c]of this._completionModel.getItemsByProvider())s.has(l)||a.set(l,c);this.trigger({auto:!0,triggerKind:1,triggerCharacter:n,retrigger:!!this._completionModel,clipboardText:(r=this._completionModel)==null?void 0:r.clipboardText,completionOptions:{providerFilter:s,providerItemsToReuse:a}})}};this._triggerCharacterListener.add(this._editor.onDidType(t)),this._triggerCharacterListener.add(this._editor.onDidCompositionEnd(()=>t()))}get state(){return this._triggerState?this._triggerState.auto?2:1:0}cancel(e=!1){var t;this._triggerState!==void 0&&(this._triggerQuickSuggest.cancel(),(t=this._requestToken)==null||t.cancel(),this._requestToken=void 0,this._triggerState=void 0,this._completionModel=void 0,this._context=void 0,this._onDidCancel.fire({retrigger:e}))}clear(){this._completionDisposables.clear()}_updateActiveSuggestSession(){this._triggerState!==void 0&&(!this._editor.hasModel()||!this._languageFeaturesService.completionProvider.has(this._editor.getModel())?this.cancel():this.trigger({auto:this._triggerState.auto,retrigger:!0}))}_onCursorChange(e){if(!this._editor.hasModel())return;const t=this._currentSelection;if(this._currentSelection=this._editor.getSelection(),!e.selection.isEmpty()||e.reason!==0&&e.reason!==3||e.source!=="keyboard"&&e.source!=="deleteLeft"){this.cancel();return}this._triggerState===void 0&&e.reason===0?(t.containsRange(this._currentSelection)||t.getEndPosition().isBeforeOrEqual(this._currentSelection.getPosition()))&&this._doTriggerQuickSuggest():this._triggerState!==void 0&&e.reason===3&&this._refilterCompletionItems()}_onCompositionEnd(){this._triggerState===void 0?this._doTriggerQuickSuggest():this._refilterCompletionItems()}_doTriggerQuickSuggest(){var e;vp.isAllOff(this._editor.getOption(90))||this._editor.getOption(119).snippetsPreventQuickSuggestions&&((e=vs.get(this._editor))!=null&&e.isInSnippet())||(this.cancel(),this._triggerQuickSuggest.cancelAndSet(()=>{if(this._triggerState!==void 0||!ku.shouldAutoTrigger(this._editor)||!this._editor.hasModel()||!this._editor.hasWidgetFocus())return;const t=this._editor.getModel(),i=this._editor.getPosition(),n=this._editor.getOption(90);if(!vp.isAllOff(n)){if(!vp.isAllOn(n)){t.tokenization.tokenizeIfCheap(i.lineNumber);const s=t.tokenization.getLineTokens(i.lineNumber),r=s.getStandardTokenType(s.findTokenIndexAtOffset(Math.max(i.column-1-1,0)));if(vp.valueFor(n,r)!=="on")return}Zke(this._editor,this._contextKeyService,this._configurationService)&&this._languageFeaturesService.completionProvider.has(t)&&this.trigger({auto:!0})}},this._editor.getOption(91)))}_refilterCompletionItems(){mt(this._editor.hasModel()),mt(this._triggerState!==void 0);const e=this._editor.getModel(),t=this._editor.getPosition(),i=new ku(e,t,{...this._triggerState,refilter:!0});this._onNewContext(i)}trigger(e){var u,g,f;if(!this._editor.hasModel())return;const t=this._editor.getModel(),i=new ku(t,this._editor.getPosition(),e);this.cancel(e.retrigger),this._triggerState=e,this._onDidTrigger.fire({auto:e.auto,shy:e.shy??!1,position:this._editor.getPosition()}),this._context=i;let n={triggerKind:e.triggerKind??0};e.triggerCharacter&&(n={triggerKind:1,triggerCharacter:e.triggerCharacter}),this._requestToken=new li;const s=this._editor.getOption(113);let r=1;switch(s){case"top":r=0;break;case"bottom":r=2;break}const{itemKind:a,showDeprecated:l}=nF.createSuggestFilter(this._editor),c=new f1(r,((u=e.completionOptions)==null?void 0:u.kindFilter)??a,(g=e.completionOptions)==null?void 0:g.providerFilter,(f=e.completionOptions)==null?void 0:f.providerItemsToReuse,l),d=nk.create(this._editorWorkerService,this._editor),h=iB(this._languageFeaturesService.completionProvider,t,this._editor.getPosition(),c,n,this._requestToken.token);Promise.all([h,d]).then(async([p,_])=>{var S;if((S=this._requestToken)==null||S.dispose(),!this._editor.hasModel())return;let b=e==null?void 0:e.clipboardText;if(!b&&p.needsClipboard&&(b=await this._clipboardService.readText()),this._triggerState===void 0)return;const C=this._editor.getModel(),v=new ku(C,this._editor.getPosition(),e),w={...yw.default,firstMatchCanBeWeak:!this._editor.getOption(119).matchOnWordStartOnly};if(this._completionModel=new $d(p.items,this._context.column,{leadingLineContent:v.leadingLineContent,characterCountDelta:v.column-this._context.column},_,this._editor.getOption(119),this._editor.getOption(113),w,b),this._completionDisposables.add(p.disposable),this._onNewContext(v),this._reportDurationsTelemetry(p.durations),!this._envService.isBuilt||this._envService.isExtensionDevelopment)for(const L of p.items)L.isInvalid&&this._logService.warn(`[suggest] did IGNORE invalid completion item from ${L.provider._debugDisplayName}`,L.completion)}).catch(Oe)}_reportDurationsTelemetry(e){this._telemetryGate++%230===0&&setTimeout(()=>{this._telemetryService.publicLog2("suggest.durations.json",{data:JSON.stringify(e)}),this._logService.debug("suggest.durations.json",e)})}static createSuggestFilter(e){const t=new Set;e.getOption(113)==="none"&&t.add(27);const n=e.getOption(119);return n.showMethods||t.add(0),n.showFunctions||t.add(1),n.showConstructors||t.add(2),n.showFields||t.add(3),n.showVariables||t.add(4),n.showClasses||t.add(5),n.showStructs||t.add(6),n.showInterfaces||t.add(7),n.showModules||t.add(8),n.showProperties||t.add(9),n.showEvents||t.add(10),n.showOperators||t.add(11),n.showUnits||t.add(12),n.showValues||t.add(13),n.showConstants||t.add(14),n.showEnums||t.add(15),n.showEnumMembers||t.add(16),n.showKeywords||t.add(17),n.showWords||t.add(18),n.showColors||t.add(19),n.showFiles||t.add(20),n.showReferences||t.add(21),n.showColors||t.add(22),n.showFolders||t.add(23),n.showTypeParameters||t.add(24),n.showSnippets||t.add(27),n.showUsers||t.add(25),n.showIssues||t.add(26),{itemKind:t,showDeprecated:n.showDeprecated}}_onNewContext(e){if(this._context){if(e.lineNumber!==this._context.lineNumber){this.cancel();return}if(Nt(e.leadingLineContent)!==Nt(this._context.leadingLineContent)){this.cancel();return}if(e.column<this._context.column){e.leadingWord.word?this.trigger({auto:this._context.triggerOptions.auto,retrigger:!0}):this.cancel();return}if(this._completionModel){if(e.leadingWord.word.length!==0&&e.leadingWord.startColumn>this._context.leadingWord.startColumn){if(ku.shouldAutoTrigger(this._editor)&&this._context){const i=this._completionModel.getItemsByProvider();this.trigger({auto:this._context.triggerOptions.auto,retrigger:!0,clipboardText:this._completionModel.clipboardText,completionOptions:{providerItemsToReuse:i}})}return}if(e.column>this._context.column&&this._completionModel.getIncompleteProvider().size>0&&e.leadingWord.word.length!==0){const t=new Map,i=new Set;for(const[n,s]of this._completionModel.getItemsByProvider())s.length>0&&s[0].container.incomplete?i.add(n):t.set(n,s);this.trigger({auto:this._context.triggerOptions.auto,triggerKind:2,retrigger:!0,clipboardText:this._completionModel.clipboardText,completionOptions:{providerFilter:i,providerItemsToReuse:t}})}else{const t=this._completionModel.lineContext;let i=!1;if(this._completionModel.lineContext={leadingLineContent:e.leadingLineContent,characterCountDelta:e.column-this._context.column},this._completionModel.items.length===0){const n=ku.shouldAutoTrigger(this._editor);if(!this._context){this.cancel();return}if(n&&this._context.leadingWord.endColumn<e.leadingWord.startColumn){this.trigger({auto:this._context.triggerOptions.auto,retrigger:!0});return}if(this._context.triggerOptions.auto){this.cancel();return}else if(this._completionModel.lineContext=t,i=this._completionModel.items.length>0,i&&e.leadingWord.word.length===0){this.cancel();return}}this._onDidSuggest.fire({completionModel:this._completionModel,triggerOptions:e.triggerOptions,isFrozen:i})}}}}};sk=nF=Gke([Ld(1,Us),Ld(2,md),Ld(3,Bn),Ld(4,Nn),Ld(5,Ee),Ld(6,qe),Ld(7,ge),Ld(8,L5)],sk);const TD=class TD{constructor(e,t){this._disposables=new K,this._lastOvertyped=[],this._locked=!1,this._disposables.add(e.onWillType(()=>{if(this._locked||!e.hasModel())return;const i=e.getSelections(),n=i.length;let s=!1;for(let a=0;a<n;a++)if(!i[a].isEmpty()){s=!0;break}if(!s){this._lastOvertyped.length!==0&&(this._lastOvertyped.length=0);return}this._lastOvertyped=[];const r=e.getModel();for(let a=0;a<n;a++){const l=i[a];if(r.getValueLengthInRange(l)>TD._maxSelectionLength)return;this._lastOvertyped[a]={value:r.getValueInRange(l),multiline:l.startLineNumber!==l.endLineNumber}}})),this._disposables.add(t.onDidTrigger(i=>{this._locked=!0})),this._disposables.add(t.onDidCancel(i=>{this._locked=!1}))}getLastOvertypedInfo(e){if(e>=0&&e<this._lastOvertyped.length)return this._lastOvertyped[e]}dispose(){this._disposables.dispose()}};TD._maxSelectionLength=51200;let sF=TD;var Xke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},ZT=function(o,e){return function(t,i){e(t,i,o)}};let oF=class{constructor(e,t,i,n,s){this._menuId=t,this._menuService=n,this._contextKeyService=s,this._menuDisposables=new K,this.element=X(e,ne(".suggest-status-bar"));const r=a=>a instanceof ao?i.createInstance(i3,a,{useComma:!0}):void 0;this._leftActions=new zo(this.element,{actionViewItemProvider:r}),this._rightActions=new zo(this.element,{actionViewItemProvider:r}),this._leftActions.domNode.classList.add("left"),this._rightActions.domNode.classList.add("right")}dispose(){this._menuDisposables.dispose(),this._leftActions.dispose(),this._rightActions.dispose(),this.element.remove()}show(){const e=this._menuService.createMenu(this._menuId,this._contextKeyService),t=()=>{const i=[],n=[];for(const[s,r]of e.getActions())s==="left"?i.push(...r):n.push(...r);this._leftActions.clear(),this._leftActions.push(i),this._rightActions.clear(),this._rightActions.push(n)};this._menuDisposables.add(e.onDidChange(()=>t())),this._menuDisposables.add(e)}hide(){this._menuDisposables.clear()}};oF=Xke([ZT(2,we),ZT(3,Go),ZT(4,Ee)],oF);var Qke=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Jke=function(o,e){return function(t,i){e(t,i,o)}};function oB(o){return!!o&&!!(o.completion.documentation||o.completion.detail&&o.completion.detail!==o.completion.label)}let rF=class{constructor(e,t){this._editor=e,this._onDidClose=new P,this.onDidClose=this._onDidClose.event,this._onDidChangeContents=new P,this.onDidChangeContents=this._onDidChangeContents.event,this._disposables=new K,this._renderDisposeable=new K,this._borderWidth=1,this._size=new kt(330,0),this.domNode=ne(".suggest-details"),this.domNode.classList.add("no-docs"),this._markdownRenderer=t.createInstance(nd,{editor:e}),this._body=ne(".body"),this._scrollbar=new z1(this._body,{alwaysConsumeMouseWheel:!0}),X(this.domNode,this._scrollbar.getDomNode()),this._disposables.add(this._scrollbar),this._header=X(this._body,ne(".header")),this._close=X(this._header,ne("span"+Ie.asCSSSelector(te.close))),this._close.title=m("details.close","Close"),this._type=X(this._header,ne("p.type")),this._docs=X(this._body,ne("p.docs")),this._configureFont(),this._disposables.add(this._editor.onDidChangeConfiguration(i=>{i.hasChanged(50)&&this._configureFont()}))}dispose(){this._disposables.dispose(),this._renderDisposeable.dispose()}_configureFont(){const e=this._editor.getOptions(),t=e.get(50),i=t.getMassagedFontFamily(),n=e.get(120)||t.fontSize,s=e.get(121)||t.lineHeight,r=t.fontWeight,a=`${n}px`,l=`${s}px`;this.domNode.style.fontSize=a,this.domNode.style.lineHeight=`${s/n}`,this.domNode.style.fontWeight=r,this.domNode.style.fontFeatureSettings=t.fontFeatureSettings,this._type.style.fontFamily=i,this._close.style.height=l,this._close.style.width=l}getLayoutInfo(){const e=this._editor.getOption(121)||this._editor.getOption(50).lineHeight,t=this._borderWidth,i=t*2;return{lineHeight:e,borderWidth:t,borderHeight:i,verticalPadding:22,horizontalPadding:14}}renderLoading(){this._type.textContent=m("loading","Loading..."),this._docs.textContent="",this.domNode.classList.remove("no-docs","no-type"),this.layout(this.size.width,this.getLayoutInfo().lineHeight*2),this._onDidChangeContents.fire(this)}renderItem(e,t){var s;this._renderDisposeable.clear();let{detail:i,documentation:n}=e.completion;if(t){let r="";r+=`score: ${e.score[0]}
`,r+=`prefix: ${e.word??"(no prefix)"}
`,r+=`word: ${e.completion.filterText?e.completion.filterText+" (filterText)":e.textLabel}
`,r+=`distance: ${e.distance} (localityBonus-setting)
`,r+=`index: ${e.idx}, based on ${e.completion.sortText&&`sortText: "${e.completion.sortText}"`||"label"}
`,r+=`commit_chars: ${(s=e.completion.commitCharacters)==null?void 0:s.join("")}
`,n=new Pn().appendCodeblock("empty",r),i=`Provider: ${e.provider._debugDisplayName}`}if(!t&&!oB(e)){this.clearContents();return}if(this.domNode.classList.remove("no-docs","no-type"),i){const r=i.length>1e5?`${i.substr(0,1e5)}…`:i;this._type.textContent=r,this._type.title=r,to(this._type),this._type.classList.toggle("auto-wrap",!/\r?\n^\s+/gmi.test(r))}else pn(this._type),this._type.title="",rs(this._type),this.domNode.classList.add("no-type");if(pn(this._docs),typeof n=="string")this._docs.classList.remove("markdown-docs"),this._docs.textContent=n;else if(n){this._docs.classList.add("markdown-docs"),pn(this._docs);const r=this._markdownRenderer.render(n);this._docs.appendChild(r.element),this._renderDisposeable.add(r),this._renderDisposeable.add(this._markdownRenderer.onDidRenderAsync(()=>{this.layout(this._size.width,this._type.clientHeight+this._docs.clientHeight),this._onDidChangeContents.fire(this)}))}this.domNode.style.userSelect="text",this.domNode.tabIndex=-1,this._close.onmousedown=r=>{r.preventDefault(),r.stopPropagation()},this._close.onclick=r=>{r.preventDefault(),r.stopPropagation(),this._onDidClose.fire()},this._body.scrollTop=0,this.layout(this._size.width,this._type.clientHeight+this._docs.clientHeight),this._onDidChangeContents.fire(this)}clearContents(){this.domNode.classList.add("no-docs"),this._type.textContent="",this._docs.textContent=""}get isEmpty(){return this.domNode.classList.contains("no-docs")}get size(){return this._size}layout(e,t){const i=new kt(e,t);kt.equals(i,this._size)||(this._size=i,Bte(this.domNode,e,t)),this._scrollbar.scanDomNode()}scrollDown(e=8){this._body.scrollTop+=e}scrollUp(e=8){this._body.scrollTop-=e}scrollTop(){this._body.scrollTop=0}scrollBottom(){this._body.scrollTop=this._body.scrollHeight}pageDown(){this.scrollDown(80)}pageUp(){this.scrollUp(80)}set borderWidth(e){this._borderWidth=e}get borderWidth(){return this._borderWidth}};rF=Qke([Jke(1,we)],rF);class eDe{constructor(e,t){this.widget=e,this._editor=t,this.allowEditorOverflow=!0,this._disposables=new K,this._added=!1,this._preferAlignAtTop=!0,this._resizable=new WW,this._resizable.domNode.classList.add("suggest-details-container"),this._resizable.domNode.appendChild(e.domNode),this._resizable.enableSashes(!1,!0,!0,!1);let i,n,s=0,r=0;this._disposables.add(this._resizable.onDidWillResize(()=>{i=this._topLeft,n=this._resizable.size})),this._disposables.add(this._resizable.onDidResize(a=>{if(i&&n){this.widget.layout(a.dimension.width,a.dimension.height);let l=!1;a.west&&(r=n.width-a.dimension.width,l=!0),a.north&&(s=n.height-a.dimension.height,l=!0),l&&this._applyTopLeft({top:i.top+s,left:i.left+r})}a.done&&(i=void 0,n=void 0,s=0,r=0,this._userSize=a.dimension)})),this._disposables.add(this.widget.onDidChangeContents(()=>{this._anchorBox&&this._placeAtAnchor(this._anchorBox,this._userSize??this.widget.size,this._preferAlignAtTop)}))}dispose(){this._resizable.dispose(),this._disposables.dispose(),this.hide()}getId(){return"suggest.details"}getDomNode(){return this._resizable.domNode}getPosition(){return this._topLeft?{preference:this._topLeft}:null}show(){this._added||(this._editor.addOverlayWidget(this),this._added=!0)}hide(e=!1){this._resizable.clearSashHoverState(),this._added&&(this._editor.removeOverlayWidget(this),this._added=!1,this._anchorBox=void 0,this._topLeft=void 0),e&&(this._userSize=void 0,this.widget.clearContents())}placeAtAnchor(e,t){const i=e.getBoundingClientRect();this._anchorBox=i,this._preferAlignAtTop=t,this._placeAtAnchor(this._anchorBox,this._userSize??this.widget.size,t)}_placeAtAnchor(e,t,i){const n=Hh(this.getDomNode().ownerDocument.body),s=this.widget.getLayoutInfo(),r=new kt(220,2*s.lineHeight),a=e.top,l=function(){const S=n.width-(e.left+e.width+s.borderWidth+s.horizontalPadding),L=-s.borderWidth+e.left+e.width,k=new kt(S,n.height-e.top-s.borderHeight-s.verticalPadding),D=k.with(void 0,e.top+e.height-s.borderHeight-s.verticalPadding);return{top:a,left:L,fit:S-t.width,maxSizeTop:k,maxSizeBottom:D,minSize:r.with(Math.min(S,r.width))}}(),c=function(){const S=e.left-s.borderWidth-s.horizontalPadding,L=Math.max(s.horizontalPadding,e.left-t.width-s.borderWidth),k=new kt(S,n.height-e.top-s.borderHeight-s.verticalPadding),D=k.with(void 0,e.top+e.height-s.borderHeight-s.verticalPadding);return{top:a,left:L,fit:S-t.width,maxSizeTop:k,maxSizeBottom:D,minSize:r.with(Math.min(S,r.width))}}(),d=function(){const S=e.left,L=-s.borderWidth+e.top+e.height,k=new kt(e.width-s.borderHeight,n.height-e.top-e.height-s.verticalPadding);return{top:L,left:S,fit:k.height-t.height,maxSizeBottom:k,maxSizeTop:k,minSize:r.with(k.width)}}(),h=[l,c,d],u=h.find(S=>S.fit>=0)??h.sort((S,L)=>L.fit-S.fit)[0],g=e.top+e.height-s.borderHeight;let f,p=t.height;const _=Math.max(u.maxSizeTop.height,u.maxSizeBottom.height);p>_&&(p=_);let b;i?p<=u.maxSizeTop.height?(f=!0,b=u.maxSizeTop):(f=!1,b=u.maxSizeBottom):p<=u.maxSizeBottom.height?(f=!1,b=u.maxSizeBottom):(f=!0,b=u.maxSizeTop);let{top:C,left:v}=u;!f&&p>e.height&&(C=g-p);const w=this._editor.getDomNode();if(w){const S=w.getBoundingClientRect();C-=S.top,v-=S.left}this._applyTopLeft({left:v,top:C}),this._resizable.enableSashes(!f,u===l,f,u!==l),this._resizable.minSize=u.minSize,this._resizable.maxSize=b,this._resizable.layout(p,Math.min(b.width,t.width)),this.widget.layout(this._resizable.size.width,this._resizable.size.height)}_applyTopLeft(e){this._topLeft=e,this._editor.layoutOverlayWidget(this)}}var Nl;(function(o){o[o.FILE=0]="FILE",o[o.FOLDER=1]="FOLDER",o[o.ROOT_FOLDER=2]="ROOT_FOLDER"})(Nl||(Nl={}));const tDe=/(?:\/|^)(?:([^\/]+)\/)?([^\/]+)$/;function I0(o,e,t,i,n){if(Ie.isThemeIcon(n))return[`codicon-${n.id}`,"predefined-file-icon"];if(Se.isUri(n))return[];const s=i===Nl.ROOT_FOLDER?["rootfolder-icon"]:i===Nl.FOLDER?["folder-icon"]:["file-icon"];if(t){let r;if(t.scheme===Fe.data)r=Uh.parseMetaData(t).get(Uh.META_DATA_LABEL);else{const a=t.path.match(tDe);a?(r=E0(a[2].toLowerCase()),a[1]&&s.push(`${E0(a[1].toLowerCase())}-name-dir-icon`)):r=E0(t.authority.toLowerCase())}if(i===Nl.ROOT_FOLDER)s.push(`${r}-root-name-folder-icon`);else if(i===Nl.FOLDER)s.push(`${r}-name-folder-icon`);else{if(r){if(s.push(`${r}-name-file-icon`),s.push("name-file-icon"),r.length<=255){const l=r.split(".");for(let c=1;c<l.length;c++)s.push(`${l.slice(c).join(".")}-ext-file-icon`)}s.push("ext-file-icon")}const a=iDe(o,e,t);a&&s.push(`${E0(a)}-lang-file-icon`)}}return s}function iDe(o,e,t){if(!t)return null;let i=null;if(t.scheme===Fe.data){const s=Uh.parseMetaData(t).get(Uh.META_DATA_MIME);s&&(i=e.getLanguageIdByMimeType(s))}else{const n=o.getModel(t);n&&(i=n.getLanguageId())}return i&&i!==ms?i:e.guessLanguageIdByFilepathOrFirstLine(t)}function E0(o){return o.replace(/[\s]/g,"/")}var nDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},YT=function(o,e){return function(t,i){e(t,i,o)}};function pY(o){return`suggest-aria-id:${o}`}const sDe=qt("suggest-more-info",te.chevronRight,m("suggestMoreInfoIcon","Icon for more information in the suggest widget."));var Sa;const oDe=new(Sa=class{extract(e,t){if(e.textLabel.match(Sa._regexStrict))return t[0]=e.textLabel,!0;if(e.completion.detail&&e.completion.detail.match(Sa._regexStrict))return t[0]=e.completion.detail,!0;if(e.completion.documentation){const i=typeof e.completion.documentation=="string"?e.completion.documentation:e.completion.documentation.value,n=Sa._regexRelaxed.exec(i);if(n&&(n.index===0||n.index+n[0].length===i.length))return t[0]=n[0],!0}return!1}},Sa._regexRelaxed=/(#([\da-fA-F]{3}){1,2}|(rgb|hsl)a\(\s*(\d{1,3}%?\s*,\s*){3}(1|0?\.\d+)\)|(rgb|hsl)\(\s*\d{1,3}%?(\s*,\s*\d{1,3}%?){2}\s*\))/,Sa._regexStrict=new RegExp(`^${Sa._regexRelaxed.source}$`,"i"),Sa);let aF=class{constructor(e,t,i,n){this._editor=e,this._modelService=t,this._languageService=i,this._themeService=n,this._onDidToggleDetails=new P,this.onDidToggleDetails=this._onDidToggleDetails.event,this.templateId="suggestion"}dispose(){this._onDidToggleDetails.dispose()}renderTemplate(e){const t=new K,i=e;i.classList.add("show-file-icons");const n=X(e,ne(".icon")),s=X(n,ne("span.colorspan")),r=X(e,ne(".contents")),a=X(r,ne(".main")),l=X(a,ne(".icon-label.codicon")),c=X(a,ne("span.left")),d=X(a,ne("span.right")),h=new qL(c,{supportHighlights:!0,supportIcons:!0});t.add(h);const u=X(c,ne("span.signature-label")),g=X(c,ne("span.qualifier-label")),f=X(d,ne("span.details-label")),p=X(d,ne("span.readMore"+Ie.asCSSSelector(sDe)));return p.title=m("readMore","Read More"),{root:i,left:c,right:d,icon:n,colorspan:s,iconLabel:h,iconContainer:l,parametersLabel:u,qualifierLabel:g,detailsLabel:f,readMore:p,disposables:t,configureFont:()=>{const b=this._editor.getOptions(),C=b.get(50),v=C.getMassagedFontFamily(),w=C.fontFeatureSettings,S=b.get(120)||C.fontSize,L=b.get(121)||C.lineHeight,k=C.fontWeight,D=C.letterSpacing,N=`${S}px`,B=`${L}px`,W=`${D}px`;i.style.fontSize=N,i.style.fontWeight=k,i.style.letterSpacing=W,a.style.fontFamily=v,a.style.fontFeatureSettings=w,a.style.lineHeight=B,n.style.height=B,n.style.width=B,p.style.height=B,p.style.width=B}}}renderElement(e,t,i){i.configureFont();const{completion:n}=e;i.root.id=pY(t),i.colorspan.style.backgroundColor="";const s={labelEscapeNewLines:!0,matches:U1(e.score)},r=[];if(n.kind===19&&oDe.extract(e,r))i.icon.className="icon customcolor",i.iconContainer.className="icon hide",i.colorspan.style.backgroundColor=r[0];else if(n.kind===20&&this._themeService.getFileIconTheme().hasFileIcons){i.icon.className="icon hide",i.iconContainer.className="icon hide";const a=I0(this._modelService,this._languageService,Se.from({scheme:"fake",path:e.textLabel}),Nl.FILE),l=I0(this._modelService,this._languageService,Se.from({scheme:"fake",path:n.detail}),Nl.FILE);s.extraClasses=a.length>l.length?a:l}else n.kind===23&&this._themeService.getFileIconTheme().hasFolderIcons?(i.icon.className="icon hide",i.iconContainer.className="icon hide",s.extraClasses=[I0(this._modelService,this._languageService,Se.from({scheme:"fake",path:e.textLabel}),Nl.FOLDER),I0(this._modelService,this._languageService,Se.from({scheme:"fake",path:n.detail}),Nl.FOLDER)].flat()):(i.icon.className="icon hide",i.iconContainer.className="",i.iconContainer.classList.add("suggest-icon",...Ie.asClassNameArray(tw.toIcon(n.kind))));n.tags&&n.tags.indexOf(1)>=0&&(s.extraClasses=(s.extraClasses||[]).concat(["deprecated"]),s.matches=[]),i.iconLabel.setLabel(e.textLabel,void 0,s),typeof n.label=="string"?(i.parametersLabel.textContent="",i.detailsLabel.textContent=XT(n.detail||""),i.root.classList.add("string-label")):(i.parametersLabel.textContent=XT(n.label.detail||""),i.detailsLabel.textContent=XT(n.label.description||""),i.root.classList.remove("string-label")),this._editor.getOption(119).showInlineDetails?to(i.detailsLabel):rs(i.detailsLabel),oB(e)?(i.right.classList.add("can-expand-details"),to(i.readMore),i.readMore.onmousedown=a=>{a.stopPropagation(),a.preventDefault()},i.readMore.onclick=a=>{a.stopPropagation(),a.preventDefault(),this._onDidToggleDetails.fire()}):(i.right.classList.remove("can-expand-details"),rs(i.readMore),i.readMore.onmousedown=null,i.readMore.onclick=null)}disposeTemplate(e){e.disposables.dispose()}};aF=nDe([YT(1,Kt),YT(2,ni),YT(3,ji)],aF);function XT(o){return o.replace(/\r\n|\r|\n/g,"")}var rDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},N0=function(o,e){return function(t,i){e(t,i,o)}},Ym;E("editorSuggestWidget.background",Wo,m("editorSuggestWidgetBackground","Background color of the suggest widget."));E("editorSuggestWidget.border",E5,m("editorSuggestWidgetBorder","Border color of the suggest widget."));const aDe=E("editorSuggestWidget.foreground",Zl,m("editorSuggestWidgetForeground","Foreground color of the suggest widget."));E("editorSuggestWidget.selectedForeground",Cw,m("editorSuggestWidgetSelectedForeground","Foreground color of the selected entry in the suggest widget."));E("editorSuggestWidget.selectedIconForeground",O5,m("editorSuggestWidgetSelectedIconForeground","Icon foreground color of the selected entry in the suggest widget."));const lDe=E("editorSuggestWidget.selectedBackground",vw,m("editorSuggestWidgetSelectedBackground","Background color of the selected entry in the suggest widget."));E("editorSuggestWidget.highlightForeground",sp,m("editorSuggestWidgetHighlightForeground","Color of the match highlights in the suggest widget."));E("editorSuggestWidget.focusHighlightForeground",Zre,m("editorSuggestWidgetFocusHighlightForeground","Color of the match highlights in the suggest widget when an item is focused."));E("editorSuggestWidgetStatus.foreground",$e(aDe,.5),m("editorSuggestWidgetStatusForeground","Foreground color of the suggest widget status."));class cDe{constructor(e,t){this._service=e,this._key=`suggestWidget.size/${t.getEditorType()}/${t instanceof Jr}`}restore(){const e=this._service.get(this._key,0)??"";try{const t=JSON.parse(e);if(kt.is(t))return kt.lift(t)}catch{}}store(e){this._service.store(this._key,JSON.stringify(e),0,1)}reset(){this._service.remove(this._key,0)}}var xh;let lF=(xh=class{constructor(e,t,i,n,s){this.editor=e,this._storageService=t,this._state=0,this._isAuto=!1,this._pendingLayout=new Ji,this._pendingShowDetails=new Ji,this._ignoreFocusEvents=!1,this._forceRenderingAbove=!1,this._explainMode=!1,this._showTimeout=new ea,this._disposables=new K,this._onDidSelect=new Lf,this._onDidFocus=new Lf,this._onDidHide=new P,this._onDidShow=new P,this.onDidSelect=this._onDidSelect.event,this.onDidFocus=this._onDidFocus.event,this.onDidHide=this._onDidHide.event,this.onDidShow=this._onDidShow.event,this._onDetailsKeydown=new P,this.onDetailsKeyDown=this._onDetailsKeydown.event,this.element=new WW,this.element.domNode.classList.add("editor-widget","suggest-widget"),this._contentWidget=new dDe(this,e),this._persistedSize=new cDe(t,e);class r{constructor(g,f,p=!1,_=!1){this.persistedSize=g,this.currentSize=f,this.persistHeight=p,this.persistWidth=_}}let a;this._disposables.add(this.element.onDidWillResize(()=>{this._contentWidget.lockPreference(),a=new r(this._persistedSize.restore(),this.element.size)})),this._disposables.add(this.element.onDidResize(u=>{var g,f;if(this._resize(u.dimension.width,u.dimension.height),a&&(a.persistHeight=a.persistHeight||!!u.north||!!u.south,a.persistWidth=a.persistWidth||!!u.east||!!u.west),!!u.done){if(a){const{itemHeight:p,defaultSize:_}=this.getLayoutInfo(),b=Math.round(p/2);let{width:C,height:v}=this.element.size;(!a.persistHeight||Math.abs(a.currentSize.height-v)<=b)&&(v=((g=a.persistedSize)==null?void 0:g.height)??_.height),(!a.persistWidth||Math.abs(a.currentSize.width-C)<=b)&&(C=((f=a.persistedSize)==null?void 0:f.width)??_.width),this._persistedSize.store(new kt(C,v))}this._contentWidget.unlockPreference(),a=void 0}})),this._messageElement=X(this.element.domNode,ne(".message")),this._listElement=X(this.element.domNode,ne(".tree"));const l=this._disposables.add(s.createInstance(rF,this.editor));l.onDidClose(this.toggleDetails,this,this._disposables),this._details=new eDe(l,this.editor);const c=()=>this.element.domNode.classList.toggle("no-icons",!this.editor.getOption(119).showIcons);c();const d=s.createInstance(aF,this.editor);this._disposables.add(d),this._disposables.add(d.onDidToggleDetails(()=>this.toggleDetails())),this._list=new Xo("SuggestWidget",this._listElement,{getHeight:u=>this.getLayoutInfo().itemHeight,getTemplateId:u=>"suggestion"},[d],{alwaysConsumeMouseWheel:!0,useShadows:!1,mouseSupport:!1,multipleSelectionSupport:!1,accessibilityProvider:{getRole:()=>"option",getWidgetAriaLabel:()=>m("suggest","Suggest"),getWidgetRole:()=>"listbox",getAriaLabel:u=>{let g=u.textLabel;if(typeof u.completion.label!="string"){const{detail:b,description:C}=u.completion.label;b&&C?g=m("label.full","{0} {1}, {2}",g,b,C):b?g=m("label.detail","{0} {1}",g,b):C&&(g=m("label.desc","{0}, {1}",g,C))}if(!u.isResolved||!this._isDetailsVisible())return g;const{documentation:f,detail:p}=u.completion,_=xf("{0}{1}",p||"",f?typeof f=="string"?f:f.value:"");return m("ariaCurrenttSuggestionReadDetails","{0}, docs: {1}",g,_)}}}),this._list.style(sm({listInactiveFocusBackground:lDe,listInactiveFocusOutline:Xt})),this._status=s.createInstance(oF,this.element.domNode,fh);const h=()=>this.element.domNode.classList.toggle("with-status-bar",this.editor.getOption(119).showStatusBar);h(),this._disposables.add(n.onDidColorThemeChange(u=>this._onThemeChange(u))),this._onThemeChange(n.getColorTheme()),this._disposables.add(this._list.onMouseDown(u=>this._onListMouseDownOrTap(u))),this._disposables.add(this._list.onTap(u=>this._onListMouseDownOrTap(u))),this._disposables.add(this._list.onDidChangeSelection(u=>this._onListSelection(u))),this._disposables.add(this._list.onDidChangeFocus(u=>this._onListFocus(u))),this._disposables.add(this.editor.onDidChangeCursorSelection(()=>this._onCursorSelectionChanged())),this._disposables.add(this.editor.onDidChangeConfiguration(u=>{u.hasChanged(119)&&(h(),c()),this._completionModel&&(u.hasChanged(50)||u.hasChanged(120)||u.hasChanged(121))&&this._list.splice(0,this._list.length,this._completionModel.items)})),this._ctxSuggestWidgetVisible=Ue.Visible.bindTo(i),this._ctxSuggestWidgetDetailsVisible=Ue.DetailsVisible.bindTo(i),this._ctxSuggestWidgetMultipleSuggestions=Ue.MultipleSuggestions.bindTo(i),this._ctxSuggestWidgetHasFocusedSuggestion=Ue.HasFocusedSuggestion.bindTo(i),this._disposables.add(hi(this._details.widget.domNode,"keydown",u=>{this._onDetailsKeydown.fire(u)})),this._disposables.add(this.editor.onMouseDown(u=>this._onEditorMouseDown(u)))}dispose(){var e;this._details.widget.dispose(),this._details.dispose(),this._list.dispose(),this._status.dispose(),this._disposables.dispose(),(e=this._loadingTimeout)==null||e.dispose(),this._pendingLayout.dispose(),this._pendingShowDetails.dispose(),this._showTimeout.dispose(),this._contentWidget.dispose(),this.element.dispose()}_onEditorMouseDown(e){this._details.widget.domNode.contains(e.target.element)?this._details.widget.domNode.focus():this.element.domNode.contains(e.target.element)&&this.editor.focus()}_onCursorSelectionChanged(){this._state!==0&&this._contentWidget.layout()}_onListMouseDownOrTap(e){typeof e.element>"u"||typeof e.index>"u"||(e.browserEvent.preventDefault(),e.browserEvent.stopPropagation(),this._select(e.element,e.index))}_onListSelection(e){e.elements.length&&this._select(e.elements[0],e.indexes[0])}_select(e,t){const i=this._completionModel;i&&(this._onDidSelect.fire({item:e,index:t,model:i}),this.editor.focus())}_onThemeChange(e){this._details.widget.borderWidth=Ta(e.type)?2:1}_onListFocus(e){var n;if(this._ignoreFocusEvents)return;if(!e.elements.length){this._currentSuggestionDetails&&(this._currentSuggestionDetails.cancel(),this._currentSuggestionDetails=void 0,this._focusedItem=void 0),this.editor.setAriaOptions({activeDescendant:void 0}),this._ctxSuggestWidgetHasFocusedSuggestion.set(!1);return}if(!this._completionModel)return;this._ctxSuggestWidgetHasFocusedSuggestion.set(!0);const t=e.elements[0],i=e.indexes[0];t!==this._focusedItem&&((n=this._currentSuggestionDetails)==null||n.cancel(),this._currentSuggestionDetails=void 0,this._focusedItem=t,this._list.reveal(i),this._currentSuggestionDetails=Yi(async s=>{const r=Bh(()=>{this._isDetailsVisible()&&this.showDetails(!0)},250),a=s.onCancellationRequested(()=>r.dispose());try{return await t.resolve(s)}finally{r.dispose(),a.dispose()}}),this._currentSuggestionDetails.then(()=>{i>=this._list.length||t!==this._list.element(i)||(this._ignoreFocusEvents=!0,this._list.splice(i,1,[t]),this._list.setFocus([i]),this._ignoreFocusEvents=!1,this._isDetailsVisible()?this.showDetails(!1):this.element.domNode.classList.remove("docs-side"),this.editor.setAriaOptions({activeDescendant:pY(i)}))}).catch(Oe)),this._onDidFocus.fire({item:t,index:i,model:this._completionModel})}_setState(e){if(this._state!==e)switch(this._state=e,this.element.domNode.classList.toggle("frozen",e===4),this.element.domNode.classList.remove("message"),e){case 0:rs(this._messageElement,this._listElement,this._status.element),this._details.hide(!0),this._status.hide(),this._contentWidget.hide(),this._ctxSuggestWidgetVisible.reset(),this._ctxSuggestWidgetMultipleSuggestions.reset(),this._ctxSuggestWidgetHasFocusedSuggestion.reset(),this._showTimeout.cancel(),this.element.domNode.classList.remove("visible"),this._list.splice(0,this._list.length),this._focusedItem=void 0,this._cappedHeight=void 0,this._explainMode=!1;break;case 1:this.element.domNode.classList.add("message"),this._messageElement.textContent=Ym.LOADING_MESSAGE,rs(this._listElement,this._status.element),to(this._messageElement),this._details.hide(),this._show(),this._focusedItem=void 0,Hl(Ym.LOADING_MESSAGE);break;case 2:this.element.domNode.classList.add("message"),this._messageElement.textContent=Ym.NO_SUGGESTIONS_MESSAGE,rs(this._listElement,this._status.element),to(this._messageElement),this._details.hide(),this._show(),this._focusedItem=void 0,Hl(Ym.NO_SUGGESTIONS_MESSAGE);break;case 3:rs(this._messageElement),to(this._listElement,this._status.element),this._show();break;case 4:rs(this._messageElement),to(this._listElement,this._status.element),this._show();break;case 5:rs(this._messageElement),to(this._listElement,this._status.element),this._details.show(),this._show();break}}_show(){this._status.show(),this._contentWidget.show(),this._layout(this._persistedSize.restore()),this._ctxSuggestWidgetVisible.set(!0),this._showTimeout.cancelAndSet(()=>{this.element.domNode.classList.add("visible"),this._onDidShow.fire(this)},100)}showTriggered(e,t){this._state===0&&(this._contentWidget.setPosition(this.editor.getPosition()),this._isAuto=!!e,this._isAuto||(this._loadingTimeout=Bh(()=>this._setState(1),t)))}showSuggestions(e,t,i,n,s){var l,c;if(this._contentWidget.setPosition(this.editor.getPosition()),(l=this._loadingTimeout)==null||l.dispose(),(c=this._currentSuggestionDetails)==null||c.cancel(),this._currentSuggestionDetails=void 0,this._completionModel!==e&&(this._completionModel=e),i&&this._state!==2&&this._state!==0){this._setState(4);return}const r=this._completionModel.items.length,a=r===0;if(this._ctxSuggestWidgetMultipleSuggestions.set(r>1),a){this._setState(n?0:2),this._completionModel=void 0;return}this._focusedItem=void 0,this._onDidFocus.pause(),this._onDidSelect.pause();try{this._list.splice(0,this._list.length,this._completionModel.items),this._setState(i?4:3),this._list.reveal(t,0),this._list.setFocus(s?[]:[t])}finally{this._onDidFocus.resume(),this._onDidSelect.resume()}this._pendingLayout.value=$y(xe(this.element.domNode),()=>{this._pendingLayout.clear(),this._layout(this.element.size),this._details.widget.domNode.classList.remove("focused")})}focusSelected(){this._list.length>0&&this._list.setFocus([0])}selectNextPage(){switch(this._state){case 0:return!1;case 5:return this._details.widget.pageDown(),!0;case 1:return!this._isAuto;default:return this._list.focusNextPage(),!0}}selectNext(){switch(this._state){case 0:return!1;case 1:return!this._isAuto;default:return this._list.focusNext(1,!0),!0}}selectLast(){switch(this._state){case 0:return!1;case 5:return this._details.widget.scrollBottom(),!0;case 1:return!this._isAuto;default:return this._list.focusLast(),!0}}selectPreviousPage(){switch(this._state){case 0:return!1;case 5:return this._details.widget.pageUp(),!0;case 1:return!this._isAuto;default:return this._list.focusPreviousPage(),!0}}selectPrevious(){switch(this._state){case 0:return!1;case 1:return!this._isAuto;default:return this._list.focusPrevious(1,!0),!1}}selectFirst(){switch(this._state){case 0:return!1;case 5:return this._details.widget.scrollTop(),!0;case 1:return!this._isAuto;default:return this._list.focusFirst(),!0}}getFocusedItem(){if(this._state!==0&&this._state!==2&&this._state!==1&&this._completionModel&&this._list.getFocus().length>0)return{item:this._list.getFocusedElements()[0],index:this._list.getFocus()[0],model:this._completionModel}}toggleDetailsFocus(){this._state===5?(this._setState(3),this._details.widget.domNode.classList.remove("focused")):this._state===3&&this._isDetailsVisible()&&(this._setState(5),this._details.widget.domNode.classList.add("focused"))}toggleDetails(){this._isDetailsVisible()?(this._pendingShowDetails.clear(),this._ctxSuggestWidgetDetailsVisible.set(!1),this._setDetailsVisible(!1),this._details.hide(),this.element.domNode.classList.remove("shows-details")):(oB(this._list.getFocusedElements()[0])||this._explainMode)&&(this._state===3||this._state===5||this._state===4)&&(this._ctxSuggestWidgetDetailsVisible.set(!0),this._setDetailsVisible(!0),this.showDetails(!1))}showDetails(e){this._pendingShowDetails.value=$y(xe(this.element.domNode),()=>{this._pendingShowDetails.clear(),this._details.show(),e?this._details.widget.renderLoading():this._details.widget.renderItem(this._list.getFocusedElements()[0],this._explainMode),this._details.widget.isEmpty?this._details.hide():(this._positionDetails(),this.element.domNode.classList.add("shows-details")),this.editor.focus()})}toggleExplainMode(){this._list.getFocusedElements()[0]&&(this._explainMode=!this._explainMode,this._isDetailsVisible()?this.showDetails(!1):this.toggleDetails())}resetPersistedSize(){this._persistedSize.reset()}hideWidget(){var i;this._pendingLayout.clear(),this._pendingShowDetails.clear(),(i=this._loadingTimeout)==null||i.dispose(),this._setState(0),this._onDidHide.fire(this),this.element.clearSashHoverState();const e=this._persistedSize.restore(),t=Math.ceil(this.getLayoutInfo().itemHeight*4.3);e&&e.height<t&&this._persistedSize.store(e.with(void 0,t))}isFrozen(){return this._state===4}_afterRender(e){if(e===null){this._isDetailsVisible()&&this._details.hide();return}this._state===2||this._state===1||(this._isDetailsVisible()&&!this._details.widget.isEmpty&&this._details.show(),this._positionDetails())}_layout(e){var r,a;if(!this.editor.hasModel()||!this.editor.getDomNode())return;const t=Hh(this.element.domNode.ownerDocument.body),i=this.getLayoutInfo();e||(e=i.defaultSize);let n=e.height,s=e.width;if(this._status.element.style.height=`${i.itemHeight}px`,this._state===2||this._state===1)n=i.itemHeight+i.borderHeight,s=i.defaultSize.width/2,this.element.enableSashes(!1,!1,!1,!1),this.element.minSize=this.element.maxSize=new kt(s,n),this._contentWidget.setPreference(2);else{const l=t.width-i.borderHeight-2*i.horizontalPadding;s>l&&(s=l);const c=this._completionModel?this._completionModel.stats.pLabelLen*i.typicalHalfwidthCharacterWidth:s,d=i.statusBarHeight+this._list.contentHeight+i.borderHeight,h=i.itemHeight+i.statusBarHeight,u=wi(this.editor.getDomNode()),g=this.editor.getScrolledVisiblePosition(this.editor.getPosition()),f=u.top+g.top+g.height,p=Math.min(t.height-f-i.verticalPadding,d),_=u.top+g.top-i.verticalPadding,b=Math.min(_,d);let C=Math.min(Math.max(b,p)+i.borderHeight,d);n===((r=this._cappedHeight)==null?void 0:r.capped)&&(n=this._cappedHeight.wanted),n<h&&(n=h),n>C&&(n=C),n>p||this._forceRenderingAbove&&_>150?(this._contentWidget.setPreference(1),this.element.enableSashes(!0,!0,!1,!1),C=b):(this._contentWidget.setPreference(2),this.element.enableSashes(!1,!0,!0,!1),C=p),this.element.preferredSize=new kt(c,i.defaultSize.height),this.element.maxSize=new kt(l,C),this.element.minSize=new kt(220,h),this._cappedHeight=n===d?{wanted:((a=this._cappedHeight)==null?void 0:a.wanted)??e.height,capped:n}:void 0}this._resize(s,n)}_resize(e,t){const{width:i,height:n}=this.element.maxSize;e=Math.min(i,e),t=Math.min(n,t);const{statusBarHeight:s}=this.getLayoutInfo();this._list.layout(t-s,e),this._listElement.style.height=`${t-s}px`,this.element.layout(t,e),this._contentWidget.layout(),this._positionDetails()}_positionDetails(){var e;this._isDetailsVisible()&&this._details.placeAtAnchor(this.element.domNode,((e=this._contentWidget.getPosition())==null?void 0:e.preference[0])===2)}getLayoutInfo(){const e=this.editor.getOption(50),t=os(this.editor.getOption(121)||e.lineHeight,8,1e3),i=!this.editor.getOption(119).showStatusBar||this._state===2||this._state===1?0:t,n=this._details.widget.borderWidth,s=2*n;return{itemHeight:t,statusBarHeight:i,borderWidth:n,borderHeight:s,typicalHalfwidthCharacterWidth:e.typicalHalfwidthCharacterWidth,verticalPadding:22,horizontalPadding:14,defaultSize:new kt(430,i+12*t+s)}}_isDetailsVisible(){return this._storageService.getBoolean("expandSuggestionDocs",0,!1)}_setDetailsVisible(e){this._storageService.store("expandSuggestionDocs",e,0,0)}forceRenderingAbove(){this._forceRenderingAbove||(this._forceRenderingAbove=!0,this._layout(this._persistedSize.restore()))}stopForceRenderingAbove(){this._forceRenderingAbove=!1}},Ym=xh,xh.LOADING_MESSAGE=m("suggestWidget.loading","Loading..."),xh.NO_SUGGESTIONS_MESSAGE=m("suggestWidget.noSuggestions","No suggestions."),xh);lF=Ym=rDe([N0(1,ia),N0(2,Ee),N0(3,ji),N0(4,we)],lF);class dDe{constructor(e,t){this._widget=e,this._editor=t,this.allowEditorOverflow=!0,this.suppressMouseDown=!1,this._preferenceLocked=!1,this._added=!1,this._hidden=!1}dispose(){this._added&&(this._added=!1,this._editor.removeContentWidget(this))}getId(){return"editor.widget.suggestWidget"}getDomNode(){return this._widget.element.domNode}show(){this._hidden=!1,this._added||(this._added=!0,this._editor.addContentWidget(this))}hide(){this._hidden||(this._hidden=!0,this.layout())}layout(){this._editor.layoutContentWidget(this)}getPosition(){return this._hidden||!this._position||!this._preference?null:{position:this._position,preference:[this._preference]}}beforeRender(){const{height:e,width:t}=this._widget.element.size,{borderWidth:i,horizontalPadding:n}=this._widget.getLayoutInfo();return new kt(t+2*i+n,e+2*i)}afterRender(e){this._widget._afterRender(e)}setPreference(e){this._preferenceLocked||(this._preference=e)}lockPreference(){this._preferenceLocked=!0}unlockPreference(){this._preferenceLocked=!1}setPosition(e){this._position=e}}var hDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Bm=function(o,e){return function(t,i){e(t,i,o)}},cF;class uDe{constructor(e,t){if(this._model=e,this._position=t,this._decorationOptions=We.register({description:"suggest-line-suffix",stickiness:1}),e.getLineMaxColumn(t.lineNumber)!==t.column){const n=e.getOffsetAt(t),s=e.getPositionAt(n+1);e.changeDecorations(r=>{this._marker&&r.removeDecoration(this._marker),this._marker=r.addDecoration(x.fromPositions(t,s),this._decorationOptions)})}}dispose(){this._marker&&!this._model.isDisposed()&&this._model.changeDecorations(e=>{e.removeDecoration(this._marker),this._marker=void 0})}delta(e){if(this._model.isDisposed()||this._position.lineNumber!==e.lineNumber)return 0;if(this._marker){const t=this._model.getDecorationRange(this._marker);return this._model.getOffsetAt(t.getStartPosition())-this._model.getOffsetAt(e)}else return this._model.getLineMaxColumn(e.lineNumber)-e.column}}var sf;let Ur=(sf=class{static get(e){return e.getContribution(cF.ID)}constructor(e,t,i,n,s,r,a){this._memoryService=t,this._commandService=i,this._contextKeyService=n,this._instantiationService=s,this._logService=r,this._telemetryService=a,this._lineSuffix=new Ji,this._toDispose=new K,this._selectors=new gDe(h=>h.priority),this._onWillInsertSuggestItem=new P,this.onWillInsertSuggestItem=this._onWillInsertSuggestItem.event,this.editor=e,this.model=s.createInstance(sk,this.editor),this._selectors.register({priority:0,select:(h,u,g)=>this._memoryService.select(h,u,g)});const l=Ue.InsertMode.bindTo(n);l.set(e.getOption(119).insertMode),this._toDispose.add(this.model.onDidTrigger(()=>l.set(e.getOption(119).insertMode))),this.widget=this._toDispose.add(new fN(xe(e.getDomNode()),()=>{const h=this._instantiationService.createInstance(lF,this.editor);this._toDispose.add(h),this._toDispose.add(h.onDidSelect(_=>this._insertSuggestion(_,0),this));const u=new qke(this.editor,h,this.model,_=>this._insertSuggestion(_,2));this._toDispose.add(u);const g=Ue.MakesTextEdit.bindTo(this._contextKeyService),f=Ue.HasInsertAndReplaceRange.bindTo(this._contextKeyService),p=Ue.CanResolve.bindTo(this._contextKeyService);return this._toDispose.add(ve(()=>{g.reset(),f.reset(),p.reset()})),this._toDispose.add(h.onDidFocus(({item:_})=>{const b=this.editor.getPosition(),C=_.editStart.column,v=b.column;let w=!0;this.editor.getOption(1)==="smart"&&this.model.state===2&&!_.completion.additionalTextEdits&&!(_.completion.insertTextRules&4)&&v-C===_.completion.insertText.length&&(w=this.editor.getModel().getValueInRange({startLineNumber:b.lineNumber,startColumn:C,endLineNumber:b.lineNumber,endColumn:v})!==_.completion.insertText),g.set(w),f.set(!A.equals(_.editInsertEnd,_.editReplaceEnd)),p.set(!!_.provider.resolveCompletionItem||!!_.completion.documentation||_.completion.detail!==_.completion.label)})),this._toDispose.add(h.onDetailsKeyDown(_=>{if(_.toKeyCodeChord().equals(new Jc(!0,!1,!1,!1,33))||Je&&_.toKeyCodeChord().equals(new Jc(!1,!1,!1,!0,33))){_.stopPropagation();return}_.toKeyCodeChord().isModifierKey()||this.editor.focus()})),h})),this._overtypingCapturer=this._toDispose.add(new fN(xe(e.getDomNode()),()=>this._toDispose.add(new sF(this.editor,this.model)))),this._alternatives=this._toDispose.add(new fN(xe(e.getDomNode()),()=>this._toDispose.add(new ob(this.editor,this._contextKeyService)))),this._toDispose.add(s.createInstance(tk,e)),this._toDispose.add(this.model.onDidTrigger(h=>{this.widget.value.showTriggered(h.auto,h.shy?250:50),this._lineSuffix.value=new uDe(this.editor.getModel(),h.position)})),this._toDispose.add(this.model.onDidSuggest(h=>{if(h.triggerOptions.shy)return;let u=-1;for(const f of this._selectors.itemsOrderedByPriorityDesc)if(u=f.select(this.editor.getModel(),this.editor.getPosition(),h.completionModel.items),u!==-1)break;if(u===-1&&(u=0),this.model.state===0)return;let g=!1;if(h.triggerOptions.auto){const f=this.editor.getOption(119);f.selectionMode==="never"||f.selectionMode==="always"?g=f.selectionMode==="never":f.selectionMode==="whenTriggerCharacter"?g=h.triggerOptions.triggerKind!==1:f.selectionMode==="whenQuickSuggestion"&&(g=h.triggerOptions.triggerKind===1&&!h.triggerOptions.refilter)}this.widget.value.showSuggestions(h.completionModel,u,h.isFrozen,h.triggerOptions.auto,g)})),this._toDispose.add(this.model.onDidCancel(h=>{h.retrigger||this.widget.value.hideWidget()})),this._toDispose.add(this.editor.onDidBlurEditorWidget(()=>{this.model.cancel(),this.model.clear()}));const c=Ue.AcceptSuggestionsOnEnter.bindTo(n),d=()=>{const h=this.editor.getOption(1);c.set(h==="on"||h==="smart")};this._toDispose.add(this.editor.onDidChangeConfiguration(()=>d())),d()}dispose(){this._alternatives.dispose(),this._toDispose.dispose(),this.widget.dispose(),this.model.dispose(),this._lineSuffix.dispose(),this._onWillInsertSuggestItem.dispose()}_insertSuggestion(e,t){if(!e||!e.item){this._alternatives.value.reset(),this.model.cancel(),this.model.clear();return}if(!this.editor.hasModel())return;const i=vs.get(this.editor);if(!i)return;this._onWillInsertSuggestItem.fire({item:e.item});const n=this.editor.getModel(),s=n.getAlternativeVersionId(),{item:r}=e,a=[],l=new li;t&1||this.editor.pushUndoStop();const c=this.getOverwriteInfo(r,!!(t&8));this._memoryService.memorize(n,this.editor.getPosition(),r);const d=r.isResolved;let h=-1,u=-1;if(Array.isArray(r.completion.additionalTextEdits)){this.model.cancel();const f=za.capture(this.editor);this.editor.executeEdits("suggestController.additionalTextEdits.sync",r.completion.additionalTextEdits.map(p=>{let _=x.lift(p.range);if(_.startLineNumber===r.position.lineNumber&&_.startColumn>r.position.column){const b=this.editor.getPosition().column-r.position.column,C=b,v=x.spansMultipleLines(_)?0:b;_=new x(_.startLineNumber,_.startColumn+C,_.endLineNumber,_.endColumn+v)}return ri.replaceMove(_,p.text)})),f.restoreRelativeVerticalPositionOfCursor(this.editor)}else if(!d){const f=new ln;let p;const _=n.onDidChangeContent(w=>{if(w.isFlush){l.cancel(),_.dispose();return}for(const S of w.changes){const L=x.getEndPosition(S.range);(!p||A.isBefore(L,p))&&(p=L)}}),b=t;t|=2;let C=!1;const v=this.editor.onWillType(()=>{v.dispose(),C=!0,b&2||this.editor.pushUndoStop()});a.push(r.resolve(l.token).then(()=>{if(!r.completion.additionalTextEdits||l.token.isCancellationRequested)return;if(p&&r.completion.additionalTextEdits.some(S=>A.isBefore(p,x.getStartPosition(S.range))))return!1;C&&this.editor.pushUndoStop();const w=za.capture(this.editor);return this.editor.executeEdits("suggestController.additionalTextEdits.async",r.completion.additionalTextEdits.map(S=>ri.replaceMove(x.lift(S.range),S.text))),w.restoreRelativeVerticalPositionOfCursor(this.editor),(C||!(b&2))&&this.editor.pushUndoStop(),!0}).then(w=>{this._logService.trace("[suggest] async resolving of edits DONE (ms, applied?)",f.elapsed(),w),u=w===!0?1:w===!1?0:-2}).finally(()=>{_.dispose(),v.dispose()}))}let{insertText:g}=r.completion;if(r.completion.insertTextRules&4||(g=zf.escape(g)),this.model.cancel(),i.insert(g,{overwriteBefore:c.overwriteBefore,overwriteAfter:c.overwriteAfter,undoStopBefore:!1,undoStopAfter:!1,adjustWhitespace:!(r.completion.insertTextRules&1),clipboardText:e.model.clipboardText,overtypingCapturer:this._overtypingCapturer.value}),t&2||this.editor.pushUndoStop(),r.completion.command)if(r.completion.command.id===ok.id)this.model.trigger({auto:!0,retrigger:!0});else{const f=new ln;a.push(this._commandService.executeCommand(r.completion.command.id,...r.completion.command.arguments?[...r.completion.command.arguments]:[]).catch(p=>{r.completion.extensionId?Si(p):Oe(p)}).finally(()=>{h=f.elapsed()}))}t&4&&this._alternatives.value.set(e,f=>{for(l.cancel();n.canUndo();){s!==n.getAlternativeVersionId()&&n.undo(),this._insertSuggestion(f,3|(t&8?8:0));break}}),this._alertCompletionItem(r),Promise.all(a).finally(()=>{this._reportSuggestionAcceptedTelemetry(r,n,d,h,u,e.index,e.model.items),this.model.clear(),l.dispose()})}_reportSuggestionAcceptedTelemetry(e,t,i,n,s,r,a){var u;if(Math.floor(Math.random()*100)===0)return;const l=new Map;for(let g=0;g<Math.min(30,a.length);g++){const f=a[g].textLabel;l.has(f)?l.get(f).push(g):l.set(f,[g])}const c=l.get(e.textLabel),h=c&&c.length>1?c[0]:-1;this._telemetryService.publicLog2("suggest.acceptedSuggestion",{extensionId:((u=e.extensionId)==null?void 0:u.value)??"unknown",providerId:e.provider._debugDisplayName??"unknown",kind:e.completion.kind,basenameHash:_I(vr(t.uri)).toString(16),languageId:t.getLanguageId(),fileExtension:ale(t.uri),resolveInfo:e.provider.resolveCompletionItem?i?1:0:-1,resolveDuration:e.resolveDuration,commandDuration:n,additionalEditsAsync:s,index:r,firstIndex:h})}getOverwriteInfo(e,t){mt(this.editor.hasModel());let i=this.editor.getOption(119).insertMode==="replace";t&&(i=!i);const n=e.position.column-e.editStart.column,s=(i?e.editReplaceEnd.column:e.editInsertEnd.column)-e.position.column,r=this.editor.getPosition().column-e.position.column,a=this._lineSuffix.value?this._lineSuffix.value.delta(this.editor.getPosition()):0;return{overwriteBefore:n+r,overwriteAfter:s+a}}_alertCompletionItem(e){if(Qn(e.completion.additionalTextEdits)){const t=m("aria.alert.snippet","Accepting '{0}' made {1} additional edits",e.textLabel,e.completion.additionalTextEdits.length);Cs(t)}}triggerSuggest(e,t,i){this.editor.hasModel()&&(this.model.trigger({auto:t??!1,completionOptions:{providerFilter:e,kindFilter:i?new Set:void 0}}),this.editor.revealPosition(this.editor.getPosition(),0),this.editor.focus())}triggerSuggestAndAcceptBest(e){if(!this.editor.hasModel())return;const t=this.editor.getPosition(),i=()=>{t.equals(this.editor.getPosition())&&this._commandService.executeCommand(e.fallback)},n=s=>{if(s.completion.insertTextRules&4||s.completion.additionalTextEdits)return!0;const r=this.editor.getPosition(),a=s.editStart.column,l=r.column;return l-a!==s.completion.insertText.length?!0:this.editor.getModel().getValueInRange({startLineNumber:r.lineNumber,startColumn:a,endLineNumber:r.lineNumber,endColumn:l})!==s.completion.insertText};oe.once(this.model.onDidTrigger)(s=>{const r=[];oe.any(this.model.onDidTrigger,this.model.onDidCancel)(()=>{Tt(r),i()},void 0,r),this.model.onDidSuggest(({completionModel:a})=>{if(Tt(r),a.items.length===0){i();return}const l=this._memoryService.select(this.editor.getModel(),this.editor.getPosition(),a.items),c=a.items[l];if(!n(c)){i();return}this.editor.pushUndoStop(),this._insertSuggestion({index:l,item:c,model:a},7)},void 0,r)}),this.model.trigger({auto:!1,shy:!0}),this.editor.revealPosition(t,0),this.editor.focus()}acceptSelectedSuggestion(e,t){const i=this.widget.value.getFocusedItem();let n=0;e&&(n|=4),t&&(n|=8),this._insertSuggestion(i,n)}acceptNextSuggestion(){this._alternatives.value.next()}acceptPrevSuggestion(){this._alternatives.value.prev()}cancelSuggestWidget(){this.model.cancel(),this.model.clear(),this.widget.value.hideWidget()}focusSuggestion(){this.widget.value.focusSelected()}selectNextSuggestion(){this.widget.value.selectNext()}selectNextPageSuggestion(){this.widget.value.selectNextPage()}selectLastSuggestion(){this.widget.value.selectLast()}selectPrevSuggestion(){this.widget.value.selectPrevious()}selectPrevPageSuggestion(){this.widget.value.selectPreviousPage()}selectFirstSuggestion(){this.widget.value.selectFirst()}toggleSuggestionDetails(){this.widget.value.toggleDetails()}toggleExplainMode(){this.widget.value.toggleExplainMode()}toggleSuggestionFocus(){this.widget.value.toggleDetailsFocus()}resetWidgetSize(){this.widget.value.resetPersistedSize()}forceRenderingAbove(){this.widget.value.forceRenderingAbove()}stopForceRenderingAbove(){this.widget.isInitialized&&this.widget.value.stopForceRenderingAbove()}registerSelector(e){return this._selectors.register(e)}},cF=sf,sf.ID="editor.contrib.suggestController",sf);Ur=cF=hDe([Bm(1,VE),Bm(2,Ht),Bm(3,Ee),Bm(4,we),Bm(5,Nn),Bm(6,Bn)],Ur);class gDe{constructor(e){this.prioritySelector=e,this._items=new Array}register(e){if(this._items.indexOf(e)!==-1)throw new Error("Value is already registered");return this._items.push(e),this._items.sort((t,i)=>this.prioritySelector(i)-this.prioritySelector(t)),{dispose:()=>{const t=this._items.indexOf(e);t>=0&&this._items.splice(t,1)}}}get itemsOrderedByPriorityDesc(){return this._items}}const RD=class RD extends de{constructor(){super({id:RD.id,label:m("suggest.trigger.label","Trigger Suggest"),alias:"Trigger Suggest",precondition:G.and(I.writable,I.hasCompletionItemProvider,Ue.Visible.toNegated()),kbOpts:{kbExpr:I.textInputFocus,primary:2058,secondary:[2087],mac:{primary:266,secondary:[521,2087]},weight:100}})}run(e,t,i){const n=Ur.get(t);if(!n)return;let s;i&&typeof i=="object"&&i.auto===!0&&(s=!0),n.triggerSuggest(void 0,s,void 0)}};RD.id="editor.action.triggerSuggest";let ok=RD;ct(Ur.ID,Ur,2);Q(ok);const qo=190,Ls=$i.bindToContribution(Ur.get);re(new Ls({id:"acceptSelectedSuggestion",precondition:G.and(Ue.Visible,Ue.HasFocusedSuggestion),handler(o){o.acceptSelectedSuggestion(!0,!1)},kbOpts:[{primary:2,kbExpr:G.and(Ue.Visible,I.textInputFocus),weight:qo},{primary:3,kbExpr:G.and(Ue.Visible,I.textInputFocus,Ue.AcceptSuggestionsOnEnter,Ue.MakesTextEdit),weight:qo}],menuOpts:[{menuId:fh,title:m("accept.insert","Insert"),group:"left",order:1,when:Ue.HasInsertAndReplaceRange.toNegated()},{menuId:fh,title:m("accept.insert","Insert"),group:"left",order:1,when:G.and(Ue.HasInsertAndReplaceRange,Ue.InsertMode.isEqualTo("insert"))},{menuId:fh,title:m("accept.replace","Replace"),group:"left",order:1,when:G.and(Ue.HasInsertAndReplaceRange,Ue.InsertMode.isEqualTo("replace"))}]}));re(new Ls({id:"acceptAlternativeSelectedSuggestion",precondition:G.and(Ue.Visible,I.textInputFocus,Ue.HasFocusedSuggestion),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:1027,secondary:[1026]},handler(o){o.acceptSelectedSuggestion(!1,!0)},menuOpts:[{menuId:fh,group:"left",order:2,when:G.and(Ue.HasInsertAndReplaceRange,Ue.InsertMode.isEqualTo("insert")),title:m("accept.replace","Replace")},{menuId:fh,group:"left",order:2,when:G.and(Ue.HasInsertAndReplaceRange,Ue.InsertMode.isEqualTo("replace")),title:m("accept.insert","Insert")}]}));rt.registerCommandAlias("acceptSelectedSuggestionOnEnter","acceptSelectedSuggestion");re(new Ls({id:"hideSuggestWidget",precondition:Ue.Visible,handler:o=>o.cancelSuggestWidget(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:9,secondary:[1033]}}));re(new Ls({id:"selectNextSuggestion",precondition:G.and(Ue.Visible,G.or(Ue.MultipleSuggestions,Ue.HasFocusedSuggestion.negate())),handler:o=>o.selectNextSuggestion(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:18,secondary:[2066],mac:{primary:18,secondary:[2066,300]}}}));re(new Ls({id:"selectNextPageSuggestion",precondition:G.and(Ue.Visible,G.or(Ue.MultipleSuggestions,Ue.HasFocusedSuggestion.negate())),handler:o=>o.selectNextPageSuggestion(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:12,secondary:[2060]}}));re(new Ls({id:"selectLastSuggestion",precondition:G.and(Ue.Visible,G.or(Ue.MultipleSuggestions,Ue.HasFocusedSuggestion.negate())),handler:o=>o.selectLastSuggestion()}));re(new Ls({id:"selectPrevSuggestion",precondition:G.and(Ue.Visible,G.or(Ue.MultipleSuggestions,Ue.HasFocusedSuggestion.negate())),handler:o=>o.selectPrevSuggestion(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:16,secondary:[2064],mac:{primary:16,secondary:[2064,302]}}}));re(new Ls({id:"selectPrevPageSuggestion",precondition:G.and(Ue.Visible,G.or(Ue.MultipleSuggestions,Ue.HasFocusedSuggestion.negate())),handler:o=>o.selectPrevPageSuggestion(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:11,secondary:[2059]}}));re(new Ls({id:"selectFirstSuggestion",precondition:G.and(Ue.Visible,G.or(Ue.MultipleSuggestions,Ue.HasFocusedSuggestion.negate())),handler:o=>o.selectFirstSuggestion()}));re(new Ls({id:"focusSuggestion",precondition:G.and(Ue.Visible,Ue.HasFocusedSuggestion.negate()),handler:o=>o.focusSuggestion(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:2058,secondary:[2087],mac:{primary:266,secondary:[2087]}}}));re(new Ls({id:"focusAndAcceptSuggestion",precondition:G.and(Ue.Visible,Ue.HasFocusedSuggestion.negate()),handler:o=>{o.focusSuggestion(),o.acceptSelectedSuggestion(!0,!1)}}));re(new Ls({id:"toggleSuggestionDetails",precondition:G.and(Ue.Visible,Ue.HasFocusedSuggestion),handler:o=>o.toggleSuggestionDetails(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:2058,secondary:[2087],mac:{primary:266,secondary:[2087]}},menuOpts:[{menuId:fh,group:"right",order:1,when:G.and(Ue.DetailsVisible,Ue.CanResolve),title:m("detail.more","Show Less")},{menuId:fh,group:"right",order:1,when:G.and(Ue.DetailsVisible.toNegated(),Ue.CanResolve),title:m("detail.less","Show More")}]}));re(new Ls({id:"toggleExplainMode",precondition:Ue.Visible,handler:o=>o.toggleExplainMode(),kbOpts:{weight:100,primary:2138}}));re(new Ls({id:"toggleSuggestionFocus",precondition:Ue.Visible,handler:o=>o.toggleSuggestionFocus(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:2570,mac:{primary:778}}}));re(new Ls({id:"insertBestCompletion",precondition:G.and(I.textInputFocus,G.equals("config.editor.tabCompletion","on"),tk.AtEnd,Ue.Visible.toNegated(),ob.OtherSuggestions.toNegated(),vs.InSnippetMode.toNegated()),handler:(o,e)=>{o.triggerSuggestAndAcceptBest(on(e)?{fallback:"tab",...e}:{fallback:"tab"})},kbOpts:{weight:qo,primary:2}}));re(new Ls({id:"insertNextSuggestion",precondition:G.and(I.textInputFocus,G.equals("config.editor.tabCompletion","on"),ob.OtherSuggestions,Ue.Visible.toNegated(),vs.InSnippetMode.toNegated()),handler:o=>o.acceptNextSuggestion(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:2}}));re(new Ls({id:"insertPrevSuggestion",precondition:G.and(I.textInputFocus,G.equals("config.editor.tabCompletion","on"),ob.OtherSuggestions,Ue.Visible.toNegated(),vs.InSnippetMode.toNegated()),handler:o=>o.acceptPrevSuggestion(),kbOpts:{weight:qo,kbExpr:I.textInputFocus,primary:1026}}));Q(class extends de{constructor(){super({id:"editor.action.resetSuggestSize",label:m("suggest.reset.label","Reset Suggest Widget Size"),alias:"Reset Suggest Widget Size",precondition:void 0})}run(o,e){var t;(t=Ur.get(e))==null||t.resetWidgetSize()}});class fDe extends F{get selectedItem(){return this._currentSuggestItemInfo}constructor(e,t,i){super(),this.editor=e,this.suggestControllerPreselector=t,this.onWillAccept=i,this.isSuggestWidgetVisible=!1,this.isShiftKeyPressed=!1,this._isActive=!1,this._currentSuggestItemInfo=void 0,this._onDidSelectedItemChange=this._register(new P),this.onDidSelectedItemChange=this._onDidSelectedItemChange.event,this._register(e.onKeyDown(s=>{s.shiftKey&&!this.isShiftKeyPressed&&(this.isShiftKeyPressed=!0,this.update(this._isActive))})),this._register(e.onKeyUp(s=>{s.shiftKey&&this.isShiftKeyPressed&&(this.isShiftKeyPressed=!1,this.update(this._isActive))}));const n=Ur.get(this.editor);if(n){this._register(n.registerSelector({priority:100,select:(a,l,c)=>{const d=this.editor.getModel();if(!d)return-1;const h=this.suggestControllerPreselector(),u=h?xg(h,d):void 0;if(!u)return-1;const g=A.lift(l),f=c.map((_,b)=>{const C=yv.fromSuggestion(n,d,g,_,this.isShiftKeyPressed),v=xg(C.toSingleTextEdit(),d),w=mY(u,v);return{index:b,valid:w,prefixLength:v.text.length,suggestItem:_}}).filter(_=>_&&_.valid&&_.prefixLength>0),p=C5(f,gs(_=>_.prefixLength,$r));return p?p.index:-1}}));let s=!1;const r=()=>{s||(s=!0,this._register(n.widget.value.onDidShow(()=>{this.isSuggestWidgetVisible=!0,this.update(!0)})),this._register(n.widget.value.onDidHide(()=>{this.isSuggestWidgetVisible=!1,this.update(!1)})),this._register(n.widget.value.onDidFocus(()=>{this.isSuggestWidgetVisible=!0,this.update(!0)})))};this._register(oe.once(n.model.onDidTrigger)(a=>{r()})),this._register(n.onWillInsertSuggestItem(a=>{const l=this.editor.getPosition(),c=this.editor.getModel();if(!l||!c)return;const d=yv.fromSuggestion(n,c,l,a.item,this.isShiftKeyPressed);this.onWillAccept(d)}))}this.update(this._isActive)}update(e){const t=this.getSuggestItemInfo();(this._isActive!==e||!mDe(this._currentSuggestItemInfo,t))&&(this._isActive=e,this._currentSuggestItemInfo=t,this._onDidSelectedItemChange.fire())}getSuggestItemInfo(){const e=Ur.get(this.editor);if(!e||!this.isSuggestWidgetVisible)return;const t=e.widget.value.getFocusedItem(),i=this.editor.getPosition(),n=this.editor.getModel();if(!(!t||!i||!n))return yv.fromSuggestion(e,n,i,t.item,this.isShiftKeyPressed)}stopForceRenderingAbove(){const e=Ur.get(this.editor);e==null||e.stopForceRenderingAbove()}forceRenderingAbove(){const e=Ur.get(this.editor);e==null||e.forceRenderingAbove()}}class yv{static fromSuggestion(e,t,i,n,s){let{insertText:r}=n.completion,a=!1;if(n.completion.insertTextRules&4){const c=new zf().parse(r);c.children.length<100&&ek.adjustWhitespace(t,i,!0,c),r=c.toString(),a=!0}const l=e.getOverwriteInfo(n,s);return new yv(x.fromPositions(i.delta(0,-l.overwriteBefore),i.delta(0,Math.max(l.overwriteAfter,0))),r,n.completion.kind,a)}constructor(e,t,i,n){this.range=e,this.insertText=t,this.completionItemKind=i,this.isSnippetText=n}equals(e){return this.range.equalsRange(e.range)&&this.insertText===e.insertText&&this.completionItemKind===e.completionItemKind&&this.isSnippetText===e.isSnippetText}toSelectedSuggestionInfo(){return new Yz(this.range,this.insertText,this.completionItemKind,this.isSnippetText)}toSingleTextEdit(){return new Fl(this.range,this.insertText)}}function mDe(o,e){return o===e?!0:!o||!e?!1:o.equals(e)}var pDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},ac=function(o,e){return function(t,i){e(t,i,o)}},dF,of;let Sr=(of=class extends F{static get(e){return e.getContribution(dF.ID)}constructor(e,t,i,n,s,r,a,l,c,d){super(),this.editor=e,this._instantiationService=t,this._contextKeyService=i,this._configurationService=n,this._commandService=s,this._debounceService=r,this._languageFeaturesService=a,this._accessibilitySignalService=l,this._keybindingService=c,this._accessibilityService=d,this._editorObs=no(this.editor),this._positions=be(this,u=>{var g;return((g=this._editorObs.selections.read(u))==null?void 0:g.map(f=>f.getEndPosition()))??[new A(1,1)]}),this._suggestWidgetAdaptor=this._register(new fDe(this.editor,()=>{var u,g;return this._editorObs.forceUpdate(),(g=(u=this.model.get())==null?void 0:u.selectedInlineCompletion.get())==null?void 0:g.toSingleTextEdit(void 0)},u=>this._editorObs.forceUpdate(g=>{var f;(f=this.model.get())==null||f.handleSuggestAccepted(u)}))),this._suggestWidgetSelectedItem=xt(this,u=>this._suggestWidgetAdaptor.onDidSelectedItemChange(()=>{this._editorObs.forceUpdate(g=>u(void 0))}),()=>this._suggestWidgetAdaptor.selectedItem),this._enabledInConfig=xt(this,this.editor.onDidChangeConfiguration,()=>this.editor.getOption(62).enabled),this._isScreenReaderEnabled=xt(this,this._accessibilityService.onDidChangeScreenReaderOptimized,()=>this._accessibilityService.isScreenReaderOptimized()),this._editorDictationInProgress=xt(this,this._contextKeyService.onDidChangeContext,()=>this._contextKeyService.getContext(this.editor.getDomNode()).getValue("editorDictation.inProgress")===!0),this._enabled=be(this,u=>this._enabledInConfig.read(u)&&(!this._isScreenReaderEnabled.read(u)||!this._editorDictationInProgress.read(u))),this._debounceValue=this._debounceService.for(this._languageFeaturesService.inlineCompletionsProvider,"InlineCompletionsDebounce",{min:50,max:50}),this.model=Gn(this,u=>{if(this._editorObs.isReadonly.read(u))return;const g=this._editorObs.model.read(u);return g?this._instantiationService.createInstance(eF,g,this._suggestWidgetSelectedItem,this._editorObs.versionId,this._positions,this._debounceValue,xt(this.editor.onDidChangeConfiguration,()=>this.editor.getOption(119).preview),xt(this.editor.onDidChangeConfiguration,()=>this.editor.getOption(119).previewMode),xt(this.editor.onDidChangeConfiguration,()=>this.editor.getOption(62).mode),this._enabled):void 0}).recomputeInitiallyAndOnChange(this._store),this._ghostTexts=be(this,u=>{const g=this.model.read(u);return(g==null?void 0:g.ghostTexts.read(u))??[]}),this._stablizedGhostTexts=_De(this._ghostTexts,this._store),this._ghostTextWidgets=tde(this,this._stablizedGhostTexts,(u,g)=>g.add(this._instantiationService.createInstance(QO,this.editor,{ghostText:u,minReservedLineCount:ur(0),targetTextModel:this.model.map(f=>f==null?void 0:f.textModel)}))).recomputeInitiallyAndOnChange(this._store),this._playAccessibilitySignal=Lb(this),this._fontFamily=xt(this,this.editor.onDidChangeConfiguration,()=>this.editor.getOption(62).fontFamily),this._register(new ho(this._contextKeyService,this.model)),this._register(WP(this._editorObs.onDidType,(u,g)=>{var f;this._enabled.get()&&((f=this.model.get())==null||f.trigger())})),this._register(this._commandService.onDidExecuteCommand(u=>{new Set([r_.Tab.id,r_.DeleteLeft.id,r_.DeleteRight.id,LZ,"acceptSelectedSuggestion"]).has(u.commandId)&&e.hasTextFocus()&&this._enabled.get()&&this._editorObs.forceUpdate(f=>{var p;(p=this.model.get())==null||p.trigger(f)})})),this._register(WP(this._editorObs.selections,(u,g)=>{var f;g.some(p=>p.reason===3||p.source==="api")&&((f=this.model.get())==null||f.stop())})),this._register(this.editor.onDidBlurEditorWidget(()=>{this._contextKeyService.getContextKeyValue("accessibleViewIsShown")||this._configurationService.getValue("editor.inlineSuggest.keepOnBlur")||e.getOption(62).keepOnBlur||J_.dropDownVisible||ai(u=>{var g;(g=this.model.get())==null||g.stop(u)})})),this._register(Ae(u=>{var f;const g=(f=this.model.read(u))==null?void 0:f.state.read(u);g!=null&&g.suggestItem?g.primaryGhostText.lineCount>=2&&this._suggestWidgetAdaptor.forceRenderingAbove():this._suggestWidgetAdaptor.stopForceRenderingAbove()})),this._register(ve(()=>{this._suggestWidgetAdaptor.stopForceRenderingAbove()}));const h=Y1(this,(u,g)=>{var _;const f=this.model.read(u),p=f==null?void 0:f.state.read(u);return this._suggestWidgetSelectedItem.get()?g:(_=p==null?void 0:p.inlineCompletion)==null?void 0:_.semanticId});this._register(kve(be(u=>(this._playAccessibilitySignal.read(u),h.read(u),{})),async(u,g,f)=>{const p=this.model.get(),_=p==null?void 0:p.state.get();if(!_||!p)return;const b=p.textModel.getLineContent(_.primaryGhostText.lineNumber);await ed(50,_R(f)),await kj(this._suggestWidgetSelectedItem,Xn,()=>!1,_R(f)),await this._accessibilitySignalService.playSignal(No.inlineSuggestion),this.editor.getOption(8)&&this._provideScreenReaderUpdate(_.primaryGhostText.renderForScreenReader(b))})),this._register(new _O(this.editor,this.model,this._instantiationService)),this._register(ake(be(u=>{const g=this._fontFamily.read(u);return g===""||g==="default"?"":`
.monaco-editor .ghost-text-decoration,
.monaco-editor .ghost-text-decoration-preview,
.monaco-editor .ghost-text {
	font-family: ${g};
}`}))),this._register(this._configurationService.onDidChangeConfiguration(u=>{u.affectsConfiguration("accessibility.verbosity.inlineCompletions")&&this.editor.updateOptions({inlineCompletionsAccessibilityVerbose:this._configurationService.getValue("accessibility.verbosity.inlineCompletions")})})),this.editor.updateOptions({inlineCompletionsAccessibilityVerbose:this._configurationService.getValue("accessibility.verbosity.inlineCompletions")})}playAccessibilitySignal(e){this._playAccessibilitySignal.trigger(e)}_provideScreenReaderUpdate(e){const t=this._contextKeyService.getContextKeyValue("accessibleViewIsShown"),i=this._keybindingService.lookupKeybinding("editor.action.accessibleView");let n;!t&&i&&this.editor.getOption(150)&&(n=m("showAccessibleViewHint","Inspect this in the accessible view ({0})",i.getAriaLabel())),Cs(n?e+", "+n:e)}shouldShowHoverAt(e){var i;const t=(i=this.model.get())==null?void 0:i.primaryGhostText.get();return t?t.parts.some(n=>e.containsPosition(new A(t.lineNumber,n.column))):!1}shouldShowHoverAtViewZone(e){var t;return((t=this._ghostTextWidgets.get()[0])==null?void 0:t.ownsViewZone(e))??!1}},dF=of,of.ID="editor.contrib.inlineCompletionsController",of);Sr=dF=pDe([ac(1,we),ac(2,Ee),ac(3,qe),ac(4,Ht),ac(5,Zo),ac(6,ge),ac(7,ru),ac(8,yt),ac(9,po)],Sr);function _De(o,e){const t=Ke("result",[]),i=[];return e.add(Ae(n=>{const s=o.read(n);ai(r=>{if(s.length!==i.length){i.length=s.length;for(let a=0;a<i.length;a++)i[a]||(i[a]=Ke("item",s[a]));t.set([...i],r)}i.forEach((a,l)=>a.set(s[l],r))})})),t}const MD=class MD extends de{constructor(){super({id:MD.ID,label:m("action.inlineSuggest.showNext","Show Next Inline Suggestion"),alias:"Show Next Inline Suggestion",precondition:G.and(I.writable,ho.inlineSuggestionVisible),kbOpts:{weight:100,primary:606}})}async run(e,t){var n;const i=Sr.get(t);(n=i==null?void 0:i.model.get())==null||n.next()}};MD.ID=kZ;let hF=MD;const AD=class AD extends de{constructor(){super({id:AD.ID,label:m("action.inlineSuggest.showPrevious","Show Previous Inline Suggestion"),alias:"Show Previous Inline Suggestion",precondition:G.and(I.writable,ho.inlineSuggestionVisible),kbOpts:{weight:100,primary:604}})}async run(e,t){var n;const i=Sr.get(t);(n=i==null?void 0:i.model.get())==null||n.previous()}};AD.ID=xZ;let uF=AD;class bDe extends de{constructor(){super({id:"editor.action.inlineSuggest.trigger",label:m("action.inlineSuggest.trigger","Trigger Inline Suggestion"),alias:"Trigger Inline Suggestion",precondition:I.writable})}async run(e,t){const i=Sr.get(t);await yj(async n=>{var s;await((s=i==null?void 0:i.model.get())==null?void 0:s.triggerExplicitly(n)),i==null||i.playAccessibilitySignal(n)})}}class CDe extends de{constructor(){super({id:"editor.action.inlineSuggest.acceptNextWord",label:m("action.inlineSuggest.acceptNextWord","Accept Next Word Of Inline Suggestion"),alias:"Accept Next Word Of Inline Suggestion",precondition:G.and(I.writable,ho.inlineSuggestionVisible),kbOpts:{weight:101,primary:2065,kbExpr:G.and(I.writable,ho.inlineSuggestionVisible)},menuOpts:[{menuId:me.InlineSuggestionToolbar,title:m("acceptWord","Accept Word"),group:"primary",order:2}]})}async run(e,t){var n;const i=Sr.get(t);await((n=i==null?void 0:i.model.get())==null?void 0:n.acceptNextWord(i.editor))}}class vDe extends de{constructor(){super({id:"editor.action.inlineSuggest.acceptNextLine",label:m("action.inlineSuggest.acceptNextLine","Accept Next Line Of Inline Suggestion"),alias:"Accept Next Line Of Inline Suggestion",precondition:G.and(I.writable,ho.inlineSuggestionVisible),kbOpts:{weight:101},menuOpts:[{menuId:me.InlineSuggestionToolbar,title:m("acceptLine","Accept Line"),group:"secondary",order:2}]})}async run(e,t){var n;const i=Sr.get(t);await((n=i==null?void 0:i.model.get())==null?void 0:n.acceptNextLine(i.editor))}}class wDe extends de{constructor(){super({id:LZ,label:m("action.inlineSuggest.accept","Accept Inline Suggestion"),alias:"Accept Inline Suggestion",precondition:ho.inlineSuggestionVisible,menuOpts:[{menuId:me.InlineSuggestionToolbar,title:m("accept","Accept"),group:"primary",order:1}],kbOpts:{primary:2,weight:200,kbExpr:G.and(ho.inlineSuggestionVisible,I.tabMovesFocus.toNegated(),ho.inlineSuggestionHasIndentationLessThanTabSize,Ue.Visible.toNegated(),I.hoverFocused.toNegated())}})}async run(e,t){var n;const i=Sr.get(t);i&&((n=i.model.get())==null||n.accept(i.editor),i.editor.focus())}}const PD=class PD extends de{constructor(){super({id:PD.ID,label:m("action.inlineSuggest.hide","Hide Inline Suggestion"),alias:"Hide Inline Suggestion",precondition:ho.inlineSuggestionVisible,kbOpts:{weight:100,primary:9}})}async run(e,t){const i=Sr.get(t);ai(n=>{var s;(s=i==null?void 0:i.model.get())==null||s.stop(n)})}};PD.ID="editor.action.inlineSuggest.hide";let gF=PD;const OD=class OD extends ws{constructor(){super({id:OD.ID,title:m("action.inlineSuggest.alwaysShowToolbar","Always Show Toolbar"),f1:!1,precondition:void 0,menu:[{id:me.InlineSuggestionToolbar,group:"secondary",order:10}],toggled:G.equals("config.editor.inlineSuggest.showToolbar","always")})}async run(e,t){const i=e.get(qe),s=i.getValue("editor.inlineSuggest.showToolbar")==="always"?"onHover":"always";i.updateValue("editor.inlineSuggest.showToolbar",s)}};OD.ID="editor.action.inlineSuggest.toggleAlwaysShowToolbar";let fF=OD;var SDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},pC=function(o,e){return function(t,i){e(t,i,o)}};class yDe{constructor(e,t,i){this.owner=e,this.range=t,this.controller=i}isValidForHoverAnchor(e){return e.type===1&&this.range.startColumn<=e.range.startColumn&&this.range.endColumn>=e.range.endColumn}}let mF=class{constructor(e,t,i,n,s,r){this._editor=e,this._languageService=t,this._openerService=i,this.accessibilityService=n,this._instantiationService=s,this._telemetryService=r,this.hoverOrdinal=4}suggestHoverAnchor(e){const t=Sr.get(this._editor);if(!t)return null;const i=e.target;if(i.type===8){const n=i.detail;if(t.shouldShowHoverAtViewZone(n.viewZoneId))return new gy(1e3,this,x.fromPositions(this._editor.getModel().validatePosition(n.positionBefore||n.position)),e.event.posx,e.event.posy,!1)}return i.type===7&&t.shouldShowHoverAt(i.range)?new gy(1e3,this,i.range,e.event.posx,e.event.posy,!1):i.type===6&&i.detail.mightBeForeignElement&&t.shouldShowHoverAt(i.range)?new gy(1e3,this,i.range,e.event.posx,e.event.posy,!1):null}computeSync(e,t){if(this._editor.getOption(62).showToolbar!=="onHover")return[];const i=Sr.get(this._editor);return i&&i.shouldShowHoverAt(e.range)?[new yDe(this,e.range,i)]:[]}renderHoverParts(e,t){const i=new K,n=t[0];this._telemetryService.publicLog2("inlineCompletionHover.shown"),this.accessibilityService.isScreenReaderOptimized()&&!this._editor.getOption(8)&&i.add(this.renderScreenReaderText(e,n));const s=n.controller.model.get(),r=this._instantiationService.createInstance(J_,this._editor,!1,ur(null),s.selectedInlineCompletionIndex,s.inlineCompletionsCount,s.activeCommands),a=r.getDomNode();e.fragment.appendChild(a),s.triggerExplicitly(),i.add(r);const l={hoverPart:n,hoverElement:a,dispose(){i.dispose()}};return new Kf([l])}renderScreenReaderText(e,t){const i=new K,n=ne,s=n("div.hover-row.markdown-hover"),r=X(s,n("div.hover-contents",{"aria-live":"assertive"})),a=i.add(new nd({editor:this._editor},this._languageService,this._openerService)),l=c=>{i.add(a.onDidRenderAsync(()=>{r.className="hover-contents code-hover-contents",e.onContentsChanged()}));const d=m("inlineSuggestionFollows","Suggestion:"),h=i.add(a.render(new Pn().appendText(d).appendCodeblock("text",c)));r.replaceChildren(h.element)};return i.add(Ae(c=>{var h;const d=(h=t.controller.model.read(c))==null?void 0:h.primaryGhostText.read(c);if(d){const u=this._editor.getModel().getLineContent(d.lineNumber);l(d.renderForScreenReader(u))}else _n(r)})),e.fragment.appendChild(s),i}};mF=SDe([pC(1,ni),pC(2,zs),pC(3,po),pC(4,we),pC(5,Bn)],mF);class LDe{}const zE=new class{constructor(){this._implementations=[]}register(e){return this._implementations.push(e),{dispose:()=>{const t=this._implementations.indexOf(e);t!==-1&&this._implementations.splice(t,1)}}}getImplementations(){return this._implementations}};ct(Sr.ID,Sr,3);Q(bDe);Q(hF);Q(uF);Q(CDe);Q(vDe);Q(wDe);Q(gF);Ot(fF);um.register(mF);zE.register(new LDe);var xDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},QT=function(o,e){return function(t,i){e(t,i,o)}},OC,kh;let m1=(kh=class{constructor(e,t,i,n){this.textModelResolverService=t,this.languageService=i,this.languageFeaturesService=n,this.toUnhook=new K,this.toUnhookForKeyboard=new K,this.currentWordAtPosition=null,this.previousPromise=null,this.editor=e,this.linkDecorations=this.editor.createDecorationsCollection();const s=new AE(e);this.toUnhook.add(s),this.toUnhook.add(s.onMouseMoveOrRelevantKeyDown(([r,a])=>{this.startFindDefinitionFromMouse(r,a??void 0)})),this.toUnhook.add(s.onExecute(r=>{this.isEnabled(r)&&this.gotoDefinition(r.target.position,r.hasSideBySideModifier).catch(a=>{Oe(a)}).finally(()=>{this.removeLinkDecorations()})})),this.toUnhook.add(s.onCancel(()=>{this.removeLinkDecorations(),this.currentWordAtPosition=null}))}static get(e){return e.getContribution(OC.ID)}async startFindDefinitionFromCursor(e){await this.startFindDefinition(e),this.toUnhookForKeyboard.add(this.editor.onDidChangeCursorPosition(()=>{this.currentWordAtPosition=null,this.removeLinkDecorations(),this.toUnhookForKeyboard.clear()})),this.toUnhookForKeyboard.add(this.editor.onKeyDown(t=>{t&&(this.currentWordAtPosition=null,this.removeLinkDecorations(),this.toUnhookForKeyboard.clear())}))}startFindDefinitionFromMouse(e,t){if(e.target.type===9&&this.linkDecorations.length>0)return;if(!this.editor.hasModel()||!this.isEnabled(e,t)){this.currentWordAtPosition=null,this.removeLinkDecorations();return}const i=e.target.position;this.startFindDefinition(i)}async startFindDefinition(e){var r;this.toUnhookForKeyboard.clear();const t=e?(r=this.editor.getModel())==null?void 0:r.getWordAtPosition(e):null;if(!t){this.currentWordAtPosition=null,this.removeLinkDecorations();return}if(this.currentWordAtPosition&&this.currentWordAtPosition.startColumn===t.startColumn&&this.currentWordAtPosition.endColumn===t.endColumn&&this.currentWordAtPosition.word===t.word)return;this.currentWordAtPosition=t;const i=new Yq(this.editor,15);this.previousPromise&&(this.previousPromise.cancel(),this.previousPromise=null),this.previousPromise=Yi(a=>this.findDefinition(e,a));let n;try{n=await this.previousPromise}catch(a){Oe(a);return}if(!n||!n.length||!i.validate(this.editor)){this.removeLinkDecorations();return}const s=n[0].originSelectionRange?x.lift(n[0].originSelectionRange):new x(e.lineNumber,t.startColumn,e.lineNumber,t.endColumn);if(n.length>1){let a=s;for(const{originSelectionRange:l}of n)l&&(a=x.plusRange(a,l));this.addDecoration(a,new Pn().appendText(m("multipleResults","Click to show {0} definitions.",n.length)))}else{const a=n[0];if(!a.uri)return;this.textModelResolverService.createModelReference(a.uri).then(l=>{if(!l.object||!l.object.textEditorModel){l.dispose();return}const{object:{textEditorModel:c}}=l,{startLineNumber:d}=a.range;if(d<1||d>c.getLineCount()){l.dispose();return}const h=this.getPreviewValue(c,d,a),u=this.languageService.guessLanguageIdByFilepathOrFirstLine(c.uri);this.addDecoration(s,h?new Pn().appendCodeblock(u||"",h):void 0),l.dispose()})}}getPreviewValue(e,t,i){let n=i.range;return n.endLineNumber-n.startLineNumber>=OC.MAX_SOURCE_PREVIEW_LINES&&(n=this.getPreviewRangeBasedOnIndentation(e,t)),this.stripIndentationFromPreviewRange(e,t,n)}stripIndentationFromPreviewRange(e,t,i){let s=e.getLineFirstNonWhitespaceColumn(t);for(let a=t+1;a<i.endLineNumber;a++){const l=e.getLineFirstNonWhitespaceColumn(a);s=Math.min(s,l)}return e.getValueInRange(i).replace(new RegExp(`^\\s{${s-1}}`,"gm"),"").trim()}getPreviewRangeBasedOnIndentation(e,t){const i=e.getLineFirstNonWhitespaceColumn(t),n=Math.min(e.getLineCount(),t+OC.MAX_SOURCE_PREVIEW_LINES);let s=t+1;for(;s<n;s++){const r=e.getLineFirstNonWhitespaceColumn(s);if(i===r)break}return new x(t,1,s+1,1)}addDecoration(e,t){const i={range:e,options:{description:"goto-definition-link",inlineClassName:"goto-definition-link",hoverMessage:t}};this.linkDecorations.set([i])}removeLinkDecorations(){this.linkDecorations.clear()}isEnabled(e,t){var i;return this.editor.hasModel()&&e.isLeftClick&&e.isNoneOrSingleMouseDown&&e.target.type===6&&!(((i=e.target.detail.injectedText)==null?void 0:i.options)instanceof Kh)&&(e.hasTriggerModifier||(t?t.keyCodeIsTriggerKey:!1))&&this.languageFeaturesService.definitionProvider.has(this.editor.getModel())}findDefinition(e,t){const i=this.editor.getModel();return i?gS(this.languageFeaturesService.definitionProvider,i,e,!1,t):Promise.resolve(null)}gotoDefinition(e,t){return this.editor.setPosition(e),this.editor.invokeWithinContext(i=>{const n=!t&&this.editor.getOption(89)&&!this.isInPeekEditor(i);return new mS({openToSide:t,openInPeek:n,muteMessage:!0},{title:{value:"",original:""},id:"",precondition:void 0}).run(i)})}isInPeekEditor(e){const t=e.get(Ee);return Ws.inPeekEditor.getValue(t)}dispose(){this.toUnhook.dispose(),this.toUnhookForKeyboard.dispose()}},OC=kh,kh.ID="editor.contrib.gotodefinitionatposition",kh.MAX_SOURCE_PREVIEW_LINES=8,kh);m1=OC=xDe([QT(1,Hs),QT(2,ni),QT(3,ge)],m1);ct(m1.ID,m1,2);var _Y=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},rk=function(o,e){return function(t,i){e(t,i,o)}};class P6{constructor(e,t,i){this.marker=e,this.index=t,this.total=i}}let pF=class{constructor(e,t,i){this._markerService=t,this._configService=i,this._onDidChange=new P,this.onDidChange=this._onDidChange.event,this._dispoables=new K,this._markers=[],this._nextIdx=-1,Se.isUri(e)?this._resourceFilter=a=>a.toString()===e.toString():e&&(this._resourceFilter=e);const n=this._configService.getValue("problems.sortOrder"),s=(a,l)=>{let c=sw(a.resource.toString(),l.resource.toString());return c===0&&(n==="position"?c=x.compareRangesUsingStarts(a,l)||pi.compare(a.severity,l.severity):c=pi.compare(a.severity,l.severity)||x.compareRangesUsingStarts(a,l)),c},r=()=>{this._markers=this._markerService.read({resource:Se.isUri(e)?e:void 0,severities:pi.Error|pi.Warning|pi.Info}),typeof e=="function"&&(this._markers=this._markers.filter(a=>this._resourceFilter(a.resource))),this._markers.sort(s)};r(),this._dispoables.add(t.onMarkerChanged(a=>{(!this._resourceFilter||a.some(l=>this._resourceFilter(l)))&&(r(),this._nextIdx=-1,this._onDidChange.fire())}))}dispose(){this._dispoables.dispose(),this._onDidChange.dispose()}matches(e){return!this._resourceFilter&&!e?!0:!this._resourceFilter||!e?!1:this._resourceFilter(e)}get selected(){const e=this._markers[this._nextIdx];return e&&new P6(e,this._nextIdx+1,this._markers.length)}_initIdx(e,t,i){let n=!1,s=this._markers.findIndex(r=>r.resource.toString()===e.uri.toString());s<0&&(s=Zv(this._markers,{resource:e.uri},(r,a)=>sw(r.resource.toString(),a.resource.toString())),s<0&&(s=~s));for(let r=s;r<this._markers.length;r++){let a=x.lift(this._markers[r]);if(a.isEmpty()){const l=e.getWordAtPosition(a.getStartPosition());l&&(a=new x(a.startLineNumber,l.startColumn,a.startLineNumber,l.endColumn))}if(t&&(a.containsPosition(t)||t.isBeforeOrEqual(a.getStartPosition()))){this._nextIdx=r,n=!0;break}if(this._markers[r].resource.toString()!==e.uri.toString())break}n||(this._nextIdx=i?0:this._markers.length-1),this._nextIdx<0&&(this._nextIdx=this._markers.length-1)}resetIndex(){this._nextIdx=-1}move(e,t,i){if(this._markers.length===0)return!1;const n=this._nextIdx;return this._nextIdx===-1?this._initIdx(t,i,e):e?this._nextIdx=(this._nextIdx+1)%this._markers.length:e||(this._nextIdx=(this._nextIdx-1+this._markers.length)%this._markers.length),n!==this._nextIdx}find(e,t){let i=this._markers.findIndex(n=>n.resource.toString()===e.toString());if(!(i<0)){for(;i<this._markers.length;i++)if(x.containsPosition(this._markers[i],t))return new P6(this._markers[i],i+1,this._markers.length)}}};pF=_Y([rk(1,Xl),rk(2,qe)],pF);const bY=et("IMarkerNavigationService");let _F=class{constructor(e,t){this._markerService=e,this._configService=t,this._provider=new cs}getMarkerList(e){for(const t of this._provider){const i=t.getMarkerList(e);if(i)return i}return new pF(e,this._markerService,this._configService)}};_F=_Y([rk(0,Xl),rk(1,qe)],_F);st(bY,_F,1);var bF;(function(o){function e(t){switch(t){case Ii.Ignore:return"severity-ignore "+Ie.asClassName(te.info);case Ii.Info:return Ie.asClassName(te.info);case Ii.Warning:return Ie.asClassName(te.warning);case Ii.Error:return Ie.asClassName(te.error);default:return""}}o.className=e})(bF||(bF={}));var kDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Hm=function(o,e){return function(t,i){e(t,i,o)}},CF;class DDe{constructor(e,t,i,n,s){this._openerService=n,this._labelService=s,this._lines=0,this._longestLineLength=0,this._relatedDiagnostics=new WeakMap,this._disposables=new K,this._editor=t;const r=document.createElement("div");r.className="descriptioncontainer",this._messageBlock=document.createElement("div"),this._messageBlock.classList.add("message"),this._messageBlock.setAttribute("aria-live","assertive"),this._messageBlock.setAttribute("role","alert"),r.appendChild(this._messageBlock),this._relatedBlock=document.createElement("div"),r.appendChild(this._relatedBlock),this._disposables.add(hi(this._relatedBlock,"click",a=>{a.preventDefault();const l=this._relatedDiagnostics.get(a.target);l&&i(l)})),this._scrollable=new Z$(r,{horizontal:1,vertical:1,useShadows:!1,horizontalScrollbarSize:6,verticalScrollbarSize:6}),e.appendChild(this._scrollable.getDomNode()),this._disposables.add(this._scrollable.onScroll(a=>{r.style.left=`-${a.scrollLeft}px`,r.style.top=`-${a.scrollTop}px`})),this._disposables.add(this._scrollable)}dispose(){Tt(this._disposables)}update(e){const{source:t,message:i,relatedInformation:n,code:s}=e;let r=((t==null?void 0:t.length)||0)+2;s&&(typeof s=="string"?r+=s.length:r+=s.value.length);const a=Kl(i);this._lines=a.length,this._longestLineLength=0;for(const u of a)this._longestLineLength=Math.max(u.length+r,this._longestLineLength);pn(this._messageBlock),this._messageBlock.setAttribute("aria-label",this.getAriaLabel(e)),this._editor.applyFontInfo(this._messageBlock);let l=this._messageBlock;for(const u of a)l=document.createElement("div"),l.innerText=u,u===""&&(l.style.height=this._messageBlock.style.lineHeight),this._messageBlock.appendChild(l);if(t||s){const u=document.createElement("span");if(u.classList.add("details"),l.appendChild(u),t){const g=document.createElement("span");g.innerText=t,g.classList.add("source"),u.appendChild(g)}if(s)if(typeof s=="string"){const g=document.createElement("span");g.innerText=`(${s})`,g.classList.add("code"),u.appendChild(g)}else{this._codeLink=ne("a.code-link"),this._codeLink.setAttribute("href",`${s.target.toString()}`),this._codeLink.onclick=f=>{this._openerService.open(s.target,{allowCommands:!0}),f.preventDefault(),f.stopPropagation()};const g=X(this._codeLink,ne("span"));g.innerText=s.value,u.appendChild(this._codeLink)}}if(pn(this._relatedBlock),this._editor.applyFontInfo(this._relatedBlock),Qn(n)){const u=this._relatedBlock.appendChild(document.createElement("div"));u.style.paddingTop=`${Math.floor(this._editor.getOption(67)*.66)}px`,this._lines+=1;for(const g of n){const f=document.createElement("div"),p=document.createElement("a");p.classList.add("filename"),p.innerText=`${this._labelService.getUriBasenameLabel(g.resource)}(${g.startLineNumber}, ${g.startColumn}): `,p.title=this._labelService.getUriLabel(g.resource),this._relatedDiagnostics.set(p,g);const _=document.createElement("span");_.innerText=g.message,f.appendChild(p),f.appendChild(_),this._lines+=1,u.appendChild(f)}}const c=this._editor.getOption(50),d=Math.ceil(c.typicalFullwidthCharacterWidth*this._longestLineLength*.75),h=c.lineHeight*this._lines;this._scrollable.setScrollDimensions({scrollWidth:d,scrollHeight:h})}layout(e,t){this._scrollable.getDomNode().style.height=`${e}px`,this._scrollable.getDomNode().style.width=`${t}px`,this._scrollable.setScrollDimensions({width:t,height:e})}getHeightInLines(){return Math.min(17,this._lines)}getAriaLabel(e){let t="";switch(e.severity){case pi.Error:t=m("Error","Error");break;case pi.Warning:t=m("Warning","Warning");break;case pi.Info:t=m("Info","Info");break;case pi.Hint:t=m("Hint","Hint");break}let i=m("marker aria","{0} at {1}. ",t,e.startLineNumber+":"+e.startColumn);const n=this._editor.getModel();return n&&e.startLineNumber<=n.getLineCount()&&e.startLineNumber>=1&&(i=`${n.getLineContent(e.startLineNumber)}, ${i}`),i}}var rf;let p1=(rf=class extends Fx{constructor(e,t,i,n,s,r,a){super(e,{showArrow:!0,showFrame:!0,isAccessible:!0,frameWidth:1},s),this._themeService=t,this._openerService=i,this._menuService=n,this._contextKeyService=r,this._labelService=a,this._callOnDispose=new K,this._onDidSelectRelatedInformation=new P,this.onDidSelectRelatedInformation=this._onDidSelectRelatedInformation.event,this._severity=pi.Warning,this._backgroundColor=Z.white,this._applyTheme(t.getColorTheme()),this._callOnDispose.add(t.onDidColorThemeChange(this._applyTheme.bind(this))),this.create()}_applyTheme(e){this._backgroundColor=e.getColor(TDe);let t=vF,i=IDe;this._severity===pi.Warning?(t=_y,i=EDe):this._severity===pi.Info&&(t=wF,i=NDe);const n=e.getColor(t),s=e.getColor(i);this.style({arrowColor:n,frameColor:n,headerBackgroundColor:s,primaryHeadingColor:e.getColor(OZ),secondaryHeadingColor:e.getColor(FZ)})}_applyStyles(){this._parentContainer&&(this._parentContainer.style.backgroundColor=this._backgroundColor?this._backgroundColor.toString():""),super._applyStyles()}dispose(){this._callOnDispose.dispose(),super.dispose()}_fillHead(e){super._fillHead(e),this._disposables.add(this._actionbarWidget.actionRunner.onWillRun(n=>this.editor.focus()));const t=[],i=this._menuService.getMenuActions(CF.TitleMenu,this._contextKeyService);sE(i,t),this._actionbarWidget.push(t,{label:!1,icon:!0,index:0})}_fillTitleIcon(e){this._icon=X(e,ne(""))}_fillBody(e){this._parentContainer=e,e.classList.add("marker-widget"),this._parentContainer.tabIndex=0,this._parentContainer.setAttribute("role","tooltip"),this._container=document.createElement("div"),e.appendChild(this._container),this._message=new DDe(this._container,this.editor,t=>this._onDidSelectRelatedInformation.fire(t),this._openerService,this._labelService),this._disposables.add(this._message)}show(){throw new Error("call showAtMarker")}showAtMarker(e,t,i){this._container.classList.remove("stale"),this._message.update(e),this._severity=e.severity,this._applyTheme(this._themeService.getColorTheme());const n=x.lift(e),s=this.editor.getPosition(),r=s&&n.containsPosition(s)?s:n.getStartPosition();super.show(r,this.computeRequiredHeight());const a=this.editor.getModel();if(a){const l=i>1?m("problems","{0} of {1} problems",t,i):m("change","{0} of {1} problem",t,i);this.setTitle(vr(a.uri),l)}this._icon.className=`codicon ${bF.className(pi.toSeverity(this._severity))}`,this.editor.revealPositionNearTop(r,0),this.editor.focus()}updateMarker(e){this._container.classList.remove("stale"),this._message.update(e)}showStale(){this._container.classList.add("stale"),this._relayout()}_doLayoutBody(e,t){super._doLayoutBody(e,t),this._heightInPixel=e,this._message.layout(e,t),this._container.style.height=`${e}px`}_onWidth(e){this._message.layout(this._heightInPixel,e)}_relayout(){super._relayout(this.computeRequiredHeight())}computeRequiredHeight(){return 3+this._message.getHeightInLines()}},CF=rf,rf.TitleMenu=new me("gotoErrorTitleMenu"),rf);p1=CF=kDe([Hm(1,ji),Hm(2,zs),Hm(3,Go),Hm(4,we),Hm(5,Ee),Hm(6,W_)],p1);const O6=mw(HI,Poe),F6=mw(id,pw),W6=mw(Bl,_w),vF=E("editorMarkerNavigationError.background",{dark:O6,light:O6,hcDark:lt,hcLight:lt},m("editorMarkerNavigationError","Editor marker navigation widget error color.")),IDe=E("editorMarkerNavigationError.headerBackground",{dark:$e(vF,.1),light:$e(vF,.1),hcDark:null,hcLight:null},m("editorMarkerNavigationErrorHeaderBackground","Editor marker navigation widget error heading background.")),_y=E("editorMarkerNavigationWarning.background",{dark:F6,light:F6,hcDark:lt,hcLight:lt},m("editorMarkerNavigationWarning","Editor marker navigation widget warning color.")),EDe=E("editorMarkerNavigationWarning.headerBackground",{dark:$e(_y,.1),light:$e(_y,.1),hcDark:"#0C141F",hcLight:$e(_y,.2)},m("editorMarkerNavigationWarningBackground","Editor marker navigation widget warning heading background.")),wF=E("editorMarkerNavigationInfo.background",{dark:W6,light:W6,hcDark:lt,hcLight:lt},m("editorMarkerNavigationInfo","Editor marker navigation widget info color.")),NDe=E("editorMarkerNavigationInfo.headerBackground",{dark:$e(wF,.1),light:$e(wF,.1),hcDark:null,hcLight:null},m("editorMarkerNavigationInfoHeaderBackground","Editor marker navigation widget info heading background.")),TDe=E("editorMarkerNavigation.background",Xr,m("editorMarkerNavigationBackground","Editor marker navigation widget background."));var RDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},T0=function(o,e){return function(t,i){e(t,i,o)}},FC,af;let Zf=(af=class{static get(e){return e.getContribution(FC.ID)}constructor(e,t,i,n,s){this._markerNavigationService=t,this._contextKeyService=i,this._editorService=n,this._instantiationService=s,this._sessionDispoables=new K,this._editor=e,this._widgetVisible=CY.bindTo(this._contextKeyService)}dispose(){this._cleanUp(),this._sessionDispoables.dispose()}_cleanUp(){this._widgetVisible.reset(),this._sessionDispoables.clear(),this._widget=void 0,this._model=void 0}_getOrCreateModel(e){if(this._model&&this._model.matches(e))return this._model;let t=!1;return this._model&&(t=!0,this._cleanUp()),this._model=this._markerNavigationService.getMarkerList(e),t&&this._model.move(!0,this._editor.getModel(),this._editor.getPosition()),this._widget=this._instantiationService.createInstance(p1,this._editor),this._widget.onDidClose(()=>this.close(),this,this._sessionDispoables),this._widgetVisible.set(!0),this._sessionDispoables.add(this._model),this._sessionDispoables.add(this._widget),this._sessionDispoables.add(this._editor.onDidChangeCursorPosition(i=>{var n,s,r;(!((n=this._model)!=null&&n.selected)||!x.containsPosition((s=this._model)==null?void 0:s.selected.marker,i.position))&&((r=this._model)==null||r.resetIndex())})),this._sessionDispoables.add(this._model.onDidChange(()=>{if(!this._widget||!this._widget.position||!this._model)return;const i=this._model.find(this._editor.getModel().uri,this._widget.position);i?this._widget.updateMarker(i.marker):this._widget.showStale()})),this._sessionDispoables.add(this._widget.onDidSelectRelatedInformation(i=>{this._editorService.openCodeEditor({resource:i.resource,options:{pinned:!0,revealIfOpened:!0,selection:x.lift(i).collapseToStart()}},this._editor),this.close(!1)})),this._sessionDispoables.add(this._editor.onDidChangeModel(()=>this._cleanUp())),this._model}close(e=!0){this._cleanUp(),e&&this._editor.focus()}showAtMarker(e){if(this._editor.hasModel()){const t=this._getOrCreateModel(this._editor.getModel().uri);t.resetIndex(),t.move(!0,this._editor.getModel(),new A(e.startLineNumber,e.startColumn)),t.selected&&this._widget.showAtMarker(t.selected.marker,t.selected.index,t.selected.total)}}async nagivate(e,t){var i,n;if(this._editor.hasModel()){const s=this._getOrCreateModel(t?void 0:this._editor.getModel().uri);if(s.move(e,this._editor.getModel(),this._editor.getPosition()),!s.selected)return;if(s.selected.marker.resource.toString()!==this._editor.getModel().uri.toString()){this._cleanUp();const r=await this._editorService.openCodeEditor({resource:s.selected.marker.resource,options:{pinned:!1,revealIfOpened:!0,selectionRevealType:2,selection:s.selected.marker}},this._editor);r&&((i=FC.get(r))==null||i.close(),(n=FC.get(r))==null||n.nagivate(e,t))}else this._widget.showAtMarker(s.selected.marker,s.selected.index,s.selected.total)}}},FC=af,af.ID="editor.contrib.markerController",af);Zf=FC=RDe([T0(1,bY),T0(2,Ee),T0(3,pt),T0(4,we)],Zf);class UE extends de{constructor(e,t,i){super(i),this._next=e,this._multiFile=t}async run(e,t){var i;t.hasModel()&&((i=Zf.get(t))==null||i.nagivate(this._next,this._multiFile))}}const sg=class sg extends UE{constructor(){super(!0,!1,{id:sg.ID,label:sg.LABEL,alias:"Go to Next Problem (Error, Warning, Info)",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:578,weight:100},menuOpts:{menuId:p1.TitleMenu,title:sg.LABEL,icon:qt("marker-navigation-next",te.arrowDown,m("nextMarkerIcon","Icon for goto next marker.")),group:"navigation",order:1}})}};sg.ID="editor.action.marker.next",sg.LABEL=m("markerAction.next.label","Go to Next Problem (Error, Warning, Info)");let ak=sg;const og=class og extends UE{constructor(){super(!1,!1,{id:og.ID,label:og.LABEL,alias:"Go to Previous Problem (Error, Warning, Info)",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:1602,weight:100},menuOpts:{menuId:p1.TitleMenu,title:og.LABEL,icon:qt("marker-navigation-previous",te.arrowUp,m("previousMarkerIcon","Icon for goto previous marker.")),group:"navigation",order:2}})}};og.ID="editor.action.marker.prev",og.LABEL=m("markerAction.previous.label","Go to Previous Problem (Error, Warning, Info)");let SF=og;class MDe extends UE{constructor(){super(!0,!0,{id:"editor.action.marker.nextInFiles",label:m("markerAction.nextInFiles.label","Go to Next Problem in Files (Error, Warning, Info)"),alias:"Go to Next Problem in Files (Error, Warning, Info)",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:66,weight:100},menuOpts:{menuId:me.MenubarGoMenu,title:m({key:"miGotoNextProblem",comment:["&& denotes a mnemonic"]},"Next &&Problem"),group:"6_problem_nav",order:1}})}}class ADe extends UE{constructor(){super(!1,!0,{id:"editor.action.marker.prevInFiles",label:m("markerAction.previousInFiles.label","Go to Previous Problem in Files (Error, Warning, Info)"),alias:"Go to Previous Problem in Files (Error, Warning, Info)",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:1090,weight:100},menuOpts:{menuId:me.MenubarGoMenu,title:m({key:"miGotoPreviousProblem",comment:["&& denotes a mnemonic"]},"Previous &&Problem"),group:"6_problem_nav",order:2}})}}ct(Zf.ID,Zf,4);Q(ak);Q(SF);Q(MDe);Q(ADe);const CY=new le("markersNavigationVisible",!1),PDe=$i.bindToContribution(Zf.get);re(new PDe({id:"closeMarkersNavigation",precondition:CY,handler:o=>o.close(),kbOpts:{weight:150,kbExpr:I.focus,primary:9,secondary:[1033]}}));var ga;(function(o){o.NoAutoFocus="noAutoFocus",o.FocusIfVisible="focusIfVisible",o.AutoFocusImmediately="autoFocusImmediately"})(ga||(ga={}));class ODe extends de{constructor(){super({id:yZ,label:m({key:"showOrFocusHover",comment:["Label for action that will trigger the showing/focusing of a hover in the editor.","If the hover is not visible, it will show the hover.","This allows for users to show the hover without using the mouse."]},"Show or Focus Hover"),metadata:{description:Te("showOrFocusHoverDescription","Show or focus the editor hover which shows documentation, references, and other content for a symbol at the current cursor position."),args:[{name:"args",schema:{type:"object",properties:{focus:{description:"Controls if and when the hover should take focus upon being triggered by this action.",enum:[ga.NoAutoFocus,ga.FocusIfVisible,ga.AutoFocusImmediately],enumDescriptions:[m("showOrFocusHover.focus.noAutoFocus","The hover will not automatically take focus."),m("showOrFocusHover.focus.focusIfVisible","The hover will take focus only if it is already visible."),m("showOrFocusHover.focus.autoFocusImmediately","The hover will automatically take focus when it appears.")],default:ga.FocusIfVisible}}}}]},alias:"Show or Focus Hover",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2087),weight:100}})}run(e,t,i){if(!t.hasModel())return;const n=es.get(t);if(!n)return;const s=i==null?void 0:i.focus;let r=ga.FocusIfVisible;Object.values(ga).includes(s)?r=s:typeof s=="boolean"&&s&&(r=ga.AutoFocusImmediately);const a=c=>{const d=t.getPosition(),h=new x(d.lineNumber,d.column,d.lineNumber,d.column);n.showContentHover(h,1,1,c)},l=t.getOption(2)===2;n.isHoverVisible?r!==ga.NoAutoFocus?n.focus():a(l):a(l||r===ga.AutoFocusImmediately)}}class FDe extends de{constructor(){super({id:eye,label:m({key:"showDefinitionPreviewHover",comment:["Label for action that will trigger the showing of definition preview hover in the editor.","This allows for users to show the definition preview hover without using the mouse."]},"Show Definition Preview Hover"),alias:"Show Definition Preview Hover",precondition:void 0,metadata:{description:Te("showDefinitionPreviewHoverDescription","Show the definition preview hover in the editor.")}})}run(e,t){const i=es.get(t);if(!i)return;const n=t.getPosition();if(!n)return;const s=new x(n.lineNumber,n.column,n.lineNumber,n.column),r=m1.get(t);if(!r)return;r.startFindDefinitionFromCursor(n).then(()=>{i.showContentHover(s,1,1,!0)})}}class WDe extends de{constructor(){super({id:tye,label:m({key:"scrollUpHover",comment:["Action that allows to scroll up in the hover widget with the up arrow when the hover widget is focused."]},"Scroll Up Hover"),alias:"Scroll Up Hover",precondition:I.hoverFocused,kbOpts:{kbExpr:I.hoverFocused,primary:16,weight:100},metadata:{description:Te("scrollUpHoverDescription","Scroll up the editor hover.")}})}run(e,t){const i=es.get(t);i&&i.scrollUp()}}class BDe extends de{constructor(){super({id:iye,label:m({key:"scrollDownHover",comment:["Action that allows to scroll down in the hover widget with the up arrow when the hover widget is focused."]},"Scroll Down Hover"),alias:"Scroll Down Hover",precondition:I.hoverFocused,kbOpts:{kbExpr:I.hoverFocused,primary:18,weight:100},metadata:{description:Te("scrollDownHoverDescription","Scroll down the editor hover.")}})}run(e,t){const i=es.get(t);i&&i.scrollDown()}}class HDe extends de{constructor(){super({id:nye,label:m({key:"scrollLeftHover",comment:["Action that allows to scroll left in the hover widget with the left arrow when the hover widget is focused."]},"Scroll Left Hover"),alias:"Scroll Left Hover",precondition:I.hoverFocused,kbOpts:{kbExpr:I.hoverFocused,primary:15,weight:100},metadata:{description:Te("scrollLeftHoverDescription","Scroll left the editor hover.")}})}run(e,t){const i=es.get(t);i&&i.scrollLeft()}}class VDe extends de{constructor(){super({id:sye,label:m({key:"scrollRightHover",comment:["Action that allows to scroll right in the hover widget with the right arrow when the hover widget is focused."]},"Scroll Right Hover"),alias:"Scroll Right Hover",precondition:I.hoverFocused,kbOpts:{kbExpr:I.hoverFocused,primary:17,weight:100},metadata:{description:Te("scrollRightHoverDescription","Scroll right the editor hover.")}})}run(e,t){const i=es.get(t);i&&i.scrollRight()}}class zDe extends de{constructor(){super({id:oye,label:m({key:"pageUpHover",comment:["Action that allows to page up in the hover widget with the page up command when the hover widget is focused."]},"Page Up Hover"),alias:"Page Up Hover",precondition:I.hoverFocused,kbOpts:{kbExpr:I.hoverFocused,primary:11,secondary:[528],weight:100},metadata:{description:Te("pageUpHoverDescription","Page up the editor hover.")}})}run(e,t){const i=es.get(t);i&&i.pageUp()}}class UDe extends de{constructor(){super({id:rye,label:m({key:"pageDownHover",comment:["Action that allows to page down in the hover widget with the page down command when the hover widget is focused."]},"Page Down Hover"),alias:"Page Down Hover",precondition:I.hoverFocused,kbOpts:{kbExpr:I.hoverFocused,primary:12,secondary:[530],weight:100},metadata:{description:Te("pageDownHoverDescription","Page down the editor hover.")}})}run(e,t){const i=es.get(t);i&&i.pageDown()}}class $De extends de{constructor(){super({id:aye,label:m({key:"goToTopHover",comment:["Action that allows to go to the top of the hover widget with the home command when the hover widget is focused."]},"Go To Top Hover"),alias:"Go To Bottom Hover",precondition:I.hoverFocused,kbOpts:{kbExpr:I.hoverFocused,primary:14,secondary:[2064],weight:100},metadata:{description:Te("goToTopHoverDescription","Go to the top of the editor hover.")}})}run(e,t){const i=es.get(t);i&&i.goToTop()}}class jDe extends de{constructor(){super({id:lye,label:m({key:"goToBottomHover",comment:["Action that allows to go to the bottom in the hover widget with the end command when the hover widget is focused."]},"Go To Bottom Hover"),alias:"Go To Bottom Hover",precondition:I.hoverFocused,kbOpts:{kbExpr:I.hoverFocused,primary:13,secondary:[2066],weight:100},metadata:{description:Te("goToBottomHoverDescription","Go to the bottom of the editor hover.")}})}run(e,t){const i=es.get(t);i&&i.goToBottom()}}class KDe extends de{constructor(){super({id:TE,label:cye,alias:"Increase Hover Verbosity Level",precondition:I.hoverVisible})}run(e,t,i){const n=es.get(t);if(!n)return;const s=(i==null?void 0:i.index)!==void 0?i.index:n.focusedHoverPartIndex();n.updateHoverVerbosityLevel(eo.Increase,s,i==null?void 0:i.focus)}}class qDe extends de{constructor(){super({id:RE,label:dye,alias:"Decrease Hover Verbosity Level",precondition:I.hoverVisible})}run(e,t,i){var r;const n=es.get(t);if(!n)return;const s=(i==null?void 0:i.index)!==void 0?i.index:n.focusedHoverPartIndex();(r=es.get(t))==null||r.updateHoverVerbosityLevel(eo.Decrease,s,i==null?void 0:i.focus)}}var GDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},JT=function(o,e){return function(t,i){e(t,i,o)}};const Er=ne;class ZDe{constructor(e,t,i){this.owner=e,this.range=t,this.marker=i}isValidForHoverAnchor(e){return e.type===1&&this.range.startColumn<=e.range.startColumn&&this.range.endColumn>=e.range.endColumn}}const B6={type:1,filter:{include:Yt.QuickFix},triggerAction:fo.QuickFixHover};let yF=class{constructor(e,t,i,n){this._editor=e,this._markerDecorationsService=t,this._openerService=i,this._languageFeaturesService=n,this.hoverOrdinal=1,this.recentMarkerCodeActionsInfo=void 0}computeSync(e,t){if(!this._editor.hasModel()||e.type!==1&&!e.supportsMarkerHover)return[];const i=this._editor.getModel(),n=e.range.startLineNumber,s=i.getLineMaxColumn(n),r=[];for(const a of t){const l=a.range.startLineNumber===n?a.range.startColumn:1,c=a.range.endLineNumber===n?a.range.endColumn:s,d=this._markerDecorationsService.getMarker(i.uri,a);if(!d)continue;const h=new x(e.range.startLineNumber,l,e.range.startLineNumber,c);r.push(new ZDe(this,h,d))}return r}renderHoverParts(e,t){if(!t.length)return new Kf([]);const i=new K,n=[];t.forEach(r=>{const a=this._renderMarkerHover(r);e.fragment.appendChild(a.hoverElement),n.push(a)});const s=t.length===1?t[0]:t.sort((r,a)=>pi.compare(r.marker.severity,a.marker.severity))[0];return this.renderMarkerStatusbar(e,s,i),new Kf(n)}_renderMarkerHover(e){const t=new K,i=Er("div.hover-row"),n=X(i,Er("div.marker.hover-contents")),{source:s,message:r,code:a,relatedInformation:l}=e.marker;this._editor.applyFontInfo(n);const c=X(n,Er("span"));if(c.style.whiteSpace="pre-wrap",c.innerText=r,s||a)if(a&&typeof a!="string"){const h=Er("span");if(s){const p=X(h,Er("span"));p.innerText=s}const u=X(h,Er("a.code-link"));u.setAttribute("href",a.target.toString()),t.add(U(u,"click",p=>{this._openerService.open(a.target,{allowCommands:!0}),p.preventDefault(),p.stopPropagation()}));const g=X(u,Er("span"));g.innerText=a.value;const f=X(n,h);f.style.opacity="0.6",f.style.paddingLeft="6px"}else{const h=X(n,Er("span"));h.style.opacity="0.6",h.style.paddingLeft="6px",h.innerText=s&&a?`${s}(${a})`:s||`(${a})`}if(Qn(l))for(const{message:h,resource:u,startLineNumber:g,startColumn:f}of l){const p=X(n,Er("div"));p.style.marginTop="8px";const _=X(p,Er("a"));_.innerText=`${vr(u)}(${g}, ${f}): `,_.style.cursor="pointer",t.add(U(_,"click",C=>{if(C.stopPropagation(),C.preventDefault(),this._openerService){const v={selection:{startLineNumber:g,startColumn:f}};this._openerService.open(u,{fromUserGesture:!0,editorOptions:v}).catch(Oe)}}));const b=X(p,Er("span"));b.innerText=h,this._editor.applyFontInfo(b)}return{hoverPart:e,hoverElement:i,dispose:()=>t.dispose()}}renderMarkerStatusbar(e,t,i){if(t.marker.severity===pi.Error||t.marker.severity===pi.Warning||t.marker.severity===pi.Info){const n=Zf.get(this._editor);n&&e.statusBar.addAction({label:m("view problem","View Problem"),commandId:ak.ID,run:()=>{e.hide(),n.showAtMarker(t.marker),this._editor.focus()}})}if(!this._editor.getOption(92)){const n=e.statusBar.append(Er("div"));this.recentMarkerCodeActionsInfo&&(EL.makeKey(this.recentMarkerCodeActionsInfo.marker)===EL.makeKey(t.marker)?this.recentMarkerCodeActionsInfo.hasCodeActions||(n.textContent=m("noQuickFixes","No quick fixes available")):this.recentMarkerCodeActionsInfo=void 0);const s=this.recentMarkerCodeActionsInfo&&!this.recentMarkerCodeActionsInfo.hasCodeActions?F.None:Bh(()=>n.textContent=m("checkingForQuickFixes","Checking for quick fixes..."),200,i);n.textContent||(n.textContent=" ");const r=this.getCodeActions(t.marker);i.add(ve(()=>r.cancel())),r.then(a=>{if(s.dispose(),this.recentMarkerCodeActionsInfo={marker:t.marker,hasCodeActions:a.validActions.length>0},!this.recentMarkerCodeActionsInfo.hasCodeActions){a.dispose(),n.textContent=m("noQuickFixes","No quick fixes available");return}n.style.display="none";let l=!1;i.add(ve(()=>{l||a.dispose()})),e.statusBar.addAction({label:m("quick fixes","Quick Fix..."),commandId:AW,run:c=>{l=!0;const d=X_.get(this._editor),h=wi(c);e.hide(),d==null||d.showCodeActions(B6,a,{x:h.left,y:h.top,width:h.width,height:h.height})}})},Oe)}}getCodeActions(e){return Yi(t=>pp(this._languageFeaturesService.codeActionProvider,this._editor.getModel(),new x(e.startLineNumber,e.startColumn,e.endLineNumber,e.endColumn),B6,ka.None,t))}};yF=GDe([JT(1,r3),JT(2,zs),JT(3,ge)],yF);class YDe{get lineNumber(){return this._lineNumber}set lineNumber(e){this._lineNumber=e}get lane(){return this._laneOrLine}set lane(e){this._laneOrLine=e}constructor(e){this._editor=e,this._lineNumber=-1,this._laneOrLine=Zr.Center}computeSync(){var s;const e=r=>({value:r}),t=this._editor.getLineDecorations(this._lineNumber),i=[],n=this._laneOrLine==="lineNo";if(!t)return i;for(const r of t){const a=((s=r.options.glyphMargin)==null?void 0:s.position)??Zr.Center;if(!n&&a!==this._laneOrLine)continue;const l=n?r.options.lineNumberHoverMessage:r.options.glyphMarginHoverMessage;!l||O_(l)||i.push(...F4(l).map(e))}return i}}var XDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},H6=function(o,e){return function(t,i){e(t,i,o)}},LF;const V6=ne;var lf;let xF=(lf=class extends F{constructor(e,t,i){super(),this._renderDisposeables=this._register(new K),this._editor=e,this._isVisible=!1,this._messages=[],this._hover=this._register(new W5),this._hover.containerDomNode.classList.toggle("hidden",!this._isVisible),this._markdownRenderer=this._register(new nd({editor:this._editor},t,i)),this._computer=new YDe(this._editor),this._hoverOperation=this._register(new DZ(this._editor,this._computer)),this._register(this._hoverOperation.onResult(n=>{this._withResult(n.value)})),this._register(this._editor.onDidChangeModelDecorations(()=>this._onModelDecorationsChanged())),this._register(this._editor.onDidChangeConfiguration(n=>{n.hasChanged(50)&&this._updateFont()})),this._register(hi(this._hover.containerDomNode,"mouseleave",n=>{this._onMouseLeave(n)})),this._editor.addOverlayWidget(this)}dispose(){this._editor.removeOverlayWidget(this),super.dispose()}getId(){return LF.ID}getDomNode(){return this._hover.containerDomNode}getPosition(){return null}_updateFont(){Array.prototype.slice.call(this._hover.contentsDomNode.getElementsByClassName("code")).forEach(t=>this._editor.applyFontInfo(t))}_onModelDecorationsChanged(){this._isVisible&&(this._hoverOperation.cancel(),this._hoverOperation.start(0))}showsOrWillShow(e){const t=e.target;return t.type===2&&t.detail.glyphMarginLane?(this._startShowingAt(t.position.lineNumber,t.detail.glyphMarginLane),!0):t.type===3?(this._startShowingAt(t.position.lineNumber,"lineNo"),!0):!1}_startShowingAt(e,t){this._computer.lineNumber===e&&this._computer.lane===t||(this._hoverOperation.cancel(),this.hide(),this._computer.lineNumber=e,this._computer.lane=t,this._hoverOperation.start(0))}hide(){this._computer.lineNumber=-1,this._hoverOperation.cancel(),this._isVisible&&(this._isVisible=!1,this._hover.containerDomNode.classList.toggle("hidden",!this._isVisible))}_withResult(e){this._messages=e,this._messages.length>0?this._renderMessages(this._computer.lineNumber,this._messages):this.hide()}_renderMessages(e,t){this._renderDisposeables.clear();const i=document.createDocumentFragment();for(const n of t){const s=V6("div.hover-row.markdown-hover"),r=X(s,V6("div.hover-contents")),a=this._renderDisposeables.add(this._markdownRenderer.render(n.value));r.appendChild(a.element),i.appendChild(s)}this._updateContents(i),this._showAt(e)}_updateContents(e){this._hover.contentsDomNode.textContent="",this._hover.contentsDomNode.appendChild(e),this._updateFont()}_showAt(e){this._isVisible||(this._isVisible=!0,this._hover.containerDomNode.classList.toggle("hidden",!this._isVisible));const t=this._editor.getLayoutInfo(),i=this._editor.getTopForLineNumber(e),n=this._editor.getScrollTop(),s=this._editor.getOption(67),r=this._hover.containerDomNode.clientHeight,a=i-n-(r-s)/2,l=t.glyphMarginLeft+t.glyphMarginWidth+(this._computer.lane==="lineNo"?t.lineNumbersWidth:0);this._hover.containerDomNode.style.left=`${l}px`,this._hover.containerDomNode.style.top=`${Math.max(Math.round(a),0)}px`}_onMouseLeave(e){const t=this._editor.getDomNode();(!t||!ME(t,e.x,e.y))&&this.hide()}},LF=lf,lf.ID="editor.contrib.modesGlyphHoverWidget",lf);xF=LF=XDe([H6(1,ni),H6(2,zs)],xF);var QDe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},JDe=function(o,e){return function(t,i){e(t,i,o)}},b_;let lk=(b_=class extends F{constructor(e,t){super(),this._editor=e,this._instantiationService=t,this.shouldKeepOpenOnEditorMouseMoveOrLeave=!1,this._listenersStore=new K,this._hoverState={mouseDown:!1},this._reactToEditorMouseMoveRunner=this._register(new It(()=>this._reactToEditorMouseMove(this._mouseMoveEvent),0)),this._hookListeners(),this._register(this._editor.onDidChangeConfiguration(i=>{i.hasChanged(60)&&(this._unhookListeners(),this._hookListeners())}))}_hookListeners(){const e=this._editor.getOption(60);this._hoverSettings={enabled:e.enabled,sticky:e.sticky,hidingDelay:e.hidingDelay},e.enabled?(this._listenersStore.add(this._editor.onMouseDown(t=>this._onEditorMouseDown(t))),this._listenersStore.add(this._editor.onMouseUp(()=>this._onEditorMouseUp())),this._listenersStore.add(this._editor.onMouseMove(t=>this._onEditorMouseMove(t))),this._listenersStore.add(this._editor.onKeyDown(t=>this._onKeyDown(t)))):(this._listenersStore.add(this._editor.onMouseMove(t=>this._onEditorMouseMove(t))),this._listenersStore.add(this._editor.onKeyDown(t=>this._onKeyDown(t)))),this._listenersStore.add(this._editor.onMouseLeave(t=>this._onEditorMouseLeave(t))),this._listenersStore.add(this._editor.onDidChangeModel(()=>{this._cancelScheduler(),this._hideWidgets()})),this._listenersStore.add(this._editor.onDidChangeModelContent(()=>this._cancelScheduler())),this._listenersStore.add(this._editor.onDidScrollChange(t=>this._onEditorScrollChanged(t)))}_unhookListeners(){this._listenersStore.clear()}_cancelScheduler(){this._mouseMoveEvent=void 0,this._reactToEditorMouseMoveRunner.cancel()}_onEditorScrollChanged(e){(e.scrollTopChanged||e.scrollLeftChanged)&&this._hideWidgets()}_onEditorMouseDown(e){this._hoverState.mouseDown=!0,!this._isMouseOnMarginHoverWidget(e)&&this._hideWidgets()}_isMouseOnMarginHoverWidget(e){var i;const t=(i=this._glyphWidget)==null?void 0:i.getDomNode();return t?ME(t,e.event.posx,e.event.posy):!1}_onEditorMouseUp(){this._hoverState.mouseDown=!1}_onEditorMouseLeave(e){this.shouldKeepOpenOnEditorMouseMoveOrLeave||(this._cancelScheduler(),this._isMouseOnMarginHoverWidget(e))||this._hideWidgets()}_shouldNotRecomputeCurrentHoverWidget(e){const t=this._hoverSettings.sticky,i=this._isMouseOnMarginHoverWidget(e);return t&&i}_onEditorMouseMove(e){if(this.shouldKeepOpenOnEditorMouseMoveOrLeave)return;if(this._mouseMoveEvent=e,this._shouldNotRecomputeCurrentHoverWidget(e)){this._reactToEditorMouseMoveRunner.cancel();return}this._reactToEditorMouseMove(e)}_reactToEditorMouseMove(e){!e||this._tryShowHoverWidget(e)||this._hideWidgets()}_tryShowHoverWidget(e){return this._getOrCreateGlyphWidget().showsOrWillShow(e)}_onKeyDown(e){this._editor.hasModel()&&(e.keyCode===5||e.keyCode===6||e.keyCode===57||e.keyCode===4||this._hideWidgets())}_hideWidgets(){var e;(e=this._glyphWidget)==null||e.hide()}_getOrCreateGlyphWidget(){return this._glyphWidget||(this._glyphWidget=this._instantiationService.createInstance(xF,this._editor)),this._glyphWidget}dispose(){var e;super.dispose(),this._unhookListeners(),this._listenersStore.dispose(),(e=this._glyphWidget)==null||e.dispose()}},b_.ID="editor.contrib.marginHover",b_);lk=QDe([JDe(1,we)],lk);class eIe{}class tIe{}class iIe{}ct(es.ID,es,2);ct(lk.ID,lk,2);Q(ODe);Q(FDe);Q(WDe);Q(BDe);Q(HDe);Q(VDe);Q(zDe);Q(UDe);Q($De);Q(jDe);Q(KDe);Q(qDe);um.register(d1);um.register(yF);Lr((o,e)=>{const t=o.getColor(O$);t&&(e.addRule(`.monaco-editor .monaco-hover .hover-row:not(:first-child):not(:empty) { border-top: 1px solid ${t.transparent(.5)}; }`),e.addRule(`.monaco-editor .monaco-hover hr { border-top: 1px solid ${t.transparent(.5)}; }`),e.addRule(`.monaco-editor .monaco-hover hr { border-bottom: 0px solid ${t.transparent(.5)}; }`))});zE.register(new eIe);zE.register(new tIe);zE.register(new iIe);function Ys(o,e){let t=0;for(let i=0;i<o.length;i++)o.charAt(i)==="	"?t+=e:t++;return t}function Lv(o,e,t){o=o<0?0:o;let i="";if(!t){const n=Math.floor(o/e);o=o%e;for(let s=0;s<n;s++)i+="	"}for(let n=0;n<o;n++)i+=" ";return i}function vY(o,e,t,i){if(o.getLineCount()===1&&o.getLineMaxColumn(1)===1)return[];const n=e.getLanguageConfiguration(o.getLanguageId()).indentRulesSupport;if(!n)return[];const s=new K3(o,n,e);for(i=Math.min(i,o.getLineCount());t<=i&&s.shouldIgnore(t);)t++;if(t>i-1)return[];const{tabSize:r,indentSize:a,insertSpaces:l}=o.getOptions(),c=(p,_)=>(_=_||1,wr.shiftIndent(p,p.length+_,r,a,l)),d=(p,_)=>(_=_||1,wr.unshiftIndent(p,p.length+_,r,a,l)),h=[],u=o.getLineContent(t);let g=Nt(u),f=g;s.shouldIncrease(t)?(f=c(f),g=c(g)):s.shouldIndentNextLine(t)&&(f=c(f)),t++;for(let p=t;p<=i;p++){if(nIe(o,p))continue;const _=o.getLineContent(p),b=Nt(_),C=f;s.shouldDecrease(p,C)&&(f=d(f),g=d(g)),b!==f&&h.push(ri.replaceMove(new pe(p,1,p,b.length+1),a3(f,a,l))),!s.shouldIgnore(p)&&(s.shouldIncrease(p,C)?(g=c(g),f=g):s.shouldIndentNextLine(p,C)?f=c(f):f=g)}return h}function nIe(o,e){return o.tokenization.isCheapToTokenize(e)?o.tokenization.getLineTokens(e).getStandardTokenType(0)===2:!1}var sIe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},oIe=function(o,e){return function(t,i){e(t,i,o)}};const FD=class FD extends de{constructor(){super({id:FD.ID,label:m("indentationToSpaces","Convert Indentation to Spaces"),alias:"Convert Indentation to Spaces",precondition:I.writable,metadata:{description:Te("indentationToSpacesDescription","Convert the tab indentation to spaces.")}})}run(e,t){const i=t.getModel();if(!i)return;const n=i.getOptions(),s=t.getSelection();if(!s)return;const r=new dIe(s,n.tabSize);t.pushUndoStop(),t.executeCommands(this.id,[r]),t.pushUndoStop(),i.updateOptions({insertSpaces:!0})}};FD.ID="editor.action.indentationToSpaces";let kF=FD;const WD=class WD extends de{constructor(){super({id:WD.ID,label:m("indentationToTabs","Convert Indentation to Tabs"),alias:"Convert Indentation to Tabs",precondition:I.writable,metadata:{description:Te("indentationToTabsDescription","Convert the spaces indentation to tabs.")}})}run(e,t){const i=t.getModel();if(!i)return;const n=i.getOptions(),s=t.getSelection();if(!s)return;const r=new hIe(s,n.tabSize);t.pushUndoStop(),t.executeCommands(this.id,[r]),t.pushUndoStop(),i.updateOptions({insertSpaces:!1})}};WD.ID="editor.action.indentationToTabs";let DF=WD;class rB extends de{constructor(e,t,i){super(i),this.insertSpaces=e,this.displaySizeOnly=t}run(e,t){const i=e.get(xr),n=e.get(Kt),s=t.getModel();if(!s)return;const r=n.getCreationOptions(s.getLanguageId(),s.uri,s.isForSimpleWidget),a=s.getOptions(),l=[1,2,3,4,5,6,7,8].map(d=>({id:d.toString(),label:d.toString(),description:d===r.tabSize&&d===a.tabSize?m("configuredTabSize","Configured Tab Size"):d===r.tabSize?m("defaultTabSize","Default Tab Size"):d===a.tabSize?m("currentTabSize","Current Tab Size"):void 0})),c=Math.min(s.getOptions().tabSize-1,7);setTimeout(()=>{i.pick(l,{placeHolder:m({key:"selectTabWidth",comment:["Tab corresponds to the tab key"]},"Select Tab Size for Current File"),activeItem:l[c]}).then(d=>{if(d&&s&&!s.isDisposed()){const h=parseInt(d.label,10);this.displaySizeOnly?s.updateOptions({tabSize:h}):s.updateOptions({tabSize:h,indentSize:h,insertSpaces:this.insertSpaces})}})},50)}}const BD=class BD extends rB{constructor(){super(!1,!1,{id:BD.ID,label:m("indentUsingTabs","Indent Using Tabs"),alias:"Indent Using Tabs",precondition:void 0,metadata:{description:Te("indentUsingTabsDescription","Use indentation with tabs.")}})}};BD.ID="editor.action.indentUsingTabs";let IF=BD;const HD=class HD extends rB{constructor(){super(!0,!1,{id:HD.ID,label:m("indentUsingSpaces","Indent Using Spaces"),alias:"Indent Using Spaces",precondition:void 0,metadata:{description:Te("indentUsingSpacesDescription","Use indentation with spaces.")}})}};HD.ID="editor.action.indentUsingSpaces";let EF=HD;const VD=class VD extends rB{constructor(){super(!0,!0,{id:VD.ID,label:m("changeTabDisplaySize","Change Tab Display Size"),alias:"Change Tab Display Size",precondition:void 0,metadata:{description:Te("changeTabDisplaySizeDescription","Change the space size equivalent of the tab.")}})}};VD.ID="editor.action.changeTabDisplaySize";let NF=VD;const zD=class zD extends de{constructor(){super({id:zD.ID,label:m("detectIndentation","Detect Indentation from Content"),alias:"Detect Indentation from Content",precondition:void 0,metadata:{description:Te("detectIndentationDescription","Detect the indentation from content.")}})}run(e,t){const i=e.get(Kt),n=t.getModel();if(!n)return;const s=i.getCreationOptions(n.getLanguageId(),n.uri,n.isForSimpleWidget);n.detectIndentation(s.insertSpaces,s.tabSize)}};zD.ID="editor.action.detectIndentation";let TF=zD;class rIe extends de{constructor(){super({id:"editor.action.reindentlines",label:m("editor.reindentlines","Reindent Lines"),alias:"Reindent Lines",precondition:I.writable,metadata:{description:Te("editor.reindentlinesDescription","Reindent the lines of the editor.")}})}run(e,t){const i=e.get(di),n=t.getModel();if(!n)return;const s=vY(n,i,1,n.getLineCount());s.length>0&&(t.pushUndoStop(),t.executeEdits(this.id,s),t.pushUndoStop())}}class aIe extends de{constructor(){super({id:"editor.action.reindentselectedlines",label:m("editor.reindentselectedlines","Reindent Selected Lines"),alias:"Reindent Selected Lines",precondition:I.writable,metadata:{description:Te("editor.reindentselectedlinesDescription","Reindent the selected lines of the editor.")}})}run(e,t){const i=e.get(di),n=t.getModel();if(!n)return;const s=t.getSelections();if(s===null)return;const r=[];for(const a of s){let l=a.startLineNumber,c=a.endLineNumber;if(l!==c&&a.endColumn===1&&c--,l===1){if(l===c)continue}else l--;const d=vY(n,i,l,c);r.push(...d)}r.length>0&&(t.pushUndoStop(),t.executeEdits(this.id,r),t.pushUndoStop())}}class lIe{constructor(e,t){this._initialSelection=t,this._edits=[],this._selectionId=null;for(const i of e)i.range&&typeof i.text=="string"&&this._edits.push(i)}getEditOperations(e,t){for(const n of this._edits)t.addEditOperation(x.lift(n.range),n.text);let i=!1;Array.isArray(this._edits)&&this._edits.length===1&&this._initialSelection.isEmpty()&&(this._edits[0].range.startColumn===this._initialSelection.endColumn&&this._edits[0].range.startLineNumber===this._initialSelection.endLineNumber?(i=!0,this._selectionId=t.trackSelection(this._initialSelection,!0)):this._edits[0].range.endColumn===this._initialSelection.startColumn&&this._edits[0].range.endLineNumber===this._initialSelection.startLineNumber&&(i=!0,this._selectionId=t.trackSelection(this._initialSelection,!1))),i||(this._selectionId=t.trackSelection(this._initialSelection))}computeCursorState(e,t){return t.getTrackedSelection(this._selectionId)}}var C_;let ck=(C_=class{constructor(e,t){this.editor=e,this._languageConfigurationService=t,this.callOnDispose=new K,this.callOnModel=new K,this.callOnDispose.add(e.onDidChangeConfiguration(()=>this.update())),this.callOnDispose.add(e.onDidChangeModel(()=>this.update())),this.callOnDispose.add(e.onDidChangeModelLanguage(()=>this.update()))}update(){this.callOnModel.clear(),!(this.editor.getOption(12)<4||this.editor.getOption(55))&&this.editor.hasModel()&&this.callOnModel.add(this.editor.onDidPaste(({range:e})=>{this.trigger(e)}))}trigger(e){const t=this.editor.getSelections();if(t===null||t.length>1)return;const i=this.editor.getModel();if(!i||this.rangeContainsOnlyWhitespaceCharacters(i,e)||cIe(i,e)||!i.tokenization.isCheapToTokenize(e.getStartPosition().lineNumber))return;const s=this.editor.getOption(12),{tabSize:r,indentSize:a,insertSpaces:l}=i.getOptions(),c=[],d={shiftIndent:f=>wr.shiftIndent(f,f.length+1,r,a,l),unshiftIndent:f=>wr.unshiftIndent(f,f.length+1,r,a,l)};let h=e.startLineNumber;for(;h<=e.endLineNumber;){if(this.shouldIgnoreLine(i,h)){h++;continue}break}if(h>e.endLineNumber)return;let u=i.getLineContent(h);if(!/\S/.test(u.substring(0,e.startColumn-1))){const f=hv(s,i,i.getLanguageId(),h,d,this._languageConfigurationService);if(f!==null){const p=Nt(u),_=Ys(f,r),b=Ys(p,r);if(_!==b){const C=Lv(_,r,l);c.push({range:new x(h,1,h,p.length+1),text:C}),u=C+u.substring(p.length)}else{const C=wq(i,h,this._languageConfigurationService);if(C===0||C===8)return}}}const g=h;for(;h<e.endLineNumber;){if(!/\S/.test(i.getLineContent(h+1))){h++;continue}break}if(h!==e.endLineNumber){const p=hv(s,{tokenization:{getLineTokens:_=>i.tokenization.getLineTokens(_),getLanguageId:()=>i.getLanguageId(),getLanguageIdAtPosition:(_,b)=>i.getLanguageIdAtPosition(_,b)},getLineContent:_=>_===g?u:i.getLineContent(_)},i.getLanguageId(),h+1,d,this._languageConfigurationService);if(p!==null){const _=Ys(p,r),b=Ys(Nt(i.getLineContent(h+1)),r);if(_!==b){const C=_-b;for(let v=h+1;v<=e.endLineNumber;v++){const w=i.getLineContent(v),S=Nt(w),k=Ys(S,r)+C,D=Lv(k,r,l);D!==S&&c.push({range:new x(v,1,v,S.length+1),text:D})}}}}if(c.length>0){this.editor.pushUndoStop();const f=new lIe(c,this.editor.getSelection());this.editor.executeCommand("autoIndentOnPaste",f),this.editor.pushUndoStop()}}rangeContainsOnlyWhitespaceCharacters(e,t){const i=s=>s.trim().length===0;let n=!0;if(t.startLineNumber===t.endLineNumber){const r=e.getLineContent(t.startLineNumber).substring(t.startColumn-1,t.endColumn-1);n=i(r)}else for(let s=t.startLineNumber;s<=t.endLineNumber;s++){const r=e.getLineContent(s);if(s===t.startLineNumber){const a=r.substring(t.startColumn-1);n=i(a)}else if(s===t.endLineNumber){const a=r.substring(0,t.endColumn-1);n=i(a)}else n=e.getLineFirstNonWhitespaceColumn(s)===0;if(!n)break}return n}shouldIgnoreLine(e,t){e.tokenization.forceTokenization(t);const i=e.getLineFirstNonWhitespaceColumn(t);if(i===0)return!0;const n=e.tokenization.getLineTokens(t);if(n.getCount()>0){const s=n.findTokenIndexAtOffset(i);if(s>=0&&n.getStandardTokenType(s)===1)return!0}return!1}dispose(){this.callOnDispose.dispose(),this.callOnModel.dispose()}},C_.ID="editor.contrib.autoIndentOnPaste",C_);ck=sIe([oIe(1,di)],ck);function cIe(o,e){const t=i=>zle(o,i)===2;return t(e.getStartPosition())||t(e.getEndPosition())}function wY(o,e,t,i){if(o.getLineCount()===1&&o.getLineMaxColumn(1)===1)return;let n="";for(let r=0;r<t;r++)n+=" ";const s=new RegExp(n,"gi");for(let r=1,a=o.getLineCount();r<=a;r++){let l=o.getLineFirstNonWhitespaceColumn(r);if(l===0&&(l=o.getLineMaxColumn(r)),l===1)continue;const c=new x(r,1,r,l),d=o.getValueInRange(c),h=i?d.replace(/\t/ig,n):d.replace(s,"	");e.addEditOperation(c,h)}}class dIe{constructor(e,t){this.selection=e,this.tabSize=t,this.selectionId=null}getEditOperations(e,t){this.selectionId=t.trackSelection(this.selection),wY(e,t,this.tabSize,!0)}computeCursorState(e,t){return t.getTrackedSelection(this.selectionId)}}class hIe{constructor(e,t){this.selection=e,this.tabSize=t,this.selectionId=null}getEditOperations(e,t){this.selectionId=t.trackSelection(this.selection),wY(e,t,this.tabSize,!1)}computeCursorState(e,t){return t.getTrackedSelection(this.selectionId)}}ct(ck.ID,ck,2);Q(kF);Q(DF);Q(IF);Q(EF);Q(NF);Q(TF);Q(rIe);Q(aIe);ct(u1.ID,u1,1);um.register(Vx);class uIe{constructor(e,t,i){this._editRange=e,this._originalSelection=t,this._text=i}getEditOperations(e,t){t.addTrackedEditOperation(this._editRange,this._text)}computeCursorState(e,t){const n=t.getInverseEditOperations()[0].range;return this._originalSelection.isEmpty()?new pe(n.endLineNumber,Math.min(this._originalSelection.positionColumn,n.endColumn),n.endLineNumber,Math.min(this._originalSelection.positionColumn,n.endColumn)):new pe(n.endLineNumber,n.endColumn-this._text.length,n.endLineNumber,n.endColumn)}}var gIe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},fIe=function(o,e){return function(t,i){e(t,i,o)}},by,Dh;let rb=(Dh=class{static get(e){return e.getContribution(by.ID)}constructor(e,t){this.editor=e,this.editorWorkerService=t,this.decorations=this.editor.createDecorationsCollection()}dispose(){}run(e,t){var l;(l=this.currentRequest)==null||l.cancel();const i=this.editor.getSelection(),n=this.editor.getModel();if(!n||!i)return;let s=i;if(s.startLineNumber!==s.endLineNumber)return;const r=new Yq(this.editor,5),a=n.uri;return this.editorWorkerService.canNavigateValueSet(a)?(this.currentRequest=Yi(c=>this.editorWorkerService.navigateValueSet(a,s,t)),this.currentRequest.then(c=>{var f;if(!c||!c.range||!c.value||!r.validate(this.editor))return;const d=x.lift(c.range);let h=c.range;const u=c.value.length-(s.endColumn-s.startColumn);h={startLineNumber:h.startLineNumber,startColumn:h.startColumn,endLineNumber:h.endLineNumber,endColumn:h.startColumn+c.value.length},u>1&&(s=new pe(s.startLineNumber,s.startColumn,s.endLineNumber,s.endColumn+u-1));const g=new uIe(d,s,c.value);this.editor.pushUndoStop(),this.editor.executeCommand(e,g),this.editor.pushUndoStop(),this.decorations.set([{range:h,options:by.DECORATION}]),(f=this.decorationRemover)==null||f.cancel(),this.decorationRemover=ed(350),this.decorationRemover.then(()=>this.decorations.clear()).catch(Oe)}).catch(Oe)):Promise.resolve(void 0)}},by=Dh,Dh.ID="editor.contrib.inPlaceReplaceController",Dh.DECORATION=We.register({description:"in-place-replace",className:"valueSetReplacement"}),Dh);rb=by=gIe([fIe(1,Us)],rb);class mIe extends de{constructor(){super({id:"editor.action.inPlaceReplace.up",label:m("InPlaceReplaceAction.previous.label","Replace with Previous Value"),alias:"Replace with Previous Value",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:3159,weight:100}})}run(e,t){const i=rb.get(t);return i?i.run(this.id,!1):Promise.resolve(void 0)}}class pIe extends de{constructor(){super({id:"editor.action.inPlaceReplace.down",label:m("InPlaceReplaceAction.next.label","Replace with Next Value"),alias:"Replace with Next Value",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:3161,weight:100}})}run(e,t){const i=rb.get(t);return i?i.run(this.id,!0):Promise.resolve(void 0)}}ct(rb.ID,rb,4);Q(mIe);Q(pIe);class _Ie extends de{constructor(){super({id:"expandLineSelection",label:m("expandLineSelection","Expand Line Selection"),alias:"Expand Line Selection",precondition:void 0,kbOpts:{weight:0,kbExpr:I.textInputFocus,primary:2090}})}run(e,t,i){if(i=i||{},!t.hasModel())return;const n=t._getViewModel();n.model.pushStackElement(),n.setCursorStates(i.source,3,un.expandLineSelection(n,n.getCursorStates())),n.revealAllCursors(i.source,!0)}}Q(_Ie);class bIe{constructor(e,t,i){this._selection=e,this._cursors=t,this._selectionId=null,this._trimInRegexesAndStrings=i}getEditOperations(e,t){const i=CIe(e,this._cursors,this._trimInRegexesAndStrings);for(let n=0,s=i.length;n<s;n++){const r=i[n];t.addEditOperation(r.range,r.text)}this._selectionId=t.trackSelection(this._selection)}computeCursorState(e,t){return t.getTrackedSelection(this._selectionId)}}function CIe(o,e,t){e.sort((a,l)=>a.lineNumber===l.lineNumber?a.column-l.column:a.lineNumber-l.lineNumber);for(let a=e.length-2;a>=0;a--)e[a].lineNumber===e[a+1].lineNumber&&e.splice(a,1);const i=[];let n=0,s=0;const r=e.length;for(let a=1,l=o.getLineCount();a<=l;a++){const c=o.getLineContent(a),d=c.length+1;let h=0;if(s<r&&e[s].lineNumber===a&&(h=e[s].column,s++,h===d)||c.length===0)continue;const u=Na(c);let g=0;if(u===-1)g=1;else if(u!==c.length-1)g=u+2;else continue;if(!t){if(!o.tokenization.hasAccurateTokensForLine(a))continue;const f=o.tokenization.getLineTokens(a),p=f.getStandardTokenType(f.findTokenIndexAtOffset(g));if(p===2||p===3)continue}g=Math.max(h,g),i[n++]=ri.delete(new x(a,g,a,d))}return i}class SY{constructor(e,t,i){this._selection=e,this._isCopyingDown=t,this._noop=i||!1,this._selectionDirection=0,this._selectionId=null,this._startLineNumberDelta=0,this._endLineNumberDelta=0}getEditOperations(e,t){let i=this._selection;this._startLineNumberDelta=0,this._endLineNumberDelta=0,i.startLineNumber<i.endLineNumber&&i.endColumn===1&&(this._endLineNumberDelta=1,i=i.setEndPosition(i.endLineNumber-1,e.getLineMaxColumn(i.endLineNumber-1)));const n=[];for(let r=i.startLineNumber;r<=i.endLineNumber;r++)n.push(e.getLineContent(r));const s=n.join(`
`);s===""&&this._isCopyingDown&&(this._startLineNumberDelta++,this._endLineNumberDelta++),this._noop?t.addEditOperation(new x(i.endLineNumber,e.getLineMaxColumn(i.endLineNumber),i.endLineNumber+1,1),i.endLineNumber===e.getLineCount()?"":`
`):this._isCopyingDown?t.addEditOperation(new x(i.startLineNumber,1,i.startLineNumber,1),s+`
`):t.addEditOperation(new x(i.endLineNumber,e.getLineMaxColumn(i.endLineNumber),i.endLineNumber,e.getLineMaxColumn(i.endLineNumber)),`
`+s),this._selectionId=t.trackSelection(i),this._selectionDirection=this._selection.getDirection()}computeCursorState(e,t){let i=t.getTrackedSelection(this._selectionId);if(this._startLineNumberDelta!==0||this._endLineNumberDelta!==0){let n=i.startLineNumber,s=i.startColumn,r=i.endLineNumber,a=i.endColumn;this._startLineNumberDelta!==0&&(n=n+this._startLineNumberDelta,s=1),this._endLineNumberDelta!==0&&(r=r+this._endLineNumberDelta,a=1),i=pe.createWithDirection(n,s,r,a,this._selectionDirection)}return i}}var vIe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},wIe=function(o,e){return function(t,i){e(t,i,o)}};let RF=class{constructor(e,t,i,n){this._languageConfigurationService=n,this._selection=e,this._isMovingDown=t,this._autoIndent=i,this._selectionId=null,this._moveEndLineSelectionShrink=!1}getEditOperations(e,t){const i=()=>e.getLanguageId(),n=(h,u)=>e.getLanguageIdAtPosition(h,u),s=e.getLineCount();if(this._isMovingDown&&this._selection.endLineNumber===s){this._selectionId=t.trackSelection(this._selection);return}if(!this._isMovingDown&&this._selection.startLineNumber===1){this._selectionId=t.trackSelection(this._selection);return}this._moveEndPositionDown=!1;let r=this._selection;r.startLineNumber<r.endLineNumber&&r.endColumn===1&&(this._moveEndPositionDown=!0,r=r.setEndPosition(r.endLineNumber-1,e.getLineMaxColumn(r.endLineNumber-1)));const{tabSize:a,indentSize:l,insertSpaces:c}=e.getOptions(),d=this.buildIndentConverter(a,l,c);if(r.startLineNumber===r.endLineNumber&&e.getLineMaxColumn(r.startLineNumber)===1){const h=r.startLineNumber,u=this._isMovingDown?h+1:h-1;e.getLineMaxColumn(u)===1?t.addEditOperation(new x(1,1,1,1),null):(t.addEditOperation(new x(h,1,h,1),e.getLineContent(u)),t.addEditOperation(new x(u,1,u,e.getLineMaxColumn(u)),null)),r=new pe(u,1,u,1)}else{let h,u;if(this._isMovingDown){h=r.endLineNumber+1,u=e.getLineContent(h),t.addEditOperation(new x(h-1,e.getLineMaxColumn(h-1),h,e.getLineMaxColumn(h)),null);let g=u;if(this.shouldAutoIndent(e,r)){const f=this.matchEnterRule(e,d,a,h,r.startLineNumber-1);if(f!==null){const _=Nt(e.getLineContent(h)),b=f+Ys(_,a);g=Lv(b,a,c)+this.trimStart(u)}else{const _={tokenization:{getLineTokens:C=>C===r.startLineNumber?e.tokenization.getLineTokens(h):e.tokenization.getLineTokens(C),getLanguageId:i,getLanguageIdAtPosition:n},getLineContent:C=>C===r.startLineNumber?e.getLineContent(h):e.getLineContent(C)},b=hv(this._autoIndent,_,e.getLanguageIdAtPosition(h,1),r.startLineNumber,d,this._languageConfigurationService);if(b!==null){const C=Nt(e.getLineContent(h)),v=Ys(b,a),w=Ys(C,a);v!==w&&(g=Lv(v,a,c)+this.trimStart(u))}}t.addEditOperation(new x(r.startLineNumber,1,r.startLineNumber,1),g+`
`);const p=this.matchEnterRuleMovingDown(e,d,a,r.startLineNumber,h,g);if(p!==null)p!==0&&this.getIndentEditsOfMovingBlock(e,t,r,a,c,p);else{const _={tokenization:{getLineTokens:C=>C===r.startLineNumber?e.tokenization.getLineTokens(h):C>=r.startLineNumber+1&&C<=r.endLineNumber+1?e.tokenization.getLineTokens(C-1):e.tokenization.getLineTokens(C),getLanguageId:i,getLanguageIdAtPosition:n},getLineContent:C=>C===r.startLineNumber?g:C>=r.startLineNumber+1&&C<=r.endLineNumber+1?e.getLineContent(C-1):e.getLineContent(C)},b=hv(this._autoIndent,_,e.getLanguageIdAtPosition(h,1),r.startLineNumber+1,d,this._languageConfigurationService);if(b!==null){const C=Nt(e.getLineContent(r.startLineNumber)),v=Ys(b,a),w=Ys(C,a);if(v!==w){const S=v-w;this.getIndentEditsOfMovingBlock(e,t,r,a,c,S)}}}}else t.addEditOperation(new x(r.startLineNumber,1,r.startLineNumber,1),g+`
`)}else if(h=r.startLineNumber-1,u=e.getLineContent(h),t.addEditOperation(new x(h,1,h+1,1),null),t.addEditOperation(new x(r.endLineNumber,e.getLineMaxColumn(r.endLineNumber),r.endLineNumber,e.getLineMaxColumn(r.endLineNumber)),`
`+u),this.shouldAutoIndent(e,r)){const g={tokenization:{getLineTokens:p=>p===h?e.tokenization.getLineTokens(r.startLineNumber):e.tokenization.getLineTokens(p),getLanguageId:i,getLanguageIdAtPosition:n},getLineContent:p=>p===h?e.getLineContent(r.startLineNumber):e.getLineContent(p)},f=this.matchEnterRule(e,d,a,r.startLineNumber,r.startLineNumber-2);if(f!==null)f!==0&&this.getIndentEditsOfMovingBlock(e,t,r,a,c,f);else{const p=hv(this._autoIndent,g,e.getLanguageIdAtPosition(r.startLineNumber,1),h,d,this._languageConfigurationService);if(p!==null){const _=Nt(e.getLineContent(r.startLineNumber)),b=Ys(p,a),C=Ys(_,a);if(b!==C){const v=b-C;this.getIndentEditsOfMovingBlock(e,t,r,a,c,v)}}}}}this._selectionId=t.trackSelection(r)}buildIndentConverter(e,t,i){return{shiftIndent:n=>wr.shiftIndent(n,n.length+1,e,t,i),unshiftIndent:n=>wr.unshiftIndent(n,n.length+1,e,t,i)}}parseEnterResult(e,t,i,n,s){if(s){let r=s.indentation;s.indentAction===Bi.None||s.indentAction===Bi.Indent?r=s.indentation+s.appendText:s.indentAction===Bi.IndentOutdent?r=s.indentation:s.indentAction===Bi.Outdent&&(r=t.unshiftIndent(s.indentation)+s.appendText);const a=e.getLineContent(n);if(this.trimStart(a).indexOf(this.trimStart(r))>=0){const l=Nt(e.getLineContent(n));let c=Nt(r);const d=wq(e,n,this._languageConfigurationService);d!==null&&d&2&&(c=t.unshiftIndent(c));const h=Ys(c,i),u=Ys(l,i);return h-u}}return null}matchEnterRuleMovingDown(e,t,i,n,s,r){if(Na(r)>=0){const a=e.getLineMaxColumn(s),l=o_(this._autoIndent,e,new x(s,a,s,a),this._languageConfigurationService);return this.parseEnterResult(e,t,i,n,l)}else{let a=n-1;for(;a>=1;){const d=e.getLineContent(a);if(Na(d)>=0)break;a--}if(a<1||n>e.getLineCount())return null;const l=e.getLineMaxColumn(a),c=o_(this._autoIndent,e,new x(a,l,a,l),this._languageConfigurationService);return this.parseEnterResult(e,t,i,n,c)}}matchEnterRule(e,t,i,n,s,r){let a=s;for(;a>=1;){let d;if(a===s&&r!==void 0?d=r:d=e.getLineContent(a),Na(d)>=0)break;a--}if(a<1||n>e.getLineCount())return null;const l=e.getLineMaxColumn(a),c=o_(this._autoIndent,e,new x(a,l,a,l),this._languageConfigurationService);return this.parseEnterResult(e,t,i,n,c)}trimStart(e){return e.replace(/^\s+/,"")}shouldAutoIndent(e,t){if(this._autoIndent<4||!e.tokenization.isCheapToTokenize(t.startLineNumber))return!1;const i=e.getLanguageIdAtPosition(t.startLineNumber,1),n=e.getLanguageIdAtPosition(t.endLineNumber,1);return!(i!==n||this._languageConfigurationService.getLanguageConfiguration(i).indentRulesSupport===null)}getIndentEditsOfMovingBlock(e,t,i,n,s,r){for(let a=i.startLineNumber;a<=i.endLineNumber;a++){const l=e.getLineContent(a),c=Nt(l),h=Ys(c,n)+r,u=Lv(h,n,s);u!==c&&(t.addEditOperation(new x(a,1,a,c.length+1),u),a===i.endLineNumber&&i.endColumn<=c.length+1&&u===""&&(this._moveEndLineSelectionShrink=!0))}}computeCursorState(e,t){let i=t.getTrackedSelection(this._selectionId);return this._moveEndPositionDown&&(i=i.setEndPosition(i.endLineNumber+1,1)),this._moveEndLineSelectionShrink&&i.startLineNumber<i.endLineNumber&&(i=i.setEndPosition(i.endLineNumber,2)),i}};RF=vIe([wIe(3,di)],RF);const Op=class Op{static getCollator(){return Op._COLLATOR||(Op._COLLATOR=new Intl.Collator),Op._COLLATOR}constructor(e,t){this.selection=e,this.descending=t,this.selectionId=null}getEditOperations(e,t){const i=SIe(e,this.selection,this.descending);i&&t.addEditOperation(i.range,i.text),this.selectionId=t.trackSelection(this.selection)}computeCursorState(e,t){return t.getTrackedSelection(this.selectionId)}static canRun(e,t,i){if(e===null)return!1;const n=yY(e,t,i);if(!n)return!1;for(let s=0,r=n.before.length;s<r;s++)if(n.before[s]!==n.after[s])return!0;return!1}};Op._COLLATOR=null;let _1=Op;function yY(o,e,t){const i=e.startLineNumber;let n=e.endLineNumber;if(e.endColumn===1&&n--,i>=n)return null;const s=[];for(let a=i;a<=n;a++)s.push(o.getLineContent(a));let r=s.slice(0);return r.sort(_1.getCollator().compare),t===!0&&(r=r.reverse()),{startLineNumber:i,endLineNumber:n,before:s,after:r}}function SIe(o,e,t){const i=yY(o,e,t);return i?ri.replace(new x(i.startLineNumber,1,i.endLineNumber,o.getLineMaxColumn(i.endLineNumber)),i.after.join(`
`)):null}class LY extends de{constructor(e,t){super(t),this.down=e}run(e,t){if(!t.hasModel())return;const i=t.getSelections().map((r,a)=>({selection:r,index:a,ignore:!1}));i.sort((r,a)=>x.compareRangesUsingStarts(r.selection,a.selection));let n=i[0];for(let r=1;r<i.length;r++){const a=i[r];n.selection.endLineNumber===a.selection.startLineNumber&&(n.index<a.index?a.ignore=!0:(n.ignore=!0,n=a))}const s=[];for(const r of i)s.push(new SY(r.selection,this.down,r.ignore));t.pushUndoStop(),t.executeCommands(this.id,s),t.pushUndoStop()}}class yIe extends LY{constructor(){super(!1,{id:"editor.action.copyLinesUpAction",label:m("lines.copyUp","Copy Line Up"),alias:"Copy Line Up",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:1552,linux:{primary:3600},weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"2_line",title:m({key:"miCopyLinesUp",comment:["&& denotes a mnemonic"]},"&&Copy Line Up"),order:1}})}}class LIe extends LY{constructor(){super(!0,{id:"editor.action.copyLinesDownAction",label:m("lines.copyDown","Copy Line Down"),alias:"Copy Line Down",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:1554,linux:{primary:3602},weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"2_line",title:m({key:"miCopyLinesDown",comment:["&& denotes a mnemonic"]},"Co&&py Line Down"),order:2}})}}class xIe extends de{constructor(){super({id:"editor.action.duplicateSelection",label:m("duplicateSelection","Duplicate Selection"),alias:"Duplicate Selection",precondition:I.writable,menuOpts:{menuId:me.MenubarSelectionMenu,group:"2_line",title:m({key:"miDuplicateSelection",comment:["&& denotes a mnemonic"]},"&&Duplicate Selection"),order:5}})}run(e,t,i){if(!t.hasModel())return;const n=[],s=t.getSelections(),r=t.getModel();for(const a of s)if(a.isEmpty())n.push(new SY(a,!0));else{const l=new pe(a.endLineNumber,a.endColumn,a.endLineNumber,a.endColumn);n.push(new S_e(l,r.getValueInRange(a)))}t.pushUndoStop(),t.executeCommands(this.id,n),t.pushUndoStop()}}class xY extends de{constructor(e,t){super(t),this.down=e}run(e,t){const i=e.get(di),n=[],s=t.getSelections()||[],r=t.getOption(12);for(const a of s)n.push(new RF(a,this.down,r,i));t.pushUndoStop(),t.executeCommands(this.id,n),t.pushUndoStop()}}class kIe extends xY{constructor(){super(!1,{id:"editor.action.moveLinesUpAction",label:m("lines.moveUp","Move Line Up"),alias:"Move Line Up",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:528,linux:{primary:528},weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"2_line",title:m({key:"miMoveLinesUp",comment:["&& denotes a mnemonic"]},"Mo&&ve Line Up"),order:3}})}}class DIe extends xY{constructor(){super(!0,{id:"editor.action.moveLinesDownAction",label:m("lines.moveDown","Move Line Down"),alias:"Move Line Down",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:530,linux:{primary:530},weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"2_line",title:m({key:"miMoveLinesDown",comment:["&& denotes a mnemonic"]},"Move &&Line Down"),order:4}})}}class kY extends de{constructor(e,t){super(t),this.descending=e}run(e,t){if(!t.hasModel())return;const i=t.getModel();let n=t.getSelections();n.length===1&&n[0].isEmpty()&&(n=[new pe(1,1,i.getLineCount(),i.getLineMaxColumn(i.getLineCount()))]);for(const r of n)if(!_1.canRun(t.getModel(),r,this.descending))return;const s=[];for(let r=0,a=n.length;r<a;r++)s[r]=new _1(n[r],this.descending);t.pushUndoStop(),t.executeCommands(this.id,s),t.pushUndoStop()}}class IIe extends kY{constructor(){super(!1,{id:"editor.action.sortLinesAscending",label:m("lines.sortAscending","Sort Lines Ascending"),alias:"Sort Lines Ascending",precondition:I.writable})}}class EIe extends kY{constructor(){super(!0,{id:"editor.action.sortLinesDescending",label:m("lines.sortDescending","Sort Lines Descending"),alias:"Sort Lines Descending",precondition:I.writable})}}class NIe extends de{constructor(){super({id:"editor.action.removeDuplicateLines",label:m("lines.deleteDuplicates","Delete Duplicate Lines"),alias:"Delete Duplicate Lines",precondition:I.writable})}run(e,t){if(!t.hasModel())return;const i=t.getModel();if(i.getLineCount()===1&&i.getLineMaxColumn(1)===1)return;const n=[],s=[];let r=0,a=!0,l=t.getSelections();l.length===1&&l[0].isEmpty()&&(l=[new pe(1,1,i.getLineCount(),i.getLineMaxColumn(i.getLineCount()))],a=!1);for(const c of l){const d=new Set,h=[];for(let p=c.startLineNumber;p<=c.endLineNumber;p++){const _=i.getLineContent(p);d.has(_)||(h.push(_),d.add(_))}const u=new pe(c.startLineNumber,1,c.endLineNumber,i.getLineMaxColumn(c.endLineNumber)),g=c.startLineNumber-r,f=new pe(g,1,g+h.length-1,h[h.length-1].length);n.push(ri.replace(u,h.join(`
`))),s.push(f),r+=c.endLineNumber-c.startLineNumber+1-h.length}t.pushUndoStop(),t.executeEdits(this.id,n,a?s:void 0),t.pushUndoStop()}}const UD=class UD extends de{constructor(){super({id:UD.ID,label:m("lines.trimTrailingWhitespace","Trim Trailing Whitespace"),alias:"Trim Trailing Whitespace",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:Hi(2089,2102),weight:100}})}run(e,t,i){let n=[];i.reason==="auto-save"&&(n=(t.getSelections()||[]).map(d=>new A(d.positionLineNumber,d.positionColumn)));const s=t.getSelection();if(s===null)return;const r=e.get(qe),a=t.getModel(),l=r.getValue("files.trimTrailingWhitespaceInRegexAndStrings",{overrideIdentifier:a==null?void 0:a.getLanguageId(),resource:a==null?void 0:a.uri}),c=new bIe(s,n,l);t.pushUndoStop(),t.executeCommands(this.id,[c]),t.pushUndoStop()}};UD.ID="editor.action.trimTrailingWhitespace";let MF=UD;class TIe extends de{constructor(){super({id:"editor.action.deleteLines",label:m("lines.delete","Delete Line"),alias:"Delete Line",precondition:I.writable,kbOpts:{kbExpr:I.textInputFocus,primary:3113,weight:100}})}run(e,t){if(!t.hasModel())return;const i=this._getLinesToRemove(t),n=t.getModel();if(n.getLineCount()===1&&n.getLineMaxColumn(1)===1)return;let s=0;const r=[],a=[];for(let l=0,c=i.length;l<c;l++){const d=i[l];let h=d.startLineNumber,u=d.endLineNumber,g=1,f=n.getLineMaxColumn(u);u<n.getLineCount()?(u+=1,f=1):h>1&&(h-=1,g=n.getLineMaxColumn(h)),r.push(ri.replace(new pe(h,g,u,f),"")),a.push(new pe(h-s,d.positionColumn,h-s,d.positionColumn)),s+=d.endLineNumber-d.startLineNumber+1}t.pushUndoStop(),t.executeEdits(this.id,r,a),t.pushUndoStop()}_getLinesToRemove(e){const t=e.getSelections().map(s=>{let r=s.endLineNumber;return s.startLineNumber<s.endLineNumber&&s.endColumn===1&&(r-=1),{startLineNumber:s.startLineNumber,selectionStartColumn:s.selectionStartColumn,endLineNumber:r,positionColumn:s.positionColumn}});t.sort((s,r)=>s.startLineNumber===r.startLineNumber?s.endLineNumber-r.endLineNumber:s.startLineNumber-r.startLineNumber);const i=[];let n=t[0];for(let s=1;s<t.length;s++)n.endLineNumber+1>=t[s].startLineNumber?n.endLineNumber=t[s].endLineNumber:(i.push(n),n=t[s]);return i.push(n),i}}class RIe extends de{constructor(){super({id:"editor.action.indentLines",label:m("lines.indent","Indent Line"),alias:"Indent Line",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:2142,weight:100}})}run(e,t){const i=t._getViewModel();i&&(t.pushUndoStop(),t.executeCommands(this.id,Ud.indent(i.cursorConfig,t.getModel(),t.getSelections())),t.pushUndoStop())}}class MIe extends de{constructor(){super({id:"editor.action.outdentLines",label:m("lines.outdent","Outdent Line"),alias:"Outdent Line",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:2140,weight:100}})}run(e,t){r_.Outdent.runEditorCommand(e,t,null)}}class AIe extends de{constructor(){super({id:"editor.action.insertLineBefore",label:m("lines.insertBefore","Insert Line Above"),alias:"Insert Line Above",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:3075,weight:100}})}run(e,t){const i=t._getViewModel();i&&(t.pushUndoStop(),t.executeCommands(this.id,CE.lineInsertBefore(i.cursorConfig,t.getModel(),t.getSelections())))}}class PIe extends de{constructor(){super({id:"editor.action.insertLineAfter",label:m("lines.insertAfter","Insert Line Below"),alias:"Insert Line Below",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:2051,weight:100}})}run(e,t){const i=t._getViewModel();i&&(t.pushUndoStop(),t.executeCommands(this.id,CE.lineInsertAfter(i.cursorConfig,t.getModel(),t.getSelections())))}}class DY extends de{run(e,t){if(!t.hasModel())return;const i=t.getSelection(),n=this._getRangesToDelete(t),s=[];for(let l=0,c=n.length-1;l<c;l++){const d=n[l],h=n[l+1];x.intersectRanges(d,h)===null?s.push(d):n[l+1]=x.plusRange(d,h)}s.push(n[n.length-1]);const r=this._getEndCursorState(i,s),a=s.map(l=>ri.replace(l,""));t.pushUndoStop(),t.executeEdits(this.id,a,r),t.pushUndoStop()}}class OIe extends DY{constructor(){super({id:"deleteAllLeft",label:m("lines.deleteAllLeft","Delete All Left"),alias:"Delete All Left",precondition:I.writable,kbOpts:{kbExpr:I.textInputFocus,primary:0,mac:{primary:2049},weight:100}})}_getEndCursorState(e,t){let i=null;const n=[];let s=0;return t.forEach(r=>{let a;if(r.endColumn===1&&s>0){const l=r.startLineNumber-s;a=new pe(l,r.startColumn,l,r.startColumn)}else a=new pe(r.startLineNumber,r.startColumn,r.startLineNumber,r.startColumn);s+=r.endLineNumber-r.startLineNumber,r.intersectRanges(e)?i=a:n.push(a)}),i&&n.unshift(i),n}_getRangesToDelete(e){const t=e.getSelections();if(t===null)return[];let i=t;const n=e.getModel();return n===null?[]:(i.sort(x.compareRangesUsingStarts),i=i.map(s=>{if(s.isEmpty())if(s.startColumn===1){const r=Math.max(1,s.startLineNumber-1),a=s.startLineNumber===1?1:n.getLineLength(r)+1;return new x(r,a,s.startLineNumber,1)}else return new x(s.startLineNumber,1,s.startLineNumber,s.startColumn);else return new x(s.startLineNumber,1,s.endLineNumber,s.endColumn)}),i)}}class FIe extends DY{constructor(){super({id:"deleteAllRight",label:m("lines.deleteAllRight","Delete All Right"),alias:"Delete All Right",precondition:I.writable,kbOpts:{kbExpr:I.textInputFocus,primary:0,mac:{primary:297,secondary:[2068]},weight:100}})}_getEndCursorState(e,t){let i=null;const n=[];for(let s=0,r=t.length,a=0;s<r;s++){const l=t[s],c=new pe(l.startLineNumber-a,l.startColumn,l.startLineNumber-a,l.startColumn);l.intersectRanges(e)?i=c:n.push(c)}return i&&n.unshift(i),n}_getRangesToDelete(e){const t=e.getModel();if(t===null)return[];const i=e.getSelections();if(i===null)return[];const n=i.map(s=>{if(s.isEmpty()){const r=t.getLineMaxColumn(s.startLineNumber);return s.startColumn===r?new x(s.startLineNumber,s.startColumn,s.startLineNumber+1,1):new x(s.startLineNumber,s.startColumn,s.startLineNumber,r)}return s});return n.sort(x.compareRangesUsingStarts),n}}class WIe extends de{constructor(){super({id:"editor.action.joinLines",label:m("lines.joinLines","Join Lines"),alias:"Join Lines",precondition:I.writable,kbOpts:{kbExpr:I.editorTextFocus,primary:0,mac:{primary:296},weight:100}})}run(e,t){const i=t.getSelections();if(i===null)return;let n=t.getSelection();if(n===null)return;i.sort(x.compareRangesUsingStarts);const s=[],r=i.reduce((u,g)=>u.isEmpty()?u.endLineNumber===g.startLineNumber?(n.equalsSelection(u)&&(n=g),g):g.startLineNumber>u.endLineNumber+1?(s.push(u),g):new pe(u.startLineNumber,u.startColumn,g.endLineNumber,g.endColumn):g.startLineNumber>u.endLineNumber?(s.push(u),g):new pe(u.startLineNumber,u.startColumn,g.endLineNumber,g.endColumn));s.push(r);const a=t.getModel();if(a===null)return;const l=[],c=[];let d=n,h=0;for(let u=0,g=s.length;u<g;u++){const f=s[u],p=f.startLineNumber,_=1;let b=0,C,v;const w=a.getLineLength(f.endLineNumber)-f.endColumn;if(f.isEmpty()||f.startLineNumber===f.endLineNumber){const k=f.getStartPosition();k.lineNumber<a.getLineCount()?(C=p+1,v=a.getLineMaxColumn(C)):(C=k.lineNumber,v=a.getLineMaxColumn(k.lineNumber))}else C=f.endLineNumber,v=a.getLineMaxColumn(C);let S=a.getLineContent(p);for(let k=p+1;k<=C;k++){const D=a.getLineContent(k),N=a.getLineFirstNonWhitespaceColumn(k);if(N>=1){let B=!0;S===""&&(B=!1),B&&(S.charAt(S.length-1)===" "||S.charAt(S.length-1)==="	")&&(B=!1,S=S.replace(/[\s\uFEFF\xA0]+$/g," "));const W=D.substr(N-1);S+=(B?" ":"")+W,B?b=W.length+1:b=W.length}else b=0}const L=new x(p,_,C,v);if(!L.isEmpty()){let k;f.isEmpty()?(l.push(ri.replace(L,S)),k=new pe(L.startLineNumber-h,S.length-b+1,p-h,S.length-b+1)):f.startLineNumber===f.endLineNumber?(l.push(ri.replace(L,S)),k=new pe(f.startLineNumber-h,f.startColumn,f.endLineNumber-h,f.endColumn)):(l.push(ri.replace(L,S)),k=new pe(f.startLineNumber-h,f.startColumn,f.startLineNumber-h,S.length-w)),x.intersectRanges(L,n)!==null?d=k:c.push(k)}h+=L.endLineNumber-L.startLineNumber}c.unshift(d),t.pushUndoStop(),t.executeEdits(this.id,l,c),t.pushUndoStop()}}class BIe extends de{constructor(){super({id:"editor.action.transpose",label:m("editor.transpose","Transpose Characters around the Cursor"),alias:"Transpose Characters around the Cursor",precondition:I.writable})}run(e,t){const i=t.getSelections();if(i===null)return;const n=t.getModel();if(n===null)return;const s=[];for(let r=0,a=i.length;r<a;r++){const l=i[r];if(!l.isEmpty())continue;const c=l.getStartPosition(),d=n.getLineMaxColumn(c.lineNumber);if(c.column>=d){if(c.lineNumber===n.getLineCount())continue;const h=new x(c.lineNumber,Math.max(1,c.column-1),c.lineNumber+1,1),u=n.getValueInRange(h).split("").reverse().join("");s.push(new kn(new pe(c.lineNumber,Math.max(1,c.column-1),c.lineNumber+1,1),u))}else{const h=new x(c.lineNumber,Math.max(1,c.column-1),c.lineNumber,c.column+1),u=n.getValueInRange(h).split("").reverse().join("");s.push(new j3(h,u,new pe(c.lineNumber,c.column+1,c.lineNumber,c.column+1)))}}t.pushUndoStop(),t.executeCommands(this.id,s),t.pushUndoStop()}}class mm extends de{run(e,t){const i=t.getSelections();if(i===null)return;const n=t.getModel();if(n===null)return;const s=t.getOption(132),r=[];for(const a of i)if(a.isEmpty()){const l=a.getStartPosition(),c=t.getConfiguredWordAtPosition(l);if(!c)continue;const d=new x(l.lineNumber,c.startColumn,l.lineNumber,c.endColumn),h=n.getValueInRange(d);r.push(ri.replace(d,this._modifyText(h,s)))}else{const l=n.getValueInRange(a);r.push(ri.replace(a,this._modifyText(l,s)))}t.pushUndoStop(),t.executeEdits(this.id,r),t.pushUndoStop()}}class HIe extends mm{constructor(){super({id:"editor.action.transformToUppercase",label:m("editor.transformToUppercase","Transform to Uppercase"),alias:"Transform to Uppercase",precondition:I.writable})}_modifyText(e,t){return e.toLocaleUpperCase()}}class VIe extends mm{constructor(){super({id:"editor.action.transformToLowercase",label:m("editor.transformToLowercase","Transform to Lowercase"),alias:"Transform to Lowercase",precondition:I.writable})}_modifyText(e,t){return e.toLocaleLowerCase()}}class Yc{constructor(e,t){this._pattern=e,this._flags=t,this._actual=null,this._evaluated=!1}get(){if(!this._evaluated){this._evaluated=!0;try{this._actual=new RegExp(this._pattern,this._flags)}catch{}}return this._actual}isSupported(){return this.get()!==null}}const $D=class $D extends mm{constructor(){super({id:"editor.action.transformToTitlecase",label:m("editor.transformToTitlecase","Transform to Title Case"),alias:"Transform to Title Case",precondition:I.writable})}_modifyText(e,t){const i=$D.titleBoundary.get();return i?e.toLocaleLowerCase().replace(i,n=>n.toLocaleUpperCase()):e}};$D.titleBoundary=new Yc("(^|[^\\p{L}\\p{N}']|((^|\\P{L})'))\\p{L}","gmu");let dk=$D;const Fp=class Fp extends mm{constructor(){super({id:"editor.action.transformToSnakecase",label:m("editor.transformToSnakecase","Transform to Snake Case"),alias:"Transform to Snake Case",precondition:I.writable})}_modifyText(e,t){const i=Fp.caseBoundary.get(),n=Fp.singleLetters.get();return!i||!n?e:e.replace(i,"$1_$2").replace(n,"$1_$2$3").toLocaleLowerCase()}};Fp.caseBoundary=new Yc("(\\p{Ll})(\\p{Lu})","gmu"),Fp.singleLetters=new Yc("(\\p{Lu}|\\p{N})(\\p{Lu})(\\p{Ll})","gmu");let xv=Fp;const jD=class jD extends mm{constructor(){super({id:"editor.action.transformToCamelcase",label:m("editor.transformToCamelcase","Transform to Camel Case"),alias:"Transform to Camel Case",precondition:I.writable})}_modifyText(e,t){const i=jD.wordBoundary.get();if(!i)return e;const n=e.split(i);return n.shift()+n.map(r=>r.substring(0,1).toLocaleUpperCase()+r.substring(1)).join("")}};jD.wordBoundary=new Yc("[_\\s-]","gm");let hk=jD;const Wp=class Wp extends mm{constructor(){super({id:"editor.action.transformToPascalcase",label:m("editor.transformToPascalcase","Transform to Pascal Case"),alias:"Transform to Pascal Case",precondition:I.writable})}_modifyText(e,t){const i=Wp.wordBoundary.get(),n=Wp.wordBoundaryToMaintain.get();return!i||!n?e:e.split(n).map(a=>a.split(i)).flat().map(a=>a.substring(0,1).toLocaleUpperCase()+a.substring(1)).join("")}};Wp.wordBoundary=new Yc("[_\\s-]","gm"),Wp.wordBoundaryToMaintain=new Yc("(?<=\\.)","gm");let uk=Wp;const Fd=class Fd extends mm{static isSupported(){return[this.caseBoundary,this.singleLetters,this.underscoreBoundary].every(t=>t.isSupported())}constructor(){super({id:"editor.action.transformToKebabcase",label:m("editor.transformToKebabcase","Transform to Kebab Case"),alias:"Transform to Kebab Case",precondition:I.writable})}_modifyText(e,t){const i=Fd.caseBoundary.get(),n=Fd.singleLetters.get(),s=Fd.underscoreBoundary.get();return!i||!n||!s?e:e.replace(s,"$1-$3").replace(i,"$1-$2").replace(n,"$1-$2").toLocaleLowerCase()}};Fd.caseBoundary=new Yc("(\\p{Ll})(\\p{Lu})","gmu"),Fd.singleLetters=new Yc("(\\p{Lu}|\\p{N})(\\p{Lu}\\p{Ll})","gmu"),Fd.underscoreBoundary=new Yc("(\\S)(_)(\\S)","gm");let gk=Fd;Q(yIe);Q(LIe);Q(xIe);Q(kIe);Q(DIe);Q(IIe);Q(EIe);Q(NIe);Q(MF);Q(TIe);Q(RIe);Q(MIe);Q(AIe);Q(PIe);Q(OIe);Q(FIe);Q(WIe);Q(BIe);Q(HIe);Q(VIe);xv.caseBoundary.isSupported()&&xv.singleLetters.isSupported()&&Q(xv);hk.wordBoundary.isSupported()&&Q(hk);uk.wordBoundary.isSupported()&&Q(uk);dk.titleBoundary.isSupported()&&Q(dk);gk.isSupported()&&Q(gk);var zIe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},R0=function(o,e){return function(t,i){e(t,i,o)}},Cy;const IY=new le("LinkedEditingInputVisible",!1),UIe="linked-editing-decoration";var Ih;let ab=(Ih=class extends F{static get(e){return e.getContribution(Cy.ID)}constructor(e,t,i,n,s){super(),this.languageConfigurationService=n,this._syncRangesToken=0,this._localToDispose=this._register(new K),this._editor=e,this._providers=i.linkedEditingRangeProvider,this._enabled=!1,this._visibleContextKey=IY.bindTo(t),this._debounceInformation=s.for(this._providers,"Linked Editing",{max:200}),this._currentDecorations=this._editor.createDecorationsCollection(),this._languageWordPattern=null,this._currentWordPattern=null,this._ignoreChangeEvent=!1,this._localToDispose=this._register(new K),this._rangeUpdateTriggerPromise=null,this._rangeSyncTriggerPromise=null,this._currentRequestCts=null,this._currentRequestPosition=null,this._currentRequestModelVersion=null,this._register(this._editor.onDidChangeModel(()=>this.reinitialize(!0))),this._register(this._editor.onDidChangeConfiguration(r=>{(r.hasChanged(70)||r.hasChanged(94))&&this.reinitialize(!1)})),this._register(this._providers.onDidChange(()=>this.reinitialize(!1))),this._register(this._editor.onDidChangeModelLanguage(()=>this.reinitialize(!0))),this.reinitialize(!0)}reinitialize(e){const t=this._editor.getModel(),i=t!==null&&(this._editor.getOption(70)||this._editor.getOption(94))&&this._providers.has(t);if(i===this._enabled&&!e||(this._enabled=i,this.clearRanges(),this._localToDispose.clear(),!i||t===null))return;this._localToDispose.add(oe.runAndSubscribe(t.onDidChangeLanguageConfiguration,()=>{this._languageWordPattern=this.languageConfigurationService.getLanguageConfiguration(t.getLanguageId()).getWordDefinition()}));const n=new _r(this._debounceInformation.get(t)),s=()=>{this._rangeUpdateTriggerPromise=n.trigger(()=>this.updateRanges(),this._debounceDuration??this._debounceInformation.get(t))},r=new _r(0),a=l=>{this._rangeSyncTriggerPromise=r.trigger(()=>this._syncRanges(l))};this._localToDispose.add(this._editor.onDidChangeCursorPosition(()=>{s()})),this._localToDispose.add(this._editor.onDidChangeModelContent(l=>{if(!this._ignoreChangeEvent&&this._currentDecorations.length>0){const c=this._currentDecorations.getRange(0);if(c&&l.changes.every(d=>c.intersectRanges(d.range))){a(this._syncRangesToken);return}}s()})),this._localToDispose.add({dispose:()=>{n.dispose(),r.dispose()}}),this.updateRanges()}_syncRanges(e){if(!this._editor.hasModel()||e!==this._syncRangesToken||this._currentDecorations.length===0)return;const t=this._editor.getModel(),i=this._currentDecorations.getRange(0);if(!i||i.startLineNumber!==i.endLineNumber)return this.clearRanges();const n=t.getValueInRange(i);if(this._currentWordPattern){const r=n.match(this._currentWordPattern);if((r?r[0].length:0)!==n.length)return this.clearRanges()}const s=[];for(let r=1,a=this._currentDecorations.length;r<a;r++){const l=this._currentDecorations.getRange(r);if(l)if(l.startLineNumber!==l.endLineNumber)s.push({range:l,text:n});else{let c=t.getValueInRange(l),d=n,h=l.startColumn,u=l.endColumn;const g=Oh(c,d);h+=g,c=c.substr(g),d=d.substr(g);const f=By(c,d);u-=f,c=c.substr(0,c.length-f),d=d.substr(0,d.length-f),(h!==u||d.length!==0)&&s.push({range:new x(l.startLineNumber,h,l.endLineNumber,u),text:d})}}if(s.length!==0)try{this._editor.popUndoStop(),this._ignoreChangeEvent=!0;const r=this._editor._getViewModel().getPrevEditOperationType();this._editor.executeEdits("linkedEditing",s),this._editor._getViewModel().setPrevEditOperationType(r)}finally{this._ignoreChangeEvent=!1}}dispose(){this.clearRanges(),super.dispose()}clearRanges(){this._visibleContextKey.set(!1),this._currentDecorations.clear(),this._currentRequestCts&&(this._currentRequestCts.cancel(),this._currentRequestCts=null,this._currentRequestPosition=null)}async updateRanges(e=!1){if(!this._editor.hasModel()){this.clearRanges();return}const t=this._editor.getPosition();if(!this._enabled&&!e||this._editor.getSelections().length>1){this.clearRanges();return}const i=this._editor.getModel(),n=i.getVersionId();if(this._currentRequestPosition&&this._currentRequestModelVersion===n){if(t.equals(this._currentRequestPosition))return;if(this._currentDecorations.length>0){const r=this._currentDecorations.getRange(0);if(r&&r.containsPosition(t))return}}this.clearRanges(),this._currentRequestPosition=t,this._currentRequestModelVersion=n;const s=this._currentRequestCts=new li;try{const r=new ln(!1),a=await EY(this._providers,i,t,s.token);if(this._debounceInformation.update(i,r.elapsed()),s!==this._currentRequestCts||(this._currentRequestCts=null,n!==i.getVersionId()))return;let l=[];a!=null&&a.ranges&&(l=a.ranges),this._currentWordPattern=(a==null?void 0:a.wordPattern)||this._languageWordPattern;let c=!1;for(let h=0,u=l.length;h<u;h++)if(x.containsPosition(l[h],t)){if(c=!0,h!==0){const g=l[h];l.splice(h,1),l.unshift(g)}break}if(!c){this.clearRanges();return}const d=l.map(h=>({range:h,options:Cy.DECORATION}));this._visibleContextKey.set(!0),this._currentDecorations.set(d),this._syncRangesToken++}catch(r){yr(r)||Oe(r),(this._currentRequestCts===s||!this._currentRequestCts)&&this.clearRanges()}}},Cy=Ih,Ih.ID="editor.contrib.linkedEditing",Ih.DECORATION=We.register({description:"linked-editing",stickiness:0,className:UIe}),Ih);ab=Cy=zIe([R0(1,Ee),R0(2,ge),R0(3,di),R0(4,Zo)],ab);class $Ie extends de{constructor(){super({id:"editor.action.linkedEditing",label:m("linkedEditing.label","Start Linked Editing"),alias:"Start Linked Editing",precondition:G.and(I.writable,I.hasRenameProvider),kbOpts:{kbExpr:I.editorTextFocus,primary:3132,weight:100}})}runCommand(e,t){const i=e.get(pt),[n,s]=Array.isArray(t)&&t||[void 0,void 0];return Se.isUri(n)&&A.isIPosition(s)?i.openCodeEditor({resource:n},i.getActiveCodeEditor()).then(r=>{r&&(r.setPosition(s),r.invokeWithinContext(a=>(this.reportTelemetry(a,r),this.run(a,r))))},Oe):super.runCommand(e,t)}run(e,t){const i=ab.get(t);return i?Promise.resolve(i.updateRanges(!0)):Promise.resolve()}}const jIe=$i.bindToContribution(ab.get);re(new jIe({id:"cancelLinkedEditingInput",precondition:IY,handler:o=>o.clearRanges(),kbOpts:{kbExpr:I.editorTextFocus,weight:199,primary:9,secondary:[1033]}}));function EY(o,e,t,i){const n=o.ordered(e);return n5(n.map(s=>async()=>{try{return await s.provideLinkedEditingRanges(e,t,i)}catch(r){Si(r);return}}),s=>!!s&&Qn(s==null?void 0:s.ranges))}E("editor.linkedEditingBackground",{dark:Z.fromHex("#f00").transparent(.3),light:Z.fromHex("#f00").transparent(.3),hcDark:Z.fromHex("#f00").transparent(.3),hcLight:Z.white},m("editorLinkedEditingBackground","Background color when the editor auto renames on type."));Vs("_executeLinkedEditingProvider",(o,e,t)=>{const{linkedEditingRangeProvider:i}=o.get(ge);return EY(i,e,t,ze.None)});ct(ab.ID,ab,1);Q($Ie);let KIe=class{constructor(e,t){this._link=e,this._provider=t}toJSON(){return{range:this.range,url:this.url,tooltip:this.tooltip}}get range(){return this._link.range}get url(){return this._link.url}get tooltip(){return this._link.tooltip}async resolve(e){return this._link.url?this._link.url:typeof this._provider.resolveLink=="function"?Promise.resolve(this._provider.resolveLink(this._link,e)).then(t=>(this._link=t||this._link,this._link.url?this.resolve(e):Promise.reject(new Error("missing")))):Promise.reject(new Error("missing"))}};class fk{constructor(e){this._disposables=new K;let t=[];for(const[i,n]of e){const s=i.links.map(r=>new KIe(r,n));t=fk._union(t,s),uI(i)&&this._disposables.add(i)}this.links=t}dispose(){this._disposables.dispose(),this.links.length=0}static _union(e,t){const i=[];let n,s,r,a;for(n=0,r=0,s=e.length,a=t.length;n<s&&r<a;){const l=e[n],c=t[r];if(x.areIntersectingOrTouching(l.range,c.range)){n++;continue}x.compareRangesUsingStarts(l.range,c.range)<0?(i.push(l),n++):(i.push(c),r++)}for(;n<s;n++)i.push(e[n]);for(;r<a;r++)i.push(t[r]);return i}}function NY(o,e,t){const i=[],n=o.ordered(e).reverse().map((s,r)=>Promise.resolve(s.provideLinks(e,t)).then(a=>{a&&(i[r]=[a,s])},Si));return Promise.all(n).then(()=>{const s=new fk(Kr(i));return t.isCancellationRequested?(s.dispose(),new fk([])):s})}rt.registerCommand("_executeLinkProvider",async(o,...e)=>{let[t,i]=e;mt(t instanceof Se),typeof i!="number"&&(i=0);const{linkProvider:n}=o.get(ge),s=o.get(Kt).getModel(t);if(!s)return[];const r=await NY(n,s,ze.None);if(!r)return[];for(let l=0;l<Math.min(i,r.links.length);l++)await r.links[l].resolve(ze.None);const a=r.links.slice(0);return r.dispose(),a});var qIe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},M0=function(o,e){return function(t,i){e(t,i,o)}},AF,cf;let b1=(cf=class extends F{static get(e){return e.getContribution(AF.ID)}constructor(e,t,i,n,s){super(),this.editor=e,this.openerService=t,this.notificationService=i,this.languageFeaturesService=n,this.providers=this.languageFeaturesService.linkProvider,this.debounceInformation=s.for(this.providers,"Links",{min:1e3,max:4e3}),this.computeLinks=this._register(new It(()=>this.computeLinksNow(),1e3)),this.computePromise=null,this.activeLinksList=null,this.currentOccurrences={},this.activeLinkDecorationId=null;const r=this._register(new AE(e));this._register(r.onMouseMoveOrRelevantKeyDown(([a,l])=>{this._onEditorMouseMove(a,l)})),this._register(r.onExecute(a=>{this.onEditorMouseUp(a)})),this._register(r.onCancel(a=>{this.cleanUpActiveLinkDecoration()})),this._register(e.onDidChangeConfiguration(a=>{a.hasChanged(71)&&(this.updateDecorations([]),this.stop(),this.computeLinks.schedule(0))})),this._register(e.onDidChangeModelContent(a=>{this.editor.hasModel()&&this.computeLinks.schedule(this.debounceInformation.get(this.editor.getModel()))})),this._register(e.onDidChangeModel(a=>{this.currentOccurrences={},this.activeLinkDecorationId=null,this.stop(),this.computeLinks.schedule(0)})),this._register(e.onDidChangeModelLanguage(a=>{this.stop(),this.computeLinks.schedule(0)})),this._register(this.providers.onDidChange(a=>{this.stop(),this.computeLinks.schedule(0)})),this.computeLinks.schedule(0)}async computeLinksNow(){if(!this.editor.hasModel()||!this.editor.getOption(71))return;const e=this.editor.getModel();if(!e.isTooLargeForSyncing()&&this.providers.has(e)){this.activeLinksList&&(this.activeLinksList.dispose(),this.activeLinksList=null),this.computePromise=Yi(t=>NY(this.providers,e,t));try{const t=new ln(!1);if(this.activeLinksList=await this.computePromise,this.debounceInformation.update(e,t.elapsed()),e.isDisposed())return;this.updateDecorations(this.activeLinksList.links)}catch(t){Oe(t)}finally{this.computePromise=null}}}updateDecorations(e){const t=this.editor.getOption(78)==="altKey",i=[],n=Object.keys(this.currentOccurrences);for(const r of n){const a=this.currentOccurrences[r];i.push(a.decorationId)}const s=[];if(e)for(const r of e)s.push(d_.decoration(r,t));this.editor.changeDecorations(r=>{const a=r.deltaDecorations(i,s);this.currentOccurrences={},this.activeLinkDecorationId=null;for(let l=0,c=a.length;l<c;l++){const d=new d_(e[l],a[l]);this.currentOccurrences[d.decorationId]=d}})}_onEditorMouseMove(e,t){const i=this.editor.getOption(78)==="altKey";if(this.isEnabled(e,t)){this.cleanUpActiveLinkDecoration();const n=this.getLinkOccurrence(e.target.position);n&&this.editor.changeDecorations(s=>{n.activate(s,i),this.activeLinkDecorationId=n.decorationId})}else this.cleanUpActiveLinkDecoration()}cleanUpActiveLinkDecoration(){const e=this.editor.getOption(78)==="altKey";if(this.activeLinkDecorationId){const t=this.currentOccurrences[this.activeLinkDecorationId];t&&this.editor.changeDecorations(i=>{t.deactivate(i,e)}),this.activeLinkDecorationId=null}}onEditorMouseUp(e){if(!this.isEnabled(e))return;const t=this.getLinkOccurrence(e.target.position);t&&this.openLinkOccurrence(t,e.hasSideBySideModifier,!0)}openLinkOccurrence(e,t,i=!1){if(!this.openerService)return;const{link:n}=e;n.resolve(ze.None).then(s=>{if(typeof s=="string"&&this.editor.hasModel()){const r=this.editor.getModel().uri;if(r.scheme===Fe.file&&s.startsWith(`${Fe.file}:`)){const a=Se.parse(s);if(a.scheme===Fe.file){const l=rl(a);let c=null;l.startsWith("/./")||l.startsWith("\\.\\")?c=`.${l.substr(1)}`:(l.startsWith("//./")||l.startsWith("\\\\.\\"))&&(c=`.${l.substr(2)}`),c&&(s=lle(r,c))}}}return this.openerService.open(s,{openToSide:t,fromUserGesture:i,allowContributedOpeners:!0,allowCommands:!0,fromWorkspace:!0})},s=>{const r=s instanceof Error?s.message:s;r==="invalid"?this.notificationService.warn(m("invalid.url","Failed to open this link because it is not well-formed: {0}",n.url.toString())):r==="missing"?this.notificationService.warn(m("missing.url","Failed to open this link because its target is missing.")):Oe(s)})}getLinkOccurrence(e){if(!this.editor.hasModel()||!e)return null;const t=this.editor.getModel().getDecorationsInRange({startLineNumber:e.lineNumber,startColumn:e.column,endLineNumber:e.lineNumber,endColumn:e.column},0,!0);for(const i of t){const n=this.currentOccurrences[i.id];if(n)return n}return null}isEnabled(e,t){return!!(e.target.type===6&&(e.hasTriggerModifier||t&&t.keyCodeIsTriggerKey))}stop(){var e;this.computeLinks.cancel(),this.activeLinksList&&((e=this.activeLinksList)==null||e.dispose(),this.activeLinksList=null),this.computePromise&&(this.computePromise.cancel(),this.computePromise=null)}dispose(){super.dispose(),this.stop()}},AF=cf,cf.ID="editor.linkDetector",cf);b1=AF=qIe([M0(1,zs),M0(2,Ei),M0(3,ge),M0(4,Zo)],b1);const z6={general:We.register({description:"detected-link",stickiness:1,collapseOnReplaceEdit:!0,inlineClassName:"detected-link"}),active:We.register({description:"detected-link-active",stickiness:1,collapseOnReplaceEdit:!0,inlineClassName:"detected-link-active"})};class d_{static decoration(e,t){return{range:e.range,options:d_._getOptions(e,t,!1)}}static _getOptions(e,t,i){const n={...i?z6.active:z6.general};return n.hoverMessage=GIe(e,t),n}constructor(e,t){this.link=e,this.decorationId=t}activate(e,t){e.changeDecorationOptions(this.decorationId,d_._getOptions(this.link,t,!0))}deactivate(e,t){e.changeDecorationOptions(this.decorationId,d_._getOptions(this.link,t,!1))}}function GIe(o,e){const t=o.url&&/^command:/i.test(o.url.toString()),i=o.tooltip?o.tooltip:t?m("links.navigate.executeCmd","Execute command"):m("links.navigate.follow","Follow link"),n=e?Je?m("links.navigate.kb.meta.mac","cmd + click"):m("links.navigate.kb.meta","ctrl + click"):Je?m("links.navigate.kb.alt.mac","option + click"):m("links.navigate.kb.alt","alt + click");if(o.url){let s="";if(/^command:/i.test(o.url.toString())){const a=o.url.toString().match(/^command:([^?#]+)/);if(a){const l=a[1];s=m("tooltip.explanation","Execute command {0}",l)}}return new Pn("",!0).appendLink(o.url.toString(!0).replace(/ /g,"%20"),i,s).appendMarkdown(` (${n})`)}else return new Pn().appendText(`${i} (${n})`)}class ZIe extends de{constructor(){super({id:"editor.action.openLink",label:m("label","Open Link"),alias:"Open Link",precondition:void 0})}run(e,t){const i=b1.get(t);if(!i||!t.hasModel())return;const n=t.getSelections();for(const s of n){const r=i.getLinkOccurrence(s.getEndPosition());r&&i.openLinkOccurrence(r,!1)}}}ct(b1.ID,b1,1);Q(ZIe);const xB=class xB extends F{constructor(e){super(),this._editor=e,this._register(this._editor.onMouseDown(t=>{const i=this._editor.getOption(118);i>=0&&t.target.type===6&&t.target.position.column>=i&&this._editor.updateOptions({stopRenderingLineAfter:-1})}))}};xB.ID="editor.contrib.longLinesHelper";let mk=xB;ct(mk.ID,mk,2);const YIe=E("editor.wordHighlightBackground",{dark:"#575757B8",light:"#57575740",hcDark:null,hcLight:null},m("wordHighlight","Background color of a symbol during read-access, like reading a variable. The color must not be opaque so as not to hide underlying decorations."),!0);E("editor.wordHighlightStrongBackground",{dark:"#004972B8",light:"#0e639c40",hcDark:null,hcLight:null},m("wordHighlightStrong","Background color of a symbol during write-access, like writing to a variable. The color must not be opaque so as not to hide underlying decorations."),!0);E("editor.wordHighlightTextBackground",YIe,m("wordHighlightText","Background color of a textual occurrence for a symbol. The color must not be opaque so as not to hide underlying decorations."),!0);const XIe=E("editor.wordHighlightBorder",{light:null,dark:null,hcDark:Xt,hcLight:Xt},m("wordHighlightBorder","Border color of a symbol during read-access, like reading a variable."));E("editor.wordHighlightStrongBorder",{light:null,dark:null,hcDark:Xt,hcLight:Xt},m("wordHighlightStrongBorder","Border color of a symbol during write-access, like writing to a variable."));E("editor.wordHighlightTextBorder",XIe,m("wordHighlightTextBorder","Border color of a textual occurrence for a symbol."));const QIe=E("editorOverviewRuler.wordHighlightForeground","#A0A0A0CC",m("overviewRulerWordHighlightForeground","Overview ruler marker color for symbol highlights. The color must not be opaque so as not to hide underlying decorations."),!0),JIe=E("editorOverviewRuler.wordHighlightStrongForeground","#C0A0C0CC",m("overviewRulerWordHighlightStrongForeground","Overview ruler marker color for write-access symbol highlights. The color must not be opaque so as not to hide underlying decorations."),!0),eEe=E("editorOverviewRuler.wordHighlightTextForeground",B$,m("overviewRulerWordHighlightTextForeground","Overview ruler marker color of a textual occurrence for a symbol. The color must not be opaque so as not to hide underlying decorations."),!0),tEe=We.register({description:"word-highlight-strong",stickiness:1,className:"wordHighlightStrong",overviewRuler:{color:mi(JIe),position:Ho.Center},minimap:{color:mi(VI),position:1}}),iEe=We.register({description:"word-highlight-text",stickiness:1,className:"wordHighlightText",overviewRuler:{color:mi(eEe),position:Ho.Center},minimap:{color:mi(VI),position:1}}),nEe=We.register({description:"selection-highlight-overview",stickiness:1,className:"selectionHighlight",overviewRuler:{color:mi(B$),position:Ho.Center},minimap:{color:mi(VI),position:1}}),sEe=We.register({description:"selection-highlight",stickiness:1,className:"selectionHighlight"}),oEe=We.register({description:"word-highlight",stickiness:1,className:"wordHighlight",overviewRuler:{color:mi(QIe),position:Ho.Center},minimap:{color:mi(VI),position:1}});function rEe(o){return o===x_.Write?tEe:o===x_.Text?iEe:oEe}function aEe(o){return o?sEe:nEe}Lr((o,e)=>{const t=o.getColor(N5);t&&e.addRule(`.monaco-editor .selectionHighlight { background-color: ${t.transparent(.5)}; }`)});var lEe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},cEe=function(o,e){return function(t,i){e(t,i,o)}},PF;function lu(o,e){const t=e.filter(i=>!o.find(n=>n.equals(i)));if(t.length>=1){const i=t.map(s=>`line ${s.viewState.position.lineNumber} column ${s.viewState.position.column}`).join(", "),n=t.length===1?m("cursorAdded","Cursor added: {0}",i):m("cursorsAdded","Cursors added: {0}",i);Hl(n)}}class dEe extends de{constructor(){super({id:"editor.action.insertCursorAbove",label:m("mutlicursor.insertAbove","Add Cursor Above"),alias:"Add Cursor Above",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:2576,linux:{primary:1552,secondary:[3088]},weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"3_multi",title:m({key:"miInsertCursorAbove",comment:["&& denotes a mnemonic"]},"&&Add Cursor Above"),order:2}})}run(e,t,i){if(!t.hasModel())return;let n=!0;i&&i.logicalLine===!1&&(n=!1);const s=t._getViewModel();if(s.cursorConfig.readOnly)return;s.model.pushStackElement();const r=s.getCursorStates();s.setCursorStates(i.source,3,un.addCursorUp(s,r,n)),s.revealTopMostCursor(i.source),lu(r,s.getCursorStates())}}class hEe extends de{constructor(){super({id:"editor.action.insertCursorBelow",label:m("mutlicursor.insertBelow","Add Cursor Below"),alias:"Add Cursor Below",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:2578,linux:{primary:1554,secondary:[3090]},weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"3_multi",title:m({key:"miInsertCursorBelow",comment:["&& denotes a mnemonic"]},"A&&dd Cursor Below"),order:3}})}run(e,t,i){if(!t.hasModel())return;let n=!0;i&&i.logicalLine===!1&&(n=!1);const s=t._getViewModel();if(s.cursorConfig.readOnly)return;s.model.pushStackElement();const r=s.getCursorStates();s.setCursorStates(i.source,3,un.addCursorDown(s,r,n)),s.revealBottomMostCursor(i.source),lu(r,s.getCursorStates())}}class uEe extends de{constructor(){super({id:"editor.action.insertCursorAtEndOfEachLineSelected",label:m("mutlicursor.insertAtEndOfEachLineSelected","Add Cursors to Line Ends"),alias:"Add Cursors to Line Ends",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:1575,weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"3_multi",title:m({key:"miInsertCursorAtEndOfEachLineSelected",comment:["&& denotes a mnemonic"]},"Add C&&ursors to Line Ends"),order:4}})}getCursorsForSelection(e,t,i){if(!e.isEmpty()){for(let n=e.startLineNumber;n<e.endLineNumber;n++){const s=t.getLineMaxColumn(n);i.push(new pe(n,s,n,s))}e.endColumn>1&&i.push(new pe(e.endLineNumber,e.endColumn,e.endLineNumber,e.endColumn))}}run(e,t){if(!t.hasModel())return;const i=t.getModel(),n=t.getSelections(),s=t._getViewModel(),r=s.getCursorStates(),a=[];n.forEach(l=>this.getCursorsForSelection(l,i,a)),a.length>0&&t.setSelections(a),lu(r,s.getCursorStates())}}class gEe extends de{constructor(){super({id:"editor.action.addCursorsToBottom",label:m("mutlicursor.addCursorsToBottom","Add Cursors To Bottom"),alias:"Add Cursors To Bottom",precondition:void 0})}run(e,t){if(!t.hasModel())return;const i=t.getSelections(),n=t.getModel().getLineCount(),s=[];for(let l=i[0].startLineNumber;l<=n;l++)s.push(new pe(l,i[0].startColumn,l,i[0].endColumn));const r=t._getViewModel(),a=r.getCursorStates();s.length>0&&t.setSelections(s),lu(a,r.getCursorStates())}}class fEe extends de{constructor(){super({id:"editor.action.addCursorsToTop",label:m("mutlicursor.addCursorsToTop","Add Cursors To Top"),alias:"Add Cursors To Top",precondition:void 0})}run(e,t){if(!t.hasModel())return;const i=t.getSelections(),n=[];for(let a=i[0].startLineNumber;a>=1;a--)n.push(new pe(a,i[0].startColumn,a,i[0].endColumn));const s=t._getViewModel(),r=s.getCursorStates();n.length>0&&t.setSelections(n),lu(r,s.getCursorStates())}}class A0{constructor(e,t,i){this.selections=e,this.revealRange=t,this.revealScrollType=i}}class C1{static create(e,t){if(!e.hasModel())return null;const i=t.getState();if(!e.hasTextFocus()&&i.isRevealed&&i.searchString.length>0)return new C1(e,t,!1,i.searchString,i.wholeWord,i.matchCase,null);let n=!1,s,r;const a=e.getSelections();a.length===1&&a[0].isEmpty()?(n=!0,s=!0,r=!0):(s=i.wholeWord,r=i.matchCase);const l=e.getSelection();let c,d=null;if(l.isEmpty()){const h=e.getConfiguredWordAtPosition(l.getStartPosition());if(!h)return null;c=h.word,d=new pe(l.startLineNumber,h.startColumn,l.startLineNumber,h.endColumn)}else c=e.getModel().getValueInRange(l).replace(/\r\n/g,`
`);return new C1(e,t,n,c,s,r,d)}constructor(e,t,i,n,s,r,a){this._editor=e,this.findController=t,this.isDisconnectedFromFindController=i,this.searchText=n,this.wholeWord=s,this.matchCase=r,this.currentMatch=a}addSelectionToNextFindMatch(){if(!this._editor.hasModel())return null;const e=this._getNextMatch();if(!e)return null;const t=this._editor.getSelections();return new A0(t.concat(e),e,0)}moveSelectionToNextFindMatch(){if(!this._editor.hasModel())return null;const e=this._getNextMatch();if(!e)return null;const t=this._editor.getSelections();return new A0(t.slice(0,t.length-1).concat(e),e,0)}_getNextMatch(){if(!this._editor.hasModel())return null;if(this.currentMatch){const n=this.currentMatch;return this.currentMatch=null,n}this.findController.highlightFindOptions();const e=this._editor.getSelections(),t=e[e.length-1],i=this._editor.getModel().findNextMatch(this.searchText,t.getEndPosition(),!1,this.matchCase,this.wholeWord?this._editor.getOption(132):null,!1);return i?new pe(i.range.startLineNumber,i.range.startColumn,i.range.endLineNumber,i.range.endColumn):null}addSelectionToPreviousFindMatch(){if(!this._editor.hasModel())return null;const e=this._getPreviousMatch();if(!e)return null;const t=this._editor.getSelections();return new A0(t.concat(e),e,0)}moveSelectionToPreviousFindMatch(){if(!this._editor.hasModel())return null;const e=this._getPreviousMatch();if(!e)return null;const t=this._editor.getSelections();return new A0(t.slice(0,t.length-1).concat(e),e,0)}_getPreviousMatch(){if(!this._editor.hasModel())return null;if(this.currentMatch){const n=this.currentMatch;return this.currentMatch=null,n}this.findController.highlightFindOptions();const e=this._editor.getSelections(),t=e[e.length-1],i=this._editor.getModel().findPreviousMatch(this.searchText,t.getStartPosition(),!1,this.matchCase,this.wholeWord?this._editor.getOption(132):null,!1);return i?new pe(i.range.startLineNumber,i.range.startColumn,i.range.endLineNumber,i.range.endColumn):null}selectAll(e){if(!this._editor.hasModel())return[];this.findController.highlightFindOptions();const t=this._editor.getModel();return e?t.findMatches(this.searchText,e,!1,this.matchCase,this.wholeWord?this._editor.getOption(132):null,!1,1073741824):t.findMatches(this.searchText,!0,!1,this.matchCase,this.wholeWord?this._editor.getOption(132):null,!1,1073741824)}}const KD=class KD extends F{static get(e){return e.getContribution(KD.ID)}constructor(e){super(),this._sessionDispose=this._register(new K),this._editor=e,this._ignoreSelectionChange=!1,this._session=null}dispose(){this._endSession(),super.dispose()}_beginSessionIfNeeded(e){if(!this._session){const t=C1.create(this._editor,e);if(!t)return;this._session=t;const i={searchString:this._session.searchText};this._session.isDisconnectedFromFindController&&(i.wholeWordOverride=1,i.matchCaseOverride=1,i.isRegexOverride=2),e.getState().change(i,!1),this._sessionDispose.add(this._editor.onDidChangeCursorSelection(n=>{this._ignoreSelectionChange||this._endSession()})),this._sessionDispose.add(this._editor.onDidBlurEditorText(()=>{this._endSession()})),this._sessionDispose.add(e.getState().onFindReplaceStateChange(n=>{(n.matchCase||n.wholeWord)&&this._endSession()}))}}_endSession(){if(this._sessionDispose.clear(),this._session&&this._session.isDisconnectedFromFindController){const e={wholeWordOverride:0,matchCaseOverride:0,isRegexOverride:0};this._session.findController.getState().change(e,!1)}this._session=null}_setSelections(e){this._ignoreSelectionChange=!0,this._editor.setSelections(e),this._ignoreSelectionChange=!1}_expandEmptyToWord(e,t){if(!t.isEmpty())return t;const i=this._editor.getConfiguredWordAtPosition(t.getStartPosition());return i?new pe(t.startLineNumber,i.startColumn,t.startLineNumber,i.endColumn):t}_applySessionResult(e){e&&(this._setSelections(e.selections),e.revealRange&&this._editor.revealRangeInCenterIfOutsideViewport(e.revealRange,e.revealScrollType))}getSession(e){return this._session}addSelectionToNextFindMatch(e){if(this._editor.hasModel()){if(!this._session){const t=this._editor.getSelections();if(t.length>1){const n=e.getState().matchCase;if(!TY(this._editor.getModel(),t,n)){const r=this._editor.getModel(),a=[];for(let l=0,c=t.length;l<c;l++)a[l]=this._expandEmptyToWord(r,t[l]);this._editor.setSelections(a);return}}}this._beginSessionIfNeeded(e),this._session&&this._applySessionResult(this._session.addSelectionToNextFindMatch())}}addSelectionToPreviousFindMatch(e){this._beginSessionIfNeeded(e),this._session&&this._applySessionResult(this._session.addSelectionToPreviousFindMatch())}moveSelectionToNextFindMatch(e){this._beginSessionIfNeeded(e),this._session&&this._applySessionResult(this._session.moveSelectionToNextFindMatch())}moveSelectionToPreviousFindMatch(e){this._beginSessionIfNeeded(e),this._session&&this._applySessionResult(this._session.moveSelectionToPreviousFindMatch())}selectAll(e){if(!this._editor.hasModel())return;let t=null;const i=e.getState();if(i.isRevealed&&i.searchString.length>0&&i.isRegex){const n=this._editor.getModel();i.searchScope?t=n.findMatches(i.searchString,i.searchScope,i.isRegex,i.matchCase,i.wholeWord?this._editor.getOption(132):null,!1,1073741824):t=n.findMatches(i.searchString,!0,i.isRegex,i.matchCase,i.wholeWord?this._editor.getOption(132):null,!1,1073741824)}else{if(this._beginSessionIfNeeded(e),!this._session)return;t=this._session.selectAll(i.searchScope)}if(t.length>0){const n=this._editor.getSelection();for(let s=0,r=t.length;s<r;s++){const a=t[s];if(a.range.intersectRanges(n)){t[s]=t[0],t[0]=a;break}}this._setSelections(t.map(s=>new pe(s.range.startLineNumber,s.range.startColumn,s.range.endLineNumber,s.range.endColumn)))}}};KD.ID="editor.contrib.multiCursorController";let lb=KD;class Fb extends de{run(e,t){const i=lb.get(t);if(!i)return;const n=t._getViewModel();if(n){const s=n.getCursorStates(),r=Bs.get(t);if(r)this._run(i,r);else{const a=e.get(we).createInstance(Bs,t);this._run(i,a),a.dispose()}lu(s,n.getCursorStates())}}}class mEe extends Fb{constructor(){super({id:"editor.action.addSelectionToNextFindMatch",label:m("addSelectionToNextFindMatch","Add Selection To Next Find Match"),alias:"Add Selection To Next Find Match",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:2082,weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"3_multi",title:m({key:"miAddSelectionToNextFindMatch",comment:["&& denotes a mnemonic"]},"Add &&Next Occurrence"),order:5}})}_run(e,t){e.addSelectionToNextFindMatch(t)}}class pEe extends Fb{constructor(){super({id:"editor.action.addSelectionToPreviousFindMatch",label:m("addSelectionToPreviousFindMatch","Add Selection To Previous Find Match"),alias:"Add Selection To Previous Find Match",precondition:void 0,menuOpts:{menuId:me.MenubarSelectionMenu,group:"3_multi",title:m({key:"miAddSelectionToPreviousFindMatch",comment:["&& denotes a mnemonic"]},"Add P&&revious Occurrence"),order:6}})}_run(e,t){e.addSelectionToPreviousFindMatch(t)}}class _Ee extends Fb{constructor(){super({id:"editor.action.moveSelectionToNextFindMatch",label:m("moveSelectionToNextFindMatch","Move Last Selection To Next Find Match"),alias:"Move Last Selection To Next Find Match",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:Hi(2089,2082),weight:100}})}_run(e,t){e.moveSelectionToNextFindMatch(t)}}class bEe extends Fb{constructor(){super({id:"editor.action.moveSelectionToPreviousFindMatch",label:m("moveSelectionToPreviousFindMatch","Move Last Selection To Previous Find Match"),alias:"Move Last Selection To Previous Find Match",precondition:void 0})}_run(e,t){e.moveSelectionToPreviousFindMatch(t)}}class CEe extends Fb{constructor(){super({id:"editor.action.selectHighlights",label:m("selectAllOccurrencesOfFindMatch","Select All Occurrences of Find Match"),alias:"Select All Occurrences of Find Match",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:3114,weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"3_multi",title:m({key:"miSelectHighlights",comment:["&& denotes a mnemonic"]},"Select All &&Occurrences"),order:7}})}_run(e,t){e.selectAll(t)}}class vEe extends Fb{constructor(){super({id:"editor.action.changeAll",label:m("changeAll.label","Change All Occurrences"),alias:"Change All Occurrences",precondition:G.and(I.writable,I.editorTextFocus),kbOpts:{kbExpr:I.editorTextFocus,primary:2108,weight:100},contextMenuOpts:{group:"1_modification",order:1.2}})}_run(e,t){e.selectAll(t)}}class wEe{constructor(e,t,i,n,s){this._model=e,this._searchText=t,this._matchCase=i,this._wordSeparators=n,this._modelVersionId=this._model.getVersionId(),this._cachedFindMatches=null,s&&this._model===s._model&&this._searchText===s._searchText&&this._matchCase===s._matchCase&&this._wordSeparators===s._wordSeparators&&this._modelVersionId===s._modelVersionId&&(this._cachedFindMatches=s._cachedFindMatches)}findMatches(){return this._cachedFindMatches===null&&(this._cachedFindMatches=this._model.findMatches(this._searchText,!0,!1,this._matchCase,this._wordSeparators,!1).map(e=>e.range),this._cachedFindMatches.sort(x.compareRangesUsingStarts)),this._cachedFindMatches}}var df;let pk=(df=class extends F{constructor(e,t){super(),this._languageFeaturesService=t,this.editor=e,this._isEnabled=e.getOption(109),this._decorations=e.createDecorationsCollection(),this.updateSoon=this._register(new It(()=>this._update(),300)),this.state=null,this._register(e.onDidChangeConfiguration(n=>{this._isEnabled=e.getOption(109)})),this._register(e.onDidChangeCursorSelection(n=>{this._isEnabled&&(n.selection.isEmpty()?n.reason===3?(this.state&&this._setState(null),this.updateSoon.schedule()):this._setState(null):this._update())})),this._register(e.onDidChangeModel(n=>{this._setState(null)})),this._register(e.onDidChangeModelContent(n=>{this._isEnabled&&this.updateSoon.schedule()}));const i=Bs.get(e);i&&this._register(i.getState().onFindReplaceStateChange(n=>{this._update()})),this.updateSoon.schedule()}_update(){this._setState(PF._createState(this.state,this._isEnabled,this.editor))}static _createState(e,t,i){if(!t||!i.hasModel())return null;const n=i.getSelection();if(n.startLineNumber!==n.endLineNumber)return null;const s=lb.get(i);if(!s)return null;const r=Bs.get(i);if(!r)return null;let a=s.getSession(r);if(!a){const d=i.getSelections();if(d.length>1){const u=r.getState().matchCase;if(!TY(i.getModel(),d,u))return null}a=C1.create(i,r)}if(!a||a.currentMatch||/^[ \t]+$/.test(a.searchText)||a.searchText.length>200)return null;const l=r.getState(),c=l.matchCase;if(l.isRevealed){let d=l.searchString;c||(d=d.toLowerCase());let h=a.searchText;if(c||(h=h.toLowerCase()),d===h&&a.matchCase===l.matchCase&&a.wholeWord===l.wholeWord&&!l.isRegex)return null}return new wEe(i.getModel(),a.searchText,a.matchCase,a.wholeWord?i.getOption(132):null,e)}_setState(e){if(this.state=e,!this.state){this._decorations.clear();return}if(!this.editor.hasModel())return;const t=this.editor.getModel();if(t.isTooLargeForTokenization())return;const i=this.state.findMatches(),n=this.editor.getSelections();n.sort(x.compareRangesUsingStarts);const s=[];for(let c=0,d=0,h=i.length,u=n.length;c<h;){const g=i[c];if(d>=u)s.push(g),c++;else{const f=x.compareRangesUsingStarts(g,n[d]);f<0?((n[d].isEmpty()||!x.areIntersecting(g,n[d]))&&s.push(g),c++):(f>0||c++,d++)}}const r=this.editor.getOption(81)!=="off",a=this._languageFeaturesService.documentHighlightProvider.has(t)&&r,l=s.map(c=>({range:c,options:aEe(a)}));this._decorations.set(l)}dispose(){this._setState(null),super.dispose()}},PF=df,df.ID="editor.contrib.selectionHighlighter",df);pk=PF=lEe([cEe(1,ge)],pk);function TY(o,e,t){const i=U6(o,e[0],!t);for(let n=1,s=e.length;n<s;n++){const r=e[n];if(r.isEmpty())return!1;const a=U6(o,r,!t);if(i!==a)return!1}return!0}function U6(o,e,t){const i=o.getValueInRange(e);return t?i.toLowerCase():i}class SEe extends de{constructor(){super({id:"editor.action.focusNextCursor",label:m("mutlicursor.focusNextCursor","Focus Next Cursor"),metadata:{description:m("mutlicursor.focusNextCursor.description","Focuses the next cursor"),args:[]},alias:"Focus Next Cursor",precondition:void 0})}run(e,t,i){if(!t.hasModel())return;const n=t._getViewModel();if(n.cursorConfig.readOnly)return;n.model.pushStackElement();const s=Array.from(n.getCursorStates()),r=s.shift();r&&(s.push(r),n.setCursorStates(i.source,3,s),n.revealPrimaryCursor(i.source,!0),lu(s,n.getCursorStates()))}}class yEe extends de{constructor(){super({id:"editor.action.focusPreviousCursor",label:m("mutlicursor.focusPreviousCursor","Focus Previous Cursor"),metadata:{description:m("mutlicursor.focusPreviousCursor.description","Focuses the previous cursor"),args:[]},alias:"Focus Previous Cursor",precondition:void 0})}run(e,t,i){if(!t.hasModel())return;const n=t._getViewModel();if(n.cursorConfig.readOnly)return;n.model.pushStackElement();const s=Array.from(n.getCursorStates()),r=s.pop();r&&(s.unshift(r),n.setCursorStates(i.source,3,s),n.revealPrimaryCursor(i.source,!0),lu(s,n.getCursorStates()))}}ct(lb.ID,lb,4);ct(pk.ID,pk,1);Q(dEe);Q(hEe);Q(uEe);Q(mEe);Q(pEe);Q(_Ee);Q(bEe);Q(CEe);Q(vEe);Q(gEe);Q(fEe);Q(SEe);Q(yEe);const LEe="editor.action.inlineEdit.accept",xEe="editor.action.inlineEdit.reject",kEe="editor.action.inlineEdit.jumpTo",DEe="editor.action.inlineEdit.jumpBack";var IEe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},EEe=function(o,e){return function(t,i){e(t,i,o)}};const eR="inline-edit";let OF=class extends F{constructor(e,t,i){super(),this.editor=e,this.model=t,this.languageService=i,this.isDisposed=Ke(this,!1),this.currentTextModel=xt(this,this.editor.onDidChangeModel,()=>this.editor.getModel()),this.uiState=be(this,n=>{var C;if(this.isDisposed.read(n))return;const s=this.currentTextModel.read(n);if(s!==this.model.targetTextModel.read(n))return;const r=this.model.ghostText.read(n);if(!r)return;let a=(C=this.model.range)==null?void 0:C.read(n);a&&a.startLineNumber===a.endLineNumber&&a.startColumn===a.endColumn&&(a=void 0);const l=(a?a.startLineNumber===a.endLineNumber:!0)&&r.parts.length===1&&r.parts[0].lines.length===1,c=r.parts.length===1&&r.parts[0].lines.every(v=>v.length===0),d=[],h=[];function u(v,w){if(h.length>0){const S=h[h.length-1];w&&S.decorations.push(new hs(S.content.length+1,S.content.length+1+v[0].length,w,0)),S.content+=v[0],v=v.slice(1)}for(const S of v)h.push({content:S,decorations:w?[new hs(1,S.length+1,w,0)]:[]})}const g=s.getLineContent(r.lineNumber);let f,p=0;if(!c&&(l||!a)){for(const v of r.parts){let w=v.lines;a&&!l&&(u(w,eR),w=[]),f===void 0?(d.push({column:v.column,text:w[0],preview:v.preview}),w=w.slice(1)):u([g.substring(p,v.column-1)],void 0),w.length>0&&(u(w,eR),f===void 0&&v.column<=g.length&&(f=v.column)),p=v.column-1}f!==void 0&&u([g.substring(p)],void 0)}const _=f!==void 0?new uY(f,g.length+1):void 0,b=l||!a?r.lineNumber:a.endLineNumber-1;return{inlineTexts:d,additionalLines:h,hiddenRange:_,lineNumber:b,additionalReservedLineCount:this.model.minReservedLineCount.read(n),targetTextModel:s,range:a,isSingleLine:l,isPureRemove:c}}),this.decorations=be(this,n=>{const s=this.uiState.read(n);if(!s)return[];const r=[];if(s.hiddenRange&&r.push({range:s.hiddenRange.toRange(s.lineNumber),options:{inlineClassName:"inline-edit-hidden",description:"inline-edit-hidden"}}),s.range){const a=[];if(s.isSingleLine)a.push(s.range);else if(!s.isPureRemove){const l=s.range.endLineNumber-s.range.startLineNumber;for(let c=0;c<l;c++){const d=s.range.startLineNumber+c,h=s.targetTextModel.getLineFirstNonWhitespaceColumn(d),u=s.targetTextModel.getLineLastNonWhitespaceColumn(d),g=new x(d,h,d,u);a.push(g)}}for(const l of a)r.push({range:l,options:G_})}if(s.range&&!s.isSingleLine&&s.isPureRemove){const a=new x(s.range.startLineNumber,1,s.range.endLineNumber-1,1);r.push({range:a,options:Yw})}for(const a of s.inlineTexts)r.push({range:x.fromPositions(new A(s.lineNumber,a.column)),options:{description:eR,after:{content:a.text,inlineClassName:a.preview?"inline-edit-decoration-preview":"inline-edit-decoration",cursorStops:zr.Left},showIfCollapsed:!0}});return r}),this._register(ve(()=>{this.isDisposed.set(!0,void 0)})),this._register(gY(this.editor,this.decorations))}};OF=IEe([EEe(2,ni)],OF);var aB=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},wl=function(o,e){return function(t,i){e(t,i,o)}},vy;let FF=class extends F{constructor(e,t,i){super(),this.editor=e,this.model=t,this.instantiationService=i,this.alwaysShowToolbar=xt(this,this.editor.onDidChangeConfiguration,()=>this.editor.getOption(63).showToolbar==="always"),this.sessionPosition=void 0,this.position=be(this,n=>{var l,c;const s=(l=this.model.read(n))==null?void 0:l.model.ghostText.read(n);if(!this.alwaysShowToolbar.read(n)||!s||s.parts.length===0)return this.sessionPosition=void 0,null;const r=s.parts[0].column;this.sessionPosition&&this.sessionPosition.lineNumber!==s.lineNumber&&(this.sessionPosition=void 0);const a=new A(s.lineNumber,Math.min(r,((c=this.sessionPosition)==null?void 0:c.column)??Number.MAX_SAFE_INTEGER));return this.sessionPosition=a,a}),this._register(Ps((n,s)=>{if(!this.model.read(n)||!this.alwaysShowToolbar.read(n))return;const a=s.add(this.instantiationService.createInstance(WF,this.editor,!0,this.position));e.addContentWidget(a),s.add(ve(()=>e.removeContentWidget(a)))}))}};FF=aB([wl(2,we)],FF);var Eh;let WF=(Eh=class extends F{constructor(e,t,i,n,s,r){super(),this.editor=e,this.withBorder=t,this._position=i,this._contextKeyService=s,this._menuService=r,this.id=`InlineEditHintsContentWidget${vy.id++}`,this.allowEditorOverflow=!0,this.suppressMouseDown=!1,this.nodes=Xe("div.inlineEditHints",{className:this.withBorder?".withBorder":""},[Xe("div@toolBar")]),this.inlineCompletionsActionsMenus=this._register(this._menuService.createMenu(me.InlineEditActions,this._contextKeyService)),this.toolBar=this._register(n.createInstance(BF,this.nodes.toolBar,this.editor,me.InlineEditToolbar,{menuOptions:{renderShortTitle:!0},toolbarOptions:{primaryGroup:a=>a.startsWith("primary")},actionViewItemProvider:(a,l)=>{if(a instanceof ao)return n.createInstance(NEe,a,void 0)},telemetrySource:"InlineEditToolbar"})),this._register(this.toolBar.onDidChangeDropdownVisibility(a=>{vy._dropDownVisible=a})),this._register(Ae(a=>{this._position.read(a),this.editor.layoutContentWidget(this)})),this._register(Ae(a=>{const l=[];for(const[c,d]of this.inlineCompletionsActionsMenus.getActions())for(const h of d)h instanceof ao&&l.push(h);l.length>0&&l.unshift(new Qi),this.toolBar.setAdditionalSecondaryActions(l)}))}getId(){return this.id}getDomNode(){return this.nodes.root}getPosition(){return{position:this._position.get(),preference:[1,2],positionAffinity:3}}},vy=Eh,Eh._dropDownVisible=!1,Eh.id=0,Eh);WF=vy=aB([wl(3,we),wl(4,Ee),wl(5,Go)],WF);class NEe extends $h{updateLabel(){const e=this._keybindingService.lookupKeybinding(this._action.id,this._contextKeyService);if(!e)return super.updateLabel();if(this.label){const t=Xe("div.keybinding").root;this._register(new Ib(t,so,{disableTitle:!0,...KK})).set(e),this.label.textContent=this._action.label,this.label.appendChild(t),this.label.classList.add("inlineEditStatusBarItemLabel")}}updateTooltip(){}}let BF=class extends Qw{constructor(e,t,i,n,s,r,a,l,c,d){super(e,{resetMenu:i,...n},s,r,a,l,c,d),this.editor=t,this.menuId=i,this.options2=n,this.menuService=s,this.contextKeyService=r,this.menu=this._store.add(this.menuService.createMenu(this.menuId,this.contextKeyService,{emitEventsForSubmenuChanges:!0})),this.additionalActions=[],this.prependedPrimaryActions=[],this._store.add(this.menu.onDidChange(()=>this.updateToolbar())),this._store.add(this.editor.onDidChangeCursorPosition(()=>this.updateToolbar())),this.updateToolbar()}updateToolbar(){var i,n,s,r,a,l,c;const e=[],t=[];sE(this.menu,(i=this.options2)==null?void 0:i.menuOptions,{primary:e,secondary:t},(s=(n=this.options2)==null?void 0:n.toolbarOptions)==null?void 0:s.primaryGroup,(a=(r=this.options2)==null?void 0:r.toolbarOptions)==null?void 0:a.shouldInlineSubmenu,(c=(l=this.options2)==null?void 0:l.toolbarOptions)==null?void 0:c.useSeparatorsInPrimaryActions),t.push(...this.additionalActions),e.unshift(...this.prependedPrimaryActions),this.setActions(e,t)}setAdditionalSecondaryActions(e){ii(this.additionalActions,e,(t,i)=>t===i)||(this.additionalActions=e,this.updateToolbar())}};BF=aB([wl(4,Go),wl(5,Ee),wl(6,mo),wl(7,yt),wl(8,Ht),wl(9,Bn)],BF);var RY=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},wy=function(o,e){return function(t,i){e(t,i,o)}},WC,HF;function*TEe(o,e,t=1){e===void 0&&([e,o]=[o,0]);for(let i=o;i<e;i+=t)yield i}function tR(o){var i;const e=((i=o[0].match(/^\s*/))==null?void 0:i[0])??"",t=e.length;return{text:o.map(n=>n.replace(new RegExp("^"+e),"")),shift:t}}var hf;let VF=(hf=class extends F{static _createUniqueUri(){return Se.from({scheme:"inline-edit-widget",path:new Date().toString()+String(WC._modelId++)})}constructor(e,t,i,n,s){super(),this._editor=e,this._model=t,this._instantiationService=i,this._diffProviderFactoryService=n,this._modelService=s,this._position=be(this,r=>{const a=this._model.read(r);if(!a||a.text.length===0||a.range.startLineNumber===a.range.endLineNumber&&!(a.range.startColumn===a.range.endColumn&&a.range.startColumn===1))return null;const l=this._editor.getModel();if(!l)return null;const c=Array.from(TEe(a.range.startLineNumber,a.range.endLineNumber+1)),d=c.map(p=>l.getLineLastNonWhitespaceColumn(p)),h=Math.max(...d),u=c[d.indexOf(h)],g=new A(u,h);return{top:a.range.startLineNumber,left:g}}),this._text=be(this,r=>{const a=this._model.read(r);if(!a)return{text:"",shift:0};const l=tR(a.text.split(`
`));return{text:l.text.join(`
`),shift:l.shift}}),this._originalModel=Gn(()=>this._modelService.createModel("",null,WC._createUniqueUri())).keepObserved(this._store),this._modifiedModel=Gn(()=>this._modelService.createModel("",null,WC._createUniqueUri())).keepObserved(this._store),this._diff=be(this,r=>{var a,l;return(l=(a=this._diffPromise.read(r))==null?void 0:a.promiseResult.read(r))==null?void 0:l.data}),this._diffPromise=be(this,r=>{const a=this._model.read(r);if(!a)return;const l=this._editor.getModel();if(!l)return;const c=tR(l.getValueInRange(a.range).split(`
`)).text.join(`
`),d=tR(a.text.split(`
`)).text.join(`
`);this._originalModel.get().setValue(c),this._modifiedModel.get().setValue(d);const h=this._diffProviderFactoryService.createDiffProvider({diffAlgorithm:"advanced"});return tE.fromFn(async()=>{const u=await h.computeDiff(this._originalModel.get(),this._modifiedModel.get(),{computeMoves:!1,ignoreTrimWhitespace:!1,maxComputationTimeMs:1e3},ze.None);if(!u.identical)return u.changes})}),this._register(Ps((r,a)=>{if(!this._model.read(r)||this._position.get()===null)return;const c=a.add(this._instantiationService.createInstance(zF,this._editor,this._position,this._text.map(d=>d.text),this._text.map(d=>d.shift),this._diff));e.addOverlayWidget(c),a.add(ve(()=>e.removeOverlayWidget(c)))}))}},WC=hf,hf._modelId=0,hf);VF=WC=RY([wy(2,we),wy(3,oS),wy(4,Kt)],VF);var uf;let zF=(uf=class extends F{constructor(e,t,i,n,s,r){var a;super(),this._editor=e,this._position=t,this._text=i,this._shift=n,this._diff=s,this._instantiationService=r,this.id=`InlineEditSideBySideContentWidget${HF.id++}`,this.allowEditorOverflow=!1,this._nodes=ne("div.inlineEditSideBySide",void 0),this._scrollChanged=xn("editor.onDidScrollChange",this._editor.onDidScrollChange),this._previewEditor=this._register(this._instantiationService.createInstance(Jr,this._nodes,{glyphMargin:!1,lineNumbers:"off",minimap:{enabled:!1},guides:{indentation:!1,bracketPairs:!1,bracketPairsHorizontal:!1,highlightActiveIndentation:!1},folding:!1,selectOnLineNumbers:!1,selectionHighlight:!1,columnSelection:!1,overviewRulerBorder:!1,overviewRulerLanes:0,lineDecorationsWidth:0,lineNumbersMinChars:0,scrollbar:{vertical:"hidden",horizontal:"hidden",alwaysConsumeMouseWheel:!1,handleMouseWheel:!1},readOnly:!0,wordWrap:"off",wordWrapOverride1:"off",wordWrapOverride2:"off",wrappingIndent:"none",wrappingStrategy:void 0},{contributions:[],isSimpleWidget:!0},this._editor)),this._previewEditorObs=no(this._previewEditor),this._editorObs=no(this._editor),this._previewTextModel=this._register(this._instantiationService.createInstance(Da,"",((a=this._editor.getModel())==null?void 0:a.getLanguageId())??ms,Da.DEFAULT_CREATION_OPTIONS,null)),this._setText=be(l=>{const c=this._text.read(l);c&&this._previewTextModel.setValue(c)}).recomputeInitiallyAndOnChange(this._store),this._decorations=be(this,l=>{this._setText.read(l);const c=this._position.read(l);if(!c)return{org:[],mod:[]};const d=this._diff.read(l);if(!d)return{org:[],mod:[]};const h=[],u=[];if(d.length===1&&d[0].innerChanges[0].modifiedRange.equalsRange(this._previewTextModel.getFullModelRange()))return{org:[],mod:[]};const g=this._shift.get(),f=p=>new x(p.startLineNumber+c.top-1,p.startColumn+g,p.endLineNumber+c.top-1,p.endColumn+g);for(const p of d)if(p.original.isEmpty||h.push({range:f(p.original.toInclusiveRange()),options:Yw}),p.modified.isEmpty||u.push({range:p.modified.toInclusiveRange(),options:vx}),p.modified.isEmpty||p.original.isEmpty)p.original.isEmpty||h.push({range:f(p.original.toInclusiveRange()),options:gW}),p.modified.isEmpty||u.push({range:p.modified.toInclusiveRange(),options:hW});else for(const _ of p.innerChanges||[])p.original.contains(_.originalRange.startLineNumber)&&h.push({range:f(_.originalRange),options:_.originalRange.isEmpty()?fW:G_}),p.modified.contains(_.modifiedRange.startLineNumber)&&u.push({range:_.modifiedRange,options:_.modifiedRange.isEmpty()?uW:wx});return{org:h,mod:u}}),this._originalDecorations=be(this,l=>this._decorations.read(l).org),this._modifiedDecorations=be(this,l=>this._decorations.read(l).mod),this._previewEditor.setModel(this._previewTextModel),this._register(this._editorObs.setDecorations(this._originalDecorations)),this._register(this._previewEditorObs.setDecorations(this._modifiedDecorations)),this._register(Ae(l=>{const c=this._previewEditorObs.contentWidth.read(l),d=this._text.read(l).split(`
`).length-1,h=this._editor.getOption(67)*d;c<=0||this._previewEditor.layout({height:h,width:c})})),this._register(Ae(l=>{this._position.read(l),this._editor.layoutOverlayWidget(this)})),this._register(Ae(l=>{this._scrollChanged.read(l),this._position.read(l)&&this._editor.layoutOverlayWidget(this)}))}getId(){return this.id}getDomNode(){return this._nodes}getPosition(){const e=this._position.get();if(!e)return null;const t=this._editor.getLayoutInfo(),i=this._editor.getScrolledVisiblePosition(new A(e.top,1));if(!i)return null;const n=i.top-1,s=this._editor.getOffsetForColumn(e.left.lineNumber,e.left.column);return{preference:{left:t.contentLeft+s+10,top:n}}}},HF=uf,uf.id=0,uf);zF=HF=RY([wy(5,we)],zF);var REe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Du=function(o,e){return function(t,i){e(t,i,o)}},BC,or;let us=(or=class extends F{static get(e){return e.getContribution(BC.ID)}constructor(e,t,i,n,s,r,a,l){super(),this.editor=e,this.instantiationService=t,this.contextKeyService=i,this.languageFeaturesService=n,this._commandService=s,this._configurationService=r,this._diffProviderFactoryService=a,this._modelService=l,this._isVisibleContext=BC.inlineEditVisibleContext.bindTo(this.contextKeyService),this._isCursorAtInlineEditContext=BC.cursorAtInlineEditContext.bindTo(this.contextKeyService),this._currentEdit=Ke(this,void 0),this._currentWidget=Gn(this._currentEdit,f=>{const p=this._currentEdit.read(f);if(!p)return;const _=p.range.endLineNumber,b=p.range.endColumn,C=p.text.endsWith(`
`)&&!(p.range.startLineNumber===p.range.endLineNumber&&p.range.startColumn===p.range.endColumn)?p.text.slice(0,-1):p.text,v=new g1(_,[new Zx(b,C,!1)]),w=p.range.startLineNumber===p.range.endLineNumber&&v.parts.length===1&&v.parts[0].lines.length===1,S=p.text==="";return!w&&!S?void 0:this.instantiationService.createInstance(OF,this.editor,{ghostText:ur(v),minReservedLineCount:ur(0),targetTextModel:ur(this.editor.getModel()??void 0),range:ur(p.range)})}),this._isAccepting=Ke(this,!1),this._enabled=xt(this,this.editor.onDidChangeConfiguration,()=>this.editor.getOption(63).enabled),this._fontFamily=xt(this,this.editor.onDidChangeConfiguration,()=>this.editor.getOption(63).fontFamily);const c=xn("InlineEditController.modelContentChangedSignal",e.onDidChangeModelContent);this._register(Ae(f=>{this._enabled.read(f)&&(c.read(f),!this._isAccepting.read(f)&&this.getInlineEdit(e,!0))}));const d=xt(this,e.onDidChangeCursorPosition,()=>e.getPosition());this._register(Ae(f=>{if(!this._enabled.read(f))return;const p=d.read(f);p&&this.checkCursorPosition(p)})),this._register(Ae(f=>{const p=this._currentEdit.read(f);if(this._isCursorAtInlineEditContext.set(!1),!p){this._isVisibleContext.set(!1);return}this._isVisibleContext.set(!0);const _=e.getPosition();_&&this.checkCursorPosition(_)}));const h=xn("InlineEditController.editorBlurSignal",e.onDidBlurEditorWidget);this._register(Ae(async f=>{var p;this._enabled.read(f)&&(h.read(f),!(this._configurationService.getValue("editor.experimentalInlineEdit.keepOnBlur")||e.getOption(63).keepOnBlur)&&((p=this._currentRequestCts)==null||p.dispose(!0),this._currentRequestCts=void 0,await this.clear(!1)))}));const u=xn("InlineEditController.editorFocusSignal",e.onDidFocusEditorText);this._register(Ae(f=>{this._enabled.read(f)&&(u.read(f),this.getInlineEdit(e,!0))}));const g=this._register(IU());this._register(Ae(f=>{const p=this._fontFamily.read(f);g.setStyle(p===""||p==="default"?"":`
.monaco-editor .inline-edit-decoration,
.monaco-editor .inline-edit-decoration-preview,
.monaco-editor .inline-edit {
	font-family: ${p};
}`)})),this._register(new FF(this.editor,this._currentWidget,this.instantiationService)),this._register(new VF(this.editor,this._currentEdit,this.instantiationService,this._diffProviderFactoryService,this._modelService))}checkCursorPosition(e){if(!this._currentEdit){this._isCursorAtInlineEditContext.set(!1);return}const t=this._currentEdit.get();if(!t){this._isCursorAtInlineEditContext.set(!1);return}this._isCursorAtInlineEditContext.set(x.containsPosition(t.range,e))}validateInlineEdit(e,t){var i;if(t.text.includes(`
`)&&t.range.startLineNumber!==t.range.endLineNumber&&t.range.startColumn!==t.range.endColumn){if(t.range.startColumn!==1)return!1;const s=t.range.endLineNumber,r=t.range.endColumn,a=((i=e.getModel())==null?void 0:i.getLineLength(s))??0;if(r!==a+1)return!1}return!0}async fetchInlineEdit(e,t){this._currentRequestCts&&this._currentRequestCts.dispose(!0);const i=e.getModel();if(!i)return;const n=i.getVersionId(),s=this.languageFeaturesService.inlineEditProvider.all(i);if(s.length===0)return;const r=s[0];this._currentRequestCts=new li;const a=this._currentRequestCts.token,l=t?Wy.Automatic:Wy.Invoke;if(t&&await MEe(50,a),a.isCancellationRequested||i.isDisposed()||i.getVersionId()!==n)return;const d=await r.provideInlineEdit(i,{triggerKind:l},a);if(d&&!(a.isCancellationRequested||i.isDisposed()||i.getVersionId()!==n)&&this.validateInlineEdit(e,d))return d}async getInlineEdit(e,t){this._isCursorAtInlineEditContext.set(!1),await this.clear();const i=await this.fetchInlineEdit(e,t);i&&this._currentEdit.set(i,void 0)}async trigger(){await this.getInlineEdit(this.editor,!1)}async jumpBack(){this._jumpBackPosition&&(this.editor.setPosition(this._jumpBackPosition),this.editor.revealPositionInCenterIfOutsideViewport(this._jumpBackPosition))}async accept(){this._isAccepting.set(!0,void 0);const e=this._currentEdit.get();if(!e)return;let t=e.text;e.text.startsWith(`
`)&&(t=e.text.substring(1)),this.editor.pushUndoStop(),this.editor.executeEdits("acceptCurrent",[ri.replace(x.lift(e.range),t)]),e.accepted&&await this._commandService.executeCommand(e.accepted.id,...e.accepted.arguments||[]).then(void 0,Si),this.freeEdit(e),ai(i=>{this._currentEdit.set(void 0,i),this._isAccepting.set(!1,i)})}jumpToCurrent(){var i;this._jumpBackPosition=(i=this.editor.getSelection())==null?void 0:i.getStartPosition();const e=this._currentEdit.get();if(!e)return;const t=A.lift({lineNumber:e.range.startLineNumber,column:e.range.startColumn});this.editor.setPosition(t),this.editor.revealPositionInCenterIfOutsideViewport(t)}async clear(e=!0){const t=this._currentEdit.get();t&&(t!=null&&t.rejected)&&e&&await this._commandService.executeCommand(t.rejected.id,...t.rejected.arguments||[]).then(void 0,Si),t&&this.freeEdit(t),this._currentEdit.set(void 0,void 0)}freeEdit(e){const t=this.editor.getModel();if(!t)return;const i=this.languageFeaturesService.inlineEditProvider.all(t);i.length!==0&&i[0].freeInlineEdit(e)}},BC=or,or.ID="editor.contrib.inlineEditController",or.inlineEditVisibleKey="inlineEditVisible",or.inlineEditVisibleContext=new le(or.inlineEditVisibleKey,!1),or.cursorAtInlineEditKey="cursorAtInlineEdit",or.cursorAtInlineEditContext=new le(or.cursorAtInlineEditKey,!1),or);us=BC=REe([Du(1,we),Du(2,Ee),Du(3,ge),Du(4,Ht),Du(5,qe),Du(6,oS),Du(7,Kt)],us);function MEe(o,e){return new Promise(t=>{let i;const n=setTimeout(()=>{i&&i.dispose(),t()},o);e&&(i=e.onCancellationRequested(()=>{clearTimeout(n),i&&i.dispose(),t()}))})}let AEe=class extends de{constructor(){super({id:LEe,label:"Accept Inline Edit",alias:"Accept Inline Edit",precondition:G.and(I.writable,us.inlineEditVisibleContext),kbOpts:[{weight:101,primary:2,kbExpr:G.and(I.writable,us.inlineEditVisibleContext,us.cursorAtInlineEditContext)}],menuOpts:[{menuId:me.InlineEditToolbar,title:"Accept",group:"primary",order:1}]})}async run(e,t){const i=us.get(t);await(i==null?void 0:i.accept())}};class PEe extends de{constructor(){const e=G.and(I.writable,G.not(us.inlineEditVisibleKey));super({id:"editor.action.inlineEdit.trigger",label:"Trigger Inline Edit",alias:"Trigger Inline Edit",precondition:e,kbOpts:{weight:101,primary:2646,kbExpr:e}})}async run(e,t){const i=us.get(t);i==null||i.trigger()}}class OEe extends de{constructor(){const e=G.and(I.writable,us.inlineEditVisibleContext,G.not(us.cursorAtInlineEditKey));super({id:kEe,label:"Jump to Inline Edit",alias:"Jump to Inline Edit",precondition:e,kbOpts:{weight:101,primary:2646,kbExpr:e},menuOpts:[{menuId:me.InlineEditToolbar,title:"Jump To Edit",group:"primary",order:3,when:e}]})}async run(e,t){const i=us.get(t);i==null||i.jumpToCurrent()}}class FEe extends de{constructor(){const e=G.and(I.writable,us.cursorAtInlineEditContext);super({id:DEe,label:"Jump Back from Inline Edit",alias:"Jump Back from Inline Edit",precondition:e,kbOpts:{weight:110,primary:2646,kbExpr:e},menuOpts:[{menuId:me.InlineEditToolbar,title:"Jump Back",group:"primary",order:3,when:e}]})}async run(e,t){const i=us.get(t);i==null||i.jumpBack()}}class WEe extends de{constructor(){const e=G.and(I.writable,us.inlineEditVisibleContext);super({id:xEe,label:"Reject Inline Edit",alias:"Reject Inline Edit",precondition:e,kbOpts:{weight:100,primary:9,kbExpr:e},menuOpts:[{menuId:me.InlineEditToolbar,title:"Reject",group:"secondary",order:2}]})}async run(e,t){const i=us.get(t);await(i==null?void 0:i.clear())}}Q(AEe);Q(WEe);Q(OEe);Q(FEe);Q(PEe);ct(us.ID,us,3);const BEe="editor.action.inlineEdits.accept",HEe="editor.action.inlineEdits.showPrevious",VEe="editor.action.inlineEdits.showNext",cb=new le("inlineEditsVisible",!1,m("inlineEditsVisible","Whether an inline edit is visible")),zEe=new le("inlineEditsIsPinned",!1,m("isPinned","Whether an inline edit is visible")),kB=class kB extends F{constructor(e){super(),this._editor=e,this._editorObs=no(this._editor),this._placeholderText=this._editorObs.getOption(88),this._state=Mo({owner:this,equalsFn:wL},t=>{const i=this._placeholderText.read(t);if(i&&this._editorObs.valueIsEmpty.read(t))return{placeholder:i}}),this._shouldViewBeAlive=UEe(this,t=>{var i;return((i=this._state.read(t))==null?void 0:i.placeholder)!==void 0}),this._view=su((t,i)=>{if(!this._shouldViewBeAlive.read(t))return;const n=Xe("div.editorPlaceholder");i.add(Ae(s=>{const r=this._state.read(s),a=(r==null?void 0:r.placeholder)!==void 0;n.root.style.display=a?"block":"none",n.root.innerText=(r==null?void 0:r.placeholder)??""})),i.add(Ae(s=>{const r=this._editorObs.layoutInfo.read(s);n.root.style.left=`${r.contentLeft}px`,n.root.style.width=r.contentWidth-r.verticalScrollbarWidth+"px",n.root.style.top=`${this._editor.getTopForLineNumber(0)}px`})),i.add(Ae(s=>{n.root.style.fontFamily=this._editorObs.getOption(49).read(s),n.root.style.fontSize=this._editorObs.getOption(52).read(s)+"px",n.root.style.lineHeight=this._editorObs.getOption(67).read(s)+"px"})),i.add(this._editorObs.createOverlayWidget({allowEditorOverflow:!1,minContentWidthInPx:ur(0),position:ur(null),domNode:n.root}))}),this._view.recomputeInitiallyAndOnChange(this._store)}};kB.ID="editor.contrib.placeholderText";let v1=kB;function UEe(o,e){return Y1(o,(t,i)=>i===!0?!0:e(t))}var $Ee=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},jEe=function(o,e){return function(t,i){e(t,i,o)}};class KEe{constructor(e,t,i){this.range=e,this.newLines=t,this.changes=i}}let UF=class extends F{constructor(e,t,i,n){super(),this._editor=e,this._edit=t,this._userPrompt=i,this._instantiationService=n,this._editorObs=no(this._editor),this._elements=Xe("div.inline-edits-widget",{style:{position:"absolute",overflow:"visible",top:"0px",left:"0px"}},[Xe("div@editorContainer",{style:{position:"absolute",top:"0px",left:"0px",width:"500px",height:"500px"}},[Xe("div.toolbar@toolbar",{style:{position:"absolute",top:"-25px",left:"0px"}}),Xe("div.promptEditor@promptEditor",{style:{position:"absolute",top:"-25px",left:"80px",width:"300px",height:"22px"}}),Xe("div.preview@editor",{style:{position:"absolute",top:"0px",left:"0px"}})]),Sm("svg",{style:{overflow:"visible",pointerEvents:"none"}},[Sm("defs",[Sm("linearGradient",{id:"Gradient2",x1:"0",y1:"0",x2:"1",y2:"0"},[Sm("stop",{offset:"0%",class:"gradient-stop"}),Sm("stop",{offset:"100%",class:"gradient-stop"})])]),Sm("path@path",{d:"",fill:"url(#Gradient2)"})])]),this._previewTextModel=this._register(this._instantiationService.createInstance(Da,"",ms,Da.DEFAULT_CREATION_OPTIONS,null)),this._setText=be(r=>{const a=this._edit.read(r);a&&this._previewTextModel.setValue(a.newLines.join(`
`))}).recomputeInitiallyAndOnChange(this._store),this._promptTextModel=this._register(this._instantiationService.createInstance(Da,"",ms,Da.DEFAULT_CREATION_OPTIONS,null)),this._promptEditor=this._register(this._instantiationService.createInstance(Jr,this._elements.promptEditor,{glyphMargin:!1,lineNumbers:"off",minimap:{enabled:!1},guides:{indentation:!1,bracketPairs:!1,bracketPairsHorizontal:!1,highlightActiveIndentation:!1},folding:!1,selectOnLineNumbers:!1,selectionHighlight:!1,columnSelection:!1,overviewRulerBorder:!1,overviewRulerLanes:0,lineDecorationsWidth:0,lineNumbersMinChars:0,placeholder:"Describe the change you want...",fontFamily:Eq},{contributions:_g.getSomeEditorContributions([Ur.ID,v1.ID,tb.ID]),isSimpleWidget:!0},this._editor)),this._previewEditor=this._register(this._instantiationService.createInstance(Jr,this._elements.editor,{glyphMargin:!1,lineNumbers:"off",minimap:{enabled:!1},guides:{indentation:!1,bracketPairs:!1,bracketPairsHorizontal:!1,highlightActiveIndentation:!1},folding:!1,selectOnLineNumbers:!1,selectionHighlight:!1,columnSelection:!1,overviewRulerBorder:!1,overviewRulerLanes:0,lineDecorationsWidth:0,lineNumbersMinChars:0},{contributions:[]},this._editor)),this._previewEditorObs=no(this._previewEditor),this._decorations=be(this,r=>{var d;this._setText.read(r);const a=(d=this._edit.read(r))==null?void 0:d.changes;if(!a)return[];const l=[],c=[];if(a.length===1&&a[0].innerChanges[0].modifiedRange.equalsRange(this._previewTextModel.getFullModelRange()))return[];for(const h of a)if(h.original.isEmpty||l.push({range:h.original.toInclusiveRange(),options:Yw}),h.modified.isEmpty||c.push({range:h.modified.toInclusiveRange(),options:vx}),h.modified.isEmpty||h.original.isEmpty)h.original.isEmpty||l.push({range:h.original.toInclusiveRange(),options:gW}),h.modified.isEmpty||c.push({range:h.modified.toInclusiveRange(),options:hW});else for(const u of h.innerChanges||[])h.original.contains(u.originalRange.startLineNumber)&&l.push({range:u.originalRange,options:u.originalRange.isEmpty()?fW:G_}),h.modified.contains(u.modifiedRange.startLineNumber)&&c.push({range:u.modifiedRange,options:u.modifiedRange.isEmpty()?uW:wx});return c}),this._layout1=be(this,r=>{const a=this._editor.getModel(),l=this._edit.read(r);if(!l)return null;const c=l.range;let d=0;for(let g=c.startLineNumber;g<c.endLineNumberExclusive;g++){const f=a.getLineMaxColumn(g),p=this._editor.getOffsetForColumn(g,f);d=Math.max(d,p)}return{left:this._editor.getLayoutInfo().contentLeft+d}}),this._layout=be(this,r=>{const a=this._edit.read(r);if(!a)return null;const l=a.range,c=this._editorObs.scrollLeft.read(r),d=this._layout1.read(r).left+20-c,h=this._editor.getTopForLineNumber(l.startLineNumber)-this._editorObs.scrollTop.read(r),u=this._editor.getTopForLineNumber(l.endLineNumberExclusive)-this._editorObs.scrollTop.read(r),g=new wp(d,h),f=new wp(d,u),p=u-h,_=50,b=this._editor.getOption(67)*a.newLines.length,C=p-b,v=new wp(d+_,h+C/2),w=new wp(d+_,u-C/2);return{topCode:g,bottomCode:f,codeHeight:p,topEdit:v,bottomEdit:w,editHeight:b}});const s=be(this,r=>this._edit.read(r)!==void 0||this._userPrompt.read(r)!==void 0);this._register(sd(this._elements.root,{display:be(this,r=>s.read(r)?"block":"none")})),this._register(fp(this._editor.getDomNode(),this._elements.root)),this._register(no(e).createOverlayWidget({domNode:this._elements.root,position:ur(null),allowEditorOverflow:!1,minContentWidthInPx:be(r=>{var c;const a=(c=this._layout1.read(r))==null?void 0:c.left;if(a===void 0)return 0;const l=this._previewEditorObs.contentWidth.read(r);return a+l})})),this._previewEditor.setModel(this._previewTextModel),this._register(this._previewEditorObs.setDecorations(this._decorations)),this._register(Ae(r=>{const a=this._layout.read(r);if(!a)return;const{topCode:l,bottomCode:c,topEdit:d,bottomEdit:h,editHeight:u}=a,g=10,f=0,p=40,_=new GEe().moveTo(l).lineTo(l.deltaX(g)).curveTo(l.deltaX(g+p),d.deltaX(-40-f),d.deltaX(-0)).lineTo(d).lineTo(h).lineTo(h.deltaX(-0)).curveTo(h.deltaX(-40-f),c.deltaX(g+p),c.deltaX(g)).lineTo(c).build();this._elements.path.setAttribute("d",_),this._elements.editorContainer.style.top=`${d.y}px`,this._elements.editorContainer.style.left=`${d.x}px`,this._elements.editorContainer.style.height=`${u}px`;const b=this._previewEditorObs.contentWidth.read(r);this._previewEditor.layout({height:u,width:b})})),this._promptEditor.setModel(this._promptTextModel),this._promptEditor.layout(),this._register(ZEe(qEe(this._userPrompt,r=>r??"",r=>r),no(this._promptEditor).value)),this._register(Ae(r=>{const a=no(this._promptEditor).isFocused.read(r);this._elements.root.classList.toggle("focused",a)}))}};UF=$Ee([jEe(3,we)],UF);function qEe(o,e,t){return q1(void 0,i=>e(o.read(i)),(i,n)=>o.set(t(i),n))}class wp{constructor(e,t){this.x=e,this.y=t}deltaX(e){return new wp(this.x+e,this.y)}}class GEe{constructor(){this._data=""}moveTo(e){return this._data+=`M ${e.x} ${e.y} `,this}lineTo(e){return this._data+=`L ${e.x} ${e.y} `,this}curveTo(e,t,i){return this._data+=`C ${e.x} ${e.y} ${t.x} ${t.y} ${i.x} ${i.y} `,this}build(){return this._data}}function ZEe(o,e){const t=new K;return t.add(Ae(i=>{const n=o.read(i);e.set(n,void 0)})),t.add(Ae(i=>{const n=e.read(i);o.set(n,void 0)})),t}var YEe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},iR=function(o,e){return function(t,i){e(t,i,o)}},HC,gf;let $F=(gf=class extends F{static _createUniqueUri(){return Se.from({scheme:"inline-edits",path:new Date().toString()+String(HC._modelId++)})}constructor(e,t,i,n,s,r,a){super(),this.textModel=e,this._textModelVersionId=t,this._selection=i,this._debounceValue=n,this.languageFeaturesService=s,this._diffProviderFactoryService=r,this._modelService=a,this._forceUpdateExplicitlySignal=Lb(this),this._selectedInlineCompletionId=Ke(this,void 0),this._isActive=Ke(this,!1),this._originalModel=Gn(()=>this._modelService.createModel("",null,HC._createUniqueUri())).keepObserved(this._store),this._modifiedModel=Gn(()=>this._modelService.createModel("",null,HC._createUniqueUri())).keepObserved(this._store),this._pinnedRange=new QEe(this.textModel,this._textModelVersionId),this.isPinned=this._pinnedRange.range.map(l=>!!l),this.userPrompt=Ke(this,void 0),this.inlineEdit=be(this,l=>{var c,d;return(d=(c=this._inlineEdit.read(l))==null?void 0:c.promiseResult.read(l))==null?void 0:d.data}),this._inlineEdit=be(this,l=>{const c=this.selectedInlineEdit.read(l);if(!c)return;const d=c.inlineCompletion.range;if(c.inlineCompletion.insertText.trim()==="")return;let h=c.inlineCompletion.insertText.split(/\r\n|\r|\n/);function u(_){var C;const b=((C=_[0].match(/^\s*/))==null?void 0:C[0])??"";return _.map(v=>v.replace(new RegExp("^"+b),""))}h=u(h);let f=this.textModel.getValueInRange(d).split(/\r\n|\r|\n/);f=u(f),this._originalModel.get().setValue(f.join(`
`)),this._modifiedModel.get().setValue(h.join(`
`));const p=this._diffProviderFactoryService.createDiffProvider({diffAlgorithm:"advanced"});return tE.fromFn(async()=>{const _=await p.computeDiff(this._originalModel.get(),this._modifiedModel.get(),{computeMoves:!1,ignoreTrimWhitespace:!1,maxComputationTimeMs:1e3},ze.None);if(!_.identical)return new KEe(Be.fromRangeInclusive(d),u(h),_.changes)})}),this._fetchStore=this._register(new K),this._inlineEditsFetchResult=Dw(this,void 0),this._inlineEdits=Mo({owner:this,equalsFn:wL},l=>{var c;return((c=this._inlineEditsFetchResult.read(l))==null?void 0:c.completions.map(d=>new XEe(d)))??[]}),this._fetchInlineEditsPromise=Lj({owner:this,createEmptyChangeSummary:()=>({inlineCompletionTriggerKind:Ea.Automatic}),handleChange:(l,c)=>(l.didChange(this._forceUpdateExplicitlySignal)&&(c.inlineCompletionTriggerKind=Ea.Explicit),!0)},async(l,c)=>{this._fetchStore.clear(),this._forceUpdateExplicitlySignal.read(l),this._textModelVersionId.read(l);function d(p,_){return _(p)}const h=this._pinnedRange.range.read(l)??d(this._selection.read(l),p=>p.isEmpty()?void 0:p);if(!h){this._inlineEditsFetchResult.set(void 0,void 0),this.userPrompt.set(void 0,void 0);return}const u={triggerKind:c.inlineCompletionTriggerKind,selectedSuggestionInfo:void 0,userPrompt:this.userPrompt.read(l)},g=_R(this._fetchStore);await ed(200,g);const f=await fY(this.languageFeaturesService.inlineCompletionsProvider,h,this.textModel,u,g);g.isCancellationRequested||this._inlineEditsFetchResult.set(f,void 0)}),this._filteredInlineEditItems=Mo({owner:this,equalsFn:vL()},l=>this._inlineEdits.read(l)),this.selectedInlineCompletionIndex=be(this,l=>{const c=this._selectedInlineCompletionId.read(l),d=this._filteredInlineEditItems.read(l),h=this._selectedInlineCompletionId===void 0?-1:d.findIndex(u=>u.semanticId===c);return h===-1?(this._selectedInlineCompletionId.set(void 0,void 0),0):h}),this.selectedInlineEdit=be(this,l=>{const c=this._filteredInlineEditItems.read(l),d=this.selectedInlineCompletionIndex.read(l);return c[d]}),this._register(xb(this._fetchInlineEditsPromise))}async triggerExplicitly(e){Pf(e,t=>{this._isActive.set(!0,t),this._forceUpdateExplicitlySignal.trigger(t)}),await this._fetchInlineEditsPromise.get()}stop(e){Pf(e,t=>{this.userPrompt.set(void 0,t),this._isActive.set(!1,t),this._inlineEditsFetchResult.set(void 0,t),this._pinnedRange.setRange(void 0,t)})}async _deltaSelectedInlineCompletionIndex(e){await this.triggerExplicitly();const t=this._filteredInlineEditItems.get()||[];if(t.length>0){const i=(this.selectedInlineCompletionIndex.get()+e+t.length)%t.length;this._selectedInlineCompletionId.set(t[i].semanticId,void 0)}else this._selectedInlineCompletionId.set(void 0,void 0)}async next(){await this._deltaSelectedInlineCompletionIndex(1)}async previous(){await this._deltaSelectedInlineCompletionIndex(-1)}async accept(e){if(e.getModel()!==this.textModel)throw new bt;const t=this.selectedInlineEdit.get();t&&(e.pushUndoStop(),e.executeEdits("inlineSuggestion.accept",[t.inlineCompletion.toSingleTextEdit().toSingleEditOperation()]),this.stop())}},HC=gf,gf._modelId=0,gf);$F=HC=YEe([iR(4,ge),iR(5,oS),iR(6,Kt)],$F);class XEe{constructor(e){this.inlineCompletion=e,this.semanticId=this.inlineCompletion.hash()}}class QEe extends F{constructor(e,t){super(),this._textModel=e,this._versionId=t,this._decorations=Ke(this,[]),this.range=be(this,i=>{this._versionId.read(i);const n=this._decorations.read(i)[0];return n?this._textModel.getDecorationRange(n)??null:null}),this._register(ve(()=>{this._textModel.deltaDecorations(this._decorations.get(),[])}))}setRange(e,t){this._decorations.set(this._textModel.deltaDecorations(this._decorations.get(),e?[{range:e,options:{description:"trackedRange"}}]:[]),t)}}var JEe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},_C=function(o,e){return function(t,i){e(t,i,o)}},jF,ff;let ld=(ff=class extends F{static get(e){return e.getContribution(jF.ID)}constructor(e,t,i,n,s,r){super(),this.editor=e,this._instantiationService=t,this._contextKeyService=i,this._debounceService=n,this._languageFeaturesService=s,this._configurationService=r,this._enabled=Rve("editor.inlineEdits.enabled",!1,this._configurationService),this._editorObs=no(this.editor),this._selection=be(this,a=>this._editorObs.cursorSelection.read(a)??new pe(1,1,1,1)),this._debounceValue=this._debounceService.for(this._languageFeaturesService.inlineCompletionsProvider,"InlineEditsDebounce",{min:50,max:50}),this.model=Gn(this,a=>{if(!this._enabled.read(a)||this._editorObs.isReadonly.read(a))return;const l=this._editorObs.model.read(a);return l?this._instantiationService.createInstance(Js($F,a),l,this._editorObs.versionId,this._selection,this._debounceValue):void 0}),this._hadInlineEdit=Y1(this,(a,l)=>{var c;return l||((c=this.model.read(a))==null?void 0:c.inlineEdit.read(a))!==void 0}),this._widget=Gn(this,a=>{if(this._hadInlineEdit.read(a))return this._instantiationService.createInstance(Js(UF,a),this.editor,this.model.map((l,c)=>l==null?void 0:l.inlineEdit.read(c)),eNe(l=>{var c;return((c=this.model.read(l))==null?void 0:c.userPrompt)??Ke("empty","")}))}),this._register(ha(cb,this._contextKeyService,a=>{var l;return!!((l=this.model.read(a))!=null&&l.inlineEdit.read(a))})),this._register(ha(zEe,this._contextKeyService,a=>{var l;return!!((l=this.model.read(a))!=null&&l.isPinned.read(a))})),this.model.recomputeInitiallyAndOnChange(this._store),this._widget.recomputeInitiallyAndOnChange(this._store)}},jF=ff,ff.ID="editor.contrib.inlineEditsController",ff);ld=jF=JEe([_C(1,we),_C(2,Ee),_C(3,Zo),_C(4,ge),_C(5,qe)],ld);function eNe(o){return q1(void 0,e=>o(e).read(e),(e,t)=>{o(void 0).set(e,t)})}function _S(o){return{label:o.value,alias:o.original}}const qD=class qD extends de{constructor(){super({id:qD.ID,..._S(Te("action.inlineEdits.showNext","Show Next Inline Edit")),precondition:G.and(I.writable,cb),kbOpts:{weight:100,primary:606}})}async run(e,t){var n;const i=ld.get(t);(n=i==null?void 0:i.model.get())==null||n.next()}};qD.ID=VEe;let KF=qD;const GD=class GD extends de{constructor(){super({id:GD.ID,..._S(Te("action.inlineEdits.showPrevious","Show Previous Inline Edit")),precondition:G.and(I.writable,cb),kbOpts:{weight:100,primary:604}})}async run(e,t){var n;const i=ld.get(t);(n=i==null?void 0:i.model.get())==null||n.previous()}};GD.ID=HEe;let qF=GD;class tNe extends de{constructor(){super({id:"editor.action.inlineEdits.trigger",..._S(Te("action.inlineEdits.trigger","Trigger Inline Edit")),precondition:I.writable})}async run(e,t){const i=ld.get(t);await yj(async n=>{var s;await((s=i==null?void 0:i.model.get())==null?void 0:s.triggerExplicitly(n))})}}class iNe extends de{constructor(){super({id:BEe,..._S(Te("action.inlineEdits.accept","Accept Inline Edit")),precondition:cb,menuOpts:{menuId:me.InlineEditsActions,title:m("inlineEditsActions","Accept Inline Edit"),group:"primary",order:1,icon:te.check},kbOpts:{primary:2058,weight:2e4,kbExpr:cb}})}async run(e,t){var n;t instanceof Jr&&(t=t.getParentEditor());const i=ld.get(t);i&&((n=i.model.get())==null||n.accept(i.editor),i.editor.focus())}}const ZD=class ZD extends de{constructor(){super({id:ZD.ID,..._S(Te("action.inlineEdits.hide","Hide Inline Edit")),precondition:cb,kbOpts:{weight:100,primary:9}})}async run(e,t){const i=ld.get(t);ai(n=>{var s;(s=i==null?void 0:i.model.get())==null||s.stop(n)})}};ZD.ID="editor.action.inlineEdits.hide";let GF=ZD;ct(ld.ID,ld,3);Q(tNe);Q(KF);Q(qF);Q(iNe);Q(GF);const Yf={Visible:new le("parameterHintsVisible",!1),MultipleSignatures:new le("parameterHintsMultipleSignatures",!1)};async function MY(o,e,t,i,n){const s=o.ordered(e);for(const r of s)try{const a=await r.provideSignatureHelp(e,t,n,i);if(a)return a}catch(a){Si(a)}}rt.registerCommand("_executeSignatureHelpProvider",async(o,...e)=>{const[t,i,n]=e;mt(Se.isUri(t)),mt(A.isIPosition(i)),mt(typeof n=="string"||!n);const s=o.get(ge),r=await o.get(Hs).createModelReference(t);try{const a=await MY(s.signatureHelpProvider,r.object.textEditorModel,A.lift(i),{triggerKind:Rl.Invoke,isRetrigger:!1,triggerCharacter:n},ze.None);return a?(setTimeout(()=>a.dispose(),0),a.value):void 0}finally{r.dispose()}});var Td;(function(o){o.Default={type:0};class e{constructor(n,s){this.request=n,this.previouslyActiveHints=s,this.type=2}}o.Pending=e;class t{constructor(n){this.hints=n,this.type=1}}o.Active=t})(Td||(Td={}));const YD=class YD extends F{constructor(e,t,i=YD.DEFAULT_DELAY){super(),this._onChangedHints=this._register(new P),this.onChangedHints=this._onChangedHints.event,this.triggerOnType=!1,this._state=Td.Default,this._pendingTriggers=[],this._lastSignatureHelpResult=this._register(new Ji),this.triggerChars=new Qy,this.retriggerChars=new Qy,this.triggerId=0,this.editor=e,this.providers=t,this.throttledDelayer=new _r(i),this._register(this.editor.onDidBlurEditorWidget(()=>this.cancel())),this._register(this.editor.onDidChangeConfiguration(()=>this.onEditorConfigurationChange())),this._register(this.editor.onDidChangeModel(n=>this.onModelChanged())),this._register(this.editor.onDidChangeModelLanguage(n=>this.onModelChanged())),this._register(this.editor.onDidChangeCursorSelection(n=>this.onCursorChange(n))),this._register(this.editor.onDidChangeModelContent(n=>this.onModelContentChange())),this._register(this.providers.onDidChange(this.onModelChanged,this)),this._register(this.editor.onDidType(n=>this.onDidType(n))),this.onEditorConfigurationChange(),this.onModelChanged()}get state(){return this._state}set state(e){this._state.type===2&&this._state.request.cancel(),this._state=e}cancel(e=!1){this.state=Td.Default,this.throttledDelayer.cancel(),e||this._onChangedHints.fire(void 0)}trigger(e,t){const i=this.editor.getModel();if(!i||!this.providers.has(i))return;const n=++this.triggerId;this._pendingTriggers.push(e),this.throttledDelayer.trigger(()=>this.doTrigger(n),t).catch(Oe)}next(){if(this.state.type!==1)return;const e=this.state.hints.signatures.length,t=this.state.hints.activeSignature,i=t%e===e-1,n=this.editor.getOption(86).cycle;if((e<2||i)&&!n){this.cancel();return}this.updateActiveSignature(i&&n?0:t+1)}previous(){if(this.state.type!==1)return;const e=this.state.hints.signatures.length,t=this.state.hints.activeSignature,i=t===0,n=this.editor.getOption(86).cycle;if((e<2||i)&&!n){this.cancel();return}this.updateActiveSignature(i&&n?e-1:t-1)}updateActiveSignature(e){this.state.type===1&&(this.state=new Td.Active({...this.state.hints,activeSignature:e}),this._onChangedHints.fire(this.state.hints))}async doTrigger(e){const t=this.state.type===1||this.state.type===2,i=this.getLastActiveHints();if(this.cancel(!0),this._pendingTriggers.length===0)return!1;const n=this._pendingTriggers.reduce(nNe);this._pendingTriggers=[];const s={triggerKind:n.triggerKind,triggerCharacter:n.triggerCharacter,isRetrigger:t,activeSignatureHelp:i};if(!this.editor.hasModel())return!1;const r=this.editor.getModel(),a=this.editor.getPosition();this.state=new Td.Pending(Yi(l=>MY(this.providers,r,a,s,l)),i);try{const l=await this.state.request;return e!==this.triggerId?(l==null||l.dispose(),!1):!l||!l.value.signatures||l.value.signatures.length===0?(l==null||l.dispose(),this._lastSignatureHelpResult.clear(),this.cancel(),!1):(this.state=new Td.Active(l.value),this._lastSignatureHelpResult.value=l,this._onChangedHints.fire(this.state.hints),!0)}catch(l){return e===this.triggerId&&(this.state=Td.Default),Oe(l),!1}}getLastActiveHints(){switch(this.state.type){case 1:return this.state.hints;case 2:return this.state.previouslyActiveHints;default:return}}get isTriggered(){return this.state.type===1||this.state.type===2||this.throttledDelayer.isTriggered()}onModelChanged(){this.cancel(),this.triggerChars.clear(),this.retriggerChars.clear();const e=this.editor.getModel();if(e)for(const t of this.providers.ordered(e)){for(const i of t.signatureHelpTriggerCharacters||[])if(i.length){const n=i.charCodeAt(0);this.triggerChars.add(n),this.retriggerChars.add(n)}for(const i of t.signatureHelpRetriggerCharacters||[])i.length&&this.retriggerChars.add(i.charCodeAt(0))}}onDidType(e){if(!this.triggerOnType)return;const t=e.length-1,i=e.charCodeAt(t);(this.triggerChars.has(i)||this.isTriggered&&this.retriggerChars.has(i))&&this.trigger({triggerKind:Rl.TriggerCharacter,triggerCharacter:e.charAt(t)})}onCursorChange(e){e.source==="mouse"?this.cancel():this.isTriggered&&this.trigger({triggerKind:Rl.ContentChange})}onModelContentChange(){this.isTriggered&&this.trigger({triggerKind:Rl.ContentChange})}onEditorConfigurationChange(){this.triggerOnType=this.editor.getOption(86).enabled,this.triggerOnType||this.cancel()}dispose(){this.cancel(!0),super.dispose()}};YD.DEFAULT_DELAY=120;let ZF=YD;function nNe(o,e){switch(e.triggerKind){case Rl.Invoke:return e;case Rl.ContentChange:return o;case Rl.TriggerCharacter:default:return e}}var sNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},P0=function(o,e){return function(t,i){e(t,i,o)}},YF;const So=ne,oNe=qt("parameter-hints-next",te.chevronDown,m("parameterHintsNextIcon","Icon for show next parameter hint.")),rNe=qt("parameter-hints-previous",te.chevronUp,m("parameterHintsPreviousIcon","Icon for show previous parameter hint."));var mf;let XF=(mf=class extends F{constructor(e,t,i,n,s,r){super(),this.editor=e,this.model=t,this.telemetryService=r,this.renderDisposeables=this._register(new K),this.visible=!1,this.announcedLabel=null,this.allowEditorOverflow=!0,this.markdownRenderer=this._register(new nd({editor:e},s,n)),this.keyVisible=Yf.Visible.bindTo(i),this.keyMultipleSignatures=Yf.MultipleSignatures.bindTo(i)}createParameterHintDOMNodes(){const e=So(".editor-widget.parameter-hints-widget"),t=X(e,So(".phwrapper"));t.tabIndex=-1;const i=X(t,So(".controls")),n=X(i,So(".button"+Ie.asCSSSelector(rNe))),s=X(i,So(".overloads")),r=X(i,So(".button"+Ie.asCSSSelector(oNe)));this._register(U(n,"click",u=>{nt.stop(u),this.previous()})),this._register(U(r,"click",u=>{nt.stop(u),this.next()}));const a=So(".body"),l=new z1(a,{alwaysConsumeMouseWheel:!0});this._register(l),t.appendChild(l.getDomNode());const c=X(a,So(".signature")),d=X(a,So(".docs"));e.style.userSelect="text",this.domNodes={element:e,signature:c,overloads:s,docs:d,scrollbar:l},this.editor.addContentWidget(this),this.hide(),this._register(this.editor.onDidChangeCursorSelection(u=>{this.visible&&this.editor.layoutContentWidget(this)}));const h=()=>{if(!this.domNodes)return;const u=this.editor.getOption(50),g=this.domNodes.element;g.style.fontSize=`${u.fontSize}px`,g.style.lineHeight=`${u.lineHeight/u.fontSize}`,g.style.setProperty("--vscode-parameterHintsWidget-editorFontFamily",u.fontFamily),g.style.setProperty("--vscode-parameterHintsWidget-editorFontFamilyDefault",fs.fontFamily)};h(),this._register(oe.chain(this.editor.onDidChangeConfiguration.bind(this.editor),u=>u.filter(g=>g.hasChanged(50)))(h)),this._register(this.editor.onDidLayoutChange(u=>this.updateMaxHeight())),this.updateMaxHeight()}show(){this.visible||(this.domNodes||this.createParameterHintDOMNodes(),this.keyVisible.set(!0),this.visible=!0,setTimeout(()=>{var e;(e=this.domNodes)==null||e.element.classList.add("visible")},100),this.editor.layoutContentWidget(this))}hide(){var e;this.renderDisposeables.clear(),this.visible&&(this.keyVisible.reset(),this.visible=!1,this.announcedLabel=null,(e=this.domNodes)==null||e.element.classList.remove("visible"),this.editor.layoutContentWidget(this))}getPosition(){return this.visible?{position:this.editor.getPosition(),preference:[1,2]}:null}render(e){if(this.renderDisposeables.clear(),!this.domNodes)return;const t=e.signatures.length>1;this.domNodes.element.classList.toggle("multiple",t),this.keyMultipleSignatures.set(t),this.domNodes.signature.innerText="",this.domNodes.docs.innerText="";const i=e.signatures[e.activeSignature];if(!i)return;const n=X(this.domNodes.signature,So(".code")),s=i.parameters.length>0,r=i.activeParameter??e.activeParameter;if(s)this.renderParameters(n,i,r);else{const c=X(n,So("span"));c.textContent=i.label}const a=i.parameters[r];if(a!=null&&a.documentation){const c=So("span.documentation");if(typeof a.documentation=="string")c.textContent=a.documentation;else{const d=this.renderMarkdownDocs(a.documentation);c.appendChild(d.element)}X(this.domNodes.docs,So("p",{},c))}if(i.documentation!==void 0)if(typeof i.documentation=="string")X(this.domNodes.docs,So("p",{},i.documentation));else{const c=this.renderMarkdownDocs(i.documentation);X(this.domNodes.docs,c.element)}const l=this.hasDocs(i,a);if(this.domNodes.signature.classList.toggle("has-docs",l),this.domNodes.docs.classList.toggle("empty",!l),this.domNodes.overloads.textContent=String(e.activeSignature+1).padStart(e.signatures.length.toString().length,"0")+"/"+e.signatures.length,a){let c="";const d=i.parameters[r];Array.isArray(d.label)?c=i.label.substring(d.label[0],d.label[1]):c=d.label,d.documentation&&(c+=typeof d.documentation=="string"?`, ${d.documentation}`:`, ${d.documentation.value}`),i.documentation&&(c+=typeof i.documentation=="string"?`, ${i.documentation}`:`, ${i.documentation.value}`),this.announcedLabel!==c&&(Cs(m("hint","{0}, hint",c)),this.announcedLabel=c)}this.editor.layoutContentWidget(this),this.domNodes.scrollbar.scanDomNode()}renderMarkdownDocs(e){const t=new ln,i=this.renderDisposeables.add(this.markdownRenderer.render(e,{asyncRenderCallback:()=>{var s;(s=this.domNodes)==null||s.scrollbar.scanDomNode()}}));i.element.classList.add("markdown-docs");const n=t.elapsed();return n>300&&this.telemetryService.publicLog2("parameterHints.parseMarkdown",{renderDuration:n}),i}hasDocs(e,t){return!!(t&&typeof t.documentation=="string"&&Kd(t.documentation).length>0||t&&typeof t.documentation=="object"&&Kd(t.documentation).value.length>0||e.documentation&&typeof e.documentation=="string"&&Kd(e.documentation).length>0||e.documentation&&typeof e.documentation=="object"&&Kd(e.documentation.value).length>0)}renderParameters(e,t,i){const[n,s]=this.getParameterLabelOffsets(t,i),r=document.createElement("span");r.textContent=t.label.substring(0,n);const a=document.createElement("span");a.textContent=t.label.substring(n,s),a.className="parameter active";const l=document.createElement("span");l.textContent=t.label.substring(s),X(e,r,a,l)}getParameterLabelOffsets(e,t){const i=e.parameters[t];if(i){if(Array.isArray(i.label))return i.label;if(i.label.length){const n=new RegExp(`(\\W|^)${Uo(i.label)}(?=\\W|$)`,"g");n.test(e.label);const s=n.lastIndex-i.label.length;return s>=0?[s,n.lastIndex]:[0,0]}else return[0,0]}else return[0,0]}next(){this.editor.focus(),this.model.next()}previous(){this.editor.focus(),this.model.previous()}getDomNode(){return this.domNodes||this.createParameterHintDOMNodes(),this.domNodes.element}getId(){return YF.ID}updateMaxHeight(){if(!this.domNodes)return;const t=`${Math.max(this.editor.getLayoutInfo().height/4,250)}px`;this.domNodes.element.style.maxHeight=t;const i=this.domNodes.element.getElementsByClassName("phwrapper");i.length&&(i[0].style.maxHeight=t)}},YF=mf,mf.ID="editor.widget.parameterHintsWidget",mf);XF=YF=sNe([P0(2,Ee),P0(3,zs),P0(4,ni),P0(5,Bn)],XF);E("editorHoverWidget.highlightForeground",sp,m("editorHoverWidgetHighlightForeground","Foreground color of the active item in the parameter hint."));var aNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},$6=function(o,e){return function(t,i){e(t,i,o)}},QF,pf;let db=(pf=class extends F{static get(e){return e.getContribution(QF.ID)}constructor(e,t,i){super(),this.editor=e,this.model=this._register(new ZF(e,i.signatureHelpProvider)),this._register(this.model.onChangedHints(n=>{var s;n?(this.widget.value.show(),this.widget.value.render(n)):(s=this.widget.rawValue)==null||s.hide()})),this.widget=new Oa(()=>this._register(t.createInstance(XF,this.editor,this.model)))}cancel(){this.model.cancel()}previous(){var e;(e=this.widget.rawValue)==null||e.previous()}next(){var e;(e=this.widget.rawValue)==null||e.next()}trigger(e){this.model.trigger(e,0)}},QF=pf,pf.ID="editor.controller.parameterHints",pf);db=QF=aNe([$6(1,we),$6(2,ge)],db);class lNe extends de{constructor(){super({id:"editor.action.triggerParameterHints",label:m("parameterHints.trigger.label","Trigger Parameter Hints"),alias:"Trigger Parameter Hints",precondition:I.hasSignatureHelpProvider,kbOpts:{kbExpr:I.editorTextFocus,primary:3082,weight:100}})}run(e,t){const i=db.get(t);i==null||i.trigger({triggerKind:Rl.Invoke})}}ct(db.ID,db,2);Q(lNe);const lB=175,cB=$i.bindToContribution(db.get);re(new cB({id:"closeParameterHints",precondition:Yf.Visible,handler:o=>o.cancel(),kbOpts:{weight:lB,kbExpr:I.focus,primary:9,secondary:[1033]}}));re(new cB({id:"showPrevParameterHint",precondition:G.and(Yf.Visible,Yf.MultipleSignatures),handler:o=>o.previous(),kbOpts:{weight:lB,kbExpr:I.focus,primary:16,secondary:[528],mac:{primary:16,secondary:[528,302]}}}));re(new cB({id:"showNextParameterHint",precondition:G.and(Yf.Visible,Yf.MultipleSignatures),handler:o=>o.next(),kbOpts:{weight:lB,kbExpr:I.focus,primary:18,secondary:[530],mac:{primary:18,secondary:[530,300]}}}));var cNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},dNe=function(o,e){return function(t,i){e(t,i,o)}};class hNe{constructor(e){this.instantiationService=e}init(...e){}}function uNe(o,e){return class extends e{constructor(){super(...arguments),this._autorun=void 0}init(...i){this._autorun=Ps((n,s)=>{const r=Js(o(),n);s.add(this.instantiationService.createInstance(r,...i))})}dispose(){var i;(i=this._autorun)==null||i.dispose()}}}function gNe(o){return SE()?uNe(o,JF):o()}let JF=class extends hNe{constructor(e,t){super(t),this.init(e)}};JF=cNe([dNe(1,we)],JF);ct(v1.ID,gNe(()=>v1),0);E("editor.placeholder.foreground",whe,m("placeholderForeground","Foreground color of the placeholder text in the editor."));var fNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},O0=function(o,e){return function(t,i){e(t,i,o)}};const Wb=new le("renameInputVisible",!1,m("renameInputVisible","Whether the rename input widget is visible"));new le("renameInputFocused",!1,m("renameInputFocused","Whether the rename input widget is focused"));let e4=class{constructor(e,t,i,n,s,r){this._editor=e,this._acceptKeybindings=t,this._themeService=i,this._keybindingService=n,this._logService=r,this.allowEditorOverflow=!0,this._disposables=new K,this._visibleContextKey=Wb.bindTo(s),this._isEditingRenameCandidate=!1,this._nRenameSuggestionsInvocations=0,this._hadAutomaticRenameSuggestionsInvocation=!1,this._candidates=new Set,this._beforeFirstInputFieldEditSW=new ln,this._inputWithButton=new mNe,this._disposables.add(this._inputWithButton),this._editor.addContentWidget(this),this._disposables.add(this._editor.onDidChangeConfiguration(a=>{a.hasChanged(50)&&this._updateFont()})),this._disposables.add(i.onDidColorThemeChange(this._updateStyles,this))}dispose(){this._disposables.dispose(),this._editor.removeContentWidget(this)}getId(){return"__renameInputWidget"}getDomNode(){return this._domNode||(this._domNode=document.createElement("div"),this._domNode.className="monaco-editor rename-box",this._domNode.appendChild(this._inputWithButton.domNode),this._renameCandidateListView=this._disposables.add(new dB(this._domNode,{fontInfo:this._editor.getOption(50),onFocusChange:e=>{this._inputWithButton.input.value=e,this._isEditingRenameCandidate=!1},onSelectionChange:()=>{this._isEditingRenameCandidate=!1,this.acceptInput(!1)}})),this._disposables.add(this._inputWithButton.onDidInputChange(()=>{var e,t,i;((e=this._renameCandidateListView)==null?void 0:e.focusedCandidate)!==void 0&&(this._isEditingRenameCandidate=!0),this._timeBeforeFirstInputFieldEdit??(this._timeBeforeFirstInputFieldEdit=this._beforeFirstInputFieldEditSW.elapsed()),((t=this._renameCandidateProvidersCts)==null?void 0:t.token.isCancellationRequested)===!1&&this._renameCandidateProvidersCts.cancel(),(i=this._renameCandidateListView)==null||i.clearFocus()})),this._label=document.createElement("div"),this._label.className="rename-label",this._domNode.appendChild(this._label),this._updateFont(),this._updateStyles(this._themeService.getColorTheme())),this._domNode}_updateStyles(e){if(!this._domNode)return;const t=e.getColor(Cb),i=e.getColor(F$);this._domNode.style.backgroundColor=String(e.getColor(Wo)??""),this._domNode.style.boxShadow=t?` 0 0 8px 2px ${t}`:"",this._domNode.style.border=i?`1px solid ${i}`:"",this._domNode.style.color=String(e.getColor(H$)??"");const n=e.getColor(V$);this._inputWithButton.domNode.style.backgroundColor=String(e.getColor(g2)??""),this._inputWithButton.input.style.backgroundColor=String(e.getColor(g2)??""),this._inputWithButton.domNode.style.borderWidth=n?"1px":"0px",this._inputWithButton.domNode.style.borderStyle=n?"solid":"none",this._inputWithButton.domNode.style.borderColor=(n==null?void 0:n.toString())??"none"}_updateFont(){if(this._domNode===void 0)return;mt(this._label!==void 0,"RenameWidget#_updateFont: _label must not be undefined given _domNode is defined"),this._editor.applyFontInfo(this._inputWithButton.input);const e=this._editor.getOption(50);this._label.style.fontSize=`${this._computeLabelFontSize(e.fontSize)}px`}_computeLabelFontSize(e){return e*.8}getPosition(){if(!this._visible||!this._editor.hasModel()||!this._editor.getDomNode())return null;const e=Hh(this.getDomNode().ownerDocument.body),t=wi(this._editor.getDomNode()),i=this._getTopForPosition();this._nPxAvailableAbove=i+t.top,this._nPxAvailableBelow=e.height-this._nPxAvailableAbove;const n=this._editor.getOption(67),{totalHeight:s}=w1.getLayoutInfo({lineHeight:n}),r=this._nPxAvailableBelow>s*6?[2,1]:[1,2];return{position:this._position,preference:r}}beforeRender(){var i,n;const[e,t]=this._acceptKeybindings;return this._label.innerText=m({key:"label",comment:['placeholders are keybindings, e.g "F2 to Rename, Shift+F2 to Preview"']},"{0} to Rename, {1} to Preview",(i=this._keybindingService.lookupKeybinding(e))==null?void 0:i.getLabel(),(n=this._keybindingService.lookupKeybinding(t))==null?void 0:n.getLabel()),this._domNode.style.minWidth="200px",null}afterRender(e){if(e===null){this.cancelInput(!0,"afterRender (because position is null)");return}if(!this._editor.hasModel()||!this._editor.getDomNode())return;mt(this._renameCandidateListView),mt(this._nPxAvailableAbove!==void 0),mt(this._nPxAvailableBelow!==void 0);const t=Tc(this._inputWithButton.domNode),i=Tc(this._label);let n;e===2?n=this._nPxAvailableBelow:n=this._nPxAvailableAbove,this._renameCandidateListView.layout({height:n-i-t,width:Xs(this._inputWithButton.domNode)})}acceptInput(e){var t;this._trace("invoking acceptInput"),(t=this._currentAcceptInput)==null||t.call(this,e)}cancelInput(e,t){var i;(i=this._currentCancelInput)==null||i.call(this,e)}focusNextRenameSuggestion(){var e;(e=this._renameCandidateListView)!=null&&e.focusNext()||(this._inputWithButton.input.value=this._currentName)}focusPreviousRenameSuggestion(){var e;(e=this._renameCandidateListView)!=null&&e.focusPrevious()||(this._inputWithButton.input.value=this._currentName)}getInput(e,t,i,n,s){const{start:r,end:a}=this._getSelection(e,t);this._renameCts=s;const l=new K;this._nRenameSuggestionsInvocations=0,this._hadAutomaticRenameSuggestionsInvocation=!1,n===void 0?this._inputWithButton.button.style.display="none":(this._inputWithButton.button.style.display="flex",this._requestRenameCandidatesOnce=n,this._requestRenameCandidates(t,!1),l.add(U(this._inputWithButton.button,"click",()=>this._requestRenameCandidates(t,!0))),l.add(U(this._inputWithButton.button,ie.KEY_DOWN,d=>{const h=new Pt(d);(h.equals(3)||h.equals(10))&&(h.stopPropagation(),h.preventDefault(),this._requestRenameCandidates(t,!0))}))),this._isEditingRenameCandidate=!1,this._domNode.classList.toggle("preview",i),this._position=new A(e.startLineNumber,e.startColumn),this._currentName=t,this._inputWithButton.input.value=t,this._inputWithButton.input.setAttribute("selectionStart",r.toString()),this._inputWithButton.input.setAttribute("selectionEnd",a.toString()),this._inputWithButton.input.size=Math.max((e.endColumn-e.startColumn)*1.1,20),this._beforeFirstInputFieldEditSW.reset(),l.add(ve(()=>{this._renameCts=void 0,s.dispose(!0)})),l.add(ve(()=>{this._renameCandidateProvidersCts!==void 0&&(this._renameCandidateProvidersCts.dispose(!0),this._renameCandidateProvidersCts=void 0)})),l.add(ve(()=>this._candidates.clear()));const c=new gb;return c.p.finally(()=>{l.dispose(),this._hide()}),this._currentCancelInput=d=>{var h;return this._trace("invoking _currentCancelInput"),this._currentAcceptInput=void 0,this._currentCancelInput=void 0,(h=this._renameCandidateListView)==null||h.clearCandidates(),c.complete(d),!0},this._currentAcceptInput=d=>{this._trace("invoking _currentAcceptInput"),mt(this._renameCandidateListView!==void 0);const h=this._renameCandidateListView.nCandidates;let u,g;const f=this._renameCandidateListView.focusedCandidate;if(f!==void 0?(this._trace("using new name from renameSuggestion"),u=f,g={k:"renameSuggestion"}):(this._trace("using new name from inputField"),u=this._inputWithButton.input.value,g=this._isEditingRenameCandidate?{k:"userEditedRenameSuggestion"}:{k:"inputField"}),u===t||u.trim().length===0){this.cancelInput(!0,"_currentAcceptInput (because newName === value || newName.trim().length === 0)");return}this._currentAcceptInput=void 0,this._currentCancelInput=void 0,this._renameCandidateListView.clearCandidates(),c.complete({newName:u,wantsPreview:i&&d,stats:{source:g,nRenameSuggestions:h,timeBeforeFirstInputFieldEdit:this._timeBeforeFirstInputFieldEdit,nRenameSuggestionsInvocations:this._nRenameSuggestionsInvocations,hadAutomaticRenameSuggestionsInvocation:this._hadAutomaticRenameSuggestionsInvocation}})},l.add(s.token.onCancellationRequested(()=>this.cancelInput(!0,"cts.token.onCancellationRequested"))),l.add(this._editor.onDidBlurEditorWidget(()=>{var d;return this.cancelInput(!((d=this._domNode)!=null&&d.ownerDocument.hasFocus()),"editor.onDidBlurEditorWidget")})),this._show(),c.p}_requestRenameCandidates(e,t){if(this._requestRenameCandidatesOnce!==void 0&&(this._renameCandidateProvidersCts!==void 0&&this._renameCandidateProvidersCts.dispose(!0),mt(this._renameCts),this._inputWithButton.buttonState!=="stop")){this._renameCandidateProvidersCts=new li;const i=t?nw.Invoke:nw.Automatic,n=this._requestRenameCandidatesOnce(i,this._renameCandidateProvidersCts.token);if(n.length===0){this._inputWithButton.setSparkleButton();return}t||(this._hadAutomaticRenameSuggestionsInvocation=!0),this._nRenameSuggestionsInvocations+=1,this._inputWithButton.setStopButton(),this._updateRenameCandidates(n,e,this._renameCts.token)}}_getSelection(e,t){mt(this._editor.hasModel());const i=this._editor.getSelection();let n=0,s=t.length;return!x.isEmpty(i)&&!x.spansMultipleLines(i)&&x.containsRange(e,i)&&(n=Math.max(0,i.startColumn-e.startColumn),s=Math.min(e.endColumn,i.endColumn)-e.startColumn),{start:n,end:s}}_show(){this._trace("invoking _show"),this._editor.revealLineInCenterIfOutsideViewport(this._position.lineNumber,0),this._visible=!0,this._visibleContextKey.set(!0),this._editor.layoutContentWidget(this),setTimeout(()=>{this._inputWithButton.input.focus(),this._inputWithButton.input.setSelectionRange(parseInt(this._inputWithButton.input.getAttribute("selectionStart")),parseInt(this._inputWithButton.input.getAttribute("selectionEnd")))},100)}async _updateRenameCandidates(e,t,i){const n=(...c)=>this._trace("_updateRenameCandidates",...c);n("start");const s=await F1(Promise.allSettled(e),i);if(this._inputWithButton.setSparkleButton(),s===void 0){n("returning early - received updateRenameCandidates results - undefined");return}const r=s.flatMap(c=>c.status==="fulfilled"&&Tl(c.value)?c.value:[]);n(`received updateRenameCandidates results - total (unfiltered) ${r.length} candidates.`);const a=Xc(r,c=>c.newSymbolName);n(`distinct candidates - ${a.length} candidates.`);const l=a.filter(({newSymbolName:c})=>c.trim().length>0&&c!==this._inputWithButton.input.value&&c!==t&&!this._candidates.has(c));if(n(`valid distinct candidates - ${r.length} candidates.`),l.forEach(c=>this._candidates.add(c.newSymbolName)),l.length<1){n("returning early - no valid distinct candidates");return}n("setting candidates"),this._renameCandidateListView.setCandidates(l),n("asking editor to re-layout"),this._editor.layoutContentWidget(this)}_hide(){this._trace("invoked _hide"),this._visible=!1,this._visibleContextKey.reset(),this._editor.layoutContentWidget(this)}_getTopForPosition(){const e=this._editor.getVisibleRanges();let t;return e.length>0?t=e[0].startLineNumber:(this._logService.warn("RenameWidget#_getTopForPosition: this should not happen - visibleRanges is empty"),t=Math.max(1,this._position.lineNumber-5)),this._editor.getTopForLineNumber(this._position.lineNumber)-this._editor.getTopForLineNumber(t)}_trace(...e){this._logService.trace("RenameWidget",...e)}};e4=fNe([O0(2,ji),O0(3,yt),O0(4,Ee),O0(5,Nn)],e4);class dB{constructor(e,t){this._disposables=new K,this._availableHeight=0,this._minimumWidth=0,this._lineHeight=t.fontInfo.lineHeight,this._typicalHalfwidthCharacterWidth=t.fontInfo.typicalHalfwidthCharacterWidth,this._listContainer=document.createElement("div"),this._listContainer.className="rename-box rename-candidate-list-container",e.appendChild(this._listContainer),this._listWidget=dB._createListWidget(this._listContainer,this._candidateViewHeight,t.fontInfo),this._listWidget.onDidChangeFocus(i=>{i.elements.length===1&&t.onFocusChange(i.elements[0].newSymbolName)},this._disposables),this._listWidget.onDidChangeSelection(i=>{i.elements.length===1&&t.onSelectionChange()},this._disposables),this._disposables.add(this._listWidget.onDidBlur(i=>{this._listWidget.setFocus([])})),this._listWidget.style(sm({listInactiveFocusForeground:Cw,listInactiveFocusBackground:vw}))}dispose(){this._listWidget.dispose(),this._disposables.dispose()}layout({height:e,width:t}){this._availableHeight=e,this._minimumWidth=t}setCandidates(e){this._listWidget.splice(0,0,e);const t=this._pickListHeight(this._listWidget.length),i=this._pickListWidth(e);this._listWidget.layout(t,i),this._listContainer.style.height=`${t}px`,this._listContainer.style.width=`${i}px`,Hl(m("renameSuggestionsReceivedAria","Received {0} rename suggestions",e.length))}clearCandidates(){this._listContainer.style.height="0px",this._listContainer.style.width="0px",this._listWidget.splice(0,this._listWidget.length,[])}get nCandidates(){return this._listWidget.length}get focusedCandidate(){if(this._listWidget.length===0)return;const e=this._listWidget.getSelectedElements()[0];if(e!==void 0)return e.newSymbolName;const t=this._listWidget.getFocusedElements()[0];if(t!==void 0)return t.newSymbolName}focusNext(){if(this._listWidget.length===0)return!1;const e=this._listWidget.getFocus();if(e.length===0)return this._listWidget.focusFirst(),this._listWidget.reveal(0),!0;if(e[0]===this._listWidget.length-1)return this._listWidget.setFocus([]),this._listWidget.reveal(0),!1;{this._listWidget.focusNext();const t=this._listWidget.getFocus()[0];return this._listWidget.reveal(t),!0}}focusPrevious(){if(this._listWidget.length===0)return!1;const e=this._listWidget.getFocus();if(e.length===0){this._listWidget.focusLast();const t=this._listWidget.getFocus()[0];return this._listWidget.reveal(t),!0}else{if(e[0]===0)return this._listWidget.setFocus([]),!1;{this._listWidget.focusPrevious();const t=this._listWidget.getFocus()[0];return this._listWidget.reveal(t),!0}}}clearFocus(){this._listWidget.setFocus([])}get _candidateViewHeight(){const{totalHeight:e}=w1.getLayoutInfo({lineHeight:this._lineHeight});return e}_pickListHeight(e){const t=this._candidateViewHeight*e;return Math.min(t,this._availableHeight,this._candidateViewHeight*7)}_pickListWidth(e){const t=Math.ceil(Math.max(...e.map(n=>n.newSymbolName.length))*this._typicalHalfwidthCharacterWidth);return Math.max(this._minimumWidth,25+t+10)}static _createListWidget(e,t,i){const n=new class{getTemplateId(r){return"candidate"}getHeight(r){return t}},s=new class{constructor(){this.templateId="candidate"}renderTemplate(r){return new w1(r,i)}renderElement(r,a,l){l.populate(r)}disposeTemplate(r){r.dispose()}};return new Xo("NewSymbolNameCandidates",e,n,[s],{keyboardSupport:!1,mouseSupport:!0,multipleSelectionSupport:!1})}}class mNe{constructor(){this._onDidInputChange=new P,this.onDidInputChange=this._onDidInputChange.event,this._disposables=new K}get domNode(){return this._domNode||(this._domNode=document.createElement("div"),this._domNode.className="rename-input-with-button",this._domNode.style.display="flex",this._domNode.style.flexDirection="row",this._domNode.style.alignItems="center",this._inputNode=document.createElement("input"),this._inputNode.className="rename-input",this._inputNode.type="text",this._inputNode.style.border="none",this._inputNode.setAttribute("aria-label",m("renameAriaLabel","Rename input. Type new name and press Enter to commit.")),this._domNode.appendChild(this._inputNode),this._buttonNode=document.createElement("div"),this._buttonNode.className="rename-suggestions-button",this._buttonNode.setAttribute("tabindex","0"),this._buttonGenHoverText=m("generateRenameSuggestionsButton","Generate new name suggestions"),this._buttonCancelHoverText=m("cancelRenameSuggestionsButton","Cancel"),this._buttonHover=ja().setupManagedHover(Ss("element"),this._buttonNode,this._buttonGenHoverText),this._disposables.add(this._buttonHover),this._domNode.appendChild(this._buttonNode),this._disposables.add(U(this.input,ie.INPUT,()=>this._onDidInputChange.fire())),this._disposables.add(U(this.input,ie.KEY_DOWN,e=>{const t=new Pt(e);(t.keyCode===15||t.keyCode===17)&&this._onDidInputChange.fire()})),this._disposables.add(U(this.input,ie.CLICK,()=>this._onDidInputChange.fire())),this._disposables.add(U(this.input,ie.FOCUS,()=>{this.domNode.style.outlineWidth="1px",this.domNode.style.outlineStyle="solid",this.domNode.style.outlineOffset="-1px",this.domNode.style.outlineColor="var(--vscode-focusBorder)"})),this._disposables.add(U(this.input,ie.BLUR,()=>{this.domNode.style.outline="none"}))),this._domNode}get input(){return mt(this._inputNode),this._inputNode}get button(){return mt(this._buttonNode),this._buttonNode}get buttonState(){return this._buttonState}setSparkleButton(){var e;this._buttonState="sparkle",this._sparkleIcon??(this._sparkleIcon=Rf(te.sparkle)),pn(this.button),this.button.appendChild(this._sparkleIcon),this.button.setAttribute("aria-label","Generating new name suggestions"),(e=this._buttonHover)==null||e.update(this._buttonGenHoverText),this.input.focus()}setStopButton(){var e;this._buttonState="stop",this._stopIcon??(this._stopIcon=Rf(te.primitiveSquare)),pn(this.button),this.button.appendChild(this._stopIcon),this.button.setAttribute("aria-label","Cancel generating new name suggestions"),(e=this._buttonHover)==null||e.update(this._buttonCancelHoverText),this.input.focus()}dispose(){this._disposables.dispose()}}const qv=class qv{constructor(e,t){this._domNode=document.createElement("div"),this._domNode.className="rename-box rename-candidate",this._domNode.style.display="flex",this._domNode.style.columnGap="5px",this._domNode.style.alignItems="center",this._domNode.style.height=`${t.lineHeight}px`,this._domNode.style.padding=`${qv._PADDING}px`;const i=document.createElement("div");i.style.display="flex",i.style.alignItems="center",i.style.width=i.style.height=`${t.lineHeight*.8}px`,this._domNode.appendChild(i),this._icon=Rf(te.sparkle),this._icon.style.display="none",i.appendChild(this._icon),this._label=document.createElement("div"),bn(this._label,t),this._domNode.appendChild(this._label),e.appendChild(this._domNode)}populate(e){this._updateIcon(e),this._updateLabel(e)}_updateIcon(e){var i;const t=!!((i=e.tags)!=null&&i.includes(LR.AIGenerated));this._icon.style.display=t?"inherit":"none"}_updateLabel(e){this._label.innerText=e.newSymbolName}static getLayoutInfo({lineHeight:e}){return{totalHeight:e+qv._PADDING*2}}dispose(){}};qv._PADDING=2;let w1=qv;var pNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},xd=function(o,e){return function(t,i){e(t,i,o)}},t4;class hB{constructor(e,t,i){this.model=e,this.position=t,this._providerRenameIdx=0,this._providers=i.ordered(e)}hasProvider(){return this._providers.length>0}async resolveRenameLocation(e){const t=[];for(this._providerRenameIdx=0;this._providerRenameIdx<this._providers.length;this._providerRenameIdx++){const n=this._providers[this._providerRenameIdx];if(!n.resolveRenameLocation)break;const s=await n.resolveRenameLocation(this.model,this.position,e);if(s){if(s.rejectReason){t.push(s.rejectReason);continue}return s}}this._providerRenameIdx=0;const i=this.model.getWordAtPosition(this.position);return i?{range:new x(this.position.lineNumber,i.startColumn,this.position.lineNumber,i.endColumn),text:i.word,rejectReason:t.length>0?t.join(`
`):void 0}:{range:x.fromPositions(this.position),text:"",rejectReason:t.length>0?t.join(`
`):void 0}}async provideRenameEdits(e,t){return this._provideRenameEdits(e,this._providerRenameIdx,[],t)}async _provideRenameEdits(e,t,i,n){const s=this._providers[t];if(!s)return{edits:[],rejectReason:i.join(`
`)};const r=await s.provideRenameEdits(this.model,this.position,e,n);if(r){if(r.rejectReason)return this._provideRenameEdits(e,t+1,i.concat(r.rejectReason),n)}else return this._provideRenameEdits(e,t+1,i.concat(m("no result","No result.")),n);return r}}async function _Ne(o,e,t,i){const n=new hB(e,t,o),s=await n.resolveRenameLocation(ze.None);return s!=null&&s.rejectReason?{edits:[],rejectReason:s.rejectReason}:n.provideRenameEdits(i,ze.None)}var _f;let Jh=(_f=class{static get(e){return e.getContribution(t4.ID)}constructor(e,t,i,n,s,r,a,l,c){this.editor=e,this._instaService=t,this._notificationService=i,this._bulkEditService=n,this._progressService=s,this._logService=r,this._configService=a,this._languageFeaturesService=l,this._telemetryService=c,this._disposableStore=new K,this._cts=new li,this._renameWidget=this._disposableStore.add(this._instaService.createInstance(e4,this.editor,["acceptRenameInput","acceptRenameInputWithPreview"]))}dispose(){this._disposableStore.dispose(),this._cts.dispose(!0)}async run(){var f,p;const e=this._logService.trace.bind(this._logService,"[rename]");if(this._cts.dispose(!0),this._cts=new li,!this.editor.hasModel()){e("editor has no model");return}const t=this.editor.getPosition(),i=new hB(this.editor.getModel(),t,this._languageFeaturesService.renameProvider);if(!i.hasProvider()){e("skeleton has no provider");return}const n=new Zh(this.editor,5,void 0,this._cts.token);let s;try{e("resolving rename location");const _=i.resolveRenameLocation(n.token);this._progressService.showWhile(_,250),s=await _,e("resolved rename location")}catch(_){_ instanceof qr?e("resolve rename location cancelled",JSON.stringify(_,null,"	")):(e("resolve rename location failed",_ instanceof Error?_:JSON.stringify(_,null,"	")),(typeof _=="string"||Aa(_))&&((f=lo.get(this.editor))==null||f.showMessage(_||m("resolveRenameLocationFailed","An unknown error occurred while resolving rename location"),t)));return}finally{n.dispose()}if(!s){e("returning early - no loc");return}if(s.rejectReason){e(`returning early - rejected with reason: ${s.rejectReason}`,s.rejectReason),(p=lo.get(this.editor))==null||p.showMessage(s.rejectReason,t);return}if(n.token.isCancellationRequested){e("returning early - cts1 cancelled");return}const r=new Zh(this.editor,5,s.range,this._cts.token),a=this.editor.getModel(),l=this._languageFeaturesService.newSymbolNamesProvider.all(a),c=await Promise.all(l.map(async _=>[_,await _.supportsAutomaticNewSymbolNamesTriggerKind??!1])),d=(_,b)=>{let C=c.slice();return _===nw.Automatic&&(C=C.filter(([v,w])=>w)),C.map(([v])=>v.provideNewSymbolNames(a,s.range,_,b))};e("creating rename input field and awaiting its result");const h=this._bulkEditService.hasPreviewHandler()&&this._configService.getValue(this.editor.getModel().uri,"editor.rename.enablePreview"),u=await this._renameWidget.getInput(s.range,s.text,h,l.length>0?d:void 0,r);if(e("received response from rename input field"),l.length>0&&this._reportTelemetry(l.length,a.getLanguageId(),u),typeof u=="boolean"){e(`returning early - rename input field response - ${u}`),u&&this.editor.focus(),r.dispose();return}this.editor.focus(),e("requesting rename edits");const g=F1(i.provideRenameEdits(u.newName,r.token),r.token).then(async _=>{if(!_){e("returning early - no rename edits result");return}if(!this.editor.hasModel()){e("returning early - no model after rename edits are provided");return}if(_.rejectReason){e(`returning early - rejected with reason: ${_.rejectReason}`),this._notificationService.info(_.rejectReason);return}this.editor.setSelection(x.fromPositions(this.editor.getSelection().getPosition())),e("applying edits"),this._bulkEditService.apply(_,{editor:this.editor,showPreview:u.wantsPreview,label:m("label","Renaming '{0}' to '{1}'",s==null?void 0:s.text,u.newName),code:"undoredo.rename",quotableLabel:m("quotableLabel","Renaming {0} to {1}",s==null?void 0:s.text,u.newName),respectAutoSaveConfig:!0}).then(b=>{e("edits applied"),b.ariaSummary&&Cs(m("aria","Successfully renamed '{0}' to '{1}'. Summary: {2}",s.text,u.newName,b.ariaSummary))}).catch(b=>{e(`error when applying edits ${JSON.stringify(b,null,"	")}`),this._notificationService.error(m("rename.failedApply","Rename failed to apply edits")),this._logService.error(b)})},_=>{e("error when providing rename edits",JSON.stringify(_,null,"	")),this._notificationService.error(m("rename.failed","Rename failed to compute edits")),this._logService.error(_)}).finally(()=>{r.dispose()});return e("returning rename operation"),this._progressService.showWhile(g,250),g}acceptRenameInput(e){this._renameWidget.acceptInput(e)}cancelRenameInput(){this._renameWidget.cancelInput(!0,"cancelRenameInput command")}focusNextRenameSuggestion(){this._renameWidget.focusNextRenameSuggestion()}focusPreviousRenameSuggestion(){this._renameWidget.focusPreviousRenameSuggestion()}_reportTelemetry(e,t,i){const n=typeof i=="boolean"?{kind:"cancelled",languageId:t,nRenameSuggestionProviders:e}:{kind:"accepted",languageId:t,nRenameSuggestionProviders:e,source:i.stats.source.k,nRenameSuggestions:i.stats.nRenameSuggestions,timeBeforeFirstInputFieldEdit:i.stats.timeBeforeFirstInputFieldEdit,wantsPreview:i.wantsPreview,nRenameSuggestionsInvocations:i.stats.nRenameSuggestionsInvocations,hadAutomaticRenameSuggestionsInvocation:i.stats.hadAutomaticRenameSuggestionsInvocation};this._telemetryService.publicLog2("renameInvokedEvent",n)}},t4=_f,_f.ID="editor.contrib.renameController",_f);Jh=t4=pNe([xd(1,we),xd(2,Ei),xd(3,j1),xd(4,nu),xd(5,Nn),xd(6,PI),xd(7,ge),xd(8,Bn)],Jh);class bNe extends de{constructor(){super({id:"editor.action.rename",label:m("rename.label","Rename Symbol"),alias:"Rename Symbol",precondition:G.and(I.writable,I.hasRenameProvider),kbOpts:{kbExpr:I.editorTextFocus,primary:60,weight:100},contextMenuOpts:{group:"1_modification",order:1.1}})}runCommand(e,t){const i=e.get(pt),[n,s]=Array.isArray(t)&&t||[void 0,void 0];return Se.isUri(n)&&A.isIPosition(s)?i.openCodeEditor({resource:n},i.getActiveCodeEditor()).then(r=>{r&&(r.setPosition(s),r.invokeWithinContext(a=>(this.reportTelemetry(a,r),this.run(a,r))))},Oe):super.runCommand(e,t)}run(e,t){const i=e.get(Nn),n=Jh.get(t);return n?(i.trace("[RenameAction] got controller, running..."),n.run()):(i.trace("[RenameAction] returning early - controller missing"),Promise.resolve())}}ct(Jh.ID,Jh,4);Q(bNe);const uB=$i.bindToContribution(Jh.get);re(new uB({id:"acceptRenameInput",precondition:Wb,handler:o=>o.acceptRenameInput(!1),kbOpts:{weight:199,kbExpr:G.and(I.focus,G.not("isComposing")),primary:3}}));re(new uB({id:"acceptRenameInputWithPreview",precondition:G.and(Wb,G.has("config.editor.rename.enablePreview")),handler:o=>o.acceptRenameInput(!0),kbOpts:{weight:199,kbExpr:G.and(I.focus,G.not("isComposing")),primary:2051}}));re(new uB({id:"cancelRenameInput",precondition:Wb,handler:o=>o.cancelRenameInput(),kbOpts:{weight:199,kbExpr:I.focus,primary:9,secondary:[1033]}}));Ot(class extends ws{constructor(){super({id:"focusNextRenameSuggestion",title:{...Te("focusNextRenameSuggestion","Focus Next Rename Suggestion")},precondition:Wb,keybinding:[{primary:18,weight:199}]})}run(e){const t=e.get(pt).getFocusedCodeEditor();if(!t)return;const i=Jh.get(t);i&&i.focusNextRenameSuggestion()}});Ot(class extends ws{constructor(){super({id:"focusPreviousRenameSuggestion",title:{...Te("focusPreviousRenameSuggestion","Focus Previous Rename Suggestion")},precondition:Wb,keybinding:[{primary:16,weight:199}]})}run(e){const t=e.get(pt).getFocusedCodeEditor();if(!t)return;const i=Jh.get(t);i&&i.focusPreviousRenameSuggestion()}});Vs("_executeDocumentRenameProvider",function(o,e,t,...i){const[n]=i;mt(typeof n=="string");const{renameProvider:s}=o.get(ge);return _Ne(s,e,t,n)});Vs("_executePrepareRename",async function(o,e,t){const{renameProvider:i}=o.get(ge),s=await new hB(e,t,i).resolveRenameLocation(ze.None);if(s!=null&&s.rejectReason)throw new Error(s.rejectReason);return s});ci.as(ta.Configuration).registerConfiguration({id:"editor",properties:{"editor.rename.enablePreview":{scope:5,description:m("enablePreview","Enable/disable the ability to preview changes before renaming"),default:!0,type:"boolean"}}});var CNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},j6=function(o,e){return function(t,i){e(t,i,o)}},v_;let _k=(v_=class extends F{constructor(e,t,i){super(),this.editor=e,this.languageConfigurationService=t,this.editorWorkerService=i,this.decorations=this.editor.createDecorationsCollection(),this.options=this.createOptions(e.getOption(73)),this.computePromise=null,this.currentOccurrences={},this._register(e.onDidChangeModel(n=>{this.currentOccurrences={},this.options=this.createOptions(e.getOption(73)),this.stop(),this.computeSectionHeaders.schedule(0)})),this._register(e.onDidChangeModelLanguage(n=>{this.currentOccurrences={},this.options=this.createOptions(e.getOption(73)),this.stop(),this.computeSectionHeaders.schedule(0)})),this._register(t.onDidChange(n=>{var r;const s=(r=this.editor.getModel())==null?void 0:r.getLanguageId();s&&n.affects(s)&&(this.currentOccurrences={},this.options=this.createOptions(e.getOption(73)),this.stop(),this.computeSectionHeaders.schedule(0))})),this._register(e.onDidChangeConfiguration(n=>{this.options&&!n.hasChanged(73)||(this.options=this.createOptions(e.getOption(73)),this.updateDecorations([]),this.stop(),this.computeSectionHeaders.schedule(0))})),this._register(this.editor.onDidChangeModelContent(n=>{this.computeSectionHeaders.schedule()})),this._register(e.onDidChangeModelTokens(n=>{this.computeSectionHeaders.isScheduled()||this.computeSectionHeaders.schedule(1e3)})),this.computeSectionHeaders=this._register(new It(()=>{this.findSectionHeaders()},250)),this.computeSectionHeaders.schedule(0)}createOptions(e){if(!e||!this.editor.hasModel())return;const t=this.editor.getModel().getLanguageId();if(!t)return;const i=this.languageConfigurationService.getLanguageConfiguration(t).comments,n=this.languageConfigurationService.getLanguageConfiguration(t).foldingRules;if(!(!i&&!(n!=null&&n.markers)))return{foldingRules:n,findMarkSectionHeaders:e.showMarkSectionHeaders,findRegionSectionHeaders:e.showRegionSectionHeaders}}findSectionHeaders(){var i,n;if(!this.editor.hasModel()||!((i=this.options)!=null&&i.findMarkSectionHeaders)&&!((n=this.options)!=null&&n.findRegionSectionHeaders))return;const e=this.editor.getModel();if(e.isDisposed()||e.isTooLargeForSyncing())return;const t=e.getVersionId();this.editorWorkerService.findSectionHeaders(e.uri,this.options).then(s=>{e.isDisposed()||e.getVersionId()!==t||this.updateDecorations(s)})}updateDecorations(e){const t=this.editor.getModel();t&&(e=e.filter(s=>{if(!s.shouldBeInComments)return!0;const r=t.validateRange(s.range),a=t.tokenization.getLineTokens(r.startLineNumber),l=a.findTokenIndexAtOffset(r.startColumn-1),c=a.getStandardTokenType(l);return a.getLanguageId(l)===t.getLanguageId()&&c===1}));const i=Object.values(this.currentOccurrences).map(s=>s.decorationId),n=e.map(s=>vNe(s));this.editor.changeDecorations(s=>{const r=s.deltaDecorations(i,n);this.currentOccurrences={};for(let a=0,l=r.length;a<l;a++){const c={sectionHeader:e[a],decorationId:r[a]};this.currentOccurrences[c.decorationId]=c}})}stop(){this.computeSectionHeaders.cancel(),this.computePromise&&(this.computePromise.cancel(),this.computePromise=null)}dispose(){super.dispose(),this.stop(),this.decorations.clear()}},v_.ID="editor.sectionHeaderDetector",v_);_k=CNe([j6(1,di),j6(2,Us)],_k);function vNe(o){return{range:o.range,options:We.createDynamic({description:"section-header",stickiness:3,collapseOnReplaceEdit:!0,minimap:{color:void 0,position:1,sectionHeaderStyle:o.hasSeparatorLine?2:1,sectionHeaderText:o.text}})}}ct(_k.ID,_k,1);function wNe(o){for(let e=0,t=o.length;e<t;e+=4){const i=o[e+0],n=o[e+1],s=o[e+2],r=o[e+3];o[e+0]=r,o[e+1]=s,o[e+2]=n,o[e+3]=i}}function SNe(o){const e=new Uint8Array(o.buffer,o.byteOffset,o.length*4);return Tz()||wNe(e),RI.wrap(e)}function AY(o){const e=new Uint32Array(yNe(o));let t=0;if(e[t++]=o.id,o.type==="full")e[t++]=1,e[t++]=o.data.length,e.set(o.data,t),t+=o.data.length;else{e[t++]=2,e[t++]=o.deltas.length;for(const i of o.deltas)e[t++]=i.start,e[t++]=i.deleteCount,i.data?(e[t++]=i.data.length,e.set(i.data,t),t+=i.data.length):e[t++]=0}return SNe(e)}function yNe(o){let e=0;if(e+=2,o.type==="full")e+=1+o.data.length;else{e+=1,e+=3*o.deltas.length;for(const t of o.deltas)t.data&&(e+=t.data.length)}return e}function $E(o){return o&&!!o.data}function PY(o){return o&&Array.isArray(o.edits)}class LNe{constructor(e,t,i){this.provider=e,this.tokens=t,this.error=i}}function OY(o,e){return o.has(e)}function xNe(o,e){const t=o.orderedGroups(e);return t.length>0?t[0]:[]}async function FY(o,e,t,i,n){const s=xNe(o,e),r=await Promise.all(s.map(async a=>{let l,c=null;try{l=await a.provideDocumentSemanticTokens(e,a===t?i:null,n)}catch(d){c=d,l=null}return(!l||!$E(l)&&!PY(l))&&(l=null),new LNe(a,l,c)}));for(const a of r){if(a.error)throw a.error;if(a.tokens)return a}return r.length>0?r[0]:null}function kNe(o,e){const t=o.orderedGroups(e);return t.length>0?t[0]:null}class DNe{constructor(e,t){this.provider=e,this.tokens=t}}function INe(o,e){return o.has(e)}function WY(o,e){const t=o.orderedGroups(e);return t.length>0?t[0]:[]}async function gB(o,e,t,i){const n=WY(o,e),s=await Promise.all(n.map(async r=>{let a;try{a=await r.provideDocumentRangeSemanticTokens(e,t,i)}catch(l){Si(l),a=null}return(!a||!$E(a))&&(a=null),new DNe(r,a)}));for(const r of s)if(r.tokens)return r;return s.length>0?s[0]:null}rt.registerCommand("_provideDocumentSemanticTokensLegend",async(o,...e)=>{const[t]=e;mt(t instanceof Se);const i=o.get(Kt).getModel(t);if(!i)return;const{documentSemanticTokensProvider:n}=o.get(ge),s=kNe(n,i);return s?s[0].getLegend():o.get(Ht).executeCommand("_provideDocumentRangeSemanticTokensLegend",t)});rt.registerCommand("_provideDocumentSemanticTokens",async(o,...e)=>{const[t]=e;mt(t instanceof Se);const i=o.get(Kt).getModel(t);if(!i)return;const{documentSemanticTokensProvider:n}=o.get(ge);if(!OY(n,i))return o.get(Ht).executeCommand("_provideDocumentRangeSemanticTokens",t,i.getFullModelRange());const s=await FY(n,i,null,null,ze.None);if(!s)return;const{provider:r,tokens:a}=s;if(!a||!$E(a))return;const l=AY({id:0,type:"full",data:a.data});return a.resultId&&r.releaseDocumentSemanticTokens(a.resultId),l});rt.registerCommand("_provideDocumentRangeSemanticTokensLegend",async(o,...e)=>{const[t,i]=e;mt(t instanceof Se);const n=o.get(Kt).getModel(t);if(!n)return;const{documentRangeSemanticTokensProvider:s}=o.get(ge),r=WY(s,n);if(r.length===0)return;if(r.length===1)return r[0].getLegend();if(!i||!x.isIRange(i))return console.warn("provideDocumentRangeSemanticTokensLegend might be out-of-sync with provideDocumentRangeSemanticTokens unless a range argument is passed in"),r[0].getLegend();const a=await gB(s,n,x.lift(i),ze.None);if(a)return a.provider.getLegend()});rt.registerCommand("_provideDocumentRangeSemanticTokens",async(o,...e)=>{const[t,i]=e;mt(t instanceof Se),mt(x.isIRange(i));const n=o.get(Kt).getModel(t);if(!n)return;const{documentRangeSemanticTokensProvider:s}=o.get(ge),r=await gB(s,n,x.lift(i),ze.None);if(!(!r||!r.tokens))return AY({id:0,type:"full",data:r.tokens.data})});const fB="editor.semanticHighlighting";function Sy(o,e,t){var n;const i=(n=t.getValue(fB,{overrideIdentifier:o.getLanguageId(),resource:o.uri}))==null?void 0:n.enabled;return typeof i=="boolean"?i:e.getColorTheme().semanticHighlighting}var BY=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Sl=function(o,e){return function(t,i){e(t,i,o)}},Ed;let i4=class extends F{constructor(e,t,i,n,s,r){super(),this._watchers=Object.create(null);const a=d=>{this._watchers[d.uri.toString()]=new n4(d,e,i,s,r)},l=(d,h)=>{h.dispose(),delete this._watchers[d.uri.toString()]},c=()=>{for(const d of t.getModels()){const h=this._watchers[d.uri.toString()];Sy(d,i,n)?h||a(d):h&&l(d,h)}};t.getModels().forEach(d=>{Sy(d,i,n)&&a(d)}),this._register(t.onModelAdded(d=>{Sy(d,i,n)&&a(d)})),this._register(t.onModelRemoved(d=>{const h=this._watchers[d.uri.toString()];h&&l(d,h)})),this._register(n.onDidChangeConfiguration(d=>{d.affectsConfiguration(fB)&&c()})),this._register(i.onDidColorThemeChange(c))}dispose(){for(const e of Object.values(this._watchers))e.dispose();super.dispose()}};i4=BY([Sl(0,FI),Sl(1,Kt),Sl(2,ji),Sl(3,qe),Sl(4,Zo),Sl(5,ge)],i4);var Nh;let n4=(Nh=class extends F{constructor(e,t,i,n,s){super(),this._semanticTokensStylingService=t,this._isDisposed=!1,this._model=e,this._provider=s.documentSemanticTokensProvider,this._debounceInformation=n.for(this._provider,"DocumentSemanticTokens",{min:Ed.REQUEST_MIN_DELAY,max:Ed.REQUEST_MAX_DELAY}),this._fetchDocumentSemanticTokens=this._register(new It(()=>this._fetchDocumentSemanticTokensNow(),Ed.REQUEST_MIN_DELAY)),this._currentDocumentResponse=null,this._currentDocumentRequestCancellationTokenSource=null,this._documentProvidersChangeListeners=[],this._providersChangedDuringRequest=!1,this._register(this._model.onDidChangeContent(()=>{this._fetchDocumentSemanticTokens.isScheduled()||this._fetchDocumentSemanticTokens.schedule(this._debounceInformation.get(this._model))})),this._register(this._model.onDidChangeAttached(()=>{this._fetchDocumentSemanticTokens.isScheduled()||this._fetchDocumentSemanticTokens.schedule(this._debounceInformation.get(this._model))})),this._register(this._model.onDidChangeLanguage(()=>{this._currentDocumentResponse&&(this._currentDocumentResponse.dispose(),this._currentDocumentResponse=null),this._currentDocumentRequestCancellationTokenSource&&(this._currentDocumentRequestCancellationTokenSource.cancel(),this._currentDocumentRequestCancellationTokenSource=null),this._setDocumentSemanticTokens(null,null,null,[]),this._fetchDocumentSemanticTokens.schedule(0)}));const r=()=>{Tt(this._documentProvidersChangeListeners),this._documentProvidersChangeListeners=[];for(const a of this._provider.all(e))typeof a.onDidChange=="function"&&this._documentProvidersChangeListeners.push(a.onDidChange(()=>{if(this._currentDocumentRequestCancellationTokenSource){this._providersChangedDuringRequest=!0;return}this._fetchDocumentSemanticTokens.schedule(0)}))};r(),this._register(this._provider.onDidChange(()=>{r(),this._fetchDocumentSemanticTokens.schedule(this._debounceInformation.get(this._model))})),this._register(i.onDidColorThemeChange(a=>{this._setDocumentSemanticTokens(null,null,null,[]),this._fetchDocumentSemanticTokens.schedule(this._debounceInformation.get(this._model))})),this._fetchDocumentSemanticTokens.schedule(0)}dispose(){this._currentDocumentResponse&&(this._currentDocumentResponse.dispose(),this._currentDocumentResponse=null),this._currentDocumentRequestCancellationTokenSource&&(this._currentDocumentRequestCancellationTokenSource.cancel(),this._currentDocumentRequestCancellationTokenSource=null),Tt(this._documentProvidersChangeListeners),this._documentProvidersChangeListeners=[],this._setDocumentSemanticTokens(null,null,null,[]),this._isDisposed=!0,super.dispose()}_fetchDocumentSemanticTokensNow(){if(this._currentDocumentRequestCancellationTokenSource)return;if(!OY(this._provider,this._model)){this._currentDocumentResponse&&this._model.tokenization.setSemanticTokens(null,!1);return}if(!this._model.isAttachedToEditor())return;const e=new li,t=this._currentDocumentResponse?this._currentDocumentResponse.provider:null,i=this._currentDocumentResponse&&this._currentDocumentResponse.resultId||null,n=FY(this._provider,this._model,t,i,e.token);this._currentDocumentRequestCancellationTokenSource=e,this._providersChangedDuringRequest=!1;const s=[],r=this._model.onDidChangeContent(l=>{s.push(l)}),a=new ln(!1);n.then(l=>{if(this._debounceInformation.update(this._model,a.elapsed()),this._currentDocumentRequestCancellationTokenSource=null,r.dispose(),!l)this._setDocumentSemanticTokens(null,null,null,s);else{const{provider:c,tokens:d}=l,h=this._semanticTokensStylingService.getStyling(c);this._setDocumentSemanticTokens(c,d||null,h,s)}},l=>{l&&(yr(l)||typeof l.message=="string"&&l.message.indexOf("busy")!==-1)||Oe(l),this._currentDocumentRequestCancellationTokenSource=null,r.dispose(),(s.length>0||this._providersChangedDuringRequest)&&(this._fetchDocumentSemanticTokens.isScheduled()||this._fetchDocumentSemanticTokens.schedule(this._debounceInformation.get(this._model)))})}static _copy(e,t,i,n,s){s=Math.min(s,i.length-n,e.length-t);for(let r=0;r<s;r++)i[n+r]=e[t+r]}_setDocumentSemanticTokens(e,t,i,n){const s=this._currentDocumentResponse,r=()=>{(n.length>0||this._providersChangedDuringRequest)&&!this._fetchDocumentSemanticTokens.isScheduled()&&this._fetchDocumentSemanticTokens.schedule(this._debounceInformation.get(this._model))};if(this._currentDocumentResponse&&(this._currentDocumentResponse.dispose(),this._currentDocumentResponse=null),this._isDisposed){e&&t&&e.releaseDocumentSemanticTokens(t.resultId);return}if(!e||!i){this._model.tokenization.setSemanticTokens(null,!1);return}if(!t){this._model.tokenization.setSemanticTokens(null,!0),r();return}if(PY(t)){if(!s){this._model.tokenization.setSemanticTokens(null,!0);return}if(t.edits.length===0)t={resultId:t.resultId,data:s.data};else{let a=0;for(const u of t.edits)a+=(u.data?u.data.length:0)-u.deleteCount;const l=s.data,c=new Uint32Array(l.length+a);let d=l.length,h=c.length;for(let u=t.edits.length-1;u>=0;u--){const g=t.edits[u];if(g.start>l.length){i.warnInvalidEditStart(s.resultId,t.resultId,u,g.start,l.length),this._model.tokenization.setSemanticTokens(null,!0);return}const f=d-(g.start+g.deleteCount);f>0&&(Ed._copy(l,d-f,c,h-f,f),h-=f),g.data&&(Ed._copy(g.data,0,c,h-g.data.length,g.data.length),h-=g.data.length),d=g.start}d>0&&Ed._copy(l,0,c,0,d),t={resultId:t.resultId,data:c}}}if($E(t)){this._currentDocumentResponse=new ENe(e,t.resultId,t.data);const a=S$(t,i,this._model.getLanguageId());if(n.length>0)for(const l of n)for(const c of a)for(const d of l.changes)c.applyEdit(d.range,d.text);this._model.tokenization.setSemanticTokens(a,!0)}else this._model.tokenization.setSemanticTokens(null,!0);r()}},Ed=Nh,Nh.REQUEST_MIN_DELAY=300,Nh.REQUEST_MAX_DELAY=2e3,Nh);n4=Ed=BY([Sl(1,FI),Sl(2,ji),Sl(3,Zo),Sl(4,ge)],n4);class ENe{constructor(e,t,i){this.provider=e,this.resultId=t,this.data=i}dispose(){this.provider.releaseDocumentSemanticTokens(this.resultId)}}Nb(i4);var NNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},bC=function(o,e){return function(t,i){e(t,i,o)}},w_;let bk=(w_=class extends F{constructor(e,t,i,n,s,r){super(),this._semanticTokensStylingService=t,this._themeService=i,this._configurationService=n,this._editor=e,this._provider=r.documentRangeSemanticTokensProvider,this._debounceInformation=s.for(this._provider,"DocumentRangeSemanticTokens",{min:100,max:500}),this._tokenizeViewport=this._register(new It(()=>this._tokenizeViewportNow(),100)),this._outstandingRequests=[];const a=()=>{this._editor.hasModel()&&this._tokenizeViewport.schedule(this._debounceInformation.get(this._editor.getModel()))};this._register(this._editor.onDidScrollChange(()=>{a()})),this._register(this._editor.onDidChangeModel(()=>{this._cancelAll(),a()})),this._register(this._editor.onDidChangeModelContent(l=>{this._cancelAll(),a()})),this._register(this._provider.onDidChange(()=>{this._cancelAll(),a()})),this._register(this._configurationService.onDidChangeConfiguration(l=>{l.affectsConfiguration(fB)&&(this._cancelAll(),a())})),this._register(this._themeService.onDidColorThemeChange(()=>{this._cancelAll(),a()})),a()}_cancelAll(){for(const e of this._outstandingRequests)e.cancel();this._outstandingRequests=[]}_removeOutstandingRequest(e){for(let t=0,i=this._outstandingRequests.length;t<i;t++)if(this._outstandingRequests[t]===e){this._outstandingRequests.splice(t,1);return}}_tokenizeViewportNow(){if(!this._editor.hasModel())return;const e=this._editor.getModel();if(e.tokenization.hasCompleteSemanticTokens())return;if(!Sy(e,this._themeService,this._configurationService)){e.tokenization.hasSomeSemanticTokens()&&e.tokenization.setSemanticTokens(null,!1);return}if(!INe(this._provider,e)){e.tokenization.hasSomeSemanticTokens()&&e.tokenization.setSemanticTokens(null,!1);return}const t=this._editor.getVisibleRangesPlusViewportAboveBelow();this._outstandingRequests=this._outstandingRequests.concat(t.map(i=>this._requestRange(e,i)))}_requestRange(e,t){const i=e.getVersionId(),n=Yi(r=>Promise.resolve(gB(this._provider,e,t,r))),s=new ln(!1);return n.then(r=>{if(this._debounceInformation.update(e,s.elapsed()),!r||!r.tokens||e.isDisposed()||e.getVersionId()!==i)return;const{provider:a,tokens:l}=r,c=this._semanticTokensStylingService.getStyling(a);e.tokenization.setPartialSemanticTokens(t,S$(l,c,e.getLanguageId()))}).then(()=>this._removeOutstandingRequest(n),()=>this._removeOutstandingRequest(n)),n}},w_.ID="editor.contrib.viewportSemanticTokens",w_);bk=NNe([bC(1,FI),bC(2,ji),bC(3,qe),bC(4,Zo),bC(5,ge)],bk);ct(bk.ID,bk,1);class TNe{constructor(e=!0){this.selectSubwords=e}provideSelectionRanges(e,t){const i=[];for(const n of t){const s=[];i.push(s),this.selectSubwords&&this._addInWordRanges(s,e,n),this._addWordRanges(s,e,n),this._addWhitespaceLine(s,e,n),s.push({range:e.getFullModelRange()})}return i}_addInWordRanges(e,t,i){const n=t.getWordAtPosition(i);if(!n)return;const{word:s,startColumn:r}=n,a=i.column-r;let l=a,c=a,d=0;for(;l>=0;l--){const h=s.charCodeAt(l);if(l!==a&&(h===95||h===45))break;if(qd(h)&&bl(d))break;d=h}for(l+=1;c<s.length;c++){const h=s.charCodeAt(c);if(bl(h)&&qd(d))break;if(h===95||h===45)break;d=h}l<c&&e.push({range:new x(i.lineNumber,r+l,i.lineNumber,r+c)})}_addWordRanges(e,t,i){const n=t.getWordAtPosition(i);n&&e.push({range:new x(i.lineNumber,n.startColumn,i.lineNumber,n.endColumn)})}_addWhitespaceLine(e,t,i){t.getLineLength(i.lineNumber)>0&&t.getLineFirstNonWhitespaceColumn(i.lineNumber)===0&&t.getLineLastNonWhitespaceColumn(i.lineNumber)===0&&e.push({range:new x(i.lineNumber,1,i.lineNumber,t.getLineMaxColumn(i.lineNumber))})}}var RNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},MNe=function(o,e){return function(t,i){e(t,i,o)}},s4;class mB{constructor(e,t){this.index=e,this.ranges=t}mov(e){const t=this.index+(e?1:-1);if(t<0||t>=this.ranges.length)return this;const i=new mB(t,this.ranges);return i.ranges[t].equalsRange(this.ranges[this.index])?i.mov(e):i}}var bf;let S1=(bf=class{static get(e){return e.getContribution(s4.ID)}constructor(e,t){this._editor=e,this._languageFeaturesService=t,this._ignoreSelection=!1}dispose(){var e;(e=this._selectionListener)==null||e.dispose()}async run(e){if(!this._editor.hasModel())return;const t=this._editor.getSelections(),i=this._editor.getModel();if(this._state||await VY(this._languageFeaturesService.selectionRangeProvider,i,t.map(s=>s.getPosition()),this._editor.getOption(114),ze.None).then(s=>{var r;if(!(!Qn(s)||s.length!==t.length)&&!(!this._editor.hasModel()||!ii(this._editor.getSelections(),t,(a,l)=>a.equalsSelection(l)))){for(let a=0;a<s.length;a++)s[a]=s[a].filter(l=>l.containsPosition(t[a].getStartPosition())&&l.containsPosition(t[a].getEndPosition())),s[a].unshift(t[a]);this._state=s.map(a=>new mB(0,a)),(r=this._selectionListener)==null||r.dispose(),this._selectionListener=this._editor.onDidChangeCursorPosition(()=>{var a;this._ignoreSelection||((a=this._selectionListener)==null||a.dispose(),this._state=void 0)})}}),!this._state)return;this._state=this._state.map(s=>s.mov(e));const n=this._state.map(s=>pe.fromPositions(s.ranges[s.index].getStartPosition(),s.ranges[s.index].getEndPosition()));this._ignoreSelection=!0;try{this._editor.setSelections(n)}finally{this._ignoreSelection=!1}}},s4=bf,bf.ID="editor.contrib.smartSelectController",bf);S1=s4=RNe([MNe(1,ge)],S1);class HY extends de{constructor(e,t){super(t),this._forward=e}async run(e,t){const i=S1.get(t);i&&await i.run(this._forward)}}class ANe extends HY{constructor(){super(!0,{id:"editor.action.smartSelect.expand",label:m("smartSelect.expand","Expand Selection"),alias:"Expand Selection",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:1553,mac:{primary:3345,secondary:[1297]},weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"1_basic",title:m({key:"miSmartSelectGrow",comment:["&& denotes a mnemonic"]},"&&Expand Selection"),order:2}})}}rt.registerCommandAlias("editor.action.smartSelect.grow","editor.action.smartSelect.expand");class PNe extends HY{constructor(){super(!1,{id:"editor.action.smartSelect.shrink",label:m("smartSelect.shrink","Shrink Selection"),alias:"Shrink Selection",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:1551,mac:{primary:3343,secondary:[1295]},weight:100},menuOpts:{menuId:me.MenubarSelectionMenu,group:"1_basic",title:m({key:"miSmartSelectShrink",comment:["&& denotes a mnemonic"]},"&&Shrink Selection"),order:3}})}}ct(S1.ID,S1,4);Q(ANe);Q(PNe);async function VY(o,e,t,i,n){const s=o.all(e).concat(new TNe(i.selectSubwords));s.length===1&&s.unshift(new ik);const r=[],a=[];for(const l of s)r.push(Promise.resolve(l.provideSelectionRanges(e,t,n)).then(c=>{if(Qn(c)&&c.length===t.length)for(let d=0;d<t.length;d++){a[d]||(a[d]=[]);for(const h of c[d])x.isIRange(h.range)&&x.containsPosition(h.range,t[d])&&a[d].push(x.lift(h.range))}},Si));return await Promise.all(r),a.map(l=>{if(l.length===0)return[];l.sort((u,g)=>A.isBefore(u.getStartPosition(),g.getStartPosition())?1:A.isBefore(g.getStartPosition(),u.getStartPosition())||A.isBefore(u.getEndPosition(),g.getEndPosition())?-1:A.isBefore(g.getEndPosition(),u.getEndPosition())?1:0);const c=[];let d;for(const u of l)(!d||x.containsRange(u,d)&&!x.equalsRange(u,d))&&(c.push(u),d=u);if(!i.selectLeadingAndTrailingWhitespace)return c;const h=[c[0]];for(let u=1;u<c.length;u++){const g=c[u-1],f=c[u];if(f.startLineNumber!==g.startLineNumber||f.endLineNumber!==g.endLineNumber){const p=new x(g.startLineNumber,e.getLineFirstNonWhitespaceColumn(g.startLineNumber),g.endLineNumber,e.getLineLastNonWhitespaceColumn(g.endLineNumber));p.containsRange(g)&&!p.equalsRange(g)&&f.containsRange(p)&&!f.equalsRange(p)&&h.push(p);const _=new x(g.startLineNumber,1,g.endLineNumber,e.getLineMaxColumn(g.endLineNumber));_.containsRange(g)&&!_.equalsRange(p)&&f.containsRange(_)&&!f.equalsRange(_)&&h.push(_)}h.push(f)}return h})}rt.registerCommand("_executeSelectionRangeProvider",async function(o,...e){const[t,i]=e;mt(Se.isUri(t));const n=o.get(ge).selectionRangeProvider,s=await o.get(Hs).createModelReference(t);try{return VY(n,s.object.textEditorModel,i,{selectLeadingAndTrailingWhitespace:!0,selectSubwords:!0},ze.None)}finally{s.dispose()}});const ONe=Object.freeze({View:Te("view","View"),Help:Te("help","Help"),Test:Te("test","Test"),File:Te("file","File"),Preferences:Te("preferences","Preferences"),Developer:Te({key:"developer",comment:["A developer on Code itself or someone diagnosing issues in Code"]},"Developer")});class kv{constructor(e,t,i,n=null){this.startLineNumbers=e,this.endLineNumbers=t,this.lastLineRelativePosition=i,this.showEndForLine=n}equals(e){return!!e&&this.lastLineRelativePosition===e.lastLineRelativePosition&&this.showEndForLine===e.showEndForLine&&ii(this.startLineNumbers,e.startLineNumbers)&&ii(this.endLineNumbers,e.endLineNumbers)}static get Empty(){return new kv([],[],0)}}const K6=hd("stickyScrollViewLayer",{createHTML:o=>o}),nR="data-sticky-line-index",q6="data-sticky-is-line",FNe="data-sticky-is-line-number",G6="data-sticky-is-folding-icon";class WNe extends F{constructor(e){super(),this._editor=e,this._foldingIconStore=new K,this._rootDomNode=document.createElement("div"),this._lineNumbersDomNode=document.createElement("div"),this._linesDomNodeScrollable=document.createElement("div"),this._linesDomNode=document.createElement("div"),this._lineHeight=this._editor.getOption(67),this._renderedStickyLines=[],this._lineNumbers=[],this._lastLineRelativePosition=0,this._minContentWidthInPx=0,this._isOnGlyphMargin=!1,this._lineNumbersDomNode.className="sticky-widget-line-numbers",this._lineNumbersDomNode.setAttribute("role","none"),this._linesDomNode.className="sticky-widget-lines",this._linesDomNode.setAttribute("role","list"),this._linesDomNodeScrollable.className="sticky-widget-lines-scrollable",this._linesDomNodeScrollable.appendChild(this._linesDomNode),this._rootDomNode.className="sticky-widget",this._rootDomNode.classList.toggle("peek",e instanceof Jr),this._rootDomNode.appendChild(this._lineNumbersDomNode),this._rootDomNode.appendChild(this._linesDomNodeScrollable);const t=()=>{this._linesDomNode.style.left=this._editor.getOption(116).scrollWithEditor?`-${this._editor.getScrollLeft()}px`:"0px"};this._register(this._editor.onDidChangeConfiguration(i=>{i.hasChanged(116)&&t(),i.hasChanged(67)&&(this._lineHeight=this._editor.getOption(67))})),this._register(this._editor.onDidScrollChange(i=>{i.scrollLeftChanged&&t(),i.scrollWidthChanged&&this._updateWidgetWidth()})),this._register(this._editor.onDidChangeModel(()=>{t(),this._updateWidgetWidth()})),this._register(this._foldingIconStore),t(),this._register(this._editor.onDidLayoutChange(i=>{this._updateWidgetWidth()})),this._updateWidgetWidth()}get lineNumbers(){return this._lineNumbers}get lineNumberCount(){return this._lineNumbers.length}getRenderedStickyLine(e){return this._renderedStickyLines.find(t=>t.lineNumber===e)}getCurrentLines(){return this._lineNumbers}setState(e,t,i){if(i===void 0&&(!this._previousState&&!e||this._previousState&&this._previousState.equals(e)))return;const n=this._isWidgetHeightZero(e),s=n?void 0:e,r=n?0:this._findLineToRebuildWidgetFrom(e,i);this._renderRootNode(s,t,r),this._previousState=e}_isWidgetHeightZero(e){if(!e)return!0;const t=e.startLineNumbers.length*this._lineHeight+e.lastLineRelativePosition;if(t>0){this._lastLineRelativePosition=e.lastLineRelativePosition;const i=[...e.startLineNumbers];e.showEndForLine!==null&&(i[e.showEndForLine]=e.endLineNumbers[e.showEndForLine]),this._lineNumbers=i}else this._lastLineRelativePosition=0,this._lineNumbers=[];return t===0}_findLineToRebuildWidgetFrom(e,t){if(!e||!this._previousState)return 0;if(t!==void 0)return t;const i=this._previousState,n=e.startLineNumbers.findIndex(s=>!i.startLineNumbers.includes(s));return n===-1?0:n}_updateWidgetWidth(){const e=this._editor.getLayoutInfo(),t=e.contentLeft;this._lineNumbersDomNode.style.width=`${t}px`,this._linesDomNodeScrollable.style.setProperty("--vscode-editorStickyScroll-scrollableWidth",`${this._editor.getScrollWidth()-e.verticalScrollbarWidth}px`),this._rootDomNode.style.width=`${e.width-e.verticalScrollbarWidth}px`}_clearStickyLinesFromLine(e){this._foldingIconStore.clear();for(let t=e;t<this._renderedStickyLines.length;t++){const i=this._renderedStickyLines[t];i.lineNumberDomNode.remove(),i.lineDomNode.remove()}this._renderedStickyLines=this._renderedStickyLines.slice(0,e),this._rootDomNode.style.display="none"}_useFoldingOpacityTransition(e){this._lineNumbersDomNode.style.setProperty("--vscode-editorStickyScroll-foldingOpacityTransition",`opacity ${e?.5:0}s`)}_setFoldingIconsVisibility(e){for(const t of this._renderedStickyLines){const i=t.foldingIcon;i&&i.setVisible(e?!0:i.isCollapsed)}}async _renderRootNode(e,t,i){if(this._clearStickyLinesFromLine(i),!e)return;for(const a of this._renderedStickyLines)this._updateTopAndZIndexOfStickyLine(a);const n=this._editor.getLayoutInfo(),s=this._lineNumbers.slice(i);for(const[a,l]of s.entries()){const c=this._renderChildNode(a+i,l,t,n);c&&(this._linesDomNode.appendChild(c.lineDomNode),this._lineNumbersDomNode.appendChild(c.lineNumberDomNode),this._renderedStickyLines.push(c))}t&&(this._setFoldingHoverListeners(),this._useFoldingOpacityTransition(!this._isOnGlyphMargin));const r=this._lineNumbers.length*this._lineHeight+this._lastLineRelativePosition;this._rootDomNode.style.display="block",this._lineNumbersDomNode.style.height=`${r}px`,this._linesDomNodeScrollable.style.height=`${r}px`,this._rootDomNode.style.height=`${r}px`,this._rootDomNode.style.marginLeft="0px",this._minContentWidthInPx=Math.max(...this._renderedStickyLines.map(a=>a.scrollWidth))+n.verticalScrollbarWidth,this._editor.layoutOverlayWidget(this)}_setFoldingHoverListeners(){this._editor.getOption(111)==="mouseover"&&(this._foldingIconStore.add(U(this._lineNumbersDomNode,ie.MOUSE_ENTER,()=>{this._isOnGlyphMargin=!0,this._setFoldingIconsVisibility(!0)})),this._foldingIconStore.add(U(this._lineNumbersDomNode,ie.MOUSE_LEAVE,()=>{this._isOnGlyphMargin=!1,this._useFoldingOpacityTransition(!0),this._setFoldingIconsVisibility(!1)})))}_renderChildNode(e,t,i,n){const s=this._editor._getViewModel();if(!s)return;const r=s.coordinatesConverter.convertModelPositionToViewPosition(new A(t,1)).lineNumber,a=s.getViewLineRenderingData(r),l=this._editor.getOption(68);let c;try{c=hs.filter(a.inlineDecorations,r,a.minColumn,a.maxColumn)}catch{c=[]}const d=new au(!0,!0,a.content,a.continuesWithWrappedLine,a.isBasicASCII,a.containsRTL,0,a.tokens,c,a.tabSize,a.startVisibleColumn,1,1,1,500,"none",!0,!0,null),h=new _b(2e3),u=nS(d,h);let g;K6?g=K6.createHTML(h.build()):g=h.build();const f=document.createElement("span");f.setAttribute(nR,String(e)),f.setAttribute(q6,""),f.setAttribute("role","listitem"),f.tabIndex=0,f.className="sticky-line-content",f.classList.add(`stickyLine${t}`),f.style.lineHeight=`${this._lineHeight}px`,f.innerHTML=g;const p=document.createElement("span");p.setAttribute(nR,String(e)),p.setAttribute(FNe,""),p.className="sticky-line-number",p.style.lineHeight=`${this._lineHeight}px`;const _=n.contentLeft;p.style.width=`${_}px`;const b=document.createElement("span");l.renderType===1||l.renderType===3&&t%10===0?b.innerText=t.toString():l.renderType===2&&(b.innerText=Math.abs(t-this._editor.getPosition().lineNumber).toString()),b.className="sticky-line-number-inner",b.style.lineHeight=`${this._lineHeight}px`,b.style.width=`${n.lineNumbersWidth}px`,b.style.paddingLeft=`${n.lineNumbersLeft}px`,p.appendChild(b);const C=this._renderFoldingIconForLine(i,t);C&&p.appendChild(C.domNode),this._editor.applyFontInfo(f),this._editor.applyFontInfo(b),p.style.lineHeight=`${this._lineHeight}px`,f.style.lineHeight=`${this._lineHeight}px`,p.style.height=`${this._lineHeight}px`,f.style.height=`${this._lineHeight}px`;const v=new BNe(e,t,f,p,C,u.characterMapping,f.scrollWidth);return this._updateTopAndZIndexOfStickyLine(v)}_updateTopAndZIndexOfStickyLine(e){var d;const t=e.index,i=e.lineDomNode,n=e.lineNumberDomNode,s=t===this._lineNumbers.length-1,r="0",a="1";i.style.zIndex=s?r:a,n.style.zIndex=s?r:a;const l=`${t*this._lineHeight+this._lastLineRelativePosition+((d=e.foldingIcon)!=null&&d.isCollapsed?1:0)}px`,c=`${t*this._lineHeight}px`;return i.style.top=s?l:c,n.style.top=s?l:c,e}_renderFoldingIconForLine(e,t){const i=this._editor.getOption(111);if(!e||i==="never")return;const n=e.regions,s=n.findRange(t),r=n.getStartLineNumber(s);if(!(t===r))return;const l=n.isCollapsed(s),c=new HNe(l,r,n.getEndLineNumber(s),this._lineHeight);return c.setVisible(this._isOnGlyphMargin?!0:l||i==="always"),c.domNode.setAttribute(G6,""),c}getId(){return"editor.contrib.stickyScrollWidget"}getDomNode(){return this._rootDomNode}getPosition(){return{preference:2,stackOridinal:10}}getMinContentWidthInPx(){return this._minContentWidthInPx}focusLineWithIndex(e){0<=e&&e<this._renderedStickyLines.length&&this._renderedStickyLines[e].lineDomNode.focus()}getEditorPositionFromNode(e){if(!e||e.children.length>0)return null;const t=this._getRenderedStickyLineFromChildDomNode(e);if(!t)return null;const i=U3(t.characterMapping,e,0);return new A(t.lineNumber,i)}getLineNumberFromChildDomNode(e){var t;return((t=this._getRenderedStickyLineFromChildDomNode(e))==null?void 0:t.lineNumber)??null}_getRenderedStickyLineFromChildDomNode(e){const t=this.getLineIndexFromChildDomNode(e);return t===null||t<0||t>=this._renderedStickyLines.length?null:this._renderedStickyLines[t]}getLineIndexFromChildDomNode(e){const t=this._getAttributeValue(e,nR);return t?parseInt(t,10):null}isInStickyLine(e){return this._getAttributeValue(e,q6)!==void 0}isInFoldingIconDomNode(e){return this._getAttributeValue(e,G6)!==void 0}_getAttributeValue(e,t){for(;e&&e!==this._rootDomNode;){const i=e.getAttribute(t);if(i!==null)return i;e=e.parentElement}}}class BNe{constructor(e,t,i,n,s,r,a){this.index=e,this.lineNumber=t,this.lineDomNode=i,this.lineNumberDomNode=n,this.foldingIcon=s,this.characterMapping=r,this.scrollWidth=a}}class HNe{constructor(e,t,i,n){this.isCollapsed=e,this.foldingStartLine=t,this.foldingEndLine=i,this.dimension=n,this.domNode=document.createElement("div"),this.domNode.style.width=`${n}px`,this.domNode.style.height=`${n}px`,this.domNode.className=Ie.asClassName(e?jx:$x)}setVisible(e){this.domNode.style.cursor=e?"pointer":"default",this.domNode.style.opacity=e?"1":"0"}}class Dv{constructor(e,t){this.startLineNumber=e,this.endLineNumber=t}}class Ck{constructor(e,t,i){this.range=e,this.children=t,this.parent=i}}class zY{constructor(e,t,i,n){this.uri=e,this.version=t,this.element=i,this.outlineProviderId=n}}var jE=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},y1=function(o,e){return function(t,i){e(t,i,o)}},Iv;(function(o){o.OUTLINE_MODEL="outlineModel",o.FOLDING_PROVIDER_MODEL="foldingProviderModel",o.INDENTATION_MODEL="indentationModel"})(Iv||(Iv={}));var oh;(function(o){o[o.VALID=0]="VALID",o[o.INVALID=1]="INVALID",o[o.CANCELED=2]="CANCELED"})(oh||(oh={}));let o4=class extends F{constructor(e,t,i,n){switch(super(),this._editor=e,this._modelProviders=[],this._modelPromise=null,this._updateScheduler=this._register(new _r(300)),this._updateOperation=this._register(new K),this._editor.getOption(116).defaultModel){case Iv.OUTLINE_MODEL:this._modelProviders.push(new r4(this._editor,n));case Iv.FOLDING_PROVIDER_MODEL:this._modelProviders.push(new l4(this._editor,t,n));case Iv.INDENTATION_MODEL:this._modelProviders.push(new a4(this._editor,i));break}}dispose(){this._modelProviders.forEach(e=>e.dispose()),this._updateOperation.clear(),this._cancelModelPromise(),super.dispose()}_cancelModelPromise(){this._modelPromise&&(this._modelPromise.cancel(),this._modelPromise=null)}async update(e){return this._updateOperation.clear(),this._updateOperation.add({dispose:()=>{this._cancelModelPromise(),this._updateScheduler.cancel()}}),this._cancelModelPromise(),await this._updateScheduler.trigger(async()=>{for(const t of this._modelProviders){const{statusPromise:i,modelPromise:n}=t.computeStickyModel(e);this._modelPromise=n;const s=await i;if(this._modelPromise!==n)return null;switch(s){case oh.CANCELED:return this._updateOperation.clear(),null;case oh.VALID:return t.stickyModel}}return null}).catch(t=>(Oe(t),null))}};o4=jE([y1(2,we),y1(3,ge)],o4);class UY extends F{constructor(e){super(),this._editor=e,this._stickyModel=null}get stickyModel(){return this._stickyModel}_invalid(){return this._stickyModel=null,oh.INVALID}computeStickyModel(e){if(e.isCancellationRequested||!this.isProviderValid())return{statusPromise:this._invalid(),modelPromise:null};const t=Yi(i=>this.createModelFromProvider(i));return{statusPromise:t.then(i=>this.isModelValid(i)?e.isCancellationRequested?oh.CANCELED:(this._stickyModel=this.createStickyModel(e,i),oh.VALID):this._invalid()).then(void 0,i=>(Oe(i),oh.CANCELED)),modelPromise:t}}isModelValid(e){return!0}isProviderValid(){return!0}}let r4=class extends UY{constructor(e,t){super(e),this._languageFeaturesService=t}createModelFromProvider(e){return Lc.create(this._languageFeaturesService.documentSymbolProvider,this._editor.getModel(),e)}createStickyModel(e,t){var r;const{stickyOutlineElement:i,providerID:n}=this._stickyModelFromOutlineModel(t,(r=this._stickyModel)==null?void 0:r.outlineProviderId),s=this._editor.getModel();return new zY(s.uri,s.getVersionId(),i,n)}isModelValid(e){return e&&e.children.size>0}_stickyModelFromOutlineModel(e,t){let i;if(ot.first(e.children.values())instanceof dY){const a=ot.find(e.children.values(),l=>l.id===t);if(a)i=a.children;else{let l="",c=-1,d;for(const[h,u]of e.children.entries()){const g=this._findSumOfRangesOfGroup(u);g>c&&(d=u,c=g,l=u.id)}t=l,i=d.children}}else i=e.children;const n=[],s=Array.from(i.values()).sort((a,l)=>{const c=new Dv(a.symbol.range.startLineNumber,a.symbol.range.endLineNumber),d=new Dv(l.symbol.range.startLineNumber,l.symbol.range.endLineNumber);return this._comparator(c,d)});for(const a of s)n.push(this._stickyModelFromOutlineElement(a,a.symbol.selectionRange.startLineNumber));return{stickyOutlineElement:new Ck(void 0,n,void 0),providerID:t}}_stickyModelFromOutlineElement(e,t){const i=[];for(const s of e.children.values())if(s.symbol.selectionRange.startLineNumber!==s.symbol.range.endLineNumber)if(s.symbol.selectionRange.startLineNumber!==t)i.push(this._stickyModelFromOutlineElement(s,s.symbol.selectionRange.startLineNumber));else for(const r of s.children.values())i.push(this._stickyModelFromOutlineElement(r,s.symbol.selectionRange.startLineNumber));i.sort((s,r)=>this._comparator(s.range,r.range));const n=new Dv(e.symbol.selectionRange.startLineNumber,e.symbol.range.endLineNumber);return new Ck(n,i,void 0)}_comparator(e,t){return e.startLineNumber!==t.startLineNumber?e.startLineNumber-t.startLineNumber:t.endLineNumber-e.endLineNumber}_findSumOfRangesOfGroup(e){let t=0;for(const i of e.children.values())t+=this._findSumOfRangesOfGroup(i);return e instanceof ZO?t+e.symbol.range.endLineNumber-e.symbol.selectionRange.startLineNumber:t}};r4=jE([y1(1,ge)],r4);class $Y extends UY{constructor(e){super(e),this._foldingLimitReporter=new aY(e)}createStickyModel(e,t){const i=this._fromFoldingRegions(t),n=this._editor.getModel();return new zY(n.uri,n.getVersionId(),i,void 0)}isModelValid(e){return e!==null}_fromFoldingRegions(e){const t=e.length,i=[],n=new Ck(void 0,[],void 0);for(let s=0;s<t;s++){const r=e.getParentIndex(s);let a;r!==-1?a=i[r]:a=n;const l=new Ck(new Dv(e.getStartLineNumber(s),e.getEndLineNumber(s)+1),[],a);a.children.push(l),i.push(l)}return n}}let a4=class extends $Y{constructor(e,t){super(e),this._languageConfigurationService=t,this.provider=this._register(new eB(e.getModel(),this._languageConfigurationService,this._foldingLimitReporter))}async createModelFromProvider(e){return this.provider.compute(e)}};a4=jE([y1(1,di)],a4);let l4=class extends $Y{constructor(e,t,i){super(e),this._languageFeaturesService=i;const n=Qh.getFoldingRangeProviders(this._languageFeaturesService,e.getModel());n.length>0&&(this.provider=this._register(new tB(e.getModel(),n,t,this._foldingLimitReporter,void 0)))}isProviderValid(){return this.provider!==void 0}async createModelFromProvider(e){var t;return((t=this.provider)==null?void 0:t.compute(e))??null}};l4=jE([y1(2,ge)],l4);var VNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Z6=function(o,e){return function(t,i){e(t,i,o)}};class zNe{constructor(e,t,i){this.startLineNumber=e,this.endLineNumber=t,this.nestingDepth=i}}let c4=class extends F{constructor(e,t,i){super(),this._languageFeaturesService=t,this._languageConfigurationService=i,this._onDidChangeStickyScroll=this._register(new P),this.onDidChangeStickyScroll=this._onDidChangeStickyScroll.event,this._model=null,this._cts=null,this._stickyModelProvider=null,this._editor=e,this._sessionStore=this._register(new K),this._updateSoon=this._register(new It(()=>this.update(),50)),this._register(this._editor.onDidChangeConfiguration(n=>{n.hasChanged(116)&&this.readConfiguration()})),this.readConfiguration()}readConfiguration(){this._sessionStore.clear(),this._editor.getOption(116).enabled&&(this._sessionStore.add(this._editor.onDidChangeModel(()=>{this._model=null,this.updateStickyModelProvider(),this._onDidChangeStickyScroll.fire(),this.update()})),this._sessionStore.add(this._editor.onDidChangeHiddenAreas(()=>this.update())),this._sessionStore.add(this._editor.onDidChangeModelContent(()=>this._updateSoon.schedule())),this._sessionStore.add(this._languageFeaturesService.documentSymbolProvider.onDidChange(()=>this.update())),this._sessionStore.add(ve(()=>{var t;(t=this._stickyModelProvider)==null||t.dispose(),this._stickyModelProvider=null})),this.updateStickyModelProvider(),this.update())}getVersionId(){var e;return(e=this._model)==null?void 0:e.version}updateStickyModelProvider(){var t;(t=this._stickyModelProvider)==null||t.dispose(),this._stickyModelProvider=null;const e=this._editor;e.hasModel()&&(this._stickyModelProvider=new o4(e,()=>this._updateSoon.schedule(),this._languageConfigurationService,this._languageFeaturesService))}async update(){var e;(e=this._cts)==null||e.dispose(!0),this._cts=new li,await this.updateStickyModel(this._cts.token),this._onDidChangeStickyScroll.fire()}async updateStickyModel(e){if(!this._editor.hasModel()||!this._stickyModelProvider||this._editor.getModel().isTooLargeForTokenization()){this._model=null;return}const t=await this._stickyModelProvider.update(e);e.isCancellationRequested||(this._model=t)}updateIndex(e){return e===-1?e=0:e<0&&(e=-e-2),e}getCandidateStickyLinesIntersectingFromStickyModel(e,t,i,n,s){if(t.children.length===0)return;let r=s;const a=[];for(let d=0;d<t.children.length;d++){const h=t.children[d];h.range&&a.push(h.range.startLineNumber)}const l=this.updateIndex(Zv(a,e.startLineNumber,(d,h)=>d-h)),c=this.updateIndex(Zv(a,e.startLineNumber+n,(d,h)=>d-h));for(let d=l;d<=c;d++){const h=t.children[d];if(!h)return;if(h.range){const u=h.range.startLineNumber,g=h.range.endLineNumber;e.startLineNumber<=g+1&&u-1<=e.endLineNumber&&u!==r&&(r=u,i.push(new zNe(u,g-1,n+1)),this.getCandidateStickyLinesIntersectingFromStickyModel(e,h,i,n+1,u))}else this.getCandidateStickyLinesIntersectingFromStickyModel(e,h,i,n,s)}}getCandidateStickyLinesIntersecting(e){var n,s;if(!((n=this._model)!=null&&n.element))return[];let t=[];this.getCandidateStickyLinesIntersectingFromStickyModel(e,this._model.element,t,0,-1);const i=(s=this._editor._getViewModel())==null?void 0:s.getHiddenAreas();if(i)for(const r of i)t=t.filter(a=>!(a.startLineNumber>=r.startLineNumber&&a.endLineNumber<=r.endLineNumber+1));return t}};c4=VNe([Z6(1,ge),Z6(2,di)],c4);var UNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Vm=function(o,e){return function(t,i){e(t,i,o)}},d4,Cf;let cd=(Cf=class extends F{constructor(e,t,i,n,s,r,a){super(),this._editor=e,this._contextMenuService=t,this._languageFeaturesService=i,this._instaService=n,this._contextKeyService=a,this._sessionStore=new K,this._maxStickyLines=Number.MAX_SAFE_INTEGER,this._candidateDefinitionsLength=-1,this._focusedStickyElementIndex=-1,this._enabled=!1,this._focused=!1,this._positionRevealed=!1,this._onMouseDown=!1,this._endLineNumbers=[],this._stickyScrollWidget=new WNe(this._editor),this._stickyLineCandidateProvider=new c4(this._editor,i,s),this._register(this._stickyScrollWidget),this._register(this._stickyLineCandidateProvider),this._widgetState=kv.Empty,this._onDidResize(),this._readConfiguration();const l=this._stickyScrollWidget.getDomNode();this._register(this._editor.onDidChangeConfiguration(d=>{this._readConfigurationChange(d)})),this._register(U(l,ie.CONTEXT_MENU,async d=>{this._onContextMenu(xe(l),d)})),this._stickyScrollFocusedContextKey=I.stickyScrollFocused.bindTo(this._contextKeyService),this._stickyScrollVisibleContextKey=I.stickyScrollVisible.bindTo(this._contextKeyService);const c=this._register(Fa(l));this._register(c.onDidBlur(d=>{this._positionRevealed===!1&&l.clientHeight===0?(this._focusedStickyElementIndex=-1,this.focus()):this._disposeFocusStickyScrollStore()})),this._register(c.onDidFocus(d=>{this.focus()})),this._registerMouseListeners(),this._register(U(l,ie.MOUSE_DOWN,d=>{this._onMouseDown=!0}))}static get(e){return e.getContribution(d4.ID)}_disposeFocusStickyScrollStore(){var e;this._stickyScrollFocusedContextKey.set(!1),(e=this._focusDisposableStore)==null||e.dispose(),this._focused=!1,this._positionRevealed=!1,this._onMouseDown=!1}focus(){if(this._onMouseDown){this._onMouseDown=!1,this._editor.focus();return}this._stickyScrollFocusedContextKey.get()!==!0&&(this._focused=!0,this._focusDisposableStore=new K,this._stickyScrollFocusedContextKey.set(!0),this._focusedStickyElementIndex=this._stickyScrollWidget.lineNumbers.length-1,this._stickyScrollWidget.focusLineWithIndex(this._focusedStickyElementIndex))}focusNext(){this._focusedStickyElementIndex<this._stickyScrollWidget.lineNumberCount-1&&this._focusNav(!0)}focusPrevious(){this._focusedStickyElementIndex>0&&this._focusNav(!1)}selectEditor(){this._editor.focus()}_focusNav(e){this._focusedStickyElementIndex=e?this._focusedStickyElementIndex+1:this._focusedStickyElementIndex-1,this._stickyScrollWidget.focusLineWithIndex(this._focusedStickyElementIndex)}goToFocused(){const e=this._stickyScrollWidget.lineNumbers;this._disposeFocusStickyScrollStore(),this._revealPosition({lineNumber:e[this._focusedStickyElementIndex],column:1})}_revealPosition(e){this._reveaInEditor(e,()=>this._editor.revealPosition(e))}_revealLineInCenterIfOutsideViewport(e){this._reveaInEditor(e,()=>this._editor.revealLineInCenterIfOutsideViewport(e.lineNumber,0))}_reveaInEditor(e,t){this._focused&&this._disposeFocusStickyScrollStore(),this._positionRevealed=!0,t(),this._editor.setSelection(x.fromPositions(e)),this._editor.focus()}_registerMouseListeners(){const e=this._register(new K),t=this._register(new AE(this._editor,{extractLineNumberFromMouseEvent:s=>{const r=this._stickyScrollWidget.getEditorPositionFromNode(s.target.element);return r?r.lineNumber:0}})),i=s=>{if(!this._editor.hasModel()||s.target.type!==12||s.target.detail!==this._stickyScrollWidget.getId())return null;const r=s.target.element;if(!r||r.innerText!==r.innerHTML)return null;const a=this._stickyScrollWidget.getEditorPositionFromNode(r);return a?{range:new x(a.lineNumber,a.column,a.lineNumber,a.column+r.innerText.length),textElement:r}:null},n=this._stickyScrollWidget.getDomNode();this._register(hi(n,ie.CLICK,s=>{if(s.ctrlKey||s.altKey||s.metaKey||!s.leftButton)return;if(s.shiftKey){const c=this._stickyScrollWidget.getLineIndexFromChildDomNode(s.target);if(c===null)return;const d=new A(this._endLineNumbers[c],1);this._revealLineInCenterIfOutsideViewport(d);return}if(this._stickyScrollWidget.isInFoldingIconDomNode(s.target)){const c=this._stickyScrollWidget.getLineNumberFromChildDomNode(s.target);this._toggleFoldingRegionForLine(c);return}if(!this._stickyScrollWidget.isInStickyLine(s.target))return;let l=this._stickyScrollWidget.getEditorPositionFromNode(s.target);if(!l){const c=this._stickyScrollWidget.getLineNumberFromChildDomNode(s.target);if(c===null)return;l=new A(c,1)}this._revealPosition(l)})),this._register(hi(n,ie.MOUSE_MOVE,s=>{if(s.shiftKey){const r=this._stickyScrollWidget.getLineIndexFromChildDomNode(s.target);if(r===null||this._showEndForLine!==null&&this._showEndForLine===r)return;this._showEndForLine=r,this._renderStickyScroll();return}this._showEndForLine!==void 0&&(this._showEndForLine=void 0,this._renderStickyScroll())})),this._register(U(n,ie.MOUSE_LEAVE,s=>{this._showEndForLine!==void 0&&(this._showEndForLine=void 0,this._renderStickyScroll())})),this._register(t.onMouseMoveOrRelevantKeyDown(([s,r])=>{const a=i(s);if(!a||!s.hasTriggerModifier||!this._editor.hasModel()){e.clear();return}const{range:l,textElement:c}=a;if(!l.equalsRange(this._stickyRangeProjectedOnEditor))this._stickyRangeProjectedOnEditor=l,e.clear();else if(c.style.textDecoration==="underline")return;const d=new li;e.add(ve(()=>d.dispose(!0)));let h;gS(this._languageFeaturesService.definitionProvider,this._editor.getModel(),new A(l.startLineNumber,l.startColumn+1),!1,d.token).then(u=>{if(!d.token.isCancellationRequested)if(u.length!==0){this._candidateDefinitionsLength=u.length;const g=c;h!==g?(e.clear(),h=g,h.style.textDecoration="underline",e.add(ve(()=>{h.style.textDecoration="none"}))):h||(h=g,h.style.textDecoration="underline",e.add(ve(()=>{h.style.textDecoration="none"})))}else e.clear()})})),this._register(t.onCancel(()=>{e.clear()})),this._register(t.onExecute(async s=>{if(s.target.type!==12||s.target.detail!==this._stickyScrollWidget.getId())return;const r=this._stickyScrollWidget.getEditorPositionFromNode(s.target.element);r&&(!this._editor.hasModel()||!this._stickyRangeProjectedOnEditor||(this._candidateDefinitionsLength>1&&(this._focused&&this._disposeFocusStickyScrollStore(),this._revealPosition({lineNumber:r.lineNumber,column:1})),this._instaService.invokeFunction($Z,s,this._editor,{uri:this._editor.getModel().uri,range:this._stickyRangeProjectedOnEditor})))}))}_onContextMenu(e,t){const i=new Vr(e,t);this._contextMenuService.showContextMenu({menuId:me.StickyScrollContext,getAnchor:()=>i})}_toggleFoldingRegionForLine(e){if(!this._foldingModel||e===null)return;const t=this._stickyScrollWidget.getRenderedStickyLine(e),i=t==null?void 0:t.foldingIcon;if(!i)return;XW(this._foldingModel,Number.MAX_VALUE,[e]),i.isCollapsed=!i.isCollapsed;const n=(i.isCollapsed?this._editor.getTopForLineNumber(i.foldingEndLine):this._editor.getTopForLineNumber(i.foldingStartLine))-this._editor.getOption(67)*t.index+1;this._editor.setScrollTop(n),this._renderStickyScroll(e)}_readConfiguration(){const e=this._editor.getOption(116);if(e.enabled===!1){this._editor.removeOverlayWidget(this._stickyScrollWidget),this._sessionStore.clear(),this._enabled=!1;return}else e.enabled&&!this._enabled&&(this._editor.addOverlayWidget(this._stickyScrollWidget),this._sessionStore.add(this._editor.onDidScrollChange(i=>{i.scrollTopChanged&&(this._showEndForLine=void 0,this._renderStickyScroll())})),this._sessionStore.add(this._editor.onDidLayoutChange(()=>this._onDidResize())),this._sessionStore.add(this._editor.onDidChangeModelTokens(i=>this._onTokensChange(i))),this._sessionStore.add(this._stickyLineCandidateProvider.onDidChangeStickyScroll(()=>{this._showEndForLine=void 0,this._renderStickyScroll()})),this._enabled=!0);this._editor.getOption(68).renderType===2&&this._sessionStore.add(this._editor.onDidChangeCursorPosition(()=>{this._showEndForLine=void 0,this._renderStickyScroll(0)}))}_readConfigurationChange(e){(e.hasChanged(116)||e.hasChanged(73)||e.hasChanged(67)||e.hasChanged(111)||e.hasChanged(68))&&this._readConfiguration(),e.hasChanged(68)&&this._renderStickyScroll(0)}_needsUpdate(e){const t=this._stickyScrollWidget.getCurrentLines();for(const i of t)for(const n of e.ranges)if(i>=n.fromLineNumber&&i<=n.toLineNumber)return!0;return!1}_onTokensChange(e){this._needsUpdate(e)&&this._renderStickyScroll(0)}_onDidResize(){const t=this._editor.getLayoutInfo().height/this._editor.getOption(67);this._maxStickyLines=Math.round(t*.25)}async _renderStickyScroll(e){const t=this._editor.getModel();if(!t||t.isTooLargeForTokenization()){this._resetState();return}const i=this._updateAndGetMinRebuildFromLine(e),n=this._stickyLineCandidateProvider.getVersionId();if(n===void 0||n===t.getVersionId())if(!this._focused)await this._updateState(i);else if(this._focusedStickyElementIndex===-1)await this._updateState(i),this._focusedStickyElementIndex=this._stickyScrollWidget.lineNumberCount-1,this._focusedStickyElementIndex!==-1&&this._stickyScrollWidget.focusLineWithIndex(this._focusedStickyElementIndex);else{const r=this._stickyScrollWidget.lineNumbers[this._focusedStickyElementIndex];await this._updateState(i),this._stickyScrollWidget.lineNumberCount===0?this._focusedStickyElementIndex=-1:(this._stickyScrollWidget.lineNumbers.includes(r)||(this._focusedStickyElementIndex=this._stickyScrollWidget.lineNumberCount-1),this._stickyScrollWidget.focusLineWithIndex(this._focusedStickyElementIndex))}}_updateAndGetMinRebuildFromLine(e){if(e!==void 0){const t=this._minRebuildFromLine!==void 0?this._minRebuildFromLine:1/0;this._minRebuildFromLine=Math.min(e,t)}return this._minRebuildFromLine}async _updateState(e){var i;this._minRebuildFromLine=void 0,this._foldingModel=await((i=Qh.get(this._editor))==null?void 0:i.getFoldingModel())??void 0,this._widgetState=this.findScrollWidgetState();const t=this._widgetState.startLineNumbers.length>0;this._stickyScrollVisibleContextKey.set(t),this._stickyScrollWidget.setState(this._widgetState,this._foldingModel,e)}async _resetState(){this._minRebuildFromLine=void 0,this._foldingModel=void 0,this._widgetState=kv.Empty,this._stickyScrollVisibleContextKey.set(!1),this._stickyScrollWidget.setState(void 0,void 0)}findScrollWidgetState(){const e=this._editor.getOption(67),t=Math.min(this._maxStickyLines,this._editor.getOption(116).maxLineCount),i=this._editor.getScrollTop();let n=0;const s=[],r=[],a=this._editor.getVisibleRanges();if(a.length!==0){const l=new Dv(a[0].startLineNumber,a[a.length-1].endLineNumber),c=this._stickyLineCandidateProvider.getCandidateStickyLinesIntersecting(l);for(const d of c){const h=d.startLineNumber,u=d.endLineNumber,g=d.nestingDepth;if(u-h>0){const f=(g-1)*e,p=g*e,_=this._editor.getBottomForLineNumber(h)-i,b=this._editor.getTopForLineNumber(u)-i,C=this._editor.getBottomForLineNumber(u)-i;if(f>b&&f<=C){s.push(h),r.push(u+1),n=C-p;break}else p>_&&p<=C&&(s.push(h),r.push(u+1));if(s.length===t)break}}}return this._endLineNumbers=r,new kv(s,r,n,this._showEndForLine)}dispose(){super.dispose(),this._sessionStore.dispose()}},d4=Cf,Cf.ID="store.contrib.stickyScrollController",Cf);cd=d4=UNe([Vm(1,mo),Vm(2,ge),Vm(3,we),Vm(4,di),Vm(5,Zo),Vm(6,Ee)],cd);class $Ne extends ws{constructor(){super({id:"editor.action.toggleStickyScroll",title:{...Te("toggleEditorStickyScroll","Toggle Editor Sticky Scroll"),mnemonicTitle:m({key:"mitoggleStickyScroll",comment:["&& denotes a mnemonic"]},"&&Toggle Editor Sticky Scroll")},metadata:{description:Te("toggleEditorStickyScroll.description","Toggle/enable the editor sticky scroll which shows the nested scopes at the top of the viewport")},category:ONe.View,toggled:{condition:G.equals("config.editor.stickyScroll.enabled",!0),title:m("stickyScroll","Sticky Scroll"),mnemonicTitle:m({key:"miStickyScroll",comment:["&& denotes a mnemonic"]},"&&Sticky Scroll")},menu:[{id:me.CommandPalette},{id:me.MenubarAppearanceMenu,group:"4_editor",order:3},{id:me.StickyScrollContext}]})}async run(e){const t=e.get(qe),i=!t.getValue("editor.stickyScroll.enabled");return t.updateValue("editor.stickyScroll.enabled",i)}}const KE=100;class jNe extends $a{constructor(){super({id:"editor.action.focusStickyScroll",title:{...Te("focusStickyScroll","Focus on the editor sticky scroll"),mnemonicTitle:m({key:"mifocusStickyScroll",comment:["&& denotes a mnemonic"]},"&&Focus Sticky Scroll")},precondition:G.and(G.has("config.editor.stickyScroll.enabled"),I.stickyScrollVisible),menu:[{id:me.CommandPalette}]})}runEditorCommand(e,t){var i;(i=cd.get(t))==null||i.focus()}}class KNe extends $a{constructor(){super({id:"editor.action.selectNextStickyScrollLine",title:Te("selectNextStickyScrollLine.title","Select the next editor sticky scroll line"),precondition:I.stickyScrollFocused.isEqualTo(!0),keybinding:{weight:KE,primary:18}})}runEditorCommand(e,t){var i;(i=cd.get(t))==null||i.focusNext()}}class qNe extends $a{constructor(){super({id:"editor.action.selectPreviousStickyScrollLine",title:Te("selectPreviousStickyScrollLine.title","Select the previous sticky scroll line"),precondition:I.stickyScrollFocused.isEqualTo(!0),keybinding:{weight:KE,primary:16}})}runEditorCommand(e,t){var i;(i=cd.get(t))==null||i.focusPrevious()}}class GNe extends $a{constructor(){super({id:"editor.action.goToFocusedStickyScrollLine",title:Te("goToFocusedStickyScrollLine.title","Go to the focused sticky scroll line"),precondition:I.stickyScrollFocused.isEqualTo(!0),keybinding:{weight:KE,primary:3}})}runEditorCommand(e,t){var i;(i=cd.get(t))==null||i.goToFocused()}}class ZNe extends $a{constructor(){super({id:"editor.action.selectEditor",title:Te("selectEditor.title","Select Editor"),precondition:I.stickyScrollFocused.isEqualTo(!0),keybinding:{weight:KE,primary:9}})}runEditorCommand(e,t){var i;(i=cd.get(t))==null||i.selectEditor()}}ct(cd.ID,cd,1);Ot($Ne);Ot(jNe);Ot(qNe);Ot(KNe);Ot(GNe);Ot(ZNe);var jY=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},VC=function(o,e){return function(t,i){e(t,i,o)}};class YNe{constructor(e,t,i,n,s,r){this.range=e,this.insertText=t,this.filterText=i,this.additionalTextEdits=n,this.command=s,this.completion=r}}let h4=class extends WJ{constructor(e,t,i,n,s,r){super(s.disposable),this.model=e,this.line=t,this.word=i,this.completionModel=n,this._suggestMemoryService=r}canBeReused(e,t,i){return this.model===e&&this.line===t&&this.word.word.length>0&&this.word.startColumn===i.startColumn&&this.word.endColumn<i.endColumn&&this.completionModel.getIncompleteProvider().size===0}get items(){const e=[],{items:t}=this.completionModel,i=this._suggestMemoryService.select(this.model,{lineNumber:this.line,column:this.word.endColumn+this.completionModel.lineContext.characterCountDelta},t),n=ot.slice(t,i),s=ot.slice(t,0,i);let r=5;for(const a of ot.concat(n,s)){if(a.score===Ma.Default)continue;const l=new x(a.editStart.lineNumber,a.editStart.column,a.editInsertEnd.lineNumber,a.editInsertEnd.column+this.completionModel.lineContext.characterCountDelta),c=a.completion.insertTextRules&&a.completion.insertTextRules&4?{snippet:a.completion.insertText}:a.completion.insertText;e.push(new YNe(l,c,a.filterTextLow??a.labelLow,a.completion.additionalTextEdits,a.completion.command,a)),r-->=0&&a.resolve(ze.None)}return e}};h4=jY([VC(5,VE)],h4);let u4=class extends F{constructor(e,t,i,n){super(),this._languageFeatureService=e,this._clipboardService=t,this._suggestMemoryService=i,this._editorService=n,this._store.add(e.inlineCompletionsProvider.register("*",this))}async provideInlineCompletions(e,t,i,n){var g;if(i.selectedSuggestionInfo)return;let s;for(const f of this._editorService.listCodeEditors())if(f.getModel()===e){s=f;break}if(!s)return;const r=s.getOption(90);if(vp.isAllOff(r))return;e.tokenization.tokenizeIfCheap(t.lineNumber);const a=e.tokenization.getLineTokens(t.lineNumber),l=a.getStandardTokenType(a.findTokenIndexAtOffset(Math.max(t.column-1-1,0)));if(vp.valueFor(r,l)!=="inline")return;let c=e.getWordAtPosition(t),d;if(c!=null&&c.word||(d=this._getTriggerCharacterInfo(e,t)),!(c!=null&&c.word)&&!d||(c||(c=e.getWordUntilPosition(t)),c.endColumn!==t.column))return;let h;const u=e.getValueInRange(new x(t.lineNumber,1,t.lineNumber,t.column));if(!d&&((g=this._lastResult)!=null&&g.canBeReused(e,t.lineNumber,c))){const f=new A6(u,t.column-this._lastResult.word.endColumn);this._lastResult.completionModel.lineContext=f,this._lastResult.acquire(),h=this._lastResult}else{const f=await iB(this._languageFeatureService.completionProvider,e,t,new f1(void 0,sk.createSuggestFilter(s).itemKind,d==null?void 0:d.providers),d&&{triggerKind:1,triggerCharacter:d.ch},n);let p;f.needsClipboard&&(p=await this._clipboardService.readText());const _=new $d(f.items,t.column,new A6(u,0),nk.None,s.getOption(119),s.getOption(113),{boostFullMatch:!1,firstMatchCanBeWeak:!1},p);h=new h4(e,t.lineNumber,c,_,f,this._suggestMemoryService)}return this._lastResult=h,h}handleItemDidShow(e,t){t.completion.resolve(ze.None)}freeInlineCompletions(e){e.release()}_getTriggerCharacterInfo(e,t){var s;const i=e.getValueInRange(x.fromPositions({lineNumber:t.lineNumber,column:t.column-1},t)),n=new Set;for(const r of this._languageFeatureService.completionProvider.all(e))(s=r.triggerCharacters)!=null&&s.includes(i)&&n.add(r);if(n.size!==0)return{providers:n,ch:i}}};u4=jY([VC(0,ge),VC(1,md),VC(2,VE),VC(3,pt)],u4);Nb(u4);class XNe extends de{constructor(){super({id:"editor.action.forceRetokenize",label:m("forceRetokenize","Developer: Force Retokenize"),alias:"Developer: Force Retokenize",precondition:void 0})}run(e,t){if(!t.hasModel())return;const i=t.getModel();i.tokenization.resetTokenization();const n=new ln;i.tokenization.forceTokenization(i.getLineCount()),n.stop(),console.log(`tokenization took ${n.elapsed()}`)}}Q(XNe);const XD=class XD extends ws{constructor(){super({id:XD.ID,title:Te({key:"toggle.tabMovesFocus",comment:["Turn on/off use of tab key for moving focus around VS Code"]},"Toggle Tab Key Moves Focus"),precondition:void 0,keybinding:{primary:2091,mac:{primary:1323},weight:100},metadata:{description:Te("tabMovesFocusDescriptions","Determines whether the tab key moves focus around the workbench or inserts the tab character in the current editor. This is also called tab trapping, tab navigation, or tab focus mode.")},f1:!0})}run(){const t=!K_.getTabFocusMode();K_.setTabFocusMode(t),Cs(t?m("toggle.tabMovesFocus.on","Pressing Tab will now move focus to the next focusable element"):m("toggle.tabMovesFocus.off","Pressing Tab will now insert the tab character"))}};XD.ID="editor.action.toggleTabFocusMode";let g4=XD;Ot(g4);var QNe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Y6=function(o,e){return function(t,i){e(t,i,o)}};let f4=class extends F{get enabled(){return this._enabled}set enabled(e){e?(this.el.setAttribute("aria-disabled","false"),this.el.tabIndex=0,this.el.style.pointerEvents="auto",this.el.style.opacity="1",this.el.style.cursor="pointer",this._enabled=!1):(this.el.setAttribute("aria-disabled","true"),this.el.tabIndex=-1,this.el.style.pointerEvents="none",this.el.style.opacity="0.4",this.el.style.cursor="default",this._enabled=!0),this._enabled=e}constructor(e,t,i={},n,s){super(),this._link=t,this._hoverService=n,this._enabled=!0,this.el=X(e,ne("a.monaco-link",{tabIndex:t.tabIndex??0,href:t.href},t.label)),this.hoverDelegate=i.hoverDelegate??Ss("mouse"),this.setTooltip(t.title),this.el.setAttribute("role","button");const r=this._register(new tt(this.el,"click")),a=this._register(new tt(this.el,"keypress")),l=oe.chain(a.event,h=>h.map(u=>new Pt(u)).filter(u=>u.keyCode===3)),c=this._register(new tt(this.el,Bt.Tap)).event;this._register(Fn.addTarget(this.el));const d=oe.any(r.event,l,c);this._register(d(h=>{this.enabled&&(nt.stop(h,!0),i!=null&&i.opener?i.opener(this._link.href):s.open(this._link.href,{allowCommands:!0}))})),this.enabled=!0}setTooltip(e){this.hoverDelegate.showNativeHover?this.el.title=e??"":!this.hover&&e?this.hover=this._register(this._hoverService.setupManagedHover(this.hoverDelegate,this.el,e)):this.hover&&this.hover.update(e)}};f4=QNe([Y6(3,Yl),Y6(4,zs)],f4);var KY=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},qY=function(o,e){return function(t,i){e(t,i,o)}};const JNe=26;let m4=class extends F{constructor(e,t){super(),this._editor=e,this.instantiationService=t,this.banner=this._register(this.instantiationService.createInstance(p4))}hide(){this._editor.setBanner(null,0),this.banner.clear()}show(e){this.banner.show({...e,onClose:()=>{var t;this.hide(),(t=e.onClose)==null||t.call(e)}}),this._editor.setBanner(this.banner.element,JNe)}};m4=KY([qY(1,we)],m4);let p4=class extends F{constructor(e){super(),this.instantiationService=e,this.markdownRenderer=this.instantiationService.createInstance(nd,{}),this.element=ne("div.editor-banner"),this.element.tabIndex=0}getAriaLabel(e){if(e.ariaLabel)return e.ariaLabel;if(typeof e.message=="string")return e.message}getBannerMessage(e){if(typeof e=="string"){const t=ne("span");return t.innerText=e,t}return this.markdownRenderer.render(e).element}clear(){pn(this.element)}show(e){pn(this.element);const t=this.getAriaLabel(e);t&&this.element.setAttribute("aria-label",t);const i=X(this.element,ne("div.icon-container"));i.setAttribute("aria-hidden","true"),e.icon&&i.appendChild(ne(`div${Ie.asCSSSelector(e.icon)}`));const n=X(this.element,ne("div.message-container"));if(n.setAttribute("aria-hidden","true"),n.appendChild(this.getBannerMessage(e.message)),this.messageActionsContainer=X(this.element,ne("div.message-actions-container")),e.actions)for(const r of e.actions)this._register(this.instantiationService.createInstance(f4,this.messageActionsContainer,{...r,tabIndex:-1},{}));const s=X(this.element,ne("div.action-container"));this.actionBar=this._register(new zo(s)),this.actionBar.push(this._register(new ro("banner.close","Close Banner",Ie.asClassName(tq),!0,()=>{typeof e.onClose=="function"&&e.onClose()})),{icon:!0,label:!1}),this.actionBar.setFocusable(!1)}};p4=KY([qY(0,we)],p4);var pB=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},h_=function(o,e){return function(t,i){e(t,i,o)}};const eTe=qt("extensions-warning-message",te.warning,m("warningIcon","Icon shown with a warning message in the extensions editor."));var S_;let L1=(S_=class extends F{constructor(e,t,i,n){super(),this._editor=e,this._editorWorkerService=t,this._workspaceTrustService=i,this._highlighter=null,this._bannerClosed=!1,this._updateState=s=>{if(s&&s.hasMore){if(this._bannerClosed)return;const r=Math.max(s.ambiguousCharacterCount,s.nonBasicAsciiCharacterCount,s.invisibleCharacterCount);let a;if(s.nonBasicAsciiCharacterCount>=r)a={message:m("unicodeHighlighting.thisDocumentHasManyNonBasicAsciiUnicodeCharacters","This document contains many non-basic ASCII unicode characters"),command:new k1};else if(s.ambiguousCharacterCount>=r)a={message:m("unicodeHighlighting.thisDocumentHasManyAmbiguousUnicodeCharacters","This document contains many ambiguous unicode characters"),command:new Xf};else if(s.invisibleCharacterCount>=r)a={message:m("unicodeHighlighting.thisDocumentHasManyInvisibleUnicodeCharacters","This document contains many invisible unicode characters"),command:new x1};else throw new Error("Unreachable");this._bannerController.show({id:"unicodeHighlightBanner",message:a.message,icon:eTe,actions:[{label:a.command.shortLabel,href:`command:${a.command.id}`}],onClose:()=>{this._bannerClosed=!0}})}else this._bannerController.hide()},this._bannerController=this._register(n.createInstance(m4,e)),this._register(this._editor.onDidChangeModel(()=>{this._bannerClosed=!1,this._updateHighlighter()})),this._options=e.getOption(126),this._register(i.onDidChangeTrust(s=>{this._updateHighlighter()})),this._register(e.onDidChangeConfiguration(s=>{s.hasChanged(126)&&(this._options=e.getOption(126),this._updateHighlighter())})),this._updateHighlighter()}dispose(){this._highlighter&&(this._highlighter.dispose(),this._highlighter=null),super.dispose()}_updateHighlighter(){if(this._updateState(null),this._highlighter&&(this._highlighter.dispose(),this._highlighter=null),!this._editor.hasModel())return;const e=tTe(this._workspaceTrustService.isWorkspaceTrusted(),this._options);if([e.nonBasicASCII,e.ambiguousCharacters,e.invisibleCharacters].every(i=>i===!1))return;const t={nonBasicASCII:e.nonBasicASCII,ambiguousCharacters:e.ambiguousCharacters,invisibleCharacters:e.invisibleCharacters,includeComments:e.includeComments,includeStrings:e.includeStrings,allowedCodePoints:Object.keys(e.allowedCharacters).map(i=>i.codePointAt(0)),allowedLocales:Object.keys(e.allowedLocales).map(i=>i==="_os"?new Intl.NumberFormat().resolvedOptions().locale:i==="_vscode"?OQ:i)};this._editorWorkerService.canComputeUnicodeHighlights(this._editor.getModel().uri)?this._highlighter=new _4(this._editor,t,this._updateState,this._editorWorkerService):this._highlighter=new iTe(this._editor,t,this._updateState)}getDecorationInfo(e){return this._highlighter?this._highlighter.getDecorationInfo(e):null}},S_.ID="editor.contrib.unicodeHighlighter",S_);L1=pB([h_(1,Us),h_(2,_j),h_(3,we)],L1);function tTe(o,e){return{nonBasicASCII:e.nonBasicASCII===xo?!o:e.nonBasicASCII,ambiguousCharacters:e.ambiguousCharacters,invisibleCharacters:e.invisibleCharacters,includeComments:e.includeComments===xo?!o:e.includeComments,includeStrings:e.includeStrings===xo?!o:e.includeStrings,allowedCharacters:e.allowedCharacters,allowedLocales:e.allowedLocales}}let _4=class extends F{constructor(e,t,i,n){super(),this._editor=e,this._options=t,this._updateState=i,this._editorWorkerService=n,this._model=this._editor.getModel(),this._decorations=this._editor.createDecorationsCollection(),this._updateSoon=this._register(new It(()=>this._update(),250)),this._register(this._editor.onDidChangeModelContent(()=>{this._updateSoon.schedule()})),this._updateSoon.schedule()}dispose(){this._decorations.clear(),super.dispose()}_update(){if(this._model.isDisposed())return;if(!this._model.mightContainNonBasicASCII()){this._decorations.clear();return}const e=this._model.getVersionId();this._editorWorkerService.computedUnicodeHighlights(this._model.uri,this._options).then(t=>{if(this._model.isDisposed()||this._model.getVersionId()!==e)return;this._updateState(t);const i=[];if(!t.hasMore)for(const n of t.ranges)i.push({range:n,options:vk.instance.getDecorationFromOptions(this._options)});this._decorations.set(i)})}getDecorationInfo(e){if(!this._decorations.has(e))return null;const t=this._editor.getModel();if(!sW(t,e))return null;const i=t.getValueInRange(e.range);return{reason:ZY(i,this._options),inComment:oW(t,e),inString:rW(t,e)}}};_4=pB([h_(3,Us)],_4);class iTe extends F{constructor(e,t,i){super(),this._editor=e,this._options=t,this._updateState=i,this._model=this._editor.getModel(),this._decorations=this._editor.createDecorationsCollection(),this._updateSoon=this._register(new It(()=>this._update(),250)),this._register(this._editor.onDidLayoutChange(()=>{this._updateSoon.schedule()})),this._register(this._editor.onDidScrollChange(()=>{this._updateSoon.schedule()})),this._register(this._editor.onDidChangeHiddenAreas(()=>{this._updateSoon.schedule()})),this._register(this._editor.onDidChangeModelContent(()=>{this._updateSoon.schedule()})),this._updateSoon.schedule()}dispose(){this._decorations.clear(),super.dispose()}_update(){if(this._model.isDisposed())return;if(!this._model.mightContainNonBasicASCII()){this._decorations.clear();return}const e=this._editor.getVisibleRanges(),t=[],i={ranges:[],ambiguousCharacterCount:0,invisibleCharacterCount:0,nonBasicAsciiCharacterCount:0,hasMore:!1};for(const n of e){const s=_5.computeUnicodeHighlights(this._model,this._options,n);for(const r of s.ranges)i.ranges.push(r);i.ambiguousCharacterCount+=i.ambiguousCharacterCount,i.invisibleCharacterCount+=i.invisibleCharacterCount,i.nonBasicAsciiCharacterCount+=i.nonBasicAsciiCharacterCount,i.hasMore=i.hasMore||s.hasMore}if(!i.hasMore)for(const n of i.ranges)t.push({range:n,options:vk.instance.getDecorationFromOptions(this._options)});this._updateState(i),this._decorations.set(t)}getDecorationInfo(e){if(!this._decorations.has(e))return null;const t=this._editor.getModel(),i=t.getValueInRange(e.range);return sW(t,e)?{reason:ZY(i,this._options),inComment:oW(t,e),inString:rW(t,e)}:null}}const GY=m("unicodeHighlight.configureUnicodeHighlightOptions","Configure Unicode Highlight Options");let b4=class{constructor(e,t,i){this._editor=e,this._languageService=t,this._openerService=i,this.hoverOrdinal=5}computeSync(e,t){if(!this._editor.hasModel()||e.type!==1)return[];const i=this._editor.getModel(),n=this._editor.getContribution(L1.ID);if(!n)return[];const s=[],r=new Set;let a=300;for(const l of t){const c=n.getDecorationInfo(l);if(!c)continue;const h=i.getValueInRange(l.range).codePointAt(0),u=sR(h);let g;switch(c.reason.kind){case 0:{P1(c.reason.confusableWith)?g=m("unicodeHighlight.characterIsAmbiguousASCII","The character {0} could be confused with the ASCII character {1}, which is more common in source code.",u,sR(c.reason.confusableWith.codePointAt(0))):g=m("unicodeHighlight.characterIsAmbiguous","The character {0} could be confused with the character {1}, which is more common in source code.",u,sR(c.reason.confusableWith.codePointAt(0)));break}case 1:g=m("unicodeHighlight.characterIsInvisible","The character {0} is invisible.",u);break;case 2:g=m("unicodeHighlight.characterIsNonBasicAscii","The character {0} is not a basic ASCII character.",u);break}if(r.has(g))continue;r.add(g);const f={codePoint:h,reason:c.reason,inComment:c.inComment,inString:c.inString},p=m("unicodeHighlight.adjustSettings","Adjust settings"),_=`command:${wk.ID}?${encodeURIComponent(JSON.stringify(f))}`,b=new Pn("",!0).appendMarkdown(g).appendText(" ").appendLink(_,p,GY);s.push(new Hr(this,l.range,[b],!1,a++))}return s}renderHoverParts(e,t){return Tye(e,t,this._editor,this._languageService,this._openerService)}};b4=pB([h_(1,ni),h_(2,zs)],b4);function C4(o){return`U+${o.toString(16).padStart(4,"0")}`}function sR(o){let e=`\`${C4(o)}\``;return fg.isInvisibleCharacter(o)||(e+=` "${`${nTe(o)}`}"`),e}function nTe(o){return o===96?"`` ` ``":"`"+String.fromCodePoint(o)+"`"}function ZY(o,e){return _5.computeUnicodeHighlightReason(o,e)}const QD=class QD{constructor(){this.map=new Map}getDecorationFromOptions(e){return this.getDecoration(!e.includeComments,!e.includeStrings)}getDecoration(e,t){const i=`${e}${t}`;let n=this.map.get(i);return n||(n=We.createDynamic({description:"unicode-highlight",stickiness:1,className:"unicode-highlight",showIfCollapsed:!0,overviewRuler:null,minimap:null,hideInCommentTokens:e,hideInStringTokens:t}),this.map.set(i,n)),n}};QD.instance=new QD;let vk=QD;class sTe extends de{constructor(){super({id:Xf.ID,label:m("action.unicodeHighlight.disableHighlightingInComments","Disable highlighting of characters in comments"),alias:"Disable highlighting of characters in comments",precondition:void 0}),this.shortLabel=m("unicodeHighlight.disableHighlightingInComments.shortLabel","Disable Highlight In Comments")}async run(e,t,i){const n=e==null?void 0:e.get(qe);n&&this.runAction(n)}async runAction(e){await e.updateValue(Es.includeComments,!1,2)}}class oTe extends de{constructor(){super({id:Xf.ID,label:m("action.unicodeHighlight.disableHighlightingInStrings","Disable highlighting of characters in strings"),alias:"Disable highlighting of characters in strings",precondition:void 0}),this.shortLabel=m("unicodeHighlight.disableHighlightingInStrings.shortLabel","Disable Highlight In Strings")}async run(e,t,i){const n=e==null?void 0:e.get(qe);n&&this.runAction(n)}async runAction(e){await e.updateValue(Es.includeStrings,!1,2)}}const JD=class JD extends de{constructor(){super({id:JD.ID,label:m("action.unicodeHighlight.disableHighlightingOfAmbiguousCharacters","Disable highlighting of ambiguous characters"),alias:"Disable highlighting of ambiguous characters",precondition:void 0}),this.shortLabel=m("unicodeHighlight.disableHighlightingOfAmbiguousCharacters.shortLabel","Disable Ambiguous Highlight")}async run(e,t,i){const n=e==null?void 0:e.get(qe);n&&this.runAction(n)}async runAction(e){await e.updateValue(Es.ambiguousCharacters,!1,2)}};JD.ID="editor.action.unicodeHighlight.disableHighlightingOfAmbiguousCharacters";let Xf=JD;const eI=class eI extends de{constructor(){super({id:eI.ID,label:m("action.unicodeHighlight.disableHighlightingOfInvisibleCharacters","Disable highlighting of invisible characters"),alias:"Disable highlighting of invisible characters",precondition:void 0}),this.shortLabel=m("unicodeHighlight.disableHighlightingOfInvisibleCharacters.shortLabel","Disable Invisible Highlight")}async run(e,t,i){const n=e==null?void 0:e.get(qe);n&&this.runAction(n)}async runAction(e){await e.updateValue(Es.invisibleCharacters,!1,2)}};eI.ID="editor.action.unicodeHighlight.disableHighlightingOfInvisibleCharacters";let x1=eI;const tI=class tI extends de{constructor(){super({id:tI.ID,label:m("action.unicodeHighlight.disableHighlightingOfNonBasicAsciiCharacters","Disable highlighting of non basic ASCII characters"),alias:"Disable highlighting of non basic ASCII characters",precondition:void 0}),this.shortLabel=m("unicodeHighlight.disableHighlightingOfNonBasicAsciiCharacters.shortLabel","Disable Non ASCII Highlight")}async run(e,t,i){const n=e==null?void 0:e.get(qe);n&&this.runAction(n)}async runAction(e){await e.updateValue(Es.nonBasicASCII,!1,2)}};tI.ID="editor.action.unicodeHighlight.disableHighlightingOfNonBasicAsciiCharacters";let k1=tI;const iI=class iI extends de{constructor(){super({id:iI.ID,label:m("action.unicodeHighlight.showExcludeOptions","Show Exclude Options"),alias:"Show Exclude Options",precondition:void 0})}async run(e,t,i){const{codePoint:n,reason:s,inString:r,inComment:a}=i,l=String.fromCodePoint(n),c=e.get(xr),d=e.get(qe);function h(f){return fg.isInvisibleCharacter(f)?m("unicodeHighlight.excludeInvisibleCharFromBeingHighlighted","Exclude {0} (invisible character) from being highlighted",C4(f)):m("unicodeHighlight.excludeCharFromBeingHighlighted","Exclude {0} from being highlighted",`${C4(f)} "${l}"`)}const u=[];if(s.kind===0)for(const f of s.notAmbiguousInLocales)u.push({label:m("unicodeHighlight.allowCommonCharactersInLanguage",'Allow unicode characters that are more common in the language "{0}".',f),run:async()=>{aTe(d,[f])}});if(u.push({label:h(n),run:()=>rTe(d,[n])}),a){const f=new sTe;u.push({label:f.label,run:async()=>f.runAction(d)})}else if(r){const f=new oTe;u.push({label:f.label,run:async()=>f.runAction(d)})}if(s.kind===0){const f=new Xf;u.push({label:f.label,run:async()=>f.runAction(d)})}else if(s.kind===1){const f=new x1;u.push({label:f.label,run:async()=>f.runAction(d)})}else if(s.kind===2){const f=new k1;u.push({label:f.label,run:async()=>f.runAction(d)})}else lTe(s);const g=await c.pick(u,{title:GY});g&&await g.run()}};iI.ID="editor.action.unicodeHighlight.showExcludeOptions";let wk=iI;async function rTe(o,e){const t=o.getValue(Es.allowedCharacters);let i;typeof t=="object"&&t?i=t:i={};for(const n of e)i[String.fromCodePoint(n)]=!0;await o.updateValue(Es.allowedCharacters,i,2)}async function aTe(o,e){var n;const t=(n=o.inspect(Es.allowedLocales).user)==null?void 0:n.value;let i;typeof t=="object"&&t?i=Object.assign({},t):i={};for(const s of e)i[s]=!0;await o.updateValue(Es.allowedLocales,i,2)}function lTe(o){throw new Error(`Unexpected value: ${o}`)}Q(Xf);Q(x1);Q(k1);Q(wk);ct(L1.ID,L1,1);um.register(b4);var cTe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},X6=function(o,e){return function(t,i){e(t,i,o)}};const YY="ignoreUnusualLineTerminators";function dTe(o,e,t){o.setModelProperty(e.uri,YY,t)}function hTe(o,e){return o.getModelProperty(e.uri,YY)}var y_;let Sk=(y_=class extends F{constructor(e,t,i){super(),this._editor=e,this._dialogService=t,this._codeEditorService=i,this._isPresentingDialog=!1,this._config=this._editor.getOption(127),this._register(this._editor.onDidChangeConfiguration(n=>{n.hasChanged(127)&&(this._config=this._editor.getOption(127),this._checkForUnusualLineTerminators())})),this._register(this._editor.onDidChangeModel(()=>{this._checkForUnusualLineTerminators()})),this._register(this._editor.onDidChangeModelContent(n=>{n.isUndoing||this._checkForUnusualLineTerminators()})),this._checkForUnusualLineTerminators()}async _checkForUnusualLineTerminators(){if(this._config==="off"||!this._editor.hasModel())return;const e=this._editor.getModel();if(!e.mightContainUnusualLineTerminators()||hTe(this._codeEditorService,e)===!0||this._editor.getOption(92))return;if(this._config==="auto"){e.removeUnusualLineTerminators(this._editor.getSelections());return}if(this._isPresentingDialog)return;let i;try{this._isPresentingDialog=!0,i=await this._dialogService.confirm({title:m("unusualLineTerminators.title","Unusual Line Terminators"),message:m("unusualLineTerminators.message","Detected unusual line terminators"),detail:m("unusualLineTerminators.detail","The file '{0}' contains one or more unusual line terminator characters, like Line Separator (LS) or Paragraph Separator (PS).\n\nIt is recommended to remove them from the file. This can be configured via `editor.unusualLineTerminators`.",vr(e.uri)),primaryButton:m({key:"unusualLineTerminators.fix",comment:["&& denotes a mnemonic"]},"&&Remove Unusual Line Terminators"),cancelButton:m("unusualLineTerminators.ignore","Ignore")})}finally{this._isPresentingDialog=!1}if(!i.confirmed){dTe(this._codeEditorService,e,!0);return}e.removeUnusualLineTerminators(this._editor.getSelections())}},y_.ID="editor.contrib.unusualLineTerminatorsDetector",y_);Sk=cTe([X6(1,H1),X6(2,pt)],Sk);ct(Sk.ID,Sk,1);var uTe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},gTe=function(o,e){return function(t,i){e(t,i,o)}};class Q6{constructor(){this.selector={language:"*"}}provideDocumentHighlights(e,t,i){const n=[],s=e.getWordAtPosition({lineNumber:t.lineNumber,column:t.column});return s?e.isDisposed()?void 0:e.findMatches(s.word,!0,!1,!0,Ny,!1).map(a=>({range:a.range,kind:x_.Text})):Promise.resolve(n)}provideMultiDocumentHighlights(e,t,i,n){const s=new an,r=e.getWordAtPosition({lineNumber:t.lineNumber,column:t.column});if(!r)return Promise.resolve(s);for(const a of[e,...i]){if(a.isDisposed())continue;const c=a.findMatches(r.word,!0,!1,!0,Ny,!1).map(d=>({range:d.range,kind:x_.Text}));c&&s.set(a.uri,c)}return s}}let v4=class extends F{constructor(e){super(),this._register(e.documentHighlightProvider.register("*",new Q6)),this._register(e.multiDocumentHighlightProvider.register("*",new Q6))}};v4=uTe([gTe(0,ge)],v4);var XY=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},yy=function(o,e){return function(t,i){e(t,i,o)}},xi,w4;const _B=new le("hasWordHighlights",!1);function QY(o,e,t,i){const n=o.ordered(e);return n5(n.map(s=>()=>Promise.resolve(s.provideDocumentHighlights(e,t,i)).then(void 0,Si)),s=>s!=null).then(s=>{if(s){const r=new an;return r.set(e.uri,s),r}return new an})}function fTe(o,e,t,i,n,s){const r=o.ordered(e);return n5(r.map(a=>()=>{const l=s.filter(c=>n$(c)).filter(c=>k5(a.selector,c.uri,c.getLanguageId(),!0,void 0,void 0)>0);return Promise.resolve(a.provideMultiDocumentHighlights(e,t,l,n)).then(void 0,Si)}),a=>a!=null)}class JY{constructor(e,t,i){this._model=e,this._selection=t,this._wordSeparators=i,this._wordRange=this._getCurrentWordRange(e,t),this._result=null}get result(){return this._result||(this._result=Yi(e=>this._compute(this._model,this._selection,this._wordSeparators,e))),this._result}_getCurrentWordRange(e,t){const i=e.getWordAtPosition(t.getPosition());return i?new x(t.startLineNumber,i.startColumn,t.startLineNumber,i.endColumn):null}isValid(e,t,i){const n=t.startLineNumber,s=t.startColumn,r=t.endColumn,a=this._getCurrentWordRange(e,t);let l=!!(this._wordRange&&this._wordRange.equalsRange(a));for(let c=0,d=i.length;!l&&c<d;c++){const h=i.getRange(c);h&&h.startLineNumber===n&&h.startColumn<=s&&h.endColumn>=r&&(l=!0)}return l}cancel(){this.result.cancel()}}class mTe extends JY{constructor(e,t,i,n){super(e,t,i),this._providers=n}_compute(e,t,i,n){return QY(this._providers,e,t.getPosition(),n).then(s=>s||new an)}}class pTe extends JY{constructor(e,t,i,n,s){super(e,t,i),this._providers=n,this._otherModels=s}_compute(e,t,i,n){return fTe(this._providers,e,t.getPosition(),i,n,this._otherModels).then(s=>s||new an)}}function _Te(o,e,t,i,n){return new mTe(e,t,n,o)}function bTe(o,e,t,i,n,s){return new pTe(e,t,n,o,s)}Vs("_executeDocumentHighlights",async(o,e,t)=>{const i=o.get(ge),n=await QY(i.documentHighlightProvider,e,t,ze.None);return n==null?void 0:n.get(e.uri)});var Th;let S4=(Th=class{constructor(e,t,i,n,s){this.toUnhook=new K,this.workerRequestTokenId=0,this.workerRequestCompleted=!1,this.workerRequestValue=new an,this.lastCursorPositionChangeTime=0,this.renderDecorationsTimer=-1,this.runDelayer=this.toUnhook.add(new _r(50)),this.editor=e,this.providers=t,this.multiDocumentProviders=i,this.codeEditorService=s,this._hasWordHighlights=_B.bindTo(n),this._ignorePositionChangeEvent=!1,this.occurrencesHighlight=this.editor.getOption(81),this.model=this.editor.getModel(),this.toUnhook.add(e.onDidChangeCursorPosition(r=>{this._ignorePositionChangeEvent||this.occurrencesHighlight!=="off"&&this.runDelayer.trigger(()=>{this._onPositionChanged(r)})})),this.toUnhook.add(e.onDidFocusEditorText(r=>{this.occurrencesHighlight!=="off"&&(this.workerRequest||this.runDelayer.trigger(()=>{this._run()}))})),this.toUnhook.add(e.onDidChangeModelContent(r=>{mI(this.model.uri,"output")||this._stopAll()})),this.toUnhook.add(e.onDidChangeModel(r=>{!r.newModelUrl&&r.oldModelUrl?this._stopSingular():xi.query&&this._run()})),this.toUnhook.add(e.onDidChangeConfiguration(r=>{var l,c;const a=this.editor.getOption(81);if(this.occurrencesHighlight!==a)switch(this.occurrencesHighlight=a,a){case"off":this._stopAll();break;case"singleFile":this._stopAll((c=(l=xi.query)==null?void 0:l.modelInfo)==null?void 0:c.model);break;case"multiFile":xi.query&&this._run(!0);break;default:console.warn("Unknown occurrencesHighlight setting value:",a);break}})),this.decorations=this.editor.createDecorationsCollection(),this.workerRequestTokenId=0,this.workerRequest=null,this.workerRequestCompleted=!1,this.lastCursorPositionChangeTime=0,this.renderDecorationsTimer=-1,xi.query&&this._run()}hasDecorations(){return this.decorations.length>0}restore(){this.occurrencesHighlight!=="off"&&(this.runDelayer.cancel(),this._run())}_getSortedHighlights(){return this.decorations.getRanges().sort(x.compareRangesUsingStarts)}moveNext(){const e=this._getSortedHighlights(),i=(e.findIndex(s=>s.containsPosition(this.editor.getPosition()))+1)%e.length,n=e[i];try{this._ignorePositionChangeEvent=!0,this.editor.setPosition(n.getStartPosition()),this.editor.revealRangeInCenterIfOutsideViewport(n);const s=this._getWord();if(s){const r=this.editor.getModel().getLineContent(n.startLineNumber);Cs(`${r}, ${i+1} of ${e.length} for '${s.word}'`)}}finally{this._ignorePositionChangeEvent=!1}}moveBack(){const e=this._getSortedHighlights(),i=(e.findIndex(s=>s.containsPosition(this.editor.getPosition()))-1+e.length)%e.length,n=e[i];try{this._ignorePositionChangeEvent=!0,this.editor.setPosition(n.getStartPosition()),this.editor.revealRangeInCenterIfOutsideViewport(n);const s=this._getWord();if(s){const r=this.editor.getModel().getLineContent(n.startLineNumber);Cs(`${r}, ${i+1} of ${e.length} for '${s.word}'`)}}finally{this._ignorePositionChangeEvent=!1}}_removeSingleDecorations(){if(!this.editor.hasModel())return;const e=xi.storedDecorationIDs.get(this.editor.getModel().uri);e&&(this.editor.removeDecorations(e),xi.storedDecorationIDs.delete(this.editor.getModel().uri),this.decorations.length>0&&(this.decorations.clear(),this._hasWordHighlights.set(!1)))}_removeAllDecorations(e){const t=this.codeEditorService.listCodeEditors(),i=[];for(const n of t){if(!n.hasModel()||Lw(n.getModel().uri,e==null?void 0:e.uri))continue;const s=xi.storedDecorationIDs.get(n.getModel().uri);if(!s)continue;n.removeDecorations(s),i.push(n.getModel().uri);const r=eu.get(n);r!=null&&r.wordHighlighter&&r.wordHighlighter.decorations.length>0&&(r.wordHighlighter.decorations.clear(),r.wordHighlighter.workerRequest=null,r.wordHighlighter._hasWordHighlights.set(!1))}for(const n of i)xi.storedDecorationIDs.delete(n)}_stopSingular(){var e,t,i,n;this._removeSingleDecorations(),this.editor.hasTextFocus()&&(((e=this.editor.getModel())==null?void 0:e.uri.scheme)!==Fe.vscodeNotebookCell&&((i=(t=xi.query)==null?void 0:t.modelInfo)==null?void 0:i.model.uri.scheme)!==Fe.vscodeNotebookCell?(xi.query=null,this._run()):(n=xi.query)!=null&&n.modelInfo&&(xi.query.modelInfo=null)),this.renderDecorationsTimer!==-1&&(clearTimeout(this.renderDecorationsTimer),this.renderDecorationsTimer=-1),this.workerRequest!==null&&(this.workerRequest.cancel(),this.workerRequest=null),this.workerRequestCompleted||(this.workerRequestTokenId++,this.workerRequestCompleted=!0)}_stopAll(e){this._removeAllDecorations(e),this.renderDecorationsTimer!==-1&&(clearTimeout(this.renderDecorationsTimer),this.renderDecorationsTimer=-1),this.workerRequest!==null&&(this.workerRequest.cancel(),this.workerRequest=null),this.workerRequestCompleted||(this.workerRequestTokenId++,this.workerRequestCompleted=!0)}_onPositionChanged(e){var t;if(this.occurrencesHighlight==="off"){this._stopAll();return}if(e.reason!==3&&((t=this.editor.getModel())==null?void 0:t.uri.scheme)!==Fe.vscodeNotebookCell){this._stopAll();return}this._run()}_getWord(){const e=this.editor.getSelection(),t=e.startLineNumber,i=e.startColumn;return this.model.isDisposed()?null:this.model.getWordAtPosition({lineNumber:t,column:i})}getOtherModelsToHighlight(e){if(!e)return[];if(e.uri.scheme===Fe.vscodeNotebookCell){const s=[],r=this.codeEditorService.listCodeEditors();for(const a of r){const l=a.getModel();l&&l!==e&&l.uri.scheme===Fe.vscodeNotebookCell&&s.push(l)}return s}const i=[],n=this.codeEditorService.listCodeEditors();for(const s of n){if(!yW(s))continue;const r=s.getModel();r&&e===r.modified&&i.push(r.modified)}if(i.length)return i;if(this.occurrencesHighlight==="singleFile")return[];for(const s of n){const r=s.getModel();r&&r!==e&&i.push(r)}return i}_run(e){var n,s;let t;if(this.editor.hasTextFocus()){const r=this.editor.getSelection();if(!r||r.startLineNumber!==r.endLineNumber){xi.query=null,this._stopAll();return}const a=r.startColumn,l=r.endColumn,c=this._getWord();if(!c||c.startColumn>a||c.endColumn<l){xi.query=null,this._stopAll();return}t=this.workerRequest&&this.workerRequest.isValid(this.model,r,this.decorations),xi.query={modelInfo:{model:this.model,selection:r},word:c}}else if(!xi.query){this._stopAll();return}if(this.lastCursorPositionChangeTime=new Date().getTime(),t)this.workerRequestCompleted&&this.renderDecorationsTimer!==-1&&(clearTimeout(this.renderDecorationsTimer),this.renderDecorationsTimer=-1,this._beginRenderDecorations());else if(Lw(this.editor.getModel().uri,(n=xi.query.modelInfo)==null?void 0:n.model.uri)){if(!e){const l=this.decorations.getRanges();for(const c of l)if(c.containsPosition(this.editor.getPosition()))return}this._stopAll(e?this.model:void 0);const r=++this.workerRequestTokenId;this.workerRequestCompleted=!1;const a=this.getOtherModelsToHighlight(this.editor.getModel());if(!xi.query||!xi.query.modelInfo||xi.query.modelInfo.model.isDisposed())return;this.workerRequest=this.computeWithModel(xi.query.modelInfo.model,xi.query.modelInfo.selection,xi.query.word,a),(s=this.workerRequest)==null||s.result.then(l=>{r===this.workerRequestTokenId&&(this.workerRequestCompleted=!0,this.workerRequestValue=l||[],this._beginRenderDecorations())},Oe)}}computeWithModel(e,t,i,n){return n.length?bTe(this.multiDocumentProviders,e,t,i,this.editor.getOption(132),n):_Te(this.providers,e,t,i,this.editor.getOption(132))}_beginRenderDecorations(){const e=new Date().getTime(),t=this.lastCursorPositionChangeTime+250;e>=t?(this.renderDecorationsTimer=-1,this.renderDecorations()):this.renderDecorationsTimer=setTimeout(()=>{this.renderDecorations()},t-e)}renderDecorations(){var t,i,n;this.renderDecorationsTimer=-1;const e=this.codeEditorService.listCodeEditors();for(const s of e){const r=eu.get(s);if(!r)continue;const a=[],l=(t=s.getModel())==null?void 0:t.uri;if(l&&this.workerRequestValue.has(l)){const c=xi.storedDecorationIDs.get(l),d=this.workerRequestValue.get(l);if(d)for(const u of d)u.range&&a.push({range:u.range,options:rEe(u.kind)});let h=[];s.changeDecorations(u=>{h=u.deltaDecorations(c??[],a)}),xi.storedDecorationIDs=xi.storedDecorationIDs.set(l,h),a.length>0&&((i=r.wordHighlighter)==null||i.decorations.set(a),(n=r.wordHighlighter)==null||n._hasWordHighlights.set(!0))}}}dispose(){this._stopSingular(),this.toUnhook.dispose()}},xi=Th,Th.storedDecorationIDs=new an,Th.query=null,Th);S4=xi=XY([yy(4,pt)],S4);var vf;let eu=(vf=class extends F{static get(e){return e.getContribution(w4.ID)}constructor(e,t,i,n){super(),this._wordHighlighter=null;const s=()=>{e.hasModel()&&!e.getModel().isTooLargeForTokenization()&&(this._wordHighlighter=new S4(e,i.documentHighlightProvider,i.multiDocumentHighlightProvider,t,n))};this._register(e.onDidChangeModel(r=>{this._wordHighlighter&&(this._wordHighlighter.dispose(),this._wordHighlighter=null),s()})),s()}get wordHighlighter(){return this._wordHighlighter}saveViewState(){return!!(this._wordHighlighter&&this._wordHighlighter.hasDecorations())}moveNext(){var e;(e=this._wordHighlighter)==null||e.moveNext()}moveBack(){var e;(e=this._wordHighlighter)==null||e.moveBack()}restoreViewState(e){this._wordHighlighter&&e&&this._wordHighlighter.restore()}dispose(){this._wordHighlighter&&(this._wordHighlighter.dispose(),this._wordHighlighter=null),super.dispose()}},w4=vf,vf.ID="editor.contrib.wordHighlighter",vf);eu=w4=XY([yy(1,Ee),yy(2,ge),yy(3,pt)],eu);class eX extends de{constructor(e,t){super(t),this._isNext=e}run(e,t){const i=eu.get(t);i&&(this._isNext?i.moveNext():i.moveBack())}}class CTe extends eX{constructor(){super(!0,{id:"editor.action.wordHighlight.next",label:m("wordHighlight.next.label","Go to Next Symbol Highlight"),alias:"Go to Next Symbol Highlight",precondition:_B,kbOpts:{kbExpr:I.editorTextFocus,primary:65,weight:100}})}}class vTe extends eX{constructor(){super(!1,{id:"editor.action.wordHighlight.prev",label:m("wordHighlight.previous.label","Go to Previous Symbol Highlight"),alias:"Go to Previous Symbol Highlight",precondition:_B,kbOpts:{kbExpr:I.editorTextFocus,primary:1089,weight:100}})}}class wTe extends de{constructor(){super({id:"editor.action.wordHighlight.trigger",label:m("wordHighlight.trigger.label","Trigger Symbol Highlight"),alias:"Trigger Symbol Highlight",precondition:void 0,kbOpts:{kbExpr:I.editorTextFocus,primary:0,weight:100}})}run(e,t,i){const n=eu.get(t);n&&n.restoreViewState(!0)}}ct(eu.ID,eu,0);Q(CTe);Q(vTe);Q(wTe);Nb(v4);class qE extends $i{constructor(e){super(e),this._inSelectionMode=e.inSelectionMode,this._wordNavigationType=e.wordNavigationType}runEditorCommand(e,t,i){if(!t.hasModel())return;const n=Cr(t.getOption(132),t.getOption(131)),s=t.getModel(),r=t.getSelections(),a=r.length>1,l=r.map(c=>{const d=new A(c.positionLineNumber,c.positionColumn),h=this._move(n,s,d,this._wordNavigationType,a);return this._moveTo(c,h,this._inSelectionMode)});if(s.pushStackElement(),t._getViewModel().setCursorStates("moveWordCommand",3,l.map(c=>ft.fromModelSelection(c))),l.length===1){const c=new A(l[0].positionLineNumber,l[0].positionColumn);t.revealPosition(c,0)}}_moveTo(e,t,i){return i?new pe(e.selectionStartLineNumber,e.selectionStartColumn,t.lineNumber,t.column):new pe(t.lineNumber,t.column,t.lineNumber,t.column)}}class cu extends qE{_move(e,t,i,n,s){return wt.moveWordLeft(e,t,i,n,s)}}class du extends qE{_move(e,t,i,n,s){return wt.moveWordRight(e,t,i,n)}}class STe extends cu{constructor(){super({inSelectionMode:!1,wordNavigationType:0,id:"cursorWordStartLeft",precondition:void 0})}}class yTe extends cu{constructor(){super({inSelectionMode:!1,wordNavigationType:2,id:"cursorWordEndLeft",precondition:void 0})}}class LTe extends cu{constructor(){var e;super({inSelectionMode:!1,wordNavigationType:1,id:"cursorWordLeft",precondition:void 0,kbOpts:{kbExpr:G.and(I.textInputFocus,(e=G.and($1,aE))==null?void 0:e.negate()),primary:2063,mac:{primary:527},weight:100}})}}class xTe extends cu{constructor(){super({inSelectionMode:!0,wordNavigationType:0,id:"cursorWordStartLeftSelect",precondition:void 0})}}class kTe extends cu{constructor(){super({inSelectionMode:!0,wordNavigationType:2,id:"cursorWordEndLeftSelect",precondition:void 0})}}class DTe extends cu{constructor(){var e;super({inSelectionMode:!0,wordNavigationType:1,id:"cursorWordLeftSelect",precondition:void 0,kbOpts:{kbExpr:G.and(I.textInputFocus,(e=G.and($1,aE))==null?void 0:e.negate()),primary:3087,mac:{primary:1551},weight:100}})}}class ITe extends cu{constructor(){super({inSelectionMode:!1,wordNavigationType:3,id:"cursorWordAccessibilityLeft",precondition:void 0})}_move(e,t,i,n,s){return super._move(Cr(Ua.wordSeparators.defaultValue,e.intlSegmenterLocales),t,i,n,s)}}class ETe extends cu{constructor(){super({inSelectionMode:!0,wordNavigationType:3,id:"cursorWordAccessibilityLeftSelect",precondition:void 0})}_move(e,t,i,n,s){return super._move(Cr(Ua.wordSeparators.defaultValue,e.intlSegmenterLocales),t,i,n,s)}}class NTe extends du{constructor(){super({inSelectionMode:!1,wordNavigationType:0,id:"cursorWordStartRight",precondition:void 0})}}class TTe extends du{constructor(){var e;super({inSelectionMode:!1,wordNavigationType:2,id:"cursorWordEndRight",precondition:void 0,kbOpts:{kbExpr:G.and(I.textInputFocus,(e=G.and($1,aE))==null?void 0:e.negate()),primary:2065,mac:{primary:529},weight:100}})}}class RTe extends du{constructor(){super({inSelectionMode:!1,wordNavigationType:2,id:"cursorWordRight",precondition:void 0})}}class MTe extends du{constructor(){super({inSelectionMode:!0,wordNavigationType:0,id:"cursorWordStartRightSelect",precondition:void 0})}}class ATe extends du{constructor(){var e;super({inSelectionMode:!0,wordNavigationType:2,id:"cursorWordEndRightSelect",precondition:void 0,kbOpts:{kbExpr:G.and(I.textInputFocus,(e=G.and($1,aE))==null?void 0:e.negate()),primary:3089,mac:{primary:1553},weight:100}})}}class PTe extends du{constructor(){super({inSelectionMode:!0,wordNavigationType:2,id:"cursorWordRightSelect",precondition:void 0})}}class OTe extends du{constructor(){super({inSelectionMode:!1,wordNavigationType:3,id:"cursorWordAccessibilityRight",precondition:void 0})}_move(e,t,i,n,s){return super._move(Cr(Ua.wordSeparators.defaultValue,e.intlSegmenterLocales),t,i,n,s)}}class FTe extends du{constructor(){super({inSelectionMode:!0,wordNavigationType:3,id:"cursorWordAccessibilityRightSelect",precondition:void 0})}_move(e,t,i,n,s){return super._move(Cr(Ua.wordSeparators.defaultValue,e.intlSegmenterLocales),t,i,n,s)}}class GE extends $i{constructor(e){super(e),this._whitespaceHeuristics=e.whitespaceHeuristics,this._wordNavigationType=e.wordNavigationType}runEditorCommand(e,t,i){const n=e.get(di);if(!t.hasModel())return;const s=Cr(t.getOption(132),t.getOption(131)),r=t.getModel(),a=t.getSelections(),l=t.getOption(6),c=t.getOption(11),d=n.getLanguageConfiguration(r.getLanguageId()).getAutoClosingPairs(),h=t._getViewModel(),u=a.map(g=>{const f=this._delete({wordSeparators:s,model:r,selection:g,whitespaceHeuristics:this._whitespaceHeuristics,autoClosingDelete:t.getOption(9),autoClosingBrackets:l,autoClosingQuotes:c,autoClosingPairs:d,autoClosedCharacters:h.getCursorAutoClosedCharacters()},this._wordNavigationType);return new kn(f,"")});t.pushUndoStop(),t.executeCommands(this.id,u),t.pushUndoStop()}}class bB extends GE{_delete(e,t){const i=wt.deleteWordLeft(e,t);return i||new x(1,1,1,1)}}class CB extends GE{_delete(e,t){const i=wt.deleteWordRight(e,t);if(i)return i;const n=e.model.getLineCount(),s=e.model.getLineMaxColumn(n);return new x(n,s,n,s)}}class WTe extends bB{constructor(){super({whitespaceHeuristics:!1,wordNavigationType:0,id:"deleteWordStartLeft",precondition:I.writable})}}class BTe extends bB{constructor(){super({whitespaceHeuristics:!1,wordNavigationType:2,id:"deleteWordEndLeft",precondition:I.writable})}}class HTe extends bB{constructor(){super({whitespaceHeuristics:!0,wordNavigationType:0,id:"deleteWordLeft",precondition:I.writable,kbOpts:{kbExpr:I.textInputFocus,primary:2049,mac:{primary:513},weight:100}})}}class VTe extends CB{constructor(){super({whitespaceHeuristics:!1,wordNavigationType:0,id:"deleteWordStartRight",precondition:I.writable})}}class zTe extends CB{constructor(){super({whitespaceHeuristics:!1,wordNavigationType:2,id:"deleteWordEndRight",precondition:I.writable})}}class UTe extends CB{constructor(){super({whitespaceHeuristics:!0,wordNavigationType:2,id:"deleteWordRight",precondition:I.writable,kbOpts:{kbExpr:I.textInputFocus,primary:2068,mac:{primary:532},weight:100}})}}class $Te extends de{constructor(){super({id:"deleteInsideWord",precondition:I.writable,label:m("deleteInsideWord","Delete Word"),alias:"Delete Word"})}run(e,t,i){if(!t.hasModel())return;const n=Cr(t.getOption(132),t.getOption(131)),s=t.getModel(),a=t.getSelections().map(l=>{const c=wt.deleteInsideWord(n,s,l);return new kn(c,"")});t.pushUndoStop(),t.executeCommands(this.id,a),t.pushUndoStop()}}re(new STe);re(new yTe);re(new LTe);re(new xTe);re(new kTe);re(new DTe);re(new NTe);re(new TTe);re(new RTe);re(new MTe);re(new ATe);re(new PTe);re(new ITe);re(new ETe);re(new OTe);re(new FTe);re(new WTe);re(new BTe);re(new HTe);re(new VTe);re(new zTe);re(new UTe);Q($Te);class jTe extends GE{constructor(){super({whitespaceHeuristics:!0,wordNavigationType:0,id:"deleteWordPartLeft",precondition:I.writable,kbOpts:{kbExpr:I.textInputFocus,primary:0,mac:{primary:769},weight:100}})}_delete(e,t){const i=bE.deleteWordPartLeft(e);return i||new x(1,1,1,1)}}class KTe extends GE{constructor(){super({whitespaceHeuristics:!0,wordNavigationType:2,id:"deleteWordPartRight",precondition:I.writable,kbOpts:{kbExpr:I.textInputFocus,primary:0,mac:{primary:788},weight:100}})}_delete(e,t){const i=bE.deleteWordPartRight(e);if(i)return i;const n=e.model.getLineCount(),s=e.model.getLineMaxColumn(n);return new x(n,s,n,s)}}class tX extends qE{_move(e,t,i,n,s){return bE.moveWordPartLeft(e,t,i,s)}}class qTe extends tX{constructor(){super({inSelectionMode:!1,wordNavigationType:0,id:"cursorWordPartLeft",precondition:void 0,kbOpts:{kbExpr:I.textInputFocus,primary:0,mac:{primary:783},weight:100}})}}rt.registerCommandAlias("cursorWordPartStartLeft","cursorWordPartLeft");class GTe extends tX{constructor(){super({inSelectionMode:!0,wordNavigationType:0,id:"cursorWordPartLeftSelect",precondition:void 0,kbOpts:{kbExpr:I.textInputFocus,primary:0,mac:{primary:1807},weight:100}})}}rt.registerCommandAlias("cursorWordPartStartLeftSelect","cursorWordPartLeftSelect");class iX extends qE{_move(e,t,i,n,s){return bE.moveWordPartRight(e,t,i)}}class ZTe extends iX{constructor(){super({inSelectionMode:!1,wordNavigationType:2,id:"cursorWordPartRight",precondition:void 0,kbOpts:{kbExpr:I.textInputFocus,primary:0,mac:{primary:785},weight:100}})}}class YTe extends iX{constructor(){super({inSelectionMode:!0,wordNavigationType:2,id:"cursorWordPartRightSelect",precondition:void 0,kbOpts:{kbExpr:I.textInputFocus,primary:0,mac:{primary:1809},weight:100}})}}re(new jTe);re(new KTe);re(new qTe);re(new GTe);re(new ZTe);re(new YTe);const DB=class DB extends F{constructor(e){super(),this.editor=e,this._register(this.editor.onDidAttemptReadOnlyEdit(()=>this._onDidAttemptReadOnlyEdit()))}_onDidAttemptReadOnlyEdit(){const e=lo.get(this.editor);if(e&&this.editor.hasModel()){let t=this.editor.getOptions().get(93);t||(this.editor.isSimpleWidget?t=new Pn(m("editor.simple.readonly","Cannot edit in read-only input")):t=new Pn(m("editor.readonly","Cannot edit in read-only editor"))),e.showMessage(t,this.editor.getPosition())}}};DB.ID="editor.contrib.readOnlyMessageController";let yk=DB;ct(yk.ID,yk,2);var XTe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},J6=function(o,e){return function(t,i){e(t,i,o)}};let y4=class extends F{constructor(e,t,i){super(),this._textModel=e,this._languageFeaturesService=t,this._outlineModelService=i,this._currentModel=Ke(this,void 0);const n=xn("documentSymbolProvider.onDidChange",this._languageFeaturesService.documentSymbolProvider.onDidChange),s=xn("_textModel.onDidChangeContent",oe.debounce(r=>this._textModel.onDidChangeContent(r),()=>{},100));this._register(Ps(async(r,a)=>{n.read(r),s.read(r);const l=a.add(new YCe),c=await this._outlineModelService.getOrCreate(this._textModel,l.token);a.isDisposed||this._currentModel.set(c,void 0)}))}getBreadcrumbItems(e,t){const i=this._currentModel.read(t);if(!i)return[];const n=i.asListOfDocumentSymbols().filter(s=>e.contains(s.range.startLineNumber)&&!e.contains(s.range.endLineNumber));return n.sort(wz(gs(s=>s.range.endLineNumber-s.range.startLineNumber,$r))),n.map(s=>({name:s.name,kind:s.kind,startLineNumber:s.range.startLineNumber}))}};y4=XTe([J6(1,ge),J6(2,pS)],y4);yx.setBreadcrumbsSourceFactory((o,e)=>e.createInstance(y4,o));const IB=class IB extends F{constructor(e){super(),this.editor=e,this.widget=null,Pa&&(this._register(e.onDidChangeConfiguration(()=>this.update())),this.update())}update(){const e=!this.editor.getOption(92);!this.widget&&e?this.widget=new L4(this.editor):this.widget&&!e&&(this.widget.dispose(),this.widget=null)}dispose(){super.dispose(),this.widget&&(this.widget.dispose(),this.widget=null)}};IB.ID="editor.contrib.iPadShowKeyboard";let Lk=IB;const nI=class nI extends F{constructor(e){super(),this.editor=e,this._domNode=document.createElement("textarea"),this._domNode.className="iPadShowKeyboard",this._register(U(this._domNode,"touchstart",t=>{this.editor.focus()})),this._register(U(this._domNode,"focus",t=>{this.editor.focus()})),this.editor.addOverlayWidget(this)}dispose(){this.editor.removeOverlayWidget(this),super.dispose()}getId(){return nI.ID}getDomNode(){return this._domNode}getPosition(){return{preference:1}}};nI.ID="editor.contrib.ShowKeyboardWidget";let L4=nI;ct(Lk.ID,Lk,3);var QTe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},ez=function(o,e){return function(t,i){e(t,i,o)}},x4,wf;let D1=(wf=class extends F{static get(e){return e.getContribution(x4.ID)}constructor(e,t,i){super(),this._editor=e,this._languageService=i,this._widget=null,this._register(this._editor.onDidChangeModel(n=>this.stop())),this._register(this._editor.onDidChangeModelLanguage(n=>this.stop())),this._register(fi.onDidChange(n=>this.stop())),this._register(this._editor.onKeyUp(n=>n.keyCode===9&&this.stop()))}dispose(){this.stop(),super.dispose()}launch(){this._widget||this._editor.hasModel()&&(this._widget=new k4(this._editor,this._languageService))}stop(){this._widget&&(this._widget.dispose(),this._widget=null)}},x4=wf,wf.ID="editor.contrib.inspectTokens",wf);D1=x4=QTe([ez(1,Qo),ez(2,ni)],D1);class JTe extends de{constructor(){super({id:"editor.action.inspectTokens",label:M2.inspectTokensAction,alias:"Developer: Inspect Tokens",precondition:void 0})}run(e,t){const i=D1.get(t);i==null||i.launch()}}function eRe(o){let e="";for(let t=0,i=o.length;t<i;t++){const n=o.charCodeAt(t);switch(n){case 9:e+="→";break;case 32:e+="·";break;default:e+=String.fromCharCode(n)}}return e}function tRe(o,e){const t=fi.get(e);if(t)return t;const i=o.encodeLanguageId(e);return{getInitialState:()=>F_,tokenize:(n,s,r)=>Z5(e,r),tokenizeEncoded:(n,s,r)=>XI(i,r)}}const sI=class sI extends F{constructor(e,t){super(),this.allowEditorOverflow=!0,this._editor=e,this._languageService=t,this._model=this._editor.getModel(),this._domNode=document.createElement("div"),this._domNode.className="tokens-inspect-widget",this._tokenizationSupport=tRe(this._languageService.languageIdCodec,this._model.getLanguageId()),this._compute(this._editor.getPosition()),this._register(this._editor.onDidChangeCursorPosition(i=>this._compute(this._editor.getPosition()))),this._editor.addContentWidget(this)}dispose(){this._editor.removeContentWidget(this),super.dispose()}getId(){return sI._ID}_compute(e){const t=this._getTokensAtLine(e.lineNumber);let i=0;for(let l=t.tokens1.length-1;l>=0;l--){const c=t.tokens1[l];if(e.column-1>=c.offset){i=l;break}}let n=0;for(let l=t.tokens2.length>>>1;l>=0;l--)if(e.column-1>=t.tokens2[l<<1]){n=l;break}const s=this._model.getLineContent(e.lineNumber);let r="";if(i<t.tokens1.length){const l=t.tokens1[i].offset,c=i+1<t.tokens1.length?t.tokens1[i+1].offset:s.length;r=s.substring(l,c)}_n(this._domNode,ne("h2.tm-token",void 0,eRe(r),ne("span.tm-token-length",void 0,`${r.length} ${r.length===1?"char":"chars"}`))),X(this._domNode,ne("hr.tokens-inspect-separator",{style:"clear:both"}));const a=(n<<1)+1<t.tokens2.length?this._decodeMetadata(t.tokens2[(n<<1)+1]):null;X(this._domNode,ne("table.tm-metadata-table",void 0,ne("tbody",void 0,ne("tr",void 0,ne("td.tm-metadata-key",void 0,"language"),ne("td.tm-metadata-value",void 0,`${a?a.languageId:"-?-"}`)),ne("tr",void 0,ne("td.tm-metadata-key",void 0,"token type"),ne("td.tm-metadata-value",void 0,`${a?this._tokenTypeToString(a.tokenType):"-?-"}`)),ne("tr",void 0,ne("td.tm-metadata-key",void 0,"font style"),ne("td.tm-metadata-value",void 0,`${a?this._fontStyleToString(a.fontStyle):"-?-"}`)),ne("tr",void 0,ne("td.tm-metadata-key",void 0,"foreground"),ne("td.tm-metadata-value",void 0,`${a?Z.Format.CSS.formatHex(a.foreground):"-?-"}`)),ne("tr",void 0,ne("td.tm-metadata-key",void 0,"background"),ne("td.tm-metadata-value",void 0,`${a?Z.Format.CSS.formatHex(a.background):"-?-"}`))))),X(this._domNode,ne("hr.tokens-inspect-separator")),i<t.tokens1.length&&X(this._domNode,ne("span.tm-token-type",void 0,t.tokens1[i].type)),this._editor.layoutContentWidget(this)}_decodeMetadata(e){const t=fi.getColorMap(),i=Ts.getLanguageId(e),n=Ts.getTokenType(e),s=Ts.getFontStyle(e),r=Ts.getForeground(e),a=Ts.getBackground(e);return{languageId:this._languageService.languageIdCodec.decodeLanguageId(i),tokenType:n,fontStyle:s,foreground:t[r],background:t[a]}}_tokenTypeToString(e){switch(e){case 0:return"Other";case 1:return"Comment";case 2:return"String";case 3:return"RegEx";default:return"??"}}_fontStyleToString(e){let t="";return e&1&&(t+="italic "),e&2&&(t+="bold "),e&4&&(t+="underline "),e&8&&(t+="strikethrough "),t.length===0&&(t="---"),t}_getTokensAtLine(e){const t=this._getStateBeforeLine(e),i=this._tokenizationSupport.tokenize(this._model.getLineContent(e),!0,t),n=this._tokenizationSupport.tokenizeEncoded(this._model.getLineContent(e),!0,t);return{startState:t,tokens1:i.tokens,tokens2:n.tokens,endState:i.endState}}_getStateBeforeLine(e){let t=this._tokenizationSupport.getInitialState();for(let i=1;i<e;i++)t=this._tokenizationSupport.tokenize(this._model.getLineContent(i),!0,t).endState;return t}getDomNode(){return this._domNode}getPosition(){return{position:this._editor.getPosition(),preference:[2,1]}}};sI._ID="editor.contrib.inspectTokensWidget";let k4=sI;ct(D1.ID,D1,4);Q(JTe);var iRe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},tz=function(o,e){return function(t,i){e(t,i,o)}},zC,Sf;let D4=(Sf=class{constructor(e,t){this.quickInputService=e,this.keybindingService=t,this.registry=ci.as(om.Quickaccess)}provide(e){const t=new K;return t.add(e.onDidAccept(()=>{const[i]=e.selectedItems;i&&this.quickInputService.quickAccess.show(i.prefix,{preserveValue:!0})})),t.add(e.onDidChangeValue(i=>{const n=this.registry.getQuickAccessProvider(i.substr(zC.PREFIX.length));n&&n.prefix&&n.prefix!==zC.PREFIX&&this.quickInputService.quickAccess.show(n.prefix,{preserveValue:!0})})),e.items=this.getQuickAccessProviders().filter(i=>i.prefix!==zC.PREFIX),t}getQuickAccessProviders(){return this.registry.getQuickAccessProviders().sort((t,i)=>t.prefix.localeCompare(i.prefix)).flatMap(t=>this.createPicks(t))}createPicks(e){return e.helpEntries.map(t=>{const i=t.prefix||e.prefix,n=i||"…";return{prefix:i,label:n,keybinding:t.commandId?this.keybindingService.lookupKeybinding(t.commandId):void 0,ariaLabel:m("helpPickAriaLabel","{0}, {1}",n,t.description),description:t.description}})}},zC=Sf,Sf.PREFIX="?",Sf);D4=zC=iRe([tz(0,xr),tz(1,yt)],D4);ci.as(om.Quickaccess).registerQuickAccessProvider({ctor:D4,prefix:"",helpEntries:[{description:A2.helpQuickAccessActionLabel}]});class nX{constructor(e){this.options=e,this.rangeHighlightDecorationId=void 0}provide(e,t,i){var r;const n=new K;e.canAcceptInBackground=!!((r=this.options)!=null&&r.canAcceptInBackground),e.matchOnLabel=e.matchOnDescription=e.matchOnDetail=e.sortByLabel=!1;const s=n.add(new Ji);return s.value=this.doProvide(e,t,i),n.add(this.onDidActiveTextEditorControlChange(()=>{s.value=void 0,s.value=this.doProvide(e,t)})),n}doProvide(e,t,i){const n=new K,s=this.activeTextEditorControl;if(s&&this.canProvideWithTextEditor(s)){const r={editor:s},a=Xq(s);if(a){let l=s.saveViewState()??void 0;n.add(a.onDidChangeCursorPosition(()=>{l=s.saveViewState()??void 0})),r.restoreViewState=()=>{l&&s===this.activeTextEditorControl&&s.restoreViewState(l)},n.add(Ah(t.onCancellationRequested)(()=>{var c;return(c=r.restoreViewState)==null?void 0:c.call(r)}))}n.add(ve(()=>this.clearDecorations(s))),n.add(this.provideWithTextEditor(r,e,t,i))}else n.add(this.provideWithoutTextEditor(e,t));return n}canProvideWithTextEditor(e){return!0}gotoLocation({editor:e},t){e.setSelection(t.range,"code.jump"),e.revealRangeInCenter(t.range,0),t.preserveFocus||e.focus();const i=e.getModel();i&&"getLineContent"in i&&Hl(`${i.getLineContent(t.range.startLineNumber)}`)}getModel(e){var t;return yW(e)?(t=e.getModel())==null?void 0:t.modified:e.getModel()}addDecorations(e,t){e.changeDecorations(i=>{const n=[];this.rangeHighlightDecorationId&&(n.push(this.rangeHighlightDecorationId.overviewRulerDecorationId),n.push(this.rangeHighlightDecorationId.rangeHighlightId),this.rangeHighlightDecorationId=void 0);const s=[{range:t,options:{description:"quick-access-range-highlight",className:"rangeHighlight",isWholeLine:!0}},{range:t,options:{description:"quick-access-range-highlight-overview",overviewRuler:{color:mi(zj),position:Ho.Full}}}],[r,a]=i.deltaDecorations(n,s);this.rangeHighlightDecorationId={rangeHighlightId:r,overviewRulerDecorationId:a}})}clearDecorations(e){const t=this.rangeHighlightDecorationId;t&&(e.changeDecorations(i=>{i.deltaDecorations([t.overviewRulerDecorationId,t.rangeHighlightId],[])}),this.rangeHighlightDecorationId=void 0)}}const oI=class oI extends nX{constructor(){super({canAcceptInBackground:!0})}provideWithoutTextEditor(e){const t=m("cannotRunGotoLine","Open a text editor first to go to a line.");return e.items=[{label:t}],e.ariaLabel=t,F.None}provideWithTextEditor(e,t,i){const n=e.editor,s=new K;s.add(t.onDidAccept(l=>{const[c]=t.selectedItems;if(c){if(!this.isValidLineNumber(n,c.lineNumber))return;this.gotoLocation(e,{range:this.toRange(c.lineNumber,c.column),keyMods:t.keyMods,preserveFocus:l.inBackground}),l.inBackground||t.hide()}}));const r=()=>{const l=this.parsePosition(n,t.value.trim().substr(oI.PREFIX.length)),c=this.getPickLabel(n,l.lineNumber,l.column);if(t.items=[{lineNumber:l.lineNumber,column:l.column,label:c}],t.ariaLabel=c,!this.isValidLineNumber(n,l.lineNumber)){this.clearDecorations(n);return}const d=this.toRange(l.lineNumber,l.column);n.revealRangeInCenter(d,0),this.addDecorations(n,d)};r(),s.add(t.onDidChangeValue(()=>r()));const a=Xq(n);return a&&a.getOptions().get(68).renderType===2&&(a.updateOptions({lineNumbers:"on"}),s.add(ve(()=>a.updateOptions({lineNumbers:"relative"})))),s}toRange(e=1,t=1){return{startLineNumber:e,startColumn:t,endLineNumber:e,endColumn:t}}parsePosition(e,t){const i=t.split(/,|:|#/).map(s=>parseInt(s,10)).filter(s=>!isNaN(s)),n=this.lineCount(e)+1;return{lineNumber:i[0]>0?i[0]:n+i[0],column:i[1]}}getPickLabel(e,t,i){if(this.isValidLineNumber(e,t))return this.isValidColumn(e,t,i)?m("gotoLineColumnLabel","Go to line {0} and character {1}.",t,i):m("gotoLineLabel","Go to line {0}.",t);const n=e.getPosition()||{lineNumber:1,column:1},s=this.lineCount(e);return s>1?m("gotoLineLabelEmptyWithLimit","Current Line: {0}, Character: {1}. Type a line number between 1 and {2} to navigate to.",n.lineNumber,n.column,s):m("gotoLineLabelEmpty","Current Line: {0}, Character: {1}. Type a line number to navigate to.",n.lineNumber,n.column)}isValidLineNumber(e,t){return!t||typeof t!="number"?!1:t>0&&t<=this.lineCount(e)}isValidColumn(e,t,i){if(!i||typeof i!="number")return!1;const n=this.getModel(e);if(!n)return!1;const s={lineNumber:t,column:i};return n.validatePosition(s).equals(s)}lineCount(e){var t;return((t=this.getModel(e))==null?void 0:t.getLineCount())??0}};oI.PREFIX=":";let I4=oI;var nRe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},sRe=function(o,e){return function(t,i){e(t,i,o)}};let I1=class extends I4{constructor(e){super(),this.editorService=e,this.onDidActiveTextEditorControlChange=oe.None}get activeTextEditorControl(){return this.editorService.getFocusedCodeEditor()??void 0}};I1=nRe([sRe(0,pt)],I1);var yf;let sX=(yf=class extends de{constructor(){super({id:yf.ID,label:bL.gotoLineActionLabel,alias:"Go to Line/Column...",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:2085,mac:{primary:293},weight:100}})}run(e){e.get(xr).quickAccess.show(I1.PREFIX)}},yf.ID="editor.action.gotoLine",yf);Q(sX);ci.as(om.Quickaccess).registerQuickAccessProvider({ctor:I1,prefix:I1.PREFIX,helpEntries:[{description:bL.gotoLineActionLabel,commandId:sX.ID}]});const oX=[void 0,[]];function oR(o,e,t=0,i=0){const n=e;return n.values&&n.values.length>1?oRe(o,n.values,t,i):rX(o,e,t,i)}function oRe(o,e,t,i){let n=0;const s=[];for(const r of e){const[a,l]=rX(o,r,t,i);if(typeof a!="number")return oX;n+=a,s.push(...l)}return[n,rRe(s)]}function rX(o,e,t,i){const n=Mf(e.original,e.originalLowercase,t,o,o.toLowerCase(),i,{firstMatchCanBeWeak:!0,boostFullMatch:!0});return n?[n[0],U1(n)]:oX}function rRe(o){const e=o.sort((n,s)=>n.start-s.start),t=[];let i;for(const n of e)!i||!aRe(i,n)?(i=n,t.push(n)):(i.start=Math.min(i.start,n.start),i.end=Math.max(i.end,n.end));return t}function aRe(o,e){return!(o.end<e.start||e.end<o.start)}function iz(o){return o.startsWith('"')&&o.endsWith('"')}const aX=" ";function E4(o){typeof o!="string"&&(o="");const e=o.toLowerCase(),{pathNormalized:t,normalized:i,normalizedLowercase:n}=nz(o),s=t.indexOf(Ia)>=0,r=iz(o);let a;const l=o.split(aX);if(l.length>1)for(const c of l){const d=iz(c),{pathNormalized:h,normalized:u,normalizedLowercase:g}=nz(c);u&&(a||(a=[]),a.push({original:c,originalLowercase:c.toLowerCase(),pathNormalized:h,normalized:u,normalizedLowercase:g,expectContiguousMatch:d}))}return{original:o,originalLowercase:e,pathNormalized:t,normalized:i,normalizedLowercase:n,values:a,containsPathSeparator:s,expectContiguousMatch:r}}function nz(o){let e;En?e=o.replace(/\//g,Ia):e=o.replace(/\\/g,Ia);const t=Tee(e).replace(/\s|"/g,"");return{pathNormalized:e,normalized:t,normalizedLowercase:t.toLowerCase()}}function sz(o){return Array.isArray(o)?E4(o.map(e=>e.original).join(aX)):E4(o.original)}var lRe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},oz=function(o,e){return function(t,i){e(t,i,o)}},Ly,ya;let kg=(ya=class extends nX{constructor(e,t,i=Object.create(null)){super(i),this._languageFeaturesService=e,this._outlineModelService=t,this.options=i,this.options.canAcceptInBackground=!0}provideWithoutTextEditor(e){return this.provideLabelPick(e,m("cannotRunGotoSymbolWithoutEditor","To go to a symbol, first open a text editor with symbol information.")),F.None}provideWithTextEditor(e,t,i,n){const s=e.editor,r=this.getModel(s);return r?this._languageFeaturesService.documentSymbolProvider.has(r)?this.doProvideWithEditorSymbols(e,r,t,i,n):this.doProvideWithoutEditorSymbols(e,r,t,i):F.None}doProvideWithoutEditorSymbols(e,t,i,n){const s=new K;return this.provideLabelPick(i,m("cannotRunGotoSymbolWithoutSymbolProvider","The active text editor does not provide symbol information.")),(async()=>!await this.waitForLanguageSymbolRegistry(t,s)||n.isCancellationRequested||s.add(this.doProvideWithEditorSymbols(e,t,i,n)))(),s}provideLabelPick(e,t){e.items=[{label:t,index:0,kind:14}],e.ariaLabel=t}async waitForLanguageSymbolRegistry(e,t){if(this._languageFeaturesService.documentSymbolProvider.has(e))return!0;const i=new gb,n=t.add(this._languageFeaturesService.documentSymbolProvider.onDidChange(()=>{this._languageFeaturesService.documentSymbolProvider.has(e)&&(n.dispose(),i.complete(!0))}));return t.add(ve(()=>i.complete(!1))),i.p}doProvideWithEditorSymbols(e,t,i,n,s){var h;const r=e.editor,a=new K;a.add(i.onDidAccept(u=>{var f;const[g]=i.selectedItems;g&&g.range&&(this.gotoLocation(e,{range:g.range.selection,keyMods:i.keyMods,preserveFocus:u.inBackground}),(f=s==null?void 0:s.handleAccept)==null||f.call(s,g),u.inBackground||i.hide())})),a.add(i.onDidTriggerItemButton(({item:u})=>{u&&u.range&&(this.gotoLocation(e,{range:u.range.selection,keyMods:i.keyMods,forceSideBySide:!0}),i.hide())}));const l=this.getDocumentSymbols(t,n);let c;const d=async u=>{c==null||c.dispose(!0),i.busy=!1,c=new li(n),i.busy=!0;try{const g=E4(i.value.substr(Ly.PREFIX.length).trim()),f=await this.doGetSymbolPicks(l,g,void 0,c.token,t);if(n.isCancellationRequested)return;if(f.length>0){if(i.items=f,u&&g.original.length===0){const p=dw(f,_=>!!(_.type!=="separator"&&_.range&&x.containsPosition(_.range.decoration,u)));p&&(i.activeItems=[p])}}else g.original.length>0?this.provideLabelPick(i,m("noMatchingSymbolResults","No matching editor symbols")):this.provideLabelPick(i,m("noSymbolResults","No editor symbols"))}finally{n.isCancellationRequested||(i.busy=!1)}};return a.add(i.onDidChangeValue(()=>d(void 0))),d((h=r.getSelection())==null?void 0:h.getPosition()),a.add(i.onDidChangeActive(()=>{const[u]=i.activeItems;u&&u.range&&(r.revealRangeInCenter(u.range.selection,0),this.addDecorations(r,u.range.decoration))})),a}async doGetSymbolPicks(e,t,i,n,s){var _,b;const r=await e;if(n.isCancellationRequested)return[];const a=t.original.indexOf(Ly.SCOPE_PREFIX)===0,l=a?1:0;let c,d;t.values&&t.values.length>1?(c=sz(t.values[0]),d=sz(t.values.slice(1))):c=t;let h;const u=(b=(_=this.options)==null?void 0:_.openSideBySideDirection)==null?void 0:b.call(_);u&&(h=[{iconClass:u==="right"?Ie.asClassName(te.splitHorizontal):Ie.asClassName(te.splitVertical),tooltip:u==="right"?m("openToSide","Open to the Side"):m("openToBottom","Open to the Bottom")}]);const g=[];for(let C=0;C<r.length;C++){const v=r[C],w=Eee(v.name),S=`$(${Oy.toIcon(v.kind).id}) ${w}`,L=S.length-w.length;let k=v.containerName;i!=null&&i.extraContainerLabel&&(k?k=`${i.extraContainerLabel} • ${k}`:k=i.extraContainerLabel);let D,N,B,W;if(t.original.length>l){let $=!1;if(c!==t&&([D,N]=oR(S,{...t,values:void 0},l,L),typeof D=="number"&&($=!0)),typeof D!="number"&&([D,N]=oR(S,c,l,L),typeof D!="number"))continue;if(!$&&d){if(k&&d.original.length>0&&([B,W]=oR(k,d)),typeof B!="number")continue;typeof D=="number"&&(D+=B)}}const V=v.tags&&v.tags.indexOf(1)>=0;g.push({index:C,kind:v.kind,score:D,label:S,ariaLabel:See(v.name,v.kind),description:k,highlights:V?void 0:{label:N,description:W},range:{selection:x.collapseToStart(v.selectionRange),decoration:v.range},uri:s.uri,symbolName:w,strikethrough:V,buttons:h})}const f=g.sort((C,v)=>a?this.compareByKindAndScore(C,v):this.compareByScore(C,v));let p=[];if(a){let S=function(){v&&typeof C=="number"&&w>0&&(v.label=xf(aR[C]||rR,w))},C,v,w=0;for(const L of f)C!==L.kind?(S(),C=L.kind,w=1,v={type:"separator"},p.push(v)):w++,p.push(L);S()}else f.length>0&&(p=[{label:m("symbols","symbols ({0})",g.length),type:"separator"},...f]);return p}compareByScore(e,t){if(typeof e.score!="number"&&typeof t.score=="number")return 1;if(typeof e.score=="number"&&typeof t.score!="number")return-1;if(typeof e.score=="number"&&typeof t.score=="number"){if(e.score>t.score)return-1;if(e.score<t.score)return 1}return e.index<t.index?-1:e.index>t.index?1:0}compareByKindAndScore(e,t){const i=aR[e.kind]||rR,n=aR[t.kind]||rR,s=i.localeCompare(n);return s===0?this.compareByScore(e,t):s}async getDocumentSymbols(e,t){const i=await this._outlineModelService.getOrCreate(e,t);return t.isCancellationRequested?[]:i.asListOfDocumentSymbols()}},Ly=ya,ya.PREFIX="@",ya.SCOPE_PREFIX=":",ya.PREFIX_BY_CATEGORY=`${ya.PREFIX}${ya.SCOPE_PREFIX}`,ya);kg=Ly=lRe([oz(0,ge),oz(1,pS)],kg);const rR=m("property","properties ({0})"),aR={5:m("method","methods ({0})"),11:m("function","functions ({0})"),8:m("_constructor","constructors ({0})"),12:m("variable","variables ({0})"),4:m("class","classes ({0})"),22:m("struct","structs ({0})"),23:m("event","events ({0})"),24:m("operator","operators ({0})"),10:m("interface","interfaces ({0})"),2:m("namespace","namespaces ({0})"),3:m("package","packages ({0})"),25:m("typeParameter","type parameters ({0})"),1:m("modules","modules ({0})"),6:m("property","properties ({0})"),9:m("enum","enumerations ({0})"),21:m("enumMember","enumeration members ({0})"),14:m("string","strings ({0})"),0:m("file","files ({0})"),17:m("array","arrays ({0})"),15:m("number","numbers ({0})"),16:m("boolean","booleans ({0})"),18:m("object","objects ({0})"),19:m("key","keys ({0})"),7:m("field","fields ({0})"),13:m("constant","constants ({0})")};var cRe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},lR=function(o,e){return function(t,i){e(t,i,o)}};let N4=class extends kg{constructor(e,t,i){super(t,i),this.editorService=e,this.onDidActiveTextEditorControlChange=oe.None}get activeTextEditorControl(){return this.editorService.getFocusedCodeEditor()??void 0}};N4=cRe([lR(0,pt),lR(1,ge),lR(2,pS)],N4);const rI=class rI extends de{constructor(){super({id:rI.ID,label:kw.quickOutlineActionLabel,alias:"Go to Symbol...",precondition:I.hasDocumentSymbolProvider,kbOpts:{kbExpr:I.focus,primary:3117,weight:100},contextMenuOpts:{group:"navigation",order:3}})}run(e){e.get(xr).quickAccess.show(kg.PREFIX,{itemActivation:pa.NONE})}};rI.ID="editor.action.quickOutline";let xk=rI;Q(xk);ci.as(om.Quickaccess).registerQuickAccessProvider({ctor:N4,prefix:kg.PREFIX,helpEntries:[{description:kw.quickOutlineActionLabel,prefix:kg.PREFIX,commandId:xk.ID},{description:kw.quickOutlineByCategoryActionLabel,prefix:kg.PREFIX_BY_CATEGORY}]});function dRe(o){const e=new Map;for(const t of o)e.set(t,(e.get(t)??0)+1);return e}class Ev{constructor(){this.chunkCount=0,this.chunkOccurrences=new Map,this.documents=new Map}calculateScores(e,t){const i=this.computeEmbedding(e),n=new Map,s=[];for(const[r,a]of this.documents){if(t.isCancellationRequested)return[];for(const l of a.chunks){const c=this.computeSimilarityScore(l,i,n);c>0&&s.push({key:r,score:c})}}return s}static termFrequencies(e){return dRe(Ev.splitTerms(e))}static*splitTerms(e){const t=i=>i.toLowerCase();for(const[i]of e.matchAll(new RegExp("\\b\\p{Letter}[\\p{Letter}\\d]{2,}\\b","gu"))){yield t(i);const n=i.replace(/([a-z])([A-Z])/g,"$1 $2").split(/\s+/g);if(n.length>1)for(const s of n)s.length>2&&new RegExp("\\p{Letter}{3,}","gu").test(s)&&(yield t(s))}}updateDocuments(e){for(const{key:t}of e)this.deleteDocument(t);for(const t of e){const i=[];for(const n of t.textChunks){const s=Ev.termFrequencies(n);for(const r of s.keys())this.chunkOccurrences.set(r,(this.chunkOccurrences.get(r)??0)+1);i.push({text:n,tf:s})}this.chunkCount+=i.length,this.documents.set(t.key,{chunks:i})}return this}deleteDocument(e){const t=this.documents.get(e);if(t){this.documents.delete(e),this.chunkCount-=t.chunks.length;for(const i of t.chunks)for(const n of i.tf.keys()){const s=this.chunkOccurrences.get(n);if(typeof s=="number"){const r=s-1;r<=0?this.chunkOccurrences.delete(n):this.chunkOccurrences.set(n,r)}}}}computeSimilarityScore(e,t,i){let n=0;for(const[s,r]of Object.entries(t)){const a=e.tf.get(s);if(!a)continue;let l=i.get(s);typeof l!="number"&&(l=this.computeIdf(s),i.set(s,l));const c=a*l;n+=c*r}return n}computeEmbedding(e){const t=Ev.termFrequencies(e);return this.computeTfidf(t)}computeIdf(e){const t=this.chunkOccurrences.get(e)??0;return t>0?Math.log((this.chunkCount+1)/t):0}computeTfidf(e){const t=Object.create(null);for(const[i,n]of e){const s=this.computeIdf(i);s>0&&(t[i]=n*s)}return t}}function hRe(o){var i;const e=o.slice(0);e.sort((n,s)=>s.score-n.score);const t=((i=e[0])==null?void 0:i.score)??0;if(t>0)for(const n of e)n.score/=t;return e}var Sp;(function(o){o[o.NO_ACTION=0]="NO_ACTION",o[o.CLOSE_PICKER=1]="CLOSE_PICKER",o[o.REFRESH_PICKER=2]="REFRESH_PICKER",o[o.REMOVE_ITEM=3]="REMOVE_ITEM"})(Sp||(Sp={}));function cR(o){const e=o;return Array.isArray(e.items)}function rz(o){const e=o;return!!e.picks&&e.additionalPicks instanceof Promise}class uRe extends F{constructor(e,t){super(),this.prefix=e,this.options=t}provide(e,t,i){var c;const n=new K;e.canAcceptInBackground=!!((c=this.options)!=null&&c.canAcceptInBackground),e.matchOnLabel=e.matchOnDescription=e.matchOnDetail=e.sortByLabel=!1;let s;const r=n.add(new Ji),a=async()=>{var _;const d=r.value=new K;s==null||s.dispose(!0),e.busy=!1,s=new li(t);const h=s.token;let u=e.value.substring(this.prefix.length);(_=this.options)!=null&&_.shouldSkipTrimPickFilter||(u=u.trim());const g=this._getPicks(u,d,h,i),f=(b,C)=>{var S;let v,w;if(cR(b)?(v=b.items,w=b.active):v=b,v.length===0){if(C)return!1;(u.length>0||e.hideInput)&&((S=this.options)!=null&&S.noResultsPick)&&(Xv(this.options.noResultsPick)?v=[this.options.noResultsPick(u)]:v=[this.options.noResultsPick])}return e.items=v,w&&(e.activeItems=[w]),!0},p=async b=>{let C=!1,v=!1;await Promise.all([(async()=>{typeof b.mergeDelay=="number"&&(await ed(b.mergeDelay),h.isCancellationRequested)||v||(C=f(b.picks,!0))})(),(async()=>{e.busy=!0;try{const w=await b.additionalPicks;if(h.isCancellationRequested)return;let S,L;cR(b.picks)?(S=b.picks.items,L=b.picks.active):S=b.picks;let k,D;if(cR(w)?(k=w.items,D=w.active):k=w,k.length>0||!C){let N;if(!L&&!D){const B=e.activeItems[0];B&&S.indexOf(B)!==-1&&(N=B)}f({items:[...S,...k],active:L||D||N})}}finally{h.isCancellationRequested||(e.busy=!1),v=!0}})()])};if(g!==null)if(rz(g))await p(g);else if(!(g instanceof Promise))f(g);else{e.busy=!0;try{const b=await g;if(h.isCancellationRequested)return;rz(b)?await p(b):f(b)}finally{h.isCancellationRequested||(e.busy=!1)}}};n.add(e.onDidChangeValue(()=>a())),a(),n.add(e.onDidAccept(d=>{var u;if(i!=null&&i.handleAccept){d.inBackground||e.hide(),(u=i.handleAccept)==null||u.call(i,e.activeItems[0]);return}const[h]=e.selectedItems;typeof(h==null?void 0:h.accept)=="function"&&(d.inBackground||e.hide(),h.accept(e.keyMods,d))}));const l=async(d,h)=>{var g;if(typeof h.trigger!="function")return;const u=((g=h.buttons)==null?void 0:g.indexOf(d))??-1;if(u>=0){const f=h.trigger(u,e.keyMods),p=typeof f=="number"?f:await f;if(t.isCancellationRequested)return;switch(p){case Sp.NO_ACTION:break;case Sp.CLOSE_PICKER:e.hide();break;case Sp.REFRESH_PICKER:a();break;case Sp.REMOVE_ITEM:{const _=e.items.indexOf(h);if(_!==-1){const b=e.items.slice(),C=b.splice(_,1),v=e.activeItems.filter(S=>S!==C[0]),w=e.keepScrollPosition;e.keepScrollPosition=!0,e.items=b,v&&(e.activeItems=v),e.keepScrollPosition=w}break}}}};return n.add(e.onDidTriggerItemButton(({button:d,item:h})=>l(d,h))),n.add(e.onDidTriggerSeparatorButton(({button:d,separator:h})=>l(d,h))),n}}var lX=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},jd=function(o,e){return function(t,i){e(t,i,o)}},Bu,Vi,Il;let T4=(Il=class extends uRe{constructor(e,t,i,n,s,r){super(Bu.PREFIX,e),this.instantiationService=t,this.keybindingService=i,this.commandService=n,this.telemetryService=s,this.dialogService=r,this.commandsHistory=this._register(this.instantiationService.createInstance(R4)),this.options=e}async _getPicks(e,t,i,n){var g,f;const s=await this.getCommandPicks(i);if(i.isCancellationRequested)return[];const r=Ah(()=>{const p=new Ev;p.updateDocuments(s.map(b=>({key:b.commandId,textChunks:[this.getTfIdfChunk(b)]})));const _=p.calculateScores(e,i);return hRe(_).filter(b=>b.score>Bu.TFIDF_THRESHOLD).slice(0,Bu.TFIDF_MAX_RESULTS)}),a=[];for(const p of s){const _=Bu.WORD_FILTER(e,p.label)??void 0,b=p.commandAlias?Bu.WORD_FILTER(e,p.commandAlias)??void 0:void 0;if(_||b)p.highlights={label:_,detail:this.options.showAlias?b:void 0},a.push(p);else if(e===p.commandId)a.push(p);else if(e.length>=3){const C=r();if(i.isCancellationRequested)return[];const v=C.find(w=>w.key===p.commandId);v&&(p.tfIdfScore=v.score,a.push(p))}}const l=new Map;for(const p of a){const _=l.get(p.label);_?(p.description=p.commandId,_.description=_.commandId):l.set(p.label,p)}a.sort((p,_)=>{if(p.tfIdfScore&&_.tfIdfScore)return p.tfIdfScore===_.tfIdfScore?p.label.localeCompare(_.label):_.tfIdfScore-p.tfIdfScore;if(p.tfIdfScore)return 1;if(_.tfIdfScore)return-1;const b=this.commandsHistory.peek(p.commandId),C=this.commandsHistory.peek(_.commandId);if(b&&C)return b>C?-1:1;if(b)return-1;if(C)return 1;if(this.options.suggestedCommandIds){const v=this.options.suggestedCommandIds.has(p.commandId),w=this.options.suggestedCommandIds.has(_.commandId);if(v&&w)return 0;if(v)return-1;if(w)return 1}return p.label.localeCompare(_.label)});const c=[];let d=!1,h=!0,u=!!this.options.suggestedCommandIds;for(let p=0;p<a.length;p++){const _=a[p];p===0&&this.commandsHistory.peek(_.commandId)&&(c.push({type:"separator",label:m("recentlyUsed","recently used")}),d=!0),h&&_.tfIdfScore!==void 0&&(c.push({type:"separator",label:m("suggested","similar commands")}),h=!1),u&&_.tfIdfScore===void 0&&!this.commandsHistory.peek(_.commandId)&&((g=this.options.suggestedCommandIds)!=null&&g.has(_.commandId))&&(c.push({type:"separator",label:m("commonlyUsed","commonly used")}),d=!0,u=!1),d&&_.tfIdfScore===void 0&&!this.commandsHistory.peek(_.commandId)&&!((f=this.options.suggestedCommandIds)!=null&&f.has(_.commandId))&&(c.push({type:"separator",label:m("morecCommands","other commands")}),d=!1),c.push(this.toCommandPick(_,n))}return this.hasAdditionalCommandPicks(e,i)?{picks:c,additionalPicks:(async()=>{var b;const p=await this.getAdditionalCommandPicks(s,a,e,i);if(i.isCancellationRequested)return[];const _=p.map(C=>this.toCommandPick(C,n));return h&&((b=_[0])==null?void 0:b.type)!=="separator"&&_.unshift({type:"separator",label:m("suggested","similar commands")}),_})()}:c}toCommandPick(e,t){if(e.type==="separator")return e;const i=this.keybindingService.lookupKeybinding(e.commandId),n=i?m("commandPickAriaLabelWithKeybinding","{0}, {1}",e.label,i.getAriaLabel()):e.label;return{...e,ariaLabel:n,detail:this.options.showAlias&&e.commandAlias!==e.label?e.commandAlias:void 0,keybinding:i,accept:async()=>{var s;this.commandsHistory.push(e.commandId),this.telemetryService.publicLog2("workbenchActionExecuted",{id:e.commandId,from:(t==null?void 0:t.from)??"quick open"});try{(s=e.args)!=null&&s.length?await this.commandService.executeCommand(e.commandId,...e.args):await this.commandService.executeCommand(e.commandId)}catch(r){yr(r)||this.dialogService.error(m("canNotRun","Command '{0}' resulted in an error",e.label),Tx(r))}}}}getTfIdfChunk({label:e,commandAlias:t,commandDescription:i}){let n=e;return t&&t!==e&&(n+=` - ${t}`),i&&i.value!==e&&(n+=` - ${i.value===i.original?i.value:`${i.value} (${i.original})`}`),n}},Bu=Il,Il.PREFIX=">",Il.TFIDF_THRESHOLD=.5,Il.TFIDF_MAX_RESULTS=5,Il.WORD_FILTER=H5(Sw,Vae,tj),Il);T4=Bu=lX([jd(1,we),jd(2,yt),jd(3,Ht),jd(4,Bn),jd(5,H1)],T4);var La;let R4=(La=class extends F{constructor(e,t,i){super(),this.storageService=e,this.configurationService=t,this.logService=i,this.configuredCommandsHistoryLength=0,this.updateConfiguration(),this.load(),this.registerListeners()}registerListeners(){this._register(this.configurationService.onDidChangeConfiguration(e=>this.updateConfiguration(e))),this._register(this.storageService.onWillSaveState(e=>{e.reason===Nw.SHUTDOWN&&this.saveState()}))}updateConfiguration(e){e&&!e.affectsConfiguration("workbench.commandPalette.history")||(this.configuredCommandsHistoryLength=Vi.getConfiguredCommandHistoryLength(this.configurationService),Vi.cache&&Vi.cache.limit!==this.configuredCommandsHistoryLength&&(Vi.cache.limit=this.configuredCommandsHistoryLength,Vi.hasChanges=!0))}load(){const e=this.storageService.get(Vi.PREF_KEY_CACHE,0);let t;if(e)try{t=JSON.parse(e)}catch(n){this.logService.error(`[CommandsHistory] invalid data: ${n}`)}const i=Vi.cache=new Gl(this.configuredCommandsHistoryLength,1);if(t){let n;t.usesLRU?n=t.entries:n=t.entries.sort((s,r)=>s.value-r.value),n.forEach(s=>i.set(s.key,s.value))}Vi.counter=this.storageService.getNumber(Vi.PREF_KEY_COUNTER,0,Vi.counter)}push(e){Vi.cache&&(Vi.cache.set(e,Vi.counter++),Vi.hasChanges=!0)}peek(e){var t;return(t=Vi.cache)==null?void 0:t.peek(e)}saveState(){if(!Vi.cache||!Vi.hasChanges)return;const e={usesLRU:!0,entries:[]};Vi.cache.forEach((t,i)=>e.entries.push({key:i,value:t})),this.storageService.store(Vi.PREF_KEY_CACHE,JSON.stringify(e),0,0),this.storageService.store(Vi.PREF_KEY_COUNTER,Vi.counter,0,0),Vi.hasChanges=!1}static getConfiguredCommandHistoryLength(e){var n,s;const i=(s=(n=e.getValue().workbench)==null?void 0:n.commandPalette)==null?void 0:s.history;return typeof i=="number"?i:Vi.DEFAULT_COMMANDS_HISTORY_LENGTH}},Vi=La,La.DEFAULT_COMMANDS_HISTORY_LENGTH=50,La.PREF_KEY_CACHE="commandPalette.mru.cache",La.PREF_KEY_COUNTER="commandPalette.mru.counter",La.counter=1,La.hasChanges=!1,La);R4=Vi=lX([jd(0,ia),jd(1,qe),jd(2,Nn)],R4);class gRe extends T4{constructor(e,t,i,n,s,r){super(e,t,i,n,s,r)}getCodeEditorCommandPicks(){var i;const e=this.activeTextEditorControl;if(!e)return[];const t=[];for(const n of e.getSupportedActions()){let s;(i=n.metadata)!=null&&i.description&&(Bde(n.metadata.description)?s=n.metadata.description:s={original:n.metadata.description,value:n.metadata.description}),t.push({commandId:n.id,commandAlias:n.alias,commandDescription:s,label:K5(n.label)||n.id})}return t}}var fRe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},zm=function(o,e){return function(t,i){e(t,i,o)}};let E1=class extends gRe{get activeTextEditorControl(){return this.codeEditorService.getFocusedCodeEditor()??void 0}constructor(e,t,i,n,s,r){super({showAlias:!1},e,i,n,s,r),this.codeEditorService=t}async getCommandPicks(){return this.getCodeEditorCommandPicks()}hasAdditionalCommandPicks(){return!1}async getAdditionalCommandPicks(){return[]}};E1=fRe([zm(0,we),zm(1,pt),zm(2,yt),zm(3,Ht),zm(4,Bn),zm(5,H1)],E1);const aI=class aI extends de{constructor(){super({id:aI.ID,label:CL.quickCommandActionLabel,alias:"Command Palette",precondition:void 0,kbOpts:{kbExpr:I.focus,primary:59,weight:100},contextMenuOpts:{group:"z_commands",order:1}})}run(e){e.get(xr).quickAccess.show(E1.PREFIX)}};aI.ID="editor.action.quickCommand";let kk=aI;Q(kk);ci.as(om.Quickaccess).registerQuickAccessProvider({ctor:E1,prefix:E1.PREFIX,helpEntries:[{description:CL.quickCommandHelp,commandId:kk.ID}]});var mRe=function(o,e,t,i){var n=arguments.length,s=n<3?e:i===null?i=Object.getOwnPropertyDescriptor(e,t):i,r;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")s=Reflect.decorate(o,e,t,i);else for(var a=o.length-1;a>=0;a--)(r=o[a])&&(s=(n<3?r(s):n>3?r(e,t,s):r(e,t))||s);return n>3&&s&&Object.defineProperty(e,t,s),s},Um=function(o,e){return function(t,i){e(t,i,o)}};let M4=class extends qf{constructor(e,t,i,n,s,r,a){super(!0,e,t,i,n,s,r,a)}};M4=mRe([Um(1,Ee),Um(2,pt),Um(3,Ei),Um(4,we),Um(5,ia),Um(6,qe)],M4);ct(qf.ID,M4,4);class pRe extends de{constructor(){super({id:"editor.action.toggleHighContrast",label:O2.toggleHighContrast,alias:"Toggle High Contrast Theme",precondition:void 0}),this._originalThemeName=null}run(e,t){const i=e.get(Qo),n=i.getColorTheme();Ta(n.type)?(i.setTheme(this._originalThemeName||(N_(n.type)?t_:Ic)),this._originalThemeName=null):(i.setTheme(N_(n.type)?wg:Sg),this._originalThemeName=n.themeName)}}Q(pRe);const _Re=Object.freeze(Object.defineProperty({__proto__:null,CancellationTokenSource:tG,Emitter:iG,KeyCode:nG,KeyMod:sG,MarkerSeverity:cG,MarkerTag:dG,Position:oG,Range:rG,Selection:aG,SelectionDirection:lG,Token:uG,Uri:hG,editor:gG,languages:fG},Symbol.toStringTag,{value:"Module"}));function bRe(o){return new Worker(""+new URL("editor.worker-oRlJJsnX.js",import.meta.url).href,{name:o==null?void 0:o.name})}function CRe(o){return new Worker(""+new URL("json.worker-BnUULff4.js",import.meta.url).href,{name:o==null?void 0:o.name})}function vRe(o){return new Worker(""+new URL("css.worker-DGFkc2qH.js",import.meta.url).href,{name:o==null?void 0:o.name})}function wRe(o){return new Worker(""+new URL("html.worker-CltiozTZ.js",import.meta.url).href,{name:o==null?void 0:o.name})}function SRe(o){return new Worker(""+new URL("ts.worker-Bd4z-OY3.js",import.meta.url).href,{name:o==null?void 0:o.name})}const yRe=({onChange:o,value:e,editorRef:t,defaultLanguage:i,format:n,height:s})=>{self.MonacoEnvironment={getWorker(a,l){return l==="json"?new CRe:l==="css"||l==="scss"||l==="less"?new vRe:l==="html"||l==="handlebars"||l==="razor"?new wRe:l==="typescript"||l==="javascript"?new SRe:new bRe}},A4.config({monaco:_Re});function r(a,l){t&&(t.current=a,n&&a.onDidChangeModelDecorations(async()=>{await t.current.getAction("editor.action.formatDocument").run()}))}return QE.jsx(QE.Fragment,{children:QE.jsx(pQ,{height:s||"65vh",value:e,theme:"vs-dark",onChange:o,defaultLanguage:i,onMount:r,options:{formatOnPaste:!0}})})},tPe=Object.freeze(Object.defineProperty({__proto__:null,default:yRe},Symbol.toStringTag,{value:"Module"}));export{tPe as e,rS as m,RG as t};
