const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["./index-CDeD9lyT.js","./index-CUuSmggi.js","./addEventListener-CzbSevz3.js","./index-BCx1q_2Y.css","./index-DEnjftNB.js","./Table-kIlrKW-9.js","./Dropdown-Clv_Pvnt.js","./index-BZ6Sq__j.js","./index-CmUtUGZt.js","./index-BfzdirmR.js","./usePagination-CGdUxo17.js","./index-7u0Bo8e4.js","./index-SGIbgpRk.js","./index-RAxRq3bL.js","./RedoOutlined-d_QTXv-s.js","./index-Cl0Hi3Gl.js","./callSuper-CKCGYiRV.js","./index-DuRuG3rR.js","./index-CkGaCd3X.js","./index-BF4b68Hp.js","./index-CSq209pq.js","./index-sAnz6zLe.js","./index-BQm1iTRH.js","./UpOutlined-B_NQN-zy.js","./index-IijEf41L.js","./index-2r9flV8X.js","./index-B-RBPprY.js","./index-Fy_dqHL_.js","./index-CUrh6kIn.js","./index-Djhsc5K4.js","./study-page-D4nenWxt.js","./index-D0tEL95I.js","./DeleteOutlined-DoDR38qq.js","./index-5ngX0NHj.js","./rgb-BwIoVOhg.js","./index-CQOo18Hr.js","./utils-D2B0dIuM.js","./index-CixgM-5n.js","./index-DhO6XalF.js","./index-D7Hz1C2T.css","./index-Ct2J_Ed7.js","./index-B2uzaINz.js","./index-ZNx73KSX.js","./index-JUwAts5R.js","./index-Bak0iqJO.js","./index-DD1QZChH.js","./index-Bzv7v5Gp.js","./index-D5JMEKyH.js","./index-HDoMyE03.js","./index-D5R5AX7X.js","./index-Cyksu2x_.js","./index-BDXjIJVp.js","./index-BDgclIvd.js","./index-DKqvC_C5.js","./index-tn0RQdqM.css","./index-BiBS7Dlv.js","./index-DrRz1_gm.js","./index-Ctvowf5L.js","./index-DMiH39tH.js","./zoom-BGsqVsDF.js","./index-n9x2b4FL.css","./index-bQ-eeI8R.js","./index-CFitg6x7.js","./index-DYrqibJD.js","./graph-view-DT7knbYz.js","./NodeIndexOutlined-CSnsbedd.js","./index-CMDvsT_M.js","./index-DPpeG1Bx.js"])))=>i.map(i=>d[i]);
function $O(e,t){for(var n=0;n<t.length;n++){const r=t[n];if(typeof r!="string"&&!Array.isArray(r)){for(const a in r)if(a!=="default"&&!(a in e)){const l=Object.getOwnPropertyDescriptor(r,a);l&&Object.defineProperty(e,a,l.get?l:{enumerable:!0,get:()=>r[a]})}}}return Object.freeze(Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}))}(function(){const t=document.createElement("link").relList;if(t&&t.supports&&t.supports("modulepreload"))return;for(const a of document.querySelectorAll('link[rel="modulepreload"]'))r(a);new MutationObserver(a=>{for(const l of a)if(l.type==="childList")for(const s of l.addedNodes)s.tagName==="LINK"&&s.rel==="modulepreload"&&r(s)}).observe(document,{childList:!0,subtree:!0});function n(a){const l={};return a.integrity&&(l.integrity=a.integrity),a.referrerPolicy&&(l.referrerPolicy=a.referrerPolicy),a.crossOrigin==="use-credentials"?l.credentials="include":a.crossOrigin==="anonymous"?l.credentials="omit":l.credentials="same-origin",l}function r(a){if(a.ep)return;a.ep=!0;const l=n(a);fetch(a.href,l)}})();var dX=typeof globalThis<"u"?globalThis:typeof window<"u"?window:typeof global<"u"?global:typeof self<"u"?self:{};function is(e){return e&&e.__esModule&&Object.prototype.hasOwnProperty.call(e,"default")?e.default:e}var ep={exports:{}},pc={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var _1;function vI(){if(_1)return pc;_1=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.fragment");function n(r,a,l){var s=null;if(l!==void 0&&(s=""+l),a.key!==void 0&&(s=""+a.key),"key"in a){l={};for(var u in a)u!=="key"&&(l[u]=a[u])}else l=a;return a=l.ref,{$$typeof:e,type:r,key:s,ref:a!==void 0?a:null,props:l}}return pc.Fragment=t,pc.jsx=n,pc.jsxs=n,pc}var T1;function yI(){return T1||(T1=1,ep.exports=vI()),ep.exports}var te=yI(),tp={exports:{}},vc={},np={exports:{}},rp={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var M1;function bI(){return M1||(M1=1,function(e){function t(j,A){var L=j.length;j.push(A);e:for(;0<L;){var q=L-1>>>1,N=j[q];if(0<a(N,A))j[q]=A,j[L]=N,L=q;else break e}}function n(j){return j.length===0?null:j[0]}function r(j){if(j.length===0)return null;var A=j[0],L=j.pop();if(L!==A){j[0]=L;e:for(var q=0,N=j.length,B=N>>>1;q<B;){var W=2*(q+1)-1,X=j[W],k=W+1,Y=j[k];if(0>a(X,L))k<N&&0>a(Y,X)?(j[q]=Y,j[k]=L,q=k):(j[q]=X,j[W]=L,q=W);else if(k<N&&0>a(Y,L))j[q]=Y,j[k]=L,q=k;else break e}}return A}function a(j,A){var L=j.sortIndex-A.sortIndex;return L!==0?L:j.id-A.id}if(e.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var l=performance;e.unstable_now=function(){return l.now()}}else{var s=Date,u=s.now();e.unstable_now=function(){return s.now()-u}}var d=[],g=[],h=1,p=null,y=3,C=!1,b=!1,x=!1,S=!1,E=typeof setTimeout=="function"?setTimeout:null,$=typeof clearTimeout=="function"?clearTimeout:null,O=typeof setImmediate<"u"?setImmediate:null;function _(j){for(var A=n(g);A!==null;){if(A.callback===null)r(g);else if(A.startTime<=j)r(g),A.sortIndex=A.expirationTime,t(d,A);else break;A=n(g)}}function R(j){if(x=!1,_(j),!b)if(n(d)!==null)b=!0,T||(T=!0,G());else{var A=n(g);A!==null&&K(R,A.startTime-j)}}var T=!1,M=-1,z=5,P=-1;function D(){return S?!0:!(e.unstable_now()-P<z)}function F(){if(S=!1,T){var j=e.unstable_now();P=j;var A=!0;try{e:{b=!1,x&&(x=!1,$(M),M=-1),C=!0;var L=y;try{t:{for(_(j),p=n(d);p!==null&&!(p.expirationTime>j&&D());){var q=p.callback;if(typeof q=="function"){p.callback=null,y=p.priorityLevel;var N=q(p.expirationTime<=j);if(j=e.unstable_now(),typeof N=="function"){p.callback=N,_(j),A=!0;break t}p===n(d)&&r(d),_(j)}else r(d);p=n(d)}if(p!==null)A=!0;else{var B=n(g);B!==null&&K(R,B.startTime-j),A=!1}}break e}finally{p=null,y=L,C=!1}A=void 0}}finally{A?G():T=!1}}}var G;if(typeof O=="function")G=function(){O(F)};else if(typeof MessageChannel<"u"){var V=new MessageChannel,U=V.port2;V.port1.onmessage=F,G=function(){U.postMessage(null)}}else G=function(){E(F,0)};function K(j,A){M=E(function(){j(e.unstable_now())},A)}e.unstable_IdlePriority=5,e.unstable_ImmediatePriority=1,e.unstable_LowPriority=4,e.unstable_NormalPriority=3,e.unstable_Profiling=null,e.unstable_UserBlockingPriority=2,e.unstable_cancelCallback=function(j){j.callback=null},e.unstable_forceFrameRate=function(j){0>j||125<j?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):z=0<j?Math.floor(1e3/j):5},e.unstable_getCurrentPriorityLevel=function(){return y},e.unstable_next=function(j){switch(y){case 1:case 2:case 3:var A=3;break;default:A=y}var L=y;y=A;try{return j()}finally{y=L}},e.unstable_requestPaint=function(){S=!0},e.unstable_runWithPriority=function(j,A){switch(j){case 1:case 2:case 3:case 4:case 5:break;default:j=3}var L=y;y=j;try{return A()}finally{y=L}},e.unstable_scheduleCallback=function(j,A,L){var q=e.unstable_now();switch(typeof L=="object"&&L!==null?(L=L.delay,L=typeof L=="number"&&0<L?q+L:q):L=q,j){case 1:var N=-1;break;case 2:N=250;break;case 5:N=1073741823;break;case 4:N=1e4;break;default:N=5e3}return N=L+N,j={id:h++,callback:A,priorityLevel:j,startTime:L,expirationTime:N,sortIndex:-1},L>q?(j.sortIndex=L,t(g,j),n(d)===null&&j===n(g)&&(x?($(M),M=-1):x=!0,K(R,L-q))):(j.sortIndex=N,t(d,j),b||C||(b=!0,T||(T=!0,G()))),j},e.unstable_shouldYield=D,e.unstable_wrapCallback=function(j){var A=y;return function(){var L=y;y=A;try{return j.apply(this,arguments)}finally{y=L}}}}(rp)),rp}var N1;function SI(){return N1||(N1=1,np.exports=bI()),np.exports}var ap={exports:{}},_t={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var j1;function CI(){if(j1)return _t;j1=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.portal"),n=Symbol.for("react.fragment"),r=Symbol.for("react.strict_mode"),a=Symbol.for("react.profiler"),l=Symbol.for("react.consumer"),s=Symbol.for("react.context"),u=Symbol.for("react.forward_ref"),d=Symbol.for("react.suspense"),g=Symbol.for("react.memo"),h=Symbol.for("react.lazy"),p=Symbol.iterator;function y(N){return N===null||typeof N!="object"?null:(N=p&&N[p]||N["@@iterator"],typeof N=="function"?N:null)}var C={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},b=Object.assign,x={};function S(N,B,W){this.props=N,this.context=B,this.refs=x,this.updater=W||C}S.prototype.isReactComponent={},S.prototype.setState=function(N,B){if(typeof N!="object"&&typeof N!="function"&&N!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,N,B,"setState")},S.prototype.forceUpdate=function(N){this.updater.enqueueForceUpdate(this,N,"forceUpdate")};function E(){}E.prototype=S.prototype;function $(N,B,W){this.props=N,this.context=B,this.refs=x,this.updater=W||C}var O=$.prototype=new E;O.constructor=$,b(O,S.prototype),O.isPureReactComponent=!0;var _=Array.isArray,R={H:null,A:null,T:null,S:null,V:null},T=Object.prototype.hasOwnProperty;function M(N,B,W,X,k,Y){return W=Y.ref,{$$typeof:e,type:N,key:B,ref:W!==void 0?W:null,props:Y}}function z(N,B){return M(N.type,B,void 0,void 0,void 0,N.props)}function P(N){return typeof N=="object"&&N!==null&&N.$$typeof===e}function D(N){var B={"=":"=0",":":"=2"};return"$"+N.replace(/[=:]/g,function(W){return B[W]})}var F=/\/+/g;function G(N,B){return typeof N=="object"&&N!==null&&N.key!=null?D(""+N.key):B.toString(36)}function V(){}function U(N){switch(N.status){case"fulfilled":return N.value;case"rejected":throw N.reason;default:switch(typeof N.status=="string"?N.then(V,V):(N.status="pending",N.then(function(B){N.status==="pending"&&(N.status="fulfilled",N.value=B)},function(B){N.status==="pending"&&(N.status="rejected",N.reason=B)})),N.status){case"fulfilled":return N.value;case"rejected":throw N.reason}}throw N}function K(N,B,W,X,k){var Y=typeof N;(Y==="undefined"||Y==="boolean")&&(N=null);var ee=!1;if(N===null)ee=!0;else switch(Y){case"bigint":case"string":case"number":ee=!0;break;case"object":switch(N.$$typeof){case e:case t:ee=!0;break;case h:return ee=N._init,K(ee(N._payload),B,W,X,k)}}if(ee)return k=k(N),ee=X===""?"."+G(N,0):X,_(k)?(W="",ee!=null&&(W=ee.replace(F,"$&/")+"/"),K(k,B,W,"",function(oe){return oe})):k!=null&&(P(k)&&(k=z(k,W+(k.key==null||N&&N.key===k.key?"":(""+k.key).replace(F,"$&/")+"/")+ee)),B.push(k)),1;ee=0;var ue=X===""?".":X+":";if(_(N))for(var ae=0;ae<N.length;ae++)X=N[ae],Y=ue+G(X,ae),ee+=K(X,B,W,Y,k);else if(ae=y(N),typeof ae=="function")for(N=ae.call(N),ae=0;!(X=N.next()).done;)X=X.value,Y=ue+G(X,ae++),ee+=K(X,B,W,Y,k);else if(Y==="object"){if(typeof N.then=="function")return K(U(N),B,W,X,k);throw B=String(N),Error("Objects are not valid as a React child (found: "+(B==="[object Object]"?"object with keys {"+Object.keys(N).join(", ")+"}":B)+"). If you meant to render a collection of children, use an array instead.")}return ee}function j(N,B,W){if(N==null)return N;var X=[],k=0;return K(N,X,"","",function(Y){return B.call(W,Y,k++)}),X}function A(N){if(N._status===-1){var B=N._result;B=B(),B.then(function(W){(N._status===0||N._status===-1)&&(N._status=1,N._result=W)},function(W){(N._status===0||N._status===-1)&&(N._status=2,N._result=W)}),N._status===-1&&(N._status=0,N._result=B)}if(N._status===1)return N._result.default;throw N._result}var L=typeof reportError=="function"?reportError:function(N){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var B=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof N=="object"&&N!==null&&typeof N.message=="string"?String(N.message):String(N),error:N});if(!window.dispatchEvent(B))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",N);return}console.error(N)};function q(){}return _t.Children={map:j,forEach:function(N,B,W){j(N,function(){B.apply(this,arguments)},W)},count:function(N){var B=0;return j(N,function(){B++}),B},toArray:function(N){return j(N,function(B){return B})||[]},only:function(N){if(!P(N))throw Error("React.Children.only expected to receive a single React element child.");return N}},_t.Component=S,_t.Fragment=n,_t.Profiler=a,_t.PureComponent=$,_t.StrictMode=r,_t.Suspense=d,_t.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=R,_t.__COMPILER_RUNTIME={__proto__:null,c:function(N){return R.H.useMemoCache(N)}},_t.cache=function(N){return function(){return N.apply(null,arguments)}},_t.cloneElement=function(N,B,W){if(N==null)throw Error("The argument must be a React element, but you passed "+N+".");var X=b({},N.props),k=N.key,Y=void 0;if(B!=null)for(ee in B.ref!==void 0&&(Y=void 0),B.key!==void 0&&(k=""+B.key),B)!T.call(B,ee)||ee==="key"||ee==="__self"||ee==="__source"||ee==="ref"&&B.ref===void 0||(X[ee]=B[ee]);var ee=arguments.length-2;if(ee===1)X.children=W;else if(1<ee){for(var ue=Array(ee),ae=0;ae<ee;ae++)ue[ae]=arguments[ae+2];X.children=ue}return M(N.type,k,void 0,void 0,Y,X)},_t.createContext=function(N){return N={$$typeof:s,_currentValue:N,_currentValue2:N,_threadCount:0,Provider:null,Consumer:null},N.Provider=N,N.Consumer={$$typeof:l,_context:N},N},_t.createElement=function(N,B,W){var X,k={},Y=null;if(B!=null)for(X in B.key!==void 0&&(Y=""+B.key),B)T.call(B,X)&&X!=="key"&&X!=="__self"&&X!=="__source"&&(k[X]=B[X]);var ee=arguments.length-2;if(ee===1)k.children=W;else if(1<ee){for(var ue=Array(ee),ae=0;ae<ee;ae++)ue[ae]=arguments[ae+2];k.children=ue}if(N&&N.defaultProps)for(X in ee=N.defaultProps,ee)k[X]===void 0&&(k[X]=ee[X]);return M(N,Y,void 0,void 0,null,k)},_t.createRef=function(){return{current:null}},_t.forwardRef=function(N){return{$$typeof:u,render:N}},_t.isValidElement=P,_t.lazy=function(N){return{$$typeof:h,_payload:{_status:-1,_result:N},_init:A}},_t.memo=function(N,B){return{$$typeof:g,type:N,compare:B===void 0?null:B}},_t.startTransition=function(N){var B=R.T,W={};R.T=W;try{var X=N(),k=R.S;k!==null&&k(W,X),typeof X=="object"&&X!==null&&typeof X.then=="function"&&X.then(q,L)}catch(Y){L(Y)}finally{R.T=B}},_t.unstable_useCacheRefresh=function(){return R.H.useCacheRefresh()},_t.use=function(N){return R.H.use(N)},_t.useActionState=function(N,B,W){return R.H.useActionState(N,B,W)},_t.useCallback=function(N,B){return R.H.useCallback(N,B)},_t.useContext=function(N){return R.H.useContext(N)},_t.useDebugValue=function(){},_t.useDeferredValue=function(N,B){return R.H.useDeferredValue(N,B)},_t.useEffect=function(N,B,W){var X=R.H;if(typeof W=="function")throw Error("useEffect CRUD overload is not enabled in this build of React.");return X.useEffect(N,B)},_t.useId=function(){return R.H.useId()},_t.useImperativeHandle=function(N,B,W){return R.H.useImperativeHandle(N,B,W)},_t.useInsertionEffect=function(N,B){return R.H.useInsertionEffect(N,B)},_t.useLayoutEffect=function(N,B){return R.H.useLayoutEffect(N,B)},_t.useMemo=function(N,B){return R.H.useMemo(N,B)},_t.useOptimistic=function(N,B){return R.H.useOptimistic(N,B)},_t.useReducer=function(N,B,W){return R.H.useReducer(N,B,W)},_t.useRef=function(N){return R.H.useRef(N)},_t.useState=function(N){return R.H.useState(N)},_t.useSyncExternalStore=function(N,B,W){return R.H.useSyncExternalStore(N,B,W)},_t.useTransition=function(){return R.H.useTransition()},_t.version="19.1.0",_t}var I1;function im(){return I1||(I1=1,ap.exports=CI()),ap.exports}var op={exports:{}},xr={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var P1;function xI(){if(P1)return xr;P1=1;var e=im();function t(d){var g="https://react.dev/errors/"+d;if(1<arguments.length){g+="?args[]="+encodeURIComponent(arguments[1]);for(var h=2;h<arguments.length;h++)g+="&args[]="+encodeURIComponent(arguments[h])}return"Minified React error #"+d+"; visit "+g+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function n(){}var r={d:{f:n,r:function(){throw Error(t(522))},D:n,C:n,L:n,m:n,X:n,S:n,M:n},p:0,findDOMNode:null},a=Symbol.for("react.portal");function l(d,g,h){var p=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:a,key:p==null?null:""+p,children:d,containerInfo:g,implementation:h}}var s=e.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function u(d,g){if(d==="font")return"";if(typeof g=="string")return g==="use-credentials"?g:""}return xr.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=r,xr.createPortal=function(d,g){var h=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!g||g.nodeType!==1&&g.nodeType!==9&&g.nodeType!==11)throw Error(t(299));return l(d,g,null,h)},xr.flushSync=function(d){var g=s.T,h=r.p;try{if(s.T=null,r.p=2,d)return d()}finally{s.T=g,r.p=h,r.d.f()}},xr.preconnect=function(d,g){typeof d=="string"&&(g?(g=g.crossOrigin,g=typeof g=="string"?g==="use-credentials"?g:"":void 0):g=null,r.d.C(d,g))},xr.prefetchDNS=function(d){typeof d=="string"&&r.d.D(d)},xr.preinit=function(d,g){if(typeof d=="string"&&g&&typeof g.as=="string"){var h=g.as,p=u(h,g.crossOrigin),y=typeof g.integrity=="string"?g.integrity:void 0,C=typeof g.fetchPriority=="string"?g.fetchPriority:void 0;h==="style"?r.d.S(d,typeof g.precedence=="string"?g.precedence:void 0,{crossOrigin:p,integrity:y,fetchPriority:C}):h==="script"&&r.d.X(d,{crossOrigin:p,integrity:y,fetchPriority:C,nonce:typeof g.nonce=="string"?g.nonce:void 0})}},xr.preinitModule=function(d,g){if(typeof d=="string")if(typeof g=="object"&&g!==null){if(g.as==null||g.as==="script"){var h=u(g.as,g.crossOrigin);r.d.M(d,{crossOrigin:h,integrity:typeof g.integrity=="string"?g.integrity:void 0,nonce:typeof g.nonce=="string"?g.nonce:void 0})}}else g==null&&r.d.M(d)},xr.preload=function(d,g){if(typeof d=="string"&&typeof g=="object"&&g!==null&&typeof g.as=="string"){var h=g.as,p=u(h,g.crossOrigin);r.d.L(d,h,{crossOrigin:p,integrity:typeof g.integrity=="string"?g.integrity:void 0,nonce:typeof g.nonce=="string"?g.nonce:void 0,type:typeof g.type=="string"?g.type:void 0,fetchPriority:typeof g.fetchPriority=="string"?g.fetchPriority:void 0,referrerPolicy:typeof g.referrerPolicy=="string"?g.referrerPolicy:void 0,imageSrcSet:typeof g.imageSrcSet=="string"?g.imageSrcSet:void 0,imageSizes:typeof g.imageSizes=="string"?g.imageSizes:void 0,media:typeof g.media=="string"?g.media:void 0})}},xr.preloadModule=function(d,g){if(typeof d=="string")if(g){var h=u(g.as,g.crossOrigin);r.d.m(d,{as:typeof g.as=="string"&&g.as!=="script"?g.as:void 0,crossOrigin:h,integrity:typeof g.integrity=="string"?g.integrity:void 0})}else r.d.m(d)},xr.requestFormReset=function(d){r.d.r(d)},xr.unstable_batchedUpdates=function(d,g){return d(g)},xr.useFormState=function(d,g,h){return s.H.useFormState(d,g,h)},xr.useFormStatus=function(){return s.H.useHostTransitionStatus()},xr.version="19.1.0",xr}var A1;function OO(){if(A1)return op.exports;A1=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),op.exports=xI(),op.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var z1;function wI(){if(z1)return vc;z1=1;var e=SI(),t=im(),n=OO();function r(o){var i="https://react.dev/errors/"+o;if(1<arguments.length){i+="?args[]="+encodeURIComponent(arguments[1]);for(var c=2;c<arguments.length;c++)i+="&args[]="+encodeURIComponent(arguments[c])}return"Minified React error #"+o+"; visit "+i+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function a(o){return!(!o||o.nodeType!==1&&o.nodeType!==9&&o.nodeType!==11)}function l(o){var i=o,c=o;if(o.alternate)for(;i.return;)i=i.return;else{o=i;do i=o,(i.flags&4098)!==0&&(c=i.return),o=i.return;while(o)}return i.tag===3?c:null}function s(o){if(o.tag===13){var i=o.memoizedState;if(i===null&&(o=o.alternate,o!==null&&(i=o.memoizedState)),i!==null)return i.dehydrated}return null}function u(o){if(l(o)!==o)throw Error(r(188))}function d(o){var i=o.alternate;if(!i){if(i=l(o),i===null)throw Error(r(188));return i!==o?null:o}for(var c=o,m=i;;){var v=c.return;if(v===null)break;var w=v.alternate;if(w===null){if(m=v.return,m!==null){c=m;continue}break}if(v.child===w.child){for(w=v.child;w;){if(w===c)return u(v),o;if(w===m)return u(v),i;w=w.sibling}throw Error(r(188))}if(c.return!==m.return)c=v,m=w;else{for(var I=!1,H=v.child;H;){if(H===c){I=!0,c=v,m=w;break}if(H===m){I=!0,m=v,c=w;break}H=H.sibling}if(!I){for(H=w.child;H;){if(H===c){I=!0,c=w,m=v;break}if(H===m){I=!0,m=w,c=v;break}H=H.sibling}if(!I)throw Error(r(189))}}if(c.alternate!==m)throw Error(r(190))}if(c.tag!==3)throw Error(r(188));return c.stateNode.current===c?o:i}function g(o){var i=o.tag;if(i===5||i===26||i===27||i===6)return o;for(o=o.child;o!==null;){if(i=g(o),i!==null)return i;o=o.sibling}return null}var h=Object.assign,p=Symbol.for("react.element"),y=Symbol.for("react.transitional.element"),C=Symbol.for("react.portal"),b=Symbol.for("react.fragment"),x=Symbol.for("react.strict_mode"),S=Symbol.for("react.profiler"),E=Symbol.for("react.provider"),$=Symbol.for("react.consumer"),O=Symbol.for("react.context"),_=Symbol.for("react.forward_ref"),R=Symbol.for("react.suspense"),T=Symbol.for("react.suspense_list"),M=Symbol.for("react.memo"),z=Symbol.for("react.lazy"),P=Symbol.for("react.activity"),D=Symbol.for("react.memo_cache_sentinel"),F=Symbol.iterator;function G(o){return o===null||typeof o!="object"?null:(o=F&&o[F]||o["@@iterator"],typeof o=="function"?o:null)}var V=Symbol.for("react.client.reference");function U(o){if(o==null)return null;if(typeof o=="function")return o.$$typeof===V?null:o.displayName||o.name||null;if(typeof o=="string")return o;switch(o){case b:return"Fragment";case S:return"Profiler";case x:return"StrictMode";case R:return"Suspense";case T:return"SuspenseList";case P:return"Activity"}if(typeof o=="object")switch(o.$$typeof){case C:return"Portal";case O:return(o.displayName||"Context")+".Provider";case $:return(o._context.displayName||"Context")+".Consumer";case _:var i=o.render;return o=o.displayName,o||(o=i.displayName||i.name||"",o=o!==""?"ForwardRef("+o+")":"ForwardRef"),o;case M:return i=o.displayName||null,i!==null?i:U(o.type)||"Memo";case z:i=o._payload,o=o._init;try{return U(o(i))}catch{}}return null}var K=Array.isArray,j=t.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,A=n.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,L={pending:!1,data:null,method:null,action:null},q=[],N=-1;function B(o){return{current:o}}function W(o){0>N||(o.current=q[N],q[N]=null,N--)}function X(o,i){N++,q[N]=o.current,o.current=i}var k=B(null),Y=B(null),ee=B(null),ue=B(null);function ae(o,i){switch(X(ee,i),X(Y,o),X(k,null),i.nodeType){case 9:case 11:o=(o=i.documentElement)&&(o=o.namespaceURI)?n1(o):0;break;default:if(o=i.tagName,i=i.namespaceURI)i=n1(i),o=r1(i,o);else switch(o){case"svg":o=1;break;case"math":o=2;break;default:o=0}}W(k),X(k,o)}function oe(){W(k),W(Y),W(ee)}function he(o){o.memoizedState!==null&&X(ue,o);var i=k.current,c=r1(i,o.type);i!==c&&(X(Y,o),X(k,c))}function ie(o){Y.current===o&&(W(k),W(Y)),ue.current===o&&(W(ue),fc._currentValue=L)}var le=Object.prototype.hasOwnProperty,be=e.unstable_scheduleCallback,Re=e.unstable_cancelCallback,fe=e.unstable_shouldYield,Se=e.unstable_requestPaint,Ee=e.unstable_now,Oe=e.unstable_getCurrentPriorityLevel,Pe=e.unstable_ImmediatePriority,Ye=e.unstable_UserBlockingPriority,Me=e.unstable_NormalPriority,De=e.unstable_LowPriority,xe=e.unstable_IdlePriority,Te=e.log,Le=e.unstable_setDisableYieldValue,We=null,Ge=null;function at(o){if(typeof Te=="function"&&Le(o),Ge&&typeof Ge.setStrictMode=="function")try{Ge.setStrictMode(We,o)}catch{}}var ke=Math.clz32?Math.clz32:pt,Qe=Math.log,ct=Math.LN2;function pt(o){return o>>>=0,o===0?32:31-(Qe(o)/ct|0)|0}var Ze=256,Be=4194304;function Ue(o){var i=o&42;if(i!==0)return i;switch(o&-o){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return o&4194048;case 4194304:case 8388608:case 16777216:case 33554432:return o&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return o}}function Ie(o,i,c){var m=o.pendingLanes;if(m===0)return 0;var v=0,w=o.suspendedLanes,I=o.pingedLanes;o=o.warmLanes;var H=m&134217727;return H!==0?(m=H&~w,m!==0?v=Ue(m):(I&=H,I!==0?v=Ue(I):c||(c=H&~o,c!==0&&(v=Ue(c))))):(H=m&~w,H!==0?v=Ue(H):I!==0?v=Ue(I):c||(c=m&~o,c!==0&&(v=Ue(c)))),v===0?0:i!==0&&i!==v&&(i&w)===0&&(w=v&-v,c=i&-i,w>=c||w===32&&(c&4194048)!==0)?i:v}function qe(o,i){return(o.pendingLanes&~(o.suspendedLanes&~o.pingedLanes)&i)===0}function wt(o,i){switch(o){case 1:case 2:case 4:case 8:case 64:return i+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return i+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function It(){var o=Ze;return Ze<<=1,(Ze&4194048)===0&&(Ze=256),o}function Nn(){var o=Be;return Be<<=1,(Be&62914560)===0&&(Be=4194304),o}function wn(o){for(var i=[],c=0;31>c;c++)i.push(o);return i}function Rn(o,i){o.pendingLanes|=i,i!==268435456&&(o.suspendedLanes=0,o.pingedLanes=0,o.warmLanes=0)}function dn(o,i,c,m,v,w){var I=o.pendingLanes;o.pendingLanes=c,o.suspendedLanes=0,o.pingedLanes=0,o.warmLanes=0,o.expiredLanes&=c,o.entangledLanes&=c,o.errorRecoveryDisabledLanes&=c,o.shellSuspendCounter=0;var H=o.entanglements,J=o.expirationTimes,me=o.hiddenUpdates;for(c=I&~c;0<c;){var we=31-ke(c),_e=1<<we;H[we]=0,J[we]=-1;var pe=me[we];if(pe!==null)for(me[we]=null,we=0;we<pe.length;we++){var ye=pe[we];ye!==null&&(ye.lane&=-536870913)}c&=~_e}m!==0&&kt(o,m,0),w!==0&&v===0&&o.tag!==0&&(o.suspendedLanes|=w&~(I&~i))}function kt(o,i,c){o.pendingLanes|=i,o.suspendedLanes&=~i;var m=31-ke(i);o.entangledLanes|=i,o.entanglements[m]=o.entanglements[m]|1073741824|c&4194090}function ln(o,i){var c=o.entangledLanes|=i;for(o=o.entanglements;c;){var m=31-ke(c),v=1<<m;v&i|o[m]&i&&(o[m]|=i),c&=~v}}function Ot(o){switch(o){case 2:o=1;break;case 8:o=4;break;case 32:o=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:o=128;break;case 268435456:o=134217728;break;default:o=0}return o}function nt(o){return o&=-o,2<o?8<o?(o&134217727)!==0?32:268435456:8:2}function Ke(){var o=A.p;return o!==0?o:(o=window.event,o===void 0?32:x1(o.type))}function Fe(o,i){var c=A.p;try{return A.p=o,i()}finally{A.p=c}}var vt=Math.random().toString(36).slice(2),Ct="__reactFiber$"+vt,Ve="__reactProps$"+vt,ze="__reactContainer$"+vt,rt="__reactEvents$"+vt,ot="__reactListeners$"+vt,tt="__reactHandles$"+vt,ut="__reactResources$"+vt,qt="__reactMarker$"+vt;function Mt(o){delete o[Ct],delete o[Ve],delete o[rt],delete o[ot],delete o[tt]}function Pt(o){var i=o[Ct];if(i)return i;for(var c=o.parentNode;c;){if(i=c[ze]||c[Ct]){if(c=i.alternate,i.child!==null||c!==null&&c.child!==null)for(o=l1(o);o!==null;){if(c=o[Ct])return c;o=l1(o)}return i}o=c,c=o.parentNode}return null}function At(o){if(o=o[Ct]||o[ze]){var i=o.tag;if(i===5||i===6||i===13||i===26||i===27||i===3)return o}return null}function tn(o){var i=o.tag;if(i===5||i===26||i===27||i===6)return o.stateNode;throw Error(r(33))}function sn(o){var i=o[ut];return i||(i=o[ut]={hoistableStyles:new Map,hoistableScripts:new Map}),i}function Tt(o){o[qt]=!0}var ar=new Set,Sn={};function cn(o,i){Ae(o,i),Ae(o+"Capture",i)}function Ae(o,i){for(Sn[o]=i,o=0;o<i.length;o++)ar.add(i[o])}var et=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),bt={},Et={};function Yt(o){return le.call(Et,o)?!0:le.call(bt,o)?!1:et.test(o)?Et[o]=!0:(bt[o]=!0,!1)}function Kn(o,i,c){if(Yt(i))if(c===null)o.removeAttribute(i);else{switch(typeof c){case"undefined":case"function":case"symbol":o.removeAttribute(i);return;case"boolean":var m=i.toLowerCase().slice(0,5);if(m!=="data-"&&m!=="aria-"){o.removeAttribute(i);return}}o.setAttribute(i,""+c)}}function zn(o,i,c){if(c===null)o.removeAttribute(i);else{switch(typeof c){case"undefined":case"function":case"symbol":case"boolean":o.removeAttribute(i);return}o.setAttribute(i,""+c)}}function Fn(o,i,c,m){if(m===null)o.removeAttribute(c);else{switch(typeof m){case"undefined":case"function":case"symbol":case"boolean":o.removeAttribute(c);return}o.setAttributeNS(i,c,""+m)}}var gr,Sr;function _n(o){if(gr===void 0)try{throw Error()}catch(c){var i=c.stack.trim().match(/\n( *(at )?)/);gr=i&&i[1]||"",Sr=-1<c.stack.indexOf(`
    at`)?" (<anonymous>)":-1<c.stack.indexOf("@")?"@unknown:0:0":""}return`
`+gr+o+Sr}var or=!1;function Fr(o,i){if(!o||or)return"";or=!0;var c=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var m={DetermineComponentFrameRoot:function(){try{if(i){var _e=function(){throw Error()};if(Object.defineProperty(_e.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(_e,[])}catch(ye){var pe=ye}Reflect.construct(o,[],_e)}else{try{_e.call()}catch(ye){pe=ye}o.call(_e.prototype)}}else{try{throw Error()}catch(ye){pe=ye}(_e=o())&&typeof _e.catch=="function"&&_e.catch(function(){})}}catch(ye){if(ye&&pe&&typeof ye.stack=="string")return[ye.stack,pe.stack]}return[null,null]}};m.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var v=Object.getOwnPropertyDescriptor(m.DetermineComponentFrameRoot,"name");v&&v.configurable&&Object.defineProperty(m.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var w=m.DetermineComponentFrameRoot(),I=w[0],H=w[1];if(I&&H){var J=I.split(`
`),me=H.split(`
`);for(v=m=0;m<J.length&&!J[m].includes("DetermineComponentFrameRoot");)m++;for(;v<me.length&&!me[v].includes("DetermineComponentFrameRoot");)v++;if(m===J.length||v===me.length)for(m=J.length-1,v=me.length-1;1<=m&&0<=v&&J[m]!==me[v];)v--;for(;1<=m&&0<=v;m--,v--)if(J[m]!==me[v]){if(m!==1||v!==1)do if(m--,v--,0>v||J[m]!==me[v]){var we=`
`+J[m].replace(" at new "," at ");return o.displayName&&we.includes("<anonymous>")&&(we=we.replace("<anonymous>",o.displayName)),we}while(1<=m&&0<=v);break}}}finally{or=!1,Error.prepareStackTrace=c}return(c=o?o.displayName||o.name:"")?_n(c):""}function ka(o){switch(o.tag){case 26:case 27:case 5:return _n(o.type);case 16:return _n("Lazy");case 13:return _n("Suspense");case 19:return _n("SuspenseList");case 0:case 15:return Fr(o.type,!1);case 11:return Fr(o.type.render,!1);case 1:return Fr(o.type,!0);case 31:return _n("Activity");default:return""}}function Xn(o){try{var i="";do i+=ka(o),o=o.return;while(o);return i}catch(c){return`
Error generating stack: `+c.message+`
`+c.stack}}function mn(o){switch(typeof o){case"bigint":case"boolean":case"number":case"string":case"undefined":return o;case"object":return o;default:return""}}function Vr(o){var i=o.type;return(o=o.nodeName)&&o.toLowerCase()==="input"&&(i==="checkbox"||i==="radio")}function la(o){var i=Vr(o)?"checked":"value",c=Object.getOwnPropertyDescriptor(o.constructor.prototype,i),m=""+o[i];if(!o.hasOwnProperty(i)&&typeof c<"u"&&typeof c.get=="function"&&typeof c.set=="function"){var v=c.get,w=c.set;return Object.defineProperty(o,i,{configurable:!0,get:function(){return v.call(this)},set:function(I){m=""+I,w.call(this,I)}}),Object.defineProperty(o,i,{enumerable:c.enumerable}),{getValue:function(){return m},setValue:function(I){m=""+I},stopTracking:function(){o._valueTracker=null,delete o[i]}}}}function Yn(o){o._valueTracker||(o._valueTracker=la(o))}function Oa(o){if(!o)return!1;var i=o._valueTracker;if(!i)return!0;var c=i.getValue(),m="";return o&&(m=Vr(o)?o.checked?"true":"false":o.value),o=m,o!==c?(i.setValue(o),!0):!1}function Ht(o){if(o=o||(typeof document<"u"?document:void 0),typeof o>"u")return null;try{return o.activeElement||o.body}catch{return o.body}}var Je=/[\n"\\]/g;function He(o){return o.replace(Je,function(i){return"\\"+i.charCodeAt(0).toString(16)+" "})}function Ft(o,i,c,m,v,w,I,H){o.name="",I!=null&&typeof I!="function"&&typeof I!="symbol"&&typeof I!="boolean"?o.type=I:o.removeAttribute("type"),i!=null?I==="number"?(i===0&&o.value===""||o.value!=i)&&(o.value=""+mn(i)):o.value!==""+mn(i)&&(o.value=""+mn(i)):I!=="submit"&&I!=="reset"||o.removeAttribute("value"),i!=null?Qt(o,I,mn(i)):c!=null?Qt(o,I,mn(c)):m!=null&&o.removeAttribute("value"),v==null&&w!=null&&(o.defaultChecked=!!w),v!=null&&(o.checked=v&&typeof v!="function"&&typeof v!="symbol"),H!=null&&typeof H!="function"&&typeof H!="symbol"&&typeof H!="boolean"?o.name=""+mn(H):o.removeAttribute("name")}function Wt(o,i,c,m,v,w,I,H){if(w!=null&&typeof w!="function"&&typeof w!="symbol"&&typeof w!="boolean"&&(o.type=w),i!=null||c!=null){if(!(w!=="submit"&&w!=="reset"||i!=null))return;c=c!=null?""+mn(c):"",i=i!=null?""+mn(i):c,H||i===o.value||(o.value=i),o.defaultValue=i}m=m??v,m=typeof m!="function"&&typeof m!="symbol"&&!!m,o.checked=H?o.checked:!!m,o.defaultChecked=!!m,I!=null&&typeof I!="function"&&typeof I!="symbol"&&typeof I!="boolean"&&(o.name=I)}function Qt(o,i,c){i==="number"&&Ht(o.ownerDocument)===o||o.defaultValue===""+c||(o.defaultValue=""+c)}function Zt(o,i,c,m){if(o=o.options,i){i={};for(var v=0;v<c.length;v++)i["$"+c[v]]=!0;for(c=0;c<o.length;c++)v=i.hasOwnProperty("$"+o[c].value),o[c].selected!==v&&(o[c].selected=v),v&&m&&(o[c].defaultSelected=!0)}else{for(c=""+mn(c),i=null,v=0;v<o.length;v++){if(o[v].value===c){o[v].selected=!0,m&&(o[v].defaultSelected=!0);return}i!==null||o[v].disabled||(i=o[v])}i!==null&&(i.selected=!0)}}function ir(o,i,c){if(i!=null&&(i=""+mn(i),i!==o.value&&(o.value=i),c==null)){o.defaultValue!==i&&(o.defaultValue=i);return}o.defaultValue=c!=null?""+mn(c):""}function Vn(o,i,c,m){if(i==null){if(m!=null){if(c!=null)throw Error(r(92));if(K(m)){if(1<m.length)throw Error(r(93));m=m[0]}c=m}c==null&&(c=""),i=c}c=mn(i),o.defaultValue=c,m=o.textContent,m===c&&m!==""&&m!==null&&(o.value=m)}function jn(o,i){if(i){var c=o.firstChild;if(c&&c===o.lastChild&&c.nodeType===3){c.nodeValue=i;return}}o.textContent=i}var lr=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function sr(o,i,c){var m=i.indexOf("--")===0;c==null||typeof c=="boolean"||c===""?m?o.setProperty(i,""):i==="float"?o.cssFloat="":o[i]="":m?o.setProperty(i,c):typeof c!="number"||c===0||lr.has(i)?i==="float"?o.cssFloat=c:o[i]=(""+c).trim():o[i]=c+"px"}function _r(o,i,c){if(i!=null&&typeof i!="object")throw Error(r(62));if(o=o.style,c!=null){for(var m in c)!c.hasOwnProperty(m)||i!=null&&i.hasOwnProperty(m)||(m.indexOf("--")===0?o.setProperty(m,""):m==="float"?o.cssFloat="":o[m]="");for(var v in i)m=i[v],i.hasOwnProperty(v)&&c[v]!==m&&sr(o,v,m)}else for(var w in i)i.hasOwnProperty(w)&&sr(o,w,i[w])}function Qn(o){if(o.indexOf("-")===-1)return!1;switch(o){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var $s=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),jr=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function ro(o){return jr.test(""+o)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":o}var qa=null;function sa(o){return o=o.target||o.srcElement||window,o.correspondingUseElement&&(o=o.correspondingUseElement),o.nodeType===3?o.parentNode:o}var Tr=null,ca=null;function xi(o){var i=At(o);if(i&&(o=i.stateNode)){var c=o[Ve]||null;e:switch(o=i.stateNode,i.type){case"input":if(Ft(o,c.value,c.defaultValue,c.defaultValue,c.checked,c.defaultChecked,c.type,c.name),i=c.name,c.type==="radio"&&i!=null){for(c=o;c.parentNode;)c=c.parentNode;for(c=c.querySelectorAll('input[name="'+He(""+i)+'"][type="radio"]'),i=0;i<c.length;i++){var m=c[i];if(m!==o&&m.form===o.form){var v=m[Ve]||null;if(!v)throw Error(r(90));Ft(m,v.value,v.defaultValue,v.defaultValue,v.checked,v.defaultChecked,v.type,v.name)}}for(i=0;i<c.length;i++)m=c[i],m.form===o.form&&Oa(m)}break e;case"textarea":ir(o,c.value,c.defaultValue);break e;case"select":i=c.value,i!=null&&Zt(o,!!c.multiple,i,!1)}}}var wi=!1;function il(o,i,c){if(wi)return o(i,c);wi=!0;try{var m=o(i);return m}finally{if(wi=!1,(Tr!==null||ca!==null)&&(wf(),Tr&&(i=Tr,o=ca,ca=Tr=null,xi(i),o)))for(i=0;i<o.length;i++)xi(o[i])}}function ao(o,i){var c=o.stateNode;if(c===null)return null;var m=c[Ve]||null;if(m===null)return null;c=m[i];e:switch(i){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(m=!m.disabled)||(o=o.type,m=!(o==="button"||o==="input"||o==="select"||o==="textarea")),o=!m;break e;default:o=!1}if(o)return null;if(c&&typeof c!="function")throw Error(r(231,i,typeof c));return c}var Xe=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),mt=!1;if(Xe)try{var rn={};Object.defineProperty(rn,"passive",{get:function(){mt=!0}}),window.addEventListener("test",rn,rn),window.removeEventListener("test",rn,rn)}catch{mt=!1}var gn=null,En=null,oo=null;function ll(){if(oo)return oo;var o,i=En,c=i.length,m,v="value"in gn?gn.value:gn.textContent,w=v.length;for(o=0;o<c&&i[o]===v[o];o++);var I=c-o;for(m=1;m<=I&&i[c-m]===v[w-m];m++);return oo=v.slice(o,1<m?1-m:void 0)}function Ei(o){var i=o.keyCode;return"charCode"in o?(o=o.charCode,o===0&&i===13&&(o=13)):o=i,o===10&&(o=13),32<=o||o===13?o:0}function io(){return!0}function lb(){return!1}function Ir(o){function i(c,m,v,w,I){this._reactName=c,this._targetInst=v,this.type=m,this.nativeEvent=w,this.target=I,this.currentTarget=null;for(var H in o)o.hasOwnProperty(H)&&(c=o[H],this[H]=c?c(w):w[H]);return this.isDefaultPrevented=(w.defaultPrevented!=null?w.defaultPrevented:w.returnValue===!1)?io:lb,this.isPropagationStopped=lb,this}return h(i.prototype,{preventDefault:function(){this.defaultPrevented=!0;var c=this.nativeEvent;c&&(c.preventDefault?c.preventDefault():typeof c.returnValue!="unknown"&&(c.returnValue=!1),this.isDefaultPrevented=io)},stopPropagation:function(){var c=this.nativeEvent;c&&(c.stopPropagation?c.stopPropagation():typeof c.cancelBubble!="unknown"&&(c.cancelBubble=!0),this.isPropagationStopped=io)},persist:function(){},isPersistent:io}),i}var $i={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(o){return o.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},Fu=Ir($i),Os=h({},$i,{view:0,detail:0}),vN=Ir(Os),og,ig,Rs,Vu=h({},Os,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:sg,button:0,buttons:0,relatedTarget:function(o){return o.relatedTarget===void 0?o.fromElement===o.srcElement?o.toElement:o.fromElement:o.relatedTarget},movementX:function(o){return"movementX"in o?o.movementX:(o!==Rs&&(Rs&&o.type==="mousemove"?(og=o.screenX-Rs.screenX,ig=o.screenY-Rs.screenY):ig=og=0,Rs=o),og)},movementY:function(o){return"movementY"in o?o.movementY:ig}}),sb=Ir(Vu),yN=h({},Vu,{dataTransfer:0}),bN=Ir(yN),SN=h({},Os,{relatedTarget:0}),lg=Ir(SN),CN=h({},$i,{animationName:0,elapsedTime:0,pseudoElement:0}),xN=Ir(CN),wN=h({},$i,{clipboardData:function(o){return"clipboardData"in o?o.clipboardData:window.clipboardData}}),EN=Ir(wN),$N=h({},$i,{data:0}),cb=Ir($N),ON={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},RN={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},_N={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function TN(o){var i=this.nativeEvent;return i.getModifierState?i.getModifierState(o):(o=_N[o])?!!i[o]:!1}function sg(){return TN}var MN=h({},Os,{key:function(o){if(o.key){var i=ON[o.key]||o.key;if(i!=="Unidentified")return i}return o.type==="keypress"?(o=Ei(o),o===13?"Enter":String.fromCharCode(o)):o.type==="keydown"||o.type==="keyup"?RN[o.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:sg,charCode:function(o){return o.type==="keypress"?Ei(o):0},keyCode:function(o){return o.type==="keydown"||o.type==="keyup"?o.keyCode:0},which:function(o){return o.type==="keypress"?Ei(o):o.type==="keydown"||o.type==="keyup"?o.keyCode:0}}),NN=Ir(MN),jN=h({},Vu,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),ub=Ir(jN),IN=h({},Os,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:sg}),PN=Ir(IN),AN=h({},$i,{propertyName:0,elapsedTime:0,pseudoElement:0}),zN=Ir(AN),DN=h({},Vu,{deltaX:function(o){return"deltaX"in o?o.deltaX:"wheelDeltaX"in o?-o.wheelDeltaX:0},deltaY:function(o){return"deltaY"in o?o.deltaY:"wheelDeltaY"in o?-o.wheelDeltaY:"wheelDelta"in o?-o.wheelDelta:0},deltaZ:0,deltaMode:0}),LN=Ir(DN),BN=h({},$i,{newState:0,oldState:0}),HN=Ir(BN),FN=[9,13,27,32],cg=Xe&&"CompositionEvent"in window,_s=null;Xe&&"documentMode"in document&&(_s=document.documentMode);var VN=Xe&&"TextEvent"in window&&!_s,fb=Xe&&(!cg||_s&&8<_s&&11>=_s),db=" ",mb=!1;function gb(o,i){switch(o){case"keyup":return FN.indexOf(i.keyCode)!==-1;case"keydown":return i.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function hb(o){return o=o.detail,typeof o=="object"&&"data"in o?o.data:null}var sl=!1;function UN(o,i){switch(o){case"compositionend":return hb(i);case"keypress":return i.which!==32?null:(mb=!0,db);case"textInput":return o=i.data,o===db&&mb?null:o;default:return null}}function kN(o,i){if(sl)return o==="compositionend"||!cg&&gb(o,i)?(o=ll(),oo=En=gn=null,sl=!1,o):null;switch(o){case"paste":return null;case"keypress":if(!(i.ctrlKey||i.altKey||i.metaKey)||i.ctrlKey&&i.altKey){if(i.char&&1<i.char.length)return i.char;if(i.which)return String.fromCharCode(i.which)}return null;case"compositionend":return fb&&i.locale!=="ko"?null:i.data;default:return null}}var qN={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function pb(o){var i=o&&o.nodeName&&o.nodeName.toLowerCase();return i==="input"?!!qN[o.type]:i==="textarea"}function vb(o,i,c,m){Tr?ca?ca.push(m):ca=[m]:Tr=m,i=Tf(i,"onChange"),0<i.length&&(c=new Fu("onChange","change",null,c,m),o.push({event:c,listeners:i}))}var Ts=null,Ms=null;function WN(o){QC(o,0)}function Uu(o){var i=tn(o);if(Oa(i))return o}function yb(o,i){if(o==="change")return i}var bb=!1;if(Xe){var ug;if(Xe){var fg="oninput"in document;if(!fg){var Sb=document.createElement("div");Sb.setAttribute("oninput","return;"),fg=typeof Sb.oninput=="function"}ug=fg}else ug=!1;bb=ug&&(!document.documentMode||9<document.documentMode)}function Cb(){Ts&&(Ts.detachEvent("onpropertychange",xb),Ms=Ts=null)}function xb(o){if(o.propertyName==="value"&&Uu(Ms)){var i=[];vb(i,Ms,o,sa(o)),il(WN,i)}}function GN(o,i,c){o==="focusin"?(Cb(),Ts=i,Ms=c,Ts.attachEvent("onpropertychange",xb)):o==="focusout"&&Cb()}function KN(o){if(o==="selectionchange"||o==="keyup"||o==="keydown")return Uu(Ms)}function XN(o,i){if(o==="click")return Uu(i)}function YN(o,i){if(o==="input"||o==="change")return Uu(i)}function QN(o,i){return o===i&&(o!==0||1/o===1/i)||o!==o&&i!==i}var Ur=typeof Object.is=="function"?Object.is:QN;function Ns(o,i){if(Ur(o,i))return!0;if(typeof o!="object"||o===null||typeof i!="object"||i===null)return!1;var c=Object.keys(o),m=Object.keys(i);if(c.length!==m.length)return!1;for(m=0;m<c.length;m++){var v=c[m];if(!le.call(i,v)||!Ur(o[v],i[v]))return!1}return!0}function wb(o){for(;o&&o.firstChild;)o=o.firstChild;return o}function Eb(o,i){var c=wb(o);o=0;for(var m;c;){if(c.nodeType===3){if(m=o+c.textContent.length,o<=i&&m>=i)return{node:c,offset:i-o};o=m}e:{for(;c;){if(c.nextSibling){c=c.nextSibling;break e}c=c.parentNode}c=void 0}c=wb(c)}}function $b(o,i){return o&&i?o===i?!0:o&&o.nodeType===3?!1:i&&i.nodeType===3?$b(o,i.parentNode):"contains"in o?o.contains(i):o.compareDocumentPosition?!!(o.compareDocumentPosition(i)&16):!1:!1}function Ob(o){o=o!=null&&o.ownerDocument!=null&&o.ownerDocument.defaultView!=null?o.ownerDocument.defaultView:window;for(var i=Ht(o.document);i instanceof o.HTMLIFrameElement;){try{var c=typeof i.contentWindow.location.href=="string"}catch{c=!1}if(c)o=i.contentWindow;else break;i=Ht(o.document)}return i}function dg(o){var i=o&&o.nodeName&&o.nodeName.toLowerCase();return i&&(i==="input"&&(o.type==="text"||o.type==="search"||o.type==="tel"||o.type==="url"||o.type==="password")||i==="textarea"||o.contentEditable==="true")}var ZN=Xe&&"documentMode"in document&&11>=document.documentMode,cl=null,mg=null,js=null,gg=!1;function Rb(o,i,c){var m=c.window===c?c.document:c.nodeType===9?c:c.ownerDocument;gg||cl==null||cl!==Ht(m)||(m=cl,"selectionStart"in m&&dg(m)?m={start:m.selectionStart,end:m.selectionEnd}:(m=(m.ownerDocument&&m.ownerDocument.defaultView||window).getSelection(),m={anchorNode:m.anchorNode,anchorOffset:m.anchorOffset,focusNode:m.focusNode,focusOffset:m.focusOffset}),js&&Ns(js,m)||(js=m,m=Tf(mg,"onSelect"),0<m.length&&(i=new Fu("onSelect","select",null,i,c),o.push({event:i,listeners:m}),i.target=cl)))}function Oi(o,i){var c={};return c[o.toLowerCase()]=i.toLowerCase(),c["Webkit"+o]="webkit"+i,c["Moz"+o]="moz"+i,c}var ul={animationend:Oi("Animation","AnimationEnd"),animationiteration:Oi("Animation","AnimationIteration"),animationstart:Oi("Animation","AnimationStart"),transitionrun:Oi("Transition","TransitionRun"),transitionstart:Oi("Transition","TransitionStart"),transitioncancel:Oi("Transition","TransitionCancel"),transitionend:Oi("Transition","TransitionEnd")},hg={},_b={};Xe&&(_b=document.createElement("div").style,"AnimationEvent"in window||(delete ul.animationend.animation,delete ul.animationiteration.animation,delete ul.animationstart.animation),"TransitionEvent"in window||delete ul.transitionend.transition);function Ri(o){if(hg[o])return hg[o];if(!ul[o])return o;var i=ul[o],c;for(c in i)if(i.hasOwnProperty(c)&&c in _b)return hg[o]=i[c];return o}var Tb=Ri("animationend"),Mb=Ri("animationiteration"),Nb=Ri("animationstart"),JN=Ri("transitionrun"),ej=Ri("transitionstart"),tj=Ri("transitioncancel"),jb=Ri("transitionend"),Ib=new Map,pg="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");pg.push("scrollEnd");function Ra(o,i){Ib.set(o,i),cn(i,[o])}var Pb=new WeakMap;function ua(o,i){if(typeof o=="object"&&o!==null){var c=Pb.get(o);return c!==void 0?c:(i={value:o,source:i,stack:Xn(i)},Pb.set(o,i),i)}return{value:o,source:i,stack:Xn(i)}}var fa=[],fl=0,vg=0;function ku(){for(var o=fl,i=vg=fl=0;i<o;){var c=fa[i];fa[i++]=null;var m=fa[i];fa[i++]=null;var v=fa[i];fa[i++]=null;var w=fa[i];if(fa[i++]=null,m!==null&&v!==null){var I=m.pending;I===null?v.next=v:(v.next=I.next,I.next=v),m.pending=v}w!==0&&Ab(c,v,w)}}function qu(o,i,c,m){fa[fl++]=o,fa[fl++]=i,fa[fl++]=c,fa[fl++]=m,vg|=m,o.lanes|=m,o=o.alternate,o!==null&&(o.lanes|=m)}function yg(o,i,c,m){return qu(o,i,c,m),Wu(o)}function dl(o,i){return qu(o,null,null,i),Wu(o)}function Ab(o,i,c){o.lanes|=c;var m=o.alternate;m!==null&&(m.lanes|=c);for(var v=!1,w=o.return;w!==null;)w.childLanes|=c,m=w.alternate,m!==null&&(m.childLanes|=c),w.tag===22&&(o=w.stateNode,o===null||o._visibility&1||(v=!0)),o=w,w=w.return;return o.tag===3?(w=o.stateNode,v&&i!==null&&(v=31-ke(c),o=w.hiddenUpdates,m=o[v],m===null?o[v]=[i]:m.push(i),i.lane=c|536870912),w):null}function Wu(o){if(50<rc)throw rc=0,Eh=null,Error(r(185));for(var i=o.return;i!==null;)o=i,i=o.return;return o.tag===3?o.stateNode:null}var ml={};function nj(o,i,c,m){this.tag=o,this.key=c,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=i,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=m,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function kr(o,i,c,m){return new nj(o,i,c,m)}function bg(o){return o=o.prototype,!(!o||!o.isReactComponent)}function lo(o,i){var c=o.alternate;return c===null?(c=kr(o.tag,i,o.key,o.mode),c.elementType=o.elementType,c.type=o.type,c.stateNode=o.stateNode,c.alternate=o,o.alternate=c):(c.pendingProps=i,c.type=o.type,c.flags=0,c.subtreeFlags=0,c.deletions=null),c.flags=o.flags&65011712,c.childLanes=o.childLanes,c.lanes=o.lanes,c.child=o.child,c.memoizedProps=o.memoizedProps,c.memoizedState=o.memoizedState,c.updateQueue=o.updateQueue,i=o.dependencies,c.dependencies=i===null?null:{lanes:i.lanes,firstContext:i.firstContext},c.sibling=o.sibling,c.index=o.index,c.ref=o.ref,c.refCleanup=o.refCleanup,c}function zb(o,i){o.flags&=65011714;var c=o.alternate;return c===null?(o.childLanes=0,o.lanes=i,o.child=null,o.subtreeFlags=0,o.memoizedProps=null,o.memoizedState=null,o.updateQueue=null,o.dependencies=null,o.stateNode=null):(o.childLanes=c.childLanes,o.lanes=c.lanes,o.child=c.child,o.subtreeFlags=0,o.deletions=null,o.memoizedProps=c.memoizedProps,o.memoizedState=c.memoizedState,o.updateQueue=c.updateQueue,o.type=c.type,i=c.dependencies,o.dependencies=i===null?null:{lanes:i.lanes,firstContext:i.firstContext}),o}function Gu(o,i,c,m,v,w){var I=0;if(m=o,typeof o=="function")bg(o)&&(I=1);else if(typeof o=="string")I=aI(o,c,k.current)?26:o==="html"||o==="head"||o==="body"?27:5;else e:switch(o){case P:return o=kr(31,c,i,v),o.elementType=P,o.lanes=w,o;case b:return _i(c.children,v,w,i);case x:I=8,v|=24;break;case S:return o=kr(12,c,i,v|2),o.elementType=S,o.lanes=w,o;case R:return o=kr(13,c,i,v),o.elementType=R,o.lanes=w,o;case T:return o=kr(19,c,i,v),o.elementType=T,o.lanes=w,o;default:if(typeof o=="object"&&o!==null)switch(o.$$typeof){case E:case O:I=10;break e;case $:I=9;break e;case _:I=11;break e;case M:I=14;break e;case z:I=16,m=null;break e}I=29,c=Error(r(130,o===null?"null":typeof o,"")),m=null}return i=kr(I,c,i,v),i.elementType=o,i.type=m,i.lanes=w,i}function _i(o,i,c,m){return o=kr(7,o,m,i),o.lanes=c,o}function Sg(o,i,c){return o=kr(6,o,null,i),o.lanes=c,o}function Cg(o,i,c){return i=kr(4,o.children!==null?o.children:[],o.key,i),i.lanes=c,i.stateNode={containerInfo:o.containerInfo,pendingChildren:null,implementation:o.implementation},i}var gl=[],hl=0,Ku=null,Xu=0,da=[],ma=0,Ti=null,so=1,co="";function Mi(o,i){gl[hl++]=Xu,gl[hl++]=Ku,Ku=o,Xu=i}function Db(o,i,c){da[ma++]=so,da[ma++]=co,da[ma++]=Ti,Ti=o;var m=so;o=co;var v=32-ke(m)-1;m&=~(1<<v),c+=1;var w=32-ke(i)+v;if(30<w){var I=v-v%5;w=(m&(1<<I)-1).toString(32),m>>=I,v-=I,so=1<<32-ke(i)+v|c<<v|m,co=w+o}else so=1<<w|c<<v|m,co=o}function xg(o){o.return!==null&&(Mi(o,1),Db(o,1,0))}function wg(o){for(;o===Ku;)Ku=gl[--hl],gl[hl]=null,Xu=gl[--hl],gl[hl]=null;for(;o===Ti;)Ti=da[--ma],da[ma]=null,co=da[--ma],da[ma]=null,so=da[--ma],da[ma]=null}var Mr=null,In=null,nn=!1,Ni=null,Wa=!1,Eg=Error(r(519));function ji(o){var i=Error(r(418,""));throw As(ua(i,o)),Eg}function Lb(o){var i=o.stateNode,c=o.type,m=o.memoizedProps;switch(i[Ct]=o,i[Ve]=m,c){case"dialog":Bt("cancel",i),Bt("close",i);break;case"iframe":case"object":case"embed":Bt("load",i);break;case"video":case"audio":for(c=0;c<oc.length;c++)Bt(oc[c],i);break;case"source":Bt("error",i);break;case"img":case"image":case"link":Bt("error",i),Bt("load",i);break;case"details":Bt("toggle",i);break;case"input":Bt("invalid",i),Wt(i,m.value,m.defaultValue,m.checked,m.defaultChecked,m.type,m.name,!0),Yn(i);break;case"select":Bt("invalid",i);break;case"textarea":Bt("invalid",i),Vn(i,m.value,m.defaultValue,m.children),Yn(i)}c=m.children,typeof c!="string"&&typeof c!="number"&&typeof c!="bigint"||i.textContent===""+c||m.suppressHydrationWarning===!0||t1(i.textContent,c)?(m.popover!=null&&(Bt("beforetoggle",i),Bt("toggle",i)),m.onScroll!=null&&Bt("scroll",i),m.onScrollEnd!=null&&Bt("scrollend",i),m.onClick!=null&&(i.onclick=Mf),i=!0):i=!1,i||ji(o)}function Bb(o){for(Mr=o.return;Mr;)switch(Mr.tag){case 5:case 13:Wa=!1;return;case 27:case 3:Wa=!0;return;default:Mr=Mr.return}}function Is(o){if(o!==Mr)return!1;if(!nn)return Bb(o),nn=!0,!1;var i=o.tag,c;if((c=i!==3&&i!==27)&&((c=i===5)&&(c=o.type,c=!(c!=="form"&&c!=="button")||Hh(o.type,o.memoizedProps)),c=!c),c&&In&&ji(o),Bb(o),i===13){if(o=o.memoizedState,o=o!==null?o.dehydrated:null,!o)throw Error(r(317));e:{for(o=o.nextSibling,i=0;o;){if(o.nodeType===8)if(c=o.data,c==="/$"){if(i===0){In=Ta(o.nextSibling);break e}i--}else c!=="$"&&c!=="$!"&&c!=="$?"||i++;o=o.nextSibling}In=null}}else i===27?(i=In,ri(o.type)?(o=kh,kh=null,In=o):In=i):In=Mr?Ta(o.stateNode.nextSibling):null;return!0}function Ps(){In=Mr=null,nn=!1}function Hb(){var o=Ni;return o!==null&&(zr===null?zr=o:zr.push.apply(zr,o),Ni=null),o}function As(o){Ni===null?Ni=[o]:Ni.push(o)}var $g=B(null),Ii=null,uo=null;function Vo(o,i,c){X($g,i._currentValue),i._currentValue=c}function fo(o){o._currentValue=$g.current,W($g)}function Og(o,i,c){for(;o!==null;){var m=o.alternate;if((o.childLanes&i)!==i?(o.childLanes|=i,m!==null&&(m.childLanes|=i)):m!==null&&(m.childLanes&i)!==i&&(m.childLanes|=i),o===c)break;o=o.return}}function Rg(o,i,c,m){var v=o.child;for(v!==null&&(v.return=o);v!==null;){var w=v.dependencies;if(w!==null){var I=v.child;w=w.firstContext;e:for(;w!==null;){var H=w;w=v;for(var J=0;J<i.length;J++)if(H.context===i[J]){w.lanes|=c,H=w.alternate,H!==null&&(H.lanes|=c),Og(w.return,c,o),m||(I=null);break e}w=H.next}}else if(v.tag===18){if(I=v.return,I===null)throw Error(r(341));I.lanes|=c,w=I.alternate,w!==null&&(w.lanes|=c),Og(I,c,o),I=null}else I=v.child;if(I!==null)I.return=v;else for(I=v;I!==null;){if(I===o){I=null;break}if(v=I.sibling,v!==null){v.return=I.return,I=v;break}I=I.return}v=I}}function zs(o,i,c,m){o=null;for(var v=i,w=!1;v!==null;){if(!w){if((v.flags&524288)!==0)w=!0;else if((v.flags&262144)!==0)break}if(v.tag===10){var I=v.alternate;if(I===null)throw Error(r(387));if(I=I.memoizedProps,I!==null){var H=v.type;Ur(v.pendingProps.value,I.value)||(o!==null?o.push(H):o=[H])}}else if(v===ue.current){if(I=v.alternate,I===null)throw Error(r(387));I.memoizedState.memoizedState!==v.memoizedState.memoizedState&&(o!==null?o.push(fc):o=[fc])}v=v.return}o!==null&&Rg(i,o,c,m),i.flags|=262144}function Yu(o){for(o=o.firstContext;o!==null;){if(!Ur(o.context._currentValue,o.memoizedValue))return!0;o=o.next}return!1}function Pi(o){Ii=o,uo=null,o=o.dependencies,o!==null&&(o.firstContext=null)}function Cr(o){return Fb(Ii,o)}function Qu(o,i){return Ii===null&&Pi(o),Fb(o,i)}function Fb(o,i){var c=i._currentValue;if(i={context:i,memoizedValue:c,next:null},uo===null){if(o===null)throw Error(r(308));uo=i,o.dependencies={lanes:0,firstContext:i},o.flags|=524288}else uo=uo.next=i;return c}var rj=typeof AbortController<"u"?AbortController:function(){var o=[],i=this.signal={aborted:!1,addEventListener:function(c,m){o.push(m)}};this.abort=function(){i.aborted=!0,o.forEach(function(c){return c()})}},aj=e.unstable_scheduleCallback,oj=e.unstable_NormalPriority,Zn={$$typeof:O,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function _g(){return{controller:new rj,data:new Map,refCount:0}}function Ds(o){o.refCount--,o.refCount===0&&aj(oj,function(){o.controller.abort()})}var Ls=null,Tg=0,pl=0,vl=null;function ij(o,i){if(Ls===null){var c=Ls=[];Tg=0,pl=Nh(),vl={status:"pending",value:void 0,then:function(m){c.push(m)}}}return Tg++,i.then(Vb,Vb),i}function Vb(){if(--Tg===0&&Ls!==null){vl!==null&&(vl.status="fulfilled");var o=Ls;Ls=null,pl=0,vl=null;for(var i=0;i<o.length;i++)(0,o[i])()}}function lj(o,i){var c=[],m={status:"pending",value:null,reason:null,then:function(v){c.push(v)}};return o.then(function(){m.status="fulfilled",m.value=i;for(var v=0;v<c.length;v++)(0,c[v])(i)},function(v){for(m.status="rejected",m.reason=v,v=0;v<c.length;v++)(0,c[v])(void 0)}),m}var Ub=j.S;j.S=function(o,i){typeof i=="object"&&i!==null&&typeof i.then=="function"&&ij(o,i),Ub!==null&&Ub(o,i)};var Ai=B(null);function Mg(){var o=Ai.current;return o!==null?o:$n.pooledCache}function Zu(o,i){i===null?X(Ai,Ai.current):X(Ai,i.pool)}function kb(){var o=Mg();return o===null?null:{parent:Zn._currentValue,pool:o}}var Bs=Error(r(460)),qb=Error(r(474)),Ju=Error(r(542)),Ng={then:function(){}};function Wb(o){return o=o.status,o==="fulfilled"||o==="rejected"}function ef(){}function Gb(o,i,c){switch(c=o[c],c===void 0?o.push(i):c!==i&&(i.then(ef,ef),i=c),i.status){case"fulfilled":return i.value;case"rejected":throw o=i.reason,Xb(o),o;default:if(typeof i.status=="string")i.then(ef,ef);else{if(o=$n,o!==null&&100<o.shellSuspendCounter)throw Error(r(482));o=i,o.status="pending",o.then(function(m){if(i.status==="pending"){var v=i;v.status="fulfilled",v.value=m}},function(m){if(i.status==="pending"){var v=i;v.status="rejected",v.reason=m}})}switch(i.status){case"fulfilled":return i.value;case"rejected":throw o=i.reason,Xb(o),o}throw Hs=i,Bs}}var Hs=null;function Kb(){if(Hs===null)throw Error(r(459));var o=Hs;return Hs=null,o}function Xb(o){if(o===Bs||o===Ju)throw Error(r(483))}var Uo=!1;function jg(o){o.updateQueue={baseState:o.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function Ig(o,i){o=o.updateQueue,i.updateQueue===o&&(i.updateQueue={baseState:o.baseState,firstBaseUpdate:o.firstBaseUpdate,lastBaseUpdate:o.lastBaseUpdate,shared:o.shared,callbacks:null})}function ko(o){return{lane:o,tag:0,payload:null,callback:null,next:null}}function qo(o,i,c){var m=o.updateQueue;if(m===null)return null;if(m=m.shared,(un&2)!==0){var v=m.pending;return v===null?i.next=i:(i.next=v.next,v.next=i),m.pending=i,i=Wu(o),Ab(o,null,c),i}return qu(o,m,i,c),Wu(o)}function Fs(o,i,c){if(i=i.updateQueue,i!==null&&(i=i.shared,(c&4194048)!==0)){var m=i.lanes;m&=o.pendingLanes,c|=m,i.lanes=c,ln(o,c)}}function Pg(o,i){var c=o.updateQueue,m=o.alternate;if(m!==null&&(m=m.updateQueue,c===m)){var v=null,w=null;if(c=c.firstBaseUpdate,c!==null){do{var I={lane:c.lane,tag:c.tag,payload:c.payload,callback:null,next:null};w===null?v=w=I:w=w.next=I,c=c.next}while(c!==null);w===null?v=w=i:w=w.next=i}else v=w=i;c={baseState:m.baseState,firstBaseUpdate:v,lastBaseUpdate:w,shared:m.shared,callbacks:m.callbacks},o.updateQueue=c;return}o=c.lastBaseUpdate,o===null?c.firstBaseUpdate=i:o.next=i,c.lastBaseUpdate=i}var Ag=!1;function Vs(){if(Ag){var o=vl;if(o!==null)throw o}}function Us(o,i,c,m){Ag=!1;var v=o.updateQueue;Uo=!1;var w=v.firstBaseUpdate,I=v.lastBaseUpdate,H=v.shared.pending;if(H!==null){v.shared.pending=null;var J=H,me=J.next;J.next=null,I===null?w=me:I.next=me,I=J;var we=o.alternate;we!==null&&(we=we.updateQueue,H=we.lastBaseUpdate,H!==I&&(H===null?we.firstBaseUpdate=me:H.next=me,we.lastBaseUpdate=J))}if(w!==null){var _e=v.baseState;I=0,we=me=J=null,H=w;do{var pe=H.lane&-536870913,ye=pe!==H.lane;if(ye?(Xt&pe)===pe:(m&pe)===pe){pe!==0&&pe===pl&&(Ag=!0),we!==null&&(we=we.next={lane:0,tag:H.tag,payload:H.payload,callback:null,next:null});e:{var St=o,gt=H;pe=i;var vn=c;switch(gt.tag){case 1:if(St=gt.payload,typeof St=="function"){_e=St.call(vn,_e,pe);break e}_e=St;break e;case 3:St.flags=St.flags&-65537|128;case 0:if(St=gt.payload,pe=typeof St=="function"?St.call(vn,_e,pe):St,pe==null)break e;_e=h({},_e,pe);break e;case 2:Uo=!0}}pe=H.callback,pe!==null&&(o.flags|=64,ye&&(o.flags|=8192),ye=v.callbacks,ye===null?v.callbacks=[pe]:ye.push(pe))}else ye={lane:pe,tag:H.tag,payload:H.payload,callback:H.callback,next:null},we===null?(me=we=ye,J=_e):we=we.next=ye,I|=pe;if(H=H.next,H===null){if(H=v.shared.pending,H===null)break;ye=H,H=ye.next,ye.next=null,v.lastBaseUpdate=ye,v.shared.pending=null}}while(!0);we===null&&(J=_e),v.baseState=J,v.firstBaseUpdate=me,v.lastBaseUpdate=we,w===null&&(v.shared.lanes=0),Jo|=I,o.lanes=I,o.memoizedState=_e}}function Yb(o,i){if(typeof o!="function")throw Error(r(191,o));o.call(i)}function Qb(o,i){var c=o.callbacks;if(c!==null)for(o.callbacks=null,o=0;o<c.length;o++)Yb(c[o],i)}var yl=B(null),tf=B(0);function Zb(o,i){o=bo,X(tf,o),X(yl,i),bo=o|i.baseLanes}function zg(){X(tf,bo),X(yl,yl.current)}function Dg(){bo=tf.current,W(yl),W(tf)}var Wo=0,Nt=null,hn=null,Un=null,nf=!1,bl=!1,zi=!1,rf=0,ks=0,Sl=null,sj=0;function Dn(){throw Error(r(321))}function Lg(o,i){if(i===null)return!1;for(var c=0;c<i.length&&c<o.length;c++)if(!Ur(o[c],i[c]))return!1;return!0}function Bg(o,i,c,m,v,w){return Wo=w,Nt=i,i.memoizedState=null,i.updateQueue=null,i.lanes=0,j.H=o===null||o.memoizedState===null?AS:zS,zi=!1,w=c(m,v),zi=!1,bl&&(w=eS(i,c,m,v)),Jb(o),w}function Jb(o){j.H=uf;var i=hn!==null&&hn.next!==null;if(Wo=0,Un=hn=Nt=null,nf=!1,ks=0,Sl=null,i)throw Error(r(300));o===null||cr||(o=o.dependencies,o!==null&&Yu(o)&&(cr=!0))}function eS(o,i,c,m){Nt=o;var v=0;do{if(bl&&(Sl=null),ks=0,bl=!1,25<=v)throw Error(r(301));if(v+=1,Un=hn=null,o.updateQueue!=null){var w=o.updateQueue;w.lastEffect=null,w.events=null,w.stores=null,w.memoCache!=null&&(w.memoCache.index=0)}j.H=hj,w=i(c,m)}while(bl);return w}function cj(){var o=j.H,i=o.useState()[0];return i=typeof i.then=="function"?qs(i):i,o=o.useState()[0],(hn!==null?hn.memoizedState:null)!==o&&(Nt.flags|=1024),i}function Hg(){var o=rf!==0;return rf=0,o}function Fg(o,i,c){i.updateQueue=o.updateQueue,i.flags&=-2053,o.lanes&=~c}function Vg(o){if(nf){for(o=o.memoizedState;o!==null;){var i=o.queue;i!==null&&(i.pending=null),o=o.next}nf=!1}Wo=0,Un=hn=Nt=null,bl=!1,ks=rf=0,Sl=null}function Pr(){var o={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Un===null?Nt.memoizedState=Un=o:Un=Un.next=o,Un}function kn(){if(hn===null){var o=Nt.alternate;o=o!==null?o.memoizedState:null}else o=hn.next;var i=Un===null?Nt.memoizedState:Un.next;if(i!==null)Un=i,hn=o;else{if(o===null)throw Nt.alternate===null?Error(r(467)):Error(r(310));hn=o,o={memoizedState:hn.memoizedState,baseState:hn.baseState,baseQueue:hn.baseQueue,queue:hn.queue,next:null},Un===null?Nt.memoizedState=Un=o:Un=Un.next=o}return Un}function Ug(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function qs(o){var i=ks;return ks+=1,Sl===null&&(Sl=[]),o=Gb(Sl,o,i),i=Nt,(Un===null?i.memoizedState:Un.next)===null&&(i=i.alternate,j.H=i===null||i.memoizedState===null?AS:zS),o}function af(o){if(o!==null&&typeof o=="object"){if(typeof o.then=="function")return qs(o);if(o.$$typeof===O)return Cr(o)}throw Error(r(438,String(o)))}function kg(o){var i=null,c=Nt.updateQueue;if(c!==null&&(i=c.memoCache),i==null){var m=Nt.alternate;m!==null&&(m=m.updateQueue,m!==null&&(m=m.memoCache,m!=null&&(i={data:m.data.map(function(v){return v.slice()}),index:0})))}if(i==null&&(i={data:[],index:0}),c===null&&(c=Ug(),Nt.updateQueue=c),c.memoCache=i,c=i.data[i.index],c===void 0)for(c=i.data[i.index]=Array(o),m=0;m<o;m++)c[m]=D;return i.index++,c}function mo(o,i){return typeof i=="function"?i(o):i}function of(o){var i=kn();return qg(i,hn,o)}function qg(o,i,c){var m=o.queue;if(m===null)throw Error(r(311));m.lastRenderedReducer=c;var v=o.baseQueue,w=m.pending;if(w!==null){if(v!==null){var I=v.next;v.next=w.next,w.next=I}i.baseQueue=v=w,m.pending=null}if(w=o.baseState,v===null)o.memoizedState=w;else{i=v.next;var H=I=null,J=null,me=i,we=!1;do{var _e=me.lane&-536870913;if(_e!==me.lane?(Xt&_e)===_e:(Wo&_e)===_e){var pe=me.revertLane;if(pe===0)J!==null&&(J=J.next={lane:0,revertLane:0,action:me.action,hasEagerState:me.hasEagerState,eagerState:me.eagerState,next:null}),_e===pl&&(we=!0);else if((Wo&pe)===pe){me=me.next,pe===pl&&(we=!0);continue}else _e={lane:0,revertLane:me.revertLane,action:me.action,hasEagerState:me.hasEagerState,eagerState:me.eagerState,next:null},J===null?(H=J=_e,I=w):J=J.next=_e,Nt.lanes|=pe,Jo|=pe;_e=me.action,zi&&c(w,_e),w=me.hasEagerState?me.eagerState:c(w,_e)}else pe={lane:_e,revertLane:me.revertLane,action:me.action,hasEagerState:me.hasEagerState,eagerState:me.eagerState,next:null},J===null?(H=J=pe,I=w):J=J.next=pe,Nt.lanes|=_e,Jo|=_e;me=me.next}while(me!==null&&me!==i);if(J===null?I=w:J.next=H,!Ur(w,o.memoizedState)&&(cr=!0,we&&(c=vl,c!==null)))throw c;o.memoizedState=w,o.baseState=I,o.baseQueue=J,m.lastRenderedState=w}return v===null&&(m.lanes=0),[o.memoizedState,m.dispatch]}function Wg(o){var i=kn(),c=i.queue;if(c===null)throw Error(r(311));c.lastRenderedReducer=o;var m=c.dispatch,v=c.pending,w=i.memoizedState;if(v!==null){c.pending=null;var I=v=v.next;do w=o(w,I.action),I=I.next;while(I!==v);Ur(w,i.memoizedState)||(cr=!0),i.memoizedState=w,i.baseQueue===null&&(i.baseState=w),c.lastRenderedState=w}return[w,m]}function tS(o,i,c){var m=Nt,v=kn(),w=nn;if(w){if(c===void 0)throw Error(r(407));c=c()}else c=i();var I=!Ur((hn||v).memoizedState,c);I&&(v.memoizedState=c,cr=!0),v=v.queue;var H=aS.bind(null,m,v,o);if(Ws(2048,8,H,[o]),v.getSnapshot!==i||I||Un!==null&&Un.memoizedState.tag&1){if(m.flags|=2048,Cl(9,lf(),rS.bind(null,m,v,c,i),null),$n===null)throw Error(r(349));w||(Wo&124)!==0||nS(m,i,c)}return c}function nS(o,i,c){o.flags|=16384,o={getSnapshot:i,value:c},i=Nt.updateQueue,i===null?(i=Ug(),Nt.updateQueue=i,i.stores=[o]):(c=i.stores,c===null?i.stores=[o]:c.push(o))}function rS(o,i,c,m){i.value=c,i.getSnapshot=m,oS(i)&&iS(o)}function aS(o,i,c){return c(function(){oS(i)&&iS(o)})}function oS(o){var i=o.getSnapshot;o=o.value;try{var c=i();return!Ur(o,c)}catch{return!0}}function iS(o){var i=dl(o,2);i!==null&&Xr(i,o,2)}function Gg(o){var i=Pr();if(typeof o=="function"){var c=o;if(o=c(),zi){at(!0);try{c()}finally{at(!1)}}}return i.memoizedState=i.baseState=o,i.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:mo,lastRenderedState:o},i}function lS(o,i,c,m){return o.baseState=c,qg(o,hn,typeof m=="function"?m:mo)}function uj(o,i,c,m,v){if(cf(o))throw Error(r(485));if(o=i.action,o!==null){var w={payload:v,action:o,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(I){w.listeners.push(I)}};j.T!==null?c(!0):w.isTransition=!1,m(w),c=i.pending,c===null?(w.next=i.pending=w,sS(i,w)):(w.next=c.next,i.pending=c.next=w)}}function sS(o,i){var c=i.action,m=i.payload,v=o.state;if(i.isTransition){var w=j.T,I={};j.T=I;try{var H=c(v,m),J=j.S;J!==null&&J(I,H),cS(o,i,H)}catch(me){Kg(o,i,me)}finally{j.T=w}}else try{w=c(v,m),cS(o,i,w)}catch(me){Kg(o,i,me)}}function cS(o,i,c){c!==null&&typeof c=="object"&&typeof c.then=="function"?c.then(function(m){uS(o,i,m)},function(m){return Kg(o,i,m)}):uS(o,i,c)}function uS(o,i,c){i.status="fulfilled",i.value=c,fS(i),o.state=c,i=o.pending,i!==null&&(c=i.next,c===i?o.pending=null:(c=c.next,i.next=c,sS(o,c)))}function Kg(o,i,c){var m=o.pending;if(o.pending=null,m!==null){m=m.next;do i.status="rejected",i.reason=c,fS(i),i=i.next;while(i!==m)}o.action=null}function fS(o){o=o.listeners;for(var i=0;i<o.length;i++)(0,o[i])()}function dS(o,i){return i}function mS(o,i){if(nn){var c=$n.formState;if(c!==null){e:{var m=Nt;if(nn){if(In){t:{for(var v=In,w=Wa;v.nodeType!==8;){if(!w){v=null;break t}if(v=Ta(v.nextSibling),v===null){v=null;break t}}w=v.data,v=w==="F!"||w==="F"?v:null}if(v){In=Ta(v.nextSibling),m=v.data==="F!";break e}}ji(m)}m=!1}m&&(i=c[0])}}return c=Pr(),c.memoizedState=c.baseState=i,m={pending:null,lanes:0,dispatch:null,lastRenderedReducer:dS,lastRenderedState:i},c.queue=m,c=jS.bind(null,Nt,m),m.dispatch=c,m=Gg(!1),w=Jg.bind(null,Nt,!1,m.queue),m=Pr(),v={state:i,dispatch:null,action:o,pending:null},m.queue=v,c=uj.bind(null,Nt,v,w,c),v.dispatch=c,m.memoizedState=o,[i,c,!1]}function gS(o){var i=kn();return hS(i,hn,o)}function hS(o,i,c){if(i=qg(o,i,dS)[0],o=of(mo)[0],typeof i=="object"&&i!==null&&typeof i.then=="function")try{var m=qs(i)}catch(I){throw I===Bs?Ju:I}else m=i;i=kn();var v=i.queue,w=v.dispatch;return c!==i.memoizedState&&(Nt.flags|=2048,Cl(9,lf(),fj.bind(null,v,c),null)),[m,w,o]}function fj(o,i){o.action=i}function pS(o){var i=kn(),c=hn;if(c!==null)return hS(i,c,o);kn(),i=i.memoizedState,c=kn();var m=c.queue.dispatch;return c.memoizedState=o,[i,m,!1]}function Cl(o,i,c,m){return o={tag:o,create:c,deps:m,inst:i,next:null},i=Nt.updateQueue,i===null&&(i=Ug(),Nt.updateQueue=i),c=i.lastEffect,c===null?i.lastEffect=o.next=o:(m=c.next,c.next=o,o.next=m,i.lastEffect=o),o}function lf(){return{destroy:void 0,resource:void 0}}function vS(){return kn().memoizedState}function sf(o,i,c,m){var v=Pr();m=m===void 0?null:m,Nt.flags|=o,v.memoizedState=Cl(1|i,lf(),c,m)}function Ws(o,i,c,m){var v=kn();m=m===void 0?null:m;var w=v.memoizedState.inst;hn!==null&&m!==null&&Lg(m,hn.memoizedState.deps)?v.memoizedState=Cl(i,w,c,m):(Nt.flags|=o,v.memoizedState=Cl(1|i,w,c,m))}function yS(o,i){sf(8390656,8,o,i)}function bS(o,i){Ws(2048,8,o,i)}function SS(o,i){return Ws(4,2,o,i)}function CS(o,i){return Ws(4,4,o,i)}function xS(o,i){if(typeof i=="function"){o=o();var c=i(o);return function(){typeof c=="function"?c():i(null)}}if(i!=null)return o=o(),i.current=o,function(){i.current=null}}function wS(o,i,c){c=c!=null?c.concat([o]):null,Ws(4,4,xS.bind(null,i,o),c)}function Xg(){}function ES(o,i){var c=kn();i=i===void 0?null:i;var m=c.memoizedState;return i!==null&&Lg(i,m[1])?m[0]:(c.memoizedState=[o,i],o)}function $S(o,i){var c=kn();i=i===void 0?null:i;var m=c.memoizedState;if(i!==null&&Lg(i,m[1]))return m[0];if(m=o(),zi){at(!0);try{o()}finally{at(!1)}}return c.memoizedState=[m,i],m}function Yg(o,i,c){return c===void 0||(Wo&1073741824)!==0?o.memoizedState=i:(o.memoizedState=c,o=_C(),Nt.lanes|=o,Jo|=o,c)}function OS(o,i,c,m){return Ur(c,i)?c:yl.current!==null?(o=Yg(o,c,m),Ur(o,i)||(cr=!0),o):(Wo&42)===0?(cr=!0,o.memoizedState=c):(o=_C(),Nt.lanes|=o,Jo|=o,i)}function RS(o,i,c,m,v){var w=A.p;A.p=w!==0&&8>w?w:8;var I=j.T,H={};j.T=H,Jg(o,!1,i,c);try{var J=v(),me=j.S;if(me!==null&&me(H,J),J!==null&&typeof J=="object"&&typeof J.then=="function"){var we=lj(J,m);Gs(o,i,we,Kr(o))}else Gs(o,i,m,Kr(o))}catch(_e){Gs(o,i,{then:function(){},status:"rejected",reason:_e},Kr())}finally{A.p=w,j.T=I}}function dj(){}function Qg(o,i,c,m){if(o.tag!==5)throw Error(r(476));var v=_S(o).queue;RS(o,v,i,L,c===null?dj:function(){return TS(o),c(m)})}function _S(o){var i=o.memoizedState;if(i!==null)return i;i={memoizedState:L,baseState:L,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:mo,lastRenderedState:L},next:null};var c={};return i.next={memoizedState:c,baseState:c,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:mo,lastRenderedState:c},next:null},o.memoizedState=i,o=o.alternate,o!==null&&(o.memoizedState=i),i}function TS(o){var i=_S(o).next.queue;Gs(o,i,{},Kr())}function Zg(){return Cr(fc)}function MS(){return kn().memoizedState}function NS(){return kn().memoizedState}function mj(o){for(var i=o.return;i!==null;){switch(i.tag){case 24:case 3:var c=Kr();o=ko(c);var m=qo(i,o,c);m!==null&&(Xr(m,i,c),Fs(m,i,c)),i={cache:_g()},o.payload=i;return}i=i.return}}function gj(o,i,c){var m=Kr();c={lane:m,revertLane:0,action:c,hasEagerState:!1,eagerState:null,next:null},cf(o)?IS(i,c):(c=yg(o,i,c,m),c!==null&&(Xr(c,o,m),PS(c,i,m)))}function jS(o,i,c){var m=Kr();Gs(o,i,c,m)}function Gs(o,i,c,m){var v={lane:m,revertLane:0,action:c,hasEagerState:!1,eagerState:null,next:null};if(cf(o))IS(i,v);else{var w=o.alternate;if(o.lanes===0&&(w===null||w.lanes===0)&&(w=i.lastRenderedReducer,w!==null))try{var I=i.lastRenderedState,H=w(I,c);if(v.hasEagerState=!0,v.eagerState=H,Ur(H,I))return qu(o,i,v,0),$n===null&&ku(),!1}catch{}finally{}if(c=yg(o,i,v,m),c!==null)return Xr(c,o,m),PS(c,i,m),!0}return!1}function Jg(o,i,c,m){if(m={lane:2,revertLane:Nh(),action:m,hasEagerState:!1,eagerState:null,next:null},cf(o)){if(i)throw Error(r(479))}else i=yg(o,c,m,2),i!==null&&Xr(i,o,2)}function cf(o){var i=o.alternate;return o===Nt||i!==null&&i===Nt}function IS(o,i){bl=nf=!0;var c=o.pending;c===null?i.next=i:(i.next=c.next,c.next=i),o.pending=i}function PS(o,i,c){if((c&4194048)!==0){var m=i.lanes;m&=o.pendingLanes,c|=m,i.lanes=c,ln(o,c)}}var uf={readContext:Cr,use:af,useCallback:Dn,useContext:Dn,useEffect:Dn,useImperativeHandle:Dn,useLayoutEffect:Dn,useInsertionEffect:Dn,useMemo:Dn,useReducer:Dn,useRef:Dn,useState:Dn,useDebugValue:Dn,useDeferredValue:Dn,useTransition:Dn,useSyncExternalStore:Dn,useId:Dn,useHostTransitionStatus:Dn,useFormState:Dn,useActionState:Dn,useOptimistic:Dn,useMemoCache:Dn,useCacheRefresh:Dn},AS={readContext:Cr,use:af,useCallback:function(o,i){return Pr().memoizedState=[o,i===void 0?null:i],o},useContext:Cr,useEffect:yS,useImperativeHandle:function(o,i,c){c=c!=null?c.concat([o]):null,sf(4194308,4,xS.bind(null,i,o),c)},useLayoutEffect:function(o,i){return sf(4194308,4,o,i)},useInsertionEffect:function(o,i){sf(4,2,o,i)},useMemo:function(o,i){var c=Pr();i=i===void 0?null:i;var m=o();if(zi){at(!0);try{o()}finally{at(!1)}}return c.memoizedState=[m,i],m},useReducer:function(o,i,c){var m=Pr();if(c!==void 0){var v=c(i);if(zi){at(!0);try{c(i)}finally{at(!1)}}}else v=i;return m.memoizedState=m.baseState=v,o={pending:null,lanes:0,dispatch:null,lastRenderedReducer:o,lastRenderedState:v},m.queue=o,o=o.dispatch=gj.bind(null,Nt,o),[m.memoizedState,o]},useRef:function(o){var i=Pr();return o={current:o},i.memoizedState=o},useState:function(o){o=Gg(o);var i=o.queue,c=jS.bind(null,Nt,i);return i.dispatch=c,[o.memoizedState,c]},useDebugValue:Xg,useDeferredValue:function(o,i){var c=Pr();return Yg(c,o,i)},useTransition:function(){var o=Gg(!1);return o=RS.bind(null,Nt,o.queue,!0,!1),Pr().memoizedState=o,[!1,o]},useSyncExternalStore:function(o,i,c){var m=Nt,v=Pr();if(nn){if(c===void 0)throw Error(r(407));c=c()}else{if(c=i(),$n===null)throw Error(r(349));(Xt&124)!==0||nS(m,i,c)}v.memoizedState=c;var w={value:c,getSnapshot:i};return v.queue=w,yS(aS.bind(null,m,w,o),[o]),m.flags|=2048,Cl(9,lf(),rS.bind(null,m,w,c,i),null),c},useId:function(){var o=Pr(),i=$n.identifierPrefix;if(nn){var c=co,m=so;c=(m&~(1<<32-ke(m)-1)).toString(32)+c,i="«"+i+"R"+c,c=rf++,0<c&&(i+="H"+c.toString(32)),i+="»"}else c=sj++,i="«"+i+"r"+c.toString(32)+"»";return o.memoizedState=i},useHostTransitionStatus:Zg,useFormState:mS,useActionState:mS,useOptimistic:function(o){var i=Pr();i.memoizedState=i.baseState=o;var c={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return i.queue=c,i=Jg.bind(null,Nt,!0,c),c.dispatch=i,[o,i]},useMemoCache:kg,useCacheRefresh:function(){return Pr().memoizedState=mj.bind(null,Nt)}},zS={readContext:Cr,use:af,useCallback:ES,useContext:Cr,useEffect:bS,useImperativeHandle:wS,useInsertionEffect:SS,useLayoutEffect:CS,useMemo:$S,useReducer:of,useRef:vS,useState:function(){return of(mo)},useDebugValue:Xg,useDeferredValue:function(o,i){var c=kn();return OS(c,hn.memoizedState,o,i)},useTransition:function(){var o=of(mo)[0],i=kn().memoizedState;return[typeof o=="boolean"?o:qs(o),i]},useSyncExternalStore:tS,useId:MS,useHostTransitionStatus:Zg,useFormState:gS,useActionState:gS,useOptimistic:function(o,i){var c=kn();return lS(c,hn,o,i)},useMemoCache:kg,useCacheRefresh:NS},hj={readContext:Cr,use:af,useCallback:ES,useContext:Cr,useEffect:bS,useImperativeHandle:wS,useInsertionEffect:SS,useLayoutEffect:CS,useMemo:$S,useReducer:Wg,useRef:vS,useState:function(){return Wg(mo)},useDebugValue:Xg,useDeferredValue:function(o,i){var c=kn();return hn===null?Yg(c,o,i):OS(c,hn.memoizedState,o,i)},useTransition:function(){var o=Wg(mo)[0],i=kn().memoizedState;return[typeof o=="boolean"?o:qs(o),i]},useSyncExternalStore:tS,useId:MS,useHostTransitionStatus:Zg,useFormState:pS,useActionState:pS,useOptimistic:function(o,i){var c=kn();return hn!==null?lS(c,hn,o,i):(c.baseState=o,[o,c.queue.dispatch])},useMemoCache:kg,useCacheRefresh:NS},xl=null,Ks=0;function ff(o){var i=Ks;return Ks+=1,xl===null&&(xl=[]),Gb(xl,o,i)}function Xs(o,i){i=i.props.ref,o.ref=i!==void 0?i:null}function df(o,i){throw i.$$typeof===p?Error(r(525)):(o=Object.prototype.toString.call(i),Error(r(31,o==="[object Object]"?"object with keys {"+Object.keys(i).join(", ")+"}":o)))}function DS(o){var i=o._init;return i(o._payload)}function LS(o){function i(ce,ne){if(o){var de=ce.deletions;de===null?(ce.deletions=[ne],ce.flags|=16):de.push(ne)}}function c(ce,ne){if(!o)return null;for(;ne!==null;)i(ce,ne),ne=ne.sibling;return null}function m(ce){for(var ne=new Map;ce!==null;)ce.key!==null?ne.set(ce.key,ce):ne.set(ce.index,ce),ce=ce.sibling;return ne}function v(ce,ne){return ce=lo(ce,ne),ce.index=0,ce.sibling=null,ce}function w(ce,ne,de){return ce.index=de,o?(de=ce.alternate,de!==null?(de=de.index,de<ne?(ce.flags|=67108866,ne):de):(ce.flags|=67108866,ne)):(ce.flags|=1048576,ne)}function I(ce){return o&&ce.alternate===null&&(ce.flags|=67108866),ce}function H(ce,ne,de,$e){return ne===null||ne.tag!==6?(ne=Sg(de,ce.mode,$e),ne.return=ce,ne):(ne=v(ne,de),ne.return=ce,ne)}function J(ce,ne,de,$e){var it=de.type;return it===b?we(ce,ne,de.props.children,$e,de.key):ne!==null&&(ne.elementType===it||typeof it=="object"&&it!==null&&it.$$typeof===z&&DS(it)===ne.type)?(ne=v(ne,de.props),Xs(ne,de),ne.return=ce,ne):(ne=Gu(de.type,de.key,de.props,null,ce.mode,$e),Xs(ne,de),ne.return=ce,ne)}function me(ce,ne,de,$e){return ne===null||ne.tag!==4||ne.stateNode.containerInfo!==de.containerInfo||ne.stateNode.implementation!==de.implementation?(ne=Cg(de,ce.mode,$e),ne.return=ce,ne):(ne=v(ne,de.children||[]),ne.return=ce,ne)}function we(ce,ne,de,$e,it){return ne===null||ne.tag!==7?(ne=_i(de,ce.mode,$e,it),ne.return=ce,ne):(ne=v(ne,de),ne.return=ce,ne)}function _e(ce,ne,de){if(typeof ne=="string"&&ne!==""||typeof ne=="number"||typeof ne=="bigint")return ne=Sg(""+ne,ce.mode,de),ne.return=ce,ne;if(typeof ne=="object"&&ne!==null){switch(ne.$$typeof){case y:return de=Gu(ne.type,ne.key,ne.props,null,ce.mode,de),Xs(de,ne),de.return=ce,de;case C:return ne=Cg(ne,ce.mode,de),ne.return=ce,ne;case z:var $e=ne._init;return ne=$e(ne._payload),_e(ce,ne,de)}if(K(ne)||G(ne))return ne=_i(ne,ce.mode,de,null),ne.return=ce,ne;if(typeof ne.then=="function")return _e(ce,ff(ne),de);if(ne.$$typeof===O)return _e(ce,Qu(ce,ne),de);df(ce,ne)}return null}function pe(ce,ne,de,$e){var it=ne!==null?ne.key:null;if(typeof de=="string"&&de!==""||typeof de=="number"||typeof de=="bigint")return it!==null?null:H(ce,ne,""+de,$e);if(typeof de=="object"&&de!==null){switch(de.$$typeof){case y:return de.key===it?J(ce,ne,de,$e):null;case C:return de.key===it?me(ce,ne,de,$e):null;case z:return it=de._init,de=it(de._payload),pe(ce,ne,de,$e)}if(K(de)||G(de))return it!==null?null:we(ce,ne,de,$e,null);if(typeof de.then=="function")return pe(ce,ne,ff(de),$e);if(de.$$typeof===O)return pe(ce,ne,Qu(ce,de),$e);df(ce,de)}return null}function ye(ce,ne,de,$e,it){if(typeof $e=="string"&&$e!==""||typeof $e=="number"||typeof $e=="bigint")return ce=ce.get(de)||null,H(ne,ce,""+$e,it);if(typeof $e=="object"&&$e!==null){switch($e.$$typeof){case y:return ce=ce.get($e.key===null?de:$e.key)||null,J(ne,ce,$e,it);case C:return ce=ce.get($e.key===null?de:$e.key)||null,me(ne,ce,$e,it);case z:var zt=$e._init;return $e=zt($e._payload),ye(ce,ne,de,$e,it)}if(K($e)||G($e))return ce=ce.get(de)||null,we(ne,ce,$e,it,null);if(typeof $e.then=="function")return ye(ce,ne,de,ff($e),it);if($e.$$typeof===O)return ye(ce,ne,de,Qu(ne,$e),it);df(ne,$e)}return null}function St(ce,ne,de,$e){for(var it=null,zt=null,ft=ne,ht=ne=0,fr=null;ft!==null&&ht<de.length;ht++){ft.index>ht?(fr=ft,ft=null):fr=ft.sibling;var Jt=pe(ce,ft,de[ht],$e);if(Jt===null){ft===null&&(ft=fr);break}o&&ft&&Jt.alternate===null&&i(ce,ft),ne=w(Jt,ne,ht),zt===null?it=Jt:zt.sibling=Jt,zt=Jt,ft=fr}if(ht===de.length)return c(ce,ft),nn&&Mi(ce,ht),it;if(ft===null){for(;ht<de.length;ht++)ft=_e(ce,de[ht],$e),ft!==null&&(ne=w(ft,ne,ht),zt===null?it=ft:zt.sibling=ft,zt=ft);return nn&&Mi(ce,ht),it}for(ft=m(ft);ht<de.length;ht++)fr=ye(ft,ce,ht,de[ht],$e),fr!==null&&(o&&fr.alternate!==null&&ft.delete(fr.key===null?ht:fr.key),ne=w(fr,ne,ht),zt===null?it=fr:zt.sibling=fr,zt=fr);return o&&ft.forEach(function(si){return i(ce,si)}),nn&&Mi(ce,ht),it}function gt(ce,ne,de,$e){if(de==null)throw Error(r(151));for(var it=null,zt=null,ft=ne,ht=ne=0,fr=null,Jt=de.next();ft!==null&&!Jt.done;ht++,Jt=de.next()){ft.index>ht?(fr=ft,ft=null):fr=ft.sibling;var si=pe(ce,ft,Jt.value,$e);if(si===null){ft===null&&(ft=fr);break}o&&ft&&si.alternate===null&&i(ce,ft),ne=w(si,ne,ht),zt===null?it=si:zt.sibling=si,zt=si,ft=fr}if(Jt.done)return c(ce,ft),nn&&Mi(ce,ht),it;if(ft===null){for(;!Jt.done;ht++,Jt=de.next())Jt=_e(ce,Jt.value,$e),Jt!==null&&(ne=w(Jt,ne,ht),zt===null?it=Jt:zt.sibling=Jt,zt=Jt);return nn&&Mi(ce,ht),it}for(ft=m(ft);!Jt.done;ht++,Jt=de.next())Jt=ye(ft,ce,ht,Jt.value,$e),Jt!==null&&(o&&Jt.alternate!==null&&ft.delete(Jt.key===null?ht:Jt.key),ne=w(Jt,ne,ht),zt===null?it=Jt:zt.sibling=Jt,zt=Jt);return o&&ft.forEach(function(pI){return i(ce,pI)}),nn&&Mi(ce,ht),it}function vn(ce,ne,de,$e){if(typeof de=="object"&&de!==null&&de.type===b&&de.key===null&&(de=de.props.children),typeof de=="object"&&de!==null){switch(de.$$typeof){case y:e:{for(var it=de.key;ne!==null;){if(ne.key===it){if(it=de.type,it===b){if(ne.tag===7){c(ce,ne.sibling),$e=v(ne,de.props.children),$e.return=ce,ce=$e;break e}}else if(ne.elementType===it||typeof it=="object"&&it!==null&&it.$$typeof===z&&DS(it)===ne.type){c(ce,ne.sibling),$e=v(ne,de.props),Xs($e,de),$e.return=ce,ce=$e;break e}c(ce,ne);break}else i(ce,ne);ne=ne.sibling}de.type===b?($e=_i(de.props.children,ce.mode,$e,de.key),$e.return=ce,ce=$e):($e=Gu(de.type,de.key,de.props,null,ce.mode,$e),Xs($e,de),$e.return=ce,ce=$e)}return I(ce);case C:e:{for(it=de.key;ne!==null;){if(ne.key===it)if(ne.tag===4&&ne.stateNode.containerInfo===de.containerInfo&&ne.stateNode.implementation===de.implementation){c(ce,ne.sibling),$e=v(ne,de.children||[]),$e.return=ce,ce=$e;break e}else{c(ce,ne);break}else i(ce,ne);ne=ne.sibling}$e=Cg(de,ce.mode,$e),$e.return=ce,ce=$e}return I(ce);case z:return it=de._init,de=it(de._payload),vn(ce,ne,de,$e)}if(K(de))return St(ce,ne,de,$e);if(G(de)){if(it=G(de),typeof it!="function")throw Error(r(150));return de=it.call(de),gt(ce,ne,de,$e)}if(typeof de.then=="function")return vn(ce,ne,ff(de),$e);if(de.$$typeof===O)return vn(ce,ne,Qu(ce,de),$e);df(ce,de)}return typeof de=="string"&&de!==""||typeof de=="number"||typeof de=="bigint"?(de=""+de,ne!==null&&ne.tag===6?(c(ce,ne.sibling),$e=v(ne,de),$e.return=ce,ce=$e):(c(ce,ne),$e=Sg(de,ce.mode,$e),$e.return=ce,ce=$e),I(ce)):c(ce,ne)}return function(ce,ne,de,$e){try{Ks=0;var it=vn(ce,ne,de,$e);return xl=null,it}catch(ft){if(ft===Bs||ft===Ju)throw ft;var zt=kr(29,ft,null,ce.mode);return zt.lanes=$e,zt.return=ce,zt}finally{}}}var wl=LS(!0),BS=LS(!1),ga=B(null),Ga=null;function Go(o){var i=o.alternate;X(Jn,Jn.current&1),X(ga,o),Ga===null&&(i===null||yl.current!==null||i.memoizedState!==null)&&(Ga=o)}function HS(o){if(o.tag===22){if(X(Jn,Jn.current),X(ga,o),Ga===null){var i=o.alternate;i!==null&&i.memoizedState!==null&&(Ga=o)}}else Ko()}function Ko(){X(Jn,Jn.current),X(ga,ga.current)}function go(o){W(ga),Ga===o&&(Ga=null),W(Jn)}var Jn=B(0);function mf(o){for(var i=o;i!==null;){if(i.tag===13){var c=i.memoizedState;if(c!==null&&(c=c.dehydrated,c===null||c.data==="$?"||Uh(c)))return i}else if(i.tag===19&&i.memoizedProps.revealOrder!==void 0){if((i.flags&128)!==0)return i}else if(i.child!==null){i.child.return=i,i=i.child;continue}if(i===o)break;for(;i.sibling===null;){if(i.return===null||i.return===o)return null;i=i.return}i.sibling.return=i.return,i=i.sibling}return null}function eh(o,i,c,m){i=o.memoizedState,c=c(m,i),c=c==null?i:h({},i,c),o.memoizedState=c,o.lanes===0&&(o.updateQueue.baseState=c)}var th={enqueueSetState:function(o,i,c){o=o._reactInternals;var m=Kr(),v=ko(m);v.payload=i,c!=null&&(v.callback=c),i=qo(o,v,m),i!==null&&(Xr(i,o,m),Fs(i,o,m))},enqueueReplaceState:function(o,i,c){o=o._reactInternals;var m=Kr(),v=ko(m);v.tag=1,v.payload=i,c!=null&&(v.callback=c),i=qo(o,v,m),i!==null&&(Xr(i,o,m),Fs(i,o,m))},enqueueForceUpdate:function(o,i){o=o._reactInternals;var c=Kr(),m=ko(c);m.tag=2,i!=null&&(m.callback=i),i=qo(o,m,c),i!==null&&(Xr(i,o,c),Fs(i,o,c))}};function FS(o,i,c,m,v,w,I){return o=o.stateNode,typeof o.shouldComponentUpdate=="function"?o.shouldComponentUpdate(m,w,I):i.prototype&&i.prototype.isPureReactComponent?!Ns(c,m)||!Ns(v,w):!0}function VS(o,i,c,m){o=i.state,typeof i.componentWillReceiveProps=="function"&&i.componentWillReceiveProps(c,m),typeof i.UNSAFE_componentWillReceiveProps=="function"&&i.UNSAFE_componentWillReceiveProps(c,m),i.state!==o&&th.enqueueReplaceState(i,i.state,null)}function Di(o,i){var c=i;if("ref"in i){c={};for(var m in i)m!=="ref"&&(c[m]=i[m])}if(o=o.defaultProps){c===i&&(c=h({},c));for(var v in o)c[v]===void 0&&(c[v]=o[v])}return c}var gf=typeof reportError=="function"?reportError:function(o){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var i=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof o=="object"&&o!==null&&typeof o.message=="string"?String(o.message):String(o),error:o});if(!window.dispatchEvent(i))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",o);return}console.error(o)};function US(o){gf(o)}function kS(o){console.error(o)}function qS(o){gf(o)}function hf(o,i){try{var c=o.onUncaughtError;c(i.value,{componentStack:i.stack})}catch(m){setTimeout(function(){throw m})}}function WS(o,i,c){try{var m=o.onCaughtError;m(c.value,{componentStack:c.stack,errorBoundary:i.tag===1?i.stateNode:null})}catch(v){setTimeout(function(){throw v})}}function nh(o,i,c){return c=ko(c),c.tag=3,c.payload={element:null},c.callback=function(){hf(o,i)},c}function GS(o){return o=ko(o),o.tag=3,o}function KS(o,i,c,m){var v=c.type.getDerivedStateFromError;if(typeof v=="function"){var w=m.value;o.payload=function(){return v(w)},o.callback=function(){WS(i,c,m)}}var I=c.stateNode;I!==null&&typeof I.componentDidCatch=="function"&&(o.callback=function(){WS(i,c,m),typeof v!="function"&&(ei===null?ei=new Set([this]):ei.add(this));var H=m.stack;this.componentDidCatch(m.value,{componentStack:H!==null?H:""})})}function pj(o,i,c,m,v){if(c.flags|=32768,m!==null&&typeof m=="object"&&typeof m.then=="function"){if(i=c.alternate,i!==null&&zs(i,c,v,!0),c=ga.current,c!==null){switch(c.tag){case 13:return Ga===null?Oh():c.alternate===null&&Pn===0&&(Pn=3),c.flags&=-257,c.flags|=65536,c.lanes=v,m===Ng?c.flags|=16384:(i=c.updateQueue,i===null?c.updateQueue=new Set([m]):i.add(m),_h(o,m,v)),!1;case 22:return c.flags|=65536,m===Ng?c.flags|=16384:(i=c.updateQueue,i===null?(i={transitions:null,markerInstances:null,retryQueue:new Set([m])},c.updateQueue=i):(c=i.retryQueue,c===null?i.retryQueue=new Set([m]):c.add(m)),_h(o,m,v)),!1}throw Error(r(435,c.tag))}return _h(o,m,v),Oh(),!1}if(nn)return i=ga.current,i!==null?((i.flags&65536)===0&&(i.flags|=256),i.flags|=65536,i.lanes=v,m!==Eg&&(o=Error(r(422),{cause:m}),As(ua(o,c)))):(m!==Eg&&(i=Error(r(423),{cause:m}),As(ua(i,c))),o=o.current.alternate,o.flags|=65536,v&=-v,o.lanes|=v,m=ua(m,c),v=nh(o.stateNode,m,v),Pg(o,v),Pn!==4&&(Pn=2)),!1;var w=Error(r(520),{cause:m});if(w=ua(w,c),nc===null?nc=[w]:nc.push(w),Pn!==4&&(Pn=2),i===null)return!0;m=ua(m,c),c=i;do{switch(c.tag){case 3:return c.flags|=65536,o=v&-v,c.lanes|=o,o=nh(c.stateNode,m,o),Pg(c,o),!1;case 1:if(i=c.type,w=c.stateNode,(c.flags&128)===0&&(typeof i.getDerivedStateFromError=="function"||w!==null&&typeof w.componentDidCatch=="function"&&(ei===null||!ei.has(w))))return c.flags|=65536,v&=-v,c.lanes|=v,v=GS(v),KS(v,o,c,m),Pg(c,v),!1}c=c.return}while(c!==null);return!1}var XS=Error(r(461)),cr=!1;function hr(o,i,c,m){i.child=o===null?BS(i,null,c,m):wl(i,o.child,c,m)}function YS(o,i,c,m,v){c=c.render;var w=i.ref;if("ref"in m){var I={};for(var H in m)H!=="ref"&&(I[H]=m[H])}else I=m;return Pi(i),m=Bg(o,i,c,I,w,v),H=Hg(),o!==null&&!cr?(Fg(o,i,v),ho(o,i,v)):(nn&&H&&xg(i),i.flags|=1,hr(o,i,m,v),i.child)}function QS(o,i,c,m,v){if(o===null){var w=c.type;return typeof w=="function"&&!bg(w)&&w.defaultProps===void 0&&c.compare===null?(i.tag=15,i.type=w,ZS(o,i,w,m,v)):(o=Gu(c.type,null,m,i,i.mode,v),o.ref=i.ref,o.return=i,i.child=o)}if(w=o.child,!uh(o,v)){var I=w.memoizedProps;if(c=c.compare,c=c!==null?c:Ns,c(I,m)&&o.ref===i.ref)return ho(o,i,v)}return i.flags|=1,o=lo(w,m),o.ref=i.ref,o.return=i,i.child=o}function ZS(o,i,c,m,v){if(o!==null){var w=o.memoizedProps;if(Ns(w,m)&&o.ref===i.ref)if(cr=!1,i.pendingProps=m=w,uh(o,v))(o.flags&131072)!==0&&(cr=!0);else return i.lanes=o.lanes,ho(o,i,v)}return rh(o,i,c,m,v)}function JS(o,i,c){var m=i.pendingProps,v=m.children,w=o!==null?o.memoizedState:null;if(m.mode==="hidden"){if((i.flags&128)!==0){if(m=w!==null?w.baseLanes|c:c,o!==null){for(v=i.child=o.child,w=0;v!==null;)w=w|v.lanes|v.childLanes,v=v.sibling;i.childLanes=w&~m}else i.childLanes=0,i.child=null;return eC(o,i,m,c)}if((c&536870912)!==0)i.memoizedState={baseLanes:0,cachePool:null},o!==null&&Zu(i,w!==null?w.cachePool:null),w!==null?Zb(i,w):zg(),HS(i);else return i.lanes=i.childLanes=536870912,eC(o,i,w!==null?w.baseLanes|c:c,c)}else w!==null?(Zu(i,w.cachePool),Zb(i,w),Ko(),i.memoizedState=null):(o!==null&&Zu(i,null),zg(),Ko());return hr(o,i,v,c),i.child}function eC(o,i,c,m){var v=Mg();return v=v===null?null:{parent:Zn._currentValue,pool:v},i.memoizedState={baseLanes:c,cachePool:v},o!==null&&Zu(i,null),zg(),HS(i),o!==null&&zs(o,i,m,!0),null}function pf(o,i){var c=i.ref;if(c===null)o!==null&&o.ref!==null&&(i.flags|=4194816);else{if(typeof c!="function"&&typeof c!="object")throw Error(r(284));(o===null||o.ref!==c)&&(i.flags|=4194816)}}function rh(o,i,c,m,v){return Pi(i),c=Bg(o,i,c,m,void 0,v),m=Hg(),o!==null&&!cr?(Fg(o,i,v),ho(o,i,v)):(nn&&m&&xg(i),i.flags|=1,hr(o,i,c,v),i.child)}function tC(o,i,c,m,v,w){return Pi(i),i.updateQueue=null,c=eS(i,m,c,v),Jb(o),m=Hg(),o!==null&&!cr?(Fg(o,i,w),ho(o,i,w)):(nn&&m&&xg(i),i.flags|=1,hr(o,i,c,w),i.child)}function nC(o,i,c,m,v){if(Pi(i),i.stateNode===null){var w=ml,I=c.contextType;typeof I=="object"&&I!==null&&(w=Cr(I)),w=new c(m,w),i.memoizedState=w.state!==null&&w.state!==void 0?w.state:null,w.updater=th,i.stateNode=w,w._reactInternals=i,w=i.stateNode,w.props=m,w.state=i.memoizedState,w.refs={},jg(i),I=c.contextType,w.context=typeof I=="object"&&I!==null?Cr(I):ml,w.state=i.memoizedState,I=c.getDerivedStateFromProps,typeof I=="function"&&(eh(i,c,I,m),w.state=i.memoizedState),typeof c.getDerivedStateFromProps=="function"||typeof w.getSnapshotBeforeUpdate=="function"||typeof w.UNSAFE_componentWillMount!="function"&&typeof w.componentWillMount!="function"||(I=w.state,typeof w.componentWillMount=="function"&&w.componentWillMount(),typeof w.UNSAFE_componentWillMount=="function"&&w.UNSAFE_componentWillMount(),I!==w.state&&th.enqueueReplaceState(w,w.state,null),Us(i,m,w,v),Vs(),w.state=i.memoizedState),typeof w.componentDidMount=="function"&&(i.flags|=4194308),m=!0}else if(o===null){w=i.stateNode;var H=i.memoizedProps,J=Di(c,H);w.props=J;var me=w.context,we=c.contextType;I=ml,typeof we=="object"&&we!==null&&(I=Cr(we));var _e=c.getDerivedStateFromProps;we=typeof _e=="function"||typeof w.getSnapshotBeforeUpdate=="function",H=i.pendingProps!==H,we||typeof w.UNSAFE_componentWillReceiveProps!="function"&&typeof w.componentWillReceiveProps!="function"||(H||me!==I)&&VS(i,w,m,I),Uo=!1;var pe=i.memoizedState;w.state=pe,Us(i,m,w,v),Vs(),me=i.memoizedState,H||pe!==me||Uo?(typeof _e=="function"&&(eh(i,c,_e,m),me=i.memoizedState),(J=Uo||FS(i,c,J,m,pe,me,I))?(we||typeof w.UNSAFE_componentWillMount!="function"&&typeof w.componentWillMount!="function"||(typeof w.componentWillMount=="function"&&w.componentWillMount(),typeof w.UNSAFE_componentWillMount=="function"&&w.UNSAFE_componentWillMount()),typeof w.componentDidMount=="function"&&(i.flags|=4194308)):(typeof w.componentDidMount=="function"&&(i.flags|=4194308),i.memoizedProps=m,i.memoizedState=me),w.props=m,w.state=me,w.context=I,m=J):(typeof w.componentDidMount=="function"&&(i.flags|=4194308),m=!1)}else{w=i.stateNode,Ig(o,i),I=i.memoizedProps,we=Di(c,I),w.props=we,_e=i.pendingProps,pe=w.context,me=c.contextType,J=ml,typeof me=="object"&&me!==null&&(J=Cr(me)),H=c.getDerivedStateFromProps,(me=typeof H=="function"||typeof w.getSnapshotBeforeUpdate=="function")||typeof w.UNSAFE_componentWillReceiveProps!="function"&&typeof w.componentWillReceiveProps!="function"||(I!==_e||pe!==J)&&VS(i,w,m,J),Uo=!1,pe=i.memoizedState,w.state=pe,Us(i,m,w,v),Vs();var ye=i.memoizedState;I!==_e||pe!==ye||Uo||o!==null&&o.dependencies!==null&&Yu(o.dependencies)?(typeof H=="function"&&(eh(i,c,H,m),ye=i.memoizedState),(we=Uo||FS(i,c,we,m,pe,ye,J)||o!==null&&o.dependencies!==null&&Yu(o.dependencies))?(me||typeof w.UNSAFE_componentWillUpdate!="function"&&typeof w.componentWillUpdate!="function"||(typeof w.componentWillUpdate=="function"&&w.componentWillUpdate(m,ye,J),typeof w.UNSAFE_componentWillUpdate=="function"&&w.UNSAFE_componentWillUpdate(m,ye,J)),typeof w.componentDidUpdate=="function"&&(i.flags|=4),typeof w.getSnapshotBeforeUpdate=="function"&&(i.flags|=1024)):(typeof w.componentDidUpdate!="function"||I===o.memoizedProps&&pe===o.memoizedState||(i.flags|=4),typeof w.getSnapshotBeforeUpdate!="function"||I===o.memoizedProps&&pe===o.memoizedState||(i.flags|=1024),i.memoizedProps=m,i.memoizedState=ye),w.props=m,w.state=ye,w.context=J,m=we):(typeof w.componentDidUpdate!="function"||I===o.memoizedProps&&pe===o.memoizedState||(i.flags|=4),typeof w.getSnapshotBeforeUpdate!="function"||I===o.memoizedProps&&pe===o.memoizedState||(i.flags|=1024),m=!1)}return w=m,pf(o,i),m=(i.flags&128)!==0,w||m?(w=i.stateNode,c=m&&typeof c.getDerivedStateFromError!="function"?null:w.render(),i.flags|=1,o!==null&&m?(i.child=wl(i,o.child,null,v),i.child=wl(i,null,c,v)):hr(o,i,c,v),i.memoizedState=w.state,o=i.child):o=ho(o,i,v),o}function rC(o,i,c,m){return Ps(),i.flags|=256,hr(o,i,c,m),i.child}var ah={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function oh(o){return{baseLanes:o,cachePool:kb()}}function ih(o,i,c){return o=o!==null?o.childLanes&~c:0,i&&(o|=ha),o}function aC(o,i,c){var m=i.pendingProps,v=!1,w=(i.flags&128)!==0,I;if((I=w)||(I=o!==null&&o.memoizedState===null?!1:(Jn.current&2)!==0),I&&(v=!0,i.flags&=-129),I=(i.flags&32)!==0,i.flags&=-33,o===null){if(nn){if(v?Go(i):Ko(),nn){var H=In,J;if(J=H){e:{for(J=H,H=Wa;J.nodeType!==8;){if(!H){H=null;break e}if(J=Ta(J.nextSibling),J===null){H=null;break e}}H=J}H!==null?(i.memoizedState={dehydrated:H,treeContext:Ti!==null?{id:so,overflow:co}:null,retryLane:536870912,hydrationErrors:null},J=kr(18,null,null,0),J.stateNode=H,J.return=i,i.child=J,Mr=i,In=null,J=!0):J=!1}J||ji(i)}if(H=i.memoizedState,H!==null&&(H=H.dehydrated,H!==null))return Uh(H)?i.lanes=32:i.lanes=536870912,null;go(i)}return H=m.children,m=m.fallback,v?(Ko(),v=i.mode,H=vf({mode:"hidden",children:H},v),m=_i(m,v,c,null),H.return=i,m.return=i,H.sibling=m,i.child=H,v=i.child,v.memoizedState=oh(c),v.childLanes=ih(o,I,c),i.memoizedState=ah,m):(Go(i),lh(i,H))}if(J=o.memoizedState,J!==null&&(H=J.dehydrated,H!==null)){if(w)i.flags&256?(Go(i),i.flags&=-257,i=sh(o,i,c)):i.memoizedState!==null?(Ko(),i.child=o.child,i.flags|=128,i=null):(Ko(),v=m.fallback,H=i.mode,m=vf({mode:"visible",children:m.children},H),v=_i(v,H,c,null),v.flags|=2,m.return=i,v.return=i,m.sibling=v,i.child=m,wl(i,o.child,null,c),m=i.child,m.memoizedState=oh(c),m.childLanes=ih(o,I,c),i.memoizedState=ah,i=v);else if(Go(i),Uh(H)){if(I=H.nextSibling&&H.nextSibling.dataset,I)var me=I.dgst;I=me,m=Error(r(419)),m.stack="",m.digest=I,As({value:m,source:null,stack:null}),i=sh(o,i,c)}else if(cr||zs(o,i,c,!1),I=(c&o.childLanes)!==0,cr||I){if(I=$n,I!==null&&(m=c&-c,m=(m&42)!==0?1:Ot(m),m=(m&(I.suspendedLanes|c))!==0?0:m,m!==0&&m!==J.retryLane))throw J.retryLane=m,dl(o,m),Xr(I,o,m),XS;H.data==="$?"||Oh(),i=sh(o,i,c)}else H.data==="$?"?(i.flags|=192,i.child=o.child,i=null):(o=J.treeContext,In=Ta(H.nextSibling),Mr=i,nn=!0,Ni=null,Wa=!1,o!==null&&(da[ma++]=so,da[ma++]=co,da[ma++]=Ti,so=o.id,co=o.overflow,Ti=i),i=lh(i,m.children),i.flags|=4096);return i}return v?(Ko(),v=m.fallback,H=i.mode,J=o.child,me=J.sibling,m=lo(J,{mode:"hidden",children:m.children}),m.subtreeFlags=J.subtreeFlags&65011712,me!==null?v=lo(me,v):(v=_i(v,H,c,null),v.flags|=2),v.return=i,m.return=i,m.sibling=v,i.child=m,m=v,v=i.child,H=o.child.memoizedState,H===null?H=oh(c):(J=H.cachePool,J!==null?(me=Zn._currentValue,J=J.parent!==me?{parent:me,pool:me}:J):J=kb(),H={baseLanes:H.baseLanes|c,cachePool:J}),v.memoizedState=H,v.childLanes=ih(o,I,c),i.memoizedState=ah,m):(Go(i),c=o.child,o=c.sibling,c=lo(c,{mode:"visible",children:m.children}),c.return=i,c.sibling=null,o!==null&&(I=i.deletions,I===null?(i.deletions=[o],i.flags|=16):I.push(o)),i.child=c,i.memoizedState=null,c)}function lh(o,i){return i=vf({mode:"visible",children:i},o.mode),i.return=o,o.child=i}function vf(o,i){return o=kr(22,o,null,i),o.lanes=0,o.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null},o}function sh(o,i,c){return wl(i,o.child,null,c),o=lh(i,i.pendingProps.children),o.flags|=2,i.memoizedState=null,o}function oC(o,i,c){o.lanes|=i;var m=o.alternate;m!==null&&(m.lanes|=i),Og(o.return,i,c)}function ch(o,i,c,m,v){var w=o.memoizedState;w===null?o.memoizedState={isBackwards:i,rendering:null,renderingStartTime:0,last:m,tail:c,tailMode:v}:(w.isBackwards=i,w.rendering=null,w.renderingStartTime=0,w.last=m,w.tail=c,w.tailMode=v)}function iC(o,i,c){var m=i.pendingProps,v=m.revealOrder,w=m.tail;if(hr(o,i,m.children,c),m=Jn.current,(m&2)!==0)m=m&1|2,i.flags|=128;else{if(o!==null&&(o.flags&128)!==0)e:for(o=i.child;o!==null;){if(o.tag===13)o.memoizedState!==null&&oC(o,c,i);else if(o.tag===19)oC(o,c,i);else if(o.child!==null){o.child.return=o,o=o.child;continue}if(o===i)break e;for(;o.sibling===null;){if(o.return===null||o.return===i)break e;o=o.return}o.sibling.return=o.return,o=o.sibling}m&=1}switch(X(Jn,m),v){case"forwards":for(c=i.child,v=null;c!==null;)o=c.alternate,o!==null&&mf(o)===null&&(v=c),c=c.sibling;c=v,c===null?(v=i.child,i.child=null):(v=c.sibling,c.sibling=null),ch(i,!1,v,c,w);break;case"backwards":for(c=null,v=i.child,i.child=null;v!==null;){if(o=v.alternate,o!==null&&mf(o)===null){i.child=v;break}o=v.sibling,v.sibling=c,c=v,v=o}ch(i,!0,c,null,w);break;case"together":ch(i,!1,null,null,void 0);break;default:i.memoizedState=null}return i.child}function ho(o,i,c){if(o!==null&&(i.dependencies=o.dependencies),Jo|=i.lanes,(c&i.childLanes)===0)if(o!==null){if(zs(o,i,c,!1),(c&i.childLanes)===0)return null}else return null;if(o!==null&&i.child!==o.child)throw Error(r(153));if(i.child!==null){for(o=i.child,c=lo(o,o.pendingProps),i.child=c,c.return=i;o.sibling!==null;)o=o.sibling,c=c.sibling=lo(o,o.pendingProps),c.return=i;c.sibling=null}return i.child}function uh(o,i){return(o.lanes&i)!==0?!0:(o=o.dependencies,!!(o!==null&&Yu(o)))}function vj(o,i,c){switch(i.tag){case 3:ae(i,i.stateNode.containerInfo),Vo(i,Zn,o.memoizedState.cache),Ps();break;case 27:case 5:he(i);break;case 4:ae(i,i.stateNode.containerInfo);break;case 10:Vo(i,i.type,i.memoizedProps.value);break;case 13:var m=i.memoizedState;if(m!==null)return m.dehydrated!==null?(Go(i),i.flags|=128,null):(c&i.child.childLanes)!==0?aC(o,i,c):(Go(i),o=ho(o,i,c),o!==null?o.sibling:null);Go(i);break;case 19:var v=(o.flags&128)!==0;if(m=(c&i.childLanes)!==0,m||(zs(o,i,c,!1),m=(c&i.childLanes)!==0),v){if(m)return iC(o,i,c);i.flags|=128}if(v=i.memoizedState,v!==null&&(v.rendering=null,v.tail=null,v.lastEffect=null),X(Jn,Jn.current),m)break;return null;case 22:case 23:return i.lanes=0,JS(o,i,c);case 24:Vo(i,Zn,o.memoizedState.cache)}return ho(o,i,c)}function lC(o,i,c){if(o!==null)if(o.memoizedProps!==i.pendingProps)cr=!0;else{if(!uh(o,c)&&(i.flags&128)===0)return cr=!1,vj(o,i,c);cr=(o.flags&131072)!==0}else cr=!1,nn&&(i.flags&1048576)!==0&&Db(i,Xu,i.index);switch(i.lanes=0,i.tag){case 16:e:{o=i.pendingProps;var m=i.elementType,v=m._init;if(m=v(m._payload),i.type=m,typeof m=="function")bg(m)?(o=Di(m,o),i.tag=1,i=nC(null,i,m,o,c)):(i.tag=0,i=rh(null,i,m,o,c));else{if(m!=null){if(v=m.$$typeof,v===_){i.tag=11,i=YS(null,i,m,o,c);break e}else if(v===M){i.tag=14,i=QS(null,i,m,o,c);break e}}throw i=U(m)||m,Error(r(306,i,""))}}return i;case 0:return rh(o,i,i.type,i.pendingProps,c);case 1:return m=i.type,v=Di(m,i.pendingProps),nC(o,i,m,v,c);case 3:e:{if(ae(i,i.stateNode.containerInfo),o===null)throw Error(r(387));m=i.pendingProps;var w=i.memoizedState;v=w.element,Ig(o,i),Us(i,m,null,c);var I=i.memoizedState;if(m=I.cache,Vo(i,Zn,m),m!==w.cache&&Rg(i,[Zn],c,!0),Vs(),m=I.element,w.isDehydrated)if(w={element:m,isDehydrated:!1,cache:I.cache},i.updateQueue.baseState=w,i.memoizedState=w,i.flags&256){i=rC(o,i,m,c);break e}else if(m!==v){v=ua(Error(r(424)),i),As(v),i=rC(o,i,m,c);break e}else{switch(o=i.stateNode.containerInfo,o.nodeType){case 9:o=o.body;break;default:o=o.nodeName==="HTML"?o.ownerDocument.body:o}for(In=Ta(o.firstChild),Mr=i,nn=!0,Ni=null,Wa=!0,c=BS(i,null,m,c),i.child=c;c;)c.flags=c.flags&-3|4096,c=c.sibling}else{if(Ps(),m===v){i=ho(o,i,c);break e}hr(o,i,m,c)}i=i.child}return i;case 26:return pf(o,i),o===null?(c=f1(i.type,null,i.pendingProps,null))?i.memoizedState=c:nn||(c=i.type,o=i.pendingProps,m=Nf(ee.current).createElement(c),m[Ct]=i,m[Ve]=o,vr(m,c,o),Tt(m),i.stateNode=m):i.memoizedState=f1(i.type,o.memoizedProps,i.pendingProps,o.memoizedState),null;case 27:return he(i),o===null&&nn&&(m=i.stateNode=s1(i.type,i.pendingProps,ee.current),Mr=i,Wa=!0,v=In,ri(i.type)?(kh=v,In=Ta(m.firstChild)):In=v),hr(o,i,i.pendingProps.children,c),pf(o,i),o===null&&(i.flags|=4194304),i.child;case 5:return o===null&&nn&&((v=m=In)&&(m=qj(m,i.type,i.pendingProps,Wa),m!==null?(i.stateNode=m,Mr=i,In=Ta(m.firstChild),Wa=!1,v=!0):v=!1),v||ji(i)),he(i),v=i.type,w=i.pendingProps,I=o!==null?o.memoizedProps:null,m=w.children,Hh(v,w)?m=null:I!==null&&Hh(v,I)&&(i.flags|=32),i.memoizedState!==null&&(v=Bg(o,i,cj,null,null,c),fc._currentValue=v),pf(o,i),hr(o,i,m,c),i.child;case 6:return o===null&&nn&&((o=c=In)&&(c=Wj(c,i.pendingProps,Wa),c!==null?(i.stateNode=c,Mr=i,In=null,o=!0):o=!1),o||ji(i)),null;case 13:return aC(o,i,c);case 4:return ae(i,i.stateNode.containerInfo),m=i.pendingProps,o===null?i.child=wl(i,null,m,c):hr(o,i,m,c),i.child;case 11:return YS(o,i,i.type,i.pendingProps,c);case 7:return hr(o,i,i.pendingProps,c),i.child;case 8:return hr(o,i,i.pendingProps.children,c),i.child;case 12:return hr(o,i,i.pendingProps.children,c),i.child;case 10:return m=i.pendingProps,Vo(i,i.type,m.value),hr(o,i,m.children,c),i.child;case 9:return v=i.type._context,m=i.pendingProps.children,Pi(i),v=Cr(v),m=m(v),i.flags|=1,hr(o,i,m,c),i.child;case 14:return QS(o,i,i.type,i.pendingProps,c);case 15:return ZS(o,i,i.type,i.pendingProps,c);case 19:return iC(o,i,c);case 31:return m=i.pendingProps,c=i.mode,m={mode:m.mode,children:m.children},o===null?(c=vf(m,c),c.ref=i.ref,i.child=c,c.return=i,i=c):(c=lo(o.child,m),c.ref=i.ref,i.child=c,c.return=i,i=c),i;case 22:return JS(o,i,c);case 24:return Pi(i),m=Cr(Zn),o===null?(v=Mg(),v===null&&(v=$n,w=_g(),v.pooledCache=w,w.refCount++,w!==null&&(v.pooledCacheLanes|=c),v=w),i.memoizedState={parent:m,cache:v},jg(i),Vo(i,Zn,v)):((o.lanes&c)!==0&&(Ig(o,i),Us(i,null,null,c),Vs()),v=o.memoizedState,w=i.memoizedState,v.parent!==m?(v={parent:m,cache:m},i.memoizedState=v,i.lanes===0&&(i.memoizedState=i.updateQueue.baseState=v),Vo(i,Zn,m)):(m=w.cache,Vo(i,Zn,m),m!==v.cache&&Rg(i,[Zn],c,!0))),hr(o,i,i.pendingProps.children,c),i.child;case 29:throw i.pendingProps}throw Error(r(156,i.tag))}function po(o){o.flags|=4}function sC(o,i){if(i.type!=="stylesheet"||(i.state.loading&4)!==0)o.flags&=-16777217;else if(o.flags|=16777216,!p1(i)){if(i=ga.current,i!==null&&((Xt&4194048)===Xt?Ga!==null:(Xt&62914560)!==Xt&&(Xt&536870912)===0||i!==Ga))throw Hs=Ng,qb;o.flags|=8192}}function yf(o,i){i!==null&&(o.flags|=4),o.flags&16384&&(i=o.tag!==22?Nn():536870912,o.lanes|=i,Rl|=i)}function Ys(o,i){if(!nn)switch(o.tailMode){case"hidden":i=o.tail;for(var c=null;i!==null;)i.alternate!==null&&(c=i),i=i.sibling;c===null?o.tail=null:c.sibling=null;break;case"collapsed":c=o.tail;for(var m=null;c!==null;)c.alternate!==null&&(m=c),c=c.sibling;m===null?i||o.tail===null?o.tail=null:o.tail.sibling=null:m.sibling=null}}function Mn(o){var i=o.alternate!==null&&o.alternate.child===o.child,c=0,m=0;if(i)for(var v=o.child;v!==null;)c|=v.lanes|v.childLanes,m|=v.subtreeFlags&65011712,m|=v.flags&65011712,v.return=o,v=v.sibling;else for(v=o.child;v!==null;)c|=v.lanes|v.childLanes,m|=v.subtreeFlags,m|=v.flags,v.return=o,v=v.sibling;return o.subtreeFlags|=m,o.childLanes=c,i}function yj(o,i,c){var m=i.pendingProps;switch(wg(i),i.tag){case 31:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return Mn(i),null;case 1:return Mn(i),null;case 3:return c=i.stateNode,m=null,o!==null&&(m=o.memoizedState.cache),i.memoizedState.cache!==m&&(i.flags|=2048),fo(Zn),oe(),c.pendingContext&&(c.context=c.pendingContext,c.pendingContext=null),(o===null||o.child===null)&&(Is(i)?po(i):o===null||o.memoizedState.isDehydrated&&(i.flags&256)===0||(i.flags|=1024,Hb())),Mn(i),null;case 26:return c=i.memoizedState,o===null?(po(i),c!==null?(Mn(i),sC(i,c)):(Mn(i),i.flags&=-16777217)):c?c!==o.memoizedState?(po(i),Mn(i),sC(i,c)):(Mn(i),i.flags&=-16777217):(o.memoizedProps!==m&&po(i),Mn(i),i.flags&=-16777217),null;case 27:ie(i),c=ee.current;var v=i.type;if(o!==null&&i.stateNode!=null)o.memoizedProps!==m&&po(i);else{if(!m){if(i.stateNode===null)throw Error(r(166));return Mn(i),null}o=k.current,Is(i)?Lb(i):(o=s1(v,m,c),i.stateNode=o,po(i))}return Mn(i),null;case 5:if(ie(i),c=i.type,o!==null&&i.stateNode!=null)o.memoizedProps!==m&&po(i);else{if(!m){if(i.stateNode===null)throw Error(r(166));return Mn(i),null}if(o=k.current,Is(i))Lb(i);else{switch(v=Nf(ee.current),o){case 1:o=v.createElementNS("http://www.w3.org/2000/svg",c);break;case 2:o=v.createElementNS("http://www.w3.org/1998/Math/MathML",c);break;default:switch(c){case"svg":o=v.createElementNS("http://www.w3.org/2000/svg",c);break;case"math":o=v.createElementNS("http://www.w3.org/1998/Math/MathML",c);break;case"script":o=v.createElement("div"),o.innerHTML="<script><\/script>",o=o.removeChild(o.firstChild);break;case"select":o=typeof m.is=="string"?v.createElement("select",{is:m.is}):v.createElement("select"),m.multiple?o.multiple=!0:m.size&&(o.size=m.size);break;default:o=typeof m.is=="string"?v.createElement(c,{is:m.is}):v.createElement(c)}}o[Ct]=i,o[Ve]=m;e:for(v=i.child;v!==null;){if(v.tag===5||v.tag===6)o.appendChild(v.stateNode);else if(v.tag!==4&&v.tag!==27&&v.child!==null){v.child.return=v,v=v.child;continue}if(v===i)break e;for(;v.sibling===null;){if(v.return===null||v.return===i)break e;v=v.return}v.sibling.return=v.return,v=v.sibling}i.stateNode=o;e:switch(vr(o,c,m),c){case"button":case"input":case"select":case"textarea":o=!!m.autoFocus;break e;case"img":o=!0;break e;default:o=!1}o&&po(i)}}return Mn(i),i.flags&=-16777217,null;case 6:if(o&&i.stateNode!=null)o.memoizedProps!==m&&po(i);else{if(typeof m!="string"&&i.stateNode===null)throw Error(r(166));if(o=ee.current,Is(i)){if(o=i.stateNode,c=i.memoizedProps,m=null,v=Mr,v!==null)switch(v.tag){case 27:case 5:m=v.memoizedProps}o[Ct]=i,o=!!(o.nodeValue===c||m!==null&&m.suppressHydrationWarning===!0||t1(o.nodeValue,c)),o||ji(i)}else o=Nf(o).createTextNode(m),o[Ct]=i,i.stateNode=o}return Mn(i),null;case 13:if(m=i.memoizedState,o===null||o.memoizedState!==null&&o.memoizedState.dehydrated!==null){if(v=Is(i),m!==null&&m.dehydrated!==null){if(o===null){if(!v)throw Error(r(318));if(v=i.memoizedState,v=v!==null?v.dehydrated:null,!v)throw Error(r(317));v[Ct]=i}else Ps(),(i.flags&128)===0&&(i.memoizedState=null),i.flags|=4;Mn(i),v=!1}else v=Hb(),o!==null&&o.memoizedState!==null&&(o.memoizedState.hydrationErrors=v),v=!0;if(!v)return i.flags&256?(go(i),i):(go(i),null)}if(go(i),(i.flags&128)!==0)return i.lanes=c,i;if(c=m!==null,o=o!==null&&o.memoizedState!==null,c){m=i.child,v=null,m.alternate!==null&&m.alternate.memoizedState!==null&&m.alternate.memoizedState.cachePool!==null&&(v=m.alternate.memoizedState.cachePool.pool);var w=null;m.memoizedState!==null&&m.memoizedState.cachePool!==null&&(w=m.memoizedState.cachePool.pool),w!==v&&(m.flags|=2048)}return c!==o&&c&&(i.child.flags|=8192),yf(i,i.updateQueue),Mn(i),null;case 4:return oe(),o===null&&Ah(i.stateNode.containerInfo),Mn(i),null;case 10:return fo(i.type),Mn(i),null;case 19:if(W(Jn),v=i.memoizedState,v===null)return Mn(i),null;if(m=(i.flags&128)!==0,w=v.rendering,w===null)if(m)Ys(v,!1);else{if(Pn!==0||o!==null&&(o.flags&128)!==0)for(o=i.child;o!==null;){if(w=mf(o),w!==null){for(i.flags|=128,Ys(v,!1),o=w.updateQueue,i.updateQueue=o,yf(i,o),i.subtreeFlags=0,o=c,c=i.child;c!==null;)zb(c,o),c=c.sibling;return X(Jn,Jn.current&1|2),i.child}o=o.sibling}v.tail!==null&&Ee()>Cf&&(i.flags|=128,m=!0,Ys(v,!1),i.lanes=4194304)}else{if(!m)if(o=mf(w),o!==null){if(i.flags|=128,m=!0,o=o.updateQueue,i.updateQueue=o,yf(i,o),Ys(v,!0),v.tail===null&&v.tailMode==="hidden"&&!w.alternate&&!nn)return Mn(i),null}else 2*Ee()-v.renderingStartTime>Cf&&c!==536870912&&(i.flags|=128,m=!0,Ys(v,!1),i.lanes=4194304);v.isBackwards?(w.sibling=i.child,i.child=w):(o=v.last,o!==null?o.sibling=w:i.child=w,v.last=w)}return v.tail!==null?(i=v.tail,v.rendering=i,v.tail=i.sibling,v.renderingStartTime=Ee(),i.sibling=null,o=Jn.current,X(Jn,m?o&1|2:o&1),i):(Mn(i),null);case 22:case 23:return go(i),Dg(),m=i.memoizedState!==null,o!==null?o.memoizedState!==null!==m&&(i.flags|=8192):m&&(i.flags|=8192),m?(c&536870912)!==0&&(i.flags&128)===0&&(Mn(i),i.subtreeFlags&6&&(i.flags|=8192)):Mn(i),c=i.updateQueue,c!==null&&yf(i,c.retryQueue),c=null,o!==null&&o.memoizedState!==null&&o.memoizedState.cachePool!==null&&(c=o.memoizedState.cachePool.pool),m=null,i.memoizedState!==null&&i.memoizedState.cachePool!==null&&(m=i.memoizedState.cachePool.pool),m!==c&&(i.flags|=2048),o!==null&&W(Ai),null;case 24:return c=null,o!==null&&(c=o.memoizedState.cache),i.memoizedState.cache!==c&&(i.flags|=2048),fo(Zn),Mn(i),null;case 25:return null;case 30:return null}throw Error(r(156,i.tag))}function bj(o,i){switch(wg(i),i.tag){case 1:return o=i.flags,o&65536?(i.flags=o&-65537|128,i):null;case 3:return fo(Zn),oe(),o=i.flags,(o&65536)!==0&&(o&128)===0?(i.flags=o&-65537|128,i):null;case 26:case 27:case 5:return ie(i),null;case 13:if(go(i),o=i.memoizedState,o!==null&&o.dehydrated!==null){if(i.alternate===null)throw Error(r(340));Ps()}return o=i.flags,o&65536?(i.flags=o&-65537|128,i):null;case 19:return W(Jn),null;case 4:return oe(),null;case 10:return fo(i.type),null;case 22:case 23:return go(i),Dg(),o!==null&&W(Ai),o=i.flags,o&65536?(i.flags=o&-65537|128,i):null;case 24:return fo(Zn),null;case 25:return null;default:return null}}function cC(o,i){switch(wg(i),i.tag){case 3:fo(Zn),oe();break;case 26:case 27:case 5:ie(i);break;case 4:oe();break;case 13:go(i);break;case 19:W(Jn);break;case 10:fo(i.type);break;case 22:case 23:go(i),Dg(),o!==null&&W(Ai);break;case 24:fo(Zn)}}function Qs(o,i){try{var c=i.updateQueue,m=c!==null?c.lastEffect:null;if(m!==null){var v=m.next;c=v;do{if((c.tag&o)===o){m=void 0;var w=c.create,I=c.inst;m=w(),I.destroy=m}c=c.next}while(c!==v)}}catch(H){Cn(i,i.return,H)}}function Xo(o,i,c){try{var m=i.updateQueue,v=m!==null?m.lastEffect:null;if(v!==null){var w=v.next;m=w;do{if((m.tag&o)===o){var I=m.inst,H=I.destroy;if(H!==void 0){I.destroy=void 0,v=i;var J=c,me=H;try{me()}catch(we){Cn(v,J,we)}}}m=m.next}while(m!==w)}}catch(we){Cn(i,i.return,we)}}function uC(o){var i=o.updateQueue;if(i!==null){var c=o.stateNode;try{Qb(i,c)}catch(m){Cn(o,o.return,m)}}}function fC(o,i,c){c.props=Di(o.type,o.memoizedProps),c.state=o.memoizedState;try{c.componentWillUnmount()}catch(m){Cn(o,i,m)}}function Zs(o,i){try{var c=o.ref;if(c!==null){switch(o.tag){case 26:case 27:case 5:var m=o.stateNode;break;case 30:m=o.stateNode;break;default:m=o.stateNode}typeof c=="function"?o.refCleanup=c(m):c.current=m}}catch(v){Cn(o,i,v)}}function Ka(o,i){var c=o.ref,m=o.refCleanup;if(c!==null)if(typeof m=="function")try{m()}catch(v){Cn(o,i,v)}finally{o.refCleanup=null,o=o.alternate,o!=null&&(o.refCleanup=null)}else if(typeof c=="function")try{c(null)}catch(v){Cn(o,i,v)}else c.current=null}function dC(o){var i=o.type,c=o.memoizedProps,m=o.stateNode;try{e:switch(i){case"button":case"input":case"select":case"textarea":c.autoFocus&&m.focus();break e;case"img":c.src?m.src=c.src:c.srcSet&&(m.srcset=c.srcSet)}}catch(v){Cn(o,o.return,v)}}function fh(o,i,c){try{var m=o.stateNode;Hj(m,o.type,c,i),m[Ve]=i}catch(v){Cn(o,o.return,v)}}function mC(o){return o.tag===5||o.tag===3||o.tag===26||o.tag===27&&ri(o.type)||o.tag===4}function dh(o){e:for(;;){for(;o.sibling===null;){if(o.return===null||mC(o.return))return null;o=o.return}for(o.sibling.return=o.return,o=o.sibling;o.tag!==5&&o.tag!==6&&o.tag!==18;){if(o.tag===27&&ri(o.type)||o.flags&2||o.child===null||o.tag===4)continue e;o.child.return=o,o=o.child}if(!(o.flags&2))return o.stateNode}}function mh(o,i,c){var m=o.tag;if(m===5||m===6)o=o.stateNode,i?(c.nodeType===9?c.body:c.nodeName==="HTML"?c.ownerDocument.body:c).insertBefore(o,i):(i=c.nodeType===9?c.body:c.nodeName==="HTML"?c.ownerDocument.body:c,i.appendChild(o),c=c._reactRootContainer,c!=null||i.onclick!==null||(i.onclick=Mf));else if(m!==4&&(m===27&&ri(o.type)&&(c=o.stateNode,i=null),o=o.child,o!==null))for(mh(o,i,c),o=o.sibling;o!==null;)mh(o,i,c),o=o.sibling}function bf(o,i,c){var m=o.tag;if(m===5||m===6)o=o.stateNode,i?c.insertBefore(o,i):c.appendChild(o);else if(m!==4&&(m===27&&ri(o.type)&&(c=o.stateNode),o=o.child,o!==null))for(bf(o,i,c),o=o.sibling;o!==null;)bf(o,i,c),o=o.sibling}function gC(o){var i=o.stateNode,c=o.memoizedProps;try{for(var m=o.type,v=i.attributes;v.length;)i.removeAttributeNode(v[0]);vr(i,m,c),i[Ct]=o,i[Ve]=c}catch(w){Cn(o,o.return,w)}}var vo=!1,Ln=!1,gh=!1,hC=typeof WeakSet=="function"?WeakSet:Set,ur=null;function Sj(o,i){if(o=o.containerInfo,Lh=Df,o=Ob(o),dg(o)){if("selectionStart"in o)var c={start:o.selectionStart,end:o.selectionEnd};else e:{c=(c=o.ownerDocument)&&c.defaultView||window;var m=c.getSelection&&c.getSelection();if(m&&m.rangeCount!==0){c=m.anchorNode;var v=m.anchorOffset,w=m.focusNode;m=m.focusOffset;try{c.nodeType,w.nodeType}catch{c=null;break e}var I=0,H=-1,J=-1,me=0,we=0,_e=o,pe=null;t:for(;;){for(var ye;_e!==c||v!==0&&_e.nodeType!==3||(H=I+v),_e!==w||m!==0&&_e.nodeType!==3||(J=I+m),_e.nodeType===3&&(I+=_e.nodeValue.length),(ye=_e.firstChild)!==null;)pe=_e,_e=ye;for(;;){if(_e===o)break t;if(pe===c&&++me===v&&(H=I),pe===w&&++we===m&&(J=I),(ye=_e.nextSibling)!==null)break;_e=pe,pe=_e.parentNode}_e=ye}c=H===-1||J===-1?null:{start:H,end:J}}else c=null}c=c||{start:0,end:0}}else c=null;for(Bh={focusedElem:o,selectionRange:c},Df=!1,ur=i;ur!==null;)if(i=ur,o=i.child,(i.subtreeFlags&1024)!==0&&o!==null)o.return=i,ur=o;else for(;ur!==null;){switch(i=ur,w=i.alternate,o=i.flags,i.tag){case 0:break;case 11:case 15:break;case 1:if((o&1024)!==0&&w!==null){o=void 0,c=i,v=w.memoizedProps,w=w.memoizedState,m=c.stateNode;try{var St=Di(c.type,v,c.elementType===c.type);o=m.getSnapshotBeforeUpdate(St,w),m.__reactInternalSnapshotBeforeUpdate=o}catch(gt){Cn(c,c.return,gt)}}break;case 3:if((o&1024)!==0){if(o=i.stateNode.containerInfo,c=o.nodeType,c===9)Vh(o);else if(c===1)switch(o.nodeName){case"HEAD":case"HTML":case"BODY":Vh(o);break;default:o.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((o&1024)!==0)throw Error(r(163))}if(o=i.sibling,o!==null){o.return=i.return,ur=o;break}ur=i.return}}function pC(o,i,c){var m=c.flags;switch(c.tag){case 0:case 11:case 15:Yo(o,c),m&4&&Qs(5,c);break;case 1:if(Yo(o,c),m&4)if(o=c.stateNode,i===null)try{o.componentDidMount()}catch(I){Cn(c,c.return,I)}else{var v=Di(c.type,i.memoizedProps);i=i.memoizedState;try{o.componentDidUpdate(v,i,o.__reactInternalSnapshotBeforeUpdate)}catch(I){Cn(c,c.return,I)}}m&64&&uC(c),m&512&&Zs(c,c.return);break;case 3:if(Yo(o,c),m&64&&(o=c.updateQueue,o!==null)){if(i=null,c.child!==null)switch(c.child.tag){case 27:case 5:i=c.child.stateNode;break;case 1:i=c.child.stateNode}try{Qb(o,i)}catch(I){Cn(c,c.return,I)}}break;case 27:i===null&&m&4&&gC(c);case 26:case 5:Yo(o,c),i===null&&m&4&&dC(c),m&512&&Zs(c,c.return);break;case 12:Yo(o,c);break;case 13:Yo(o,c),m&4&&bC(o,c),m&64&&(o=c.memoizedState,o!==null&&(o=o.dehydrated,o!==null&&(c=Tj.bind(null,c),Gj(o,c))));break;case 22:if(m=c.memoizedState!==null||vo,!m){i=i!==null&&i.memoizedState!==null||Ln,v=vo;var w=Ln;vo=m,(Ln=i)&&!w?Qo(o,c,(c.subtreeFlags&8772)!==0):Yo(o,c),vo=v,Ln=w}break;case 30:break;default:Yo(o,c)}}function vC(o){var i=o.alternate;i!==null&&(o.alternate=null,vC(i)),o.child=null,o.deletions=null,o.sibling=null,o.tag===5&&(i=o.stateNode,i!==null&&Mt(i)),o.stateNode=null,o.return=null,o.dependencies=null,o.memoizedProps=null,o.memoizedState=null,o.pendingProps=null,o.stateNode=null,o.updateQueue=null}var Tn=null,Ar=!1;function yo(o,i,c){for(c=c.child;c!==null;)yC(o,i,c),c=c.sibling}function yC(o,i,c){if(Ge&&typeof Ge.onCommitFiberUnmount=="function")try{Ge.onCommitFiberUnmount(We,c)}catch{}switch(c.tag){case 26:Ln||Ka(c,i),yo(o,i,c),c.memoizedState?c.memoizedState.count--:c.stateNode&&(c=c.stateNode,c.parentNode.removeChild(c));break;case 27:Ln||Ka(c,i);var m=Tn,v=Ar;ri(c.type)&&(Tn=c.stateNode,Ar=!1),yo(o,i,c),lc(c.stateNode),Tn=m,Ar=v;break;case 5:Ln||Ka(c,i);case 6:if(m=Tn,v=Ar,Tn=null,yo(o,i,c),Tn=m,Ar=v,Tn!==null)if(Ar)try{(Tn.nodeType===9?Tn.body:Tn.nodeName==="HTML"?Tn.ownerDocument.body:Tn).removeChild(c.stateNode)}catch(w){Cn(c,i,w)}else try{Tn.removeChild(c.stateNode)}catch(w){Cn(c,i,w)}break;case 18:Tn!==null&&(Ar?(o=Tn,i1(o.nodeType===9?o.body:o.nodeName==="HTML"?o.ownerDocument.body:o,c.stateNode),hc(o)):i1(Tn,c.stateNode));break;case 4:m=Tn,v=Ar,Tn=c.stateNode.containerInfo,Ar=!0,yo(o,i,c),Tn=m,Ar=v;break;case 0:case 11:case 14:case 15:Ln||Xo(2,c,i),Ln||Xo(4,c,i),yo(o,i,c);break;case 1:Ln||(Ka(c,i),m=c.stateNode,typeof m.componentWillUnmount=="function"&&fC(c,i,m)),yo(o,i,c);break;case 21:yo(o,i,c);break;case 22:Ln=(m=Ln)||c.memoizedState!==null,yo(o,i,c),Ln=m;break;default:yo(o,i,c)}}function bC(o,i){if(i.memoizedState===null&&(o=i.alternate,o!==null&&(o=o.memoizedState,o!==null&&(o=o.dehydrated,o!==null))))try{hc(o)}catch(c){Cn(i,i.return,c)}}function Cj(o){switch(o.tag){case 13:case 19:var i=o.stateNode;return i===null&&(i=o.stateNode=new hC),i;case 22:return o=o.stateNode,i=o._retryCache,i===null&&(i=o._retryCache=new hC),i;default:throw Error(r(435,o.tag))}}function hh(o,i){var c=Cj(o);i.forEach(function(m){var v=Mj.bind(null,o,m);c.has(m)||(c.add(m),m.then(v,v))})}function qr(o,i){var c=i.deletions;if(c!==null)for(var m=0;m<c.length;m++){var v=c[m],w=o,I=i,H=I;e:for(;H!==null;){switch(H.tag){case 27:if(ri(H.type)){Tn=H.stateNode,Ar=!1;break e}break;case 5:Tn=H.stateNode,Ar=!1;break e;case 3:case 4:Tn=H.stateNode.containerInfo,Ar=!0;break e}H=H.return}if(Tn===null)throw Error(r(160));yC(w,I,v),Tn=null,Ar=!1,w=v.alternate,w!==null&&(w.return=null),v.return=null}if(i.subtreeFlags&13878)for(i=i.child;i!==null;)SC(i,o),i=i.sibling}var _a=null;function SC(o,i){var c=o.alternate,m=o.flags;switch(o.tag){case 0:case 11:case 14:case 15:qr(i,o),Wr(o),m&4&&(Xo(3,o,o.return),Qs(3,o),Xo(5,o,o.return));break;case 1:qr(i,o),Wr(o),m&512&&(Ln||c===null||Ka(c,c.return)),m&64&&vo&&(o=o.updateQueue,o!==null&&(m=o.callbacks,m!==null&&(c=o.shared.hiddenCallbacks,o.shared.hiddenCallbacks=c===null?m:c.concat(m))));break;case 26:var v=_a;if(qr(i,o),Wr(o),m&512&&(Ln||c===null||Ka(c,c.return)),m&4){var w=c!==null?c.memoizedState:null;if(m=o.memoizedState,c===null)if(m===null)if(o.stateNode===null){e:{m=o.type,c=o.memoizedProps,v=v.ownerDocument||v;t:switch(m){case"title":w=v.getElementsByTagName("title")[0],(!w||w[qt]||w[Ct]||w.namespaceURI==="http://www.w3.org/2000/svg"||w.hasAttribute("itemprop"))&&(w=v.createElement(m),v.head.insertBefore(w,v.querySelector("head > title"))),vr(w,m,c),w[Ct]=o,Tt(w),m=w;break e;case"link":var I=g1("link","href",v).get(m+(c.href||""));if(I){for(var H=0;H<I.length;H++)if(w=I[H],w.getAttribute("href")===(c.href==null||c.href===""?null:c.href)&&w.getAttribute("rel")===(c.rel==null?null:c.rel)&&w.getAttribute("title")===(c.title==null?null:c.title)&&w.getAttribute("crossorigin")===(c.crossOrigin==null?null:c.crossOrigin)){I.splice(H,1);break t}}w=v.createElement(m),vr(w,m,c),v.head.appendChild(w);break;case"meta":if(I=g1("meta","content",v).get(m+(c.content||""))){for(H=0;H<I.length;H++)if(w=I[H],w.getAttribute("content")===(c.content==null?null:""+c.content)&&w.getAttribute("name")===(c.name==null?null:c.name)&&w.getAttribute("property")===(c.property==null?null:c.property)&&w.getAttribute("http-equiv")===(c.httpEquiv==null?null:c.httpEquiv)&&w.getAttribute("charset")===(c.charSet==null?null:c.charSet)){I.splice(H,1);break t}}w=v.createElement(m),vr(w,m,c),v.head.appendChild(w);break;default:throw Error(r(468,m))}w[Ct]=o,Tt(w),m=w}o.stateNode=m}else h1(v,o.type,o.stateNode);else o.stateNode=m1(v,m,o.memoizedProps);else w!==m?(w===null?c.stateNode!==null&&(c=c.stateNode,c.parentNode.removeChild(c)):w.count--,m===null?h1(v,o.type,o.stateNode):m1(v,m,o.memoizedProps)):m===null&&o.stateNode!==null&&fh(o,o.memoizedProps,c.memoizedProps)}break;case 27:qr(i,o),Wr(o),m&512&&(Ln||c===null||Ka(c,c.return)),c!==null&&m&4&&fh(o,o.memoizedProps,c.memoizedProps);break;case 5:if(qr(i,o),Wr(o),m&512&&(Ln||c===null||Ka(c,c.return)),o.flags&32){v=o.stateNode;try{jn(v,"")}catch(ye){Cn(o,o.return,ye)}}m&4&&o.stateNode!=null&&(v=o.memoizedProps,fh(o,v,c!==null?c.memoizedProps:v)),m&1024&&(gh=!0);break;case 6:if(qr(i,o),Wr(o),m&4){if(o.stateNode===null)throw Error(r(162));m=o.memoizedProps,c=o.stateNode;try{c.nodeValue=m}catch(ye){Cn(o,o.return,ye)}}break;case 3:if(Pf=null,v=_a,_a=jf(i.containerInfo),qr(i,o),_a=v,Wr(o),m&4&&c!==null&&c.memoizedState.isDehydrated)try{hc(i.containerInfo)}catch(ye){Cn(o,o.return,ye)}gh&&(gh=!1,CC(o));break;case 4:m=_a,_a=jf(o.stateNode.containerInfo),qr(i,o),Wr(o),_a=m;break;case 12:qr(i,o),Wr(o);break;case 13:qr(i,o),Wr(o),o.child.flags&8192&&o.memoizedState!==null!=(c!==null&&c.memoizedState!==null)&&(Ch=Ee()),m&4&&(m=o.updateQueue,m!==null&&(o.updateQueue=null,hh(o,m)));break;case 22:v=o.memoizedState!==null;var J=c!==null&&c.memoizedState!==null,me=vo,we=Ln;if(vo=me||v,Ln=we||J,qr(i,o),Ln=we,vo=me,Wr(o),m&8192)e:for(i=o.stateNode,i._visibility=v?i._visibility&-2:i._visibility|1,v&&(c===null||J||vo||Ln||Li(o)),c=null,i=o;;){if(i.tag===5||i.tag===26){if(c===null){J=c=i;try{if(w=J.stateNode,v)I=w.style,typeof I.setProperty=="function"?I.setProperty("display","none","important"):I.display="none";else{H=J.stateNode;var _e=J.memoizedProps.style,pe=_e!=null&&_e.hasOwnProperty("display")?_e.display:null;H.style.display=pe==null||typeof pe=="boolean"?"":(""+pe).trim()}}catch(ye){Cn(J,J.return,ye)}}}else if(i.tag===6){if(c===null){J=i;try{J.stateNode.nodeValue=v?"":J.memoizedProps}catch(ye){Cn(J,J.return,ye)}}}else if((i.tag!==22&&i.tag!==23||i.memoizedState===null||i===o)&&i.child!==null){i.child.return=i,i=i.child;continue}if(i===o)break e;for(;i.sibling===null;){if(i.return===null||i.return===o)break e;c===i&&(c=null),i=i.return}c===i&&(c=null),i.sibling.return=i.return,i=i.sibling}m&4&&(m=o.updateQueue,m!==null&&(c=m.retryQueue,c!==null&&(m.retryQueue=null,hh(o,c))));break;case 19:qr(i,o),Wr(o),m&4&&(m=o.updateQueue,m!==null&&(o.updateQueue=null,hh(o,m)));break;case 30:break;case 21:break;default:qr(i,o),Wr(o)}}function Wr(o){var i=o.flags;if(i&2){try{for(var c,m=o.return;m!==null;){if(mC(m)){c=m;break}m=m.return}if(c==null)throw Error(r(160));switch(c.tag){case 27:var v=c.stateNode,w=dh(o);bf(o,w,v);break;case 5:var I=c.stateNode;c.flags&32&&(jn(I,""),c.flags&=-33);var H=dh(o);bf(o,H,I);break;case 3:case 4:var J=c.stateNode.containerInfo,me=dh(o);mh(o,me,J);break;default:throw Error(r(161))}}catch(we){Cn(o,o.return,we)}o.flags&=-3}i&4096&&(o.flags&=-4097)}function CC(o){if(o.subtreeFlags&1024)for(o=o.child;o!==null;){var i=o;CC(i),i.tag===5&&i.flags&1024&&i.stateNode.reset(),o=o.sibling}}function Yo(o,i){if(i.subtreeFlags&8772)for(i=i.child;i!==null;)pC(o,i.alternate,i),i=i.sibling}function Li(o){for(o=o.child;o!==null;){var i=o;switch(i.tag){case 0:case 11:case 14:case 15:Xo(4,i,i.return),Li(i);break;case 1:Ka(i,i.return);var c=i.stateNode;typeof c.componentWillUnmount=="function"&&fC(i,i.return,c),Li(i);break;case 27:lc(i.stateNode);case 26:case 5:Ka(i,i.return),Li(i);break;case 22:i.memoizedState===null&&Li(i);break;case 30:Li(i);break;default:Li(i)}o=o.sibling}}function Qo(o,i,c){for(c=c&&(i.subtreeFlags&8772)!==0,i=i.child;i!==null;){var m=i.alternate,v=o,w=i,I=w.flags;switch(w.tag){case 0:case 11:case 15:Qo(v,w,c),Qs(4,w);break;case 1:if(Qo(v,w,c),m=w,v=m.stateNode,typeof v.componentDidMount=="function")try{v.componentDidMount()}catch(me){Cn(m,m.return,me)}if(m=w,v=m.updateQueue,v!==null){var H=m.stateNode;try{var J=v.shared.hiddenCallbacks;if(J!==null)for(v.shared.hiddenCallbacks=null,v=0;v<J.length;v++)Yb(J[v],H)}catch(me){Cn(m,m.return,me)}}c&&I&64&&uC(w),Zs(w,w.return);break;case 27:gC(w);case 26:case 5:Qo(v,w,c),c&&m===null&&I&4&&dC(w),Zs(w,w.return);break;case 12:Qo(v,w,c);break;case 13:Qo(v,w,c),c&&I&4&&bC(v,w);break;case 22:w.memoizedState===null&&Qo(v,w,c),Zs(w,w.return);break;case 30:break;default:Qo(v,w,c)}i=i.sibling}}function ph(o,i){var c=null;o!==null&&o.memoizedState!==null&&o.memoizedState.cachePool!==null&&(c=o.memoizedState.cachePool.pool),o=null,i.memoizedState!==null&&i.memoizedState.cachePool!==null&&(o=i.memoizedState.cachePool.pool),o!==c&&(o!=null&&o.refCount++,c!=null&&Ds(c))}function vh(o,i){o=null,i.alternate!==null&&(o=i.alternate.memoizedState.cache),i=i.memoizedState.cache,i!==o&&(i.refCount++,o!=null&&Ds(o))}function Xa(o,i,c,m){if(i.subtreeFlags&10256)for(i=i.child;i!==null;)xC(o,i,c,m),i=i.sibling}function xC(o,i,c,m){var v=i.flags;switch(i.tag){case 0:case 11:case 15:Xa(o,i,c,m),v&2048&&Qs(9,i);break;case 1:Xa(o,i,c,m);break;case 3:Xa(o,i,c,m),v&2048&&(o=null,i.alternate!==null&&(o=i.alternate.memoizedState.cache),i=i.memoizedState.cache,i!==o&&(i.refCount++,o!=null&&Ds(o)));break;case 12:if(v&2048){Xa(o,i,c,m),o=i.stateNode;try{var w=i.memoizedProps,I=w.id,H=w.onPostCommit;typeof H=="function"&&H(I,i.alternate===null?"mount":"update",o.passiveEffectDuration,-0)}catch(J){Cn(i,i.return,J)}}else Xa(o,i,c,m);break;case 13:Xa(o,i,c,m);break;case 23:break;case 22:w=i.stateNode,I=i.alternate,i.memoizedState!==null?w._visibility&2?Xa(o,i,c,m):Js(o,i):w._visibility&2?Xa(o,i,c,m):(w._visibility|=2,El(o,i,c,m,(i.subtreeFlags&10256)!==0)),v&2048&&ph(I,i);break;case 24:Xa(o,i,c,m),v&2048&&vh(i.alternate,i);break;default:Xa(o,i,c,m)}}function El(o,i,c,m,v){for(v=v&&(i.subtreeFlags&10256)!==0,i=i.child;i!==null;){var w=o,I=i,H=c,J=m,me=I.flags;switch(I.tag){case 0:case 11:case 15:El(w,I,H,J,v),Qs(8,I);break;case 23:break;case 22:var we=I.stateNode;I.memoizedState!==null?we._visibility&2?El(w,I,H,J,v):Js(w,I):(we._visibility|=2,El(w,I,H,J,v)),v&&me&2048&&ph(I.alternate,I);break;case 24:El(w,I,H,J,v),v&&me&2048&&vh(I.alternate,I);break;default:El(w,I,H,J,v)}i=i.sibling}}function Js(o,i){if(i.subtreeFlags&10256)for(i=i.child;i!==null;){var c=o,m=i,v=m.flags;switch(m.tag){case 22:Js(c,m),v&2048&&ph(m.alternate,m);break;case 24:Js(c,m),v&2048&&vh(m.alternate,m);break;default:Js(c,m)}i=i.sibling}}var ec=8192;function $l(o){if(o.subtreeFlags&ec)for(o=o.child;o!==null;)wC(o),o=o.sibling}function wC(o){switch(o.tag){case 26:$l(o),o.flags&ec&&o.memoizedState!==null&&iI(_a,o.memoizedState,o.memoizedProps);break;case 5:$l(o);break;case 3:case 4:var i=_a;_a=jf(o.stateNode.containerInfo),$l(o),_a=i;break;case 22:o.memoizedState===null&&(i=o.alternate,i!==null&&i.memoizedState!==null?(i=ec,ec=16777216,$l(o),ec=i):$l(o));break;default:$l(o)}}function EC(o){var i=o.alternate;if(i!==null&&(o=i.child,o!==null)){i.child=null;do i=o.sibling,o.sibling=null,o=i;while(o!==null)}}function tc(o){var i=o.deletions;if((o.flags&16)!==0){if(i!==null)for(var c=0;c<i.length;c++){var m=i[c];ur=m,OC(m,o)}EC(o)}if(o.subtreeFlags&10256)for(o=o.child;o!==null;)$C(o),o=o.sibling}function $C(o){switch(o.tag){case 0:case 11:case 15:tc(o),o.flags&2048&&Xo(9,o,o.return);break;case 3:tc(o);break;case 12:tc(o);break;case 22:var i=o.stateNode;o.memoizedState!==null&&i._visibility&2&&(o.return===null||o.return.tag!==13)?(i._visibility&=-3,Sf(o)):tc(o);break;default:tc(o)}}function Sf(o){var i=o.deletions;if((o.flags&16)!==0){if(i!==null)for(var c=0;c<i.length;c++){var m=i[c];ur=m,OC(m,o)}EC(o)}for(o=o.child;o!==null;){switch(i=o,i.tag){case 0:case 11:case 15:Xo(8,i,i.return),Sf(i);break;case 22:c=i.stateNode,c._visibility&2&&(c._visibility&=-3,Sf(i));break;default:Sf(i)}o=o.sibling}}function OC(o,i){for(;ur!==null;){var c=ur;switch(c.tag){case 0:case 11:case 15:Xo(8,c,i);break;case 23:case 22:if(c.memoizedState!==null&&c.memoizedState.cachePool!==null){var m=c.memoizedState.cachePool.pool;m!=null&&m.refCount++}break;case 24:Ds(c.memoizedState.cache)}if(m=c.child,m!==null)m.return=c,ur=m;else e:for(c=o;ur!==null;){m=ur;var v=m.sibling,w=m.return;if(vC(m),m===c){ur=null;break e}if(v!==null){v.return=w,ur=v;break e}ur=w}}}var xj={getCacheForType:function(o){var i=Cr(Zn),c=i.data.get(o);return c===void 0&&(c=o(),i.data.set(o,c)),c}},wj=typeof WeakMap=="function"?WeakMap:Map,un=0,$n=null,Lt=null,Xt=0,fn=0,Gr=null,Zo=!1,Ol=!1,yh=!1,bo=0,Pn=0,Jo=0,Bi=0,bh=0,ha=0,Rl=0,nc=null,zr=null,Sh=!1,Ch=0,Cf=1/0,xf=null,ei=null,pr=0,ti=null,_l=null,Tl=0,xh=0,wh=null,RC=null,rc=0,Eh=null;function Kr(){if((un&2)!==0&&Xt!==0)return Xt&-Xt;if(j.T!==null){var o=pl;return o!==0?o:Nh()}return Ke()}function _C(){ha===0&&(ha=(Xt&536870912)===0||nn?It():536870912);var o=ga.current;return o!==null&&(o.flags|=32),ha}function Xr(o,i,c){(o===$n&&(fn===2||fn===9)||o.cancelPendingCommit!==null)&&(Ml(o,0),ni(o,Xt,ha,!1)),Rn(o,c),((un&2)===0||o!==$n)&&(o===$n&&((un&2)===0&&(Bi|=c),Pn===4&&ni(o,Xt,ha,!1)),Ya(o))}function TC(o,i,c){if((un&6)!==0)throw Error(r(327));var m=!c&&(i&124)===0&&(i&o.expiredLanes)===0||qe(o,i),v=m?Oj(o,i):Rh(o,i,!0),w=m;do{if(v===0){Ol&&!m&&ni(o,i,0,!1);break}else{if(c=o.current.alternate,w&&!Ej(c)){v=Rh(o,i,!1),w=!1;continue}if(v===2){if(w=i,o.errorRecoveryDisabledLanes&w)var I=0;else I=o.pendingLanes&-536870913,I=I!==0?I:I&536870912?536870912:0;if(I!==0){i=I;e:{var H=o;v=nc;var J=H.current.memoizedState.isDehydrated;if(J&&(Ml(H,I).flags|=256),I=Rh(H,I,!1),I!==2){if(yh&&!J){H.errorRecoveryDisabledLanes|=w,Bi|=w,v=4;break e}w=zr,zr=v,w!==null&&(zr===null?zr=w:zr.push.apply(zr,w))}v=I}if(w=!1,v!==2)continue}}if(v===1){Ml(o,0),ni(o,i,0,!0);break}e:{switch(m=o,w=v,w){case 0:case 1:throw Error(r(345));case 4:if((i&4194048)!==i)break;case 6:ni(m,i,ha,!Zo);break e;case 2:zr=null;break;case 3:case 5:break;default:throw Error(r(329))}if((i&62914560)===i&&(v=Ch+300-Ee(),10<v)){if(ni(m,i,ha,!Zo),Ie(m,0,!0)!==0)break e;m.timeoutHandle=a1(MC.bind(null,m,c,zr,xf,Sh,i,ha,Bi,Rl,Zo,w,2,-0,0),v);break e}MC(m,c,zr,xf,Sh,i,ha,Bi,Rl,Zo,w,0,-0,0)}}break}while(!0);Ya(o)}function MC(o,i,c,m,v,w,I,H,J,me,we,_e,pe,ye){if(o.timeoutHandle=-1,_e=i.subtreeFlags,(_e&8192||(_e&16785408)===16785408)&&(uc={stylesheets:null,count:0,unsuspend:oI},wC(i),_e=lI(),_e!==null)){o.cancelPendingCommit=_e(DC.bind(null,o,i,w,c,m,v,I,H,J,we,1,pe,ye)),ni(o,w,I,!me);return}DC(o,i,w,c,m,v,I,H,J)}function Ej(o){for(var i=o;;){var c=i.tag;if((c===0||c===11||c===15)&&i.flags&16384&&(c=i.updateQueue,c!==null&&(c=c.stores,c!==null)))for(var m=0;m<c.length;m++){var v=c[m],w=v.getSnapshot;v=v.value;try{if(!Ur(w(),v))return!1}catch{return!1}}if(c=i.child,i.subtreeFlags&16384&&c!==null)c.return=i,i=c;else{if(i===o)break;for(;i.sibling===null;){if(i.return===null||i.return===o)return!0;i=i.return}i.sibling.return=i.return,i=i.sibling}}return!0}function ni(o,i,c,m){i&=~bh,i&=~Bi,o.suspendedLanes|=i,o.pingedLanes&=~i,m&&(o.warmLanes|=i),m=o.expirationTimes;for(var v=i;0<v;){var w=31-ke(v),I=1<<w;m[w]=-1,v&=~I}c!==0&&kt(o,c,i)}function wf(){return(un&6)===0?(ac(0),!1):!0}function $h(){if(Lt!==null){if(fn===0)var o=Lt.return;else o=Lt,uo=Ii=null,Vg(o),xl=null,Ks=0,o=Lt;for(;o!==null;)cC(o.alternate,o),o=o.return;Lt=null}}function Ml(o,i){var c=o.timeoutHandle;c!==-1&&(o.timeoutHandle=-1,Vj(c)),c=o.cancelPendingCommit,c!==null&&(o.cancelPendingCommit=null,c()),$h(),$n=o,Lt=c=lo(o.current,null),Xt=i,fn=0,Gr=null,Zo=!1,Ol=qe(o,i),yh=!1,Rl=ha=bh=Bi=Jo=Pn=0,zr=nc=null,Sh=!1,(i&8)!==0&&(i|=i&32);var m=o.entangledLanes;if(m!==0)for(o=o.entanglements,m&=i;0<m;){var v=31-ke(m),w=1<<v;i|=o[v],m&=~w}return bo=i,ku(),c}function NC(o,i){Nt=null,j.H=uf,i===Bs||i===Ju?(i=Kb(),fn=3):i===qb?(i=Kb(),fn=4):fn=i===XS?8:i!==null&&typeof i=="object"&&typeof i.then=="function"?6:1,Gr=i,Lt===null&&(Pn=1,hf(o,ua(i,o.current)))}function jC(){var o=j.H;return j.H=uf,o===null?uf:o}function IC(){var o=j.A;return j.A=xj,o}function Oh(){Pn=4,Zo||(Xt&4194048)!==Xt&&ga.current!==null||(Ol=!0),(Jo&134217727)===0&&(Bi&134217727)===0||$n===null||ni($n,Xt,ha,!1)}function Rh(o,i,c){var m=un;un|=2;var v=jC(),w=IC();($n!==o||Xt!==i)&&(xf=null,Ml(o,i)),i=!1;var I=Pn;e:do try{if(fn!==0&&Lt!==null){var H=Lt,J=Gr;switch(fn){case 8:$h(),I=6;break e;case 3:case 2:case 9:case 6:ga.current===null&&(i=!0);var me=fn;if(fn=0,Gr=null,Nl(o,H,J,me),c&&Ol){I=0;break e}break;default:me=fn,fn=0,Gr=null,Nl(o,H,J,me)}}$j(),I=Pn;break}catch(we){NC(o,we)}while(!0);return i&&o.shellSuspendCounter++,uo=Ii=null,un=m,j.H=v,j.A=w,Lt===null&&($n=null,Xt=0,ku()),I}function $j(){for(;Lt!==null;)PC(Lt)}function Oj(o,i){var c=un;un|=2;var m=jC(),v=IC();$n!==o||Xt!==i?(xf=null,Cf=Ee()+500,Ml(o,i)):Ol=qe(o,i);e:do try{if(fn!==0&&Lt!==null){i=Lt;var w=Gr;t:switch(fn){case 1:fn=0,Gr=null,Nl(o,i,w,1);break;case 2:case 9:if(Wb(w)){fn=0,Gr=null,AC(i);break}i=function(){fn!==2&&fn!==9||$n!==o||(fn=7),Ya(o)},w.then(i,i);break e;case 3:fn=7;break e;case 4:fn=5;break e;case 7:Wb(w)?(fn=0,Gr=null,AC(i)):(fn=0,Gr=null,Nl(o,i,w,7));break;case 5:var I=null;switch(Lt.tag){case 26:I=Lt.memoizedState;case 5:case 27:var H=Lt;if(!I||p1(I)){fn=0,Gr=null;var J=H.sibling;if(J!==null)Lt=J;else{var me=H.return;me!==null?(Lt=me,Ef(me)):Lt=null}break t}}fn=0,Gr=null,Nl(o,i,w,5);break;case 6:fn=0,Gr=null,Nl(o,i,w,6);break;case 8:$h(),Pn=6;break e;default:throw Error(r(462))}}Rj();break}catch(we){NC(o,we)}while(!0);return uo=Ii=null,j.H=m,j.A=v,un=c,Lt!==null?0:($n=null,Xt=0,ku(),Pn)}function Rj(){for(;Lt!==null&&!fe();)PC(Lt)}function PC(o){var i=lC(o.alternate,o,bo);o.memoizedProps=o.pendingProps,i===null?Ef(o):Lt=i}function AC(o){var i=o,c=i.alternate;switch(i.tag){case 15:case 0:i=tC(c,i,i.pendingProps,i.type,void 0,Xt);break;case 11:i=tC(c,i,i.pendingProps,i.type.render,i.ref,Xt);break;case 5:Vg(i);default:cC(c,i),i=Lt=zb(i,bo),i=lC(c,i,bo)}o.memoizedProps=o.pendingProps,i===null?Ef(o):Lt=i}function Nl(o,i,c,m){uo=Ii=null,Vg(i),xl=null,Ks=0;var v=i.return;try{if(pj(o,v,i,c,Xt)){Pn=1,hf(o,ua(c,o.current)),Lt=null;return}}catch(w){if(v!==null)throw Lt=v,w;Pn=1,hf(o,ua(c,o.current)),Lt=null;return}i.flags&32768?(nn||m===1?o=!0:Ol||(Xt&536870912)!==0?o=!1:(Zo=o=!0,(m===2||m===9||m===3||m===6)&&(m=ga.current,m!==null&&m.tag===13&&(m.flags|=16384))),zC(i,o)):Ef(i)}function Ef(o){var i=o;do{if((i.flags&32768)!==0){zC(i,Zo);return}o=i.return;var c=yj(i.alternate,i,bo);if(c!==null){Lt=c;return}if(i=i.sibling,i!==null){Lt=i;return}Lt=i=o}while(i!==null);Pn===0&&(Pn=5)}function zC(o,i){do{var c=bj(o.alternate,o);if(c!==null){c.flags&=32767,Lt=c;return}if(c=o.return,c!==null&&(c.flags|=32768,c.subtreeFlags=0,c.deletions=null),!i&&(o=o.sibling,o!==null)){Lt=o;return}Lt=o=c}while(o!==null);Pn=6,Lt=null}function DC(o,i,c,m,v,w,I,H,J){o.cancelPendingCommit=null;do $f();while(pr!==0);if((un&6)!==0)throw Error(r(327));if(i!==null){if(i===o.current)throw Error(r(177));if(w=i.lanes|i.childLanes,w|=vg,dn(o,c,w,I,H,J),o===$n&&(Lt=$n=null,Xt=0),_l=i,ti=o,Tl=c,xh=w,wh=v,RC=m,(i.subtreeFlags&10256)!==0||(i.flags&10256)!==0?(o.callbackNode=null,o.callbackPriority=0,Nj(Me,function(){return VC(),null})):(o.callbackNode=null,o.callbackPriority=0),m=(i.flags&13878)!==0,(i.subtreeFlags&13878)!==0||m){m=j.T,j.T=null,v=A.p,A.p=2,I=un,un|=4;try{Sj(o,i,c)}finally{un=I,A.p=v,j.T=m}}pr=1,LC(),BC(),HC()}}function LC(){if(pr===1){pr=0;var o=ti,i=_l,c=(i.flags&13878)!==0;if((i.subtreeFlags&13878)!==0||c){c=j.T,j.T=null;var m=A.p;A.p=2;var v=un;un|=4;try{SC(i,o);var w=Bh,I=Ob(o.containerInfo),H=w.focusedElem,J=w.selectionRange;if(I!==H&&H&&H.ownerDocument&&$b(H.ownerDocument.documentElement,H)){if(J!==null&&dg(H)){var me=J.start,we=J.end;if(we===void 0&&(we=me),"selectionStart"in H)H.selectionStart=me,H.selectionEnd=Math.min(we,H.value.length);else{var _e=H.ownerDocument||document,pe=_e&&_e.defaultView||window;if(pe.getSelection){var ye=pe.getSelection(),St=H.textContent.length,gt=Math.min(J.start,St),vn=J.end===void 0?gt:Math.min(J.end,St);!ye.extend&&gt>vn&&(I=vn,vn=gt,gt=I);var ce=Eb(H,gt),ne=Eb(H,vn);if(ce&&ne&&(ye.rangeCount!==1||ye.anchorNode!==ce.node||ye.anchorOffset!==ce.offset||ye.focusNode!==ne.node||ye.focusOffset!==ne.offset)){var de=_e.createRange();de.setStart(ce.node,ce.offset),ye.removeAllRanges(),gt>vn?(ye.addRange(de),ye.extend(ne.node,ne.offset)):(de.setEnd(ne.node,ne.offset),ye.addRange(de))}}}}for(_e=[],ye=H;ye=ye.parentNode;)ye.nodeType===1&&_e.push({element:ye,left:ye.scrollLeft,top:ye.scrollTop});for(typeof H.focus=="function"&&H.focus(),H=0;H<_e.length;H++){var $e=_e[H];$e.element.scrollLeft=$e.left,$e.element.scrollTop=$e.top}}Df=!!Lh,Bh=Lh=null}finally{un=v,A.p=m,j.T=c}}o.current=i,pr=2}}function BC(){if(pr===2){pr=0;var o=ti,i=_l,c=(i.flags&8772)!==0;if((i.subtreeFlags&8772)!==0||c){c=j.T,j.T=null;var m=A.p;A.p=2;var v=un;un|=4;try{pC(o,i.alternate,i)}finally{un=v,A.p=m,j.T=c}}pr=3}}function HC(){if(pr===4||pr===3){pr=0,Se();var o=ti,i=_l,c=Tl,m=RC;(i.subtreeFlags&10256)!==0||(i.flags&10256)!==0?pr=5:(pr=0,_l=ti=null,FC(o,o.pendingLanes));var v=o.pendingLanes;if(v===0&&(ei=null),nt(c),i=i.stateNode,Ge&&typeof Ge.onCommitFiberRoot=="function")try{Ge.onCommitFiberRoot(We,i,void 0,(i.current.flags&128)===128)}catch{}if(m!==null){i=j.T,v=A.p,A.p=2,j.T=null;try{for(var w=o.onRecoverableError,I=0;I<m.length;I++){var H=m[I];w(H.value,{componentStack:H.stack})}}finally{j.T=i,A.p=v}}(Tl&3)!==0&&$f(),Ya(o),v=o.pendingLanes,(c&4194090)!==0&&(v&42)!==0?o===Eh?rc++:(rc=0,Eh=o):rc=0,ac(0)}}function FC(o,i){(o.pooledCacheLanes&=i)===0&&(i=o.pooledCache,i!=null&&(o.pooledCache=null,Ds(i)))}function $f(o){return LC(),BC(),HC(),VC()}function VC(){if(pr!==5)return!1;var o=ti,i=xh;xh=0;var c=nt(Tl),m=j.T,v=A.p;try{A.p=32>c?32:c,j.T=null,c=wh,wh=null;var w=ti,I=Tl;if(pr=0,_l=ti=null,Tl=0,(un&6)!==0)throw Error(r(331));var H=un;if(un|=4,$C(w.current),xC(w,w.current,I,c),un=H,ac(0,!1),Ge&&typeof Ge.onPostCommitFiberRoot=="function")try{Ge.onPostCommitFiberRoot(We,w)}catch{}return!0}finally{A.p=v,j.T=m,FC(o,i)}}function UC(o,i,c){i=ua(c,i),i=nh(o.stateNode,i,2),o=qo(o,i,2),o!==null&&(Rn(o,2),Ya(o))}function Cn(o,i,c){if(o.tag===3)UC(o,o,c);else for(;i!==null;){if(i.tag===3){UC(i,o,c);break}else if(i.tag===1){var m=i.stateNode;if(typeof i.type.getDerivedStateFromError=="function"||typeof m.componentDidCatch=="function"&&(ei===null||!ei.has(m))){o=ua(c,o),c=GS(2),m=qo(i,c,2),m!==null&&(KS(c,m,i,o),Rn(m,2),Ya(m));break}}i=i.return}}function _h(o,i,c){var m=o.pingCache;if(m===null){m=o.pingCache=new wj;var v=new Set;m.set(i,v)}else v=m.get(i),v===void 0&&(v=new Set,m.set(i,v));v.has(c)||(yh=!0,v.add(c),o=_j.bind(null,o,i,c),i.then(o,o))}function _j(o,i,c){var m=o.pingCache;m!==null&&m.delete(i),o.pingedLanes|=o.suspendedLanes&c,o.warmLanes&=~c,$n===o&&(Xt&c)===c&&(Pn===4||Pn===3&&(Xt&62914560)===Xt&&300>Ee()-Ch?(un&2)===0&&Ml(o,0):bh|=c,Rl===Xt&&(Rl=0)),Ya(o)}function kC(o,i){i===0&&(i=Nn()),o=dl(o,i),o!==null&&(Rn(o,i),Ya(o))}function Tj(o){var i=o.memoizedState,c=0;i!==null&&(c=i.retryLane),kC(o,c)}function Mj(o,i){var c=0;switch(o.tag){case 13:var m=o.stateNode,v=o.memoizedState;v!==null&&(c=v.retryLane);break;case 19:m=o.stateNode;break;case 22:m=o.stateNode._retryCache;break;default:throw Error(r(314))}m!==null&&m.delete(i),kC(o,c)}function Nj(o,i){return be(o,i)}var Of=null,jl=null,Th=!1,Rf=!1,Mh=!1,Hi=0;function Ya(o){o!==jl&&o.next===null&&(jl===null?Of=jl=o:jl=jl.next=o),Rf=!0,Th||(Th=!0,Ij())}function ac(o,i){if(!Mh&&Rf){Mh=!0;do for(var c=!1,m=Of;m!==null;){if(o!==0){var v=m.pendingLanes;if(v===0)var w=0;else{var I=m.suspendedLanes,H=m.pingedLanes;w=(1<<31-ke(42|o)+1)-1,w&=v&~(I&~H),w=w&201326741?w&201326741|1:w?w|2:0}w!==0&&(c=!0,KC(m,w))}else w=Xt,w=Ie(m,m===$n?w:0,m.cancelPendingCommit!==null||m.timeoutHandle!==-1),(w&3)===0||qe(m,w)||(c=!0,KC(m,w));m=m.next}while(c);Mh=!1}}function jj(){qC()}function qC(){Rf=Th=!1;var o=0;Hi!==0&&(Fj()&&(o=Hi),Hi=0);for(var i=Ee(),c=null,m=Of;m!==null;){var v=m.next,w=WC(m,i);w===0?(m.next=null,c===null?Of=v:c.next=v,v===null&&(jl=c)):(c=m,(o!==0||(w&3)!==0)&&(Rf=!0)),m=v}ac(o)}function WC(o,i){for(var c=o.suspendedLanes,m=o.pingedLanes,v=o.expirationTimes,w=o.pendingLanes&-62914561;0<w;){var I=31-ke(w),H=1<<I,J=v[I];J===-1?((H&c)===0||(H&m)!==0)&&(v[I]=wt(H,i)):J<=i&&(o.expiredLanes|=H),w&=~H}if(i=$n,c=Xt,c=Ie(o,o===i?c:0,o.cancelPendingCommit!==null||o.timeoutHandle!==-1),m=o.callbackNode,c===0||o===i&&(fn===2||fn===9)||o.cancelPendingCommit!==null)return m!==null&&m!==null&&Re(m),o.callbackNode=null,o.callbackPriority=0;if((c&3)===0||qe(o,c)){if(i=c&-c,i===o.callbackPriority)return i;switch(m!==null&&Re(m),nt(c)){case 2:case 8:c=Ye;break;case 32:c=Me;break;case 268435456:c=xe;break;default:c=Me}return m=GC.bind(null,o),c=be(c,m),o.callbackPriority=i,o.callbackNode=c,i}return m!==null&&m!==null&&Re(m),o.callbackPriority=2,o.callbackNode=null,2}function GC(o,i){if(pr!==0&&pr!==5)return o.callbackNode=null,o.callbackPriority=0,null;var c=o.callbackNode;if($f()&&o.callbackNode!==c)return null;var m=Xt;return m=Ie(o,o===$n?m:0,o.cancelPendingCommit!==null||o.timeoutHandle!==-1),m===0?null:(TC(o,m,i),WC(o,Ee()),o.callbackNode!=null&&o.callbackNode===c?GC.bind(null,o):null)}function KC(o,i){if($f())return null;TC(o,i,!0)}function Ij(){Uj(function(){(un&6)!==0?be(Pe,jj):qC()})}function Nh(){return Hi===0&&(Hi=It()),Hi}function XC(o){return o==null||typeof o=="symbol"||typeof o=="boolean"?null:typeof o=="function"?o:ro(""+o)}function YC(o,i){var c=i.ownerDocument.createElement("input");return c.name=i.name,c.value=i.value,o.id&&c.setAttribute("form",o.id),i.parentNode.insertBefore(c,i),o=new FormData(o),c.parentNode.removeChild(c),o}function Pj(o,i,c,m,v){if(i==="submit"&&c&&c.stateNode===v){var w=XC((v[Ve]||null).action),I=m.submitter;I&&(i=(i=I[Ve]||null)?XC(i.formAction):I.getAttribute("formAction"),i!==null&&(w=i,I=null));var H=new Fu("action","action",null,m,v);o.push({event:H,listeners:[{instance:null,listener:function(){if(m.defaultPrevented){if(Hi!==0){var J=I?YC(v,I):new FormData(v);Qg(c,{pending:!0,data:J,method:v.method,action:w},null,J)}}else typeof w=="function"&&(H.preventDefault(),J=I?YC(v,I):new FormData(v),Qg(c,{pending:!0,data:J,method:v.method,action:w},w,J))},currentTarget:v}]})}}for(var jh=0;jh<pg.length;jh++){var Ih=pg[jh],Aj=Ih.toLowerCase(),zj=Ih[0].toUpperCase()+Ih.slice(1);Ra(Aj,"on"+zj)}Ra(Tb,"onAnimationEnd"),Ra(Mb,"onAnimationIteration"),Ra(Nb,"onAnimationStart"),Ra("dblclick","onDoubleClick"),Ra("focusin","onFocus"),Ra("focusout","onBlur"),Ra(JN,"onTransitionRun"),Ra(ej,"onTransitionStart"),Ra(tj,"onTransitionCancel"),Ra(jb,"onTransitionEnd"),Ae("onMouseEnter",["mouseout","mouseover"]),Ae("onMouseLeave",["mouseout","mouseover"]),Ae("onPointerEnter",["pointerout","pointerover"]),Ae("onPointerLeave",["pointerout","pointerover"]),cn("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),cn("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),cn("onBeforeInput",["compositionend","keypress","textInput","paste"]),cn("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),cn("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),cn("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var oc="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),Dj=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(oc));function QC(o,i){i=(i&4)!==0;for(var c=0;c<o.length;c++){var m=o[c],v=m.event;m=m.listeners;e:{var w=void 0;if(i)for(var I=m.length-1;0<=I;I--){var H=m[I],J=H.instance,me=H.currentTarget;if(H=H.listener,J!==w&&v.isPropagationStopped())break e;w=H,v.currentTarget=me;try{w(v)}catch(we){gf(we)}v.currentTarget=null,w=J}else for(I=0;I<m.length;I++){if(H=m[I],J=H.instance,me=H.currentTarget,H=H.listener,J!==w&&v.isPropagationStopped())break e;w=H,v.currentTarget=me;try{w(v)}catch(we){gf(we)}v.currentTarget=null,w=J}}}}function Bt(o,i){var c=i[rt];c===void 0&&(c=i[rt]=new Set);var m=o+"__bubble";c.has(m)||(ZC(i,o,2,!1),c.add(m))}function Ph(o,i,c){var m=0;i&&(m|=4),ZC(c,o,m,i)}var _f="_reactListening"+Math.random().toString(36).slice(2);function Ah(o){if(!o[_f]){o[_f]=!0,ar.forEach(function(c){c!=="selectionchange"&&(Dj.has(c)||Ph(c,!1,o),Ph(c,!0,o))});var i=o.nodeType===9?o:o.ownerDocument;i===null||i[_f]||(i[_f]=!0,Ph("selectionchange",!1,i))}}function ZC(o,i,c,m){switch(x1(i)){case 2:var v=uI;break;case 8:v=fI;break;default:v=Xh}c=v.bind(null,i,c,o),v=void 0,!mt||i!=="touchstart"&&i!=="touchmove"&&i!=="wheel"||(v=!0),m?v!==void 0?o.addEventListener(i,c,{capture:!0,passive:v}):o.addEventListener(i,c,!0):v!==void 0?o.addEventListener(i,c,{passive:v}):o.addEventListener(i,c,!1)}function zh(o,i,c,m,v){var w=m;if((i&1)===0&&(i&2)===0&&m!==null)e:for(;;){if(m===null)return;var I=m.tag;if(I===3||I===4){var H=m.stateNode.containerInfo;if(H===v)break;if(I===4)for(I=m.return;I!==null;){var J=I.tag;if((J===3||J===4)&&I.stateNode.containerInfo===v)return;I=I.return}for(;H!==null;){if(I=Pt(H),I===null)return;if(J=I.tag,J===5||J===6||J===26||J===27){m=w=I;continue e}H=H.parentNode}}m=m.return}il(function(){var me=w,we=sa(c),_e=[];e:{var pe=Ib.get(o);if(pe!==void 0){var ye=Fu,St=o;switch(o){case"keypress":if(Ei(c)===0)break e;case"keydown":case"keyup":ye=NN;break;case"focusin":St="focus",ye=lg;break;case"focusout":St="blur",ye=lg;break;case"beforeblur":case"afterblur":ye=lg;break;case"click":if(c.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":ye=sb;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":ye=bN;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":ye=PN;break;case Tb:case Mb:case Nb:ye=xN;break;case jb:ye=zN;break;case"scroll":case"scrollend":ye=vN;break;case"wheel":ye=LN;break;case"copy":case"cut":case"paste":ye=EN;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":ye=ub;break;case"toggle":case"beforetoggle":ye=HN}var gt=(i&4)!==0,vn=!gt&&(o==="scroll"||o==="scrollend"),ce=gt?pe!==null?pe+"Capture":null:pe;gt=[];for(var ne=me,de;ne!==null;){var $e=ne;if(de=$e.stateNode,$e=$e.tag,$e!==5&&$e!==26&&$e!==27||de===null||ce===null||($e=ao(ne,ce),$e!=null&&gt.push(ic(ne,$e,de))),vn)break;ne=ne.return}0<gt.length&&(pe=new ye(pe,St,null,c,we),_e.push({event:pe,listeners:gt}))}}if((i&7)===0){e:{if(pe=o==="mouseover"||o==="pointerover",ye=o==="mouseout"||o==="pointerout",pe&&c!==qa&&(St=c.relatedTarget||c.fromElement)&&(Pt(St)||St[ze]))break e;if((ye||pe)&&(pe=we.window===we?we:(pe=we.ownerDocument)?pe.defaultView||pe.parentWindow:window,ye?(St=c.relatedTarget||c.toElement,ye=me,St=St?Pt(St):null,St!==null&&(vn=l(St),gt=St.tag,St!==vn||gt!==5&&gt!==27&&gt!==6)&&(St=null)):(ye=null,St=me),ye!==St)){if(gt=sb,$e="onMouseLeave",ce="onMouseEnter",ne="mouse",(o==="pointerout"||o==="pointerover")&&(gt=ub,$e="onPointerLeave",ce="onPointerEnter",ne="pointer"),vn=ye==null?pe:tn(ye),de=St==null?pe:tn(St),pe=new gt($e,ne+"leave",ye,c,we),pe.target=vn,pe.relatedTarget=de,$e=null,Pt(we)===me&&(gt=new gt(ce,ne+"enter",St,c,we),gt.target=de,gt.relatedTarget=vn,$e=gt),vn=$e,ye&&St)t:{for(gt=ye,ce=St,ne=0,de=gt;de;de=Il(de))ne++;for(de=0,$e=ce;$e;$e=Il($e))de++;for(;0<ne-de;)gt=Il(gt),ne--;for(;0<de-ne;)ce=Il(ce),de--;for(;ne--;){if(gt===ce||ce!==null&&gt===ce.alternate)break t;gt=Il(gt),ce=Il(ce)}gt=null}else gt=null;ye!==null&&JC(_e,pe,ye,gt,!1),St!==null&&vn!==null&&JC(_e,vn,St,gt,!0)}}e:{if(pe=me?tn(me):window,ye=pe.nodeName&&pe.nodeName.toLowerCase(),ye==="select"||ye==="input"&&pe.type==="file")var it=yb;else if(pb(pe))if(bb)it=YN;else{it=KN;var zt=GN}else ye=pe.nodeName,!ye||ye.toLowerCase()!=="input"||pe.type!=="checkbox"&&pe.type!=="radio"?me&&Qn(me.elementType)&&(it=yb):it=XN;if(it&&(it=it(o,me))){vb(_e,it,c,we);break e}zt&&zt(o,pe,me),o==="focusout"&&me&&pe.type==="number"&&me.memoizedProps.value!=null&&Qt(pe,"number",pe.value)}switch(zt=me?tn(me):window,o){case"focusin":(pb(zt)||zt.contentEditable==="true")&&(cl=zt,mg=me,js=null);break;case"focusout":js=mg=cl=null;break;case"mousedown":gg=!0;break;case"contextmenu":case"mouseup":case"dragend":gg=!1,Rb(_e,c,we);break;case"selectionchange":if(ZN)break;case"keydown":case"keyup":Rb(_e,c,we)}var ft;if(cg)e:{switch(o){case"compositionstart":var ht="onCompositionStart";break e;case"compositionend":ht="onCompositionEnd";break e;case"compositionupdate":ht="onCompositionUpdate";break e}ht=void 0}else sl?gb(o,c)&&(ht="onCompositionEnd"):o==="keydown"&&c.keyCode===229&&(ht="onCompositionStart");ht&&(fb&&c.locale!=="ko"&&(sl||ht!=="onCompositionStart"?ht==="onCompositionEnd"&&sl&&(ft=ll()):(gn=we,En="value"in gn?gn.value:gn.textContent,sl=!0)),zt=Tf(me,ht),0<zt.length&&(ht=new cb(ht,o,null,c,we),_e.push({event:ht,listeners:zt}),ft?ht.data=ft:(ft=hb(c),ft!==null&&(ht.data=ft)))),(ft=VN?UN(o,c):kN(o,c))&&(ht=Tf(me,"onBeforeInput"),0<ht.length&&(zt=new cb("onBeforeInput","beforeinput",null,c,we),_e.push({event:zt,listeners:ht}),zt.data=ft)),Pj(_e,o,me,c,we)}QC(_e,i)})}function ic(o,i,c){return{instance:o,listener:i,currentTarget:c}}function Tf(o,i){for(var c=i+"Capture",m=[];o!==null;){var v=o,w=v.stateNode;if(v=v.tag,v!==5&&v!==26&&v!==27||w===null||(v=ao(o,c),v!=null&&m.unshift(ic(o,v,w)),v=ao(o,i),v!=null&&m.push(ic(o,v,w))),o.tag===3)return m;o=o.return}return[]}function Il(o){if(o===null)return null;do o=o.return;while(o&&o.tag!==5&&o.tag!==27);return o||null}function JC(o,i,c,m,v){for(var w=i._reactName,I=[];c!==null&&c!==m;){var H=c,J=H.alternate,me=H.stateNode;if(H=H.tag,J!==null&&J===m)break;H!==5&&H!==26&&H!==27||me===null||(J=me,v?(me=ao(c,w),me!=null&&I.unshift(ic(c,me,J))):v||(me=ao(c,w),me!=null&&I.push(ic(c,me,J)))),c=c.return}I.length!==0&&o.push({event:i,listeners:I})}var Lj=/\r\n?/g,Bj=/\u0000|\uFFFD/g;function e1(o){return(typeof o=="string"?o:""+o).replace(Lj,`
`).replace(Bj,"")}function t1(o,i){return i=e1(i),e1(o)===i}function Mf(){}function pn(o,i,c,m,v,w){switch(c){case"children":typeof m=="string"?i==="body"||i==="textarea"&&m===""||jn(o,m):(typeof m=="number"||typeof m=="bigint")&&i!=="body"&&jn(o,""+m);break;case"className":zn(o,"class",m);break;case"tabIndex":zn(o,"tabindex",m);break;case"dir":case"role":case"viewBox":case"width":case"height":zn(o,c,m);break;case"style":_r(o,m,w);break;case"data":if(i!=="object"){zn(o,"data",m);break}case"src":case"href":if(m===""&&(i!=="a"||c!=="href")){o.removeAttribute(c);break}if(m==null||typeof m=="function"||typeof m=="symbol"||typeof m=="boolean"){o.removeAttribute(c);break}m=ro(""+m),o.setAttribute(c,m);break;case"action":case"formAction":if(typeof m=="function"){o.setAttribute(c,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof w=="function"&&(c==="formAction"?(i!=="input"&&pn(o,i,"name",v.name,v,null),pn(o,i,"formEncType",v.formEncType,v,null),pn(o,i,"formMethod",v.formMethod,v,null),pn(o,i,"formTarget",v.formTarget,v,null)):(pn(o,i,"encType",v.encType,v,null),pn(o,i,"method",v.method,v,null),pn(o,i,"target",v.target,v,null)));if(m==null||typeof m=="symbol"||typeof m=="boolean"){o.removeAttribute(c);break}m=ro(""+m),o.setAttribute(c,m);break;case"onClick":m!=null&&(o.onclick=Mf);break;case"onScroll":m!=null&&Bt("scroll",o);break;case"onScrollEnd":m!=null&&Bt("scrollend",o);break;case"dangerouslySetInnerHTML":if(m!=null){if(typeof m!="object"||!("__html"in m))throw Error(r(61));if(c=m.__html,c!=null){if(v.children!=null)throw Error(r(60));o.innerHTML=c}}break;case"multiple":o.multiple=m&&typeof m!="function"&&typeof m!="symbol";break;case"muted":o.muted=m&&typeof m!="function"&&typeof m!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(m==null||typeof m=="function"||typeof m=="boolean"||typeof m=="symbol"){o.removeAttribute("xlink:href");break}c=ro(""+m),o.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",c);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":m!=null&&typeof m!="function"&&typeof m!="symbol"?o.setAttribute(c,""+m):o.removeAttribute(c);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":m&&typeof m!="function"&&typeof m!="symbol"?o.setAttribute(c,""):o.removeAttribute(c);break;case"capture":case"download":m===!0?o.setAttribute(c,""):m!==!1&&m!=null&&typeof m!="function"&&typeof m!="symbol"?o.setAttribute(c,m):o.removeAttribute(c);break;case"cols":case"rows":case"size":case"span":m!=null&&typeof m!="function"&&typeof m!="symbol"&&!isNaN(m)&&1<=m?o.setAttribute(c,m):o.removeAttribute(c);break;case"rowSpan":case"start":m==null||typeof m=="function"||typeof m=="symbol"||isNaN(m)?o.removeAttribute(c):o.setAttribute(c,m);break;case"popover":Bt("beforetoggle",o),Bt("toggle",o),Kn(o,"popover",m);break;case"xlinkActuate":Fn(o,"http://www.w3.org/1999/xlink","xlink:actuate",m);break;case"xlinkArcrole":Fn(o,"http://www.w3.org/1999/xlink","xlink:arcrole",m);break;case"xlinkRole":Fn(o,"http://www.w3.org/1999/xlink","xlink:role",m);break;case"xlinkShow":Fn(o,"http://www.w3.org/1999/xlink","xlink:show",m);break;case"xlinkTitle":Fn(o,"http://www.w3.org/1999/xlink","xlink:title",m);break;case"xlinkType":Fn(o,"http://www.w3.org/1999/xlink","xlink:type",m);break;case"xmlBase":Fn(o,"http://www.w3.org/XML/1998/namespace","xml:base",m);break;case"xmlLang":Fn(o,"http://www.w3.org/XML/1998/namespace","xml:lang",m);break;case"xmlSpace":Fn(o,"http://www.w3.org/XML/1998/namespace","xml:space",m);break;case"is":Kn(o,"is",m);break;case"innerText":case"textContent":break;default:(!(2<c.length)||c[0]!=="o"&&c[0]!=="O"||c[1]!=="n"&&c[1]!=="N")&&(c=$s.get(c)||c,Kn(o,c,m))}}function Dh(o,i,c,m,v,w){switch(c){case"style":_r(o,m,w);break;case"dangerouslySetInnerHTML":if(m!=null){if(typeof m!="object"||!("__html"in m))throw Error(r(61));if(c=m.__html,c!=null){if(v.children!=null)throw Error(r(60));o.innerHTML=c}}break;case"children":typeof m=="string"?jn(o,m):(typeof m=="number"||typeof m=="bigint")&&jn(o,""+m);break;case"onScroll":m!=null&&Bt("scroll",o);break;case"onScrollEnd":m!=null&&Bt("scrollend",o);break;case"onClick":m!=null&&(o.onclick=Mf);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!Sn.hasOwnProperty(c))e:{if(c[0]==="o"&&c[1]==="n"&&(v=c.endsWith("Capture"),i=c.slice(2,v?c.length-7:void 0),w=o[Ve]||null,w=w!=null?w[c]:null,typeof w=="function"&&o.removeEventListener(i,w,v),typeof m=="function")){typeof w!="function"&&w!==null&&(c in o?o[c]=null:o.hasAttribute(c)&&o.removeAttribute(c)),o.addEventListener(i,m,v);break e}c in o?o[c]=m:m===!0?o.setAttribute(c,""):Kn(o,c,m)}}}function vr(o,i,c){switch(i){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":Bt("error",o),Bt("load",o);var m=!1,v=!1,w;for(w in c)if(c.hasOwnProperty(w)){var I=c[w];if(I!=null)switch(w){case"src":m=!0;break;case"srcSet":v=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(r(137,i));default:pn(o,i,w,I,c,null)}}v&&pn(o,i,"srcSet",c.srcSet,c,null),m&&pn(o,i,"src",c.src,c,null);return;case"input":Bt("invalid",o);var H=w=I=v=null,J=null,me=null;for(m in c)if(c.hasOwnProperty(m)){var we=c[m];if(we!=null)switch(m){case"name":v=we;break;case"type":I=we;break;case"checked":J=we;break;case"defaultChecked":me=we;break;case"value":w=we;break;case"defaultValue":H=we;break;case"children":case"dangerouslySetInnerHTML":if(we!=null)throw Error(r(137,i));break;default:pn(o,i,m,we,c,null)}}Wt(o,w,H,J,me,I,v,!1),Yn(o);return;case"select":Bt("invalid",o),m=I=w=null;for(v in c)if(c.hasOwnProperty(v)&&(H=c[v],H!=null))switch(v){case"value":w=H;break;case"defaultValue":I=H;break;case"multiple":m=H;default:pn(o,i,v,H,c,null)}i=w,c=I,o.multiple=!!m,i!=null?Zt(o,!!m,i,!1):c!=null&&Zt(o,!!m,c,!0);return;case"textarea":Bt("invalid",o),w=v=m=null;for(I in c)if(c.hasOwnProperty(I)&&(H=c[I],H!=null))switch(I){case"value":m=H;break;case"defaultValue":v=H;break;case"children":w=H;break;case"dangerouslySetInnerHTML":if(H!=null)throw Error(r(91));break;default:pn(o,i,I,H,c,null)}Vn(o,m,v,w),Yn(o);return;case"option":for(J in c)if(c.hasOwnProperty(J)&&(m=c[J],m!=null))switch(J){case"selected":o.selected=m&&typeof m!="function"&&typeof m!="symbol";break;default:pn(o,i,J,m,c,null)}return;case"dialog":Bt("beforetoggle",o),Bt("toggle",o),Bt("cancel",o),Bt("close",o);break;case"iframe":case"object":Bt("load",o);break;case"video":case"audio":for(m=0;m<oc.length;m++)Bt(oc[m],o);break;case"image":Bt("error",o),Bt("load",o);break;case"details":Bt("toggle",o);break;case"embed":case"source":case"link":Bt("error",o),Bt("load",o);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(me in c)if(c.hasOwnProperty(me)&&(m=c[me],m!=null))switch(me){case"children":case"dangerouslySetInnerHTML":throw Error(r(137,i));default:pn(o,i,me,m,c,null)}return;default:if(Qn(i)){for(we in c)c.hasOwnProperty(we)&&(m=c[we],m!==void 0&&Dh(o,i,we,m,c,void 0));return}}for(H in c)c.hasOwnProperty(H)&&(m=c[H],m!=null&&pn(o,i,H,m,c,null))}function Hj(o,i,c,m){switch(i){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var v=null,w=null,I=null,H=null,J=null,me=null,we=null;for(ye in c){var _e=c[ye];if(c.hasOwnProperty(ye)&&_e!=null)switch(ye){case"checked":break;case"value":break;case"defaultValue":J=_e;default:m.hasOwnProperty(ye)||pn(o,i,ye,null,m,_e)}}for(var pe in m){var ye=m[pe];if(_e=c[pe],m.hasOwnProperty(pe)&&(ye!=null||_e!=null))switch(pe){case"type":w=ye;break;case"name":v=ye;break;case"checked":me=ye;break;case"defaultChecked":we=ye;break;case"value":I=ye;break;case"defaultValue":H=ye;break;case"children":case"dangerouslySetInnerHTML":if(ye!=null)throw Error(r(137,i));break;default:ye!==_e&&pn(o,i,pe,ye,m,_e)}}Ft(o,I,H,J,me,we,w,v);return;case"select":ye=I=H=pe=null;for(w in c)if(J=c[w],c.hasOwnProperty(w)&&J!=null)switch(w){case"value":break;case"multiple":ye=J;default:m.hasOwnProperty(w)||pn(o,i,w,null,m,J)}for(v in m)if(w=m[v],J=c[v],m.hasOwnProperty(v)&&(w!=null||J!=null))switch(v){case"value":pe=w;break;case"defaultValue":H=w;break;case"multiple":I=w;default:w!==J&&pn(o,i,v,w,m,J)}i=H,c=I,m=ye,pe!=null?Zt(o,!!c,pe,!1):!!m!=!!c&&(i!=null?Zt(o,!!c,i,!0):Zt(o,!!c,c?[]:"",!1));return;case"textarea":ye=pe=null;for(H in c)if(v=c[H],c.hasOwnProperty(H)&&v!=null&&!m.hasOwnProperty(H))switch(H){case"value":break;case"children":break;default:pn(o,i,H,null,m,v)}for(I in m)if(v=m[I],w=c[I],m.hasOwnProperty(I)&&(v!=null||w!=null))switch(I){case"value":pe=v;break;case"defaultValue":ye=v;break;case"children":break;case"dangerouslySetInnerHTML":if(v!=null)throw Error(r(91));break;default:v!==w&&pn(o,i,I,v,m,w)}ir(o,pe,ye);return;case"option":for(var St in c)if(pe=c[St],c.hasOwnProperty(St)&&pe!=null&&!m.hasOwnProperty(St))switch(St){case"selected":o.selected=!1;break;default:pn(o,i,St,null,m,pe)}for(J in m)if(pe=m[J],ye=c[J],m.hasOwnProperty(J)&&pe!==ye&&(pe!=null||ye!=null))switch(J){case"selected":o.selected=pe&&typeof pe!="function"&&typeof pe!="symbol";break;default:pn(o,i,J,pe,m,ye)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var gt in c)pe=c[gt],c.hasOwnProperty(gt)&&pe!=null&&!m.hasOwnProperty(gt)&&pn(o,i,gt,null,m,pe);for(me in m)if(pe=m[me],ye=c[me],m.hasOwnProperty(me)&&pe!==ye&&(pe!=null||ye!=null))switch(me){case"children":case"dangerouslySetInnerHTML":if(pe!=null)throw Error(r(137,i));break;default:pn(o,i,me,pe,m,ye)}return;default:if(Qn(i)){for(var vn in c)pe=c[vn],c.hasOwnProperty(vn)&&pe!==void 0&&!m.hasOwnProperty(vn)&&Dh(o,i,vn,void 0,m,pe);for(we in m)pe=m[we],ye=c[we],!m.hasOwnProperty(we)||pe===ye||pe===void 0&&ye===void 0||Dh(o,i,we,pe,m,ye);return}}for(var ce in c)pe=c[ce],c.hasOwnProperty(ce)&&pe!=null&&!m.hasOwnProperty(ce)&&pn(o,i,ce,null,m,pe);for(_e in m)pe=m[_e],ye=c[_e],!m.hasOwnProperty(_e)||pe===ye||pe==null&&ye==null||pn(o,i,_e,pe,m,ye)}var Lh=null,Bh=null;function Nf(o){return o.nodeType===9?o:o.ownerDocument}function n1(o){switch(o){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function r1(o,i){if(o===0)switch(i){case"svg":return 1;case"math":return 2;default:return 0}return o===1&&i==="foreignObject"?0:o}function Hh(o,i){return o==="textarea"||o==="noscript"||typeof i.children=="string"||typeof i.children=="number"||typeof i.children=="bigint"||typeof i.dangerouslySetInnerHTML=="object"&&i.dangerouslySetInnerHTML!==null&&i.dangerouslySetInnerHTML.__html!=null}var Fh=null;function Fj(){var o=window.event;return o&&o.type==="popstate"?o===Fh?!1:(Fh=o,!0):(Fh=null,!1)}var a1=typeof setTimeout=="function"?setTimeout:void 0,Vj=typeof clearTimeout=="function"?clearTimeout:void 0,o1=typeof Promise=="function"?Promise:void 0,Uj=typeof queueMicrotask=="function"?queueMicrotask:typeof o1<"u"?function(o){return o1.resolve(null).then(o).catch(kj)}:a1;function kj(o){setTimeout(function(){throw o})}function ri(o){return o==="head"}function i1(o,i){var c=i,m=0,v=0;do{var w=c.nextSibling;if(o.removeChild(c),w&&w.nodeType===8)if(c=w.data,c==="/$"){if(0<m&&8>m){c=m;var I=o.ownerDocument;if(c&1&&lc(I.documentElement),c&2&&lc(I.body),c&4)for(c=I.head,lc(c),I=c.firstChild;I;){var H=I.nextSibling,J=I.nodeName;I[qt]||J==="SCRIPT"||J==="STYLE"||J==="LINK"&&I.rel.toLowerCase()==="stylesheet"||c.removeChild(I),I=H}}if(v===0){o.removeChild(w),hc(i);return}v--}else c==="$"||c==="$?"||c==="$!"?v++:m=c.charCodeAt(0)-48;else m=0;c=w}while(c);hc(i)}function Vh(o){var i=o.firstChild;for(i&&i.nodeType===10&&(i=i.nextSibling);i;){var c=i;switch(i=i.nextSibling,c.nodeName){case"HTML":case"HEAD":case"BODY":Vh(c),Mt(c);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(c.rel.toLowerCase()==="stylesheet")continue}o.removeChild(c)}}function qj(o,i,c,m){for(;o.nodeType===1;){var v=c;if(o.nodeName.toLowerCase()!==i.toLowerCase()){if(!m&&(o.nodeName!=="INPUT"||o.type!=="hidden"))break}else if(m){if(!o[qt])switch(i){case"meta":if(!o.hasAttribute("itemprop"))break;return o;case"link":if(w=o.getAttribute("rel"),w==="stylesheet"&&o.hasAttribute("data-precedence"))break;if(w!==v.rel||o.getAttribute("href")!==(v.href==null||v.href===""?null:v.href)||o.getAttribute("crossorigin")!==(v.crossOrigin==null?null:v.crossOrigin)||o.getAttribute("title")!==(v.title==null?null:v.title))break;return o;case"style":if(o.hasAttribute("data-precedence"))break;return o;case"script":if(w=o.getAttribute("src"),(w!==(v.src==null?null:v.src)||o.getAttribute("type")!==(v.type==null?null:v.type)||o.getAttribute("crossorigin")!==(v.crossOrigin==null?null:v.crossOrigin))&&w&&o.hasAttribute("async")&&!o.hasAttribute("itemprop"))break;return o;default:return o}}else if(i==="input"&&o.type==="hidden"){var w=v.name==null?null:""+v.name;if(v.type==="hidden"&&o.getAttribute("name")===w)return o}else return o;if(o=Ta(o.nextSibling),o===null)break}return null}function Wj(o,i,c){if(i==="")return null;for(;o.nodeType!==3;)if((o.nodeType!==1||o.nodeName!=="INPUT"||o.type!=="hidden")&&!c||(o=Ta(o.nextSibling),o===null))return null;return o}function Uh(o){return o.data==="$!"||o.data==="$?"&&o.ownerDocument.readyState==="complete"}function Gj(o,i){var c=o.ownerDocument;if(o.data!=="$?"||c.readyState==="complete")i();else{var m=function(){i(),c.removeEventListener("DOMContentLoaded",m)};c.addEventListener("DOMContentLoaded",m),o._reactRetry=m}}function Ta(o){for(;o!=null;o=o.nextSibling){var i=o.nodeType;if(i===1||i===3)break;if(i===8){if(i=o.data,i==="$"||i==="$!"||i==="$?"||i==="F!"||i==="F")break;if(i==="/$")return null}}return o}var kh=null;function l1(o){o=o.previousSibling;for(var i=0;o;){if(o.nodeType===8){var c=o.data;if(c==="$"||c==="$!"||c==="$?"){if(i===0)return o;i--}else c==="/$"&&i++}o=o.previousSibling}return null}function s1(o,i,c){switch(i=Nf(c),o){case"html":if(o=i.documentElement,!o)throw Error(r(452));return o;case"head":if(o=i.head,!o)throw Error(r(453));return o;case"body":if(o=i.body,!o)throw Error(r(454));return o;default:throw Error(r(451))}}function lc(o){for(var i=o.attributes;i.length;)o.removeAttributeNode(i[0]);Mt(o)}var pa=new Map,c1=new Set;function jf(o){return typeof o.getRootNode=="function"?o.getRootNode():o.nodeType===9?o:o.ownerDocument}var So=A.d;A.d={f:Kj,r:Xj,D:Yj,C:Qj,L:Zj,m:Jj,X:tI,S:eI,M:nI};function Kj(){var o=So.f(),i=wf();return o||i}function Xj(o){var i=At(o);i!==null&&i.tag===5&&i.type==="form"?TS(i):So.r(o)}var Pl=typeof document>"u"?null:document;function u1(o,i,c){var m=Pl;if(m&&typeof i=="string"&&i){var v=He(i);v='link[rel="'+o+'"][href="'+v+'"]',typeof c=="string"&&(v+='[crossorigin="'+c+'"]'),c1.has(v)||(c1.add(v),o={rel:o,crossOrigin:c,href:i},m.querySelector(v)===null&&(i=m.createElement("link"),vr(i,"link",o),Tt(i),m.head.appendChild(i)))}}function Yj(o){So.D(o),u1("dns-prefetch",o,null)}function Qj(o,i){So.C(o,i),u1("preconnect",o,i)}function Zj(o,i,c){So.L(o,i,c);var m=Pl;if(m&&o&&i){var v='link[rel="preload"][as="'+He(i)+'"]';i==="image"&&c&&c.imageSrcSet?(v+='[imagesrcset="'+He(c.imageSrcSet)+'"]',typeof c.imageSizes=="string"&&(v+='[imagesizes="'+He(c.imageSizes)+'"]')):v+='[href="'+He(o)+'"]';var w=v;switch(i){case"style":w=Al(o);break;case"script":w=zl(o)}pa.has(w)||(o=h({rel:"preload",href:i==="image"&&c&&c.imageSrcSet?void 0:o,as:i},c),pa.set(w,o),m.querySelector(v)!==null||i==="style"&&m.querySelector(sc(w))||i==="script"&&m.querySelector(cc(w))||(i=m.createElement("link"),vr(i,"link",o),Tt(i),m.head.appendChild(i)))}}function Jj(o,i){So.m(o,i);var c=Pl;if(c&&o){var m=i&&typeof i.as=="string"?i.as:"script",v='link[rel="modulepreload"][as="'+He(m)+'"][href="'+He(o)+'"]',w=v;switch(m){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":w=zl(o)}if(!pa.has(w)&&(o=h({rel:"modulepreload",href:o},i),pa.set(w,o),c.querySelector(v)===null)){switch(m){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(c.querySelector(cc(w)))return}m=c.createElement("link"),vr(m,"link",o),Tt(m),c.head.appendChild(m)}}}function eI(o,i,c){So.S(o,i,c);var m=Pl;if(m&&o){var v=sn(m).hoistableStyles,w=Al(o);i=i||"default";var I=v.get(w);if(!I){var H={loading:0,preload:null};if(I=m.querySelector(sc(w)))H.loading=5;else{o=h({rel:"stylesheet",href:o,"data-precedence":i},c),(c=pa.get(w))&&qh(o,c);var J=I=m.createElement("link");Tt(J),vr(J,"link",o),J._p=new Promise(function(me,we){J.onload=me,J.onerror=we}),J.addEventListener("load",function(){H.loading|=1}),J.addEventListener("error",function(){H.loading|=2}),H.loading|=4,If(I,i,m)}I={type:"stylesheet",instance:I,count:1,state:H},v.set(w,I)}}}function tI(o,i){So.X(o,i);var c=Pl;if(c&&o){var m=sn(c).hoistableScripts,v=zl(o),w=m.get(v);w||(w=c.querySelector(cc(v)),w||(o=h({src:o,async:!0},i),(i=pa.get(v))&&Wh(o,i),w=c.createElement("script"),Tt(w),vr(w,"link",o),c.head.appendChild(w)),w={type:"script",instance:w,count:1,state:null},m.set(v,w))}}function nI(o,i){So.M(o,i);var c=Pl;if(c&&o){var m=sn(c).hoistableScripts,v=zl(o),w=m.get(v);w||(w=c.querySelector(cc(v)),w||(o=h({src:o,async:!0,type:"module"},i),(i=pa.get(v))&&Wh(o,i),w=c.createElement("script"),Tt(w),vr(w,"link",o),c.head.appendChild(w)),w={type:"script",instance:w,count:1,state:null},m.set(v,w))}}function f1(o,i,c,m){var v=(v=ee.current)?jf(v):null;if(!v)throw Error(r(446));switch(o){case"meta":case"title":return null;case"style":return typeof c.precedence=="string"&&typeof c.href=="string"?(i=Al(c.href),c=sn(v).hoistableStyles,m=c.get(i),m||(m={type:"style",instance:null,count:0,state:null},c.set(i,m)),m):{type:"void",instance:null,count:0,state:null};case"link":if(c.rel==="stylesheet"&&typeof c.href=="string"&&typeof c.precedence=="string"){o=Al(c.href);var w=sn(v).hoistableStyles,I=w.get(o);if(I||(v=v.ownerDocument||v,I={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},w.set(o,I),(w=v.querySelector(sc(o)))&&!w._p&&(I.instance=w,I.state.loading=5),pa.has(o)||(c={rel:"preload",as:"style",href:c.href,crossOrigin:c.crossOrigin,integrity:c.integrity,media:c.media,hrefLang:c.hrefLang,referrerPolicy:c.referrerPolicy},pa.set(o,c),w||rI(v,o,c,I.state))),i&&m===null)throw Error(r(528,""));return I}if(i&&m!==null)throw Error(r(529,""));return null;case"script":return i=c.async,c=c.src,typeof c=="string"&&i&&typeof i!="function"&&typeof i!="symbol"?(i=zl(c),c=sn(v).hoistableScripts,m=c.get(i),m||(m={type:"script",instance:null,count:0,state:null},c.set(i,m)),m):{type:"void",instance:null,count:0,state:null};default:throw Error(r(444,o))}}function Al(o){return'href="'+He(o)+'"'}function sc(o){return'link[rel="stylesheet"]['+o+"]"}function d1(o){return h({},o,{"data-precedence":o.precedence,precedence:null})}function rI(o,i,c,m){o.querySelector('link[rel="preload"][as="style"]['+i+"]")?m.loading=1:(i=o.createElement("link"),m.preload=i,i.addEventListener("load",function(){return m.loading|=1}),i.addEventListener("error",function(){return m.loading|=2}),vr(i,"link",c),Tt(i),o.head.appendChild(i))}function zl(o){return'[src="'+He(o)+'"]'}function cc(o){return"script[async]"+o}function m1(o,i,c){if(i.count++,i.instance===null)switch(i.type){case"style":var m=o.querySelector('style[data-href~="'+He(c.href)+'"]');if(m)return i.instance=m,Tt(m),m;var v=h({},c,{"data-href":c.href,"data-precedence":c.precedence,href:null,precedence:null});return m=(o.ownerDocument||o).createElement("style"),Tt(m),vr(m,"style",v),If(m,c.precedence,o),i.instance=m;case"stylesheet":v=Al(c.href);var w=o.querySelector(sc(v));if(w)return i.state.loading|=4,i.instance=w,Tt(w),w;m=d1(c),(v=pa.get(v))&&qh(m,v),w=(o.ownerDocument||o).createElement("link"),Tt(w);var I=w;return I._p=new Promise(function(H,J){I.onload=H,I.onerror=J}),vr(w,"link",m),i.state.loading|=4,If(w,c.precedence,o),i.instance=w;case"script":return w=zl(c.src),(v=o.querySelector(cc(w)))?(i.instance=v,Tt(v),v):(m=c,(v=pa.get(w))&&(m=h({},c),Wh(m,v)),o=o.ownerDocument||o,v=o.createElement("script"),Tt(v),vr(v,"link",m),o.head.appendChild(v),i.instance=v);case"void":return null;default:throw Error(r(443,i.type))}else i.type==="stylesheet"&&(i.state.loading&4)===0&&(m=i.instance,i.state.loading|=4,If(m,c.precedence,o));return i.instance}function If(o,i,c){for(var m=c.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),v=m.length?m[m.length-1]:null,w=v,I=0;I<m.length;I++){var H=m[I];if(H.dataset.precedence===i)w=H;else if(w!==v)break}w?w.parentNode.insertBefore(o,w.nextSibling):(i=c.nodeType===9?c.head:c,i.insertBefore(o,i.firstChild))}function qh(o,i){o.crossOrigin==null&&(o.crossOrigin=i.crossOrigin),o.referrerPolicy==null&&(o.referrerPolicy=i.referrerPolicy),o.title==null&&(o.title=i.title)}function Wh(o,i){o.crossOrigin==null&&(o.crossOrigin=i.crossOrigin),o.referrerPolicy==null&&(o.referrerPolicy=i.referrerPolicy),o.integrity==null&&(o.integrity=i.integrity)}var Pf=null;function g1(o,i,c){if(Pf===null){var m=new Map,v=Pf=new Map;v.set(c,m)}else v=Pf,m=v.get(c),m||(m=new Map,v.set(c,m));if(m.has(o))return m;for(m.set(o,null),c=c.getElementsByTagName(o),v=0;v<c.length;v++){var w=c[v];if(!(w[qt]||w[Ct]||o==="link"&&w.getAttribute("rel")==="stylesheet")&&w.namespaceURI!=="http://www.w3.org/2000/svg"){var I=w.getAttribute(i)||"";I=o+I;var H=m.get(I);H?H.push(w):m.set(I,[w])}}return m}function h1(o,i,c){o=o.ownerDocument||o,o.head.insertBefore(c,i==="title"?o.querySelector("head > title"):null)}function aI(o,i,c){if(c===1||i.itemProp!=null)return!1;switch(o){case"meta":case"title":return!0;case"style":if(typeof i.precedence!="string"||typeof i.href!="string"||i.href==="")break;return!0;case"link":if(typeof i.rel!="string"||typeof i.href!="string"||i.href===""||i.onLoad||i.onError)break;switch(i.rel){case"stylesheet":return o=i.disabled,typeof i.precedence=="string"&&o==null;default:return!0}case"script":if(i.async&&typeof i.async!="function"&&typeof i.async!="symbol"&&!i.onLoad&&!i.onError&&i.src&&typeof i.src=="string")return!0}return!1}function p1(o){return!(o.type==="stylesheet"&&(o.state.loading&3)===0)}var uc=null;function oI(){}function iI(o,i,c){if(uc===null)throw Error(r(475));var m=uc;if(i.type==="stylesheet"&&(typeof c.media!="string"||matchMedia(c.media).matches!==!1)&&(i.state.loading&4)===0){if(i.instance===null){var v=Al(c.href),w=o.querySelector(sc(v));if(w){o=w._p,o!==null&&typeof o=="object"&&typeof o.then=="function"&&(m.count++,m=Af.bind(m),o.then(m,m)),i.state.loading|=4,i.instance=w,Tt(w);return}w=o.ownerDocument||o,c=d1(c),(v=pa.get(v))&&qh(c,v),w=w.createElement("link"),Tt(w);var I=w;I._p=new Promise(function(H,J){I.onload=H,I.onerror=J}),vr(w,"link",c),i.instance=w}m.stylesheets===null&&(m.stylesheets=new Map),m.stylesheets.set(i,o),(o=i.state.preload)&&(i.state.loading&3)===0&&(m.count++,i=Af.bind(m),o.addEventListener("load",i),o.addEventListener("error",i))}}function lI(){if(uc===null)throw Error(r(475));var o=uc;return o.stylesheets&&o.count===0&&Gh(o,o.stylesheets),0<o.count?function(i){var c=setTimeout(function(){if(o.stylesheets&&Gh(o,o.stylesheets),o.unsuspend){var m=o.unsuspend;o.unsuspend=null,m()}},6e4);return o.unsuspend=i,function(){o.unsuspend=null,clearTimeout(c)}}:null}function Af(){if(this.count--,this.count===0){if(this.stylesheets)Gh(this,this.stylesheets);else if(this.unsuspend){var o=this.unsuspend;this.unsuspend=null,o()}}}var zf=null;function Gh(o,i){o.stylesheets=null,o.unsuspend!==null&&(o.count++,zf=new Map,i.forEach(sI,o),zf=null,Af.call(o))}function sI(o,i){if(!(i.state.loading&4)){var c=zf.get(o);if(c)var m=c.get(null);else{c=new Map,zf.set(o,c);for(var v=o.querySelectorAll("link[data-precedence],style[data-precedence]"),w=0;w<v.length;w++){var I=v[w];(I.nodeName==="LINK"||I.getAttribute("media")!=="not all")&&(c.set(I.dataset.precedence,I),m=I)}m&&c.set(null,m)}v=i.instance,I=v.getAttribute("data-precedence"),w=c.get(I)||m,w===m&&c.set(null,v),c.set(I,v),this.count++,m=Af.bind(this),v.addEventListener("load",m),v.addEventListener("error",m),w?w.parentNode.insertBefore(v,w.nextSibling):(o=o.nodeType===9?o.head:o,o.insertBefore(v,o.firstChild)),i.state.loading|=4}}var fc={$$typeof:O,Provider:null,Consumer:null,_currentValue:L,_currentValue2:L,_threadCount:0};function cI(o,i,c,m,v,w,I,H){this.tag=1,this.containerInfo=o,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=wn(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=wn(0),this.hiddenUpdates=wn(null),this.identifierPrefix=m,this.onUncaughtError=v,this.onCaughtError=w,this.onRecoverableError=I,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=H,this.incompleteTransitions=new Map}function v1(o,i,c,m,v,w,I,H,J,me,we,_e){return o=new cI(o,i,c,I,H,J,me,_e),i=1,w===!0&&(i|=24),w=kr(3,null,null,i),o.current=w,w.stateNode=o,i=_g(),i.refCount++,o.pooledCache=i,i.refCount++,w.memoizedState={element:m,isDehydrated:c,cache:i},jg(w),o}function y1(o){return o?(o=ml,o):ml}function b1(o,i,c,m,v,w){v=y1(v),m.context===null?m.context=v:m.pendingContext=v,m=ko(i),m.payload={element:c},w=w===void 0?null:w,w!==null&&(m.callback=w),c=qo(o,m,i),c!==null&&(Xr(c,o,i),Fs(c,o,i))}function S1(o,i){if(o=o.memoizedState,o!==null&&o.dehydrated!==null){var c=o.retryLane;o.retryLane=c!==0&&c<i?c:i}}function Kh(o,i){S1(o,i),(o=o.alternate)&&S1(o,i)}function C1(o){if(o.tag===13){var i=dl(o,67108864);i!==null&&Xr(i,o,67108864),Kh(o,67108864)}}var Df=!0;function uI(o,i,c,m){var v=j.T;j.T=null;var w=A.p;try{A.p=2,Xh(o,i,c,m)}finally{A.p=w,j.T=v}}function fI(o,i,c,m){var v=j.T;j.T=null;var w=A.p;try{A.p=8,Xh(o,i,c,m)}finally{A.p=w,j.T=v}}function Xh(o,i,c,m){if(Df){var v=Yh(m);if(v===null)zh(o,i,m,Lf,c),w1(o,m);else if(mI(v,o,i,c,m))m.stopPropagation();else if(w1(o,m),i&4&&-1<dI.indexOf(o)){for(;v!==null;){var w=At(v);if(w!==null)switch(w.tag){case 3:if(w=w.stateNode,w.current.memoizedState.isDehydrated){var I=Ue(w.pendingLanes);if(I!==0){var H=w;for(H.pendingLanes|=2,H.entangledLanes|=2;I;){var J=1<<31-ke(I);H.entanglements[1]|=J,I&=~J}Ya(w),(un&6)===0&&(Cf=Ee()+500,ac(0))}}break;case 13:H=dl(w,2),H!==null&&Xr(H,w,2),wf(),Kh(w,2)}if(w=Yh(m),w===null&&zh(o,i,m,Lf,c),w===v)break;v=w}v!==null&&m.stopPropagation()}else zh(o,i,m,null,c)}}function Yh(o){return o=sa(o),Qh(o)}var Lf=null;function Qh(o){if(Lf=null,o=Pt(o),o!==null){var i=l(o);if(i===null)o=null;else{var c=i.tag;if(c===13){if(o=s(i),o!==null)return o;o=null}else if(c===3){if(i.stateNode.current.memoizedState.isDehydrated)return i.tag===3?i.stateNode.containerInfo:null;o=null}else i!==o&&(o=null)}}return Lf=o,null}function x1(o){switch(o){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(Oe()){case Pe:return 2;case Ye:return 8;case Me:case De:return 32;case xe:return 268435456;default:return 32}default:return 32}}var Zh=!1,ai=null,oi=null,ii=null,dc=new Map,mc=new Map,li=[],dI="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function w1(o,i){switch(o){case"focusin":case"focusout":ai=null;break;case"dragenter":case"dragleave":oi=null;break;case"mouseover":case"mouseout":ii=null;break;case"pointerover":case"pointerout":dc.delete(i.pointerId);break;case"gotpointercapture":case"lostpointercapture":mc.delete(i.pointerId)}}function gc(o,i,c,m,v,w){return o===null||o.nativeEvent!==w?(o={blockedOn:i,domEventName:c,eventSystemFlags:m,nativeEvent:w,targetContainers:[v]},i!==null&&(i=At(i),i!==null&&C1(i)),o):(o.eventSystemFlags|=m,i=o.targetContainers,v!==null&&i.indexOf(v)===-1&&i.push(v),o)}function mI(o,i,c,m,v){switch(i){case"focusin":return ai=gc(ai,o,i,c,m,v),!0;case"dragenter":return oi=gc(oi,o,i,c,m,v),!0;case"mouseover":return ii=gc(ii,o,i,c,m,v),!0;case"pointerover":var w=v.pointerId;return dc.set(w,gc(dc.get(w)||null,o,i,c,m,v)),!0;case"gotpointercapture":return w=v.pointerId,mc.set(w,gc(mc.get(w)||null,o,i,c,m,v)),!0}return!1}function E1(o){var i=Pt(o.target);if(i!==null){var c=l(i);if(c!==null){if(i=c.tag,i===13){if(i=s(c),i!==null){o.blockedOn=i,Fe(o.priority,function(){if(c.tag===13){var m=Kr();m=Ot(m);var v=dl(c,m);v!==null&&Xr(v,c,m),Kh(c,m)}});return}}else if(i===3&&c.stateNode.current.memoizedState.isDehydrated){o.blockedOn=c.tag===3?c.stateNode.containerInfo:null;return}}}o.blockedOn=null}function Bf(o){if(o.blockedOn!==null)return!1;for(var i=o.targetContainers;0<i.length;){var c=Yh(o.nativeEvent);if(c===null){c=o.nativeEvent;var m=new c.constructor(c.type,c);qa=m,c.target.dispatchEvent(m),qa=null}else return i=At(c),i!==null&&C1(i),o.blockedOn=c,!1;i.shift()}return!0}function $1(o,i,c){Bf(o)&&c.delete(i)}function gI(){Zh=!1,ai!==null&&Bf(ai)&&(ai=null),oi!==null&&Bf(oi)&&(oi=null),ii!==null&&Bf(ii)&&(ii=null),dc.forEach($1),mc.forEach($1)}function Hf(o,i){o.blockedOn===i&&(o.blockedOn=null,Zh||(Zh=!0,e.unstable_scheduleCallback(e.unstable_NormalPriority,gI)))}var Ff=null;function O1(o){Ff!==o&&(Ff=o,e.unstable_scheduleCallback(e.unstable_NormalPriority,function(){Ff===o&&(Ff=null);for(var i=0;i<o.length;i+=3){var c=o[i],m=o[i+1],v=o[i+2];if(typeof m!="function"){if(Qh(m||c)===null)continue;break}var w=At(c);w!==null&&(o.splice(i,3),i-=3,Qg(w,{pending:!0,data:v,method:c.method,action:m},m,v))}}))}function hc(o){function i(J){return Hf(J,o)}ai!==null&&Hf(ai,o),oi!==null&&Hf(oi,o),ii!==null&&Hf(ii,o),dc.forEach(i),mc.forEach(i);for(var c=0;c<li.length;c++){var m=li[c];m.blockedOn===o&&(m.blockedOn=null)}for(;0<li.length&&(c=li[0],c.blockedOn===null);)E1(c),c.blockedOn===null&&li.shift();if(c=(o.ownerDocument||o).$$reactFormReplay,c!=null)for(m=0;m<c.length;m+=3){var v=c[m],w=c[m+1],I=v[Ve]||null;if(typeof w=="function")I||O1(c);else if(I){var H=null;if(w&&w.hasAttribute("formAction")){if(v=w,I=w[Ve]||null)H=I.formAction;else if(Qh(v)!==null)continue}else H=I.action;typeof H=="function"?c[m+1]=H:(c.splice(m,3),m-=3),O1(c)}}}function Jh(o){this._internalRoot=o}Vf.prototype.render=Jh.prototype.render=function(o){var i=this._internalRoot;if(i===null)throw Error(r(409));var c=i.current,m=Kr();b1(c,m,o,i,null,null)},Vf.prototype.unmount=Jh.prototype.unmount=function(){var o=this._internalRoot;if(o!==null){this._internalRoot=null;var i=o.containerInfo;b1(o.current,2,null,o,null,null),wf(),i[ze]=null}};function Vf(o){this._internalRoot=o}Vf.prototype.unstable_scheduleHydration=function(o){if(o){var i=Ke();o={blockedOn:null,target:o,priority:i};for(var c=0;c<li.length&&i!==0&&i<li[c].priority;c++);li.splice(c,0,o),c===0&&E1(o)}};var R1=t.version;if(R1!=="19.1.0")throw Error(r(527,R1,"19.1.0"));A.findDOMNode=function(o){var i=o._reactInternals;if(i===void 0)throw typeof o.render=="function"?Error(r(188)):(o=Object.keys(o).join(","),Error(r(268,o)));return o=d(i),o=o!==null?g(o):null,o=o===null?null:o.stateNode,o};var hI={bundleType:0,version:"19.1.0",rendererPackageName:"react-dom",currentDispatcherRef:j,reconcilerVersion:"19.1.0"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var Uf=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!Uf.isDisabled&&Uf.supportsFiber)try{We=Uf.inject(hI),Ge=Uf}catch{}}return vc.createRoot=function(o,i){if(!a(o))throw Error(r(299));var c=!1,m="",v=US,w=kS,I=qS,H=null;return i!=null&&(i.unstable_strictMode===!0&&(c=!0),i.identifierPrefix!==void 0&&(m=i.identifierPrefix),i.onUncaughtError!==void 0&&(v=i.onUncaughtError),i.onCaughtError!==void 0&&(w=i.onCaughtError),i.onRecoverableError!==void 0&&(I=i.onRecoverableError),i.unstable_transitionCallbacks!==void 0&&(H=i.unstable_transitionCallbacks)),i=v1(o,1,!1,null,null,c,m,v,w,I,H,null),o[ze]=i.current,Ah(o),new Jh(i)},vc.hydrateRoot=function(o,i,c){if(!a(o))throw Error(r(299));var m=!1,v="",w=US,I=kS,H=qS,J=null,me=null;return c!=null&&(c.unstable_strictMode===!0&&(m=!0),c.identifierPrefix!==void 0&&(v=c.identifierPrefix),c.onUncaughtError!==void 0&&(w=c.onUncaughtError),c.onCaughtError!==void 0&&(I=c.onCaughtError),c.onRecoverableError!==void 0&&(H=c.onRecoverableError),c.unstable_transitionCallbacks!==void 0&&(J=c.unstable_transitionCallbacks),c.formState!==void 0&&(me=c.formState)),i=v1(o,1,!0,i,c??null,m,v,w,I,H,J,me),i.context=y1(null),c=i.current,m=Kr(),m=Ot(m),v=ko(m),v.callback=null,qo(c,v,m),c=m,i.current.lanes=c,Rn(i,c),Ya(i),o[ze]=i.current,Ah(o),new Vf(i)},vc.version="19.1.0",vc}var D1;function EI(){if(D1)return tp.exports;D1=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),tp.exports=wI(),tp.exports}var $I=EI();const OI="modulepreload",RI=function(e,t){return new URL(e,t).href},L1={},Ut=function(t,n,r){let a=Promise.resolve();if(n&&n.length>0){let s=function(h){return Promise.all(h.map(p=>Promise.resolve(p).then(y=>({status:"fulfilled",value:y}),y=>({status:"rejected",reason:y}))))};const u=document.getElementsByTagName("link"),d=document.querySelector("meta[property=csp-nonce]"),g=(d==null?void 0:d.nonce)||(d==null?void 0:d.getAttribute("nonce"));a=s(n.map(h=>{if(h=RI(h,r),h in L1)return;L1[h]=!0;const p=h.endsWith(".css"),y=p?'[rel="stylesheet"]':"";if(!!r)for(let x=u.length-1;x>=0;x--){const S=u[x];if(S.href===h&&(!p||S.rel==="stylesheet"))return}else if(document.querySelector(`link[href="${h}"]${y}`))return;const b=document.createElement("link");if(b.rel=p?"stylesheet":OI,p||(b.as="script"),b.crossOrigin="",b.href=h,g&&b.setAttribute("nonce",g),document.head.appendChild(b),p)return new Promise((x,S)=>{b.addEventListener("load",x),b.addEventListener("error",()=>S(new Error(`Unable to preload CSS for ${h}`)))})}))}function l(s){const u=new Event("vite:preloadError",{cancelable:!0});if(u.payload=s,window.dispatchEvent(u),!u.defaultPrevented)throw s}return a.then(s=>{for(const u of s||[])u.status==="rejected"&&l(u.reason);return t().catch(l)})};var f=im();const se=is(f),lm=$O({__proto__:null,default:se},[f]);var V0=f.createContext({});function je(){return je=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)({}).hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},je.apply(null,arguments)}function RO(e){if(Array.isArray(e))return e}function _I(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,a,l,s,u=[],d=!0,g=!1;try{if(l=(n=n.call(e)).next,t===0){if(Object(n)!==n)return;d=!1}else for(;!(d=(r=l.call(n)).done)&&(u.push(r.value),u.length!==t);d=!0);}catch(h){g=!0,a=h}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(g)throw a}}return u}}function Cv(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=Array(t);n<t;n++)r[n]=e[n];return r}function U0(e,t){if(e){if(typeof e=="string")return Cv(e,t);var n={}.toString.call(e).slice(8,-1);return n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set"?Array.from(e):n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?Cv(e,t):void 0}}function _O(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function ge(e,t){return RO(e)||_I(e,t)||U0(e,t)||_O()}function st(e){"@babel/helpers - typeof";return st=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},st(e)}function TI(e,t){if(st(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(st(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function TO(e){var t=TI(e,"string");return st(t)=="symbol"?t:t+""}function Z(e,t,n){return(t=TO(t))in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function MI(e,t){if(e==null)return{};var n={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)!==-1)continue;n[r]=e[r]}return n}function $t(e,t){if(e==null)return{};var n,r,a=MI(e,t);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(r=0;r<l.length;r++)n=l[r],t.indexOf(n)===-1&&{}.propertyIsEnumerable.call(e,n)&&(a[n]=e[n])}return a}var ip={exports:{}};/*!
	Copyright (c) 2018 Jed Watson.
	Licensed under the MIT License (MIT), see
	http://jedwatson.github.io/classnames
*/var B1;function NI(){return B1||(B1=1,function(e){(function(){var t={}.hasOwnProperty;function n(){for(var l="",s=0;s<arguments.length;s++){var u=arguments[s];u&&(l=a(l,r(u)))}return l}function r(l){if(typeof l=="string"||typeof l=="number")return l;if(typeof l!="object")return"";if(Array.isArray(l))return n.apply(null,l);if(l.toString!==Object.prototype.toString&&!l.toString.toString().includes("[native code]"))return l.toString();var s="";for(var u in l)t.call(l,u)&&l[u]&&(s=a(s,u));return s}function a(l,s){return s?l?l+" "+s:l+s:l}e.exports?(n.default=n,e.exports=n):window.classNames=n})()}(ip)),ip.exports}var jI=NI();const re=is(jI),dr=Math.round;function lp(e,t){const n=e.replace(/^[^(]*\((.*)/,"$1").replace(/\).*/,"").match(/\d*\.?\d+%?/g)||[],r=n.map(a=>parseFloat(a));for(let a=0;a<3;a+=1)r[a]=t(r[a]||0,n[a]||"",a);return n[3]?r[3]=n[3].includes("%")?r[3]/100:r[3]:r[3]=1,r}const H1=(e,t,n)=>n===0?e:e/100;function yc(e,t){const n=t||255;return e>n?n:e<0?0:e}class en{constructor(t){Z(this,"isValid",!0),Z(this,"r",0),Z(this,"g",0),Z(this,"b",0),Z(this,"a",1),Z(this,"_h",void 0),Z(this,"_s",void 0),Z(this,"_l",void 0),Z(this,"_v",void 0),Z(this,"_max",void 0),Z(this,"_min",void 0),Z(this,"_brightness",void 0);function n(r){return r[0]in t&&r[1]in t&&r[2]in t}if(t)if(typeof t=="string"){let a=function(l){return r.startsWith(l)};const r=t.trim();/^#?[A-F\d]{3,8}$/i.test(r)?this.fromHexString(r):a("rgb")?this.fromRgbString(r):a("hsl")?this.fromHslString(r):(a("hsv")||a("hsb"))&&this.fromHsvString(r)}else if(t instanceof en)this.r=t.r,this.g=t.g,this.b=t.b,this.a=t.a,this._h=t._h,this._s=t._s,this._l=t._l,this._v=t._v;else if(n("rgb"))this.r=yc(t.r),this.g=yc(t.g),this.b=yc(t.b),this.a=typeof t.a=="number"?yc(t.a,1):1;else if(n("hsl"))this.fromHsl(t);else if(n("hsv"))this.fromHsv(t);else throw new Error("@ant-design/fast-color: unsupported input "+JSON.stringify(t))}setR(t){return this._sc("r",t)}setG(t){return this._sc("g",t)}setB(t){return this._sc("b",t)}setA(t){return this._sc("a",t,1)}setHue(t){const n=this.toHsv();return n.h=t,this._c(n)}getLuminance(){function t(l){const s=l/255;return s<=.03928?s/12.92:Math.pow((s+.055)/1.055,2.4)}const n=t(this.r),r=t(this.g),a=t(this.b);return .2126*n+.7152*r+.0722*a}getHue(){if(typeof this._h>"u"){const t=this.getMax()-this.getMin();t===0?this._h=0:this._h=dr(60*(this.r===this.getMax()?(this.g-this.b)/t+(this.g<this.b?6:0):this.g===this.getMax()?(this.b-this.r)/t+2:(this.r-this.g)/t+4))}return this._h}getSaturation(){if(typeof this._s>"u"){const t=this.getMax()-this.getMin();t===0?this._s=0:this._s=t/this.getMax()}return this._s}getLightness(){return typeof this._l>"u"&&(this._l=(this.getMax()+this.getMin())/510),this._l}getValue(){return typeof this._v>"u"&&(this._v=this.getMax()/255),this._v}getBrightness(){return typeof this._brightness>"u"&&(this._brightness=(this.r*299+this.g*587+this.b*114)/1e3),this._brightness}darken(t=10){const n=this.getHue(),r=this.getSaturation();let a=this.getLightness()-t/100;return a<0&&(a=0),this._c({h:n,s:r,l:a,a:this.a})}lighten(t=10){const n=this.getHue(),r=this.getSaturation();let a=this.getLightness()+t/100;return a>1&&(a=1),this._c({h:n,s:r,l:a,a:this.a})}mix(t,n=50){const r=this._c(t),a=n/100,l=u=>(r[u]-this[u])*a+this[u],s={r:dr(l("r")),g:dr(l("g")),b:dr(l("b")),a:dr(l("a")*100)/100};return this._c(s)}tint(t=10){return this.mix({r:255,g:255,b:255,a:1},t)}shade(t=10){return this.mix({r:0,g:0,b:0,a:1},t)}onBackground(t){const n=this._c(t),r=this.a+n.a*(1-this.a),a=l=>dr((this[l]*this.a+n[l]*n.a*(1-this.a))/r);return this._c({r:a("r"),g:a("g"),b:a("b"),a:r})}isDark(){return this.getBrightness()<128}isLight(){return this.getBrightness()>=128}equals(t){return this.r===t.r&&this.g===t.g&&this.b===t.b&&this.a===t.a}clone(){return this._c(this)}toHexString(){let t="#";const n=(this.r||0).toString(16);t+=n.length===2?n:"0"+n;const r=(this.g||0).toString(16);t+=r.length===2?r:"0"+r;const a=(this.b||0).toString(16);if(t+=a.length===2?a:"0"+a,typeof this.a=="number"&&this.a>=0&&this.a<1){const l=dr(this.a*255).toString(16);t+=l.length===2?l:"0"+l}return t}toHsl(){return{h:this.getHue(),s:this.getSaturation(),l:this.getLightness(),a:this.a}}toHslString(){const t=this.getHue(),n=dr(this.getSaturation()*100),r=dr(this.getLightness()*100);return this.a!==1?`hsla(${t},${n}%,${r}%,${this.a})`:`hsl(${t},${n}%,${r}%)`}toHsv(){return{h:this.getHue(),s:this.getSaturation(),v:this.getValue(),a:this.a}}toRgb(){return{r:this.r,g:this.g,b:this.b,a:this.a}}toRgbString(){return this.a!==1?`rgba(${this.r},${this.g},${this.b},${this.a})`:`rgb(${this.r},${this.g},${this.b})`}toString(){return this.toRgbString()}_sc(t,n,r){const a=this.clone();return a[t]=yc(n,r),a}_c(t){return new this.constructor(t)}getMax(){return typeof this._max>"u"&&(this._max=Math.max(this.r,this.g,this.b)),this._max}getMin(){return typeof this._min>"u"&&(this._min=Math.min(this.r,this.g,this.b)),this._min}fromHexString(t){const n=t.replace("#","");function r(a,l){return parseInt(n[a]+n[l||a],16)}n.length<6?(this.r=r(0),this.g=r(1),this.b=r(2),this.a=n[3]?r(3)/255:1):(this.r=r(0,1),this.g=r(2,3),this.b=r(4,5),this.a=n[6]?r(6,7)/255:1)}fromHsl({h:t,s:n,l:r,a}){if(this._h=t%360,this._s=n,this._l=r,this.a=typeof a=="number"?a:1,n<=0){const y=dr(r*255);this.r=y,this.g=y,this.b=y}let l=0,s=0,u=0;const d=t/60,g=(1-Math.abs(2*r-1))*n,h=g*(1-Math.abs(d%2-1));d>=0&&d<1?(l=g,s=h):d>=1&&d<2?(l=h,s=g):d>=2&&d<3?(s=g,u=h):d>=3&&d<4?(s=h,u=g):d>=4&&d<5?(l=h,u=g):d>=5&&d<6&&(l=g,u=h);const p=r-g/2;this.r=dr((l+p)*255),this.g=dr((s+p)*255),this.b=dr((u+p)*255)}fromHsv({h:t,s:n,v:r,a}){this._h=t%360,this._s=n,this._v=r,this.a=typeof a=="number"?a:1;const l=dr(r*255);if(this.r=l,this.g=l,this.b=l,n<=0)return;const s=t/60,u=Math.floor(s),d=s-u,g=dr(r*(1-n)*255),h=dr(r*(1-n*d)*255),p=dr(r*(1-n*(1-d))*255);switch(u){case 0:this.g=p,this.b=g;break;case 1:this.r=h,this.b=g;break;case 2:this.r=g,this.b=p;break;case 3:this.r=g,this.g=h;break;case 4:this.r=p,this.g=g;break;case 5:default:this.g=g,this.b=h;break}}fromHsvString(t){const n=lp(t,H1);this.fromHsv({h:n[0],s:n[1],v:n[2],a:n[3]})}fromHslString(t){const n=lp(t,H1);this.fromHsl({h:n[0],s:n[1],l:n[2],a:n[3]})}fromRgbString(t){const n=lp(t,(r,a)=>a.includes("%")?dr(r/100*255):r);this.r=n[0],this.g=n[1],this.b=n[2],this.a=n[3]}}var kf=2,F1=.16,II=.05,PI=.05,AI=.15,MO=5,NO=4,zI=[{index:7,amount:15},{index:6,amount:25},{index:5,amount:30},{index:5,amount:45},{index:5,amount:65},{index:5,amount:85},{index:4,amount:90},{index:3,amount:95},{index:2,amount:97},{index:1,amount:98}];function V1(e,t,n){var r;return Math.round(e.h)>=60&&Math.round(e.h)<=240?r=n?Math.round(e.h)-kf*t:Math.round(e.h)+kf*t:r=n?Math.round(e.h)+kf*t:Math.round(e.h)-kf*t,r<0?r+=360:r>=360&&(r-=360),r}function U1(e,t,n){if(e.h===0&&e.s===0)return e.s;var r;return n?r=e.s-F1*t:t===NO?r=e.s+F1:r=e.s+II*t,r>1&&(r=1),n&&t===MO&&r>.1&&(r=.1),r<.06&&(r=.06),Math.round(r*100)/100}function k1(e,t,n){var r;return n?r=e.v+PI*t:r=e.v-AI*t,r=Math.max(0,Math.min(1,r)),Math.round(r*100)/100}function Yi(e){for(var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=[],r=new en(e),a=r.toHsv(),l=MO;l>0;l-=1){var s=new en({h:V1(a,l,!0),s:U1(a,l,!0),v:k1(a,l,!0)});n.push(s)}n.push(r);for(var u=1;u<=NO;u+=1){var d=new en({h:V1(a,u),s:U1(a,u),v:k1(a,u)});n.push(d)}return t.theme==="dark"?zI.map(function(g){var h=g.index,p=g.amount;return new en(t.backgroundColor||"#141414").mix(n[h],p).toHexString()}):n.map(function(g){return g.toHexString()})}var sp={red:"#F5222D",volcano:"#FA541C",orange:"#FA8C16",gold:"#FAAD14",yellow:"#FADB14",lime:"#A0D911",green:"#52C41A",cyan:"#13C2C2",blue:"#1677FF",geekblue:"#2F54EB",purple:"#722ED1",magenta:"#EB2F96",grey:"#666666"},xv=["#fff1f0","#ffccc7","#ffa39e","#ff7875","#ff4d4f","#f5222d","#cf1322","#a8071a","#820014","#5c0011"];xv.primary=xv[5];var wv=["#fff2e8","#ffd8bf","#ffbb96","#ff9c6e","#ff7a45","#fa541c","#d4380d","#ad2102","#871400","#610b00"];wv.primary=wv[5];var Ev=["#fff7e6","#ffe7ba","#ffd591","#ffc069","#ffa940","#fa8c16","#d46b08","#ad4e00","#873800","#612500"];Ev.primary=Ev[5];var Rd=["#fffbe6","#fff1b8","#ffe58f","#ffd666","#ffc53d","#faad14","#d48806","#ad6800","#874d00","#613400"];Rd.primary=Rd[5];var $v=["#feffe6","#ffffb8","#fffb8f","#fff566","#ffec3d","#fadb14","#d4b106","#ad8b00","#876800","#614700"];$v.primary=$v[5];var Ov=["#fcffe6","#f4ffb8","#eaff8f","#d3f261","#bae637","#a0d911","#7cb305","#5b8c00","#3f6600","#254000"];Ov.primary=Ov[5];var Rv=["#f6ffed","#d9f7be","#b7eb8f","#95de64","#73d13d","#52c41a","#389e0d","#237804","#135200","#092b00"];Rv.primary=Rv[5];var _v=["#e6fffb","#b5f5ec","#87e8de","#5cdbd3","#36cfc9","#13c2c2","#08979c","#006d75","#00474f","#002329"];_v.primary=_v[5];var _d=["#e6f4ff","#bae0ff","#91caff","#69b1ff","#4096ff","#1677ff","#0958d9","#003eb3","#002c8c","#001d66"];_d.primary=_d[5];var Tv=["#f0f5ff","#d6e4ff","#adc6ff","#85a5ff","#597ef7","#2f54eb","#1d39c4","#10239e","#061178","#030852"];Tv.primary=Tv[5];var Mv=["#f9f0ff","#efdbff","#d3adf7","#b37feb","#9254de","#722ed1","#531dab","#391085","#22075e","#120338"];Mv.primary=Mv[5];var Nv=["#fff0f6","#ffd6e7","#ffadd2","#ff85c0","#f759ab","#eb2f96","#c41d7f","#9e1068","#780650","#520339"];Nv.primary=Nv[5];var jv=["#a6a6a6","#999999","#8c8c8c","#808080","#737373","#666666","#404040","#1a1a1a","#000000","#000000"];jv.primary=jv[5];var cp={red:xv,volcano:wv,orange:Ev,gold:Rd,yellow:$v,lime:Ov,green:Rv,cyan:_v,blue:_d,geekblue:Tv,purple:Mv,magenta:Nv,grey:jv};function q1(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(a){return Object.getOwnPropertyDescriptor(e,a).enumerable})),n.push.apply(n,r)}return n}function Q(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?q1(Object(n),!0).forEach(function(r){Z(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):q1(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function $r(){return!!(typeof window<"u"&&window.document&&window.document.createElement)}function Iv(e,t){if(!e)return!1;if(e.contains)return e.contains(t);for(var n=t;n;){if(n===e)return!0;n=n.parentNode}return!1}var W1="data-rc-order",G1="data-rc-priority",DI="rc-util-key",Pv=new Map;function jO(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},t=e.mark;return t?t.startsWith("data-")?t:"data-".concat(t):DI}function sm(e){if(e.attachTo)return e.attachTo;var t=document.querySelector("head");return t||document.body}function LI(e){return e==="queue"?"prependQueue":e?"prepend":"append"}function k0(e){return Array.from((Pv.get(e)||e).children).filter(function(t){return t.tagName==="STYLE"})}function IO(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};if(!$r())return null;var n=t.csp,r=t.prepend,a=t.priority,l=a===void 0?0:a,s=LI(r),u=s==="prependQueue",d=document.createElement("style");d.setAttribute(W1,s),u&&l&&d.setAttribute(G1,"".concat(l)),n!=null&&n.nonce&&(d.nonce=n==null?void 0:n.nonce),d.innerHTML=e;var g=sm(t),h=g.firstChild;if(r){if(u){var p=(t.styles||k0(g)).filter(function(y){if(!["prepend","prependQueue"].includes(y.getAttribute(W1)))return!1;var C=Number(y.getAttribute(G1)||0);return l>=C});if(p.length)return g.insertBefore(d,p[p.length-1].nextSibling),d}g.insertBefore(d,h)}else g.appendChild(d);return d}function PO(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=sm(t);return(t.styles||k0(n)).find(function(r){return r.getAttribute(jO(t))===e})}function Zc(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=PO(e,t);if(n){var r=sm(t);r.removeChild(n)}}function BI(e,t){var n=Pv.get(e);if(!n||!Iv(document,n)){var r=IO("",t),a=r.parentNode;Pv.set(e,a),e.removeChild(r)}}function xo(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},r=sm(n),a=k0(r),l=Q(Q({},n),{},{styles:a});BI(r,l);var s=PO(t,l);if(s){var u,d;if((u=l.csp)!==null&&u!==void 0&&u.nonce&&s.nonce!==((d=l.csp)===null||d===void 0?void 0:d.nonce)){var g;s.nonce=(g=l.csp)===null||g===void 0?void 0:g.nonce}return s.innerHTML!==e&&(s.innerHTML=e),s}var h=IO(e,l);return h.setAttribute(jO(l),t),h}function AO(e){var t;return e==null||(t=e.getRootNode)===null||t===void 0?void 0:t.call(e)}function HI(e){return AO(e)instanceof ShadowRoot}function Td(e){return HI(e)?AO(e):null}var Av={},FI=function(t){};function VI(e,t){}function UI(e,t){}function kI(){Av={}}function zO(e,t,n){!t&&!Av[n]&&(e(!1,n),Av[n]=!0)}function mr(e,t){zO(VI,e,t)}function qI(e,t){zO(UI,e,t)}mr.preMessage=FI;mr.resetWarned=kI;mr.noteOnce=qI;function WI(e){return e.replace(/-(.)/g,function(t,n){return n.toUpperCase()})}function GI(e,t){mr(e,"[@ant-design/icons] ".concat(t))}function K1(e){return st(e)==="object"&&typeof e.name=="string"&&typeof e.theme=="string"&&(st(e.icon)==="object"||typeof e.icon=="function")}function X1(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};return Object.keys(e).reduce(function(t,n){var r=e[n];switch(n){case"class":t.className=r,delete t.class;break;default:delete t[n],t[WI(n)]=r}return t},{})}function zv(e,t,n){return n?se.createElement(e.tag,Q(Q({key:t},X1(e.attrs)),n),(e.children||[]).map(function(r,a){return zv(r,"".concat(t,"-").concat(e.tag,"-").concat(a))})):se.createElement(e.tag,Q({key:t},X1(e.attrs)),(e.children||[]).map(function(r,a){return zv(r,"".concat(t,"-").concat(e.tag,"-").concat(a))}))}function DO(e){return Yi(e)[0]}function LO(e){return e?Array.isArray(e)?e:[e]:[]}var KI=`
.anticon {
  display: inline-flex;
  align-items: center;
  color: inherit;
  font-style: normal;
  line-height: 0;
  text-align: center;
  text-transform: none;
  vertical-align: -0.125em;
  text-rendering: optimizeLegibility;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.anticon > * {
  line-height: 1;
}

.anticon svg {
  display: inline-block;
}

.anticon::before {
  display: none;
}

.anticon .anticon-icon {
  display: block;
}

.anticon[tabindex] {
  cursor: pointer;
}

.anticon-spin::before,
.anticon-spin {
  display: inline-block;
  -webkit-animation: loadingCircle 1s infinite linear;
  animation: loadingCircle 1s infinite linear;
}

@-webkit-keyframes loadingCircle {
  100% {
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg);
  }
}

@keyframes loadingCircle {
  100% {
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg);
  }
}
`,XI=function(t){var n=f.useContext(V0),r=n.csp,a=n.prefixCls,l=n.layer,s=KI;a&&(s=s.replace(/anticon/g,a)),l&&(s="@layer ".concat(l,` {
`).concat(s,`
}`)),f.useEffect(function(){var u=t.current,d=Td(u);xo(s,"@ant-design-icons",{prepend:!l,csp:r,attachTo:d})},[])},YI=["icon","className","onClick","style","primaryColor","secondaryColor"],Wc={primaryColor:"#333",secondaryColor:"#E6E6E6",calculated:!1};function QI(e){var t=e.primaryColor,n=e.secondaryColor;Wc.primaryColor=t,Wc.secondaryColor=n||DO(t),Wc.calculated=!!n}function ZI(){return Q({},Wc)}var ls=function(t){var n=t.icon,r=t.className,a=t.onClick,l=t.style,s=t.primaryColor,u=t.secondaryColor,d=$t(t,YI),g=f.useRef(),h=Wc;if(s&&(h={primaryColor:s,secondaryColor:u||DO(s)}),XI(g),GI(K1(n),"icon should be icon definiton, but got ".concat(n)),!K1(n))return null;var p=n;return p&&typeof p.icon=="function"&&(p=Q(Q({},p),{},{icon:p.icon(h.primaryColor,h.secondaryColor)})),zv(p.icon,"svg-".concat(p.name),Q(Q({className:r,onClick:a,style:l,"data-icon":p.name,width:"1em",height:"1em",fill:"currentColor","aria-hidden":"true"},d),{},{ref:g}))};ls.displayName="IconReact";ls.getTwoToneColors=ZI;ls.setTwoToneColors=QI;function BO(e){var t=LO(e),n=ge(t,2),r=n[0],a=n[1];return ls.setTwoToneColors({primaryColor:r,secondaryColor:a})}function JI(){var e=ls.getTwoToneColors();return e.calculated?[e.primaryColor,e.secondaryColor]:e.primaryColor}var e4=["className","icon","spin","rotate","tabIndex","onClick","twoToneColor"];BO(_d.primary);var on=f.forwardRef(function(e,t){var n=e.className,r=e.icon,a=e.spin,l=e.rotate,s=e.tabIndex,u=e.onClick,d=e.twoToneColor,g=$t(e,e4),h=f.useContext(V0),p=h.prefixCls,y=p===void 0?"anticon":p,C=h.rootClassName,b=re(C,y,Z(Z({},"".concat(y,"-").concat(r.name),!!r.name),"".concat(y,"-spin"),!!a||r.name==="loading"),n),x=s;x===void 0&&u&&(x=-1);var S=l?{msTransform:"rotate(".concat(l,"deg)"),transform:"rotate(".concat(l,"deg)")}:void 0,E=LO(d),$=ge(E,2),O=$[0],_=$[1];return f.createElement("span",je({role:"img","aria-label":r.name},g,{ref:t,tabIndex:x,onClick:u,className:b}),f.createElement(ls,{icon:r,primaryColor:O,secondaryColor:_,style:S}))});on.displayName="AntdIcon";on.getTwoToneColor=JI;on.setTwoToneColor=BO;var t4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M917.7 148.8l-42.4-42.4c-1.6-1.6-3.6-2.3-5.7-2.3s-4.1.8-5.7 2.3l-76.1 76.1a199.27 199.27 0 00-112.1-34.3c-51.2 0-102.4 19.5-141.5 58.6L432.3 308.7a8.03 8.03 0 000 11.3L704 591.7c1.6 1.6 3.6 2.3 5.7 2.3 2 0 4.1-.8 5.7-2.3l101.9-101.9c68.9-69 77-175.7 24.3-253.5l76.1-76.1c3.1-3.2 3.1-8.3 0-11.4zM769.1 441.7l-59.4 59.4-186.8-186.8 59.4-59.4c24.9-24.9 58.1-38.7 93.4-38.7 35.3 0 68.4 13.7 93.4 38.7 24.9 24.9 38.7 58.1 38.7 93.4 0 35.3-13.8 68.4-38.7 93.4zm-190.2 105a8.03 8.03 0 00-11.3 0L501 613.3 410.7 523l66.7-66.7c3.1-3.1 3.1-8.2 0-11.3L441 408.6a8.03 8.03 0 00-11.3 0L363 475.3l-43-43a7.85 7.85 0 00-5.7-2.3c-2 0-4.1.8-5.7 2.3L206.8 534.2c-68.9 69-77 175.7-24.3 253.5l-76.1 76.1a8.03 8.03 0 000 11.3l42.4 42.4c1.6 1.6 3.6 2.3 5.7 2.3s4.1-.8 5.7-2.3l76.1-76.1c33.7 22.9 72.9 34.3 112.1 34.3 51.2 0 102.4-19.5 141.5-58.6l101.9-101.9c3.1-3.1 3.1-8.2 0-11.3l-43-43 66.7-66.7c3.1-3.1 3.1-8.2 0-11.3l-36.6-36.2zM441.7 769.1a131.32 131.32 0 01-93.4 38.7c-35.3 0-68.4-13.7-93.4-38.7a131.32 131.32 0 01-38.7-93.4c0-35.3 13.7-68.4 38.7-93.4l59.4-59.4 186.8 186.8-59.4 59.4z"}}]},name:"api",theme:"outlined"},n4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:t4}))},r4=f.forwardRef(n4),a4={icon:{tag:"svg",attrs:{viewBox:"0 0 1024 1024",focusable:"false"},children:[{tag:"path",attrs:{d:"M912 192H328c-4.4 0-8 3.6-8 8v56c0 4.4 3.6 8 8 8h584c4.4 0 8-3.6 8-8v-56c0-4.4-3.6-8-8-8zm0 284H328c-4.4 0-8 3.6-8 8v56c0 4.4 3.6 8 8 8h584c4.4 0 8-3.6 8-8v-56c0-4.4-3.6-8-8-8zm0 284H328c-4.4 0-8 3.6-8 8v56c0 4.4 3.6 8 8 8h584c4.4 0 8-3.6 8-8v-56c0-4.4-3.6-8-8-8zM104 228a56 56 0 10112 0 56 56 0 10-112 0zm0 284a56 56 0 10112 0 56 56 0 10-112 0zm0 284a56 56 0 10112 0 56 56 0 10-112 0z"}}]},name:"bars",theme:"outlined"},o4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:a4}))},i4=f.forwardRef(o4),l4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M832 64H192c-17.7 0-32 14.3-32 32v832c0 17.7 14.3 32 32 32h640c17.7 0 32-14.3 32-32V96c0-17.7-14.3-32-32-32zm-260 72h96v209.9L621.5 312 572 347.4V136zm220 752H232V136h280v296.9c0 3.3 1 6.6 3 9.3a15.9 15.9 0 0022.3 3.7l83.8-59.9 81.4 59.4c2.7 2 6 3.1 9.4 3.1 8.8 0 16-7.2 16-16V136h64v752z"}}]},name:"book",theme:"outlined"},s4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:l4}))},c4=f.forwardRef(s4),u4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64C264.6 64 64 264.6 64 512s200.6 448 448 448 448-200.6 448-448S759.4 64 512 64zm193.5 301.7l-210.6 292a31.8 31.8 0 01-51.7 0L318.5 484.9c-3.8-5.3 0-12.7 6.5-12.7h46.9c10.2 0 19.9 4.9 25.9 13.3l71.2 98.8 157.2-218c6-8.3 15.6-13.3 25.9-13.3H699c6.5 0 10.3 7.4 6.5 12.7z"}}]},name:"check-circle",theme:"filled"},f4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:u4}))},cm=f.forwardRef(f4),d4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M912 190h-69.9c-9.8 0-19.1 4.5-25.1 12.2L404.7 724.5 207 474a32 32 0 00-25.1-12.2H112c-6.7 0-10.4 7.7-6.3 12.9l273.9 347c12.8 16.2 37.4 16.2 50.3 0l488.4-618.9c4.1-5.1.4-12.8-6.3-12.8z"}}]},name:"check",theme:"outlined"},m4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:d4}))},HO=f.forwardRef(m4),g4={icon:{tag:"svg",attrs:{"fill-rule":"evenodd",viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64c247.4 0 448 200.6 448 448S759.4 960 512 960 64 759.4 64 512 264.6 64 512 64zm127.98 274.82h-.04l-.08.06L512 466.75 384.14 338.88c-.04-.05-.06-.06-.08-.06a.12.12 0 00-.07 0c-.03 0-.05.01-.09.05l-45.02 45.02a.2.2 0 00-.05.09.12.12 0 000 .07v.02a.27.27 0 00.06.06L466.75 512 338.88 639.86c-.05.04-.06.06-.06.08a.12.12 0 000 .07c0 .03.01.05.05.09l45.02 45.02a.2.2 0 00.09.05.12.12 0 00.07 0c.02 0 .04-.01.08-.05L512 557.25l127.86 127.87c.04.04.06.05.08.05a.12.12 0 00.07 0c.03 0 .05-.01.09-.05l45.02-45.02a.2.2 0 00.05-.09.12.12 0 000-.07v-.02a.27.27 0 00-.05-.06L557.25 512l127.87-127.86c.04-.04.05-.06.05-.08a.12.12 0 000-.07c0-.03-.01-.05-.05-.09l-45.02-45.02a.2.2 0 00-.09-.05.12.12 0 00-.07 0z"}}]},name:"close-circle",theme:"filled"},h4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:g4}))},ss=f.forwardRef(h4),p4={icon:{tag:"svg",attrs:{"fill-rule":"evenodd",viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M799.86 166.31c.02 0 .04.02.08.06l57.69 57.7c.04.03.05.05.06.08a.12.12 0 010 .06c0 .03-.02.05-.06.09L569.93 512l287.7 287.7c.04.04.05.06.06.09a.12.12 0 010 .07c0 .02-.02.04-.06.08l-57.7 57.69c-.03.04-.05.05-.07.06a.12.12 0 01-.07 0c-.03 0-.05-.02-.09-.06L512 569.93l-287.7 287.7c-.04.04-.06.05-.09.06a.12.12 0 01-.07 0c-.02 0-.04-.02-.08-.06l-57.69-57.7c-.04-.03-.05-.05-.06-.07a.12.12 0 010-.07c0-.03.02-.05.06-.09L454.07 512l-287.7-287.7c-.04-.04-.05-.06-.06-.09a.12.12 0 010-.07c0-.02.02-.04.06-.08l57.7-57.69c.03-.04.05-.05.07-.06a.12.12 0 01.07 0c.03 0 .05.02.09.06L512 454.07l287.7-287.7c.04-.04.06-.05.09-.06a.12.12 0 01.07 0z"}}]},name:"close",theme:"outlined"},v4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:p4}))},cs=f.forwardRef(v4),y4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M832 64H296c-4.4 0-8 3.6-8 8v56c0 4.4 3.6 8 8 8h496v688c0 4.4 3.6 8 8 8h56c4.4 0 8-3.6 8-8V96c0-17.7-14.3-32-32-32zM704 192H192c-17.7 0-32 14.3-32 32v530.7c0 8.5 3.4 16.6 9.4 22.6l173.3 173.3c2.2 2.2 4.7 4 7.4 5.5v1.9h4.2c3.5 1.3 7.2 2 11 2H704c17.7 0 32-14.3 32-32V224c0-17.7-14.3-32-32-32zM350 856.2L263.9 770H350v86.2zM664 888H414V746c0-22.1-17.9-40-40-40H232V264h432v624z"}}]},name:"copy",theme:"outlined"},b4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:y4}))},S4=f.forwardRef(b4),C4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M884 256h-75c-5.1 0-9.9 2.5-12.9 6.6L512 654.2 227.9 262.6c-3-4.1-7.8-6.6-12.9-6.6h-75c-6.5 0-10.3 7.4-6.5 12.7l352.6 486.1c12.8 17.6 39 17.6 51.7 0l352.6-486.1c3.9-5.3.1-12.7-6.4-12.7z"}}]},name:"down",theme:"outlined"},x4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:C4}))},w4=f.forwardRef(x4),E4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M257.7 752c2 0 4-.2 6-.5L431.9 722c2-.4 3.9-1.3 5.3-2.8l423.9-423.9a9.96 9.96 0 000-14.1L694.9 114.9c-1.9-1.9-4.4-2.9-7.1-2.9s-5.2 1-7.1 2.9L256.8 538.8c-1.5 1.5-2.4 3.3-2.8 5.3l-29.5 168.2a33.5 33.5 0 009.4 29.8c6.6 6.4 14.9 9.9 23.8 9.9zm67.4-174.4L687.8 215l73.3 73.3-362.7 362.6-88.9 15.7 15.6-89zM880 836H144c-17.7 0-32 14.3-32 32v36c0 4.4 3.6 8 8 8h784c4.4 0 8-3.6 8-8v-36c0-17.7-14.3-32-32-32z"}}]},name:"edit",theme:"outlined"},$4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:E4}))},O4=f.forwardRef($4),R4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M176 511a56 56 0 10112 0 56 56 0 10-112 0zm280 0a56 56 0 10112 0 56 56 0 10-112 0zm280 0a56 56 0 10112 0 56 56 0 10-112 0z"}}]},name:"ellipsis",theme:"outlined"},_4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:R4}))},T4=f.forwardRef(_4),M4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M864 170h-60c-4.4 0-8 3.6-8 8v518H310v-73c0-6.7-7.8-10.5-13-6.3l-141.9 112a8 8 0 000 12.6l141.9 112c5.3 4.2 13 .4 13-6.3v-75h498c35.3 0 64-28.7 64-64V178c0-4.4-3.6-8-8-8z"}}]},name:"enter",theme:"outlined"},N4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:M4}))},j4=f.forwardRef(N4),I4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64C264.6 64 64 264.6 64 512s200.6 448 448 448 448-200.6 448-448S759.4 64 512 64zm-32 232c0-4.4 3.6-8 8-8h48c4.4 0 8 3.6 8 8v272c0 4.4-3.6 8-8 8h-48c-4.4 0-8-3.6-8-8V296zm32 440a48.01 48.01 0 010-96 48.01 48.01 0 010 96z"}}]},name:"exclamation-circle",theme:"filled"},P4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:I4}))},us=f.forwardRef(P4),A4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M942.2 486.2Q889.47 375.11 816.7 305l-50.88 50.88C807.31 395.53 843.45 447.4 874.7 512 791.5 684.2 673.4 766 512 766q-72.67 0-133.87-22.38L323 798.75Q408 838 512 838q288.3 0 430.2-300.3a60.29 60.29 0 000-51.5zm-63.57-320.64L836 122.88a8 8 0 00-11.32 0L715.31 232.2Q624.86 186 512 186q-288.3 0-430.2 300.3a60.3 60.3 0 000 51.5q56.69 119.4 136.5 191.41L112.48 835a8 8 0 000 11.31L155.17 889a8 8 0 0011.31 0l712.15-712.12a8 8 0 000-11.32zM149.3 512C232.6 339.8 350.7 258 512 258c54.54 0 104.13 9.36 149.12 28.39l-70.3 70.3a176 176 0 00-238.13 238.13l-83.42 83.42C223.1 637.49 183.3 582.28 149.3 512zm246.7 0a112.11 112.11 0 01146.2-106.69L401.31 546.2A112 112 0 01396 512z"}},{tag:"path",attrs:{d:"M508 624c-3.46 0-6.87-.16-10.25-.47l-52.82 52.82a176.09 176.09 0 00227.42-227.42l-52.82 52.82c.31 3.38.47 6.79.47 10.25a111.94 111.94 0 01-112 112z"}}]},name:"eye-invisible",theme:"outlined"},z4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:A4}))},D4=f.forwardRef(z4),L4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M942.2 486.2C847.4 286.5 704.1 186 512 186c-192.2 0-335.4 100.5-430.2 300.3a60.3 60.3 0 000 51.5C176.6 737.5 319.9 838 512 838c192.2 0 335.4-100.5 430.2-300.3 7.7-16.2 7.7-35 0-51.5zM512 766c-161.3 0-279.4-81.8-362.7-254C232.6 339.8 350.7 258 512 258c161.3 0 279.4 81.8 362.7 254C791.5 684.2 673.4 766 512 766zm-4-430c-97.2 0-176 78.8-176 176s78.8 176 176 176 176-78.8 176-176-78.8-176-176-176zm0 288c-61.9 0-112-50.1-112-112s50.1-112 112-112 112 50.1 112 112-50.1 112-112 112z"}}]},name:"eye",theme:"outlined"},B4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:L4}))},H4=f.forwardRef(B4),F4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64C264.6 64 64 264.6 64 512s200.6 448 448 448 448-200.6 448-448S759.4 64 512 64zm32 664c0 4.4-3.6 8-8 8h-48c-4.4 0-8-3.6-8-8V456c0-4.4 3.6-8 8-8h48c4.4 0 8 3.6 8 8v272zm-32-344a48.01 48.01 0 010-96 48.01 48.01 0 010 96z"}}]},name:"info-circle",theme:"filled"},V4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:F4}))},q0=f.forwardRef(V4),U4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M956.9 845.1L896.4 632V168c0-17.7-14.3-32-32-32h-704c-17.7 0-32 14.3-32 32v464L67.9 845.1C60.4 866 75.8 888 98 888h828.8c22.2 0 37.6-22 30.1-42.9zM200.4 208h624v395h-624V208zm228.3 608l8.1-37h150.3l8.1 37H428.7zm224 0l-19.1-86.7c-.8-3.7-4.1-6.3-7.8-6.3H398.2c-3.8 0-7 2.6-7.8 6.3L371.3 816H151l42.3-149h638.2l42.3 149H652.7z"}}]},name:"laptop",theme:"outlined"},k4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:U4}))},FO=f.forwardRef(k4),q4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M724 218.3V141c0-6.7-7.7-10.4-12.9-6.3L260.3 486.8a31.86 31.86 0 000 50.3l450.8 352.1c5.3 4.1 12.9.4 12.9-6.3v-77.3c0-4.9-2.3-9.6-6.1-12.6l-360-281 360-281.1c3.8-3 6.1-7.7 6.1-12.6z"}}]},name:"left",theme:"outlined"},W4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:q4}))},Y1=f.forwardRef(W4),G4={icon:{tag:"svg",attrs:{viewBox:"0 0 1024 1024",focusable:"false"},children:[{tag:"path",attrs:{d:"M988 548c-19.9 0-36-16.1-36-36 0-59.4-11.6-117-34.6-171.3a440.45 440.45 0 00-94.3-139.9 437.71 437.71 0 00-139.9-94.3C629 83.6 571.4 72 512 72c-19.9 0-36-16.1-36-36s16.1-36 36-36c69.1 0 136.2 13.5 199.3 40.3C772.3 66 827 103 874 150c47 47 83.9 101.8 109.7 162.7 26.7 63.1 40.2 130.2 40.2 199.3.1 19.9-16 36-35.9 36z"}}]},name:"loading",theme:"outlined"},K4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:G4}))},Cu=f.forwardRef(K4),X4={icon:{tag:"svg",attrs:{"fill-rule":"evenodd",viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M489.5 111.66c30.65-1.8 45.98 36.44 22.58 56.33A243.35 243.35 0 00426 354c0 134.76 109.24 244 244 244 72.58 0 139.9-31.83 186.01-86.08 19.87-23.38 58.07-8.1 56.34 22.53C900.4 745.82 725.15 912 512.5 912 291.31 912 112 732.69 112 511.5c0-211.39 164.29-386.02 374.2-399.65l.2-.01zm-81.15 79.75l-4.11 1.36C271.1 237.94 176 364.09 176 511.5 176 697.34 326.66 848 512.5 848c148.28 0 274.94-96.2 319.45-230.41l.63-1.93-.11.07a307.06 307.06 0 01-159.73 46.26L670 662c-170.1 0-308-137.9-308-308 0-58.6 16.48-114.54 46.27-162.47z"}}]},name:"moon",theme:"outlined"},Y4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:X4}))},Q4=f.forwardRef(Y4),Z4={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M880 112c-3.8 0-7.7.7-11.6 2.3L292 345.9H128c-8.8 0-16 7.4-16 16.6v299c0 9.2 7.2 16.6 16 16.6h101.7c-3.7 11.6-5.7 23.9-5.7 36.4 0 65.9 53.8 119.5 120 119.5 55.4 0 102.1-37.6 115.9-88.4l408.6 164.2c3.9 1.5 7.8 2.3 11.6 2.3 16.9 0 32-14.2 32-33.2V145.2C912 126.2 897 112 880 112zM344 762.3c-26.5 0-48-21.4-48-47.8 0-11.2 3.9-21.9 11-30.4l84.9 34.1c-2 24.6-22.7 44.1-47.9 44.1zm496 58.4L318.8 611.3l-12.9-5.2H184V417.9h121.9l12.9-5.2L840 203.3v617.4z"}}]},name:"notification",theme:"outlined"},J4=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:Z4}))},VO=f.forwardRef(J4),eP={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64C264.6 64 64 264.6 64 512s200.6 448 448 448 448-200.6 448-448S759.4 64 512 64zm0 820c-205.4 0-372-166.6-372-372s166.6-372 372-372 372 166.6 372 372-166.6 372-372 372z"}},{tag:"path",attrs:{d:"M623.6 316.7C593.6 290.4 554 276 512 276s-81.6 14.5-111.6 40.7C369.2 344 352 380.7 352 420v7.6c0 4.4 3.6 8 8 8h48c4.4 0 8-3.6 8-8V420c0-44.1 43.1-80 96-80s96 35.9 96 80c0 31.1-22 59.6-56.1 72.7-21.2 8.1-39.2 22.3-52.1 40.9-13.1 19-19.9 41.8-19.9 64.9V620c0 4.4 3.6 8 8 8h48c4.4 0 8-3.6 8-8v-22.7a48.3 48.3 0 0130.9-44.8c59-22.7 97.1-74.7 97.1-132.5.1-39.3-17.1-76-48.3-103.3zM472 732a40 40 0 1080 0 40 40 0 10-80 0z"}}]},name:"question-circle",theme:"outlined"},tP=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:eP}))},nP=f.forwardRef(tP),rP={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M765.7 486.8L314.9 134.7A7.97 7.97 0 00302 141v77.3c0 4.9 2.3 9.6 6.1 12.6l360 281.1-360 281.1c-3.9 3-6.1 7.7-6.1 12.6V883c0 6.7 7.7 10.4 12.9 6.3l450.8-352.1a31.96 31.96 0 000-50.4z"}}]},name:"right",theme:"outlined"},aP=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:rP}))},Dv=f.forwardRef(aP),oP={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M909.6 854.5L649.9 594.8C690.2 542.7 712 479 712 412c0-80.2-31.3-155.4-87.9-212.1-56.6-56.7-132-87.9-212.1-87.9s-155.5 31.3-212.1 87.9C143.2 256.5 112 331.8 112 412c0 80.1 31.3 155.5 87.9 212.1C256.5 680.8 331.8 712 412 712c67 0 130.6-21.8 182.7-62l259.7 259.6a8.2 8.2 0 0011.6 0l43.6-43.5a8.2 8.2 0 000-11.6zM570.4 570.4C528 612.7 471.8 636 412 636s-116-23.3-158.4-65.6C211.3 528 188 471.8 188 412s23.3-116.1 65.6-158.4C296 211.3 352.2 188 412 188s116.1 23.2 158.4 65.6S636 352.2 636 412s-23.3 116.1-65.6 158.4z"}}]},name:"search",theme:"outlined"},iP=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:oP}))},UO=f.forwardRef(iP),lP={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M924.8 625.7l-65.5-56c3.1-19 4.7-38.4 4.7-57.8s-1.6-38.8-4.7-57.8l65.5-56a32.03 32.03 0 009.3-35.2l-.9-2.6a443.74 443.74 0 00-79.7-137.9l-1.8-2.1a32.12 32.12 0 00-35.1-9.5l-81.3 28.9c-30-24.6-63.5-44-99.7-57.6l-15.7-85a32.05 32.05 0 00-25.8-25.7l-2.7-.5c-52.1-9.4-106.9-9.4-159 0l-2.7.5a32.05 32.05 0 00-25.8 25.7l-15.8 85.4a351.86 351.86 0 00-99 57.4l-81.9-29.1a32 32 0 00-35.1 9.5l-1.8 2.1a446.02 446.02 0 00-79.7 137.9l-.9 2.6c-4.5 12.5-.8 26.5 9.3 35.2l66.3 56.6c-3.1 18.8-4.6 38-4.6 57.1 0 19.2 1.5 38.4 4.6 57.1L99 625.5a32.03 32.03 0 00-9.3 35.2l.9 2.6c18.1 50.4 44.9 96.9 79.7 137.9l1.8 2.1a32.12 32.12 0 0035.1 9.5l81.9-29.1c29.8 24.5 63.1 43.9 99 57.4l15.8 85.4a32.05 32.05 0 0025.8 25.7l2.7.5a449.4 449.4 0 00159 0l2.7-.5a32.05 32.05 0 0025.8-25.7l15.7-85a350 350 0 0099.7-57.6l81.3 28.9a32 32 0 0035.1-9.5l1.8-2.1c34.8-41.1 61.6-87.5 79.7-137.9l.9-2.6c4.5-12.3.8-26.3-9.3-35zM788.3 465.9c2.5 15.1 3.8 30.6 3.8 46.1s-1.3 31-3.8 46.1l-6.6 40.1 74.7 63.9a370.03 370.03 0 01-42.6 73.6L721 702.8l-31.4 25.8c-23.9 19.6-50.5 35-79.3 45.8l-38.1 14.3-17.9 97a377.5 377.5 0 01-85 0l-17.9-97.2-37.8-14.5c-28.5-10.8-55-26.2-78.7-45.7l-31.4-25.9-93.4 33.2c-17-22.9-31.2-47.6-42.6-73.6l75.5-64.5-6.5-40c-2.4-14.9-3.7-30.3-3.7-45.5 0-15.3 1.2-30.6 3.7-45.5l6.5-40-75.5-64.5c11.3-26.1 25.6-50.7 42.6-73.6l93.4 33.2 31.4-25.9c23.7-19.5 50.2-34.9 78.7-45.7l37.9-14.3 17.9-97.2c28.1-3.2 56.8-3.2 85 0l17.9 97 38.1 14.3c28.7 10.8 55.4 26.2 79.3 45.8l31.4 25.8 92.8-32.9c17 22.9 31.2 47.6 42.6 73.6L781.8 426l6.5 39.9zM512 326c-97.2 0-176 78.8-176 176s78.8 176 176 176 176-78.8 176-176-78.8-176-176-176zm79.2 255.2A111.6 111.6 0 01512 614c-29.9 0-58-11.7-79.2-32.8A111.6 111.6 0 01400 502c0-29.9 11.7-58 32.8-79.2C454 401.6 482.1 390 512 390c29.9 0 58 11.6 79.2 32.8A111.6 111.6 0 01624 502c0 29.9-11.7 58-32.8 79.2z"}}]},name:"setting",theme:"outlined"},sP=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:lP}))},cP=f.forwardRef(sP),uP={icon:{tag:"svg",attrs:{"fill-rule":"evenodd",viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M548 818v126a16 16 0 01-16 16h-40a16 16 0 01-16-16V818c15.85 1.64 27.84 2.46 36 2.46 8.15 0 20.16-.82 36-2.46m205.25-115.66l89.1 89.1a16 16 0 010 22.62l-28.29 28.29a16 16 0 01-22.62 0l-89.1-89.1c12.37-10.04 21.43-17.95 27.2-23.71 5.76-5.77 13.67-14.84 23.71-27.2m-482.5 0c10.04 12.36 17.95 21.43 23.71 27.2 5.77 5.76 14.84 13.67 27.2 23.71l-89.1 89.1a16 16 0 01-22.62 0l-28.29-28.29a16 16 0 010-22.63zM512 278c129.24 0 234 104.77 234 234S641.24 746 512 746 278 641.24 278 512s104.77-234 234-234m0 72c-89.47 0-162 72.53-162 162s72.53 162 162 162 162-72.53 162-162-72.53-162-162-162M206 476c-1.64 15.85-2.46 27.84-2.46 36 0 8.15.82 20.16 2.46 36H80a16 16 0 01-16-16v-40a16 16 0 0116-16zm738 0a16 16 0 0116 16v40a16 16 0 01-16 16H818c1.64-15.85 2.46-27.84 2.46-36 0-8.15-.82-20.16-2.46-36zM814.06 180.65l28.29 28.29a16 16 0 010 22.63l-89.1 89.09c-10.04-12.37-17.95-21.43-23.71-27.2-5.77-5.76-14.84-13.67-27.2-23.71l89.1-89.1a16 16 0 0122.62 0m-581.5 0l89.1 89.1c-12.37 10.04-21.43 17.95-27.2 23.71-5.76 5.77-13.67 14.84-23.71 27.2l-89.1-89.1a16 16 0 010-22.62l28.29-28.29a16 16 0 0122.62 0M532 64a16 16 0 0116 16v126c-15.85-1.64-27.84-2.46-36-2.46-8.15 0-20.16.82-36 2.46V80a16 16 0 0116-16z"}}]},name:"sun",theme:"outlined"},fP=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:uP}))},dP=f.forwardRef(fP),mP={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M858.5 763.6a374 374 0 00-80.6-119.5 375.63 375.63 0 00-119.5-80.6c-.4-.2-.8-.3-1.2-.5C719.5 518 760 444.7 760 362c0-137-111-248-248-248S264 225 264 362c0 82.7 40.5 156 102.8 201.1-.4.2-.8.3-1.2.5-44.8 18.9-85 46-119.5 80.6a375.63 375.63 0 00-80.6 119.5A371.7 371.7 0 00136 901.8a8 8 0 008 8.2h60c4.4 0 7.9-3.5 8-7.8 2-77.2 33-149.5 87.8-204.3 56.7-56.7 132-87.9 212.2-87.9s155.5 31.2 212.2 87.9C779 752.7 810 825 812 902.2c.1 4.4 3.6 7.8 8 7.8h60a8 8 0 008-8.2c-1-47.8-10.9-94.3-29.5-138.2zM512 534c-45.9 0-89.1-17.9-121.6-50.4S340 407.9 340 362c0-45.9 17.9-89.1 50.4-121.6S466.1 190 512 190s89.1 17.9 121.6 50.4S684 316.1 684 362c0 45.9-17.9 89.1-50.4 121.6S557.9 534 512 534z"}}]},name:"user",theme:"outlined"},gP=function(t,n){return f.createElement(on,je({},t,{ref:n,icon:mP}))},kO=f.forwardRef(gP),up={exports:{}},an={};/**
 * @license React
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Q1;function hP(){if(Q1)return an;Q1=1;var e=Symbol.for("react.element"),t=Symbol.for("react.portal"),n=Symbol.for("react.fragment"),r=Symbol.for("react.strict_mode"),a=Symbol.for("react.profiler"),l=Symbol.for("react.provider"),s=Symbol.for("react.context"),u=Symbol.for("react.server_context"),d=Symbol.for("react.forward_ref"),g=Symbol.for("react.suspense"),h=Symbol.for("react.suspense_list"),p=Symbol.for("react.memo"),y=Symbol.for("react.lazy"),C=Symbol.for("react.offscreen"),b;b=Symbol.for("react.module.reference");function x(S){if(typeof S=="object"&&S!==null){var E=S.$$typeof;switch(E){case e:switch(S=S.type,S){case n:case a:case r:case g:case h:return S;default:switch(S=S&&S.$$typeof,S){case u:case s:case d:case y:case p:case l:return S;default:return E}}case t:return E}}}return an.ContextConsumer=s,an.ContextProvider=l,an.Element=e,an.ForwardRef=d,an.Fragment=n,an.Lazy=y,an.Memo=p,an.Portal=t,an.Profiler=a,an.StrictMode=r,an.Suspense=g,an.SuspenseList=h,an.isAsyncMode=function(){return!1},an.isConcurrentMode=function(){return!1},an.isContextConsumer=function(S){return x(S)===s},an.isContextProvider=function(S){return x(S)===l},an.isElement=function(S){return typeof S=="object"&&S!==null&&S.$$typeof===e},an.isForwardRef=function(S){return x(S)===d},an.isFragment=function(S){return x(S)===n},an.isLazy=function(S){return x(S)===y},an.isMemo=function(S){return x(S)===p},an.isPortal=function(S){return x(S)===t},an.isProfiler=function(S){return x(S)===a},an.isStrictMode=function(S){return x(S)===r},an.isSuspense=function(S){return x(S)===g},an.isSuspenseList=function(S){return x(S)===h},an.isValidElementType=function(S){return typeof S=="string"||typeof S=="function"||S===n||S===a||S===r||S===g||S===h||S===C||typeof S=="object"&&S!==null&&(S.$$typeof===y||S.$$typeof===p||S.$$typeof===l||S.$$typeof===s||S.$$typeof===d||S.$$typeof===b||S.getModuleId!==void 0)},an.typeOf=x,an}var Z1;function pP(){return Z1||(Z1=1,up.exports=hP()),up.exports}var fp=pP();function xu(e,t,n){var r=f.useRef({});return(!("value"in r.current)||n(r.current.condition,t))&&(r.current.value=e(),r.current.condition=t),r.current.value}var vP=Symbol.for("react.element"),yP=Symbol.for("react.transitional.element"),bP=Symbol.for("react.fragment");function qO(e){return e&&st(e)==="object"&&(e.$$typeof===vP||e.$$typeof===yP)&&e.type===bP}var SP=Number(f.version.split(".")[0]),W0=function(t,n){typeof t=="function"?t(n):st(t)==="object"&&t&&"current"in t&&(t.current=n)},Ha=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];var a=n.filter(Boolean);return a.length<=1?a[0]:function(l){n.forEach(function(s){W0(s,l)})}},Io=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return xu(function(){return Ha.apply(void 0,n)},n,function(a,l){return a.length!==l.length||a.every(function(s,u){return s!==l[u]})})},rl=function(t){var n,r;if(!t)return!1;if(G0(t)&&SP>=19)return!0;var a=fp.isMemo(t)?t.type.type:t.type;return!(typeof a=="function"&&!((n=a.prototype)!==null&&n!==void 0&&n.render)&&a.$$typeof!==fp.ForwardRef||typeof t=="function"&&!((r=t.prototype)!==null&&r!==void 0&&r.render)&&t.$$typeof!==fp.ForwardRef)};function G0(e){return f.isValidElement(e)&&!qO(e)}var CP=function(t){return G0(t)&&rl(t)},fs=function(t){if(t&&G0(t)){var n=t;return n.props.propertyIsEnumerable("ref")?n.props.ref:n.ref}return null};function Lr(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=[];return se.Children.forEach(e,function(r){r==null&&!t.keepEmpty||(Array.isArray(r)?n=n.concat(Lr(r)):qO(r)&&r.props?n=n.concat(Lr(r.props.children,t)):n.push(r))}),n}var pi=OO();const Lv=is(pi),xP=$O({__proto__:null,default:Lv},[pi]);function Jc(e){return e instanceof HTMLElement||e instanceof SVGElement}function WO(e){return e&&st(e)==="object"&&Jc(e.nativeElement)?e.nativeElement:Jc(e)?e:null}function sd(e){var t=WO(e);if(t)return t;if(e instanceof se.Component){var n;return(n=Lv.findDOMNode)===null||n===void 0?void 0:n.call(Lv,e)}return null}var Bv=f.createContext(null);function wP(e){var t=e.children,n=e.onBatchResize,r=f.useRef(0),a=f.useRef([]),l=f.useContext(Bv),s=f.useCallback(function(u,d,g){r.current+=1;var h=r.current;a.current.push({size:u,element:d,data:g}),Promise.resolve().then(function(){h===r.current&&(n==null||n(a.current),a.current=[])}),l==null||l(u,d,g)},[n,l]);return f.createElement(Bv.Provider,{value:s},t)}var GO=function(){if(typeof Map<"u")return Map;function e(t,n){var r=-1;return t.some(function(a,l){return a[0]===n?(r=l,!0):!1}),r}return function(){function t(){this.__entries__=[]}return Object.defineProperty(t.prototype,"size",{get:function(){return this.__entries__.length},enumerable:!0,configurable:!0}),t.prototype.get=function(n){var r=e(this.__entries__,n),a=this.__entries__[r];return a&&a[1]},t.prototype.set=function(n,r){var a=e(this.__entries__,n);~a?this.__entries__[a][1]=r:this.__entries__.push([n,r])},t.prototype.delete=function(n){var r=this.__entries__,a=e(r,n);~a&&r.splice(a,1)},t.prototype.has=function(n){return!!~e(this.__entries__,n)},t.prototype.clear=function(){this.__entries__.splice(0)},t.prototype.forEach=function(n,r){r===void 0&&(r=null);for(var a=0,l=this.__entries__;a<l.length;a++){var s=l[a];n.call(r,s[1],s[0])}},t}()}(),Hv=typeof window<"u"&&typeof document<"u"&&window.document===document,Md=function(){return typeof global<"u"&&global.Math===Math?global:typeof self<"u"&&self.Math===Math?self:typeof window<"u"&&window.Math===Math?window:Function("return this")()}(),EP=function(){return typeof requestAnimationFrame=="function"?requestAnimationFrame.bind(Md):function(e){return setTimeout(function(){return e(Date.now())},1e3/60)}}(),$P=2;function OP(e,t){var n=!1,r=!1,a=0;function l(){n&&(n=!1,e()),r&&u()}function s(){EP(l)}function u(){var d=Date.now();if(n){if(d-a<$P)return;r=!0}else n=!0,r=!1,setTimeout(s,t);a=d}return u}var RP=20,_P=["top","right","bottom","left","width","height","size","weight"],TP=typeof MutationObserver<"u",MP=function(){function e(){this.connected_=!1,this.mutationEventsAdded_=!1,this.mutationsObserver_=null,this.observers_=[],this.onTransitionEnd_=this.onTransitionEnd_.bind(this),this.refresh=OP(this.refresh.bind(this),RP)}return e.prototype.addObserver=function(t){~this.observers_.indexOf(t)||this.observers_.push(t),this.connected_||this.connect_()},e.prototype.removeObserver=function(t){var n=this.observers_,r=n.indexOf(t);~r&&n.splice(r,1),!n.length&&this.connected_&&this.disconnect_()},e.prototype.refresh=function(){var t=this.updateObservers_();t&&this.refresh()},e.prototype.updateObservers_=function(){var t=this.observers_.filter(function(n){return n.gatherActive(),n.hasActive()});return t.forEach(function(n){return n.broadcastActive()}),t.length>0},e.prototype.connect_=function(){!Hv||this.connected_||(document.addEventListener("transitionend",this.onTransitionEnd_),window.addEventListener("resize",this.refresh),TP?(this.mutationsObserver_=new MutationObserver(this.refresh),this.mutationsObserver_.observe(document,{attributes:!0,childList:!0,characterData:!0,subtree:!0})):(document.addEventListener("DOMSubtreeModified",this.refresh),this.mutationEventsAdded_=!0),this.connected_=!0)},e.prototype.disconnect_=function(){!Hv||!this.connected_||(document.removeEventListener("transitionend",this.onTransitionEnd_),window.removeEventListener("resize",this.refresh),this.mutationsObserver_&&this.mutationsObserver_.disconnect(),this.mutationEventsAdded_&&document.removeEventListener("DOMSubtreeModified",this.refresh),this.mutationsObserver_=null,this.mutationEventsAdded_=!1,this.connected_=!1)},e.prototype.onTransitionEnd_=function(t){var n=t.propertyName,r=n===void 0?"":n,a=_P.some(function(l){return!!~r.indexOf(l)});a&&this.refresh()},e.getInstance=function(){return this.instance_||(this.instance_=new e),this.instance_},e.instance_=null,e}(),KO=function(e,t){for(var n=0,r=Object.keys(t);n<r.length;n++){var a=r[n];Object.defineProperty(e,a,{value:t[a],enumerable:!1,writable:!1,configurable:!0})}return e},Zl=function(e){var t=e&&e.ownerDocument&&e.ownerDocument.defaultView;return t||Md},XO=um(0,0,0,0);function Nd(e){return parseFloat(e)||0}function J1(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return t.reduce(function(r,a){var l=e["border-"+a+"-width"];return r+Nd(l)},0)}function NP(e){for(var t=["top","right","bottom","left"],n={},r=0,a=t;r<a.length;r++){var l=a[r],s=e["padding-"+l];n[l]=Nd(s)}return n}function jP(e){var t=e.getBBox();return um(0,0,t.width,t.height)}function IP(e){var t=e.clientWidth,n=e.clientHeight;if(!t&&!n)return XO;var r=Zl(e).getComputedStyle(e),a=NP(r),l=a.left+a.right,s=a.top+a.bottom,u=Nd(r.width),d=Nd(r.height);if(r.boxSizing==="border-box"&&(Math.round(u+l)!==t&&(u-=J1(r,"left","right")+l),Math.round(d+s)!==n&&(d-=J1(r,"top","bottom")+s)),!AP(e)){var g=Math.round(u+l)-t,h=Math.round(d+s)-n;Math.abs(g)!==1&&(u-=g),Math.abs(h)!==1&&(d-=h)}return um(a.left,a.top,u,d)}var PP=function(){return typeof SVGGraphicsElement<"u"?function(e){return e instanceof Zl(e).SVGGraphicsElement}:function(e){return e instanceof Zl(e).SVGElement&&typeof e.getBBox=="function"}}();function AP(e){return e===Zl(e).document.documentElement}function zP(e){return Hv?PP(e)?jP(e):IP(e):XO}function DP(e){var t=e.x,n=e.y,r=e.width,a=e.height,l=typeof DOMRectReadOnly<"u"?DOMRectReadOnly:Object,s=Object.create(l.prototype);return KO(s,{x:t,y:n,width:r,height:a,top:n,right:t+r,bottom:a+n,left:t}),s}function um(e,t,n,r){return{x:e,y:t,width:n,height:r}}var LP=function(){function e(t){this.broadcastWidth=0,this.broadcastHeight=0,this.contentRect_=um(0,0,0,0),this.target=t}return e.prototype.isActive=function(){var t=zP(this.target);return this.contentRect_=t,t.width!==this.broadcastWidth||t.height!==this.broadcastHeight},e.prototype.broadcastRect=function(){var t=this.contentRect_;return this.broadcastWidth=t.width,this.broadcastHeight=t.height,t},e}(),BP=function(){function e(t,n){var r=DP(n);KO(this,{target:t,contentRect:r})}return e}(),HP=function(){function e(t,n,r){if(this.activeObservations_=[],this.observations_=new GO,typeof t!="function")throw new TypeError("The callback provided as parameter 1 is not a function.");this.callback_=t,this.controller_=n,this.callbackCtx_=r}return e.prototype.observe=function(t){if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");if(!(typeof Element>"u"||!(Element instanceof Object))){if(!(t instanceof Zl(t).Element))throw new TypeError('parameter 1 is not of type "Element".');var n=this.observations_;n.has(t)||(n.set(t,new LP(t)),this.controller_.addObserver(this),this.controller_.refresh())}},e.prototype.unobserve=function(t){if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");if(!(typeof Element>"u"||!(Element instanceof Object))){if(!(t instanceof Zl(t).Element))throw new TypeError('parameter 1 is not of type "Element".');var n=this.observations_;n.has(t)&&(n.delete(t),n.size||this.controller_.removeObserver(this))}},e.prototype.disconnect=function(){this.clearActive(),this.observations_.clear(),this.controller_.removeObserver(this)},e.prototype.gatherActive=function(){var t=this;this.clearActive(),this.observations_.forEach(function(n){n.isActive()&&t.activeObservations_.push(n)})},e.prototype.broadcastActive=function(){if(this.hasActive()){var t=this.callbackCtx_,n=this.activeObservations_.map(function(r){return new BP(r.target,r.broadcastRect())});this.callback_.call(t,n,t),this.clearActive()}},e.prototype.clearActive=function(){this.activeObservations_.splice(0)},e.prototype.hasActive=function(){return this.activeObservations_.length>0},e}(),YO=typeof WeakMap<"u"?new WeakMap:new GO,QO=function(){function e(t){if(!(this instanceof e))throw new TypeError("Cannot call a class as a function.");if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");var n=MP.getInstance(),r=new HP(t,n,this);YO.set(this,r)}return e}();["observe","unobserve","disconnect"].forEach(function(e){QO.prototype[e]=function(){var t;return(t=YO.get(this))[e].apply(t,arguments)}});var FP=function(){return typeof Md.ResizeObserver<"u"?Md.ResizeObserver:QO}(),mi=new Map;function VP(e){e.forEach(function(t){var n,r=t.target;(n=mi.get(r))===null||n===void 0||n.forEach(function(a){return a(r)})})}var ZO=new FP(VP);function UP(e,t){mi.has(e)||(mi.set(e,new Set),ZO.observe(e)),mi.get(e).add(t)}function kP(e,t){mi.has(e)&&(mi.get(e).delete(t),mi.get(e).size||(ZO.unobserve(e),mi.delete(e)))}function Wn(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function ex(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,TO(r.key),r)}}function Gn(e,t,n){return t&&ex(e.prototype,t),n&&ex(e,n),Object.defineProperty(e,"prototype",{writable:!1}),e}function eu(e,t){return eu=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(n,r){return n.__proto__=r,n},eu(e,t)}function Po(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&eu(e,t)}function tu(e){return tu=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(t){return t.__proto__||Object.getPrototypeOf(t)},tu(e)}function K0(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(K0=function(){return!!e})()}function jt(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function qP(e,t){if(t&&(st(t)=="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return jt(e)}function Ao(e){var t=K0();return function(){var n,r=tu(e);if(t){var a=tu(this).constructor;n=Reflect.construct(r,arguments,a)}else n=r.apply(this,arguments);return qP(this,n)}}var WP=function(e){Po(n,e);var t=Ao(n);function n(){return Wn(this,n),t.apply(this,arguments)}return Gn(n,[{key:"render",value:function(){return this.props.children}}]),n}(f.Component);function GP(e,t){var n=e.children,r=e.disabled,a=f.useRef(null),l=f.useRef(null),s=f.useContext(Bv),u=typeof n=="function",d=u?n(a):n,g=f.useRef({width:-1,height:-1,offsetWidth:-1,offsetHeight:-1}),h=!u&&f.isValidElement(d)&&rl(d),p=h?fs(d):null,y=Io(p,a),C=function(){var E;return sd(a.current)||(a.current&&st(a.current)==="object"?sd((E=a.current)===null||E===void 0?void 0:E.nativeElement):null)||sd(l.current)};f.useImperativeHandle(t,function(){return C()});var b=f.useRef(e);b.current=e;var x=f.useCallback(function(S){var E=b.current,$=E.onResize,O=E.data,_=S.getBoundingClientRect(),R=_.width,T=_.height,M=S.offsetWidth,z=S.offsetHeight,P=Math.floor(R),D=Math.floor(T);if(g.current.width!==P||g.current.height!==D||g.current.offsetWidth!==M||g.current.offsetHeight!==z){var F={width:P,height:D,offsetWidth:M,offsetHeight:z};g.current=F;var G=M===Math.round(R)?R:M,V=z===Math.round(T)?T:z,U=Q(Q({},F),{},{offsetWidth:G,offsetHeight:V});s==null||s(U,S,O),$&&Promise.resolve().then(function(){$(U,S)})}},[]);return f.useEffect(function(){var S=C();return S&&!r&&UP(S,x),function(){return kP(S,x)}},[a.current,r]),f.createElement(WP,{ref:l},h?f.cloneElement(d,{ref:y}):d)}var KP=f.forwardRef(GP),XP="rc-observer-key";function YP(e,t){var n=e.children,r=typeof n=="function"?[n]:Lr(n);return r.map(function(a,l){var s=(a==null?void 0:a.key)||"".concat(XP,"-").concat(l);return f.createElement(KP,je({},e,{key:s,ref:l===0?t:void 0}),a)})}var zo=f.forwardRef(YP);zo.Collection=wP;function QP(e){if(Array.isArray(e))return Cv(e)}function JO(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function ZP(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function Ne(e){return QP(e)||JO(e)||U0(e)||ZP()}var e2=function(t){return+setTimeout(t,16)},t2=function(t){return clearTimeout(t)};typeof window<"u"&&"requestAnimationFrame"in window&&(e2=function(t){return window.requestAnimationFrame(t)},t2=function(t){return window.cancelAnimationFrame(t)});var tx=0,X0=new Map;function n2(e){X0.delete(e)}var bn=function(t){var n=arguments.length>1&&arguments[1]!==void 0?arguments[1]:1;tx+=1;var r=tx;function a(l){if(l===0)n2(r),t();else{var s=e2(function(){a(l-1)});X0.set(r,s)}}return a(n),r};bn.cancel=function(e){var t=X0.get(e);return n2(e),t2(t)};function nu(e){for(var t=0,n,r=0,a=e.length;a>=4;++r,a-=4)n=e.charCodeAt(r)&255|(e.charCodeAt(++r)&255)<<8|(e.charCodeAt(++r)&255)<<16|(e.charCodeAt(++r)&255)<<24,n=(n&65535)*1540483477+((n>>>16)*59797<<16),n^=n>>>24,t=(n&65535)*1540483477+((n>>>16)*59797<<16)^(t&65535)*1540483477+((t>>>16)*59797<<16);switch(a){case 3:t^=(e.charCodeAt(r+2)&255)<<16;case 2:t^=(e.charCodeAt(r+1)&255)<<8;case 1:t^=e.charCodeAt(r)&255,t=(t&65535)*1540483477+((t>>>16)*59797<<16)}return t^=t>>>13,t=(t&65535)*1540483477+((t>>>16)*59797<<16),((t^t>>>15)>>>0).toString(36)}function ru(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!1,r=new Set;function a(l,s){var u=arguments.length>2&&arguments[2]!==void 0?arguments[2]:1,d=r.has(l);if(mr(!d,"Warning: There may be circular references"),d)return!1;if(l===s)return!0;if(n&&u>1)return!1;r.add(l);var g=u+1;if(Array.isArray(l)){if(!Array.isArray(s)||l.length!==s.length)return!1;for(var h=0;h<l.length;h++)if(!a(l[h],s[h],g))return!1;return!0}if(l&&s&&st(l)==="object"&&st(s)==="object"){var p=Object.keys(l);return p.length!==Object.keys(s).length?!1:p.every(function(y){return a(l[y],s[y],g)})}return!1}return a(e,t)}var JP="%";function Fv(e){return e.join(JP)}var e3=function(){function e(t){Wn(this,e),Z(this,"instanceId",void 0),Z(this,"cache",new Map),this.instanceId=t}return Gn(e,[{key:"get",value:function(n){return this.opGet(Fv(n))}},{key:"opGet",value:function(n){return this.cache.get(n)||null}},{key:"update",value:function(n,r){return this.opUpdate(Fv(n),r)}},{key:"opUpdate",value:function(n,r){var a=this.cache.get(n),l=r(a);l===null?this.cache.delete(n):this.cache.set(n,l)}}]),e}(),Jl="data-token-hash",Pa="data-css-hash",gi="__cssinjs_instance__";function t3(){var e=Math.random().toString(12).slice(2);if(typeof document<"u"&&document.head&&document.body){var t=document.body.querySelectorAll("style[".concat(Pa,"]"))||[],n=document.head.firstChild;Array.from(t).forEach(function(a){a[gi]=a[gi]||e,a[gi]===e&&document.head.insertBefore(a,n)});var r={};Array.from(document.querySelectorAll("style[".concat(Pa,"]"))).forEach(function(a){var l=a.getAttribute(Pa);if(r[l]){if(a[gi]===e){var s;(s=a.parentNode)===null||s===void 0||s.removeChild(a)}}else r[l]=!0})}return new e3(e)}var wu=f.createContext({hashPriority:"low",cache:t3(),defaultCache:!0});function n3(e,t){if(e.length!==t.length)return!1;for(var n=0;n<e.length;n++)if(e[n]!==t[n])return!1;return!0}var Y0=function(){function e(){Wn(this,e),Z(this,"cache",void 0),Z(this,"keys",void 0),Z(this,"cacheCallTimes",void 0),this.cache=new Map,this.keys=[],this.cacheCallTimes=0}return Gn(e,[{key:"size",value:function(){return this.keys.length}},{key:"internalGet",value:function(n){var r,a,l=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,s={map:this.cache};return n.forEach(function(u){if(!s)s=void 0;else{var d;s=(d=s)===null||d===void 0||(d=d.map)===null||d===void 0?void 0:d.get(u)}}),(r=s)!==null&&r!==void 0&&r.value&&l&&(s.value[1]=this.cacheCallTimes++),(a=s)===null||a===void 0?void 0:a.value}},{key:"get",value:function(n){var r;return(r=this.internalGet(n,!0))===null||r===void 0?void 0:r[0]}},{key:"has",value:function(n){return!!this.internalGet(n)}},{key:"set",value:function(n,r){var a=this;if(!this.has(n)){if(this.size()+1>e.MAX_CACHE_SIZE+e.MAX_CACHE_OFFSET){var l=this.keys.reduce(function(g,h){var p=ge(g,2),y=p[1];return a.internalGet(h)[1]<y?[h,a.internalGet(h)[1]]:g},[this.keys[0],this.cacheCallTimes]),s=ge(l,1),u=s[0];this.delete(u)}this.keys.push(n)}var d=this.cache;n.forEach(function(g,h){if(h===n.length-1)d.set(g,{value:[r,a.cacheCallTimes++]});else{var p=d.get(g);p?p.map||(p.map=new Map):d.set(g,{map:new Map}),d=d.get(g).map}})}},{key:"deleteByPath",value:function(n,r){var a=n.get(r[0]);if(r.length===1){var l;return a.map?n.set(r[0],{map:a.map}):n.delete(r[0]),(l=a.value)===null||l===void 0?void 0:l[0]}var s=this.deleteByPath(a.map,r.slice(1));return(!a.map||a.map.size===0)&&!a.value&&n.delete(r[0]),s}},{key:"delete",value:function(n){if(this.has(n))return this.keys=this.keys.filter(function(r){return!n3(r,n)}),this.deleteByPath(this.cache,n)}}]),e}();Z(Y0,"MAX_CACHE_SIZE",20);Z(Y0,"MAX_CACHE_OFFSET",5);var nx=0,r2=function(){function e(t){Wn(this,e),Z(this,"derivatives",void 0),Z(this,"id",void 0),this.derivatives=Array.isArray(t)?t:[t],this.id=nx,t.length===0&&(t.length>0,void 0),nx+=1}return Gn(e,[{key:"getDerivativeToken",value:function(n){return this.derivatives.reduce(function(r,a){return a(n,r)},void 0)}}]),e}(),dp=new Y0;function Vv(e){var t=Array.isArray(e)?e:[e];return dp.has(t)||dp.set(t,new r2(t)),dp.get(t)}var r3=new WeakMap,mp={};function a3(e,t){for(var n=r3,r=0;r<t.length;r+=1){var a=t[r];n.has(a)||n.set(a,new WeakMap),n=n.get(a)}return n.has(mp)||n.set(mp,e()),n.get(mp)}var rx=new WeakMap;function Gc(e){var t=rx.get(e)||"";return t||(Object.keys(e).forEach(function(n){var r=e[n];t+=n,r instanceof r2?t+=r.id:r&&st(r)==="object"?t+=Gc(r):t+=r}),t=nu(t),rx.set(e,t)),t}function ax(e,t){return nu("".concat(t,"_").concat(Gc(e)))}var Uv=$r();function ve(e){return typeof e=="number"?"".concat(e,"px"):e}function jd(e,t,n){var r,a=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{},l=arguments.length>4&&arguments[4]!==void 0?arguments[4]:!1;if(l)return e;var s=Q(Q({},a),{},(r={},Z(r,Jl,t),Z(r,Pa,n),r)),u=Object.keys(s).map(function(d){var g=s[d];return g?"".concat(d,'="').concat(g,'"'):null}).filter(function(d){return d}).join(" ");return"<style ".concat(u,">").concat(e,"</style>")}var cd=function(t){var n=arguments.length>1&&arguments[1]!==void 0?arguments[1]:"";return"--".concat(n?"".concat(n,"-"):"").concat(t).replace(/([a-z0-9])([A-Z])/g,"$1-$2").replace(/([A-Z]+)([A-Z][a-z0-9]+)/g,"$1-$2").replace(/([a-z])([A-Z0-9])/g,"$1-$2").toLowerCase()},o3=function(t,n,r){return Object.keys(t).length?".".concat(n).concat(r!=null&&r.scope?".".concat(r.scope):"","{").concat(Object.entries(t).map(function(a){var l=ge(a,2),s=l[0],u=l[1];return"".concat(s,":").concat(u,";")}).join(""),"}"):""},a2=function(t,n,r){var a={},l={};return Object.entries(t).forEach(function(s){var u,d,g=ge(s,2),h=g[0],p=g[1];if(r!=null&&(u=r.preserve)!==null&&u!==void 0&&u[h])l[h]=p;else if((typeof p=="string"||typeof p=="number")&&!(r!=null&&(d=r.ignore)!==null&&d!==void 0&&d[h])){var y,C=cd(h,r==null?void 0:r.prefix);a[C]=typeof p=="number"&&!(r!=null&&(y=r.unitless)!==null&&y!==void 0&&y[h])?"".concat(p,"px"):String(p),l[h]="var(".concat(C,")")}}),[l,o3(a,n,{scope:r==null?void 0:r.scope})]},ox=$r()?f.useLayoutEffect:f.useEffect,Gt=function(t,n){var r=f.useRef(!0);ox(function(){return t(r.current)},n),ox(function(){return r.current=!1,function(){r.current=!0}},[])},ix=function(t,n){Gt(function(r){if(!r)return t()},n)},i3=Q({},lm),lx=i3.useInsertionEffect,l3=function(t,n,r){f.useMemo(t,r),Gt(function(){return n(!0)},r)},s3=lx?function(e,t,n){return lx(function(){return e(),t()},n)}:l3,c3=Q({},lm),u3=c3.useInsertionEffect,f3=function(t){var n=[],r=!1;function a(l){r||n.push(l)}return f.useEffect(function(){return r=!1,function(){r=!0,n.length&&n.forEach(function(l){return l()})}},t),a},d3=function(){return function(t){t()}},m3=typeof u3<"u"?f3:d3;function Q0(e,t,n,r,a){var l=f.useContext(wu),s=l.cache,u=[e].concat(Ne(t)),d=Fv(u),g=m3([d]),h=function(b){s.opUpdate(d,function(x){var S=x||[void 0,void 0],E=ge(S,2),$=E[0],O=$===void 0?0:$,_=E[1],R=_,T=R||n(),M=[O,T];return b?b(M):M})};f.useMemo(function(){h()},[d]);var p=s.opGet(d),y=p[1];return s3(function(){a==null||a(y)},function(C){return h(function(b){var x=ge(b,2),S=x[0],E=x[1];return C&&S===0&&(a==null||a(y)),[S+1,E]}),function(){s.opUpdate(d,function(b){var x=b||[],S=ge(x,2),E=S[0],$=E===void 0?0:E,O=S[1],_=$-1;return _===0?(g(function(){(C||!s.opGet(d))&&(r==null||r(O,!1))}),null):[$-1,O]})}},[d]),y}var g3={},h3="css",Ui=new Map;function p3(e){Ui.set(e,(Ui.get(e)||0)+1)}function v3(e,t){if(typeof document<"u"){var n=document.querySelectorAll("style[".concat(Jl,'="').concat(e,'"]'));n.forEach(function(r){if(r[gi]===t){var a;(a=r.parentNode)===null||a===void 0||a.removeChild(r)}})}}var y3=0;function b3(e,t){Ui.set(e,(Ui.get(e)||0)-1);var n=Array.from(Ui.keys()),r=n.filter(function(a){var l=Ui.get(a)||0;return l<=0});n.length-r.length>y3&&r.forEach(function(a){v3(a,t),Ui.delete(a)})}var S3=function(t,n,r,a){var l=r.getDerivativeToken(t),s=Q(Q({},l),n);return a&&(s=a(s)),s},o2="token";function C3(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},r=f.useContext(wu),a=r.cache.instanceId,l=r.container,s=n.salt,u=s===void 0?"":s,d=n.override,g=d===void 0?g3:d,h=n.formatToken,p=n.getComputedToken,y=n.cssVar,C=a3(function(){return Object.assign.apply(Object,[{}].concat(Ne(t)))},t),b=Gc(C),x=Gc(g),S=y?Gc(y):"",E=Q0(o2,[u,e.id,b,x,S],function(){var $,O=p?p(C,g,e):S3(C,g,e,h),_=Q({},O),R="";if(y){var T=a2(O,y.key,{prefix:y.prefix,ignore:y.ignore,unitless:y.unitless,preserve:y.preserve}),M=ge(T,2);O=M[0],R=M[1]}var z=ax(O,u);O._tokenKey=z,_._tokenKey=ax(_,u);var P=($=y==null?void 0:y.key)!==null&&$!==void 0?$:z;O._themeKey=P,p3(P);var D="".concat(h3,"-").concat(nu(z));return O._hashId=D,[O,D,_,R,(y==null?void 0:y.key)||""]},function($){b3($[0]._themeKey,a)},function($){var O=ge($,4),_=O[0],R=O[3];if(y&&R){var T=xo(R,nu("css-variables-".concat(_._themeKey)),{mark:Pa,prepend:"queue",attachTo:l,priority:-999});T[gi]=a,T.setAttribute(Jl,_._themeKey)}});return E}var x3=function(t,n,r){var a=ge(t,5),l=a[2],s=a[3],u=a[4],d=r||{},g=d.plain;if(!s)return null;var h=l._tokenKey,p=-999,y={"data-rc-order":"prependQueue","data-rc-priority":"".concat(p)},C=jd(s,u,h,y,g);return[p,h,C]},w3={animationIterationCount:1,borderImageOutset:1,borderImageSlice:1,borderImageWidth:1,boxFlex:1,boxFlexGroup:1,boxOrdinalGroup:1,columnCount:1,columns:1,flex:1,flexGrow:1,flexPositive:1,flexShrink:1,flexNegative:1,flexOrder:1,gridRow:1,gridRowEnd:1,gridRowSpan:1,gridRowStart:1,gridColumn:1,gridColumnEnd:1,gridColumnSpan:1,gridColumnStart:1,msGridRow:1,msGridRowSpan:1,msGridColumn:1,msGridColumnSpan:1,fontWeight:1,lineHeight:1,opacity:1,order:1,orphans:1,tabSize:1,widows:1,zIndex:1,zoom:1,WebkitLineClamp:1,fillOpacity:1,floodOpacity:1,stopOpacity:1,strokeDasharray:1,strokeDashoffset:1,strokeMiterlimit:1,strokeOpacity:1,strokeWidth:1},i2="comm",l2="rule",s2="decl",E3="@import",$3="@namespace",O3="@keyframes",R3="@layer",c2=Math.abs,Z0=String.fromCharCode;function u2(e){return e.trim()}function ud(e,t,n){return e.replace(t,n)}function _3(e,t,n){return e.indexOf(t,n)}function Xl(e,t){return e.charCodeAt(t)|0}function es(e,t,n){return e.slice(t,n)}function Za(e){return e.length}function T3(e){return e.length}function qf(e,t){return t.push(e),e}var fm=1,ts=1,f2=0,Ea=0,qn=0,ds="";function J0(e,t,n,r,a,l,s,u){return{value:e,root:t,parent:n,type:r,props:a,children:l,line:fm,column:ts,length:s,return:"",siblings:u}}function M3(){return qn}function N3(){return qn=Ea>0?Xl(ds,--Ea):0,ts--,qn===10&&(ts=1,fm--),qn}function Aa(){return qn=Ea<f2?Xl(ds,Ea++):0,ts++,qn===10&&(ts=1,fm++),qn}function hi(){return Xl(ds,Ea)}function fd(){return Ea}function dm(e,t){return es(ds,e,t)}function au(e){switch(e){case 0:case 9:case 10:case 13:case 32:return 5;case 33:case 43:case 44:case 47:case 62:case 64:case 126:case 59:case 123:case 125:return 4;case 58:return 3;case 34:case 39:case 40:case 91:return 2;case 41:case 93:return 1}return 0}function j3(e){return fm=ts=1,f2=Za(ds=e),Ea=0,[]}function I3(e){return ds="",e}function gp(e){return u2(dm(Ea-1,kv(e===91?e+2:e===40?e+1:e)))}function P3(e){for(;(qn=hi())&&qn<33;)Aa();return au(e)>2||au(qn)>3?"":" "}function A3(e,t){for(;--t&&Aa()&&!(qn<48||qn>102||qn>57&&qn<65||qn>70&&qn<97););return dm(e,fd()+(t<6&&hi()==32&&Aa()==32))}function kv(e){for(;Aa();)switch(qn){case e:return Ea;case 34:case 39:e!==34&&e!==39&&kv(qn);break;case 40:e===41&&kv(e);break;case 92:Aa();break}return Ea}function z3(e,t){for(;Aa()&&e+qn!==57;)if(e+qn===84&&hi()===47)break;return"/*"+dm(t,Ea-1)+"*"+Z0(e===47?e:Aa())}function D3(e){for(;!au(hi());)Aa();return dm(e,Ea)}function L3(e){return I3(dd("",null,null,null,[""],e=j3(e),0,[0],e))}function dd(e,t,n,r,a,l,s,u,d){for(var g=0,h=0,p=s,y=0,C=0,b=0,x=1,S=1,E=1,$=0,O="",_=a,R=l,T=r,M=O;S;)switch(b=$,$=Aa()){case 40:if(b!=108&&Xl(M,p-1)==58){_3(M+=ud(gp($),"&","&\f"),"&\f",c2(g?u[g-1]:0))!=-1&&(E=-1);break}case 34:case 39:case 91:M+=gp($);break;case 9:case 10:case 13:case 32:M+=P3(b);break;case 92:M+=A3(fd()-1,7);continue;case 47:switch(hi()){case 42:case 47:qf(B3(z3(Aa(),fd()),t,n,d),d),(au(b||1)==5||au(hi()||1)==5)&&Za(M)&&es(M,-1,void 0)!==" "&&(M+=" ");break;default:M+="/"}break;case 123*x:u[g++]=Za(M)*E;case 125*x:case 59:case 0:switch($){case 0:case 125:S=0;case 59+h:E==-1&&(M=ud(M,/\f/g,"")),C>0&&(Za(M)-p||x===0&&b===47)&&qf(C>32?cx(M+";",r,n,p-1,d):cx(ud(M," ","")+";",r,n,p-2,d),d);break;case 59:M+=";";default:if(qf(T=sx(M,t,n,g,h,a,u,O,_=[],R=[],p,l),l),$===123)if(h===0)dd(M,t,T,T,_,l,p,u,R);else{switch(y){case 99:if(Xl(M,3)===110)break;case 108:if(Xl(M,2)===97)break;default:h=0;case 100:case 109:case 115:}h?dd(e,T,T,r&&qf(sx(e,T,T,0,0,a,u,O,a,_=[],p,R),R),a,R,p,u,r?_:R):dd(M,T,T,T,[""],R,0,u,R)}}g=h=C=0,x=E=1,O=M="",p=s;break;case 58:p=1+Za(M),C=b;default:if(x<1){if($==123)--x;else if($==125&&x++==0&&N3()==125)continue}switch(M+=Z0($),$*x){case 38:E=h>0?1:(M+="\f",-1);break;case 44:u[g++]=(Za(M)-1)*E,E=1;break;case 64:hi()===45&&(M+=gp(Aa())),y=hi(),h=p=Za(O=M+=D3(fd())),$++;break;case 45:b===45&&Za(M)==2&&(x=0)}}return l}function sx(e,t,n,r,a,l,s,u,d,g,h,p){for(var y=a-1,C=a===0?l:[""],b=T3(C),x=0,S=0,E=0;x<r;++x)for(var $=0,O=es(e,y+1,y=c2(S=s[x])),_=e;$<b;++$)(_=u2(S>0?C[$]+" "+O:ud(O,/&\f/g,C[$])))&&(d[E++]=_);return J0(e,t,n,a===0?l2:u,d,g,h,p)}function B3(e,t,n,r){return J0(e,t,n,i2,Z0(M3()),es(e,2,-2),0,r)}function cx(e,t,n,r,a){return J0(e,t,n,s2,es(e,0,r),es(e,r+1,-1),r,a)}function qv(e,t){for(var n="",r=0;r<e.length;r++)n+=t(e[r],r,e,t)||"";return n}function H3(e,t,n,r){switch(e.type){case R3:if(e.children.length)break;case E3:case $3:case s2:return e.return=e.return||e.value;case i2:return"";case O3:return e.return=e.value+"{"+qv(e.children,r)+"}";case l2:if(!Za(e.value=e.props.join(",")))return""}return Za(n=qv(e.children,r))?e.return=e.value+"{"+n+"}":""}var ux="data-ant-cssinjs-cache-path",d2="_FILE_STYLE__",Ki,m2=!0;function F3(){if(!Ki&&(Ki={},$r())){var e=document.createElement("div");e.className=ux,e.style.position="fixed",e.style.visibility="hidden",e.style.top="-9999px",document.body.appendChild(e);var t=getComputedStyle(e).content||"";t=t.replace(/^"/,"").replace(/"$/,""),t.split(";").forEach(function(a){var l=a.split(":"),s=ge(l,2),u=s[0],d=s[1];Ki[u]=d});var n=document.querySelector("style[".concat(ux,"]"));if(n){var r;m2=!1,(r=n.parentNode)===null||r===void 0||r.removeChild(n)}document.body.removeChild(e)}}function V3(e){return F3(),!!Ki[e]}function U3(e){var t=Ki[e],n=null;if(t&&$r())if(m2)n=d2;else{var r=document.querySelector("style[".concat(Pa,'="').concat(Ki[e],'"]'));r?n=r.innerHTML:delete Ki[e]}return[n,t]}var k3="_skip_check_",g2="_multi_value_";function md(e){var t=qv(L3(e),H3);return t.replace(/\{%%%\:[^;];}/g,";")}function q3(e){return st(e)==="object"&&e&&(k3 in e||g2 in e)}function fx(e,t,n){if(!t)return e;var r=".".concat(t),a=n==="low"?":where(".concat(r,")"):r,l=e.split(",").map(function(s){var u,d=s.trim().split(/\s+/),g=d[0]||"",h=((u=g.match(/^\w+/))===null||u===void 0?void 0:u[0])||"";return g="".concat(h).concat(a).concat(g.slice(h.length)),[g].concat(Ne(d.slice(1))).join(" ")});return l.join(",")}var W3=function e(t){var n=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},r=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{root:!0,parentSelectors:[]},a=r.root,l=r.injectHash,s=r.parentSelectors,u=n.hashId,d=n.layer;n.path;var g=n.hashPriority,h=n.transformers,p=h===void 0?[]:h;n.linters;var y="",C={};function b(E){var $=E.getName(u);if(!C[$]){var O=e(E.style,n,{root:!1,parentSelectors:s}),_=ge(O,1),R=_[0];C[$]="@keyframes ".concat(E.getName(u)).concat(R)}}function x(E){var $=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[];return E.forEach(function(O){Array.isArray(O)?x(O,$):O&&$.push(O)}),$}var S=x(Array.isArray(t)?t:[t]);return S.forEach(function(E){var $=typeof E=="string"&&!a?{}:E;if(typeof $=="string")y+="".concat($,`
`);else if($._keyframe)b($);else{var O=p.reduce(function(_,R){var T;return(R==null||(T=R.visit)===null||T===void 0?void 0:T.call(R,_))||_},$);Object.keys(O).forEach(function(_){var R=O[_];if(st(R)==="object"&&R&&(_!=="animationName"||!R._keyframe)&&!q3(R)){var T=!1,M=_.trim(),z=!1;(a||l)&&u?M.startsWith("@")?T=!0:M==="&"?M=fx("",u,g):M=fx(_,u,g):a&&!u&&(M==="&"||M==="")&&(M="",z=!0);var P=e(R,n,{root:z,injectHash:T,parentSelectors:[].concat(Ne(s),[M])}),D=ge(P,2),F=D[0],G=D[1];C=Q(Q({},C),G),y+="".concat(M).concat(F)}else{let K=function(j,A){var L=j.replace(/[A-Z]/g,function(N){return"-".concat(N.toLowerCase())}),q=A;!w3[j]&&typeof q=="number"&&q!==0&&(q="".concat(q,"px")),j==="animationName"&&A!==null&&A!==void 0&&A._keyframe&&(b(A),q=A.getName(u)),y+="".concat(L,":").concat(q,";")};var V,U=(V=R==null?void 0:R.value)!==null&&V!==void 0?V:R;st(R)==="object"&&R!==null&&R!==void 0&&R[g2]&&Array.isArray(U)?U.forEach(function(j){K(_,j)}):K(_,U)}})}}),a?d&&(y&&(y="@layer ".concat(d.name," {").concat(y,"}")),d.dependencies&&(C["@layer ".concat(d.name)]=d.dependencies.map(function(E){return"@layer ".concat(E,", ").concat(d.name,";")}).join(`
`))):y="{".concat(y,"}"),[y,C]};function h2(e,t){return nu("".concat(e.join("%")).concat(t))}function G3(){return null}var p2="style";function Wv(e,t){var n=e.token,r=e.path,a=e.hashId,l=e.layer,s=e.nonce,u=e.clientOnly,d=e.order,g=d===void 0?0:d,h=f.useContext(wu),p=h.autoClear;h.mock;var y=h.defaultCache,C=h.hashPriority,b=h.container,x=h.ssrInline,S=h.transformers,E=h.linters,$=h.cache,O=h.layer,_=n._tokenKey,R=[_];O&&R.push("layer"),R.push.apply(R,Ne(r));var T=Uv,M=Q0(p2,R,function(){var G=R.join("|");if(V3(G)){var V=U3(G),U=ge(V,2),K=U[0],j=U[1];if(K)return[K,_,j,{},u,g]}var A=t(),L=W3(A,{hashId:a,hashPriority:C,layer:O?l:void 0,path:r.join("-"),transformers:S,linters:E}),q=ge(L,2),N=q[0],B=q[1],W=md(N),X=h2(R,W);return[W,_,X,B,u,g]},function(G,V){var U=ge(G,3),K=U[2];(V||p)&&Uv&&Zc(K,{mark:Pa})},function(G){var V=ge(G,4),U=V[0];V[1];var K=V[2],j=V[3];if(T&&U!==d2){var A={mark:Pa,prepend:O?!1:"queue",attachTo:b,priority:g},L=typeof s=="function"?s():s;L&&(A.csp={nonce:L});var q=[],N=[];Object.keys(j).forEach(function(W){W.startsWith("@layer")?q.push(W):N.push(W)}),q.forEach(function(W){xo(md(j[W]),"_layer-".concat(W),Q(Q({},A),{},{prepend:!0}))});var B=xo(U,K,A);B[gi]=$.instanceId,B.setAttribute(Jl,_),N.forEach(function(W){xo(md(j[W]),"_effect-".concat(W),A)})}}),z=ge(M,3),P=z[0],D=z[1],F=z[2];return function(G){var V;if(!x||T||!y)V=f.createElement(G3,null);else{var U;V=f.createElement("style",je({},(U={},Z(U,Jl,D),Z(U,Pa,F),U),{dangerouslySetInnerHTML:{__html:P}}))}return f.createElement(f.Fragment,null,V,G)}}var K3=function(t,n,r){var a=ge(t,6),l=a[0],s=a[1],u=a[2],d=a[3],g=a[4],h=a[5],p=r||{},y=p.plain;if(g)return null;var C=l,b={"data-rc-order":"prependQueue","data-rc-priority":"".concat(h)};return C=jd(l,s,u,b,y),d&&Object.keys(d).forEach(function(x){if(!n[x]){n[x]=!0;var S=md(d[x]),E=jd(S,s,"_effect-".concat(x),b,y);x.startsWith("@layer")?C=E+C:C+=E}}),[h,u,C]},v2="cssVar",X3=function(t,n){var r=t.key,a=t.prefix,l=t.unitless,s=t.ignore,u=t.token,d=t.scope,g=d===void 0?"":d,h=f.useContext(wu),p=h.cache.instanceId,y=h.container,C=u._tokenKey,b=[].concat(Ne(t.path),[r,g,C]),x=Q0(v2,b,function(){var S=n(),E=a2(S,r,{prefix:a,unitless:l,ignore:s,scope:g}),$=ge(E,2),O=$[0],_=$[1],R=h2(b,_);return[O,_,R,r]},function(S){var E=ge(S,3),$=E[2];Uv&&Zc($,{mark:Pa})},function(S){var E=ge(S,3),$=E[1],O=E[2];if($){var _=xo($,O,{mark:Pa,prepend:"queue",attachTo:y,priority:-999});_[gi]=p,_.setAttribute(Jl,r)}});return x},Y3=function(t,n,r){var a=ge(t,4),l=a[1],s=a[2],u=a[3],d=r||{},g=d.plain;if(!l)return null;var h=-999,p={"data-rc-order":"prependQueue","data-rc-priority":"".concat(h)},y=jd(l,u,s,p,g);return[h,s,y]},bc;bc={},Z(bc,p2,K3),Z(bc,o2,x3),Z(bc,v2,Y3);var Dt=function(){function e(t,n){Wn(this,e),Z(this,"name",void 0),Z(this,"style",void 0),Z(this,"_keyframe",!0),this.name=t,this.style=n}return Gn(e,[{key:"getName",value:function(){var n=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"";return n?"".concat(n,"-").concat(this.name):this.name}}]),e}();function Dl(e){return e.notSplit=!0,e}Dl(["borderTop","borderBottom"]),Dl(["borderTop"]),Dl(["borderBottom"]),Dl(["borderLeft","borderRight"]),Dl(["borderLeft"]),Dl(["borderRight"]);function y2(e){return RO(e)||JO(e)||U0(e)||_O()}function wa(e,t){for(var n=e,r=0;r<t.length;r+=1){if(n==null)return;n=n[t[r]]}return n}function b2(e,t,n,r){if(!t.length)return n;var a=y2(t),l=a[0],s=a.slice(1),u;return!e&&typeof l=="number"?u=[]:Array.isArray(e)?u=Ne(e):u=Q({},e),r&&n===void 0&&s.length===1?delete u[l][s[0]]:u[l]=b2(u[l],s,n,r),u}function xa(e,t,n){var r=arguments.length>3&&arguments[3]!==void 0?arguments[3]:!1;return t.length&&r&&n===void 0&&!wa(e,t.slice(0,-1))?e:b2(e,t,n,r)}function Q3(e){return st(e)==="object"&&e!==null&&Object.getPrototypeOf(e)===Object.prototype}function dx(e){return Array.isArray(e)?[]:{}}var Z3=typeof Reflect>"u"?Object.keys:Reflect.ownKeys;function Wl(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];var r=dx(t[0]);return t.forEach(function(a){function l(s,u){var d=new Set(u),g=wa(a,s),h=Array.isArray(g);if(h||Q3(g)){if(!d.has(g)){d.add(g);var p=wa(r,s);h?r=xa(r,s,[]):(!p||st(p)!=="object")&&(r=xa(r,s,dx(g))),Z3(g).forEach(function(y){l([].concat(Ne(s),[y]),d)})}}else r=xa(r,s,g)}l([])}),r}function J3(){}const eA=f.createContext({}),Eu=()=>{const e=()=>{};return e.deprecated=J3,e},S2=f.createContext(void 0);var tA={items_per_page:"/ page",jump_to:"Go to",jump_to_confirm:"confirm",page:"Page",prev_page:"Previous Page",next_page:"Next Page",prev_5:"Previous 5 Pages",next_5:"Next 5 Pages",prev_3:"Previous 3 Pages",next_3:"Next 3 Pages",page_size:"Page Size"},nA={yearFormat:"YYYY",dayFormat:"D",cellMeridiemFormat:"A",monthBeforeYear:!0},rA=Q(Q({},nA),{},{locale:"en_US",today:"Today",now:"Now",backToToday:"Back to today",ok:"OK",clear:"Clear",week:"Week",month:"Month",year:"Year",timeSelect:"select time",dateSelect:"select date",weekSelect:"Choose a week",monthSelect:"Choose a month",yearSelect:"Choose a year",decadeSelect:"Choose a decade",dateFormat:"M/D/YYYY",dateTimeFormat:"M/D/YYYY HH:mm:ss",previousMonth:"Previous month (PageUp)",nextMonth:"Next month (PageDown)",previousYear:"Last year (Control + left)",nextYear:"Next year (Control + right)",previousDecade:"Last decade",nextDecade:"Next decade",previousCentury:"Last century",nextCentury:"Next century"});const C2={placeholder:"Select time",rangePlaceholder:["Start time","End time"]},mx={lang:Object.assign({placeholder:"Select date",yearPlaceholder:"Select year",quarterPlaceholder:"Select quarter",monthPlaceholder:"Select month",weekPlaceholder:"Select week",rangePlaceholder:["Start date","End date"],rangeYearPlaceholder:["Start year","End year"],rangeQuarterPlaceholder:["Start quarter","End quarter"],rangeMonthPlaceholder:["Start month","End month"],rangeWeekPlaceholder:["Start week","End week"]},rA),timePickerLocale:Object.assign({},C2)},Yr="${label} is not a valid ${type}",Oo={locale:"en",Pagination:tA,DatePicker:mx,TimePicker:C2,Calendar:mx,global:{placeholder:"Please select"},Table:{filterTitle:"Filter menu",filterConfirm:"OK",filterReset:"Reset",filterEmptyText:"No filters",filterCheckAll:"Select all items",filterSearchPlaceholder:"Search in filters",emptyText:"No data",selectAll:"Select current page",selectInvert:"Invert current page",selectNone:"Clear all data",selectionAll:"Select all data",sortTitle:"Sort",expand:"Expand row",collapse:"Collapse row",triggerDesc:"Click to sort descending",triggerAsc:"Click to sort ascending",cancelSort:"Click to cancel sorting"},Tour:{Next:"Next",Previous:"Previous",Finish:"Finish"},Modal:{okText:"OK",cancelText:"Cancel",justOkText:"OK"},Popconfirm:{okText:"OK",cancelText:"Cancel"},Transfer:{titles:["",""],searchPlaceholder:"Search here",itemUnit:"item",itemsUnit:"items",remove:"Remove",selectCurrent:"Select current page",removeCurrent:"Remove current page",selectAll:"Select all data",deselectAll:"Deselect all data",removeAll:"Remove all data",selectInvert:"Invert current page"},Upload:{uploading:"Uploading...",removeFile:"Remove file",uploadError:"Upload error",previewFile:"Preview file",downloadFile:"Download file"},Empty:{description:"No data"},Icon:{icon:"icon"},Text:{edit:"Edit",copy:"Copy",copied:"Copied",expand:"Expand",collapse:"Collapse"},Form:{optional:"(optional)",defaultValidateMessages:{default:"Field validation error for ${label}",required:"Please enter ${label}",enum:"${label} must be one of [${enum}]",whitespace:"${label} cannot be a blank character",date:{format:"${label} date format is invalid",parse:"${label} cannot be converted to a date",invalid:"${label} is an invalid date"},types:{string:Yr,method:Yr,array:Yr,object:Yr,number:Yr,date:Yr,boolean:Yr,integer:Yr,float:Yr,regexp:Yr,email:Yr,url:Yr,hex:Yr},string:{len:"${label} must be ${len} characters",min:"${label} must be at least ${min} characters",max:"${label} must be up to ${max} characters",range:"${label} must be between ${min}-${max} characters"},number:{len:"${label} must be equal to ${len}",min:"${label} must be minimum ${min}",max:"${label} must be maximum ${max}",range:"${label} must be between ${min}-${max}"},array:{len:"Must be ${len} ${label}",min:"At least ${min} ${label}",max:"At most ${max} ${label}",range:"The amount of ${label} must be between ${min}-${max}"},pattern:{mismatch:"${label} does not match the pattern ${pattern}"}}},Image:{preview:"Preview"},QRCode:{expired:"QR code expired",refresh:"Refresh",scanned:"Scanned"},ColorPicker:{presetEmpty:"Empty",transparent:"Transparent",singleColor:"Single",gradientColor:"Gradient"}};let gd=Object.assign({},Oo.Modal),hd=[];const gx=()=>hd.reduce((e,t)=>Object.assign(Object.assign({},e),t),Oo.Modal);function aA(e){if(e){const t=Object.assign({},e);return hd.push(t),gd=gx(),()=>{hd=hd.filter(n=>n!==t),gd=gx()}}gd=Object.assign({},Oo.Modal)}function x2(){return gd}const ey=f.createContext(void 0),Do=(e,t)=>{const n=f.useContext(ey),r=f.useMemo(()=>{var l;const s=t||Oo[e],u=(l=n==null?void 0:n[e])!==null&&l!==void 0?l:{};return Object.assign(Object.assign({},typeof s=="function"?s():s),u||{})},[e,t,n]),a=f.useMemo(()=>{const l=n==null?void 0:n.locale;return n!=null&&n.exist&&!l?Oo.locale:l},[n]);return[r,a]},oA="internalMark",iA=e=>{const{locale:t={},children:n,_ANT_MARK__:r}=e;f.useEffect(()=>aA(t==null?void 0:t.Modal),[t]);const a=f.useMemo(()=>Object.assign(Object.assign({},t),{exist:!0}),[t]);return f.createElement(ey.Provider,{value:a},n)},ty={blue:"#1677FF",purple:"#722ED1",cyan:"#13C2C2",green:"#52C41A",magenta:"#EB2F96",pink:"#EB2F96",red:"#F5222D",orange:"#FA8C16",yellow:"#FADB14",volcano:"#FA541C",geekblue:"#2F54EB",gold:"#FAAD14",lime:"#A0D911"},ou=Object.assign(Object.assign({},ty),{colorPrimary:"#1677ff",colorSuccess:"#52c41a",colorWarning:"#faad14",colorError:"#ff4d4f",colorInfo:"#1677ff",colorLink:"",colorTextBase:"",colorBgBase:"",fontFamily:`-apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial,
'Noto Sans', sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol',
'Noto Color Emoji'`,fontFamilyCode:"'SFMono-Regular', Consolas, 'Liberation Mono', Menlo, Courier, monospace",fontSize:14,lineWidth:1,lineType:"solid",motionUnit:.1,motionBase:0,motionEaseOutCirc:"cubic-bezier(0.08, 0.82, 0.17, 1)",motionEaseInOutCirc:"cubic-bezier(0.78, 0.14, 0.15, 0.86)",motionEaseOut:"cubic-bezier(0.215, 0.61, 0.355, 1)",motionEaseInOut:"cubic-bezier(0.645, 0.045, 0.355, 1)",motionEaseOutBack:"cubic-bezier(0.12, 0.4, 0.29, 1.46)",motionEaseInBack:"cubic-bezier(0.71, -0.46, 0.88, 0.6)",motionEaseInQuint:"cubic-bezier(0.755, 0.05, 0.855, 0.06)",motionEaseOutQuint:"cubic-bezier(0.23, 1, 0.32, 1)",borderRadius:6,sizeUnit:4,sizeStep:4,sizePopupArrow:16,controlHeight:32,zIndexBase:0,zIndexPopupBase:1e3,opacityImage:1,wireframe:!1,motion:!0});function w2(e,t){let{generateColorPalettes:n,generateNeutralColorPalettes:r}=t;const{colorSuccess:a,colorWarning:l,colorError:s,colorInfo:u,colorPrimary:d,colorBgBase:g,colorTextBase:h}=e,p=n(d),y=n(a),C=n(l),b=n(s),x=n(u),S=r(g,h),E=e.colorLink||e.colorInfo,$=n(E),O=new en(b[1]).mix(new en(b[3]),50).toHexString();return Object.assign(Object.assign({},S),{colorPrimaryBg:p[1],colorPrimaryBgHover:p[2],colorPrimaryBorder:p[3],colorPrimaryBorderHover:p[4],colorPrimaryHover:p[5],colorPrimary:p[6],colorPrimaryActive:p[7],colorPrimaryTextHover:p[8],colorPrimaryText:p[9],colorPrimaryTextActive:p[10],colorSuccessBg:y[1],colorSuccessBgHover:y[2],colorSuccessBorder:y[3],colorSuccessBorderHover:y[4],colorSuccessHover:y[4],colorSuccess:y[6],colorSuccessActive:y[7],colorSuccessTextHover:y[8],colorSuccessText:y[9],colorSuccessTextActive:y[10],colorErrorBg:b[1],colorErrorBgHover:b[2],colorErrorBgFilledHover:O,colorErrorBgActive:b[3],colorErrorBorder:b[3],colorErrorBorderHover:b[4],colorErrorHover:b[5],colorError:b[6],colorErrorActive:b[7],colorErrorTextHover:b[8],colorErrorText:b[9],colorErrorTextActive:b[10],colorWarningBg:C[1],colorWarningBgHover:C[2],colorWarningBorder:C[3],colorWarningBorderHover:C[4],colorWarningHover:C[4],colorWarning:C[6],colorWarningActive:C[7],colorWarningTextHover:C[8],colorWarningText:C[9],colorWarningTextActive:C[10],colorInfoBg:x[1],colorInfoBgHover:x[2],colorInfoBorder:x[3],colorInfoBorderHover:x[4],colorInfoHover:x[4],colorInfo:x[6],colorInfoActive:x[7],colorInfoTextHover:x[8],colorInfoText:x[9],colorInfoTextActive:x[10],colorLinkHover:$[4],colorLink:$[6],colorLinkActive:$[7],colorBgMask:new en("#000").setA(.45).toRgbString(),colorWhite:"#fff"})}const lA=e=>{let t=e,n=e,r=e,a=e;return e<6&&e>=5?t=e+1:e<16&&e>=6?t=e+2:e>=16&&(t=16),e<7&&e>=5?n=4:e<8&&e>=7?n=5:e<14&&e>=8?n=6:e<16&&e>=14?n=7:e>=16&&(n=8),e<6&&e>=2?r=1:e>=6&&(r=2),e>4&&e<8?a=4:e>=8&&(a=6),{borderRadius:e,borderRadiusXS:r,borderRadiusSM:n,borderRadiusLG:t,borderRadiusOuter:a}};function sA(e){const{motionUnit:t,motionBase:n,borderRadius:r,lineWidth:a}=e;return Object.assign({motionDurationFast:`${(n+t).toFixed(1)}s`,motionDurationMid:`${(n+t*2).toFixed(1)}s`,motionDurationSlow:`${(n+t*3).toFixed(1)}s`,lineWidthBold:a+1},lA(r))}const cA=e=>{const{controlHeight:t}=e;return{controlHeightSM:t*.75,controlHeightXS:t*.5,controlHeightLG:t*1.25}};function pd(e){return(e+8)/e}function uA(e){const t=Array.from({length:10}).map((n,r)=>{const a=r-1,l=e*Math.pow(Math.E,a/5),s=r>1?Math.floor(l):Math.ceil(l);return Math.floor(s/2)*2});return t[1]=e,t.map(n=>({size:n,lineHeight:pd(n)}))}const fA=e=>{const t=uA(e),n=t.map(h=>h.size),r=t.map(h=>h.lineHeight),a=n[1],l=n[0],s=n[2],u=r[1],d=r[0],g=r[2];return{fontSizeSM:l,fontSize:a,fontSizeLG:s,fontSizeXL:n[3],fontSizeHeading1:n[6],fontSizeHeading2:n[5],fontSizeHeading3:n[4],fontSizeHeading4:n[3],fontSizeHeading5:n[2],lineHeight:u,lineHeightLG:g,lineHeightSM:d,fontHeight:Math.round(u*a),fontHeightLG:Math.round(g*s),fontHeightSM:Math.round(d*l),lineHeightHeading1:r[6],lineHeightHeading2:r[5],lineHeightHeading3:r[4],lineHeightHeading4:r[3],lineHeightHeading5:r[2]}};function dA(e){const{sizeUnit:t,sizeStep:n}=e;return{sizeXXL:t*(n+8),sizeXL:t*(n+4),sizeLG:t*(n+2),sizeMD:t*(n+1),sizeMS:t*n,size:t*n,sizeSM:t*(n-1),sizeXS:t*(n-2),sizeXXS:t*(n-3)}}const va=(e,t)=>new en(e).setA(t).toRgbString(),Sc=(e,t)=>new en(e).darken(t).toHexString(),mA=e=>{const t=Yi(e);return{1:t[0],2:t[1],3:t[2],4:t[3],5:t[4],6:t[5],7:t[6],8:t[4],9:t[5],10:t[6]}},gA=(e,t)=>{const n=e||"#fff",r=t||"#000";return{colorBgBase:n,colorTextBase:r,colorText:va(r,.88),colorTextSecondary:va(r,.65),colorTextTertiary:va(r,.45),colorTextQuaternary:va(r,.25),colorFill:va(r,.15),colorFillSecondary:va(r,.06),colorFillTertiary:va(r,.04),colorFillQuaternary:va(r,.02),colorBgSolid:va(r,1),colorBgSolidHover:va(r,.75),colorBgSolidActive:va(r,.95),colorBgLayout:Sc(n,4),colorBgContainer:Sc(n,0),colorBgElevated:Sc(n,0),colorBgSpotlight:va(r,.85),colorBgBlur:"transparent",colorBorder:Sc(n,15),colorBorderSecondary:Sc(n,6)}};function ny(e){sp.pink=sp.magenta,cp.pink=cp.magenta;const t=Object.keys(ty).map(n=>{const r=e[n]===sp[n]?cp[n]:Yi(e[n]);return Array.from({length:10},()=>1).reduce((a,l,s)=>(a[`${n}-${s+1}`]=r[s],a[`${n}${s+1}`]=r[s],a),{})}).reduce((n,r)=>(n=Object.assign(Object.assign({},n),r),n),{});return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},e),t),w2(e,{generateColorPalettes:mA,generateNeutralColorPalettes:gA})),fA(e.fontSize)),dA(e)),cA(e)),sA(e))}const E2=Vv(ny),Id={token:ou,override:{override:ou},hashed:!0},ry=se.createContext(Id),iu="ant",mm="anticon",hA=["outlined","borderless","filled","underlined"],pA=(e,t)=>t||(e?`${iu}-${e}`:iu),dt=f.createContext({getPrefixCls:pA,iconPrefixCls:mm}),{Consumer:mX}=dt,hx={};function Or(e){const t=f.useContext(dt),{getPrefixCls:n,direction:r,getPopupContainer:a}=t,l=t[e];return Object.assign(Object.assign({classNames:hx,styles:hx},l),{getPrefixCls:n,direction:r,getPopupContainer:a})}const vA=`-ant-${Date.now()}-${Math.random()}`;function yA(e,t){const n={},r=(s,u)=>{let d=s.clone();return d=(u==null?void 0:u(d))||d,d.toRgbString()},a=(s,u)=>{const d=new en(s),g=Yi(d.toRgbString());n[`${u}-color`]=r(d),n[`${u}-color-disabled`]=g[1],n[`${u}-color-hover`]=g[4],n[`${u}-color-active`]=g[6],n[`${u}-color-outline`]=d.clone().setA(.2).toRgbString(),n[`${u}-color-deprecated-bg`]=g[0],n[`${u}-color-deprecated-border`]=g[2]};if(t.primaryColor){a(t.primaryColor,"primary");const s=new en(t.primaryColor),u=Yi(s.toRgbString());u.forEach((g,h)=>{n[`primary-${h+1}`]=g}),n["primary-color-deprecated-l-35"]=r(s,g=>g.lighten(35)),n["primary-color-deprecated-l-20"]=r(s,g=>g.lighten(20)),n["primary-color-deprecated-t-20"]=r(s,g=>g.tint(20)),n["primary-color-deprecated-t-50"]=r(s,g=>g.tint(50)),n["primary-color-deprecated-f-12"]=r(s,g=>g.setA(g.a*.12));const d=new en(u[0]);n["primary-color-active-deprecated-f-30"]=r(d,g=>g.setA(g.a*.3)),n["primary-color-active-deprecated-d-02"]=r(d,g=>g.darken(2))}return t.successColor&&a(t.successColor,"success"),t.warningColor&&a(t.warningColor,"warning"),t.errorColor&&a(t.errorColor,"error"),t.infoColor&&a(t.infoColor,"info"),`
  :root {
    ${Object.keys(n).map(s=>`--${e}-${s}: ${n[s]};`).join(`
`)}
  }
  `.trim()}function bA(e,t){const n=yA(e,t);$r()&&xo(n,`${vA}-dynamic-theme`)}const Ro=f.createContext(!1),ay=e=>{let{children:t,disabled:n}=e;const r=f.useContext(Ro);return f.createElement(Ro.Provider,{value:n??r},t)},Qi=f.createContext(void 0),SA=e=>{let{children:t,size:n}=e;const r=f.useContext(Qi);return f.createElement(Qi.Provider,{value:n||r},t)};function CA(){const e=f.useContext(Ro),t=f.useContext(Qi);return{componentDisabled:e,componentSize:t}}var $2=Gn(function e(){Wn(this,e)}),O2="CALC_UNIT",xA=new RegExp(O2,"g");function hp(e){return typeof e=="number"?"".concat(e).concat(O2):e}var wA=function(e){Po(n,e);var t=Ao(n);function n(r,a){var l;Wn(this,n),l=t.call(this),Z(jt(l),"result",""),Z(jt(l),"unitlessCssVar",void 0),Z(jt(l),"lowPriority",void 0);var s=st(r);return l.unitlessCssVar=a,r instanceof n?l.result="(".concat(r.result,")"):s==="number"?l.result=hp(r):s==="string"&&(l.result=r),l}return Gn(n,[{key:"add",value:function(a){return a instanceof n?this.result="".concat(this.result," + ").concat(a.getResult()):(typeof a=="number"||typeof a=="string")&&(this.result="".concat(this.result," + ").concat(hp(a))),this.lowPriority=!0,this}},{key:"sub",value:function(a){return a instanceof n?this.result="".concat(this.result," - ").concat(a.getResult()):(typeof a=="number"||typeof a=="string")&&(this.result="".concat(this.result," - ").concat(hp(a))),this.lowPriority=!0,this}},{key:"mul",value:function(a){return this.lowPriority&&(this.result="(".concat(this.result,")")),a instanceof n?this.result="".concat(this.result," * ").concat(a.getResult(!0)):(typeof a=="number"||typeof a=="string")&&(this.result="".concat(this.result," * ").concat(a)),this.lowPriority=!1,this}},{key:"div",value:function(a){return this.lowPriority&&(this.result="(".concat(this.result,")")),a instanceof n?this.result="".concat(this.result," / ").concat(a.getResult(!0)):(typeof a=="number"||typeof a=="string")&&(this.result="".concat(this.result," / ").concat(a)),this.lowPriority=!1,this}},{key:"getResult",value:function(a){return this.lowPriority||a?"(".concat(this.result,")"):this.result}},{key:"equal",value:function(a){var l=this,s=a||{},u=s.unit,d=!0;return typeof u=="boolean"?d=u:Array.from(this.unitlessCssVar).some(function(g){return l.result.includes(g)})&&(d=!1),this.result=this.result.replace(xA,d?"px":""),typeof this.lowPriority<"u"?"calc(".concat(this.result,")"):this.result}}]),n}($2),EA=function(e){Po(n,e);var t=Ao(n);function n(r){var a;return Wn(this,n),a=t.call(this),Z(jt(a),"result",0),r instanceof n?a.result=r.result:typeof r=="number"&&(a.result=r),a}return Gn(n,[{key:"add",value:function(a){return a instanceof n?this.result+=a.result:typeof a=="number"&&(this.result+=a),this}},{key:"sub",value:function(a){return a instanceof n?this.result-=a.result:typeof a=="number"&&(this.result-=a),this}},{key:"mul",value:function(a){return a instanceof n?this.result*=a.result:typeof a=="number"&&(this.result*=a),this}},{key:"div",value:function(a){return a instanceof n?this.result/=a.result:typeof a=="number"&&(this.result/=a),this}},{key:"equal",value:function(){return this.result}}]),n}($2),$A=function(t,n){var r=t==="css"?wA:EA;return function(a){return new r(a,n)}},px=function(t,n){return"".concat([n,t.replace(/([A-Z]+)([A-Z][a-z]+)/g,"$1-$2").replace(/([a-z])([A-Z])/g,"$1-$2")].filter(Boolean).join("-"))};function An(e){var t=f.useRef();t.current=e;var n=f.useCallback(function(){for(var r,a=arguments.length,l=new Array(a),s=0;s<a;s++)l[s]=arguments[s];return(r=t.current)===null||r===void 0?void 0:r.call.apply(r,[t].concat(l))},[]);return n}function Zi(e){var t=f.useRef(!1),n=f.useState(e),r=ge(n,2),a=r[0],l=r[1];f.useEffect(function(){return t.current=!1,function(){t.current=!0}},[]);function s(u,d){d&&t.current||l(u)}return[a,s]}function pp(e){return e!==void 0}function br(e,t){var n=t||{},r=n.defaultValue,a=n.value,l=n.onChange,s=n.postState,u=Zi(function(){return pp(a)?a:pp(r)?typeof r=="function"?r():r:typeof e=="function"?e():e}),d=ge(u,2),g=d[0],h=d[1],p=a!==void 0?a:g,y=s?s(p):p,C=An(l),b=Zi([p]),x=ge(b,2),S=x[0],E=x[1];ix(function(){var O=S[0];g!==O&&C(g,O)},[S]),ix(function(){pp(a)||h(a)},[a]);var $=An(function(O,_){h(O,_),E([p],_)});return[y,$]}function vx(e,t,n,r){var a=Q({},t[e]);if(r!=null&&r.deprecatedTokens){var l=r.deprecatedTokens;l.forEach(function(u){var d=ge(u,2),g=d[0],h=d[1];if(a!=null&&a[g]||a!=null&&a[h]){var p;(p=a[h])!==null&&p!==void 0||(a[h]=a==null?void 0:a[g])}})}var s=Q(Q({},n),a);return Object.keys(s).forEach(function(u){s[u]===t[u]&&delete s[u]}),s}var R2=typeof CSSINJS_STATISTIC<"u",Gv=!0;function Kt(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];if(!R2)return Object.assign.apply(Object,[{}].concat(t));Gv=!1;var r={};return t.forEach(function(a){if(st(a)==="object"){var l=Object.keys(a);l.forEach(function(s){Object.defineProperty(r,s,{configurable:!0,enumerable:!0,get:function(){return a[s]}})})}}),Gv=!0,r}var yx={};function OA(){}var RA=function(t){var n,r=t,a=OA;return R2&&typeof Proxy<"u"&&(n=new Set,r=new Proxy(t,{get:function(s,u){if(Gv){var d;(d=n)===null||d===void 0||d.add(u)}return s[u]}}),a=function(s,u){var d;yx[s]={global:Array.from(n),component:Q(Q({},(d=yx[s])===null||d===void 0?void 0:d.component),u)}}),{token:r,keys:n,flush:a}};function bx(e,t,n){if(typeof n=="function"){var r;return n(Kt(t,(r=t[e])!==null&&r!==void 0?r:{}))}return n??{}}function _A(e){return e==="js"?{max:Math.max,min:Math.min}:{max:function(){for(var n=arguments.length,r=new Array(n),a=0;a<n;a++)r[a]=arguments[a];return"max(".concat(r.map(function(l){return ve(l)}).join(","),")")},min:function(){for(var n=arguments.length,r=new Array(n),a=0;a<n;a++)r[a]=arguments[a];return"min(".concat(r.map(function(l){return ve(l)}).join(","),")")}}}var TA=1e3*60*10,MA=function(){function e(){Wn(this,e),Z(this,"map",new Map),Z(this,"objectIDMap",new WeakMap),Z(this,"nextID",0),Z(this,"lastAccessBeat",new Map),Z(this,"accessBeat",0)}return Gn(e,[{key:"set",value:function(n,r){this.clear();var a=this.getCompositeKey(n);this.map.set(a,r),this.lastAccessBeat.set(a,Date.now())}},{key:"get",value:function(n){var r=this.getCompositeKey(n),a=this.map.get(r);return this.lastAccessBeat.set(r,Date.now()),this.accessBeat+=1,a}},{key:"getCompositeKey",value:function(n){var r=this,a=n.map(function(l){return l&&st(l)==="object"?"obj_".concat(r.getObjectID(l)):"".concat(st(l),"_").concat(l)});return a.join("|")}},{key:"getObjectID",value:function(n){if(this.objectIDMap.has(n))return this.objectIDMap.get(n);var r=this.nextID;return this.objectIDMap.set(n,r),this.nextID+=1,r}},{key:"clear",value:function(){var n=this;if(this.accessBeat>1e4){var r=Date.now();this.lastAccessBeat.forEach(function(a,l){r-a>TA&&(n.map.delete(l),n.lastAccessBeat.delete(l))}),this.accessBeat=0}}}]),e}(),Sx=new MA;function NA(e,t){return se.useMemo(function(){var n=Sx.get(t);if(n)return n;var r=e();return Sx.set(t,r),r},t)}var jA=function(){return{}};function IA(e){var t=e.useCSP,n=t===void 0?jA:t,r=e.useToken,a=e.usePrefix,l=e.getResetStyles,s=e.getCommonStyle,u=e.getCompUnitless;function d(y,C,b,x){var S=Array.isArray(y)?y[0]:y;function E(z){return"".concat(String(S)).concat(z.slice(0,1).toUpperCase()).concat(z.slice(1))}var $=(x==null?void 0:x.unitless)||{},O=typeof u=="function"?u(y):{},_=Q(Q({},O),{},Z({},E("zIndexPopup"),!0));Object.keys($).forEach(function(z){_[E(z)]=$[z]});var R=Q(Q({},x),{},{unitless:_,prefixToken:E}),T=h(y,C,b,R),M=g(S,b,R);return function(z){var P=arguments.length>1&&arguments[1]!==void 0?arguments[1]:z,D=T(z,P),F=ge(D,2),G=F[1],V=M(P),U=ge(V,2),K=U[0],j=U[1];return[K,G,j]}}function g(y,C,b){var x=b.unitless,S=b.injectStyle,E=S===void 0?!0:S,$=b.prefixToken,O=b.ignore,_=function(M){var z=M.rootCls,P=M.cssVar,D=P===void 0?{}:P,F=r(),G=F.realToken;return X3({path:[y],prefix:D.prefix,key:D.key,unitless:x,ignore:O,token:G,scope:z},function(){var V=bx(y,G,C),U=vx(y,G,V,{deprecatedTokens:b==null?void 0:b.deprecatedTokens});return Object.keys(V).forEach(function(K){U[$(K)]=U[K],delete U[K]}),U}),null},R=function(M){var z=r(),P=z.cssVar;return[function(D){return E&&P?se.createElement(se.Fragment,null,se.createElement(_,{rootCls:M,cssVar:P,component:y}),D):D},P==null?void 0:P.key]};return R}function h(y,C,b){var x=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{},S=Array.isArray(y)?y:[y,y],E=ge(S,1),$=E[0],O=S.join("-"),_=e.layer||{name:"antd"};return function(R){var T=arguments.length>1&&arguments[1]!==void 0?arguments[1]:R,M=r(),z=M.theme,P=M.realToken,D=M.hashId,F=M.token,G=M.cssVar,V=a(),U=V.rootPrefixCls,K=V.iconPrefixCls,j=n(),A=G?"css":"js",L=NA(function(){var k=new Set;return G&&Object.keys(x.unitless||{}).forEach(function(Y){k.add(cd(Y,G.prefix)),k.add(cd(Y,px($,G.prefix)))}),$A(A,k)},[A,$,G==null?void 0:G.prefix]),q=_A(A),N=q.max,B=q.min,W={theme:z,token:F,hashId:D,nonce:function(){return j.nonce},clientOnly:x.clientOnly,layer:_,order:x.order||-999};typeof l=="function"&&Wv(Q(Q({},W),{},{clientOnly:!1,path:["Shared",U]}),function(){return l(F,{prefix:{rootPrefixCls:U,iconPrefixCls:K},csp:j})});var X=Wv(Q(Q({},W),{},{path:[O,R,K]}),function(){if(x.injectStyle===!1)return[];var k=RA(F),Y=k.token,ee=k.flush,ue=bx($,P,b),ae=".".concat(R),oe=vx($,P,ue,{deprecatedTokens:x.deprecatedTokens});G&&ue&&st(ue)==="object"&&Object.keys(ue).forEach(function(be){ue[be]="var(".concat(cd(be,px($,G.prefix)),")")});var he=Kt(Y,{componentCls:ae,prefixCls:R,iconCls:".".concat(K),antCls:".".concat(U),calc:L,max:N,min:B},G?ue:oe),ie=C(he,{hashId:D,prefixCls:R,rootPrefixCls:U,iconPrefixCls:K});ee($,oe);var le=typeof s=="function"?s(he,R,T,x.resetFont):null;return[x.resetStyle===!1?null:le,ie]});return[X,D]}}function p(y,C,b){var x=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{},S=h(y,C,b,Q({resetStyle:!1,order:-998},x)),E=function(O){var _=O.prefixCls,R=O.rootCls,T=R===void 0?_:R;return S(_,T),null};return E}return{genStyleHooks:d,genSubStyleComponent:p,genComponentStyleHook:h}}const vi=["blue","purple","cyan","green","magenta","pink","red","orange","yellow","volcano","geekblue","lime","gold"],PA="5.24.8";function vp(e){return e>=0&&e<=255}function Vc(e,t){const{r:n,g:r,b:a,a:l}=new en(e).toRgb();if(l<1)return e;const{r:s,g:u,b:d}=new en(t).toRgb();for(let g=.01;g<=1;g+=.01){const h=Math.round((n-s*(1-g))/g),p=Math.round((r-u*(1-g))/g),y=Math.round((a-d*(1-g))/g);if(vp(h)&&vp(p)&&vp(y))return new en({r:h,g:p,b:y,a:Math.round(g*100)/100}).toRgbString()}return new en({r:n,g:r,b:a,a:1}).toRgbString()}var AA=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function _2(e){const{override:t}=e,n=AA(e,["override"]),r=Object.assign({},t);Object.keys(ou).forEach(y=>{delete r[y]});const a=Object.assign(Object.assign({},n),r),l=480,s=576,u=768,d=992,g=1200,h=1600;if(a.motion===!1){const y="0s";a.motionDurationFast=y,a.motionDurationMid=y,a.motionDurationSlow=y}return Object.assign(Object.assign(Object.assign({},a),{colorFillContent:a.colorFillSecondary,colorFillContentHover:a.colorFill,colorFillAlter:a.colorFillQuaternary,colorBgContainerDisabled:a.colorFillTertiary,colorBorderBg:a.colorBgContainer,colorSplit:Vc(a.colorBorderSecondary,a.colorBgContainer),colorTextPlaceholder:a.colorTextQuaternary,colorTextDisabled:a.colorTextQuaternary,colorTextHeading:a.colorText,colorTextLabel:a.colorTextSecondary,colorTextDescription:a.colorTextTertiary,colorTextLightSolid:a.colorWhite,colorHighlight:a.colorError,colorBgTextHover:a.colorFillSecondary,colorBgTextActive:a.colorFill,colorIcon:a.colorTextTertiary,colorIconHover:a.colorText,colorErrorOutline:Vc(a.colorErrorBg,a.colorBgContainer),colorWarningOutline:Vc(a.colorWarningBg,a.colorBgContainer),fontSizeIcon:a.fontSizeSM,lineWidthFocus:a.lineWidth*3,lineWidth:a.lineWidth,controlOutlineWidth:a.lineWidth*2,controlInteractiveSize:a.controlHeight/2,controlItemBgHover:a.colorFillTertiary,controlItemBgActive:a.colorPrimaryBg,controlItemBgActiveHover:a.colorPrimaryBgHover,controlItemBgActiveDisabled:a.colorFill,controlTmpOutline:a.colorFillQuaternary,controlOutline:Vc(a.colorPrimaryBg,a.colorBgContainer),lineType:a.lineType,borderRadius:a.borderRadius,borderRadiusXS:a.borderRadiusXS,borderRadiusSM:a.borderRadiusSM,borderRadiusLG:a.borderRadiusLG,fontWeightStrong:600,opacityLoading:.65,linkDecoration:"none",linkHoverDecoration:"none",linkFocusDecoration:"none",controlPaddingHorizontal:12,controlPaddingHorizontalSM:8,paddingXXS:a.sizeXXS,paddingXS:a.sizeXS,paddingSM:a.sizeSM,padding:a.size,paddingMD:a.sizeMD,paddingLG:a.sizeLG,paddingXL:a.sizeXL,paddingContentHorizontalLG:a.sizeLG,paddingContentVerticalLG:a.sizeMS,paddingContentHorizontal:a.sizeMS,paddingContentVertical:a.sizeSM,paddingContentHorizontalSM:a.size,paddingContentVerticalSM:a.sizeXS,marginXXS:a.sizeXXS,marginXS:a.sizeXS,marginSM:a.sizeSM,margin:a.size,marginMD:a.sizeMD,marginLG:a.sizeLG,marginXL:a.sizeXL,marginXXL:a.sizeXXL,boxShadow:`
      0 6px 16px 0 rgba(0, 0, 0, 0.08),
      0 3px 6px -4px rgba(0, 0, 0, 0.12),
      0 9px 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowSecondary:`
      0 6px 16px 0 rgba(0, 0, 0, 0.08),
      0 3px 6px -4px rgba(0, 0, 0, 0.12),
      0 9px 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowTertiary:`
      0 1px 2px 0 rgba(0, 0, 0, 0.03),
      0 1px 6px -1px rgba(0, 0, 0, 0.02),
      0 2px 4px 0 rgba(0, 0, 0, 0.02)
    `,screenXS:l,screenXSMin:l,screenXSMax:s-1,screenSM:s,screenSMMin:s,screenSMMax:u-1,screenMD:u,screenMDMin:u,screenMDMax:d-1,screenLG:d,screenLGMin:d,screenLGMax:g-1,screenXL:g,screenXLMin:g,screenXLMax:h-1,screenXXL:h,screenXXLMin:h,boxShadowPopoverArrow:"2px 2px 5px rgba(0, 0, 0, 0.05)",boxShadowCard:`
      0 1px 2px -2px ${new en("rgba(0, 0, 0, 0.16)").toRgbString()},
      0 3px 6px 0 ${new en("rgba(0, 0, 0, 0.12)").toRgbString()},
      0 5px 12px 4px ${new en("rgba(0, 0, 0, 0.09)").toRgbString()}
    `,boxShadowDrawerRight:`
      -6px 0 16px 0 rgba(0, 0, 0, 0.08),
      -3px 0 6px -4px rgba(0, 0, 0, 0.12),
      -9px 0 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowDrawerLeft:`
      6px 0 16px 0 rgba(0, 0, 0, 0.08),
      3px 0 6px -4px rgba(0, 0, 0, 0.12),
      9px 0 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowDrawerUp:`
      0 6px 16px 0 rgba(0, 0, 0, 0.08),
      0 3px 6px -4px rgba(0, 0, 0, 0.12),
      0 9px 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowDrawerDown:`
      0 -6px 16px 0 rgba(0, 0, 0, 0.08),
      0 -3px 6px -4px rgba(0, 0, 0, 0.12),
      0 -9px 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowTabsOverflowLeft:"inset 10px 0 8px -8px rgba(0, 0, 0, 0.08)",boxShadowTabsOverflowRight:"inset -10px 0 8px -8px rgba(0, 0, 0, 0.08)",boxShadowTabsOverflowTop:"inset 0 10px 8px -8px rgba(0, 0, 0, 0.08)",boxShadowTabsOverflowBottom:"inset 0 -10px 8px -8px rgba(0, 0, 0, 0.08)"}),r)}var Cx=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const T2={lineHeight:!0,lineHeightSM:!0,lineHeightLG:!0,lineHeightHeading1:!0,lineHeightHeading2:!0,lineHeightHeading3:!0,lineHeightHeading4:!0,lineHeightHeading5:!0,opacityLoading:!0,fontWeightStrong:!0,zIndexPopupBase:!0,zIndexBase:!0,opacityImage:!0},zA={size:!0,sizeSM:!0,sizeLG:!0,sizeMD:!0,sizeXS:!0,sizeXXS:!0,sizeMS:!0,sizeXL:!0,sizeXXL:!0,sizeUnit:!0,sizeStep:!0,motionBase:!0,motionUnit:!0},DA={screenXS:!0,screenXSMin:!0,screenXSMax:!0,screenSM:!0,screenSMMin:!0,screenSMMax:!0,screenMD:!0,screenMDMin:!0,screenMDMax:!0,screenLG:!0,screenLGMin:!0,screenLGMax:!0,screenXL:!0,screenXLMin:!0,screenXLMax:!0,screenXXL:!0,screenXXLMin:!0},M2=(e,t,n)=>{const r=n.getDerivativeToken(e),{override:a}=t,l=Cx(t,["override"]);let s=Object.assign(Object.assign({},r),{override:a});return s=_2(s),l&&Object.entries(l).forEach(u=>{let[d,g]=u;const{theme:h}=g,p=Cx(g,["theme"]);let y=p;h&&(y=M2(Object.assign(Object.assign({},s),p),{override:p},h)),s[d]=y}),s};function Br(){const{token:e,hashed:t,theme:n,override:r,cssVar:a}=se.useContext(ry),l=`${PA}-${t||""}`,s=n||E2,[u,d,g]=C3(s,[ou,e],{salt:l,override:r,getComputedToken:M2,formatToken:_2,cssVar:a&&{prefix:a.prefix,key:a.key,unitless:T2,ignore:zA,preserve:DA}});return[s,g,t?d:"",u,a]}const Pd={overflow:"hidden",whiteSpace:"nowrap",textOverflow:"ellipsis"},Rr=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;return{boxSizing:"border-box",margin:0,padding:0,color:e.colorText,fontSize:e.fontSize,lineHeight:e.lineHeight,listStyle:"none",fontFamily:t?"inherit":e.fontFamily}},ms=()=>({display:"inline-flex",alignItems:"center",color:"inherit",fontStyle:"normal",lineHeight:0,textAlign:"center",textTransform:"none",verticalAlign:"-0.125em",textRendering:"optimizeLegibility","-webkit-font-smoothing":"antialiased","-moz-osx-font-smoothing":"grayscale","> *":{lineHeight:1},svg:{display:"inline-block"}}),Ad=()=>({"&::before":{display:"table",content:'""'},"&::after":{display:"table",clear:"both",content:'""'}}),LA=e=>({a:{color:e.colorLink,textDecoration:e.linkDecoration,backgroundColor:"transparent",outline:"none",cursor:"pointer",transition:`color ${e.motionDurationSlow}`,"-webkit-text-decoration-skip":"objects","&:hover":{color:e.colorLinkHover},"&:active":{color:e.colorLinkActive},"&:active, &:hover":{textDecoration:e.linkHoverDecoration,outline:0},"&:focus":{textDecoration:e.linkFocusDecoration,outline:0},"&[disabled]":{color:e.colorTextDisabled,cursor:"not-allowed"}}}),BA=(e,t,n,r)=>{const a=`[class^="${t}"], [class*=" ${t}"]`,l=n?`.${n}`:a,s={boxSizing:"border-box","&::before, &::after":{boxSizing:"border-box"}};let u={};return r!==!1&&(u={fontFamily:e.fontFamily,fontSize:e.fontSize}),{[l]:Object.assign(Object.assign(Object.assign({},u),s),{[a]:s})}},N2=(e,t)=>({outline:`${ve(e.lineWidthFocus)} solid ${e.colorPrimaryBorder}`,outlineOffset:t??1,transition:"outline-offset 0s, outline 0s"}),gs=(e,t)=>({"&:focus-visible":Object.assign({},N2(e,t))}),j2=e=>({[`.${e}`]:Object.assign(Object.assign({},ms()),{[`.${e} .${e}-icon`]:{display:"block"}})}),I2=e=>Object.assign(Object.assign({color:e.colorLink,textDecoration:e.linkDecoration,outline:"none",cursor:"pointer",transition:`all ${e.motionDurationSlow}`,border:0,padding:0,background:"none",userSelect:"none"},gs(e)),{"&:focus, &:hover":{color:e.colorLinkHover},"&:active":{color:e.colorLinkActive}}),{genStyleHooks:xn,genComponentStyleHook:HA,genSubStyleComponent:hs}=IA({usePrefix:()=>{const{getPrefixCls:e,iconPrefixCls:t}=f.useContext(dt);return{rootPrefixCls:e(),iconPrefixCls:t}},useToken:()=>{const[e,t,n,r,a]=Br();return{theme:e,realToken:t,hashId:n,token:r,cssVar:a}},useCSP:()=>{const{csp:e}=f.useContext(dt);return e??{}},getResetStyles:(e,t)=>{var n;const r=LA(e);return[r,{"&":r},j2((n=t==null?void 0:t.prefix.iconPrefixCls)!==null&&n!==void 0?n:mm)]},getCommonStyle:BA,getCompUnitless:()=>T2});function P2(e,t){return vi.reduce((n,r)=>{const a=e[`${r}1`],l=e[`${r}3`],s=e[`${r}6`],u=e[`${r}7`];return Object.assign(Object.assign({},n),t(r,{lightColor:a,lightBorderColor:l,darkColor:s,textColor:u}))},{})}const FA=(e,t)=>{const[n,r]=Br();return Wv({token:r,hashId:"",path:["ant-design-icons",e],nonce:()=>t==null?void 0:t.nonce,layer:{name:"antd"}},()=>[j2(e)])},VA=Object.assign({},lm),{useId:xx}=VA,UA=()=>"",kA=typeof xx>"u"?UA:xx;function qA(e,t,n){var r;Eu();const a=e||{},l=a.inherit===!1||!t?Object.assign(Object.assign({},Id),{hashed:(r=t==null?void 0:t.hashed)!==null&&r!==void 0?r:Id.hashed,cssVar:t==null?void 0:t.cssVar}):t,s=kA();return xu(()=>{var u,d;if(!e)return t;const g=Object.assign({},l.components);Object.keys(e.components||{}).forEach(y=>{g[y]=Object.assign(Object.assign({},g[y]),e.components[y])});const h=`css-var-${s.replace(/:/g,"")}`,p=((u=a.cssVar)!==null&&u!==void 0?u:l.cssVar)&&Object.assign(Object.assign(Object.assign({prefix:n==null?void 0:n.prefixCls},typeof l.cssVar=="object"?l.cssVar:{}),typeof a.cssVar=="object"?a.cssVar:{}),{key:typeof a.cssVar=="object"&&((d=a.cssVar)===null||d===void 0?void 0:d.key)||h});return Object.assign(Object.assign(Object.assign({},l),a),{token:Object.assign(Object.assign({},l.token),a.token),components:g,cssVar:p})},[a,l],(u,d)=>u.some((g,h)=>{const p=d[h];return!ru(g,p,!0)}))}var WA=["children"],A2=f.createContext({});function GA(e){var t=e.children,n=$t(e,WA);return f.createElement(A2.Provider,{value:n},t)}var KA=function(e){Po(n,e);var t=Ao(n);function n(){return Wn(this,n),t.apply(this,arguments)}return Gn(n,[{key:"render",value:function(){return this.props.children}}]),n}(f.Component);function XA(e){var t=f.useReducer(function(u){return u+1},0),n=ge(t,2),r=n[1],a=f.useRef(e),l=An(function(){return a.current}),s=An(function(u){a.current=typeof u=="function"?u(a.current):u,r()});return[l,s]}var fi="none",Wf="appear",Gf="enter",Kf="leave",wx="none",Ma="prepare",Gl="start",Kl="active",oy="end",z2="prepared";function Ex(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit".concat(e)]="webkit".concat(t),n["Moz".concat(e)]="moz".concat(t),n["ms".concat(e)]="MS".concat(t),n["O".concat(e)]="o".concat(t.toLowerCase()),n}function YA(e,t){var n={animationend:Ex("Animation","AnimationEnd"),transitionend:Ex("Transition","TransitionEnd")};return e&&("AnimationEvent"in t||delete n.animationend.animation,"TransitionEvent"in t||delete n.transitionend.transition),n}var QA=YA($r(),typeof window<"u"?window:{}),D2={};if($r()){var ZA=document.createElement("div");D2=ZA.style}var Xf={};function L2(e){if(Xf[e])return Xf[e];var t=QA[e];if(t)for(var n=Object.keys(t),r=n.length,a=0;a<r;a+=1){var l=n[a];if(Object.prototype.hasOwnProperty.call(t,l)&&l in D2)return Xf[e]=t[l],Xf[e]}return""}var B2=L2("animationend"),H2=L2("transitionend"),F2=!!(B2&&H2),$x=B2||"animationend",Ox=H2||"transitionend";function Rx(e,t){if(!e)return null;if(st(e)==="object"){var n=t.replace(/-\w/g,function(r){return r[1].toUpperCase()});return e[n]}return"".concat(e,"-").concat(t)}const JA=function(e){var t=f.useRef();function n(a){a&&(a.removeEventListener(Ox,e),a.removeEventListener($x,e))}function r(a){t.current&&t.current!==a&&n(t.current),a&&a!==t.current&&(a.addEventListener(Ox,e),a.addEventListener($x,e),t.current=a)}return f.useEffect(function(){return function(){n(t.current)}},[]),[r,n]};var V2=$r()?f.useLayoutEffect:f.useEffect;const e6=function(){var e=f.useRef(null);function t(){bn.cancel(e.current)}function n(r){var a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:2;t();var l=bn(function(){a<=1?r({isCanceled:function(){return l!==e.current}}):n(r,a-1)});e.current=l}return f.useEffect(function(){return function(){t()}},[]),[n,t]};var t6=[Ma,Gl,Kl,oy],n6=[Ma,z2],U2=!1,r6=!0;function k2(e){return e===Kl||e===oy}const a6=function(e,t,n){var r=Zi(wx),a=ge(r,2),l=a[0],s=a[1],u=e6(),d=ge(u,2),g=d[0],h=d[1];function p(){s(Ma,!0)}var y=t?n6:t6;return V2(function(){if(l!==wx&&l!==oy){var C=y.indexOf(l),b=y[C+1],x=n(l);x===U2?s(b,!0):b&&g(function(S){function E(){S.isCanceled()||s(b,!0)}x===!0?E():Promise.resolve(x).then(E)})}},[e,l]),f.useEffect(function(){return function(){h()}},[]),[p,l]};function o6(e,t,n,r){var a=r.motionEnter,l=a===void 0?!0:a,s=r.motionAppear,u=s===void 0?!0:s,d=r.motionLeave,g=d===void 0?!0:d,h=r.motionDeadline,p=r.motionLeaveImmediately,y=r.onAppearPrepare,C=r.onEnterPrepare,b=r.onLeavePrepare,x=r.onAppearStart,S=r.onEnterStart,E=r.onLeaveStart,$=r.onAppearActive,O=r.onEnterActive,_=r.onLeaveActive,R=r.onAppearEnd,T=r.onEnterEnd,M=r.onLeaveEnd,z=r.onVisibleChanged,P=Zi(),D=ge(P,2),F=D[0],G=D[1],V=XA(fi),U=ge(V,2),K=U[0],j=U[1],A=Zi(null),L=ge(A,2),q=L[0],N=L[1],B=K(),W=f.useRef(!1),X=f.useRef(null);function k(){return n()}var Y=f.useRef(!1);function ee(){j(fi),N(null,!0)}var ue=An(function(Me){var De=K();if(De!==fi){var xe=k();if(!(Me&&!Me.deadline&&Me.target!==xe)){var Te=Y.current,Le;De===Wf&&Te?Le=R==null?void 0:R(xe,Me):De===Gf&&Te?Le=T==null?void 0:T(xe,Me):De===Kf&&Te&&(Le=M==null?void 0:M(xe,Me)),Te&&Le!==!1&&ee()}}}),ae=JA(ue),oe=ge(ae,1),he=oe[0],ie=function(De){switch(De){case Wf:return Z(Z(Z({},Ma,y),Gl,x),Kl,$);case Gf:return Z(Z(Z({},Ma,C),Gl,S),Kl,O);case Kf:return Z(Z(Z({},Ma,b),Gl,E),Kl,_);default:return{}}},le=f.useMemo(function(){return ie(B)},[B]),be=a6(B,!e,function(Me){if(Me===Ma){var De=le[Ma];return De?De(k()):U2}if(Se in le){var xe;N(((xe=le[Se])===null||xe===void 0?void 0:xe.call(le,k(),null))||null)}return Se===Kl&&B!==fi&&(he(k()),h>0&&(clearTimeout(X.current),X.current=setTimeout(function(){ue({deadline:!0})},h))),Se===z2&&ee(),r6}),Re=ge(be,2),fe=Re[0],Se=Re[1],Ee=k2(Se);Y.current=Ee;var Oe=f.useRef(null);V2(function(){if(!(W.current&&Oe.current===t)){G(t);var Me=W.current;W.current=!0;var De;!Me&&t&&u&&(De=Wf),Me&&t&&l&&(De=Gf),(Me&&!t&&g||!Me&&p&&!t&&g)&&(De=Kf);var xe=ie(De);De&&(e||xe[Ma])?(j(De),fe()):j(fi),Oe.current=t}},[t]),f.useEffect(function(){(B===Wf&&!u||B===Gf&&!l||B===Kf&&!g)&&j(fi)},[u,l,g]),f.useEffect(function(){return function(){W.current=!1,clearTimeout(X.current)}},[]);var Pe=f.useRef(!1);f.useEffect(function(){F&&(Pe.current=!0),F!==void 0&&B===fi&&((Pe.current||F)&&(z==null||z(F)),Pe.current=!0)},[F,B]);var Ye=q;return le[Ma]&&Se===Gl&&(Ye=Q({transition:"none"},Ye)),[B,Se,Ye,F??t]}function i6(e){var t=e;st(e)==="object"&&(t=e.transitionSupport);function n(a,l){return!!(a.motionName&&t&&l!==!1)}var r=f.forwardRef(function(a,l){var s=a.visible,u=s===void 0?!0:s,d=a.removeOnLeave,g=d===void 0?!0:d,h=a.forceRender,p=a.children,y=a.motionName,C=a.leavedClassName,b=a.eventProps,x=f.useContext(A2),S=x.motion,E=n(a,S),$=f.useRef(),O=f.useRef();function _(){try{return $.current instanceof HTMLElement?$.current:sd(O.current)}catch{return null}}var R=o6(E,u,_,a),T=ge(R,4),M=T[0],z=T[1],P=T[2],D=T[3],F=f.useRef(D);D&&(F.current=!0);var G=f.useCallback(function(L){$.current=L,W0(l,L)},[l]),V,U=Q(Q({},b),{},{visible:u});if(!p)V=null;else if(M===fi)D?V=p(Q({},U),G):!g&&F.current&&C?V=p(Q(Q({},U),{},{className:C}),G):h||!g&&!C?V=p(Q(Q({},U),{},{style:{display:"none"}}),G):V=null;else{var K;z===Ma?K="prepare":k2(z)?K="active":z===Gl&&(K="start");var j=Rx(y,"".concat(M,"-").concat(K));V=p(Q(Q({},U),{},{className:re(Rx(y,M),Z(Z({},j,j&&K),y,typeof y=="string")),style:P}),G)}if(f.isValidElement(V)&&rl(V)){var A=fs(V);A||(V=f.cloneElement(V,{ref:G}))}return f.createElement(KA,{ref:O},V)});return r.displayName="CSSMotion",r}const $a=i6(F2);var Kv="add",Xv="keep",Yv="remove",yp="removed";function l6(e){var t;return e&&st(e)==="object"&&"key"in e?t=e:t={key:e},Q(Q({},t),{},{key:String(t.key)})}function Qv(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[];return e.map(l6)}function s6(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[],t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[],n=[],r=0,a=t.length,l=Qv(e),s=Qv(t);l.forEach(function(g){for(var h=!1,p=r;p<a;p+=1){var y=s[p];if(y.key===g.key){r<p&&(n=n.concat(s.slice(r,p).map(function(C){return Q(Q({},C),{},{status:Kv})})),r=p),n.push(Q(Q({},y),{},{status:Xv})),r+=1,h=!0;break}}h||n.push(Q(Q({},g),{},{status:Yv}))}),r<a&&(n=n.concat(s.slice(r).map(function(g){return Q(Q({},g),{},{status:Kv})})));var u={};n.forEach(function(g){var h=g.key;u[h]=(u[h]||0)+1});var d=Object.keys(u).filter(function(g){return u[g]>1});return d.forEach(function(g){n=n.filter(function(h){var p=h.key,y=h.status;return p!==g||y!==Yv}),n.forEach(function(h){h.key===g&&(h.status=Xv)})}),n}var c6=["component","children","onVisibleChanged","onAllRemoved"],u6=["status"],f6=["eventProps","visible","children","motionName","motionAppear","motionEnter","motionLeave","motionLeaveImmediately","motionDeadline","removeOnLeave","leavedClassName","onAppearPrepare","onAppearStart","onAppearActive","onAppearEnd","onEnterStart","onEnterActive","onEnterEnd","onLeaveStart","onLeaveActive","onLeaveEnd"];function d6(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:$a,n=function(r){Po(l,r);var a=Ao(l);function l(){var s;Wn(this,l);for(var u=arguments.length,d=new Array(u),g=0;g<u;g++)d[g]=arguments[g];return s=a.call.apply(a,[this].concat(d)),Z(jt(s),"state",{keyEntities:[]}),Z(jt(s),"removeKey",function(h){s.setState(function(p){var y=p.keyEntities.map(function(C){return C.key!==h?C:Q(Q({},C),{},{status:yp})});return{keyEntities:y}},function(){var p=s.state.keyEntities,y=p.filter(function(C){var b=C.status;return b!==yp}).length;y===0&&s.props.onAllRemoved&&s.props.onAllRemoved()})}),s}return Gn(l,[{key:"render",value:function(){var u=this,d=this.state.keyEntities,g=this.props,h=g.component,p=g.children,y=g.onVisibleChanged;g.onAllRemoved;var C=$t(g,c6),b=h||f.Fragment,x={};return f6.forEach(function(S){x[S]=C[S],delete C[S]}),delete C.keys,f.createElement(b,C,d.map(function(S,E){var $=S.status,O=$t(S,u6),_=$===Kv||$===Xv;return f.createElement(t,je({},x,{key:O.key,visible:_,eventProps:O,onVisibleChanged:function(T){y==null||y(T,{key:O.key}),T||u.removeKey(O.key)}}),function(R,T){return p(Q(Q({},R),{},{index:E}),T)})}))}}],[{key:"getDerivedStateFromProps",value:function(u,d){var g=u.keys,h=d.keyEntities,p=Qv(g),y=s6(h,p);return{keyEntities:y.filter(function(C){var b=h.find(function(x){var S=x.key;return C.key===S});return!(b&&b.status===yp&&C.status===Yv)})}}}]),l}(f.Component);return Z(n,"defaultProps",{component:"div"}),n}const q2=d6(F2);function m6(e){const{children:t}=e,[,n]=Br(),{motion:r}=n,a=f.useRef(!1);return a.current=a.current||r===!1,a.current?f.createElement(GA,{motion:r},t):t}const g6=()=>null;var h6=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const p6=["getTargetContainer","getPopupContainer","renderEmpty","input","pagination","form","select","button"];let zd,W2,G2,K2;function vd(){return zd||iu}function v6(){return W2||mm}function y6(e){return Object.keys(e).some(t=>t.endsWith("Color"))}const b6=e=>{const{prefixCls:t,iconPrefixCls:n,theme:r,holderRender:a}=e;t!==void 0&&(zd=t),n!==void 0&&(W2=n),"holderRender"in e&&(K2=a),r&&(y6(r)?bA(vd(),r):G2=r)},iy=()=>({getPrefixCls:(e,t)=>t||(e?`${vd()}-${e}`:vd()),getIconPrefixCls:v6,getRootPrefixCls:()=>zd||vd(),getTheme:()=>G2,holderRender:K2}),S6=e=>{const{children:t,csp:n,autoInsertSpaceInButton:r,alert:a,anchor:l,form:s,locale:u,componentSize:d,direction:g,space:h,splitter:p,virtual:y,dropdownMatchSelectWidth:C,popupMatchSelectWidth:b,popupOverflow:x,legacyLocale:S,parentContext:E,iconPrefixCls:$,theme:O,componentDisabled:_,segmented:R,statistic:T,spin:M,calendar:z,carousel:P,cascader:D,collapse:F,typography:G,checkbox:V,descriptions:U,divider:K,drawer:j,skeleton:A,steps:L,image:q,layout:N,list:B,mentions:W,modal:X,progress:k,result:Y,slider:ee,breadcrumb:ue,menu:ae,pagination:oe,input:he,textArea:ie,empty:le,badge:be,radio:Re,rate:fe,switch:Se,transfer:Ee,avatar:Oe,message:Pe,tag:Ye,table:Me,card:De,tabs:xe,timeline:Te,timePicker:Le,upload:We,notification:Ge,tree:at,colorPicker:ke,datePicker:Qe,rangePicker:ct,flex:pt,wave:Ze,dropdown:Be,warning:Ue,tour:Ie,tooltip:qe,popover:wt,popconfirm:It,floatButtonGroup:Nn,variant:wn,inputNumber:Rn,treeSelect:dn}=e,kt=f.useCallback((tt,ut)=>{const{prefixCls:qt}=e;if(ut)return ut;const Mt=qt||E.getPrefixCls("");return tt?`${Mt}-${tt}`:Mt},[E.getPrefixCls,e.prefixCls]),ln=$||E.iconPrefixCls||mm,Ot=n||E.csp;FA(ln,Ot);const nt=qA(O,E.theme,{prefixCls:kt("")}),Ke={csp:Ot,autoInsertSpaceInButton:r,alert:a,anchor:l,locale:u||S,direction:g,space:h,splitter:p,virtual:y,popupMatchSelectWidth:b??C,popupOverflow:x,getPrefixCls:kt,iconPrefixCls:ln,theme:nt,segmented:R,statistic:T,spin:M,calendar:z,carousel:P,cascader:D,collapse:F,typography:G,checkbox:V,descriptions:U,divider:K,drawer:j,skeleton:A,steps:L,image:q,input:he,textArea:ie,layout:N,list:B,mentions:W,modal:X,progress:k,result:Y,slider:ee,breadcrumb:ue,menu:ae,pagination:oe,empty:le,badge:be,radio:Re,rate:fe,switch:Se,transfer:Ee,avatar:Oe,message:Pe,tag:Ye,table:Me,card:De,tabs:xe,timeline:Te,timePicker:Le,upload:We,notification:Ge,tree:at,colorPicker:ke,datePicker:Qe,rangePicker:ct,flex:pt,wave:Ze,dropdown:Be,warning:Ue,tour:Ie,tooltip:qe,popover:wt,popconfirm:It,floatButtonGroup:Nn,variant:wn,inputNumber:Rn,treeSelect:dn},Fe=Object.assign({},E);Object.keys(Ke).forEach(tt=>{Ke[tt]!==void 0&&(Fe[tt]=Ke[tt])}),p6.forEach(tt=>{const ut=e[tt];ut&&(Fe[tt]=ut)}),typeof r<"u"&&(Fe.button=Object.assign({autoInsertSpace:r},Fe.button));const vt=xu(()=>Fe,Fe,(tt,ut)=>{const qt=Object.keys(tt),Mt=Object.keys(ut);return qt.length!==Mt.length||qt.some(Pt=>tt[Pt]!==ut[Pt])}),{layer:Ct}=f.useContext(wu),Ve=f.useMemo(()=>({prefixCls:ln,csp:Ot,layer:Ct?"antd":void 0}),[ln,Ot,Ct]);let ze=f.createElement(f.Fragment,null,f.createElement(g6,{dropdownMatchSelectWidth:C}),t);const rt=f.useMemo(()=>{var tt,ut,qt,Mt;return Wl(((tt=Oo.Form)===null||tt===void 0?void 0:tt.defaultValidateMessages)||{},((qt=(ut=vt.locale)===null||ut===void 0?void 0:ut.Form)===null||qt===void 0?void 0:qt.defaultValidateMessages)||{},((Mt=vt.form)===null||Mt===void 0?void 0:Mt.validateMessages)||{},(s==null?void 0:s.validateMessages)||{})},[vt,s==null?void 0:s.validateMessages]);Object.keys(rt).length>0&&(ze=f.createElement(S2.Provider,{value:rt},ze)),u&&(ze=f.createElement(iA,{locale:u,_ANT_MARK__:oA},ze)),ze=f.createElement(V0.Provider,{value:Ve},ze),d&&(ze=f.createElement(SA,{size:d},ze)),ze=f.createElement(m6,null,ze);const ot=f.useMemo(()=>{const tt=nt||{},{algorithm:ut,token:qt,components:Mt,cssVar:Pt}=tt,At=h6(tt,["algorithm","token","components","cssVar"]),tn=ut&&(!Array.isArray(ut)||ut.length>0)?Vv(ut):E2,sn={};Object.entries(Mt||{}).forEach(ar=>{let[Sn,cn]=ar;const Ae=Object.assign({},cn);"algorithm"in Ae&&(Ae.algorithm===!0?Ae.theme=tn:(Array.isArray(Ae.algorithm)||typeof Ae.algorithm=="function")&&(Ae.theme=Vv(Ae.algorithm)),delete Ae.algorithm),sn[Sn]=Ae});const Tt=Object.assign(Object.assign({},ou),qt);return Object.assign(Object.assign({},At),{theme:tn,token:Tt,components:sn,override:Object.assign({override:Tt},sn),cssVar:Pt})},[nt]);return O&&(ze=f.createElement(ry.Provider,{value:ot},ze)),vt.warning&&(ze=f.createElement(eA.Provider,{value:vt.warning},ze)),_!==void 0&&(ze=f.createElement(ay,{disabled:_},ze)),f.createElement(dt.Provider,{value:vt},ze)},Fa=e=>{const t=f.useContext(dt),n=f.useContext(ey);return f.createElement(S6,Object.assign({parentContext:t,legacyLocale:n},e))};Fa.ConfigContext=dt;Fa.SizeContext=Qi;Fa.config=b6;Fa.useConfig=CA;Object.defineProperty(Fa,"SizeContext",{get:()=>Qi});var C6=`accept acceptCharset accessKey action allowFullScreen allowTransparency
    alt async autoComplete autoFocus autoPlay capture cellPadding cellSpacing challenge
    charSet checked classID className colSpan cols content contentEditable contextMenu
    controls coords crossOrigin data dateTime default defer dir disabled download draggable
    encType form formAction formEncType formMethod formNoValidate formTarget frameBorder
    headers height hidden high href hrefLang htmlFor httpEquiv icon id inputMode integrity
    is keyParams keyType kind label lang list loop low manifest marginHeight marginWidth max maxLength media
    mediaGroup method min minLength multiple muted name noValidate nonce open
    optimum pattern placeholder poster preload radioGroup readOnly rel required
    reversed role rowSpan rows sandbox scope scoped scrolling seamless selected
    shape size sizes span spellCheck src srcDoc srcLang srcSet start step style
    summary tabIndex target title type useMap value width wmode wrap`,x6=`onCopy onCut onPaste onCompositionEnd onCompositionStart onCompositionUpdate onKeyDown
    onKeyPress onKeyUp onFocus onBlur onChange onInput onSubmit onClick onContextMenu onDoubleClick
    onDrag onDragEnd onDragEnter onDragExit onDragLeave onDragOver onDragStart onDrop onMouseDown
    onMouseEnter onMouseLeave onMouseMove onMouseOut onMouseOver onMouseUp onSelect onTouchCancel
    onTouchEnd onTouchMove onTouchStart onScroll onWheel onAbort onCanPlay onCanPlayThrough
    onDurationChange onEmptied onEncrypted onEnded onError onLoadedData onLoadedMetadata
    onLoadStart onPause onPlay onPlaying onProgress onRateChange onSeeked onSeeking onStalled onSuspend onTimeUpdate onVolumeChange onWaiting onLoad onError`,w6="".concat(C6," ").concat(x6).split(/[\s\n]+/),E6="aria-",$6="data-";function _x(e,t){return e.indexOf(t)===0}function ta(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,n;t===!1?n={aria:!0,data:!0,attr:!0}:t===!0?n={aria:!0}:n=Q({},t);var r={};return Object.keys(e).forEach(function(a){(n.aria&&(a==="role"||_x(a,E6))||n.data&&_x(a,$6)||n.attr&&w6.includes(a))&&(r[a]=e[a])}),r}function X2(e){return e&&se.isValidElement(e)&&e.type===se.Fragment}const Y2=(e,t,n)=>se.isValidElement(e)?se.cloneElement(e,typeof n=="function"?n(e.props||{}):n):t;function na(e,t){return Y2(e,e,t)}const Tx=e=>typeof e=="object"&&e!=null&&e.nodeType===1,Mx=(e,t)=>(!t||e!=="hidden")&&e!=="visible"&&e!=="clip",Yf=(e,t)=>{if(e.clientHeight<e.scrollHeight||e.clientWidth<e.scrollWidth){const n=getComputedStyle(e,null);return Mx(n.overflowY,t)||Mx(n.overflowX,t)||(r=>{const a=(l=>{if(!l.ownerDocument||!l.ownerDocument.defaultView)return null;try{return l.ownerDocument.defaultView.frameElement}catch{return null}})(r);return!!a&&(a.clientHeight<r.scrollHeight||a.clientWidth<r.scrollWidth)})(e)}return!1},Qf=(e,t,n,r,a,l,s,u)=>l<e&&s>t||l>e&&s<t?0:l<=e&&u<=n||s>=t&&u>=n?l-e-r:s>t&&u<n||l<e&&u>n?s-t+a:0,O6=e=>{const t=e.parentElement;return t??(e.getRootNode().host||null)},Nx=(e,t)=>{var n,r,a,l;if(typeof document>"u")return[];const{scrollMode:s,block:u,inline:d,boundary:g,skipOverflowHiddenElements:h}=t,p=typeof g=="function"?g:j=>j!==g;if(!Tx(e))throw new TypeError("Invalid target");const y=document.scrollingElement||document.documentElement,C=[];let b=e;for(;Tx(b)&&p(b);){if(b=O6(b),b===y){C.push(b);break}b!=null&&b===document.body&&Yf(b)&&!Yf(document.documentElement)||b!=null&&Yf(b,h)&&C.push(b)}const x=(r=(n=window.visualViewport)==null?void 0:n.width)!=null?r:innerWidth,S=(l=(a=window.visualViewport)==null?void 0:a.height)!=null?l:innerHeight,{scrollX:E,scrollY:$}=window,{height:O,width:_,top:R,right:T,bottom:M,left:z}=e.getBoundingClientRect(),{top:P,right:D,bottom:F,left:G}=(j=>{const A=window.getComputedStyle(j);return{top:parseFloat(A.scrollMarginTop)||0,right:parseFloat(A.scrollMarginRight)||0,bottom:parseFloat(A.scrollMarginBottom)||0,left:parseFloat(A.scrollMarginLeft)||0}})(e);let V=u==="start"||u==="nearest"?R-P:u==="end"?M+F:R+O/2-P+F,U=d==="center"?z+_/2-G+D:d==="end"?T+D:z-G;const K=[];for(let j=0;j<C.length;j++){const A=C[j],{height:L,width:q,top:N,right:B,bottom:W,left:X}=A.getBoundingClientRect();if(s==="if-needed"&&R>=0&&z>=0&&M<=S&&T<=x&&(A===y&&!Yf(A)||R>=N&&M<=W&&z>=X&&T<=B))return K;const k=getComputedStyle(A),Y=parseInt(k.borderLeftWidth,10),ee=parseInt(k.borderTopWidth,10),ue=parseInt(k.borderRightWidth,10),ae=parseInt(k.borderBottomWidth,10);let oe=0,he=0;const ie="offsetWidth"in A?A.offsetWidth-A.clientWidth-Y-ue:0,le="offsetHeight"in A?A.offsetHeight-A.clientHeight-ee-ae:0,be="offsetWidth"in A?A.offsetWidth===0?0:q/A.offsetWidth:0,Re="offsetHeight"in A?A.offsetHeight===0?0:L/A.offsetHeight:0;if(y===A)oe=u==="start"?V:u==="end"?V-S:u==="nearest"?Qf($,$+S,S,ee,ae,$+V,$+V+O,O):V-S/2,he=d==="start"?U:d==="center"?U-x/2:d==="end"?U-x:Qf(E,E+x,x,Y,ue,E+U,E+U+_,_),oe=Math.max(0,oe+$),he=Math.max(0,he+E);else{oe=u==="start"?V-N-ee:u==="end"?V-W+ae+le:u==="nearest"?Qf(N,W,L,ee,ae+le,V,V+O,O):V-(N+L/2)+le/2,he=d==="start"?U-X-Y:d==="center"?U-(X+q/2)+ie/2:d==="end"?U-B+ue+ie:Qf(X,B,q,Y,ue+ie,U,U+_,_);const{scrollLeft:fe,scrollTop:Se}=A;oe=Re===0?0:Math.max(0,Math.min(Se+oe/Re,A.scrollHeight-L/Re+le)),he=be===0?0:Math.max(0,Math.min(fe+he/be,A.scrollWidth-q/be+ie)),V+=Se-oe,U+=fe-he}K.push({el:A,top:oe,left:he})}return K},R6=e=>e===!1?{block:"end",inline:"nearest"}:(t=>t===Object(t)&&Object.keys(t).length!==0)(e)?e:{block:"start",inline:"nearest"};function _6(e,t){if(!e.isConnected||!(a=>{let l=a;for(;l&&l.parentNode;){if(l.parentNode===document)return!0;l=l.parentNode instanceof ShadowRoot?l.parentNode.host:l.parentNode}return!1})(e))return;const n=(a=>{const l=window.getComputedStyle(a);return{top:parseFloat(l.scrollMarginTop)||0,right:parseFloat(l.scrollMarginRight)||0,bottom:parseFloat(l.scrollMarginBottom)||0,left:parseFloat(l.scrollMarginLeft)||0}})(e);if((a=>typeof a=="object"&&typeof a.behavior=="function")(t))return t.behavior(Nx(e,t));const r=typeof t=="boolean"||t==null?void 0:t.behavior;for(const{el:a,top:l,left:s}of Nx(e,R6(t))){const u=l-n.top+n.bottom,d=s-n.left+n.right;a.scroll({top:u,left:d,behavior:r})}}const ia=e=>{const[,,,,t]=Br();return t?`${e}-css-var`:""};var lt={BACKSPACE:8,TAB:9,ENTER:13,SHIFT:16,CTRL:17,ALT:18,CAPS_LOCK:20,ESC:27,SPACE:32,END:35,HOME:36,LEFT:37,UP:38,RIGHT:39,DOWN:40,N:78,P:80,META:91,WIN_KEY_RIGHT:92,CONTEXT_MENU:93,F1:112,F2:113,F3:114,F4:115,F5:116,F6:117,F7:118,F8:119,F9:120,F10:121,F11:122,F12:123,SEMICOLON:186,EQUALS:187,WIN_KEY:224},ly=f.forwardRef(function(e,t){var n=e.prefixCls,r=e.style,a=e.className,l=e.duration,s=l===void 0?4.5:l,u=e.showProgress,d=e.pauseOnHover,g=d===void 0?!0:d,h=e.eventKey,p=e.content,y=e.closable,C=e.closeIcon,b=C===void 0?"x":C,x=e.props,S=e.onClick,E=e.onNoticeClose,$=e.times,O=e.hovering,_=f.useState(!1),R=ge(_,2),T=R[0],M=R[1],z=f.useState(0),P=ge(z,2),D=P[0],F=P[1],G=f.useState(0),V=ge(G,2),U=V[0],K=V[1],j=O||T,A=s>0&&u,L=function(){E(h)},q=function(Y){(Y.key==="Enter"||Y.code==="Enter"||Y.keyCode===lt.ENTER)&&L()};f.useEffect(function(){if(!j&&s>0){var k=Date.now()-U,Y=setTimeout(function(){L()},s*1e3-U);return function(){g&&clearTimeout(Y),K(Date.now()-k)}}},[s,j,$]),f.useEffect(function(){if(!j&&A&&(g||U===0)){var k=performance.now(),Y,ee=function ue(){cancelAnimationFrame(Y),Y=requestAnimationFrame(function(ae){var oe=ae+U-k,he=Math.min(oe/(s*1e3),1);F(he*100),he<1&&ue()})};return ee(),function(){g&&cancelAnimationFrame(Y)}}},[s,U,j,A,$]);var N=f.useMemo(function(){return st(y)==="object"&&y!==null?y:y?{closeIcon:b}:{}},[y,b]),B=ta(N,!0),W=100-(!D||D<0?0:D>100?100:D),X="".concat(n,"-notice");return f.createElement("div",je({},x,{ref:t,className:re(X,a,Z({},"".concat(X,"-closable"),y)),style:r,onMouseEnter:function(Y){var ee;M(!0),x==null||(ee=x.onMouseEnter)===null||ee===void 0||ee.call(x,Y)},onMouseLeave:function(Y){var ee;M(!1),x==null||(ee=x.onMouseLeave)===null||ee===void 0||ee.call(x,Y)},onClick:S}),f.createElement("div",{className:"".concat(X,"-content")},p),y&&f.createElement("a",je({tabIndex:0,className:"".concat(X,"-close"),onKeyDown:q,"aria-label":"Close"},B,{onClick:function(Y){Y.preventDefault(),Y.stopPropagation(),L()}}),N.closeIcon),A&&f.createElement("progress",{className:"".concat(X,"-progress"),max:"100",value:W},W+"%"))}),Q2=se.createContext({}),Z2=function(t){var n=t.children,r=t.classNames;return se.createElement(Q2.Provider,{value:{classNames:r}},n)},jx=8,Ix=3,Px=16,T6=function(t){var n={offset:jx,threshold:Ix,gap:Px};if(t&&st(t)==="object"){var r,a,l;n.offset=(r=t.offset)!==null&&r!==void 0?r:jx,n.threshold=(a=t.threshold)!==null&&a!==void 0?a:Ix,n.gap=(l=t.gap)!==null&&l!==void 0?l:Px}return[!!t,n]},M6=["className","style","classNames","styles"],N6=function(t){var n=t.configList,r=t.placement,a=t.prefixCls,l=t.className,s=t.style,u=t.motion,d=t.onAllNoticeRemoved,g=t.onNoticeClose,h=t.stack,p=f.useContext(Q2),y=p.classNames,C=f.useRef({}),b=f.useState(null),x=ge(b,2),S=x[0],E=x[1],$=f.useState([]),O=ge($,2),_=O[0],R=O[1],T=n.map(function(j){return{config:j,key:String(j.key)}}),M=T6(h),z=ge(M,2),P=z[0],D=z[1],F=D.offset,G=D.threshold,V=D.gap,U=P&&(_.length>0||T.length<=G),K=typeof u=="function"?u(r):u;return f.useEffect(function(){P&&_.length>1&&R(function(j){return j.filter(function(A){return T.some(function(L){var q=L.key;return A===q})})})},[_,T,P]),f.useEffect(function(){var j;if(P&&C.current[(j=T[T.length-1])===null||j===void 0?void 0:j.key]){var A;E(C.current[(A=T[T.length-1])===null||A===void 0?void 0:A.key])}},[T,P]),se.createElement(q2,je({key:r,className:re(a,"".concat(a,"-").concat(r),y==null?void 0:y.list,l,Z(Z({},"".concat(a,"-stack"),!!P),"".concat(a,"-stack-expanded"),U)),style:s,keys:T,motionAppear:!0},K,{onAllRemoved:function(){d(r)}}),function(j,A){var L=j.config,q=j.className,N=j.style,B=j.index,W=L,X=W.key,k=W.times,Y=String(X),ee=L,ue=ee.className,ae=ee.style,oe=ee.classNames,he=ee.styles,ie=$t(ee,M6),le=T.findIndex(function(Te){return Te.key===Y}),be={};if(P){var Re=T.length-1-(le>-1?le:B-1),fe=r==="top"||r==="bottom"?"-50%":"0";if(Re>0){var Se,Ee,Oe;be.height=U?(Se=C.current[Y])===null||Se===void 0?void 0:Se.offsetHeight:S==null?void 0:S.offsetHeight;for(var Pe=0,Ye=0;Ye<Re;Ye++){var Me;Pe+=((Me=C.current[T[T.length-1-Ye].key])===null||Me===void 0?void 0:Me.offsetHeight)+V}var De=(U?Pe:Re*F)*(r.startsWith("top")?1:-1),xe=!U&&S!==null&&S!==void 0&&S.offsetWidth&&(Ee=C.current[Y])!==null&&Ee!==void 0&&Ee.offsetWidth?((S==null?void 0:S.offsetWidth)-F*2*(Re<3?Re:3))/((Oe=C.current[Y])===null||Oe===void 0?void 0:Oe.offsetWidth):1;be.transform="translate3d(".concat(fe,", ").concat(De,"px, 0) scaleX(").concat(xe,")")}else be.transform="translate3d(".concat(fe,", 0, 0)")}return se.createElement("div",{ref:A,className:re("".concat(a,"-notice-wrapper"),q,oe==null?void 0:oe.wrapper),style:Q(Q(Q({},N),be),he==null?void 0:he.wrapper),onMouseEnter:function(){return R(function(Le){return Le.includes(Y)?Le:[].concat(Ne(Le),[Y])})},onMouseLeave:function(){return R(function(Le){return Le.filter(function(We){return We!==Y})})}},se.createElement(ly,je({},ie,{ref:function(Le){le>-1?C.current[Y]=Le:delete C.current[Y]},prefixCls:a,classNames:oe,styles:he,className:re(ue,y==null?void 0:y.notice),style:ae,times:k,key:X,eventKey:X,onNoticeClose:g,hovering:P&&_.length>0})))})},j6=f.forwardRef(function(e,t){var n=e.prefixCls,r=n===void 0?"rc-notification":n,a=e.container,l=e.motion,s=e.maxCount,u=e.className,d=e.style,g=e.onAllRemoved,h=e.stack,p=e.renderNotifications,y=f.useState([]),C=ge(y,2),b=C[0],x=C[1],S=function(P){var D,F=b.find(function(G){return G.key===P});F==null||(D=F.onClose)===null||D===void 0||D.call(F),x(function(G){return G.filter(function(V){return V.key!==P})})};f.useImperativeHandle(t,function(){return{open:function(P){x(function(D){var F=Ne(D),G=F.findIndex(function(K){return K.key===P.key}),V=Q({},P);if(G>=0){var U;V.times=(((U=D[G])===null||U===void 0?void 0:U.times)||0)+1,F[G]=V}else V.times=0,F.push(V);return s>0&&F.length>s&&(F=F.slice(-s)),F})},close:function(P){S(P)},destroy:function(){x([])}}});var E=f.useState({}),$=ge(E,2),O=$[0],_=$[1];f.useEffect(function(){var z={};b.forEach(function(P){var D=P.placement,F=D===void 0?"topRight":D;F&&(z[F]=z[F]||[],z[F].push(P))}),Object.keys(O).forEach(function(P){z[P]=z[P]||[]}),_(z)},[b]);var R=function(P){_(function(D){var F=Q({},D),G=F[P]||[];return G.length||delete F[P],F})},T=f.useRef(!1);if(f.useEffect(function(){Object.keys(O).length>0?T.current=!0:T.current&&(g==null||g(),T.current=!1)},[O]),!a)return null;var M=Object.keys(O);return pi.createPortal(f.createElement(f.Fragment,null,M.map(function(z){var P=O[z],D=f.createElement(N6,{key:z,configList:P,placement:z,prefixCls:r,className:u==null?void 0:u(z),style:d==null?void 0:d(z),motion:l,onNoticeClose:S,onAllNoticeRemoved:R,stack:h});return p?p(D,{prefixCls:r,key:z}):D})),a)}),I6=["getContainer","motion","prefixCls","maxCount","className","style","onAllRemoved","stack","renderNotifications"],P6=function(){return document.body},Ax=0;function A6(){for(var e={},t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return n.forEach(function(a){a&&Object.keys(a).forEach(function(l){var s=a[l];s!==void 0&&(e[l]=s)})}),e}function J2(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},t=e.getContainer,n=t===void 0?P6:t,r=e.motion,a=e.prefixCls,l=e.maxCount,s=e.className,u=e.style,d=e.onAllRemoved,g=e.stack,h=e.renderNotifications,p=$t(e,I6),y=f.useState(),C=ge(y,2),b=C[0],x=C[1],S=f.useRef(),E=f.createElement(j6,{container:b,ref:S,prefixCls:a,motion:r,maxCount:l,className:s,style:u,onAllRemoved:d,stack:g,renderNotifications:h}),$=f.useState([]),O=ge($,2),_=O[0],R=O[1],T=An(function(z){var P=A6(p,z);(P.key===null||P.key===void 0)&&(P.key="rc-notification-".concat(Ax),Ax+=1),R(function(D){return[].concat(Ne(D),[{type:"open",config:P}])})}),M=f.useMemo(function(){return{open:T,close:function(P){R(function(D){return[].concat(Ne(D),[{type:"close",key:P}])})},destroy:function(){R(function(P){return[].concat(Ne(P),[{type:"destroy"}])})}}},[]);return f.useEffect(function(){x(n())}),f.useEffect(function(){if(S.current&&_.length){_.forEach(function(D){switch(D.type){case"open":S.current.open(D.config);break;case"close":S.current.close(D.key);break;case"destroy":S.current.destroy();break}});var z,P;R(function(D){return(z!==D||!P)&&(z=D,P=D.filter(function(F){return!_.includes(F)})),P})}},[_]),[M,E]}const gm=se.createContext(void 0),di=100,z6=10,sy=di*z6,eR={Modal:di,Drawer:di,Popover:di,Popconfirm:di,Tooltip:di,Tour:di,FloatButton:di},D6={SelectLike:50,Dropdown:50,DatePicker:50,Menu:50,ImagePreview:1};function L6(e){return e in eR}const $u=(e,t)=>{const[,n]=Br(),r=se.useContext(gm),a=L6(e);let l;if(t!==void 0)l=[t,t];else{let s=r??0;a?s+=(r?0:n.zIndexPopupBase)+eR[e]:s+=D6[e],l=[r===void 0?t:s,s]}return l},B6=e=>{const{componentCls:t,iconCls:n,boxShadow:r,colorText:a,colorSuccess:l,colorError:s,colorWarning:u,colorInfo:d,fontSizeLG:g,motionEaseInOutCirc:h,motionDurationSlow:p,marginXS:y,paddingXS:C,borderRadiusLG:b,zIndexPopup:x,contentPadding:S,contentBg:E}=e,$=`${t}-notice`,O=new Dt("MessageMoveIn",{"0%":{padding:0,transform:"translateY(-100%)",opacity:0},"100%":{padding:C,transform:"translateY(0)",opacity:1}}),_=new Dt("MessageMoveOut",{"0%":{maxHeight:e.height,padding:C,opacity:1},"100%":{maxHeight:0,padding:0,opacity:0}}),R={padding:C,textAlign:"center",[`${t}-custom-content`]:{display:"flex",alignItems:"center"},[`${t}-custom-content > ${n}`]:{marginInlineEnd:y,fontSize:g},[`${$}-content`]:{display:"inline-block",padding:S,background:E,borderRadius:b,boxShadow:r,pointerEvents:"all"},[`${t}-success > ${n}`]:{color:l},[`${t}-error > ${n}`]:{color:s},[`${t}-warning > ${n}`]:{color:u},[`${t}-info > ${n},
      ${t}-loading > ${n}`]:{color:d}};return[{[t]:Object.assign(Object.assign({},Rr(e)),{color:a,position:"fixed",top:y,width:"100%",pointerEvents:"none",zIndex:x,[`${t}-move-up`]:{animationFillMode:"forwards"},[`
        ${t}-move-up-appear,
        ${t}-move-up-enter
      `]:{animationName:O,animationDuration:p,animationPlayState:"paused",animationTimingFunction:h},[`
        ${t}-move-up-appear${t}-move-up-appear-active,
        ${t}-move-up-enter${t}-move-up-enter-active
      `]:{animationPlayState:"running"},[`${t}-move-up-leave`]:{animationName:_,animationDuration:p,animationPlayState:"paused",animationTimingFunction:h},[`${t}-move-up-leave${t}-move-up-leave-active`]:{animationPlayState:"running"},"&-rtl":{direction:"rtl",span:{direction:"rtl"}}})},{[t]:{[`${$}-wrapper`]:Object.assign({},R)}},{[`${t}-notice-pure-panel`]:Object.assign(Object.assign({},R),{padding:0,textAlign:"start"})}]},H6=e=>({zIndexPopup:e.zIndexPopupBase+sy+10,contentBg:e.colorBgElevated,contentPadding:`${(e.controlHeightLG-e.fontSize*e.lineHeight)/2}px ${e.paddingSM}px`}),tR=xn("Message",e=>{const t=Kt(e,{height:150});return[B6(t)]},H6);var F6=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const V6={info:f.createElement(q0,null),success:f.createElement(cm,null),error:f.createElement(ss,null),warning:f.createElement(us,null),loading:f.createElement(Cu,null)},nR=e=>{let{prefixCls:t,type:n,icon:r,children:a}=e;return f.createElement("div",{className:re(`${t}-custom-content`,`${t}-${n}`)},r||V6[n],f.createElement("span",null,a))},U6=e=>{const{prefixCls:t,className:n,type:r,icon:a,content:l}=e,s=F6(e,["prefixCls","className","type","icon","content"]),{getPrefixCls:u}=f.useContext(dt),d=t||u("message"),g=ia(d),[h,p,y]=tR(d,g);return h(f.createElement(ly,Object.assign({},s,{prefixCls:d,className:re(n,p,`${d}-notice-pure-panel`,y,g),eventKey:"pure",duration:null,content:f.createElement(nR,{prefixCls:d,type:r,icon:a},l)})))};function k6(e,t){return{motionName:t??`${e}-move-up`}}function cy(e){let t;const n=new Promise(a=>{t=e(()=>{a(!0)})}),r=()=>{t==null||t()};return r.then=(a,l)=>n.then(a,l),r.promise=n,r}var q6=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const W6=8,G6=3,K6=e=>{let{children:t,prefixCls:n}=e;const r=ia(n),[a,l,s]=tR(n,r);return a(f.createElement(Z2,{classNames:{list:re(l,s,r)}},t))},X6=(e,t)=>{let{prefixCls:n,key:r}=t;return f.createElement(K6,{prefixCls:n,key:r},e)},Y6=f.forwardRef((e,t)=>{const{top:n,prefixCls:r,getContainer:a,maxCount:l,duration:s=G6,rtl:u,transitionName:d,onAllRemoved:g}=e,{getPrefixCls:h,getPopupContainer:p,message:y,direction:C}=f.useContext(dt),b=r||h("message"),x=()=>({left:"50%",transform:"translateX(-50%)",top:n??W6}),S=()=>re({[`${b}-rtl`]:u??C==="rtl"}),E=()=>k6(b,d),$=f.createElement("span",{className:`${b}-close-x`},f.createElement(cs,{className:`${b}-close-icon`})),[O,_]=J2({prefixCls:b,style:x,className:S,motion:E,closable:!1,closeIcon:$,duration:s,getContainer:()=>(a==null?void 0:a())||(p==null?void 0:p())||document.body,maxCount:l,onAllRemoved:g,renderNotifications:X6});return f.useImperativeHandle(t,()=>Object.assign(Object.assign({},O),{prefixCls:b,message:y})),_});let zx=0;function rR(e){const t=f.useRef(null);return Eu(),[f.useMemo(()=>{const r=d=>{var g;(g=t.current)===null||g===void 0||g.close(d)},a=d=>{if(!t.current){const T=()=>{};return T.then=()=>{},T}const{open:g,prefixCls:h,message:p}=t.current,y=`${h}-notice`,{content:C,icon:b,type:x,key:S,className:E,style:$,onClose:O}=d,_=q6(d,["content","icon","type","key","className","style","onClose"]);let R=S;return R==null&&(zx+=1,R=`antd-message-${zx}`),cy(T=>(g(Object.assign(Object.assign({},_),{key:R,content:f.createElement(nR,{prefixCls:h,type:x,icon:b},C),placement:"top",className:re(x&&`${y}-${x}`,E,p==null?void 0:p.className),style:Object.assign(Object.assign({},p==null?void 0:p.style),$),onClose:()=>{O==null||O(),T()}})),()=>{r(R)}))},s={open:a,destroy:d=>{var g;d!==void 0?r(d):(g=t.current)===null||g===void 0||g.destroy()}};return["info","success","warning","error","loading"].forEach(d=>{const g=(h,p,y)=>{let C;h&&typeof h=="object"&&"content"in h?C=h:C={content:h};let b,x;typeof p=="function"?x=p:(b=p,x=y);const S=Object.assign(Object.assign({onClose:x,duration:b},C),{type:d});return a(S)};s[d]=g}),s},[]),f.createElement(Y6,Object.assign({key:"message-holder"},e,{ref:t}))]}function Q6(e){return rR(e)}function Z6(){const[e,t]=f.useState([]),n=f.useCallback(r=>(t(a=>[].concat(Ne(a),[r])),()=>{t(a=>a.filter(l=>l!==r))}),[]);return[e,n]}function Nr(){Nr=function(){return t};var e,t={},n=Object.prototype,r=n.hasOwnProperty,a=Object.defineProperty||function(j,A,L){j[A]=L.value},l=typeof Symbol=="function"?Symbol:{},s=l.iterator||"@@iterator",u=l.asyncIterator||"@@asyncIterator",d=l.toStringTag||"@@toStringTag";function g(j,A,L){return Object.defineProperty(j,A,{value:L,enumerable:!0,configurable:!0,writable:!0}),j[A]}try{g({},"")}catch{g=function(L,q,N){return L[q]=N}}function h(j,A,L,q){var N=A&&A.prototype instanceof E?A:E,B=Object.create(N.prototype),W=new U(q||[]);return a(B,"_invoke",{value:D(j,L,W)}),B}function p(j,A,L){try{return{type:"normal",arg:j.call(A,L)}}catch(q){return{type:"throw",arg:q}}}t.wrap=h;var y="suspendedStart",C="suspendedYield",b="executing",x="completed",S={};function E(){}function $(){}function O(){}var _={};g(_,s,function(){return this});var R=Object.getPrototypeOf,T=R&&R(R(K([])));T&&T!==n&&r.call(T,s)&&(_=T);var M=O.prototype=E.prototype=Object.create(_);function z(j){["next","throw","return"].forEach(function(A){g(j,A,function(L){return this._invoke(A,L)})})}function P(j,A){function L(N,B,W,X){var k=p(j[N],j,B);if(k.type!=="throw"){var Y=k.arg,ee=Y.value;return ee&&st(ee)=="object"&&r.call(ee,"__await")?A.resolve(ee.__await).then(function(ue){L("next",ue,W,X)},function(ue){L("throw",ue,W,X)}):A.resolve(ee).then(function(ue){Y.value=ue,W(Y)},function(ue){return L("throw",ue,W,X)})}X(k.arg)}var q;a(this,"_invoke",{value:function(B,W){function X(){return new A(function(k,Y){L(B,W,k,Y)})}return q=q?q.then(X,X):X()}})}function D(j,A,L){var q=y;return function(N,B){if(q===b)throw Error("Generator is already running");if(q===x){if(N==="throw")throw B;return{value:e,done:!0}}for(L.method=N,L.arg=B;;){var W=L.delegate;if(W){var X=F(W,L);if(X){if(X===S)continue;return X}}if(L.method==="next")L.sent=L._sent=L.arg;else if(L.method==="throw"){if(q===y)throw q=x,L.arg;L.dispatchException(L.arg)}else L.method==="return"&&L.abrupt("return",L.arg);q=b;var k=p(j,A,L);if(k.type==="normal"){if(q=L.done?x:C,k.arg===S)continue;return{value:k.arg,done:L.done}}k.type==="throw"&&(q=x,L.method="throw",L.arg=k.arg)}}}function F(j,A){var L=A.method,q=j.iterator[L];if(q===e)return A.delegate=null,L==="throw"&&j.iterator.return&&(A.method="return",A.arg=e,F(j,A),A.method==="throw")||L!=="return"&&(A.method="throw",A.arg=new TypeError("The iterator does not provide a '"+L+"' method")),S;var N=p(q,j.iterator,A.arg);if(N.type==="throw")return A.method="throw",A.arg=N.arg,A.delegate=null,S;var B=N.arg;return B?B.done?(A[j.resultName]=B.value,A.next=j.nextLoc,A.method!=="return"&&(A.method="next",A.arg=e),A.delegate=null,S):B:(A.method="throw",A.arg=new TypeError("iterator result is not an object"),A.delegate=null,S)}function G(j){var A={tryLoc:j[0]};1 in j&&(A.catchLoc=j[1]),2 in j&&(A.finallyLoc=j[2],A.afterLoc=j[3]),this.tryEntries.push(A)}function V(j){var A=j.completion||{};A.type="normal",delete A.arg,j.completion=A}function U(j){this.tryEntries=[{tryLoc:"root"}],j.forEach(G,this),this.reset(!0)}function K(j){if(j||j===""){var A=j[s];if(A)return A.call(j);if(typeof j.next=="function")return j;if(!isNaN(j.length)){var L=-1,q=function N(){for(;++L<j.length;)if(r.call(j,L))return N.value=j[L],N.done=!1,N;return N.value=e,N.done=!0,N};return q.next=q}}throw new TypeError(st(j)+" is not iterable")}return $.prototype=O,a(M,"constructor",{value:O,configurable:!0}),a(O,"constructor",{value:$,configurable:!0}),$.displayName=g(O,d,"GeneratorFunction"),t.isGeneratorFunction=function(j){var A=typeof j=="function"&&j.constructor;return!!A&&(A===$||(A.displayName||A.name)==="GeneratorFunction")},t.mark=function(j){return Object.setPrototypeOf?Object.setPrototypeOf(j,O):(j.__proto__=O,g(j,d,"GeneratorFunction")),j.prototype=Object.create(M),j},t.awrap=function(j){return{__await:j}},z(P.prototype),g(P.prototype,u,function(){return this}),t.AsyncIterator=P,t.async=function(j,A,L,q,N){N===void 0&&(N=Promise);var B=new P(h(j,A,L,q),N);return t.isGeneratorFunction(A)?B:B.next().then(function(W){return W.done?W.value:B.next()})},z(M),g(M,d,"Generator"),g(M,s,function(){return this}),g(M,"toString",function(){return"[object Generator]"}),t.keys=function(j){var A=Object(j),L=[];for(var q in A)L.push(q);return L.reverse(),function N(){for(;L.length;){var B=L.pop();if(B in A)return N.value=B,N.done=!1,N}return N.done=!0,N}},t.values=K,U.prototype={constructor:U,reset:function(A){if(this.prev=0,this.next=0,this.sent=this._sent=e,this.done=!1,this.delegate=null,this.method="next",this.arg=e,this.tryEntries.forEach(V),!A)for(var L in this)L.charAt(0)==="t"&&r.call(this,L)&&!isNaN(+L.slice(1))&&(this[L]=e)},stop:function(){this.done=!0;var A=this.tryEntries[0].completion;if(A.type==="throw")throw A.arg;return this.rval},dispatchException:function(A){if(this.done)throw A;var L=this;function q(Y,ee){return W.type="throw",W.arg=A,L.next=Y,ee&&(L.method="next",L.arg=e),!!ee}for(var N=this.tryEntries.length-1;N>=0;--N){var B=this.tryEntries[N],W=B.completion;if(B.tryLoc==="root")return q("end");if(B.tryLoc<=this.prev){var X=r.call(B,"catchLoc"),k=r.call(B,"finallyLoc");if(X&&k){if(this.prev<B.catchLoc)return q(B.catchLoc,!0);if(this.prev<B.finallyLoc)return q(B.finallyLoc)}else if(X){if(this.prev<B.catchLoc)return q(B.catchLoc,!0)}else{if(!k)throw Error("try statement without catch or finally");if(this.prev<B.finallyLoc)return q(B.finallyLoc)}}}},abrupt:function(A,L){for(var q=this.tryEntries.length-1;q>=0;--q){var N=this.tryEntries[q];if(N.tryLoc<=this.prev&&r.call(N,"finallyLoc")&&this.prev<N.finallyLoc){var B=N;break}}B&&(A==="break"||A==="continue")&&B.tryLoc<=L&&L<=B.finallyLoc&&(B=null);var W=B?B.completion:{};return W.type=A,W.arg=L,B?(this.method="next",this.next=B.finallyLoc,S):this.complete(W)},complete:function(A,L){if(A.type==="throw")throw A.arg;return A.type==="break"||A.type==="continue"?this.next=A.arg:A.type==="return"?(this.rval=this.arg=A.arg,this.method="return",this.next="end"):A.type==="normal"&&L&&(this.next=L),S},finish:function(A){for(var L=this.tryEntries.length-1;L>=0;--L){var q=this.tryEntries[L];if(q.finallyLoc===A)return this.complete(q.completion,q.afterLoc),V(q),S}},catch:function(A){for(var L=this.tryEntries.length-1;L>=0;--L){var q=this.tryEntries[L];if(q.tryLoc===A){var N=q.completion;if(N.type==="throw"){var B=N.arg;V(q)}return B}}throw Error("illegal catch attempt")},delegateYield:function(A,L,q){return this.delegate={iterator:K(A),resultName:L,nextLoc:q},this.method==="next"&&(this.arg=e),S}},t}function Dx(e,t,n,r,a,l,s){try{var u=e[l](s),d=u.value}catch(g){return void n(g)}u.done?t(d):Promise.resolve(d).then(r,a)}function al(e){return function(){var t=this,n=arguments;return new Promise(function(r,a){var l=e.apply(t,n);function s(d){Dx(l,r,a,s,u,"next",d)}function u(d){Dx(l,r,a,s,u,"throw",d)}s(void 0)})}}var Ou=Q({},xP),J6=Ou.version,bp=Ou.render,ez=Ou.unmountComponentAtNode,hm;try{var tz=Number((J6||"").split(".")[0]);tz>=18&&(hm=Ou.createRoot)}catch{}function Lx(e){var t=Ou.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;t&&st(t)==="object"&&(t.usingClientEntryPoint=e)}var Dd="__rc_react_root__";function nz(e,t){Lx(!0);var n=t[Dd]||hm(t);Lx(!1),n.render(e),t[Dd]=n}function rz(e,t){bp==null||bp(e,t)}function az(e,t){if(hm){nz(e,t);return}rz(e,t)}function oz(e){return Zv.apply(this,arguments)}function Zv(){return Zv=al(Nr().mark(function e(t){return Nr().wrap(function(r){for(;;)switch(r.prev=r.next){case 0:return r.abrupt("return",Promise.resolve().then(function(){var a;(a=t[Dd])===null||a===void 0||a.unmount(),delete t[Dd]}));case 1:case"end":return r.stop()}},e)})),Zv.apply(this,arguments)}function iz(e){ez(e)}function lz(e){return Jv.apply(this,arguments)}function Jv(){return Jv=al(Nr().mark(function e(t){return Nr().wrap(function(r){for(;;)switch(r.prev=r.next){case 0:if(hm===void 0){r.next=2;break}return r.abrupt("return",oz(t));case 2:iz(t);case 3:case"end":return r.stop()}},e)})),Jv.apply(this,arguments)}const sz=(e,t)=>(az(e,t),()=>lz(t));let cz=sz;function pm(){return cz}const Sp=()=>({height:0,opacity:0}),Bx=e=>{const{scrollHeight:t}=e;return{height:t,opacity:1}},uz=e=>({height:e?e.offsetHeight:0}),Cp=(e,t)=>(t==null?void 0:t.deadline)===!0||t.propertyName==="height",Ld=function(){return{motionName:`${arguments.length>0&&arguments[0]!==void 0?arguments[0]:iu}-motion-collapse`,onAppearStart:Sp,onEnterStart:Sp,onAppearActive:Bx,onEnterActive:Bx,onLeaveStart:uz,onLeaveActive:Sp,onAppearEnd:Cp,onEnterEnd:Cp,onLeaveEnd:Cp,motionDeadline:500}},_o=(e,t,n)=>n!==void 0?n:`${e}-${t}`;function On(e,t){var n=Object.assign({},e);return Array.isArray(t)&&t.forEach(function(r){delete n[r]}),n}const vm=function(e){if(!e)return!1;if(e instanceof Element){if(e.offsetParent)return!0;if(e.getBBox){var t=e.getBBox(),n=t.width,r=t.height;if(n||r)return!0}if(e.getBoundingClientRect){var a=e.getBoundingClientRect(),l=a.width,s=a.height;if(l||s)return!0}}return!1},fz=e=>{const{componentCls:t,colorPrimary:n}=e;return{[t]:{position:"absolute",background:"transparent",pointerEvents:"none",boxSizing:"border-box",color:`var(--wave-color, ${n})`,boxShadow:"0 0 0 0 currentcolor",opacity:.2,"&.wave-motion-appear":{transition:[`box-shadow 0.4s ${e.motionEaseOutCirc}`,`opacity 2s ${e.motionEaseOutCirc}`].join(","),"&-active":{boxShadow:"0 0 0 6px currentcolor",opacity:0},"&.wave-quick":{transition:[`box-shadow ${e.motionDurationSlow} ${e.motionEaseInOut}`,`opacity ${e.motionDurationSlow} ${e.motionEaseInOut}`].join(",")}}}}},dz=HA("Wave",e=>[fz(e)]),aR=`${iu}-wave-target`;function xp(e){return e&&e!=="#fff"&&e!=="#ffffff"&&e!=="rgb(255, 255, 255)"&&e!=="rgba(255, 255, 255, 1)"&&!/rgba\((?:\d*, ){3}0\)/.test(e)&&e!=="transparent"}function mz(e){const{borderTopColor:t,borderColor:n,backgroundColor:r}=getComputedStyle(e);return xp(t)?t:xp(n)?n:xp(r)?r:null}function wp(e){return Number.isNaN(e)?0:e}const gz=e=>{const{className:t,target:n,component:r,registerUnmount:a}=e,l=f.useRef(null),s=f.useRef(null);f.useEffect(()=>{s.current=a()},[]);const[u,d]=f.useState(null),[g,h]=f.useState([]),[p,y]=f.useState(0),[C,b]=f.useState(0),[x,S]=f.useState(0),[E,$]=f.useState(0),[O,_]=f.useState(!1),R={left:p,top:C,width:x,height:E,borderRadius:g.map(z=>`${z}px`).join(" ")};u&&(R["--wave-color"]=u);function T(){const z=getComputedStyle(n);d(mz(n));const P=z.position==="static",{borderLeftWidth:D,borderTopWidth:F}=z;y(P?n.offsetLeft:wp(-parseFloat(D))),b(P?n.offsetTop:wp(-parseFloat(F))),S(n.offsetWidth),$(n.offsetHeight);const{borderTopLeftRadius:G,borderTopRightRadius:V,borderBottomLeftRadius:U,borderBottomRightRadius:K}=z;h([G,V,K,U].map(j=>wp(parseFloat(j))))}if(f.useEffect(()=>{if(n){const z=bn(()=>{T(),_(!0)});let P;return typeof ResizeObserver<"u"&&(P=new ResizeObserver(T),P.observe(n)),()=>{bn.cancel(z),P==null||P.disconnect()}}},[]),!O)return null;const M=(r==="Checkbox"||r==="Radio")&&(n==null?void 0:n.classList.contains(aR));return f.createElement($a,{visible:!0,motionAppear:!0,motionName:"wave-motion",motionDeadline:5e3,onAppearEnd:(z,P)=>{var D,F;if(P.deadline||P.propertyName==="opacity"){const G=(D=l.current)===null||D===void 0?void 0:D.parentElement;(F=s.current)===null||F===void 0||F.call(s).then(()=>{G==null||G.remove()})}return!1}},(z,P)=>{let{className:D}=z;return f.createElement("div",{ref:Ha(l,P),className:re(t,D,{"wave-quick":M}),style:R})})},hz=(e,t)=>{var n;const{component:r}=t;if(r==="Checkbox"&&!(!((n=e.querySelector("input"))===null||n===void 0)&&n.checked))return;const a=document.createElement("div");a.style.position="absolute",a.style.left="0px",a.style.top="0px",e==null||e.insertBefore(a,e==null?void 0:e.firstChild);const l=pm();let s=null;function u(){return s}s=l(f.createElement(gz,Object.assign({},t,{target:e,registerUnmount:u})),a)},pz=(e,t,n)=>{const{wave:r}=f.useContext(dt),[,a,l]=Br(),s=An(g=>{const h=e.current;if(r!=null&&r.disabled||!h)return;const p=h.querySelector(`.${aR}`)||h,{showEffect:y}=r||{};(y||hz)(p,{className:t,token:a,component:n,event:g,hashId:l})}),u=f.useRef(null);return g=>{bn.cancel(u.current),u.current=bn(()=>{s(g)})}},oR=e=>{const{children:t,disabled:n,component:r}=e,{getPrefixCls:a}=f.useContext(dt),l=f.useRef(null),s=a("wave"),[,u]=dz(s),d=pz(l,re(s,u),r);if(se.useEffect(()=>{const h=l.current;if(!h||h.nodeType!==1||n)return;const p=y=>{!vm(y.target)||!h.getAttribute||h.getAttribute("disabled")||h.disabled||h.className.includes("disabled")||h.className.includes("-leave")||d(y)};return h.addEventListener("click",p,!0),()=>{h.removeEventListener("click",p,!0)}},[n]),!se.isValidElement(t))return t??null;const g=rl(t)?Ha(fs(t),l):l;return na(t,{ref:g})},Lo=e=>{const t=se.useContext(Qi);return se.useMemo(()=>e?typeof e=="string"?e??t:typeof e=="function"?e(t):t:t,[e,t])},vz=e=>{const{componentCls:t}=e;return{[t]:{"&-block":{display:"flex",width:"100%"},"&-vertical":{flexDirection:"column"}}}},yz=e=>{const{componentCls:t,antCls:n}=e;return{[t]:{display:"inline-flex","&-rtl":{direction:"rtl"},"&-vertical":{flexDirection:"column"},"&-align":{flexDirection:"column","&-center":{alignItems:"center"},"&-start":{alignItems:"flex-start"},"&-end":{alignItems:"flex-end"},"&-baseline":{alignItems:"baseline"}},[`${t}-item:empty`]:{display:"none"},[`${t}-item > ${n}-badge-not-a-wrapper:only-child`]:{display:"block"}}}},bz=e=>{const{componentCls:t}=e;return{[t]:{"&-gap-row-small":{rowGap:e.spaceGapSmallSize},"&-gap-row-middle":{rowGap:e.spaceGapMiddleSize},"&-gap-row-large":{rowGap:e.spaceGapLargeSize},"&-gap-col-small":{columnGap:e.spaceGapSmallSize},"&-gap-col-middle":{columnGap:e.spaceGapMiddleSize},"&-gap-col-large":{columnGap:e.spaceGapLargeSize}}}},Sz=xn("Space",e=>{const t=Kt(e,{spaceGapSmallSize:e.paddingXS,spaceGapMiddleSize:e.padding,spaceGapLargeSize:e.paddingLG});return[yz(t),bz(t),vz(t)]},()=>({}),{resetStyle:!1});var iR=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const ym=f.createContext(null),Ru=(e,t)=>{const n=f.useContext(ym),r=f.useMemo(()=>{if(!n)return"";const{compactDirection:a,isFirstItem:l,isLastItem:s}=n,u=a==="vertical"?"-vertical-":"-";return re(`${e}-compact${u}item`,{[`${e}-compact${u}first-item`]:l,[`${e}-compact${u}last-item`]:s,[`${e}-compact${u}item-rtl`]:t==="rtl"})},[e,t,n]);return{compactSize:n==null?void 0:n.compactSize,compactDirection:n==null?void 0:n.compactDirection,compactItemClassnames:r}},Cz=e=>{const{children:t}=e;return f.createElement(ym.Provider,{value:null},t)},xz=e=>{const{children:t}=e,n=iR(e,["children"]);return f.createElement(ym.Provider,{value:f.useMemo(()=>n,[n])},t)},gX=e=>{const{getPrefixCls:t,direction:n}=f.useContext(dt),{size:r,direction:a,block:l,prefixCls:s,className:u,rootClassName:d,children:g}=e,h=iR(e,["size","direction","block","prefixCls","className","rootClassName","children"]),p=Lo(O=>r??O),y=t("space-compact",s),[C,b]=Sz(y),x=re(y,b,{[`${y}-rtl`]:n==="rtl",[`${y}-block`]:l,[`${y}-vertical`]:a==="vertical"},u,d),S=f.useContext(ym),E=Lr(g),$=f.useMemo(()=>E.map((O,_)=>{const R=(O==null?void 0:O.key)||`${y}-item-${_}`;return f.createElement(xz,{key:R,compactSize:p,compactDirection:a,isFirstItem:_===0&&(!S||(S==null?void 0:S.isFirstItem)),isLastItem:_===E.length-1&&(!S||(S==null?void 0:S.isLastItem))},O)}),[r,E,S]);return E.length===0?null:C(f.createElement("div",Object.assign({className:x},h),$))};var wz=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const lR=f.createContext(void 0),Ez=e=>{const{getPrefixCls:t,direction:n}=f.useContext(dt),{prefixCls:r,size:a,className:l}=e,s=wz(e,["prefixCls","size","className"]),u=t("btn-group",r),[,,d]=Br(),g=f.useMemo(()=>{switch(a){case"large":return"lg";case"small":return"sm";default:return""}},[a]),h=re(u,{[`${u}-${g}`]:g,[`${u}-rtl`]:n==="rtl"},l,d);return f.createElement(lR.Provider,{value:a},f.createElement("div",Object.assign({},s,{className:h})))},Hx=/^[\u4E00-\u9FA5]{2}$/,e0=Hx.test.bind(Hx);function uy(e){return e==="danger"?{danger:!0}:{type:e}}function Fx(e){return typeof e=="string"}function Ep(e){return e==="text"||e==="link"}function $z(e,t){if(e==null)return;const n=t?" ":"";return typeof e!="string"&&typeof e!="number"&&Fx(e.type)&&e0(e.props.children)?na(e,{children:e.props.children.split("").join(n)}):Fx(e)?e0(e)?se.createElement("span",null,e.split("").join(n)):se.createElement("span",null,e):X2(e)?se.createElement("span",null,e):e}function Oz(e,t){let n=!1;const r=[];return se.Children.forEach(e,a=>{const l=typeof a,s=l==="string"||l==="number";if(n&&s){const u=r.length-1,d=r[u];r[u]=`${d}${a}`}else r.push(a);n=s}),se.Children.map(r,a=>$z(a,t))}["default","primary","danger"].concat(Ne(vi));const t0=f.forwardRef((e,t)=>{const{className:n,style:r,children:a,prefixCls:l}=e,s=re(`${l}-icon`,n);return se.createElement("span",{ref:t,className:s,style:r},a)}),Vx=f.forwardRef((e,t)=>{const{prefixCls:n,className:r,style:a,iconClassName:l}=e,s=re(`${n}-loading-icon`,r);return se.createElement(t0,{prefixCls:n,className:s,style:a,ref:t},se.createElement(Cu,{className:l}))}),$p=()=>({width:0,opacity:0,transform:"scale(0)"}),Op=e=>({width:e.scrollWidth,opacity:1,transform:"scale(1)"}),Rz=e=>{const{prefixCls:t,loading:n,existIcon:r,className:a,style:l,mount:s}=e,u=!!n;return r?se.createElement(Vx,{prefixCls:t,className:a,style:l}):se.createElement($a,{visible:u,motionName:`${t}-loading-icon-motion`,motionAppear:!s,motionEnter:!s,motionLeave:!s,removeOnLeave:!0,onAppearStart:$p,onAppearActive:Op,onEnterStart:$p,onEnterActive:Op,onLeaveStart:Op,onLeaveActive:$p},(d,g)=>{let{className:h,style:p}=d;const y=Object.assign(Object.assign({},l),p);return se.createElement(Vx,{prefixCls:t,className:re(a,h),style:y,ref:g})})},Ux=(e,t)=>({[`> span, > ${e}`]:{"&:not(:last-child)":{[`&, & > ${e}`]:{"&:not(:disabled)":{borderInlineEndColor:t}}},"&:not(:first-child)":{[`&, & > ${e}`]:{"&:not(:disabled)":{borderInlineStartColor:t}}}}}),_z=e=>{const{componentCls:t,fontSize:n,lineWidth:r,groupBorderColor:a,colorErrorHover:l}=e;return{[`${t}-group`]:[{position:"relative",display:"inline-flex",[`> span, > ${t}`]:{"&:not(:last-child)":{[`&, & > ${t}`]:{borderStartEndRadius:0,borderEndEndRadius:0}},"&:not(:first-child)":{marginInlineStart:e.calc(r).mul(-1).equal(),[`&, & > ${t}`]:{borderStartStartRadius:0,borderEndStartRadius:0}}},[t]:{position:"relative",zIndex:1,"&:hover, &:focus, &:active":{zIndex:2},"&[disabled]":{zIndex:0}},[`${t}-icon-only`]:{fontSize:n}},Ux(`${t}-primary`,a),Ux(`${t}-danger`,l)]}};var Tz=["b"],Mz=["v"],Rp=function(t){return Math.round(Number(t||0))},Nz=function(t){if(t instanceof en)return t;if(t&&st(t)==="object"&&"h"in t&&"b"in t){var n=t,r=n.b,a=$t(n,Tz);return Q(Q({},a),{},{v:r})}return typeof t=="string"&&/hsb/.test(t)?t.replace(/hsb/,"hsv"):t},lu=function(e){Po(n,e);var t=Ao(n);function n(r){return Wn(this,n),t.call(this,Nz(r))}return Gn(n,[{key:"toHsbString",value:function(){var a=this.toHsb(),l=Rp(a.s*100),s=Rp(a.b*100),u=Rp(a.h),d=a.a,g="hsb(".concat(u,", ").concat(l,"%, ").concat(s,"%)"),h="hsba(".concat(u,", ").concat(l,"%, ").concat(s,"%, ").concat(d.toFixed(d===0?0:2),")");return d===1?g:h}},{key:"toHsb",value:function(){var a=this.toHsv(),l=a.v,s=$t(a,Mz);return Q(Q({},s),{},{b:l,a:this.a})}}]),n}(en),jz=function(t){return t instanceof lu?t:new lu(t)};jz("#1677ff");const Iz=(e,t)=>(e==null?void 0:e.replace(/[^\w/]/g,"").slice(0,t?8:6))||"",Pz=(e,t)=>e?Iz(e,t):"";let Az=function(){function e(t){Wn(this,e);var n;if(this.cleared=!1,t instanceof e){this.metaColor=t.metaColor.clone(),this.colors=(n=t.colors)===null||n===void 0?void 0:n.map(a=>({color:new e(a.color),percent:a.percent})),this.cleared=t.cleared;return}const r=Array.isArray(t);r&&t.length?(this.colors=t.map(a=>{let{color:l,percent:s}=a;return{color:new e(l),percent:s}}),this.metaColor=new lu(this.colors[0].color.metaColor)):this.metaColor=new lu(r?"":t),(!t||r&&!this.colors)&&(this.metaColor=this.metaColor.setA(0),this.cleared=!0)}return Gn(e,[{key:"toHsb",value:function(){return this.metaColor.toHsb()}},{key:"toHsbString",value:function(){return this.metaColor.toHsbString()}},{key:"toHex",value:function(){return Pz(this.toHexString(),this.metaColor.a<1)}},{key:"toHexString",value:function(){return this.metaColor.toHexString()}},{key:"toRgb",value:function(){return this.metaColor.toRgb()}},{key:"toRgbString",value:function(){return this.metaColor.toRgbString()}},{key:"isGradient",value:function(){return!!this.colors&&!this.cleared}},{key:"getColors",value:function(){return this.colors||[{color:this,percent:0}]}},{key:"toCssString",value:function(){const{colors:n}=this;return n?`linear-gradient(90deg, ${n.map(a=>`${a.color.toRgbString()} ${a.percent}%`).join(", ")})`:this.metaColor.toRgbString()}},{key:"equals",value:function(n){return!n||this.isGradient()!==n.isGradient()?!1:this.isGradient()?this.colors.length===n.colors.length&&this.colors.every((r,a)=>{const l=n.colors[a];return r.percent===l.percent&&r.color.equals(l.color)}):this.toHexString()===n.toHexString()}}])}();var sR=se.forwardRef(function(e,t){var n=e.prefixCls,r=e.forceRender,a=e.className,l=e.style,s=e.children,u=e.isActive,d=e.role,g=e.classNames,h=e.styles,p=se.useState(u||r),y=ge(p,2),C=y[0],b=y[1];return se.useEffect(function(){(r||u)&&b(!0)},[r,u]),C?se.createElement("div",{ref:t,className:re("".concat(n,"-content"),Z(Z({},"".concat(n,"-content-active"),u),"".concat(n,"-content-inactive"),!u),a),style:l,role:d},se.createElement("div",{className:re("".concat(n,"-content-box"),g==null?void 0:g.body),style:h==null?void 0:h.body},s)):null});sR.displayName="PanelContent";var zz=["showArrow","headerClass","isActive","onItemClick","forceRender","className","classNames","styles","prefixCls","collapsible","accordion","panelKey","extra","header","expandIcon","openMotion","destroyInactivePanel","children"],cR=se.forwardRef(function(e,t){var n=e.showArrow,r=n===void 0?!0:n,a=e.headerClass,l=e.isActive,s=e.onItemClick,u=e.forceRender,d=e.className,g=e.classNames,h=g===void 0?{}:g,p=e.styles,y=p===void 0?{}:p,C=e.prefixCls,b=e.collapsible,x=e.accordion,S=e.panelKey,E=e.extra,$=e.header,O=e.expandIcon,_=e.openMotion,R=e.destroyInactivePanel,T=e.children,M=$t(e,zz),z=b==="disabled",P=E!=null&&typeof E!="boolean",D=Z(Z(Z({onClick:function(){s==null||s(S)},onKeyDown:function(A){(A.key==="Enter"||A.keyCode===lt.ENTER||A.which===lt.ENTER)&&(s==null||s(S))},role:x?"tab":"button"},"aria-expanded",l),"aria-disabled",z),"tabIndex",z?-1:0),F=typeof O=="function"?O(e):se.createElement("i",{className:"arrow"}),G=F&&se.createElement("div",je({className:"".concat(C,"-expand-icon")},["header","icon"].includes(b)?D:{}),F),V=re("".concat(C,"-item"),Z(Z({},"".concat(C,"-item-active"),l),"".concat(C,"-item-disabled"),z),d),U=re(a,"".concat(C,"-header"),Z({},"".concat(C,"-collapsible-").concat(b),!!b),h.header),K=Q({className:U,style:y.header},["header","icon"].includes(b)?{}:D);return se.createElement("div",je({},M,{ref:t,className:V}),se.createElement("div",K,r&&G,se.createElement("span",je({className:"".concat(C,"-header-text")},b==="header"?D:{}),$),P&&se.createElement("div",{className:"".concat(C,"-extra")},E)),se.createElement($a,je({visible:l,leavedClassName:"".concat(C,"-content-hidden")},_,{forceRender:u,removeOnLeave:R}),function(j,A){var L=j.className,q=j.style;return se.createElement(sR,{ref:A,prefixCls:C,className:L,classNames:h,style:q,styles:y,isActive:l,forceRender:u,role:x?"tabpanel":void 0},T)}))}),Dz=["children","label","key","collapsible","onItemClick","destroyInactivePanel"],Lz=function(t,n){var r=n.prefixCls,a=n.accordion,l=n.collapsible,s=n.destroyInactivePanel,u=n.onItemClick,d=n.activeKey,g=n.openMotion,h=n.expandIcon;return t.map(function(p,y){var C=p.children,b=p.label,x=p.key,S=p.collapsible,E=p.onItemClick,$=p.destroyInactivePanel,O=$t(p,Dz),_=String(x??y),R=S??l,T=$??s,M=function(D){R!=="disabled"&&(u(D),E==null||E(D))},z=!1;return a?z=d[0]===_:z=d.indexOf(_)>-1,se.createElement(cR,je({},O,{prefixCls:r,key:_,panelKey:_,isActive:z,accordion:a,openMotion:g,expandIcon:h,header:b,collapsible:R,onItemClick:M,destroyInactivePanel:T}),C)})},Bz=function(t,n,r){if(!t)return null;var a=r.prefixCls,l=r.accordion,s=r.collapsible,u=r.destroyInactivePanel,d=r.onItemClick,g=r.activeKey,h=r.openMotion,p=r.expandIcon,y=t.key||String(n),C=t.props,b=C.header,x=C.headerClass,S=C.destroyInactivePanel,E=C.collapsible,$=C.onItemClick,O=!1;l?O=g[0]===y:O=g.indexOf(y)>-1;var _=E??s,R=function(z){_!=="disabled"&&(d(z),$==null||$(z))},T={key:y,panelKey:y,header:b,headerClass:x,isActive:O,prefixCls:a,destroyInactivePanel:S??u,openMotion:h,accordion:l,children:t.props.children,onItemClick:R,expandIcon:p,collapsible:_};return typeof t.type=="string"?t:(Object.keys(T).forEach(function(M){typeof T[M]>"u"&&delete T[M]}),se.cloneElement(t,T))};function Hz(e,t,n){return Array.isArray(e)?Lz(e,n):Lr(t).map(function(r,a){return Bz(r,a,n)})}function Fz(e){var t=e;if(!Array.isArray(t)){var n=st(t);t=n==="number"||n==="string"?[t]:[]}return t.map(function(r){return String(r)})}var Vz=se.forwardRef(function(e,t){var n=e.prefixCls,r=n===void 0?"rc-collapse":n,a=e.destroyInactivePanel,l=a===void 0?!1:a,s=e.style,u=e.accordion,d=e.className,g=e.children,h=e.collapsible,p=e.openMotion,y=e.expandIcon,C=e.activeKey,b=e.defaultActiveKey,x=e.onChange,S=e.items,E=re(r,d),$=br([],{value:C,onChange:function(P){return x==null?void 0:x(P)},defaultValue:b,postState:Fz}),O=ge($,2),_=O[0],R=O[1],T=function(P){return R(function(){if(u)return _[0]===P?[]:[P];var D=_.indexOf(P),F=D>-1;return F?_.filter(function(G){return G!==P}):[].concat(Ne(_),[P])})};mr(!g,"[rc-collapse] `children` will be removed in next major version. Please use `items` instead.");var M=Hz(S,g,{prefixCls:r,accordion:u,openMotion:p,expandIcon:y,collapsible:h,destroyInactivePanel:l,onItemClick:T,activeKey:_});return se.createElement("div",je({ref:t,className:E,style:s,role:u?"tablist":void 0},ta(e,{aria:!0,data:!0})),M)});const fy=Object.assign(Vz,{Panel:cR});fy.Panel;const Uz=f.forwardRef((e,t)=>{const{getPrefixCls:n}=f.useContext(dt),{prefixCls:r,className:a,showArrow:l=!0}=e,s=n("collapse",r),u=re({[`${s}-no-arrow`]:!l},a);return f.createElement(fy.Panel,Object.assign({ref:t},e,{prefixCls:s,className:u}))}),dy=e=>({[e.componentCls]:{[`${e.antCls}-motion-collapse-legacy`]:{overflow:"hidden","&-active":{transition:`height ${e.motionDurationMid} ${e.motionEaseInOut},
        opacity ${e.motionDurationMid} ${e.motionEaseInOut} !important`}},[`${e.antCls}-motion-collapse`]:{overflow:"hidden",transition:`height ${e.motionDurationMid} ${e.motionEaseInOut},
        opacity ${e.motionDurationMid} ${e.motionEaseInOut} !important`}}}),kz=e=>({animationDuration:e,animationFillMode:"both"}),qz=e=>({animationDuration:e,animationFillMode:"both"}),bm=function(e,t,n,r){const l=(arguments.length>4&&arguments[4]!==void 0?arguments[4]:!1)?"&":"";return{[`
      ${l}${e}-enter,
      ${l}${e}-appear
    `]:Object.assign(Object.assign({},kz(r)),{animationPlayState:"paused"}),[`${l}${e}-leave`]:Object.assign(Object.assign({},qz(r)),{animationPlayState:"paused"}),[`
      ${l}${e}-enter${e}-enter-active,
      ${l}${e}-appear${e}-appear-active
    `]:{animationName:t,animationPlayState:"running"},[`${l}${e}-leave${e}-leave-active`]:{animationName:n,animationPlayState:"running",pointerEvents:"none"}}},Wz=new Dt("antFadeIn",{"0%":{opacity:0},"100%":{opacity:1}}),Gz=new Dt("antFadeOut",{"0%":{opacity:1},"100%":{opacity:0}}),Kz=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;const{antCls:n}=e,r=`${n}-fade`,a=t?"&":"";return[bm(r,Wz,Gz,e.motionDurationMid,t),{[`
        ${a}${r}-enter,
        ${a}${r}-appear
      `]:{opacity:0,animationTimingFunction:"linear"},[`${a}${r}-leave`]:{animationTimingFunction:"linear"}}]},Xz=new Dt("antMoveDownIn",{"0%":{transform:"translate3d(0, 100%, 0)",transformOrigin:"0 0",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1}}),Yz=new Dt("antMoveDownOut",{"0%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1},"100%":{transform:"translate3d(0, 100%, 0)",transformOrigin:"0 0",opacity:0}}),Qz=new Dt("antMoveLeftIn",{"0%":{transform:"translate3d(-100%, 0, 0)",transformOrigin:"0 0",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1}}),Zz=new Dt("antMoveLeftOut",{"0%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1},"100%":{transform:"translate3d(-100%, 0, 0)",transformOrigin:"0 0",opacity:0}}),Jz=new Dt("antMoveRightIn",{"0%":{transform:"translate3d(100%, 0, 0)",transformOrigin:"0 0",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1}}),e5=new Dt("antMoveRightOut",{"0%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1},"100%":{transform:"translate3d(100%, 0, 0)",transformOrigin:"0 0",opacity:0}}),t5=new Dt("antMoveUpIn",{"0%":{transform:"translate3d(0, -100%, 0)",transformOrigin:"0 0",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1}}),n5=new Dt("antMoveUpOut",{"0%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1},"100%":{transform:"translate3d(0, -100%, 0)",transformOrigin:"0 0",opacity:0}}),r5={"move-up":{inKeyframes:t5,outKeyframes:n5},"move-down":{inKeyframes:Xz,outKeyframes:Yz},"move-left":{inKeyframes:Qz,outKeyframes:Zz},"move-right":{inKeyframes:Jz,outKeyframes:e5}},kx=(e,t)=>{const{antCls:n}=e,r=`${n}-${t}`,{inKeyframes:a,outKeyframes:l}=r5[t];return[bm(r,a,l,e.motionDurationMid),{[`
        ${r}-enter,
        ${r}-appear
      `]:{opacity:0,animationTimingFunction:e.motionEaseOutCirc},[`${r}-leave`]:{animationTimingFunction:e.motionEaseInOutCirc}}]},uR=new Dt("antSlideUpIn",{"0%":{transform:"scaleY(0.8)",transformOrigin:"0% 0%",opacity:0},"100%":{transform:"scaleY(1)",transformOrigin:"0% 0%",opacity:1}}),fR=new Dt("antSlideUpOut",{"0%":{transform:"scaleY(1)",transformOrigin:"0% 0%",opacity:1},"100%":{transform:"scaleY(0.8)",transformOrigin:"0% 0%",opacity:0}}),dR=new Dt("antSlideDownIn",{"0%":{transform:"scaleY(0.8)",transformOrigin:"100% 100%",opacity:0},"100%":{transform:"scaleY(1)",transformOrigin:"100% 100%",opacity:1}}),mR=new Dt("antSlideDownOut",{"0%":{transform:"scaleY(1)",transformOrigin:"100% 100%",opacity:1},"100%":{transform:"scaleY(0.8)",transformOrigin:"100% 100%",opacity:0}}),a5=new Dt("antSlideLeftIn",{"0%":{transform:"scaleX(0.8)",transformOrigin:"0% 0%",opacity:0},"100%":{transform:"scaleX(1)",transformOrigin:"0% 0%",opacity:1}}),o5=new Dt("antSlideLeftOut",{"0%":{transform:"scaleX(1)",transformOrigin:"0% 0%",opacity:1},"100%":{transform:"scaleX(0.8)",transformOrigin:"0% 0%",opacity:0}}),i5=new Dt("antSlideRightIn",{"0%":{transform:"scaleX(0.8)",transformOrigin:"100% 0%",opacity:0},"100%":{transform:"scaleX(1)",transformOrigin:"100% 0%",opacity:1}}),l5=new Dt("antSlideRightOut",{"0%":{transform:"scaleX(1)",transformOrigin:"100% 0%",opacity:1},"100%":{transform:"scaleX(0.8)",transformOrigin:"100% 0%",opacity:0}}),s5={"slide-up":{inKeyframes:uR,outKeyframes:fR},"slide-down":{inKeyframes:dR,outKeyframes:mR},"slide-left":{inKeyframes:a5,outKeyframes:o5},"slide-right":{inKeyframes:i5,outKeyframes:l5}},Bd=(e,t)=>{const{antCls:n}=e,r=`${n}-${t}`,{inKeyframes:a,outKeyframes:l}=s5[t];return[bm(r,a,l,e.motionDurationMid),{[`
      ${r}-enter,
      ${r}-appear
    `]:{transform:"scale(0)",transformOrigin:"0% 0%",opacity:0,animationTimingFunction:e.motionEaseOutQuint,"&-prepare":{transform:"scale(1)"}},[`${r}-leave`]:{animationTimingFunction:e.motionEaseInQuint}}]},my=new Dt("antZoomIn",{"0%":{transform:"scale(0.2)",opacity:0},"100%":{transform:"scale(1)",opacity:1}}),c5=new Dt("antZoomOut",{"0%":{transform:"scale(1)"},"100%":{transform:"scale(0.2)",opacity:0}}),qx=new Dt("antZoomBigIn",{"0%":{transform:"scale(0.8)",opacity:0},"100%":{transform:"scale(1)",opacity:1}}),Wx=new Dt("antZoomBigOut",{"0%":{transform:"scale(1)"},"100%":{transform:"scale(0.8)",opacity:0}}),u5=new Dt("antZoomUpIn",{"0%":{transform:"scale(0.8)",transformOrigin:"50% 0%",opacity:0},"100%":{transform:"scale(1)",transformOrigin:"50% 0%"}}),f5=new Dt("antZoomUpOut",{"0%":{transform:"scale(1)",transformOrigin:"50% 0%"},"100%":{transform:"scale(0.8)",transformOrigin:"50% 0%",opacity:0}}),d5=new Dt("antZoomLeftIn",{"0%":{transform:"scale(0.8)",transformOrigin:"0% 50%",opacity:0},"100%":{transform:"scale(1)",transformOrigin:"0% 50%"}}),m5=new Dt("antZoomLeftOut",{"0%":{transform:"scale(1)",transformOrigin:"0% 50%"},"100%":{transform:"scale(0.8)",transformOrigin:"0% 50%",opacity:0}}),g5=new Dt("antZoomRightIn",{"0%":{transform:"scale(0.8)",transformOrigin:"100% 50%",opacity:0},"100%":{transform:"scale(1)",transformOrigin:"100% 50%"}}),h5=new Dt("antZoomRightOut",{"0%":{transform:"scale(1)",transformOrigin:"100% 50%"},"100%":{transform:"scale(0.8)",transformOrigin:"100% 50%",opacity:0}}),p5=new Dt("antZoomDownIn",{"0%":{transform:"scale(0.8)",transformOrigin:"50% 100%",opacity:0},"100%":{transform:"scale(1)",transformOrigin:"50% 100%"}}),v5=new Dt("antZoomDownOut",{"0%":{transform:"scale(1)",transformOrigin:"50% 100%"},"100%":{transform:"scale(0.8)",transformOrigin:"50% 100%",opacity:0}}),y5={zoom:{inKeyframes:my,outKeyframes:c5},"zoom-big":{inKeyframes:qx,outKeyframes:Wx},"zoom-big-fast":{inKeyframes:qx,outKeyframes:Wx},"zoom-left":{inKeyframes:d5,outKeyframes:m5},"zoom-right":{inKeyframes:g5,outKeyframes:h5},"zoom-up":{inKeyframes:u5,outKeyframes:f5},"zoom-down":{inKeyframes:p5,outKeyframes:v5}},Sm=(e,t)=>{const{antCls:n}=e,r=`${n}-${t}`,{inKeyframes:a,outKeyframes:l}=y5[t];return[bm(r,a,l,t==="zoom-big-fast"?e.motionDurationFast:e.motionDurationMid),{[`
        ${r}-enter,
        ${r}-appear
      `]:{transform:"scale(0)",opacity:0,animationTimingFunction:e.motionEaseOutCirc,"&-prepare":{transform:"none"}},[`${r}-leave`]:{animationTimingFunction:e.motionEaseInOutCirc}}]},b5=e=>{const{componentCls:t,contentBg:n,padding:r,headerBg:a,headerPadding:l,collapseHeaderPaddingSM:s,collapseHeaderPaddingLG:u,collapsePanelBorderRadius:d,lineWidth:g,lineType:h,colorBorder:p,colorText:y,colorTextHeading:C,colorTextDisabled:b,fontSizeLG:x,lineHeight:S,lineHeightLG:E,marginSM:$,paddingSM:O,paddingLG:_,paddingXS:R,motionDurationSlow:T,fontSizeIcon:M,contentPadding:z,fontHeight:P,fontHeightLG:D}=e,F=`${ve(g)} ${h} ${p}`;return{[t]:Object.assign(Object.assign({},Rr(e)),{backgroundColor:a,border:F,borderRadius:d,"&-rtl":{direction:"rtl"},[`& > ${t}-item`]:{borderBottom:F,"&:first-child":{[`
            &,
            & > ${t}-header`]:{borderRadius:`${ve(d)} ${ve(d)} 0 0`}},"&:last-child":{[`
            &,
            & > ${t}-header`]:{borderRadius:`0 0 ${ve(d)} ${ve(d)}`}},[`> ${t}-header`]:Object.assign(Object.assign({position:"relative",display:"flex",flexWrap:"nowrap",alignItems:"flex-start",padding:l,color:C,lineHeight:S,cursor:"pointer",transition:`all ${T}, visibility 0s`},gs(e)),{[`> ${t}-header-text`]:{flex:"auto"},[`${t}-expand-icon`]:{height:P,display:"flex",alignItems:"center",paddingInlineEnd:$},[`${t}-arrow`]:Object.assign(Object.assign({},ms()),{fontSize:M,transition:`transform ${T}`,svg:{transition:`transform ${T}`}}),[`${t}-header-text`]:{marginInlineEnd:"auto"}}),[`${t}-collapsible-header`]:{cursor:"default",[`${t}-header-text`]:{flex:"none",cursor:"pointer"}},[`${t}-collapsible-icon`]:{cursor:"unset",[`${t}-expand-icon`]:{cursor:"pointer"}}},[`${t}-content`]:{color:y,backgroundColor:n,borderTop:F,[`& > ${t}-content-box`]:{padding:z},"&-hidden":{display:"none"}},"&-small":{[`> ${t}-item`]:{[`> ${t}-header`]:{padding:s,paddingInlineStart:R,[`> ${t}-expand-icon`]:{marginInlineStart:e.calc(O).sub(R).equal()}},[`> ${t}-content > ${t}-content-box`]:{padding:O}}},"&-large":{[`> ${t}-item`]:{fontSize:x,lineHeight:E,[`> ${t}-header`]:{padding:u,paddingInlineStart:r,[`> ${t}-expand-icon`]:{height:D,marginInlineStart:e.calc(_).sub(r).equal()}},[`> ${t}-content > ${t}-content-box`]:{padding:_}}},[`${t}-item:last-child`]:{borderBottom:0,[`> ${t}-content`]:{borderRadius:`0 0 ${ve(d)} ${ve(d)}`}},[`& ${t}-item-disabled > ${t}-header`]:{"\n          &,\n          & > .arrow\n        ":{color:b,cursor:"not-allowed"}},[`&${t}-icon-position-end`]:{[`& > ${t}-item`]:{[`> ${t}-header`]:{[`${t}-expand-icon`]:{order:1,paddingInlineEnd:0,paddingInlineStart:$}}}}})}},S5=e=>{const{componentCls:t}=e,n=`> ${t}-item > ${t}-header ${t}-arrow`;return{[`${t}-rtl`]:{[n]:{transform:"rotate(180deg)"}}}},C5=e=>{const{componentCls:t,headerBg:n,paddingXXS:r,colorBorder:a}=e;return{[`${t}-borderless`]:{backgroundColor:n,border:0,[`> ${t}-item`]:{borderBottom:`1px solid ${a}`},[`
        > ${t}-item:last-child,
        > ${t}-item:last-child ${t}-header
      `]:{borderRadius:0},[`> ${t}-item:last-child`]:{borderBottom:0},[`> ${t}-item > ${t}-content`]:{backgroundColor:"transparent",borderTop:0},[`> ${t}-item > ${t}-content > ${t}-content-box`]:{paddingTop:r}}}},x5=e=>{const{componentCls:t,paddingSM:n}=e;return{[`${t}-ghost`]:{backgroundColor:"transparent",border:0,[`> ${t}-item`]:{borderBottom:0,[`> ${t}-content`]:{backgroundColor:"transparent",border:0,[`> ${t}-content-box`]:{paddingBlock:n}}}}}},w5=e=>({headerPadding:`${e.paddingSM}px ${e.padding}px`,headerBg:e.colorFillAlter,contentPadding:`${e.padding}px 16px`,contentBg:e.colorBgContainer}),E5=xn("Collapse",e=>{const t=Kt(e,{collapseHeaderPaddingSM:`${ve(e.paddingXS)} ${ve(e.paddingSM)}`,collapseHeaderPaddingLG:`${ve(e.padding)} ${ve(e.paddingLG)}`,collapsePanelBorderRadius:e.borderRadiusLG});return[b5(t),C5(t),x5(t),S5(t),dy(t)]},w5),$5=f.forwardRef((e,t)=>{const{getPrefixCls:n,direction:r,expandIcon:a,className:l,style:s}=Or("collapse"),{prefixCls:u,className:d,rootClassName:g,style:h,bordered:p=!0,ghost:y,size:C,expandIconPosition:b="start",children:x,expandIcon:S}=e,E=Lo(V=>{var U;return(U=C??V)!==null&&U!==void 0?U:"middle"}),$=n("collapse",u),O=n(),[_,R,T]=E5($),M=f.useMemo(()=>b==="left"?"start":b==="right"?"end":b,[b]),z=S??a,P=f.useCallback(function(){let V=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};const U=typeof z=="function"?z(V):f.createElement(Dv,{rotate:V.isActive?r==="rtl"?-90:90:void 0,"aria-label":V.isActive?"expanded":"collapsed"});return na(U,()=>{var K;return{className:re((K=U==null?void 0:U.props)===null||K===void 0?void 0:K.className,`${$}-arrow`)}})},[z,$]),D=re(`${$}-icon-position-${M}`,{[`${$}-borderless`]:!p,[`${$}-rtl`]:r==="rtl",[`${$}-ghost`]:!!y,[`${$}-${E}`]:E!=="middle"},l,d,g,R,T),F=Object.assign(Object.assign({},Ld(O)),{motionAppear:!1,leavedClassName:`${$}-content-hidden`}),G=f.useMemo(()=>x?Lr(x).map((V,U)=>{var K,j;const A=V.props;if(A!=null&&A.disabled){const L=(K=V.key)!==null&&K!==void 0?K:String(U),q=Object.assign(Object.assign({},On(V.props,["disabled"])),{key:L,collapsible:(j=A.collapsible)!==null&&j!==void 0?j:"disabled"});return na(V,q)}return V}):null,[x]);return _(f.createElement(fy,Object.assign({ref:t,openMotion:F},On(e,["rootClassName"]),{expandIcon:P,prefixCls:$,className:D,style:Object.assign(Object.assign({},s),h)}),G))}),O5=Object.assign($5,{Panel:Uz}),R5=(e,t)=>{const{r:n,g:r,b:a,a:l}=e.toRgb(),s=new lu(e.toRgbString()).onBackground(t).toHsv();return l<=.5?s.v>.5:n*.299+r*.587+a*.114>192},gR=e=>{const{paddingInline:t,onlyIconSize:n}=e;return Kt(e,{buttonPaddingHorizontal:t,buttonPaddingVertical:0,buttonIconOnlyFontSize:n})},hR=e=>{var t,n,r,a,l,s;const u=(t=e.contentFontSize)!==null&&t!==void 0?t:e.fontSize,d=(n=e.contentFontSizeSM)!==null&&n!==void 0?n:e.fontSize,g=(r=e.contentFontSizeLG)!==null&&r!==void 0?r:e.fontSizeLG,h=(a=e.contentLineHeight)!==null&&a!==void 0?a:pd(u),p=(l=e.contentLineHeightSM)!==null&&l!==void 0?l:pd(d),y=(s=e.contentLineHeightLG)!==null&&s!==void 0?s:pd(g),C=R5(new Az(e.colorBgSolid),"#fff")?"#000":"#fff",b=vi.reduce((x,S)=>Object.assign(Object.assign({},x),{[`${S}ShadowColor`]:`0 ${ve(e.controlOutlineWidth)} 0 ${Vc(e[`${S}1`],e.colorBgContainer)}`}),{});return Object.assign(Object.assign({},b),{fontWeight:400,defaultShadow:`0 ${e.controlOutlineWidth}px 0 ${e.controlTmpOutline}`,primaryShadow:`0 ${e.controlOutlineWidth}px 0 ${e.controlOutline}`,dangerShadow:`0 ${e.controlOutlineWidth}px 0 ${e.colorErrorOutline}`,primaryColor:e.colorTextLightSolid,dangerColor:e.colorTextLightSolid,borderColorDisabled:e.colorBorder,defaultGhostColor:e.colorBgContainer,ghostBg:"transparent",defaultGhostBorderColor:e.colorBgContainer,paddingInline:e.paddingContentHorizontal-e.lineWidth,paddingInlineLG:e.paddingContentHorizontal-e.lineWidth,paddingInlineSM:8-e.lineWidth,onlyIconSize:"inherit",onlyIconSizeSM:"inherit",onlyIconSizeLG:"inherit",groupBorderColor:e.colorPrimaryHover,linkHoverBg:"transparent",textTextColor:e.colorText,textTextHoverColor:e.colorText,textTextActiveColor:e.colorText,textHoverBg:e.colorFillTertiary,defaultColor:e.colorText,defaultBg:e.colorBgContainer,defaultBorderColor:e.colorBorder,defaultBorderColorDisabled:e.colorBorder,defaultHoverBg:e.colorBgContainer,defaultHoverColor:e.colorPrimaryHover,defaultHoverBorderColor:e.colorPrimaryHover,defaultActiveBg:e.colorBgContainer,defaultActiveColor:e.colorPrimaryActive,defaultActiveBorderColor:e.colorPrimaryActive,solidTextColor:C,contentFontSize:u,contentFontSizeSM:d,contentFontSizeLG:g,contentLineHeight:h,contentLineHeightSM:p,contentLineHeightLG:y,paddingBlock:Math.max((e.controlHeight-u*h)/2-e.lineWidth,0),paddingBlockSM:Math.max((e.controlHeightSM-d*p)/2-e.lineWidth,0),paddingBlockLG:Math.max((e.controlHeightLG-g*y)/2-e.lineWidth,0)})},_5=e=>{const{componentCls:t,iconCls:n,fontWeight:r,opacityLoading:a,motionDurationSlow:l,motionEaseInOut:s,marginXS:u,calc:d}=e;return{[t]:{outline:"none",position:"relative",display:"inline-flex",gap:e.marginXS,alignItems:"center",justifyContent:"center",fontWeight:r,whiteSpace:"nowrap",textAlign:"center",backgroundImage:"none",background:"transparent",border:`${ve(e.lineWidth)} ${e.lineType} transparent`,cursor:"pointer",transition:`all ${e.motionDurationMid} ${e.motionEaseInOut}`,userSelect:"none",touchAction:"manipulation",color:e.colorText,"&:disabled > *":{pointerEvents:"none"},[`${t}-icon > svg`]:ms(),"> a":{color:"currentColor"},"&:not(:disabled)":gs(e),[`&${t}-two-chinese-chars::first-letter`]:{letterSpacing:"0.34em"},[`&${t}-two-chinese-chars > *:not(${n})`]:{marginInlineEnd:"-0.34em",letterSpacing:"0.34em"},[`&${t}-icon-only`]:{paddingInline:0,[`&${t}-compact-item`]:{flex:"none"},[`&${t}-round`]:{width:"auto"}},[`&${t}-loading`]:{opacity:a,cursor:"default"},[`${t}-loading-icon`]:{transition:["width","opacity","margin"].map(g=>`${g} ${l} ${s}`).join(",")},[`&:not(${t}-icon-end)`]:{[`${t}-loading-icon-motion`]:{"&-appear-start, &-enter-start":{marginInlineEnd:d(u).mul(-1).equal()},"&-appear-active, &-enter-active":{marginInlineEnd:0},"&-leave-start":{marginInlineEnd:0},"&-leave-active":{marginInlineEnd:d(u).mul(-1).equal()}}},"&-icon-end":{flexDirection:"row-reverse",[`${t}-loading-icon-motion`]:{"&-appear-start, &-enter-start":{marginInlineStart:d(u).mul(-1).equal()},"&-appear-active, &-enter-active":{marginInlineStart:0},"&-leave-start":{marginInlineStart:0},"&-leave-active":{marginInlineStart:d(u).mul(-1).equal()}}}}}},pR=(e,t,n)=>({[`&:not(:disabled):not(${e}-disabled)`]:{"&:hover":t,"&:active":n}}),T5=e=>({minWidth:e.controlHeight,paddingInlineStart:0,paddingInlineEnd:0,borderRadius:"50%"}),M5=e=>({borderRadius:e.controlHeight,paddingInlineStart:e.calc(e.controlHeight).div(2).equal(),paddingInlineEnd:e.calc(e.controlHeight).div(2).equal()}),N5=e=>({cursor:"not-allowed",borderColor:e.borderColorDisabled,color:e.colorTextDisabled,background:e.colorBgContainerDisabled,boxShadow:"none"}),Cm=(e,t,n,r,a,l,s,u)=>({[`&${e}-background-ghost`]:Object.assign(Object.assign({color:n||void 0,background:t,borderColor:r||void 0,boxShadow:"none"},pR(e,Object.assign({background:t},s),Object.assign({background:t},u))),{"&:disabled":{cursor:"not-allowed",color:a||void 0,borderColor:l||void 0}})}),j5=e=>({[`&:disabled, &${e.componentCls}-disabled`]:Object.assign({},N5(e))}),I5=e=>({[`&:disabled, &${e.componentCls}-disabled`]:{cursor:"not-allowed",color:e.colorTextDisabled}}),xm=(e,t,n,r)=>{const l=r&&["link","text"].includes(r)?I5:j5;return Object.assign(Object.assign({},l(e)),pR(e.componentCls,t,n))},wm=(e,t,n,r,a)=>({[`&${e.componentCls}-variant-solid`]:Object.assign({color:t,background:n},xm(e,r,a))}),Em=(e,t,n,r,a)=>({[`&${e.componentCls}-variant-outlined, &${e.componentCls}-variant-dashed`]:Object.assign({borderColor:t,background:n},xm(e,r,a))}),$m=e=>({[`&${e.componentCls}-variant-dashed`]:{borderStyle:"dashed"}}),Om=(e,t,n,r)=>({[`&${e.componentCls}-variant-filled`]:Object.assign({boxShadow:"none",background:t},xm(e,n,r))}),to=(e,t,n,r,a)=>({[`&${e.componentCls}-variant-${n}`]:Object.assign({color:t,boxShadow:"none"},xm(e,r,a,n))}),P5=e=>{const{componentCls:t}=e;return vi.reduce((n,r)=>{const a=e[`${r}6`],l=e[`${r}1`],s=e[`${r}5`],u=e[`${r}2`],d=e[`${r}3`],g=e[`${r}7`];return Object.assign(Object.assign({},n),{[`&${t}-color-${r}`]:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:a,boxShadow:e[`${r}ShadowColor`]},wm(e,e.colorTextLightSolid,a,{background:s},{background:g})),Em(e,a,e.colorBgContainer,{color:s,borderColor:s,background:e.colorBgContainer},{color:g,borderColor:g,background:e.colorBgContainer})),$m(e)),Om(e,l,{background:u},{background:d})),to(e,a,"link",{color:s},{color:g})),to(e,a,"text",{color:s,background:l},{color:g,background:d}))})},{})},A5=e=>Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:e.defaultColor,boxShadow:e.defaultShadow},wm(e,e.solidTextColor,e.colorBgSolid,{color:e.solidTextColor,background:e.colorBgSolidHover},{color:e.solidTextColor,background:e.colorBgSolidActive})),$m(e)),Om(e,e.colorFillTertiary,{background:e.colorFillSecondary},{background:e.colorFill})),Cm(e.componentCls,e.ghostBg,e.defaultGhostColor,e.defaultGhostBorderColor,e.colorTextDisabled,e.colorBorder)),to(e,e.textTextColor,"link",{color:e.colorLinkHover,background:e.linkHoverBg},{color:e.colorLinkActive})),z5=e=>Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:e.colorPrimary,boxShadow:e.primaryShadow},Em(e,e.colorPrimary,e.colorBgContainer,{color:e.colorPrimaryTextHover,borderColor:e.colorPrimaryHover,background:e.colorBgContainer},{color:e.colorPrimaryTextActive,borderColor:e.colorPrimaryActive,background:e.colorBgContainer})),$m(e)),Om(e,e.colorPrimaryBg,{background:e.colorPrimaryBgHover},{background:e.colorPrimaryBorder})),to(e,e.colorPrimaryText,"text",{color:e.colorPrimaryTextHover,background:e.colorPrimaryBg},{color:e.colorPrimaryTextActive,background:e.colorPrimaryBorder})),to(e,e.colorPrimaryText,"link",{color:e.colorPrimaryTextHover,background:e.linkHoverBg},{color:e.colorPrimaryTextActive})),Cm(e.componentCls,e.ghostBg,e.colorPrimary,e.colorPrimary,e.colorTextDisabled,e.colorBorder,{color:e.colorPrimaryHover,borderColor:e.colorPrimaryHover},{color:e.colorPrimaryActive,borderColor:e.colorPrimaryActive})),D5=e=>Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:e.colorError,boxShadow:e.dangerShadow},wm(e,e.dangerColor,e.colorError,{background:e.colorErrorHover},{background:e.colorErrorActive})),Em(e,e.colorError,e.colorBgContainer,{color:e.colorErrorHover,borderColor:e.colorErrorBorderHover},{color:e.colorErrorActive,borderColor:e.colorErrorActive})),$m(e)),Om(e,e.colorErrorBg,{background:e.colorErrorBgFilledHover},{background:e.colorErrorBgActive})),to(e,e.colorError,"text",{color:e.colorErrorHover,background:e.colorErrorBg},{color:e.colorErrorHover,background:e.colorErrorBgActive})),to(e,e.colorError,"link",{color:e.colorErrorHover},{color:e.colorErrorActive})),Cm(e.componentCls,e.ghostBg,e.colorError,e.colorError,e.colorTextDisabled,e.colorBorder,{color:e.colorErrorHover,borderColor:e.colorErrorHover},{color:e.colorErrorActive,borderColor:e.colorErrorActive})),L5=e=>Object.assign(Object.assign({},to(e,e.colorLink,"link",{color:e.colorLinkHover},{color:e.colorLinkActive})),Cm(e.componentCls,e.ghostBg,e.colorInfo,e.colorInfo,e.colorTextDisabled,e.colorBorder,{color:e.colorInfoHover,borderColor:e.colorInfoHover},{color:e.colorInfoActive,borderColor:e.colorInfoActive})),B5=e=>{const{componentCls:t}=e;return Object.assign({[`${t}-color-default`]:A5(e),[`${t}-color-primary`]:z5(e),[`${t}-color-dangerous`]:D5(e),[`${t}-color-link`]:L5(e)},P5(e))},H5=e=>Object.assign(Object.assign(Object.assign(Object.assign({},Em(e,e.defaultBorderColor,e.defaultBg,{color:e.defaultHoverColor,borderColor:e.defaultHoverBorderColor,background:e.defaultHoverBg},{color:e.defaultActiveColor,borderColor:e.defaultActiveBorderColor,background:e.defaultActiveBg})),to(e,e.textTextColor,"text",{color:e.textTextHoverColor,background:e.textHoverBg},{color:e.textTextActiveColor,background:e.colorBgTextActive})),wm(e,e.primaryColor,e.colorPrimary,{background:e.colorPrimaryHover,color:e.primaryColor},{background:e.colorPrimaryActive,color:e.primaryColor})),to(e,e.colorLink,"link",{color:e.colorLinkHover,background:e.linkHoverBg},{color:e.colorLinkActive})),gy=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:"";const{componentCls:n,controlHeight:r,fontSize:a,borderRadius:l,buttonPaddingHorizontal:s,iconCls:u,buttonPaddingVertical:d,buttonIconOnlyFontSize:g}=e;return[{[t]:{fontSize:a,height:r,padding:`${ve(d)} ${ve(s)}`,borderRadius:l,[`&${n}-icon-only`]:{width:r,[u]:{fontSize:g}}}},{[`${n}${n}-circle${t}`]:T5(e)},{[`${n}${n}-round${t}`]:M5(e)}]},F5=e=>{const t=Kt(e,{fontSize:e.contentFontSize});return gy(t,e.componentCls)},V5=e=>{const t=Kt(e,{controlHeight:e.controlHeightSM,fontSize:e.contentFontSizeSM,padding:e.paddingXS,buttonPaddingHorizontal:e.paddingInlineSM,buttonPaddingVertical:0,borderRadius:e.borderRadiusSM,buttonIconOnlyFontSize:e.onlyIconSizeSM});return gy(t,`${e.componentCls}-sm`)},U5=e=>{const t=Kt(e,{controlHeight:e.controlHeightLG,fontSize:e.contentFontSizeLG,buttonPaddingHorizontal:e.paddingInlineLG,buttonPaddingVertical:0,borderRadius:e.borderRadiusLG,buttonIconOnlyFontSize:e.onlyIconSizeLG});return gy(t,`${e.componentCls}-lg`)},k5=e=>{const{componentCls:t}=e;return{[t]:{[`&${t}-block`]:{width:"100%"}}}},q5=xn("Button",e=>{const t=gR(e);return[_5(t),F5(t),V5(t),U5(t),k5(t),B5(t),H5(t),_z(t)]},hR,{unitless:{fontWeight:!0,contentLineHeight:!0,contentLineHeightSM:!0,contentLineHeightLG:!0}});function W5(e,t,n){const{focusElCls:r,focus:a,borderElCls:l}=n,s=l?"> *":"",u=["hover",a?"focus":null,"active"].filter(Boolean).map(d=>`&:${d} ${s}`).join(",");return{[`&-item:not(${t}-last-item)`]:{marginInlineEnd:e.calc(e.lineWidth).mul(-1).equal()},"&-item":Object.assign(Object.assign({[u]:{zIndex:2}},r?{[`&${r}`]:{zIndex:2}}:{}),{[`&[disabled] ${s}`]:{zIndex:0}})}}function G5(e,t,n){const{borderElCls:r}=n,a=r?`> ${r}`:"";return{[`&-item:not(${t}-first-item):not(${t}-last-item) ${a}`]:{borderRadius:0},[`&-item:not(${t}-last-item)${t}-first-item`]:{[`& ${a}, &${e}-sm ${a}, &${e}-lg ${a}`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`&-item:not(${t}-first-item)${t}-last-item`]:{[`& ${a}, &${e}-sm ${a}, &${e}-lg ${a}`]:{borderStartStartRadius:0,borderEndStartRadius:0}}}}function hy(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{focus:!0};const{componentCls:n}=e,r=`${n}-compact`;return{[r]:Object.assign(Object.assign({},W5(e,r,t)),G5(n,r,t))}}function K5(e,t){return{[`&-item:not(${t}-last-item)`]:{marginBottom:e.calc(e.lineWidth).mul(-1).equal()},"&-item":{"&:hover,&:focus,&:active":{zIndex:2},"&[disabled]":{zIndex:0}}}}function X5(e,t){return{[`&-item:not(${t}-first-item):not(${t}-last-item)`]:{borderRadius:0},[`&-item${t}-first-item:not(${t}-last-item)`]:{[`&, &${e}-sm, &${e}-lg`]:{borderEndEndRadius:0,borderEndStartRadius:0}},[`&-item${t}-last-item:not(${t}-first-item)`]:{[`&, &${e}-sm, &${e}-lg`]:{borderStartStartRadius:0,borderStartEndRadius:0}}}}function Y5(e){const t=`${e.componentCls}-compact-vertical`;return{[t]:Object.assign(Object.assign({},K5(e,t)),X5(e.componentCls,t))}}const Q5=e=>{const{componentCls:t,colorPrimaryHover:n,lineWidth:r,calc:a}=e,l=a(r).mul(-1).equal(),s=u=>{const d=`${t}-compact${u?"-vertical":""}-item${t}-primary:not([disabled])`;return{[`${d} + ${d}::before`]:{position:"absolute",top:u?l:0,insetInlineStart:u?0:l,backgroundColor:n,content:'""',width:u?"100%":r,height:u?r:"100%"}}};return Object.assign(Object.assign({},s()),s(!0))},Z5=hs(["Button","compact"],e=>{const t=gR(e);return[hy(t),Y5(t),Q5(t)]},hR);var J5=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function eD(e){if(typeof e=="object"&&e){let t=e==null?void 0:e.delay;return t=!Number.isNaN(t)&&typeof t=="number"?t:0,{loading:t<=0,delay:t}}return{loading:!!e,delay:0}}const tD={default:["default","outlined"],primary:["primary","solid"],dashed:["default","dashed"],link:["link","link"],text:["default","text"]},nD=se.forwardRef((e,t)=>{var n,r;const{loading:a=!1,prefixCls:l,color:s,variant:u,type:d,danger:g=!1,shape:h="default",size:p,styles:y,disabled:C,className:b,rootClassName:x,children:S,icon:E,iconPosition:$="start",ghost:O=!1,block:_=!1,htmlType:R="button",classNames:T,style:M={},autoInsertSpace:z,autoFocus:P}=e,D=J5(e,["loading","prefixCls","color","variant","type","danger","shape","size","styles","disabled","className","rootClassName","children","icon","iconPosition","ghost","block","htmlType","classNames","style","autoInsertSpace","autoFocus"]),F=d||"default",[G,V]=f.useMemo(()=>{if(s&&u)return[s,u];const Ue=tD[F]||[];return g?["danger",Ue[1]]:Ue},[d,s,u,g]),K=G==="danger"?"dangerous":G,{getPrefixCls:j,direction:A,autoInsertSpace:L,className:q,style:N,classNames:B,styles:W}=Or("button"),X=(n=z??L)!==null&&n!==void 0?n:!0,k=j("btn",l),[Y,ee,ue]=q5(k),ae=f.useContext(Ro),oe=C??ae,he=f.useContext(lR),ie=f.useMemo(()=>eD(a),[a]),[le,be]=f.useState(ie.loading),[Re,fe]=f.useState(!1),Se=f.useRef(null),Ee=Io(t,Se),Oe=f.Children.count(S)===1&&!E&&!Ep(V),Pe=f.useRef(!0);se.useEffect(()=>(Pe.current=!1,()=>{Pe.current=!0}),[]),f.useEffect(()=>{let Ue=null;ie.delay>0?Ue=setTimeout(()=>{Ue=null,be(!0)},ie.delay):be(ie.loading);function Ie(){Ue&&(clearTimeout(Ue),Ue=null)}return Ie},[ie]),f.useEffect(()=>{if(!Se.current||!X)return;const Ue=Se.current.textContent||"";Oe&&e0(Ue)?Re||fe(!0):Re&&fe(!1)}),f.useEffect(()=>{P&&Se.current&&Se.current.focus()},[]);const Ye=se.useCallback(Ue=>{var Ie;if(le||oe){Ue.preventDefault();return}(Ie=e.onClick)===null||Ie===void 0||Ie.call(e,("href"in e,Ue))},[e.onClick,le,oe]),{compactSize:Me,compactItemClassnames:De}=Ru(k,A),xe={large:"lg",small:"sm",middle:void 0},Te=Lo(Ue=>{var Ie,qe;return(qe=(Ie=p??Me)!==null&&Ie!==void 0?Ie:he)!==null&&qe!==void 0?qe:Ue}),Le=Te&&(r=xe[Te])!==null&&r!==void 0?r:"",We=le?"loading":E,Ge=On(D,["navigate"]),at=re(k,ee,ue,{[`${k}-${h}`]:h!=="default"&&h,[`${k}-${F}`]:F,[`${k}-dangerous`]:g,[`${k}-color-${K}`]:K,[`${k}-variant-${V}`]:V,[`${k}-${Le}`]:Le,[`${k}-icon-only`]:!S&&S!==0&&!!We,[`${k}-background-ghost`]:O&&!Ep(V),[`${k}-loading`]:le,[`${k}-two-chinese-chars`]:Re&&X&&!le,[`${k}-block`]:_,[`${k}-rtl`]:A==="rtl",[`${k}-icon-end`]:$==="end"},De,b,x,q),ke=Object.assign(Object.assign({},N),M),Qe=re(T==null?void 0:T.icon,B.icon),ct=Object.assign(Object.assign({},(y==null?void 0:y.icon)||{}),W.icon||{}),pt=E&&!le?se.createElement(t0,{prefixCls:k,className:Qe,style:ct},E):a&&typeof a=="object"&&a.icon?se.createElement(t0,{prefixCls:k,className:Qe,style:ct},a.icon):se.createElement(Rz,{existIcon:!!E,prefixCls:k,loading:le,mount:Pe.current}),Ze=S||S===0?Oz(S,Oe&&X):null;if(Ge.href!==void 0)return Y(se.createElement("a",Object.assign({},Ge,{className:re(at,{[`${k}-disabled`]:oe}),href:oe?void 0:Ge.href,style:ke,onClick:Ye,ref:Ee,tabIndex:oe?-1:0}),pt,Ze));let Be=se.createElement("button",Object.assign({},D,{type:R,className:at,style:ke,onClick:Ye,disabled:oe,ref:Ee}),pt,Ze,De&&se.createElement(Z5,{prefixCls:k}));return Ep(V)||(Be=se.createElement(oR,{component:"Button",disabled:le},Be)),Y(Be)}),er=nD;er.Group=Ez;er.__ANT_BUTTON=!0;function _p(e){return!!(e!=null&&e.then)}const py=e=>{const{type:t,children:n,prefixCls:r,buttonProps:a,close:l,autoFocus:s,emitEvent:u,isSilent:d,quitOnNullishReturnValue:g,actionFn:h}=e,p=f.useRef(!1),y=f.useRef(null),[C,b]=Zi(!1),x=function(){l==null||l.apply(void 0,arguments)};f.useEffect(()=>{let $=null;return s&&($=setTimeout(()=>{var O;(O=y.current)===null||O===void 0||O.focus({preventScroll:!0})})),()=>{$&&clearTimeout($)}},[]);const S=$=>{_p($)&&(b(!0),$.then(function(){b(!1,!0),x.apply(void 0,arguments),p.current=!1},O=>{if(b(!1,!0),p.current=!1,!(d!=null&&d()))return Promise.reject(O)}))},E=$=>{if(p.current)return;if(p.current=!0,!h){x();return}let O;if(u){if(O=h($),g&&!_p(O)){p.current=!1,x($);return}}else if(h.length)O=h(l),p.current=!1;else if(O=h(),!_p(O)){x();return}S(O)};return f.createElement(er,Object.assign({},uy(t),{onClick:E,loading:C,prefixCls:r},a,{ref:y}),n)},_u=se.createContext({}),{Provider:vR}=_u,Gx=()=>{const{autoFocusButton:e,cancelButtonProps:t,cancelTextLocale:n,isSilent:r,mergedOkCancel:a,rootPrefixCls:l,close:s,onCancel:u,onConfirm:d}=f.useContext(_u);return a?se.createElement(py,{isSilent:r,actionFn:u,close:function(){s==null||s.apply(void 0,arguments),d==null||d(!1)},autoFocus:e==="cancel",buttonProps:t,prefixCls:`${l}-btn`},n):null},Kx=()=>{const{autoFocusButton:e,close:t,isSilent:n,okButtonProps:r,rootPrefixCls:a,okTextLocale:l,okType:s,onConfirm:u,onOk:d}=f.useContext(_u);return se.createElement(py,{isSilent:n,type:s||"primary",actionFn:d,close:function(){t==null||t.apply(void 0,arguments),u==null||u(!0)},autoFocus:e==="ok",buttonProps:r,prefixCls:`${a}-btn`},l)};var yR=f.createContext(null),Xx=[];function rD(e,t){var n=f.useState(function(){if(!$r())return null;var b=document.createElement("div");return b}),r=ge(n,1),a=r[0],l=f.useRef(!1),s=f.useContext(yR),u=f.useState(Xx),d=ge(u,2),g=d[0],h=d[1],p=s||(l.current?void 0:function(b){h(function(x){var S=[b].concat(Ne(x));return S})});function y(){a.parentElement||document.body.appendChild(a),l.current=!0}function C(){var b;(b=a.parentElement)===null||b===void 0||b.removeChild(a),l.current=!1}return Gt(function(){return e?s?s(y):y():C(),C},[e]),Gt(function(){g.length&&(g.forEach(function(b){return b()}),h(Xx))},[g]),[a,p]}var Tp;function bR(e){var t="rc-scrollbar-measure-".concat(Math.random().toString(36).substring(7)),n=document.createElement("div");n.id=t;var r=n.style;r.position="absolute",r.left="0",r.top="0",r.width="100px",r.height="100px",r.overflow="scroll";var a,l;if(e){var s=getComputedStyle(e);r.scrollbarColor=s.scrollbarColor,r.scrollbarWidth=s.scrollbarWidth;var u=getComputedStyle(e,"::-webkit-scrollbar"),d=parseInt(u.width,10),g=parseInt(u.height,10);try{var h=d?"width: ".concat(u.width,";"):"",p=g?"height: ".concat(u.height,";"):"";xo(`
#`.concat(t,`::-webkit-scrollbar {
`).concat(h,`
`).concat(p,`
}`),t)}catch(b){console.error(b),a=d,l=g}}document.body.appendChild(n);var y=e&&a&&!isNaN(a)?a:n.offsetWidth-n.clientWidth,C=e&&l&&!isNaN(l)?l:n.offsetHeight-n.clientHeight;return document.body.removeChild(n),Zc(t),{width:y,height:C}}function hX(e){return typeof document>"u"?0:(Tp===void 0&&(Tp=bR()),Tp.width)}function aD(e){return typeof document>"u"||!e||!(e instanceof Element)?{width:0,height:0}:bR(e)}function oD(){return document.body.scrollHeight>(window.innerHeight||document.documentElement.clientHeight)&&window.innerWidth>document.body.offsetWidth}var iD="rc-util-locker-".concat(Date.now()),Yx=0;function lD(e){var t=!!e,n=f.useState(function(){return Yx+=1,"".concat(iD,"_").concat(Yx)}),r=ge(n,1),a=r[0];Gt(function(){if(t){var l=aD(document.body).width,s=oD();xo(`
html body {
  overflow-y: hidden;
  `.concat(s?"width: calc(100% - ".concat(l,"px);"):"",`
}`),a)}else Zc(a);return function(){Zc(a)}},[t,a])}var sD=!1;function cD(e){return sD}var Qx=function(t){return t===!1?!1:!$r()||!t?null:typeof t=="string"?document.querySelector(t):typeof t=="function"?t():t},Rm=f.forwardRef(function(e,t){var n=e.open,r=e.autoLock,a=e.getContainer;e.debug;var l=e.autoDestroy,s=l===void 0?!0:l,u=e.children,d=f.useState(n),g=ge(d,2),h=g[0],p=g[1],y=h||n;f.useEffect(function(){(s||n)&&p(n)},[n,s]);var C=f.useState(function(){return Qx(a)}),b=ge(C,2),x=b[0],S=b[1];f.useEffect(function(){var F=Qx(a);S(F??null)});var E=rD(y&&!x),$=ge(E,2),O=$[0],_=$[1],R=x??O;lD(r&&n&&$r()&&(R===O||R===document.body));var T=null;if(u&&rl(u)&&t){var M=u;T=M.ref}var z=Io(T,t);if(!y||!$r()||x===void 0)return null;var P=R===!1||cD(),D=u;return t&&(D=f.cloneElement(u,{ref:z})),f.createElement(yR.Provider,{value:_},P?D:pi.createPortal(D,R))}),SR=f.createContext({});function uD(){var e=Q({},lm);return e.useId}var Zx=0,Jx=uD();const vy=Jx?function(t){var n=Jx();return t||n}:function(t){var n=f.useState("ssr-id"),r=ge(n,2),a=r[0],l=r[1];return f.useEffect(function(){var s=Zx;Zx+=1,l("rc_unique_".concat(s))},[]),t||a};function ew(e,t,n){var r=t;return!r&&n&&(r="".concat(e,"-").concat(n)),r}function tw(e,t){var n=e["page".concat(t?"Y":"X","Offset")],r="scroll".concat(t?"Top":"Left");if(typeof n!="number"){var a=e.document;n=a.documentElement[r],typeof n!="number"&&(n=a.body[r])}return n}function fD(e){var t=e.getBoundingClientRect(),n={left:t.left,top:t.top},r=e.ownerDocument,a=r.defaultView||r.parentWindow;return n.left+=tw(a),n.top+=tw(a,!0),n}const dD=f.memo(function(e){var t=e.children;return t},function(e,t){var n=t.shouldUpdate;return!n});var mD={width:0,height:0,overflow:"hidden",outline:"none"},gD={outline:"none"},CR=se.forwardRef(function(e,t){var n=e.prefixCls,r=e.className,a=e.style,l=e.title,s=e.ariaId,u=e.footer,d=e.closable,g=e.closeIcon,h=e.onClose,p=e.children,y=e.bodyStyle,C=e.bodyProps,b=e.modalRender,x=e.onMouseDown,S=e.onMouseUp,E=e.holderRef,$=e.visible,O=e.forceRender,_=e.width,R=e.height,T=e.classNames,M=e.styles,z=se.useContext(SR),P=z.panel,D=Io(E,P),F=f.useRef(),G=f.useRef();se.useImperativeHandle(t,function(){return{focus:function(){var W;(W=F.current)===null||W===void 0||W.focus({preventScroll:!0})},changeActive:function(W){var X=document,k=X.activeElement;W&&k===G.current?F.current.focus({preventScroll:!0}):!W&&k===F.current&&G.current.focus({preventScroll:!0})}}});var V={};_!==void 0&&(V.width=_),R!==void 0&&(V.height=R);var U=u?se.createElement("div",{className:re("".concat(n,"-footer"),T==null?void 0:T.footer),style:Q({},M==null?void 0:M.footer)},u):null,K=l?se.createElement("div",{className:re("".concat(n,"-header"),T==null?void 0:T.header),style:Q({},M==null?void 0:M.header)},se.createElement("div",{className:"".concat(n,"-title"),id:s},l)):null,j=f.useMemo(function(){return st(d)==="object"&&d!==null?d:d?{closeIcon:g??se.createElement("span",{className:"".concat(n,"-close-x")})}:{}},[d,g,n]),A=ta(j,!0),L=st(d)==="object"&&d.disabled,q=d?se.createElement("button",je({type:"button",onClick:h,"aria-label":"Close"},A,{className:"".concat(n,"-close"),disabled:L}),j.closeIcon):null,N=se.createElement("div",{className:re("".concat(n,"-content"),T==null?void 0:T.content),style:M==null?void 0:M.content},q,K,se.createElement("div",je({className:re("".concat(n,"-body"),T==null?void 0:T.body),style:Q(Q({},y),M==null?void 0:M.body)},C),p),U);return se.createElement("div",{key:"dialog-element",role:"dialog","aria-labelledby":l?s:null,"aria-modal":"true",ref:D,style:Q(Q({},a),V),className:re(n,r),onMouseDown:x,onMouseUp:S},se.createElement("div",{ref:F,tabIndex:0,style:gD},se.createElement(dD,{shouldUpdate:$||O},b?b(N):N)),se.createElement("div",{tabIndex:0,ref:G,style:mD}))}),xR=f.forwardRef(function(e,t){var n=e.prefixCls,r=e.title,a=e.style,l=e.className,s=e.visible,u=e.forceRender,d=e.destroyOnClose,g=e.motionName,h=e.ariaId,p=e.onVisibleChanged,y=e.mousePosition,C=f.useRef(),b=f.useState(),x=ge(b,2),S=x[0],E=x[1],$={};S&&($.transformOrigin=S);function O(){var _=fD(C.current);E(y&&(y.x||y.y)?"".concat(y.x-_.left,"px ").concat(y.y-_.top,"px"):"")}return f.createElement($a,{visible:s,onVisibleChanged:p,onAppearPrepare:O,onEnterPrepare:O,forceRender:u,motionName:g,removeOnLeave:d,ref:C},function(_,R){var T=_.className,M=_.style;return f.createElement(CR,je({},e,{ref:t,title:r,ariaId:h,prefixCls:n,holderRef:R,style:Q(Q(Q({},M),a),$),className:re(l,T)}))})});xR.displayName="Content";var hD=function(t){var n=t.prefixCls,r=t.style,a=t.visible,l=t.maskProps,s=t.motionName,u=t.className;return f.createElement($a,{key:"mask",visible:a,motionName:s,leavedClassName:"".concat(n,"-mask-hidden")},function(d,g){var h=d.className,p=d.style;return f.createElement("div",je({ref:g,style:Q(Q({},p),r),className:re("".concat(n,"-mask"),h,u)},l))})},pD=function(t){var n=t.prefixCls,r=n===void 0?"rc-dialog":n,a=t.zIndex,l=t.visible,s=l===void 0?!1:l,u=t.keyboard,d=u===void 0?!0:u,g=t.focusTriggerAfterClose,h=g===void 0?!0:g,p=t.wrapStyle,y=t.wrapClassName,C=t.wrapProps,b=t.onClose,x=t.afterOpenChange,S=t.afterClose,E=t.transitionName,$=t.animation,O=t.closable,_=O===void 0?!0:O,R=t.mask,T=R===void 0?!0:R,M=t.maskTransitionName,z=t.maskAnimation,P=t.maskClosable,D=P===void 0?!0:P,F=t.maskStyle,G=t.maskProps,V=t.rootClassName,U=t.classNames,K=t.styles,j=f.useRef(),A=f.useRef(),L=f.useRef(),q=f.useState(s),N=ge(q,2),B=N[0],W=N[1],X=vy();function k(){Iv(A.current,document.activeElement)||(j.current=document.activeElement)}function Y(){if(!Iv(A.current,document.activeElement)){var fe;(fe=L.current)===null||fe===void 0||fe.focus()}}function ee(fe){if(fe)Y();else{if(W(!1),T&&j.current&&h){try{j.current.focus({preventScroll:!0})}catch{}j.current=null}B&&(S==null||S())}x==null||x(fe)}function ue(fe){b==null||b(fe)}var ae=f.useRef(!1),oe=f.useRef(),he=function(){clearTimeout(oe.current),ae.current=!0},ie=function(){oe.current=setTimeout(function(){ae.current=!1})},le=null;D&&(le=function(Se){ae.current?ae.current=!1:A.current===Se.target&&ue(Se)});function be(fe){if(d&&fe.keyCode===lt.ESC){fe.stopPropagation(),ue(fe);return}s&&fe.keyCode===lt.TAB&&L.current.changeActive(!fe.shiftKey)}f.useEffect(function(){s&&(W(!0),k())},[s]),f.useEffect(function(){return function(){clearTimeout(oe.current)}},[]);var Re=Q(Q(Q({zIndex:a},p),K==null?void 0:K.wrapper),{},{display:B?null:"none"});return f.createElement("div",je({className:re("".concat(r,"-root"),V)},ta(t,{data:!0})),f.createElement(hD,{prefixCls:r,visible:T&&s,motionName:ew(r,M,z),style:Q(Q({zIndex:a},F),K==null?void 0:K.mask),maskProps:G,className:U==null?void 0:U.mask}),f.createElement("div",je({tabIndex:-1,onKeyDown:be,className:re("".concat(r,"-wrap"),y,U==null?void 0:U.wrapper),ref:A,onClick:le,style:Re},C),f.createElement(xR,je({},t,{onMouseDown:he,onMouseUp:ie,ref:L,closable:_,ariaId:X,prefixCls:r,visible:s&&B,onClose:ue,onVisibleChanged:ee,motionName:ew(r,E,$)}))))},wR=function(t){var n=t.visible,r=t.getContainer,a=t.forceRender,l=t.destroyOnClose,s=l===void 0?!1:l,u=t.afterClose,d=t.panelRef,g=f.useState(n),h=ge(g,2),p=h[0],y=h[1],C=f.useMemo(function(){return{panel:d}},[d]);return f.useEffect(function(){n&&y(!0)},[n]),!a&&s&&!p?null:f.createElement(SR.Provider,{value:C},f.createElement(Rm,{open:n||a||p,autoDestroy:!1,getContainer:r,autoLock:n||p},f.createElement(pD,je({},t,{destroyOnClose:s,afterClose:function(){u==null||u(),y(!1)}}))))};wR.displayName="Dialog";var ki="RC_FORM_INTERNAL_HOOKS",yn=function(){mr(!1,"Can not find FormContext. Please make sure you wrap Field under Form.")},Ji=f.createContext({getFieldValue:yn,getFieldsValue:yn,getFieldError:yn,getFieldWarning:yn,getFieldsError:yn,isFieldsTouched:yn,isFieldTouched:yn,isFieldValidating:yn,isFieldsValidating:yn,resetFields:yn,setFields:yn,setFieldValue:yn,setFieldsValue:yn,validateFields:yn,submit:yn,getInternalHooks:function(){return yn(),{dispatch:yn,initEntityValue:yn,registerField:yn,useSubscribe:yn,setInitialValues:yn,destroyForm:yn,setCallbacks:yn,registerWatch:yn,getFields:yn,setValidateMessages:yn,setPreserve:yn,getInitialValue:yn}}}),su=f.createContext(null);function n0(e){return e==null?[]:Array.isArray(e)?e:[e]}function vD(e){return e&&!!e._init}function r0(){return{default:"Validation error on field %s",required:"%s is required",enum:"%s must be one of %s",whitespace:"%s cannot be empty",date:{format:"%s date %s is invalid for format %s",parse:"%s date could not be parsed, %s is invalid ",invalid:"%s date %s is invalid"},types:{string:"%s is not a %s",method:"%s is not a %s (function)",array:"%s is not an %s",object:"%s is not an %s",number:"%s is not a %s",date:"%s is not a %s",boolean:"%s is not a %s",integer:"%s is not an %s",float:"%s is not a %s",regexp:"%s is not a valid %s",email:"%s is not a valid %s",url:"%s is not a valid %s",hex:"%s is not a valid %s"},string:{len:"%s must be exactly %s characters",min:"%s must be at least %s characters",max:"%s cannot be longer than %s characters",range:"%s must be between %s and %s characters"},number:{len:"%s must equal %s",min:"%s cannot be less than %s",max:"%s cannot be greater than %s",range:"%s must be between %s and %s"},array:{len:"%s must be exactly %s in length",min:"%s cannot be less than %s in length",max:"%s cannot be greater than %s in length",range:"%s must be between %s and %s in length"},pattern:{mismatch:"%s value %s does not match pattern %s"},clone:function(){var t=JSON.parse(JSON.stringify(this));return t.clone=this.clone,t}}}var a0=r0();function yD(e){try{return Function.toString.call(e).indexOf("[native code]")!==-1}catch{return typeof e=="function"}}function bD(e,t,n){if(K0())return Reflect.construct.apply(null,arguments);var r=[null];r.push.apply(r,t);var a=new(e.bind.apply(e,r));return n&&eu(a,n.prototype),a}function o0(e){var t=typeof Map=="function"?new Map:void 0;return o0=function(r){if(r===null||!yD(r))return r;if(typeof r!="function")throw new TypeError("Super expression must either be null or a function");if(t!==void 0){if(t.has(r))return t.get(r);t.set(r,a)}function a(){return bD(r,arguments,tu(this).constructor)}return a.prototype=Object.create(r.prototype,{constructor:{value:a,enumerable:!1,writable:!0,configurable:!0}}),eu(a,r)},o0(e)}var SD=/%[sdj%]/g,CD=function(){};function i0(e){if(!e||!e.length)return null;var t={};return e.forEach(function(n){var r=n.field;t[r]=t[r]||[],t[r].push(n)}),t}function Jr(e){for(var t=arguments.length,n=new Array(t>1?t-1:0),r=1;r<t;r++)n[r-1]=arguments[r];var a=0,l=n.length;if(typeof e=="function")return e.apply(null,n);if(typeof e=="string"){var s=e.replace(SD,function(u){if(u==="%%")return"%";if(a>=l)return u;switch(u){case"%s":return String(n[a++]);case"%d":return Number(n[a++]);case"%j":try{return JSON.stringify(n[a++])}catch{return"[Circular]"}break;default:return u}});return s}return e}function xD(e){return e==="string"||e==="url"||e==="hex"||e==="email"||e==="date"||e==="pattern"}function rr(e,t){return!!(e==null||t==="array"&&Array.isArray(e)&&!e.length||xD(t)&&typeof e=="string"&&!e)}function wD(e,t,n){var r=[],a=0,l=e.length;function s(u){r.push.apply(r,Ne(u||[])),a++,a===l&&n(r)}e.forEach(function(u){t(u,s)})}function nw(e,t,n){var r=0,a=e.length;function l(s){if(s&&s.length){n(s);return}var u=r;r=r+1,u<a?t(e[u],l):n([])}l([])}function ED(e){var t=[];return Object.keys(e).forEach(function(n){t.push.apply(t,Ne(e[n]||[]))}),t}var rw=function(e){Po(n,e);var t=Ao(n);function n(r,a){var l;return Wn(this,n),l=t.call(this,"Async Validation Error"),Z(jt(l),"errors",void 0),Z(jt(l),"fields",void 0),l.errors=r,l.fields=a,l}return Gn(n)}(o0(Error));function $D(e,t,n,r,a){if(t.first){var l=new Promise(function(y,C){var b=function(E){return r(E),E.length?C(new rw(E,i0(E))):y(a)},x=ED(e);nw(x,n,b)});return l.catch(function(y){return y}),l}var s=t.firstFields===!0?Object.keys(e):t.firstFields||[],u=Object.keys(e),d=u.length,g=0,h=[],p=new Promise(function(y,C){var b=function(S){if(h.push.apply(h,S),g++,g===d)return r(h),h.length?C(new rw(h,i0(h))):y(a)};u.length||(r(h),y(a)),u.forEach(function(x){var S=e[x];s.indexOf(x)!==-1?nw(S,n,b):wD(S,n,b)})});return p.catch(function(y){return y}),p}function OD(e){return!!(e&&e.message!==void 0)}function RD(e,t){for(var n=e,r=0;r<t.length;r++){if(n==null)return n;n=n[t[r]]}return n}function aw(e,t){return function(n){var r;return e.fullFields?r=RD(t,e.fullFields):r=t[n.field||e.fullField],OD(n)?(n.field=n.field||e.fullField,n.fieldValue=r,n):{message:typeof n=="function"?n():n,fieldValue:r,field:n.field||e.fullField}}}function ow(e,t){if(t){for(var n in t)if(t.hasOwnProperty(n)){var r=t[n];st(r)==="object"&&st(e[n])==="object"?e[n]=Q(Q({},e[n]),r):e[n]=r}}return e}var Ll="enum",_D=function(t,n,r,a,l){t[Ll]=Array.isArray(t[Ll])?t[Ll]:[],t[Ll].indexOf(n)===-1&&a.push(Jr(l.messages[Ll],t.fullField,t[Ll].join(", ")))},TD=function(t,n,r,a,l){if(t.pattern){if(t.pattern instanceof RegExp)t.pattern.lastIndex=0,t.pattern.test(n)||a.push(Jr(l.messages.pattern.mismatch,t.fullField,n,t.pattern));else if(typeof t.pattern=="string"){var s=new RegExp(t.pattern);s.test(n)||a.push(Jr(l.messages.pattern.mismatch,t.fullField,n,t.pattern))}}},MD=function(t,n,r,a,l){var s=typeof t.len=="number",u=typeof t.min=="number",d=typeof t.max=="number",g=/[\uD800-\uDBFF][\uDC00-\uDFFF]/g,h=n,p=null,y=typeof n=="number",C=typeof n=="string",b=Array.isArray(n);if(y?p="number":C?p="string":b&&(p="array"),!p)return!1;b&&(h=n.length),C&&(h=n.replace(g,"_").length),s?h!==t.len&&a.push(Jr(l.messages[p].len,t.fullField,t.len)):u&&!d&&h<t.min?a.push(Jr(l.messages[p].min,t.fullField,t.min)):d&&!u&&h>t.max?a.push(Jr(l.messages[p].max,t.fullField,t.max)):u&&d&&(h<t.min||h>t.max)&&a.push(Jr(l.messages[p].range,t.fullField,t.min,t.max))},ER=function(t,n,r,a,l,s){t.required&&(!r.hasOwnProperty(t.field)||rr(n,s||t.type))&&a.push(Jr(l.messages.required,t.fullField))},Zf;const ND=function(){if(Zf)return Zf;var e="[a-fA-F\\d:]",t=function(T){return T&&T.includeBoundaries?"(?:(?<=\\s|^)(?=".concat(e,")|(?<=").concat(e,")(?=\\s|$))"):""},n="(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}",r="[a-fA-F\\d]{1,4}",a=["(?:".concat(r,":){7}(?:").concat(r,"|:)"),"(?:".concat(r,":){6}(?:").concat(n,"|:").concat(r,"|:)"),"(?:".concat(r,":){5}(?::").concat(n,"|(?::").concat(r,"){1,2}|:)"),"(?:".concat(r,":){4}(?:(?::").concat(r,"){0,1}:").concat(n,"|(?::").concat(r,"){1,3}|:)"),"(?:".concat(r,":){3}(?:(?::").concat(r,"){0,2}:").concat(n,"|(?::").concat(r,"){1,4}|:)"),"(?:".concat(r,":){2}(?:(?::").concat(r,"){0,3}:").concat(n,"|(?::").concat(r,"){1,5}|:)"),"(?:".concat(r,":){1}(?:(?::").concat(r,"){0,4}:").concat(n,"|(?::").concat(r,"){1,6}|:)"),"(?::(?:(?::".concat(r,"){0,5}:").concat(n,"|(?::").concat(r,"){1,7}|:))")],l="(?:%[0-9a-zA-Z]{1,})?",s="(?:".concat(a.join("|"),")").concat(l),u=new RegExp("(?:^".concat(n,"$)|(?:^").concat(s,"$)")),d=new RegExp("^".concat(n,"$")),g=new RegExp("^".concat(s,"$")),h=function(T){return T&&T.exact?u:new RegExp("(?:".concat(t(T)).concat(n).concat(t(T),")|(?:").concat(t(T)).concat(s).concat(t(T),")"),"g")};h.v4=function(R){return R&&R.exact?d:new RegExp("".concat(t(R)).concat(n).concat(t(R)),"g")},h.v6=function(R){return R&&R.exact?g:new RegExp("".concat(t(R)).concat(s).concat(t(R)),"g")};var p="(?:(?:[a-z]+:)?//)",y="(?:\\S+(?::\\S*)?@)?",C=h.v4().source,b=h.v6().source,x="(?:(?:[a-z\\u00a1-\\uffff0-9][-_]*)*[a-z\\u00a1-\\uffff0-9]+)",S="(?:\\.(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)*",E="(?:\\.(?:[a-z\\u00a1-\\uffff]{2,}))",$="(?::\\d{2,5})?",O='(?:[/?#][^\\s"]*)?',_="(?:".concat(p,"|www\\.)").concat(y,"(?:localhost|").concat(C,"|").concat(b,"|").concat(x).concat(S).concat(E,")").concat($).concat(O);return Zf=new RegExp("(?:^".concat(_,"$)"),"i"),Zf};var iw={email:/^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+\.)+[a-zA-Z\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]{2,}))$/,hex:/^#?([a-f0-9]{6}|[a-f0-9]{3})$/i},Uc={integer:function(t){return Uc.number(t)&&parseInt(t,10)===t},float:function(t){return Uc.number(t)&&!Uc.integer(t)},array:function(t){return Array.isArray(t)},regexp:function(t){if(t instanceof RegExp)return!0;try{return!!new RegExp(t)}catch{return!1}},date:function(t){return typeof t.getTime=="function"&&typeof t.getMonth=="function"&&typeof t.getYear=="function"&&!isNaN(t.getTime())},number:function(t){return isNaN(t)?!1:typeof t=="number"},object:function(t){return st(t)==="object"&&!Uc.array(t)},method:function(t){return typeof t=="function"},email:function(t){return typeof t=="string"&&t.length<=320&&!!t.match(iw.email)},url:function(t){return typeof t=="string"&&t.length<=2048&&!!t.match(ND())},hex:function(t){return typeof t=="string"&&!!t.match(iw.hex)}},jD=function(t,n,r,a,l){if(t.required&&n===void 0){ER(t,n,r,a,l);return}var s=["integer","float","array","regexp","object","method","email","number","date","url","hex"],u=t.type;s.indexOf(u)>-1?Uc[u](n)||a.push(Jr(l.messages.types[u],t.fullField,t.type)):u&&st(n)!==t.type&&a.push(Jr(l.messages.types[u],t.fullField,t.type))},ID=function(t,n,r,a,l){(/^\s+$/.test(n)||n==="")&&a.push(Jr(l.messages.whitespace,t.fullField))};const Vt={required:ER,whitespace:ID,type:jD,range:MD,enum:_D,pattern:TD};var PD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n)&&!t.required)return r();Vt.required(t,n,a,s,l)}r(s)},AD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(n==null&&!t.required)return r();Vt.required(t,n,a,s,l,"array"),n!=null&&(Vt.type(t,n,a,s,l),Vt.range(t,n,a,s,l))}r(s)},zD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n)&&!t.required)return r();Vt.required(t,n,a,s,l),n!==void 0&&Vt.type(t,n,a,s,l)}r(s)},DD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n,"date")&&!t.required)return r();if(Vt.required(t,n,a,s,l),!rr(n,"date")){var d;n instanceof Date?d=n:d=new Date(n),Vt.type(t,d,a,s,l),d&&Vt.range(t,d.getTime(),a,s,l)}}r(s)},LD="enum",BD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n)&&!t.required)return r();Vt.required(t,n,a,s,l),n!==void 0&&Vt[LD](t,n,a,s,l)}r(s)},HD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n)&&!t.required)return r();Vt.required(t,n,a,s,l),n!==void 0&&(Vt.type(t,n,a,s,l),Vt.range(t,n,a,s,l))}r(s)},FD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n)&&!t.required)return r();Vt.required(t,n,a,s,l),n!==void 0&&(Vt.type(t,n,a,s,l),Vt.range(t,n,a,s,l))}r(s)},VD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n)&&!t.required)return r();Vt.required(t,n,a,s,l),n!==void 0&&Vt.type(t,n,a,s,l)}r(s)},UD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(n===""&&(n=void 0),rr(n)&&!t.required)return r();Vt.required(t,n,a,s,l),n!==void 0&&(Vt.type(t,n,a,s,l),Vt.range(t,n,a,s,l))}r(s)},kD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n)&&!t.required)return r();Vt.required(t,n,a,s,l),n!==void 0&&Vt.type(t,n,a,s,l)}r(s)},qD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n,"string")&&!t.required)return r();Vt.required(t,n,a,s,l),rr(n,"string")||Vt.pattern(t,n,a,s,l)}r(s)},WD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n)&&!t.required)return r();Vt.required(t,n,a,s,l),rr(n)||Vt.type(t,n,a,s,l)}r(s)},GD=function(t,n,r,a,l){var s=[],u=Array.isArray(n)?"array":st(n);Vt.required(t,n,a,s,l,u),r(s)},KD=function(t,n,r,a,l){var s=[],u=t.required||!t.required&&a.hasOwnProperty(t.field);if(u){if(rr(n,"string")&&!t.required)return r();Vt.required(t,n,a,s,l,"string"),rr(n,"string")||(Vt.type(t,n,a,s,l),Vt.range(t,n,a,s,l),Vt.pattern(t,n,a,s,l),t.whitespace===!0&&Vt.whitespace(t,n,a,s,l))}r(s)},Mp=function(t,n,r,a,l){var s=t.type,u=[],d=t.required||!t.required&&a.hasOwnProperty(t.field);if(d){if(rr(n,s)&&!t.required)return r();Vt.required(t,n,a,u,l,s),rr(n,s)||Vt.type(t,n,a,u,l)}r(u)};const Kc={string:KD,method:VD,number:UD,boolean:zD,regexp:WD,integer:FD,float:HD,array:AD,object:kD,enum:BD,pattern:qD,date:DD,url:Mp,hex:Mp,email:Mp,required:GD,any:PD};var Tu=function(){function e(t){Wn(this,e),Z(this,"rules",null),Z(this,"_messages",a0),this.define(t)}return Gn(e,[{key:"define",value:function(n){var r=this;if(!n)throw new Error("Cannot configure a schema with no rules");if(st(n)!=="object"||Array.isArray(n))throw new Error("Rules must be an object");this.rules={},Object.keys(n).forEach(function(a){var l=n[a];r.rules[a]=Array.isArray(l)?l:[l]})}},{key:"messages",value:function(n){return n&&(this._messages=ow(r0(),n)),this._messages}},{key:"validate",value:function(n){var r=this,a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},l=arguments.length>2&&arguments[2]!==void 0?arguments[2]:function(){},s=n,u=a,d=l;if(typeof u=="function"&&(d=u,u={}),!this.rules||Object.keys(this.rules).length===0)return d&&d(null,s),Promise.resolve(s);function g(b){var x=[],S={};function E(O){if(Array.isArray(O)){var _;x=(_=x).concat.apply(_,Ne(O))}else x.push(O)}for(var $=0;$<b.length;$++)E(b[$]);x.length?(S=i0(x),d(x,S)):d(null,s)}if(u.messages){var h=this.messages();h===a0&&(h=r0()),ow(h,u.messages),u.messages=h}else u.messages=this.messages();var p={},y=u.keys||Object.keys(this.rules);y.forEach(function(b){var x=r.rules[b],S=s[b];x.forEach(function(E){var $=E;typeof $.transform=="function"&&(s===n&&(s=Q({},s)),S=s[b]=$.transform(S),S!=null&&($.type=$.type||(Array.isArray(S)?"array":st(S)))),typeof $=="function"?$={validator:$}:$=Q({},$),$.validator=r.getValidationMethod($),$.validator&&($.field=b,$.fullField=$.fullField||b,$.type=r.getType($),p[b]=p[b]||[],p[b].push({rule:$,value:S,source:s,field:b}))})});var C={};return $D(p,u,function(b,x){var S=b.rule,E=(S.type==="object"||S.type==="array")&&(st(S.fields)==="object"||st(S.defaultField)==="object");E=E&&(S.required||!S.required&&b.value),S.field=b.field;function $(M,z){return Q(Q({},z),{},{fullField:"".concat(S.fullField,".").concat(M),fullFields:S.fullFields?[].concat(Ne(S.fullFields),[M]):[M]})}function O(){var M=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[],z=Array.isArray(M)?M:[M];!u.suppressWarning&&z.length&&e.warning("async-validator:",z),z.length&&S.message!==void 0&&(z=[].concat(S.message));var P=z.map(aw(S,s));if(u.first&&P.length)return C[S.field]=1,x(P);if(!E)x(P);else{if(S.required&&!b.value)return S.message!==void 0?P=[].concat(S.message).map(aw(S,s)):u.error&&(P=[u.error(S,Jr(u.messages.required,S.field))]),x(P);var D={};S.defaultField&&Object.keys(b.value).map(function(V){D[V]=S.defaultField}),D=Q(Q({},D),b.rule.fields);var F={};Object.keys(D).forEach(function(V){var U=D[V],K=Array.isArray(U)?U:[U];F[V]=K.map($.bind(null,V))});var G=new e(F);G.messages(u.messages),b.rule.options&&(b.rule.options.messages=u.messages,b.rule.options.error=u.error),G.validate(b.value,b.rule.options||u,function(V){var U=[];P&&P.length&&U.push.apply(U,Ne(P)),V&&V.length&&U.push.apply(U,Ne(V)),x(U.length?U:null)})}}var _;if(S.asyncValidator)_=S.asyncValidator(S,b.value,O,b.source,u);else if(S.validator){try{_=S.validator(S,b.value,O,b.source,u)}catch(M){var R,T;(R=(T=console).error)===null||R===void 0||R.call(T,M),u.suppressValidatorError||setTimeout(function(){throw M},0),O(M.message)}_===!0?O():_===!1?O(typeof S.message=="function"?S.message(S.fullField||S.field):S.message||"".concat(S.fullField||S.field," fails")):_ instanceof Array?O(_):_ instanceof Error&&O(_.message)}_&&_.then&&_.then(function(){return O()},function(M){return O(M)})},function(b){g(b)},s)}},{key:"getType",value:function(n){if(n.type===void 0&&n.pattern instanceof RegExp&&(n.type="pattern"),typeof n.validator!="function"&&n.type&&!Kc.hasOwnProperty(n.type))throw new Error(Jr("Unknown rule type %s",n.type));return n.type||"string"}},{key:"getValidationMethod",value:function(n){if(typeof n.validator=="function")return n.validator;var r=Object.keys(n),a=r.indexOf("message");return a!==-1&&r.splice(a,1),r.length===1&&r[0]==="required"?Kc.required:Kc[this.getType(n)]||void 0}}]),e}();Z(Tu,"register",function(t,n){if(typeof n!="function")throw new Error("Cannot register a validator by type, validator is not a function");Kc[t]=n});Z(Tu,"warning",CD);Z(Tu,"messages",a0);Z(Tu,"validators",Kc);var Qr="'${name}' is not a valid ${type}",$R={default:"Validation error on field '${name}'",required:"'${name}' is required",enum:"'${name}' must be one of [${enum}]",whitespace:"'${name}' cannot be empty",date:{format:"'${name}' is invalid for format date",parse:"'${name}' could not be parsed as date",invalid:"'${name}' is invalid date"},types:{string:Qr,method:Qr,array:Qr,object:Qr,number:Qr,date:Qr,boolean:Qr,integer:Qr,float:Qr,regexp:Qr,email:Qr,url:Qr,hex:Qr},string:{len:"'${name}' must be exactly ${len} characters",min:"'${name}' must be at least ${min} characters",max:"'${name}' cannot be longer than ${max} characters",range:"'${name}' must be between ${min} and ${max} characters"},number:{len:"'${name}' must equal ${len}",min:"'${name}' cannot be less than ${min}",max:"'${name}' cannot be greater than ${max}",range:"'${name}' must be between ${min} and ${max}"},array:{len:"'${name}' must be exactly ${len} in length",min:"'${name}' cannot be less than ${min} in length",max:"'${name}' cannot be greater than ${max} in length",range:"'${name}' must be between ${min} and ${max} in length"},pattern:{mismatch:"'${name}' does not match pattern ${pattern}"}},lw=Tu;function XD(e,t){return e.replace(/\\?\$\{\w+\}/g,function(n){if(n.startsWith("\\"))return n.slice(1);var r=n.slice(2,-1);return t[r]})}var sw="CODE_LOGIC_ERROR";function l0(e,t,n,r,a){return s0.apply(this,arguments)}function s0(){return s0=al(Nr().mark(function e(t,n,r,a,l){var s,u,d,g,h,p,y,C,b;return Nr().wrap(function(S){for(;;)switch(S.prev=S.next){case 0:return s=Q({},r),delete s.ruleIndex,lw.warning=function(){},s.validator&&(u=s.validator,s.validator=function(){try{return u.apply(void 0,arguments)}catch(E){return console.error(E),Promise.reject(sw)}}),d=null,s&&s.type==="array"&&s.defaultField&&(d=s.defaultField,delete s.defaultField),g=new lw(Z({},t,[s])),h=Wl($R,a.validateMessages),g.messages(h),p=[],S.prev=10,S.next=13,Promise.resolve(g.validate(Z({},t,n),Q({},a)));case 13:S.next=18;break;case 15:S.prev=15,S.t0=S.catch(10),S.t0.errors&&(p=S.t0.errors.map(function(E,$){var O=E.message,_=O===sw?h.default:O;return f.isValidElement(_)?f.cloneElement(_,{key:"error_".concat($)}):_}));case 18:if(!(!p.length&&d)){S.next=23;break}return S.next=21,Promise.all(n.map(function(E,$){return l0("".concat(t,".").concat($),E,d,a,l)}));case 21:return y=S.sent,S.abrupt("return",y.reduce(function(E,$){return[].concat(Ne(E),Ne($))},[]));case 23:return C=Q(Q({},r),{},{name:t,enum:(r.enum||[]).join(", ")},l),b=p.map(function(E){return typeof E=="string"?XD(E,C):E}),S.abrupt("return",b);case 26:case"end":return S.stop()}},e,null,[[10,15]])})),s0.apply(this,arguments)}function YD(e,t,n,r,a,l){var s=e.join("."),u=n.map(function(h,p){var y=h.validator,C=Q(Q({},h),{},{ruleIndex:p});return y&&(C.validator=function(b,x,S){var E=!1,$=function(){for(var R=arguments.length,T=new Array(R),M=0;M<R;M++)T[M]=arguments[M];Promise.resolve().then(function(){mr(!E,"Your validator function has already return a promise. `callback` will be ignored."),E||S.apply(void 0,T)})},O=y(b,x,$);E=O&&typeof O.then=="function"&&typeof O.catch=="function",mr(E,"`callback` is deprecated. Please return a promise instead."),E&&O.then(function(){S()}).catch(function(_){S(_||" ")})}),C}).sort(function(h,p){var y=h.warningOnly,C=h.ruleIndex,b=p.warningOnly,x=p.ruleIndex;return!!y==!!b?C-x:y?1:-1}),d;if(a===!0)d=new Promise(function(){var h=al(Nr().mark(function p(y,C){var b,x,S;return Nr().wrap(function($){for(;;)switch($.prev=$.next){case 0:b=0;case 1:if(!(b<u.length)){$.next=12;break}return x=u[b],$.next=5,l0(s,t,x,r,l);case 5:if(S=$.sent,!S.length){$.next=9;break}return C([{errors:S,rule:x}]),$.abrupt("return");case 9:b+=1,$.next=1;break;case 12:y([]);case 13:case"end":return $.stop()}},p)}));return function(p,y){return h.apply(this,arguments)}}());else{var g=u.map(function(h){return l0(s,t,h,r,l).then(function(p){return{errors:p,rule:h}})});d=(a?ZD(g):QD(g)).then(function(h){return Promise.reject(h)})}return d.catch(function(h){return h}),d}function QD(e){return c0.apply(this,arguments)}function c0(){return c0=al(Nr().mark(function e(t){return Nr().wrap(function(r){for(;;)switch(r.prev=r.next){case 0:return r.abrupt("return",Promise.all(t).then(function(a){var l,s=(l=[]).concat.apply(l,Ne(a));return s}));case 1:case"end":return r.stop()}},e)})),c0.apply(this,arguments)}function ZD(e){return u0.apply(this,arguments)}function u0(){return u0=al(Nr().mark(function e(t){var n;return Nr().wrap(function(a){for(;;)switch(a.prev=a.next){case 0:return n=0,a.abrupt("return",new Promise(function(l){t.forEach(function(s){s.then(function(u){u.errors.length&&l([u]),n+=1,n===t.length&&l([])})})}));case 2:case"end":return a.stop()}},e)})),u0.apply(this,arguments)}function Bn(e){return n0(e)}function cw(e,t){var n={};return t.forEach(function(r){var a=wa(e,r);n=xa(n,r,a)}),n}function Yl(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!1;return e&&e.some(function(r){return OR(t,r,n)})}function OR(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!1;return!e||!t||!n&&e.length!==t.length?!1:t.every(function(r,a){return e[a]===r})}function JD(e,t){if(e===t)return!0;if(!e&&t||e&&!t||!e||!t||st(e)!=="object"||st(t)!=="object")return!1;var n=Object.keys(e),r=Object.keys(t),a=new Set([].concat(n,r));return Ne(a).every(function(l){var s=e[l],u=t[l];return typeof s=="function"&&typeof u=="function"?!0:s===u})}function eL(e){var t=arguments.length<=1?void 0:arguments[1];return t&&t.target&&st(t.target)==="object"&&e in t.target?t.target[e]:t}function uw(e,t,n){var r=e.length;if(t<0||t>=r||n<0||n>=r)return e;var a=e[t],l=t-n;return l>0?[].concat(Ne(e.slice(0,n)),[a],Ne(e.slice(n,t)),Ne(e.slice(t+1,r))):l<0?[].concat(Ne(e.slice(0,t)),Ne(e.slice(t+1,n+1)),[a],Ne(e.slice(n+1,r))):e}var tL=["name"],ya=[];function Np(e,t,n,r,a,l){return typeof e=="function"?e(t,n,"source"in l?{source:l.source}:{}):r!==a}var yy=function(e){Po(n,e);var t=Ao(n);function n(r){var a;if(Wn(this,n),a=t.call(this,r),Z(jt(a),"state",{resetCount:0}),Z(jt(a),"cancelRegisterFunc",null),Z(jt(a),"mounted",!1),Z(jt(a),"touched",!1),Z(jt(a),"dirty",!1),Z(jt(a),"validatePromise",void 0),Z(jt(a),"prevValidating",void 0),Z(jt(a),"errors",ya),Z(jt(a),"warnings",ya),Z(jt(a),"cancelRegister",function(){var d=a.props,g=d.preserve,h=d.isListField,p=d.name;a.cancelRegisterFunc&&a.cancelRegisterFunc(h,g,Bn(p)),a.cancelRegisterFunc=null}),Z(jt(a),"getNamePath",function(){var d=a.props,g=d.name,h=d.fieldContext,p=h.prefixName,y=p===void 0?[]:p;return g!==void 0?[].concat(Ne(y),Ne(g)):[]}),Z(jt(a),"getRules",function(){var d=a.props,g=d.rules,h=g===void 0?[]:g,p=d.fieldContext;return h.map(function(y){return typeof y=="function"?y(p):y})}),Z(jt(a),"refresh",function(){a.mounted&&a.setState(function(d){var g=d.resetCount;return{resetCount:g+1}})}),Z(jt(a),"metaCache",null),Z(jt(a),"triggerMetaEvent",function(d){var g=a.props.onMetaChange;if(g){var h=Q(Q({},a.getMeta()),{},{destroy:d});ru(a.metaCache,h)||g(h),a.metaCache=h}else a.metaCache=null}),Z(jt(a),"onStoreChange",function(d,g,h){var p=a.props,y=p.shouldUpdate,C=p.dependencies,b=C===void 0?[]:C,x=p.onReset,S=h.store,E=a.getNamePath(),$=a.getValue(d),O=a.getValue(S),_=g&&Yl(g,E);switch(h.type==="valueUpdate"&&h.source==="external"&&!ru($,O)&&(a.touched=!0,a.dirty=!0,a.validatePromise=null,a.errors=ya,a.warnings=ya,a.triggerMetaEvent()),h.type){case"reset":if(!g||_){a.touched=!1,a.dirty=!1,a.validatePromise=void 0,a.errors=ya,a.warnings=ya,a.triggerMetaEvent(),x==null||x(),a.refresh();return}break;case"remove":{if(y&&Np(y,d,S,$,O,h)){a.reRender();return}break}case"setField":{var R=h.data;if(_){"touched"in R&&(a.touched=R.touched),"validating"in R&&!("originRCField"in R)&&(a.validatePromise=R.validating?Promise.resolve([]):null),"errors"in R&&(a.errors=R.errors||ya),"warnings"in R&&(a.warnings=R.warnings||ya),a.dirty=!0,a.triggerMetaEvent(),a.reRender();return}else if("value"in R&&Yl(g,E,!0)){a.reRender();return}if(y&&!E.length&&Np(y,d,S,$,O,h)){a.reRender();return}break}case"dependenciesUpdate":{var T=b.map(Bn);if(T.some(function(M){return Yl(h.relatedFields,M)})){a.reRender();return}break}default:if(_||(!b.length||E.length||y)&&Np(y,d,S,$,O,h)){a.reRender();return}break}y===!0&&a.reRender()}),Z(jt(a),"validateRules",function(d){var g=a.getNamePath(),h=a.getValue(),p=d||{},y=p.triggerName,C=p.validateOnly,b=C===void 0?!1:C,x=Promise.resolve().then(al(Nr().mark(function S(){var E,$,O,_,R,T,M;return Nr().wrap(function(P){for(;;)switch(P.prev=P.next){case 0:if(a.mounted){P.next=2;break}return P.abrupt("return",[]);case 2:if(E=a.props,$=E.validateFirst,O=$===void 0?!1:$,_=E.messageVariables,R=E.validateDebounce,T=a.getRules(),y&&(T=T.filter(function(D){return D}).filter(function(D){var F=D.validateTrigger;if(!F)return!0;var G=n0(F);return G.includes(y)})),!(R&&y)){P.next=10;break}return P.next=8,new Promise(function(D){setTimeout(D,R)});case 8:if(a.validatePromise===x){P.next=10;break}return P.abrupt("return",[]);case 10:return M=YD(g,h,T,d,O,_),M.catch(function(D){return D}).then(function(){var D=arguments.length>0&&arguments[0]!==void 0?arguments[0]:ya;if(a.validatePromise===x){var F;a.validatePromise=null;var G=[],V=[];(F=D.forEach)===null||F===void 0||F.call(D,function(U){var K=U.rule.warningOnly,j=U.errors,A=j===void 0?ya:j;K?V.push.apply(V,Ne(A)):G.push.apply(G,Ne(A))}),a.errors=G,a.warnings=V,a.triggerMetaEvent(),a.reRender()}}),P.abrupt("return",M);case 13:case"end":return P.stop()}},S)})));return b||(a.validatePromise=x,a.dirty=!0,a.errors=ya,a.warnings=ya,a.triggerMetaEvent(),a.reRender()),x}),Z(jt(a),"isFieldValidating",function(){return!!a.validatePromise}),Z(jt(a),"isFieldTouched",function(){return a.touched}),Z(jt(a),"isFieldDirty",function(){if(a.dirty||a.props.initialValue!==void 0)return!0;var d=a.props.fieldContext,g=d.getInternalHooks(ki),h=g.getInitialValue;return h(a.getNamePath())!==void 0}),Z(jt(a),"getErrors",function(){return a.errors}),Z(jt(a),"getWarnings",function(){return a.warnings}),Z(jt(a),"isListField",function(){return a.props.isListField}),Z(jt(a),"isList",function(){return a.props.isList}),Z(jt(a),"isPreserve",function(){return a.props.preserve}),Z(jt(a),"getMeta",function(){a.prevValidating=a.isFieldValidating();var d={touched:a.isFieldTouched(),validating:a.prevValidating,errors:a.errors,warnings:a.warnings,name:a.getNamePath(),validated:a.validatePromise===null};return d}),Z(jt(a),"getOnlyChild",function(d){if(typeof d=="function"){var g=a.getMeta();return Q(Q({},a.getOnlyChild(d(a.getControlled(),g,a.props.fieldContext))),{},{isFunction:!0})}var h=Lr(d);return h.length!==1||!f.isValidElement(h[0])?{child:h,isFunction:!1}:{child:h[0],isFunction:!1}}),Z(jt(a),"getValue",function(d){var g=a.props.fieldContext.getFieldsValue,h=a.getNamePath();return wa(d||g(!0),h)}),Z(jt(a),"getControlled",function(){var d=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},g=a.props,h=g.name,p=g.trigger,y=g.validateTrigger,C=g.getValueFromEvent,b=g.normalize,x=g.valuePropName,S=g.getValueProps,E=g.fieldContext,$=y!==void 0?y:E.validateTrigger,O=a.getNamePath(),_=E.getInternalHooks,R=E.getFieldsValue,T=_(ki),M=T.dispatch,z=a.getValue(),P=S||function(U){return Z({},x,U)},D=d[p],F=h!==void 0?P(z):{},G=Q(Q({},d),F);G[p]=function(){a.touched=!0,a.dirty=!0,a.triggerMetaEvent();for(var U,K=arguments.length,j=new Array(K),A=0;A<K;A++)j[A]=arguments[A];C?U=C.apply(void 0,j):U=eL.apply(void 0,[x].concat(j)),b&&(U=b(U,z,R(!0))),U!==z&&M({type:"updateValue",namePath:O,value:U}),D&&D.apply(void 0,j)};var V=n0($||[]);return V.forEach(function(U){var K=G[U];G[U]=function(){K&&K.apply(void 0,arguments);var j=a.props.rules;j&&j.length&&M({type:"validateField",namePath:O,triggerName:U})}}),G}),r.fieldContext){var l=r.fieldContext.getInternalHooks,s=l(ki),u=s.initEntityValue;u(jt(a))}return a}return Gn(n,[{key:"componentDidMount",value:function(){var a=this.props,l=a.shouldUpdate,s=a.fieldContext;if(this.mounted=!0,s){var u=s.getInternalHooks,d=u(ki),g=d.registerField;this.cancelRegisterFunc=g(this)}l===!0&&this.reRender()}},{key:"componentWillUnmount",value:function(){this.cancelRegister(),this.triggerMetaEvent(!0),this.mounted=!1}},{key:"reRender",value:function(){this.mounted&&this.forceUpdate()}},{key:"render",value:function(){var a=this.state.resetCount,l=this.props.children,s=this.getOnlyChild(l),u=s.child,d=s.isFunction,g;return d?g=u:f.isValidElement(u)?g=f.cloneElement(u,this.getControlled(u.props)):(mr(!u,"`children` of Field is not validate ReactElement."),g=u),f.createElement(f.Fragment,{key:a},g)}}]),n}(f.Component);Z(yy,"contextType",Ji);Z(yy,"defaultProps",{trigger:"onChange",valuePropName:"value"});function by(e){var t,n=e.name,r=$t(e,tL),a=f.useContext(Ji),l=f.useContext(su),s=n!==void 0?Bn(n):void 0,u=(t=r.isListField)!==null&&t!==void 0?t:!!l,d="keep";return u||(d="_".concat((s||[]).join("_"))),f.createElement(yy,je({key:d,name:s,isListField:u},r,{fieldContext:a}))}function RR(e){var t=e.name,n=e.initialValue,r=e.children,a=e.rules,l=e.validateTrigger,s=e.isListField,u=f.useContext(Ji),d=f.useContext(su),g=f.useRef({keys:[],id:0}),h=g.current,p=f.useMemo(function(){var x=Bn(u.prefixName)||[];return[].concat(Ne(x),Ne(Bn(t)))},[u.prefixName,t]),y=f.useMemo(function(){return Q(Q({},u),{},{prefixName:p})},[u,p]),C=f.useMemo(function(){return{getKey:function(S){var E=p.length,$=S[E];return[h.keys[$],S.slice(E+1)]}}},[p]);if(typeof r!="function")return mr(!1,"Form.List only accepts function as children."),null;var b=function(S,E,$){var O=$.source;return O==="internal"?!1:S!==E};return f.createElement(su.Provider,{value:C},f.createElement(Ji.Provider,{value:y},f.createElement(by,{name:[],shouldUpdate:b,rules:a,validateTrigger:l,initialValue:n,isList:!0,isListField:s??!!d},function(x,S){var E=x.value,$=E===void 0?[]:E,O=x.onChange,_=u.getFieldValue,R=function(){var P=_(p||[]);return P||[]},T={add:function(P,D){var F=R();D>=0&&D<=F.length?(h.keys=[].concat(Ne(h.keys.slice(0,D)),[h.id],Ne(h.keys.slice(D))),O([].concat(Ne(F.slice(0,D)),[P],Ne(F.slice(D))))):(h.keys=[].concat(Ne(h.keys),[h.id]),O([].concat(Ne(F),[P]))),h.id+=1},remove:function(P){var D=R(),F=new Set(Array.isArray(P)?P:[P]);F.size<=0||(h.keys=h.keys.filter(function(G,V){return!F.has(V)}),O(D.filter(function(G,V){return!F.has(V)})))},move:function(P,D){if(P!==D){var F=R();P<0||P>=F.length||D<0||D>=F.length||(h.keys=uw(h.keys,P,D),O(uw(F,P,D)))}}},M=$||[];return Array.isArray(M)||(M=[]),r(M.map(function(z,P){var D=h.keys[P];return D===void 0&&(h.keys[P]=h.id,D=h.keys[P],h.id+=1),{name:P,key:D,isListField:!0}}),T,S)})))}function nL(e){var t=!1,n=e.length,r=[];return e.length?new Promise(function(a,l){e.forEach(function(s,u){s.catch(function(d){return t=!0,d}).then(function(d){n-=1,r[u]=d,!(n>0)&&(t&&l(r),a(r))})})}):Promise.resolve([])}var _R="__@field_split__";function jp(e){return e.map(function(t){return"".concat(st(t),":").concat(t)}).join(_R)}var Bl=function(){function e(){Wn(this,e),Z(this,"kvs",new Map)}return Gn(e,[{key:"set",value:function(n,r){this.kvs.set(jp(n),r)}},{key:"get",value:function(n){return this.kvs.get(jp(n))}},{key:"update",value:function(n,r){var a=this.get(n),l=r(a);l?this.set(n,l):this.delete(n)}},{key:"delete",value:function(n){this.kvs.delete(jp(n))}},{key:"map",value:function(n){return Ne(this.kvs.entries()).map(function(r){var a=ge(r,2),l=a[0],s=a[1],u=l.split(_R);return n({key:u.map(function(d){var g=d.match(/^([^:]*):(.*)$/),h=ge(g,3),p=h[1],y=h[2];return p==="number"?Number(y):y}),value:s})})}},{key:"toJSON",value:function(){var n={};return this.map(function(r){var a=r.key,l=r.value;return n[a.join(".")]=l,null}),n}}]),e}(),rL=["name"],aL=Gn(function e(t){var n=this;Wn(this,e),Z(this,"formHooked",!1),Z(this,"forceRootUpdate",void 0),Z(this,"subscribable",!0),Z(this,"store",{}),Z(this,"fieldEntities",[]),Z(this,"initialValues",{}),Z(this,"callbacks",{}),Z(this,"validateMessages",null),Z(this,"preserve",null),Z(this,"lastValidatePromise",null),Z(this,"getForm",function(){return{getFieldValue:n.getFieldValue,getFieldsValue:n.getFieldsValue,getFieldError:n.getFieldError,getFieldWarning:n.getFieldWarning,getFieldsError:n.getFieldsError,isFieldsTouched:n.isFieldsTouched,isFieldTouched:n.isFieldTouched,isFieldValidating:n.isFieldValidating,isFieldsValidating:n.isFieldsValidating,resetFields:n.resetFields,setFields:n.setFields,setFieldValue:n.setFieldValue,setFieldsValue:n.setFieldsValue,validateFields:n.validateFields,submit:n.submit,_init:!0,getInternalHooks:n.getInternalHooks}}),Z(this,"getInternalHooks",function(r){return r===ki?(n.formHooked=!0,{dispatch:n.dispatch,initEntityValue:n.initEntityValue,registerField:n.registerField,useSubscribe:n.useSubscribe,setInitialValues:n.setInitialValues,destroyForm:n.destroyForm,setCallbacks:n.setCallbacks,setValidateMessages:n.setValidateMessages,getFields:n.getFields,setPreserve:n.setPreserve,getInitialValue:n.getInitialValue,registerWatch:n.registerWatch}):(mr(!1,"`getInternalHooks` is internal usage. Should not call directly."),null)}),Z(this,"useSubscribe",function(r){n.subscribable=r}),Z(this,"prevWithoutPreserves",null),Z(this,"setInitialValues",function(r,a){if(n.initialValues=r||{},a){var l,s=Wl(r,n.store);(l=n.prevWithoutPreserves)===null||l===void 0||l.map(function(u){var d=u.key;s=xa(s,d,wa(r,d))}),n.prevWithoutPreserves=null,n.updateStore(s)}}),Z(this,"destroyForm",function(r){if(r)n.updateStore({});else{var a=new Bl;n.getFieldEntities(!0).forEach(function(l){n.isMergedPreserve(l.isPreserve())||a.set(l.getNamePath(),!0)}),n.prevWithoutPreserves=a}}),Z(this,"getInitialValue",function(r){var a=wa(n.initialValues,r);return r.length?Wl(a):a}),Z(this,"setCallbacks",function(r){n.callbacks=r}),Z(this,"setValidateMessages",function(r){n.validateMessages=r}),Z(this,"setPreserve",function(r){n.preserve=r}),Z(this,"watchList",[]),Z(this,"registerWatch",function(r){return n.watchList.push(r),function(){n.watchList=n.watchList.filter(function(a){return a!==r})}}),Z(this,"notifyWatch",function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[];if(n.watchList.length){var a=n.getFieldsValue(),l=n.getFieldsValue(!0);n.watchList.forEach(function(s){s(a,l,r)})}}),Z(this,"timeoutId",null),Z(this,"warningUnhooked",function(){}),Z(this,"updateStore",function(r){n.store=r}),Z(this,"getFieldEntities",function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1;return r?n.fieldEntities.filter(function(a){return a.getNamePath().length}):n.fieldEntities}),Z(this,"getFieldsMap",function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1,a=new Bl;return n.getFieldEntities(r).forEach(function(l){var s=l.getNamePath();a.set(s,l)}),a}),Z(this,"getFieldEntitiesForNamePathList",function(r){if(!r)return n.getFieldEntities(!0);var a=n.getFieldsMap(!0);return r.map(function(l){var s=Bn(l);return a.get(s)||{INVALIDATE_NAME_PATH:Bn(l)}})}),Z(this,"getFieldsValue",function(r,a){n.warningUnhooked();var l,s,u;if(r===!0||Array.isArray(r)?(l=r,s=a):r&&st(r)==="object"&&(u=r.strict,s=r.filter),l===!0&&!s)return n.store;var d=n.getFieldEntitiesForNamePathList(Array.isArray(l)?l:null),g=[];return d.forEach(function(h){var p,y,C="INVALIDATE_NAME_PATH"in h?h.INVALIDATE_NAME_PATH:h.getNamePath();if(u){var b,x;if((b=(x=h).isList)!==null&&b!==void 0&&b.call(x))return}else if(!l&&(p=(y=h).isListField)!==null&&p!==void 0&&p.call(y))return;if(!s)g.push(C);else{var S="getMeta"in h?h.getMeta():null;s(S)&&g.push(C)}}),cw(n.store,g.map(Bn))}),Z(this,"getFieldValue",function(r){n.warningUnhooked();var a=Bn(r);return wa(n.store,a)}),Z(this,"getFieldsError",function(r){n.warningUnhooked();var a=n.getFieldEntitiesForNamePathList(r);return a.map(function(l,s){return l&&!("INVALIDATE_NAME_PATH"in l)?{name:l.getNamePath(),errors:l.getErrors(),warnings:l.getWarnings()}:{name:Bn(r[s]),errors:[],warnings:[]}})}),Z(this,"getFieldError",function(r){n.warningUnhooked();var a=Bn(r),l=n.getFieldsError([a])[0];return l.errors}),Z(this,"getFieldWarning",function(r){n.warningUnhooked();var a=Bn(r),l=n.getFieldsError([a])[0];return l.warnings}),Z(this,"isFieldsTouched",function(){n.warningUnhooked();for(var r=arguments.length,a=new Array(r),l=0;l<r;l++)a[l]=arguments[l];var s=a[0],u=a[1],d,g=!1;a.length===0?d=null:a.length===1?Array.isArray(s)?(d=s.map(Bn),g=!1):(d=null,g=s):(d=s.map(Bn),g=u);var h=n.getFieldEntities(!0),p=function(S){return S.isFieldTouched()};if(!d)return g?h.every(function(x){return p(x)||x.isList()}):h.some(p);var y=new Bl;d.forEach(function(x){y.set(x,[])}),h.forEach(function(x){var S=x.getNamePath();d.forEach(function(E){E.every(function($,O){return S[O]===$})&&y.update(E,function($){return[].concat(Ne($),[x])})})});var C=function(S){return S.some(p)},b=y.map(function(x){var S=x.value;return S});return g?b.every(C):b.some(C)}),Z(this,"isFieldTouched",function(r){return n.warningUnhooked(),n.isFieldsTouched([r])}),Z(this,"isFieldsValidating",function(r){n.warningUnhooked();var a=n.getFieldEntities();if(!r)return a.some(function(s){return s.isFieldValidating()});var l=r.map(Bn);return a.some(function(s){var u=s.getNamePath();return Yl(l,u)&&s.isFieldValidating()})}),Z(this,"isFieldValidating",function(r){return n.warningUnhooked(),n.isFieldsValidating([r])}),Z(this,"resetWithFieldInitialValue",function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},a=new Bl,l=n.getFieldEntities(!0);l.forEach(function(d){var g=d.props.initialValue,h=d.getNamePath();if(g!==void 0){var p=a.get(h)||new Set;p.add({entity:d,value:g}),a.set(h,p)}});var s=function(g){g.forEach(function(h){var p=h.props.initialValue;if(p!==void 0){var y=h.getNamePath(),C=n.getInitialValue(y);if(C!==void 0)mr(!1,"Form already set 'initialValues' with path '".concat(y.join("."),"'. Field can not overwrite it."));else{var b=a.get(y);if(b&&b.size>1)mr(!1,"Multiple Field with path '".concat(y.join("."),"' set 'initialValue'. Can not decide which one to pick."));else if(b){var x=n.getFieldValue(y),S=h.isListField();!S&&(!r.skipExist||x===void 0)&&n.updateStore(xa(n.store,y,Ne(b)[0].value))}}}})},u;r.entities?u=r.entities:r.namePathList?(u=[],r.namePathList.forEach(function(d){var g=a.get(d);if(g){var h;(h=u).push.apply(h,Ne(Ne(g).map(function(p){return p.entity})))}})):u=l,s(u)}),Z(this,"resetFields",function(r){n.warningUnhooked();var a=n.store;if(!r){n.updateStore(Wl(n.initialValues)),n.resetWithFieldInitialValue(),n.notifyObservers(a,null,{type:"reset"}),n.notifyWatch();return}var l=r.map(Bn);l.forEach(function(s){var u=n.getInitialValue(s);n.updateStore(xa(n.store,s,u))}),n.resetWithFieldInitialValue({namePathList:l}),n.notifyObservers(a,l,{type:"reset"}),n.notifyWatch(l)}),Z(this,"setFields",function(r){n.warningUnhooked();var a=n.store,l=[];r.forEach(function(s){var u=s.name,d=$t(s,rL),g=Bn(u);l.push(g),"value"in d&&n.updateStore(xa(n.store,g,d.value)),n.notifyObservers(a,[g],{type:"setField",data:s})}),n.notifyWatch(l)}),Z(this,"getFields",function(){var r=n.getFieldEntities(!0),a=r.map(function(l){var s=l.getNamePath(),u=l.getMeta(),d=Q(Q({},u),{},{name:s,value:n.getFieldValue(s)});return Object.defineProperty(d,"originRCField",{value:!0}),d});return a}),Z(this,"initEntityValue",function(r){var a=r.props.initialValue;if(a!==void 0){var l=r.getNamePath(),s=wa(n.store,l);s===void 0&&n.updateStore(xa(n.store,l,a))}}),Z(this,"isMergedPreserve",function(r){var a=r!==void 0?r:n.preserve;return a??!0}),Z(this,"registerField",function(r){n.fieldEntities.push(r);var a=r.getNamePath();if(n.notifyWatch([a]),r.props.initialValue!==void 0){var l=n.store;n.resetWithFieldInitialValue({entities:[r],skipExist:!0}),n.notifyObservers(l,[r.getNamePath()],{type:"valueUpdate",source:"internal"})}return function(s,u){var d=arguments.length>2&&arguments[2]!==void 0?arguments[2]:[];if(n.fieldEntities=n.fieldEntities.filter(function(p){return p!==r}),!n.isMergedPreserve(u)&&(!s||d.length>1)){var g=s?void 0:n.getInitialValue(a);if(a.length&&n.getFieldValue(a)!==g&&n.fieldEntities.every(function(p){return!OR(p.getNamePath(),a)})){var h=n.store;n.updateStore(xa(h,a,g,!0)),n.notifyObservers(h,[a],{type:"remove"}),n.triggerDependenciesUpdate(h,a)}}n.notifyWatch([a])}}),Z(this,"dispatch",function(r){switch(r.type){case"updateValue":{var a=r.namePath,l=r.value;n.updateValue(a,l);break}case"validateField":{var s=r.namePath,u=r.triggerName;n.validateFields([s],{triggerName:u});break}}}),Z(this,"notifyObservers",function(r,a,l){if(n.subscribable){var s=Q(Q({},l),{},{store:n.getFieldsValue(!0)});n.getFieldEntities().forEach(function(u){var d=u.onStoreChange;d(r,a,s)})}else n.forceRootUpdate()}),Z(this,"triggerDependenciesUpdate",function(r,a){var l=n.getDependencyChildrenFields(a);return l.length&&n.validateFields(l),n.notifyObservers(r,l,{type:"dependenciesUpdate",relatedFields:[a].concat(Ne(l))}),l}),Z(this,"updateValue",function(r,a){var l=Bn(r),s=n.store;n.updateStore(xa(n.store,l,a)),n.notifyObservers(s,[l],{type:"valueUpdate",source:"internal"}),n.notifyWatch([l]);var u=n.triggerDependenciesUpdate(s,l),d=n.callbacks.onValuesChange;if(d){var g=cw(n.store,[l]);d(g,n.getFieldsValue())}n.triggerOnFieldsChange([l].concat(Ne(u)))}),Z(this,"setFieldsValue",function(r){n.warningUnhooked();var a=n.store;if(r){var l=Wl(n.store,r);n.updateStore(l)}n.notifyObservers(a,null,{type:"valueUpdate",source:"external"}),n.notifyWatch()}),Z(this,"setFieldValue",function(r,a){n.setFields([{name:r,value:a,errors:[],warnings:[]}])}),Z(this,"getDependencyChildrenFields",function(r){var a=new Set,l=[],s=new Bl;n.getFieldEntities().forEach(function(d){var g=d.props.dependencies;(g||[]).forEach(function(h){var p=Bn(h);s.update(p,function(){var y=arguments.length>0&&arguments[0]!==void 0?arguments[0]:new Set;return y.add(d),y})})});var u=function d(g){var h=s.get(g)||new Set;h.forEach(function(p){if(!a.has(p)){a.add(p);var y=p.getNamePath();p.isFieldDirty()&&y.length&&(l.push(y),d(y))}})};return u(r),l}),Z(this,"triggerOnFieldsChange",function(r,a){var l=n.callbacks.onFieldsChange;if(l){var s=n.getFields();if(a){var u=new Bl;a.forEach(function(g){var h=g.name,p=g.errors;u.set(h,p)}),s.forEach(function(g){g.errors=u.get(g.name)||g.errors})}var d=s.filter(function(g){var h=g.name;return Yl(r,h)});d.length&&l(d,s)}}),Z(this,"validateFields",function(r,a){n.warningUnhooked();var l,s;Array.isArray(r)||typeof r=="string"||typeof a=="string"?(l=r,s=a):s=r;var u=!!l,d=u?l.map(Bn):[],g=[],h=String(Date.now()),p=new Set,y=s||{},C=y.recursive,b=y.dirty;n.getFieldEntities(!0).forEach(function($){if(u||d.push($.getNamePath()),!(!$.props.rules||!$.props.rules.length)&&!(b&&!$.isFieldDirty())){var O=$.getNamePath();if(p.add(O.join(h)),!u||Yl(d,O,C)){var _=$.validateRules(Q({validateMessages:Q(Q({},$R),n.validateMessages)},s));g.push(_.then(function(){return{name:O,errors:[],warnings:[]}}).catch(function(R){var T,M=[],z=[];return(T=R.forEach)===null||T===void 0||T.call(R,function(P){var D=P.rule.warningOnly,F=P.errors;D?z.push.apply(z,Ne(F)):M.push.apply(M,Ne(F))}),M.length?Promise.reject({name:O,errors:M,warnings:z}):{name:O,errors:M,warnings:z}}))}}});var x=nL(g);n.lastValidatePromise=x,x.catch(function($){return $}).then(function($){var O=$.map(function(_){var R=_.name;return R});n.notifyObservers(n.store,O,{type:"validateFinish"}),n.triggerOnFieldsChange(O,$)});var S=x.then(function(){return n.lastValidatePromise===x?Promise.resolve(n.getFieldsValue(d)):Promise.reject([])}).catch(function($){var O=$.filter(function(_){return _&&_.errors.length});return Promise.reject({values:n.getFieldsValue(d),errorFields:O,outOfDate:n.lastValidatePromise!==x})});S.catch(function($){return $});var E=d.filter(function($){return p.has($.join(h))});return n.triggerOnFieldsChange(E),S}),Z(this,"submit",function(){n.warningUnhooked(),n.validateFields().then(function(r){var a=n.callbacks.onFinish;if(a)try{a(r)}catch(l){console.error(l)}}).catch(function(r){var a=n.callbacks.onFinishFailed;a&&a(r)})}),this.forceRootUpdate=t});function Sy(e){var t=f.useRef(),n=f.useState({}),r=ge(n,2),a=r[1];if(!t.current)if(e)t.current=e;else{var l=function(){a({})},s=new aL(l);t.current=s.getForm()}return[t.current]}var f0=f.createContext({triggerFormChange:function(){},triggerFormFinish:function(){},registerForm:function(){},unregisterForm:function(){}}),TR=function(t){var n=t.validateMessages,r=t.onFormChange,a=t.onFormFinish,l=t.children,s=f.useContext(f0),u=f.useRef({});return f.createElement(f0.Provider,{value:Q(Q({},s),{},{validateMessages:Q(Q({},s.validateMessages),n),triggerFormChange:function(g,h){r&&r(g,{changedFields:h,forms:u.current}),s.triggerFormChange(g,h)},triggerFormFinish:function(g,h){a&&a(g,{values:h,forms:u.current}),s.triggerFormFinish(g,h)},registerForm:function(g,h){g&&(u.current=Q(Q({},u.current),{},Z({},g,h))),s.registerForm(g,h)},unregisterForm:function(g){var h=Q({},u.current);delete h[g],u.current=h,s.unregisterForm(g)}})},l)},oL=["name","initialValues","fields","form","preserve","children","component","validateMessages","validateTrigger","onValuesChange","onFieldsChange","onFinish","onFinishFailed","clearOnDestroy"],iL=function(t,n){var r=t.name,a=t.initialValues,l=t.fields,s=t.form,u=t.preserve,d=t.children,g=t.component,h=g===void 0?"form":g,p=t.validateMessages,y=t.validateTrigger,C=y===void 0?"onChange":y,b=t.onValuesChange,x=t.onFieldsChange,S=t.onFinish,E=t.onFinishFailed,$=t.clearOnDestroy,O=$t(t,oL),_=f.useRef(null),R=f.useContext(f0),T=Sy(s),M=ge(T,1),z=M[0],P=z.getInternalHooks(ki),D=P.useSubscribe,F=P.setInitialValues,G=P.setCallbacks,V=P.setValidateMessages,U=P.setPreserve,K=P.destroyForm;f.useImperativeHandle(n,function(){return Q(Q({},z),{},{nativeElement:_.current})}),f.useEffect(function(){return R.registerForm(r,z),function(){R.unregisterForm(r)}},[R,z,r]),V(Q(Q({},R.validateMessages),p)),G({onValuesChange:b,onFieldsChange:function(k){if(R.triggerFormChange(r,k),x){for(var Y=arguments.length,ee=new Array(Y>1?Y-1:0),ue=1;ue<Y;ue++)ee[ue-1]=arguments[ue];x.apply(void 0,[k].concat(ee))}},onFinish:function(k){R.triggerFormFinish(r,k),S&&S(k)},onFinishFailed:E}),U(u);var j=f.useRef(null);F(a,!j.current),j.current||(j.current=!0),f.useEffect(function(){return function(){return K($)}},[]);var A,L=typeof d=="function";if(L){var q=z.getFieldsValue(!0);A=d(q,z)}else A=d;D(!L);var N=f.useRef();f.useEffect(function(){JD(N.current||[],l||[])||z.setFields(l||[]),N.current=l},[l,z]);var B=f.useMemo(function(){return Q(Q({},z),{},{validateTrigger:C})},[z,C]),W=f.createElement(su.Provider,{value:null},f.createElement(Ji.Provider,{value:B},A));return h===!1?W:f.createElement(h,je({},O,{ref:_,onSubmit:function(k){k.preventDefault(),k.stopPropagation(),z.submit()},onReset:function(k){var Y;k.preventDefault(),z.resetFields(),(Y=O.onReset)===null||Y===void 0||Y.call(O,k)}}),W)};function fw(e){try{return JSON.stringify(e)}catch{return Math.random()}}function MR(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];var r=t[0],a=t[1],l=a===void 0?{}:a,s=vD(l)?{form:l}:l,u=s.form,d=f.useState(),g=ge(d,2),h=g[0],p=g[1],y=f.useMemo(function(){return fw(h)},[h]),C=f.useRef(y);C.current=y;var b=f.useContext(Ji),x=u||b,S=x&&x._init,E=Bn(r),$=f.useRef(E);return $.current=E,f.useEffect(function(){if(S){var O=x.getFieldsValue,_=x.getInternalHooks,R=_(ki),T=R.registerWatch,M=function(F,G){var V=s.preserve?G:F;return typeof r=="function"?r(V):wa(V,$.current)},z=T(function(D,F){var G=M(D,F),V=fw(G);C.current!==V&&(C.current=V,p(G))}),P=M(O(),O(!0));return h!==P&&p(P),z}},[S]),h}var lL=f.forwardRef(iL),ps=lL;ps.FormProvider=TR;ps.Field=by;ps.List=RR;ps.useForm=Sy;ps.useWatch=MR;const To=f.createContext({labelAlign:"right",vertical:!1,itemRef:()=>{}}),NR=f.createContext(null),jR=e=>{const t=On(e,["prefixCls"]);return f.createElement(TR,Object.assign({},t))},Cy=f.createContext({prefixCls:""}),ra=f.createContext({}),sL=e=>{let{children:t,status:n,override:r}=e;const a=f.useContext(ra),l=f.useMemo(()=>{const s=Object.assign({},a);return r&&delete s.isFormItemInput,n&&(delete s.status,delete s.hasFeedback,delete s.feedbackIcon),s},[n,r,a]);return f.createElement(ra.Provider,{value:l},t)},IR=f.createContext(void 0),ns=e=>{const{space:t,form:n,children:r}=e;if(r==null)return null;let a=r;return n&&(a=se.createElement(sL,{override:!0,status:!0},a)),t&&(a=se.createElement(Cz,null,a)),a};function rs(e){if(e)return{closable:e.closable,closeIcon:e.closeIcon}}function dw(e){const{closable:t,closeIcon:n}=e||{};return se.useMemo(()=>{if(!t&&(t===!1||n===!1||n===null))return!1;if(t===void 0&&n===void 0)return null;let r={closeIcon:typeof n!="boolean"&&n!==null?n:void 0};return t&&typeof t=="object"&&(r=Object.assign(Object.assign({},r),t)),r},[t,n])}function mw(){const e={};for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return n.forEach(a=>{a&&Object.keys(a).forEach(l=>{a[l]!==void 0&&(e[l]=a[l])})}),e}const cL={};function xy(e,t){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:cL;const r=dw(e),a=dw(t),l=typeof r!="boolean"?!!(r!=null&&r.disabled):!1,s=se.useMemo(()=>Object.assign({closeIcon:se.createElement(cs,null)},n),[n]),u=se.useMemo(()=>r===!1?!1:r?mw(s,a,r):a===!1?!1:a?mw(s,a):s.closable?s:!1,[r,a,s]);return se.useMemo(()=>{if(u===!1)return[!1,null,l];const{closeIconRender:d}=s,{closeIcon:g}=u;let h=g;if(h!=null){d&&(h=d(g));const p=ta(u,!0);Object.keys(p).length&&(h=se.isValidElement(h)?se.cloneElement(h,p):se.createElement("span",Object.assign({},p),h))}return[!0,h,l]},[u,s])}var PR=function(t){if($r()&&window.document.documentElement){var n=Array.isArray(t)?t:[t],r=window.document.documentElement;return n.some(function(a){return a in r.style})}return!1},uL=function(t,n){if(!PR(t))return!1;var r=document.createElement("div"),a=r.style[t];return r.style[t]=n,r.style[t]!==a};function gw(e,t){return!Array.isArray(e)&&t!==void 0?uL(e,t):PR(e)}const fL=()=>$r()&&window.document.documentElement,_m=e=>{const{prefixCls:t,className:n,style:r,size:a,shape:l}=e,s=re({[`${t}-lg`]:a==="large",[`${t}-sm`]:a==="small"}),u=re({[`${t}-circle`]:l==="circle",[`${t}-square`]:l==="square",[`${t}-round`]:l==="round"}),d=f.useMemo(()=>typeof a=="number"?{width:a,height:a,lineHeight:`${a}px`}:{},[a]);return f.createElement("span",{className:re(t,s,u,n),style:Object.assign(Object.assign({},d),r)})},dL=new Dt("ant-skeleton-loading",{"0%":{backgroundPosition:"100% 50%"},"100%":{backgroundPosition:"0 50%"}}),Tm=e=>({height:e,lineHeight:ve(e)}),Ql=e=>Object.assign({width:e},Tm(e)),mL=e=>({background:e.skeletonLoadingBackground,backgroundSize:"400% 100%",animationName:dL,animationDuration:e.skeletonLoadingMotionDuration,animationTimingFunction:"ease",animationIterationCount:"infinite"}),Ip=(e,t)=>Object.assign({width:t(e).mul(5).equal(),minWidth:t(e).mul(5).equal()},Tm(e)),gL=e=>{const{skeletonAvatarCls:t,gradientFromColor:n,controlHeight:r,controlHeightLG:a,controlHeightSM:l}=e;return{[t]:Object.assign({display:"inline-block",verticalAlign:"top",background:n},Ql(r)),[`${t}${t}-circle`]:{borderRadius:"50%"},[`${t}${t}-lg`]:Object.assign({},Ql(a)),[`${t}${t}-sm`]:Object.assign({},Ql(l))}},hL=e=>{const{controlHeight:t,borderRadiusSM:n,skeletonInputCls:r,controlHeightLG:a,controlHeightSM:l,gradientFromColor:s,calc:u}=e;return{[r]:Object.assign({display:"inline-block",verticalAlign:"top",background:s,borderRadius:n},Ip(t,u)),[`${r}-lg`]:Object.assign({},Ip(a,u)),[`${r}-sm`]:Object.assign({},Ip(l,u))}},hw=e=>Object.assign({width:e},Tm(e)),pL=e=>{const{skeletonImageCls:t,imageSizeBase:n,gradientFromColor:r,borderRadiusSM:a,calc:l}=e;return{[t]:Object.assign(Object.assign({display:"inline-flex",alignItems:"center",justifyContent:"center",verticalAlign:"middle",background:r,borderRadius:a},hw(l(n).mul(2).equal())),{[`${t}-path`]:{fill:"#bfbfbf"},[`${t}-svg`]:Object.assign(Object.assign({},hw(n)),{maxWidth:l(n).mul(4).equal(),maxHeight:l(n).mul(4).equal()}),[`${t}-svg${t}-svg-circle`]:{borderRadius:"50%"}}),[`${t}${t}-circle`]:{borderRadius:"50%"}}},Pp=(e,t,n)=>{const{skeletonButtonCls:r}=e;return{[`${n}${r}-circle`]:{width:t,minWidth:t,borderRadius:"50%"},[`${n}${r}-round`]:{borderRadius:t}}},Ap=(e,t)=>Object.assign({width:t(e).mul(2).equal(),minWidth:t(e).mul(2).equal()},Tm(e)),vL=e=>{const{borderRadiusSM:t,skeletonButtonCls:n,controlHeight:r,controlHeightLG:a,controlHeightSM:l,gradientFromColor:s,calc:u}=e;return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({[n]:Object.assign({display:"inline-block",verticalAlign:"top",background:s,borderRadius:t,width:u(r).mul(2).equal(),minWidth:u(r).mul(2).equal()},Ap(r,u))},Pp(e,r,n)),{[`${n}-lg`]:Object.assign({},Ap(a,u))}),Pp(e,a,`${n}-lg`)),{[`${n}-sm`]:Object.assign({},Ap(l,u))}),Pp(e,l,`${n}-sm`))},yL=e=>{const{componentCls:t,skeletonAvatarCls:n,skeletonTitleCls:r,skeletonParagraphCls:a,skeletonButtonCls:l,skeletonInputCls:s,skeletonImageCls:u,controlHeight:d,controlHeightLG:g,controlHeightSM:h,gradientFromColor:p,padding:y,marginSM:C,borderRadius:b,titleHeight:x,blockRadius:S,paragraphLiHeight:E,controlHeightXS:$,paragraphMarginTop:O}=e;return{[t]:{display:"table",width:"100%",[`${t}-header`]:{display:"table-cell",paddingInlineEnd:y,verticalAlign:"top",[n]:Object.assign({display:"inline-block",verticalAlign:"top",background:p},Ql(d)),[`${n}-circle`]:{borderRadius:"50%"},[`${n}-lg`]:Object.assign({},Ql(g)),[`${n}-sm`]:Object.assign({},Ql(h))},[`${t}-content`]:{display:"table-cell",width:"100%",verticalAlign:"top",[r]:{width:"100%",height:x,background:p,borderRadius:S,[`+ ${a}`]:{marginBlockStart:h}},[a]:{padding:0,"> li":{width:"100%",height:E,listStyle:"none",background:p,borderRadius:S,"+ li":{marginBlockStart:$}}},[`${a}> li:last-child:not(:first-child):not(:nth-child(2))`]:{width:"61%"}},[`&-round ${t}-content`]:{[`${r}, ${a} > li`]:{borderRadius:b}}},[`${t}-with-avatar ${t}-content`]:{[r]:{marginBlockStart:C,[`+ ${a}`]:{marginBlockStart:O}}},[`${t}${t}-element`]:Object.assign(Object.assign(Object.assign(Object.assign({display:"inline-block",width:"auto"},vL(e)),gL(e)),hL(e)),pL(e)),[`${t}${t}-block`]:{width:"100%",[l]:{width:"100%"},[s]:{width:"100%"}},[`${t}${t}-active`]:{[`
        ${r},
        ${a} > li,
        ${n},
        ${l},
        ${s},
        ${u}
      `]:Object.assign({},mL(e))}}},bL=e=>{const{colorFillContent:t,colorFill:n}=e,r=t,a=n;return{color:r,colorGradientEnd:a,gradientFromColor:r,gradientToColor:a,titleHeight:e.controlHeight/2,blockRadius:e.borderRadiusSM,paragraphMarginTop:e.marginLG+e.marginXXS,paragraphLiHeight:e.controlHeight/2}},vs=xn("Skeleton",e=>{const{componentCls:t,calc:n}=e,r=Kt(e,{skeletonAvatarCls:`${t}-avatar`,skeletonTitleCls:`${t}-title`,skeletonParagraphCls:`${t}-paragraph`,skeletonButtonCls:`${t}-button`,skeletonInputCls:`${t}-input`,skeletonImageCls:`${t}-image`,imageSizeBase:n(e.controlHeight).mul(1.5).equal(),borderRadius:100,skeletonLoadingBackground:`linear-gradient(90deg, ${e.gradientFromColor} 25%, ${e.gradientToColor} 37%, ${e.gradientFromColor} 63%)`,skeletonLoadingMotionDuration:"1.4s"});return[yL(r)]},bL,{deprecatedTokens:[["color","gradientFromColor"],["colorGradientEnd","gradientToColor"]]}),SL=e=>{const{prefixCls:t,className:n,rootClassName:r,active:a,shape:l="circle",size:s="default"}=e,{getPrefixCls:u}=f.useContext(dt),d=u("skeleton",t),[g,h,p]=vs(d),y=On(e,["prefixCls","className"]),C=re(d,`${d}-element`,{[`${d}-active`]:a},n,r,h,p);return g(f.createElement("div",{className:C},f.createElement(_m,Object.assign({prefixCls:`${d}-avatar`,shape:l,size:s},y))))},CL=e=>{const{prefixCls:t,className:n,rootClassName:r,active:a,block:l=!1,size:s="default"}=e,{getPrefixCls:u}=f.useContext(dt),d=u("skeleton",t),[g,h,p]=vs(d),y=On(e,["prefixCls"]),C=re(d,`${d}-element`,{[`${d}-active`]:a,[`${d}-block`]:l},n,r,h,p);return g(f.createElement("div",{className:C},f.createElement(_m,Object.assign({prefixCls:`${d}-button`,size:s},y))))},xL="M365.714286 329.142857q0 45.714286-32.036571 77.677714t-77.677714 32.036571-77.677714-32.036571-32.036571-77.677714 32.036571-77.677714 77.677714-32.036571 77.677714 32.036571 32.036571 77.677714zM950.857143 548.571429l0 256-804.571429 0 0-109.714286 182.857143-182.857143 91.428571 91.428571 292.571429-292.571429zM1005.714286 146.285714l-914.285714 0q-7.460571 0-12.873143 5.412571t-5.412571 12.873143l0 694.857143q0 7.460571 5.412571 12.873143t12.873143 5.412571l914.285714 0q7.460571 0 12.873143-5.412571t5.412571-12.873143l0-694.857143q0-7.460571-5.412571-12.873143t-12.873143-5.412571zM1097.142857 164.571429l0 694.857143q0 37.741714-26.843429 64.585143t-64.585143 26.843429l-914.285714 0q-37.741714 0-64.585143-26.843429t-26.843429-64.585143l0-694.857143q0-37.741714 26.843429-64.585143t64.585143-26.843429l914.285714 0q37.741714 0 64.585143 26.843429t26.843429 64.585143z",wL=e=>{const{prefixCls:t,className:n,rootClassName:r,style:a,active:l}=e,{getPrefixCls:s}=f.useContext(dt),u=s("skeleton",t),[d,g,h]=vs(u),p=re(u,`${u}-element`,{[`${u}-active`]:l},n,r,g,h);return d(f.createElement("div",{className:p},f.createElement("div",{className:re(`${u}-image`,n),style:a},f.createElement("svg",{viewBox:"0 0 1098 1024",xmlns:"http://www.w3.org/2000/svg",className:`${u}-image-svg`},f.createElement("title",null,"Image placeholder"),f.createElement("path",{d:xL,className:`${u}-image-path`})))))},EL=e=>{const{prefixCls:t,className:n,rootClassName:r,active:a,block:l,size:s="default"}=e,{getPrefixCls:u}=f.useContext(dt),d=u("skeleton",t),[g,h,p]=vs(d),y=On(e,["prefixCls"]),C=re(d,`${d}-element`,{[`${d}-active`]:a,[`${d}-block`]:l},n,r,h,p);return g(f.createElement("div",{className:C},f.createElement(_m,Object.assign({prefixCls:`${d}-input`,size:s},y))))},$L=e=>{const{prefixCls:t,className:n,rootClassName:r,style:a,active:l,children:s}=e,{getPrefixCls:u}=f.useContext(dt),d=u("skeleton",t),[g,h,p]=vs(d),y=re(d,`${d}-element`,{[`${d}-active`]:l},h,n,r,p);return g(f.createElement("div",{className:y},f.createElement("div",{className:re(`${d}-image`,n),style:a},s)))},OL=(e,t)=>{const{width:n,rows:r=2}=t;if(Array.isArray(n))return n[e];if(r-1===e)return n},RL=e=>{const{prefixCls:t,className:n,style:r,rows:a=0}=e,l=Array.from({length:a}).map((s,u)=>f.createElement("li",{key:u,style:{width:OL(u,e)}}));return f.createElement("ul",{className:re(t,n),style:r},l)},_L=e=>{let{prefixCls:t,className:n,width:r,style:a}=e;return f.createElement("h3",{className:re(t,n),style:Object.assign({width:r},a)})};function zp(e){return e&&typeof e=="object"?e:{}}function TL(e,t){return e&&!t?{size:"large",shape:"square"}:{size:"large",shape:"circle"}}function ML(e,t){return!e&&t?{width:"38%"}:e&&t?{width:"50%"}:{}}function NL(e,t){const n={};return(!e||!t)&&(n.width="61%"),!e&&t?n.rows=3:n.rows=2,n}const Bo=e=>{const{prefixCls:t,loading:n,className:r,rootClassName:a,style:l,children:s,avatar:u=!1,title:d=!0,paragraph:g=!0,active:h,round:p}=e,{getPrefixCls:y,direction:C,className:b,style:x}=Or("skeleton"),S=y("skeleton",t),[E,$,O]=vs(S);if(n||!("loading"in e)){const _=!!u,R=!!d,T=!!g;let M;if(_){const D=Object.assign(Object.assign({prefixCls:`${S}-avatar`},TL(R,T)),zp(u));M=f.createElement("div",{className:`${S}-header`},f.createElement(_m,Object.assign({},D)))}let z;if(R||T){let D;if(R){const G=Object.assign(Object.assign({prefixCls:`${S}-title`},ML(_,T)),zp(d));D=f.createElement(_L,Object.assign({},G))}let F;if(T){const G=Object.assign(Object.assign({prefixCls:`${S}-paragraph`},NL(_,R)),zp(g));F=f.createElement(RL,Object.assign({},G))}z=f.createElement("div",{className:`${S}-content`},D,F)}const P=re(S,{[`${S}-with-avatar`]:_,[`${S}-active`]:h,[`${S}-rtl`]:C==="rtl",[`${S}-round`]:p},b,r,a,$,O);return E(f.createElement("div",{className:P,style:Object.assign(Object.assign({},x),l)},M,z))}return s??null};Bo.Button=CL;Bo.Avatar=SL;Bo.Input=EL;Bo.Image=wL;Bo.Node=$L;function pw(){}const jL=f.createContext({add:pw,remove:pw});function AR(e){const t=f.useContext(jL),n=f.useRef(null);return An(a=>{if(a){const l=e?a.querySelector(e):a;t.add(l),n.current=l}else t.remove(n.current)})}const vw=()=>{const{cancelButtonProps:e,cancelTextLocale:t,onCancel:n}=f.useContext(_u);return se.createElement(er,Object.assign({onClick:n},e),t)},yw=()=>{const{confirmLoading:e,okButtonProps:t,okType:n,okTextLocale:r,onOk:a}=f.useContext(_u);return se.createElement(er,Object.assign({},uy(n),{loading:e,onClick:a},t),r)};function zR(e,t){return se.createElement("span",{className:`${e}-close-x`},t||se.createElement(cs,{className:`${e}-close-icon`}))}const DR=e=>{const{okText:t,okType:n="primary",cancelText:r,confirmLoading:a,onOk:l,onCancel:s,okButtonProps:u,cancelButtonProps:d,footer:g}=e,[h]=Do("Modal",x2()),p=t||(h==null?void 0:h.okText),y=r||(h==null?void 0:h.cancelText),C={confirmLoading:a,okButtonProps:u,cancelButtonProps:d,okTextLocale:p,cancelTextLocale:y,okType:n,onOk:l,onCancel:s},b=se.useMemo(()=>C,Ne(Object.values(C)));let x;return typeof g=="function"||typeof g>"u"?(x=se.createElement(se.Fragment,null,se.createElement(vw,null),se.createElement(yw,null)),typeof g=="function"&&(x=g(x,{OkBtn:yw,CancelBtn:vw})),x=se.createElement(vR,{value:b},x)):x=g,se.createElement(ay,{disabled:!1},x)},IL=e=>{const{componentCls:t}=e;return{[t]:{display:"flex",flexFlow:"row wrap",minWidth:0,"&::before, &::after":{display:"flex"},"&-no-wrap":{flexWrap:"nowrap"},"&-start":{justifyContent:"flex-start"},"&-center":{justifyContent:"center"},"&-end":{justifyContent:"flex-end"},"&-space-between":{justifyContent:"space-between"},"&-space-around":{justifyContent:"space-around"},"&-space-evenly":{justifyContent:"space-evenly"},"&-top":{alignItems:"flex-start"},"&-middle":{alignItems:"center"},"&-bottom":{alignItems:"flex-end"}}}},PL=e=>{const{componentCls:t}=e;return{[t]:{position:"relative",maxWidth:"100%",minHeight:1}}},AL=(e,t)=>{const{prefixCls:n,componentCls:r,gridColumns:a}=e,l={};for(let s=a;s>=0;s--)s===0?(l[`${r}${t}-${s}`]={display:"none"},l[`${r}-push-${s}`]={insetInlineStart:"auto"},l[`${r}-pull-${s}`]={insetInlineEnd:"auto"},l[`${r}${t}-push-${s}`]={insetInlineStart:"auto"},l[`${r}${t}-pull-${s}`]={insetInlineEnd:"auto"},l[`${r}${t}-offset-${s}`]={marginInlineStart:0},l[`${r}${t}-order-${s}`]={order:0}):(l[`${r}${t}-${s}`]=[{"--ant-display":"block",display:"block"},{display:"var(--ant-display)",flex:`0 0 ${s/a*100}%`,maxWidth:`${s/a*100}%`}],l[`${r}${t}-push-${s}`]={insetInlineStart:`${s/a*100}%`},l[`${r}${t}-pull-${s}`]={insetInlineEnd:`${s/a*100}%`},l[`${r}${t}-offset-${s}`]={marginInlineStart:`${s/a*100}%`},l[`${r}${t}-order-${s}`]={order:s});return l[`${r}${t}-flex`]={flex:`var(--${n}${t}-flex)`},l},d0=(e,t)=>AL(e,t),zL=(e,t,n)=>({[`@media (min-width: ${ve(t)})`]:Object.assign({},d0(e,n))}),DL=()=>({}),LL=()=>({}),BL=xn("Grid",IL,DL),LR=e=>({xs:e.screenXSMin,sm:e.screenSMMin,md:e.screenMDMin,lg:e.screenLGMin,xl:e.screenXLMin,xxl:e.screenXXLMin}),HL=xn("Grid",e=>{const t=Kt(e,{gridColumns:24}),n=LR(t);return delete n.xs,[PL(t),d0(t,""),d0(t,"-xs"),Object.keys(n).map(r=>zL(t,n[r],`-${r}`)).reduce((r,a)=>Object.assign(Object.assign({},r),a),{})]},LL);function bw(e){return{position:e,inset:0}}const FL=e=>{const{componentCls:t,antCls:n}=e;return[{[`${t}-root`]:{[`${t}${n}-zoom-enter, ${t}${n}-zoom-appear`]:{transform:"none",opacity:0,animationDuration:e.motionDurationSlow,userSelect:"none"},[`${t}${n}-zoom-leave ${t}-content`]:{pointerEvents:"none"},[`${t}-mask`]:Object.assign(Object.assign({},bw("fixed")),{zIndex:e.zIndexPopupBase,height:"100%",backgroundColor:e.colorBgMask,pointerEvents:"none",[`${t}-hidden`]:{display:"none"}}),[`${t}-wrap`]:Object.assign(Object.assign({},bw("fixed")),{zIndex:e.zIndexPopupBase,overflow:"auto",outline:0,WebkitOverflowScrolling:"touch"})}},{[`${t}-root`]:Kz(e)}]},VL=e=>{const{componentCls:t}=e;return[{[`${t}-root`]:{[`${t}-wrap-rtl`]:{direction:"rtl"},[`${t}-centered`]:{textAlign:"center","&::before":{display:"inline-block",width:0,height:"100%",verticalAlign:"middle",content:'""'},[t]:{top:0,display:"inline-block",paddingBottom:0,textAlign:"start",verticalAlign:"middle"}},[`@media (max-width: ${e.screenSMMax}px)`]:{[t]:{maxWidth:"calc(100vw - 16px)",margin:`${ve(e.marginXS)} auto`},[`${t}-centered`]:{[t]:{flex:1}}}}},{[t]:Object.assign(Object.assign({},Rr(e)),{pointerEvents:"none",position:"relative",top:100,width:"auto",maxWidth:`calc(100vw - ${ve(e.calc(e.margin).mul(2).equal())})`,margin:"0 auto",paddingBottom:e.paddingLG,[`${t}-title`]:{margin:0,color:e.titleColor,fontWeight:e.fontWeightStrong,fontSize:e.titleFontSize,lineHeight:e.titleLineHeight,wordWrap:"break-word"},[`${t}-content`]:{position:"relative",backgroundColor:e.contentBg,backgroundClip:"padding-box",border:0,borderRadius:e.borderRadiusLG,boxShadow:e.boxShadow,pointerEvents:"auto",padding:e.contentPadding},[`${t}-close`]:Object.assign({position:"absolute",top:e.calc(e.modalHeaderHeight).sub(e.modalCloseBtnSize).div(2).equal(),insetInlineEnd:e.calc(e.modalHeaderHeight).sub(e.modalCloseBtnSize).div(2).equal(),zIndex:e.calc(e.zIndexPopupBase).add(10).equal(),padding:0,color:e.modalCloseIconColor,fontWeight:e.fontWeightStrong,lineHeight:1,textDecoration:"none",background:"transparent",borderRadius:e.borderRadiusSM,width:e.modalCloseBtnSize,height:e.modalCloseBtnSize,border:0,outline:0,cursor:"pointer",transition:`color ${e.motionDurationMid}, background-color ${e.motionDurationMid}`,"&-x":{display:"flex",fontSize:e.fontSizeLG,fontStyle:"normal",lineHeight:ve(e.modalCloseBtnSize),justifyContent:"center",textTransform:"none",textRendering:"auto"},"&:disabled":{pointerEvents:"none"},"&:hover":{color:e.modalCloseIconHoverColor,backgroundColor:e.colorBgTextHover,textDecoration:"none"},"&:active":{backgroundColor:e.colorBgTextActive}},gs(e)),[`${t}-header`]:{color:e.colorText,background:e.headerBg,borderRadius:`${ve(e.borderRadiusLG)} ${ve(e.borderRadiusLG)} 0 0`,marginBottom:e.headerMarginBottom,padding:e.headerPadding,borderBottom:e.headerBorderBottom},[`${t}-body`]:{fontSize:e.fontSize,lineHeight:e.lineHeight,wordWrap:"break-word",padding:e.bodyPadding,[`${t}-body-skeleton`]:{width:"100%",height:"100%",display:"flex",justifyContent:"center",alignItems:"center",margin:`${ve(e.margin)} auto`}},[`${t}-footer`]:{textAlign:"end",background:e.footerBg,marginTop:e.footerMarginTop,padding:e.footerPadding,borderTop:e.footerBorderTop,borderRadius:e.footerBorderRadius,[`> ${e.antCls}-btn + ${e.antCls}-btn`]:{marginInlineStart:e.marginXS}},[`${t}-open`]:{overflow:"hidden"}})},{[`${t}-pure-panel`]:{top:"auto",padding:0,display:"flex",flexDirection:"column",[`${t}-content,
          ${t}-body,
          ${t}-confirm-body-wrapper`]:{display:"flex",flexDirection:"column",flex:"auto"},[`${t}-confirm-body`]:{marginBottom:"auto"}}}]},UL=e=>{const{componentCls:t}=e;return{[`${t}-root`]:{[`${t}-wrap-rtl`]:{direction:"rtl",[`${t}-confirm-body`]:{direction:"rtl"}}}}},kL=e=>{const{componentCls:t}=e,n=LR(e);delete n.xs;const r=Object.keys(n).map(a=>({[`@media (min-width: ${ve(n[a])})`]:{width:`var(--${t.replace(".","")}-${a}-width)`}}));return{[`${t}-root`]:{[t]:[{width:`var(--${t.replace(".","")}-xs-width)`}].concat(Ne(r))}}},BR=e=>{const t=e.padding,n=e.fontSizeHeading5,r=e.lineHeightHeading5;return Kt(e,{modalHeaderHeight:e.calc(e.calc(r).mul(n).equal()).add(e.calc(t).mul(2).equal()).equal(),modalFooterBorderColorSplit:e.colorSplit,modalFooterBorderStyle:e.lineType,modalFooterBorderWidth:e.lineWidth,modalCloseIconColor:e.colorIcon,modalCloseIconHoverColor:e.colorIconHover,modalCloseBtnSize:e.controlHeight,modalConfirmIconSize:e.fontHeight,modalTitleHeight:e.calc(e.titleFontSize).mul(e.titleLineHeight).equal()})},HR=e=>({footerBg:"transparent",headerBg:e.colorBgElevated,titleLineHeight:e.lineHeightHeading5,titleFontSize:e.fontSizeHeading5,contentBg:e.colorBgElevated,titleColor:e.colorTextHeading,contentPadding:e.wireframe?0:`${ve(e.paddingMD)} ${ve(e.paddingContentHorizontalLG)}`,headerPadding:e.wireframe?`${ve(e.padding)} ${ve(e.paddingLG)}`:0,headerBorderBottom:e.wireframe?`${ve(e.lineWidth)} ${e.lineType} ${e.colorSplit}`:"none",headerMarginBottom:e.wireframe?0:e.marginXS,bodyPadding:e.wireframe?e.paddingLG:0,footerPadding:e.wireframe?`${ve(e.paddingXS)} ${ve(e.padding)}`:0,footerBorderTop:e.wireframe?`${ve(e.lineWidth)} ${e.lineType} ${e.colorSplit}`:"none",footerBorderRadius:e.wireframe?`0 0 ${ve(e.borderRadiusLG)} ${ve(e.borderRadiusLG)}`:0,footerMarginTop:e.wireframe?0:e.marginSM,confirmBodyPadding:e.wireframe?`${ve(e.padding*2)} ${ve(e.padding*2)} ${ve(e.paddingLG)}`:0,confirmIconMarginInlineEnd:e.wireframe?e.margin:e.marginSM,confirmBtnsMarginTop:e.wireframe?e.marginLG:e.marginSM}),FR=xn("Modal",e=>{const t=BR(e);return[VL(t),UL(t),FL(t),Sm(t,"zoom"),kL(t)]},HR,{unitless:{titleLineHeight:!0}});var qL=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};let m0;const WL=e=>{m0={x:e.pageX,y:e.pageY},setTimeout(()=>{m0=null},100)};fL()&&document.documentElement.addEventListener("click",WL,!0);const VR=e=>{const{prefixCls:t,className:n,rootClassName:r,open:a,wrapClassName:l,centered:s,getContainer:u,focusTriggerAfterClose:d=!0,style:g,visible:h,width:p=520,footer:y,classNames:C,styles:b,children:x,loading:S,confirmLoading:E,zIndex:$,mousePosition:O,onOk:_,onCancel:R}=e,T=qL(e,["prefixCls","className","rootClassName","open","wrapClassName","centered","getContainer","focusTriggerAfterClose","style","visible","width","footer","classNames","styles","children","loading","confirmLoading","zIndex","mousePosition","onOk","onCancel"]),{getPopupContainer:M,getPrefixCls:z,direction:P,modal:D}=f.useContext(dt),F=he=>{E||R==null||R(he)},G=he=>{_==null||_(he)},V=z("modal",t),U=z(),K=ia(V),[j,A,L]=FR(V,K),q=re(l,{[`${V}-centered`]:s??(D==null?void 0:D.centered),[`${V}-wrap-rtl`]:P==="rtl"}),N=y!==null&&!S?f.createElement(DR,Object.assign({},e,{onOk:G,onCancel:F})):null,[B,W,X]=xy(rs(e),rs(D),{closable:!0,closeIcon:f.createElement(cs,{className:`${V}-close-icon`}),closeIconRender:he=>zR(V,he)}),k=AR(`.${V}-content`),[Y,ee]=$u("Modal",$),[ue,ae]=f.useMemo(()=>p&&typeof p=="object"?[void 0,p]:[p,void 0],[p]),oe=f.useMemo(()=>{const he={};return ae&&Object.keys(ae).forEach(ie=>{const le=ae[ie];le!==void 0&&(he[`--${V}-${ie}-width`]=typeof le=="number"?`${le}px`:le)}),he},[ae]);return j(f.createElement(ns,{form:!0,space:!0},f.createElement(gm.Provider,{value:ee},f.createElement(wR,Object.assign({width:ue},T,{zIndex:Y,getContainer:u===void 0?M:u,prefixCls:V,rootClassName:re(A,r,L,K),footer:N,visible:a??h,mousePosition:O??m0,onClose:F,closable:B&&{disabled:X,closeIcon:W},closeIcon:W,focusTriggerAfterClose:d,transitionName:_o(U,"zoom",e.transitionName),maskTransitionName:_o(U,"fade",e.maskTransitionName),className:re(A,n,D==null?void 0:D.className),style:Object.assign(Object.assign(Object.assign({},D==null?void 0:D.style),g),oe),classNames:Object.assign(Object.assign(Object.assign({},D==null?void 0:D.classNames),C),{wrapper:re(q,C==null?void 0:C.wrapper)}),styles:Object.assign(Object.assign({},D==null?void 0:D.styles),b),panelRef:k}),S?f.createElement(Bo,{active:!0,title:!1,paragraph:{rows:4},className:`${V}-body-skeleton`}):x))))},GL=e=>{const{componentCls:t,titleFontSize:n,titleLineHeight:r,modalConfirmIconSize:a,fontSize:l,lineHeight:s,modalTitleHeight:u,fontHeight:d,confirmBodyPadding:g}=e,h=`${t}-confirm`;return{[h]:{"&-rtl":{direction:"rtl"},[`${e.antCls}-modal-header`]:{display:"none"},[`${h}-body-wrapper`]:Object.assign({},Ad()),[`&${t} ${t}-body`]:{padding:g},[`${h}-body`]:{display:"flex",flexWrap:"nowrap",alignItems:"start",[`> ${e.iconCls}`]:{flex:"none",fontSize:a,marginInlineEnd:e.confirmIconMarginInlineEnd,marginTop:e.calc(e.calc(d).sub(a).equal()).div(2).equal()},[`&-has-title > ${e.iconCls}`]:{marginTop:e.calc(e.calc(u).sub(a).equal()).div(2).equal()}},[`${h}-paragraph`]:{display:"flex",flexDirection:"column",flex:"auto",rowGap:e.marginXS,maxWidth:`calc(100% - ${ve(e.marginSM)})`},[`${e.iconCls} + ${h}-paragraph`]:{maxWidth:`calc(100% - ${ve(e.calc(e.modalConfirmIconSize).add(e.marginSM).equal())})`},[`${h}-title`]:{color:e.colorTextHeading,fontWeight:e.fontWeightStrong,fontSize:n,lineHeight:r},[`${h}-content`]:{color:e.colorText,fontSize:l,lineHeight:s},[`${h}-btns`]:{textAlign:"end",marginTop:e.confirmBtnsMarginTop,[`${e.antCls}-btn + ${e.antCls}-btn`]:{marginBottom:0,marginInlineStart:e.marginXS}}},[`${h}-error ${h}-body > ${e.iconCls}`]:{color:e.colorError},[`${h}-warning ${h}-body > ${e.iconCls},
        ${h}-confirm ${h}-body > ${e.iconCls}`]:{color:e.colorWarning},[`${h}-info ${h}-body > ${e.iconCls}`]:{color:e.colorInfo},[`${h}-success ${h}-body > ${e.iconCls}`]:{color:e.colorSuccess}}},KL=hs(["Modal","confirm"],e=>{const t=BR(e);return[GL(t)]},HR,{order:-1e3});var XL=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function UR(e){const{prefixCls:t,icon:n,okText:r,cancelText:a,confirmPrefixCls:l,type:s,okCancel:u,footer:d,locale:g}=e,h=XL(e,["prefixCls","icon","okText","cancelText","confirmPrefixCls","type","okCancel","footer","locale"]);let p=n;if(!n&&n!==null)switch(s){case"info":p=f.createElement(q0,null);break;case"success":p=f.createElement(cm,null);break;case"error":p=f.createElement(ss,null);break;default:p=f.createElement(us,null)}const y=u??s==="confirm",C=e.autoFocusButton===null?!1:e.autoFocusButton||"ok",[b]=Do("Modal"),x=g||b,S=r||(y?x==null?void 0:x.okText:x==null?void 0:x.justOkText),E=a||(x==null?void 0:x.cancelText),$=Object.assign({autoFocusButton:C,cancelTextLocale:E,okTextLocale:S,mergedOkCancel:y},h),O=f.useMemo(()=>$,Ne(Object.values($))),_=f.createElement(f.Fragment,null,f.createElement(Gx,null),f.createElement(Kx,null)),R=e.title!==void 0&&e.title!==null,T=`${l}-body`;return f.createElement("div",{className:`${l}-body-wrapper`},f.createElement("div",{className:re(T,{[`${T}-has-title`]:R})},p,f.createElement("div",{className:`${l}-paragraph`},R&&f.createElement("span",{className:`${l}-title`},e.title),f.createElement("div",{className:`${l}-content`},e.content))),d===void 0||typeof d=="function"?f.createElement(vR,{value:O},f.createElement("div",{className:`${l}-btns`},typeof d=="function"?d(_,{OkBtn:Kx,CancelBtn:Gx}):_)):d,f.createElement(KL,{prefixCls:t}))}const YL=e=>{const{close:t,zIndex:n,maskStyle:r,direction:a,prefixCls:l,wrapClassName:s,rootPrefixCls:u,bodyStyle:d,closable:g=!1,onConfirm:h,styles:p}=e,y=`${l}-confirm`,C=e.width||416,b=e.style||{},x=e.mask===void 0?!0:e.mask,S=e.maskClosable===void 0?!1:e.maskClosable,E=re(y,`${y}-${e.type}`,{[`${y}-rtl`]:a==="rtl"},e.className),[,$]=Br(),O=f.useMemo(()=>n!==void 0?n:$.zIndexPopupBase+sy,[n,$]);return f.createElement(VR,Object.assign({},e,{className:E,wrapClassName:re({[`${y}-centered`]:!!e.centered},s),onCancel:()=>{t==null||t({triggerCancel:!0}),h==null||h(!1)},title:"",footer:null,transitionName:_o(u||"","zoom",e.transitionName),maskTransitionName:_o(u||"","fade",e.maskTransitionName),mask:x,maskClosable:S,style:b,styles:Object.assign({body:d,mask:r},p),width:C,zIndex:O,closable:g}),f.createElement(UR,Object.assign({},e,{confirmPrefixCls:y})))},kR=e=>{const{rootPrefixCls:t,iconPrefixCls:n,direction:r,theme:a}=e;return f.createElement(Fa,{prefixCls:t,iconPrefixCls:n,direction:r,theme:a},f.createElement(YL,Object.assign({},e)))},qi=[];let qR="";function WR(){return qR}const QL=e=>{var t,n;const{prefixCls:r,getContainer:a,direction:l}=e,s=x2(),u=f.useContext(dt),d=WR()||u.getPrefixCls(),g=r||`${d}-modal`;let h=a;return h===!1&&(h=void 0),se.createElement(kR,Object.assign({},e,{rootPrefixCls:d,prefixCls:g,iconPrefixCls:u.iconPrefixCls,theme:u.theme,direction:l??u.direction,locale:(n=(t=u.locale)===null||t===void 0?void 0:t.Modal)!==null&&n!==void 0?n:s,getContainer:h}))};function Mu(e){const t=iy(),n=document.createDocumentFragment();let r=Object.assign(Object.assign({},e),{close:d,open:!0}),a,l;function s(){for(var h,p=arguments.length,y=new Array(p),C=0;C<p;C++)y[C]=arguments[C];if(y.some(S=>S==null?void 0:S.triggerCancel)){var x;(h=e.onCancel)===null||h===void 0||(x=h).call.apply(x,[e,()=>{}].concat(Ne(y.slice(1))))}for(let S=0;S<qi.length;S++)if(qi[S]===d){qi.splice(S,1);break}l()}function u(h){clearTimeout(a),a=setTimeout(()=>{const p=t.getPrefixCls(void 0,WR()),y=t.getIconPrefixCls(),C=t.getTheme(),b=se.createElement(QL,Object.assign({},h));l=pm()(se.createElement(Fa,{prefixCls:p,iconPrefixCls:y,theme:C},t.holderRender?t.holderRender(b):b),n)})}function d(){for(var h=arguments.length,p=new Array(h),y=0;y<h;y++)p[y]=arguments[y];r=Object.assign(Object.assign({},r),{open:!1,afterClose:()=>{typeof e.afterClose=="function"&&e.afterClose(),s.apply(this,p)}}),r.visible&&delete r.visible,u(r)}function g(h){typeof h=="function"?r=h(r):r=Object.assign(Object.assign({},r),h),u(r)}return u(r),qi.push(d),{destroy:d,update:g}}function GR(e){return Object.assign(Object.assign({},e),{type:"warning"})}function KR(e){return Object.assign(Object.assign({},e),{type:"info"})}function XR(e){return Object.assign(Object.assign({},e),{type:"success"})}function YR(e){return Object.assign(Object.assign({},e),{type:"error"})}function QR(e){return Object.assign(Object.assign({},e),{type:"confirm"})}function ZL(e){let{rootPrefixCls:t}=e;qR=t}var JL=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const e8=(e,t)=>{var n,{afterClose:r,config:a}=e,l=JL(e,["afterClose","config"]);const[s,u]=f.useState(!0),[d,g]=f.useState(a),{direction:h,getPrefixCls:p}=f.useContext(dt),y=p("modal"),C=p(),b=()=>{var $;r(),($=d.afterClose)===null||$===void 0||$.call(d)},x=function(){var $;u(!1);for(var O=arguments.length,_=new Array(O),R=0;R<O;R++)_[R]=arguments[R];if(_.some(z=>z==null?void 0:z.triggerCancel)){var M;($=d.onCancel)===null||$===void 0||(M=$).call.apply(M,[d,()=>{}].concat(Ne(_.slice(1))))}};f.useImperativeHandle(t,()=>({destroy:x,update:$=>{g(O=>Object.assign(Object.assign({},O),$))}}));const S=(n=d.okCancel)!==null&&n!==void 0?n:d.type==="confirm",[E]=Do("Modal",Oo.Modal);return f.createElement(kR,Object.assign({prefixCls:y,rootPrefixCls:C},d,{close:x,open:s,afterClose:b,okText:d.okText||(S?E==null?void 0:E.okText:E==null?void 0:E.justOkText),direction:d.direction||h,cancelText:d.cancelText||(E==null?void 0:E.cancelText)},l))},t8=f.forwardRef(e8);let Sw=0;const n8=f.memo(f.forwardRef((e,t)=>{const[n,r]=Z6();return f.useImperativeHandle(t,()=>({patchElement:r}),[]),f.createElement(f.Fragment,null,n)}));function r8(){const e=f.useRef(null),[t,n]=f.useState([]);f.useEffect(()=>{t.length&&(Ne(t).forEach(s=>{s()}),n([]))},[t]);const r=f.useCallback(l=>function(u){var d;Sw+=1;const g=f.createRef();let h;const p=new Promise(S=>{h=S});let y=!1,C;const b=f.createElement(t8,{key:`modal-${Sw}`,config:l(u),ref:g,afterClose:()=>{C==null||C()},isSilent:()=>y,onConfirm:S=>{h(S)}});return C=(d=e.current)===null||d===void 0?void 0:d.patchElement(b),C&&qi.push(C),{destroy:()=>{function S(){var E;(E=g.current)===null||E===void 0||E.destroy()}g.current?S():n(E=>[].concat(Ne(E),[S]))},update:S=>{function E(){var $;($=g.current)===null||$===void 0||$.update(S)}g.current?E():n($=>[].concat(Ne($),[E]))},then:S=>(y=!0,p.then(S))}},[]);return[f.useMemo(()=>({info:r(KR),success:r(XR),error:r(YR),warning:r(GR),confirm:r(QR)}),[]),f.createElement(n8,{key:"modal-holder",ref:e})]}const a8=e=>{const{componentCls:t,notificationMarginEdge:n,animationMaxHeight:r}=e,a=`${t}-notice`,l=new Dt("antNotificationFadeIn",{"0%":{transform:"translate3d(100%, 0, 0)",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",opacity:1}}),s=new Dt("antNotificationTopFadeIn",{"0%":{top:-r,opacity:0},"100%":{top:0,opacity:1}}),u=new Dt("antNotificationBottomFadeIn",{"0%":{bottom:e.calc(r).mul(-1).equal(),opacity:0},"100%":{bottom:0,opacity:1}}),d=new Dt("antNotificationLeftFadeIn",{"0%":{transform:"translate3d(-100%, 0, 0)",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",opacity:1}});return{[t]:{[`&${t}-top, &${t}-bottom`]:{marginInline:0,[a]:{marginInline:"auto auto"}},[`&${t}-top`]:{[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationName:s}},[`&${t}-bottom`]:{[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationName:u}},[`&${t}-topRight, &${t}-bottomRight`]:{[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationName:l}},[`&${t}-topLeft, &${t}-bottomLeft`]:{marginRight:{value:0,_skip_check_:!0},marginLeft:{value:n,_skip_check_:!0},[a]:{marginInlineEnd:"auto",marginInlineStart:0},[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationName:d}}}}},o8=["top","topLeft","topRight","bottom","bottomLeft","bottomRight"],i8={topLeft:"left",topRight:"right",bottomLeft:"left",bottomRight:"right",top:"left",bottom:"left"},l8=(e,t)=>{const{componentCls:n}=e;return{[`${n}-${t}`]:{[`&${n}-stack > ${n}-notice-wrapper`]:{[t.startsWith("top")?"top":"bottom"]:0,[i8[t]]:{value:0,_skip_check_:!0}}}}},s8=e=>{const t={};for(let n=1;n<e.notificationStackLayer;n++)t[`&:nth-last-child(${n+1})`]={overflow:"hidden",[`& > ${e.componentCls}-notice`]:{opacity:0,transition:`opacity ${e.motionDurationMid}`}};return Object.assign({[`&:not(:nth-last-child(-n+${e.notificationStackLayer}))`]:{opacity:0,overflow:"hidden",color:"transparent",pointerEvents:"none"}},t)},c8=e=>{const t={};for(let n=1;n<e.notificationStackLayer;n++)t[`&:nth-last-child(${n+1})`]={background:e.colorBgBlur,backdropFilter:"blur(10px)","-webkit-backdrop-filter":"blur(10px)"};return Object.assign({},t)},u8=e=>{const{componentCls:t}=e;return Object.assign({[`${t}-stack`]:{[`& > ${t}-notice-wrapper`]:Object.assign({transition:`transform ${e.motionDurationSlow}, backdrop-filter 0s`,willChange:"transform, opacity",position:"absolute"},s8(e))},[`${t}-stack:not(${t}-stack-expanded)`]:{[`& > ${t}-notice-wrapper`]:Object.assign({},c8(e))},[`${t}-stack${t}-stack-expanded`]:{[`& > ${t}-notice-wrapper`]:{"&:not(:nth-last-child(-n + 1))":{opacity:1,overflow:"unset",color:"inherit",pointerEvents:"auto",[`& > ${e.componentCls}-notice`]:{opacity:1}},"&:after":{content:'""',position:"absolute",height:e.margin,width:"100%",insetInline:0,bottom:e.calc(e.margin).mul(-1).equal(),background:"transparent",pointerEvents:"auto"}}}},o8.map(n=>l8(e,n)).reduce((n,r)=>Object.assign(Object.assign({},n),r),{}))},ZR=e=>{const{iconCls:t,componentCls:n,boxShadow:r,fontSizeLG:a,notificationMarginBottom:l,borderRadiusLG:s,colorSuccess:u,colorInfo:d,colorWarning:g,colorError:h,colorTextHeading:p,notificationBg:y,notificationPadding:C,notificationMarginEdge:b,notificationProgressBg:x,notificationProgressHeight:S,fontSize:E,lineHeight:$,width:O,notificationIconSize:_,colorText:R}=e,T=`${n}-notice`;return{position:"relative",marginBottom:l,marginInlineStart:"auto",background:y,borderRadius:s,boxShadow:r,[T]:{padding:C,width:O,maxWidth:`calc(100vw - ${ve(e.calc(b).mul(2).equal())})`,overflow:"hidden",lineHeight:$,wordWrap:"break-word"},[`${T}-message`]:{marginBottom:e.marginXS,color:p,fontSize:a,lineHeight:e.lineHeightLG},[`${T}-description`]:{fontSize:E,color:R},[`${T}-closable ${T}-message`]:{paddingInlineEnd:e.paddingLG},[`${T}-with-icon ${T}-message`]:{marginBottom:e.marginXS,marginInlineStart:e.calc(e.marginSM).add(_).equal(),fontSize:a},[`${T}-with-icon ${T}-description`]:{marginInlineStart:e.calc(e.marginSM).add(_).equal(),fontSize:E},[`${T}-icon`]:{position:"absolute",fontSize:_,lineHeight:1,[`&-success${t}`]:{color:u},[`&-info${t}`]:{color:d},[`&-warning${t}`]:{color:g},[`&-error${t}`]:{color:h}},[`${T}-close`]:Object.assign({position:"absolute",top:e.notificationPaddingVertical,insetInlineEnd:e.notificationPaddingHorizontal,color:e.colorIcon,outline:"none",width:e.notificationCloseButtonSize,height:e.notificationCloseButtonSize,borderRadius:e.borderRadiusSM,transition:`background-color ${e.motionDurationMid}, color ${e.motionDurationMid}`,display:"flex",alignItems:"center",justifyContent:"center",background:"none",border:"none","&:hover":{color:e.colorIconHover,backgroundColor:e.colorBgTextHover},"&:active":{backgroundColor:e.colorBgTextActive}},gs(e)),[`${T}-progress`]:{position:"absolute",display:"block",appearance:"none",inlineSize:`calc(100% - ${ve(s)} * 2)`,left:{_skip_check_:!0,value:s},right:{_skip_check_:!0,value:s},bottom:0,blockSize:S,border:0,"&, &::-webkit-progress-bar":{borderRadius:s,backgroundColor:"rgba(0, 0, 0, 0.04)"},"&::-moz-progress-bar":{background:x},"&::-webkit-progress-value":{borderRadius:s,background:x}},[`${T}-actions`]:{float:"right",marginTop:e.marginSM}}},f8=e=>{const{componentCls:t,notificationMarginBottom:n,notificationMarginEdge:r,motionDurationMid:a,motionEaseInOut:l}=e,s=`${t}-notice`,u=new Dt("antNotificationFadeOut",{"0%":{maxHeight:e.animationMaxHeight,marginBottom:n},"100%":{maxHeight:0,marginBottom:0,paddingTop:0,paddingBottom:0,opacity:0}});return[{[t]:Object.assign(Object.assign({},Rr(e)),{position:"fixed",zIndex:e.zIndexPopup,marginRight:{value:r,_skip_check_:!0},[`${t}-hook-holder`]:{position:"relative"},[`${t}-fade-appear-prepare`]:{opacity:"0 !important"},[`${t}-fade-enter, ${t}-fade-appear`]:{animationDuration:e.motionDurationMid,animationTimingFunction:l,animationFillMode:"both",opacity:0,animationPlayState:"paused"},[`${t}-fade-leave`]:{animationTimingFunction:l,animationFillMode:"both",animationDuration:a,animationPlayState:"paused"},[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationPlayState:"running"},[`${t}-fade-leave${t}-fade-leave-active`]:{animationName:u,animationPlayState:"running"},"&-rtl":{direction:"rtl",[`${s}-actions`]:{float:"left"}}})},{[t]:{[`${s}-wrapper`]:Object.assign({},ZR(e))}}]},JR=e=>({zIndexPopup:e.zIndexPopupBase+sy+50,width:384}),e_=e=>{const t=e.paddingMD,n=e.paddingLG;return Kt(e,{notificationBg:e.colorBgElevated,notificationPaddingVertical:t,notificationPaddingHorizontal:n,notificationIconSize:e.calc(e.fontSizeLG).mul(e.lineHeightLG).equal(),notificationCloseButtonSize:e.calc(e.controlHeightLG).mul(.55).equal(),notificationMarginBottom:e.margin,notificationPadding:`${ve(e.paddingMD)} ${ve(e.paddingContentHorizontalLG)}`,notificationMarginEdge:e.marginLG,animationMaxHeight:150,notificationStackLayer:3,notificationProgressHeight:2,notificationProgressBg:`linear-gradient(90deg, ${e.colorPrimaryBorderHover}, ${e.colorPrimary})`})},t_=xn("Notification",e=>{const t=e_(e);return[f8(t),a8(t),u8(t)]},JR),d8=hs(["Notification","PurePanel"],e=>{const t=`${e.componentCls}-notice`,n=e_(e);return{[`${t}-pure-panel`]:Object.assign(Object.assign({},ZR(n)),{width:n.width,maxWidth:`calc(100vw - ${ve(e.calc(n.notificationMarginEdge).mul(2).equal())})`,margin:0})}},JR);var m8=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function wy(e,t){return t===null||t===!1?null:t||f.createElement(cs,{className:`${e}-close-icon`})}const g8={success:cm,info:q0,error:ss,warning:us},n_=e=>{const{prefixCls:t,icon:n,type:r,message:a,description:l,actions:s,role:u="alert"}=e;let d=null;return n?d=f.createElement("span",{className:`${t}-icon`},n):r&&(d=f.createElement(g8[r]||null,{className:re(`${t}-icon`,`${t}-icon-${r}`)})),f.createElement("div",{className:re({[`${t}-with-icon`]:d}),role:u},d,f.createElement("div",{className:`${t}-message`},a),f.createElement("div",{className:`${t}-description`},l),s&&f.createElement("div",{className:`${t}-actions`},s))},h8=e=>{const{prefixCls:t,className:n,icon:r,type:a,message:l,description:s,btn:u,actions:d,closable:g=!0,closeIcon:h,className:p}=e,y=m8(e,["prefixCls","className","icon","type","message","description","btn","actions","closable","closeIcon","className"]),{getPrefixCls:C}=f.useContext(dt),b=d??u,x=t||C("notification"),S=`${x}-notice`,E=ia(x),[$,O,_]=t_(x,E);return $(f.createElement("div",{className:re(`${S}-pure-panel`,O,n,_,E)},f.createElement(d8,{prefixCls:x}),f.createElement(ly,Object.assign({},y,{prefixCls:x,eventKey:"pure",duration:null,closable:g,className:re({notificationClassName:p}),closeIcon:wy(x,h),content:f.createElement(n_,{prefixCls:S,icon:r,type:a,message:l,description:s,actions:b})}))))};function p8(e,t,n){let r;switch(e){case"top":r={left:"50%",transform:"translateX(-50%)",right:"auto",top:t,bottom:"auto"};break;case"topLeft":r={left:0,top:t,bottom:"auto"};break;case"topRight":r={right:0,top:t,bottom:"auto"};break;case"bottom":r={left:"50%",transform:"translateX(-50%)",right:"auto",top:"auto",bottom:n};break;case"bottomLeft":r={left:0,top:"auto",bottom:n};break;default:r={right:0,top:"auto",bottom:n};break}return r}function v8(e){return{motionName:`${e}-fade`}}function y8(e,t,n){return typeof e<"u"?e:typeof(t==null?void 0:t.closeIcon)<"u"?t.closeIcon:n==null?void 0:n.closeIcon}var b8=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const Cw=24,S8=4.5,C8="topRight",x8=e=>{let{children:t,prefixCls:n}=e;const r=ia(n),[a,l,s]=t_(n,r);return a(se.createElement(Z2,{classNames:{list:re(l,s,r)}},t))},w8=(e,t)=>{let{prefixCls:n,key:r}=t;return se.createElement(x8,{prefixCls:n,key:r},e)},E8=se.forwardRef((e,t)=>{const{top:n,bottom:r,prefixCls:a,getContainer:l,maxCount:s,rtl:u,onAllRemoved:d,stack:g,duration:h,pauseOnHover:p=!0,showProgress:y}=e,{getPrefixCls:C,getPopupContainer:b,notification:x,direction:S}=f.useContext(dt),[,E]=Br(),$=a||C("notification"),O=z=>p8(z,n??Cw,r??Cw),_=()=>re({[`${$}-rtl`]:u??S==="rtl"}),R=()=>v8($),[T,M]=J2({prefixCls:$,style:O,className:_,motion:R,closable:!0,closeIcon:wy($),duration:h??S8,getContainer:()=>(l==null?void 0:l())||(b==null?void 0:b())||document.body,maxCount:s,pauseOnHover:p,showProgress:y,onAllRemoved:d,renderNotifications:w8,stack:g===!1?!1:{threshold:typeof g=="object"?g==null?void 0:g.threshold:void 0,offset:8,gap:E.margin}});return se.useImperativeHandle(t,()=>Object.assign(Object.assign({},T),{prefixCls:$,notification:x})),M});function r_(e){const t=se.useRef(null);return Eu(),[se.useMemo(()=>{const r=u=>{var d;if(!t.current)return;const{open:g,prefixCls:h,notification:p}=t.current,y=`${h}-notice`,{message:C,description:b,icon:x,type:S,btn:E,actions:$,className:O,style:_,role:R="alert",closeIcon:T,closable:M}=u,z=b8(u,["message","description","icon","type","btn","actions","className","style","role","closeIcon","closable"]),P=$??E,D=wy(y,y8(T,e,p));return g(Object.assign(Object.assign({placement:(d=e==null?void 0:e.placement)!==null&&d!==void 0?d:C8},z),{content:se.createElement(n_,{prefixCls:y,icon:x,type:S,message:C,description:b,actions:P,role:R}),className:re(S&&`${y}-${S}`,O,p==null?void 0:p.className),style:Object.assign(Object.assign({},p==null?void 0:p.style),_),closeIcon:D,closable:M??!!D}))},l={open:r,destroy:u=>{var d,g;u!==void 0?(d=t.current)===null||d===void 0||d.close(u):(g=t.current)===null||g===void 0||g.destroy()}};return["success","info","warning","error"].forEach(u=>{l[u]=d=>r(Object.assign(Object.assign({},d),{type:u}))}),l},[]),se.createElement(E8,Object.assign({key:"notification-holder"},e,{ref:t}))]}function $8(e){return r_(e)}const a_=se.createContext({});function o_(e){return t=>f.createElement(Fa,{theme:{token:{motion:!1,zIndexPopupBase:0}}},f.createElement(e,Object.assign({},t)))}const O8=(e,t,n,r,a)=>o_(s=>{const{prefixCls:u,style:d}=s,g=f.useRef(null),[h,p]=f.useState(0),[y,C]=f.useState(0),[b,x]=br(!1,{value:s.open}),{getPrefixCls:S}=f.useContext(dt),E=S(r||"select",u);f.useEffect(()=>{if(x(!0),typeof ResizeObserver<"u"){const _=new ResizeObserver(T=>{const M=T[0].target;p(M.offsetHeight+8),C(M.offsetWidth)}),R=setInterval(()=>{var T;const M=a?`.${a(E)}`:`.${E}-dropdown`,z=(T=g.current)===null||T===void 0?void 0:T.querySelector(M);z&&(clearInterval(R),_.observe(z))},10);return()=>{clearInterval(R),_.disconnect()}}},[]);let $=Object.assign(Object.assign({},s),{style:Object.assign(Object.assign({},d),{margin:0}),open:b,visible:b,getPopupContainer:()=>g.current});t&&Object.assign($,{[t]:{overflow:{adjustX:!1,adjustY:!1}}});const O={paddingBottom:h,position:"relative",minWidth:y};return f.createElement("div",{ref:g,style:O},f.createElement(e,Object.assign({},$)))}),i_=function(){if(typeof navigator>"u"||typeof window>"u")return!1;var e=navigator.userAgent||navigator.vendor||window.opera;return/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino|android|ipad|playbook|silk/i.test(e)||/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw-(n|u)|c55\/|capi|ccwa|cdm-|cell|chtm|cldc|cmd-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc-s|devi|dica|dmob|do(c|p)o|ds(12|-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(-|_)|g1 u|g560|gene|gf-5|g-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd-(m|p|t)|hei-|hi(pt|ta)|hp( i|ip)|hs-c|ht(c(-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i-(20|go|ma)|i230|iac( |-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|-[a-w])|libw|lynx|m1-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|-([1-8]|c))|phil|pire|pl(ay|uc)|pn-2|po(ck|rt|se)|prox|psio|pt-g|qa-a|qc(07|12|21|32|60|-[2-7]|i-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h-|oo|p-)|sdk\/|se(c(-|0|1)|47|mc|nd|ri)|sgh-|shar|sie(-|m)|sk-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h-|v-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl-|tdg-|tel(i|m)|tim-|t-mo|to(pl|sh)|ts(70|m-|m3|m5)|tx-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas-|your|zeto|zte-/i.test(e==null?void 0:e.substr(0,4))};var Mm=function(t){var n=t.className,r=t.customizeIcon,a=t.customizeIconProps,l=t.children,s=t.onMouseDown,u=t.onClick,d=typeof r=="function"?r(a):r;return f.createElement("span",{className:n,onMouseDown:function(h){h.preventDefault(),s==null||s(h)},style:{userSelect:"none",WebkitUserSelect:"none"},unselectable:"on",onClick:u,"aria-hidden":!0},d!==void 0?d:f.createElement("span",{className:re(n.split(/\s+/).map(function(g){return"".concat(g,"-icon")}))},l))},R8=function(t,n,r,a,l){var s=arguments.length>5&&arguments[5]!==void 0?arguments[5]:!1,u=arguments.length>6?arguments[6]:void 0,d=arguments.length>7?arguments[7]:void 0,g=se.useMemo(function(){if(st(a)==="object")return a.clearIcon;if(l)return l},[a,l]),h=se.useMemo(function(){return!!(!s&&a&&(r.length||u)&&!(d==="combobox"&&u===""))},[a,s,r.length,u,d]);return{allowClear:h,clearIcon:se.createElement(Mm,{className:"".concat(t,"-clear"),onMouseDown:n,customizeIcon:g},"×")}},l_=f.createContext(null);function _8(){return f.useContext(l_)}function T8(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:10,t=f.useState(!1),n=ge(t,2),r=n[0],a=n[1],l=f.useRef(null),s=function(){window.clearTimeout(l.current)};f.useEffect(function(){return s},[]);var u=function(g,h){s(),l.current=window.setTimeout(function(){a(g),h&&h()},e)};return[r,u,s]}function s_(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:250,t=f.useRef(null),n=f.useRef(null);f.useEffect(function(){return function(){window.clearTimeout(n.current)}},[]);function r(a){(a||t.current===null)&&(t.current=a),window.clearTimeout(n.current),n.current=window.setTimeout(function(){t.current=null},e)}return[function(){return t.current},r]}function M8(e,t,n,r){var a=f.useRef(null);a.current={open:t,triggerOpen:n,customizedTrigger:r},f.useEffect(function(){function l(s){var u;if(!((u=a.current)!==null&&u!==void 0&&u.customizedTrigger)){var d=s.target;d.shadowRoot&&s.composed&&(d=s.composedPath()[0]||d),a.current.open&&e().filter(function(g){return g}).every(function(g){return!g.contains(d)&&g!==d})&&a.current.triggerOpen(!1)}}return window.addEventListener("mousedown",l),function(){return window.removeEventListener("mousedown",l)}},[])}function N8(e){return e&&![lt.ESC,lt.SHIFT,lt.BACKSPACE,lt.TAB,lt.WIN_KEY,lt.ALT,lt.META,lt.WIN_KEY_RIGHT,lt.CTRL,lt.SEMICOLON,lt.EQUALS,lt.CAPS_LOCK,lt.CONTEXT_MENU,lt.F1,lt.F2,lt.F3,lt.F4,lt.F5,lt.F6,lt.F7,lt.F8,lt.F9,lt.F10,lt.F11,lt.F12].includes(e)}var j8=["prefixCls","invalidate","item","renderItem","responsive","responsiveDisabled","registerSize","itemKey","className","style","children","display","order","component"],Hl=void 0;function I8(e,t){var n=e.prefixCls,r=e.invalidate,a=e.item,l=e.renderItem,s=e.responsive,u=e.responsiveDisabled,d=e.registerSize,g=e.itemKey,h=e.className,p=e.style,y=e.children,C=e.display,b=e.order,x=e.component,S=x===void 0?"div":x,E=$t(e,j8),$=s&&!C;function O(z){d(g,z)}f.useEffect(function(){return function(){O(null)}},[]);var _=l&&a!==Hl?l(a,{index:b}):y,R;r||(R={opacity:$?0:1,height:$?0:Hl,overflowY:$?"hidden":Hl,order:s?b:Hl,pointerEvents:$?"none":Hl,position:$?"absolute":Hl});var T={};$&&(T["aria-hidden"]=!0);var M=f.createElement(S,je({className:re(!r&&n,h),style:Q(Q({},R),p)},T,E,{ref:t}),_);return s&&(M=f.createElement(zo,{onResize:function(P){var D=P.offsetWidth;O(D)},disabled:u},M)),M}var Xc=f.forwardRef(I8);Xc.displayName="Item";function P8(e){if(typeof MessageChannel>"u")bn(e);else{var t=new MessageChannel;t.port1.onmessage=function(){return e()},t.port2.postMessage(void 0)}}function A8(){var e=f.useRef(null),t=function(r){e.current||(e.current=[],P8(function(){pi.unstable_batchedUpdates(function(){e.current.forEach(function(a){a()}),e.current=null})})),e.current.push(r)};return t}function Cc(e,t){var n=f.useState(t),r=ge(n,2),a=r[0],l=r[1],s=An(function(u){e(function(){l(u)})});return[a,s]}var Hd=se.createContext(null),z8=["component"],D8=["className"],L8=["className"],B8=function(t,n){var r=f.useContext(Hd);if(!r){var a=t.component,l=a===void 0?"div":a,s=$t(t,z8);return f.createElement(l,je({},s,{ref:n}))}var u=r.className,d=$t(r,D8),g=t.className,h=$t(t,L8);return f.createElement(Hd.Provider,{value:null},f.createElement(Xc,je({ref:n,className:re(u,g)},d,h)))},c_=f.forwardRef(B8);c_.displayName="RawItem";var H8=["prefixCls","data","renderItem","renderRawItem","itemKey","itemWidth","ssr","style","className","maxCount","renderRest","renderRawRest","suffix","component","itemComponent","onVisibleChange"],u_="responsive",f_="invalidate";function F8(e){return"+ ".concat(e.length," ...")}function V8(e,t){var n=e.prefixCls,r=n===void 0?"rc-overflow":n,a=e.data,l=a===void 0?[]:a,s=e.renderItem,u=e.renderRawItem,d=e.itemKey,g=e.itemWidth,h=g===void 0?10:g,p=e.ssr,y=e.style,C=e.className,b=e.maxCount,x=e.renderRest,S=e.renderRawRest,E=e.suffix,$=e.component,O=$===void 0?"div":$,_=e.itemComponent,R=e.onVisibleChange,T=$t(e,H8),M=p==="full",z=A8(),P=Cc(z,null),D=ge(P,2),F=D[0],G=D[1],V=F||0,U=Cc(z,new Map),K=ge(U,2),j=K[0],A=K[1],L=Cc(z,0),q=ge(L,2),N=q[0],B=q[1],W=Cc(z,0),X=ge(W,2),k=X[0],Y=X[1],ee=Cc(z,0),ue=ge(ee,2),ae=ue[0],oe=ue[1],he=f.useState(null),ie=ge(he,2),le=ie[0],be=ie[1],Re=f.useState(null),fe=ge(Re,2),Se=fe[0],Ee=fe[1],Oe=f.useMemo(function(){return Se===null&&M?Number.MAX_SAFE_INTEGER:Se||0},[Se,F]),Pe=f.useState(!1),Ye=ge(Pe,2),Me=Ye[0],De=Ye[1],xe="".concat(r,"-item"),Te=Math.max(N,k),Le=b===u_,We=l.length&&Le,Ge=b===f_,at=We||typeof b=="number"&&l.length>b,ke=f.useMemo(function(){var nt=l;return We?F===null&&M?nt=l:nt=l.slice(0,Math.min(l.length,V/h)):typeof b=="number"&&(nt=l.slice(0,b)),nt},[l,h,F,b,We]),Qe=f.useMemo(function(){return We?l.slice(Oe+1):l.slice(ke.length)},[l,ke,We,Oe]),ct=f.useCallback(function(nt,Ke){var Fe;return typeof d=="function"?d(nt):(Fe=d&&(nt==null?void 0:nt[d]))!==null&&Fe!==void 0?Fe:Ke},[d]),pt=f.useCallback(s||function(nt){return nt},[s]);function Ze(nt,Ke,Fe){Se===nt&&(Ke===void 0||Ke===le)||(Ee(nt),Fe||(De(nt<l.length-1),R==null||R(nt)),Ke!==void 0&&be(Ke))}function Be(nt,Ke){G(Ke.clientWidth)}function Ue(nt,Ke){A(function(Fe){var vt=new Map(Fe);return Ke===null?vt.delete(nt):vt.set(nt,Ke),vt})}function Ie(nt,Ke){Y(Ke),B(k)}function qe(nt,Ke){oe(Ke)}function wt(nt){return j.get(ct(ke[nt],nt))}Gt(function(){if(V&&typeof Te=="number"&&ke){var nt=ae,Ke=ke.length,Fe=Ke-1;if(!Ke){Ze(0,null);return}for(var vt=0;vt<Ke;vt+=1){var Ct=wt(vt);if(M&&(Ct=Ct||0),Ct===void 0){Ze(vt-1,void 0,!0);break}if(nt+=Ct,Fe===0&&nt<=V||vt===Fe-1&&nt+wt(Fe)<=V){Ze(Fe,null);break}else if(nt+Te>V){Ze(vt-1,nt-Ct-ae+k);break}}E&&wt(0)+ae>V&&be(null)}},[V,j,k,ae,ct,ke]);var It=Me&&!!Qe.length,Nn={};le!==null&&We&&(Nn={position:"absolute",left:le,top:0});var wn={prefixCls:xe,responsive:We,component:_,invalidate:Ge},Rn=u?function(nt,Ke){var Fe=ct(nt,Ke);return f.createElement(Hd.Provider,{key:Fe,value:Q(Q({},wn),{},{order:Ke,item:nt,itemKey:Fe,registerSize:Ue,display:Ke<=Oe})},u(nt,Ke))}:function(nt,Ke){var Fe=ct(nt,Ke);return f.createElement(Xc,je({},wn,{order:Ke,key:Fe,item:nt,renderItem:pt,itemKey:Fe,registerSize:Ue,display:Ke<=Oe}))},dn={order:It?Oe:Number.MAX_SAFE_INTEGER,className:"".concat(xe,"-rest"),registerSize:Ie,display:It},kt=x||F8,ln=S?f.createElement(Hd.Provider,{value:Q(Q({},wn),dn)},S(Qe)):f.createElement(Xc,je({},wn,dn),typeof kt=="function"?kt(Qe):kt),Ot=f.createElement(O,je({className:re(!Ge&&r,C),style:y,ref:t},T),ke.map(Rn),at?ln:null,E&&f.createElement(Xc,je({},wn,{responsive:Le,responsiveDisabled:!We,order:Oe,className:"".concat(xe,"-suffix"),registerSize:qe,display:!0,style:Nn}),E));return Le?f.createElement(zo,{onResize:Be,disabled:!We},Ot):Ot}var eo=f.forwardRef(V8);eo.displayName="Overflow";eo.Item=c_;eo.RESPONSIVE=u_;eo.INVALIDATE=f_;var U8=function(t,n){var r,a=t.prefixCls,l=t.id,s=t.inputElement,u=t.disabled,d=t.tabIndex,g=t.autoFocus,h=t.autoComplete,p=t.editable,y=t.activeDescendantId,C=t.value,b=t.maxLength,x=t.onKeyDown,S=t.onMouseDown,E=t.onChange,$=t.onPaste,O=t.onCompositionStart,_=t.onCompositionEnd,R=t.onBlur,T=t.open,M=t.attrs,z=s||f.createElement("input",null),P=z,D=P.ref,F=P.props,G=F.onKeyDown,V=F.onChange,U=F.onMouseDown,K=F.onCompositionStart,j=F.onCompositionEnd,A=F.onBlur,L=F.style;return"maxLength"in z.props,z=f.cloneElement(z,Q(Q(Q({type:"search"},F),{},{id:l,ref:Ha(n,D),disabled:u,tabIndex:d,autoComplete:h||"off",autoFocus:g,className:re("".concat(a,"-selection-search-input"),(r=z)===null||r===void 0||(r=r.props)===null||r===void 0?void 0:r.className),role:"combobox","aria-expanded":T||!1,"aria-haspopup":"listbox","aria-owns":"".concat(l,"_list"),"aria-autocomplete":"list","aria-controls":"".concat(l,"_list"),"aria-activedescendant":T?y:void 0},M),{},{value:p?C:"",maxLength:b,readOnly:!p,unselectable:p?null:"on",style:Q(Q({},L),{},{opacity:p?null:0}),onKeyDown:function(N){x(N),G&&G(N)},onMouseDown:function(N){S(N),U&&U(N)},onChange:function(N){E(N),V&&V(N)},onCompositionStart:function(N){O(N),K&&K(N)},onCompositionEnd:function(N){_(N),j&&j(N)},onPaste:$,onBlur:function(N){R(N),A&&A(N)}})),z},d_=f.forwardRef(U8);function m_(e){return Array.isArray(e)?e:e!==void 0?[e]:[]}var k8=typeof window<"u"&&window.document&&window.document.documentElement,q8=k8;function W8(e){return e!=null}function G8(e){return!e&&e!==0}function xw(e){return["string","number"].includes(st(e))}function g_(e){var t=void 0;return e&&(xw(e.title)?t=e.title.toString():xw(e.label)&&(t=e.label.toString())),t}function K8(e,t){q8?f.useLayoutEffect(e,t):f.useEffect(e,t)}function X8(e){var t;return(t=e.key)!==null&&t!==void 0?t:e.value}var ww=function(t){t.preventDefault(),t.stopPropagation()},Y8=function(t){var n=t.id,r=t.prefixCls,a=t.values,l=t.open,s=t.searchValue,u=t.autoClearSearchValue,d=t.inputRef,g=t.placeholder,h=t.disabled,p=t.mode,y=t.showSearch,C=t.autoFocus,b=t.autoComplete,x=t.activeDescendantId,S=t.tabIndex,E=t.removeIcon,$=t.maxTagCount,O=t.maxTagTextLength,_=t.maxTagPlaceholder,R=_===void 0?function(Re){return"+ ".concat(Re.length," ...")}:_,T=t.tagRender,M=t.onToggleOpen,z=t.onRemove,P=t.onInputChange,D=t.onInputPaste,F=t.onInputKeyDown,G=t.onInputMouseDown,V=t.onInputCompositionStart,U=t.onInputCompositionEnd,K=t.onInputBlur,j=f.useRef(null),A=f.useState(0),L=ge(A,2),q=L[0],N=L[1],B=f.useState(!1),W=ge(B,2),X=W[0],k=W[1],Y="".concat(r,"-selection"),ee=l||p==="multiple"&&u===!1||p==="tags"?s:"",ue=p==="tags"||p==="multiple"&&u===!1||y&&(l||X);K8(function(){N(j.current.scrollWidth)},[ee]);var ae=function(fe,Se,Ee,Oe,Pe){return f.createElement("span",{title:g_(fe),className:re("".concat(Y,"-item"),Z({},"".concat(Y,"-item-disabled"),Ee))},f.createElement("span",{className:"".concat(Y,"-item-content")},Se),Oe&&f.createElement(Mm,{className:"".concat(Y,"-item-remove"),onMouseDown:ww,onClick:Pe,customizeIcon:E},"×"))},oe=function(fe,Se,Ee,Oe,Pe,Ye){var Me=function(xe){ww(xe),M(!l)};return f.createElement("span",{onMouseDown:Me},T({label:Se,value:fe,disabled:Ee,closable:Oe,onClose:Pe,isMaxTag:!!Ye}))},he=function(fe){var Se=fe.disabled,Ee=fe.label,Oe=fe.value,Pe=!h&&!Se,Ye=Ee;if(typeof O=="number"&&(typeof Ee=="string"||typeof Ee=="number")){var Me=String(Ye);Me.length>O&&(Ye="".concat(Me.slice(0,O),"..."))}var De=function(Te){Te&&Te.stopPropagation(),z(fe)};return typeof T=="function"?oe(Oe,Ye,Se,Pe,De):ae(fe,Ye,Se,Pe,De)},ie=function(fe){if(!a.length)return null;var Se=typeof R=="function"?R(fe):R;return typeof T=="function"?oe(void 0,Se,!1,!1,void 0,!0):ae({title:Se},Se,!1)},le=f.createElement("div",{className:"".concat(Y,"-search"),style:{width:q},onFocus:function(){k(!0)},onBlur:function(){k(!1)}},f.createElement(d_,{ref:d,open:l,prefixCls:r,id:n,inputElement:null,disabled:h,autoFocus:C,autoComplete:b,editable:ue,activeDescendantId:x,value:ee,onKeyDown:F,onMouseDown:G,onChange:P,onPaste:D,onCompositionStart:V,onCompositionEnd:U,onBlur:K,tabIndex:S,attrs:ta(t,!0)}),f.createElement("span",{ref:j,className:"".concat(Y,"-search-mirror"),"aria-hidden":!0},ee," ")),be=f.createElement(eo,{prefixCls:"".concat(Y,"-overflow"),data:a,renderItem:he,renderRest:ie,suffix:le,itemKey:X8,maxCount:$});return f.createElement("span",{className:"".concat(Y,"-wrap")},be,!a.length&&!ee&&f.createElement("span",{className:"".concat(Y,"-placeholder")},g))},Q8=function(t){var n=t.inputElement,r=t.prefixCls,a=t.id,l=t.inputRef,s=t.disabled,u=t.autoFocus,d=t.autoComplete,g=t.activeDescendantId,h=t.mode,p=t.open,y=t.values,C=t.placeholder,b=t.tabIndex,x=t.showSearch,S=t.searchValue,E=t.activeValue,$=t.maxLength,O=t.onInputKeyDown,_=t.onInputMouseDown,R=t.onInputChange,T=t.onInputPaste,M=t.onInputCompositionStart,z=t.onInputCompositionEnd,P=t.onInputBlur,D=t.title,F=f.useState(!1),G=ge(F,2),V=G[0],U=G[1],K=h==="combobox",j=K||x,A=y[0],L=S||"";K&&E&&!V&&(L=E),f.useEffect(function(){K&&U(!1)},[K,E]);var q=h!=="combobox"&&!p&&!x?!1:!!L,N=D===void 0?g_(A):D,B=f.useMemo(function(){return A?null:f.createElement("span",{className:"".concat(r,"-selection-placeholder"),style:q?{visibility:"hidden"}:void 0},C)},[A,q,C,r]);return f.createElement("span",{className:"".concat(r,"-selection-wrap")},f.createElement("span",{className:"".concat(r,"-selection-search")},f.createElement(d_,{ref:l,prefixCls:r,id:a,open:p,inputElement:n,disabled:s,autoFocus:u,autoComplete:d,editable:j,activeDescendantId:g,value:L,onKeyDown:O,onMouseDown:_,onChange:function(X){U(!0),R(X)},onPaste:T,onCompositionStart:M,onCompositionEnd:z,onBlur:P,tabIndex:b,attrs:ta(t,!0),maxLength:K?$:void 0})),!K&&A?f.createElement("span",{className:"".concat(r,"-selection-item"),title:N,style:q?{visibility:"hidden"}:void 0},A.label):null,B)},Z8=function(t,n){var r=f.useRef(null),a=f.useRef(!1),l=t.prefixCls,s=t.open,u=t.mode,d=t.showSearch,g=t.tokenWithEnter,h=t.disabled,p=t.prefix,y=t.autoClearSearchValue,C=t.onSearch,b=t.onSearchSubmit,x=t.onToggleOpen,S=t.onInputKeyDown,E=t.onInputBlur,$=t.domRef;f.useImperativeHandle(n,function(){return{focus:function(N){r.current.focus(N)},blur:function(){r.current.blur()}}});var O=s_(0),_=ge(O,2),R=_[0],T=_[1],M=function(N){var B=N.which,W=r.current instanceof HTMLTextAreaElement;!W&&s&&(B===lt.UP||B===lt.DOWN)&&N.preventDefault(),S&&S(N),B===lt.ENTER&&u==="tags"&&!a.current&&!s&&(b==null||b(N.target.value)),!(W&&!s&&~[lt.UP,lt.DOWN,lt.LEFT,lt.RIGHT].indexOf(B))&&N8(B)&&x(!0)},z=function(){T(!0)},P=f.useRef(null),D=function(N){C(N,!0,a.current)!==!1&&x(!0)},F=function(){a.current=!0},G=function(N){a.current=!1,u!=="combobox"&&D(N.target.value)},V=function(N){var B=N.target.value;if(g&&P.current&&/[\r\n]/.test(P.current)){var W=P.current.replace(/[\r\n]+$/,"").replace(/\r\n/g," ").replace(/[\r\n]/g," ");B=B.replace(W,P.current)}P.current=null,D(B)},U=function(N){var B=N.clipboardData,W=B==null?void 0:B.getData("text");P.current=W||""},K=function(N){var B=N.target;if(B!==r.current){var W=document.body.style.msTouchAction!==void 0;W?setTimeout(function(){r.current.focus()}):r.current.focus()}},j=function(N){var B=R();N.target!==r.current&&!B&&!(u==="combobox"&&h)&&N.preventDefault(),(u!=="combobox"&&(!d||!B)||!s)&&(s&&y!==!1&&C("",!0,!1),x())},A={inputRef:r,onInputKeyDown:M,onInputMouseDown:z,onInputChange:V,onInputPaste:U,onInputCompositionStart:F,onInputCompositionEnd:G,onInputBlur:E},L=u==="multiple"||u==="tags"?f.createElement(Y8,je({},t,A)):f.createElement(Q8,je({},t,A));return f.createElement("div",{ref:$,className:"".concat(l,"-selector"),onClick:K,onMouseDown:j},p&&f.createElement("div",{className:"".concat(l,"-prefix")},p),L)},J8=f.forwardRef(Z8);function eB(e){var t=e.prefixCls,n=e.align,r=e.arrow,a=e.arrowPos,l=r||{},s=l.className,u=l.content,d=a.x,g=d===void 0?0:d,h=a.y,p=h===void 0?0:h,y=f.useRef();if(!n||!n.points)return null;var C={position:"absolute"};if(n.autoArrow!==!1){var b=n.points[0],x=n.points[1],S=b[0],E=b[1],$=x[0],O=x[1];S===$||!["t","b"].includes(S)?C.top=p:S==="t"?C.top=0:C.bottom=0,E===O||!["l","r"].includes(E)?C.left=g:E==="l"?C.left=0:C.right=0}return f.createElement("div",{ref:y,className:re("".concat(t,"-arrow"),s),style:C},u)}function tB(e){var t=e.prefixCls,n=e.open,r=e.zIndex,a=e.mask,l=e.motion;return a?f.createElement($a,je({},l,{motionAppear:!0,visible:n,removeOnLeave:!0}),function(s){var u=s.className;return f.createElement("div",{style:{zIndex:r},className:re("".concat(t,"-mask"),u)})}):null}var nB=f.memo(function(e){var t=e.children;return t},function(e,t){return t.cache}),rB=f.forwardRef(function(e,t){var n=e.popup,r=e.className,a=e.prefixCls,l=e.style,s=e.target,u=e.onVisibleChanged,d=e.open,g=e.keepDom,h=e.fresh,p=e.onClick,y=e.mask,C=e.arrow,b=e.arrowPos,x=e.align,S=e.motion,E=e.maskMotion,$=e.forceRender,O=e.getPopupContainer,_=e.autoDestroy,R=e.portal,T=e.zIndex,M=e.onMouseEnter,z=e.onMouseLeave,P=e.onPointerEnter,D=e.onPointerDownCapture,F=e.ready,G=e.offsetX,V=e.offsetY,U=e.offsetR,K=e.offsetB,j=e.onAlign,A=e.onPrepare,L=e.stretch,q=e.targetWidth,N=e.targetHeight,B=typeof n=="function"?n():n,W=d||g,X=(O==null?void 0:O.length)>0,k=f.useState(!O||!X),Y=ge(k,2),ee=Y[0],ue=Y[1];if(Gt(function(){!ee&&X&&s&&ue(!0)},[ee,X,s]),!ee)return null;var ae="auto",oe={left:"-1000vw",top:"-1000vh",right:ae,bottom:ae};if(F||!d){var he,ie=x.points,le=x.dynamicInset||((he=x._experimental)===null||he===void 0?void 0:he.dynamicInset),be=le&&ie[0][1]==="r",Re=le&&ie[0][0]==="b";be?(oe.right=U,oe.left=ae):(oe.left=G,oe.right=ae),Re?(oe.bottom=K,oe.top=ae):(oe.top=V,oe.bottom=ae)}var fe={};return L&&(L.includes("height")&&N?fe.height=N:L.includes("minHeight")&&N&&(fe.minHeight=N),L.includes("width")&&q?fe.width=q:L.includes("minWidth")&&q&&(fe.minWidth=q)),d||(fe.pointerEvents="none"),f.createElement(R,{open:$||W,getContainer:O&&function(){return O(s)},autoDestroy:_},f.createElement(tB,{prefixCls:a,open:d,zIndex:T,mask:y,motion:E}),f.createElement(zo,{onResize:j,disabled:!d},function(Se){return f.createElement($a,je({motionAppear:!0,motionEnter:!0,motionLeave:!0,removeOnLeave:!1,forceRender:$,leavedClassName:"".concat(a,"-hidden")},S,{onAppearPrepare:A,onEnterPrepare:A,visible:d,onVisibleChanged:function(Oe){var Pe;S==null||(Pe=S.onVisibleChanged)===null||Pe===void 0||Pe.call(S,Oe),u(Oe)}}),function(Ee,Oe){var Pe=Ee.className,Ye=Ee.style,Me=re(a,Pe,r);return f.createElement("div",{ref:Ha(Se,t,Oe),className:Me,style:Q(Q(Q(Q({"--arrow-x":"".concat(b.x||0,"px"),"--arrow-y":"".concat(b.y||0,"px")},oe),fe),Ye),{},{boxSizing:"border-box",zIndex:T},l),onMouseEnter:M,onMouseLeave:z,onPointerEnter:P,onClick:p,onPointerDownCapture:D},C&&f.createElement(eB,{prefixCls:a,arrow:C,arrowPos:b,align:x}),f.createElement(nB,{cache:!d&&!h},B))})}))}),aB=f.forwardRef(function(e,t){var n=e.children,r=e.getTriggerDOMNode,a=rl(n),l=f.useCallback(function(u){W0(t,r?r(u):u)},[r]),s=Io(l,fs(n));return a?f.cloneElement(n,{ref:s}):n}),Ew=f.createContext(null);function $w(e){return e?Array.isArray(e)?e:[e]:[]}function oB(e,t,n,r){return f.useMemo(function(){var a=$w(n??t),l=$w(r??t),s=new Set(a),u=new Set(l);return e&&(s.has("hover")&&(s.delete("hover"),s.add("click")),u.has("hover")&&(u.delete("hover"),u.add("click"))),[s,u]},[e,t,n,r])}function iB(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[],t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[],n=arguments.length>2?arguments[2]:void 0;return n?e[0]===t[0]:e[0]===t[0]&&e[1]===t[1]}function lB(e,t,n,r){for(var a=n.points,l=Object.keys(e),s=0;s<l.length;s+=1){var u,d=l[s];if(iB((u=e[d])===null||u===void 0?void 0:u.points,a,r))return"".concat(t,"-placement-").concat(d)}return""}function Ow(e,t,n,r){return t||(n?{motionName:"".concat(e,"-").concat(n)}:r?{motionName:r}:null)}function Nu(e){return e.ownerDocument.defaultView}function g0(e){for(var t=[],n=e==null?void 0:e.parentElement,r=["hidden","scroll","clip","auto"];n;){var a=Nu(n).getComputedStyle(n),l=a.overflowX,s=a.overflowY,u=a.overflow;[l,s,u].some(function(d){return r.includes(d)})&&t.push(n),n=n.parentElement}return t}function cu(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:1;return Number.isNaN(e)?t:e}function xc(e){return cu(parseFloat(e),0)}function Rw(e,t){var n=Q({},e);return(t||[]).forEach(function(r){if(!(r instanceof HTMLBodyElement||r instanceof HTMLHtmlElement)){var a=Nu(r).getComputedStyle(r),l=a.overflow,s=a.overflowClipMargin,u=a.borderTopWidth,d=a.borderBottomWidth,g=a.borderLeftWidth,h=a.borderRightWidth,p=r.getBoundingClientRect(),y=r.offsetHeight,C=r.clientHeight,b=r.offsetWidth,x=r.clientWidth,S=xc(u),E=xc(d),$=xc(g),O=xc(h),_=cu(Math.round(p.width/b*1e3)/1e3),R=cu(Math.round(p.height/y*1e3)/1e3),T=(b-x-$-O)*_,M=(y-C-S-E)*R,z=S*R,P=E*R,D=$*_,F=O*_,G=0,V=0;if(l==="clip"){var U=xc(s);G=U*_,V=U*R}var K=p.x+D-G,j=p.y+z-V,A=K+p.width+2*G-D-F-T,L=j+p.height+2*V-z-P-M;n.left=Math.max(n.left,K),n.top=Math.max(n.top,j),n.right=Math.min(n.right,A),n.bottom=Math.min(n.bottom,L)}}),n}function _w(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,n="".concat(t),r=n.match(/^(.*)\%$/);return r?e*(parseFloat(r[1])/100):parseFloat(n)}function Tw(e,t){var n=t||[],r=ge(n,2),a=r[0],l=r[1];return[_w(e.width,a),_w(e.height,l)]}function Mw(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"";return[e[0],e[1]]}function Fl(e,t){var n=t[0],r=t[1],a,l;return n==="t"?l=e.y:n==="b"?l=e.y+e.height:l=e.y+e.height/2,r==="l"?a=e.x:r==="r"?a=e.x+e.width:a=e.x+e.width/2,{x:a,y:l}}function ci(e,t){var n={t:"b",b:"t",l:"r",r:"l"};return e.map(function(r,a){return a===t?n[r]||"c":r}).join("")}function sB(e,t,n,r,a,l,s){var u=f.useState({ready:!1,offsetX:0,offsetY:0,offsetR:0,offsetB:0,arrowX:0,arrowY:0,scaleX:1,scaleY:1,align:a[r]||{}}),d=ge(u,2),g=d[0],h=d[1],p=f.useRef(0),y=f.useMemo(function(){return t?g0(t):[]},[t]),C=f.useRef({}),b=function(){C.current={}};e||b();var x=An(function(){if(t&&n&&e){let jr=function(qa,sa){var Tr=arguments.length>2&&arguments[2]!==void 0?arguments[2]:Te,ca=k.x+qa,xi=k.y+sa,wi=ca+be,il=xi+le,ao=Math.max(ca,Tr.left),Xe=Math.max(xi,Tr.top),mt=Math.min(wi,Tr.right),rn=Math.min(il,Tr.bottom);return Math.max(0,(mt-ao)*(rn-Xe))},ro=function(){tn=k.y+Fe,sn=tn+le,Tt=k.x+Ke,ar=Tt+be};var $,O,_,R,T=t,M=T.ownerDocument,z=Nu(T),P=z.getComputedStyle(T),D=P.width,F=P.height,G=P.position,V=T.style.left,U=T.style.top,K=T.style.right,j=T.style.bottom,A=T.style.overflow,L=Q(Q({},a[r]),l),q=M.createElement("div");($=T.parentElement)===null||$===void 0||$.appendChild(q),q.style.left="".concat(T.offsetLeft,"px"),q.style.top="".concat(T.offsetTop,"px"),q.style.position=G,q.style.height="".concat(T.offsetHeight,"px"),q.style.width="".concat(T.offsetWidth,"px"),T.style.left="0",T.style.top="0",T.style.right="auto",T.style.bottom="auto",T.style.overflow="hidden";var N;if(Array.isArray(n))N={x:n[0],y:n[1],width:0,height:0};else{var B,W,X=n.getBoundingClientRect();X.x=(B=X.x)!==null&&B!==void 0?B:X.left,X.y=(W=X.y)!==null&&W!==void 0?W:X.top,N={x:X.x,y:X.y,width:X.width,height:X.height}}var k=T.getBoundingClientRect();k.x=(O=k.x)!==null&&O!==void 0?O:k.left,k.y=(_=k.y)!==null&&_!==void 0?_:k.top;var Y=M.documentElement,ee=Y.clientWidth,ue=Y.clientHeight,ae=Y.scrollWidth,oe=Y.scrollHeight,he=Y.scrollTop,ie=Y.scrollLeft,le=k.height,be=k.width,Re=N.height,fe=N.width,Se={left:0,top:0,right:ee,bottom:ue},Ee={left:-ie,top:-he,right:ae-ie,bottom:oe-he},Oe=L.htmlRegion,Pe="visible",Ye="visibleFirst";Oe!=="scroll"&&Oe!==Ye&&(Oe=Pe);var Me=Oe===Ye,De=Rw(Ee,y),xe=Rw(Se,y),Te=Oe===Pe?xe:De,Le=Me?xe:Te;T.style.left="auto",T.style.top="auto",T.style.right="0",T.style.bottom="0";var We=T.getBoundingClientRect();T.style.left=V,T.style.top=U,T.style.right=K,T.style.bottom=j,T.style.overflow=A,(R=T.parentElement)===null||R===void 0||R.removeChild(q);var Ge=cu(Math.round(be/parseFloat(D)*1e3)/1e3),at=cu(Math.round(le/parseFloat(F)*1e3)/1e3);if(Ge===0||at===0||Jc(n)&&!vm(n))return;var ke=L.offset,Qe=L.targetOffset,ct=Tw(k,ke),pt=ge(ct,2),Ze=pt[0],Be=pt[1],Ue=Tw(N,Qe),Ie=ge(Ue,2),qe=Ie[0],wt=Ie[1];N.x-=qe,N.y-=wt;var It=L.points||[],Nn=ge(It,2),wn=Nn[0],Rn=Nn[1],dn=Mw(Rn),kt=Mw(wn),ln=Fl(N,dn),Ot=Fl(k,kt),nt=Q({},L),Ke=ln.x-Ot.x+Ze,Fe=ln.y-Ot.y+Be,vt=jr(Ke,Fe),Ct=jr(Ke,Fe,xe),Ve=Fl(N,["t","l"]),ze=Fl(k,["t","l"]),rt=Fl(N,["b","r"]),ot=Fl(k,["b","r"]),tt=L.overflow||{},ut=tt.adjustX,qt=tt.adjustY,Mt=tt.shiftX,Pt=tt.shiftY,At=function(sa){return typeof sa=="boolean"?sa:sa>=0},tn,sn,Tt,ar;ro();var Sn=At(qt),cn=kt[0]===dn[0];if(Sn&&kt[0]==="t"&&(sn>Le.bottom||C.current.bt)){var Ae=Fe;cn?Ae-=le-Re:Ae=Ve.y-ot.y-Be;var et=jr(Ke,Ae),bt=jr(Ke,Ae,xe);et>vt||et===vt&&(!Me||bt>=Ct)?(C.current.bt=!0,Fe=Ae,Be=-Be,nt.points=[ci(kt,0),ci(dn,0)]):C.current.bt=!1}if(Sn&&kt[0]==="b"&&(tn<Le.top||C.current.tb)){var Et=Fe;cn?Et+=le-Re:Et=rt.y-ze.y-Be;var Yt=jr(Ke,Et),Kn=jr(Ke,Et,xe);Yt>vt||Yt===vt&&(!Me||Kn>=Ct)?(C.current.tb=!0,Fe=Et,Be=-Be,nt.points=[ci(kt,0),ci(dn,0)]):C.current.tb=!1}var zn=At(ut),Fn=kt[1]===dn[1];if(zn&&kt[1]==="l"&&(ar>Le.right||C.current.rl)){var gr=Ke;Fn?gr-=be-fe:gr=Ve.x-ot.x-Ze;var Sr=jr(gr,Fe),_n=jr(gr,Fe,xe);Sr>vt||Sr===vt&&(!Me||_n>=Ct)?(C.current.rl=!0,Ke=gr,Ze=-Ze,nt.points=[ci(kt,1),ci(dn,1)]):C.current.rl=!1}if(zn&&kt[1]==="r"&&(Tt<Le.left||C.current.lr)){var or=Ke;Fn?or+=be-fe:or=rt.x-ze.x-Ze;var Fr=jr(or,Fe),ka=jr(or,Fe,xe);Fr>vt||Fr===vt&&(!Me||ka>=Ct)?(C.current.lr=!0,Ke=or,Ze=-Ze,nt.points=[ci(kt,1),ci(dn,1)]):C.current.lr=!1}ro();var Xn=Mt===!0?0:Mt;typeof Xn=="number"&&(Tt<xe.left&&(Ke-=Tt-xe.left-Ze,N.x+fe<xe.left+Xn&&(Ke+=N.x-xe.left+fe-Xn)),ar>xe.right&&(Ke-=ar-xe.right-Ze,N.x>xe.right-Xn&&(Ke+=N.x-xe.right+Xn)));var mn=Pt===!0?0:Pt;typeof mn=="number"&&(tn<xe.top&&(Fe-=tn-xe.top-Be,N.y+Re<xe.top+mn&&(Fe+=N.y-xe.top+Re-mn)),sn>xe.bottom&&(Fe-=sn-xe.bottom-Be,N.y>xe.bottom-mn&&(Fe+=N.y-xe.bottom+mn)));var Vr=k.x+Ke,la=Vr+be,Yn=k.y+Fe,Oa=Yn+le,Ht=N.x,Je=Ht+fe,He=N.y,Ft=He+Re,Wt=Math.max(Vr,Ht),Qt=Math.min(la,Je),Zt=(Wt+Qt)/2,ir=Zt-Vr,Vn=Math.max(Yn,He),jn=Math.min(Oa,Ft),lr=(Vn+jn)/2,sr=lr-Yn;s==null||s(t,nt);var _r=We.right-k.x-(Ke+k.width),Qn=We.bottom-k.y-(Fe+k.height);Ge===1&&(Ke=Math.round(Ke),_r=Math.round(_r)),at===1&&(Fe=Math.round(Fe),Qn=Math.round(Qn));var $s={ready:!0,offsetX:Ke/Ge,offsetY:Fe/at,offsetR:_r/Ge,offsetB:Qn/at,arrowX:ir/Ge,arrowY:sr/at,scaleX:Ge,scaleY:at,align:nt};h($s)}}),S=function(){p.current+=1;var O=p.current;Promise.resolve().then(function(){p.current===O&&x()})},E=function(){h(function(O){return Q(Q({},O),{},{ready:!1})})};return Gt(E,[r]),Gt(function(){e||E()},[e]),[g.ready,g.offsetX,g.offsetY,g.offsetR,g.offsetB,g.arrowX,g.arrowY,g.scaleX,g.scaleY,g.align,S]}function cB(e,t,n,r,a){Gt(function(){if(e&&t&&n){let p=function(){r(),a()};var l=t,s=n,u=g0(l),d=g0(s),g=Nu(s),h=new Set([g].concat(Ne(u),Ne(d)));return h.forEach(function(y){y.addEventListener("scroll",p,{passive:!0})}),g.addEventListener("resize",p,{passive:!0}),r(),function(){h.forEach(function(y){y.removeEventListener("scroll",p),g.removeEventListener("resize",p)})}}},[e,t,n])}function uB(e,t,n,r,a,l,s,u){var d=f.useRef(e);d.current=e;var g=f.useRef(!1);f.useEffect(function(){if(t&&r&&(!a||l)){var p=function(){g.current=!1},y=function(S){var E;d.current&&!s(((E=S.composedPath)===null||E===void 0||(E=E.call(S))===null||E===void 0?void 0:E[0])||S.target)&&!g.current&&u(!1)},C=Nu(r);C.addEventListener("pointerdown",p,!0),C.addEventListener("mousedown",y,!0),C.addEventListener("contextmenu",y,!0);var b=Td(n);return b&&(b.addEventListener("mousedown",y,!0),b.addEventListener("contextmenu",y,!0)),function(){C.removeEventListener("pointerdown",p,!0),C.removeEventListener("mousedown",y,!0),C.removeEventListener("contextmenu",y,!0),b&&(b.removeEventListener("mousedown",y,!0),b.removeEventListener("contextmenu",y,!0))}}},[t,n,r,a,l]);function h(){g.current=!0}return h}var fB=["prefixCls","children","action","showAction","hideAction","popupVisible","defaultPopupVisible","onPopupVisibleChange","afterPopupVisibleChange","mouseEnterDelay","mouseLeaveDelay","focusDelay","blurDelay","mask","maskClosable","getPopupContainer","forceRender","autoDestroy","destroyPopupOnHide","popup","popupClassName","popupStyle","popupPlacement","builtinPlacements","popupAlign","zIndex","stretch","getPopupClassNameFromAlign","fresh","alignPoint","onPopupClick","onPopupAlign","arrow","popupMotion","maskMotion","popupTransitionName","popupAnimation","maskTransitionName","maskAnimation","className","getTriggerDOMNode"];function dB(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:Rm,t=f.forwardRef(function(n,r){var a=n.prefixCls,l=a===void 0?"rc-trigger-popup":a,s=n.children,u=n.action,d=u===void 0?"hover":u,g=n.showAction,h=n.hideAction,p=n.popupVisible,y=n.defaultPopupVisible,C=n.onPopupVisibleChange,b=n.afterPopupVisibleChange,x=n.mouseEnterDelay,S=n.mouseLeaveDelay,E=S===void 0?.1:S,$=n.focusDelay,O=n.blurDelay,_=n.mask,R=n.maskClosable,T=R===void 0?!0:R,M=n.getPopupContainer,z=n.forceRender,P=n.autoDestroy,D=n.destroyPopupOnHide,F=n.popup,G=n.popupClassName,V=n.popupStyle,U=n.popupPlacement,K=n.builtinPlacements,j=K===void 0?{}:K,A=n.popupAlign,L=n.zIndex,q=n.stretch,N=n.getPopupClassNameFromAlign,B=n.fresh,W=n.alignPoint,X=n.onPopupClick,k=n.onPopupAlign,Y=n.arrow,ee=n.popupMotion,ue=n.maskMotion,ae=n.popupTransitionName,oe=n.popupAnimation,he=n.maskTransitionName,ie=n.maskAnimation,le=n.className,be=n.getTriggerDOMNode,Re=$t(n,fB),fe=P||D||!1,Se=f.useState(!1),Ee=ge(Se,2),Oe=Ee[0],Pe=Ee[1];Gt(function(){Pe(i_())},[]);var Ye=f.useRef({}),Me=f.useContext(Ew),De=f.useMemo(function(){return{registerSubPopup:function(mt,rn){Ye.current[mt]=rn,Me==null||Me.registerSubPopup(mt,rn)}}},[Me]),xe=vy(),Te=f.useState(null),Le=ge(Te,2),We=Le[0],Ge=Le[1],at=f.useRef(null),ke=An(function(Xe){at.current=Xe,Jc(Xe)&&We!==Xe&&Ge(Xe),Me==null||Me.registerSubPopup(xe,Xe)}),Qe=f.useState(null),ct=ge(Qe,2),pt=ct[0],Ze=ct[1],Be=f.useRef(null),Ue=An(function(Xe){Jc(Xe)&&pt!==Xe&&(Ze(Xe),Be.current=Xe)}),Ie=f.Children.only(s),qe=(Ie==null?void 0:Ie.props)||{},wt={},It=An(function(Xe){var mt,rn,gn=pt;return(gn==null?void 0:gn.contains(Xe))||((mt=Td(gn))===null||mt===void 0?void 0:mt.host)===Xe||Xe===gn||(We==null?void 0:We.contains(Xe))||((rn=Td(We))===null||rn===void 0?void 0:rn.host)===Xe||Xe===We||Object.values(Ye.current).some(function(En){return(En==null?void 0:En.contains(Xe))||Xe===En})}),Nn=Ow(l,ee,oe,ae),wn=Ow(l,ue,ie,he),Rn=f.useState(y||!1),dn=ge(Rn,2),kt=dn[0],ln=dn[1],Ot=p??kt,nt=An(function(Xe){p===void 0&&ln(Xe)});Gt(function(){ln(p||!1)},[p]);var Ke=f.useRef(Ot);Ke.current=Ot;var Fe=f.useRef([]);Fe.current=[];var vt=An(function(Xe){var mt;nt(Xe),((mt=Fe.current[Fe.current.length-1])!==null&&mt!==void 0?mt:Ot)!==Xe&&(Fe.current.push(Xe),C==null||C(Xe))}),Ct=f.useRef(),Ve=function(){clearTimeout(Ct.current)},ze=function(mt){var rn=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0;Ve(),rn===0?vt(mt):Ct.current=setTimeout(function(){vt(mt)},rn*1e3)};f.useEffect(function(){return Ve},[]);var rt=f.useState(!1),ot=ge(rt,2),tt=ot[0],ut=ot[1];Gt(function(Xe){(!Xe||Ot)&&ut(!0)},[Ot]);var qt=f.useState(null),Mt=ge(qt,2),Pt=Mt[0],At=Mt[1],tn=f.useState(null),sn=ge(tn,2),Tt=sn[0],ar=sn[1],Sn=function(mt){ar([mt.clientX,mt.clientY])},cn=sB(Ot,We,W&&Tt!==null?Tt:pt,U,j,A,k),Ae=ge(cn,11),et=Ae[0],bt=Ae[1],Et=Ae[2],Yt=Ae[3],Kn=Ae[4],zn=Ae[5],Fn=Ae[6],gr=Ae[7],Sr=Ae[8],_n=Ae[9],or=Ae[10],Fr=oB(Oe,d,g,h),ka=ge(Fr,2),Xn=ka[0],mn=ka[1],Vr=Xn.has("click"),la=mn.has("click")||mn.has("contextMenu"),Yn=An(function(){tt||or()}),Oa=function(){Ke.current&&W&&la&&ze(!1)};cB(Ot,pt,We,Yn,Oa),Gt(function(){Yn()},[Tt,U]),Gt(function(){Ot&&!(j!=null&&j[U])&&Yn()},[JSON.stringify(A)]);var Ht=f.useMemo(function(){var Xe=lB(j,l,_n,W);return re(Xe,N==null?void 0:N(_n))},[_n,N,j,l,W]);f.useImperativeHandle(r,function(){return{nativeElement:Be.current,popupElement:at.current,forceAlign:Yn}});var Je=f.useState(0),He=ge(Je,2),Ft=He[0],Wt=He[1],Qt=f.useState(0),Zt=ge(Qt,2),ir=Zt[0],Vn=Zt[1],jn=function(){if(q&&pt){var mt=pt.getBoundingClientRect();Wt(mt.width),Vn(mt.height)}},lr=function(){jn(),Yn()},sr=function(mt){ut(!1),or(),b==null||b(mt)},_r=function(){return new Promise(function(mt){jn(),At(function(){return mt})})};Gt(function(){Pt&&(or(),Pt(),At(null))},[Pt]);function Qn(Xe,mt,rn,gn){wt[Xe]=function(En){var oo;gn==null||gn(En),ze(mt,rn);for(var ll=arguments.length,Ei=new Array(ll>1?ll-1:0),io=1;io<ll;io++)Ei[io-1]=arguments[io];(oo=qe[Xe])===null||oo===void 0||oo.call.apply(oo,[qe,En].concat(Ei))}}(Vr||la)&&(wt.onClick=function(Xe){var mt;Ke.current&&la?ze(!1):!Ke.current&&Vr&&(Sn(Xe),ze(!0));for(var rn=arguments.length,gn=new Array(rn>1?rn-1:0),En=1;En<rn;En++)gn[En-1]=arguments[En];(mt=qe.onClick)===null||mt===void 0||mt.call.apply(mt,[qe,Xe].concat(gn))});var $s=uB(Ot,la,pt,We,_,T,It,ze),jr=Xn.has("hover"),ro=mn.has("hover"),qa,sa;jr&&(Qn("onMouseEnter",!0,x,function(Xe){Sn(Xe)}),Qn("onPointerEnter",!0,x,function(Xe){Sn(Xe)}),qa=function(mt){(Ot||tt)&&We!==null&&We!==void 0&&We.contains(mt.target)&&ze(!0,x)},W&&(wt.onMouseMove=function(Xe){var mt;(mt=qe.onMouseMove)===null||mt===void 0||mt.call(qe,Xe)})),ro&&(Qn("onMouseLeave",!1,E),Qn("onPointerLeave",!1,E),sa=function(){ze(!1,E)}),Xn.has("focus")&&Qn("onFocus",!0,$),mn.has("focus")&&Qn("onBlur",!1,O),Xn.has("contextMenu")&&(wt.onContextMenu=function(Xe){var mt;Ke.current&&mn.has("contextMenu")?ze(!1):(Sn(Xe),ze(!0)),Xe.preventDefault();for(var rn=arguments.length,gn=new Array(rn>1?rn-1:0),En=1;En<rn;En++)gn[En-1]=arguments[En];(mt=qe.onContextMenu)===null||mt===void 0||mt.call.apply(mt,[qe,Xe].concat(gn))}),le&&(wt.className=re(qe.className,le));var Tr=Q(Q({},qe),wt),ca={},xi=["onContextMenu","onClick","onMouseDown","onTouchStart","onMouseEnter","onMouseLeave","onFocus","onBlur"];xi.forEach(function(Xe){Re[Xe]&&(ca[Xe]=function(){for(var mt,rn=arguments.length,gn=new Array(rn),En=0;En<rn;En++)gn[En]=arguments[En];(mt=Tr[Xe])===null||mt===void 0||mt.call.apply(mt,[Tr].concat(gn)),Re[Xe].apply(Re,gn)})});var wi=f.cloneElement(Ie,Q(Q({},Tr),ca)),il={x:zn,y:Fn},ao=Y?Q({},Y!==!0?Y:{}):null;return f.createElement(f.Fragment,null,f.createElement(zo,{disabled:!Ot,ref:Ue,onResize:lr},f.createElement(aB,{getTriggerDOMNode:be},wi)),f.createElement(Ew.Provider,{value:De},f.createElement(rB,{portal:e,ref:ke,prefixCls:l,popup:F,className:re(G,Ht),style:V,target:pt,onMouseEnter:qa,onMouseLeave:sa,onPointerEnter:qa,zIndex:L,open:Ot,keepDom:tt,fresh:B,onClick:X,onPointerDownCapture:$s,mask:_,motion:Nn,maskMotion:wn,onVisibleChanged:sr,onPrepare:_r,forceRender:z,autoDestroy:fe,getPopupContainer:M,align:_n,arrow:ao,arrowPos:il,ready:et,offsetX:bt,offsetY:Et,offsetR:Yt,offsetB:Kn,onAlign:Yn,stretch:q,targetWidth:Ft/gr,targetHeight:ir/Sr})))});return t}const Ey=dB(Rm);var mB=["prefixCls","disabled","visible","children","popupElement","animation","transitionName","dropdownStyle","dropdownClassName","direction","placement","builtinPlacements","dropdownMatchSelectWidth","dropdownRender","dropdownAlign","getPopupContainer","empty","getTriggerDOMNode","onPopupVisibleChange","onPopupMouseEnter"],gB=function(t){var n=t===!0?0:1;return{bottomLeft:{points:["tl","bl"],offset:[0,4],overflow:{adjustX:n,adjustY:1},htmlRegion:"scroll"},bottomRight:{points:["tr","br"],offset:[0,4],overflow:{adjustX:n,adjustY:1},htmlRegion:"scroll"},topLeft:{points:["bl","tl"],offset:[0,-4],overflow:{adjustX:n,adjustY:1},htmlRegion:"scroll"},topRight:{points:["br","tr"],offset:[0,-4],overflow:{adjustX:n,adjustY:1},htmlRegion:"scroll"}}},hB=function(t,n){var r=t.prefixCls;t.disabled;var a=t.visible,l=t.children,s=t.popupElement,u=t.animation,d=t.transitionName,g=t.dropdownStyle,h=t.dropdownClassName,p=t.direction,y=p===void 0?"ltr":p,C=t.placement,b=t.builtinPlacements,x=t.dropdownMatchSelectWidth,S=t.dropdownRender,E=t.dropdownAlign,$=t.getPopupContainer,O=t.empty,_=t.getTriggerDOMNode,R=t.onPopupVisibleChange,T=t.onPopupMouseEnter,M=$t(t,mB),z="".concat(r,"-dropdown"),P=s;S&&(P=S(s));var D=f.useMemo(function(){return b||gB(x)},[b,x]),F=u?"".concat(z,"-").concat(u):d,G=typeof x=="number",V=f.useMemo(function(){return G?null:x===!1?"minWidth":"width"},[x,G]),U=g;G&&(U=Q(Q({},U),{},{width:x}));var K=f.useRef(null);return f.useImperativeHandle(n,function(){return{getPopupElement:function(){var A;return(A=K.current)===null||A===void 0?void 0:A.popupElement}}}),f.createElement(Ey,je({},M,{showAction:R?["click"]:[],hideAction:R?["click"]:[],popupPlacement:C||(y==="rtl"?"bottomRight":"bottomLeft"),builtinPlacements:D,prefixCls:z,popupTransitionName:F,popup:f.createElement("div",{onMouseEnter:T},P),ref:K,stretch:V,popupAlign:E,popupVisible:a,getPopupContainer:$,popupClassName:re(h,Z({},"".concat(z,"-empty"),O)),popupStyle:U,getTriggerDOMNode:_,onPopupVisibleChange:R}),l)},pB=f.forwardRef(hB);function Nw(e,t){var n=e.key,r;return"value"in e&&(r=e.value),n??(r!==void 0?r:"rc-index-key-".concat(t))}function h0(e){return typeof e<"u"&&!Number.isNaN(e)}function h_(e,t){var n=e||{},r=n.label,a=n.value,l=n.options,s=n.groupLabel,u=r||(t?"children":"label");return{label:u,value:a||"value",options:l||"options",groupLabel:s||u}}function vB(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=t.fieldNames,r=t.childrenAsData,a=[],l=h_(n,!1),s=l.label,u=l.value,d=l.options,g=l.groupLabel;function h(p,y){Array.isArray(p)&&p.forEach(function(C){if(y||!(d in C)){var b=C[u];a.push({key:Nw(C,a.length),groupOption:y,data:C,label:C[s],value:b})}else{var x=C[g];x===void 0&&r&&(x=C.label),a.push({key:Nw(C,a.length),group:!0,data:C,label:x}),h(C[d],!0)}})}return h(e,!1),a}function p0(e){var t=Q({},e);return"props"in t||Object.defineProperty(t,"props",{get:function(){return mr(!1,"Return type is option instead of Option instance. Please read value directly instead of reading from `props`."),t}}),t}var yB=function(t,n,r){if(!n||!n.length)return null;var a=!1,l=function u(d,g){var h=y2(g),p=h[0],y=h.slice(1);if(!p)return[d];var C=d.split(p);return a=a||C.length>1,C.reduce(function(b,x){return[].concat(Ne(b),Ne(u(x,y)))},[]).filter(Boolean)},s=l(t,n);return a?typeof r<"u"?s.slice(0,r):s:null},$y=f.createContext(null);function bB(e){var t=e.visible,n=e.values;if(!t)return null;var r=50;return f.createElement("span",{"aria-live":"polite",style:{width:0,height:0,position:"absolute",overflow:"hidden",opacity:0}},"".concat(n.slice(0,r).map(function(a){var l=a.label,s=a.value;return["number","string"].includes(st(l))?l:s}).join(", ")),n.length>r?", ...":null)}var SB=["id","prefixCls","className","showSearch","tagRender","direction","omitDomProps","displayValues","onDisplayValuesChange","emptyOptions","notFoundContent","onClear","mode","disabled","loading","getInputElement","getRawInputElement","open","defaultOpen","onDropdownVisibleChange","activeValue","onActiveValueChange","activeDescendantId","searchValue","autoClearSearchValue","onSearch","onSearchSplit","tokenSeparators","allowClear","prefix","suffixIcon","clearIcon","OptionList","animation","transitionName","dropdownStyle","dropdownClassName","dropdownMatchSelectWidth","dropdownRender","dropdownAlign","placement","builtinPlacements","getPopupContainer","showAction","onFocus","onBlur","onKeyUp","onKeyDown","onMouseDown"],CB=["value","onChange","removeIcon","placeholder","autoFocus","maxTagCount","maxTagTextLength","maxTagPlaceholder","choiceTransitionName","onInputKeyDown","onPopupScroll","tabIndex"],v0=function(t){return t==="tags"||t==="multiple"},xB=f.forwardRef(function(e,t){var n,r=e.id,a=e.prefixCls,l=e.className,s=e.showSearch,u=e.tagRender,d=e.direction,g=e.omitDomProps,h=e.displayValues,p=e.onDisplayValuesChange,y=e.emptyOptions,C=e.notFoundContent,b=C===void 0?"Not Found":C,x=e.onClear,S=e.mode,E=e.disabled,$=e.loading,O=e.getInputElement,_=e.getRawInputElement,R=e.open,T=e.defaultOpen,M=e.onDropdownVisibleChange,z=e.activeValue,P=e.onActiveValueChange,D=e.activeDescendantId,F=e.searchValue,G=e.autoClearSearchValue,V=e.onSearch,U=e.onSearchSplit,K=e.tokenSeparators,j=e.allowClear,A=e.prefix,L=e.suffixIcon,q=e.clearIcon,N=e.OptionList,B=e.animation,W=e.transitionName,X=e.dropdownStyle,k=e.dropdownClassName,Y=e.dropdownMatchSelectWidth,ee=e.dropdownRender,ue=e.dropdownAlign,ae=e.placement,oe=e.builtinPlacements,he=e.getPopupContainer,ie=e.showAction,le=ie===void 0?[]:ie,be=e.onFocus,Re=e.onBlur,fe=e.onKeyUp,Se=e.onKeyDown,Ee=e.onMouseDown,Oe=$t(e,SB),Pe=v0(S),Ye=(s!==void 0?s:Pe)||S==="combobox",Me=Q({},Oe);CB.forEach(function(Je){delete Me[Je]}),g==null||g.forEach(function(Je){delete Me[Je]});var De=f.useState(!1),xe=ge(De,2),Te=xe[0],Le=xe[1];f.useEffect(function(){Le(i_())},[]);var We=f.useRef(null),Ge=f.useRef(null),at=f.useRef(null),ke=f.useRef(null),Qe=f.useRef(null),ct=f.useRef(!1),pt=T8(),Ze=ge(pt,3),Be=Ze[0],Ue=Ze[1],Ie=Ze[2];f.useImperativeHandle(t,function(){var Je,He;return{focus:(Je=ke.current)===null||Je===void 0?void 0:Je.focus,blur:(He=ke.current)===null||He===void 0?void 0:He.blur,scrollTo:function(Wt){var Qt;return(Qt=Qe.current)===null||Qt===void 0?void 0:Qt.scrollTo(Wt)},nativeElement:We.current||Ge.current}});var qe=f.useMemo(function(){var Je;if(S!=="combobox")return F;var He=(Je=h[0])===null||Je===void 0?void 0:Je.value;return typeof He=="string"||typeof He=="number"?String(He):""},[F,S,h]),wt=S==="combobox"&&typeof O=="function"&&O()||null,It=typeof _=="function"&&_(),Nn=Io(Ge,It==null||(n=It.props)===null||n===void 0?void 0:n.ref),wn=f.useState(!1),Rn=ge(wn,2),dn=Rn[0],kt=Rn[1];Gt(function(){kt(!0)},[]);var ln=br(!1,{defaultValue:T,value:R}),Ot=ge(ln,2),nt=Ot[0],Ke=Ot[1],Fe=dn?nt:!1,vt=!b&&y;(E||vt&&Fe&&S==="combobox")&&(Fe=!1);var Ct=vt?!1:Fe,Ve=f.useCallback(function(Je){var He=Je!==void 0?Je:!Fe;E||(Ke(He),Fe!==He&&(M==null||M(He)))},[E,Fe,Ke,M]),ze=f.useMemo(function(){return(K||[]).some(function(Je){return[`
`,`\r
`].includes(Je)})},[K]),rt=f.useContext($y)||{},ot=rt.maxCount,tt=rt.rawValues,ut=function(He,Ft,Wt){if(!(Pe&&h0(ot)&&(tt==null?void 0:tt.size)>=ot)){var Qt=!0,Zt=He;P==null||P(null);var ir=yB(He,K,h0(ot)?ot-tt.size:void 0),Vn=Wt?null:ir;return S!=="combobox"&&Vn&&(Zt="",U==null||U(Vn),Ve(!1),Qt=!1),V&&qe!==Zt&&V(Zt,{source:Ft?"typing":"effect"}),Qt}},qt=function(He){!He||!He.trim()||V(He,{source:"submit"})};f.useEffect(function(){!Fe&&!Pe&&S!=="combobox"&&ut("",!1,!1)},[Fe]),f.useEffect(function(){nt&&E&&Ke(!1),E&&!ct.current&&Ue(!1)},[E]);var Mt=s_(),Pt=ge(Mt,2),At=Pt[0],tn=Pt[1],sn=f.useRef(!1),Tt=function(He){var Ft=At(),Wt=He.key,Qt=Wt==="Enter";if(Qt&&(S!=="combobox"&&He.preventDefault(),Fe||Ve(!0)),tn(!!qe),Wt==="Backspace"&&!Ft&&Pe&&!qe&&h.length){for(var Zt=Ne(h),ir=null,Vn=Zt.length-1;Vn>=0;Vn-=1){var jn=Zt[Vn];if(!jn.disabled){Zt.splice(Vn,1),ir=jn;break}}ir&&p(Zt,{type:"remove",values:[ir]})}for(var lr=arguments.length,sr=new Array(lr>1?lr-1:0),_r=1;_r<lr;_r++)sr[_r-1]=arguments[_r];if(Fe&&(!Qt||!sn.current)){var Qn;Qt&&(sn.current=!0),(Qn=Qe.current)===null||Qn===void 0||Qn.onKeyDown.apply(Qn,[He].concat(sr))}Se==null||Se.apply(void 0,[He].concat(sr))},ar=function(He){for(var Ft=arguments.length,Wt=new Array(Ft>1?Ft-1:0),Qt=1;Qt<Ft;Qt++)Wt[Qt-1]=arguments[Qt];if(Fe){var Zt;(Zt=Qe.current)===null||Zt===void 0||Zt.onKeyUp.apply(Zt,[He].concat(Wt))}He.key==="Enter"&&(sn.current=!1),fe==null||fe.apply(void 0,[He].concat(Wt))},Sn=function(He){var Ft=h.filter(function(Wt){return Wt!==He});p(Ft,{type:"remove",values:[He]})},cn=function(){sn.current=!1},Ae=f.useRef(!1),et=function(){Ue(!0),E||(be&&!Ae.current&&be.apply(void 0,arguments),le.includes("focus")&&Ve(!0)),Ae.current=!0},bt=function(){ct.current=!0,Ue(!1,function(){Ae.current=!1,ct.current=!1,Ve(!1)}),!E&&(qe&&(S==="tags"?V(qe,{source:"submit"}):S==="multiple"&&V("",{source:"blur"})),Re&&Re.apply(void 0,arguments))},Et=[];f.useEffect(function(){return function(){Et.forEach(function(Je){return clearTimeout(Je)}),Et.splice(0,Et.length)}},[]);var Yt=function(He){var Ft,Wt=He.target,Qt=(Ft=at.current)===null||Ft===void 0?void 0:Ft.getPopupElement();if(Qt&&Qt.contains(Wt)){var Zt=setTimeout(function(){var lr=Et.indexOf(Zt);if(lr!==-1&&Et.splice(lr,1),Ie(),!Te&&!Qt.contains(document.activeElement)){var sr;(sr=ke.current)===null||sr===void 0||sr.focus()}});Et.push(Zt)}for(var ir=arguments.length,Vn=new Array(ir>1?ir-1:0),jn=1;jn<ir;jn++)Vn[jn-1]=arguments[jn];Ee==null||Ee.apply(void 0,[He].concat(Vn))},Kn=f.useState({}),zn=ge(Kn,2),Fn=zn[1];function gr(){Fn({})}var Sr;It&&(Sr=function(He){Ve(He)}),M8(function(){var Je;return[We.current,(Je=at.current)===null||Je===void 0?void 0:Je.getPopupElement()]},Ct,Ve,!!It);var _n=f.useMemo(function(){return Q(Q({},e),{},{notFoundContent:b,open:Fe,triggerOpen:Ct,id:r,showSearch:Ye,multiple:Pe,toggleOpen:Ve})},[e,b,Ct,Fe,r,Ye,Pe,Ve]),or=!!L||$,Fr;or&&(Fr=f.createElement(Mm,{className:re("".concat(a,"-arrow"),Z({},"".concat(a,"-arrow-loading"),$)),customizeIcon:L,customizeIconProps:{loading:$,searchValue:qe,open:Fe,focused:Be,showSearch:Ye}}));var ka=function(){var He;x==null||x(),(He=ke.current)===null||He===void 0||He.focus(),p([],{type:"clear",values:h}),ut("",!1,!1)},Xn=R8(a,ka,h,j,q,E,qe,S),mn=Xn.allowClear,Vr=Xn.clearIcon,la=f.createElement(N,{ref:Qe}),Yn=re(a,l,Z(Z(Z(Z(Z(Z(Z(Z(Z(Z({},"".concat(a,"-focused"),Be),"".concat(a,"-multiple"),Pe),"".concat(a,"-single"),!Pe),"".concat(a,"-allow-clear"),j),"".concat(a,"-show-arrow"),or),"".concat(a,"-disabled"),E),"".concat(a,"-loading"),$),"".concat(a,"-open"),Fe),"".concat(a,"-customize-input"),wt),"".concat(a,"-show-search"),Ye)),Oa=f.createElement(pB,{ref:at,disabled:E,prefixCls:a,visible:Ct,popupElement:la,animation:B,transitionName:W,dropdownStyle:X,dropdownClassName:k,direction:d,dropdownMatchSelectWidth:Y,dropdownRender:ee,dropdownAlign:ue,placement:ae,builtinPlacements:oe,getPopupContainer:he,empty:y,getTriggerDOMNode:function(He){return Ge.current||He},onPopupVisibleChange:Sr,onPopupMouseEnter:gr},It?f.cloneElement(It,{ref:Nn}):f.createElement(J8,je({},e,{domRef:Ge,prefixCls:a,inputElement:wt,ref:ke,id:r,prefix:A,showSearch:Ye,autoClearSearchValue:G,mode:S,activeDescendantId:D,tagRender:u,values:h,open:Fe,onToggleOpen:Ve,activeValue:z,searchValue:qe,onSearch:ut,onSearchSubmit:qt,onRemove:Sn,tokenWithEnter:ze,onInputBlur:cn}))),Ht;return It?Ht=Oa:Ht=f.createElement("div",je({className:Yn},Me,{ref:We,onMouseDown:Yt,onKeyDown:Tt,onKeyUp:ar,onFocus:et,onBlur:bt}),f.createElement(bB,{visible:Be&&!Fe,values:h}),Oa,Fr,mn&&Vr),f.createElement(l_.Provider,{value:_n},Ht)}),Oy=function(){return null};Oy.isSelectOptGroup=!0;var Ry=function(){return null};Ry.isSelectOption=!0;var p_=f.forwardRef(function(e,t){var n=e.height,r=e.offsetY,a=e.offsetX,l=e.children,s=e.prefixCls,u=e.onInnerResize,d=e.innerProps,g=e.rtl,h=e.extra,p={},y={display:"flex",flexDirection:"column"};return r!==void 0&&(p={height:n,position:"relative",overflow:"hidden"},y=Q(Q({},y),{},Z(Z(Z(Z(Z({transform:"translateY(".concat(r,"px)")},g?"marginRight":"marginLeft",-a),"position","absolute"),"left",0),"right",0),"top",0))),f.createElement("div",{style:p},f.createElement(zo,{onResize:function(b){var x=b.offsetHeight;x&&u&&u()}},f.createElement("div",je({style:y,className:re(Z({},"".concat(s,"-holder-inner"),s)),ref:t},d),l,h)))});p_.displayName="Filler";function wB(e){var t=e.children,n=e.setRef,r=f.useCallback(function(a){n(a)},[]);return f.cloneElement(t,{ref:r})}function EB(e,t,n,r,a,l,s,u){var d=u.getKey;return e.slice(t,n+1).map(function(g,h){var p=t+h,y=s(g,p,{style:{width:r},offsetX:a}),C=d(g);return f.createElement(wB,{key:C,setRef:function(x){return l(g,x)}},y)})}function $B(e,t,n){var r=e.length,a=t.length,l,s;if(r===0&&a===0)return null;r<a?(l=e,s=t):(l=t,s=e);var u={__EMPTY_ITEM__:!0};function d(b){return b!==void 0?n(b):u}for(var g=null,h=Math.abs(r-a)!==1,p=0;p<s.length;p+=1){var y=d(l[p]),C=d(s[p]);if(y!==C){g=p,h=h||y!==d(s[p+1]);break}}return g===null?null:{index:g,multiple:h}}function OB(e,t,n){var r=f.useState(e),a=ge(r,2),l=a[0],s=a[1],u=f.useState(null),d=ge(u,2),g=d[0],h=d[1];return f.useEffect(function(){var p=$B(l||[],e||[],t);(p==null?void 0:p.index)!==void 0&&h(e[p.index]),s(e)},[e]),[g]}var jw=(typeof navigator>"u"?"undefined":st(navigator))==="object"&&/Firefox/i.test(navigator.userAgent);const v_=function(e,t,n,r){var a=f.useRef(!1),l=f.useRef(null);function s(){clearTimeout(l.current),a.current=!0,l.current=setTimeout(function(){a.current=!1},50)}var u=f.useRef({top:e,bottom:t,left:n,right:r});return u.current.top=e,u.current.bottom=t,u.current.left=n,u.current.right=r,function(d,g){var h=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!1,p=d?g<0&&u.current.left||g>0&&u.current.right:g<0&&u.current.top||g>0&&u.current.bottom;return h&&p?(clearTimeout(l.current),a.current=!1):(!p||a.current)&&s(),!a.current&&p}};function RB(e,t,n,r,a,l,s){var u=f.useRef(0),d=f.useRef(null),g=f.useRef(null),h=f.useRef(!1),p=v_(t,n,r,a);function y($,O){if(bn.cancel(d.current),!p(!1,O)){var _=$;if(!_._virtualHandled)_._virtualHandled=!0;else return;u.current+=O,g.current=O,jw||_.preventDefault(),d.current=bn(function(){var R=h.current?10:1;s(u.current*R,!1),u.current=0})}}function C($,O){s(O,!0),jw||$.preventDefault()}var b=f.useRef(null),x=f.useRef(null);function S($){if(e){bn.cancel(x.current),x.current=bn(function(){b.current=null},2);var O=$.deltaX,_=$.deltaY,R=$.shiftKey,T=O,M=_;(b.current==="sx"||!b.current&&R&&_&&!O)&&(T=_,M=0,b.current="sx");var z=Math.abs(T),P=Math.abs(M);b.current===null&&(b.current=l&&z>P?"x":"y"),b.current==="y"?y($,M):C($,T)}}function E($){e&&(h.current=$.detail===g.current)}return[S,E]}function _B(e,t,n,r){var a=f.useMemo(function(){return[new Map,[]]},[e,n.id,r]),l=ge(a,2),s=l[0],u=l[1],d=function(h){var p=arguments.length>1&&arguments[1]!==void 0?arguments[1]:h,y=s.get(h),C=s.get(p);if(y===void 0||C===void 0)for(var b=e.length,x=u.length;x<b;x+=1){var S,E=e[x],$=t(E);s.set($,x);var O=(S=n.get($))!==null&&S!==void 0?S:r;if(u[x]=(u[x-1]||0)+O,$===h&&(y=x),$===p&&(C=x),y!==void 0&&C!==void 0)break}return{top:u[y-1]||0,bottom:u[C]}};return d}var TB=function(){function e(){Wn(this,e),Z(this,"maps",void 0),Z(this,"id",0),Z(this,"diffKeys",new Set),this.maps=Object.create(null)}return Gn(e,[{key:"set",value:function(n,r){this.maps[n]=r,this.id+=1,this.diffKeys.add(n)}},{key:"get",value:function(n){return this.maps[n]}},{key:"resetRecord",value:function(){this.diffKeys.clear()}},{key:"getRecord",value:function(){return this.diffKeys}}]),e}();function Iw(e){var t=parseFloat(e);return isNaN(t)?0:t}function MB(e,t,n){var r=f.useState(0),a=ge(r,2),l=a[0],s=a[1],u=f.useRef(new Map),d=f.useRef(new TB),g=f.useRef(0);function h(){g.current+=1}function p(){var C=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1;h();var b=function(){var E=!1;u.current.forEach(function($,O){if($&&$.offsetParent){var _=$.offsetHeight,R=getComputedStyle($),T=R.marginTop,M=R.marginBottom,z=Iw(T),P=Iw(M),D=_+z+P;d.current.get(O)!==D&&(d.current.set(O,D),E=!0)}}),E&&s(function($){return $+1})};if(C)b();else{g.current+=1;var x=g.current;Promise.resolve().then(function(){x===g.current&&b()})}}function y(C,b){var x=e(C);u.current.get(x),b?(u.current.set(x,b),p()):u.current.delete(x)}return f.useEffect(function(){return h},[]),[y,p,d.current,l]}var Pw=14/15;function NB(e,t,n){var r=f.useRef(!1),a=f.useRef(0),l=f.useRef(0),s=f.useRef(null),u=f.useRef(null),d,g=function(C){if(r.current){var b=Math.ceil(C.touches[0].pageX),x=Math.ceil(C.touches[0].pageY),S=a.current-b,E=l.current-x,$=Math.abs(S)>Math.abs(E);$?a.current=b:l.current=x;var O=n($,$?S:E,!1,C);O&&C.preventDefault(),clearInterval(u.current),O&&(u.current=setInterval(function(){$?S*=Pw:E*=Pw;var _=Math.floor($?S:E);(!n($,_,!0)||Math.abs(_)<=.1)&&clearInterval(u.current)},16))}},h=function(){r.current=!1,d()},p=function(C){d(),C.touches.length===1&&!r.current&&(r.current=!0,a.current=Math.ceil(C.touches[0].pageX),l.current=Math.ceil(C.touches[0].pageY),s.current=C.target,s.current.addEventListener("touchmove",g,{passive:!1}),s.current.addEventListener("touchend",h,{passive:!0}))};d=function(){s.current&&(s.current.removeEventListener("touchmove",g),s.current.removeEventListener("touchend",h))},Gt(function(){return e&&t.current.addEventListener("touchstart",p,{passive:!0}),function(){var y;(y=t.current)===null||y===void 0||y.removeEventListener("touchstart",p),d(),clearInterval(u.current)}},[e])}function Aw(e){return Math.floor(Math.pow(e,.5))}function y0(e,t){var n="touches"in e?e.touches[0]:e;return n[t?"pageX":"pageY"]-window[t?"scrollX":"scrollY"]}function jB(e,t,n){f.useEffect(function(){var r=t.current;if(e&&r){var a=!1,l,s,u=function(){bn.cancel(l)},d=function y(){u(),l=bn(function(){n(s),y()})},g=function(C){if(!(C.target.draggable||C.button!==0)){var b=C;b._virtualHandled||(b._virtualHandled=!0,a=!0)}},h=function(){a=!1,u()},p=function(C){if(a){var b=y0(C,!1),x=r.getBoundingClientRect(),S=x.top,E=x.bottom;if(b<=S){var $=S-b;s=-Aw($),d()}else if(b>=E){var O=b-E;s=Aw(O),d()}else u()}};return r.addEventListener("mousedown",g),r.ownerDocument.addEventListener("mouseup",h),r.ownerDocument.addEventListener("mousemove",p),function(){r.removeEventListener("mousedown",g),r.ownerDocument.removeEventListener("mouseup",h),r.ownerDocument.removeEventListener("mousemove",p),u()}}},[e])}var IB=10;function PB(e,t,n,r,a,l,s,u){var d=f.useRef(),g=f.useState(null),h=ge(g,2),p=h[0],y=h[1];return Gt(function(){if(p&&p.times<IB){if(!e.current){y(function(q){return Q({},q)});return}l();var C=p.targetAlign,b=p.originAlign,x=p.index,S=p.offset,E=e.current.clientHeight,$=!1,O=C,_=null;if(E){for(var R=C||b,T=0,M=0,z=0,P=Math.min(t.length-1,x),D=0;D<=P;D+=1){var F=a(t[D]);M=T;var G=n.get(F);z=M+(G===void 0?r:G),T=z}for(var V=R==="top"?S:E-S,U=P;U>=0;U-=1){var K=a(t[U]),j=n.get(K);if(j===void 0){$=!0;break}if(V-=j,V<=0)break}switch(R){case"top":_=M-S;break;case"bottom":_=z-E+S;break;default:{var A=e.current.scrollTop,L=A+E;M<A?O="top":z>L&&(O="bottom")}}_!==null&&s(_),_!==p.lastTop&&($=!0)}$&&y(Q(Q({},p),{},{times:p.times+1,targetAlign:O,lastTop:_}))}},[p,e.current]),function(C){if(C==null){u();return}if(bn.cancel(d.current),typeof C=="number")s(C);else if(C&&st(C)==="object"){var b,x=C.align;"index"in C?b=C.index:b=t.findIndex(function($){return a($)===C.key});var S=C.offset,E=S===void 0?0:S;y({times:0,index:b,offset:E,originAlign:x})}}}var zw=f.forwardRef(function(e,t){var n=e.prefixCls,r=e.rtl,a=e.scrollOffset,l=e.scrollRange,s=e.onStartMove,u=e.onStopMove,d=e.onScroll,g=e.horizontal,h=e.spinSize,p=e.containerSize,y=e.style,C=e.thumbStyle,b=e.showScrollBar,x=f.useState(!1),S=ge(x,2),E=S[0],$=S[1],O=f.useState(null),_=ge(O,2),R=_[0],T=_[1],M=f.useState(null),z=ge(M,2),P=z[0],D=z[1],F=!r,G=f.useRef(),V=f.useRef(),U=f.useState(b),K=ge(U,2),j=K[0],A=K[1],L=f.useRef(),q=function(){b===!0||b===!1||(clearTimeout(L.current),A(!0),L.current=setTimeout(function(){A(!1)},3e3))},N=l-p||0,B=p-h||0,W=f.useMemo(function(){if(a===0||N===0)return 0;var ie=a/N;return ie*B},[a,N,B]),X=function(le){le.stopPropagation(),le.preventDefault()},k=f.useRef({top:W,dragging:E,pageY:R,startTop:P});k.current={top:W,dragging:E,pageY:R,startTop:P};var Y=function(le){$(!0),T(y0(le,g)),D(k.current.top),s(),le.stopPropagation(),le.preventDefault()};f.useEffect(function(){var ie=function(fe){fe.preventDefault()},le=G.current,be=V.current;return le.addEventListener("touchstart",ie,{passive:!1}),be.addEventListener("touchstart",Y,{passive:!1}),function(){le.removeEventListener("touchstart",ie),be.removeEventListener("touchstart",Y)}},[]);var ee=f.useRef();ee.current=N;var ue=f.useRef();ue.current=B,f.useEffect(function(){if(E){var ie,le=function(fe){var Se=k.current,Ee=Se.dragging,Oe=Se.pageY,Pe=Se.startTop;bn.cancel(ie);var Ye=G.current.getBoundingClientRect(),Me=p/(g?Ye.width:Ye.height);if(Ee){var De=(y0(fe,g)-Oe)*Me,xe=Pe;!F&&g?xe-=De:xe+=De;var Te=ee.current,Le=ue.current,We=Le?xe/Le:0,Ge=Math.ceil(We*Te);Ge=Math.max(Ge,0),Ge=Math.min(Ge,Te),ie=bn(function(){d(Ge,g)})}},be=function(){$(!1),u()};return window.addEventListener("mousemove",le,{passive:!0}),window.addEventListener("touchmove",le,{passive:!0}),window.addEventListener("mouseup",be,{passive:!0}),window.addEventListener("touchend",be,{passive:!0}),function(){window.removeEventListener("mousemove",le),window.removeEventListener("touchmove",le),window.removeEventListener("mouseup",be),window.removeEventListener("touchend",be),bn.cancel(ie)}}},[E]),f.useEffect(function(){return q(),function(){clearTimeout(L.current)}},[a]),f.useImperativeHandle(t,function(){return{delayHidden:q}});var ae="".concat(n,"-scrollbar"),oe={position:"absolute",visibility:j?null:"hidden"},he={position:"absolute",background:"rgba(0, 0, 0, 0.5)",borderRadius:99,cursor:"pointer",userSelect:"none"};return g?(oe.height=8,oe.left=0,oe.right=0,oe.bottom=0,he.height="100%",he.width=h,F?he.left=W:he.right=W):(oe.width=8,oe.top=0,oe.bottom=0,F?oe.right=0:oe.left=0,he.width="100%",he.height=h,he.top=W),f.createElement("div",{ref:G,className:re(ae,Z(Z(Z({},"".concat(ae,"-horizontal"),g),"".concat(ae,"-vertical"),!g),"".concat(ae,"-visible"),j)),style:Q(Q({},oe),y),onMouseDown:X,onMouseMove:q},f.createElement("div",{ref:V,className:re("".concat(ae,"-thumb"),Z({},"".concat(ae,"-thumb-moving"),E)),style:Q(Q({},he),C),onMouseDown:Y}))}),AB=20;function Dw(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:0,t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,n=e/t*e;return isNaN(n)&&(n=0),n=Math.max(n,AB),Math.floor(n)}var zB=["prefixCls","className","height","itemHeight","fullHeight","style","data","children","itemKey","virtual","direction","scrollWidth","component","onScroll","onVirtualScroll","onVisibleChange","innerProps","extraRender","styles","showScrollBar"],DB=[],LB={overflowY:"auto",overflowAnchor:"none"};function BB(e,t){var n=e.prefixCls,r=n===void 0?"rc-virtual-list":n,a=e.className,l=e.height,s=e.itemHeight,u=e.fullHeight,d=u===void 0?!0:u,g=e.style,h=e.data,p=e.children,y=e.itemKey,C=e.virtual,b=e.direction,x=e.scrollWidth,S=e.component,E=S===void 0?"div":S,$=e.onScroll,O=e.onVirtualScroll,_=e.onVisibleChange,R=e.innerProps,T=e.extraRender,M=e.styles,z=e.showScrollBar,P=z===void 0?"optional":z,D=$t(e,zB),F=f.useCallback(function(Ae){return typeof y=="function"?y(Ae):Ae==null?void 0:Ae[y]},[y]),G=MB(F),V=ge(G,4),U=V[0],K=V[1],j=V[2],A=V[3],L=!!(C!==!1&&l&&s),q=f.useMemo(function(){return Object.values(j.maps).reduce(function(Ae,et){return Ae+et},0)},[j.id,j.maps]),N=L&&h&&(Math.max(s*h.length,q)>l||!!x),B=b==="rtl",W=re(r,Z({},"".concat(r,"-rtl"),B),a),X=h||DB,k=f.useRef(),Y=f.useRef(),ee=f.useRef(),ue=f.useState(0),ae=ge(ue,2),oe=ae[0],he=ae[1],ie=f.useState(0),le=ge(ie,2),be=le[0],Re=le[1],fe=f.useState(!1),Se=ge(fe,2),Ee=Se[0],Oe=Se[1],Pe=function(){Oe(!0)},Ye=function(){Oe(!1)},Me={getKey:F};function De(Ae){he(function(et){var bt;typeof Ae=="function"?bt=Ae(et):bt=Ae;var Et=kt(bt);return k.current.scrollTop=Et,Et})}var xe=f.useRef({start:0,end:X.length}),Te=f.useRef(),Le=OB(X,F),We=ge(Le,1),Ge=We[0];Te.current=Ge;var at=f.useMemo(function(){if(!L)return{scrollHeight:void 0,start:0,end:X.length-1,offset:void 0};if(!N){var Ae;return{scrollHeight:((Ae=Y.current)===null||Ae===void 0?void 0:Ae.offsetHeight)||0,start:0,end:X.length-1,offset:void 0}}for(var et=0,bt,Et,Yt,Kn=X.length,zn=0;zn<Kn;zn+=1){var Fn=X[zn],gr=F(Fn),Sr=j.get(gr),_n=et+(Sr===void 0?s:Sr);_n>=oe&&bt===void 0&&(bt=zn,Et=et),_n>oe+l&&Yt===void 0&&(Yt=zn),et=_n}return bt===void 0&&(bt=0,Et=0,Yt=Math.ceil(l/s)),Yt===void 0&&(Yt=X.length-1),Yt=Math.min(Yt+1,X.length-1),{scrollHeight:et,start:bt,end:Yt,offset:Et}},[N,L,oe,X,A,l]),ke=at.scrollHeight,Qe=at.start,ct=at.end,pt=at.offset;xe.current.start=Qe,xe.current.end=ct,f.useLayoutEffect(function(){var Ae=j.getRecord();if(Ae.size===1){var et=Array.from(Ae)[0],bt=X[Qe];if(bt){var Et=F(bt);if(Et===et){var Yt=j.get(et),Kn=Yt-s;De(function(zn){return zn+Kn})}}}j.resetRecord()},[ke]);var Ze=f.useState({width:0,height:l}),Be=ge(Ze,2),Ue=Be[0],Ie=Be[1],qe=function(et){Ie({width:et.offsetWidth,height:et.offsetHeight})},wt=f.useRef(),It=f.useRef(),Nn=f.useMemo(function(){return Dw(Ue.width,x)},[Ue.width,x]),wn=f.useMemo(function(){return Dw(Ue.height,ke)},[Ue.height,ke]),Rn=ke-l,dn=f.useRef(Rn);dn.current=Rn;function kt(Ae){var et=Ae;return Number.isNaN(dn.current)||(et=Math.min(et,dn.current)),et=Math.max(et,0),et}var ln=oe<=0,Ot=oe>=Rn,nt=be<=0,Ke=be>=x,Fe=v_(ln,Ot,nt,Ke),vt=function(){return{x:B?-be:be,y:oe}},Ct=f.useRef(vt()),Ve=An(function(Ae){if(O){var et=Q(Q({},vt()),Ae);(Ct.current.x!==et.x||Ct.current.y!==et.y)&&(O(et),Ct.current=et)}});function ze(Ae,et){var bt=Ae;et?(pi.flushSync(function(){Re(bt)}),Ve()):De(bt)}function rt(Ae){var et=Ae.currentTarget.scrollTop;et!==oe&&De(et),$==null||$(Ae),Ve()}var ot=function(et){var bt=et,Et=x?x-Ue.width:0;return bt=Math.max(bt,0),bt=Math.min(bt,Et),bt},tt=An(function(Ae,et){et?(pi.flushSync(function(){Re(function(bt){var Et=bt+(B?-Ae:Ae);return ot(Et)})}),Ve()):De(function(bt){var Et=bt+Ae;return Et})}),ut=RB(L,ln,Ot,nt,Ke,!!x,tt),qt=ge(ut,2),Mt=qt[0],Pt=qt[1];NB(L,k,function(Ae,et,bt,Et){var Yt=Et;return Fe(Ae,et,bt)?!1:!Yt||!Yt._virtualHandled?(Yt&&(Yt._virtualHandled=!0),Mt({preventDefault:function(){},deltaX:Ae?et:0,deltaY:Ae?0:et}),!0):!1}),jB(N,k,function(Ae){De(function(et){return et+Ae})}),Gt(function(){function Ae(bt){var Et=ln&&bt.detail<0,Yt=Ot&&bt.detail>0;L&&!Et&&!Yt&&bt.preventDefault()}var et=k.current;return et.addEventListener("wheel",Mt,{passive:!1}),et.addEventListener("DOMMouseScroll",Pt,{passive:!0}),et.addEventListener("MozMousePixelScroll",Ae,{passive:!1}),function(){et.removeEventListener("wheel",Mt),et.removeEventListener("DOMMouseScroll",Pt),et.removeEventListener("MozMousePixelScroll",Ae)}},[L,ln,Ot]),Gt(function(){if(x){var Ae=ot(be);Re(Ae),Ve({x:Ae})}},[Ue.width,x]);var At=function(){var et,bt;(et=wt.current)===null||et===void 0||et.delayHidden(),(bt=It.current)===null||bt===void 0||bt.delayHidden()},tn=PB(k,X,j,s,F,function(){return K(!0)},De,At);f.useImperativeHandle(t,function(){return{nativeElement:ee.current,getScrollInfo:vt,scrollTo:function(et){function bt(Et){return Et&&st(Et)==="object"&&("left"in Et||"top"in Et)}bt(et)?(et.left!==void 0&&Re(ot(et.left)),tn(et.top)):tn(et)}}}),Gt(function(){if(_){var Ae=X.slice(Qe,ct+1);_(Ae,X)}},[Qe,ct,X]);var sn=_B(X,F,j,s),Tt=T==null?void 0:T({start:Qe,end:ct,virtual:N,offsetX:be,offsetY:pt,rtl:B,getSize:sn}),ar=EB(X,Qe,ct,x,be,U,p,Me),Sn=null;l&&(Sn=Q(Z({},d?"height":"maxHeight",l),LB),L&&(Sn.overflowY="hidden",x&&(Sn.overflowX="hidden"),Ee&&(Sn.pointerEvents="none")));var cn={};return B&&(cn.dir="rtl"),f.createElement("div",je({ref:ee,style:Q(Q({},g),{},{position:"relative"}),className:W},cn,D),f.createElement(zo,{onResize:qe},f.createElement(E,{className:"".concat(r,"-holder"),style:Sn,ref:k,onScroll:rt,onMouseEnter:At},f.createElement(p_,{prefixCls:r,height:ke,offsetX:be,offsetY:pt,scrollWidth:x,onInnerResize:K,ref:Y,innerProps:R,rtl:B,extra:Tt},ar))),N&&ke>l&&f.createElement(zw,{ref:wt,prefixCls:r,scrollOffset:oe,scrollRange:ke,rtl:B,onScroll:ze,onStartMove:Pe,onStopMove:Ye,spinSize:wn,containerSize:Ue.height,style:M==null?void 0:M.verticalScrollBar,thumbStyle:M==null?void 0:M.verticalScrollBarThumb,showScrollBar:P}),N&&x>Ue.width&&f.createElement(zw,{ref:It,prefixCls:r,scrollOffset:be,scrollRange:x,rtl:B,onScroll:ze,onStartMove:Pe,onStopMove:Ye,spinSize:Nn,containerSize:Ue.width,horizontal:!0,style:M==null?void 0:M.horizontalScrollBar,thumbStyle:M==null?void 0:M.horizontalScrollBarThumb,showScrollBar:P}))}var y_=f.forwardRef(BB);y_.displayName="List";function HB(){return/(mac\sos|macintosh)/i.test(navigator.appVersion)}var FB=["disabled","title","children","style","className"];function Lw(e){return typeof e=="string"||typeof e=="number"}var VB=function(t,n){var r=_8(),a=r.prefixCls,l=r.id,s=r.open,u=r.multiple,d=r.mode,g=r.searchValue,h=r.toggleOpen,p=r.notFoundContent,y=r.onPopupScroll,C=f.useContext($y),b=C.maxCount,x=C.flattenOptions,S=C.onActiveValue,E=C.defaultActiveFirstOption,$=C.onSelect,O=C.menuItemSelectedIcon,_=C.rawValues,R=C.fieldNames,T=C.virtual,M=C.direction,z=C.listHeight,P=C.listItemHeight,D=C.optionRender,F="".concat(a,"-item"),G=xu(function(){return x},[s,x],function(ie,le){return le[0]&&ie[1]!==le[1]}),V=f.useRef(null),U=f.useMemo(function(){return u&&h0(b)&&(_==null?void 0:_.size)>=b},[u,b,_==null?void 0:_.size]),K=function(le){le.preventDefault()},j=function(le){var be;(be=V.current)===null||be===void 0||be.scrollTo(typeof le=="number"?{index:le}:le)},A=f.useCallback(function(ie){return d==="combobox"?!1:_.has(ie)},[d,Ne(_).toString(),_.size]),L=function(le){for(var be=arguments.length>1&&arguments[1]!==void 0?arguments[1]:1,Re=G.length,fe=0;fe<Re;fe+=1){var Se=(le+fe*be+Re)%Re,Ee=G[Se]||{},Oe=Ee.group,Pe=Ee.data;if(!Oe&&!(Pe!=null&&Pe.disabled)&&(A(Pe.value)||!U))return Se}return-1},q=f.useState(function(){return L(0)}),N=ge(q,2),B=N[0],W=N[1],X=function(le){var be=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;W(le);var Re={source:be?"keyboard":"mouse"},fe=G[le];if(!fe){S(null,-1,Re);return}S(fe.value,le,Re)};f.useEffect(function(){X(E!==!1?L(0):-1)},[G.length,g]);var k=f.useCallback(function(ie){return d==="combobox"?String(ie).toLowerCase()===g.toLowerCase():_.has(ie)},[d,g,Ne(_).toString(),_.size]);f.useEffect(function(){var ie=setTimeout(function(){if(!u&&s&&_.size===1){var be=Array.from(_)[0],Re=G.findIndex(function(fe){var Se=fe.data;return Se.value===be});Re!==-1&&(X(Re),j(Re))}});if(s){var le;(le=V.current)===null||le===void 0||le.scrollTo(void 0)}return function(){return clearTimeout(ie)}},[s,g]);var Y=function(le){le!==void 0&&$(le,{selected:!_.has(le)}),u||h(!1)};if(f.useImperativeHandle(n,function(){return{onKeyDown:function(le){var be=le.which,Re=le.ctrlKey;switch(be){case lt.N:case lt.P:case lt.UP:case lt.DOWN:{var fe=0;if(be===lt.UP?fe=-1:be===lt.DOWN?fe=1:HB()&&Re&&(be===lt.N?fe=1:be===lt.P&&(fe=-1)),fe!==0){var Se=L(B+fe,fe);j(Se),X(Se,!0)}break}case lt.TAB:case lt.ENTER:{var Ee,Oe=G[B];Oe&&!(Oe!=null&&(Ee=Oe.data)!==null&&Ee!==void 0&&Ee.disabled)&&!U?Y(Oe.value):Y(void 0),s&&le.preventDefault();break}case lt.ESC:h(!1),s&&le.stopPropagation()}},onKeyUp:function(){},scrollTo:function(le){j(le)}}}),G.length===0)return f.createElement("div",{role:"listbox",id:"".concat(l,"_list"),className:"".concat(F,"-empty"),onMouseDown:K},p);var ee=Object.keys(R).map(function(ie){return R[ie]}),ue=function(le){return le.label};function ae(ie,le){var be=ie.group;return{role:be?"presentation":"option",id:"".concat(l,"_list_").concat(le)}}var oe=function(le){var be=G[le];if(!be)return null;var Re=be.data||{},fe=Re.value,Se=be.group,Ee=ta(Re,!0),Oe=ue(be);return be?f.createElement("div",je({"aria-label":typeof Oe=="string"&&!Se?Oe:null},Ee,{key:le},ae(be,le),{"aria-selected":k(fe)}),fe):null},he={role:"listbox",id:"".concat(l,"_list")};return f.createElement(f.Fragment,null,T&&f.createElement("div",je({},he,{style:{height:0,width:0,overflow:"hidden"}}),oe(B-1),oe(B),oe(B+1)),f.createElement(y_,{itemKey:"key",ref:V,data:G,height:z,itemHeight:P,fullHeight:!1,onMouseDown:K,onScroll:y,virtual:T,direction:M,innerProps:T?null:he},function(ie,le){var be=ie.group,Re=ie.groupOption,fe=ie.data,Se=ie.label,Ee=ie.value,Oe=fe.key;if(be){var Pe,Ye=(Pe=fe.title)!==null&&Pe!==void 0?Pe:Lw(Se)?Se.toString():void 0;return f.createElement("div",{className:re(F,"".concat(F,"-group"),fe.className),title:Ye},Se!==void 0?Se:Oe)}var Me=fe.disabled,De=fe.title;fe.children;var xe=fe.style,Te=fe.className,Le=$t(fe,FB),We=On(Le,ee),Ge=A(Ee),at=Me||!Ge&&U,ke="".concat(F,"-option"),Qe=re(F,ke,Te,Z(Z(Z(Z({},"".concat(ke,"-grouped"),Re),"".concat(ke,"-active"),B===le&&!at),"".concat(ke,"-disabled"),at),"".concat(ke,"-selected"),Ge)),ct=ue(ie),pt=!O||typeof O=="function"||Ge,Ze=typeof ct=="number"?ct:ct||Ee,Be=Lw(Ze)?Ze.toString():void 0;return De!==void 0&&(Be=De),f.createElement("div",je({},ta(We),T?{}:ae(ie,le),{"aria-selected":k(Ee),className:Qe,title:Be,onMouseMove:function(){B===le||at||X(le)},onClick:function(){at||Y(Ee)},style:xe}),f.createElement("div",{className:"".concat(ke,"-content")},typeof D=="function"?D(ie,{index:le}):Ze),f.isValidElement(O)||Ge,pt&&f.createElement(Mm,{className:"".concat(F,"-option-state"),customizeIcon:O,customizeIconProps:{value:Ee,disabled:at,isSelected:Ge}},Ge?"✓":null))}))},UB=f.forwardRef(VB);const kB=function(e,t){var n=f.useRef({values:new Map,options:new Map}),r=f.useMemo(function(){var l=n.current,s=l.values,u=l.options,d=e.map(function(p){if(p.label===void 0){var y;return Q(Q({},p),{},{label:(y=s.get(p.value))===null||y===void 0?void 0:y.label})}return p}),g=new Map,h=new Map;return d.forEach(function(p){g.set(p.value,p),h.set(p.value,t.get(p.value)||u.get(p.value))}),n.current.values=g,n.current.options=h,d},[e,t]),a=f.useCallback(function(l){return t.get(l)||n.current.options.get(l)},[t]);return[r,a]};function Dp(e,t){return m_(e).join("").toUpperCase().includes(t)}const qB=function(e,t,n,r,a){return f.useMemo(function(){if(!n||r===!1)return e;var l=t.options,s=t.label,u=t.value,d=[],g=typeof r=="function",h=n.toUpperCase(),p=g?r:function(C,b){return a?Dp(b[a],h):b[l]?Dp(b[s!=="children"?s:"label"],h):Dp(b[u],h)},y=g?function(C){return p0(C)}:function(C){return C};return e.forEach(function(C){if(C[l]){var b=p(n,y(C));if(b)d.push(C);else{var x=C[l].filter(function(S){return p(n,y(S))});x.length&&d.push(Q(Q({},C),{},Z({},l,x)))}return}p(n,y(C))&&d.push(C)}),d},[e,r,a,n,t])};var Bw=0,WB=$r();function GB(){var e;return WB?(e=Bw,Bw+=1):e="TEST_OR_SSR",e}function KB(e){var t=f.useState(),n=ge(t,2),r=n[0],a=n[1];return f.useEffect(function(){a("rc_select_".concat(GB()))},[]),e||r}var XB=["children","value"],YB=["children"];function QB(e){var t=e,n=t.key,r=t.props,a=r.children,l=r.value,s=$t(r,XB);return Q({key:n,value:l!==void 0?l:n,children:a},s)}function b_(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;return Lr(e).map(function(n,r){if(!f.isValidElement(n)||!n.type)return null;var a=n,l=a.type.isSelectOptGroup,s=a.key,u=a.props,d=u.children,g=$t(u,YB);return t||!l?QB(n):Q(Q({key:"__RC_SELECT_GRP__".concat(s===null?r:s,"__"),label:s},g),{},{options:b_(d)})}).filter(function(n){return n})}var ZB=function(t,n,r,a,l){return f.useMemo(function(){var s=t,u=!t;u&&(s=b_(n));var d=new Map,g=new Map,h=function(C,b,x){x&&typeof x=="string"&&C.set(b[x],b)},p=function y(C){for(var b=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,x=0;x<C.length;x+=1){var S=C[x];!S[r.options]||b?(d.set(S[r.value],S),h(g,S,r.label),h(g,S,a),h(g,S,l)):y(S[r.options],!0)}};return p(s),{options:s,valueOptions:d,labelOptions:g}},[t,n,r,a,l])};function Hw(e){var t=f.useRef();t.current=e;var n=f.useCallback(function(){return t.current.apply(t,arguments)},[]);return n}var JB=["id","mode","prefixCls","backfill","fieldNames","inputValue","searchValue","onSearch","autoClearSearchValue","onSelect","onDeselect","dropdownMatchSelectWidth","filterOption","filterSort","optionFilterProp","optionLabelProp","options","optionRender","children","defaultActiveFirstOption","menuItemSelectedIcon","virtual","direction","listHeight","listItemHeight","labelRender","value","defaultValue","labelInValue","onChange","maxCount"],e7=["inputValue"];function t7(e){return!e||st(e)!=="object"}var n7=f.forwardRef(function(e,t){var n=e.id,r=e.mode,a=e.prefixCls,l=a===void 0?"rc-select":a,s=e.backfill,u=e.fieldNames,d=e.inputValue,g=e.searchValue,h=e.onSearch,p=e.autoClearSearchValue,y=p===void 0?!0:p,C=e.onSelect,b=e.onDeselect,x=e.dropdownMatchSelectWidth,S=x===void 0?!0:x,E=e.filterOption,$=e.filterSort,O=e.optionFilterProp,_=e.optionLabelProp,R=e.options,T=e.optionRender,M=e.children,z=e.defaultActiveFirstOption,P=e.menuItemSelectedIcon,D=e.virtual,F=e.direction,G=e.listHeight,V=G===void 0?200:G,U=e.listItemHeight,K=U===void 0?20:U,j=e.labelRender,A=e.value,L=e.defaultValue,q=e.labelInValue,N=e.onChange,B=e.maxCount,W=$t(e,JB),X=KB(n),k=v0(r),Y=!!(!R&&M),ee=f.useMemo(function(){return E===void 0&&r==="combobox"?!1:E},[E,r]),ue=f.useMemo(function(){return h_(u,Y)},[JSON.stringify(u),Y]),ae=br("",{value:g!==void 0?g:d,postState:function(ze){return ze||""}}),oe=ge(ae,2),he=oe[0],ie=oe[1],le=ZB(R,M,ue,O,_),be=le.valueOptions,Re=le.labelOptions,fe=le.options,Se=f.useCallback(function(Ve){var ze=m_(Ve);return ze.map(function(rt){var ot,tt,ut,qt,Mt;if(t7(rt))ot=rt;else{var Pt;ut=rt.key,tt=rt.label,ot=(Pt=rt.value)!==null&&Pt!==void 0?Pt:ut}var At=be.get(ot);if(At){var tn;tt===void 0&&(tt=At==null?void 0:At[_||ue.label]),ut===void 0&&(ut=(tn=At==null?void 0:At.key)!==null&&tn!==void 0?tn:ot),qt=At==null?void 0:At.disabled,Mt=At==null?void 0:At.title}return{label:tt,value:ot,key:ut,disabled:qt,title:Mt}})},[ue,_,be]),Ee=br(L,{value:A}),Oe=ge(Ee,2),Pe=Oe[0],Ye=Oe[1],Me=f.useMemo(function(){var Ve,ze=k&&Pe===null?[]:Pe,rt=Se(ze);return r==="combobox"&&G8((Ve=rt[0])===null||Ve===void 0?void 0:Ve.value)?[]:rt},[Pe,Se,r,k]),De=kB(Me,be),xe=ge(De,2),Te=xe[0],Le=xe[1],We=f.useMemo(function(){if(!r&&Te.length===1){var Ve=Te[0];if(Ve.value===null&&(Ve.label===null||Ve.label===void 0))return[]}return Te.map(function(ze){var rt;return Q(Q({},ze),{},{label:(rt=typeof j=="function"?j(ze):ze.label)!==null&&rt!==void 0?rt:ze.value})})},[r,Te,j]),Ge=f.useMemo(function(){return new Set(Te.map(function(Ve){return Ve.value}))},[Te]);f.useEffect(function(){if(r==="combobox"){var Ve,ze=(Ve=Te[0])===null||Ve===void 0?void 0:Ve.value;ie(W8(ze)?String(ze):"")}},[Te]);var at=Hw(function(Ve,ze){var rt=ze??Ve;return Z(Z({},ue.value,Ve),ue.label,rt)}),ke=f.useMemo(function(){if(r!=="tags")return fe;var Ve=Ne(fe),ze=function(ot){return be.has(ot)};return Ne(Te).sort(function(rt,ot){return rt.value<ot.value?-1:1}).forEach(function(rt){var ot=rt.value;ze(ot)||Ve.push(at(ot,rt.label))}),Ve},[at,fe,be,Te,r]),Qe=qB(ke,ue,he,ee,O),ct=f.useMemo(function(){return r!=="tags"||!he||Qe.some(function(Ve){return Ve[O||"value"]===he})||Qe.some(function(Ve){return Ve[ue.value]===he})?Qe:[at(he)].concat(Ne(Qe))},[at,O,r,Qe,he,ue]),pt=function Ve(ze){var rt=Ne(ze).sort(function(ot,tt){return $(ot,tt,{searchValue:he})});return rt.map(function(ot){return Array.isArray(ot.options)?Q(Q({},ot),{},{options:ot.options.length>0?Ve(ot.options):ot.options}):ot})},Ze=f.useMemo(function(){return $?pt(ct):ct},[ct,$,he]),Be=f.useMemo(function(){return vB(Ze,{fieldNames:ue,childrenAsData:Y})},[Ze,ue,Y]),Ue=function(ze){var rt=Se(ze);if(Ye(rt),N&&(rt.length!==Te.length||rt.some(function(ut,qt){var Mt;return((Mt=Te[qt])===null||Mt===void 0?void 0:Mt.value)!==(ut==null?void 0:ut.value)}))){var ot=q?rt:rt.map(function(ut){return ut.value}),tt=rt.map(function(ut){return p0(Le(ut.value))});N(k?ot:ot[0],k?tt:tt[0])}},Ie=f.useState(null),qe=ge(Ie,2),wt=qe[0],It=qe[1],Nn=f.useState(0),wn=ge(Nn,2),Rn=wn[0],dn=wn[1],kt=z!==void 0?z:r!=="combobox",ln=f.useCallback(function(Ve,ze){var rt=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},ot=rt.source,tt=ot===void 0?"keyboard":ot;dn(ze),s&&r==="combobox"&&Ve!==null&&tt==="keyboard"&&It(String(Ve))},[s,r]),Ot=function(ze,rt,ot){var tt=function(){var Sn,cn=Le(ze);return[q?{label:cn==null?void 0:cn[ue.label],value:ze,key:(Sn=cn==null?void 0:cn.key)!==null&&Sn!==void 0?Sn:ze}:ze,p0(cn)]};if(rt&&C){var ut=tt(),qt=ge(ut,2),Mt=qt[0],Pt=qt[1];C(Mt,Pt)}else if(!rt&&b&&ot!=="clear"){var At=tt(),tn=ge(At,2),sn=tn[0],Tt=tn[1];b(sn,Tt)}},nt=Hw(function(Ve,ze){var rt,ot=k?ze.selected:!0;ot?rt=k?[].concat(Ne(Te),[Ve]):[Ve]:rt=Te.filter(function(tt){return tt.value!==Ve}),Ue(rt),Ot(Ve,ot),r==="combobox"?It(""):(!v0||y)&&(ie(""),It(""))}),Ke=function(ze,rt){Ue(ze);var ot=rt.type,tt=rt.values;(ot==="remove"||ot==="clear")&&tt.forEach(function(ut){Ot(ut.value,!1,ot)})},Fe=function(ze,rt){if(ie(ze),It(null),rt.source==="submit"){var ot=(ze||"").trim();if(ot){var tt=Array.from(new Set([].concat(Ne(Ge),[ot])));Ue(tt),Ot(ot,!0),ie("")}return}rt.source!=="blur"&&(r==="combobox"&&Ue(ze),h==null||h(ze))},vt=function(ze){var rt=ze;r!=="tags"&&(rt=ze.map(function(tt){var ut=Re.get(tt);return ut==null?void 0:ut.value}).filter(function(tt){return tt!==void 0}));var ot=Array.from(new Set([].concat(Ne(Ge),Ne(rt))));Ue(ot),ot.forEach(function(tt){Ot(tt,!0)})},Ct=f.useMemo(function(){var Ve=D!==!1&&S!==!1;return Q(Q({},le),{},{flattenOptions:Be,onActiveValue:ln,defaultActiveFirstOption:kt,onSelect:nt,menuItemSelectedIcon:P,rawValues:Ge,fieldNames:ue,virtual:Ve,direction:F,listHeight:V,listItemHeight:K,childrenAsData:Y,maxCount:B,optionRender:T})},[B,le,Be,ln,kt,nt,P,Ge,ue,D,S,F,V,K,Y,T]);return f.createElement($y.Provider,{value:Ct},f.createElement(xB,je({},W,{id:X,prefixCls:l,ref:t,omitDomProps:e7,mode:r,displayValues:We,onDisplayValuesChange:Ke,direction:F,searchValue:he,onSearch:Fe,autoClearSearchValue:y,onSearchSplit:vt,dropdownMatchSelectWidth:S,OptionList:UB,emptyOptions:!Be.length,activeValue:wt,activeDescendantId:"".concat(X,"_list_").concat(Rn)})))}),_y=n7;_y.Option=Ry;_y.OptGroup=Oy;function Fd(e,t,n){return re({[`${e}-status-success`]:t==="success",[`${e}-status-warning`]:t==="warning",[`${e}-status-error`]:t==="error",[`${e}-status-validating`]:t==="validating",[`${e}-has-feedback`]:n})}const Nm=(e,t)=>t||e,r7=()=>{const[,e]=Br(),[t]=Do("Empty"),r=new en(e.colorBgBase).toHsl().l<.5?{opacity:.65}:{};return f.createElement("svg",{style:r,width:"184",height:"152",viewBox:"0 0 184 152",xmlns:"http://www.w3.org/2000/svg"},f.createElement("title",null,(t==null?void 0:t.description)||"Empty"),f.createElement("g",{fill:"none",fillRule:"evenodd"},f.createElement("g",{transform:"translate(24 31.67)"},f.createElement("ellipse",{fillOpacity:".8",fill:"#F5F5F7",cx:"67.797",cy:"106.89",rx:"67.797",ry:"12.668"}),f.createElement("path",{d:"M122.034 69.674L98.109 40.229c-1.148-1.386-2.826-2.225-4.593-2.225h-51.44c-1.766 0-3.444.839-4.592 2.225L13.56 69.674v15.383h108.475V69.674z",fill:"#AEB8C2"}),f.createElement("path",{d:"M101.537 86.214L80.63 61.102c-1.001-1.207-2.507-1.867-4.048-1.867H31.724c-1.54 0-3.047.66-4.048 1.867L6.769 86.214v13.792h94.768V86.214z",fill:"url(#linearGradient-1)",transform:"translate(13.56)"}),f.createElement("path",{d:"M33.83 0h67.933a4 4 0 0 1 4 4v93.344a4 4 0 0 1-4 4H33.83a4 4 0 0 1-4-4V4a4 4 0 0 1 4-4z",fill:"#F5F5F7"}),f.createElement("path",{d:"M42.678 9.953h50.237a2 2 0 0 1 2 2V36.91a2 2 0 0 1-2 2H42.678a2 2 0 0 1-2-2V11.953a2 2 0 0 1 2-2zM42.94 49.767h49.713a2.262 2.262 0 1 1 0 4.524H42.94a2.262 2.262 0 0 1 0-4.524zM42.94 61.53h49.713a2.262 2.262 0 1 1 0 4.525H42.94a2.262 2.262 0 0 1 0-4.525zM121.813 105.032c-.775 3.071-3.497 5.36-6.735 5.36H20.515c-3.238 0-5.96-2.29-6.734-5.36a7.309 7.309 0 0 1-.222-1.79V69.675h26.318c2.907 0 5.25 2.448 5.25 5.42v.04c0 2.971 2.37 5.37 5.277 5.37h34.785c2.907 0 5.277-2.421 5.277-5.393V75.1c0-2.972 2.343-5.426 5.25-5.426h26.318v33.569c0 .617-.077 1.216-.221 1.789z",fill:"#DCE0E6"})),f.createElement("path",{d:"M149.121 33.292l-6.83 2.65a1 1 0 0 1-1.317-1.23l1.937-6.207c-2.589-2.944-4.109-6.534-4.109-10.408C138.802 8.102 148.92 0 161.402 0 173.881 0 184 8.102 184 18.097c0 9.995-10.118 18.097-22.599 18.097-4.528 0-8.744-1.066-12.28-2.902z",fill:"#DCE0E6"}),f.createElement("g",{transform:"translate(149.65 15.383)",fill:"#FFF"},f.createElement("ellipse",{cx:"20.654",cy:"3.167",rx:"2.849",ry:"2.815"}),f.createElement("path",{d:"M5.698 5.63H0L2.898.704zM9.259.704h4.985V5.63H9.259z"}))))},a7=()=>{const[,e]=Br(),[t]=Do("Empty"),{colorFill:n,colorFillTertiary:r,colorFillQuaternary:a,colorBgContainer:l}=e,{borderColor:s,shadowColor:u,contentColor:d}=f.useMemo(()=>({borderColor:new en(n).onBackground(l).toHexString(),shadowColor:new en(r).onBackground(l).toHexString(),contentColor:new en(a).onBackground(l).toHexString()}),[n,r,a,l]);return f.createElement("svg",{width:"64",height:"41",viewBox:"0 0 64 41",xmlns:"http://www.w3.org/2000/svg"},f.createElement("title",null,(t==null?void 0:t.description)||"Empty"),f.createElement("g",{transform:"translate(0 1)",fill:"none",fillRule:"evenodd"},f.createElement("ellipse",{fill:u,cx:"32",cy:"33",rx:"32",ry:"7"}),f.createElement("g",{fillRule:"nonzero",stroke:s},f.createElement("path",{d:"M55 12.76L44.854 1.258C44.367.474 43.656 0 42.907 0H21.093c-.749 0-1.46.474-1.947 1.257L9 12.761V22h46v-9.24z"}),f.createElement("path",{d:"M41.613 15.931c0-1.605.994-2.93 2.227-2.931H55v18.137C55 33.26 53.68 35 52.05 35h-40.1C10.32 35 9 33.259 9 31.137V13h11.16c1.233 0 2.227 1.323 2.227 2.928v.022c0 1.605 1.005 2.901 2.237 2.901h14.752c1.232 0 2.237-1.308 2.237-2.913v-.007z",fill:d}))))},o7=e=>{const{componentCls:t,margin:n,marginXS:r,marginXL:a,fontSize:l,lineHeight:s}=e;return{[t]:{marginInline:r,fontSize:l,lineHeight:s,textAlign:"center",[`${t}-image`]:{height:e.emptyImgHeight,marginBottom:r,opacity:e.opacityImage,img:{height:"100%"},svg:{maxWidth:"100%",height:"100%",margin:"auto"}},[`${t}-description`]:{color:e.colorTextDescription},[`${t}-footer`]:{marginTop:n},"&-normal":{marginBlock:a,color:e.colorTextDescription,[`${t}-description`]:{color:e.colorTextDescription},[`${t}-image`]:{height:e.emptyImgHeightMD}},"&-small":{marginBlock:r,color:e.colorTextDescription,[`${t}-image`]:{height:e.emptyImgHeightSM}}}}},i7=xn("Empty",e=>{const{componentCls:t,controlHeightLG:n,calc:r}=e,a=Kt(e,{emptyImgCls:`${t}-img`,emptyImgHeight:r(n).mul(2.5).equal(),emptyImgHeightMD:n,emptyImgHeightSM:r(n).mul(.875).equal()});return[o7(a)]});var l7=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const S_=f.createElement(r7,null),C_=f.createElement(a7,null),Co=e=>{const{className:t,rootClassName:n,prefixCls:r,image:a=S_,description:l,children:s,imageStyle:u,style:d,classNames:g,styles:h}=e,p=l7(e,["className","rootClassName","prefixCls","image","description","children","imageStyle","style","classNames","styles"]),{getPrefixCls:y,direction:C,className:b,style:x,classNames:S,styles:E}=Or("empty"),$=y("empty",r),[O,_,R]=i7($),[T]=Do("Empty"),M=typeof l<"u"?l:T==null?void 0:T.description,z=typeof M=="string"?M:"empty";let P=null;return typeof a=="string"?P=f.createElement("img",{alt:z,src:a}):P=a,O(f.createElement("div",Object.assign({className:re(_,R,$,b,{[`${$}-normal`]:a===C_,[`${$}-rtl`]:C==="rtl"},t,n,S.root,g==null?void 0:g.root),style:Object.assign(Object.assign(Object.assign(Object.assign({},E.root),x),h==null?void 0:h.root),d)},p),f.createElement("div",{className:re(`${$}-image`,S.image,g==null?void 0:g.image),style:Object.assign(Object.assign(Object.assign({},u),E.image),h==null?void 0:h.image)},P),M&&f.createElement("div",{className:re(`${$}-description`,S.description,g==null?void 0:g.description),style:Object.assign(Object.assign({},E.description),h==null?void 0:h.description)},M),s&&f.createElement("div",{className:re(`${$}-footer`,S.footer,g==null?void 0:g.footer),style:Object.assign(Object.assign({},E.footer),h==null?void 0:h.footer)},s)))};Co.PRESENTED_IMAGE_DEFAULT=S_;Co.PRESENTED_IMAGE_SIMPLE=C_;const s7=e=>{const{componentName:t}=e,{getPrefixCls:n}=f.useContext(dt),r=n("empty");switch(t){case"Table":case"List":return se.createElement(Co,{image:Co.PRESENTED_IMAGE_SIMPLE});case"Select":case"TreeSelect":case"Cascader":case"Transfer":case"Mentions":return se.createElement(Co,{image:Co.PRESENTED_IMAGE_SIMPLE,className:`${r}-small`});case"Table.filter":return null;default:return se.createElement(Co,null)}},Ty=function(e,t){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:void 0;var r,a;const{variant:l,[e]:s}=f.useContext(dt),u=f.useContext(IR),d=s==null?void 0:s.variant;let g;typeof t<"u"?g=t:n===!1?g="borderless":g=(a=(r=u??d)!==null&&r!==void 0?r:l)!==null&&a!==void 0?a:"outlined";const h=hA.includes(g);return[g,h]},c7=e=>{const n={overflow:{adjustX:!0,adjustY:!0,shiftY:!0},htmlRegion:e==="scroll"?"scroll":"visible",dynamicInset:!0};return{bottomLeft:Object.assign(Object.assign({},n),{points:["tl","bl"],offset:[0,4]}),bottomRight:Object.assign(Object.assign({},n),{points:["tr","br"],offset:[0,4]}),topLeft:Object.assign(Object.assign({},n),{points:["bl","tl"],offset:[0,-4]}),topRight:Object.assign(Object.assign({},n),{points:["br","tr"],offset:[0,-4]})}};function u7(e,t){return e||c7(t)}const Fw=e=>{const{optionHeight:t,optionFontSize:n,optionLineHeight:r,optionPadding:a}=e;return{position:"relative",display:"block",minHeight:t,padding:a,color:e.colorText,fontWeight:"normal",fontSize:n,lineHeight:r,boxSizing:"border-box"}},f7=e=>{const{antCls:t,componentCls:n}=e,r=`${n}-item`,a=`&${t}-slide-up-enter${t}-slide-up-enter-active`,l=`&${t}-slide-up-appear${t}-slide-up-appear-active`,s=`&${t}-slide-up-leave${t}-slide-up-leave-active`,u=`${n}-dropdown-placement-`,d=`${r}-option-selected`;return[{[`${n}-dropdown`]:Object.assign(Object.assign({},Rr(e)),{position:"absolute",top:-9999,zIndex:e.zIndexPopup,boxSizing:"border-box",padding:e.paddingXXS,overflow:"hidden",fontSize:e.fontSize,fontVariant:"initial",backgroundColor:e.colorBgElevated,borderRadius:e.borderRadiusLG,outline:"none",boxShadow:e.boxShadowSecondary,[`
          ${a}${u}bottomLeft,
          ${l}${u}bottomLeft
        `]:{animationName:uR},[`
          ${a}${u}topLeft,
          ${l}${u}topLeft,
          ${a}${u}topRight,
          ${l}${u}topRight
        `]:{animationName:dR},[`${s}${u}bottomLeft`]:{animationName:fR},[`
          ${s}${u}topLeft,
          ${s}${u}topRight
        `]:{animationName:mR},"&-hidden":{display:"none"},[r]:Object.assign(Object.assign({},Fw(e)),{cursor:"pointer",transition:`background ${e.motionDurationSlow} ease`,borderRadius:e.borderRadiusSM,"&-group":{color:e.colorTextDescription,fontSize:e.fontSizeSM,cursor:"default"},"&-option":{display:"flex","&-content":Object.assign({flex:"auto"},Pd),"&-state":{flex:"none",display:"flex",alignItems:"center"},[`&-active:not(${r}-option-disabled)`]:{backgroundColor:e.optionActiveBg},[`&-selected:not(${r}-option-disabled)`]:{color:e.optionSelectedColor,fontWeight:e.optionSelectedFontWeight,backgroundColor:e.optionSelectedBg,[`${r}-option-state`]:{color:e.colorPrimary}},"&-disabled":{[`&${r}-option-selected`]:{backgroundColor:e.colorBgContainerDisabled},color:e.colorTextDisabled,cursor:"not-allowed"},"&-grouped":{paddingInlineStart:e.calc(e.controlPaddingHorizontal).mul(2).equal()}},"&-empty":Object.assign(Object.assign({},Fw(e)),{color:e.colorTextDisabled})}),[`${d}:has(+ ${d})`]:{borderEndStartRadius:0,borderEndEndRadius:0,[`& + ${d}`]:{borderStartStartRadius:0,borderStartEndRadius:0}},"&-rtl":{direction:"rtl"}})},Bd(e,"slide-up"),Bd(e,"slide-down"),kx(e,"move-up"),kx(e,"move-down")]},d7=e=>{const{multipleSelectItemHeight:t,paddingXXS:n,lineWidth:r,INTERNAL_FIXED_ITEM_MARGIN:a}=e,l=e.max(e.calc(n).sub(r).equal(),0),s=e.max(e.calc(l).sub(a).equal(),0);return{basePadding:l,containerPadding:s,itemHeight:ve(t),itemLineHeight:ve(e.calc(t).sub(e.calc(e.lineWidth).mul(2)).equal())}},m7=e=>{const{multipleSelectItemHeight:t,selectHeight:n,lineWidth:r}=e;return e.calc(n).sub(t).div(2).sub(r).equal()},g7=e=>{const{componentCls:t,iconCls:n,borderRadiusSM:r,motionDurationSlow:a,paddingXS:l,multipleItemColorDisabled:s,multipleItemBorderColorDisabled:u,colorIcon:d,colorIconHover:g,INTERNAL_FIXED_ITEM_MARGIN:h}=e;return{[`${t}-selection-overflow`]:{position:"relative",display:"flex",flex:"auto",flexWrap:"wrap",maxWidth:"100%","&-item":{flex:"none",alignSelf:"center",maxWidth:"100%",display:"inline-flex"},[`${t}-selection-item`]:{display:"flex",alignSelf:"center",flex:"none",boxSizing:"border-box",maxWidth:"100%",marginBlock:h,borderRadius:r,cursor:"default",transition:`font-size ${a}, line-height ${a}, height ${a}`,marginInlineEnd:e.calc(h).mul(2).equal(),paddingInlineStart:l,paddingInlineEnd:e.calc(l).div(2).equal(),[`${t}-disabled&`]:{color:s,borderColor:u,cursor:"not-allowed"},"&-content":{display:"inline-block",marginInlineEnd:e.calc(l).div(2).equal(),overflow:"hidden",whiteSpace:"pre",textOverflow:"ellipsis"},"&-remove":Object.assign(Object.assign({},ms()),{display:"inline-flex",alignItems:"center",color:d,fontWeight:"bold",fontSize:10,lineHeight:"inherit",cursor:"pointer",[`> ${n}`]:{verticalAlign:"-0.2em"},"&:hover":{color:g}})}}}},h7=(e,t)=>{const{componentCls:n,INTERNAL_FIXED_ITEM_MARGIN:r}=e,a=`${n}-selection-overflow`,l=e.multipleSelectItemHeight,s=m7(e),u=t?`${n}-${t}`:"",d=d7(e);return{[`${n}-multiple${u}`]:Object.assign(Object.assign({},g7(e)),{[`${n}-selector`]:{display:"flex",alignItems:"center",width:"100%",height:"100%",paddingInline:d.basePadding,paddingBlock:d.containerPadding,borderRadius:e.borderRadius,[`${n}-disabled&`]:{background:e.multipleSelectorBgDisabled,cursor:"not-allowed"},"&:after":{display:"inline-block",width:0,margin:`${ve(r)} 0`,lineHeight:ve(l),visibility:"hidden",content:'"\\a0"'}},[`${n}-selection-item`]:{height:d.itemHeight,lineHeight:ve(d.itemLineHeight)},[`${n}-selection-wrap`]:{alignSelf:"flex-start","&:after":{lineHeight:ve(l),marginBlock:r}},[`${n}-prefix`]:{marginInlineStart:e.calc(e.inputPaddingHorizontalBase).sub(d.basePadding).equal()},[`${a}-item + ${a}-item,
        ${n}-prefix + ${n}-selection-wrap
      `]:{[`${n}-selection-search`]:{marginInlineStart:0},[`${n}-selection-placeholder`]:{insetInlineStart:0}},[`${a}-item-suffix`]:{minHeight:d.itemHeight,marginBlock:r},[`${n}-selection-search`]:{display:"inline-flex",position:"relative",maxWidth:"100%",marginInlineStart:e.calc(e.inputPaddingHorizontalBase).sub(s).equal(),"\n          &-input,\n          &-mirror\n        ":{height:l,fontFamily:e.fontFamily,lineHeight:ve(l),transition:`all ${e.motionDurationSlow}`},"&-input":{width:"100%",minWidth:4.1},"&-mirror":{position:"absolute",top:0,insetInlineStart:0,insetInlineEnd:"auto",zIndex:999,whiteSpace:"pre",visibility:"hidden"}},[`${n}-selection-placeholder`]:{position:"absolute",top:"50%",insetInlineStart:e.calc(e.inputPaddingHorizontalBase).sub(d.basePadding).equal(),insetInlineEnd:e.inputPaddingHorizontalBase,transform:"translateY(-50%)",transition:`all ${e.motionDurationSlow}`}})}};function Lp(e,t){const{componentCls:n}=e,r=t?`${n}-${t}`:"",a={[`${n}-multiple${r}`]:{fontSize:e.fontSize,[`${n}-selector`]:{[`${n}-show-search&`]:{cursor:"text"}},[`
        &${n}-show-arrow ${n}-selector,
        &${n}-allow-clear ${n}-selector
      `]:{paddingInlineEnd:e.calc(e.fontSizeIcon).add(e.controlPaddingHorizontal).equal()}}};return[h7(e,t),a]}const p7=e=>{const{componentCls:t}=e,n=Kt(e,{selectHeight:e.controlHeightSM,multipleSelectItemHeight:e.multipleItemHeightSM,borderRadius:e.borderRadiusSM,borderRadiusSM:e.borderRadiusXS}),r=Kt(e,{fontSize:e.fontSizeLG,selectHeight:e.controlHeightLG,multipleSelectItemHeight:e.multipleItemHeightLG,borderRadius:e.borderRadiusLG,borderRadiusSM:e.borderRadius});return[Lp(e),Lp(n,"sm"),{[`${t}-multiple${t}-sm`]:{[`${t}-selection-placeholder`]:{insetInline:e.calc(e.controlPaddingHorizontalSM).sub(e.lineWidth).equal()},[`${t}-selection-search`]:{marginInlineStart:2}}},Lp(r,"lg")]};function Bp(e,t){const{componentCls:n,inputPaddingHorizontalBase:r,borderRadius:a}=e,l=e.calc(e.controlHeight).sub(e.calc(e.lineWidth).mul(2)).equal(),s=t?`${n}-${t}`:"";return{[`${n}-single${s}`]:{fontSize:e.fontSize,height:e.controlHeight,[`${n}-selector`]:Object.assign(Object.assign({},Rr(e,!0)),{display:"flex",borderRadius:a,flex:"1 1 auto",[`${n}-selection-wrap:after`]:{lineHeight:ve(l)},[`${n}-selection-search`]:{position:"absolute",inset:0,width:"100%","&-input":{width:"100%",WebkitAppearance:"textfield"}},[`
          ${n}-selection-item,
          ${n}-selection-placeholder
        `]:{display:"block",padding:0,lineHeight:ve(l),transition:`all ${e.motionDurationSlow}, visibility 0s`,alignSelf:"center"},[`${n}-selection-placeholder`]:{transition:"none",pointerEvents:"none"},[["&:after",`${n}-selection-item:empty:after`,`${n}-selection-placeholder:empty:after`].join(",")]:{display:"inline-block",width:0,visibility:"hidden",content:'"\\a0"'}}),[`
        &${n}-show-arrow ${n}-selection-item,
        &${n}-show-arrow ${n}-selection-search,
        &${n}-show-arrow ${n}-selection-placeholder
      `]:{paddingInlineEnd:e.showArrowPaddingInlineEnd},[`&${n}-open ${n}-selection-item`]:{color:e.colorTextPlaceholder},[`&:not(${n}-customize-input)`]:{[`${n}-selector`]:{width:"100%",height:"100%",alignItems:"center",padding:`0 ${ve(r)}`,[`${n}-selection-search-input`]:{height:l,fontSize:e.fontSize},"&:after":{lineHeight:ve(l)}}},[`&${n}-customize-input`]:{[`${n}-selector`]:{"&:after":{display:"none"},[`${n}-selection-search`]:{position:"static",width:"100%"},[`${n}-selection-placeholder`]:{position:"absolute",insetInlineStart:0,insetInlineEnd:0,padding:`0 ${ve(r)}`,"&:after":{display:"none"}}}}}}}function v7(e){const{componentCls:t}=e,n=e.calc(e.controlPaddingHorizontalSM).sub(e.lineWidth).equal();return[Bp(e),Bp(Kt(e,{controlHeight:e.controlHeightSM,borderRadius:e.borderRadiusSM}),"sm"),{[`${t}-single${t}-sm`]:{[`&:not(${t}-customize-input)`]:{[`${t}-selector`]:{padding:`0 ${ve(n)}`},[`&${t}-show-arrow ${t}-selection-search`]:{insetInlineEnd:e.calc(n).add(e.calc(e.fontSize).mul(1.5)).equal()},[`
            &${t}-show-arrow ${t}-selection-item,
            &${t}-show-arrow ${t}-selection-placeholder
          `]:{paddingInlineEnd:e.calc(e.fontSize).mul(1.5).equal()}}}},Bp(Kt(e,{controlHeight:e.singleItemHeightLG,fontSize:e.fontSizeLG,borderRadius:e.borderRadiusLG}),"lg")]}const y7=e=>{const{fontSize:t,lineHeight:n,lineWidth:r,controlHeight:a,controlHeightSM:l,controlHeightLG:s,paddingXXS:u,controlPaddingHorizontal:d,zIndexPopupBase:g,colorText:h,fontWeightStrong:p,controlItemBgActive:y,controlItemBgHover:C,colorBgContainer:b,colorFillSecondary:x,colorBgContainerDisabled:S,colorTextDisabled:E,colorPrimaryHover:$,colorPrimary:O,controlOutline:_}=e,R=u*2,T=r*2,M=Math.min(a-R,a-T),z=Math.min(l-R,l-T),P=Math.min(s-R,s-T);return{INTERNAL_FIXED_ITEM_MARGIN:Math.floor(u/2),zIndexPopup:g+50,optionSelectedColor:h,optionSelectedFontWeight:p,optionSelectedBg:y,optionActiveBg:C,optionPadding:`${(a-t*n)/2}px ${d}px`,optionFontSize:t,optionLineHeight:n,optionHeight:a,selectorBg:b,clearBg:b,singleItemHeightLG:s,multipleItemBg:x,multipleItemBorderColor:"transparent",multipleItemHeight:M,multipleItemHeightSM:z,multipleItemHeightLG:P,multipleSelectorBgDisabled:S,multipleItemColorDisabled:E,multipleItemBorderColorDisabled:"transparent",showArrowPaddingInlineEnd:Math.ceil(e.fontSize*1.25),hoverBorderColor:$,activeBorderColor:O,activeOutlineColor:_,selectAffixPadding:u}},x_=(e,t)=>{const{componentCls:n,antCls:r,controlOutlineWidth:a}=e;return{[`&:not(${n}-customize-input) ${n}-selector`]:{border:`${ve(e.lineWidth)} ${e.lineType} ${t.borderColor}`,background:e.selectorBg},[`&:not(${n}-disabled):not(${n}-customize-input):not(${r}-pagination-size-changer)`]:{[`&:hover ${n}-selector`]:{borderColor:t.hoverBorderHover},[`${n}-focused& ${n}-selector`]:{borderColor:t.activeBorderColor,boxShadow:`0 0 0 ${ve(a)} ${t.activeOutlineColor}`,outline:0},[`${n}-prefix`]:{color:t.color}}}},Vw=(e,t)=>({[`&${e.componentCls}-status-${t.status}`]:Object.assign({},x_(e,t))}),b7=e=>({"&-outlined":Object.assign(Object.assign(Object.assign(Object.assign({},x_(e,{borderColor:e.colorBorder,hoverBorderHover:e.hoverBorderColor,activeBorderColor:e.activeBorderColor,activeOutlineColor:e.activeOutlineColor,color:e.colorText})),Vw(e,{status:"error",borderColor:e.colorError,hoverBorderHover:e.colorErrorHover,activeBorderColor:e.colorError,activeOutlineColor:e.colorErrorOutline,color:e.colorError})),Vw(e,{status:"warning",borderColor:e.colorWarning,hoverBorderHover:e.colorWarningHover,activeBorderColor:e.colorWarning,activeOutlineColor:e.colorWarningOutline,color:e.colorWarning})),{[`&${e.componentCls}-disabled`]:{[`&:not(${e.componentCls}-customize-input) ${e.componentCls}-selector`]:{background:e.colorBgContainerDisabled,color:e.colorTextDisabled}},[`&${e.componentCls}-multiple ${e.componentCls}-selection-item`]:{background:e.multipleItemBg,border:`${ve(e.lineWidth)} ${e.lineType} ${e.multipleItemBorderColor}`}})}),w_=(e,t)=>{const{componentCls:n,antCls:r}=e;return{[`&:not(${n}-customize-input) ${n}-selector`]:{background:t.bg,border:`${ve(e.lineWidth)} ${e.lineType} transparent`,color:t.color},[`&:not(${n}-disabled):not(${n}-customize-input):not(${r}-pagination-size-changer)`]:{[`&:hover ${n}-selector`]:{background:t.hoverBg},[`${n}-focused& ${n}-selector`]:{background:e.selectorBg,borderColor:t.activeBorderColor,outline:0}}}},Uw=(e,t)=>({[`&${e.componentCls}-status-${t.status}`]:Object.assign({},w_(e,t))}),S7=e=>({"&-filled":Object.assign(Object.assign(Object.assign(Object.assign({},w_(e,{bg:e.colorFillTertiary,hoverBg:e.colorFillSecondary,activeBorderColor:e.activeBorderColor,color:e.colorText})),Uw(e,{status:"error",bg:e.colorErrorBg,hoverBg:e.colorErrorBgHover,activeBorderColor:e.colorError,color:e.colorError})),Uw(e,{status:"warning",bg:e.colorWarningBg,hoverBg:e.colorWarningBgHover,activeBorderColor:e.colorWarning,color:e.colorWarning})),{[`&${e.componentCls}-disabled`]:{[`&:not(${e.componentCls}-customize-input) ${e.componentCls}-selector`]:{borderColor:e.colorBorder,background:e.colorBgContainerDisabled,color:e.colorTextDisabled}},[`&${e.componentCls}-multiple ${e.componentCls}-selection-item`]:{background:e.colorBgContainer,border:`${ve(e.lineWidth)} ${e.lineType} ${e.colorSplit}`}})}),C7=e=>({"&-borderless":{[`${e.componentCls}-selector`]:{background:"transparent",border:`${ve(e.lineWidth)} ${e.lineType} transparent`},[`&${e.componentCls}-disabled`]:{[`&:not(${e.componentCls}-customize-input) ${e.componentCls}-selector`]:{color:e.colorTextDisabled}},[`&${e.componentCls}-multiple ${e.componentCls}-selection-item`]:{background:e.multipleItemBg,border:`${ve(e.lineWidth)} ${e.lineType} ${e.multipleItemBorderColor}`},[`&${e.componentCls}-status-error`]:{[`${e.componentCls}-prefix, ${e.componentCls}-selection-item`]:{color:e.colorError}},[`&${e.componentCls}-status-warning`]:{[`${e.componentCls}-prefix, ${e.componentCls}-selection-item`]:{color:e.colorWarning}}}}),E_=(e,t)=>{const{componentCls:n,antCls:r}=e;return{[`&:not(${n}-customize-input) ${n}-selector`]:{borderWidth:`0 0 ${ve(e.lineWidth)} 0`,borderStyle:`none none ${e.lineType} none`,borderColor:t.borderColor,background:e.selectorBg,borderRadius:0},[`&:not(${n}-disabled):not(${n}-customize-input):not(${r}-pagination-size-changer)`]:{[`&:hover ${n}-selector`]:{borderColor:t.hoverBorderHover},[`${n}-focused& ${n}-selector`]:{borderColor:t.activeBorderColor,outline:0},[`${n}-prefix`]:{color:t.color}}}},kw=(e,t)=>({[`&${e.componentCls}-status-${t.status}`]:Object.assign({},E_(e,t))}),x7=e=>({"&-underlined":Object.assign(Object.assign(Object.assign(Object.assign({},E_(e,{borderColor:e.colorBorder,hoverBorderHover:e.hoverBorderColor,activeBorderColor:e.activeBorderColor,activeOutlineColor:e.activeOutlineColor,color:e.colorText})),kw(e,{status:"error",borderColor:e.colorError,hoverBorderHover:e.colorErrorHover,activeBorderColor:e.colorError,activeOutlineColor:e.colorErrorOutline,color:e.colorError})),kw(e,{status:"warning",borderColor:e.colorWarning,hoverBorderHover:e.colorWarningHover,activeBorderColor:e.colorWarning,activeOutlineColor:e.colorWarningOutline,color:e.colorWarning})),{[`&${e.componentCls}-disabled`]:{[`&:not(${e.componentCls}-customize-input) ${e.componentCls}-selector`]:{color:e.colorTextDisabled}},[`&${e.componentCls}-multiple ${e.componentCls}-selection-item`]:{background:e.multipleItemBg,border:`${ve(e.lineWidth)} ${e.lineType} ${e.multipleItemBorderColor}`}})}),w7=e=>({[e.componentCls]:Object.assign(Object.assign(Object.assign(Object.assign({},b7(e)),S7(e)),C7(e)),x7(e))}),E7=e=>{const{componentCls:t}=e;return{position:"relative",transition:`all ${e.motionDurationMid} ${e.motionEaseInOut}`,input:{cursor:"pointer"},[`${t}-show-search&`]:{cursor:"text",input:{cursor:"auto",color:"inherit",height:"100%"}},[`${t}-disabled&`]:{cursor:"not-allowed",input:{cursor:"not-allowed"}}}},$7=e=>{const{componentCls:t}=e;return{[`${t}-selection-search-input`]:{margin:0,padding:0,background:"transparent",border:"none",outline:"none",appearance:"none",fontFamily:"inherit","&::-webkit-search-cancel-button":{display:"none",appearance:"none"}}}},O7=e=>{const{antCls:t,componentCls:n,inputPaddingHorizontalBase:r,iconCls:a}=e;return{[n]:Object.assign(Object.assign({},Rr(e)),{position:"relative",display:"inline-flex",cursor:"pointer",[`&:not(${n}-customize-input) ${n}-selector`]:Object.assign(Object.assign({},E7(e)),$7(e)),[`${n}-selection-item`]:Object.assign(Object.assign({flex:1,fontWeight:"normal",position:"relative",userSelect:"none"},Pd),{[`> ${t}-typography`]:{display:"inline"}}),[`${n}-selection-placeholder`]:Object.assign(Object.assign({},Pd),{flex:1,color:e.colorTextPlaceholder,pointerEvents:"none"}),[`${n}-arrow`]:Object.assign(Object.assign({},ms()),{position:"absolute",top:"50%",insetInlineStart:"auto",insetInlineEnd:r,height:e.fontSizeIcon,marginTop:e.calc(e.fontSizeIcon).mul(-1).div(2).equal(),color:e.colorTextQuaternary,fontSize:e.fontSizeIcon,lineHeight:1,textAlign:"center",pointerEvents:"none",display:"flex",alignItems:"center",transition:`opacity ${e.motionDurationSlow} ease`,[a]:{verticalAlign:"top",transition:`transform ${e.motionDurationSlow}`,"> svg":{verticalAlign:"top"},[`&:not(${n}-suffix)`]:{pointerEvents:"auto"}},[`${n}-disabled &`]:{cursor:"not-allowed"},"> *:not(:last-child)":{marginInlineEnd:8}}),[`${n}-selection-wrap`]:{display:"flex",width:"100%",position:"relative",minWidth:0,"&:after":{content:'"\\a0"',width:0,overflow:"hidden"}},[`${n}-prefix`]:{flex:"none",marginInlineEnd:e.selectAffixPadding},[`${n}-clear`]:{position:"absolute",top:"50%",insetInlineStart:"auto",insetInlineEnd:r,zIndex:1,display:"inline-block",width:e.fontSizeIcon,height:e.fontSizeIcon,marginTop:e.calc(e.fontSizeIcon).mul(-1).div(2).equal(),color:e.colorTextQuaternary,fontSize:e.fontSizeIcon,fontStyle:"normal",lineHeight:1,textAlign:"center",textTransform:"none",cursor:"pointer",opacity:0,transition:`color ${e.motionDurationMid} ease, opacity ${e.motionDurationSlow} ease`,textRendering:"auto","&:before":{display:"block"},"&:hover":{color:e.colorIcon}},[`&:hover ${n}-clear`]:{opacity:1,background:e.colorBgBase,borderRadius:"50%"}}),[`${n}-status`]:{"&-error, &-warning, &-success, &-validating":{[`&${n}-has-feedback`]:{[`${n}-clear`]:{insetInlineEnd:e.calc(r).add(e.fontSize).add(e.paddingXS).equal()}}}}}},R7=e=>{const{componentCls:t}=e;return[{[t]:{[`&${t}-in-form-item`]:{width:"100%"}}},O7(e),v7(e),p7(e),f7(e),{[`${t}-rtl`]:{direction:"rtl"}},hy(e,{borderElCls:`${t}-selector`,focusElCls:`${t}-focused`})]},_7=xn("Select",(e,t)=>{let{rootPrefixCls:n}=t;const r=Kt(e,{rootPrefixCls:n,inputPaddingHorizontalBase:e.calc(e.paddingSM).sub(1).equal(),multipleSelectItemHeight:e.multipleItemHeight,selectHeight:e.controlHeight});return[R7(r),w7(r)]},y7,{unitless:{optionLineHeight:!0,optionSelectedFontWeight:!0}});function T7(e){let{suffixIcon:t,clearIcon:n,menuItemSelectedIcon:r,removeIcon:a,loading:l,multiple:s,hasFeedback:u,prefixCls:d,showSuffixIcon:g,feedbackIcon:h,showArrow:p,componentName:y}=e;const C=n??f.createElement(ss,null),b=$=>t===null&&!u&&!p?null:f.createElement(f.Fragment,null,g!==!1&&$,u&&h);let x=null;if(t!==void 0)x=b(t);else if(l)x=b(f.createElement(Cu,{spin:!0}));else{const $=`${d}-suffix`;x=O=>{let{open:_,showSearch:R}=O;return b(_&&R?f.createElement(UO,{className:$}):f.createElement(w4,{className:$}))}}let S=null;r!==void 0?S=r:s?S=f.createElement(HO,null):S=null;let E=null;return a!==void 0?E=a:E=f.createElement(cs,null),{clearIcon:C,suffixIcon:x,itemIcon:S,removeIcon:E}}function M7(e,t){return t!==void 0?t:e!==null}var N7=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const $_="SECRET_COMBOBOX_MODE_DO_NOT_USE",j7=(e,t)=>{var n;const{prefixCls:r,bordered:a,className:l,rootClassName:s,getPopupContainer:u,popupClassName:d,dropdownClassName:g,listHeight:h=256,placement:p,listItemHeight:y,size:C,disabled:b,notFoundContent:x,status:S,builtinPlacements:E,dropdownMatchSelectWidth:$,popupMatchSelectWidth:O,direction:_,style:R,allowClear:T,variant:M,dropdownStyle:z,transitionName:P,tagRender:D,maxCount:F,prefix:G}=e,V=N7(e,["prefixCls","bordered","className","rootClassName","getPopupContainer","popupClassName","dropdownClassName","listHeight","placement","listItemHeight","size","disabled","notFoundContent","status","builtinPlacements","dropdownMatchSelectWidth","popupMatchSelectWidth","direction","style","allowClear","variant","dropdownStyle","transitionName","tagRender","maxCount","prefix"]),{getPopupContainer:U,getPrefixCls:K,renderEmpty:j,direction:A,virtual:L,popupMatchSelectWidth:q,popupOverflow:N}=f.useContext(dt),B=Or("select"),[,W]=Br(),X=y??(W==null?void 0:W.controlHeight),k=K("select",r),Y=K(),ee=_??A,{compactSize:ue,compactItemClassnames:ae}=Ru(k,ee),[oe,he]=Ty("select",M,a),ie=ia(k),[le,be,Re]=_7(k,ie),fe=f.useMemo(()=>{const{mode:wt}=e;if(wt!=="combobox")return wt===$_?"combobox":wt},[e.mode]),Se=fe==="multiple"||fe==="tags",Ee=M7(e.suffixIcon,e.showArrow),Oe=(n=O??$)!==null&&n!==void 0?n:q,{status:Pe,hasFeedback:Ye,isFormItemInput:Me,feedbackIcon:De}=f.useContext(ra),xe=Nm(Pe,S);let Te;x!==void 0?Te=x:fe==="combobox"?Te=null:Te=(j==null?void 0:j("Select"))||f.createElement(s7,{componentName:"Select"});const{suffixIcon:Le,itemIcon:We,removeIcon:Ge,clearIcon:at}=T7(Object.assign(Object.assign({},V),{multiple:Se,hasFeedback:Ye,feedbackIcon:De,showSuffixIcon:Ee,prefixCls:k,componentName:"Select"})),ke=T===!0?{clearIcon:at}:T,Qe=On(V,["suffixIcon","itemIcon"]),ct=re(d||g,{[`${k}-dropdown-${ee}`]:ee==="rtl"},s,Re,ie,be),pt=Lo(wt=>{var It;return(It=C??ue)!==null&&It!==void 0?It:wt}),Ze=f.useContext(Ro),Be=b??Ze,Ue=re({[`${k}-lg`]:pt==="large",[`${k}-sm`]:pt==="small",[`${k}-rtl`]:ee==="rtl",[`${k}-${oe}`]:he,[`${k}-in-form-item`]:Me},Fd(k,xe,Ye),ae,B.className,l,s,Re,ie,be),Ie=f.useMemo(()=>p!==void 0?p:ee==="rtl"?"bottomRight":"bottomLeft",[p,ee]),[qe]=$u("SelectLike",z==null?void 0:z.zIndex);return le(f.createElement(_y,Object.assign({ref:t,virtual:L,showSearch:B.showSearch},Qe,{style:Object.assign(Object.assign({},B.style),R),dropdownMatchSelectWidth:Oe,transitionName:_o(Y,"slide-up",P),builtinPlacements:u7(E,N),listHeight:h,listItemHeight:X,mode:fe,prefixCls:k,placement:Ie,direction:ee,prefix:G,suffixIcon:Le,menuItemSelectedIcon:We,removeIcon:Ge,allowClear:ke,notFoundContent:Te,className:Ue,getPopupContainer:u||U,dropdownClassName:ct,disabled:Be,dropdownStyle:Object.assign(Object.assign({},z),{zIndex:qe}),maxCount:Se?F:void 0,tagRender:Se?D:void 0})))},bi=f.forwardRef(j7),I7=O8(bi,"dropdownAlign");bi.SECRET_COMBOBOX_MODE_DO_NOT_USE=$_;bi.Option=Ry;bi.OptGroup=Oy;bi._InternalPanelDoNotUseOrYouWillBeFired=I7;const uu=["xxl","xl","lg","md","sm","xs"],P7=e=>({xs:`(max-width: ${e.screenXSMax}px)`,sm:`(min-width: ${e.screenSM}px)`,md:`(min-width: ${e.screenMD}px)`,lg:`(min-width: ${e.screenLG}px)`,xl:`(min-width: ${e.screenXL}px)`,xxl:`(min-width: ${e.screenXXL}px)`}),A7=e=>{const t=e,n=[].concat(uu).reverse();return n.forEach((r,a)=>{const l=r.toUpperCase(),s=`screen${l}Min`,u=`screen${l}`;if(!(t[s]<=t[u]))throw new Error(`${s}<=${u} fails : !(${t[s]}<=${t[u]})`);if(a<n.length-1){const d=`screen${l}Max`;if(!(t[u]<=t[d]))throw new Error(`${u}<=${d} fails : !(${t[u]}<=${t[d]})`);const h=`screen${n[a+1].toUpperCase()}Min`;if(!(t[d]<=t[h]))throw new Error(`${d}<=${h} fails : !(${t[d]}<=${t[h]})`)}}),e},z7=()=>{const[,e]=Br(),t=P7(A7(e));return se.useMemo(()=>{const n=new Map;let r=-1,a={};return{responsiveMap:t,matchHandlers:{},dispatch(l){return a=l,n.forEach(s=>s(a)),n.size>=1},subscribe(l){return n.size||this.register(),r+=1,n.set(r,l),l(a),r},unsubscribe(l){n.delete(l),n.size||this.unregister()},register(){Object.keys(t).forEach(l=>{const s=t[l],u=g=>{let{matches:h}=g;this.dispatch(Object.assign(Object.assign({},a),{[l]:h}))},d=window.matchMedia(s);d.addListener(u),this.matchHandlers[s]={mql:d,listener:u},u(d)})},unregister(){Object.keys(t).forEach(l=>{const s=t[l],u=this.matchHandlers[s];u==null||u.mql.removeListener(u==null?void 0:u.listener)}),n.clear()}}},[e])};function D7(){const[,e]=f.useReducer(t=>t+1,0);return e}function L7(){let e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!0,t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};const n=f.useRef(t),r=D7(),a=z7();return Gt(()=>{const l=a.subscribe(s=>{n.current=s,e&&r()});return()=>a.unsubscribe(l)},[]),n.current}const as=e=>e?typeof e=="function"?e():e:null;function My(e){var t=e.children,n=e.prefixCls,r=e.id,a=e.overlayInnerStyle,l=e.bodyClassName,s=e.className,u=e.style;return f.createElement("div",{className:re("".concat(n,"-content"),s),style:u},f.createElement("div",{className:re("".concat(n,"-inner"),l),id:r,role:"tooltip",style:a},typeof t=="function"?t():t))}var Vl={shiftX:64,adjustY:1},Ul={adjustX:1,shiftY:!0},ba=[0,0],B7={left:{points:["cr","cl"],overflow:Ul,offset:[-4,0],targetOffset:ba},right:{points:["cl","cr"],overflow:Ul,offset:[4,0],targetOffset:ba},top:{points:["bc","tc"],overflow:Vl,offset:[0,-4],targetOffset:ba},bottom:{points:["tc","bc"],overflow:Vl,offset:[0,4],targetOffset:ba},topLeft:{points:["bl","tl"],overflow:Vl,offset:[0,-4],targetOffset:ba},leftTop:{points:["tr","tl"],overflow:Ul,offset:[-4,0],targetOffset:ba},topRight:{points:["br","tr"],overflow:Vl,offset:[0,-4],targetOffset:ba},rightTop:{points:["tl","tr"],overflow:Ul,offset:[4,0],targetOffset:ba},bottomRight:{points:["tr","br"],overflow:Vl,offset:[0,4],targetOffset:ba},rightBottom:{points:["bl","br"],overflow:Ul,offset:[4,0],targetOffset:ba},bottomLeft:{points:["tl","bl"],overflow:Vl,offset:[0,4],targetOffset:ba},leftBottom:{points:["br","bl"],overflow:Ul,offset:[-4,0],targetOffset:ba}},H7=["overlayClassName","trigger","mouseEnterDelay","mouseLeaveDelay","overlayStyle","prefixCls","children","onVisibleChange","afterVisibleChange","transitionName","animation","motion","placement","align","destroyTooltipOnHide","defaultVisible","getTooltipContainer","overlayInnerStyle","arrowContent","overlay","id","showArrow","classNames","styles"],F7=function(t,n){var r=t.overlayClassName,a=t.trigger,l=a===void 0?["hover"]:a,s=t.mouseEnterDelay,u=s===void 0?0:s,d=t.mouseLeaveDelay,g=d===void 0?.1:d,h=t.overlayStyle,p=t.prefixCls,y=p===void 0?"rc-tooltip":p,C=t.children,b=t.onVisibleChange,x=t.afterVisibleChange,S=t.transitionName,E=t.animation,$=t.motion,O=t.placement,_=O===void 0?"right":O,R=t.align,T=R===void 0?{}:R,M=t.destroyTooltipOnHide,z=M===void 0?!1:M,P=t.defaultVisible,D=t.getTooltipContainer,F=t.overlayInnerStyle;t.arrowContent;var G=t.overlay,V=t.id,U=t.showArrow,K=U===void 0?!0:U,j=t.classNames,A=t.styles,L=$t(t,H7),q=vy(V),N=f.useRef(null);f.useImperativeHandle(n,function(){return N.current});var B=Q({},L);"visible"in t&&(B.popupVisible=t.visible);var W=function(){return f.createElement(My,{key:"content",prefixCls:y,id:q,bodyClassName:j==null?void 0:j.body,overlayInnerStyle:Q(Q({},F),A==null?void 0:A.body)},G)},X=function(){var Y=f.Children.only(C),ee=(Y==null?void 0:Y.props)||{},ue=Q(Q({},ee),{},{"aria-describedby":G?q:null});return f.cloneElement(C,ue)};return f.createElement(Ey,je({popupClassName:re(r,j==null?void 0:j.root),prefixCls:y,popup:W,action:l,builtinPlacements:B7,popupPlacement:_,ref:N,popupAlign:T,getPopupContainer:D,onPopupVisibleChange:b,afterPopupVisibleChange:x,popupTransitionName:S,popupAnimation:E,popupMotion:$,defaultPopupVisible:P,autoDestroy:z,mouseLeaveDelay:g,popupStyle:Q(Q({},h),A==null?void 0:A.root),mouseEnterDelay:u,arrow:K},B),X())};const V7=f.forwardRef(F7);function O_(e){const{sizePopupArrow:t,borderRadiusXS:n,borderRadiusOuter:r}=e,a=t/2,l=0,s=a,u=r*1/Math.sqrt(2),d=a-r*(1-1/Math.sqrt(2)),g=a-n*(1/Math.sqrt(2)),h=r*(Math.sqrt(2)-1)+n*(1/Math.sqrt(2)),p=2*a-g,y=h,C=2*a-u,b=d,x=2*a-l,S=s,E=a*Math.sqrt(2)+r*(Math.sqrt(2)-2),$=r*(Math.sqrt(2)-1),O=`polygon(${$}px 100%, 50% ${$}px, ${2*a-$}px 100%, ${$}px 100%)`,_=`path('M ${l} ${s} A ${r} ${r} 0 0 0 ${u} ${d} L ${g} ${h} A ${n} ${n} 0 0 1 ${p} ${y} L ${C} ${b} A ${r} ${r} 0 0 0 ${x} ${S} Z')`;return{arrowShadowWidth:E,arrowPath:_,arrowPolygon:O}}const U7=(e,t,n)=>{const{sizePopupArrow:r,arrowPolygon:a,arrowPath:l,arrowShadowWidth:s,borderRadiusXS:u,calc:d}=e;return{pointerEvents:"none",width:r,height:r,overflow:"hidden","&::before":{position:"absolute",bottom:0,insetInlineStart:0,width:r,height:d(r).div(2).equal(),background:t,clipPath:{_multi_value_:!0,value:[a,l]},content:'""'},"&::after":{content:'""',position:"absolute",width:s,height:s,bottom:0,insetInline:0,margin:"auto",borderRadius:{_skip_check_:!0,value:`0 0 ${ve(u)} 0`},transform:"translateY(50%) rotate(-135deg)",boxShadow:n,zIndex:0,background:"transparent"}}},R_=8;function Ny(e){const{contentRadius:t,limitVerticalRadius:n}=e,r=t>12?t+2:12;return{arrowOffsetHorizontal:r,arrowOffsetVertical:n?R_:r}}function Jf(e,t){return e?t:{}}function __(e,t,n){const{componentCls:r,boxShadowPopoverArrow:a,arrowOffsetVertical:l,arrowOffsetHorizontal:s}=e,{arrowDistance:u=0,arrowPlacement:d={left:!0,right:!0,top:!0,bottom:!0}}=n||{};return{[r]:Object.assign(Object.assign(Object.assign(Object.assign({[`${r}-arrow`]:[Object.assign(Object.assign({position:"absolute",zIndex:1,display:"block"},U7(e,t,a)),{"&:before":{background:t}})]},Jf(!!d.top,{[[`&-placement-top > ${r}-arrow`,`&-placement-topLeft > ${r}-arrow`,`&-placement-topRight > ${r}-arrow`].join(",")]:{bottom:u,transform:"translateY(100%) rotate(180deg)"},[`&-placement-top > ${r}-arrow`]:{left:{_skip_check_:!0,value:"50%"},transform:"translateX(-50%) translateY(100%) rotate(180deg)"},"&-placement-topLeft":{"--arrow-offset-horizontal":s,[`> ${r}-arrow`]:{left:{_skip_check_:!0,value:s}}},"&-placement-topRight":{"--arrow-offset-horizontal":`calc(100% - ${ve(s)})`,[`> ${r}-arrow`]:{right:{_skip_check_:!0,value:s}}}})),Jf(!!d.bottom,{[[`&-placement-bottom > ${r}-arrow`,`&-placement-bottomLeft > ${r}-arrow`,`&-placement-bottomRight > ${r}-arrow`].join(",")]:{top:u,transform:"translateY(-100%)"},[`&-placement-bottom > ${r}-arrow`]:{left:{_skip_check_:!0,value:"50%"},transform:"translateX(-50%) translateY(-100%)"},"&-placement-bottomLeft":{"--arrow-offset-horizontal":s,[`> ${r}-arrow`]:{left:{_skip_check_:!0,value:s}}},"&-placement-bottomRight":{"--arrow-offset-horizontal":`calc(100% - ${ve(s)})`,[`> ${r}-arrow`]:{right:{_skip_check_:!0,value:s}}}})),Jf(!!d.left,{[[`&-placement-left > ${r}-arrow`,`&-placement-leftTop > ${r}-arrow`,`&-placement-leftBottom > ${r}-arrow`].join(",")]:{right:{_skip_check_:!0,value:u},transform:"translateX(100%) rotate(90deg)"},[`&-placement-left > ${r}-arrow`]:{top:{_skip_check_:!0,value:"50%"},transform:"translateY(-50%) translateX(100%) rotate(90deg)"},[`&-placement-leftTop > ${r}-arrow`]:{top:l},[`&-placement-leftBottom > ${r}-arrow`]:{bottom:l}})),Jf(!!d.right,{[[`&-placement-right > ${r}-arrow`,`&-placement-rightTop > ${r}-arrow`,`&-placement-rightBottom > ${r}-arrow`].join(",")]:{left:{_skip_check_:!0,value:u},transform:"translateX(-100%) rotate(-90deg)"},[`&-placement-right > ${r}-arrow`]:{top:{_skip_check_:!0,value:"50%"},transform:"translateY(-50%) translateX(-100%) rotate(-90deg)"},[`&-placement-rightTop > ${r}-arrow`]:{top:l},[`&-placement-rightBottom > ${r}-arrow`]:{bottom:l}}))}}function k7(e,t,n,r){if(r===!1)return{adjustX:!1,adjustY:!1};const a=r&&typeof r=="object"?r:{},l={};switch(e){case"top":case"bottom":l.shiftX=t.arrowOffsetHorizontal*2+n,l.shiftY=!0,l.adjustY=!0;break;case"left":case"right":l.shiftY=t.arrowOffsetVertical*2+n,l.shiftX=!0,l.adjustX=!0;break}const s=Object.assign(Object.assign({},l),a);return s.shiftX||(s.adjustX=!0),s.shiftY||(s.adjustY=!0),s}const qw={left:{points:["cr","cl"]},right:{points:["cl","cr"]},top:{points:["bc","tc"]},bottom:{points:["tc","bc"]},topLeft:{points:["bl","tl"]},leftTop:{points:["tr","tl"]},topRight:{points:["br","tr"]},rightTop:{points:["tl","tr"]},bottomRight:{points:["tr","br"]},rightBottom:{points:["bl","br"]},bottomLeft:{points:["tl","bl"]},leftBottom:{points:["br","bl"]}},q7={topLeft:{points:["bl","tc"]},leftTop:{points:["tr","cl"]},topRight:{points:["br","tc"]},rightTop:{points:["tl","cr"]},bottomRight:{points:["tr","bc"]},rightBottom:{points:["bl","cr"]},bottomLeft:{points:["tl","bc"]},leftBottom:{points:["br","cl"]}},W7=new Set(["topLeft","topRight","bottomLeft","bottomRight","leftTop","leftBottom","rightTop","rightBottom"]);function G7(e){const{arrowWidth:t,autoAdjustOverflow:n,arrowPointAtCenter:r,offset:a,borderRadius:l,visibleFirst:s}=e,u=t/2,d={};return Object.keys(qw).forEach(g=>{const h=r&&q7[g]||qw[g],p=Object.assign(Object.assign({},h),{offset:[0,0],dynamicInset:!0});switch(d[g]=p,W7.has(g)&&(p.autoArrow=!1),g){case"top":case"topLeft":case"topRight":p.offset[1]=-u-a;break;case"bottom":case"bottomLeft":case"bottomRight":p.offset[1]=u+a;break;case"left":case"leftTop":case"leftBottom":p.offset[0]=-u-a;break;case"right":case"rightTop":case"rightBottom":p.offset[0]=u+a;break}const y=Ny({contentRadius:l,limitVerticalRadius:!0});if(r)switch(g){case"topLeft":case"bottomLeft":p.offset[0]=-y.arrowOffsetHorizontal-u;break;case"topRight":case"bottomRight":p.offset[0]=y.arrowOffsetHorizontal+u;break;case"leftTop":case"rightTop":p.offset[1]=-y.arrowOffsetHorizontal*2+u;break;case"leftBottom":case"rightBottom":p.offset[1]=y.arrowOffsetHorizontal*2-u;break}p.overflow=k7(g,y,t,n),s&&(p.htmlRegion="visibleFirst")}),d}const K7=e=>{const{calc:t,componentCls:n,tooltipMaxWidth:r,tooltipColor:a,tooltipBg:l,tooltipBorderRadius:s,zIndexPopup:u,controlHeight:d,boxShadowSecondary:g,paddingSM:h,paddingXS:p,arrowOffsetHorizontal:y,sizePopupArrow:C}=e,b=t(s).add(C).add(y).equal(),x=t(s).mul(2).add(C).equal();return[{[n]:Object.assign(Object.assign(Object.assign(Object.assign({},Rr(e)),{position:"absolute",zIndex:u,display:"block",width:"max-content",maxWidth:r,visibility:"visible","--valid-offset-x":"var(--arrow-offset-horizontal, var(--arrow-x))",transformOrigin:["var(--valid-offset-x, 50%)","var(--arrow-y, 50%)"].join(" "),"&-hidden":{display:"none"},"--antd-arrow-background-color":l,[`${n}-inner`]:{minWidth:x,minHeight:d,padding:`${ve(e.calc(h).div(2).equal())} ${ve(p)}`,color:a,textAlign:"start",textDecoration:"none",wordWrap:"break-word",backgroundColor:l,borderRadius:s,boxShadow:g,boxSizing:"border-box"},[["&-placement-topLeft","&-placement-topRight","&-placement-bottomLeft","&-placement-bottomRight"].join(",")]:{minWidth:b},[["&-placement-left","&-placement-leftTop","&-placement-leftBottom","&-placement-right","&-placement-rightTop","&-placement-rightBottom"].join(",")]:{[`${n}-inner`]:{borderRadius:e.min(s,R_)}},[`${n}-content`]:{position:"relative"}}),P2(e,(S,E)=>{let{darkColor:$}=E;return{[`&${n}-${S}`]:{[`${n}-inner`]:{backgroundColor:$},[`${n}-arrow`]:{"--antd-arrow-background-color":$}}}})),{"&-rtl":{direction:"rtl"}})},__(e,"var(--antd-arrow-background-color)"),{[`${n}-pure`]:{position:"relative",maxWidth:"none",margin:e.sizePopupArrow}}]},X7=e=>Object.assign(Object.assign({zIndexPopup:e.zIndexPopupBase+70},Ny({contentRadius:e.borderRadius,limitVerticalRadius:!0})),O_(Kt(e,{borderRadiusOuter:Math.min(e.borderRadiusOuter,4)}))),T_=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0;return xn("Tooltip",r=>{const{borderRadius:a,colorTextLightSolid:l,colorBgSpotlight:s}=r,u=Kt(r,{tooltipMaxWidth:250,tooltipColor:l,tooltipBorderRadius:a,tooltipBg:s});return[K7(u),Sm(r,"zoom-big-fast")]},X7,{resetStyle:!1,injectStyle:t})(e)},Y7=vi.map(e=>`${e}-inverse`),Q7=["success","processing","error","default","warning"];function M_(e){return(arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0)?[].concat(Ne(Y7),Ne(vi)).includes(e):vi.includes(e)}function Z7(e){return Q7.includes(e)}function N_(e,t){const n=M_(t),r=re({[`${e}-${t}`]:t&&n}),a={},l={};return t&&!n&&(a.background=t,l["--antd-arrow-background-color"]=t),{className:r,overlayStyle:a,arrowStyle:l}}const J7=e=>{const{prefixCls:t,className:n,placement:r="top",title:a,color:l,overlayInnerStyle:s}=e,{getPrefixCls:u}=f.useContext(dt),d=u("tooltip",t),[g,h,p]=T_(d),y=N_(d,l),C=y.arrowStyle,b=Object.assign(Object.assign({},s),y.overlayStyle),x=re(h,p,d,`${d}-pure`,`${d}-placement-${r}`,n,y.className);return g(f.createElement("div",{className:x,style:C},f.createElement("div",{className:`${d}-arrow`}),f.createElement(My,Object.assign({},e,{className:h,prefixCls:d,overlayInnerStyle:b}),a)))};var eH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const tH=f.forwardRef((e,t)=>{var n,r;const{prefixCls:a,openClassName:l,getTooltipContainer:s,color:u,overlayInnerStyle:d,children:g,afterOpenChange:h,afterVisibleChange:p,destroyTooltipOnHide:y,arrow:C=!0,title:b,overlay:x,builtinPlacements:S,arrowPointAtCenter:E=!1,autoAdjustOverflow:$=!0,motion:O,getPopupContainer:_,placement:R="top",mouseEnterDelay:T=.1,mouseLeaveDelay:M=.1,overlayStyle:z,rootClassName:P,overlayClassName:D,styles:F,classNames:G}=e,V=eH(e,["prefixCls","openClassName","getTooltipContainer","color","overlayInnerStyle","children","afterOpenChange","afterVisibleChange","destroyTooltipOnHide","arrow","title","overlay","builtinPlacements","arrowPointAtCenter","autoAdjustOverflow","motion","getPopupContainer","placement","mouseEnterDelay","mouseLeaveDelay","overlayStyle","rootClassName","overlayClassName","styles","classNames"]),U=!!C,[,K]=Br(),{getPopupContainer:j,getPrefixCls:A,direction:L,className:q,style:N,classNames:B,styles:W}=Or("tooltip"),X=Eu(),k=f.useRef(null),Y=()=>{var Qe;(Qe=k.current)===null||Qe===void 0||Qe.forceAlign()};f.useImperativeHandle(t,()=>{var Qe,ct;return{forceAlign:Y,forcePopupAlign:()=>{X.deprecated(!1,"forcePopupAlign","forceAlign"),Y()},nativeElement:(Qe=k.current)===null||Qe===void 0?void 0:Qe.nativeElement,popupElement:(ct=k.current)===null||ct===void 0?void 0:ct.popupElement}});const[ee,ue]=br(!1,{value:(n=e.open)!==null&&n!==void 0?n:e.visible,defaultValue:(r=e.defaultOpen)!==null&&r!==void 0?r:e.defaultVisible}),ae=!b&&!x&&b!==0,oe=Qe=>{var ct,pt;ue(ae?!1:Qe),ae||((ct=e.onOpenChange)===null||ct===void 0||ct.call(e,Qe),(pt=e.onVisibleChange)===null||pt===void 0||pt.call(e,Qe))},he=f.useMemo(()=>{var Qe,ct;let pt=E;return typeof C=="object"&&(pt=(ct=(Qe=C.pointAtCenter)!==null&&Qe!==void 0?Qe:C.arrowPointAtCenter)!==null&&ct!==void 0?ct:E),S||G7({arrowPointAtCenter:pt,autoAdjustOverflow:$,arrowWidth:U?K.sizePopupArrow:0,borderRadius:K.borderRadius,offset:K.marginXXS,visibleFirst:!0})},[E,C,S,K]),ie=f.useMemo(()=>b===0?b:x||b||"",[x,b]),le=f.createElement(ns,{space:!0},typeof ie=="function"?ie():ie),be=A("tooltip",a),Re=A(),fe=e["data-popover-inject"];let Se=ee;!("open"in e)&&!("visible"in e)&&ae&&(Se=!1);const Ee=f.isValidElement(g)&&!X2(g)?g:f.createElement("span",null,g),Oe=Ee.props,Pe=!Oe.className||typeof Oe.className=="string"?re(Oe.className,l||`${be}-open`):Oe.className,[Ye,Me,De]=T_(be,!fe),xe=N_(be,u),Te=xe.arrowStyle,Le=re(D,{[`${be}-rtl`]:L==="rtl"},xe.className,P,Me,De,q,B.root,G==null?void 0:G.root),We=re(B.body,G==null?void 0:G.body),[Ge,at]=$u("Tooltip",V.zIndex),ke=f.createElement(V7,Object.assign({},V,{zIndex:Ge,showArrow:U,placement:R,mouseEnterDelay:T,mouseLeaveDelay:M,prefixCls:be,classNames:{root:Le,body:We},styles:{root:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},Te),W.root),N),z),F==null?void 0:F.root),body:Object.assign(Object.assign(Object.assign(Object.assign({},W.body),d),F==null?void 0:F.body),xe.overlayStyle)},getTooltipContainer:_||s||j,ref:k,builtinPlacements:he,overlay:le,visible:Se,onVisibleChange:oe,afterVisibleChange:h??p,arrowContent:f.createElement("span",{className:`${be}-arrow-content`}),motion:{motionName:_o(Re,"zoom-big-fast",e.transitionName),motionDeadline:1e3},destroyTooltipOnHide:!!y}),Se?na(Ee,{className:Pe}):Ee);return Ye(f.createElement(gm.Provider,{value:at},ke))}),Ho=tH;Ho._InternalPanelDoNotUseOrYouWillBeFired=J7;const nH=e=>{const{componentCls:t,popoverColor:n,titleMinWidth:r,fontWeightStrong:a,innerPadding:l,boxShadowSecondary:s,colorTextHeading:u,borderRadiusLG:d,zIndexPopup:g,titleMarginBottom:h,colorBgElevated:p,popoverBg:y,titleBorderBottom:C,innerContentPadding:b,titlePadding:x}=e;return[{[t]:Object.assign(Object.assign({},Rr(e)),{position:"absolute",top:0,left:{_skip_check_:!0,value:0},zIndex:g,fontWeight:"normal",whiteSpace:"normal",textAlign:"start",cursor:"auto",userSelect:"text","--valid-offset-x":"var(--arrow-offset-horizontal, var(--arrow-x))",transformOrigin:["var(--valid-offset-x, 50%)","var(--arrow-y, 50%)"].join(" "),"--antd-arrow-background-color":p,width:"max-content",maxWidth:"100vw","&-rtl":{direction:"rtl"},"&-hidden":{display:"none"},[`${t}-content`]:{position:"relative"},[`${t}-inner`]:{backgroundColor:y,backgroundClip:"padding-box",borderRadius:d,boxShadow:s,padding:l},[`${t}-title`]:{minWidth:r,marginBottom:h,color:u,fontWeight:a,borderBottom:C,padding:x},[`${t}-inner-content`]:{color:n,padding:b}})},__(e,"var(--antd-arrow-background-color)"),{[`${t}-pure`]:{position:"relative",maxWidth:"none",margin:e.sizePopupArrow,display:"inline-block",[`${t}-content`]:{display:"inline-block"}}}]},rH=e=>{const{componentCls:t}=e;return{[t]:vi.map(n=>{const r=e[`${n}6`];return{[`&${t}-${n}`]:{"--antd-arrow-background-color":r,[`${t}-inner`]:{backgroundColor:r},[`${t}-arrow`]:{background:"transparent"}}}})}},aH=e=>{const{lineWidth:t,controlHeight:n,fontHeight:r,padding:a,wireframe:l,zIndexPopupBase:s,borderRadiusLG:u,marginXS:d,lineType:g,colorSplit:h,paddingSM:p}=e,y=n-r,C=y/2,b=y/2-t,x=a;return Object.assign(Object.assign(Object.assign({titleMinWidth:177,zIndexPopup:s+30},O_(e)),Ny({contentRadius:u,limitVerticalRadius:!0})),{innerPadding:l?0:12,titleMarginBottom:l?0:d,titlePadding:l?`${C}px ${x}px ${b}px`:0,titleBorderBottom:l?`${t}px ${g} ${h}`:"none",innerContentPadding:l?`${p}px ${x}px`:0})},j_=xn("Popover",e=>{const{colorBgElevated:t,colorText:n}=e,r=Kt(e,{popoverBg:t,popoverColor:n});return[nH(r),rH(r),Sm(r,"zoom-big")]},aH,{resetStyle:!1,deprecatedTokens:[["width","titleMinWidth"],["minWidth","titleMinWidth"]]});var oH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const I_=e=>{let{title:t,content:n,prefixCls:r}=e;return!t&&!n?null:f.createElement(f.Fragment,null,t&&f.createElement("div",{className:`${r}-title`},t),n&&f.createElement("div",{className:`${r}-inner-content`},n))},iH=e=>{const{hashId:t,prefixCls:n,className:r,style:a,placement:l="top",title:s,content:u,children:d}=e,g=as(s),h=as(u),p=re(t,n,`${n}-pure`,`${n}-placement-${l}`,r);return f.createElement("div",{className:p,style:a},f.createElement("div",{className:`${n}-arrow`}),f.createElement(My,Object.assign({},e,{className:t,prefixCls:n}),d||f.createElement(I_,{prefixCls:n,title:g,content:h})))},P_=e=>{const{prefixCls:t,className:n}=e,r=oH(e,["prefixCls","className"]),{getPrefixCls:a}=f.useContext(dt),l=a("popover",t),[s,u,d]=j_(l);return s(f.createElement(iH,Object.assign({},r,{prefixCls:l,hashId:u,className:re(n,d)})))};var lH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const sH=f.forwardRef((e,t)=>{var n,r;const{prefixCls:a,title:l,content:s,overlayClassName:u,placement:d="top",trigger:g="hover",children:h,mouseEnterDelay:p=.1,mouseLeaveDelay:y=.1,onOpenChange:C,overlayStyle:b={},styles:x,classNames:S}=e,E=lH(e,["prefixCls","title","content","overlayClassName","placement","trigger","children","mouseEnterDelay","mouseLeaveDelay","onOpenChange","overlayStyle","styles","classNames"]),{getPrefixCls:$,className:O,style:_,classNames:R,styles:T}=Or("popover"),M=$("popover",a),[z,P,D]=j_(M),F=$(),G=re(u,P,D,O,R.root,S==null?void 0:S.root),V=re(R.body,S==null?void 0:S.body),[U,K]=br(!1,{value:(n=e.open)!==null&&n!==void 0?n:e.visible,defaultValue:(r=e.defaultOpen)!==null&&r!==void 0?r:e.defaultVisible}),j=(B,W)=>{K(B,!0),C==null||C(B,W)},A=B=>{B.keyCode===lt.ESC&&j(!1,B)},L=B=>{j(B)},q=as(l),N=as(s);return z(f.createElement(Ho,Object.assign({placement:d,trigger:g,mouseEnterDelay:p,mouseLeaveDelay:y},E,{prefixCls:M,classNames:{root:G,body:V},styles:{root:Object.assign(Object.assign(Object.assign(Object.assign({},T.root),_),b),x==null?void 0:x.root),body:Object.assign(Object.assign({},T.body),x==null?void 0:x.body)},ref:t,open:U,onOpenChange:L,overlay:q||N?f.createElement(I_,{prefixCls:M,title:q,content:N}):null,transitionName:_o(F,"zoom-big",E.transitionName),"data-popover-inject":!0}),na(h,{onKeyDown:B=>{var W,X;f.isValidElement(h)&&((X=h==null?void 0:(W=h.props).onKeyDown)===null||X===void 0||X.call(W,B)),A(B)}})))}),A_=sH;A_._InternalPanelDoNotUseOrYouWillBeFired=P_;var z_=f.createContext(null);function D_(e,t){return e===void 0?null:"".concat(e,"-").concat(t)}function L_(e){var t=f.useContext(z_);return D_(t,e)}var cH=["children","locked"],Da=f.createContext(null);function uH(e,t){var n=Q({},e);return Object.keys(t).forEach(function(r){var a=t[r];a!==void 0&&(n[r]=a)}),n}function fu(e){var t=e.children,n=e.locked,r=$t(e,cH),a=f.useContext(Da),l=xu(function(){return uH(a,r)},[a,r],function(s,u){return!n&&(s[0]!==u[0]||!ru(s[1],u[1],!0))});return f.createElement(Da.Provider,{value:l},t)}var fH=[],B_=f.createContext(null);function jm(){return f.useContext(B_)}var H_=f.createContext(fH);function ys(e){var t=f.useContext(H_);return f.useMemo(function(){return e!==void 0?[].concat(Ne(t),[e]):t},[t,e])}var F_=f.createContext(null),jy=f.createContext({});function Ww(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;if(vm(e)){var n=e.nodeName.toLowerCase(),r=["input","select","textarea","button"].includes(n)||e.isContentEditable||n==="a"&&!!e.getAttribute("href"),a=e.getAttribute("tabindex"),l=Number(a),s=null;return a&&!Number.isNaN(l)?s=l:r&&s===null&&(s=0),r&&e.disabled&&(s=null),s!==null&&(s>=0||t&&s<0)}return!1}function dH(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,n=Ne(e.querySelectorAll("*")).filter(function(r){return Ww(r,t)});return Ww(e,t)&&n.unshift(e),n}var b0=lt.LEFT,S0=lt.RIGHT,C0=lt.UP,yd=lt.DOWN,bd=lt.ENTER,V_=lt.ESC,wc=lt.HOME,Ec=lt.END,Gw=[C0,yd,b0,S0];function mH(e,t,n,r){var a,l="prev",s="next",u="children",d="parent";if(e==="inline"&&r===bd)return{inlineTrigger:!0};var g=Z(Z({},C0,l),yd,s),h=Z(Z(Z(Z({},b0,n?s:l),S0,n?l:s),yd,u),bd,u),p=Z(Z(Z(Z(Z(Z({},C0,l),yd,s),bd,u),V_,d),b0,n?u:d),S0,n?d:u),y={inline:g,horizontal:h,vertical:p,inlineSub:g,horizontalSub:p,verticalSub:p},C=(a=y["".concat(e).concat(t?"":"Sub")])===null||a===void 0?void 0:a[r];switch(C){case l:return{offset:-1,sibling:!0};case s:return{offset:1,sibling:!0};case d:return{offset:-1,sibling:!1};case u:return{offset:1,sibling:!1};default:return null}}function gH(e){for(var t=e;t;){if(t.getAttribute("data-menu-list"))return t;t=t.parentElement}return null}function hH(e,t){for(var n=e||document.activeElement;n;){if(t.has(n))return n;n=n.parentElement}return null}function Iy(e,t){var n=dH(e,!0);return n.filter(function(r){return t.has(r)})}function Kw(e,t,n){var r=arguments.length>3&&arguments[3]!==void 0?arguments[3]:1;if(!e)return null;var a=Iy(e,t),l=a.length,s=a.findIndex(function(u){return n===u});return r<0?s===-1?s=l-1:s-=1:r>0&&(s+=1),s=(s+l)%l,a[s]}var x0=function(t,n){var r=new Set,a=new Map,l=new Map;return t.forEach(function(s){var u=document.querySelector("[data-menu-id='".concat(D_(n,s),"']"));u&&(r.add(u),l.set(u,s),a.set(s,u))}),{elements:r,key2element:a,element2key:l}};function pH(e,t,n,r,a,l,s,u,d,g){var h=f.useRef(),p=f.useRef();p.current=t;var y=function(){bn.cancel(h.current)};return f.useEffect(function(){return function(){y()}},[]),function(C){var b=C.which;if([].concat(Gw,[bd,V_,wc,Ec]).includes(b)){var x=l(),S=x0(x,r),E=S,$=E.elements,O=E.key2element,_=E.element2key,R=O.get(t),T=hH(R,$),M=_.get(T),z=mH(e,s(M,!0).length===1,n,b);if(!z&&b!==wc&&b!==Ec)return;(Gw.includes(b)||[wc,Ec].includes(b))&&C.preventDefault();var P=function(A){if(A){var L=A,q=A.querySelector("a");q!=null&&q.getAttribute("href")&&(L=q);var N=_.get(A);u(N),y(),h.current=bn(function(){p.current===N&&L.focus()})}};if([wc,Ec].includes(b)||z.sibling||!T){var D;!T||e==="inline"?D=a.current:D=gH(T);var F,G=Iy(D,$);b===wc?F=G[0]:b===Ec?F=G[G.length-1]:F=Kw(D,$,T,z.offset),P(F)}else if(z.inlineTrigger)d(M);else if(z.offset>0)d(M,!0),y(),h.current=bn(function(){S=x0(x,r);var j=T.getAttribute("aria-controls"),A=document.getElementById(j),L=Kw(A,S.elements);P(L)},5);else if(z.offset<0){var V=s(M,!0),U=V[V.length-2],K=O.get(U);d(U,!1),P(K)}}g==null||g(C)}}function vH(e){Promise.resolve().then(e)}var Py="__RC_UTIL_PATH_SPLIT__",Xw=function(t){return t.join(Py)},yH=function(t){return t.split(Py)},w0="rc-menu-more";function bH(){var e=f.useState({}),t=ge(e,2),n=t[1],r=f.useRef(new Map),a=f.useRef(new Map),l=f.useState([]),s=ge(l,2),u=s[0],d=s[1],g=f.useRef(0),h=f.useRef(!1),p=function(){h.current||n({})},y=f.useCallback(function(O,_){var R=Xw(_);a.current.set(R,O),r.current.set(O,R),g.current+=1;var T=g.current;vH(function(){T===g.current&&p()})},[]),C=f.useCallback(function(O,_){var R=Xw(_);a.current.delete(R),r.current.delete(O)},[]),b=f.useCallback(function(O){d(O)},[]),x=f.useCallback(function(O,_){var R=r.current.get(O)||"",T=yH(R);return _&&u.includes(T[0])&&T.unshift(w0),T},[u]),S=f.useCallback(function(O,_){return O.filter(function(R){return R!==void 0}).some(function(R){var T=x(R,!0);return T.includes(_)})},[x]),E=function(){var _=Ne(r.current.keys());return u.length&&_.push(w0),_},$=f.useCallback(function(O){var _="".concat(r.current.get(O)).concat(Py),R=new Set;return Ne(a.current.keys()).forEach(function(T){T.startsWith(_)&&R.add(a.current.get(T))}),R},[]);return f.useEffect(function(){return function(){h.current=!0}},[]),{registerPath:y,unregisterPath:C,refreshOverflowKeys:b,isSubPathKey:S,getKeyPath:x,getKeys:E,getSubPathKeys:$}}function kc(e){var t=f.useRef(e);t.current=e;var n=f.useCallback(function(){for(var r,a=arguments.length,l=new Array(a),s=0;s<a;s++)l[s]=arguments[s];return(r=t.current)===null||r===void 0?void 0:r.call.apply(r,[t].concat(l))},[]);return e?n:void 0}var SH=Math.random().toFixed(5).toString().slice(2),Yw=0;function CH(e){var t=br(e,{value:e}),n=ge(t,2),r=n[0],a=n[1];return f.useEffect(function(){Yw+=1;var l="".concat(SH,"-").concat(Yw);a("rc-menu-uuid-".concat(l))},[]),r}function U_(e,t,n,r){var a=f.useContext(Da),l=a.activeKey,s=a.onActive,u=a.onInactive,d={active:l===e};return t||(d.onMouseEnter=function(g){n==null||n({key:e,domEvent:g}),s(e)},d.onMouseLeave=function(g){r==null||r({key:e,domEvent:g}),u(e)}),d}function k_(e){var t=f.useContext(Da),n=t.mode,r=t.rtl,a=t.inlineIndent;if(n!=="inline")return null;var l=e;return r?{paddingRight:l*a}:{paddingLeft:l*a}}function q_(e){var t=e.icon,n=e.props,r=e.children,a;return t===null||t===!1?null:(typeof t=="function"?a=f.createElement(t,Q({},n)):typeof t!="boolean"&&(a=t),a||r||null)}var xH=["item"];function Vd(e){var t=e.item,n=$t(e,xH);return Object.defineProperty(n,"item",{get:function(){return mr(!1,"`info.item` is deprecated since we will move to function component that not provides React Node instance in future."),t}}),n}var wH=["title","attribute","elementRef"],EH=["style","className","eventKey","warnKey","disabled","itemIcon","children","role","onMouseEnter","onMouseLeave","onClick","onKeyDown","onFocus"],$H=["active"],OH=function(e){Po(n,e);var t=Ao(n);function n(){return Wn(this,n),t.apply(this,arguments)}return Gn(n,[{key:"render",value:function(){var a=this.props,l=a.title,s=a.attribute,u=a.elementRef,d=$t(a,wH),g=On(d,["eventKey","popupClassName","popupOffset","onTitleClick"]);return mr(!s,"`attribute` of Menu.Item is deprecated. Please pass attribute directly."),f.createElement(eo.Item,je({},s,{title:typeof l=="string"?l:void 0},g,{ref:u}))}}]),n}(f.Component),RH=f.forwardRef(function(e,t){var n=e.style,r=e.className,a=e.eventKey;e.warnKey;var l=e.disabled,s=e.itemIcon,u=e.children,d=e.role,g=e.onMouseEnter,h=e.onMouseLeave,p=e.onClick,y=e.onKeyDown,C=e.onFocus,b=$t(e,EH),x=L_(a),S=f.useContext(Da),E=S.prefixCls,$=S.onItemClick,O=S.disabled,_=S.overflowDisabled,R=S.itemIcon,T=S.selectedKeys,M=S.onActive,z=f.useContext(jy),P=z._internalRenderMenuItem,D="".concat(E,"-item"),F=f.useRef(),G=f.useRef(),V=O||l,U=Io(t,G),K=ys(a),j=function(oe){return{key:a,keyPath:Ne(K).reverse(),item:F.current,domEvent:oe}},A=s||R,L=U_(a,V,g,h),q=L.active,N=$t(L,$H),B=T.includes(a),W=k_(K.length),X=function(oe){if(!V){var he=j(oe);p==null||p(Vd(he)),$(he)}},k=function(oe){if(y==null||y(oe),oe.which===lt.ENTER){var he=j(oe);p==null||p(Vd(he)),$(he)}},Y=function(oe){M(a),C==null||C(oe)},ee={};e.role==="option"&&(ee["aria-selected"]=B);var ue=f.createElement(OH,je({ref:F,elementRef:U,role:d===null?"none":d||"menuitem",tabIndex:l?null:-1,"data-menu-id":_&&x?null:x},On(b,["extra"]),N,ee,{component:"li","aria-disabled":l,style:Q(Q({},W),n),className:re(D,Z(Z(Z({},"".concat(D,"-active"),q),"".concat(D,"-selected"),B),"".concat(D,"-disabled"),V),r),onClick:X,onKeyDown:k,onFocus:Y}),u,f.createElement(q_,{props:Q(Q({},e),{},{isSelected:B}),icon:A}));return P&&(ue=P(ue,e,{selected:B})),ue});function _H(e,t){var n=e.eventKey,r=jm(),a=ys(n);return f.useEffect(function(){if(r)return r.registerPath(n,a),function(){r.unregisterPath(n,a)}},[a]),r?null:f.createElement(RH,je({},e,{ref:t}))}const Im=f.forwardRef(_H);var TH=["className","children"],MH=function(t,n){var r=t.className,a=t.children,l=$t(t,TH),s=f.useContext(Da),u=s.prefixCls,d=s.mode,g=s.rtl;return f.createElement("ul",je({className:re(u,g&&"".concat(u,"-rtl"),"".concat(u,"-sub"),"".concat(u,"-").concat(d==="inline"?"inline":"vertical"),r),role:"menu"},l,{"data-menu-list":!0,ref:n}),a)},Ay=f.forwardRef(MH);Ay.displayName="SubMenuList";function zy(e,t){return Lr(e).map(function(n,r){if(f.isValidElement(n)){var a,l,s=n.key,u=(a=(l=n.props)===null||l===void 0?void 0:l.eventKey)!==null&&a!==void 0?a:s,d=u==null;d&&(u="tmp_key-".concat([].concat(Ne(t),[r]).join("-")));var g={key:u,eventKey:u};return f.cloneElement(n,g)}return n})}var wr={adjustX:1,adjustY:1},NH={topLeft:{points:["bl","tl"],overflow:wr},topRight:{points:["br","tr"],overflow:wr},bottomLeft:{points:["tl","bl"],overflow:wr},bottomRight:{points:["tr","br"],overflow:wr},leftTop:{points:["tr","tl"],overflow:wr},leftBottom:{points:["br","bl"],overflow:wr},rightTop:{points:["tl","tr"],overflow:wr},rightBottom:{points:["bl","br"],overflow:wr}},jH={topLeft:{points:["bl","tl"],overflow:wr},topRight:{points:["br","tr"],overflow:wr},bottomLeft:{points:["tl","bl"],overflow:wr},bottomRight:{points:["tr","br"],overflow:wr},rightTop:{points:["tr","tl"],overflow:wr},rightBottom:{points:["br","bl"],overflow:wr},leftTop:{points:["tl","tr"],overflow:wr},leftBottom:{points:["bl","br"],overflow:wr}};function W_(e,t,n){if(t)return t;if(n)return n[e]||n.other}var IH={horizontal:"bottomLeft",vertical:"rightTop","vertical-left":"rightTop","vertical-right":"leftTop"};function PH(e){var t=e.prefixCls,n=e.visible,r=e.children,a=e.popup,l=e.popupStyle,s=e.popupClassName,u=e.popupOffset,d=e.disabled,g=e.mode,h=e.onVisibleChange,p=f.useContext(Da),y=p.getPopupContainer,C=p.rtl,b=p.subMenuOpenDelay,x=p.subMenuCloseDelay,S=p.builtinPlacements,E=p.triggerSubMenuAction,$=p.forceSubMenuRender,O=p.rootClassName,_=p.motion,R=p.defaultMotions,T=f.useState(!1),M=ge(T,2),z=M[0],P=M[1],D=Q(C?Q({},jH):Q({},NH),S),F=IH[g],G=W_(g,_,R),V=f.useRef(G);g!=="inline"&&(V.current=G);var U=Q(Q({},V.current),{},{leavedClassName:"".concat(t,"-hidden"),removeOnLeave:!1,motionAppear:!0}),K=f.useRef();return f.useEffect(function(){return K.current=bn(function(){P(n)}),function(){bn.cancel(K.current)}},[n]),f.createElement(Ey,{prefixCls:t,popupClassName:re("".concat(t,"-popup"),Z({},"".concat(t,"-rtl"),C),s,O),stretch:g==="horizontal"?"minWidth":null,getPopupContainer:y,builtinPlacements:D,popupPlacement:F,popupVisible:z,popup:a,popupStyle:l,popupAlign:u&&{offset:u},action:d?[]:[E],mouseEnterDelay:b,mouseLeaveDelay:x,onPopupVisibleChange:h,forceRender:$,popupMotion:U,fresh:!0},r)}function AH(e){var t=e.id,n=e.open,r=e.keyPath,a=e.children,l="inline",s=f.useContext(Da),u=s.prefixCls,d=s.forceSubMenuRender,g=s.motion,h=s.defaultMotions,p=s.mode,y=f.useRef(!1);y.current=p===l;var C=f.useState(!y.current),b=ge(C,2),x=b[0],S=b[1],E=y.current?n:!1;f.useEffect(function(){y.current&&S(!1)},[p]);var $=Q({},W_(l,g,h));r.length>1&&($.motionAppear=!1);var O=$.onVisibleChanged;return $.onVisibleChanged=function(_){return!y.current&&!_&&S(!0),O==null?void 0:O(_)},x?null:f.createElement(fu,{mode:l,locked:!y.current},f.createElement($a,je({visible:E},$,{forceRender:d,removeOnLeave:!1,leavedClassName:"".concat(u,"-hidden")}),function(_){var R=_.className,T=_.style;return f.createElement(Ay,{id:t,className:R,style:T},a)}))}var zH=["style","className","title","eventKey","warnKey","disabled","internalPopupClose","children","itemIcon","expandIcon","popupClassName","popupOffset","popupStyle","onClick","onMouseEnter","onMouseLeave","onTitleClick","onTitleMouseEnter","onTitleMouseLeave"],DH=["active"],LH=f.forwardRef(function(e,t){var n=e.style,r=e.className,a=e.title,l=e.eventKey;e.warnKey;var s=e.disabled,u=e.internalPopupClose,d=e.children,g=e.itemIcon,h=e.expandIcon,p=e.popupClassName,y=e.popupOffset,C=e.popupStyle,b=e.onClick,x=e.onMouseEnter,S=e.onMouseLeave,E=e.onTitleClick,$=e.onTitleMouseEnter,O=e.onTitleMouseLeave,_=$t(e,zH),R=L_(l),T=f.useContext(Da),M=T.prefixCls,z=T.mode,P=T.openKeys,D=T.disabled,F=T.overflowDisabled,G=T.activeKey,V=T.selectedKeys,U=T.itemIcon,K=T.expandIcon,j=T.onItemClick,A=T.onOpenChange,L=T.onActive,q=f.useContext(jy),N=q._internalRenderSubMenuItem,B=f.useContext(F_),W=B.isSubPathKey,X=ys(),k="".concat(M,"-submenu"),Y=D||s,ee=f.useRef(),ue=f.useRef(),ae=g??U,oe=h??K,he=P.includes(l),ie=!F&&he,le=W(V,l),be=U_(l,Y,$,O),Re=be.active,fe=$t(be,DH),Se=f.useState(!1),Ee=ge(Se,2),Oe=Ee[0],Pe=Ee[1],Ye=function(Ie){Y||Pe(Ie)},Me=function(Ie){Ye(!0),x==null||x({key:l,domEvent:Ie})},De=function(Ie){Ye(!1),S==null||S({key:l,domEvent:Ie})},xe=f.useMemo(function(){return Re||(z!=="inline"?Oe||W([G],l):!1)},[z,Re,G,Oe,l,W]),Te=k_(X.length),Le=function(Ie){Y||(E==null||E({key:l,domEvent:Ie}),z==="inline"&&A(l,!he))},We=kc(function(Ue){b==null||b(Vd(Ue)),j(Ue)}),Ge=function(Ie){z!=="inline"&&A(l,Ie)},at=function(){L(l)},ke=R&&"".concat(R,"-popup"),Qe=f.useMemo(function(){return f.createElement(q_,{icon:z!=="horizontal"?oe:void 0,props:Q(Q({},e),{},{isOpen:ie,isSubMenu:!0})},f.createElement("i",{className:"".concat(k,"-arrow")}))},[z,oe,e,ie,k]),ct=f.createElement("div",je({role:"menuitem",style:Te,className:"".concat(k,"-title"),tabIndex:Y?null:-1,ref:ee,title:typeof a=="string"?a:null,"data-menu-id":F&&R?null:R,"aria-expanded":ie,"aria-haspopup":!0,"aria-controls":ke,"aria-disabled":Y,onClick:Le,onFocus:at},fe),a,Qe),pt=f.useRef(z);if(z!=="inline"&&X.length>1?pt.current="vertical":pt.current=z,!F){var Ze=pt.current;ct=f.createElement(PH,{mode:Ze,prefixCls:k,visible:!u&&ie&&z!=="inline",popupClassName:p,popupOffset:y,popupStyle:C,popup:f.createElement(fu,{mode:Ze==="horizontal"?"vertical":Ze},f.createElement(Ay,{id:ke,ref:ue},d)),disabled:Y,onVisibleChange:Ge},ct)}var Be=f.createElement(eo.Item,je({ref:t,role:"none"},_,{component:"li",style:n,className:re(k,"".concat(k,"-").concat(z),r,Z(Z(Z(Z({},"".concat(k,"-open"),ie),"".concat(k,"-active"),xe),"".concat(k,"-selected"),le),"".concat(k,"-disabled"),Y)),onMouseEnter:Me,onMouseLeave:De}),ct,!F&&f.createElement(AH,{id:ke,open:ie,keyPath:X},d));return N&&(Be=N(Be,e,{selected:le,active:xe,open:ie,disabled:Y})),f.createElement(fu,{onItemClick:We,mode:z==="horizontal"?"vertical":z,itemIcon:ae,expandIcon:oe},Be)}),Pm=f.forwardRef(function(e,t){var n=e.eventKey,r=e.children,a=ys(n),l=zy(r,a),s=jm();f.useEffect(function(){if(s)return s.registerPath(n,a),function(){s.unregisterPath(n,a)}},[a]);var u;return s?u=l:u=f.createElement(LH,je({ref:t},e),l),f.createElement(H_.Provider,{value:a},u)});function Dy(e){var t=e.className,n=e.style,r=f.useContext(Da),a=r.prefixCls,l=jm();return l?null:f.createElement("li",{role:"separator",className:re("".concat(a,"-item-divider"),t),style:n})}var BH=["className","title","eventKey","children"],HH=f.forwardRef(function(e,t){var n=e.className,r=e.title;e.eventKey;var a=e.children,l=$t(e,BH),s=f.useContext(Da),u=s.prefixCls,d="".concat(u,"-item-group");return f.createElement("li",je({ref:t,role:"presentation"},l,{onClick:function(h){return h.stopPropagation()},className:re(d,n)}),f.createElement("div",{role:"presentation",className:"".concat(d,"-title"),title:typeof r=="string"?r:void 0},r),f.createElement("ul",{role:"group",className:"".concat(d,"-list")},a))}),Ly=f.forwardRef(function(e,t){var n=e.eventKey,r=e.children,a=ys(n),l=zy(r,a),s=jm();return s?l:f.createElement(HH,je({ref:t},On(e,["warnKey"])),l)}),FH=["label","children","key","type","extra"];function E0(e,t,n){var r=t.item,a=t.group,l=t.submenu,s=t.divider;return(e||[]).map(function(u,d){if(u&&st(u)==="object"){var g=u,h=g.label,p=g.children,y=g.key,C=g.type,b=g.extra,x=$t(g,FH),S=y??"tmp-".concat(d);return p||C==="group"?C==="group"?f.createElement(a,je({key:S},x,{title:h}),E0(p,t,n)):f.createElement(l,je({key:S},x,{title:h}),E0(p,t,n)):C==="divider"?f.createElement(s,je({key:S},x)):f.createElement(r,je({key:S},x,{extra:b}),h,(!!b||b===0)&&f.createElement("span",{className:"".concat(n,"-item-extra")},b))}return null}).filter(function(u){return u})}function Qw(e,t,n,r,a){var l=e,s=Q({divider:Dy,item:Im,group:Ly,submenu:Pm},r);return t&&(l=E0(t,s,a)),zy(l,n)}var VH=["prefixCls","rootClassName","style","className","tabIndex","items","children","direction","id","mode","inlineCollapsed","disabled","disabledOverflow","subMenuOpenDelay","subMenuCloseDelay","forceSubMenuRender","defaultOpenKeys","openKeys","activeKey","defaultActiveFirst","selectable","multiple","defaultSelectedKeys","selectedKeys","onSelect","onDeselect","inlineIndent","motion","defaultMotions","triggerSubMenuAction","builtinPlacements","itemIcon","expandIcon","overflowedIndicator","overflowedIndicatorPopupClassName","getPopupContainer","onClick","onOpenChange","onKeyDown","openAnimation","openTransitionName","_internalRenderMenuItem","_internalRenderSubMenuItem","_internalComponents"],Fi=[],UH=f.forwardRef(function(e,t){var n,r=e,a=r.prefixCls,l=a===void 0?"rc-menu":a,s=r.rootClassName,u=r.style,d=r.className,g=r.tabIndex,h=g===void 0?0:g,p=r.items,y=r.children,C=r.direction,b=r.id,x=r.mode,S=x===void 0?"vertical":x,E=r.inlineCollapsed,$=r.disabled,O=r.disabledOverflow,_=r.subMenuOpenDelay,R=_===void 0?.1:_,T=r.subMenuCloseDelay,M=T===void 0?.1:T,z=r.forceSubMenuRender,P=r.defaultOpenKeys,D=r.openKeys,F=r.activeKey,G=r.defaultActiveFirst,V=r.selectable,U=V===void 0?!0:V,K=r.multiple,j=K===void 0?!1:K,A=r.defaultSelectedKeys,L=r.selectedKeys,q=r.onSelect,N=r.onDeselect,B=r.inlineIndent,W=B===void 0?24:B,X=r.motion,k=r.defaultMotions,Y=r.triggerSubMenuAction,ee=Y===void 0?"hover":Y,ue=r.builtinPlacements,ae=r.itemIcon,oe=r.expandIcon,he=r.overflowedIndicator,ie=he===void 0?"...":he,le=r.overflowedIndicatorPopupClassName,be=r.getPopupContainer,Re=r.onClick,fe=r.onOpenChange,Se=r.onKeyDown;r.openAnimation,r.openTransitionName;var Ee=r._internalRenderMenuItem,Oe=r._internalRenderSubMenuItem,Pe=r._internalComponents,Ye=$t(r,VH),Me=f.useMemo(function(){return[Qw(y,p,Fi,Pe,l),Qw(y,p,Fi,{},l)]},[y,p,Pe]),De=ge(Me,2),xe=De[0],Te=De[1],Le=f.useState(!1),We=ge(Le,2),Ge=We[0],at=We[1],ke=f.useRef(),Qe=CH(b),ct=C==="rtl",pt=br(P,{value:D,postState:function(Je){return Je||Fi}}),Ze=ge(pt,2),Be=Ze[0],Ue=Ze[1],Ie=function(Je){var He=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;function Ft(){Ue(Je),fe==null||fe(Je)}He?pi.flushSync(Ft):Ft()},qe=f.useState(Be),wt=ge(qe,2),It=wt[0],Nn=wt[1],wn=f.useRef(!1),Rn=f.useMemo(function(){return(S==="inline"||S==="vertical")&&E?["vertical",E]:[S,!1]},[S,E]),dn=ge(Rn,2),kt=dn[0],ln=dn[1],Ot=kt==="inline",nt=f.useState(kt),Ke=ge(nt,2),Fe=Ke[0],vt=Ke[1],Ct=f.useState(ln),Ve=ge(Ct,2),ze=Ve[0],rt=Ve[1];f.useEffect(function(){vt(kt),rt(ln),wn.current&&(Ot?Ue(It):Ie(Fi))},[kt,ln]);var ot=f.useState(0),tt=ge(ot,2),ut=tt[0],qt=tt[1],Mt=ut>=xe.length-1||Fe!=="horizontal"||O;f.useEffect(function(){Ot&&Nn(Be)},[Be]),f.useEffect(function(){return wn.current=!0,function(){wn.current=!1}},[]);var Pt=bH(),At=Pt.registerPath,tn=Pt.unregisterPath,sn=Pt.refreshOverflowKeys,Tt=Pt.isSubPathKey,ar=Pt.getKeyPath,Sn=Pt.getKeys,cn=Pt.getSubPathKeys,Ae=f.useMemo(function(){return{registerPath:At,unregisterPath:tn}},[At,tn]),et=f.useMemo(function(){return{isSubPathKey:Tt}},[Tt]);f.useEffect(function(){sn(Mt?Fi:xe.slice(ut+1).map(function(Ht){return Ht.key}))},[ut,Mt]);var bt=br(F||G&&((n=xe[0])===null||n===void 0?void 0:n.key),{value:F}),Et=ge(bt,2),Yt=Et[0],Kn=Et[1],zn=kc(function(Ht){Kn(Ht)}),Fn=kc(function(){Kn(void 0)});f.useImperativeHandle(t,function(){return{list:ke.current,focus:function(Je){var He,Ft=Sn(),Wt=x0(Ft,Qe),Qt=Wt.elements,Zt=Wt.key2element,ir=Wt.element2key,Vn=Iy(ke.current,Qt),jn=Yt??(Vn[0]?ir.get(Vn[0]):(He=xe.find(function(_r){return!_r.props.disabled}))===null||He===void 0?void 0:He.key),lr=Zt.get(jn);if(jn&&lr){var sr;lr==null||(sr=lr.focus)===null||sr===void 0||sr.call(lr,Je)}}}});var gr=br(A||[],{value:L,postState:function(Je){return Array.isArray(Je)?Je:Je==null?Fi:[Je]}}),Sr=ge(gr,2),_n=Sr[0],or=Sr[1],Fr=function(Je){if(U){var He=Je.key,Ft=_n.includes(He),Wt;j?Ft?Wt=_n.filter(function(Zt){return Zt!==He}):Wt=[].concat(Ne(_n),[He]):Wt=[He],or(Wt);var Qt=Q(Q({},Je),{},{selectedKeys:Wt});Ft?N==null||N(Qt):q==null||q(Qt)}!j&&Be.length&&Fe!=="inline"&&Ie(Fi)},ka=kc(function(Ht){Re==null||Re(Vd(Ht)),Fr(Ht)}),Xn=kc(function(Ht,Je){var He=Be.filter(function(Wt){return Wt!==Ht});if(Je)He.push(Ht);else if(Fe!=="inline"){var Ft=cn(Ht);He=He.filter(function(Wt){return!Ft.has(Wt)})}ru(Be,He,!0)||Ie(He,!0)}),mn=function(Je,He){var Ft=He??!Be.includes(Je);Xn(Je,Ft)},Vr=pH(Fe,Yt,ct,Qe,ke,Sn,ar,Kn,mn,Se);f.useEffect(function(){at(!0)},[]);var la=f.useMemo(function(){return{_internalRenderMenuItem:Ee,_internalRenderSubMenuItem:Oe}},[Ee,Oe]),Yn=Fe!=="horizontal"||O?xe:xe.map(function(Ht,Je){return f.createElement(fu,{key:Ht.key,overflowDisabled:Je>ut},Ht)}),Oa=f.createElement(eo,je({id:b,ref:ke,prefixCls:"".concat(l,"-overflow"),component:"ul",itemComponent:Im,className:re(l,"".concat(l,"-root"),"".concat(l,"-").concat(Fe),d,Z(Z({},"".concat(l,"-inline-collapsed"),ze),"".concat(l,"-rtl"),ct),s),dir:C,style:u,role:"menu",tabIndex:h,data:Yn,renderRawItem:function(Je){return Je},renderRawRest:function(Je){var He=Je.length,Ft=He?xe.slice(-He):null;return f.createElement(Pm,{eventKey:w0,title:ie,disabled:Mt,internalPopupClose:He===0,popupClassName:le},Ft)},maxCount:Fe!=="horizontal"||O?eo.INVALIDATE:eo.RESPONSIVE,ssr:"full","data-menu-list":!0,onVisibleChange:function(Je){qt(Je)},onKeyDown:Vr},Ye));return f.createElement(jy.Provider,{value:la},f.createElement(z_.Provider,{value:Qe},f.createElement(fu,{prefixCls:l,rootClassName:s,mode:Fe,openKeys:Be,rtl:ct,disabled:$,motion:Ge?X:null,defaultMotions:Ge?k:null,activeKey:Yt,onActive:zn,onInactive:Fn,selectedKeys:_n,inlineIndent:W,subMenuOpenDelay:R,subMenuCloseDelay:M,forceSubMenuRender:z,builtinPlacements:ue,triggerSubMenuAction:ee,getPopupContainer:be,itemIcon:ae,expandIcon:oe,onItemClick:ka,onOpenChange:Xn},f.createElement(F_.Provider,{value:et},Oa),f.createElement("div",{style:{display:"none"},"aria-hidden":!0},f.createElement(B_.Provider,{value:Ae},Te)))))}),ju=UH;ju.Item=Im;ju.SubMenu=Pm;ju.ItemGroup=Ly;ju.Divider=Dy;const G_=f.createContext({siderHook:{addSider:()=>null,removeSider:()=>null}}),kH=e=>{const{antCls:t,componentCls:n,colorText:r,footerBg:a,headerHeight:l,headerPadding:s,headerColor:u,footerPadding:d,fontSize:g,bodyBg:h,headerBg:p}=e;return{[n]:{display:"flex",flex:"auto",flexDirection:"column",minHeight:0,background:h,"&, *":{boxSizing:"border-box"},[`&${n}-has-sider`]:{flexDirection:"row",[`> ${n}, > ${n}-content`]:{width:0}},[`${n}-header, &${n}-footer`]:{flex:"0 0 auto"},"&-rtl":{direction:"rtl"}},[`${n}-header`]:{height:l,padding:s,color:u,lineHeight:ve(l),background:p,[`${t}-menu`]:{lineHeight:"inherit"}},[`${n}-footer`]:{padding:d,color:r,fontSize:g,background:a},[`${n}-content`]:{flex:"auto",color:r,minHeight:0}}},K_=e=>{const{colorBgLayout:t,controlHeight:n,controlHeightLG:r,colorText:a,controlHeightSM:l,marginXXS:s,colorTextLightSolid:u,colorBgContainer:d}=e,g=r*1.25;return{colorBgHeader:"#001529",colorBgBody:t,colorBgTrigger:"#002140",bodyBg:t,headerBg:"#001529",headerHeight:n*2,headerPadding:`0 ${g}px`,headerColor:a,footerPadding:`${l}px ${g}px`,footerBg:t,siderBg:"#001529",triggerHeight:r+s*2,triggerBg:"#002140",triggerColor:u,zeroTriggerWidth:r,zeroTriggerHeight:r,lightSiderBg:d,lightTriggerBg:d,lightTriggerColor:a}},X_=[["colorBgBody","bodyBg"],["colorBgHeader","headerBg"],["colorBgTrigger","triggerBg"]],Y_=xn("Layout",e=>[kH(e)],K_,{deprecatedTokens:X_}),qH=e=>{const{componentCls:t,siderBg:n,motionDurationMid:r,motionDurationSlow:a,antCls:l,triggerHeight:s,triggerColor:u,triggerBg:d,headerHeight:g,zeroTriggerWidth:h,zeroTriggerHeight:p,borderRadiusLG:y,lightSiderBg:C,lightTriggerColor:b,lightTriggerBg:x,bodyBg:S}=e;return{[t]:{position:"relative",minWidth:0,background:n,transition:`all ${r}, background 0s`,"&-has-trigger":{paddingBottom:s},"&-right":{order:1},[`${t}-children`]:{height:"100%",marginTop:-.1,paddingTop:.1,[`${l}-menu${l}-menu-inline-collapsed`]:{width:"auto"}},[`&-zero-width ${t}-children`]:{overflow:"hidden"},[`${t}-trigger`]:{position:"fixed",bottom:0,zIndex:1,height:s,color:u,lineHeight:ve(s),textAlign:"center",background:d,cursor:"pointer",transition:`all ${r}`},[`${t}-zero-width-trigger`]:{position:"absolute",top:g,insetInlineEnd:e.calc(h).mul(-1).equal(),zIndex:1,width:h,height:p,color:u,fontSize:e.fontSizeXL,display:"flex",alignItems:"center",justifyContent:"center",background:n,borderRadius:`0 ${ve(y)} ${ve(y)} 0`,cursor:"pointer",transition:`background ${a} ease`,"&::after":{position:"absolute",inset:0,background:"transparent",transition:`all ${a}`,content:'""'},"&:hover::after":{background:"rgba(255, 255, 255, 0.2)"},"&-right":{insetInlineStart:e.calc(h).mul(-1).equal(),borderRadius:`${ve(y)} 0 0 ${ve(y)}`}},"&-light":{background:C,[`${t}-trigger`]:{color:b,background:x},[`${t}-zero-width-trigger`]:{color:b,background:x,border:`1px solid ${S}`,borderInlineStart:0}}}}},WH=xn(["Layout","Sider"],e=>[qH(e)],K_,{deprecatedTokens:X_});var GH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const Zw={xs:"479.98px",sm:"575.98px",md:"767.98px",lg:"991.98px",xl:"1199.98px",xxl:"1599.98px"},KH=e=>!Number.isNaN(Number.parseFloat(e))&&isFinite(e),Am=f.createContext({}),XH=(()=>{let e=0;return function(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"";return e+=1,`${t}${e}`}})(),Q_=f.forwardRef((e,t)=>{const{prefixCls:n,className:r,trigger:a,children:l,defaultCollapsed:s=!1,theme:u="dark",style:d={},collapsible:g=!1,reverseArrow:h=!1,width:p=200,collapsedWidth:y=80,zeroWidthTriggerStyle:C,breakpoint:b,onCollapse:x,onBreakpoint:S}=e,E=GH(e,["prefixCls","className","trigger","children","defaultCollapsed","theme","style","collapsible","reverseArrow","width","collapsedWidth","zeroWidthTriggerStyle","breakpoint","onCollapse","onBreakpoint"]),{siderHook:$}=f.useContext(G_),[O,_]=f.useState("collapsed"in e?e.collapsed:s),[R,T]=f.useState(!1);f.useEffect(()=>{"collapsed"in e&&_(e.collapsed)},[e.collapsed]);const M=(ae,oe)=>{"collapsed"in e||_(ae),x==null||x(ae,oe)},{getPrefixCls:z,direction:P}=f.useContext(dt),D=z("layout-sider",n),[F,G,V]=WH(D),U=f.useRef(null);U.current=ae=>{T(ae.matches),S==null||S(ae.matches),O!==ae.matches&&M(ae.matches,"responsive")},f.useEffect(()=>{function ae(he){return U.current(he)}let oe;if(typeof window<"u"){const{matchMedia:he}=window;if(he&&b&&b in Zw){oe=he(`screen and (max-width: ${Zw[b]})`);try{oe.addEventListener("change",ae)}catch{oe.addListener(ae)}ae(oe)}}return()=>{try{oe==null||oe.removeEventListener("change",ae)}catch{oe==null||oe.removeListener(ae)}}},[b]),f.useEffect(()=>{const ae=XH("ant-sider-");return $.addSider(ae),()=>$.removeSider(ae)},[]);const K=()=>{M(!O,"clickTrigger")},j=On(E,["collapsed"]),A=O?y:p,L=KH(A)?`${A}px`:String(A),q=parseFloat(String(y||0))===0?f.createElement("span",{onClick:K,className:re(`${D}-zero-width-trigger`,`${D}-zero-width-trigger-${h?"right":"left"}`),style:C},a||f.createElement(i4,null)):null,N=P==="rtl"==!h,X={expanded:N?f.createElement(Dv,null):f.createElement(Y1,null),collapsed:N?f.createElement(Y1,null):f.createElement(Dv,null)}[O?"collapsed":"expanded"],k=a!==null?q||f.createElement("div",{className:`${D}-trigger`,onClick:K,style:{width:L}},a||X):null,Y=Object.assign(Object.assign({},d),{flex:`0 0 ${L}`,maxWidth:L,minWidth:L,width:L}),ee=re(D,`${D}-${u}`,{[`${D}-collapsed`]:!!O,[`${D}-has-trigger`]:g&&a!==null&&!q,[`${D}-below`]:!!R,[`${D}-zero-width`]:parseFloat(L)===0},r,G,V),ue=f.useMemo(()=>({siderCollapsed:O}),[O]);return F(f.createElement(Am.Provider,{value:ue},f.createElement("aside",Object.assign({className:ee},j,{style:Y,ref:t}),f.createElement("div",{className:`${D}-children`},l),g||R&&q?k:null)))}),Ud=f.createContext({prefixCls:"",firstLevel:!0,inlineCollapsed:!1});var YH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const Z_=e=>{const{prefixCls:t,className:n,dashed:r}=e,a=YH(e,["prefixCls","className","dashed"]),{getPrefixCls:l}=f.useContext(dt),s=l("menu",t),u=re({[`${s}-item-divider-dashed`]:!!r},n);return f.createElement(Dy,Object.assign({className:u},a))},J_=e=>{var t;const{className:n,children:r,icon:a,title:l,danger:s,extra:u}=e,{prefixCls:d,firstLevel:g,direction:h,disableMenuItemTitleTooltip:p,inlineCollapsed:y}=f.useContext(Ud),C=O=>{const _=r==null?void 0:r[0],R=f.createElement("span",{className:re(`${d}-title-content`,{[`${d}-title-content-with-extra`]:!!u||u===0})},r);return(!a||f.isValidElement(r)&&r.type==="span")&&r&&O&&g&&typeof _=="string"?f.createElement("div",{className:`${d}-inline-collapsed-noicon`},_.charAt(0)):R},{siderCollapsed:b}=f.useContext(Am);let x=l;typeof l>"u"?x=g?r:"":l===!1&&(x="");const S={title:x};!b&&!y&&(S.title=null,S.open=!1);const E=Lr(r).length;let $=f.createElement(Im,Object.assign({},On(e,["title","icon","danger"]),{className:re({[`${d}-item-danger`]:s,[`${d}-item-only-child`]:(a?E+1:E)===1},n),title:typeof l=="string"?l:void 0}),na(a,{className:re(f.isValidElement(a)?(t=a.props)===null||t===void 0?void 0:t.className:"",`${d}-item-icon`)}),C(y));return p||($=f.createElement(Ho,Object.assign({},S,{placement:h==="rtl"?"left":"right",classNames:{root:`${d}-inline-collapsed-tooltip`}}),$)),$};var QH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const kd=f.createContext(null),pX=f.forwardRef((e,t)=>{const{children:n}=e,r=QH(e,["children"]),a=f.useContext(kd),l=f.useMemo(()=>Object.assign(Object.assign({},a),r),[a,r.prefixCls,r.mode,r.selectable,r.rootClassName]),s=CP(n),u=Io(t,s?fs(n):null);return f.createElement(kd.Provider,{value:l},f.createElement(ns,{space:!0},s?f.cloneElement(n,{ref:u}):n))}),ZH=e=>{const{componentCls:t,motionDurationSlow:n,horizontalLineHeight:r,colorSplit:a,lineWidth:l,lineType:s,itemPaddingInline:u}=e;return{[`${t}-horizontal`]:{lineHeight:r,border:0,borderBottom:`${ve(l)} ${s} ${a}`,boxShadow:"none","&::after":{display:"block",clear:"both",height:0,content:'"\\20"'},[`${t}-item, ${t}-submenu`]:{position:"relative",display:"inline-block",verticalAlign:"bottom",paddingInline:u},[`> ${t}-item:hover,
        > ${t}-item-active,
        > ${t}-submenu ${t}-submenu-title:hover`]:{backgroundColor:"transparent"},[`${t}-item, ${t}-submenu-title`]:{transition:[`border-color ${n}`,`background ${n}`].join(",")},[`${t}-submenu-arrow`]:{display:"none"}}}},JH=e=>{let{componentCls:t,menuArrowOffset:n,calc:r}=e;return{[`${t}-rtl`]:{direction:"rtl"},[`${t}-submenu-rtl`]:{transformOrigin:"100% 0"},[`${t}-rtl${t}-vertical,
    ${t}-submenu-rtl ${t}-vertical`]:{[`${t}-submenu-arrow`]:{"&::before":{transform:`rotate(-45deg) translateY(${ve(r(n).mul(-1).equal())})`},"&::after":{transform:`rotate(45deg) translateY(${ve(n)})`}}}}},Jw=e=>Object.assign({},N2(e)),eE=(e,t)=>{const{componentCls:n,itemColor:r,itemSelectedColor:a,subMenuItemSelectedColor:l,groupTitleColor:s,itemBg:u,subMenuItemBg:d,itemSelectedBg:g,activeBarHeight:h,activeBarWidth:p,activeBarBorderWidth:y,motionDurationSlow:C,motionEaseInOut:b,motionEaseOut:x,itemPaddingInline:S,motionDurationMid:E,itemHoverColor:$,lineType:O,colorSplit:_,itemDisabledColor:R,dangerItemColor:T,dangerItemHoverColor:M,dangerItemSelectedColor:z,dangerItemActiveBg:P,dangerItemSelectedBg:D,popupBg:F,itemHoverBg:G,itemActiveBg:V,menuSubMenuBg:U,horizontalItemSelectedColor:K,horizontalItemSelectedBg:j,horizontalItemBorderRadius:A,horizontalItemHoverBg:L}=e;return{[`${n}-${t}, ${n}-${t} > ${n}`]:{color:r,background:u,[`&${n}-root:focus-visible`]:Object.assign({},Jw(e)),[`${n}-item`]:{"&-group-title, &-extra":{color:s}},[`${n}-submenu-selected > ${n}-submenu-title`]:{color:l},[`${n}-item, ${n}-submenu-title`]:{color:r,[`&:not(${n}-item-disabled):focus-visible`]:Object.assign({},Jw(e))},[`${n}-item-disabled, ${n}-submenu-disabled`]:{color:`${R} !important`},[`${n}-item:not(${n}-item-selected):not(${n}-submenu-selected)`]:{[`&:hover, > ${n}-submenu-title:hover`]:{color:$}},[`&:not(${n}-horizontal)`]:{[`${n}-item:not(${n}-item-selected)`]:{"&:hover":{backgroundColor:G},"&:active":{backgroundColor:V}},[`${n}-submenu-title`]:{"&:hover":{backgroundColor:G},"&:active":{backgroundColor:V}}},[`${n}-item-danger`]:{color:T,[`&${n}-item:hover`]:{[`&:not(${n}-item-selected):not(${n}-submenu-selected)`]:{color:M}},[`&${n}-item:active`]:{background:P}},[`${n}-item a`]:{"&, &:hover":{color:"inherit"}},[`${n}-item-selected`]:{color:a,[`&${n}-item-danger`]:{color:z},"a, a:hover":{color:"inherit"}},[`& ${n}-item-selected`]:{backgroundColor:g,[`&${n}-item-danger`]:{backgroundColor:D}},[`&${n}-submenu > ${n}`]:{backgroundColor:U},[`&${n}-popup > ${n}`]:{backgroundColor:F},[`&${n}-submenu-popup > ${n}`]:{backgroundColor:F},[`&${n}-horizontal`]:Object.assign(Object.assign({},t==="dark"?{borderBottom:0}:{}),{[`> ${n}-item, > ${n}-submenu`]:{top:y,marginTop:e.calc(y).mul(-1).equal(),marginBottom:0,borderRadius:A,"&::after":{position:"absolute",insetInline:S,bottom:0,borderBottom:`${ve(h)} solid transparent`,transition:`border-color ${C} ${b}`,content:'""'},"&:hover, &-active, &-open":{background:L,"&::after":{borderBottomWidth:h,borderBottomColor:K}},"&-selected":{color:K,backgroundColor:j,"&:hover":{backgroundColor:j},"&::after":{borderBottomWidth:h,borderBottomColor:K}}}}),[`&${n}-root`]:{[`&${n}-inline, &${n}-vertical`]:{borderInlineEnd:`${ve(y)} ${O} ${_}`}},[`&${n}-inline`]:{[`${n}-sub${n}-inline`]:{background:d},[`${n}-item`]:{position:"relative","&::after":{position:"absolute",insetBlock:0,insetInlineEnd:0,borderInlineEnd:`${ve(p)} solid ${a}`,transform:"scaleY(0.0001)",opacity:0,transition:[`transform ${E} ${x}`,`opacity ${E} ${x}`].join(","),content:'""'},[`&${n}-item-danger`]:{"&::after":{borderInlineEndColor:z}}},[`${n}-selected, ${n}-item-selected`]:{"&::after":{transform:"scaleY(1)",opacity:1,transition:[`transform ${E} ${b}`,`opacity ${E} ${b}`].join(",")}}}}}},tE=e=>{const{componentCls:t,itemHeight:n,itemMarginInline:r,padding:a,menuArrowSize:l,marginXS:s,itemMarginBlock:u,itemWidth:d,itemPaddingInline:g}=e,h=e.calc(l).add(a).add(s).equal();return{[`${t}-item`]:{position:"relative",overflow:"hidden"},[`${t}-item, ${t}-submenu-title`]:{height:n,lineHeight:ve(n),paddingInline:g,overflow:"hidden",textOverflow:"ellipsis",marginInline:r,marginBlock:u,width:d},[`> ${t}-item,
            > ${t}-submenu > ${t}-submenu-title`]:{height:n,lineHeight:ve(n)},[`${t}-item-group-list ${t}-submenu-title,
            ${t}-submenu-title`]:{paddingInlineEnd:h}}},e9=e=>{const{componentCls:t,iconCls:n,itemHeight:r,colorTextLightSolid:a,dropdownWidth:l,controlHeightLG:s,motionEaseOut:u,paddingXL:d,itemMarginInline:g,fontSizeLG:h,motionDurationFast:p,motionDurationSlow:y,paddingXS:C,boxShadowSecondary:b,collapsedWidth:x,collapsedIconSize:S}=e,E={height:r,lineHeight:ve(r),listStylePosition:"inside",listStyleType:"disc"};return[{[t]:{"&-inline, &-vertical":Object.assign({[`&${t}-root`]:{boxShadow:"none"}},tE(e))},[`${t}-submenu-popup`]:{[`${t}-vertical`]:Object.assign(Object.assign({},tE(e)),{boxShadow:b})}},{[`${t}-submenu-popup ${t}-vertical${t}-sub`]:{minWidth:l,maxHeight:`calc(100vh - ${ve(e.calc(s).mul(2.5).equal())})`,padding:"0",overflow:"hidden",borderInlineEnd:0,"&:not([class*='-active'])":{overflowX:"hidden",overflowY:"auto"}}},{[`${t}-inline`]:{width:"100%",[`&${t}-root`]:{[`${t}-item, ${t}-submenu-title`]:{display:"flex",alignItems:"center",transition:[`border-color ${y}`,`background ${y}`,`padding ${p} ${u}`].join(","),[`> ${t}-title-content`]:{flex:"auto",minWidth:0,overflow:"hidden",textOverflow:"ellipsis"},"> *":{flex:"none"}}},[`${t}-sub${t}-inline`]:{padding:0,border:0,borderRadius:0,boxShadow:"none",[`& > ${t}-submenu > ${t}-submenu-title`]:E,[`& ${t}-item-group-title`]:{paddingInlineStart:d}},[`${t}-item`]:E}},{[`${t}-inline-collapsed`]:{width:x,[`&${t}-root`]:{[`${t}-item, ${t}-submenu ${t}-submenu-title`]:{[`> ${t}-inline-collapsed-noicon`]:{fontSize:h,textAlign:"center"}}},[`> ${t}-item,
          > ${t}-item-group > ${t}-item-group-list > ${t}-item,
          > ${t}-item-group > ${t}-item-group-list > ${t}-submenu > ${t}-submenu-title,
          > ${t}-submenu > ${t}-submenu-title`]:{insetInlineStart:0,paddingInline:`calc(50% - ${ve(e.calc(S).div(2).equal())} - ${ve(g)})`,textOverflow:"clip",[`
            ${t}-submenu-arrow,
            ${t}-submenu-expand-icon
          `]:{opacity:0},[`${t}-item-icon, ${n}`]:{margin:0,fontSize:S,lineHeight:ve(r),"+ span":{display:"inline-block",opacity:0}}},[`${t}-item-icon, ${n}`]:{display:"inline-block"},"&-tooltip":{pointerEvents:"none",[`${t}-item-icon, ${n}`]:{display:"none"},"a, a:hover":{color:a}},[`${t}-item-group-title`]:Object.assign(Object.assign({},Pd),{paddingInline:C})}}]},nE=e=>{const{componentCls:t,motionDurationSlow:n,motionDurationMid:r,motionEaseInOut:a,motionEaseOut:l,iconCls:s,iconSize:u,iconMarginInlineEnd:d}=e;return{[`${t}-item, ${t}-submenu-title`]:{position:"relative",display:"block",margin:0,whiteSpace:"nowrap",cursor:"pointer",transition:[`border-color ${n}`,`background ${n}`,`padding calc(${n} + 0.1s) ${a}`].join(","),[`${t}-item-icon, ${s}`]:{minWidth:u,fontSize:u,transition:[`font-size ${r} ${l}`,`margin ${n} ${a}`,`color ${n}`].join(","),"+ span":{marginInlineStart:d,opacity:1,transition:[`opacity ${n} ${a}`,`margin ${n}`,`color ${n}`].join(",")}},[`${t}-item-icon`]:Object.assign({},ms()),[`&${t}-item-only-child`]:{[`> ${s}, > ${t}-item-icon`]:{marginInlineEnd:0}}},[`${t}-item-disabled, ${t}-submenu-disabled`]:{background:"none !important",cursor:"not-allowed","&::after":{borderColor:"transparent !important"},a:{color:"inherit !important",cursor:"not-allowed",pointerEvents:"none"},[`> ${t}-submenu-title`]:{color:"inherit !important",cursor:"not-allowed"}}}},rE=e=>{const{componentCls:t,motionDurationSlow:n,motionEaseInOut:r,borderRadius:a,menuArrowSize:l,menuArrowOffset:s}=e;return{[`${t}-submenu`]:{"&-expand-icon, &-arrow":{position:"absolute",top:"50%",insetInlineEnd:e.margin,width:l,color:"currentcolor",transform:"translateY(-50%)",transition:`transform ${n} ${r}, opacity ${n}`},"&-arrow":{"&::before, &::after":{position:"absolute",width:e.calc(l).mul(.6).equal(),height:e.calc(l).mul(.15).equal(),backgroundColor:"currentcolor",borderRadius:a,transition:[`background ${n} ${r}`,`transform ${n} ${r}`,`top ${n} ${r}`,`color ${n} ${r}`].join(","),content:'""'},"&::before":{transform:`rotate(45deg) translateY(${ve(e.calc(s).mul(-1).equal())})`},"&::after":{transform:`rotate(-45deg) translateY(${ve(s)})`}}}}},t9=e=>{const{antCls:t,componentCls:n,fontSize:r,motionDurationSlow:a,motionDurationMid:l,motionEaseInOut:s,paddingXS:u,padding:d,colorSplit:g,lineWidth:h,zIndexPopup:p,borderRadiusLG:y,subMenuItemBorderRadius:C,menuArrowSize:b,menuArrowOffset:x,lineType:S,groupTitleLineHeight:E,groupTitleFontSize:$}=e;return[{"":{[n]:Object.assign(Object.assign({},Ad()),{"&-hidden":{display:"none"}})},[`${n}-submenu-hidden`]:{display:"none"}},{[n]:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},Rr(e)),Ad()),{marginBottom:0,paddingInlineStart:0,fontSize:r,lineHeight:0,listStyle:"none",outline:"none",transition:`width ${a} cubic-bezier(0.2, 0, 0, 1) 0s`,"ul, ol":{margin:0,padding:0,listStyle:"none"},"&-overflow":{display:"flex",[`${n}-item`]:{flex:"none"}},[`${n}-item, ${n}-submenu, ${n}-submenu-title`]:{borderRadius:e.itemBorderRadius},[`${n}-item-group-title`]:{padding:`${ve(u)} ${ve(d)}`,fontSize:$,lineHeight:E,transition:`all ${a}`},[`&-horizontal ${n}-submenu`]:{transition:[`border-color ${a} ${s}`,`background ${a} ${s}`].join(",")},[`${n}-submenu, ${n}-submenu-inline`]:{transition:[`border-color ${a} ${s}`,`background ${a} ${s}`,`padding ${l} ${s}`].join(",")},[`${n}-submenu ${n}-sub`]:{cursor:"initial",transition:[`background ${a} ${s}`,`padding ${a} ${s}`].join(",")},[`${n}-title-content`]:{transition:`color ${a}`,"&-with-extra":{display:"inline-flex",alignItems:"center",width:"100%"},[`> ${t}-typography-ellipsis-single-line`]:{display:"inline",verticalAlign:"unset"},[`${n}-item-extra`]:{marginInlineStart:"auto",paddingInlineStart:e.padding}},[`${n}-item a`]:{"&::before":{position:"absolute",inset:0,backgroundColor:"transparent",content:'""'}},[`${n}-item-divider`]:{overflow:"hidden",lineHeight:0,borderColor:g,borderStyle:S,borderWidth:0,borderTopWidth:h,marginBlock:h,padding:0,"&-dashed":{borderStyle:"dashed"}}}),nE(e)),{[`${n}-item-group`]:{[`${n}-item-group-list`]:{margin:0,padding:0,[`${n}-item, ${n}-submenu-title`]:{paddingInline:`${ve(e.calc(r).mul(2).equal())} ${ve(d)}`}}},"&-submenu":{"&-popup":{position:"absolute",zIndex:p,borderRadius:y,boxShadow:"none",transformOrigin:"0 0",[`&${n}-submenu`]:{background:"transparent"},"&::before":{position:"absolute",inset:0,zIndex:-1,width:"100%",height:"100%",opacity:0,content:'""'},[`> ${n}`]:Object.assign(Object.assign(Object.assign({borderRadius:y},nE(e)),rE(e)),{[`${n}-item, ${n}-submenu > ${n}-submenu-title`]:{borderRadius:C},[`${n}-submenu-title::after`]:{transition:`transform ${a} ${s}`}})},"\n          &-placement-leftTop,\n          &-placement-bottomRight,\n          ":{transformOrigin:"100% 0"},"\n          &-placement-leftBottom,\n          &-placement-topRight,\n          ":{transformOrigin:"100% 100%"},"\n          &-placement-rightBottom,\n          &-placement-topLeft,\n          ":{transformOrigin:"0 100%"},"\n          &-placement-bottomLeft,\n          &-placement-rightTop,\n          ":{transformOrigin:"0 0"},"\n          &-placement-leftTop,\n          &-placement-leftBottom\n          ":{paddingInlineEnd:e.paddingXS},"\n          &-placement-rightTop,\n          &-placement-rightBottom\n          ":{paddingInlineStart:e.paddingXS},"\n          &-placement-topRight,\n          &-placement-topLeft\n          ":{paddingBottom:e.paddingXS},"\n          &-placement-bottomRight,\n          &-placement-bottomLeft\n          ":{paddingTop:e.paddingXS}}}),rE(e)),{[`&-inline-collapsed ${n}-submenu-arrow,
        &-inline ${n}-submenu-arrow`]:{"&::before":{transform:`rotate(-45deg) translateX(${ve(x)})`},"&::after":{transform:`rotate(45deg) translateX(${ve(e.calc(x).mul(-1).equal())})`}},[`${n}-submenu-open${n}-submenu-inline > ${n}-submenu-title > ${n}-submenu-arrow`]:{transform:`translateY(${ve(e.calc(b).mul(.2).mul(-1).equal())})`,"&::after":{transform:`rotate(-45deg) translateX(${ve(e.calc(x).mul(-1).equal())})`},"&::before":{transform:`rotate(45deg) translateX(${ve(x)})`}}})},{[`${t}-layout-header`]:{[n]:{lineHeight:"inherit"}}}]},n9=e=>{var t,n,r;const{colorPrimary:a,colorError:l,colorTextDisabled:s,colorErrorBg:u,colorText:d,colorTextDescription:g,colorBgContainer:h,colorFillAlter:p,colorFillContent:y,lineWidth:C,lineWidthBold:b,controlItemBgActive:x,colorBgTextHover:S,controlHeightLG:E,lineHeight:$,colorBgElevated:O,marginXXS:_,padding:R,fontSize:T,controlHeightSM:M,fontSizeLG:z,colorTextLightSolid:P,colorErrorHover:D}=e,F=(t=e.activeBarWidth)!==null&&t!==void 0?t:0,G=(n=e.activeBarBorderWidth)!==null&&n!==void 0?n:C,V=(r=e.itemMarginInline)!==null&&r!==void 0?r:e.marginXXS,U=new en(P).setA(.65).toRgbString();return{dropdownWidth:160,zIndexPopup:e.zIndexPopupBase+50,radiusItem:e.borderRadiusLG,itemBorderRadius:e.borderRadiusLG,radiusSubMenuItem:e.borderRadiusSM,subMenuItemBorderRadius:e.borderRadiusSM,colorItemText:d,itemColor:d,colorItemTextHover:d,itemHoverColor:d,colorItemTextHoverHorizontal:a,horizontalItemHoverColor:a,colorGroupTitle:g,groupTitleColor:g,colorItemTextSelected:a,itemSelectedColor:a,subMenuItemSelectedColor:a,colorItemTextSelectedHorizontal:a,horizontalItemSelectedColor:a,colorItemBg:h,itemBg:h,colorItemBgHover:S,itemHoverBg:S,colorItemBgActive:y,itemActiveBg:x,colorSubItemBg:p,subMenuItemBg:p,colorItemBgSelected:x,itemSelectedBg:x,colorItemBgSelectedHorizontal:"transparent",horizontalItemSelectedBg:"transparent",colorActiveBarWidth:0,activeBarWidth:F,colorActiveBarHeight:b,activeBarHeight:b,colorActiveBarBorderSize:C,activeBarBorderWidth:G,colorItemTextDisabled:s,itemDisabledColor:s,colorDangerItemText:l,dangerItemColor:l,colorDangerItemTextHover:l,dangerItemHoverColor:l,colorDangerItemTextSelected:l,dangerItemSelectedColor:l,colorDangerItemBgActive:u,dangerItemActiveBg:u,colorDangerItemBgSelected:u,dangerItemSelectedBg:u,itemMarginInline:V,horizontalItemBorderRadius:0,horizontalItemHoverBg:"transparent",itemHeight:E,groupTitleLineHeight:$,collapsedWidth:E*2,popupBg:O,itemMarginBlock:_,itemPaddingInline:R,horizontalLineHeight:`${E*1.15}px`,iconSize:T,iconMarginInlineEnd:M-T,collapsedIconSize:z,groupTitleFontSize:T,darkItemDisabledColor:new en(P).setA(.25).toRgbString(),darkItemColor:U,darkDangerItemColor:l,darkItemBg:"#001529",darkPopupBg:"#001529",darkSubMenuItemBg:"#000c17",darkItemSelectedColor:P,darkItemSelectedBg:a,darkDangerItemSelectedBg:l,darkItemHoverBg:"transparent",darkGroupTitleColor:U,darkItemHoverColor:P,darkDangerItemHoverColor:D,darkDangerItemSelectedColor:P,darkDangerItemActiveBg:l,itemWidth:F?`calc(100% + ${G}px)`:`calc(100% - ${V*2}px)`}},r9=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:e,n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!0;return xn("Menu",a=>{const{colorBgElevated:l,controlHeightLG:s,fontSize:u,darkItemColor:d,darkDangerItemColor:g,darkItemBg:h,darkSubMenuItemBg:p,darkItemSelectedColor:y,darkItemSelectedBg:C,darkDangerItemSelectedBg:b,darkItemHoverBg:x,darkGroupTitleColor:S,darkItemHoverColor:E,darkItemDisabledColor:$,darkDangerItemHoverColor:O,darkDangerItemSelectedColor:_,darkDangerItemActiveBg:R,popupBg:T,darkPopupBg:M}=a,z=a.calc(u).div(7).mul(5).equal(),P=Kt(a,{menuArrowSize:z,menuHorizontalHeight:a.calc(s).mul(1.15).equal(),menuArrowOffset:a.calc(z).mul(.25).equal(),menuSubMenuBg:l,calc:a.calc,popupBg:T}),D=Kt(P,{itemColor:d,itemHoverColor:E,groupTitleColor:S,itemSelectedColor:y,subMenuItemSelectedColor:y,itemBg:h,popupBg:M,subMenuItemBg:p,itemActiveBg:"transparent",itemSelectedBg:C,activeBarHeight:0,activeBarBorderWidth:0,itemHoverBg:x,itemDisabledColor:$,dangerItemColor:g,dangerItemHoverColor:O,dangerItemSelectedColor:_,dangerItemActiveBg:R,dangerItemSelectedBg:b,menuSubMenuBg:p,horizontalItemSelectedColor:y,horizontalItemSelectedBg:C});return[t9(P),ZH(P),e9(P),eE(P,"light"),eE(D,"dark"),JH(P),dy(P),Bd(P,"slide-up"),Bd(P,"slide-down"),Sm(P,"zoom-big")]},n9,{deprecatedTokens:[["colorGroupTitle","groupTitleColor"],["radiusItem","itemBorderRadius"],["radiusSubMenuItem","subMenuItemBorderRadius"],["colorItemText","itemColor"],["colorItemTextHover","itemHoverColor"],["colorItemTextHoverHorizontal","horizontalItemHoverColor"],["colorItemTextSelected","itemSelectedColor"],["colorItemTextSelectedHorizontal","horizontalItemSelectedColor"],["colorItemTextDisabled","itemDisabledColor"],["colorDangerItemText","dangerItemColor"],["colorDangerItemTextHover","dangerItemHoverColor"],["colorDangerItemTextSelected","dangerItemSelectedColor"],["colorDangerItemBgActive","dangerItemActiveBg"],["colorDangerItemBgSelected","dangerItemSelectedBg"],["colorItemBg","itemBg"],["colorItemBgHover","itemHoverBg"],["colorSubItemBg","subMenuItemBg"],["colorItemBgActive","itemActiveBg"],["colorItemBgSelectedHorizontal","horizontalItemSelectedBg"],["colorActiveBarWidth","activeBarWidth"],["colorActiveBarHeight","activeBarHeight"],["colorActiveBarBorderSize","activeBarBorderWidth"],["colorItemBgSelected","itemSelectedBg"]],injectStyle:n,unitless:{groupTitleLineHeight:!0}})(e,t)},eT=e=>{var t;const{popupClassName:n,icon:r,title:a,theme:l}=e,s=f.useContext(Ud),{prefixCls:u,inlineCollapsed:d,theme:g}=s,h=ys();let p;if(!r)p=d&&!h.length&&a&&typeof a=="string"?f.createElement("div",{className:`${u}-inline-collapsed-noicon`},a.charAt(0)):f.createElement("span",{className:`${u}-title-content`},a);else{const b=f.isValidElement(a)&&a.type==="span";p=f.createElement(f.Fragment,null,na(r,{className:re(f.isValidElement(r)?(t=r.props)===null||t===void 0?void 0:t.className:"",`${u}-item-icon`)}),b?a:f.createElement("span",{className:`${u}-title-content`},a))}const y=f.useMemo(()=>Object.assign(Object.assign({},s),{firstLevel:!1}),[s]),[C]=$u("Menu");return f.createElement(Ud.Provider,{value:y},f.createElement(Pm,Object.assign({},On(e,["icon"]),{title:p,popupClassName:re(u,n,`${u}-${l||g}`),popupStyle:Object.assign({zIndex:C},e.popupStyle)})))};var a9=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function Hp(e){return e===null||e===!1}const o9={item:J_,submenu:eT,divider:Z_},i9=f.forwardRef((e,t)=>{var n;const r=f.useContext(kd),a=r||{},{getPrefixCls:l,getPopupContainer:s,direction:u,menu:d}=f.useContext(dt),g=l(),{prefixCls:h,className:p,style:y,theme:C="light",expandIcon:b,_internalDisableMenuItemTitleTooltip:x,inlineCollapsed:S,siderCollapsed:E,rootClassName:$,mode:O,selectable:_,onClick:R,overflowedIndicatorPopupClassName:T}=e,M=a9(e,["prefixCls","className","style","theme","expandIcon","_internalDisableMenuItemTitleTooltip","inlineCollapsed","siderCollapsed","rootClassName","mode","selectable","onClick","overflowedIndicatorPopupClassName"]),z=On(M,["collapsedWidth"]);(n=a.validator)===null||n===void 0||n.call(a,{mode:O});const P=An(function(){var W;R==null||R.apply(void 0,arguments),(W=a.onClick)===null||W===void 0||W.call(a)}),D=a.mode||O,F=_??a.selectable,G=S??E,V={horizontal:{motionName:`${g}-slide-up`},inline:Ld(g),other:{motionName:`${g}-zoom-big`}},U=l("menu",h||a.prefixCls),K=ia(U),[j,A,L]=r9(U,K,!r),q=re(`${U}-${C}`,d==null?void 0:d.className,p),N=f.useMemo(()=>{var W,X;if(typeof b=="function"||Hp(b))return b||null;if(typeof a.expandIcon=="function"||Hp(a.expandIcon))return a.expandIcon||null;if(typeof(d==null?void 0:d.expandIcon)=="function"||Hp(d==null?void 0:d.expandIcon))return(d==null?void 0:d.expandIcon)||null;const k=(W=b??(a==null?void 0:a.expandIcon))!==null&&W!==void 0?W:d==null?void 0:d.expandIcon;return na(k,{className:re(`${U}-submenu-expand-icon`,f.isValidElement(k)?(X=k.props)===null||X===void 0?void 0:X.className:void 0)})},[b,a==null?void 0:a.expandIcon,d==null?void 0:d.expandIcon,U]),B=f.useMemo(()=>({prefixCls:U,inlineCollapsed:G||!1,direction:u,firstLevel:!0,theme:C,mode:D,disableMenuItemTitleTooltip:x}),[U,G,u,x,C]);return j(f.createElement(kd.Provider,{value:null},f.createElement(Ud.Provider,{value:B},f.createElement(ju,Object.assign({getPopupContainer:s,overflowedIndicator:f.createElement(T4,null),overflowedIndicatorPopupClassName:re(U,`${U}-${C}`,T),mode:D,selectable:F,onClick:P},z,{inlineCollapsed:G,style:Object.assign(Object.assign({},d==null?void 0:d.style),y),className:q,prefixCls:U,direction:u,defaultMotions:V,expandIcon:N,ref:t,rootClassName:re($,A,a.rootClassName,L,K),_internalComponents:o9})))))}),bs=f.forwardRef((e,t)=>{const n=f.useRef(null),r=f.useContext(Am);return f.useImperativeHandle(t,()=>({menu:n.current,focus:a=>{var l;(l=n.current)===null||l===void 0||l.focus(a)}})),f.createElement(i9,Object.assign({ref:n},e,r))});bs.Item=J_;bs.SubMenu=eT;bs.Divider=Z_;bs.ItemGroup=Ly;function zm(e){return Kt(e,{inputAffixPadding:e.paddingXXS})}const Dm=e=>{const{controlHeight:t,fontSize:n,lineHeight:r,lineWidth:a,controlHeightSM:l,controlHeightLG:s,fontSizeLG:u,lineHeightLG:d,paddingSM:g,controlPaddingHorizontalSM:h,controlPaddingHorizontal:p,colorFillAlter:y,colorPrimaryHover:C,colorPrimary:b,controlOutlineWidth:x,controlOutline:S,colorErrorOutline:E,colorWarningOutline:$,colorBgContainer:O,inputFontSize:_,inputFontSizeLG:R,inputFontSizeSM:T}=e,M=_||n,z=T||M,P=R||u,D=Math.round((t-M*r)/2*10)/10-a,F=Math.round((l-z*r)/2*10)/10-a,G=Math.ceil((s-P*d)/2*10)/10-a;return{paddingBlock:Math.max(D,0),paddingBlockSM:Math.max(F,0),paddingBlockLG:Math.max(G,0),paddingInline:g-a,paddingInlineSM:h-a,paddingInlineLG:p-a,addonBg:y,activeBorderColor:b,hoverBorderColor:C,activeShadow:`0 0 0 ${x}px ${S}`,errorActiveShadow:`0 0 0 ${x}px ${E}`,warningActiveShadow:`0 0 0 ${x}px ${$}`,hoverBg:O,activeBg:O,inputFontSize:M,inputFontSizeLG:P,inputFontSizeSM:z}},l9=e=>({borderColor:e.hoverBorderColor,backgroundColor:e.hoverBg}),By=e=>({color:e.colorTextDisabled,backgroundColor:e.colorBgContainerDisabled,borderColor:e.colorBorder,boxShadow:"none",cursor:"not-allowed",opacity:1,"input[disabled], textarea[disabled]":{cursor:"not-allowed"},"&:hover:not([disabled])":Object.assign({},l9(Kt(e,{hoverBorderColor:e.colorBorder,hoverBg:e.colorBgContainerDisabled})))}),tT=(e,t)=>({background:e.colorBgContainer,borderWidth:e.lineWidth,borderStyle:e.lineType,borderColor:t.borderColor,"&:hover":{borderColor:t.hoverBorderColor,backgroundColor:e.hoverBg},"&:focus, &:focus-within":{borderColor:t.activeBorderColor,boxShadow:t.activeShadow,outline:0,backgroundColor:e.activeBg}}),aE=(e,t)=>({[`&${e.componentCls}-status-${t.status}:not(${e.componentCls}-disabled)`]:Object.assign(Object.assign({},tT(e,t)),{[`${e.componentCls}-prefix, ${e.componentCls}-suffix`]:{color:t.affixColor}}),[`&${e.componentCls}-status-${t.status}${e.componentCls}-disabled`]:{borderColor:t.borderColor}}),s9=(e,t)=>({"&-outlined":Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},tT(e,{borderColor:e.colorBorder,hoverBorderColor:e.hoverBorderColor,activeBorderColor:e.activeBorderColor,activeShadow:e.activeShadow})),{[`&${e.componentCls}-disabled, &[disabled]`]:Object.assign({},By(e))}),aE(e,{status:"error",borderColor:e.colorError,hoverBorderColor:e.colorErrorBorderHover,activeBorderColor:e.colorError,activeShadow:e.errorActiveShadow,affixColor:e.colorError})),aE(e,{status:"warning",borderColor:e.colorWarning,hoverBorderColor:e.colorWarningBorderHover,activeBorderColor:e.colorWarning,activeShadow:e.warningActiveShadow,affixColor:e.colorWarning})),t)}),oE=(e,t)=>({[`&${e.componentCls}-group-wrapper-status-${t.status}`]:{[`${e.componentCls}-group-addon`]:{borderColor:t.addonBorderColor,color:t.addonColor}}}),c9=e=>({"&-outlined":Object.assign(Object.assign(Object.assign({[`${e.componentCls}-group`]:{"&-addon":{background:e.addonBg,border:`${ve(e.lineWidth)} ${e.lineType} ${e.colorBorder}`},"&-addon:first-child":{borderInlineEnd:0},"&-addon:last-child":{borderInlineStart:0}}},oE(e,{status:"error",addonBorderColor:e.colorError,addonColor:e.colorErrorText})),oE(e,{status:"warning",addonBorderColor:e.colorWarning,addonColor:e.colorWarningText})),{[`&${e.componentCls}-group-wrapper-disabled`]:{[`${e.componentCls}-group-addon`]:Object.assign({},By(e))}})}),u9=(e,t)=>{const{componentCls:n}=e;return{"&-borderless":Object.assign({background:"transparent",border:"none","&:focus, &:focus-within":{outline:"none"},[`&${n}-disabled, &[disabled]`]:{color:e.colorTextDisabled,cursor:"not-allowed"},[`&${n}-status-error`]:{"&, & input, & textarea":{color:e.colorError}},[`&${n}-status-warning`]:{"&, & input, & textarea":{color:e.colorWarning}}},t)}},nT=(e,t)=>{var n;return{background:t.bg,borderWidth:e.lineWidth,borderStyle:e.lineType,borderColor:"transparent","input&, & input, textarea&, & textarea":{color:(n=t==null?void 0:t.inputColor)!==null&&n!==void 0?n:"unset"},"&:hover":{background:t.hoverBg},"&:focus, &:focus-within":{outline:0,borderColor:t.activeBorderColor,backgroundColor:e.activeBg}}},iE=(e,t)=>({[`&${e.componentCls}-status-${t.status}:not(${e.componentCls}-disabled)`]:Object.assign(Object.assign({},nT(e,t)),{[`${e.componentCls}-prefix, ${e.componentCls}-suffix`]:{color:t.affixColor}})}),f9=(e,t)=>({"&-filled":Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},nT(e,{bg:e.colorFillTertiary,hoverBg:e.colorFillSecondary,activeBorderColor:e.activeBorderColor})),{[`&${e.componentCls}-disabled, &[disabled]`]:Object.assign({},By(e))}),iE(e,{status:"error",bg:e.colorErrorBg,hoverBg:e.colorErrorBgHover,activeBorderColor:e.colorError,inputColor:e.colorErrorText,affixColor:e.colorError})),iE(e,{status:"warning",bg:e.colorWarningBg,hoverBg:e.colorWarningBgHover,activeBorderColor:e.colorWarning,inputColor:e.colorWarningText,affixColor:e.colorWarning})),t)}),lE=(e,t)=>({[`&${e.componentCls}-group-wrapper-status-${t.status}`]:{[`${e.componentCls}-group-addon`]:{background:t.addonBg,color:t.addonColor}}}),d9=e=>({"&-filled":Object.assign(Object.assign(Object.assign({[`${e.componentCls}-group`]:{"&-addon":{background:e.colorFillTertiary},[`${e.componentCls}-filled:not(:focus):not(:focus-within)`]:{"&:not(:first-child)":{borderInlineStart:`${ve(e.lineWidth)} ${e.lineType} ${e.colorSplit}`},"&:not(:last-child)":{borderInlineEnd:`${ve(e.lineWidth)} ${e.lineType} ${e.colorSplit}`}}}},lE(e,{status:"error",addonBg:e.colorErrorBg,addonColor:e.colorErrorText})),lE(e,{status:"warning",addonBg:e.colorWarningBg,addonColor:e.colorWarningText})),{[`&${e.componentCls}-group-wrapper-disabled`]:{[`${e.componentCls}-group`]:{"&-addon":{background:e.colorFillTertiary,color:e.colorTextDisabled},"&-addon:first-child":{borderInlineStart:`${ve(e.lineWidth)} ${e.lineType} ${e.colorBorder}`,borderTop:`${ve(e.lineWidth)} ${e.lineType} ${e.colorBorder}`,borderBottom:`${ve(e.lineWidth)} ${e.lineType} ${e.colorBorder}`},"&-addon:last-child":{borderInlineEnd:`${ve(e.lineWidth)} ${e.lineType} ${e.colorBorder}`,borderTop:`${ve(e.lineWidth)} ${e.lineType} ${e.colorBorder}`,borderBottom:`${ve(e.lineWidth)} ${e.lineType} ${e.colorBorder}`}}}})}),rT=(e,t)=>({background:e.colorBgContainer,borderWidth:`${ve(e.lineWidth)} 0`,borderStyle:`${e.lineType} none`,borderColor:`transparent transparent ${t.borderColor} transparent`,borderRadius:0,"&:hover":{borderColor:`transparent transparent ${t.borderColor} transparent`,backgroundColor:e.hoverBg},"&:focus, &:focus-within":{borderColor:`transparent transparent ${t.borderColor} transparent`,outline:0,backgroundColor:e.activeBg}}),sE=(e,t)=>({[`&${e.componentCls}-status-${t.status}:not(${e.componentCls}-disabled)`]:Object.assign(Object.assign({},rT(e,t)),{[`${e.componentCls}-prefix, ${e.componentCls}-suffix`]:{color:t.affixColor}}),[`&${e.componentCls}-status-${t.status}${e.componentCls}-disabled`]:{borderColor:`transparent transparent ${t.borderColor} transparent`}}),m9=(e,t)=>({"&-underlined":Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},rT(e,{borderColor:e.colorBorder,hoverBorderColor:e.hoverBorderColor,activeBorderColor:e.activeBorderColor,activeShadow:e.activeShadow})),{[`&${e.componentCls}-disabled, &[disabled]`]:{color:e.colorTextDisabled,boxShadow:"none",cursor:"not-allowed","&:hover":{borderColor:`transparent transparent ${e.colorBorder} transparent`}},"input[disabled], textarea[disabled]":{cursor:"not-allowed"}}),sE(e,{status:"error",borderColor:e.colorError,hoverBorderColor:e.colorErrorBorderHover,activeBorderColor:e.colorError,activeShadow:e.errorActiveShadow,affixColor:e.colorError})),sE(e,{status:"warning",borderColor:e.colorWarning,hoverBorderColor:e.colorWarningBorderHover,activeBorderColor:e.colorWarning,activeShadow:e.warningActiveShadow,affixColor:e.colorWarning})),t)}),g9=e=>({"&::-moz-placeholder":{opacity:1},"&::placeholder":{color:e,userSelect:"none"},"&:placeholder-shown":{textOverflow:"ellipsis"}}),aT=e=>{const{paddingBlockLG:t,lineHeightLG:n,borderRadiusLG:r,paddingInlineLG:a}=e;return{padding:`${ve(t)} ${ve(a)}`,fontSize:e.inputFontSizeLG,lineHeight:n,borderRadius:r}},oT=e=>({padding:`${ve(e.paddingBlockSM)} ${ve(e.paddingInlineSM)}`,fontSize:e.inputFontSizeSM,borderRadius:e.borderRadiusSM}),iT=e=>Object.assign(Object.assign({position:"relative",display:"inline-block",width:"100%",minWidth:0,padding:`${ve(e.paddingBlock)} ${ve(e.paddingInline)}`,color:e.colorText,fontSize:e.inputFontSize,lineHeight:e.lineHeight,borderRadius:e.borderRadius,transition:`all ${e.motionDurationMid}`},g9(e.colorTextPlaceholder)),{"&-lg":Object.assign({},aT(e)),"&-sm":Object.assign({},oT(e)),"&-rtl, &-textarea-rtl":{direction:"rtl"}}),h9=e=>{const{componentCls:t,antCls:n}=e;return{position:"relative",display:"table",width:"100%",borderCollapse:"separate",borderSpacing:0,"&[class*='col-']":{paddingInlineEnd:e.paddingXS,"&:last-child":{paddingInlineEnd:0}},[`&-lg ${t}, &-lg > ${t}-group-addon`]:Object.assign({},aT(e)),[`&-sm ${t}, &-sm > ${t}-group-addon`]:Object.assign({},oT(e)),[`&-lg ${n}-select-single ${n}-select-selector`]:{height:e.controlHeightLG},[`&-sm ${n}-select-single ${n}-select-selector`]:{height:e.controlHeightSM},[`> ${t}`]:{display:"table-cell","&:not(:first-child):not(:last-child)":{borderRadius:0}},[`${t}-group`]:{"&-addon, &-wrap":{display:"table-cell",width:1,whiteSpace:"nowrap",verticalAlign:"middle","&:not(:first-child):not(:last-child)":{borderRadius:0}},"&-wrap > *":{display:"block !important"},"&-addon":{position:"relative",padding:`0 ${ve(e.paddingInline)}`,color:e.colorText,fontWeight:"normal",fontSize:e.inputFontSize,textAlign:"center",borderRadius:e.borderRadius,transition:`all ${e.motionDurationSlow}`,lineHeight:1,[`${n}-select`]:{margin:`${ve(e.calc(e.paddingBlock).add(1).mul(-1).equal())} ${ve(e.calc(e.paddingInline).mul(-1).equal())}`,[`&${n}-select-single:not(${n}-select-customize-input):not(${n}-pagination-size-changer)`]:{[`${n}-select-selector`]:{backgroundColor:"inherit",border:`${ve(e.lineWidth)} ${e.lineType} transparent`,boxShadow:"none"}}},[`${n}-cascader-picker`]:{margin:`-9px ${ve(e.calc(e.paddingInline).mul(-1).equal())}`,backgroundColor:"transparent",[`${n}-cascader-input`]:{textAlign:"start",border:0,boxShadow:"none"}}}},[t]:{width:"100%",marginBottom:0,textAlign:"inherit","&:focus":{zIndex:1,borderInlineEndWidth:1},"&:hover":{zIndex:1,borderInlineEndWidth:1,[`${t}-search-with-button &`]:{zIndex:0}}},[`> ${t}:first-child, ${t}-group-addon:first-child`]:{borderStartEndRadius:0,borderEndEndRadius:0,[`${n}-select ${n}-select-selector`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`> ${t}-affix-wrapper`]:{[`&:not(:first-child) ${t}`]:{borderStartStartRadius:0,borderEndStartRadius:0},[`&:not(:last-child) ${t}`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`> ${t}:last-child, ${t}-group-addon:last-child`]:{borderStartStartRadius:0,borderEndStartRadius:0,[`${n}-select ${n}-select-selector`]:{borderStartStartRadius:0,borderEndStartRadius:0}},[`${t}-affix-wrapper`]:{"&:not(:last-child)":{borderStartEndRadius:0,borderEndEndRadius:0,[`${t}-search &`]:{borderStartStartRadius:e.borderRadius,borderEndStartRadius:e.borderRadius}},[`&:not(:first-child), ${t}-search &:not(:first-child)`]:{borderStartStartRadius:0,borderEndStartRadius:0}},[`&${t}-group-compact`]:Object.assign(Object.assign({display:"block"},Ad()),{[`${t}-group-addon, ${t}-group-wrap, > ${t}`]:{"&:not(:first-child):not(:last-child)":{borderInlineEndWidth:e.lineWidth,"&:hover, &:focus":{zIndex:1}}},"& > *":{display:"inline-flex",float:"none",verticalAlign:"top",borderRadius:0},[`
        & > ${t}-affix-wrapper,
        & > ${t}-number-affix-wrapper,
        & > ${n}-picker-range
      `]:{display:"inline-flex"},"& > *:not(:last-child)":{marginInlineEnd:e.calc(e.lineWidth).mul(-1).equal(),borderInlineEndWidth:e.lineWidth},[t]:{float:"none"},[`& > ${n}-select > ${n}-select-selector,
      & > ${n}-select-auto-complete ${t},
      & > ${n}-cascader-picker ${t},
      & > ${t}-group-wrapper ${t}`]:{borderInlineEndWidth:e.lineWidth,borderRadius:0,"&:hover, &:focus":{zIndex:1}},[`& > ${n}-select-focused`]:{zIndex:1},[`& > ${n}-select > ${n}-select-arrow`]:{zIndex:1},[`& > *:first-child,
      & > ${n}-select:first-child > ${n}-select-selector,
      & > ${n}-select-auto-complete:first-child ${t},
      & > ${n}-cascader-picker:first-child ${t}`]:{borderStartStartRadius:e.borderRadius,borderEndStartRadius:e.borderRadius},[`& > *:last-child,
      & > ${n}-select:last-child > ${n}-select-selector,
      & > ${n}-cascader-picker:last-child ${t},
      & > ${n}-cascader-picker-focused:last-child ${t}`]:{borderInlineEndWidth:e.lineWidth,borderStartEndRadius:e.borderRadius,borderEndEndRadius:e.borderRadius},[`& > ${n}-select-auto-complete ${t}`]:{verticalAlign:"top"},[`${t}-group-wrapper + ${t}-group-wrapper`]:{marginInlineStart:e.calc(e.lineWidth).mul(-1).equal(),[`${t}-affix-wrapper`]:{borderRadius:0}},[`${t}-group-wrapper:not(:last-child)`]:{[`&${t}-search > ${t}-group`]:{[`& > ${t}-group-addon > ${t}-search-button`]:{borderRadius:0},[`& > ${t}`]:{borderStartStartRadius:e.borderRadius,borderStartEndRadius:0,borderEndEndRadius:0,borderEndStartRadius:e.borderRadius}}}})}},p9=e=>{const{componentCls:t,controlHeightSM:n,lineWidth:r,calc:a}=e,s=a(n).sub(a(r).mul(2)).sub(16).div(2).equal();return{[t]:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},Rr(e)),iT(e)),s9(e)),f9(e)),u9(e)),m9(e)),{'&[type="color"]':{height:e.controlHeight,[`&${t}-lg`]:{height:e.controlHeightLG},[`&${t}-sm`]:{height:n,paddingTop:s,paddingBottom:s}},'&[type="search"]::-webkit-search-cancel-button, &[type="search"]::-webkit-search-decoration':{appearance:"none"}})}},v9=e=>{const{componentCls:t}=e;return{[`${t}-clear-icon`]:{margin:0,padding:0,lineHeight:0,color:e.colorTextQuaternary,fontSize:e.fontSizeIcon,verticalAlign:-1,cursor:"pointer",transition:`color ${e.motionDurationSlow}`,border:"none",outline:"none",backgroundColor:"transparent","&:hover":{color:e.colorIcon},"&:active":{color:e.colorText},"&-hidden":{visibility:"hidden"},"&-has-suffix":{margin:`0 ${ve(e.inputAffixPadding)}`}}}},y9=e=>{const{componentCls:t,inputAffixPadding:n,colorTextDescription:r,motionDurationSlow:a,colorIcon:l,colorIconHover:s,iconCls:u}=e,d=`${t}-affix-wrapper`,g=`${t}-affix-wrapper-disabled`;return{[d]:Object.assign(Object.assign(Object.assign(Object.assign({},iT(e)),{display:"inline-flex",[`&:not(${t}-disabled):hover`]:{zIndex:1,[`${t}-search-with-button &`]:{zIndex:0}},"&-focused, &:focus":{zIndex:1},[`> input${t}`]:{padding:0},[`> input${t}, > textarea${t}`]:{fontSize:"inherit",border:"none",borderRadius:0,outline:"none",background:"transparent",color:"inherit","&::-ms-reveal":{display:"none"},"&:focus":{boxShadow:"none !important"}},"&::before":{display:"inline-block",width:0,visibility:"hidden",content:'"\\a0"'},[t]:{"&-prefix, &-suffix":{display:"flex",flex:"none",alignItems:"center","> *:not(:last-child)":{marginInlineEnd:e.paddingXS}},"&-show-count-suffix":{color:r,direction:"ltr"},"&-show-count-has-suffix":{marginInlineEnd:e.paddingXXS},"&-prefix":{marginInlineEnd:n},"&-suffix":{marginInlineStart:n}}}),v9(e)),{[`${u}${t}-password-icon`]:{color:l,cursor:"pointer",transition:`all ${a}`,"&:hover":{color:s}}}),[`${t}-underlined`]:{borderRadius:0},[g]:{[`${u}${t}-password-icon`]:{color:l,cursor:"not-allowed","&:hover":{color:l}}}}},b9=e=>{const{componentCls:t,borderRadiusLG:n,borderRadiusSM:r}=e;return{[`${t}-group`]:Object.assign(Object.assign(Object.assign({},Rr(e)),h9(e)),{"&-rtl":{direction:"rtl"},"&-wrapper":Object.assign(Object.assign(Object.assign({display:"inline-block",width:"100%",textAlign:"start",verticalAlign:"top","&-rtl":{direction:"rtl"},"&-lg":{[`${t}-group-addon`]:{borderRadius:n,fontSize:e.inputFontSizeLG}},"&-sm":{[`${t}-group-addon`]:{borderRadius:r}}},c9(e)),d9(e)),{[`&:not(${t}-compact-first-item):not(${t}-compact-last-item)${t}-compact-item`]:{[`${t}, ${t}-group-addon`]:{borderRadius:0}},[`&:not(${t}-compact-last-item)${t}-compact-first-item`]:{[`${t}, ${t}-group-addon`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`&:not(${t}-compact-first-item)${t}-compact-last-item`]:{[`${t}, ${t}-group-addon`]:{borderStartStartRadius:0,borderEndStartRadius:0}},[`&:not(${t}-compact-last-item)${t}-compact-item`]:{[`${t}-affix-wrapper`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`&:not(${t}-compact-first-item)${t}-compact-item`]:{[`${t}-affix-wrapper`]:{borderStartStartRadius:0,borderEndStartRadius:0}}})})}},S9=e=>{const{componentCls:t,antCls:n}=e,r=`${t}-search`;return{[r]:{[t]:{"&:hover, &:focus":{[`+ ${t}-group-addon ${r}-button:not(${n}-btn-primary)`]:{borderInlineStartColor:e.colorPrimaryHover}}},[`${t}-affix-wrapper`]:{height:e.controlHeight,borderRadius:0},[`${t}-lg`]:{lineHeight:e.calc(e.lineHeightLG).sub(2e-4).equal()},[`> ${t}-group`]:{[`> ${t}-group-addon:last-child`]:{insetInlineStart:-1,padding:0,border:0,[`${r}-button`]:{marginInlineEnd:-1,borderStartStartRadius:0,borderEndStartRadius:0,boxShadow:"none"},[`${r}-button:not(${n}-btn-primary)`]:{color:e.colorTextDescription,"&:hover":{color:e.colorPrimaryHover},"&:active":{color:e.colorPrimaryActive},[`&${n}-btn-loading::before`]:{insetInlineStart:0,insetInlineEnd:0,insetBlockStart:0,insetBlockEnd:0}}}},[`${r}-button`]:{height:e.controlHeight,"&:hover, &:focus":{zIndex:1}},"&-large":{[`${t}-affix-wrapper, ${r}-button`]:{height:e.controlHeightLG}},"&-small":{[`${t}-affix-wrapper, ${r}-button`]:{height:e.controlHeightSM}},"&-rtl":{direction:"rtl"},[`&${t}-compact-item`]:{[`&:not(${t}-compact-last-item)`]:{[`${t}-group-addon`]:{[`${t}-search-button`]:{marginInlineEnd:e.calc(e.lineWidth).mul(-1).equal(),borderRadius:0}}},[`&:not(${t}-compact-first-item)`]:{[`${t},${t}-affix-wrapper`]:{borderRadius:0}},[`> ${t}-group-addon ${t}-search-button,
        > ${t},
        ${t}-affix-wrapper`]:{"&:hover, &:focus, &:active":{zIndex:2}},[`> ${t}-affix-wrapper-focused`]:{zIndex:2}}}}},C9=e=>{const{componentCls:t}=e;return{[`${t}-out-of-range`]:{[`&, & input, & textarea, ${t}-show-count-suffix, ${t}-data-count`]:{color:e.colorError}}}},lT=xn(["Input","Shared"],e=>{const t=Kt(e,zm(e));return[p9(t),y9(t)]},Dm,{resetFont:!1}),sT=xn(["Input","Component"],e=>{const t=Kt(e,zm(e));return[b9(t),S9(t),C9(t),hy(t)]},Dm,{resetFont:!1}),cT=f.createContext({});var x9=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function cE(e){return typeof e=="number"?`${e} ${e} auto`:/^\d+(\.\d+)?(px|em|rem|%)$/.test(e)?`0 0 ${e}`:e}const w9=["xs","sm","md","lg","xl","xxl"],uT=f.forwardRef((e,t)=>{const{getPrefixCls:n,direction:r}=f.useContext(dt),{gutter:a,wrap:l}=f.useContext(cT),{prefixCls:s,span:u,order:d,offset:g,push:h,pull:p,className:y,children:C,flex:b,style:x}=e,S=x9(e,["prefixCls","span","order","offset","push","pull","className","children","flex","style"]),E=n("col",s),[$,O,_]=HL(E),R={};let T={};w9.forEach(P=>{let D={};const F=e[P];typeof F=="number"?D.span=F:typeof F=="object"&&(D=F||{}),delete S[P],T=Object.assign(Object.assign({},T),{[`${E}-${P}-${D.span}`]:D.span!==void 0,[`${E}-${P}-order-${D.order}`]:D.order||D.order===0,[`${E}-${P}-offset-${D.offset}`]:D.offset||D.offset===0,[`${E}-${P}-push-${D.push}`]:D.push||D.push===0,[`${E}-${P}-pull-${D.pull}`]:D.pull||D.pull===0,[`${E}-rtl`]:r==="rtl"}),D.flex&&(T[`${E}-${P}-flex`]=!0,R[`--${E}-${P}-flex`]=cE(D.flex))});const M=re(E,{[`${E}-${u}`]:u!==void 0,[`${E}-order-${d}`]:d,[`${E}-offset-${g}`]:g,[`${E}-push-${h}`]:h,[`${E}-pull-${p}`]:p},y,T,O,_),z={};if(a&&a[0]>0){const P=a[0]/2;z.paddingLeft=P,z.paddingRight=P}return b&&(z.flex=cE(b),l===!1&&!z.minWidth&&(z.minWidth=0)),$(f.createElement("div",Object.assign({},S,{style:Object.assign(Object.assign(Object.assign({},z),x),R),className:M,ref:t}),C))});function E9(e,t){const n=[void 0,void 0],r=Array.isArray(e)?e:[e,void 0],a=t||{xs:!0,sm:!0,md:!0,lg:!0,xl:!0,xxl:!0};return r.forEach((l,s)=>{if(typeof l=="object"&&l!==null)for(let u=0;u<uu.length;u++){const d=uu[u];if(a[d]&&l[d]!==void 0){n[s]=l[d];break}}else n[s]=l}),n}var $9=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function uE(e,t){const[n,r]=f.useState(typeof e=="string"?e:""),a=()=>{if(typeof e=="string"&&r(e),typeof e=="object")for(let l=0;l<uu.length;l++){const s=uu[l];if(!t||!t[s])continue;const u=e[s];if(u!==void 0){r(u);return}}};return f.useEffect(()=>{a()},[JSON.stringify(e),t]),n}const O9=f.forwardRef((e,t)=>{const{prefixCls:n,justify:r,align:a,className:l,style:s,children:u,gutter:d=0,wrap:g}=e,h=$9(e,["prefixCls","justify","align","className","style","children","gutter","wrap"]),{getPrefixCls:p,direction:y}=f.useContext(dt),C=L7(!0,null),b=uE(a,C),x=uE(r,C),S=p("row",n),[E,$,O]=BL(S),_=E9(d,C),R=re(S,{[`${S}-no-wrap`]:g===!1,[`${S}-${x}`]:x,[`${S}-${b}`]:b,[`${S}-rtl`]:y==="rtl"},l,$,O),T={},M=_[0]!=null&&_[0]>0?_[0]/-2:void 0;M&&(T.marginLeft=M,T.marginRight=M);const[z,P]=_;T.rowGap=P;const D=f.useMemo(()=>({gutter:[z,P],wrap:g}),[z,P,g]);return E(f.createElement(cT.Provider,{value:D},f.createElement("div",Object.assign({},h,{className:R,style:Object.assign(Object.assign({},T),s),ref:t}),u)))}),R9=e=>{const{componentCls:t,sizePaddingEdgeHorizontal:n,colorSplit:r,lineWidth:a,textPaddingInline:l,orientationMargin:s,verticalMarginInline:u}=e;return{[t]:Object.assign(Object.assign({},Rr(e)),{borderBlockStart:`${ve(a)} solid ${r}`,"&-vertical":{position:"relative",top:"-0.06em",display:"inline-block",height:"0.9em",marginInline:u,marginBlock:0,verticalAlign:"middle",borderTop:0,borderInlineStart:`${ve(a)} solid ${r}`},"&-horizontal":{display:"flex",clear:"both",width:"100%",minWidth:"100%",margin:`${ve(e.dividerHorizontalGutterMargin)} 0`},[`&-horizontal${t}-with-text`]:{display:"flex",alignItems:"center",margin:`${ve(e.dividerHorizontalWithTextGutterMargin)} 0`,color:e.colorTextHeading,fontWeight:500,fontSize:e.fontSizeLG,whiteSpace:"nowrap",textAlign:"center",borderBlockStart:`0 ${r}`,"&::before, &::after":{position:"relative",width:"50%",borderBlockStart:`${ve(a)} solid transparent`,borderBlockStartColor:"inherit",borderBlockEnd:0,transform:"translateY(50%)",content:"''"}},[`&-horizontal${t}-with-text-start`]:{"&::before":{width:`calc(${s} * 100%)`},"&::after":{width:`calc(100% - ${s} * 100%)`}},[`&-horizontal${t}-with-text-end`]:{"&::before":{width:`calc(100% - ${s} * 100%)`},"&::after":{width:`calc(${s} * 100%)`}},[`${t}-inner-text`]:{display:"inline-block",paddingBlock:0,paddingInline:l},"&-dashed":{background:"none",borderColor:r,borderStyle:"dashed",borderWidth:`${ve(a)} 0 0`},[`&-horizontal${t}-with-text${t}-dashed`]:{"&::before, &::after":{borderStyle:"dashed none none"}},[`&-vertical${t}-dashed`]:{borderInlineStartWidth:a,borderInlineEnd:0,borderBlockStart:0,borderBlockEnd:0},"&-dotted":{background:"none",borderColor:r,borderStyle:"dotted",borderWidth:`${ve(a)} 0 0`},[`&-horizontal${t}-with-text${t}-dotted`]:{"&::before, &::after":{borderStyle:"dotted none none"}},[`&-vertical${t}-dotted`]:{borderInlineStartWidth:a,borderInlineEnd:0,borderBlockStart:0,borderBlockEnd:0},[`&-plain${t}-with-text`]:{color:e.colorText,fontWeight:"normal",fontSize:e.fontSize},[`&-horizontal${t}-with-text-start${t}-no-default-orientation-margin-start`]:{"&::before":{width:0},"&::after":{width:"100%"},[`${t}-inner-text`]:{paddingInlineStart:n}},[`&-horizontal${t}-with-text-end${t}-no-default-orientation-margin-end`]:{"&::before":{width:"100%"},"&::after":{width:0},[`${t}-inner-text`]:{paddingInlineEnd:n}}})}},_9=e=>({textPaddingInline:"1em",orientationMargin:.05,verticalMarginInline:e.marginXS}),T9=xn("Divider",e=>{const t=Kt(e,{dividerHorizontalWithTextGutterMargin:e.margin,dividerHorizontalGutterMargin:e.marginLG,sizePaddingEdgeHorizontal:0});return[R9(t)]},_9,{unitless:{orientationMargin:!0}});var M9=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const qd=e=>{const{getPrefixCls:t,direction:n,className:r,style:a}=Or("divider"),{prefixCls:l,type:s="horizontal",orientation:u="center",orientationMargin:d,className:g,rootClassName:h,children:p,dashed:y,variant:C="solid",plain:b,style:x}=e,S=M9(e,["prefixCls","type","orientation","orientationMargin","className","rootClassName","children","dashed","variant","plain","style"]),E=t("divider",l),[$,O,_]=T9(E),R=!!p,T=f.useMemo(()=>u==="left"?n==="rtl"?"end":"start":u==="right"?n==="rtl"?"start":"end":u,[n,u]),M=T==="start"&&d!=null,z=T==="end"&&d!=null,P=re(E,r,O,_,`${E}-${s}`,{[`${E}-with-text`]:R,[`${E}-with-text-${T}`]:R,[`${E}-dashed`]:!!y,[`${E}-${C}`]:C!=="solid",[`${E}-plain`]:!!b,[`${E}-rtl`]:n==="rtl",[`${E}-no-default-orientation-margin-start`]:M,[`${E}-no-default-orientation-margin-end`]:z},g,h),D=f.useMemo(()=>typeof d=="number"?d:/^\d+$/.test(d)?Number(d):d,[d]),F={marginInlineStart:M?D:void 0,marginInlineEnd:z?D:void 0};return $(f.createElement("div",Object.assign({className:P,style:Object.assign(Object.assign({},a),x)},S,{role:"separator"}),p&&s!=="vertical"&&f.createElement("span",{className:`${E}-inner-text`,style:F},p)))};function N9(e){return!!(e.addonBefore||e.addonAfter)}function j9(e){return!!(e.prefix||e.suffix||e.allowClear)}function fE(e,t,n){var r=t.cloneNode(!0),a=Object.create(e,{target:{value:r},currentTarget:{value:r}});return r.value=n,typeof t.selectionStart=="number"&&typeof t.selectionEnd=="number"&&(r.selectionStart=t.selectionStart,r.selectionEnd=t.selectionEnd),r.setSelectionRange=function(){t.setSelectionRange.apply(t,arguments)},a}function Wd(e,t,n,r){if(n){var a=t;if(t.type==="click"){a=fE(t,e,""),n(a);return}if(e.type!=="file"&&r!==void 0){a=fE(t,e,r),n(a);return}n(a)}}function fT(e,t){if(e){e.focus(t);var n=t||{},r=n.cursor;if(r){var a=e.value.length;switch(r){case"start":e.setSelectionRange(0,0);break;case"end":e.setSelectionRange(a,a);break;default:e.setSelectionRange(0,a)}}}}var dT=se.forwardRef(function(e,t){var n,r,a,l=e.inputElement,s=e.children,u=e.prefixCls,d=e.prefix,g=e.suffix,h=e.addonBefore,p=e.addonAfter,y=e.className,C=e.style,b=e.disabled,x=e.readOnly,S=e.focused,E=e.triggerFocus,$=e.allowClear,O=e.value,_=e.handleReset,R=e.hidden,T=e.classes,M=e.classNames,z=e.dataAttrs,P=e.styles,D=e.components,F=e.onClear,G=s??l,V=(D==null?void 0:D.affixWrapper)||"span",U=(D==null?void 0:D.groupWrapper)||"span",K=(D==null?void 0:D.wrapper)||"span",j=(D==null?void 0:D.groupAddon)||"span",A=f.useRef(null),L=function(fe){var Se;(Se=A.current)!==null&&Se!==void 0&&Se.contains(fe.target)&&(E==null||E())},q=j9(e),N=f.cloneElement(G,{value:O,className:re((n=G.props)===null||n===void 0?void 0:n.className,!q&&(M==null?void 0:M.variant))||null}),B=f.useRef(null);if(se.useImperativeHandle(t,function(){return{nativeElement:B.current||A.current}}),q){var W=null;if($){var X=!b&&!x&&O,k="".concat(u,"-clear-icon"),Y=st($)==="object"&&$!==null&&$!==void 0&&$.clearIcon?$.clearIcon:"✖";W=se.createElement("button",{type:"button",tabIndex:-1,onClick:function(fe){_==null||_(fe),F==null||F()},onMouseDown:function(fe){return fe.preventDefault()},className:re(k,Z(Z({},"".concat(k,"-hidden"),!X),"".concat(k,"-has-suffix"),!!g))},Y)}var ee="".concat(u,"-affix-wrapper"),ue=re(ee,Z(Z(Z(Z(Z({},"".concat(u,"-disabled"),b),"".concat(ee,"-disabled"),b),"".concat(ee,"-focused"),S),"".concat(ee,"-readonly"),x),"".concat(ee,"-input-with-clear-btn"),g&&$&&O),T==null?void 0:T.affixWrapper,M==null?void 0:M.affixWrapper,M==null?void 0:M.variant),ae=(g||$)&&se.createElement("span",{className:re("".concat(u,"-suffix"),M==null?void 0:M.suffix),style:P==null?void 0:P.suffix},W,g);N=se.createElement(V,je({className:ue,style:P==null?void 0:P.affixWrapper,onClick:L},z==null?void 0:z.affixWrapper,{ref:A}),d&&se.createElement("span",{className:re("".concat(u,"-prefix"),M==null?void 0:M.prefix),style:P==null?void 0:P.prefix},d),N,ae)}if(N9(e)){var oe="".concat(u,"-group"),he="".concat(oe,"-addon"),ie="".concat(oe,"-wrapper"),le=re("".concat(u,"-wrapper"),oe,T==null?void 0:T.wrapper,M==null?void 0:M.wrapper),be=re(ie,Z({},"".concat(ie,"-disabled"),b),T==null?void 0:T.group,M==null?void 0:M.groupWrapper);N=se.createElement(U,{className:be,ref:B},se.createElement(K,{className:le},h&&se.createElement(j,{className:he},h),N,p&&se.createElement(j,{className:he},p)))}return se.cloneElement(N,{className:re((r=N.props)===null||r===void 0?void 0:r.className,y)||null,style:Q(Q({},(a=N.props)===null||a===void 0?void 0:a.style),C),hidden:R})}),I9=["show"];function mT(e,t){return f.useMemo(function(){var n={};t&&(n.show=st(t)==="object"&&t.formatter?t.formatter:!!t),n=Q(Q({},n),e);var r=n,a=r.show,l=$t(r,I9);return Q(Q({},l),{},{show:!!a,showFormatter:typeof a=="function"?a:void 0,strategy:l.strategy||function(s){return s.length}})},[e,t])}var P9=["autoComplete","onChange","onFocus","onBlur","onPressEnter","onKeyDown","onKeyUp","prefixCls","disabled","htmlSize","className","maxLength","suffix","showCount","count","type","classes","classNames","styles","onCompositionStart","onCompositionEnd"],A9=f.forwardRef(function(e,t){var n=e.autoComplete,r=e.onChange,a=e.onFocus,l=e.onBlur,s=e.onPressEnter,u=e.onKeyDown,d=e.onKeyUp,g=e.prefixCls,h=g===void 0?"rc-input":g,p=e.disabled,y=e.htmlSize,C=e.className,b=e.maxLength,x=e.suffix,S=e.showCount,E=e.count,$=e.type,O=$===void 0?"text":$,_=e.classes,R=e.classNames,T=e.styles,M=e.onCompositionStart,z=e.onCompositionEnd,P=$t(e,P9),D=f.useState(!1),F=ge(D,2),G=F[0],V=F[1],U=f.useRef(!1),K=f.useRef(!1),j=f.useRef(null),A=f.useRef(null),L=function(Te){j.current&&fT(j.current,Te)},q=br(e.defaultValue,{value:e.value}),N=ge(q,2),B=N[0],W=N[1],X=B==null?"":String(B),k=f.useState(null),Y=ge(k,2),ee=Y[0],ue=Y[1],ae=mT(E,S),oe=ae.max||b,he=ae.strategy(X),ie=!!oe&&he>oe;f.useImperativeHandle(t,function(){var xe;return{focus:L,blur:function(){var Le;(Le=j.current)===null||Le===void 0||Le.blur()},setSelectionRange:function(Le,We,Ge){var at;(at=j.current)===null||at===void 0||at.setSelectionRange(Le,We,Ge)},select:function(){var Le;(Le=j.current)===null||Le===void 0||Le.select()},input:j.current,nativeElement:((xe=A.current)===null||xe===void 0?void 0:xe.nativeElement)||j.current}}),f.useEffect(function(){K.current&&(K.current=!1),V(function(xe){return xe&&p?!1:xe})},[p]);var le=function(Te,Le,We){var Ge=Le;if(!U.current&&ae.exceedFormatter&&ae.max&&ae.strategy(Le)>ae.max){if(Ge=ae.exceedFormatter(Le,{max:ae.max}),Le!==Ge){var at,ke;ue([((at=j.current)===null||at===void 0?void 0:at.selectionStart)||0,((ke=j.current)===null||ke===void 0?void 0:ke.selectionEnd)||0])}}else if(We.source==="compositionEnd")return;W(Ge),j.current&&Wd(j.current,Te,r,Ge)};f.useEffect(function(){if(ee){var xe;(xe=j.current)===null||xe===void 0||xe.setSelectionRange.apply(xe,Ne(ee))}},[ee]);var be=function(Te){le(Te,Te.target.value,{source:"change"})},Re=function(Te){U.current=!1,le(Te,Te.currentTarget.value,{source:"compositionEnd"}),z==null||z(Te)},fe=function(Te){s&&Te.key==="Enter"&&!K.current&&(K.current=!0,s(Te)),u==null||u(Te)},Se=function(Te){Te.key==="Enter"&&(K.current=!1),d==null||d(Te)},Ee=function(Te){V(!0),a==null||a(Te)},Oe=function(Te){K.current&&(K.current=!1),V(!1),l==null||l(Te)},Pe=function(Te){W(""),L(),j.current&&Wd(j.current,Te,r)},Ye=ie&&"".concat(h,"-out-of-range"),Me=function(){var Te=On(e,["prefixCls","onPressEnter","addonBefore","addonAfter","prefix","suffix","allowClear","defaultValue","showCount","count","classes","htmlSize","styles","classNames","onClear"]);return se.createElement("input",je({autoComplete:n},Te,{onChange:be,onFocus:Ee,onBlur:Oe,onKeyDown:fe,onKeyUp:Se,className:re(h,Z({},"".concat(h,"-disabled"),p),R==null?void 0:R.input),style:T==null?void 0:T.input,ref:j,size:y,type:O,onCompositionStart:function(We){U.current=!0,M==null||M(We)},onCompositionEnd:Re}))},De=function(){var Te=Number(oe)>0;if(x||ae.show){var Le=ae.showFormatter?ae.showFormatter({value:X,count:he,maxLength:oe}):"".concat(he).concat(Te?" / ".concat(oe):"");return se.createElement(se.Fragment,null,ae.show&&se.createElement("span",{className:re("".concat(h,"-show-count-suffix"),Z({},"".concat(h,"-show-count-has-suffix"),!!x),R==null?void 0:R.count),style:Q({},T==null?void 0:T.count)},Le),x)}return null};return se.createElement(dT,je({},P,{prefixCls:h,className:re(C,Ye),handleReset:Pe,value:X,focused:G,triggerFocus:L,suffix:De(),disabled:p,classes:_,classNames:R,styles:T,ref:A}),Me())});const gT=e=>{let t;return typeof e=="object"&&(e!=null&&e.clearIcon)?t=e:e&&(t={clearIcon:se.createElement(ss,null)}),t};function hT(e,t){const n=f.useRef([]),r=()=>{n.current.push(setTimeout(()=>{var a,l,s,u;!((a=e.current)===null||a===void 0)&&a.input&&((l=e.current)===null||l===void 0?void 0:l.input.getAttribute("type"))==="password"&&(!((s=e.current)===null||s===void 0)&&s.input.hasAttribute("value"))&&((u=e.current)===null||u===void 0||u.input.removeAttribute("value"))}))};return f.useEffect(()=>(t&&r(),()=>n.current.forEach(a=>{a&&clearTimeout(a)})),[]),r}function z9(e){return!!(e.prefix||e.suffix||e.allowClear||e.showCount)}var D9=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const Lm=f.forwardRef((e,t)=>{const{prefixCls:n,bordered:r=!0,status:a,size:l,disabled:s,onBlur:u,onFocus:d,suffix:g,allowClear:h,addonAfter:p,addonBefore:y,className:C,style:b,styles:x,rootClassName:S,onChange:E,classNames:$,variant:O}=e,_=D9(e,["prefixCls","bordered","status","size","disabled","onBlur","onFocus","suffix","allowClear","addonAfter","addonBefore","className","style","styles","rootClassName","onChange","classNames","variant"]),{getPrefixCls:R,direction:T,allowClear:M,autoComplete:z,className:P,style:D,classNames:F,styles:G}=Or("input"),V=R("input",n),U=f.useRef(null),K=ia(V),[j,A,L]=lT(V,S),[q]=sT(V,K),{compactSize:N,compactItemClassnames:B}=Ru(V,T),W=Lo(Oe=>{var Pe;return(Pe=l??N)!==null&&Pe!==void 0?Pe:Oe}),X=se.useContext(Ro),k=s??X,{status:Y,hasFeedback:ee,feedbackIcon:ue}=f.useContext(ra),ae=Nm(Y,a),oe=z9(e)||!!ee;f.useRef(oe);const he=hT(U,!0),ie=Oe=>{he(),u==null||u(Oe)},le=Oe=>{he(),d==null||d(Oe)},be=Oe=>{he(),E==null||E(Oe)},Re=(ee||g)&&se.createElement(se.Fragment,null,g,ee&&ue),fe=gT(h??M),[Se,Ee]=Ty("input",O,r);return j(q(se.createElement(A9,Object.assign({ref:Ha(t,U),prefixCls:V,autoComplete:z},_,{disabled:k,onBlur:ie,onFocus:le,style:Object.assign(Object.assign({},D),b),styles:Object.assign(Object.assign({},G),x),suffix:Re,allowClear:fe,className:re(C,S,L,K,B,P),onChange:be,addonBefore:y&&se.createElement(ns,{form:!0,space:!0},y),addonAfter:p&&se.createElement(ns,{form:!0,space:!0},p),classNames:Object.assign(Object.assign(Object.assign({},$),F),{input:re({[`${V}-sm`]:W==="small",[`${V}-lg`]:W==="large",[`${V}-rtl`]:T==="rtl"},$==null?void 0:$.input,F.input,A),variant:re({[`${V}-${Se}`]:Ee},Fd(V,ae)),affixWrapper:re({[`${V}-affix-wrapper-sm`]:W==="small",[`${V}-affix-wrapper-lg`]:W==="large",[`${V}-affix-wrapper-rtl`]:T==="rtl"},A),wrapper:re({[`${V}-group-rtl`]:T==="rtl"},A),groupWrapper:re({[`${V}-group-wrapper-sm`]:W==="small",[`${V}-group-wrapper-lg`]:W==="large",[`${V}-group-wrapper-rtl`]:T==="rtl",[`${V}-group-wrapper-${Se}`]:Ee},Fd(`${V}-group-wrapper`,ae,ee),A)})}))))});var dE=f.createContext(null),pT=f.createContext({}),L9=["prefixCls","className","containerRef"],B9=function(t){var n=t.prefixCls,r=t.className,a=t.containerRef,l=$t(t,L9),s=f.useContext(pT),u=s.panel,d=Io(u,a);return f.createElement("div",je({className:re("".concat(n,"-content"),r),role:"dialog",ref:d},ta(t,{aria:!0}),{"aria-modal":"true"},l))};function mE(e){return typeof e=="string"&&String(Number(e))===e?(mr(!1,"Invalid value type of `width` or `height` which should be number type instead."),Number(e)):e}var gE={width:0,height:0,overflow:"hidden",outline:"none",position:"absolute"};function H9(e,t){var n,r,a,l=e.prefixCls,s=e.open,u=e.placement,d=e.inline,g=e.push,h=e.forceRender,p=e.autoFocus,y=e.keyboard,C=e.classNames,b=e.rootClassName,x=e.rootStyle,S=e.zIndex,E=e.className,$=e.id,O=e.style,_=e.motion,R=e.width,T=e.height,M=e.children,z=e.mask,P=e.maskClosable,D=e.maskMotion,F=e.maskClassName,G=e.maskStyle,V=e.afterOpenChange,U=e.onClose,K=e.onMouseEnter,j=e.onMouseOver,A=e.onMouseLeave,L=e.onClick,q=e.onKeyDown,N=e.onKeyUp,B=e.styles,W=e.drawerRender,X=f.useRef(),k=f.useRef(),Y=f.useRef();f.useImperativeHandle(t,function(){return X.current});var ee=function(De){var xe=De.keyCode,Te=De.shiftKey;switch(xe){case lt.TAB:{if(xe===lt.TAB){if(!Te&&document.activeElement===Y.current){var Le;(Le=k.current)===null||Le===void 0||Le.focus({preventScroll:!0})}else if(Te&&document.activeElement===k.current){var We;(We=Y.current)===null||We===void 0||We.focus({preventScroll:!0})}}break}case lt.ESC:{U&&y&&(De.stopPropagation(),U(De));break}}};f.useEffect(function(){if(s&&p){var Me;(Me=X.current)===null||Me===void 0||Me.focus({preventScroll:!0})}},[s]);var ue=f.useState(!1),ae=ge(ue,2),oe=ae[0],he=ae[1],ie=f.useContext(dE),le;typeof g=="boolean"?le=g?{}:{distance:0}:le=g||{};var be=(n=(r=(a=le)===null||a===void 0?void 0:a.distance)!==null&&r!==void 0?r:ie==null?void 0:ie.pushDistance)!==null&&n!==void 0?n:180,Re=f.useMemo(function(){return{pushDistance:be,push:function(){he(!0)},pull:function(){he(!1)}}},[be]);f.useEffect(function(){if(s){var Me;ie==null||(Me=ie.push)===null||Me===void 0||Me.call(ie)}else{var De;ie==null||(De=ie.pull)===null||De===void 0||De.call(ie)}},[s]),f.useEffect(function(){return function(){var Me;ie==null||(Me=ie.pull)===null||Me===void 0||Me.call(ie)}},[]);var fe=z&&f.createElement($a,je({key:"mask"},D,{visible:s}),function(Me,De){var xe=Me.className,Te=Me.style;return f.createElement("div",{className:re("".concat(l,"-mask"),xe,C==null?void 0:C.mask,F),style:Q(Q(Q({},Te),G),B==null?void 0:B.mask),onClick:P&&s?U:void 0,ref:De})}),Se=typeof _=="function"?_(u):_,Ee={};if(oe&&be)switch(u){case"top":Ee.transform="translateY(".concat(be,"px)");break;case"bottom":Ee.transform="translateY(".concat(-be,"px)");break;case"left":Ee.transform="translateX(".concat(be,"px)");break;default:Ee.transform="translateX(".concat(-be,"px)");break}u==="left"||u==="right"?Ee.width=mE(R):Ee.height=mE(T);var Oe={onMouseEnter:K,onMouseOver:j,onMouseLeave:A,onClick:L,onKeyDown:q,onKeyUp:N},Pe=f.createElement($a,je({key:"panel"},Se,{visible:s,forceRender:h,onVisibleChanged:function(De){V==null||V(De)},removeOnLeave:!1,leavedClassName:"".concat(l,"-content-wrapper-hidden")}),function(Me,De){var xe=Me.className,Te=Me.style,Le=f.createElement(B9,je({id:$,containerRef:De,prefixCls:l,className:re(E,C==null?void 0:C.content),style:Q(Q({},O),B==null?void 0:B.content)},ta(e,{aria:!0}),Oe),M);return f.createElement("div",je({className:re("".concat(l,"-content-wrapper"),C==null?void 0:C.wrapper,xe),style:Q(Q(Q({},Ee),Te),B==null?void 0:B.wrapper)},ta(e,{data:!0})),W?W(Le):Le)}),Ye=Q({},x);return S&&(Ye.zIndex=S),f.createElement(dE.Provider,{value:Re},f.createElement("div",{className:re(l,"".concat(l,"-").concat(u),b,Z(Z({},"".concat(l,"-open"),s),"".concat(l,"-inline"),d)),style:Ye,tabIndex:-1,ref:X,onKeyDown:ee},fe,f.createElement("div",{tabIndex:0,ref:k,style:gE,"aria-hidden":"true","data-sentinel":"start"}),Pe,f.createElement("div",{tabIndex:0,ref:Y,style:gE,"aria-hidden":"true","data-sentinel":"end"})))}var F9=f.forwardRef(H9),V9=function(t){var n=t.open,r=n===void 0?!1:n,a=t.prefixCls,l=a===void 0?"rc-drawer":a,s=t.placement,u=s===void 0?"right":s,d=t.autoFocus,g=d===void 0?!0:d,h=t.keyboard,p=h===void 0?!0:h,y=t.width,C=y===void 0?378:y,b=t.mask,x=b===void 0?!0:b,S=t.maskClosable,E=S===void 0?!0:S,$=t.getContainer,O=t.forceRender,_=t.afterOpenChange,R=t.destroyOnClose,T=t.onMouseEnter,M=t.onMouseOver,z=t.onMouseLeave,P=t.onClick,D=t.onKeyDown,F=t.onKeyUp,G=t.panelRef,V=f.useState(!1),U=ge(V,2),K=U[0],j=U[1],A=f.useState(!1),L=ge(A,2),q=L[0],N=L[1];Gt(function(){N(!0)},[]);var B=q?r:!1,W=f.useRef(),X=f.useRef();Gt(function(){B&&(X.current=document.activeElement)},[B]);var k=function(oe){var he;if(j(oe),_==null||_(oe),!oe&&X.current&&!((he=W.current)!==null&&he!==void 0&&he.contains(X.current))){var ie;(ie=X.current)===null||ie===void 0||ie.focus({preventScroll:!0})}},Y=f.useMemo(function(){return{panel:G}},[G]);if(!O&&!K&&!B&&R)return null;var ee={onMouseEnter:T,onMouseOver:M,onMouseLeave:z,onClick:P,onKeyDown:D,onKeyUp:F},ue=Q(Q({},t),{},{open:B,prefixCls:l,placement:u,autoFocus:g,keyboard:p,width:C,mask:x,maskClosable:E,inline:$===!1,afterOpenChange:k,ref:W},ee);return f.createElement(pT.Provider,{value:Y},f.createElement(Rm,{open:B||O||K,autoDestroy:!1,getContainer:$,autoLock:x&&(B||K)},f.createElement(F9,ue)))};const vT=e=>{var t,n;const{prefixCls:r,title:a,footer:l,extra:s,loading:u,onClose:d,headerStyle:g,bodyStyle:h,footerStyle:p,children:y,classNames:C,styles:b}=e,x=Or("drawer"),S=f.useCallback(R=>f.createElement("button",{type:"button",onClick:d,"aria-label":"Close",className:`${r}-close`},R),[d]),[E,$]=xy(rs(e),rs(x),{closable:!0,closeIconRender:S}),O=f.useMemo(()=>{var R,T;return!a&&!E?null:f.createElement("div",{style:Object.assign(Object.assign(Object.assign({},(R=x.styles)===null||R===void 0?void 0:R.header),g),b==null?void 0:b.header),className:re(`${r}-header`,{[`${r}-header-close-only`]:E&&!a&&!s},(T=x.classNames)===null||T===void 0?void 0:T.header,C==null?void 0:C.header)},f.createElement("div",{className:`${r}-header-title`},$,a&&f.createElement("div",{className:`${r}-title`},a)),s&&f.createElement("div",{className:`${r}-extra`},s))},[E,$,s,g,r,a]),_=f.useMemo(()=>{var R,T;if(!l)return null;const M=`${r}-footer`;return f.createElement("div",{className:re(M,(R=x.classNames)===null||R===void 0?void 0:R.footer,C==null?void 0:C.footer),style:Object.assign(Object.assign(Object.assign({},(T=x.styles)===null||T===void 0?void 0:T.footer),p),b==null?void 0:b.footer)},l)},[l,p,r]);return f.createElement(f.Fragment,null,O,f.createElement("div",{className:re(`${r}-body`,C==null?void 0:C.body,(t=x.classNames)===null||t===void 0?void 0:t.body),style:Object.assign(Object.assign(Object.assign({},(n=x.styles)===null||n===void 0?void 0:n.body),h),b==null?void 0:b.body)},u?f.createElement(Bo,{active:!0,title:!1,paragraph:{rows:5},className:`${r}-body-skeleton`}):y),_)},U9=e=>{const t="100%";return{left:`translateX(-${t})`,right:`translateX(${t})`,top:`translateY(-${t})`,bottom:`translateY(${t})`}[e]},yT=(e,t)=>({"&-enter, &-appear":Object.assign(Object.assign({},e),{"&-active":t}),"&-leave":Object.assign(Object.assign({},t),{"&-active":e})}),bT=(e,t)=>Object.assign({"&-enter, &-appear, &-leave":{"&-start":{transition:"none"},"&-active":{transition:`all ${t}`}}},yT({opacity:e},{opacity:1})),k9=(e,t)=>[bT(.7,t),yT({transform:U9(e)},{transform:"none"})],q9=e=>{const{componentCls:t,motionDurationSlow:n}=e;return{[t]:{[`${t}-mask-motion`]:bT(0,n),[`${t}-panel-motion`]:["left","right","top","bottom"].reduce((r,a)=>Object.assign(Object.assign({},r),{[`&-${a}`]:k9(a,n)}),{})}}},W9=e=>{const{borderRadiusSM:t,componentCls:n,zIndexPopup:r,colorBgMask:a,colorBgElevated:l,motionDurationSlow:s,motionDurationMid:u,paddingXS:d,padding:g,paddingLG:h,fontSizeLG:p,lineHeightLG:y,lineWidth:C,lineType:b,colorSplit:x,marginXS:S,colorIcon:E,colorIconHover:$,colorBgTextHover:O,colorBgTextActive:_,colorText:R,fontWeightStrong:T,footerPaddingBlock:M,footerPaddingInline:z,calc:P}=e,D=`${n}-content-wrapper`;return{[n]:{position:"fixed",inset:0,zIndex:r,pointerEvents:"none",color:R,"&-pure":{position:"relative",background:l,display:"flex",flexDirection:"column",[`&${n}-left`]:{boxShadow:e.boxShadowDrawerLeft},[`&${n}-right`]:{boxShadow:e.boxShadowDrawerRight},[`&${n}-top`]:{boxShadow:e.boxShadowDrawerUp},[`&${n}-bottom`]:{boxShadow:e.boxShadowDrawerDown}},"&-inline":{position:"absolute"},[`${n}-mask`]:{position:"absolute",inset:0,zIndex:r,background:a,pointerEvents:"auto"},[D]:{position:"absolute",zIndex:r,maxWidth:"100vw",transition:`all ${s}`,"&-hidden":{display:"none"}},[`&-left > ${D}`]:{top:0,bottom:0,left:{_skip_check_:!0,value:0},boxShadow:e.boxShadowDrawerLeft},[`&-right > ${D}`]:{top:0,right:{_skip_check_:!0,value:0},bottom:0,boxShadow:e.boxShadowDrawerRight},[`&-top > ${D}`]:{top:0,insetInline:0,boxShadow:e.boxShadowDrawerUp},[`&-bottom > ${D}`]:{bottom:0,insetInline:0,boxShadow:e.boxShadowDrawerDown},[`${n}-content`]:{display:"flex",flexDirection:"column",width:"100%",height:"100%",overflow:"auto",background:l,pointerEvents:"auto"},[`${n}-header`]:{display:"flex",flex:0,alignItems:"center",padding:`${ve(g)} ${ve(h)}`,fontSize:p,lineHeight:y,borderBottom:`${ve(C)} ${b} ${x}`,"&-title":{display:"flex",flex:1,alignItems:"center",minWidth:0,minHeight:0}},[`${n}-extra`]:{flex:"none"},[`${n}-close`]:Object.assign({display:"inline-flex",width:P(p).add(d).equal(),height:P(p).add(d).equal(),borderRadius:t,justifyContent:"center",alignItems:"center",marginInlineEnd:S,color:E,fontWeight:T,fontSize:p,fontStyle:"normal",lineHeight:1,textAlign:"center",textTransform:"none",textDecoration:"none",background:"transparent",border:0,cursor:"pointer",transition:`all ${u}`,textRendering:"auto","&:hover":{color:$,backgroundColor:O,textDecoration:"none"},"&:active":{backgroundColor:_}},gs(e)),[`${n}-title`]:{flex:1,margin:0,fontWeight:e.fontWeightStrong,fontSize:p,lineHeight:y},[`${n}-body`]:{flex:1,minWidth:0,minHeight:0,padding:h,overflow:"auto",[`${n}-body-skeleton`]:{width:"100%",height:"100%",display:"flex",justifyContent:"center"}},[`${n}-footer`]:{flexShrink:0,padding:`${ve(M)} ${ve(z)}`,borderTop:`${ve(C)} ${b} ${x}`},"&-rtl":{direction:"rtl"}}}},G9=e=>({zIndexPopup:e.zIndexPopupBase,footerPaddingBlock:e.paddingXS,footerPaddingInline:e.padding}),ST=xn("Drawer",e=>{const t=Kt(e,{});return[W9(t),q9(t)]},G9);var CT=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const K9={distance:180},xT=e=>{const{rootClassName:t,width:n,height:r,size:a="default",mask:l=!0,push:s=K9,open:u,afterOpenChange:d,onClose:g,prefixCls:h,getContainer:p,style:y,className:C,visible:b,afterVisibleChange:x,maskStyle:S,drawerStyle:E,contentWrapperStyle:$}=e,O=CT(e,["rootClassName","width","height","size","mask","push","open","afterOpenChange","onClose","prefixCls","getContainer","style","className","visible","afterVisibleChange","maskStyle","drawerStyle","contentWrapperStyle"]),{getPopupContainer:_,getPrefixCls:R,direction:T,className:M,style:z,classNames:P,styles:D}=Or("drawer"),F=R("drawer",h),[G,V,U]=ST(F),K=p===void 0&&_?()=>_(document.body):p,j=re({"no-mask":!l,[`${F}-rtl`]:T==="rtl"},t,V,U),A=f.useMemo(()=>n??(a==="large"?736:378),[n,a]),L=f.useMemo(()=>r??(a==="large"?736:378),[r,a]),q={motionName:_o(F,"mask-motion"),motionAppear:!0,motionEnter:!0,motionLeave:!0,motionDeadline:500},N=ee=>({motionName:_o(F,`panel-motion-${ee}`),motionAppear:!0,motionEnter:!0,motionLeave:!0,motionDeadline:500}),B=AR(),[W,X]=$u("Drawer",O.zIndex),{classNames:k={},styles:Y={}}=O;return G(f.createElement(ns,{form:!0,space:!0},f.createElement(gm.Provider,{value:X},f.createElement(V9,Object.assign({prefixCls:F,onClose:g,maskMotion:q,motion:N},O,{classNames:{mask:re(k.mask,P.mask),content:re(k.content,P.content),wrapper:re(k.wrapper,P.wrapper)},styles:{mask:Object.assign(Object.assign(Object.assign({},Y.mask),S),D.mask),content:Object.assign(Object.assign(Object.assign({},Y.content),E),D.content),wrapper:Object.assign(Object.assign(Object.assign({},Y.wrapper),$),D.wrapper)},open:u??b,mask:l,push:s,width:A,height:L,style:Object.assign(Object.assign({},z),y),className:re(M,C),rootClassName:j,getContainer:K,afterOpenChange:d??x,panelRef:B,zIndex:W}),f.createElement(vT,Object.assign({prefixCls:F},O,{onClose:g}))))))},X9=e=>{const{prefixCls:t,style:n,className:r,placement:a="right"}=e,l=CT(e,["prefixCls","style","className","placement"]),{getPrefixCls:s}=f.useContext(dt),u=s("drawer",t),[d,g,h]=ST(u),p=re(u,`${u}-pure`,`${u}-${a}`,g,h,r);return d(f.createElement("div",{className:p,style:n},f.createElement(vT,Object.assign({prefixCls:u},l))))};xT._InternalPanelDoNotUseOrYouWillBeFired=X9;function hE(e){return["small","middle","large"].includes(e)}function vX(e){return e?typeof e=="number"&&!Number.isNaN(e):!1}const wT=["wrap","nowrap","wrap-reverse"],ET=["flex-start","flex-end","start","end","center","space-between","space-around","space-evenly","stretch","normal","left","right"],$T=["center","start","end","flex-start","flex-end","self-start","self-end","baseline","normal","stretch"],Y9=(e,t)=>{const n=t.wrap===!0?"wrap":t.wrap;return{[`${e}-wrap-${n}`]:n&&wT.includes(n)}},Q9=(e,t)=>{const n={};return $T.forEach(r=>{n[`${e}-align-${r}`]=t.align===r}),n[`${e}-align-stretch`]=!t.align&&!!t.vertical,n},Z9=(e,t)=>{const n={};return ET.forEach(r=>{n[`${e}-justify-${r}`]=t.justify===r}),n};function J9(e,t){return re(Object.assign(Object.assign(Object.assign({},Y9(e,t)),Q9(e,t)),Z9(e,t)))}const eF=e=>{const{componentCls:t}=e;return{[t]:{display:"flex",margin:0,padding:0,"&-vertical":{flexDirection:"column"},"&-rtl":{direction:"rtl"},"&:empty":{display:"none"}}}},tF=e=>{const{componentCls:t}=e;return{[t]:{"&-gap-small":{gap:e.flexGapSM},"&-gap-middle":{gap:e.flexGap},"&-gap-large":{gap:e.flexGapLG}}}},nF=e=>{const{componentCls:t}=e,n={};return wT.forEach(r=>{n[`${t}-wrap-${r}`]={flexWrap:r}}),n},rF=e=>{const{componentCls:t}=e,n={};return $T.forEach(r=>{n[`${t}-align-${r}`]={alignItems:r}}),n},aF=e=>{const{componentCls:t}=e,n={};return ET.forEach(r=>{n[`${t}-justify-${r}`]={justifyContent:r}}),n},oF=()=>({}),iF=xn("Flex",e=>{const{paddingXS:t,padding:n,paddingLG:r}=e,a=Kt(e,{flexGapSM:t,flexGap:n,flexGapLG:r});return[eF(a),tF(a),nF(a),rF(a),aF(a)]},oF,{resetStyle:!1});var lF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const Mo=se.forwardRef((e,t)=>{const{prefixCls:n,rootClassName:r,className:a,style:l,flex:s,gap:u,children:d,vertical:g=!1,component:h="div"}=e,p=lF(e,["prefixCls","rootClassName","className","style","flex","gap","children","vertical","component"]),{flex:y,direction:C,getPrefixCls:b}=se.useContext(dt),x=b("flex",n),[S,E,$]=iF(x),O=g??(y==null?void 0:y.vertical),_=re(a,r,y==null?void 0:y.className,x,E,$,J9(x,e),{[`${x}-rtl`]:C==="rtl",[`${x}-gap-${u}`]:hE(u),[`${x}-vertical`]:O}),R=Object.assign(Object.assign({},y==null?void 0:y.style),l);return s&&(R.flex=s),u&&!hE(u)&&(R.gap=u),S(se.createElement(h,Object.assign({ref:t,className:_,style:R},On(p,["justify","wrap","align"])),d))});function Gd(e){const[t,n]=f.useState(e);return f.useEffect(()=>{const r=setTimeout(()=>{n(e)},e.length?0:10);return()=>{clearTimeout(r)}},[e]),t}const sF=e=>{const{componentCls:t}=e,n=`${t}-show-help`,r=`${t}-show-help-item`;return{[n]:{transition:`opacity ${e.motionDurationFast} ${e.motionEaseInOut}`,"&-appear, &-enter":{opacity:0,"&-active":{opacity:1}},"&-leave":{opacity:1,"&-active":{opacity:0}},[r]:{overflow:"hidden",transition:`height ${e.motionDurationFast} ${e.motionEaseInOut},
                     opacity ${e.motionDurationFast} ${e.motionEaseInOut},
                     transform ${e.motionDurationFast} ${e.motionEaseInOut} !important`,[`&${r}-appear, &${r}-enter`]:{transform:"translateY(-5px)",opacity:0,"&-active":{transform:"translateY(0)",opacity:1}},[`&${r}-leave-active`]:{transform:"translateY(-5px)"}}}}},cF=e=>({legend:{display:"block",width:"100%",marginBottom:e.marginLG,padding:0,color:e.colorTextDescription,fontSize:e.fontSizeLG,lineHeight:"inherit",border:0,borderBottom:`${ve(e.lineWidth)} ${e.lineType} ${e.colorBorder}`},'input[type="search"]':{boxSizing:"border-box"},'input[type="radio"], input[type="checkbox"]':{lineHeight:"normal"},'input[type="file"]':{display:"block"},'input[type="range"]':{display:"block",width:"100%"},"select[multiple], select[size]":{height:"auto"},"input[type='file']:focus,\n  input[type='radio']:focus,\n  input[type='checkbox']:focus":{outline:0,boxShadow:`0 0 0 ${ve(e.controlOutlineWidth)} ${e.controlOutline}`},output:{display:"block",paddingTop:15,color:e.colorText,fontSize:e.fontSize,lineHeight:e.lineHeight}}),pE=(e,t)=>{const{formItemCls:n}=e;return{[n]:{[`${n}-label > label`]:{height:t},[`${n}-control-input`]:{minHeight:t}}}},uF=e=>{const{componentCls:t}=e;return{[e.componentCls]:Object.assign(Object.assign(Object.assign({},Rr(e)),cF(e)),{[`${t}-text`]:{display:"inline-block",paddingInlineEnd:e.paddingSM},"&-small":Object.assign({},pE(e,e.controlHeightSM)),"&-large":Object.assign({},pE(e,e.controlHeightLG))})}},fF=e=>{const{formItemCls:t,iconCls:n,rootPrefixCls:r,antCls:a,labelRequiredMarkColor:l,labelColor:s,labelFontSize:u,labelHeight:d,labelColonMarginInlineStart:g,labelColonMarginInlineEnd:h,itemMarginBottom:p}=e;return{[t]:Object.assign(Object.assign({},Rr(e)),{marginBottom:p,verticalAlign:"top","&-with-help":{transition:"none"},[`&-hidden,
        &-hidden${a}-row`]:{display:"none"},"&-has-warning":{[`${t}-split`]:{color:e.colorError}},"&-has-error":{[`${t}-split`]:{color:e.colorWarning}},[`${t}-label`]:{flexGrow:0,overflow:"hidden",whiteSpace:"nowrap",textAlign:"end",verticalAlign:"middle","&-left":{textAlign:"start"},"&-wrap":{overflow:"unset",lineHeight:e.lineHeight,whiteSpace:"unset"},"> label":{position:"relative",display:"inline-flex",alignItems:"center",maxWidth:"100%",height:d,color:s,fontSize:u,[`> ${n}`]:{fontSize:e.fontSize,verticalAlign:"top"},[`&${t}-required`]:{"&::before":{display:"inline-block",marginInlineEnd:e.marginXXS,color:l,fontSize:e.fontSize,fontFamily:"SimSun, sans-serif",lineHeight:1,content:'"*"'},[`&${t}-required-mark-hidden, &${t}-required-mark-optional`]:{"&::before":{display:"none"}}},[`${t}-optional`]:{display:"inline-block",marginInlineStart:e.marginXXS,color:e.colorTextDescription,[`&${t}-required-mark-hidden`]:{display:"none"}},[`${t}-tooltip`]:{color:e.colorTextDescription,cursor:"help",writingMode:"horizontal-tb",marginInlineStart:e.marginXXS},"&::after":{content:'":"',position:"relative",marginBlock:0,marginInlineStart:g,marginInlineEnd:h},[`&${t}-no-colon::after`]:{content:'"\\a0"'}}},[`${t}-control`]:{"--ant-display":"flex",flexDirection:"column",flexGrow:1,[`&:first-child:not([class^="'${r}-col-'"]):not([class*="' ${r}-col-'"])`]:{width:"100%"},"&-input":{position:"relative",display:"flex",alignItems:"center",minHeight:e.controlHeight,"&-content":{flex:"auto",maxWidth:"100%"}}},[t]:{"&-additional":{display:"flex",flexDirection:"column"},"&-explain, &-extra":{clear:"both",color:e.colorTextDescription,fontSize:e.fontSize,lineHeight:e.lineHeight},"&-explain-connected":{width:"100%"},"&-extra":{minHeight:e.controlHeightSM,transition:`color ${e.motionDurationMid} ${e.motionEaseOut}`},"&-explain":{"&-error":{color:e.colorError},"&-warning":{color:e.colorWarning}}},[`&-with-help ${t}-explain`]:{height:"auto",opacity:1},[`${t}-feedback-icon`]:{fontSize:e.fontSize,textAlign:"center",visibility:"visible",animationName:my,animationDuration:e.motionDurationMid,animationTimingFunction:e.motionEaseOutBack,pointerEvents:"none","&-success":{color:e.colorSuccess},"&-error":{color:e.colorError},"&-warning":{color:e.colorWarning},"&-validating":{color:e.colorPrimary}}})}},vE=(e,t)=>{const{formItemCls:n}=e;return{[`${t}-horizontal`]:{[`${n}-label`]:{flexGrow:0},[`${n}-control`]:{flex:"1 1 0",minWidth:0},[`${n}-label[class$='-24'], ${n}-label[class*='-24 ']`]:{[`& + ${n}-control`]:{minWidth:"unset"}}}}},dF=e=>{const{componentCls:t,formItemCls:n,inlineItemMarginBottom:r}=e;return{[`${t}-inline`]:{display:"flex",flexWrap:"wrap",[n]:{flex:"none",marginInlineEnd:e.margin,marginBottom:r,"&-row":{flexWrap:"nowrap"},[`> ${n}-label,
        > ${n}-control`]:{display:"inline-block",verticalAlign:"top"},[`> ${n}-label`]:{flex:"none"},[`${t}-text`]:{display:"inline-block"},[`${n}-has-feedback`]:{display:"inline-block"}}}}},ja=e=>({padding:e.verticalLabelPadding,margin:e.verticalLabelMargin,whiteSpace:"initial",textAlign:"start","> label":{margin:0,"&::after":{visibility:"hidden"}}}),OT=e=>{const{componentCls:t,formItemCls:n,rootPrefixCls:r}=e;return{[`${n} ${n}-label`]:ja(e),[`${t}:not(${t}-inline)`]:{[n]:{flexWrap:"wrap",[`${n}-label, ${n}-control`]:{[`&:not([class*=" ${r}-col-xs"])`]:{flex:"0 0 100%",maxWidth:"100%"}}}}}},mF=e=>{const{componentCls:t,formItemCls:n,antCls:r}=e;return{[`${t}-vertical`]:{[`${n}:not(${n}-horizontal)`]:{[`${n}-row`]:{flexDirection:"column"},[`${n}-label > label`]:{height:"auto"},[`${n}-control`]:{width:"100%"},[`${n}-label,
        ${r}-col-24${n}-label,
        ${r}-col-xl-24${n}-label`]:ja(e)}},[`@media (max-width: ${ve(e.screenXSMax)})`]:[OT(e),{[t]:{[`${n}:not(${n}-horizontal)`]:{[`${r}-col-xs-24${n}-label`]:ja(e)}}}],[`@media (max-width: ${ve(e.screenSMMax)})`]:{[t]:{[`${n}:not(${n}-horizontal)`]:{[`${r}-col-sm-24${n}-label`]:ja(e)}}},[`@media (max-width: ${ve(e.screenMDMax)})`]:{[t]:{[`${n}:not(${n}-horizontal)`]:{[`${r}-col-md-24${n}-label`]:ja(e)}}},[`@media (max-width: ${ve(e.screenLGMax)})`]:{[t]:{[`${n}:not(${n}-horizontal)`]:{[`${r}-col-lg-24${n}-label`]:ja(e)}}}}},gF=e=>{const{formItemCls:t,antCls:n}=e;return{[`${t}-vertical`]:{[`${t}-row`]:{flexDirection:"column"},[`${t}-label > label`]:{height:"auto"},[`${t}-control`]:{width:"100%"}},[`${t}-vertical ${t}-label,
      ${n}-col-24${t}-label,
      ${n}-col-xl-24${t}-label`]:ja(e),[`@media (max-width: ${ve(e.screenXSMax)})`]:[OT(e),{[t]:{[`${n}-col-xs-24${t}-label`]:ja(e)}}],[`@media (max-width: ${ve(e.screenSMMax)})`]:{[t]:{[`${n}-col-sm-24${t}-label`]:ja(e)}},[`@media (max-width: ${ve(e.screenMDMax)})`]:{[t]:{[`${n}-col-md-24${t}-label`]:ja(e)}},[`@media (max-width: ${ve(e.screenLGMax)})`]:{[t]:{[`${n}-col-lg-24${t}-label`]:ja(e)}}}},hF=e=>({labelRequiredMarkColor:e.colorError,labelColor:e.colorTextHeading,labelFontSize:e.fontSize,labelHeight:e.controlHeight,labelColonMarginInlineStart:e.marginXXS/2,labelColonMarginInlineEnd:e.marginXS,itemMarginBottom:e.marginLG,verticalLabelPadding:`0 0 ${e.paddingXS}px`,verticalLabelMargin:0,inlineItemMarginBottom:0}),RT=(e,t)=>Kt(e,{formItemCls:`${e.componentCls}-item`,rootPrefixCls:t}),Hy=xn("Form",(e,t)=>{let{rootPrefixCls:n}=t;const r=RT(e,n);return[uF(r),fF(r),sF(r),vE(r,r.componentCls),vE(r,r.formItemCls),dF(r),mF(r),gF(r),dy(r),my]},hF,{order:-1e3}),yE=[];function Fp(e,t,n){let r=arguments.length>3&&arguments[3]!==void 0?arguments[3]:0;return{key:typeof e=="string"?e:`${t}-${r}`,error:e,errorStatus:n}}const _T=e=>{let{help:t,helpStatus:n,errors:r=yE,warnings:a=yE,className:l,fieldId:s,onVisibleChanged:u}=e;const{prefixCls:d}=f.useContext(Cy),g=`${d}-item-explain`,h=ia(d),[p,y,C]=Hy(d,h),b=f.useMemo(()=>Ld(d),[d]),x=Gd(r),S=Gd(a),E=f.useMemo(()=>t!=null?[Fp(t,"help",n)]:[].concat(Ne(x.map((_,R)=>Fp(_,"error","error",R))),Ne(S.map((_,R)=>Fp(_,"warning","warning",R)))),[t,n,x,S]),$=f.useMemo(()=>{const _={};return E.forEach(R=>{let{key:T}=R;_[T]=(_[T]||0)+1}),E.map((R,T)=>Object.assign(Object.assign({},R),{key:_[R.key]>1?`${R.key}-fallback-${T}`:R.key}))},[E]),O={};return s&&(O.id=`${s}_help`),p(f.createElement($a,{motionDeadline:b.motionDeadline,motionName:`${d}-show-help`,visible:!!$.length,onVisibleChanged:u},_=>{const{className:R,style:T}=_;return f.createElement("div",Object.assign({},O,{className:re(g,R,C,h,l,y),style:T}),f.createElement(q2,Object.assign({keys:$},Ld(d),{motionName:`${d}-show-help-item`,component:!1}),M=>{const{key:z,error:P,errorStatus:D,className:F,style:G}=M;return f.createElement("div",{key:z,className:re(F,{[`${g}-${D}`]:D}),style:G},P)}))}))},pF=["parentNode"],vF="form_item";function Yc(e){return e===void 0||e===!1?[]:Array.isArray(e)?e:[e]}function TT(e,t){if(!e.length)return;const n=e.join("_");return t?`${t}_${n}`:pF.includes(n)?`${vF}_${n}`:n}function MT(e,t,n,r,a,l){let s=r;return l!==void 0?s=l:n.validating?s="validating":e.length?s="error":t.length?s="warning":(n.touched||a&&n.validated)&&(s="success"),s}var yF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function bE(e){return Yc(e).join("_")}function SE(e,t){const n=t.getFieldInstance(e),r=WO(n);if(r)return r;const a=TT(Yc(e),t.__INTERNAL__.name);if(a)return document.getElementById(a)}function NT(e){const[t]=Sy(),n=f.useRef({}),r=f.useMemo(()=>e??Object.assign(Object.assign({},t),{__INTERNAL__:{itemRef:a=>l=>{const s=bE(a);l?n.current[s]=l:delete n.current[s]}},scrollToField:function(a){let l=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};const{focus:s}=l,u=yF(l,["focus"]),d=SE(a,r);d&&(_6(d,Object.assign({scrollMode:"if-needed",block:"nearest"},u)),s&&r.focusField(a))},focusField:a=>{var l,s;const u=r.getFieldInstance(a);typeof(u==null?void 0:u.focus)=="function"?u.focus():(s=(l=SE(a,r))===null||l===void 0?void 0:l.focus)===null||s===void 0||s.call(l)},getFieldInstance:a=>{const l=bE(a);return n.current[l]}}),[e,t]);return[r]}var bF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const SF=(e,t)=>{const n=f.useContext(Ro),{getPrefixCls:r,direction:a,requiredMark:l,colon:s,scrollToFirstError:u,className:d,style:g}=Or("form"),{prefixCls:h,className:p,rootClassName:y,size:C,disabled:b=n,form:x,colon:S,labelAlign:E,labelWrap:$,labelCol:O,wrapperCol:_,hideRequiredMark:R,layout:T="horizontal",scrollToFirstError:M,requiredMark:z,onFinishFailed:P,name:D,style:F,feedbackIcons:G,variant:V}=e,U=bF(e,["prefixCls","className","rootClassName","size","disabled","form","colon","labelAlign","labelWrap","labelCol","wrapperCol","hideRequiredMark","layout","scrollToFirstError","requiredMark","onFinishFailed","name","style","feedbackIcons","variant"]),K=Lo(C),j=f.useContext(S2),A=f.useMemo(()=>z!==void 0?z:R?!1:l!==void 0?l:!0,[R,z,l]),L=S??s,q=r("form",h),N=ia(q),[B,W,X]=Hy(q,N),k=re(q,`${q}-${T}`,{[`${q}-hide-required-mark`]:A===!1,[`${q}-rtl`]:a==="rtl",[`${q}-${K}`]:K},X,N,W,d,p,y),[Y]=NT(x),{__INTERNAL__:ee}=Y;ee.name=D;const ue=f.useMemo(()=>({name:D,labelAlign:E,labelCol:O,labelWrap:$,wrapperCol:_,vertical:T==="vertical",colon:L,requiredMark:A,itemRef:ee.itemRef,form:Y,feedbackIcons:G}),[D,E,O,_,T,L,A,Y,G]),ae=f.useRef(null);f.useImperativeHandle(t,()=>{var ie;return Object.assign(Object.assign({},Y),{nativeElement:(ie=ae.current)===null||ie===void 0?void 0:ie.nativeElement})});const oe=(ie,le)=>{if(ie){let be={block:"nearest"};typeof ie=="object"&&(be=Object.assign(Object.assign({},be),ie)),Y.scrollToField(le,be)}},he=ie=>{if(P==null||P(ie),ie.errorFields.length){const le=ie.errorFields[0].name;if(M!==void 0){oe(M,le);return}u!==void 0&&oe(u,le)}};return B(f.createElement(IR.Provider,{value:V},f.createElement(ay,{disabled:b},f.createElement(Qi.Provider,{value:K},f.createElement(jR,{validateMessages:j},f.createElement(To.Provider,{value:ue},f.createElement(ps,Object.assign({id:D},U,{name:D,onFinishFailed:he,form:Y,ref:ae,style:Object.assign(Object.assign({},g),F),className:k}))))))))},CF=f.forwardRef(SF);function xF(e){if(typeof e=="function")return e;const t=Lr(e);return t.length<=1?t[0]:t}const jT=()=>{const{status:e,errors:t=[],warnings:n=[]}=f.useContext(ra);return{status:e,errors:t,warnings:n}};jT.Context=ra;function wF(e){const[t,n]=f.useState(e),r=f.useRef(null),a=f.useRef([]),l=f.useRef(!1);f.useEffect(()=>(l.current=!1,()=>{l.current=!0,bn.cancel(r.current),r.current=null}),[]);function s(u){l.current||(r.current===null&&(a.current=[],r.current=bn(()=>{r.current=null,n(d=>{let g=d;return a.current.forEach(h=>{g=h(g)}),g})})),a.current.push(u))}return[t,s]}function EF(){const{itemRef:e}=f.useContext(To),t=f.useRef({});function n(r,a){const l=a&&typeof a=="object"&&fs(a),s=r.join("_");return(t.current.name!==s||t.current.originRef!==l)&&(t.current.name=s,t.current.originRef=l,t.current.ref=Ha(e(r),l)),t.current.ref}return n}const $F=e=>{const{formItemCls:t}=e;return{"@media screen and (-ms-high-contrast: active), (-ms-high-contrast: none)":{[`${t}-control`]:{display:"flex"}}}},OF=hs(["Form","item-item"],(e,t)=>{let{rootPrefixCls:n}=t;const r=RT(e,n);return[$F(r)]});var RF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const _F=24,TF=e=>{const{prefixCls:t,status:n,labelCol:r,wrapperCol:a,children:l,errors:s,warnings:u,_internalItemRender:d,extra:g,help:h,fieldId:p,marginBottom:y,onErrorVisibleChanged:C,label:b}=e,x=`${t}-item`,S=f.useContext(To),E=f.useMemo(()=>{let U=Object.assign({},a||S.wrapperCol||{});return b===null&&!r&&!a&&S.labelCol&&[void 0,"xs","sm","md","lg","xl","xxl"].forEach(j=>{const A=j?[j]:[],L=wa(S.labelCol,A),q=typeof L=="object"?L:{},N=wa(U,A),B=typeof N=="object"?N:{};"span"in q&&!("offset"in B)&&q.span<_F&&(U=xa(U,[].concat(A,["offset"]),q.span))}),U},[a,S]),$=re(`${x}-control`,E.className),O=f.useMemo(()=>{const{labelCol:U,wrapperCol:K}=S;return RF(S,["labelCol","wrapperCol"])},[S]),_=f.useRef(null),[R,T]=f.useState(0);Gt(()=>{g&&_.current?T(_.current.clientHeight):T(0)},[g]);const M=f.createElement("div",{className:`${x}-control-input`},f.createElement("div",{className:`${x}-control-input-content`},l)),z=f.useMemo(()=>({prefixCls:t,status:n}),[t,n]),P=y!==null||s.length||u.length?f.createElement(Cy.Provider,{value:z},f.createElement(_T,{fieldId:p,errors:s,warnings:u,help:h,helpStatus:n,className:`${x}-explain-connected`,onVisibleChanged:C})):null,D={};p&&(D.id=`${p}_extra`);const F=g?f.createElement("div",Object.assign({},D,{className:`${x}-extra`,ref:_}),g):null,G=P||F?f.createElement("div",{className:`${x}-additional`,style:y?{minHeight:y+R}:{}},P,F):null,V=d&&d.mark==="pro_table_render"&&d.render?d.render(e,{input:M,errorList:P,extra:F}):f.createElement(f.Fragment,null,M,G);return f.createElement(To.Provider,{value:O},f.createElement(uT,Object.assign({},E,{className:$}),V),f.createElement(OF,{prefixCls:t}))};var MF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function NF(e){return e?typeof e=="object"&&!f.isValidElement(e)?e:{title:e}:null}const jF=e=>{let{prefixCls:t,label:n,htmlFor:r,labelCol:a,labelAlign:l,colon:s,required:u,requiredMark:d,tooltip:g,vertical:h}=e;var p;const[y]=Do("Form"),{labelAlign:C,labelCol:b,labelWrap:x,colon:S}=f.useContext(To);if(!n)return null;const E=a||b||{},$=l||C,O=`${t}-item-label`,_=re(O,$==="left"&&`${O}-left`,E.className,{[`${O}-wrap`]:!!x});let R=n;const T=s===!0||S!==!1&&s!==!1;T&&!h&&typeof n=="string"&&n.trim()&&(R=n.replace(/[:|：]\s*$/,""));const z=NF(g);if(z){const{icon:U=f.createElement(nP,null)}=z,K=MF(z,["icon"]),j=f.createElement(Ho,Object.assign({},K),f.cloneElement(U,{className:`${t}-item-tooltip`,title:"",onClick:A=>{A.preventDefault()},tabIndex:null}));R=f.createElement(f.Fragment,null,R,j)}const P=d==="optional",D=typeof d=="function",F=d===!1;D?R=d(R,{required:!!u}):P&&!u&&(R=f.createElement(f.Fragment,null,R,f.createElement("span",{className:`${t}-item-optional`,title:""},(y==null?void 0:y.optional)||((p=Oo.Form)===null||p===void 0?void 0:p.optional))));let G;F?G="hidden":(P||D)&&(G="optional");const V=re({[`${t}-item-required`]:u,[`${t}-item-required-mark-${G}`]:G,[`${t}-item-no-colon`]:!T});return f.createElement(uT,Object.assign({},E,{className:_}),f.createElement("label",{htmlFor:r,className:V,title:typeof n=="string"?n:""},R))},IF={success:cm,warning:us,error:ss,validating:Cu};function IT(e){let{children:t,errors:n,warnings:r,hasFeedback:a,validateStatus:l,prefixCls:s,meta:u,noStyle:d}=e;const g=`${s}-item`,{feedbackIcons:h}=f.useContext(To),p=MT(n,r,u,null,!!a,l),{isFormItemInput:y,status:C,hasFeedback:b,feedbackIcon:x}=f.useContext(ra),S=f.useMemo(()=>{var E;let $;if(a){const _=a!==!0&&a.icons||h,R=p&&((E=_==null?void 0:_({status:p,errors:n,warnings:r}))===null||E===void 0?void 0:E[p]),T=p&&IF[p];$=R!==!1&&T?f.createElement("span",{className:re(`${g}-feedback-icon`,`${g}-feedback-icon-${p}`)},R||f.createElement(T,null)):null}const O={status:p||"",errors:n,warnings:r,hasFeedback:!!a,feedbackIcon:$,isFormItemInput:!0};return d&&(O.status=(p??C)||"",O.isFormItemInput=y,O.hasFeedback=!!(a??b),O.feedbackIcon=a!==void 0?O.feedbackIcon:x),O},[p,a,d,y,C]);return f.createElement(ra.Provider,{value:S},t)}var PF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function AF(e){const{prefixCls:t,className:n,rootClassName:r,style:a,help:l,errors:s,warnings:u,validateStatus:d,meta:g,hasFeedback:h,hidden:p,children:y,fieldId:C,required:b,isRequired:x,onSubItemMetaChange:S,layout:E}=e,$=PF(e,["prefixCls","className","rootClassName","style","help","errors","warnings","validateStatus","meta","hasFeedback","hidden","children","fieldId","required","isRequired","onSubItemMetaChange","layout"]),O=`${t}-item`,{requiredMark:_,vertical:R}=f.useContext(To),T=R||E==="vertical",M=f.useRef(null),z=Gd(s),P=Gd(u),D=l!=null,F=!!(D||s.length||u.length),G=!!M.current&&vm(M.current),[V,U]=f.useState(null);Gt(()=>{if(F&&M.current){const q=getComputedStyle(M.current);U(parseInt(q.marginBottom,10))}},[F,G]);const K=q=>{q||U(null)},A=function(){let q=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1;const N=q?z:g.errors,B=q?P:g.warnings;return MT(N,B,g,"",!!h,d)}(),L=re(O,n,r,{[`${O}-with-help`]:D||z.length||P.length,[`${O}-has-feedback`]:A&&h,[`${O}-has-success`]:A==="success",[`${O}-has-warning`]:A==="warning",[`${O}-has-error`]:A==="error",[`${O}-is-validating`]:A==="validating",[`${O}-hidden`]:p,[`${O}-${E}`]:E});return f.createElement("div",{className:L,style:a,ref:M},f.createElement(O9,Object.assign({className:`${O}-row`},On($,["_internalItemRender","colon","dependencies","extra","fieldKey","getValueFromEvent","getValueProps","htmlFor","id","initialValue","isListField","label","labelAlign","labelCol","labelWrap","messageVariables","name","normalize","noStyle","preserve","requiredMark","rules","shouldUpdate","trigger","tooltip","validateFirst","validateTrigger","valuePropName","wrapperCol","validateDebounce"])),f.createElement(jF,Object.assign({htmlFor:C},e,{requiredMark:_,required:b??x,prefixCls:t,vertical:T})),f.createElement(TF,Object.assign({},e,g,{errors:z,warnings:P,prefixCls:t,status:A,help:l,marginBottom:V,onErrorVisibleChanged:K}),f.createElement(NR.Provider,{value:S},f.createElement(IT,{prefixCls:t,meta:g,errors:g.errors,warnings:g.warnings,hasFeedback:h,validateStatus:A},y)))),!!V&&f.createElement("div",{className:`${O}-margin-offset`,style:{marginBottom:-V}}))}const zF="__SPLIT__";function DF(e,t){const n=Object.keys(e),r=Object.keys(t);return n.length===r.length&&n.every(a=>{const l=e[a],s=t[a];return l===s||typeof l=="function"||typeof s=="function"})}const LF=f.memo(e=>{let{children:t}=e;return t},(e,t)=>DF(e.control,t.control)&&e.update===t.update&&e.childProps.length===t.childProps.length&&e.childProps.every((n,r)=>n===t.childProps[r]));function CE(){return{errors:[],warnings:[],touched:!1,validating:!1,name:[],validated:!1}}function BF(e){const{name:t,noStyle:n,className:r,dependencies:a,prefixCls:l,shouldUpdate:s,rules:u,children:d,required:g,label:h,messageVariables:p,trigger:y="onChange",validateTrigger:C,hidden:b,help:x,layout:S}=e,{getPrefixCls:E}=f.useContext(dt),{name:$}=f.useContext(To),O=xF(d),_=typeof O=="function",R=f.useContext(NR),{validateTrigger:T}=f.useContext(Ji),M=C!==void 0?C:T,z=t!=null,P=E("form",l),D=ia(P),[F,G,V]=Hy(P,D);Eu();const U=f.useContext(su),K=f.useRef(null),[j,A]=wF({}),[L,q]=Zi(()=>CE()),N=ue=>{const ae=U==null?void 0:U.getKey(ue.name);if(q(ue.destroy?CE():ue,!0),n&&x!==!1&&R){let oe=ue.name;if(ue.destroy)oe=K.current||oe;else if(ae!==void 0){const[he,ie]=ae;oe=[he].concat(Ne(ie)),K.current=oe}R(ue,oe)}},B=(ue,ae)=>{A(oe=>{const he=Object.assign({},oe),le=[].concat(Ne(ue.name.slice(0,-1)),Ne(ae)).join(zF);return ue.destroy?delete he[le]:he[le]=ue,he})},[W,X]=f.useMemo(()=>{const ue=Ne(L.errors),ae=Ne(L.warnings);return Object.values(j).forEach(oe=>{ue.push.apply(ue,Ne(oe.errors||[])),ae.push.apply(ae,Ne(oe.warnings||[]))}),[ue,ae]},[j,L.errors,L.warnings]),k=EF();function Y(ue,ae,oe){return n&&!b?f.createElement(IT,{prefixCls:P,hasFeedback:e.hasFeedback,validateStatus:e.validateStatus,meta:L,errors:W,warnings:X,noStyle:!0},ue):f.createElement(AF,Object.assign({key:"row"},e,{className:re(r,V,D,G),prefixCls:P,fieldId:ae,isRequired:oe,errors:W,warnings:X,meta:L,onSubItemMetaChange:B,layout:S}),ue)}if(!z&&!_&&!a)return F(Y(O));let ee={};return typeof h=="string"?ee.label=h:t&&(ee.label=String(t)),p&&(ee=Object.assign(Object.assign({},ee),p)),F(f.createElement(by,Object.assign({},e,{messageVariables:ee,trigger:y,validateTrigger:M,onMetaChange:N}),(ue,ae,oe)=>{const he=Yc(t).length&&ae?ae.name:[],ie=TT(he,$),le=g!==void 0?g:!!(u!=null&&u.some(fe=>{if(fe&&typeof fe=="object"&&fe.required&&!fe.warningOnly)return!0;if(typeof fe=="function"){const Se=fe(oe);return(Se==null?void 0:Se.required)&&!(Se!=null&&Se.warningOnly)}return!1})),be=Object.assign({},ue);let Re=null;if(Array.isArray(O)&&z)Re=O;else if(!(_&&(!(s||a)||z))){if(!(a&&!_&&!z))if(f.isValidElement(O)){const fe=Object.assign(Object.assign({},O.props),be);if(fe.id||(fe.id=ie),x||W.length>0||X.length>0||e.extra){const Oe=[];(x||W.length>0)&&Oe.push(`${ie}_help`),e.extra&&Oe.push(`${ie}_extra`),fe["aria-describedby"]=Oe.join(" ")}W.length>0&&(fe["aria-invalid"]="true"),le&&(fe["aria-required"]="true"),rl(O)&&(fe.ref=k(he,O)),new Set([].concat(Ne(Yc(y)),Ne(Yc(M)))).forEach(Oe=>{fe[Oe]=function(){for(var Pe,Ye,Me,De,xe,Te=arguments.length,Le=new Array(Te),We=0;We<Te;We++)Le[We]=arguments[We];(Me=be[Oe])===null||Me===void 0||(Pe=Me).call.apply(Pe,[be].concat(Le)),(xe=(De=O.props)[Oe])===null||xe===void 0||(Ye=xe).call.apply(Ye,[De].concat(Le))}});const Ee=[fe["aria-required"],fe["aria-invalid"],fe["aria-describedby"]];Re=f.createElement(LF,{control:be,update:O,childProps:Ee},na(O,fe))}else _&&(s||a)&&!z?Re=O(oe):Re=O}return Y(Re,ie,le)}))}const PT=BF;PT.useStatus=jT;var HF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const FF=e=>{var{prefixCls:t,children:n}=e,r=HF(e,["prefixCls","children"]);const{getPrefixCls:a}=f.useContext(dt),l=a("form",t),s=f.useMemo(()=>({prefixCls:l,status:"error"}),[l]);return f.createElement(RR,Object.assign({},r),(u,d,g)=>f.createElement(Cy.Provider,{value:s},n(u.map(h=>Object.assign(Object.assign({},h),{fieldKey:h.key})),d,{errors:g.errors,warnings:g.warnings})))};function VF(){const{form:e}=f.useContext(To);return e}const Hn=CF;Hn.Item=PT;Hn.List=FF;Hn.ErrorList=_T;Hn.useForm=NT;Hn.useFormInstance=VF;Hn.useWatch=MR;Hn.Provider=jR;Hn.create=()=>{};const UF=e=>{const{getPrefixCls:t,direction:n}=f.useContext(dt),{prefixCls:r,className:a}=e,l=t("input-group",r),s=t("input"),[u,d,g]=sT(s),h=re(l,g,{[`${l}-lg`]:e.size==="large",[`${l}-sm`]:e.size==="small",[`${l}-compact`]:e.compact,[`${l}-rtl`]:n==="rtl"},d,a),p=f.useContext(ra),y=f.useMemo(()=>Object.assign(Object.assign({},p),{isFormItemInput:!1}),[p]);return u(f.createElement("span",{className:h,style:e.style,onMouseEnter:e.onMouseEnter,onMouseLeave:e.onMouseLeave,onFocus:e.onFocus,onBlur:e.onBlur},f.createElement(ra.Provider,{value:y},e.children)))},kF=e=>{const{componentCls:t,paddingXS:n}=e;return{[t]:{display:"inline-flex",alignItems:"center",flexWrap:"nowrap",columnGap:n,[`${t}-input-wrapper`]:{position:"relative",[`${t}-mask-icon`]:{position:"absolute",zIndex:"1",top:"50%",right:"50%",transform:"translate(50%, -50%)",pointerEvents:"none"},[`${t}-mask-input`]:{color:"transparent",caretColor:"var(--ant-color-text)"},[`${t}-mask-input[type=number]::-webkit-inner-spin-button`]:{"-webkit-appearance":"none",margin:0},[`${t}-mask-input[type=number]`]:{"-moz-appearance":"textfield"}},"&-rtl":{direction:"rtl"},[`${t}-input`]:{textAlign:"center",paddingInline:e.paddingXXS},[`&${t}-sm ${t}-input`]:{paddingInline:e.calc(e.paddingXXS).div(2).equal()},[`&${t}-lg ${t}-input`]:{paddingInline:e.paddingXS}}}},qF=xn(["Input","OTP"],e=>{const t=Kt(e,zm(e));return[kF(t)]},Dm);var WF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const GF=f.forwardRef((e,t)=>{const{className:n,value:r,onChange:a,onActiveChange:l,index:s,mask:u}=e,d=WF(e,["className","value","onChange","onActiveChange","index","mask"]),{getPrefixCls:g}=f.useContext(dt),h=g("otp"),p=typeof u=="string"?u:r,y=f.useRef(null);f.useImperativeHandle(t,()=>y.current);const C=E=>{a(s,E.target.value)},b=()=>{bn(()=>{var E;const $=(E=y.current)===null||E===void 0?void 0:E.input;document.activeElement===$&&$&&$.select()})},x=E=>{const{key:$,ctrlKey:O,metaKey:_}=E;$==="ArrowLeft"?l(s-1):$==="ArrowRight"?l(s+1):$==="z"&&(O||_)&&E.preventDefault(),b()},S=E=>{E.key==="Backspace"&&!r&&l(s-1),b()};return f.createElement("span",{className:`${h}-input-wrapper`,role:"presentation"},u&&r!==""&&r!==void 0&&f.createElement("span",{className:`${h}-mask-icon`,"aria-hidden":"true"},p),f.createElement(Lm,Object.assign({"aria-label":`OTP Input ${s+1}`,type:u===!0?"password":"text"},d,{ref:y,value:r,onInput:C,onFocus:b,onKeyDown:x,onKeyUp:S,onMouseDown:b,onMouseUp:b,className:re(n,{[`${h}-mask-input`]:u})})))});var KF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function ed(e){return(e||"").split("")}const XF=e=>{const{index:t,prefixCls:n,separator:r}=e,a=typeof r=="function"?r(t):r;return a?f.createElement("span",{className:`${n}-separator`},a):null},YF=f.forwardRef((e,t)=>{const{prefixCls:n,length:r=6,size:a,defaultValue:l,value:s,onChange:u,formatter:d,separator:g,variant:h,disabled:p,status:y,autoFocus:C,mask:b,type:x,onInput:S,inputMode:E}=e,$=KF(e,["prefixCls","length","size","defaultValue","value","onChange","formatter","separator","variant","disabled","status","autoFocus","mask","type","onInput","inputMode"]),{getPrefixCls:O,direction:_}=f.useContext(dt),R=O("otp",n),T=ta($,{aria:!0,data:!0,attr:!0}),[M,z,P]=qF(R),D=Lo(k=>a??k),F=f.useContext(ra),G=Nm(F.status,y),V=f.useMemo(()=>Object.assign(Object.assign({},F),{status:G,hasFeedback:!1,feedbackIcon:null}),[F,G]),U=f.useRef(null),K=f.useRef({});f.useImperativeHandle(t,()=>({focus:()=>{var k;(k=K.current[0])===null||k===void 0||k.focus()},blur:()=>{var k;for(let Y=0;Y<r;Y+=1)(k=K.current[Y])===null||k===void 0||k.blur()},nativeElement:U.current}));const j=k=>d?d(k):k,[A,L]=f.useState(()=>ed(j(l||"")));f.useEffect(()=>{s!==void 0&&L(ed(s))},[s]);const q=An(k=>{L(k),S&&S(k),u&&k.length===r&&k.every(Y=>Y)&&k.some((Y,ee)=>A[ee]!==Y)&&u(k.join(""))}),N=An((k,Y)=>{let ee=Ne(A);for(let ae=0;ae<k;ae+=1)ee[ae]||(ee[ae]="");Y.length<=1?ee[k]=Y:ee=ee.slice(0,k).concat(ed(Y)),ee=ee.slice(0,r);for(let ae=ee.length-1;ae>=0&&!ee[ae];ae-=1)ee.pop();const ue=j(ee.map(ae=>ae||" ").join(""));return ee=ed(ue).map((ae,oe)=>ae===" "&&!ee[oe]?ee[oe]:ae),ee}),B=(k,Y)=>{var ee;const ue=N(k,Y),ae=Math.min(k+Y.length,r-1);ae!==k&&ue[k]!==void 0&&((ee=K.current[ae])===null||ee===void 0||ee.focus()),q(ue)},W=k=>{var Y;(Y=K.current[k])===null||Y===void 0||Y.focus()},X={variant:h,disabled:p,status:G,mask:b,type:x,inputMode:E};return M(f.createElement("div",Object.assign({},T,{ref:U,className:re(R,{[`${R}-sm`]:D==="small",[`${R}-lg`]:D==="large",[`${R}-rtl`]:_==="rtl"},P,z),role:"group"}),f.createElement(ra.Provider,{value:V},Array.from({length:r}).map((k,Y)=>{const ee=`otp-${Y}`,ue=A[Y]||"";return f.createElement(f.Fragment,{key:ee},f.createElement(GF,Object.assign({ref:ae=>{K.current[Y]=ae},index:Y,size:D,htmlSize:1,className:`${R}-input`,onChange:B,value:ue,onActiveChange:W,autoFocus:Y===0&&C},X)),Y<r-1&&f.createElement(XF,{separator:g,index:Y,prefixCls:R}))}))))});var QF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const ZF=e=>e?f.createElement(H4,null):f.createElement(D4,null),JF={click:"onClick",hover:"onMouseOver"},eV=f.forwardRef((e,t)=>{const{disabled:n,action:r="click",visibilityToggle:a=!0,iconRender:l=ZF}=e,s=f.useContext(Ro),u=n??s,d=typeof a=="object"&&a.visible!==void 0,[g,h]=f.useState(()=>d?a.visible:!1),p=f.useRef(null);f.useEffect(()=>{d&&h(a.visible)},[d,a]);const y=hT(p),C=()=>{var D;if(u)return;g&&y();const F=!g;h(F),typeof a=="object"&&((D=a.onVisibleChange)===null||D===void 0||D.call(a,F))},b=D=>{const F=JF[r]||"",G=l(g),V={[F]:C,className:`${D}-icon`,key:"passwordIcon",onMouseDown:U=>{U.preventDefault()},onMouseUp:U=>{U.preventDefault()}};return f.cloneElement(f.isValidElement(G)?G:f.createElement("span",null,G),V)},{className:x,prefixCls:S,inputPrefixCls:E,size:$}=e,O=QF(e,["className","prefixCls","inputPrefixCls","size"]),{getPrefixCls:_}=f.useContext(dt),R=_("input",E),T=_("input-password",S),M=a&&b(T),z=re(T,x,{[`${T}-${$}`]:!!$}),P=Object.assign(Object.assign({},On(O,["suffix","iconRender","visibilityToggle"])),{type:g?"text":"password",className:z,prefixCls:R,suffix:M});return $&&(P.size=$),f.createElement(Lm,Object.assign({ref:Ha(t,p)},P))});var tV=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const nV=f.forwardRef((e,t)=>{const{prefixCls:n,inputPrefixCls:r,className:a,size:l,suffix:s,enterButton:u=!1,addonAfter:d,loading:g,disabled:h,onSearch:p,onChange:y,onCompositionStart:C,onCompositionEnd:b}=e,x=tV(e,["prefixCls","inputPrefixCls","className","size","suffix","enterButton","addonAfter","loading","disabled","onSearch","onChange","onCompositionStart","onCompositionEnd"]),{getPrefixCls:S,direction:E}=f.useContext(dt),$=f.useRef(!1),O=S("input-search",n),_=S("input",r),{compactSize:R}=Ru(O,E),T=Lo(B=>{var W;return(W=l??R)!==null&&W!==void 0?W:B}),M=f.useRef(null),z=B=>{B!=null&&B.target&&B.type==="click"&&p&&p(B.target.value,B,{source:"clear"}),y==null||y(B)},P=B=>{var W;document.activeElement===((W=M.current)===null||W===void 0?void 0:W.input)&&B.preventDefault()},D=B=>{var W,X;p&&p((X=(W=M.current)===null||W===void 0?void 0:W.input)===null||X===void 0?void 0:X.value,B,{source:"input"})},F=B=>{$.current||g||D(B)},G=typeof u=="boolean"?f.createElement(UO,null):null,V=`${O}-button`;let U;const K=u||{},j=K.type&&K.type.__ANT_BUTTON===!0;j||K.type==="button"?U=na(K,Object.assign({onMouseDown:P,onClick:B=>{var W,X;(X=(W=K==null?void 0:K.props)===null||W===void 0?void 0:W.onClick)===null||X===void 0||X.call(W,B),D(B)},key:"enterButton"},j?{className:V,size:T}:{})):U=f.createElement(er,{className:V,type:u?"primary":void 0,size:T,disabled:h,key:"enterButton",onMouseDown:P,onClick:D,loading:g,icon:G},u),d&&(U=[U,na(d,{key:"addonAfter"})]);const A=re(O,{[`${O}-rtl`]:E==="rtl",[`${O}-${T}`]:!!T,[`${O}-with-button`]:!!u},a),L=Object.assign(Object.assign({},x),{className:A,prefixCls:_,type:"search"}),q=B=>{$.current=!0,C==null||C(B)},N=B=>{$.current=!1,b==null||b(B)};return f.createElement(Lm,Object.assign({ref:Ha(M,t),onPressEnter:F},L,{size:T,onCompositionStart:q,onCompositionEnd:N,addonAfter:U,suffix:s,onChange:z,disabled:h}))});var rV=`
  min-height:0 !important;
  max-height:none !important;
  height:0 !important;
  visibility:hidden !important;
  overflow:hidden !important;
  position:absolute !important;
  z-index:-1000 !important;
  top:0 !important;
  right:0 !important;
  pointer-events: none !important;
`,aV=["letter-spacing","line-height","padding-top","padding-bottom","font-family","font-weight","font-size","font-variant","text-rendering","text-transform","width","text-indent","padding-left","padding-right","border-width","box-sizing","word-break","white-space"],Vp={},Zr;function oV(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,n=e.getAttribute("id")||e.getAttribute("data-reactid")||e.getAttribute("name");if(t&&Vp[n])return Vp[n];var r=window.getComputedStyle(e),a=r.getPropertyValue("box-sizing")||r.getPropertyValue("-moz-box-sizing")||r.getPropertyValue("-webkit-box-sizing"),l=parseFloat(r.getPropertyValue("padding-bottom"))+parseFloat(r.getPropertyValue("padding-top")),s=parseFloat(r.getPropertyValue("border-bottom-width"))+parseFloat(r.getPropertyValue("border-top-width")),u=aV.map(function(g){return"".concat(g,":").concat(r.getPropertyValue(g))}).join(";"),d={sizingStyle:u,paddingSize:l,borderSize:s,boxSizing:a};return t&&n&&(Vp[n]=d),d}function iV(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:null,r=arguments.length>3&&arguments[3]!==void 0?arguments[3]:null;Zr||(Zr=document.createElement("textarea"),Zr.setAttribute("tab-index","-1"),Zr.setAttribute("aria-hidden","true"),Zr.setAttribute("name","hiddenTextarea"),document.body.appendChild(Zr)),e.getAttribute("wrap")?Zr.setAttribute("wrap",e.getAttribute("wrap")):Zr.removeAttribute("wrap");var a=oV(e,t),l=a.paddingSize,s=a.borderSize,u=a.boxSizing,d=a.sizingStyle;Zr.setAttribute("style","".concat(d,";").concat(rV)),Zr.value=e.value||e.placeholder||"";var g=void 0,h=void 0,p,y=Zr.scrollHeight;if(u==="border-box"?y+=s:u==="content-box"&&(y-=l),n!==null||r!==null){Zr.value=" ";var C=Zr.scrollHeight-l;n!==null&&(g=C*n,u==="border-box"&&(g=g+l+s),y=Math.max(g,y)),r!==null&&(h=C*r,u==="border-box"&&(h=h+l+s),p=y>h?"":"hidden",y=Math.min(h,y))}var b={height:y,overflowY:p,resize:"none"};return g&&(b.minHeight=g),h&&(b.maxHeight=h),b}var lV=["prefixCls","defaultValue","value","autoSize","onResize","className","style","disabled","onChange","onInternalAutoSize"],Up=0,kp=1,qp=2,sV=f.forwardRef(function(e,t){var n=e,r=n.prefixCls,a=n.defaultValue,l=n.value,s=n.autoSize,u=n.onResize,d=n.className,g=n.style,h=n.disabled,p=n.onChange;n.onInternalAutoSize;var y=$t(n,lV),C=br(a,{value:l,postState:function(k){return k??""}}),b=ge(C,2),x=b[0],S=b[1],E=function(k){S(k.target.value),p==null||p(k)},$=f.useRef();f.useImperativeHandle(t,function(){return{textArea:$.current}});var O=f.useMemo(function(){return s&&st(s)==="object"?[s.minRows,s.maxRows]:[]},[s]),_=ge(O,2),R=_[0],T=_[1],M=!!s,z=function(){try{if(document.activeElement===$.current){var k=$.current,Y=k.selectionStart,ee=k.selectionEnd,ue=k.scrollTop;$.current.setSelectionRange(Y,ee),$.current.scrollTop=ue}}catch{}},P=f.useState(qp),D=ge(P,2),F=D[0],G=D[1],V=f.useState(),U=ge(V,2),K=U[0],j=U[1],A=function(){G(Up)};Gt(function(){M&&A()},[l,R,T,M]),Gt(function(){if(F===Up)G(kp);else if(F===kp){var X=iV($.current,!1,R,T);G(qp),j(X)}else z()},[F]);var L=f.useRef(),q=function(){bn.cancel(L.current)},N=function(k){F===qp&&(u==null||u(k),s&&(q(),L.current=bn(function(){A()})))};f.useEffect(function(){return q},[]);var B=M?K:null,W=Q(Q({},g),B);return(F===Up||F===kp)&&(W.overflowY="hidden",W.overflowX="hidden"),f.createElement(zo,{onResize:N,disabled:!(s||u)},f.createElement("textarea",je({},y,{ref:$,style:W,className:re(r,d,Z({},"".concat(r,"-disabled"),h)),disabled:h,value:x,onChange:E})))}),cV=["defaultValue","value","onFocus","onBlur","onChange","allowClear","maxLength","onCompositionStart","onCompositionEnd","suffix","prefixCls","showCount","count","className","style","disabled","hidden","classNames","styles","onResize","onClear","onPressEnter","readOnly","autoSize","onKeyDown"],uV=se.forwardRef(function(e,t){var n,r=e.defaultValue,a=e.value,l=e.onFocus,s=e.onBlur,u=e.onChange,d=e.allowClear,g=e.maxLength,h=e.onCompositionStart,p=e.onCompositionEnd,y=e.suffix,C=e.prefixCls,b=C===void 0?"rc-textarea":C,x=e.showCount,S=e.count,E=e.className,$=e.style,O=e.disabled,_=e.hidden,R=e.classNames,T=e.styles,M=e.onResize,z=e.onClear,P=e.onPressEnter,D=e.readOnly,F=e.autoSize,G=e.onKeyDown,V=$t(e,cV),U=br(r,{value:a,defaultValue:r}),K=ge(U,2),j=K[0],A=K[1],L=j==null?"":String(j),q=se.useState(!1),N=ge(q,2),B=N[0],W=N[1],X=se.useRef(!1),k=se.useState(null),Y=ge(k,2),ee=Y[0],ue=Y[1],ae=f.useRef(null),oe=f.useRef(null),he=function(){var Be;return(Be=oe.current)===null||Be===void 0?void 0:Be.textArea},ie=function(){he().focus()};f.useImperativeHandle(t,function(){var Ze;return{resizableTextArea:oe.current,focus:ie,blur:function(){he().blur()},nativeElement:((Ze=ae.current)===null||Ze===void 0?void 0:Ze.nativeElement)||he()}}),f.useEffect(function(){W(function(Ze){return!O&&Ze})},[O]);var le=se.useState(null),be=ge(le,2),Re=be[0],fe=be[1];se.useEffect(function(){if(Re){var Ze;(Ze=he()).setSelectionRange.apply(Ze,Ne(Re))}},[Re]);var Se=mT(S,x),Ee=(n=Se.max)!==null&&n!==void 0?n:g,Oe=Number(Ee)>0,Pe=Se.strategy(L),Ye=!!Ee&&Pe>Ee,Me=function(Be,Ue){var Ie=Ue;!X.current&&Se.exceedFormatter&&Se.max&&Se.strategy(Ue)>Se.max&&(Ie=Se.exceedFormatter(Ue,{max:Se.max}),Ue!==Ie&&fe([he().selectionStart||0,he().selectionEnd||0])),A(Ie),Wd(Be.currentTarget,Be,u,Ie)},De=function(Be){X.current=!0,h==null||h(Be)},xe=function(Be){X.current=!1,Me(Be,Be.currentTarget.value),p==null||p(Be)},Te=function(Be){Me(Be,Be.target.value)},Le=function(Be){Be.key==="Enter"&&P&&P(Be),G==null||G(Be)},We=function(Be){W(!0),l==null||l(Be)},Ge=function(Be){W(!1),s==null||s(Be)},at=function(Be){A(""),ie(),Wd(he(),Be,u)},ke=y,Qe;Se.show&&(Se.showFormatter?Qe=Se.showFormatter({value:L,count:Pe,maxLength:Ee}):Qe="".concat(Pe).concat(Oe?" / ".concat(Ee):""),ke=se.createElement(se.Fragment,null,ke,se.createElement("span",{className:re("".concat(b,"-data-count"),R==null?void 0:R.count),style:T==null?void 0:T.count},Qe)));var ct=function(Be){var Ue;M==null||M(Be),(Ue=he())!==null&&Ue!==void 0&&Ue.style.height&&ue(!0)},pt=!F&&!x&&!d;return se.createElement(dT,{ref:ae,value:L,allowClear:d,handleReset:at,suffix:ke,prefixCls:b,classNames:Q(Q({},R),{},{affixWrapper:re(R==null?void 0:R.affixWrapper,Z(Z({},"".concat(b,"-show-count"),x),"".concat(b,"-textarea-allow-clear"),d))}),disabled:O,focused:B,className:re(E,Ye&&"".concat(b,"-out-of-range")),style:Q(Q({},$),ee&&!pt?{height:"auto"}:{}),dataAttrs:{affixWrapper:{"data-count":typeof Qe=="string"?Qe:void 0}},hidden:_,readOnly:D,onClear:z},se.createElement(sV,je({},V,{autoSize:F,maxLength:g,onKeyDown:Le,onChange:Te,onFocus:We,onBlur:Ge,onCompositionStart:De,onCompositionEnd:xe,className:re(R==null?void 0:R.textarea),style:Q(Q({},T==null?void 0:T.textarea),{},{resize:$==null?void 0:$.resize}),disabled:O,prefixCls:b,onResize:ct,ref:oe,readOnly:D})))});const fV=e=>{const{componentCls:t,paddingLG:n}=e,r=`${t}-textarea`;return{[`textarea${t}`]:{maxWidth:"100%",height:"auto",minHeight:e.controlHeight,lineHeight:e.lineHeight,verticalAlign:"bottom",transition:`all ${e.motionDurationSlow}`,resize:"vertical",[`&${t}-mouse-active`]:{transition:`all ${e.motionDurationSlow}, height 0s, width 0s`}},[`${t}-textarea-affix-wrapper-resize-dirty`]:{width:"auto"},[r]:{position:"relative","&-show-count":{[`${t}-data-count`]:{position:"absolute",bottom:e.calc(e.fontSize).mul(e.lineHeight).mul(-1).equal(),insetInlineEnd:0,color:e.colorTextDescription,whiteSpace:"nowrap",pointerEvents:"none"}},[`
        &-allow-clear > ${t},
        &-affix-wrapper${r}-has-feedback ${t}
      `]:{paddingInlineEnd:n},[`&-affix-wrapper${t}-affix-wrapper`]:{padding:0,[`> textarea${t}`]:{fontSize:"inherit",border:"none",outline:"none",background:"transparent",minHeight:e.calc(e.controlHeight).sub(e.calc(e.lineWidth).mul(2)).equal(),"&:focus":{boxShadow:"none !important"}},[`${t}-suffix`]:{margin:0,"> *:not(:last-child)":{marginInline:0},[`${t}-clear-icon`]:{position:"absolute",insetInlineEnd:e.paddingInline,insetBlockStart:e.paddingXS},[`${r}-suffix`]:{position:"absolute",top:0,insetInlineEnd:e.paddingInline,bottom:0,zIndex:1,display:"inline-flex",alignItems:"center",margin:"auto",pointerEvents:"none"}}},[`&-affix-wrapper${t}-affix-wrapper-rtl`]:{[`${t}-suffix`]:{[`${t}-data-count`]:{direction:"ltr",insetInlineStart:0}}},[`&-affix-wrapper${t}-affix-wrapper-sm`]:{[`${t}-suffix`]:{[`${t}-clear-icon`]:{insetInlineEnd:e.paddingInlineSM}}}}}},dV=xn(["Input","TextArea"],e=>{const t=Kt(e,zm(e));return[fV(t)]},Dm,{resetFont:!1});var mV=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const Bm=f.forwardRef((e,t)=>{var n;const{prefixCls:r,bordered:a=!0,size:l,disabled:s,status:u,allowClear:d,classNames:g,rootClassName:h,className:p,style:y,styles:C,variant:b,showCount:x,onMouseDown:S,onResize:E}=e,$=mV(e,["prefixCls","bordered","size","disabled","status","allowClear","classNames","rootClassName","className","style","styles","variant","showCount","onMouseDown","onResize"]),{getPrefixCls:O,direction:_,allowClear:R,autoComplete:T,className:M,style:z,classNames:P,styles:D}=Or("textArea"),F=f.useContext(Ro),G=s??F,{status:V,hasFeedback:U,feedbackIcon:K}=f.useContext(ra),j=Nm(V,u),A=f.useRef(null);f.useImperativeHandle(t,()=>{var Se;return{resizableTextArea:(Se=A.current)===null||Se===void 0?void 0:Se.resizableTextArea,focus:Ee=>{var Oe,Pe;fT((Pe=(Oe=A.current)===null||Oe===void 0?void 0:Oe.resizableTextArea)===null||Pe===void 0?void 0:Pe.textArea,Ee)},blur:()=>{var Ee;return(Ee=A.current)===null||Ee===void 0?void 0:Ee.blur()}}});const L=O("input",r),q=ia(L),[N,B,W]=lT(L,h),[X]=dV(L,q),{compactSize:k,compactItemClassnames:Y}=Ru(L,_),ee=Lo(Se=>{var Ee;return(Ee=l??k)!==null&&Ee!==void 0?Ee:Se}),[ue,ae]=Ty("textArea",b,a),oe=gT(d??R),[he,ie]=f.useState(!1),[le,be]=f.useState(!1),Re=Se=>{ie(!0),S==null||S(Se);const Ee=()=>{ie(!1),document.removeEventListener("mouseup",Ee)};document.addEventListener("mouseup",Ee)},fe=Se=>{var Ee,Oe;if(E==null||E(Se),he&&typeof getComputedStyle=="function"){const Pe=(Oe=(Ee=A.current)===null||Ee===void 0?void 0:Ee.nativeElement)===null||Oe===void 0?void 0:Oe.querySelector("textarea");Pe&&getComputedStyle(Pe).resize==="both"&&be(!0)}};return N(X(f.createElement(uV,Object.assign({autoComplete:T},$,{style:Object.assign(Object.assign({},z),y),styles:Object.assign(Object.assign({},D),C),disabled:G,allowClear:oe,className:re(W,q,p,h,Y,M,le&&`${L}-textarea-affix-wrapper-resize-dirty`),classNames:Object.assign(Object.assign(Object.assign({},g),P),{textarea:re({[`${L}-sm`]:ee==="small",[`${L}-lg`]:ee==="large"},B,g==null?void 0:g.textarea,P.textarea,he&&`${L}-mouse-active`),variant:re({[`${L}-${ue}`]:ae},Fd(L,j)),affixWrapper:re(`${L}-textarea-affix-wrapper`,{[`${L}-affix-wrapper-rtl`]:_==="rtl",[`${L}-affix-wrapper-sm`]:ee==="small",[`${L}-affix-wrapper-lg`]:ee==="large",[`${L}-textarea-show-count`]:x||((n=e.count)===null||n===void 0?void 0:n.show)},B)}),prefixCls:L,suffix:U&&f.createElement("span",{className:`${L}-textarea-suffix`},K),showCount:x,ref:A,onResize:fe,onMouseDown:Re}))))}),za=Lm;za.Group=UF;za.Search=nV;za.TextArea=Bm;za.Password=eV;za.OTP=YF;function gV(e,t,n){return typeof n=="boolean"?n:e.length?!0:Lr(t).some(a=>a.type===Q_)}var AT=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function Hm(e){let{suffixCls:t,tagName:n,displayName:r}=e;return a=>f.forwardRef((s,u)=>f.createElement(a,Object.assign({ref:u,suffixCls:t,tagName:n},s)))}const Fy=f.forwardRef((e,t)=>{const{prefixCls:n,suffixCls:r,className:a,tagName:l}=e,s=AT(e,["prefixCls","suffixCls","className","tagName"]),{getPrefixCls:u}=f.useContext(dt),d=u("layout",n),[g,h,p]=Y_(d),y=r?`${d}-${r}`:d;return g(f.createElement(l,Object.assign({className:re(n||y,a,h,p),ref:t},s)))}),hV=f.forwardRef((e,t)=>{const{direction:n}=f.useContext(dt),[r,a]=f.useState([]),{prefixCls:l,className:s,rootClassName:u,children:d,hasSider:g,tagName:h,style:p}=e,y=AT(e,["prefixCls","className","rootClassName","children","hasSider","tagName","style"]),C=On(y,["suffixCls"]),{getPrefixCls:b,className:x,style:S}=Or("layout"),E=b("layout",l),$=gV(r,d,g),[O,_,R]=Y_(E),T=re(E,{[`${E}-has-sider`]:$,[`${E}-rtl`]:n==="rtl"},x,s,u,_,R),M=f.useMemo(()=>({siderHook:{addSider:z=>{a(P=>[].concat(Ne(P),[z]))},removeSider:z=>{a(P=>P.filter(D=>D!==z))}}}),[]);return O(f.createElement(G_.Provider,{value:M},f.createElement(h,Object.assign({ref:t,className:T,style:Object.assign(Object.assign({},S),p)},C),d)))}),pV=Hm({tagName:"div",displayName:"Layout"})(hV),Vy=Hm({suffixCls:"header",tagName:"header",displayName:"Header"})(Fy),vV=Hm({suffixCls:"footer",tagName:"footer",displayName:"Footer"})(Fy),yV=Hm({suffixCls:"content",tagName:"main",displayName:"Content"})(Fy),La=pV;La.Header=Vy;La.Footer=vV;La.Content=yV;La.Sider=Q_;La._InternalSiderContext=Am;const bV=function(e){return(arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1)&&e==null?[]:Array.isArray(e)?e:[e]};let Ca=null,Wi=e=>e(),du=[],mu={};function xE(){const{getContainer:e,duration:t,rtl:n,maxCount:r,top:a}=mu,l=(e==null?void 0:e())||document.body;return{getContainer:()=>l,duration:t,rtl:n,maxCount:r,top:a}}const SV=se.forwardRef((e,t)=>{const{messageConfig:n,sync:r}=e,{getPrefixCls:a}=f.useContext(dt),l=mu.prefixCls||a("message"),s=f.useContext(a_),[u,d]=rR(Object.assign(Object.assign(Object.assign({},n),{prefixCls:l}),s.message));return se.useImperativeHandle(t,()=>{const g=Object.assign({},u);return Object.keys(g).forEach(h=>{g[h]=function(){return r(),u[h].apply(u,arguments)}}),{instance:g,sync:r}}),d}),CV=se.forwardRef((e,t)=>{const[n,r]=se.useState(xE),a=()=>{r(xE)};se.useEffect(a,[]);const l=iy(),s=l.getRootPrefixCls(),u=l.getIconPrefixCls(),d=l.getTheme(),g=se.createElement(SV,{ref:t,sync:a,messageConfig:n});return se.createElement(Fa,{prefixCls:s,iconPrefixCls:u,theme:d},l.holderRender?l.holderRender(g):g)});function Fm(){if(!Ca){const e=document.createDocumentFragment(),t={fragment:e};Ca=t,Wi(()=>{pm()(se.createElement(CV,{ref:r=>{const{instance:a,sync:l}=r||{};Promise.resolve().then(()=>{!t.instance&&a&&(t.instance=a,t.sync=l,Fm())})}}),e)});return}Ca.instance&&(du.forEach(e=>{const{type:t,skipped:n}=e;if(!n)switch(t){case"open":{Wi(()=>{const r=Ca.instance.open(Object.assign(Object.assign({},mu),e.config));r==null||r.then(e.resolve),e.setCloseFn(r)});break}case"destroy":Wi(()=>{Ca==null||Ca.instance.destroy(e.key)});break;default:Wi(()=>{var r;const a=(r=Ca.instance)[t].apply(r,Ne(e.args));a==null||a.then(e.resolve),e.setCloseFn(a)})}}),du=[])}function xV(e){mu=Object.assign(Object.assign({},mu),e),Wi(()=>{var t;(t=Ca==null?void 0:Ca.sync)===null||t===void 0||t.call(Ca)})}function wV(e){const t=cy(n=>{let r;const a={type:"open",config:e,resolve:n,setCloseFn:l=>{r=l}};return du.push(a),()=>{r?Wi(()=>{r()}):a.skipped=!0}});return Fm(),t}function EV(e,t){const n=cy(r=>{let a;const l={type:e,args:t,resolve:r,setCloseFn:s=>{a=s}};return du.push(l),()=>{a?Wi(()=>{a()}):l.skipped=!0}});return Fm(),n}const $V=e=>{du.push({type:"destroy",key:e}),Fm()},OV=["success","info","warning","error","loading"],RV={open:wV,destroy:$V,config:xV,useMessage:Q6,_InternalPanelDoNotUseOrYouWillBeFired:U6},Iu=RV;OV.forEach(e=>{Iu[e]=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return EV(e,n)}});var _V=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const TV=e=>{const{prefixCls:t,className:n,closeIcon:r,closable:a,type:l,title:s,children:u,footer:d}=e,g=_V(e,["prefixCls","className","closeIcon","closable","type","title","children","footer"]),{getPrefixCls:h}=f.useContext(dt),p=h(),y=t||h("modal"),C=ia(p),[b,x,S]=FR(y,C),E=`${y}-confirm`;let $={};return l?$={closable:a??!1,title:"",footer:"",children:f.createElement(UR,Object.assign({},e,{prefixCls:y,confirmPrefixCls:E,rootPrefixCls:p,content:u}))}:$={closable:a??!0,title:s,footer:d!==null&&f.createElement(DR,Object.assign({},e)),children:u},b(f.createElement(CR,Object.assign({prefixCls:y,className:re(x,`${y}-pure-panel`,l&&E,l&&`${E}-${l}`,n,S,C)},g,{closeIcon:zR(y,r),closable:a},$)))},MV=o_(TV);function zT(e){return Mu(GR(e))}const Hr=VR;Hr.useModal=r8;Hr.info=function(t){return Mu(KR(t))};Hr.success=function(t){return Mu(XR(t))};Hr.error=function(t){return Mu(YR(t))};Hr.warning=zT;Hr.warn=zT;Hr.confirm=function(t){return Mu(QR(t))};Hr.destroyAll=function(){for(;qi.length;){const t=qi.pop();t&&t()}};Hr.config=ZL;Hr._InternalPanelDoNotUseOrYouWillBeFired=MV;let Na=null,Sd=e=>e(),Kd=[],gu={};function wE(){const{getContainer:e,rtl:t,maxCount:n,top:r,bottom:a,showProgress:l,pauseOnHover:s}=gu,u=(e==null?void 0:e())||document.body;return{getContainer:()=>u,rtl:t,maxCount:n,top:r,bottom:a,showProgress:l,pauseOnHover:s}}const NV=se.forwardRef((e,t)=>{const{notificationConfig:n,sync:r}=e,{getPrefixCls:a}=f.useContext(dt),l=gu.prefixCls||a("notification"),s=f.useContext(a_),[u,d]=r_(Object.assign(Object.assign(Object.assign({},n),{prefixCls:l}),s.notification));return se.useEffect(r,[]),se.useImperativeHandle(t,()=>{const g=Object.assign({},u);return Object.keys(g).forEach(h=>{g[h]=function(){return r(),u[h].apply(u,arguments)}}),{instance:g,sync:r}}),d}),jV=se.forwardRef((e,t)=>{const[n,r]=se.useState(wE),a=()=>{r(wE)};se.useEffect(a,[]);const l=iy(),s=l.getRootPrefixCls(),u=l.getIconPrefixCls(),d=l.getTheme(),g=se.createElement(NV,{ref:t,sync:a,notificationConfig:n});return se.createElement(Fa,{prefixCls:s,iconPrefixCls:u,theme:d},l.holderRender?l.holderRender(g):g)});function Uy(){if(!Na){const e=document.createDocumentFragment(),t={fragment:e};Na=t,Sd(()=>{pm()(se.createElement(jV,{ref:r=>{const{instance:a,sync:l}=r||{};Promise.resolve().then(()=>{!t.instance&&a&&(t.instance=a,t.sync=l,Uy())})}}),e)});return}Na.instance&&(Kd.forEach(e=>{switch(e.type){case"open":{Sd(()=>{Na.instance.open(Object.assign(Object.assign({},gu),e.config))});break}case"destroy":Sd(()=>{Na==null||Na.instance.destroy(e.key)});break}}),Kd=[])}function IV(e){gu=Object.assign(Object.assign({},gu),e),Sd(()=>{var t;(t=Na==null?void 0:Na.sync)===null||t===void 0||t.call(Na)})}function DT(e){Kd.push({type:"open",config:e}),Uy()}const PV=e=>{Kd.push({type:"destroy",key:e}),Uy()},AV=["success","info","warning","error"],zV={open:DT,destroy:PV,config:IV,useNotification:$8,_InternalPanelDoNotUseOrYouWillBeFired:h8},ky=zV;AV.forEach(e=>{ky[e]=t=>DT(Object.assign(Object.assign({},t),{type:e}))});const DV=e=>{const{componentCls:t,iconCls:n,antCls:r,zIndexPopup:a,colorText:l,colorWarning:s,marginXXS:u,marginXS:d,fontSize:g,fontWeightStrong:h,colorTextHeading:p}=e;return{[t]:{zIndex:a,[`&${r}-popover`]:{fontSize:g},[`${t}-message`]:{marginBottom:d,display:"flex",flexWrap:"nowrap",alignItems:"start",[`> ${t}-message-icon ${n}`]:{color:s,fontSize:g,lineHeight:1,marginInlineEnd:d},[`${t}-title`]:{fontWeight:h,color:p,"&:only-child":{fontWeight:"normal"}},[`${t}-description`]:{marginTop:u,color:l}},[`${t}-buttons`]:{textAlign:"end",whiteSpace:"nowrap",button:{marginInlineStart:d}}}}},LV=e=>{const{zIndexPopupBase:t}=e;return{zIndexPopup:t+60}},LT=xn("Popconfirm",e=>DV(e),LV,{resetStyle:!1});var BV=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const BT=e=>{const{prefixCls:t,okButtonProps:n,cancelButtonProps:r,title:a,description:l,cancelText:s,okText:u,okType:d="primary",icon:g=f.createElement(us,null),showCancel:h=!0,close:p,onConfirm:y,onCancel:C,onPopupClick:b}=e,{getPrefixCls:x}=f.useContext(dt),[S]=Do("Popconfirm",Oo.Popconfirm),E=as(a),$=as(l);return f.createElement("div",{className:`${t}-inner-content`,onClick:b},f.createElement("div",{className:`${t}-message`},g&&f.createElement("span",{className:`${t}-message-icon`},g),f.createElement("div",{className:`${t}-message-text`},E&&f.createElement("div",{className:`${t}-title`},E),$&&f.createElement("div",{className:`${t}-description`},$))),f.createElement("div",{className:`${t}-buttons`},h&&f.createElement(er,Object.assign({onClick:C,size:"small"},r),s||(S==null?void 0:S.cancelText)),f.createElement(py,{buttonProps:Object.assign(Object.assign({size:"small"},uy(d)),n),actionFn:y,close:p,prefixCls:x("btn"),quitOnNullishReturnValue:!0,emitEvent:!0},u||(S==null?void 0:S.okText))))},HV=e=>{const{prefixCls:t,placement:n,className:r,style:a}=e,l=BV(e,["prefixCls","placement","className","style"]),{getPrefixCls:s}=f.useContext(dt),u=s("popconfirm",t),[d]=LT(u);return d(f.createElement(P_,{placement:n,className:re(u,r),style:a,content:f.createElement(BT,Object.assign({prefixCls:u},l))}))};var FV=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const VV=f.forwardRef((e,t)=>{var n,r;const{prefixCls:a,placement:l="top",trigger:s="click",okType:u="primary",icon:d=f.createElement(us,null),children:g,overlayClassName:h,onOpenChange:p,onVisibleChange:y,overlayStyle:C,styles:b,classNames:x}=e,S=FV(e,["prefixCls","placement","trigger","okType","icon","children","overlayClassName","onOpenChange","onVisibleChange","overlayStyle","styles","classNames"]),{getPrefixCls:E,className:$,style:O,classNames:_,styles:R}=Or("popconfirm"),[T,M]=br(!1,{value:(n=e.open)!==null&&n!==void 0?n:e.visible,defaultValue:(r=e.defaultOpen)!==null&&r!==void 0?r:e.defaultVisible}),z=(A,L)=>{M(A,!0),y==null||y(A),p==null||p(A,L)},P=A=>{z(!1,A)},D=A=>{var L;return(L=e.onConfirm)===null||L===void 0?void 0:L.call(void 0,A)},F=A=>{var L;z(!1,A),(L=e.onCancel)===null||L===void 0||L.call(void 0,A)},G=(A,L)=>{const{disabled:q=!1}=e;q||z(A,L)},V=E("popconfirm",a),U=re(V,$,h,_.root,x==null?void 0:x.root),K=re(_.body,x==null?void 0:x.body),[j]=LT(V);return j(f.createElement(A_,Object.assign({},On(S,["title"]),{trigger:s,placement:l,onOpenChange:G,open:T,ref:t,classNames:{root:U,body:K},styles:{root:Object.assign(Object.assign(Object.assign(Object.assign({},R.root),O),C),b==null?void 0:b.root),body:Object.assign(Object.assign({},R.body),b==null?void 0:b.body)},content:f.createElement(BT,Object.assign({okType:u,icon:d},e,{prefixCls:V,close:P,onConfirm:D,onCancel:F})),"data-popover-inject":!0}),g))}),Vm=VV;Vm._InternalPanelDoNotUseOrYouWillBeFired=HV;const UV=e=>{const{paddingXXS:t,lineWidth:n,tagPaddingHorizontal:r,componentCls:a,calc:l}=e,s=l(r).sub(n).equal(),u=l(t).sub(n).equal();return{[a]:Object.assign(Object.assign({},Rr(e)),{display:"inline-block",height:"auto",marginInlineEnd:e.marginXS,paddingInline:s,fontSize:e.tagFontSize,lineHeight:e.tagLineHeight,whiteSpace:"nowrap",background:e.defaultBg,border:`${ve(e.lineWidth)} ${e.lineType} ${e.colorBorder}`,borderRadius:e.borderRadiusSM,opacity:1,transition:`all ${e.motionDurationMid}`,textAlign:"start",position:"relative",[`&${a}-rtl`]:{direction:"rtl"},"&, a, a:hover":{color:e.defaultColor},[`${a}-close-icon`]:{marginInlineStart:u,fontSize:e.tagIconSize,color:e.colorIcon,cursor:"pointer",transition:`all ${e.motionDurationMid}`,"&:hover":{color:e.colorTextHeading}},[`&${a}-has-color`]:{borderColor:"transparent",[`&, a, a:hover, ${e.iconCls}-close, ${e.iconCls}-close:hover`]:{color:e.colorTextLightSolid}},"&-checkable":{backgroundColor:"transparent",borderColor:"transparent",cursor:"pointer",[`&:not(${a}-checkable-checked):hover`]:{color:e.colorPrimary,backgroundColor:e.colorFillSecondary},"&:active, &-checked":{color:e.colorTextLightSolid},"&-checked":{backgroundColor:e.colorPrimary,"&:hover":{backgroundColor:e.colorPrimaryHover}},"&:active":{backgroundColor:e.colorPrimaryActive}},"&-hidden":{display:"none"},[`> ${e.iconCls} + span, > span + ${e.iconCls}`]:{marginInlineStart:s}}),[`${a}-borderless`]:{borderColor:"transparent",background:e.tagBorderlessBg}}},qy=e=>{const{lineWidth:t,fontSizeIcon:n,calc:r}=e,a=e.fontSizeSM;return Kt(e,{tagFontSize:a,tagLineHeight:ve(r(e.lineHeightSM).mul(a).equal()),tagIconSize:r(n).sub(r(t).mul(2)).equal(),tagPaddingHorizontal:8,tagBorderlessBg:e.defaultBg})},Wy=e=>({defaultBg:new en(e.colorFillQuaternary).onBackground(e.colorBgContainer).toHexString(),defaultColor:e.colorText}),HT=xn("Tag",e=>{const t=qy(e);return UV(t)},Wy);var kV=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const qV=f.forwardRef((e,t)=>{const{prefixCls:n,style:r,className:a,checked:l,onChange:s,onClick:u}=e,d=kV(e,["prefixCls","style","className","checked","onChange","onClick"]),{getPrefixCls:g,tag:h}=f.useContext(dt),p=E=>{s==null||s(!l),u==null||u(E)},y=g("tag",n),[C,b,x]=HT(y),S=re(y,`${y}-checkable`,{[`${y}-checkable-checked`]:l},h==null?void 0:h.className,a,b,x);return C(f.createElement("span",Object.assign({},d,{ref:t,style:Object.assign(Object.assign({},r),h==null?void 0:h.style),className:S,onClick:p})))}),WV=e=>P2(e,(t,n)=>{let{textColor:r,lightBorderColor:a,lightColor:l,darkColor:s}=n;return{[`${e.componentCls}${e.componentCls}-${t}`]:{color:r,background:l,borderColor:a,"&-inverse":{color:e.colorTextLightSolid,background:s,borderColor:s},[`&${e.componentCls}-borderless`]:{borderColor:"transparent"}}}}),GV=hs(["Tag","preset"],e=>{const t=qy(e);return WV(t)},Wy);function KV(e){return typeof e!="string"?e:e.charAt(0).toUpperCase()+e.slice(1)}const td=(e,t,n)=>{const r=KV(n);return{[`${e.componentCls}${e.componentCls}-${t}`]:{color:e[`color${n}`],background:e[`color${r}Bg`],borderColor:e[`color${r}Border`],[`&${e.componentCls}-borderless`]:{borderColor:"transparent"}}}},XV=hs(["Tag","status"],e=>{const t=qy(e);return[td(t,"success","Success"),td(t,"processing","Info"),td(t,"error","Error"),td(t,"warning","Warning")]},Wy);var YV=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const QV=f.forwardRef((e,t)=>{const{prefixCls:n,className:r,rootClassName:a,style:l,children:s,icon:u,color:d,onClose:g,bordered:h=!0,visible:p}=e,y=YV(e,["prefixCls","className","rootClassName","style","children","icon","color","onClose","bordered","visible"]),{getPrefixCls:C,direction:b,tag:x}=f.useContext(dt),[S,E]=f.useState(!0),$=On(y,["closeIcon","closable"]);f.useEffect(()=>{p!==void 0&&E(p)},[p]);const O=M_(d),_=Z7(d),R=O||_,T=Object.assign(Object.assign({backgroundColor:d&&!R?d:void 0},x==null?void 0:x.style),l),M=C("tag",n),[z,P,D]=HT(M),F=re(M,x==null?void 0:x.className,{[`${M}-${d}`]:R,[`${M}-has-color`]:d&&!R,[`${M}-hidden`]:!S,[`${M}-rtl`]:b==="rtl",[`${M}-borderless`]:!h},r,a,P,D),G=L=>{L.stopPropagation(),g==null||g(L),!L.defaultPrevented&&E(!1)},[,V]=xy(rs(e),rs(x),{closable:!1,closeIconRender:L=>{const q=f.createElement("span",{className:`${M}-close-icon`,onClick:G},L);return Y2(L,q,N=>({onClick:B=>{var W;(W=N==null?void 0:N.onClick)===null||W===void 0||W.call(N,B),G(B)},className:re(N==null?void 0:N.className,`${M}-close-icon`)}))}}),U=typeof y.onClick=="function"||s&&s.type==="a",K=u||null,j=K?f.createElement(f.Fragment,null,K,s&&f.createElement("span",null,s)):s,A=f.createElement("span",Object.assign({},$,{ref:t,className:F,style:T}),j,V,O&&f.createElement(GV,{key:"preset",prefixCls:M}),_&&f.createElement(XV,{key:"status",prefixCls:M}));return z(U?f.createElement(oR,{component:"Tag"},A):A)}),FT=QV;FT.CheckableTag=qV;const Sa=(e,t)=>new en(e).setA(t).toRgbString(),kl=(e,t)=>new en(e).lighten(t).toHexString(),ZV=e=>{const t=Yi(e,{theme:"dark"});return{1:t[0],2:t[1],3:t[2],4:t[3],5:t[6],6:t[5],7:t[4],8:t[6],9:t[5],10:t[4]}},JV=(e,t)=>{const n=e||"#000",r=t||"#fff";return{colorBgBase:n,colorTextBase:r,colorText:Sa(r,.85),colorTextSecondary:Sa(r,.65),colorTextTertiary:Sa(r,.45),colorTextQuaternary:Sa(r,.25),colorFill:Sa(r,.18),colorFillSecondary:Sa(r,.12),colorFillTertiary:Sa(r,.08),colorFillQuaternary:Sa(r,.04),colorBgSolid:Sa(r,.95),colorBgSolidHover:Sa(r,1),colorBgSolidActive:Sa(r,.9),colorBgElevated:kl(n,12),colorBgContainer:kl(n,8),colorBgLayout:kl(n,0),colorBgSpotlight:kl(n,26),colorBgBlur:Sa(r,.04),colorBorder:kl(n,26),colorBorderSecondary:kl(n,19)}},eU=(e,t)=>{const n=Object.keys(ty).map(a=>{const l=Yi(e[a],{theme:"dark"});return Array.from({length:10},()=>1).reduce((s,u,d)=>(s[`${a}-${d+1}`]=l[d],s[`${a}${d+1}`]=l[d],s),{})}).reduce((a,l)=>(a=Object.assign(Object.assign({},a),l),a),{}),r=t??ny(e);return Object.assign(Object.assign(Object.assign({},r),n),w2(e,{generateColorPalettes:ZV,generateNeutralColorPalettes:JV}))},EE={defaultSeed:Id.token,defaultAlgorithm:ny,darkAlgorithm:eU,_internalContext:ry},tU=(e,t,n,r)=>{const{titleMarginBottom:a,fontWeightStrong:l}=r;return{marginBottom:a,color:n,fontWeight:l,fontSize:e,lineHeight:t}},nU=e=>{const t=[1,2,3,4,5],n={};return t.forEach(r=>{n[`
      h${r}&,
      div&-h${r},
      div&-h${r} > textarea,
      h${r}
    `]=tU(e[`fontSizeHeading${r}`],e[`lineHeightHeading${r}`],e.colorTextHeading,e)}),n},rU=e=>{const{componentCls:t}=e;return{"a&, a":Object.assign(Object.assign({},I2(e)),{userSelect:"text",[`&[disabled], &${t}-disabled`]:{color:e.colorTextDisabled,cursor:"not-allowed","&:active, &:hover":{color:e.colorTextDisabled},"&:active":{pointerEvents:"none"}}})}},aU=e=>({code:{margin:"0 0.2em",paddingInline:"0.4em",paddingBlock:"0.2em 0.1em",fontSize:"85%",fontFamily:e.fontFamilyCode,background:"rgba(150, 150, 150, 0.1)",border:"1px solid rgba(100, 100, 100, 0.2)",borderRadius:3},kbd:{margin:"0 0.2em",paddingInline:"0.4em",paddingBlock:"0.15em 0.1em",fontSize:"90%",fontFamily:e.fontFamilyCode,background:"rgba(150, 150, 150, 0.06)",border:"1px solid rgba(100, 100, 100, 0.2)",borderBottomWidth:2,borderRadius:3},mark:{padding:0,backgroundColor:Rd[2]},"u, ins":{textDecoration:"underline",textDecorationSkipInk:"auto"},"s, del":{textDecoration:"line-through"},strong:{fontWeight:600},"ul, ol":{marginInline:0,marginBlock:"0 1em",padding:0,li:{marginInline:"20px 0",marginBlock:0,paddingInline:"4px 0",paddingBlock:0}},ul:{listStyleType:"circle",ul:{listStyleType:"disc"}},ol:{listStyleType:"decimal"},"pre, blockquote":{margin:"1em 0"},pre:{padding:"0.4em 0.6em",whiteSpace:"pre-wrap",wordWrap:"break-word",background:"rgba(150, 150, 150, 0.1)",border:"1px solid rgba(100, 100, 100, 0.2)",borderRadius:3,fontFamily:e.fontFamilyCode,code:{display:"inline",margin:0,padding:0,fontSize:"inherit",fontFamily:"inherit",background:"transparent",border:0}},blockquote:{paddingInline:"0.6em 0",paddingBlock:0,borderInlineStart:"4px solid rgba(100, 100, 100, 0.2)",opacity:.85}}),oU=e=>{const{componentCls:t,paddingSM:n}=e,r=n;return{"&-edit-content":{position:"relative","div&":{insetInlineStart:e.calc(e.paddingSM).mul(-1).equal(),marginTop:e.calc(r).mul(-1).equal(),marginBottom:`calc(1em - ${ve(r)})`},[`${t}-edit-content-confirm`]:{position:"absolute",insetInlineEnd:e.calc(e.marginXS).add(2).equal(),insetBlockEnd:e.marginXS,color:e.colorIcon,fontWeight:"normal",fontSize:e.fontSize,fontStyle:"normal",pointerEvents:"none"},textarea:{margin:"0!important",MozTransition:"none",height:"1em"}}}},iU=e=>({[`${e.componentCls}-copy-success`]:{"\n    &,\n    &:hover,\n    &:focus":{color:e.colorSuccess}},[`${e.componentCls}-copy-icon-only`]:{marginInlineStart:0}}),lU=()=>({"\n  a&-ellipsis,\n  span&-ellipsis\n  ":{display:"inline-block",maxWidth:"100%"},"&-ellipsis-single-line":{whiteSpace:"nowrap",overflow:"hidden",textOverflow:"ellipsis","a&, span&":{verticalAlign:"bottom"},"> code":{paddingBlock:0,maxWidth:"calc(100% - 1.2em)",display:"inline-block",overflow:"hidden",textOverflow:"ellipsis",verticalAlign:"bottom",boxSizing:"content-box"}},"&-ellipsis-multiple-line":{display:"-webkit-box",overflow:"hidden",WebkitLineClamp:3,WebkitBoxOrient:"vertical"}}),sU=e=>{const{componentCls:t,titleMarginTop:n}=e;return{[t]:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:e.colorText,wordBreak:"break-word",lineHeight:e.lineHeight,[`&${t}-secondary`]:{color:e.colorTextDescription},[`&${t}-success`]:{color:e.colorSuccessText},[`&${t}-warning`]:{color:e.colorWarningText},[`&${t}-danger`]:{color:e.colorErrorText,"a&:active, a&:focus":{color:e.colorErrorTextActive},"a&:hover":{color:e.colorErrorTextHover}},[`&${t}-disabled`]:{color:e.colorTextDisabled,cursor:"not-allowed",userSelect:"none"},"\n        div&,\n        p\n      ":{marginBottom:"1em"}},nU(e)),{[`
      & + h1${t},
      & + h2${t},
      & + h3${t},
      & + h4${t},
      & + h5${t}
      `]:{marginTop:n},"\n      div,\n      ul,\n      li,\n      p,\n      h1,\n      h2,\n      h3,\n      h4,\n      h5":{"\n        + h1,\n        + h2,\n        + h3,\n        + h4,\n        + h5\n        ":{marginTop:n}}}),aU(e)),rU(e)),{[`
        ${t}-expand,
        ${t}-collapse,
        ${t}-edit,
        ${t}-copy
      `]:Object.assign(Object.assign({},I2(e)),{marginInlineStart:e.marginXXS})}),oU(e)),iU(e)),lU()),{"&-rtl":{direction:"rtl"}})}},cU=()=>({titleMarginTop:"1.2em",titleMarginBottom:"0.5em"}),VT=xn("Typography",e=>[sU(e)],cU),uU=e=>{const{prefixCls:t,"aria-label":n,className:r,style:a,direction:l,maxLength:s,autoSize:u=!0,value:d,onSave:g,onCancel:h,onEnd:p,component:y,enterIcon:C=f.createElement(j4,null)}=e,b=f.useRef(null),x=f.useRef(!1),S=f.useRef(null),[E,$]=f.useState(d);f.useEffect(()=>{$(d)},[d]),f.useEffect(()=>{var U;if(!((U=b.current)===null||U===void 0)&&U.resizableTextArea){const{textArea:K}=b.current.resizableTextArea;K.focus();const{length:j}=K.value;K.setSelectionRange(j,j)}},[]);const O=U=>{let{target:K}=U;$(K.value.replace(/[\n\r]/g,""))},_=()=>{x.current=!0},R=()=>{x.current=!1},T=U=>{let{keyCode:K}=U;x.current||(S.current=K)},M=()=>{g(E.trim())},z=U=>{let{keyCode:K,ctrlKey:j,altKey:A,metaKey:L,shiftKey:q}=U;S.current!==K||x.current||j||A||L||q||(K===lt.ENTER?(M(),p==null||p()):K===lt.ESC&&h())},P=()=>{M()},[D,F,G]=VT(t),V=re(t,`${t}-edit-content`,{[`${t}-rtl`]:l==="rtl",[`${t}-${y}`]:!!y},r,F,G);return D(f.createElement("div",{className:V,style:a},f.createElement(Bm,{ref:b,maxLength:s,value:E,onChange:O,onKeyDown:T,onKeyUp:z,onCompositionStart:_,onCompositionEnd:R,onBlur:P,"aria-label":n,rows:1,autoSize:u}),C!==null?na(C,{className:`${t}-edit-content-confirm`}):null))};var Wp,$E;function fU(){return $E||($E=1,Wp=function(){var e=document.getSelection();if(!e.rangeCount)return function(){};for(var t=document.activeElement,n=[],r=0;r<e.rangeCount;r++)n.push(e.getRangeAt(r));switch(t.tagName.toUpperCase()){case"INPUT":case"TEXTAREA":t.blur();break;default:t=null;break}return e.removeAllRanges(),function(){e.type==="Caret"&&e.removeAllRanges(),e.rangeCount||n.forEach(function(a){e.addRange(a)}),t&&t.focus()}}),Wp}var Gp,OE;function dU(){if(OE)return Gp;OE=1;var e=fU(),t={"text/plain":"Text","text/html":"Url",default:"Text"},n="Copy to clipboard: #{key}, Enter";function r(l){var s=(/mac os x/i.test(navigator.userAgent)?"⌘":"Ctrl")+"+C";return l.replace(/#{\s*key\s*}/g,s)}function a(l,s){var u,d,g,h,p,y,C=!1;s||(s={}),u=s.debug||!1;try{g=e(),h=document.createRange(),p=document.getSelection(),y=document.createElement("span"),y.textContent=l,y.ariaHidden="true",y.style.all="unset",y.style.position="fixed",y.style.top=0,y.style.clip="rect(0, 0, 0, 0)",y.style.whiteSpace="pre",y.style.webkitUserSelect="text",y.style.MozUserSelect="text",y.style.msUserSelect="text",y.style.userSelect="text",y.addEventListener("copy",function(x){if(x.stopPropagation(),s.format)if(x.preventDefault(),typeof x.clipboardData>"u"){u&&console.warn("unable to use e.clipboardData"),u&&console.warn("trying IE specific stuff"),window.clipboardData.clearData();var S=t[s.format]||t.default;window.clipboardData.setData(S,l)}else x.clipboardData.clearData(),x.clipboardData.setData(s.format,l);s.onCopy&&(x.preventDefault(),s.onCopy(x.clipboardData))}),document.body.appendChild(y),h.selectNodeContents(y),p.addRange(h);var b=document.execCommand("copy");if(!b)throw new Error("copy command was unsuccessful");C=!0}catch(x){u&&console.error("unable to copy using execCommand: ",x),u&&console.warn("trying IE specific stuff");try{window.clipboardData.setData(s.format||"text",l),s.onCopy&&s.onCopy(window.clipboardData),C=!0}catch(S){u&&console.error("unable to copy using clipboardData: ",S),u&&console.error("falling back to prompt"),d=r("message"in s?s.message:n),window.prompt(d,l)}}finally{p&&(typeof p.removeRange=="function"?p.removeRange(h):p.removeAllRanges()),y&&document.body.removeChild(y),g()}return C}return Gp=a,Gp}var mU=dU();const gU=is(mU);var hU=function(e,t,n,r){function a(l){return l instanceof n?l:new n(function(s){s(l)})}return new(n||(n=Promise))(function(l,s){function u(h){try{g(r.next(h))}catch(p){s(p)}}function d(h){try{g(r.throw(h))}catch(p){s(p)}}function g(h){h.done?l(h.value):a(h.value).then(u,d)}g((r=r.apply(e,t||[])).next())})};const pU=e=>{let{copyConfig:t,children:n}=e;const[r,a]=f.useState(!1),[l,s]=f.useState(!1),u=f.useRef(null),d=()=>{u.current&&clearTimeout(u.current)},g={};t.format&&(g.format=t.format),f.useEffect(()=>d,[]);const h=An(p=>hU(void 0,void 0,void 0,function*(){var y;p==null||p.preventDefault(),p==null||p.stopPropagation(),s(!0);try{const C=typeof t.text=="function"?yield t.text():t.text;gU(C||bV(n,!0).join("")||"",g),s(!1),a(!0),d(),u.current=setTimeout(()=>{a(!1)},3e3),(y=t.onCopy)===null||y===void 0||y.call(t,p)}catch(C){throw s(!1),C}}));return{copied:r,copyLoading:l,onClick:h}};function Kp(e,t){return f.useMemo(()=>{const n=!!e;return[n,Object.assign(Object.assign({},t),n&&typeof e=="object"?e:null)]},[e])}const vU=e=>{const t=f.useRef(void 0);return f.useEffect(()=>{t.current=e}),t.current},yU=(e,t,n)=>f.useMemo(()=>e===!0?{title:t??n}:f.isValidElement(e)?{title:e}:typeof e=="object"?Object.assign({title:t??n},e):{title:e},[e,t,n]);var bU=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const UT=f.forwardRef((e,t)=>{const{prefixCls:n,component:r="article",className:a,rootClassName:l,setContentRef:s,children:u,direction:d,style:g}=e,h=bU(e,["prefixCls","component","className","rootClassName","setContentRef","children","direction","style"]),{getPrefixCls:p,direction:y,className:C,style:b}=Or("typography"),x=d??y,S=s?Ha(t,s):t,E=p("typography",n),[$,O,_]=VT(E),R=re(E,C,{[`${E}-rtl`]:x==="rtl"},a,l,O,_),T=Object.assign(Object.assign({},b),g);return $(f.createElement(r,Object.assign({className:R,style:T,ref:S},h),u))});function RE(e){return e===!1?[!1,!1]:Array.isArray(e)?e:[e]}function Xp(e,t,n){return e===!0||e===void 0?t:e||n&&t}function SU(e){const t=document.createElement("em");e.appendChild(t);const n=e.getBoundingClientRect(),r=t.getBoundingClientRect();return e.removeChild(t),n.left>r.left||r.right>n.right||n.top>r.top||r.bottom>n.bottom}const Gy=e=>["string","number"].includes(typeof e),CU=e=>{let{prefixCls:t,copied:n,locale:r,iconOnly:a,tooltips:l,icon:s,tabIndex:u,onCopy:d,loading:g}=e;const h=RE(l),p=RE(s),{copied:y,copy:C}=r??{},b=n?y:C,x=Xp(h[n?1:0],b),S=typeof x=="string"?x:b;return f.createElement(Ho,{title:x},f.createElement("button",{type:"button",className:re(`${t}-copy`,{[`${t}-copy-success`]:n,[`${t}-copy-icon-only`]:a}),onClick:d,"aria-label":S,tabIndex:u},n?Xp(p[1],f.createElement(HO,null),!0):Xp(p[0],g?f.createElement(Cu,null):f.createElement(S4,null),!0)))},nd=f.forwardRef((e,t)=>{let{style:n,children:r}=e;const a=f.useRef(null);return f.useImperativeHandle(t,()=>({isExceed:()=>{const l=a.current;return l.scrollHeight>l.clientHeight},getHeight:()=>a.current.clientHeight})),f.createElement("span",{"aria-hidden":!0,ref:a,style:Object.assign({position:"fixed",display:"block",left:0,top:0,pointerEvents:"none",backgroundColor:"rgba(255, 0, 0, 0.65)"},n)},r)}),xU=e=>e.reduce((t,n)=>t+(Gy(n)?String(n).length:1),0);function _E(e,t){let n=0;const r=[];for(let a=0;a<e.length;a+=1){if(n===t)return r;const l=e[a],u=Gy(l)?String(l).length:1,d=n+u;if(d>t){const g=t-n;return r.push(String(l).slice(0,g)),r}r.push(l),n=d}return e}const Yp=0,Qp=1,Zp=2,Jp=3,TE=4,rd={display:"-webkit-box",overflow:"hidden",WebkitBoxOrient:"vertical"};function wU(e){const{enableMeasure:t,width:n,text:r,children:a,rows:l,expanded:s,miscDeps:u,onEllipsis:d}=e,g=f.useMemo(()=>Lr(r),[r]),h=f.useMemo(()=>xU(g),[r]),p=f.useMemo(()=>a(g,!1),[r]),[y,C]=f.useState(null),b=f.useRef(null),x=f.useRef(null),S=f.useRef(null),E=f.useRef(null),$=f.useRef(null),[O,_]=f.useState(!1),[R,T]=f.useState(Yp),[M,z]=f.useState(0),[P,D]=f.useState(null);Gt(()=>{T(t&&n&&h?Qp:Yp)},[n,r,l,t,g]),Gt(()=>{var U,K,j,A;if(R===Qp){T(Zp);const L=x.current&&getComputedStyle(x.current).whiteSpace;D(L)}else if(R===Zp){const L=!!(!((U=S.current)===null||U===void 0)&&U.isExceed());T(L?Jp:TE),C(L?[0,h]:null),_(L);const q=((K=S.current)===null||K===void 0?void 0:K.getHeight())||0,N=l===1?0:((j=E.current)===null||j===void 0?void 0:j.getHeight())||0,B=((A=$.current)===null||A===void 0?void 0:A.getHeight())||0,W=Math.max(q,N+B);z(W+1),d(L)}},[R]);const F=y?Math.ceil((y[0]+y[1])/2):0;Gt(()=>{var U;const[K,j]=y||[0,0];if(K!==j){const L=(((U=b.current)===null||U===void 0?void 0:U.getHeight())||0)>M;let q=F;j-K===1&&(q=L?K:j),C(L?[K,q]:[q,j])}},[y,F]);const G=f.useMemo(()=>{if(!t)return a(g,!1);if(R!==Jp||!y||y[0]!==y[1]){const U=a(g,!1);return[TE,Yp].includes(R)?U:f.createElement("span",{style:Object.assign(Object.assign({},rd),{WebkitLineClamp:l})},U)}return a(s?g:_E(g,y[0]),O)},[s,R,y,g].concat(Ne(u))),V={width:n,margin:0,padding:0,whiteSpace:P==="nowrap"?"normal":"inherit"};return f.createElement(f.Fragment,null,G,R===Zp&&f.createElement(f.Fragment,null,f.createElement(nd,{style:Object.assign(Object.assign(Object.assign({},V),rd),{WebkitLineClamp:l}),ref:S},p),f.createElement(nd,{style:Object.assign(Object.assign(Object.assign({},V),rd),{WebkitLineClamp:l-1}),ref:E},p),f.createElement(nd,{style:Object.assign(Object.assign(Object.assign({},V),rd),{WebkitLineClamp:1}),ref:$},a([],!0))),R===Jp&&y&&y[0]!==y[1]&&f.createElement(nd,{style:Object.assign(Object.assign({},V),{top:400}),ref:b},a(_E(g,F),!0)),R===Qp&&f.createElement("span",{style:{whiteSpace:"inherit"},ref:x}))}const EU=e=>{let{enableEllipsis:t,isEllipsis:n,children:r,tooltipProps:a}=e;return!(a!=null&&a.title)||!t?r:f.createElement(Ho,Object.assign({open:n?void 0:!1},a),r)};var $U=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};function OU(e,t){let{mark:n,code:r,underline:a,delete:l,strong:s,keyboard:u,italic:d}=e,g=t;function h(p,y){y&&(g=f.createElement(p,{},g))}return h("strong",s),h("u",a),h("del",l),h("code",r),h("mark",n),h("kbd",u),h("i",d),g}const RU="...",Um=f.forwardRef((e,t)=>{var n;const{prefixCls:r,className:a,style:l,type:s,disabled:u,children:d,ellipsis:g,editable:h,copyable:p,component:y,title:C}=e,b=$U(e,["prefixCls","className","style","type","disabled","children","ellipsis","editable","copyable","component","title"]),{getPrefixCls:x,direction:S}=f.useContext(dt),[E]=Do("Text"),$=f.useRef(null),O=f.useRef(null),_=x("typography",r),R=On(b,["mark","code","delete","underline","strong","keyboard","italic"]),[T,M]=Kp(h),[z,P]=br(!1,{value:M.editing}),{triggerType:D=["icon"]}=M,F=Ie=>{var qe;Ie&&((qe=M.onStart)===null||qe===void 0||qe.call(M)),P(Ie)},G=vU(z);Gt(()=>{var Ie;!z&&G&&((Ie=O.current)===null||Ie===void 0||Ie.focus())},[z]);const V=Ie=>{Ie==null||Ie.preventDefault(),F(!0)},U=Ie=>{var qe;(qe=M.onChange)===null||qe===void 0||qe.call(M,Ie),F(!1)},K=()=>{var Ie;(Ie=M.onCancel)===null||Ie===void 0||Ie.call(M),F(!1)},[j,A]=Kp(p),{copied:L,copyLoading:q,onClick:N}=pU({copyConfig:A,children:d}),[B,W]=f.useState(!1),[X,k]=f.useState(!1),[Y,ee]=f.useState(!1),[ue,ae]=f.useState(!1),[oe,he]=f.useState(!0),[ie,le]=Kp(g,{expandable:!1,symbol:Ie=>Ie?E==null?void 0:E.collapse:E==null?void 0:E.expand}),[be,Re]=br(le.defaultExpanded||!1,{value:le.expanded}),fe=ie&&(!be||le.expandable==="collapsible"),{rows:Se=1}=le,Ee=f.useMemo(()=>fe&&(le.suffix!==void 0||le.onEllipsis||le.expandable||T||j),[fe,le,T,j]);Gt(()=>{ie&&!Ee&&(W(gw("webkitLineClamp")),k(gw("textOverflow")))},[Ee,ie]);const[Oe,Pe]=f.useState(fe),Ye=f.useMemo(()=>Ee?!1:Se===1?X:B,[Ee,X,B]);Gt(()=>{Pe(Ye&&fe)},[Ye,fe]);const Me=fe&&(Oe?ue:Y),De=fe&&Se===1&&Oe,xe=fe&&Se>1&&Oe,Te=(Ie,qe)=>{var wt;Re(qe.expanded),(wt=le.onExpand)===null||wt===void 0||wt.call(le,Ie,qe)},[Le,We]=f.useState(0),Ge=Ie=>{let{offsetWidth:qe}=Ie;We(qe)},at=Ie=>{var qe;ee(Ie),Y!==Ie&&((qe=le.onEllipsis)===null||qe===void 0||qe.call(le,Ie))};f.useEffect(()=>{const Ie=$.current;if(ie&&Oe&&Ie){const qe=SU(Ie);ue!==qe&&ae(qe)}},[ie,Oe,d,xe,oe,Le]),f.useEffect(()=>{const Ie=$.current;if(typeof IntersectionObserver>"u"||!Ie||!Oe||!fe)return;const qe=new IntersectionObserver(()=>{he(!!Ie.offsetParent)});return qe.observe(Ie),()=>{qe.disconnect()}},[Oe,fe]);const ke=yU(le.tooltip,M.text,d),Qe=f.useMemo(()=>{if(!(!ie||Oe))return[M.text,d,C,ke.title].find(Gy)},[ie,Oe,C,ke.title,Me]);if(z)return f.createElement(uU,{value:(n=M.text)!==null&&n!==void 0?n:typeof d=="string"?d:"",onSave:U,onCancel:K,onEnd:M.onEnd,prefixCls:_,className:a,style:l,direction:S,component:y,maxLength:M.maxLength,autoSize:M.autoSize,enterIcon:M.enterIcon});const ct=()=>{const{expandable:Ie,symbol:qe}=le;return Ie?f.createElement("button",{type:"button",key:"expand",className:`${_}-${be?"collapse":"expand"}`,onClick:wt=>Te(wt,{expanded:!be}),"aria-label":be?E.collapse:E==null?void 0:E.expand},typeof qe=="function"?qe(be):qe):null},pt=()=>{if(!T)return;const{icon:Ie,tooltip:qe,tabIndex:wt}=M,It=Lr(qe)[0]||(E==null?void 0:E.edit),Nn=typeof It=="string"?It:"";return D.includes("icon")?f.createElement(Ho,{key:"edit",title:qe===!1?"":It},f.createElement("button",{type:"button",ref:O,className:`${_}-edit`,onClick:V,"aria-label":Nn,tabIndex:wt},Ie||f.createElement(O4,{role:"button"}))):null},Ze=()=>j?f.createElement(CU,Object.assign({key:"copy"},A,{prefixCls:_,copied:L,locale:E,onCopy:N,loading:q,iconOnly:d==null})):null,Be=Ie=>[Ie&&ct(),pt(),Ze()],Ue=Ie=>[Ie&&!be&&f.createElement("span",{"aria-hidden":!0,key:"ellipsis"},RU),le.suffix,Be(Ie)];return f.createElement(zo,{onResize:Ge,disabled:!fe},Ie=>f.createElement(EU,{tooltipProps:ke,enableEllipsis:fe,isEllipsis:Me},f.createElement(UT,Object.assign({className:re({[`${_}-${s}`]:s,[`${_}-disabled`]:u,[`${_}-ellipsis`]:ie,[`${_}-ellipsis-single-line`]:De,[`${_}-ellipsis-multiple-line`]:xe},a),prefixCls:r,style:Object.assign(Object.assign({},l),{WebkitLineClamp:xe?Se:void 0}),component:y,ref:Ha(Ie,$,t),direction:S,onClick:D.includes("text")?V:void 0,"aria-label":Qe==null?void 0:Qe.toString(),title:C},R),f.createElement(wU,{enableMeasure:fe&&!Oe,text:d,rows:Se,width:Le,onEllipsis:at,expanded:be,miscDeps:[L,be,q,T,j,E]},(qe,wt)=>OU(e,f.createElement(f.Fragment,null,qe.length>0&&wt&&!be&&Qe?f.createElement("span",{key:"show-content","aria-hidden":!0},qe):qe,Ue(wt)))))))});var _U=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const TU=f.forwardRef((e,t)=>{var{ellipsis:n,rel:r}=e,a=_U(e,["ellipsis","rel"]);const l=Object.assign(Object.assign({},a),{rel:r===void 0&&a.target==="_blank"?"noopener noreferrer":r});return delete l.navigate,f.createElement(Um,Object.assign({},l,{ref:t,ellipsis:!!n,component:"a"}))}),MU=f.forwardRef((e,t)=>f.createElement(Um,Object.assign({ref:t},e,{component:"div"})));var NU=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const jU=(e,t)=>{var{ellipsis:n}=e,r=NU(e,["ellipsis"]);const a=f.useMemo(()=>n&&typeof n=="object"?On(n,["expandable","rows"]):n,[n]);return f.createElement(Um,Object.assign({ref:t},r,{ellipsis:a,component:"span"}))},IU=f.forwardRef(jU);var PU=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,r=Object.getOwnPropertySymbols(e);a<r.length;a++)t.indexOf(r[a])<0&&Object.prototype.propertyIsEnumerable.call(e,r[a])&&(n[r[a]]=e[r[a]]);return n};const AU=[1,2,3,4,5],zU=f.forwardRef((e,t)=>{const{level:n=1}=e,r=PU(e,["level"]),a=AU.includes(n)?`h${n}`:"h1";return f.createElement(Um,Object.assign({ref:t},r,{component:a}))}),Pu=UT;Pu.Text=IU;Pu.Link=TU;Pu.Title=zU;Pu.Paragraph=MU;var $c={},ME;function DU(){if(ME)return $c;ME=1,Object.defineProperty($c,"__esModule",{value:!0}),$c.parse=s,$c.serialize=g;const e=/^[\u0021-\u003A\u003C\u003E-\u007E]+$/,t=/^[\u0021-\u003A\u003C-\u007E]*$/,n=/^([.]?[a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?)([.][a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?)*$/i,r=/^[\u0020-\u003A\u003D-\u007E]*$/,a=Object.prototype.toString,l=(()=>{const y=function(){};return y.prototype=Object.create(null),y})();function s(y,C){const b=new l,x=y.length;if(x<2)return b;const S=(C==null?void 0:C.decode)||h;let E=0;do{const $=y.indexOf("=",E);if($===-1)break;const O=y.indexOf(";",E),_=O===-1?x:O;if($>_){E=y.lastIndexOf(";",$-1)+1;continue}const R=u(y,E,$),T=d(y,$,R),M=y.slice(R,T);if(b[M]===void 0){let z=u(y,$+1,_),P=d(y,_,z);const D=S(y.slice(z,P));b[M]=D}E=_+1}while(E<x);return b}function u(y,C,b){do{const x=y.charCodeAt(C);if(x!==32&&x!==9)return C}while(++C<b);return b}function d(y,C,b){for(;C>b;){const x=y.charCodeAt(--C);if(x!==32&&x!==9)return C+1}return b}function g(y,C,b){const x=(b==null?void 0:b.encode)||encodeURIComponent;if(!e.test(y))throw new TypeError(`argument name is invalid: ${y}`);const S=x(C);if(!t.test(S))throw new TypeError(`argument val is invalid: ${C}`);let E=y+"="+S;if(!b)return E;if(b.maxAge!==void 0){if(!Number.isInteger(b.maxAge))throw new TypeError(`option maxAge is invalid: ${b.maxAge}`);E+="; Max-Age="+b.maxAge}if(b.domain){if(!n.test(b.domain))throw new TypeError(`option domain is invalid: ${b.domain}`);E+="; Domain="+b.domain}if(b.path){if(!r.test(b.path))throw new TypeError(`option path is invalid: ${b.path}`);E+="; Path="+b.path}if(b.expires){if(!p(b.expires)||!Number.isFinite(b.expires.valueOf()))throw new TypeError(`option expires is invalid: ${b.expires}`);E+="; Expires="+b.expires.toUTCString()}if(b.httpOnly&&(E+="; HttpOnly"),b.secure&&(E+="; Secure"),b.partitioned&&(E+="; Partitioned"),b.priority)switch(typeof b.priority=="string"?b.priority.toLowerCase():void 0){case"low":E+="; Priority=Low";break;case"medium":E+="; Priority=Medium";break;case"high":E+="; Priority=High";break;default:throw new TypeError(`option priority is invalid: ${b.priority}`)}if(b.sameSite)switch(typeof b.sameSite=="string"?b.sameSite.toLowerCase():b.sameSite){case!0:case"strict":E+="; SameSite=Strict";break;case"lax":E+="; SameSite=Lax";break;case"none":E+="; SameSite=None";break;default:throw new TypeError(`option sameSite is invalid: ${b.sameSite}`)}return E}function h(y){if(y.indexOf("%")===-1)return y;try{return decodeURIComponent(y)}catch{return y}}function p(y){return a.call(y)==="[object Date]"}return $c}DU();/**
 * react-router v7.5.1
 *
 * Copyright (c) Remix Software Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.md file in the root directory of this source tree.
 *
 * @license MIT
 */var NE="popstate";function LU(e={}){function t(a,l){let{pathname:s="/",search:u="",hash:d=""}=Ss(a.location.hash.substring(1));return!s.startsWith("/")&&!s.startsWith(".")&&(s="/"+s),$0("",{pathname:s,search:u,hash:d},l.state&&l.state.usr||null,l.state&&l.state.key||"default")}function n(a,l){let s=a.document.querySelector("base"),u="";if(s&&s.getAttribute("href")){let d=a.location.href,g=d.indexOf("#");u=g===-1?d:d.slice(0,g)}return u+"#"+(typeof l=="string"?l:hu(l))}function r(a,l){Ba(a.pathname.charAt(0)==="/",`relative pathnames are not supported in hash history.push(${JSON.stringify(l)})`)}return HU(t,n,r,e)}function tr(e,t){if(e===!1||e===null||typeof e>"u")throw new Error(t)}function Ba(e,t){if(!e){typeof console<"u"&&console.warn(t);try{throw new Error(t)}catch{}}}function BU(){return Math.random().toString(36).substring(2,10)}function jE(e,t){return{usr:e.state,key:e.key,idx:t}}function $0(e,t,n=null,r){return{pathname:typeof e=="string"?e:e.pathname,search:"",hash:"",...typeof t=="string"?Ss(t):t,state:n,key:t&&t.key||r||BU()}}function hu({pathname:e="/",search:t="",hash:n=""}){return t&&t!=="?"&&(e+=t.charAt(0)==="?"?t:"?"+t),n&&n!=="#"&&(e+=n.charAt(0)==="#"?n:"#"+n),e}function Ss(e){let t={};if(e){let n=e.indexOf("#");n>=0&&(t.hash=e.substring(n),e=e.substring(0,n));let r=e.indexOf("?");r>=0&&(t.search=e.substring(r),e=e.substring(0,r)),e&&(t.pathname=e)}return t}function HU(e,t,n,r={}){let{window:a=document.defaultView,v5Compat:l=!1}=r,s=a.history,u="POP",d=null,g=h();g==null&&(g=0,s.replaceState({...s.state,idx:g},""));function h(){return(s.state||{idx:null}).idx}function p(){u="POP";let S=h(),E=S==null?null:S-g;g=S,d&&d({action:u,location:x.location,delta:E})}function y(S,E){u="PUSH";let $=$0(x.location,S,E);n&&n($,S),g=h()+1;let O=jE($,g),_=x.createHref($);try{s.pushState(O,"",_)}catch(R){if(R instanceof DOMException&&R.name==="DataCloneError")throw R;a.location.assign(_)}l&&d&&d({action:u,location:x.location,delta:1})}function C(S,E){u="REPLACE";let $=$0(x.location,S,E);n&&n($,S),g=h();let O=jE($,g),_=x.createHref($);s.replaceState(O,"",_),l&&d&&d({action:u,location:x.location,delta:0})}function b(S){let E=a.location.origin!=="null"?a.location.origin:a.location.href,$=typeof S=="string"?S:hu(S);return $=$.replace(/ $/,"%20"),tr(E,`No window.location.(origin|href) available to create URL for href: ${$}`),new URL($,E)}let x={get action(){return u},get location(){return e(a,s)},listen(S){if(d)throw new Error("A history only accepts one active listener");return a.addEventListener(NE,p),d=S,()=>{a.removeEventListener(NE,p),d=null}},createHref(S){return t(a,S)},createURL:b,encodeLocation(S){let E=b(S);return{pathname:E.pathname,search:E.search,hash:E.hash}},push:y,replace:C,go(S){return s.go(S)}};return x}function kT(e,t,n="/"){return FU(e,t,n,!1)}function FU(e,t,n,r){let a=typeof t=="string"?Ss(t):t,l=No(a.pathname||"/",n);if(l==null)return null;let s=qT(e);VU(s);let u=null;for(let d=0;u==null&&d<s.length;++d){let g=JU(l);u=QU(s[d],g,r)}return u}function qT(e,t=[],n=[],r=""){let a=(l,s,u)=>{let d={relativePath:u===void 0?l.path||"":u,caseSensitive:l.caseSensitive===!0,childrenIndex:s,route:l};d.relativePath.startsWith("/")&&(tr(d.relativePath.startsWith(r),`Absolute route path "${d.relativePath}" nested under path "${r}" is not valid. An absolute child route path must start with the combined path of all its parent routes.`),d.relativePath=d.relativePath.slice(r.length));let g=wo([r,d.relativePath]),h=n.concat(d);l.children&&l.children.length>0&&(tr(l.index!==!0,`Index routes must not have child routes. Please remove all child routes from route path "${g}".`),qT(l.children,t,h,g)),!(l.path==null&&!l.index)&&t.push({path:g,score:XU(g,l.index),routesMeta:h})};return e.forEach((l,s)=>{var u;if(l.path===""||!((u=l.path)!=null&&u.includes("?")))a(l,s);else for(let d of WT(l.path))a(l,s,d)}),t}function WT(e){let t=e.split("/");if(t.length===0)return[];let[n,...r]=t,a=n.endsWith("?"),l=n.replace(/\?$/,"");if(r.length===0)return a?[l,""]:[l];let s=WT(r.join("/")),u=[];return u.push(...s.map(d=>d===""?l:[l,d].join("/"))),a&&u.push(...s),u.map(d=>e.startsWith("/")&&d===""?"/":d)}function VU(e){e.sort((t,n)=>t.score!==n.score?n.score-t.score:YU(t.routesMeta.map(r=>r.childrenIndex),n.routesMeta.map(r=>r.childrenIndex)))}var UU=/^:[\w-]+$/,kU=3,qU=2,WU=1,GU=10,KU=-2,IE=e=>e==="*";function XU(e,t){let n=e.split("/"),r=n.length;return n.some(IE)&&(r+=KU),t&&(r+=qU),n.filter(a=>!IE(a)).reduce((a,l)=>a+(UU.test(l)?kU:l===""?WU:GU),r)}function YU(e,t){return e.length===t.length&&e.slice(0,-1).every((r,a)=>r===t[a])?e[e.length-1]-t[t.length-1]:0}function QU(e,t,n=!1){let{routesMeta:r}=e,a={},l="/",s=[];for(let u=0;u<r.length;++u){let d=r[u],g=u===r.length-1,h=l==="/"?t:t.slice(l.length)||"/",p=Xd({path:d.relativePath,caseSensitive:d.caseSensitive,end:g},h),y=d.route;if(!p&&g&&n&&!r[r.length-1].route.index&&(p=Xd({path:d.relativePath,caseSensitive:d.caseSensitive,end:!1},h)),!p)return null;Object.assign(a,p.params),s.push({params:a,pathname:wo([l,p.pathname]),pathnameBase:rk(wo([l,p.pathnameBase])),route:y}),p.pathnameBase!=="/"&&(l=wo([l,p.pathnameBase]))}return s}function Xd(e,t){typeof e=="string"&&(e={path:e,caseSensitive:!1,end:!0});let[n,r]=ZU(e.path,e.caseSensitive,e.end),a=t.match(n);if(!a)return null;let l=a[0],s=l.replace(/(.)\/+$/,"$1"),u=a.slice(1);return{params:r.reduce((g,{paramName:h,isOptional:p},y)=>{if(h==="*"){let b=u[y]||"";s=l.slice(0,l.length-b.length).replace(/(.)\/+$/,"$1")}const C=u[y];return p&&!C?g[h]=void 0:g[h]=(C||"").replace(/%2F/g,"/"),g},{}),pathname:l,pathnameBase:s,pattern:e}}function ZU(e,t=!1,n=!0){Ba(e==="*"||!e.endsWith("*")||e.endsWith("/*"),`Route path "${e}" will be treated as if it were "${e.replace(/\*$/,"/*")}" because the \`*\` character must always follow a \`/\` in the pattern. To get rid of this warning, please change the route path to "${e.replace(/\*$/,"/*")}".`);let r=[],a="^"+e.replace(/\/*\*?$/,"").replace(/^\/*/,"/").replace(/[\\.*+^${}|()[\]]/g,"\\$&").replace(/\/:([\w-]+)(\?)?/g,(s,u,d)=>(r.push({paramName:u,isOptional:d!=null}),d?"/?([^\\/]+)?":"/([^\\/]+)"));return e.endsWith("*")?(r.push({paramName:"*"}),a+=e==="*"||e==="/*"?"(.*)$":"(?:\\/(.+)|\\/*)$"):n?a+="\\/*$":e!==""&&e!=="/"&&(a+="(?:(?=\\/|$))"),[new RegExp(a,t?void 0:"i"),r]}function JU(e){try{return e.split("/").map(t=>decodeURIComponent(t).replace(/\//g,"%2F")).join("/")}catch(t){return Ba(!1,`The URL path "${e}" could not be decoded because it is a malformed URL segment. This is probably due to a bad percent encoding (${t}).`),e}}function No(e,t){if(t==="/")return e;if(!e.toLowerCase().startsWith(t.toLowerCase()))return null;let n=t.endsWith("/")?t.length-1:t.length,r=e.charAt(n);return r&&r!=="/"?null:e.slice(n)||"/"}function ek(e,t="/"){let{pathname:n,search:r="",hash:a=""}=typeof e=="string"?Ss(e):e;return{pathname:n?n.startsWith("/")?n:tk(n,t):t,search:ak(r),hash:ok(a)}}function tk(e,t){let n=t.replace(/\/+$/,"").split("/");return e.split("/").forEach(a=>{a===".."?n.length>1&&n.pop():a!=="."&&n.push(a)}),n.length>1?n.join("/"):"/"}function ev(e,t,n,r){return`Cannot include a '${e}' character in a manually specified \`to.${t}\` field [${JSON.stringify(r)}].  Please separate it out to the \`to.${n}\` field. Alternatively you may provide the full path as a string in <Link to="..."> and the router will parse it for you.`}function nk(e){return e.filter((t,n)=>n===0||t.route.path&&t.route.path.length>0)}function GT(e){let t=nk(e);return t.map((n,r)=>r===t.length-1?n.pathname:n.pathnameBase)}function KT(e,t,n,r=!1){let a;typeof e=="string"?a=Ss(e):(a={...e},tr(!a.pathname||!a.pathname.includes("?"),ev("?","pathname","search",a)),tr(!a.pathname||!a.pathname.includes("#"),ev("#","pathname","hash",a)),tr(!a.search||!a.search.includes("#"),ev("#","search","hash",a)));let l=e===""||a.pathname==="",s=l?"/":a.pathname,u;if(s==null)u=n;else{let p=t.length-1;if(!r&&s.startsWith("..")){let y=s.split("/");for(;y[0]==="..";)y.shift(),p-=1;a.pathname=y.join("/")}u=p>=0?t[p]:"/"}let d=ek(a,u),g=s&&s!=="/"&&s.endsWith("/"),h=(l||s===".")&&n.endsWith("/");return!d.pathname.endsWith("/")&&(g||h)&&(d.pathname+="/"),d}var wo=e=>e.join("/").replace(/\/\/+/g,"/"),rk=e=>e.replace(/\/+$/,"").replace(/^\/*/,"/"),ak=e=>!e||e==="?"?"":e.startsWith("?")?e:"?"+e,ok=e=>!e||e==="#"?"":e.startsWith("#")?e:"#"+e;function ik(e){return e!=null&&typeof e.status=="number"&&typeof e.statusText=="string"&&typeof e.internal=="boolean"&&"data"in e}var XT=["POST","PUT","PATCH","DELETE"];new Set(XT);var lk=["GET",...XT];new Set(lk);var Cs=f.createContext(null);Cs.displayName="DataRouter";var km=f.createContext(null);km.displayName="DataRouterState";var YT=f.createContext({isTransitioning:!1});YT.displayName="ViewTransition";var sk=f.createContext(new Map);sk.displayName="Fetchers";var ck=f.createContext(null);ck.displayName="Await";var no=f.createContext(null);no.displayName="Navigation";var qm=f.createContext(null);qm.displayName="Location";var Va=f.createContext({outlet:null,matches:[],isDataRoute:!1});Va.displayName="Route";var Ky=f.createContext(null);Ky.displayName="RouteError";function uk(e,{relative:t}={}){tr(Au(),"useHref() may be used only in the context of a <Router> component.");let{basename:n,navigator:r}=f.useContext(no),{hash:a,pathname:l,search:s}=zu(e,{relative:t}),u=l;return n!=="/"&&(u=l==="/"?n:wo([n,l])),r.createHref({pathname:u,search:s,hash:a})}function Au(){return f.useContext(qm)!=null}function Fo(){return tr(Au(),"useLocation() may be used only in the context of a <Router> component."),f.useContext(qm).location}var QT="You should call navigate() in a React.useEffect(), not when your component is first rendered.";function ZT(e){f.useContext(no).static||f.useLayoutEffect(e)}function Xy(){let{isDataRoute:e}=f.useContext(Va);return e?Ek():fk()}function fk(){tr(Au(),"useNavigate() may be used only in the context of a <Router> component.");let e=f.useContext(Cs),{basename:t,navigator:n}=f.useContext(no),{matches:r}=f.useContext(Va),{pathname:a}=Fo(),l=JSON.stringify(GT(r)),s=f.useRef(!1);return ZT(()=>{s.current=!0}),f.useCallback((d,g={})=>{if(Ba(s.current,QT),!s.current)return;if(typeof d=="number"){n.go(d);return}let h=KT(d,JSON.parse(l),a,g.relative==="path");e==null&&t!=="/"&&(h.pathname=h.pathname==="/"?t:wo([t,h.pathname])),(g.replace?n.replace:n.push)(h,g.state,g)},[t,n,l,a,e])}var JT=f.createContext(null);function yX(){return f.useContext(JT)}function dk(e){let t=f.useContext(Va).outlet;return t&&f.createElement(JT.Provider,{value:e},t)}function bX(){let{matches:e}=f.useContext(Va),t=e[e.length-1];return t?t.params:{}}function zu(e,{relative:t}={}){let{matches:n}=f.useContext(Va),{pathname:r}=Fo(),a=JSON.stringify(GT(n));return f.useMemo(()=>KT(e,JSON.parse(a),r,t==="path"),[e,a,r,t])}function mk(e,t){return eM(e)}function eM(e,t,n,r){tr(Au(),"useRoutes() may be used only in the context of a <Router> component.");let{navigator:a,static:l}=f.useContext(no),{matches:s}=f.useContext(Va),u=s[s.length-1],d=u?u.params:{},g=u?u.pathname:"/",h=u?u.pathnameBase:"/",p=u&&u.route;{let $=p&&p.path||"";tM(g,!p||$.endsWith("*")||$.endsWith("*?"),`You rendered descendant <Routes> (or called \`useRoutes()\`) at "${g}" (under <Route path="${$}">) but the parent route path has no trailing "*". This means if you navigate deeper, the parent won't match anymore and therefore the child routes will never render.

Please change the parent <Route path="${$}"> to <Route path="${$==="/"?"*":`${$}/*`}">.`)}let y=Fo(),C;C=y;let b=C.pathname||"/",x=b;if(h!=="/"){let $=h.replace(/^\//,"").split("/");x="/"+b.replace(/^\//,"").split("/").slice($.length).join("/")}let S=!l&&n&&n.matches&&n.matches.length>0?n.matches:kT(e,{pathname:x});return Ba(p||S!=null,`No routes matched location "${C.pathname}${C.search}${C.hash}" `),Ba(S==null||S[S.length-1].route.element!==void 0||S[S.length-1].route.Component!==void 0||S[S.length-1].route.lazy!==void 0,`Matched leaf route at location "${C.pathname}${C.search}${C.hash}" does not have an element or Component. This means it will render an <Outlet /> with a null value by default resulting in an "empty" page.`),yk(S&&S.map($=>Object.assign({},$,{params:Object.assign({},d,$.params),pathname:wo([h,a.encodeLocation?a.encodeLocation($.pathname).pathname:$.pathname]),pathnameBase:$.pathnameBase==="/"?h:wo([h,a.encodeLocation?a.encodeLocation($.pathnameBase).pathname:$.pathnameBase])})),s,n,r)}function gk(){let e=wk(),t=ik(e)?`${e.status} ${e.statusText}`:e instanceof Error?e.message:JSON.stringify(e),n=e instanceof Error?e.stack:null,r="rgba(200,200,200, 0.5)",a={padding:"0.5rem",backgroundColor:r},l={padding:"2px 4px",backgroundColor:r},s=null;return console.error("Error handled by React Router default ErrorBoundary:",e),s=f.createElement(f.Fragment,null,f.createElement("p",null,"💿 Hey developer 👋"),f.createElement("p",null,"You can provide a way better UX than this when your app throws errors by providing your own ",f.createElement("code",{style:l},"ErrorBoundary")," or"," ",f.createElement("code",{style:l},"errorElement")," prop on your route.")),f.createElement(f.Fragment,null,f.createElement("h2",null,"Unexpected Application Error!"),f.createElement("h3",{style:{fontStyle:"italic"}},t),n?f.createElement("pre",{style:a},n):null,s)}var hk=f.createElement(gk,null),pk=class extends f.Component{constructor(e){super(e),this.state={location:e.location,revalidation:e.revalidation,error:e.error}}static getDerivedStateFromError(e){return{error:e}}static getDerivedStateFromProps(e,t){return t.location!==e.location||t.revalidation!=="idle"&&e.revalidation==="idle"?{error:e.error,location:e.location,revalidation:e.revalidation}:{error:e.error!==void 0?e.error:t.error,location:t.location,revalidation:e.revalidation||t.revalidation}}componentDidCatch(e,t){console.error("React Router caught the following error during render",e,t)}render(){return this.state.error!==void 0?f.createElement(Va.Provider,{value:this.props.routeContext},f.createElement(Ky.Provider,{value:this.state.error,children:this.props.component})):this.props.children}};function vk({routeContext:e,match:t,children:n}){let r=f.useContext(Cs);return r&&r.static&&r.staticContext&&(t.route.errorElement||t.route.ErrorBoundary)&&(r.staticContext._deepestRenderedBoundaryId=t.route.id),f.createElement(Va.Provider,{value:e},n)}function yk(e,t=[],n=null,r=null){if(e==null){if(!n)return null;if(n.errors)e=n.matches;else if(t.length===0&&!n.initialized&&n.matches.length>0)e=n.matches;else return null}let a=e,l=n==null?void 0:n.errors;if(l!=null){let d=a.findIndex(g=>g.route.id&&(l==null?void 0:l[g.route.id])!==void 0);tr(d>=0,`Could not find a matching route for errors on route IDs: ${Object.keys(l).join(",")}`),a=a.slice(0,Math.min(a.length,d+1))}let s=!1,u=-1;if(n)for(let d=0;d<a.length;d++){let g=a[d];if((g.route.HydrateFallback||g.route.hydrateFallbackElement)&&(u=d),g.route.id){let{loaderData:h,errors:p}=n,y=g.route.loader&&!h.hasOwnProperty(g.route.id)&&(!p||p[g.route.id]===void 0);if(g.route.lazy||y){s=!0,u>=0?a=a.slice(0,u+1):a=[a[0]];break}}}return a.reduceRight((d,g,h)=>{let p,y=!1,C=null,b=null;n&&(p=l&&g.route.id?l[g.route.id]:void 0,C=g.route.errorElement||hk,s&&(u<0&&h===0?(tM("route-fallback",!1,"No `HydrateFallback` element provided to render during initial hydration"),y=!0,b=null):u===h&&(y=!0,b=g.route.hydrateFallbackElement||null)));let x=t.concat(a.slice(0,h+1)),S=()=>{let E;return p?E=C:y?E=b:g.route.Component?E=f.createElement(g.route.Component,null):g.route.element?E=g.route.element:E=d,f.createElement(vk,{match:g,routeContext:{outlet:d,matches:x,isDataRoute:n!=null},children:E})};return n&&(g.route.ErrorBoundary||g.route.errorElement||h===0)?f.createElement(pk,{location:n.location,revalidation:n.revalidation,component:C,error:p,children:S(),routeContext:{outlet:null,matches:x,isDataRoute:!0}}):S()},null)}function Yy(e){return`${e} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function bk(e){let t=f.useContext(Cs);return tr(t,Yy(e)),t}function Sk(e){let t=f.useContext(km);return tr(t,Yy(e)),t}function Ck(e){let t=f.useContext(Va);return tr(t,Yy(e)),t}function Qy(e){let t=Ck(e),n=t.matches[t.matches.length-1];return tr(n.route.id,`${e} can only be used on routes that contain a unique "id"`),n.route.id}function xk(){return Qy("useRouteId")}function wk(){var r;let e=f.useContext(Ky),t=Sk("useRouteError"),n=Qy("useRouteError");return e!==void 0?e:(r=t.errors)==null?void 0:r[n]}function Ek(){let{router:e}=bk("useNavigate"),t=Qy("useNavigate"),n=f.useRef(!1);return ZT(()=>{n.current=!0}),f.useCallback(async(a,l={})=>{Ba(n.current,QT),n.current&&(typeof a=="number"?e.navigate(a):await e.navigate(a,{fromRouteId:t,...l}))},[e,t])}var PE={};function tM(e,t,n){!t&&!PE[e]&&(PE[e]=!0,Ba(!1,n))}f.memo($k);function $k({routes:e,future:t,state:n}){return eM(e,void 0,n,t)}function nM(e){return dk(e.context)}function Ok({basename:e="/",children:t=null,location:n,navigationType:r="POP",navigator:a,static:l=!1}){tr(!Au(),"You cannot render a <Router> inside another <Router>. You should never have more than one in your app.");let s=e.replace(/^\/*/,"/"),u=f.useMemo(()=>({basename:s,navigator:a,static:l,future:{}}),[s,a,l]);typeof n=="string"&&(n=Ss(n));let{pathname:d="/",search:g="",hash:h="",state:p=null,key:y="default"}=n,C=f.useMemo(()=>{let b=No(d,s);return b==null?null:{location:{pathname:b,search:g,hash:h,state:p,key:y},navigationType:r}},[s,d,g,h,p,y,r]);return Ba(C!=null,`<Router basename="${s}"> is not able to match the URL "${d}${g}${h}" because it does not start with the basename, so the <Router> won't render anything.`),C==null?null:f.createElement(no.Provider,{value:u},f.createElement(qm.Provider,{children:t,value:C}))}var Cd="get",xd="application/x-www-form-urlencoded";function Wm(e){return e!=null&&typeof e.tagName=="string"}function Rk(e){return Wm(e)&&e.tagName.toLowerCase()==="button"}function _k(e){return Wm(e)&&e.tagName.toLowerCase()==="form"}function Tk(e){return Wm(e)&&e.tagName.toLowerCase()==="input"}function Mk(e){return!!(e.metaKey||e.altKey||e.ctrlKey||e.shiftKey)}function Nk(e,t){return e.button===0&&(!t||t==="_self")&&!Mk(e)}var ad=null;function jk(){if(ad===null)try{new FormData(document.createElement("form"),0),ad=!1}catch{ad=!0}return ad}var Ik=new Set(["application/x-www-form-urlencoded","multipart/form-data","text/plain"]);function tv(e){return e!=null&&!Ik.has(e)?(Ba(!1,`"${e}" is not a valid \`encType\` for \`<Form>\`/\`<fetcher.Form>\` and will default to "${xd}"`),null):e}function Pk(e,t){let n,r,a,l,s;if(_k(e)){let u=e.getAttribute("action");r=u?No(u,t):null,n=e.getAttribute("method")||Cd,a=tv(e.getAttribute("enctype"))||xd,l=new FormData(e)}else if(Rk(e)||Tk(e)&&(e.type==="submit"||e.type==="image")){let u=e.form;if(u==null)throw new Error('Cannot submit a <button> or <input type="submit"> without a <form>');let d=e.getAttribute("formaction")||u.getAttribute("action");if(r=d?No(d,t):null,n=e.getAttribute("formmethod")||u.getAttribute("method")||Cd,a=tv(e.getAttribute("formenctype"))||tv(u.getAttribute("enctype"))||xd,l=new FormData(u,e),!jk()){let{name:g,type:h,value:p}=e;if(h==="image"){let y=g?`${g}.`:"";l.append(`${y}x`,"0"),l.append(`${y}y`,"0")}else g&&l.append(g,p)}}else{if(Wm(e))throw new Error('Cannot submit element that is not <form>, <button>, or <input type="submit|image">');n=Cd,r=null,a=xd,s=e}return l&&a==="text/plain"&&(s=l,l=void 0),{action:r,method:n.toLowerCase(),encType:a,formData:l,body:s}}function Zy(e,t){if(e===!1||e===null||typeof e>"u")throw new Error(t)}async function Ak(e,t){if(e.id in t)return t[e.id];try{let n=await import(e.module);return t[e.id]=n,n}catch(n){return console.error(`Error loading route module \`${e.module}\`, reloading page...`),console.error(n),window.__reactRouterContext&&window.__reactRouterContext.isSpaMode,window.location.reload(),new Promise(()=>{})}}function zk(e){return e==null?!1:e.href==null?e.rel==="preload"&&typeof e.imageSrcSet=="string"&&typeof e.imageSizes=="string":typeof e.rel=="string"&&typeof e.href=="string"}async function Dk(e,t,n){let r=await Promise.all(e.map(async a=>{let l=t.routes[a.route.id];if(l){let s=await Ak(l,n);return s.links?s.links():[]}return[]}));return Fk(r.flat(1).filter(zk).filter(a=>a.rel==="stylesheet"||a.rel==="preload").map(a=>a.rel==="stylesheet"?{...a,rel:"prefetch",as:"style"}:{...a,rel:"prefetch"}))}function AE(e,t,n,r,a,l){let s=(d,g)=>n[g]?d.route.id!==n[g].route.id:!0,u=(d,g)=>{var h;return n[g].pathname!==d.pathname||((h=n[g].route.path)==null?void 0:h.endsWith("*"))&&n[g].params["*"]!==d.params["*"]};return l==="assets"?t.filter((d,g)=>s(d,g)||u(d,g)):l==="data"?t.filter((d,g)=>{var p;let h=r.routes[d.route.id];if(!h||!h.hasLoader)return!1;if(s(d,g)||u(d,g))return!0;if(d.route.shouldRevalidate){let y=d.route.shouldRevalidate({currentUrl:new URL(a.pathname+a.search+a.hash,window.origin),currentParams:((p=n[0])==null?void 0:p.params)||{},nextUrl:new URL(e,window.origin),nextParams:d.params,defaultShouldRevalidate:!0});if(typeof y=="boolean")return y}return!0}):[]}function Lk(e,t,{includeHydrateFallback:n}={}){return Bk(e.map(r=>{let a=t.routes[r.route.id];if(!a)return[];let l=[a.module];return a.clientActionModule&&(l=l.concat(a.clientActionModule)),a.clientLoaderModule&&(l=l.concat(a.clientLoaderModule)),n&&a.hydrateFallbackModule&&(l=l.concat(a.hydrateFallbackModule)),a.imports&&(l=l.concat(a.imports)),l}).flat(1))}function Bk(e){return[...new Set(e)]}function Hk(e){let t={},n=Object.keys(e).sort();for(let r of n)t[r]=e[r];return t}function Fk(e,t){let n=new Set;return new Set(t),e.reduce((r,a)=>{let l=JSON.stringify(Hk(a));return n.has(l)||(n.add(l),r.push({key:l,link:a})),r},[])}var Vk=new Set([100,101,204,205]);function Uk(e,t){let n=typeof e=="string"?new URL(e,typeof window>"u"?"server://singlefetch/":window.location.origin):e;return n.pathname==="/"?n.pathname="_root.data":t&&No(n.pathname,t)==="/"?n.pathname=`${t.replace(/\/$/,"")}/_root.data`:n.pathname=`${n.pathname.replace(/\/$/,"")}.data`,n}function rM(){let e=f.useContext(Cs);return Zy(e,"You must render this element inside a <DataRouterContext.Provider> element"),e}function kk(){let e=f.useContext(km);return Zy(e,"You must render this element inside a <DataRouterStateContext.Provider> element"),e}var Jy=f.createContext(void 0);Jy.displayName="FrameworkContext";function aM(){let e=f.useContext(Jy);return Zy(e,"You must render this element inside a <HydratedRouter> element"),e}function qk(e,t){let n=f.useContext(Jy),[r,a]=f.useState(!1),[l,s]=f.useState(!1),{onFocus:u,onBlur:d,onMouseEnter:g,onMouseLeave:h,onTouchStart:p}=t,y=f.useRef(null);f.useEffect(()=>{if(e==="render"&&s(!0),e==="viewport"){let x=E=>{E.forEach($=>{s($.isIntersecting)})},S=new IntersectionObserver(x,{threshold:.5});return y.current&&S.observe(y.current),()=>{S.disconnect()}}},[e]),f.useEffect(()=>{if(r){let x=setTimeout(()=>{s(!0)},100);return()=>{clearTimeout(x)}}},[r]);let C=()=>{a(!0)},b=()=>{a(!1),s(!1)};return n?e!=="intent"?[l,y,{}]:[l,y,{onFocus:Oc(u,C),onBlur:Oc(d,b),onMouseEnter:Oc(g,C),onMouseLeave:Oc(h,b),onTouchStart:Oc(p,C)}]:[!1,y,{}]}function Oc(e,t){return n=>{e&&e(n),n.defaultPrevented||t(n)}}function Wk({page:e,...t}){let{router:n}=rM(),r=f.useMemo(()=>kT(n.routes,e,n.basename),[n.routes,e,n.basename]);return r?f.createElement(Kk,{page:e,matches:r,...t}):null}function Gk(e){let{manifest:t,routeModules:n}=aM(),[r,a]=f.useState([]);return f.useEffect(()=>{let l=!1;return Dk(e,t,n).then(s=>{l||a(s)}),()=>{l=!0}},[e,t,n]),r}function Kk({page:e,matches:t,...n}){let r=Fo(),{manifest:a,routeModules:l}=aM(),{basename:s}=rM(),{loaderData:u,matches:d}=kk(),g=f.useMemo(()=>AE(e,t,d,a,r,"data"),[e,t,d,a,r]),h=f.useMemo(()=>AE(e,t,d,a,r,"assets"),[e,t,d,a,r]),p=f.useMemo(()=>{if(e===r.pathname+r.search+r.hash)return[];let b=new Set,x=!1;if(t.forEach(E=>{var O;let $=a.routes[E.route.id];!$||!$.hasLoader||(!g.some(_=>_.route.id===E.route.id)&&E.route.id in u&&((O=l[E.route.id])!=null&&O.shouldRevalidate)||$.hasClientLoader?x=!0:b.add(E.route.id))}),b.size===0)return[];let S=Uk(e,s);return x&&b.size>0&&S.searchParams.set("_routes",t.filter(E=>b.has(E.route.id)).map(E=>E.route.id).join(",")),[S.pathname+S.search]},[s,u,r,a,g,t,e,l]),y=f.useMemo(()=>Lk(h,a),[h,a]),C=Gk(h);return f.createElement(f.Fragment,null,p.map(b=>f.createElement("link",{key:b,rel:"prefetch",as:"fetch",href:b,...n})),y.map(b=>f.createElement("link",{key:b,rel:"modulepreload",href:b,...n})),C.map(({key:b,link:x})=>f.createElement("link",{key:b,...x})))}function Xk(...e){return t=>{e.forEach(n=>{typeof n=="function"?n(t):n!=null&&(n.current=t)})}}var oM=typeof window<"u"&&typeof window.document<"u"&&typeof window.document.createElement<"u";try{oM&&(window.__reactRouterVersion="7.5.1")}catch{}function Yk({basename:e,children:t,window:n}){let r=f.useRef();r.current==null&&(r.current=LU({window:n,v5Compat:!0}));let a=r.current,[l,s]=f.useState({action:a.action,location:a.location}),u=f.useCallback(d=>{f.startTransition(()=>s(d))},[s]);return f.useLayoutEffect(()=>a.listen(u),[a,u]),f.createElement(Ok,{basename:e,children:t,location:l.location,navigationType:l.action,navigator:a})}var iM=/^(?:[a-z][a-z0-9+.-]*:|\/\/)/i,lM=f.forwardRef(function({onClick:t,discover:n="render",prefetch:r="none",relative:a,reloadDocument:l,replace:s,state:u,target:d,to:g,preventScrollReset:h,viewTransition:p,...y},C){let{basename:b}=f.useContext(no),x=typeof g=="string"&&iM.test(g),S,E=!1;if(typeof g=="string"&&x&&(S=g,oM))try{let P=new URL(window.location.href),D=g.startsWith("//")?new URL(P.protocol+g):new URL(g),F=No(D.pathname,b);D.origin===P.origin&&F!=null?g=F+D.search+D.hash:E=!0}catch{Ba(!1,`<Link to="${g}"> contains an invalid URL which will probably break when clicked - please update to a valid URL path.`)}let $=uk(g,{relative:a}),[O,_,R]=qk(r,y),T=eq(g,{replace:s,state:u,target:d,preventScrollReset:h,relative:a,viewTransition:p});function M(P){t&&t(P),P.defaultPrevented||T(P)}let z=f.createElement("a",{...y,...R,href:S||$,onClick:E||l?t:M,ref:Xk(C,_),target:d,"data-discover":!x&&n==="render"?"true":void 0});return O&&!x?f.createElement(f.Fragment,null,z,f.createElement(Wk,{page:$})):z});lM.displayName="Link";var Qk=f.forwardRef(function({"aria-current":t="page",caseSensitive:n=!1,className:r="",end:a=!1,style:l,to:s,viewTransition:u,children:d,...g},h){let p=zu(s,{relative:g.relative}),y=Fo(),C=f.useContext(km),{navigator:b,basename:x}=f.useContext(no),S=C!=null&&oq(p)&&u===!0,E=b.encodeLocation?b.encodeLocation(p).pathname:p.pathname,$=y.pathname,O=C&&C.navigation&&C.navigation.location?C.navigation.location.pathname:null;n||($=$.toLowerCase(),O=O?O.toLowerCase():null,E=E.toLowerCase()),O&&x&&(O=No(O,x)||O);const _=E!=="/"&&E.endsWith("/")?E.length-1:E.length;let R=$===E||!a&&$.startsWith(E)&&$.charAt(_)==="/",T=O!=null&&(O===E||!a&&O.startsWith(E)&&O.charAt(E.length)==="/"),M={isActive:R,isPending:T,isTransitioning:S},z=R?t:void 0,P;typeof r=="function"?P=r(M):P=[r,R?"active":null,T?"pending":null,S?"transitioning":null].filter(Boolean).join(" ");let D=typeof l=="function"?l(M):l;return f.createElement(lM,{...g,"aria-current":z,className:P,ref:h,style:D,to:s,viewTransition:u},typeof d=="function"?d(M):d)});Qk.displayName="NavLink";var Zk=f.forwardRef(({discover:e="render",fetcherKey:t,navigate:n,reloadDocument:r,replace:a,state:l,method:s=Cd,action:u,onSubmit:d,relative:g,preventScrollReset:h,viewTransition:p,...y},C)=>{let b=rq(),x=aq(u,{relative:g}),S=s.toLowerCase()==="get"?"get":"post",E=typeof u=="string"&&iM.test(u),$=O=>{if(d&&d(O),O.defaultPrevented)return;O.preventDefault();let _=O.nativeEvent.submitter,R=(_==null?void 0:_.getAttribute("formmethod"))||s;b(_||O.currentTarget,{fetcherKey:t,method:R,navigate:n,replace:a,state:l,relative:g,preventScrollReset:h,viewTransition:p})};return f.createElement("form",{ref:C,method:S,action:x,onSubmit:r?d:$,...y,"data-discover":!E&&e==="render"?"true":void 0})});Zk.displayName="Form";function Jk(e){return`${e} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function sM(e){let t=f.useContext(Cs);return tr(t,Jk(e)),t}function eq(e,{target:t,replace:n,state:r,preventScrollReset:a,relative:l,viewTransition:s}={}){let u=Xy(),d=Fo(),g=zu(e,{relative:l});return f.useCallback(h=>{if(Nk(h,t)){h.preventDefault();let p=n!==void 0?n:hu(d)===hu(g);u(e,{replace:p,state:r,preventScrollReset:a,relative:l,viewTransition:s})}},[d,u,g,n,r,t,e,a,l,s])}var tq=0,nq=()=>`__${String(++tq)}__`;function rq(){let{router:e}=sM("useSubmit"),{basename:t}=f.useContext(no),n=xk();return f.useCallback(async(r,a={})=>{let{action:l,method:s,encType:u,formData:d,body:g}=Pk(r,t);if(a.navigate===!1){let h=a.fetcherKey||nq();await e.fetch(h,n,a.action||l,{preventScrollReset:a.preventScrollReset,formData:d,body:g,formMethod:a.method||s,formEncType:a.encType||u,flushSync:a.flushSync})}else await e.navigate(a.action||l,{preventScrollReset:a.preventScrollReset,formData:d,body:g,formMethod:a.method||s,formEncType:a.encType||u,replace:a.replace,state:a.state,fromRouteId:n,flushSync:a.flushSync,viewTransition:a.viewTransition})},[e,t,n])}function aq(e,{relative:t}={}){let{basename:n}=f.useContext(no),r=f.useContext(Va);tr(r,"useFormAction must be used inside a RouteContext");let[a]=r.matches.slice(-1),l={...zu(e||".",{relative:t})},s=Fo();if(e==null){l.search=s.search;let u=new URLSearchParams(l.search),d=u.getAll("index");if(d.some(h=>h==="")){u.delete("index"),d.filter(p=>p).forEach(p=>u.append("index",p));let h=u.toString();l.search=h?`?${h}`:""}}return(!e||e===".")&&a.route.index&&(l.search=l.search?l.search.replace(/^\?/,"?index&"):"?index"),n!=="/"&&(l.pathname=l.pathname==="/"?n:wo([n,l.pathname])),hu(l)}function oq(e,t={}){let n=f.useContext(YT);tr(n!=null,"`useViewTransitionState` must be used within `react-router-dom`'s `RouterProvider`.  Did you accidentally import `RouterProvider` from `react-router`?");let{basename:r}=sM("useViewTransitionState"),a=zu(e,{relative:t.relative});if(!n.isTransitioning)return!1;let l=No(n.currentLocation.pathname,r)||n.currentLocation.pathname,s=No(n.nextLocation.pathname,r)||n.nextLocation.pathname;return Xd(a.pathname,s)!=null||Xd(a.pathname,l)!=null}new TextEncoder;[...Vk];var nv={exports:{}},rv={};/**
 * @license React
 * use-sync-external-store-with-selector.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var zE;function iq(){if(zE)return rv;zE=1;var e=im();function t(d,g){return d===g&&(d!==0||1/d===1/g)||d!==d&&g!==g}var n=typeof Object.is=="function"?Object.is:t,r=e.useSyncExternalStore,a=e.useRef,l=e.useEffect,s=e.useMemo,u=e.useDebugValue;return rv.useSyncExternalStoreWithSelector=function(d,g,h,p,y){var C=a(null);if(C.current===null){var b={hasValue:!1,value:null};C.current=b}else b=C.current;C=s(function(){function S(R){if(!E){if(E=!0,$=R,R=p(R),y!==void 0&&b.hasValue){var T=b.value;if(y(T,R))return O=T}return O=R}if(T=O,n($,R))return T;var M=p(R);return y!==void 0&&y(T,M)?($=R,T):($=R,O=M)}var E=!1,$,O,_=h===void 0?null:h;return[function(){return S(g())},_===null?void 0:function(){return S(_())}]},[g,h,p,y]);var x=r(d,C[0],C[1]);return l(function(){b.hasValue=!0,b.value=x},[x]),u(x),x},rv}var DE;function lq(){return DE||(DE=1,nv.exports=iq()),nv.exports}var sq=lq();function cq(e){e()}function uq(){let e=null,t=null;return{clear(){e=null,t=null},notify(){cq(()=>{let n=e;for(;n;)n.callback(),n=n.next})},get(){const n=[];let r=e;for(;r;)n.push(r),r=r.next;return n},subscribe(n){let r=!0;const a=t={callback:n,next:null,prev:t};return a.prev?a.prev.next=a:e=a,function(){!r||e===null||(r=!1,a.next?a.next.prev=a.prev:t=a.prev,a.prev?a.prev.next=a.next:e=a.next)}}}}var LE={notify(){},get:()=>[]};function fq(e,t){let n,r=LE,a=0,l=!1;function s(x){h();const S=r.subscribe(x);let E=!1;return()=>{E||(E=!0,S(),p())}}function u(){r.notify()}function d(){b.onStateChange&&b.onStateChange()}function g(){return l}function h(){a++,n||(n=e.subscribe(d),r=uq())}function p(){a--,n&&a===0&&(n(),n=void 0,r.clear(),r=LE)}function y(){l||(l=!0,h())}function C(){l&&(l=!1,p())}const b={addNestedSub:s,notifyNestedSubs:u,handleChangeWrapper:d,isSubscribed:g,trySubscribe:y,tryUnsubscribe:C,getListeners:()=>r};return b}var dq=()=>typeof window<"u"&&typeof window.document<"u"&&typeof window.document.createElement<"u",mq=dq(),gq=()=>typeof navigator<"u"&&navigator.product==="ReactNative",hq=gq(),pq=()=>mq||hq?f.useLayoutEffect:f.useEffect,vq=pq(),av=Symbol.for("react-redux-context"),ov=typeof globalThis<"u"?globalThis:{};function yq(){if(!f.createContext)return{};const e=ov[av]??(ov[av]=new Map);let t=e.get(f.createContext);return t||(t=f.createContext(null),e.set(f.createContext,t)),t}var yi=yq();function bq(e){const{children:t,context:n,serverState:r,store:a}=e,l=f.useMemo(()=>{const d=fq(a);return{store:a,subscription:d,getServerState:r?()=>r:void 0}},[a,r]),s=f.useMemo(()=>a.getState(),[a]);vq(()=>{const{subscription:d}=l;return d.onStateChange=d.notifyNestedSubs,d.trySubscribe(),s!==a.getState()&&d.notifyNestedSubs(),()=>{d.tryUnsubscribe(),d.onStateChange=void 0}},[l,s]);const u=n||yi;return f.createElement(u.Provider,{value:l},t)}var Sq=bq;function eb(e=yi){return function(){return f.useContext(e)}}var cM=eb();function uM(e=yi){const t=e===yi?cM:eb(e),n=()=>{const{store:r}=t();return r};return Object.assign(n,{withTypes:()=>n}),n}var Cq=uM();function xq(e=yi){const t=e===yi?Cq:uM(e),n=()=>t().dispatch;return Object.assign(n,{withTypes:()=>n}),n}var ol=xq(),wq=(e,t)=>e===t;function Eq(e=yi){const t=e===yi?cM:eb(e),n=(r,a={})=>{const{equalityFn:l=wq}=typeof a=="function"?{equalityFn:a}:a,s=t(),{store:u,subscription:d,getServerState:g}=s;f.useRef(!0);const h=f.useCallback({[r.name](y){return r(y)}}[r.name],[r]),p=sq.useSyncExternalStoreWithSelector(d.addNestedSub,u.getState,g||u.getState,h,l);return f.useDebugValue(p),p};return Object.assign(n,{withTypes:()=>n}),n}var Si=Eq();function fM(e,t){return function(){return e.apply(t,arguments)}}const{toString:$q}=Object.prototype,{getPrototypeOf:tb}=Object,Gm=(e=>t=>{const n=$q.call(t);return e[n]||(e[n]=n.slice(8,-1).toLowerCase())})(Object.create(null)),Ua=e=>(e=e.toLowerCase(),t=>Gm(t)===e),Km=e=>t=>typeof t===e,{isArray:xs}=Array,pu=Km("undefined");function Oq(e){return e!==null&&!pu(e)&&e.constructor!==null&&!pu(e.constructor)&&ea(e.constructor.isBuffer)&&e.constructor.isBuffer(e)}const dM=Ua("ArrayBuffer");function Rq(e){let t;return typeof ArrayBuffer<"u"&&ArrayBuffer.isView?t=ArrayBuffer.isView(e):t=e&&e.buffer&&dM(e.buffer),t}const _q=Km("string"),ea=Km("function"),mM=Km("number"),Xm=e=>e!==null&&typeof e=="object",Tq=e=>e===!0||e===!1,wd=e=>{if(Gm(e)!=="object")return!1;const t=tb(e);return(t===null||t===Object.prototype||Object.getPrototypeOf(t)===null)&&!(Symbol.toStringTag in e)&&!(Symbol.iterator in e)},Mq=Ua("Date"),Nq=Ua("File"),jq=Ua("Blob"),Iq=Ua("FileList"),Pq=e=>Xm(e)&&ea(e.pipe),Aq=e=>{let t;return e&&(typeof FormData=="function"&&e instanceof FormData||ea(e.append)&&((t=Gm(e))==="formdata"||t==="object"&&ea(e.toString)&&e.toString()==="[object FormData]"))},zq=Ua("URLSearchParams"),[Dq,Lq,Bq,Hq]=["ReadableStream","Request","Response","Headers"].map(Ua),Fq=e=>e.trim?e.trim():e.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,"");function Du(e,t,{allOwnKeys:n=!1}={}){if(e===null||typeof e>"u")return;let r,a;if(typeof e!="object"&&(e=[e]),xs(e))for(r=0,a=e.length;r<a;r++)t.call(null,e[r],r,e);else{const l=n?Object.getOwnPropertyNames(e):Object.keys(e),s=l.length;let u;for(r=0;r<s;r++)u=l[r],t.call(null,e[u],u,e)}}function gM(e,t){t=t.toLowerCase();const n=Object.keys(e);let r=n.length,a;for(;r-- >0;)if(a=n[r],t===a.toLowerCase())return a;return null}const Gi=typeof globalThis<"u"?globalThis:typeof self<"u"?self:typeof window<"u"?window:global,hM=e=>!pu(e)&&e!==Gi;function O0(){const{caseless:e}=hM(this)&&this||{},t={},n=(r,a)=>{const l=e&&gM(t,a)||a;wd(t[l])&&wd(r)?t[l]=O0(t[l],r):wd(r)?t[l]=O0({},r):xs(r)?t[l]=r.slice():t[l]=r};for(let r=0,a=arguments.length;r<a;r++)arguments[r]&&Du(arguments[r],n);return t}const Vq=(e,t,n,{allOwnKeys:r}={})=>(Du(t,(a,l)=>{n&&ea(a)?e[l]=fM(a,n):e[l]=a},{allOwnKeys:r}),e),Uq=e=>(e.charCodeAt(0)===65279&&(e=e.slice(1)),e),kq=(e,t,n,r)=>{e.prototype=Object.create(t.prototype,r),e.prototype.constructor=e,Object.defineProperty(e,"super",{value:t.prototype}),n&&Object.assign(e.prototype,n)},qq=(e,t,n,r)=>{let a,l,s;const u={};if(t=t||{},e==null)return t;do{for(a=Object.getOwnPropertyNames(e),l=a.length;l-- >0;)s=a[l],(!r||r(s,e,t))&&!u[s]&&(t[s]=e[s],u[s]=!0);e=n!==!1&&tb(e)}while(e&&(!n||n(e,t))&&e!==Object.prototype);return t},Wq=(e,t,n)=>{e=String(e),(n===void 0||n>e.length)&&(n=e.length),n-=t.length;const r=e.indexOf(t,n);return r!==-1&&r===n},Gq=e=>{if(!e)return null;if(xs(e))return e;let t=e.length;if(!mM(t))return null;const n=new Array(t);for(;t-- >0;)n[t]=e[t];return n},Kq=(e=>t=>e&&t instanceof e)(typeof Uint8Array<"u"&&tb(Uint8Array)),Xq=(e,t)=>{const r=(e&&e[Symbol.iterator]).call(e);let a;for(;(a=r.next())&&!a.done;){const l=a.value;t.call(e,l[0],l[1])}},Yq=(e,t)=>{let n;const r=[];for(;(n=e.exec(t))!==null;)r.push(n);return r},Qq=Ua("HTMLFormElement"),Zq=e=>e.toLowerCase().replace(/[-_\s]([a-z\d])(\w*)/g,function(n,r,a){return r.toUpperCase()+a}),BE=(({hasOwnProperty:e})=>(t,n)=>e.call(t,n))(Object.prototype),Jq=Ua("RegExp"),pM=(e,t)=>{const n=Object.getOwnPropertyDescriptors(e),r={};Du(n,(a,l)=>{let s;(s=t(a,l,e))!==!1&&(r[l]=s||a)}),Object.defineProperties(e,r)},eW=e=>{pM(e,(t,n)=>{if(ea(e)&&["arguments","caller","callee"].indexOf(n)!==-1)return!1;const r=e[n];if(ea(r)){if(t.enumerable=!1,"writable"in t){t.writable=!1;return}t.set||(t.set=()=>{throw Error("Can not rewrite read-only method '"+n+"'")})}})},tW=(e,t)=>{const n={},r=a=>{a.forEach(l=>{n[l]=!0})};return xs(e)?r(e):r(String(e).split(t)),n},nW=()=>{},rW=(e,t)=>e!=null&&Number.isFinite(e=+e)?e:t;function aW(e){return!!(e&&ea(e.append)&&e[Symbol.toStringTag]==="FormData"&&e[Symbol.iterator])}const oW=e=>{const t=new Array(10),n=(r,a)=>{if(Xm(r)){if(t.indexOf(r)>=0)return;if(!("toJSON"in r)){t[a]=r;const l=xs(r)?[]:{};return Du(r,(s,u)=>{const d=n(s,a+1);!pu(d)&&(l[u]=d)}),t[a]=void 0,l}}return r};return n(e,0)},iW=Ua("AsyncFunction"),lW=e=>e&&(Xm(e)||ea(e))&&ea(e.then)&&ea(e.catch),vM=((e,t)=>e?setImmediate:t?((n,r)=>(Gi.addEventListener("message",({source:a,data:l})=>{a===Gi&&l===n&&r.length&&r.shift()()},!1),a=>{r.push(a),Gi.postMessage(n,"*")}))(`axios@${Math.random()}`,[]):n=>setTimeout(n))(typeof setImmediate=="function",ea(Gi.postMessage)),sW=typeof queueMicrotask<"u"?queueMicrotask.bind(Gi):typeof process<"u"&&process.nextTick||vM,Ce={isArray:xs,isArrayBuffer:dM,isBuffer:Oq,isFormData:Aq,isArrayBufferView:Rq,isString:_q,isNumber:mM,isBoolean:Tq,isObject:Xm,isPlainObject:wd,isReadableStream:Dq,isRequest:Lq,isResponse:Bq,isHeaders:Hq,isUndefined:pu,isDate:Mq,isFile:Nq,isBlob:jq,isRegExp:Jq,isFunction:ea,isStream:Pq,isURLSearchParams:zq,isTypedArray:Kq,isFileList:Iq,forEach:Du,merge:O0,extend:Vq,trim:Fq,stripBOM:Uq,inherits:kq,toFlatObject:qq,kindOf:Gm,kindOfTest:Ua,endsWith:Wq,toArray:Gq,forEachEntry:Xq,matchAll:Yq,isHTMLForm:Qq,hasOwnProperty:BE,hasOwnProp:BE,reduceDescriptors:pM,freezeMethods:eW,toObjectSet:tW,toCamelCase:Zq,noop:nW,toFiniteNumber:rW,findKey:gM,global:Gi,isContextDefined:hM,isSpecCompliantForm:aW,toJSONObject:oW,isAsyncFn:iW,isThenable:lW,setImmediate:vM,asap:sW};function Rt(e,t,n,r,a){Error.call(this),Error.captureStackTrace?Error.captureStackTrace(this,this.constructor):this.stack=new Error().stack,this.message=e,this.name="AxiosError",t&&(this.code=t),n&&(this.config=n),r&&(this.request=r),a&&(this.response=a,this.status=a.status?a.status:null)}Ce.inherits(Rt,Error,{toJSON:function(){return{message:this.message,name:this.name,description:this.description,number:this.number,fileName:this.fileName,lineNumber:this.lineNumber,columnNumber:this.columnNumber,stack:this.stack,config:Ce.toJSONObject(this.config),code:this.code,status:this.status}}});const yM=Rt.prototype,bM={};["ERR_BAD_OPTION_VALUE","ERR_BAD_OPTION","ECONNABORTED","ETIMEDOUT","ERR_NETWORK","ERR_FR_TOO_MANY_REDIRECTS","ERR_DEPRECATED","ERR_BAD_RESPONSE","ERR_BAD_REQUEST","ERR_CANCELED","ERR_NOT_SUPPORT","ERR_INVALID_URL"].forEach(e=>{bM[e]={value:e}});Object.defineProperties(Rt,bM);Object.defineProperty(yM,"isAxiosError",{value:!0});Rt.from=(e,t,n,r,a,l)=>{const s=Object.create(yM);return Ce.toFlatObject(e,s,function(d){return d!==Error.prototype},u=>u!=="isAxiosError"),Rt.call(s,e.message,t,n,r,a),s.cause=e,s.name=e.name,l&&Object.assign(s,l),s};const cW=null;function R0(e){return Ce.isPlainObject(e)||Ce.isArray(e)}function SM(e){return Ce.endsWith(e,"[]")?e.slice(0,-2):e}function HE(e,t,n){return e?e.concat(t).map(function(a,l){return a=SM(a),!n&&l?"["+a+"]":a}).join(n?".":""):t}function uW(e){return Ce.isArray(e)&&!e.some(R0)}const fW=Ce.toFlatObject(Ce,{},null,function(t){return/^is[A-Z]/.test(t)});function Ym(e,t,n){if(!Ce.isObject(e))throw new TypeError("target must be an object");t=t||new FormData,n=Ce.toFlatObject(n,{metaTokens:!0,dots:!1,indexes:!1},!1,function(x,S){return!Ce.isUndefined(S[x])});const r=n.metaTokens,a=n.visitor||h,l=n.dots,s=n.indexes,d=(n.Blob||typeof Blob<"u"&&Blob)&&Ce.isSpecCompliantForm(t);if(!Ce.isFunction(a))throw new TypeError("visitor must be a function");function g(b){if(b===null)return"";if(Ce.isDate(b))return b.toISOString();if(!d&&Ce.isBlob(b))throw new Rt("Blob is not supported. Use a Buffer instead.");return Ce.isArrayBuffer(b)||Ce.isTypedArray(b)?d&&typeof Blob=="function"?new Blob([b]):Buffer.from(b):b}function h(b,x,S){let E=b;if(b&&!S&&typeof b=="object"){if(Ce.endsWith(x,"{}"))x=r?x:x.slice(0,-2),b=JSON.stringify(b);else if(Ce.isArray(b)&&uW(b)||(Ce.isFileList(b)||Ce.endsWith(x,"[]"))&&(E=Ce.toArray(b)))return x=SM(x),E.forEach(function(O,_){!(Ce.isUndefined(O)||O===null)&&t.append(s===!0?HE([x],_,l):s===null?x:x+"[]",g(O))}),!1}return R0(b)?!0:(t.append(HE(S,x,l),g(b)),!1)}const p=[],y=Object.assign(fW,{defaultVisitor:h,convertValue:g,isVisitable:R0});function C(b,x){if(!Ce.isUndefined(b)){if(p.indexOf(b)!==-1)throw Error("Circular reference detected in "+x.join("."));p.push(b),Ce.forEach(b,function(E,$){(!(Ce.isUndefined(E)||E===null)&&a.call(t,E,Ce.isString($)?$.trim():$,x,y))===!0&&C(E,x?x.concat($):[$])}),p.pop()}}if(!Ce.isObject(e))throw new TypeError("data must be an object");return C(e),t}function FE(e){const t={"!":"%21","'":"%27","(":"%28",")":"%29","~":"%7E","%20":"+","%00":"\0"};return encodeURIComponent(e).replace(/[!'()~]|%20|%00/g,function(r){return t[r]})}function nb(e,t){this._pairs=[],e&&Ym(e,this,t)}const CM=nb.prototype;CM.append=function(t,n){this._pairs.push([t,n])};CM.toString=function(t){const n=t?function(r){return t.call(this,r,FE)}:FE;return this._pairs.map(function(a){return n(a[0])+"="+n(a[1])},"").join("&")};function dW(e){return encodeURIComponent(e).replace(/%3A/gi,":").replace(/%24/g,"$").replace(/%2C/gi,",").replace(/%20/g,"+").replace(/%5B/gi,"[").replace(/%5D/gi,"]")}function xM(e,t,n){if(!t)return e;const r=n&&n.encode||dW;Ce.isFunction(n)&&(n={serialize:n});const a=n&&n.serialize;let l;if(a?l=a(t,n):l=Ce.isURLSearchParams(t)?t.toString():new nb(t,n).toString(r),l){const s=e.indexOf("#");s!==-1&&(e=e.slice(0,s)),e+=(e.indexOf("?")===-1?"?":"&")+l}return e}class VE{constructor(){this.handlers=[]}use(t,n,r){return this.handlers.push({fulfilled:t,rejected:n,synchronous:r?r.synchronous:!1,runWhen:r?r.runWhen:null}),this.handlers.length-1}eject(t){this.handlers[t]&&(this.handlers[t]=null)}clear(){this.handlers&&(this.handlers=[])}forEach(t){Ce.forEach(this.handlers,function(r){r!==null&&t(r)})}}const wM={silentJSONParsing:!0,forcedJSONParsing:!0,clarifyTimeoutError:!1},mW=typeof URLSearchParams<"u"?URLSearchParams:nb,gW=typeof FormData<"u"?FormData:null,hW=typeof Blob<"u"?Blob:null,pW={isBrowser:!0,classes:{URLSearchParams:mW,FormData:gW,Blob:hW},protocols:["http","https","file","blob","url","data"]},rb=typeof window<"u"&&typeof document<"u",_0=typeof navigator=="object"&&navigator||void 0,vW=rb&&(!_0||["ReactNative","NativeScript","NS"].indexOf(_0.product)<0),yW=typeof WorkerGlobalScope<"u"&&self instanceof WorkerGlobalScope&&typeof self.importScripts=="function",bW=rb&&window.location.href||"http://localhost",SW=Object.freeze(Object.defineProperty({__proto__:null,hasBrowserEnv:rb,hasStandardBrowserEnv:vW,hasStandardBrowserWebWorkerEnv:yW,navigator:_0,origin:bW},Symbol.toStringTag,{value:"Module"})),Er={...SW,...pW};function CW(e,t){return Ym(e,new Er.classes.URLSearchParams,Object.assign({visitor:function(n,r,a,l){return Er.isNode&&Ce.isBuffer(n)?(this.append(r,n.toString("base64")),!1):l.defaultVisitor.apply(this,arguments)}},t))}function xW(e){return Ce.matchAll(/\w+|\[(\w*)]/g,e).map(t=>t[0]==="[]"?"":t[1]||t[0])}function wW(e){const t={},n=Object.keys(e);let r;const a=n.length;let l;for(r=0;r<a;r++)l=n[r],t[l]=e[l];return t}function EM(e){function t(n,r,a,l){let s=n[l++];if(s==="__proto__")return!0;const u=Number.isFinite(+s),d=l>=n.length;return s=!s&&Ce.isArray(a)?a.length:s,d?(Ce.hasOwnProp(a,s)?a[s]=[a[s],r]:a[s]=r,!u):((!a[s]||!Ce.isObject(a[s]))&&(a[s]=[]),t(n,r,a[s],l)&&Ce.isArray(a[s])&&(a[s]=wW(a[s])),!u)}if(Ce.isFormData(e)&&Ce.isFunction(e.entries)){const n={};return Ce.forEachEntry(e,(r,a)=>{t(xW(r),a,n,0)}),n}return null}function EW(e,t,n){if(Ce.isString(e))try{return(t||JSON.parse)(e),Ce.trim(e)}catch(r){if(r.name!=="SyntaxError")throw r}return(n||JSON.stringify)(e)}const Lu={transitional:wM,adapter:["xhr","http","fetch"],transformRequest:[function(t,n){const r=n.getContentType()||"",a=r.indexOf("application/json")>-1,l=Ce.isObject(t);if(l&&Ce.isHTMLForm(t)&&(t=new FormData(t)),Ce.isFormData(t))return a?JSON.stringify(EM(t)):t;if(Ce.isArrayBuffer(t)||Ce.isBuffer(t)||Ce.isStream(t)||Ce.isFile(t)||Ce.isBlob(t)||Ce.isReadableStream(t))return t;if(Ce.isArrayBufferView(t))return t.buffer;if(Ce.isURLSearchParams(t))return n.setContentType("application/x-www-form-urlencoded;charset=utf-8",!1),t.toString();let u;if(l){if(r.indexOf("application/x-www-form-urlencoded")>-1)return CW(t,this.formSerializer).toString();if((u=Ce.isFileList(t))||r.indexOf("multipart/form-data")>-1){const d=this.env&&this.env.FormData;return Ym(u?{"files[]":t}:t,d&&new d,this.formSerializer)}}return l||a?(n.setContentType("application/json",!1),EW(t)):t}],transformResponse:[function(t){const n=this.transitional||Lu.transitional,r=n&&n.forcedJSONParsing,a=this.responseType==="json";if(Ce.isResponse(t)||Ce.isReadableStream(t))return t;if(t&&Ce.isString(t)&&(r&&!this.responseType||a)){const s=!(n&&n.silentJSONParsing)&&a;try{return JSON.parse(t)}catch(u){if(s)throw u.name==="SyntaxError"?Rt.from(u,Rt.ERR_BAD_RESPONSE,this,null,this.response):u}}return t}],timeout:0,xsrfCookieName:"XSRF-TOKEN",xsrfHeaderName:"X-XSRF-TOKEN",maxContentLength:-1,maxBodyLength:-1,env:{FormData:Er.classes.FormData,Blob:Er.classes.Blob},validateStatus:function(t){return t>=200&&t<300},headers:{common:{Accept:"application/json, text/plain, */*","Content-Type":void 0}}};Ce.forEach(["delete","get","head","post","put","patch"],e=>{Lu.headers[e]={}});const $W=Ce.toObjectSet(["age","authorization","content-length","content-type","etag","expires","from","host","if-modified-since","if-unmodified-since","last-modified","location","max-forwards","proxy-authorization","referer","retry-after","user-agent"]),OW=e=>{const t={};let n,r,a;return e&&e.split(`
`).forEach(function(s){a=s.indexOf(":"),n=s.substring(0,a).trim().toLowerCase(),r=s.substring(a+1).trim(),!(!n||t[n]&&$W[n])&&(n==="set-cookie"?t[n]?t[n].push(r):t[n]=[r]:t[n]=t[n]?t[n]+", "+r:r)}),t},UE=Symbol("internals");function Rc(e){return e&&String(e).trim().toLowerCase()}function Ed(e){return e===!1||e==null?e:Ce.isArray(e)?e.map(Ed):String(e)}function RW(e){const t=Object.create(null),n=/([^\s,;=]+)\s*(?:=\s*([^,;]+))?/g;let r;for(;r=n.exec(e);)t[r[1]]=r[2];return t}const _W=e=>/^[-_a-zA-Z0-9^`|~,!#$%&'*+.]+$/.test(e.trim());function iv(e,t,n,r,a){if(Ce.isFunction(r))return r.call(this,t,n);if(a&&(t=n),!!Ce.isString(t)){if(Ce.isString(r))return t.indexOf(r)!==-1;if(Ce.isRegExp(r))return r.test(t)}}function TW(e){return e.trim().toLowerCase().replace(/([a-z\d])(\w*)/g,(t,n,r)=>n.toUpperCase()+r)}function MW(e,t){const n=Ce.toCamelCase(" "+t);["get","set","has"].forEach(r=>{Object.defineProperty(e,r+n,{value:function(a,l,s){return this[r].call(this,t,a,l,s)},configurable:!0})})}let Dr=class{constructor(t){t&&this.set(t)}set(t,n,r){const a=this;function l(u,d,g){const h=Rc(d);if(!h)throw new Error("header name must be a non-empty string");const p=Ce.findKey(a,h);(!p||a[p]===void 0||g===!0||g===void 0&&a[p]!==!1)&&(a[p||d]=Ed(u))}const s=(u,d)=>Ce.forEach(u,(g,h)=>l(g,h,d));if(Ce.isPlainObject(t)||t instanceof this.constructor)s(t,n);else if(Ce.isString(t)&&(t=t.trim())&&!_W(t))s(OW(t),n);else if(Ce.isHeaders(t))for(const[u,d]of t.entries())l(d,u,r);else t!=null&&l(n,t,r);return this}get(t,n){if(t=Rc(t),t){const r=Ce.findKey(this,t);if(r){const a=this[r];if(!n)return a;if(n===!0)return RW(a);if(Ce.isFunction(n))return n.call(this,a,r);if(Ce.isRegExp(n))return n.exec(a);throw new TypeError("parser must be boolean|regexp|function")}}}has(t,n){if(t=Rc(t),t){const r=Ce.findKey(this,t);return!!(r&&this[r]!==void 0&&(!n||iv(this,this[r],r,n)))}return!1}delete(t,n){const r=this;let a=!1;function l(s){if(s=Rc(s),s){const u=Ce.findKey(r,s);u&&(!n||iv(r,r[u],u,n))&&(delete r[u],a=!0)}}return Ce.isArray(t)?t.forEach(l):l(t),a}clear(t){const n=Object.keys(this);let r=n.length,a=!1;for(;r--;){const l=n[r];(!t||iv(this,this[l],l,t,!0))&&(delete this[l],a=!0)}return a}normalize(t){const n=this,r={};return Ce.forEach(this,(a,l)=>{const s=Ce.findKey(r,l);if(s){n[s]=Ed(a),delete n[l];return}const u=t?TW(l):String(l).trim();u!==l&&delete n[l],n[u]=Ed(a),r[u]=!0}),this}concat(...t){return this.constructor.concat(this,...t)}toJSON(t){const n=Object.create(null);return Ce.forEach(this,(r,a)=>{r!=null&&r!==!1&&(n[a]=t&&Ce.isArray(r)?r.join(", "):r)}),n}[Symbol.iterator](){return Object.entries(this.toJSON())[Symbol.iterator]()}toString(){return Object.entries(this.toJSON()).map(([t,n])=>t+": "+n).join(`
`)}get[Symbol.toStringTag](){return"AxiosHeaders"}static from(t){return t instanceof this?t:new this(t)}static concat(t,...n){const r=new this(t);return n.forEach(a=>r.set(a)),r}static accessor(t){const r=(this[UE]=this[UE]={accessors:{}}).accessors,a=this.prototype;function l(s){const u=Rc(s);r[u]||(MW(a,s),r[u]=!0)}return Ce.isArray(t)?t.forEach(l):l(t),this}};Dr.accessor(["Content-Type","Content-Length","Accept","Accept-Encoding","User-Agent","Authorization"]);Ce.reduceDescriptors(Dr.prototype,({value:e},t)=>{let n=t[0].toUpperCase()+t.slice(1);return{get:()=>e,set(r){this[n]=r}}});Ce.freezeMethods(Dr);function lv(e,t){const n=this||Lu,r=t||n,a=Dr.from(r.headers);let l=r.data;return Ce.forEach(e,function(u){l=u.call(n,l,a.normalize(),t?t.status:void 0)}),a.normalize(),l}function $M(e){return!!(e&&e.__CANCEL__)}function ws(e,t,n){Rt.call(this,e??"canceled",Rt.ERR_CANCELED,t,n),this.name="CanceledError"}Ce.inherits(ws,Rt,{__CANCEL__:!0});function OM(e,t,n){const r=n.config.validateStatus;!n.status||!r||r(n.status)?e(n):t(new Rt("Request failed with status code "+n.status,[Rt.ERR_BAD_REQUEST,Rt.ERR_BAD_RESPONSE][Math.floor(n.status/100)-4],n.config,n.request,n))}function NW(e){const t=/^([-+\w]{1,25})(:?\/\/|:)/.exec(e);return t&&t[1]||""}function jW(e,t){e=e||10;const n=new Array(e),r=new Array(e);let a=0,l=0,s;return t=t!==void 0?t:1e3,function(d){const g=Date.now(),h=r[l];s||(s=g),n[a]=d,r[a]=g;let p=l,y=0;for(;p!==a;)y+=n[p++],p=p%e;if(a=(a+1)%e,a===l&&(l=(l+1)%e),g-s<t)return;const C=h&&g-h;return C?Math.round(y*1e3/C):void 0}}function IW(e,t){let n=0,r=1e3/t,a,l;const s=(g,h=Date.now())=>{n=h,a=null,l&&(clearTimeout(l),l=null),e.apply(null,g)};return[(...g)=>{const h=Date.now(),p=h-n;p>=r?s(g,h):(a=g,l||(l=setTimeout(()=>{l=null,s(a)},r-p)))},()=>a&&s(a)]}const Yd=(e,t,n=3)=>{let r=0;const a=jW(50,250);return IW(l=>{const s=l.loaded,u=l.lengthComputable?l.total:void 0,d=s-r,g=a(d),h=s<=u;r=s;const p={loaded:s,total:u,progress:u?s/u:void 0,bytes:d,rate:g||void 0,estimated:g&&u&&h?(u-s)/g:void 0,event:l,lengthComputable:u!=null,[t?"download":"upload"]:!0};e(p)},n)},kE=(e,t)=>{const n=e!=null;return[r=>t[0]({lengthComputable:n,total:e,loaded:r}),t[1]]},qE=e=>(...t)=>Ce.asap(()=>e(...t)),PW=Er.hasStandardBrowserEnv?((e,t)=>n=>(n=new URL(n,Er.origin),e.protocol===n.protocol&&e.host===n.host&&(t||e.port===n.port)))(new URL(Er.origin),Er.navigator&&/(msie|trident)/i.test(Er.navigator.userAgent)):()=>!0,AW=Er.hasStandardBrowserEnv?{write(e,t,n,r,a,l){const s=[e+"="+encodeURIComponent(t)];Ce.isNumber(n)&&s.push("expires="+new Date(n).toGMTString()),Ce.isString(r)&&s.push("path="+r),Ce.isString(a)&&s.push("domain="+a),l===!0&&s.push("secure"),document.cookie=s.join("; ")},read(e){const t=document.cookie.match(new RegExp("(^|;\\s*)("+e+")=([^;]*)"));return t?decodeURIComponent(t[3]):null},remove(e){this.write(e,"",Date.now()-864e5)}}:{write(){},read(){return null},remove(){}};function zW(e){return/^([a-z][a-z\d+\-.]*:)?\/\//i.test(e)}function DW(e,t){return t?e.replace(/\/?\/$/,"")+"/"+t.replace(/^\/+/,""):e}function RM(e,t,n){let r=!zW(t);return e&&(r||n==!1)?DW(e,t):t}const WE=e=>e instanceof Dr?{...e}:e;function el(e,t){t=t||{};const n={};function r(g,h,p,y){return Ce.isPlainObject(g)&&Ce.isPlainObject(h)?Ce.merge.call({caseless:y},g,h):Ce.isPlainObject(h)?Ce.merge({},h):Ce.isArray(h)?h.slice():h}function a(g,h,p,y){if(Ce.isUndefined(h)){if(!Ce.isUndefined(g))return r(void 0,g,p,y)}else return r(g,h,p,y)}function l(g,h){if(!Ce.isUndefined(h))return r(void 0,h)}function s(g,h){if(Ce.isUndefined(h)){if(!Ce.isUndefined(g))return r(void 0,g)}else return r(void 0,h)}function u(g,h,p){if(p in t)return r(g,h);if(p in e)return r(void 0,g)}const d={url:l,method:l,data:l,baseURL:s,transformRequest:s,transformResponse:s,paramsSerializer:s,timeout:s,timeoutMessage:s,withCredentials:s,withXSRFToken:s,adapter:s,responseType:s,xsrfCookieName:s,xsrfHeaderName:s,onUploadProgress:s,onDownloadProgress:s,decompress:s,maxContentLength:s,maxBodyLength:s,beforeRedirect:s,transport:s,httpAgent:s,httpsAgent:s,cancelToken:s,socketPath:s,responseEncoding:s,validateStatus:u,headers:(g,h,p)=>a(WE(g),WE(h),p,!0)};return Ce.forEach(Object.keys(Object.assign({},e,t)),function(h){const p=d[h]||a,y=p(e[h],t[h],h);Ce.isUndefined(y)&&p!==u||(n[h]=y)}),n}const _M=e=>{const t=el({},e);let{data:n,withXSRFToken:r,xsrfHeaderName:a,xsrfCookieName:l,headers:s,auth:u}=t;t.headers=s=Dr.from(s),t.url=xM(RM(t.baseURL,t.url,t.allowAbsoluteUrls),e.params,e.paramsSerializer),u&&s.set("Authorization","Basic "+btoa((u.username||"")+":"+(u.password?unescape(encodeURIComponent(u.password)):"")));let d;if(Ce.isFormData(n)){if(Er.hasStandardBrowserEnv||Er.hasStandardBrowserWebWorkerEnv)s.setContentType(void 0);else if((d=s.getContentType())!==!1){const[g,...h]=d?d.split(";").map(p=>p.trim()).filter(Boolean):[];s.setContentType([g||"multipart/form-data",...h].join("; "))}}if(Er.hasStandardBrowserEnv&&(r&&Ce.isFunction(r)&&(r=r(t)),r||r!==!1&&PW(t.url))){const g=a&&l&&AW.read(l);g&&s.set(a,g)}return t},LW=typeof XMLHttpRequest<"u",BW=LW&&function(e){return new Promise(function(n,r){const a=_M(e);let l=a.data;const s=Dr.from(a.headers).normalize();let{responseType:u,onUploadProgress:d,onDownloadProgress:g}=a,h,p,y,C,b;function x(){C&&C(),b&&b(),a.cancelToken&&a.cancelToken.unsubscribe(h),a.signal&&a.signal.removeEventListener("abort",h)}let S=new XMLHttpRequest;S.open(a.method.toUpperCase(),a.url,!0),S.timeout=a.timeout;function E(){if(!S)return;const O=Dr.from("getAllResponseHeaders"in S&&S.getAllResponseHeaders()),R={data:!u||u==="text"||u==="json"?S.responseText:S.response,status:S.status,statusText:S.statusText,headers:O,config:e,request:S};OM(function(M){n(M),x()},function(M){r(M),x()},R),S=null}"onloadend"in S?S.onloadend=E:S.onreadystatechange=function(){!S||S.readyState!==4||S.status===0&&!(S.responseURL&&S.responseURL.indexOf("file:")===0)||setTimeout(E)},S.onabort=function(){S&&(r(new Rt("Request aborted",Rt.ECONNABORTED,e,S)),S=null)},S.onerror=function(){r(new Rt("Network Error",Rt.ERR_NETWORK,e,S)),S=null},S.ontimeout=function(){let _=a.timeout?"timeout of "+a.timeout+"ms exceeded":"timeout exceeded";const R=a.transitional||wM;a.timeoutErrorMessage&&(_=a.timeoutErrorMessage),r(new Rt(_,R.clarifyTimeoutError?Rt.ETIMEDOUT:Rt.ECONNABORTED,e,S)),S=null},l===void 0&&s.setContentType(null),"setRequestHeader"in S&&Ce.forEach(s.toJSON(),function(_,R){S.setRequestHeader(R,_)}),Ce.isUndefined(a.withCredentials)||(S.withCredentials=!!a.withCredentials),u&&u!=="json"&&(S.responseType=a.responseType),g&&([y,b]=Yd(g,!0),S.addEventListener("progress",y)),d&&S.upload&&([p,C]=Yd(d),S.upload.addEventListener("progress",p),S.upload.addEventListener("loadend",C)),(a.cancelToken||a.signal)&&(h=O=>{S&&(r(!O||O.type?new ws(null,e,S):O),S.abort(),S=null)},a.cancelToken&&a.cancelToken.subscribe(h),a.signal&&(a.signal.aborted?h():a.signal.addEventListener("abort",h)));const $=NW(a.url);if($&&Er.protocols.indexOf($)===-1){r(new Rt("Unsupported protocol "+$+":",Rt.ERR_BAD_REQUEST,e));return}S.send(l||null)})},HW=(e,t)=>{const{length:n}=e=e?e.filter(Boolean):[];if(t||n){let r=new AbortController,a;const l=function(g){if(!a){a=!0,u();const h=g instanceof Error?g:this.reason;r.abort(h instanceof Rt?h:new ws(h instanceof Error?h.message:h))}};let s=t&&setTimeout(()=>{s=null,l(new Rt(`timeout ${t} of ms exceeded`,Rt.ETIMEDOUT))},t);const u=()=>{e&&(s&&clearTimeout(s),s=null,e.forEach(g=>{g.unsubscribe?g.unsubscribe(l):g.removeEventListener("abort",l)}),e=null)};e.forEach(g=>g.addEventListener("abort",l));const{signal:d}=r;return d.unsubscribe=()=>Ce.asap(u),d}},FW=function*(e,t){let n=e.byteLength;if(n<t){yield e;return}let r=0,a;for(;r<n;)a=r+t,yield e.slice(r,a),r=a},VW=async function*(e,t){for await(const n of UW(e))yield*FW(n,t)},UW=async function*(e){if(e[Symbol.asyncIterator]){yield*e;return}const t=e.getReader();try{for(;;){const{done:n,value:r}=await t.read();if(n)break;yield r}}finally{await t.cancel()}},GE=(e,t,n,r)=>{const a=VW(e,t);let l=0,s,u=d=>{s||(s=!0,r&&r(d))};return new ReadableStream({async pull(d){try{const{done:g,value:h}=await a.next();if(g){u(),d.close();return}let p=h.byteLength;if(n){let y=l+=p;n(y)}d.enqueue(new Uint8Array(h))}catch(g){throw u(g),g}},cancel(d){return u(d),a.return()}},{highWaterMark:2})},Qm=typeof fetch=="function"&&typeof Request=="function"&&typeof Response=="function",TM=Qm&&typeof ReadableStream=="function",kW=Qm&&(typeof TextEncoder=="function"?(e=>t=>e.encode(t))(new TextEncoder):async e=>new Uint8Array(await new Response(e).arrayBuffer())),MM=(e,...t)=>{try{return!!e(...t)}catch{return!1}},qW=TM&&MM(()=>{let e=!1;const t=new Request(Er.origin,{body:new ReadableStream,method:"POST",get duplex(){return e=!0,"half"}}).headers.has("Content-Type");return e&&!t}),KE=64*1024,T0=TM&&MM(()=>Ce.isReadableStream(new Response("").body)),Qd={stream:T0&&(e=>e.body)};Qm&&(e=>{["text","arrayBuffer","blob","formData","stream"].forEach(t=>{!Qd[t]&&(Qd[t]=Ce.isFunction(e[t])?n=>n[t]():(n,r)=>{throw new Rt(`Response type '${t}' is not supported`,Rt.ERR_NOT_SUPPORT,r)})})})(new Response);const WW=async e=>{if(e==null)return 0;if(Ce.isBlob(e))return e.size;if(Ce.isSpecCompliantForm(e))return(await new Request(Er.origin,{method:"POST",body:e}).arrayBuffer()).byteLength;if(Ce.isArrayBufferView(e)||Ce.isArrayBuffer(e))return e.byteLength;if(Ce.isURLSearchParams(e)&&(e=e+""),Ce.isString(e))return(await kW(e)).byteLength},GW=async(e,t)=>{const n=Ce.toFiniteNumber(e.getContentLength());return n??WW(t)},KW=Qm&&(async e=>{let{url:t,method:n,data:r,signal:a,cancelToken:l,timeout:s,onDownloadProgress:u,onUploadProgress:d,responseType:g,headers:h,withCredentials:p="same-origin",fetchOptions:y}=_M(e);g=g?(g+"").toLowerCase():"text";let C=HW([a,l&&l.toAbortSignal()],s),b;const x=C&&C.unsubscribe&&(()=>{C.unsubscribe()});let S;try{if(d&&qW&&n!=="get"&&n!=="head"&&(S=await GW(h,r))!==0){let R=new Request(t,{method:"POST",body:r,duplex:"half"}),T;if(Ce.isFormData(r)&&(T=R.headers.get("content-type"))&&h.setContentType(T),R.body){const[M,z]=kE(S,Yd(qE(d)));r=GE(R.body,KE,M,z)}}Ce.isString(p)||(p=p?"include":"omit");const E="credentials"in Request.prototype;b=new Request(t,{...y,signal:C,method:n.toUpperCase(),headers:h.normalize().toJSON(),body:r,duplex:"half",credentials:E?p:void 0});let $=await fetch(b);const O=T0&&(g==="stream"||g==="response");if(T0&&(u||O&&x)){const R={};["status","statusText","headers"].forEach(P=>{R[P]=$[P]});const T=Ce.toFiniteNumber($.headers.get("content-length")),[M,z]=u&&kE(T,Yd(qE(u),!0))||[];$=new Response(GE($.body,KE,M,()=>{z&&z(),x&&x()}),R)}g=g||"text";let _=await Qd[Ce.findKey(Qd,g)||"text"]($,e);return!O&&x&&x(),await new Promise((R,T)=>{OM(R,T,{data:_,headers:Dr.from($.headers),status:$.status,statusText:$.statusText,config:e,request:b})})}catch(E){throw x&&x(),E&&E.name==="TypeError"&&/fetch/i.test(E.message)?Object.assign(new Rt("Network Error",Rt.ERR_NETWORK,e,b),{cause:E.cause||E}):Rt.from(E,E&&E.code,e,b)}}),M0={http:cW,xhr:BW,fetch:KW};Ce.forEach(M0,(e,t)=>{if(e){try{Object.defineProperty(e,"name",{value:t})}catch{}Object.defineProperty(e,"adapterName",{value:t})}});const XE=e=>`- ${e}`,XW=e=>Ce.isFunction(e)||e===null||e===!1,NM={getAdapter:e=>{e=Ce.isArray(e)?e:[e];const{length:t}=e;let n,r;const a={};for(let l=0;l<t;l++){n=e[l];let s;if(r=n,!XW(n)&&(r=M0[(s=String(n)).toLowerCase()],r===void 0))throw new Rt(`Unknown adapter '${s}'`);if(r)break;a[s||"#"+l]=r}if(!r){const l=Object.entries(a).map(([u,d])=>`adapter ${u} `+(d===!1?"is not supported by the environment":"is not available in the build"));let s=t?l.length>1?`since :
`+l.map(XE).join(`
`):" "+XE(l[0]):"as no adapter specified";throw new Rt("There is no suitable adapter to dispatch the request "+s,"ERR_NOT_SUPPORT")}return r},adapters:M0};function sv(e){if(e.cancelToken&&e.cancelToken.throwIfRequested(),e.signal&&e.signal.aborted)throw new ws(null,e)}function YE(e){return sv(e),e.headers=Dr.from(e.headers),e.data=lv.call(e,e.transformRequest),["post","put","patch"].indexOf(e.method)!==-1&&e.headers.setContentType("application/x-www-form-urlencoded",!1),NM.getAdapter(e.adapter||Lu.adapter)(e).then(function(r){return sv(e),r.data=lv.call(e,e.transformResponse,r),r.headers=Dr.from(r.headers),r},function(r){return $M(r)||(sv(e),r&&r.response&&(r.response.data=lv.call(e,e.transformResponse,r.response),r.response.headers=Dr.from(r.response.headers))),Promise.reject(r)})}const jM="1.8.4",Zm={};["object","boolean","number","function","string","symbol"].forEach((e,t)=>{Zm[e]=function(r){return typeof r===e||"a"+(t<1?"n ":" ")+e}});const QE={};Zm.transitional=function(t,n,r){function a(l,s){return"[Axios v"+jM+"] Transitional option '"+l+"'"+s+(r?". "+r:"")}return(l,s,u)=>{if(t===!1)throw new Rt(a(s," has been removed"+(n?" in "+n:"")),Rt.ERR_DEPRECATED);return n&&!QE[s]&&(QE[s]=!0,console.warn(a(s," has been deprecated since v"+n+" and will be removed in the near future"))),t?t(l,s,u):!0}};Zm.spelling=function(t){return(n,r)=>(console.warn(`${r} is likely a misspelling of ${t}`),!0)};function YW(e,t,n){if(typeof e!="object")throw new Rt("options must be an object",Rt.ERR_BAD_OPTION_VALUE);const r=Object.keys(e);let a=r.length;for(;a-- >0;){const l=r[a],s=t[l];if(s){const u=e[l],d=u===void 0||s(u,l,e);if(d!==!0)throw new Rt("option "+l+" must be "+d,Rt.ERR_BAD_OPTION_VALUE);continue}if(n!==!0)throw new Rt("Unknown option "+l,Rt.ERR_BAD_OPTION)}}const $d={assertOptions:YW,validators:Zm},Qa=$d.validators;let Xi=class{constructor(t){this.defaults=t,this.interceptors={request:new VE,response:new VE}}async request(t,n){try{return await this._request(t,n)}catch(r){if(r instanceof Error){let a={};Error.captureStackTrace?Error.captureStackTrace(a):a=new Error;const l=a.stack?a.stack.replace(/^.+\n/,""):"";try{r.stack?l&&!String(r.stack).endsWith(l.replace(/^.+\n.+\n/,""))&&(r.stack+=`
`+l):r.stack=l}catch{}}throw r}}_request(t,n){typeof t=="string"?(n=n||{},n.url=t):n=t||{},n=el(this.defaults,n);const{transitional:r,paramsSerializer:a,headers:l}=n;r!==void 0&&$d.assertOptions(r,{silentJSONParsing:Qa.transitional(Qa.boolean),forcedJSONParsing:Qa.transitional(Qa.boolean),clarifyTimeoutError:Qa.transitional(Qa.boolean)},!1),a!=null&&(Ce.isFunction(a)?n.paramsSerializer={serialize:a}:$d.assertOptions(a,{encode:Qa.function,serialize:Qa.function},!0)),n.allowAbsoluteUrls!==void 0||(this.defaults.allowAbsoluteUrls!==void 0?n.allowAbsoluteUrls=this.defaults.allowAbsoluteUrls:n.allowAbsoluteUrls=!0),$d.assertOptions(n,{baseUrl:Qa.spelling("baseURL"),withXsrfToken:Qa.spelling("withXSRFToken")},!0),n.method=(n.method||this.defaults.method||"get").toLowerCase();let s=l&&Ce.merge(l.common,l[n.method]);l&&Ce.forEach(["delete","get","head","post","put","patch","common"],b=>{delete l[b]}),n.headers=Dr.concat(s,l);const u=[];let d=!0;this.interceptors.request.forEach(function(x){typeof x.runWhen=="function"&&x.runWhen(n)===!1||(d=d&&x.synchronous,u.unshift(x.fulfilled,x.rejected))});const g=[];this.interceptors.response.forEach(function(x){g.push(x.fulfilled,x.rejected)});let h,p=0,y;if(!d){const b=[YE.bind(this),void 0];for(b.unshift.apply(b,u),b.push.apply(b,g),y=b.length,h=Promise.resolve(n);p<y;)h=h.then(b[p++],b[p++]);return h}y=u.length;let C=n;for(p=0;p<y;){const b=u[p++],x=u[p++];try{C=b(C)}catch(S){x.call(this,S);break}}try{h=YE.call(this,C)}catch(b){return Promise.reject(b)}for(p=0,y=g.length;p<y;)h=h.then(g[p++],g[p++]);return h}getUri(t){t=el(this.defaults,t);const n=RM(t.baseURL,t.url,t.allowAbsoluteUrls);return xM(n,t.params,t.paramsSerializer)}};Ce.forEach(["delete","get","head","options"],function(t){Xi.prototype[t]=function(n,r){return this.request(el(r||{},{method:t,url:n,data:(r||{}).data}))}});Ce.forEach(["post","put","patch"],function(t){function n(r){return function(l,s,u){return this.request(el(u||{},{method:t,headers:r?{"Content-Type":"multipart/form-data"}:{},url:l,data:s}))}}Xi.prototype[t]=n(),Xi.prototype[t+"Form"]=n(!0)});let QW=class IM{constructor(t){if(typeof t!="function")throw new TypeError("executor must be a function.");let n;this.promise=new Promise(function(l){n=l});const r=this;this.promise.then(a=>{if(!r._listeners)return;let l=r._listeners.length;for(;l-- >0;)r._listeners[l](a);r._listeners=null}),this.promise.then=a=>{let l;const s=new Promise(u=>{r.subscribe(u),l=u}).then(a);return s.cancel=function(){r.unsubscribe(l)},s},t(function(l,s,u){r.reason||(r.reason=new ws(l,s,u),n(r.reason))})}throwIfRequested(){if(this.reason)throw this.reason}subscribe(t){if(this.reason){t(this.reason);return}this._listeners?this._listeners.push(t):this._listeners=[t]}unsubscribe(t){if(!this._listeners)return;const n=this._listeners.indexOf(t);n!==-1&&this._listeners.splice(n,1)}toAbortSignal(){const t=new AbortController,n=r=>{t.abort(r)};return this.subscribe(n),t.signal.unsubscribe=()=>this.unsubscribe(n),t.signal}static source(){let t;return{token:new IM(function(a){t=a}),cancel:t}}};function ZW(e){return function(n){return e.apply(null,n)}}function JW(e){return Ce.isObject(e)&&e.isAxiosError===!0}const N0={Continue:100,SwitchingProtocols:101,Processing:102,EarlyHints:103,Ok:200,Created:201,Accepted:202,NonAuthoritativeInformation:203,NoContent:204,ResetContent:205,PartialContent:206,MultiStatus:207,AlreadyReported:208,ImUsed:226,MultipleChoices:300,MovedPermanently:301,Found:302,SeeOther:303,NotModified:304,UseProxy:305,Unused:306,TemporaryRedirect:307,PermanentRedirect:308,BadRequest:400,Unauthorized:401,PaymentRequired:402,Forbidden:403,NotFound:404,MethodNotAllowed:405,NotAcceptable:406,ProxyAuthenticationRequired:407,RequestTimeout:408,Conflict:409,Gone:410,LengthRequired:411,PreconditionFailed:412,PayloadTooLarge:413,UriTooLong:414,UnsupportedMediaType:415,RangeNotSatisfiable:416,ExpectationFailed:417,ImATeapot:418,MisdirectedRequest:421,UnprocessableEntity:422,Locked:423,FailedDependency:424,TooEarly:425,UpgradeRequired:426,PreconditionRequired:428,TooManyRequests:429,RequestHeaderFieldsTooLarge:431,UnavailableForLegalReasons:451,InternalServerError:500,NotImplemented:501,BadGateway:502,ServiceUnavailable:503,GatewayTimeout:504,HttpVersionNotSupported:505,VariantAlsoNegotiates:506,InsufficientStorage:507,LoopDetected:508,NotExtended:510,NetworkAuthenticationRequired:511};Object.entries(N0).forEach(([e,t])=>{N0[t]=e});function PM(e){const t=new Xi(e),n=fM(Xi.prototype.request,t);return Ce.extend(n,Xi.prototype,t,{allOwnKeys:!0}),Ce.extend(n,t,null,{allOwnKeys:!0}),n.create=function(a){return PM(el(e,a))},n}const yt=PM(Lu);yt.Axios=Xi;yt.CanceledError=ws;yt.CancelToken=QW;yt.isCancel=$M;yt.VERSION=jM;yt.toFormData=Ym;yt.AxiosError=Rt;yt.Cancel=yt.CanceledError;yt.all=function(t){return Promise.all(t)};yt.spread=ZW;yt.isAxiosError=JW;yt.mergeConfig=el;yt.AxiosHeaders=Dr;yt.formToJSON=e=>EM(Ce.isHTMLForm(e)?new FormData(e):e);yt.getAdapter=NM.getAdapter;yt.HttpStatusCode=N0;yt.default=yt;const{Axios:xX,AxiosError:wX,CanceledError:EX,isCancel:$X,CancelToken:OX,VERSION:RX,all:_X,Cancel:TX,isAxiosError:MX,spread:NX,toFormData:jX,AxiosHeaders:IX,HttpStatusCode:PX,formToJSON:AX,getAdapter:zX,mergeConfig:DX}=yt;function yr(e){return`Minified Redux error #${e}; visit https://redux.js.org/Errors?code=${e} for the full message or use the non-minified dev environment for full errors. `}var eG=typeof Symbol=="function"&&Symbol.observable||"@@observable",ZE=eG,cv=()=>Math.random().toString(36).substring(7).split("").join("."),tG={INIT:`@@redux/INIT${cv()}`,REPLACE:`@@redux/REPLACE${cv()}`,PROBE_UNKNOWN_ACTION:()=>`@@redux/PROBE_UNKNOWN_ACTION${cv()}`},Zd=tG;function ab(e){if(typeof e!="object"||e===null)return!1;let t=e;for(;Object.getPrototypeOf(t)!==null;)t=Object.getPrototypeOf(t);return Object.getPrototypeOf(e)===t||Object.getPrototypeOf(e)===null}function AM(e,t,n){if(typeof e!="function")throw new Error(yr(2));if(typeof t=="function"&&typeof n=="function"||typeof n=="function"&&typeof arguments[3]=="function")throw new Error(yr(0));if(typeof t=="function"&&typeof n>"u"&&(n=t,t=void 0),typeof n<"u"){if(typeof n!="function")throw new Error(yr(1));return n(AM)(e,t)}let r=e,a=t,l=new Map,s=l,u=0,d=!1;function g(){s===l&&(s=new Map,l.forEach((S,E)=>{s.set(E,S)}))}function h(){if(d)throw new Error(yr(3));return a}function p(S){if(typeof S!="function")throw new Error(yr(4));if(d)throw new Error(yr(5));let E=!0;g();const $=u++;return s.set($,S),function(){if(E){if(d)throw new Error(yr(6));E=!1,g(),s.delete($),l=null}}}function y(S){if(!ab(S))throw new Error(yr(7));if(typeof S.type>"u")throw new Error(yr(8));if(typeof S.type!="string")throw new Error(yr(17));if(d)throw new Error(yr(9));try{d=!0,a=r(a,S)}finally{d=!1}return(l=s).forEach($=>{$()}),S}function C(S){if(typeof S!="function")throw new Error(yr(10));r=S,y({type:Zd.REPLACE})}function b(){const S=p;return{subscribe(E){if(typeof E!="object"||E===null)throw new Error(yr(11));function $(){const _=E;_.next&&_.next(h())}return $(),{unsubscribe:S($)}},[ZE](){return this}}}return y({type:Zd.INIT}),{dispatch:y,subscribe:p,getState:h,replaceReducer:C,[ZE]:b}}function nG(e){Object.keys(e).forEach(t=>{const n=e[t];if(typeof n(void 0,{type:Zd.INIT})>"u")throw new Error(yr(12));if(typeof n(void 0,{type:Zd.PROBE_UNKNOWN_ACTION()})>"u")throw new Error(yr(13))})}function rG(e){const t=Object.keys(e),n={};for(let l=0;l<t.length;l++){const s=t[l];typeof e[s]=="function"&&(n[s]=e[s])}const r=Object.keys(n);let a;try{nG(n)}catch(l){a=l}return function(s={},u){if(a)throw a;let d=!1;const g={};for(let h=0;h<r.length;h++){const p=r[h],y=n[p],C=s[p],b=y(C,u);if(typeof b>"u")throw u&&u.type,new Error(yr(14));g[p]=b,d=d||b!==C}return d=d||r.length!==Object.keys(s).length,d?g:s}}function Jd(...e){return e.length===0?t=>t:e.length===1?e[0]:e.reduce((t,n)=>(...r)=>t(n(...r)))}function aG(...e){return t=>(n,r)=>{const a=t(n,r);let l=()=>{throw new Error(yr(15))};const s={getState:a.getState,dispatch:(d,...g)=>l(d,...g)},u=e.map(d=>d(s));return l=Jd(...u)(a.dispatch),{...a,dispatch:l}}}function oG(e){return ab(e)&&"type"in e&&typeof e.type=="string"}var zM=Symbol.for("immer-nothing"),JE=Symbol.for("immer-draftable"),aa=Symbol.for("immer-state");function Ia(e,...t){throw new Error(`[Immer] minified error nr: ${e}. Full error at: https://bit.ly/3cXEKWf`)}var os=Object.getPrototypeOf;function tl(e){return!!e&&!!e[aa]}function jo(e){var t;return e?DM(e)||Array.isArray(e)||!!e[JE]||!!((t=e.constructor)!=null&&t[JE])||eg(e)||tg(e):!1}var iG=Object.prototype.constructor.toString();function DM(e){if(!e||typeof e!="object")return!1;const t=os(e);if(t===null)return!0;const n=Object.hasOwnProperty.call(t,"constructor")&&t.constructor;return n===Object?!0:typeof n=="function"&&Function.toString.call(n)===iG}function em(e,t){Jm(e)===0?Reflect.ownKeys(e).forEach(n=>{t(n,e[n],e)}):e.forEach((n,r)=>t(r,n,e))}function Jm(e){const t=e[aa];return t?t.type_:Array.isArray(e)?1:eg(e)?2:tg(e)?3:0}function j0(e,t){return Jm(e)===2?e.has(t):Object.prototype.hasOwnProperty.call(e,t)}function LM(e,t,n){const r=Jm(e);r===2?e.set(t,n):r===3?e.add(n):e[t]=n}function lG(e,t){return e===t?e!==0||1/e===1/t:e!==e&&t!==t}function eg(e){return e instanceof Map}function tg(e){return e instanceof Set}function Vi(e){return e.copy_||e.base_}function I0(e,t){if(eg(e))return new Map(e);if(tg(e))return new Set(e);if(Array.isArray(e))return Array.prototype.slice.call(e);const n=DM(e);if(t===!0||t==="class_only"&&!n){const r=Object.getOwnPropertyDescriptors(e);delete r[aa];let a=Reflect.ownKeys(r);for(let l=0;l<a.length;l++){const s=a[l],u=r[s];u.writable===!1&&(u.writable=!0,u.configurable=!0),(u.get||u.set)&&(r[s]={configurable:!0,writable:!0,enumerable:u.enumerable,value:e[s]})}return Object.create(os(e),r)}else{const r=os(e);if(r!==null&&n)return{...e};const a=Object.create(r);return Object.assign(a,e)}}function ob(e,t=!1){return ng(e)||tl(e)||!jo(e)||(Jm(e)>1&&(e.set=e.add=e.clear=e.delete=sG),Object.freeze(e),t&&Object.entries(e).forEach(([n,r])=>ob(r,!0))),e}function sG(){Ia(2)}function ng(e){return Object.isFrozen(e)}var cG={};function nl(e){const t=cG[e];return t||Ia(0,e),t}var vu;function BM(){return vu}function uG(e,t){return{drafts_:[],parent_:e,immer_:t,canAutoFreeze_:!0,unfinalizedDrafts_:0}}function e$(e,t){t&&(nl("Patches"),e.patches_=[],e.inversePatches_=[],e.patchListener_=t)}function P0(e){A0(e),e.drafts_.forEach(fG),e.drafts_=null}function A0(e){e===vu&&(vu=e.parent_)}function t$(e){return vu=uG(vu,e)}function fG(e){const t=e[aa];t.type_===0||t.type_===1?t.revoke_():t.revoked_=!0}function n$(e,t){t.unfinalizedDrafts_=t.drafts_.length;const n=t.drafts_[0];return e!==void 0&&e!==n?(n[aa].modified_&&(P0(t),Ia(4)),jo(e)&&(e=tm(t,e),t.parent_||nm(t,e)),t.patches_&&nl("Patches").generateReplacementPatches_(n[aa].base_,e,t.patches_,t.inversePatches_)):e=tm(t,n,[]),P0(t),t.patches_&&t.patchListener_(t.patches_,t.inversePatches_),e!==zM?e:void 0}function tm(e,t,n){if(ng(t))return t;const r=t[aa];if(!r)return em(t,(a,l)=>r$(e,r,t,a,l,n)),t;if(r.scope_!==e)return t;if(!r.modified_)return nm(e,r.base_,!0),r.base_;if(!r.finalized_){r.finalized_=!0,r.scope_.unfinalizedDrafts_--;const a=r.copy_;let l=a,s=!1;r.type_===3&&(l=new Set(a),a.clear(),s=!0),em(l,(u,d)=>r$(e,r,a,u,d,n,s)),nm(e,a,!1),n&&e.patches_&&nl("Patches").generatePatches_(r,n,e.patches_,e.inversePatches_)}return r.copy_}function r$(e,t,n,r,a,l,s){if(tl(a)){const u=l&&t&&t.type_!==3&&!j0(t.assigned_,r)?l.concat(r):void 0,d=tm(e,a,u);if(LM(n,r,d),tl(d))e.canAutoFreeze_=!1;else return}else s&&n.add(a);if(jo(a)&&!ng(a)){if(!e.immer_.autoFreeze_&&e.unfinalizedDrafts_<1)return;tm(e,a),(!t||!t.scope_.parent_)&&typeof r!="symbol"&&Object.prototype.propertyIsEnumerable.call(n,r)&&nm(e,a)}}function nm(e,t,n=!1){!e.parent_&&e.immer_.autoFreeze_&&e.canAutoFreeze_&&ob(t,n)}function dG(e,t){const n=Array.isArray(e),r={type_:n?1:0,scope_:t?t.scope_:BM(),modified_:!1,finalized_:!1,assigned_:{},parent_:t,base_:e,draft_:null,copy_:null,revoke_:null,isManual_:!1};let a=r,l=ib;n&&(a=[r],l=yu);const{revoke:s,proxy:u}=Proxy.revocable(a,l);return r.draft_=u,r.revoke_=s,u}var ib={get(e,t){if(t===aa)return e;const n=Vi(e);if(!j0(n,t))return mG(e,n,t);const r=n[t];return e.finalized_||!jo(r)?r:r===uv(e.base_,t)?(fv(e),e.copy_[t]=D0(r,e)):r},has(e,t){return t in Vi(e)},ownKeys(e){return Reflect.ownKeys(Vi(e))},set(e,t,n){const r=HM(Vi(e),t);if(r!=null&&r.set)return r.set.call(e.draft_,n),!0;if(!e.modified_){const a=uv(Vi(e),t),l=a==null?void 0:a[aa];if(l&&l.base_===n)return e.copy_[t]=n,e.assigned_[t]=!1,!0;if(lG(n,a)&&(n!==void 0||j0(e.base_,t)))return!0;fv(e),z0(e)}return e.copy_[t]===n&&(n!==void 0||t in e.copy_)||Number.isNaN(n)&&Number.isNaN(e.copy_[t])||(e.copy_[t]=n,e.assigned_[t]=!0),!0},deleteProperty(e,t){return uv(e.base_,t)!==void 0||t in e.base_?(e.assigned_[t]=!1,fv(e),z0(e)):delete e.assigned_[t],e.copy_&&delete e.copy_[t],!0},getOwnPropertyDescriptor(e,t){const n=Vi(e),r=Reflect.getOwnPropertyDescriptor(n,t);return r&&{writable:!0,configurable:e.type_!==1||t!=="length",enumerable:r.enumerable,value:n[t]}},defineProperty(){Ia(11)},getPrototypeOf(e){return os(e.base_)},setPrototypeOf(){Ia(12)}},yu={};em(ib,(e,t)=>{yu[e]=function(){return arguments[0]=arguments[0][0],t.apply(this,arguments)}});yu.deleteProperty=function(e,t){return yu.set.call(this,e,t,void 0)};yu.set=function(e,t,n){return ib.set.call(this,e[0],t,n,e[0])};function uv(e,t){const n=e[aa];return(n?Vi(n):e)[t]}function mG(e,t,n){var a;const r=HM(t,n);return r?"value"in r?r.value:(a=r.get)==null?void 0:a.call(e.draft_):void 0}function HM(e,t){if(!(t in e))return;let n=os(e);for(;n;){const r=Object.getOwnPropertyDescriptor(n,t);if(r)return r;n=os(n)}}function z0(e){e.modified_||(e.modified_=!0,e.parent_&&z0(e.parent_))}function fv(e){e.copy_||(e.copy_=I0(e.base_,e.scope_.immer_.useStrictShallowCopy_))}var gG=class{constructor(e){this.autoFreeze_=!0,this.useStrictShallowCopy_=!1,this.produce=(t,n,r)=>{if(typeof t=="function"&&typeof n!="function"){const l=n;n=t;const s=this;return function(d=l,...g){return s.produce(d,h=>n.call(this,h,...g))}}typeof n!="function"&&Ia(6),r!==void 0&&typeof r!="function"&&Ia(7);let a;if(jo(t)){const l=t$(this),s=D0(t,void 0);let u=!0;try{a=n(s),u=!1}finally{u?P0(l):A0(l)}return e$(l,r),n$(a,l)}else if(!t||typeof t!="object"){if(a=n(t),a===void 0&&(a=t),a===zM&&(a=void 0),this.autoFreeze_&&ob(a,!0),r){const l=[],s=[];nl("Patches").generateReplacementPatches_(t,a,l,s),r(l,s)}return a}else Ia(1,t)},this.produceWithPatches=(t,n)=>{if(typeof t=="function")return(s,...u)=>this.produceWithPatches(s,d=>t(d,...u));let r,a;return[this.produce(t,n,(s,u)=>{r=s,a=u}),r,a]},typeof(e==null?void 0:e.autoFreeze)=="boolean"&&this.setAutoFreeze(e.autoFreeze),typeof(e==null?void 0:e.useStrictShallowCopy)=="boolean"&&this.setUseStrictShallowCopy(e.useStrictShallowCopy)}createDraft(e){jo(e)||Ia(8),tl(e)&&(e=hG(e));const t=t$(this),n=D0(e,void 0);return n[aa].isManual_=!0,A0(t),n}finishDraft(e,t){const n=e&&e[aa];(!n||!n.isManual_)&&Ia(9);const{scope_:r}=n;return e$(r,t),n$(void 0,r)}setAutoFreeze(e){this.autoFreeze_=e}setUseStrictShallowCopy(e){this.useStrictShallowCopy_=e}applyPatches(e,t){let n;for(n=t.length-1;n>=0;n--){const a=t[n];if(a.path.length===0&&a.op==="replace"){e=a.value;break}}n>-1&&(t=t.slice(n+1));const r=nl("Patches").applyPatches_;return tl(e)?r(e,t):this.produce(e,a=>r(a,t))}};function D0(e,t){const n=eg(e)?nl("MapSet").proxyMap_(e,t):tg(e)?nl("MapSet").proxySet_(e,t):dG(e,t);return(t?t.scope_:BM()).drafts_.push(n),n}function hG(e){return tl(e)||Ia(10,e),FM(e)}function FM(e){if(!jo(e)||ng(e))return e;const t=e[aa];let n;if(t){if(!t.modified_)return t.base_;t.finalized_=!0,n=I0(e,t.scope_.immer_.useStrictShallowCopy_)}else n=I0(e,!0);return em(n,(r,a)=>{LM(n,r,FM(a))}),t&&(t.finalized_=!1),n}var oa=new gG,VM=oa.produce;oa.produceWithPatches.bind(oa);oa.setAutoFreeze.bind(oa);oa.setUseStrictShallowCopy.bind(oa);oa.applyPatches.bind(oa);oa.createDraft.bind(oa);oa.finishDraft.bind(oa);function UM(e){return({dispatch:n,getState:r})=>a=>l=>typeof l=="function"?l(n,r,e):a(l)}var pG=UM(),vG=UM,yG=typeof window<"u"&&window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__?window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__:function(){if(arguments.length!==0)return typeof arguments[0]=="object"?Jd:Jd.apply(null,arguments)};function a$(e,t){function n(...r){if(t){let a=t(...r);if(!a)throw new Error(Eo(0));return{type:e,payload:a.payload,..."meta"in a&&{meta:a.meta},..."error"in a&&{error:a.error}}}return{type:e,payload:r[0]}}return n.toString=()=>`${e}`,n.type=e,n.match=r=>oG(r)&&r.type===e,n}var kM=class qc extends Array{constructor(...t){super(...t),Object.setPrototypeOf(this,qc.prototype)}static get[Symbol.species](){return qc}concat(...t){return super.concat.apply(this,t)}prepend(...t){return t.length===1&&Array.isArray(t[0])?new qc(...t[0].concat(this)):new qc(...t.concat(this))}};function o$(e){return jo(e)?VM(e,()=>{}):e}function od(e,t,n){return e.has(t)?e.get(t):e.set(t,n(t)).get(t)}function bG(e){return typeof e=="boolean"}var SG=()=>function(t){const{thunk:n=!0,immutableCheck:r=!0,serializableCheck:a=!0,actionCreatorCheck:l=!0}=t??{};let s=new kM;return n&&(bG(n)?s.push(pG):s.push(vG(n.extraArgument))),s},CG="RTK_autoBatch",i$=e=>t=>{setTimeout(t,e)},xG=(e={type:"raf"})=>t=>(...n)=>{const r=t(...n);let a=!0,l=!1,s=!1;const u=new Set,d=e.type==="tick"?queueMicrotask:e.type==="raf"?typeof window<"u"&&window.requestAnimationFrame?window.requestAnimationFrame:i$(10):e.type==="callback"?e.queueNotification:i$(e.timeout),g=()=>{s=!1,l&&(l=!1,u.forEach(h=>h()))};return Object.assign({},r,{subscribe(h){const p=()=>a&&h(),y=r.subscribe(p);return u.add(h),()=>{y(),u.delete(h)}},dispatch(h){var p;try{return a=!((p=h==null?void 0:h.meta)!=null&&p[CG]),l=!a,l&&(s||(s=!0,d(g))),r.dispatch(h)}finally{a=!0}}})},wG=e=>function(n){const{autoBatch:r=!0}=n??{};let a=new kM(e);return r&&a.push(xG(typeof r=="object"?r:void 0)),a};function EG(e){const t=SG(),{reducer:n=void 0,middleware:r,devTools:a=!0,preloadedState:l=void 0,enhancers:s=void 0}=e||{};let u;if(typeof n=="function")u=n;else if(ab(n))u=rG(n);else throw new Error(Eo(1));let d;typeof r=="function"?d=r(t):d=t();let g=Jd;a&&(g=yG({trace:!1,...typeof a=="object"&&a}));const h=aG(...d),p=wG(h);let y=typeof s=="function"?s(p):p();const C=g(...y);return AM(u,l,C)}function qM(e){const t={},n=[];let r;const a={addCase(l,s){const u=typeof l=="string"?l:l.type;if(!u)throw new Error(Eo(28));if(u in t)throw new Error(Eo(29));return t[u]=s,a},addMatcher(l,s){return n.push({matcher:l,reducer:s}),a},addDefaultCase(l){return r=l,a}};return e(a),[t,n,r]}function $G(e){return typeof e=="function"}function OG(e,t){let[n,r,a]=qM(t),l;if($G(e))l=()=>o$(e());else{const u=o$(e);l=()=>u}function s(u=l(),d){let g=[n[d.type],...r.filter(({matcher:h})=>h(d)).map(({reducer:h})=>h)];return g.filter(h=>!!h).length===0&&(g=[a]),g.reduce((h,p)=>{if(p)if(tl(h)){const C=p(h,d);return C===void 0?h:C}else{if(jo(h))return VM(h,y=>p(y,d));{const y=p(h,d);if(y===void 0){if(h===null)return h;throw Error("A case reducer on a non-draftable value must not return undefined")}return y}}return h},u)}return s.getInitialState=l,s}var RG=Symbol.for("rtk-slice-createasyncthunk");function _G(e,t){return`${e}/${t}`}function TG({creators:e}={}){var n;const t=(n=e==null?void 0:e.asyncThunk)==null?void 0:n[RG];return function(a){const{name:l,reducerPath:s=l}=a;if(!l)throw new Error(Eo(11));const u=(typeof a.reducers=="function"?a.reducers(NG()):a.reducers)||{},d=Object.keys(u),g={sliceCaseReducersByName:{},sliceCaseReducersByType:{},actionCreators:{},sliceMatchers:[]},h={addCase(_,R){const T=typeof _=="string"?_:_.type;if(!T)throw new Error(Eo(12));if(T in g.sliceCaseReducersByType)throw new Error(Eo(13));return g.sliceCaseReducersByType[T]=R,h},addMatcher(_,R){return g.sliceMatchers.push({matcher:_,reducer:R}),h},exposeAction(_,R){return g.actionCreators[_]=R,h},exposeCaseReducer(_,R){return g.sliceCaseReducersByName[_]=R,h}};d.forEach(_=>{const R=u[_],T={reducerName:_,type:_G(l,_),createNotation:typeof a.reducers=="function"};IG(R)?AG(T,R,h,t):jG(T,R,h)});function p(){const[_={},R=[],T=void 0]=typeof a.extraReducers=="function"?qM(a.extraReducers):[a.extraReducers],M={..._,...g.sliceCaseReducersByType};return OG(a.initialState,z=>{for(let P in M)z.addCase(P,M[P]);for(let P of g.sliceMatchers)z.addMatcher(P.matcher,P.reducer);for(let P of R)z.addMatcher(P.matcher,P.reducer);T&&z.addDefaultCase(T)})}const y=_=>_,C=new Map,b=new WeakMap;let x;function S(_,R){return x||(x=p()),x(_,R)}function E(){return x||(x=p()),x.getInitialState()}function $(_,R=!1){function T(z){let P=z[_];return typeof P>"u"&&R&&(P=od(b,T,E)),P}function M(z=y){const P=od(C,R,()=>new WeakMap);return od(P,z,()=>{const D={};for(const[F,G]of Object.entries(a.selectors??{}))D[F]=MG(G,z,()=>od(b,z,E),R);return D})}return{reducerPath:_,getSelectors:M,get selectors(){return M(T)},selectSlice:T}}const O={name:l,reducer:S,actions:g.actionCreators,caseReducers:g.sliceCaseReducersByName,getInitialState:E,...$(s),injectInto(_,{reducerPath:R,...T}={}){const M=R??s;return _.inject({reducerPath:M,reducer:S},T),{...O,...$(M,!0)}}};return O}}function MG(e,t,n,r){function a(l,...s){let u=t(l);return typeof u>"u"&&r&&(u=n()),e(u,...s)}return a.unwrapped=e,a}var Bu=TG();function NG(){function e(t,n){return{_reducerDefinitionType:"asyncThunk",payloadCreator:t,...n}}return e.withTypes=()=>e,{reducer(t){return Object.assign({[t.name](...n){return t(...n)}}[t.name],{_reducerDefinitionType:"reducer"})},preparedReducer(t,n){return{_reducerDefinitionType:"reducerWithPrepare",prepare:t,reducer:n}},asyncThunk:e}}function jG({type:e,reducerName:t,createNotation:n},r,a){let l,s;if("reducer"in r){if(n&&!PG(r))throw new Error(Eo(17));l=r.reducer,s=r.prepare}else l=r;a.addCase(e,l).exposeCaseReducer(t,l).exposeAction(t,s?a$(e,s):a$(e))}function IG(e){return e._reducerDefinitionType==="asyncThunk"}function PG(e){return e._reducerDefinitionType==="reducerWithPrepare"}function AG({type:e,reducerName:t},n,r,a){if(!a)throw new Error(Eo(18));const{payloadCreator:l,fulfilled:s,pending:u,rejected:d,settled:g,options:h}=n,p=a(e,l,h);r.exposeAction(t,p),s&&r.addCase(p.fulfilled,s),u&&r.addCase(p.pending,u),d&&r.addCase(p.rejected,d),g&&r.addMatcher(p.settled,g),r.exposeCaseReducer(t,{fulfilled:s||id,pending:u||id,rejected:d||id,settled:g||id})}function id(){}function Eo(e){return`Minified Redux Toolkit error #${e}; visit https://redux-toolkit.js.org/Errors?code=${e} for the full message or use the non-minified dev environment for full errors. `}const l$=localStorage.getItem("locale"),s$=localStorage.getItem("theme"),c$=localStorage.getItem("baseURL"),zG=localStorage.getItem("authorization"),u$=localStorage.getItem("containerURL"),f$=localStorage.getItem("namespace"),d$=localStorage.getItem("project"),WM=Bu({name:"user",initialState:{locale:l$||"en_US",theme:s$||"light",baseURL:c$?`${c$}`:"",containerURL:u$?`${u$}`:"",authorization:zG,namespace:f$?`${f$}`:"default",project:d$?`${d$}`:"default",projectObj:{}},reducers:{setUserItem(e,t){Object.assign(e,t.payload),t.payload.locale&&localStorage.setItem("locale",t.payload.locale),t.payload.theme&&localStorage.setItem("theme",t.payload.theme),t.payload.baseURL&&localStorage.setItem("baseURL",t.payload.baseURL),t.payload.authorization&&localStorage.setItem("authorization",t.payload.authorization),t.payload.containerURL&&localStorage.setItem("containerURL",t.payload.containerURL),t.payload.namespace&&localStorage.setItem("namespace",t.payload.namespace),t.payload.project&&localStorage.setItem("project",t.payload.project)}}}),{setUserItem:$o}=WM.actions,DG=WM.reducer,GM=Bu({name:"global",initialState:{sseData:{},setting:{}},reducers:{setSseData:(e,t)=>{e.sseData=t.payload},setSetting:(e,t)=>{e.setting=t.payload}}}),{setSseData:KM,setSetting:XM}=GM.actions,LG=GM.reducer;function Hu(){const[e,t]=f.useState({key:null,visible:!1,params:null});return{modal:e,openModal:(a,l)=>{t({key:a,visible:!0,params:l})},closeModal:()=>{t({key:null,visible:!1,params:null})}}}function BG(e){const[t,n]=f.useState(Object.fromEntries(e.map(l=>[l,{visible:!1,params:null}])));return{modals:t,openModals:(l,s)=>{n(u=>({...u,[l]:{visible:!0,params:s}}))},closeModals:l=>{n(s=>({...s,[l]:{visible:!1,params:null}}))}}}const YM=f.createContext(null),HG=({url:e="/brave-api/sse-group",retryInterval:t=5e3,children:n})=>{const r=f.useRef(null),[a,l]=f.useState("connecting"),s=f.useRef(null),u=f.useRef(null),d=f.useRef(Date.now()),g=15e3,{baseURL:h,authorization:p}=Si(x=>x.user),y=()=>{r.current&&r.current.close(),l("connecting");const x=p?`${h}${e}?authorization=${p}`:`${h}${e}`,S=new EventSource(x);r.current=S,S.onopen=()=>{l("open"),d.current=Date.now(),console.log("✅ SSE connected")};const E=()=>{l("closed"),S.close(),r.current=null,s.current||(s.current=setTimeout(()=>{s.current=null,y()},t))};u.current=setInterval(()=>{Date.now()-d.current>g&&(console.warn("⚠️ SSE 心跳超时，服务器可能异常退出"),E())},5e3),S.onerror=$=>{console.warn("❌ SSE error",$)},S.onmessage=$=>{d.current=Date.now(),JSON.parse($.data).type!="ping"&&console.log("📩 SSE message:",$.data)}},C=()=>{var x;s.current&&clearTimeout(s.current),s.current=null,(x=r.current)==null||x.close(),r.current=null,l("closed")},b=()=>{C(),y()};return f.useEffect(()=>(console.log(">>>>>>>>>>建立连接................................."),y(),()=>{C(),u.current&&clearInterval(u.current)}),[h]),te.jsx(YM.Provider,{value:{eventSourceRef:r,status:a,close:C,reconnect:b},children:n})},QM=()=>{const e=f.useContext(YM);if(!e)throw new Error("useSSEContext must be used within <SSEProvider>");return e},FG=e=>yt.post("/project/add-project",e),VG=e=>yt.post("/project/update-project",e),UG=e=>yt.get(`/project/find-by-project-id/${e}`),kG=e=>yt.delete(`/project/delete-project/${e}`);let ZM=null;const qG=()=>{const[e,t]=Iu.useMessage();return ZM=e,t},Es=()=>ZM;za.TextArea;const JM=({visible:e,onClose:t,params:n,callback:r})=>{const[a]=Hn.useForm(),l=Es(),s=ol();if(f.useEffect(()=>{console.log(n),n!=null&&n.project_id?d():a.resetFields()},[n==null?void 0:n.project_id]),!e)return null;const u=h=>({...h,project_id:n==null?void 0:n.project_id}),d=async()=>{const h=await UG(n.project_id);console.log(h.data);const p=h.data;p.metadata_form=JSON.stringify(p.metadata_form),a.setFieldsValue(p)},g=async()=>{const h=await a.validateFields(),p=u(h);if(p.project_id)await VG(p),l.success("更新成功"),r==null||r(p.project_id);else{const C=await FG(p);console.log(C.data.project_id),l.success("添加成功"),r==null||r(C.data.project_id)}const y=await yt.get(`/project/find-by-project-id/${p.project_id}`);s($o({projectObj:y.data})),t(),r&&r()};return te.jsx(Hr,{onOk:g,open:e,onClose:t,onCancel:t,title:`${n!=null&&n.project_id?"编辑":"添加"}项目`,children:te.jsxs(Hn,{form:a,children:[te.jsx(Hn.Item,{label:"Project Name",name:"project_name",rules:[{required:!0,message:"This field cannot be empty!"}],children:te.jsx(za,{})}),te.jsx(Hn.Item,{initialValue:"[]",label:"Research Variable",name:"metadata_form",rules:[{required:!0,message:"This field cannot be empty!"}],children:te.jsx(Bm,{rows:6})}),te.jsx(O5,{ghost:!0,items:[{key:"1",label:"更多",children:te.jsx(te.Fragment,{children:te.jsx(Hn.Item,{noStyle:!0,shouldUpdate:!0,children:()=>te.jsx(Pu,{children:te.jsx("pre",{children:JSON.stringify(u(a.getFieldsValue()),null,2)})})})})}]})]})})},xt=e=>typeof e=="string",_c=()=>{let e,t;const n=new Promise((r,a)=>{e=r,t=a});return n.resolve=e,n.reject=t,n},m$=e=>e==null?"":""+e,WG=(e,t,n)=>{e.forEach(r=>{t[r]&&(n[r]=t[r])})},GG=/###/g,g$=e=>e&&e.indexOf("###")>-1?e.replace(GG,"."):e,h$=e=>!e||xt(e),Qc=(e,t,n)=>{const r=xt(t)?t.split("."):t;let a=0;for(;a<r.length-1;){if(h$(e))return{};const l=g$(r[a]);!e[l]&&n&&(e[l]=new n),Object.prototype.hasOwnProperty.call(e,l)?e=e[l]:e={},++a}return h$(e)?{}:{obj:e,k:g$(r[a])}},p$=(e,t,n)=>{const{obj:r,k:a}=Qc(e,t,Object);if(r!==void 0||t.length===1){r[a]=n;return}let l=t[t.length-1],s=t.slice(0,t.length-1),u=Qc(e,s,Object);for(;u.obj===void 0&&s.length;)l=`${s[s.length-1]}.${l}`,s=s.slice(0,s.length-1),u=Qc(e,s,Object),u!=null&&u.obj&&typeof u.obj[`${u.k}.${l}`]<"u"&&(u.obj=void 0);u.obj[`${u.k}.${l}`]=n},KG=(e,t,n,r)=>{const{obj:a,k:l}=Qc(e,t,Object);a[l]=a[l]||[],a[l].push(n)},rm=(e,t)=>{const{obj:n,k:r}=Qc(e,t);if(n&&Object.prototype.hasOwnProperty.call(n,r))return n[r]},XG=(e,t,n)=>{const r=rm(e,n);return r!==void 0?r:rm(t,n)},eN=(e,t,n)=>{for(const r in t)r!=="__proto__"&&r!=="constructor"&&(r in e?xt(e[r])||e[r]instanceof String||xt(t[r])||t[r]instanceof String?n&&(e[r]=t[r]):eN(e[r],t[r],n):e[r]=t[r]);return e},ql=e=>e.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g,"\\$&");var YG={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;","/":"&#x2F;"};const QG=e=>xt(e)?e.replace(/[&<>"'\/]/g,t=>YG[t]):e;class ZG{constructor(t){this.capacity=t,this.regExpMap=new Map,this.regExpQueue=[]}getRegExp(t){const n=this.regExpMap.get(t);if(n!==void 0)return n;const r=new RegExp(t);return this.regExpQueue.length===this.capacity&&this.regExpMap.delete(this.regExpQueue.shift()),this.regExpMap.set(t,r),this.regExpQueue.push(t),r}}const JG=[" ",",","?","!",";"],eK=new ZG(20),tK=(e,t,n)=>{t=t||"",n=n||"";const r=JG.filter(s=>t.indexOf(s)<0&&n.indexOf(s)<0);if(r.length===0)return!0;const a=eK.getRegExp(`(${r.map(s=>s==="?"?"\\?":s).join("|")})`);let l=!a.test(e);if(!l){const s=e.indexOf(n);s>0&&!a.test(e.substring(0,s))&&(l=!0)}return l},L0=(e,t,n=".")=>{if(!e)return;if(e[t])return Object.prototype.hasOwnProperty.call(e,t)?e[t]:void 0;const r=t.split(n);let a=e;for(let l=0;l<r.length;){if(!a||typeof a!="object")return;let s,u="";for(let d=l;d<r.length;++d)if(d!==l&&(u+=n),u+=r[d],s=a[u],s!==void 0){if(["string","number","boolean"].indexOf(typeof s)>-1&&d<r.length-1)continue;l+=d-l+1;break}a=s}return a},bu=e=>e==null?void 0:e.replace("_","-"),nK={type:"logger",log(e){this.output("log",e)},warn(e){this.output("warn",e)},error(e){this.output("error",e)},output(e,t){var n,r;(r=(n=console==null?void 0:console[e])==null?void 0:n.apply)==null||r.call(n,console,t)}};class am{constructor(t,n={}){this.init(t,n)}init(t,n={}){this.prefix=n.prefix||"i18next:",this.logger=t||nK,this.options=n,this.debug=n.debug}log(...t){return this.forward(t,"log","",!0)}warn(...t){return this.forward(t,"warn","",!0)}error(...t){return this.forward(t,"error","")}deprecate(...t){return this.forward(t,"warn","WARNING DEPRECATED: ",!0)}forward(t,n,r,a){return a&&!this.debug?null:(xt(t[0])&&(t[0]=`${r}${this.prefix} ${t[0]}`),this.logger[n](t))}create(t){return new am(this.logger,{prefix:`${this.prefix}:${t}:`,...this.options})}clone(t){return t=t||this.options,t.prefix=t.prefix||this.prefix,new am(this.logger,t)}}var Ja=new am;class rg{constructor(){this.observers={}}on(t,n){return t.split(" ").forEach(r=>{this.observers[r]||(this.observers[r]=new Map);const a=this.observers[r].get(n)||0;this.observers[r].set(n,a+1)}),this}off(t,n){if(this.observers[t]){if(!n){delete this.observers[t];return}this.observers[t].delete(n)}}emit(t,...n){this.observers[t]&&Array.from(this.observers[t].entries()).forEach(([a,l])=>{for(let s=0;s<l;s++)a(...n)}),this.observers["*"]&&Array.from(this.observers["*"].entries()).forEach(([a,l])=>{for(let s=0;s<l;s++)a.apply(a,[t,...n])})}}class v$ extends rg{constructor(t,n={ns:["translation"],defaultNS:"translation"}){super(),this.data=t||{},this.options=n,this.options.keySeparator===void 0&&(this.options.keySeparator="."),this.options.ignoreJSONStructure===void 0&&(this.options.ignoreJSONStructure=!0)}addNamespaces(t){this.options.ns.indexOf(t)<0&&this.options.ns.push(t)}removeNamespaces(t){const n=this.options.ns.indexOf(t);n>-1&&this.options.ns.splice(n,1)}getResource(t,n,r,a={}){var g,h;const l=a.keySeparator!==void 0?a.keySeparator:this.options.keySeparator,s=a.ignoreJSONStructure!==void 0?a.ignoreJSONStructure:this.options.ignoreJSONStructure;let u;t.indexOf(".")>-1?u=t.split("."):(u=[t,n],r&&(Array.isArray(r)?u.push(...r):xt(r)&&l?u.push(...r.split(l)):u.push(r)));const d=rm(this.data,u);return!d&&!n&&!r&&t.indexOf(".")>-1&&(t=u[0],n=u[1],r=u.slice(2).join(".")),d||!s||!xt(r)?d:L0((h=(g=this.data)==null?void 0:g[t])==null?void 0:h[n],r,l)}addResource(t,n,r,a,l={silent:!1}){const s=l.keySeparator!==void 0?l.keySeparator:this.options.keySeparator;let u=[t,n];r&&(u=u.concat(s?r.split(s):r)),t.indexOf(".")>-1&&(u=t.split("."),a=n,n=u[1]),this.addNamespaces(n),p$(this.data,u,a),l.silent||this.emit("added",t,n,r,a)}addResources(t,n,r,a={silent:!1}){for(const l in r)(xt(r[l])||Array.isArray(r[l]))&&this.addResource(t,n,l,r[l],{silent:!0});a.silent||this.emit("added",t,n,r)}addResourceBundle(t,n,r,a,l,s={silent:!1,skipCopy:!1}){let u=[t,n];t.indexOf(".")>-1&&(u=t.split("."),a=r,r=n,n=u[1]),this.addNamespaces(n);let d=rm(this.data,u)||{};s.skipCopy||(r=JSON.parse(JSON.stringify(r))),a?eN(d,r,l):d={...d,...r},p$(this.data,u,d),s.silent||this.emit("added",t,n,r)}removeResourceBundle(t,n){this.hasResourceBundle(t,n)&&delete this.data[t][n],this.removeNamespaces(n),this.emit("removed",t,n)}hasResourceBundle(t,n){return this.getResource(t,n)!==void 0}getResourceBundle(t,n){return n||(n=this.options.defaultNS),this.getResource(t,n)}getDataByLanguage(t){return this.data[t]}hasLanguageSomeTranslations(t){const n=this.getDataByLanguage(t);return!!(n&&Object.keys(n)||[]).find(a=>n[a]&&Object.keys(n[a]).length>0)}toJSON(){return this.data}}var tN={processors:{},addPostProcessor(e){this.processors[e.name]=e},handle(e,t,n,r,a){return e.forEach(l=>{var s;t=((s=this.processors[l])==null?void 0:s.process(t,n,r,a))??t}),t}};const nN=Symbol("i18next/PATH_KEY");function rK(){const e=[],t=Object.create(null);let n;return t.get=(r,a)=>{var l;return(l=n==null?void 0:n.revoke)==null||l.call(n),a===nN?e:(e.push(a),n=Proxy.revocable(r,t),n.proxy)},Proxy.revocable(Object.create(null),t).proxy}function B0(e,t){const{[nN]:n}=e(rK());return n.join((t==null?void 0:t.keySeparator)??".")}const y$={},b$=e=>!xt(e)&&typeof e!="boolean"&&typeof e!="number";class om extends rg{constructor(t,n={}){super(),WG(["resourceStore","languageUtils","pluralResolver","interpolator","backendConnector","i18nFormat","utils"],t,this),this.options=n,this.options.keySeparator===void 0&&(this.options.keySeparator="."),this.logger=Ja.create("translator")}changeLanguage(t){t&&(this.language=t)}exists(t,n={interpolation:{}}){const r={...n};if(t==null)return!1;const a=this.resolve(t,r);return(a==null?void 0:a.res)!==void 0}extractFromKey(t,n){let r=n.nsSeparator!==void 0?n.nsSeparator:this.options.nsSeparator;r===void 0&&(r=":");const a=n.keySeparator!==void 0?n.keySeparator:this.options.keySeparator;let l=n.ns||this.options.defaultNS||[];const s=r&&t.indexOf(r)>-1,u=!this.options.userDefinedKeySeparator&&!n.keySeparator&&!this.options.userDefinedNsSeparator&&!n.nsSeparator&&!tK(t,r,a);if(s&&!u){const d=t.match(this.interpolator.nestingRegexp);if(d&&d.length>0)return{key:t,namespaces:xt(l)?[l]:l};const g=t.split(r);(r!==a||r===a&&this.options.ns.indexOf(g[0])>-1)&&(l=g.shift()),t=g.join(a)}return{key:t,namespaces:xt(l)?[l]:l}}translate(t,n,r){let a=typeof n=="object"?{...n}:n;if(typeof a!="object"&&this.options.overloadTranslationOptionHandler&&(a=this.options.overloadTranslationOptionHandler(arguments)),typeof a=="object"&&(a={...a}),a||(a={}),t==null)return"";typeof t=="function"&&(t=B0(t,{...this.options,...a})),Array.isArray(t)||(t=[String(t)]);const l=a.returnDetails!==void 0?a.returnDetails:this.options.returnDetails,s=a.keySeparator!==void 0?a.keySeparator:this.options.keySeparator,{key:u,namespaces:d}=this.extractFromKey(t[t.length-1],a),g=d[d.length-1];let h=a.nsSeparator!==void 0?a.nsSeparator:this.options.nsSeparator;h===void 0&&(h=":");const p=a.lng||this.language,y=a.appendNamespaceToCIMode||this.options.appendNamespaceToCIMode;if((p==null?void 0:p.toLowerCase())==="cimode")return y?l?{res:`${g}${h}${u}`,usedKey:u,exactUsedKey:u,usedLng:p,usedNS:g,usedParams:this.getUsedParamsDetails(a)}:`${g}${h}${u}`:l?{res:u,usedKey:u,exactUsedKey:u,usedLng:p,usedNS:g,usedParams:this.getUsedParamsDetails(a)}:u;const C=this.resolve(t,a);let b=C==null?void 0:C.res;const x=(C==null?void 0:C.usedKey)||u,S=(C==null?void 0:C.exactUsedKey)||u,E=["[object Number]","[object Function]","[object RegExp]"],$=a.joinArrays!==void 0?a.joinArrays:this.options.joinArrays,O=!this.i18nFormat||this.i18nFormat.handleAsObject,_=a.count!==void 0&&!xt(a.count),R=om.hasDefaultValue(a),T=_?this.pluralResolver.getSuffix(p,a.count,a):"",M=a.ordinal&&_?this.pluralResolver.getSuffix(p,a.count,{ordinal:!1}):"",z=_&&!a.ordinal&&a.count===0,P=z&&a[`defaultValue${this.options.pluralSeparator}zero`]||a[`defaultValue${T}`]||a[`defaultValue${M}`]||a.defaultValue;let D=b;O&&!b&&R&&(D=P);const F=b$(D),G=Object.prototype.toString.apply(D);if(O&&D&&F&&E.indexOf(G)<0&&!(xt($)&&Array.isArray(D))){if(!a.returnObjects&&!this.options.returnObjects){this.options.returnedObjectHandler||this.logger.warn("accessing an object - but returnObjects options is not enabled!");const V=this.options.returnedObjectHandler?this.options.returnedObjectHandler(x,D,{...a,ns:d}):`key '${u} (${this.language})' returned an object instead of string.`;return l?(C.res=V,C.usedParams=this.getUsedParamsDetails(a),C):V}if(s){const V=Array.isArray(D),U=V?[]:{},K=V?S:x;for(const j in D)if(Object.prototype.hasOwnProperty.call(D,j)){const A=`${K}${s}${j}`;R&&!b?U[j]=this.translate(A,{...a,defaultValue:b$(P)?P[j]:void 0,joinArrays:!1,ns:d}):U[j]=this.translate(A,{...a,joinArrays:!1,ns:d}),U[j]===A&&(U[j]=D[j])}b=U}}else if(O&&xt($)&&Array.isArray(b))b=b.join($),b&&(b=this.extendTranslation(b,t,a,r));else{let V=!1,U=!1;!this.isValidLookup(b)&&R&&(V=!0,b=P),this.isValidLookup(b)||(U=!0,b=u);const j=(a.missingKeyNoValueFallbackToKey||this.options.missingKeyNoValueFallbackToKey)&&U?void 0:b,A=R&&P!==b&&this.options.updateMissing;if(U||V||A){if(this.logger.log(A?"updateKey":"missingKey",p,g,u,A?P:b),s){const B=this.resolve(u,{...a,keySeparator:!1});B&&B.res&&this.logger.warn("Seems the loaded translations were in flat JSON format instead of nested. Either set keySeparator: false on init or make sure your translations are published in nested format.")}let L=[];const q=this.languageUtils.getFallbackCodes(this.options.fallbackLng,a.lng||this.language);if(this.options.saveMissingTo==="fallback"&&q&&q[0])for(let B=0;B<q.length;B++)L.push(q[B]);else this.options.saveMissingTo==="all"?L=this.languageUtils.toResolveHierarchy(a.lng||this.language):L.push(a.lng||this.language);const N=(B,W,X)=>{var Y;const k=R&&X!==b?X:j;this.options.missingKeyHandler?this.options.missingKeyHandler(B,g,W,k,A,a):(Y=this.backendConnector)!=null&&Y.saveMissing&&this.backendConnector.saveMissing(B,g,W,k,A,a),this.emit("missingKey",B,g,W,b)};this.options.saveMissing&&(this.options.saveMissingPlurals&&_?L.forEach(B=>{const W=this.pluralResolver.getSuffixes(B,a);z&&a[`defaultValue${this.options.pluralSeparator}zero`]&&W.indexOf(`${this.options.pluralSeparator}zero`)<0&&W.push(`${this.options.pluralSeparator}zero`),W.forEach(X=>{N([B],u+X,a[`defaultValue${X}`]||P)})}):N(L,u,P))}b=this.extendTranslation(b,t,a,C,r),U&&b===u&&this.options.appendNamespaceToMissingKey&&(b=`${g}${h}${u}`),(U||V)&&this.options.parseMissingKeyHandler&&(b=this.options.parseMissingKeyHandler(this.options.appendNamespaceToMissingKey?`${g}${h}${u}`:u,V?b:void 0,a))}return l?(C.res=b,C.usedParams=this.getUsedParamsDetails(a),C):b}extendTranslation(t,n,r,a,l){var d,g;if((d=this.i18nFormat)!=null&&d.parse)t=this.i18nFormat.parse(t,{...this.options.interpolation.defaultVariables,...r},r.lng||this.language||a.usedLng,a.usedNS,a.usedKey,{resolved:a});else if(!r.skipInterpolation){r.interpolation&&this.interpolator.init({...r,interpolation:{...this.options.interpolation,...r.interpolation}});const h=xt(t)&&(((g=r==null?void 0:r.interpolation)==null?void 0:g.skipOnVariables)!==void 0?r.interpolation.skipOnVariables:this.options.interpolation.skipOnVariables);let p;if(h){const C=t.match(this.interpolator.nestingRegexp);p=C&&C.length}let y=r.replace&&!xt(r.replace)?r.replace:r;if(this.options.interpolation.defaultVariables&&(y={...this.options.interpolation.defaultVariables,...y}),t=this.interpolator.interpolate(t,y,r.lng||this.language||a.usedLng,r),h){const C=t.match(this.interpolator.nestingRegexp),b=C&&C.length;p<b&&(r.nest=!1)}!r.lng&&a&&a.res&&(r.lng=this.language||a.usedLng),r.nest!==!1&&(t=this.interpolator.nest(t,(...C)=>(l==null?void 0:l[0])===C[0]&&!r.context?(this.logger.warn(`It seems you are nesting recursively key: ${C[0]} in key: ${n[0]}`),null):this.translate(...C,n),r)),r.interpolation&&this.interpolator.reset()}const s=r.postProcess||this.options.postProcess,u=xt(s)?[s]:s;return t!=null&&(u!=null&&u.length)&&r.applyPostProcessor!==!1&&(t=tN.handle(u,t,n,this.options&&this.options.postProcessPassResolved?{i18nResolved:{...a,usedParams:this.getUsedParamsDetails(r)},...r}:r,this)),t}resolve(t,n={}){let r,a,l,s,u;return xt(t)&&(t=[t]),t.forEach(d=>{if(this.isValidLookup(r))return;const g=this.extractFromKey(d,n),h=g.key;a=h;let p=g.namespaces;this.options.fallbackNS&&(p=p.concat(this.options.fallbackNS));const y=n.count!==void 0&&!xt(n.count),C=y&&!n.ordinal&&n.count===0,b=n.context!==void 0&&(xt(n.context)||typeof n.context=="number")&&n.context!=="",x=n.lngs?n.lngs:this.languageUtils.toResolveHierarchy(n.lng||this.language,n.fallbackLng);p.forEach(S=>{var E,$;this.isValidLookup(r)||(u=S,!y$[`${x[0]}-${S}`]&&((E=this.utils)!=null&&E.hasLoadedNamespace)&&!(($=this.utils)!=null&&$.hasLoadedNamespace(u))&&(y$[`${x[0]}-${S}`]=!0,this.logger.warn(`key "${a}" for languages "${x.join(", ")}" won't get resolved as namespace "${u}" was not yet loaded`,"This means something IS WRONG in your setup. You access the t function before i18next.init / i18next.loadNamespace / i18next.changeLanguage was done. Wait for the callback or Promise to resolve before accessing it!!!")),x.forEach(O=>{var T;if(this.isValidLookup(r))return;s=O;const _=[h];if((T=this.i18nFormat)!=null&&T.addLookupKeys)this.i18nFormat.addLookupKeys(_,h,O,S,n);else{let M;y&&(M=this.pluralResolver.getSuffix(O,n.count,n));const z=`${this.options.pluralSeparator}zero`,P=`${this.options.pluralSeparator}ordinal${this.options.pluralSeparator}`;if(y&&(n.ordinal&&M.indexOf(P)===0&&_.push(h+M.replace(P,this.options.pluralSeparator)),_.push(h+M),C&&_.push(h+z)),b){const D=`${h}${this.options.contextSeparator||"_"}${n.context}`;_.push(D),y&&(n.ordinal&&M.indexOf(P)===0&&_.push(D+M.replace(P,this.options.pluralSeparator)),_.push(D+M),C&&_.push(D+z))}}let R;for(;R=_.pop();)this.isValidLookup(r)||(l=R,r=this.getResource(O,S,R,n))}))})}),{res:r,usedKey:a,exactUsedKey:l,usedLng:s,usedNS:u}}isValidLookup(t){return t!==void 0&&!(!this.options.returnNull&&t===null)&&!(!this.options.returnEmptyString&&t==="")}getResource(t,n,r,a={}){var l;return(l=this.i18nFormat)!=null&&l.getResource?this.i18nFormat.getResource(t,n,r,a):this.resourceStore.getResource(t,n,r,a)}getUsedParamsDetails(t={}){const n=["defaultValue","ordinal","context","replace","lng","lngs","fallbackLng","ns","keySeparator","nsSeparator","returnObjects","returnDetails","joinArrays","postProcess","interpolation"],r=t.replace&&!xt(t.replace);let a=r?t.replace:t;if(r&&typeof t.count<"u"&&(a.count=t.count),this.options.interpolation.defaultVariables&&(a={...this.options.interpolation.defaultVariables,...a}),!r){a={...a};for(const l of n)delete a[l]}return a}static hasDefaultValue(t){const n="defaultValue";for(const r in t)if(Object.prototype.hasOwnProperty.call(t,r)&&n===r.substring(0,n.length)&&t[r]!==void 0)return!0;return!1}}class S${constructor(t){this.options=t,this.supportedLngs=this.options.supportedLngs||!1,this.logger=Ja.create("languageUtils")}getScriptPartFromCode(t){if(t=bu(t),!t||t.indexOf("-")<0)return null;const n=t.split("-");return n.length===2||(n.pop(),n[n.length-1].toLowerCase()==="x")?null:this.formatLanguageCode(n.join("-"))}getLanguagePartFromCode(t){if(t=bu(t),!t||t.indexOf("-")<0)return t;const n=t.split("-");return this.formatLanguageCode(n[0])}formatLanguageCode(t){if(xt(t)&&t.indexOf("-")>-1){let n;try{n=Intl.getCanonicalLocales(t)[0]}catch{}return n&&this.options.lowerCaseLng&&(n=n.toLowerCase()),n||(this.options.lowerCaseLng?t.toLowerCase():t)}return this.options.cleanCode||this.options.lowerCaseLng?t.toLowerCase():t}isSupportedCode(t){return(this.options.load==="languageOnly"||this.options.nonExplicitSupportedLngs)&&(t=this.getLanguagePartFromCode(t)),!this.supportedLngs||!this.supportedLngs.length||this.supportedLngs.indexOf(t)>-1}getBestMatchFromCodes(t){if(!t)return null;let n;return t.forEach(r=>{if(n)return;const a=this.formatLanguageCode(r);(!this.options.supportedLngs||this.isSupportedCode(a))&&(n=a)}),!n&&this.options.supportedLngs&&t.forEach(r=>{if(n)return;const a=this.getScriptPartFromCode(r);if(this.isSupportedCode(a))return n=a;const l=this.getLanguagePartFromCode(r);if(this.isSupportedCode(l))return n=l;n=this.options.supportedLngs.find(s=>{if(s===l)return s;if(!(s.indexOf("-")<0&&l.indexOf("-")<0)&&(s.indexOf("-")>0&&l.indexOf("-")<0&&s.substring(0,s.indexOf("-"))===l||s.indexOf(l)===0&&l.length>1))return s})}),n||(n=this.getFallbackCodes(this.options.fallbackLng)[0]),n}getFallbackCodes(t,n){if(!t)return[];if(typeof t=="function"&&(t=t(n)),xt(t)&&(t=[t]),Array.isArray(t))return t;if(!n)return t.default||[];let r=t[n];return r||(r=t[this.getScriptPartFromCode(n)]),r||(r=t[this.formatLanguageCode(n)]),r||(r=t[this.getLanguagePartFromCode(n)]),r||(r=t.default),r||[]}toResolveHierarchy(t,n){const r=this.getFallbackCodes((n===!1?[]:n)||this.options.fallbackLng||[],t),a=[],l=s=>{s&&(this.isSupportedCode(s)?a.push(s):this.logger.warn(`rejecting language code not found in supportedLngs: ${s}`))};return xt(t)&&(t.indexOf("-")>-1||t.indexOf("_")>-1)?(this.options.load!=="languageOnly"&&l(this.formatLanguageCode(t)),this.options.load!=="languageOnly"&&this.options.load!=="currentOnly"&&l(this.getScriptPartFromCode(t)),this.options.load!=="currentOnly"&&l(this.getLanguagePartFromCode(t))):xt(t)&&l(this.formatLanguageCode(t)),r.forEach(s=>{a.indexOf(s)<0&&l(this.formatLanguageCode(s))}),a}}const C$={zero:0,one:1,two:2,few:3,many:4,other:5},x$={select:e=>e===1?"one":"other",resolvedOptions:()=>({pluralCategories:["one","other"]})};class aK{constructor(t,n={}){this.languageUtils=t,this.options=n,this.logger=Ja.create("pluralResolver"),this.pluralRulesCache={}}addRule(t,n){this.rules[t]=n}clearCache(){this.pluralRulesCache={}}getRule(t,n={}){const r=bu(t==="dev"?"en":t),a=n.ordinal?"ordinal":"cardinal",l=JSON.stringify({cleanedCode:r,type:a});if(l in this.pluralRulesCache)return this.pluralRulesCache[l];let s;try{s=new Intl.PluralRules(r,{type:a})}catch{if(!Intl)return this.logger.error("No Intl support, please use an Intl polyfill!"),x$;if(!t.match(/-|_/))return x$;const d=this.languageUtils.getLanguagePartFromCode(t);s=this.getRule(d,n)}return this.pluralRulesCache[l]=s,s}needsPlural(t,n={}){let r=this.getRule(t,n);return r||(r=this.getRule("dev",n)),(r==null?void 0:r.resolvedOptions().pluralCategories.length)>1}getPluralFormsOfKey(t,n,r={}){return this.getSuffixes(t,r).map(a=>`${n}${a}`)}getSuffixes(t,n={}){let r=this.getRule(t,n);return r||(r=this.getRule("dev",n)),r?r.resolvedOptions().pluralCategories.sort((a,l)=>C$[a]-C$[l]).map(a=>`${this.options.prepend}${n.ordinal?`ordinal${this.options.prepend}`:""}${a}`):[]}getSuffix(t,n,r={}){const a=this.getRule(t,r);return a?`${this.options.prepend}${r.ordinal?`ordinal${this.options.prepend}`:""}${a.select(n)}`:(this.logger.warn(`no plural rule found for: ${t}`),this.getSuffix("dev",n,r))}}const w$=(e,t,n,r=".",a=!0)=>{let l=XG(e,t,n);return!l&&a&&xt(n)&&(l=L0(e,n,r),l===void 0&&(l=L0(t,n,r))),l},dv=e=>e.replace(/\$/g,"$$$$");class oK{constructor(t={}){var n;this.logger=Ja.create("interpolator"),this.options=t,this.format=((n=t==null?void 0:t.interpolation)==null?void 0:n.format)||(r=>r),this.init(t)}init(t={}){t.interpolation||(t.interpolation={escapeValue:!0});const{escape:n,escapeValue:r,useRawValueToEscape:a,prefix:l,prefixEscaped:s,suffix:u,suffixEscaped:d,formatSeparator:g,unescapeSuffix:h,unescapePrefix:p,nestingPrefix:y,nestingPrefixEscaped:C,nestingSuffix:b,nestingSuffixEscaped:x,nestingOptionsSeparator:S,maxReplaces:E,alwaysFormat:$}=t.interpolation;this.escape=n!==void 0?n:QG,this.escapeValue=r!==void 0?r:!0,this.useRawValueToEscape=a!==void 0?a:!1,this.prefix=l?ql(l):s||"{{",this.suffix=u?ql(u):d||"}}",this.formatSeparator=g||",",this.unescapePrefix=h?"":p||"-",this.unescapeSuffix=this.unescapePrefix?"":h||"",this.nestingPrefix=y?ql(y):C||ql("$t("),this.nestingSuffix=b?ql(b):x||ql(")"),this.nestingOptionsSeparator=S||",",this.maxReplaces=E||1e3,this.alwaysFormat=$!==void 0?$:!1,this.resetRegExp()}reset(){this.options&&this.init(this.options)}resetRegExp(){const t=(n,r)=>(n==null?void 0:n.source)===r?(n.lastIndex=0,n):new RegExp(r,"g");this.regexp=t(this.regexp,`${this.prefix}(.+?)${this.suffix}`),this.regexpUnescape=t(this.regexpUnescape,`${this.prefix}${this.unescapePrefix}(.+?)${this.unescapeSuffix}${this.suffix}`),this.nestingRegexp=t(this.nestingRegexp,`${this.nestingPrefix}((?:[^()"']+|"[^"]*"|'[^']*'|\\((?:[^()]|"[^"]*"|'[^']*')*\\))*?)${this.nestingSuffix}`)}interpolate(t,n,r,a){var C;let l,s,u;const d=this.options&&this.options.interpolation&&this.options.interpolation.defaultVariables||{},g=b=>{if(b.indexOf(this.formatSeparator)<0){const $=w$(n,d,b,this.options.keySeparator,this.options.ignoreJSONStructure);return this.alwaysFormat?this.format($,void 0,r,{...a,...n,interpolationkey:b}):$}const x=b.split(this.formatSeparator),S=x.shift().trim(),E=x.join(this.formatSeparator).trim();return this.format(w$(n,d,S,this.options.keySeparator,this.options.ignoreJSONStructure),E,r,{...a,...n,interpolationkey:S})};this.resetRegExp();const h=(a==null?void 0:a.missingInterpolationHandler)||this.options.missingInterpolationHandler,p=((C=a==null?void 0:a.interpolation)==null?void 0:C.skipOnVariables)!==void 0?a.interpolation.skipOnVariables:this.options.interpolation.skipOnVariables;return[{regex:this.regexpUnescape,safeValue:b=>dv(b)},{regex:this.regexp,safeValue:b=>this.escapeValue?dv(this.escape(b)):dv(b)}].forEach(b=>{for(u=0;l=b.regex.exec(t);){const x=l[1].trim();if(s=g(x),s===void 0)if(typeof h=="function"){const E=h(t,l,a);s=xt(E)?E:""}else if(a&&Object.prototype.hasOwnProperty.call(a,x))s="";else if(p){s=l[0];continue}else this.logger.warn(`missed to pass in variable ${x} for interpolating ${t}`),s="";else!xt(s)&&!this.useRawValueToEscape&&(s=m$(s));const S=b.safeValue(s);if(t=t.replace(l[0],S),p?(b.regex.lastIndex+=s.length,b.regex.lastIndex-=l[0].length):b.regex.lastIndex=0,u++,u>=this.maxReplaces)break}}),t}nest(t,n,r={}){let a,l,s;const u=(d,g)=>{const h=this.nestingOptionsSeparator;if(d.indexOf(h)<0)return d;const p=d.split(new RegExp(`${h}[ ]*{`));let y=`{${p[1]}`;d=p[0],y=this.interpolate(y,s);const C=y.match(/'/g),b=y.match(/"/g);(((C==null?void 0:C.length)??0)%2===0&&!b||b.length%2!==0)&&(y=y.replace(/'/g,'"'));try{s=JSON.parse(y),g&&(s={...g,...s})}catch(x){return this.logger.warn(`failed parsing options string in nesting for key ${d}`,x),`${d}${h}${y}`}return s.defaultValue&&s.defaultValue.indexOf(this.prefix)>-1&&delete s.defaultValue,d};for(;a=this.nestingRegexp.exec(t);){let d=[];s={...r},s=s.replace&&!xt(s.replace)?s.replace:s,s.applyPostProcessor=!1,delete s.defaultValue;const g=/{.*}/.test(a[1])?a[1].lastIndexOf("}")+1:a[1].indexOf(this.formatSeparator);if(g!==-1&&(d=a[1].slice(g).split(this.formatSeparator).map(h=>h.trim()).filter(Boolean),a[1]=a[1].slice(0,g)),l=n(u.call(this,a[1].trim(),s),s),l&&a[0]===t&&!xt(l))return l;xt(l)||(l=m$(l)),l||(this.logger.warn(`missed to resolve ${a[1]} for nesting ${t}`),l=""),d.length&&(l=d.reduce((h,p)=>this.format(h,p,r.lng,{...r,interpolationkey:a[1].trim()}),l.trim())),t=t.replace(a[0],l),this.regexp.lastIndex=0}return t}}const iK=e=>{let t=e.toLowerCase().trim();const n={};if(e.indexOf("(")>-1){const r=e.split("(");t=r[0].toLowerCase().trim();const a=r[1].substring(0,r[1].length-1);t==="currency"&&a.indexOf(":")<0?n.currency||(n.currency=a.trim()):t==="relativetime"&&a.indexOf(":")<0?n.range||(n.range=a.trim()):a.split(";").forEach(s=>{if(s){const[u,...d]=s.split(":"),g=d.join(":").trim().replace(/^'+|'+$/g,""),h=u.trim();n[h]||(n[h]=g),g==="false"&&(n[h]=!1),g==="true"&&(n[h]=!0),isNaN(g)||(n[h]=parseInt(g,10))}})}return{formatName:t,formatOptions:n}},E$=e=>{const t={};return(n,r,a)=>{let l=a;a&&a.interpolationkey&&a.formatParams&&a.formatParams[a.interpolationkey]&&a[a.interpolationkey]&&(l={...l,[a.interpolationkey]:void 0});const s=r+JSON.stringify(l);let u=t[s];return u||(u=e(bu(r),a),t[s]=u),u(n)}},lK=e=>(t,n,r)=>e(bu(n),r)(t);class sK{constructor(t={}){this.logger=Ja.create("formatter"),this.options=t,this.init(t)}init(t,n={interpolation:{}}){this.formatSeparator=n.interpolation.formatSeparator||",";const r=n.cacheInBuiltFormats?E$:lK;this.formats={number:r((a,l)=>{const s=new Intl.NumberFormat(a,{...l});return u=>s.format(u)}),currency:r((a,l)=>{const s=new Intl.NumberFormat(a,{...l,style:"currency"});return u=>s.format(u)}),datetime:r((a,l)=>{const s=new Intl.DateTimeFormat(a,{...l});return u=>s.format(u)}),relativetime:r((a,l)=>{const s=new Intl.RelativeTimeFormat(a,{...l});return u=>s.format(u,l.range||"day")}),list:r((a,l)=>{const s=new Intl.ListFormat(a,{...l});return u=>s.format(u)})}}add(t,n){this.formats[t.toLowerCase().trim()]=n}addCached(t,n){this.formats[t.toLowerCase().trim()]=E$(n)}format(t,n,r,a={}){const l=n.split(this.formatSeparator);if(l.length>1&&l[0].indexOf("(")>1&&l[0].indexOf(")")<0&&l.find(u=>u.indexOf(")")>-1)){const u=l.findIndex(d=>d.indexOf(")")>-1);l[0]=[l[0],...l.splice(1,u)].join(this.formatSeparator)}return l.reduce((u,d)=>{var p;const{formatName:g,formatOptions:h}=iK(d);if(this.formats[g]){let y=u;try{const C=((p=a==null?void 0:a.formatParams)==null?void 0:p[a.interpolationkey])||{},b=C.locale||C.lng||a.locale||a.lng||r;y=this.formats[g](u,b,{...h,...a,...C})}catch(C){this.logger.warn(C)}return y}else this.logger.warn(`there was no format function for ${g}`);return u},t)}}const cK=(e,t)=>{e.pending[t]!==void 0&&(delete e.pending[t],e.pendingCount--)};class uK extends rg{constructor(t,n,r,a={}){var l,s;super(),this.backend=t,this.store=n,this.services=r,this.languageUtils=r.languageUtils,this.options=a,this.logger=Ja.create("backendConnector"),this.waitingReads=[],this.maxParallelReads=a.maxParallelReads||10,this.readingCalls=0,this.maxRetries=a.maxRetries>=0?a.maxRetries:5,this.retryTimeout=a.retryTimeout>=1?a.retryTimeout:350,this.state={},this.queue=[],(s=(l=this.backend)==null?void 0:l.init)==null||s.call(l,r,a.backend,a)}queueLoad(t,n,r,a){const l={},s={},u={},d={};return t.forEach(g=>{let h=!0;n.forEach(p=>{const y=`${g}|${p}`;!r.reload&&this.store.hasResourceBundle(g,p)?this.state[y]=2:this.state[y]<0||(this.state[y]===1?s[y]===void 0&&(s[y]=!0):(this.state[y]=1,h=!1,s[y]===void 0&&(s[y]=!0),l[y]===void 0&&(l[y]=!0),d[p]===void 0&&(d[p]=!0)))}),h||(u[g]=!0)}),(Object.keys(l).length||Object.keys(s).length)&&this.queue.push({pending:s,pendingCount:Object.keys(s).length,loaded:{},errors:[],callback:a}),{toLoad:Object.keys(l),pending:Object.keys(s),toLoadLanguages:Object.keys(u),toLoadNamespaces:Object.keys(d)}}loaded(t,n,r){const a=t.split("|"),l=a[0],s=a[1];n&&this.emit("failedLoading",l,s,n),!n&&r&&this.store.addResourceBundle(l,s,r,void 0,void 0,{skipCopy:!0}),this.state[t]=n?-1:2,n&&r&&(this.state[t]=0);const u={};this.queue.forEach(d=>{KG(d.loaded,[l],s),cK(d,t),n&&d.errors.push(n),d.pendingCount===0&&!d.done&&(Object.keys(d.loaded).forEach(g=>{u[g]||(u[g]={});const h=d.loaded[g];h.length&&h.forEach(p=>{u[g][p]===void 0&&(u[g][p]=!0)})}),d.done=!0,d.errors.length?d.callback(d.errors):d.callback())}),this.emit("loaded",u),this.queue=this.queue.filter(d=>!d.done)}read(t,n,r,a=0,l=this.retryTimeout,s){if(!t.length)return s(null,{});if(this.readingCalls>=this.maxParallelReads){this.waitingReads.push({lng:t,ns:n,fcName:r,tried:a,wait:l,callback:s});return}this.readingCalls++;const u=(g,h)=>{if(this.readingCalls--,this.waitingReads.length>0){const p=this.waitingReads.shift();this.read(p.lng,p.ns,p.fcName,p.tried,p.wait,p.callback)}if(g&&h&&a<this.maxRetries){setTimeout(()=>{this.read.call(this,t,n,r,a+1,l*2,s)},l);return}s(g,h)},d=this.backend[r].bind(this.backend);if(d.length===2){try{const g=d(t,n);g&&typeof g.then=="function"?g.then(h=>u(null,h)).catch(u):u(null,g)}catch(g){u(g)}return}return d(t,n,u)}prepareLoading(t,n,r={},a){if(!this.backend)return this.logger.warn("No backend was added via i18next.use. Will not load resources."),a&&a();xt(t)&&(t=this.languageUtils.toResolveHierarchy(t)),xt(n)&&(n=[n]);const l=this.queueLoad(t,n,r,a);if(!l.toLoad.length)return l.pending.length||a(),null;l.toLoad.forEach(s=>{this.loadOne(s)})}load(t,n,r){this.prepareLoading(t,n,{},r)}reload(t,n,r){this.prepareLoading(t,n,{reload:!0},r)}loadOne(t,n=""){const r=t.split("|"),a=r[0],l=r[1];this.read(a,l,"read",void 0,void 0,(s,u)=>{s&&this.logger.warn(`${n}loading namespace ${l} for language ${a} failed`,s),!s&&u&&this.logger.log(`${n}loaded namespace ${l} for language ${a}`,u),this.loaded(t,s,u)})}saveMissing(t,n,r,a,l,s={},u=()=>{}){var d,g,h,p,y;if((g=(d=this.services)==null?void 0:d.utils)!=null&&g.hasLoadedNamespace&&!((p=(h=this.services)==null?void 0:h.utils)!=null&&p.hasLoadedNamespace(n))){this.logger.warn(`did not save key "${r}" as the namespace "${n}" was not yet loaded`,"This means something IS WRONG in your setup. You access the t function before i18next.init / i18next.loadNamespace / i18next.changeLanguage was done. Wait for the callback or Promise to resolve before accessing it!!!");return}if(!(r==null||r==="")){if((y=this.backend)!=null&&y.create){const C={...s,isUpdate:l},b=this.backend.create.bind(this.backend);if(b.length<6)try{let x;b.length===5?x=b(t,n,r,a,C):x=b(t,n,r,a),x&&typeof x.then=="function"?x.then(S=>u(null,S)).catch(u):u(null,x)}catch(x){u(x)}else b(t,n,r,a,u,C)}!t||!t[0]||this.store.addResource(t[0],n,r,a)}}}const $$=()=>({debug:!1,initAsync:!0,ns:["translation"],defaultNS:["translation"],fallbackLng:["dev"],fallbackNS:!1,supportedLngs:!1,nonExplicitSupportedLngs:!1,load:"all",preload:!1,simplifyPluralSuffix:!0,keySeparator:".",nsSeparator:":",pluralSeparator:"_",contextSeparator:"_",partialBundledLanguages:!1,saveMissing:!1,updateMissing:!1,saveMissingTo:"fallback",saveMissingPlurals:!0,missingKeyHandler:!1,missingInterpolationHandler:!1,postProcess:!1,postProcessPassResolved:!1,returnNull:!1,returnEmptyString:!0,returnObjects:!1,joinArrays:!1,returnedObjectHandler:!1,parseMissingKeyHandler:!1,appendNamespaceToMissingKey:!1,appendNamespaceToCIMode:!1,overloadTranslationOptionHandler:e=>{let t={};if(typeof e[1]=="object"&&(t=e[1]),xt(e[1])&&(t.defaultValue=e[1]),xt(e[2])&&(t.tDescription=e[2]),typeof e[2]=="object"||typeof e[3]=="object"){const n=e[3]||e[2];Object.keys(n).forEach(r=>{t[r]=n[r]})}return t},interpolation:{escapeValue:!0,format:e=>e,prefix:"{{",suffix:"}}",formatSeparator:",",unescapePrefix:"-",nestingPrefix:"$t(",nestingSuffix:")",nestingOptionsSeparator:",",maxReplaces:1e3,skipOnVariables:!0},cacheInBuiltFormats:!0}),O$=e=>{var t,n;return xt(e.ns)&&(e.ns=[e.ns]),xt(e.fallbackLng)&&(e.fallbackLng=[e.fallbackLng]),xt(e.fallbackNS)&&(e.fallbackNS=[e.fallbackNS]),((n=(t=e.supportedLngs)==null?void 0:t.indexOf)==null?void 0:n.call(t,"cimode"))<0&&(e.supportedLngs=e.supportedLngs.concat(["cimode"])),typeof e.initImmediate=="boolean"&&(e.initAsync=e.initImmediate),e},ld=()=>{},fK=e=>{Object.getOwnPropertyNames(Object.getPrototypeOf(e)).forEach(n=>{typeof e[n]=="function"&&(e[n]=e[n].bind(e))})};class Su extends rg{constructor(t={},n){if(super(),this.options=O$(t),this.services={},this.logger=Ja,this.modules={external:[]},fK(this),n&&!this.isInitialized&&!t.isClone){if(!this.options.initAsync)return this.init(t,n),this;setTimeout(()=>{this.init(t,n)},0)}}init(t={},n){this.isInitializing=!0,typeof t=="function"&&(n=t,t={}),t.defaultNS==null&&t.ns&&(xt(t.ns)?t.defaultNS=t.ns:t.ns.indexOf("translation")<0&&(t.defaultNS=t.ns[0]));const r=$$();this.options={...r,...this.options,...O$(t)},this.options.interpolation={...r.interpolation,...this.options.interpolation},t.keySeparator!==void 0&&(this.options.userDefinedKeySeparator=t.keySeparator),t.nsSeparator!==void 0&&(this.options.userDefinedNsSeparator=t.nsSeparator);const a=g=>g?typeof g=="function"?new g:g:null;if(!this.options.isClone){this.modules.logger?Ja.init(a(this.modules.logger),this.options):Ja.init(null,this.options);let g;this.modules.formatter?g=this.modules.formatter:g=sK;const h=new S$(this.options);this.store=new v$(this.options.resources,this.options);const p=this.services;p.logger=Ja,p.resourceStore=this.store,p.languageUtils=h,p.pluralResolver=new aK(h,{prepend:this.options.pluralSeparator,simplifyPluralSuffix:this.options.simplifyPluralSuffix}),this.options.interpolation.format&&this.options.interpolation.format!==r.interpolation.format&&this.logger.deprecate("init: you are still using the legacy format function, please use the new approach: https://www.i18next.com/translation-function/formatting"),g&&(!this.options.interpolation.format||this.options.interpolation.format===r.interpolation.format)&&(p.formatter=a(g),p.formatter.init&&p.formatter.init(p,this.options),this.options.interpolation.format=p.formatter.format.bind(p.formatter)),p.interpolator=new oK(this.options),p.utils={hasLoadedNamespace:this.hasLoadedNamespace.bind(this)},p.backendConnector=new uK(a(this.modules.backend),p.resourceStore,p,this.options),p.backendConnector.on("*",(C,...b)=>{this.emit(C,...b)}),this.modules.languageDetector&&(p.languageDetector=a(this.modules.languageDetector),p.languageDetector.init&&p.languageDetector.init(p,this.options.detection,this.options)),this.modules.i18nFormat&&(p.i18nFormat=a(this.modules.i18nFormat),p.i18nFormat.init&&p.i18nFormat.init(this)),this.translator=new om(this.services,this.options),this.translator.on("*",(C,...b)=>{this.emit(C,...b)}),this.modules.external.forEach(C=>{C.init&&C.init(this)})}if(this.format=this.options.interpolation.format,n||(n=ld),this.options.fallbackLng&&!this.services.languageDetector&&!this.options.lng){const g=this.services.languageUtils.getFallbackCodes(this.options.fallbackLng);g.length>0&&g[0]!=="dev"&&(this.options.lng=g[0])}!this.services.languageDetector&&!this.options.lng&&this.logger.warn("init: no languageDetector is used and no lng is defined"),["getResource","hasResourceBundle","getResourceBundle","getDataByLanguage"].forEach(g=>{this[g]=(...h)=>this.store[g](...h)}),["addResource","addResources","addResourceBundle","removeResourceBundle"].forEach(g=>{this[g]=(...h)=>(this.store[g](...h),this)});const u=_c(),d=()=>{const g=(h,p)=>{this.isInitializing=!1,this.isInitialized&&!this.initializedStoreOnce&&this.logger.warn("init: i18next is already initialized. You should call init just once!"),this.isInitialized=!0,this.options.isClone||this.logger.log("initialized",this.options),this.emit("initialized",this.options),u.resolve(p),n(h,p)};if(this.languages&&!this.isInitialized)return g(null,this.t.bind(this));this.changeLanguage(this.options.lng,g)};return this.options.resources||!this.options.initAsync?d():setTimeout(d,0),u}loadResources(t,n=ld){var l,s;let r=n;const a=xt(t)?t:this.language;if(typeof t=="function"&&(r=t),!this.options.resources||this.options.partialBundledLanguages){if((a==null?void 0:a.toLowerCase())==="cimode"&&(!this.options.preload||this.options.preload.length===0))return r();const u=[],d=g=>{if(!g||g==="cimode")return;this.services.languageUtils.toResolveHierarchy(g).forEach(p=>{p!=="cimode"&&u.indexOf(p)<0&&u.push(p)})};a?d(a):this.services.languageUtils.getFallbackCodes(this.options.fallbackLng).forEach(h=>d(h)),(s=(l=this.options.preload)==null?void 0:l.forEach)==null||s.call(l,g=>d(g)),this.services.backendConnector.load(u,this.options.ns,g=>{!g&&!this.resolvedLanguage&&this.language&&this.setResolvedLanguage(this.language),r(g)})}else r(null)}reloadResources(t,n,r){const a=_c();return typeof t=="function"&&(r=t,t=void 0),typeof n=="function"&&(r=n,n=void 0),t||(t=this.languages),n||(n=this.options.ns),r||(r=ld),this.services.backendConnector.reload(t,n,l=>{a.resolve(),r(l)}),a}use(t){if(!t)throw new Error("You are passing an undefined module! Please check the object you are passing to i18next.use()");if(!t.type)throw new Error("You are passing a wrong module! Please check the object you are passing to i18next.use()");return t.type==="backend"&&(this.modules.backend=t),(t.type==="logger"||t.log&&t.warn&&t.error)&&(this.modules.logger=t),t.type==="languageDetector"&&(this.modules.languageDetector=t),t.type==="i18nFormat"&&(this.modules.i18nFormat=t),t.type==="postProcessor"&&tN.addPostProcessor(t),t.type==="formatter"&&(this.modules.formatter=t),t.type==="3rdParty"&&this.modules.external.push(t),this}setResolvedLanguage(t){if(!(!t||!this.languages)&&!(["cimode","dev"].indexOf(t)>-1)){for(let n=0;n<this.languages.length;n++){const r=this.languages[n];if(!(["cimode","dev"].indexOf(r)>-1)&&this.store.hasLanguageSomeTranslations(r)){this.resolvedLanguage=r;break}}!this.resolvedLanguage&&this.languages.indexOf(t)<0&&this.store.hasLanguageSomeTranslations(t)&&(this.resolvedLanguage=t,this.languages.unshift(t))}}changeLanguage(t,n){this.isLanguageChangingTo=t;const r=_c();this.emit("languageChanging",t);const a=u=>{this.language=u,this.languages=this.services.languageUtils.toResolveHierarchy(u),this.resolvedLanguage=void 0,this.setResolvedLanguage(u)},l=(u,d)=>{d?this.isLanguageChangingTo===t&&(a(d),this.translator.changeLanguage(d),this.isLanguageChangingTo=void 0,this.emit("languageChanged",d),this.logger.log("languageChanged",d)):this.isLanguageChangingTo=void 0,r.resolve((...g)=>this.t(...g)),n&&n(u,(...g)=>this.t(...g))},s=u=>{var h,p;!t&&!u&&this.services.languageDetector&&(u=[]);const d=xt(u)?u:u&&u[0],g=this.store.hasLanguageSomeTranslations(d)?d:this.services.languageUtils.getBestMatchFromCodes(xt(u)?[u]:u);g&&(this.language||a(g),this.translator.language||this.translator.changeLanguage(g),(p=(h=this.services.languageDetector)==null?void 0:h.cacheUserLanguage)==null||p.call(h,g)),this.loadResources(g,y=>{l(y,g)})};return!t&&this.services.languageDetector&&!this.services.languageDetector.async?s(this.services.languageDetector.detect()):!t&&this.services.languageDetector&&this.services.languageDetector.async?this.services.languageDetector.detect.length===0?this.services.languageDetector.detect().then(s):this.services.languageDetector.detect(s):s(t),r}getFixedT(t,n,r){const a=(l,s,...u)=>{let d;typeof s!="object"?d=this.options.overloadTranslationOptionHandler([l,s].concat(u)):d={...s},d.lng=d.lng||a.lng,d.lngs=d.lngs||a.lngs,d.ns=d.ns||a.ns,d.keyPrefix!==""&&(d.keyPrefix=d.keyPrefix||r||a.keyPrefix);const g=this.options.keySeparator||".";let h;return d.keyPrefix&&Array.isArray(l)?h=l.map(p=>(typeof p=="function"&&(p=B0(p,{...this.options,...s})),`${d.keyPrefix}${g}${p}`)):(typeof l=="function"&&(l=B0(l,{...this.options,...s})),h=d.keyPrefix?`${d.keyPrefix}${g}${l}`:l),this.t(h,d)};return xt(t)?a.lng=t:a.lngs=t,a.ns=n,a.keyPrefix=r,a}t(...t){var n;return(n=this.translator)==null?void 0:n.translate(...t)}exists(...t){var n;return(n=this.translator)==null?void 0:n.exists(...t)}setDefaultNamespace(t){this.options.defaultNS=t}hasLoadedNamespace(t,n={}){if(!this.isInitialized)return this.logger.warn("hasLoadedNamespace: i18next was not initialized",this.languages),!1;if(!this.languages||!this.languages.length)return this.logger.warn("hasLoadedNamespace: i18n.languages were undefined or empty",this.languages),!1;const r=n.lng||this.resolvedLanguage||this.languages[0],a=this.options?this.options.fallbackLng:!1,l=this.languages[this.languages.length-1];if(r.toLowerCase()==="cimode")return!0;const s=(u,d)=>{const g=this.services.backendConnector.state[`${u}|${d}`];return g===-1||g===0||g===2};if(n.precheck){const u=n.precheck(this,s);if(u!==void 0)return u}return!!(this.hasResourceBundle(r,t)||!this.services.backendConnector.backend||this.options.resources&&!this.options.partialBundledLanguages||s(r,t)&&(!a||s(l,t)))}loadNamespaces(t,n){const r=_c();return this.options.ns?(xt(t)&&(t=[t]),t.forEach(a=>{this.options.ns.indexOf(a)<0&&this.options.ns.push(a)}),this.loadResources(a=>{r.resolve(),n&&n(a)}),r):(n&&n(),Promise.resolve())}loadLanguages(t,n){const r=_c();xt(t)&&(t=[t]);const a=this.options.preload||[],l=t.filter(s=>a.indexOf(s)<0&&this.services.languageUtils.isSupportedCode(s));return l.length?(this.options.preload=a.concat(l),this.loadResources(s=>{r.resolve(),n&&n(s)}),r):(n&&n(),Promise.resolve())}dir(t){var a,l;if(t||(t=this.resolvedLanguage||(((a=this.languages)==null?void 0:a.length)>0?this.languages[0]:this.language)),!t)return"rtl";try{const s=new Intl.Locale(t);if(s&&s.getTextInfo){const u=s.getTextInfo();if(u&&u.direction)return u.direction}}catch{}const n=["ar","shu","sqr","ssh","xaa","yhd","yud","aao","abh","abv","acm","acq","acw","acx","acy","adf","ads","aeb","aec","afb","ajp","apc","apd","arb","arq","ars","ary","arz","auz","avl","ayh","ayl","ayn","ayp","bbz","pga","he","iw","ps","pbt","pbu","pst","prp","prd","ug","ur","ydd","yds","yih","ji","yi","hbo","men","xmn","fa","jpr","peo","pes","prs","dv","sam","ckb"],r=((l=this.services)==null?void 0:l.languageUtils)||new S$($$());return t.toLowerCase().indexOf("-latn")>1?"ltr":n.indexOf(r.getLanguagePartFromCode(t))>-1||t.toLowerCase().indexOf("-arab")>1?"rtl":"ltr"}static createInstance(t={},n){return new Su(t,n)}cloneInstance(t={},n=ld){const r=t.forkResourceStore;r&&delete t.forkResourceStore;const a={...this.options,...t,isClone:!0},l=new Su(a);if((t.debug!==void 0||t.prefix!==void 0)&&(l.logger=l.logger.clone(t)),["store","services","language"].forEach(u=>{l[u]=this[u]}),l.services={...this.services},l.services.utils={hasLoadedNamespace:l.hasLoadedNamespace.bind(l)},r){const u=Object.keys(this.store.data).reduce((d,g)=>(d[g]={...this.store.data[g]},d[g]=Object.keys(d[g]).reduce((h,p)=>(h[p]={...d[g][p]},h),d[g]),d),{});l.store=new v$(u,a),l.services.resourceStore=l.store}return l.translator=new om(l.services,a),l.translator.on("*",(u,...d)=>{l.emit(u,...d)}),l.init(a,n),l.translator.options=a,l.translator.backendConnector.services.utils={hasLoadedNamespace:l.hasLoadedNamespace.bind(l)},l}toJSON(){return{options:this.options,store:this.store,language:this.language,languages:this.languages,resolvedLanguage:this.resolvedLanguage}}}const nr=Su.createInstance();nr.createInstance=Su.createInstance;nr.createInstance;nr.dir;nr.init;nr.loadResources;nr.reloadResources;nr.use;nr.changeLanguage;nr.getFixedT;nr.t;nr.exists;nr.setDefaultNamespace;nr.hasLoadedNamespace;nr.loadNamespaces;nr.loadLanguages;const dK=/&(?:amp|#38|lt|#60|gt|#62|apos|#39|quot|#34|nbsp|#160|copy|#169|reg|#174|hellip|#8230|#x2F|#47);/g,mK={"&amp;":"&","&#38;":"&","&lt;":"<","&#60;":"<","&gt;":">","&#62;":">","&apos;":"'","&#39;":"'","&quot;":'"',"&#34;":'"',"&nbsp;":" ","&#160;":" ","&copy;":"©","&#169;":"©","&reg;":"®","&#174;":"®","&hellip;":"…","&#8230;":"…","&#x2F;":"/","&#47;":"/"},gK=e=>mK[e],hK=e=>e.replace(dK,gK);let R$={bindI18n:"languageChanged",bindI18nStore:"",transEmptyNodeValue:"",transSupportBasicHtmlNodes:!0,transWrapTextNodes:"",transKeepBasicHtmlNodesFor:["br","strong","i","p"],useSuspense:!0,unescape:hK};const pK=(e={})=>{R$={...R$,...e}},vK={type:"3rdParty",init(e){pK(e.options.react)}},yK="Home",bK="This is an internationalized React application.",SK={home:yK,description:bK},CK="主页",xK="这是一个支持国际化的 React 应用。",wK={home:CK,description:xK},EK=localStorage.getItem("locale");nr.use(vK).init({resources:{en:{translation:SK},zh:{translation:wK}},lng:EK==="zh_CN"?"zh":"en",fallbackLng:"zh",interpolation:{escapeValue:!1}});const ag=()=>{const e=ol(),t=Si(r=>r.user.locale),n=r=>{e($o({locale:r})),nr.changeLanguage(r==="zh_CN"?"zh":"en")};return{t:nr.t.bind(nr),locale:t,changeLanguage:n}},rN=()=>{const{locale:e,changeLanguage:t}=ag();return te.jsx(bi,{size:"small",value:e,style:{width:85},onChange:t,options:[{value:"zh_CN",label:"中文"},{value:"en_US",label:"English"}]})},aN=()=>{const{theme:e}=Si(n=>n.user),t=ol();return te.jsx(te.Fragment,{children:e=="light"?te.jsx(Q4,{onClick:()=>t($o({theme:"dark"}))}):te.jsx(dP,{onClick:()=>t($o({theme:"light"}))})})},$K=({visible:e,onClose:t,params:n,callback:r})=>{const a=Es(),[l]=Hn.useForm();if(f.useEffect(()=>{n!=null&&n.namespace_id?u():l.resetFields()},[n==null?void 0:n.namespace_id]),!e)return;const s=async()=>{const d=await l.validateFields();n!=null&&n.namespace_id?(await yt.post("/save-or-update-namespace",{...d,namespace_id:n==null?void 0:n.namespace_id}),a.success("successfully update!")):(await yt.post("/save-or-update-namespace",{...d}),a.success("successfully added!")),r&&r(),t()},u=async()=>{const d=await yt.get(`/find-namespace-by-id/${n==null?void 0:n.namespace_id}`);l.setFieldsValue(d.data)};return te.jsx(Hr,{title:te.jsxs(Mo,{gap:"small",children:[n!=null&&n.namespace_id?"Update":"Create"," Namespace"]}),open:e,onCancel:t,footer:null,children:te.jsxs(Hn,{form:l,onFinish:s,children:[te.jsx(Hn.Item,{name:"name",label:"Name",rules:[{required:!0,message:"This field cannot be empty!"}],children:te.jsx(za,{})}),te.jsx(Hn.Item,{name:"volumes",label:"Volumes",rules:[{required:!0,message:"This field cannot be empty!"}],children:te.jsx(Bm,{})}),te.jsx(Hn.Item,{children:te.jsx(er,{size:"small",color:"cyan",variant:"solid",htmlType:"submit",children:"Submit"})})]})})},OK=({visible:e,onClose:t,params:n,callback:r})=>{if(!e)return null;const a=Es(),[l,s]=f.useState([]),u=async()=>{const h=(await yt.get("/list-namespace-file")).data;s(h)};f.useEffect(()=>{u()},[]);const d=async g=>{try{await yt.post(`/import-namespace-component?namespace=${g}`),u(),a.success("install successfully!"),t(),r&&r()}catch{}};return te.jsx(Hr,{footer:null,title:"Install namespace",open:e,onCancel:t,children:l&&l.map(g=>te.jsxs(Mo,{style:{display:"flex",marginBottom:"0.5rem",justifyContent:"space-between"},children:[te.jsxs("div",{children:[g.name,"(",g.namespace_id,")"]}),te.jsx(Vm,{title:"Are you sure to install it?",onConfirm:()=>{d(g.namespace_id)},children:te.jsx(er,{size:"small",color:"cyan",variant:"solid",children:"Install"})})]},g.namespace_id))})},{Content:RK,Sider:LX}=La;[kO,FO,VO].map((e,t)=>{const n=String(t+1);return{key:`sub${n}`,icon:se.createElement(e),label:`subnav ${n}`,children:Array.from({length:4}).map((r,a)=>{const l=t*4+a+1;return{key:l,label:`option${l}`}})}});const oN=()=>te.jsx(Bo,{active:!0}),_K=()=>{const e=Xy(),t=Fo(),[n,r]=f.useState([]),a=ol(),[l,s]=ky.useNotification(),[u,d]=Iu.useMessage(),{modals:g,openModals:h,closeModals:p}=BG(["setting","project"]),[y,C]=f.useState("/"),[b,x]=f.useState([]),[S,E]=f.useState(),{locale:$}=ag(),{theme:O,baseURL:_,projectObj:R,project:T}=Si(Y=>Y.user),M=O==="dark",z=M?"#001529":"#fff",P=M?"#fff":"#000",D=({type:Y,message:ee="",description:ue=""})=>{l[Y]({message:ee,description:ue,placement:"bottomRight"})};console.log(T);const F=Y=>{console.log(Y),e(Y),C(Y)},G=async()=>{const Y=await yt.get("/setting/get-setting");console.log(Y.data),a(XM(Y.data))},{eventSourceRef:V,status:U,reconnect:K}=QM();f.useEffect(()=>{var ee;if(console.log("layout eventSourceRef",V),!V)return;const Y=ue=>{const ae=JSON.parse(ue.data);if(ae.msgType==="process_end"){const oe=ae.analysis,he=`${oe.analysis_name}(${oe.analysis_id}) 分析完成`;D({type:"info",message:he})}ae.msgType==="analysis_result"&&D({type:"info",message:ae.msg}),ae.msgType==="test"&&D({type:"info",message:ae.msg}),a(KM(ue.data))};return(ee=V.current)==null||ee.addEventListener("message",Y),()=>{var ue;console.log("removeEventListener"),(ue=V.current)==null||ue.removeEventListener("message",Y)}},[V.current]),f.useEffect(()=>{G()},[]);const j=[{key:"/",label:{zh_CN:"项目介绍",en_US:"Introduction"}},{key:"/pipeline-card",label:{zh_CN:"分析管道",en_US:"Pipeline"},children:[{key:"/component/pipeline",hidden:!0}]},{key:"/software-card",label:{zh_CN:"分析软件",en_US:"Software"},children:[{key:"/component/software",hidden:!0}]},{key:"/file-card",label:{zh_CN:"分析文件",en_US:"File"},children:[{key:"/component/file",hidden:!0}]},{key:"/script-card",label:{zh_CN:"分析脚本",en_US:"Script"},children:[{key:"/component/script",hidden:!0}]},{key:"/analysis-report",label:{zh_CN:"分析报告",en_US:"Report"}},{key:"/tool-kit",label:{zh_CN:"工具集",en_US:"TookKit"}},{key:"/more",label:{zh_CN:"更多",en_US:"More"},children:[{key:"/pipeline-monitor-panal",label:{zh_CN:"管道监控",en_US:"Monitor"}},{key:"/psycmicrograph",label:{zh_CN:"菌群知识库",en_US:"PsycMicroGraph"}},{key:"/analysis-result",label:{zh_CN:"分析结果",en_US:"Analysis Result"}},{key:"/container-page",label:{zh_CN:"容器管理",en_US:"Container"}},{key:"/literature",label:{zh_CN:"文献资料",en_US:"Literature"}}]}],A=Y=>Y.filter(ee=>!ee.hidden).map(ee=>{const ue={...ee};return ue.label=ee.label[$],ee.children&&(ue.children=A(ee.children),ue.children.length===0&&delete ue.children),ue}),L=Y=>{const ee=[],ue=(ae,oe)=>{for(const he of ae){const ie=he.hidden&&oe?oe:he.key;ee.push({key:he.key,selectedKey:ie}),he.children&&ue(he.children,ie)}};return ue(Y),ee.sort((ae,oe)=>oe.key.length-ae.key.length),ee},q=(Y,ee)=>{for(const ue of ee)if(Y.startsWith(ue.key))return ue.selectedKey;return"/"};f.useEffect(()=>{const Y=A(j);x(Y)},[$]),f.useEffect(()=>{if(S){const Y=q(t.pathname,S);C(Y)}else{const Y=L(j);E(Y);const ee=q(t.pathname,Y);console.log("not exist selectedKeyMap",ee),C(ee)}},[t.pathname]);const N=()=>T?!0:!(t.pathname.startsWith("/component")||t.pathname.startsWith("/analysis-report")),[B,W]=f.useState("UNKNOW"),X=async()=>{try{await yt.get(`${_}/brave-api/ping`),W("CONNECT")}catch{W("NOT_CONNECT")}},k=async()=>{if(!T)return;const Y=await yt.get(`/project/find-by-project-id/${T}`);a($o({projectObj:Y.data}))};return f.useEffect(()=>{X()},[_]),f.useEffect(()=>{k()},[_,T]),te.jsxs(La,{style:{minHeight:"100vh"},children:[s,d,te.jsxs(Vy,{style:{position:"sticky",top:0,zIndex:10,width:"100%",display:"flex",alignItems:"center",justifyContent:"space-between",background:z,color:P,boxShadow:"0 1px 4px rgba(0,0,0,0.1)"},children:[te.jsxs("div",{style:{display:"flex",alignItems:"center",flex:1,minWidth:0},children:[te.jsx("div",{style:{color:P,marginRight:"1rem",cursor:"pointer"},onClick:async()=>{await yt.get("/send-test")},children:"BRAVE"}),te.jsx(bs,{mode:"horizontal",selectedKeys:[y],items:b,onSelect:Y=>{if(Y.key=="/psycmicrograph"){window.open(`${window.location.origin}${window.location.pathname}psycmicrograph.html`,"_blank");return}F(Y.key),console.log(Y)},style:{flex:1,minWidth:0,background:"transparent",borderInlineEnd:0}})]}),te.jsxs(Mo,{align:"center",gap:"small",children:[te.jsx(er,{size:"small",color:U==="open"?"green":U==="connecting"?"blue":"red",variant:"solid",onClick:K,children:U==="open"?"connected":U==="connecting"?"connecting":"connection fail"}),te.jsx(c4,{style:{cursor:"pointer"},onClick:()=>{window.open("https://pybrave.github.io/brave-doc","_blank")}}),te.jsx(NK,{open:B=="NOT_CONNECT"}),te.jsx(cP,{style:{cursor:"pointer"},onClick:()=>{h("setting")}})]})]}),te.jsx(La,{style:{padding:"0 0 0  0"},children:te.jsx(RK,{style:{},children:te.jsx(f.Suspense,{fallback:te.jsx(oN,{}),children:N()?te.jsx(te.Fragment,{children:te.jsx(nM,{context:{project:T,projectObj:R,messageApi:u}})}):te.jsx(Co,{description:"Please create/select the project first",image:Co.PRESENTED_IMAGE_SIMPLE,children:te.jsx(iN,{project_id:T,openModal:h})})},t.key)})}),te.jsx(JM,{params:g.project.params,visible:g.project.visible,onClose:()=>p("project")}),te.jsx(TK,{visible:g.setting.visible,onClose:()=>p("setting"),params:g.setting.params,project_id:T,openModal:h})]})},TK=({visible:e,onClose:t,project_id:n,openModal:r})=>{const{modal:a,closeModal:l}=Hu();return te.jsxs(xT,{title:"Setting",open:e,onClose:t,children:[te.jsxs(Mo,{vertical:!0,gap:"small",children:[te.jsxs("div",{children:["Language: ",te.jsx(rN,{})]}),te.jsxs("div",{children:["Theme: ",te.jsx(aN,{})]}),te.jsxs("div",{children:["Project: ",te.jsx(iN,{project_id:n,openModal:r})]}),te.jsxs("div",{children:["Namespace: ",te.jsx(jK,{})]})]}),te.jsx(OK,{callback:t,visible:a.key=="installNamespace"&&a.visible,onClose:l,params:a.params})]})},MK=f.lazy(()=>Ut(()=>import("./index-CDeD9lyT.js"),__vite__mapDeps([0,1,2,3]),import.meta.url)),NK=({open:e})=>{const{modal:t,openModal:n,closeModal:r}=Hu(),{baseURL:a,authorization:l,containerURL:s}=Si(S=>S.user),[u,d]=f.useState(a),[g,h]=f.useState(l),[p,y]=f.useState(s),C=ol(),[b,x]=Iu.useMessage();return f.useEffect(()=>{e&&n("apiComp")},[e]),te.jsxs(te.Fragment,{children:[x,te.jsx(r4,{style:{cursor:"pointer"},onClick:()=>{n("apiComp")}}),te.jsx(Hr,{width:"40%",title:"Edit api",open:t.key=="apiComp"&&t.visible,onClose:r,onCancel:r,onOk:async()=>{try{await yt.get(`${u}/brave-api/ping`,{headers:{Authorization:`Bearer ${g}`}}),C($o({baseURL:u})),g&&C($o({authorization:`${g}`})),p&&C($o({containerURL:`${p}`})),r(),b.success("Connection successful!")}catch{b.error("Connection failed!")}},children:t.key=="apiComp"&&t.visible&&te.jsxs(te.Fragment,{children:[te.jsx(Hn.Item,{label:"API",children:te.jsx(za,{value:u,onChange:S=>d(S.target.value)})}),te.jsx(FT,{style:{cursor:"pointer"},onClick:()=>{d("https://brave-eu0y.onrender.com")},children:"Test: https://brave-eu0y.onrender.com"}),te.jsx("a",{target:"_blank",href:`${u}/brave-api/ping`,children:"Certificate Verification"}),te.jsx("p",{style:{marginTop:8,color:"#888",fontSize:13},children:"⚠️ If your API uses a self-signed HTTPS certificate, the browser may show a “Connection not private” or “Unsafe” warning. Please click “Advanced” → “Proceed anyway” once to trust the certificate before testing the connection."}),te.jsx(Hn.Item,{label:"Authorization",children:te.jsx(za,{placeholder:"Optional",value:g,onChange:S=>h(S.target.value)})}),te.jsx(Hn.Item,{label:"Container URL",children:te.jsx(za,{placeholder:"Optional http://localhost:8089",value:p,onChange:S=>y(S.target.value)})}),te.jsx(f.Suspense,{fallback:te.jsx(oN,{}),children:te.jsx(MK,{data:`
brave      --mysql-url root:123456@localhost:63306/brave     --port 5008                 `})})]})})]})},jK=()=>{const[e,t]=f.useState(),[n,r]=f.useState([]),[a,l]=f.useState(!1),{modal:s,openModal:u,closeModal:d}=Hu(),g=Es(),h=async()=>{const C=(await yt.get("/list-namespace")).data.map(b=>({label:`${b.name}`,value:b.namespace_id}));r(C)},p=async()=>{l(!0);const y=await yt.get("/get-used-namespace");t(y.data),l(!1)};return f.useEffect(()=>{p(),h()},[]),te.jsxs(te.Fragment,{children:[te.jsx(bi,{loading:a,size:"small",dropdownRender:y=>te.jsxs(te.Fragment,{children:[y,te.jsx(qd,{style:{margin:"8px 0"}}),te.jsxs(Mo,{gap:"small",justify:"space-between",children:[te.jsx(er,{type:"text",size:"small",color:"cyan",variant:"solid",onClick:()=>{u("namespace")},children:"Create"}),te.jsx(er,{type:"text",size:"small",color:"cyan",variant:"solid",onClick:()=>{h()},children:"Refresh"})]}),(e==null?void 0:e.namespace_id)&&te.jsxs(te.Fragment,{children:[te.jsx(qd,{style:{margin:"8px 0"}}),te.jsx(Ho,{title:e.namespace_id,placement:"bottom",children:te.jsxs(Mo,{gap:"small",justify:"space-between",children:[te.jsx(er,{type:"text",size:"small",color:"cyan",variant:"solid",onClick:()=>{u("namespace",{namespace_id:e.namespace_id})},children:"Update"}),te.jsx(Vm,{title:"Are you sure about the deletion?",onConfirm:async()=>{await yt.delete(`/delete-namespace-by-id/${e.namespace_id}`),g.success("successfully delete"),h()},children:te.jsx(er,{type:"text",size:"small",color:"danger",variant:"solid",children:"Delete"})})]})})]})]}),onChange:async y=>{console.log("onChange",y),l(!0),await yt.post(`/set-used-namespace/${y}`),await p(),l(!1),g.success(`Switching Namespaces: ${y}`)},value:e==null?void 0:e.namespace_id,style:{width:130},placeholder:"Select Namespace",options:n}),te.jsx($K,{callback:h,visible:s.key=="namespace"&&s.visible,onClose:d,params:s.params})]})},iN=({})=>{const[e,t]=f.useState([]),n=ol(),r=Es(),{modal:a,openModal:l,closeModal:s}=Hu(),{project:u}=Si(g=>g.user),d=async()=>{const h=(await yt.get("/project/list-project")).data.map(p=>({label:`${p.project_name}`,value:p.project_id}));t(h)};return f.useEffect(()=>{d()},[]),te.jsxs(te.Fragment,{children:[te.jsx(bi,{size:"small",dropdownRender:g=>te.jsxs(te.Fragment,{children:[g,te.jsx(qd,{style:{margin:"8px 0"}}),te.jsxs(Mo,{gap:"small",justify:"space-between",children:[te.jsx(er,{type:"text",size:"small",color:"cyan",variant:"solid",onClick:()=>{l("project")},children:"Create"}),te.jsx(er,{type:"text",size:"small",color:"cyan",variant:"solid",onClick:()=>{d()},children:"Refresh"})]}),u&&te.jsxs(te.Fragment,{children:[te.jsx(qd,{style:{margin:"8px 0"}}),te.jsx(Ho,{title:u,placement:"bottom",children:te.jsxs(Mo,{gap:"small",justify:"space-between",children:[te.jsx(er,{type:"text",size:"small",color:"cyan",variant:"solid",onClick:()=>{l("project",{project_id:u})},children:"Update"}),te.jsx(Vm,{title:"Are you sure about the deletion?",onConfirm:async()=>{await kG(u),r.success("successfully delete"),d()},children:te.jsx(er,{type:"text",size:"small",color:"danger",variant:"solid",children:"Delete"})})]})})]})]}),onChange:g=>{console.log("onChange",g),n($o({project:g})),r.success(`Switching Project: ${g}`)},value:u,style:{width:130},placeholder:"Select Project",options:e}),te.jsx(JM,{callback:d,visible:a.key=="project"&&a.visible,onClose:s,params:a.params})]})},_$=localStorage.getItem("currenct_project"),lN=Bu({name:"context",initialState:{project:_$?JSON.parse(_$):{name:"default",project_id:"default"}},reducers:{setProject:(e,t)=>{e.project=t.payload,localStorage.setItem("currenct_project",JSON.stringify(t.payload))}}}),{setProject:BX}=lN.actions,IK=lN.reducer,{Content:PK,Sider:HX}=La;[kO,FO,VO].map((e,t)=>{const n=String(t+1);return{key:`sub${n}`,icon:se.createElement(e),label:`subnav ${n}`,children:Array.from({length:4}).map((r,a)=>{const l=t*4+a+1;return{key:l,label:`option${l}`}})}});const AK=()=>te.jsx(Bo,{active:!0}),zK=()=>{const e=Xy(),t=Fo(),[n,r]=f.useState([]),a=ol(),[l,s]=ky.useNotification(),[u,d]=Iu.useMessage();Hu();const[g,h]=f.useState({}),[p,y]=f.useState("/"),[C,b]=f.useState([]),[x,S]=f.useState(),[E,$]=f.useState(),{locale:O}=ag(),{theme:_}=Si(L=>L.user),R=_==="dark",T=R?"#001529":"#fff",M=R?"#fff":"#000",z=({type:L,message:q="",description:N=""})=>{l[L]({message:q,description:N,placement:"bottomRight"})},P=L=>{console.log(L),e(L),y(L)},D=async()=>{const L=await yt.get("/setting/get-setting");console.log(L.data),a(XM(L.data))},{eventSourceRef:F,status:G,reconnect:V}=QM();f.useEffect(()=>{var q;if(console.log("layout eventSourceRef",F),!F)return;const L=N=>{const B=JSON.parse(N.data);if(B.msgType==="process_end"){const W=B.analysis,X=`${W.analysis_name}(${W.analysis_id}) 分析完成`;z({type:"info",message:X})}B.msgType==="analysis_result"&&z({type:"info",message:B.msg}),B.msgType==="test"&&z({type:"info",message:B.msg}),a(KM(N.data))};return(q=F.current)==null||q.addEventListener("message",L),()=>{var N;console.log("removeEventListener"),(N=F.current)==null||N.removeEventListener("message",L)}},[F.current]),f.useEffect(()=>{D()},[]);const U=[{key:"/",label:{zh_CN:"主页",en_US:"Home"}},{key:"/entity-relation",label:{zh_CN:"Microbial Graph",en_US:"Microbial Graph"}},{key:"/entity-page",label:{zh_CN:"Ontology",en_US:"Ontology"}},{key:"/mining",label:{zh_CN:"Mining",en_US:"Mining"},children:[{key:"/mining-data",label:{zh_CN:"Mining Data",en_US:"Mining Data"},hidden:!0}]}],K=L=>L.filter(q=>!q.hidden).map(q=>{const N={...q};return N.label=q.label[O],q.children&&(N.children=K(q.children),N.children.length===0&&delete N.children),N}),j=L=>{const q=[],N=(B,W)=>{for(const X of B){const k=X.hidden&&W?W:X.key;q.push({key:X.key,selectedKey:k}),X.children&&N(X.children,k)}};return N(L),q.sort((B,W)=>W.key.length-B.key.length),q},A=(L,q)=>{for(const N of q)if(L.startsWith(N.key))return N.selectedKey;return"/"};return f.useEffect(()=>{const L=K(U);b(L)},[O]),f.useEffect(()=>{if(x){const L=A(t.pathname,x);y(L)}else{const L=j(U);S(L);const q=A(t.pathname,L);console.log("not exist selectedKeyMap",q),y(q)}},[t.pathname]),te.jsxs(La,{style:{minHeight:"100vh"},children:[s,d,te.jsxs(Vy,{style:{position:"sticky",top:0,zIndex:10,width:"100%",display:"flex",alignItems:"center",justifyContent:"space-between",background:T,color:M,boxShadow:"0 1px 4px rgba(0,0,0,0.1)"},children:[te.jsxs("div",{style:{display:"flex",alignItems:"center",flex:1,minWidth:0},children:[te.jsx("div",{style:{color:M,marginRight:"1rem",cursor:"pointer"},onClick:async()=>{await yt.get("/send-test")},children:"PsycMicroGraph"}),te.jsx(bs,{mode:"horizontal",selectedKeys:[p],items:C,onSelect:L=>{P(L.key),console.log(L)},style:{flex:1,minWidth:0,background:"transparent"}})]}),te.jsxs(Mo,{align:"center",gap:"small",children:[te.jsx(er,{size:"small",color:G==="open"?"green":G==="connecting"?"blue":"red",variant:"solid",onClick:V,children:G==="open"?"connected":G==="connecting"?"connecting":"connection fail"}),te.jsx(rN,{}),te.jsx(aN,{})]})]}),te.jsx(La,{children:te.jsx(PK,{style:{},children:te.jsx(f.Suspense,{fallback:te.jsx(AK,{}),children:te.jsx(nM,{context:{messageApi:u}})},t.key)})})]})},T$=f.lazy(()=>Ut(()=>import("./index-DEnjftNB.js").then(e=>e.i),__vite__mapDeps([4,5,2,6,7]),import.meta.url)),DK=f.lazy(()=>Ut(()=>import("./index-CmUtUGZt.js"),__vite__mapDeps([8,0,1,2,3,9,10,5,6,7,11,12,13,14,15,16]),import.meta.url)),LK=f.lazy(()=>Ut(()=>import("./index-CVMVBDGX.js"),[],import.meta.url)),M$=f.lazy(()=>Ut(()=>import("./index-DuRuG3rR.js"),__vite__mapDeps([17,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35,36]),import.meta.url)),N$=f.lazy(()=>Ut(()=>import("./index-CixgM-5n.js"),__vite__mapDeps([37,4,5,2,6,7,33,16,34,11,12,1]),import.meta.url)),j$=f.lazy(()=>Ut(()=>import("./index-DhO6XalF.js"),__vite__mapDeps([38,19,20,21,22,23,11,12,6,5,2,7,1,39]),import.meta.url)),I$=f.lazy(()=>Ut(()=>import("./index-Ct2J_Ed7.js"),__vite__mapDeps([40,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35]),import.meta.url)),P$=f.lazy(()=>Ut(()=>import("./index-B2uzaINz.js"),__vite__mapDeps([41,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35]),import.meta.url)),A$=f.lazy(()=>Ut(()=>import("./index-ZNx73KSX.js"),__vite__mapDeps([42,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35]),import.meta.url)),z$=f.lazy(()=>Ut(()=>import("./index-JUwAts5R.js"),__vite__mapDeps([43,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35,44]),import.meta.url)),D$=f.lazy(()=>Ut(()=>import("./index-DD1QZChH.js"),__vite__mapDeps([45,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35,44]),import.meta.url)),L$=f.lazy(()=>Ut(()=>import("./index-Bzv7v5Gp.js"),__vite__mapDeps([46,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35,44]),import.meta.url)),B$=f.lazy(()=>Ut(()=>import("./index-D5JMEKyH.js"),__vite__mapDeps([47,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35]),import.meta.url)),H$=f.lazy(()=>Ut(()=>import("./index-HDoMyE03.js"),__vite__mapDeps([48,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35,36]),import.meta.url)),F$=f.lazy(()=>Ut(()=>import("./index-D5R5AX7X.js"),__vite__mapDeps([49,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35,36]),import.meta.url)),V$=f.lazy(()=>Ut(()=>import("./index-Cyksu2x_.js"),__vite__mapDeps([50,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35,44]),import.meta.url));f.lazy(()=>Ut(()=>import("./index-BDXjIJVp.js"),__vite__mapDeps([51,52,11,12,6,36,31,27,9,10,5,2,7,22,23,32,53,14,54]),import.meta.url));const U$=f.lazy(()=>Ut(()=>import("./index-BiBS7Dlv.js"),__vite__mapDeps([55,19,20,21,22,23,11,12,6,5,2,7,26,0,1,3,27,28,29,30,10,14,15,16,35,56,36]),import.meta.url)),k$=f.lazy(()=>Ut(()=>import("./index-DrRz1_gm.js"),__vite__mapDeps([56,36,7,11,12,6,1,2]),import.meta.url)),q$=f.lazy(()=>Ut(()=>import("./index-Ctvowf5L.js"),__vite__mapDeps([57,24,25,26,0,1,2,3,11,12,6,27,20,21,22,23,5,7,28,29,30,10,14,15,16,13,31,9,32,19,33,34]),import.meta.url)),W$=f.lazy(()=>Ut(()=>import("./index-DL0lGe3T.js"),[],import.meta.url)),G$=f.lazy(()=>Ut(()=>import("./index-37k-tbTK.js"),[],import.meta.url)),K$=f.lazy(()=>Ut(()=>import("./index-1eyAAlBr.js"),[],import.meta.url)),X$=f.lazy(()=>Ut(()=>import("./index-DMiH39tH.js"),__vite__mapDeps([58,18,19,20,21,22,23,11,12,6,5,2,7,24,25,26,0,1,3,27,28,29,30,10,14,15,16,13,31,9,32,33,34,35,36,53,4,59,60]),import.meta.url)),ui=f.lazy(()=>Ut(()=>import("./index-BDgclIvd.js"),__vite__mapDeps([52,11,12,6,36,31,27,9,10,5,2,7,22,23,32,53,14,54]),import.meta.url)),Y$=f.lazy(()=>Ut(()=>import("./index-bQ-eeI8R.js"),__vite__mapDeps([61,27,26,0,1,2,3,11,12,6,20,21,22,23,5,7,28,29,30,10,14,15,16,25,13,31,9,32,19,33,34]),import.meta.url)),Q$=f.lazy(()=>Ut(()=>import("./index-CFitg6x7.js"),__vite__mapDeps([62,26,0,1,2,3,11,12,6,27,20,21,22,23,5,7,28,29,30,10,14,15,16]),import.meta.url)),Z$=f.lazy(()=>Ut(()=>import("./index-Djhsc5K4.js").then(e=>e.i),__vite__mapDeps([29,30,10,5,2,6,7,21,14,12]),import.meta.url)),J$=f.lazy(()=>Ut(()=>import("./index-DYrqibJD.js"),__vite__mapDeps([63,64,11,12,6,7,30,10,5,2,21,14,65,29,23]),import.meta.url)),BK=f.lazy(()=>Ut(()=>import("./index-CMDvsT_M.js"),__vite__mapDeps([66,11,12,6,65]),import.meta.url)),eO=f.lazy(()=>Ut(()=>import("./test-ubo_hT31.js"),[],import.meta.url)),HK=f.lazy(()=>Ut(()=>import("./index-DPpeG1Bx.js"),__vite__mapDeps([67,30,10,5,2,6,7,21,14]),import.meta.url)),FK=f.lazy(()=>Ut(()=>import("./index-ElkDSW1y.js"),[],import.meta.url)),VK=f.lazy(()=>Ut(()=>import("./index-Cyy-zllt.js"),[],import.meta.url)),tO=f.lazy(()=>Ut(()=>import("./index-BfzdirmR.js").then(e=>e.i),__vite__mapDeps([9,10,5,2,6,7]),import.meta.url)),UK=document.getElementById("root"),H0=UK.getAttribute("data-app");console.log("data-app: ",H0);let F0=[];if(H0=="index"){const e=[{path:"/",element:te.jsx(DK,{})},{path:"/sample",element:te.jsx(T$,{})},{path:"/sample-qc",element:te.jsx(F$,{})},{path:"/analysis-result",element:te.jsx(U$,{})},{path:"/literature",element:te.jsx(k$,{})},{path:"/pipeline-monitor-panal",element:te.jsx(q$,{})},{path:"/container-page",element:te.jsx(tO,{})},{path:"/analysis-report",element:te.jsx(Q$,{})},{path:"/pipeline-card",element:te.jsx(ui,{params:{component_type:"pipeline"}})},{path:"/tool-kit",element:te.jsx(VK,{})},{path:"/software-card",element:te.jsx(ui,{map:t=>({...t,name:t.component_name,path:`/software/${t.component_id}`}),params:{component_type:"software"}})},{path:"/file-card",element:te.jsx(ui,{map:t=>({...t,name:t.component_name,path:`/software/${t.component_id}`}),params:{component_type:"file"}})},{path:"/script-card",element:te.jsx(ui,{map:t=>({...t,name:t.component_name,path:`/software/${t.component_id}`}),params:{component_type:"script"}})},{path:"/software-analysis-editor/:analysisId",element:te.jsx(Y$,{})},{path:"/component/:component_type/:component_id",element:te.jsx(X$,{})},{path:"/software/:softwareId",element:te.jsx(W$,{})},{path:"/file/:fileId",element:te.jsx(G$,{})},{path:"/script/:scriptId",element:te.jsx(K$,{})},{path:"/entity-page",element:te.jsx(Z$,{})},{path:"/entity-relation",element:te.jsx(J$,{})},{path:"/:project/meta_genome/reads-based-abundance-analysis",element:te.jsx(D$,{})},{path:"/:project/meta_genome/remove-host",element:te.jsx(L$,{})},{path:"/:project/meta_genome/recovering-mag",element:te.jsx(H$,{})},{path:"/:project/meta_genome/abundance-meta",element:te.jsx(B$,{})},{path:"/:project/meta_genome/abundance",element:te.jsx(j$,{})},{path:"/:project/meta_genome/function-analysis",element:te.jsx(V$,{})},{path:"/:project/single_genome/mutation",element:te.jsx(M$,{})},{path:"/:project/single_genome/mutation-compare",element:te.jsx(N$,{})},{path:"/:project/single_genome/assembly",element:te.jsx(I$,{})},{path:"/:project/single_genome/gene-prediction",element:te.jsx(P$,{})},{path:"/:project/single_genome/gene-annotation",element:te.jsx(A$,{})},{path:"/:project/single_genome/gene-expression",element:te.jsx(z$,{})}];F0=[{path:"/doc",element:te.jsx(LK,{})},{path:"/",element:te.jsx(_K,{}),children:[...e]},{path:"/test",element:te.jsx(eO,{})}]}else if(H0=="micrograph"){const e=[{path:"/",element:te.jsx(BK,{})},{path:"/sample",element:te.jsx(T$,{})},{path:"/sample-qc",element:te.jsx(F$,{})},{path:"/analysis-result",element:te.jsx(U$,{})},{path:"/literature",element:te.jsx(k$,{})},{path:"/pipeline-monitor-panal",element:te.jsx(q$,{})},{path:"/container-page",element:te.jsx(tO,{})},{path:"/analysis-report",element:te.jsx(Q$,{})},{path:"/pipeline-card",element:te.jsx(ui,{params:{component_type:"pipeline"}})},{path:"/software-card",element:te.jsx(ui,{map:t=>({id:t.id,component_id:t.component_id,name:t.component_name,category:t.category,img:t.img,tags:t.tags,description:t.description,order:t.order_index,path:`/software/${t.component_id}`,namespace:t.namespace,namespace_name:t.namespace_name}),params:{component_type:"software"}})},{path:"/file-card",element:te.jsx(ui,{map:t=>({id:t.id,component_id:t.component_id,name:t.component_name,category:t.category,img:t.img,tags:t.tags,description:t.description,order:t.order_index,path:`/file/${t.component_id}`,namespace:t.namespace,namespace_name:t.namespace_name}),params:{component_type:"file"}})},{path:"/script-card",element:te.jsx(ui,{map:t=>({id:t.id,component_id:t.component_id,name:t.component_name,category:t.category,img:t.img,tags:t.tags,description:t.description,order:t.order_index,path:`/script/${t.component_id}`,namespace:t.namespace,namespace_name:t.namespace_name}),params:{component_type:"script"}})},{path:"/software-analysis-editor/:analysisId",element:te.jsx(Y$,{})},{path:"/component/:component_type/:component_id",element:te.jsx(X$,{})},{path:"/software/:softwareId",element:te.jsx(W$,{})},{path:"/file/:fileId",element:te.jsx(G$,{})},{path:"/script/:scriptId",element:te.jsx(K$,{})},{path:"/entity-page",element:te.jsx(Z$,{})},{path:"/entity-relation",element:te.jsx(J$,{})},{path:"/mining",element:te.jsx(HK,{})},{path:"/mining-data/:entity_id",element:te.jsx(FK,{})},{path:"/:project/meta_genome/reads-based-abundance-analysis",element:te.jsx(D$,{})},{path:"/:project/meta_genome/remove-host",element:te.jsx(L$,{})},{path:"/:project/meta_genome/recovering-mag",element:te.jsx(H$,{})},{path:"/:project/meta_genome/abundance-meta",element:te.jsx(B$,{})},{path:"/:project/meta_genome/abundance",element:te.jsx(j$,{})},{path:"/:project/meta_genome/function-analysis",element:te.jsx(V$,{})},{path:"/:project/single_genome/mutation",element:te.jsx(M$,{})},{path:"/:project/single_genome/mutation-compare",element:te.jsx(N$,{})},{path:"/:project/single_genome/assembly",element:te.jsx(I$,{})},{path:"/:project/single_genome/gene-prediction",element:te.jsx(P$,{})},{path:"/:project/single_genome/gene-annotation",element:te.jsx(A$,{})},{path:"/:project/single_genome/gene-expression",element:te.jsx(z$,{})}];F0=[{path:"/",element:te.jsx(zK,{}),children:[...e]},{path:"/test",element:te.jsx(eO,{})}]}const kK=()=>mk(F0);var Tc={},mv={exports:{}},nO;function Ci(){return nO||(nO=1,function(e){function t(n){return n&&n.__esModule?n:{default:n}}e.exports=t,e.exports.__esModule=!0,e.exports.default=e.exports}(mv)),mv.exports}var Mc={},rO;function qK(){if(rO)return Mc;rO=1,Object.defineProperty(Mc,"__esModule",{value:!0}),Mc.default=void 0;var e={items_per_page:"条/页",jump_to:"跳至",jump_to_confirm:"确定",page:"页",prev_page:"上一页",next_page:"下一页",prev_5:"向前 5 页",next_5:"向后 5 页",prev_3:"向前 3 页",next_3:"向后 3 页",page_size:"页码"};return Mc.default=e,Mc}var Nc={},jc={},Ic={},gv={exports:{}},hv={exports:{}},pv={exports:{}},vv={exports:{}},aO;function sN(){return aO||(aO=1,function(e){function t(n){"@babel/helpers - typeof";return e.exports=t=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(r){return typeof r}:function(r){return r&&typeof Symbol=="function"&&r.constructor===Symbol&&r!==Symbol.prototype?"symbol":typeof r},e.exports.__esModule=!0,e.exports.default=e.exports,t(n)}e.exports=t,e.exports.__esModule=!0,e.exports.default=e.exports}(vv)),vv.exports}var yv={exports:{}},oO;function WK(){return oO||(oO=1,function(e){var t=sN().default;function n(r,a){if(t(r)!="object"||!r)return r;var l=r[Symbol.toPrimitive];if(l!==void 0){var s=l.call(r,a||"default");if(t(s)!="object")return s;throw new TypeError("@@toPrimitive must return a primitive value.")}return(a==="string"?String:Number)(r)}e.exports=n,e.exports.__esModule=!0,e.exports.default=e.exports}(yv)),yv.exports}var iO;function GK(){return iO||(iO=1,function(e){var t=sN().default,n=WK();function r(a){var l=n(a,"string");return t(l)=="symbol"?l:l+""}e.exports=r,e.exports.__esModule=!0,e.exports.default=e.exports}(pv)),pv.exports}var lO;function KK(){return lO||(lO=1,function(e){var t=GK();function n(r,a,l){return(a=t(a))in r?Object.defineProperty(r,a,{value:l,enumerable:!0,configurable:!0,writable:!0}):r[a]=l,r}e.exports=n,e.exports.__esModule=!0,e.exports.default=e.exports}(hv)),hv.exports}var sO;function cN(){return sO||(sO=1,function(e){var t=KK();function n(a,l){var s=Object.keys(a);if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(a);l&&(u=u.filter(function(d){return Object.getOwnPropertyDescriptor(a,d).enumerable})),s.push.apply(s,u)}return s}function r(a){for(var l=1;l<arguments.length;l++){var s=arguments[l]!=null?arguments[l]:{};l%2?n(Object(s),!0).forEach(function(u){t(a,u,s[u])}):Object.getOwnPropertyDescriptors?Object.defineProperties(a,Object.getOwnPropertyDescriptors(s)):n(Object(s)).forEach(function(u){Object.defineProperty(a,u,Object.getOwnPropertyDescriptor(s,u))})}return a}e.exports=r,e.exports.__esModule=!0,e.exports.default=e.exports}(gv)),gv.exports}var Pc={},cO;function uN(){return cO||(cO=1,Object.defineProperty(Pc,"__esModule",{value:!0}),Pc.commonLocale=void 0,Pc.commonLocale={yearFormat:"YYYY",dayFormat:"D",cellMeridiemFormat:"A",monthBeforeYear:!0}),Pc}var uO;function XK(){if(uO)return Ic;uO=1;var e=Ci().default;Object.defineProperty(Ic,"__esModule",{value:!0}),Ic.default=void 0;var t=e(cN()),n=uN(),r=(0,t.default)((0,t.default)({},n.commonLocale),{},{locale:"zh_CN",today:"今天",now:"此刻",backToToday:"返回今天",ok:"确定",timeSelect:"选择时间",dateSelect:"选择日期",weekSelect:"选择周",clear:"清除",week:"周",month:"月",year:"年",previousMonth:"上个月 (翻页上键)",nextMonth:"下个月 (翻页下键)",monthSelect:"选择月份",yearSelect:"选择年份",decadeSelect:"选择年代",previousYear:"上一年 (Control键加左方向键)",nextYear:"下一年 (Control键加右方向键)",previousDecade:"上一年代",nextDecade:"下一年代",previousCentury:"上一世纪",nextCentury:"下一世纪",yearFormat:"YYYY年",cellDateFormat:"D",monthBeforeYear:!1});return Ic.default=r,Ic}var Ac={},fO;function fN(){if(fO)return Ac;fO=1,Object.defineProperty(Ac,"__esModule",{value:!0}),Ac.default=void 0;const e={placeholder:"请选择时间",rangePlaceholder:["开始时间","结束时间"]};return Ac.default=e,Ac}var dO;function dN(){if(dO)return jc;dO=1;var e=Ci().default;Object.defineProperty(jc,"__esModule",{value:!0}),jc.default=void 0;var t=e(XK()),n=e(fN());const r={lang:Object.assign({placeholder:"请选择日期",yearPlaceholder:"请选择年份",quarterPlaceholder:"请选择季度",monthPlaceholder:"请选择月份",weekPlaceholder:"请选择周",rangePlaceholder:["开始日期","结束日期"],rangeYearPlaceholder:["开始年份","结束年份"],rangeMonthPlaceholder:["开始月份","结束月份"],rangeQuarterPlaceholder:["开始季度","结束季度"],rangeWeekPlaceholder:["开始周","结束周"]},t.default),timePickerLocale:Object.assign({},n.default)};return r.lang.ok="确定",jc.default=r,jc}var mO;function YK(){if(mO)return Nc;mO=1;var e=Ci().default;Object.defineProperty(Nc,"__esModule",{value:!0}),Nc.default=void 0;var t=e(dN());return Nc.default=t.default,Nc}var gO;function QK(){if(gO)return Tc;gO=1;var e=Ci().default;Object.defineProperty(Tc,"__esModule",{value:!0}),Tc.default=void 0;var t=e(qK()),n=e(YK()),r=e(dN()),a=e(fN());const l="${label}不是一个有效的${type}",s={locale:"zh-cn",Pagination:t.default,DatePicker:r.default,TimePicker:a.default,Calendar:n.default,global:{placeholder:"请选择"},Table:{filterTitle:"筛选",filterConfirm:"确定",filterReset:"重置",filterEmptyText:"无筛选项",filterCheckAll:"全选",filterSearchPlaceholder:"在筛选项中搜索",emptyText:"暂无数据",selectAll:"全选当页",selectInvert:"反选当页",selectNone:"清空所有",selectionAll:"全选所有",sortTitle:"排序",expand:"展开行",collapse:"关闭行",triggerDesc:"点击降序",triggerAsc:"点击升序",cancelSort:"取消排序"},Modal:{okText:"确定",cancelText:"取消",justOkText:"知道了"},Tour:{Next:"下一步",Previous:"上一步",Finish:"结束导览"},Popconfirm:{cancelText:"取消",okText:"确定"},Transfer:{titles:["",""],searchPlaceholder:"请输入搜索内容",itemUnit:"项",itemsUnit:"项",remove:"删除",selectCurrent:"全选当页",removeCurrent:"删除当页",selectAll:"全选所有",deselectAll:"取消全选",removeAll:"删除全部",selectInvert:"反选当页"},Upload:{uploading:"文件上传中",removeFile:"删除文件",uploadError:"上传错误",previewFile:"预览文件",downloadFile:"下载文件"},Empty:{description:"暂无数据"},Icon:{icon:"图标"},Text:{edit:"编辑",copy:"复制",copied:"复制成功",expand:"展开",collapse:"收起"},Form:{optional:"（可选）",defaultValidateMessages:{default:"字段验证错误${label}",required:"请输入${label}",enum:"${label}必须是其中一个[${enum}]",whitespace:"${label}不能为空字符",date:{format:"${label}日期格式无效",parse:"${label}不能转换为日期",invalid:"${label}是一个无效日期"},types:{string:l,method:l,array:l,object:l,number:l,date:l,boolean:l,integer:l,float:l,regexp:l,email:l,url:l,hex:l},string:{len:"${label}须为${len}个字符",min:"${label}最少${min}个字符",max:"${label}最多${max}个字符",range:"${label}须在${min}-${max}字符之间"},number:{len:"${label}必须等于${len}",min:"${label}最小值为${min}",max:"${label}最大值为${max}",range:"${label}须在${min}-${max}之间"},array:{len:"须为${len}个${label}",min:"最少${min}个${label}",max:"最多${max}个${label}",range:"${label}数量须在${min}-${max}之间"},pattern:{mismatch:"${label}与模式不匹配${pattern}"}}},Image:{preview:"预览"},QRCode:{expired:"二维码过期",refresh:"点击刷新",scanned:"已扫描"},ColorPicker:{presetEmpty:"暂无",transparent:"无色",singleColor:"单色",gradientColor:"渐变色"}};return Tc.default=s,Tc}var bv,hO;function ZK(){return hO||(hO=1,bv=QK()),bv}var JK=ZK();const eX=is(JK);var zc={},Dc={},pO;function tX(){if(pO)return Dc;pO=1,Object.defineProperty(Dc,"__esModule",{value:!0}),Dc.default=void 0;var e={items_per_page:"/ page",jump_to:"Go to",jump_to_confirm:"confirm",page:"Page",prev_page:"Previous Page",next_page:"Next Page",prev_5:"Previous 5 Pages",next_5:"Next 5 Pages",prev_3:"Previous 3 Pages",next_3:"Next 3 Pages",page_size:"Page Size"};return Dc.default=e,Dc}var Lc={},Bc={},Hc={},vO;function nX(){if(vO)return Hc;vO=1;var e=Ci().default;Object.defineProperty(Hc,"__esModule",{value:!0}),Hc.default=void 0;var t=e(cN()),n=uN(),r=(0,t.default)((0,t.default)({},n.commonLocale),{},{locale:"en_US",today:"Today",now:"Now",backToToday:"Back to today",ok:"OK",clear:"Clear",week:"Week",month:"Month",year:"Year",timeSelect:"select time",dateSelect:"select date",weekSelect:"Choose a week",monthSelect:"Choose a month",yearSelect:"Choose a year",decadeSelect:"Choose a decade",dateFormat:"M/D/YYYY",dateTimeFormat:"M/D/YYYY HH:mm:ss",previousMonth:"Previous month (PageUp)",nextMonth:"Next month (PageDown)",previousYear:"Last year (Control + left)",nextYear:"Next year (Control + right)",previousDecade:"Last decade",nextDecade:"Next decade",previousCentury:"Last century",nextCentury:"Next century"});return Hc.default=r,Hc}var Fc={},yO;function mN(){if(yO)return Fc;yO=1,Object.defineProperty(Fc,"__esModule",{value:!0}),Fc.default=void 0;const e={placeholder:"Select time",rangePlaceholder:["Start time","End time"]};return Fc.default=e,Fc}var bO;function gN(){if(bO)return Bc;bO=1;var e=Ci().default;Object.defineProperty(Bc,"__esModule",{value:!0}),Bc.default=void 0;var t=e(nX()),n=e(mN());const r={lang:Object.assign({placeholder:"Select date",yearPlaceholder:"Select year",quarterPlaceholder:"Select quarter",monthPlaceholder:"Select month",weekPlaceholder:"Select week",rangePlaceholder:["Start date","End date"],rangeYearPlaceholder:["Start year","End year"],rangeQuarterPlaceholder:["Start quarter","End quarter"],rangeMonthPlaceholder:["Start month","End month"],rangeWeekPlaceholder:["Start week","End week"]},t.default),timePickerLocale:Object.assign({},n.default)};return Bc.default=r,Bc}var SO;function rX(){if(SO)return Lc;SO=1;var e=Ci().default;Object.defineProperty(Lc,"__esModule",{value:!0}),Lc.default=void 0;var t=e(gN());return Lc.default=t.default,Lc}var CO;function aX(){if(CO)return zc;CO=1;var e=Ci().default;Object.defineProperty(zc,"__esModule",{value:!0}),zc.default=void 0;var t=e(tX()),n=e(rX()),r=e(gN()),a=e(mN());const l="${label} is not a valid ${type}",s={locale:"en",Pagination:t.default,DatePicker:r.default,TimePicker:a.default,Calendar:n.default,global:{placeholder:"Please select"},Table:{filterTitle:"Filter menu",filterConfirm:"OK",filterReset:"Reset",filterEmptyText:"No filters",filterCheckAll:"Select all items",filterSearchPlaceholder:"Search in filters",emptyText:"No data",selectAll:"Select current page",selectInvert:"Invert current page",selectNone:"Clear all data",selectionAll:"Select all data",sortTitle:"Sort",expand:"Expand row",collapse:"Collapse row",triggerDesc:"Click to sort descending",triggerAsc:"Click to sort ascending",cancelSort:"Click to cancel sorting"},Tour:{Next:"Next",Previous:"Previous",Finish:"Finish"},Modal:{okText:"OK",cancelText:"Cancel",justOkText:"OK"},Popconfirm:{okText:"OK",cancelText:"Cancel"},Transfer:{titles:["",""],searchPlaceholder:"Search here",itemUnit:"item",itemsUnit:"items",remove:"Remove",selectCurrent:"Select current page",removeCurrent:"Remove current page",selectAll:"Select all data",deselectAll:"Deselect all data",removeAll:"Remove all data",selectInvert:"Invert current page"},Upload:{uploading:"Uploading...",removeFile:"Remove file",uploadError:"Upload error",previewFile:"Preview file",downloadFile:"Download file"},Empty:{description:"No data"},Icon:{icon:"icon"},Text:{edit:"Edit",copy:"Copy",copied:"Copied",expand:"Expand",collapse:"Collapse"},Form:{optional:"(optional)",defaultValidateMessages:{default:"Field validation error for ${label}",required:"Please enter ${label}",enum:"${label} must be one of [${enum}]",whitespace:"${label} cannot be a blank character",date:{format:"${label} date format is invalid",parse:"${label} cannot be converted to a date",invalid:"${label} is an invalid date"},types:{string:l,method:l,array:l,object:l,number:l,date:l,boolean:l,integer:l,float:l,regexp:l,email:l,url:l,hex:l},string:{len:"${label} must be ${len} characters",min:"${label} must be at least ${min} characters",max:"${label} must be up to ${max} characters",range:"${label} must be between ${min}-${max} characters"},number:{len:"${label} must be equal to ${len}",min:"${label} must be minimum ${min}",max:"${label} must be maximum ${max}",range:"${label} must be between ${min}-${max}"},array:{len:"Must be ${len} ${label}",min:"At least ${min} ${label}",max:"At most ${max} ${label}",range:"The amount of ${label} must be between ${min}-${max}"},pattern:{mismatch:"${label} does not match the pattern ${pattern}"}}},Image:{preview:"Preview"},QRCode:{expired:"QR code expired",refresh:"Refresh",scanned:"Scanned"},ColorPicker:{presetEmpty:"Empty",transparent:"Transparent",singleColor:"Single",gradientColor:"Gradient"}};return zc.default=s,zc}var Sv,xO;function oX(){return xO||(xO=1,Sv=aX()),Sv}var iX=oX();const lX=is(iX),sX=()=>{const{locale:e}=ag(),t=e==="zh_CN"?eX:lX,{theme:n}=Si(a=>a.user),r=qG();return te.jsxs(te.Fragment,{children:[r,te.jsx(Fa,{theme:{algorithm:n==="dark"?EE.darkAlgorithm:EE.defaultAlgorithm},locale:t,children:te.jsx(Yk,{children:te.jsx(kK,{})})})]})},hN=Bu({name:"menu",initialState:{items:[],selectedKey:""},reducers:{setMenuItems:(e,t)=>{e.items=t.payload},setSelectedKey:(e,t)=>{e.selectedKey=t.payload}}}),{setMenuItems:FX,setSelectedKey:VX}=hN.actions,cX=hN.reducer,wO=localStorage.getItem("display_node"),pN=Bu({name:"graph",initialState:{displayNode:wO?JSON.parse(wO):["disease","taxonomy","diet_and_food"]},reducers:{setDisplayNode:(e,t)=>{e.displayNode=t.payload,localStorage.setItem("display_node",JSON.stringify(t.payload))}}}),{setDisplayNode:UX}=pN.actions,uX=pN.reducer,Od=EG({reducer:{menu:cX,global:LG,context:IK,graph:uX,user:DG}});Od.subscribe(()=>{const e=Od.getState().user.baseURL,t=Od.getState().user.authorization;yt.defaults.baseURL!==e&&(console.log("currentBaseURL: ",e),yt.defaults.baseURL=`${e}/brave-api`,t&&(yt.defaults.headers.common.Authorization=`Bearer ${t}`))});console.log("production");const fX=localStorage.getItem("baseURL")||"";yt.defaults.baseURL=`${fX}/brave-api`;const EO=localStorage.getItem("authorization");EO&&(yt.defaults.headers.common.Authorization=`Bearer ${EO}`);yt.defaults.timeout=5e3;yt.interceptors.response.use(e=>e,e=>{const t=Es();if(e.response){const{status:n,data:r}=e.response;console.error("HTTP Error:",n),console.error(r==null?void 0:r.detail),t.error(r==null?void 0:r.detail)}else console.error("网络异常:",e.message);return Promise.reject(e)});$I.createRoot(document.getElementById("root")).render(te.jsx(Sq,{store:Od,children:te.jsx(HG,{children:te.jsx(sX,{})})}));export{Es as $,Ao as A,er as B,uT as C,xT as D,Co as E,Mo as F,Wn as G,Gn as H,za as I,$r as J,bV as K,An as L,Hr as M,bn as N,ru as O,Vm as P,Br as Q,O9 as R,Bo as S,Bm as T,Ne as U,re as V,jL as W,Fo as X,QM as Y,w4 as Z,je as _,yt as a,hy as a$,dt as a0,ta as a1,xn as a2,Kt as a3,Rr as a4,ve as a5,gs as a6,Lr as a7,na as a8,Or as a9,Kz as aA,_d as aB,dy as aC,H4 as aD,$a as aE,D7 as aF,Ld as aG,q2 as aH,Cu as aI,Ro as aJ,br as aK,Do as aL,Oo as aM,Ha as aN,Y2 as aO,us as aP,q0 as aQ,qP as aR,K0 as aS,Ut as aT,Gt as aU,i_ as aV,dT as aW,fT as aX,ix as aY,Dm as aZ,zm as a_,Lo as aa,L7 as ab,uu as ac,s7 as ad,U0 as ae,tu as af,Q as ag,ge as ah,Z as ai,al as aj,$t as ak,Nr as al,dX as am,st as an,sp as ao,Dt as ap,en as aq,On as ar,ss as as,cs as at,cm as au,HO as av,mr as aw,jt as ax,Ad as ay,Pd as az,Hu as b,By as b$,iT as b0,s9 as b1,f9 as b2,m9 as b3,u9 as b4,g9 as b5,h9 as b6,c9 as b7,d9 as b8,ms as b9,gm as bA,bs as bB,pX as bC,O8 as bD,N2 as bE,oR as bF,aR as bG,vy as bH,Sz as bI,hE as bJ,gX as bK,vX as bL,T4 as bM,rl as bN,xu as bO,wa as bP,W0 as bQ,WO as bR,hX as bS,vm as bT,aD as bU,gw as bV,y_ as bW,lt as bX,Lm as bY,I2 as bZ,tT as b_,ia as ba,Ru as bb,ra as bc,Ty as bd,Fd as be,ns as bf,Nm as bg,Fa as bh,Y1 as bi,Dv as bj,zo as bk,UO as bl,qd as bm,mR as bn,fR as bo,dR as bp,uR as bq,__ as br,Bd as bs,kx as bt,Sm as bu,Ny as bv,O_ as bw,Eu as bx,G7 as by,$u as bz,Xy as c,oT as c0,hs as c1,tA as c2,Lv as c3,fs as c4,Ey as c5,Rm as c6,wR as c7,FL as c8,_o as c9,ju as ca,Im as cb,Io as cc,NT as cd,ag as ce,ol as cf,UX as cg,La as ch,VI as ci,nu as cj,lm as ck,xo as cl,qv as cm,H3 as cn,L3 as co,EE as cp,zA as cq,T2 as cr,_2 as cs,IA as ct,kO as cu,Hn as d,bi as e,se as f,bX as g,ky as h,Pu as i,te as j,yX as k,FT as l,O5 as m,im as n,is as o,A_ as p,pi as q,f as r,Iu as s,Ho as t,Si as u,BG as v,JM as w,on as x,nP as y,Po as z};
